(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    281305,       7290]*)
(*NotebookOutlinePosition[    283251,       7349]*)
(*  CellTagsIndexPosition[    283207,       7345]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 1. 
Advanced Tools for Rocket Science\
\>", "Title",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and Mathematical Finance"
}], "Subsubtitle",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[TextData[{
  "1.1 Why You Should Use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " for Derivatives Modelling"
}], "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "\nWhen expressed in mathematical terms, the modelling of a derivative \
security amounts to understanding the behaviour of a function of several \
variables in considerable detail. We need not just to know how to work out \
the function (the value of an option), but also to secure an appreciation of \
the sensitivity of the value to changes in any of the parameters of the \
function. Such an appreciation is most reliable when we have both a view of \
the local sensitivities, expressed through partial derivatives, and a global \
view, expressed by graphical means. The partial derivatives, expressed in \
financial terms, are the \"Greeks\" of the option value, and may be passive \
sensitivity variables, or be active hedging parameters. Such option \
valuations may, in simpler cases, be based on analytical closed forms \
involving special functions, or, failing that, may require intensive \
numerical computation requiring some extensive programming. \n\nIf one were \
asked afresh what sort of a system would combine together the ability to\n\n\
(i) deal with a myriad of special functions, and do symbolic calculus with \
them,\n(ii) manage advanced numerical computation,\n(iii) allow complex \
structures to be programmed,\n(iv) visualize functions in two, three or more \
dimensions,\n\nand to do so on a range of computer platforms, one would come \
up with a very short list of modelling systems. Neither spreadsheets nor \
C/C++ would feature on the list, due to their fundamental failure to cope \
with point (i) -  symbolic algebra and calculus. \n\nOne of the very few \
complete solutions to this list of requirements is the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " system. Originally subtitled as \"A System for Doing Mathematics by \
Computer\", it is uniquely able as a derivatives modelling tool. In \
particular, release 3.0 and later versions include substantial new numerical \
functionality, and are capable of efficient compiled numerical modelling on \
large structures. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " can manage not just scalar, vector and matrix data, but tensors of \
arbitrary rank, and its performance scales well to large and realistic \
problems. This book is, in part, an exploration of these capabilities. It is \
also an exploration of valuation algorithms and what can go wrong with them."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakBelow->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[TextData[{
  "Exploring the Fundamentals with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "    ",
  " "
}], "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakBelow->Automatic],

Cell[TextData[{
  "This book is not just an exploration of a few well-known models. We shall \
also explore the matter of solving the Black-Scholes differential equation in \
a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "-assisted fashion. We can explore"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[TextData[{
  "(i) separation of variables,\n(ii) similarity methods,\n(iii) Green's \
functions and images,\n(iv) finite-difference numerics,\n(v) tree numerics,\n\
(vi) Monte Carlo simulation\n\nas solution techniques, all within the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " environment. A comparison of these approaches and their outcomes is more \
than just an interesting academic exercise, as we shall consider next."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.2 The Concept of Model Risk and the Need for Verification", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakBelow->False],

Cell[CellGroupData[{

Cell["Informal Definitions of Model and Algorithm Risk", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Model risk can arise in a variety of ways. Given a financial instrument, \
we generally formulate a three-stage valuation process:\n\n(a) conceptual \
description;\n(b) mathematical formulation of the conceptual description;\n\
(c) solution of the mathematical model through an analytical and/or numerical \
process.\n\nIt is possible to make mistakes, or perhaps unreasonable \
simplifying assumptions, at any one of these three stages. Each of these \
gives rise to a form of model risk - the risk that the process from reading a \
contract to responding with a set of answers (fair value, Delta, Gamma, ..., \
implied volatility) has led to the wrong answer being supplied. \n\nThis book \
uses ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to look intelligently at the third point, which we might call ",
  StyleBox["algorithm risk",
    FontSlant->"Italic"],
  ". One might think that this can be eliminated just by being careful - the \
real story is that derivative securities are capable of exhibiting some \
diverse forms of mathematical pathology that confound our intuition and play \
havoc with standard or even state-of-the-art algorithms. This book will \
present some familiar and rather less familiar examples of trouble.\n\nI wish \
to emphasize that this is not just a matter of worrying about a level of \
accuracy that is below the \"noise level\" of traders. It is possible to get \
horribly wrong answers even when one thinks that one is being rather careful. \
I also tend to think that it is the job of a Quants team to supply as \
accurate an answer as possible to trading and sales teams. If they want to \
modify your answer by 50 basis points, for whatever reasons, that is their \
responsibility!"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Verification and Resolving Disagreement amongst Experts", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakWithin->Automatic],

Cell[TextData[{
  "A good rule of thumb is that in general you can give what you think is the \
same problem to six different groups and obtain at least three materially \
different answers. In derivatives modelling you can usually get four \
different answers, especially if you include\n\n(a) one analytics fanatic,\n\
(b) one tree-model fanatic,\n(c) one Monte-Carlo fanatic,\n(d) one \
finite-difference fanatic\n\namong the group of people you choose. I use the \
term \"fanatic\" quite deliberately. I once had a party where I was told \
categorically that \"you absolutely had to use martingales - these PDE people \
have got it all wrong\", and I know of at least one PDE book that makes \
almost no reference to simulation, and one excellent martingale book that \
went so far as to describe the PDE approach as \"notorious\". In fact, all \
these methods should give the same answer when applied carefully to a problem \
to which they can all be applied. You can get small but irritating \
non-material differences out of models because of differences of view over \
things like calendar management, units, whether to treat parameters on a \
continuously compounded or annual basis, and so on. That sort of thing can \
and should generally be resolved by getting the risk management team to bash \
heads together. The issues arising from genuine differences between models of \
a different type, and more worryingly still, between models of a similar \
type, require a much more disciplined approach. \n\nThe only way to sort out \
material differences in answers is to perform some form of ",
  StyleBox["verification study",
    FontSlant->"Italic"],
  ". This can be done in various ways, of which the two most important ones \
are:\n\na) comparison of modelling systems with some form of absolutely \
accurate benchmark (typically only available for simpler models, but we must \
check that complex algorithms do work correctly when specialized to simple \
cases);\nb) intercomparison of models and codes when applied to complex \
instruments.\n\nThe basic idea is to define a comprehensive set of test \
problems, and run off a benchmark model and operational systems against as \
many others as are available. \n\nIn other fields of study this is actually \
quite a common process. I hope this text encourages similar efforts within \
the derivatives modelling community."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Resource Management", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The essential requirements of such an approach include ",
  StyleBox["duplication of effort",
    FontSlant->"Italic"],
  ". My view is that no number should be believed unless it can be justified \
as the result of at least two different modelling efforts (or at least that \
the same system has been through such an intercomparison or benchmarking \
study in the past, and that it has been documented). This is generally not \
what management involved in resource allocation want to hear! The point is \
that quality is only achieved through careful and critical testing and \
appraisal. "
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1.3 The Power of Symbolic Computer Calculus", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
In recent years a number of tools have emerged that extend the \
utility of computers from \"number crunching\" to the domain of carrying out \
algebra and calculus. More mature systems combine such symbolic capability \
with advanced numerical algorithms, 2D, 3D and animated graphics, and a \
programming environment. Such tools are particularly relevant to the \
derivatives modelling and testing area due to 

(a) the fact that they have built-in routines for most basic common \
operations;
(b) the ability to do calculus with simple and complex functions, which can \
eliminate many of the headaches of computing Greeks;
(c) the ability to compare analytical symbolic models directly with the \
results from a numerical algorithm;
(d) the ability to visualize the results, and literally see the errors if \
there are any.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Time to Wean Your Team from Spreadsheets and C/C++?", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[TextData[{
  "It is my firm belief that in the long term such hybrid \
symbolic-numeric-graphical systems will replace the spreadsheet and languages \
such as C or C++ for derivatives modelling. Their primary advantage lies in \
the provision of advanced symbolic capabilities. In a numerically based \
system the only two ways to differentiate, to get a Greek such as Delta, are \
(a) to work out the derivative on paper and code it up, (b) to work out the \
value of an option for one or two neighbouring values of the parameter, and \
take differences. Both of these have drawbacks, and the potential for human \
error at the pen and paper stage, and numerical errors arising from numerical \
differentiation problems. Instead we can now ask the computer to do the \
differentiation for us. Such symbolic environments, rich as they are in \
special-function libraries, can dramatically reduce development time below \
that for spreadsheet or C/C++ systems. Furthermore, in the testing and \
verification context they offer a simple integrated environment where a \
numerical model can be compared directly with its exact form. And I mean \
exact - current systems can do arbitrary-precision arithmetic with ease, and \
can do so with complicated \"special functions\". The latest versions of \
these systems, as is the case with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3.0 or later, can also produce mathematical typeset material, so you can \
write your code, write down a mathematical model, test them, plot the \
results, and produce a report for risk management/trading/regulatory bodies \
all within one environment."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Spreadsheet Woes", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[TextData[{
  "There are many problems with spreadsheet environments. An extensive \
discussion of the issues is given by R. Miller (1990) in Chapter 1 of his \
text, ",
  StyleBox["Computer-Aided Financial Analysis",
    FontSlant->"Italic"],
  ", where he outlines several principles to which financial modelling \
environments should conform, and explains why spreadsheets fail to meet them. \
My own simplistic interpretation of Miller's views (i.e., the author's own \
prejudice) is that spreadsheets are the best way yet invented of muddling up \
input data, models, and output data. More seriously, their fabled capability \
for doing \"what-if\" calculations is at once both erroneous and misleading. \
In the particular context of derivatives modelling, the modelling of Greeks \
within spreadsheets by revaluing for neighbouring values of the parameters is \
an abomination. This is not to say that computing differences ",
  StyleBox["in addition",
    FontSlant->"Italic"],
  " to partial derivatives is not a valuable exercise - this separate \
information can reveal interesting pathology. The point is that you should \
use exact calculus wherever possible to extract partial derivatives. As \
Miller points out, the \"what-if\" concept is also limited to numerical \
variations. We want a system where we can also vary structural properties \
(\"what if American rather than European\"). This leads rather naturally to \
the desire for an object-oriented approach.\n\nThere is one virtue of the \
spreadsheet environment, and that is the tabular user interface. There is no \
doubt, that for instruments requiring a small number of input and output \
parameters, such a table of data for several such instruments is extremely \
useful. However, even the interface virtues of spreadsheets are strained by \
the complexities of instruments such as Convertible Bonds. The information \
relevant to one such bond can easily require several interlinked sheets. "
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Flushing POO Away?", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[TextData[{
  "Not for nothing, in my view, is \"POO\" the Spanish acronym for \
object-oriented programming, rather than OOP. C++ has become entrenched as a \
modelling system in many finance houses. In the view of the author, the job \
of a \"Quant\" is to analyse, test and price financial instruments as quickly \
and reliably as possible to respond to trading needs. There can be no greater \
waste of a Quant's time than having to write and debug algorithms in C++, let \
alone waste further energy embedding them in a spreadsheet. If one needs to \
do a highly numerically intensive model for which a system such as ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " may, at least for the present, be inadequate, the solution, at least for \
now, is to use a system properly designed for the task of numerical \
computation, such as highly optimized FORTRAN 90. Objections to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " based on speed of execution are now essentially irrelevant due to \
improvements in low-price high-speed CPUs, and the increased efficiency of \
the internal compiled numerical environment in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". C++ systems are much slower to write, slow to compile, and their \
execution time advantage is now marginal compared to a model suitably adapted \
to the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " internal compiler. The great dream of system managers is that once the \
headaches of building a library of core C++ library routines are overcome, \
modellers can build new algorithms quickly. Sadly, this is only a dream. OOP \
projects mysteriously seem to require massive increases in headcounts and \
budgets, yet are nevertheless the beloved of system managers.\n\nThis is NOT \
to say that an object-oriented approach is a bad idea - the problem is \
getting the dream back on track. My view is that the base-line system for \
object-oriented derivatives development should be at the level of a system \
such as ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", where there is a symbolic capability, and already a huge collection of \
basic objects already in place with their mathematics sorted out. An \
excellent question to ask an OOP-based derivative team is how much time they \
have wasted coding up interpolation, the cumulative normal distribution, \
Greeks and all kinds of special functions and capabilities that already exist \
in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". In this text I have not attempted an object-oriented approach. My focus \
has been on making sure that the answers are correct. I leave it to others to \
adopt some of these methods and algorithms into a more structured \
environment."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "The Use of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " "
}], "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The power of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " as an investigative tool becomes clear in this context. We can define a \
series of test problems and use the special-function capabilities of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to characterize the solution exactly, using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s unlimited-precision arithmetic to get the results with as much accuracy \
as we desire. Next we use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s symbolic calculus capabilities to define the Greeks by differentiation. \
Then, for more complex models, we build tree, Monte Carlo and \
finite-difference models within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". Then we make a comparison, within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", between these analytical forms and the numerical solutions, and the \
solutions from other company systems. The results are compared, and the \
reasons for any differences or other problems are explored and resolved. \n"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Challenge!", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "To those who find my comments on traditional modelling systems extreme or \
ill informed, I issue this challenge. There is now available an exact \
solution for the continuously and arithmetically averaged Asian option. This \
is the Geman-Yor model described in Chapter 10. I got this up and running in \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", and tested it, in a total of 6 hours. To the first person who can \
implement this in C++, and get answers accurate to 4 significant figures over \
a wide range of parameters, I offer a bottle of quality vintage champagne. If \
they can do it in less than 40 hours of coding time, two bottles. To the \
first person who can get it working as accurately in a spreadsheet ",
  StyleBox["alone",
    FontSlant->"Italic"],
  ", I offer a case of good red wine, however long it takes. This offer is \
conditional on me receiving and satisfactorily testing the implementation, \
and being allowed to distribute it to others - the mathematics underlying \
that model is so beautiful it deserves wide appreciation of its power."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
1.4 A Case Study in Chaos - Does Implied Volatility Make Any Sense?\
\
\>", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "As an illustration of something that is easy to explore in a system such \
as ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", we give an example of how one can investigate a topic within a system \
combining symbolic models and graphics. The topic chosen is \"implied \
volatility\", and the point is to make it clear just how infrequently this \
concept makes any sense. This is important both from a mathematical point of \
view, as we need to note when a function cannot be inverted as it is \"many \
to one\", and from a trading point of view - implied volatility is often used \
as a key parameter.\n\nTraders are persistent in asking for implied \
volatilities. You can give them a number, but it might not make any sense at \
all. Or rather, you can feed a market price for an instrument to a system \
which does all sorts of fancy things to find the volatility consistent with \
the market price given all the other data, and this volatility can make no \
sense at all. It is a good idea to list the nice situation and some of the \
horrors that can arise. I emphasize that these are not issues linked to a \
particular solution method, such as Newton-Raphson or bisection, where \
particular problems linked to the solution method may arise. These are \
matters of principle that will cause trouble whatever solution technique is \
employed. I also wish to emphasize that although the examples presented below \
may seem somewhat contrived, the type of phenomenon described can easily \
creep into less obviously pathological derivatives as new features are added. \
"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Things Are OK", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "When the valuation is a monotonic function of volatility (strictly \
increasing or decreasing), there is usually no problem. The simple case of a \
vanilla European Call option, as developed by Black and Scholes (1973), falls \
into this category. Let's build a symbolic model of this option and plot the \
value as a function of volatility - we take this opportunity to show how easy \
it is to write down a model in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ":"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Norm[x_] := 1/2*(1 + Erf[x/Sqrt[2]]); 
done[s_, \[Sigma]_, k_, t_, r_, q_] := 
((r - q)*t + Log[s/k])/(\[Sigma]*Sqrt[t]) + (\[Sigma]*Sqrt[t])/2; 
dtwo[s_, \[Sigma]_, k_, t_, r_, q_] := 
done[s, \[Sigma], k, t, r, q] - \[Sigma]*Sqrt[t]; 
  
BlackScholesCall[s_, k_, v_, r_, q_, t_] := 
s*Exp[-q*t]*Norm[done[s, v, k, t, r, q]] - k*Exp[-r*t]*Norm[dtwo[s, v, k, t, \
r, q]]; \
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We plot the option value as a function of the volatility, for an \
option where the strike is at 10, the underlying at 11, for a vanilla \
Euro-Call with one year to expiry, 5% risk-free rate (continuously \
compounded) and zero dividends:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      BlackScholesCall[11, \ 10, \ vol, \ 0.05, \ 0, \ 1], \ {vol, \ 0.05, \ 
        0.4}, \ PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.112245 2.72109 -0.873414 0.59687 [
[.02381 .30783 -12 -9 ]
[.02381 .30783 12 0 ]
[.29592 .30783 -12 -9 ]
[.29592 .30783 12 0 ]
[.43197 .30783 -9 -9 ]
[.43197 .30783 9 0 ]
[.56803 .30783 -12 -9 ]
[.56803 .30783 12 0 ]
[.70408 .30783 -9 -9 ]
[.70408 .30783 9 0 ]
[.84014 .30783 -12 -9 ]
[.84014 .30783 12 0 ]
[.97619 .30783 -9 -9 ]
[.97619 .30783 9 0 ]
[.14736 .08158 -18 -4.5 ]
[.14736 .08158 0 4.5 ]
[.14736 .20095 -18 -4.5 ]
[.14736 .20095 0 4.5 ]
[.14736 .4397 -18 -4.5 ]
[.14736 .4397 0 4.5 ]
[.14736 .55907 -18 -4.5 ]
[.14736 .55907 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .32033 m
.02381 .32658 L
s
[(0.05)] .02381 .30783 0 1 Mshowa
.29592 .32033 m
.29592 .32658 L
s
[(0.15)] .29592 .30783 0 1 Mshowa
.43197 .32033 m
.43197 .32658 L
s
[(0.2)] .43197 .30783 0 1 Mshowa
.56803 .32033 m
.56803 .32658 L
s
[(0.25)] .56803 .30783 0 1 Mshowa
.70408 .32033 m
.70408 .32658 L
s
[(0.3)] .70408 .30783 0 1 Mshowa
.84014 .32033 m
.84014 .32658 L
s
[(0.35)] .84014 .30783 0 1 Mshowa
.97619 .32033 m
.97619 .32658 L
s
[(0.4)] .97619 .30783 0 1 Mshowa
.125 Mabswid
.05102 .32033 m
.05102 .32408 L
s
.07823 .32033 m
.07823 .32408 L
s
.10544 .32033 m
.10544 .32408 L
s
.13265 .32033 m
.13265 .32408 L
s
.18707 .32033 m
.18707 .32408 L
s
.21429 .32033 m
.21429 .32408 L
s
.2415 .32033 m
.2415 .32408 L
s
.26871 .32033 m
.26871 .32408 L
s
.32313 .32033 m
.32313 .32408 L
s
.35034 .32033 m
.35034 .32408 L
s
.37755 .32033 m
.37755 .32408 L
s
.40476 .32033 m
.40476 .32408 L
s
.45918 .32033 m
.45918 .32408 L
s
.48639 .32033 m
.48639 .32408 L
s
.51361 .32033 m
.51361 .32408 L
s
.54082 .32033 m
.54082 .32408 L
s
.59524 .32033 m
.59524 .32408 L
s
.62245 .32033 m
.62245 .32408 L
s
.64966 .32033 m
.64966 .32408 L
s
.67687 .32033 m
.67687 .32408 L
s
.73129 .32033 m
.73129 .32408 L
s
.7585 .32033 m
.7585 .32408 L
s
.78571 .32033 m
.78571 .32408 L
s
.81293 .32033 m
.81293 .32408 L
s
.86735 .32033 m
.86735 .32408 L
s
.89456 .32033 m
.89456 .32408 L
s
.92177 .32033 m
.92177 .32408 L
s
.94898 .32033 m
.94898 .32408 L
s
.25 Mabswid
0 .32033 m
1 .32033 L
s
.15986 .08158 m
.16611 .08158 L
s
[(1.6)] .14736 .08158 1 0 Mshowa
.15986 .20095 m
.16611 .20095 L
s
[(1.8)] .14736 .20095 1 0 Mshowa
.15986 .4397 m
.16611 .4397 L
s
[(2.2)] .14736 .4397 1 0 Mshowa
.15986 .55907 m
.16611 .55907 L
s
[(2.4)] .14736 .55907 1 0 Mshowa
.125 Mabswid
.15986 .11142 m
.16361 .11142 L
s
.15986 .14127 m
.16361 .14127 L
s
.15986 .17111 m
.16361 .17111 L
s
.15986 .2308 m
.16361 .2308 L
s
.15986 .26064 m
.16361 .26064 L
s
.15986 .29048 m
.16361 .29048 L
s
.15986 .35017 m
.16361 .35017 L
s
.15986 .38001 m
.16361 .38001 L
s
.15986 .40986 m
.16361 .40986 L
s
.15986 .46954 m
.16361 .46954 L
s
.15986 .49939 m
.16361 .49939 L
s
.15986 .52923 m
.16361 .52923 L
s
.15986 .05173 m
.16361 .05173 L
s
.15986 .02189 m
.16361 .02189 L
s
.15986 .58892 m
.16361 .58892 L
s
.25 Mabswid
.15986 0 m
.15986 .61803 L
s
.5 Mabswid
.02381 .01472 m
.02846 .01479 L
.03279 .01487 L
.03754 .01499 L
.04262 .01514 L
.053 .01558 L
.06244 .01615 L
.07283 .01698 L
.0841 .01816 L
.10458 .02109 L
.12442 .02497 L
.14545 .0302 L
.18328 .04236 L
.22357 .05876 L
.26234 .07738 L
.30357 .09967 L
.34327 .12313 L
.38147 .14719 L
.42211 .17412 L
.46124 .20112 L
.50282 .23077 L
.54289 .26011 L
.58144 .28894 L
.62244 .32016 L
.66193 .35067 L
.6999 .38037 L
.74033 .41232 L
.77924 .44334 L
.8206 .47657 L
.86044 .5088 L
.89877 .53996 L
.93955 .57328 L
.97619 .60332 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{198.125, 121.938},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004Y0000]Q000`40O003h00Oogoo:Woo003oOolZ
Ool00?moobYoo`00ogoo:Woo003oOolZOol003Moo`03001oogoo0>moo`00=goo00<007ooOol0kgoo
000gOol00`00Oomoo`3_Ool003Moo`03001oogoo0>moo`004Woo40005Goo00<007ooOol0kgoo000R
Ool4000AOol00`00Oomoo`3_Ool002Ioo`H000]oo`800?1oo`00;7oo1@001Woo00<007ooOol0kgoo
000aOol500000goo001oo`3`Ool003Ioo`@00>moo`00=goo00<007ooOol00`00k7oo000gOol00`00
Oomoo`03Ool3003YOol003Moo`03001oogoo00Ioo`800>Moo`00=goo00<007ooOol027oo0`00i7oo
000gOol2000<Ool2003ROol003Moo`03001oogoo00eoo`800>1oo`00=goo00<007ooOol03goo0P00
gWoo000gOol00`00Oomoo`0AOol2003LOol003Moo`03001oogoo01=oo`<00=Uoo`00=goo00<007oo
Ool05Woo0P00egoo000KOol50005Ool30005Ool40006Ool00`00Oomoo`0HOol2003EOol001eoo`03
001oogoo00Eoo`<000Aoo`03001oogoo009oo`03001oogoo00=oo`03001oogoo01Yoo`800==oo`00
7Goo00<007ooOol01Goo0`0017oo00<007ooOol00Woo00<007ooOol00goo0`0077oo0P00dGoo000M
Ool00`00Oomoo`0<Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`0NOol2003?Ool001eo
o`03001oogoo00aoo`8000=oo`03001oogoo00=oo`03001oogoo021oo`03001oogoo0<aoo`007Goo
00<007ooOol037oo00<007oo00000P001Woo00<007ooOol08Goo0P00c7oo000MOol00`00Oomoo`0=
Ool00`00Oomoo`07Ool00`00Oomoo`0SOol2003:Ool001]oo`<000moo`8000Qoo`03001oogoo02Eo
o`800<Qoo`0077oo0P004Goo0`001Goo00<007ooOol09goo00<007ooOol0aGoo000gOol00`00Oomo
o`0XOol20035Ool003Moo`03001oogoo02Yoo`800<=oo`00=goo0P00;Goo00<007ooOol0`7oo000g
Ool00`00Oomoo`0]Ool20030Ool003Moo`03001oogoo02moo`03001oogoo0;eoo`00=goo00<007oo
Ool0<7oo0P00_Goo000gOol00`00Oomoo`0bOol00`00Oomoo`2jOol003Moo`03001oogoo03=oo`80
0;Yoo`00=goo00<007ooOol0=Goo0P00^7oo000gOol00`00Oomoo`0gOol00`00Oomoo`2eOol003Mo
o`8003Uoo`800;Eoo`00=goo00<007ooOol0>Woo00<007ooOol0/Woo000gOol00`00Oomoo`0kOol2
002bOol003Moo`03001oogoo03eoo`03001oogoo0:moo`00=goo00<007ooOol0?Woo0P00[goo000g
Ool00`00Oomoo`10Ool2002]Ool003Moo`03001oogoo049oo`03001oogoo0:Yoo`00=goo00<007oo
Ool0@goo0P00ZWoo000gOol20016Ool00`00Oomoo`2WOol003Moo`03001oogoo04Ioo`800:Moo`00
=goo00<007ooOol0B7oo00<007ooOol0Y7oo000gOol00`00Oomoo`19Ool00`00Oomoo`2SOol003Mo
o`03001oogoo04Yoo`800:=oo`00=goo00<007ooOol0C7oo00<007ooOol0X7oo000KOol50005Ool3
0005Ool40006Ool00`00Oomoo`1=Ool2002POol001eoo`03001oogoo00Eoo`<000Aoo`03001oogoo
009oo`03001oogoo00=oo`03001oogoo04moo`03001oogoo09eoo`007Goo00<007ooOol01Goo0`00
17oo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0D7oo00<007ooOol0W7oo000MOol00`00
Oomoo`0<Ool00`00Oomoo`02Ool00`00Oomoo`03Ool3001AOol2002LOol001eoo`03001oogoo00eo
o`@000Ioo`03001oogoo05=oo`03001oogoo09Uoo`007Goo00<007ooOol037oo00<007ooOol00Woo
00<007ooOol00goo00<007ooOol0E7oo0P00VGoo000MOol00`00Oomoo`0<Ool00`00Oomoo`02Ool0
0`00Oomoo`03Ool00`00Oomoo`1FOol00`00Oomoo`2FOol001]oo`<000ioo`03001oogoo009oo`03
001oogoo00=oo`03001oogoo05Moo`8009Ioo`0077oo0P003goo10001Woo00<007ooOol0FGoo00<0
07ooOol0Tgoo000gOol00`00Oomoo`1JOol2002COol003Moo`03001oogoo05aoo`03001oogoo091o
o`00=goo0P00GWoo0P00T7oo000gOol00`00Oomoo`1OOol00`00Oomoo`2=Ool003Moo`03001oogoo
061oo`03001oogoo08aoo`00=goo00<007ooOol0HGoo0P00S7oo000gOol00`00Oomoo`1SOol00`00
Oomoo`29Ool003Moo`03001oogoo06Aoo`8008Uoo`00=goo00<007ooOol0IWoo00<007ooOol0QWoo
000gOol00`00Oomoo`1WOol00`00Oomoo`25Ool003Moo`8006Uoo`8008Eoo`00=goo00<007ooOol0
JWoo00<007ooOol0PWoo000gOol00`00Oomoo`1[Ool00`00Oomoo`21Ool000=oo`8000Moo`<000Io
o`8000Ioo`@001Ioo`03001oogoo01Aoo`8000Moo`<000Eoo`D000Aoo`@000aoo`8000Moo`<000Ao
o`H000aoo`8000Moo`@000=oo`H000Aoo`@000aoo`8000Moo`<000Eoo`@000aoo`8000Moo`<000Eo
o`@000Eoo`@000eoo`8000Moo`<000Moo`<000=oo`000Woo00@007ooOol000Ioo`<000Eoo`04001o
ogoo0004Ool00`00Oomoo`02Ool00`00Oomoo`0COol00`00Oomoo`0COol01000Oomoo`001Woo0`00
1goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol027oo00@007ooOol000Ioo`<000Eoo`05
001oogooOol0000;Ool01000Oomoo`001Woo0`0000=oo`0000000Woo00D007ooOomoo`0000=oo`03
001oogoo009oo`03001oogoo00Qoo`04001oogoo0006Ool30004Ool00`00Oomoo`02Ool00`00Oomo
o`08Ool01000Oomoo`001Woo0`0017oo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo
00Uoo`04001oogoo0006Ool30008Ool00`00Oomoo`02Ool00003Ool007oo00=oo`03001oogoo00=o
o`<000Aoo`03001oogoo009oo`03001oogoo00Ioo`03001oogoo01=oo`03001oogoo019oo`03001o
ogoo009oo`03001oogoo00=oo`<000Moo`03001oogoo00Qoo`03001oogoo00Moo`03001oogoo009o
o`03001oogoo00=oo`<000Ioo`03001oogoo00]oo`03001oogoo009oo`03001oogoo00=oo`<000=o
o`04001oogoo000;Ool00`00Oomoo`07Ool00`00Oomoo`02Ool00`00Oomoo`03Ool30009Ool00`00
Oomoo`07Ool00`00Oomoo`02Ool00`00Oomoo`03Ool30009Ool00`00Oomoo`06Ool00`00Oomoo`08
Ool00`00Oomoo`02Ool00`00Oomoo`03Ool30004Ool60003Ool00003Ool007oo00=oo`03001oogoo
00Yoo`03001oogoo009oo`03001oogoo00Ioo`03001oogoo01=oo`03001oogoo019oo`03001oogoo
009oo`03001oogoo00eoo`03001oogoo00Qoo`03001oogoo00Moo`03001oogoo009oo`03001oogoo
00eoo`03001oogoo00Yoo`03001oogoo009oo`03001oogoo00Yoo`04001oogoo000:Ool00`00Oomo
o`07Ool00`00Oomoo`02Ool00`00Oomoo`0?Ool00`00Oomoo`07Ool00`00Oomoo`02Ool00`00Oomo
o`0?Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`02Ool00`00Oomoo`0:Ool01@00Oomo
ogoo000017oo00000goo001oo`03Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool01@00Oomoogoo0000
17oo00<007ooOol04goo00<007ooOol04Woo00<007ooOol00Woo00<007ooOol03Goo00<007ooOol0
0goo00<007ooOol00Woo00<007ooOol01goo00<007ooOol00Woo00<007ooOol03Woo00<007ooOol0
2Goo00<007ooOol00Woo00<007ooOol02goo00@007ooOol000Aoo`03001oogoo009oo`03001oogoo
00Moo`03001oogoo009oo`03001oogoo00aoo`<000Yoo`03001oogoo009oo`03001oogoo00aoo`<0
00Aoo`03001oogoo009oo`03001oogoo00Qoo`03001oogoo009oo`03001oogoo00]oo`04001oogoo
0004Ool00003Ool007oo00=oo`03001oogoo00Yoo`03001oogoo009oo`04001oogooOol5000FOol2
000COol00`00Oomoo`02Ool00`00Oomoo`0=Ool00`00Oomoo`03Ool5000:Ool00`00Oomoo`02Ool0
0`00Oomoo`0?Ool00`00Oomoo`08Ool00`00Oomoo`02Ool00`00Oomoo`0<Ool200000goo001oo`02
Ool5000:Ool00`00Oomoo`02Ool00`00Oomoo`0?Ool00`00Oomoo`07Ool00`00Oomoo`02Ool00`00
Oomoo`0?Ool01000Oomoogoo1@002goo00<007ooOol00Woo00<007ooOol037oo00<007oo000017oo
00000goo001oo`03Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool01@00Oomoogoo00006Woo00<007oo
Ool04Woo00<007ooOol00Woo00<007ooOol03Goo00<007ooOol00goo00<007ooOol037oo00<007oo
Ool00Woo00<007ooOol03goo00<007ooOol027oo00<007ooOol00Woo00<007ooOol03Woo0P000goo
00<007ooOol037oo00<007ooOol00Woo00<007ooOol03goo00<007ooOol01goo00<007ooOol00Woo
00<007ooOol03goo00D007ooOomoo`0000moo`03001oogoo009oo`03001oogoo00aoo`03001oo`00
00Aoo`000Woo00@007ooOol000ioo`04001oogoo0004Ool00`00Oomoo`0HOol00`00Oomoo`0COol0
1000Oomoo`003Woo0`001Goo00<007ooOol03Goo00@007ooOol000eoo`03001oogoo009oo`03001o
ogoo00Uoo`04001oogoo000=Ool00`00Oomoo`02Ool01@00Oomoogoo00003goo00@007ooOol000eo
o`03001oogoo009oo`03001oogoo00Qoo`04001oogoo000=Ool00`00Oomoo`02Ool01@00Oomoogoo
000047oo00@007ooOol0011oo`8000Aoo`000goo0P0047oo0P001Goo1@005Woo00<007ooOol057oo
0P0047oo0P001Goo1@0037oo0P003goo10003Goo0P003goo100000Aoo`00001oo`D000aoo`8000mo
o`@000aoo`8000moo`@000Aoo`D000eoo`80019oo`03001oogoo009oo`00=goo00<007ooOol0N7oo
00<007ooOol0M7oo000gOol00`00Oomoo`1iOol00`00Oomoo`1cOol003Moo`03001oogoo07Yoo`80
07=oo`00=goo00<007ooOol0O7oo00<007ooOol0L7oo000gOol00`00Oomoo`1mOol00`00Oomoo`1_
Ool000]oool001P000Moo`004Woo00<007ooOol017oo00<007ooOol01Goo00<007ooOol017oo00<0
07ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<0
07ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<0
07ooOol017oo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<0
07ooOol017oo00<007ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<007oo000017oo00<0
07ooOol01Goo00<007ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<007ooOol017oo00<0
07ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol017oo00<0
07ooOol01Goo00<007ooOol017oo00<007ooOol037oo000gOol00`00Oomoo`21Ool00`00Oomoo`1[
Ool003Moo`03001oogoo089oo`8006]oo`00=goo00<007ooOol0Q7oo00<007ooOol0J7oo000gOol0
0`00Oomoo`25Ool2001XOol003Moo`03001oogoo08Moo`03001oogoo06Eoo`00=goo00<007ooOol0
R7oo00<007ooOol0I7oo000gOol2002:Ool2001TOol003Moo`03001oogoo08]oo`03001oogoo065o
o`00=goo00<007ooOol0S7oo00<007ooOol0H7oo000gOol00`00Oomoo`2=Ool00`00Oomoo`1OOol0
03Moo`03001oogoo08ioo`8005moo`00=goo00<007ooOol0T7oo00<007ooOol0G7oo000gOol00`00
Oomoo`2AOol00`00Oomoo`1KOol003Moo`03001oogoo099oo`03001oogoo05Yoo`00=goo0P00U7oo
0P00FWoo000gOol00`00Oomoo`2EOol00`00Oomoo`1GOol003Moo`03001oogoo09Ioo`03001oogoo
05Ioo`00=goo00<007ooOol0Ugoo0P00EWoo000gOol00`00Oomoo`2IOol00`00Oomoo`1COol003Mo
o`03001oogoo09Yoo`03001oogoo059oo`00=goo00<007ooOol0Vgoo0P00DWoo000gOol00`00Oomo
o`2MOol00`00Oomoo`1?Ool003Moo`03001oogoo09ioo`03001oogoo04ioo`00=goo0P00X7oo0P00
CWoo000gOol00`00Oomoo`2QOol00`00Oomoo`1;Ool003Moo`03001oogoo0:9oo`03001oogoo04Yo
o`00=goo00<007ooOol0Xgoo00<007ooOol0BGoo000gOol00`00Oomoo`2TOol00`00Oomoo`18Ool0
01Yoo`H000Eoo`<000Aoo`H000Eoo`03001oogoo0:Eoo`03001oogoo04Moo`006goo00D007ooOomo
o`0000Eoo`<000Eoo`05001oogooOol00005Ool00`00Oomoo`2VOol00`00Oomoo`16Ool001aoo`03
001oogoo00Ioo`<000Ioo`03001oogoo00Ioo`03001oogoo0:Moo`03001oogoo04Eoo`007Goo00<0
07ooOol03goo00<007ooOol01Goo0`00Z7oo00<007ooOol0A7oo000NOol00`00Oomoo`0?Ool00`00
Oomoo`04Ool00`00Oomoo`2YOol20014Ool001moo`03001oogoo00moo`03001oogoo00=oo`03001o
ogoo0:]oo`03001oogoo045oo`007goo00<007ooOol03goo00<007ooOol00goo00<007ooOol0[7oo
00<007ooOol0@7oo000JOol00`00Oomoo`02Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool00`00Oomo
o`03Ool00`00Oomoo`2]Ool20010Ool001]oo`@000ioo`@000Ioo`03001oogoo0:moo`03001oogoo
03eoo`00=goo00<007ooOol0/7oo00<007ooOol0?7oo000gOol00`00Oomoo`2aOol2000lOol003Mo
o`800;Aoo`03001oogoo03Uoo`00=goo00<007ooOol0]7oo00<007ooOol0>7oo000gOol00`00Oomo
o`2eOol2000hOol003Moo`03001oogoo0;Moo`03001oogoo03Eoo`00=goo00<007ooOol0^7oo00<0
07ooOol0=7oo000gOol00`00Oomoo`2iOol00`00Oomoo`0cOol003Moo`03001oogoo0;Yoo`8003=o
o`00=goo00<007ooOol0_7oo00<007ooOol0<7oo000gOol2002nOol00`00Oomoo`0_Ool003Moo`03
001oogoo0;ioo`03001oogoo02ioo`00=goo00<007ooOol0_goo00<007ooOol0;Goo000gOol00`00
Oomoo`30Ool2000]Ool003Moo`03001oogoo0<9oo`03001oogoo02Yoo`00=goo00<007ooOol0`goo
00<007ooOol0:Goo000gOol00`00Oomoo`34Ool00`00Oomoo`0XOol003Moo`03001oogoo0<Eoo`80
02Qoo`00=goo00<007ooOol0agoo00<007ooOol09Goo000gOol20039Ool00`00Oomoo`0TOol003Mo
o`03001oogoo0<Uoo`03001oogoo02=oo`00=goo00<007ooOol0bWoo00<007ooOol08Woo000gOol0
0`00Oomoo`3;Ool2000ROol003Moo`03001oogoo0<eoo`03001oogoo01moo`00=goo00<007ooOol0
cWoo00<007ooOol07Woo000JOol60005Ool30007Ool30005Ool00`00Oomoo`3?Ool00`00Oomoo`0M
Ool001]oo`05001oogooOol00005Ool30008Ool00`00Oomoo`04Ool00`00Oomoo`3@Ool2000MOol0
01aoo`03001oogoo00Ioo`<000Aoo`H000Eoo`<00=9oo`03001oogoo01Yoo`007Goo00<007ooOol0
37oo00D007ooOomoo`0000Ioo`03001oogoo0==oo`03001oogoo01Uoo`007Woo00<007ooOol037oo
00@007ooOol000Ioo`03001oogoo0=Aoo`03001oogoo01Qoo`007goo00<007ooOol037oo00<007oo
00001Woo00<007ooOol0eGoo00<007ooOol05goo000OOol00`00Oomoo`0<Ool00`00Ool00006Ool0
0`00Oomoo`3FOol2000GOol001Yoo`03001oogoo009oo`03001oogoo00eoo`8000Ioo`03001oogoo
0=Qoo`03001oogoo01Aoo`006goo10004Goo00<007ooOol017oo00<007ooOol0fGoo00<007ooOol0
4goo000gOol00`00Oomoo`3JOol00`00Oomoo`0BOol003Moo`800=aoo`80019oo`00=goo00<007oo
Ool0gGoo00<007ooOol03goo000gOol00`00Oomoo`3NOol00`00Oomoo`0>Ool003Moo`03001oogoo
0=moo`03001oogoo00eoo`00=goo00<007ooOol0h7oo00<007ooOol037oo000gOol00`00Oomoo`3_
Ool003Moo`03001oogoo0>moo`00=goo00<007ooOol0kgoo000gOol00`00Oomoo`3_Ool00?moobYo
o`00ogoo:Woo003oOolZOol00?moobYoo`00ogoo:Woo0000\
\>"],
  ImageRangeCache->{{{0, 296.188}, {181.875, 0}} -> {0.0261966, 1.43284, 
  0.00198349, 0.00904259}}]
}, Open  ]],

Cell[TextData[{
  "We can also build an implied volatility calculator in a moment (",
  StyleBox["FindRoot ",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "is a combination Newton-Raphson/secant solver):"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
BlackScholesCallImpVol[s_,k_,r_,q_,t_,price_] :=sd /. \
FindRoot[BlackScholesCall[s,k, sd, r, q, t]==price,{sd,0.2}];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Because the function is strictly increasing, we get a one-one \
mapping between prices and volatilities:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesCall[11, \ 10, \ 0.25, \ 0.05, \ 0, \ 1]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1.93050915293114044`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesCallImpVol[11, \ 10, \ \ 0.05, \ 0, \ 1, \ 1.93051]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.25000024771142284`\)], "Output"]
}, Open  ]],

Cell["\<\
The results are also stable - a small change in the market price \
causes a small shift in the implied volatility:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesCallImpVol[11, \ 10, \ \ 0.05, \ 0, \ 1, \ 1.92]\)], "Input",\

  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.246920744498679223`\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Things Are Sometimes OK, Sometimes Highly Unstable", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
It does not take much to mess up the calculation of implied \
volatility. Let's add a dilution effect to the Black-Scholes model, providing \
a simple model for a warrant. This is one of the simpler warrant-pricing \
models discussed by Lauterbach and Schultz (1990).\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(WarrantEqn[p_, k_, sd_, r_, t_, warprice_, \ shares_, \ warrants_, \n
        shperwar_, \ 
        q_]\  := \((shares*shperwar/\((shares + shperwar*warrants)\))\)*\n
        BlackScholesCall[p*Exp[\(-q\)*t] + warrants*warprice/shares, k, sd, 
          r, 0, t]\n\), 
    \(WarrantValue[p_, k_, sd_, r_, t_, \ shares_, \ warrants_, \ shperwar_, 
        q_]\  := \n
      warprice\  /. \ 
        FindRoot[\n
          warprice\  == \ 
            WarrantEqn[p, k, sd, r, t, warprice, shares, warrants, 
              shperwar, \ q], {warprice, \ 10}]\)}], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
When we plot the result for price against volatility, we see that \
there is a nice monotonic region for larger volatilities, but that the curve \
becomes very flat as the volatility dips below 20%. The corresponding \
inversion for the implied volatility then becomes either unstable or \
impossible.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      WarrantValue[15, \ 10, \ vol, \ 0.05, \ 1, \ 1000, \ 100, \ 1, \ 
        0], \ {vol, \ 0.05, \ 0.4}, \ PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.112245 2.72109 -12.8052 2.33612 [
[.02381 .03094 -12 -9 ]
[.02381 .03094 12 0 ]
[.29592 .03094 -12 -9 ]
[.29592 .03094 12 0 ]
[.43197 .03094 -9 -9 ]
[.43197 .03094 9 0 ]
[.56803 .03094 -12 -9 ]
[.56803 .03094 12 0 ]
[.70408 .03094 -9 -9 ]
[.70408 .03094 9 0 ]
[.84014 .03094 -12 -9 ]
[.84014 .03094 12 0 ]
[.97619 .03094 -9 -9 ]
[.97619 .03094 9 0 ]
[.14736 .16024 -24 -4.5 ]
[.14736 .16024 0 4.5 ]
[.14736 .27705 -18 -4.5 ]
[.14736 .27705 0 4.5 ]
[.14736 .39385 -24 -4.5 ]
[.14736 .39385 0 4.5 ]
[.14736 .51066 -18 -4.5 ]
[.14736 .51066 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .04344 m
.02381 .04969 L
s
[(0.05)] .02381 .03094 0 1 Mshowa
.29592 .04344 m
.29592 .04969 L
s
[(0.15)] .29592 .03094 0 1 Mshowa
.43197 .04344 m
.43197 .04969 L
s
[(0.2)] .43197 .03094 0 1 Mshowa
.56803 .04344 m
.56803 .04969 L
s
[(0.25)] .56803 .03094 0 1 Mshowa
.70408 .04344 m
.70408 .04969 L
s
[(0.3)] .70408 .03094 0 1 Mshowa
.84014 .04344 m
.84014 .04969 L
s
[(0.35)] .84014 .03094 0 1 Mshowa
.97619 .04344 m
.97619 .04969 L
s
[(0.4)] .97619 .03094 0 1 Mshowa
.125 Mabswid
.05102 .04344 m
.05102 .04719 L
s
.07823 .04344 m
.07823 .04719 L
s
.10544 .04344 m
.10544 .04719 L
s
.13265 .04344 m
.13265 .04719 L
s
.18707 .04344 m
.18707 .04719 L
s
.21429 .04344 m
.21429 .04719 L
s
.2415 .04344 m
.2415 .04719 L
s
.26871 .04344 m
.26871 .04719 L
s
.32313 .04344 m
.32313 .04719 L
s
.35034 .04344 m
.35034 .04719 L
s
.37755 .04344 m
.37755 .04719 L
s
.40476 .04344 m
.40476 .04719 L
s
.45918 .04344 m
.45918 .04719 L
s
.48639 .04344 m
.48639 .04719 L
s
.51361 .04344 m
.51361 .04719 L
s
.54082 .04344 m
.54082 .04719 L
s
.59524 .04344 m
.59524 .04719 L
s
.62245 .04344 m
.62245 .04719 L
s
.64966 .04344 m
.64966 .04719 L
s
.67687 .04344 m
.67687 .04719 L
s
.73129 .04344 m
.73129 .04719 L
s
.7585 .04344 m
.7585 .04719 L
s
.78571 .04344 m
.78571 .04719 L
s
.81293 .04344 m
.81293 .04719 L
s
.86735 .04344 m
.86735 .04719 L
s
.89456 .04344 m
.89456 .04719 L
s
.92177 .04344 m
.92177 .04719 L
s
.94898 .04344 m
.94898 .04719 L
s
.25 Mabswid
0 .04344 m
1 .04344 L
s
.15986 .16024 m
.16611 .16024 L
s
[(5.55)] .14736 .16024 1 0 Mshowa
.15986 .27705 m
.16611 .27705 L
s
[(5.6)] .14736 .27705 1 0 Mshowa
.15986 .39385 m
.16611 .39385 L
s
[(5.65)] .14736 .39385 1 0 Mshowa
.15986 .51066 m
.16611 .51066 L
s
[(5.7)] .14736 .51066 1 0 Mshowa
.125 Mabswid
.15986 .0668 m
.16361 .0668 L
s
.15986 .09016 m
.16361 .09016 L
s
.15986 .11352 m
.16361 .11352 L
s
.15986 .13688 m
.16361 .13688 L
s
.15986 .1836 m
.16361 .1836 L
s
.15986 .20696 m
.16361 .20696 L
s
.15986 .23033 m
.16361 .23033 L
s
.15986 .25369 m
.16361 .25369 L
s
.15986 .30041 m
.16361 .30041 L
s
.15986 .32377 m
.16361 .32377 L
s
.15986 .34713 m
.16361 .34713 L
s
.15986 .37049 m
.16361 .37049 L
s
.15986 .41721 m
.16361 .41721 L
s
.15986 .44058 m
.16361 .44058 L
s
.15986 .46394 m
.16361 .46394 L
s
.15986 .4873 m
.16361 .4873 L
s
.15986 .02007 m
.16361 .02007 L
s
.15986 .53402 m
.16361 .53402 L
s
.15986 .55738 m
.16361 .55738 L
s
.15986 .58074 m
.16361 .58074 L
s
.15986 .6041 m
.16361 .6041 L
s
.25 Mabswid
.15986 0 m
.15986 .61803 L
s
.5 Mabswid
.02381 .01472 m
.03279 .01472 L
.04262 .01472 L
.04758 .01472 L
.05288 .01472 L
.05546 .01472 L
.05789 .01472 L
.06007 .01472 L
.06131 .01472 L
.06244 .01472 L
.06369 .01472 L
.06487 .01472 L
.06552 .01472 L
.06623 .01472 L
.06747 .01472 L
.06865 .01472 L
.06991 .01472 L
.0711 .01472 L
.07218 .01472 L
.07285 .01472 L
.07356 .01472 L
.07483 .01472 L
.07549 .01472 L
.0762 .01472 L
.07697 .01472 L
.0777 .01472 L
.07837 .01472 L
.07909 .01472 L
.07978 .01472 L
.08042 .01472 L
.08171 .01472 L
.0829 .01472 L
.08414 .01472 L
.08547 .01472 L
.08665 .01472 L
.08789 .01472 L
.08855 .01472 L
.08926 .01472 L
.09053 .01472 L
.09204 .01472 L
.09341 .01472 L
.09485 .01472 L
.09617 .01472 L
.09762 .01472 L
.09916 .01472 L
.10049 .01472 L
.10174 .01472 L
.10311 .01472 L
.10458 .01472 L
.10592 .01472 L
Mistroke
.10713 .01472 L
.10852 .01472 L
.10984 .01472 L
.11218 .01472 L
.1135 .01472 L
.11471 .01472 L
.11727 .01472 L
.11873 .01472 L
.12008 .01472 L
.12282 .01472 L
.12436 .01472 L
.12575 .01472 L
.12833 .01472 L
.13071 .01472 L
.13332 .01472 L
.13608 .01472 L
.13848 .01472 L
.14073 .01472 L
.1432 .01472 L
.14586 .01472 L
.14826 .01472 L
.15083 .01472 L
.15325 .01472 L
.15545 .01472 L
.15804 .01472 L
.16085 .01472 L
.1635 .01472 L
.16594 .01472 L
.17083 .01472 L
.1736 .01472 L
.1762 .01472 L
.18143 .01472 L
.18437 .01472 L
.18705 .01472 L
.19185 .01472 L
.19695 .01472 L
.19904 .01472 L
.2013 .01472 L
.20259 .01472 L
.20378 .01472 L
.20491 .01472 L
.20611 .01472 L
.2109 .01473 L
.21602 .01473 L
.22163 .01474 L
.22686 .01475 L
.23218 .01476 L
.23709 .01477 L
.24237 .01479 L
.24818 .01481 L
Mistroke
.25796 .01486 L
.26332 .01489 L
.26833 .01493 L
.27872 .01504 L
.28859 .01518 L
.29745 .01534 L
.30712 .01556 L
.31745 .01586 L
.32681 .0162 L
.33773 .01669 L
.34778 .01725 L
.36705 .01864 L
.38787 .02072 L
.40671 .02322 L
.42738 .02675 L
.44854 .03133 L
.46782 .03644 L
.50675 .04984 L
.54662 .06828 L
.58497 .09089 L
.62577 .12047 L
.66505 .15446 L
.70282 .19223 L
.74305 .23789 L
.78175 .28699 L
.82291 .34455 L
.86256 .405 L
.90069 .46755 L
.94127 .53863 L
.97619 .60332 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{200.125, 123.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003j0000VQ000`40O003h00OnWoo003jOol00?Yo
o`00nWoo003jOol00004Ool000000006Ool20005Ool30004Ool4000UOol30006Ool20005Ool50002
Ool40008Ool30006Ool20004Ool50008Ool30006Ool20004Ool50003Ool40009Ool30006Ool20005
Ool30009Ool30006Ool20005Ool30004Ool40009Ool30006Ool20006Ool30002Ool00005001oogoo
Ool00005Ool20004Ool01@00Oomoogoo00001goo00<007ooOol08Goo00D007ooOomoo`0000Eoo`80
00Moo`03001oogoo00Ioo`03001oogoo00Aoo`05001oogooOol00005Ool20005Ool01000Oomoo`00
1goo00D007ooOomoo`0000Eoo`8000Eoo`04001oogoo0007Ool00`00Oomoo`05Ool01@00Oomoogoo
00001Goo0P0017oo00D007ooOomoo`0000Moo`05001oogooOol00005Ool20004Ool01@00Oomoogoo
00001goo00<007ooOol01Goo00D007ooOomoo`0000Eoo`8000Moo`03001oogoo005oo`0000D007oo
Oomoo`0000]oo`05001oogooOol00007Ool00`00Oomoo`0?Ool00`00Oomoo`0?Ool01@00Oomoogoo
00003Woo00<007ooOol01Woo00<007ooOol017oo00D007ooOomoo`0000eoo`03001oogoo00Moo`05
001oogooOol0000=Ool00`00Oomoo`07Ool00`00Oomoo`05Ool01@00Oomoogoo00003goo00<007oo
Ool01Goo00D007ooOomoo`0000moo`03001oogoo00Eoo`03001oogoo00Eoo`05001oogooOol0000;
Ool50002Ool00005001oogooOol0000;Ool01@00Oomoogoo00001goo00<007ooOol03goo00<007oo
Ool03goo00D007ooOomoo`0000ioo`03001oogoo00Ioo`03001oogoo00Aoo`05001oogooOol0000>
Ool00`00Oomoo`06Ool01@00Oomoogoo00003Woo00<007ooOol01Woo00<007ooOol01Goo00D007oo
Oomoo`0000moo`03001oogoo00Eoo`05001oogooOol0000?Ool00`00Oomoo`05Ool00`00Oomoo`05
Ool01@00Oomoogoo00002goo00@007ooOol000=oo`0000D007ooOomoo`0000]oo`05001oogooOol0
0003Ool4000BOol00`00Oomoo`0?Ool01@00Oomoogoo00003Woo00<007ooOol00Woo10001goo00D0
07ooOomoo`0000moo`03001oogoo00Eoo`05001oogooOol0000?Ool01000Oomoogoo100027oo00D0
07ooOomoo`0000eoo`8000Qoo`05001oogooOol0000=Ool20004Ool40008Ool01@00Oomoogoo0000
37oo00<007oo00000goo00001@00Oomoogoo00002WooB`002Goo00D007ooOomoo`0000moo`03001o
ogoo00Eoo`05001oogooOol0000?Ool01@00Oomoogoo00002goo00D007ooOomoo`0000moo`03001o
ogoo00Eoo`05001oogooOol0000?Ool01@00Oomoogoo00002goo00D007ooOomoo`0000aoo`03001o
o`0000=oo`0000D007ooOomoo`0000]oo`05001oogooOol00003Ool00`00Oomoo`0COol2000@Ool0
1@00Oomoogoo00003Woo00<007ooOol00Woo00<007oo00003@002goo00D007ooOomoo`0000Moo`05
001oogooOol0000;Ool01@00Oomoogoo00000goo00<007ooOol02Goo00D007ooOomoo`0000]oo`05
001oogooOol00007Ool01@00Oomoogoo00002goo00D007ooOomoo`0000=oo`03001oogoo00Uoo`05
001oogooOol0000=Ool20003Ool00004Ool00000000=Ool30004Ool4000BOol00`00Oomoo`0@Ool3
000=Ool30004Ool40008Ool80008Ool30009Ool3000=Ool30004Ool40009Ool3000=Ool30009Ool3
000=Ool30004Ool40009Ool3000?Ool00`00Oomoo`01Ool002ioo`03001oogoo03]oo`D008Uoo`00
;Woo00<007ooOol0@7oo1000QGoo000^Ool00`00Oomoo`14Ool40021Ool002ioo`03001oogoo04Qo
o`<007ioo`002Gooj`001Woo000?Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`04Ool0
0`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`04Ool00`00Oomoo`03Ool00`00Oomoo`03Ool0
0`00Oomoo`04Ool00`00Oomoo`03Ool00`00Oomoo`04Ool00`00Oomoo`03Ool00`00Oomoo`03Ool0
0`00Oomoo`04Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`04Ool01000Oomoogoo0`00
1Goo00<007ooOol017oo00<007ooOol00goo00<007ooOol017oo00<007ooOol00goo00<007ooOol0
0goo00<007ooOol017oo00<007ooOol00goo00<007ooOol00goo00<007ooOol017oo00<007ooOol0
0goo00<007ooOol017oo00<007ooOol00goo00<007ooOol00goo00<007ooOol017oo00<007ooOol0
0goo00<007ooOol00goo00<007ooOol02Woo000^Ool00`00Oomoo`1AOol2001fOol002ioo`03001o
ogoo05=oo`8007Aoo`00;Woo00<007ooOol0EGoo00<007ooOol0LGoo000^Ool2001GOol2001aOol0
02ioo`03001oogoo05Qoo`8006moo`00;Woo00<007ooOol0FWoo0P00KGoo000^Ool00`00Oomoo`1L
Ool2001[Ool002ioo`03001oogoo05ioo`03001oogoo06Qoo`00;Woo00<007ooOol0Ggoo0P00J7oo
000^Ool2001ROol2001VOol002ioo`03001oogoo06=oo`03001oogoo06=oo`00;Woo00<007ooOol0
I7oo00<007ooOol0HWoo000^Ool00`00Oomoo`1UOol2001ROol002ioo`03001oogoo06Moo`03001o
ogoo05moo`00;Woo0P00JGoo00<007ooOol0GWoo000^Ool00`00Oomoo`1YOol00`00Oomoo`1MOol0
02ioo`03001oogoo06Yoo`8005eoo`00;Woo00<007ooOol0K7oo00<007ooOol0FWoo000^Ool00`00
Oomoo`1]Ool00`00Oomoo`1IOol002ioo`03001oogoo06ioo`03001oogoo05Qoo`00;Woo0P00L7oo
0P00F7oo000^Ool00`00Oomoo`1aOol00`00Oomoo`1EOol002ioo`03001oogoo079oo`03001oogoo
05Aoo`003Woo10001Goo0P001Goo10000goo10001Goo00<007ooOol0Lgoo00<007ooOol0Dgoo000B
Ool00`00Oomoo`02Ool20009Ool00`00Oomoo`04Ool00`00Oomoo`02Ool00`00Oomoo`1dOol00`00
Oomoo`1BOol0019oo`03001oogoo00eoo`03001oogoo00Aoo`03001oogoo009oo`<007Aoo`03001o
ogoo059oo`004Woo00<007ooOol03Goo00<007ooOol017oo00<007ooOol00Woo00<007ooOol0MGoo
00<007ooOol0DGoo000>Ool4000<Ool40003Ool40005Ool00`00Oomoo`1fOol00`00Oomoo`1@Ool0
00ioo`03001oogoo00eoo`03001oogoo00Aoo`03001oogoo00Ioo`03001oogoo07Moo`03001oogoo
04moo`003Woo00<007ooOol03Goo00<007ooOol017oo00<007ooOol01Woo00<007ooOol0N7oo00<0
07ooOol0CWoo000>Ool4000<Ool40003Ool40005Ool00`00Oomoo`1iOol00`00Oomoo`1=Ool002io
o`8007]oo`03001oogoo04aoo`00;Woo00<007ooOol0Ngoo00<007ooOol0Bgoo000^Ool00`00Oomo
o`1lOol00`00Oomoo`1:Ool002ioo`03001oogoo07eoo`03001oogoo04Uoo`00;Woo00<007ooOol0
OWoo00<007ooOol0B7oo000^Ool20020Ool00`00Oomoo`17Ool002ioo`03001oogoo081oo`03001o
ogoo04Ioo`00;Woo00<007ooOol0PGoo00<007ooOol0AGoo000^Ool00`00Oomoo`22Ool00`00Oomo
o`14Ool002ioo`03001oogoo08=oo`03001oogoo04=oo`00;Woo0P00QGoo00<007ooOol0@Woo000^
Ool00`00Oomoo`25Ool00`00Oomoo`11Ool002ioo`03001oogoo08Ioo`03001oogoo041oo`00;Woo
00<007ooOol0QWoo00<007ooOol0@7oo000^Ool00`00Oomoo`27Ool00`00Oomoo`0oOol002ioo`03
001oogoo08Qoo`03001oogoo03ioo`00;Woo0P00RWoo00<007ooOol0?Goo000^Ool00`00Oomoo`29
Ool00`00Oomoo`0mOol002ioo`03001oogoo08Yoo`03001oogoo03aoo`005Woo10001Goo0P001Goo
0`001Goo00<007ooOol0Rgoo00<007ooOol0>goo000JOol00`00Oomoo`02Ool20004Ool01@00Oomo
ogoo000017oo00<007ooOol0S7oo00<007ooOol0>Woo000JOol00`00Oomoo`08Ool01@00Oomoogoo
000017oo0`00S7oo00<007ooOol0>Woo000JOol00`00Oomoo`08Ool01@00Oomoogoo000017oo00<0
07ooOol0SGoo00<007ooOol0>Goo000FOol4000;Ool40005Ool00`00Oomoo`2>Ool00`00Oomoo`0h
Ool001Ioo`03001oogoo00aoo`03001oogoo00Ioo`03001oogoo08moo`03001oogoo03Moo`005Woo
00<007ooOol03Goo00<007ooOol01Goo00<007ooOol0Sgoo00<007ooOol0=goo000FOol4000=Ool3
0004Ool00`00Oomoo`2@Ool00`00Oomoo`0fOol002ioo`80099oo`03001oogoo03Eoo`00;Woo00<0
07ooOol0TWoo00<007ooOol0=7oo000^Ool00`00Oomoo`2BOol00`00Oomoo`0dOol002ioo`03001o
ogoo09=oo`03001oogoo03=oo`00;Woo00<007ooOol0U7oo00<007ooOol0<Woo000^Ool2002FOol0
0`00Oomoo`0aOol002ioo`03001oogoo09Ioo`03001oogoo031oo`00;Woo00<007ooOol0UWoo00<0
07ooOol0<7oo000^Ool00`00Oomoo`2GOol00`00Oomoo`0_Ool002ioo`03001oogoo09Qoo`03001o
ogoo02ioo`00;Woo00<007ooOol0VGoo00<007ooOol0;Goo000^Ool2002JOol00`00Oomoo`0]Ool0
02ioo`03001oogoo09Yoo`03001oogoo02aoo`00;Woo00<007ooOol0Vgoo00<007ooOol0:goo000^
Ool00`00Oomoo`2KOol00`00Oomoo`0[Ool002ioo`03001oogoo09aoo`03001oogoo02Yoo`00;Woo
0P00WWoo00<007ooOol0:Goo000^Ool00`00Oomoo`2MOol00`00Oomoo`0YOol002ioo`03001oogoo
09ioo`03001oogoo02Qoo`003Woo10001Goo0P001Goo0`0017oo10001Goo00<007ooOol0WWoo00<0
07ooOol0:7oo000BOol00`00Oomoo`02Ool20004Ool01@00Oomoogoo00001goo00<007ooOol00Woo
00<007ooOol0Wgoo00<007ooOol09goo000BOol00`00Oomoo`08Ool01@00Oomoogoo00001goo00<0
07ooOol00Woo00<007ooOol0X7oo00<007ooOol09Woo000BOol00`00Oomoo`08Ool01@00Oomoogoo
00001goo00<007ooOol00Woo0`00X7oo00<007ooOol09Woo000>Ool4000;Ool40004Ool40005Ool0
0`00Oomoo`2QOol00`00Oomoo`0UOol000ioo`03001oogoo00aoo`03001oogoo00Eoo`03001oogoo
00Ioo`03001oogoo0:9oo`03001oogoo02Aoo`003Woo00<007ooOol03Goo00<007ooOol017oo00<0
07ooOol01Woo00<007ooOol0XWoo00<007ooOol097oo000>Ool4000=Ool30003Ool40005Ool00`00
Oomoo`2SOol00`00Oomoo`0SOol002ioo`800:Eoo`03001oogoo029oo`00;Woo00<007ooOol0Y7oo
00<007ooOol08Woo000^Ool00`00Oomoo`2UOol00`00Oomoo`0QOol002ioo`03001oogoo0:Eoo`03
001oogoo025oo`00;Woo00<007ooOol0YWoo00<007ooOol087oo000^Ool00`00Oomoo`2WOol00`00
Oomoo`0OOol002ioo`800:Qoo`03001oogoo01moo`00;Woo00<007ooOol0Z7oo00<007ooOol07Woo
000^Ool00`00Oomoo`2XOol00`00Oomoo`0NOol002ioo`03001oogoo0:Uoo`03001oogoo01eoo`00
;Woo00<007ooOol0ZWoo00<007ooOol077oo000^Ool2002[Ool00`00Oomoo`0LOol002ioo`03001o
ogoo0:]oo`03001oogoo01]oo`00;Woo00<007ooOol0Zgoo00<007ooOol06goo000^Ool00`00Oomo
o`2/Ool00`00Oomoo`0JOol002ioo`03001oogoo0:aoo`03001oogoo01Yoo`00;Woo00<007ooOol0
[Goo00<007ooOol06Goo000^Ool2002^Ool00`00Oomoo`0IOol002ioo`03001oogoo0:ioo`03001o
ogoo01Qoo`00;Woo00<007ooOol0[Woo00<007ooOol067oo000FOol40005Ool20006Ool00`00Oomo
o`04Ool00`00Oomoo`2_Ool00`00Oomoo`0GOol001Yoo`03001oogoo009oo`8000Ioo`03001oogoo
00Aoo`03001oogoo0;1oo`03001oogoo01Ioo`006Woo00<007ooOol02goo00<007ooOol00goo0`00
/7oo00<007ooOol05Woo000JOol00`00Oomoo`0;Ool00`00Oomoo`03Ool00`00Oomoo`2aOol00`00
Oomoo`0EOol001Ioo`@000ioo`03001oogoo00=oo`03001oogoo0;5oo`03001oogoo01Eoo`005Woo
00<007ooOol047oo00<007ooOol00Woo00<007ooOol0/Woo00<007ooOol057oo000FOol00`00Oomo
o`0<Ool01@00Oomoogoo000017oo00<007ooOol0/Woo00<007ooOol057oo000FOol4000;Ool50004
Ool00`00Oomoo`2cOol00`00Oomoo`0COol002ioo`800;Aoo`03001oogoo01=oo`00;Woo00<007oo
Ool0]7oo00<007ooOol04Woo000^Ool00`00Oomoo`2dOol00`00Oomoo`0BOol002ioo`03001oogoo
0;Eoo`03001oogoo015oo`00;Woo00<007ooOol0]Goo00<007ooOol04Goo000^Ool2002gOol00`00
Oomoo`0@Ool002ioo`03001oogoo0;Ioo`03001oogoo011oo`00;Woo00<007ooOol0]goo00<007oo
Ool03goo000^Ool00`00Oomoo`2gOol00`00Oomoo`0?Ool002ioo`03001oogoo0;Qoo`03001oogoo
00ioo`00;Woo0P00^Woo00<007ooOol03Goo000^Ool00`00Oomoo`2iOol00`00Oomoo`0=Ool002io
o`03001oogoo0;Yoo`03001oogoo00aoo`00;Woo00<007ooOol0^Woo00<007ooOol037oo000^Ool0
0`00Oomoo`2kOol00`00Oomoo`0;Ool002ioo`03001oogoo0;]oo`03001oogoo00]oo`00;Woo0P00
_Goo00<007ooOol02Woo000^Ool00`00Oomoo`39Ool002ioo`03001oogoo0<Uoo`00;Woo00<007oo
Ool0bGoo003jOol00?Yoo`00\
\>"],
  ImageRangeCache->{{{0, 249.125}, {153.25, 0}} -> {0.0264424, 5.46985, 
  0.001963, 0.00228648}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Things Are Definitely Not OK - There Are Two or Even Three Implied \
Volatilities\
\>", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "You do not have to make the option that much more complicated to really \
get in a mess. A simple Barrier option will exhibit the phenomenon of there \
being two volatilities consistent with a given valuation. Suppose we consider \
an Up-and-Out Call where the underlying is at 45, the strike at 50 and the \
knock-out barrier at 60. As we first increase the volatility from a very low \
value the option value increases. Then, as the volatility increases further \
the probability of knock-out increases, lowering the value of the option. By \
tweaking in a rebate we can arrange for there to be three solutions for a \
small range of market option prices. The model of Barriers we use is the \
analytical model developed by Rubenstein and Reiner (1991) - this has again \
been built in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", but we suppress the details of the symbolic model here - it is given in \
Chapter 8. The two graphs following show cases where there are two, and \
three, implied volatilities, respectively:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      UpAndOutCall[0, \ 45, \ 50, \ 60, \ vol, 0.05, \ 0.0, \ 1], \n{vol, \ 
        0.05, \ 0.4}, \ PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.112245 2.72109 -0.1003 0.881062 [
[.02381 .06341 -12 -9 ]
[.02381 .06341 12 0 ]
[.29592 .06341 -12 -9 ]
[.29592 .06341 12 0 ]
[.43197 .06341 -9 -9 ]
[.43197 .06341 9 0 ]
[.56803 .06341 -12 -9 ]
[.56803 .06341 12 0 ]
[.70408 .06341 -9 -9 ]
[.70408 .06341 9 0 ]
[.84014 .06341 -12 -9 ]
[.84014 .06341 12 0 ]
[.97619 .06341 -9 -9 ]
[.97619 .06341 9 0 ]
[.14736 .16402 -18 -4.5 ]
[.14736 .16402 0 4.5 ]
[.14736 .25213 -18 -4.5 ]
[.14736 .25213 0 4.5 ]
[.14736 .34023 -18 -4.5 ]
[.14736 .34023 0 4.5 ]
[.14736 .42834 -18 -4.5 ]
[.14736 .42834 0 4.5 ]
[.14736 .51644 -18 -4.5 ]
[.14736 .51644 0 4.5 ]
[.14736 .60455 -18 -4.5 ]
[.14736 .60455 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .07591 m
.02381 .08216 L
s
[(0.05)] .02381 .06341 0 1 Mshowa
.29592 .07591 m
.29592 .08216 L
s
[(0.15)] .29592 .06341 0 1 Mshowa
.43197 .07591 m
.43197 .08216 L
s
[(0.2)] .43197 .06341 0 1 Mshowa
.56803 .07591 m
.56803 .08216 L
s
[(0.25)] .56803 .06341 0 1 Mshowa
.70408 .07591 m
.70408 .08216 L
s
[(0.3)] .70408 .06341 0 1 Mshowa
.84014 .07591 m
.84014 .08216 L
s
[(0.35)] .84014 .06341 0 1 Mshowa
.97619 .07591 m
.97619 .08216 L
s
[(0.4)] .97619 .06341 0 1 Mshowa
.125 Mabswid
.05102 .07591 m
.05102 .07966 L
s
.07823 .07591 m
.07823 .07966 L
s
.10544 .07591 m
.10544 .07966 L
s
.13265 .07591 m
.13265 .07966 L
s
.18707 .07591 m
.18707 .07966 L
s
.21429 .07591 m
.21429 .07966 L
s
.2415 .07591 m
.2415 .07966 L
s
.26871 .07591 m
.26871 .07966 L
s
.32313 .07591 m
.32313 .07966 L
s
.35034 .07591 m
.35034 .07966 L
s
.37755 .07591 m
.37755 .07966 L
s
.40476 .07591 m
.40476 .07966 L
s
.45918 .07591 m
.45918 .07966 L
s
.48639 .07591 m
.48639 .07966 L
s
.51361 .07591 m
.51361 .07966 L
s
.54082 .07591 m
.54082 .07966 L
s
.59524 .07591 m
.59524 .07966 L
s
.62245 .07591 m
.62245 .07966 L
s
.64966 .07591 m
.64966 .07966 L
s
.67687 .07591 m
.67687 .07966 L
s
.73129 .07591 m
.73129 .07966 L
s
.7585 .07591 m
.7585 .07966 L
s
.78571 .07591 m
.78571 .07966 L
s
.81293 .07591 m
.81293 .07966 L
s
.86735 .07591 m
.86735 .07966 L
s
.89456 .07591 m
.89456 .07966 L
s
.92177 .07591 m
.92177 .07966 L
s
.94898 .07591 m
.94898 .07966 L
s
.25 Mabswid
0 .07591 m
1 .07591 L
s
.15986 .16402 m
.16611 .16402 L
s
[(0.3)] .14736 .16402 1 0 Mshowa
.15986 .25213 m
.16611 .25213 L
s
[(0.4)] .14736 .25213 1 0 Mshowa
.15986 .34023 m
.16611 .34023 L
s
[(0.5)] .14736 .34023 1 0 Mshowa
.15986 .42834 m
.16611 .42834 L
s
[(0.6)] .14736 .42834 1 0 Mshowa
.15986 .51644 m
.16611 .51644 L
s
[(0.7)] .14736 .51644 1 0 Mshowa
.15986 .60455 m
.16611 .60455 L
s
[(0.8)] .14736 .60455 1 0 Mshowa
.125 Mabswid
.15986 .09353 m
.16361 .09353 L
s
.15986 .11116 m
.16361 .11116 L
s
.15986 .12878 m
.16361 .12878 L
s
.15986 .1464 m
.16361 .1464 L
s
.15986 .18164 m
.16361 .18164 L
s
.15986 .19926 m
.16361 .19926 L
s
.15986 .21688 m
.16361 .21688 L
s
.15986 .2345 m
.16361 .2345 L
s
.15986 .26975 m
.16361 .26975 L
s
.15986 .28737 m
.16361 .28737 L
s
.15986 .30499 m
.16361 .30499 L
s
.15986 .32261 m
.16361 .32261 L
s
.15986 .35785 m
.16361 .35785 L
s
.15986 .37547 m
.16361 .37547 L
s
.15986 .3931 m
.16361 .3931 L
s
.15986 .41072 m
.16361 .41072 L
s
.15986 .44596 m
.16361 .44596 L
s
.15986 .46358 m
.16361 .46358 L
s
.15986 .4812 m
.16361 .4812 L
s
.15986 .49882 m
.16361 .49882 L
s
.15986 .53407 m
.16361 .53407 L
s
.15986 .55169 m
.16361 .55169 L
s
.15986 .56931 m
.16361 .56931 L
s
.15986 .58693 m
.16361 .58693 L
s
.15986 .05829 m
.16361 .05829 L
s
.15986 .04067 m
.16361 .04067 L
s
.15986 .02305 m
.16361 .02305 L
s
.15986 .00543 m
.16361 .00543 L
s
.25 Mabswid
.15986 0 m
.15986 .61803 L
s
.5 Mabswid
.02381 .03753 m
.06244 .17966 L
.10458 .34869 L
.12507 .42278 L
.14415 .4818 L
.16254 .52769 L
.18221 .56422 L
.19242 .57816 L
.19781 .58418 L
.20351 .5896 L
.2085 .59355 L
.21399 .59708 L
.21898 .59957 L
.22365 .60131 L
.22623 .60203 L
.22862 .60256 L
.22997 .60279 L
.23124 .60297 L
.2327 .60313 L
.23334 .60319 L
.23403 .60323 L
.23523 .60329 L
.2365 .60332 L
.23768 .60331 L
.23877 .60328 L
.24007 .6032 L
.24125 .6031 L
.24261 .60295 L
.2439 .60277 L
.24614 .60237 L
.24851 .60184 L
.25277 .60061 L
.25736 .59891 L
.26243 .5966 L
.28177 .58404 L
.30263 .56511 L
.34377 .51713 L
.38339 .46435 L
.4215 .41251 L
.46206 .35935 L
.50111 .31174 L
.5426 .26577 L
.58259 .22614 L
.62105 .1922 L
.66197 .16025 L
.70137 .13315 L
.73926 .11013 L
.7796 .08849 L
.81843 .07014 L
.8597 .05294 L
Mistroke
.89946 .03834 L
.93771 .02588 L
.97619 .01472 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{201.125, 124.063},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000390000O1000`40O003h00ObGoo0039Ool00<Uo
o`009Goo00<007ooOol0XGoo000UOol2002ROol002Eoo`03001oogoo0:5oo`009Goo00<007ooOol0
UGoo0`002Goo000UOol2002BOol4000<Ool0009oo`8000Aoo`8000Aoo`8000=oo`<000moo`03001o
ogoo00eoo`8000Aoo`8000=oo`<000=oo`<000Qoo`8000Aoo`8000=oo`@000Qoo`8000Aoo`8000=o
o`@0009oo`<000Qoo`8000Aoo`8000=oo`<000Uoo`8000Aoo`8000=oo`<000=oo`<000Ioo`D000Ao
o`8000Aoo`<0009oo`0000Eoo`00Oomoo`0000=oo`8000=oo`04001oogoo0005Ool00`00Oomoo`0<
Ool00`00Oomoo`0<Ool01000Oomoo`000goo0P0017oo00<007ooOol01Goo00<007ooOol017oo00@0
07ooOol000=oo`8000Aoo`03001oogoo00Moo`04001oogoo0003Ool20004Ool00`00Oomoo`05Ool0
0`00Oomoo`04Ool01000Oomoo`000goo0P001Woo00<007ooOol01Goo00@007ooOol000=oo`8000Io
o`03001oogoo00=oo`03001oogoo00<0009oo`04001oogoo0003Ool20005Ool00`00Oomoo`01Ool0
0005Ool007ooOol00007Ool20002Ool00`00Oomoo`03Ool00`00Oomoo`0<Ool00`00Oomoo`0<Ool0
1000Oomoo`002Goo00<007ooOol01Goo00<007ooOol017oo00@007ooOol000Yoo`03001oogoo00Io
o`04001oogoo000:Ool00`00Oomoo`04Ool00`00Oomoo`04Ool01000Oomoo`002goo00<007ooOol0
1Goo00@007ooOol000]oo`03001oogoo00=oo`<000Eoo`04001oogoo0008Ool40002Ool00005Ool0
07ooOol00007Ool20002Ool00`00Oomoo`03000?Ool2000=Ool01000Oomoo`002Goo00<007ooOol0
0Woo0`001goo00@007ooOol000]oo`03001oogoo00Eoo`04001oogoo000;Ool00`00Oomoo`030007
Ool01000Oomoo`002Goo0P0027oo00@007ooOol000Uoo`8000=oo`<000Qoo`04001oogoo0008Ool0
0`00Ool00003Ool00005Ool007ooOol00007Ool20002Ool01000Oomoo`004Goo00<007ooOol037oo
00@007ooOol000Uoo`03001oogoo009oo`03001oogoo00Moo`04001oogoo0008Ool01000Oomoo`00
1goo00@007ooOol000Qoo`04001oogoo0002Ool00`00Oomoo`07Ool01000Oomoo`002goo00<007oo
Ool01Goo00@007ooOol000]oo`@000Yoo`04001oogoo0008Ool00`00Ool00003Ool0009oo`8000Uo
o`<000=oo`@000ioo`03001oogoo00eoo`8000Uoo`8000Aoo`@000Moo`8000Yoo`8000Uoo`8000Yo
o`8000=oo`@000Moo`8000Uoo`<000Uoo`8000Uoo`@0009oo`@000Qoo`8000Yoo`8000=oo`003Goo
00<007ooOol05Goo0P00OWoo0`008Goo000=Ool00`00Oomoo`0EOol00`00Oomoo`1jOol3000TOol0
00eoo`03001oogoo01Eoo`03001oogoo07Qoo`8002Moo`001goo_@001Goo000<Ool00`00Ool00002
Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02
Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`02
Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02
Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02
Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`03
Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Ool00002Ool00`00Oomoo`02Ool00`00Oomoo`02
Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`07
Ool000ioo`03001oogoo01Aoo`03001oogoo07=oo`8002aoo`003Woo00<007ooOol057oo0P00LWoo
0P00;Woo000?Ool00`00Oomoo`0COol00`00Oomoo`1_Ool2000`Ool000moo`03001oogoo01=oo`03
001oogoo06eoo`80039oo`003goo00<007ooOol04goo00<007ooOol0Jgoo0P00=7oo000@Ool00`00
Oomoo`0BOol2001ZOol2000fOol0011oo`03001oogoo019oo`03001oogoo06Moo`8003Qoo`0047oo
00<007ooOol04Woo00<007ooOol0IWoo00<007ooOol0>7oo000@Ool00`00Oomoo`0BOol2001UOol2
000kOol0015oo`03001oogoo015oo`03001oogoo069oo`8003eoo`004Goo00<007ooOol04Goo00<0
07ooOol0HGoo00<007ooOol0?Goo000AOol00`00Oomoo`0AOol2001POol20010Ool0015oo`03001o
ogoo015oo`03001oogoo05ioo`03001oogoo041oo`004Woo0`0017oo0P000goo0`0017oo00<007oo
Ool0GGoo00<007ooOol0@Goo000BOol01000Oomoo`000goo0P001Woo00D007ooOomoo`0005eoo`80
04Aoo`004Woo00@007ooOol000]oo`04001oogooOol2001KOol00`00Oomoo`14Ool0019oo`04001o
ogoo0009Ool20004Ool00`00Oomoo`1IOol00`00Oomoo`15Ool0019oo`800003Ool007oo00Yoo`05
001oogooOol0001IOol20018Ool001=oo`8000Uoo`<000Aoo`8005Moo`03001oogoo04Qoo`004goo
00<007ooOol03goo00<007ooOol0EGoo00<007ooOol0BGoo000COol00`00Oomoo`0?Ool00`00Oomo
o`1COol2001<Ool001Aoo`03001oogoo00ioo`8005=oo`03001oogoo04aoo`0057oo00<007ooOol0
3Woo00<007ooOol0DGoo00<007ooOol0CGoo000DOol00`00Oomoo`0>Ool00`00Oomoo`1@Ool00`00
Oomoo`1>Ool001Aoo`03001oogoo00ioo`03001oogoo04moo`03001oogoo04moo`005Goo00<007oo
Ool03Goo0P00Cgoo00<007ooOol0D7oo000EOol00`00Oomoo`0=Ool00`00Oomoo`1=Ool00`00Oomo
o`1AOol001Eoo`03001oogoo00eoo`03001oogoo04aoo`03001oogoo059oo`005Goo00<007ooOol0
3Goo0P00C7oo00<007ooOol0Dgoo000COol2000017oo001oogoo0P0017oo0`000goo00<007ooOol0
BWoo00<007ooOol0E7oo000BOol00`00Oomoo`020002Ool20005Ool00`00Oomoo`02Ool00`00Oomo
o`19Ool00`00Oomoo`1EOol0019oo`03001oogoo008000Moo`@000=oo`8004Uoo`03001oogoo05Io
o`004Woo00<007ooOol00P001goo00<007oo000017oo00<007ooOol0Agoo00<007ooOol0Egoo000B
Ool01P00Oomoo`00Ool000Ioo`03001oo`0000Aoo`03001oogoo04Ioo`03001oogoo05Qoo`004goo
0P000Woo00<007ooOol01Goo0P0017oo00<007ooOol0AGoo00<007ooOol0FGoo000GOol00`00Oomo
o`0;Ool20015Ool00`00Oomoo`1JOol001Qoo`03001oogoo00Yoo`03001oogoo04=oo`03001oogoo
05]oo`0067oo00<007ooOol02Woo00<007ooOol0@Woo00<007ooOol0G7oo000HOol00`00Oomoo`0:
Ool20012Ool00`00Oomoo`1MOol001Qoo`03001oogoo00Yoo`03001oogoo041oo`03001oogoo05io
o`006Goo00<007ooOol02Goo00<007ooOol0?goo00<007ooOol0Ggoo000IOol00`00Oomoo`09Ool2
000oOol00`00Oomoo`1POol001Uoo`03001oogoo00Uoo`03001oogoo03eoo`03001oogoo065oo`00
6Goo00<007ooOol02Goo00<007ooOol0?Goo00<007ooOol0HGoo000JOol00`00Oomoo`08Ool00`00
Oomoo`0lOol00`00Oomoo`1ROol001Yoo`03001oogoo00Qoo`8003aoo`03001oogoo06=oo`004goo
0P0017oo0P000goo0`0017oo00<007ooOol0>Woo00<007ooOol0I7oo000BOol01000Oomoo`000goo
0P001Woo00D007ooOomoo`0003]oo`03001oogoo06Eoo`004Woo00@007ooOol000Eoo`03001oogoo
00=oo`04001oogooOol2000jOol00`00Oomoo`1UOol0019oo`04001oogoo0005Ool00`00Oomoo`03
0004Ool00`00Oomoo`0hOol00`00Oomoo`1VOol0019oo`04001oogoo0005Ool01000Oomoo`001Woo
00<007ooOol0=goo00<007ooOol0Igoo000COol20006Ool00`00Oomoo`040003Ool2000gOol00`00
Oomoo`1XOol001aoo`03001oogoo00Ioo`03001oogoo03Ioo`03001oogoo06Qoo`0077oo00<007oo
Ool01Woo00<007ooOol0=Goo00<007ooOol0JGoo000LOol00`00Oomoo`06Ool2000eOol00`00Oomo
o`1ZOol001eoo`03001oogoo00Eoo`03001oogoo03=oo`03001oogoo06]oo`007Goo00<007ooOol0
1Goo00<007ooOol0<Woo00<007ooOol0K7oo000MOol00`00Oomoo`05Ool00`00Oomoo`0bOol00`00
Oomoo`1/Ool001eoo`03001oogoo00Eoo`80039oo`03001oogoo06eoo`007Woo00<007ooOol017oo
00<007ooOol0<7oo00<007ooOol0KWoo000NOol00`00Oomoo`04Ool00`00Oomoo`0_Ool00`00Oomo
o`1_Ool001ioo`03001oogoo00Aoo`8002moo`03001oogoo071oo`004goo0P0017oo0P0017oo0P00
17oo00<007ooOol0;Woo00<007ooOol0L7oo000BOol01000Oomoo`000goo0P000goo0P0000=oo`00
Ool00Woo00<007ooOol0;Goo00<007ooOol0LGoo000BOol01000Oomoo`0027oo0P0000=oo`00Ool0
0Woo0P00;Goo00<007ooOol0LWoo000BOol01000Oomoo`0027oo0`0017oo00<007ooOol0;7oo00<0
07ooOol0LWoo000BOol01000Oomoo`0027oo00<007oo000017oo00<007ooOol0:goo00<007ooOol0
Lgoo000COol2000:Ool30003Ool00`00Oomoo`0ZOol00`00Oomoo`1dOol0021oo`03001oogoo009o
o`8002Yoo`03001oogoo07Eoo`008Goo00D007ooOomoo`0002]oo`03001oogoo07Eoo`008Goo00D0
07ooOomoo`0002Yoo`03001oogoo07Ioo`008Goo00@007ooOomoo`8002Qoo`03001oogoo07Moo`00
8Goo00D007ooOomoo`0002Uoo`03001oogoo07Moo`008Woo00@007ooOol002Qoo`03001oogoo07Qo
o`008Woo00<007ooOol00P009Woo00<007ooOol0NGoo000ROol01000Oomoo`009goo00<007ooOol0
NGoo000SOol00`00Ool0000VOol00`00Oomoo`1jOol002=oo`03001oo`0002Eoo`03001oogoo07]o
o`0097oo0`008goo00<007ooOol0O7oo000COol20004Ool20004Ool00`00Oomoo`02Ool2000TOol0
0`00Oomoo`1lOol0019oo`04001oogoo0003Ool20004Ool00`00Oomoo`02Ool2000SOol00`00Oomo
o`1mOol0019oo`04001oogoo000:Ool00`00Oomoo`02Ool2000QOol00`00Oomoo`1nOol0019oo`04
001oogoo000:Ool00`00Oomoo`02Ool00`00Oomoo`0POol00`00Oomoo`1nOol0019oo`04001oogoo
000;Ool01000Oomoogoo0P0087oo00<007ooOol0Ogoo000COol20009Ool40003Ool2000OOol00`00
Oomoo`20Ool002Eoo`03001oo`0001eoo`03001oogoo085oo`009Goo00<007oo000077oo00<007oo
Ool0PWoo000UOol01000Oomoo`006Woo00<007ooOol0Pgoo000UOol200000goo001oo`0HOol00`00
Oomoo`24Ool002Eoo`04001oogoo000HOol00`00Oomoo`25Ool002Eoo`05001oogooOol0000FOol0
0`00Oomoo`26Ool002Eoo`80009oo`03001oogoo01=oo`03001oogoo08Moo`009Goo00<007ooOol0
0Woo00<007ooOol04Goo00<007ooOol0R7oo000UOol00`00Oomoo`03Ool00`00Oomoo`0?Ool00`00
Oomoo`29Ool002Eoo`8000Eoo`03001oogoo00aoo`8008aoo`009Goo00<007ooOol01Goo0P002Woo
0P00SWoo000COol20004Ool20004Ool20004Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomo
o`2>Ool0019oo`04001oogoo0003Ool20003Ool01000Oomoo`000goo00<007ooOol027oo2000TGoo
000BOol01000Oomoo`0027oo00@007ooOol000=oo`800:9oo`004Woo00@007ooOol000Uoo`8000Ao
o`03001oogoo0:5oo`004Woo00@007ooOol000Qoo`04001oogoo0003Ool00`00Oomoo`2QOol001=o
o`8000Yoo`800:Qoo`00bGoo0039Ool00<Uoo`00\
\>"],
  ImageRangeCache->{{{0, 200.125}, {123.063, 0}} -> {0.0265509, 0.0991787, 
  0.00195396, 0.00603465}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      UpAndOutCall[1.7, \ 45, \ 50, \ 60, \ vol, 0.05, \ 0.0, \ 1], \n{vol, \ 
        0.05, \ 0.4}, \ PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.112245 2.72109 -0.107854 0.783523 [
[.02381 .03635 -12 -9 ]
[.02381 .03635 12 0 ]
[.29592 .03635 -12 -9 ]
[.29592 .03635 12 0 ]
[.43197 .03635 -9 -9 ]
[.43197 .03635 9 0 ]
[.56803 .03635 -12 -9 ]
[.56803 .03635 12 0 ]
[.70408 .03635 -9 -9 ]
[.70408 .03635 9 0 ]
[.84014 .03635 -12 -9 ]
[.84014 .03635 12 0 ]
[.97619 .03635 -9 -9 ]
[.97619 .03635 9 0 ]
[.14736 .1272 -18 -4.5 ]
[.14736 .1272 0 4.5 ]
[.14736 .20556 -18 -4.5 ]
[.14736 .20556 0 4.5 ]
[.14736 .28391 -18 -4.5 ]
[.14736 .28391 0 4.5 ]
[.14736 .36226 -18 -4.5 ]
[.14736 .36226 0 4.5 ]
[.14736 .44061 -18 -4.5 ]
[.14736 .44061 0 4.5 ]
[.14736 .51896 -18 -4.5 ]
[.14736 .51896 0 4.5 ]
[.14736 .59732 -18 -4.5 ]
[.14736 .59732 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .04885 m
.02381 .0551 L
s
[(0.05)] .02381 .03635 0 1 Mshowa
.29592 .04885 m
.29592 .0551 L
s
[(0.15)] .29592 .03635 0 1 Mshowa
.43197 .04885 m
.43197 .0551 L
s
[(0.2)] .43197 .03635 0 1 Mshowa
.56803 .04885 m
.56803 .0551 L
s
[(0.25)] .56803 .03635 0 1 Mshowa
.70408 .04885 m
.70408 .0551 L
s
[(0.3)] .70408 .03635 0 1 Mshowa
.84014 .04885 m
.84014 .0551 L
s
[(0.35)] .84014 .03635 0 1 Mshowa
.97619 .04885 m
.97619 .0551 L
s
[(0.4)] .97619 .03635 0 1 Mshowa
.125 Mabswid
.05102 .04885 m
.05102 .0526 L
s
.07823 .04885 m
.07823 .0526 L
s
.10544 .04885 m
.10544 .0526 L
s
.13265 .04885 m
.13265 .0526 L
s
.18707 .04885 m
.18707 .0526 L
s
.21429 .04885 m
.21429 .0526 L
s
.2415 .04885 m
.2415 .0526 L
s
.26871 .04885 m
.26871 .0526 L
s
.32313 .04885 m
.32313 .0526 L
s
.35034 .04885 m
.35034 .0526 L
s
.37755 .04885 m
.37755 .0526 L
s
.40476 .04885 m
.40476 .0526 L
s
.45918 .04885 m
.45918 .0526 L
s
.48639 .04885 m
.48639 .0526 L
s
.51361 .04885 m
.51361 .0526 L
s
.54082 .04885 m
.54082 .0526 L
s
.59524 .04885 m
.59524 .0526 L
s
.62245 .04885 m
.62245 .0526 L
s
.64966 .04885 m
.64966 .0526 L
s
.67687 .04885 m
.67687 .0526 L
s
.73129 .04885 m
.73129 .0526 L
s
.7585 .04885 m
.7585 .0526 L
s
.78571 .04885 m
.78571 .0526 L
s
.81293 .04885 m
.81293 .0526 L
s
.86735 .04885 m
.86735 .0526 L
s
.89456 .04885 m
.89456 .0526 L
s
.92177 .04885 m
.92177 .0526 L
s
.94898 .04885 m
.94898 .0526 L
s
.25 Mabswid
0 .04885 m
1 .04885 L
s
.15986 .1272 m
.16611 .1272 L
s
[(0.3)] .14736 .1272 1 0 Mshowa
.15986 .20556 m
.16611 .20556 L
s
[(0.4)] .14736 .20556 1 0 Mshowa
.15986 .28391 m
.16611 .28391 L
s
[(0.5)] .14736 .28391 1 0 Mshowa
.15986 .36226 m
.16611 .36226 L
s
[(0.6)] .14736 .36226 1 0 Mshowa
.15986 .44061 m
.16611 .44061 L
s
[(0.7)] .14736 .44061 1 0 Mshowa
.15986 .51896 m
.16611 .51896 L
s
[(0.8)] .14736 .51896 1 0 Mshowa
.15986 .59732 m
.16611 .59732 L
s
[(0.9)] .14736 .59732 1 0 Mshowa
.125 Mabswid
.15986 .06452 m
.16361 .06452 L
s
.15986 .08019 m
.16361 .08019 L
s
.15986 .09586 m
.16361 .09586 L
s
.15986 .11153 m
.16361 .11153 L
s
.15986 .14287 m
.16361 .14287 L
s
.15986 .15854 m
.16361 .15854 L
s
.15986 .17421 m
.16361 .17421 L
s
.15986 .18988 m
.16361 .18988 L
s
.15986 .22123 m
.16361 .22123 L
s
.15986 .2369 m
.16361 .2369 L
s
.15986 .25257 m
.16361 .25257 L
s
.15986 .26824 m
.16361 .26824 L
s
.15986 .29958 m
.16361 .29958 L
s
.15986 .31525 m
.16361 .31525 L
s
.15986 .33092 m
.16361 .33092 L
s
.15986 .34659 m
.16361 .34659 L
s
.15986 .37793 m
.16361 .37793 L
s
.15986 .3936 m
.16361 .3936 L
s
.15986 .40927 m
.16361 .40927 L
s
.15986 .42494 m
.16361 .42494 L
s
.15986 .45628 m
.16361 .45628 L
s
.15986 .47195 m
.16361 .47195 L
s
.15986 .48762 m
.16361 .48762 L
s
.15986 .50329 m
.16361 .50329 L
s
.15986 .53463 m
.16361 .53463 L
s
.15986 .55031 m
.16361 .55031 L
s
.15986 .56598 m
.16361 .56598 L
s
.15986 .58165 m
.16361 .58165 L
s
.15986 .03318 m
.16361 .03318 L
s
.15986 .01751 m
.16361 .01751 L
s
.15986 .00184 m
.16361 .00184 L
s
.15986 .61299 m
.16361 .61299 L
s
.25 Mabswid
.15986 0 m
.15986 .61803 L
s
.5 Mabswid
.02381 .01472 m
.06244 .14132 L
.10458 .29422 L
.14415 .42125 L
.18221 .51082 L
.20178 .54337 L
.22272 .56896 L
.23293 .57837 L
.24402 .58656 L
.25449 .59255 L
.26416 .59674 L
.26882 .59834 L
.27383 .59979 L
.27816 .60081 L
.28289 .60171 L
.28575 .60215 L
.28841 .6025 L
.29116 .60278 L
.29366 .60299 L
.29484 .60307 L
.29611 .60314 L
.29722 .60319 L
.29843 .60324 L
.29972 .60328 L
.30042 .60329 L
.30108 .60331 L
.30225 .60332 L
.30351 .60332 L
.30473 .60331 L
.30542 .6033 L
.30608 .60329 L
.30739 .60326 L
.3088 .60321 L
.31135 .60309 L
.31369 .60294 L
.31908 .60249 L
.32475 .60184 L
.33436 .6004 L
.34487 .59839 L
.38544 .58807 L
.42448 .57671 L
.46598 .5654 L
.50596 .55637 L
.54443 .54981 L
.5643 .54726 L
.58535 .54517 L
.59478 .54444 L
.60492 .54377 L
.61442 .54327 L
.62323 .54291 L
Mistroke
.62855 .54274 L
.63336 .54261 L
.63871 .5425 L
.64165 .54245 L
.64441 .54241 L
.64686 .54239 L
.6482 .54238 L
.64944 .54237 L
.65051 .54237 L
.65165 .54236 L
.65292 .54236 L
.65407 .54236 L
.65472 .54236 L
.65541 .54236 L
.65664 .54236 L
.65733 .54236 L
.65797 .54236 L
.65943 .54237 L
.66071 .54238 L
.66188 .54238 L
.66451 .54241 L
.667 .54244 L
.66926 .54247 L
.67434 .54256 L
.68347 .54278 L
.69413 .54315 L
.70419 .54358 L
.72424 .54471 L
.7427 .54603 L
.78216 .54962 L
.8201 .55392 L
.86049 .55921 L
.89937 .56485 L
.9407 .57128 L
.97619 .57707 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{201.125, 124.063},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000390000O1000`40O003h00ObGoo0039Ool00<Uo
o`00bGoo0002Ool20004Ool20004Ool20003Ool3000OOol20004Ool20003Ool30003Ool30008Ool2
0004Ool20003Ool40008Ool20004Ool20003Ool40002Ool30008Ool20004Ool20003Ool30009Ool2
0004Ool20003Ool30003Ool30009Ool20004Ool20004Ool30002Ool00005Ool007ooOol00003Ool2
0003Ool01000Oomoo`001Goo00<007ooOol037oo0P003Goo00@007ooOol000=oo`8000Aoo`03001o
ogoo00Eoo`03001oogoo00Aoo`04001oogoo0003Ool20004Ool00`00Oomoo`07Ool01000Oomoo`00
0goo0P0017oo00<007ooOol01Goo00<007ooOol017oo00@007ooOol000=oo`8000Ioo`03001oogoo
00Eoo`04001oogoo0003Ool20006Ool00`00Oomoo`03Ool00`00Oomoo`05Ool01000Oomoo`000goo
0P001Goo00<007ooOol00Goo00001Goo001oogoo000027oo00@007ooOol000Eoo`03001oogoo00ao
o`03001oogoo00aoo`04001oogoo0009Ool00`00Oomoo`05Ool00`00Oomoo`04Ool01000Oomoo`00
2Woo00<007ooOol01Woo00@007ooOol000Yoo`03001oogoo00Aoo`03001oogoo00Aoo`04001oogoo
000;Ool00`00Oomoo`05Ool01000Oomoo`002goo00<007ooOol00goo00<007ooOol01Goo00@007oo
Ool000Qoo`@0009oo`0000Eoo`00Oomoo`0000Moo`80009oo`03001oogoo00<000moo`03001oogoo
00aoo`04001oogoo0009Ool00`00Oomoo`02Ool30007Ool01000Oomoo`002goo00<007ooOol01Goo
00@007ooOol000]oo`03001oogoo00<000Moo`04001oogoo0009Ool20008Ool01000Oomoo`002Goo
0P000goo0`0027oo00@007ooOol000Qoo`03001oo`0000=oo`0000Eoo`00Oomoo`0000Moo`80009o
o`04001oogoo000AOol2000=Ool01000Oomoo`002Goo00<007ooOol00Woo00<007ooOol01goo00@0
07ooOol000Qoo`04001oogoo0007Ool01000Oomoo`0027oo00@007ooOol0009oo`03001oogoo00Mo
o`04001oogoo000;Ool00`00Oomoo`05Ool01000Oomoo`002goo00@007ooOol000Yoo`04001oogoo
0008Ool00`00Ool00003Ool0009oo`8000Qoo`04001oo`000003Ool4000>Ool00`00Oomoo`0=Ool2
0009Ool20004Ool40007Ool2000:Ool20009Ool2000:Ool20003Ool40007Ool20009Ool30009Ool2
0009Ool30003Ool40008Ool2000:Ool20003Ool000eoo`03001oogoo01Eoo`03001oogoo0:5oo`00
3Goo00<007ooOol05Goo0P00XWoo000=Ool00`00Oomoo`0EOol00`00Oomoo`2QOol000ioo`03001o
ogoo01Aoo`03001oogoo0:5oo`001goo_@001Goo000<Ool00`00Ool00002Ool00`00Oomoo`02Ool0
0`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool0
0`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool0
0`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`03Ool0
0`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool0
0`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool0
0`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool0
0`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`07Ool000ioo`03001oogoo
01Aoo`03001oogoo0:5oo`003goo00<007ooOol04goo0P00XWoo000?Ool00`00Oomoo`0COol00`00
Oomoo`2QOol000moo`03001oogoo01=oo`03001oogoo0:5oo`0047oo00<007ooOol04Woo0P00XWoo
000@Ool00`00Oomoo`0BOol00`00Oomoo`2QOol0011oo`03001oogoo019oo`03001oogoo0:5oo`00
4Goo00<007ooOol04Goo0P00XWoo000AOol00`00Oomoo`0AOol00`00Oomoo`2QOol0015oo`03001o
ogoo015oo`03001oogoo0:5oo`004Goo00<007ooOol04Goo0P00XWoo000BOol30004Ool20003Ool3
0004Ool00`00Oomoo`2QOol0019oo`04001oogoo0003Ool20006Ool01@00Oomoogoo0000Xgoo000B
Ool01000Oomoo`002goo00@007ooOomoo`800:9oo`004Woo0P0000=oo`00Ool027oo0P0017oo00<0
07ooOol0XGoo000BOol200000goo001oo`0:Ool01000Oomoogoo0P00XWoo000COol20009Ool30004
Ool00`00Oomoo`2QOol001=oo`03001oogoo00moo`03001oogoo0:5oo`0057oo00<007ooOol03Woo
0P00XWoo000DOol00`00Oomoo`0>Ool00`00Oomoo`2QOol001Aoo`03001oogoo00ioo`03001oogoo
0:5oo`005Goo00<007ooOol03Goo0P00XWoo000EOol00`00Oomoo`0=Ool00`00Oomoo`2QOol001Eo
o`03001oogoo00eoo`03001oogoo0:5oo`005Goo00<007ooOol03Goo0P00XWoo000COol2000017oo
001oogoo0P0017oo0`000goo00<007ooOol0XGoo000BOol00`00Oomoo`020002Ool20005Ool00`00
Oomoo`02Ool00`00Oomoo`2QOol0019oo`03001oogoo008000Moo`@000=oo`800:9oo`004Woo00<0
07ooOol00P001goo00<007oo000017oo00<007ooOol0XGoo000BOol01P00Oomoo`00Ool000Ioo`03
001oo`0000Aoo`03001oogoo0:5oo`004goo0P000Woo00<007ooOol01Goo0P0017oo0P00XWoo000G
Ool00`00Oomoo`0;Ool00`00Oomoo`2QOol001Qoo`03001oogoo00Yoo`03001oogoo0:5oo`0067oo
00<007ooOol02Woo0P00XWoo000HOol00`00Oomoo`0:Ool00`00Oomoo`2QOol001Qoo`03001oogoo
00Yoo`03001oogoo0:5oo`006Goo00<007ooOol02Goo0P00XWoo000IOol00`00Oomoo`09Ool00`00
Oomoo`2QOol001Uoo`03001oogoo00Uoo`03001oogoo0:5oo`006Goo00<007ooOol02Goo0P00XWoo
000COol20004Ool20003Ool30004Ool00`00Oomoo`2QOol0019oo`04001oogoo0003Ool20006Ool0
1@00Oomoogoo0000Xgoo000BOol01000Oomoo`0017oo00<007ooOol017oo00@007ooOomoo`800:9o
o`004Woo00@007ooOol000Eoo`03001oogoo00<000Aoo`03001oogoo0:5oo`004Woo00@007ooOol0
00Eoo`04001oogoo0006Ool00`00Oomoo`2QOol001=oo`8000Ioo`03001oogoo00@000=oo`800:9o
o`006goo00<007ooOol01goo00<007ooOol0XGoo000LOol00`00Oomoo`06Ool00`00Oomoo`2QOol0
01aoo`03001oogoo00Ioo`800:9oo`0077oo00<007ooOol01Woo00<007ooOol0XGoo000MOol00`00
Oomoo`05Ool00`00Oomoo`2QOol001eoo`03001oogoo00Eoo`800:9oo`007Goo00<007ooOol01Goo
00<007ooOol0XGoo000MOol00`00Oomoo`05Ool00`00Oomoo`2QOol001ioo`03001oogoo00Aoo`80
0:9oo`004goo0P0017oo0P000goo0`0017oo00<007ooOol0XGoo000BOol01000Oomoo`000goo0P00
0goo00@007ooOol000=oo`03001oogoo0:5oo`004Woo00@007ooOol000Qoo`800003Ool007oo009o
o`800:9oo`004Woo00@007ooOol000Qoo`<000Aoo`03001oogoo0:5oo`004Woo00@007ooOol000Qo
o`8000Eoo`03001oogoo0:5oo`004goo0P002Woo0`000goo0P00XWoo000POol00`00Oomoo`02Ool0
0`00Oomoo`2QOol0021oo`03001oogoo009oo`03001oogoo0:5oo`0087oo00<007ooOol00Woo0P00
XWoo000QOol01@00Oomoogoo0000Xgoo000QOol01@00Oomoogoo0000Xgoo000QOol01000Oomoogoo
0P00XWoo000ROol01000Oomoo`00Xgoo000ROol01000Oomoo`00Xgoo000COol20004Ool20004Ool0
1000Oomoo`000Woo0P00XWoo000BOol01000Oomoo`000goo0P0017oo00L007ooOomoo`00Ool0002S
Ool0019oo`04001oogoo000:Ool01@00Oomoo`00Ool00P00XWoo000BOol01000Oomoo`002Woo00H0
07ooOol007oo002SOol0019oo`04001oogoo000;Ool00`00Oomoo`02002SOol001=oo`8000Uoo`@0
009oo`<00:9oo`009Goo00<007ooOol0XGoo000UOol00`00Oomoo`2QOol002Eoo`800:9oo`009Goo
0P00XWoo000UOol2002ROol002Eoo`<00:5oo`009Goo00<007oo0000XGoo000UOol01000Oomoo`00
X7oo000UOol200000goo001oo`2OOol001=oo`8000Aoo`8000Aoo`8000Aoo`04001oogoo002POol0
019oo`04001oogoo0003Ool20003Ool01000Oomoo`000goo00D007ooOomoo`0009moo`004Woo00@0
07ooOol000Qoo`04001oogoo0003Ool20002Ool00`00Oomoo`2MOol0019oo`04001oogoo0009Ool2
0004Ool00`00Oomoo`02Ool00`00Oomoo`2LOol0019oo`04001oogoo0008Ool01000Oomoo`000goo
00<007ooOol00goo00<007ooOol0Vgoo000COol2000:Ool20004Ool20004Ool00`00Oomoo`2KOol0
02Eoo`03001oogoo00Aoo`03001oogoo09Yoo`009Goo00<007ooOol01Goo00<007ooOol0@goo9@00
<Goo000UOol20006Ool00`00Oomoo`0jOol9000UOol7000ZOol002Eoo`03001oogoo00Ioo`03001o
ogoo03=oo`H003Eoo`L002=oo`009Goo00<007ooOol01goo00<007ooOol0;goo0`00@Woo20006goo
000UOol20009Ool00`00Oomoo`0ZOol4001=Ool7000DOol002Eoo`03001oogoo00Uoo`8002Ioo`@0
05Qoo`P000aoo`009Goo00<007ooOol02goo00<007ooOol07goo1000I7oo0`002Goo000UOol2000=
Ool00`00Oomoo`0KOol3001dOol001=oo`8000Aoo`8000=oo`<000Aoo`03001oogoo00eoo`8001Mo
o`@007Moo`004Woo00@007ooOol000=oo`8000Ioo`05001oogooOol0000AOol2000AOol4001kOol0
019oo`04001oogoo0009Ool30003Ool2000BOol3000;Ool3001oOol0019oo`04001oogoo0008Ool0
1000Oomoo`000goo00<007ooOol04goo3000PWoo000BOol01000Oomoo`0027oo00@007ooOol000=o
o`03001oogoo0:5oo`004goo0P002Woo0P0017oo0P00XWoo000UOol00`00Oomoo`2QOol00<Uoo`00
bGoo0000\
\>"],
  ImageRangeCache->{{{0, 200.125}, {123.063, 0}} -> {0.0265509, 0.109213, 
  0.00195396, 0.00678589}}]
}, Open  ]],

Cell["\<\
Another way of describing this effect is shown in the next plot, \
where we see that Vega changes sign twice:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      UpAndOutCallVega[1.7, \ 45, \ 50, \ 60, \ vol, 0.05, \ 0.0, \ 
        1], \n{vol, \ 0.05, \ 0.4}, \ PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.112245 2.72109 0.0582568 0.0427886 [
[.02381 .04576 -12 -9 ]
[.02381 .04576 12 0 ]
[.29592 .04576 -12 -9 ]
[.29592 .04576 12 0 ]
[.43197 .04576 -9 -9 ]
[.43197 .04576 9 0 ]
[.56803 .04576 -12 -9 ]
[.56803 .04576 12 0 ]
[.70408 .04576 -9 -9 ]
[.70408 .04576 9 0 ]
[.84014 .04576 -12 -9 ]
[.84014 .04576 12 0 ]
[.97619 .04576 -9 -9 ]
[.97619 .04576 9 0 ]
[.14736 .14383 -6 -4.5 ]
[.14736 .14383 0 4.5 ]
[.14736 .22941 -6 -4.5 ]
[.14736 .22941 0 4.5 ]
[.14736 .31499 -6 -4.5 ]
[.14736 .31499 0 4.5 ]
[.14736 .40057 -6 -4.5 ]
[.14736 .40057 0 4.5 ]
[.14736 .48614 -12 -4.5 ]
[.14736 .48614 0 4.5 ]
[.14736 .57172 -12 -4.5 ]
[.14736 .57172 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .05826 m
.02381 .06451 L
s
[(0.05)] .02381 .04576 0 1 Mshowa
.29592 .05826 m
.29592 .06451 L
s
[(0.15)] .29592 .04576 0 1 Mshowa
.43197 .05826 m
.43197 .06451 L
s
[(0.2)] .43197 .04576 0 1 Mshowa
.56803 .05826 m
.56803 .06451 L
s
[(0.25)] .56803 .04576 0 1 Mshowa
.70408 .05826 m
.70408 .06451 L
s
[(0.3)] .70408 .04576 0 1 Mshowa
.84014 .05826 m
.84014 .06451 L
s
[(0.35)] .84014 .04576 0 1 Mshowa
.97619 .05826 m
.97619 .06451 L
s
[(0.4)] .97619 .04576 0 1 Mshowa
.125 Mabswid
.05102 .05826 m
.05102 .06201 L
s
.07823 .05826 m
.07823 .06201 L
s
.10544 .05826 m
.10544 .06201 L
s
.13265 .05826 m
.13265 .06201 L
s
.18707 .05826 m
.18707 .06201 L
s
.21429 .05826 m
.21429 .06201 L
s
.2415 .05826 m
.2415 .06201 L
s
.26871 .05826 m
.26871 .06201 L
s
.32313 .05826 m
.32313 .06201 L
s
.35034 .05826 m
.35034 .06201 L
s
.37755 .05826 m
.37755 .06201 L
s
.40476 .05826 m
.40476 .06201 L
s
.45918 .05826 m
.45918 .06201 L
s
.48639 .05826 m
.48639 .06201 L
s
.51361 .05826 m
.51361 .06201 L
s
.54082 .05826 m
.54082 .06201 L
s
.59524 .05826 m
.59524 .06201 L
s
.62245 .05826 m
.62245 .06201 L
s
.64966 .05826 m
.64966 .06201 L
s
.67687 .05826 m
.67687 .06201 L
s
.73129 .05826 m
.73129 .06201 L
s
.7585 .05826 m
.7585 .06201 L
s
.78571 .05826 m
.78571 .06201 L
s
.81293 .05826 m
.81293 .06201 L
s
.86735 .05826 m
.86735 .06201 L
s
.89456 .05826 m
.89456 .06201 L
s
.92177 .05826 m
.92177 .06201 L
s
.94898 .05826 m
.94898 .06201 L
s
.25 Mabswid
0 .05826 m
1 .05826 L
s
.15986 .14383 m
.16611 .14383 L
s
[(2)] .14736 .14383 1 0 Mshowa
.15986 .22941 m
.16611 .22941 L
s
[(4)] .14736 .22941 1 0 Mshowa
.15986 .31499 m
.16611 .31499 L
s
[(6)] .14736 .31499 1 0 Mshowa
.15986 .40057 m
.16611 .40057 L
s
[(8)] .14736 .40057 1 0 Mshowa
.15986 .48614 m
.16611 .48614 L
s
[(10)] .14736 .48614 1 0 Mshowa
.15986 .57172 m
.16611 .57172 L
s
[(12)] .14736 .57172 1 0 Mshowa
.125 Mabswid
.15986 .07965 m
.16361 .07965 L
s
.15986 .10105 m
.16361 .10105 L
s
.15986 .12244 m
.16361 .12244 L
s
.15986 .16523 m
.16361 .16523 L
s
.15986 .18662 m
.16361 .18662 L
s
.15986 .20802 m
.16361 .20802 L
s
.15986 .25081 m
.16361 .25081 L
s
.15986 .2722 m
.16361 .2722 L
s
.15986 .29359 m
.16361 .29359 L
s
.15986 .33638 m
.16361 .33638 L
s
.15986 .35778 m
.16361 .35778 L
s
.15986 .37917 m
.16361 .37917 L
s
.15986 .42196 m
.16361 .42196 L
s
.15986 .44335 m
.16361 .44335 L
s
.15986 .46475 m
.16361 .46475 L
s
.15986 .50754 m
.16361 .50754 L
s
.15986 .52893 m
.16361 .52893 L
s
.15986 .55033 m
.16361 .55033 L
s
.15986 .03686 m
.16361 .03686 L
s
.15986 .01547 m
.16361 .01547 L
s
.15986 .59311 m
.16361 .59311 L
s
.15986 .61451 m
.16361 .61451 L
s
.25 Mabswid
.15986 0 m
.15986 .61803 L
s
.5 Mabswid
.02381 .4857 m
.03279 .51772 L
.04262 .54757 L
.05288 .57253 L
.05747 .58147 L
.06244 .58952 L
.06529 .59332 L
.06792 .59631 L
.07039 .59863 L
.0731 .60065 L
.07428 .60135 L
.07553 .60199 L
.07662 .60244 L
.07781 .60282 L
.0785 .603 L
.07915 .60312 L
.08042 .60328 L
.08157 .60332 L
.0828 .60324 L
.08352 .60315 L
.08427 .603 L
.08563 .60263 L
.08708 .60207 L
.08868 .60127 L
.09155 .59936 L
.09422 .59702 L
.09922 .59126 L
.10458 .58315 L
.11475 .56272 L
.12582 .53408 L
.14594 .46982 L
.18672 .32336 L
.22599 .19998 L
.26374 .11426 L
.28309 .08288 L
.30394 .05735 L
.32398 .03973 L
.33406 .03305 L
.34508 .02719 L
.35536 .02294 L
.36469 .01996 L
.36986 .01865 L
.37547 .01746 L
.38073 .01657 L
.38564 .0159 L
.39021 .01543 L
.39511 .01506 L
.3979 .01491 L
.39927 .01486 L
.40055 .01481 L
Mistroke
.4018 .01478 L
.40293 .01475 L
.40418 .01473 L
.40487 .01472 L
.40552 .01472 L
.40674 .01472 L
.40744 .01472 L
.40808 .01472 L
.40939 .01473 L
.41012 .01474 L
.41079 .01475 L
.41198 .01478 L
.41328 .01481 L
.41564 .01489 L
.41839 .01502 L
.42131 .01518 L
.42656 .01555 L
.43596 .01645 L
.44607 .0177 L
.46437 .02056 L
.50556 .02867 L
.54523 .03726 L
.58339 .04529 L
.62401 .05306 L
.66311 .05958 L
.70466 .06543 L
.74469 .07006 L
.78321 .07368 L
.82418 .07675 L
.86363 .07904 L
.90157 .08074 L
.94197 .08207 L
.97619 .08289 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{201.125, 124.063},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000390000O1000`40O003h00ObGoo0039Ool00<Uo
o`00bGoo000UOol00`00Oomoo`2QOol002Eoo`03001oogoo0:5oo`000Woo0P0017oo0P0017oo0P00
0goo0`003goo00<007ooOol03Goo0P0017oo0P000goo0`000goo0`0027oo0P0017oo0P000goo1000
27oo0P0017oo0P000goo10000Woo0`0027oo0P0017oo0P000goo0`002Goo0P0017oo0P000goo0`00
0goo0`002Goo0P0017oo0P0017oo0`000Woo00001Goo001oogoo00000goo0P000goo00@007ooOol0
00Eoo`03001oogoo00aoo`8000eoo`04001oogoo0003Ool20004Ool00`00Oomoo`05Ool00`00Oomo
o`02Ool@0009Ool01000Oomoo`000goo0P0017oo00<007ooOol01Goo00<007ooOol017oo00@007oo
Ool000=oo`8000Ioo`03001oogoo00Eoo`04001oogoo0003Ool20006Ool00`00Oomoo`03Ool00`00
Oomoo`05Ool01000Oomoo`000goo0P001Goo00<007ooOol00Goo00001Goo001oogoo000027oo00@0
07ooOol000Eoo`03001oogoo00aoo`03001oogoo00aoo`04001oogoo0009Ool00`00Oomoo`05Ool5
0002Ool01000Oomoo`002Woo1@0017oo00@007ooOol000Yoo`03001oogoo00Aoo`03001oogoo00Ao
o`04001oogoo000;Ool00`00Oomoo`05Ool01000Oomoo`002goo00<007ooOol00goo00<007ooOol0
1Goo00@007ooOol000Qoo`@0009oo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool3000?Ool0
0`00Oomoo`0<Ool01000Oomoo`002Goo00<007ooOol00Woo0`001goo00@007ooOol000]oo`04001o
ogooOol600000goo001oo`0:Ool00`00Oomoo`030007Ool01000Oomoo`002Goo0P0027oo00@007oo
Ool000Uoo`8000=oo`<000Qoo`04001oogoo0008Ool00`00Ool00003Ool00005Ool007ooOol00008
Ool01000Oomoo`000Woo00<007ooOol03goo00<007ooOol037oo00@007ooOol000Uoo`03001oogoo
009oo`03001oogoo00Moo`04001oogoo0008Ool01000Oomoo`001goo00<007oo00000`001Woo00@0
07ooOol0009oo`03001oogoo00Moo`04001oogoo000;Ool00`00Oomoo`05Ool01000Oomoo`002goo
00@007ooOol000Yoo`04001oogoo0008Ool00`00Ool00003Ool0009oo`8000Yoo`8000=oo`@000io
o`8000ioo`8000Uoo`8000=oo`D000Moo`8000Yoo`8000Uoo`8000=oo`@000=oo`8000=oo`@000Mo
o`8000Uoo`<000Uoo`8000Uoo`<000=oo`@000Qoo`8000Yoo`8000=oo`009Goo00<007ooOol06goo
0P00;7oo0`00EGoo000UOol00`00Oomoo`0JOol00`00Oomoo`0_Ool6001?Ool002Eoo`03001oogoo
01Uoo`03001oogoo03Ioo`L004Qoo`001goo_@001Goo000<Ool00`00Oomoo`02Ool00`00Oomoo`02
Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02
Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool20003Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`090004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool0
0`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`03Ool0
0`00Oomoo`02Ool00`00Oomoo`07Ool002Eoo`03001oogoo01Moo`03001oogoo04moo`L0035oo`00
9Goo00<007ooOol05Woo00<007ooOol0Egoo3`008Woo000UOol2000FOol00`00Oomoo`1WOolF000<
Ool002Eoo`03001oogoo01Aoo`03001oogoo07ioo`<000Uoo`009Goo00<007ooOol057oo00<007oo
Ool0RWoo000UOol00`00Oomoo`0COol00`00Oomoo`2;Ool002Eoo`8001Aoo`03001oogoo08]oo`00
9Goo00<007ooOol04Woo00<007ooOol0S7oo000UOol00`00Oomoo`0BOol00`00Oomoo`2<Ool002Eo
o`03001oogoo015oo`03001oogoo08eoo`009Goo0P004Woo00<007ooOol0SGoo000UOol00`00Oomo
o`0@Ool00`00Oomoo`2>Ool001ioo`@000=oo`03001oogoo011oo`03001oogoo08ioo`007goo00<0
07ooOol00goo00<007ooOol03goo00<007ooOol0Sgoo000POol00`00Oomoo`02Ool2000@Ool00`00
Oomoo`2?Ool0025oo`05001oogooOol0000AOol00`00Oomoo`2?Ool001ioo`04001oogoo0003Ool0
0`00Oomoo`0>Ool00`00Oomoo`2@Ool001moo`8000Aoo`03001oogoo00ioo`03001oogoo091oo`00
9Goo0P003Woo00<007ooOol0TGoo000UOol00`00Oomoo`0=Ool00`00Oomoo`2AOol002Eoo`03001o
ogoo00aoo`03001oogoo099oo`009Goo00<007ooOol037oo00<007ooOol0TWoo000UOol2000=Ool0
0`00Oomoo`2BOol002Eoo`03001oogoo00]oo`03001oogoo09=oo`009Goo00<007ooOol02goo00<0
07ooOol0Tgoo000UOol00`00Oomoo`0:Ool00`00Oomoo`2DOol002Eoo`8000]oo`03001oogoo09Ao
o`009Goo00<007ooOol02Goo00<007ooOol0UGoo000OOol30003Ool00`00Oomoo`09Ool00`00Oomo
o`2EOol0021oo`03001oogoo009oo`03001oogoo00Uoo`03001oogoo09Eoo`007Woo10000goo0P00
2Goo00<007ooOol0UWoo000NOol00`00Ool00004Ool00`00Oomoo`08Ool00`00Oomoo`2FOol001io
o`03001oo`0000Aoo`03001oogoo00Qoo`03001oogoo09Ioo`007goo0P0017oo00<007ooOol01goo
00<007ooOol0Ugoo000UOol20008Ool00`00Oomoo`2GOol002Eoo`03001oogoo00Moo`03001oogoo
09Moo`009Goo00<007ooOol01Woo00<007ooOol0V7oo000UOol00`00Oomoo`06Ool00`00Oomoo`2H
Ool002Eoo`8000Moo`03001oogoo09Qoo`009Goo00<007ooOol01Goo00<007ooOol0VGoo000UOol0
0`00Oomoo`05Ool00`00Oomoo`2IOol002Eoo`03001oogoo00Eoo`03001oogoo09Uoo`009Goo0P00
1Goo00<007ooOol0VWoo000UOol00`00Oomoo`04Ool00`00Oomoo`2JOol001moo`8000Aoo`03001o
ogoo00Aoo`03001oogoo09Yoo`007Woo00@007ooOol000=oo`03001oogoo00=oo`03001oogoo09]o
o`007Woo00@007ooOol000=oo`8000Aoo`03001oogoo09]oo`007Woo0`0017oo00<007ooOol00goo
00<007ooOol0Vgoo000NOol00`00Oomoo`04Ool00`00Oomoo`02Ool00`00Oomoo`2LOol001moo`<0
00=oo`03001oogoo009oo`03001oogoo09aoo`009Goo0P000goo00<007ooOol0W7oo000UOol01@00
Oomoogoo0000Wgoo000UOol01@00Oomoogoo0000Wgoo000UOol01@00Oomoogoo0000Wgoo000UOol2
00000goo001oo`2OOol002Eoo`04001oogoo002POol002Eoo`04001oogoo002POol002Eoo`03001o
o`000:5oo`009Goo0`00XGoo000UOol00`00Ool0002QOol001moo`8000Aoo`03001oo`000:5oo`00
7Woo00@007ooOol000=oo`800:9oo`007Woo00@007ooOol000=oo`800:9oo`007goo0P0017oo0P00
XWoo000NOol01000Oomoo`000goo00<007ooOol0XGoo000OOol20004Ool00`00Oomoo`2QOol002Eo
o`800:9oo`009Goo00<007ooOol0XGoo000TOol2002SOol002Aoo`800:=oo`0097oo0`00XWoo000S
Ool00`00Ool0002SOol002=oo`03001oo`000:=oo`008goo00<007oo0000Xgoo000ROol00`00Oomo
o`02002ROol0029oo`04001oogoo002SOol001Qoo`<000Aoo`800005Ool007ooOol0002SOol001Uo
o`03001oogoo009oo`03001oogoo0080009oo`03001oogoo0:5oo`0037oo00<007ooOol02Woo00<0
07ooOol00Woo00@007ooOol000=oo`800:9oo`0037oo00<007ooOol02Woo00<007ooOol00Woo00@0
07ooOol000=oo`03001oogoo0:5oo`0037oo00<007ooOol02Woo00<007ooOol00Woo00@007ooOol0
00=oo`03001oogoo0:5oo`0037oo00<007ooOol02Goo0P001Goo0`000goo00<007ooOol0XGoo000=
Ool00`00Oomoo`0@Ool00`00Oomoo`02Ool00`00Oomoo`2QOol000eoo`03001oogoo011oo`03001o
ogoo009oo`800:9oo`003Goo00<007ooOol047oo00<007ooOol00Woo00<007ooOol0XGoo000=Ool0
0`00Oomoo`0@Ool00`00Oomoo`02Ool00`00Oomoo`2QOol000ioo`03001oogoo00ioo`03001oogoo
00=oo`03001oogoo0:5oo`003Woo00<007ooOol03Woo00<007ooOol00goo0P00XWoo000>Ool00`00
Oomoo`0>Ool00`00Oomoo`03Ool00`00Oomoo`2QOol000moo`03001oogoo00aoo`03001oogoo00Ao
o`03001oogoo0:5oo`003goo00<007ooOol037oo00<007ooOol017oo00<007ooOol0XGoo000?Ool0
0`00Oomoo`0<Ool00`00Oomoo`04Ool2002ROol0011oo`03001oogoo00Yoo`03001oogoo00Eoo`03
001oogoo0:5oo`0047oo00<007ooOol01Goo0`000Woo1@000goo00<007ooOol0XGoo000@Ool00`00
Oomoo`06Ool01`00Oomoogoo001oo`0000Eoo`03001oogoo0:5oo`004Goo00<007ooOol01Goo00@0
07ooOol000=oo`03001oogoo009oo`800:9oo`004Woo00<007ooOol017oo00@007ooOol000Aoo`05
001oogooOol0002SOol0019oo`03001oogoo00Aoo`03001oo`00009oo`04001oogoo0003Ool00`00
Oomoo`2QOol001=oo`03001oogoo009oo`<000Aoo`8000Aoo`03001oogoo0:5oo`004goo0P0017oo
00<007ooOol02Goo0P00XWoo000DOol5000<Ool00`00Oomoo`2QOol001Ioo`8000eoo`03001oogoo
0:5oo`009Goo00<007ooOol0XGoo000UOol2002ROol00<Uoo`00bGoo0039Ool00001\
\>"],
  ImageRangeCache->{{{0, 200.125}, {123.063, 0}} -> {0.0265509, -1.80990, 
  0.00195396, 0.12426}}]
}, Open  ]],

Cell[TextData[{
  "In Chapter 8 you will see how Vega is defined by elementary \
differentiation using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s \"D\" operator."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Things Are Impossible - There Are Infinitely Many Answers", 
  "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[TextData[{
  "If you want to see just how bad it can get, a Barrier option can do still \
more interesting things. If we consider an Up-and-Out Put, where the strike \
coincides with the barrier, and arrange for the risk-free rate and the \
dividend yield to coincide, we can get a dead zero Vega (and zero Gamma too). \
So if the market price happens to coincide with the computed value, ",
  StyleBox["you can have any implied volatility you want",
    FontSlant->"Italic"],
  ". Otherwise there is no implied volatility."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Plot3D[UpAndOutPut[0, S, 50, 50, vol, 0.1, 0.1, 1],
{S, 30, 50}, {vol, 0.05, 0.4}, PlotRange -> {-0.01, 0.01}, 
PlotPoints -> 40];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.74908 .0689 0 -6.18931 ]
[.74908 .0689 18 2.81069 ]
[.83635 .20254 0 -6.06047 ]
[.83635 .20254 18 2.93953 ]
[.9134 .32052 0 -5.94988 ]
[.9134 .32052 18 3.05012 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 18 3.14607 ]
[.02494 .25992 -30 -2.73794 ]
[.02494 .25992 0 6.26206 ]
[.01611 .31527 -36 -2.78472 ]
[.01611 .31527 0 6.21528 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00254 .43215 -30 -2.88407 ]
[-0.00254 .43215 0 6.11593 ]
[-0.0124 .49393 -24 -2.93688 ]
[-0.0124 .49393 0 6.06312 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.7368 .07352 m
.73065 .07582 L
s
[(0.1)] .74908 .0689 -1 .3754 Mshowa
.82395 .20683 m
.81775 .20898 L
s
[(0.2)] .83635 .20254 -1 .34677 Mshowa
.90091 .32455 m
.89466 .32656 L
s
[(0.3)] .9134 .32052 -1 .3222 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(0.4)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.75514 .10158 m
.75145 .10294 L
s
.77301 .12891 m
.76931 .13025 L
s
.79042 .15555 m
.78672 .15687 L
s
.8074 .18151 m
.80369 .18282 L
s
.8401 .23153 m
.83637 .23281 L
s
.85585 .25563 m
.85212 .25689 L
s
.87123 .27915 m
.8675 .28039 L
s
.88624 .30212 m
.8825 .30334 L
s
.91523 .34645 m
.91148 .34765 L
s
.92922 .36786 m
.92547 .36904 L
s
.9429 .38878 m
.93914 .38994 L
s
.95627 .40924 m
.95251 .41038 L
s
.71796 .0447 m
.71428 .04611 L
s
.69861 .0151 m
.69494 .01653 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03716 .25514 m
.04327 .25274 L
s
[(-0.01)] .02494 .25992 1 -0.39157 Mshowa
.02837 .31059 m
.0345 .30825 L
s
[(-0.005)] .01611 .31527 1 -0.38117 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.00981 .42772 m
.01599 .4255 L
s
[(0.005)] -0.00254 .43215 1 -0.3591 Mshowa
0 .48963 m
.0062 .48748 L
s
[(0.01)] -0.0124 .49393 1 -0.34736 Mshowa
.125 Mabswid
.03543 .26607 m
.0391 .26464 L
s
.03368 .27708 m
.03735 .27566 L
s
.03193 .28817 m
.0356 .28676 L
s
.03016 .29934 m
.03383 .29793 L
s
.02658 .32192 m
.03026 .32053 L
s
.02477 .33334 m
.02845 .33195 L
s
.02295 .34483 m
.02663 .34345 L
s
.02111 .35641 m
.0248 .35504 L
s
.0174 .37983 m
.02109 .37847 L
s
.01552 .39167 m
.01922 .39032 L
s
.01363 .4036 m
.01733 .40225 L
s
.01173 .41561 m
.01543 .41428 L
s
.00788 .43991 m
.01159 .43859 L
s
.00593 .4522 m
.00964 .45089 L
s
.00397 .46458 m
.00768 .46328 L
s
.00199 .47706 m
.00571 .47576 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.645 .763 .934 r
.38953 .80439 .39787 .81114 .41179 .80749 .40349 .80071 Metetra
.645 .763 .934 r
.40349 .80071 .41179 .80749 .42578 .80383 .41753 .79702 Metetra
.645 .763 .934 r
.41753 .79702 .42578 .80383 .43985 .80015 .43164 .7933 Metetra
.645 .763 .934 r
.43164 .7933 .43985 .80015 .45398 .79645 .44583 .78956 Metetra
.645 .763 .934 r
.44583 .78956 .45398 .79645 .4682 .79273 .46008 .7858 Metetra
.645 .763 .934 r
.46008 .7858 .4682 .79273 .48248 .78898 .47442 .78203 Metetra
.645 .763 .934 r
.47442 .78203 .48248 .78898 .49684 .78523 .48883 .77823 Metetra
.645 .763 .934 r
.48883 .77823 .49684 .78523 .51128 .78145 .50332 .77441 Metetra
.645 .763 .934 r
.50332 .77441 .51128 .78145 .5258 .77765 .51788 .77058 Metetra
.645 .763 .934 r
.51788 .77058 .5258 .77765 .54039 .77383 .53252 .76672 Metetra
.645 .763 .934 r
.53252 .76672 .54039 .77383 .55506 .76999 .54724 .76284 Metetra
.645 .763 .934 r
.54724 .76284 .55506 .76999 .5698 .76612 .56204 .75894 Metetra
.645 .763 .934 r
.56204 .75894 .5698 .76612 .58463 .76224 .57692 .75502 Metetra
.645 .763 .934 r
.57692 .75502 .58463 .76224 .59953 .75834 .59188 .75108 Metetra
.645 .763 .934 r
.59188 .75108 .59953 .75834 .61452 .75442 .60692 .74712 Metetra
.645 .763 .934 r
.60692 .74712 .61452 .75442 .62958 .75047 .62204 .74314 Metetra
.645 .763 .934 r
.62204 .74314 .62958 .75047 .64473 .74651 .63724 .73913 Metetra
.645 .763 .934 r
.63724 .73913 .64473 .74651 .65996 .74252 .65253 .7351 Metetra
.645 .763 .934 r
.65253 .7351 .65996 .74252 .67527 .73851 .6679 .73105 Metetra
.645 .763 .934 r
.6679 .73105 .67527 .73851 .69067 .73448 .68335 .72698 Metetra
.645 .763 .934 r
.68335 .72698 .69067 .73448 .70615 .73043 .69889 .72289 Metetra
.645 .763 .934 r
.69889 .72289 .70615 .73043 .72171 .72635 .71451 .71877 Metetra
.645 .763 .934 r
.71451 .71877 .72171 .72635 .73736 .72226 .73022 .71463 Metetra
.645 .763 .934 r
.73022 .71463 .73736 .72226 .7531 .71814 .74602 .71047 Metetra
.645 .763 .934 r
.74602 .71047 .7531 .71814 .76892 .714 .7619 .70629 Metetra
.645 .763 .934 r
.7619 .70629 .76892 .714 .78483 .70983 .77788 .70208 Metetra
.645 .763 .934 r
.77788 .70208 .78483 .70983 .80083 .70564 .79394 .69785 Metetra
.645 .763 .934 r
.79394 .69785 .80083 .70564 .81692 .70143 .81009 .69359 Metetra
.645 .763 .934 r
.81009 .69359 .81692 .70143 .8331 .69719 .82633 .68931 Metetra
.645 .763 .934 r
.82633 .68931 .8331 .69719 .84937 .69293 .84267 .68501 Metetra
.645 .763 .934 r
.84267 .68501 .84937 .69293 .86573 .68865 .8591 .68068 Metetra
.645 .763 .934 r
.8591 .68068 .86573 .68865 .88218 .68434 .87562 .67633 Metetra
.645 .763 .934 r
.87562 .67633 .88218 .68434 .89873 .68001 .89223 .67195 Metetra
.645 .763 .934 r
.89223 .67195 .89873 .68001 .91537 .67566 .90894 .66755 Metetra
.645 .763 .934 r
.90894 .66755 .91537 .67566 .9321 .67128 .92574 .66312 Metetra
.645 .763 .934 r
.92574 .66312 .9321 .67128 .94893 .66687 .94264 .65867 Metetra
.645 .763 .934 r
.94264 .65867 .94893 .66687 .96586 .66244 .95964 .65419 Metetra
.645 .763 .934 r
.95964 .65419 .96586 .66244 .98288 .65798 .97673 .64969 Metetra
.988 .732 .448 r
.97673 .64969 .98288 .65798 .98416 .53763 .97821 .52897 Metetra
.645 .763 .934 r
.3811 .79759 .38953 .80439 .40349 .80071 .39511 .79387 Metetra
.645 .763 .934 r
.39511 .79387 .40349 .80071 .41753 .79702 .4092 .79014 Metetra
.645 .763 .934 r
.4092 .79014 .41753 .79702 .43164 .7933 .42335 .78638 Metetra
.645 .763 .934 r
.42335 .78638 .43164 .7933 .44583 .78956 .43759 .78261 Metetra
.645 .763 .934 r
.43759 .78261 .44583 .78956 .46008 .7858 .45189 .77882 Metetra
.645 .763 .934 r
.45189 .77882 .46008 .7858 .47442 .78203 .46628 .775 Metetra
.645 .763 .934 r
.46628 .775 .47442 .78203 .48883 .77823 .48074 .77117 Metetra
.645 .763 .934 r
.48074 .77117 .48883 .77823 .50332 .77441 .49527 .76732 Metetra
.645 .763 .934 r
.49527 .76732 .50332 .77441 .51788 .77058 .50989 .76344 Metetra
.645 .763 .934 r
.50989 .76344 .51788 .77058 .53252 .76672 .52458 .75954 Metetra
.645 .763 .934 r
.52458 .75954 .53252 .76672 .54724 .76284 .53935 .75563 Metetra
.645 .763 .934 r
.53935 .75563 .54724 .76284 .56204 .75894 .5542 .75169 Metetra
.645 .763 .934 r
.5542 .75169 .56204 .75894 .57692 .75502 .56913 .74773 Metetra
.645 .763 .934 r
.56913 .74773 .57692 .75502 .59188 .75108 .58415 .74375 Metetra
.645 .763 .934 r
.58415 .74375 .59188 .75108 .60692 .74712 .59924 .73975 Metetra
.645 .763 .934 r
.59924 .73975 .60692 .74712 .62204 .74314 .61442 .73572 Metetra
.645 .763 .934 r
.61442 .73572 .62204 .74314 .63724 .73913 .62967 .73168 Metetra
.645 .763 .934 r
.62967 .73168 .63724 .73913 .65253 .7351 .64502 .72761 Metetra
.645 .763 .934 r
.64502 .72761 .65253 .7351 .6679 .73105 .66044 .72352 Metetra
.645 .763 .934 r
.66044 .72352 .6679 .73105 .68335 .72698 .67595 .71941 Metetra
.645 .763 .934 r
.67595 .71941 .68335 .72698 .69889 .72289 .69155 .71527 Metetra
.645 .763 .934 r
.69155 .71527 .69889 .72289 .71451 .71877 .70723 .71111 Metetra
.645 .763 .934 r
.70723 .71111 .71451 .71877 .73022 .71463 .72301 .70693 Metetra
.645 .763 .934 r
.72301 .70693 .73022 .71463 .74602 .71047 .73886 .70273 Metetra
.645 .763 .934 r
.73886 .70273 .74602 .71047 .7619 .70629 .75481 .6985 Metetra
.645 .763 .934 r
.75481 .6985 .7619 .70629 .77788 .70208 .77085 .69425 Metetra
.645 .763 .934 r
.77085 .69425 .77788 .70208 .79394 .69785 .78697 .68997 Metetra
.645 .763 .934 r
.78697 .68997 .79394 .69785 .81009 .69359 .80319 .68567 Metetra
.645 .763 .934 r
.80319 .68567 .81009 .69359 .82633 .68931 .8195 .68135 Metetra
.645 .763 .934 r
.8195 .68135 .82633 .68931 .84267 .68501 .8359 .677 Metetra
.645 .763 .934 r
.8359 .677 .84267 .68501 .8591 .68068 .85239 .67263 Metetra
.645 .763 .934 r
.85239 .67263 .8591 .68068 .87562 .67633 .86898 .66823 Metetra
.645 .763 .934 r
.86898 .66823 .87562 .67633 .89223 .67195 .88566 .6638 Metetra
.645 .763 .934 r
.88566 .6638 .89223 .67195 .90894 .66755 .90244 .65936 Metetra
.645 .763 .934 r
.90244 .65936 .90894 .66755 .92574 .66312 .91931 .65488 Metetra
.645 .763 .934 r
.91931 .65488 .92574 .66312 .94264 .65867 .93628 .65038 Metetra
.645 .763 .934 r
.93628 .65038 .94264 .65867 .95964 .65419 .95335 .64586 Metetra
.645 .763 .934 r
.95335 .64586 .95964 .65419 .97673 .64969 .97052 .6413 Metetra
.988 .732 .448 r
.97052 .6413 .97673 .64969 .97821 .52897 .9722 .52022 Metetra
.645 .763 .934 r
.3726 .79071 .3811 .79759 .39511 .79387 .38665 .78696 Metetra
.645 .763 .934 r
.38665 .78696 .39511 .79387 .4092 .79014 .40078 .78319 Metetra
.645 .763 .934 r
.40078 .78319 .4092 .79014 .42335 .78638 .41499 .7794 Metetra
.645 .763 .934 r
.41499 .7794 .42335 .78638 .43759 .78261 .42927 .77559 Metetra
.645 .763 .934 r
.42927 .77559 .43759 .78261 .45189 .77882 .44362 .77176 Metetra
.645 .763 .934 r
.44362 .77176 .45189 .77882 .46628 .775 .45805 .76791 Metetra
.645 .763 .934 r
.45805 .76791 .46628 .775 .48074 .77117 .47256 .76404 Metetra
.645 .763 .934 r
.47256 .76404 .48074 .77117 .49527 .76732 .48715 .76015 Metetra
.645 .763 .934 r
.48715 .76015 .49527 .76732 .50989 .76344 .50181 .75623 Metetra
.645 .763 .934 r
.50181 .75623 .50989 .76344 .52458 .75954 .51656 .7523 Metetra
.645 .763 .934 r
.51656 .7523 .52458 .75954 .53935 .75563 .53138 .74834 Metetra
.645 .763 .934 r
.53138 .74834 .53935 .75563 .5542 .75169 .54629 .74436 Metetra
.645 .763 .934 r
.54629 .74436 .5542 .75169 .56913 .74773 .56127 .74037 Metetra
.645 .763 .934 r
.56127 .74037 .56913 .74773 .58415 .74375 .57634 .73635 Metetra
.645 .763 .934 r
.57634 .73635 .58415 .74375 .59924 .73975 .59148 .7323 Metetra
.645 .763 .934 r
.59148 .7323 .59924 .73975 .61442 .73572 .60672 .72824 Metetra
.645 .763 .934 r
.60672 .72824 .61442 .73572 .62967 .73168 .62203 .72415 Metetra
.645 .763 .934 r
.62203 .72415 .62967 .73168 .64502 .72761 .63743 .72004 Metetra
.645 .763 .934 r
.63743 .72004 .64502 .72761 .66044 .72352 .65291 .71591 Metetra
.645 .763 .934 r
.65291 .71591 .66044 .72352 .67595 .71941 .66848 .71176 Metetra
.645 .763 .934 r
.66848 .71176 .67595 .71941 .69155 .71527 .68414 .70758 Metetra
.645 .763 .934 r
.68414 .70758 .69155 .71527 .70723 .71111 .69988 .70338 Metetra
.645 .763 .934 r
.69988 .70338 .70723 .71111 .72301 .70693 .71571 .69915 Metetra
.645 .763 .934 r
.71571 .69915 .72301 .70693 .73886 .70273 .73163 .6949 Metetra
.645 .763 .934 r
.73163 .6949 .73886 .70273 .75481 .6985 .74764 .69063 Metetra
.645 .763 .934 r
.74764 .69063 .75481 .6985 .77085 .69425 .76374 .68634 Metetra
.645 .763 .934 r
.76374 .68634 .77085 .69425 .78697 .68997 .77993 .68202 Metetra
.645 .763 .934 r
.77993 .68202 .78697 .68997 .80319 .68567 .79621 .67767 Metetra
.645 .763 .934 r
.79621 .67767 .80319 .68567 .8195 .68135 .81259 .6733 Metetra
.645 .763 .934 r
.81259 .6733 .8195 .68135 .8359 .677 .82906 .66891 Metetra
.645 .763 .934 r
.82906 .66891 .8359 .677 .85239 .67263 .84562 .66449 Metetra
.645 .763 .934 r
.84562 .66449 .85239 .67263 .86898 .66823 .86227 .66004 Metetra
.645 .763 .934 r
.86227 .66004 .86898 .66823 .88566 .6638 .87902 .65557 Metetra
.645 .763 .934 r
.87902 .65557 .88566 .6638 .90244 .65936 .89587 .65108 Metetra
.645 .763 .934 r
.89587 .65108 .90244 .65936 .91931 .65488 .91282 .64655 Metetra
.645 .763 .934 r
.91282 .64655 .91931 .65488 .93628 .65038 .92986 .64201 Metetra
.645 .763 .934 r
.92986 .64201 .93628 .65038 .95335 .64586 .947 .63743 Metetra
.645 .763 .934 r
.947 .63743 .95335 .64586 .97052 .6413 .96424 .63283 Metetra
.988 .732 .448 r
.96424 .63283 .97052 .6413 .9722 .52022 .96613 .51138 Metetra
.645 .763 .934 r
.36401 .78377 .3726 .79071 .38665 .78696 .37811 .77999 Metetra
.645 .763 .934 r
.37811 .77999 .38665 .78696 .40078 .78319 .39229 .77618 Metetra
.645 .763 .934 r
.39229 .77618 .40078 .78319 .41499 .7794 .40654 .77235 Metetra
.645 .763 .934 r
.40654 .77235 .41499 .7794 .42927 .77559 .42087 .7685 Metetra
.645 .763 .934 r
.42087 .7685 .42927 .77559 .44362 .77176 .43527 .76463 Metetra
.645 .763 .934 r
.43527 .76463 .44362 .77176 .45805 .76791 .44975 .76075 Metetra
.645 .763 .934 r
.44975 .76075 .45805 .76791 .47256 .76404 .46431 .75684 Metetra
.645 .763 .934 r
.46431 .75684 .47256 .76404 .48715 .76015 .47895 .7529 Metetra
.645 .763 .934 r
.47895 .7529 .48715 .76015 .50181 .75623 .49366 .74895 Metetra
.645 .763 .934 r
.49366 .74895 .50181 .75623 .51656 .7523 .50846 .74498 Metetra
.645 .763 .934 r
.50846 .74498 .51656 .7523 .53138 .74834 .52333 .74098 Metetra
.645 .763 .934 r
.52333 .74098 .53138 .74834 .54629 .74436 .53829 .73697 Metetra
.645 .763 .934 r
.53829 .73697 .54629 .74436 .56127 .74037 .55333 .73293 Metetra
.645 .763 .934 r
.55333 .73293 .56127 .74037 .57634 .73635 .56845 .72887 Metetra
.645 .763 .934 r
.56845 .72887 .57634 .73635 .59148 .7323 .58365 .72478 Metetra
.645 .763 .934 r
.58365 .72478 .59148 .7323 .60672 .72824 .59894 .72068 Metetra
.645 .763 .934 r
.59894 .72068 .60672 .72824 .62203 .72415 .61431 .71655 Metetra
.645 .763 .934 r
.61431 .71655 .62203 .72415 .63743 .72004 .62977 .7124 Metetra
.645 .763 .934 r
.62977 .7124 .63743 .72004 .65291 .71591 .64531 .70822 Metetra
.645 .763 .934 r
.64531 .70822 .65291 .71591 .66848 .71176 .66094 .70402 Metetra
.645 .763 .934 r
.66094 .70402 .66848 .71176 .68414 .70758 .67665 .6998 Metetra
.645 .763 .934 r
.67665 .6998 .68414 .70758 .69988 .70338 .69246 .69556 Metetra
.645 .763 .934 r
.69246 .69556 .69988 .70338 .71571 .69915 .70835 .69129 Metetra
.645 .763 .934 r
.70835 .69129 .71571 .69915 .73163 .6949 .72433 .687 Metetra
.645 .763 .934 r
.72433 .687 .73163 .6949 .74764 .69063 .7404 .68268 Metetra
.645 .763 .934 r
.7404 .68268 .74764 .69063 .76374 .68634 .75656 .67834 Metetra
.645 .763 .934 r
.75656 .67834 .76374 .68634 .77993 .68202 .77282 .67397 Metetra
.645 .763 .934 r
.77282 .67397 .77993 .68202 .79621 .67767 .78917 .66958 Metetra
.645 .763 .934 r
.78917 .66958 .79621 .67767 .81259 .6733 .80561 .66517 Metetra
.645 .763 .934 r
.80561 .66517 .81259 .6733 .82906 .66891 .82214 .66073 Metetra
.645 .763 .934 r
.82214 .66073 .82906 .66891 .84562 .66449 .83877 .65626 Metetra
.645 .763 .934 r
.83877 .65626 .84562 .66449 .86227 .66004 .85549 .65177 Metetra
.645 .763 .934 r
.85549 .65177 .86227 .66004 .87902 .65557 .87231 .64725 Metetra
.645 .763 .934 r
.87231 .64725 .87902 .65557 .89587 .65108 .88923 .64271 Metetra
.645 .763 .934 r
.88923 .64271 .89587 .65108 .91282 .64655 .90625 .63814 Metetra
.645 .763 .934 r
.90625 .63814 .91282 .64655 .92986 .64201 .92337 .63354 Metetra
.645 .763 .934 r
.92337 .63354 .92986 .64201 .947 .63743 .94058 .62892 Metetra
.645 .763 .934 r
.94058 .62892 .947 .63743 .96424 .63283 .9579 .62427 Metetra
.988 .732 .448 r
.9579 .62427 .96424 .63283 .96613 .51138 .95999 .50245 Metetra
.645 .763 .934 r
.35534 .77677 .36401 .78377 .37811 .77999 .36949 .77294 Metetra
.645 .763 .934 r
.36949 .77294 .37811 .77999 .39229 .77618 .38371 .7691 Metetra
.645 .763 .934 r
.38371 .7691 .39229 .77618 .40654 .77235 .39801 .76523 Metetra
.645 .763 .934 r
.39801 .76523 .40654 .77235 .42087 .7685 .41238 .76135 Metetra
.645 .763 .934 r
.41238 .76135 .42087 .7685 .43527 .76463 .42683 .75744 Metetra
.645 .763 .934 r
.42683 .75744 .43527 .76463 .44975 .76075 .44136 .75351 Metetra
.645 .763 .934 r
.44136 .75351 .44975 .76075 .46431 .75684 .45597 .74956 Metetra
.645 .763 .934 r
.45597 .74956 .46431 .75684 .47895 .7529 .47066 .74559 Metetra
.645 .763 .934 r
.47066 .74559 .47895 .7529 .49366 .74895 .48543 .7416 Metetra
.645 .763 .934 r
.48543 .7416 .49366 .74895 .50846 .74498 .50027 .73758 Metetra
.645 .763 .934 r
.50027 .73758 .50846 .74498 .52333 .74098 .5152 .73355 Metetra
.645 .763 .934 r
.5152 .73355 .52333 .74098 .53829 .73697 .53021 .72949 Metetra
.645 .763 .934 r
.53021 .72949 .53829 .73697 .55333 .73293 .5453 .72541 Metetra
.645 .763 .934 r
.5453 .72541 .55333 .73293 .56845 .72887 .56048 .72131 Metetra
.645 .763 .934 r
.56048 .72131 .56845 .72887 .58365 .72478 .57574 .71718 Metetra
.645 .763 .934 r
.57574 .71718 .58365 .72478 .59894 .72068 .59108 .71304 Metetra
.645 .763 .934 r
.59108 .71304 .59894 .72068 .61431 .71655 .60651 .70887 Metetra
.645 .763 .934 r
.60651 .70887 .61431 .71655 .62977 .7124 .62202 .70467 Metetra
.645 .763 .934 r
.62202 .70467 .62977 .7124 .64531 .70822 .63762 .70045 Metetra
.645 .763 .934 r
.63762 .70045 .64531 .70822 .66094 .70402 .65331 .69621 Metetra
.645 .763 .934 r
.65331 .69621 .66094 .70402 .67665 .6998 .66909 .69195 Metetra
.645 .763 .934 r
.66909 .69195 .67665 .6998 .69246 .69556 .68495 .68766 Metetra
.645 .763 .934 r
.68495 .68766 .69246 .69556 .70835 .69129 .70091 .68335 Metetra
.645 .763 .934 r
.70091 .68335 .70835 .69129 .72433 .687 .71695 .67901 Metetra
.645 .763 .934 r
.71695 .67901 .72433 .687 .7404 .68268 .73309 .67465 Metetra
.645 .763 .934 r
.73309 .67465 .7404 .68268 .75656 .67834 .74931 .67026 Metetra
.645 .763 .934 r
.74931 .67026 .75656 .67834 .77282 .67397 .76563 .66585 Metetra
.645 .763 .934 r
.76563 .66585 .77282 .67397 .78917 .66958 .78204 .66141 Metetra
.645 .763 .934 r
.78204 .66141 .78917 .66958 .80561 .66517 .79855 .65695 Metetra
.645 .763 .934 r
.79855 .65695 .80561 .66517 .82214 .66073 .81515 .65246 Metetra
.645 .763 .934 r
.81515 .65246 .82214 .66073 .83877 .65626 .83185 .64795 Metetra
.645 .763 .934 r
.83185 .64795 .83877 .65626 .85549 .65177 .84864 .64341 Metetra
.645 .763 .934 r
.84864 .64341 .85549 .65177 .87231 .64725 .86553 .63884 Metetra
.645 .763 .934 r
.86553 .63884 .87231 .64725 .88923 .64271 .88252 .63425 Metetra
.645 .763 .934 r
.88252 .63425 .88923 .64271 .90625 .63814 .89961 .62963 Metetra
.645 .763 .934 r
.89961 .62963 .90625 .63814 .92337 .63354 .9168 .62498 Metetra
.645 .763 .934 r
.9168 .62498 .92337 .63354 .94058 .62892 .93409 .62031 Metetra
.645 .763 .934 r
.93409 .62031 .94058 .62892 .9579 .62427 .95148 .61561 Metetra
.988 .732 .448 r
.95148 .61561 .9579 .62427 .95999 .50245 .95378 .49342 Metetra
.645 .763 .934 r
.34658 .76969 .35534 .77677 .36949 .77294 .36077 .76583 Metetra
.645 .763 .934 r
.36077 .76583 .36949 .77294 .38371 .7691 .37504 .76194 Metetra
.645 .763 .934 r
.37504 .76194 .38371 .7691 .39801 .76523 .38939 .75804 Metetra
.645 .763 .934 r
.38939 .75804 .39801 .76523 .41238 .76135 .40381 .75412 Metetra
.645 .763 .934 r
.40381 .75412 .41238 .76135 .42683 .75744 .41831 .75017 Metetra
.645 .763 .934 r
.41831 .75017 .42683 .75744 .44136 .75351 .43289 .7462 Metetra
.645 .763 .934 r
.43289 .7462 .44136 .75351 .45597 .74956 .44755 .74221 Metetra
.645 .763 .934 r
.44755 .74221 .45597 .74956 .47066 .74559 .46229 .7382 Metetra
.645 .763 .934 r
.46229 .7382 .47066 .74559 .48543 .7416 .47711 .73417 Metetra
.645 .763 .934 r
.47711 .73417 .48543 .7416 .50027 .73758 .49201 .73012 Metetra
.645 .763 .934 r
.49201 .73012 .50027 .73758 .5152 .73355 .50699 .72604 Metetra
.645 .763 .934 r
.50699 .72604 .5152 .73355 .53021 .72949 .52205 .72194 Metetra
.645 .763 .934 r
.52205 .72194 .53021 .72949 .5453 .72541 .5372 .71782 Metetra
.645 .763 .934 r
.5372 .71782 .5453 .72541 .56048 .72131 .55243 .71368 Metetra
.645 .763 .934 r
.55243 .71368 .56048 .72131 .57574 .71718 .56774 .70951 Metetra
.645 .763 .934 r
.56774 .70951 .57574 .71718 .59108 .71304 .58314 .70532 Metetra
.645 .763 .934 r
.58314 .70532 .59108 .71304 .60651 .70887 .59863 .7011 Metetra
.645 .763 .934 r
.59863 .7011 .60651 .70887 .62202 .70467 .6142 .69687 Metetra
.645 .763 .934 r
.6142 .69687 .62202 .70467 .63762 .70045 .62986 .69261 Metetra
.645 .763 .934 r
.62986 .69261 .63762 .70045 .65331 .69621 .64561 .68832 Metetra
.645 .763 .934 r
.64561 .68832 .65331 .69621 .66909 .69195 .66144 .68401 Metetra
.645 .763 .934 r
.66144 .68401 .66909 .69195 .68495 .68766 .67737 .67968 Metetra
.645 .763 .934 r
.67737 .67968 .68495 .68766 .70091 .68335 .69339 .67532 Metetra
.645 .763 .934 r
.69339 .67532 .70091 .68335 .71695 .67901 .70949 .67094 Metetra
.645 .763 .934 r
.70949 .67094 .71695 .67901 .73309 .67465 .72569 .66653 Metetra
.645 .763 .934 r
.72569 .66653 .73309 .67465 .74931 .67026 .74198 .6621 Metetra
.645 .763 .934 r
.74198 .6621 .74931 .67026 .76563 .66585 .75837 .65764 Metetra
.645 .763 .934 r
.75837 .65764 .76563 .66585 .78204 .66141 .77485 .65315 Metetra
.645 .763 .934 r
.77485 .65315 .78204 .66141 .79855 .65695 .79142 .64864 Metetra
.645 .763 .934 r
.79142 .64864 .79855 .65695 .81515 .65246 .80809 .64411 Metetra
.645 .763 .934 r
.80809 .64411 .81515 .65246 .83185 .64795 .82486 .63955 Metetra
.645 .763 .934 r
.82486 .63955 .83185 .64795 .84864 .64341 .84172 .63496 Metetra
.645 .763 .934 r
.84172 .63496 .84864 .64341 .86553 .63884 .85868 .63034 Metetra
.645 .763 .934 r
.85868 .63034 .86553 .63884 .88252 .63425 .87574 .6257 Metetra
.645 .763 .934 r
.87574 .6257 .88252 .63425 .89961 .62963 .8929 .62103 Metetra
.645 .763 .934 r
.8929 .62103 .89961 .62963 .9168 .62498 .91017 .61633 Metetra
.645 .763 .934 r
.91017 .61633 .9168 .62498 .93409 .62031 .92753 .61161 Metetra
.645 .763 .934 r
.92753 .61161 .93409 .62031 .95148 .61561 .945 .60685 Metetra
.988 .732 .448 r
.945 .60685 .95148 .61561 .95378 .49342 .94752 .4843 Metetra
.645 .763 .934 r
.33774 .76254 .34658 .76969 .36077 .76583 .35198 .75864 Metetra
.645 .763 .934 r
.35198 .75864 .36077 .76583 .37504 .76194 .36629 .75472 Metetra
.645 .763 .934 r
.36629 .75472 .37504 .76194 .38939 .75804 .38069 .75078 Metetra
.645 .763 .934 r
.38069 .75078 .38939 .75804 .40381 .75412 .39516 .74681 Metetra
.645 .763 .934 r
.39516 .74681 .40381 .75412 .41831 .75017 .40971 .74283 Metetra
.645 .763 .934 r
.40971 .74283 .41831 .75017 .43289 .7462 .42434 .73882 Metetra
.645 .763 .934 r
.42434 .73882 .43289 .7462 .44755 .74221 .43905 .73479 Metetra
.645 .763 .934 r
.43905 .73479 .44755 .74221 .46229 .7382 .45384 .73074 Metetra
.645 .763 .934 r
.45384 .73074 .46229 .7382 .47711 .73417 .46871 .72667 Metetra
.645 .763 .934 r
.46871 .72667 .47711 .73417 .49201 .73012 .48366 .72257 Metetra
.645 .763 .934 r
.48366 .72257 .49201 .73012 .50699 .72604 .49869 .71846 Metetra
.645 .763 .934 r
.49869 .71846 .50699 .72604 .52205 .72194 .51381 .71431 Metetra
.645 .763 .934 r
.51381 .71431 .52205 .72194 .5372 .71782 .52901 .71015 Metetra
.645 .763 .934 r
.52901 .71015 .5372 .71782 .55243 .71368 .5443 .70596 Metetra
.645 .763 .934 r
.5443 .70596 .55243 .71368 .56774 .70951 .55967 .70175 Metetra
.645 .763 .934 r
.55967 .70175 .56774 .70951 .58314 .70532 .57512 .69752 Metetra
.645 .763 .934 r
.57512 .69752 .58314 .70532 .59863 .7011 .59067 .69326 Metetra
.645 .763 .934 r
.59067 .69326 .59863 .7011 .6142 .69687 .6063 .68898 Metetra
.645 .763 .934 r
.6063 .68898 .6142 .69687 .62986 .69261 .62202 .68468 Metetra
.645 .763 .934 r
.62202 .68468 .62986 .69261 .64561 .68832 .63782 .68035 Metetra
.645 .763 .934 r
.63782 .68035 .64561 .68832 .66144 .68401 .65372 .67599 Metetra
.645 .763 .934 r
.65372 .67599 .66144 .68401 .67737 .67968 .66971 .67161 Metetra
.645 .763 .934 r
.66971 .67161 .67737 .67968 .69339 .67532 .68579 .66721 Metetra
.645 .763 .934 r
.68579 .66721 .69339 .67532 .70949 .67094 .70196 .66278 Metetra
.645 .763 .934 r
.70196 .66278 .70949 .67094 .72569 .66653 .71822 .65833 Metetra
.645 .763 .934 r
.71822 .65833 .72569 .66653 .74198 .6621 .73458 .65385 Metetra
.645 .763 .934 r
.73458 .65385 .74198 .6621 .75837 .65764 .75103 .64934 Metetra
.645 .763 .934 r
.75103 .64934 .75837 .65764 .77485 .65315 .76757 .64481 Metetra
.645 .763 .934 r
.76757 .64481 .77485 .65315 .79142 .64864 .78421 .64025 Metetra
.645 .763 .934 r
.78421 .64025 .79142 .64864 .80809 .64411 .80095 .63567 Metetra
.645 .763 .934 r
.80095 .63567 .80809 .64411 .82486 .63955 .81779 .63105 Metetra
.645 .763 .934 r
.81779 .63105 .82486 .63955 .84172 .63496 .83472 .62642 Metetra
.645 .763 .934 r
.83472 .62642 .84172 .63496 .85868 .63034 .85175 .62175 Metetra
.645 .763 .934 r
.85175 .62175 .85868 .63034 .87574 .6257 .86889 .61706 Metetra
.645 .763 .934 r
.86889 .61706 .87574 .6257 .8929 .62103 .88612 .61234 Metetra
.645 .763 .934 r
.88612 .61234 .8929 .62103 .91017 .61633 .90346 .60759 Metetra
.645 .763 .934 r
.90346 .60759 .91017 .61633 .92753 .61161 .9209 .60281 Metetra
.645 .763 .934 r
.9209 .60281 .92753 .61161 .945 .60685 .93844 .59801 Metetra
.988 .732 .448 r
.93844 .59801 .945 .60685 .94752 .4843 .94118 .47507 Metetra
.645 .763 .934 r
.32881 .75533 .33774 .76254 .35198 .75864 .34309 .75139 Metetra
.645 .763 .934 r
.34309 .75139 .35198 .75864 .36629 .75472 .35746 .74743 Metetra
.645 .763 .934 r
.35746 .74743 .36629 .75472 .38069 .75078 .3719 .74344 Metetra
.645 .763 .934 r
.3719 .74344 .38069 .75078 .39516 .74681 .38642 .73944 Metetra
.645 .763 .934 r
.38642 .73944 .39516 .74681 .40971 .74283 .40102 .73541 Metetra
.645 .763 .934 r
.40102 .73541 .40971 .74283 .42434 .73882 .4157 .73137 Metetra
.645 .763 .934 r
.4157 .73137 .42434 .73882 .43905 .73479 .43046 .7273 Metetra
.645 .763 .934 r
.43046 .7273 .43905 .73479 .45384 .73074 .4453 .72321 Metetra
.645 .763 .934 r
.4453 .72321 .45384 .73074 .46871 .72667 .46022 .71909 Metetra
.645 .763 .934 r
.46022 .71909 .46871 .72667 .48366 .72257 .47522 .71495 Metetra
.645 .763 .934 r
.47522 .71495 .48366 .72257 .49869 .71846 .49031 .71079 Metetra
.645 .763 .934 r
.49031 .71079 .49869 .71846 .51381 .71431 .50548 .70661 Metetra
.645 .763 .934 r
.50548 .70661 .51381 .71431 .52901 .71015 .52074 .7024 Metetra
.645 .763 .934 r
.52074 .7024 .52901 .71015 .5443 .70596 .53608 .69817 Metetra
.645 .763 .934 r
.53608 .69817 .5443 .70596 .55967 .70175 .55151 .69392 Metetra
.645 .763 .934 r
.55151 .69392 .55967 .70175 .57512 .69752 .56702 .68964 Metetra
.645 .763 .934 r
.56702 .68964 .57512 .69752 .59067 .69326 .58262 .68534 Metetra
.645 .763 .934 r
.58262 .68534 .59067 .69326 .6063 .68898 .59831 .68101 Metetra
.645 .763 .934 r
.59831 .68101 .6063 .68898 .62202 .68468 .61409 .67666 Metetra
.645 .763 .934 r
.61409 .67666 .62202 .68468 .63782 .68035 .62996 .67229 Metetra
.645 .763 .934 r
.62996 .67229 .63782 .68035 .65372 .67599 .64592 .66789 Metetra
.645 .763 .934 r
.64592 .66789 .65372 .67599 .66971 .67161 .66197 .66346 Metetra
.645 .763 .934 r
.66197 .66346 .66971 .67161 .68579 .66721 .67811 .65901 Metetra
.645 .763 .934 r
.67811 .65901 .68579 .66721 .70196 .66278 .69434 .65454 Metetra
.645 .763 .934 r
.69434 .65454 .70196 .66278 .71822 .65833 .71067 .65004 Metetra
.645 .763 .934 r
.71067 .65004 .71822 .65833 .73458 .65385 .72709 .64551 Metetra
.645 .763 .934 r
.72709 .64551 .73458 .65385 .75103 .64934 .74361 .64095 Metetra
.645 .763 .934 r
.74361 .64095 .75103 .64934 .76757 .64481 .76022 .63637 Metetra
.645 .763 .934 r
.76022 .63637 .76757 .64481 .78421 .64025 .77693 .63177 Metetra
.645 .763 .934 r
.77693 .63177 .78421 .64025 .80095 .63567 .79373 .62713 Metetra
.645 .763 .934 r
.79373 .62713 .80095 .63567 .81779 .63105 .81064 .62247 Metetra
.645 .763 .934 r
.81064 .62247 .81779 .63105 .83472 .62642 .82765 .61778 Metetra
.645 .763 .934 r
.82765 .61778 .83472 .62642 .85175 .62175 .84475 .61306 Metetra
.645 .763 .934 r
.84475 .61306 .85175 .62175 .86889 .61706 .86196 .60832 Metetra
.645 .763 .934 r
.86196 .60832 .86889 .61706 .88612 .61234 .87927 .60355 Metetra
.645 .763 .934 r
.87927 .60355 .88612 .61234 .90346 .60759 .89668 .59875 Metetra
.645 .763 .934 r
.89668 .59875 .90346 .60759 .9209 .60281 .91419 .59392 Metetra
.645 .763 .934 r
.91419 .59392 .9209 .60281 .93844 .59801 .93181 .58906 Metetra
.988 .732 .448 r
.93181 .58906 .93844 .59801 .94118 .47507 .93478 .46576 Metetra
.645 .763 .934 r
.31978 .74804 .32881 .75533 .34309 .75139 .33412 .74406 Metetra
.645 .763 .934 r
.33412 .74406 .34309 .75139 .35746 .74743 .34853 .74006 Metetra
.645 .763 .934 r
.34853 .74006 .35746 .74743 .3719 .74344 .36302 .73604 Metetra
.645 .763 .934 r
.36302 .73604 .3719 .74344 .38642 .73944 .37759 .73199 Metetra
.645 .763 .934 r
.37759 .73199 .38642 .73944 .40102 .73541 .39224 .72792 Metetra
.645 .763 .934 r
.39224 .72792 .40102 .73541 .4157 .73137 .40697 .72384 Metetra
.645 .763 .934 r
.40697 .72384 .4157 .73137 .43046 .7273 .42178 .71973 Metetra
.645 .763 .934 r
.42178 .71973 .43046 .7273 .4453 .72321 .43667 .71559 Metetra
.645 .763 .934 r
.43667 .71559 .4453 .72321 .46022 .71909 .45165 .71143 Metetra
.645 .763 .934 r
.45165 .71143 .46022 .71909 .47522 .71495 .4667 .70726 Metetra
.645 .763 .934 r
.4667 .70726 .47522 .71495 .49031 .71079 .48185 .70305 Metetra
.645 .763 .934 r
.48185 .70305 .49031 .71079 .50548 .70661 .49707 .69883 Metetra
.645 .763 .934 r
.49707 .69883 .50548 .70661 .52074 .7024 .51238 .69458 Metetra
.645 .763 .934 r
.51238 .69458 .52074 .7024 .53608 .69817 .52778 .6903 Metetra
.645 .763 .934 r
.52778 .6903 .53608 .69817 .55151 .69392 .54326 .686 Metetra
.645 .763 .934 r
.54326 .686 .55151 .69392 .56702 .68964 .55884 .68168 Metetra
.645 .763 .934 r
.55884 .68168 .56702 .68964 .58262 .68534 .5745 .67734 Metetra
.645 .763 .934 r
.5745 .67734 .58262 .68534 .59831 .68101 .59024 .67296 Metetra
.645 .763 .934 r
.59024 .67296 .59831 .68101 .61409 .67666 .60608 .66857 Metetra
.645 .763 .934 r
.60608 .66857 .61409 .67666 .62996 .67229 .62201 .66415 Metetra
.645 .763 .934 r
.62201 .66415 .62996 .67229 .64592 .66789 .63803 .6597 Metetra
.645 .763 .934 r
.63803 .6597 .64592 .66789 .66197 .66346 .65414 .65523 Metetra
.645 .763 .934 r
.65414 .65523 .66197 .66346 .67811 .65901 .67034 .65073 Metetra
.645 .763 .934 r
.67034 .65073 .67811 .65901 .69434 .65454 .68664 .64621 Metetra
.645 .763 .934 r
.68664 .64621 .69434 .65454 .71067 .65004 .70303 .64166 Metetra
.645 .763 .934 r
.70303 .64166 .71067 .65004 .72709 .64551 .71952 .63708 Metetra
.645 .763 .934 r
.71952 .63708 .72709 .64551 .74361 .64095 .73611 .63248 Metetra
.645 .763 .934 r
.73611 .63248 .74361 .64095 .76022 .63637 .75279 .62785 Metetra
.645 .763 .934 r
.75279 .62785 .76022 .63637 .77693 .63177 .76956 .62319 Metetra
.645 .763 .934 r
.76956 .62319 .77693 .63177 .79373 .62713 .78644 .6185 Metetra
.645 .763 .934 r
.78644 .6185 .79373 .62713 .81064 .62247 .80342 .61379 Metetra
.645 .763 .934 r
.80342 .61379 .81064 .62247 .82765 .61778 .82049 .60905 Metetra
.645 .763 .934 r
.82049 .60905 .82765 .61778 .84475 .61306 .83767 .60428 Metetra
.645 .763 .934 r
.83767 .60428 .84475 .61306 .86196 .60832 .85495 .59949 Metetra
.645 .763 .934 r
.85495 .59949 .86196 .60832 .87927 .60355 .87233 .59466 Metetra
.645 .763 .934 r
.87233 .59466 .87927 .60355 .89668 .59875 .88982 .58981 Metetra
.645 .763 .934 r
.88982 .58981 .89668 .59875 .91419 .59392 .90741 .58493 Metetra
.645 .763 .934 r
.90741 .58493 .91419 .59392 .93181 .58906 .92511 .58001 Metetra
.988 .732 .448 r
.92511 .58001 .93181 .58906 .93478 .46576 .92831 .45634 Metetra
.645 .763 .934 r
.31067 .74067 .31978 .74804 .33412 .74406 .32505 .73666 Metetra
.645 .763 .934 r
.32505 .73666 .33412 .74406 .34853 .74006 .33951 .73261 Metetra
.645 .763 .934 r
.33951 .73261 .34853 .74006 .36302 .73604 .35405 .72855 Metetra
.645 .763 .934 r
.35405 .72855 .36302 .73604 .37759 .73199 .36867 .72447 Metetra
.645 .763 .934 r
.36867 .72447 .37759 .73199 .39224 .72792 .38337 .72036 Metetra
.645 .763 .934 r
.38337 .72036 .39224 .72792 .40697 .72384 .39815 .71623 Metetra
.645 .763 .934 r
.39815 .71623 .40697 .72384 .42178 .71973 .41301 .71208 Metetra
.645 .763 .934 r
.41301 .71208 .42178 .71973 .43667 .71559 .42796 .7079 Metetra
.645 .763 .934 r
.42796 .7079 .43667 .71559 .45165 .71143 .44298 .7037 Metetra
.645 .763 .934 r
.44298 .7037 .45165 .71143 .4667 .70726 .4581 .69948 Metetra
.645 .763 .934 r
.4581 .69948 .4667 .70726 .48185 .70305 .47329 .69523 Metetra
.645 .763 .934 r
.47329 .69523 .48185 .70305 .49707 .69883 .48857 .69096 Metetra
.645 .763 .934 r
.48857 .69096 .49707 .69883 .51238 .69458 .50394 .68667 Metetra
.645 .763 .934 r
.50394 .68667 .51238 .69458 .52778 .6903 .51939 .68235 Metetra
.645 .763 .934 r
.51939 .68235 .52778 .6903 .54326 .686 .53493 .67801 Metetra
.645 .763 .934 r
.53493 .67801 .54326 .686 .55884 .68168 .55056 .67364 Metetra
.645 .763 .934 r
.55056 .67364 .55884 .68168 .5745 .67734 .56628 .66925 Metetra
.645 .763 .934 r
.56628 .66925 .5745 .67734 .59024 .67296 .58209 .66483 Metetra
.645 .763 .934 r
.58209 .66483 .59024 .67296 .60608 .66857 .59799 .66039 Metetra
.645 .763 .934 r
.59799 .66039 .60608 .66857 .62201 .66415 .61398 .65592 Metetra
.645 .763 .934 r
.61398 .65592 .62201 .66415 .63803 .6597 .63006 .65142 Metetra
.645 .763 .934 r
.63006 .65142 .63803 .6597 .65414 .65523 .64623 .6469 Metetra
.645 .763 .934 r
.64623 .6469 .65414 .65523 .67034 .65073 .6625 .64236 Metetra
.645 .763 .934 r
.6625 .64236 .67034 .65073 .68664 .64621 .67886 .63778 Metetra
.645 .763 .934 r
.67886 .63778 .68664 .64621 .70303 .64166 .69532 .63319 Metetra
.645 .763 .934 r
.69532 .63319 .70303 .64166 .71952 .63708 .71187 .62856 Metetra
.645 .763 .934 r
.71187 .62856 .71952 .63708 .73611 .63248 .72852 .62391 Metetra
.645 .763 .934 r
.72852 .62391 .73611 .63248 .75279 .62785 .74527 .61923 Metetra
.645 .763 .934 r
.74527 .61923 .75279 .62785 .76956 .62319 .76212 .61452 Metetra
.645 .763 .934 r
.76212 .61452 .76956 .62319 .78644 .6185 .77907 .60978 Metetra
.645 .763 .934 r
.77907 .60978 .78644 .6185 .80342 .61379 .79611 .60502 Metetra
.645 .763 .934 r
.79611 .60502 .80342 .61379 .82049 .60905 .81326 .60023 Metetra
.645 .763 .934 r
.81326 .60023 .82049 .60905 .83767 .60428 .83051 .59541 Metetra
.645 .763 .934 r
.83051 .59541 .83767 .60428 .85495 .59949 .84787 .59056 Metetra
.645 .763 .934 r
.84787 .59056 .85495 .59949 .87233 .59466 .86532 .58568 Metetra
.645 .763 .934 r
.86532 .58568 .87233 .59466 .88982 .58981 .88289 .58077 Metetra
.645 .763 .934 r
.88289 .58077 .88982 .58981 .90741 .58493 .90056 .57583 Metetra
.645 .763 .934 r
.90056 .57583 .90741 .58493 .92511 .58001 .91833 .57087 Metetra
.988 .732 .448 r
.91833 .57087 .92511 .58001 .92831 .45634 .92176 .44682 Metetra
.645 .763 .934 r
.30147 .73324 .31067 .74067 .32505 .73666 .3159 .72918 Metetra
.645 .763 .934 r
.3159 .72918 .32505 .73666 .33951 .73261 .33041 .7251 Metetra
.645 .763 .934 r
.33041 .7251 .33951 .73261 .35405 .72855 .34499 .72099 Metetra
.645 .763 .934 r
.34499 .72099 .35405 .72855 .36867 .72447 .35966 .71687 Metetra
.645 .763 .934 r
.35966 .71687 .36867 .72447 .38337 .72036 .37441 .71272 Metetra
.645 .763 .934 r
.37441 .71272 .38337 .72036 .39815 .71623 .38924 .70854 Metetra
.645 .763 .934 r
.38924 .70854 .39815 .71623 .41301 .71208 .40416 .70435 Metetra
.645 .763 .934 r
.40416 .70435 .41301 .71208 .42796 .7079 .41915 .70013 Metetra
.645 .763 .934 r
.41915 .70013 .42796 .7079 .44298 .7037 .43423 .69589 Metetra
.645 .763 .934 r
.43423 .69589 .44298 .7037 .4581 .69948 .4494 .69162 Metetra
.645 .763 .934 r
.4494 .69162 .4581 .69948 .47329 .69523 .46465 .68733 Metetra
.645 .763 .934 r
.46465 .68733 .47329 .69523 .48857 .69096 .47998 .68301 Metetra
.645 .763 .934 r
.47998 .68301 .48857 .69096 .50394 .68667 .49541 .67868 Metetra
.645 .763 .934 r
.49541 .67868 .50394 .68667 .51939 .68235 .51092 .67431 Metetra
.645 .763 .934 r
.51092 .67431 .51939 .68235 .53493 .67801 .52652 .66992 Metetra
.645 .763 .934 r
.52652 .66992 .53493 .67801 .55056 .67364 .5422 .66551 Metetra
.645 .763 .934 r
.5422 .66551 .55056 .67364 .56628 .66925 .55798 .66107 Metetra
.645 .763 .934 r
.55798 .66107 .56628 .66925 .58209 .66483 .57385 .65661 Metetra
.645 .763 .934 r
.57385 .65661 .58209 .66483 .59799 .66039 .58981 .65212 Metetra
.645 .763 .934 r
.58981 .65212 .59799 .66039 .61398 .65592 .60586 .6476 Metetra
.645 .763 .934 r
.60586 .6476 .61398 .65592 .63006 .65142 .622 .64306 Metetra
.645 .763 .934 r
.622 .64306 .63006 .65142 .64623 .6469 .63824 .63849 Metetra
.645 .763 .934 r
.63824 .63849 .64623 .6469 .6625 .64236 .65457 .6339 Metetra
.645 .763 .934 r
.65457 .6339 .6625 .64236 .67886 .63778 .671 .62927 Metetra
.645 .763 .934 r
.671 .62927 .67886 .63778 .69532 .63319 .68752 .62462 Metetra
.645 .763 .934 r
.68752 .62462 .69532 .63319 .71187 .62856 .70414 .61995 Metetra
.645 .763 .934 r
.70414 .61995 .71187 .62856 .72852 .62391 .72086 .61524 Metetra
.645 .763 .934 r
.72086 .61524 .72852 .62391 .74527 .61923 .73768 .61051 Metetra
.645 .763 .934 r
.73768 .61051 .74527 .61923 .76212 .61452 .7546 .60575 Metetra
.645 .763 .934 r
.7546 .60575 .76212 .61452 .77907 .60978 .77161 .60097 Metetra
.645 .763 .934 r
.77161 .60097 .77907 .60978 .79611 .60502 .78873 .59615 Metetra
.645 .763 .934 r
.78873 .59615 .79611 .60502 .81326 .60023 .80595 .59131 Metetra
.645 .763 .934 r
.80595 .59131 .81326 .60023 .83051 .59541 .82328 .58643 Metetra
.645 .763 .934 r
.82328 .58643 .83051 .59541 .84787 .59056 .8407 .58153 Metetra
.645 .763 .934 r
.8407 .58153 .84787 .59056 .86532 .58568 .85824 .57659 Metetra
.645 .763 .934 r
.85824 .57659 .86532 .58568 .88289 .58077 .87588 .57163 Metetra
.645 .763 .934 r
.87588 .57163 .88289 .58077 .90056 .57583 .89362 .56664 Metetra
.645 .763 .934 r
.89362 .56664 .90056 .57583 .91833 .57087 .91148 .56162 Metetra
.988 .732 .448 r
.91148 .56162 .91833 .57087 .92176 .44682 .91515 .43719 Metetra
.645 .763 .934 r
.29218 .72573 .30147 .73324 .3159 .72918 .30665 .72163 Metetra
.645 .763 .934 r
.30665 .72163 .3159 .72918 .33041 .7251 .32121 .7175 Metetra
.645 .763 .934 r
.32121 .7175 .33041 .7251 .34499 .72099 .33584 .71336 Metetra
.645 .763 .934 r
.33584 .71336 .34499 .72099 .35966 .71687 .35056 .70919 Metetra
.645 .763 .934 r
.35056 .70919 .35966 .71687 .37441 .71272 .36536 .70499 Metetra
.645 .763 .934 r
.36536 .70499 .37441 .71272 .38924 .70854 .38024 .70078 Metetra
.645 .763 .934 r
.38024 .70078 .38924 .70854 .40416 .70435 .39521 .69654 Metetra
.645 .763 .934 r
.39521 .69654 .40416 .70435 .41915 .70013 .41026 .69228 Metetra
.645 .763 .934 r
.41026 .69228 .41915 .70013 .43423 .69589 .42539 .68799 Metetra
.645 .763 .934 r
.42539 .68799 .43423 .69589 .4494 .69162 .44061 .68368 Metetra
.645 .763 .934 r
.44061 .68368 .4494 .69162 .46465 .68733 .45591 .67934 Metetra
.645 .763 .934 r
.45591 .67934 .46465 .68733 .47998 .68301 .47131 .67498 Metetra
.645 .763 .934 r
.47131 .67498 .47998 .68301 .49541 .67868 .48679 .6706 Metetra
.645 .763 .934 r
.48679 .6706 .49541 .67868 .51092 .67431 .50235 .66619 Metetra
.645 .763 .934 r
.50235 .66619 .51092 .67431 .52652 .66992 .51801 .66175 Metetra
.645 .763 .934 r
.51801 .66175 .52652 .66992 .5422 .66551 .53376 .65729 Metetra
.645 .763 .934 r
.53376 .65729 .5422 .66551 .55798 .66107 .54959 .65281 Metetra
.645 .763 .934 r
.54959 .65281 .55798 .66107 .57385 .65661 .56552 .6483 Metetra
.645 .763 .934 r
.56552 .6483 .57385 .65661 .58981 .65212 .58154 .64376 Metetra
.645 .763 .934 r
.58154 .64376 .58981 .65212 .60586 .6476 .59765 .63919 Metetra
.645 .763 .934 r
.59765 .63919 .60586 .6476 .622 .64306 .61386 .6346 Metetra
.645 .763 .934 r
.61386 .6346 .622 .64306 .63824 .63849 .63016 .62999 Metetra
.645 .763 .934 r
.63016 .62999 .63824 .63849 .65457 .6339 .64656 .62534 Metetra
.645 .763 .934 r
.64656 .62534 .65457 .6339 .671 .62927 .66305 .62067 Metetra
.645 .763 .934 r
.66305 .62067 .671 .62927 .68752 .62462 .67964 .61597 Metetra
.645 .763 .934 r
.67964 .61597 .68752 .62462 .70414 .61995 .69633 .61124 Metetra
.645 .763 .934 r
.69633 .61124 .70414 .61995 .72086 .61524 .71311 .60649 Metetra
.645 .763 .934 r
.71311 .60649 .72086 .61524 .73768 .61051 .73 .6017 Metetra
.645 .763 .934 r
.73 .6017 .73768 .61051 .7546 .60575 .74699 .59689 Metetra
.645 .763 .934 r
.74699 .59689 .7546 .60575 .77161 .60097 .76408 .59205 Metetra
.645 .763 .934 r
.76408 .59205 .77161 .60097 .78873 .59615 .78127 .58718 Metetra
.645 .763 .934 r
.78127 .58718 .78873 .59615 .80595 .59131 .79856 .58228 Metetra
.645 .763 .934 r
.79856 .58228 .80595 .59131 .82328 .58643 .81596 .57736 Metetra
.645 .763 .934 r
.81596 .57736 .82328 .58643 .8407 .58153 .83346 .5724 Metetra
.645 .763 .934 r
.83346 .5724 .8407 .58153 .85824 .57659 .85107 .56741 Metetra
.645 .763 .934 r
.85107 .56741 .85824 .57659 .87588 .57163 .86879 .56239 Metetra
.645 .763 .934 r
.86879 .56239 .87588 .57163 .89362 .56664 .88661 .55734 Metetra
.645 .763 .934 r
.88661 .55734 .89362 .56664 .91148 .56162 .90455 .55226 Metetra
.988 .732 .448 r
.90455 .55226 .91148 .56162 .91515 .43719 .90846 .42746 Metetra
.645 .763 .934 r
.28278 .71814 .29218 .72573 .30665 .72163 .29731 .714 Metetra
.645 .763 .934 r
.29731 .714 .30665 .72163 .32121 .7175 .31191 .70983 Metetra
.645 .763 .934 r
.31191 .70983 .32121 .7175 .33584 .71336 .3266 .70564 Metetra
.645 .763 .934 r
.3266 .70564 .33584 .71336 .35056 .70919 .34137 .70143 Metetra
.645 .763 .934 r
.34137 .70143 .35056 .70919 .36536 .70499 .35622 .69719 Metetra
.645 .763 .934 r
.35622 .69719 .36536 .70499 .38024 .70078 .37115 .69293 Metetra
.645 .763 .934 r
.37115 .69293 .38024 .70078 .39521 .69654 .38617 .68865 Metetra
.645 .763 .934 r
.38617 .68865 .39521 .69654 .41026 .69228 .40127 .68434 Metetra
.645 .763 .934 r
.40127 .68434 .41026 .69228 .42539 .68799 .41646 .68001 Metetra
.645 .763 .934 r
.41646 .68001 .42539 .68799 .44061 .68368 .43173 .67566 Metetra
.645 .763 .934 r
.43173 .67566 .44061 .68368 .45591 .67934 .44709 .67128 Metetra
.645 .763 .934 r
.44709 .67128 .45591 .67934 .47131 .67498 .46254 .66687 Metetra
.645 .763 .934 r
.46254 .66687 .47131 .67498 .48679 .6706 .47807 .66244 Metetra
.645 .763 .934 r
.47807 .66244 .48679 .6706 .50235 .66619 .4937 .65798 Metetra
.645 .763 .934 r
.4937 .65798 .50235 .66619 .51801 .66175 .50941 .6535 Metetra
.645 .763 .934 r
.50941 .6535 .51801 .66175 .53376 .65729 .52522 .64899 Metetra
.645 .763 .934 r
.52522 .64899 .53376 .65729 .54959 .65281 .54111 .64446 Metetra
.645 .763 .934 r
.54111 .64446 .54959 .65281 .56552 .6483 .5571 .6399 Metetra
.645 .763 .934 r
.5571 .6399 .56552 .6483 .58154 .64376 .57318 .63531 Metetra
.645 .763 .934 r
.57318 .63531 .58154 .64376 .59765 .63919 .58936 .6307 Metetra
.645 .763 .934 r
.58936 .6307 .59765 .63919 .61386 .6346 .60563 .62606 Metetra
.645 .763 .934 r
.60563 .62606 .61386 .6346 .63016 .62999 .62199 .62139 Metetra
.645 .763 .934 r
.62199 .62139 .63016 .62999 .64656 .62534 .63846 .6167 Metetra
.645 .763 .934 r
.63846 .6167 .64656 .62534 .66305 .62067 .65501 .61197 Metetra
.645 .763 .934 r
.65501 .61197 .66305 .62067 .67964 .61597 .67167 .60722 Metetra
.645 .763 .934 r
.67167 .60722 .67964 .61597 .69633 .61124 .68843 .60244 Metetra
.645 .763 .934 r
.68843 .60244 .69633 .61124 .71311 .60649 .70528 .59764 Metetra
.645 .763 .934 r
.70528 .59764 .71311 .60649 .73 .6017 .72224 .5928 Metetra
.645 .763 .934 r
.72224 .5928 .73 .6017 .74699 .59689 .7393 .58793 Metetra
.645 .763 .934 r
.7393 .58793 .74699 .59689 .76408 .59205 .75646 .58304 Metetra
.645 .763 .934 r
.75646 .58304 .76408 .59205 .78127 .58718 .77372 .57812 Metetra
.645 .763 .934 r
.77372 .57812 .78127 .58718 .79856 .58228 .79109 .57316 Metetra
.645 .763 .934 r
.79109 .57316 .79856 .58228 .81596 .57736 .80856 .56818 Metetra
.645 .763 .934 r
.80856 .56818 .81596 .57736 .83346 .5724 .82614 .56316 Metetra
.645 .763 .934 r
.82614 .56316 .83346 .5724 .85107 .56741 .84383 .55812 Metetra
.645 .763 .934 r
.84383 .55812 .85107 .56741 .86879 .56239 .86162 .55305 Metetra
.645 .763 .934 r
.86162 .55305 .86879 .56239 .88661 .55734 .87952 .54794 Metetra
.645 .763 .934 r
.87952 .54794 .88661 .55734 .90455 .55226 .89754 .5428 Metetra
.988 .732 .448 r
.89754 .5428 .90455 .55226 .90846 .42746 .90171 .41762 Metetra
.645 .763 .934 r
.2733 .71047 .28278 .71814 .29731 .714 .28787 .70629 Metetra
.645 .763 .934 r
.28787 .70629 .29731 .714 .31191 .70983 .30252 .70208 Metetra
.645 .763 .934 r
.30252 .70208 .31191 .70983 .3266 .70564 .31726 .69785 Metetra
.645 .763 .934 r
.31726 .69785 .3266 .70564 .34137 .70143 .33208 .69359 Metetra
.645 .763 .934 r
.33208 .69359 .34137 .70143 .35622 .69719 .34698 .68931 Metetra
.645 .763 .934 r
.34698 .68931 .35622 .69719 .37115 .69293 .36196 .68501 Metetra
.645 .763 .934 r
.36196 .68501 .37115 .69293 .38617 .68865 .37703 .68068 Metetra
.645 .763 .934 r
.37703 .68068 .38617 .68865 .40127 .68434 .39219 .67633 Metetra
.645 .763 .934 r
.39219 .67633 .40127 .68434 .41646 .68001 .40743 .67195 Metetra
.645 .763 .934 r
.40743 .67195 .41646 .68001 .43173 .67566 .42276 .66755 Metetra
.645 .763 .934 r
.42276 .66755 .43173 .67566 .44709 .67128 .43817 .66312 Metetra
.645 .763 .934 r
.43817 .66312 .44709 .67128 .46254 .66687 .45367 .65867 Metetra
.645 .763 .934 r
.45367 .65867 .46254 .66687 .47807 .66244 .46927 .65419 Metetra
.645 .763 .934 r
.46927 .65419 .47807 .66244 .4937 .65798 .48495 .64969 Metetra
.645 .763 .934 r
.48495 .64969 .4937 .65798 .50941 .6535 .50072 .64516 Metetra
.645 .763 .934 r
.50072 .64516 .50941 .6535 .52522 .64899 .51659 .6406 Metetra
.645 .763 .934 r
.51659 .6406 .52522 .64899 .54111 .64446 .53254 .63602 Metetra
.645 .763 .934 r
.53254 .63602 .54111 .64446 .5571 .6399 .54859 .63141 Metetra
.645 .763 .934 r
.54859 .63141 .5571 .6399 .57318 .63531 .56474 .62677 Metetra
.645 .763 .934 r
.56474 .62677 .57318 .63531 .58936 .6307 .58098 .62211 Metetra
.645 .763 .934 r
.58098 .62211 .58936 .6307 .60563 .62606 .59731 .61742 Metetra
.645 .763 .934 r
.59731 .61742 .60563 .62606 .62199 .62139 .61374 .6127 Metetra
.645 .763 .934 r
.61374 .6127 .62199 .62139 .63846 .6167 .63027 .60795 Metetra
.645 .763 .934 r
.63027 .60795 .63846 .6167 .65501 .61197 .64689 .60318 Metetra
.645 .763 .934 r
.64689 .60318 .65501 .61197 .67167 .60722 .66362 .59838 Metetra
.645 .763 .934 r
.66362 .59838 .67167 .60722 .68843 .60244 .68044 .59354 Metetra
.645 .763 .934 r
.68044 .59354 .68843 .60244 .70528 .59764 .69736 .58868 Metetra
.645 .763 .934 r
.69736 .58868 .70528 .59764 .72224 .5928 .71439 .58379 Metetra
.645 .763 .934 r
.71439 .58379 .72224 .5928 .7393 .58793 .73152 .57888 Metetra
.645 .763 .934 r
.73152 .57888 .7393 .58793 .75646 .58304 .74875 .57393 Metetra
.645 .763 .934 r
.74875 .57393 .75646 .58304 .77372 .57812 .76609 .56895 Metetra
.645 .763 .934 r
.76609 .56895 .77372 .57812 .79109 .57316 .78353 .56394 Metetra
.645 .763 .934 r
.78353 .56394 .79109 .57316 .80856 .56818 .80108 .5589 Metetra
.645 .763 .934 r
.80108 .5589 .80856 .56818 .82614 .56316 .81873 .55383 Metetra
.645 .763 .934 r
.81873 .55383 .82614 .56316 .84383 .55812 .8365 .54873 Metetra
.645 .763 .934 r
.8365 .54873 .84383 .55812 .86162 .55305 .85437 .54359 Metetra
.645 .763 .934 r
.85437 .54359 .86162 .55305 .87952 .54794 .87235 .53843 Metetra
.645 .763 .934 r
.87235 .53843 .87952 .54794 .89754 .5428 .89045 .53323 Metetra
.988 .732 .448 r
.89045 .53323 .89754 .5428 .90171 .41762 .89487 .40768 Metetra
.645 .763 .934 r
.26371 .70273 .2733 .71047 .28787 .70629 .27834 .6985 Metetra
.645 .763 .934 r
.27834 .6985 .28787 .70629 .30252 .70208 .29304 .69425 Metetra
.645 .763 .934 r
.29304 .69425 .30252 .70208 .31726 .69785 .30782 .68997 Metetra
.645 .763 .934 r
.30782 .68997 .31726 .69785 .33208 .69359 .32269 .68567 Metetra
.645 .763 .934 r
.32269 .68567 .33208 .69359 .34698 .68931 .33764 .68135 Metetra
.645 .763 .934 r
.33764 .68135 .34698 .68931 .36196 .68501 .35268 .677 Metetra
.645 .763 .934 r
.35268 .677 .36196 .68501 .37703 .68068 .3678 .67263 Metetra
.645 .763 .934 r
.3678 .67263 .37703 .68068 .39219 .67633 .38301 .66823 Metetra
.645 .763 .934 r
.38301 .66823 .39219 .67633 .40743 .67195 .39831 .6638 Metetra
.645 .763 .934 r
.39831 .6638 .40743 .67195 .42276 .66755 .41369 .65936 Metetra
.645 .763 .934 r
.41369 .65936 .42276 .66755 .43817 .66312 .42916 .65488 Metetra
.645 .763 .934 r
.42916 .65488 .43817 .66312 .45367 .65867 .44472 .65038 Metetra
.645 .763 .934 r
.44472 .65038 .45367 .65867 .46927 .65419 .46037 .64586 Metetra
.645 .763 .934 r
.46037 .64586 .46927 .65419 .48495 .64969 .47611 .6413 Metetra
.645 .763 .934 r
.47611 .6413 .48495 .64969 .50072 .64516 .49194 .63673 Metetra
.645 .763 .934 r
.49194 .63673 .50072 .64516 .51659 .6406 .50787 .63212 Metetra
.645 .763 .934 r
.50787 .63212 .51659 .6406 .53254 .63602 .52388 .62749 Metetra
.645 .763 .934 r
.52388 .62749 .53254 .63602 .54859 .63141 .53999 .62283 Metetra
.645 .763 .934 r
.53999 .62283 .54859 .63141 .56474 .62677 .5562 .61814 Metetra
.645 .763 .934 r
.5562 .61814 .56474 .62677 .58098 .62211 .5725 .61343 Metetra
.645 .763 .934 r
.5725 .61343 .58098 .62211 .59731 .61742 .5889 .60869 Metetra
.645 .763 .934 r
.5889 .60869 .59731 .61742 .61374 .6127 .60539 .60392 Metetra
.645 .763 .934 r
.60539 .60392 .61374 .6127 .63027 .60795 .62199 .59912 Metetra
.645 .763 .934 r
.62199 .59912 .63027 .60795 .64689 .60318 .63868 .59429 Metetra
.645 .763 .934 r
.63868 .59429 .64689 .60318 .66362 .59838 .65547 .58943 Metetra
.645 .763 .934 r
.65547 .58943 .66362 .59838 .68044 .59354 .67236 .58455 Metetra
.645 .763 .934 r
.67236 .58455 .68044 .59354 .69736 .58868 .68936 .57963 Metetra
.645 .763 .934 r
.68936 .57963 .69736 .58868 .71439 .58379 .70645 .57469 Metetra
.645 .763 .934 r
.70645 .57469 .71439 .58379 .73152 .57888 .72365 .56972 Metetra
.645 .763 .934 r
.72365 .56972 .73152 .57888 .74875 .57393 .74096 .56471 Metetra
.645 .763 .934 r
.74096 .56471 .74875 .57393 .76609 .56895 .75837 .55968 Metetra
.645 .763 .934 r
.75837 .55968 .76609 .56895 .78353 .56394 .77589 .55461 Metetra
.645 .763 .934 r
.77589 .55461 .78353 .56394 .80108 .5589 .79351 .54951 Metetra
.645 .763 .934 r
.79351 .54951 .80108 .5589 .81873 .55383 .81124 .54439 Metetra
.645 .763 .934 r
.81124 .54439 .81873 .55383 .8365 .54873 .82908 .53923 Metetra
.645 .763 .934 r
.82908 .53923 .8365 .54873 .85437 .54359 .84704 .53403 Metetra
.645 .763 .934 r
.84704 .53403 .85437 .54359 .87235 .53843 .8651 .52881 Metetra
.645 .763 .934 r
.8651 .52881 .87235 .53843 .89045 .53323 .88328 .52355 Metetra
.988 .732 .448 r
.88328 .52355 .89045 .53323 .89487 .40768 .88796 .39762 Metetra
.645 .763 .934 r
.25403 .6949 .26371 .70273 .27834 .6985 .2687 .69063 Metetra
.645 .763 .934 r
.2687 .69063 .27834 .6985 .29304 .69425 .28345 .68634 Metetra
.645 .763 .934 r
.28345 .68634 .29304 .69425 .30782 .68997 .29829 .68202 Metetra
.645 .763 .934 r
.29829 .68202 .30782 .68997 .32269 .68567 .31321 .67767 Metetra
.645 .763 .934 r
.31321 .67767 .32269 .68567 .33764 .68135 .32821 .6733 Metetra
.645 .763 .934 r
.32821 .6733 .33764 .68135 .35268 .677 .3433 .66891 Metetra
.645 .763 .934 r
.3433 .66891 .35268 .677 .3678 .67263 .35847 .66449 Metetra
.645 .763 .934 r
.35847 .66449 .3678 .67263 .38301 .66823 .37374 .66004 Metetra
.645 .763 .934 r
.37374 .66004 .38301 .66823 .39831 .6638 .38909 .65557 Metetra
.645 .763 .934 r
.38909 .65557 .39831 .6638 .41369 .65936 .40452 .65108 Metetra
.645 .763 .934 r
.40452 .65108 .41369 .65936 .42916 .65488 .42005 .64655 Metetra
.645 .763 .934 r
.42005 .64655 .42916 .65488 .44472 .65038 .43567 .64201 Metetra
.645 .763 .934 r
.43567 .64201 .44472 .65038 .46037 .64586 .45137 .63743 Metetra
.645 .763 .934 r
.45137 .63743 .46037 .64586 .47611 .6413 .46717 .63283 Metetra
.645 .763 .934 r
.46717 .63283 .47611 .6413 .49194 .63673 .48306 .6282 Metetra
.645 .763 .934 r
.48306 .6282 .49194 .63673 .50787 .63212 .49905 .62355 Metetra
.645 .763 .934 r
.49905 .62355 .50787 .63212 .52388 .62749 .51513 .61887 Metetra
.645 .763 .934 r
.51513 .61887 .52388 .62749 .53999 .62283 .5313 .61416 Metetra
.645 .763 .934 r
.5313 .61416 .53999 .62283 .5562 .61814 .54757 .60942 Metetra
.645 .763 .934 r
.54757 .60942 .5562 .61814 .5725 .61343 .56393 .60465 Metetra
.645 .763 .934 r
.56393 .60465 .5725 .61343 .5889 .60869 .5804 .59986 Metetra
.645 .763 .934 r
.5804 .59986 .5889 .60869 .60539 .60392 .59696 .59503 Metetra
.645 .763 .934 r
.59696 .59503 .60539 .60392 .62199 .59912 .61362 .59018 Metetra
.645 .763 .934 r
.61362 .59018 .62199 .59912 .63868 .59429 .63037 .5853 Metetra
.645 .763 .934 r
.63037 .5853 .63868 .59429 .65547 .58943 .64723 .58039 Metetra
.645 .763 .934 r
.64723 .58039 .65547 .58943 .67236 .58455 .6642 .57545 Metetra
.645 .763 .934 r
.6642 .57545 .67236 .58455 .68936 .57963 .68126 .57048 Metetra
.645 .763 .934 r
.68126 .57048 .68936 .57963 .70645 .57469 .69843 .56548 Metetra
.645 .763 .934 r
.69843 .56548 .70645 .57469 .72365 .56972 .7157 .56045 Metetra
.645 .763 .934 r
.7157 .56045 .72365 .56972 .74096 .56471 .73308 .55539 Metetra
.645 .763 .934 r
.73308 .55539 .74096 .56471 .75837 .55968 .75056 .5503 Metetra
.645 .763 .934 r
.75056 .5503 .75837 .55968 .77589 .55461 .76816 .54518 Metetra
.645 .763 .934 r
.76816 .54518 .77589 .55461 .79351 .54951 .78586 .54002 Metetra
.645 .763 .934 r
.78586 .54002 .79351 .54951 .81124 .54439 .80367 .53483 Metetra
.645 .763 .934 r
.80367 .53483 .81124 .54439 .82908 .53923 .82159 .52962 Metetra
.645 .763 .934 r
.82159 .52962 .82908 .53923 .84704 .53403 .83962 .52436 Metetra
.645 .763 .934 r
.83962 .52436 .84704 .53403 .8651 .52881 .85776 .51908 Metetra
.645 .763 .934 r
.85776 .51908 .8651 .52881 .88328 .52355 .87602 .51376 Metetra
.988 .732 .448 r
.87602 .51376 .88328 .52355 .88796 .39762 .88098 .38746 Metetra
.645 .763 .934 r
.24425 .687 .25403 .6949 .2687 .69063 .25897 .68268 Metetra
.645 .763 .934 r
.25897 .68268 .2687 .69063 .28345 .68634 .27377 .67834 Metetra
.645 .763 .934 r
.27377 .67834 .28345 .68634 .29829 .68202 .28865 .67397 Metetra
.645 .763 .934 r
.28865 .67397 .29829 .68202 .31321 .67767 .30362 .66958 Metetra
.645 .763 .934 r
.30362 .66958 .31321 .67767 .32821 .6733 .31868 .66517 Metetra
.645 .763 .934 r
.31868 .66517 .32821 .6733 .3433 .66891 .33382 .66073 Metetra
.645 .763 .934 r
.33382 .66073 .3433 .66891 .35847 .66449 .34905 .65626 Metetra
.645 .763 .934 r
.34905 .65626 .35847 .66449 .37374 .66004 .36436 .65177 Metetra
.645 .763 .934 r
.36436 .65177 .37374 .66004 .38909 .65557 .37977 .64725 Metetra
.645 .763 .934 r
.37977 .64725 .38909 .65557 .40452 .65108 .39526 .64271 Metetra
.645 .763 .934 r
.39526 .64271 .40452 .65108 .42005 .64655 .41084 .63814 Metetra
.645 .763 .934 r
.41084 .63814 .42005 .64655 .43567 .64201 .42652 .63354 Metetra
.645 .763 .934 r
.42652 .63354 .43567 .64201 .45137 .63743 .44228 .62892 Metetra
.645 .763 .934 r
.44228 .62892 .45137 .63743 .46717 .63283 .45814 .62427 Metetra
.645 .763 .934 r
.45814 .62427 .46717 .63283 .48306 .6282 .47409 .61959 Metetra
.645 .763 .934 r
.47409 .61959 .48306 .6282 .49905 .62355 .49014 .61488 Metetra
.645 .763 .934 r
.49014 .61488 .49905 .62355 .51513 .61887 .50628 .61015 Metetra
.645 .763 .934 r
.50628 .61015 .51513 .61887 .5313 .61416 .52251 .60539 Metetra
.645 .763 .934 r
.52251 .60539 .5313 .61416 .54757 .60942 .53884 .6006 Metetra
.645 .763 .934 r
.53884 .6006 .54757 .60942 .56393 .60465 .55527 .59578 Metetra
.645 .763 .934 r
.55527 .59578 .56393 .60465 .5804 .59986 .5718 .59093 Metetra
.645 .763 .934 r
.5718 .59093 .5804 .59986 .59696 .59503 .58843 .58605 Metetra
.645 .763 .934 r
.58843 .58605 .59696 .59503 .61362 .59018 .60515 .58115 Metetra
.645 .763 .934 r
.60515 .58115 .61362 .59018 .63037 .5853 .62198 .57621 Metetra
.645 .763 .934 r
.62198 .57621 .63037 .5853 .64723 .58039 .63891 .57125 Metetra
.645 .763 .934 r
.63891 .57125 .64723 .58039 .6642 .57545 .65594 .56625 Metetra
.645 .763 .934 r
.65594 .56625 .6642 .57545 .68126 .57048 .67307 .56123 Metetra
.645 .763 .934 r
.67307 .56123 .68126 .57048 .69843 .56548 .69031 .55617 Metetra
.645 .763 .934 r
.69031 .55617 .69843 .56548 .7157 .56045 .70766 .55109 Metetra
.645 .763 .934 r
.70766 .55109 .7157 .56045 .73308 .55539 .72511 .54597 Metetra
.645 .763 .934 r
.72511 .54597 .73308 .55539 .75056 .5503 .74267 .54082 Metetra
.645 .763 .934 r
.74267 .54082 .75056 .5503 .76816 .54518 .76034 .53563 Metetra
.645 .763 .934 r
.76034 .53563 .76816 .54518 .78586 .54002 .77812 .53042 Metetra
.645 .763 .934 r
.77812 .53042 .78586 .54002 .80367 .53483 .796 .52517 Metetra
.645 .763 .934 r
.796 .52517 .80367 .53483 .82159 .52962 .814 .5199 Metetra
.645 .763 .934 r
.814 .5199 .82159 .52962 .83962 .52436 .83212 .51458 Metetra
.645 .763 .934 r
.83212 .51458 .83962 .52436 .85776 .51908 .85034 .50924 Metetra
.645 .763 .934 r
.85034 .50924 .85776 .51908 .87602 .51376 .86868 .50386 Metetra
.988 .732 .448 r
.86868 .50386 .87602 .51376 .88098 .38746 .87391 .37717 Metetra
.645 .763 .934 r
.23436 .67901 .24425 .687 .25897 .68268 .24913 .67465 Metetra
.645 .763 .934 r
.24913 .67465 .25897 .68268 .27377 .67834 .26398 .67026 Metetra
.645 .763 .934 r
.26398 .67026 .27377 .67834 .28865 .67397 .27892 .66585 Metetra
.645 .763 .934 r
.27892 .66585 .28865 .67397 .30362 .66958 .29394 .66141 Metetra
.645 .763 .934 r
.29394 .66141 .30362 .66958 .31868 .66517 .30904 .65695 Metetra
.645 .763 .934 r
.30904 .65695 .31868 .66517 .33382 .66073 .32424 .65246 Metetra
.645 .763 .934 r
.32424 .65246 .33382 .66073 .34905 .65626 .33952 .64795 Metetra
.645 .763 .934 r
.33952 .64795 .34905 .65626 .36436 .65177 .35489 .64341 Metetra
.645 .763 .934 r
.35489 .64341 .36436 .65177 .37977 .64725 .37035 .63884 Metetra
.645 .763 .934 r
.37035 .63884 .37977 .64725 .39526 .64271 .3859 .63425 Metetra
.645 .763 .934 r
.3859 .63425 .39526 .64271 .41084 .63814 .40154 .62963 Metetra
.645 .763 .934 r
.40154 .62963 .41084 .63814 .42652 .63354 .41727 .62498 Metetra
.645 .763 .934 r
.41727 .62498 .42652 .63354 .44228 .62892 .43309 .62031 Metetra
.645 .763 .934 r
.43309 .62031 .44228 .62892 .45814 .62427 .44901 .61561 Metetra
.645 .763 .934 r
.44901 .61561 .45814 .62427 .47409 .61959 .46502 .61088 Metetra
.645 .763 .934 r
.46502 .61088 .47409 .61959 .49014 .61488 .48113 .60612 Metetra
.645 .763 .934 r
.48113 .60612 .49014 .61488 .50628 .61015 .49733 .60134 Metetra
.645 .763 .934 r
.49733 .60134 .50628 .61015 .52251 .60539 .51363 .59652 Metetra
.645 .763 .934 r
.51363 .59652 .52251 .60539 .53884 .6006 .53002 .59168 Metetra
.645 .763 .934 r
.53002 .59168 .53884 .6006 .55527 .59578 .54651 .58681 Metetra
.645 .763 .934 r
.54651 .58681 .55527 .59578 .5718 .59093 .56311 .58191 Metetra
.645 .763 .934 r
.56311 .58191 .5718 .59093 .58843 .58605 .5798 .57698 Metetra
.645 .763 .934 r
.5798 .57698 .58843 .58605 .60515 .58115 .59659 .57201 Metetra
.645 .763 .934 r
.59659 .57201 .60515 .58115 .62198 .57621 .61349 .56702 Metetra
.645 .763 .934 r
.61349 .56702 .62198 .57621 .63891 .57125 .63049 .562 Metetra
.645 .763 .934 r
.63049 .562 .63891 .57125 .65594 .56625 .64759 .55695 Metetra
.645 .763 .934 r
.64759 .55695 .65594 .56625 .67307 .56123 .66479 .55187 Metetra
.645 .763 .934 r
.66479 .55187 .67307 .56123 .69031 .55617 .68211 .54676 Metetra
.645 .763 .934 r
.68211 .54676 .69031 .55617 .70766 .55109 .69953 .54161 Metetra
.645 .763 .934 r
.69953 .54161 .70766 .55109 .72511 .54597 .71705 .53643 Metetra
.645 .763 .934 r
.71705 .53643 .72511 .54597 .74267 .54082 .73469 .53123 Metetra
.645 .763 .934 r
.73469 .53123 .74267 .54082 .76034 .53563 .75243 .52598 Metetra
.645 .763 .934 r
.75243 .52598 .76034 .53563 .77812 .53042 .77029 .52071 Metetra
.645 .763 .934 r
.77029 .52071 .77812 .53042 .796 .52517 .78825 .5154 Metetra
.645 .763 .934 r
.78825 .5154 .796 .52517 .814 .5199 .80633 .51006 Metetra
.645 .763 .934 r
.80633 .51006 .814 .5199 .83212 .51458 .82453 .50469 Metetra
.645 .763 .934 r
.82453 .50469 .83212 .51458 .85034 .50924 .84283 .49928 Metetra
.645 .763 .934 r
.84283 .49928 .85034 .50924 .86868 .50386 .86126 .49384 Metetra
.988 .732 .448 r
.86126 .49384 .86868 .50386 .87391 .37717 .86677 .36677 Metetra
.645 .763 .934 r
.22437 .67094 .23436 .67901 .24913 .67465 .23919 .66653 Metetra
.645 .763 .934 r
.23919 .66653 .24913 .67465 .26398 .67026 .25409 .6621 Metetra
.645 .763 .934 r
.25409 .6621 .26398 .67026 .27892 .66585 .26908 .65764 Metetra
.645 .763 .934 r
.26908 .65764 .27892 .66585 .29394 .66141 .28415 .65315 Metetra
.645 .763 .934 r
.28415 .65315 .29394 .66141 .30904 .65695 .29931 .64864 Metetra
.645 .763 .934 r
.29931 .64864 .30904 .65695 .32424 .65246 .31456 .64411 Metetra
.645 .763 .934 r
.31456 .64411 .32424 .65246 .33952 .64795 .32989 .63955 Metetra
.645 .763 .934 r
.32989 .63955 .33952 .64795 .35489 .64341 .34532 .63496 Metetra
.645 .763 .934 r
.34532 .63496 .35489 .64341 .37035 .63884 .36083 .63034 Metetra
.645 .763 .934 r
.36083 .63034 .37035 .63884 .3859 .63425 .37644 .6257 Metetra
.645 .763 .934 r
.37644 .6257 .3859 .63425 .40154 .62963 .39213 .62103 Metetra
.645 .763 .934 r
.39213 .62103 .40154 .62963 .41727 .62498 .40792 .61633 Metetra
.645 .763 .934 r
.40792 .61633 .41727 .62498 .43309 .62031 .4238 .61161 Metetra
.645 .763 .934 r
.4238 .61161 .43309 .62031 .44901 .61561 .43978 .60685 Metetra
.645 .763 .934 r
.43978 .60685 .44901 .61561 .46502 .61088 .45585 .60207 Metetra
.645 .763 .934 r
.45585 .60207 .46502 .61088 .48113 .60612 .47202 .59726 Metetra
.645 .763 .934 r
.47202 .59726 .48113 .60612 .49733 .60134 .48828 .59243 Metetra
.645 .763 .934 r
.48828 .59243 .49733 .60134 .51363 .59652 .50464 .58756 Metetra
.645 .763 .934 r
.50464 .58756 .51363 .59652 .53002 .59168 .5211 .58266 Metetra
.645 .763 .934 r
.5211 .58266 .53002 .59168 .54651 .58681 .53766 .57774 Metetra
.645 .763 .934 r
.53766 .57774 .54651 .58681 .56311 .58191 .55432 .57278 Metetra
.645 .763 .934 r
.55432 .57278 .56311 .58191 .5798 .57698 .57108 .56779 Metetra
.645 .763 .934 r
.57108 .56779 .5798 .57698 .59659 .57201 .58794 .56278 Metetra
.645 .763 .934 r
.58794 .56278 .59659 .57201 .61349 .56702 .6049 .55773 Metetra
.645 .763 .934 r
.6049 .55773 .61349 .56702 .63049 .562 .62197 .55265 Metetra
.645 .763 .934 r
.62197 .55265 .63049 .562 .64759 .55695 .63914 .54754 Metetra
.645 .763 .934 r
.63914 .54754 .64759 .55695 .66479 .55187 .65642 .5424 Metetra
.645 .763 .934 r
.65642 .5424 .66479 .55187 .68211 .54676 .67381 .53723 Metetra
.645 .763 .934 r
.67381 .53723 .68211 .54676 .69953 .54161 .6913 .53203 Metetra
.645 .763 .934 r
.6913 .53203 .69953 .54161 .71705 .53643 .7089 .52679 Metetra
.645 .763 .934 r
.7089 .52679 .71705 .53643 .73469 .53123 .72661 .52152 Metetra
.645 .763 .934 r
.72661 .52152 .73469 .53123 .75243 .52598 .74443 .51622 Metetra
.645 .763 .934 r
.74443 .51622 .75243 .52598 .77029 .52071 .76237 .51089 Metetra
.645 .763 .934 r
.76237 .51089 .77029 .52071 .78825 .5154 .78041 .50552 Metetra
.645 .763 .934 r
.78041 .50552 .78825 .5154 .80633 .51006 .79857 .50011 Metetra
.645 .763 .934 r
.79857 .50011 .80633 .51006 .82453 .50469 .81685 .49468 Metetra
.645 .763 .934 r
.81685 .49468 .82453 .50469 .84283 .49928 .83524 .48921 Metetra
.645 .763 .934 r
.83524 .48921 .84283 .49928 .86126 .49384 .85375 .4837 Metetra
.988 .732 .448 r
.85375 .4837 .86126 .49384 .86677 .36677 .85954 .35626 Metetra
.645 .763 .934 r
.21428 .66278 .22437 .67094 .23919 .66653 .22914 .65833 Metetra
.645 .763 .934 r
.22914 .65833 .23919 .66653 .25409 .6621 .2441 .65385 Metetra
.645 .763 .934 r
.2441 .65385 .25409 .6621 .26908 .65764 .25913 .64934 Metetra
.645 .763 .934 r
.25913 .64934 .26908 .65764 .28415 .65315 .27426 .64481 Metetra
.645 .763 .934 r
.27426 .64481 .28415 .65315 .29931 .64864 .28947 .64025 Metetra
.645 .763 .934 r
.28947 .64025 .29931 .64864 .31456 .64411 .30477 .63567 Metetra
.645 .763 .934 r
.30477 .63567 .31456 .64411 .32989 .63955 .32016 .63105 Metetra
.645 .763 .934 r
.32016 .63105 .32989 .63955 .34532 .63496 .33564 .62642 Metetra
.645 .763 .934 r
.33564 .62642 .34532 .63496 .36083 .63034 .35121 .62175 Metetra
.645 .763 .934 r
.35121 .62175 .36083 .63034 .37644 .6257 .36687 .61706 Metetra
.645 .763 .934 r
.36687 .61706 .37644 .6257 .39213 .62103 .38262 .61234 Metetra
.645 .763 .934 r
.38262 .61234 .39213 .62103 .40792 .61633 .39847 .60759 Metetra
.645 .763 .934 r
.39847 .60759 .40792 .61633 .4238 .61161 .41441 .60281 Metetra
.645 .763 .934 r
.41441 .60281 .4238 .61161 .43978 .60685 .43045 .59801 Metetra
.645 .763 .934 r
.43045 .59801 .43978 .60685 .45585 .60207 .44658 .59317 Metetra
.645 .763 .934 r
.44658 .59317 .45585 .60207 .47202 .59726 .46281 .58831 Metetra
.645 .763 .934 r
.46281 .58831 .47202 .59726 .48828 .59243 .47913 .58342 Metetra
.645 .763 .934 r
.47913 .58342 .48828 .59243 .50464 .58756 .49556 .5785 Metetra
.645 .763 .934 r
.49556 .5785 .50464 .58756 .5211 .58266 .51208 .57354 Metetra
.645 .763 .934 r
.51208 .57354 .5211 .58266 .53766 .57774 .52871 .56856 Metetra
.645 .763 .934 r
.52871 .56856 .53766 .57774 .55432 .57278 .54543 .56355 Metetra
.645 .763 .934 r
.54543 .56355 .55432 .57278 .57108 .56779 .56226 .55851 Metetra
.645 .763 .934 r
.56226 .55851 .57108 .56779 .58794 .56278 .57919 .55344 Metetra
.645 .763 .934 r
.57919 .55344 .58794 .56278 .6049 .55773 .59622 .54833 Metetra
.645 .763 .934 r
.59622 .54833 .6049 .55773 .62197 .55265 .61336 .5432 Metetra
.645 .763 .934 r
.61336 .5432 .62197 .55265 .63914 .54754 .6306 .53803 Metetra
.645 .763 .934 r
.6306 .53803 .63914 .54754 .65642 .5424 .64795 .53283 Metetra
.645 .763 .934 r
.64795 .53283 .65642 .5424 .67381 .53723 .66541 .5276 Metetra
.645 .763 .934 r
.66541 .5276 .67381 .53723 .6913 .53203 .68298 .52234 Metetra
.645 .763 .934 r
.68298 .52234 .6913 .53203 .7089 .52679 .70065 .51704 Metetra
.645 .763 .934 r
.70065 .51704 .7089 .52679 .72661 .52152 .71844 .51171 Metetra
.645 .763 .934 r
.71844 .51171 .72661 .52152 .74443 .51622 .73634 .50635 Metetra
.645 .763 .934 r
.73634 .50635 .74443 .51622 .76237 .51089 .75435 .50095 Metetra
.645 .763 .934 r
.75435 .50095 .76237 .51089 .78041 .50552 .77248 .49552 Metetra
.645 .763 .934 r
.77248 .49552 .78041 .50552 .79857 .50011 .79072 .49005 Metetra
.645 .763 .934 r
.79072 .49005 .79857 .50011 .81685 .49468 .80908 .48455 Metetra
.645 .763 .934 r
.80908 .48455 .81685 .49468 .83524 .48921 .82755 .47901 Metetra
.645 .763 .934 r
.82755 .47901 .83524 .48921 .85375 .4837 .84615 .47344 Metetra
.988 .732 .448 r
.84615 .47344 .85375 .4837 .85954 .35626 .85223 .34562 Metetra
.645 .763 .934 r
.20408 .65454 .21428 .66278 .22914 .65833 .21899 .65004 Metetra
.645 .763 .934 r
.21899 .65004 .22914 .65833 .2441 .65385 .23399 .64551 Metetra
.645 .763 .934 r
.23399 .64551 .2441 .65385 .25913 .64934 .24908 .64095 Metetra
.645 .763 .934 r
.24908 .64095 .25913 .64934 .27426 .64481 .26426 .63637 Metetra
.645 .763 .934 r
.26426 .63637 .27426 .64481 .28947 .64025 .27952 .63177 Metetra
.645 .763 .934 r
.27952 .63177 .28947 .64025 .30477 .63567 .29488 .62713 Metetra
.645 .763 .934 r
.29488 .62713 .30477 .63567 .32016 .63105 .31032 .62247 Metetra
.645 .763 .934 r
.31032 .62247 .32016 .63105 .33564 .62642 .32585 .61778 Metetra
.645 .763 .934 r
.32585 .61778 .33564 .62642 .35121 .62175 .34148 .61306 Metetra
.645 .763 .934 r
.34148 .61306 .35121 .62175 .36687 .61706 .3572 .60832 Metetra
.645 .763 .934 r
.3572 .60832 .36687 .61706 .38262 .61234 .37301 .60355 Metetra
.645 .763 .934 r
.37301 .60355 .38262 .61234 .39847 .60759 .38892 .59875 Metetra
.645 .763 .934 r
.38892 .59875 .39847 .60759 .41441 .60281 .40492 .59392 Metetra
.645 .763 .934 r
.40492 .59392 .41441 .60281 .43045 .59801 .42101 .58906 Metetra
.645 .763 .934 r
.42101 .58906 .43045 .59801 .44658 .59317 .43721 .58417 Metetra
.645 .763 .934 r
.43721 .58417 .44658 .59317 .46281 .58831 .4535 .57925 Metetra
.645 .763 .934 r
.4535 .57925 .46281 .58831 .47913 .58342 .46989 .57431 Metetra
.645 .763 .934 r
.46989 .57431 .47913 .58342 .49556 .5785 .48637 .56933 Metetra
.645 .763 .934 r
.48637 .56933 .49556 .5785 .51208 .57354 .50296 .56432 Metetra
.645 .763 .934 r
.50296 .56432 .51208 .57354 .52871 .56856 .51965 .55929 Metetra
.645 .763 .934 r
.51965 .55929 .52871 .56856 .54543 .56355 .53644 .55422 Metetra
.645 .763 .934 r
.53644 .55422 .54543 .56355 .56226 .55851 .55334 .54912 Metetra
.645 .763 .934 r
.55334 .54912 .56226 .55851 .57919 .55344 .57033 .54399 Metetra
.645 .763 .934 r
.57033 .54399 .57919 .55344 .59622 .54833 .58744 .53883 Metetra
.645 .763 .934 r
.58744 .53883 .59622 .54833 .61336 .5432 .60465 .53363 Metetra
.645 .763 .934 r
.60465 .53363 .61336 .5432 .6306 .53803 .62196 .52841 Metetra
.645 .763 .934 r
.62196 .52841 .6306 .53803 .64795 .53283 .63938 .52315 Metetra
.645 .763 .934 r
.63938 .52315 .64795 .53283 .66541 .5276 .65692 .51786 Metetra
.645 .763 .934 r
.65692 .51786 .66541 .5276 .68298 .52234 .67456 .51253 Metetra
.645 .763 .934 r
.67456 .51253 .68298 .52234 .70065 .51704 .69231 .50717 Metetra
.645 .763 .934 r
.69231 .50717 .70065 .51704 .71844 .51171 .71018 .50178 Metetra
.645 .763 .934 r
.71018 .50178 .71844 .51171 .73634 .50635 .72815 .49635 Metetra
.645 .763 .934 r
.72815 .49635 .73634 .50635 .75435 .50095 .74625 .49089 Metetra
.645 .763 .934 r
.74625 .49089 .75435 .50095 .77248 .49552 .76445 .4854 Metetra
.645 .763 .934 r
.76445 .4854 .77248 .49552 .79072 .49005 .78278 .47987 Metetra
.645 .763 .934 r
.78278 .47987 .79072 .49005 .80908 .48455 .80122 .4743 Metetra
.645 .763 .934 r
.80122 .4743 .80908 .48455 .82755 .47901 .81978 .4687 Metetra
.645 .763 .934 r
.81978 .4687 .82755 .47901 .84615 .47344 .83846 .46306 Metetra
.988 .732 .448 r
.83846 .46306 .84615 .47344 .85223 .34562 .84484 .33486 Metetra
.645 .763 .934 r
.19377 .64621 .20408 .65454 .21899 .65004 .20873 .64166 Metetra
.645 .763 .934 r
.20873 .64166 .21899 .65004 .23399 .64551 .22378 .63708 Metetra
.645 .763 .934 r
.22378 .63708 .23399 .64551 .24908 .64095 .23892 .63248 Metetra
.645 .763 .934 r
.23892 .63248 .24908 .64095 .26426 .63637 .25415 .62785 Metetra
.645 .763 .934 r
.25415 .62785 .26426 .63637 .27952 .63177 .26947 .62319 Metetra
.645 .763 .934 r
.26947 .62319 .27952 .63177 .29488 .62713 .28488 .6185 Metetra
.645 .763 .934 r
.28488 .6185 .29488 .62713 .31032 .62247 .30037 .61379 Metetra
.645 .763 .934 r
.30037 .61379 .31032 .62247 .32585 .61778 .31596 .60905 Metetra
.645 .763 .934 r
.31596 .60905 .32585 .61778 .34148 .61306 .33165 .60428 Metetra
.645 .763 .934 r
.33165 .60428 .34148 .61306 .3572 .60832 .34742 .59949 Metetra
.645 .763 .934 r
.34742 .59949 .3572 .60832 .37301 .60355 .36329 .59466 Metetra
.645 .763 .934 r
.36329 .59466 .37301 .60355 .38892 .59875 .37926 .58981 Metetra
.645 .763 .934 r
.37926 .58981 .38892 .59875 .40492 .59392 .39532 .58493 Metetra
.645 .763 .934 r
.39532 .58493 .40492 .59392 .42101 .58906 .41147 .58001 Metetra
.645 .763 .934 r
.41147 .58001 .42101 .58906 .43721 .58417 .42773 .57507 Metetra
.645 .763 .934 r
.42773 .57507 .43721 .58417 .4535 .57925 .44408 .5701 Metetra
.645 .763 .934 r
.44408 .5701 .4535 .57925 .46989 .57431 .46053 .5651 Metetra
.645 .763 .934 r
.46053 .5651 .46989 .57431 .48637 .56933 .47708 .56006 Metetra
.645 .763 .934 r
.47708 .56006 .48637 .56933 .50296 .56432 .49374 .555 Metetra
.645 .763 .934 r
.49374 .555 .50296 .56432 .51965 .55929 .51049 .54991 Metetra
.645 .763 .934 r
.51049 .54991 .51965 .55929 .53644 .55422 .52735 .54478 Metetra
.645 .763 .934 r
.52735 .54478 .53644 .55422 .55334 .54912 .54431 .53962 Metetra
.645 .763 .934 r
.54431 .53962 .55334 .54912 .57033 .54399 .56138 .53443 Metetra
.645 .763 .934 r
.56138 .53443 .57033 .54399 .58744 .53883 .57855 .52921 Metetra
.645 .763 .934 r
.57855 .52921 .58744 .53883 .60465 .53363 .59583 .52396 Metetra
.645 .763 .934 r
.59583 .52396 .60465 .53363 .62196 .52841 .61322 .51867 Metetra
.645 .763 .934 r
.61322 .51867 .62196 .52841 .63938 .52315 .63072 .51335 Metetra
.645 .763 .934 r
.63072 .51335 .63938 .52315 .65692 .51786 .64833 .508 Metetra
.645 .763 .934 r
.64833 .508 .65692 .51786 .67456 .51253 .66604 .50261 Metetra
.645 .763 .934 r
.66604 .50261 .67456 .51253 .69231 .50717 .68387 .49719 Metetra
.645 .763 .934 r
.68387 .49719 .69231 .50717 .71018 .50178 .70182 .49174 Metetra
.645 .763 .934 r
.70182 .49174 .71018 .50178 .72815 .49635 .71987 .48625 Metetra
.645 .763 .934 r
.71987 .48625 .72815 .49635 .74625 .49089 .73804 .48072 Metetra
.645 .763 .934 r
.73804 .48072 .74625 .49089 .76445 .4854 .75633 .47516 Metetra
.645 .763 .934 r
.75633 .47516 .76445 .4854 .78278 .47987 .77474 .46956 Metetra
.645 .763 .934 r
.77474 .46956 .78278 .47987 .80122 .4743 .79326 .46393 Metetra
.645 .763 .934 r
.79326 .46393 .80122 .4743 .81978 .4687 .81191 .45826 Metetra
.645 .763 .934 r
.81191 .45826 .81978 .4687 .83846 .46306 .83067 .45256 Metetra
.988 .732 .448 r
.83067 .45256 .83846 .46306 .84484 .33486 .83736 .32398 Metetra
.645 .763 .934 r
.18335 .63778 .19377 .64621 .20873 .64166 .19836 .63319 Metetra
.645 .763 .934 r
.19836 .63319 .20873 .64166 .22378 .63708 .21346 .62856 Metetra
.645 .763 .934 r
.21346 .62856 .22378 .63708 .23892 .63248 .22866 .62391 Metetra
.645 .763 .934 r
.22866 .62391 .23892 .63248 .25415 .62785 .24394 .61923 Metetra
.645 .763 .934 r
.24394 .61923 .25415 .62785 .26947 .62319 .25931 .61452 Metetra
.645 .763 .934 r
.25931 .61452 .26947 .62319 .28488 .6185 .27477 .60978 Metetra
.645 .763 .934 r
.27477 .60978 .28488 .6185 .30037 .61379 .29032 .60502 Metetra
.645 .763 .934 r
.29032 .60502 .30037 .61379 .31596 .60905 .30597 .60023 Metetra
.645 .763 .934 r
.30597 .60023 .31596 .60905 .33165 .60428 .32171 .59541 Metetra
.645 .763 .934 r
.32171 .59541 .33165 .60428 .34742 .59949 .33754 .59056 Metetra
.645 .763 .934 r
.33754 .59056 .34742 .59949 .36329 .59466 .35347 .58568 Metetra
.645 .763 .934 r
.35347 .58568 .36329 .59466 .37926 .58981 .36949 .58077 Metetra
.645 .763 .934 r
.36949 .58077 .37926 .58981 .39532 .58493 .38561 .57583 Metetra
.645 .763 .934 r
.38561 .57583 .39532 .58493 .41147 .58001 .40183 .57087 Metetra
.645 .763 .934 r
.40183 .57087 .41147 .58001 .42773 .57507 .41814 .56587 Metetra
.645 .763 .934 r
.41814 .56587 .42773 .57507 .44408 .5701 .43456 .56084 Metetra
.645 .763 .934 r
.43456 .56084 .44408 .5701 .46053 .5651 .45107 .55578 Metetra
.645 .763 .934 r
.45107 .55578 .46053 .5651 .47708 .56006 .46769 .55069 Metetra
.645 .763 .934 r
.46769 .55069 .47708 .56006 .49374 .555 .48441 .54557 Metetra
.645 .763 .934 r
.48441 .54557 .49374 .555 .51049 .54991 .50123 .54042 Metetra
.645 .763 .934 r
.50123 .54042 .51049 .54991 .52735 .54478 .51816 .53523 Metetra
.645 .763 .934 r
.51816 .53523 .52735 .54478 .54431 .53962 .53519 .53002 Metetra
.645 .763 .934 r
.53519 .53002 .54431 .53962 .56138 .53443 .55233 .52477 Metetra
.645 .763 .934 r
.55233 .52477 .56138 .53443 .57855 .52921 .56957 .51949 Metetra
.645 .763 .934 r
.56957 .51949 .57855 .52921 .59583 .52396 .58692 .51417 Metetra
.645 .763 .934 r
.58692 .51417 .59583 .52396 .61322 .51867 .60438 .50883 Metetra
.645 .763 .934 r
.60438 .50883 .61322 .51867 .63072 .51335 .62195 .50344 Metetra
.645 .763 .934 r
.62195 .50344 .63072 .51335 .64833 .508 .63963 .49803 Metetra
.645 .763 .934 r
.63963 .49803 .64833 .508 .66604 .50261 .65743 .49258 Metetra
.645 .763 .934 r
.65743 .49258 .66604 .50261 .68387 .49719 .67533 .48709 Metetra
.645 .763 .934 r
.67533 .48709 .68387 .49719 .70182 .49174 .69336 .48157 Metetra
.645 .763 .934 r
.69336 .48157 .70182 .49174 .71987 .48625 .71149 .47602 Metetra
.645 .763 .934 r
.71149 .47602 .71987 .48625 .73804 .48072 .72975 .47043 Metetra
.645 .763 .934 r
.72975 .47043 .73804 .48072 .75633 .47516 .74812 .4648 Metetra
.645 .763 .934 r
.74812 .4648 .75633 .47516 .77474 .46956 .7666 .45914 Metetra
.645 .763 .934 r
.7666 .45914 .77474 .46956 .79326 .46393 .78521 .45344 Metetra
.645 .763 .934 r
.78521 .45344 .79326 .46393 .81191 .45826 .80394 .4477 Metetra
.645 .763 .934 r
.80394 .4477 .81191 .45826 .83067 .45256 .8228 .44193 Metetra
.988 .732 .448 r
.8228 .44193 .83067 .45256 .83736 .32398 .8298 .31297 Metetra
.645 .763 .934 r
.17281 .62927 .18335 .63778 .19836 .63319 .18788 .62462 Metetra
.645 .763 .934 r
.18788 .62462 .19836 .63319 .21346 .62856 .20303 .61995 Metetra
.645 .763 .934 r
.20303 .61995 .21346 .62856 .22866 .62391 .21828 .61524 Metetra
.645 .763 .934 r
.21828 .61524 .22866 .62391 .24394 .61923 .23361 .61051 Metetra
.645 .763 .934 r
.23361 .61051 .24394 .61923 .25931 .61452 .24903 .60575 Metetra
.645 .763 .934 r
.24903 .60575 .25931 .61452 .27477 .60978 .26455 .60097 Metetra
.645 .763 .934 r
.26455 .60097 .27477 .60978 .29032 .60502 .28016 .59615 Metetra
.645 .763 .934 r
.28016 .59615 .29032 .60502 .30597 .60023 .29586 .59131 Metetra
.645 .763 .934 r
.29586 .59131 .30597 .60023 .32171 .59541 .31165 .58643 Metetra
.645 .763 .934 r
.31165 .58643 .32171 .59541 .33754 .59056 .32754 .58153 Metetra
.645 .763 .934 r
.32754 .58153 .33754 .59056 .35347 .58568 .34353 .57659 Metetra
.645 .763 .934 r
.34353 .57659 .35347 .58568 .36949 .58077 .35961 .57163 Metetra
.645 .763 .934 r
.35961 .57163 .36949 .58077 .38561 .57583 .37579 .56664 Metetra
.645 .763 .934 r
.37579 .56664 .38561 .57583 .40183 .57087 .39207 .56162 Metetra
.645 .763 .934 r
.39207 .56162 .40183 .57087 .41814 .56587 .40845 .55656 Metetra
.645 .763 .934 r
.40845 .55656 .41814 .56587 .43456 .56084 .42493 .55148 Metetra
.645 .763 .934 r
.42493 .55148 .43456 .56084 .45107 .55578 .44151 .54636 Metetra
.645 .763 .934 r
.44151 .54636 .45107 .55578 .46769 .55069 .45819 .54121 Metetra
.645 .763 .934 r
.45819 .54121 .46769 .55069 .48441 .54557 .47498 .53603 Metetra
.645 .763 .934 r
.47498 .53603 .48441 .54557 .50123 .54042 .49186 .53082 Metetra
.645 .763 .934 r
.49186 .53082 .50123 .54042 .51816 .53523 .50886 .52558 Metetra
.645 .763 .934 r
.50886 .52558 .51816 .53523 .53519 .53002 .52596 .5203 Metetra
.645 .763 .934 r
.52596 .5203 .53519 .53002 .55233 .52477 .54317 .51499 Metetra
.645 .763 .934 r
.54317 .51499 .55233 .52477 .56957 .51949 .56048 .50965 Metetra
.645 .763 .934 r
.56048 .50965 .56957 .51949 .58692 .51417 .5779 .50427 Metetra
.645 .763 .934 r
.5779 .50427 .58692 .51417 .60438 .50883 .59544 .49886 Metetra
.645 .763 .934 r
.59544 .49886 .60438 .50883 .62195 .50344 .61308 .49342 Metetra
.645 .763 .934 r
.61308 .49342 .62195 .50344 .63963 .49803 .63084 .48794 Metetra
.645 .763 .934 r
.63084 .48794 .63963 .49803 .65743 .49258 .64871 .48242 Metetra
.645 .763 .934 r
.64871 .48242 .65743 .49258 .67533 .48709 .6667 .47688 Metetra
.645 .763 .934 r
.6667 .47688 .67533 .48709 .69336 .48157 .6848 .47129 Metetra
.645 .763 .934 r
.6848 .47129 .69336 .48157 .71149 .47602 .70301 .46567 Metetra
.645 .763 .934 r
.70301 .46567 .71149 .47602 .72975 .47043 .72135 .46001 Metetra
.645 .763 .934 r
.72135 .46001 .72975 .47043 .74812 .4648 .7398 .45432 Metetra
.645 .763 .934 r
.7398 .45432 .74812 .4648 .7666 .45914 .75837 .44859 Metetra
.645 .763 .934 r
.75837 .44859 .7666 .45914 .78521 .45344 .77707 .44282 Metetra
.645 .763 .934 r
.77707 .44282 .78521 .45344 .80394 .4477 .79588 .43701 Metetra
.645 .763 .934 r
.79588 .43701 .80394 .4477 .8228 .44193 .81482 .43117 Metetra
.988 .732 .448 r
.81482 .43117 .8228 .44193 .8298 .31297 .82214 .30183 Metetra
.645 .763 .934 r
.16217 .62067 .17281 .62927 .18788 .62462 .17728 .61597 Metetra
.645 .763 .934 r
.17728 .61597 .18788 .62462 .20303 .61995 .19249 .61124 Metetra
.645 .763 .934 r
.19249 .61124 .20303 .61995 .21828 .61524 .20778 .60649 Metetra
.645 .763 .934 r
.20778 .60649 .21828 .61524 .23361 .61051 .22317 .6017 Metetra
.645 .763 .934 r
.22317 .6017 .23361 .61051 .24903 .60575 .23865 .59689 Metetra
.645 .763 .934 r
.23865 .59689 .24903 .60575 .26455 .60097 .25422 .59205 Metetra
.645 .763 .934 r
.25422 .59205 .26455 .60097 .28016 .59615 .26988 .58718 Metetra
.645 .763 .934 r
.26988 .58718 .28016 .59615 .29586 .59131 .28564 .58228 Metetra
.645 .763 .934 r
.28564 .58228 .29586 .59131 .31165 .58643 .30149 .57736 Metetra
.645 .763 .934 r
.30149 .57736 .31165 .58643 .32754 .58153 .31744 .5724 Metetra
.645 .763 .934 r
.31744 .5724 .32754 .58153 .34353 .57659 .33348 .56741 Metetra
.645 .763 .934 r
.33348 .56741 .34353 .57659 .35961 .57163 .34963 .56239 Metetra
.645 .763 .934 r
.34963 .56239 .35961 .57163 .37579 .56664 .36587 .55734 Metetra
.645 .763 .934 r
.36587 .55734 .37579 .56664 .39207 .56162 .38221 .55226 Metetra
.645 .763 .934 r
.38221 .55226 .39207 .56162 .40845 .55656 .39865 .54715 Metetra
.645 .763 .934 r
.39865 .54715 .40845 .55656 .42493 .55148 .41519 .54201 Metetra
.645 .763 .934 r
.41519 .54201 .42493 .55148 .44151 .54636 .43183 .53683 Metetra
.645 .763 .934 r
.43183 .53683 .44151 .54636 .45819 .54121 .44858 .53163 Metetra
.645 .763 .934 r
.44858 .53163 .45819 .54121 .47498 .53603 .46543 .52639 Metetra
.645 .763 .934 r
.46543 .52639 .47498 .53603 .49186 .53082 .48239 .52112 Metetra
.645 .763 .934 r
.48239 .52112 .49186 .53082 .50886 .52558 .49945 .51581 Metetra
.645 .763 .934 r
.49945 .51581 .50886 .52558 .52596 .5203 .51662 .51047 Metetra
.645 .763 .934 r
.51662 .51047 .52596 .5203 .54317 .51499 .5339 .5051 Metetra
.645 .763 .934 r
.5339 .5051 .54317 .51499 .56048 .50965 .55129 .4997 Metetra
.645 .763 .934 r
.55129 .4997 .56048 .50965 .5779 .50427 .56878 .49426 Metetra
.645 .763 .934 r
.56878 .49426 .5779 .50427 .59544 .49886 .58639 .48878 Metetra
.645 .763 .934 r
.58639 .48878 .59544 .49886 .61308 .49342 .60411 .48328 Metetra
.645 .763 .934 r
.60411 .48328 .61308 .49342 .63084 .48794 .62194 .47773 Metetra
.645 .763 .934 r
.62194 .47773 .63084 .48794 .64871 .48242 .63989 .47215 Metetra
.645 .763 .934 r
.63989 .47215 .64871 .48242 .6667 .47688 .65795 .46654 Metetra
.645 .763 .934 r
.65795 .46654 .6667 .47688 .6848 .47129 .67613 .46088 Metetra
.645 .763 .934 r
.67613 .46088 .6848 .47129 .70301 .46567 .69443 .4552 Metetra
.645 .763 .934 r
.69443 .4552 .70301 .46567 .72135 .46001 .71285 .44947 Metetra
.645 .763 .934 r
.71285 .44947 .72135 .46001 .7398 .45432 .73138 .44371 Metetra
.645 .763 .934 r
.73138 .44371 .7398 .45432 .75837 .44859 .75004 .43791 Metetra
.645 .763 .934 r
.75004 .43791 .75837 .44859 .77707 .44282 .76882 .43207 Metetra
.645 .763 .934 r
.76882 .43207 .77707 .44282 .79588 .43701 .78773 .42619 Metetra
.645 .763 .934 r
.78773 .42619 .79588 .43701 .81482 .43117 .80675 .42028 Metetra
.988 .732 .448 r
.80675 .42028 .81482 .43117 .82214 .30183 .8144 .29056 Metetra
.645 .763 .934 r
.1514 .61197 .16217 .62067 .17728 .61597 .16657 .60722 Metetra
.645 .763 .934 r
.16657 .60722 .17728 .61597 .19249 .61124 .18183 .60244 Metetra
.645 .763 .934 r
.18183 .60244 .19249 .61124 .20778 .60649 .19717 .59764 Metetra
.645 .763 .934 r
.19717 .59764 .20778 .60649 .22317 .6017 .21261 .5928 Metetra
.645 .763 .934 r
.21261 .5928 .22317 .6017 .23865 .59689 .22814 .58793 Metetra
.645 .763 .934 r
.22814 .58793 .23865 .59689 .25422 .59205 .24377 .58304 Metetra
.645 .763 .934 r
.24377 .58304 .25422 .59205 .26988 .58718 .25949 .57812 Metetra
.645 .763 .934 r
.25949 .57812 .26988 .58718 .28564 .58228 .2753 .57316 Metetra
.645 .763 .934 r
.2753 .57316 .28564 .58228 .30149 .57736 .29121 .56818 Metetra
.645 .763 .934 r
.29121 .56818 .30149 .57736 .31744 .5724 .30722 .56316 Metetra
.645 .763 .934 r
.30722 .56316 .31744 .5724 .33348 .56741 .32332 .55812 Metetra
.645 .763 .934 r
.32332 .55812 .33348 .56741 .34963 .56239 .33953 .55305 Metetra
.645 .763 .934 r
.33953 .55305 .34963 .56239 .36587 .55734 .35583 .54794 Metetra
.645 .763 .934 r
.35583 .54794 .36587 .55734 .38221 .55226 .37223 .5428 Metetra
.645 .763 .934 r
.37223 .5428 .38221 .55226 .39865 .54715 .38873 .53763 Metetra
.645 .763 .934 r
.38873 .53763 .39865 .54715 .41519 .54201 .40534 .53243 Metetra
.645 .763 .934 r
.40534 .53243 .41519 .54201 .43183 .53683 .42205 .5272 Metetra
.645 .763 .934 r
.42205 .5272 .43183 .53683 .44858 .53163 .43886 .52193 Metetra
.645 .763 .934 r
.43886 .52193 .44858 .53163 .46543 .52639 .45578 .51663 Metetra
.645 .763 .934 r
.45578 .51663 .46543 .52639 .48239 .52112 .4728 .5113 Metetra
.645 .763 .934 r
.4728 .5113 .48239 .52112 .49945 .51581 .48994 .50593 Metetra
.645 .763 .934 r
.48994 .50593 .49945 .51581 .51662 .51047 .50718 .50053 Metetra
.645 .763 .934 r
.50718 .50053 .51662 .51047 .5339 .5051 .52452 .4951 Metetra
.645 .763 .934 r
.52452 .4951 .5339 .5051 .55129 .4997 .54198 .48963 Metetra
.645 .763 .934 r
.54198 .48963 .55129 .4997 .56878 .49426 .55955 .48413 Metetra
.645 .763 .934 r
.55955 .48413 .56878 .49426 .58639 .48878 .57724 .47859 Metetra
.645 .763 .934 r
.57724 .47859 .58639 .48878 .60411 .48328 .59503 .47301 Metetra
.645 .763 .934 r
.59503 .47301 .60411 .48328 .62194 .47773 .61294 .4674 Metetra
.645 .763 .934 r
.61294 .4674 .62194 .47773 .63989 .47215 .63097 .46176 Metetra
.645 .763 .934 r
.63097 .46176 .63989 .47215 .65795 .46654 .64911 .45607 Metetra
.645 .763 .934 r
.64911 .45607 .65795 .46654 .67613 .46088 .66737 .45035 Metetra
.645 .763 .934 r
.66737 .45035 .67613 .46088 .69443 .4552 .68575 .4446 Metetra
.645 .763 .934 r
.68575 .4446 .69443 .4552 .71285 .44947 .70425 .4388 Metetra
.645 .763 .934 r
.70425 .4388 .71285 .44947 .73138 .44371 .72287 .43297 Metetra
.645 .763 .934 r
.72287 .43297 .73138 .44371 .75004 .43791 .74161 .4271 Metetra
.645 .763 .934 r
.74161 .4271 .75004 .43791 .76882 .43207 .76048 .42119 Metetra
.645 .763 .934 r
.76048 .42119 .76882 .43207 .78773 .42619 .77947 .41524 Metetra
.645 .763 .934 r
.77947 .41524 .78773 .42619 .80675 .42028 .79859 .40925 Metetra
.988 .732 .448 r
.79859 .40925 .80675 .42028 .8144 .29056 .80656 .27915 Metetra
.645 .763 .934 r
.14052 .60318 .1514 .61197 .16657 .60722 .15574 .59838 Metetra
.645 .763 .934 r
.15574 .59838 .16657 .60722 .18183 .60244 .17105 .59354 Metetra
.645 .763 .934 r
.17105 .59354 .18183 .60244 .19717 .59764 .18645 .58868 Metetra
.645 .763 .934 r
.18645 .58868 .19717 .59764 .21261 .5928 .20194 .58379 Metetra
.645 .763 .934 r
.20194 .58379 .21261 .5928 .22814 .58793 .21753 .57888 Metetra
.645 .763 .934 r
.21753 .57888 .22814 .58793 .24377 .58304 .23321 .57393 Metetra
.645 .763 .934 r
.23321 .57393 .24377 .58304 .25949 .57812 .24898 .56895 Metetra
.645 .763 .934 r
.24898 .56895 .25949 .57812 .2753 .57316 .26485 .56394 Metetra
.645 .763 .934 r
.26485 .56394 .2753 .57316 .29121 .56818 .28082 .5589 Metetra
.645 .763 .934 r
.28082 .5589 .29121 .56818 .30722 .56316 .29688 .55383 Metetra
.645 .763 .934 r
.29688 .55383 .30722 .56316 .32332 .55812 .31305 .54873 Metetra
.645 .763 .934 r
.31305 .54873 .32332 .55812 .33953 .55305 .32931 .54359 Metetra
.645 .763 .934 r
.32931 .54359 .33953 .55305 .35583 .54794 .34568 .53843 Metetra
.645 .763 .934 r
.34568 .53843 .35583 .54794 .37223 .5428 .36214 .53323 Metetra
.645 .763 .934 r
.36214 .53323 .37223 .5428 .38873 .53763 .37871 .528 Metetra
.645 .763 .934 r
.37871 .528 .38873 .53763 .40534 .53243 .39538 .52274 Metetra
.645 .763 .934 r
.39538 .52274 .40534 .53243 .42205 .5272 .41215 .51745 Metetra
.645 .763 .934 r
.41215 .51745 .42205 .5272 .43886 .52193 .42903 .51212 Metetra
.645 .763 .934 r
.42903 .51212 .43886 .52193 .45578 .51663 .44602 .50676 Metetra
.645 .763 .934 r
.44602 .50676 .45578 .51663 .4728 .5113 .46311 .50136 Metetra
.645 .763 .934 r
.46311 .50136 .4728 .5113 .48994 .50593 .48031 .49594 Metetra
.645 .763 .934 r
.48031 .49594 .48994 .50593 .50718 .50053 .49762 .49047 Metetra
.645 .763 .934 r
.49762 .49047 .50718 .50053 .52452 .4951 .51504 .48497 Metetra
.645 .763 .934 r
.51504 .48497 .52452 .4951 .54198 .48963 .53257 .47944 Metetra
.645 .763 .934 r
.53257 .47944 .54198 .48963 .55955 .48413 .55021 .47387 Metetra
.645 .763 .934 r
.55021 .47387 .55955 .48413 .57724 .47859 .56797 .46827 Metetra
.645 .763 .934 r
.56797 .46827 .57724 .47859 .59503 .47301 .58584 .46263 Metetra
.645 .763 .934 r
.58584 .46263 .59503 .47301 .61294 .4674 .60383 .45695 Metetra
.645 .763 .934 r
.60383 .45695 .61294 .4674 .63097 .46176 .62193 .45124 Metetra
.645 .763 .934 r
.62193 .45124 .63097 .46176 .64911 .45607 .64016 .44548 Metetra
.645 .763 .934 r
.64016 .44548 .64911 .45607 .66737 .45035 .6585 .4397 Metetra
.645 .763 .934 r
.6585 .4397 .66737 .45035 .68575 .4446 .67696 .43387 Metetra
.645 .763 .934 r
.67696 .43387 .68575 .4446 .70425 .4388 .69554 .428 Metetra
.645 .763 .934 r
.69554 .428 .70425 .4388 .72287 .43297 .71425 .4221 Metetra
.645 .763 .934 r
.71425 .4221 .72287 .43297 .74161 .4271 .73307 .41616 Metetra
.645 .763 .934 r
.73307 .41616 .74161 .4271 .76048 .42119 .75203 .41018 Metetra
.645 .763 .934 r
.75203 .41018 .76048 .42119 .77947 .41524 .77111 .40415 Metetra
.645 .763 .934 r
.77111 .40415 .77947 .41524 .79859 .40925 .79032 .39809 Metetra
.988 .732 .448 r
.79032 .39809 .79859 .40925 .80656 .27915 .79863 .26762 Metetra
.645 .763 .934 r
.12952 .59429 .14052 .60318 .15574 .59838 .14479 .58943 Metetra
.645 .763 .934 r
.14479 .58943 .15574 .59838 .17105 .59354 .16015 .58455 Metetra
.645 .763 .934 r
.16015 .58455 .17105 .59354 .18645 .58868 .1756 .57963 Metetra
.645 .763 .934 r
.1756 .57963 .18645 .58868 .20194 .58379 .19115 .57469 Metetra
.645 .763 .934 r
.19115 .57469 .20194 .58379 .21753 .57888 .20679 .56972 Metetra
.645 .763 .934 r
.20679 .56972 .21753 .57888 .23321 .57393 .22252 .56471 Metetra
.645 .763 .934 r
.22252 .56471 .23321 .57393 .24898 .56895 .23836 .55968 Metetra
.645 .763 .934 r
.23836 .55968 .24898 .56895 .26485 .56394 .25428 .55461 Metetra
.645 .763 .934 r
.25428 .55461 .26485 .56394 .28082 .5589 .27031 .54951 Metetra
.645 .763 .934 r
.27031 .54951 .28082 .5589 .29688 .55383 .28643 .54439 Metetra
.645 .763 .934 r
.28643 .54439 .29688 .55383 .31305 .54873 .30266 .53923 Metetra
.645 .763 .934 r
.30266 .53923 .31305 .54873 .32931 .54359 .31898 .53403 Metetra
.645 .763 .934 r
.31898 .53403 .32931 .54359 .34568 .53843 .33541 .52881 Metetra
.645 .763 .934 r
.33541 .52881 .34568 .53843 .36214 .53323 .35193 .52355 Metetra
.645 .763 .934 r
.35193 .52355 .36214 .53323 .37871 .528 .36856 .51826 Metetra
.645 .763 .934 r
.36856 .51826 .37871 .528 .39538 .52274 .3853 .51294 Metetra
.645 .763 .934 r
.3853 .51294 .39538 .52274 .41215 .51745 .40214 .50759 Metetra
.645 .763 .934 r
.40214 .50759 .41215 .51745 .42903 .51212 .41908 .5022 Metetra
.645 .763 .934 r
.41908 .5022 .42903 .51212 .44602 .50676 .43614 .49677 Metetra
.645 .763 .934 r
.43614 .49677 .44602 .50676 .46311 .50136 .4533 .49132 Metetra
.645 .763 .934 r
.4533 .49132 .46311 .50136 .48031 .49594 .47057 .48582 Metetra
.645 .763 .934 r
.47057 .48582 .48031 .49594 .49762 .49047 .48795 .48029 Metetra
.645 .763 .934 r
.48795 .48029 .49762 .49047 .51504 .48497 .50544 .47473 Metetra
.645 .763 .934 r
.50544 .47473 .51504 .48497 .53257 .47944 .52305 .46913 Metetra
.645 .763 .934 r
.52305 .46913 .53257 .47944 .55021 .47387 .54076 .4635 Metetra
.645 .763 .934 r
.54076 .4635 .55021 .47387 .56797 .46827 .5586 .45783 Metetra
.645 .763 .934 r
.5586 .45783 .56797 .46827 .58584 .46263 .57655 .45212 Metetra
.645 .763 .934 r
.57655 .45212 .58584 .46263 .60383 .45695 .59461 .44637 Metetra
.645 .763 .934 r
.59461 .44637 .60383 .45695 .62193 .45124 .61279 .44059 Metetra
.645 .763 .934 r
.61279 .44059 .62193 .45124 .64016 .44548 .6311 .43477 Metetra
.645 .763 .934 r
.6311 .43477 .64016 .44548 .6585 .4397 .64952 .42891 Metetra
.645 .763 .934 r
.64952 .42891 .6585 .4397 .67696 .43387 .66806 .42301 Metetra
.645 .763 .934 r
.66806 .42301 .67696 .43387 .69554 .428 .68673 .41707 Metetra
.645 .763 .934 r
.68673 .41707 .69554 .428 .71425 .4221 .70552 .4111 Metetra
.645 .763 .934 r
.70552 .4111 .71425 .4221 .73307 .41616 .72443 .40508 Metetra
.645 .763 .934 r
.72443 .40508 .73307 .41616 .75203 .41018 .74348 .39903 Metetra
.645 .763 .934 r
.74348 .39903 .75203 .41018 .77111 .40415 .76265 .39293 Metetra
.645 .763 .934 r
.76265 .39293 .77111 .40415 .79032 .39809 .78194 .38679 Metetra
.988 .732 .448 r
.78194 .38679 .79032 .39809 .79863 .26762 .79061 .25594 Metetra
.645 .763 .934 r
.1184 .5853 .12952 .59429 .14479 .58943 .13372 .58039 Metetra
.645 .763 .934 r
.13372 .58039 .14479 .58943 .16015 .58455 .14913 .57545 Metetra
.645 .763 .934 r
.14913 .57545 .16015 .58455 .1756 .57963 .16464 .57048 Metetra
.645 .763 .934 r
.16464 .57048 .1756 .57963 .19115 .57469 .18024 .56548 Metetra
.645 .763 .934 r
.18024 .56548 .19115 .57469 .20679 .56972 .19593 .56045 Metetra
.645 .763 .934 r
.19593 .56045 .20679 .56972 .22252 .56471 .21172 .55539 Metetra
.645 .763 .934 r
.21172 .55539 .22252 .56471 .23836 .55968 .22761 .5503 Metetra
.645 .763 .934 r
.22761 .5503 .23836 .55968 .25428 .55461 .2436 .54518 Metetra
.645 .763 .934 r
.2436 .54518 .25428 .55461 .27031 .54951 .25968 .54002 Metetra
.645 .763 .934 r
.25968 .54002 .27031 .54951 .28643 .54439 .27586 .53483 Metetra
.645 .763 .934 r
.27586 .53483 .28643 .54439 .30266 .53923 .29215 .52962 Metetra
.645 .763 .934 r
.29215 .52962 .30266 .53923 .31898 .53403 .30853 .52436 Metetra
.645 .763 .934 r
.30853 .52436 .31898 .53403 .33541 .52881 .32502 .51908 Metetra
.645 .763 .934 r
.32502 .51908 .33541 .52881 .35193 .52355 .34161 .51376 Metetra
.645 .763 .934 r
.34161 .51376 .35193 .52355 .36856 .51826 .3583 .50841 Metetra
.645 .763 .934 r
.3583 .50841 .36856 .51826 .3853 .51294 .3751 .50303 Metetra
.645 .763 .934 r
.3751 .50303 .3853 .51294 .40214 .50759 .39201 .49761 Metetra
.645 .763 .934 r
.39201 .49761 .40214 .50759 .41908 .5022 .40902 .49216 Metetra
.645 .763 .934 r
.40902 .49216 .41908 .5022 .43614 .49677 .42614 .48667 Metetra
.645 .763 .934 r
.42614 .48667 .43614 .49677 .4533 .49132 .44337 .48115 Metetra
.645 .763 .934 r
.44337 .48115 .4533 .49132 .47057 .48582 .46071 .47559 Metetra
.645 .763 .934 r
.46071 .47559 .47057 .48582 .48795 .48029 .47817 .47 Metetra
.645 .763 .934 r
.47817 .47 .48795 .48029 .50544 .47473 .49573 .46437 Metetra
.645 .763 .934 r
.49573 .46437 .50544 .47473 .52305 .46913 .51341 .4587 Metetra
.645 .763 .934 r
.51341 .4587 .52305 .46913 .54076 .4635 .5312 .453 Metetra
.645 .763 .934 r
.5312 .453 .54076 .4635 .5586 .45783 .54911 .44726 Metetra
.645 .763 .934 r
.54911 .44726 .5586 .45783 .57655 .45212 .56714 .44148 Metetra
.645 .763 .934 r
.56714 .44148 .57655 .45212 .59461 .44637 .58528 .43567 Metetra
.645 .763 .934 r
.58528 .43567 .59461 .44637 .61279 .44059 .60354 .42981 Metetra
.645 .763 .934 r
.60354 .42981 .61279 .44059 .6311 .43477 .62192 .42392 Metetra
.645 .763 .934 r
.62192 .42392 .6311 .43477 .64952 .42891 .64043 .41799 Metetra
.645 .763 .934 r
.64043 .41799 .64952 .42891 .66806 .42301 .65906 .41202 Metetra
.645 .763 .934 r
.65906 .41202 .66806 .42301 .68673 .41707 .67781 .40601 Metetra
.645 .763 .934 r
.67781 .40601 .68673 .41707 .70552 .4111 .69668 .39996 Metetra
.645 .763 .934 r
.69668 .39996 .70552 .4111 .72443 .40508 .71569 .39387 Metetra
.645 .763 .934 r
.71569 .39387 .72443 .40508 .74348 .39903 .73482 .38774 Metetra
.645 .763 .934 r
.73482 .38774 .74348 .39903 .76265 .39293 .75408 .38157 Metetra
.645 .763 .934 r
.75408 .38157 .76265 .39293 .78194 .38679 .77347 .37535 Metetra
.988 .732 .448 r
.77347 .37535 .78194 .38679 .79061 .25594 .78249 .24412 Metetra
.645 .763 .934 r
.10715 .57621 .1184 .5853 .13372 .58039 .12252 .57125 Metetra
.645 .763 .934 r
.12252 .57125 .13372 .58039 .14913 .57545 .13799 .56625 Metetra
.645 .763 .934 r
.13799 .56625 .14913 .57545 .16464 .57048 .15355 .56123 Metetra
.645 .763 .934 r
.15355 .56123 .16464 .57048 .18024 .56548 .1692 .55617 Metetra
.645 .763 .934 r
.1692 .55617 .18024 .56548 .19593 .56045 .18495 .55109 Metetra
.645 .763 .934 r
.18495 .55109 .19593 .56045 .21172 .55539 .2008 .54597 Metetra
.645 .763 .934 r
.2008 .54597 .21172 .55539 .22761 .5503 .21674 .54082 Metetra
.645 .763 .934 r
.21674 .54082 .22761 .5503 .2436 .54518 .23278 .53563 Metetra
.645 .763 .934 r
.23278 .53563 .2436 .54518 .25968 .54002 .24893 .53042 Metetra
.645 .763 .934 r
.24893 .53042 .25968 .54002 .27586 .53483 .26517 .52517 Metetra
.645 .763 .934 r
.26517 .52517 .27586 .53483 .29215 .52962 .28151 .5199 Metetra
.645 .763 .934 r
.28151 .5199 .29215 .52962 .30853 .52436 .29796 .51458 Metetra
.645 .763 .934 r
.29796 .51458 .30853 .52436 .32502 .51908 .31451 .50924 Metetra
.645 .763 .934 r
.31451 .50924 .32502 .51908 .34161 .51376 .33116 .50386 Metetra
.645 .763 .934 r
.33116 .50386 .34161 .51376 .3583 .50841 .34792 .49845 Metetra
.645 .763 .934 r
.34792 .49845 .3583 .50841 .3751 .50303 .36479 .493 Metetra
.645 .763 .934 r
.36479 .493 .3751 .50303 .39201 .49761 .38176 .48752 Metetra
.645 .763 .934 r
.38176 .48752 .39201 .49761 .40902 .49216 .39884 .482 Metetra
.645 .763 .934 r
.39884 .482 .40902 .49216 .42614 .48667 .41603 .47645 Metetra
.645 .763 .934 r
.41603 .47645 .42614 .48667 .44337 .48115 .43333 .47086 Metetra
.645 .763 .934 r
.43333 .47086 .44337 .48115 .46071 .47559 .45074 .46524 Metetra
.645 .763 .934 r
.45074 .46524 .46071 .47559 .47817 .47 .46827 .45957 Metetra
.645 .763 .934 r
.46827 .45957 .47817 .47 .49573 .46437 .4859 .45388 Metetra
.645 .763 .934 r
.4859 .45388 .49573 .46437 .51341 .4587 .50366 .44814 Metetra
.645 .763 .934 r
.50366 .44814 .51341 .4587 .5312 .453 .52152 .44237 Metetra
.645 .763 .934 r
.52152 .44237 .5312 .453 .54911 .44726 .53951 .43656 Metetra
.645 .763 .934 r
.53951 .43656 .54911 .44726 .56714 .44148 .55761 .43072 Metetra
.645 .763 .934 r
.55761 .43072 .56714 .44148 .58528 .43567 .57583 .42483 Metetra
.645 .763 .934 r
.57583 .42483 .58528 .43567 .60354 .42981 .59418 .41891 Metetra
.645 .763 .934 r
.59418 .41891 .60354 .42981 .62192 .42392 .61264 .41294 Metetra
.645 .763 .934 r
.61264 .41294 .62192 .42392 .64043 .41799 .63123 .40694 Metetra
.645 .763 .934 r
.63123 .40694 .64043 .41799 .65906 .41202 .64994 .40089 Metetra
.645 .763 .934 r
.64994 .40089 .65906 .41202 .67781 .40601 .66878 .39481 Metetra
.645 .763 .934 r
.66878 .39481 .67781 .40601 .69668 .39996 .68774 .38869 Metetra
.645 .763 .934 r
.68774 .38869 .69668 .39996 .71569 .39387 .70683 .38252 Metetra
.645 .763 .934 r
.70683 .38252 .71569 .39387 .73482 .38774 .72605 .37631 Metetra
.645 .763 .934 r
.72605 .37631 .73482 .38774 .75408 .38157 .7454 .37006 Metetra
.645 .763 .934 r
.7454 .37006 .75408 .38157 .77347 .37535 .76488 .36377 Metetra
.988 .732 .448 r
.76488 .36377 .77347 .37535 .78249 .24412 .77427 .23216 Metetra
.645 .763 .934 r
.09578 .56702 .10715 .57621 .12252 .57125 .1112 .562 Metetra
.645 .763 .934 r
.1112 .562 .12252 .57125 .13799 .56625 .12672 .55695 Metetra
.645 .763 .934 r
.12672 .55695 .13799 .56625 .15355 .56123 .14233 .55187 Metetra
.645 .763 .934 r
.14233 .55187 .15355 .56123 .1692 .55617 .15804 .54676 Metetra
.645 .763 .934 r
.15804 .54676 .1692 .55617 .18495 .55109 .17385 .54161 Metetra
.645 .763 .934 r
.17385 .54161 .18495 .55109 .2008 .54597 .18975 .53643 Metetra
.645 .763 .934 r
.18975 .53643 .2008 .54597 .21674 .54082 .20575 .53123 Metetra
.645 .763 .934 r
.20575 .53123 .21674 .54082 .23278 .53563 .22185 .52598 Metetra
.645 .763 .934 r
.22185 .52598 .23278 .53563 .24893 .53042 .23805 .52071 Metetra
.645 .763 .934 r
.23805 .52071 .24893 .53042 .26517 .52517 .25435 .5154 Metetra
.645 .763 .934 r
.25435 .5154 .26517 .52517 .28151 .5199 .27076 .51006 Metetra
.645 .763 .934 r
.27076 .51006 .28151 .5199 .29796 .51458 .28727 .50469 Metetra
.645 .763 .934 r
.28727 .50469 .29796 .51458 .31451 .50924 .30388 .49928 Metetra
.645 .763 .934 r
.30388 .49928 .31451 .50924 .33116 .50386 .3206 .49384 Metetra
.645 .763 .934 r
.3206 .49384 .33116 .50386 .34792 .49845 .33742 .48836 Metetra
.645 .763 .934 r
.33742 .48836 .34792 .49845 .36479 .493 .35435 .48285 Metetra
.645 .763 .934 r
.35435 .48285 .36479 .493 .38176 .48752 .37139 .4773 Metetra
.645 .763 .934 r
.37139 .4773 .38176 .48752 .39884 .482 .38854 .47172 Metetra
.645 .763 .934 r
.38854 .47172 .39884 .482 .41603 .47645 .4058 .4661 Metetra
.645 .763 .934 r
.4058 .4661 .41603 .47645 .43333 .47086 .42317 .46045 Metetra
.645 .763 .934 r
.42317 .46045 .43333 .47086 .45074 .46524 .44065 .45476 Metetra
.645 .763 .934 r
.44065 .45476 .45074 .46524 .46827 .45957 .45825 .44903 Metetra
.645 .763 .934 r
.45825 .44903 .46827 .45957 .4859 .45388 .47596 .44326 Metetra
.645 .763 .934 r
.47596 .44326 .4859 .45388 .50366 .44814 .49378 .43746 Metetra
.645 .763 .934 r
.49378 .43746 .50366 .44814 .52152 .44237 .51173 .43162 Metetra
.645 .763 .934 r
.51173 .43162 .52152 .44237 .53951 .43656 .52979 .42574 Metetra
.645 .763 .934 r
.52979 .42574 .53951 .43656 .55761 .43072 .54797 .41982 Metetra
.645 .763 .934 r
.54797 .41982 .55761 .43072 .57583 .42483 .56627 .41386 Metetra
.645 .763 .934 r
.56627 .41386 .57583 .42483 .59418 .41891 .5847 .40786 Metetra
.645 .763 .934 r
.5847 .40786 .59418 .41891 .61264 .41294 .60324 .40183 Metetra
.645 .763 .934 r
.60324 .40183 .61264 .41294 .63123 .40694 .62191 .39575 Metetra
.645 .763 .934 r
.62191 .39575 .63123 .40694 .64994 .40089 .64071 .38963 Metetra
.645 .763 .934 r
.64071 .38963 .64994 .40089 .66878 .39481 .65963 .38347 Metetra
.645 .763 .934 r
.65963 .38347 .66878 .39481 .68774 .38869 .67868 .37727 Metetra
.645 .763 .934 r
.67868 .37727 .68774 .38869 .70683 .38252 .69786 .37102 Metetra
.645 .763 .934 r
.69786 .37102 .70683 .38252 .72605 .37631 .71717 .36474 Metetra
.645 .763 .934 r
.71717 .36474 .72605 .37631 .7454 .37006 .73662 .35841 Metetra
.645 .763 .934 r
.73662 .35841 .7454 .37006 .76488 .36377 .75619 .35204 Metetra
.988 .732 .448 r
.75619 .35204 .76488 .36377 .77427 .23216 .76595 .22005 Metetra
.645 .763 .934 r
.08428 .55773 .09578 .56702 .1112 .562 .09975 .55265 Metetra
.645 .763 .934 r
.09975 .55265 .1112 .562 .12672 .55695 .11532 .54754 Metetra
.645 .763 .934 r
.11532 .54754 .12672 .55695 .14233 .55187 .13099 .5424 Metetra
.645 .763 .934 r
.13099 .5424 .14233 .55187 .15804 .54676 .14675 .53723 Metetra
.645 .763 .934 r
.14675 .53723 .15804 .54676 .17385 .54161 .16261 .53203 Metetra
.645 .763 .934 r
.16261 .53203 .17385 .54161 .18975 .53643 .17857 .52679 Metetra
.645 .763 .934 r
.17857 .52679 .18975 .53643 .20575 .53123 .19463 .52152 Metetra
.645 .763 .934 r
.19463 .52152 .20575 .53123 .22185 .52598 .21079 .51622 Metetra
.645 .763 .934 r
.21079 .51622 .22185 .52598 .23805 .52071 .22705 .51089 Metetra
.645 .763 .934 r
.22705 .51089 .23805 .52071 .25435 .5154 .24341 .50552 Metetra
.645 .763 .934 r
.24341 .50552 .25435 .5154 .27076 .51006 .25988 .50011 Metetra
.645 .763 .934 r
.25988 .50011 .27076 .51006 .28727 .50469 .27645 .49468 Metetra
.645 .763 .934 r
.27645 .49468 .28727 .50469 .30388 .49928 .29312 .48921 Metetra
.645 .763 .934 r
.29312 .48921 .30388 .49928 .3206 .49384 .3099 .4837 Metetra
.645 .763 .934 r
.3099 .4837 .3206 .49384 .33742 .48836 .32679 .47816 Metetra
.645 .763 .934 r
.32679 .47816 .33742 .48836 .35435 .48285 .34379 .47258 Metetra
.645 .763 .934 r
.34379 .47258 .35435 .48285 .37139 .4773 .3609 .46697 Metetra
.645 .763 .934 r
.3609 .46697 .37139 .4773 .38854 .47172 .37811 .46132 Metetra
.645 .763 .934 r
.37811 .46132 .38854 .47172 .4058 .4661 .39544 .45563 Metetra
.645 .763 .934 r
.39544 .45563 .4058 .4661 .42317 .46045 .41288 .44991 Metetra
.645 .763 .934 r
.41288 .44991 .42317 .46045 .44065 .45476 .43044 .44415 Metetra
.645 .763 .934 r
.43044 .44415 .44065 .45476 .45825 .44903 .44811 .43835 Metetra
.645 .763 .934 r
.44811 .43835 .45825 .44903 .47596 .44326 .46589 .43252 Metetra
.645 .763 .934 r
.46589 .43252 .47596 .44326 .49378 .43746 .48379 .42665 Metetra
.645 .763 .934 r
.48379 .42665 .49378 .43746 .51173 .43162 .50181 .42073 Metetra
.645 .763 .934 r
.50181 .42073 .51173 .43162 .52979 .42574 .51995 .41478 Metetra
.645 .763 .934 r
.51995 .41478 .52979 .42574 .54797 .41982 .53821 .40879 Metetra
.645 .763 .934 r
.53821 .40879 .54797 .41982 .56627 .41386 .5566 .40276 Metetra
.645 .763 .934 r
.5566 .40276 .56627 .41386 .5847 .40786 .5751 .39669 Metetra
.645 .763 .934 r
.5751 .39669 .5847 .40786 .60324 .40183 .59373 .39057 Metetra
.645 .763 .934 r
.59373 .39057 .60324 .40183 .62191 .39575 .61248 .38442 Metetra
.645 .763 .934 r
.61248 .38442 .62191 .39575 .64071 .38963 .63137 .37823 Metetra
.645 .763 .934 r
.63137 .37823 .64071 .38963 .65963 .38347 .65038 .37199 Metetra
.645 .763 .934 r
.65038 .37199 .65963 .38347 .67868 .37727 .66951 .36571 Metetra
.645 .763 .934 r
.66951 .36571 .67868 .37727 .69786 .37102 .68878 .35939 Metetra
.645 .763 .934 r
.68878 .35939 .69786 .37102 .71717 .36474 .70819 .35302 Metetra
.645 .763 .934 r
.70819 .35302 .71717 .36474 .73662 .35841 .72772 .34661 Metetra
.645 .763 .934 r
.72772 .34661 .73662 .35841 .75619 .35204 .74739 .34016 Metetra
.988 .732 .448 r
.74739 .34016 .75619 .35204 .76595 .22005 .75753 .2078 Metetra
.645 .763 .934 r
.07265 .54833 .08428 .55773 .09975 .55265 .08817 .5432 Metetra
.645 .763 .934 r
.08817 .5432 .09975 .55265 .11532 .54754 .1038 .53803 Metetra
.645 .763 .934 r
.1038 .53803 .11532 .54754 .13099 .5424 .11952 .53283 Metetra
.645 .763 .934 r
.11952 .53283 .13099 .5424 .14675 .53723 .13534 .5276 Metetra
.645 .763 .934 r
.13534 .5276 .14675 .53723 .16261 .53203 .15125 .52234 Metetra
.645 .763 .934 r
.15125 .52234 .16261 .53203 .17857 .52679 .16727 .51704 Metetra
.645 .763 .934 r
.16727 .51704 .17857 .52679 .19463 .52152 .18338 .51171 Metetra
.645 .763 .934 r
.18338 .51171 .19463 .52152 .21079 .51622 .1996 .50635 Metetra
.645 .763 .934 r
.1996 .50635 .21079 .51622 .22705 .51089 .21592 .50095 Metetra
.645 .763 .934 r
.21592 .50095 .22705 .51089 .24341 .50552 .23234 .49552 Metetra
.645 .763 .934 r
.23234 .49552 .24341 .50552 .25988 .50011 .24887 .49005 Metetra
.645 .763 .934 r
.24887 .49005 .25988 .50011 .27645 .49468 .2655 .48455 Metetra
.645 .763 .934 r
.2655 .48455 .27645 .49468 .29312 .48921 .28224 .47901 Metetra
.645 .763 .934 r
.28224 .47901 .29312 .48921 .3099 .4837 .29909 .47344 Metetra
.645 .763 .934 r
.29909 .47344 .3099 .4837 .32679 .47816 .31604 .46784 Metetra
.645 .763 .934 r
.31604 .46784 .32679 .47816 .34379 .47258 .3331 .46219 Metetra
.645 .763 .934 r
.3331 .46219 .34379 .47258 .3609 .46697 .35028 .45651 Metetra
.645 .763 .934 r
.35028 .45651 .3609 .46697 .37811 .46132 .36756 .4508 Metetra
.645 .763 .934 r
.36756 .4508 .37811 .46132 .39544 .45563 .38496 .44504 Metetra
.645 .763 .934 r
.38496 .44504 .39544 .45563 .41288 .44991 .40247 .43925 Metetra
.645 .763 .934 r
.40247 .43925 .41288 .44991 .43044 .44415 .4201 .43342 Metetra
.645 .763 .934 r
.4201 .43342 .43044 .44415 .44811 .43835 .43784 .42755 Metetra
.645 .763 .934 r
.43784 .42755 .44811 .43835 .46589 .43252 .4557 .42165 Metetra
.645 .763 .934 r
.4557 .42165 .46589 .43252 .48379 .42665 .47368 .4157 Metetra
.645 .763 .934 r
.47368 .4157 .48379 .42665 .50181 .42073 .49178 .40971 Metetra
.645 .763 .934 r
.49178 .40971 .50181 .42073 .51995 .41478 .51 .40369 Metetra
.645 .763 .934 r
.51 .40369 .51995 .41478 .53821 .40879 .52833 .39762 Metetra
.645 .763 .934 r
.52833 .39762 .53821 .40879 .5566 .40276 .5468 .39152 Metetra
.645 .763 .934 r
.5468 .39152 .5566 .40276 .5751 .39669 .56538 .38537 Metetra
.645 .763 .934 r
.56538 .38537 .5751 .39669 .59373 .39057 .5841 .37918 Metetra
.645 .763 .934 r
.5841 .37918 .59373 .39057 .61248 .38442 .60294 .37295 Metetra
.645 .763 .934 r
.60294 .37295 .61248 .38442 .63137 .37823 .6219 .36668 Metetra
.645 .763 .934 r
.6219 .36668 .63137 .37823 .65038 .37199 .641 .36036 Metetra
.645 .763 .934 r
.641 .36036 .65038 .37199 .66951 .36571 .66023 .354 Metetra
.645 .763 .934 r
.66023 .354 .66951 .36571 .68878 .35939 .67959 .3476 Metetra
.645 .763 .934 r
.67959 .3476 .68878 .35939 .70819 .35302 .69908 .34115 Metetra
.645 .763 .934 r
.69908 .34115 .70819 .35302 .72772 .34661 .71871 .33466 Metetra
.645 .763 .934 r
.71871 .33466 .72772 .34661 .74739 .34016 .73847 .32812 Metetra
.988 .732 .448 r
.73847 .32812 .74739 .34016 .75753 .2078 .74901 .19539 Metetra
.645 .763 .934 r
.06088 .53883 .07265 .54833 .08817 .5432 .07646 .53363 Metetra
.645 .763 .934 r
.07646 .53363 .08817 .5432 .1038 .53803 .09214 .52841 Metetra
.645 .763 .934 r
.09214 .52841 .1038 .53803 .11952 .53283 .10791 .52315 Metetra
.645 .763 .934 r
.10791 .52315 .11952 .53283 .13534 .5276 .12379 .51786 Metetra
.645 .763 .934 r
.12379 .51786 .13534 .5276 .15125 .52234 .13976 .51253 Metetra
.645 .763 .934 r
.13976 .51253 .15125 .52234 .16727 .51704 .15583 .50717 Metetra
.645 .763 .934 r
.15583 .50717 .16727 .51704 .18338 .51171 .17201 .50178 Metetra
.645 .763 .934 r
.17201 .50178 .18338 .51171 .1996 .50635 .18828 .49635 Metetra
.645 .763 .934 r
.18828 .49635 .1996 .50635 .21592 .50095 .20466 .49089 Metetra
.645 .763 .934 r
.20466 .49089 .21592 .50095 .23234 .49552 .22114 .4854 Metetra
.645 .763 .934 r
.22114 .4854 .23234 .49552 .24887 .49005 .23773 .47987 Metetra
.645 .763 .934 r
.23773 .47987 .24887 .49005 .2655 .48455 .25443 .4743 Metetra
.645 .763 .934 r
.25443 .4743 .2655 .48455 .28224 .47901 .27123 .4687 Metetra
.645 .763 .934 r
.27123 .4687 .28224 .47901 .29909 .47344 .28814 .46306 Metetra
.645 .763 .934 r
.28814 .46306 .29909 .47344 .31604 .46784 .30516 .45739 Metetra
.645 .763 .934 r
.30516 .45739 .31604 .46784 .3331 .46219 .32229 .45168 Metetra
.645 .763 .934 r
.32229 .45168 .3331 .46219 .35028 .45651 .33953 .44593 Metetra
.645 .763 .934 r
.33953 .44593 .35028 .45651 .36756 .4508 .35689 .44014 Metetra
.645 .763 .934 r
.35689 .44014 .36756 .4508 .38496 .44504 .37436 .43432 Metetra
.645 .763 .934 r
.37436 .43432 .38496 .44504 .40247 .43925 .39194 .42846 Metetra
.645 .763 .934 r
.39194 .42846 .40247 .43925 .4201 .43342 .40964 .42256 Metetra
.645 .763 .934 r
.40964 .42256 .4201 .43342 .43784 .42755 .42745 .41662 Metetra
.645 .763 .934 r
.42745 .41662 .43784 .42755 .4557 .42165 .44539 .41064 Metetra
.645 .763 .934 r
.44539 .41064 .4557 .42165 .47368 .4157 .46344 .40462 Metetra
.645 .763 .934 r
.46344 .40462 .47368 .4157 .49178 .40971 .48162 .39856 Metetra
.645 .763 .934 r
.48162 .39856 .49178 .40971 .51 .40369 .49991 .39246 Metetra
.645 .763 .934 r
.49991 .39246 .51 .40369 .52833 .39762 .51833 .38632 Metetra
.645 .763 .934 r
.51833 .38632 .52833 .39762 .5468 .39152 .53688 .38014 Metetra
.645 .763 .934 r
.53688 .38014 .5468 .39152 .56538 .38537 .55555 .37391 Metetra
.645 .763 .934 r
.55555 .37391 .56538 .38537 .5841 .37918 .57434 .36765 Metetra
.645 .763 .934 r
.57434 .36765 .5841 .37918 .60294 .37295 .59327 .36134 Metetra
.645 .763 .934 r
.59327 .36134 .60294 .37295 .6219 .36668 .61232 .35498 Metetra
.645 .763 .934 r
.61232 .35498 .6219 .36668 .641 .36036 .63151 .34859 Metetra
.645 .763 .934 r
.63151 .34859 .641 .36036 .66023 .354 .65082 .34215 Metetra
.645 .763 .934 r
.65082 .34215 .66023 .354 .67959 .3476 .67028 .33566 Metetra
.645 .763 .934 r
.67028 .33566 .67959 .3476 .69908 .34115 .68986 .32913 Metetra
.645 .763 .934 r
.68986 .32913 .69908 .34115 .71871 .33466 .70958 .32256 Metetra
.645 .763 .934 r
.70958 .32256 .71871 .33466 .73847 .32812 .72944 .31594 Metetra
.988 .732 .448 r
.72944 .31594 .73847 .32812 .74901 .19539 .74038 .18283 Metetra
.645 .763 .934 r
.04899 .52921 .06088 .53883 .07646 .53363 .06462 .52396 Metetra
.645 .763 .934 r
.06462 .52396 .07646 .53363 .09214 .52841 .08035 .51867 Metetra
.645 .763 .934 r
.08035 .51867 .09214 .52841 .10791 .52315 .09618 .51335 Metetra
.645 .763 .934 r
.09618 .51335 .10791 .52315 .12379 .51786 .1121 .508 Metetra
.645 .763 .934 r
.1121 .508 .12379 .51786 .13976 .51253 .12813 .50261 Metetra
.645 .763 .934 r
.12813 .50261 .13976 .51253 .15583 .50717 .14426 .49719 Metetra
.645 .763 .934 r
.14426 .49719 .15583 .50717 .17201 .50178 .16049 .49174 Metetra
.645 .763 .934 r
.16049 .49174 .17201 .50178 .18828 .49635 .17683 .48625 Metetra
.645 .763 .934 r
.17683 .48625 .18828 .49635 .20466 .49089 .19327 .48072 Metetra
.645 .763 .934 r
.19327 .48072 .20466 .49089 .22114 .4854 .20981 .47516 Metetra
.645 .763 .934 r
.20981 .47516 .22114 .4854 .23773 .47987 .22646 .46956 Metetra
.645 .763 .934 r
.22646 .46956 .23773 .47987 .25443 .4743 .24322 .46393 Metetra
.645 .763 .934 r
.24322 .46393 .25443 .4743 .27123 .4687 .26009 .45826 Metetra
.645 .763 .934 r
.26009 .45826 .27123 .4687 .28814 .46306 .27706 .45256 Metetra
.645 .763 .934 r
.27706 .45256 .28814 .46306 .30516 .45739 .29415 .44682 Metetra
.645 .763 .934 r
.29415 .44682 .30516 .45739 .32229 .45168 .31135 .44104 Metetra
.645 .763 .934 r
.31135 .44104 .32229 .45168 .33953 .44593 .32866 .43522 Metetra
.645 .763 .934 r
.32866 .43522 .33953 .44593 .35689 .44014 .34608 .42936 Metetra
.645 .763 .934 r
.34608 .42936 .35689 .44014 .37436 .43432 .36362 .42347 Metetra
.645 .763 .934 r
.36362 .42347 .37436 .43432 .39194 .42846 .38128 .41753 Metetra
.645 .763 .934 r
.38128 .41753 .39194 .42846 .40964 .42256 .39905 .41156 Metetra
.645 .763 .934 r
.39905 .41156 .40964 .42256 .42745 .41662 .41694 .40555 Metetra
.645 .763 .934 r
.41694 .40555 .42745 .41662 .44539 .41064 .43495 .39949 Metetra
.645 .763 .934 r
.43495 .39949 .44539 .41064 .46344 .40462 .45308 .3934 Metetra
.645 .763 .934 r
.45308 .3934 .46344 .40462 .48162 .39856 .47133 .38727 Metetra
.645 .763 .934 r
.47133 .38727 .48162 .39856 .49991 .39246 .48971 .38109 Metetra
.645 .763 .934 r
.48971 .38109 .49991 .39246 .51833 .38632 .50821 .37487 Metetra
.645 .763 .934 r
.50821 .37487 .51833 .38632 .53688 .38014 .52683 .36861 Metetra
.645 .763 .934 r
.52683 .36861 .53688 .38014 .55555 .37391 .54559 .36231 Metetra
.645 .763 .934 r
.54559 .36231 .55555 .37391 .57434 .36765 .56447 .35596 Metetra
.645 .763 .934 r
.56447 .35596 .57434 .36765 .59327 .36134 .58348 .34957 Metetra
.645 .763 .934 r
.58348 .34957 .59327 .36134 .61232 .35498 .60262 .34314 Metetra
.645 .763 .934 r
.60262 .34314 .61232 .35498 .63151 .34859 .62189 .33666 Metetra
.645 .763 .934 r
.62189 .33666 .63151 .34859 .65082 .34215 .6413 .33014 Metetra
.645 .763 .934 r
.6413 .33014 .65082 .34215 .67028 .33566 .66084 .32357 Metetra
.645 .763 .934 r
.66084 .32357 .67028 .33566 .68986 .32913 .68052 .31696 Metetra
.645 .763 .934 r
.68052 .31696 .68986 .32913 .70958 .32256 .70034 .3103 Metetra
.645 .763 .934 r
.70034 .3103 .70958 .32256 .72944 .31594 .7203 .30359 Metetra
.988 .732 .448 r
.7203 .30359 .72944 .31594 .74038 .18283 .73164 .17011 Metetra
.645 .763 .934 r
.03695 .51949 .04899 .52921 .06462 .52396 .05264 .51417 Metetra
.645 .763 .934 r
.05264 .51417 .06462 .52396 .08035 .51867 .06842 .50883 Metetra
.645 .763 .934 r
.06842 .50883 .08035 .51867 .09618 .51335 .0843 .50344 Metetra
.645 .763 .934 r
.0843 .50344 .09618 .51335 .1121 .508 .10029 .49803 Metetra
.645 .763 .934 r
.10029 .49803 .1121 .508 .12813 .50261 .11637 .49258 Metetra
.645 .763 .934 r
.11637 .49258 .12813 .50261 .14426 .49719 .13256 .48709 Metetra
.645 .763 .934 r
.13256 .48709 .14426 .49719 .16049 .49174 .14885 .48157 Metetra
.645 .763 .934 r
.14885 .48157 .16049 .49174 .17683 .48625 .16524 .47602 Metetra
.645 .763 .934 r
.16524 .47602 .17683 .48625 .19327 .48072 .18174 .47043 Metetra
.645 .763 .934 r
.18174 .47043 .19327 .48072 .20981 .47516 .19835 .4648 Metetra
.645 .763 .934 r
.19835 .4648 .20981 .47516 .22646 .46956 .21506 .45914 Metetra
.645 .763 .934 r
.21506 .45914 .22646 .46956 .24322 .46393 .23188 .45344 Metetra
.645 .763 .934 r
.23188 .45344 .24322 .46393 .26009 .45826 .24881 .4477 Metetra
.645 .763 .934 r
.24881 .4477 .26009 .45826 .27706 .45256 .26585 .44193 Metetra
.645 .763 .934 r
.26585 .44193 .27706 .45256 .29415 .44682 .283 .43612 Metetra
.645 .763 .934 r
.283 .43612 .29415 .44682 .31135 .44104 .30027 .43026 Metetra
.645 .763 .934 r
.30027 .43026 .31135 .44104 .32866 .43522 .31765 .42438 Metetra
.645 .763 .934 r
.31765 .42438 .32866 .43522 .34608 .42936 .33514 .41845 Metetra
.645 .763 .934 r
.33514 .41845 .34608 .42936 .36362 .42347 .35275 .41248 Metetra
.645 .763 .934 r
.35275 .41248 .36362 .42347 .38128 .41753 .37048 .40647 Metetra
.645 .763 .934 r
.37048 .40647 .38128 .41753 .39905 .41156 .38833 .40043 Metetra
.645 .763 .934 r
.38833 .40043 .39905 .41156 .41694 .40555 .40629 .39434 Metetra
.645 .763 .934 r
.40629 .39434 .41694 .40555 .43495 .39949 .42438 .38821 Metetra
.645 .763 .934 r
.42438 .38821 .43495 .39949 .45308 .3934 .44259 .38204 Metetra
.645 .763 .934 r
.44259 .38204 .45308 .3934 .47133 .38727 .46092 .37583 Metetra
.645 .763 .934 r
.46092 .37583 .47133 .38727 .48971 .38109 .47937 .36958 Metetra
.645 .763 .934 r
.47937 .36958 .48971 .38109 .50821 .37487 .49795 .36328 Metetra
.645 .763 .934 r
.49795 .36328 .50821 .37487 .52683 .36861 .51666 .35694 Metetra
.645 .763 .934 r
.51666 .35694 .52683 .36861 .54559 .36231 .5355 .35056 Metetra
.645 .763 .934 r
.5355 .35056 .54559 .36231 .56447 .35596 .55446 .34413 Metetra
.645 .763 .934 r
.55446 .34413 .56447 .35596 .58348 .34957 .57356 .33766 Metetra
.645 .763 .934 r
.57356 .33766 .58348 .34957 .60262 .34314 .59279 .33115 Metetra
.645 .763 .934 r
.59279 .33115 .60262 .34314 .62189 .33666 .61216 .32459 Metetra
.645 .763 .934 r
.61216 .32459 .62189 .33666 .6413 .33014 .63165 .31798 Metetra
.645 .763 .934 r
.63165 .31798 .6413 .33014 .66084 .32357 .65129 .31133 Metetra
.645 .763 .934 r
.65129 .31133 .66084 .32357 .68052 .31696 .67106 .30463 Metetra
.645 .763 .934 r
.67106 .30463 .68052 .31696 .70034 .3103 .69098 .29788 Metetra
.645 .763 .934 r
.69098 .29788 .70034 .3103 .7203 .30359 .71103 .29108 Metetra
.988 .732 .448 r
.71103 .29108 .7203 .30359 .73164 .17011 .72279 .15723 Metetra
.645 .763 .934 r
.02478 .50965 .03695 .51949 .05264 .51417 .04051 .50427 Metetra
.645 .763 .934 r
.04051 .50427 .05264 .51417 .06842 .50883 .05635 .49886 Metetra
.645 .763 .934 r
.05635 .49886 .06842 .50883 .0843 .50344 .07229 .49342 Metetra
.645 .763 .934 r
.07229 .49342 .0843 .50344 .10029 .49803 .08833 .48794 Metetra
.645 .763 .934 r
.08833 .48794 .10029 .49803 .11637 .49258 .10447 .48242 Metetra
.645 .763 .934 r
.10447 .48242 .11637 .49258 .13256 .48709 .12071 .47688 Metetra
.645 .763 .934 r
.12071 .47688 .13256 .48709 .14885 .48157 .13706 .47129 Metetra
.645 .763 .934 r
.13706 .47129 .14885 .48157 .16524 .47602 .15352 .46567 Metetra
.645 .763 .934 r
.15352 .46567 .16524 .47602 .18174 .47043 .17008 .46001 Metetra
.645 .763 .934 r
.17008 .46001 .18174 .47043 .19835 .4648 .18674 .45432 Metetra
.645 .763 .934 r
.18674 .45432 .19835 .4648 .21506 .45914 .20352 .44859 Metetra
.645 .763 .934 r
.20352 .44859 .21506 .45914 .23188 .45344 .2204 .44282 Metetra
.645 .763 .934 r
.2204 .44282 .23188 .45344 .24881 .4477 .2374 .43701 Metetra
.645 .763 .934 r
.2374 .43701 .24881 .4477 .26585 .44193 .25451 .43117 Metetra
.645 .763 .934 r
.25451 .43117 .26585 .44193 .283 .43612 .27172 .42528 Metetra
.645 .763 .934 r
.27172 .42528 .283 .43612 .30027 .43026 .28906 .41936 Metetra
.645 .763 .934 r
.28906 .41936 .30027 .43026 .31765 .42438 .30651 .4134 Metetra
.645 .763 .934 r
.30651 .4134 .31765 .42438 .33514 .41845 .32407 .4074 Metetra
.645 .763 .934 r
.32407 .4074 .33514 .41845 .35275 .41248 .34175 .40136 Metetra
.645 .763 .934 r
.34175 .40136 .35275 .41248 .37048 .40647 .35955 .39528 Metetra
.645 .763 .934 r
.35955 .39528 .37048 .40647 .38833 .40043 .37747 .38916 Metetra
.645 .763 .934 r
.37747 .38916 .38833 .40043 .40629 .39434 .39551 .38299 Metetra
.645 .763 .934 r
.39551 .38299 .40629 .39434 .42438 .38821 .41367 .37679 Metetra
.645 .763 .934 r
.41367 .37679 .42438 .38821 .44259 .38204 .43196 .37054 Metetra
.645 .763 .934 r
.43196 .37054 .44259 .38204 .46092 .37583 .45037 .36425 Metetra
.645 .763 .934 r
.45037 .36425 .46092 .37583 .47937 .36958 .46891 .35792 Metetra
.645 .763 .934 r
.46891 .35792 .47937 .36958 .49795 .36328 .48757 .35154 Metetra
.645 .763 .934 r
.48757 .35154 .49795 .36328 .51666 .35694 .50636 .34512 Metetra
.645 .763 .934 r
.50636 .34512 .51666 .35694 .5355 .35056 .52528 .33866 Metetra
.645 .763 .934 r
.52528 .33866 .5355 .35056 .55446 .34413 .54433 .33215 Metetra
.645 .763 .934 r
.54433 .33215 .55446 .34413 .57356 .33766 .56352 .3256 Metetra
.645 .763 .934 r
.56352 .3256 .57356 .33766 .59279 .33115 .58284 .319 Metetra
.645 .763 .934 r
.58284 .319 .59279 .33115 .61216 .32459 .60229 .31235 Metetra
.645 .763 .934 r
.60229 .31235 .61216 .32459 .63165 .31798 .62188 .30566 Metetra
.645 .763 .934 r
.62188 .30566 .63165 .31798 .65129 .31133 .64161 .29892 Metetra
.645 .763 .934 r
.64161 .29892 .65129 .31133 .67106 .30463 .66148 .29213 Metetra
.645 .763 .934 r
.66148 .29213 .67106 .30463 .69098 .29788 .68149 .2853 Metetra
.645 .763 .934 r
.68149 .2853 .69098 .29788 .71103 .29108 .70164 .27841 Metetra
.988 .732 .448 r
.70164 .27841 .71103 .29108 .72279 .15723 .71383 .14419 Metetra
.645 .763 .934 r
.01246 .4997 .02478 .50965 .04051 .50427 .02825 .49426 Metetra
.645 .763 .934 r
.02825 .49426 .04051 .50427 .05635 .49886 .04414 .48878 Metetra
.645 .763 .934 r
.04414 .48878 .05635 .49886 .07229 .49342 .06013 .48328 Metetra
.645 .763 .934 r
.06013 .48328 .07229 .49342 .08833 .48794 .07623 .47773 Metetra
.645 .763 .934 r
.07623 .47773 .08833 .48794 .10447 .48242 .09243 .47215 Metetra
.645 .763 .934 r
.09243 .47215 .10447 .48242 .12071 .47688 .10873 .46654 Metetra
.645 .763 .934 r
.10873 .46654 .12071 .47688 .13706 .47129 .12514 .46088 Metetra
.645 .763 .934 r
.12514 .46088 .13706 .47129 .15352 .46567 .14165 .4552 Metetra
.645 .763 .934 r
.14165 .4552 .15352 .46567 .17008 .46001 .15827 .44947 Metetra
.645 .763 .934 r
.15827 .44947 .17008 .46001 .18674 .45432 .175 .44371 Metetra
.645 .763 .934 r
.175 .44371 .18674 .45432 .20352 .44859 .19184 .43791 Metetra
.645 .763 .934 r
.19184 .43791 .20352 .44859 .2204 .44282 .20879 .43207 Metetra
.645 .763 .934 r
.20879 .43207 .2204 .44282 .2374 .43701 .22585 .42619 Metetra
.645 .763 .934 r
.22585 .42619 .2374 .43701 .25451 .43117 .24302 .42028 Metetra
.645 .763 .934 r
.24302 .42028 .25451 .43117 .27172 .42528 .26031 .41432 Metetra
.645 .763 .934 r
.26031 .41432 .27172 .42528 .28906 .41936 .27771 .40833 Metetra
.645 .763 .934 r
.27771 .40833 .28906 .41936 .30651 .4134 .29523 .40229 Metetra
.645 .763 .934 r
.29523 .40229 .30651 .4134 .32407 .4074 .31286 .39622 Metetra
.645 .763 .934 r
.31286 .39622 .32407 .4074 .34175 .40136 .33062 .3901 Metetra
.645 .763 .934 r
.33062 .3901 .34175 .40136 .35955 .39528 .34849 .38395 Metetra
.645 .763 .934 r
.34849 .38395 .35955 .39528 .37747 .38916 .36648 .37775 Metetra
.645 .763 .934 r
.36648 .37775 .37747 .38916 .39551 .38299 .3846 .37151 Metetra
.645 .763 .934 r
.3846 .37151 .39551 .38299 .41367 .37679 .40284 .36522 Metetra
.645 .763 .934 r
.40284 .36522 .41367 .37679 .43196 .37054 .4212 .3589 Metetra
.645 .763 .934 r
.4212 .3589 .43196 .37054 .45037 .36425 .43969 .35253 Metetra
.645 .763 .934 r
.43969 .35253 .45037 .36425 .46891 .35792 .45831 .34612 Metetra
.645 .763 .934 r
.45831 .34612 .46891 .35792 .48757 .35154 .47705 .33966 Metetra
.645 .763 .934 r
.47705 .33966 .48757 .35154 .50636 .34512 .49593 .33316 Metetra
.645 .763 .934 r
.49593 .33316 .50636 .34512 .52528 .33866 .51494 .32661 Metetra
.645 .763 .934 r
.51494 .32661 .52528 .33866 .54433 .33215 .53407 .32002 Metetra
.645 .763 .934 r
.53407 .32002 .54433 .33215 .56352 .3256 .55335 .31338 Metetra
.645 .763 .934 r
.55335 .31338 .56352 .3256 .58284 .319 .57276 .30669 Metetra
.645 .763 .934 r
.57276 .30669 .58284 .319 .60229 .31235 .5923 .29996 Metetra
.645 .763 .934 r
.5923 .29996 .60229 .31235 .62188 .30566 .61198 .29318 Metetra
.645 .763 .934 r
.61198 .29318 .62188 .30566 .64161 .29892 .63181 .28635 Metetra
.645 .763 .934 r
.63181 .28635 .64161 .29892 .66148 .29213 .65177 .27947 Metetra
.645 .763 .934 r
.65177 .27947 .66148 .29213 .68149 .2853 .67188 .27255 Metetra
.645 .763 .934 r
.67188 .27255 .68149 .2853 .70164 .27841 .69213 .26557 Metetra
.988 .732 .448 r
.69213 .26557 .70164 .27841 .71383 .14419 .70475 .13098 Metetra
.645 .763 .934 r
0 .48963 .01246 .4997 .02825 .49426 .01584 .48413 Metetra
.645 .763 .934 r
.01584 .48413 .02825 .49426 .04414 .48878 .03179 .47859 Metetra
.645 .763 .934 r
.03179 .47859 .04414 .48878 .06013 .48328 .04784 .47301 Metetra
.645 .763 .934 r
.04784 .47301 .06013 .48328 .07623 .47773 .06399 .4674 Metetra
.645 .763 .934 r
.06399 .4674 .07623 .47773 .09243 .47215 .08024 .46176 Metetra
.645 .763 .934 r
.08024 .46176 .09243 .47215 .10873 .46654 .0966 .45607 Metetra
.645 .763 .934 r
.0966 .45607 .10873 .46654 .12514 .46088 .11307 .45035 Metetra
.645 .763 .934 r
.11307 .45035 .12514 .46088 .14165 .4552 .12964 .4446 Metetra
.645 .763 .934 r
.12964 .4446 .14165 .4552 .15827 .44947 .14632 .4388 Metetra
.645 .763 .934 r
.14632 .4388 .15827 .44947 .175 .44371 .16311 .43297 Metetra
.645 .763 .934 r
.16311 .43297 .175 .44371 .19184 .43791 .18002 .4271 Metetra
.645 .763 .934 r
.18002 .4271 .19184 .43791 .20879 .43207 .19703 .42119 Metetra
.645 .763 .934 r
.19703 .42119 .20879 .43207 .22585 .42619 .21415 .41524 Metetra
.645 .763 .934 r
.21415 .41524 .22585 .42619 .24302 .42028 .23139 .40925 Metetra
.645 .763 .934 r
.23139 .40925 .24302 .42028 .26031 .41432 .24875 .40322 Metetra
.645 .763 .934 r
.24875 .40322 .26031 .41432 .27771 .40833 .26622 .39716 Metetra
.645 .763 .934 r
.26622 .39716 .27771 .40833 .29523 .40229 .28381 .39105 Metetra
.645 .763 .934 r
.28381 .39105 .29523 .40229 .31286 .39622 .30151 .3849 Metetra
.645 .763 .934 r
.30151 .3849 .31286 .39622 .33062 .3901 .31934 .3787 Metetra
.645 .763 .934 r
.31934 .3787 .33062 .3901 .34849 .38395 .33729 .37247 Metetra
.645 .763 .934 r
.33729 .37247 .34849 .38395 .36648 .37775 .35535 .36619 Metetra
.645 .763 .934 r
.35535 .36619 .36648 .37775 .3846 .37151 .37355 .35987 Metetra
.645 .763 .934 r
.37355 .35987 .3846 .37151 .40284 .36522 .39186 .35351 Metetra
.645 .763 .934 r
.39186 .35351 .40284 .36522 .4212 .3589 .41031 .3471 Metetra
.645 .763 .934 r
.41031 .3471 .4212 .3589 .43969 .35253 .42888 .34065 Metetra
.645 .763 .934 r
.42888 .34065 .43969 .35253 .45831 .34612 .44757 .33416 Metetra
.645 .763 .934 r
.44757 .33416 .45831 .34612 .47705 .33966 .4664 .32762 Metetra
.645 .763 .934 r
.4664 .32762 .47705 .33966 .49593 .33316 .48536 .32103 Metetra
.645 .763 .934 r
.48536 .32103 .49593 .33316 .51494 .32661 .50445 .3144 Metetra
.645 .763 .934 r
.50445 .3144 .51494 .32661 .53407 .32002 .52368 .30772 Metetra
.645 .763 .934 r
.52368 .30772 .53407 .32002 .55335 .31338 .54304 .301 Metetra
.645 .763 .934 r
.54304 .301 .55335 .31338 .57276 .30669 .56254 .29423 Metetra
.645 .763 .934 r
.56254 .29423 .57276 .30669 .5923 .29996 .58218 .2874 Metetra
.645 .763 .934 r
.58218 .2874 .5923 .29996 .61198 .29318 .60195 .28053 Metetra
.645 .763 .934 r
.60195 .28053 .61198 .29318 .63181 .28635 .62187 .27362 Metetra
.645 .763 .934 r
.62187 .27362 .63181 .28635 .65177 .27947 .64193 .26665 Metetra
.645 .763 .934 r
.64193 .26665 .65177 .27947 .67188 .27255 .66214 .25963 Metetra
.645 .763 .934 r
.66214 .25963 .67188 .27255 .69213 .26557 .68249 .25256 Metetra
.988 .732 .448 r
.68249 .25256 .69213 .26557 .70475 .13098 .69556 .11761 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{247.5, 152.75},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004e0000_Q000`40O003h00Oogoo=Woo003oOolf
Ool00<9oo`@000Aoo`<006Qoo`00aWoo00@007ooOol000=oo`03001oogoo06Eoo`00aWoo00@007oo
Ool000=oo`03001oogoo06Eoo`00aWoo00@007ooOol000=oo`03001oogoo06Eoo`00`Woo10000goo
00D007ooOomoo`0006Moo`00`Woo00<007ooOol017oo00D007ooOomoo`0006Moo`00`Woo00<007oo
Ool017oo00D007ooOomoo`0006Moo`00`Woo100017oo0`00J7oo003oOolfOol00?moocIoo`00ogoo
=Woo003=Ool2001VOol00<Yoo`<00003Ool0000006Eoo`00b7oo0P0017oo0P00IGoo0035Ool30006
Ool00`00Ool0001TOol009]oo`<000Aoo`@001eoo`80009oo`03001oogoo00Aoo`03001oo`0006Ao
o`00W7oo00<007ooOol01goo00<007ooOol05goo0`002goo00@007ooOol006=oo`00VGoo1@0027oo
00<007ooOol05Goo0P003Woo00D007ooOomoo`00069oo`00VGoo00@007ooOol000Uoo`03001oogoo
019oo`<00003Ool007oo00eoo`05001oogooOol0001ROol009Yoo`03001oo`0000Eoo`@001=oo`80
01=oo`03001oogoo009oo`03001oogoo05moo`00VWoo00<007oo00001Goo00<007ooOol04Goo0`00
5Goo00<007ooOol00goo00<007ooOol0GWoo002KOol20005Ool00`00Oomoo`0?Ool200000goo001o
o`0FOol00`00Oomoo`02Ool20007Ool30006Ool20005Ool50013Ool009aoo`03001oogoo00=oo`@0
00]oo`<001]oo`03001oogoo00=oo`03001oogoo00Aoo`05001oogooOol00005Ool20007Ool00`00
Oomoo`13Ool00:moo`8001ioo`03001oogoo00Aoo`03001oogoo00=oo`05001oogooOol0000>Ool0
0`00Oomoo`13Ool00:aoo`@001moo`03001oogoo00Aoo`03001oogoo00=oo`05001oogooOol0000>
Ool00`00Oomoo`13Ool00:Uoo`<002=oo`03001oogoo00Eoo`03001oogoo009oo`05001oogooOol0
000>Ool00`00Oomoo`13Ool00:Moo`8002Ioo`03001oogoo00Ioo`05001oogooOol00003Ool00`00
Oomoo`0<Ool00`00Oomoo`13Ool00:Aoo`@002Moo`03001oogoo00Ioo`80009oo`05001oogooOol0
000>Ool00`00Oomoo`13Ool00:9oo`8000Aoo`03001oogoo02Aoo`03001oogoo00Eoo`03001oo`00
00=oo`<000eoo`<004Eoo`00MWoo0`0017oo0`007goo0`00;Goo00<007ooOol01goo00<007ooOol0
FGoo001gOol00`00Oomoo`02Ool01@00Oomoogoo000077oo0P0000=oo`00Ool0;Goo00<007ooOol0
27oo00<007ooOol0F7oo001dOol50003Ool01@00Oomoogoo00006Goo0`00<Woo00<007ooOol02Goo
00<007ooOol0Egoo001dOol01000Oomoo`0017oo00D007ooOomoo`0001Moo`8003Eoo`03001oogoo
00Uoo`03001oogoo05Moo`00MGoo00<007oo000017oo00D007ooOomoo`0001Aoo`<00003Ool007oo
03Aoo`03001oogoo00Yoo`8005Moo`00MGoo00<007oo000017oo00D007ooOomoo`00019oo`8003Yo
o`03001oogoo00Yoo`8005Moo`00MWoo0P0017oo00D007ooOomoo`0000moo`<003aoo`03001oogoo
00]oo`03001oogoo05Eoo`00Mgoo00<007ooOol00goo0`003Woo0P000Woo00<007ooOol0>Woo00<0
07ooOol037oo00<007ooOol0E7oo002;Ool30011Ool2000>Ool00`00Oomoo`1COol008Uoo`<004=o
o`8000ioo`03001oogoo05=oo`00QWoo0`00Agoo0P003Woo00<007ooOol0DWoo0024Ool2001:Ool0
0`00O^h0000=Ool2001COol0085oo`<004aoo`03001nkP0000ioo`03001oogoo055oo`00Ogoo0P00
0goo00<007ooOol0BGoo00@007k^000000ioo`03001oogoo051oo`00DWoo0`001Goo10007Woo0`00
D7oo0P0000AnkP00O^h000ioo`03001oogoo04moo`00DGoo00D007ooOomoo`0000Qoo`03001oogoo
01Uoo`800003Ool007oo051oo`800003O^h007k^008000eoo`03001oogoo04moo`00EGoo00<007oo
Ool01Woo00<007ooOol05Woo0`00EGoo0P0000InkP00O^h007k^000<Ool2001@Ool005Eoo`03001o
ogoo00Ioo`03001oogoo01Aoo`8005Qoo`<0009nkP03001nkP0000ioo`03001oogoo04eoo`00Dgoo
0P001Goo100057oo0`0000=oo`00Ool0Egoo0`000Wk^00@007k^000000eoo`03001oogoo04eoo`00
EGoo00<007ooOol00Woo00<007ooOol04goo0P00GGoo0`000Wk^00D007k^001nkP0000eoo`03001o
ogoo04aoo`00DGoo00D007ooOomoo`0000Aoo`03001oogoo011oo`<005moo`<00007O^h007k^O^h0
07k^00003Woo00<007ooOol0Bgoo001BOol30005Ool4000=Ool2001ROol300001Wk^001nkWk^001n
kP8000eoo`03001oogoo00Ioo`<000Ioo`8000Aoo`D0035oo`00Igoo1000I7oo0`0000AnkP00O^h0
009nkP8000eoo`8000Ioo`05001oogooOol00005Ool20005Ool01000Oomoo`00<Goo001VOol3001U
Ool02000O^h00000O^h007k^0002O^h2000?Ool00`00Oomoo`03Ool01@00Oomoogoo00003Goo00<0
07ooOol0<Goo001SOol3001XOol02000O^h00000O^h007k^0002O^h3000>Ool00`00Oomoo`03Ool0
1@00Oomoogoo00003Woo00<007ooOol0<7oo001POol3001[Ool03P00O^h00000O^h007k^001nkP00
O^h007k^000>Ool00`00Oomoo`02Ool01@00Oomoogoo00003goo00<007ooOol0;goo001NOol20002
Ool00`00Oomoo`1YOol00`00O^h00002O^h02000O^h007k^001nkP00O^h2000>Ool01@00Oomoogoo
00000goo00<007ooOol03Goo00<007ooOol0;goo000aOol30005Ool3000OOol3001`Ool00`00O^h0
0002O^h02`00O^h007k^001nkP00O^h007k^00003Goo0P000Woo00D007ooOomoo`0000]oo`05001o
ogooOol0000aOol0031oo`05001oogooOol00003Ool01@00Oomoogoo000077oo0`00LWoo010007k^
000007k^001nkP00O^h007k^001nkP00O^h000aoo`03001oo`0000=oo`<000eoo`<0039oo`00=7oo
00D007ooOomoo`0000=oo`03001oogoo01Moo`<007Eoo`03001nkP000080009nkP08001nkP00O^h0
07k^001nkP8000ioo`03001oogoo04Eoo`00=7oo00D007ooOomoo`0000=oo`03001oogoo01Eoo`80
07Qoo`03001nkP000080009nkP04001nkP000002O^h01@00O^h007k^00003Goo00<007ooOol0AGoo
000bOol20004Ool01@00Oomoogoo000057oo1000NGoo00<007k^00000P0000AnkP00O^inkP80009n
kP05001nkP00O^h0000>Ool00`00Oomoo`14Ool003Aoo`05001oogooOol00003Ool00`00Oomoo`0@
Ool2001mOol00`00O^h00002000017k^001nkWk^0P000Wk^00@007k^001nkP8000eoo`03001oogoo
04Aoo`00<7oo00D007ooOomoo`0000=oo`05001oogooOol0000?Ool3001nOol00`00O^inkP030000
17k^001nkWk^0P0000UnkP00O^inkP00O^h007k^000037oo0P00AGoo000aOol30005Ool3000>Ool2
0021Ool00`00O^inkP03000027k^001nkP00O^h007k^0002O^h01000O^h007k^0P003Goo00<007oo
Ool0@Woo0017Ool30023Ool00`00O^inkP0300002Wk^001nkP00O^h007k^001nkP000Wk^00D007k^
001nkP0000aoo`03001oogoo049oo`00AGoo0`00QGoo00<007k^00000`0000YnkP00O^h007k^001n
kP00O^h0009nkP05001nkP00O^h0000=Ool00`00Oomoo`11Ool0049oo`<008Moo`@0000=O^h00000
O^h007k^001nkP00O^h007k^00000Wk^00@007k^001nkP8000eoo`03001oogoo041oo`00@Goo0P00
QWoo1@00015nkP00O^inkP00O^h007k^001nkP00O^h007k^001nkP00009nkP04001nkP00000=Ool0
0`00Oomoo`10Ool0025oo`<000Ioo`8000Aoo`<000Eoo`D000Aoo`<0089oo`@000AC7@800003O^h0
07k^0080000:O^h007k^001nkP00O^h007k^0002O^h00`00O^h00002000=Ool00`00Oomoo`0oOol0
021oo`05001oogooOol00005Ool20003Ool01@00Oomoogoo00001Woo00<007ooOol00goo0P000Woo
00<007ooOol0O7oo10000e<M00<005<MDad00`0000mnkP00O^inkP00O^h007k^001nkP00O^h007k^
00000Wk^00H007k^000007k^000=Ool00`00Oomoo`0nOol001Qoo`H0009oo`05001oogooOol0000:
Ool01@00Oomoogoo00001Woo00<007ooOol017oo00D007ooOomoo`0007]oo`@000EC7@<0009C7@04
001nkP00O^h2000037k^001nkP00O^h007k^001nkP00O^h0009nkP800003O^h007oo00aoo`03001o
ogoo03ioo`0087oo00D007ooOomoo`0000Yoo`05001oogooOol00006Ool00`00Oomoo`03Ool20004
Ool00`00Oomoo`1eOol30004Dad00`00Dad0000200000e<M001C7@03Dad200000gk^001nkP020000
2Wk^001nkP00O^h007k^001nkP000Wk^0P0000=nkP0000003Goo00<007ooOol0?Goo000POol01@00
Oomoogoo00002Woo00D007ooOomoo`0000Ioo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo
071oo`@000=C7@03001C7E<M00<000EC7@03001C7E<M00<0000GO^h007k^O^h007k^001nkP00O^h0
07k^001nkP00O^h007k^001nkP00O^h0000<Ool2000nOol0021oo`05001oogooOol0000:Ool01@00
Oomoogoo00001Woo00<007ooOol00goo00<007ooOol01Goo00<007ooOol0KGoo10001E<M0`000U<M
00<005<MDad00e<M0`000e<M0P0000=nkP00O^h00P00011nkP00O^h007k^001nkP00O^h007k^001n
kP00O^h000eoo`03001oogoo03aoo`0087oo00D007ooOomoo`0000Yoo`05001oogooOol00006Ool0
0`00Oomoo`02Ool20008Ool00`00Oomoo`1YOol30004Dad00`00Dad0000200000e<M001C7@03Dad0
0`00Dad0000200000e<M001C7@03Dad2000017k^001nkP000Wk^00h007k^001nkP00O^h007k^001n
kP00O^h007k^0P003Goo00<007ooOol0>goo000QOol3000<Ool30005Ool30005Ool00`00Oomoo`07
Ool00`00Oomoo`1UOol40002Dad00`00DaeC7@030005Dad00`00DaeC7@030005Dad02@00Dad00000
Dad007k^001nkP0200002Gk^001nkP00O^h007k^001nkP020002O^h01000O^h000003Woo00<007oo
Ool0>Woo000oOol2000:Ool00`00Oomoo`1ROol40004Dad30003Dad00`00DaeC7@02Dad30003Dad0
1000DaeC7E<M0`0015<M0P0000=nkP00O^h00P0000MnkP00O^h007k^001nkP020002O^h00`00O^h0
0002000=Ool00`00Oomoo`06Ool30006Ool20005Ool3000QOol003moo`03001oogoo00Yoo`03001o
ogoo05ioo`<000AC7@06001C7@00001C7@001E<M00H005<M000005<M0004Dad40002Dad00`00DaeC
7@02Dad2000017k^001nkP000Wk^00H007k^001nkP00O^h2000017k^001nkWk^0P0000=nkP00Ool0
2goo0P001Woo00D007ooOomoo`0000Eoo`8000Aoo`05001oogooOol0000POol003moo`8000aoo`03
001oogoo05Uoo`@000=C7@03001C7E<M00<000AC7@04001C7E<MDad30004Dad01P00Dad00000Dad0
00EC7@09001C7@00001C7@00O^h007k^00800007O^h007k^001nkP00O^h00P0000AnkP00O^inkP80
0003O^h0000000eoo`03001oogoo00=oo`05001oogooOol0000?Ool00`00Oomoo`0NOol003moo`03
001oogoo00aoo`8005Moo`<00003Dad005<M00=C7@<0009C7@03001C7E<M009C7@@000=C7@03001C
7E<M00<000AC7@04001C7E<MDad30004Dad2000017k^001nkP000Wk^00@007k^001nkP800004O^h0
07k^O^h2000017k^001nkP0037oo00<007ooOol00goo00D007ooOomoo`0000moo`03001oogoo01io
o`005goo0`001Woo0P0017oo0`001Goo0`001Goo10001Goo00<007ooOol03Woo00<007ooOol0DGoo
0`001E<M00H005<M000005<M0004Dad01000Dad000000U<M00<005<MDad00U<M0`000U<M00<005<M
Dad00U<M10000e<M00<005<MDad00`0000=nkP00O^h00P0000EnkP00O^h007k^0080000:O^h007k^
001nkP00O^h007k^000=Ool00`00Oomoo`02Ool01@00Oomoogoo00003Goo0P008Goo000FOol01@00
Oomoogoo00001Goo0P000goo00D007ooOomoo`0000=oo`05001oogooOol00008Ool01000Oomoogoo
0P004Goo00<007ooOol0C7oo10000e<M00@005<MDaeC7@<000AC7@03001C7E<M00<000EC7@06001C
7@00001C7@0015<M00H005<M000005<M0005Dad30002Dad01000O^h007k^0P0000EnkP00O^h007k^
00800009O^h007k^001nkP00O^h007k^008000eoo`05001oogooOol00003Ool00`00Oomoo`0=Ool0
0`00Oomoo`0NOol000ioo`H0009oo`05001oogooOol0000:Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000Qoo`05001oogooOol0000COol00`00Oomoo`18Ool400000e<M001C7@03Dad40003Dad0
0`00DaeC7@040003Dad01000DaeC7E<M0`0015<M00<005<MDad00`0015<M00<005<M00000P0000=C
7@00Dad00e<M0P0000AnkP00O^h0009nkP0=001nkP00001nkP00O^h007k^001nkP00O^h00P0037oo
0P000goo00D007ooOomoo`0000]oo`05001oogooOol0000POol001Ioo`05001oogooOol0000:Ool0
1@00Oomoogoo00000goo00D007ooOomoo`0000Qoo`04001oogooOol2000COol00`00Oomoo`15Ool4
0004Dad01000Dad000000U<M00<005<MDad00U<M0`000U<M00<005<MDad00U<M10000e<M00<005<M
Dad010000e<M00<005<MDad00`001E<M00<005<MDad00`0000=nkP00O^h00P0000=nkP00O^h00P00
00UnkP00O^h007k^001nkP00O^h00`003Goo00@007ooOomoo`<000eoo`<0025oo`005Woo00D007oo
Oomoo`0000Yoo`05001oogooOol00003Ool01@00Oomoogoo000017oo100017oo00<007ooOol04goo
00<007ooOol0@Goo10000e<M00<005<MDad0100015<M00H005<M000005<M0004Dad01P00Dad00000
Dad000EC7@<0009C7@03001C7E<M009C7@<000=C7@04001C7E<MDad40003Dad2000017k^001nkP00
0Wk^0P0000UnkP00O^h007k^001nkP00O^h00P0000=nkP00Ool037oo00<007ooOol0=7oo000FOol0
1@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05001oogooOol00004Ool00`00Oomoo`05Ool0
0`00Oomoo`0DOol00`00Oomoo`0mOol40002Dad00`00DaeC7@040004Dad00`00DaeC7@030004Dad0
0`00DaeC7@030004Dad00`00Dad0000200000e<M001C7@03Dad01P00Dad00000Dad000AC7@@0009C
7@03001C7E<M009C7@800004O^h007k^0002O^h200002Gk^001nkP00O^h007k^001nkP0200000gk^
0000000<Ool00`00Oomoo`0dOol001Ioo`05001oogooOol0000:Ool01@00Oomoogoo00000goo00D0
07ooOomoo`0000Aoo`03001oogoo00Aoo`8001Moo`03001oogoo03Yoo`@000AC7@<0009C7@03001C
7E<M00=C7@<000=C7@03001C7E<M00@000=C7@03001C7E<M00@000AC7@03001C7E<M00<000AC7@06
001C7@00001C7@001E<M00T005<M000005<M001nkP00O^h00P0000ankP00001nkP00O^h007k^001n
kP00O^h2000017k^001nkP0037oo00<007ooOol0<goo000GOol3000<Ool30005Ool30005Ool40004
Ool00`00Oomoo`0FOol00`00Oomoo`0fOol30004Dad01P00Dad00000Dad000AC7@03001C7@000080
0003Dad005<M00=C7@<0009C7@03001C7E<M009C7@<000AC7@03001C7E<M00@000=C7@03001C7E<M
00<000AC7@04001C7E<MDad30004Dad200003Gk^001nkP00O^h007k^001nkP00O^h007k^00800006
O^h007k^001nkP0037oo0P00=7oo000mOol2000IOol00`00Oomoo`0aOol40003Dad00`00DaeC7@03
0004Dad00`00DaeC7@040004Dad01P00Dad00000Dad000AC7@06001C7@00001C7@001E<M0`000U<M
00<005<MDad00U<M0`000e<M00<005<MDad01@000e<M00<005<MDad00`0000=nkP00O^h00`0000Mn
kP00O^h007k^001nkP0200001Gk^001nkP00O^h00P0037oo00<007ooOol0<Woo000mOol00`00Oomo
o`0IOol00`00Oomoo`0]Ool400000e<M001C7@03Dad30003Dad00`00DaeC7@040004Dad00`00DaeC
7@030004Dad00`00DaeC7@030004Dad00`00Dad0000200000e<M001C7@03Dad01P00Dad00000Dad0
00AC7@<000=C7@04001C7E<MDad40003Dad200000gk^00000002O^h01000O^h000000Wk^0P0000En
kP00O^h007k^008000eoo`03001oogoo035oo`00?Goo00<007ooOol06Woo00<007ooOol0:Woo1000
15<M00H005<M000005<M0004Dad30003Dad00`00DaeC7@02Dad30003Dad00`00DaeC7@040003Dad0
0`00DaeC7@040004Dad00`00DaeC7@030004Dad01P00Dad00000Dad000AC7@@0009C7@03001C7E<M
009C7@800007O^h00000001nkP00O^h00P000Wk^0P0000EnkP00O^h007k^00<000aoo`03001oogoo
035oo`00?Goo0P0077oo00<007ooOol09Woo0`0015<M00<005<MDad00`0015<M00@005<M0000009C
7@03001C7E<M009C7@06001C7@00001C7@0015<M0`000e<M00<005<MDad0100015<M00<005<MDad0
10000e<M00<005<MDad00`0015<M00H005<M000005<M0005Dad400000e<M001nkP02000017k^0000
07k^0P000Wk^0P0000EnkP00O^h007k^00800003O^h007oo00]oo`03001oogoo031oo`00=7oo0`00
1Woo00<007ooOol077oo0P008goo0`000e<M00<005<MDad010000e<M00<005<MDad0100015<M00<0
05<MDad00`0015<M00H005<M000005<M0004Dad30002Dad00`00DaeC7@03Dad30003Dad00`00DaeC
7@040003Dad00`00DaeC7@030004Dad00`00DaeC7@030005Dad3000017k^000007k^0P0000UnkP00
O^h007k^001nkP00O^h00P0000=nkP00Ool037oo00<007ooOol0;goo000cOol01@00Oomoogoo0000
17oo0P0087oo00<007ooOol07Woo100015<M0`000e<M00<005<MDad0100015<M00<005<MDad01000
0e<M00<005<MDad00`0015<M00H005<M000005<M0004Dad00`00Dad0000200000e<M001C7@03Dad3
0002Dad00`00DaeC7@02Dad30003Dad00`00DaeC7@040003Dad01000DaeC7E<M100000AnkP00001n
kP800009O^h007k^001nkP00O^h007k^00800003O^h0000000aoo`03001oogoo02moo`00<goo00D0
07ooOomoo`0000Aoo`03001oogoo021oo`03001oogoo01Yoo`<000AC7@04001C7@000002Dad01000
DaeC7E<M0`000e<M00<005<MDad00U<M0`000e<M00<005<MDad010000e<M00<005<MDad00`0015<M
00<005<MDad0100015<M00H005<M000005<M0004Dad01P00Dad00000Dad000AC7@<000=C7@03001C
7E<M00D0009C7@80000=O^h007k^001nkP00O^h007k^001nkP00O^h00P0000=nkP00O^h00`0037oo
00<007ooOol0;Woo000cOol01@00Oomoogoo000017oo00<007ooOol08Goo00<007ooOol05Goo1000
0e<M00<005<MDad0100015<M00<005<M00000U<M00<005<MDad00U<M00H005<M000005<M0004Dad3
0003Dad00`00DaeC7@040003Dad00`00DaeC7@040004Dad00`00DaeC7@030004Dad00`00DaeC7@03
0004Dad01P00Dad00000Dad000AC7@<0009C7@03001C7E<M009C7@800003O^h007k^00<00007O^h0
07k^001nkP00O^h00P0000=nkP00O^h00P0000=nkP00Ool02goo00<007ooOol0;Goo000cOol01@00
Oomoogoo000017oo0P008goo00<007ooOol04Goo10000U<M00<005<MDad0100015<M00<005<MDad0
0`0015<M00<005<MDad00`0015<M00H005<M000005<M0004Dad30002Dad00`00DaeC7@02Dad30003
Dad01000DaeC7E<M10000e<M00<005<MDad010000e<M00<005<MDad00`0015<M00H005<M000005<M
0004Dad04`00Dad00000Dad007k^001nkP00O^h007k^001nkP00O^h007k^00800003O^h007k^0080
0003O^h007oo00]oo`03001oogoo02eoo`00<goo00D007ooOomoo`0000Aoo`03001oogoo02=oo`03
001oogoo00ioo`<00003Dad005<M009C7@<000AC7@03001C7E<M00@000=C7@03001C7E<M00@000=C
7@03001C7E<M00<000AC7@06001C7@00001C7@0015<M00H005<M000005<M0004Dad40002Dad00`00
DaeC7@02Dad30003Dad00`00DaeC7@040003Dad00`00DaeC7@030004Dad00`00DaeC7@030004Dad2
00001gk^001nkP00O^h007k^0080009nkP800003O^h007k^00800003O^h0000000aoo`03001oogoo
02aoo`00<goo00D007ooOomoo`0000Aoo`03001oogoo02Aoo`03001oogoo00Yoo`@000AC7@03001C
7@00009C7@03001C7E<M009C7@<0009C7@03001C7E<M009C7@<000=C7@03001C7E<M00@000=C7@03
001C7E<M00<000AC7@03001C7E<M00<000AC7@04001C7@000002Dad00`00DaeC7@02Dad01P00Dad0
0000Dad000AC7@<000=C7@03001C7E<M00@000=C7@03001C7E<M00@000=C7@03001C7E<M00<00003
O^h007k^00<00003O^h00000009nkP800003O^h007k^00800003O^h0000000eoo`03001oogoo02]o
o`00=7oo0`0017oo0P009goo00<007ooOol01Woo10000e<M00<005<MDad00`0015<M00H005<M0000
05<M0004Dad01P00Dad00000Dad000AC7@<000=C7@03001C7E<M00@000=C7@03001C7E<M00@000=C
7@03001C7E<M00@000AC7@03001C7E<M00<000AC7@06001C7@00001C7@0015<M0`000U<M00<005<M
Dad00U<M0`000e<M00<005<MDad010000e<M0P0000EnkP00O^h007k^00800007O^h007k^001nkP00
O^h00P0000AnkP00000000aoo`03001oogoo00Ioo`<000Ioo`8000Ioo`<0015oo`00>goo00<007oo
Ool09goo00@007ooOomoo`@000=C7@03001C7E<M00@000=C7@03001C7E<M00<000AC7@03001C7E<M
00<000AC7@06001C7@00001C7@0015<M0`000U<M00<005<MDad00U<M0`000e<M00<005<MDad01000
15<M00<005<MDad010000e<M00<005<MDad00`0015<M00H005<M000005<M0004Dad01P00Dad00000
Dad000AC7@<000=C7@03001C7E<M00<00005O^h007k^001nkP0200001gk^001nkP00O^h007k^0080
0005O^h00000O^h0000<Ool00`00Oomoo`04Ool01@00Oomoogoo00001Goo0P001goo00<007ooOol0
47oo000kOol2000YOol60004Dad30003Dad00`00DaeC7@040003Dad00`00DaeC7@040003Dad00`00
DaeC7@040003Dad01000Dad000000U<M00H005<MDaeC7@00Dad20002Dad01000DaeC7E<M0`0015<M
00<005<MDad010000e<M00<005<MDad010000e<M00<005<MDad00`0015<M00<005<MDad00`0015<M
00H005<M000005<M0004Dad30002Dad01000O^h007k^0P0000UnkP00O^h007k^001nkP00O^h00P00
00EnkP00001nkP0000eoo`03001oogoo00=oo`05001oogooOol0000;Ool5000AOol003]oo`03001o
ogoo02Ioo`D000=C7@04001C7@000002Dad01000DaeC7E<M0`0015<M00<005<MDad00`0015<M00<0
05<MDad00`0015<M00<005<MDad00`0015<M00<005<MDad00`0015<M00<005<M00000U<M00<005<M
Dad00U<M0`000U<M00<005<MDad00U<M0`000e<M00<005<MDad010000e<M00<005<MDad010000e<M
00<005<MDad00`0015<M00H005<M000005<M0004Dad200000gk^0000000200001gk^001nkP00O^h0
07k^00800004O^h00000O^h2000<Ool00`00Oomoo`03Ool01@00Oomoogoo00002goo00@007ooOol0
019oo`00>goo00<007ooOol097oo0`0015<M00<005<MDad00`0015<M00<005<M00000U<M00<005<M
Dad00U<M0`000U<M00<005<MDad00U<M0`000e<M00<005<MDad010000e<M00<005<MDad010000e<M
00<005<MDad00`0015<M00H005<M000005<M0004Dad01P00Dad00000Dad000AC7@<000=C7@03001C
7E<M00@000=C7@03001C7E<M00@000=C7@03001C7E<M00<000AC7@03001C7E<M00<00008O^h00000
O^h007k^001nkP80009nkP800004O^h00000O^h3000<Ool00`00Oomoo`02Ool01@00Oomoogoo0000
37oo00<007oo00004Woo000kOol2000QOol40003Dad00`00DaeC7@040003Dad00`00DaeC7@030004
Dad01P00Dad00000Dad000AC7@06001C7@00001C7@0015<M0`000e<M00<005<MDad010000e<M00<0
05<MDad010000e<M00<005<MDad00`0015<M00<005<MDad00`0015<M00H005<M000005<M0004Dad3
0003Dad00`00DaeC7@040003Dad00`00DaeC7@040003Dad00`00DaeC7@040003Dad300001Gk^001n
kP00O^h00P000Wk^0P0000AnkP00001nkP800003O^h007oo00Uoo`<000=oo`05001oogooOol0000<
Ool00`00Ool0000BOol001=oo`<000Ioo`8000Eoo`<000Aoo`<000Eoo`@000Aoo`03001oogoo01io
o`@0009C7@03001C7E<M00@000=C7@03001C7E<M00@000=C7@03001C7E<M00<000AC7@03001C7E<M
00<000AC7@06001C7@00001C7@0015<M0`000e<M00<005<MDad010000e<M00<005<MDad010000e<M
00<005<MDad010000e<M00<005<MDad00`0015<M00H005<M000005<M0004Dad30002Dad00`00DaeC
7@02Dad30003Dad00`00DaeC7@040002Dad01000DaeC7E<M100000EnkP00000007k^0080009nkP80
0004O^h00000O^h200000gk^001oo`06Ool300000goo00000003Ool01@00Oomoogoo00003Goo0P00
4Woo000BOol01@00Oomoogoo00001Goo0P0017oo00D007ooOomoo`00009oo`05001oogooOol00008
Ool01@00Oomoogoo00007Woo0`0000=C7@00Dad00U<M0`000e<M00<005<MDad010000e<M00<005<M
Dad010000e<M00<005<MDad010000e<M00<005<MDad00`0015<M00H005<M000005<M0004Dad30002
Dad00`00DaeC7@02Dad30003Dad00`00DaeC7@040003Dad00`00DaeC7@040003Dad00`00DaeC7@03
0004Dad01P00Dad00000Dad000AC7@06001C7@00001C7@0015<M0`000U<M00<005<MDad01@000U<M
0P0000EnkP00O^h007k^00800005O^h007k^001nkP02000017k^000007k^0P000goo0`001Goo00<0
07ooOol00Woo0`003goo00<007ooOol047oo000BOol01@00Oomoogoo00002goo00D007ooOomoo`00
009oo`05001oogooOol00008Ool01000Oomoogoo0P006Woo100015<M00<005<M00000U<M00@005<M
DaeC7@<000=C7@04001C7E<MDad30003Dad00`00DaeC7@040003Dad00`00DaeC7@040003Dad00`00
DaeC7@030004Dad01P00Dad00000Dad000AC7@06001C7@00001C7@0015<M0`000e<M00<005<MDad0
10000e<M00<005<MDad010000e<M00<005<MDad00`0015<M00<005<MDad00`0015<M100000=C7@00
Dad00U<M0`000U<M00<005<MDad00U<M0P0000=nkP00O^h0100000EnkP00O^h007k^00800004O^h0
0000O^h50008Ool00`00Oomoo`0WOol0019oo`05001oogooOol0000;Ool01@00Oomoogoo00000Woo
00D007ooOomoo`0000Qoo`05001oogooOol0000HOol40003Dad00`00DaeC7@030004Dad00`00Dad0
0002Dad00`00DaeC7@02Dad00`00Dad00002Dad01000DaeC7E<M0`0015<M00<005<MDad00`000e<M
00<005<MDad010000e<M00<005<MDad00`0015<M00<005<MDad00`0015<M00H005<M000005<M0004
Dad30003Dad00`00DaeC7@040003Dad00`00DaeC7@040003Dad00`00DaeC7@040002Dad00`00DaeC
7@030005Dad300000e<M001C7@03Dad02@00Dad00000Dad007k^001nkP0200000gk^001nkP020002
O^h2000017k^000007k^0P0000=nkP00Ool02Goo00<007ooOol09Woo000BOol01@00Oomoogoo0000
2goo00D007ooOomoo`00009oo`05001oogooOol00004Ool40004Ool2000COol40003Dad00`00DaeC
7@040003Dad00`00DaeC7@030004Dad00`00DaeC7@030004Dad00`00Dad00002Dad00`00DaeC7@02
Dad30003Dad00`00DaeC7@030003Dad00`00DaeC7@040003Dad00`00DaeC7@040003Dad00`00DaeC
7@030004Dad01P00Dad00000Dad000AC7@<0009C7@03001C7E<M009C7@<000=C7@03001C7E<M00@0
009C7@03001C7E<M00@000=C7@03001C7E<M00<000AC7@03001C7E<M00<000AC7@800003O^h00000
00800003O^h00000009nkP800004O^h00000O^h200000gk^001oo`09Ool00`00Oomoo`0VOol0019o
o`05001oogooOol0000;Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000Aoo`03001oogoo00Ao
o`03001oogoo011oo`D0009C7@03001C7E<M00<000=C7@03001C7E<M00@000=C7@03001C7E<M00@0
00=C7@03001C7E<M00<000AC7@06001C7@00001C7@0015<M0`000e<M00<005<MDad00`000e<M00<0
05<MDad010000e<M00<005<MDad010000e<M00<005<MDad00`0015<M00H005<M000005<M0004Dad0
1P00Dad00000Dad000AC7@<0009C7@03001C7E<M00@000=C7@03001C7E<M00@000=C7@03001C7E<M
00@000=C7@03001C7E<M00<00006O^h00000O^h007k^0P0000EnkP00O^h007k^00800004O^h00000
O^h20009Ool00`00Oomoo`0VOol0019oo`05001oogooOol0000;Ool01@00Oomoogoo00000Woo00D0
07ooOomoo`0000Aoo`03001oogoo00Aoo`03001oogoo00moo`@000AC7@<000=C7@03001C7E<M00<0
00AC7@03001C7E<M00<000=C7@03001C7E<M00@000=C7@03001C7E<M00<000AC7@06001C7@00001C
7@0015<M0`000U<M00@005<MDaeC7@<000=C7@03001C7E<M00@000=C7@03001C7E<M00@000=C7@03
001C7E<M00<000AC7@03001C7E<M00<000AC7@06001C7@00001C7@000e<M0`000e<M00<005<MDad0
10000e<M00<005<MDad010000e<M00<005<MDad010000e<M0`0000=nkP0000000P0000EnkP00O^h0
07k^00800004O^h00000O^h30008Ool00`00Oomoo`0VOol001=oo`<000eoo`<000Aoo`<000Eoo`@0
00=oo`8000eoo`@000=C7@06001C7@00001C7@0015<M0`000e<M00<005<MDad010000e<M00<005<M
Dad00`000e<M00<005<MDad010000e<M00<005<MDad00`0015<M00H005<M000005<M0004Dad00`00
Dad00002Dad01000DaeC7E<M0`000e<M00<005<MDad010000e<M00<005<MDad010000e<M00<005<M
Dad010000e<M00<005<MDad00`0015<M0`0000=C7@00Dad00e<M0`000e<M00<005<MDad010000e<M
00<005<MDad010000e<M00<005<MDad0100000=nkP0000000P0000=nkP0000000Wk^0P0000AnkP00
001nkP800003O^h007oo00Ioo`03001oogoo02Ioo`00>Goo00<007ooOol02Goo0`0015<M00<005<M
Dad00`000e<M00H005<M000005<M0004Dad30003Dad00`00DaeC7@040003Dad00`00DaeC7@030003
Dad00`00DaeC7@040003Dad00`00DaeC7@030004Dad00`00DaeC7@030004Dad00`00Dad00002Dad0
1000DaeC7E<M0`000e<M00<005<MDad010000e<M00<005<MDad010000e<M00<005<MDad010000e<M
00<005<MDad00P0015<M00H005<M000005<M0004Dad30003Dad00`00DaeC7@040003Dad00`00DaeC
7@040002Dad200000gk^001nkP0200000gk^00000002O^h2000017k^000007k^0P0000=nkP00Ool0
1Woo00<007ooOol09Woo000iOol00`00Oomoo`05Ool40003Dad00`00DaeC7@040003Dad00`00DaeC
7@030003Dad01P00Dad00000Dad000AC7@<000=C7@03001C7E<M00@000=C7@03001C7E<M00<000AC
7@03001C7E<M00<000=C7@03001C7E<M00@000=C7@03001C7E<M00<000AC7@03001C7@00009C7@04
001C7E<MDad30003Dad00`00DaeC7@040003Dad00`00DaeC7@040003Dad00`00DaeC7@040002Dad0
0`00DaeC7@030004Dad01P00Dad00000Dad000AC7@<000=C7@03001C7E<M00@0009C7@03001C7E<M
00<00003O^h007k^00800008O^h00000O^h007k^001nkP800004O^h00000O^h20006Ool00`00Oomo
o`0VOol003Uoo`8000=oo`@000=C7@03001C7E<M00<000=C7@03001C7E<M00@000=C7@03001C7E<M
00<000=C7@04001C7@000002Dad01000DaeC7E<M0`000U<M00<005<MDad00U<M0`000e<M00<005<M
Dad010000U<M00<005<MDad010000e<M00<005<MDad010000e<M00<005<MDad00`0015<M00D005<M
001C7@0000AC7@<000=C7@03001C7E<M00@000=C7@03001C7E<M00@0009C7@03001C7E<M00@000=C
7@03001C7E<M00<000AC7@06001C7@00001C7@0015<M0`000U<M00@005<MDaeC7@<000=C7@800003
O^h0000000<00005O^h007k^001nkP02000017k^000007k^0P001goo00<007ooOol09Goo000hOol0
0`00Oomoo`040002Dad00`00DaeC7@040003Dad00`00DaeC7@030003Dad00`00DaeC7@040003Dad0
0`00DaeC7@030004Dad00`00Dad00002Dad01P00DaeC7E<M001C7@800003Dad005<M00=C7@<000=C
7@03001C7E<M00<000=C7@03001C7E<M00@000=C7@03001C7E<M00@000=C7@03001C7E<M00<000=C
7@06001C7@00001C7@0015<M0`000e<M00<005<MDad010000e<M00<005<MDad00`000e<M00<005<M
Dad010000e<M00<005<MDad00`0015<M00H005<M000005<M0004Dad300000e<M001C7@03Dad20000
0gk^0000000300001Gk^001nkP00O^h00P0000AnkP00001nkP<000Ioo`03001oogoo02Eoo`00>7oo
100000=C7@00Dad00U<M0`000e<M00<005<MDad010000e<M00<005<MDad00`000e<M00<005<MDad0
10000e<M00<005<MDad00`0015<M00<005<MDad00`000e<M00H005<M000005<M0004Dad30003Dad0
0`00DaeC7@030003Dad00`00DaeC7@040003Dad00`00DaeC7@040003Dad00`00DaeC7@020004Dad0
1P00Dad00000Dad000AC7@<000=C7@03001C7E<M00@0009C7@03001C7E<M00@000=C7@03001C7E<M
00@000=C7@03001C7E<M00<000AC7@03001C7@00008000AC7@07001C7@00001C7@00O^h00P0000Qn
kP00001nkP00O^h007k^0P0000AnkP00001nkP@000Eoo`03001oogoo02Eoo`006Goo0`001Woo0P00
1Goo0`0017oo1@000goo0P0015<M00<005<M00000U<M00@005<MDaeC7@<000=C7@03001C7E<M00@0
00=C7@03001C7E<M00<000=C7@03001C7E<M00@000=C7@03001C7E<M00@000=C7@03001C7E<M00<0
00=C7@06001C7@00001C7@0015<M0`000e<M00<005<MDad00`000e<M00<005<MDad010000e<M00<0
05<MDad010000U<M00<005<MDad00`0015<M00H005<M000005<M0004Dad30003Dad00`00DaeC7@03
0003Dad00`00DaeC7@040003Dad00`00DaeC7@040002Dad00`00DaeC7@040003Dad00`00DaeC7@03
0004Dad300001Wk^000007k^001nkP800003O^h007k^00800006O^h00000001nkP0017oo00<007oo
Ool09Goo000HOol01@00Oomoogoo00001Goo0P0017oo00D007ooOomoo`0000Eoo`03001oogoo00Eo
o`03001C7E<M00<000AC7@03001C7@00009C7@04001C7E<MDad30003Dad00`00DaeC7@040003Dad0
0`00DaeC7@030003Dad00`00DaeC7@040003Dad00`00DaeC7@040003Dad00`00DaeC7@020004Dad0
1P00Dad00000Dad000AC7@<000=C7@03001C7E<M00<000=C7@03001C7E<M00@000=C7@03001C7E<M
00<000=C7@03001C7E<M00<000AC7@06001C7@00001C7@0015<M0`000U<M00<005<MDad010000e<M
00<005<MDad010000U<M00<005<MDad010000e<M00<005<MDad010000e<M00<005<MDad0100000=n
kP0000000P0000InkP00001nkP00O^h200001Wk^00000000O^h000Aoo`03001oogoo02Eoo`0067oo
00D007ooOomoo`0000]oo`05001oogooOol00005Ool00`00Oomoo`06Ool20003Dad00`00DaeC7@03
0004Dad00`00Dad00002Dad01000DaeC7E<M0`000e<M00<005<MDad010000e<M00<005<MDad00`00
0e<M00<005<MDad010000e<M00<005<MDad010000U<M00<005<MDad00`0015<M00H005<M000005<M
0004Dad30002Dad00`00DaeC7@040003Dad00`00DaeC7@040003Dad00`00DaeC7@030003Dad00`00
DaeC7@030004Dad01P00Dad00000Dad000=C7@<000=C7@03001C7E<M00@000=C7@03001C7E<M00<0
00=C7@03001C7E<M00@000=C7@03001C7E<M00@0009C7@800003O^h0000000800006O^h00000O^h0
07k^0P0000EnkP00000007k^008000=oo`03001oogoo02Eoo`0067oo00D007ooOomoo`0000]oo`05
001oogooOol00005Ool00`00Oomoo`07Ool20002Dad30003Dad00`00DaeC7@030004Dad00`00Dad0
0002Dad01000DaeC7E<M0`000e<M00<005<MDad010000e<M00<005<MDad00`000e<M00<005<MDad0
10000e<M00<005<MDad00`000e<M00<005<MDad00`0015<M00H005<M000005<M0003Dad30003Dad0
0`00DaeC7@040003Dad00`00DaeC7@040002Dad00`00DaeC7@040003Dad00`00DaeC7@030004Dad3
00000e<M001C7@03Dad30003Dad00`00DaeC7@040002Dad00`00DaeC7@040003Dad00`00DaeC7@04
0002Dad00`00DaeC7@0300001Wk^000007k^001nkP800003O^h007k^00800005O^h00000001nkP02
0003Ool00`00Oomoo`0UOol001Qoo`05001oogooOol0000;Ool01@00Oomoogoo00001Goo00<007oo
Ool027oo0`000e<M00<005<MDad00`000e<M00<005<MDad00`0015<M00<005<M00000U<M00@005<M
DaeC7@<000=C7@03001C7E<M00@000=C7@03001C7E<M00<000=C7@03001C7E<M00@000=C7@03001C
7E<M00<000=C7@03001C7E<M00<000AC7@03001C7@0000=C7@03001C7E<M00<000=C7@03001C7E<M
00@000=C7@03001C7E<M00<000=C7@03001C7E<M00@000=C7@03001C7E<M008000AC7@06001C7@00
001C7@0015<M0`000U<M00<005<MDad010000e<M00<005<MDad010000U<M00<005<MDad010000e<M
0`0000=nkP0000000P0000InkP00001nkP00O^h3000017k^000000000goo00<007ooOol097oo000H
Ool01@00Oomoogoo00002goo00D007ooOomoo`0000Eoo`03001oogoo00Yoo`80009C7@<000=C7@03
001C7E<M00<000=C7@03001C7E<M00<000AC7@03001C7@00009C7@04001C7E<MDad30003Dad00`00
DaeC7@040003Dad00`00DaeC7@030003Dad00`00DaeC7@040003Dad00`00DaeC7@030003Dad00`00
DaeC7@030004Dad30002Dad00`00DaeC7@040003Dad00`00DaeC7@040002Dad00`00DaeC7@040003
Dad00`00DaeC7@040002Dad00`00DaeC7@030004Dad01P00Dad00000Dad000=C7@<000=C7@03001C
7E<M00@0009C7@03001C7E<M00@000=C7@03001C7E<M00@00003O^h0000000800006O^h00000O^h0
07k^0`0000=nkP0000000P000Woo00<007ooOol097oo000HOol01@00Oomoogoo00002goo00D007oo
Oomoo`0000Eoo`03001oogoo00]oo`<000=C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@03
001C7E<M00<000AC7@03001C7@00009C7@04001C7E<MDad30003Dad00`00DaeC7@040003Dad00`00
DaeC7@030003Dad00`00DaeC7@040002Dad00`00DaeC7@040003Dad01P00Dad00000Dad000=C7@<0
00=C7@03001C7E<M00@000=C7@03001C7E<M00<000=C7@03001C7E<M00@0009C7@03001C7E<M00@0
00=C7@03001C7E<M00<000AC7@<00003Dad005<M00=C7@<0009C7@03001C7E<M00@000=C7@03001C
7E<M00@0009C7@800003O^h007k^00800006O^h00000O^h007k^0`0000QnkP00000007k^001oo`00
9Woo000IOol3000=Ool30004Ool3000>Ool20002Dad40003Dad00`00DaeC7@030003Dad00`00DaeC
7@030003Dad00`00DaeC7@030004Dad00`00Dad00003Dad00`00DaeC7@030003Dad00`00DaeC7@04
0002Dad00`00DaeC7@040003Dad00`00DaeC7@030003Dad00`00DaeC7@030004Dad01@00Dad005<M
000015<M0`000e<M00<005<MDad010000U<M00<005<MDad010000e<M00<005<MDad00`000e<M00<0
05<MDad010000e<M00<005<MDad00P0015<M00@005<M0000009C7@03001C7E<M00<000=C7@03001C
7E<M00@000=C7@03001C7E<M00800003O^h007k^00800006O^h00000O^h007k^0`0000QnkP000000
07k^001oo`009Woo0012Ool30003Dad00`00DaeC7@040002Dad00`00DaeC7@040003Dad00`00DaeC
7@030003Dad00`00DaeC7@030004Dad30002Dad00`00DaeC7@040003Dad00`00DaeC7@030003Dad0
0`00DaeC7@040003Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad01P00Dad00000Dad000AC
7@<0009C7@03001C7E<M00@000=C7@03001C7E<M00@0009C7@03001C7E<M00@000=C7@03001C7E<M
00@0009C7@03001C7E<M00<000AC7@<0009C7@03001C7E<M00@000=C7@03001C7E<M00@0009C7@80
0003O^h0000000<00004O^h00000O^h200001Gk^00000000O^h00`009Woo0014Ool20002Dad30003
Dad00`00DaeC7@030003Dad00`00DaeC7@040003Dad00`00DaeC7@030003Dad01P00Dad00000Dad0
00=C7@<000=C7@03001C7E<M00@000=C7@03001C7E<M00<000=C7@03001C7E<M00@0009C7@03001C
7E<M00@000=C7@03001C7E<M008000AC7@04001C7@000002Dad00`00DaeC7@030003Dad00`00DaeC
7@040003Dad00`00DaeC7@030003Dad00`00DaeC7@040002Dad00`00DaeC7@040003Dad01P00Dad0
0000Dad000=C7@<000=C7@03001C7E<M00@0009C7@03001C7E<M00<00003O^h0000000<00004O^h0
0000O^h200001Gk^00000000O^h00`009Woo0015Ool30003Dad00`00DaeC7@030003Dad00`00DaeC
7@030003Dad00`00DaeC7@040003Dad00`00DaeC7@020004Dad00`00Dad00002Dad01000DaeC7E<M
0`000e<M00<005<MDad010000U<M00<005<MDad010000e<M00<005<MDad00`000e<M00<005<MDad0
10000U<M00<005<MDad00`0015<M0`000U<M00<005<MDad010000e<M00<005<MDad010000U<M00<0
05<MDad010000e<M00<005<MDad00`000e<M00<005<MDad00`0015<M0`0000=C7@00Dad00e<M0`00
0U<M00<005<MDad010000e<M0`0000AnkP00001nkP<00003O^h007k^00<00004O^h00000000VOol0
04Ioo`80009C7@@000=C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@03001C7E<M00@0009C
7@03001C7E<M00<000AC7@03001C7@0000=C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@03
001C7E<M00@0009C7@03001C7E<M00@000=C7@03001C7E<M00<000=C7@06001C7@00001C7@000e<M
0`000e<M00<005<MDad010000U<M00<005<MDad010000e<M00<005<MDad010000U<M00<005<MDad0
10000U<M00<005<MDad00`0015<M00H005<M000005<M0002Dad40003Dad00`00DaeC7@04000017k^
000007k^0`0000=nkP00O^h00`0000=nkP0000000P009Goo0018Ool20003Dad00`00DaeC7@040002
Dad00`00DaeC7@040003Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad00`00DaeC7@030004
Dad30002Dad00`00DaeC7@040003Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad00`00DaeC
7@040002Dad00`00DaeC7@030004Dad00`00Dad00002Dad00`00DaeC7@040003Dad00`00DaeC7@03
0003Dad00`00DaeC7@040002Dad00`00DaeC7@040003Dad00`00DaeC7@030003Dad00`00DaeC7@03
0003Dad30003Dad00`00DaeC7@040002Dad200000gk^0000000400000gk^001nkP0300000gk^0000
0002000UOol004Uoo`80009C7@<000=C7@03001C7E<M00<000=C7@03001C7E<M00@0009C7@03001C
7E<M00@000=C7@03001C7E<M00<000=C7@06001C7@00001C7@000e<M0`000e<M00<005<MDad01000
0U<M00<005<MDad010000e<M00<005<MDad00`000e<M00<005<MDad00`000e<M00<005<MDad00`00
0e<M0`000e<M00<005<MDad010000U<M00<005<MDad010000e<M00<005<MDad00`000e<M00<005<M
Dad010000U<M00<005<MDad010000U<M00H005<M000005<M0004Dad30002Dad00`00DaeC7@030000
0gk^0000000400000gk^001nkP0300000gk^00000002000UOol004Yoo`<000=C7@03001C7E<M00<0
00=C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@03001C7E<M00@000=C7@03001C7E<M0080
00AC7@03001C7@00009C7@03001C7E<M00@000=C7@03001C7E<M00<000=C7@03001C7E<M00@0009C
7@03001C7E<M00@0009C7@03001C7E<M00@000=C7@05001C7@00Dad00004Dad30003Dad00`00DaeC
7@030003Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@040003Dad00`00DaeC
7@020004Dad01P00Dad00000Dad0009C7@@000=C7@<00005O^h00000001nkP02000017k^000007k^
10009Goo001;Ool20002Dad40002Dad00`00DaeC7@040003Dad00`00DaeC7@030003Dad00`00DaeC
7@030003Dad00`00DaeC7@040002Dad00`00DaeC7@030003Dad30003Dad00`00DaeC7@040002Dad0
0`00DaeC7@040003Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad0
1000Dad000000U<M00<005<MDad010000U<M00<005<MDad010000U<M00<005<MDad010000e<M00<0
05<MDad00`000e<M00<005<MDad010000U<M00<005<MDad00`000e<M0`000e<M00<005<MDad01000
00EnkP00000007k^00800004O^h00000O^h4000UOol004eoo`8000=C7@03001C7E<M00<000=C7@03
001C7E<M00@0009C7@03001C7E<M00@000=C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@03
001C7@0000=C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@03001C7E<M00@0009C7@03001C
7E<M00@0009C7@03001C7E<M00@000=C7@03001C7E<M008000AC7@<0009C7@03001C7E<M00@000=C
7@03001C7E<M00<000=C7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C
7@04001C7@000002Dad00`00DaeC7@040002Dad200001Gk^00000000O^h00P0000AnkP00001nkP@0
02Eoo`00CWoo0P000U<M0`000e<M00<005<MDad00`000e<M00<005<MDad00`000e<M00<005<MDad0
10000U<M00<005<MDad010000U<M00<005<MDad00`0015<M0`000U<M00<005<MDad010000U<M00<0
05<MDad010000e<M00<005<MDad00`000e<M00<005<MDad00`000e<M00<005<MDad010000U<M00@0
05<M0000009C7@03001C7E<M00<000=C7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@03001C
7E<M00@0009C7@03001C7E<M00@000=C7@03001C7E<M008000AC7@<0009C7@03001C7E<M00<00005
O^h00000001nkP02000017k^000007k^10009Goo001?Ool30003Dad00`00DaeC7@030003Dad00`00
DaeC7@030003Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad01000
Dad000000U<M00<005<MDad00`000e<M00<005<MDad00`000e<M00<005<MDad010000U<M00<005<M
Dad010000U<M00<005<MDad010000U<M00<005<MDad00`0015<M0`000U<M00<005<MDad010000U<M
00<005<MDad010000U<M00<005<MDad010000e<M00<005<MDad00`000e<M00<005<MDad010000U<M
00H005<M000005<M0002Dad40003Dad7000017k^000007k^0`0000=oo`00Ool08goo001@Ool20002
Dad40002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@040003Dad00`00DaeC
7@030003Dad00`00DaeC7@030002Dad40002Dad00`00DaeC7@040003Dad00`00DaeC7@030003Dad0
0`00DaeC7@030003Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad01P00Dad00000Dad0009C
7@@000=C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@03001C7E<M00@0009C7@03001C7E<M
00@0009C7@03001C7E<M00@0009C7@<000=C7@03001C7E<M00D00004O^h00000O^h200001Wk^0000
0000Ool002Aoo`00DWoo0P000e<M00<005<MDad00`000e<M00<005<MDad00`000e<M00<005<MDad0
0`000e<M00<005<MDad010000U<M00<005<MDad010000U<M0`000e<M00<005<MDad00`000e<M00<0
05<MDad010000U<M00<005<MDad010000U<M00<005<MDad010000U<M00<005<MDad010000U<M00<0
05<MDad010000U<M0`000e<M00<005<MDad010000U<M00<005<MDad010000U<M00<005<MDad01000
0U<M00<005<MDad010000U<M00<005<MDad010000e<M0`000U<M00<005<MDad010000U<M0`0000An
kP00001nkP<00005O^h00000Ool0000TOol005=oo`80009C7@<000=C7@03001C7E<M00<000=C7@03
001C7E<M00<000=C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@03001C7@00009C7@03001C
7E<M00@0009C7@03001C7E<M00@000=C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@03001C
7E<M00<000=C7@03001C7E<M00<000=C7@<0009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C
7@03001C7E<M00@000=C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@06001C7@00001C7@00
0U<M10000U<M00<005<MDad0100000AnkP00001nkP<00005O^h00000Ool0000TOol005Aoo`<0009C
7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C
7@03001C7E<M00@0009C7@<000=C7@03001C7E<M00<000=C7@03001C7E<M00@0009C7@03001C7E<M
00@0009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@04001C7@000002Dad00`00DaeC7@03
0003Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad00`00DaeC7@040002Dad00`00DaeC7@04
0002Dad00`00DaeC7@030002Dad40002Dad00`00DaeC7@040002Dad200000gk^0000000400001Gk^
000007oo000097oo001EOol20002Dad30003Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad0
0`00DaeC7@030003Dad00`00DaeC7@030003Dad30003Dad00`00DaeC7@030003Dad00`00DaeC7@03
0003Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad00`00DaeC7@03
0002Dad40002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00
DaeC7@040002Dad00`00DaeC7@040002Dad30003Dad00`00DaeC7@030003Dad00`00DaeC7@020000
17k^000007k^0`0000EnkP00001oo`0002Aoo`00Egoo0P000e<M00<005<MDad00`000e<M00<005<M
Dad00`000e<M00<005<MDad00`000e<M00<005<MDad00`000e<M00<005<M00000U<M00<005<MDad0
10000U<M00<005<MDad010000U<M00<005<MDad010000U<M00<005<MDad010000U<M00<005<MDad0
10000U<M00<005<MDad0100000=C7@0000000P000U<M00<005<MDad010000U<M00<005<MDad01000
0U<M00<005<MDad010000U<M00<005<MDad010000e<M00<005<MDad00`000e<M00<005<M00000U<M
00<005<MDad010000U<M00<005<MDad010000U<M0`0000EnkP00000007k^00800003Ool007oo02=o
o`00F7oo00<005<MDad010000U<M00<005<MDad010000U<M00<005<MDad010000U<M00<005<MDad0
10000U<M00<005<MDad010000U<M0`000e<M00<005<MDad00`000e<M00<005<MDad00`000e<M00<0
05<MDad00`000e<M00<005<MDad00`000U<M00<005<MDad010000U<M0`000e<M00<005<MDad00`00
0e<M00<005<MDad00`000e<M00<005<MDad00`000e<M00<005<MDad00`000e<M00<005<MDad01000
0U<M00<005<MDad00`000U<M10000U<M00<005<MDad010000U<M00<005<MDad0100000EnkP000000
07k^0080009oo`03001oogoo025oo`00FGoo0P000e<M00<005<MDad00`000e<M00<005<MDad00`00
0e<M00<005<MDad00`000e<M00<005<MDad00`000U<M10000U<M00<005<MDad010000U<M00<005<M
Dad010000U<M00<005<MDad010000U<M00<005<MDad010000U<M00<005<MDad00`000e<M00<005<M
00000U<M00<005<MDad010000U<M00<005<MDad010000U<M00<005<MDad010000U<M00<005<MDad0
10000U<M00<005<MDad010000U<M00<005<MDad0100000=C7@0000000P000U<M00<005<MDad01000
0U<M00<005<MDad010000U<M0P0000EnkP00000007k^0080009oo`03001oogoo025oo`00FWoo0P00
0U<M0`000e<M00<005<MDad00`000e<M00<005<MDad00`000U<M00<005<MDad010000U<M0`000e<M
00<005<MDad00`000e<M00<005<MDad00`000e<M00<005<MDad00`000e<M00<005<MDad00`000U<M
00<005<MDad010000U<M00<005<MDad010000U<M0`000e<M00<005<MDad00`000e<M00<005<MDad0
0`000e<M00<005<MDad00`000U<M00<005<MDad010000U<M00<005<MDad010000U<M0`000e<M00<0
05<MDad00`000e<M00<005<MDad00`000e<M00<005<MDad00P0000=nkP00000010000Woo00<007oo
Ool08Goo001LOol20002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@030003
Dad30002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC
7@040002Dad00`00DaeC7@030003Dad00`00DaeC7@030002Dad40002Dad00`00DaeC7@040002Dad0
0`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad3
0002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad70002Ool00`00
Oomoo`0QOol005eoo`03001C7E<M00<000=C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@03
001C7@00009C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@03001C7E<M00<000=C7@03001C
7E<M00<0009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@<000=C7@03001C7E<M00<0009C
7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C
7@06001C7@00001C7@000U<M10000U<M00<005<MDad010000U<M00<005<MDad010000U<M00<005<M
Dad01@0000=nkP0000000Woo00<007ooOol08Goo001NOol20003Dad00`00DaeC7@030002Dad00`00
DaeC7@040002Dad00`00DaeC7@040002Dad30003Dad00`00DaeC7@030002Dad00`00DaeC7@040002
Dad00`00DaeC7@040002Dad00`00DaeC7@030003Dad00`00DaeC7@030002Dad40002Dad00`00DaeC
7@040002Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad00`00DaeC7@030002Dad00`00DaeC
7@040002Dad00`00DaeC7@040002Dad30002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad0
0`00DaeC7@040002Dad300000gk^00000002Ool00`00Oomoo`0QOol005moo`03001C7E<M00@0009C
7@03001C7E<M00<000=C7@03001C7E<M00<0009C7@@0009C7@03001C7E<M00@0009C7@03001C7E<M
00<000=C7@03001C7E<M00<000=C7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@<000=C7@03
001C7E<M00<0009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@03
001C7E<M00<000=C7@03001C7E<M00<0009C7@@0009C7@03001C7E<M00<000=C7@03001C7E<M00<0
009C7@03001C7E<M00@0009C7@03001C7E<M00@00006O^h00000001oo`008goo001POol20003Dad0
0`00DaeC7@030003Dad00`00DaeC7@030002Dad30003Dad00`00DaeC7@030003Dad00`00DaeC7@03
0002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad40002Dad00`00
DaeC7@040002Dad00`00DaeC7@030003Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00
DaeC7@040002Dad00`00DaeC7@0400000e<M000000020002Dad00`00DaeC7@040002Dad00`00DaeC
7@040002Dad00`00DaeC7@030003Dad00`00DaeC7@030002Dad2000017k^000000000Woo00<007oo
Ool087oo001ROol00`00DaeC7@030002Dad70002Dad30003Dad00`00DaeC7@030002Dad00`00DaeC
7@040002Dad00`00DaeC7@030003Dad00`00DaeC7@030003Dad00`00DaeC7@030002Dad30003Dad0
0`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@030003Dad0
0`00DaeC7@030002Dad70002Dad30002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00
DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@05000017oo001oo`008Woo001SOol20002
Dad30003Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00DaeC7@030003Dad00`00DaeC
7@030002Dad00`00DaeC7@040002Dad00`00DaeC7@0400000e<M000000020002Dad00`00DaeC7@04
0002Dad00`00DaeC7@030003Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00DaeC7@04
00000e<M000000020002Dad00`00DaeC7@040002Dad00`00DaeC7@030003Dad00`00DaeC7@030002
Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@0400000e<M000000020002Ool2
000ROol006Eoo`80009C7@03001C7E<M00@0009C7@03001C7E<M00<000=C7@03001C7E<M00<0009C
7@03001C7E<M00@0009C7@03001C7E<M00<000=C7@03001C7E<M00<0009C7@<000=C7@03001C7E<M
00<0009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@03001C7E<M00<000=C7@03001C7E<M
00<0009C7@<000=C7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@03
001C7E<M00<000=C7@03001C7E<M00<0009C7@03001C7E<M00@00003Dad000000080009C7@800004
Ool00000000ROol006Ioo`03001C7E<M00<000=C7@03001C7E<M00<0009C7@03001C7E<M00@0009C
7@03001C7E<M00<000=C7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@<0009C7@03001C7E<M
00@0009C7@03001C7E<M00<000=C7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@03001C7E<M
00@00003Dad000000080009C7@03001C7E<M00@0009C7@03001C7E<M00<0009C7@03001C7E<M00@0
009C7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@03001C7E<M00<0009C7@<0009C7@03001C
7E<M00@002Eoo`00Igoo0P000U<M00<005<MDad010000U<M00<005<MDad00`000e<M00<005<MDad0
0`000U<M00<005<MDad010000U<M1P000U<M10000U<M00<005<MDad00`000e<M00<005<MDad00`00
0U<M00<005<MDad010000U<M00<005<MDad00`000e<M00<005<MDad00`000U<M0`000e<M00<005<M
Dad00`000U<M00<005<MDad010000U<M00<005<MDad00`000e<M00<005<MDad00`000U<M00<005<M
Dad0100000=C7@0000001P0000=C7@0000000P000U<M00<005<MDad00`00:Goo001XOol00`00DaeC
7@030003Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad40002Dad0
0`00DaeC7@030003Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00DaeC7@030003Dad0
0`00DaeC7@030002Dad00`00DaeC7@0400000e<M000000020002Dad00`00DaeC7@040002Dad00`00
DaeC7@030002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad30002
Dad00`00DaeC7@040002Dad00`00DaeC7@04000/Ool006Uoo`8000=C7@03001C7E<M00<0009C7@03
001C7E<M00@0009C7@03001C7E<M00<0009C7@<000=C7@03001C7E<M00<0009C7@03001C7E<M00@0
009C7@03001C7E<M00<000=C7@03001C7E<M00<0009C7@03001C7E<M00@00003Dad0000000D0009C
7@<000=C7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@03001C7E<M00<0009C7@03001C7E<M
00@0009C7@03001C7E<M00@00003Dad000000080009C7@03001C7E<M00<0009C7@03001C7E<M00@0
031oo`00JWoo00<005<MDad010000U<M00<005<MDad00`000U<M1`000U<M0`000U<M00<005<MDad0
10000U<M00<005<MDad00`000e<M00<005<MDad00`000U<M00<005<MDad010000U<M00<005<MDad0
0`000U<M0`000e<M00<005<MDad00`000U<M00<005<MDad010000U<M00<005<MDad00`000U<M00<0
05<MDad010000U<M00<005<MDad00`000U<M1`000U<M0`000U<M00<005<MDad010000U<M00<005<M
Dad00`00=7oo001[Ool20003Dad00`00DaeC7@030002Dad30003Dad00`00DaeC7@030002Dad00`00
DaeC7@030003Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad00`00
DaeC7@040002Dad30002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad00`00DaeC7@040002
Dad00`00DaeC7@030002Dad00`00DaeC7@0400000e<M000000020002Dad00`00DaeC7@040002Dad0
0`00DaeC7@030002Dad00`00DaeC7@04000gOol006eoo`03001C7E<M00<0009C7@<0009C7@03001C
7E<M00@0009C7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@03001C7E<M00<0009C7@03001C
7E<M00@0009C7@H0009C7@@0009C7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@03001C7E<M
00<0009C7@03001C7E<M00@0009C7@03001C7E<M00<0009C7@<000=C7@03001C7E<M00<0009C7@03
001C7E<M00@0009C7@03001C7E<M00<003]oo`00KWoo0P000U<M0`000U<M00<005<MDad00`000e<M
00<005<MDad00`000U<M00<005<MDad00`000e<M00<005<MDad00`000U<M00<005<MDad00`000U<M
10000U<M00<005<MDad00`000U<M00<005<MDad010000U<M00<005<MDad00`000U<M00<005<MDad0
10000U<M00<005<MDad00`000U<M1`0000=C7@0000000P000U<M00<005<MDad010000U<M00<005<M
Dad00`000U<M00<005<MDad01000?Woo001`Ool20002Dad00`00DaeC7@030002Dad00`00DaeC7@04
0002Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad60002Dad30003Dad00`00DaeC7@030002
Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad00`00DaeC7@040002
Dad30002Dad00`00DaeC7@030003Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00DaeC
7@030012Ool0075oo`03001C7E<M00<0009C7@03001C7E<M00<000=C7@03001C7E<M00<0009C7@03
001C7E<M00<0009C7@@0009C7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@03001C7E<M00<0
009C7@03001C7E<M00<000=C7@03001C7E<M00<0009C7@H0009C7@@0009C7@03001C7E<M00<0009C
7@03001C7E<M00@0009C7@03001C7E<M00<0009C7@03001C7E<M00@004Eoo`00LWoo0P000U<M00<0
05<MDad00`000U<M00<005<MDad010000U<M1P000U<M0`000U<M00<005<MDad010000U<M00<005<M
Dad00`000U<M00<005<MDad010000U<M00<005<MDad00`000U<M1`0000=C7@0000000P000U<M00<0
05<MDad00`000U<M00<005<MDad010000U<M00<005<MDad00`000U<M00<005<MDad010000U<M00<0
05<MDad00`00BGoo001cOol00`00DaeC7@030002Dad60002Dad40002Dad00`00DaeC7@030002Dad0
0`00DaeC7@030003Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad3
0002Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad00`00DaeC7@04
0002Dad00`00DaeC7@030002Dad7001<Ool007Aoo`80009C7@<0009C7@03001C7E<M00<000=C7@03
001C7E<M00<0009C7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@03001C7E<M00<0009C7@H0
009C7@@0009C7@03001C7E<M00<0009C7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@03001C
7E<M00<0009C7@03001C7E<M00@00003Dad00000008005=oo`00MWoo0P000U<M00<005<MDad00`00
0U<M00<005<MDad010000U<M00<005<MDad00`000U<M00<005<MDad00`000U<M1`000U<M0`000U<M
00<005<MDad00`000U<M00<005<MDad010000U<M00<005<MDad00`000U<M00<005<MDad00`000U<M
00<005<MDad0100000=C7@0000001@000U<M0`00Egoo001gOol00`00DaeC7@030002Dad00`00DaeC
7@030003Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad30002Dad00`00DaeC7@040002Dad0
0`00DaeC7@030002Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad6
0002Dad30002Dad00`00DaeC7@04001JOol007Qoo`80009C7@03001C7E<M00<0009C7@03001C7E<M
00@0009C7@H0009C7@<0009C7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@03001C7E<M00<0
009C7@03001C7E<M00<0009C7@L00003Dad000000080009C7@03001C7E<M00<0009C7@03001C7E<M
00<005ioo`00NGoo00<005<MDad00`000U<M1P000U<M10000U<M00<005<MDad00`000U<M00<005<M
Dad00`000U<M00<005<MDad00`000U<M00<005<MDad0100000=C7@0000001@000U<M0`000U<M00<0
05<MDad00`000U<M00<005<MDad010000U<M00<005<MDad00`00HGoo001jOol20002Dad30002Dad0
0`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad0
0`00DaeC7@030002Dad30002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad00`00DaeC7@03
0002Dad00`00DaeC7@04001TOol007aoo`80009C7@03001C7E<M00<0009C7@03001C7E<M00<000=C
7@03001C7E<M00<0009C7@03001C7E<M00<0009C7@H0009C7@<0009C7@03001C7E<M00<0009C7@03
001C7E<M00@0009C7@03001C7E<M00<0009C7@03001C7E<M00<006Qoo`00OGoo00<005<MDad00`00
0U<M00<005<MDad00`000U<M00<005<MDad0100000=C7@0000001@000U<M0`000U<M00<005<MDad0
0`000U<M00<005<MDad00`000U<M00<005<MDad00`000U<M00<005<MDad0100000=C7@0000001@00
Jgoo001nOol20002Dad00`00DaeC7@030002Dad60002Dad30002Dad00`00DaeC7@040002Dad00`00
DaeC7@030002Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad60002Dad3001bOol0081oo`D0
009C7@<0009C7@03001C7E<M00<0009C7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@03001C
7E<M00<00003Dad0000000D0009C7@<0009C7@03001C7E<M00<007Eoo`00PGoo00<005<MDad00`00
0U<M00<005<MDad00`000U<M00<005<MDad00`000U<M00<005<MDad00`000U<M1`0000AC7@000000
009C7@03001C7E<M00@0009C7@03001C7E<M00<007Qoo`00PWoo0P000U<M00<005<MDad00`000U<M
00<005<MDad00`000U<M1P000U<M0`000U<M00<005<MDad00`000U<M00<005<MDad00`000U<M00<0
05<MDad01000Ngoo0024Ool50002Dad60002Dad30002Dad00`00DaeC7@030002Dad00`00DaeC7@03
0002Dad00`00DaeC7@030002Dad00`00DaeC7@03001oOol008Eoo`03001C7E<M00<0009C7@03001C
7E<M00<0009C7@03001C7E<M00<0009C7@03001C7E<M00<0009C7@03001C7E<M00<0009C7@H0089o
o`00QWoo0P000U<M00<005<MDad00`000U<M00<005<MDad00`000U<M1P000U<M1P000U<M0`00R7oo
0027Ool00`00DaeC7@030002Dad600000e<M00000002000015<M001C7E<M100000AC7@00DaeC7@@0
08]oo`00R7oo0P0000=C7@0000000P0000AC7@00DaeC7@<0009C7@03001C7E<M00<0009C7@03001C
7E<M00<008moo`00RGoo0P000U<M00<005<MDad00`000U<M00<005<MDad00`000U<M1P00TWoo002:
Ool00`00DaeC7@030002Dad60002Dad3002HOol008]oo`80009C7@<0009C7@03001C7E<M00<009]o
o`00SGoo0P000U<M00<005<MDad00`00WWoo002?Ool5002QOol00001\
\>"],
  ImageRangeCache->{{{0, 308.375}, {189.938, 0}} -> {-0.249727, 0.00496695, 
  0.00547849, 0.00547849}}]
}, Open  ]],

Cell["\<\
You should be grateful that this is all that goes wrong. \
Fortunately derivatives modellers tend to use real numbers. In complex \
function theory, Newton-Raphson and other iterative solution techniques are \
famous for their generation of fractals and chaotic behaviour. But it is a \
good idea to avoid quoting implied volatilities unless you have a firm grasp \
on the characteristics of an instrument. Only the very simplest options \
behave well, in this respect. More generally, you should not report implied \
volatility results unless you have first checked that the absolute value of \
Vega is bounded away from zero. Otherwise the implied volatility is at best \
unstable and possibly undefined - in such cases it should not be reported. \
\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Remarks", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This little study has illustrated several points about the use of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ":\n\n(a) you can build symbolic models very quickly;\n(b) Greeks can be \
investigated by symbolic differentiation;\n(c) we can visualize our models \
using two- and three-dimensional graphics;\n(d) we can expose peculiar \
situations with relative ease.\n\nLater in this book we shall see how the \
exposure of significant problems can be extended to intensively numerical \
processes. \n\nFor now we return to the fundamentals. In the next three \
chapters we lay the foundations of\n\n(a) using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ";\n(b) financial mathematics of derivatives pricing;\n(c) the mathematics \
for solving partial differential equations (the Black-Scholes PDE)."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Final Note", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We live in a world where applied quantitative science in general, and \
financial modelling in particular, is increasingly dominated by purely \
numerical modelling.  The view of the author is that the analytical approach \
is being inappropriately eclipsed by a rather mindless approach involving \
flinging models at a computer and seeing what numbers come out. \n\nThis is \
not the best way to use computers. We are seeking insight into the behaviour \
of complex systems, and this is best obtained by first attempting an \
analytical treatment. This first focuses our instincts, and gives us test \
cases to check against when we finally come to adopt a purely numerical \
approach. Symbolic algebra systems such as ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " allow us to extend the analytical approach to much more complicated \
systems. The numerical approach should always be the method of absolute last \
resort, and should then be checked extensively against the analytical results \
before being used. This is the guiding principle behind the approach taken by \
this text."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 1 Bibliography", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "\nBlack, F. and Scholes, M., 1973, The pricing of options and corporate \
liabilities, ",
  StyleBox["Journal of Political Economy",
    FontSlant->"Italic"],
  ", 81, p. 637. (Reprinted and widely accessible as Chapter 1 of ",
  StyleBox["Vasicek and Beyond",
    FontSlant->"Italic"],
  ", ed. L.P. Hughston, Risk Publications, 1996.)\n\nLauterbach, B. and \
Schultz, P., 1990, An empirical study of the Black-Scholes model and its \
alternatives, ",
  StyleBox["Journal of Finance",
    FontSlant->"Italic"],
  ", 45, p. 1181.\n\nMiller, R.M., 1990, ",
  StyleBox["Computer-Aided Financial Analysis",
    FontSlant->"Italic"],
  ", Addison-Wesley.\n\nRubenstein, M. and Reiner, E., 1991, Breaking down \
the barriers, ",
  StyleBox["RISK Magazine",
    FontSlant->"Italic"],
  ", September."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
ScreenStyleEnvironment->"Printout",
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{658, 694},
WindowMargins->{{11, Automatic}, {Automatic, 2}},
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"1. Advanced Tools for Rocket Science"}]]}, {Cell[ 
        TextData[ {"1. Advanced Tools for Rocket Science"}]], Inherited, 
      Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True,
"FirstPageFace"->Right},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
PageBreakAbove->False,
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
PrivateFontOptions->{"FontType"->"Outline"},
Magnification->1.25,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;H8`_n3ohL==`Tf0b85N`?P0080004/0B`000006G<AMP01
0000I00000400@410?l00BL?00400@S<@00000000000064/01T1T00000000000
00000000007oo@S<@63oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 222, 8, 118, "Title"],
Cell[1956, 61, 173, 5, 41, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2154, 70, 247, 8, 55, "Section"],
Cell[2404, 80, 2539, 40, 415, "Text",
  PageBreakBelow->False],

Cell[CellGroupData[{
Cell[4968, 124, 250, 9, 52, "Subsection",
  PageBreakAbove->False,
  PageBreakBelow->Automatic],
Cell[5221, 135, 363, 9, 47, "Text",
  PageBreakAbove->False],
Cell[5587, 146, 539, 10, 161, "Text",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6175, 162, 160, 2, 38, "Section",
  PageBreakBelow->False],

Cell[CellGroupData[{
Cell[6360, 168, 127, 1, 31, "Subsection"],
Cell[6490, 171, 1827, 29, 260, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8354, 205, 189, 3, 31, "Subsection",
  PageBreakAbove->False,
  PageBreakWithin->Automatic],
Cell[8546, 210, 2457, 35, 381, "Text"],

Cell[CellGroupData[{
Cell[11028, 249, 101, 1, 27, "Subsubsection"],
Cell[11132, 252, 679, 12, 62, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11872, 271, 144, 2, 38, "Section",
  PageBreakAbove->False],
Cell[12019, 275, 908, 17, 117, "Text"],

Cell[CellGroupData[{
Cell[12952, 296, 159, 2, 31, "Subsection",
  PageBreakAbove->Automatic],
Cell[13114, 300, 1718, 26, 172, "Text"],

Cell[CellGroupData[{
Cell[14857, 330, 123, 2, 27, "Subsubsection",
  PageBreakAbove->False],
Cell[14983, 334, 2046, 31, 216, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17066, 370, 125, 2, 27, "Subsubsection",
  PageBreakAbove->False],
Cell[17194, 374, 2842, 50, 293, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20073, 429, 168, 6, 27, "Subsubsection"],
Cell[20244, 437, 1172, 27, 106, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[21465, 470, 91, 1, 31, "Subsection"],
Cell[21559, 473, 1174, 20, 117, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[22782, 499, 153, 4, 53, "Section"],
Cell[22938, 505, 1677, 26, 183, "Text"],

Cell[CellGroupData[{
Cell[24640, 535, 95, 1, 27, "Subsubsection"],
Cell[24738, 538, 566, 11, 51, "Text"],
Cell[25307, 551, 454, 11, 112, "Input"],
Cell[25764, 564, 320, 6, 40, "Text"],

Cell[CellGroupData[{
Cell[26109, 574, 217, 4, 37, "Input"],
Cell[26329, 580, 15286, 422, 128, 3912, 277, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[41630, 1005, 282, 7, 30, "Text"],
Cell[41915, 1014, 225, 5, 35, "Input"],
Cell[42143, 1021, 185, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[42353, 1029, 142, 2, 25, "Input"],
Cell[42498, 1033, 54, 1, 35, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42589, 1039, 156, 3, 25, "Input"],
Cell[42748, 1044, 54, 1, 35, "Output"]
}, Open  ]],
Cell[42817, 1048, 195, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[43037, 1056, 152, 3, 25, "Input"],
Cell[43192, 1061, 55, 1, 35, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[43296, 1068, 132, 1, 27, "Subsubsection"],
Cell[43431, 1071, 348, 6, 40, "Text"],
Cell[43782, 1079, 644, 13, 113, "Input"],
Cell[44429, 1094, 382, 7, 40, "Text"],

Cell[CellGroupData[{
Cell[44836, 1105, 233, 4, 37, "Input"],
Cell[45072, 1111, 15251, 517, 130, 5686, 394, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[60372, 1634, 171, 4, 27, "Subsubsection"],
Cell[60546, 1640, 1127, 18, 117, "Text"],

Cell[CellGroupData[{
Cell[61698, 1662, 224, 4, 37, "Input"],
Cell[61925, 1668, 13365, 458, 131, 4913, 349, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[75327, 2131, 226, 4, 37, "Input"],
Cell[75556, 2137, 12929, 494, 131, 5644, 399, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[88500, 2634, 190, 4, 30, "Text"],

Cell[CellGroupData[{
Cell[88715, 2642, 230, 4, 37, "Input"],
Cell[88948, 2648, 12204, 452, 131, 5186, 361, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[101167, 3103, 249, 7, 30, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[101453, 3115, 167, 3, 27, "Subsubsection",
  PageBreakAbove->False],
Cell[101623, 3120, 598, 10, 62, "Text"],

Cell[CellGroupData[{
Cell[102246, 3134, 238, 6, 46, "Input"],
Cell[102487, 3142, 174554, 4046, 200, 127694, 3463, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[277056, 7191, 832, 13, 123, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[277925, 7209, 86, 1, 40, "Subsection"],
Cell[278014, 7212, 912, 16, 222, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[278963, 7233, 91, 1, 40, "Subsection"],
Cell[279057, 7236, 1188, 19, 180, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[280294, 7261, 98, 1, 48, "Section"],
Cell[280395, 7264, 882, 22, 165, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

