(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    852204,      24834]*)
(*NotebookOutlinePosition[    854095,      24891]*)
(*  CellTagsIndexPosition[    854051,      24887]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 6. 
Binary Options and the Normal Distribution  \
\>", "Title",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["\<\
When \[CapitalDelta] Becomes a Delta Function, Hedging Gets Nasty, \
and Continued Fractions Make an Elegant Appearance\
\>", "Subsubtitle",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["6.1 Introduction", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "In Chapter 4, after finding the simpler log and power solutions of the \
Black-Scholes PDE, we introduced general methods for solving the PDE with a \
given initial condition. We met some examples of Binary options, based on a \
simple bet that pays off a fixed amount at maturity if the stock price is \
above or below a certain value. There are several variations on this theme, \
some of which are described by Hull (1996), and a substantial catalogue is \
given by Rubenstein and Reiner (1991). These include cases where the cash is \
paid when the strike is hit, or if the amount paid at maturity is based on \
the stock price. \n\nWe shall confine the discussion to a careful \
consideration of the simple Binary Call and Put defined in Section 4.6, where \
an amount ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  ", or nothing, is paid at maturity based on whether the asset price at \
maturity is above or below a certain strike level ",
  Cell[BoxData[
      \(TraditionalForm\`K\)]],
  ". \n\nThis simple case alone will raise issues of financial and \
mathematical importance. In particular, we need to address:\n\n(a) the \
difficulties in hedging binary options;\n(b) how to compute the cumulative \
normal distribution.\n\nMany other cases can be valued using a combination of \
the rebate solutions and cut-off power calls/puts discussed previously."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["6.2 Binary Cash Options Defined and Visualized", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We shall need a model of the cumulative normal distribution. This is \
usually denoted by \"N\" in standard derivatives texts, but in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " the ",
  StyleBox["N",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function is reserved for purely numerical modelling. So we shall use the \
term \"Norm\", as before, and define it in terms of the Error function, for \
which ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " has both a symbolic characterization and a high-precision numerical \
method for evaluation. At the end of this chapter, in Section 6.5, we shall \
consider other numerical techniques for estimating the cumulative normal \
distribution function. So we define"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Norm[(z_)?NumberQ] := N[0.5*Erf[z/Sqrt[2]] + 0.5]; 
Norm[x_] := (1 + Erf[x/Sqrt[2]])/2\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Using the normal distribution, we can define the formulae for Binary Calls \
and Puts. We do it in two stages - first we introduce the ",
  Cell[BoxData[
      \(TraditionalForm\`d\_2\)]],
  " function defined in Chapter 4:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
dtwo[S_, \[Sigma]_, K_, t_, r_, q_] :=
 ((r - q)*t + Log[S/K])/(\[Sigma]*Sqrt[t]) - (\[Sigma]*Sqrt[t])/2; \
\>", 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["The valuation of the simple Binary Call and Put is then", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
BinaryCall[B_, S_, K_, \[Sigma]_, r_, q_, t_] := 
B*Exp[-r*t]*Norm[dtwo[S, \[Sigma], K, t, r, q]]; 
BinaryPut[B_, S_, K_, \[Sigma]_, r_, q_, t_] :=
B*Exp[-r*t]*Norm[-dtwo[S, \[Sigma], K, t, r, q]]; \
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot3D[
      BinaryCall[1, \ S, \ 100, \ 0.2, \ 0.1, \ 0, \ t], \ {S, \ 90, \ 
        110}, \ {t, \ 0.0001, \ 1}, \ PlotPoints\  -> \ 40]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
6.10623e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -16.7601 -9 ]
[.3345 .12542 1.2399 0 ]
[.50253 .05935 -15.8219 -9 ]
[.50253 .05935 2.17811 0 ]
[.68156 -0.01098 -14.8837 -9 ]
[.68156 -0.01098 3.11632 0 ]
[.76742 .09699 0 -6.16191 ]
[.76742 .09699 18 2.83809 ]
[.8281 .1899 0 -6.07249 ]
[.8281 .1899 18 2.92751 ]
[.88368 .27501 0 -5.9922 ]
[.88368 .27501 18 3.0078 ]
[.93477 .35326 0 -5.91971 ]
[.93477 .35326 18 3.08029 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.02411 .26511 -6 -2.74232 ]
[.02411 .26511 0 6.25768 ]
[.01568 .31795 -24 -2.78699 ]
[.01568 .31795 0 6.21301 ]
[.00696 .37264 -18 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00208 .42927 -24 -2.88161 ]
[-0.00208 .42927 0 6.11839 ]
[-0.01144 .48795 -6 -2.93176 ]
[-0.01144 .48795 0 6.06824 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(90)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(95)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(100)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(105)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(110)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.75512 .10154 m
.74896 .10381 L
s
[(0.2)] .76742 .09699 -1 .36931 Mshowa
.81571 .19423 m
.80952 .19639 L
s
[(0.4)] .8281 .1899 -1 .34944 Mshowa
.87122 .27914 m
.86499 .2812 L
s
[(0.6)] .88368 .27501 -1 .3316 Mshowa
.92226 .35721 m
.916 .35918 L
s
[(0.8)] .93477 .35326 -1 .31549 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(1)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.7059 .02625 m
.70222 .02766 L
s
.72269 .05193 m
.71901 .05333 L
s
.73909 .07702 m
.7354 .0784 L
s
.77078 .1255 m
.76708 .12684 L
s
.78609 .14892 m
.78239 .15025 L
s
.80107 .17182 m
.79735 .17314 L
s
.83004 .21614 m
.82632 .21743 L
s
.84406 .23759 m
.84033 .23886 L
s
.85779 .25859 m
.85405 .25984 L
s
.88438 .29926 m
.88064 .30049 L
s
.89727 .31898 m
.89352 .32019 L
s
.90989 .33829 m
.90614 .33949 L
s
.93438 .37576 m
.93063 .37693 L
s
.94627 .39394 m
.94251 .39509 L
s
.95792 .41176 m
.95415 .41291 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
[(0)] .02411 .26511 1 -0.3906 Mshowa
.02795 .31328 m
.03408 .31095 L
s
[(0.25)] .01568 .31795 1 -0.38067 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0.5)] .00696 .37264 1 -0.37036 Mshowa
.01027 .42483 m
.01644 .42261 L
s
[(0.75)] -0.00208 .42927 1 -0.35964 Mshowa
.00095 .48363 m
.00715 .48147 L
s
[(1)] -0.01144 .48795 1 -0.3485 Mshowa
.125 Mabswid
.03468 .27078 m
.03835 .26935 L
s
.03301 .2813 m
.03669 .27988 L
s
.03134 .29189 m
.03501 .29047 L
s
.02965 .30255 m
.03332 .30114 L
s
.02623 .32409 m
.02991 .3227 L
s
.02451 .33497 m
.02819 .33359 L
s
.02277 .34593 m
.02646 .34455 L
s
.02102 .35697 m
.02471 .35559 L
s
.01749 .37927 m
.02118 .37791 L
s
.0157 .39054 m
.0194 .38919 L
s
.0139 .40189 m
.0176 .40054 L
s
.01209 .41332 m
.01579 .41198 L
s
.00843 .43642 m
.01214 .4351 L
s
.00658 .4481 m
.01029 .44678 L
s
.00472 .45986 m
.00843 .45855 L
s
.00284 .4717 m
.00656 .4704 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.593 .713 .927 r
.39434 .67538 .40235 .68313 .41572 .68133 .40775 .67358 Metetra
.593 .714 .928 r
.40775 .67358 .41572 .68133 .42918 .67952 .42124 .67176 Metetra
.593 .715 .929 r
.42124 .67176 .42918 .67952 .44272 .67769 .43482 .66993 Metetra
.593 .716 .929 r
.43482 .66993 .44272 .67769 .45635 .67584 .44848 .66808 Metetra
.593 .717 .93 r
.44848 .66808 .45635 .67584 .47006 .67396 .46223 .6662 Metetra
.593 .717 .93 r
.46223 .6662 .47006 .67396 .48386 .67207 .47607 .6643 Metetra
.593 .718 .931 r
.47607 .6643 .48386 .67207 .49774 .67014 .48999 .66238 Metetra
.594 .719 .932 r
.48999 .66238 .49774 .67014 .51171 .66819 .504 .66043 Metetra
.594 .72 .932 r
.504 .66043 .51171 .66819 .52577 .66622 .5181 .65845 Metetra
.594 .721 .933 r
.5181 .65845 .52577 .66622 .53991 .66421 .53229 .65644 Metetra
.595 .722 .933 r
.53229 .65644 .53991 .66421 .55415 .66217 .54657 .6544 Metetra
.595 .723 .934 r
.54657 .6544 .55415 .66217 .56848 .6601 .56094 .65233 Metetra
.595 .724 .934 r
.56094 .65233 .56848 .6601 .58289 .65799 .5754 .65023 Metetra
.596 .726 .935 r
.5754 .65023 .58289 .65799 .5974 .65585 .58995 .64808 Metetra
.596 .727 .935 r
.58995 .64808 .5974 .65585 .61199 .65367 .60459 .6459 Metetra
.597 .728 .936 r
.60459 .6459 .61199 .65367 .62668 .65146 .61933 .64369 Metetra
.597 .729 .936 r
.61933 .64369 .62668 .65146 .64146 .6492 .63416 .64143 Metetra
.598 .73 .937 r
.63416 .64143 .64146 .6492 .65634 .64691 .64908 .63913 Metetra
.599 .731 .937 r
.64908 .63913 .65634 .64691 .67131 .64457 .6641 .63679 Metetra
.599 .732 .938 r
.6641 .63679 .67131 .64457 .68637 .64219 .67922 .63441 Metetra
.6 .733 .938 r
.67922 .63441 .68637 .64219 .70152 .63977 .69443 .63198 Metetra
.601 .735 .939 r
.69443 .63198 .70152 .63977 .71678 .63731 .70973 .62951 Metetra
.601 .736 .939 r
.70973 .62951 .71678 .63731 .73212 .6348 .72513 .62699 Metetra
.602 .737 .939 r
.72513 .62699 .73212 .6348 .74756 .63224 .74063 .62443 Metetra
.603 .738 .94 r
.74063 .62443 .74756 .63224 .7631 .62964 .75623 .62182 Metetra
.603 .739 .94 r
.75623 .62182 .7631 .62964 .77874 .62699 .77192 .61916 Metetra
.604 .74 .941 r
.77192 .61916 .77874 .62699 .79447 .62429 .78772 .61645 Metetra
.605 .741 .941 r
.78772 .61645 .79447 .62429 .8103 .62154 .80361 .61369 Metetra
.606 .742 .941 r
.80361 .61369 .8103 .62154 .82623 .61875 .8196 .61088 Metetra
.607 .743 .942 r
.8196 .61088 .82623 .61875 .84226 .6159 .83569 .60803 Metetra
.607 .744 .942 r
.83569 .60803 .84226 .6159 .85839 .61301 .85189 .60512 Metetra
.608 .745 .942 r
.85189 .60512 .85839 .61301 .87462 .61006 .86818 .60215 Metetra
.609 .746 .942 r
.86818 .60215 .87462 .61006 .89094 .60707 .88458 .59914 Metetra
.61 .747 .943 r
.88458 .59914 .89094 .60707 .90737 .60402 .90107 .59607 Metetra
.61 .748 .943 r
.90107 .59607 .90737 .60402 .9239 .60092 .91767 .59296 Metetra
.611 .749 .943 r
.91767 .59296 .9239 .60092 .94053 .59777 .93438 .58978 Metetra
.612 .75 .943 r
.93438 .58978 .94053 .59777 .95727 .59456 .95118 .58656 Metetra
.613 .751 .944 r
.95118 .58656 .95727 .59456 .97411 .59131 .96809 .58328 Metetra
.614 .752 .944 r
.96809 .58328 .97411 .59131 .99105 .588 .98511 .57994 Metetra
.592 .712 .927 r
.38627 .66753 .39434 .67538 .40775 .67358 .39971 .66573 Metetra
.592 .713 .928 r
.39971 .66573 .40775 .67358 .42124 .67176 .41324 .66391 Metetra
.592 .714 .928 r
.41324 .66391 .42124 .67176 .43482 .66993 .42685 .66208 Metetra
.592 .715 .929 r
.42685 .66208 .43482 .66993 .44848 .66808 .44055 .66022 Metetra
.592 .716 .93 r
.44055 .66022 .44848 .66808 .46223 .6662 .45434 .65835 Metetra
.592 .716 .93 r
.45434 .65835 .46223 .6662 .47607 .6643 .46821 .65645 Metetra
.592 .717 .931 r
.46821 .65645 .47607 .6643 .48999 .66238 .48217 .65453 Metetra
.593 .718 .931 r
.48217 .65453 .48999 .66238 .504 .66043 .49622 .65258 Metetra
.593 .719 .932 r
.49622 .65258 .504 .66043 .5181 .65845 .51036 .6506 Metetra
.593 .72 .933 r
.51036 .6506 .5181 .65845 .53229 .65644 .52459 .64859 Metetra
.593 .721 .933 r
.52459 .64859 .53229 .65644 .54657 .6544 .53891 .64655 Metetra
.594 .723 .934 r
.53891 .64655 .54657 .6544 .56094 .65233 .55332 .64448 Metetra
.594 .724 .934 r
.55332 .64448 .56094 .65233 .5754 .65023 .56783 .64237 Metetra
.595 .725 .935 r
.56783 .64237 .5754 .65023 .58995 .64808 .58243 .64023 Metetra
.595 .726 .935 r
.58243 .64023 .58995 .64808 .60459 .6459 .59712 .63805 Metetra
.596 .727 .936 r
.59712 .63805 .60459 .6459 .61933 .64369 .6119 .63583 Metetra
.596 .728 .936 r
.6119 .63583 .61933 .64369 .63416 .64143 .62678 .63357 Metetra
.597 .729 .937 r
.62678 .63357 .63416 .64143 .64908 .63913 .64176 .63127 Metetra
.598 .73 .937 r
.64176 .63127 .64908 .63913 .6641 .63679 .65683 .62893 Metetra
.598 .732 .938 r
.65683 .62893 .6641 .63679 .67922 .63441 .67199 .62654 Metetra
.599 .733 .938 r
.67199 .62654 .67922 .63441 .69443 .63198 .68726 .62411 Metetra
.6 .734 .939 r
.68726 .62411 .69443 .63198 .70973 .62951 .70262 .62163 Metetra
.6 .735 .939 r
.70262 .62163 .70973 .62951 .72513 .62699 .71808 .61911 Metetra
.601 .736 .94 r
.71808 .61911 .72513 .62699 .74063 .62443 .73363 .61654 Metetra
.602 .737 .94 r
.73363 .61654 .74063 .62443 .75623 .62182 .74929 .61392 Metetra
.602 .739 .94 r
.74929 .61392 .75623 .62182 .77192 .61916 .76504 .61125 Metetra
.603 .74 .941 r
.76504 .61125 .77192 .61916 .78772 .61645 .7809 .60853 Metetra
.604 .741 .941 r
.7809 .60853 .78772 .61645 .80361 .61369 .79685 .60576 Metetra
.605 .742 .942 r
.79685 .60576 .80361 .61369 .8196 .61088 .8129 .60294 Metetra
.606 .743 .942 r
.8129 .60294 .8196 .61088 .83569 .60803 .82906 .60007 Metetra
.606 .744 .942 r
.82906 .60007 .83569 .60803 .85189 .60512 .84532 .59715 Metetra
.607 .745 .942 r
.84532 .59715 .85189 .60512 .86818 .60215 .86168 .59417 Metetra
.608 .746 .943 r
.86168 .59417 .86818 .60215 .88458 .59914 .87814 .59114 Metetra
.609 .747 .943 r
.87814 .59114 .88458 .59914 .90107 .59607 .89471 .58805 Metetra
.61 .748 .943 r
.89471 .58805 .90107 .59607 .91767 .59296 .91138 .58492 Metetra
.611 .749 .944 r
.91138 .58492 .91767 .59296 .93438 .58978 .92815 .58172 Metetra
.611 .75 .944 r
.92815 .58172 .93438 .58978 .95118 .58656 .94503 .57847 Metetra
.612 .751 .944 r
.94503 .57847 .95118 .58656 .96809 .58328 .96201 .57517 Metetra
.613 .752 .944 r
.96201 .57517 .96809 .58328 .98511 .57994 .9791 .57181 Metetra
.591 .711 .927 r
.37813 .65958 .38627 .66753 .39971 .66573 .3916 .65778 Metetra
.591 .712 .927 r
.3916 .65778 .39971 .66573 .41324 .66391 .40516 .65596 Metetra
.591 .713 .928 r
.40516 .65596 .41324 .66391 .42685 .66208 .41881 .65413 Metetra
.591 .714 .929 r
.41881 .65413 .42685 .66208 .44055 .66022 .43254 .65227 Metetra
.591 .715 .929 r
.43254 .65227 .44055 .66022 .45434 .65835 .44637 .6504 Metetra
.591 .715 .93 r
.44637 .6504 .45434 .65835 .46821 .65645 .46028 .6485 Metetra
.591 .716 .931 r
.46028 .6485 .46821 .65645 .48217 .65453 .47428 .64658 Metetra
.591 .717 .931 r
.47428 .64658 .48217 .65453 .49622 .65258 .48837 .64463 Metetra
.592 .718 .932 r
.48837 .64463 .49622 .65258 .51036 .6506 .50255 .64265 Metetra
.592 .72 .932 r
.50255 .64265 .51036 .6506 .52459 .64859 .51682 .64064 Metetra
.592 .721 .933 r
.51682 .64064 .52459 .64859 .53891 .64655 .53118 .6386 Metetra
.593 .722 .934 r
.53118 .6386 .53891 .64655 .55332 .64448 .54564 .63653 Metetra
.593 .723 .934 r
.54564 .63653 .55332 .64448 .56783 .64237 .56019 .63443 Metetra
.594 .724 .935 r
.56019 .63443 .56783 .64237 .58243 .64023 .57483 .63228 Metetra
.594 .725 .935 r
.57483 .63228 .58243 .64023 .59712 .63805 .58957 .6301 Metetra
.595 .726 .936 r
.58957 .6301 .59712 .63805 .6119 .63583 .6044 .62788 Metetra
.595 .727 .936 r
.6044 .62788 .6119 .63583 .62678 .63357 .61933 .62562 Metetra
.596 .729 .937 r
.61933 .62562 .62678 .63357 .64176 .63127 .63436 .62332 Metetra
.596 .73 .937 r
.63436 .62332 .64176 .63127 .65683 .62893 .64948 .62098 Metetra
.597 .731 .938 r
.64948 .62098 .65683 .62893 .67199 .62654 .6647 .61859 Metetra
.598 .732 .938 r
.6647 .61859 .67199 .62654 .68726 .62411 .68002 .61615 Metetra
.598 .733 .939 r
.68002 .61615 .68726 .62411 .70262 .62163 .69543 .61367 Metetra
.599 .735 .939 r
.69543 .61367 .70262 .62163 .71808 .61911 .71095 .61114 Metetra
.6 .736 .94 r
.71095 .61114 .71808 .61911 .73363 .61654 .72656 .60857 Metetra
.601 .737 .94 r
.72656 .60857 .73363 .61654 .74929 .61392 .74227 .60594 Metetra
.601 .738 .941 r
.74227 .60594 .74929 .61392 .76504 .61125 .75809 .60326 Metetra
.602 .739 .941 r
.75809 .60326 .76504 .61125 .7809 .60853 .774 .60053 Metetra
.603 .741 .941 r
.774 .60053 .7809 .60853 .79685 .60576 .79002 .59775 Metetra
.604 .742 .942 r
.79002 .59775 .79685 .60576 .8129 .60294 .80614 .59492 Metetra
.605 .743 .942 r
.80614 .59492 .8129 .60294 .82906 .60007 .82236 .59204 Metetra
.606 .744 .942 r
.82236 .59204 .82906 .60007 .84532 .59715 .83868 .5891 Metetra
.606 .745 .943 r
.83868 .5891 .84532 .59715 .86168 .59417 .85511 .58611 Metetra
.607 .746 .943 r
.85511 .58611 .86168 .59417 .87814 .59114 .87164 .58306 Metetra
.608 .747 .943 r
.87164 .58306 .87814 .59114 .89471 .58805 .88828 .57996 Metetra
.609 .748 .944 r
.88828 .57996 .89471 .58805 .91138 .58492 .90502 .5768 Metetra
.61 .749 .944 r
.90502 .5768 .91138 .58492 .92815 .58172 .92186 .57359 Metetra
.611 .75 .944 r
.92186 .57359 .92815 .58172 .94503 .57847 .93881 .57032 Metetra
.611 .751 .944 r
.93881 .57032 .94503 .57847 .96201 .57517 .95587 .56699 Metetra
.612 .752 .945 r
.95587 .56699 .96201 .57517 .9791 .57181 .97304 .56361 Metetra
.59 .71 .926 r
.36991 .65153 .37813 .65958 .3916 .65778 .38342 .64973 Metetra
.59 .711 .927 r
.38342 .64973 .3916 .65778 .40516 .65596 .39701 .64791 Metetra
.59 .712 .928 r
.39701 .64791 .40516 .65596 .41881 .65413 .4107 .64607 Metetra
.59 .713 .928 r
.4107 .64607 .41881 .65413 .43254 .65227 .42447 .64422 Metetra
.59 .713 .929 r
.42447 .64422 .43254 .65227 .44637 .6504 .43832 .64234 Metetra
.59 .714 .93 r
.43832 .64234 .44637 .6504 .46028 .6485 .45227 .64045 Metetra
.59 .715 .93 r
.45227 .64045 .46028 .6485 .47428 .64658 .46631 .63853 Metetra
.59 .716 .931 r
.46631 .63853 .47428 .64658 .48837 .64463 .48044 .63658 Metetra
.591 .717 .932 r
.48044 .63658 .48837 .64463 .50255 .64265 .49466 .6346 Metetra
.591 .719 .932 r
.49466 .6346 .50255 .64265 .51682 .64064 .50897 .6326 Metetra
.591 .72 .933 r
.50897 .6326 .51682 .64064 .53118 .6386 .52338 .63056 Metetra
.592 .721 .934 r
.52338 .63056 .53118 .6386 .54564 .63653 .53788 .62849 Metetra
.592 .722 .934 r
.53788 .62849 .54564 .63653 .56019 .63443 .55248 .62639 Metetra
.592 .723 .935 r
.55248 .62639 .56019 .63443 .57483 .63228 .56716 .62425 Metetra
.593 .724 .935 r
.56716 .62425 .57483 .63228 .58957 .6301 .58195 .62207 Metetra
.594 .725 .936 r
.58195 .62207 .58957 .6301 .6044 .62788 .59683 .61985 Metetra
.594 .727 .936 r
.59683 .61985 .6044 .62788 .61933 .62562 .61181 .61759 Metetra
.595 .728 .937 r
.61181 .61759 .61933 .62562 .63436 .62332 .62689 .61529 Metetra
.595 .729 .938 r
.62689 .61529 .63436 .62332 .64948 .62098 .64206 .61294 Metetra
.596 .73 .938 r
.64206 .61294 .64948 .62098 .6647 .61859 .65733 .61055 Metetra
.597 .732 .939 r
.65733 .61055 .6647 .61859 .68002 .61615 .6727 .60811 Metetra
.597 .733 .939 r
.6727 .60811 .68002 .61615 .69543 .61367 .68817 .60563 Metetra
.598 .734 .94 r
.68817 .60563 .69543 .61367 .71095 .61114 .70374 .60309 Metetra
.599 .735 .94 r
.70374 .60309 .71095 .61114 .72656 .60857 .71942 .60051 Metetra
.6 .737 .94 r
.71942 .60051 .72656 .60857 .74227 .60594 .73519 .59788 Metetra
.6 .738 .941 r
.73519 .59788 .74227 .60594 .75809 .60326 .75106 .59519 Metetra
.601 .739 .941 r
.75106 .59519 .75809 .60326 .774 .60053 .76704 .59246 Metetra
.602 .74 .942 r
.76704 .59246 .774 .60053 .79002 .59775 .78312 .58967 Metetra
.603 .741 .942 r
.78312 .58967 .79002 .59775 .80614 .59492 .7993 .58682 Metetra
.604 .743 .942 r
.7993 .58682 .80614 .59492 .82236 .59204 .81559 .58393 Metetra
.605 .744 .943 r
.81559 .58393 .82236 .59204 .83868 .5891 .83198 .58097 Metetra
.606 .745 .943 r
.83198 .58097 .83868 .5891 .85511 .58611 .84847 .57797 Metetra
.606 .746 .943 r
.84847 .57797 .85511 .58611 .87164 .58306 .86507 .5749 Metetra
.607 .747 .944 r
.86507 .5749 .87164 .58306 .88828 .57996 .88178 .57178 Metetra
.608 .748 .944 r
.88178 .57178 .88828 .57996 .90502 .5768 .89859 .56861 Metetra
.609 .749 .944 r
.89859 .56861 .90502 .5768 .92186 .57359 .91551 .56537 Metetra
.61 .75 .944 r
.91551 .56537 .92186 .57359 .93881 .57032 .93253 .56208 Metetra
.611 .751 .945 r
.93253 .56208 .93881 .57032 .95587 .56699 .94966 .55873 Metetra
.612 .752 .945 r
.94966 .55873 .95587 .56699 .97304 .56361 .96691 .55532 Metetra
.589 .709 .926 r
.36162 .64338 .36991 .65153 .38342 .64973 .37516 .64157 Metetra
.589 .71 .927 r
.37516 .64157 .38342 .64973 .39701 .64791 .38879 .63975 Metetra
.589 .711 .927 r
.38879 .63975 .39701 .64791 .4107 .64607 .40251 .63792 Metetra
.589 .711 .928 r
.40251 .63792 .4107 .64607 .42447 .64422 .41631 .63606 Metetra
.589 .712 .929 r
.41631 .63606 .42447 .64422 .43832 .64234 .43021 .63419 Metetra
.589 .713 .929 r
.43021 .63419 .43832 .64234 .45227 .64045 .4442 .6323 Metetra
.589 .714 .93 r
.4442 .6323 .45227 .64045 .46631 .63853 .45827 .63038 Metetra
.589 .715 .931 r
.45827 .63038 .46631 .63853 .48044 .63658 .47244 .62843 Metetra
.589 .716 .932 r
.47244 .62843 .48044 .63658 .49466 .6346 .4867 .62646 Metetra
.59 .718 .932 r
.4867 .62646 .49466 .6346 .50897 .6326 .50106 .62446 Metetra
.59 .719 .933 r
.50106 .62446 .50897 .6326 .52338 .63056 .5155 .62243 Metetra
.59 .72 .933 r
.5155 .62243 .52338 .63056 .53788 .62849 .53005 .62036 Metetra
.591 .721 .934 r
.53005 .62036 .53788 .62849 .55248 .62639 .54469 .61826 Metetra
.591 .722 .935 r
.54469 .61826 .55248 .62639 .56716 .62425 .55942 .61612 Metetra
.592 .723 .935 r
.55942 .61612 .56716 .62425 .58195 .62207 .57426 .61394 Metetra
.592 .725 .936 r
.57426 .61394 .58195 .62207 .59683 .61985 .58918 .61172 Metetra
.593 .726 .937 r
.58918 .61172 .59683 .61985 .61181 .61759 .60421 .60946 Metetra
.593 .727 .937 r
.60421 .60946 .61181 .61759 .62689 .61529 .61934 .60716 Metetra
.594 .728 .938 r
.61934 .60716 .62689 .61529 .64206 .61294 .63456 .60482 Metetra
.595 .73 .938 r
.63456 .60482 .64206 .61294 .65733 .61055 .64989 .60242 Metetra
.595 .731 .939 r
.64989 .60242 .65733 .61055 .6727 .60811 .66531 .59998 Metetra
.596 .732 .939 r
.66531 .59998 .6727 .60811 .68817 .60563 .68084 .5975 Metetra
.597 .734 .94 r
.68084 .5975 .68817 .60563 .70374 .60309 .69647 .59496 Metetra
.598 .735 .94 r
.69647 .59496 .70374 .60309 .71942 .60051 .7122 .59237 Metetra
.599 .736 .941 r
.7122 .59237 .71942 .60051 .73519 .59788 .72803 .58973 Metetra
.599 .737 .941 r
.72803 .58973 .73519 .59788 .75106 .59519 .74396 .58704 Metetra
.6 .739 .942 r
.74396 .58704 .75106 .59519 .76704 .59246 .76 .5843 Metetra
.601 .74 .942 r
.76 .5843 .76704 .59246 .78312 .58967 .77615 .5815 Metetra
.602 .741 .942 r
.77615 .5815 .78312 .58967 .7993 .58682 .79239 .57865 Metetra
.603 .742 .943 r
.79239 .57865 .7993 .58682 .81559 .58393 .80874 .57574 Metetra
.604 .743 .943 r
.80874 .57574 .81559 .58393 .83198 .58097 .8252 .57277 Metetra
.605 .745 .943 r
.8252 .57277 .83198 .58097 .84847 .57797 .84176 .56975 Metetra
.605 .746 .944 r
.84176 .56975 .84847 .57797 .86507 .5749 .85843 .56667 Metetra
.606 .747 .944 r
.85843 .56667 .86507 .5749 .88178 .57178 .87521 .56353 Metetra
.607 .748 .944 r
.87521 .56353 .88178 .57178 .89859 .56861 .89209 .56034 Metetra
.608 .749 .945 r
.89209 .56034 .89859 .56861 .91551 .56537 .90908 .55708 Metetra
.609 .75 .945 r
.90908 .55708 .91551 .56537 .93253 .56208 .92618 .55377 Metetra
.61 .751 .945 r
.92618 .55377 .93253 .56208 .94966 .55873 .94339 .5504 Metetra
.611 .752 .945 r
.94339 .5504 .94966 .55873 .96691 .55532 .96071 .54696 Metetra
.588 .708 .926 r
.35326 .63512 .36162 .64338 .37516 .64157 .36683 .63331 Metetra
.588 .709 .926 r
.36683 .63331 .37516 .64157 .38879 .63975 .3805 .63149 Metetra
.588 .709 .927 r
.3805 .63149 .38879 .63975 .40251 .63792 .39425 .62966 Metetra
.588 .71 .928 r
.39425 .62966 .40251 .63792 .41631 .63606 .40809 .6278 Metetra
.588 .711 .929 r
.40809 .6278 .41631 .63606 .43021 .63419 .42202 .62593 Metetra
.588 .712 .929 r
.42202 .62593 .43021 .63419 .4442 .6323 .43604 .62404 Metetra
.588 .713 .93 r
.43604 .62404 .4442 .6323 .45827 .63038 .45016 .62213 Metetra
.588 .714 .931 r
.45016 .62213 .45827 .63038 .47244 .62843 .46437 .62018 Metetra
.588 .715 .931 r
.46437 .62018 .47244 .62843 .4867 .62646 .47867 .61822 Metetra
.589 .716 .932 r
.47867 .61822 .4867 .62646 .50106 .62446 .49306 .61622 Metetra
.589 .718 .933 r
.49306 .61622 .50106 .62446 .5155 .62243 .50755 .61419 Metetra
.589 .719 .933 r
.50755 .61419 .5155 .62243 .53005 .62036 .52214 .61213 Metetra
.59 .72 .934 r
.52214 .61213 .53005 .62036 .54469 .61826 .53682 .61003 Metetra
.59 .721 .935 r
.53682 .61003 .54469 .61826 .55942 .61612 .55161 .60789 Metetra
.591 .723 .935 r
.55161 .60789 .55942 .61612 .57426 .61394 .56649 .60572 Metetra
.591 .724 .936 r
.56649 .60572 .57426 .61394 .58918 .61172 .58146 .6035 Metetra
.592 .725 .937 r
.58146 .6035 .58918 .61172 .60421 .60946 .59654 .60125 Metetra
.592 .726 .937 r
.59654 .60125 .60421 .60946 .61934 .60716 .61172 .59895 Metetra
.593 .728 .938 r
.61172 .59895 .61934 .60716 .63456 .60482 .62699 .5966 Metetra
.594 .729 .938 r
.62699 .5966 .63456 .60482 .64989 .60242 .64237 .59421 Metetra
.594 .73 .939 r
.64237 .59421 .64989 .60242 .66531 .59998 .65785 .59177 Metetra
.595 .732 .939 r
.65785 .59177 .66531 .59998 .68084 .5975 .67343 .58928 Metetra
.596 .733 .94 r
.67343 .58928 .68084 .5975 .69647 .59496 .68912 .58674 Metetra
.597 .734 .94 r
.68912 .58674 .69647 .59496 .7122 .59237 .70491 .58415 Metetra
.597 .736 .941 r
.70491 .58415 .7122 .59237 .72803 .58973 .7208 .58151 Metetra
.598 .737 .941 r
.7208 .58151 .72803 .58973 .74396 .58704 .73679 .57881 Metetra
.599 .738 .942 r
.73679 .57881 .74396 .58704 .76 .5843 .75289 .57606 Metetra
.6 .739 .942 r
.75289 .57606 .76 .5843 .77615 .5815 .7691 .57325 Metetra
.601 .741 .943 r
.7691 .57325 .77615 .5815 .79239 .57865 .78541 .57039 Metetra
.602 .742 .943 r
.78541 .57039 .79239 .57865 .80874 .57574 .80183 .56747 Metetra
.603 .743 .943 r
.80183 .56747 .80874 .57574 .8252 .57277 .81835 .56449 Metetra
.604 .744 .944 r
.81835 .56449 .8252 .57277 .84176 .56975 .83498 .56145 Metetra
.605 .746 .944 r
.83498 .56145 .84176 .56975 .85843 .56667 .85172 .55836 Metetra
.605 .747 .944 r
.85172 .55836 .85843 .56667 .87521 .56353 .86857 .5552 Metetra
.606 .748 .945 r
.86857 .5552 .87521 .56353 .89209 .56034 .88553 .55199 Metetra
.607 .749 .945 r
.88553 .55199 .89209 .56034 .90908 .55708 .90259 .54871 Metetra
.608 .75 .945 r
.90259 .54871 .90908 .55708 .92618 .55377 .91977 .54538 Metetra
.609 .751 .945 r
.91977 .54538 .92618 .55377 .94339 .5504 .93705 .54198 Metetra
.61 .752 .946 r
.93705 .54198 .94339 .5504 .96071 .54696 .95445 .53853 Metetra
.587 .707 .925 r
.34483 .62675 .35326 .63512 .36683 .63331 .35843 .62494 Metetra
.587 .707 .926 r
.35843 .62494 .36683 .63331 .3805 .63149 .37213 .62312 Metetra
.587 .708 .927 r
.37213 .62312 .3805 .63149 .39425 .62966 .38591 .62128 Metetra
.587 .709 .927 r
.38591 .62128 .39425 .62966 .40809 .6278 .39979 .61943 Metetra
.587 .71 .928 r
.39979 .61943 .40809 .6278 .42202 .62593 .41376 .61757 Metetra
.587 .711 .929 r
.41376 .61757 .42202 .62593 .43604 .62404 .42782 .61568 Metetra
.587 .712 .93 r
.42782 .61568 .43604 .62404 .45016 .62213 .44197 .61377 Metetra
.587 .713 .93 r
.44197 .61377 .45016 .62213 .46437 .62018 .45621 .61183 Metetra
.587 .714 .931 r
.45621 .61183 .46437 .62018 .47867 .61822 .47056 .60987 Metetra
.587 .715 .932 r
.47056 .60987 .47867 .61822 .49306 .61622 .48499 .60787 Metetra
.588 .717 .933 r
.48499 .60787 .49306 .61622 .50755 .61419 .49953 .60585 Metetra
.588 .718 .933 r
.49953 .60585 .50755 .61419 .52214 .61213 .51416 .60379 Metetra
.588 .719 .934 r
.51416 .60379 .52214 .61213 .53682 .61003 .52889 .6017 Metetra
.589 .72 .935 r
.52889 .6017 .53682 .61003 .55161 .60789 .54371 .59957 Metetra
.589 .722 .935 r
.54371 .59957 .55161 .60789 .56649 .60572 .55864 .5974 Metetra
.59 .723 .936 r
.55864 .5974 .56649 .60572 .58146 .6035 .57366 .59519 Metetra
.59 .724 .937 r
.57366 .59519 .58146 .6035 .59654 .60125 .58879 .59294 Metetra
.591 .726 .937 r
.58879 .59294 .59654 .60125 .61172 .59895 .60402 .59064 Metetra
.592 .727 .938 r
.60402 .59064 .61172 .59895 .62699 .5966 .61935 .58829 Metetra
.592 .728 .938 r
.61935 .58829 .62699 .5966 .64237 .59421 .63478 .5859 Metetra
.593 .73 .939 r
.63478 .5859 .64237 .59421 .65785 .59177 .65031 .58346 Metetra
.594 .731 .94 r
.65031 .58346 .65785 .59177 .67343 .58928 .66595 .58098 Metetra
.595 .732 .94 r
.66595 .58098 .67343 .58928 .68912 .58674 .68169 .57843 Metetra
.595 .734 .941 r
.68169 .57843 .68912 .58674 .70491 .58415 .69754 .57584 Metetra
.596 .735 .941 r
.69754 .57584 .70491 .58415 .7208 .58151 .71349 .57319 Metetra
.597 .736 .942 r
.71349 .57319 .7208 .58151 .73679 .57881 .72955 .57049 Metetra
.598 .738 .942 r
.72955 .57049 .73679 .57881 .75289 .57606 .74571 .56773 Metetra
.599 .739 .943 r
.74571 .56773 .75289 .57606 .7691 .57325 .76198 .56492 Metetra
.6 .74 .943 r
.76198 .56492 .7691 .57325 .78541 .57039 .77836 .56205 Metetra
.601 .742 .943 r
.77836 .56205 .78541 .57039 .80183 .56747 .79484 .55911 Metetra
.602 .743 .944 r
.79484 .55911 .80183 .56747 .81835 .56449 .81143 .55612 Metetra
.603 .744 .944 r
.81143 .55612 .81835 .56449 .83498 .56145 .82813 .55308 Metetra
.604 .745 .944 r
.82813 .55308 .83498 .56145 .85172 .55836 .84494 .54997 Metetra
.604 .747 .945 r
.84494 .54997 .85172 .55836 .86857 .5552 .86186 .54679 Metetra
.605 .748 .945 r
.86186 .54679 .86857 .5552 .88553 .55199 .87889 .54356 Metetra
.606 .749 .945 r
.87889 .54356 .88553 .55199 .90259 .54871 .89603 .54027 Metetra
.607 .75 .946 r
.89603 .54027 .90259 .54871 .91977 .54538 .91328 .53691 Metetra
.608 .751 .946 r
.91328 .53691 .91977 .54538 .93705 .54198 .93064 .53349 Metetra
.609 .752 .946 r
.93064 .53349 .93705 .54198 .95445 .53853 .94812 .53001 Metetra
.587 .705 .925 r
.33632 .61826 .34483 .62675 .35843 .62494 .34996 .61645 Metetra
.586 .706 .926 r
.34996 .61645 .35843 .62494 .37213 .62312 .36368 .61463 Metetra
.586 .707 .926 r
.36368 .61463 .37213 .62312 .38591 .62128 .3775 .6128 Metetra
.586 .708 .927 r
.3775 .6128 .38591 .62128 .39979 .61943 .39141 .61095 Metetra
.586 .709 .928 r
.39141 .61095 .39979 .61943 .41376 .61757 .40542 .60909 Metetra
.586 .71 .929 r
.40542 .60909 .41376 .61757 .42782 .61568 .41951 .6072 Metetra
.586 .711 .929 r
.41951 .6072 .42782 .61568 .44197 .61377 .4337 .6053 Metetra
.586 .712 .93 r
.4337 .6053 .44197 .61377 .45621 .61183 .44799 .60337 Metetra
.586 .713 .931 r
.44799 .60337 .45621 .61183 .47056 .60987 .46237 .60141 Metetra
.586 .714 .932 r
.46237 .60141 .47056 .60987 .48499 .60787 .47685 .59942 Metetra
.586 .715 .932 r
.47685 .59942 .48499 .60787 .49953 .60585 .49142 .59741 Metetra
.587 .717 .933 r
.49142 .59741 .49953 .60585 .51416 .60379 .5061 .59536 Metetra
.587 .718 .934 r
.5061 .59536 .51416 .60379 .52889 .6017 .52087 .59327 Metetra
.587 .719 .935 r
.52087 .59327 .52889 .6017 .54371 .59957 .53574 .59114 Metetra
.588 .721 .935 r
.53574 .59114 .54371 .59957 .55864 .5974 .55071 .58898 Metetra
.588 .722 .936 r
.55071 .58898 .55864 .5974 .57366 .59519 .56579 .58678 Metetra
.589 .723 .937 r
.56579 .58678 .57366 .59519 .58879 .59294 .58096 .58453 Metetra
.59 .725 .937 r
.58096 .58453 .58879 .59294 .60402 .59064 .59624 .58223 Metetra
.59 .726 .938 r
.59624 .58223 .60402 .59064 .61935 .58829 .61162 .57989 Metetra
.591 .727 .939 r
.61162 .57989 .61935 .58829 .63478 .5859 .62711 .57751 Metetra
.592 .729 .939 r
.62711 .57751 .63478 .5859 .65031 .58346 .6427 .57507 Metetra
.592 .73 .94 r
.6427 .57507 .65031 .58346 .66595 .58098 .65839 .57258 Metetra
.593 .732 .94 r
.65839 .57258 .66595 .58098 .68169 .57843 .67419 .57004 Metetra
.594 .733 .941 r
.67419 .57004 .68169 .57843 .69754 .57584 .6901 .56744 Metetra
.595 .735 .941 r
.6901 .56744 .69754 .57584 .71349 .57319 .70611 .56479 Metetra
.596 .736 .942 r
.70611 .56479 .71349 .57319 .72955 .57049 .72223 .56209 Metetra
.597 .737 .942 r
.72223 .56209 .72955 .57049 .74571 .56773 .73845 .55933 Metetra
.598 .739 .943 r
.73845 .55933 .74571 .56773 .76198 .56492 .75478 .5565 Metetra
.599 .74 .943 r
.75478 .5565 .76198 .56492 .77836 .56205 .77123 .55362 Metetra
.6 .741 .944 r
.77123 .55362 .77836 .56205 .79484 .55911 .78778 .55068 Metetra
.601 .743 .944 r
.78778 .55068 .79484 .55911 .81143 .55612 .80444 .54768 Metetra
.602 .744 .945 r
.80444 .54768 .81143 .55612 .82813 .55308 .82121 .54462 Metetra
.602 .745 .945 r
.82121 .54462 .82813 .55308 .84494 .54997 .83809 .54149 Metetra
.603 .747 .945 r
.83809 .54149 .84494 .54997 .86186 .54679 .85508 .53831 Metetra
.604 .748 .946 r
.85508 .53831 .86186 .54679 .87889 .54356 .87218 .53506 Metetra
.605 .749 .946 r
.87218 .53506 .87889 .54356 .89603 .54027 .8894 .53174 Metetra
.606 .75 .946 r
.8894 .53174 .89603 .54027 .91328 .53691 .90672 .52837 Metetra
.607 .751 .946 r
.90672 .52837 .91328 .53691 .93064 .53349 .92416 .52492 Metetra
.608 .752 .947 r
.92416 .52492 .93064 .53349 .94812 .53001 .94172 .52142 Metetra
.586 .704 .924 r
.32773 .60965 .33632 .61826 .34996 .61645 .3414 .60784 Metetra
.585 .705 .925 r
.3414 .60784 .34996 .61645 .36368 .61463 .35517 .60603 Metetra
.585 .705 .926 r
.35517 .60603 .36368 .61463 .3775 .6128 .36902 .6042 Metetra
.585 .706 .927 r
.36902 .6042 .3775 .6128 .39141 .61095 .38296 .60235 Metetra
.584 .707 .927 r
.38296 .60235 .39141 .61095 .40542 .60909 .397 .60049 Metetra
.584 .708 .928 r
.397 .60049 .40542 .60909 .41951 .6072 .41113 .59862 Metetra
.584 .709 .929 r
.41113 .59862 .41951 .6072 .4337 .6053 .42536 .59672 Metetra
.584 .71 .93 r
.42536 .59672 .4337 .6053 .44799 .60337 .43968 .59479 Metetra
.584 .712 .931 r
.43968 .59479 .44799 .60337 .46237 .60141 .4541 .59284 Metetra
.585 .713 .931 r
.4541 .59284 .46237 .60141 .47685 .59942 .46862 .59086 Metetra
.585 .714 .932 r
.46862 .59086 .47685 .59942 .49142 .59741 .48324 .58885 Metetra
.585 .715 .933 r
.48324 .58885 .49142 .59741 .5061 .59536 .49796 .58681 Metetra
.585 .717 .934 r
.49796 .58681 .5061 .59536 .52087 .59327 .51277 .58473 Metetra
.586 .718 .935 r
.51277 .58473 .52087 .59327 .53574 .59114 .52769 .58262 Metetra
.586 .719 .935 r
.52769 .58262 .53574 .59114 .55071 .58898 .54271 .58046 Metetra
.587 .721 .936 r
.54271 .58046 .55071 .58898 .56579 .58678 .55783 .57826 Metetra
.587 .722 .937 r
.55783 .57826 .56579 .58678 .58096 .58453 .57306 .57602 Metetra
.588 .724 .937 r
.57306 .57602 .58096 .58453 .59624 .58223 .58839 .57373 Metetra
.589 .725 .938 r
.58839 .57373 .59624 .58223 .61162 .57989 .60382 .5714 Metetra
.589 .727 .939 r
.60382 .5714 .61162 .57989 .62711 .57751 .61936 .56902 Metetra
.59 .728 .939 r
.61936 .56902 .62711 .57751 .6427 .57507 .635 .56658 Metetra
.591 .73 .94 r
.635 .56658 .6427 .57507 .65839 .57258 .65075 .5641 Metetra
.592 .731 .941 r
.65075 .5641 .65839 .57258 .67419 .57004 .66661 .56156 Metetra
.593 .733 .941 r
.66661 .56156 .67419 .57004 .6901 .56744 .68257 .55896 Metetra
.594 .734 .942 r
.68257 .55896 .6901 .56744 .70611 .56479 .69865 .55631 Metetra
.595 .735 .942 r
.69865 .55631 .70611 .56479 .72223 .56209 .71483 .5536 Metetra
.595 .737 .943 r
.71483 .5536 .72223 .56209 .73845 .55933 .73111 .55083 Metetra
.596 .738 .943 r
.73111 .55083 .73845 .55933 .75478 .5565 .74751 .54801 Metetra
.597 .74 .944 r
.74751 .54801 .75478 .5565 .77123 .55362 .76402 .54512 Metetra
.598 .741 .944 r
.76402 .54512 .77123 .55362 .78778 .55068 .78064 .54217 Metetra
.599 .743 .945 r
.78064 .54217 .78778 .55068 .80444 .54768 .79737 .53916 Metetra
.6 .744 .945 r
.79737 .53916 .80444 .54768 .82121 .54462 .81421 .53608 Metetra
.601 .745 .945 r
.81421 .53608 .82121 .54462 .83809 .54149 .83116 .53295 Metetra
.602 .747 .946 r
.83116 .53295 .83809 .54149 .85508 .53831 .84822 .52974 Metetra
.603 .748 .946 r
.84822 .52974 .85508 .53831 .87218 .53506 .8654 .52648 Metetra
.605 .749 .946 r
.8654 .52648 .87218 .53506 .8894 .53174 .88269 .52314 Metetra
.606 .75 .947 r
.88269 .52314 .8894 .53174 .90672 .52837 .90009 .51974 Metetra
.607 .751 .947 r
.90009 .51974 .90672 .52837 .92416 .52492 .91761 .51628 Metetra
.608 .753 .947 r
.91761 .51628 .92416 .52492 .94172 .52142 .93525 .51275 Metetra
.584 .702 .924 r
.31908 .60092 .32773 .60965 .3414 .60784 .33278 .59911 Metetra
.584 .703 .925 r
.33278 .59911 .3414 .60784 .35517 .60603 .34657 .5973 Metetra
.584 .704 .925 r
.34657 .5973 .35517 .60603 .36902 .6042 .36046 .59547 Metetra
.583 .705 .926 r
.36046 .59547 .36902 .6042 .38296 .60235 .37443 .59364 Metetra
.583 .706 .927 r
.37443 .59364 .38296 .60235 .397 .60049 .38851 .59178 Metetra
.583 .707 .928 r
.38851 .59178 .397 .60049 .41113 .59862 .40267 .58991 Metetra
.583 .708 .929 r
.40267 .58991 .41113 .59862 .42536 .59672 .41694 .58802 Metetra
.583 .709 .93 r
.41694 .58802 .42536 .59672 .43968 .59479 .4313 .5861 Metetra
.583 .71 .93 r
.4313 .5861 .43968 .59479 .4541 .59284 .44576 .58416 Metetra
.583 .712 .931 r
.44576 .58416 .4541 .59284 .46862 .59086 .46032 .58219 Metetra
.583 .713 .932 r
.46032 .58219 .46862 .59086 .48324 .58885 .47498 .58019 Metetra
.584 .714 .933 r
.47498 .58019 .48324 .58885 .49796 .58681 .48974 .57816 Metetra
.584 .716 .934 r
.48974 .57816 .49796 .58681 .51277 .58473 .5046 .57609 Metetra
.584 .717 .934 r
.5046 .57609 .51277 .58473 .52769 .58262 .51956 .57398 Metetra
.585 .718 .935 r
.51956 .57398 .52769 .58262 .54271 .58046 .53463 .57184 Metetra
.585 .72 .936 r
.53463 .57184 .54271 .58046 .55783 .57826 .5498 .56965 Metetra
.586 .721 .937 r
.5498 .56965 .55783 .57826 .57306 .57602 .56507 .56741 Metetra
.587 .723 .937 r
.56507 .56741 .57306 .57602 .58839 .57373 .58045 .56513 Metetra
.587 .724 .938 r
.58045 .56513 .58839 .57373 .60382 .5714 .59594 .56281 Metetra
.588 .726 .939 r
.59594 .56281 .60382 .5714 .61936 .56902 .61153 .56043 Metetra
.589 .727 .94 r
.61153 .56043 .61936 .56902 .635 .56658 .62723 .558 Metetra
.59 .729 .94 r
.62723 .558 .635 .56658 .65075 .5641 .64304 .55552 Metetra
.59 .73 .941 r
.64304 .55552 .65075 .5641 .66661 .56156 .65895 .55298 Metetra
.591 .732 .941 r
.65895 .55298 .66661 .56156 .68257 .55896 .67497 .55039 Metetra
.592 .733 .942 r
.67497 .55039 .68257 .55896 .69865 .55631 .69111 .54774 Metetra
.593 .735 .943 r
.69111 .54774 .69865 .55631 .71483 .5536 .70735 .54503 Metetra
.594 .736 .943 r
.70735 .54503 .71483 .5536 .73111 .55083 .7237 .54226 Metetra
.595 .738 .944 r
.7237 .54226 .73111 .55083 .74751 .54801 .74016 .53943 Metetra
.596 .739 .944 r
.74016 .53943 .74751 .54801 .76402 .54512 .75674 .53653 Metetra
.597 .741 .945 r
.75674 .53653 .76402 .54512 .78064 .54217 .77342 .53357 Metetra
.598 .742 .945 r
.77342 .53357 .78064 .54217 .79737 .53916 .79022 .53055 Metetra
.599 .744 .945 r
.79022 .53055 .79737 .53916 .81421 .53608 .80713 .52747 Metetra
.6 .745 .946 r
.80713 .52747 .81421 .53608 .83116 .53295 .82416 .52432 Metetra
.601 .746 .946 r
.82416 .52432 .83116 .53295 .84822 .52974 .84129 .5211 Metetra
.602 .748 .947 r
.84129 .5211 .84822 .52974 .8654 .52648 .85855 .51782 Metetra
.603 .749 .947 r
.85855 .51782 .8654 .52648 .88269 .52314 .87591 .51446 Metetra
.605 .75 .947 r
.87591 .51446 .88269 .52314 .90009 .51974 .89339 .51104 Metetra
.606 .752 .947 r
.89339 .51104 .90009 .51974 .91761 .51628 .91099 .50756 Metetra
.607 .753 .948 r
.91099 .50756 .91761 .51628 .93525 .51275 .92871 .504 Metetra
.583 .701 .923 r
.31034 .59207 .31908 .60092 .33278 .59911 .32407 .59026 Metetra
.583 .702 .924 r
.32407 .59026 .33278 .59911 .34657 .5973 .3379 .58845 Metetra
.582 .702 .925 r
.3379 .58845 .34657 .5973 .36046 .59547 .35182 .58663 Metetra
.582 .703 .926 r
.35182 .58663 .36046 .59547 .37443 .59364 .36583 .58479 Metetra
.582 .704 .927 r
.36583 .58479 .37443 .59364 .38851 .59178 .37994 .58295 Metetra
.582 .705 .927 r
.37994 .58295 .38851 .59178 .40267 .58991 .39414 .58108 Metetra
.582 .706 .928 r
.39414 .58108 .40267 .58991 .41694 .58802 .40844 .5792 Metetra
.582 .708 .929 r
.40844 .5792 .41694 .58802 .4313 .5861 .42284 .57729 Metetra
.582 .709 .93 r
.42284 .57729 .4313 .5861 .44576 .58416 .43734 .57536 Metetra
.582 .71 .931 r
.43734 .57536 .44576 .58416 .46032 .58219 .45194 .57341 Metetra
.582 .711 .932 r
.45194 .57341 .46032 .58219 .47498 .58019 .46664 .57142 Metetra
.582 .713 .933 r
.46664 .57142 .47498 .58019 .48974 .57816 .48144 .56939 Metetra
.582 .714 .933 r
.48144 .56939 .48974 .57816 .5046 .57609 .49634 .56734 Metetra
.583 .716 .934 r
.49634 .56734 .5046 .57609 .51956 .57398 .51135 .56524 Metetra
.583 .717 .935 r
.51135 .56524 .51956 .57398 .53463 .57184 .52647 .56311 Metetra
.584 .719 .936 r
.52647 .56311 .53463 .57184 .5498 .56965 .54168 .56093 Metetra
.584 .72 .937 r
.54168 .56093 .5498 .56965 .56507 .56741 .55701 .5587 Metetra
.585 .722 .938 r
.55701 .5587 .56507 .56741 .58045 .56513 .57244 .55643 Metetra
.586 .723 .938 r
.57244 .55643 .58045 .56513 .59594 .56281 .58798 .55412 Metetra
.586 .725 .939 r
.58798 .55412 .59594 .56281 .61153 .56043 .60362 .55175 Metetra
.587 .727 .94 r
.60362 .55175 .61153 .56043 .62723 .558 .61938 .54932 Metetra
.588 .728 .94 r
.61938 .54932 .62723 .558 .64304 .55552 .63524 .54685 Metetra
.589 .73 .941 r
.63524 .54685 .64304 .55552 .65895 .55298 .65121 .54431 Metetra
.59 .731 .942 r
.65121 .54431 .65895 .55298 .67497 .55039 .66729 .54172 Metetra
.591 .733 .942 r
.66729 .54172 .67497 .55039 .69111 .54774 .68349 .53908 Metetra
.592 .734 .943 r
.68349 .53908 .69111 .54774 .70735 .54503 .69979 .53637 Metetra
.593 .736 .943 r
.69979 .53637 .70735 .54503 .7237 .54226 .71621 .53359 Metetra
.594 .738 .944 r
.71621 .53359 .7237 .54226 .74016 .53943 .73274 .53076 Metetra
.595 .739 .945 r
.73274 .53076 .74016 .53943 .75674 .53653 .74938 .52786 Metetra
.596 .741 .945 r
.74938 .52786 .75674 .53653 .77342 .53357 .76613 .5249 Metetra
.597 .742 .945 r
.76613 .5249 .77342 .53357 .79022 .53055 .783 .52187 Metetra
.598 .744 .946 r
.783 .52187 .79022 .53055 .80713 .52747 .79998 .51877 Metetra
.599 .745 .946 r
.79998 .51877 .80713 .52747 .82416 .52432 .81708 .51561 Metetra
.6 .746 .947 r
.81708 .51561 .82416 .52432 .84129 .5211 .83429 .51238 Metetra
.601 .748 .947 r
.83429 .51238 .84129 .5211 .85855 .51782 .85162 .50908 Metetra
.602 .749 .947 r
.85162 .50908 .85855 .51782 .87591 .51446 .86906 .50571 Metetra
.604 .75 .948 r
.86906 .50571 .87591 .51446 .89339 .51104 .88662 .50227 Metetra
.605 .752 .948 r
.88662 .50227 .89339 .51104 .91099 .50756 .9043 .49876 Metetra
.606 .753 .948 r
.9043 .49876 .91099 .50756 .92871 .504 .92209 .49518 Metetra
.582 .699 .922 r
.30153 .58308 .31034 .59207 .32407 .59026 .31529 .58127 Metetra
.582 .7 .923 r
.31529 .58127 .32407 .59026 .3379 .58845 .32915 .57946 Metetra
.581 .701 .924 r
.32915 .57946 .3379 .58845 .35182 .58663 .3431 .57765 Metetra
.581 .702 .925 r
.3431 .57765 .35182 .58663 .36583 .58479 .35714 .57582 Metetra
.581 .703 .926 r
.35714 .57582 .36583 .58479 .37994 .58295 .37129 .57398 Metetra
.58 .704 .927 r
.37129 .57398 .37994 .58295 .39414 .58108 .38552 .57213 Metetra
.58 .705 .928 r
.38552 .57213 .39414 .58108 .40844 .5792 .39986 .57026 Metetra
.58 .706 .929 r
.39986 .57026 .40844 .5792 .42284 .57729 .4143 .56836 Metetra
.58 .707 .93 r
.4143 .56836 .42284 .57729 .43734 .57536 .42884 .56644 Metetra
.58 .709 .931 r
.42884 .56644 .43734 .57536 .45194 .57341 .44347 .5645 Metetra
.58 .71 .932 r
.44347 .5645 .45194 .57341 .46664 .57142 .45822 .56252 Metetra
.581 .711 .932 r
.45822 .56252 .46664 .57142 .48144 .56939 .47306 .56051 Metetra
.581 .713 .933 r
.47306 .56051 .48144 .56939 .49634 .56734 .48801 .55847 Metetra
.581 .714 .934 r
.48801 .55847 .49634 .56734 .51135 .56524 .50306 .55639 Metetra
.582 .716 .935 r
.50306 .55639 .51135 .56524 .52647 .56311 .51822 .55426 Metetra
.582 .717 .936 r
.51822 .55426 .52647 .56311 .54168 .56093 .53349 .5521 Metetra
.583 .719 .937 r
.53349 .5521 .54168 .56093 .55701 .5587 .54886 .54989 Metetra
.583 .721 .938 r
.54886 .54989 .55701 .5587 .57244 .55643 .56434 .54763 Metetra
.584 .722 .938 r
.56434 .54763 .57244 .55643 .58798 .55412 .57993 .54532 Metetra
.585 .724 .939 r
.57993 .54532 .58798 .55412 .60362 .55175 .59563 .54296 Metetra
.586 .726 .94 r
.59563 .54296 .60362 .55175 .61938 .54932 .61144 .54055 Metetra
.586 .727 .941 r
.61144 .54055 .61938 .54932 .63524 .54685 .62736 .53808 Metetra
.587 .729 .941 r
.62736 .53808 .63524 .54685 .65121 .54431 .64339 .53556 Metetra
.588 .731 .942 r
.64339 .53556 .65121 .54431 .66729 .54172 .65953 .53297 Metetra
.589 .732 .943 r
.65953 .53297 .66729 .54172 .68349 .53908 .67579 .53032 Metetra
.59 .734 .943 r
.67579 .53032 .68349 .53908 .69979 .53637 .69215 .52762 Metetra
.591 .736 .944 r
.69215 .52762 .69979 .53637 .71621 .53359 .70863 .52485 Metetra
.592 .737 .944 r
.70863 .52485 .71621 .53359 .73274 .53076 .72523 .52201 Metetra
.593 .739 .945 r
.72523 .52201 .73274 .53076 .74938 .52786 .74194 .51911 Metetra
.594 .74 .946 r
.74194 .51911 .74938 .52786 .76613 .5249 .75876 .51614 Metetra
.596 .742 .946 r
.75876 .51614 .76613 .5249 .783 .52187 .7757 .5131 Metetra
.597 .743 .946 r
.7757 .5131 .783 .52187 .79998 .51877 .79275 .51 Metetra
.598 .745 .947 r
.79275 .51 .79998 .51877 .81708 .51561 .80992 .50682 Metetra
.599 .746 .947 r
.80992 .50682 .81708 .51561 .83429 .51238 .82721 .50358 Metetra
.6 .748 .948 r
.82721 .50358 .83429 .51238 .85162 .50908 .84461 .50026 Metetra
.601 .749 .948 r
.84461 .50026 .85162 .50908 .86906 .50571 .86213 .49687 Metetra
.602 .751 .948 r
.86213 .49687 .86906 .50571 .88662 .50227 .87977 .49342 Metetra
.604 .752 .949 r
.87977 .49342 .88662 .50227 .9043 .49876 .89753 .48988 Metetra
.605 .753 .949 r
.89753 .48988 .9043 .49876 .92209 .49518 .91541 .48628 Metetra
.581 .697 .922 r
.29264 .57396 .30153 .58308 .31529 .58127 .30644 .57215 Metetra
.581 .698 .923 r
.30644 .57215 .31529 .58127 .32915 .57946 .32032 .57035 Metetra
.58 .699 .924 r
.32032 .57035 .32915 .57946 .3431 .57765 .3343 .56854 Metetra
.58 .7 .925 r
.3343 .56854 .3431 .57765 .35714 .57582 .34838 .56672 Metetra
.579 .701 .925 r
.34838 .56672 .35714 .57582 .37129 .57398 .36256 .56489 Metetra
.579 .702 .926 r
.36256 .56489 .37129 .57398 .38552 .57213 .37683 .56304 Metetra
.579 .703 .927 r
.37683 .56304 .38552 .57213 .39986 .57026 .3912 .56118 Metetra
.579 .704 .928 r
.3912 .56118 .39986 .57026 .4143 .56836 .40568 .5593 Metetra
.578 .706 .929 r
.40568 .5593 .4143 .56836 .42884 .56644 .42025 .5574 Metetra
.578 .707 .93 r
.42025 .5574 .42884 .56644 .44347 .5645 .43493 .55547 Metetra
.579 .708 .931 r
.43493 .55547 .44347 .5645 .45822 .56252 .44971 .5535 Metetra
.579 .71 .932 r
.44971 .5535 .45822 .56252 .47306 .56051 .4646 .55151 Metetra
.579 .711 .933 r
.4646 .55151 .47306 .56051 .48801 .55847 .47959 .54948 Metetra
.579 .713 .934 r
.47959 .54948 .48801 .55847 .50306 .55639 .49469 .54742 Metetra
.58 .715 .935 r
.49469 .54742 .50306 .55639 .51822 .55426 .50989 .54531 Metetra
.58 .716 .936 r
.50989 .54531 .51822 .55426 .53349 .5521 .52521 .54316 Metetra
.581 .718 .937 r
.52521 .54316 .53349 .5521 .54886 .54989 .54063 .54096 Metetra
.582 .72 .938 r
.54063 .54096 .54886 .54989 .56434 .54763 .55616 .53872 Metetra
.582 .721 .938 r
.55616 .53872 .56434 .54763 .57993 .54532 .5718 .53643 Metetra
.583 .723 .939 r
.5718 .53643 .57993 .54532 .59563 .54296 .58756 .53408 Metetra
.584 .725 .94 r
.58756 .53408 .59563 .54296 .61144 .54055 .60342 .53168 Metetra
.585 .726 .941 r
.60342 .53168 .61144 .54055 .62736 .53808 .6194 .52922 Metetra
.586 .728 .942 r
.6194 .52922 .62736 .53808 .64339 .53556 .63548 .5267 Metetra
.586 .73 .942 r
.63548 .5267 .64339 .53556 .65953 .53297 .65169 .52412 Metetra
.587 .732 .943 r
.65169 .52412 .65953 .53297 .67579 .53032 .668 .52148 Metetra
.588 .733 .944 r
.668 .52148 .67579 .53032 .69215 .52762 .68443 .51878 Metetra
.59 .735 .944 r
.68443 .51878 .69215 .52762 .70863 .52485 .70098 .51601 Metetra
.591 .737 .945 r
.70098 .51601 .70863 .52485 .72523 .52201 .71764 .51317 Metetra
.592 .738 .945 r
.71764 .51317 .72523 .52201 .74194 .51911 .73441 .51027 Metetra
.593 .74 .946 r
.73441 .51027 .74194 .51911 .75876 .51614 .75131 .5073 Metetra
.594 .742 .947 r
.75131 .5073 .75876 .51614 .7757 .5131 .76831 .50426 Metetra
.595 .743 .947 r
.76831 .50426 .7757 .5131 .79275 .51 .78544 .50114 Metetra
.596 .745 .947 r
.78544 .50114 .79275 .51 .80992 .50682 .80269 .49796 Metetra
.598 .746 .948 r
.80269 .49796 .80992 .50682 .82721 .50358 .82005 .4947 Metetra
.599 .748 .948 r
.82005 .4947 .82721 .50358 .84461 .50026 .83753 .49137 Metetra
.6 .749 .949 r
.83753 .49137 .84461 .50026 .86213 .49687 .85513 .48797 Metetra
.601 .751 .949 r
.85513 .48797 .86213 .49687 .87977 .49342 .87285 .48449 Metetra
.603 .752 .949 r
.87285 .48449 .87977 .49342 .89753 .48988 .89069 .48093 Metetra
.604 .753 .949 r
.89069 .48093 .89753 .48988 .91541 .48628 .90865 .47731 Metetra
.58 .696 .921 r
.28368 .56469 .29264 .57396 .30644 .57215 .2975 .56289 Metetra
.579 .696 .922 r
.2975 .56289 .30644 .57215 .32032 .57035 .31142 .56109 Metetra
.579 .697 .923 r
.31142 .56109 .32032 .57035 .3343 .56854 .32543 .55928 Metetra
.578 .698 .924 r
.32543 .55928 .3343 .56854 .34838 .56672 .33954 .55747 Metetra
.578 .699 .925 r
.33954 .55747 .34838 .56672 .36256 .56489 .35375 .55565 Metetra
.577 .7 .926 r
.35375 .55565 .36256 .56489 .37683 .56304 .36806 .55382 Metetra
.577 .701 .927 r
.36806 .55382 .37683 .56304 .3912 .56118 .38246 .55197 Metetra
.577 .703 .928 r
.38246 .55197 .3912 .56118 .40568 .5593 .39697 .55011 Metetra
.577 .704 .929 r
.39697 .55011 .40568 .5593 .42025 .5574 .41159 .54822 Metetra
.577 .705 .93 r
.41159 .54822 .42025 .5574 .43493 .55547 .4263 .5463 Metetra
.577 .707 .931 r
.4263 .5463 .43493 .55547 .44971 .5535 .44113 .54436 Metetra
.577 .708 .932 r
.44113 .54436 .44971 .5535 .4646 .55151 .45605 .54239 Metetra
.577 .71 .933 r
.45605 .54239 .4646 .55151 .47959 .54948 .47109 .54038 Metetra
.578 .711 .934 r
.47109 .54038 .47959 .54948 .49469 .54742 .48623 .53833 Metetra
.578 .713 .935 r
.48623 .53833 .49469 .54742 .50989 .54531 .50148 .53624 Metetra
.578 .715 .936 r
.50148 .53624 .50989 .54531 .52521 .54316 .51684 .53411 Metetra
.579 .717 .937 r
.51684 .53411 .52521 .54316 .54063 .54096 .53231 .53193 Metetra
.58 .718 .938 r
.53231 .53193 .54063 .54096 .55616 .53872 .5479 .5297 Metetra
.58 .72 .939 r
.5479 .5297 .55616 .53872 .5718 .53643 .56359 .52742 Metetra
.581 .722 .939 r
.56359 .52742 .5718 .53643 .58756 .53408 .5794 .52509 Metetra
.582 .724 .94 r
.5794 .52509 .58756 .53408 .60342 .53168 .59532 .5227 Metetra
.583 .725 .941 r
.59532 .5227 .60342 .53168 .6194 .52922 .61135 .52026 Metetra
.584 .727 .942 r
.61135 .52026 .6194 .52922 .63548 .5267 .6275 .51775 Metetra
.585 .729 .943 r
.6275 .51775 .63548 .5267 .65169 .52412 .64376 .51518 Metetra
.586 .731 .943 r
.64376 .51518 .65169 .52412 .668 .52148 .66014 .51255 Metetra
.587 .733 .944 r
.66014 .51255 .668 .52148 .68443 .51878 .67663 .50985 Metetra
.588 .734 .945 r
.67663 .50985 .68443 .51878 .70098 .51601 .69324 .50709 Metetra
.589 .736 .945 r
.69324 .50709 .70098 .51601 .71764 .51317 .70997 .50425 Metetra
.59 .738 .946 r
.70997 .50425 .71764 .51317 .73441 .51027 .72681 .50135 Metetra
.591 .74 .947 r
.72681 .50135 .73441 .51027 .75131 .5073 .74377 .49838 Metetra
.593 .741 .947 r
.74377 .49838 .75131 .5073 .76831 .50426 .76085 .49533 Metetra
.594 .743 .948 r
.76085 .49533 .76831 .50426 .78544 .50114 .77805 .49221 Metetra
.595 .745 .948 r
.77805 .49221 .78544 .50114 .80269 .49796 .79537 .48902 Metetra
.596 .746 .948 r
.79537 .48902 .80269 .49796 .82005 .4947 .81281 .48575 Metetra
.598 .748 .949 r
.81281 .48575 .82005 .4947 .83753 .49137 .83037 .4824 Metetra
.599 .749 .949 r
.83037 .4824 .83753 .49137 .85513 .48797 .84805 .47898 Metetra
.6 .751 .95 r
.84805 .47898 .85513 .48797 .87285 .48449 .86585 .47548 Metetra
.601 .752 .95 r
.86585 .47548 .87285 .48449 .89069 .48093 .88377 .47191 Metetra
.603 .754 .95 r
.88377 .47191 .89069 .48093 .90865 .47731 .90181 .46826 Metetra
.579 .694 .92 r
.27464 .55527 .28368 .56469 .2975 .56289 .28849 .55347 Metetra
.578 .694 .921 r
.28849 .55347 .2975 .56289 .31142 .56109 .30244 .55168 Metetra
.577 .695 .922 r
.30244 .55168 .31142 .56109 .32543 .55928 .31648 .54988 Metetra
.577 .696 .923 r
.31648 .54988 .32543 .55928 .33954 .55747 .33062 .54808 Metetra
.576 .697 .924 r
.33062 .54808 .33954 .55747 .35375 .55565 .34486 .54627 Metetra
.576 .698 .925 r
.34486 .54627 .35375 .55565 .36806 .55382 .3592 .54446 Metetra
.575 .699 .926 r
.3592 .54446 .36806 .55382 .38246 .55197 .37364 .54262 Metetra
.575 .701 .927 r
.37364 .54262 .38246 .55197 .39697 .55011 .38819 .54078 Metetra
.575 .702 .928 r
.38819 .54078 .39697 .55011 .41159 .54822 .40284 .5389 Metetra
.575 .704 .929 r
.40284 .5389 .41159 .54822 .4263 .5463 .4176 .53701 Metetra
.575 .705 .931 r
.4176 .53701 .4263 .5463 .44113 .54436 .43246 .53509 Metetra
.575 .707 .932 r
.43246 .53509 .44113 .54436 .45605 .54239 .44743 .53313 Metetra
.575 .708 .933 r
.44743 .53313 .45605 .54239 .47109 .54038 .4625 .53114 Metetra
.576 .71 .934 r
.4625 .53114 .47109 .54038 .48623 .53833 .47769 .52912 Metetra
.576 .712 .935 r
.47769 .52912 .48623 .53833 .50148 .53624 .49299 .52705 Metetra
.576 .713 .936 r
.49299 .52705 .50148 .53624 .51684 .53411 .50839 .52494 Metetra
.577 .715 .937 r
.50839 .52494 .51684 .53411 .53231 .53193 .52391 .52278 Metetra
.578 .717 .938 r
.52391 .52278 .53231 .53193 .5479 .5297 .53955 .52057 Metetra
.578 .719 .939 r
.53955 .52057 .5479 .5297 .56359 .52742 .55529 .51831 Metetra
.579 .721 .94 r
.55529 .51831 .56359 .52742 .5794 .52509 .57115 .516 Metetra
.58 .723 .941 r
.57115 .516 .5794 .52509 .59532 .5227 .58713 .51363 Metetra
.581 .724 .941 r
.58713 .51363 .59532 .5227 .61135 .52026 .60322 .5112 Metetra
.582 .726 .942 r
.60322 .5112 .61135 .52026 .6275 .51775 .61942 .5087 Metetra
.583 .728 .943 r
.61942 .5087 .6275 .51775 .64376 .51518 .63574 .50615 Metetra
.584 .73 .944 r
.63574 .50615 .64376 .51518 .66014 .51255 .65218 .50353 Metetra
.585 .732 .945 r
.65218 .50353 .66014 .51255 .67663 .50985 .66874 .50084 Metetra
.586 .734 .945 r
.66874 .50084 .67663 .50985 .69324 .50709 .68541 .49808 Metetra
.587 .736 .946 r
.68541 .49808 .69324 .50709 .70997 .50425 .70221 .49525 Metetra
.588 .738 .947 r
.70221 .49525 .70997 .50425 .72681 .50135 .71912 .49235 Metetra
.59 .739 .947 r
.71912 .49235 .72681 .50135 .74377 .49838 .73615 .48937 Metetra
.591 .741 .948 r
.73615 .48937 .74377 .49838 .76085 .49533 .75331 .48632 Metetra
.592 .743 .948 r
.75331 .48632 .76085 .49533 .77805 .49221 .77058 .4832 Metetra
.594 .745 .949 r
.77058 .4832 .77805 .49221 .79537 .48902 .78797 .48 Metetra
.595 .746 .949 r
.78797 .48 .79537 .48902 .81281 .48575 .80549 .47672 Metetra
.596 .748 .95 r
.80549 .47672 .81281 .48575 .83037 .4824 .82312 .47336 Metetra
.597 .75 .95 r
.82312 .47336 .83037 .4824 .84805 .47898 .84088 .46992 Metetra
.599 .751 .95 r
.84088 .46992 .84805 .47898 .86585 .47548 .85877 .46641 Metetra
.6 .753 .951 r
.85877 .46641 .86585 .47548 .88377 .47191 .87677 .46281 Metetra
.602 .754 .951 r
.87677 .46281 .88377 .47191 .90181 .46826 .8949 .45914 Metetra
.578 .692 .919 r
.26552 .5457 .27464 .55527 .28849 .55347 .2794 .5439 Metetra
.577 .692 .92 r
.2794 .5439 .28849 .55347 .30244 .55168 .29338 .54211 Metetra
.576 .693 .921 r
.29338 .54211 .30244 .55168 .31648 .54988 .30745 .54033 Metetra
.575 .694 .922 r
.30745 .54033 .31648 .54988 .33062 .54808 .32162 .53854 Metetra
.575 .695 .923 r
.32162 .53854 .33062 .54808 .34486 .54627 .33589 .53674 Metetra
.574 .696 .924 r
.33589 .53674 .34486 .54627 .3592 .54446 .35027 .53494 Metetra
.574 .697 .926 r
.35027 .53494 .3592 .54446 .37364 .54262 .36474 .53313 Metetra
.573 .699 .927 r
.36474 .53313 .37364 .54262 .38819 .54078 .37932 .5313 Metetra
.573 .7 .928 r
.37932 .5313 .38819 .54078 .40284 .5389 .39401 .52945 Metetra
.573 .702 .929 r
.39401 .52945 .40284 .5389 .4176 .53701 .4088 .52757 Metetra
.573 .703 .93 r
.4088 .52757 .4176 .53701 .43246 .53509 .4237 .52567 Metetra
.573 .705 .931 r
.4237 .52567 .43246 .53509 .44743 .53313 .43871 .52374 Metetra
.573 .706 .932 r
.43871 .52374 .44743 .53313 .4625 .53114 .45383 .52178 Metetra
.574 .708 .934 r
.45383 .52178 .4625 .53114 .47769 .52912 .46906 .51978 Metetra
.574 .71 .935 r
.46906 .51978 .47769 .52912 .49299 .52705 .4844 .51773 Metetra
.574 .712 .936 r
.4844 .51773 .49299 .52705 .50839 .52494 .49986 .51565 Metetra
.575 .714 .937 r
.49986 .51565 .50839 .52494 .52391 .52278 .51543 .51351 Metetra
.576 .716 .938 r
.51543 .51351 .52391 .52278 .53955 .52057 .53111 .51133 Metetra
.576 .718 .939 r
.53111 .51133 .53955 .52057 .55529 .51831 .54691 .50909 Metetra
.577 .719 .94 r
.54691 .50909 .55529 .51831 .57115 .516 .56282 .5068 Metetra
.578 .721 .941 r
.56282 .5068 .57115 .516 .58713 .51363 .57885 .50445 Metetra
.579 .723 .942 r
.57885 .50445 .58713 .51363 .60322 .5112 .59499 .50204 Metetra
.58 .725 .943 r
.59499 .50204 .60322 .5112 .61942 .5087 .61126 .49956 Metetra
.581 .727 .943 r
.61126 .49956 .61942 .5087 .63574 .50615 .62764 .49702 Metetra
.582 .729 .944 r
.62764 .49702 .63574 .50615 .65218 .50353 .64414 .49441 Metetra
.583 .731 .945 r
.64414 .49441 .65218 .50353 .66874 .50084 .66076 .49173 Metetra
.584 .733 .946 r
.66076 .49173 .66874 .50084 .68541 .49808 .67751 .48898 Metetra
.585 .735 .947 r
.67751 .48898 .68541 .49808 .70221 .49525 .69437 .48616 Metetra
.587 .737 .947 r
.69437 .48616 .70221 .49525 .71912 .49235 .71135 .48326 Metetra
.588 .739 .948 r
.71135 .48326 .71912 .49235 .73615 .48937 .72845 .48029 Metetra
.589 .741 .948 r
.72845 .48029 .73615 .48937 .75331 .48632 .74568 .47724 Metetra
.591 .743 .949 r
.74568 .47724 .75331 .48632 .77058 .4832 .76302 .47411 Metetra
.592 .745 .949 r
.76302 .47411 .77058 .4832 .78797 .48 .78049 .4709 Metetra
.593 .746 .95 r
.78049 .4709 .78797 .48 .80549 .47672 .79809 .46762 Metetra
.595 .748 .95 r
.79809 .46762 .80549 .47672 .82312 .47336 .8158 .46425 Metetra
.596 .75 .951 r
.8158 .46425 .82312 .47336 .84088 .46992 .83364 .4608 Metetra
.597 .751 .951 r
.83364 .4608 .84088 .46992 .85877 .46641 .85161 .45726 Metetra
.599 .753 .951 r
.85161 .45726 .85877 .46641 .87677 .46281 .86969 .45365 Metetra
.6 .754 .952 r
.86969 .45365 .87677 .46281 .8949 .45914 .88791 .44995 Metetra
.577 .69 .918 r
.25633 .53597 .26552 .5457 .2794 .5439 .27023 .53417 Metetra
.576 .69 .919 r
.27023 .53417 .2794 .5439 .29338 .54211 .28424 .53239 Metetra
.575 .691 .92 r
.28424 .53239 .29338 .54211 .30745 .54033 .29834 .53061 Metetra
.574 .692 .921 r
.29834 .53061 .30745 .54033 .32162 .53854 .31254 .52883 Metetra
.573 .693 .923 r
.31254 .52883 .32162 .53854 .33589 .53674 .32684 .52706 Metetra
.573 .694 .924 r
.32684 .52706 .33589 .53674 .35027 .53494 .34125 .52527 Metetra
.572 .695 .925 r
.34125 .52527 .35027 .53494 .36474 .53313 .35576 .52348 Metetra
.572 .697 .926 r
.35576 .52348 .36474 .53313 .37932 .5313 .37037 .52167 Metetra
.571 .698 .927 r
.37037 .52167 .37932 .5313 .39401 .52945 .3851 .51984 Metetra
.571 .699 .928 r
.3851 .51984 .39401 .52945 .4088 .52757 .39993 .51799 Metetra
.571 .701 .93 r
.39993 .51799 .4088 .52757 .4237 .52567 .41486 .51612 Metetra
.571 .703 .931 r
.41486 .51612 .4237 .52567 .43871 .52374 .42991 .51422 Metetra
.571 .704 .932 r
.42991 .51422 .43871 .52374 .45383 .52178 .44507 .51228 Metetra
.571 .706 .933 r
.44507 .51228 .45383 .52178 .46906 .51978 .46035 .51031 Metetra
.572 .708 .934 r
.46035 .51031 .46906 .51978 .4844 .51773 .47573 .50829 Metetra
.572 .71 .936 r
.47573 .50829 .4844 .51773 .49986 .51565 .49123 .50623 Metetra
.573 .712 .937 r
.49123 .50623 .49986 .51565 .51543 .51351 .50685 .50413 Metetra
.573 .714 .938 r
.50685 .50413 .51543 .51351 .53111 .51133 .52258 .50197 Metetra
.574 .716 .939 r
.52258 .50197 .53111 .51133 .54691 .50909 .53843 .49976 Metetra
.575 .718 .94 r
.53843 .49976 .54691 .50909 .56282 .5068 .5544 .49749 Metetra
.576 .72 .941 r
.5544 .49749 .56282 .5068 .57885 .50445 .57048 .49516 Metetra
.576 .722 .942 r
.57048 .49516 .57885 .50445 .59499 .50204 .58669 .49277 Metetra
.577 .724 .943 r
.58669 .49277 .59499 .50204 .61126 .49956 .60301 .49032 Metetra
.579 .727 .944 r
.60301 .49032 .61126 .49956 .62764 .49702 .61945 .48779 Metetra
.58 .729 .945 r
.61945 .48779 .62764 .49702 .64414 .49441 .63602 .4852 Metetra
.581 .731 .946 r
.63602 .4852 .64414 .49441 .66076 .49173 .6527 .48254 Metetra
.582 .733 .946 r
.6527 .48254 .66076 .49173 .67751 .48898 .66951 .4798 Metetra
.583 .735 .947 r
.66951 .4798 .67751 .48898 .69437 .48616 .68644 .47699 Metetra
.585 .737 .948 r
.68644 .47699 .69437 .48616 .71135 .48326 .70349 .4741 Metetra
.586 .739 .949 r
.70349 .4741 .71135 .48326 .72845 .48029 .72066 .47113 Metetra
.587 .741 .949 r
.72066 .47113 .72845 .48029 .74568 .47724 .73796 .46808 Metetra
.589 .743 .95 r
.73796 .46808 .74568 .47724 .76302 .47411 .75538 .46495 Metetra
.59 .745 .95 r
.75538 .46495 .76302 .47411 .78049 .4709 .77293 .46174 Metetra
.592 .746 .951 r
.77293 .46174 .78049 .4709 .79809 .46762 .7906 .45844 Metetra
.593 .748 .951 r
.7906 .45844 .79809 .46762 .8158 .46425 .8084 .45506 Metetra
.595 .75 .952 r
.8084 .45506 .8158 .46425 .83364 .4608 .82632 .4516 Metetra
.596 .752 .952 r
.82632 .4516 .83364 .4608 .85161 .45726 .84437 .44805 Metetra
.598 .753 .952 r
.84437 .44805 .85161 .45726 .86969 .45365 .86254 .44441 Metetra
.599 .755 .953 r
.86254 .44441 .86969 .45365 .88791 .44995 .88084 .44069 Metetra
.576 .687 .917 r
.24705 .52606 .25633 .53597 .27023 .53417 .26099 .52427 Metetra
.574 .688 .918 r
.26099 .52427 .27023 .53417 .28424 .53239 .27502 .52249 Metetra
.573 .689 .919 r
.27502 .52249 .28424 .53239 .29834 .53061 .28915 .52073 Metetra
.572 .69 .92 r
.28915 .52073 .29834 .53061 .31254 .52883 .30338 .51896 Metetra
.572 .691 .922 r
.30338 .51896 .31254 .52883 .32684 .52706 .31771 .5172 Metetra
.571 .692 .923 r
.31771 .5172 .32684 .52706 .34125 .52527 .33215 .51544 Metetra
.57 .693 .924 r
.33215 .51544 .34125 .52527 .35576 .52348 .34669 .51367 Metetra
.57 .694 .925 r
.34669 .51367 .35576 .52348 .37037 .52167 .36134 .51188 Metetra
.569 .696 .927 r
.36134 .51188 .37037 .52167 .3851 .51984 .3761 .51008 Metetra
.569 .697 .928 r
.3761 .51008 .3851 .51984 .39993 .51799 .39096 .50826 Metetra
.569 .699 .929 r
.39096 .50826 .39993 .51799 .41486 .51612 .40594 .50642 Metetra
.569 .701 .93 r
.40594 .50642 .41486 .51612 .42991 .51422 .42103 .50454 Metetra
.569 .702 .932 r
.42103 .50454 .42991 .51422 .44507 .51228 .43623 .50264 Metetra
.569 .704 .933 r
.43623 .50264 .44507 .51228 .46035 .51031 .45155 .5007 Metetra
.569 .706 .934 r
.45155 .5007 .46035 .51031 .47573 .50829 .46698 .49871 Metetra
.57 .708 .935 r
.46698 .49871 .47573 .50829 .49123 .50623 .48252 .49669 Metetra
.57 .71 .937 r
.48252 .49669 .49123 .50623 .50685 .50413 .49819 .49461 Metetra
.571 .712 .938 r
.49819 .49461 .50685 .50413 .52258 .50197 .51397 .49249 Metetra
.571 .715 .939 r
.51397 .49249 .52258 .50197 .53843 .49976 .52987 .49031 Metetra
.572 .717 .94 r
.52987 .49031 .53843 .49976 .5544 .49749 .54589 .48807 Metetra
.573 .719 .941 r
.54589 .48807 .5544 .49749 .57048 .49516 .56203 .48577 Metetra
.574 .721 .942 r
.56203 .48577 .57048 .49516 .58669 .49277 .57829 .4834 Metetra
.575 .723 .943 r
.57829 .4834 .58669 .49277 .60301 .49032 .59467 .48097 Metetra
.576 .726 .944 r
.59467 .48097 .60301 .49032 .61945 .48779 .61117 .47847 Metetra
.577 .728 .945 r
.61117 .47847 .61945 .48779 .63602 .4852 .6278 .4759 Metetra
.579 .73 .946 r
.6278 .4759 .63602 .4852 .6527 .48254 .64455 .47326 Metetra
.58 .732 .947 r
.64455 .47326 .6527 .48254 .66951 .4798 .66142 .47053 Metetra
.581 .734 .948 r
.66142 .47053 .66951 .4798 .68644 .47699 .67842 .46773 Metetra
.583 .737 .949 r
.67842 .46773 .68644 .47699 .70349 .4741 .69554 .46485 Metetra
.584 .739 .949 r
.69554 .46485 .70349 .4741 .72066 .47113 .71279 .46189 Metetra
.585 .741 .95 r
.71279 .46189 .72066 .47113 .73796 .46808 .73016 .45885 Metetra
.587 .743 .951 r
.73016 .45885 .73796 .46808 .75538 .46495 .74766 .45572 Metetra
.588 .745 .951 r
.74766 .45572 .75538 .46495 .77293 .46174 .76528 .4525 Metetra
.59 .747 .952 r
.76528 .4525 .77293 .46174 .7906 .45844 .78303 .4492 Metetra
.591 .748 .952 r
.78303 .4492 .7906 .45844 .8084 .45506 .80091 .44581 Metetra
.593 .75 .953 r
.80091 .44581 .8084 .45506 .82632 .4516 .81891 .44233 Metetra
.595 .752 .953 r
.81891 .44233 .82632 .4516 .84437 .44805 .83704 .43877 Metetra
.596 .754 .953 r
.83704 .43877 .84437 .44805 .86254 .44441 .8553 .43511 Metetra
.598 .755 .954 r
.8553 .43511 .86254 .44441 .88084 .44069 .87369 .43137 Metetra
.575 .685 .916 r
.2377 .51597 .24705 .52606 .26099 .52427 .25166 .51419 Metetra
.573 .685 .917 r
.25166 .51419 .26099 .52427 .27502 .52249 .26572 .51242 Metetra
.572 .686 .918 r
.26572 .51242 .27502 .52249 .28915 .52073 .27988 .51066 Metetra
.571 .687 .919 r
.27988 .51066 .28915 .52073 .30338 .51896 .29414 .50891 Metetra
.57 .688 .92 r
.29414 .50891 .30338 .51896 .31771 .5172 .3085 .50717 Metetra
.569 .689 .922 r
.3085 .50717 .31771 .5172 .33215 .51544 .32297 .50543 Metetra
.568 .69 .923 r
.32297 .50543 .33215 .51544 .34669 .51367 .33754 .50368 Metetra
.568 .692 .924 r
.33754 .50368 .34669 .51367 .36134 .51188 .35222 .50193 Metetra
.567 .693 .926 r
.35222 .50193 .36134 .51188 .3761 .51008 .36702 .50016 Metetra
.567 .695 .927 r
.36702 .50016 .3761 .51008 .39096 .50826 .38192 .49837 Metetra
.567 .696 .928 r
.38192 .49837 .39096 .50826 .40594 .50642 .39693 .49656 Metetra
.566 .698 .93 r
.39693 .49656 .40594 .50642 .42103 .50454 .41206 .49472 Metetra
.566 .7 .931 r
.41206 .49472 .42103 .50454 .43623 .50264 .4273 .49285 Metetra
.567 .702 .933 r
.4273 .49285 .43623 .50264 .45155 .5007 .44266 .49094 Metetra
.567 .704 .934 r
.44266 .49094 .45155 .5007 .46698 .49871 .45813 .489 Metetra
.567 .706 .935 r
.45813 .489 .46698 .49871 .48252 .49669 .47372 .48701 Metetra
.568 .708 .937 r
.47372 .48701 .48252 .49669 .49819 .49461 .48943 .48497 Metetra
.568 .711 .938 r
.48943 .48497 .49819 .49461 .51397 .49249 .50526 .48288 Metetra
.569 .713 .939 r
.50526 .48288 .51397 .49249 .52987 .49031 .52121 .48074 Metetra
.57 .715 .94 r
.52121 .48074 .52987 .49031 .54589 .48807 .53729 .47853 Metetra
.571 .718 .942 r
.53729 .47853 .54589 .48807 .56203 .48577 .55348 .47626 Metetra
.571 .72 .943 r
.55348 .47626 .56203 .48577 .57829 .4834 .5698 .47393 Metetra
.573 .722 .944 r
.5698 .47393 .57829 .4834 .59467 .48097 .58624 .47153 Metetra
.574 .725 .945 r
.58624 .47153 .59467 .48097 .61117 .47847 .6028 .46906 Metetra
.575 .727 .946 r
.6028 .46906 .61117 .47847 .6278 .4759 .61949 .46651 Metetra
.576 .729 .947 r
.61949 .46651 .6278 .4759 .64455 .47326 .63631 .46389 Metetra
.577 .732 .948 r
.63631 .46389 .64455 .47326 .66142 .47053 .65325 .46118 Metetra
.579 .734 .949 r
.65325 .46118 .66142 .47053 .67842 .46773 .67031 .4584 Metetra
.58 .736 .949 r
.67031 .4584 .67842 .46773 .69554 .46485 .6875 .45553 Metetra
.582 .738 .95 r
.6875 .45553 .69554 .46485 .71279 .46189 .70482 .45258 Metetra
.583 .741 .951 r
.70482 .45258 .71279 .46189 .73016 .45885 .72227 .44954 Metetra
.585 .743 .952 r
.72227 .44954 .73016 .45885 .74766 .45572 .73984 .44641 Metetra
.586 .745 .952 r
.73984 .44641 .74766 .45572 .76528 .4525 .75754 .4432 Metetra
.588 .747 .953 r
.75754 .4432 .76528 .4525 .78303 .4492 .77537 .43989 Metetra
.59 .749 .953 r
.77537 .43989 .78303 .4492 .80091 .44581 .79333 .4365 Metetra
.591 .751 .954 r
.79333 .4365 .80091 .44581 .81891 .44233 .81142 .43301 Metetra
.593 .753 .954 r
.81142 .43301 .81891 .44233 .83704 .43877 .82964 .42943 Metetra
.595 .754 .954 r
.82964 .42943 .83704 .43877 .8553 .43511 .84798 .42575 Metetra
.596 .756 .955 r
.84798 .42575 .8553 .43511 .87369 .43137 .86646 .42198 Metetra
.573 .682 .914 r
.22828 .5057 .2377 .51597 .25166 .51419 .24226 .50391 Metetra
.572 .683 .915 r
.24226 .50391 .25166 .51419 .26572 .51242 .25635 .50215 Metetra
.571 .683 .917 r
.25635 .50215 .26572 .51242 .27988 .51066 .27053 .50041 Metetra
.569 .684 .918 r
.27053 .50041 .27988 .51066 .29414 .50891 .28482 .49868 Metetra
.568 .685 .919 r
.28482 .49868 .29414 .50891 .3085 .50717 .29921 .49696 Metetra
.567 .686 .921 r
.29921 .49696 .3085 .50717 .32297 .50543 .31371 .49524 Metetra
.566 .688 .922 r
.31371 .49524 .32297 .50543 .33754 .50368 .32831 .49352 Metetra
.566 .689 .923 r
.32831 .49352 .33754 .50368 .35222 .50193 .34302 .49179 Metetra
.565 .69 .925 r
.34302 .49179 .35222 .50193 .36702 .50016 .35785 .49006 Metetra
.564 .692 .926 r
.35785 .49006 .36702 .50016 .38192 .49837 .37278 .4883 Metetra
.564 .694 .928 r
.37278 .4883 .38192 .49837 .39693 .49656 .38783 .48653 Metetra
.564 .696 .929 r
.38783 .48653 .39693 .49656 .41206 .49472 .403 .48473 Metetra
.564 .698 .931 r
.403 .48473 .41206 .49472 .4273 .49285 .41828 .4829 Metetra
.564 .7 .932 r
.41828 .4829 .4273 .49285 .44266 .49094 .43368 .48104 Metetra
.564 .702 .934 r
.43368 .48104 .44266 .49094 .45813 .489 .44919 .47914 Metetra
.564 .704 .935 r
.44919 .47914 .45813 .489 .47372 .48701 .46483 .47719 Metetra
.565 .706 .936 r
.46483 .47719 .47372 .48701 .48943 .48497 .48059 .47519 Metetra
.565 .709 .938 r
.48059 .47519 .48943 .48497 .50526 .48288 .49647 .47315 Metetra
.566 .711 .939 r
.49647 .47315 .50526 .48288 .52121 .48074 .51247 .47104 Metetra
.567 .714 .941 r
.51247 .47104 .52121 .48074 .53729 .47853 .52859 .46888 Metetra
.568 .716 .942 r
.52859 .46888 .53729 .47853 .55348 .47626 .54484 .46665 Metetra
.569 .719 .943 r
.54484 .46665 .55348 .47626 .5698 .47393 .56122 .46435 Metetra
.57 .721 .944 r
.56122 .46435 .5698 .47393 .58624 .47153 .57771 .46199 Metetra
.571 .724 .945 r
.57771 .46199 .58624 .47153 .6028 .46906 .59434 .45955 Metetra
.572 .726 .947 r
.59434 .45955 .6028 .46906 .61949 .46651 .61109 .45703 Metetra
.573 .728 .948 r
.61109 .45703 .61949 .46651 .63631 .46389 .62797 .45443 Metetra
.575 .731 .949 r
.62797 .45443 .63631 .46389 .65325 .46118 .64498 .45175 Metetra
.576 .733 .949 r
.64498 .45175 .65325 .46118 .67031 .4584 .66211 .44899 Metetra
.578 .736 .95 r
.66211 .44899 .67031 .4584 .6875 .45553 .67938 .44614 Metetra
.579 .738 .951 r
.67938 .44614 .6875 .45553 .70482 .45258 .69677 .4432 Metetra
.581 .74 .952 r
.69677 .4432 .70482 .45258 .72227 .44954 .71429 .44017 Metetra
.583 .743 .953 r
.71429 .44017 .72227 .44954 .73984 .44641 .73194 .43705 Metetra
.584 .745 .953 r
.73194 .43705 .73984 .44641 .75754 .4432 .74972 .43383 Metetra
.586 .747 .954 r
.74972 .43383 .75754 .4432 .77537 .43989 .76763 .43052 Metetra
.588 .749 .954 r
.76763 .43052 .77537 .43989 .79333 .4365 .78567 .42712 Metetra
.589 .751 .955 r
.78567 .42712 .79333 .4365 .81142 .43301 .80384 .42362 Metetra
.591 .753 .955 r
.80384 .42362 .81142 .43301 .82964 .42943 .82215 .42003 Metetra
.593 .755 .955 r
.82215 .42003 .82964 .42943 .84798 .42575 .84058 .41633 Metetra
.595 .757 .956 r
.84058 .41633 .84798 .42575 .86646 .42198 .85915 .41254 Metetra
.572 .679 .913 r
.21878 .49522 .22828 .5057 .24226 .50391 .23279 .49344 Metetra
.571 .68 .914 r
.23279 .49344 .24226 .50391 .25635 .50215 .2469 .49168 Metetra
.569 .681 .915 r
.2469 .49168 .25635 .50215 .27053 .50041 .26111 .48995 Metetra
.568 .681 .916 r
.26111 .48995 .27053 .50041 .28482 .49868 .27542 .48824 Metetra
.566 .682 .918 r
.27542 .48824 .28482 .49868 .29921 .49696 .28984 .48654 Metetra
.565 .683 .919 r
.28984 .48654 .29921 .49696 .31371 .49524 .30436 .48485 Metetra
.564 .685 .921 r
.30436 .48485 .31371 .49524 .32831 .49352 .319 .48316 Metetra
.563 .686 .922 r
.319 .48316 .32831 .49352 .34302 .49179 .33374 .48147 Metetra
.563 .687 .924 r
.33374 .48147 .34302 .49179 .35785 .49006 .3486 .47977 Metetra
.562 .689 .925 r
.3486 .47977 .35785 .49006 .37278 .4883 .36357 .47806 Metetra
.562 .691 .927 r
.36357 .47806 .37278 .4883 .38783 .48653 .37865 .47633 Metetra
.561 .693 .929 r
.37865 .47633 .38783 .48653 .403 .48473 .39385 .47458 Metetra
.561 .695 .93 r
.39385 .47458 .403 .48473 .41828 .4829 .40917 .4728 Metetra
.561 .697 .932 r
.40917 .4728 .41828 .4829 .43368 .48104 .42461 .47098 Metetra
.561 .699 .933 r
.42461 .47098 .43368 .48104 .44919 .47914 .44017 .46912 Metetra
.561 .702 .935 r
.44017 .46912 .44919 .47914 .46483 .47719 .45585 .46723 Metetra
.562 .704 .936 r
.45585 .46723 .46483 .47719 .48059 .47519 .47165 .46528 Metetra
.562 .707 .938 r
.47165 .46528 .48059 .47519 .49647 .47315 .48758 .46328 Metetra
.563 .709 .939 r
.48758 .46328 .49647 .47315 .51247 .47104 .50363 .46122 Metetra
.564 .712 .941 r
.50363 .46122 .51247 .47104 .52859 .46888 .51981 .45911 Metetra
.565 .714 .942 r
.51981 .45911 .52859 .46888 .54484 .46665 .53611 .45692 Metetra
.566 .717 .943 r
.53611 .45692 .54484 .46665 .56122 .46435 .55254 .45467 Metetra
.567 .72 .945 r
.55254 .45467 .56122 .46435 .57771 .46199 .5691 .45234 Metetra
.568 .722 .946 r
.5691 .45234 .57771 .46199 .59434 .45955 .58578 .44994 Metetra
.569 .725 .947 r
.58578 .44994 .59434 .45955 .61109 .45703 .6026 .44745 Metetra
.571 .728 .948 r
.6026 .44745 .61109 .45703 .62797 .45443 .61954 .44489 Metetra
.572 .73 .949 r
.61954 .44489 .62797 .45443 .64498 .45175 .63662 .44224 Metetra
.573 .733 .95 r
.63662 .44224 .64498 .45175 .66211 .44899 .65382 .4395 Metetra
.575 .735 .951 r
.65382 .4395 .66211 .44899 .67938 .44614 .67115 .43667 Metetra
.577 .738 .952 r
.67115 .43667 .67938 .44614 .69677 .4432 .68862 .43375 Metetra
.578 .74 .953 r
.68862 .43375 .69677 .4432 .71429 .44017 .70622 .43073 Metetra
.58 .743 .954 r
.70622 .43073 .71429 .44017 .73194 .43705 .72394 .42762 Metetra
.582 .745 .954 r
.72394 .42762 .73194 .43705 .74972 .43383 .74181 .42441 Metetra
.584 .747 .955 r
.74181 .42441 .74972 .43383 .76763 .43052 .7598 .4211 Metetra
.586 .75 .955 r
.7598 .4211 .76763 .43052 .78567 .42712 .77792 .41769 Metetra
.587 .752 .956 r
.77792 .41769 .78567 .42712 .80384 .42362 .79618 .41419 Metetra
.589 .754 .956 r
.79618 .41419 .80384 .42362 .82215 .42003 .81457 .41058 Metetra
.591 .756 .957 r
.81457 .41058 .82215 .42003 .84058 .41633 .83309 .40687 Metetra
.593 .758 .957 r
.83309 .40687 .84058 .41633 .85915 .41254 .85175 .40306 Metetra
.571 .676 .911 r
.2092 .48452 .21878 .49522 .23279 .49344 .22323 .48274 Metetra
.569 .677 .912 r
.22323 .48274 .23279 .49344 .2469 .49168 .23737 .481 Metetra
.568 .677 .913 r
.23737 .481 .2469 .49168 .26111 .48995 .2516 .47928 Metetra
.566 .678 .915 r
.2516 .47928 .26111 .48995 .27542 .48824 .26594 .47759 Metetra
.565 .679 .916 r
.26594 .47759 .27542 .48824 .28984 .48654 .28039 .47591 Metetra
.563 .68 .918 r
.28039 .47591 .28984 .48654 .30436 .48485 .29494 .47425 Metetra
.562 .681 .919 r
.29494 .47425 .30436 .48485 .319 .48316 .3096 .4726 Metetra
.561 .683 .921 r
.3096 .4726 .319 .48316 .33374 .48147 .32437 .47095 Metetra
.56 .684 .923 r
.32437 .47095 .33374 .48147 .3486 .47977 .33926 .46929 Metetra
.559 .686 .924 r
.33926 .46929 .3486 .47977 .36357 .47806 .35426 .46763 Metetra
.559 .688 .926 r
.35426 .46763 .36357 .47806 .37865 .47633 .36938 .46594 Metetra
.558 .69 .928 r
.36938 .46594 .37865 .47633 .39385 .47458 .38462 .46424 Metetra
.558 .692 .929 r
.38462 .46424 .39385 .47458 .40917 .4728 .39997 .46251 Metetra
.558 .694 .931 r
.39997 .46251 .40917 .4728 .42461 .47098 .41545 .46075 Metetra
.558 .697 .933 r
.41545 .46075 .42461 .47098 .44017 .46912 .43105 .45895 Metetra
.558 .699 .934 r
.43105 .45895 .44017 .46912 .45585 .46723 .44677 .45711 Metetra
.559 .702 .936 r
.44677 .45711 .45585 .46723 .47165 .46528 .46262 .45522 Metetra
.559 .704 .938 r
.46262 .45522 .47165 .46528 .48758 .46328 .47859 .45328 Metetra
.56 .707 .939 r
.47859 .45328 .48758 .46328 .50363 .46122 .49469 .45127 Metetra
.56 .71 .941 r
.49469 .45127 .50363 .46122 .51981 .45911 .51092 .44921 Metetra
.561 .713 .942 r
.51092 .44921 .51981 .45911 .53611 .45692 .52728 .44708 Metetra
.562 .715 .944 r
.52728 .44708 .53611 .45692 .55254 .45467 .54377 .44487 Metetra
.563 .718 .945 r
.54377 .44487 .55254 .45467 .5691 .45234 .56038 .44259 Metetra
.565 .721 .947 r
.56038 .44259 .5691 .45234 .58578 .44994 .57713 .44024 Metetra
.566 .724 .948 r
.57713 .44024 .58578 .44994 .6026 .44745 .59401 .43779 Metetra
.567 .727 .949 r
.59401 .43779 .6026 .44745 .61954 .44489 .61102 .43527 Metetra
.569 .73 .95 r
.61102 .43527 .61954 .44489 .63662 .44224 .62816 .43265 Metetra
.57 .732 .951 r
.62816 .43265 .63662 .44224 .65382 .4395 .64543 .42994 Metetra
.572 .735 .952 r
.64543 .42994 .65382 .4395 .67115 .43667 .66284 .42714 Metetra
.574 .738 .953 r
.66284 .42714 .67115 .43667 .68862 .43375 .68038 .42424 Metetra
.576 .74 .954 r
.68038 .42424 .68862 .43375 .70622 .43073 .69805 .42124 Metetra
.577 .743 .955 r
.69805 .42124 .70622 .43073 .72394 .42762 .71586 .41814 Metetra
.579 .746 .956 r
.71586 .41814 .72394 .42762 .74181 .42441 .7338 .41493 Metetra
.581 .748 .956 r
.7338 .41493 .74181 .42441 .7598 .4211 .75187 .41163 Metetra
.583 .75 .957 r
.75187 .41163 .7598 .4211 .77792 .41769 .77008 .40822 Metetra
.585 .753 .957 r
.77008 .40822 .77792 .41769 .79618 .41419 .78843 .40471 Metetra
.587 .755 .958 r
.78843 .40471 .79618 .41419 .81457 .41058 .80691 .40108 Metetra
.589 .757 .958 r
.80691 .40108 .81457 .41058 .83309 .40687 .82552 .39736 Metetra
.591 .759 .958 r
.82552 .39736 .83309 .40687 .85175 .40306 .84427 .39352 Metetra
.57 .673 .909 r
.19955 .4736 .2092 .48452 .22323 .48274 .21361 .47182 Metetra
.568 .674 .91 r
.21361 .47182 .22323 .48274 .23737 .481 .22777 .47008 Metetra
.566 .674 .911 r
.22777 .47008 .23737 .481 .2516 .47928 .24202 .46838 Metetra
.564 .675 .913 r
.24202 .46838 .2516 .47928 .26594 .47759 .25639 .46671 Metetra
.563 .675 .914 r
.25639 .46671 .26594 .47759 .28039 .47591 .27086 .46506 Metetra
.561 .676 .916 r
.27086 .46506 .28039 .47591 .29494 .47425 .28543 .46343 Metetra
.56 .678 .918 r
.28543 .46343 .29494 .47425 .3096 .4726 .30012 .46182 Metetra
.559 .679 .919 r
.30012 .46182 .3096 .4726 .32437 .47095 .31492 .46021 Metetra
.557 .681 .921 r
.31492 .46021 .32437 .47095 .33926 .46929 .32984 .4586 Metetra
.557 .682 .923 r
.32984 .4586 .33926 .46929 .35426 .46763 .34487 .45698 Metetra
.556 .684 .925 r
.34487 .45698 .35426 .46763 .36938 .46594 .36002 .45536 Metetra
.555 .686 .927 r
.36002 .45536 .36938 .46594 .38462 .46424 .37529 .45371 Metetra
.555 .689 .928 r
.37529 .45371 .38462 .46424 .39997 .46251 .39068 .45205 Metetra
.555 .691 .93 r
.39068 .45205 .39997 .46251 .41545 .46075 .4062 .45035 Metetra
.555 .694 .932 r
.4062 .45035 .41545 .46075 .43105 .45895 .42184 .44861 Metetra
.555 .696 .934 r
.42184 .44861 .43105 .45895 .44677 .45711 .4376 .44683 Metetra
.555 .699 .936 r
.4376 .44683 .44677 .45711 .46262 .45522 .45349 .44501 Metetra
.555 .702 .938 r
.45349 .44501 .46262 .45522 .47859 .45328 .46951 .44313 Metetra
.556 .705 .939 r
.46951 .44313 .47859 .45328 .49469 .45127 .48566 .44119 Metetra
.557 .708 .941 r
.48566 .44119 .49469 .45127 .51092 .44921 .50194 .43919 Metetra
.558 .711 .943 r
.50194 .43919 .51092 .44921 .52728 .44708 .51835 .43712 Metetra
.559 .714 .944 r
.51835 .43712 .52728 .44708 .54377 .44487 .5349 .43497 Metetra
.56 .717 .946 r
.5349 .43497 .54377 .44487 .56038 .44259 .55157 .43275 Metetra
.561 .72 .947 r
.55157 .43275 .56038 .44259 .57713 .44024 .56838 .43044 Metetra
.562 .723 .949 r
.56838 .43044 .57713 .44024 .59401 .43779 .58532 .42805 Metetra
.564 .726 .95 r
.58532 .42805 .59401 .43779 .61102 .43527 .6024 .42556 Metetra
.565 .729 .951 r
.6024 .42556 .61102 .43527 .62816 .43265 .6196 .42299 Metetra
.567 .732 .953 r
.6196 .42299 .62816 .43265 .64543 .42994 .63695 .42032 Metetra
.569 .735 .954 r
.63695 .42032 .64543 .42994 .66284 .42714 .65443 .41754 Metetra
.571 .738 .955 r
.65443 .41754 .66284 .42714 .68038 .42424 .67204 .41467 Metetra
.573 .741 .956 r
.67204 .41467 .68038 .42424 .69805 .42124 .68979 .41169 Metetra
.575 .743 .956 r
.68979 .41169 .69805 .42124 .71586 .41814 .70768 .40861 Metetra
.577 .746 .957 r
.70768 .40861 .71586 .41814 .7338 .41493 .7257 .40542 Metetra
.579 .749 .958 r
.7257 .40542 .7338 .41493 .75187 .41163 .74386 .40212 Metetra
.581 .751 .958 r
.74386 .40212 .75187 .41163 .77008 .40822 .76215 .39871 Metetra
.583 .753 .959 r
.76215 .39871 .77008 .40822 .78843 .40471 .78058 .39519 Metetra
.585 .756 .959 r
.78058 .39519 .78843 .40471 .80691 .40108 .79915 .39156 Metetra
.587 .758 .96 r
.79915 .39156 .80691 .40108 .82552 .39736 .81786 .38781 Metetra
.589 .76 .96 r
.81786 .38781 .82552 .39736 .84427 .39352 .8367 .38396 Metetra
.569 .67 .906 r
.18983 .46242 .19955 .4736 .21361 .47182 .20391 .46064 Metetra
.567 .67 .908 r
.20391 .46064 .21361 .47182 .22777 .47008 .21809 .45891 Metetra
.565 .67 .909 r
.21809 .45891 .22777 .47008 .24202 .46838 .23237 .45723 Metetra
.563 .671 .911 r
.23237 .45723 .24202 .46838 .25639 .46671 .24675 .45558 Metetra
.561 .672 .912 r
.24675 .45558 .25639 .46671 .27086 .46506 .26125 .45396 Metetra
.559 .672 .914 r
.26125 .45396 .27086 .46506 .28543 .46343 .27585 .45237 Metetra
.557 .674 .916 r
.27585 .45237 .28543 .46343 .30012 .46182 .29056 .45079 Metetra
.556 .675 .918 r
.29056 .45079 .30012 .46182 .31492 .46021 .30539 .44923 Metetra
.555 .677 .92 r
.30539 .44923 .31492 .46021 .32984 .4586 .32033 .44768 Metetra
.554 .678 .921 r
.32033 .44768 .32984 .4586 .34487 .45698 .33539 .44612 Metetra
.553 .68 .923 r
.33539 .44612 .34487 .45698 .36002 .45536 .35057 .44456 Metetra
.552 .683 .925 r
.35057 .44456 .36002 .45536 .37529 .45371 .36588 .44298 Metetra
.551 .685 .927 r
.36588 .44298 .37529 .45371 .39068 .45205 .3813 .44138 Metetra
.551 .688 .929 r
.3813 .44138 .39068 .45205 .4062 .45035 .39685 .43975 Metetra
.551 .69 .932 r
.39685 .43975 .4062 .45035 .42184 .44861 .41253 .43809 Metetra
.551 .693 .934 r
.41253 .43809 .42184 .44861 .4376 .44683 .42834 .43639 Metetra
.551 .696 .936 r
.42834 .43639 .4376 .44683 .45349 .44501 .44427 .43464 Metetra
.551 .699 .938 r
.44427 .43464 .45349 .44501 .46951 .44313 .46034 .43283 Metetra
.552 .702 .939 r
.46034 .43283 .46951 .44313 .48566 .44119 .47654 .43097 Metetra
.553 .705 .941 r
.47654 .43097 .48566 .44119 .50194 .43919 .49287 .42904 Metetra
.553 .708 .943 r
.49287 .42904 .50194 .43919 .51835 .43712 .50933 .42704 Metetra
.554 .712 .945 r
.50933 .42704 .51835 .43712 .5349 .43497 .52593 .42496 Metetra
.556 .715 .947 r
.52593 .42496 .5349 .43497 .55157 .43275 .54266 .4228 Metetra
.557 .718 .948 r
.54266 .4228 .55157 .43275 .56838 .43044 .55953 .42056 Metetra
.558 .722 .95 r
.55953 .42056 .56838 .43044 .58532 .42805 .57653 .41822 Metetra
.56 .725 .951 r
.57653 .41822 .58532 .42805 .6024 .42556 .59367 .41579 Metetra
.562 .728 .953 r
.59367 .41579 .6024 .42556 .6196 .42299 .61095 .41326 Metetra
.563 .731 .954 r
.61095 .41326 .6196 .42299 .63695 .42032 .62837 .41063 Metetra
.565 .735 .955 r
.62837 .41063 .63695 .42032 .65443 .41754 .64592 .4079 Metetra
.567 .738 .956 r
.64592 .4079 .65443 .41754 .67204 .41467 .66361 .40506 Metetra
.569 .741 .957 r
.66361 .40506 .67204 .41467 .68979 .41169 .68144 .40211 Metetra
.571 .744 .958 r
.68144 .40211 .68979 .41169 .70768 .40861 .6994 .39905 Metetra
.574 .747 .959 r
.6994 .39905 .70768 .40861 .7257 .40542 .71751 .39587 Metetra
.576 .749 .959 r
.71751 .39587 .7257 .40542 .74386 .40212 .73575 .39258 Metetra
.578 .752 .96 r
.73575 .39258 .74386 .40212 .76215 .39871 .75413 .38917 Metetra
.58 .755 .961 r
.75413 .38917 .76215 .39871 .78058 .39519 .77265 .38565 Metetra
.583 .757 .961 r
.77265 .38565 .78058 .39519 .79915 .39156 .79131 .38201 Metetra
.585 .759 .961 r
.79131 .38201 .79915 .39156 .81786 .38781 .81011 .37825 Metetra
.587 .761 .962 r
.81011 .37825 .81786 .38781 .8367 .38396 .82904 .37437 Metetra
.569 .666 .904 r
.18004 .45099 .18983 .46242 .20391 .46064 .19414 .4492 Metetra
.566 .666 .905 r
.19414 .4492 .20391 .46064 .21809 .45891 .20834 .44748 Metetra
.563 .666 .907 r
.20834 .44748 .21809 .45891 .23237 .45723 .22264 .4458 Metetra
.561 .667 .908 r
.22264 .4458 .23237 .45723 .24675 .45558 .23705 .44417 Metetra
.559 .667 .91 r
.23705 .44417 .24675 .45558 .26125 .45396 .25156 .44259 Metetra
.557 .668 .912 r
.25156 .44259 .26125 .45396 .27585 .45237 .26619 .44103 Metetra
.555 .669 .914 r
.26619 .44103 .27585 .45237 .29056 .45079 .28092 .43951 Metetra
.553 .671 .916 r
.28092 .43951 .29056 .45079 .30539 .44923 .29578 .438 Metetra
.552 .672 .918 r
.29578 .438 .30539 .44923 .32033 .44768 .31074 .43651 Metetra
.55 .674 .92 r
.31074 .43651 .32033 .44768 .33539 .44612 .32583 .43502 Metetra
.549 .676 .922 r
.32583 .43502 .33539 .44612 .35057 .44456 .34104 .43352 Metetra
.548 .678 .924 r
.34104 .43352 .35057 .44456 .36588 .44298 .35638 .43202 Metetra
.548 .681 .926 r
.35638 .43202 .36588 .44298 .3813 .44138 .37183 .4305 Metetra
.547 .684 .928 r
.37183 .4305 .3813 .44138 .39685 .43975 .38742 .42896 Metetra
.547 .686 .931 r
.38742 .42896 .39685 .43975 .41253 .43809 .40314 .42738 Metetra
.547 .689 .933 r
.40314 .42738 .41253 .43809 .42834 .43639 .41898 .42576 Metetra
.547 .692 .935 r
.41898 .42576 .42834 .43639 .44427 .43464 .43496 .4241 Metetra
.547 .696 .937 r
.43496 .4241 .44427 .43464 .46034 .43283 .45107 .42238 Metetra
.547 .699 .939 r
.45107 .42238 .46034 .43283 .47654 .43097 .46731 .4206 Metetra
.548 .702 .942 r
.46731 .4206 .47654 .43097 .49287 .42904 .48369 .41876 Metetra
.549 .706 .944 r
.48369 .41876 .49287 .42904 .50933 .42704 .50021 .41684 Metetra
.55 .71 .945 r
.50021 .41684 .50933 .42704 .52593 .42496 .51686 .41484 Metetra
.551 .713 .947 r
.51686 .41484 .52593 .42496 .54266 .4228 .53365 .41276 Metetra
.552 .717 .949 r
.53365 .41276 .54266 .4228 .55953 .42056 .55058 .41058 Metetra
.554 .72 .951 r
.55058 .41058 .55953 .42056 .57653 .41822 .56764 .40832 Metetra
.556 .724 .952 r
.56764 .40832 .57653 .41822 .59367 .41579 .58485 .40595 Metetra
.557 .728 .954 r
.58485 .40595 .59367 .41579 .61095 .41326 .6022 .40348 Metetra
.559 .731 .955 r
.6022 .40348 .61095 .41326 .62837 .41063 .61968 .4009 Metetra
.561 .734 .957 r
.61968 .4009 .62837 .41063 .64592 .4079 .63731 .39821 Metetra
.563 .738 .958 r
.63731 .39821 .64592 .4079 .66361 .40506 .65508 .39541 Metetra
.566 .741 .959 r
.65508 .39541 .66361 .40506 .68144 .40211 .67298 .39249 Metetra
.568 .744 .96 r
.67298 .39249 .68144 .40211 .6994 .39905 .69103 .38946 Metetra
.57 .747 .961 r
.69103 .38946 .6994 .39905 .71751 .39587 .70922 .3863 Metetra
.573 .75 .961 r
.70922 .3863 .71751 .39587 .73575 .39258 .72755 .38302 Metetra
.575 .753 .962 r
.72755 .38302 .73575 .39258 .75413 .38917 .74602 .37962 Metetra
.578 .756 .963 r
.74602 .37962 .75413 .38917 .77265 .38565 .76463 .3761 Metetra
.58 .758 .963 r
.76463 .3761 .77265 .38565 .79131 .38201 .78338 .37245 Metetra
.583 .761 .963 r
.78338 .37245 .79131 .38201 .81011 .37825 .80227 .36867 Metetra
.585 .763 .964 r
.80227 .36867 .81011 .37825 .82904 .37437 .8213 .36477 Metetra
.568 .662 .901 r
.17018 .43926 .18004 .45099 .19414 .4492 .1843 .43747 Metetra
.565 .662 .902 r
.1843 .43747 .19414 .4492 .20834 .44748 .19852 .43574 Metetra
.562 .662 .904 r
.19852 .43574 .20834 .44748 .22264 .4458 .21284 .43408 Metetra
.56 .662 .905 r
.21284 .43408 .22264 .4458 .23705 .44417 .22727 .43248 Metetra
.557 .662 .907 r
.22727 .43248 .23705 .44417 .25156 .44259 .24181 .43092 Metetra
.555 .663 .909 r
.24181 .43092 .25156 .44259 .26619 .44103 .25645 .42941 Metetra
.552 .664 .911 r
.25645 .42941 .26619 .44103 .28092 .43951 .27121 .42794 Metetra
.55 .666 .913 r
.27121 .42794 .28092 .43951 .29578 .438 .28608 .42649 Metetra
.549 .667 .915 r
.28608 .42649 .29578 .438 .31074 .43651 .30108 .42506 Metetra
.547 .669 .918 r
.30108 .42506 .31074 .43651 .32583 .43502 .31619 .42365 Metetra
.546 .671 .92 r
.31619 .42365 .32583 .43502 .34104 .43352 .33143 .42223 Metetra
.545 .674 .922 r
.33143 .42223 .34104 .43352 .35638 .43202 .34679 .42082 Metetra
.544 .676 .925 r
.34679 .42082 .35638 .43202 .37183 .4305 .36228 .41939 Metetra
.543 .679 .927 r
.36228 .41939 .37183 .4305 .38742 .42896 .37789 .41794 Metetra
.542 .682 .93 r
.37789 .41794 .38742 .42896 .40314 .42738 .39364 .41646 Metetra
.542 .685 .932 r
.39364 .41646 .40314 .42738 .41898 .42576 .40953 .41494 Metetra
.542 .689 .935 r
.40953 .41494 .41898 .42576 .43496 .4241 .42554 .41338 Metetra
.542 .692 .937 r
.42554 .41338 .43496 .4241 .45107 .42238 .4417 .41176 Metetra
.542 .696 .939 r
.4417 .41176 .45107 .42238 .46731 .4206 .45799 .41008 Metetra
.543 .699 .942 r
.45799 .41008 .46731 .4206 .48369 .41876 .47441 .40834 Metetra
.544 .703 .944 r
.47441 .40834 .48369 .41876 .50021 .41684 .49098 .40652 Metetra
.545 .707 .946 r
.49098 .40652 .50021 .41684 .51686 .41484 .50769 .40461 Metetra
.546 .711 .948 r
.50769 .40461 .51686 .41484 .53365 .41276 .52453 .40262 Metetra
.547 .715 .95 r
.52453 .40262 .53365 .41276 .55058 .41058 .54152 .40053 Metetra
.549 .719 .952 r
.54152 .40053 .55058 .41058 .56764 .40832 .55865 .39834 Metetra
.551 .723 .954 r
.55865 .39834 .56764 .40832 .58485 .40595 .57593 .39605 Metetra
.552 .727 .956 r
.57593 .39605 .58485 .40595 .6022 .40348 .59334 .39365 Metetra
.554 .731 .957 r
.59334 .39365 .6022 .40348 .61968 .4009 .6109 .39113 Metetra
.557 .734 .958 r
.6109 .39113 .61968 .4009 .63731 .39821 .6286 .3885 Metetra
.559 .738 .96 r
.6286 .3885 .63731 .39821 .65508 .39541 .64644 .38575 Metetra
.561 .742 .961 r
.64644 .38575 .65508 .39541 .67298 .39249 .66443 .38287 Metetra
.564 .745 .962 r
.66443 .38287 .67298 .39249 .69103 .38946 .68256 .37987 Metetra
.566 .748 .963 r
.68256 .37987 .69103 .38946 .70922 .3863 .70083 .37673 Metetra
.569 .752 .964 r
.70083 .37673 .70922 .3863 .72755 .38302 .71925 .37347 Metetra
.572 .755 .964 r
.71925 .37347 .72755 .38302 .74602 .37962 .73781 .37008 Metetra
.574 .758 .965 r
.73781 .37008 .74602 .37962 .76463 .3761 .75651 .36655 Metetra
.577 .76 .965 r
.75651 .36655 .76463 .3761 .78338 .37245 .77535 .36289 Metetra
.58 .763 .965 r
.77535 .36289 .78338 .37245 .80227 .36867 .79434 .3591 Metetra
.583 .765 .966 r
.79434 .3591 .80227 .36867 .8213 .36477 .81347 .35518 Metetra
.568 .658 .897 r
.16026 .42722 .17018 .43926 .1843 .43747 .1744 .42541 Metetra
.564 .657 .899 r
.1744 .42541 .1843 .43747 .19852 .43574 .18864 .42368 Metetra
.561 .657 .9 r
.18864 .42368 .19852 .43574 .21284 .43408 .20298 .42203 Metetra
.558 .657 .902 r
.20298 .42203 .21284 .43408 .22727 .43248 .21743 .42045 Metetra
.555 .657 .904 r
.21743 .42045 .22727 .43248 .24181 .43092 .23198 .41893 Metetra
.552 .658 .906 r
.23198 .41893 .24181 .43092 .25645 .42941 .24664 .41746 Metetra
.55 .659 .908 r
.24664 .41746 .25645 .42941 .27121 .42794 .26142 .41604 Metetra
.548 .66 .91 r
.26142 .41604 .27121 .42794 .28608 .42649 .27631 .41466 Metetra
.545 .662 .913 r
.27631 .41466 .28608 .42649 .30108 .42506 .29133 .41331 Metetra
.544 .664 .915 r
.29133 .41331 .30108 .42506 .31619 .42365 .30646 .41198 Metetra
.542 .666 .918 r
.30646 .41198 .31619 .42365 .33143 .42223 .32172 .41066 Metetra
.54 .668 .92 r
.32172 .41066 .33143 .42223 .34679 .42082 .33711 .40934 Metetra
.539 .671 .923 r
.33711 .40934 .34679 .42082 .36228 .41939 .35263 .40802 Metetra
.538 .674 .926 r
.35263 .40802 .36228 .41939 .37789 .41794 .36828 .40668 Metetra
.537 .677 .928 r
.36828 .40668 .37789 .41794 .39364 .41646 .38406 .40531 Metetra
.537 .681 .931 r
.38406 .40531 .39364 .41646 .40953 .41494 .39998 .40391 Metetra
.537 .684 .934 r
.39998 .40391 .40953 .41494 .42554 .41338 .41603 .40247 Metetra
.537 .688 .937 r
.41603 .40247 .42554 .41338 .4417 .41176 .43223 .40097 Metetra
.537 .692 .939 r
.43223 .40097 .4417 .41176 .45799 .41008 .44856 .39941 Metetra
.537 .696 .942 r
.44856 .39941 .45799 .41008 .47441 .40834 .46503 .39778 Metetra
.538 .7 .944 r
.46503 .39778 .47441 .40834 .49098 .40652 .48165 .39607 Metetra
.539 .704 .947 r
.48165 .39607 .49098 .40652 .50769 .40461 .49841 .39428 Metetra
.54 .709 .949 r
.49841 .39428 .50769 .40461 .52453 .40262 .51531 .39239 Metetra
.541 .713 .951 r
.51531 .39239 .52453 .40262 .54152 .40053 .53236 .39041 Metetra
.543 .717 .954 r
.53236 .39041 .54152 .40053 .55865 .39834 .54956 .38831 Metetra
.545 .722 .955 r
.54956 .38831 .55865 .39834 .57593 .39605 .5669 .38611 Metetra
.547 .726 .957 r
.5669 .38611 .57593 .39605 .59334 .39365 .58438 .38379 Metetra
.549 .73 .959 r
.58438 .38379 .59334 .39365 .6109 .39113 .60201 .38135 Metetra
.551 .734 .961 r
.60201 .38135 .6109 .39113 .6286 .3885 .61979 .37878 Metetra
.554 .738 .962 r
.61979 .37878 .6286 .3885 .64644 .38575 .63771 .37608 Metetra
.557 .742 .963 r
.63771 .37608 .64644 .38575 .66443 .38287 .65578 .37325 Metetra
.559 .746 .964 r
.65578 .37325 .66443 .38287 .68256 .37987 .67399 .37029 Metetra
.562 .75 .965 r
.67399 .37029 .68256 .37987 .70083 .37673 .69235 .36719 Metetra
.565 .753 .966 r
.69235 .36719 .70083 .37673 .71925 .37347 .71085 .36394 Metetra
.568 .757 .967 r
.71085 .36394 .71925 .37347 .73781 .37008 .7295 .36056 Metetra
.571 .76 .967 r
.7295 .36056 .73781 .37008 .75651 .36655 .74829 .35704 Metetra
.574 .763 .968 r
.74829 .35704 .75651 .36655 .77535 .36289 .76723 .35337 Metetra
.577 .765 .968 r
.76723 .35337 .77535 .36289 .79434 .3591 .78632 .34956 Metetra
.58 .768 .968 r
.78632 .34956 .79434 .3591 .81347 .35518 .80554 .34562 Metetra
.567 .653 .894 r
.15029 .41483 .16026 .42722 .1744 .42541 .16444 .413 Metetra
.564 .652 .895 r
.16444 .413 .1744 .42541 .18864 .42368 .1787 .41126 Metetra
.56 .652 .896 r
.1787 .41126 .18864 .42368 .20298 .42203 .19306 .40962 Metetra
.557 .651 .898 r
.19306 .40962 .20298 .42203 .21743 .42045 .20752 .40805 Metetra
.553 .651 .9 r
.20752 .40805 .21743 .42045 .23198 .41893 .22209 .40657 Metetra
.55 .652 .902 r
.22209 .40657 .23198 .41893 .24664 .41746 .23677 .40515 Metetra
.547 .653 .904 r
.23677 .40515 .24664 .41746 .26142 .41604 .25156 .40379 Metetra
.545 .654 .907 r
.25156 .40379 .26142 .41604 .27631 .41466 .26647 .40248 Metetra
.542 .655 .909 r
.26647 .40248 .27631 .41466 .29133 .41331 .2815 .40122 Metetra
.54 .657 .912 r
.2815 .40122 .29133 .41331 .30646 .41198 .29666 .39998 Metetra
.538 .659 .915 r
.29666 .39998 .30646 .41198 .32172 .41066 .31194 .39877 Metetra
.536 .662 .918 r
.31194 .39877 .32172 .41066 .33711 .40934 .32735 .39757 Metetra
.534 .665 .921 r
.32735 .39757 .33711 .40934 .35263 .40802 .34289 .39636 Metetra
.533 .668 .924 r
.34289 .39636 .35263 .40802 .36828 .40668 .35857 .39515 Metetra
.532 .671 .927 r
.35857 .39515 .36828 .40668 .38406 .40531 .37438 .39392 Metetra
.531 .675 .93 r
.37438 .39392 .38406 .40531 .39998 .40391 .39033 .39265 Metetra
.53 .679 .933 r
.39033 .39265 .39998 .40391 .41603 .40247 .40642 .39135 Metetra
.53 .683 .936 r
.40642 .39135 .41603 .40247 .43223 .40097 .42265 .38999 Metetra
.53 .688 .939 r
.42265 .38999 .43223 .40097 .44856 .39941 .43903 .38857 Metetra
.531 .692 .942 r
.43903 .38857 .44856 .39941 .46503 .39778 .45555 .38708 Metetra
.531 .697 .945 r
.45555 .38708 .46503 .39778 .48165 .39607 .47222 .38551 Metetra
.532 .701 .948 r
.47222 .38551 .48165 .39607 .49841 .39428 .48903 .38384 Metetra
.533 .706 .95 r
.48903 .38384 .49841 .39428 .51531 .39239 .50599 .38208 Metetra
.535 .711 .953 r
.50599 .38208 .51531 .39239 .53236 .39041 .5231 .38022 Metetra
.536 .716 .955 r
.5231 .38022 .53236 .39041 .54956 .38831 .54035 .37824 Metetra
.538 .721 .957 r
.54035 .37824 .54956 .38831 .5669 .38611 .55776 .37614 Metetra
.541 .725 .959 r
.55776 .37614 .5669 .38611 .58438 .38379 .57531 .37392 Metetra
.543 .73 .961 r
.57531 .37392 .58438 .38379 .60201 .38135 .59301 .37156 Metetra
.545 .735 .963 r
.59301 .37156 .60201 .38135 .61979 .37878 .61087 .36907 Metetra
.548 .739 .965 r
.61087 .36907 .61979 .37878 .63771 .37608 .62887 .36644 Metetra
.551 .743 .966 r
.62887 .36644 .63771 .37608 .65578 .37325 .64702 .36367 Metetra
.554 .748 .967 r
.64702 .36367 .65578 .37325 .67399 .37029 .66531 .36075 Metetra
.557 .752 .968 r
.66531 .36075 .67399 .37029 .69235 .36719 .68376 .35769 Metetra
.56 .755 .969 r
.68376 .35769 .69235 .36719 .71085 .36394 .70236 .35447 Metetra
.564 .759 .97 r
.70236 .35447 .71085 .36394 .7295 .36056 .7211 .3511 Metetra
.567 .762 .97 r
.7211 .3511 .7295 .36056 .74829 .35704 .73999 .34758 Metetra
.57 .765 .971 r
.73999 .34758 .74829 .35704 .76723 .35337 .75902 .34391 Metetra
.574 .768 .971 r
.75902 .34391 .76723 .35337 .78632 .34956 .7782 .34008 Metetra
.577 .771 .971 r
.7782 .34008 .78632 .34956 .80554 .34562 .79753 .3361 Metetra
.568 .648 .889 r
.14025 .40207 .15029 .41483 .16444 .413 .15443 .4002 Metetra
.564 .647 .89 r
.15443 .4002 .16444 .413 .1787 .41126 .1687 .39844 Metetra
.559 .646 .892 r
.1687 .39844 .1787 .41126 .19306 .40962 .18308 .39679 Metetra
.556 .645 .894 r
.18308 .39679 .19306 .40962 .20752 .40805 .19755 .39524 Metetra
.552 .645 .895 r
.19755 .39524 .20752 .40805 .22209 .40657 .21214 .39379 Metetra
.548 .645 .898 r
.21214 .39379 .22209 .40657 .23677 .40515 .22683 .39242 Metetra
.545 .646 .9 r
.22683 .39242 .23677 .40515 .25156 .40379 .24164 .39112 Metetra
.541 .647 .902 r
.24164 .39112 .25156 .40379 .26647 .40248 .25656 .3899 Metetra
.538 .648 .905 r
.25656 .3899 .26647 .40248 .2815 .40122 .27161 .38873 Metetra
.536 .65 .908 r
.27161 .38873 .2815 .40122 .29666 .39998 .28678 .3876 Metetra
.533 .652 .911 r
.28678 .3876 .29666 .39998 .31194 .39877 .30208 .38651 Metetra
.531 .655 .914 r
.30208 .38651 .31194 .39877 .32735 .39757 .31751 .38544 Metetra
.529 .658 .918 r
.31751 .38544 .32735 .39757 .34289 .39636 .33307 .38438 Metetra
.527 .661 .921 r
.33307 .38438 .34289 .39636 .35857 .39515 .34877 .38332 Metetra
.526 .665 .925 r
.34877 .38332 .35857 .39515 .37438 .39392 .36461 .38224 Metetra
.524 .669 .928 r
.36461 .38224 .37438 .39392 .39033 .39265 .38059 .38114 Metetra
.524 .673 .932 r
.38059 .38114 .39033 .39265 .40642 .39135 .39672 .38 Metetra
.523 .678 .935 r
.39672 .38 .40642 .39135 .42265 .38999 .41298 .37881 Metetra
.523 .682 .939 r
.41298 .37881 .42265 .38999 .43903 .38857 .4294 .37755 Metetra
.523 .687 .942 r
.4294 .37755 .43903 .38857 .45555 .38708 .44596 .37623 Metetra
.524 .693 .945 r
.44596 .37623 .45555 .38708 .47222 .38551 .46267 .37482 Metetra
.524 .698 .948 r
.46267 .37482 .47222 .38551 .48903 .38384 .47954 .37331 Metetra
.525 .703 .952 r
.47954 .37331 .48903 .38384 .50599 .38208 .49655 .3717 Metetra
.527 .709 .954 r
.49655 .3717 .50599 .38208 .5231 .38022 .51372 .36998 Metetra
.529 .714 .957 r
.51372 .36998 .5231 .38022 .54035 .37824 .53104 .36814 Metetra
.531 .72 .96 r
.53104 .36814 .54035 .37824 .55776 .37614 .54851 .36617 Metetra
.533 .725 .962 r
.54851 .36617 .55776 .37614 .57531 .37392 .56613 .36406 Metetra
.535 .73 .964 r
.56613 .36406 .57531 .37392 .59301 .37156 .58391 .36181 Metetra
.538 .735 .966 r
.58391 .36181 .59301 .37156 .61087 .36907 .60184 .35942 Metetra
.541 .74 .968 r
.60184 .35942 .61087 .36907 .62887 .36644 .61992 .35687 Metetra
.544 .745 .969 r
.61992 .35687 .62887 .36644 .64702 .36367 .63815 .35416 Metetra
.548 .75 .97 r
.63815 .35416 .64702 .36367 .66531 .36075 .65654 .3513 Metetra
.551 .754 .971 r
.65654 .3513 .66531 .36075 .68376 .35769 .67507 .34828 Metetra
.555 .758 .972 r
.67507 .34828 .68376 .35769 .70236 .35447 .69376 .34509 Metetra
.559 .762 .973 r
.69376 .34509 .70236 .35447 .7211 .3511 .71259 .34174 Metetra
.562 .766 .973 r
.71259 .34174 .7211 .3511 .73999 .34758 .73158 .33822 Metetra
.566 .769 .974 r
.73158 .33822 .73999 .34758 .75902 .34391 .75071 .33454 Metetra
.57 .772 .974 r
.75071 .33454 .75902 .34391 .7782 .34008 .77 .33069 Metetra
.574 .775 .974 r
.77 .33069 .7782 .34008 .79753 .3361 .78943 .32668 Metetra
.568 .643 .884 r
.13017 .38889 .14025 .40207 .15443 .4002 .14437 .38696 Metetra
.564 .641 .885 r
.14437 .38696 .15443 .4002 .1687 .39844 .15866 .38517 Metetra
.559 .639 .886 r
.15866 .38517 .1687 .39844 .18308 .39679 .17305 .3835 Metetra
.555 .638 .888 r
.17305 .3835 .18308 .39679 .19755 .39524 .18754 .38196 Metetra
.55 .638 .89 r
.18754 .38196 .19755 .39524 .21214 .39379 .20214 .38053 Metetra
.546 .637 .892 r
.20214 .38053 .21214 .39379 .22683 .39242 .21684 .37921 Metetra
.542 .638 .895 r
.21684 .37921 .22683 .39242 .24164 .39112 .23166 .37798 Metetra
.538 .638 .897 r
.23166 .37798 .24164 .39112 .25656 .3899 .2466 .37684 Metetra
.535 .64 .9 r
.2466 .37684 .25656 .3899 .27161 .38873 .26165 .37578 Metetra
.531 .641 .903 r
.26165 .37578 .27161 .38873 .28678 .3876 .27684 .37478 Metetra
.528 .643 .907 r
.27684 .37478 .28678 .3876 .30208 .38651 .29215 .37383 Metetra
.525 .646 .91 r
.29215 .37383 .30208 .38651 .31751 .38544 .30759 .37291 Metetra
.523 .649 .914 r
.30759 .37291 .31751 .38544 .33307 .38438 .32317 .37202 Metetra
.521 .653 .918 r
.32317 .37202 .33307 .38438 .34877 .38332 .33889 .37113 Metetra
.519 .657 .922 r
.33889 .37113 .34877 .38332 .36461 .38224 .35475 .37025 Metetra
.517 .661 .926 r
.35475 .37025 .36461 .38224 .38059 .38114 .37076 .36934 Metetra
.516 .666 .93 r
.37076 .36934 .38059 .38114 .39672 .38 .38691 .36839 Metetra
.515 .671 .934 r
.38691 .36839 .39672 .38 .41298 .37881 .40321 .3674 Metetra
.514 .676 .938 r
.40321 .3674 .41298 .37881 .4294 .37755 .41966 .36635 Metetra
.514 .682 .942 r
.41966 .36635 .4294 .37755 .44596 .37623 .43627 .36522 Metetra
.514 .688 .946 r
.43627 .36522 .44596 .37623 .46267 .37482 .45302 .36401 Metetra
.515 .694 .949 r
.45302 .36401 .46267 .37482 .47954 .37331 .46994 .36269 Metetra
.516 .7 .953 r
.46994 .36269 .47954 .37331 .49655 .3717 .48701 .36127 Metetra
.518 .706 .956 r
.48701 .36127 .49655 .3717 .51372 .36998 .50423 .35972 Metetra
.519 .712 .959 r
.50423 .35972 .51372 .36998 .53104 .36814 .52161 .35805 Metetra
.521 .718 .962 r
.52161 .35805 .53104 .36814 .54851 .36617 .53915 .35623 Metetra
.524 .724 .965 r
.53915 .35623 .54851 .36617 .56613 .36406 .55684 .35426 Metetra
.527 .73 .967 r
.55684 .35426 .56613 .36406 .58391 .36181 .57469 .35214 Metetra
.53 .736 .969 r
.57469 .35214 .58391 .36181 .60184 .35942 .5927 .34985 Metetra
.533 .742 .971 r
.5927 .34985 .60184 .35942 .61992 .35687 .61086 .3474 Metetra
.537 .747 .973 r
.61086 .3474 .61992 .35687 .63815 .35416 .62918 .34478 Metetra
.54 .752 .974 r
.62918 .34478 .63815 .35416 .65654 .3513 .64765 .34198 Metetra
.544 .757 .975 r
.64765 .34198 .65654 .3513 .67507 .34828 .66628 .33901 Metetra
.549 .762 .976 r
.66628 .33901 .67507 .34828 .69376 .34509 .68506 .33585 Metetra
.553 .766 .977 r
.68506 .33585 .69376 .34509 .71259 .34174 .70399 .33252 Metetra
.557 .77 .977 r
.70399 .33252 .71259 .34174 .73158 .33822 .72308 .329 Metetra
.561 .773 .978 r
.72308 .329 .73158 .33822 .75071 .33454 .74231 .3253 Metetra
.566 .777 .978 r
.74231 .3253 .75071 .33454 .77 .33069 .7617 .32143 Metetra
.57 .779 .977 r
.7617 .32143 .77 .33069 .78943 .32668 .78123 .31738 Metetra
.57 .637 .878 r
.12006 .37526 .13017 .38889 .14437 .38696 .13427 .37324 Metetra
.565 .634 .879 r
.13427 .37324 .14437 .38696 .15866 .38517 .14858 .37138 Metetra
.56 .632 .88 r
.14858 .37138 .15866 .38517 .17305 .3835 .16299 .36968 Metetra
.555 .631 .882 r
.16299 .36968 .17305 .3835 .18754 .38196 .17749 .36813 Metetra
.55 .629 .883 r
.17749 .36813 .18754 .38196 .20214 .38053 .1921 .36671 Metetra
.545 .629 .885 r
.1921 .36671 .20214 .38053 .21684 .37921 .20681 .36543 Metetra
.54 .628 .888 r
.20681 .36543 .21684 .37921 .23166 .37798 .22164 .36428 Metetra
.535 .629 .891 r
.22164 .36428 .23166 .37798 .2466 .37684 .23658 .36323 Metetra
.531 .63 .894 r
.23658 .36323 .2466 .37684 .26165 .37578 .25164 .36229 Metetra
.527 .631 .897 r
.25164 .36229 .26165 .37578 .27684 .37478 .26683 .36142 Metetra
.523 .633 .901 r
.26683 .36142 .27684 .37478 .29215 .37383 .28215 .36063 Metetra
.519 .636 .905 r
.28215 .36063 .29215 .37383 .30759 .37291 .29761 .3599 Metetra
.516 .639 .909 r
.29761 .3599 .30759 .37291 .32317 .37202 .3132 .35921 Metetra
.513 .643 .914 r
.3132 .35921 .32317 .37202 .33889 .37113 .32893 .35854 Metetra
.511 .647 .918 r
.32893 .35854 .33889 .37113 .35475 .37025 .34481 .35787 Metetra
.508 .652 .923 r
.34481 .35787 .35475 .37025 .37076 .36934 .36083 .3572 Metetra
.507 .657 .928 r
.36083 .3572 .37076 .36934 .38691 .36839 .37701 .35649 Metetra
.505 .663 .932 r
.37701 .35649 .38691 .36839 .40321 .3674 .39334 .35575 Metetra
.504 .669 .937 r
.39334 .35575 .40321 .3674 .41966 .36635 .40982 .35494 Metetra
.504 .675 .942 r
.40982 .35494 .41966 .36635 .43627 .36522 .42646 .35406 Metetra
.503 .682 .946 r
.42646 .35406 .43627 .36522 .45302 .36401 .44326 .35309 Metetra
.504 .689 .951 r
.44326 .35309 .45302 .36401 .46994 .36269 .46022 .35201 Metetra
.505 .696 .955 r
.46022 .35201 .46994 .36269 .48701 .36127 .47734 .35081 Metetra
.506 .703 .959 r
.47734 .35081 .48701 .36127 .50423 .35972 .49462 .34948 Metetra
.508 .71 .962 r
.49462 .34948 .50423 .35972 .52161 .35805 .51207 .348 Metetra
.51 .717 .966 r
.51207 .348 .52161 .35805 .53915 .35623 .52967 .34637 Metetra
.513 .724 .969 r
.52967 .34637 .53915 .35623 .55684 .35426 .54744 .34457 Metetra
.516 .731 .971 r
.54744 .34457 .55684 .35426 .57469 .35214 .56536 .3426 Metetra
.519 .737 .974 r
.56536 .3426 .57469 .35214 .5927 .34985 .58345 .34045 Metetra
.523 .744 .976 r
.58345 .34045 .5927 .34985 .61086 .3474 .6017 .33811 Metetra
.527 .75 .977 r
.6017 .33811 .61086 .3474 .62918 .34478 .6201 .33559 Metetra
.531 .756 .979 r
.6201 .33559 .62918 .34478 .64765 .34198 .63866 .33286 Metetra
.536 .761 .98 r
.63866 .33286 .64765 .34198 .66628 .33901 .65738 .32994 Metetra
.541 .766 .981 r
.65738 .32994 .66628 .33901 .68506 .33585 .67626 .32682 Metetra
.546 .771 .981 r
.67626 .32682 .68506 .33585 .70399 .33252 .69529 .32351 Metetra
.551 .775 .982 r
.69529 .32351 .70399 .33252 .72308 .329 .71447 .31999 Metetra
.556 .779 .982 r
.71447 .31999 .72308 .329 .74231 .3253 .73381 .31627 Metetra
.561 .782 .982 r
.73381 .31627 .74231 .3253 .7617 .32143 .7533 .31236 Metetra
.566 .785 .981 r
.7533 .31236 .7617 .32143 .78123 .31738 .77294 .30825 Metetra
.573 .631 .872 r
.10991 .36111 .12006 .37526 .13427 .37324 .12415 .35897 Metetra
.567 .628 .872 r
.12415 .35897 .13427 .37324 .14858 .37138 .13848 .35701 Metetra
.561 .625 .873 r
.13848 .35701 .14858 .37138 .16299 .36968 .1529 .35524 Metetra
.555 .622 .874 r
.1529 .35524 .16299 .36968 .17749 .36813 .16741 .35365 Metetra
.549 .62 .875 r
.16741 .35365 .17749 .36813 .1921 .36671 .18203 .35223 Metetra
.544 .619 .877 r
.18203 .35223 .1921 .36671 .20681 .36543 .19675 .35098 Metetra
.538 .618 .88 r
.19675 .35098 .20681 .36543 .22164 .36428 .21158 .34989 Metetra
.533 .617 .882 r
.21158 .34989 .22164 .36428 .23658 .36323 .22653 .34894 Metetra
.527 .618 .886 r
.22653 .34894 .23658 .36323 .25164 .36229 .24159 .34812 Metetra
.522 .619 .889 r
.24159 .34812 .25164 .36229 .26683 .36142 .25678 .34742 Metetra
.518 .621 .893 r
.25678 .34742 .26683 .36142 .28215 .36063 .2721 .34682 Metetra
.513 .623 .898 r
.2721 .34682 .28215 .36063 .29761 .3599 .28756 .3463 Metetra
.509 .627 .903 r
.28756 .3463 .29761 .3599 .3132 .35921 .30316 .34584 Metetra
.505 .631 .908 r
.30316 .34584 .3132 .35921 .32893 .35854 .3189 .34543 Metetra
.501 .635 .913 r
.3189 .34543 .32893 .35854 .34481 .35787 .33478 .34504 Metetra
.498 .641 .919 r
.33478 .34504 .34481 .35787 .36083 .3572 .35082 .34465 Metetra
.496 .646 .924 r
.35082 .34465 .36083 .3572 .37701 .35649 .36701 .34425 Metetra
.494 .653 .93 r
.36701 .34425 .37701 .35649 .39334 .35575 .38337 .34381 Metetra
.492 .66 .936 r
.38337 .34381 .39334 .35575 .40982 .35494 .39988 .34331 Metetra
.491 .667 .941 r
.39988 .34331 .40982 .35494 .42646 .35406 .41655 .34273 Metetra
.49 .675 .947 r
.41655 .34273 .42646 .35406 .44326 .35309 .43339 .34206 Metetra
.49 .683 .952 r
.43339 .34206 .44326 .35309 .46022 .35201 .45039 .34128 Metetra
.49 .691 .957 r
.45039 .34128 .46022 .35201 .47734 .35081 .46756 .34036 Metetra
.491 .699 .961 r
.46756 .34036 .47734 .35081 .49462 .34948 .4849 .33929 Metetra
.493 .708 .966 r
.4849 .33929 .49462 .34948 .51207 .348 .5024 .33807 Metetra
.495 .716 .97 r
.5024 .33807 .51207 .348 .52967 .34637 .52007 .33666 Metetra
.498 .724 .973 r
.52007 .33666 .52967 .34637 .54744 .34457 .53791 .33508 Metetra
.502 .732 .976 r
.53791 .33508 .54744 .34457 .56536 .3426 .55591 .33329 Metetra
.505 .74 .979 r
.55591 .33329 .56536 .3426 .58345 .34045 .57408 .33131 Metetra
.51 .747 .981 r
.57408 .33131 .58345 .34045 .6017 .33811 .59241 .32911 Metetra
.515 .754 .983 r
.59241 .32911 .6017 .33811 .6201 .33559 .61091 .32669 Metetra
.52 .76 .984 r
.61091 .32669 .6201 .33559 .63866 .33286 .62956 .32405 Metetra
.525 .767 .985 r
.62956 .32405 .63866 .33286 .65738 .32994 .64838 .32119 Metetra
.531 .772 .986 r
.64838 .32119 .65738 .32994 .67626 .32682 .66735 .31811 Metetra
.537 .777 .986 r
.66735 .31811 .67626 .32682 .69529 .32351 .68649 .3148 Metetra
.543 .782 .987 r
.68649 .3148 .69529 .32351 .71447 .31999 .70578 .31127 Metetra
.549 .786 .987 r
.70578 .31127 .71447 .31999 .73381 .31627 .72522 .30751 Metetra
.556 .79 .986 r
.72522 .30751 .73381 .31627 .7533 .31236 .74481 .30354 Metetra
.562 .793 .986 r
.74481 .30354 .7533 .31236 .77294 .30825 .76456 .29936 Metetra
.577 .626 .864 r
.09974 .34642 .10991 .36111 .12415 .35897 .11401 .34409 Metetra
.57 .621 .864 r
.11401 .34409 .12415 .35897 .13848 .35701 .12836 .34197 Metetra
.564 .617 .864 r
.12836 .34197 .13848 .35701 .1529 .35524 .1428 .34008 Metetra
.557 .613 .864 r
.1428 .34008 .1529 .35524 .16741 .35365 .15733 .33841 Metetra
.55 .61 .865 r
.15733 .33841 .16741 .35365 .18203 .35223 .17196 .33695 Metetra
.543 .607 .867 r
.17196 .33695 .18203 .35223 .19675 .35098 .18669 .33571 Metetra
.537 .605 .869 r
.18669 .33571 .19675 .35098 .21158 .34989 .20152 .33466 Metetra
.53 .604 .872 r
.20152 .33466 .21158 .34989 .22653 .34894 .21646 .3338 Metetra
.524 .604 .875 r
.21646 .3338 .22653 .34894 .24159 .34812 .23152 .33312 Metetra
.518 .604 .879 r
.23152 .33312 .24159 .34812 .25678 .34742 .24671 .33259 Metetra
.512 .606 .884 r
.24671 .33259 .25678 .34742 .2721 .34682 .26202 .33221 Metetra
.506 .608 .888 r
.26202 .33221 .2721 .34682 .28756 .3463 .27747 .33194 Metetra
.501 .611 .894 r
.27747 .33194 .28756 .3463 .30316 .34584 .29306 .33177 Metetra
.496 .615 .9 r
.29306 .33177 .30316 .34584 .3189 .34543 .3088 .33168 Metetra
.491 .62 .906 r
.3088 .33168 .3189 .34543 .33478 .34504 .32469 .33163 Metetra
.487 .626 .913 r
.32469 .33163 .33478 .34504 .35082 .34465 .34073 .33161 Metetra
.483 .633 .92 r
.34073 .33161 .35082 .34465 .36701 .34425 .35693 .33158 Metetra
.479 .64 .926 r
.35693 .33158 .36701 .34425 .38337 .34381 .3733 .33153 Metetra
.476 .648 .933 r
.3733 .33153 .38337 .34381 .39988 .34331 .38983 .33142 Metetra
.474 .657 .94 r
.38983 .33142 .39988 .34331 .41655 .34273 .40653 .33124 Metetra
.473 .666 .947 r
.40653 .33124 .41655 .34273 .43339 .34206 .4234 .33095 Metetra
.472 .675 .953 r
.4234 .33095 .43339 .34206 .45039 .34128 .44044 .33054 Metetra
.472 .685 .959 r
.44044 .33054 .45039 .34128 .46756 .34036 .45766 .32998 Metetra
.472 .695 .965 r
.45766 .32998 .46756 .34036 .4849 .33929 .47505 .32926 Metetra
.474 .705 .97 r
.47505 .32926 .4849 .33929 .5024 .33807 .49261 .32835 Metetra
.476 .715 .974 r
.49261 .32835 .5024 .33807 .52007 .33666 .51035 .32724 Metetra
.479 .724 .978 r
.51035 .32724 .52007 .33666 .53791 .33508 .52826 .32591 Metetra
.482 .734 .982 r
.52826 .32591 .53791 .33508 .55591 .33329 .54634 .32435 Metetra
.487 .743 .985 r
.54634 .32435 .55591 .33329 .57408 .33131 .56459 .32256 Metetra
.492 .751 .987 r
.56459 .32256 .57408 .33131 .59241 .32911 .58301 .32052 Metetra
.498 .76 .989 r
.58301 .32052 .59241 .32911 .61091 .32669 .6016 .31824 Metetra
.504 .767 .99 r
.6016 .31824 .61091 .32669 .62956 .32405 .62035 .31569 Metetra
.511 .774 .991 r
.62035 .31569 .62956 .32405 .64838 .32119 .63927 .31289 Metetra
.519 .78 .992 r
.63927 .31289 .64838 .32119 .66735 .31811 .65835 .30983 Metetra
.526 .786 .992 r
.65835 .30983 .66735 .31811 .68649 .3148 .67758 .30652 Metetra
.534 .791 .992 r
.67758 .30652 .68649 .3148 .70578 .31127 .69698 .30295 Metetra
.541 .795 .992 r
.69698 .30295 .70578 .31127 .72522 .30751 .71653 .29914 Metetra
.549 .799 .991 r
.71653 .29914 .72522 .30751 .74481 .30354 .73623 .29508 Metetra
.557 .802 .99 r
.73623 .29508 .74481 .30354 .76456 .29936 .75609 .29079 Metetra
.583 .622 .856 r
.08955 .33117 .09974 .34642 .11401 .34409 .10387 .32855 Metetra
.576 .615 .855 r
.10387 .32855 .11401 .34409 .12836 .34197 .11826 .32618 Metetra
.568 .609 .854 r
.11826 .32618 .12836 .34197 .1428 .34008 .13272 .32408 Metetra
.56 .603 .853 r
.13272 .32408 .1428 .34008 .15733 .33841 .14727 .32225 Metetra
.553 .599 .853 r
.14727 .32225 .15733 .33841 .17196 .33695 .16191 .32069 Metetra
.545 .594 .854 r
.16191 .32069 .17196 .33695 .18669 .33571 .17664 .31939 Metetra
.537 .591 .856 r
.17664 .31939 .18669 .33571 .20152 .33466 .19148 .31835 Metetra
.529 .588 .858 r
.19148 .31835 .20152 .33466 .21646 .3338 .20641 .31756 Metetra
.521 .587 .861 r
.20641 .31756 .21646 .3338 .23152 .33312 .22147 .31701 Metetra
.514 .586 .865 r
.22147 .31701 .23152 .33312 .24671 .33259 .23664 .31668 Metetra
.506 .587 .87 r
.23664 .31668 .24671 .33259 .26202 .33221 .25193 .31654 Metetra
.499 .589 .875 r
.25193 .31654 .26202 .33221 .27747 .33194 .26737 .31658 Metetra
.492 .591 .881 r
.26737 .31658 .27747 .33194 .29306 .33177 .28294 .31676 Metetra
.485 .595 .888 r
.28294 .31676 .29306 .33177 .3088 .33168 .29866 .31706 Metetra
.479 .601 .896 r
.29866 .31706 .3088 .33168 .32469 .33163 .31453 .31745 Metetra
.472 .607 .904 r
.31453 .31745 .32469 .33163 .34073 .33161 .33057 .3179 Metetra
.467 .614 .912 r
.33057 .3179 .34073 .33161 .35693 .33158 .34677 .31837 Metetra
.461 .623 .921 r
.34677 .31837 .35693 .33158 .3733 .33153 .36313 .31883 Metetra
.457 .632 .93 r
.36313 .31883 .3733 .33153 .38983 .33142 .37967 .31924 Metetra
.453 .642 .939 r
.37967 .31924 .38983 .33142 .40653 .33124 .39639 .31957 Metetra
.449 .653 .947 r
.39639 .31957 .40653 .33124 .4234 .33095 .41329 .3198 Metetra
.447 .665 .955 r
.41329 .3198 .4234 .33095 .44044 .33054 .43036 .31988 Metetra
.446 .677 .962 r
.43036 .31988 .44044 .33054 .45766 .32998 .44762 .3198 Metetra
.445 .689 .969 r
.44762 .3198 .45766 .32998 .47505 .32926 .46506 .31952 Metetra
.446 .701 .975 r
.46506 .31952 .47505 .32926 .49261 .32835 .48269 .31902 Metetra
.448 .713 .98 r
.48269 .31902 .49261 .32835 .51035 .32724 .50049 .31828 Metetra
.451 .725 .985 r
.50049 .31828 .51035 .32724 .52826 .32591 .51847 .31728 Metetra
.455 .736 .988 r
.51847 .31728 .52826 .32591 .54634 .32435 .53664 .31601 Metetra
.46 .747 .991 r
.53664 .31601 .54634 .32435 .56459 .32256 .55498 .31445 Metetra
.467 .758 .993 r
.55498 .31445 .56459 .32256 .58301 .32052 .57349 .31261 Metetra
.474 .767 .995 r
.57349 .31261 .58301 .32052 .6016 .31824 .59218 .31046 Metetra
.483 .776 .996 r
.59218 .31046 .6016 .31824 .62035 .31569 .61103 .30801 Metetra
.492 .784 .997 r
.61103 .30801 .62035 .31569 .63927 .31289 .63005 .30525 Metetra
.501 .791 .997 r
.63005 .30525 .63927 .31289 .65835 .30983 .64924 .3022 Metetra
.511 .797 .997 r
.64924 .3022 .65835 .30983 .67758 .30652 .66858 .29885 Metetra
.522 .803 .997 r
.66858 .29885 .67758 .30652 .69698 .30295 .68809 .2952 Metetra
.532 .807 .996 r
.68809 .2952 .69698 .30295 .71653 .29914 .70775 .29128 Metetra
.542 .811 .996 r
.70775 .29128 .71653 .29914 .73623 .29508 .72756 .28708 Metetra
.552 .813 .994 r
.72756 .28708 .73623 .29508 .75609 .29079 .74753 .28261 Metetra
.593 .62 .848 r
.07934 .31543 .08955 .33117 .10387 .32855 .09372 .31236 Metetra
.584 .611 .845 r
.09372 .31236 .10387 .32855 .11826 .32618 .10817 .30959 Metetra
.576 .603 .842 r
.10817 .30959 .11826 .32618 .13272 .32408 .12268 .30713 Metetra
.567 .594 .84 r
.12268 .30713 .13272 .32408 .14727 .32225 .13726 .305 Metetra
.558 .587 .839 r
.13726 .305 .14727 .32225 .16191 .32069 .15193 .3032 Metetra
.549 .581 .838 r
.15193 .3032 .16191 .32069 .17664 .31939 .16667 .30174 Metetra
.54 .575 .839 r
.16667 .30174 .17664 .31939 .19148 .31835 .18151 .30063 Metetra
.53 .57 .84 r
.18151 .30063 .19148 .31835 .20641 .31756 .19644 .29984 Metetra
.521 .567 .843 r
.19644 .29984 .20641 .31756 .22147 .31701 .21147 .29938 Metetra
.511 .564 .846 r
.21147 .29938 .22147 .31701 .23664 .31668 .22662 .29923 Metetra
.502 .563 .851 r
.22662 .29923 .23664 .31668 .25193 .31654 .24189 .29937 Metetra
.492 .564 .856 r
.24189 .29937 .25193 .31654 .26737 .31658 .25729 .29976 Metetra
.483 .565 .863 r
.25729 .29976 .26737 .31658 .28294 .31676 .27283 .30039 Metetra
.473 .569 .871 r
.27283 .30039 .28294 .31676 .29866 .31706 .28852 .3012 Metetra
.464 .574 .88 r
.28852 .3012 .29866 .31706 .31453 .31745 .30436 .30217 Metetra
.455 .58 .89 r
.30436 .30217 .31453 .31745 .33057 .3179 .32036 .30325 Metetra
.447 .589 .901 r
.32036 .30325 .33057 .3179 .34677 .31837 .33654 .30439 Metetra
.438 .599 .912 r
.33654 .30439 .34677 .31837 .36313 .31883 .35289 .30555 Metetra
.43 .61 .924 r
.35289 .30555 .36313 .31883 .37967 .31924 .36942 .30668 Metetra
.423 .622 .935 r
.36942 .30668 .37967 .31924 .39639 .31957 .38614 .30774 Metetra
.416 .636 .946 r
.38614 .30774 .39639 .31957 .41329 .3198 .40305 .30867 Metetra
.411 .65 .956 r
.40305 .30867 .41329 .3198 .43036 .31988 .42015 .30944 Metetra
.407 .665 .966 r
.42015 .30944 .43036 .31988 .44762 .3198 .43745 .31001 Metetra
.404 .681 .974 r
.43745 .31001 .44762 .3198 .46506 .31952 .45493 .31033 Metetra
.403 .696 .98 r
.45493 .31033 .46506 .31952 .48269 .31902 .47261 .31039 Metetra
.403 .711 .985 r
.47261 .31039 .48269 .31902 .50049 .31828 .49049 .31014 Metetra
.406 .725 .989 r
.49049 .31014 .50049 .31828 .51847 .31728 .50855 .30957 Metetra
.411 .739 .992 r
.50855 .30957 .51847 .31728 .53664 .31601 .5268 .30865 Metetra
.418 .752 .995 r
.5268 .30865 .53664 .31601 .55498 .31445 .54523 .30738 Metetra
.427 .765 .996 r
.54523 .30738 .55498 .31445 .57349 .31261 .56384 .30574 Metetra
.437 .776 .997 r
.56384 .30574 .57349 .31261 .59218 .31046 .58263 .30373 Metetra
.449 .787 .998 r
.58263 .30373 .59218 .31046 .61103 .30801 .60159 .30135 Metetra
.463 .796 .999 r
.60159 .30135 .61103 .30801 .63005 .30525 .62072 .2986 Metetra
.477 .805 1 r
.62072 .2986 .63005 .30525 .64924 .3022 .64002 .29548 Metetra
.491 .812 1 r
.64002 .29548 .64924 .3022 .66858 .29885 .65948 .29202 Metetra
.506 .818 1 r
.65948 .29202 .66858 .29885 .68809 .2952 .6791 .28821 Metetra
.52 .822 1 r
.6791 .28821 .68809 .2952 .70775 .29128 .69887 .28408 Metetra
.534 .826 .999 r
.69887 .28408 .70775 .29128 .72756 .28708 .7188 .27964 Metetra
.547 .828 .998 r
.7188 .27964 .72756 .28708 .74753 .28261 .73888 .27491 Metetra
.607 .625 .843 r
.06908 .29946 .07934 .31543 .09372 .31236 .08355 .29572 Metetra
.598 .612 .837 r
.08355 .29572 .09372 .31236 .10817 .30959 .09809 .2923 Metetra
.589 .6 .832 r
.09809 .2923 .10817 .30959 .12268 .30713 .11268 .28923 Metetra
.579 .589 .827 r
.11268 .28923 .12268 .30713 .13726 .305 .12733 .28653 Metetra
.568 .578 .823 r
.12733 .28653 .13726 .305 .15193 .3032 .14204 .28425 Metetra
.558 .567 .82 r
.14204 .28425 .15193 .3032 .16667 .30174 .15682 .28239 Metetra
.547 .558 .818 r
.15682 .28239 .16667 .30174 .18151 .30063 .17168 .28098 Metetra
.535 .55 .817 r
.17168 .28098 .18151 .30063 .19644 .29984 .18661 .28003 Metetra
.523 .543 .817 r
.18661 .28003 .19644 .29984 .21147 .29938 .20163 .27954 Metetra
.512 .537 .819 r
.20163 .27954 .21147 .29938 .22662 .29923 .21675 .2795 Metetra
.499 .533 .823 r
.21675 .2795 .22662 .29923 .24189 .29937 .23198 .2799 Metetra
.487 .531 .828 r
.23198 .2799 .24189 .29937 .25729 .29976 .24733 .28069 Metetra
.474 .53 .835 r
.24733 .28069 .25729 .29976 .27283 .30039 .26281 .28186 Metetra
.461 .532 .844 r
.26281 .28186 .27283 .30039 .28852 .3012 .27843 .28336 Metetra
.448 .536 .854 r
.27843 .28336 .28852 .3012 .30436 .30217 .29421 .28512 Metetra
.435 .542 .867 r
.29421 .28512 .30436 .30217 .32036 .30325 .31016 .2871 Metetra
.421 .551 .881 r
.31016 .2871 .32036 .30325 .33654 .30439 .32628 .28922 Metetra
.407 .562 .896 r
.32628 .28922 .33654 .30439 .35289 .30555 .34258 .29141 Metetra
.392 .575 .912 r
.34258 .29141 .35289 .30555 .36942 .30668 .35908 .29362 Metetra
.378 .591 .928 r
.35908 .29362 .36942 .30668 .38614 .30774 .37578 .29575 Metetra
.364 .608 .943 r
.37578 .29575 .38614 .30774 .40305 .30867 .39268 .29775 Metetra
.351 .626 .956 r
.39268 .29775 .40305 .30867 .42015 .30944 .40979 .29954 Metetra
.34 .645 .966 r
.40979 .29954 .42015 .30944 .43745 .31001 .42711 .30107 Metetra
.33 .664 .974 r
.42711 .30107 .43745 .31001 .45493 .31033 .44463 .30228 Metetra
.324 .683 .978 r
.44463 .30228 .45493 .31033 .47261 .31039 .46237 .30311 Metetra
.321 .701 .98 r
.46237 .30311 .47261 .31039 .49049 .31014 .48031 .30354 Metetra
.322 .718 .981 r
.48031 .30354 .49049 .31014 .50855 .30957 .49846 .30352 Metetra
.328 .734 .98 r
.49846 .30352 .50855 .30957 .5268 .30865 .5168 .30304 Metetra
.338 .75 .98 r
.5168 .30304 .5268 .30865 .54523 .30738 .53534 .30208 Metetra
.353 .766 .98 r
.53534 .30208 .54523 .30738 .56384 .30574 .55406 .30063 Metetra
.371 .781 .981 r
.55406 .30063 .56384 .30574 .58263 .30373 .57296 .2987 Metetra
.392 .795 .984 r
.57296 .2987 .58263 .30373 .60159 .30135 .59204 .29629 Metetra
.415 .808 .987 r
.59204 .29629 .60159 .30135 .62072 .2986 .61129 .29341 Metetra
.439 .819 .99 r
.61129 .29341 .62072 .2986 .64002 .29548 .63071 .29009 Metetra
.463 .829 .993 r
.63071 .29009 .64002 .29548 .65948 .29202 .65028 .28635 Metetra
.487 .836 .996 r
.65028 .28635 .65948 .29202 .6791 .28821 .67001 .2822 Metetra
.509 .841 .998 r
.67001 .2822 .6791 .28821 .69887 .28408 .6899 .27768 Metetra
.529 .844 .999 r
.6899 .27768 .69887 .28408 .7188 .27964 .70994 .27282 Metetra
.548 .845 .998 r
.70994 .27282 .7188 .27964 .73888 .27491 .73012 .26764 Metetra
.629 .644 .846 r
.05863 .28409 .06908 .29946 .08355 .29572 .07325 .2794 Metetra
.62 .628 .837 r
.07325 .2794 .08355 .29572 .09809 .2923 .08793 .27497 Metetra
.611 .611 .827 r
.08793 .27497 .09809 .2923 .11268 .28923 .10267 .27086 Metetra
.6 .594 .818 r
.10267 .27086 .11268 .28923 .12733 .28653 .11745 .26712 Metetra
.588 .578 .809 r
.11745 .26712 .12733 .28653 .14204 .28425 .13228 .26383 Metetra
.576 .561 .801 r
.13228 .26383 .14204 .28425 .15682 .28239 .14715 .26105 Metetra
.563 .546 .793 r
.14715 .26105 .15682 .28239 .17168 .28098 .16208 .25883 Metetra
.549 .531 .788 r
.16208 .25883 .17168 .28098 .18661 .28003 .17706 .25724 Metetra
.535 .518 .784 r
.17706 .25724 .18661 .28003 .20163 .27954 .19209 .2563 Metetra
.52 .506 .782 r
.19209 .2563 .20163 .27954 .21675 .2795 .2072 .25604 Metetra
.504 .496 .782 r
.2072 .25604 .21675 .2795 .23198 .2799 .22239 .25648 Metetra
.487 .489 .784 r
.22239 .25648 .23198 .2799 .24733 .28069 .23767 .25761 Metetra
.47 .483 .789 r
.23767 .25761 .24733 .28069 .26281 .28186 .25306 .25939 Metetra
.452 .48 .797 r
.25306 .25939 .26281 .28186 .27843 .28336 .26858 .26178 Metetra
.432 .48 .809 r
.26858 .26178 .27843 .28336 .29421 .28512 .28424 .26471 Metetra
.411 .483 .824 r
.28424 .26471 .29421 .28512 .31016 .2871 .30006 .26808 Metetra
.388 .49 .842 r
.30006 .26808 .31016 .2871 .32628 .28922 .31606 .27179 Metetra
.362 .501 .863 r
.31606 .27179 .32628 .28922 .34258 .29141 .33226 .27571 Metetra
.333 .516 .886 r
.33226 .27571 .34258 .29141 .35908 .29362 .34866 .27973 Metetra
.301 .534 .91 r
.34866 .27973 .35908 .29362 .37578 .29575 .36528 .28371 Metetra
.266 .554 .929 r
.36528 .28371 .37578 .29575 .39268 .29775 .38214 .28752 Metetra
.229 .575 .942 r
.38214 .28752 .39268 .29775 .40979 .29954 .39922 .29104 Metetra
.193 .595 .944 r
.39922 .29104 .40979 .29954 .42711 .30107 .41655 .29415 Metetra
.162 .611 .935 r
.41655 .29415 .42711 .30107 .44463 .30228 .43411 .29676 Metetra
.138 .625 .917 r
.43411 .29676 .44463 .30228 .46237 .30311 .4519 .29878 Metetra
.126 .638 .895 r
.4519 .29878 .46237 .30311 .48031 .30354 .46992 .30015 Metetra
.127 .653 .877 r
.46992 .30015 .48031 .30354 .49846 .30352 .48817 .30084 Metetra
.14 .671 .865 r
.48817 .30084 .49846 .30352 .5168 .30304 .50662 .30082 Metetra
.164 .692 .863 r
.50662 .30082 .5168 .30304 .53534 .30208 .52528 .30007 Metetra
.198 .718 .869 r
.52528 .30007 .53534 .30208 .55406 .30063 .54413 .29863 Metetra
.24 .747 .884 r
.54413 .29863 .55406 .30063 .57296 .2987 .56316 .2965 Metetra
.287 .776 .904 r
.56316 .2965 .57296 .2987 .59204 .29629 .58237 .29372 Metetra
.336 .803 .926 r
.58237 .29372 .59204 .29629 .61129 .29341 .60175 .29035 Metetra
.386 .827 .948 r
.60175 .29035 .61129 .29341 .63071 .29009 .62129 .28643 Metetra
.433 .845 .967 r
.62129 .28643 .63071 .29009 .65028 .28635 .64098 .282 Metetra
.475 .857 .981 r
.64098 .282 .65028 .28635 .67001 .2822 .66082 .27713 Metetra
.511 .863 .99 r
.66082 .27713 .67001 .2822 .6899 .27768 .68081 .27187 Metetra
.541 .864 .995 r
.68081 .27187 .6899 .27768 .70994 .27282 .70095 .26626 Metetra
.566 .861 .996 r
.70095 .26626 .70994 .27282 .73012 .26764 .72123 .26034 Metetra
.655 .692 .873 r
.04772 .27109 .05863 .28409 .07325 .2794 .06251 .26546 Metetra
.651 .677 .861 r
.06251 .26546 .07325 .2794 .08793 .27497 .07738 .25987 Metetra
.645 .658 .848 r
.07738 .25987 .08793 .27497 .10267 .27086 .09231 .25438 Metetra
.637 .636 .833 r
.09231 .25438 .10267 .27086 .11745 .26712 .10732 .24903 Metetra
.627 .612 .816 r
.10732 .24903 .11745 .26712 .13228 .26383 .12239 .24391 Metetra
.615 .587 .799 r
.12239 .24391 .13228 .26383 .14715 .26105 .1375 .23913 Metetra
.601 .561 .782 r
.1375 .23913 .14715 .26105 .16208 .25883 .15266 .23482 Metetra
.586 .535 .765 r
.15266 .23482 .16208 .25883 .17706 .25724 .16785 .23112 Metetra
.569 .51 .75 r
.16785 .23112 .17706 .25724 .19209 .2563 .18306 .22822 Metetra
.55 .487 .737 r
.18306 .22822 .19209 .2563 .2072 .25604 .19829 .22628 Metetra
.53 .465 .727 r
.19829 .22628 .2072 .25604 .22239 .25648 .21354 .22546 Metetra
.508 .445 .72 r
.21354 .22546 .22239 .25648 .23767 .25761 .22881 .22591 Metetra
.485 .427 .717 r
.22881 .22591 .23767 .25761 .25306 .25939 .24411 .22773 Metetra
.459 .412 .718 r
.24411 .22773 .25306 .25939 .26858 .26178 .25948 .23094 Metetra
.429 .399 .724 r
.25948 .23094 .26858 .26178 .28424 .26471 .27492 .23552 Metetra
.395 .389 .735 r
.27492 .23552 .28424 .26471 .30006 .26808 .29048 .24134 Metetra
.353 .384 .754 r
.29048 .24134 .30006 .26808 .31606 .27179 .3062 .24821 Metetra
.3 .383 .781 r
.3062 .24821 .31606 .27179 .33226 .27571 .32211 .25585 Metetra
.228 .386 .814 r
.32211 .25585 .33226 .27571 .34866 .27973 .33824 .26395 Metetra
.128 .393 .845 r
.33824 .26395 .34866 .27973 .36528 .28371 .35464 .27214 Metetra
0 .393 .847 r
.35464 .27214 .36528 .28371 .38214 .28752 .37132 .28006 Metetra
0 .372 .778 r
.37132 .28006 .38214 .28752 .39922 .29104 .38829 .28735 Metetra
.259 0 0 r
.38829 .28735 .39922 .29104 .41655 .29415 .40558 .29372 Metetra
.311 0 0 r
.40558 .29372 .41655 .29415 .43411 .29676 .42316 .29894 Metetra
.317 0 0 r
.42316 .29894 .43411 .29676 .4519 .29878 .44103 .30283 Metetra
.296 0 0 r
.44103 .30283 .4519 .29878 .46992 .30015 .45917 .30533 Metetra
.258 0 0 r
.45917 .30533 .46992 .30015 .48817 .30084 .47756 .30642 Metetra
.204 0 0 r
.47756 .30642 .48817 .30084 .50662 .30082 .49618 .30616 Metetra
.135 0 0 r
.49618 .30616 .50662 .30082 .52528 .30007 .515 .30466 Metetra
.049 0 0 r
.515 .30466 .52528 .30007 .54413 .29863 .53402 .30206 Metetra
.53402 .30206 .54413 .29863 .56316 .2965 .55321 .29849 Metetra
.168 .701 .706 r
.55321 .29849 .56316 .2965 .58237 .29372 .57256 .29413 Metetra
.285 .786 .812 r
.57256 .29413 .58237 .29372 .60175 .29035 .59207 .28911 Metetra
.39 .846 .897 r
.59207 .28911 .60175 .29035 .62129 .28643 .61171 .28357 Metetra
.474 .878 .953 r
.61171 .28357 .62129 .28643 .64098 .282 .63151 .27762 Metetra
.535 .886 .982 r
.63151 .27762 .64098 .282 .66082 .27713 .65144 .27135 Metetra
.575 .88 .993 r
.65144 .27135 .66082 .27713 .68081 .27187 .67151 .26483 Metetra
.601 .867 .993 r
.67151 .26483 .68081 .27187 .70095 .26626 .69172 .25812 Metetra
.617 .851 .987 r
.69172 .25812 .70095 .26626 .72123 .26034 .71208 .25127 Metetra
.663 .739 .906 r
.03634 .26033 .04772 .27109 .06251 .26546 .05119 .25453 Metetra
.663 .736 .904 r
.05119 .25453 .06251 .26546 .07738 .25987 .06613 .2487 Metetra
.663 .733 .902 r
.06613 .2487 .07738 .25987 .09231 .25438 .08116 .24283 Metetra
.662 .726 .897 r
.08116 .24283 .09231 .25438 .10732 .24903 .09628 .23692 Metetra
.661 .717 .89 r
.09628 .23692 .10732 .24903 .12239 .24391 .11148 .23098 Metetra
.659 .703 .879 r
.11148 .23098 .12239 .24391 .1375 .23913 .12679 .225 Metetra
.655 .683 .864 r
.12679 .225 .1375 .23913 .15266 .23482 .14218 .21899 Metetra
.65 .656 .843 r
.14218 .21899 .15266 .23482 .16785 .23112 .15767 .21294 Metetra
.642 .623 .816 r
.15767 .21294 .16785 .23112 .18306 .22822 .17325 .20686 Metetra
.632 .585 .784 r
.17325 .20686 .18306 .22822 .19829 .22628 .18892 .20073 Metetra
.619 .545 .749 r
.18892 .20073 .19829 .22628 .21354 .22546 .20469 .19457 Metetra
.605 .505 .714 r
.20469 .19457 .21354 .22546 .22881 .22591 .22056 .18838 Metetra
.591 .468 .681 r
.22056 .18838 .22881 .22591 .24411 .22773 .23652 .18214 Metetra
.578 .436 .651 r
.23652 .18214 .24411 .22773 .25948 .23094 .25259 .17587 Metetra
.567 .408 .626 r
.25259 .17587 .25948 .23094 .27492 .23552 .26875 .16955 Metetra
.557 .386 .605 r
.26875 .16955 .27492 .23552 .29048 .24134 .28501 .1632 Metetra
.549 .367 .587 r
.28501 .1632 .29048 .24134 .3062 .24821 .30138 .15681 Metetra
.542 .353 .574 r
.30138 .15681 .3062 .24821 .32211 .25585 .31784 .15039 Metetra
.426 .257 .547 r
.31784 .15039 .32211 .25585 .33824 .26395 .33304 .16536 Metetra
.819 .62 .066 r
.33304 .16536 .33824 .26395 .35464 .27214 .33908 .34194 Metetra
.577 .26 .394 r
.33908 .34194 .35464 .27214 .37132 .28006 .35568 .35999 Metetra
.439 .144 .366 r
.35568 .35999 .37132 .28006 .38829 .28735 .37384 .35368 Metetra
.426 .12 .34 r
.37384 .35368 .38829 .28735 .40558 .29372 .39212 .34731 Metetra
.408 .086 .305 r
.39212 .34731 .40558 .29372 .42316 .29894 .41053 .34089 Metetra
.38 .038 .254 r
.41053 .34089 .42316 .29894 .44103 .30283 .42907 .33443 Metetra
.339 0 .179 r
.42907 .33443 .44103 .30283 .45917 .30533 .44773 .32793 Metetra
.272 0 .064 r
.44773 .32793 .45917 .30533 .47756 .30642 .46652 .32138 Metetra
.162 0 0 r
.46652 .32138 .47756 .30642 .49618 .30616 .48544 .31479 Metetra
.48544 .31479 .49618 .30616 .515 .30466 .5045 .30815 Metetra
.242 .743 .679 r
.5045 .30815 .515 .30466 .53402 .30206 .52368 .30146 Metetra
.439 .875 .892 r
.52368 .30146 .53402 .30206 .55321 .29849 .54301 .29473 Metetra
.552 .9 .976 r
.54301 .29473 .55321 .29849 .57256 .29413 .56246 .28794 Metetra
.603 .879 .992 r
.56246 .28794 .57256 .29413 .59207 .28911 .58206 .28111 Metetra
.625 .851 .985 r
.58206 .28111 .59207 .28911 .61171 .28357 .6018 .27424 Metetra
.635 .827 .974 r
.6018 .27424 .61171 .28357 .63151 .27762 .62167 .26731 Metetra
.64 .81 .965 r
.62167 .26731 .63151 .27762 .65144 .27135 .64169 .26033 Metetra
.642 .798 .957 r
.64169 .26033 .65144 .27135 .67151 .26483 .66185 .25331 Metetra
.644 .79 .952 r
.66185 .25331 .67151 .26483 .69172 .25812 .68216 .24623 Metetra
.644 .784 .949 r
.68216 .24623 .69172 .25812 .71208 .25127 .70262 .2391 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(90)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(95)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(100)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(105)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(110)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{179.438, 145.25},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002c0000TA000`40O003h00O/goo002cOol00;=o
o`00/goo002cOol00;=oo`00/goo002cOol00;=oo`00/goo001]Ool30003Ool30004Ool2000gOol0
06ioo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000fOol006ioo`03001oogoo00=oo`03
001oogoo009oo`04001oogoo000fOol006ioo`03001oogoo00=oo`03001oogoo009oo`04001oogoo
000fOol006ioo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000fOol006eoo`8000Aoo`80
00Eoo`8003Moo`00/goo002cOol00;=oo`00Ngoo0`00=Goo001BOol30004Ool20003Ool3000GOol3
00000goo001oo`0eOol005=oo`03001oogoo009oo`04001oogoo0005Ool00`00Oomoo`0BOol20004
Ool2000eOol005=oo`03001oogoo009oo`04001oogoo0005Ool00`00Oomoo`0?Ool30006Ool01000
Ool00000<goo001COol00`00Oomoo`02Ool01000Oomoo`000Woo0`003goo0`002Goo00<007oo0000
=7oo001COol00`00Oomoo`02Ool01000Oomoo`000Woo00<007ooOol03Goo0P0037oo00@007ooOol0
03=oo`00DWoo0P001Goo0P000goo10002Goo0`003Woo00D007ooOomoo`00039oo`00JGoo0P004Goo
00@007ooOomoo`80035oo`00IWoo0`004goo00<007ooOol00Woo00<007ooOol0;goo001TOol2000F
Ool00`00Oomoo`03Ool00`00Oomoo`0^Ool0065oo`<001Uoo`03001oogoo009oo`03001oogoo00Eo
o`8000Aoo`8000=oo`@001Yoo`00>Goo0`0017oo0P0017oo0P005Woo0`000Woo00<007ooOol05goo
00<007ooOol00goo00<007ooOol00goo00@007ooOol000=oo`8000Aoo`03001oogoo01Yoo`00>Woo
00<007ooOol00Woo00@007ooOol0009oo`04001oogoo000COol2000OOol00`00Oomoo`04Ool00`00
Oomoo`02Ool01000Oomoo`002Woo00<007ooOol06Goo000jOol00`00Oomoo`02Ool01000Oomoo`00
0Woo00@007ooOol0011oo`<0025oo`03001oogoo00Aoo`03001oogoo009oo`04001oogoo000;Ool0
0`00Oomoo`0HOol003Yoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`003Woo0`008goo
00<007ooOol01Goo0P000Woo00@007ooOol000Qoo`04001oogoo000JOol003Yoo`03001oogoo009o
o`04001oogoo0002Ool01000Oomoo`002goo0`009Woo00<007ooOol01Goo0P000goo0P002Woo0P00
6goo000iOol20005Ool20004Ool2000:Ool3000XOol00`00Oomoo`06Ool00`00Oomoo`0ZOol004mo
o`<002]oo`03001oogoo00Moo`03001oogoo02Uoo`00C7oo1000;Goo00<007ooOol027oo00<007oo
Ool0:7oo001:Ool2000aOol00`00Oomoo`08Ool00`00Oomoo`0XOol002Moo`<000=oo`<001Moo`@0
039oo`03001oogoo00Uoo`03001oogoo02Moo`00:Woo00<007ooOol00goo00<007ooOol04Woo0P00
=Woo00<007ooOol02Woo00<007ooOol09Woo000XOol30005Ool00`00Oomoo`0?Ool5000fOol00`00
Oomoo`0:Ool00`00Oomoo`0VOol002Moo`04001oogoo0002Ool3000@Ool300000dEb0015LP02000e
Ool00`00Oomoo`0;Ool00`00Oomoo`0UOol002Moo`04001oogoo0002Ool00`00Oomoo`0=Ool30000
2DFC0015LTEb0015LP00=A40000dOol00`00Oomoo`0<Ool00`00Oomoo`03Ool20004Ool20004Ool3
000BOol002Qoo`8000=oo`@000Yoo`<00004BK@004FC0002AG801@00AG8003DA0000=7oo00<007oo
Ool037oo00<007ooOol00Woo00@007ooOol000=oo`8000Eoo`03001oogoo015oo`00>7oo100000I9
]000BK@004FC0002AG801@00AG8003DA0000=7oo00<007ooOol03Goo00D007ooOomoo`00009oo`03
001oogoo00Ioo`@0019oo`00=Goo100000U9e@00BKA9]000BK@004FC00000TEb00D004Eb000e4@00
03Eoo`03001oogoo00eoo`04001oogoo0002Ool00`00Oomoo`06Ool00`00Ool0000COol003=oo`@0
0004CQH004WE0002BK@01@00BK@004FC00000TEb00D004Eb000e4@0003Eoo`03001oogoo00eoo`80
0005Ool007ooOol00008Ool00`00Ool0000COol0015oo`<000Aoo`8001Ioo`D00006CSL004hF0019
e@000TVd00D004Vd0015T`000095LP050015LP00=A40000eOol00`00Oomoo`0>Ool00`00Oomoo`02
000:Ool2000COol001Aoo`04001oogoo0002Ool00`00Oomoo`0AOol400001e9I001>=dhg001>5P00
0099e@0:0019]4Vd0019]000AI<004Eb0002AG82000eOol00`00Oomoo`0?Ool00`00Oomoo`0POol0
019oo`<0009oo`04001oogoo000@Ool500001E9j001BFE9I00000Thg00D004hF0019e@000099]007
0019]000AI<004Eb00000TEb0P00=Goo00<007ooOol03goo0P008Goo000AOol01000Oomoo`000Woo
00@007ooOol000ioo`D00008DYX0059jDWX0059I001>=`@0000;BMD004VdBK@004Vd0015T`00AG80
0002AG83000dOol00`00Oomoo`0@Ool00`00Oomoo`0OOol0015oo`04001oogoo0002Ool01000Oomo
o`002goo0`000UKL00@005:k001BVPL00003@ML0047G00P00004AI<004Eb0002AG83000dOol00`00
Oomoo`0@Ool00`00Oomoo`0OOol0019oo`8000Aoo`8000Uoo`@0009Fg003001Fg00000<00004BSP0
04XG0002AOL01`00@ML0047G000m]P00008i]P03000eU`0000800004AG8004EbAG83000cOol2000C
Ool2000OOol0025oo`@00004E_`005KLE]`2000014iI001>>@000TXh00D004XG0015m`000091e`80
008m]P06000i]P00=IL0036G0`000TEb0`00<7oo1P004goo00<007ooOol07Goo000NOol500000eKl
001Fo0030000159j001>F@000Thi00D004Xh001:5`000095m`070011e`00@ML003ff00000SVf0P00
00<aU`00;IP01P00;Goo1P0000AC?P000000019oo`03001oogoo00Aoo`8000Aoo`8000Aoo`8000]o
o`007Goo100000=Fo00000000P0000=BVP00DWX00U9j00<004iI00004P0000<eU`00<IL00P000RFH
0`00;7oo10000U<n1@004goo0P000goo00@007ooOol000=oo`8000=oo`04001oogoo000:Ool001Mo
o`8000Aoo`800005E_`00000D[/00002D[/01000DYX0059j100000e:>@00AQU66@00AQT0048I0011
n@00?OT00002?OT00`00>OT0000400001BfH000UV0007IX00P00:Goo0`000U<n0`0000=?O`000000
0`004goo00D007ooOomoo`00009oo`03001oogoo00Ioo`04001oogoo000:Ool001Ioo`04001oogoo
0003Ool01@00Ool005;K00000U:k00<005:k00000`0000A:F@00BST0009:>@0B00166DHI00166@00
@QT0047i000mn@00?OT003Wi000en@000SGj1P0000AZHP00000002=oo`<0009C?P<00005Bil00000
0017O`05000BOol01@00Oomoogoo00000Woo00<007ooOol01Woo0`002goo000FOol01000Oomoo`00
0goo00@007oo001Bf`D00004CWX004ij0002BUT01P00BST004Xi00166@d00009>OT003Gi000enP00
<OX002hK00800004JV8004T<B@`2000NOol30002Del300001DNO00000000@gl00`0000=3O`000000
10004Goo0P0000Eoo`00Oomoo`0000Qoo`03001oogoo00]oo`005Woo00@007ooOol0009oo`80009o
o`03001BV`00009>VP03001>NTij00X00006ASX0048j0012>P000Shj00@003hJ000n6PX00004;Q/0
06YR0002B@`00`00>8/00002000IOol20002Cgl;0002@el20002Ael5000@Ool20002Ool2000:Ool3
000:Ool001Ioo`04001oogoo0002Ool20003Ool01000DY/004jJ100000A:FP00BUX00096FP030016
>P000096>P050012>P00@SX00002?SX03000?QX003hJ000j>`00>S/003Hk000f?0@000=93003000h
R`00008dJP@0019oo`d00003>el0000000@00005@el000000017?`0200000dMO0000000@Ool00`00
Oomoo`0IOol001Moo`8000=oo`8000Aoo`<00009CWX004YjBWX004YJ001:FP000096FQD00003=S/0
03Hl00<000=93003000hR`00008dJP03000dB@0002400003?cl0000000@00003Acl0000000@00003
Acl00000011oo`8001Uoo`0077oo00<007ooOol00goo00<004ij00000Tij00<004Yj00001`0000A6
FP00@UX00092FP070012F`00@U/003iK00000SiK00@003iK000jG0L000993003000hR`0000<dJP03
000dB@00008`:1H0008k?`P00003@cl0000000@00003Acl0000000@00003Acl00000011oo`03001o
ogoo01Moo`0077oo0P001Goo1P0000I:NP00BWX004IJ0002AUX01@00AUX0049J00000T9J00L0049K
0012F`00?U/00002?U/02000?U/003YL000jG000>U`20002B@`00`00>8/00002=6X01000=4TdB@00
0S0X00D002P5:0D0020200T0008g7`P00004?al003lo?cl50002@cl50002Acl00`00Acm7?`020002
Bcl00`00Oomoo`0=Ool00`00Oomoo`0GOol000Aoo`8000Aoo`8000=oo`@0009oo`<000Aoo`03001o
ogoo00Aoo`06001:NdYk001:N`000TYj00<004Yj00007@000TT<00<003R;00000SAZ00H003A9=4T0
030X0002:0D>000013[o000k7c/O200000=37`000000100000A77`00Acm7?`H00003Acl0000000<0
00moo`03001oogoo01Ioo`000goo00@007ooOol000=oo`8000Aoo`03001oogoo00Eoo`05001oogoo
Ool00007Ool00`00BW/0000700002DIk0016NdIk0016F`00@U/00002@U/02@00@U/0049K000nF`00
?U`00002?W`01000?W`003il0P000dT<00<003R;00000SAZ00<003A900000S0X1P000S[O3@0000@o
7`00?alo7`D000937`0300137`0000D00004Aal004LOAal50002Bal2000?Ool00`00Oomoo`04Ool2
0004Ool20004Ool20003Ool000=oo`04001oogoo000:Ool00`00Oomoo`04Ool00`00Oomoo`020008
Ool01@00BW/004Yk00000TYk00H004Ik0016N`00AW/H0003B@`00`00>8/00002=6X00`00=4T0000:
00001SkO000ngckO000no`/00003@al0000000H000977`0300177`0000D00003Aal0000000@00003
Bal007oo00ioo`03001oogoo00=oo`04001oogoo0003Ool20003Ool01000Oomoo`000Woo0003Ool0
1000Oomoo`002goo00<007ooOol00`000goo00<007ooOol01goo00<004YkBW/02P0000A6N`00AW/0
0092N`070012N`00@W`0049l00000T9l00D0049l000nO000008nO08000=93080008dJP03000dB@00
0140008no`030012o`000092o`D00004@_l004<O@al80002Aal5000014/O001;7d/O1@003goo00<0
07ooOol00Woo00@007ooOol000Qoo`04001oogoo0002Ool000=oo`04001oogoo0008Ool01000Oomo
o`000Woo00<007ooOol00goo0P002Goo0`0000U:N`00BW]:N`00AW/004Ik00000TIk00@004Ik0012
Na<00005B@`003R;000dJP0600001cjn000n_Sjn000ngP00008ngP03000ngP0000X00004@_l004;o
@_l5000014LO00177dLO1P0000A77`00Aam77`L0009;7`8000moo`05001oogooOol00002Ool00`00
Oomoo`07Ool20003Ool000Aoo`8000Yoo`8000=oo`@0009oo`03001oogoo00Uoo`06001:V`00BW/0
04Yk2`0000A6O000@W`00092O0070012O000@W`004:L00000T:M00@004:M000nWAH00092gP030012
gP0000X00004A_l004KoA_l5000014LO00177dLO1P0000A;7`00Bam;7`D00003Bah0000000ioo`05
001oogooOol00002Ool00`00Oomoo`06Ool01000Oomoo`000Woo000KOol00`00Oomoo`0:Ool50002
AW/02@00AW/004Ik0016O000AW`00002AW`F0002@[d01@00@[d004:n00000T:n00<004;N00002@00
0T;N00<004;n00000T;n200000A6o`00A_m6o`D00004Aal004/OBal600000d/N0000000300000d/N
001oo`0=Ool20003Ool2000:Ool20003Ool001]oo`8000]oo`05001:V`00BY/00002AW/=00001T:L
0012W000@Y`00092W0060012W@00@Yd004:M0P0000=2W@00000030000T;N00<004;N00000T;N2P00
00A6oP00A_i6oPP00004A_h004LNAah5000014/N001;7T/N0`0000=;7P00Bah01P003Woo00<007oo
Ool04Goo0009Ool20004Ool20003Ool30004Ool00`00Oomoo`0;Ool6000024JL0016W000AY`004JL
0002AY`00`00AY`0000A00002D:m0012_D:m0012_@00@[h00002@[h:000014;N0012gP000T;n2000
00A6oP00A_i6oPP00003A_h0000000H00003Bah0000000@00004Bah004/NBah2000=Ool2000BOol0
00Qoo`04001oogoo0003Ool20006Ool00`00Oomoo`02000>Ool01P00BY`004ZL0016W0/00004AY`0
04JL0002@Y`00`00@Yd000040002@Yd>000024:n0012gT;N0012gP00@]h800001TKn0016oTKn0016
oPH00004A_h004KnA_h5000014[n001;7T/N1@0000A;7P00Bai;7PH00003Bah007oo00aoo`80019o
o`0027oo00@007ooOol000]oo`03001oogoo008000moo`D0000<AY`004JLAY`004JL0016W000AY`0
04JL3P0000A2_@00@[d00092_@040012_@00@[d<0002A]h00`00A]h000090002A_h00`00A_h00005
00000dKn001:oP06000014[n001;7T/N1@0000=;7P00Bah01P003Woo00<007ooOol03goo0008Ool0
1000Oomoo`0027oo0`000goo00<007ooOol03Woo00H004ZL001:W000AY`;0002AY`300001TJM0016
W@00@Yd00092_@/00092_@030012gP000092gP030012gP0000H00096gP030016gP000096oPL00096
oP030016oP0000@0009:oP03001:oP0000D00003Bah004/N00H00003Bah0000000<000eoo`80011o
o`0027oo00@007ooOol000Qoo`03001oogoo00=oo`03001oogoo00moo`D00096W0080016W000AY`0
04JL0016W0h00004A[d004Jm0002A[d00`00@[d0000;00001TKN0016gTKN0016gPP00004A_h004Kn
A_h7000014[n001:oT[n1@0000A:oP00B_i:oPD0009;7P03001;7P0000800004Bah004/NBah2000=
Ool00`00Oomoo`03Ool30008Ool000Uoo`8000Uoo`@0009oo`03001oogoo011oo`06001:W000BY`0
04JL2@0000Q6W000AYd004JM0016_@000TJm2`0000M6_@00A[e6_@00A]d00002A]h90002A]h00`00
A]h0000700000dKn0016oP08000014[n001:oT[n1@0000A:oP00Bai;7PD00003Bah0000000@00003
Bah007oo00eoo`03001oogoo00=oo`03001oogoo00Moo`006Woo00<007ooOol04Goo1@0000=6W000
AY`0100000=6W00000002@0000A6_@00A[d00096_@030016_@0000/00004A]h004KN0002A]h70000
1TKN0016oTKn0016oPD0009:oP03001:oP0000L00004B_h004[nB_h500000d[n000000030002Bah7
000=Ool00`00Oomoo`03Ool00`00Oomoo`07Ool001Uoo`8001=oo`04001:W000BY`900001TJl0016
_@00A[d00096_@030016_@0000P00008A[d004JmA[d004Jm0016g@X00003A]h004KN00X00096oP03
0016oP0000@00004B_h004[nB_h800000d[n00000003000014[n001;7T/N1@0000A;7P00Cai?7P80
00eoo`03001oogoo009oo`03001oogoo00Moo`000Woo0P0017oo0P0017oo00<007ooOol00Woo0`00
0goo00<007ooOol04goo100000M:_000A[`004Jl0016_00;000014Jm0016_@000TJm00<004Jm0000
20000TKM00@004KM0016gPL00096gP030016gP000096gPL00004B_h004[nB_h5000014[n001:oT[n
1@0000A:oP00B_i:oPD00004Bah004/NBah500000dlN0000000;Ool30003Ool00`00Oomoo`07Ool0
0005Ool007ooOol00003Ool20004Ool00`00Oomoo`05Ool00`00Oomoo`02000BOol400000dZl0000
0006000024Jm0016_DJm0016_@00A[d;00001TJm0016_DJm0016_@T00096gP040016gP00A]h80000
0dKn0016oP08000014[n001:oT[n1@0000=:oP00B_h01P0000=:oP00B_h01P0000=;7P0000000`00
00=?7P00Ool01goo0`000Woo00<007ooOol00P002Goo00001Goo001oogoo00002Woo00<007ooOol0
17oo00@007ooOol0011oo`<000Aoo`08001:_000B[`004Jl0016_0T00004A[d004Jm0002A[d00`00
A[d0000800001TKM0016gDKM0016g@T00096gP030016gP0000H0009:oP03001:oP0000L00003B_h0
04[n00D0009:oP03001:oP0000@00004B_h004/NBah500000dlN000000030004Ool30005Ool00`00
Oomoo`0;Ool00005Ool007ooOol0000:Ool01000Oomoogoo0`000goo00<007ooOol037oo0P0027oo
200000E6_@00A[d004Jm00/00004A[d004Jm0002A[d900001TKM0016gTKN0016gPL00004A]h004[N
B]h7000014[n001:oT[n1`0000A:oP00B_i:oPD00004B_h004[nB_h500000d[n00000003000014lN
001?7TlN1@0027oo00<007ooOol02goo00001Goo001oogoo00002goo00@007ooOol000Eoo`03001o
ogoo00Uoo`<000Yoo`05001:_4Zl001:_008000024Jm0016_DJm0016_@00A[d9000014KM0016g@00
0TKM2@0000A6gP00A]i6gPL00004B_h004[n0002B_h500000d[n001:oP0800000d[n000000060000
0d[n00000003000014kn001?7TlN1@0000=?7P0000002Woo00<007ooOol02goo0002Ool20009Ool4
0002Ool400000goo00000008Ool3000>Ool400001dZm001:_@00B[d004Jm00T00004A[d004Jm0002
A[d900001DKM0016g@00A]d0200000A:gP00B]h0009:gPL00003B_h004[n00D0009:oP03001:oP00
00@00004B_h004[nB_h5000014[n001:oT[n1@0000=?7P0000000`0000=?7P00Ool02Woo00<007oo
Ool02Woo000HOol20005Ool3000BOol00`00B[`0000800001DZm0016_@00A[d02@0000E6g@00A]d0
04KM00X00005A]d004KN001:gP0800000d[n001:oP070002B_h00`00B_h00004000014[n001:oT[n
1@0000=:oP00B_h01P0000=>oP00C_h01@000TlN10002Woo00<007ooOol02Woo000HOol01000Oomo
ogoo0`005Woo00H004Zm001:_@00B[d9000014Jm0016_@000TJm20000TKM00@004KM0016g@T0009:
gP03001:gP0000H00004B_h004[nB_h700000d[n0000000600000d[n001:oP050002B_h00`00B_h0
000400000dkn0000000400000dlN001?7P020009Ool00`00Oomoo`0:Ool0015oo`<000Aoo`@001Yo
o`H00005B[d004Zm001:_@0900001DZm0016g@00A]d02@000T[M00@004[M001:g@L00004B]h004[N
B]h700000d[n001:oP05000014[n001:oT[n1`000T[n1`0000=>oP0000000`0000A>oP00Cai?7PD0
0003Cah0000000Qoo`03001oogoo00Yoo`004Woo00<007ooOol00goo0P007Goo00<004Zm00001`00
00A:_@00B[d0009:_@P0009:g@04001:g@00B]d9000014[M001:gT[N1`0000=:oP00B_h01`000T[n
00<004[n0000100000A:oP00B_i:oPD00003B_h004[n00D00004C_h004knC_h500000dlN00000003
00000dlN001oo`07Ool00`00Oomoo`0:Ool0019oo`03001oogoo00Eoo`03001oogoo01]oo`06001:
_@00B[d004Zm2@0000E:g@00B]d004[M00T00006B]d004[MB]d004[M1`0000=:gP00B]h01`0000A:
oP00B_i:oPL00004B_h004[nB_h500000d[n001:oP05000014kn001>oTkn1@0000=>oP0000000`00
00A?7P00Cai?7P@000Moo`03001oogoo00Yoo`004Woo00<007ooOol01Woo00<007ooOol06goo1P00
00E:_@00B[d004Zm00P0009:g@04001:g@00B]d900000d[M001:g@07000014[N001:gT[N1`0000=:
oP00B_h0200000=:oP0000001@0000=:oP0000001P0000=>oP0000000`0000=>oP00C_h01P0000=?
7P00Cah00P001Woo00<007ooOol02Woo000BOol00`00Oomoo`07Ool00`00Oomoo`0JOol00`00B[d0
000800001D[M001:g@00B]d02@0000=:g@00B]d02@000T[M00<004[M00001P0000=:oP00B_h01`00
0T[n00<004[n0000100000=:oP00B_h01@0000A:oP00C_i>oPD00003C_h004kn00D00004C_h004kn
C_h500000dlN00000005Ool00`00Oomoo`0:Ool0015oo`8000Yoo`8001]oo`800005B[d004Zm001:
_@0900001D[M001:g@00B]d01`0000E:g@00B]d004[M00L00003B]d004[N00L0009:oP03001:oP00
00H00003B_h004[n00D00004B_h004[nB_h500000dkn001>oP0500000dkn0000000600000dkn0000
000300000dlN001oo`05Ool00`00Oomoo`09Ool001moo`03001oogoo01Yoo`D00005B]d004[M001:
g@0900000d[M001:g@09000014[M001:gD[M1P000T[N00<004[n00001P0000=:oP00B_h01`000T[n
1`0000=:oP0000001@0000=>oP0000000`0000A>oP00C_i>oPD00003C_h004lN00D000Eoo`03001o
ogoo00Uoo`0087oo00<007ooOol06Goo00<004Zm0000200000E:g@00B]d004[M00H0009:g@04001:
g@00B]d700000d[M001:g@0700000d[n001:oP070002B_h00`00B_h0000400000d[n001:oP050000
0dkn001>oP05000014kn001>oTkn1@0000=>oP00C_h01@0000=?7P0000000`0017oo00<007ooOol0
2Goo000QOol00`00Oomoo`0IOol00`00B]d00002B]d00`00B]d0000700000d[M001:g@0900000d[M
001:g@07000014[M001:gD[M1P000T[n00<004[n00001P0000=:oP00B_h01@0000A:oP00C_i>oP@0
0004C_h004knC_h500000dkn0000000500000dkn00000003000014lN001?7TlN0P000goo00<007oo
Ool02Goo000ROol00`00Oomoo`0IOol600000d[M001:g@0900000d[M001:g@080002B]d00`00B]d0
000600000d[M001:oP0700000d[n001:oP0700000d[n0000000600000dkn0000000500000dkn0000
000300000dkn001>oP05000014kn001>oTkn1@0000=?7P0000000Woo00<007ooOol02Goo000SOol2
000KOol700001D[M001:g@00B]d01`0000E:g@00B]d004[M00L00003B]d004[M00L00003B_h004[n
00L00004B_h004[nB_h40002B_h00`00B_h0000400000dkn001>oP0500000dkn001>oP0500000dkn
0000000600000dkn0000000300001DlN000007oo00002goo000UOol00`00Oomoo`0HOol01000B]d0
04[M2@0000=:g@00B]d020000T[M00<004[M00001@0000A:g@00B]e:g@H0009:oP03001:oP0000H0
0003B_h0000000D00003C_h0000000D00003C_h0000000D00003C_h0000000<00004C_h004knC_h5
00000dkn00000003000014lN001oo`002goo000VOol00`00Oomoo`0HOol300000d[M001:g@090000
0d[M001:g@0900000d[M001:g@0700000d[m001:o@0700000d[n0000000500000d[n001:oP050000
0dkn001>oP05000014kn001>oTkn100000A>oP00C_i>oPD00003C_h0000000D00003Cah0000000@0
00]oo`009goo00<007ooOol067oo1P0000=:g@00B]d0200000=:g@00B]d02@0000=:g@00B]d01`00
00=:o@0000001@0000=:oP00B_h01`0000=>oP0000001@0000A>oP00C_i>oPD00003C_h0000000D0
0003C_h0000000<00003C_h004kn00D00003Cah0000000<000]oo`00:7oo0P006Goo00<004[M0000
1@0000E:g@00B]d004[M00L00003B]d004[M00P0009:g@H00004B_d004[mB_d60002B_h00`00B_h0
0003000014kn001>oTkn1`0000=>oP0000000`0000=>oP00C_h01@0000=>oP00C_h01@0000=>oP00
00000`0000=?7P00Cah00`002Woo000ZOol00`00Oomoo`0GOol01000B]d004[M200000A:g@00B]e:
g@H00003B]d004[M00L00003B]d004[m00L00003B_d004[n00L00003C_h0000000D00003C_h00000
00D00003C_h0000000D00003C_h0000000D00003C_h0000000<00003C_h0000000D00003Cah00000
00Yoo`00:goo00<007ooOol05goo0`0000=:g@00B]d02@0000=:g@00B]d01P0000A:g@00B]e:g@H0
0003B_d004[m00L00003B_h0000000D00003C_h004kn00D00003C_h004kn00D00003C_h004kn00D0
0003C_h004kn00D00003C_h0000000<00003C_h004kn00D00003Ool007oo00Uoo`00;7oo00<007oo
Ool05goo1P0000=:g@00B]d0200000=:g@0000001`0000=:g@00B]d01P0000=:o@0000001@0000=:
oP00C_h01`0000=>oP0000001@0000=>oP0000001@0000=>oP0000001@0000=>oP0000000`0000=>
oP00C_h01@0000=oo`00Ool00goo00<007ooOol027oo000]Ool2000IOol700000d[M001:g@060000
0d[M001:g@0900000d[m000000040002B_d00`00B_d000050002C_h00`00C_h00003000014kn001>
oTkn100000A>oP00C_i>oP@00004C_h004knC_h400000dkn000000050007Ool01@00Oomoogoo0000
2Woo000_Ool00`00Oomoo`0FOol00`00B]d0000700000d[M001:g@060002B]d00`00B]d000050000
0d[m001:o@0700000d[m001:o@0700000dkn0000000500000dkn0000000500000dkn0000000:000>
Ool01@00Oomoogoo00002Woo000`Ool00`00Oomoo`0FOol01000B]d004[M200000=:g@0000001`00
00=:g@00B]d01P0000A:o@00B_e:o@H00003C_d0000000D00003C_h0000000D00003C_h004kn00D0
0003C_h004kn00@001Aoo`04001oogoo000:Ool0035oo`03001oogoo01Ioo`<00003B]d004[M00H0
0004B]d004[MB]d800000d[m0000000700000d[m0000000400000dkm001>o@0500000dkn001>oP07
00000dkn00000005000IOol00`00Ool0000:Ool0039oo`8001Moo`D00003B]d004[M00L00003B]d0
04[M00H00003B_d004[m00L00003C_d004km00H0009>o@H00003C_h0000000D0021oo`04001oogoo
0009Ool003Aoo`03001oogoo01Eoo`P00003B]d0000000H00003B]d0000000H00003B_d0000000L0
0003C_d0000000D00003C_d0000000@002Qoo`03001oo`0000Uoo`00=Goo00<007ooOol05Woo1`00
00=:g@00B]d01P0000=:g@00B]d01P0000A>o@00C_e>o@H00003C_d0000000D002moo`8000Uoo`00
=Woo00<007ooOol05Goo00<004[M00001`0000=:g@0000001P0000=:o@00B_d01`0000=>o@000000
1000=Goo0`002Goo000gOol00`00Oomoo`0EOol01000B]d004[M1P0000=:g@00B]d0200000=>o@00
00001@00=goo100037oo000hOol2000FOol300000d[M0000000600000d[M00000006000kOol3000@
Ool003Yoo`03001oogoo01Aoo`<00003B]d004[M00H003moo`@001=oo`00>goo00<007ooOol057oo
1@00@Goo10005goo000lOol00`00Oomoo`0DOol00`00Oomoo`0nOol4000KOol003eoo`8001Aoo`03
001oogoo03Yoo`@001moo`00?goo00<007ooOol04Goo00<007ooOol0=goo0`008goo0010Ool00`00
Oomoo`0?Ool00`00Oomoo`0dOol4000VOol0045oo`03001oogoo00ioo`03001oogoo031oo`@002Yo
o`00@Woo0P003Woo00<007ooOol0;7oo1000;Woo0014Ool00`00Oomoo`0;Ool00`00Oomoo`0XOol4
000bOol004Eoo`03001oogoo00Yoo`03001oogoo02Eoo`<003Ioo`00AWoo00<007ooOol02Goo00<0
07ooOol08Goo1000>Goo0017Ool20009Ool00`00Oomoo`0MOol4000mOol004Uoo`03001oogoo00Io
o`03001oogoo01Uoo`@0045oo`00BWoo00<007ooOol01Goo00<007ooOol05Woo0`00AGoo001;Ool0
0`00Oomoo`04Ool00`00Oomoo`0BOol40018Ool004aoo`03001oogoo00=oo`03001oogoo00ioo`@0
04aoo`00CGoo0P000goo00<007ooOol02Woo1000D7oo001?Ool01000Oomoo`0027oo1000E7oo001@
Ool00`00Ool00005Ool3001HOol0055oo`800003Ool00000008005]oo`00DWoo0P00Ggoo002cOol0
0;=oo`00/goo002cOol00;=oo`00/goo002cOol00001\
\>"],
  ImageRangeCache->{{{0, 178.438}, {144.25, 0}} -> {-0.158971, -0.055358, 
  0.00654104, 0.00654104}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot3D[
      BinaryPut[1, \ S, \ 100, \ 0.2, \ 0.1, \ 0, \ t], \ {S, \ 90, \ 
        110}, \ {t, \ 0.0001, \ 1}, \ PlotPoints\  -> \ 40]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
6.10623e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -16.7601 -9 ]
[.3345 .12542 1.2399 0 ]
[.50253 .05935 -15.8219 -9 ]
[.50253 .05935 2.17811 0 ]
[.68156 -0.01098 -14.8837 -9 ]
[.68156 -0.01098 3.11632 0 ]
[.76742 .09699 0 -6.16191 ]
[.76742 .09699 18 2.83809 ]
[.8281 .1899 0 -6.07249 ]
[.8281 .1899 18 2.92751 ]
[.88368 .27501 0 -5.9922 ]
[.88368 .27501 18 3.0078 ]
[.93477 .35326 0 -5.91971 ]
[.93477 .35326 18 3.08029 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.02411 .26511 -6 -2.74232 ]
[.02411 .26511 0 6.25768 ]
[.01568 .31795 -24 -2.78699 ]
[.01568 .31795 0 6.21301 ]
[.00696 .37264 -18 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00208 .42927 -24 -2.88161 ]
[-0.00208 .42927 0 6.11839 ]
[-0.01144 .48795 -6 -2.93176 ]
[-0.01144 .48795 0 6.06824 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(90)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(95)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(100)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(105)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(110)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.75512 .10154 m
.74896 .10381 L
s
[(0.2)] .76742 .09699 -1 .36931 Mshowa
.81571 .19423 m
.80952 .19639 L
s
[(0.4)] .8281 .1899 -1 .34944 Mshowa
.87122 .27914 m
.86499 .2812 L
s
[(0.6)] .88368 .27501 -1 .3316 Mshowa
.92226 .35721 m
.916 .35918 L
s
[(0.8)] .93477 .35326 -1 .31549 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(1)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.7059 .02625 m
.70222 .02766 L
s
.72269 .05193 m
.71901 .05333 L
s
.73909 .07702 m
.7354 .0784 L
s
.77078 .1255 m
.76708 .12684 L
s
.78609 .14892 m
.78239 .15025 L
s
.80107 .17182 m
.79735 .17314 L
s
.83004 .21614 m
.82632 .21743 L
s
.84406 .23759 m
.84033 .23886 L
s
.85779 .25859 m
.85405 .25984 L
s
.88438 .29926 m
.88064 .30049 L
s
.89727 .31898 m
.89352 .32019 L
s
.90989 .33829 m
.90614 .33949 L
s
.93438 .37576 m
.93063 .37693 L
s
.94627 .39394 m
.94251 .39509 L
s
.95792 .41176 m
.95415 .41291 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
[(0)] .02411 .26511 1 -0.3906 Mshowa
.02795 .31328 m
.03408 .31095 L
s
[(0.25)] .01568 .31795 1 -0.38067 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0.5)] .00696 .37264 1 -0.37036 Mshowa
.01027 .42483 m
.01644 .42261 L
s
[(0.75)] -0.00208 .42927 1 -0.35964 Mshowa
.00095 .48363 m
.00715 .48147 L
s
[(1)] -0.01144 .48795 1 -0.3485 Mshowa
.125 Mabswid
.03468 .27078 m
.03835 .26935 L
s
.03301 .2813 m
.03669 .27988 L
s
.03134 .29189 m
.03501 .29047 L
s
.02965 .30255 m
.03332 .30114 L
s
.02623 .32409 m
.02991 .3227 L
s
.02451 .33497 m
.02819 .33359 L
s
.02277 .34593 m
.02646 .34455 L
s
.02102 .35697 m
.02471 .35559 L
s
.01749 .37927 m
.02118 .37791 L
s
.0157 .39054 m
.0194 .38919 L
s
.0139 .40189 m
.0176 .40054 L
s
.01209 .41332 m
.01579 .41198 L
s
.00843 .43642 m
.01214 .4351 L
s
.00658 .4481 m
.01029 .44678 L
s
.00472 .45986 m
.00843 .45855 L
s
.00284 .4717 m
.00656 .4704 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.712 .793 .915 r
.39359 .69555 .40171 .70146 .41528 .6956 .40721 .68961 Metetra
.711 .792 .914 r
.40721 .68961 .41528 .6956 .4289 .68971 .42089 .68366 Metetra
.711 .791 .914 r
.42089 .68366 .4289 .68971 .44258 .68381 .43462 .67768 Metetra
.711 .79 .914 r
.43462 .67768 .44258 .68381 .45631 .6779 .44841 .6717 Metetra
.711 .789 .913 r
.44841 .6717 .45631 .6779 .47009 .67197 .46224 .6657 Metetra
.71 .789 .913 r
.46224 .6657 .47009 .67197 .48393 .66603 .47613 .65969 Metetra
.71 .788 .912 r
.47613 .65969 .48393 .66603 .49782 .66009 .49008 .65366 Metetra
.71 .787 .912 r
.49008 .65366 .49782 .66009 .51176 .65413 .50408 .64763 Metetra
.709 .786 .911 r
.50408 .64763 .51176 .65413 .52576 .64817 .51813 .64159 Metetra
.709 .785 .911 r
.51813 .64159 .52576 .64817 .53982 .64219 .53224 .63555 Metetra
.708 .784 .911 r
.53224 .63555 .53982 .64219 .55393 .63622 .54641 .62949 Metetra
.708 .783 .91 r
.54641 .62949 .55393 .63622 .5681 .63024 .56064 .62344 Metetra
.707 .782 .91 r
.56064 .62344 .5681 .63024 .58233 .62426 .57492 .61738 Metetra
.707 .781 .909 r
.57492 .61738 .58233 .62426 .59661 .61827 .58926 .61131 Metetra
.706 .78 .909 r
.58926 .61131 .59661 .61827 .61096 .61228 .60366 .60525 Metetra
.705 .779 .909 r
.60366 .60525 .61096 .61228 .62536 .60629 .61812 .59918 Metetra
.705 .778 .908 r
.61812 .59918 .62536 .60629 .63982 .6003 .63263 .59312 Metetra
.704 .777 .908 r
.63263 .59312 .63982 .6003 .65435 .59431 .64721 .58705 Metetra
.703 .776 .908 r
.64721 .58705 .65435 .59431 .66893 .58832 .66186 .58099 Metetra
.703 .775 .908 r
.66186 .58099 .66893 .58832 .68358 .58234 .67656 .57492 Metetra
.702 .774 .907 r
.67656 .57492 .68358 .58234 .69829 .57635 .69133 .56886 Metetra
.701 .773 .907 r
.69133 .56886 .69829 .57635 .71306 .57037 .70616 .5628 Metetra
.701 .772 .907 r
.70616 .5628 .71306 .57037 .7279 .56439 .72106 .55675 Metetra
.7 .771 .906 r
.72106 .55675 .7279 .56439 .74281 .55841 .73602 .55069 Metetra
.699 .77 .906 r
.73602 .55069 .74281 .55841 .75778 .55243 .75105 .54464 Metetra
.699 .769 .906 r
.75105 .54464 .75778 .55243 .77281 .54646 .76614 .5386 Metetra
.698 .768 .906 r
.76614 .5386 .77281 .54646 .78792 .54049 .78131 .53255 Metetra
.697 .767 .906 r
.78131 .53255 .78792 .54049 .80309 .53453 .79654 .52652 Metetra
.696 .766 .905 r
.79654 .52652 .80309 .53453 .81834 .52857 .81184 .52048 Metetra
.696 .765 .905 r
.81184 .52048 .81834 .52857 .83365 .52261 .82722 .51445 Metetra
.695 .764 .905 r
.82722 .51445 .83365 .52261 .84904 .51666 .84266 .50843 Metetra
.694 .763 .905 r
.84266 .50843 .84904 .51666 .86449 .51071 .85818 .5024 Metetra
.693 .762 .905 r
.85818 .5024 .86449 .51071 .88002 .50476 .87377 .49638 Metetra
.693 .762 .905 r
.87377 .49638 .88002 .50476 .89563 .49881 .88944 .49037 Metetra
.692 .761 .904 r
.88944 .49037 .89563 .49881 .91131 .49287 .90518 .48436 Metetra
.691 .76 .904 r
.90518 .48436 .91131 .49287 .92707 .48694 .921 .47835 Metetra
.691 .759 .904 r
.921 .47835 .92707 .48694 .9429 .481 .9369 .47234 Metetra
.69 .758 .904 r
.9369 .47234 .9429 .481 .95881 .47507 .95287 .46634 Metetra
.689 .758 .904 r
.95287 .46634 .95881 .47507 .9748 .46913 .96893 .46034 Metetra
.712 .794 .915 r
.38539 .68961 .39359 .69555 .40721 .68961 .39907 .6836 Metetra
.712 .793 .915 r
.39907 .6836 .40721 .68961 .42089 .68366 .4128 .67757 Metetra
.712 .792 .914 r
.4128 .67757 .42089 .68366 .43462 .67768 .42659 .67152 Metetra
.712 .791 .914 r
.42659 .67152 .43462 .67768 .44841 .6717 .44042 .66546 Metetra
.711 .79 .913 r
.44042 .66546 .44841 .6717 .46224 .6657 .45431 .65939 Metetra
.711 .789 .913 r
.45431 .65939 .46224 .6657 .47613 .65969 .46826 .6533 Metetra
.711 .788 .912 r
.46826 .6533 .47613 .65969 .49008 .65366 .48226 .6472 Metetra
.71 .787 .912 r
.48226 .6472 .49008 .65366 .50408 .64763 .49631 .64109 Metetra
.71 .786 .911 r
.49631 .64109 .50408 .64763 .51813 .64159 .51042 .63497 Metetra
.709 .785 .911 r
.51042 .63497 .51813 .64159 .53224 .63555 .52459 .62885 Metetra
.709 .784 .911 r
.52459 .62885 .53224 .63555 .54641 .62949 .53881 .62272 Metetra
.708 .783 .91 r
.53881 .62272 .54641 .62949 .56064 .62344 .55309 .61659 Metetra
.708 .782 .91 r
.55309 .61659 .56064 .62344 .57492 .61738 .56743 .61045 Metetra
.707 .781 .909 r
.56743 .61045 .57492 .61738 .58926 .61131 .58183 .6043 Metetra
.707 .78 .909 r
.58183 .6043 .58926 .61131 .60366 .60525 .59628 .59816 Metetra
.706 .779 .909 r
.59628 .59816 .60366 .60525 .61812 .59918 .6108 .59202 Metetra
.706 .778 .908 r
.6108 .59202 .61812 .59918 .63263 .59312 .62538 .58587 Metetra
.705 .777 .908 r
.62538 .58587 .63263 .59312 .64721 .58705 .64001 .57973 Metetra
.704 .776 .908 r
.64001 .57973 .64721 .58705 .66186 .58099 .65471 .57358 Metetra
.704 .775 .907 r
.65471 .57358 .66186 .58099 .67656 .57492 .66947 .56744 Metetra
.703 .774 .907 r
.66947 .56744 .67656 .57492 .69133 .56886 .6843 .5613 Metetra
.702 .773 .907 r
.6843 .5613 .69133 .56886 .70616 .5628 .69919 .55516 Metetra
.701 .772 .907 r
.69919 .55516 .70616 .5628 .72106 .55675 .71414 .54903 Metetra
.701 .771 .906 r
.71414 .54903 .72106 .55675 .73602 .55069 .72916 .5429 Metetra
.7 .77 .906 r
.72916 .5429 .73602 .55069 .75105 .54464 .74425 .53677 Metetra
.699 .769 .906 r
.74425 .53677 .75105 .54464 .76614 .5386 .7594 .53065 Metetra
.698 .768 .906 r
.7594 .53065 .76614 .5386 .78131 .53255 .77463 .52453 Metetra
.698 .767 .905 r
.77463 .52453 .78131 .53255 .79654 .52652 .78992 .51842 Metetra
.697 .766 .905 r
.78992 .51842 .79654 .52652 .81184 .52048 .80528 .51231 Metetra
.696 .765 .905 r
.80528 .51231 .81184 .52048 .82722 .51445 .82072 .50621 Metetra
.695 .764 .905 r
.82072 .50621 .82722 .51445 .84266 .50843 .83623 .50011 Metetra
.695 .763 .905 r
.83623 .50011 .84266 .50843 .85818 .5024 .85181 .49401 Metetra
.694 .762 .904 r
.85181 .49401 .85818 .5024 .87377 .49638 .86746 .48792 Metetra
.693 .762 .904 r
.86746 .48792 .87377 .49638 .88944 .49037 .88319 .48183 Metetra
.692 .761 .904 r
.88319 .48183 .88944 .49037 .90518 .48436 .899 .47575 Metetra
.692 .76 .904 r
.899 .47575 .90518 .48436 .921 .47835 .91488 .46967 Metetra
.691 .759 .904 r
.91488 .46967 .921 .47835 .9369 .47234 .93084 .46359 Metetra
.69 .758 .904 r
.93084 .46359 .9369 .47234 .95287 .46634 .94688 .45752 Metetra
.69 .758 .904 r
.94688 .45752 .95287 .46634 .96893 .46034 .963 .45145 Metetra
.713 .795 .915 r
.37711 .68363 .38539 .68961 .39907 .6836 .39084 .67755 Metetra
.713 .794 .915 r
.39084 .67755 .39907 .6836 .4128 .67757 .40463 .67145 Metetra
.713 .793 .914 r
.40463 .67145 .4128 .67757 .42659 .67152 .41847 .66533 Metetra
.713 .792 .914 r
.41847 .66533 .42659 .67152 .44042 .66546 .43236 .65919 Metetra
.712 .791 .913 r
.43236 .65919 .44042 .66546 .45431 .65939 .44631 .65304 Metetra
.712 .79 .913 r
.44631 .65304 .45431 .65939 .46826 .6533 .46031 .64687 Metetra
.712 .789 .912 r
.46031 .64687 .46826 .6533 .48226 .6472 .47436 .6407 Metetra
.711 .788 .912 r
.47436 .6407 .48226 .6472 .49631 .64109 .48847 .63451 Metetra
.711 .787 .911 r
.48847 .63451 .49631 .64109 .51042 .63497 .50264 .62831 Metetra
.71 .786 .911 r
.50264 .62831 .51042 .63497 .52459 .62885 .51686 .62211 Metetra
.71 .785 .91 r
.51686 .62211 .52459 .62885 .53881 .62272 .53114 .6159 Metetra
.709 .784 .91 r
.53114 .6159 .53881 .62272 .55309 .61659 .54548 .60968 Metetra
.709 .783 .91 r
.54548 .60968 .55309 .61659 .56743 .61045 .55987 .60346 Metetra
.708 .782 .909 r
.55987 .60346 .56743 .61045 .58183 .6043 .57433 .59724 Metetra
.708 .781 .909 r
.57433 .59724 .58183 .6043 .59628 .59816 .58884 .59101 Metetra
.707 .78 .908 r
.58884 .59101 .59628 .59816 .6108 .59202 .60341 .58479 Metetra
.706 .779 .908 r
.60341 .58479 .6108 .59202 .62538 .58587 .61804 .57856 Metetra
.706 .778 .908 r
.61804 .57856 .62538 .58587 .64001 .57973 .63274 .57234 Metetra
.705 .776 .907 r
.63274 .57234 .64001 .57973 .65471 .57358 .64749 .56611 Metetra
.704 .775 .907 r
.64749 .56611 .65471 .57358 .66947 .56744 .66231 .55989 Metetra
.704 .774 .907 r
.66231 .55989 .66947 .56744 .6843 .5613 .6772 .55367 Metetra
.703 .773 .907 r
.6772 .55367 .6843 .5613 .69919 .55516 .69215 .54746 Metetra
.702 .772 .906 r
.69215 .54746 .69919 .55516 .71414 .54903 .70716 .54124 Metetra
.701 .771 .906 r
.70716 .54124 .71414 .54903 .72916 .5429 .72224 .53503 Metetra
.701 .77 .906 r
.72224 .53503 .72916 .5429 .74425 .53677 .73738 .52883 Metetra
.7 .769 .905 r
.73738 .52883 .74425 .53677 .7594 .53065 .7526 .52263 Metetra
.699 .768 .905 r
.7526 .52263 .7594 .53065 .77463 .52453 .76788 .51643 Metetra
.698 .767 .905 r
.76788 .51643 .77463 .52453 .78992 .51842 .78324 .51024 Metetra
.698 .766 .905 r
.78324 .51024 .78992 .51842 .80528 .51231 .79866 .50406 Metetra
.697 .765 .905 r
.79866 .50406 .80528 .51231 .82072 .50621 .81416 .49788 Metetra
.696 .764 .904 r
.81416 .49788 .82072 .50621 .83623 .50011 .82972 .4917 Metetra
.695 .763 .904 r
.82972 .4917 .83623 .50011 .85181 .49401 .84537 .48553 Metetra
.694 .763 .904 r
.84537 .48553 .85181 .49401 .86746 .48792 .86108 .47936 Metetra
.694 .762 .904 r
.86108 .47936 .86746 .48792 .88319 .48183 .87687 .4732 Metetra
.693 .761 .904 r
.87687 .4732 .88319 .48183 .899 .47575 .89274 .46705 Metetra
.692 .76 .904 r
.89274 .46705 .899 .47575 .91488 .46967 .90869 .46089 Metetra
.691 .759 .904 r
.90869 .46089 .91488 .46967 .93084 .46359 .92471 .45475 Metetra
.691 .758 .903 r
.92471 .45475 .93084 .46359 .94688 .45752 .94082 .4486 Metetra
.69 .758 .903 r
.94082 .4486 .94688 .45752 .963 .45145 .957 .44246 Metetra
.714 .796 .915 r
.36875 .67763 .37711 .68363 .39084 .67755 .38253 .67147 Metetra
.714 .795 .915 r
.38253 .67147 .39084 .67755 .40463 .67145 .39637 .6653 Metetra
.714 .794 .914 r
.39637 .6653 .40463 .67145 .41847 .66533 .41027 .6591 Metetra
.713 .793 .914 r
.41027 .6591 .41847 .66533 .43236 .65919 .42422 .65288 Metetra
.713 .792 .913 r
.42422 .65288 .43236 .65919 .44631 .65304 .43822 .64665 Metetra
.713 .791 .913 r
.43822 .64665 .44631 .65304 .46031 .64687 .45227 .64041 Metetra
.713 .79 .912 r
.45227 .64041 .46031 .64687 .47436 .6407 .46639 .63415 Metetra
.712 .789 .912 r
.46639 .63415 .47436 .6407 .48847 .63451 .48055 .62788 Metetra
.712 .788 .911 r
.48055 .62788 .48847 .63451 .50264 .62831 .49477 .6216 Metetra
.711 .787 .911 r
.49477 .6216 .50264 .62831 .51686 .62211 .50905 .61532 Metetra
.711 .786 .91 r
.50905 .61532 .51686 .62211 .53114 .6159 .52339 .60902 Metetra
.71 .785 .91 r
.52339 .60902 .53114 .6159 .54548 .60968 .53778 .60273 Metetra
.71 .784 .91 r
.53778 .60273 .54548 .60968 .55987 .60346 .55224 .59642 Metetra
.709 .782 .909 r
.55224 .59642 .55987 .60346 .57433 .59724 .56675 .59012 Metetra
.709 .781 .909 r
.56675 .59012 .57433 .59724 .58884 .59101 .58132 .58381 Metetra
.708 .78 .908 r
.58132 .58381 .58884 .59101 .60341 .58479 .59595 .5775 Metetra
.707 .779 .908 r
.59595 .5775 .60341 .58479 .61804 .57856 .61064 .57119 Metetra
.707 .778 .908 r
.61064 .57119 .61804 .57856 .63274 .57234 .62539 .56488 Metetra
.706 .777 .907 r
.62539 .56488 .63274 .57234 .64749 .56611 .6402 .55858 Metetra
.705 .776 .907 r
.6402 .55858 .64749 .56611 .66231 .55989 .65508 .55227 Metetra
.704 .775 .907 r
.65508 .55227 .66231 .55989 .6772 .55367 .67002 .54597 Metetra
.704 .774 .906 r
.67002 .54597 .6772 .55367 .69215 .54746 .68503 .53967 Metetra
.703 .773 .906 r
.68503 .53967 .69215 .54746 .70716 .54124 .7001 .53338 Metetra
.702 .771 .906 r
.7001 .53338 .70716 .54124 .72224 .53503 .71524 .52709 Metetra
.701 .77 .905 r
.71524 .52709 .72224 .53503 .73738 .52883 .73045 .5208 Metetra
.701 .769 .905 r
.73045 .5208 .73738 .52883 .7526 .52263 .74572 .51452 Metetra
.7 .768 .905 r
.74572 .51452 .7526 .52263 .76788 .51643 .76107 .50825 Metetra
.699 .767 .905 r
.76107 .50825 .76788 .51643 .78324 .51024 .77648 .50198 Metetra
.698 .766 .905 r
.77648 .50198 .78324 .51024 .79866 .50406 .79197 .49571 Metetra
.697 .765 .904 r
.79197 .49571 .79866 .50406 .81416 .49788 .80752 .48946 Metetra
.697 .764 .904 r
.80752 .48946 .81416 .49788 .82972 .4917 .82315 .4832 Metetra
.696 .763 .904 r
.82315 .4832 .82972 .4917 .84537 .48553 .83886 .47695 Metetra
.695 .763 .904 r
.83886 .47695 .84537 .48553 .86108 .47936 .85464 .47071 Metetra
.694 .762 .904 r
.85464 .47071 .86108 .47936 .87687 .4732 .87049 .46448 Metetra
.693 .761 .903 r
.87049 .46448 .87687 .4732 .89274 .46705 .88643 .45824 Metetra
.693 .76 .903 r
.88643 .45824 .89274 .46705 .90869 .46089 .90244 .45202 Metetra
.692 .759 .903 r
.90244 .45202 .90869 .46089 .92471 .45475 .91853 .4458 Metetra
.691 .758 .903 r
.91853 .4458 .92471 .45475 .94082 .4486 .9347 .43958 Metetra
.69 .757 .903 r
.9347 .43958 .94082 .4486 .957 .44246 .95095 .43336 Metetra
.715 .797 .915 r
.36029 .6716 .36875 .67763 .38253 .67147 .37413 .66537 Metetra
.715 .796 .915 r
.37413 .66537 .38253 .67147 .39637 .6653 .38803 .65911 Metetra
.715 .795 .914 r
.38803 .65911 .39637 .6653 .41027 .6591 .40198 .65284 Metetra
.714 .794 .914 r
.40198 .65284 .41027 .6591 .42422 .65288 .41599 .64654 Metetra
.714 .793 .913 r
.41599 .64654 .42422 .65288 .43822 .64665 .43004 .64023 Metetra
.714 .792 .913 r
.43004 .64023 .43822 .64665 .45227 .64041 .44416 .6339 Metetra
.713 .791 .912 r
.44416 .6339 .45227 .64041 .46639 .63415 .45833 .62756 Metetra
.713 .79 .912 r
.45833 .62756 .46639 .63415 .48055 .62788 .47255 .62121 Metetra
.713 .789 .911 r
.47255 .62121 .48055 .62788 .49477 .6216 .48683 .61485 Metetra
.712 .788 .911 r
.48683 .61485 .49477 .6216 .50905 .61532 .50117 .60848 Metetra
.712 .787 .91 r
.50117 .60848 .50905 .61532 .52339 .60902 .51556 .6021 Metetra
.711 .785 .91 r
.51556 .6021 .52339 .60902 .53778 .60273 .53001 .59572 Metetra
.711 .784 .909 r
.53001 .59572 .53778 .60273 .55224 .59642 .54452 .58933 Metetra
.71 .783 .909 r
.54452 .58933 .55224 .59642 .56675 .59012 .55909 .58294 Metetra
.709 .782 .909 r
.55909 .58294 .56675 .59012 .58132 .58381 .57372 .57655 Metetra
.709 .781 .908 r
.57372 .57655 .58132 .58381 .59595 .5775 .58841 .57015 Metetra
.708 .78 .908 r
.58841 .57015 .59595 .5775 .61064 .57119 .60316 .56376 Metetra
.707 .779 .907 r
.60316 .56376 .61064 .57119 .62539 .56488 .61797 .55737 Metetra
.707 .777 .907 r
.61797 .55737 .62539 .56488 .6402 .55858 .63284 .55097 Metetra
.706 .776 .907 r
.63284 .55097 .6402 .55858 .65508 .55227 .64778 .54458 Metetra
.705 .775 .906 r
.64778 .54458 .65508 .55227 .67002 .54597 .66278 .5382 Metetra
.704 .774 .906 r
.66278 .5382 .67002 .54597 .68503 .53967 .67785 .53182 Metetra
.704 .773 .906 r
.67785 .53182 .68503 .53967 .7001 .53338 .69298 .52544 Metetra
.703 .772 .905 r
.69298 .52544 .7001 .53338 .71524 .52709 .70818 .51906 Metetra
.702 .771 .905 r
.70818 .51906 .71524 .52709 .73045 .5208 .72344 .5127 Metetra
.701 .77 .905 r
.72344 .5127 .73045 .5208 .74572 .51452 .73878 .50633 Metetra
.7 .769 .905 r
.73878 .50633 .74572 .51452 .76107 .50825 .75418 .49998 Metetra
.7 .768 .904 r
.75418 .49998 .76107 .50825 .77648 .50198 .76966 .49363 Metetra
.699 .767 .904 r
.76966 .49363 .77648 .50198 .79197 .49571 .78521 .48728 Metetra
.698 .765 .904 r
.78521 .48728 .79197 .49571 .80752 .48946 .80083 .48094 Metetra
.697 .765 .904 r
.80083 .48094 .80752 .48946 .82315 .4832 .81652 .47461 Metetra
.696 .764 .904 r
.81652 .47461 .82315 .4832 .83886 .47695 .83228 .46829 Metetra
.696 .763 .903 r
.83228 .46829 .83886 .47695 .85464 .47071 .84813 .46197 Metetra
.695 .762 .903 r
.84813 .46197 .85464 .47071 .87049 .46448 .86405 .45565 Metetra
.694 .761 .903 r
.86405 .45565 .87049 .46448 .88643 .45824 .88004 .44934 Metetra
.693 .76 .903 r
.88004 .44934 .88643 .45824 .90244 .45202 .89612 .44304 Metetra
.692 .759 .903 r
.89612 .44304 .90244 .45202 .91853 .4458 .91227 .43674 Metetra
.692 .758 .903 r
.91227 .43674 .91853 .4458 .9347 .43958 .92851 .43045 Metetra
.691 .757 .903 r
.92851 .43045 .9347 .43958 .95095 .43336 .94483 .42416 Metetra
.716 .798 .916 r
.35175 .66555 .36029 .6716 .37413 .66537 .36565 .65924 Metetra
.716 .797 .915 r
.36565 .65924 .37413 .66537 .38803 .65911 .3796 .6529 Metetra
.715 .796 .914 r
.3796 .6529 .38803 .65911 .40198 .65284 .39361 .64654 Metetra
.715 .795 .914 r
.39361 .64654 .40198 .65284 .41599 .64654 .40767 .64017 Metetra
.715 .794 .913 r
.40767 .64017 .41599 .64654 .43004 .64023 .42179 .63377 Metetra
.715 .793 .913 r
.42179 .63377 .43004 .64023 .44416 .6339 .43596 .62736 Metetra
.714 .792 .912 r
.43596 .62736 .44416 .6339 .45833 .62756 .45018 .62094 Metetra
.714 .791 .912 r
.45018 .62094 .45833 .62756 .47255 .62121 .46447 .6145 Metetra
.714 .79 .911 r
.46447 .6145 .47255 .62121 .48683 .61485 .4788 .60805 Metetra
.713 .789 .911 r
.4788 .60805 .48683 .61485 .50117 .60848 .4932 .6016 Metetra
.713 .787 .91 r
.4932 .6016 .50117 .60848 .51556 .6021 .50765 .59513 Metetra
.712 .786 .91 r
.50765 .59513 .51556 .6021 .53001 .59572 .52216 .58866 Metetra
.712 .785 .909 r
.52216 .58866 .53001 .59572 .54452 .58933 .53673 .58219 Metetra
.711 .784 .909 r
.53673 .58219 .54452 .58933 .55909 .58294 .55136 .57571 Metetra
.71 .783 .908 r
.55136 .57571 .55909 .58294 .57372 .57655 .56604 .56923 Metetra
.71 .781 .908 r
.56604 .56923 .57372 .57655 .58841 .57015 .58079 .56274 Metetra
.709 .78 .908 r
.58079 .56274 .58841 .57015 .60316 .56376 .5956 .55626 Metetra
.708 .779 .907 r
.5956 .55626 .60316 .56376 .61797 .55737 .61047 .54978 Metetra
.708 .778 .907 r
.61047 .54978 .61797 .55737 .63284 .55097 .6254 .5433 Metetra
.707 .777 .906 r
.6254 .5433 .63284 .55097 .64778 .54458 .6404 .53683 Metetra
.706 .776 .906 r
.6404 .53683 .64778 .54458 .66278 .5382 .65546 .53035 Metetra
.705 .774 .906 r
.65546 .53035 .66278 .5382 .67785 .53182 .67059 .52388 Metetra
.705 .773 .905 r
.67059 .52388 .67785 .53182 .69298 .52544 .68578 .51742 Metetra
.704 .772 .905 r
.68578 .51742 .69298 .52544 .70818 .51906 .70104 .51096 Metetra
.703 .771 .905 r
.70104 .51096 .70818 .51906 .72344 .5127 .71637 .50451 Metetra
.702 .77 .905 r
.71637 .50451 .72344 .5127 .73878 .50633 .73176 .49806 Metetra
.701 .769 .904 r
.73176 .49806 .73878 .50633 .75418 .49998 .74723 .49162 Metetra
.7 .768 .904 r
.74723 .49162 .75418 .49998 .76966 .49363 .76277 .48519 Metetra
.7 .767 .904 r
.76277 .48519 .76966 .49363 .78521 .48728 .77838 .47876 Metetra
.699 .766 .904 r
.77838 .47876 .78521 .48728 .80083 .48094 .79406 .47234 Metetra
.698 .765 .903 r
.79406 .47234 .80083 .48094 .81652 .47461 .80981 .46593 Metetra
.697 .764 .903 r
.80981 .46593 .81652 .47461 .83228 .46829 .82564 .45952 Metetra
.696 .763 .903 r
.82564 .45952 .83228 .46829 .84813 .46197 .84155 .45312 Metetra
.695 .762 .903 r
.84155 .45312 .84813 .46197 .86405 .45565 .85753 .44673 Metetra
.694 .761 .903 r
.85753 .44673 .86405 .45565 .88004 .44934 .87359 .44034 Metetra
.694 .76 .903 r
.87359 .44034 .88004 .44934 .89612 .44304 .88973 .43396 Metetra
.693 .759 .902 r
.88973 .43396 .89612 .44304 .91227 .43674 .90595 .42759 Metetra
.692 .758 .902 r
.90595 .42759 .91227 .43674 .92851 .43045 .92226 .42122 Metetra
.691 .757 .902 r
.92226 .42122 .92851 .43045 .94483 .42416 .93864 .41486 Metetra
.716 .799 .916 r
.34312 .65947 .35175 .66555 .36565 .65924 .35707 .65308 Metetra
.716 .798 .915 r
.35707 .65308 .36565 .65924 .3796 .6529 .37108 .64666 Metetra
.716 .797 .915 r
.37108 .64666 .3796 .6529 .39361 .64654 .38515 .64022 Metetra
.716 .796 .914 r
.38515 .64022 .39361 .64654 .40767 .64017 .39927 .63376 Metetra
.716 .795 .913 r
.39927 .63376 .40767 .64017 .42179 .63377 .41344 .62728 Metetra
.716 .794 .913 r
.41344 .62728 .42179 .63377 .43596 .62736 .42767 .62079 Metetra
.715 .793 .912 r
.42767 .62079 .43596 .62736 .45018 .62094 .44196 .61428 Metetra
.715 .792 .912 r
.44196 .61428 .45018 .62094 .46447 .6145 .4563 .60775 Metetra
.715 .791 .911 r
.4563 .60775 .46447 .6145 .4788 .60805 .47069 .60121 Metetra
.714 .789 .911 r
.47069 .60121 .4788 .60805 .4932 .6016 .48515 .59467 Metetra
.714 .788 .91 r
.48515 .59467 .4932 .6016 .50765 .59513 .49966 .58811 Metetra
.713 .787 .91 r
.49966 .58811 .50765 .59513 .52216 .58866 .51423 .58155 Metetra
.713 .786 .909 r
.51423 .58155 .52216 .58866 .53673 .58219 .52885 .57499 Metetra
.712 .785 .909 r
.52885 .57499 .53673 .58219 .55136 .57571 .54354 .56842 Metetra
.711 .783 .908 r
.54354 .56842 .55136 .57571 .56604 .56923 .55829 .56185 Metetra
.711 .782 .908 r
.55829 .56185 .56604 .56923 .58079 .56274 .57309 .55528 Metetra
.71 .781 .907 r
.57309 .55528 .58079 .56274 .5956 .55626 .58796 .5487 Metetra
.709 .78 .907 r
.58796 .5487 .5956 .55626 .61047 .54978 .60289 .54213 Metetra
.709 .778 .907 r
.60289 .54213 .61047 .54978 .6254 .5433 .61789 .53556 Metetra
.708 .777 .906 r
.61789 .53556 .6254 .5433 .6404 .53683 .63294 .529 Metetra
.707 .776 .906 r
.63294 .529 .6404 .53683 .65546 .53035 .64807 .52243 Metetra
.706 .775 .905 r
.64807 .52243 .65546 .53035 .67059 .52388 .66325 .51588 Metetra
.705 .774 .905 r
.66325 .51588 .67059 .52388 .68578 .51742 .67851 .50932 Metetra
.705 .772 .905 r
.67851 .50932 .68578 .51742 .70104 .51096 .69383 .50278 Metetra
.704 .771 .905 r
.69383 .50278 .70104 .51096 .71637 .50451 .70922 .49624 Metetra
.703 .77 .904 r
.70922 .49624 .71637 .50451 .73176 .49806 .72467 .4897 Metetra
.702 .769 .904 r
.72467 .4897 .73176 .49806 .74723 .49162 .7402 .48318 Metetra
.701 .768 .904 r
.7402 .48318 .74723 .49162 .76277 .48519 .7558 .47666 Metetra
.7 .767 .903 r
.7558 .47666 .76277 .48519 .77838 .47876 .77147 .47014 Metetra
.699 .766 .903 r
.77147 .47014 .77838 .47876 .79406 .47234 .78722 .46364 Metetra
.698 .765 .903 r
.78722 .46364 .79406 .47234 .80981 .46593 .80304 .45714 Metetra
.698 .764 .903 r
.80304 .45714 .80981 .46593 .82564 .45952 .81893 .45066 Metetra
.697 .763 .903 r
.81893 .45066 .82564 .45952 .84155 .45312 .8349 .44417 Metetra
.696 .762 .902 r
.8349 .44417 .84155 .45312 .85753 .44673 .85095 .4377 Metetra
.695 .761 .902 r
.85095 .4377 .85753 .44673 .87359 .44034 .86708 .43123 Metetra
.694 .76 .902 r
.86708 .43123 .87359 .44034 .88973 .43396 .88328 .42477 Metetra
.693 .759 .902 r
.88328 .42477 .88973 .43396 .90595 .42759 .89957 .41832 Metetra
.692 .758 .902 r
.89957 .41832 .90595 .42759 .92226 .42122 .91594 .41188 Metetra
.692 .757 .902 r
.91594 .41188 .92226 .42122 .93864 .41486 .93239 .40544 Metetra
.717 .8 .916 r
.33439 .65337 .34312 .65947 .35707 .65308 .34841 .6469 Metetra
.717 .799 .915 r
.34841 .6469 .35707 .65308 .37108 .64666 .36248 .6404 Metetra
.717 .798 .915 r
.36248 .6404 .37108 .64666 .38515 .64022 .3766 .63387 Metetra
.717 .797 .914 r
.3766 .63387 .38515 .64022 .39927 .63376 .39078 .62733 Metetra
.717 .796 .914 r
.39078 .62733 .39927 .63376 .41344 .62728 .40501 .62076 Metetra
.717 .795 .913 r
.40501 .62076 .41344 .62728 .42767 .62079 .4193 .61418 Metetra
.716 .794 .912 r
.4193 .61418 .42767 .62079 .44196 .61428 .43364 .60758 Metetra
.716 .793 .912 r
.43364 .60758 .44196 .61428 .4563 .60775 .44804 .60096 Metetra
.716 .792 .911 r
.44804 .60096 .4563 .60775 .47069 .60121 .4625 .59433 Metetra
.715 .79 .911 r
.4625 .59433 .47069 .60121 .48515 .59467 .47701 .5877 Metetra
.715 .789 .91 r
.47701 .5877 .48515 .59467 .49966 .58811 .49158 .58105 Metetra
.714 .788 .91 r
.49158 .58105 .49966 .58811 .51423 .58155 .50621 .5744 Metetra
.714 .787 .909 r
.50621 .5744 .51423 .58155 .52885 .57499 .5209 .56774 Metetra
.713 .785 .909 r
.5209 .56774 .52885 .57499 .54354 .56842 .53565 .56107 Metetra
.712 .784 .908 r
.53565 .56107 .54354 .56842 .55829 .56185 .55045 .55441 Metetra
.712 .783 .908 r
.55045 .55441 .55829 .56185 .57309 .55528 .56532 .54774 Metetra
.711 .782 .907 r
.56532 .54774 .57309 .55528 .58796 .5487 .58025 .54108 Metetra
.71 .78 .907 r
.58025 .54108 .58796 .5487 .60289 .54213 .59524 .53441 Metetra
.709 .779 .906 r
.59524 .53441 .60289 .54213 .61789 .53556 .61029 .52775 Metetra
.709 .778 .906 r
.61029 .52775 .61789 .53556 .63294 .529 .62541 .52109 Metetra
.708 .776 .906 r
.62541 .52109 .63294 .529 .64807 .52243 .6406 .51444 Metetra
.707 .775 .905 r
.6406 .51444 .64807 .52243 .66325 .51588 .65584 .50779 Metetra
.706 .774 .905 r
.65584 .50779 .66325 .51588 .67851 .50932 .67116 .50115 Metetra
.705 .773 .904 r
.67116 .50115 .67851 .50932 .69383 .50278 .68654 .49451 Metetra
.705 .772 .904 r
.68654 .49451 .69383 .50278 .70922 .49624 .70199 .48788 Metetra
.704 .77 .904 r
.70199 .48788 .70922 .49624 .72467 .4897 .71751 .48126 Metetra
.703 .769 .904 r
.71751 .48126 .72467 .4897 .7402 .48318 .7331 .47464 Metetra
.702 .768 .903 r
.7331 .47464 .7402 .48318 .7558 .47666 .74877 .46803 Metetra
.701 .767 .903 r
.74877 .46803 .7558 .47666 .77147 .47014 .7645 .46143 Metetra
.7 .766 .903 r
.7645 .46143 .77147 .47014 .78722 .46364 .78031 .45484 Metetra
.699 .765 .903 r
.78031 .45484 .78722 .46364 .80304 .45714 .79619 .44826 Metetra
.698 .764 .902 r
.79619 .44826 .80304 .45714 .81893 .45066 .81215 .44169 Metetra
.697 .763 .902 r
.81215 .44169 .81893 .45066 .8349 .44417 .82818 .43512 Metetra
.696 .762 .902 r
.82818 .43512 .8349 .44417 .85095 .4377 .8443 .42857 Metetra
.696 .761 .902 r
.8443 .42857 .85095 .4377 .86708 .43123 .86049 .42202 Metetra
.695 .76 .902 r
.86049 .42202 .86708 .43123 .88328 .42477 .87676 .41548 Metetra
.694 .759 .902 r
.87676 .41548 .88328 .42477 .89957 .41832 .89311 .40895 Metetra
.693 .758 .901 r
.89311 .40895 .89957 .41832 .91594 .41188 .90955 .40242 Metetra
.692 .757 .901 r
.90955 .40242 .91594 .41188 .93239 .40544 .92607 .3959 Metetra
.718 .802 .916 r
.32557 .64725 .33439 .65337 .34841 .6469 .33965 .64069 Metetra
.718 .801 .916 r
.33965 .64069 .34841 .6469 .36248 .6404 .35378 .63411 Metetra
.718 .799 .915 r
.35378 .63411 .36248 .6404 .3766 .63387 .36796 .6275 Metetra
.718 .798 .914 r
.36796 .6275 .3766 .63387 .39078 .62733 .38219 .62086 Metetra
.718 .797 .914 r
.38219 .62086 .39078 .62733 .40501 .62076 .39649 .61421 Metetra
.718 .796 .913 r
.39649 .61421 .40501 .62076 .4193 .61418 .41084 .60753 Metetra
.718 .795 .912 r
.41084 .60753 .4193 .61418 .43364 .60758 .42524 .60084 Metetra
.717 .794 .912 r
.42524 .60084 .43364 .60758 .44804 .60096 .4397 .59413 Metetra
.717 .793 .911 r
.4397 .59413 .44804 .60096 .4625 .59433 .45422 .58741 Metetra
.716 .791 .911 r
.45422 .58741 .4625 .59433 .47701 .5877 .46879 .58068 Metetra
.716 .79 .91 r
.46879 .58068 .47701 .5877 .49158 .58105 .48342 .57394 Metetra
.715 .789 .909 r
.48342 .57394 .49158 .58105 .50621 .5744 .49811 .56719 Metetra
.715 .787 .909 r
.49811 .56719 .50621 .5744 .5209 .56774 .51286 .56043 Metetra
.714 .786 .908 r
.51286 .56043 .5209 .56774 .53565 .56107 .52767 .55367 Metetra
.713 .785 .908 r
.52767 .55367 .53565 .56107 .55045 .55441 .54254 .54691 Metetra
.713 .784 .907 r
.54254 .54691 .55045 .55441 .56532 .54774 .55746 .54015 Metetra
.712 .782 .907 r
.55746 .54015 .56532 .54774 .58025 .54108 .57245 .53339 Metetra
.711 .781 .906 r
.57245 .53339 .58025 .54108 .59524 .53441 .58751 .52663 Metetra
.711 .78 .906 r
.58751 .52663 .59524 .53441 .61029 .52775 .60262 .51987 Metetra
.71 .778 .906 r
.60262 .51987 .61029 .52775 .62541 .52109 .6178 .51312 Metetra
.709 .777 .905 r
.6178 .51312 .62541 .52109 .6406 .51444 .63305 .50637 Metetra
.708 .776 .905 r
.63305 .50637 .6406 .51444 .65584 .50779 .64836 .49962 Metetra
.707 .774 .904 r
.64836 .49962 .65584 .50779 .67116 .50115 .66373 .49289 Metetra
.706 .773 .904 r
.66373 .49289 .67116 .50115 .68654 .49451 .67918 .48616 Metetra
.705 .772 .904 r
.67918 .48616 .68654 .49451 .70199 .48788 .69469 .47943 Metetra
.704 .771 .903 r
.69469 .47943 .70199 .48788 .71751 .48126 .71028 .47272 Metetra
.704 .769 .903 r
.71028 .47272 .71751 .48126 .7331 .47464 .72593 .46601 Metetra
.703 .768 .903 r
.72593 .46601 .7331 .47464 .74877 .46803 .74165 .45932 Metetra
.702 .767 .903 r
.74165 .45932 .74877 .46803 .7645 .46143 .75745 .45263 Metetra
.701 .766 .902 r
.75745 .45263 .7645 .46143 .78031 .45484 .77332 .44595 Metetra
.7 .765 .902 r
.77332 .44595 .78031 .45484 .79619 .44826 .78927 .43928 Metetra
.699 .764 .902 r
.78927 .43928 .79619 .44826 .81215 .44169 .80529 .43262 Metetra
.698 .763 .902 r
.80529 .43262 .81215 .44169 .82818 .43512 .8214 .42597 Metetra
.697 .762 .902 r
.8214 .42597 .82818 .43512 .8443 .42857 .83757 .41933 Metetra
.696 .761 .901 r
.83757 .41933 .8443 .42857 .86049 .42202 .85383 .4127 Metetra
.695 .76 .901 r
.85383 .4127 .86049 .42202 .87676 .41548 .87017 .40607 Metetra
.694 .759 .901 r
.87017 .40607 .87676 .41548 .89311 .40895 .88659 .39946 Metetra
.693 .758 .901 r
.88659 .39946 .89311 .40895 .90955 .40242 .90309 .39285 Metetra
.693 .757 .901 r
.90309 .39285 .90955 .40242 .92607 .3959 .91968 .38625 Metetra
.719 .803 .917 r
.31666 .64111 .32557 .64725 .33965 .64069 .33079 .63447 Metetra
.719 .802 .916 r
.33079 .63447 .33965 .64069 .35378 .63411 .34498 .6278 Metetra
.719 .801 .915 r
.34498 .6278 .35378 .63411 .36796 .6275 .35922 .6211 Metetra
.719 .8 .914 r
.35922 .6211 .36796 .6275 .38219 .62086 .37352 .61437 Metetra
.719 .799 .914 r
.37352 .61437 .38219 .62086 .39649 .61421 .38787 .60763 Metetra
.719 .797 .913 r
.38787 .60763 .39649 .61421 .41084 .60753 .40228 .60086 Metetra
.719 .796 .912 r
.40228 .60086 .41084 .60753 .42524 .60084 .41675 .59407 Metetra
.718 .795 .912 r
.41675 .59407 .42524 .60084 .4397 .59413 .43127 .58726 Metetra
.718 .794 .911 r
.43127 .58726 .4397 .59413 .45422 .58741 .44585 .58045 Metetra
.717 .792 .911 r
.44585 .58045 .45422 .58741 .46879 .58068 .46048 .57362 Metetra
.717 .791 .91 r
.46048 .57362 .46879 .58068 .48342 .57394 .47518 .56678 Metetra
.716 .79 .909 r
.47518 .56678 .48342 .57394 .49811 .56719 .48993 .55993 Metetra
.716 .788 .909 r
.48993 .55993 .49811 .56719 .51286 .56043 .50474 .55308 Metetra
.715 .787 .908 r
.50474 .55308 .51286 .56043 .52767 .55367 .51961 .54622 Metetra
.715 .786 .908 r
.51961 .54622 .52767 .55367 .54254 .54691 .53454 .53936 Metetra
.714 .784 .907 r
.53454 .53936 .54254 .54691 .55746 .54015 .54953 .5325 Metetra
.713 .783 .907 r
.54953 .5325 .55746 .54015 .57245 .53339 .56458 .52564 Metetra
.712 .782 .906 r
.56458 .52564 .57245 .53339 .58751 .52663 .57969 .51878 Metetra
.712 .78 .906 r
.57969 .51878 .58751 .52663 .60262 .51987 .59487 .51192 Metetra
.711 .779 .905 r
.59487 .51192 .60262 .51987 .6178 .51312 .61011 .50507 Metetra
.71 .777 .905 r
.61011 .50507 .6178 .51312 .63305 .50637 .62542 .49822 Metetra
.709 .776 .905 r
.62542 .49822 .63305 .50637 .64836 .49962 .64079 .49138 Metetra
.708 .775 .904 r
.64079 .49138 .64836 .49962 .66373 .49289 .65623 .48454 Metetra
.707 .773 .904 r
.65623 .48454 .66373 .49289 .67918 .48616 .67174 .47772 Metetra
.706 .772 .903 r
.67174 .47772 .67918 .48616 .69469 .47943 .68732 .4709 Metetra
.705 .771 .903 r
.68732 .4709 .69469 .47943 .71028 .47272 .70296 .46409 Metetra
.704 .77 .903 r
.70296 .46409 .71028 .47272 .72593 .46601 .71868 .45729 Metetra
.703 .768 .902 r
.71868 .45729 .72593 .46601 .74165 .45932 .73447 .4505 Metetra
.703 .767 .902 r
.73447 .4505 .74165 .45932 .75745 .45263 .75033 .44372 Metetra
.702 .766 .902 r
.75033 .44372 .75745 .45263 .77332 .44595 .76627 .43695 Metetra
.701 .765 .902 r
.76627 .43695 .77332 .44595 .78927 .43928 .78228 .43019 Metetra
.7 .764 .901 r
.78228 .43019 .78927 .43928 .80529 .43262 .79837 .42344 Metetra
.699 .763 .901 r
.79837 .42344 .80529 .43262 .8214 .42597 .81453 .4167 Metetra
.698 .761 .901 r
.81453 .4167 .8214 .42597 .83757 .41933 .83078 .40998 Metetra
.697 .76 .901 r
.83078 .40998 .83757 .41933 .85383 .4127 .8471 .40326 Metetra
.696 .759 .901 r
.8471 .40326 .85383 .4127 .87017 .40607 .86351 .39655 Metetra
.695 .758 .901 r
.86351 .39655 .87017 .40607 .88659 .39946 .88 .38985 Metetra
.694 .757 .901 r
.88 .38985 .88659 .39946 .90309 .39285 .89657 .38316 Metetra
.693 .756 .9 r
.89657 .38316 .90309 .39285 .91968 .38625 .91322 .37648 Metetra
.72 .804 .917 r
.30765 .63496 .31666 .64111 .33079 .63447 .32184 .62823 Metetra
.72 .803 .916 r
.32184 .62823 .33079 .63447 .34498 .6278 .33609 .62147 Metetra
.72 .802 .915 r
.33609 .62147 .34498 .6278 .35922 .6211 .35039 .61468 Metetra
.72 .801 .915 r
.35039 .61468 .35922 .6211 .37352 .61437 .36475 .60786 Metetra
.72 .8 .914 r
.36475 .60786 .37352 .61437 .38787 .60763 .37917 .60102 Metetra
.72 .799 .913 r
.37917 .60102 .38787 .60763 .40228 .60086 .39364 .59415 Metetra
.72 .797 .913 r
.39364 .59415 .40228 .60086 .41675 .59407 .40816 .58727 Metetra
.719 .796 .912 r
.40816 .58727 .41675 .59407 .43127 .58726 .42275 .58036 Metetra
.719 .795 .911 r
.42275 .58036 .43127 .58726 .44585 .58045 .43739 .57345 Metetra
.719 .793 .911 r
.43739 .57345 .44585 .58045 .46048 .57362 .45209 .56652 Metetra
.718 .792 .91 r
.45209 .56652 .46048 .57362 .47518 .56678 .46684 .55958 Metetra
.718 .791 .909 r
.46684 .55958 .47518 .56678 .48993 .55993 .48165 .55263 Metetra
.717 .789 .909 r
.48165 .55263 .48993 .55993 .50474 .55308 .49653 .54567 Metetra
.716 .788 .908 r
.49653 .54567 .50474 .55308 .51961 .54622 .51146 .53871 Metetra
.716 .787 .908 r
.51146 .53871 .51961 .54622 .53454 .53936 .52645 .53175 Metetra
.715 .785 .907 r
.52645 .53175 .53454 .53936 .54953 .5325 .5415 .52478 Metetra
.714 .784 .906 r
.5415 .52478 .54953 .5325 .56458 .52564 .55662 .51782 Metetra
.714 .782 .906 r
.55662 .51782 .56458 .52564 .57969 .51878 .5718 .51086 Metetra
.713 .781 .905 r
.5718 .51086 .57969 .51878 .59487 .51192 .58704 .5039 Metetra
.712 .779 .905 r
.58704 .5039 .59487 .51192 .61011 .50507 .60234 .49694 Metetra
.711 .778 .905 r
.60234 .49694 .61011 .50507 .62542 .49822 .61771 .48999 Metetra
.71 .777 .904 r
.61771 .48999 .62542 .49822 .64079 .49138 .63315 .48305 Metetra
.709 .775 .904 r
.63315 .48305 .64079 .49138 .65623 .48454 .64865 .47612 Metetra
.708 .774 .903 r
.64865 .47612 .65623 .48454 .67174 .47772 .66422 .46919 Metetra
.707 .772 .903 r
.66422 .46919 .67174 .47772 .68732 .4709 .67986 .46228 Metetra
.706 .771 .903 r
.67986 .46228 .68732 .4709 .70296 .46409 .69557 .45537 Metetra
.705 .77 .902 r
.69557 .45537 .70296 .46409 .71868 .45729 .71135 .44847 Metetra
.704 .769 .902 r
.71135 .44847 .71868 .45729 .73447 .4505 .72721 .44159 Metetra
.703 .767 .902 r
.72721 .44159 .73447 .4505 .75033 .44372 .74314 .43471 Metetra
.702 .766 .901 r
.74314 .43471 .75033 .44372 .76627 .43695 .75914 .42785 Metetra
.701 .765 .901 r
.75914 .42785 .76627 .43695 .78228 .43019 .77521 .421 Metetra
.7 .764 .901 r
.77521 .421 .78228 .43019 .79837 .42344 .79137 .41416 Metetra
.699 .762 .901 r
.79137 .41416 .79837 .42344 .81453 .4167 .8076 .40733 Metetra
.698 .761 .901 r
.8076 .40733 .81453 .4167 .83078 .40998 .82391 .40051 Metetra
.697 .76 .9 r
.82391 .40051 .83078 .40998 .8471 .40326 .8403 .3937 Metetra
.696 .759 .9 r
.8403 .3937 .8471 .40326 .86351 .39655 .85678 .38691 Metetra
.695 .758 .9 r
.85678 .38691 .86351 .39655 .88 .38985 .87333 .38012 Metetra
.694 .757 .9 r
.87333 .38012 .88 .38985 .89657 .38316 .88997 .37335 Metetra
.694 .756 .9 r
.88997 .37335 .89657 .38316 .91322 .37648 .9067 .36659 Metetra
.721 .806 .917 r
.29853 .62879 .30765 .63496 .32184 .62823 .31279 .62197 Metetra
.722 .805 .916 r
.31279 .62197 .32184 .62823 .33609 .62147 .3271 .61512 Metetra
.722 .804 .916 r
.3271 .61512 .33609 .62147 .35039 .61468 .34146 .60824 Metetra
.722 .802 .915 r
.34146 .60824 .35039 .61468 .36475 .60786 .35589 .60132 Metetra
.721 .801 .914 r
.35589 .60132 .36475 .60786 .37917 .60102 .37036 .59438 Metetra
.721 .8 .913 r
.37036 .59438 .37917 .60102 .39364 .59415 .3849 .58742 Metetra
.721 .799 .913 r
.3849 .58742 .39364 .59415 .40816 .58727 .39949 .58043 Metetra
.721 .797 .912 r
.39949 .58043 .40816 .58727 .42275 .58036 .41413 .57343 Metetra
.72 .796 .911 r
.41413 .57343 .42275 .58036 .43739 .57345 .42884 .56641 Metetra
.72 .795 .91 r
.42884 .56641 .43739 .57345 .45209 .56652 .4436 .55937 Metetra
.719 .793 .91 r
.4436 .55937 .45209 .56652 .46684 .55958 .45842 .55233 Metetra
.719 .792 .909 r
.45842 .55233 .46684 .55958 .48165 .55263 .47329 .54527 Metetra
.718 .79 .909 r
.47329 .54527 .48165 .55263 .49653 .54567 .48823 .53821 Metetra
.718 .789 .908 r
.48823 .53821 .49653 .54567 .51146 .53871 .50322 .53115 Metetra
.717 .787 .907 r
.50322 .53115 .51146 .53871 .52645 .53175 .51828 .52408 Metetra
.716 .786 .907 r
.51828 .52408 .52645 .53175 .5415 .52478 .5334 .517 Metetra
.715 .784 .906 r
.5334 .517 .5415 .52478 .55662 .51782 .54858 .50993 Metetra
.715 .783 .906 r
.54858 .50993 .55662 .51782 .5718 .51086 .56382 .50287 Metetra
.714 .781 .905 r
.56382 .50287 .5718 .51086 .58704 .5039 .57912 .4958 Metetra
.713 .78 .905 r
.57912 .4958 .58704 .5039 .60234 .49694 .59449 .48874 Metetra
.712 .779 .904 r
.59449 .48874 .60234 .49694 .61771 .48999 .60992 .48169 Metetra
.711 .777 .904 r
.60992 .48169 .61771 .48999 .63315 .48305 .62542 .47464 Metetra
.71 .776 .903 r
.62542 .47464 .63315 .48305 .64865 .47612 .64099 .4676 Metetra
.709 .774 .903 r
.64099 .4676 .64865 .47612 .66422 .46919 .65663 .46058 Metetra
.708 .773 .903 r
.65663 .46058 .66422 .46919 .67986 .46228 .67233 .45356 Metetra
.707 .771 .902 r
.67233 .45356 .67986 .46228 .69557 .45537 .68811 .44655 Metetra
.706 .77 .902 r
.68811 .44655 .69557 .45537 .71135 .44847 .70395 .43955 Metetra
.705 .769 .902 r
.70395 .43955 .71135 .44847 .72721 .44159 .71987 .43257 Metetra
.704 .767 .901 r
.71987 .43257 .72721 .44159 .74314 .43471 .73586 .4256 Metetra
.703 .766 .901 r
.73586 .4256 .74314 .43471 .75914 .42785 .75193 .41864 Metetra
.702 .765 .901 r
.75193 .41864 .75914 .42785 .77521 .421 .76807 .41169 Metetra
.701 .764 .9 r
.76807 .41169 .77521 .421 .79137 .41416 .78429 .40476 Metetra
.7 .762 .9 r
.78429 .40476 .79137 .41416 .8076 .40733 .80059 .39784 Metetra
.699 .761 .9 r
.80059 .39784 .8076 .40733 .82391 .40051 .81697 .39093 Metetra
.698 .76 .9 r
.81697 .39093 .82391 .40051 .8403 .3937 .83343 .38403 Metetra
.697 .759 .9 r
.83343 .38403 .8403 .3937 .85678 .38691 .84997 .37715 Metetra
.696 .758 .9 r
.84997 .37715 .85678 .38691 .87333 .38012 .86659 .37027 Metetra
.695 .757 .899 r
.86659 .37027 .87333 .38012 .88997 .37335 .8833 .36341 Metetra
.694 .756 .899 r
.8833 .36341 .88997 .37335 .9067 .36659 .9001 .35656 Metetra
.722 .807 .918 r
.28932 .62262 .29853 .62879 .31279 .62197 .30363 .61571 Metetra
.723 .806 .917 r
.30363 .61571 .31279 .62197 .3271 .61512 .31801 .60876 Metetra
.723 .805 .916 r
.31801 .60876 .3271 .61512 .34146 .60824 .33244 .60178 Metetra
.723 .804 .915 r
.33244 .60178 .34146 .60824 .35589 .60132 .34692 .59477 Metetra
.723 .803 .914 r
.34692 .59477 .35589 .60132 .37036 .59438 .36146 .58773 Metetra
.723 .801 .913 r
.36146 .58773 .37036 .59438 .3849 .58742 .37606 .58066 Metetra
.722 .8 .913 r
.37606 .58066 .3849 .58742 .39949 .58043 .39071 .57357 Metetra
.722 .799 .912 r
.39071 .57357 .39949 .58043 .41413 .57343 .40542 .56646 Metetra
.722 .797 .911 r
.40542 .56646 .41413 .57343 .42884 .56641 .42019 .55934 Metetra
.721 .796 .91 r
.42019 .55934 .42884 .56641 .4436 .55937 .43502 .55219 Metetra
.721 .794 .91 r
.43502 .55219 .4436 .55937 .45842 .55233 .4499 .54504 Metetra
.72 .793 .909 r
.4499 .54504 .45842 .55233 .47329 .54527 .46484 .53788 Metetra
.72 .791 .908 r
.46484 .53788 .47329 .54527 .48823 .53821 .47984 .53071 Metetra
.719 .79 .908 r
.47984 .53071 .48823 .53821 .50322 .53115 .4949 .52353 Metetra
.718 .788 .907 r
.4949 .52353 .50322 .53115 .51828 .52408 .51002 .51635 Metetra
.718 .787 .907 r
.51002 .51635 .51828 .52408 .5334 .517 .5252 .50917 Metetra
.717 .785 .906 r
.5252 .50917 .5334 .517 .54858 .50993 .54045 .50199 Metetra
.716 .784 .905 r
.54045 .50199 .54858 .50993 .56382 .50287 .55575 .49481 Metetra
.715 .782 .905 r
.55575 .49481 .56382 .50287 .57912 .4958 .57112 .48763 Metetra
.714 .781 .904 r
.57112 .48763 .57912 .4958 .59449 .48874 .58655 .48046 Metetra
.713 .779 .904 r
.58655 .48046 .59449 .48874 .60992 .48169 .60205 .4733 Metetra
.712 .778 .903 r
.60205 .4733 .60992 .48169 .62542 .47464 .61762 .46615 Metetra
.711 .776 .903 r
.61762 .46615 .62542 .47464 .64099 .4676 .63325 .459 Metetra
.71 .775 .902 r
.63325 .459 .64099 .4676 .65663 .46058 .64895 .45187 Metetra
.709 .773 .902 r
.64895 .45187 .65663 .46058 .67233 .45356 .66472 .44475 Metetra
.708 .772 .902 r
.66472 .44475 .67233 .45356 .68811 .44655 .68056 .43763 Metetra
.707 .77 .901 r
.68056 .43763 .68811 .44655 .70395 .43955 .69647 .43054 Metetra
.706 .769 .901 r
.69647 .43054 .70395 .43955 .71987 .43257 .71245 .42345 Metetra
.705 .767 .901 r
.71245 .42345 .71987 .43257 .73586 .4256 .72851 .41638 Metetra
.704 .766 .9 r
.72851 .41638 .73586 .4256 .75193 .41864 .74464 .40932 Metetra
.703 .765 .9 r
.74464 .40932 .75193 .41864 .76807 .41169 .76085 .40227 Metetra
.702 .763 .9 r
.76085 .40227 .76807 .41169 .78429 .40476 .77714 .39524 Metetra
.701 .762 .9 r
.77714 .39524 .78429 .40476 .80059 .39784 .7935 .38823 Metetra
.7 .761 .899 r
.7935 .38823 .80059 .39784 .81697 .39093 .80995 .38122 Metetra
.699 .76 .899 r
.80995 .38122 .81697 .39093 .83343 .38403 .82648 .37423 Metetra
.698 .759 .899 r
.82648 .37423 .83343 .38403 .84997 .37715 .84309 .36726 Metetra
.697 .758 .899 r
.84309 .36726 .84997 .37715 .86659 .37027 .85978 .3603 Metetra
.696 .757 .899 r
.85978 .3603 .86659 .37027 .8833 .36341 .87656 .35335 Metetra
.694 .756 .899 r
.87656 .35335 .8833 .36341 .9001 .35656 .89342 .34641 Metetra
.724 .809 .918 r
.27999 .61644 .28932 .62262 .30363 .61571 .29438 .60943 Metetra
.724 .808 .917 r
.29438 .60943 .30363 .61571 .31801 .60876 .30881 .60239 Metetra
.724 .807 .916 r
.30881 .60239 .31801 .60876 .33244 .60178 .32331 .59531 Metetra
.724 .806 .915 r
.32331 .59531 .33244 .60178 .34692 .59477 .33786 .5882 Metetra
.724 .804 .914 r
.33786 .5882 .34692 .59477 .36146 .58773 .35246 .58106 Metetra
.724 .803 .914 r
.35246 .58106 .36146 .58773 .37606 .58066 .36712 .57388 Metetra
.724 .802 .913 r
.36712 .57388 .37606 .58066 .39071 .57357 .38184 .56669 Metetra
.723 .8 .912 r
.38184 .56669 .39071 .57357 .40542 .56646 .39662 .55947 Metetra
.723 .799 .911 r
.39662 .55947 .40542 .56646 .42019 .55934 .41145 .55223 Metetra
.723 .797 .91 r
.41145 .55223 .42019 .55934 .43502 .55219 .42634 .54498 Metetra
.722 .796 .91 r
.42634 .54498 .43502 .55219 .4499 .54504 .44129 .53771 Metetra
.722 .794 .909 r
.44129 .53771 .4499 .54504 .46484 .53788 .4563 .53044 Metetra
.721 .793 .908 r
.4563 .53044 .46484 .53788 .47984 .53071 .47136 .52315 Metetra
.72 .791 .908 r
.47136 .52315 .47984 .53071 .4949 .52353 .48649 .51586 Metetra
.72 .789 .907 r
.48649 .51586 .4949 .52353 .51002 .51635 .50167 .50856 Metetra
.719 .788 .906 r
.50167 .50856 .51002 .51635 .5252 .50917 .51692 .50127 Metetra
.718 .786 .906 r
.51692 .50127 .5252 .50917 .54045 .50199 .53223 .49397 Metetra
.717 .784 .905 r
.53223 .49397 .54045 .50199 .55575 .49481 .5476 .48668 Metetra
.716 .783 .904 r
.5476 .48668 .55575 .49481 .57112 .48763 .56303 .47939 Metetra
.715 .781 .904 r
.56303 .47939 .57112 .48763 .58655 .48046 .57853 .47211 Metetra
.714 .78 .903 r
.57853 .47211 .58655 .48046 .60205 .4733 .5941 .46483 Metetra
.713 .778 .903 r
.5941 .46483 .60205 .4733 .61762 .46615 .60973 .45757 Metetra
.712 .776 .902 r
.60973 .45757 .61762 .46615 .63325 .459 .62542 .45031 Metetra
.711 .775 .902 r
.62542 .45031 .63325 .459 .64895 .45187 .64119 .44307 Metetra
.71 .773 .902 r
.64119 .44307 .64895 .45187 .66472 .44475 .65702 .43584 Metetra
.709 .772 .901 r
.65702 .43584 .66472 .44475 .68056 .43763 .67293 .42862 Metetra
.708 .77 .901 r
.67293 .42862 .68056 .43763 .69647 .43054 .68891 .42141 Metetra
.707 .769 .9 r
.68891 .42141 .69647 .43054 .71245 .42345 .70496 .41422 Metetra
.706 .767 .9 r
.70496 .41422 .71245 .42345 .72851 .41638 .72108 .40704 Metetra
.705 .766 .9 r
.72108 .40704 .72851 .41638 .74464 .40932 .73728 .39988 Metetra
.704 .765 .899 r
.73728 .39988 .74464 .40932 .76085 .40227 .75356 .39274 Metetra
.703 .763 .899 r
.75356 .39274 .76085 .40227 .77714 .39524 .76991 .38561 Metetra
.702 .762 .899 r
.76991 .38561 .77714 .39524 .7935 .38823 .78634 .37849 Metetra
.7 .761 .899 r
.78634 .37849 .7935 .38823 .80995 .38122 .80286 .37139 Metetra
.699 .76 .899 r
.80286 .37139 .80995 .38122 .82648 .37423 .81945 .36431 Metetra
.698 .758 .898 r
.81945 .36431 .82648 .37423 .84309 .36726 .83613 .35724 Metetra
.697 .757 .898 r
.83613 .35724 .84309 .36726 .85978 .3603 .85289 .35019 Metetra
.696 .756 .898 r
.85289 .35019 .85978 .3603 .87656 .35335 .86974 .34315 Metetra
.695 .755 .898 r
.86974 .34315 .87656 .35335 .89342 .34641 .88668 .33612 Metetra
.725 .811 .918 r
.27057 .61026 .27999 .61644 .29438 .60943 .28501 .60316 Metetra
.725 .81 .917 r
.28501 .60316 .29438 .60943 .30881 .60239 .29951 .59602 Metetra
.725 .809 .917 r
.29951 .59602 .30881 .60239 .32331 .59531 .31407 .58884 Metetra
.725 .807 .916 r
.31407 .58884 .32331 .59531 .33786 .5882 .32869 .58162 Metetra
.725 .806 .915 r
.32869 .58162 .33786 .5882 .35246 .58106 .34336 .57437 Metetra
.725 .805 .914 r
.34336 .57437 .35246 .58106 .36712 .57388 .35809 .56709 Metetra
.725 .803 .913 r
.35809 .56709 .36712 .57388 .38184 .56669 .37287 .55978 Metetra
.725 .802 .912 r
.37287 .55978 .38184 .56669 .39662 .55947 .38772 .55245 Metetra
.724 .8 .911 r
.38772 .55245 .39662 .55947 .41145 .55223 .40262 .5451 Metetra
.724 .799 .91 r
.40262 .5451 .41145 .55223 .42634 .54498 .41757 .53773 Metetra
.724 .797 .91 r
.41757 .53773 .42634 .54498 .44129 .53771 .43259 .53035 Metetra
.723 .795 .909 r
.43259 .53035 .44129 .53771 .4563 .53044 .44766 .52295 Metetra
.722 .794 .908 r
.44766 .52295 .4563 .53044 .47136 .52315 .46279 .51555 Metetra
.722 .792 .907 r
.46279 .51555 .47136 .52315 .48649 .51586 .47798 .50814 Metetra
.721 .79 .907 r
.47798 .50814 .48649 .51586 .50167 .50856 .49324 .50072 Metetra
.72 .789 .906 r
.49324 .50072 .50167 .50856 .51692 .50127 .50855 .49331 Metetra
.72 .787 .905 r
.50855 .49331 .51692 .50127 .53223 .49397 .52392 .48589 Metetra
.719 .785 .905 r
.52392 .48589 .53223 .49397 .5476 .48668 .53936 .47848 Metetra
.718 .784 .904 r
.53936 .47848 .5476 .48668 .56303 .47939 .55486 .47108 Metetra
.717 .782 .903 r
.55486 .47108 .56303 .47939 .57853 .47211 .57043 .46368 Metetra
.716 .78 .903 r
.57043 .46368 .57853 .47211 .5941 .46483 .58606 .45628 Metetra
.715 .779 .902 r
.58606 .45628 .5941 .46483 .60973 .45757 .60175 .4489 Metetra
.714 .777 .902 r
.60175 .4489 .60973 .45757 .62542 .45031 .61751 .44153 Metetra
.713 .775 .901 r
.61751 .44153 .62542 .45031 .64119 .44307 .63335 .43417 Metetra
.712 .774 .901 r
.63335 .43417 .64119 .44307 .65702 .43584 .64925 .42683 Metetra
.71 .772 .901 r
.64925 .42683 .65702 .43584 .67293 .42862 .66522 .4195 Metetra
.709 .771 .9 r
.66522 .4195 .67293 .42862 .68891 .42141 .68126 .41218 Metetra
.708 .769 .9 r
.68126 .41218 .68891 .42141 .70496 .41422 .69738 .40488 Metetra
.707 .768 .899 r
.69738 .40488 .70496 .41422 .72108 .40704 .71357 .3976 Metetra
.706 .766 .899 r
.71357 .3976 .72108 .40704 .73728 .39988 .72984 .39033 Metetra
.705 .765 .899 r
.72984 .39033 .73728 .39988 .75356 .39274 .74618 .38308 Metetra
.703 .763 .899 r
.74618 .38308 .75356 .39274 .76991 .38561 .7626 .37585 Metetra
.702 .762 .898 r
.7626 .37585 .76991 .38561 .78634 .37849 .7791 .36863 Metetra
.701 .761 .898 r
.7791 .36863 .78634 .37849 .80286 .37139 .79568 .36143 Metetra
.7 .759 .898 r
.79568 .36143 .80286 .37139 .81945 .36431 .81235 .35425 Metetra
.699 .758 .898 r
.81235 .35425 .81945 .36431 .83613 .35724 .82909 .34709 Metetra
.698 .757 .898 r
.82909 .34709 .83613 .35724 .85289 .35019 .84593 .33994 Metetra
.697 .756 .897 r
.84593 .33994 .85289 .35019 .86974 .34315 .86285 .33281 Metetra
.696 .755 .897 r
.86285 .33281 .86974 .34315 .88668 .33612 .87985 .32569 Metetra
.726 .813 .919 r
.26103 .60408 .27057 .61026 .28501 .60316 .27554 .59689 Metetra
.726 .812 .918 r
.27554 .59689 .28501 .60316 .29951 .59602 .29011 .58964 Metetra
.726 .811 .917 r
.29011 .58964 .29951 .59602 .31407 .58884 .30473 .58236 Metetra
.727 .809 .916 r
.30473 .58236 .31407 .58884 .32869 .58162 .31941 .57504 Metetra
.727 .808 .915 r
.31941 .57504 .32869 .58162 .34336 .57437 .33415 .56767 Metetra
.727 .806 .914 r
.33415 .56767 .34336 .57437 .35809 .56709 .34895 .56028 Metetra
.727 .805 .913 r
.34895 .56028 .35809 .56709 .37287 .55978 .3638 .55286 Metetra
.726 .803 .912 r
.3638 .55286 .37287 .55978 .38772 .55245 .37871 .54541 Metetra
.726 .802 .911 r
.37871 .54541 .38772 .55245 .40262 .5451 .39368 .53794 Metetra
.726 .8 .91 r
.39368 .53794 .40262 .5451 .41757 .53773 .4087 .53045 Metetra
.725 .799 .91 r
.4087 .53045 .41757 .53773 .43259 .53035 .42379 .52295 Metetra
.725 .797 .909 r
.42379 .52295 .43259 .53035 .44766 .52295 .43893 .51543 Metetra
.724 .795 .908 r
.43893 .51543 .44766 .52295 .46279 .51555 .45413 .5079 Metetra
.723 .793 .907 r
.45413 .5079 .46279 .51555 .47798 .50814 .46939 .50037 Metetra
.723 .792 .906 r
.46939 .50037 .47798 .50814 .49324 .50072 .48471 .49283 Metetra
.722 .79 .906 r
.48471 .49283 .49324 .50072 .50855 .49331 .50009 .48529 Metetra
.721 .788 .905 r
.50009 .48529 .50855 .49331 .52392 .48589 .51553 .47775 Metetra
.72 .786 .904 r
.51553 .47775 .52392 .48589 .53936 .47848 .53103 .47022 Metetra
.719 .784 .904 r
.53103 .47022 .53936 .47848 .55486 .47108 .5466 .46269 Metetra
.718 .783 .903 r
.5466 .46269 .55486 .47108 .57043 .46368 .56223 .45516 Metetra
.717 .781 .902 r
.56223 .45516 .57043 .46368 .58606 .45628 .57793 .44765 Metetra
.716 .779 .902 r
.57793 .44765 .58606 .45628 .60175 .4489 .59369 .44015 Metetra
.715 .777 .901 r
.59369 .44015 .60175 .4489 .61751 .44153 .60952 .43266 Metetra
.714 .776 .901 r
.60952 .43266 .61751 .44153 .63335 .43417 .62542 .42518 Metetra
.713 .774 .9 r
.62542 .42518 .63335 .43417 .64925 .42683 .64139 .41772 Metetra
.712 .772 .9 r
.64139 .41772 .64925 .42683 .66522 .4195 .65742 .41027 Metetra
.71 .771 .899 r
.65742 .41027 .66522 .4195 .68126 .41218 .67354 .40284 Metetra
.709 .769 .899 r
.67354 .40284 .68126 .41218 .69738 .40488 .68972 .39543 Metetra
.708 .768 .899 r
.68972 .39543 .69738 .40488 .71357 .3976 .70598 .38803 Metetra
.707 .766 .898 r
.70598 .38803 .71357 .3976 .72984 .39033 .72231 .38066 Metetra
.706 .765 .898 r
.72231 .38066 .72984 .39033 .74618 .38308 .73872 .3733 Metetra
.704 .763 .898 r
.73872 .3733 .74618 .38308 .7626 .37585 .75521 .36596 Metetra
.703 .762 .898 r
.75521 .36596 .7626 .37585 .7791 .36863 .77178 .35864 Metetra
.702 .76 .897 r
.77178 .35864 .7791 .36863 .79568 .36143 .78843 .35134 Metetra
.701 .759 .897 r
.78843 .35134 .79568 .36143 .81235 .35425 .80516 .34406 Metetra
.7 .758 .897 r
.80516 .34406 .81235 .35425 .82909 .34709 .82198 .33679 Metetra
.698 .756 .897 r
.82198 .33679 .82909 .34709 .84593 .33994 .83888 .32955 Metetra
.697 .755 .897 r
.83888 .32955 .84593 .33994 .86285 .33281 .85587 .32232 Metetra
.696 .754 .897 r
.85587 .32232 .86285 .33281 .87985 .32569 .87295 .31511 Metetra
.727 .815 .919 r
.25137 .59791 .26103 .60408 .27554 .59689 .26595 .59062 Metetra
.727 .814 .918 r
.26595 .59062 .27554 .59689 .29011 .58964 .28059 .58328 Metetra
.728 .813 .917 r
.28059 .58328 .29011 .58964 .30473 .58236 .29528 .57588 Metetra
.728 .811 .916 r
.29528 .57588 .30473 .58236 .31941 .57504 .31003 .56845 Metetra
.728 .81 .915 r
.31003 .56845 .31941 .57504 .33415 .56767 .32484 .56098 Metetra
.728 .808 .914 r
.32484 .56098 .33415 .56767 .34895 .56028 .3397 .55347 Metetra
.728 .807 .913 r
.3397 .55347 .34895 .56028 .3638 .55286 .35462 .54592 Metetra
.728 .805 .912 r
.35462 .54592 .3638 .55286 .37871 .54541 .3696 .53835 Metetra
.728 .804 .911 r
.3696 .53835 .37871 .54541 .39368 .53794 .38464 .53076 Metetra
.727 .802 .911 r
.38464 .53076 .39368 .53794 .4087 .53045 .39973 .52315 Metetra
.727 .8 .91 r
.39973 .52315 .4087 .53045 .42379 .52295 .41488 .51552 Metetra
.726 .798 .909 r
.41488 .51552 .42379 .52295 .43893 .51543 .43009 .50787 Metetra
.726 .797 .908 r
.43009 .50787 .43893 .51543 .45413 .5079 .44536 .50022 Metetra
.725 .795 .907 r
.44536 .50022 .45413 .5079 .46939 .50037 .46069 .49255 Metetra
.724 .793 .906 r
.46069 .49255 .46939 .50037 .48471 .49283 .47608 .48488 Metetra
.723 .791 .905 r
.47608 .48488 .48471 .49283 .50009 .48529 .49153 .47722 Metetra
.723 .789 .905 r
.49153 .47722 .50009 .48529 .51553 .47775 .50704 .46955 Metetra
.722 .787 .904 r
.50704 .46955 .51553 .47775 .53103 .47022 .52261 .46188 Metetra
.721 .785 .903 r
.52261 .46188 .53103 .47022 .5466 .46269 .53825 .45422 Metetra
.72 .783 .903 r
.53825 .45422 .5466 .46269 .56223 .45516 .55395 .44657 Metetra
.719 .782 .902 r
.55395 .44657 .56223 .45516 .57793 .44765 .56971 .43893 Metetra
.717 .78 .901 r
.56971 .43893 .57793 .44765 .59369 .44015 .58554 .4313 Metetra
.716 .778 .901 r
.58554 .4313 .59369 .44015 .60952 .43266 .60144 .42369 Metetra
.715 .776 .9 r
.60144 .42369 .60952 .43266 .62542 .42518 .6174 .41608 Metetra
.714 .774 .9 r
.6174 .41608 .62542 .42518 .64139 .41772 .63344 .4085 Metetra
.713 .773 .899 r
.63344 .4085 .64139 .41772 .65742 .41027 .64955 .40093 Metetra
.712 .771 .899 r
.64955 .40093 .65742 .41027 .67354 .40284 .66572 .39338 Metetra
.71 .769 .898 r
.66572 .39338 .67354 .40284 .68972 .39543 .68198 .38585 Metetra
.709 .767 .898 r
.68198 .38585 .68972 .39543 .70598 .38803 .6983 .37834 Metetra
.708 .766 .898 r
.6983 .37834 .70598 .38803 .72231 .38066 .7147 .37085 Metetra
.706 .764 .897 r
.7147 .37085 .72231 .38066 .73872 .3733 .73118 .36339 Metetra
.705 .763 .897 r
.73118 .36339 .73872 .3733 .75521 .36596 .74774 .35594 Metetra
.704 .761 .897 r
.74774 .35594 .75521 .36596 .77178 .35864 .76438 .34851 Metetra
.703 .76 .896 r
.76438 .34851 .77178 .35864 .78843 .35134 .7811 .34111 Metetra
.701 .759 .896 r
.7811 .34111 .78843 .35134 .80516 .34406 .7979 .33372 Metetra
.7 .757 .896 r
.7979 .33372 .80516 .34406 .82198 .33679 .81479 .32636 Metetra
.699 .756 .896 r
.81479 .32636 .82198 .33679 .83888 .32955 .83176 .31901 Metetra
.698 .755 .896 r
.83176 .31901 .83888 .32955 .85587 .32232 .84882 .31169 Metetra
.697 .754 .896 r
.84882 .31169 .85587 .32232 .87295 .31511 .86597 .30438 Metetra
.728 .817 .92 r
.2416 .59177 .25137 .59791 .26595 .59062 .25625 .58437 Metetra
.729 .816 .919 r
.25625 .58437 .26595 .59062 .28059 .58328 .27095 .57692 Metetra
.729 .815 .918 r
.27095 .57692 .28059 .58328 .29528 .57588 .28572 .56942 Metetra
.73 .813 .917 r
.28572 .56942 .29528 .57588 .31003 .56845 .30054 .56187 Metetra
.73 .812 .916 r
.30054 .56187 .31003 .56845 .32484 .56098 .31541 .55428 Metetra
.73 .81 .915 r
.31541 .55428 .32484 .56098 .3397 .55347 .33035 .54665 Metetra
.73 .809 .914 r
.33035 .54665 .3397 .55347 .35462 .54592 .34534 .53898 Metetra
.73 .807 .913 r
.34534 .53898 .35462 .54592 .3696 .53835 .36039 .53129 Metetra
.729 .805 .912 r
.36039 .53129 .3696 .53835 .38464 .53076 .3755 .52357 Metetra
.729 .804 .911 r
.3755 .52357 .38464 .53076 .39973 .52315 .39066 .51582 Metetra
.729 .802 .91 r
.39066 .51582 .39973 .52315 .41488 .51552 .40588 .50806 Metetra
.728 .8 .909 r
.40588 .50806 .41488 .51552 .43009 .50787 .42116 .50028 Metetra
.727 .798 .908 r
.42116 .50028 .43009 .50787 .44536 .50022 .4365 .49249 Metetra
.727 .796 .907 r
.4365 .49249 .44536 .50022 .46069 .49255 .4519 .48469 Metetra
.726 .794 .906 r
.4519 .48469 .46069 .49255 .47608 .48488 .46736 .47689 Metetra
.725 .792 .905 r
.46736 .47689 .47608 .48488 .49153 .47722 .48288 .46908 Metetra
.724 .79 .904 r
.48288 .46908 .49153 .47722 .50704 .46955 .49846 .46128 Metetra
.723 .788 .904 r
.49846 .46128 .50704 .46955 .52261 .46188 .5141 .45348 Metetra
.722 .786 .903 r
.5141 .45348 .52261 .46188 .53825 .45422 .5298 .44568 Metetra
.721 .784 .902 r
.5298 .44568 .53825 .45422 .55395 .44657 .54557 .4379 Metetra
.72 .782 .901 r
.54557 .4379 .55395 .44657 .56971 .43893 .5614 .43013 Metetra
.719 .78 .901 r
.5614 .43013 .56971 .43893 .58554 .4313 .5773 .42236 Metetra
.718 .778 .9 r
.5773 .42236 .58554 .4313 .60144 .42369 .59327 .41462 Metetra
.717 .776 .9 r
.59327 .41462 .60144 .42369 .6174 .41608 .6093 .40689 Metetra
.715 .775 .899 r
.6093 .40689 .6174 .41608 .63344 .4085 .62541 .39918 Metetra
.714 .773 .898 r
.62541 .39918 .63344 .4085 .64955 .40093 .64158 .39148 Metetra
.713 .771 .898 r
.64158 .39148 .64955 .40093 .66572 .39338 .65783 .38381 Metetra
.711 .769 .898 r
.65783 .38381 .66572 .39338 .68198 .38585 .67415 .37616 Metetra
.71 .767 .897 r
.67415 .37616 .68198 .38585 .6983 .37834 .69054 .36853 Metetra
.709 .766 .897 r
.69054 .36853 .6983 .37834 .7147 .37085 .70701 .36092 Metetra
.707 .764 .896 r
.70701 .36092 .7147 .37085 .73118 .36339 .72356 .35334 Metetra
.706 .763 .896 r
.72356 .35334 .73118 .36339 .74774 .35594 .74019 .34578 Metetra
.705 .761 .896 r
.74019 .34578 .74774 .35594 .76438 .34851 .75689 .33824 Metetra
.704 .76 .896 r
.75689 .33824 .76438 .34851 .7811 .34111 .77368 .33073 Metetra
.702 .758 .895 r
.77368 .33073 .7811 .34111 .7979 .33372 .79056 .32324 Metetra
.701 .757 .895 r
.79056 .32324 .7979 .33372 .81479 .32636 .80751 .31577 Metetra
.7 .755 .895 r
.80751 .31577 .81479 .32636 .83176 .31901 .82456 .30832 Metetra
.698 .754 .895 r
.82456 .30832 .83176 .31901 .84882 .31169 .84169 .3009 Metetra
.697 .753 .895 r
.84169 .3009 .84882 .31169 .86597 .30438 .85891 .29349 Metetra
.73 .82 .921 r
.23171 .58564 .2416 .59177 .25625 .58437 .24643 .57814 Metetra
.73 .819 .92 r
.24643 .57814 .25625 .58437 .27095 .57692 .2612 .57058 Metetra
.731 .817 .918 r
.2612 .57058 .27095 .57692 .28572 .56942 .27604 .56297 Metetra
.731 .816 .917 r
.27604 .56297 .28572 .56942 .30054 .56187 .29093 .5553 Metetra
.731 .814 .916 r
.29093 .5553 .30054 .56187 .31541 .55428 .30588 .54759 Metetra
.731 .813 .915 r
.30588 .54759 .31541 .55428 .33035 .54665 .32088 .53983 Metetra
.731 .811 .914 r
.32088 .53983 .33035 .54665 .34534 .53898 .33595 .53204 Metetra
.731 .809 .913 r
.33595 .53204 .34534 .53898 .36039 .53129 .35107 .52422 Metetra
.731 .808 .912 r
.35107 .52422 .36039 .53129 .3755 .52357 .36625 .51636 Metetra
.731 .806 .911 r
.36625 .51636 .3755 .52357 .39066 .51582 .38148 .50848 Metetra
.73 .804 .91 r
.38148 .50848 .39066 .51582 .40588 .50806 .39678 .50058 Metetra
.73 .802 .909 r
.39678 .50058 .40588 .50806 .42116 .50028 .41213 .49266 Metetra
.729 .8 .908 r
.41213 .49266 .42116 .50028 .4365 .49249 .42754 .48473 Metetra
.729 .798 .907 r
.42754 .48473 .4365 .49249 .4519 .48469 .44301 .47679 Metetra
.728 .796 .906 r
.44301 .47679 .4519 .48469 .46736 .47689 .45854 .46884 Metetra
.727 .793 .905 r
.45854 .46884 .46736 .47689 .48288 .46908 .47413 .4609 Metetra
.726 .791 .904 r
.47413 .4609 .48288 .46908 .49846 .46128 .48978 .45295 Metetra
.725 .789 .903 r
.48978 .45295 .49846 .46128 .5141 .45348 .50549 .44501 Metetra
.724 .787 .902 r
.50549 .44501 .5141 .45348 .5298 .44568 .52126 .43707 Metetra
.723 .785 .901 r
.52126 .43707 .5298 .44568 .54557 .4379 .5371 .42915 Metetra
.722 .783 .901 r
.5371 .42915 .54557 .4379 .5614 .43013 .553 .42123 Metetra
.721 .781 .9 r
.553 .42123 .5614 .43013 .5773 .42236 .56897 .41333 Metetra
.719 .779 .899 r
.56897 .41333 .5773 .42236 .59327 .41462 .58501 .40545 Metetra
.718 .777 .899 r
.58501 .40545 .59327 .41462 .6093 .40689 .60111 .39758 Metetra
.717 .775 .898 r
.60111 .39758 .6093 .40689 .62541 .39918 .61728 .38974 Metetra
.715 .773 .898 r
.61728 .38974 .62541 .39918 .64158 .39148 .63353 .38191 Metetra
.714 .771 .897 r
.63353 .38191 .64158 .39148 .65783 .38381 .64984 .37411 Metetra
.713 .769 .897 r
.64984 .37411 .65783 .38381 .67415 .37616 .66623 .36634 Metetra
.711 .767 .896 r
.66623 .36634 .67415 .37616 .69054 .36853 .68269 .35858 Metetra
.71 .766 .896 r
.68269 .35858 .69054 .36853 .70701 .36092 .69923 .35085 Metetra
.709 .764 .895 r
.69923 .35085 .70701 .36092 .72356 .35334 .71585 .34315 Metetra
.707 .762 .895 r
.71585 .34315 .72356 .35334 .74019 .34578 .73255 .33547 Metetra
.706 .761 .895 r
.73255 .33547 .74019 .34578 .75689 .33824 .74932 .32782 Metetra
.704 .759 .895 r
.74932 .32782 .75689 .33824 .77368 .33073 .76618 .32019 Metetra
.703 .758 .894 r
.76618 .32019 .77368 .33073 .79056 .32324 .78313 .31259 Metetra
.702 .756 .894 r
.78313 .31259 .79056 .32324 .80751 .31577 .80016 .30502 Metetra
.7 .755 .894 r
.80016 .30502 .80751 .31577 .82456 .30832 .81727 .29747 Metetra
.699 .753 .894 r
.81727 .29747 .82456 .30832 .84169 .3009 .83447 .28994 Metetra
.698 .752 .894 r
.83447 .28994 .84169 .3009 .85891 .29349 .85177 .28244 Metetra
.731 .822 .922 r
.2217 .57955 .23171 .58564 .24643 .57814 .23648 .57195 Metetra
.732 .821 .92 r
.23648 .57195 .24643 .57814 .2612 .57058 .25133 .56428 Metetra
.732 .82 .919 r
.25133 .56428 .2612 .57058 .27604 .56297 .26624 .55655 Metetra
.733 .818 .918 r
.26624 .55655 .27604 .56297 .29093 .5553 .2812 .54876 Metetra
.733 .817 .917 r
.2812 .54876 .29093 .5553 .30588 .54759 .29622 .54092 Metetra
.733 .815 .915 r
.29622 .54092 .30588 .54759 .32088 .53983 .3113 .53304 Metetra
.733 .814 .914 r
.3113 .53304 .32088 .53983 .33595 .53204 .32644 .52511 Metetra
.733 .812 .913 r
.32644 .52511 .33595 .53204 .35107 .52422 .34163 .51715 Metetra
.733 .81 .912 r
.34163 .51715 .35107 .52422 .36625 .51636 .35689 .50915 Metetra
.733 .808 .911 r
.35689 .50915 .36625 .51636 .38148 .50848 .3722 .50113 Metetra
.732 .806 .91 r
.3722 .50113 .38148 .50848 .39678 .50058 .38756 .49308 Metetra
.732 .804 .909 r
.38756 .49308 .39678 .50058 .41213 .49266 .40299 .48502 Metetra
.731 .802 .907 r
.40299 .48502 .41213 .49266 .42754 .48473 .41847 .47694 Metetra
.731 .799 .906 r
.41847 .47694 .42754 .48473 .44301 .47679 .43401 .46885 Metetra
.73 .797 .905 r
.43401 .46885 .44301 .47679 .45854 .46884 .44962 .46075 Metetra
.729 .795 .904 r
.44962 .46075 .45854 .46884 .47413 .4609 .46528 .45266 Metetra
.728 .793 .904 r
.46528 .45266 .47413 .4609 .48978 .45295 .481 .44456 Metetra
.727 .79 .903 r
.481 .44456 .48978 .45295 .50549 .44501 .49678 .43647 Metetra
.726 .788 .902 r
.49678 .43647 .50549 .44501 .52126 .43707 .51263 .42838 Metetra
.725 .786 .901 r
.51263 .42838 .52126 .43707 .5371 .42915 .52854 .42031 Metetra
.724 .784 .9 r
.52854 .42031 .5371 .42915 .553 .42123 .54451 .41225 Metetra
.722 .782 .899 r
.54451 .41225 .553 .42123 .56897 .41333 .56055 .4042 Metetra
.721 .779 .899 r
.56055 .4042 .56897 .41333 .58501 .40545 .57666 .39618 Metetra
.72 .777 .898 r
.57666 .39618 .58501 .40545 .60111 .39758 .59283 .38817 Metetra
.718 .775 .897 r
.59283 .38817 .60111 .39758 .61728 .38974 .60907 .38019 Metetra
.717 .773 .897 r
.60907 .38019 .61728 .38974 .63353 .38191 .62539 .37222 Metetra
.715 .771 .896 r
.62539 .37222 .63353 .38191 .64984 .37411 .64177 .36429 Metetra
.714 .769 .896 r
.64177 .36429 .64984 .37411 .66623 .36634 .65823 .35638 Metetra
.713 .767 .895 r
.65823 .35638 .66623 .36634 .68269 .35858 .67476 .34849 Metetra
.711 .765 .895 r
.67476 .34849 .68269 .35858 .69923 .35085 .69137 .34064 Metetra
.71 .763 .894 r
.69137 .34064 .69923 .35085 .71585 .34315 .70806 .33281 Metetra
.708 .762 .894 r
.70806 .33281 .71585 .34315 .73255 .33547 .72482 .32501 Metetra
.707 .76 .894 r
.72482 .32501 .73255 .33547 .74932 .32782 .74167 .31724 Metetra
.705 .758 .894 r
.74167 .31724 .74932 .32782 .76618 .32019 .7586 .3095 Metetra
.704 .757 .893 r
.7586 .3095 .76618 .32019 .78313 .31259 .77561 .30179 Metetra
.702 .755 .893 r
.77561 .30179 .78313 .31259 .80016 .30502 .79271 .2941 Metetra
.701 .754 .893 r
.79271 .2941 .80016 .30502 .81727 .29747 .8099 .28645 Metetra
.699 .753 .893 r
.8099 .28645 .81727 .29747 .83447 .28994 .82718 .27882 Metetra
.698 .751 .893 r
.82718 .27882 .83447 .28994 .85177 .28244 .84454 .27121 Metetra
.732 .825 .923 r
.21155 .57351 .2217 .57955 .23648 .57195 .22641 .56579 Metetra
.733 .824 .921 r
.22641 .56579 .23648 .57195 .25133 .56428 .24133 .55801 Metetra
.734 .823 .92 r
.24133 .55801 .25133 .56428 .26624 .55655 .25631 .55016 Metetra
.734 .821 .919 r
.25631 .55016 .26624 .55655 .2812 .54876 .27135 .54225 Metetra
.735 .82 .917 r
.27135 .54225 .2812 .54876 .29622 .54092 .28645 .53428 Metetra
.735 .818 .916 r
.28645 .53428 .29622 .54092 .3113 .53304 .3016 .52626 Metetra
.735 .816 .915 r
.3016 .52626 .3113 .53304 .32644 .52511 .31681 .51819 Metetra
.735 .814 .913 r
.31681 .51819 .32644 .52511 .34163 .51715 .33208 .51009 Metetra
.735 .812 .912 r
.33208 .51009 .34163 .51715 .35689 .50915 .34741 .50195 Metetra
.735 .81 .911 r
.34741 .50195 .35689 .50915 .3722 .50113 .3628 .49377 Metetra
.735 .808 .91 r
.3628 .49377 .3722 .50113 .38756 .49308 .37824 .48558 Metetra
.734 .806 .909 r
.37824 .48558 .38756 .49308 .40299 .48502 .39374 .47736 Metetra
.733 .804 .907 r
.39374 .47736 .40299 .48502 .41847 .47694 .4093 .46912 Metetra
.733 .801 .906 r
.4093 .46912 .41847 .47694 .43401 .46885 .42492 .46088 Metetra
.732 .799 .905 r
.42492 .46088 .43401 .46885 .44962 .46075 .44059 .45262 Metetra
.731 .796 .904 r
.44059 .45262 .44962 .46075 .46528 .45266 .45633 .44437 Metetra
.73 .794 .903 r
.45633 .44437 .46528 .45266 .481 .44456 .47212 .43611 Metetra
.729 .792 .902 r
.47212 .43611 .481 .44456 .49678 .43647 .48798 .42786 Metetra
.728 .789 .901 r
.48798 .42786 .49678 .43647 .51263 .42838 .5039 .41962 Metetra
.727 .787 .9 r
.5039 .41962 .51263 .42838 .52854 .42031 .51988 .41139 Metetra
.725 .785 .899 r
.51988 .41139 .52854 .42031 .54451 .41225 .53593 .40318 Metetra
.724 .782 .899 r
.53593 .40318 .54451 .41225 .56055 .4042 .55204 .39498 Metetra
.723 .78 .898 r
.55204 .39498 .56055 .4042 .57666 .39618 .56821 .3868 Metetra
.721 .778 .897 r
.56821 .3868 .57666 .39618 .59283 .38817 .58446 .37864 Metetra
.72 .775 .896 r
.58446 .37864 .59283 .38817 .60907 .38019 .60077 .37051 Metetra
.718 .773 .896 r
.60077 .37051 .60907 .38019 .62539 .37222 .61715 .36241 Metetra
.717 .771 .895 r
.61715 .36241 .62539 .37222 .64177 .36429 .63361 .35433 Metetra
.715 .769 .895 r
.63361 .35433 .64177 .36429 .65823 .35638 .65014 .34628 Metetra
.714 .767 .894 r
.65014 .34628 .65823 .35638 .67476 .34849 .66674 .33826 Metetra
.712 .765 .894 r
.66674 .33826 .67476 .34849 .69137 .34064 .68342 .33027 Metetra
.711 .763 .893 r
.68342 .33027 .69137 .34064 .70806 .33281 .70017 .32232 Metetra
.709 .761 .893 r
.70017 .32232 .70806 .33281 .72482 .32501 .71701 .31439 Metetra
.708 .759 .893 r
.71701 .31439 .72482 .32501 .74167 .31724 .73393 .3065 Metetra
.706 .758 .892 r
.73393 .3065 .74167 .31724 .7586 .3095 .75093 .29864 Metetra
.705 .756 .892 r
.75093 .29864 .7586 .3095 .77561 .30179 .76801 .29081 Metetra
.703 .754 .892 r
.76801 .29081 .77561 .30179 .79271 .2941 .78518 .28301 Metetra
.702 .753 .892 r
.78518 .28301 .79271 .2941 .8099 .28645 .80244 .27525 Metetra
.7 .752 .892 r
.80244 .27525 .8099 .28645 .82718 .27882 .81979 .26751 Metetra
.699 .75 .892 r
.81979 .26751 .82718 .27882 .84454 .27121 .83723 .2598 Metetra
.734 .829 .924 r
.20128 .56752 .21155 .57351 .22641 .56579 .21621 .5597 Metetra
.735 .827 .922 r
.21621 .5597 .22641 .56579 .24133 .55801 .23121 .5518 Metetra
.735 .826 .921 r
.23121 .5518 .24133 .55801 .25631 .55016 .24626 .54382 Metetra
.736 .825 .919 r
.24626 .54382 .25631 .55016 .27135 .54225 .26138 .53578 Metetra
.737 .823 .918 r
.26138 .53578 .27135 .54225 .28645 .53428 .27655 .52768 Metetra
.737 .821 .917 r
.27655 .52768 .28645 .53428 .3016 .52626 .29178 .51952 Metetra
.737 .819 .915 r
.29178 .51952 .3016 .52626 .31681 .51819 .30707 .51131 Metetra
.737 .817 .914 r
.30707 .51131 .31681 .51819 .33208 .51009 .32242 .50305 Metetra
.737 .815 .912 r
.32242 .50305 .33208 .51009 .34741 .50195 .33782 .49476 Metetra
.737 .813 .911 r
.33782 .49476 .34741 .50195 .3628 .49377 .35329 .48643 Metetra
.737 .81 .91 r
.35329 .48643 .3628 .49377 .37824 .48558 .36881 .47807 Metetra
.736 .808 .909 r
.36881 .47807 .37824 .48558 .39374 .47736 .38438 .46969 Metetra
.736 .806 .907 r
.38438 .46969 .39374 .47736 .4093 .46912 .40002 .46129 Metetra
.735 .803 .906 r
.40002 .46129 .4093 .46912 .42492 .46088 .41571 .45288 Metetra
.734 .801 .905 r
.41571 .45288 .42492 .46088 .44059 .45262 .43146 .44446 Metetra
.733 .798 .904 r
.43146 .44446 .44059 .45262 .45633 .44437 .44727 .43603 Metetra
.732 .796 .903 r
.44727 .43603 .45633 .44437 .47212 .43611 .46314 .42761 Metetra
.731 .793 .902 r
.46314 .42761 .47212 .43611 .48798 .42786 .47908 .41919 Metetra
.73 .79 .9 r
.47908 .41919 .48798 .42786 .5039 .41962 .49507 .41079 Metetra
.729 .788 .9 r
.49507 .41079 .5039 .41962 .51988 .41139 .51112 .40239 Metetra
.727 .785 .899 r
.51112 .40239 .51988 .41139 .53593 .40318 .52724 .39401 Metetra
.726 .783 .898 r
.52724 .39401 .53593 .40318 .55204 .39498 .54342 .38565 Metetra
.725 .78 .897 r
.54342 .38565 .55204 .39498 .56821 .3868 .55967 .37731 Metetra
.723 .778 .896 r
.55967 .37731 .56821 .3868 .58446 .37864 .57599 .369 Metetra
.722 .776 .895 r
.57599 .369 .58446 .37864 .60077 .37051 .59237 .36071 Metetra
.72 .773 .895 r
.59237 .36071 .60077 .37051 .61715 .36241 .60883 .35246 Metetra
.718 .771 .894 r
.60883 .35246 .61715 .36241 .63361 .35433 .62535 .34423 Metetra
.717 .769 .893 r
.62535 .34423 .63361 .35433 .65014 .34628 .64195 .33604 Metetra
.715 .767 .893 r
.64195 .33604 .65014 .34628 .66674 .33826 .65863 .32788 Metetra
.713 .764 .892 r
.65863 .32788 .66674 .33826 .68342 .33027 .67538 .31975 Metetra
.712 .762 .892 r
.67538 .31975 .68342 .33027 .70017 .32232 .6922 .31166 Metetra
.71 .76 .892 r
.6922 .31166 .70017 .32232 .71701 .31439 .70911 .3036 Metetra
.709 .759 .891 r
.70911 .3036 .71701 .31439 .73393 .3065 .7261 .29558 Metetra
.707 .757 .891 r
.7261 .29558 .73393 .3065 .75093 .29864 .74317 .2876 Metetra
.705 .755 .891 r
.74317 .2876 .75093 .29864 .76801 .29081 .76033 .27965 Metetra
.704 .753 .891 r
.76033 .27965 .76801 .29081 .78518 .28301 .77757 .27174 Metetra
.702 .752 .89 r
.77757 .27174 .78518 .28301 .80244 .27525 .7949 .26386 Metetra
.701 .75 .89 r
.7949 .26386 .80244 .27525 .81979 .26751 .81232 .25601 Metetra
.699 .749 .89 r
.81232 .25601 .81979 .26751 .83723 .2598 .82984 .2482 Metetra
.735 .832 .925 r
.19087 .5616 .20128 .56752 .21621 .5597 .20587 .55367 Metetra
.736 .831 .923 r
.20587 .55367 .21621 .5597 .23121 .5518 .22095 .54565 Metetra
.737 .83 .922 r
.22095 .54565 .23121 .5518 .24626 .54382 .23608 .53755 Metetra
.738 .828 .92 r
.23608 .53755 .24626 .54382 .26138 .53578 .25127 .52938 Metetra
.739 .826 .919 r
.25127 .52938 .26138 .53578 .27655 .52768 .26652 .52113 Metetra
.739 .824 .917 r
.26652 .52113 .27655 .52768 .29178 .51952 .28183 .51283 Metetra
.739 .822 .916 r
.28183 .51283 .29178 .51952 .30707 .51131 .2972 .50447 Metetra
.74 .82 .914 r
.2972 .50447 .30707 .51131 .32242 .50305 .31263 .49605 Metetra
.74 .818 .913 r
.31263 .49605 .32242 .50305 .33782 .49476 .32811 .4876 Metetra
.74 .816 .911 r
.32811 .4876 .33782 .49476 .35329 .48643 .34366 .4791 Metetra
.739 .813 .91 r
.34366 .4791 .35329 .48643 .36881 .47807 .35926 .47057 Metetra
.739 .811 .909 r
.35926 .47057 .36881 .47807 .38438 .46969 .37491 .46202 Metetra
.738 .808 .907 r
.37491 .46202 .38438 .46969 .40002 .46129 .39063 .45345 Metetra
.738 .805 .906 r
.39063 .45345 .40002 .46129 .41571 .45288 .4064 .44486 Metetra
.737 .803 .905 r
.4064 .44486 .41571 .45288 .43146 .44446 .42223 .43626 Metetra
.736 .8 .903 r
.42223 .43626 .43146 .44446 .44727 .43603 .43811 .42766 Metetra
.735 .797 .902 r
.43811 .42766 .44727 .43603 .46314 .42761 .45406 .41906 Metetra
.734 .795 .901 r
.45406 .41906 .46314 .42761 .47908 .41919 .47007 .41046 Metetra
.732 .792 .9 r
.47007 .41046 .47908 .41919 .49507 .41079 .48614 .40188 Metetra
.731 .789 .899 r
.48614 .40188 .49507 .41079 .51112 .40239 .50227 .39331 Metetra
.73 .786 .898 r
.50227 .39331 .51112 .40239 .52724 .39401 .51846 .38475 Metetra
.728 .784 .897 r
.51846 .38475 .52724 .39401 .54342 .38565 .53472 .37622 Metetra
.727 .781 .896 r
.53472 .37622 .54342 .38565 .55967 .37731 .55104 .36771 Metetra
.725 .778 .895 r
.55104 .36771 .55967 .37731 .57599 .369 .56743 .35923 Metetra
.723 .776 .894 r
.56743 .35923 .57599 .369 .59237 .36071 .58388 .35078 Metetra
.722 .773 .893 r
.58388 .35078 .59237 .36071 .60883 .35246 .60041 .34237 Metetra
.72 .771 .893 r
.60041 .34237 .60883 .35246 .62535 .34423 .61701 .33398 Metetra
.718 .768 .892 r
.61701 .33398 .62535 .34423 .64195 .33604 .63368 .32564 Metetra
.717 .766 .892 r
.63368 .32564 .64195 .33604 .65863 .32788 .65042 .31733 Metetra
.715 .764 .891 r
.65042 .31733 .65863 .32788 .67538 .31975 .66724 .30906 Metetra
.713 .762 .891 r
.66724 .30906 .67538 .31975 .6922 .31166 .68414 .30083 Metetra
.711 .76 .89 r
.68414 .30083 .6922 .31166 .70911 .3036 .70112 .29263 Metetra
.71 .758 .89 r
.70112 .29263 .70911 .3036 .7261 .29558 .71818 .28448 Metetra
.708 .756 .89 r
.71818 .28448 .7261 .29558 .74317 .2876 .73532 .27637 Metetra
.706 .754 .889 r
.73532 .27637 .74317 .2876 .76033 .27965 .75255 .2683 Metetra
.704 .752 .889 r
.75255 .2683 .76033 .27965 .77757 .27174 .76986 .26026 Metetra
.703 .751 .889 r
.76986 .26026 .77757 .27174 .7949 .26386 .78727 .25227 Metetra
.701 .749 .889 r
.78727 .25227 .7949 .26386 .81232 .25601 .80476 .24431 Metetra
.7 .748 .889 r
.80476 .24431 .81232 .25601 .82984 .2482 .82235 .23639 Metetra
.736 .836 .926 r
.18031 .55577 .19087 .5616 .20587 .55367 .1954 .54772 Metetra
.738 .835 .925 r
.1954 .54772 .20587 .55367 .22095 .54565 .21055 .53959 Metetra
.739 .833 .923 r
.21055 .53959 .22095 .54565 .23608 .53755 .22576 .53136 Metetra
.74 .832 .921 r
.22576 .53136 .23608 .53755 .25127 .52938 .24103 .52305 Metetra
.741 .83 .92 r
.24103 .52305 .25127 .52938 .26652 .52113 .25636 .51466 Metetra
.741 .828 .918 r
.25636 .51466 .26652 .52113 .28183 .51283 .27175 .5062 Metetra
.742 .826 .917 r
.27175 .5062 .28183 .51283 .2972 .50447 .28721 .49768 Metetra
.742 .824 .915 r
.28721 .49768 .2972 .50447 .31263 .49605 .30271 .4891 Metetra
.742 .821 .913 r
.30271 .4891 .31263 .49605 .32811 .4876 .31828 .48048 Metetra
.742 .819 .912 r
.31828 .48048 .32811 .4876 .34366 .4791 .3339 .47181 Metetra
.742 .816 .91 r
.3339 .47181 .34366 .4791 .35926 .47057 .34959 .4631 Metetra
.741 .814 .909 r
.34959 .4631 .35926 .47057 .37491 .46202 .36532 .45436 Metetra
.741 .811 .907 r
.36532 .45436 .37491 .46202 .39063 .45345 .38112 .4456 Metetra
.74 .808 .906 r
.38112 .4456 .39063 .45345 .4064 .44486 .39697 .43683 Metetra
.739 .805 .904 r
.39697 .43683 .4064 .44486 .42223 .43626 .41288 .42804 Metetra
.738 .802 .903 r
.41288 .42804 .42223 .43626 .43811 .42766 .42885 .41925 Metetra
.737 .799 .902 r
.42885 .41925 .43811 .42766 .45406 .41906 .44487 .41046 Metetra
.736 .796 .9 r
.44487 .41046 .45406 .41906 .47007 .41046 .46096 .40167 Metetra
.735 .793 .899 r
.46096 .40167 .47007 .41046 .48614 .40188 .4771 .3929 Metetra
.733 .79 .898 r
.4771 .3929 .48614 .40188 .50227 .39331 .49331 .38414 Metetra
.732 .787 .897 r
.49331 .38414 .50227 .39331 .51846 .38475 .50958 .3754 Metetra
.73 .784 .896 r
.50958 .3754 .51846 .38475 .53472 .37622 .52591 .36668 Metetra
.729 .781 .895 r
.52591 .36668 .53472 .37622 .55104 .36771 .54231 .35799 Metetra
.727 .779 .894 r
.54231 .35799 .55104 .36771 .56743 .35923 .55877 .34934 Metetra
.725 .776 .893 r
.55877 .34934 .56743 .35923 .58388 .35078 .5753 .34071 Metetra
.723 .773 .892 r
.5753 .34071 .58388 .35078 .60041 .34237 .5919 .33213 Metetra
.722 .77 .891 r
.5919 .33213 .60041 .34237 .61701 .33398 .60857 .32358 Metetra
.72 .768 .891 r
.60857 .32358 .61701 .33398 .63368 .32564 .62531 .31507 Metetra
.718 .765 .89 r
.62531 .31507 .63368 .32564 .65042 .31733 .64212 .3066 Metetra
.716 .763 .889 r
.64212 .3066 .65042 .31733 .66724 .30906 .65902 .29818 Metetra
.714 .761 .889 r
.65902 .29818 .66724 .30906 .68414 .30083 .67599 .2898 Metetra
.712 .759 .889 r
.67599 .2898 .68414 .30083 .70112 .29263 .69303 .28147 Metetra
.711 .757 .888 r
.69303 .28147 .70112 .29263 .71818 .28448 .71017 .27318 Metetra
.709 .755 .888 r
.71017 .27318 .71818 .28448 .73532 .27637 .72738 .26494 Metetra
.707 .753 .888 r
.72738 .26494 .73532 .27637 .75255 .2683 .74468 .25674 Metetra
.705 .751 .888 r
.74468 .25674 .75255 .2683 .76986 .26026 .76207 .24858 Metetra
.703 .749 .887 r
.76207 .24858 .76986 .26026 .78727 .25227 .77954 .24047 Metetra
.702 .748 .887 r
.77954 .24047 .78727 .25227 .80476 .24431 .79711 .2324 Metetra
.7 .746 .887 r
.79711 .2324 .80476 .24431 .82235 .23639 .81477 .22437 Metetra
.737 .84 .928 r
.16961 .55004 .18031 .55577 .1954 .54772 .18477 .54189 Metetra
.739 .839 .926 r
.18477 .54189 .1954 .54772 .21055 .53959 .2 .53363 Metetra
.741 .838 .924 r
.2 .53363 .21055 .53959 .22576 .53136 .21529 .52527 Metetra
.742 .836 .923 r
.21529 .52527 .22576 .53136 .24103 .52305 .23065 .51682 Metetra
.743 .834 .921 r
.23065 .51682 .24103 .52305 .25636 .51466 .24606 .50828 Metetra
.744 .832 .919 r
.24606 .50828 .25636 .51466 .27175 .5062 .26154 .49967 Metetra
.744 .83 .917 r
.26154 .49967 .27175 .5062 .28721 .49768 .27707 .49098 Metetra
.745 .828 .916 r
.27707 .49098 .28721 .49768 .30271 .4891 .29267 .48223 Metetra
.745 .825 .914 r
.29267 .48223 .30271 .4891 .31828 .48048 .30832 .47342 Metetra
.745 .823 .912 r
.30832 .47342 .31828 .48048 .3339 .47181 .32403 .46457 Metetra
.745 .82 .91 r
.32403 .46457 .3339 .47181 .34959 .4631 .33979 .45567 Metetra
.744 .817 .909 r
.33979 .45567 .34959 .4631 .36532 .45436 .35561 .44674 Metetra
.744 .814 .907 r
.35561 .44674 .36532 .45436 .38112 .4456 .37149 .43778 Metetra
.743 .811 .906 r
.37149 .43778 .38112 .4456 .39697 .43683 .38743 .4288 Metetra
.742 .808 .904 r
.38743 .4288 .39697 .43683 .41288 .42804 .40342 .41981 Metetra
.741 .804 .902 r
.40342 .41981 .41288 .42804 .42885 .41925 .41947 .41081 Metetra
.74 .801 .901 r
.41947 .41081 .42885 .41925 .44487 .41046 .43557 .40181 Metetra
.739 .798 .9 r
.43557 .40181 .44487 .41046 .46096 .40167 .45174 .39282 Metetra
.738 .795 .898 r
.45174 .39282 .46096 .40167 .4771 .3929 .46796 .38385 Metetra
.736 .791 .897 r
.46796 .38385 .4771 .3929 .49331 .38414 .48425 .37488 Metetra
.734 .788 .896 r
.48425 .37488 .49331 .38414 .50958 .3754 .50059 .36595 Metetra
.733 .785 .894 r
.50059 .36595 .50958 .3754 .52591 .36668 .517 .35703 Metetra
.731 .782 .893 r
.517 .35703 .52591 .36668 .54231 .35799 .53347 .34815 Metetra
.729 .779 .892 r
.53347 .34815 .54231 .35799 .55877 .34934 .55001 .3393 Metetra
.727 .776 .891 r
.55001 .3393 .55877 .34934 .5753 .34071 .56662 .3305 Metetra
.725 .773 .89 r
.56662 .3305 .5753 .34071 .5919 .33213 .58329 .32173 Metetra
.724 .77 .89 r
.58329 .32173 .5919 .33213 .60857 .32358 .60003 .313 Metetra
.722 .767 .889 r
.60003 .313 .60857 .32358 .62531 .31507 .61684 .30432 Metetra
.72 .765 .888 r
.61684 .30432 .62531 .31507 .64212 .3066 .63373 .29569 Metetra
.718 .762 .888 r
.63373 .29569 .64212 .3066 .65902 .29818 .6507 .28711 Metetra
.716 .76 .887 r
.6507 .28711 .65902 .29818 .67599 .2898 .66774 .27858 Metetra
.714 .757 .887 r
.66774 .27858 .67599 .2898 .69303 .28147 .68486 .27009 Metetra
.712 .755 .886 r
.68486 .27009 .69303 .28147 .71017 .27318 .70206 .26166 Metetra
.71 .753 .886 r
.70206 .26166 .71017 .27318 .72738 .26494 .71935 .25328 Metetra
.708 .751 .886 r
.71935 .25328 .72738 .26494 .74468 .25674 .73672 .24495 Metetra
.706 .749 .886 r
.73672 .24495 .74468 .25674 .76207 .24858 .75418 .23667 Metetra
.704 .747 .886 r
.75418 .23667 .76207 .24858 .77954 .24047 .77173 .22843 Metetra
.702 .746 .886 r
.77173 .22843 .77954 .24047 .79711 .2324 .78937 .22025 Metetra
.7 .744 .886 r
.78937 .22025 .79711 .2324 .81477 .22437 .8071 .21211 Metetra
.739 .845 .93 r
.15875 .54444 .16961 .55004 .18477 .54189 .17399 .53618 Metetra
.741 .844 .928 r
.17399 .53618 .18477 .54189 .2 .53363 .1893 .5278 Metetra
.742 .843 .926 r
.1893 .5278 .2 .53363 .21529 .52527 .20468 .51931 Metetra
.744 .841 .924 r
.20468 .51931 .21529 .52527 .23065 .51682 .22012 .51072 Metetra
.745 .839 .922 r
.22012 .51072 .23065 .51682 .24606 .50828 .23562 .50203 Metetra
.746 .837 .92 r
.23562 .50203 .24606 .50828 .26154 .49967 .25118 .49325 Metetra
.747 .835 .918 r
.25118 .49325 .26154 .49967 .27707 .49098 .2668 .48438 Metetra
.748 .833 .917 r
.2668 .48438 .27707 .49098 .29267 .48223 .28248 .47545 Metetra
.748 .83 .915 r
.28248 .47545 .29267 .48223 .30832 .47342 .29822 .46645 Metetra
.748 .827 .913 r
.29822 .46645 .30832 .47342 .32403 .46457 .31402 .4574 Metetra
.748 .824 .911 r
.31402 .4574 .32403 .46457 .33979 .45567 .32987 .44829 Metetra
.748 .821 .909 r
.32987 .44829 .33979 .45567 .35561 .44674 .34578 .43915 Metetra
.747 .817 .907 r
.34578 .43915 .35561 .44674 .37149 .43778 .36174 .42998 Metetra
.746 .814 .905 r
.36174 .42998 .37149 .43778 .38743 .4288 .37777 .42079 Metetra
.746 .811 .904 r
.37777 .42079 .38743 .4288 .40342 .41981 .39384 .41157 Metetra
.745 .807 .902 r
.39384 .41157 .40342 .41981 .41947 .41081 .40998 .40236 Metetra
.743 .804 .9 r
.40998 .40236 .41947 .41081 .43557 .40181 .42616 .39314 Metetra
.742 .8 .899 r
.42616 .39314 .43557 .40181 .45174 .39282 .44241 .38393 Metetra
.741 .796 .897 r
.44241 .38393 .45174 .39282 .46796 .38385 .45872 .37473 Metetra
.739 .793 .896 r
.45872 .37473 .46796 .38385 .48425 .37488 .47508 .36555 Metetra
.737 .789 .894 r
.47508 .36555 .48425 .37488 .50059 .36595 .49151 .35639 Metetra
.735 .786 .893 r
.49151 .35639 .50059 .36595 .517 .35703 .50799 .34727 Metetra
.734 .782 .892 r
.50799 .34727 .517 .35703 .53347 .34815 .52454 .33818 Metetra
.732 .779 .891 r
.52454 .33818 .53347 .34815 .55001 .3393 .54115 .32913 Metetra
.73 .776 .89 r
.54115 .32913 .55001 .3393 .56662 .3305 .55783 .32012 Metetra
.728 .773 .889 r
.55783 .32012 .56662 .3305 .58329 .32173 .57458 .31116 Metetra
.725 .77 .888 r
.57458 .31116 .58329 .32173 .60003 .313 .5914 .30225 Metetra
.723 .767 .887 r
.5914 .30225 .60003 .313 .61684 .30432 .60828 .29338 Metetra
.721 .764 .886 r
.60828 .29338 .61684 .30432 .63373 .29569 .62524 .28458 Metetra
.719 .761 .886 r
.62524 .28458 .63373 .29569 .6507 .28711 .64228 .27583 Metetra
.717 .758 .885 r
.64228 .27583 .6507 .28711 .66774 .27858 .65939 .26713 Metetra
.715 .756 .885 r
.65939 .26713 .66774 .27858 .68486 .27009 .67658 .25849 Metetra
.713 .753 .884 r
.67658 .25849 .68486 .27009 .70206 .26166 .69386 .24991 Metetra
.711 .751 .884 r
.69386 .24991 .70206 .26166 .71935 .25328 .71122 .24138 Metetra
.708 .749 .884 r
.71122 .24138 .71935 .25328 .73672 .24495 .72866 .23292 Metetra
.706 .747 .884 r
.72866 .23292 .73672 .24495 .75418 .23667 .74619 .22451 Metetra
.704 .745 .883 r
.74619 .22451 .75418 .23667 .77173 .22843 .76381 .21615 Metetra
.702 .744 .883 r
.76381 .21615 .77173 .22843 .78937 .22025 .78153 .20785 Metetra
.701 .742 .884 r
.78153 .20785 .78937 .22025 .8071 .21211 .79934 .19959 Metetra
.74 .851 .932 r
.14772 .539 .15875 .54444 .17399 .53618 .16305 .53063 Metetra
.742 .85 .93 r
.16305 .53063 .17399 .53618 .1893 .5278 .17844 .52214 Metetra
.744 .848 .928 r
.17844 .52214 .1893 .5278 .20468 .51931 .19391 .51352 Metetra
.746 .847 .926 r
.19391 .51352 .20468 .51931 .22012 .51072 .20943 .50477 Metetra
.747 .845 .924 r
.20943 .50477 .22012 .51072 .23562 .50203 .22502 .49592 Metetra
.749 .843 .922 r
.22502 .49592 .23562 .50203 .25118 .49325 .24067 .48697 Metetra
.75 .84 .92 r
.24067 .48697 .25118 .49325 .2668 .48438 .25639 .47793 Metetra
.751 .838 .918 r
.25639 .47793 .2668 .48438 .28248 .47545 .27216 .4688 Metetra
.751 .835 .916 r
.27216 .4688 .28248 .47545 .29822 .46645 .28799 .4596 Metetra
.751 .832 .913 r
.28799 .4596 .29822 .46645 .31402 .4574 .30387 .45033 Metetra
.751 .829 .911 r
.30387 .45033 .31402 .4574 .32987 .44829 .31982 .44101 Metetra
.751 .825 .909 r
.31982 .44101 .32987 .44829 .34578 .43915 .33582 .43164 Metetra
.751 .822 .907 r
.33582 .43164 .34578 .43915 .36174 .42998 .35187 .42224 Metetra
.75 .818 .905 r
.35187 .42224 .36174 .42998 .37777 .42079 .36798 .41281 Metetra
.749 .814 .903 r
.36798 .41281 .37777 .42079 .39384 .41157 .38414 .40336 Metetra
.748 .81 .901 r
.38414 .40336 .39384 .41157 .40998 .40236 .40036 .3939 Metetra
.747 .806 .9 r
.40036 .3939 .40998 .40236 .42616 .39314 .41664 .38444 Metetra
.745 .802 .898 r
.41664 .38444 .42616 .39314 .44241 .38393 .43297 .37498 Metetra
.744 .798 .896 r
.43297 .37498 .44241 .38393 .45872 .37473 .44936 .36555 Metetra
.742 .794 .895 r
.44936 .36555 .45872 .37473 .47508 .36555 .46581 .35613 Metetra
.74 .791 .893 r
.46581 .35613 .47508 .36555 .49151 .35639 .48231 .34674 Metetra
.738 .787 .892 r
.48231 .34674 .49151 .35639 .50799 .34727 .49888 .33738 Metetra
.736 .783 .89 r
.49888 .33738 .50799 .34727 .52454 .33818 .51551 .32806 Metetra
.734 .779 .889 r
.51551 .32806 .52454 .33818 .54115 .32913 .5322 .31879 Metetra
.732 .776 .888 r
.5322 .31879 .54115 .32913 .55783 .32012 .54895 .30957 Metetra
.73 .772 .887 r
.54895 .30957 .55783 .32012 .57458 .31116 .56577 .3004 Metetra
.728 .769 .886 r
.56577 .3004 .57458 .31116 .5914 .30225 .58266 .29129 Metetra
.725 .765 .885 r
.58266 .29129 .5914 .30225 .60828 .29338 .59962 .28223 Metetra
.723 .762 .884 r
.59962 .28223 .60828 .29338 .62524 .28458 .61666 .27324 Metetra
.721 .759 .883 r
.61666 .27324 .62524 .28458 .64228 .27583 .63377 .2643 Metetra
.718 .757 .883 r
.63377 .2643 .64228 .27583 .65939 .26713 .65095 .25544 Metetra
.716 .754 .882 r
.65095 .25544 .65939 .26713 .67658 .25849 .66821 .24663 Metetra
.714 .751 .882 r
.66821 .24663 .67658 .25849 .69386 .24991 .68556 .2379 Metetra
.711 .749 .881 r
.68556 .2379 .69386 .24991 .71122 .24138 .70299 .22922 Metetra
.709 .747 .881 r
.70299 .22922 .71122 .24138 .72866 .23292 .7205 .22062 Metetra
.707 .745 .881 r
.7205 .22062 .72866 .23292 .74619 .22451 .73811 .21207 Metetra
.705 .743 .881 r
.73811 .21207 .74619 .22451 .76381 .21615 .7558 .20359 Metetra
.703 .741 .881 r
.7558 .20359 .76381 .21615 .78153 .20785 .77359 .19517 Metetra
.701 .74 .881 r
.77359 .19517 .78153 .20785 .79934 .19959 .79148 .18681 Metetra
.74 .857 .934 r
.13653 .53374 .14772 .539 .16305 .53063 .15194 .52528 Metetra
.743 .856 .932 r
.15194 .52528 .16305 .53063 .17844 .52214 .16742 .51667 Metetra
.746 .855 .93 r
.16742 .51667 .17844 .52214 .19391 .51352 .18297 .50792 Metetra
.748 .853 .928 r
.18297 .50792 .19391 .51352 .20943 .50477 .19858 .49903 Metetra
.75 .852 .926 r
.19858 .49903 .20943 .50477 .22502 .49592 .21426 .49002 Metetra
.751 .849 .924 r
.21426 .49002 .22502 .49592 .24067 .48697 .23001 .48089 Metetra
.753 .847 .921 r
.23001 .48089 .24067 .48697 .25639 .47793 .24581 .47165 Metetra
.754 .844 .919 r
.24581 .47165 .25639 .47793 .27216 .4688 .26168 .46232 Metetra
.755 .841 .917 r
.26168 .46232 .27216 .4688 .28799 .4596 .2776 .4529 Metetra
.755 .838 .914 r
.2776 .4529 .28799 .4596 .30387 .45033 .29358 .4434 Metetra
.755 .834 .912 r
.29358 .4434 .30387 .45033 .31982 .44101 .30962 .43384 Metetra
.755 .83 .91 r
.30962 .43384 .31982 .44101 .33582 .43164 .32572 .42423 Metetra
.755 .826 .908 r
.32572 .42423 .33582 .43164 .35187 .42224 .34186 .41457 Metetra
.754 .822 .905 r
.34186 .41457 .35187 .42224 .36798 .41281 .35807 .40489 Metetra
.753 .818 .903 r
.35807 .40489 .36798 .41281 .38414 .40336 .37432 .39517 Metetra
.752 .814 .901 r
.37432 .39517 .38414 .40336 .40036 .3939 .39063 .38545 Metetra
.751 .809 .899 r
.39063 .38545 .40036 .3939 .41664 .38444 .407 .37573 Metetra
.749 .805 .897 r
.407 .37573 .41664 .38444 .43297 .37498 .42342 .36601 Metetra
.748 .801 .895 r
.42342 .36601 .43297 .37498 .44936 .36555 .43989 .3563 Metetra
.746 .796 .893 r
.43989 .3563 .44936 .36555 .46581 .35613 .45642 .34662 Metetra
.744 .792 .891 r
.45642 .34662 .46581 .35613 .48231 .34674 .47301 .33698 Metetra
.742 .787 .89 r
.47301 .33698 .48231 .34674 .49888 .33738 .48966 .32737 Metetra
.739 .783 .888 r
.48966 .32737 .49888 .33738 .51551 .32806 .50637 .3178 Metetra
.737 .779 .887 r
.50637 .3178 .51551 .32806 .5322 .31879 .52314 .30829 Metetra
.735 .775 .885 r
.52314 .30829 .5322 .31879 .54895 .30957 .53997 .29883 Metetra
.732 .771 .884 r
.53997 .29883 .54895 .30957 .56577 .3004 .55687 .28943 Metetra
.73 .768 .883 r
.55687 .28943 .56577 .3004 .58266 .29129 .57383 .2801 Metetra
.727 .764 .882 r
.57383 .2801 .58266 .29129 .59962 .28223 .59087 .27084 Metetra
.725 .761 .881 r
.59087 .27084 .59962 .28223 .61666 .27324 .60797 .26164 Metetra
.722 .757 .881 r
.60797 .26164 .61666 .27324 .63377 .2643 .62515 .25252 Metetra
.72 .754 .88 r
.62515 .25252 .63377 .2643 .65095 .25544 .64241 .24347 Metetra
.717 .752 .879 r
.64241 .24347 .65095 .25544 .66821 .24663 .65975 .2345 Metetra
.715 .749 .879 r
.65975 .2345 .66821 .24663 .68556 .2379 .67716 .2256 Metetra
.712 .746 .879 r
.67716 .2256 .68556 .2379 .70299 .22922 .69466 .21677 Metetra
.71 .744 .879 r
.69466 .21677 .70299 .22922 .7205 .22062 .71225 .20802 Metetra
.707 .742 .878 r
.71225 .20802 .7205 .22062 .73811 .21207 .72993 .19934 Metetra
.705 .74 .878 r
.72993 .19934 .73811 .21207 .7558 .20359 .74769 .19073 Metetra
.703 .738 .879 r
.74769 .19073 .7558 .20359 .77359 .19517 .76556 .18219 Metetra
.701 .736 .879 r
.76556 .18219 .77359 .19517 .79148 .18681 .78351 .17372 Metetra
.741 .864 .937 r
.12515 .52871 .13653 .53374 .15194 .52528 .14064 .52017 Metetra
.744 .863 .935 r
.14064 .52017 .15194 .52528 .16742 .51667 .15621 .51145 Metetra
.747 .862 .933 r
.15621 .51145 .16742 .51667 .18297 .50792 .17185 .50257 Metetra
.75 .861 .931 r
.17185 .50257 .18297 .50792 .19858 .49903 .18756 .49354 Metetra
.752 .859 .928 r
.18756 .49354 .19858 .49903 .21426 .49002 .20333 .48436 Metetra
.754 .857 .926 r
.20333 .48436 .21426 .49002 .23001 .48089 .21917 .47504 Metetra
.756 .854 .923 r
.21917 .47504 .23001 .48089 .24581 .47165 .23507 .46561 Metetra
.757 .851 .921 r
.23507 .46561 .24581 .47165 .26168 .46232 .25104 .45605 Metetra
.758 .848 .918 r
.25104 .45605 .26168 .46232 .2776 .4529 .26706 .4464 Metetra
.759 .844 .916 r
.26706 .4464 .2776 .4529 .29358 .4434 .28314 .43666 Metetra
.76 .841 .913 r
.28314 .43666 .29358 .4434 .30962 .43384 .29928 .42685 Metetra
.76 .836 .91 r
.29928 .42685 .30962 .43384 .32572 .42423 .31547 .41696 Metetra
.759 .832 .908 r
.31547 .41696 .32572 .42423 .34186 .41457 .33172 .40703 Metetra
.759 .827 .905 r
.33172 .40703 .34186 .41457 .35807 .40489 .34802 .39706 Metetra
.758 .823 .903 r
.34802 .39706 .35807 .40489 .37432 .39517 .36437 .38706 Metetra
.757 .818 .9 r
.36437 .38706 .37432 .39517 .39063 .38545 .38078 .37704 Metetra
.755 .813 .898 r
.38078 .37704 .39063 .38545 .407 .37573 .39723 .36702 Metetra
.754 .808 .896 r
.39723 .36702 .407 .37573 .42342 .36601 .41375 .357 Metetra
.752 .803 .894 r
.41375 .357 .42342 .36601 .43989 .3563 .43031 .34701 Metetra
.75 .798 .892 r
.43031 .34701 .43989 .3563 .45642 .34662 .44693 .33704 Metetra
.748 .793 .89 r
.44693 .33704 .45642 .34662 .47301 .33698 .4636 .3271 Metetra
.745 .788 .888 r
.4636 .3271 .47301 .33698 .48966 .32737 .48033 .31721 Metetra
.743 .784 .886 r
.48033 .31721 .48966 .32737 .50637 .3178 .49712 .30737 Metetra
.74 .779 .884 r
.49712 .30737 .50637 .3178 .52314 .30829 .51397 .29759 Metetra
.738 .775 .883 r
.51397 .29759 .52314 .30829 .53997 .29883 .53088 .28788 Metetra
.735 .77 .881 r
.53088 .28788 .53997 .29883 .55687 .28943 .54786 .27823 Metetra
.732 .766 .88 r
.54786 .27823 .55687 .28943 .57383 .2801 .5649 .26866 Metetra
.729 .762 .879 r
.5649 .26866 .57383 .2801 .59087 .27084 .58201 .25917 Metetra
.726 .759 .878 r
.58201 .25917 .59087 .27084 .60797 .26164 .59919 .24976 Metetra
.724 .755 .877 r
.59919 .24976 .60797 .26164 .62515 .25252 .61644 .24043 Metetra
.721 .752 .877 r
.61644 .24043 .62515 .25252 .64241 .24347 .63377 .23119 Metetra
.718 .749 .876 r
.63377 .23119 .64241 .24347 .65975 .2345 .65118 .22204 Metetra
.715 .746 .876 r
.65118 .22204 .65975 .2345 .67716 .2256 .66866 .21297 Metetra
.713 .743 .875 r
.66866 .21297 .67716 .2256 .69466 .21677 .68624 .20398 Metetra
.71 .741 .875 r
.68624 .20398 .69466 .21677 .71225 .20802 .70389 .19508 Metetra
.708 .738 .875 r
.70389 .19508 .71225 .20802 .72993 .19934 .72164 .18627 Metetra
.705 .736 .875 r
.72164 .18627 .72993 .19934 .74769 .19073 .73948 .17753 Metetra
.703 .734 .876 r
.73948 .17753 .74769 .19073 .76556 .18219 .75742 .16887 Metetra
.7 .733 .876 r
.75742 .16887 .76556 .18219 .78351 .17372 .77545 .16029 Metetra
.741 .871 .941 r
.11357 .52395 .12515 .52871 .14064 .52017 .12915 .51534 Metetra
.744 .871 .939 r
.12915 .51534 .14064 .52017 .15621 .51145 .14481 .50653 Metetra
.748 .871 .936 r
.14481 .50653 .15621 .51145 .17185 .50257 .16054 .49753 Metetra
.751 .87 .934 r
.16054 .49753 .17185 .50257 .18756 .49354 .17634 .48835 Metetra
.754 .868 .931 r
.17634 .48835 .18756 .49354 .20333 .48436 .19221 .47901 Metetra
.757 .866 .928 r
.19221 .47901 .20333 .48436 .21917 .47504 .20815 .46951 Metetra
.759 .863 .926 r
.20815 .46951 .21917 .47504 .23507 .46561 .22416 .45986 Metetra
.761 .86 .923 r
.22416 .45986 .23507 .46561 .25104 .45605 .24023 .45008 Metetra
.763 .857 .92 r
.24023 .45008 .25104 .45605 .26706 .4464 .25635 .44018 Metetra
.764 .853 .917 r
.25635 .44018 .26706 .4464 .28314 .43666 .27254 .43017 Metetra
.764 .848 .914 r
.27254 .43017 .28314 .43666 .29928 .42685 .28878 .42007 Metetra
.765 .844 .911 r
.28878 .42007 .29928 .42685 .31547 .41696 .30508 .4099 Metetra
.765 .839 .908 r
.30508 .4099 .31547 .41696 .33172 .40703 .32143 .39965 Metetra
.764 .834 .906 r
.32143 .39965 .33172 .40703 .34802 .39706 .33783 .38936 Metetra
.763 .828 .903 r
.33783 .38936 .34802 .39706 .36437 .38706 .35428 .37904 Metetra
.762 .823 .9 r
.35428 .37904 .36437 .38706 .38078 .37704 .37079 .36869 Metetra
.761 .817 .897 r
.37079 .36869 .38078 .37704 .39723 .36702 .38735 .35834 Metetra
.759 .812 .895 r
.38735 .35834 .39723 .36702 .41375 .357 .40395 .34799 Metetra
.757 .806 .892 r
.40395 .34799 .41375 .357 .43031 .34701 .42061 .33767 Metetra
.754 .8 .89 r
.42061 .33767 .43031 .34701 .44693 .33704 .43732 .32737 Metetra
.752 .795 .887 r
.43732 .32737 .44693 .33704 .4636 .3271 .45408 .31711 Metetra
.749 .789 .885 r
.45408 .31711 .4636 .3271 .48033 .31721 .4709 .30691 Metetra
.747 .784 .883 r
.4709 .30691 .48033 .31721 .49712 .30737 .48777 .29676 Metetra
.744 .779 .881 r
.48777 .29676 .49712 .30737 .51397 .29759 .5047 .28668 Metetra
.741 .774 .88 r
.5047 .28668 .51397 .29759 .53088 .28788 .52169 .27668 Metetra
.738 .769 .878 r
.52169 .27668 .53088 .28788 .54786 .27823 .53875 .26676 Metetra
.735 .764 .877 r
.53875 .26676 .54786 .27823 .5649 .26866 .55586 .25693 Metetra
.731 .76 .875 r
.55586 .25693 .5649 .26866 .58201 .25917 .57305 .24719 Metetra
.728 .756 .874 r
.57305 .24719 .58201 .25917 .59919 .24976 .5903 .23755 Metetra
.725 .752 .873 r
.5903 .23755 .59919 .24976 .61644 .24043 .60763 .228 Metetra
.722 .748 .873 r
.60763 .228 .61644 .24043 .63377 .23119 .62503 .21855 Metetra
.719 .745 .872 r
.62503 .21855 .63377 .23119 .65118 .22204 .6425 .20921 Metetra
.716 .742 .872 r
.6425 .20921 .65118 .22204 .66866 .21297 .66006 .19996 Metetra
.713 .739 .872 r
.66006 .19996 .66866 .21297 .68624 .20398 .6777 .19081 Metetra
.71 .736 .871 r
.6777 .19081 .68624 .20398 .70389 .19508 .69543 .18176 Metetra
.708 .734 .871 r
.69543 .18176 .70389 .19508 .72164 .18627 .71325 .17281 Metetra
.705 .732 .872 r
.71325 .17281 .72164 .18627 .73948 .17753 .73117 .16395 Metetra
.702 .73 .872 r
.73117 .16395 .73948 .17753 .75742 .16887 .74917 .15518 Metetra
.7 .728 .872 r
.74917 .15518 .75742 .16887 .77545 .16029 .76728 .1465 Metetra
.739 .88 .945 r
.10179 .51951 .11357 .52395 .12915 .51534 .11745 .51085 Metetra
.744 .881 .943 r
.11745 .51085 .12915 .51534 .14481 .50653 .1332 .50197 Metetra
.748 .881 .94 r
.1332 .50197 .14481 .50653 .16054 .49753 .14902 .49288 Metetra
.752 .88 .937 r
.14902 .49288 .16054 .49753 .17634 .48835 .16492 .48357 Metetra
.756 .879 .934 r
.16492 .48357 .17634 .48835 .19221 .47901 .18089 .47407 Metetra
.759 .877 .932 r
.18089 .47407 .19221 .47901 .20815 .46951 .19694 .46438 Metetra
.762 .874 .928 r
.19694 .46438 .20815 .46951 .22416 .45986 .21305 .45452 Metetra
.765 .871 .925 r
.21305 .45452 .22416 .45986 .24023 .45008 .22923 .44449 Metetra
.767 .867 .922 r
.22923 .44449 .24023 .45008 .25635 .44018 .24546 .43433 Metetra
.769 .863 .919 r
.24546 .43433 .25635 .44018 .27254 .43017 .26176 .42403 Metetra
.77 .858 .916 r
.26176 .42403 .27254 .43017 .28878 .42007 .27811 .41362 Metetra
.77 .853 .912 r
.27811 .41362 .28878 .42007 .30508 .4099 .29452 .40311 Metetra
.77 .847 .909 r
.29452 .40311 .30508 .4099 .32143 .39965 .31098 .39252 Metetra
.77 .841 .906 r
.31098 .39252 .32143 .39965 .33783 .38936 .32749 .38187 Metetra
.769 .835 .903 r
.32749 .38187 .33783 .38936 .35428 .37904 .34405 .37118 Metetra
.768 .829 .899 r
.34405 .37118 .35428 .37904 .37079 .36869 .36067 .36046 Metetra
.767 .822 .896 r
.36067 .36046 .37079 .36869 .38735 .35834 .37732 .34973 Metetra
.765 .816 .893 r
.37732 .34973 .38735 .35834 .40395 .34799 .39403 .339 Metetra
.762 .809 .89 r
.39403 .339 .40395 .34799 .42061 .33767 .41079 .32829 Metetra
.76 .803 .887 r
.41079 .32829 .42061 .33767 .43732 .32737 .42759 .31762 Metetra
.757 .796 .885 r
.42759 .31762 .43732 .32737 .45408 .31711 .44445 .30699 Metetra
.754 .79 .882 r
.44445 .30699 .45408 .31711 .4709 .30691 .46135 .29643 Metetra
.751 .784 .88 r
.46135 .29643 .4709 .30691 .48777 .29676 .47831 .28593 Metetra
.747 .778 .878 r
.47831 .28593 .48777 .29676 .5047 .28668 .49533 .27552 Metetra
.744 .772 .876 r
.49533 .27552 .5047 .28668 .52169 .27668 .5124 .2652 Metetra
.741 .766 .874 r
.5124 .2652 .52169 .27668 .53875 .26676 .52953 .25497 Metetra
.737 .761 .872 r
.52953 .25497 .53875 .26676 .55586 .25693 .54673 .24485 Metetra
.734 .756 .871 r
.54673 .24485 .55586 .25693 .57305 .24719 .56398 .23484 Metetra
.73 .752 .87 r
.56398 .23484 .57305 .24719 .5903 .23755 .58131 .22494 Metetra
.727 .747 .869 r
.58131 .22494 .5903 .23755 .60763 .228 .59871 .21515 Metetra
.723 .744 .868 r
.59871 .21515 .60763 .228 .62503 .21855 .61618 .20549 Metetra
.72 .74 .867 r
.61618 .20549 .62503 .21855 .6425 .20921 .63373 .19594 Metetra
.717 .737 .867 r
.63373 .19594 .6425 .20921 .66006 .19996 .65135 .18651 Metetra
.713 .734 .867 r
.65135 .18651 .66006 .19996 .6777 .19081 .66907 .1772 Metetra
.71 .731 .867 r
.66907 .1772 .6777 .19081 .69543 .18176 .68687 .168 Metetra
.707 .729 .867 r
.68687 .168 .69543 .18176 .71325 .17281 .70476 .15891 Metetra
.704 .726 .867 r
.70476 .15891 .71325 .17281 .73117 .16395 .72274 .14993 Metetra
.702 .725 .868 r
.72274 .14993 .73117 .16395 .74917 .15518 .74082 .14106 Metetra
.699 .723 .869 r
.74082 .14106 .74917 .15518 .76728 .1465 .759 .13228 Metetra
.736 .89 .95 r
.08979 .51544 .10179 .51951 .11745 .51085 .10553 .50679 Metetra
.742 .891 .947 r
.10553 .50679 .11745 .51085 .1332 .50197 .12136 .49787 Metetra
.747 .892 .944 r
.12136 .49787 .1332 .50197 .14902 .49288 .13728 .48871 Metetra
.752 .892 .942 r
.13728 .48871 .14902 .49288 .16492 .48357 .15327 .47929 Metetra
.757 .891 .938 r
.15327 .47929 .16492 .48357 .18089 .47407 .16935 .46965 Metetra
.761 .89 .935 r
.16935 .46965 .18089 .47407 .19694 .46438 .1855 .45978 Metetra
.765 .887 .932 r
.1855 .45978 .19694 .46438 .21305 .45452 .20172 .4497 Metetra
.769 .884 .929 r
.20172 .4497 .21305 .45452 .22923 .44449 .21801 .43942 Metetra
.772 .88 .925 r
.21801 .43942 .22923 .44449 .24546 .43433 .23437 .42897 Metetra
.774 .876 .921 r
.23437 .42897 .24546 .43433 .26176 .42403 .25078 .41835 Metetra
.776 .87 .918 r
.25078 .41835 .26176 .42403 .27811 .41362 .26726 .40759 Metetra
.777 .864 .914 r
.26726 .40759 .27811 .41362 .29452 .40311 .28379 .39671 Metetra
.777 .858 .91 r
.28379 .39671 .29452 .40311 .31098 .39252 .30036 .38573 Metetra
.777 .851 .906 r
.30036 .38573 .31098 .39252 .32749 .38187 .31699 .37467 Metetra
.777 .844 .903 r
.31699 .37467 .32749 .38187 .34405 .37118 .33367 .36355 Metetra
.775 .837 .899 r
.33367 .36355 .34405 .37118 .36067 .36046 .3504 .3524 Metetra
.774 .829 .895 r
.3504 .3524 .36067 .36046 .37732 .34973 .36717 .34122 Metetra
.772 .821 .892 r
.36717 .34122 .37732 .34973 .39403 .339 .38398 .33005 Metetra
.769 .813 .888 r
.38398 .33005 .39403 .339 .41079 .32829 .40084 .3189 Metetra
.766 .806 .885 r
.40084 .3189 .41079 .32829 .42759 .31762 .41775 .30779 Metetra
.763 .798 .881 r
.41775 .30779 .42759 .31762 .44445 .30699 .4347 .29673 Metetra
.759 .79 .878 r
.4347 .29673 .44445 .30699 .46135 .29643 .4517 .28575 Metetra
.756 .783 .876 r
.4517 .28575 .46135 .29643 .47831 .28593 .46875 .27485 Metetra
.752 .776 .873 r
.46875 .27485 .47831 .28593 .49533 .27552 .48585 .26405 Metetra
.748 .77 .871 r
.48585 .26405 .49533 .27552 .5124 .2652 .503 .25336 Metetra
.744 .763 .869 r
.503 .25336 .5124 .2652 .52953 .25497 .52022 .24278 Metetra
.74 .757 .867 r
.52022 .24278 .52953 .25497 .54673 .24485 .53749 .23233 Metetra
.736 .752 .865 r
.53749 .23233 .54673 .24485 .56398 .23484 .55482 .22202 Metetra
.732 .746 .864 r
.55482 .22202 .56398 .23484 .58131 .22494 .57222 .21184 Metetra
.728 .742 .863 r
.57222 .21184 .58131 .22494 .59871 .21515 .58969 .2018 Metetra
.724 .737 .862 r
.58969 .2018 .59871 .21515 .61618 .20549 .60723 .1919 Metetra
.72 .733 .861 r
.60723 .1919 .61618 .20549 .63373 .19594 .62484 .18214 Metetra
.716 .73 .861 r
.62484 .18214 .63373 .19594 .65135 .18651 .64254 .17252 Metetra
.713 .727 .861 r
.64254 .17252 .65135 .18651 .66907 .1772 .66032 .16304 Metetra
.709 .724 .861 r
.66032 .16304 .66907 .1772 .68687 .168 .67819 .1537 Metetra
.706 .722 .862 r
.67819 .1537 .68687 .168 .70476 .15891 .69615 .14449 Metetra
.703 .72 .862 r
.69615 .14449 .70476 .15891 .72274 .14993 .7142 .1354 Metetra
.7 .718 .863 r
.7142 .1354 .72274 .14993 .74082 .14106 .73235 .12644 Metetra
.697 .717 .864 r
.73235 .12644 .74082 .14106 .759 .13228 .75061 .11759 Metetra
.73 .9 .955 r
.07755 .5118 .08979 .51544 .10553 .50679 .09336 .50321 Metetra
.737 .903 .952 r
.09336 .50321 .10553 .50679 .12136 .49787 .10928 .49432 Metetra
.743 .905 .949 r
.10928 .49432 .12136 .49787 .13728 .48871 .12528 .48514 Metetra
.75 .906 .946 r
.12528 .48514 .13728 .48871 .15327 .47929 .14137 .47566 Metetra
.756 .906 .943 r
.14137 .47566 .15327 .47929 .16935 .46965 .15756 .46591 Metetra
.762 .906 .94 r
.15756 .46591 .16935 .46965 .1855 .45978 .17382 .45588 Metetra
.767 .904 .936 r
.17382 .45588 .1855 .45978 .20172 .4497 .19016 .44559 Metetra
.772 .901 .932 r
.19016 .44559 .20172 .4497 .21801 .43942 .20657 .43505 Metetra
.776 .897 .928 r
.20657 .43505 .21801 .43942 .23437 .42897 .22305 .42429 Metetra
.78 .892 .924 r
.22305 .42429 .23437 .42897 .25078 .41835 .23959 .41333 Metetra
.782 .886 .92 r
.23959 .41333 .25078 .41835 .26726 .40759 .25619 .40219 Metetra
.784 .88 .916 r
.25619 .40219 .26726 .40759 .28379 .39671 .27285 .39088 Metetra
.785 .872 .912 r
.27285 .39088 .28379 .39671 .30036 .38573 .28956 .37945 Metetra
.786 .864 .907 r
.28956 .37945 .30036 .38573 .31699 .37467 .30632 .36791 Metetra
.785 .856 .903 r
.30632 .36791 .31699 .37467 .33367 .36355 .32312 .35629 Metetra
.784 .847 .898 r
.32312 .35629 .33367 .36355 .3504 .3524 .33997 .34461 Metetra
.782 .837 .894 r
.33997 .34461 .3504 .3524 .36717 .34122 .35686 .33291 Metetra
.78 .828 .89 r
.35686 .33291 .36717 .34122 .38398 .33005 .37379 .3212 Metetra
.777 .819 .885 r
.37379 .3212 .38398 .33005 .40084 .3189 .39077 .30952 Metetra
.774 .809 .881 r
.39077 .30952 .40084 .3189 .41775 .30779 .40778 .29788 Metetra
.77 .8 .877 r
.40778 .29788 .41775 .30779 .4347 .29673 .42483 .28631 Metetra
.765 .791 .874 r
.42483 .28631 .4347 .29673 .4517 .28575 .44193 .27483 Metetra
.761 .782 .87 r
.44193 .27483 .4517 .28575 .46875 .27485 .45908 .26345 Metetra
.756 .774 .867 r
.45908 .26345 .46875 .27485 .48585 .26405 .47627 .25219 Metetra
.752 .766 .864 r
.47627 .25219 .48585 .26405 .503 .25336 .4935 .24106 Metetra
.747 .758 .862 r
.4935 .24106 .503 .25336 .52022 .24278 .5108 .23009 Metetra
.742 .751 .86 r
.5108 .23009 .52022 .24278 .53749 .23233 .52814 .21926 Metetra
.738 .745 .858 r
.52814 .21926 .53749 .23233 .55482 .22202 .54555 .2086 Metetra
.733 .739 .856 r
.54555 .2086 .55482 .22202 .57222 .21184 .56302 .19811 Metetra
.728 .734 .855 r
.56302 .19811 .57222 .21184 .58969 .2018 .58056 .18779 Metetra
.724 .729 .855 r
.58056 .18779 .58969 .2018 .60723 .1919 .59817 .17765 Metetra
.72 .725 .854 r
.59817 .17765 .60723 .1919 .62484 .18214 .61585 .16767 Metetra
.716 .721 .854 r
.61585 .16767 .62484 .18214 .64254 .17252 .63361 .15787 Metetra
.712 .718 .854 r
.63361 .15787 .64254 .17252 .66032 .16304 .65146 .14823 Metetra
.708 .715 .855 r
.65146 .14823 .66032 .16304 .67819 .1537 .6694 .13876 Metetra
.704 .713 .855 r
.6694 .13876 .67819 .1537 .69615 .14449 .68742 .12944 Metetra
.701 .711 .856 r
.68742 .12944 .69615 .14449 .7142 .1354 .70555 .12027 Metetra
.698 .71 .857 r
.70555 .12027 .7142 .1354 .73235 .12644 .72377 .11125 Metetra
.695 .709 .859 r
.72377 .11125 .73235 .12644 .75061 .11759 .7421 .10235 Metetra
.72 .91 .961 r
.06506 .5086 .07755 .5118 .09336 .50321 .08093 .50018 Metetra
.727 .915 .958 r
.08093 .50018 .09336 .50321 .10928 .49432 .09692 .49142 Metetra
.735 .919 .955 r
.09692 .49142 .10928 .49432 .12528 .48514 .113 .48231 Metetra
.742 .922 .951 r
.113 .48231 .12528 .48514 .14137 .47566 .12919 .47286 Metetra
.75 .924 .948 r
.12919 .47286 .14137 .47566 .15756 .46591 .14547 .46305 Metetra
.758 .925 .944 r
.14547 .46305 .15756 .46591 .17382 .45588 .16185 .45291 Metetra
.765 .924 .94 r
.16185 .45291 .17382 .45588 .19016 .44559 .1783 .44245 Metetra
.772 .922 .936 r
.1783 .44245 .19016 .44559 .20657 .43505 .19484 .43167 Metetra
.779 .918 .932 r
.19484 .43167 .20657 .43505 .22305 .42429 .21145 .4206 Metetra
.784 .914 .927 r
.21145 .4206 .22305 .42429 .23959 .41333 .22813 .40926 Metetra
.789 .908 .923 r
.22813 .40926 .23959 .41333 .25619 .40219 .24488 .39769 Metetra
.792 .9 .918 r
.24488 .39769 .25619 .40219 .27285 .39088 .26168 .3859 Metetra
.795 .892 .913 r
.26168 .3859 .27285 .39088 .28956 .37945 .27853 .37392 Metetra
.796 .882 .908 r
.27853 .37392 .28956 .37945 .30632 .36791 .29544 .3618 Metetra
.796 .872 .903 r
.29544 .3618 .30632 .36791 .32312 .35629 .31238 .34957 Metetra
.795 .861 .898 r
.31238 .34957 .32312 .35629 .33997 .34461 .32937 .33726 Metetra
.793 .849 .893 r
.32937 .33726 .33997 .34461 .35686 .33291 .3464 .3249 Metetra
.791 .837 .887 r
.3464 .3249 .35686 .33291 .37379 .3212 .36346 .31253 Metetra
.787 .825 .882 r
.36346 .31253 .37379 .3212 .39077 .30952 .38056 .30018 Metetra
.783 .813 .877 r
.38056 .30018 .39077 .30952 .40778 .29788 .39769 .28789 Metetra
.778 .802 .872 r
.39769 .28789 .40778 .29788 .42483 .28631 .41485 .27568 Metetra
.773 .79 .867 r
.41485 .27568 .42483 .28631 .44193 .27483 .43206 .26358 Metetra
.767 .78 .863 r
.43206 .26358 .44193 .27483 .45908 .26345 .4493 .25161 Metetra
.762 .769 .859 r
.4493 .25161 .45908 .26345 .47627 .25219 .46658 .23979 Metetra
.756 .76 .856 r
.46658 .23979 .47627 .25219 .4935 .24106 .48391 .22815 Metetra
.75 .751 .853 r
.48391 .22815 .4935 .24106 .5108 .23009 .50128 .21669 Metetra
.744 .743 .85 r
.50128 .21669 .5108 .23009 .52814 .21926 .5187 .20544 Metetra
.739 .735 .848 r
.5187 .20544 .52814 .21926 .54555 .2086 .53618 .19439 Metetra
.733 .729 .847 r
.53618 .19439 .54555 .2086 .56302 .19811 .55372 .18355 Metetra
.728 .723 .845 r
.55372 .18355 .56302 .19811 .58056 .18779 .57133 .17293 Metetra
.723 .717 .845 r
.57133 .17293 .58056 .18779 .59817 .17765 .589 .16253 Metetra
.718 .713 .844 r
.589 .16253 .59817 .17765 .61585 .16767 .60675 .15234 Metetra
.713 .709 .845 r
.60675 .15234 .61585 .16767 .63361 .15787 .62457 .14236 Metetra
.709 .706 .845 r
.62457 .14236 .63361 .15787 .65146 .14823 .64249 .13259 Metetra
.705 .704 .846 r
.64249 .13259 .65146 .14823 .6694 .13876 .66049 .12302 Metetra
.701 .702 .847 r
.66049 .12302 .6694 .13876 .68742 .12944 .67858 .11363 Metetra
.697 .7 .849 r
.67858 .11363 .68742 .12944 .70555 .12027 .69677 .10442 Metetra
.694 .7 .851 r
.69677 .10442 .70555 .12027 .72377 .11125 .71507 .09538 Metetra
.691 .699 .853 r
.71507 .09538 .72377 .11125 .7421 .10235 .73347 .08649 Metetra
.702 .917 .967 r
.05231 .50578 .06506 .5086 .08093 .50018 .06823 .49771 Metetra
.709 .925 .964 r
.06823 .49771 .08093 .50018 .09692 .49142 .08426 .48925 Metetra
.718 .932 .96 r
.08426 .48925 .09692 .49142 .113 .48231 .10041 .48038 Metetra
.726 .938 .955 r
.10041 .48038 .113 .48231 .12919 .47286 .11668 .47109 Metetra
.736 .943 .951 r
.11668 .47109 .12919 .47286 .14547 .46305 .13305 .46138 Metetra
.746 .946 .946 r
.13305 .46138 .14547 .46305 .16185 .45291 .14953 .45124 Metetra
.756 .948 .942 r
.14953 .45124 .16185 .45291 .1783 .44245 .16611 .44068 Metetra
.766 .948 .937 r
.16611 .44068 .1783 .44245 .19484 .43167 .18278 .42971 Metetra
.776 .946 .933 r
.18278 .42971 .19484 .43167 .21145 .4206 .19953 .41836 Metetra
.785 .942 .929 r
.19953 .41836 .21145 .4206 .22813 .40926 .21636 .40664 Metetra
.793 .936 .924 r
.21636 .40664 .22813 .40926 .24488 .39769 .23326 .39458 Metetra
.799 .929 .92 r
.23326 .39458 .24488 .39769 .26168 .3859 .25022 .38222 Metetra
.805 .919 .915 r
.25022 .38222 .26168 .3859 .27853 .37392 .26724 .3696 Metetra
.808 .908 .909 r
.26724 .3696 .27853 .37392 .29544 .3618 .28431 .35677 Metetra
.81 .895 .903 r
.28431 .35677 .29544 .3618 .31238 .34957 .30142 .34376 Metetra
.81 .881 .897 r
.30142 .34376 .31238 .34957 .32937 .33726 .31857 .33063 Metetra
.808 .866 .891 r
.31857 .33063 .32937 .33726 .3464 .3249 .33575 .31742 Metetra
.805 .85 .884 r
.33575 .31742 .3464 .3249 .36346 .31253 .35296 .30419 Metetra
.8 .835 .878 r
.35296 .30419 .36346 .31253 .38056 .30018 .3702 .29097 Metetra
.795 .819 .871 r
.3702 .29097 .38056 .30018 .39769 .28789 .38747 .27781 Metetra
.789 .804 .865 r
.38747 .27781 .39769 .28789 .41485 .27568 .40476 .26477 Metetra
.782 .789 .859 r
.40476 .26477 .41485 .27568 .43206 .26358 .42208 .25186 Metetra
.774 .775 .853 r
.42208 .25186 .43206 .26358 .4493 .25161 .43942 .23913 Metetra
.767 .762 .848 r
.43942 .23913 .4493 .25161 .46658 .23979 .4568 .22661 Metetra
.76 .75 .844 r
.4568 .22661 .46658 .23979 .48391 .22815 .47422 .21432 Metetra
.752 .739 .84 r
.47422 .21432 .48391 .22815 .50128 .21669 .49167 .20228 Metetra
.745 .729 .837 r
.49167 .20228 .50128 .21669 .5187 .20544 .50917 .1905 Metetra
.738 .72 .835 r
.50917 .1905 .5187 .20544 .53618 .19439 .52672 .179 Metetra
.732 .713 .833 r
.52672 .179 .53618 .19439 .55372 .18355 .54432 .16779 Metetra
.725 .706 .832 r
.54432 .16779 .55372 .18355 .57133 .17293 .56199 .15686 Metetra
.719 .701 .832 r
.56199 .15686 .57133 .17293 .589 .16253 .57972 .14621 Metetra
.714 .696 .832 r
.57972 .14621 .589 .16253 .60675 .15234 .59753 .13584 Metetra
.709 .693 .833 r
.59753 .13584 .60675 .15234 .62457 .14236 .61542 .12573 Metetra
.704 .69 .834 r
.61542 .12573 .62457 .14236 .64249 .13259 .63339 .11588 Metetra
.699 .688 .836 r
.63339 .11588 .64249 .13259 .66049 .12302 .65145 .10627 Metetra
.695 .687 .838 r
.65145 .10627 .66049 .12302 .67858 .11363 .66961 .0969 Metetra
.692 .687 .84 r
.66961 .0969 .67858 .11363 .69677 .10442 .68788 .08773 Metetra
.688 .687 .843 r
.68788 .08773 .69677 .10442 .71507 .09538 .70624 .07876 Metetra
.685 .688 .846 r
.70624 .07876 .71507 .09538 .73347 .08649 .72472 .06996 Metetra
.676 .916 .975 r
.03935 .50305 .05231 .50578 .06823 .49771 .05527 .49559 Metetra
.681 .928 .97 r
.05527 .49559 .06823 .49771 .08426 .48925 .07131 .4877 Metetra
.686 .939 .964 r
.07131 .4877 .08426 .48925 .10041 .48038 .08749 .47936 Metetra
.693 .949 .957 r
.08749 .47936 .10041 .48038 .11668 .47109 .1038 .47053 Metetra
.702 .957 .949 r
.1038 .47053 .11668 .47109 .13305 .46138 .12024 .46119 Metetra
.712 .964 .941 r
.12024 .46119 .13305 .46138 .14953 .45124 .1368 .4513 Metetra
.724 .969 .933 r
.1368 .4513 .14953 .45124 .16611 .44068 .15348 .44087 Metetra
.737 .973 .927 r
.15348 .44087 .16611 .44068 .18278 .42971 .17027 .42989 Metetra
.752 .975 .922 r
.17027 .42989 .18278 .42971 .19953 .41836 .18717 .41836 Metetra
.768 .975 .918 r
.18717 .41836 .19953 .41836 .21636 .40664 .20416 .40631 Metetra
.783 .973 .916 r
.20416 .40631 .21636 .40664 .23326 .39458 .22123 .39376 Metetra
.797 .967 .913 r
.22123 .39376 .23326 .39458 .25022 .38222 .23838 .38076 Metetra
.81 .958 .91 r
.23838 .38076 .25022 .38222 .26724 .3696 .25559 .36735 Metetra
.819 .946 .906 r
.25559 .36735 .26724 .3696 .28431 .35677 .27286 .3536 Metetra
.825 .931 .901 r
.27286 .3536 .28431 .35677 .30142 .34376 .29017 .33956 Metetra
.828 .913 .895 r
.29017 .33956 .30142 .34376 .31857 .33063 .30752 .32531 Metetra
.828 .892 .888 r
.30752 .32531 .31857 .33063 .33575 .31742 .32489 .31091 Metetra
.824 .871 .88 r
.32489 .31091 .33575 .31742 .35296 .30419 .34229 .29645 Metetra
.819 .849 .871 r
.34229 .29645 .35296 .30419 .3702 .29097 .3597 .282 Metetra
.811 .827 .862 r
.3597 .282 .3702 .29097 .38747 .27781 .37712 .26763 Metetra
.802 .805 .854 r
.37712 .26763 .38747 .27781 .40476 .26477 .39456 .25339 Metetra
.792 .785 .845 r
.39456 .25339 .40476 .26477 .42208 .25186 .412 .23936 Metetra
.782 .766 .838 r
.412 .23936 .42208 .25186 .43942 .23913 .42947 .22559 Metetra
.772 .748 .831 r
.42947 .22559 .43942 .23913 .4568 .22661 .44695 .21211 Metetra
.762 .733 .826 r
.44695 .21211 .4568 .22661 .47422 .21432 .46445 .19897 Metetra
.752 .719 .821 r
.46445 .19897 .47422 .21432 .49167 .20228 .48199 .18619 Metetra
.743 .707 .818 r
.48199 .18619 .49167 .20228 .50917 .1905 .49956 .17379 Metetra
.734 .697 .815 r
.49956 .17379 .50917 .1905 .52672 .179 .51717 .16178 Metetra
.726 .688 .813 r
.51717 .16178 .52672 .179 .54432 .16779 .53483 .15018 Metetra
.718 .681 .813 r
.53483 .15018 .54432 .16779 .56199 .15686 .55255 .13896 Metetra
.712 .676 .813 r
.55255 .13896 .56199 .15686 .57972 .14621 .57033 .12813 Metetra
.705 .672 .814 r
.57033 .12813 .57972 .14621 .59753 .13584 .5882 .11766 Metetra
.7 .669 .817 r
.5882 .11766 .59753 .13584 .61542 .12573 .60614 .10755 Metetra
.695 .668 .819 r
.60614 .10755 .61542 .12573 .63339 .11588 .62417 .09776 Metetra
.69 .667 .823 r
.62417 .09776 .63339 .11588 .65145 .10627 .64229 .08827 Metetra
.686 .668 .827 r
.64229 .08827 .65145 .10627 .66961 .0969 .66052 .07905 Metetra
.683 .67 .831 r
.66052 .07905 .66961 .0969 .68788 .08773 .67885 .07009 Metetra
.68 .672 .836 r
.67885 .07009 .68788 .08773 .70624 .07876 .69729 .06134 Metetra
.677 .675 .841 r
.69729 .06134 .70624 .07876 .72472 .06996 .71585 .05278 Metetra
.646 .896 .984 r
.02631 .49945 .03935 .50305 .05527 .49559 .04217 .49293 Metetra
.644 .913 .98 r
.04217 .49293 .05527 .49559 .07131 .4877 .05816 .48605 Metetra
.641 .929 .972 r
.05816 .48605 .07131 .4877 .08749 .47936 .07429 .47874 Metetra
.638 .943 .957 r
.07429 .47874 .08749 .47936 .1038 .47053 .09057 .47092 Metetra
.634 .952 .938 r
.09057 .47092 .1038 .47053 .12024 .46119 .107 .46253 Metetra
.632 .958 .916 r
.107 .46253 .12024 .46119 .1368 .4513 .12358 .4535 Metetra
.634 .963 .893 r
.12358 .4535 .1368 .4513 .15348 .44087 .1403 .44377 Metetra
.641 .966 .873 r
.1403 .44377 .15348 .44087 .17027 .42989 .15717 .43329 Metetra
.655 .971 .858 r
.15717 .43329 .17027 .42989 .18717 .41836 .17418 .42202 Metetra
.677 .977 .849 r
.17418 .42202 .18717 .41836 .20416 .40631 .19132 .40996 Metetra
.704 .985 .848 r
.19132 .40996 .20416 .40631 .22123 .39376 .20858 .39711 Metetra
.737 .992 .853 r
.20858 .39711 .22123 .39376 .23838 .38076 .22594 .38349 Metetra
.771 .996 .861 r
.22594 .38349 .23838 .38076 .25559 .36735 .24338 .36915 Metetra
.802 .994 .87 r
.24338 .36915 .25559 .36735 .27286 .3536 .2609 .35418 Metetra
.828 .984 .877 r
.2609 .35418 .27286 .3536 .29017 .33956 .27848 .33867 Metetra
.846 .966 .88 r
.27848 .33867 .29017 .33956 .30752 .32531 .29609 .32272 Metetra
.855 .94 .877 r
.29609 .32272 .30752 .32531 .32489 .31091 .31372 .30647 Metetra
.854 .908 .87 r
.31372 .30647 .32489 .31091 .34229 .29645 .33137 .29005 Metetra
.848 .873 .86 r
.33137 .29005 .34229 .29645 .3597 .282 .34901 .27359 Metetra
.836 .838 .848 r
.34901 .27359 .3597 .282 .37712 .26763 .36665 .25723 Metetra
.821 .805 .835 r
.36665 .25723 .37712 .26763 .39456 .25339 .38427 .24108 Metetra
.805 .773 .823 r
.38427 .24108 .39456 .25339 .412 .23936 .40188 .22526 Metetra
.789 .745 .812 r
.40188 .22526 .412 .23936 .42947 .22559 .41948 .20987 Metetra
.773 .72 .803 r
.41948 .20987 .42947 .22559 .44695 .21211 .43707 .19497 Metetra
.758 .699 .795 r
.43707 .19497 .44695 .21211 .46445 .19897 .45466 .18064 Metetra
.745 .681 .79 r
.45466 .18064 .46445 .19897 .48199 .18619 .47227 .1669 Metetra
.732 .667 .786 r
.47227 .1669 .48199 .18619 .49956 .17379 .48989 .15378 Metetra
.721 .655 .784 r
.48989 .15378 .49956 .17379 .51717 .16178 .50755 .14128 Metetra
.711 .646 .783 r
.50755 .14128 .51717 .16178 .53483 .15018 .52526 .12939 Metetra
.702 .64 .784 r
.52526 .12939 .53483 .15018 .55255 .13896 .54302 .11808 Metetra
.695 .637 .787 r
.54302 .11808 .55255 .13896 .57033 .12813 .56084 .10732 Metetra
.688 .635 .791 r
.56084 .10732 .57033 .12813 .5882 .11766 .57874 .09707 Metetra
.683 .635 .796 r
.57874 .09707 .5882 .11766 .60614 .10755 .59673 .08728 Metetra
.678 .638 .802 r
.59673 .08728 .60614 .10755 .62417 .09776 .61482 .0779 Metetra
.675 .641 .809 r
.61482 .0779 .62417 .09776 .64229 .08827 .63301 .06887 Metetra
.672 .646 .816 r
.63301 .06887 .64229 .08827 .66052 .07905 .65131 .06015 Metetra
.669 .652 .824 r
.65131 .06015 .66052 .07905 .67885 .07009 .66972 .05169 Metetra
.668 .658 .832 r
.66972 .05169 .67885 .07009 .69729 .06134 .68825 .04344 Metetra
.666 .666 .84 r
.68825 .04344 .69729 .06134 .71585 .05278 .7069 .03537 Metetra
.637 .835 .976 r
.0135 .49295 .02631 .49945 .04217 .49293 .02928 .48738 Metetra
.632 .855 .984 r
.02928 .48738 .04217 .49293 .05816 .48605 .04517 .48168 Metetra
.622 .879 .989 r
.04517 .48168 .05816 .48605 .07429 .47874 .06119 .4758 Metetra
.605 .902 .986 r
.06119 .4758 .07429 .47874 .09057 .47092 .07732 .46968 Metetra
.577 .918 .966 r
.07732 .46968 .09057 .47092 .107 .46253 .0936 .46322 Metetra
.533 .919 .919 r
.0936 .46322 .107 .46253 .12358 .4535 .11002 .4563 Metetra
.476 .896 .843 r
.11002 .4563 .12358 .4535 .1403 .44377 .12661 .44876 Metetra
.415 .853 .745 r
.12661 .44876 .1403 .44377 .15717 .43329 .14337 .44045 Metetra
.362 .803 .645 r
.14337 .44045 .15717 .43329 .17418 .42202 .16032 .43116 Metetra
.16032 .43116 .17418 .42202 .19132 .40996 .17746 .42071 Metetra
.17746 .42071 .19132 .40996 .20858 .39711 .1948 .40894 Metetra
.1948 .40894 .20858 .39711 .22594 .38349 .21232 .39572 Metetra
.21232 .39572 .22594 .38349 .24338 .36915 .23001 .38097 Metetra
.495 .831 .541 r
.23001 .38097 .24338 .36915 .2609 .35418 .24785 .3647 Metetra
.609 .902 .614 r
.24785 .3647 .2609 .35418 .27848 .33867 .26581 .34701 Metetra
.731 .966 .699 r
.26581 .34701 .27848 .33867 .29609 .32272 .28385 .32809 Metetra
.832 .997 .774 r
.28385 .32809 .29609 .32272 .31372 .30647 .30193 .30819 Metetra
.889 .981 .817 r
.30193 .30819 .31372 .30647 .33137 .29005 .32001 .28764 Metetra
.9 .928 .826 r
.32001 .28764 .33137 .29005 .34901 .27359 .33807 .26682 Metetra
.882 .86 .815 r
.33807 .26682 .34901 .27359 .36665 .25723 .35607 .2461 Metetra
.85 .794 .796 r
.35607 .2461 .36665 .25723 .38427 .24108 .374 .22583 Metetra
.816 .736 .776 r
.374 .22583 .38427 .24108 .40188 .22526 .39185 .20632 Metetra
.784 .689 .759 r
.39185 .20632 .40188 .22526 .41948 .20987 .40963 .18782 Metetra
.756 .651 .745 r
.40963 .18782 .41948 .20987 .43707 .19497 .42734 .1705 Metetra
.732 .622 .736 r
.42734 .1705 .43707 .19497 .45466 .18064 .44501 .15445 Metetra
.712 .601 .731 r
.44501 .15445 .45466 .18064 .47227 .1669 .46265 .13971 Metetra
.695 .586 .729 r
.46265 .13971 .47227 .1669 .48989 .15378 .48028 .12622 Metetra
.682 .577 .731 r
.48028 .12622 .48989 .15378 .50755 .14128 .49794 .11391 Metetra
.671 .572 .735 r
.49794 .11391 .50755 .14128 .52526 .12939 .51564 .10264 Metetra
.663 .573 .743 r
.51564 .10264 .52526 .12939 .54302 .11808 .53341 .09229 Metetra
.658 .577 .753 r
.53341 .09229 .54302 .11808 .56084 .10732 .55125 .08271 Metetra
.654 .585 .764 r
.55125 .08271 .56084 .10732 .57874 .09707 .56919 .07374 Metetra
.653 .595 .777 r
.56919 .07374 .57874 .09707 .59673 .08728 .58723 .06526 Metetra
.652 .608 .792 r
.58723 .06526 .59673 .08728 .61482 .0779 .60538 .05714 Metetra
.652 .622 .806 r
.60538 .05714 .61482 .0779 .63301 .06887 .62365 .04929 Metetra
.653 .637 .821 r
.62365 .04929 .63301 .06887 .65131 .06015 .64204 .04164 Metetra
.655 .652 .835 r
.64204 .04164 .65131 .06015 .66972 .05169 .66055 .0341 Metetra
.656 .666 .848 r
.66055 .0341 .66972 .05169 .68825 .04344 .67919 .02663 Metetra
.658 .679 .859 r
.67919 .02663 .68825 .04344 .7069 .03537 .69795 .0192 Metetra
.645 .778 .944 r
.00095 .48363 .0135 .49295 .02928 .48738 .01677 .47812 Metetra
.645 .781 .946 r
.01677 .47812 .02928 .48738 .04517 .48168 .03269 .47257 Metetra
.644 .785 .949 r
.03269 .47257 .04517 .48168 .06119 .4758 .04871 .46699 Metetra
.643 .792 .954 r
.04871 .46699 .06119 .4758 .07732 .46968 .06483 .46137 Metetra
.641 .804 .961 r
.06483 .46137 .07732 .46968 .0936 .46322 .08106 .45571 Metetra
.637 .822 .971 r
.08106 .45571 .0936 .46322 .11002 .4563 .09739 .45002 Metetra
.627 .847 .984 r
.09739 .45002 .11002 .4563 .12661 .44876 .11383 .44429 Metetra
.603 .879 .992 r
.11383 .44429 .12661 .44876 .14337 .44045 .13038 .43853 Metetra
.541 .9 .971 r
.13038 .43853 .14337 .44045 .16032 .43116 .14703 .43272 Metetra
.398 .854 .852 r
.14703 .43272 .16032 .43116 .17746 .42071 .16379 .42688 Metetra
.16379 .42688 .17746 .42071 .1948 .40894 .18066 .421 Metetra
.074 0 0 r
.18066 .421 .1948 .40894 .21232 .39572 .19765 .41508 Metetra
.227 0 0 r
.19765 .41508 .21232 .39572 .23001 .38097 .21474 .40913 Metetra
.316 0 .139 r
.21474 .40913 .23001 .38097 .24785 .3647 .23195 .40313 Metetra
.368 .018 .233 r
.23195 .40313 .24785 .3647 .26581 .34701 .24927 .39709 Metetra
.401 .074 .293 r
.24927 .39709 .26581 .34701 .28385 .32809 .26671 .39101 Metetra
.422 .112 .333 r
.26671 .39101 .28385 .32809 .30193 .30819 .28427 .3849 Metetra
.437 .139 .361 r
.28427 .3849 .30193 .30819 .32001 .28764 .30194 .37872 Metetra
.28 .017 .324 r
.30194 .37872 .32001 .28764 .33807 .26682 .32131 .34802 Metetra
.942 .789 .329 r
.32131 .34802 .33807 .26682 .35607 .2461 .34984 .15869 Metetra
.639 .428 .581 r
.34984 .15869 .35607 .2461 .374 .22583 .36787 .13085 Metetra
.541 .35 .571 r
.36787 .13085 .374 .22583 .39185 .20632 .38475 .12424 Metetra
.547 .364 .584 r
.38475 .12424 .39185 .20632 .40963 .18782 .40174 .11761 Metetra
.555 .38 .6 r
.40174 .11761 .40963 .18782 .42734 .1705 .41884 .11093 Metetra
.564 .401 .619 r
.41884 .11093 .42734 .1705 .44501 .15445 .43605 .10421 Metetra
.574 .425 .641 r
.43605 .10421 .44501 .15445 .46265 .13971 .45337 .09744 Metetra
.586 .454 .668 r
.45337 .09744 .46265 .13971 .48028 .12622 .4708 .09063 Metetra
.598 .486 .697 r
.4708 .09063 .48028 .12622 .49794 .11391 .48835 .08378 Metetra
.611 .521 .729 r
.48835 .08378 .49794 .11391 .51564 .10264 .506 .07688 Metetra
.623 .558 .761 r
.506 .07688 .51564 .10264 .53341 .09229 .52377 .06994 Metetra
.634 .594 .792 r
.52377 .06994 .53341 .09229 .55125 .08271 .54166 .06296 Metetra
.643 .627 .819 r
.54166 .06296 .55125 .08271 .56919 .07374 .55966 .05592 Metetra
.65 .656 .843 r
.55966 .05592 .56919 .07374 .58723 .06526 .57778 .04885 Metetra
.655 .68 .862 r
.57778 .04885 .58723 .06526 .60538 .05714 .59602 .04172 Metetra
.658 .698 .876 r
.59602 .04172 .60538 .05714 .62365 .04929 .61438 .03455 Metetra
.66 .712 .886 r
.61438 .03455 .62365 .04929 .64204 .04164 .63286 .02733 Metetra
.661 .722 .894 r
.63286 .02733 .64204 .04164 .66055 .0341 .65147 .02007 Metetra
.662 .729 .899 r
.65147 .02007 .66055 .0341 .67919 .02663 .6702 .01275 Metetra
.663 .733 .902 r
.6702 .01275 .67919 .02663 .69795 .0192 .68905 .00539 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(90)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(95)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(100)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(105)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(110)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{179.438, 145.25},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002c0000TA000`40O003h00O/goo002cOol00;=o
o`00/goo002cOol00;=oo`00/goo002cOol00;=oo`00/goo001]Ool30003Ool30004Ool2000gOol0
06ioo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000fOol006ioo`03001oogoo00=oo`03
001oogoo009oo`04001oogoo000fOol006ioo`03001oogoo00=oo`03001oogoo009oo`04001oogoo
000fOol006ioo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000fOol006eoo`8000Aoo`80
00Eoo`8003Moo`00/goo002cOol00;=oo`00Ngoo0`00=Goo001BOol30004Ool20003Ool3000GOol3
00000eKl001oo`0eOol005=oo`03001oogoo009oo`04001oogoo0005Ool00`00Oomoo`0BOol30000
0eKl001Fo002000eOol005=oo`03001oogoo009oo`04001oogoo0005Ool00`00Oomoo`0?Ool50002
E_`6000cOol005=oo`03001oogoo009oo`04001oogoo0002Ool3000?Ool600001EKl001Fo000D[/0
0P0000=F^`00Ool0<goo001COol00`00Oomoo`02Ool01000Oomoo`000Woo00<007ooOol03Goo1@00
00AFg000E_aFo08000=B^`04001F^`00000cOol0059oo`8000Eoo`8000=oo`@000Uoo`D0009Fg003
001Fg00000800004DYX005:kD[/300000eJj0000000bOol006Uoo`D00004D[/005KLE]`20002DYX0
0`00DYX00002000015Jj001F^UJj0`00<Goo001VOol5000015:J001B^e:k0`0000ABN@00DYYBVP80
0004EYX005JjE[X300000eJj0000000aOol006Aoo`@0009BVP03001BVP0000800003DWT0059i009B
N@800006EYX005JJEYX005Jj0P000eJj0P00<7oo001QOol50000159i001BVU:J0P0000=BN000DWT0
0U9i00@0059i000000=FVP05001FVP00001F^P02000015Jj000000001goo0P0017oo0P000goo1000
6Woo000iOol30004Ool20004Ool2000FOol70003DWT200001U9H001BN59h001BN@<00004EYT005JJ
EYX300000eJj001F^P030002E]/20005Ool01000Oomoo`000goo0P0017oo00<007ooOol06Woo000j
Ool00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol001=oo`@00003CQL004hh009>>004001B
N@000002DUP00`00DWQBN00200000eJI001FV@02EYT2000015Jj001F^UJj0`0000AJ^P00E]]Ff`<0
00Aoo`04001oogoo000:Ool00`00Oomoo`0IOol003Yoo`03001oogoo009oo`04001oogoo0002Ool0
1000Oomoo`0047oo1@0000E=mP00CQM>5`00009>>0800003EUP0059H009BF0800006EYT005JIEYT0
05JI0P000eJj0P0000AJ^P00001J^P<00003F]/0000000Aoo`04001oogoo000;Ool00`00Oomoo`0H
Ool003Yoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`003Woo100000E9e@00COI=mP00
009>5`04001>>0000003EUP01000DUP000000eJI00@005JI0000009J^P04001F^P000003F[X20000
0e[K001Jf`02F]/30002Ool01000Oomoo`0027oo00@007ooOol001Yoo`00>Woo00<007ooOol00Woo
00@007ooOol0009oo`04001oogoo000;Ool500001DVd0019eDWE00000Tgf00@004hG0000009FE`03
001FF5IH00800004FYT005JIEYT3000015Zj001J^UZj0P000UZj00@005Zj000000=Jf`03001Jf`00
00<000=oo`8000Yoo`8001]oo`00>Goo0P001Goo0P0017oo0P002Woo100000E9T`00BKA9]0000099
e@03001=mTgf00800006EUL005IGEUL005IH0P000eZI0P0000AJ^P00F[YJ^P<00004F]X005ZjF[X2
000015[K000005[K0`0000AJf`00Ool002aoo`00Cgoo1@0000A5T`00BI<00099]0050019eDWE001=
mP020003EUL01000EUL000000UZI00@005ZI000000=J^P800003F]X005[J009JfP800003F]/005[K
00<00004F]/005[KF]/200000goo001oo`0ZOol004aoo`D00005AG8004FCAI<00002BI<01@00BKA9
]000BMD00P000UIG00<005IGEUL00P0000AJV@00FYUJV@80009J^P04001J^P000003F]X01000F]X0
00000U[K0`0000=Jf`00F]/00`0000EJf`00Oomoo`0002Yoo`00BWoo0P0000Moo`000015LP00AG80
0002AI<01000BI=9T`000TVd0P0000IJE`00EUMFE`00EUL20003FYT2000015Zj001J^UZj0P000U[J
00<005[J00000P0000EJf`00F]/005[K00800003F]/00000009Jf`06001Jf`00001oo`00:Woo000W
Ool30003Ool3000GOol40002Ool01P00AG95LP00AG800095T`040019TdVC0002BK@01000FUMJE`00
0UIG0P000UZI00@005ZI000000=J^P800004F]X005[JF]X20002F]/01000F]/000000e[K0P0000AJ
n`00F]]Jf`80009Jn`03001oo`0002Uoo`00:Woo00<007ooOol00goo00<007ooOol04Woo0P001Goo
0P000TEb00<004Eb00000dFC00<004VCBI<00`000eYG0P0000ANV@00FYUJV@80009J^P04001J^P00
0003F]X2000015[K001Jfe[K0P000U[K00@005[K0000009Jn`<00004F_/005[kF_/200000goo001o
o`0WOol002Qoo`<000Eoo`03001oogoo00moo`@000Ioo`800095LP030015LP0000=5T`040019T`00
0002FWL01000FUMJE`000ejI0P0000ANfP00F[YJ^P80009NfP04001JfP000003F]/2000015[K001J
fe[K0P0000IJn`00F_/005[k0002F_/3000015[k001oo`00:7oo000WOol01000Oomoo`000Woo0`00
47oo0P002Goo00<0056b00000TEb00@004EbAG800095T`800006GWL005YgFWL005YG0P000ejI00@0
05kJG]YNfP800004G]X005kJG]X2000015[K001Jfe[K0P000U[K0`0000=Jn`00F_/00P000e[k0P00
00AJn`00F_]Jn`800003Ool007oo02Ioo`009goo00@007ooOol0009oo`03001oogoo00eoo`<000]o
o`03001A/P000095LP040015LTEb0002AI<01000GWMNM`000UYg0P000Uji00@005jI000000=NfP04
001NfUkJG]X200000ekK001Jf`0300001U[k001Jf`00F_/0009Jn`<00003F_/005[k0080009Jn`<0
0005F_`007ooOol00005Ool20004Ool20004Ool3000BOol002Qoo`8000=oo`@000Yoo`8000ioo`03
001A/P0000=5LP040015LP00AI<20003GWL2000015ji001N^Eji0P000UkJ00@005kJ0000009NfP80
009Nf`80009Nn`04001Jn`000003F_/2000015[k000005[k0P000U[k0P0000AJo000F_aJo0800003
Ool007oo00=oo`04001oogoo0003Ool20005Ool00`00Oomoo`0AOol003Qoo`<0011oo`04001A/U6b
0002AG801P00AG800000HYL0009NM`8000=N^@800004G]X005kJG]X20002G]X200001Ukk001Nf`00
G_/0009Nn`80009Jn`06001Jn`00F_/005[k0`0000MJn`00F_/005[k001Jo00300001E[l001oogoo
00000goo00@007ooOol000Qoo`@0019oo`00=Goo10004Goo0P000U6b00H004EbAG8004Eb0003HYL2
000016;I001N^Eji0P000UkJ0`0000MNnP00G]X005kk001Nn`020003G_/2000015kk001Jne[k0P00
0U[k0P0000AJn`00001Jn`80009Jn`800004F_`005[lF_`20002Ool01000Oomoo`000Woo00<007oo
Ool01Woo00<007oo00004goo000cOol2000EOol20002DK801P00AG95LP00AG8000=RU`04001RfF;I
H]T200001V;j001NfP00G_X0009NnP80009Nn`80009Nn`04001Nn`000003G_/200001e[k001Jn`00
F_/005[k00<00007F_/005[k001Jo000F_`00`0000AJo000Oomoo`800005Ool007ooOol00008Ool0
0`00Ool0000COol0015oo`<000Aoo`8001Ioo`@001Ioo`80009A/P030015LTEb00800007I]P006:G
001Rn@00H]T00P000f;j00@005kjG_YNnP800006G_/005kk001Nn`000Ukk0P000Ukk00@005kk0000
009Jn`80009Jn`800004F_/00000F_/20002F_`20002F_`01000F_`000000Woo00<007ooOol00P00
2Woo0P004goo000DOol01000Oomoo`000Woo00<007ooOol04Goo0P006Woo00H007/Z001A/P00AG82
0002I]P20002H_T20002H_X01P00H_X006;j001NnP8000=Nn`04001NnekkG_/2000015kk001Nnekk
0P0000]Nn`00000005[k001Jn`00F_/005[k00<0009Jo0800004F_`005[lF_`20002F_`01000Oomo
o`008Woo000BOol30002Ool01000Oomoo`0047oo0`0077oo00@007/Z001A/P80000:J_P006KH001V
n@00H_T006;j0002H_X20002H_X20002G_/01P00G_/005kk001Nn`80009Nn`<00003G_/00000009N
n`04001Jn`000002F_/300000e[l001Jo0020002F_`300000e[l001Jo0020002Ool2000QOol0015o
o`04001oogoo0002Ool01000Oomoo`003Woo0P007goo00D007/Z001A/P0000=Zn005001VnFKiI_T0
0003H_X200001V;k001RnP00Ga/0009Nn`80009Nn`80009Nn`800006G_/005kk001Nn`000Ukk0P00
0U[l0P0000AJo000001Jo080009Jo0800004F_`005[lF_`200000e[l00000002Ool00`00Oomoo`0O
Ool0015oo`04001oogoo0002Ool01000Oomoo`002goo0`008Goo00D007/Z001A/P0000=Zn0800005
I_T006LJ001RnP020003H_/01000Ga]O6elK0P000UlK0P0000MO6`00G_/005lK001Nn`020003G_/2
00001e[l001Jo000F_`005[l00<00007F_`005[l001Jo000F_`00`0000=Jo00000000U[l00<007oo
00008Goo000BOol20004Ool20009Ool4000ROol00`00NbYk:P02000016li001Zn0000VLJ00<006LJ
IaX00P000V<K00X006;k001S6`00Ga/005lK001O6`80009O6`80009O6`80009O6`04001Nn`000002
F_`20002F_`2000015[l001Jo5[l0P000U[l0P0000IJo000F_`005[l0002F_`200000goo0000000O
Ool0025oo`8002Ioo`04001k:W/Z0002KcT20002IaX2000016LJ001S6`000V<K0P000V<K0P000UlK
0P000UlK0P0000IO6`00Ga/005lK0002Ga/200002ElL00000000Fa`005[l001K700500001e[l0000
0000F_`005[l0080009Jo0<00005F_`007ooOol0000OOol001ioo`<00003Ool007oo02Eoo`0:001k
:P00001_>@00JcX006LJ0002Ic/01000Ha]S6f<K0P000V<K0P0000]S6`00Ga/005lK001O6`00Ga/0
05lK00<00005Ga/00000001O70020002Ga`00`00Fa`0000600001E/L000005/L00000U[l0P000U[l
0P0000AJo000F_aJo0800003Ool007oo00Eoo`8000Aoo`8000Aoo`8000]oo`007Goo0P00:Woo0P00
0W=j00@006/jJcY[>P80000=Ic/006Lk001S6`00Ha/006<K001S6`00Ha/00P000UlK0P000UlK0P00
0UlK0P0000IO7000Ga`005lL0002Ga`600001e/L001K7000Fa`005/L00<00007Fa`005[l001K7000
F_`00`0000AJo000001oo`8000=oo`04001oogoo0003Ool20003Ool01000Oomoo`002Woo000GOol2
0004Ool00`00Ool0000YOol200001g=j001[F`00JcX006Lk0080009W>`80009S6`80009S6`80009O
6`80009O6`800007Ga`005lK001O7000Ga`00`0000=O700000000`0000EK7000Fa`005/L00800006
Fa`00000001K70000U/L0P000U/L0P0000IJo000F_`005[l0002Ool01@00Oomoogoo00000Woo00<0
07ooOol01Woo00@007ooOol000Yoo`005Woo00@007ooOol000=oo`04001oogoo000WOol2000017>j
001[Ff]K0P0001UW>`00Ic/006Lk001S>`00Ha/006<k001S6`00Ha/005lK001O7000Ga/005lL001O
70020002Ga`20002Ga`40002Fa`20002Fa`20002Fa`400001E/L00000000Fa`00`0000=K7000F_`0
0P000U[l0P0000=oo`00Ool00Woo00@007ooOol000Qoo`<000]oo`005Woo00@007ooOol000=oo`05
001oogooOol0000UOol00`0080X0000200001F]K001WFfMK00000fLk0P0000ES>`00000006<k0080
009S6`80009O7080009O7080009O70800007Ga`00000001O7000Ga`00`0000MK7000000005/L001K
7002000015/L000000000U/L0P0000AK7000FaaK70800006Fa`00000001K70000Woo0P0000Eoo`00
Oomoo`0000Qoo`03001oogoo00]oo`005Woo00@007ooOol0009oo`8000Aoo`03001oogoo025oo`04
000P2P000002Kg/200001fMK001WF`00Ic/006<k0080009S>`80009S?080009O70800009Ga`005lL
001O7000Ga`005lL00@0009O7080009O70800004Fa`005/LFa`200000e/L0000000200001U/L0000
05/L001K70<00005Fa`00000Fa`00002Fa`200000goo00000002Ool2000:Ool3000:Ool001Ioo`04
001oogoo0002Ool20005Ool00`00Oomoo`0POol01`0080X00000Kg/006]k0080009WF`80000AHc/0
06<l001S>`00Hc`006<l001S?000Ga`005lL001O70020002Ga`60002Ga`200001elL001O7000Fa`0
05/L00<00003Fa`00000009K7004001K70000002Fa`20002Fa`200000e/L001K700300001E/L0000
07oo00006goo000GOol20003Ool20006Ool00`00Oomoo`0NOol200001R0:00000000Jg/0009WG080
009WG0800005Hc`00000001S?0020002Hc`20002Ga`20002Ga`300002UlL000005lL001O7000Ga`0
05lL0P000U/L0P000U/L0P0000AK7000FaaK70H00003Fa`005/L0080009K7080009K70800004Fa`0
07ooOol2000IOol001aoo`03001oogoo00Ioo`03001oogoo01aoo`05000dR`0080X00002Jg`20000
1fML001WG000Ie`006<l0080009S?080009S?080009O?0800007Ga`00000001O7000Ga`00P000UlL
0P000UlL0P0000]O7000000005/L001K7000Fa`005/L00<00003Fa`000000080009K70800007Fa`0
0000001K7000Fa`00P000U/L0P000Woo00<007ooOol05goo000LOol20008Ool00`00Oomoo`0JOol3
000280X01000Jg`006Ml0P000VML0P0000eS?000Hc`006<l001S?000Hc`005ll001O?0060002Ga`2
00001ElL00000000Ga`00`0000=O700000000U/L00@005/L0000009K7080009K70800003Fa`005/L
00<00005Fa`00000Fa`00002Fa`200001E/L00000000Fa`00P0000=oo`00Ool067oo0004Ool20004
Ool20003Ool40002Ool30004Ool00`00Oomoo`08Ool00`00Oomoo`0HOol01000=6X000000R0:0P00
00AWO000IeaWG080009SG0800003Hc`00000009S?080009O?0<00004Gc`00000Ga`30002Ga`00`00
GaaO700200001UlL001O7000Ga`0009K70800007Fa`00000001K7000Fa`00P000U/L0P0000MK7000
000005/L001K700300002E/L000005/L001K7000Fa`00002Ool00`00Oomoo`0FOol000=oo`04001o
ogoo0003Ool20004Ool00`00Oomoo`05Ool01@00Oomoogoo00002goo00<007ooOol05Woo00D003AZ
000dR`00008P2P0;001WO000001WG000He`006=L001SG0020002Hc`20002Hc`20002Gc`200000ell
00000002Ga`02@00Ga`00000Ga`005lL001O700300001ElL00000000Fa`00P0000AK7000FaaK7080
0005Fa`00000001K70040002Fa`200001e/L00000000Fa`005/L0080009K7080009oo`03001oogoo
00Aoo`8000Aoo`8000Aoo`8000=oo`000goo00@007ooOol000Yoo`03001oogoo00Aoo`03001oogoo
008000aoo`03001oogoo01Eoo`0:000dJP00=8/0020:001WO000Ig`20002He`200001f=L001S?000
Hc`006<l00<00003Gc`005ll00800004Gc`005ll0002Ga`200001ElL00000000Ga`00P000UlL0P00
00MK7000Fa`005/L001K700300000e/L00000002Fa`200000e/L001K700300000e/L00000002Fa`0
1@00Fa`00000Fa`00`0000EK7000Oomoo`0000Eoo`04001oogoo0003Ool20003Ool01000Oomoo`00
0Woo0003Ool01000Oomoo`002goo00<007ooOol00`000goo00<007ooOol037oo0P0057oo0P0000Dd
JP00=8/0020:00800004Ig`006MLIe`20002He`200000f<l000000040002Gc`200001Ell00000000
Gc`00`0000=O700000000UlL0P000UlL0P0000=O7000Fa`00P000U/L0P0000]K7000000005/L001K
7000Fa`005/L00800006Fa`00000001K70000U/L0P0000EK7000001K7000009K7080009oo`03001o
ogoo009oo`04001oogoo0008Ool01000Oomoo`000Woo0003Ool01000Oomoo`0027oo00@007ooOol0
009oo`03001oogoo00=oo`8000moo`03001oogoo011oo`80008dJP04000dR`0080X400001fML001S
G000He`006=L00<00004Hc`00000Gc`300000ell00000002Gc`20002Gc`20002Ga`200000elL001O
70020002Ga`2000015/L000000000U/L00D005/LFa`005/L0080009K70800007Fa`00000001K7000
Fa`00`0000IK7000001K7000Fa`300000e/L00000002Ool01@00Oomoogoo00000Woo00<007ooOol0
1goo0P000goo0004Ool2000:Ool20003Ool40002Ool00`00Oomoo`0?Ool00`00Oomoo`0=Ool40000
1cAZ000dRcB;000P2P00009WO080009SG0@00005Hc`006<l001S?0020002Gc`200002ell001O?000
Gc`005ll001O?000Ga`00P000UlL0P0000EO7000000005/L0080009K7004001K70000002Fa`20000
1E/L00000000Fa`00P0000=K7000Fa`00`0000MK7000000005/L001K700200001U/L00000000Fa`0
009oo`05001oogooOol00002Ool00`00Oomoo`06Ool01000Oomoo`000Woo000KOol00`00Oomoo`0@
Ool00`00Oomoo`0;Ool300001c19000dJP00=8/0020:00800004Ig`006=LHe`6000036<l000006<l
001O?000Gc`005ll001O?0<00003Gc`00000009O?0800005Ga`00000001O70020002Ga`2000015/L
001K75/L0P0000EK7000000005/L00800007Fa`005/L001K7000Fa`01P0000AK7000FaaK70800005
Fa`00000Fa`00002Fa`200000goo00000003Ool2000:Ool20003Ool001]oo`80019oo`03001oogoo
00Uoo`800007;0L00319000dJP00=8/00P0000=WO00000000`0000=SG00000000V=L0P0000ES?000
000005ll00<00003Gc`00000009O?0800005Gc`00000001O70020002Ga`200001elL001O7000Fa`0
05/L00<00004Fa`00000Fa`300000e/L001K70020002Fa`200000e/L0000000300001E/L00000000
Fa`00P0000=K7000Fa`00`0000=K700000000Woo00<007ooOol04Goo0009Ool20004Ool20003Ool3
0004Ool00`00Oomoo`0BOol00`00Oomoo`07Ool01000:0@000000S1900@003AZ000dR`L00006He`0
06=L001SG0000V=L00<005llGc`00P000Ull0P000Ull0P000Ull00<005llGc`00P0000MO7000Ga`0
05lL001O7003000015/L000005/L0`000U/L00D005/LFa`005/L00800005Fa`00000001K70020000
0e/L001K700300001e/L00000000Fa`005/L00800005Fa`00000001K700200000goo0000000BOol0
00Qoo`04001oogoo0003Ool20006Ool00`00Oomoo`02000EOol00`00Oomoo`04Ool200002BP4000/
1`00<4T003AZ000dR`0600001F=L00000000He`00`00011SG000001O?000Gc`005ll001O?000Gc`0
05ll001O?0800005Gc`005ll001O700300000elL00000002Ga`200000e/L00000002Fa`01000Fa`0
00000U/L0P0000AK7000001K70<00003Fa`005/L00H00004Fa`005/LFa`2000015/L000005/L0`00
00=K7000Ool00P004Woo0008Ool01000Oomoo`002goo00<007ooOol00P005Woo00<007ooOol00Woo
0`0000DX1000;0L0031900H00005Hg`006=LHe`00002He`20002He`200001F<l00000000Gc`00`00
00=O?00000000Ull0P0000EO?000000005ll0080009O70800005Ga`00000001K7002000015/L001K
75/L0P0000EK7000000005/L0080009K7005001K75/L001K700200000e/L0000000300001E/L0000
0000Fa`00P0000EK7000001K7000009K7080009oo`03001oogoo00moo`0027oo00@007ooOol000Qo
o`<000=oo`03001oogoo01Ioo`03001oogoo00<0008X10P00007Hgd00000001SG000He`00P000V=L
0P0000ESG000000005ll0080009O?0800005Gc`00000001O?0020002Gc`200001Ell00000000Ga`0
0P0000MO7000Fa`005/L001K7003000015/L000005/L0`0000=K7000Fa`00P000U/L0P0000IK7000
001K7000Fa`300001E/L00000000Fa`00P0000=K7000Fa`00`0000AK7000001oo`80011oo`0027oo
00@007ooOol000Qoo`03001oogoo00=oo`03001oogoo01Moo`8000032000000000H00003Iid006=m
0080009SO@800007He`006=L001SG000He`00P000Ull0P0000EO?000000005ll0080009O?0800004
Gc`000000002Gc`00`00GcaO?00200000elL001O7003000015/L000005/L0`0000EK7000001K7000
009K70800005Fa`00000001K700200000e/L001K700600001e/L00000000Fa`005/L00800006Fa`0
0000001K70000Woo00<007ooOol00goo0`0027oo0009Ool20009Ool40002Ool00`00Oomoo`0FOol2
00000`P00000000300000fNm001WW@02000016=m001SOF=m0P0000ESG@00000006=L00800006He`0
05mL001O?0000Ull00<005llGc`00P000Ull0P000Ull00D005llGc`005ll00800005Gc`005ll001O
70020002Fa`200000e/L00000002Fa`200000e/L001K7003000015/L000005/L0`0000=K7000Fa`0
0P0000=K70000000100000AK7000FaaK70800005Fa`00000Fa`00002Fa`20002Ool00`00Oomoo`03
Ool00`00Oomoo`07Ool001Yoo`03001oogoo01Eoo`L00006Ikd00000Iid006=m0`0000=SO@00Hgd0
0P000V=M0P0000ESG000000005mL00<0000<Gc`00000Gc`005ll001O?000Gc`005ll0P000Ull0P00
00=O?00000000Ull0P0000EO7000000005/L0080009K70800003Fa`005/L00800005Fa`00000001K
7002000015/L001K75/L0P0000AK7000001K70D00003Fa`0000000@00003Fa`005/L00<00005Fa`0
0000Ool00005Ool00`00Oomoo`07Ool001Uoo`8001Eoo`X00003Hid000000080009SO@800004Hgd0
06=MHed200000f=M00000002Ge`20002Gc`200001Ell00000000Gc`00P000Ull0P0000EO?0000000
05ll00800005Gc`00000001O700200001e/L00000000Fa`005/L0080009K70800003Fa`00000009K
70800003Fa`005/L00<00004Fa`00000Fa`200000e/L001K7003000015/L000005/L1@0000=K7000
00000`000Woo00<007ooOol00Woo00<007ooOol01goo0002Ool20004Ool20004Ool00`00Oomoo`02
Ool30003Ool00`00Oomoo`0BOol8000016>m001SWF>M0P0000=SW@00Hgd00P0000MSO@00000006=M
001SG@020002Hed200001EmL00000000Gc`00P000Ull0P0000EO?000000005ll00800005Gc`00000
001O?0020002Gc`20002Ga`01@00FaaK7000Fa`00P0000EK7000000005/L00800007Fa`00000001K
7000Fa`00P0000EK7000000005/L00800003Fa`005/L00H00005Fa`00000Fa`00002Fa`200001U/L
000005/L001oo`<000=oo`03001oogoo00Moo`0000Eoo`00Oomoo`0000=oo`8000Aoo`03001oogoo
00Eoo`03001oogoo008000moo`H00005God006?M001S_@0300000f>M001SW@02000016=m001SOF=m
0P0000USG@00001SG@00Hed006=M00000UmM00<005llGc`00P0000EO?000000005ll0080009O?080
0003Gc`00000009O?080009O?0800003Gc`005lL0080009K70800004Fa`00000Fa`300001E/L0000
05/L00000U/L0P0000AK7000001K70<00003Fa`005/L00800003Fa`0000000@00003Fa`005/L00<0
0006Fa`00000Fa`005/L0P000Woo00<007ooOol00P002Goo00001Goo001oogoo00002Woo00<007oo
Ool017oo00@007ooOol000eoo``00004Hkd006>MHid400001f=m00000000Hgd006=m00800005Hed0
0000001SG@0300001EmM000005ll00000Ull00<005llGc`00P000Ull0P000Ull00<005llGc`00P00
00=O?000Gc`00P000Ull0P0000EK7000000005/L00800003Fa`00000009K70800003Fa`005/L00<0
0004Fa`00000Fa`2000015/L001K75/L0P0000AK7000001K70<00003Fa`005/L00800007Fa`00000
001K7000Fa`00P0000EK7000001oo`0000eoo`0000Eoo`00Oomoo`0000Yoo`04001oogooOol30003
Ool00`00Oomoo`09Ool:00000f?N001S_P0500000f>M001SO@020002Hgd2000016=m001SGF=M0P00
00=SG@0000000UmM0P0000EOG@00000005ll00800009Gc`005ll001O?000Gc`005ll00800005Gc`0
05ll001O?00200001Ell00000000Gc`00P0000EK7000000005/L00800007Fa`00000001K7000Fa`0
0P0000EK7000000005/L00800003Fa`005/L00<00004Fa`00000Fa`200000e/L001K7003000015/L
000005/L1@0000AK7000001K70<000eoo`0000Eoo`00Oomoo`0000]oo`04001oogoo0005Ool00`00
Oomoo`07Ool800000e_N001OgP0500000f>M001SW@0200000f=m001SO@0300001f=m00000000Hed0
06=M00800005Ged00000001OG@020002Gcd200001Ell00000000Gc`00P000Ull0P0000=O?0000000
0Ull0P0000=O?00000000Ull0P0000=K700000000U/L0P0000EK7000001K7000009K70800004Fa`0
0000Fa`300000e/L001K700200001E/L00000000Fa`00P0000=K7000Fa`01P0000IK7000001K7000
Fa`3000015/L000005/L0P003Goo0002Ool20009Ool40002Ool400000goo00000006Ool@0002Hih4
00000f=m0000000300000f=m00000002Hed200001F=M001OGEmM00000UmM0P0000EOG@00000005lm
0080009O?0800005Gc`00000001O?00200001Ell00000000Gc`00P0000EO?000000005/l00800005
Fa`00000001K700200000e/L001K7003000015/L000005/L0P000U/L100000AK7000001K70<00003
Fa`0000000800003Fa`0000000@00003Fa`005/L00800007Fa`00000001K7000Fa`00`0037oo000H
Ool20002Ool@00000enn001SWP0300000f>M001SW@0300000f=m001SO@0200001F=m00000000Hed0
0`0000=OG@00Ged00P0000EOG@00000005lm00800005Gcd00000001O?0020002Gc`200000ell0000
0002Gc`200000ell00000002Gc`200002U/l000005/L001K7000Fa`005/L0P000U/L0P0000AK7000
001K70<00003Fa`005/L00800004Fa`00000Fa`300000e/L001K700200001e/L00000000Fa`005/L
00800004Fa`00000Fa`500000e/L0000000<Ool001Qoo`L00003Dch0000000@00005Fmh005oN001O
_P0500000f>N000000020002Hgd2000016=m001SOF=m0P0000=SG@0000000UmM0P0000EOG@00GeeO
G@00009OG@800003Gcd00000009O?@800005Gc`00000001O?00200001Ell00000000Gc`00P0000EO
?000000005/l00800003Fa`005/L00<00004Fa`00000Fa`3000015/L000005/L0P0000AK7000FaaK
70800004Fa`00000Fa`2000015/L001K75/L0P0000AK7000001K70D00004Fa`00000Fa`200000e/L
001K700300000goo001oo`0;Ool0015oo`<000Aoo`@00003Dah0000000<00003Dkl005ON00L00003
Gih006>N00800003Hid006=m00<00007Hgd00000001SO@00Ged00P0000EOG@00000005mM00<00003
Ged005mM0080009O?@03001O?@0000800003Gcd00000009O?@800003Gc`00000009O?0800003Gc`0
0000009O?0800003Fc`005/L0080009K70800004Fa`00000Fa`300001U/L000005/L001K70<00004
Fa`00000Fa`200000e/L001K7003000015/L000005/L0P0000=K7000Fa`00`0000IK7000001K7000
Fa`20004Ool00`00Oomoo`0:Ool0019oo`03001oogoo00=oo`P00003Dkl0000000D00003Gkh005nN
00@00003Hid0000000<00005Hgd00000001SO@0400001EmM00000000Ged00P000UmM0P0000QOG@00
Ged005lm001O?@000elm00<005lm00000P0000AO?@00001O?0<0009O?008001O?000Gc`005/l001K
?0800005Fa`00000001K7002000015/L000005/L0`0000=K7000Fa`00P0000EK7000000005/L0080
0003Fa`0000000800005Fa`00000001K700200000e/L001K700600000e/L000000020006Ool00`00
Oomoo`0:Ool0019oo`03001oogoo00Eoo`05001oo`00001CW`060002Gkh400000enN001OWP030000
0f=m001OO@020002Ggd200001EmM001OGEmM00000UmM0P0000EOG@00000005mM0080009OG@800008
Gcd005lm001O?@00Gcd0009O?@80009O?@05001O?5ll001O?00200001Ell00000000Fc`00P0000=K
?00000000U/L0P0000AK7000001K7080009K7005001K7000001K700200001E/L00000000Fa`00P00
00=K7000Fa`00P0000EK7000000005/L00@00003Fa`0000000@000Yoo`03001oogoo00Yoo`004Woo
00<007ooOol01Woo00<007ooOol00`0000=K_P00Gkh01@0000=OWP0000000`0000IOO@00001OO@00
Ggd300001emm00000000Ged005mM00800005Ged00000001OG@0200001EmM00000000Gcd00P000Ulm
0P0000QO?@00001O?@00Gcd005lm0P000Ull0P0000=O?00000000U/l0P0000AK?000001K70<00004
Fa`00000Fa`2000015/L001K75/L0P0000AK7000001K70800007Fa`00000001K7000Fa`00P0000AK
7000001K70<00003Fa`005/L00800004Fa`00000000<Ool00`00Oomoo`0:Ool0019oo`03001oogoo
00Moo`03001oogoo009oo`@00003Gih005nN00<00003Gih005mn00800003Ggd0000000<00003Ggd0
0000009OG@800005Ged005mMGed00002Ged200001EmM00000000Gcd00P0000EO?@00000005lm0080
0005Gcd00000001O?@0200001Elm00000000Gcd00P0000EO?000000005/l00800004Fc`00000Fa`3
00001U/L000005/L001K70<00004Fa`00000Fa`200001E/L000005/L00000U/L0P0000AK7000001K
70800004Fa`005/LFa`200000e/L00000003Ool00`00Oomoo`09Ool00`00Oomoo`0:Ool0015oo`80
00Yoo`8000Ioo`<0009OWP@00007Ggd00000001OO@00Ggd00P0000EOG@00000005mM00<00003Ged0
000000800003Ged00000009OG@800003Gcd00000009O?@800005Gcd00000001O?@0200000elm0000
0002Gcd2000015lm000005/m0`0000AK?000001K?080009K70800003Fa`0000000800005Fa`00000
001K7002000015/L000005/L1@0000AK7000001K70800003Fa`0000000H000Eoo`03001oogoo00Yo
o`03001oogoo00Uoo`007goo00<007ooOol01Woo0P0000=OOP00Ggd00`0000=OO@0000000P0000EO
G@00000005mM0080009OG@@00004Ged00000Ged3000015lm000005lm0`0000=O?@0000000Ulm0P00
00AO?@00001O?@<00004Gcd00000Fcd20002Fcd2000015/l000005/l0`0000IK7000001K7000Fa`2
00001E/L00000000Fa`00P0000IK7000001K7000Fa`3000015/L000005/L0P0000=K7000Fa`00P00
2Goo00<007ooOol02Goo00<007ooOol02Goo000POol00`00Oomoo`07Ool30002Ggd200001Emm001O
OEmm00000UmM0P0000MOG@00000005mM001OG@0200001EmM00000000Ged00P0000EO?@00000005lm
00800005Gcd00000001O?@0200001Elm00000000Gcd00P0000AO?@00001K?@<00004Fcd00000Fcd2
00001e/L00000000Fa`005/L00800003Fa`00000009K70800006Fa`00000Fa`005/L0P0000EK7000
000005/L00800003Fa`00000008000aoo`03001oogoo00Qoo`03001oogoo00Uoo`008Goo00<007oo
Ool02Goo0P0000EOO@00000005mm00@00005Ged00000001OG@0200001EmM001OGEmM00000UmM0P00
00=O?@0000000Ulm0P0000=O?@0000000Ulm0P0000=O?@0000000Ulm0P0000AO?@00001K?@80009K
?@800004Fcd00000Fcd20002Fad01@00Fa`00000Fa`00P0000EK7000000005/L00800003Fa`00000
00800004Fa`00000Fa`300000e/L0000000@Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0029oo`03
001oogoo00Yoo`800003Ggd0000000800003Ged00000009OG@800005Ged00000001OG@04000015mM
000005lm0`0000AO?@00001O?@<00004Gcd00000Gcd3000015lm000005/m0P0000EK?@00000005/m
00800005Fcd00000Fcd00002Fad2000015/M000005/L0P0000EK7000000005/L00800003Fa`005/L
00800004Fa`00000Fa`3000COol00`00Oomoo`07Ool00`00Oomoo`09Ool002=oo`8000aoo`04001O
G@00Ged200001EmM00000000Ged00`0000=OG@0000000P0000AOG@00001O?@<00004Gcd00000Gcd3
000015lm000005lm0P000Ulm0P0000AO?@00001K?@80009K?@800004Fcd00000Fcd5000015/M0000
05/M0P0000AK7@00001K7@<00003Fad0000000800004Fa`00000Fa`2000GOol00`00Oomoo`06Ool0
0`00Oomoo`09Ool002Eoo`03001oogoo00Yoo`<00005Ged00000001OG@020002Ged200000emM001O
G@0200000elm00000002Gcd200000elm00000002Gcd2000015lm000005lm0`0000AO?@00001K?@80
0005Fcd00000001K?@0200001E/m000005/m00000U/M0P0000AK7@00001K7@800004Fad00000Fad3
00000e/M001K7@0200000e/L0000000JOol00`00Oomoo`05Ool00`00Oomoo`09Ool002Ioo`03001o
ogoo00aoo`800004Ged00000Ged300001UmM000005mM001OG@800005Gcd00000001O?@0200001Elm
00000000Gcd00P0000EO?@00000005lm00800004Fcd00000Fcd3000015/m000005/m0P0000=K?@00
Fcd00`0000AK7@00001K7@800004Fad00000Fad200000e/M0000000300000e/M0000000LOol00`00
Oomoo`05Ool00`00Oomoo`09Ool002Moo`03001oogoo00eoo`03001OG@0000800004Ged00000Ged4
00001Elm00000000Gcd00P0000EO?@00000005lm00800003Gcd00000009O?@800004Gcd00000Fcd2
0002Fcd2000015/m000005/m100000EK7@00000005/M00800004Fad00000Fad200001E/M000005/M
00000U/M0P007goo00<007ooOol017oo00<007ooOol02Goo000XOol2000>Ool2000015mM000005mM
0`0000=OG@0000000P0000AO?@00001O?@80009O?@800004Gcd00000Gcd3000015lm000005/m0P00
00EK?@00000005/m00800006Fcd00000Fcd005/m0P000U/M0P0000AK7@00001K7@800004Fad00000
Fad5000ROol00`00Oomoo`04Ool00`00Oomoo`08Ool002Yoo`03001oogoo00eoo`03001OG@000080
0004Ged000000002Gcd2000015lm000005lm0`0000AO?@00001O?@80009O?@800004Fcd00000Fcd2
0002Fcd2000015/m000005/m100000EK?@00000005/M00800004Fad00000Fad200000e/M0000000W
Ool00`00Oomoo`04Ool00`00Oomoo`08Ool002]oo`03001oogoo00eoo`03001OGEmM00800003Ged0
000000800004Gcd00000Gcd3000015lm000005lm0P0000EO?@00000005/m00800004Fcd00000Fcd3
000015/m000005/m0P0000=K?@0000000P000U/M0P0000AK7@00001K7@800003Fad0000002Yoo`03
001oogoo00=oo`03001oogoo00Qoo`00;7oo00<007ooOol03Goo0P0000=OG@0000000P0000AO?@00
001O?@<00004Gcd00000Gcd200001Elm00000000Gcd00P0000AK?@00001K?@<00004Fcd00000Fcd2
000015/m000005/m1@0000AK7@00001K7@800003Fad0000002eoo`03001oogoo009oo`03001oogoo
00Qoo`00;Goo0P003goo0`0000AO?@00001O?@80009O?@800004Gcd00000Gcd3000015lm000005/m
0P0000=K?@0000000U/m0P0000AK?@00001K?@800005Fcd00000Fcd00002Fcd2000015/M000005/M
0P00<7oo00D007ooOomoo`0000Yoo`00;goo00<007ooOol03goo00D005lm000005lm00<00004Gcd0
0000Gcd20002Gcd2000015/m000005/m0P0000EK?@00000005/m00800004Fcd00000Fcd200000e/m
00000003000015/M000005/M0P00<Woo00D007ooOomoo`0000Yoo`00<7oo00<007ooOol03goo00<0
05lm00000P0000AO?@00001O?@800005Gcd00000001K?@02000015/m000005/m0`0000AK?@00001K
?@800004Fcd00000Fcd200000e/m0000000300000e/M0000000eOol01000Oomoo`002Woo000aOol0
0`00Oomoo`0?Ool2000015lm000005lm0P0000EO?@00000005/m00800004Fcd00000Fcd20002Fcd2
000015/m000005/m0P0000AK?@00001K?@80009K?@@003Qoo`03001oo`0000Yoo`00<Woo0P004Goo
00D005lm000005lm00<00004Fcd00000Fcd2000015/m000005/m0`0000AK?@00001K?@800004Fcd0
0000Fcd2000015/m00000000?7oo00@007ooOol000Uoo`00=7oo00<007ooOol03goo00<005lm0000
0P0000AK?@00001K?@800004Fcd00000Fcd3000015/m000005/m0P0000AK?@00001K?@800003Fcd0
0000041oo`03001oo`0000Uoo`00=Goo00<007ooOol03goo0P0000AK?@00001K?@800004Fcd00000
Fcd3000015/m000005/m0P0000AK?@00001K?@800003Fcd0000004=oo`8000Uoo`00=Woo00<007oo
Ool047oo00D005/m000005/m00800005Fcd00000001K?@02000015/m000005/m0P0000AK?@00001K
?@8004Aoo`<000Uoo`00=goo00<007ooOol047oo00D005/m000005/m00H00004Fcd00000Fcd20000
15/m000005/m0P00@Woo100037oo000hOol2000AOol600000e/m0000000;0011Ool3000@Ool003Yo
o`03001oogoo015oo`05001K?@00001K?@02000015/m000005/m0P00@Goo10004goo000kOol00`00
Oomoo`0AOol01@00Fcd00000Fcd00P0000=K?@000000?goo10005goo000lOol00`00Oomoo`0AOol0
1@00Fcd00000Fcd00P00?Goo10006goo000mOol2000BOol01000Fcd00000>goo10007goo000oOol0
0`00Oomoo`0@Ool2000iOol3000SOol0041oo`03001oogoo00moo`03001oogoo03Aoo`@002Ioo`00
@Goo00<007ooOol03Woo00<007ooOol0<7oo1000:Woo0012Ool2000>Ool00`00Oomoo`0/Ool4000^
Ool004Aoo`03001oogoo00]oo`03001oogoo02Qoo`@0039oo`00AGoo00<007ooOol02Woo00<007oo
Ool09Goo0`00=Woo0016Ool00`00Oomoo`09Ool00`00Oomoo`0QOol4000iOol004Moo`8000Uoo`03
001oogoo01eoo`@003eoo`00BGoo00<007ooOol01Woo00<007ooOol06Goo1000@Goo001:Ool00`00
Oomoo`05Ool00`00Oomoo`0FOol30015Ool004]oo`03001oogoo00Aoo`03001oogoo019oo`@004Qo
o`00C7oo00<007ooOol00goo00<007ooOol03Woo1000C7oo001=Ool20003Ool00`00Oomoo`0:Ool4
001@Ool004moo`04001oogoo0008Ool4001DOol0051oo`03001oo`0000Eoo`<005Qoo`00DGoo0P00
00=oo`0000000P00Fgoo001BOol2001OOol00;=oo`00/goo002cOol00;=oo`00/goo002cOol00;=o
o`00\
\>"],
  ImageRangeCache->{{{0, 178.438}, {144.25, 0}} -> {-0.158971, -0.055358, 
  0.00654104, 0.00654104}}]
}, Open  ]],

Cell["\<\
The striking feature of these plots is the fact that the slope \
becomes vertical at the strike as maturity is approached, and elsewhere \
becomes zero. This fact plays havoc with the Greeks, as we shall see in \
Section 6.4.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["6.3 Greeks for Binary Cash Options", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[CapitalDelta] = BinaryCallDelta[B_, S_, K_, \[Sigma]_, r_, q_, \
t_] = 
D[BinaryCall[B, S, K, \[Sigma], r, q, t], S]; \
TraditionalForm[\[CapitalDelta]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(B\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ 
              \((\(\((r - q)\)\ t + log(S\/K)\)\/\(\@t\ \[Sigma]\) - 
                    \(\@t\ \[Sigma]\)\/2)\)\^2 - r\ t\)\)\/\(\@\(2\ \[Pi]\)\ S
        \ \@t\ \[Sigma]\)\)], "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[CapitalGamma] = BinaryCallGamma[B_, S_, K_, \[Sigma]_, r_, q_, \
t_] = 
D[BinaryCall[B, S, K, \[Sigma], r, q, t], {S, 2}];
TraditionalForm[\[CapitalGamma]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(-\(\(\[ExponentialE]\^\(\(-\(1\/2\)\)\ 
                    \((\(\((r - q)\)\ t + log(S\/K)\)\/\(\@t\ \[Sigma]\) - 
                          \(\@t\ \[Sigma]\)\/2)\)\^2 - r\ t\)\ 
              \((\(\((r - q)\)\ t + log(S\/K)\)\/\(\@t\ \[Sigma]\) - 
                  \(\@t\ \[Sigma]\)\/2)\)\ B\)\/\(\@\(2\ \[Pi]\)\ S\^2\ t\ 
              \[Sigma]\^2\)\)\) - 
      \(\[ExponentialE]\^\(\(-\(1\/2\)\)\ 
                \((\(\((r - q)\)\ t + log(S\/K)\)\/\(\@t\ \[Sigma]\) - 
                      \(\@t\ \[Sigma]\)\/2)\)\^2 - r\ t\)\ B
          \)\/\(\@\(2\ \[Pi]\)\ S\^2\ \@t\ \[Sigma]\)\)], "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[Theta] = BinaryCallTheta[B_, S_, K_, \[Sigma]_, r_, q_, t_] = 
-D[BinaryCall[B, S, K, \[Sigma], r, q, t], t]; TraditionalForm[\[Theta]]\
\>",
   "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`1\/2\ B\ \[ExponentialE]\^\(\(-r\)\ t\)\ r\ 
        \((erf(\(\(\((r - q)\)\ t + log(S\/K)\)\/\(\@t\ \[Sigma]\) - 
                  \(\@t\ \[Sigma]\)\/2\)\/\@2) + 1)\) - 
      \(B\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ 
                \((\(\((r - q)\)\ t + log(S\/K)\)\/\(\@t\ \[Sigma]\) - 
                      \(\@t\ \[Sigma]\)\/2)\)\^2 - r\ t\)\ 
          \((\(r - q\)\/\(\@t\ \[Sigma]\) - \[Sigma]\/\(4\ \@t\) - 
              \(\((r - q)\)\ t + log(S\/K)\)\/\(2\ t\^\(3/2\)\ \[Sigma]\))
            \)\)\/\@\(2\ \[Pi]\)\)], "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[Rho] = BinaryCallRho[B_, S_, K_, \[Sigma]_, r_, q_, t_] = 
D[BinaryCall[B, S, K, \[Sigma], r, q, t], r]; TraditionalForm[\[Rho]]\
\>", 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(B\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ 
                \((\(\((r - q)\)\ t + log(S\/K)\)\/\(\@t\ \[Sigma]\) - 
                      \(\@t\ \[Sigma]\)\/2)\)\^2 - r\ t\)\ 
          \@t\)\/\(\@\(2\ \[Pi]\)\ \[Sigma]\) - 
      1\/2\ B\ \[ExponentialE]\^\(\(-r\)\ t\)\ t\ 
        \((erf(\(\(\((r - q)\)\ t + log(S\/K)\)\/\(\@t\ \[Sigma]\) - 
                  \(\@t\ \[Sigma]\)\/2\)\/\@2) + 1)\)\)], "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[CapitalLambda] = BinaryCallVega[B_, S_, K_, \[Sigma]_, r_, q_, \
t_] = 
D[BinaryCall[B, S, K, \[Sigma], r, q, t], \[Sigma]]; TraditionalForm[\
\[CapitalLambda]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(B\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ 
              \((\(\((r - q)\)\ t + log(S\/K)\)\/\(\@t\ \[Sigma]\) - 
                    \(\@t\ \[Sigma]\)\/2)\)\^2 - r\ t\)\ 
        \((\(-\(\(\((r - q)\)\ t + log(S\/K)\)\/\(\@t\ \[Sigma]\^2\)\)\) - 
            \@t\/2)\)\)\/\@\(2\ \[Pi]\)\)], "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Check of Differential Constraints", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We confirm the differential constraints linking the Greeks. First \
we check that the Black-Scholes equation is indeed satisfied, expressed in \
terms of Greeks:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(V\  = \ 
      BinaryCall[B, \ S, K, \[Sigma], r, q, 
        t]; \[IndentingNewLine]Simplify[\[Theta]\  + \ 
        S \((r - q)\) \[CapitalDelta]\  + \ 
        1/2\ \[Sigma]^2\ S^2\ \[CapitalGamma]\  - \ r\ V]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell["We also check the Vega-Gamma and Rho-Delta constraints:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[{\[CapitalLambda]\  - \ 
          S^2\ \[Sigma]\ t\ \[CapitalGamma], \ \[Rho]\  + \ 
          t\ \((V\  - \ S\ \[CapitalDelta])\)}]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0, 0}\)], "Output"]
}, Open  ]],

Cell["\<\
Note that the fact that these last two identities hold is very \
significant - it means that the identities linking Vega and Gamma, and Rho \
and Delta, hold even when these quantities have singular initial data. \
\>", 
  "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Put Variants", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The Greeks for the \"Put\" case can all be defined similarly - we \
do not repeat all the details and checks:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
BinaryPutDelta[B_, S_, K_, \[Sigma]_, r_, q_, t_] = 
D[BinaryPut[B, S, K, \[Sigma], r, q, t], S]; 
BinaryPutGamma[B_, S_, K_, \[Sigma]_, r_, q_, t_] = 
D[BinaryPut[B, S, K, \[Sigma], r, q, t], {S, 2}]; 
BinaryPutTheta[B_, S_, K_, \[Sigma]_, r_, q_, t_] = 
-D[BinaryPut[B, S, K, \[Sigma], r, q, t], t]; 
BinaryPutRho[B_, S_, K_, \[Sigma]_, r_, q_, t_] = 
D[BinaryPut[B, S, K, \[Sigma], r, q, t], r]; 
BinaryPutVega[B_, S_, K_, \[Sigma]_, r_, q_, t_] = 
D[BinaryPut[B, S, K, \[Sigma], r, q, t], \[Sigma]]; \
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["6.4 The Hedge Nightmare", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The hedge parameters for the Binary options are particularly \
interesting. Let's take a look at \[CapitalDelta] for a Call, over the range \
from near expiry to one year:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot3D[
      BinaryCallDelta[1, \ S, \ 100, \ 0.2, \ 0.1, \ 0, \ t], \ {S, \ 90, \ 
        110}, \ {t, \ 0.0001, \ 1}, \ PlotPoints\  -> \ 41, \ 
      PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
6.10623e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -16.7601 -9 ]
[.3345 .12542 1.2399 0 ]
[.50253 .05935 -15.8219 -9 ]
[.50253 .05935 2.17811 0 ]
[.68156 -0.01098 -14.8837 -9 ]
[.68156 -0.01098 3.11632 0 ]
[.76742 .09699 0 -6.16191 ]
[.76742 .09699 18 2.83809 ]
[.8281 .1899 0 -6.07249 ]
[.8281 .1899 18 2.92751 ]
[.88368 .27501 0 -5.9922 ]
[.88368 .27501 18 3.0078 ]
[.93477 .35326 0 -5.91971 ]
[.93477 .35326 18 3.08029 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.02411 .26511 -6 -2.74232 ]
[.02411 .26511 0 6.25768 ]
[.01566 .31809 -18 -2.78712 ]
[.01566 .31809 0 6.21288 ]
[.00691 .37293 -6 -2.83364 ]
[.00691 .37293 0 6.16636 ]
[-0.00215 .42973 -18 -2.882 ]
[-0.00215 .42973 0 6.118 ]
[-0.01154 .48859 -6 -2.9323 ]
[-0.01154 .48859 0 6.0677 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(90)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(95)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(100)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(105)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(110)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.75512 .10154 m
.74896 .10381 L
s
[(0.2)] .76742 .09699 -1 .36931 Mshowa
.81571 .19423 m
.80952 .19639 L
s
[(0.4)] .8281 .1899 -1 .34944 Mshowa
.87122 .27914 m
.86499 .2812 L
s
[(0.6)] .88368 .27501 -1 .3316 Mshowa
.92226 .35721 m
.916 .35918 L
s
[(0.8)] .93477 .35326 -1 .31549 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(1)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.7059 .02625 m
.70222 .02766 L
s
.72269 .05193 m
.71901 .05333 L
s
.73909 .07702 m
.7354 .0784 L
s
.77078 .1255 m
.76708 .12684 L
s
.78609 .14892 m
.78239 .15025 L
s
.80107 .17182 m
.79735 .17314 L
s
.83004 .21614 m
.82632 .21743 L
s
.84406 .23759 m
.84033 .23886 L
s
.85779 .25859 m
.85405 .25984 L
s
.88438 .29926 m
.88064 .30049 L
s
.89727 .31898 m
.89352 .32019 L
s
.90989 .33829 m
.90614 .33949 L
s
.93438 .37576 m
.93063 .37693 L
s
.94627 .39394 m
.94251 .39509 L
s
.95792 .41176 m
.95415 .41291 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
[(0)] .02411 .26511 1 -0.3906 Mshowa
.02792 .31343 m
.03405 .31109 L
s
[(0.5)] .01566 .31809 1 -0.38064 Mshowa
.01921 .36838 m
.02537 .3661 L
s
[(1)] .00691 .37293 1 -0.3703 Mshowa
.0102 .42529 m
.01637 .42307 L
s
[(1.5)] -0.00215 .42973 1 -0.35955 Mshowa
.00085 .48427 m
.00704 .48211 L
s
[(2)] -0.01154 .48859 1 -0.34838 Mshowa
.125 Mabswid
.03468 .27081 m
.03835 .26938 L
s
.03301 .28135 m
.03668 .27993 L
s
.03132 .29197 m
.035 .29056 L
s
.02963 .30266 m
.03331 .30125 L
s
.02621 .32426 m
.02989 .32287 L
s
.02448 .33518 m
.02816 .33379 L
s
.02273 .34617 m
.02642 .34479 L
s
.02098 .35723 m
.02467 .35586 L
s
.01744 .3796 m
.02113 .37824 L
s
.01565 .3909 m
.01934 .38955 L
s
.01384 .40228 m
.01754 .40094 L
s
.01203 .41375 m
.01573 .41241 L
s
.00835 .43692 m
.01206 .43559 L
s
.0065 .44863 m
.01021 .44731 L
s
.00463 .46042 m
.00834 .45911 L
s
.00275 .4723 m
.00646 .471 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.663 .742 .909 r
.39741 .59854 .40506 .60562 .41787 .60177 .41027 .59466 Metetra
.664 .742 .909 r
.41027 .59466 .41787 .60177 .43075 .59791 .42319 .59076 Metetra
.664 .742 .909 r
.42319 .59076 .43075 .59791 .44369 .59402 .43617 .58684 Metetra
.664 .742 .909 r
.43617 .58684 .44369 .59402 .4567 .59012 .44921 .5829 Metetra
.664 .742 .909 r
.44921 .5829 .4567 .59012 .46976 .58619 .46232 .57894 Metetra
.664 .742 .909 r
.46232 .57894 .46976 .58619 .48289 .58225 .47549 .57496 Metetra
.664 .742 .909 r
.47549 .57496 .48289 .58225 .49608 .57828 .48872 .57096 Metetra
.664 .742 .909 r
.48872 .57096 .49608 .57828 .50934 .57429 .50202 .56694 Metetra
.664 .742 .909 r
.50202 .56694 .50934 .57429 .52266 .57029 .51538 .5629 Metetra
.664 .742 .909 r
.51538 .5629 .52266 .57029 .53605 .56626 .52881 .55883 Metetra
.664 .742 .909 r
.52881 .55883 .53605 .56626 .5495 .56221 .5423 .55475 Metetra
.664 .742 .909 r
.5423 .55475 .5495 .56221 .56301 .55814 .55586 .55064 Metetra
.664 .742 .909 r
.55586 .55064 .56301 .55814 .57659 .55405 .56949 .54652 Metetra
.664 .742 .909 r
.56949 .54652 .57659 .55405 .59024 .54994 .58318 .54237 Metetra
.664 .742 .909 r
.58318 .54237 .59024 .54994 .60396 .54581 .59694 .5382 Metetra
.664 .742 .908 r
.59694 .5382 .60396 .54581 .61774 .54166 .61077 .53401 Metetra
.664 .742 .908 r
.61077 .53401 .61774 .54166 .63159 .53749 .62467 .5298 Metetra
.664 .742 .908 r
.62467 .5298 .63159 .53749 .64551 .53329 .63864 .52556 Metetra
.664 .742 .908 r
.63864 .52556 .64551 .53329 .6595 .52908 .65267 .52131 Metetra
.664 .742 .908 r
.65267 .52131 .6595 .52908 .67356 .52484 .66678 .51703 Metetra
.664 .742 .908 r
.66678 .51703 .67356 .52484 .68769 .52058 .68096 .51273 Metetra
.664 .742 .908 r
.68096 .51273 .68769 .52058 .70189 .5163 .69521 .50841 Metetra
.664 .742 .908 r
.69521 .50841 .70189 .5163 .71616 .51199 .70953 .50407 Metetra
.664 .742 .908 r
.70953 .50407 .71616 .51199 .7305 .50767 .72392 .4997 Metetra
.664 .742 .908 r
.72392 .4997 .7305 .50767 .74492 .50332 .73839 .49531 Metetra
.664 .742 .908 r
.73839 .49531 .74492 .50332 .7594 .49895 .75292 .4909 Metetra
.664 .742 .908 r
.75292 .4909 .7594 .49895 .77396 .49456 .76754 .48647 Metetra
.664 .742 .908 r
.76754 .48647 .77396 .49456 .7886 .49014 .78222 .48201 Metetra
.664 .742 .908 r
.78222 .48201 .7886 .49014 .80331 .48571 .79699 .47753 Metetra
.664 .742 .908 r
.79699 .47753 .80331 .48571 .81809 .48125 .81182 .47303 Metetra
.664 .742 .908 r
.81182 .47303 .81809 .48125 .83295 .47676 .82674 .4685 Metetra
.664 .742 .908 r
.82674 .4685 .83295 .47676 .84789 .47226 .84173 .46395 Metetra
.664 .742 .908 r
.84173 .46395 .84789 .47226 .8629 .46773 .8568 .45938 Metetra
.664 .742 .908 r
.8568 .45938 .8629 .46773 .87799 .46318 .87194 .45478 Metetra
.664 .742 .908 r
.87194 .45478 .87799 .46318 .89315 .4586 .88717 .45016 Metetra
.664 .742 .908 r
.88717 .45016 .89315 .4586 .9084 .454 .90247 .44552 Metetra
.664 .742 .908 r
.90247 .44552 .9084 .454 .92373 .44938 .91786 .44085 Metetra
.664 .742 .908 r
.91786 .44085 .92373 .44938 .93913 .44473 .93332 .43616 Metetra
.664 .742 .908 r
.93332 .43616 .93913 .44473 .95462 .44006 .94887 .43144 Metetra
.664 .742 .908 r
.94887 .43144 .95462 .44006 .97018 .43536 .9645 .4267 Metetra
.663 .742 .909 r
.38971 .5914 .39741 .59854 .41027 .59466 .4026 .58748 Metetra
.664 .742 .909 r
.4026 .58748 .41027 .59466 .42319 .59076 .41556 .58355 Metetra
.664 .742 .909 r
.41556 .58355 .42319 .59076 .43617 .58684 .42858 .5796 Metetra
.664 .742 .909 r
.42858 .5796 .43617 .58684 .44921 .5829 .44166 .57562 Metetra
.664 .742 .909 r
.44166 .57562 .44921 .5829 .46232 .57894 .45481 .57163 Metetra
.664 .742 .909 r
.45481 .57163 .46232 .57894 .47549 .57496 .46802 .56761 Metetra
.664 .742 .909 r
.46802 .56761 .47549 .57496 .48872 .57096 .48129 .56358 Metetra
.664 .742 .909 r
.48129 .56358 .48872 .57096 .50202 .56694 .49463 .55952 Metetra
.664 .742 .909 r
.49463 .55952 .50202 .56694 .51538 .5629 .50804 .55544 Metetra
.664 .742 .909 r
.50804 .55544 .51538 .5629 .52881 .55883 .52151 .55134 Metetra
.664 .742 .909 r
.52151 .55134 .52881 .55883 .5423 .55475 .53505 .54722 Metetra
.664 .742 .909 r
.53505 .54722 .5423 .55475 .55586 .55064 .54865 .54308 Metetra
.664 .742 .909 r
.54865 .54308 .55586 .55064 .56949 .54652 .56232 .53891 Metetra
.664 .742 .909 r
.56232 .53891 .56949 .54652 .58318 .54237 .57606 .53473 Metetra
.664 .742 .909 r
.57606 .53473 .58318 .54237 .59694 .5382 .58987 .53052 Metetra
.664 .742 .909 r
.58987 .53052 .59694 .5382 .61077 .53401 .60374 .52629 Metetra
.664 .742 .909 r
.60374 .52629 .61077 .53401 .62467 .5298 .61769 .52204 Metetra
.664 .742 .908 r
.61769 .52204 .62467 .5298 .63864 .52556 .6317 .51777 Metetra
.664 .742 .908 r
.6317 .51777 .63864 .52556 .65267 .52131 .64578 .51347 Metetra
.664 .742 .908 r
.64578 .51347 .65267 .52131 .66678 .51703 .65994 .50915 Metetra
.664 .742 .908 r
.65994 .50915 .66678 .51703 .68096 .51273 .67417 .50481 Metetra
.664 .742 .908 r
.67417 .50481 .68096 .51273 .69521 .50841 .68846 .50045 Metetra
.664 .742 .908 r
.68846 .50045 .69521 .50841 .70953 .50407 .70283 .49607 Metetra
.664 .742 .908 r
.70283 .49607 .70953 .50407 .72392 .4997 .71728 .49166 Metetra
.664 .742 .908 r
.71728 .49166 .72392 .4997 .73839 .49531 .73179 .48723 Metetra
.664 .742 .908 r
.73179 .48723 .73839 .49531 .75292 .4909 .74638 .48278 Metetra
.664 .742 .908 r
.74638 .48278 .75292 .4909 .76754 .48647 .76105 .4783 Metetra
.664 .742 .908 r
.76105 .4783 .76754 .48647 .78222 .48201 .77579 .4738 Metetra
.664 .742 .908 r
.77579 .4738 .78222 .48201 .79699 .47753 .79061 .46928 Metetra
.664 .742 .908 r
.79061 .46928 .79699 .47753 .81182 .47303 .8055 .46473 Metetra
.664 .742 .908 r
.8055 .46473 .81182 .47303 .82674 .4685 .82047 .46016 Metetra
.664 .742 .908 r
.82047 .46016 .82674 .4685 .84173 .46395 .83551 .45557 Metetra
.664 .742 .908 r
.83551 .45557 .84173 .46395 .8568 .45938 .85064 .45095 Metetra
.664 .742 .908 r
.85064 .45095 .8568 .45938 .87194 .45478 .86584 .44631 Metetra
.664 .742 .908 r
.86584 .44631 .87194 .45478 .88717 .45016 .88113 .44164 Metetra
.664 .742 .908 r
.88113 .44164 .88717 .45016 .90247 .44552 .89649 .43695 Metetra
.664 .742 .908 r
.89649 .43695 .90247 .44552 .91786 .44085 .91193 .43224 Metetra
.664 .742 .908 r
.91193 .43224 .91786 .44085 .93332 .43616 .92746 .4275 Metetra
.664 .742 .908 r
.92746 .4275 .93332 .43616 .94887 .43144 .94306 .42274 Metetra
.664 .742 .908 r
.94306 .42274 .94887 .43144 .9645 .4267 .95875 .41795 Metetra
.663 .742 .909 r
.38193 .58419 .38971 .5914 .4026 .58748 .39486 .58025 Metetra
.663 .742 .909 r
.39486 .58025 .4026 .58748 .41556 .58355 .40786 .57628 Metetra
.664 .742 .909 r
.40786 .57628 .41556 .58355 .42858 .5796 .42092 .57229 Metetra
.664 .742 .909 r
.42092 .57229 .42858 .5796 .44166 .57562 .43404 .56828 Metetra
.664 .742 .909 r
.43404 .56828 .44166 .57562 .45481 .57163 .44723 .56425 Metetra
.664 .742 .909 r
.44723 .56425 .45481 .57163 .46802 .56761 .46048 .5602 Metetra
.664 .742 .909 r
.46048 .5602 .46802 .56761 .48129 .56358 .4738 .55613 Metetra
.664 .742 .909 r
.4738 .55613 .48129 .56358 .49463 .55952 .48718 .55203 Metetra
.664 .742 .909 r
.48718 .55203 .49463 .55952 .50804 .55544 .50063 .54792 Metetra
.664 .742 .909 r
.50063 .54792 .50804 .55544 .52151 .55134 .51414 .54378 Metetra
.664 .742 .909 r
.51414 .54378 .52151 .55134 .53505 .54722 .52772 .53962 Metetra
.664 .742 .909 r
.52772 .53962 .53505 .54722 .54865 .54308 .54137 .53544 Metetra
.664 .742 .909 r
.54137 .53544 .54865 .54308 .56232 .53891 .55509 .53124 Metetra
.664 .742 .909 r
.55509 .53124 .56232 .53891 .57606 .53473 .56887 .52702 Metetra
.664 .742 .909 r
.56887 .52702 .57606 .53473 .58987 .53052 .58272 .52277 Metetra
.664 .742 .909 r
.58272 .52277 .58987 .53052 .60374 .52629 .59664 .5185 Metetra
.664 .742 .909 r
.59664 .5185 .60374 .52629 .61769 .52204 .61064 .51421 Metetra
.664 .742 .909 r
.61064 .51421 .61769 .52204 .6317 .51777 .6247 .5099 Metetra
.664 .742 .908 r
.6247 .5099 .6317 .51777 .64578 .51347 .63883 .50556 Metetra
.664 .742 .908 r
.63883 .50556 .64578 .51347 .65994 .50915 .65303 .5012 Metetra
.664 .742 .908 r
.65303 .5012 .65994 .50915 .67417 .50481 .66731 .49682 Metetra
.664 .742 .908 r
.66731 .49682 .67417 .50481 .68846 .50045 .68166 .49242 Metetra
.664 .742 .908 r
.68166 .49242 .68846 .50045 .70283 .49607 .69608 .48799 Metetra
.664 .742 .908 r
.69608 .48799 .70283 .49607 .71728 .49166 .71057 .48354 Metetra
.664 .742 .908 r
.71057 .48354 .71728 .49166 .73179 .48723 .72514 .47907 Metetra
.664 .742 .908 r
.72514 .47907 .73179 .48723 .74638 .48278 .73978 .47457 Metetra
.664 .742 .908 r
.73978 .47457 .74638 .48278 .76105 .4783 .7545 .47006 Metetra
.664 .742 .908 r
.7545 .47006 .76105 .4783 .77579 .4738 .7693 .46551 Metetra
.664 .742 .908 r
.7693 .46551 .77579 .4738 .79061 .46928 .78417 .46095 Metetra
.664 .742 .908 r
.78417 .46095 .79061 .46928 .8055 .46473 .79911 .45636 Metetra
.664 .742 .908 r
.79911 .45636 .8055 .46473 .82047 .46016 .81414 .45174 Metetra
.664 .742 .908 r
.81414 .45174 .82047 .46016 .83551 .45557 .82924 .4471 Metetra
.664 .742 .908 r
.82924 .4471 .83551 .45557 .85064 .45095 .84442 .44244 Metetra
.664 .742 .908 r
.84442 .44244 .85064 .45095 .86584 .44631 .85968 .43775 Metetra
.664 .742 .908 r
.85968 .43775 .86584 .44631 .88113 .44164 .87502 .43304 Metetra
.664 .742 .908 r
.87502 .43304 .88113 .44164 .89649 .43695 .89044 .4283 Metetra
.664 .742 .908 r
.89044 .4283 .89649 .43695 .91193 .43224 .90595 .42354 Metetra
.664 .742 .908 r
.90595 .42354 .91193 .43224 .92746 .4275 .92153 .41876 Metetra
.664 .742 .908 r
.92153 .41876 .92746 .4275 .94306 .42274 .9372 .41395 Metetra
.664 .742 .908 r
.9372 .41395 .94306 .42274 .95875 .41795 .95295 .40911 Metetra
.663 .742 .909 r
.37408 .57692 .38193 .58419 .39486 .58025 .38705 .57294 Metetra
.663 .742 .909 r
.38705 .57294 .39486 .58025 .40786 .57628 .40009 .56894 Metetra
.664 .742 .909 r
.40009 .56894 .40786 .57628 .42092 .57229 .41319 .56492 Metetra
.664 .742 .909 r
.41319 .56492 .42092 .57229 .43404 .56828 .42635 .56087 Metetra
.664 .742 .909 r
.42635 .56087 .43404 .56828 .44723 .56425 .43958 .55681 Metetra
.664 .742 .909 r
.43958 .55681 .44723 .56425 .46048 .5602 .45287 .55272 Metetra
.664 .742 .909 r
.45287 .55272 .46048 .5602 .4738 .55613 .46623 .54861 Metetra
.664 .742 .909 r
.46623 .54861 .4738 .55613 .48718 .55203 .47966 .54448 Metetra
.664 .742 .909 r
.47966 .54448 .48718 .55203 .50063 .54792 .49315 .54033 Metetra
.664 .742 .909 r
.49315 .54033 .50063 .54792 .51414 .54378 .50671 .53615 Metetra
.664 .742 .909 r
.50671 .53615 .51414 .54378 .52772 .53962 .52033 .53196 Metetra
.664 .743 .909 r
.52033 .53196 .52772 .53962 .54137 .53544 .53402 .52774 Metetra
.664 .743 .909 r
.53402 .52774 .54137 .53544 .55509 .53124 .54778 .5235 Metetra
.664 .743 .909 r
.54778 .5235 .55509 .53124 .56887 .52702 .56161 .51923 Metetra
.664 .743 .909 r
.56161 .51923 .56887 .52702 .58272 .52277 .57551 .51495 Metetra
.664 .743 .909 r
.57551 .51495 .58272 .52277 .59664 .5185 .58948 .51064 Metetra
.664 .743 .909 r
.58948 .51064 .59664 .5185 .61064 .51421 .60352 .50631 Metetra
.664 .743 .909 r
.60352 .50631 .61064 .51421 .6247 .5099 .61763 .50195 Metetra
.664 .743 .909 r
.61763 .50195 .6247 .5099 .63883 .50556 .63181 .49758 Metetra
.664 .743 .909 r
.63181 .49758 .63883 .50556 .65303 .5012 .64606 .49318 Metetra
.664 .743 .908 r
.64606 .49318 .65303 .5012 .66731 .49682 .66039 .48876 Metetra
.664 .743 .908 r
.66039 .48876 .66731 .49682 .68166 .49242 .67479 .48431 Metetra
.664 .743 .908 r
.67479 .48431 .68166 .49242 .69608 .48799 .68926 .47984 Metetra
.664 .742 .908 r
.68926 .47984 .69608 .48799 .71057 .48354 .7038 .47535 Metetra
.664 .742 .908 r
.7038 .47535 .71057 .48354 .72514 .47907 .71842 .47083 Metetra
.664 .742 .908 r
.71842 .47083 .72514 .47907 .73978 .47457 .73312 .4663 Metetra
.664 .742 .908 r
.73312 .4663 .73978 .47457 .7545 .47006 .74789 .46173 Metetra
.664 .742 .908 r
.74789 .46173 .7545 .47006 .7693 .46551 .76274 .45715 Metetra
.664 .742 .908 r
.76274 .45715 .7693 .46551 .78417 .46095 .77766 .45253 Metetra
.664 .742 .908 r
.77766 .45253 .78417 .46095 .79911 .45636 .79266 .4479 Metetra
.664 .742 .908 r
.79266 .4479 .79911 .45636 .81414 .45174 .80775 .44324 Metetra
.664 .742 .908 r
.80775 .44324 .81414 .45174 .82924 .4471 .8229 .43856 Metetra
.664 .742 .908 r
.8229 .43856 .82924 .4471 .84442 .44244 .83814 .43385 Metetra
.664 .742 .908 r
.83814 .43385 .84442 .44244 .85968 .43775 .85346 .42911 Metetra
.664 .742 .908 r
.85346 .42911 .85968 .43775 .87502 .43304 .86886 .42436 Metetra
.664 .742 .908 r
.86886 .42436 .87502 .43304 .89044 .4283 .88434 .41957 Metetra
.664 .742 .908 r
.88434 .41957 .89044 .4283 .90595 .42354 .8999 .41476 Metetra
.664 .742 .908 r
.8999 .41476 .90595 .42354 .92153 .41876 .91555 .40993 Metetra
.664 .742 .908 r
.91555 .40993 .92153 .41876 .9372 .41395 .93128 .40507 Metetra
.664 .742 .908 r
.93128 .40507 .9372 .41395 .95295 .40911 .94709 .40019 Metetra
.663 .742 .909 r
.36616 .56959 .37408 .57692 .38705 .57294 .37917 .56558 Metetra
.663 .742 .909 r
.37917 .56558 .38705 .57294 .40009 .56894 .39225 .56154 Metetra
.664 .742 .909 r
.39225 .56154 .40009 .56894 .41319 .56492 .40539 .55748 Metetra
.664 .742 .909 r
.40539 .55748 .41319 .56492 .42635 .56087 .41859 .5534 Metetra
.664 .742 .909 r
.41859 .5534 .42635 .56087 .43958 .55681 .43186 .5493 Metetra
.664 .742 .909 r
.43186 .5493 .43958 .55681 .45287 .55272 .4452 .54517 Metetra
.664 .742 .909 r
.4452 .54517 .45287 .55272 .46623 .54861 .4586 .54103 Metetra
.664 .742 .909 r
.4586 .54103 .46623 .54861 .47966 .54448 .47207 .53686 Metetra
.664 .742 .909 r
.47207 .53686 .47966 .54448 .49315 .54033 .4856 .53267 Metetra
.664 .743 .909 r
.4856 .53267 .49315 .54033 .50671 .53615 .4992 .52845 Metetra
.664 .743 .909 r
.4992 .52845 .50671 .53615 .52033 .53196 .51287 .52422 Metetra
.664 .743 .909 r
.51287 .52422 .52033 .53196 .53402 .52774 .52661 .51996 Metetra
.664 .743 .909 r
.52661 .51996 .53402 .52774 .54778 .5235 .54041 .51568 Metetra
.664 .743 .909 r
.54041 .51568 .54778 .5235 .56161 .51923 .55429 .51138 Metetra
.664 .743 .909 r
.55429 .51138 .56161 .51923 .57551 .51495 .56823 .50705 Metetra
.664 .743 .909 r
.56823 .50705 .57551 .51495 .58948 .51064 .58225 .50271 Metetra
.664 .743 .909 r
.58225 .50271 .58948 .51064 .60352 .50631 .59634 .49833 Metetra
.664 .743 .909 r
.59634 .49833 .60352 .50631 .61763 .50195 .61049 .49394 Metetra
.664 .743 .909 r
.61049 .49394 .61763 .50195 .63181 .49758 .62472 .48952 Metetra
.664 .743 .909 r
.62472 .48952 .63181 .49758 .64606 .49318 .63903 .48508 Metetra
.664 .743 .909 r
.63903 .48508 .64606 .49318 .66039 .48876 .6534 .48062 Metetra
.664 .743 .908 r
.6534 .48062 .66039 .48876 .67479 .48431 .66785 .47613 Metetra
.664 .743 .908 r
.66785 .47613 .67479 .48431 .68926 .47984 .68237 .47162 Metetra
.664 .743 .908 r
.68237 .47162 .68926 .47984 .7038 .47535 .69697 .46708 Metetra
.664 .743 .908 r
.69697 .46708 .7038 .47535 .71842 .47083 .71164 .46252 Metetra
.664 .743 .908 r
.71164 .46252 .71842 .47083 .73312 .4663 .72639 .45794 Metetra
.664 .742 .908 r
.72639 .45794 .73312 .4663 .74789 .46173 .74122 .45333 Metetra
.664 .742 .908 r
.74122 .45333 .74789 .46173 .76274 .45715 .75612 .4487 Metetra
.664 .742 .908 r
.75612 .4487 .76274 .45715 .77766 .45253 .7711 .44404 Metetra
.664 .742 .908 r
.7711 .44404 .77766 .45253 .79266 .4479 .78615 .43936 Metetra
.664 .742 .908 r
.78615 .43936 .79266 .4479 .80775 .44324 .80129 .43466 Metetra
.664 .742 .908 r
.80129 .43466 .80775 .44324 .8229 .43856 .81651 .42993 Metetra
.664 .742 .908 r
.81651 .42993 .8229 .43856 .83814 .43385 .8318 .42517 Metetra
.664 .742 .908 r
.8318 .42517 .83814 .43385 .85346 .42911 .84718 .42039 Metetra
.664 .742 .908 r
.84718 .42039 .85346 .42911 .86886 .42436 .86264 .41559 Metetra
.664 .742 .908 r
.86264 .41559 .86886 .42436 .88434 .41957 .87818 .41076 Metetra
.664 .742 .908 r
.87818 .41076 .88434 .41957 .8999 .41476 .8938 .4059 Metetra
.664 .742 .908 r
.8938 .4059 .8999 .41476 .91555 .40993 .90951 .40102 Metetra
.664 .742 .908 r
.90951 .40102 .91555 .40993 .93128 .40507 .9253 .39611 Metetra
.664 .742 .908 r
.9253 .39611 .93128 .40507 .94709 .40019 .94117 .39117 Metetra
.663 .742 .909 r
.35817 .56219 .36616 .56959 .37917 .56558 .37122 .55814 Metetra
.663 .742 .909 r
.37122 .55814 .37917 .56558 .39225 .56154 .38434 .55407 Metetra
.663 .742 .909 r
.38434 .55407 .39225 .56154 .40539 .55748 .39752 .54997 Metetra
.664 .742 .909 r
.39752 .54997 .40539 .55748 .41859 .5534 .41076 .54586 Metetra
.664 .742 .909 r
.41076 .54586 .41859 .5534 .43186 .5493 .42408 .54172 Metetra
.664 .742 .909 r
.42408 .54172 .43186 .5493 .4452 .54517 .43745 .53756 Metetra
.664 .742 .909 r
.43745 .53756 .4452 .54517 .4586 .54103 .4509 .53337 Metetra
.664 .742 .909 r
.4509 .53337 .4586 .54103 .47207 .53686 .46441 .52917 Metetra
.664 .743 .909 r
.46441 .52917 .47207 .53686 .4856 .53267 .47798 .52494 Metetra
.664 .743 .909 r
.47798 .52494 .4856 .53267 .4992 .52845 .49163 .52069 Metetra
.664 .743 .909 r
.49163 .52069 .4992 .52845 .51287 .52422 .50534 .51641 Metetra
.664 .743 .909 r
.50534 .51641 .51287 .52422 .52661 .51996 .51912 .51212 Metetra
.664 .743 .909 r
.51912 .51212 .52661 .51996 .54041 .51568 .53298 .5078 Metetra
.664 .743 .909 r
.53298 .5078 .54041 .51568 .55429 .51138 .5469 .50345 Metetra
.664 .743 .909 r
.5469 .50345 .55429 .51138 .56823 .50705 .56089 .49909 Metetra
.664 .743 .909 r
.56089 .49909 .56823 .50705 .58225 .50271 .57495 .4947 Metetra
.664 .743 .909 r
.57495 .4947 .58225 .50271 .59634 .49833 .58909 .49029 Metetra
.664 .743 .909 r
.58909 .49029 .59634 .49833 .61049 .49394 .60329 .48585 Metetra
.664 .743 .909 r
.60329 .48585 .61049 .49394 .62472 .48952 .61757 .48139 Metetra
.664 .743 .909 r
.61757 .48139 .62472 .48952 .63903 .48508 .63192 .47691 Metetra
.664 .743 .909 r
.63192 .47691 .63903 .48508 .6534 .48062 .64635 .4724 Metetra
.664 .743 .909 r
.64635 .4724 .6534 .48062 .66785 .47613 .66085 .46787 Metetra
.664 .743 .908 r
.66085 .46787 .66785 .47613 .68237 .47162 .67542 .46331 Metetra
.664 .743 .908 r
.67542 .46331 .68237 .47162 .69697 .46708 .69007 .45873 Metetra
.664 .743 .908 r
.69007 .45873 .69697 .46708 .71164 .46252 .7048 .45413 Metetra
.664 .743 .908 r
.7048 .45413 .71164 .46252 .72639 .45794 .7196 .4495 Metetra
.664 .743 .908 r
.7196 .4495 .72639 .45794 .74122 .45333 .73448 .44485 Metetra
.664 .743 .908 r
.73448 .44485 .74122 .45333 .75612 .4487 .74943 .44017 Metetra
.664 .742 .908 r
.74943 .44017 .75612 .4487 .7711 .44404 .76447 .43547 Metetra
.664 .742 .908 r
.76447 .43547 .7711 .44404 .78615 .43936 .77958 .43074 Metetra
.664 .742 .908 r
.77958 .43074 .78615 .43936 .80129 .43466 .79477 .42599 Metetra
.664 .742 .908 r
.79477 .42599 .80129 .43466 .81651 .42993 .81005 .42121 Metetra
.664 .742 .908 r
.81005 .42121 .81651 .42993 .8318 .42517 .8254 .41641 Metetra
.664 .742 .908 r
.8254 .41641 .8318 .42517 .84718 .42039 .84084 .41158 Metetra
.664 .742 .908 r
.84084 .41158 .84718 .42039 .86264 .41559 .85635 .40673 Metetra
.664 .742 .908 r
.85635 .40673 .86264 .41559 .87818 .41076 .87195 .40185 Metetra
.664 .742 .908 r
.87195 .40185 .87818 .41076 .8938 .4059 .88764 .39695 Metetra
.664 .742 .908 r
.88764 .39695 .8938 .4059 .90951 .40102 .90341 .39201 Metetra
.664 .742 .908 r
.90341 .39201 .90951 .40102 .9253 .39611 .91926 .38706 Metetra
.664 .742 .908 r
.91926 .38706 .9253 .39611 .94117 .39117 .9352 .38207 Metetra
.663 .742 .909 r
.35011 .55473 .35817 .56219 .37122 .55814 .3632 .55064 Metetra
.663 .742 .909 r
.3632 .55064 .37122 .55814 .38434 .55407 .37636 .54653 Metetra
.663 .742 .909 r
.37636 .54653 .38434 .55407 .39752 .54997 .38958 .5424 Metetra
.664 .742 .909 r
.38958 .5424 .39752 .54997 .41076 .54586 .40286 .53825 Metetra
.664 .742 .909 r
.40286 .53825 .41076 .54586 .42408 .54172 .41622 .53407 Metetra
.664 .742 .909 r
.41622 .53407 .42408 .54172 .43745 .53756 .42963 .52987 Metetra
.664 .743 .909 r
.42963 .52987 .43745 .53756 .4509 .53337 .44312 .52565 Metetra
.664 .743 .909 r
.44312 .52565 .4509 .53337 .46441 .52917 .45667 .52141 Metetra
.664 .743 .909 r
.45667 .52141 .46441 .52917 .47798 .52494 .47029 .51714 Metetra
.664 .743 .909 r
.47029 .51714 .47798 .52494 .49163 .52069 .48398 .51285 Metetra
.664 .743 .909 r
.48398 .51285 .49163 .52069 .50534 .51641 .49774 .50854 Metetra
.664 .743 .909 r
.49774 .50854 .50534 .51641 .51912 .51212 .51157 .5042 Metetra
.664 .743 .909 r
.51157 .5042 .51912 .51212 .53298 .5078 .52547 .49984 Metetra
.664 .743 .909 r
.52547 .49984 .53298 .5078 .5469 .50345 .53944 .49545 Metetra
.664 .743 .909 r
.53944 .49545 .5469 .50345 .56089 .49909 .55347 .49105 Metetra
.664 .743 .909 r
.55347 .49105 .56089 .49909 .57495 .4947 .56758 .48662 Metetra
.664 .743 .909 r
.56758 .48662 .57495 .4947 .58909 .49029 .58177 .48216 Metetra
.664 .743 .909 r
.58177 .48216 .58909 .49029 .60329 .48585 .59602 .47768 Metetra
.664 .743 .909 r
.59602 .47768 .60329 .48585 .61757 .48139 .61035 .47318 Metetra
.664 .743 .909 r
.61035 .47318 .61757 .48139 .63192 .47691 .62475 .46866 Metetra
.664 .743 .909 r
.62475 .46866 .63192 .47691 .64635 .4724 .63923 .4641 Metetra
.664 .743 .909 r
.63923 .4641 .64635 .4724 .66085 .46787 .65378 .45953 Metetra
.664 .743 .909 r
.65378 .45953 .66085 .46787 .67542 .46331 .6684 .45493 Metetra
.664 .743 .908 r
.6684 .45493 .67542 .46331 .69007 .45873 .68311 .45031 Metetra
.664 .743 .908 r
.68311 .45031 .69007 .45873 .7048 .45413 .69788 .44566 Metetra
.664 .743 .908 r
.69788 .44566 .7048 .45413 .7196 .4495 .71274 .44098 Metetra
.664 .743 .908 r
.71274 .44098 .7196 .4495 .73448 .44485 .72767 .43629 Metetra
.664 .743 .908 r
.72767 .43629 .73448 .44485 .74943 .44017 .74268 .43156 Metetra
.664 .743 .908 r
.74268 .43156 .74943 .44017 .76447 .43547 .75777 .42682 Metetra
.664 .743 .908 r
.75777 .42682 .76447 .43547 .77958 .43074 .77294 .42204 Metetra
.664 .742 .908 r
.77294 .42204 .77958 .43074 .79477 .42599 .78819 .41724 Metetra
.664 .742 .908 r
.78819 .41724 .79477 .42599 .81005 .42121 .80352 .41242 Metetra
.664 .742 .908 r
.80352 .41242 .81005 .42121 .8254 .41641 .81894 .40757 Metetra
.664 .742 .908 r
.81894 .40757 .8254 .41641 .84084 .41158 .83443 .40269 Metetra
.664 .742 .908 r
.83443 .40269 .84084 .41158 .85635 .40673 .85001 .39779 Metetra
.664 .742 .908 r
.85001 .39779 .85635 .40673 .87195 .40185 .86567 .39286 Metetra
.664 .742 .908 r
.86567 .39286 .87195 .40185 .88764 .39695 .88141 .3879 Metetra
.664 .742 .908 r
.88141 .3879 .88764 .39695 .90341 .39201 .89724 .38292 Metetra
.664 .742 .908 r
.89724 .38292 .90341 .39201 .91926 .38706 .91316 .37791 Metetra
.664 .742 .908 r
.91316 .37791 .91926 .38706 .9352 .38207 .92916 .37288 Metetra
.663 .742 .909 r
.34197 .5472 .35011 .55473 .3632 .55064 .35511 .54307 Metetra
.663 .742 .909 r
.35511 .54307 .3632 .55064 .37636 .54653 .3683 .53893 Metetra
.663 .742 .909 r
.3683 .53893 .37636 .54653 .38958 .5424 .38156 .53476 Metetra
.663 .742 .909 r
.38156 .53476 .38958 .5424 .40286 .53825 .39489 .53057 Metetra
.664 .742 .909 r
.39489 .53057 .40286 .53825 .41622 .53407 .40828 .52636 Metetra
.664 .742 .909 r
.40828 .52636 .41622 .53407 .42963 .52987 .42174 .52212 Metetra
.664 .743 .909 r
.42174 .52212 .42963 .52987 .44312 .52565 .43527 .51786 Metetra
.664 .743 .909 r
.43527 .51786 .44312 .52565 .45667 .52141 .44887 .51358 Metetra
.664 .743 .909 r
.44887 .51358 .45667 .52141 .47029 .51714 .46253 .50927 Metetra
.664 .743 .909 r
.46253 .50927 .47029 .51714 .48398 .51285 .47627 .50494 Metetra
.664 .743 .909 r
.47627 .50494 .48398 .51285 .49774 .50854 .49007 .50059 Metetra
.664 .743 .909 r
.49007 .50059 .49774 .50854 .51157 .5042 .50394 .49621 Metetra
.664 .743 .909 r
.50394 .49621 .51157 .5042 .52547 .49984 .51789 .49181 Metetra
.664 .743 .909 r
.51789 .49181 .52547 .49984 .53944 .49545 .5319 .48738 Metetra
.664 .743 .909 r
.5319 .48738 .53944 .49545 .55347 .49105 .54599 .48293 Metetra
.664 .743 .909 r
.54599 .48293 .55347 .49105 .56758 .48662 .56015 .47846 Metetra
.664 .743 .909 r
.56015 .47846 .56758 .48662 .58177 .48216 .57438 .47396 Metetra
.664 .743 .909 r
.57438 .47396 .58177 .48216 .59602 .47768 .58868 .46944 Metetra
.664 .743 .909 r
.58868 .46944 .59602 .47768 .61035 .47318 .60306 .4649 Metetra
.664 .743 .909 r
.60306 .4649 .61035 .47318 .62475 .46866 .61751 .46033 Metetra
.664 .743 .909 r
.61751 .46033 .62475 .46866 .63923 .4641 .63204 .45573 Metetra
.664 .743 .909 r
.63204 .45573 .63923 .4641 .65378 .45953 .64664 .45111 Metetra
.664 .743 .909 r
.64664 .45111 .65378 .45953 .6684 .45493 .66132 .44647 Metetra
.664 .743 .909 r
.66132 .44647 .6684 .45493 .68311 .45031 .67607 .4418 Metetra
.664 .743 .908 r
.67607 .4418 .68311 .45031 .69788 .44566 .6909 .43711 Metetra
.664 .743 .908 r
.6909 .43711 .69788 .44566 .71274 .44098 .70581 .43239 Metetra
.664 .743 .908 r
.70581 .43239 .71274 .44098 .72767 .43629 .7208 .42764 Metetra
.664 .743 .908 r
.7208 .42764 .72767 .43629 .74268 .43156 .73587 .42287 Metetra
.664 .743 .908 r
.73587 .42287 .74268 .43156 .75777 .42682 .75101 .41808 Metetra
.664 .743 .908 r
.75101 .41808 .75777 .42682 .77294 .42204 .76624 .41325 Metetra
.664 .743 .908 r
.76624 .41325 .77294 .42204 .78819 .41724 .78155 .40841 Metetra
.664 .743 .908 r
.78155 .40841 .78819 .41724 .80352 .41242 .79694 .40353 Metetra
.664 .742 .908 r
.79694 .40353 .80352 .41242 .81894 .40757 .81241 .39864 Metetra
.664 .742 .908 r
.81241 .39864 .81894 .40757 .83443 .40269 .82796 .39371 Metetra
.664 .742 .908 r
.82796 .39371 .83443 .40269 .85001 .39779 .8436 .38876 Metetra
.664 .742 .908 r
.8436 .38876 .85001 .39779 .86567 .39286 .85932 .38378 Metetra
.664 .742 .908 r
.85932 .38378 .86567 .39286 .88141 .3879 .87513 .37877 Metetra
.664 .742 .908 r
.87513 .37877 .88141 .3879 .89724 .38292 .89102 .37374 Metetra
.664 .742 .908 r
.89102 .37374 .89724 .38292 .91316 .37791 .907 .36868 Metetra
.664 .742 .908 r
.907 .36868 .91316 .37791 .92916 .37288 .92306 .36359 Metetra
.663 .742 .909 r
.33376 .5396 .34197 .5472 .35511 .54307 .34693 .53544 Metetra
.663 .742 .909 r
.34693 .53544 .35511 .54307 .3683 .53893 .36017 .53125 Metetra
.663 .742 .909 r
.36017 .53125 .3683 .53893 .38156 .53476 .37347 .52705 Metetra
.663 .742 .909 r
.37347 .52705 .38156 .53476 .39489 .53057 .38684 .52282 Metetra
.664 .742 .909 r
.38684 .52282 .39489 .53057 .40828 .52636 .40028 .51857 Metetra
.664 .743 .909 r
.40028 .51857 .40828 .52636 .42174 .52212 .41378 .51429 Metetra
.664 .743 .909 r
.41378 .51429 .42174 .52212 .43527 .51786 .42735 .50999 Metetra
.664 .743 .909 r
.42735 .50999 .43527 .51786 .44887 .51358 .44099 .50567 Metetra
.664 .743 .909 r
.44099 .50567 .44887 .51358 .46253 .50927 .4547 .50133 Metetra
.664 .743 .909 r
.4547 .50133 .46253 .50927 .47627 .50494 .46848 .49696 Metetra
.664 .743 .909 r
.46848 .49696 .47627 .50494 .49007 .50059 .48233 .49256 Metetra
.664 .743 .909 r
.48233 .49256 .49007 .50059 .50394 .49621 .49625 .48814 Metetra
.664 .743 .909 r
.49625 .48814 .50394 .49621 .51789 .49181 .51024 .4837 Metetra
.664 .743 .909 r
.51024 .4837 .51789 .49181 .5319 .48738 .5243 .47923 Metetra
.664 .743 .909 r
.5243 .47923 .5319 .48738 .54599 .48293 .53843 .47474 Metetra
.664 .743 .909 r
.53843 .47474 .54599 .48293 .56015 .47846 .55264 .47023 Metetra
.664 .743 .909 r
.55264 .47023 .56015 .47846 .57438 .47396 .56692 .46569 Metetra
.664 .743 .909 r
.56692 .46569 .57438 .47396 .58868 .46944 .58127 .46112 Metetra
.664 .743 .909 r
.58127 .46112 .58868 .46944 .60306 .4649 .5957 .45653 Metetra
.664 .743 .909 r
.5957 .45653 .60306 .4649 .61751 .46033 .6102 .45192 Metetra
.664 .743 .909 r
.6102 .45192 .61751 .46033 .63204 .45573 .62478 .44728 Metetra
.664 .743 .909 r
.62478 .44728 .63204 .45573 .64664 .45111 .63943 .44262 Metetra
.664 .743 .909 r
.63943 .44262 .64664 .45111 .66132 .44647 .65416 .43793 Metetra
.664 .743 .909 r
.65416 .43793 .66132 .44647 .67607 .4418 .66897 .43321 Metetra
.664 .743 .909 r
.66897 .43321 .67607 .4418 .6909 .43711 .68386 .42847 Metetra
.664 .743 .908 r
.68386 .42847 .6909 .43711 .70581 .43239 .69882 .42371 Metetra
.664 .743 .908 r
.69882 .42371 .70581 .43239 .7208 .42764 .71386 .41891 Metetra
.664 .743 .908 r
.71386 .41891 .7208 .42764 .73587 .42287 .72899 .4141 Metetra
.664 .743 .908 r
.72899 .4141 .73587 .42287 .75101 .41808 .74419 .40925 Metetra
.664 .743 .908 r
.74419 .40925 .75101 .41808 .76624 .41325 .75947 .40438 Metetra
.664 .743 .908 r
.75947 .40438 .76624 .41325 .78155 .40841 .77484 .39949 Metetra
.664 .743 .908 r
.77484 .39949 .78155 .40841 .79694 .40353 .79028 .39456 Metetra
.664 .743 .908 r
.79028 .39456 .79694 .40353 .81241 .39864 .80581 .38962 Metetra
.664 .742 .908 r
.80581 .38962 .81241 .39864 .82796 .39371 .82143 .38464 Metetra
.664 .742 .908 r
.82143 .38464 .82796 .39371 .8436 .38876 .83713 .37964 Metetra
.664 .742 .908 r
.83713 .37964 .8436 .38876 .85932 .38378 .85291 .37461 Metetra
.664 .742 .908 r
.85291 .37461 .85932 .38378 .87513 .37877 .86878 .36955 Metetra
.664 .742 .908 r
.86878 .36955 .87513 .37877 .89102 .37374 .88473 .36447 Metetra
.664 .742 .908 r
.88473 .36447 .89102 .37374 .907 .36868 .90077 .35936 Metetra
.664 .742 .908 r
.90077 .35936 .907 .36868 .92306 .36359 .9169 .35422 Metetra
.663 .742 .909 r
.32548 .53193 .33376 .5396 .34693 .53544 .33869 .52773 Metetra
.663 .742 .909 r
.33869 .52773 .34693 .53544 .36017 .53125 .35197 .52351 Metetra
.663 .742 .909 r
.35197 .52351 .36017 .53125 .37347 .52705 .36531 .51927 Metetra
.663 .742 .909 r
.36531 .51927 .37347 .52705 .38684 .52282 .37872 .515 Metetra
.663 .743 .909 r
.37872 .515 .38684 .52282 .40028 .51857 .3922 .51071 Metetra
.664 .743 .909 r
.3922 .51071 .40028 .51857 .41378 .51429 .40574 .5064 Metetra
.664 .743 .909 r
.40574 .5064 .41378 .51429 .42735 .50999 .41936 .50206 Metetra
.664 .743 .909 r
.41936 .50206 .42735 .50999 .44099 .50567 .43304 .4977 Metetra
.664 .743 .909 r
.43304 .4977 .44099 .50567 .4547 .50133 .4468 .49331 Metetra
.664 .743 .909 r
.4468 .49331 .4547 .50133 .46848 .49696 .46062 .4889 Metetra
.664 .743 .909 r
.46062 .4889 .46848 .49696 .48233 .49256 .47451 .48446 Metetra
.664 .743 .909 r
.47451 .48446 .48233 .49256 .49625 .48814 .48848 .48 Metetra
.664 .743 .909 r
.48848 .48 .49625 .48814 .51024 .4837 .50251 .47552 Metetra
.664 .743 .909 r
.50251 .47552 .51024 .4837 .5243 .47923 .51662 .47101 Metetra
.664 .743 .909 r
.51662 .47101 .5243 .47923 .53843 .47474 .5308 .46648 Metetra
.664 .743 .909 r
.5308 .46648 .53843 .47474 .55264 .47023 .54506 .46192 Metetra
.664 .743 .909 r
.54506 .46192 .55264 .47023 .56692 .46569 .55939 .45734 Metetra
.664 .743 .909 r
.55939 .45734 .56692 .46569 .58127 .46112 .57379 .45273 Metetra
.664 .743 .909 r
.57379 .45273 .58127 .46112 .5957 .45653 .58827 .44809 Metetra
.664 .743 .909 r
.58827 .44809 .5957 .45653 .6102 .45192 .60282 .44343 Metetra
.664 .743 .909 r
.60282 .44343 .6102 .45192 .62478 .44728 .61745 .43875 Metetra
.664 .743 .909 r
.61745 .43875 .62478 .44728 .63943 .44262 .63216 .43404 Metetra
.664 .743 .909 r
.63216 .43404 .63943 .44262 .65416 .43793 .64694 .4293 Metetra
.664 .743 .909 r
.64694 .4293 .65416 .43793 .66897 .43321 .6618 .42454 Metetra
.664 .743 .909 r
.6618 .42454 .66897 .43321 .68386 .42847 .67674 .41975 Metetra
.664 .743 .909 r
.67674 .41975 .68386 .42847 .69882 .42371 .69176 .41494 Metetra
.664 .743 .908 r
.69176 .41494 .69882 .42371 .71386 .41891 .70686 .4101 Metetra
.664 .743 .908 r
.70686 .4101 .71386 .41891 .72899 .4141 .72203 .40523 Metetra
.664 .743 .908 r
.72203 .40523 .72899 .4141 .74419 .40925 .73729 .40034 Metetra
.664 .743 .908 r
.73729 .40034 .74419 .40925 .75947 .40438 .75264 .39542 Metetra
.664 .743 .908 r
.75264 .39542 .75947 .40438 .77484 .39949 .76806 .39048 Metetra
.664 .743 .908 r
.76806 .39048 .77484 .39949 .79028 .39456 .78356 .38551 Metetra
.664 .743 .908 r
.78356 .38551 .79028 .39456 .80581 .38962 .79915 .38051 Metetra
.664 .743 .908 r
.79915 .38051 .80581 .38962 .82143 .38464 .81483 .37548 Metetra
.664 .743 .908 r
.81483 .37548 .82143 .38464 .83713 .37964 .83059 .37043 Metetra
.664 .742 .908 r
.83059 .37043 .83713 .37964 .85291 .37461 .84643 .36534 Metetra
.664 .742 .908 r
.84643 .36534 .85291 .37461 .86878 .36955 .86236 .36024 Metetra
.664 .742 .908 r
.86236 .36024 .86878 .36955 .88473 .36447 .87838 .3551 Metetra
.664 .742 .908 r
.87838 .3551 .88473 .36447 .90077 .35936 .89449 .34994 Metetra
.664 .742 .908 r
.89449 .34994 .90077 .35936 .9169 .35422 .91068 .34474 Metetra
.663 .742 .909 r
.31712 .52419 .32548 .53193 .33869 .52773 .33037 .51996 Metetra
.663 .742 .909 r
.33037 .51996 .33869 .52773 .35197 .52351 .34368 .5157 Metetra
.663 .742 .909 r
.34368 .5157 .35197 .52351 .36531 .51927 .35707 .51142 Metetra
.663 .742 .909 r
.35707 .51142 .36531 .51927 .37872 .515 .37052 .50711 Metetra
.663 .743 .909 r
.37052 .50711 .37872 .515 .3922 .51071 .38404 .50278 Metetra
.664 .743 .909 r
.38404 .50278 .3922 .51071 .40574 .5064 .39763 .49843 Metetra
.664 .743 .909 r
.39763 .49843 .40574 .5064 .41936 .50206 .41129 .49405 Metetra
.664 .743 .909 r
.41129 .49405 .41936 .50206 .43304 .4977 .42502 .48965 Metetra
.664 .743 .909 r
.42502 .48965 .43304 .4977 .4468 .49331 .43881 .48522 Metetra
.664 .743 .909 r
.43881 .48522 .4468 .49331 .46062 .4889 .45268 .48077 Metetra
.664 .743 .909 r
.45268 .48077 .46062 .4889 .47451 .48446 .46662 .47629 Metetra
.664 .743 .909 r
.46662 .47629 .47451 .48446 .48848 .48 .48063 .47179 Metetra
.664 .743 .909 r
.48063 .47179 .48848 .48 .50251 .47552 .49472 .46726 Metetra
.664 .743 .909 r
.49472 .46726 .50251 .47552 .51662 .47101 .50887 .46271 Metetra
.664 .743 .909 r
.50887 .46271 .51662 .47101 .5308 .46648 .5231 .45813 Metetra
.664 .743 .909 r
.5231 .45813 .5308 .46648 .54506 .46192 .53741 .45353 Metetra
.664 .743 .909 r
.53741 .45353 .54506 .46192 .55939 .45734 .55178 .4489 Metetra
.664 .743 .909 r
.55178 .4489 .55939 .45734 .57379 .45273 .56624 .44425 Metetra
.664 .743 .909 r
.56624 .44425 .57379 .45273 .58827 .44809 .58077 .43957 Metetra
.664 .743 .909 r
.58077 .43957 .58827 .44809 .60282 .44343 .59537 .43487 Metetra
.664 .743 .909 r
.59537 .43487 .60282 .44343 .61745 .43875 .61005 .43014 Metetra
.664 .743 .909 r
.61005 .43014 .61745 .43875 .63216 .43404 .62481 .42538 Metetra
.664 .743 .909 r
.62481 .42538 .63216 .43404 .64694 .4293 .63964 .4206 Metetra
.664 .743 .909 r
.63964 .4206 .64694 .4293 .6618 .42454 .65456 .41579 Metetra
.664 .743 .909 r
.65456 .41579 .6618 .42454 .67674 .41975 .66955 .41095 Metetra
.664 .743 .909 r
.66955 .41095 .67674 .41975 .69176 .41494 .68463 .40609 Metetra
.664 .743 .909 r
.68463 .40609 .69176 .41494 .70686 .4101 .69978 .4012 Metetra
.664 .743 .908 r
.69978 .4012 .70686 .4101 .72203 .40523 .71502 .39629 Metetra
.664 .743 .908 r
.71502 .39629 .72203 .40523 .73729 .40034 .73033 .39135 Metetra
.664 .743 .908 r
.73033 .39135 .73729 .40034 .75264 .39542 .74573 .38638 Metetra
.664 .743 .908 r
.74573 .38638 .75264 .39542 .76806 .39048 .76121 .38138 Metetra
.664 .743 .908 r
.76121 .38138 .76806 .39048 .78356 .38551 .77678 .37636 Metetra
.664 .743 .908 r
.77678 .37636 .78356 .38551 .79915 .38051 .79243 .37131 Metetra
.664 .743 .908 r
.79243 .37131 .79915 .38051 .81483 .37548 .80816 .36623 Metetra
.664 .743 .908 r
.80816 .36623 .81483 .37548 .83059 .37043 .82398 .36112 Metetra
.664 .743 .908 r
.82398 .36112 .83059 .37043 .84643 .36534 .83989 .35599 Metetra
.664 .742 .908 r
.83989 .35599 .84643 .36534 .86236 .36024 .85588 .35083 Metetra
.664 .742 .908 r
.85588 .35083 .86236 .36024 .87838 .3551 .87197 .34564 Metetra
.664 .742 .908 r
.87197 .34564 .87838 .3551 .89449 .34994 .88814 .34042 Metetra
.664 .742 .908 r
.88814 .34042 .89449 .34994 .91068 .34474 .9044 .33517 Metetra
.663 .742 .909 r
.30868 .51638 .31712 .52419 .33037 .51996 .32197 .51211 Metetra
.663 .742 .909 r
.32197 .51211 .33037 .51996 .34368 .5157 .33533 .50781 Metetra
.663 .742 .909 r
.33533 .50781 .34368 .5157 .35707 .51142 .34875 .50349 Metetra
.663 .743 .909 r
.34875 .50349 .35707 .51142 .37052 .50711 .36225 .49915 Metetra
.663 .743 .909 r
.36225 .49915 .37052 .50711 .38404 .50278 .37581 .49478 Metetra
.663 .743 .909 r
.37581 .49478 .38404 .50278 .39763 .49843 .38944 .49039 Metetra
.664 .743 .909 r
.38944 .49039 .39763 .49843 .41129 .49405 .40314 .48597 Metetra
.664 .743 .909 r
.40314 .48597 .41129 .49405 .42502 .48965 .41691 .48152 Metetra
.664 .743 .909 r
.41691 .48152 .42502 .48965 .43881 .48522 .43076 .47706 Metetra
.664 .743 .909 r
.43076 .47706 .43881 .48522 .45268 .48077 .44467 .47256 Metetra
.664 .743 .909 r
.44467 .47256 .45268 .48077 .46662 .47629 .45866 .46804 Metetra
.664 .743 .909 r
.45866 .46804 .46662 .47629 .48063 .47179 .47271 .4635 Metetra
.664 .743 .909 r
.47271 .4635 .48063 .47179 .49472 .46726 .48684 .45893 Metetra
.664 .743 .909 r
.48684 .45893 .49472 .46726 .50887 .46271 .50105 .45433 Metetra
.664 .743 .909 r
.50105 .45433 .50887 .46271 .5231 .45813 .51533 .44971 Metetra
.664 .743 .909 r
.51533 .44971 .5231 .45813 .53741 .45353 .52968 .44507 Metetra
.664 .743 .909 r
.52968 .44507 .53741 .45353 .55178 .4489 .54411 .44039 Metetra
.664 .743 .909 r
.54411 .44039 .55178 .4489 .56624 .44425 .55861 .43569 Metetra
.664 .743 .909 r
.55861 .43569 .56624 .44425 .58077 .43957 .57319 .43097 Metetra
.664 .743 .909 r
.57319 .43097 .58077 .43957 .59537 .43487 .58784 .42622 Metetra
.664 .743 .909 r
.58784 .42622 .59537 .43487 .61005 .43014 .60258 .42144 Metetra
.664 .743 .909 r
.60258 .42144 .61005 .43014 .62481 .42538 .61739 .41664 Metetra
.664 .743 .909 r
.61739 .41664 .62481 .42538 .63964 .4206 .63228 .41181 Metetra
.664 .743 .909 r
.63228 .41181 .63964 .4206 .65456 .41579 .64725 .40695 Metetra
.664 .743 .909 r
.64725 .40695 .65456 .41579 .66955 .41095 .6623 .40207 Metetra
.664 .743 .909 r
.6623 .40207 .66955 .41095 .68463 .40609 .67742 .39715 Metetra
.664 .743 .909 r
.67742 .39715 .68463 .40609 .69978 .4012 .69263 .39222 Metetra
.664 .743 .909 r
.69263 .39222 .69978 .4012 .71502 .39629 .70793 .38725 Metetra
.664 .743 .908 r
.70793 .38725 .71502 .39629 .73033 .39135 .7233 .38226 Metetra
.664 .743 .908 r
.7233 .38226 .73033 .39135 .74573 .38638 .73876 .37724 Metetra
.664 .743 .908 r
.73876 .37724 .74573 .38638 .76121 .38138 .7543 .37219 Metetra
.664 .743 .908 r
.7543 .37219 .76121 .38138 .77678 .37636 .76992 .36712 Metetra
.664 .743 .908 r
.76992 .36712 .77678 .37636 .79243 .37131 .78563 .36202 Metetra
.664 .743 .908 r
.78563 .36202 .79243 .37131 .80816 .36623 .80143 .35688 Metetra
.664 .743 .908 r
.80143 .35688 .80816 .36623 .82398 .36112 .81731 .35173 Metetra
.664 .743 .908 r
.81731 .35173 .82398 .36112 .83989 .35599 .83328 .34654 Metetra
.664 .743 .908 r
.83328 .34654 .83989 .35599 .85588 .35083 .84934 .34132 Metetra
.664 .742 .908 r
.84934 .34132 .85588 .35083 .87197 .34564 .86548 .33608 Metetra
.664 .742 .908 r
.86548 .33608 .87197 .34564 .88814 .34042 .88172 .33081 Metetra
.664 .742 .908 r
.88172 .33081 .88814 .34042 .9044 .33517 .89805 .3255 Metetra
.663 .742 .909 r
.30016 .5085 .30868 .51638 .32197 .51211 .31349 .50419 Metetra
.663 .742 .909 r
.31349 .50419 .32197 .51211 .33533 .50781 .32689 .49985 Metetra
.663 .742 .909 r
.32689 .49985 .33533 .50781 .34875 .50349 .34036 .49549 Metetra
.663 .743 .909 r
.34036 .49549 .34875 .50349 .36225 .49915 .35389 .49111 Metetra
.663 .743 .909 r
.35389 .49111 .36225 .49915 .37581 .49478 .3675 .4867 Metetra
.663 .743 .909 r
.3675 .4867 .37581 .49478 .38944 .49039 .38117 .48227 Metetra
.664 .743 .909 r
.38117 .48227 .38944 .49039 .40314 .48597 .39492 .47781 Metetra
.664 .743 .909 r
.39492 .47781 .40314 .48597 .41691 .48152 .40873 .47333 Metetra
.664 .743 .909 r
.40873 .47333 .41691 .48152 .43076 .47706 .42262 .46882 Metetra
.664 .743 .909 r
.42262 .46882 .43076 .47706 .44467 .47256 .43658 .46428 Metetra
.664 .743 .909 r
.43658 .46428 .44467 .47256 .45866 .46804 .45061 .45972 Metetra
.664 .743 .909 r
.45061 .45972 .45866 .46804 .47271 .4635 .46472 .45513 Metetra
.664 .743 .909 r
.46472 .45513 .47271 .4635 .48684 .45893 .4789 .45052 Metetra
.664 .743 .909 r
.4789 .45052 .48684 .45893 .50105 .45433 .49315 .44588 Metetra
.664 .743 .909 r
.49315 .44588 .50105 .45433 .51533 .44971 .50748 .44121 Metetra
.664 .743 .909 r
.50748 .44121 .51533 .44971 .52968 .44507 .52188 .43652 Metetra
.664 .743 .909 r
.52188 .43652 .52968 .44507 .54411 .44039 .53635 .4318 Metetra
.664 .743 .909 r
.53635 .4318 .54411 .44039 .55861 .43569 .55091 .42706 Metetra
.664 .743 .909 r
.55091 .42706 .55861 .43569 .57319 .43097 .56554 .42229 Metetra
.664 .743 .909 r
.56554 .42229 .57319 .43097 .58784 .42622 .58025 .41749 Metetra
.664 .743 .909 r
.58025 .41749 .58784 .42622 .60258 .42144 .59503 .41266 Metetra
.664 .743 .909 r
.59503 .41266 .60258 .42144 .61739 .41664 .6099 .40781 Metetra
.664 .743 .909 r
.6099 .40781 .61739 .41664 .63228 .41181 .62484 .40293 Metetra
.664 .743 .909 r
.62484 .40293 .63228 .41181 .64725 .40695 .63986 .39803 Metetra
.664 .743 .909 r
.63986 .39803 .64725 .40695 .6623 .40207 .65497 .39309 Metetra
.664 .743 .909 r
.65497 .39309 .6623 .40207 .67742 .39715 .67015 .38813 Metetra
.664 .743 .909 r
.67015 .38813 .67742 .39715 .69263 .39222 .68542 .38314 Metetra
.664 .743 .909 r
.68542 .38314 .69263 .39222 .70793 .38725 .70077 .37813 Metetra
.664 .743 .909 r
.70077 .37813 .70793 .38725 .7233 .38226 .7162 .37308 Metetra
.664 .743 .908 r
.7162 .37308 .7233 .38226 .73876 .37724 .73171 .36801 Metetra
.664 .743 .908 r
.73171 .36801 .73876 .37724 .7543 .37219 .74731 .36291 Metetra
.664 .743 .908 r
.74731 .36291 .7543 .37219 .76992 .36712 .763 .35779 Metetra
.664 .743 .908 r
.763 .35779 .76992 .36712 .78563 .36202 .77877 .35263 Metetra
.664 .743 .908 r
.77877 .35263 .78563 .36202 .80143 .35688 .79463 .34745 Metetra
.664 .743 .908 r
.79463 .34745 .80143 .35688 .81731 .35173 .81057 .34223 Metetra
.664 .743 .908 r
.81057 .34223 .81731 .35173 .83328 .34654 .8266 .33699 Metetra
.664 .743 .908 r
.8266 .33699 .83328 .34654 .84934 .34132 .84273 .33172 Metetra
.664 .743 .908 r
.84273 .33172 .84934 .34132 .86548 .33608 .85894 .32642 Metetra
.664 .742 .908 r
.85894 .32642 .86548 .33608 .88172 .33081 .87524 .3211 Metetra
.664 .742 .908 r
.87524 .3211 .88172 .33081 .89805 .3255 .89163 .31574 Metetra
.663 .742 .909 r
.29156 .50054 .30016 .5085 .31349 .50419 .30493 .49619 Metetra
.663 .742 .909 r
.30493 .49619 .31349 .50419 .32689 .49985 .31837 .49182 Metetra
.663 .742 .909 r
.31837 .49182 .32689 .49985 .34036 .49549 .33188 .48742 Metetra
.663 .743 .909 r
.33188 .48742 .34036 .49549 .35389 .49111 .34546 .483 Metetra
.663 .743 .909 r
.34546 .483 .35389 .49111 .3675 .4867 .35911 .47855 Metetra
.663 .743 .909 r
.35911 .47855 .3675 .4867 .38117 .48227 .37283 .47408 Metetra
.663 .743 .909 r
.37283 .47408 .38117 .48227 .39492 .47781 .38662 .46958 Metetra
.664 .743 .909 r
.38662 .46958 .39492 .47781 .40873 .47333 .40048 .46505 Metetra
.664 .743 .909 r
.40048 .46505 .40873 .47333 .42262 .46882 .41441 .4605 Metetra
.664 .743 .909 r
.41441 .4605 .42262 .46882 .43658 .46428 .42842 .45592 Metetra
.664 .743 .909 r
.42842 .45592 .43658 .46428 .45061 .45972 .44249 .45132 Metetra
.664 .743 .909 r
.44249 .45132 .45061 .45972 .46472 .45513 .45665 .44668 Metetra
.664 .743 .909 r
.45665 .44668 .46472 .45513 .4789 .45052 .47087 .44203 Metetra
.664 .743 .909 r
.47087 .44203 .4789 .45052 .49315 .44588 .48517 .43734 Metetra
.664 .743 .909 r
.48517 .43734 .49315 .44588 .50748 .44121 .49955 .43263 Metetra
.664 .743 .909 r
.49955 .43263 .50748 .44121 .52188 .43652 .514 .42789 Metetra
.664 .743 .909 r
.514 .42789 .52188 .43652 .53635 .4318 .52853 .42313 Metetra
.664 .743 .909 r
.52853 .42313 .53635 .4318 .55091 .42706 .54313 .41834 Metetra
.664 .743 .909 r
.54313 .41834 .55091 .42706 .56554 .42229 .55781 .41352 Metetra
.664 .743 .909 r
.55781 .41352 .56554 .42229 .58025 .41749 .57257 .40867 Metetra
.664 .743 .909 r
.57257 .40867 .58025 .41749 .59503 .41266 .58741 .4038 Metetra
.664 .743 .909 r
.58741 .4038 .59503 .41266 .6099 .40781 .60233 .3989 Metetra
.664 .743 .909 r
.60233 .3989 .6099 .40781 .62484 .40293 .61733 .39397 Metetra
.664 .743 .909 r
.61733 .39397 .62484 .40293 .63986 .39803 .6324 .38902 Metetra
.664 .743 .909 r
.6324 .38902 .63986 .39803 .65497 .39309 .64756 .38403 Metetra
.664 .743 .909 r
.64756 .38403 .65497 .39309 .67015 .38813 .6628 .37902 Metetra
.664 .743 .909 r
.6628 .37902 .67015 .38813 .68542 .38314 .67813 .37398 Metetra
.664 .743 .909 r
.67813 .37398 .68542 .38314 .70077 .37813 .69353 .36891 Metetra
.664 .743 .909 r
.69353 .36891 .70077 .37813 .7162 .37308 .70902 .36382 Metetra
.664 .743 .909 r
.70902 .36382 .7162 .37308 .73171 .36801 .7246 .35869 Metetra
.664 .743 .908 r
.7246 .35869 .73171 .36801 .74731 .36291 .74026 .35354 Metetra
.665 .743 .908 r
.74026 .35354 .74731 .36291 .763 .35779 .756 .34836 Metetra
.665 .743 .908 r
.756 .34836 .763 .35779 .77877 .35263 .77184 .34315 Metetra
.665 .743 .908 r
.77184 .34315 .77877 .35263 .79463 .34745 .78775 .33791 Metetra
.665 .743 .908 r
.78775 .33791 .79463 .34745 .81057 .34223 .80376 .33265 Metetra
.665 .743 .908 r
.80376 .33265 .81057 .34223 .8266 .33699 .81986 .32735 Metetra
.665 .743 .908 r
.81986 .32735 .8266 .33699 .84273 .33172 .83604 .32202 Metetra
.665 .743 .908 r
.83604 .32202 .84273 .33172 .85894 .32642 .85232 .31667 Metetra
.665 .743 .908 r
.85232 .31667 .85894 .32642 .87524 .3211 .86869 .31128 Metetra
.664 .742 .908 r
.86869 .31128 .87524 .3211 .89163 .31574 .88515 .30587 Metetra
.663 .742 .909 r
.28288 .49251 .29156 .50054 .30493 .49619 .29629 .48813 Metetra
.663 .742 .909 r
.29629 .48813 .30493 .49619 .31837 .49182 .30977 .48371 Metetra
.663 .743 .909 r
.30977 .48371 .31837 .49182 .33188 .48742 .32332 .47928 Metetra
.663 .743 .909 r
.32332 .47928 .33188 .48742 .34546 .483 .33695 .47481 Metetra
.663 .743 .909 r
.33695 .47481 .34546 .483 .35911 .47855 .35064 .47032 Metetra
.663 .743 .909 r
.35064 .47032 .35911 .47855 .37283 .47408 .3644 .46581 Metetra
.663 .743 .909 r
.3644 .46581 .37283 .47408 .38662 .46958 .37823 .46127 Metetra
.664 .743 .909 r
.37823 .46127 .38662 .46958 .40048 .46505 .39214 .4567 Metetra
.664 .743 .909 r
.39214 .4567 .40048 .46505 .41441 .4605 .40612 .4521 Metetra
.664 .743 .909 r
.40612 .4521 .41441 .4605 .42842 .45592 .42017 .44748 Metetra
.664 .743 .909 r
.42017 .44748 .42842 .45592 .44249 .45132 .43429 .44283 Metetra
.664 .743 .909 r
.43429 .44283 .44249 .45132 .45665 .44668 .44849 .43816 Metetra
.664 .743 .909 r
.44849 .43816 .45665 .44668 .47087 .44203 .46277 .43346 Metetra
.664 .743 .909 r
.46277 .43346 .47087 .44203 .48517 .43734 .47712 .42873 Metetra
.664 .743 .909 r
.47712 .42873 .48517 .43734 .49955 .43263 .49154 .42397 Metetra
.664 .743 .909 r
.49154 .42397 .49955 .43263 .514 .42789 .50604 .41919 Metetra
.664 .743 .909 r
.50604 .41919 .514 .42789 .52853 .42313 .52062 .41437 Metetra
.664 .743 .909 r
.52062 .41437 .52853 .42313 .54313 .41834 .53528 .40953 Metetra
.664 .743 .909 r
.53528 .40953 .54313 .41834 .55781 .41352 .55001 .40467 Metetra
.664 .743 .909 r
.55001 .40467 .55781 .41352 .57257 .40867 .56482 .39977 Metetra
.664 .743 .909 r
.56482 .39977 .57257 .40867 .58741 .4038 .57971 .39485 Metetra
.664 .743 .909 r
.57971 .39485 .58741 .4038 .60233 .3989 .59468 .3899 Metetra
.664 .743 .909 r
.59468 .3899 .60233 .3989 .61733 .39397 .60974 .38492 Metetra
.664 .743 .909 r
.60974 .38492 .61733 .39397 .6324 .38902 .62487 .37992 Metetra
.664 .743 .909 r
.62487 .37992 .6324 .38902 .64756 .38403 .64008 .37488 Metetra
.664 .743 .909 r
.64008 .37488 .64756 .38403 .6628 .37902 .65538 .36982 Metetra
.664 .743 .909 r
.65538 .36982 .6628 .37902 .67813 .37398 .67076 .36473 Metetra
.664 .743 .909 r
.67076 .36473 .67813 .37398 .69353 .36891 .68623 .35961 Metetra
.665 .743 .909 r
.68623 .35961 .69353 .36891 .70902 .36382 .70178 .35446 Metetra
.665 .743 .909 r
.70178 .35446 .70902 .36382 .7246 .35869 .71741 .34928 Metetra
.665 .743 .908 r
.71741 .34928 .7246 .35869 .74026 .35354 .73313 .34408 Metetra
.665 .743 .908 r
.73313 .34408 .74026 .35354 .756 .34836 .74894 .33884 Metetra
.665 .743 .908 r
.74894 .33884 .756 .34836 .77184 .34315 .76483 .33358 Metetra
.665 .743 .908 r
.76483 .33358 .77184 .34315 .78775 .33791 .78081 .32828 Metetra
.665 .743 .908 r
.78081 .32828 .78775 .33791 .80376 .33265 .79688 .32296 Metetra
.665 .743 .908 r
.79688 .32296 .80376 .33265 .81986 .32735 .81304 .31761 Metetra
.665 .743 .908 r
.81304 .31761 .81986 .32735 .83604 .32202 .82929 .31223 Metetra
.665 .743 .908 r
.82929 .31223 .83604 .32202 .85232 .31667 .84564 .30681 Metetra
.665 .743 .908 r
.84564 .30681 .85232 .31667 .86869 .31128 .86207 .30137 Metetra
.665 .743 .908 r
.86207 .30137 .86869 .31128 .88515 .30587 .8786 .2959 Metetra
.663 .742 .909 r
.27412 .48441 .28288 .49251 .29629 .48813 .28757 .47998 Metetra
.663 .742 .909 r
.28757 .47998 .29629 .48813 .30977 .48371 .30109 .47553 Metetra
.663 .743 .909 r
.30109 .47553 .30977 .48371 .32332 .47928 .31469 .47105 Metetra
.663 .743 .909 r
.31469 .47105 .32332 .47928 .33695 .47481 .32835 .46655 Metetra
.663 .743 .909 r
.32835 .46655 .33695 .47481 .35064 .47032 .34209 .46202 Metetra
.663 .743 .909 r
.34209 .46202 .35064 .47032 .3644 .46581 .35589 .45746 Metetra
.663 .743 .909 r
.35589 .45746 .3644 .46581 .37823 .46127 .36977 .45288 Metetra
.663 .743 .909 r
.36977 .45288 .37823 .46127 .39214 .4567 .38372 .44827 Metetra
.664 .743 .909 r
.38372 .44827 .39214 .4567 .40612 .4521 .39775 .44363 Metetra
.664 .743 .909 r
.39775 .44363 .40612 .4521 .42017 .44748 .41184 .43896 Metetra
.664 .743 .909 r
.41184 .43896 .42017 .44748 .43429 .44283 .42602 .43427 Metetra
.664 .743 .909 r
.42602 .43427 .43429 .44283 .44849 .43816 .44026 .42955 Metetra
.664 .743 .909 r
.44026 .42955 .44849 .43816 .46277 .43346 .45458 .4248 Metetra
.664 .743 .909 r
.45458 .4248 .46277 .43346 .47712 .42873 .46898 .42003 Metetra
.664 .743 .909 r
.46898 .42003 .47712 .42873 .49154 .42397 .48346 .41522 Metetra
.664 .743 .909 r
.48346 .41522 .49154 .42397 .50604 .41919 .49801 .41039 Metetra
.664 .743 .909 r
.49801 .41039 .50604 .41919 .52062 .41437 .51264 .40553 Metetra
.664 .743 .909 r
.51264 .40553 .52062 .41437 .53528 .40953 .52734 .40065 Metetra
.664 .743 .909 r
.52734 .40065 .53528 .40953 .55001 .40467 .54213 .39573 Metetra
.664 .743 .909 r
.54213 .39573 .55001 .40467 .56482 .39977 .55699 .39079 Metetra
.664 .743 .909 r
.55699 .39079 .56482 .39977 .57971 .39485 .57194 .38582 Metetra
.664 .743 .909 r
.57194 .38582 .57971 .39485 .59468 .3899 .58696 .38082 Metetra
.664 .743 .909 r
.58696 .38082 .59468 .3899 .60974 .38492 .60207 .37579 Metetra
.664 .743 .909 r
.60207 .37579 .60974 .38492 .62487 .37992 .61726 .37073 Metetra
.664 .743 .909 r
.61726 .37073 .62487 .37992 .64008 .37488 .63253 .36564 Metetra
.664 .743 .909 r
.63253 .36564 .64008 .37488 .65538 .36982 .64789 .36053 Metetra
.665 .743 .909 r
.64789 .36053 .65538 .36982 .67076 .36473 .66332 .35538 Metetra
.665 .743 .909 r
.66332 .35538 .67076 .36473 .68623 .35961 .67885 .35021 Metetra
.665 .743 .909 r
.67885 .35021 .68623 .35961 .70178 .35446 .69445 .34501 Metetra
.665 .743 .909 r
.69445 .34501 .70178 .35446 .71741 .34928 .71015 .33978 Metetra
.665 .743 .909 r
.71015 .33978 .71741 .34928 .73313 .34408 .72593 .33452 Metetra
.665 .743 .908 r
.72593 .33452 .73313 .34408 .74894 .33884 .7418 .32923 Metetra
.665 .743 .908 r
.7418 .32923 .74894 .33884 .76483 .33358 .75775 .32391 Metetra
.665 .743 .908 r
.75775 .32391 .76483 .33358 .78081 .32828 .7738 .31856 Metetra
.665 .743 .908 r
.7738 .31856 .78081 .32828 .79688 .32296 .78993 .31318 Metetra
.665 .743 .908 r
.78993 .31318 .79688 .32296 .81304 .31761 .80616 .30777 Metetra
.665 .743 .908 r
.80616 .30777 .81304 .31761 .82929 .31223 .82247 .30233 Metetra
.665 .743 .908 r
.82247 .30233 .82929 .31223 .84564 .30681 .83888 .29686 Metetra
.665 .743 .908 r
.83888 .29686 .84564 .30681 .86207 .30137 .85538 .29135 Metetra
.665 .743 .908 r
.85538 .29135 .86207 .30137 .8786 .2959 .87198 .28582 Metetra
.663 .742 .909 r
.26527 .47623 .27412 .48441 .28757 .47998 .27877 .47176 Metetra
.663 .742 .909 r
.27877 .47176 .28757 .47998 .30109 .47553 .29233 .46727 Metetra
.663 .743 .909 r
.29233 .46727 .30109 .47553 .31469 .47105 .30597 .46275 Metetra
.663 .743 .909 r
.30597 .46275 .31469 .47105 .32835 .46655 .31967 .45821 Metetra
.663 .743 .909 r
.31967 .45821 .32835 .46655 .34209 .46202 .33345 .45364 Metetra
.663 .743 .909 r
.33345 .45364 .34209 .46202 .35589 .45746 .3473 .44904 Metetra
.663 .743 .909 r
.3473 .44904 .35589 .45746 .36977 .45288 .36123 .44441 Metetra
.663 .743 .909 r
.36123 .44441 .36977 .45288 .38372 .44827 .37522 .43976 Metetra
.664 .743 .909 r
.37522 .43976 .38372 .44827 .39775 .44363 .38929 .43508 Metetra
.664 .743 .909 r
.38929 .43508 .39775 .44363 .41184 .43896 .40344 .43037 Metetra
.664 .743 .909 r
.40344 .43037 .41184 .43896 .42602 .43427 .41766 .42563 Metetra
.664 .743 .909 r
.41766 .42563 .42602 .43427 .44026 .42955 .43195 .42086 Metetra
.664 .743 .909 r
.43195 .42086 .44026 .42955 .45458 .4248 .44632 .41607 Metetra
.664 .743 .909 r
.44632 .41607 .45458 .4248 .46898 .42003 .46077 .41125 Metetra
.664 .743 .909 r
.46077 .41125 .46898 .42003 .48346 .41522 .47529 .4064 Metetra
.664 .743 .909 r
.47529 .4064 .48346 .41522 .49801 .41039 .48989 .40152 Metetra
.664 .743 .909 r
.48989 .40152 .49801 .41039 .51264 .40553 .50457 .39661 Metetra
.664 .743 .909 r
.50457 .39661 .51264 .40553 .52734 .40065 .51933 .39168 Metetra
.664 .743 .909 r
.51933 .39168 .52734 .40065 .54213 .39573 .53417 .38671 Metetra
.664 .743 .909 r
.53417 .38671 .54213 .39573 .55699 .39079 .54909 .38172 Metetra
.664 .743 .909 r
.54909 .38172 .55699 .39079 .57194 .38582 .56409 .37669 Metetra
.664 .743 .909 r
.56409 .37669 .57194 .38582 .58696 .38082 .57917 .37164 Metetra
.664 .743 .909 r
.57917 .37164 .58696 .38082 .60207 .37579 .59433 .36656 Metetra
.664 .743 .909 r
.59433 .36656 .60207 .37579 .61726 .37073 .60957 .36145 Metetra
.665 .743 .909 r
.60957 .36145 .61726 .37073 .63253 .36564 .6249 .35631 Metetra
.665 .743 .909 r
.6249 .35631 .63253 .36564 .64789 .36053 .64031 .35115 Metetra
.665 .743 .909 r
.64031 .35115 .64789 .36053 .66332 .35538 .65581 .34595 Metetra
.665 .743 .909 r
.65581 .34595 .66332 .35538 .67885 .35021 .67139 .34072 Metetra
.665 .743 .909 r
.67139 .34072 .67885 .35021 .69445 .34501 .68706 .33546 Metetra
.665 .743 .909 r
.68706 .33546 .69445 .34501 .71015 .33978 .70281 .33018 Metetra
.665 .743 .909 r
.70281 .33018 .71015 .33978 .72593 .33452 .71865 .32486 Metetra
.665 .743 .909 r
.71865 .32486 .72593 .33452 .7418 .32923 .73458 .31951 Metetra
.665 .743 .908 r
.73458 .31951 .7418 .32923 .75775 .32391 .7506 .31414 Metetra
.665 .743 .908 r
.7506 .31414 .75775 .32391 .7738 .31856 .76671 .30873 Metetra
.665 .743 .908 r
.76671 .30873 .7738 .31856 .78993 .31318 .78291 .30329 Metetra
.665 .743 .908 r
.78291 .30329 .78993 .31318 .80616 .30777 .7992 .29782 Metetra
.665 .743 .908 r
.7992 .29782 .80616 .30777 .82247 .30233 .81558 .29232 Metetra
.665 .743 .908 r
.81558 .29232 .82247 .30233 .83888 .29686 .83206 .28679 Metetra
.665 .743 .908 r
.83206 .28679 .83888 .29686 .85538 .29135 .84862 .28123 Metetra
.665 .743 .908 r
.84862 .28123 .85538 .29135 .87198 .28582 .86529 .27564 Metetra
.663 .742 .909 r
.25634 .46797 .26527 .47623 .27877 .47176 .26988 .46346 Metetra
.663 .742 .909 r
.26988 .46346 .27877 .47176 .29233 .46727 .28348 .45893 Metetra
.663 .743 .909 r
.28348 .45893 .29233 .46727 .30597 .46275 .29716 .45438 Metetra
.663 .743 .909 r
.29716 .45438 .30597 .46275 .31967 .45821 .31091 .44979 Metetra
.663 .743 .909 r
.31091 .44979 .31967 .45821 .33345 .45364 .32473 .44518 Metetra
.663 .743 .909 r
.32473 .44518 .33345 .45364 .3473 .44904 .33863 .44053 Metetra
.663 .743 .909 r
.33863 .44053 .3473 .44904 .36123 .44441 .3526 .43587 Metetra
.663 .743 .909 r
.3526 .43587 .36123 .44441 .37522 .43976 .36664 .43117 Metetra
.663 .743 .909 r
.36664 .43117 .37522 .43976 .38929 .43508 .38076 .42644 Metetra
.664 .743 .909 r
.38076 .42644 .38929 .43508 .40344 .43037 .39495 .42169 Metetra
.664 .743 .909 r
.39495 .42169 .40344 .43037 .41766 .42563 .40921 .41691 Metetra
.664 .743 .909 r
.40921 .41691 .41766 .42563 .43195 .42086 .42356 .41209 Metetra
.664 .743 .909 r
.42356 .41209 .43195 .42086 .44632 .41607 .43798 .40725 Metetra
.664 .743 .909 r
.43798 .40725 .44632 .41607 .46077 .41125 .45247 .40238 Metetra
.664 .743 .909 r
.45247 .40238 .46077 .41125 .47529 .4064 .46705 .39749 Metetra
.664 .743 .909 r
.46705 .39749 .47529 .4064 .48989 .40152 .4817 .39256 Metetra
.664 .743 .909 r
.4817 .39256 .48989 .40152 .50457 .39661 .49643 .3876 Metetra
.664 .743 .909 r
.49643 .3876 .50457 .39661 .51933 .39168 .51124 .38262 Metetra
.664 .743 .909 r
.51124 .38262 .51933 .39168 .53417 .38671 .52613 .3776 Metetra
.664 .743 .909 r
.52613 .3776 .53417 .38671 .54909 .38172 .5411 .37256 Metetra
.664 .743 .909 r
.5411 .37256 .54909 .38172 .56409 .37669 .55616 .36748 Metetra
.664 .743 .909 r
.55616 .36748 .56409 .37669 .57917 .37164 .57129 .36238 Metetra
.664 .743 .909 r
.57129 .36238 .57917 .37164 .59433 .36656 .58651 .35725 Metetra
.665 .743 .909 r
.58651 .35725 .59433 .36656 .60957 .36145 .60181 .35208 Metetra
.665 .743 .909 r
.60181 .35208 .60957 .36145 .6249 .35631 .61719 .34689 Metetra
.665 .743 .909 r
.61719 .34689 .6249 .35631 .64031 .35115 .63266 .34167 Metetra
.665 .743 .909 r
.63266 .34167 .64031 .35115 .65581 .34595 .64822 .33642 Metetra
.665 .743 .909 r
.64822 .33642 .65581 .34595 .67139 .34072 .66386 .33113 Metetra
.665 .743 .909 r
.66386 .33113 .67139 .34072 .68706 .33546 .67959 .32582 Metetra
.665 .743 .909 r
.67959 .32582 .68706 .33546 .70281 .33018 .6954 .32048 Metetra
.665 .743 .909 r
.6954 .32048 .70281 .33018 .71865 .32486 .7113 .3151 Metetra
.665 .743 .909 r
.7113 .3151 .71865 .32486 .73458 .31951 .7273 .3097 Metetra
.665 .743 .908 r
.7273 .3097 .73458 .31951 .7506 .31414 .74338 .30427 Metetra
.665 .743 .908 r
.74338 .30427 .7506 .31414 .76671 .30873 .75955 .2988 Metetra
.665 .743 .908 r
.75955 .2988 .76671 .30873 .78291 .30329 .77581 .2933 Metetra
.665 .743 .908 r
.77581 .2933 .78291 .30329 .7992 .29782 .79217 .28778 Metetra
.665 .743 .908 r
.79217 .28778 .7992 .29782 .81558 .29232 .80862 .28222 Metetra
.665 .743 .908 r
.80862 .28222 .81558 .29232 .83206 .28679 .82516 .27663 Metetra
.665 .743 .908 r
.82516 .27663 .83206 .28679 .84862 .28123 .8418 .27101 Metetra
.665 .743 .908 r
.8418 .27101 .84862 .28123 .86529 .27564 .85853 .26535 Metetra
.663 .742 .909 r
.24733 .45963 .25634 .46797 .26988 .46346 .2609 .45509 Metetra
.663 .742 .909 r
.2609 .45509 .26988 .46346 .28348 .45893 .27455 .45052 Metetra
.663 .743 .909 r
.27455 .45052 .28348 .45893 .29716 .45438 .28827 .44592 Metetra
.663 .743 .909 r
.28827 .44592 .29716 .45438 .31091 .44979 .30207 .44129 Metetra
.663 .743 .909 r
.30207 .44129 .31091 .44979 .32473 .44518 .31593 .43663 Metetra
.663 .743 .909 r
.31593 .43663 .32473 .44518 .33863 .44053 .32987 .43195 Metetra
.663 .743 .909 r
.32987 .43195 .33863 .44053 .3526 .43587 .34388 .42724 Metetra
.663 .743 .909 r
.34388 .42724 .3526 .43587 .36664 .43117 .35797 .4225 Metetra
.663 .743 .909 r
.35797 .4225 .36664 .43117 .38076 .42644 .37213 .41773 Metetra
.664 .743 .909 r
.37213 .41773 .38076 .42644 .39495 .42169 .38637 .41293 Metetra
.664 .743 .909 r
.38637 .41293 .39495 .42169 .40921 .41691 .40069 .4081 Metetra
.664 .743 .909 r
.40069 .4081 .40921 .41691 .42356 .41209 .41508 .40324 Metetra
.664 .743 .909 r
.41508 .40324 .42356 .41209 .43798 .40725 .42955 .39835 Metetra
.664 .743 .909 r
.42955 .39835 .43798 .40725 .45247 .40238 .44409 .39344 Metetra
.664 .743 .909 r
.44409 .39344 .45247 .40238 .46705 .39749 .45872 .38849 Metetra
.664 .743 .909 r
.45872 .38849 .46705 .39749 .4817 .39256 .47342 .38351 Metetra
.664 .743 .909 r
.47342 .38351 .4817 .39256 .49643 .3876 .4882 .37851 Metetra
.664 .744 .909 r
.4882 .37851 .49643 .3876 .51124 .38262 .50307 .37347 Metetra
.664 .744 .909 r
.50307 .37347 .51124 .38262 .52613 .3776 .51801 .36841 Metetra
.664 .744 .909 r
.51801 .36841 .52613 .3776 .5411 .37256 .53304 .36331 Metetra
.664 .744 .909 r
.53304 .36331 .5411 .37256 .55616 .36748 .54814 .35818 Metetra
.664 .744 .909 r
.54814 .35818 .55616 .36748 .57129 .36238 .56333 .35303 Metetra
.665 .743 .909 r
.56333 .35303 .57129 .36238 .58651 .35725 .57861 .34784 Metetra
.665 .743 .909 r
.57861 .34784 .58651 .35725 .60181 .35208 .59396 .34262 Metetra
.665 .743 .909 r
.59396 .34262 .60181 .35208 .61719 .34689 .60941 .33738 Metetra
.665 .743 .909 r
.60941 .33738 .61719 .34689 .63266 .34167 .62493 .3321 Metetra
.665 .743 .909 r
.62493 .3321 .63266 .34167 .64822 .33642 .64055 .32679 Metetra
.665 .743 .909 r
.64055 .32679 .64822 .33642 .66386 .33113 .65625 .32145 Metetra
.665 .743 .909 r
.65625 .32145 .66386 .33113 .67959 .32582 .67204 .31608 Metetra
.665 .743 .909 r
.67204 .31608 .67959 .32582 .6954 .32048 .68791 .31068 Metetra
.665 .743 .909 r
.68791 .31068 .6954 .32048 .7113 .3151 .70388 .30525 Metetra
.665 .743 .909 r
.70388 .30525 .7113 .3151 .7273 .3097 .71993 .29979 Metetra
.665 .743 .909 r
.71993 .29979 .7273 .3097 .74338 .30427 .73608 .29429 Metetra
.665 .743 .908 r
.73608 .29429 .74338 .30427 .75955 .2988 .75231 .28877 Metetra
.665 .743 .908 r
.75231 .28877 .75955 .2988 .77581 .2933 .76864 .28321 Metetra
.665 .743 .908 r
.76864 .28321 .77581 .2933 .79217 .28778 .78506 .27762 Metetra
.665 .743 .908 r
.78506 .27762 .79217 .28778 .80862 .28222 .80158 .272 Metetra
.665 .743 .908 r
.80158 .272 .80862 .28222 .82516 .27663 .81819 .26635 Metetra
.665 .743 .908 r
.81819 .26635 .82516 .27663 .8418 .27101 .83489 .26067 Metetra
.665 .743 .908 r
.83489 .26067 .8418 .27101 .85853 .26535 .85169 .25495 Metetra
.662 .742 .909 r
.23822 .45122 .24733 .45963 .2609 .45509 .25184 .44663 Metetra
.663 .742 .909 r
.25184 .44663 .2609 .45509 .27455 .45052 .26553 .44202 Metetra
.663 .743 .909 r
.26553 .44202 .27455 .45052 .28827 .44592 .2793 .43738 Metetra
.663 .743 .909 r
.2793 .43738 .28827 .44592 .30207 .44129 .29313 .43271 Metetra
.663 .743 .909 r
.29313 .43271 .30207 .44129 .31593 .43663 .30704 .42801 Metetra
.663 .743 .909 r
.30704 .42801 .31593 .43663 .32987 .43195 .32103 .42329 Metetra
.663 .743 .909 r
.32103 .42329 .32987 .43195 .34388 .42724 .33508 .41853 Metetra
.663 .743 .909 r
.33508 .41853 .34388 .42724 .35797 .4225 .34922 .41374 Metetra
.663 .743 .909 r
.34922 .41374 .35797 .4225 .37213 .41773 .36343 .40893 Metetra
.664 .743 .909 r
.36343 .40893 .37213 .41773 .38637 .41293 .37771 .40408 Metetra
.664 .743 .909 r
.37771 .40408 .38637 .41293 .40069 .4081 .39208 .39921 Metetra
.664 .743 .909 r
.39208 .39921 .40069 .4081 .41508 .40324 .40652 .3943 Metetra
.664 .743 .909 r
.40652 .3943 .41508 .40324 .42955 .39835 .42103 .38937 Metetra
.664 .744 .909 r
.42103 .38937 .42955 .39835 .44409 .39344 .43563 .3844 Metetra
.664 .744 .909 r
.43563 .3844 .44409 .39344 .45872 .38849 .4503 .37941 Metetra
.664 .744 .909 r
.4503 .37941 .45872 .38849 .47342 .38351 .46506 .37438 Metetra
.664 .744 .909 r
.46506 .37438 .47342 .38351 .4882 .37851 .47989 .36933 Metetra
.664 .744 .909 r
.47989 .36933 .4882 .37851 .50307 .37347 .49481 .36424 Metetra
.664 .744 .909 r
.49481 .36424 .50307 .37347 .51801 .36841 .50981 .35912 Metetra
.664 .744 .909 r
.50981 .35912 .51801 .36841 .53304 .36331 .52489 .35397 Metetra
.664 .744 .909 r
.52489 .35397 .53304 .36331 .54814 .35818 .54005 .34879 Metetra
.665 .744 .909 r
.54005 .34879 .54814 .35818 .56333 .35303 .55529 .34358 Metetra
.665 .744 .909 r
.55529 .34358 .56333 .35303 .57861 .34784 .57062 .33834 Metetra
.665 .744 .909 r
.57062 .33834 .57861 .34784 .59396 .34262 .58604 .33307 Metetra
.665 .744 .909 r
.58604 .33307 .59396 .34262 .60941 .33738 .60154 .32777 Metetra
.665 .744 .909 r
.60154 .32777 .60941 .33738 .62493 .3321 .61713 .32243 Metetra
.665 .743 .909 r
.61713 .32243 .62493 .3321 .64055 .32679 .6328 .31707 Metetra
.665 .743 .909 r
.6328 .31707 .64055 .32679 .65625 .32145 .64856 .31167 Metetra
.665 .743 .909 r
.64856 .31167 .65625 .32145 .67204 .31608 .66441 .30625 Metetra
.665 .743 .909 r
.66441 .30625 .67204 .31608 .68791 .31068 .68035 .30079 Metetra
.665 .743 .909 r
.68035 .30079 .68791 .31068 .70388 .30525 .69637 .2953 Metetra
.665 .743 .909 r
.69637 .2953 .70388 .30525 .71993 .29979 .71249 .28977 Metetra
.665 .743 .909 r
.71249 .28977 .71993 .29979 .73608 .29429 .7287 .28422 Metetra
.665 .743 .908 r
.7287 .28422 .73608 .29429 .75231 .28877 .745 .27863 Metetra
.665 .743 .908 r
.745 .27863 .75231 .28877 .76864 .28321 .7614 .27302 Metetra
.665 .743 .908 r
.7614 .27302 .76864 .28321 .78506 .27762 .77788 .26737 Metetra
.665 .743 .908 r
.77788 .26737 .78506 .27762 .80158 .272 .79447 .26168 Metetra
.665 .743 .908 r
.79447 .26168 .80158 .272 .81819 .26635 .81114 .25597 Metetra
.665 .743 .908 r
.81114 .25597 .81819 .26635 .83489 .26067 .82792 .25022 Metetra
.665 .743 .908 r
.82792 .25022 .83489 .26067 .85169 .25495 .84479 .24444 Metetra
.662 .742 .909 r
.22903 .44272 .23822 .45122 .25184 .44663 .24269 .43809 Metetra
.662 .742 .909 r
.24269 .43809 .25184 .44663 .26553 .44202 .25643 .43344 Metetra
.663 .743 .909 r
.25643 .43344 .26553 .44202 .2793 .43738 .27023 .42876 Metetra
.663 .743 .91 r
.27023 .42876 .2793 .43738 .29313 .43271 .28411 .42405 Metetra
.663 .743 .91 r
.28411 .42405 .29313 .43271 .30704 .42801 .29807 .41931 Metetra
.663 .743 .91 r
.29807 .41931 .30704 .42801 .32103 .42329 .31209 .41454 Metetra
.663 .743 .91 r
.31209 .41454 .32103 .42329 .33508 .41853 .3262 .40974 Metetra
.663 .743 .91 r
.3262 .40974 .33508 .41853 .34922 .41374 .34038 .40491 Metetra
.663 .743 .91 r
.34038 .40491 .34922 .41374 .36343 .40893 .35463 .40005 Metetra
.663 .743 .91 r
.35463 .40005 .36343 .40893 .37771 .40408 .36897 .39516 Metetra
.664 .743 .91 r
.36897 .39516 .37771 .40408 .39208 .39921 .38338 .39024 Metetra
.664 .744 .91 r
.38338 .39024 .39208 .39921 .40652 .3943 .39787 .38528 Metetra
.664 .744 .91 r
.39787 .38528 .40652 .3943 .42103 .38937 .41243 .3803 Metetra
.664 .744 .909 r
.41243 .3803 .42103 .38937 .43563 .3844 .42708 .37529 Metetra
.664 .744 .909 r
.42708 .37529 .43563 .3844 .4503 .37941 .44181 .37024 Metetra
.664 .744 .909 r
.44181 .37024 .4503 .37941 .46506 .37438 .45661 .36516 Metetra
.664 .744 .909 r
.45661 .36516 .46506 .37438 .47989 .36933 .4715 .36006 Metetra
.664 .744 .909 r
.4715 .36006 .47989 .36933 .49481 .36424 .48647 .35492 Metetra
.664 .744 .909 r
.48647 .35492 .49481 .36424 .50981 .35912 .50152 .34975 Metetra
.664 .744 .909 r
.50152 .34975 .50981 .35912 .52489 .35397 .51665 .34455 Metetra
.664 .744 .909 r
.51665 .34455 .52489 .35397 .54005 .34879 .53187 .33931 Metetra
.665 .744 .909 r
.53187 .33931 .54005 .34879 .55529 .34358 .54717 .33405 Metetra
.665 .744 .909 r
.54717 .33405 .55529 .34358 .57062 .33834 .56256 .32875 Metetra
.665 .744 .909 r
.56256 .32875 .57062 .33834 .58604 .33307 .57803 .32342 Metetra
.665 .744 .909 r
.57803 .32342 .58604 .33307 .60154 .32777 .59359 .31807 Metetra
.665 .744 .909 r
.59359 .31807 .60154 .32777 .61713 .32243 .60924 .31267 Metetra
.665 .744 .909 r
.60924 .31267 .61713 .32243 .6328 .31707 .62497 .30725 Metetra
.665 .744 .909 r
.62497 .30725 .6328 .31707 .64856 .31167 .64079 .3018 Metetra
.665 .744 .909 r
.64079 .3018 .64856 .31167 .66441 .30625 .6567 .29631 Metetra
.665 .743 .909 r
.6567 .29631 .66441 .30625 .68035 .30079 .6727 .29079 Metetra
.665 .743 .909 r
.6727 .29079 .68035 .30079 .69637 .2953 .68879 .28524 Metetra
.665 .743 .909 r
.68879 .28524 .69637 .2953 .71249 .28977 .70497 .27966 Metetra
.665 .743 .909 r
.70497 .27966 .71249 .28977 .7287 .28422 .72124 .27404 Metetra
.665 .743 .909 r
.72124 .27404 .7287 .28422 .745 .27863 .73761 .26839 Metetra
.665 .743 .908 r
.73761 .26839 .745 .27863 .7614 .27302 .75407 .26271 Metetra
.665 .743 .908 r
.75407 .26271 .7614 .27302 .77788 .26737 .77063 .257 Metetra
.665 .743 .908 r
.77063 .257 .77788 .26737 .79447 .26168 .78728 .25126 Metetra
.665 .743 .908 r
.78728 .25126 .79447 .26168 .81114 .25597 .80402 .24548 Metetra
.665 .743 .908 r
.80402 .24548 .81114 .25597 .82792 .25022 .82087 .23967 Metetra
.665 .743 .908 r
.82087 .23967 .82792 .25022 .84479 .24444 .83781 .23382 Metetra
.662 .742 .909 r
.21975 .43414 .22903 .44272 .24269 .43809 .23345 .42947 Metetra
.662 .742 .91 r
.23345 .42947 .24269 .43809 .25643 .43344 .24723 .42478 Metetra
.662 .743 .91 r
.24723 .42478 .25643 .43344 .27023 .42876 .26108 .42006 Metetra
.663 .743 .91 r
.26108 .42006 .27023 .42876 .28411 .42405 .275 .4153 Metetra
.663 .743 .91 r
.275 .4153 .28411 .42405 .29807 .41931 .289 .41052 Metetra
.663 .743 .91 r
.289 .41052 .29807 .41931 .31209 .41454 .30307 .40571 Metetra
.663 .743 .91 r
.30307 .40571 .31209 .41454 .3262 .40974 .31722 .40086 Metetra
.663 .743 .91 r
.31722 .40086 .3262 .40974 .34038 .40491 .33145 .39599 Metetra
.663 .743 .91 r
.33145 .39599 .34038 .40491 .35463 .40005 .34575 .39108 Metetra
.663 .743 .91 r
.34575 .39108 .35463 .40005 .36897 .39516 .36013 .38615 Metetra
.664 .744 .91 r
.36013 .38615 .36897 .39516 .38338 .39024 .37459 .38118 Metetra
.664 .744 .91 r
.37459 .38118 .38338 .39024 .39787 .38528 .38913 .37618 Metetra
.664 .744 .91 r
.38913 .37618 .39787 .38528 .41243 .3803 .40375 .37115 Metetra
.664 .744 .91 r
.40375 .37115 .41243 .3803 .42708 .37529 .41844 .36608 Metetra
.664 .744 .91 r
.41844 .36608 .42708 .37529 .44181 .37024 .43322 .36099 Metetra
.664 .744 .91 r
.43322 .36099 .44181 .37024 .45661 .36516 .44808 .35586 Metetra
.664 .744 .91 r
.44808 .35586 .45661 .36516 .4715 .36006 .46302 .3507 Metetra
.664 .744 .909 r
.46302 .3507 .4715 .36006 .48647 .35492 .47804 .34551 Metetra
.664 .744 .909 r
.47804 .34551 .48647 .35492 .50152 .34975 .49315 .34028 Metetra
.664 .744 .909 r
.49315 .34028 .50152 .34975 .51665 .34455 .50834 .33503 Metetra
.665 .744 .909 r
.50834 .33503 .51665 .34455 .53187 .33931 .52361 .32974 Metetra
.665 .744 .909 r
.52361 .32974 .53187 .33931 .54717 .33405 .53897 .32442 Metetra
.665 .744 .909 r
.53897 .32442 .54717 .33405 .56256 .32875 .55441 .31907 Metetra
.665 .744 .909 r
.55441 .31907 .56256 .32875 .57803 .32342 .56994 .31368 Metetra
.665 .744 .909 r
.56994 .31368 .57803 .32342 .59359 .31807 .58556 .30827 Metetra
.665 .744 .909 r
.58556 .30827 .59359 .31807 .60924 .31267 .60126 .30282 Metetra
.665 .744 .909 r
.60126 .30282 .60924 .31267 .62497 .30725 .61706 .29733 Metetra
.665 .744 .909 r
.61706 .29733 .62497 .30725 .64079 .3018 .63294 .29182 Metetra
.665 .744 .909 r
.63294 .29182 .64079 .3018 .6567 .29631 .64891 .28627 Metetra
.665 .744 .909 r
.64891 .28627 .6567 .29631 .6727 .29079 .66497 .28069 Metetra
.665 .744 .909 r
.66497 .28069 .6727 .29079 .68879 .28524 .68113 .27508 Metetra
.665 .743 .909 r
.68113 .27508 .68879 .28524 .70497 .27966 .69737 .26944 Metetra
.665 .743 .909 r
.69737 .26944 .70497 .27966 .72124 .27404 .71371 .26376 Metetra
.665 .743 .909 r
.71371 .26376 .72124 .27404 .73761 .26839 .73014 .25805 Metetra
.665 .743 .908 r
.73014 .25805 .73761 .26839 .75407 .26271 .74667 .2523 Metetra
.665 .743 .908 r
.74667 .2523 .75407 .26271 .77063 .257 .76329 .24653 Metetra
.665 .743 .908 r
.76329 .24653 .77063 .257 .78728 .25126 .78001 .24072 Metetra
.665 .743 .908 r
.78001 .24072 .78728 .25126 .80402 .24548 .79682 .23487 Metetra
.665 .743 .908 r
.79682 .23487 .80402 .24548 .82087 .23967 .81374 .229 Metetra
.665 .743 .908 r
.81374 .229 .82087 .23967 .83781 .23382 .83075 .22309 Metetra
.662 .742 .91 r
.21038 .42547 .21975 .43414 .23345 .42947 .22412 .42077 Metetra
.662 .742 .91 r
.22412 .42077 .23345 .42947 .24723 .42478 .23794 .41603 Metetra
.662 .743 .91 r
.23794 .41603 .24723 .42478 .26108 .42006 .25183 .41127 Metetra
.662 .743 .91 r
.25183 .41127 .26108 .42006 .275 .4153 .2658 .40647 Metetra
.663 .743 .91 r
.2658 .40647 .275 .4153 .289 .41052 .27984 .40165 Metetra
.663 .743 .91 r
.27984 .40165 .289 .41052 .30307 .40571 .29396 .39679 Metetra
.663 .743 .91 r
.29396 .39679 .30307 .40571 .31722 .40086 .30816 .3919 Metetra
.663 .743 .91 r
.30816 .3919 .31722 .40086 .33145 .39599 .32243 .38698 Metetra
.663 .743 .91 r
.32243 .38698 .33145 .39599 .34575 .39108 .33678 .38203 Metetra
.663 .744 .91 r
.33678 .38203 .34575 .39108 .36013 .38615 .35121 .37705 Metetra
.663 .744 .91 r
.35121 .37705 .36013 .38615 .37459 .38118 .36572 .37203 Metetra
.664 .744 .91 r
.36572 .37203 .37459 .38118 .38913 .37618 .38031 .36699 Metetra
.664 .744 .91 r
.38031 .36699 .38913 .37618 .40375 .37115 .39497 .3619 Metetra
.664 .744 .91 r
.39497 .3619 .40375 .37115 .41844 .36608 .40972 .35679 Metetra
.664 .744 .91 r
.40972 .35679 .41844 .36608 .43322 .36099 .42455 .35164 Metetra
.664 .744 .91 r
.42455 .35164 .43322 .36099 .44808 .35586 .43946 .34647 Metetra
.664 .744 .91 r
.43946 .34647 .44808 .35586 .46302 .3507 .45445 .34125 Metetra
.664 .744 .91 r
.45445 .34125 .46302 .3507 .47804 .34551 .46953 .33601 Metetra
.664 .744 .91 r
.46953 .33601 .47804 .34551 .49315 .34028 .48469 .33073 Metetra
.664 .744 .909 r
.48469 .33073 .49315 .34028 .50834 .33503 .49993 .32542 Metetra
.665 .744 .909 r
.49993 .32542 .50834 .33503 .52361 .32974 .51526 .32008 Metetra
.665 .744 .909 r
.51526 .32008 .52361 .32974 .53897 .32442 .53068 .3147 Metetra
.665 .744 .909 r
.53068 .3147 .53897 .32442 .55441 .31907 .54618 .30929 Metetra
.665 .744 .909 r
.54618 .30929 .55441 .31907 .56994 .31368 .56177 .30385 Metetra
.665 .744 .909 r
.56177 .30385 .56994 .31368 .58556 .30827 .57744 .29837 Metetra
.665 .744 .909 r
.57744 .29837 .58556 .30827 .60126 .30282 .59321 .29286 Metetra
.665 .744 .909 r
.59321 .29286 .60126 .30282 .61706 .29733 .60906 .28732 Metetra
.665 .744 .909 r
.60906 .28732 .61706 .29733 .63294 .29182 .625 .28174 Metetra
.665 .744 .909 r
.625 .28174 .63294 .29182 .64891 .28627 .64104 .27613 Metetra
.665 .744 .909 r
.64104 .27613 .64891 .28627 .66497 .28069 .65716 .27049 Metetra
.665 .744 .909 r
.65716 .27049 .66497 .28069 .68113 .27508 .67338 .26482 Metetra
.665 .744 .909 r
.67338 .26482 .68113 .27508 .69737 .26944 .68969 .25911 Metetra
.665 .744 .909 r
.68969 .25911 .69737 .26944 .71371 .26376 .70609 .25337 Metetra
.665 .743 .909 r
.70609 .25337 .71371 .26376 .73014 .25805 .72259 .24759 Metetra
.665 .743 .909 r
.72259 .24759 .73014 .25805 .74667 .2523 .73919 .24178 Metetra
.665 .743 .908 r
.73919 .24178 .74667 .2523 .76329 .24653 .75588 .23594 Metetra
.665 .743 .908 r
.75588 .23594 .76329 .24653 .78001 .24072 .77266 .23006 Metetra
.665 .743 .908 r
.77266 .23006 .78001 .24072 .79682 .23487 .78955 .22415 Metetra
.665 .743 .908 r
.78955 .22415 .79682 .23487 .81374 .229 .80653 .21821 Metetra
.665 .743 .908 r
.80653 .21821 .81374 .229 .83075 .22309 .82362 .21223 Metetra
.662 .742 .91 r
.20092 .41672 .21038 .42547 .22412 .42077 .2147 .41198 Metetra
.662 .742 .91 r
.2147 .41198 .22412 .42077 .23794 .41603 .22856 .4072 Metetra
.662 .743 .91 r
.22856 .4072 .23794 .41603 .25183 .41127 .2425 .4024 Metetra
.662 .743 .91 r
.2425 .4024 .25183 .41127 .2658 .40647 .25651 .39756 Metetra
.662 .743 .91 r
.25651 .39756 .2658 .40647 .27984 .40165 .2706 .39269 Metetra
.663 .743 .91 r
.2706 .39269 .27984 .40165 .29396 .39679 .28476 .38779 Metetra
.663 .743 .91 r
.28476 .38779 .29396 .39679 .30816 .3919 .299 .38286 Metetra
.663 .743 .91 r
.299 .38286 .30816 .3919 .32243 .38698 .31332 .3779 Metetra
.663 .744 .91 r
.31332 .3779 .32243 .38698 .33678 .38203 .32772 .3729 Metetra
.663 .744 .91 r
.32772 .3729 .33678 .38203 .35121 .37705 .3422 .36787 Metetra
.663 .744 .91 r
.3422 .36787 .35121 .37705 .36572 .37203 .35675 .3628 Metetra
.664 .744 .91 r
.35675 .3628 .36572 .37203 .38031 .36699 .37139 .35771 Metetra
.664 .744 .91 r
.37139 .35771 .38031 .36699 .39497 .3619 .38611 .35258 Metetra
.664 .744 .91 r
.38611 .35258 .39497 .3619 .40972 .35679 .40091 .34741 Metetra
.664 .744 .91 r
.40091 .34741 .40972 .35679 .42455 .35164 .41579 .34221 Metetra
.664 .744 .91 r
.41579 .34221 .42455 .35164 .43946 .34647 .43075 .33698 Metetra
.664 .744 .91 r
.43075 .33698 .43946 .34647 .45445 .34125 .4458 .33172 Metetra
.664 .744 .91 r
.4458 .33172 .45445 .34125 .46953 .33601 .46093 .32642 Metetra
.664 .744 .91 r
.46093 .32642 .46953 .33601 .48469 .33073 .47614 .32109 Metetra
.665 .744 .91 r
.47614 .32109 .48469 .33073 .49993 .32542 .49144 .31572 Metetra
.665 .744 .91 r
.49144 .31572 .49993 .32542 .51526 .32008 .50683 .31032 Metetra
.665 .744 .909 r
.50683 .31032 .51526 .32008 .53068 .3147 .5223 .30488 Metetra
.665 .744 .909 r
.5223 .30488 .53068 .3147 .54618 .30929 .53786 .29942 Metetra
.665 .744 .909 r
.53786 .29942 .54618 .30929 .56177 .30385 .55351 .29391 Metetra
.665 .744 .909 r
.55351 .29391 .56177 .30385 .57744 .29837 .56924 .28838 Metetra
.665 .744 .909 r
.56924 .28838 .57744 .29837 .59321 .29286 .58507 .28281 Metetra
.665 .744 .909 r
.58507 .28281 .59321 .29286 .60906 .28732 .60098 .2772 Metetra
.665 .744 .909 r
.60098 .2772 .60906 .28732 .625 .28174 .61698 .27156 Metetra
.665 .744 .909 r
.61698 .27156 .625 .28174 .64104 .27613 .63308 .26589 Metetra
.665 .744 .909 r
.63308 .26589 .64104 .27613 .65716 .27049 .64927 .26019 Metetra
.665 .744 .909 r
.64927 .26019 .65716 .27049 .67338 .26482 .66555 .25445 Metetra
.665 .744 .909 r
.66555 .25445 .67338 .26482 .68969 .25911 .68193 .24867 Metetra
.665 .744 .909 r
.68193 .24867 .68969 .25911 .70609 .25337 .6984 .24287 Metetra
.665 .744 .909 r
.6984 .24287 .70609 .25337 .72259 .24759 .71496 .23703 Metetra
.665 .743 .909 r
.71496 .23703 .72259 .24759 .73919 .24178 .73162 .23115 Metetra
.665 .743 .908 r
.73162 .23115 .73919 .24178 .75588 .23594 .74838 .22524 Metetra
.665 .743 .908 r
.74838 .22524 .75588 .23594 .77266 .23006 .76524 .2193 Metetra
.665 .743 .908 r
.76524 .2193 .77266 .23006 .78955 .22415 .78219 .21332 Metetra
.665 .743 .908 r
.78219 .21332 .78955 .22415 .80653 .21821 .79925 .20731 Metetra
.665 .743 .908 r
.79925 .20731 .80653 .21821 .82362 .21223 .8164 .20126 Metetra
.662 .742 .91 r
.19136 .40788 .20092 .41672 .2147 .41198 .20519 .4031 Metetra
.662 .742 .91 r
.20519 .4031 .2147 .41198 .22856 .4072 .21909 .39828 Metetra
.662 .742 .91 r
.21909 .39828 .22856 .4072 .2425 .4024 .23307 .39344 Metetra
.662 .743 .91 r
.23307 .39344 .2425 .4024 .25651 .39756 .24712 .38856 Metetra
.662 .743 .91 r
.24712 .38856 .25651 .39756 .2706 .39269 .26126 .38365 Metetra
.662 .743 .91 r
.26126 .38365 .2706 .39269 .28476 .38779 .27546 .37871 Metetra
.663 .743 .91 r
.27546 .37871 .28476 .38779 .299 .38286 .28975 .37373 Metetra
.663 .743 .91 r
.28975 .37373 .299 .38286 .31332 .3779 .30412 .36872 Metetra
.663 .744 .91 r
.30412 .36872 .31332 .3779 .32772 .3729 .31856 .36368 Metetra
.663 .744 .91 r
.31856 .36368 .32772 .3729 .3422 .36787 .33309 .3586 Metetra
.663 .744 .91 r
.33309 .3586 .3422 .36787 .35675 .3628 .3477 .35349 Metetra
.663 .744 .91 r
.3477 .35349 .35675 .3628 .37139 .35771 .36238 .34834 Metetra
.664 .744 .91 r
.36238 .34834 .37139 .35771 .38611 .35258 .37715 .34316 Metetra
.664 .744 .91 r
.37715 .34316 .38611 .35258 .40091 .34741 .392 .33795 Metetra
.664 .744 .91 r
.392 .33795 .40091 .34741 .41579 .34221 .40693 .3327 Metetra
.664 .744 .91 r
.40693 .3327 .41579 .34221 .43075 .33698 .42195 .32741 Metetra
.664 .744 .91 r
.42195 .32741 .43075 .33698 .4458 .33172 .43705 .32209 Metetra
.664 .745 .91 r
.43705 .32209 .4458 .33172 .46093 .32642 .45224 .31674 Metetra
.664 .745 .91 r
.45224 .31674 .46093 .32642 .47614 .32109 .46751 .31135 Metetra
.665 .745 .91 r
.46751 .31135 .47614 .32109 .49144 .31572 .48286 .30593 Metetra
.665 .745 .91 r
.48286 .30593 .49144 .31572 .50683 .31032 .4983 .30047 Metetra
.665 .745 .91 r
.4983 .30047 .50683 .31032 .5223 .30488 .51383 .29497 Metetra
.665 .745 .91 r
.51383 .29497 .5223 .30488 .53786 .29942 .52945 .28945 Metetra
.665 .745 .909 r
.52945 .28945 .53786 .29942 .55351 .29391 .54516 .28388 Metetra
.665 .744 .909 r
.54516 .28388 .55351 .29391 .56924 .28838 .56095 .27829 Metetra
.665 .744 .909 r
.56095 .27829 .56924 .28838 .58507 .28281 .57684 .27265 Metetra
.665 .744 .909 r
.57684 .27265 .58507 .28281 .60098 .2772 .59281 .26699 Metetra
.665 .744 .909 r
.59281 .26699 .60098 .2772 .61698 .27156 .60888 .26128 Metetra
.665 .744 .909 r
.60888 .26128 .61698 .27156 .63308 .26589 .62504 .25555 Metetra
.665 .744 .909 r
.62504 .25555 .63308 .26589 .64927 .26019 .64129 .24978 Metetra
.665 .744 .909 r
.64129 .24978 .64927 .26019 .66555 .25445 .65764 .24397 Metetra
.665 .744 .909 r
.65764 .24397 .66555 .25445 .68193 .24867 .67408 .23813 Metetra
.665 .744 .909 r
.67408 .23813 .68193 .24867 .6984 .24287 .69062 .23226 Metetra
.665 .744 .909 r
.69062 .23226 .6984 .24287 .71496 .23703 .70725 .22635 Metetra
.665 .744 .909 r
.70725 .22635 .71496 .23703 .73162 .23115 .72398 .2204 Metetra
.665 .743 .908 r
.72398 .2204 .73162 .23115 .74838 .22524 .7408 .21443 Metetra
.665 .743 .908 r
.7408 .21443 .74838 .22524 .76524 .2193 .75773 .20841 Metetra
.665 .743 .908 r
.75773 .20841 .76524 .2193 .78219 .21332 .77476 .20237 Metetra
.665 .743 .908 r
.77476 .20237 .78219 .21332 .79925 .20731 .79188 .19628 Metetra
.665 .743 .908 r
.79188 .19628 .79925 .20731 .8164 .20126 .80911 .19016 Metetra
.661 .742 .91 r
.18171 .39895 .19136 .40788 .20519 .4031 .19558 .39413 Metetra
.662 .742 .91 r
.19558 .39413 .20519 .4031 .21909 .39828 .20952 .38927 Metetra
.662 .742 .91 r
.20952 .38927 .21909 .39828 .23307 .39344 .22354 .38439 Metetra
.662 .743 .91 r
.22354 .38439 .23307 .39344 .24712 .38856 .23764 .37947 Metetra
.662 .743 .91 r
.23764 .37947 .24712 .38856 .26126 .38365 .25182 .37452 Metetra
.662 .743 .91 r
.25182 .37452 .26126 .38365 .27546 .37871 .26607 .36953 Metetra
.662 .743 .91 r
.26607 .36953 .27546 .37871 .28975 .37373 .28041 .36451 Metetra
.663 .744 .91 r
.28041 .36451 .28975 .37373 .30412 .36872 .29482 .35946 Metetra
.663 .744 .91 r
.29482 .35946 .30412 .36872 .31856 .36368 .30932 .35437 Metetra
.663 .744 .91 r
.30932 .35437 .31856 .36368 .33309 .3586 .32389 .34924 Metetra
.663 .744 .91 r
.32389 .34924 .33309 .3586 .3477 .35349 .33854 .34408 Metetra
.663 .744 .91 r
.33854 .34408 .3477 .35349 .36238 .34834 .35328 .33889 Metetra
.664 .744 .91 r
.35328 .33889 .36238 .34834 .37715 .34316 .3681 .33366 Metetra
.664 .744 .91 r
.3681 .33366 .37715 .34316 .392 .33795 .383 .32839 Metetra
.664 .745 .91 r
.383 .32839 .392 .33795 .40693 .3327 .39799 .32309 Metetra
.664 .745 .91 r
.39799 .32309 .40693 .3327 .42195 .32741 .41306 .31775 Metetra
.664 .745 .91 r
.41306 .31775 .42195 .32741 .43705 .32209 .42821 .31238 Metetra
.664 .745 .91 r
.42821 .31238 .43705 .32209 .45224 .31674 .44345 .30697 Metetra
.664 .745 .91 r
.44345 .30697 .45224 .31674 .46751 .31135 .45878 .30152 Metetra
.665 .745 .91 r
.45878 .30152 .46751 .31135 .48286 .30593 .47419 .29604 Metetra
.665 .745 .91 r
.47419 .29604 .48286 .30593 .4983 .30047 .48969 .29052 Metetra
.665 .745 .91 r
.48969 .29052 .4983 .30047 .51383 .29497 .50528 .28497 Metetra
.665 .745 .91 r
.50528 .28497 .51383 .29497 .52945 .28945 .52095 .27938 Metetra
.665 .745 .91 r
.52095 .27938 .52945 .28945 .54516 .28388 .53672 .27376 Metetra
.665 .745 .91 r
.53672 .27376 .54516 .28388 .56095 .27829 .55257 .2681 Metetra
.665 .745 .909 r
.55257 .2681 .56095 .27829 .57684 .27265 .56852 .2624 Metetra
.665 .745 .909 r
.56852 .2624 .57684 .27265 .59281 .26699 .58456 .25667 Metetra
.665 .745 .909 r
.58456 .25667 .59281 .26699 .60888 .26128 .60069 .2509 Metetra
.665 .744 .909 r
.60069 .2509 .60888 .26128 .62504 .25555 .61691 .2451 Metetra
.665 .744 .909 r
.61691 .2451 .62504 .25555 .64129 .24978 .63323 .23926 Metetra
.665 .744 .909 r
.63323 .23926 .64129 .24978 .65764 .24397 .64964 .23339 Metetra
.665 .744 .909 r
.64964 .23339 .65764 .24397 .67408 .23813 .66615 .22748 Metetra
.665 .744 .909 r
.66615 .22748 .67408 .23813 .69062 .23226 .68275 .22154 Metetra
.665 .744 .909 r
.68275 .22154 .69062 .23226 .70725 .22635 .69945 .21556 Metetra
.665 .744 .909 r
.69945 .21556 .70725 .22635 .72398 .2204 .71625 .20954 Metetra
.665 .744 .908 r
.71625 .20954 .72398 .2204 .7408 .21443 .73314 .2035 Metetra
.665 .743 .908 r
.73314 .2035 .7408 .21443 .75773 .20841 .75014 .19741 Metetra
.665 .743 .908 r
.75014 .19741 .75773 .20841 .77476 .20237 .76724 .19129 Metetra
.665 .743 .908 r
.76724 .19129 .77476 .20237 .79188 .19628 .78444 .18514 Metetra
.665 .743 .908 r
.78444 .18514 .79188 .19628 .80911 .19016 .80174 .17895 Metetra
.661 .742 .91 r
.17196 .38993 .18171 .39895 .19558 .39413 .18587 .38507 Metetra
.661 .742 .91 r
.18587 .38507 .19558 .39413 .20952 .38927 .19986 .38017 Metetra
.661 .742 .91 r
.19986 .38017 .20952 .38927 .22354 .38439 .21392 .37525 Metetra
.662 .743 .91 r
.21392 .37525 .22354 .38439 .23764 .37947 .22807 .37029 Metetra
.662 .743 .91 r
.22807 .37029 .23764 .37947 .25182 .37452 .24229 .3653 Metetra
.662 .743 .91 r
.24229 .3653 .25182 .37452 .26607 .36953 .25659 .36027 Metetra
.662 .743 .91 r
.25659 .36027 .26607 .36953 .28041 .36451 .27097 .3552 Metetra
.662 .744 .91 r
.27097 .3552 .28041 .36451 .29482 .35946 .28543 .35011 Metetra
.663 .744 .91 r
.28543 .35011 .29482 .35946 .30932 .35437 .29997 .34497 Metetra
.663 .744 .91 r
.29997 .34497 .30932 .35437 .32389 .34924 .31459 .3398 Metetra
.663 .744 .91 r
.31459 .3398 .32389 .34924 .33854 .34408 .3293 .33459 Metetra
.663 .744 .91 r
.3293 .33459 .33854 .34408 .35328 .33889 .34409 .32935 Metetra
.663 .745 .91 r
.34409 .32935 .35328 .33889 .3681 .33366 .35896 .32407 Metetra
.664 .745 .91 r
.35896 .32407 .3681 .33366 .383 .32839 .37391 .31875 Metetra
.664 .745 .91 r
.37391 .31875 .383 .32839 .39799 .32309 .38895 .3134 Metetra
.664 .745 .91 r
.38895 .3134 .39799 .32309 .41306 .31775 .40407 .30801 Metetra
.664 .745 .91 r
.40407 .30801 .41306 .31775 .42821 .31238 .41928 .30258 Metetra
.664 .745 .91 r
.41928 .30258 .42821 .31238 .44345 .30697 .43458 .29711 Metetra
.665 .745 .91 r
.43458 .29711 .44345 .30697 .45878 .30152 .44996 .29161 Metetra
.665 .745 .91 r
.44996 .29161 .45878 .30152 .47419 .29604 .46543 .28607 Metetra
.665 .745 .91 r
.46543 .28607 .47419 .29604 .48969 .29052 .48098 .28049 Metetra
.665 .745 .91 r
.48098 .28049 .48969 .29052 .50528 .28497 .49663 .27487 Metetra
.665 .745 .91 r
.49663 .27487 .50528 .28497 .52095 .27938 .51237 .26922 Metetra
.665 .745 .91 r
.51237 .26922 .52095 .27938 .53672 .27376 .52819 .26353 Metetra
.665 .745 .91 r
.52819 .26353 .53672 .27376 .55257 .2681 .54411 .25781 Metetra
.665 .745 .91 r
.54411 .25781 .55257 .2681 .56852 .2624 .56012 .25205 Metetra
.665 .745 .909 r
.56012 .25205 .56852 .2624 .58456 .25667 .57622 .24625 Metetra
.665 .745 .909 r
.57622 .24625 .58456 .25667 .60069 .2509 .59241 .24042 Metetra
.666 .745 .909 r
.59241 .24042 .60069 .2509 .61691 .2451 .6087 .23454 Metetra
.666 .745 .909 r
.6087 .23454 .61691 .2451 .63323 .23926 .62508 .22864 Metetra
.666 .744 .909 r
.62508 .22864 .63323 .23926 .64964 .23339 .64155 .2227 Metetra
.666 .744 .909 r
.64155 .2227 .64964 .23339 .66615 .22748 .65813 .21672 Metetra
.666 .744 .909 r
.65813 .21672 .66615 .22748 .68275 .22154 .6748 .2107 Metetra
.666 .744 .909 r
.6748 .2107 .68275 .22154 .69945 .21556 .69157 .20465 Metetra
.666 .744 .909 r
.69157 .20465 .69945 .21556 .71625 .20954 .70843 .19857 Metetra
.666 .744 .908 r
.70843 .19857 .71625 .20954 .73314 .2035 .7254 .19245 Metetra
.666 .744 .908 r
.7254 .19245 .73314 .2035 .75014 .19741 .74247 .18629 Metetra
.666 .743 .908 r
.74247 .18629 .75014 .19741 .76724 .19129 .75964 .1801 Metetra
.666 .743 .908 r
.75964 .1801 .76724 .19129 .78444 .18514 .77691 .17387 Metetra
.666 .743 .908 r
.77691 .17387 .78444 .18514 .80174 .17895 .79428 .1676 Metetra
.661 .742 .91 r
.16212 .38081 .17196 .38993 .18587 .38507 .17607 .37591 Metetra
.661 .742 .91 r
.17607 .37591 .18587 .38507 .19986 .38017 .1901 .37098 Metetra
.661 .742 .91 r
.1901 .37098 .19986 .38017 .21392 .37525 .20421 .36601 Metetra
.661 .743 .91 r
.20421 .36601 .21392 .37525 .22807 .37029 .21839 .36102 Metetra
.662 .743 .91 r
.21839 .36102 .22807 .37029 .24229 .3653 .23266 .35598 Metetra
.662 .743 .91 r
.23266 .35598 .24229 .3653 .25659 .36027 .24701 .35091 Metetra
.662 .743 .911 r
.24701 .35091 .25659 .36027 .27097 .3552 .26143 .34581 Metetra
.662 .744 .911 r
.26143 .34581 .27097 .3552 .28543 .35011 .27594 .34067 Metetra
.662 .744 .911 r
.27594 .34067 .28543 .35011 .29997 .34497 .29053 .33549 Metetra
.663 .744 .911 r
.29053 .33549 .29997 .34497 .31459 .3398 .3052 .33027 Metetra
.663 .744 .911 r
.3052 .33027 .31459 .3398 .3293 .33459 .31996 .32502 Metetra
.663 .745 .911 r
.31996 .32502 .3293 .33459 .34409 .32935 .33479 .31973 Metetra
.663 .745 .911 r
.33479 .31973 .34409 .32935 .35896 .32407 .34972 .3144 Metetra
.664 .745 .911 r
.34972 .3144 .35896 .32407 .37391 .31875 .36472 .30903 Metetra
.664 .745 .911 r
.36472 .30903 .37391 .31875 .38895 .3134 .37981 .30362 Metetra
.664 .745 .911 r
.37981 .30362 .38895 .3134 .40407 .30801 .39499 .29817 Metetra
.664 .745 .911 r
.39499 .29817 .40407 .30801 .41928 .30258 .41025 .29269 Metetra
.664 .745 .911 r
.41025 .29269 .41928 .30258 .43458 .29711 .42561 .28717 Metetra
.665 .746 .911 r
.42561 .28717 .43458 .29711 .44996 .29161 .44104 .2816 Metetra
.665 .746 .91 r
.44104 .2816 .44996 .29161 .46543 .28607 .45657 .276 Metetra
.665 .746 .91 r
.45657 .276 .46543 .28607 .48098 .28049 .47219 .27036 Metetra
.665 .746 .91 r
.47219 .27036 .48098 .28049 .49663 .27487 .48789 .26469 Metetra
.665 .746 .91 r
.48789 .26469 .49663 .27487 .51237 .26922 .50369 .25897 Metetra
.665 .745 .91 r
.50369 .25897 .51237 .26922 .52819 .26353 .51957 .25322 Metetra
.665 .745 .91 r
.51957 .25322 .52819 .26353 .54411 .25781 .53555 .24742 Metetra
.665 .745 .91 r
.53555 .24742 .54411 .25781 .56012 .25205 .55162 .2416 Metetra
.666 .745 .91 r
.55162 .2416 .56012 .25205 .57622 .24625 .56778 .23573 Metetra
.666 .745 .91 r
.56778 .23573 .57622 .24625 .59241 .24042 .58404 .22983 Metetra
.666 .745 .909 r
.58404 .22983 .59241 .24042 .6087 .23454 .60039 .22388 Metetra
.666 .745 .909 r
.60039 .22388 .6087 .23454 .62508 .22864 .61684 .21791 Metetra
.666 .745 .909 r
.61684 .21791 .62508 .22864 .64155 .2227 .63338 .21189 Metetra
.666 .745 .909 r
.63338 .21189 .64155 .2227 .65813 .21672 .65002 .20584 Metetra
.666 .744 .909 r
.65002 .20584 .65813 .21672 .6748 .2107 .66676 .19975 Metetra
.666 .744 .909 r
.66676 .19975 .6748 .2107 .69157 .20465 .6836 .19363 Metetra
.666 .744 .909 r
.6836 .19363 .69157 .20465 .70843 .19857 .70053 .18747 Metetra
.666 .744 .908 r
.70053 .18747 .70843 .19857 .7254 .19245 .71757 .18128 Metetra
.666 .744 .908 r
.71757 .18128 .7254 .19245 .74247 .18629 .73471 .17504 Metetra
.666 .744 .908 r
.73471 .17504 .74247 .18629 .75964 .1801 .75195 .16877 Metetra
.666 .743 .908 r
.75195 .16877 .75964 .1801 .77691 .17387 .76929 .16247 Metetra
.666 .743 .908 r
.76929 .16247 .77691 .17387 .79428 .1676 .78674 .15613 Metetra
.66 .741 .91 r
.15217 .37159 .16212 .38081 .17607 .37591 .16617 .36665 Metetra
.661 .742 .91 r
.16617 .36665 .17607 .37591 .1901 .37098 .18024 .36169 Metetra
.661 .742 .91 r
.18024 .36169 .1901 .37098 .20421 .36601 .19439 .35668 Metetra
.661 .742 .91 r
.19439 .35668 .20421 .36601 .21839 .36102 .20862 .35165 Metetra
.661 .743 .91 r
.20862 .35165 .21839 .36102 .23266 .35598 .22293 .34658 Metetra
.661 .743 .911 r
.22293 .34658 .23266 .35598 .24701 .35091 .23732 .34147 Metetra
.662 .743 .911 r
.23732 .34147 .24701 .35091 .26143 .34581 .2518 .33632 Metetra
.662 .744 .911 r
.2518 .33632 .26143 .34581 .27594 .34067 .26635 .33114 Metetra
.662 .744 .911 r
.26635 .33114 .27594 .34067 .29053 .33549 .28099 .32592 Metetra
.662 .744 .911 r
.28099 .32592 .29053 .33549 .3052 .33027 .29571 .32066 Metetra
.663 .745 .911 r
.29571 .32066 .3052 .33027 .31996 .32502 .31051 .31536 Metetra
.663 .745 .911 r
.31051 .31536 .31996 .32502 .33479 .31973 .3254 .31002 Metetra
.663 .745 .911 r
.3254 .31002 .33479 .31973 .34972 .3144 .34038 .30464 Metetra
.663 .745 .911 r
.34038 .30464 .34972 .3144 .36472 .30903 .35544 .29922 Metetra
.664 .746 .911 r
.35544 .29922 .36472 .30903 .37981 .30362 .37058 .29376 Metetra
.664 .746 .911 r
.37058 .29376 .37981 .30362 .39499 .29817 .38581 .28826 Metetra
.664 .746 .911 r
.38581 .28826 .39499 .29817 .41025 .29269 .40113 .28271 Metetra
.664 .746 .911 r
.40113 .28271 .41025 .29269 .42561 .28717 .41654 .27713 Metetra
.665 .746 .911 r
.41654 .27713 .42561 .28717 .44104 .2816 .43203 .27151 Metetra
.665 .746 .911 r
.43203 .27151 .44104 .2816 .45657 .276 .44762 .26585 Metetra
.665 .746 .911 r
.44762 .26585 .45657 .276 .47219 .27036 .46329 .26015 Metetra
.665 .746 .911 r
.46329 .26015 .47219 .27036 .48789 .26469 .47906 .25441 Metetra
.665 .746 .91 r
.47906 .25441 .48789 .26469 .50369 .25897 .49491 .24862 Metetra
.665 .746 .91 r
.49491 .24862 .50369 .25897 .51957 .25322 .51086 .2428 Metetra
.666 .746 .91 r
.51086 .2428 .51957 .25322 .53555 .24742 .5269 .23694 Metetra
.666 .746 .91 r
.5269 .23694 .53555 .24742 .55162 .2416 .54303 .23104 Metetra
.666 .746 .91 r
.54303 .23104 .55162 .2416 .56778 .23573 .55926 .22511 Metetra
.666 .745 .91 r
.55926 .22511 .56778 .23573 .58404 .22983 .57558 .21913 Metetra
.666 .745 .91 r
.57558 .21913 .58404 .22983 .60039 .22388 .59199 .21312 Metetra
.666 .745 .909 r
.59199 .21312 .60039 .22388 .61684 .21791 .60851 .20707 Metetra
.666 .745 .909 r
.60851 .20707 .61684 .21791 .63338 .21189 .62512 .20098 Metetra
.666 .745 .909 r
.62512 .20098 .63338 .21189 .65002 .20584 .64182 .19485 Metetra
.666 .745 .909 r
.64182 .19485 .65002 .20584 .66676 .19975 .65863 .18869 Metetra
.666 .744 .909 r
.65863 .18869 .66676 .19975 .6836 .19363 .67554 .18249 Metetra
.666 .744 .909 r
.67554 .18249 .6836 .19363 .70053 .18747 .69254 .17625 Metetra
.666 .744 .908 r
.69254 .17625 .70053 .18747 .71757 .18128 .70965 .16998 Metetra
.666 .744 .908 r
.70965 .16998 .71757 .18128 .73471 .17504 .72686 .16367 Metetra
.666 .744 .908 r
.72686 .16367 .73471 .17504 .75195 .16877 .74418 .15733 Metetra
.666 .743 .908 r
.74418 .15733 .75195 .16877 .76929 .16247 .76159 .15094 Metetra
.666 .743 .908 r
.76159 .15094 .76929 .16247 .78674 .15613 .77912 .14452 Metetra
.66 .741 .91 r
.14213 .36226 .15217 .37159 .16617 .36665 .15617 .35729 Metetra
.66 .741 .91 r
.15617 .35729 .16617 .36665 .18024 .36169 .17028 .35229 Metetra
.66 .742 .91 r
.17028 .35229 .18024 .36169 .19439 .35668 .18447 .34725 Metetra
.66 .742 .91 r
.18447 .34725 .19439 .35668 .20862 .35165 .19875 .34218 Metetra
.661 .743 .911 r
.19875 .34218 .20862 .35165 .22293 .34658 .2131 .33707 Metetra
.661 .743 .911 r
.2131 .33707 .22293 .34658 .23732 .34147 .22754 .33193 Metetra
.661 .743 .911 r
.22754 .33193 .23732 .34147 .2518 .33632 .24206 .32674 Metetra
.662 .744 .911 r
.24206 .32674 .2518 .33632 .26635 .33114 .25666 .32152 Metetra
.662 .744 .911 r
.25666 .32152 .26635 .33114 .28099 .32592 .27135 .31626 Metetra
.662 .745 .911 r
.27135 .31626 .28099 .32592 .29571 .32066 .28612 .31095 Metetra
.662 .745 .911 r
.28612 .31095 .29571 .32066 .31051 .31536 .30097 .30561 Metetra
.663 .745 .911 r
.30097 .30561 .31051 .31536 .3254 .31002 .31591 .30022 Metetra
.663 .746 .911 r
.31591 .30022 .3254 .31002 .34038 .30464 .33094 .29479 Metetra
.663 .746 .911 r
.33094 .29479 .34038 .30464 .35544 .29922 .34605 .28932 Metetra
.664 .746 .911 r
.34605 .28932 .35544 .29922 .37058 .29376 .36125 .28381 Metetra
.664 .746 .911 r
.36125 .28381 .37058 .29376 .38581 .28826 .37653 .27826 Metetra
.664 .746 .911 r
.37653 .27826 .38581 .28826 .40113 .28271 .39191 .27266 Metetra
.664 .746 .911 r
.39191 .27266 .40113 .28271 .41654 .27713 .40737 .26702 Metetra
.665 .747 .911 r
.40737 .26702 .41654 .27713 .43203 .27151 .42292 .26134 Metetra
.665 .747 .911 r
.42292 .26134 .43203 .27151 .44762 .26585 .43857 .25561 Metetra
.665 .747 .911 r
.43857 .25561 .44762 .26585 .46329 .26015 .4543 .24985 Metetra
.665 .747 .911 r
.4543 .24985 .46329 .26015 .47906 .25441 .47012 .24404 Metetra
.665 .747 .911 r
.47012 .24404 .47906 .25441 .49491 .24862 .48604 .23819 Metetra
.666 .747 .911 r
.48604 .23819 .49491 .24862 .51086 .2428 .50205 .2323 Metetra
.666 .746 .91 r
.50205 .2323 .51086 .2428 .5269 .23694 .51815 .22637 Metetra
.666 .746 .91 r
.51815 .22637 .5269 .23694 .54303 .23104 .53435 .2204 Metetra
.666 .746 .91 r
.53435 .2204 .54303 .23104 .55926 .22511 .55064 .21439 Metetra
.666 .746 .91 r
.55064 .21439 .55926 .22511 .57558 .21913 .56702 .20834 Metetra
.666 .746 .91 r
.56702 .20834 .57558 .21913 .59199 .21312 .5835 .20225 Metetra
.666 .746 .909 r
.5835 .20225 .59199 .21312 .60851 .20707 .60008 .19612 Metetra
.666 .745 .909 r
.60008 .19612 .60851 .20707 .62512 .20098 .61676 .18996 Metetra
.666 .745 .909 r
.61676 .18996 .62512 .20098 .64182 .19485 .63354 .18375 Metetra
.666 .745 .909 r
.63354 .18375 .64182 .19485 .65863 .18869 .65041 .17751 Metetra
.666 .745 .909 r
.65041 .17751 .65863 .18869 .67554 .18249 .66739 .17123 Metetra
.666 .744 .909 r
.66739 .17123 .67554 .18249 .69254 .17625 .68446 .16491 Metetra
.666 .744 .908 r
.68446 .16491 .69254 .17625 .70965 .16998 .70164 .15856 Metetra
.666 .744 .908 r
.70164 .15856 .70965 .16998 .72686 .16367 .71893 .15217 Metetra
.666 .744 .908 r
.71893 .15217 .72686 .16367 .74418 .15733 .73631 .14574 Metetra
.666 .743 .908 r
.73631 .14574 .74418 .15733 .76159 .15094 .75381 .13928 Metetra
.666 .743 .908 r
.75381 .13928 .76159 .15094 .77912 .14452 .77141 .13278 Metetra
.659 .74 .91 r
.13199 .35282 .14213 .36226 .15617 .35729 .14607 .34782 Metetra
.66 .741 .91 r
.14607 .34782 .15617 .35729 .17028 .35229 .16022 .34278 Metetra
.66 .741 .91 r
.16022 .34278 .17028 .35229 .18447 .34725 .17446 .33772 Metetra
.66 .742 .911 r
.17446 .33772 .18447 .34725 .19875 .34218 .18877 .33261 Metetra
.66 .742 .911 r
.18877 .33261 .19875 .34218 .2131 .33707 .20317 .32747 Metetra
.66 .743 .911 r
.20317 .32747 .2131 .33707 .22754 .33193 .21766 .32229 Metetra
.661 .743 .911 r
.21766 .32229 .22754 .33193 .24206 .32674 .23222 .31707 Metetra
.661 .744 .911 r
.23222 .31707 .24206 .32674 .25666 .32152 .24687 .31181 Metetra
.661 .744 .912 r
.24687 .31181 .25666 .32152 .27135 .31626 .2616 .30651 Metetra
.662 .745 .912 r
.2616 .30651 .27135 .31626 .28612 .31095 .27642 .30116 Metetra
.662 .745 .912 r
.27642 .30116 .28612 .31095 .30097 .30561 .29133 .29578 Metetra
.662 .746 .912 r
.29133 .29578 .30097 .30561 .31591 .30022 .30632 .29035 Metetra
.663 .746 .912 r
.30632 .29035 .31591 .30022 .33094 .29479 .32139 .28487 Metetra
.663 .746 .912 r
.32139 .28487 .33094 .29479 .34605 .28932 .33656 .27935 Metetra
.663 .747 .912 r
.33656 .27935 .34605 .28932 .36125 .28381 .35181 .27379 Metetra
.664 .747 .912 r
.35181 .27379 .36125 .28381 .37653 .27826 .36715 .26818 Metetra
.664 .747 .912 r
.36715 .26818 .37653 .27826 .39191 .27266 .38258 .26252 Metetra
.664 .747 .912 r
.38258 .26252 .39191 .27266 .40737 .26702 .3981 .25683 Metetra
.665 .747 .912 r
.3981 .25683 .40737 .26702 .42292 .26134 .41372 .25108 Metetra
.665 .747 .912 r
.41372 .25108 .42292 .26134 .43857 .25561 .42942 .24529 Metetra
.665 .747 .911 r
.42942 .24529 .43857 .25561 .4543 .24985 .44521 .23946 Metetra
.665 .747 .911 r
.44521 .23946 .4543 .24985 .47012 .24404 .46109 .23359 Metetra
.666 .747 .911 r
.46109 .23359 .47012 .24404 .48604 .23819 .47707 .22767 Metetra
.666 .747 .911 r
.47707 .22767 .48604 .23819 .50205 .2323 .49314 .22171 Metetra
.666 .747 .911 r
.49314 .22171 .50205 .2323 .51815 .22637 .50931 .2157 Metetra
.666 .747 .911 r
.50931 .2157 .51815 .22637 .53435 .2204 .52557 .20966 Metetra
.666 .747 .91 r
.52557 .20966 .53435 .2204 .55064 .21439 .54192 .20357 Metetra
.666 .747 .91 r
.54192 .20357 .55064 .21439 .56702 .20834 .55837 .19744 Metetra
.667 .746 .91 r
.55837 .19744 .56702 .20834 .5835 .20225 .57492 .19127 Metetra
.667 .746 .91 r
.57492 .19127 .5835 .20225 .60008 .19612 .59157 .18507 Metetra
.667 .746 .909 r
.59157 .18507 .60008 .19612 .61676 .18996 .60831 .17882 Metetra
.667 .745 .909 r
.60831 .17882 .61676 .18996 .63354 .18375 .62516 .17253 Metetra
.667 .745 .909 r
.62516 .17253 .63354 .18375 .65041 .17751 .6421 .16621 Metetra
.667 .745 .909 r
.6421 .16621 .65041 .17751 .66739 .17123 .65915 .15985 Metetra
.667 .745 .909 r
.65915 .15985 .66739 .17123 .68446 .16491 .67629 .15345 Metetra
.667 .744 .908 r
.67629 .15345 .68446 .16491 .70164 .15856 .69355 .14701 Metetra
.667 .744 .908 r
.69355 .14701 .70164 .15856 .71893 .15217 .7109 .14054 Metetra
.666 .744 .908 r
.7109 .14054 .71893 .15217 .73631 .14574 .72836 .13403 Metetra
.666 .743 .908 r
.72836 .13403 .73631 .14574 .75381 .13928 .74593 .12748 Metetra
.666 .743 .908 r
.74593 .12748 .75381 .13928 .77141 .13278 .76361 .1209 Metetra
.659 .74 .91 r
.12175 .34325 .13199 .35282 .14607 .34782 .13586 .33822 Metetra
.659 .74 .91 r
.13586 .33822 .14607 .34782 .16022 .34278 .15006 .33316 Metetra
.659 .741 .91 r
.15006 .33316 .16022 .34278 .17446 .33772 .16434 .32806 Metetra
.659 .741 .911 r
.16434 .32806 .17446 .33772 .18877 .33261 .1787 .32293 Metetra
.659 .742 .911 r
.1787 .32293 .18877 .33261 .20317 .32747 .19314 .31776 Metetra
.66 .742 .911 r
.19314 .31776 .20317 .32747 .21766 .32229 .20767 .31255 Metetra
.66 .743 .911 r
.20767 .31255 .21766 .32229 .23222 .31707 .22228 .3073 Metetra
.66 .744 .912 r
.22228 .3073 .23222 .31707 .24687 .31181 .23697 .30201 Metetra
.661 .744 .912 r
.23697 .30201 .24687 .31181 .2616 .30651 .25176 .29667 Metetra
.661 .745 .912 r
.25176 .29667 .2616 .30651 .27642 .30116 .26662 .29129 Metetra
.662 .745 .912 r
.26662 .29129 .27642 .30116 .29133 .29578 .28158 .28587 Metetra
.662 .746 .912 r
.28158 .28587 .29133 .29578 .30632 .29035 .29662 .28039 Metetra
.662 .746 .913 r
.29662 .28039 .30632 .29035 .32139 .28487 .31175 .27487 Metetra
.663 .747 .913 r
.31175 .27487 .32139 .28487 .33656 .27935 .32697 .26931 Metetra
.663 .747 .913 r
.32697 .26931 .33656 .27935 .35181 .27379 .34227 .26369 Metetra
.664 .748 .913 r
.34227 .26369 .35181 .27379 .36715 .26818 .35767 .25803 Metetra
.664 .748 .913 r
.35767 .25803 .36715 .26818 .38258 .26252 .37316 .25232 Metetra
.664 .748 .913 r
.37316 .25232 .38258 .26252 .3981 .25683 .38874 .24656 Metetra
.665 .748 .912 r
.38874 .24656 .3981 .25683 .41372 .25108 .4044 .24076 Metetra
.665 .748 .912 r
.4044 .24076 .41372 .25108 .42942 .24529 .42016 .23491 Metetra
.665 .748 .912 r
.42016 .23491 .42942 .24529 .44521 .23946 .43602 .22901 Metetra
.666 .748 .912 r
.43602 .22901 .44521 .23946 .46109 .23359 .45196 .22306 Metetra
.666 .748 .912 r
.45196 .22306 .46109 .23359 .47707 .22767 .468 .21707 Metetra
.666 .748 .911 r
.468 .21707 .47707 .22767 .49314 .22171 .48414 .21103 Metetra
.666 .748 .911 r
.48414 .21103 .49314 .22171 .50931 .2157 .50036 .20495 Metetra
.667 .748 .911 r
.50036 .20495 .50931 .2157 .52557 .20966 .51669 .19882 Metetra
.667 .748 .911 r
.51669 .19882 .52557 .20966 .54192 .20357 .53311 .19266 Metetra
.667 .747 .91 r
.53311 .19266 .54192 .20357 .55837 .19744 .54963 .18645 Metetra
.667 .747 .91 r
.54963 .18645 .55837 .19744 .57492 .19127 .56624 .18019 Metetra
.667 .747 .91 r
.56624 .18019 .57492 .19127 .59157 .18507 .58295 .1739 Metetra
.667 .746 .909 r
.58295 .1739 .59157 .18507 .60831 .17882 .59977 .16757 Metetra
.667 .746 .909 r
.59977 .16757 .60831 .17882 .62516 .17253 .61668 .1612 Metetra
.667 .746 .909 r
.61668 .1612 .62516 .17253 .6421 .16621 .6337 .15479 Metetra
.667 .745 .909 r
.6337 .15479 .6421 .16621 .65915 .15985 .65081 .14834 Metetra
.667 .745 .908 r
.65081 .14834 .65915 .15985 .67629 .15345 .66803 .14185 Metetra
.667 .744 .908 r
.66803 .14185 .67629 .15345 .69355 .14701 .68536 .13533 Metetra
.667 .744 .908 r
.68536 .13533 .69355 .14701 .7109 .14054 .70279 .12876 Metetra
.667 .744 .908 r
.70279 .12876 .7109 .14054 .72836 .13403 .72032 .12217 Metetra
.667 .743 .908 r
.72032 .12217 .72836 .13403 .74593 .12748 .73796 .11553 Metetra
.667 .743 .907 r
.73796 .11553 .74593 .12748 .76361 .1209 .75571 .10886 Metetra
.658 .738 .909 r
.11141 .33355 .12175 .34325 .13586 .33822 .12556 .32849 Metetra
.658 .739 .91 r
.12556 .32849 .13586 .33822 .15006 .33316 .1398 .32341 Metetra
.658 .74 .91 r
.1398 .32341 .15006 .33316 .16434 .32806 .15412 .31829 Metetra
.658 .74 .91 r
.15412 .31829 .16434 .32806 .1787 .32293 .16852 .31313 Metetra
.659 .741 .911 r
.16852 .31313 .1787 .32293 .19314 .31776 .18301 .30794 Metetra
.659 .742 .911 r
.18301 .30794 .19314 .31776 .20767 .31255 .19758 .3027 Metetra
.659 .743 .912 r
.19758 .3027 .20767 .31255 .22228 .3073 .21223 .29743 Metetra
.66 .743 .912 r
.21223 .29743 .22228 .3073 .23697 .30201 .22697 .29211 Metetra
.66 .744 .912 r
.22697 .29211 .23697 .30201 .25176 .29667 .2418 .28674 Metetra
.66 .745 .913 r
.2418 .28674 .25176 .29667 .26662 .29129 .25672 .28133 Metetra
.661 .746 .913 r
.25672 .28133 .26662 .29129 .28158 .28587 .27172 .27587 Metetra
.661 .746 .913 r
.27172 .27587 .28158 .28587 .29662 .28039 .28681 .27037 Metetra
.662 .747 .913 r
.28681 .27037 .29662 .28039 .31175 .27487 .30199 .26481 Metetra
.662 .748 .913 r
.30199 .26481 .31175 .27487 .32697 .26931 .31727 .2592 Metetra
.663 .748 .914 r
.31727 .2592 .32697 .26931 .34227 .26369 .33263 .25354 Metetra
.663 .749 .914 r
.33263 .25354 .34227 .26369 .35767 .25803 .34808 .24783 Metetra
.664 .749 .914 r
.34808 .24783 .35767 .25803 .37316 .25232 .36362 .24206 Metetra
.664 .749 .913 r
.36362 .24206 .37316 .25232 .38874 .24656 .37926 .23625 Metetra
.665 .749 .913 r
.37926 .23625 .38874 .24656 .4044 .24076 .39499 .23038 Metetra
.665 .75 .913 r
.39499 .23038 .4044 .24076 .42016 .23491 .41081 .22446 Metetra
.666 .75 .913 r
.41081 .22446 .42016 .23491 .43602 .22901 .42672 .21849 Metetra
.666 .75 .913 r
.42672 .21849 .43602 .22901 .45196 .22306 .44273 .21247 Metetra
.666 .75 .912 r
.44273 .21247 .45196 .22306 .468 .21707 .45883 .2064 Metetra
.667 .749 .912 r
.45883 .2064 .468 .21707 .48414 .21103 .47503 .20028 Metetra
.667 .749 .912 r
.47503 .20028 .48414 .21103 .50036 .20495 .49132 .19412 Metetra
.667 .749 .911 r
.49132 .19412 .50036 .20495 .51669 .19882 .50771 .18791 Metetra
.667 .749 .911 r
.50771 .18791 .51669 .19882 .53311 .19266 .5242 .18166 Metetra
.667 .748 .911 r
.5242 .18166 .53311 .19266 .54963 .18645 .54078 .17536 Metetra
.668 .748 .91 r
.54078 .17536 .54963 .18645 .56624 .18019 .55746 .16901 Metetra
.668 .747 .91 r
.55746 .16901 .56624 .18019 .58295 .1739 .57425 .16263 Metetra
.668 .747 .91 r
.57425 .16263 .58295 .1739 .59977 .16757 .59113 .15621 Metetra
.668 .746 .909 r
.59113 .15621 .59977 .16757 .61668 .1612 .60811 .14974 Metetra
.668 .746 .909 r
.60811 .14974 .61668 .1612 .6337 .15479 .6252 .14324 Metetra
.668 .745 .909 r
.6252 .14324 .6337 .15479 .65081 .14834 .64238 .13669 Metetra
.667 .745 .908 r
.64238 .13669 .65081 .14834 .66803 .14185 .65968 .13011 Metetra
.667 .745 .908 r
.65968 .13011 .66803 .14185 .68536 .13533 .67707 .1235 Metetra
.667 .744 .908 r
.67707 .1235 .68536 .13533 .70279 .12876 .69457 .11684 Metetra
.667 .744 .907 r
.69457 .11684 .70279 .12876 .72032 .12217 .71218 .11015 Metetra
.667 .743 .907 r
.71218 .11015 .72032 .12217 .73796 .11553 .7299 .10343 Metetra
.667 .743 .907 r
.7299 .10343 .73796 .11553 .75571 .10886 .74773 .09667 Metetra
.657 .737 .909 r
.10097 .32368 .11141 .33355 .12556 .32849 .11516 .3186 Metetra
.657 .738 .909 r
.11516 .3186 .12556 .32849 .1398 .32341 .12944 .3135 Metetra
.657 .738 .91 r
.12944 .3135 .1398 .32341 .15412 .31829 .1438 .30836 Metetra
.657 .739 .91 r
.1438 .30836 .15412 .31829 .16852 .31313 .15824 .30319 Metetra
.657 .74 .911 r
.15824 .30319 .16852 .31313 .18301 .30794 .17277 .29798 Metetra
.658 .741 .911 r
.17277 .29798 .18301 .30794 .19758 .3027 .18738 .29273 Metetra
.658 .742 .912 r
.18738 .29273 .19758 .3027 .21223 .29743 .20208 .28744 Metetra
.659 .743 .912 r
.20208 .28744 .21223 .29743 .22697 .29211 .21686 .2821 Metetra
.659 .744 .913 r
.21686 .2821 .22697 .29211 .2418 .28674 .23174 .27672 Metetra
.659 .745 .913 r
.23174 .27672 .2418 .28674 .25672 .28133 .2467 .27129 Metetra
.66 .746 .914 r
.2467 .27129 .25672 .28133 .27172 .27587 .26175 .26581 Metetra
.661 .747 .914 r
.26175 .26581 .27172 .27587 .28681 .27037 .2769 .26027 Metetra
.661 .748 .914 r
.2769 .26027 .28681 .27037 .30199 .26481 .29213 .25469 Metetra
.662 .749 .915 r
.29213 .25469 .30199 .26481 .31727 .2592 .30746 .24904 Metetra
.662 .749 .915 r
.30746 .24904 .31727 .2592 .33263 .25354 .32287 .24334 Metetra
.663 .75 .915 r
.32287 .24334 .33263 .25354 .34808 .24783 .33838 .23758 Metetra
.664 .75 .915 r
.33838 .23758 .34808 .24783 .36362 .24206 .35398 .23177 Metetra
.664 .751 .915 r
.35398 .23177 .36362 .24206 .37926 .23625 .36968 .22589 Metetra
.665 .751 .915 r
.36968 .22589 .37926 .23625 .39499 .23038 .38546 .21996 Metetra
.665 .751 .914 r
.38546 .21996 .39499 .23038 .41081 .22446 .40134 .21398 Metetra
.666 .751 .914 r
.40134 .21398 .41081 .22446 .42672 .21849 .41732 .20793 Metetra
.666 .751 .914 r
.41732 .20793 .42672 .21849 .44273 .21247 .43339 .20184 Metetra
.667 .751 .913 r
.43339 .20184 .44273 .21247 .45883 .2064 .44956 .19569 Metetra
.667 .751 .913 r
.44956 .19569 .45883 .2064 .47503 .20028 .46582 .18948 Metetra
.668 .751 .913 r
.46582 .18948 .47503 .20028 .49132 .19412 .48218 .18323 Metetra
.668 .75 .912 r
.48218 .18323 .49132 .19412 .50771 .18791 .49863 .17692 Metetra
.668 .75 .912 r
.49863 .17692 .50771 .18791 .5242 .18166 .51519 .17057 Metetra
.668 .749 .911 r
.51519 .17057 .5242 .18166 .54078 .17536 .53184 .16418 Metetra
.668 .749 .911 r
.53184 .16418 .54078 .17536 .55746 .16901 .54859 .15774 Metetra
.668 .748 .91 r
.54859 .15774 .55746 .16901 .57425 .16263 .56544 .15125 Metetra
.668 .748 .91 r
.56544 .15125 .57425 .16263 .59113 .15621 .58239 .14473 Metetra
.668 .747 .909 r
.58239 .14473 .59113 .15621 .60811 .14974 .59944 .13816 Metetra
.668 .746 .909 r
.59944 .13816 .60811 .14974 .6252 .14324 .6166 .13155 Metetra
.668 .746 .908 r
.6166 .13155 .6252 .14324 .64238 .13669 .63386 .12491 Metetra
.668 .745 .908 r
.63386 .12491 .64238 .13669 .65968 .13011 .65122 .11823 Metetra
.668 .744 .908 r
.65122 .11823 .65968 .13011 .67707 .1235 .66869 .11152 Metetra
.668 .744 .907 r
.66869 .11152 .67707 .1235 .69457 .11684 .68627 .10476 Metetra
.668 .743 .907 r
.68627 .10476 .69457 .11684 .71218 .11015 .70395 .09798 Metetra
.667 .743 .907 r
.70395 .09798 .71218 .11015 .7299 .10343 .72175 .09116 Metetra
.667 .742 .906 r
.72175 .09116 .7299 .10343 .74773 .09667 .73965 .0843 Metetra
.656 .735 .907 r
.09044 .31362 .10097 .32368 .11516 .3186 .10467 .30852 Metetra
.656 .735 .908 r
.10467 .30852 .11516 .3186 .12944 .3135 .11898 .3034 Metetra
.656 .736 .909 r
.11898 .3034 .12944 .3135 .1438 .30836 .13338 .29825 Metetra
.656 .737 .909 r
.13338 .29825 .1438 .30836 .15824 .30319 .14786 .29307 Metetra
.656 .738 .91 r
.14786 .29307 .15824 .30319 .17277 .29798 .16243 .28785 Metetra
.656 .739 .911 r
.16243 .28785 .17277 .29798 .18738 .29273 .17708 .2826 Metetra
.657 .741 .912 r
.17708 .2826 .18738 .29273 .20208 .28744 .19182 .27731 Metetra
.657 .742 .912 r
.19182 .27731 .20208 .28744 .21686 .2821 .20665 .27198 Metetra
.657 .743 .913 r
.20665 .27198 .21686 .2821 .23174 .27672 .22157 .26659 Metetra
.658 .745 .914 r
.22157 .26659 .23174 .27672 .2467 .27129 .23658 .26116 Metetra
.659 .746 .915 r
.23658 .26116 .2467 .27129 .26175 .26581 .25168 .25567 Metetra
.659 .747 .915 r
.25168 .25567 .26175 .26581 .2769 .26027 .26687 .25013 Metetra
.66 .749 .916 r
.26687 .25013 .2769 .26027 .29213 .25469 .28215 .24452 Metetra
.661 .75 .916 r
.28215 .24452 .29213 .25469 .30746 .24904 .29753 .23885 Metetra
.662 .751 .916 r
.29753 .23885 .30746 .24904 .32287 .24334 .313 .23312 Metetra
.662 .752 .916 r
.313 .23312 .32287 .24334 .33838 .23758 .32857 .22733 Metetra
.663 .753 .917 r
.32857 .22733 .33838 .23758 .35398 .23177 .34422 .22146 Metetra
.664 .753 .917 r
.34422 .22146 .35398 .23177 .36968 .22589 .35998 .21554 Metetra
.665 .754 .916 r
.35998 .21554 .36968 .22589 .38546 .21996 .37583 .20955 Metetra
.666 .754 .916 r
.37583 .20955 .38546 .21996 .40134 .21398 .39177 .20349 Metetra
.666 .754 .916 r
.39177 .20349 .40134 .21398 .41732 .20793 .40781 .19737 Metetra
.667 .754 .915 r
.40781 .19737 .41732 .20793 .43339 .20184 .42394 .19119 Metetra
.668 .754 .915 r
.42394 .19119 .43339 .20184 .44956 .19569 .44017 .18495 Metetra
.668 .753 .914 r
.44017 .18495 .44956 .19569 .46582 .18948 .4565 .17865 Metetra
.668 .753 .914 r
.4565 .17865 .46582 .18948 .48218 .18323 .47293 .17229 Metetra
.669 .752 .913 r
.47293 .17229 .48218 .18323 .49863 .17692 .48945 .16589 Metetra
.669 .752 .912 r
.48945 .16589 .49863 .17692 .51519 .17057 .50607 .15943 Metetra
.669 .751 .912 r
.50607 .15943 .51519 .17057 .53184 .16418 .52279 .15292 Metetra
.669 .75 .911 r
.52279 .15292 .53184 .16418 .54859 .15774 .53961 .14636 Metetra
.669 .749 .91 r
.53961 .14636 .54859 .15774 .56544 .15125 .55653 .13976 Metetra
.669 .748 .91 r
.55653 .13976 .56544 .15125 .58239 .14473 .57355 .13312 Metetra
.669 .748 .909 r
.57355 .13312 .58239 .14473 .59944 .13816 .59067 .12644 Metetra
.669 .747 .908 r
.59067 .12644 .59944 .13816 .6166 .13155 .6079 .11973 Metetra
.669 .746 .908 r
.6079 .11973 .6166 .13155 .63386 .12491 .62523 .11297 Metetra
.669 .745 .907 r
.62523 .11297 .63386 .12491 .65122 .11823 .64267 .10618 Metetra
.669 .744 .907 r
.64267 .10618 .65122 .11823 .66869 .11152 .66021 .09936 Metetra
.668 .743 .906 r
.66021 .09936 .66869 .11152 .68627 .10476 .67786 .0925 Metetra
.668 .743 .906 r
.67786 .0925 .68627 .10476 .70395 .09798 .69562 .08561 Metetra
.668 .742 .906 r
.69562 .08561 .70395 .09798 .72175 .09116 .71349 .07869 Metetra
.668 .742 .906 r
.71349 .07869 .72175 .09116 .73965 .0843 .73147 .07174 Metetra
.655 .731 .906 r
.07982 .30331 .09044 .31362 .10467 .30852 .09409 .29818 Metetra
.655 .732 .906 r
.09409 .29818 .10467 .30852 .11898 .3034 .10844 .29305 Metetra
.654 .733 .907 r
.10844 .29305 .11898 .3034 .13338 .29825 .12287 .28789 Metetra
.654 .734 .908 r
.12287 .28789 .13338 .29825 .14786 .29307 .13739 .28271 Metetra
.654 .735 .909 r
.13739 .28271 .14786 .29307 .16243 .28785 .15199 .27751 Metetra
.654 .736 .91 r
.15199 .27751 .16243 .28785 .17708 .2826 .16668 .27227 Metetra
.654 .738 .911 r
.16668 .27227 .17708 .2826 .19182 .27731 .18146 .267 Metetra
.655 .74 .912 r
.18146 .267 .19182 .27731 .20665 .27198 .19633 .26169 Metetra
.655 .742 .913 r
.19633 .26169 .20665 .27198 .22157 .26659 .21129 .25634 Metetra
.656 .743 .914 r
.21129 .25634 .22157 .26659 .23658 .26116 .22634 .25093 Metetra
.656 .745 .916 r
.22634 .25093 .23658 .26116 .25168 .25567 .24149 .24546 Metetra
.657 .747 .916 r
.24149 .24546 .25168 .25567 .26687 .25013 .25672 .23993 Metetra
.658 .749 .917 r
.25672 .23993 .26687 .25013 .28215 .24452 .27206 .23433 Metetra
.659 .751 .918 r
.27206 .23433 .28215 .24452 .29753 .23885 .28749 .22867 Metetra
.66 .753 .919 r
.28749 .22867 .29753 .23885 .313 .23312 .30301 .22293 Metetra
.661 .754 .919 r
.30301 .22293 .313 .23312 .32857 .22733 .31863 .21711 Metetra
.663 .756 .919 r
.31863 .21711 .32857 .22733 .34422 .22146 .33435 .21122 Metetra
.664 .757 .919 r
.33435 .21122 .34422 .22146 .35998 .21554 .35016 .20525 Metetra
.665 .758 .919 r
.35016 .20525 .35998 .21554 .37583 .20955 .36607 .1992 Metetra
.666 .758 .919 r
.36607 .1992 .37583 .20955 .39177 .20349 .38207 .19307 Metetra
.667 .758 .919 r
.38207 .19307 .39177 .20349 .40781 .19737 .39818 .18687 Metetra
.668 .758 .918 r
.39818 .18687 .40781 .19737 .42394 .19119 .41438 .18059 Metetra
.669 .758 .917 r
.41438 .18059 .42394 .19119 .44017 .18495 .43068 .17425 Metetra
.669 .757 .916 r
.43068 .17425 .44017 .18495 .4565 .17865 .44707 .16783 Metetra
.67 .757 .915 r
.44707 .16783 .4565 .17865 .47293 .17229 .46357 .16136 Metetra
.67 .756 .914 r
.46357 .16136 .47293 .17229 .48945 .16589 .48016 .15482 Metetra
.671 .754 .913 r
.48016 .15482 .48945 .16589 .50607 .15943 .49685 .14823 Metetra
.671 .753 .912 r
.49685 .14823 .50607 .15943 .52279 .15292 .51364 .14159 Metetra
.671 .752 .911 r
.51364 .14159 .52279 .15292 .53961 .14636 .53053 .1349 Metetra
.671 .751 .91 r
.53053 .1349 .53961 .14636 .55653 .13976 .54752 .12816 Metetra
.671 .749 .909 r
.54752 .12816 .55653 .13976 .57355 .13312 .56461 .12138 Metetra
.671 .748 .908 r
.56461 .12138 .57355 .13312 .59067 .12644 .58181 .11457 Metetra
.67 .747 .908 r
.58181 .11457 .59067 .12644 .6079 .11973 .5991 .10772 Metetra
.67 .745 .907 r
.5991 .10772 .6079 .11973 .62523 .11297 .61651 .10084 Metetra
.67 .744 .906 r
.61651 .10084 .62523 .11297 .64267 .10618 .63402 .09393 Metetra
.669 .743 .906 r
.63402 .09393 .64267 .10618 .66021 .09936 .65163 .08699 Metetra
.669 .742 .905 r
.65163 .08699 .66021 .09936 .67786 .0925 .66936 .08002 Metetra
.669 .741 .905 r
.66936 .08002 .67786 .0925 .69562 .08561 .68719 .07302 Metetra
.668 .741 .904 r
.68719 .07302 .69562 .08561 .71349 .07869 .70514 .066 Metetra
.668 .74 .904 r
.70514 .066 .71349 .07869 .73147 .07174 .7232 .05895 Metetra
.654 .727 .902 r
.06912 .29269 .07982 .30331 .09409 .29818 .08343 .28751 Metetra
.654 .727 .903 r
.08343 .28751 .09409 .29818 .10844 .29305 .09781 .28234 Metetra
.653 .727 .904 r
.09781 .28234 .10844 .29305 .12287 .28789 .11228 .27717 Metetra
.652 .728 .905 r
.11228 .27717 .12287 .28789 .13739 .28271 .12684 .272 Metetra
.652 .729 .906 r
.12684 .272 .13739 .28271 .15199 .27751 .14148 .26681 Metetra
.651 .73 .907 r
.14148 .26681 .15199 .27751 .16668 .27227 .1562 .26161 Metetra
.651 .732 .909 r
.1562 .26161 .16668 .27227 .18146 .267 .17101 .25639 Metetra
.651 .735 .91 r
.17101 .25639 .18146 .267 .19633 .26169 .18591 .25115 Metetra
.651 .737 .912 r
.18591 .25115 .19633 .26169 .21129 .25634 .20091 .24586 Metetra
.652 .74 .914 r
.20091 .24586 .21129 .25634 .22634 .25093 .216 .24053 Metetra
.653 .743 .916 r
.216 .24053 .22634 .25093 .24149 .24546 .23118 .23514 Metetra
.654 .747 .918 r
.23118 .23514 .24149 .24546 .25672 .23993 .24646 .22968 Metetra
.655 .75 .92 r
.24646 .22968 .25672 .23993 .27206 .23433 .26184 .22415 Metetra
.656 .753 .921 r
.26184 .22415 .27206 .23433 .28749 .22867 .27731 .21854 Metetra
.658 .756 .922 r
.27731 .21854 .28749 .22867 .30301 .22293 .29289 .21284 Metetra
.659 .759 .923 r
.29289 .21284 .30301 .22293 .31863 .21711 .30856 .20705 Metetra
.661 .761 .924 r
.30856 .20705 .31863 .21711 .33435 .21122 .32433 .20115 Metetra
.663 .763 .924 r
.32433 .20115 .33435 .21122 .35016 .20525 .34021 .19516 Metetra
.665 .764 .924 r
.34021 .19516 .35016 .20525 .36607 .1992 .35618 .18906 Metetra
.666 .765 .924 r
.35618 .18906 .36607 .1992 .38207 .19307 .37225 .18286 Metetra
.668 .766 .923 r
.37225 .18286 .38207 .19307 .39818 .18687 .38842 .17657 Metetra
.669 .766 .922 r
.38842 .17657 .39818 .18687 .41438 .18059 .40469 .17018 Metetra
.671 .765 .921 r
.40469 .17018 .41438 .18059 .43068 .17425 .42106 .1637 Metetra
.672 .764 .92 r
.42106 .1637 .43068 .17425 .44707 .16783 .43753 .15713 Metetra
.672 .762 .918 r
.43753 .15713 .44707 .16783 .46357 .16136 .45409 .15049 Metetra
.673 .761 .917 r
.45409 .15049 .46357 .16136 .48016 .15482 .47076 .14378 Metetra
.673 .759 .915 r
.47076 .14378 .48016 .15482 .49685 .14823 .48752 .13701 Metetra
.674 .756 .913 r
.48752 .13701 .49685 .14823 .51364 .14159 .50438 .13019 Metetra
.674 .754 .911 r
.50438 .13019 .51364 .14159 .53053 .1349 .52134 .12332 Metetra
.673 .752 .91 r
.52134 .12332 .53053 .1349 .54752 .12816 .5384 .1164 Metetra
.673 .75 .908 r
.5384 .1164 .54752 .12816 .56461 .12138 .55557 .10945 Metetra
.673 .748 .907 r
.55557 .10945 .56461 .12138 .58181 .11457 .57283 .10247 Metetra
.672 .746 .906 r
.57283 .10247 .58181 .11457 .5991 .10772 .5902 .09546 Metetra
.672 .744 .905 r
.5902 .09546 .5991 .10772 .61651 .10084 .60768 .08844 Metetra
.671 .742 .904 r
.60768 .08844 .61651 .10084 .63402 .09393 .62526 .08139 Metetra
.67 .741 .903 r
.62526 .08139 .63402 .09393 .65163 .08699 .64295 .07432 Metetra
.67 .739 .903 r
.64295 .07432 .65163 .08699 .66936 .08002 .66074 .06723 Metetra
.669 .738 .902 r
.66074 .06723 .66936 .08002 .68719 .07302 .67865 .06013 Metetra
.668 .738 .902 r
.67865 .06013 .68719 .07302 .70514 .066 .69668 .053 Metetra
.668 .737 .902 r
.69668 .053 .70514 .066 .7232 .05895 .71481 .04586 Metetra
.656 .723 .898 r
.05834 .28173 .06912 .29269 .08343 .28751 .0727 .27643 Metetra
.654 .721 .898 r
.0727 .27643 .08343 .28751 .09781 .28234 .08714 .27116 Metetra
.653 .72 .898 r
.08714 .27116 .09781 .28234 .11228 .27717 .10165 .26591 Metetra
.651 .719 .898 r
.10165 .26591 .11228 .27717 .12684 .272 .11624 .26069 Metetra
.65 .719 .899 r
.11624 .26069 .12684 .272 .14148 .26681 .13092 .25549 Metetra
.648 .719 .9 r
.13092 .25549 .14148 .26681 .1562 .26161 .14567 .25032 Metetra
.647 .721 .902 r
.14567 .25032 .1562 .26161 .17101 .25639 .16051 .24517 Metetra
.646 .723 .904 r
.16051 .24517 .17101 .25639 .18591 .25115 .17544 .24003 Metetra
.645 .726 .907 r
.17544 .24003 .18591 .25115 .20091 .24586 .19046 .23489 Metetra
.645 .73 .911 r
.19046 .23489 .20091 .24586 .216 .24053 .20557 .22973 Metetra
.645 .735 .914 r
.20557 .22973 .216 .24053 .23118 .23514 .22078 .22453 Metetra
.646 .741 .918 r
.22078 .22453 .23118 .23514 .24646 .22968 .23608 .21929 Metetra
.647 .747 .922 r
.23608 .21929 .24646 .22968 .26184 .22415 .25149 .21396 Metetra
.649 .753 .926 r
.25149 .21396 .26184 .22415 .27731 .21854 .267 .20855 Metetra
.651 .759 .929 r
.267 .20855 .27731 .21854 .29289 .21284 .28262 .20302 Metetra
.654 .765 .931 r
.28262 .20302 .29289 .21284 .30856 .20705 .29834 .19736 Metetra
.657 .771 .933 r
.29834 .19736 .30856 .20705 .32433 .20115 .31416 .19156 Metetra
.66 .775 .935 r
.31416 .19156 .32433 .20115 .34021 .19516 .33009 .18561 Metetra
.663 .779 .935 r
.33009 .18561 .34021 .19516 .35618 .18906 .34613 .1795 Metetra
.666 .781 .934 r
.34613 .1795 .35618 .18906 .37225 .18286 .36227 .17324 Metetra
.669 .782 .933 r
.36227 .17324 .37225 .18286 .38842 .17657 .37852 .16682 Metetra
.672 .781 .931 r
.37852 .16682 .38842 .17657 .40469 .17018 .39486 .16026 Metetra
.675 .78 .929 r
.39486 .16026 .40469 .17018 .42106 .1637 .41131 .15357 Metetra
.677 .777 .926 r
.41131 .15357 .42106 .1637 .43753 .15713 .42785 .14676 Metetra
.678 .774 .923 r
.42785 .14676 .43753 .15713 .45409 .15049 .4445 .13984 Metetra
.679 .77 .92 r
.4445 .13984 .45409 .15049 .47076 .14378 .46124 .13285 Metetra
.679 .765 .916 r
.46124 .13285 .47076 .14378 .48752 .13701 .47808 .12578 Metetra
.679 .761 .913 r
.47808 .12578 .48752 .13701 .50438 .13019 .49501 .11866 Metetra
.679 .756 .91 r
.49501 .11866 .50438 .13019 .52134 .12332 .51205 .11151 Metetra
.678 .752 .907 r
.51205 .11151 .52134 .12332 .5384 .1164 .52918 .10433 Metetra
.677 .747 .904 r
.52918 .10433 .5384 .1164 .55557 .10945 .54641 .09714 Metetra
.675 .744 .902 r
.54641 .09714 .55557 .10945 .57283 .10247 .56375 .08994 Metetra
.674 .741 .9 r
.56375 .08994 .57283 .10247 .5902 .09546 .58119 .08274 Metetra
.673 .738 .899 r
.58119 .08274 .5902 .09546 .60768 .08844 .59873 .07555 Metetra
.672 .736 .898 r
.59873 .07555 .60768 .08844 .62526 .08139 .61638 .06836 Metetra
.67 .734 .898 r
.61638 .06836 .62526 .08139 .64295 .07432 .63415 .06117 Metetra
.669 .733 .898 r
.63415 .06117 .64295 .07432 .66074 .06723 .65202 .05398 Metetra
.668 .732 .898 r
.65202 .05398 .66074 .06723 .67865 .06013 .67 .0468 Metetra
.667 .732 .898 r
.67 .0468 .67865 .06013 .69668 .053 .6881 .03961 Metetra
.667 .732 .898 r
.6881 .03961 .69668 .053 .71481 .04586 .70632 .03241 Metetra
.661 .726 .898 r
.04745 .27076 .05834 .28173 .0727 .27643 .06187 .26522 Metetra
.66 .722 .895 r
.06187 .26522 .0727 .27643 .08714 .27116 .07638 .25967 Metetra
.658 .717 .892 r
.07638 .25967 .08714 .27116 .10165 .26591 .09096 .25414 Metetra
.656 .713 .89 r
.09096 .25414 .10165 .26591 .11624 .26069 .10563 .24864 Metetra
.653 .708 .887 r
.10563 .24864 .11624 .26069 .13092 .25549 .12036 .24319 Metetra
.65 .703 .885 r
.12036 .24319 .13092 .25549 .14567 .25032 .13518 .23781 Metetra
.647 .699 .885 r
.13518 .23781 .14567 .25032 .16051 .24517 .15006 .23253 Metetra
.643 .697 .885 r
.15006 .23253 .16051 .24517 .17544 .24003 .16503 .22736 Metetra
.639 .696 .887 r
.16503 .22736 .17544 .24003 .19046 .23489 .18006 .22232 Metetra
.635 .698 .891 r
.18006 .22232 .19046 .23489 .20557 .22973 .19518 .2174 Metetra
.632 .703 .897 r
.19518 .2174 .20557 .22973 .22078 .22453 .21039 .2126 Metetra
.629 .711 .905 r
.21039 .2126 .22078 .22453 .23608 .21929 .22568 .20788 Metetra
.628 .722 .915 r
.22568 .20788 .23608 .21929 .25149 .21396 .24107 .20319 Metetra
.627 .736 .926 r
.24107 .20319 .25149 .21396 .267 .20855 .25657 .19848 Metetra
.628 .753 .937 r
.25657 .19848 .267 .20855 .28262 .20302 .27217 .19367 Metetra
.631 .772 .948 r
.27217 .19367 .28262 .20302 .29834 .19736 .2879 .18868 Metetra
.635 .79 .956 r
.2879 .18868 .29834 .19736 .31416 .19156 .30375 .18344 Metetra
.642 .806 .962 r
.30375 .18344 .31416 .19156 .33009 .18561 .31973 .17788 Metetra
.65 .82 .965 r
.31973 .17788 .33009 .18561 .34613 .1795 .33582 .17197 Metetra
.66 .829 .965 r
.33582 .17197 .34613 .1795 .36227 .17324 .35204 .16569 Metetra
.67 .833 .962 r
.35204 .16569 .36227 .17324 .37852 .16682 .36838 .15905 Metetra
.68 .831 .956 r
.36838 .15905 .37852 .16682 .39486 .16026 .38482 .15206 Metetra
.687 .825 .949 r
.38482 .15206 .39486 .16026 .41131 .15357 .40138 .14479 Metetra
.693 .815 .94 r
.40138 .14479 .41131 .15357 .42785 .14676 .41802 .13729 Metetra
.695 .802 .931 r
.41802 .13729 .42785 .14676 .4445 .13984 .43477 .12963 Metetra
.696 .787 .921 r
.43477 .12963 .4445 .13984 .46124 .13285 .4516 .12189 Metetra
.694 .772 .912 r
.4516 .12189 .46124 .13285 .47808 .12578 .46853 .11413 Metetra
.691 .759 .904 r
.46853 .11413 .47808 .12578 .49501 .11866 .48554 .10638 Metetra
.687 .747 .897 r
.48554 .10638 .49501 .11866 .51205 .11151 .50265 .0987 Metetra
.683 .737 .892 r
.50265 .0987 .51205 .11151 .52918 .10433 .51985 .09111 Metetra
.679 .729 .888 r
.51985 .09111 .52918 .10433 .54641 .09714 .53715 .08361 Metetra
.675 .723 .886 r
.53715 .08361 .54641 .09714 .56375 .08994 .55455 .0762 Metetra
.672 .72 .885 r
.55455 .0762 .56375 .08994 .58119 .08274 .57206 .06888 Metetra
.67 .718 .885 r
.57206 .06888 .58119 .08274 .59873 .07555 .58967 .06163 Metetra
.668 .718 .886 r
.58967 .06163 .59873 .07555 .61638 .06836 .60739 .05443 Metetra
.666 .718 .888 r
.60739 .05443 .61638 .06836 .63415 .06117 .62523 .04728 Metetra
.665 .72 .89 r
.62523 .04728 .63415 .06117 .65202 .05398 .64317 .04014 Metetra
.664 .722 .892 r
.64317 .04014 .65202 .05398 .67 .0468 .66124 .03301 Metetra
.664 .724 .894 r
.66124 .03301 .67 .0468 .6881 .03961 .67942 .02587 Metetra
.663 .727 .896 r
.67942 .02587 .6881 .03961 .70632 .03241 .69772 .01872 Metetra
.663 .74 .907 r
.03634 .26033 .04745 .27076 .06187 .26522 .05082 .25468 Metetra
.663 .739 .906 r
.05082 .25468 .06187 .26522 .07638 .25967 .06538 .24899 Metetra
.663 .737 .905 r
.06538 .24899 .07638 .25967 .09096 .25414 .08003 .24327 Metetra
.663 .734 .903 r
.08003 .24327 .09096 .25414 .10563 .24864 .09476 .23751 Metetra
.662 .73 .9 r
.09476 .23751 .10563 .24864 .12036 .24319 .10958 .23173 Metetra
.662 .725 .896 r
.10958 .23173 .12036 .24319 .13518 .23781 .12448 .2259 Metetra
.661 .717 .89 r
.12448 .2259 .13518 .23781 .15006 .23253 .13948 .22005 Metetra
.66 .707 .883 r
.13948 .22005 .15006 .23253 .16503 .22736 .15456 .21416 Metetra
.658 .695 .874 r
.15456 .21416 .16503 .22736 .18006 .22232 .16973 .20823 Metetra
.655 .681 .862 r
.16973 .20823 .18006 .22232 .19518 .2174 .18499 .20227 Metetra
.652 .664 .849 r
.18499 .20227 .19518 .2174 .21039 .2126 .20035 .19627 Metetra
.648 .646 .835 r
.20035 .19627 .21039 .2126 .22568 .20788 .21579 .19024 Metetra
.643 .628 .82 r
.21579 .19024 .22568 .20788 .24107 .20319 .23133 .18417 Metetra
.639 .611 .805 r
.23133 .18417 .24107 .20319 .25657 .19848 .24695 .17807 Metetra
.634 .594 .792 r
.24695 .17807 .25657 .19848 .27217 .19367 .26268 .17193 Metetra
.63 .58 .78 r
.26268 .17193 .27217 .19367 .2879 .18868 .2785 .16575 Metetra
.626 .568 .77 r
.2785 .16575 .2879 .18868 .30375 .18344 .29441 .15953 Metetra
.624 .559 .762 r
.29441 .15953 .30375 .18344 .31973 .17788 .31042 .15328 Metetra
.439 .448 .771 r
.31042 .15328 .31973 .17788 .33582 .17197 .32591 .15633 Metetra
.968 .663 .388 r
.32591 .15633 .33582 .17197 .35204 .16569 .32868 .36942 Metetra
0 0 .076 r
.32868 .36942 .35204 .16569 .36838 .15905 .35852 .14366 Metetra
.786 .719 .79 r
.35852 .14366 .36838 .15905 .38482 .15206 .37545 .12788 Metetra
.624 .561 .763 r
.37545 .12788 .38482 .15206 .40138 .14479 .39196 .12143 Metetra
.627 .57 .771 r
.39196 .12143 .40138 .14479 .41802 .13729 .40857 .11494 Metetra
.63 .581 .781 r
.40857 .11494 .41802 .13729 .43477 .12963 .42528 .10841 Metetra
.634 .595 .792 r
.42528 .10841 .43477 .12963 .4516 .12189 .4421 .10184 Metetra
.639 .611 .805 r
.4421 .10184 .4516 .12189 .46853 .11413 .45903 .09523 Metetra
.643 .627 .819 r
.45903 .09523 .46853 .11413 .48554 .10638 .47605 .08858 Metetra
.647 .644 .833 r
.47605 .08858 .48554 .10638 .50265 .0987 .49319 .08189 Metetra
.651 .66 .846 r
.49319 .08189 .50265 .0987 .51985 .09111 .51043 .07515 Metetra
.654 .675 .858 r
.51043 .07515 .51985 .09111 .53715 .08361 .52779 .06838 Metetra
.657 .689 .869 r
.52779 .06838 .53715 .08361 .55455 .0762 .54525 .06155 Metetra
.659 .701 .878 r
.54525 .06155 .55455 .0762 .57206 .06888 .56282 .05469 Metetra
.66 .711 .886 r
.56282 .05469 .57206 .06888 .58967 .06163 .58051 .04778 Metetra
.661 .719 .892 r
.58051 .04778 .58967 .06163 .60739 .05443 .59831 .04083 Metetra
.662 .725 .896 r
.59831 .04083 .60739 .05443 .62523 .04728 .61623 .03383 Metetra
.662 .73 .9 r
.61623 .03383 .62523 .04728 .64317 .04014 .63426 .02679 Metetra
.663 .734 .902 r
.63426 .02679 .64317 .04014 .66124 .03301 .6524 .0197 Metetra
.663 .736 .904 r
.6524 .0197 .66124 .03301 .67942 .02587 .67067 .01257 Metetra
.663 .738 .905 r
.67067 .01257 .67942 .02587 .69772 .01872 .68905 .00539 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(90)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(95)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(100)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(105)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(110)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakBelow->False,
  ImageSize->{167.562, 135.75},
  ImageMargins->{{33.125, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002W0000Qa000`40O003h00OYgoo002WOol00:Mo
o`00Ygoo002WOol00:Moo`00Ygoo001SOol30003Ool30004Ool2000eOol006Aoo`03001oogoo00=o
o`03001oogoo009oo`04001oogoo000dOol006Aoo`03001oogoo00=oo`03001oogoo009oo`04001o
ogoo000dOol006Aoo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000dOol006Aoo`03001o
ogoo00=oo`03001oogoo009oo`04001oogoo000dOol006=oo`8000Aoo`8000Eoo`8003Eoo`00Ygoo
002WOol00:Moo`00LGoo0P00=7oo0019Ool30004Ool20003Ool3000FOol5000dOol004Yoo`03001o
ogoo009oo`04001oogoo0005Ool00`00Oomoo`0@Ool600000eKl0000000cOol004Yoo`03001oogoo
009oo`04001oogoo0005Ool00`00Oomoo`0>Ool2000017oo000005Kl1P00<Woo001:Ool00`00Oomo
o`02Ool01000Oomoo`000Woo0`003Woo1P0000=Fo00000000UKl1000<Woo001:Ool00`00Oomoo`02
Ool01000Oomoo`000Woo00<007ooOol037oo0P0000Aoo`00001Fo0<00004E_`005KlE_`200000eKl
0000000aOol004Uoo`8000Eoo`8000=oo`@000Qoo`H00003E_`00000009Fo003001Fo00000800003
E_`005Kl00<0031oo`00GWoo1P0000IFo000000005KL0002E_`2000015Kl001Fo5Kl0P000UKl0P00
;goo001LOol30002E]`01000E]`000000UKL00<005KL00000P0000=Fo000E_`01@000UKl0P00;goo
001IOol4000015KK001Fg5KL0P0000AFg000E]aFg0800004E_`005KlE_`20002E_`00`00E_`00002
00000eKl0000000^Ool0035oo`<000Aoo`8000Aoo`8001Moo`<00003D[/005KK009Ff`800003E_`0
05KL00<00003E_`005Kl00<00004E_`005KlE_`40002E_`20006Ool20004Ool20003Ool4000IOol0
039oo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`004goo100000IB^`00D[]B^`00E]/2
0002E_`2000015Kl001Fo5Kl0P000UKl00<005Kl00000P0000=Fo00000000`0000=Fo000000017oo
00@007ooOol000=oo`8000Aoo`03001oogoo01Uoo`00<Woo00<007ooOol00Woo00@007ooOol0009o
o`04001oogoo000@Ool500001U:J001B^e:k001B^`800007E_`00000001Fo000E_`00`0000AFo000
E_aFo0800004E_`005KlE_`2000015Kl001Fo5Kl0P0017oo00@007ooOol000Yoo`03001oogoo01Qo
o`00<Woo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo000>Ool400000e:J001BVP02DYX4
00000eKl00000002E_`01000E_`000000UKl00<005Kl00000P0000=Fo000E_`00`0000=Fo000E_`0
0`0000=Fo00000000goo00@007ooOol000]oo`03001oogoo01Moo`00<Woo00<007ooOol00Woo00@0
07ooOol0009oo`04001oogoo000;Ool500001U9i001BVU:J001BVP80009Fo0800004E_`005KlE_`2
000015Kl001Fo5Kl0P0000AFo000E_aFo0800003E_`0000000<0009Fo@800003E_d00000009oo`04
001oogoo0008Ool01000Oomoo`006Goo000aOol20005Ool20004Ool2000:Ool400000e9i001BN@02
DWT400001eKl00000000Ead005Kl00<00003E_d005Kl00<00003E_d005Kl00<00003E_d00000009F
o@D00003E_d005Km008000=oo`8000Yoo`8001Yoo`00Agoo1@0000IBF000DWUBN@00DWT20002Fa`2
000015LM001G7ELM0P0000AFo@00E_eFo@80009Fo@04001Fo@000002E_d2000015Km001FoEKm0P00
00AFo@00E_eFo@800003E_d0000002Yoo`00A7oo1P0000IBF000DUQBF000DWT200001e/M00000000
Ead005LM00<00003Ead005Km00<00003E_d0000000<00007E_d00000001Fo@00E_d00`0000=Fo@00
E_d00`0000=Fo@00E_d00`00:Goo0012Ool20002Ool00`00CSP00003DUP400000e/m00000002Ead0
1000Ead000000ULM00@005LM0000009Fo@800004E_d005KmE_d2000015Km001FoEKm0P0000AFo@00
E_eFo@800004E_d005KmE_d200000eKm0000000YOol0021oo`<000=oo`<001Ioo`@0009oo`80009>
>004001BF0000002Fcd2000015LM001G7ELM0P0000AG7@00EaeG7@800007Ead00000001Fo@00E_d0
0`0000=Fo@00E_d00`0000=Fo@00E_d02@0000AFo@00001Fo@8002Qoo`008goo00<007ooOol00goo
00<007ooOol04Goo0P001Goo00D006KiI_T004hh00800007Feh00000001G7@00Ead00`0000=G7@00
Ead00`0000=G7@0000000UKm00@005Km0000009Fo@06001Fo@00001Fo@000UKm0P0000MFo@000000
05Km001Fo@0300000eKm001Fo@03000WOol0025oo`<000Eoo`03001oogoo00ioo`D000Eoo`03001V
n@0000800003Eeh00000009G7@04001G7@000002Ead40002Ead2000015Km001FoEKm0P0000AFo@00
E_eFo@800003E_d0000000<00003E_d00000009Fo@04001Fo@000002E_d01@00E_d00000E_d00P00
9goo000POol01000Oomoo`000Woo0`003Woo1@0000A>>000>MP000Aoo`<00005Eeh00000Ead00002
Ead200001eLM00000000Ead005LM00<00003Ead005Km00<00003E_d005Km00<00005E_d00000E_d0
0002E_d2000015Km001FoEKm0P0000AFo@00E_eFo@800003E_d005Km00<002Ioo`0087oo00@007oo
Ool0009oo`03001oogoo00aoo`800008Ool00000DUP004hh000if0800005Ool00000001GGP020000
0eLM001G7@0300000eLM00000002Ead01000Ead000000ULM00D005LM000005Km00D00007E_d00000
001Fo@00E_d00`0000=Fo@00E_d01@0000EFo@00000005Km00@00004E_d000000005Ool20004Ool2
0004Ool3000AOol0025oo`8000=oo`@000Qoo`H00006DUP0059HDUP004hh0P0000=j[000NZ`00`00
00=G?@0000000`0000EG7@00001G7@00009G7@800004Ead005LMEad200000eKm00000002E_d01000
E_d000000UKm00H005Km000005Km0002E_d200000eKm0000000300001eKm00000000E_d005Km0080
009Fo@03001oogoo009oo`04001oogoo0003Ool20005Ool00`00Oomoo`0@Ool0031oo`800005Ool0
0000DWT00003DUP60003NZ`01P00Ead00000Ead0009G7@800003Ead005LM00<00003Ead0000000<0
0005E_d00000E_d00002E_d2000015Km001FoEKm0P0000=Fo@0000000`0000AFo@00001Fo@800005
E_d00000E_d00002E_d2000015Km001FoEKm0P000goo00@007ooOol000Qoo`@0015oo`00;Goo1P00
00IBN@00DWUBN@00DUP20002Dch00`00Dch00003NZ`200000eLM001G7@0300000eLM001G7@020002
Ead2000015LM001FoEKm0P0000=Fo@00E_d00`0000=Fo@00E_d00`0000EFo@00001Fo@00009Fo@80
0003E_d005Km00<00003E_d005Km00<00003E_d005Km00<00003E_d00000009oo`04001oogoo0008
Ool00`00Ool0000BOol002Yoo`D00004DYX005:J0002DWT01@00DWT00000Dah01P000gZ/0P0000=G
7@0000000ULM100000MG7@00000005LM001G7@0300000eKm001Fo@020002E_d400001eKm00000000
E_d005Km00<00003E_d005Km0080009Fo@05001Fo@00001Fo@0500001UKm00000000E_d0009oo`04
001oogoo0008Ool00`00Ool0000BOol000]oo`<000Aoo`8001Aoo`800005Ool00000D[/00002DYX6
00000e<M00000002Dad01000Ead000000gZ/0`0000MG7@00000005LM001G7@020002Ead01000Ead0
00000UKm100000MFo@00000005Km001Fo@020002E_d01P00E_d00000E_d0009Fo@800003E_d00000
00<00005E_d00000E_d00002E_d2000015Km001FoEKm0P000Woo0P002Woo0P004Woo000>Ool01000
Oomoo`000Woo00<007ooOol03Woo1P0000=B^`00D[/01@0000=Bo@00D_d00P000U<M00H005<M0000
05LM0002NZ`200000eLM00000002Ead01@00Ead00000Ead01@0000MFo@00000005Km001Fo@020002
E_d01P00E_d00000E_d0009Fo@800003E_d0000000<00006E_d00000E_d005Km0`0000=Fo@00E_d0
0`0000=Fo@00E_d00`0000=Fo@0000008Goo000<Ool30002Ool01000Oomoo`003Woo0P0000=oo`00
00000UKK00@005:k0000009Bg003001Bg00000<00003Dad0000000<00004Ead000000002NZ`30002
Ead01@00Ead00000Ead00P000UKm00@005Km0000009Fo@06001Fo@00001Fo@000UKm0P0000=Fo@00
00000`0000EFo@00001Fo@00009Fo@800003E_d005Km00<00003E_d005Km00800003E_d0000000<0
0006E_d00000001Fo@008Goo000;Ool01000Oomoo`000Woo00@007ooOol000]oo`L00003E]`005KK
00800004D]`005;LD]`200000e;m001Bo@020002Dad01000Ead000000ULM00<007Z/NZ`01`0000=G
7@0000000UKm00H005Km000005Km0002E_d200000eKm0000000300001UKm000005Km001Fo@<00003
E_d005Km00<00003E_d005Km00800003E_d0000000<00005E_d00000E_d00002E_d2000015Km001F
oEKm0P0087oo000;Ool01000Oomoo`000Woo00@007ooOol000Qoo`D0009Fo003001Fg00000<00003
D]`005;L00<00003D_d005;m00<00003D_d005<M00800003Ead000000080009j[080009G7@06001G
7@00001Fo@000UKm0P0000=Fo@0000000`0000IFo@00001Fo@00E_d300000eKm001Fo@020002E_d0
1@00E_d00000E_d01@0000EFo@00001Fo@00009Fo@800003E_d005Km00<00003E_d005Km00<00003
E_d0000001moo`0037oo0P0017oo0P001goo0P0000Moo`00000005Kl001Fo00200000e;L001Bg005
00000e;m000000020002D_d400000eLM00000002Ead20002NZ`20002Ead200000eKm000000030000
1EKm000005Km00000UKm0P0000=Fo@00E_d00P000UKm00D005Km000005Km00D00005E_d00000E_d0
0002E_d200000eKm001Fo@0300000eKm001Fo@0200000eKm0000000300001UKm00000000E_d001mo
o`0067oo1P000UKl00L005Kl000005;L001Bg0020002D_`01`00D_d00000D_d005;m00<00003E_d0
05Km00<00003E_d005LM0080009j[0@00005E_d00000E_d00002E_d200000eKm001Fo@0300000eKm
001Fo@0200000eKm0000000300000eKm00000002E_d01`00E_d00000E_d005Km00<00003E_d005Km
00P00003E_d00000009Fo@06001Fo@00001Fo@000UKm0P007Woo000GOol500000eKl000000030000
0e;L0000000300000e;l001Bo00200000e;m001Bo@0300000e;m000000020002E_d4000015Km001j
[7Z/0P0000IFo@00001Fo@00E_d300000eKm000000020002E_d400000eKm00000002E_d01P00E_d0
0000E_d0009Fo@800003E_d000000080009Fo@L0009Fo@06001Fo@00001Fo@000UKm0P0000=Fo@00
E_d01@001Woo0P0017oo0P0017oo0P002Woo000GOol300000eKl00000002E]`01000D]`000000U;l
00<005;l00000`0000=Bo@0000000`0000IBo@00001Fo@00E_d300000eKm001Fo@030002NZ`30000
0eKm0000000300001UKm000005Km001Fo@<00003E_d005Km00800004E_d005KmE_d200000eKm0000
000300001UKm000005Km001Fo@D00004E_d00000E_d2000015Km000005Km1@0000=Fo@0000000P00
00AFo@00001Fo@8000Aoo`04001oogoo0003Ool20003Ool01000Oomoo`002Goo000AOol20004Ool0
0`00E_d0000200000eKl001Fg002000015;l001Bo5;l0P0000=Bo@00D_d00P000U;m00L005;m0000
05Km001Fo@0300000eKm001Fo@0200001eKm00000000NZ`005Km00800004E_d005KmE_d200000eKm
001Fo@020002E_d01@00E_d00000E_d01@0000IFo@00001Fo@00E_d300000eKm001Fo@040002E_d2
00000eKm0000000300001UKm000005Km001Fo@<00003E_d005Km00800003E_d005Km00<000=oo`04
001oogoo0008Ool01000Oomoo`002Goo000@Ool01000Oomoo`000goo0`0000=Fo00000000P0000=B
o00000000`0000=Bo000D_d00P000U;m00D005;m000005Km00D00003E_d0000000800003E_d00000
009Fo@@00003E_d005Km00<00003E_d0000000800003E_d0000000<00005E_d00000E_d00002E_d2
00000eKm001Fo@020002E_d:00001UKm000005Km001Fo@<00003E_d005Km00800004E_d005KmE_d2
00000eKm0000000200000eKm00000003Ool01000Oomoo`0027oo0`002Woo000@Ool01000Oomoo`00
0Woo0P000Woo00@005;l001Bo080009Bo003001Bo00000<00003D_d0000000<00005E_d00000E_d0
0002E_d200000eKm001Fo@020002E_d00`00E_d0000700001UKm000005Km001Fo@80009Fo@07001F
o@00001Fo@00E_d00`0000=Fo@0000000P0000=Fo@0000000`0000IFo@00001Fo@00E_d300000eKm
001Fo@0200000eKm0000000300000eKm0000000600000eKm001Fo@0200000eKm00000002Ool01000
Oomoo`0027oo00<007ooOol02Woo000@Ool01000Oomoo`000Woo0P000Woo00<005;l00000P0000=B
o000D_`00P0000=Bo@00D_d00`0000=Bo@00E_d00P0000=Fo@00E_d00`0000=Fo@0000000P0000=F
o@0000001P0000AFo@00E_eFo@800003E_d005Km0080009Fo@05001Fo@00001Fo@040002E_d20000
0eKm001Fo@040002E_d200000eKm001Fo@020002E_d4000015Km000005Km0P0000IFo@00001Fo@00
E_d2000015Km001FoEKm0P0000=Fo@0000000`000Woo0P002Woo0`002Goo000@Ool01000Oomoo`00
0Woo00<007ooOol00Woo0P0000=Bo00000000P0000=Bo@0000000`0000=Bo@0000000P0000=Fo@00
00000`0000IFo@00001Fo@00E_d300000eKm001Fo@0400001EKm00000000E_d0100000=Fo@000000
0`0000IFo@00001Fo@00E_d300000eKm001Fo@0800000eKm0000000200001eKm00000000E_d005Km
00800003E_d005Km00<00003E_d005Km00800003E_d0000000<00006E_d00000E_d005Km0P006Woo
000AOol20003Ool00`00Oomoo`04Ool01000D_`005;l0P000U;m00L005;m000005Km001Fo@030000
0eKm001Fo@0200000eKm001Fo@0300000eKm0000000200000eKm0000000300001UKm000005Km001F
o@80009Fo@07001Fo@00001Fo@00E_d00P000UKm100000EFo@00001Fo@00009Fo@800003E_d005Km
00800004E_d005KmE_d200000eKm0000000200000eKm0000000300001UKm000005Km001Fo@<00003
E_d005Km00800003E_d0000001Uoo`005Woo0P001Goo00<005;l00000P0000=Bo@00D_d00P0000=F
o@00E_d00`0000=Fo@0000000P0000=Fo@0000000`0000IFo@00001Fo@00E_d500001UKm000005Km
001Fo@800003E_d0000000<00003E_d0000000800003E_d0000000<00006E_d00000E_d005Km0`00
00=Fo@0000000P0000=Fo@0000000`0000IFo@00001Fo@00E_d2000015Km000005Km0P0000=Fo@00
E_d00P0000=Fo@0000000`0000AFo@00001Fo@8001Qoo`0017oo0P0017oo0P000goo0`0017oo00<0
07ooOol01Goo0P0000=Bo@0000000P0000=Fo@0000000`0000IFo@00001Fo@00E_d300000eKm001F
o@0200000eKm001Fo@0200000eKm0000000300000eKm0000000200001eKm00000000E_d005Km0080
0003E_d005Km00P00003E_d000000080009Fo@800003E_d005Km00800003E_d005Km00<00003E_d0
05Km00800003E_d0000000<00003E_d0000000800005E_d00000E_d00002E_d200000eKm001Fo@03
000GOol000=oo`04001oogoo0003Ool20006Ool00`00Oomoo`020009Ool01000E_d005Km0`0000=F
o@00E_d00P0000=Fo@00E_d00`0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000000`0000IF
o@00001Fo@00E_d300000eKm0000000200000eKm0000000600000eKm001Fo@0200000eKm001Fo@03
00000eKm001Fo@0200000eKm000000020002E_d400001UKm000005Km001Fo@<00003E_d005Km0080
0003E_d0000000P00003E_d0000001Moo`000goo00@007ooOol000]oo`03001oogoo008000Yoo`@0
0003E_d0000000800003E_d000000080009Fo@800003E_d005Km00800003E_d005Km00<00003E_d0
000000800003E_d000000080009Fo@800003E_d005Km00@00007E_d00000001Fo@00E_d00P0000=F
o@0000000P0000=Fo@0000000`0000IFo@00001Fo@00E_d300000eKm001Fo@0200000eKm00000002
00000eKm0000000300001UKm000005Km001Fo@800004E_d005KmE_d200000eKm00000005Ool20004
Ool20004Ool20003Ool000=oo`04001oogoo0008Ool30003Ool00`00Oomoo`0:Ool01`00E_d00000
E_d005Km00@00005E_d00000001Fo@0400000eKm00000003000015Km000005Km100000=Fo@00E_d0
0`0000=Fo@00E_d01`000UKm100000IFo@00001Fo@00E_d20002E_d01@00E_d00000E_d0100000=F
o@0000000`0000IFo@00001Fo@00E_d2000015Km000005Km0P0000=Fo@00E_d00P0000=Fo@000000
0`0000=Fo@0000000`000goo00@007ooOol000=oo`8000=oo`04001oogoo0002Ool000=oo`04001o
ogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`0;Ool00`00E_d0000400000eKm0000000200001UKm
000005Km001Fo@<00003E_d005Km00800003E_d005Km00800003E_d0000000<00003E_d000000080
0006E_d00000E_d005Km0`0000=Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000`0000IFo@00
001Fo@00E_d2000015Km001FoEKm0P0000=Fo@00E_d00P0000=Fo@0000000`0000=Fo@0000000P00
00IFo@00001Fo@00E_d300000eKm001Fo@020003Ool01000Oomoo`0027oo00@007ooOol0009oo`00
17oo0P002Goo10000Woo00<007ooOol037oo0P0000=Fo@00E_d00P0000=Fo@00E_d00P0000=Fo@00
00000`0000=Fo@0000000P0000=Fo@0000001P0000=Fo@00E_d0100000AFo@00001Fo@D00003E_d0
000000800006E_d00000E_d005Km0P000UKm00L005Km000005Km001Fo@0200000eKm000000030000
0eKm0000000200001UKm000005Km001Fo@<00003E_d005Km00800003E_d0000000800003E_d00000
00<00003E_d00000009oo`04001oogoo0009Ool20003Ool001Eoo`03001oogoo00ioo`<00003E_d0
000000800007E_d00000001Fo@00E_d00P0000=Fo@00E_d00P0000=Fo@00E_d00P000UKm100000=F
o@0000000P0000EFo@00001Fo@00009Fo@800003E_d005Km00800003E_d0000000800003E_d00000
00<00003E_d0000000800006E_d00000E_d005Km0`0000=Fo@00E_d00P0000=Fo@0000000P0000=F
o@0000000`0000IFo@00001Fo@00E_d200000eKm001Fo@0300001EKm001oogoo00000Woo00<007oo
Ool01Woo00@007ooOol0009oo`0057oo0P0047oo00L005Km000005Km001Fo@0300000eKm00000002
00000eKm0000000200000eKm0000000200000eKm0000000300001UKm000005Km001Fo@800003E_d0
000000<00003E_d0000000800006E_d00000E_d005Km0P000UKm00L005Km000005Km001Fo@020000
0eKm0000000200000eKm0000000300001UKm000005Km001Fo@800003E_d0000000<00003E_d00000
00800003E_d0000000800003E_d0000000<0009oo`8000Yoo`8000=oo`003Woo0`000goo00<007oo
Ool047oo00<005Km0000100000IFo@00001Fo@00E_d200000eKm001Fo@0200000eKm001Fo@080000
0eKm0000000200001eKm00000000E_d005Km00800003E_d005Km00800003E_d0000000800003E_d0
000000<00003E_d0000000800006E_d00000E_d005Km0P0000AFo@00E_eFo@800003E_d000000080
0003E_d0000000H00003E_d005Km00800003E_d005Km00800003E_d005Km00<0019oo`003goo00<0
07ooOol00Woo0P004Woo0P0000=Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@00
00000`0000IFo@00001Fo@00E_d200000eKm001Fo@020002E_d400000eKm0000000200001UKm0000
05Km001Fo@80009Fo@07001Fo@00001Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000`0000IF
o@00001Fo@00E_d200000eKm001Fo@0200000eKm0000000300000eKm0000000200000eKm00000002
00000eKm0000000BOol000moo`03001oogoo009oo`03001oogoo01=oo`<00006E_d00000E_d005Km
0`0000=Fo@00E_d01`0000=Fo@0000000P0000=Fo@0000000P0000MFo@00000005Km001Fo@020000
0eKm001Fo@0200000eKm0000000200000eKm0000000300000eKm0000000200001UKm000005Km001F
o@800003E_d005Km00<00003E_d0000000800003E_d0000000800006E_d00000E_d005Km0`0000=F
o@00E_d00P0000=Fo@0000000P0000=Fo@0000004Goo000?Ool00`00Oomoo`02Ool00`00Oomoo`0D
Ool300000eKm0000000300000eKm0000000200001UKm000005Km001Fo@800003E_d005Km00800003
E_d005Km00<00003E_d0000000800003E_d0000000800006E_d00000E_d005Km0P000UKm00D005Km
000005Km00@00003E_d0000000800003E_d0000000800007E_d00000001Fo@00E_d00P0000=Fo@00
E_d00P0000=Fo@0000000P0000=Fo@0000001P0000=Fo@00E_d00P0000=Fo@00000047oo000?Ool0
0`00Oomoo`02Ool00`00Oomoo`0DOol00`00E_d0000200000eKm001Fo@04000015Km000005Km1000
00=Fo@0000000P0000=Fo@0000000`0000IFo@00001Fo@00E_d200000eKm001Fo@0200000eKm0000
000200000eKm0000000300001UKm000005Km001Fo@800003E_d005Km00800003E_d0000000800003
E_d0000000<00003E_d0000000800006E_d00000E_d005Km0P0000=Fo@00E_d00P0000=Fo@000000
0`0000=Fo@0000000P001Goo0`0027oo000>Ool20004Ool00`00Oomoo`0EOol200000eKm00000002
00000eKm0000000200001UKm000005Km001Fo@800003E_d005Km00<00003E_d0000000800003E_d0
000000800003E_d0000000800006E_d00000E_d005Km0P000UKm100000=Fo@0000000P0000=Fo@00
00000P0000IFo@00001Fo@00E_d2000015Km001FoEKm0P0000=Fo@0000000P0000=Fo@0000000P00
00=Fo@0000000P0000IFo@00001Fo@00E_d300000eKm001Fo@020005Ool00`00Oomoo`07Ool001=o
o`8001Qoo`@00004E_d00000E_d400000eKm0000000300000eKm0000000200001UKm000005Km001F
o@800003E_d005Km00800003E_d0000000800003E_d0000000800007E_d00000001Fo@00E_d00P00
00=Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000001P0000=Fo@00E_d00P00
00=Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000001@001Goo00<007ooOol0
1goo000COol2000EOol30002Ool300000eKm0000000200001eKm00000000E_d005Km00800003E_d0
000000800003E_d0000000800003E_d0000000800006E_d00000E_d005Km0P0000=Fo@0000000`00
00=Fo@0000000P0000=Fo@0000000P0000IFo@00001Fo@00E_d200000eKm001Fo@0200000eKm0000
000200000eKm0000000300000eKm0000000200001UKm000005Km001Fo@800003E_d005Km00800003
E_d0000000800003E_d0000000Aoo`03001oogoo00Moo`0000Aoo`00000000Aoo`8000=oo`<000=o
o`03001oogoo019oo`8000Ioo`@00003E_d0000000800003E_d0000000800006E_d00000E_d005Km
0P0000=Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000P0000MFo@00000005Km001Fo@020000
0eKm001Fo@0200000eKm0000000200000eKm0000000200000eKm0000000200001UKm000005Km001F
o@800003E_d005Km00P00003E_d0000000800003E_d0000000800006E_d00000E_d005Km0P0000=F
o@0000000goo00<007ooOol01goo0002Ool00`00Oomoo`03Ool20006Ool01000Oomoo`004Goo0`00
2Goo00D005Km000005Km00@00003E_d0000000800003E_d0000000800003E_d0000000800006E_d0
0000E_d005Km0P0000=Fo@0000000`0000=Fo@0000000P0000=Fo@0000000P0000IFo@00001Fo@00
E_d200000eKm001Fo@0200000eKm0000000200000eKm0000000200000eKm0000000200001UKm0000
05Km001Fo@<00003E_d005Km00800003E_d0000000800003E_d0000000800003E_d000000080009o
o`8000Uoo`000Woo00<007ooOol02goo00@007ooOol000ioo`<000eoo`07001Fo@00001Fo@00E_d0
0P0000=Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000000P0000MFo@000000
05Km001Fo@0200000eKm001Fo@0200000eKm0000000200000eKm0000000200000eKm000000020000
1UKm000005Km001Fo@800003E_d005Km00800003E_d0000000800003E_d0000000800003E_d00000
00H00003E_d005Km00800003E_d00000008000=oo`03001oogoo00]oo`000Woo00<007ooOol027oo
0`000goo00<007ooOol02Goo0`004Goo00<005Km00000`0000=Fo@0000000P0000IFo@00001Fo@00
E_d400001EKm00000000E_d0100000=Fo@0000000P0000=Fo@0000000P0000IFo@00001Fo@00E_d2
00000eKm0000000200000eKm0000000200000eKm0000000200000eKm0000000200001UKm000005Km
001Fo@800003E_d0000000800003E_d0000000/000Moo`03001oogoo00]oo`000Woo00<007ooOol0
27oo00<007ooOol00Woo0P0027oo0`005Goo0P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@00
00000P0000=Fo@0000000P0000IFo@00001Fo@00E_d200000eKm0000000200000eKm000000020000
0eKm0000000200001UKm000005Km001Fo@800003E_d005Km00800003E_d0000000800003E_d00000
00800003E_d0000000800006E_d00000E_d005Km0P0000=Fo@00E_d00P0000=Fo@0000000P002Woo
00<007ooOol02goo00000goo0000000:Ool40002Ool00`00Oomoo`03Ool3000JOol01000E_d005Km
0P0000=Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000000P0000IFo@00001F
o@00E_d200000eKm001Fo@0200000eKm0000000200000eKm0000000200000eKm0000000200001UKm
000005Km001Fo@800003E_d0000000800003E_d0000000800003E_d0000000800003E_d000000080
0003E_d0000000moo`03001oogoo00Yoo`004Woo0P000Woo0`007Woo0`0000=Fo@0000000P0000IF
o@00001Fo@00E_d200000eKm0000000200000eKm0000000200000eKm0000000200000eKm00000002
00001UKm000005Km001Fo@800003E_d0000000800003E_d0000000800003E_d0000000800006E_d0
0000E_d005Km0P0000=Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000P000Woo00<007ooOol0
37oo00<007ooOol02Woo000BOol4000ROol300000eKm0000000200000eKm0000000200001UKm0000
05Km001Fo@800003E_d0000000800003E_d0000000800003E_d0000000800003E_d0000000800006
E_d00000E_d005Km0P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@000000
0P0000IFo@00001Fo@00E_d20006Ool00`00Oomoo`0<Ool00`00Oomoo`0:Ool000]oo`@000=oo`80
02Aoo`05001Fo@00001Fo@0400000eKm0000000200000eKm0000000200001UKm000005Km001Fo@80
0003E_d005Km00800003E_d0000000800003E_d0000000800003E_d0000000800006E_d00000E_d0
05Km0P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000000P002Goo00<0
07ooOol02goo00<007ooOol02Woo000<Ool00`00Oomoo`05Ool00`00Oomoo`0ROol01`00E_d00000
E_d005Km00800003E_d0000000800003E_d0000000800003E_d0000000800003E_d0000000800006
E_d00000E_d005Km0P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000000P0000IFo@00001F
o@00E_d200000eKm001Fo@0200000eKm00000002000>Ool00`00Oomoo`0:Ool00`00Oomoo`0:Ool0
00eoo`03001oogoo00Eoo`03001oogoo029oo`800003E_d0000000800005E_d00000001Fo@020000
0eKm0000000200000eKm0000000200000eKm0000000200000eKm0000000200001UKm000005Km001F
o@800003E_d0000000800003E_d0000000800003E_d0000000800003E_d0000000800003E_d00000
01=oo`03001oogoo00Uoo`03001oogoo00Yoo`003Woo00<007ooOol01Goo00<007ooOol08goo00<0
05Km00000`0000=Fo@0000000P0000EFo@00000005Km00800003E_d0000000800003E_d000000080
0003E_d0000000800003E_d0000000800006E_d00000E_d005Km0P0000=Fo@0000000P0000=Fo@00
00000P0000=Fo@0000000P005Goo00<007ooOol02Goo00<007ooOol02Woo000;Ool01000Oomoo`00
27oo0P0097oo00<005Km00000`0000=Fo@0000000P0000=Fo@0000000P0000EFo@00000005Km0080
0003E_d0000000800003E_d0000000800003E_d0000000800003E_d0000000800006E_d00000E_d0
05Km0P0000=Fo@0000000P006Woo00<007ooOol027oo00<007ooOol02Woo000<Ool2000;Ool00`00
Oomoo`0ROol00`00E_d0000300000eKm0000000200000eKm0000000200000eKm0000000200001EKm
001Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000000P00
00=Fo@0000007goo00<007ooOol027oo00<007ooOol02Goo000JOol00`00Oomoo`0ROol00`00E_d0
000300000eKm0000000200000eKm0000000200000eKm0000000200001EKm00000000E_d00P0000=F
o@0000000P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000000P008Goo00<007ooOol027oo
00<007ooOol02Goo000KOol00`00Oomoo`0ROol01000E_d005Km0P0000=Fo@0000000P0000=Fo@00
00000P0000=Fo@0000000P0000EFo@00000005Km00800003E_d005Km00800003E_d0000000800003
E_d00000008002Ioo`03001oogoo00Moo`03001oogoo00Uoo`0077oo00<007ooOol08Woo0`0000IF
o@00001Fo@00E_d200000eKm0000000200000eKm0000000200001EKm00000000E_d0100000IFo@00
001Fo@00E_d2000[Ool00`00Oomoo`06Ool00`00Oomoo`09Ool001eoo`8002Aoo`<00005E_d00000
001Fo@0200000eKm0000000200000eKm0000000200001EKm00000000E_d0100000=Fo@0000000P00
;Woo00<007ooOol01Goo00<007ooOol02Goo000OOol00`00Oomoo`0ROol300001EKm00000000E_d0
0P0000=Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000P0000AFo@000000039oo`03001oogoo
00Eoo`03001oogoo00Uoo`0087oo00<007ooOol08Woo0`0000EFo@00000005Km00@00004E_d00000
E_d400000eKm00000002000fOol00`00Oomoo`04Ool00`00Oomoo`09Ool0025oo`03001oogoo029o
o`<00003E_d0000000800005E_d00000001Fo@0200000eKm00000002000kOol00`00Oomoo`04Ool0
0`00Oomoo`08Ool0029oo`8002=oo`<00003E_d0000000800005E_d00000001Fo@02000oOol00`00
Oomoo`04Ool00`00Oomoo`08Ool002Aoo`03001oogoo025oo`<00003E_d0000000800004E_d00000
0012Ool00`00Oomoo`03Ool00`00Oomoo`08Ool002Eoo`03001oogoo025oo`<00003E_d000000080
04Ioo`03001oogoo009oo`03001oogoo00Qoo`009Woo00<007ooOol08Goo0`00Bgoo00D007ooOomo
o`0000Yoo`009goo00<007ooOol087oo00<007ooOol0Bgoo00D007ooOomoo`0000Yoo`00:7oo0P00
87oo00<007ooOol0C7oo00@007ooOol000Yoo`00:Woo00<007ooOol07Goo00<007ooOol0CGoo00<0
07oo00002Woo000[Ool00`00Oomoo`0LOol00`00Oomoo`1=Ool01000Oomoo`002Goo000/Ool00`00
Oomoo`0KOol00`00Oomoo`1>Ool00`00Ool00009Ool002eoo`8001]oo`03001oogoo04moo`8000Uo
o`00;goo00<007ooOol067oo00<007ooOol0CWoo0`002Goo000`Ool00`00Oomoo`0GOol00`00Oomo
o`1:Ool4000<Ool0035oo`03001oogoo01Ioo`03001oogoo04Ioo`@0011oo`00<Woo00<007ooOol0
5Goo00<007ooOol0@goo0`0057oo000cOol2000EOol00`00Oomoo`0oOol4000GOol003Eoo`03001o
ogoo019oo`03001oogoo03]oo`@001]oo`00=Woo00<007ooOol04Goo00<007ooOol0=goo10007goo
000gOol00`00Oomoo`0@Ool00`00Oomoo`0cOol4000SOol003Qoo`8000moo`03001oogoo035oo`<0
02Moo`00>Woo00<007ooOol037oo00<007ooOol0;Goo1000:Woo000kOol00`00Oomoo`0;Ool00`00
Oomoo`0YOol4000^Ool003aoo`03001oogoo00Yoo`03001oogoo02Eoo`@0039oo`00?Goo00<007oo
Ool02Goo00<007ooOol08Goo1000=Woo000nOol20009Ool00`00Oomoo`0MOol4000jOol0041oo`03
001oogoo00Ioo`03001oogoo01Yoo`<003ioo`00@Goo00<007ooOol01Goo00<007ooOol05Woo1000
@Goo0012Ool00`00Oomoo`04Ool00`00Oomoo`0BOol40015Ool004=oo`8000Aoo`03001oogoo00io
o`@004Uoo`00AGoo00D007ooOomoo`0000aoo`@004eoo`00AWoo00@007ooOol000Uoo`<0055oo`00
Agoo00<007oo00001Goo1000E7oo0018Ool200000goo00000002001HOol004Uoo`8005aoo`00Ygoo
002WOol00:Moo`00Ygoo002WOol00001\
\>"],
  ImageRangeCache->{{{0, 166.562}, {134.75, 0}} -> {-0.125098, -0.0454799, 
  0.00683588, 0.00683588}}]
}, Open  ]],

Cell["Let's take a closer look at the function:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakBelow->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(ExpandAll[\[CapitalDelta]]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakBelow->False],

Cell[BoxData[
    \(\(B\ E\^\(\(-\(\(q\ t\)\/2\)\) - \(r\ t\)\/2 - \(q\^2\ t\)\/\(2\ \
\[Sigma]\^2\) + \(q\ r\ t\)\/\[Sigma]\^2 - \(r\^2\ t\)\/\(2\ \[Sigma]\^2\) - \
\(t\ \[Sigma]\^2\)\/8 - Log[S\/K]\^2\/\(2\ t\ \[Sigma]\^2\)\)\ \
\((S\/K)\)\^\(1\/2 + q\/\[Sigma]\^2 - r\/\[Sigma]\^2\)\)\/\(\@\(2\ \[Pi]\)\ S\
\ \@t\ \[Sigma]\)\)], "Output"]
}, Open  ]],

Cell["\<\
As we approach maturity, this function becomes increasingly \
localized at the strike, where it becomes infinite. Elsewhere, it becomes \
zero. Essentially it has become what mathematicians term a \"Delta \
Function.\" In fact, the Delta for these options is essentially just the \
Green's function we wrote down in Chapter 4. This makes the hedging of Binary \
options that are near the money well-nigh impossible as expiry approaches. If \
the asset price wanders around the strike, near maturity, huge positions in \
the underlying can be defined as the hedge position, then bought and sold \
repeatedly. Coupled with transaction costs, this can wreak havoc with risk \
management. Gamma is similarly pathological. Just as Delta approaches a Delta \
Function, Gamma approaches the derivative of a Delta Function at \
maturity:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakBelow->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot3D[
      BinaryCallGamma[1, \ S, \ 100, \ 0.2, \ 0.1, \ 0, \ t], \ {S, \ 90, \ 
        110}, \ {t, \ 0.0001, \ 1}, \ PlotPoints\  -> \ 41, \ 
      PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakBelow->False],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -16.7601 -9 ]
[.3345 .12542 1.2399 0 ]
[.50253 .05935 -15.8219 -9 ]
[.50253 .05935 2.17811 0 ]
[.68156 -0.01098 -14.8837 -9 ]
[.68156 -0.01098 3.11632 0 ]
[.76742 .09699 0 -6.16191 ]
[.76742 .09699 18 2.83809 ]
[.8281 .1899 0 -6.07249 ]
[.8281 .1899 18 2.92751 ]
[.88368 .27501 0 -5.9922 ]
[.88368 .27501 18 3.0078 ]
[.93477 .35326 0 -5.91971 ]
[.93477 .35326 18 3.08029 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.02274 .27372 -12 -2.74959 ]
[.02274 .27372 0 6.25041 ]
[.01501 .32216 -24 -2.79056 ]
[.01501 .32216 0 6.20944 ]
[.00704 .37214 -6 -2.83297 ]
[.00704 .37214 0 6.16703 ]
[-0.0012 .42374 -18 -2.87689 ]
[-0.0012 .42374 0 6.12311 ]
[-0.0097 .47704 -6 -2.92242 ]
[-0.0097 .47704 0 6.07758 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(90)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(95)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(100)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(105)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(110)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.75512 .10154 m
.74896 .10381 L
s
[(0.2)] .76742 .09699 -1 .36931 Mshowa
.81571 .19423 m
.80952 .19639 L
s
[(0.4)] .8281 .1899 -1 .34944 Mshowa
.87122 .27914 m
.86499 .2812 L
s
[(0.6)] .88368 .27501 -1 .3316 Mshowa
.92226 .35721 m
.916 .35918 L
s
[(0.8)] .93477 .35326 -1 .31549 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(1)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.7059 .02625 m
.70222 .02766 L
s
.72269 .05193 m
.71901 .05333 L
s
.73909 .07702 m
.7354 .0784 L
s
.77078 .1255 m
.76708 .12684 L
s
.78609 .14892 m
.78239 .15025 L
s
.80107 .17182 m
.79735 .17314 L
s
.83004 .21614 m
.82632 .21743 L
s
.84406 .23759 m
.84033 .23886 L
s
.85779 .25859 m
.85405 .25984 L
s
.88438 .29926 m
.88064 .30049 L
s
.89727 .31898 m
.89352 .32019 L
s
.90989 .33829 m
.90614 .33949 L
s
.93438 .37576 m
.93063 .37693 L
s
.94627 .39394 m
.94251 .39509 L
s
.95792 .41176 m
.95415 .41291 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03497 .26897 m
.04108 .26659 L
s
[(-1)] .02274 .27372 1 -0.38898 Mshowa
.02728 .3175 m
.03341 .31517 L
s
[(-0.5)] .01501 .32216 1 -0.37988 Mshowa
.01934 .36758 m
.02549 .3653 L
s
[(0)] .00704 .37214 1 -0.37045 Mshowa
.01115 .41929 m
.01732 .41706 L
s
[(0.5)] -0.0012 .42374 1 -0.36069 Mshowa
.00268 .47269 m
.00888 .47052 L
s
[(1)] -0.0097 .47704 1 -0.35057 Mshowa
.125 Mabswid
.03345 .27855 m
.03712 .27713 L
s
.03192 .2882 m
.03559 .28678 L
s
.03038 .29791 m
.03406 .2965 L
s
.02883 .30767 m
.03251 .30627 L
s
.02571 .32739 m
.02939 .326 L
s
.02413 .33734 m
.02782 .33596 L
s
.02255 .34736 m
.02623 .34598 L
s
.02095 .35744 m
.02464 .35606 L
s
.01772 .37779 m
.02142 .37643 L
s
.0161 .38806 m
.01979 .38671 L
s
.01446 .3984 m
.01815 .39706 L
s
.01281 .40881 m
.01651 .40747 L
s
.00948 .42983 m
.01318 .4285 L
s
.00779 .44044 m
.0115 .43912 L
s
.0061 .45112 m
.00981 .44981 L
s
.0044 .46187 m
.00811 .46056 L
s
.03648 .25944 m
.04014 .25801 L
s
.00096 .48359 m
.00467 .48229 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.656 .751 .92 r
.39384 .69452 .40171 .70139 .41488 .69768 .40705 .69077 Metetra
.656 .751 .92 r
.40705 .69077 .41488 .69768 .42811 .69394 .42032 .687 Metetra
.656 .751 .92 r
.42032 .687 .42811 .69394 .44141 .69019 .43366 .68322 Metetra
.656 .751 .92 r
.43366 .68322 .44141 .69019 .45477 .68642 .44707 .67941 Metetra
.656 .751 .92 r
.44707 .67941 .45477 .68642 .4682 .68263 .46054 .67558 Metetra
.656 .751 .92 r
.46054 .67558 .4682 .68263 .4817 .67882 .47408 .67174 Metetra
.656 .751 .92 r
.47408 .67174 .4817 .67882 .49527 .67499 .48769 .66788 Metetra
.656 .751 .92 r
.48769 .66788 .49527 .67499 .5089 .67114 .50136 .664 Metetra
.656 .751 .92 r
.50136 .664 .5089 .67114 .5226 .66728 .51511 .66009 Metetra
.656 .751 .92 r
.51511 .66009 .5226 .66728 .53637 .66339 .52892 .65617 Metetra
.656 .751 .92 r
.52892 .65617 .53637 .66339 .5502 .65949 .54281 .65223 Metetra
.656 .751 .92 r
.54281 .65223 .5502 .65949 .56411 .65556 .55676 .64827 Metetra
.656 .751 .92 r
.55676 .64827 .56411 .65556 .57809 .65162 .57078 .64429 Metetra
.656 .751 .92 r
.57078 .64429 .57809 .65162 .59214 .64766 .58488 .64029 Metetra
.656 .751 .92 r
.58488 .64029 .59214 .64766 .60626 .64367 .59905 .63627 Metetra
.656 .751 .92 r
.59905 .63627 .60626 .64367 .62045 .63967 .61329 .63223 Metetra
.656 .751 .92 r
.61329 .63223 .62045 .63967 .63471 .63564 .6276 .62816 Metetra
.656 .751 .92 r
.6276 .62816 .63471 .63564 .64905 .6316 .64199 .62408 Metetra
.656 .751 .92 r
.64199 .62408 .64905 .6316 .66346 .62753 .65645 .61998 Metetra
.656 .751 .92 r
.65645 .61998 .66346 .62753 .67794 .62345 .67098 .61585 Metetra
.656 .751 .92 r
.67098 .61585 .67794 .62345 .6925 .61934 .68559 .61171 Metetra
.656 .751 .92 r
.68559 .61171 .6925 .61934 .70713 .61521 .70028 .60754 Metetra
.656 .751 .92 r
.70028 .60754 .70713 .61521 .72184 .61106 .71504 .60335 Metetra
.656 .751 .92 r
.71504 .60335 .72184 .61106 .73663 .60689 .72988 .59914 Metetra
.656 .751 .92 r
.72988 .59914 .73663 .60689 .75149 .6027 .74479 .5949 Metetra
.656 .751 .92 r
.74479 .5949 .75149 .6027 .76643 .59849 .75979 .59065 Metetra
.656 .751 .92 r
.75979 .59065 .76643 .59849 .78145 .59425 .77486 .58637 Metetra
.656 .751 .92 r
.77486 .58637 .78145 .59425 .79655 .58999 .79002 .58207 Metetra
.656 .751 .92 r
.79002 .58207 .79655 .58999 .81172 .58571 .80525 .57775 Metetra
.656 .751 .92 r
.80525 .57775 .81172 .58571 .82698 .58141 .82056 .5734 Metetra
.656 .751 .92 r
.82056 .5734 .82698 .58141 .84232 .57708 .83596 .56904 Metetra
.656 .751 .92 r
.83596 .56904 .84232 .57708 .85773 .57274 .85144 .56465 Metetra
.656 .751 .92 r
.85144 .56465 .85773 .57274 .87324 .56836 .867 .56023 Metetra
.656 .751 .92 r
.867 .56023 .87324 .56836 .88882 .56397 .88264 .55579 Metetra
.656 .751 .92 r
.88264 .55579 .88882 .56397 .90449 .55955 .89837 .55133 Metetra
.656 .751 .92 r
.89837 .55133 .90449 .55955 .92024 .55511 .91418 .54684 Metetra
.656 .751 .92 r
.91418 .54684 .92024 .55511 .93607 .55064 .93008 .54233 Metetra
.656 .751 .92 r
.93008 .54233 .93607 .55064 .95199 .54615 .94606 .5378 Metetra
.656 .751 .92 r
.94606 .5378 .95199 .54615 .968 .54164 .96213 .53324 Metetra
.656 .751 .92 r
.96213 .53324 .968 .54164 .98409 .5371 .97829 .52866 Metetra
.656 .751 .92 r
.3859 .68759 .39384 .69452 .40705 .69077 .39915 .68381 Metetra
.656 .751 .92 r
.39915 .68381 .40705 .69077 .42032 .687 .41246 .68 Metetra
.656 .751 .92 r
.41246 .68 .42032 .687 .43366 .68322 .42584 .67618 Metetra
.656 .751 .92 r
.42584 .67618 .43366 .68322 .44707 .67941 .43929 .67234 Metetra
.656 .751 .92 r
.43929 .67234 .44707 .67941 .46054 .67558 .45281 .66848 Metetra
.656 .751 .92 r
.45281 .66848 .46054 .67558 .47408 .67174 .46639 .6646 Metetra
.656 .751 .92 r
.46639 .6646 .47408 .67174 .48769 .66788 .48004 .6607 Metetra
.656 .751 .92 r
.48004 .6607 .48769 .66788 .50136 .664 .49376 .65678 Metetra
.656 .751 .92 r
.49376 .65678 .50136 .664 .51511 .66009 .50755 .65284 Metetra
.656 .751 .92 r
.50755 .65284 .51511 .66009 .52892 .65617 .52141 .64888 Metetra
.656 .751 .92 r
.52141 .64888 .52892 .65617 .54281 .65223 .53534 .64491 Metetra
.656 .751 .92 r
.53534 .64491 .54281 .65223 .55676 .64827 .54934 .64091 Metetra
.656 .751 .92 r
.54934 .64091 .55676 .64827 .57078 .64429 .56341 .63689 Metetra
.656 .751 .92 r
.56341 .63689 .57078 .64429 .58488 .64029 .57755 .63285 Metetra
.656 .751 .92 r
.57755 .63285 .58488 .64029 .59905 .63627 .59177 .62879 Metetra
.656 .751 .92 r
.59177 .62879 .59905 .63627 .61329 .63223 .60606 .62471 Metetra
.656 .751 .92 r
.60606 .62471 .61329 .63223 .6276 .62816 .62042 .62061 Metetra
.656 .751 .92 r
.62042 .62061 .6276 .62816 .64199 .62408 .63486 .61649 Metetra
.656 .751 .92 r
.63486 .61649 .64199 .62408 .65645 .61998 .64937 .61235 Metetra
.656 .751 .92 r
.64937 .61235 .65645 .61998 .67098 .61585 .66395 .60818 Metetra
.656 .751 .92 r
.66395 .60818 .67098 .61585 .68559 .61171 .67862 .604 Metetra
.656 .751 .92 r
.67862 .604 .68559 .61171 .70028 .60754 .69335 .59979 Metetra
.656 .751 .92 r
.69335 .59979 .70028 .60754 .71504 .60335 .70817 .59556 Metetra
.656 .751 .92 r
.70817 .59556 .71504 .60335 .72988 .59914 .72306 .59131 Metetra
.656 .751 .92 r
.72306 .59131 .72988 .59914 .74479 .5949 .73803 .58703 Metetra
.656 .751 .92 r
.73803 .58703 .74479 .5949 .75979 .59065 .75308 .58273 Metetra
.656 .751 .92 r
.75308 .58273 .75979 .59065 .77486 .58637 .76821 .57842 Metetra
.656 .751 .92 r
.76821 .57842 .77486 .58637 .79002 .58207 .78342 .57407 Metetra
.656 .751 .92 r
.78342 .57407 .79002 .58207 .80525 .57775 .79871 .56971 Metetra
.656 .751 .92 r
.79871 .56971 .80525 .57775 .82056 .5734 .81409 .56532 Metetra
.656 .751 .92 r
.81409 .56532 .82056 .5734 .83596 .56904 .82954 .56091 Metetra
.656 .751 .92 r
.82954 .56091 .83596 .56904 .85144 .56465 .84508 .55647 Metetra
.656 .751 .92 r
.84508 .55647 .85144 .56465 .867 .56023 .8607 .55202 Metetra
.656 .751 .92 r
.8607 .55202 .867 .56023 .88264 .55579 .8764 .54753 Metetra
.656 .751 .92 r
.8764 .54753 .88264 .55579 .89837 .55133 .89219 .54303 Metetra
.656 .751 .92 r
.89219 .54303 .89837 .55133 .91418 .54684 .90806 .5385 Metetra
.656 .751 .92 r
.90806 .5385 .91418 .54684 .93008 .54233 .92402 .53394 Metetra
.656 .751 .92 r
.92402 .53394 .93008 .54233 .94606 .5378 .94007 .52936 Metetra
.656 .751 .92 r
.94007 .52936 .94606 .5378 .96213 .53324 .95621 .52476 Metetra
.656 .751 .92 r
.95621 .52476 .96213 .53324 .97829 .52866 .97243 .52012 Metetra
.656 .751 .92 r
.37788 .6806 .3859 .68759 .39915 .68381 .39117 .67678 Metetra
.656 .751 .92 r
.39117 .67678 .39915 .68381 .41246 .68 .40453 .67294 Metetra
.656 .751 .92 r
.40453 .67294 .41246 .68 .42584 .67618 .41795 .66908 Metetra
.656 .751 .92 r
.41795 .66908 .42584 .67618 .43929 .67234 .43144 .6652 Metetra
.656 .751 .92 r
.43144 .6652 .43929 .67234 .45281 .66848 .445 .66131 Metetra
.656 .751 .92 r
.445 .66131 .45281 .66848 .46639 .6646 .45863 .65739 Metetra
.656 .751 .92 r
.45863 .65739 .46639 .6646 .48004 .6607 .47233 .65345 Metetra
.656 .751 .92 r
.47233 .65345 .48004 .6607 .49376 .65678 .48609 .6495 Metetra
.656 .751 .92 r
.48609 .6495 .49376 .65678 .50755 .65284 .49992 .64552 Metetra
.656 .751 .92 r
.49992 .64552 .50755 .65284 .52141 .64888 .51383 .64153 Metetra
.656 .751 .92 r
.51383 .64153 .52141 .64888 .53534 .64491 .5278 .63751 Metetra
.656 .751 .92 r
.5278 .63751 .53534 .64491 .54934 .64091 .54185 .63348 Metetra
.656 .751 .92 r
.54185 .63348 .54934 .64091 .56341 .63689 .55597 .62942 Metetra
.656 .751 .92 r
.55597 .62942 .56341 .63689 .57755 .63285 .57016 .62534 Metetra
.656 .751 .92 r
.57016 .62534 .57755 .63285 .59177 .62879 .58442 .62125 Metetra
.656 .751 .92 r
.58442 .62125 .59177 .62879 .60606 .62471 .59876 .61713 Metetra
.656 .751 .92 r
.59876 .61713 .60606 .62471 .62042 .62061 .61317 .61299 Metetra
.656 .751 .92 r
.61317 .61299 .62042 .62061 .63486 .61649 .62766 .60882 Metetra
.656 .751 .92 r
.62766 .60882 .63486 .61649 .64937 .61235 .64222 .60464 Metetra
.656 .751 .92 r
.64222 .60464 .64937 .61235 .66395 .60818 .65686 .60044 Metetra
.656 .751 .92 r
.65686 .60044 .66395 .60818 .67862 .604 .67157 .59621 Metetra
.656 .751 .92 r
.67157 .59621 .67862 .604 .69335 .59979 .68636 .59196 Metetra
.656 .751 .92 r
.68636 .59196 .69335 .59979 .70817 .59556 .70123 .58769 Metetra
.656 .751 .92 r
.70123 .58769 .70817 .59556 .72306 .59131 .71618 .5834 Metetra
.656 .751 .92 r
.71618 .5834 .72306 .59131 .73803 .58703 .73121 .57908 Metetra
.656 .751 .92 r
.73121 .57908 .73803 .58703 .75308 .58273 .74631 .57474 Metetra
.656 .751 .92 r
.74631 .57474 .75308 .58273 .76821 .57842 .7615 .57038 Metetra
.656 .751 .92 r
.7615 .57038 .76821 .57842 .78342 .57407 .77676 .566 Metetra
.656 .751 .92 r
.77676 .566 .78342 .57407 .79871 .56971 .79211 .56159 Metetra
.656 .751 .92 r
.79211 .56159 .79871 .56971 .81409 .56532 .80754 .55716 Metetra
.656 .751 .92 r
.80754 .55716 .81409 .56532 .82954 .56091 .82306 .5527 Metetra
.656 .751 .92 r
.82306 .5527 .82954 .56091 .84508 .55647 .83865 .54822 Metetra
.656 .751 .92 r
.83865 .54822 .84508 .55647 .8607 .55202 .85433 .54372 Metetra
.656 .751 .92 r
.85433 .54372 .8607 .55202 .8764 .54753 .8701 .53919 Metetra
.656 .751 .92 r
.8701 .53919 .8764 .54753 .89219 .54303 .88595 .53464 Metetra
.656 .751 .92 r
.88595 .53464 .89219 .54303 .90806 .5385 .90189 .53006 Metetra
.656 .751 .92 r
.90189 .53006 .90806 .5385 .92402 .53394 .91791 .52546 Metetra
.656 .751 .92 r
.91791 .52546 .92402 .53394 .94007 .52936 .93402 .52084 Metetra
.656 .751 .92 r
.93402 .52084 .94007 .52936 .95621 .52476 .95022 .51618 Metetra
.656 .751 .92 r
.95022 .51618 .95621 .52476 .97243 .52012 .96651 .51151 Metetra
.656 .751 .92 r
.36979 .67354 .37788 .6806 .39117 .67678 .38313 .66968 Metetra
.656 .751 .92 r
.38313 .66968 .39117 .67678 .40453 .67294 .39652 .66581 Metetra
.656 .751 .92 r
.39652 .66581 .40453 .67294 .41795 .66908 .40999 .66191 Metetra
.656 .751 .92 r
.40999 .66191 .41795 .66908 .43144 .6652 .42352 .658 Metetra
.656 .751 .92 r
.42352 .658 .43144 .6652 .445 .66131 .43713 .65407 Metetra
.656 .751 .92 r
.43713 .65407 .445 .66131 .45863 .65739 .4508 .65012 Metetra
.656 .751 .92 r
.4508 .65012 .45863 .65739 .47233 .65345 .46454 .64614 Metetra
.656 .751 .92 r
.46454 .64614 .47233 .65345 .48609 .6495 .47835 .64215 Metetra
.656 .751 .92 r
.47835 .64215 .48609 .6495 .49992 .64552 .49222 .63814 Metetra
.656 .751 .92 r
.49222 .63814 .49992 .64552 .51383 .64153 .50618 .6341 Metetra
.656 .751 .92 r
.50618 .6341 .51383 .64153 .5278 .63751 .5202 .63005 Metetra
.656 .751 .92 r
.5202 .63005 .5278 .63751 .54185 .63348 .53429 .62598 Metetra
.656 .751 .92 r
.53429 .62598 .54185 .63348 .55597 .62942 .54846 .62188 Metetra
.656 .751 .92 r
.54846 .62188 .55597 .62942 .57016 .62534 .5627 .61776 Metetra
.656 .751 .92 r
.5627 .61776 .57016 .62534 .58442 .62125 .57701 .61363 Metetra
.656 .751 .92 r
.57701 .61363 .58442 .62125 .59876 .61713 .5914 .60947 Metetra
.656 .751 .92 r
.5914 .60947 .59876 .61713 .61317 .61299 .60586 .60529 Metetra
.656 .751 .92 r
.60586 .60529 .61317 .61299 .62766 .60882 .62039 .60109 Metetra
.656 .751 .92 r
.62039 .60109 .62766 .60882 .64222 .60464 .63501 .59686 Metetra
.656 .751 .92 r
.63501 .59686 .64222 .60464 .65686 .60044 .6497 .59262 Metetra
.656 .751 .92 r
.6497 .59262 .65686 .60044 .67157 .59621 .66446 .58835 Metetra
.656 .751 .92 r
.66446 .58835 .67157 .59621 .68636 .59196 .67931 .58406 Metetra
.656 .751 .92 r
.67931 .58406 .68636 .59196 .70123 .58769 .69423 .57975 Metetra
.656 .751 .92 r
.69423 .57975 .70123 .58769 .71618 .5834 .70923 .57541 Metetra
.656 .751 .92 r
.70923 .57541 .71618 .5834 .73121 .57908 .72431 .57105 Metetra
.656 .751 .92 r
.72431 .57105 .73121 .57908 .74631 .57474 .73947 .56667 Metetra
.656 .751 .92 r
.73947 .56667 .74631 .57474 .7615 .57038 .75472 .56227 Metetra
.656 .751 .92 r
.75472 .56227 .7615 .57038 .77676 .566 .77004 .55784 Metetra
.656 .751 .92 r
.77004 .55784 .77676 .566 .79211 .56159 .78545 .55339 Metetra
.656 .751 .92 r
.78545 .55339 .79211 .56159 .80754 .55716 .80094 .54891 Metetra
.656 .751 .92 r
.80094 .54891 .80754 .55716 .82306 .5527 .81651 .54441 Metetra
.656 .751 .92 r
.81651 .54441 .82306 .5527 .83865 .54822 .83216 .53989 Metetra
.656 .751 .92 r
.83216 .53989 .83865 .54822 .85433 .54372 .84791 .53534 Metetra
.656 .751 .92 r
.84791 .53534 .85433 .54372 .8701 .53919 .86373 .53077 Metetra
.656 .751 .92 r
.86373 .53077 .8701 .53919 .88595 .53464 .87965 .52617 Metetra
.656 .751 .92 r
.87965 .52617 .88595 .53464 .90189 .53006 .89565 .52155 Metetra
.656 .751 .92 r
.89565 .52155 .90189 .53006 .91791 .52546 .91174 .5169 Metetra
.656 .751 .92 r
.91174 .5169 .91791 .52546 .93402 .52084 .92791 .51223 Metetra
.656 .751 .92 r
.92791 .51223 .93402 .52084 .95022 .51618 .94418 .50753 Metetra
.656 .751 .92 r
.94418 .50753 .95022 .51618 .96651 .51151 .96053 .5028 Metetra
.656 .751 .92 r
.36163 .66641 .36979 .67354 .38313 .66968 .375 .66252 Metetra
.656 .751 .92 r
.375 .66252 .38313 .66968 .39652 .66581 .38845 .65861 Metetra
.656 .751 .92 r
.38845 .65861 .39652 .66581 .40999 .66191 .40195 .65468 Metetra
.656 .751 .92 r
.40195 .65468 .40999 .66191 .42352 .658 .41553 .65073 Metetra
.656 .751 .92 r
.41553 .65073 .42352 .658 .43713 .65407 .42917 .64676 Metetra
.656 .751 .92 r
.42917 .64676 .43713 .65407 .4508 .65012 .44289 .64277 Metetra
.656 .751 .92 r
.44289 .64277 .4508 .65012 .46454 .64614 .45667 .63876 Metetra
.656 .751 .92 r
.45667 .63876 .46454 .64614 .47835 .64215 .47053 .63473 Metetra
.656 .751 .92 r
.47053 .63473 .47835 .64215 .49222 .63814 .48445 .63068 Metetra
.656 .751 .92 r
.48445 .63068 .49222 .63814 .50618 .6341 .49845 .62661 Metetra
.656 .751 .92 r
.49845 .62661 .50618 .6341 .5202 .63005 .51252 .62252 Metetra
.656 .751 .92 r
.51252 .62252 .5202 .63005 .53429 .62598 .52666 .6184 Metetra
.656 .751 .92 r
.52666 .6184 .53429 .62598 .54846 .62188 .54087 .61427 Metetra
.656 .751 .92 r
.54087 .61427 .54846 .62188 .5627 .61776 .55516 .61011 Metetra
.656 .751 .92 r
.55516 .61011 .5627 .61776 .57701 .61363 .56952 .60594 Metetra
.656 .751 .92 r
.56952 .60594 .57701 .61363 .5914 .60947 .58396 .60174 Metetra
.656 .751 .92 r
.58396 .60174 .5914 .60947 .60586 .60529 .59847 .59752 Metetra
.656 .751 .92 r
.59847 .59752 .60586 .60529 .62039 .60109 .61306 .59327 Metetra
.656 .751 .92 r
.61306 .59327 .62039 .60109 .63501 .59686 .62772 .58901 Metetra
.656 .751 .92 r
.62772 .58901 .63501 .59686 .6497 .59262 .64246 .58472 Metetra
.656 .751 .92 r
.64246 .58472 .6497 .59262 .66446 .58835 .65728 .58041 Metetra
.656 .751 .92 r
.65728 .58041 .66446 .58835 .67931 .58406 .67218 .57608 Metetra
.656 .751 .92 r
.67218 .57608 .67931 .58406 .69423 .57975 .68716 .57173 Metetra
.656 .751 .92 r
.68716 .57173 .69423 .57975 .70923 .57541 .70221 .56735 Metetra
.656 .751 .92 r
.70221 .56735 .70923 .57541 .72431 .57105 .71735 .56295 Metetra
.656 .751 .92 r
.71735 .56295 .72431 .57105 .73947 .56667 .73257 .55852 Metetra
.656 .751 .92 r
.73257 .55852 .73947 .56667 .75472 .56227 .74787 .55407 Metetra
.656 .751 .92 r
.74787 .55407 .75472 .56227 .77004 .55784 .76325 .5496 Metetra
.656 .751 .92 r
.76325 .5496 .77004 .55784 .78545 .55339 .77871 .54511 Metetra
.656 .751 .92 r
.77871 .54511 .78545 .55339 .80094 .54891 .79426 .54059 Metetra
.656 .751 .92 r
.79426 .54059 .80094 .54891 .81651 .54441 .80989 .53604 Metetra
.656 .751 .92 r
.80989 .53604 .81651 .54441 .83216 .53989 .82561 .53147 Metetra
.656 .751 .92 r
.82561 .53147 .83216 .53989 .84791 .53534 .84141 .52688 Metetra
.656 .751 .92 r
.84141 .52688 .84791 .53534 .86373 .53077 .8573 .52226 Metetra
.656 .751 .92 r
.8573 .52226 .86373 .53077 .87965 .52617 .87328 .51761 Metetra
.656 .751 .92 r
.87328 .51761 .87965 .52617 .89565 .52155 .88934 .51294 Metetra
.656 .751 .92 r
.88934 .51294 .89565 .52155 .91174 .5169 .9055 .50825 Metetra
.656 .751 .92 r
.9055 .50825 .91174 .5169 .92791 .51223 .92174 .50353 Metetra
.656 .751 .92 r
.92174 .50353 .92791 .51223 .94418 .50753 .93807 .49878 Metetra
.656 .751 .92 r
.93807 .49878 .94418 .50753 .96053 .5028 .95449 .49401 Metetra
.656 .751 .92 r
.35339 .65922 .36163 .66641 .375 .66252 .36681 .6553 Metetra
.656 .751 .92 r
.36681 .6553 .375 .66252 .38845 .65861 .38029 .65135 Metetra
.656 .751 .92 r
.38029 .65135 .38845 .65861 .40195 .65468 .39384 .64738 Metetra
.656 .751 .92 r
.39384 .64738 .40195 .65468 .41553 .65073 .40746 .64339 Metetra
.656 .751 .92 r
.40746 .64339 .41553 .65073 .42917 .64676 .42115 .63939 Metetra
.656 .751 .92 r
.42115 .63939 .42917 .64676 .44289 .64277 .43491 .63536 Metetra
.656 .751 .92 r
.43491 .63536 .44289 .64277 .45667 .63876 .44874 .63131 Metetra
.656 .751 .92 r
.44874 .63131 .45667 .63876 .47053 .63473 .46264 .62724 Metetra
.656 .751 .92 r
.46264 .62724 .47053 .63473 .48445 .63068 .47661 .62315 Metetra
.656 .751 .92 r
.47661 .62315 .48445 .63068 .49845 .62661 .49065 .61904 Metetra
.656 .751 .92 r
.49065 .61904 .49845 .62661 .51252 .62252 .50476 .61491 Metetra
.656 .751 .92 r
.50476 .61491 .51252 .62252 .52666 .6184 .51895 .61076 Metetra
.656 .751 .92 r
.51895 .61076 .52666 .6184 .54087 .61427 .53322 .60658 Metetra
.656 .751 .92 r
.53322 .60658 .54087 .61427 .55516 .61011 .54755 .60239 Metetra
.656 .751 .92 r
.54755 .60239 .55516 .61011 .56952 .60594 .56196 .59817 Metetra
.656 .751 .92 r
.56196 .59817 .56952 .60594 .58396 .60174 .57645 .59393 Metetra
.656 .751 .92 r
.57645 .59393 .58396 .60174 .59847 .59752 .59101 .58967 Metetra
.656 .751 .92 r
.59101 .58967 .59847 .59752 .61306 .59327 .60565 .58539 Metetra
.656 .751 .92 r
.60565 .58539 .61306 .59327 .62772 .58901 .62037 .58108 Metetra
.656 .751 .92 r
.62037 .58108 .62772 .58901 .64246 .58472 .63516 .57675 Metetra
.656 .751 .92 r
.63516 .57675 .64246 .58472 .65728 .58041 .65003 .5724 Metetra
.656 .751 .92 r
.65003 .5724 .65728 .58041 .67218 .57608 .66499 .56802 Metetra
.656 .751 .92 r
.66499 .56802 .67218 .57608 .68716 .57173 .68002 .56363 Metetra
.656 .751 .92 r
.68002 .56363 .68716 .57173 .70221 .56735 .69513 .5592 Metetra
.656 .751 .92 r
.69513 .5592 .70221 .56735 .71735 .56295 .71032 .55476 Metetra
.656 .751 .92 r
.71032 .55476 .71735 .56295 .73257 .55852 .7256 .55029 Metetra
.656 .751 .92 r
.7256 .55029 .73257 .55852 .74787 .55407 .74095 .5458 Metetra
.656 .751 .92 r
.74095 .5458 .74787 .55407 .76325 .5496 .75639 .54128 Metetra
.656 .751 .92 r
.75639 .54128 .76325 .5496 .77871 .54511 .77191 .53674 Metetra
.656 .751 .92 r
.77191 .53674 .77871 .54511 .79426 .54059 .78752 .53218 Metetra
.656 .751 .92 r
.78752 .53218 .79426 .54059 .80989 .53604 .80321 .52758 Metetra
.656 .751 .92 r
.80321 .52758 .80989 .53604 .82561 .53147 .81899 .52297 Metetra
.656 .751 .92 r
.81899 .52297 .82561 .53147 .84141 .52688 .83486 .51833 Metetra
.656 .751 .92 r
.83486 .51833 .84141 .52688 .8573 .52226 .85081 .51366 Metetra
.656 .751 .92 r
.85081 .51366 .8573 .52226 .87328 .51761 .86685 .50897 Metetra
.656 .751 .92 r
.86685 .50897 .87328 .51761 .88934 .51294 .88298 .50425 Metetra
.656 .751 .92 r
.88298 .50425 .88934 .51294 .9055 .50825 .89919 .49951 Metetra
.656 .751 .92 r
.89919 .49951 .9055 .50825 .92174 .50353 .9155 .49474 Metetra
.656 .751 .92 r
.9155 .49474 .92174 .50353 .93807 .49878 .9319 .48994 Metetra
.656 .751 .92 r
.9319 .48994 .93807 .49878 .95449 .49401 .94839 .48512 Metetra
.656 .751 .92 r
.34508 .65197 .35339 .65922 .36681 .6553 .35853 .648 Metetra
.656 .751 .92 r
.35853 .648 .36681 .6553 .38029 .65135 .37206 .64402 Metetra
.656 .751 .92 r
.37206 .64402 .38029 .65135 .39384 .64738 .38565 .64001 Metetra
.656 .751 .92 r
.38565 .64001 .39384 .64738 .40746 .64339 .39932 .63599 Metetra
.656 .751 .92 r
.39932 .63599 .40746 .64339 .42115 .63939 .41305 .63194 Metetra
.656 .751 .92 r
.41305 .63194 .42115 .63939 .43491 .63536 .42685 .62788 Metetra
.656 .751 .92 r
.42685 .62788 .43491 .63536 .44874 .63131 .44072 .62379 Metetra
.656 .751 .92 r
.44072 .62379 .44874 .63131 .46264 .62724 .45467 .61968 Metetra
.656 .751 .92 r
.45467 .61968 .46264 .62724 .47661 .62315 .46869 .61555 Metetra
.656 .751 .92 r
.46869 .61555 .47661 .62315 .49065 .61904 .48278 .6114 Metetra
.656 .751 .92 r
.48278 .6114 .49065 .61904 .50476 .61491 .49694 .60723 Metetra
.656 .751 .92 r
.49694 .60723 .50476 .61491 .51895 .61076 .51117 .60304 Metetra
.656 .751 .92 r
.51117 .60304 .51895 .61076 .53322 .60658 .52548 .59883 Metetra
.656 .751 .92 r
.52548 .59883 .53322 .60658 .54755 .60239 .53987 .59459 Metetra
.656 .751 .92 r
.53987 .59459 .54755 .60239 .56196 .59817 .55433 .59033 Metetra
.656 .751 .92 r
.55433 .59033 .56196 .59817 .57645 .59393 .56887 .58605 Metetra
.656 .751 .92 r
.56887 .58605 .57645 .59393 .59101 .58967 .58348 .58175 Metetra
.656 .751 .92 r
.58348 .58175 .59101 .58967 .60565 .58539 .59817 .57742 Metetra
.656 .751 .92 r
.59817 .57742 .60565 .58539 .62037 .58108 .61294 .57307 Metetra
.656 .751 .92 r
.61294 .57307 .62037 .58108 .63516 .57675 .62779 .5687 Metetra
.656 .751 .92 r
.62779 .5687 .63516 .57675 .65003 .5724 .64271 .56431 Metetra
.656 .751 .92 r
.64271 .56431 .65003 .5724 .66499 .56802 .65772 .55989 Metetra
.656 .751 .92 r
.65772 .55989 .66499 .56802 .68002 .56363 .67281 .55545 Metetra
.656 .751 .92 r
.67281 .55545 .68002 .56363 .69513 .5592 .68797 .55098 Metetra
.656 .751 .92 r
.68797 .55098 .69513 .5592 .71032 .55476 .70322 .54649 Metetra
.656 .751 .92 r
.70322 .54649 .71032 .55476 .7256 .55029 .71855 .54198 Metetra
.656 .751 .92 r
.71855 .54198 .7256 .55029 .74095 .5458 .73397 .53744 Metetra
.656 .751 .92 r
.73397 .53744 .74095 .5458 .75639 .54128 .74946 .53288 Metetra
.656 .751 .92 r
.74946 .53288 .75639 .54128 .77191 .53674 .76505 .52829 Metetra
.656 .751 .92 r
.76505 .52829 .77191 .53674 .78752 .53218 .78071 .52368 Metetra
.656 .751 .92 r
.78071 .52368 .78752 .53218 .80321 .52758 .79647 .51904 Metetra
.656 .751 .92 r
.79647 .51904 .80321 .52758 .81899 .52297 .81231 .51438 Metetra
.656 .751 .92 r
.81231 .51438 .81899 .52297 .83486 .51833 .82823 .50969 Metetra
.656 .751 .92 r
.82823 .50969 .83486 .51833 .85081 .51366 .84425 .50498 Metetra
.656 .751 .92 r
.84425 .50498 .85081 .51366 .86685 .50897 .86035 .50024 Metetra
.656 .751 .92 r
.86035 .50024 .86685 .50897 .88298 .50425 .87654 .49547 Metetra
.656 .751 .92 r
.87654 .49547 .88298 .50425 .89919 .49951 .89283 .49068 Metetra
.656 .751 .92 r
.89283 .49068 .89919 .49951 .9155 .49474 .9092 .48586 Metetra
.656 .751 .92 r
.9092 .48586 .9155 .49474 .9319 .48994 .92567 .48102 Metetra
.656 .751 .92 r
.92567 .48102 .9319 .48994 .94839 .48512 .94222 .47614 Metetra
.656 .751 .92 r
.33669 .64464 .34508 .65197 .35853 .648 .35018 .64064 Metetra
.656 .751 .92 r
.35018 .64064 .35853 .648 .37206 .64402 .36375 .63662 Metetra
.656 .751 .92 r
.36375 .63662 .37206 .64402 .38565 .64001 .37739 .63257 Metetra
.656 .751 .92 r
.37739 .63257 .38565 .64001 .39932 .63599 .39109 .62851 Metetra
.656 .751 .92 r
.39109 .62851 .39932 .63599 .41305 .63194 .40487 .62443 Metetra
.656 .751 .92 r
.40487 .62443 .41305 .63194 .42685 .62788 .41872 .62032 Metetra
.656 .751 .92 r
.41872 .62032 .42685 .62788 .44072 .62379 .43264 .6162 Metetra
.656 .751 .92 r
.43264 .6162 .44072 .62379 .45467 .61968 .44663 .61205 Metetra
.656 .751 .92 r
.44663 .61205 .45467 .61968 .46869 .61555 .46069 .60788 Metetra
.656 .751 .92 r
.46069 .60788 .46869 .61555 .48278 .6114 .47483 .60369 Metetra
.656 .751 .92 r
.47483 .60369 .48278 .6114 .49694 .60723 .48904 .59948 Metetra
.656 .751 .92 r
.48904 .59948 .49694 .60723 .51117 .60304 .50332 .59525 Metetra
.656 .751 .92 r
.50332 .59525 .51117 .60304 .52548 .59883 .51768 .59099 Metetra
.656 .751 .92 r
.51768 .59099 .52548 .59883 .53987 .59459 .53211 .58671 Metetra
.656 .751 .92 r
.53211 .58671 .53987 .59459 .55433 .59033 .54662 .58241 Metetra
.656 .751 .92 r
.54662 .58241 .55433 .59033 .56887 .58605 .56121 .57809 Metetra
.656 .751 .92 r
.56121 .57809 .56887 .58605 .58348 .58175 .57588 .57375 Metetra
.656 .751 .92 r
.57588 .57375 .58348 .58175 .59817 .57742 .59062 .56938 Metetra
.656 .751 .92 r
.59062 .56938 .59817 .57742 .61294 .57307 .60544 .56499 Metetra
.656 .751 .92 r
.60544 .56499 .61294 .57307 .62779 .5687 .62034 .56057 Metetra
.656 .751 .92 r
.62034 .56057 .62779 .5687 .64271 .56431 .63532 .55613 Metetra
.656 .751 .92 r
.63532 .55613 .64271 .56431 .65772 .55989 .65038 .55167 Metetra
.656 .751 .92 r
.65038 .55167 .65772 .55989 .67281 .55545 .66552 .54718 Metetra
.656 .751 .92 r
.66552 .54718 .67281 .55545 .68797 .55098 .68074 .54267 Metetra
.656 .751 .92 r
.68074 .54267 .68797 .55098 .70322 .54649 .69605 .53814 Metetra
.656 .751 .92 r
.69605 .53814 .70322 .54649 .71855 .54198 .71144 .53358 Metetra
.656 .751 .92 r
.71144 .53358 .71855 .54198 .73397 .53744 .72691 .529 Metetra
.656 .751 .92 r
.72691 .529 .73397 .53744 .74946 .53288 .74247 .52439 Metetra
.656 .751 .92 r
.74247 .52439 .74946 .53288 .76505 .52829 .75811 .51976 Metetra
.656 .751 .92 r
.75811 .51976 .76505 .52829 .78071 .52368 .77384 .5151 Metetra
.656 .751 .92 r
.77384 .5151 .78071 .52368 .79647 .51904 .78965 .51041 Metetra
.656 .751 .92 r
.78965 .51041 .79647 .51904 .81231 .51438 .80555 .5057 Metetra
.656 .751 .92 r
.80555 .5057 .81231 .51438 .82823 .50969 .82154 .50097 Metetra
.656 .751 .92 r
.82154 .50097 .82823 .50969 .84425 .50498 .83762 .49621 Metetra
.656 .751 .92 r
.83762 .49621 .84425 .50498 .86035 .50024 .85379 .49142 Metetra
.656 .751 .92 r
.85379 .49142 .86035 .50024 .87654 .49547 .87005 .4866 Metetra
.656 .751 .92 r
.87005 .4866 .87654 .49547 .89283 .49068 .88639 .48176 Metetra
.656 .751 .92 r
.88639 .48176 .89283 .49068 .9092 .48586 .90283 .47689 Metetra
.656 .751 .92 r
.90283 .47689 .9092 .48586 .92567 .48102 .91937 .472 Metetra
.656 .751 .92 r
.91937 .472 .92567 .48102 .94222 .47614 .93599 .46707 Metetra
.656 .751 .92 r
.32821 .63725 .33669 .64464 .35018 .64064 .34175 .63321 Metetra
.656 .751 .92 r
.34175 .63321 .35018 .64064 .36375 .63662 .35536 .62915 Metetra
.656 .751 .92 r
.35536 .62915 .36375 .63662 .37739 .63257 .36904 .62507 Metetra
.656 .751 .92 r
.36904 .62507 .37739 .63257 .39109 .62851 .38279 .62096 Metetra
.656 .751 .92 r
.38279 .62096 .39109 .62851 .40487 .62443 .39661 .61684 Metetra
.656 .751 .92 r
.39661 .61684 .40487 .62443 .41872 .62032 .4105 .6127 Metetra
.656 .751 .92 r
.4105 .6127 .41872 .62032 .43264 .6162 .42447 .60853 Metetra
.656 .751 .92 r
.42447 .60853 .43264 .6162 .44663 .61205 .43851 .60435 Metetra
.656 .751 .92 r
.43851 .60435 .44663 .61205 .46069 .60788 .45262 .60014 Metetra
.656 .751 .92 r
.45262 .60014 .46069 .60788 .47483 .60369 .4668 .59591 Metetra
.656 .751 .92 r
.4668 .59591 .47483 .60369 .48904 .59948 .48106 .59165 Metetra
.656 .751 .92 r
.48106 .59165 .48904 .59948 .50332 .59525 .49539 .58738 Metetra
.656 .751 .92 r
.49539 .58738 .50332 .59525 .51768 .59099 .5098 .58308 Metetra
.656 .751 .92 r
.5098 .58308 .51768 .59099 .53211 .58671 .52428 .57876 Metetra
.656 .751 .92 r
.52428 .57876 .53211 .58671 .54662 .58241 .53884 .57442 Metetra
.656 .751 .92 r
.53884 .57442 .54662 .58241 .56121 .57809 .55348 .57005 Metetra
.656 .751 .92 r
.55348 .57005 .56121 .57809 .57588 .57375 .5682 .56567 Metetra
.656 .751 .92 r
.5682 .56567 .57588 .57375 .59062 .56938 .58299 .56125 Metetra
.656 .751 .92 r
.58299 .56125 .59062 .56938 .60544 .56499 .59787 .55682 Metetra
.656 .751 .92 r
.59787 .55682 .60544 .56499 .62034 .56057 .61282 .55236 Metetra
.656 .751 .92 r
.61282 .55236 .62034 .56057 .63532 .55613 .62785 .54788 Metetra
.656 .751 .92 r
.62785 .54788 .63532 .55613 .65038 .55167 .64297 .54337 Metetra
.656 .751 .92 r
.64297 .54337 .65038 .55167 .66552 .54718 .65816 .53884 Metetra
.656 .751 .92 r
.65816 .53884 .66552 .54718 .68074 .54267 .67344 .53428 Metetra
.656 .751 .92 r
.67344 .53428 .68074 .54267 .69605 .53814 .68881 .52971 Metetra
.656 .751 .92 r
.68881 .52971 .69605 .53814 .71144 .53358 .70425 .5251 Metetra
.656 .751 .92 r
.70425 .5251 .71144 .53358 .72691 .529 .71978 .52047 Metetra
.656 .751 .92 r
.71978 .52047 .72691 .529 .74247 .52439 .7354 .51582 Metetra
.656 .751 .92 r
.7354 .51582 .74247 .52439 .75811 .51976 .7511 .51114 Metetra
.656 .751 .92 r
.7511 .51114 .75811 .51976 .77384 .5151 .76689 .50643 Metetra
.656 .751 .92 r
.76689 .50643 .77384 .5151 .78965 .51041 .78277 .5017 Metetra
.656 .751 .92 r
.78277 .5017 .78965 .51041 .80555 .5057 .79873 .49694 Metetra
.656 .751 .92 r
.79873 .49694 .80555 .5057 .82154 .50097 .81478 .49215 Metetra
.656 .751 .92 r
.81478 .49215 .82154 .50097 .83762 .49621 .83092 .48734 Metetra
.656 .751 .92 r
.83092 .48734 .83762 .49621 .85379 .49142 .84716 .48251 Metetra
.656 .751 .92 r
.84716 .48251 .85379 .49142 .87005 .4866 .86348 .47764 Metetra
.656 .751 .92 r
.86348 .47764 .87005 .4866 .88639 .48176 .87989 .47275 Metetra
.656 .751 .92 r
.87989 .47275 .88639 .48176 .90283 .47689 .8964 .46783 Metetra
.656 .751 .92 r
.8964 .46783 .90283 .47689 .91937 .472 .913 .46288 Metetra
.656 .751 .92 r
.913 .46288 .91937 .472 .93599 .46707 .9297 .45791 Metetra
.656 .751 .92 r
.31966 .62978 .32821 .63725 .34175 .63321 .33324 .62571 Metetra
.656 .751 .92 r
.33324 .62571 .34175 .63321 .35536 .62915 .3469 .62161 Metetra
.656 .751 .92 r
.3469 .62161 .35536 .62915 .36904 .62507 .36062 .61749 Metetra
.656 .751 .92 r
.36062 .61749 .36904 .62507 .38279 .62096 .37441 .61335 Metetra
.656 .751 .92 r
.37441 .61335 .38279 .62096 .39661 .61684 .38828 .60918 Metetra
.656 .751 .92 r
.38828 .60918 .39661 .61684 .4105 .6127 .40221 .605 Metetra
.656 .751 .92 r
.40221 .605 .4105 .6127 .42447 .60853 .41622 .60079 Metetra
.656 .751 .92 r
.41622 .60079 .42447 .60853 .43851 .60435 .43031 .59657 Metetra
.656 .751 .92 r
.43031 .59657 .43851 .60435 .45262 .60014 .44446 .59232 Metetra
.656 .751 .92 r
.44446 .59232 .45262 .60014 .4668 .59591 .45869 .58804 Metetra
.656 .751 .92 r
.45869 .58804 .4668 .59591 .48106 .59165 .473 .58375 Metetra
.656 .751 .92 r
.473 .58375 .48106 .59165 .49539 .58738 .48738 .57943 Metetra
.656 .751 .92 r
.48738 .57943 .49539 .58738 .5098 .58308 .50184 .57509 Metetra
.656 .751 .92 r
.50184 .57509 .5098 .58308 .52428 .57876 .51637 .57073 Metetra
.656 .751 .92 r
.51637 .57073 .52428 .57876 .53884 .57442 .53098 .56635 Metetra
.656 .751 .92 r
.53098 .56635 .53884 .57442 .55348 .57005 .54567 .56194 Metetra
.656 .751 .92 r
.54567 .56194 .55348 .57005 .5682 .56567 .56044 .55751 Metetra
.656 .751 .92 r
.56044 .55751 .5682 .56567 .58299 .56125 .57529 .55305 Metetra
.656 .751 .92 r
.57529 .55305 .58299 .56125 .59787 .55682 .59022 .54857 Metetra
.656 .751 .92 r
.59022 .54857 .59787 .55682 .61282 .55236 .60522 .54407 Metetra
.656 .751 .92 r
.60522 .54407 .61282 .55236 .62785 .54788 .62031 .53954 Metetra
.656 .751 .92 r
.62031 .53954 .62785 .54788 .64297 .54337 .63548 .53499 Metetra
.656 .751 .92 r
.63548 .53499 .64297 .54337 .65816 .53884 .65073 .53041 Metetra
.656 .751 .92 r
.65073 .53041 .65816 .53884 .67344 .53428 .66607 .52581 Metetra
.656 .751 .92 r
.66607 .52581 .67344 .53428 .68881 .52971 .68149 .52118 Metetra
.656 .751 .92 r
.68149 .52118 .68881 .52971 .70425 .5251 .69699 .51653 Metetra
.656 .751 .92 r
.69699 .51653 .70425 .5251 .71978 .52047 .71258 .51186 Metetra
.656 .751 .92 r
.71258 .51186 .71978 .52047 .7354 .51582 .72826 .50715 Metetra
.656 .751 .92 r
.72826 .50715 .7354 .51582 .7511 .51114 .74402 .50243 Metetra
.656 .751 .92 r
.74402 .50243 .7511 .51114 .76689 .50643 .75987 .49767 Metetra
.656 .751 .92 r
.75987 .49767 .76689 .50643 .78277 .5017 .77581 .49289 Metetra
.656 .751 .92 r
.77581 .49289 .78277 .5017 .79873 .49694 .79184 .48808 Metetra
.656 .751 .92 r
.79184 .48808 .79873 .49694 .81478 .49215 .80795 .48325 Metetra
.656 .751 .92 r
.80795 .48325 .81478 .49215 .83092 .48734 .82416 .47839 Metetra
.656 .751 .92 r
.82416 .47839 .83092 .48734 .84716 .48251 .84046 .4735 Metetra
.656 .751 .92 r
.84046 .4735 .84716 .48251 .86348 .47764 .85684 .46858 Metetra
.656 .751 .92 r
.85684 .46858 .86348 .47764 .87989 .47275 .87333 .46364 Metetra
.656 .751 .92 r
.87333 .46364 .87989 .47275 .8964 .46783 .8899 .45867 Metetra
.656 .751 .92 r
.8899 .45867 .8964 .46783 .913 .46288 .90657 .45367 Metetra
.656 .751 .92 r
.90657 .45367 .913 .46288 .9297 .45791 .92334 .44864 Metetra
.656 .751 .92 r
.31103 .62225 .31966 .62978 .33324 .62571 .32465 .61813 Metetra
.656 .751 .92 r
.32465 .61813 .33324 .62571 .3469 .62161 .33835 .614 Metetra
.656 .751 .92 r
.33835 .614 .3469 .62161 .36062 .61749 .35211 .60984 Metetra
.656 .751 .92 r
.35211 .60984 .36062 .61749 .37441 .61335 .36595 .60565 Metetra
.656 .751 .92 r
.36595 .60565 .37441 .61335 .38828 .60918 .37986 .60145 Metetra
.656 .751 .92 r
.37986 .60145 .38828 .60918 .40221 .605 .39384 .59723 Metetra
.656 .751 .92 r
.39384 .59723 .40221 .605 .41622 .60079 .4079 .59298 Metetra
.656 .751 .92 r
.4079 .59298 .41622 .60079 .43031 .59657 .42203 .58871 Metetra
.656 .751 .92 r
.42203 .58871 .43031 .59657 .44446 .59232 .43623 .58442 Metetra
.656 .751 .92 r
.43623 .58442 .44446 .59232 .45869 .58804 .45051 .58011 Metetra
.656 .751 .92 r
.45051 .58011 .45869 .58804 .473 .58375 .46487 .57577 Metetra
.656 .751 .92 r
.46487 .57577 .473 .58375 .48738 .57943 .4793 .57141 Metetra
.656 .751 .92 r
.4793 .57141 .48738 .57943 .50184 .57509 .4938 .56703 Metetra
.656 .751 .92 r
.4938 .56703 .50184 .57509 .51637 .57073 .50839 .56262 Metetra
.656 .751 .92 r
.50839 .56262 .51637 .57073 .53098 .56635 .52305 .55819 Metetra
.656 .751 .92 r
.52305 .55819 .53098 .56635 .54567 .56194 .53779 .55374 Metetra
.656 .751 .92 r
.53779 .55374 .54567 .56194 .56044 .55751 .55261 .54927 Metetra
.656 .751 .92 r
.55261 .54927 .56044 .55751 .57529 .55305 .56751 .54477 Metetra
.656 .751 .92 r
.56751 .54477 .57529 .55305 .59022 .54857 .58249 .54024 Metetra
.656 .751 .92 r
.58249 .54024 .59022 .54857 .60522 .54407 .59755 .53569 Metetra
.656 .751 .92 r
.59755 .53569 .60522 .54407 .62031 .53954 .61269 .53112 Metetra
.656 .751 .92 r
.61269 .53112 .62031 .53954 .63548 .53499 .62792 .52652 Metetra
.656 .751 .92 r
.62792 .52652 .63548 .53499 .65073 .53041 .64323 .5219 Metetra
.656 .751 .92 r
.64323 .5219 .65073 .53041 .66607 .52581 .65862 .51725 Metetra
.656 .751 .92 r
.65862 .51725 .66607 .52581 .68149 .52118 .6741 .51258 Metetra
.656 .751 .92 r
.6741 .51258 .68149 .52118 .69699 .51653 .68966 .50788 Metetra
.656 .751 .92 r
.68966 .50788 .69699 .51653 .71258 .51186 .70531 .50315 Metetra
.656 .751 .92 r
.70531 .50315 .71258 .51186 .72826 .50715 .72105 .4984 Metetra
.656 .751 .92 r
.72105 .4984 .72826 .50715 .74402 .50243 .73687 .49363 Metetra
.656 .751 .92 r
.73687 .49363 .74402 .50243 .75987 .49767 .75278 .48882 Metetra
.656 .751 .92 r
.75278 .48882 .75987 .49767 .77581 .49289 .76878 .48399 Metetra
.656 .751 .92 r
.76878 .48399 .77581 .49289 .79184 .48808 .78487 .47914 Metetra
.656 .751 .92 r
.78487 .47914 .79184 .48808 .80795 .48325 .80105 .47425 Metetra
.656 .751 .92 r
.80105 .47425 .80795 .48325 .82416 .47839 .81732 .46934 Metetra
.656 .751 .92 r
.81732 .46934 .82416 .47839 .84046 .4735 .83369 .4644 Metetra
.656 .751 .92 r
.83369 .4644 .84046 .4735 .85684 .46858 .85014 .45943 Metetra
.656 .751 .92 r
.85014 .45943 .85684 .46858 .87333 .46364 .86669 .45444 Metetra
.656 .751 .92 r
.86669 .45444 .87333 .46364 .8899 .45867 .88333 .44942 Metetra
.656 .751 .92 r
.88333 .44942 .8899 .45867 .90657 .45367 .90007 .44436 Metetra
.656 .751 .92 r
.90007 .44436 .90657 .45367 .92334 .44864 .91691 .43928 Metetra
.656 .751 .92 r
.30231 .61465 .31103 .62225 .32465 .61813 .31598 .61049 Metetra
.656 .751 .92 r
.31598 .61049 .32465 .61813 .33835 .614 .32972 .60631 Metetra
.656 .751 .92 r
.32972 .60631 .33835 .614 .35211 .60984 .34353 .60211 Metetra
.656 .751 .92 r
.34353 .60211 .35211 .60984 .36595 .60565 .35741 .59789 Metetra
.656 .751 .92 r
.35741 .59789 .36595 .60565 .37986 .60145 .37137 .59365 Metetra
.656 .751 .92 r
.37137 .59365 .37986 .60145 .39384 .59723 .38539 .58938 Metetra
.656 .751 .92 r
.38539 .58938 .39384 .59723 .4079 .59298 .39949 .58509 Metetra
.656 .751 .92 r
.39949 .58509 .4079 .59298 .42203 .58871 .41367 .58078 Metetra
.656 .751 .92 r
.41367 .58078 .42203 .58871 .43623 .58442 .42792 .57645 Metetra
.656 .751 .92 r
.42792 .57645 .43623 .58442 .45051 .58011 .44225 .57209 Metetra
.656 .751 .92 r
.44225 .57209 .45051 .58011 .46487 .57577 .45665 .56771 Metetra
.656 .751 .92 r
.45665 .56771 .46487 .57577 .4793 .57141 .47113 .56331 Metetra
.656 .751 .92 r
.47113 .56331 .4793 .57141 .4938 .56703 .48569 .55888 Metetra
.656 .751 .92 r
.48569 .55888 .4938 .56703 .50839 .56262 .50032 .55443 Metetra
.656 .751 .92 r
.50032 .55443 .50839 .56262 .52305 .55819 .51503 .54996 Metetra
.656 .751 .92 r
.51503 .54996 .52305 .55819 .53779 .55374 .52983 .54546 Metetra
.656 .751 .92 r
.52983 .54546 .53779 .55374 .55261 .54927 .5447 .54094 Metetra
.656 .751 .92 r
.5447 .54094 .55261 .54927 .56751 .54477 .55965 .5364 Metetra
.656 .751 .92 r
.55965 .5364 .56751 .54477 .58249 .54024 .57469 .53183 Metetra
.656 .751 .92 r
.57469 .53183 .58249 .54024 .59755 .53569 .5898 .52723 Metetra
.656 .751 .92 r
.5898 .52723 .59755 .53569 .61269 .53112 .605 .52261 Metetra
.656 .751 .92 r
.605 .52261 .61269 .53112 .62792 .52652 .62028 .51797 Metetra
.656 .751 .92 r
.62028 .51797 .62792 .52652 .64323 .5219 .63565 .5133 Metetra
.656 .751 .92 r
.63565 .5133 .64323 .5219 .65862 .51725 .6511 .5086 Metetra
.656 .751 .92 r
.6511 .5086 .65862 .51725 .6741 .51258 .66663 .50388 Metetra
.656 .751 .92 r
.66663 .50388 .6741 .51258 .68966 .50788 .68226 .49914 Metetra
.656 .751 .92 r
.68226 .49914 .68966 .50788 .70531 .50315 .69796 .49436 Metetra
.656 .751 .92 r
.69796 .49436 .70531 .50315 .72105 .4984 .71376 .48956 Metetra
.656 .751 .92 r
.71376 .48956 .72105 .4984 .73687 .49363 .72964 .48474 Metetra
.656 .751 .92 r
.72964 .48474 .73687 .49363 .75278 .48882 .74562 .47988 Metetra
.656 .751 .92 r
.74562 .47988 .75278 .48882 .76878 .48399 .76168 .475 Metetra
.656 .751 .92 r
.76168 .475 .76878 .48399 .78487 .47914 .77783 .4701 Metetra
.656 .751 .92 r
.77783 .4701 .78487 .47914 .80105 .47425 .79408 .46516 Metetra
.656 .751 .92 r
.79408 .46516 .80105 .47425 .81732 .46934 .81041 .4602 Metetra
.656 .751 .92 r
.81041 .4602 .81732 .46934 .83369 .4644 .82684 .45521 Metetra
.656 .751 .92 r
.82684 .45521 .83369 .4644 .85014 .45943 .84337 .45019 Metetra
.656 .751 .92 r
.84337 .45019 .85014 .45943 .86669 .45444 .85998 .44514 Metetra
.656 .751 .92 r
.85998 .44514 .86669 .45444 .88333 .44942 .8767 .44006 Metetra
.656 .751 .92 r
.8767 .44006 .88333 .44942 .90007 .44436 .89351 .43496 Metetra
.656 .751 .92 r
.89351 .43496 .90007 .44436 .91691 .43928 .91041 .42982 Metetra
.656 .751 .92 r
.29351 .60697 .30231 .61465 .31598 .61049 .30722 .60277 Metetra
.656 .751 .92 r
.30722 .60277 .31598 .61049 .32972 .60631 .32101 .59855 Metetra
.656 .751 .92 r
.32101 .59855 .32972 .60631 .34353 .60211 .33486 .59431 Metetra
.656 .751 .92 r
.33486 .59431 .34353 .60211 .35741 .59789 .34879 .59005 Metetra
.656 .751 .92 r
.34879 .59005 .35741 .59789 .37137 .59365 .36279 .58576 Metetra
.656 .751 .92 r
.36279 .58576 .37137 .59365 .38539 .58938 .37686 .58146 Metetra
.656 .751 .92 r
.37686 .58146 .38539 .58938 .39949 .58509 .39101 .57713 Metetra
.656 .751 .92 r
.39101 .57713 .39949 .58509 .41367 .58078 .40523 .57277 Metetra
.656 .751 .92 r
.40523 .57277 .41367 .58078 .42792 .57645 .41953 .5684 Metetra
.656 .751 .92 r
.41953 .5684 .42792 .57645 .44225 .57209 .4339 .564 Metetra
.656 .751 .92 r
.4339 .564 .44225 .57209 .45665 .56771 .44835 .55957 Metetra
.656 .751 .92 r
.44835 .55957 .45665 .56771 .47113 .56331 .46288 .55513 Metetra
.656 .751 .92 r
.46288 .55513 .47113 .56331 .48569 .55888 .47749 .55066 Metetra
.656 .751 .92 r
.47749 .55066 .48569 .55888 .50032 .55443 .49218 .54616 Metetra
.656 .751 .92 r
.49218 .54616 .50032 .55443 .51503 .54996 .50694 .54165 Metetra
.656 .751 .92 r
.50694 .54165 .51503 .54996 .52983 .54546 .52178 .5371 Metetra
.656 .751 .92 r
.52178 .5371 .52983 .54546 .5447 .54094 .53671 .53254 Metetra
.656 .751 .92 r
.53671 .53254 .5447 .54094 .55965 .5364 .55172 .52795 Metetra
.656 .751 .92 r
.55172 .52795 .55965 .5364 .57469 .53183 .5668 .52333 Metetra
.656 .751 .92 r
.5668 .52333 .57469 .53183 .5898 .52723 .58197 .51869 Metetra
.656 .751 .92 r
.58197 .51869 .5898 .52723 .605 .52261 .59723 .51402 Metetra
.656 .751 .92 r
.59723 .51402 .605 .52261 .62028 .51797 .61257 .50933 Metetra
.656 .751 .92 r
.61257 .50933 .62028 .51797 .63565 .5133 .62799 .50461 Metetra
.656 .751 .92 r
.62799 .50461 .63565 .5133 .6511 .5086 .6435 .49987 Metetra
.656 .751 .92 r
.6435 .49987 .6511 .5086 .66663 .50388 .65909 .4951 Metetra
.656 .751 .92 r
.65909 .4951 .66663 .50388 .68226 .49914 .67477 .4903 Metetra
.656 .751 .92 r
.67477 .4903 .68226 .49914 .69796 .49436 .69054 .48548 Metetra
.656 .751 .92 r
.69054 .48548 .69796 .49436 .71376 .48956 .7064 .48063 Metetra
.656 .751 .92 r
.7064 .48063 .71376 .48956 .72964 .48474 .72234 .47575 Metetra
.656 .751 .92 r
.72234 .47575 .72964 .48474 .74562 .47988 .73838 .47085 Metetra
.656 .751 .92 r
.73838 .47085 .74562 .47988 .76168 .475 .75451 .46592 Metetra
.656 .751 .92 r
.75451 .46592 .76168 .475 .77783 .4701 .77072 .46096 Metetra
.656 .751 .92 r
.77072 .46096 .77783 .4701 .79408 .46516 .78703 .45597 Metetra
.656 .751 .92 r
.78703 .45597 .79408 .46516 .81041 .4602 .80343 .45096 Metetra
.656 .751 .92 r
.80343 .45096 .81041 .4602 .82684 .45521 .81993 .44591 Metetra
.656 .751 .92 r
.81993 .44591 .82684 .45521 .84337 .45019 .83652 .44084 Metetra
.656 .751 .92 r
.83652 .44084 .84337 .45019 .85998 .44514 .85321 .43574 Metetra
.656 .751 .92 r
.85321 .43574 .85998 .44514 .8767 .44006 .86999 .43061 Metetra
.656 .751 .92 r
.86999 .43061 .8767 .44006 .89351 .43496 .88687 .42545 Metetra
.656 .751 .92 r
.88687 .42545 .89351 .43496 .91041 .42982 .90384 .42026 Metetra
.656 .751 .92 r
.28463 .59922 .29351 .60697 .30722 .60277 .29838 .59498 Metetra
.656 .751 .92 r
.29838 .59498 .30722 .60277 .32101 .59855 .31221 .59072 Metetra
.656 .751 .92 r
.31221 .59072 .32101 .59855 .33486 .59431 .32611 .58644 Metetra
.656 .751 .92 r
.32611 .58644 .33486 .59431 .34879 .59005 .34008 .58213 Metetra
.656 .751 .92 r
.34008 .58213 .34879 .59005 .36279 .58576 .35412 .5778 Metetra
.656 .751 .92 r
.35412 .5778 .36279 .58576 .37686 .58146 .36824 .57345 Metetra
.656 .751 .92 r
.36824 .57345 .37686 .58146 .39101 .57713 .38244 .56908 Metetra
.656 .751 .92 r
.38244 .56908 .39101 .57713 .40523 .57277 .39671 .56469 Metetra
.656 .751 .92 r
.39671 .56469 .40523 .57277 .41953 .5684 .41105 .56027 Metetra
.656 .751 .92 r
.41105 .56027 .41953 .5684 .4339 .564 .42548 .55582 Metetra
.656 .751 .92 r
.42548 .55582 .4339 .564 .44835 .55957 .43998 .55136 Metetra
.656 .751 .92 r
.43998 .55136 .44835 .55957 .46288 .55513 .45455 .54686 Metetra
.656 .751 .92 r
.45455 .54686 .46288 .55513 .47749 .55066 .46921 .54235 Metetra
.656 .751 .92 r
.46921 .54235 .47749 .55066 .49218 .54616 .48395 .53781 Metetra
.656 .751 .92 r
.48395 .53781 .49218 .54616 .50694 .54165 .49876 .53325 Metetra
.656 .751 .92 r
.49876 .53325 .50694 .54165 .52178 .5371 .51366 .52866 Metetra
.656 .751 .92 r
.51366 .52866 .52178 .5371 .53671 .53254 .52864 .52405 Metetra
.656 .751 .92 r
.52864 .52405 .53671 .53254 .55172 .52795 .5437 .51941 Metetra
.656 .751 .92 r
.5437 .51941 .55172 .52795 .5668 .52333 .55884 .51474 Metetra
.656 .751 .92 r
.55884 .51474 .5668 .52333 .58197 .51869 .57407 .51006 Metetra
.656 .751 .92 r
.57407 .51006 .58197 .51869 .59723 .51402 .58938 .50534 Metetra
.656 .751 .92 r
.58938 .50534 .59723 .51402 .61257 .50933 .60477 .5006 Metetra
.656 .751 .92 r
.60477 .5006 .61257 .50933 .62799 .50461 .62025 .49584 Metetra
.656 .751 .92 r
.62025 .49584 .62799 .50461 .6435 .49987 .63582 .49104 Metetra
.656 .751 .92 r
.63582 .49104 .6435 .49987 .65909 .4951 .65147 .48622 Metetra
.656 .751 .92 r
.65147 .48622 .65909 .4951 .67477 .4903 .66721 .48138 Metetra
.656 .751 .92 r
.66721 .48138 .67477 .4903 .69054 .48548 .68304 .47651 Metetra
.656 .751 .92 r
.68304 .47651 .69054 .48548 .7064 .48063 .69896 .47161 Metetra
.656 .751 .92 r
.69896 .47161 .7064 .48063 .72234 .47575 .71497 .46668 Metetra
.656 .751 .92 r
.71497 .46668 .72234 .47575 .73838 .47085 .73107 .46172 Metetra
.656 .751 .92 r
.73107 .46172 .73838 .47085 .75451 .46592 .74726 .45674 Metetra
.656 .751 .92 r
.74726 .45674 .75451 .46592 .77072 .46096 .76354 .45173 Metetra
.656 .751 .92 r
.76354 .45173 .77072 .46096 .78703 .45597 .77991 .44669 Metetra
.656 .751 .92 r
.77991 .44669 .78703 .45597 .80343 .45096 .79638 .44162 Metetra
.656 .751 .92 r
.79638 .44162 .80343 .45096 .81993 .44591 .81294 .43652 Metetra
.656 .751 .92 r
.81294 .43652 .81993 .44591 .83652 .44084 .8296 .4314 Metetra
.656 .751 .92 r
.8296 .4314 .83652 .44084 .85321 .43574 .84636 .42624 Metetra
.656 .751 .92 r
.84636 .42624 .85321 .43574 .86999 .43061 .86321 .42106 Metetra
.656 .751 .92 r
.86321 .42106 .86999 .43061 .88687 .42545 .88016 .41584 Metetra
.656 .751 .92 r
.88016 .41584 .88687 .42545 .90384 .42026 .89721 .41059 Metetra
.656 .752 .92 r
.27566 .59139 .28463 .59922 .29838 .59498 .28946 .58711 Metetra
.656 .751 .92 r
.28946 .58711 .29838 .59498 .31221 .59072 .30333 .58281 Metetra
.656 .751 .92 r
.30333 .58281 .31221 .59072 .32611 .58644 .31727 .57849 Metetra
.656 .751 .92 r
.31727 .57849 .32611 .58644 .34008 .58213 .33128 .57414 Metetra
.656 .751 .92 r
.33128 .57414 .34008 .58213 .35412 .5778 .34538 .56977 Metetra
.656 .751 .92 r
.34538 .56977 .35412 .5778 .36824 .57345 .35954 .56538 Metetra
.656 .751 .92 r
.35954 .56538 .36824 .57345 .38244 .56908 .37378 .56096 Metetra
.656 .751 .92 r
.37378 .56096 .38244 .56908 .39671 .56469 .3881 .55652 Metetra
.656 .751 .92 r
.3881 .55652 .39671 .56469 .41105 .56027 .40249 .55205 Metetra
.656 .751 .92 r
.40249 .55205 .41105 .56027 .42548 .55582 .41696 .54757 Metetra
.656 .751 .92 r
.41696 .54757 .42548 .55582 .43998 .55136 .43151 .54306 Metetra
.656 .751 .92 r
.43151 .54306 .43998 .55136 .45455 .54686 .44614 .53852 Metetra
.656 .751 .92 r
.44614 .53852 .45455 .54686 .46921 .54235 .46085 .53396 Metetra
.656 .751 .92 r
.46085 .53396 .46921 .54235 .48395 .53781 .47564 .52937 Metetra
.656 .751 .92 r
.47564 .52937 .48395 .53781 .49876 .53325 .49051 .52476 Metetra
.656 .751 .92 r
.49051 .52476 .49876 .53325 .51366 .52866 .50546 .52013 Metetra
.656 .751 .92 r
.50546 .52013 .51366 .52866 .52864 .52405 .52049 .51547 Metetra
.656 .751 .92 r
.52049 .51547 .52864 .52405 .5437 .51941 .5356 .51078 Metetra
.656 .751 .92 r
.5356 .51078 .5437 .51941 .55884 .51474 .5508 .50607 Metetra
.656 .751 .92 r
.5508 .50607 .55884 .51474 .57407 .51006 .56608 .50134 Metetra
.656 .751 .92 r
.56608 .50134 .57407 .51006 .58938 .50534 .58145 .49657 Metetra
.656 .751 .92 r
.58145 .49657 .58938 .50534 .60477 .5006 .5969 .49178 Metetra
.656 .751 .92 r
.5969 .49178 .60477 .5006 .62025 .49584 .61244 .48697 Metetra
.656 .751 .92 r
.61244 .48697 .62025 .49584 .63582 .49104 .62806 .48213 Metetra
.656 .751 .92 r
.62806 .48213 .63582 .49104 .65147 .48622 .64377 .47726 Metetra
.656 .751 .92 r
.64377 .47726 .65147 .48622 .66721 .48138 .65957 .47236 Metetra
.656 .751 .92 r
.65957 .47236 .66721 .48138 .68304 .47651 .67546 .46744 Metetra
.656 .751 .92 r
.67546 .46744 .68304 .47651 .69896 .47161 .69144 .46249 Metetra
.656 .751 .92 r
.69144 .46249 .69896 .47161 .71497 .46668 .70751 .45751 Metetra
.656 .751 .92 r
.70751 .45751 .71497 .46668 .73107 .46172 .72368 .4525 Metetra
.656 .751 .92 r
.72368 .4525 .73107 .46172 .74726 .45674 .73993 .44746 Metetra
.656 .751 .92 r
.73993 .44746 .74726 .45674 .76354 .45173 .75628 .4424 Metetra
.656 .751 .92 r
.75628 .4424 .76354 .45173 .77991 .44669 .77272 .43731 Metetra
.656 .751 .92 r
.77272 .43731 .77991 .44669 .79638 .44162 .78925 .43218 Metetra
.656 .751 .92 r
.78925 .43218 .79638 .44162 .81294 .43652 .80588 .42703 Metetra
.656 .751 .92 r
.80588 .42703 .81294 .43652 .8296 .4314 .82261 .42185 Metetra
.656 .751 .92 r
.82261 .42185 .8296 .4314 .84636 .42624 .83943 .41664 Metetra
.656 .751 .92 r
.83943 .41664 .84636 .42624 .86321 .42106 .85636 .4114 Metetra
.656 .751 .92 r
.85636 .4114 .86321 .42106 .88016 .41584 .87338 .40613 Metetra
.656 .751 .92 r
.87338 .40613 .88016 .41584 .89721 .41059 .8905 .40082 Metetra
.656 .752 .92 r
.2666 .58349 .27566 .59139 .28946 .58711 .28044 .57917 Metetra
.656 .752 .92 r
.28044 .57917 .28946 .58711 .30333 .58281 .29436 .57482 Metetra
.656 .752 .92 r
.29436 .57482 .30333 .58281 .31727 .57849 .30834 .57046 Metetra
.656 .751 .92 r
.30834 .57046 .31727 .57849 .33128 .57414 .3224 .56607 Metetra
.656 .751 .92 r
.3224 .56607 .33128 .57414 .34538 .56977 .33654 .56165 Metetra
.656 .751 .92 r
.33654 .56165 .34538 .56977 .35954 .56538 .35075 .55722 Metetra
.656 .751 .92 r
.35075 .55722 .35954 .56538 .37378 .56096 .36504 .55276 Metetra
.656 .751 .92 r
.36504 .55276 .37378 .56096 .3881 .55652 .37941 .54827 Metetra
.656 .751 .92 r
.37941 .54827 .3881 .55652 .40249 .55205 .39385 .54376 Metetra
.656 .751 .92 r
.39385 .54376 .40249 .55205 .41696 .54757 .40837 .53923 Metetra
.656 .751 .92 r
.40837 .53923 .41696 .54757 .43151 .54306 .42297 .53467 Metetra
.656 .751 .92 r
.42297 .53467 .43151 .54306 .44614 .53852 .43765 .53009 Metetra
.656 .751 .92 r
.43765 .53009 .44614 .53852 .46085 .53396 .45241 .52548 Metetra
.656 .751 .92 r
.45241 .52548 .46085 .53396 .47564 .52937 .46724 .52085 Metetra
.656 .751 .92 r
.46724 .52085 .47564 .52937 .49051 .52476 .48216 .5162 Metetra
.656 .751 .92 r
.48216 .5162 .49051 .52476 .50546 .52013 .49717 .51151 Metetra
.656 .751 .92 r
.49717 .51151 .50546 .52013 .52049 .51547 .51225 .50681 Metetra
.656 .751 .92 r
.51225 .50681 .52049 .51547 .5356 .51078 .52742 .50207 Metetra
.656 .751 .92 r
.52742 .50207 .5356 .51078 .5508 .50607 .54267 .49731 Metetra
.656 .751 .92 r
.54267 .49731 .5508 .50607 .56608 .50134 .55801 .49253 Metetra
.656 .751 .92 r
.55801 .49253 .56608 .50134 .58145 .49657 .57343 .48771 Metetra
.656 .751 .92 r
.57343 .48771 .58145 .49657 .5969 .49178 .58894 .48288 Metetra
.656 .751 .92 r
.58894 .48288 .5969 .49178 .61244 .48697 .60454 .47801 Metetra
.656 .751 .92 r
.60454 .47801 .61244 .48697 .62806 .48213 .62022 .47312 Metetra
.656 .751 .92 r
.62022 .47312 .62806 .48213 .64377 .47726 .63599 .4682 Metetra
.656 .751 .92 r
.63599 .4682 .64377 .47726 .65957 .47236 .65186 .46325 Metetra
.656 .751 .92 r
.65186 .46325 .65957 .47236 .67546 .46744 .66781 .45827 Metetra
.656 .751 .92 r
.66781 .45827 .67546 .46744 .69144 .46249 .68385 .45327 Metetra
.656 .751 .92 r
.68385 .45327 .69144 .46249 .70751 .45751 .69998 .44824 Metetra
.656 .751 .92 r
.69998 .44824 .70751 .45751 .72368 .4525 .71621 .44318 Metetra
.656 .751 .92 r
.71621 .44318 .72368 .4525 .73993 .44746 .73252 .43809 Metetra
.656 .751 .92 r
.73252 .43809 .73993 .44746 .75628 .4424 .74894 .43297 Metetra
.656 .751 .92 r
.74894 .43297 .75628 .4424 .77272 .43731 .76544 .42782 Metetra
.656 .751 .92 r
.76544 .42782 .77272 .43731 .78925 .43218 .78205 .42265 Metetra
.656 .751 .92 r
.78205 .42265 .78925 .43218 .80588 .42703 .79875 .41744 Metetra
.656 .751 .92 r
.79875 .41744 .80588 .42703 .82261 .42185 .81554 .4122 Metetra
.656 .751 .92 r
.81554 .4122 .82261 .42185 .83943 .41664 .83244 .40693 Metetra
.656 .751 .92 r
.83244 .40693 .83943 .41664 .85636 .4114 .84943 .40164 Metetra
.656 .751 .92 r
.84943 .40164 .85636 .4114 .87338 .40613 .86652 .39631 Metetra
.656 .751 .92 r
.86652 .39631 .87338 .40613 .8905 .40082 .88371 .39095 Metetra
.656 .752 .92 r
.25746 .57551 .2666 .58349 .28044 .57917 .27134 .57115 Metetra
.656 .752 .92 r
.27134 .57115 .28044 .57917 .29436 .57482 .2853 .56676 Metetra
.656 .752 .92 r
.2853 .56676 .29436 .57482 .30834 .57046 .29933 .56235 Metetra
.656 .752 .92 r
.29933 .56235 .30834 .57046 .3224 .56607 .31344 .55792 Metetra
.656 .752 .92 r
.31344 .55792 .3224 .56607 .33654 .56165 .32762 .55346 Metetra
.656 .751 .92 r
.32762 .55346 .33654 .56165 .35075 .55722 .34188 .54898 Metetra
.656 .751 .92 r
.34188 .54898 .35075 .55722 .36504 .55276 .35621 .54447 Metetra
.656 .751 .92 r
.35621 .54447 .36504 .55276 .37941 .54827 .37063 .53994 Metetra
.656 .751 .92 r
.37063 .53994 .37941 .54827 .39385 .54376 .38512 .53539 Metetra
.656 .751 .92 r
.38512 .53539 .39385 .54376 .40837 .53923 .39969 .53081 Metetra
.656 .751 .92 r
.39969 .53081 .40837 .53923 .42297 .53467 .41434 .52621 Metetra
.656 .751 .92 r
.41434 .52621 .42297 .53467 .43765 .53009 .42906 .52158 Metetra
.656 .751 .92 r
.42906 .52158 .43765 .53009 .45241 .52548 .44387 .51692 Metetra
.656 .751 .92 r
.44387 .51692 .45241 .52548 .46724 .52085 .45877 .51224 Metetra
.656 .751 .92 r
.45877 .51224 .46724 .52085 .48216 .5162 .47374 .50754 Metetra
.656 .751 .92 r
.47374 .50754 .48216 .5162 .49717 .51151 .48879 .50281 Metetra
.656 .751 .92 r
.48879 .50281 .49717 .51151 .51225 .50681 .50393 .49805 Metetra
.656 .751 .92 r
.50393 .49805 .51225 .50681 .52742 .50207 .51916 .49327 Metetra
.656 .751 .92 r
.51916 .49327 .52742 .50207 .54267 .49731 .53446 .48846 Metetra
.656 .751 .92 r
.53446 .48846 .54267 .49731 .55801 .49253 .54986 .48363 Metetra
.656 .751 .92 r
.54986 .48363 .55801 .49253 .57343 .48771 .56534 .47876 Metetra
.656 .751 .92 r
.56534 .47876 .57343 .48771 .58894 .48288 .5809 .47387 Metetra
.656 .751 .92 r
.5809 .47387 .58894 .48288 .60454 .47801 .59656 .46896 Metetra
.656 .751 .92 r
.59656 .46896 .60454 .47801 .62022 .47312 .6123 .46401 Metetra
.656 .751 .92 r
.6123 .46401 .62022 .47312 .63599 .4682 .62813 .45904 Metetra
.656 .751 .92 r
.62813 .45904 .63599 .4682 .65186 .46325 .64406 .45404 Metetra
.656 .751 .92 r
.64406 .45404 .65186 .46325 .66781 .45827 .66007 .44901 Metetra
.656 .751 .92 r
.66007 .44901 .66781 .45827 .68385 .45327 .67617 .44396 Metetra
.656 .751 .92 r
.67617 .44396 .68385 .45327 .69998 .44824 .69237 .43887 Metetra
.656 .751 .92 r
.69237 .43887 .69998 .44824 .71621 .44318 .70866 .43376 Metetra
.656 .751 .92 r
.70866 .43376 .71621 .44318 .73252 .43809 .72504 .42861 Metetra
.656 .751 .92 r
.72504 .42861 .73252 .43809 .74894 .43297 .74152 .42344 Metetra
.656 .751 .92 r
.74152 .42344 .74894 .43297 .76544 .42782 .75809 .41824 Metetra
.656 .751 .92 r
.75809 .41824 .76544 .42782 .78205 .42265 .77476 .41301 Metetra
.656 .751 .92 r
.77476 .41301 .78205 .42265 .79875 .41744 .79153 .40774 Metetra
.656 .751 .92 r
.79153 .40774 .79875 .41744 .81554 .4122 .8084 .40245 Metetra
.656 .751 .92 r
.8084 .40245 .81554 .4122 .83244 .40693 .82536 .39712 Metetra
.656 .751 .92 r
.82536 .39712 .83244 .40693 .84943 .40164 .84243 .39177 Metetra
.656 .751 .92 r
.84243 .39177 .84943 .40164 .86652 .39631 .85959 .38638 Metetra
.656 .751 .92 r
.85959 .38638 .86652 .39631 .88371 .39095 .87686 .38096 Metetra
.656 .752 .92 r
.24822 .56746 .25746 .57551 .27134 .57115 .26215 .56305 Metetra
.656 .752 .92 r
.26215 .56305 .27134 .57115 .2853 .56676 .27615 .55862 Metetra
.656 .752 .92 r
.27615 .55862 .2853 .56676 .29933 .56235 .29023 .55416 Metetra
.656 .752 .92 r
.29023 .55416 .29933 .56235 .31344 .55792 .30438 .54969 Metetra
.656 .752 .92 r
.30438 .54969 .31344 .55792 .32762 .55346 .31861 .54518 Metetra
.656 .752 .92 r
.31861 .54518 .32762 .55346 .34188 .54898 .33291 .54066 Metetra
.656 .751 .92 r
.33291 .54066 .34188 .54898 .35621 .54447 .3473 .5361 Metetra
.656 .751 .92 r
.3473 .5361 .35621 .54447 .37063 .53994 .36176 .53153 Metetra
.656 .751 .92 r
.36176 .53153 .37063 .53994 .38512 .53539 .3763 .52693 Metetra
.656 .751 .92 r
.3763 .52693 .38512 .53539 .39969 .53081 .39092 .5223 Metetra
.656 .751 .92 r
.39092 .5223 .39969 .53081 .41434 .52621 .40562 .51765 Metetra
.656 .751 .92 r
.40562 .51765 .41434 .52621 .42906 .52158 .4204 .51298 Metetra
.656 .751 .92 r
.4204 .51298 .42906 .52158 .44387 .51692 .43526 .50827 Metetra
.656 .751 .92 r
.43526 .50827 .44387 .51692 .45877 .51224 .4502 .50355 Metetra
.656 .751 .92 r
.4502 .50355 .45877 .51224 .47374 .50754 .46523 .49879 Metetra
.656 .751 .92 r
.46523 .49879 .47374 .50754 .48879 .50281 .48034 .49402 Metetra
.656 .751 .92 r
.48034 .49402 .48879 .50281 .50393 .49805 .49553 .48921 Metetra
.656 .751 .92 r
.49553 .48921 .50393 .49805 .51916 .49327 .51081 .48438 Metetra
.656 .751 .92 r
.51081 .48438 .51916 .49327 .53446 .48846 .52617 .47952 Metetra
.656 .751 .92 r
.52617 .47952 .53446 .48846 .54986 .48363 .54162 .47463 Metetra
.656 .751 .92 r
.54162 .47463 .54986 .48363 .56534 .47876 .55716 .46972 Metetra
.656 .751 .92 r
.55716 .46972 .56534 .47876 .5809 .47387 .57278 .46478 Metetra
.656 .751 .92 r
.57278 .46478 .5809 .47387 .59656 .46896 .5885 .45981 Metetra
.656 .751 .92 r
.5885 .45981 .59656 .46896 .6123 .46401 .6043 .45481 Metetra
.656 .751 .92 r
.6043 .45481 .6123 .46401 .62813 .45904 .62019 .44979 Metetra
.656 .751 .92 r
.62019 .44979 .62813 .45904 .64406 .45404 .63617 .44474 Metetra
.656 .751 .92 r
.63617 .44474 .64406 .45404 .66007 .44901 .65225 .43966 Metetra
.656 .751 .92 r
.65225 .43966 .66007 .44901 .67617 .44396 .66842 .43454 Metetra
.656 .751 .92 r
.66842 .43454 .67617 .44396 .69237 .43887 .68468 .42941 Metetra
.656 .751 .92 r
.68468 .42941 .69237 .43887 .70866 .43376 .70103 .42424 Metetra
.656 .751 .92 r
.70103 .42424 .70866 .43376 .72504 .42861 .71748 .41904 Metetra
.656 .751 .92 r
.71748 .41904 .72504 .42861 .74152 .42344 .73402 .41381 Metetra
.656 .751 .92 r
.73402 .41381 .74152 .42344 .75809 .41824 .75067 .40855 Metetra
.656 .751 .92 r
.75067 .40855 .75809 .41824 .77476 .41301 .7674 .40326 Metetra
.656 .751 .92 r
.7674 .40326 .77476 .41301 .79153 .40774 .78424 .39794 Metetra
.656 .751 .92 r
.78424 .39794 .79153 .40774 .8084 .40245 .80118 .39259 Metetra
.656 .751 .92 r
.80118 .39259 .8084 .40245 .82536 .39712 .81821 .38721 Metetra
.656 .751 .92 r
.81821 .38721 .82536 .39712 .84243 .39177 .83535 .38179 Metetra
.656 .751 .92 r
.83535 .38179 .84243 .39177 .85959 .38638 .85259 .37635 Metetra
.656 .751 .92 r
.85259 .37635 .85959 .38638 .87686 .38096 .86993 .37087 Metetra
.656 .752 .92 r
.2389 .55932 .24822 .56746 .26215 .56305 .25287 .55487 Metetra
.656 .752 .92 r
.25287 .55487 .26215 .56305 .27615 .55862 .26691 .5504 Metetra
.656 .752 .92 r
.26691 .5504 .27615 .55862 .29023 .55416 .28103 .5459 Metetra
.656 .752 .92 r
.28103 .5459 .29023 .55416 .30438 .54969 .29523 .54137 Metetra
.656 .752 .92 r
.29523 .54137 .30438 .54969 .31861 .54518 .30951 .53682 Metetra
.656 .752 .92 r
.30951 .53682 .31861 .54518 .33291 .54066 .32386 .53225 Metetra
.656 .752 .92 r
.32386 .53225 .33291 .54066 .3473 .5361 .33829 .52765 Metetra
.656 .752 .92 r
.33829 .52765 .3473 .5361 .36176 .53153 .3528 .52303 Metetra
.656 .751 .92 r
.3528 .52303 .36176 .53153 .3763 .52693 .36739 .51838 Metetra
.656 .751 .92 r
.36739 .51838 .3763 .52693 .39092 .5223 .38206 .51371 Metetra
.656 .751 .92 r
.38206 .51371 .39092 .5223 .40562 .51765 .39681 .50901 Metetra
.656 .751 .92 r
.39681 .50901 .40562 .51765 .4204 .51298 .41164 .50429 Metetra
.656 .751 .92 r
.41164 .50429 .4204 .51298 .43526 .50827 .42655 .49954 Metetra
.656 .751 .92 r
.42655 .49954 .43526 .50827 .4502 .50355 .44155 .49476 Metetra
.656 .751 .92 r
.44155 .49476 .4502 .50355 .46523 .49879 .45663 .48996 Metetra
.656 .751 .92 r
.45663 .48996 .46523 .49879 .48034 .49402 .47179 .48513 Metetra
.656 .751 .92 r
.47179 .48513 .48034 .49402 .49553 .48921 .48704 .48028 Metetra
.656 .751 .92 r
.48704 .48028 .49553 .48921 .51081 .48438 .50237 .47539 Metetra
.656 .751 .92 r
.50237 .47539 .51081 .48438 .52617 .47952 .51779 .47048 Metetra
.656 .751 .92 r
.51779 .47048 .52617 .47952 .54162 .47463 .5333 .46555 Metetra
.656 .751 .92 r
.5333 .46555 .54162 .47463 .55716 .46972 .54889 .46058 Metetra
.656 .751 .92 r
.54889 .46058 .55716 .46972 .57278 .46478 .56458 .45559 Metetra
.656 .751 .92 r
.56458 .45559 .57278 .46478 .5885 .45981 .58035 .45057 Metetra
.656 .751 .92 r
.58035 .45057 .5885 .45981 .6043 .45481 .59621 .44552 Metetra
.656 .751 .92 r
.59621 .44552 .6043 .45481 .62019 .44979 .61216 .44044 Metetra
.656 .751 .92 r
.61216 .44044 .62019 .44979 .63617 .44474 .62821 .43533 Metetra
.656 .751 .92 r
.62821 .43533 .63617 .44474 .65225 .43966 .64435 .4302 Metetra
.656 .751 .92 r
.64435 .4302 .65225 .43966 .66842 .43454 .66058 .42503 Metetra
.656 .751 .92 r
.66058 .42503 .66842 .43454 .68468 .42941 .6769 .41984 Metetra
.656 .751 .92 r
.6769 .41984 .68468 .42941 .70103 .42424 .69332 .41461 Metetra
.656 .751 .92 r
.69332 .41461 .70103 .42424 .71748 .41904 .70984 .40936 Metetra
.656 .751 .92 r
.70984 .40936 .71748 .41904 .73402 .41381 .72645 .40407 Metetra
.656 .751 .92 r
.72645 .40407 .73402 .41381 .75067 .40855 .74316 .39876 Metetra
.656 .751 .92 r
.74316 .39876 .75067 .40855 .7674 .40326 .75996 .39341 Metetra
.656 .751 .92 r
.75996 .39341 .7674 .40326 .78424 .39794 .77687 .38803 Metetra
.656 .751 .92 r
.77687 .38803 .78424 .39794 .80118 .39259 .79388 .38262 Metetra
.656 .751 .92 r
.79388 .38262 .80118 .39259 .81821 .38721 .81098 .37718 Metetra
.656 .751 .92 r
.81098 .37718 .81821 .38721 .83535 .38179 .82819 .37171 Metetra
.656 .751 .92 r
.82819 .37171 .83535 .38179 .85259 .37635 .8455 .3662 Metetra
.656 .751 .92 r
.8455 .3662 .85259 .37635 .86993 .37087 .86292 .36066 Metetra
.656 .752 .92 r
.22948 .55111 .2389 .55932 .25287 .55487 .24349 .54661 Metetra
.656 .752 .92 r
.24349 .54661 .25287 .55487 .26691 .5504 .25758 .54209 Metetra
.656 .752 .92 r
.25758 .54209 .26691 .5504 .28103 .5459 .27175 .53755 Metetra
.656 .752 .92 r
.27175 .53755 .28103 .5459 .29523 .54137 .28599 .53298 Metetra
.656 .752 .92 r
.28599 .53298 .29523 .54137 .30951 .53682 .30032 .52838 Metetra
.656 .752 .92 r
.30032 .52838 .30951 .53682 .32386 .53225 .31472 .52376 Metetra
.656 .752 .92 r
.31472 .52376 .32386 .53225 .33829 .52765 .32919 .51912 Metetra
.656 .752 .92 r
.32919 .51912 .33829 .52765 .3528 .52303 .34375 .51445 Metetra
.656 .752 .92 r
.34375 .51445 .3528 .52303 .36739 .51838 .35839 .50975 Metetra
.656 .751 .92 r
.35839 .50975 .36739 .51838 .38206 .51371 .37311 .50503 Metetra
.656 .751 .92 r
.37311 .50503 .38206 .51371 .39681 .50901 .38791 .50029 Metetra
.656 .751 .92 r
.38791 .50029 .39681 .50901 .41164 .50429 .40279 .49551 Metetra
.656 .751 .92 r
.40279 .49551 .41164 .50429 .42655 .49954 .41776 .49071 Metetra
.656 .751 .92 r
.41776 .49071 .42655 .49954 .44155 .49476 .4328 .48589 Metetra
.656 .751 .92 r
.4328 .48589 .44155 .49476 .45663 .48996 .44794 .48103 Metetra
.656 .751 .92 r
.44794 .48103 .45663 .48996 .47179 .48513 .46315 .47616 Metetra
.656 .751 .92 r
.46315 .47616 .47179 .48513 .48704 .48028 .47846 .47125 Metetra
.656 .751 .92 r
.47846 .47125 .48704 .48028 .50237 .47539 .49385 .46631 Metetra
.656 .751 .92 r
.49385 .46631 .50237 .47539 .51779 .47048 .50932 .46135 Metetra
.656 .751 .92 r
.50932 .46135 .51779 .47048 .5333 .46555 .52489 .45636 Metetra
.656 .751 .92 r
.52489 .45636 .5333 .46555 .54889 .46058 .54054 .45135 Metetra
.656 .751 .92 r
.54054 .45135 .54889 .46058 .56458 .45559 .55628 .4463 Metetra
.656 .751 .92 r
.55628 .4463 .56458 .45559 .58035 .45057 .57211 .44123 Metetra
.656 .751 .92 r
.57211 .44123 .58035 .45057 .59621 .44552 .58804 .43612 Metetra
.656 .751 .92 r
.58804 .43612 .59621 .44552 .61216 .44044 .60405 .43099 Metetra
.656 .751 .92 r
.60405 .43099 .61216 .44044 .62821 .43533 .62016 .42583 Metetra
.656 .751 .92 r
.62016 .42583 .62821 .43533 .64435 .4302 .63636 .42064 Metetra
.656 .751 .92 r
.63636 .42064 .64435 .4302 .66058 .42503 .65265 .41542 Metetra
.656 .751 .92 r
.65265 .41542 .66058 .42503 .6769 .41984 .66904 .41017 Metetra
.656 .751 .92 r
.66904 .41017 .6769 .41984 .69332 .41461 .68553 .40488 Metetra
.656 .751 .92 r
.68553 .40488 .69332 .41461 .70984 .40936 .70211 .39957 Metetra
.656 .751 .92 r
.70211 .39957 .70984 .40936 .72645 .40407 .71879 .39423 Metetra
.656 .751 .92 r
.71879 .39423 .72645 .40407 .74316 .39876 .73556 .38886 Metetra
.656 .751 .92 r
.73556 .38886 .74316 .39876 .75996 .39341 .75244 .38345 Metetra
.656 .751 .92 r
.75244 .38345 .75996 .39341 .77687 .38803 .76942 .37801 Metetra
.656 .751 .92 r
.76942 .37801 .77687 .38803 .79388 .38262 .78649 .37255 Metetra
.656 .751 .92 r
.78649 .37255 .79388 .38262 .81098 .37718 .80367 .36704 Metetra
.656 .751 .92 r
.80367 .36704 .81098 .37718 .82819 .37171 .82096 .36151 Metetra
.656 .751 .92 r
.82096 .36151 .82819 .37171 .8455 .3662 .83834 .35594 Metetra
.656 .751 .92 r
.83834 .35594 .8455 .3662 .86292 .36066 .85583 .35034 Metetra
.656 .752 .92 r
.21996 .54281 .22948 .55111 .24349 .54661 .23402 .53827 Metetra
.656 .752 .92 r
.23402 .53827 .24349 .54661 .25758 .54209 .24816 .5337 Metetra
.656 .752 .92 r
.24816 .5337 .25758 .54209 .27175 .53755 .26237 .52911 Metetra
.656 .752 .92 r
.26237 .52911 .27175 .53755 .28599 .53298 .27666 .5245 Metetra
.656 .752 .92 r
.27666 .5245 .28599 .53298 .30032 .52838 .29103 .51986 Metetra
.656 .752 .92 r
.29103 .51986 .30032 .52838 .31472 .52376 .30548 .51519 Metetra
.656 .752 .92 r
.30548 .51519 .31472 .52376 .32919 .51912 .32 .5105 Metetra
.656 .752 .92 r
.32 .5105 .32919 .51912 .34375 .51445 .33461 .50578 Metetra
.656 .752 .92 r
.33461 .50578 .34375 .51445 .35839 .50975 .3493 .50104 Metetra
.656 .752 .92 r
.3493 .50104 .35839 .50975 .37311 .50503 .36407 .49627 Metetra
.656 .751 .92 r
.36407 .49627 .37311 .50503 .38791 .50029 .37892 .49147 Metetra
.656 .751 .92 r
.37892 .49147 .38791 .50029 .40279 .49551 .39385 .48665 Metetra
.656 .751 .92 r
.39385 .48665 .40279 .49551 .41776 .49071 .40887 .4818 Metetra
.656 .751 .92 r
.40887 .4818 .41776 .49071 .4328 .48589 .42397 .47692 Metetra
.656 .751 .92 r
.42397 .47692 .4328 .48589 .44794 .48103 .43916 .47202 Metetra
.656 .751 .92 r
.43916 .47202 .44794 .48103 .46315 .47616 .45443 .46709 Metetra
.656 .751 .92 r
.45443 .46709 .46315 .47616 .47846 .47125 .46979 .46213 Metetra
.656 .751 .92 r
.46979 .46213 .47846 .47125 .49385 .46631 .48523 .45714 Metetra
.656 .751 .92 r
.48523 .45714 .49385 .46631 .50932 .46135 .50077 .45213 Metetra
.656 .751 .92 r
.50077 .45213 .50932 .46135 .52489 .45636 .51639 .44708 Metetra
.656 .751 .92 r
.51639 .44708 .52489 .45636 .54054 .45135 .5321 .44201 Metetra
.656 .751 .92 r
.5321 .44201 .54054 .45135 .55628 .4463 .5479 .43691 Metetra
.656 .751 .92 r
.5479 .43691 .55628 .4463 .57211 .44123 .56379 .43178 Metetra
.656 .751 .92 r
.56379 .43178 .57211 .44123 .58804 .43612 .57978 .42663 Metetra
.656 .751 .92 r
.57978 .42663 .58804 .43612 .60405 .43099 .59585 .42144 Metetra
.656 .751 .92 r
.59585 .42144 .60405 .43099 .62016 .42583 .61202 .41622 Metetra
.656 .751 .92 r
.61202 .41622 .62016 .42583 .63636 .42064 .62828 .41097 Metetra
.656 .751 .92 r
.62828 .41097 .63636 .42064 .65265 .41542 .64464 .4057 Metetra
.656 .751 .92 r
.64464 .4057 .65265 .41542 .66904 .41017 .6611 .40039 Metetra
.656 .751 .92 r
.6611 .40039 .66904 .41017 .68553 .40488 .67765 .39505 Metetra
.656 .751 .92 r
.67765 .39505 .68553 .40488 .70211 .39957 .6943 .38968 Metetra
.656 .751 .92 r
.6943 .38968 .70211 .39957 .71879 .39423 .71104 .38428 Metetra
.656 .751 .92 r
.71104 .38428 .71879 .39423 .73556 .38886 .72789 .37885 Metetra
.656 .751 .92 r
.72789 .37885 .73556 .38886 .75244 .38345 .74484 .37338 Metetra
.656 .751 .92 r
.74484 .37338 .75244 .38345 .76942 .37801 .76188 .36789 Metetra
.656 .751 .92 r
.76188 .36789 .76942 .37801 .78649 .37255 .77903 .36236 Metetra
.656 .751 .92 r
.77903 .36236 .78649 .37255 .80367 .36704 .79628 .35679 Metetra
.656 .751 .92 r
.79628 .35679 .80367 .36704 .82096 .36151 .81364 .3512 Metetra
.656 .751 .92 r
.81364 .3512 .82096 .36151 .83834 .35594 .8311 .34557 Metetra
.656 .751 .92 r
.8311 .34557 .83834 .35594 .85583 .35034 .84867 .33991 Metetra
.656 .752 .92 r
.21035 .53443 .21996 .54281 .23402 .53827 .22446 .52985 Metetra
.656 .752 .92 r
.22446 .52985 .23402 .53827 .24816 .5337 .23864 .52524 Metetra
.656 .752 .92 r
.23864 .52524 .24816 .5337 .26237 .52911 .2529 .5206 Metetra
.656 .752 .92 r
.2529 .5206 .26237 .52911 .27666 .5245 .26724 .51593 Metetra
.656 .752 .92 r
.26724 .51593 .27666 .5245 .29103 .51986 .28165 .51125 Metetra
.656 .752 .92 r
.28165 .51125 .29103 .51986 .30548 .51519 .29615 .50653 Metetra
.656 .752 .92 r
.29615 .50653 .30548 .51519 .32 .5105 .31072 .50179 Metetra
.656 .752 .92 r
.31072 .50179 .32 .5105 .33461 .50578 .32538 .49702 Metetra
.656 .752 .92 r
.32538 .49702 .33461 .50578 .3493 .50104 .34011 .49223 Metetra
.656 .752 .92 r
.34011 .49223 .3493 .50104 .36407 .49627 .35493 .48741 Metetra
.656 .752 .92 r
.35493 .48741 .36407 .49627 .37892 .49147 .36984 .48256 Metetra
.656 .751 .92 r
.36984 .48256 .37892 .49147 .39385 .48665 .38482 .47769 Metetra
.656 .751 .92 r
.38482 .47769 .39385 .48665 .40887 .4818 .39989 .47279 Metetra
.656 .751 .92 r
.39989 .47279 .40887 .4818 .42397 .47692 .41505 .46786 Metetra
.656 .751 .92 r
.41505 .46786 .42397 .47692 .43916 .47202 .43029 .46291 Metetra
.656 .751 .92 r
.43029 .46291 .43916 .47202 .45443 .46709 .44561 .45792 Metetra
.656 .751 .92 r
.44561 .45792 .45443 .46709 .46979 .46213 .46103 .45291 Metetra
.656 .751 .92 r
.46103 .45291 .46979 .46213 .48523 .45714 .47653 .44787 Metetra
.656 .751 .92 r
.47653 .44787 .48523 .45714 .50077 .45213 .49212 .4428 Metetra
.656 .751 .92 r
.49212 .4428 .50077 .45213 .51639 .44708 .5078 .43771 Metetra
.656 .751 .92 r
.5078 .43771 .51639 .44708 .5321 .44201 .52357 .43258 Metetra
.656 .751 .92 r
.52357 .43258 .5321 .44201 .5479 .43691 .53943 .42743 Metetra
.656 .751 .92 r
.53943 .42743 .5479 .43691 .56379 .43178 .55538 .42224 Metetra
.656 .751 .92 r
.55538 .42224 .56379 .43178 .57978 .42663 .57143 .41703 Metetra
.656 .751 .92 r
.57143 .41703 .57978 .42663 .59585 .42144 .58756 .41178 Metetra
.656 .751 .92 r
.58756 .41178 .59585 .42144 .61202 .41622 .6038 .40651 Metetra
.656 .751 .92 r
.6038 .40651 .61202 .41622 .62828 .41097 .62012 .40121 Metetra
.656 .751 .92 r
.62012 .40121 .62828 .41097 .64464 .4057 .63655 .39587 Metetra
.656 .751 .92 r
.63655 .39587 .64464 .4057 .6611 .40039 .65307 .39051 Metetra
.656 .751 .92 r
.65307 .39051 .6611 .40039 .67765 .39505 .66969 .38511 Metetra
.656 .751 .92 r
.66969 .38511 .67765 .39505 .6943 .38968 .6864 .37968 Metetra
.656 .751 .92 r
.6864 .37968 .6943 .38968 .71104 .38428 .70322 .37422 Metetra
.656 .751 .92 r
.70322 .37422 .71104 .38428 .72789 .37885 .72013 .36873 Metetra
.656 .751 .92 r
.72013 .36873 .72789 .37885 .74484 .37338 .73715 .3632 Metetra
.656 .751 .92 r
.73715 .3632 .74484 .37338 .76188 .36789 .75427 .35764 Metetra
.656 .751 .92 r
.75427 .35764 .76188 .36789 .77903 .36236 .77149 .35205 Metetra
.656 .751 .92 r
.77149 .35205 .77903 .36236 .79628 .35679 .78881 .34643 Metetra
.656 .751 .92 r
.78881 .34643 .79628 .35679 .81364 .3512 .80624 .34077 Metetra
.656 .751 .92 r
.80624 .34077 .81364 .3512 .8311 .34557 .82378 .33508 Metetra
.656 .751 .92 r
.82378 .33508 .8311 .34557 .84867 .33991 .84142 .32935 Metetra
.656 .752 .92 r
.20065 .52597 .21035 .53443 .22446 .52985 .2148 .52134 Metetra
.656 .752 .92 r
.2148 .52134 .22446 .52985 .23864 .52524 .22902 .51668 Metetra
.656 .752 .92 r
.22902 .51668 .23864 .52524 .2529 .5206 .24333 .512 Metetra
.656 .752 .92 r
.24333 .512 .2529 .5206 .26724 .51593 .25771 .50728 Metetra
.656 .752 .92 r
.25771 .50728 .26724 .51593 .28165 .51125 .27218 .50255 Metetra
.656 .752 .92 r
.27218 .50255 .28165 .51125 .29615 .50653 .28672 .49778 Metetra
.656 .752 .92 r
.28672 .49778 .29615 .50653 .31072 .50179 .30134 .49299 Metetra
.656 .752 .92 r
.30134 .49299 .31072 .50179 .32538 .49702 .31605 .48818 Metetra
.656 .752 .92 r
.31605 .48818 .32538 .49702 .34011 .49223 .33084 .48333 Metetra
.656 .752 .92 r
.33084 .48333 .34011 .49223 .35493 .48741 .34571 .47846 Metetra
.656 .752 .92 r
.34571 .47846 .35493 .48741 .36984 .48256 .36066 .47357 Metetra
.656 .752 .92 r
.36066 .47357 .36984 .48256 .38482 .47769 .3757 .46864 Metetra
.656 .752 .92 r
.3757 .46864 .38482 .47769 .39989 .47279 .39082 .46369 Metetra
.656 .751 .92 r
.39082 .46369 .39989 .47279 .41505 .46786 .40603 .45871 Metetra
.656 .751 .92 r
.40603 .45871 .41505 .46786 .43029 .46291 .42132 .4537 Metetra
.656 .751 .92 r
.42132 .4537 .43029 .46291 .44561 .45792 .4367 .44866 Metetra
.656 .751 .92 r
.4367 .44866 .44561 .45792 .46103 .45291 .45217 .4436 Metetra
.656 .751 .92 r
.45217 .4436 .46103 .45291 .47653 .44787 .46773 .4385 Metetra
.656 .751 .92 r
.46773 .4385 .47653 .44787 .49212 .4428 .48338 .43338 Metetra
.656 .751 .92 r
.48338 .43338 .49212 .4428 .5078 .43771 .49912 .42823 Metetra
.656 .751 .92 r
.49912 .42823 .5078 .43771 .52357 .43258 .51495 .42305 Metetra
.656 .751 .92 r
.51495 .42305 .52357 .43258 .53943 .42743 .53087 .41784 Metetra
.656 .751 .92 r
.53087 .41784 .53943 .42743 .55538 .42224 .54688 .4126 Metetra
.656 .751 .92 r
.54688 .4126 .55538 .42224 .57143 .41703 .56299 .40733 Metetra
.656 .751 .92 r
.56299 .40733 .57143 .41703 .58756 .41178 .57919 .40203 Metetra
.656 .751 .92 r
.57919 .40203 .58756 .41178 .6038 .40651 .59548 .39669 Metetra
.656 .751 .92 r
.59548 .39669 .6038 .40651 .62012 .40121 .61188 .39133 Metetra
.656 .751 .92 r
.61188 .39133 .62012 .40121 .63655 .39587 .62836 .38594 Metetra
.656 .751 .92 r
.62836 .38594 .63655 .39587 .65307 .39051 .64495 .38051 Metetra
.656 .751 .92 r
.64495 .38051 .65307 .39051 .66969 .38511 .66163 .37506 Metetra
.656 .751 .92 r
.66163 .37506 .66969 .38511 .6864 .37968 .67842 .36957 Metetra
.656 .751 .92 r
.67842 .36957 .6864 .37968 .70322 .37422 .6953 .36405 Metetra
.656 .751 .92 r
.6953 .36405 .70322 .37422 .72013 .36873 .71229 .35849 Metetra
.656 .751 .92 r
.71229 .35849 .72013 .36873 .73715 .3632 .72937 .35291 Metetra
.656 .751 .92 r
.72937 .35291 .73715 .3632 .75427 .35764 .74656 .34729 Metetra
.656 .751 .92 r
.74656 .34729 .75427 .35764 .77149 .35205 .76386 .34163 Metetra
.656 .751 .92 r
.76386 .34163 .77149 .35205 .78881 .34643 .78126 .33595 Metetra
.656 .751 .92 r
.78126 .33595 .78881 .34643 .80624 .34077 .79876 .33023 Metetra
.656 .751 .92 r
.79876 .33023 .80624 .34077 .82378 .33508 .81637 .32447 Metetra
.656 .751 .92 r
.81637 .32447 .82378 .33508 .84142 .32935 .83409 .31868 Metetra
.656 .752 .92 r
.19084 .51743 .20065 .52597 .2148 .52134 .20504 .51275 Metetra
.656 .752 .92 r
.20504 .51275 .2148 .52134 .22902 .51668 .21931 .50804 Metetra
.656 .752 .92 r
.21931 .50804 .22902 .51668 .24333 .512 .23366 .50331 Metetra
.656 .752 .92 r
.23366 .50331 .24333 .512 .25771 .50728 .24809 .49855 Metetra
.656 .752 .92 r
.24809 .49855 .25771 .50728 .27218 .50255 .2626 .49376 Metetra
.656 .752 .92 r
.2626 .49376 .27218 .50255 .28672 .49778 .27719 .48895 Metetra
.656 .752 .92 r
.27719 .48895 .28672 .49778 .30134 .49299 .29187 .48411 Metetra
.656 .752 .92 r
.29187 .48411 .30134 .49299 .31605 .48818 .30662 .47924 Metetra
.656 .752 .92 r
.30662 .47924 .31605 .48818 .33084 .48333 .32146 .47435 Metetra
.656 .752 .92 r
.32146 .47435 .33084 .48333 .34571 .47846 .33638 .46942 Metetra
.656 .752 .92 r
.33638 .46942 .34571 .47846 .36066 .47357 .35139 .46447 Metetra
.656 .752 .92 r
.35139 .46447 .36066 .47357 .3757 .46864 .36648 .4595 Metetra
.656 .752 .92 r
.36648 .4595 .3757 .46864 .39082 .46369 .38165 .45449 Metetra
.656 .752 .92 r
.38165 .45449 .39082 .46369 .40603 .45871 .39692 .44946 Metetra
.656 .751 .92 r
.39692 .44946 .40603 .45871 .42132 .4537 .41226 .44439 Metetra
.656 .751 .92 r
.41226 .44439 .42132 .4537 .4367 .44866 .4277 .4393 Metetra
.656 .751 .92 r
.4277 .4393 .4367 .44866 .45217 .4436 .44323 .43418 Metetra
.656 .751 .92 r
.44323 .43418 .45217 .4436 .46773 .4385 .45884 .42903 Metetra
.656 .751 .92 r
.45884 .42903 .46773 .4385 .48338 .43338 .47455 .42386 Metetra
.656 .751 .92 r
.47455 .42386 .48338 .43338 .49912 .42823 .49035 .41865 Metetra
.656 .751 .92 r
.49035 .41865 .49912 .42823 .51495 .42305 .50623 .41341 Metetra
.656 .751 .92 r
.50623 .41341 .51495 .42305 .53087 .41784 .52221 .40814 Metetra
.656 .751 .92 r
.52221 .40814 .53087 .41784 .54688 .4126 .53829 .40285 Metetra
.656 .751 .92 r
.53829 .40285 .54688 .4126 .56299 .40733 .55446 .39752 Metetra
.656 .751 .92 r
.55446 .39752 .56299 .40733 .57919 .40203 .57072 .39216 Metetra
.656 .751 .92 r
.57072 .39216 .57919 .40203 .59548 .39669 .58708 .38677 Metetra
.656 .751 .92 r
.58708 .38677 .59548 .39669 .61188 .39133 .60354 .38135 Metetra
.656 .751 .92 r
.60354 .38135 .61188 .39133 .62836 .38594 .62009 .3759 Metetra
.656 .751 .92 r
.62009 .3759 .62836 .38594 .64495 .38051 .63674 .37041 Metetra
.656 .751 .92 r
.63674 .37041 .64495 .38051 .66163 .37506 .65349 .36489 Metetra
.656 .751 .92 r
.65349 .36489 .66163 .37506 .67842 .36957 .67035 .35934 Metetra
.656 .751 .92 r
.67035 .35934 .67842 .36957 .6953 .36405 .6873 .35376 Metetra
.656 .751 .92 r
.6873 .35376 .6953 .36405 .71229 .35849 .70436 .34815 Metetra
.656 .751 .92 r
.70436 .34815 .71229 .35849 .72937 .35291 .72151 .3425 Metetra
.656 .751 .92 r
.72151 .3425 .72937 .35291 .74656 .34729 .73878 .33682 Metetra
.656 .751 .92 r
.73878 .33682 .74656 .34729 .76386 .34163 .75614 .3311 Metetra
.656 .751 .92 r
.75614 .3311 .76386 .34163 .78126 .33595 .77362 .32535 Metetra
.656 .751 .92 r
.77362 .32535 .78126 .33595 .79876 .33023 .7912 .31956 Metetra
.656 .751 .92 r
.7912 .31956 .79876 .33023 .81637 .32447 .80889 .31374 Metetra
.656 .751 .92 r
.80889 .31374 .81637 .32447 .83409 .31868 .82668 .30788 Metetra
.656 .752 .92 r
.18094 .5088 .19084 .51743 .20504 .51275 .19518 .50407 Metetra
.656 .752 .92 r
.19518 .50407 .20504 .51275 .21931 .50804 .2095 .49932 Metetra
.656 .752 .92 r
.2095 .49932 .21931 .50804 .23366 .50331 .22389 .49453 Metetra
.656 .752 .92 r
.22389 .49453 .23366 .50331 .24809 .49855 .23837 .48972 Metetra
.656 .752 .92 r
.23837 .48972 .24809 .49855 .2626 .49376 .25293 .48489 Metetra
.656 .752 .92 r
.25293 .48489 .2626 .49376 .27719 .48895 .26757 .48002 Metetra
.656 .752 .92 r
.26757 .48002 .27719 .48895 .29187 .48411 .28229 .47513 Metetra
.656 .752 .92 r
.28229 .47513 .29187 .48411 .30662 .47924 .2971 .47021 Metetra
.656 .752 .92 r
.2971 .47021 .30662 .47924 .32146 .47435 .31199 .46527 Metetra
.656 .752 .92 r
.31199 .46527 .32146 .47435 .33638 .46942 .32696 .46029 Metetra
.656 .752 .92 r
.32696 .46029 .33638 .46942 .35139 .46447 .34202 .45529 Metetra
.656 .752 .92 r
.34202 .45529 .35139 .46447 .36648 .4595 .35716 .45026 Metetra
.656 .752 .92 r
.35716 .45026 .36648 .4595 .38165 .45449 .37239 .4452 Metetra
.656 .752 .92 r
.37239 .4452 .38165 .45449 .39692 .44946 .38771 .44011 Metetra
.656 .751 .92 r
.38771 .44011 .39692 .44946 .41226 .44439 .40311 .43499 Metetra
.656 .751 .92 r
.40311 .43499 .41226 .44439 .4277 .4393 .4186 .42985 Metetra
.656 .751 .92 r
.4186 .42985 .4277 .4393 .44323 .43418 .43419 .42467 Metetra
.656 .751 .92 r
.43419 .42467 .44323 .43418 .45884 .42903 .44986 .41947 Metetra
.656 .751 .92 r
.44986 .41947 .45884 .42903 .47455 .42386 .46562 .41423 Metetra
.656 .751 .92 r
.46562 .41423 .47455 .42386 .49035 .41865 .48148 .40897 Metetra
.656 .751 .92 r
.48148 .40897 .49035 .41865 .50623 .41341 .49743 .40367 Metetra
.656 .751 .92 r
.49743 .40367 .50623 .41341 .52221 .40814 .51347 .39835 Metetra
.656 .751 .92 r
.51347 .39835 .52221 .40814 .53829 .40285 .5296 .39299 Metetra
.656 .751 .92 r
.5296 .39299 .53829 .40285 .55446 .39752 .54583 .3876 Metetra
.656 .751 .92 r
.54583 .3876 .55446 .39752 .57072 .39216 .56216 .38219 Metetra
.656 .751 .92 r
.56216 .38219 .57072 .39216 .58708 .38677 .57858 .37674 Metetra
.656 .751 .92 r
.57858 .37674 .58708 .38677 .60354 .38135 .59511 .37125 Metetra
.656 .751 .92 r
.59511 .37125 .60354 .38135 .62009 .3759 .61173 .36574 Metetra
.656 .751 .92 r
.61173 .36574 .62009 .3759 .63674 .37041 .62844 .3602 Metetra
.656 .751 .92 r
.62844 .3602 .63674 .37041 .65349 .36489 .64526 .35462 Metetra
.656 .751 .92 r
.64526 .35462 .65349 .36489 .67035 .35934 .66219 .34901 Metetra
.656 .751 .92 r
.66219 .34901 .67035 .35934 .6873 .35376 .67921 .34336 Metetra
.656 .751 .92 r
.67921 .34336 .6873 .35376 .70436 .34815 .69633 .33768 Metetra
.656 .751 .92 r
.69633 .33768 .70436 .34815 .72151 .3425 .71356 .33197 Metetra
.656 .751 .92 r
.71356 .33197 .72151 .3425 .73878 .33682 .7309 .32622 Metetra
.656 .751 .92 r
.7309 .32622 .73878 .33682 .75614 .3311 .74834 .32044 Metetra
.656 .751 .92 r
.74834 .32044 .75614 .3311 .77362 .32535 .76589 .31463 Metetra
.656 .751 .92 r
.76589 .31463 .77362 .32535 .7912 .31956 .78355 .30877 Metetra
.656 .751 .92 r
.78355 .30877 .7912 .31956 .80889 .31374 .80131 .30289 Metetra
.656 .751 .92 r
.80131 .30289 .80889 .31374 .82668 .30788 .81919 .29696 Metetra
.656 .752 .92 r
.17093 .50009 .18094 .5088 .19518 .50407 .18522 .49531 Metetra
.656 .752 .92 r
.18522 .49531 .19518 .50407 .2095 .49932 .19958 .4905 Metetra
.656 .752 .92 r
.19958 .4905 .2095 .49932 .22389 .49453 .21403 .48567 Metetra
.656 .752 .92 r
.21403 .48567 .22389 .49453 .23837 .48972 .22855 .48081 Metetra
.656 .752 .92 r
.22855 .48081 .23837 .48972 .25293 .48489 .24316 .47592 Metetra
.656 .752 .92 r
.24316 .47592 .25293 .48489 .26757 .48002 .25785 .47101 Metetra
.656 .752 .92 r
.25785 .47101 .26757 .48002 .28229 .47513 .27262 .46606 Metetra
.656 .752 .92 r
.27262 .46606 .28229 .47513 .2971 .47021 .28747 .46109 Metetra
.656 .752 .92 r
.28747 .46109 .2971 .47021 .31199 .46527 .30241 .45609 Metetra
.656 .752 .92 r
.30241 .45609 .31199 .46527 .32696 .46029 .31744 .45106 Metetra
.656 .752 .92 r
.31744 .45106 .32696 .46029 .34202 .45529 .33255 .44601 Metetra
.656 .752 .92 r
.33255 .44601 .34202 .45529 .35716 .45026 .34774 .44092 Metetra
.656 .752 .92 r
.34774 .44092 .35716 .45026 .37239 .4452 .36303 .43581 Metetra
.656 .752 .92 r
.36303 .43581 .37239 .4452 .38771 .44011 .3784 .43066 Metetra
.656 .752 .92 r
.3784 .43066 .38771 .44011 .40311 .43499 .39386 .42549 Metetra
.656 .751 .92 r
.39386 .42549 .40311 .43499 .4186 .42985 .40941 .42029 Metetra
.656 .751 .92 r
.40941 .42029 .4186 .42985 .43419 .42467 .42505 .41506 Metetra
.656 .751 .92 r
.42505 .41506 .43419 .42467 .44986 .41947 .44078 .40979 Metetra
.656 .751 .92 r
.44078 .40979 .44986 .41947 .46562 .41423 .4566 .4045 Metetra
.656 .751 .92 r
.4566 .4045 .46562 .41423 .48148 .40897 .47251 .39918 Metetra
.656 .751 .92 r
.47251 .39918 .48148 .40897 .49743 .40367 .48852 .39383 Metetra
.656 .751 .92 r
.48852 .39383 .49743 .40367 .51347 .39835 .50462 .38844 Metetra
.656 .751 .92 r
.50462 .38844 .51347 .39835 .5296 .39299 .52082 .38303 Metetra
.656 .751 .92 r
.52082 .38303 .5296 .39299 .54583 .3876 .53712 .37758 Metetra
.656 .751 .92 r
.53712 .37758 .54583 .3876 .56216 .38219 .55351 .3721 Metetra
.656 .751 .92 r
.55351 .3721 .56216 .38219 .57858 .37674 .56999 .36659 Metetra
.656 .751 .92 r
.56999 .36659 .57858 .37674 .59511 .37125 .58658 .36105 Metetra
.656 .751 .92 r
.58658 .36105 .59511 .37125 .61173 .36574 .60327 .35547 Metetra
.656 .751 .92 r
.60327 .35547 .61173 .36574 .62844 .3602 .62005 .34987 Metetra
.656 .751 .92 r
.62005 .34987 .62844 .3602 .64526 .35462 .63694 .34422 Metetra
.656 .751 .92 r
.63694 .34422 .64526 .35462 .66219 .34901 .65393 .33855 Metetra
.656 .751 .92 r
.65393 .33855 .66219 .34901 .67921 .34336 .67103 .33284 Metetra
.656 .751 .92 r
.67103 .33284 .67921 .34336 .69633 .33768 .68822 .3271 Metetra
.656 .751 .92 r
.68822 .3271 .69633 .33768 .71356 .33197 .70553 .32132 Metetra
.656 .751 .92 r
.70553 .32132 .71356 .33197 .7309 .32622 .72294 .31551 Metetra
.656 .751 .92 r
.72294 .31551 .7309 .32622 .74834 .32044 .74045 .30966 Metetra
.656 .751 .92 r
.74045 .30966 .74834 .32044 .76589 .31463 .75808 .30378 Metetra
.656 .751 .92 r
.75808 .30378 .76589 .31463 .78355 .30877 .77581 .29786 Metetra
.656 .751 .92 r
.77581 .29786 .78355 .30877 .80131 .30289 .79365 .29191 Metetra
.656 .751 .92 r
.79365 .29191 .80131 .30289 .81919 .29696 .81161 .28592 Metetra
.656 .752 .92 r
.16083 .49128 .17093 .50009 .18522 .49531 .17516 .48646 Metetra
.656 .752 .92 r
.17516 .48646 .18522 .49531 .19958 .4905 .18957 .4816 Metetra
.656 .752 .92 r
.18957 .4816 .19958 .4905 .21403 .48567 .20406 .47672 Metetra
.656 .752 .92 r
.20406 .47672 .21403 .48567 .22855 .48081 .21863 .47181 Metetra
.656 .752 .92 r
.21863 .47181 .22855 .48081 .24316 .47592 .23328 .46687 Metetra
.656 .752 .92 r
.23328 .46687 .24316 .47592 .25785 .47101 .24802 .4619 Metetra
.656 .752 .92 r
.24802 .4619 .25785 .47101 .27262 .46606 .26284 .4569 Metetra
.656 .752 .92 r
.26284 .4569 .27262 .46606 .28747 .46109 .27775 .45188 Metetra
.656 .752 .92 r
.27775 .45188 .28747 .46109 .30241 .45609 .29274 .44682 Metetra
.656 .752 .92 r
.29274 .44682 .30241 .45609 .31744 .45106 .30782 .44174 Metetra
.656 .752 .92 r
.30782 .44174 .31744 .45106 .33255 .44601 .32298 .43663 Metetra
.656 .752 .92 r
.32298 .43663 .33255 .44601 .34774 .44092 .33823 .43149 Metetra
.656 .752 .92 r
.33823 .43149 .34774 .44092 .36303 .43581 .35356 .42632 Metetra
.656 .752 .92 r
.35356 .42632 .36303 .43581 .3784 .43066 .36899 .42112 Metetra
.656 .752 .92 r
.36899 .42112 .3784 .43066 .39386 .42549 .38451 .41589 Metetra
.656 .752 .92 r
.38451 .41589 .39386 .42549 .40941 .42029 .40011 .41063 Metetra
.656 .751 .92 r
.40011 .41063 .40941 .42029 .42505 .41506 .41581 .40534 Metetra
.656 .751 .92 r
.41581 .40534 .42505 .41506 .44078 .40979 .4316 .40002 Metetra
.656 .751 .92 r
.4316 .40002 .44078 .40979 .4566 .4045 .44748 .39467 Metetra
.656 .751 .92 r
.44748 .39467 .4566 .4045 .47251 .39918 .46345 .38928 Metetra
.656 .751 .92 r
.46345 .38928 .47251 .39918 .48852 .39383 .47952 .38387 Metetra
.656 .751 .92 r
.47952 .38387 .48852 .39383 .50462 .38844 .49569 .37843 Metetra
.656 .751 .92 r
.49569 .37843 .50462 .38844 .52082 .38303 .51195 .37295 Metetra
.656 .751 .92 r
.51195 .37295 .52082 .38303 .53712 .37758 .5283 .36744 Metetra
.656 .751 .92 r
.5283 .36744 .53712 .37758 .55351 .3721 .54476 .3619 Metetra
.656 .751 .92 r
.54476 .3619 .55351 .3721 .56999 .36659 .56131 .35633 Metetra
.656 .751 .92 r
.56131 .35633 .56999 .36659 .58658 .36105 .57796 .35073 Metetra
.656 .751 .92 r
.57796 .35073 .58658 .36105 .60327 .35547 .59472 .34509 Metetra
.656 .751 .92 r
.59472 .34509 .60327 .35547 .62005 .34987 .61157 .33942 Metetra
.656 .751 .92 r
.61157 .33942 .62005 .34987 .63694 .34422 .62853 .33371 Metetra
.656 .751 .92 r
.62853 .33371 .63694 .34422 .65393 .33855 .64559 .32797 Metetra
.656 .751 .92 r
.64559 .32797 .65393 .33855 .67103 .33284 .66275 .3222 Metetra
.656 .751 .92 r
.66275 .3222 .67103 .33284 .68822 .3271 .68002 .31639 Metetra
.656 .751 .92 r
.68002 .31639 .68822 .3271 .70553 .32132 .6974 .31055 Metetra
.656 .751 .92 r
.6974 .31055 .70553 .32132 .72294 .31551 .71488 .30467 Metetra
.656 .751 .92 r
.71488 .30467 .72294 .31551 .74045 .30966 .73247 .29876 Metetra
.656 .751 .92 r
.73247 .29876 .74045 .30966 .75808 .30378 .75017 .29281 Metetra
.656 .751 .92 r
.75017 .29281 .75808 .30378 .77581 .29786 .76798 .28682 Metetra
.656 .751 .92 r
.76798 .28682 .77581 .29786 .79365 .29191 .7859 .2808 Metetra
.656 .751 .92 r
.7859 .2808 .79365 .29191 .81161 .28592 .80394 .27474 Metetra
.656 .752 .92 r
.15061 .48239 .16083 .49128 .17516 .48646 .16499 .47752 Metetra
.656 .752 .92 r
.16499 .47752 .17516 .48646 .18957 .4816 .17944 .47261 Metetra
.656 .752 .92 r
.17944 .47261 .18957 .4816 .20406 .47672 .19398 .46768 Metetra
.656 .752 .92 r
.19398 .46768 .20406 .47672 .21863 .47181 .2086 .46271 Metetra
.656 .752 .92 r
.2086 .46271 .21863 .47181 .23328 .46687 .22331 .45772 Metetra
.656 .752 .92 r
.22331 .45772 .23328 .46687 .24802 .4619 .23809 .4527 Metetra
.656 .752 .92 r
.23809 .4527 .24802 .4619 .26284 .4569 .25296 .44765 Metetra
.656 .752 .92 r
.25296 .44765 .26284 .4569 .27775 .45188 .26792 .44257 Metetra
.656 .752 .92 r
.26792 .44257 .27775 .45188 .29274 .44682 .28296 .43746 Metetra
.656 .752 .92 r
.28296 .43746 .29274 .44682 .30782 .44174 .29809 .43232 Metetra
.656 .752 .92 r
.29809 .43232 .30782 .44174 .32298 .43663 .31331 .42715 Metetra
.656 .752 .92 r
.31331 .42715 .32298 .43663 .33823 .43149 .32861 .42196 Metetra
.656 .752 .92 r
.32861 .42196 .33823 .43149 .35356 .42632 .344 .41673 Metetra
.656 .752 .92 r
.344 .41673 .35356 .42632 .36899 .42112 .35948 .41147 Metetra
.656 .752 .92 r
.35948 .41147 .36899 .42112 .38451 .41589 .37505 .40618 Metetra
.656 .752 .92 r
.37505 .40618 .38451 .41589 .40011 .41063 .39072 .40086 Metetra
.656 .752 .92 r
.39072 .40086 .40011 .41063 .41581 .40534 .40647 .39551 Metetra
.656 .751 .92 r
.40647 .39551 .41581 .40534 .4316 .40002 .42232 .39013 Metetra
.656 .751 .92 r
.42232 .39013 .4316 .40002 .44748 .39467 .43826 .38472 Metetra
.656 .751 .92 r
.43826 .38472 .44748 .39467 .46345 .38928 .45429 .37928 Metetra
.656 .751 .92 r
.45429 .37928 .46345 .38928 .47952 .38387 .47042 .37381 Metetra
.656 .751 .92 r
.47042 .37381 .47952 .38387 .49569 .37843 .48665 .3683 Metetra
.656 .751 .92 r
.48665 .3683 .49569 .37843 .51195 .37295 .50297 .36276 Metetra
.656 .751 .92 r
.50297 .36276 .51195 .37295 .5283 .36744 .51939 .35719 Metetra
.656 .751 .92 r
.51939 .35719 .5283 .36744 .54476 .3619 .53591 .35159 Metetra
.656 .751 .92 r
.53591 .35159 .54476 .3619 .56131 .35633 .55253 .34596 Metetra
.656 .751 .92 r
.55253 .34596 .56131 .35633 .57796 .35073 .56925 .34029 Metetra
.656 .751 .92 r
.56925 .34029 .57796 .35073 .59472 .34509 .58607 .33459 Metetra
.656 .751 .92 r
.58607 .33459 .59472 .34509 .61157 .33942 .60299 .32885 Metetra
.656 .751 .92 r
.60299 .32885 .61157 .33942 .62853 .33371 .62002 .32308 Metetra
.656 .751 .92 r
.62002 .32308 .62853 .33371 .64559 .32797 .63715 .31728 Metetra
.656 .751 .92 r
.63715 .31728 .64559 .32797 .66275 .3222 .65438 .31144 Metetra
.656 .751 .92 r
.65438 .31144 .66275 .3222 .68002 .31639 .67173 .30557 Metetra
.656 .751 .92 r
.67173 .30557 .68002 .31639 .6974 .31055 .68917 .29966 Metetra
.656 .751 .92 r
.68917 .29966 .6974 .31055 .71488 .30467 .70673 .29371 Metetra
.656 .751 .92 r
.70673 .29371 .71488 .30467 .73247 .29876 .7244 .28773 Metetra
.656 .751 .92 r
.7244 .28773 .73247 .29876 .75017 .29281 .74217 .28171 Metetra
.656 .751 .92 r
.74217 .28171 .75017 .29281 .76798 .28682 .76006 .27566 Metetra
.656 .751 .92 r
.76006 .27566 .76798 .28682 .7859 .2808 .77806 .26956 Metetra
.656 .751 .92 r
.77806 .26956 .7859 .2808 .80394 .27474 .79618 .26343 Metetra
.656 .752 .921 r
.14029 .47341 .15061 .48239 .16499 .47752 .15471 .46849 Metetra
.656 .752 .921 r
.15471 .46849 .16499 .47752 .17944 .47261 .16922 .46353 Metetra
.656 .752 .921 r
.16922 .46353 .17944 .47261 .19398 .46768 .1838 .45854 Metetra
.656 .752 .921 r
.1838 .45854 .19398 .46768 .2086 .46271 .19847 .45353 Metetra
.656 .752 .92 r
.19847 .45353 .2086 .46271 .22331 .45772 .21322 .44848 Metetra
.656 .752 .92 r
.21322 .44848 .22331 .45772 .23809 .4527 .22806 .44341 Metetra
.656 .752 .92 r
.22806 .44341 .23809 .4527 .25296 .44765 .24298 .4383 Metetra
.656 .752 .92 r
.24298 .4383 .25296 .44765 .26792 .44257 .25799 .43317 Metetra
.656 .752 .92 r
.25799 .43317 .26792 .44257 .28296 .43746 .27308 .428 Metetra
.656 .752 .92 r
.27308 .428 .28296 .43746 .29809 .43232 .28826 .42281 Metetra
.656 .752 .92 r
.28826 .42281 .29809 .43232 .31331 .42715 .30353 .41758 Metetra
.656 .752 .92 r
.30353 .41758 .31331 .42715 .32861 .42196 .31889 .41232 Metetra
.656 .752 .92 r
.31889 .41232 .32861 .42196 .344 .41673 .33433 .40704 Metetra
.656 .752 .92 r
.33433 .40704 .344 .41673 .35948 .41147 .34987 .40172 Metetra
.656 .752 .92 r
.34987 .40172 .35948 .41147 .37505 .40618 .3655 .39637 Metetra
.656 .752 .92 r
.3655 .39637 .37505 .40618 .39072 .40086 .38122 .39099 Metetra
.656 .752 .92 r
.38122 .39099 .39072 .40086 .40647 .39551 .39703 .38558 Metetra
.656 .752 .92 r
.39703 .38558 .40647 .39551 .42232 .39013 .41294 .38014 Metetra
.656 .751 .92 r
.41294 .38014 .42232 .39013 .43826 .38472 .42894 .37467 Metetra
.656 .751 .92 r
.42894 .37467 .43826 .38472 .45429 .37928 .44503 .36917 Metetra
.656 .751 .92 r
.44503 .36917 .45429 .37928 .47042 .37381 .46122 .36363 Metetra
.656 .751 .92 r
.46122 .36363 .47042 .37381 .48665 .3683 .47751 .35806 Metetra
.656 .751 .92 r
.47751 .35806 .48665 .3683 .50297 .36276 .4939 .35246 Metetra
.656 .751 .92 r
.4939 .35246 .50297 .36276 .51939 .35719 .51038 .34683 Metetra
.656 .751 .92 r
.51038 .34683 .51939 .35719 .53591 .35159 .52697 .34116 Metetra
.656 .751 .92 r
.52697 .34116 .53591 .35159 .55253 .34596 .54365 .33546 Metetra
.656 .751 .92 r
.54365 .33546 .55253 .34596 .56925 .34029 .56044 .32973 Metetra
.656 .751 .92 r
.56044 .32973 .56925 .34029 .58607 .33459 .57732 .32396 Metetra
.656 .751 .92 r
.57732 .32396 .58607 .33459 .60299 .32885 .59432 .31816 Metetra
.656 .751 .92 r
.59432 .31816 .60299 .32885 .62002 .32308 .61141 .31233 Metetra
.656 .751 .92 r
.61141 .31233 .62002 .32308 .63715 .31728 .62861 .30646 Metetra
.656 .751 .92 r
.62861 .30646 .63715 .31728 .65438 .31144 .64592 .30055 Metetra
.656 .751 .92 r
.64592 .30055 .65438 .31144 .67173 .30557 .66333 .29461 Metetra
.656 .751 .92 r
.66333 .29461 .67173 .30557 .68917 .29966 .68086 .28863 Metetra
.656 .751 .92 r
.68086 .28863 .68917 .29966 .70673 .29371 .69849 .28262 Metetra
.656 .751 .92 r
.69849 .28262 .70673 .29371 .7244 .28773 .71623 .27657 Metetra
.656 .751 .92 r
.71623 .27657 .7244 .28773 .74217 .28171 .73409 .27048 Metetra
.656 .751 .92 r
.73409 .27048 .74217 .28171 .76006 .27566 .75205 .26436 Metetra
.656 .751 .92 r
.75205 .26436 .76006 .27566 .77806 .26956 .77013 .25819 Metetra
.656 .751 .92 r
.77013 .25819 .77806 .26956 .79618 .26343 .78833 .25199 Metetra
.656 .752 .921 r
.12986 .46435 .14029 .47341 .15471 .46849 .14433 .45937 Metetra
.656 .752 .921 r
.14433 .45937 .15471 .46849 .16922 .46353 .15888 .45436 Metetra
.656 .752 .921 r
.15888 .45436 .16922 .46353 .1838 .45854 .17351 .44932 Metetra
.656 .752 .921 r
.17351 .44932 .1838 .45854 .19847 .45353 .18823 .44425 Metetra
.656 .752 .921 r
.18823 .44425 .19847 .45353 .21322 .44848 .20303 .43915 Metetra
.656 .752 .921 r
.20303 .43915 .21322 .44848 .22806 .44341 .21792 .43402 Metetra
.656 .752 .921 r
.21792 .43402 .22806 .44341 .24298 .4383 .23289 .42886 Metetra
.656 .752 .92 r
.23289 .42886 .24298 .4383 .25799 .43317 .24795 .42367 Metetra
.656 .752 .92 r
.24795 .42367 .25799 .43317 .27308 .428 .26309 .41844 Metetra
.656 .752 .92 r
.26309 .41844 .27308 .428 .28826 .42281 .27833 .41319 Metetra
.656 .752 .92 r
.27833 .41319 .28826 .42281 .30353 .41758 .29365 .40791 Metetra
.656 .752 .92 r
.29365 .40791 .30353 .41758 .31889 .41232 .30906 .40259 Metetra
.656 .752 .92 r
.30906 .40259 .31889 .41232 .33433 .40704 .32456 .39725 Metetra
.656 .752 .92 r
.32456 .39725 .33433 .40704 .34987 .40172 .34016 .39187 Metetra
.656 .752 .92 r
.34016 .39187 .34987 .40172 .3655 .39637 .35584 .38646 Metetra
.656 .752 .92 r
.35584 .38646 .3655 .39637 .38122 .39099 .37162 .38102 Metetra
.656 .752 .92 r
.37162 .38102 .38122 .39099 .39703 .38558 .38749 .37555 Metetra
.656 .752 .92 r
.38749 .37555 .39703 .38558 .41294 .38014 .40345 .37004 Metetra
.656 .751 .92 r
.40345 .37004 .41294 .38014 .42894 .37467 .41951 .36451 Metetra
.656 .751 .92 r
.41951 .36451 .42894 .37467 .44503 .36917 .43567 .35894 Metetra
.656 .751 .92 r
.43567 .35894 .44503 .36917 .46122 .36363 .45192 .35334 Metetra
.656 .751 .92 r
.45192 .35334 .46122 .36363 .47751 .35806 .46827 .34771 Metetra
.656 .751 .92 r
.46827 .34771 .47751 .35806 .4939 .35246 .48472 .34204 Metetra
.656 .751 .92 r
.48472 .34204 .4939 .35246 .51038 .34683 .50127 .33634 Metetra
.656 .751 .92 r
.50127 .33634 .51038 .34683 .52697 .34116 .51792 .33061 Metetra
.656 .751 .92 r
.51792 .33061 .52697 .34116 .54365 .33546 .53467 .32485 Metetra
.656 .751 .92 r
.53467 .32485 .54365 .33546 .56044 .32973 .55152 .31905 Metetra
.656 .751 .92 r
.55152 .31905 .56044 .32973 .57732 .32396 .56848 .31322 Metetra
.656 .751 .92 r
.56848 .31322 .57732 .32396 .59432 .31816 .58554 .30735 Metetra
.656 .751 .92 r
.58554 .30735 .59432 .31816 .61141 .31233 .60271 .30145 Metetra
.656 .751 .92 r
.60271 .30145 .61141 .31233 .62861 .30646 .61998 .29551 Metetra
.656 .751 .92 r
.61998 .29551 .62861 .30646 .64592 .30055 .63736 .28953 Metetra
.656 .751 .92 r
.63736 .28953 .64592 .30055 .66333 .29461 .65485 .28353 Metetra
.656 .751 .92 r
.65485 .28353 .66333 .29461 .68086 .28863 .67244 .27748 Metetra
.656 .751 .92 r
.67244 .27748 .68086 .28863 .69849 .28262 .69015 .2714 Metetra
.656 .751 .92 r
.69015 .2714 .69849 .28262 .71623 .27657 .70797 .26528 Metetra
.656 .751 .92 r
.70797 .26528 .71623 .27657 .73409 .27048 .7259 .25912 Metetra
.656 .751 .92 r
.7259 .25912 .73409 .27048 .75205 .26436 .74395 .25292 Metetra
.656 .751 .92 r
.74395 .25292 .75205 .26436 .77013 .25819 .76211 .24669 Metetra
.656 .751 .92 r
.76211 .24669 .77013 .25819 .78833 .25199 .78039 .24042 Metetra
.656 .752 .921 r
.11932 .45519 .12986 .46435 .14433 .45937 .13384 .45016 Metetra
.656 .752 .921 r
.13384 .45016 .14433 .45937 .15888 .45436 .14843 .4451 Metetra
.656 .752 .921 r
.14843 .4451 .15888 .45436 .17351 .44932 .16311 .44 Metetra
.656 .752 .921 r
.16311 .44 .17351 .44932 .18823 .44425 .17788 .43488 Metetra
.656 .752 .921 r
.17788 .43488 .18823 .44425 .20303 .43915 .19273 .42973 Metetra
.656 .752 .921 r
.19273 .42973 .20303 .43915 .21792 .43402 .20767 .42454 Metetra
.656 .752 .921 r
.20767 .42454 .21792 .43402 .23289 .42886 .22269 .41932 Metetra
.656 .752 .921 r
.22269 .41932 .23289 .42886 .24795 .42367 .2378 .41407 Metetra
.656 .752 .921 r
.2378 .41407 .24795 .42367 .26309 .41844 .253 .40879 Metetra
.656 .752 .92 r
.253 .40879 .26309 .41844 .27833 .41319 .26828 .40348 Metetra
.656 .752 .92 r
.26828 .40348 .27833 .41319 .29365 .40791 .28366 .39814 Metetra
.656 .752 .92 r
.28366 .39814 .29365 .40791 .30906 .40259 .29913 .39276 Metetra
.656 .752 .92 r
.29913 .39276 .30906 .40259 .32456 .39725 .31468 .38735 Metetra
.656 .752 .92 r
.31468 .38735 .32456 .39725 .34016 .39187 .33033 .38191 Metetra
.656 .752 .92 r
.33033 .38191 .34016 .39187 .35584 .38646 .34607 .37644 Metetra
.656 .752 .92 r
.34607 .37644 .35584 .38646 .37162 .38102 .36191 .37094 Metetra
.656 .752 .92 r
.36191 .37094 .37162 .38102 .38749 .37555 .37784 .3654 Metetra
.656 .752 .92 r
.37784 .3654 .38749 .37555 .40345 .37004 .39386 .35983 Metetra
.656 .752 .92 r
.39386 .35983 .40345 .37004 .41951 .36451 .40998 .35423 Metetra
.656 .751 .92 r
.40998 .35423 .41951 .36451 .43567 .35894 .4262 .3486 Metetra
.656 .751 .92 r
.4262 .3486 .43567 .35894 .45192 .35334 .44252 .34293 Metetra
.656 .751 .92 r
.44252 .34293 .45192 .35334 .46827 .34771 .45893 .33724 Metetra
.656 .751 .92 r
.45893 .33724 .46827 .34771 .48472 .34204 .47544 .3315 Metetra
.656 .751 .92 r
.47544 .3315 .48472 .34204 .50127 .33634 .49206 .32574 Metetra
.656 .751 .92 r
.49206 .32574 .50127 .33634 .51792 .33061 .50877 .31994 Metetra
.656 .751 .92 r
.50877 .31994 .51792 .33061 .53467 .32485 .52559 .31411 Metetra
.656 .751 .92 r
.52559 .31411 .53467 .32485 .55152 .31905 .54251 .30824 Metetra
.656 .751 .92 r
.54251 .30824 .55152 .31905 .56848 .31322 .55954 .30234 Metetra
.656 .751 .92 r
.55954 .30234 .56848 .31322 .58554 .30735 .57667 .29641 Metetra
.656 .751 .92 r
.57667 .29641 .58554 .30735 .60271 .30145 .5939 .29044 Metetra
.656 .751 .92 r
.5939 .29044 .60271 .30145 .61998 .29551 .61125 .28443 Metetra
.656 .751 .92 r
.61125 .28443 .61998 .29551 .63736 .28953 .6287 .27839 Metetra
.656 .751 .92 r
.6287 .27839 .63736 .28953 .65485 .28353 .64626 .27231 Metetra
.656 .751 .92 r
.64626 .27231 .65485 .28353 .67244 .27748 .66393 .26619 Metetra
.656 .751 .92 r
.66393 .26619 .67244 .27748 .69015 .2714 .68172 .26004 Metetra
.656 .751 .92 r
.68172 .26004 .69015 .2714 .70797 .26528 .69961 .25385 Metetra
.656 .751 .92 r
.69961 .25385 .70797 .26528 .7259 .25912 .71762 .24762 Metetra
.656 .751 .92 r
.71762 .24762 .7259 .25912 .74395 .25292 .73575 .24135 Metetra
.656 .751 .92 r
.73575 .24135 .74395 .25292 .76211 .24669 .75399 .23504 Metetra
.656 .751 .92 r
.75399 .23504 .76211 .24669 .78039 .24042 .77235 .2287 Metetra
.656 .752 .921 r
.10867 .44594 .11932 .45519 .13384 .45016 .12323 .44086 Metetra
.656 .752 .921 r
.12323 .44086 .13384 .45016 .14843 .4451 .13788 .43575 Metetra
.656 .753 .921 r
.13788 .43575 .14843 .4451 .16311 .44 .1526 .4306 Metetra
.656 .753 .921 r
.1526 .4306 .16311 .44 .17788 .43488 .16742 .42542 Metetra
.656 .753 .921 r
.16742 .42542 .17788 .43488 .19273 .42973 .18232 .42021 Metetra
.656 .753 .921 r
.18232 .42021 .19273 .42973 .20767 .42454 .1973 .41497 Metetra
.656 .753 .921 r
.1973 .41497 .20767 .42454 .22269 .41932 .21238 .40969 Metetra
.656 .753 .921 r
.21238 .40969 .22269 .41932 .2378 .41407 .22754 .40439 Metetra
.657 .753 .921 r
.22754 .40439 .2378 .41407 .253 .40879 .24279 .39905 Metetra
.657 .752 .921 r
.24279 .39905 .253 .40879 .26828 .40348 .25813 .39367 Metetra
.657 .752 .921 r
.25813 .39367 .26828 .40348 .28366 .39814 .27356 .38827 Metetra
.657 .752 .92 r
.27356 .38827 .28366 .39814 .29913 .39276 .28908 .38283 Metetra
.657 .752 .92 r
.28908 .38283 .29913 .39276 .31468 .38735 .3047 .37736 Metetra
.657 .752 .92 r
.3047 .37736 .31468 .38735 .33033 .38191 .3204 .37185 Metetra
.657 .752 .92 r
.3204 .37185 .33033 .38191 .34607 .37644 .3362 .36632 Metetra
.657 .752 .92 r
.3362 .36632 .34607 .37644 .36191 .37094 .3521 .36075 Metetra
.657 .752 .92 r
.3521 .36075 .36191 .37094 .37784 .3654 .36808 .35514 Metetra
.657 .752 .92 r
.36808 .35514 .37784 .3654 .39386 .35983 .38417 .34951 Metetra
.657 .752 .92 r
.38417 .34951 .39386 .35983 .40998 .35423 .40035 .34384 Metetra
.656 .751 .92 r
.40035 .34384 .40998 .35423 .4262 .3486 .41663 .33814 Metetra
.656 .751 .92 r
.41663 .33814 .4262 .3486 .44252 .34293 .43301 .33241 Metetra
.656 .751 .92 r
.43301 .33241 .44252 .34293 .45893 .33724 .44948 .32664 Metetra
.656 .751 .92 r
.44948 .32664 .45893 .33724 .47544 .3315 .46606 .32084 Metetra
.656 .751 .92 r
.46606 .32084 .47544 .3315 .49206 .32574 .48274 .31501 Metetra
.656 .751 .92 r
.48274 .31501 .49206 .32574 .50877 .31994 .49952 .30914 Metetra
.656 .751 .92 r
.49952 .30914 .50877 .31994 .52559 .31411 .51641 .30324 Metetra
.656 .751 .92 r
.51641 .30324 .52559 .31411 .54251 .30824 .53339 .29731 Metetra
.656 .751 .92 r
.53339 .29731 .54251 .30824 .55954 .30234 .55049 .29134 Metetra
.656 .751 .92 r
.55049 .29134 .55954 .30234 .57667 .29641 .56769 .28533 Metetra
.656 .751 .92 r
.56769 .28533 .57667 .29641 .5939 .29044 .585 .27929 Metetra
.656 .751 .92 r
.585 .27929 .5939 .29044 .61125 .28443 .60241 .27321 Metetra
.656 .751 .92 r
.60241 .27321 .61125 .28443 .6287 .27839 .61994 .2671 Metetra
.656 .751 .92 r
.61994 .2671 .6287 .27839 .64626 .27231 .63758 .26095 Metetra
.656 .751 .92 r
.63758 .26095 .64626 .27231 .66393 .26619 .65532 .25477 Metetra
.656 .751 .92 r
.65532 .25477 .66393 .26619 .68172 .26004 .67318 .24854 Metetra
.656 .751 .92 r
.67318 .24854 .68172 .26004 .69961 .25385 .69116 .24228 Metetra
.656 .751 .92 r
.69116 .24228 .69961 .25385 .71762 .24762 .70925 .23598 Metetra
.656 .751 .92 r
.70925 .23598 .71762 .24762 .73575 .24135 .72745 .22964 Metetra
.656 .751 .92 r
.72745 .22964 .73575 .24135 .75399 .23504 .74578 .22326 Metetra
.656 .751 .92 r
.74578 .22326 .75399 .23504 .77235 .2287 .76422 .21684 Metetra
.656 .753 .921 r
.09791 .4366 .10867 .44594 .12323 .44086 .11251 .43147 Metetra
.656 .753 .921 r
.11251 .43147 .12323 .44086 .13788 .43575 .1272 .4263 Metetra
.656 .753 .921 r
.1272 .4263 .13788 .43575 .1526 .4306 .14198 .42111 Metetra
.656 .753 .921 r
.14198 .42111 .1526 .4306 .16742 .42542 .15684 .41587 Metetra
.656 .753 .921 r
.15684 .41587 .16742 .42542 .18232 .42021 .17179 .41061 Metetra
.656 .753 .921 r
.17179 .41061 .18232 .42021 .1973 .41497 .18683 .40531 Metetra
.656 .753 .921 r
.18683 .40531 .1973 .41497 .21238 .40969 .20195 .39998 Metetra
.657 .753 .921 r
.20195 .39998 .21238 .40969 .22754 .40439 .21717 .39461 Metetra
.657 .753 .921 r
.21717 .39461 .22754 .40439 .24279 .39905 .23247 .38921 Metetra
.657 .753 .921 r
.23247 .38921 .24279 .39905 .25813 .39367 .24787 .38377 Metetra
.657 .753 .921 r
.24787 .38377 .25813 .39367 .27356 .38827 .26335 .3783 Metetra
.657 .753 .921 r
.26335 .3783 .27356 .38827 .28908 .38283 .27893 .3728 Metetra
.657 .753 .921 r
.27893 .3728 .28908 .38283 .3047 .37736 .2946 .36726 Metetra
.657 .752 .92 r
.2946 .36726 .3047 .37736 .3204 .37185 .31036 .36169 Metetra
.657 .752 .92 r
.31036 .36169 .3204 .37185 .3362 .36632 .32622 .35609 Metetra
.657 .752 .92 r
.32622 .35609 .3362 .36632 .3521 .36075 .34217 .35045 Metetra
.657 .752 .92 r
.34217 .35045 .3521 .36075 .36808 .35514 .35822 .34478 Metetra
.657 .752 .92 r
.35822 .34478 .36808 .35514 .38417 .34951 .37436 .33907 Metetra
.657 .752 .92 r
.37436 .33907 .38417 .34951 .40035 .34384 .39061 .33333 Metetra
.657 .752 .92 r
.39061 .33333 .40035 .34384 .41663 .33814 .40695 .32756 Metetra
.657 .751 .92 r
.40695 .32756 .41663 .33814 .43301 .33241 .42339 .32176 Metetra
.657 .751 .92 r
.42339 .32176 .43301 .33241 .44948 .32664 .43993 .31592 Metetra
.656 .751 .92 r
.43993 .31592 .44948 .32664 .46606 .32084 .45657 .31005 Metetra
.656 .751 .92 r
.45657 .31005 .46606 .32084 .48274 .31501 .47332 .30415 Metetra
.656 .751 .92 r
.47332 .30415 .48274 .31501 .49952 .30914 .49016 .29821 Metetra
.656 .751 .92 r
.49016 .29821 .49952 .30914 .51641 .30324 .50712 .29224 Metetra
.656 .751 .919 r
.50712 .29224 .51641 .30324 .53339 .29731 .52417 .28623 Metetra
.656 .751 .919 r
.52417 .28623 .53339 .29731 .55049 .29134 .54134 .28019 Metetra
.656 .751 .919 r
.54134 .28019 .55049 .29134 .56769 .28533 .55861 .27412 Metetra
.656 .75 .919 r
.55861 .27412 .56769 .28533 .585 .27929 .57599 .26801 Metetra
.656 .75 .919 r
.57599 .26801 .585 .27929 .60241 .27321 .59348 .26186 Metetra
.656 .75 .919 r
.59348 .26186 .60241 .27321 .61994 .2671 .61108 .25568 Metetra
.656 .75 .92 r
.61108 .25568 .61994 .2671 .63758 .26095 .62879 .24946 Metetra
.656 .75 .92 r
.62879 .24946 .63758 .26095 .65532 .25477 .64661 .2432 Metetra
.656 .75 .92 r
.64661 .2432 .65532 .25477 .67318 .24854 .66455 .2369 Metetra
.656 .75 .92 r
.66455 .2369 .67318 .24854 .69116 .24228 .6826 .23057 Metetra
.656 .75 .92 r
.6826 .23057 .69116 .24228 .70925 .23598 .70077 .22419 Metetra
.656 .75 .92 r
.70077 .22419 .70925 .23598 .72745 .22964 .71906 .21778 Metetra
.656 .751 .92 r
.71906 .21778 .72745 .22964 .74578 .22326 .73746 .21133 Metetra
.656 .751 .92 r
.73746 .21133 .74578 .22326 .76422 .21684 .75599 .20483 Metetra
.656 .753 .921 r
.08703 .42717 .09791 .4366 .11251 .43147 .10168 .42199 Metetra
.656 .753 .921 r
.10168 .42199 .11251 .43147 .1272 .4263 .11642 .41677 Metetra
.656 .753 .921 r
.11642 .41677 .1272 .4263 .14198 .42111 .13124 .41152 Metetra
.656 .753 .921 r
.13124 .41152 .14198 .42111 .15684 .41587 .14615 .40624 Metetra
.656 .753 .921 r
.14615 .40624 .15684 .41587 .17179 .41061 .16115 .40092 Metetra
.656 .753 .921 r
.16115 .40092 .17179 .41061 .18683 .40531 .17624 .39556 Metetra
.656 .753 .921 r
.17624 .39556 .18683 .40531 .20195 .39998 .19141 .39017 Metetra
.657 .753 .921 r
.19141 .39017 .20195 .39998 .21717 .39461 .20668 .38474 Metetra
.657 .753 .921 r
.20668 .38474 .21717 .39461 .23247 .38921 .22204 .37928 Metetra
.657 .753 .921 r
.22204 .37928 .23247 .38921 .24787 .38377 .23749 .37378 Metetra
.657 .753 .921 r
.23749 .37378 .24787 .38377 .26335 .3783 .25303 .36825 Metetra
.657 .753 .921 r
.25303 .36825 .26335 .3783 .27893 .3728 .26866 .36267 Metetra
.657 .753 .921 r
.26866 .36267 .27893 .3728 .2946 .36726 .28439 .35707 Metetra
.657 .753 .921 r
.28439 .35707 .2946 .36726 .31036 .36169 .30021 .35143 Metetra
.657 .753 .92 r
.30021 .35143 .31036 .36169 .32622 .35609 .31612 .34575 Metetra
.657 .752 .92 r
.31612 .34575 .32622 .35609 .34217 .35045 .33214 .34004 Metetra
.657 .752 .92 r
.33214 .34004 .34217 .35045 .35822 .34478 .34824 .3343 Metetra
.657 .752 .92 r
.34824 .3343 .35822 .34478 .37436 .33907 .36445 .32852 Metetra
.657 .752 .92 r
.36445 .32852 .37436 .33907 .39061 .33333 .38075 .32271 Metetra
.657 .752 .92 r
.38075 .32271 .39061 .33333 .40695 .32756 .39716 .31687 Metetra
.657 .751 .92 r
.39716 .31687 .40695 .32756 .42339 .32176 .41366 .31099 Metetra
.657 .751 .92 r
.41366 .31099 .42339 .32176 .43993 .31592 .43027 .30508 Metetra
.657 .751 .919 r
.43027 .30508 .43993 .31592 .45657 .31005 .44698 .29913 Metetra
.657 .751 .919 r
.44698 .29913 .45657 .31005 .47332 .30415 .46379 .29315 Metetra
.656 .751 .919 r
.46379 .29315 .47332 .30415 .49016 .29821 .4807 .28714 Metetra
.656 .751 .919 r
.4807 .28714 .49016 .29821 .50712 .29224 .49772 .2811 Metetra
.656 .75 .919 r
.49772 .2811 .50712 .29224 .52417 .28623 .51485 .27502 Metetra
.656 .75 .919 r
.51485 .27502 .52417 .28623 .54134 .28019 .53208 .26891 Metetra
.656 .75 .919 r
.53208 .26891 .54134 .28019 .55861 .27412 .54942 .26276 Metetra
.656 .75 .919 r
.54942 .26276 .55861 .27412 .57599 .26801 .56687 .25658 Metetra
.656 .75 .919 r
.56687 .25658 .57599 .26801 .59348 .26186 .58444 .25036 Metetra
.656 .75 .919 r
.58444 .25036 .59348 .26186 .61108 .25568 .60211 .2441 Metetra
.656 .75 .919 r
.60211 .2441 .61108 .25568 .62879 .24946 .6199 .23781 Metetra
.656 .75 .919 r
.6199 .23781 .62879 .24946 .64661 .2432 .6378 .23148 Metetra
.656 .75 .919 r
.6378 .23148 .64661 .2432 .66455 .2369 .65581 .22511 Metetra
.656 .75 .919 r
.65581 .22511 .66455 .2369 .6826 .23057 .67395 .21871 Metetra
.656 .75 .92 r
.67395 .21871 .6826 .23057 .70077 .22419 .69219 .21226 Metetra
.656 .75 .92 r
.69219 .21226 .70077 .22419 .71906 .21778 .71056 .20578 Metetra
.655 .75 .92 r
.71056 .20578 .71906 .21778 .73746 .21133 .72905 .19925 Metetra
.655 .75 .92 r
.72905 .19925 .73746 .21133 .75599 .20483 .74766 .19268 Metetra
.655 .753 .921 r
.07603 .41763 .08703 .42717 .10168 .42199 .09073 .41241 Metetra
.656 .753 .921 r
.09073 .41241 .10168 .42199 .11642 .41677 .10551 .40715 Metetra
.656 .753 .922 r
.10551 .40715 .11642 .41677 .13124 .41152 .12038 .40185 Metetra
.656 .753 .922 r
.12038 .40185 .13124 .41152 .14615 .40624 .13534 .39652 Metetra
.656 .754 .922 r
.13534 .39652 .14615 .40624 .16115 .40092 .15039 .39115 Metetra
.656 .754 .922 r
.15039 .39115 .16115 .40092 .17624 .39556 .16553 .38574 Metetra
.656 .754 .922 r
.16553 .38574 .17624 .39556 .19141 .39017 .18076 .38029 Metetra
.657 .754 .922 r
.18076 .38029 .19141 .39017 .20668 .38474 .19607 .3748 Metetra
.657 .754 .922 r
.19607 .3748 .20668 .38474 .22204 .37928 .21149 .36928 Metetra
.657 .754 .922 r
.21149 .36928 .22204 .37928 .23749 .37378 .22699 .36371 Metetra
.657 .754 .921 r
.22699 .36371 .23749 .37378 .25303 .36825 .24259 .35811 Metetra
.657 .754 .921 r
.24259 .35811 .25303 .36825 .26866 .36267 .25827 .35247 Metetra
.657 .754 .921 r
.25827 .35247 .26866 .36267 .28439 .35707 .27406 .34679 Metetra
.657 .754 .921 r
.27406 .34679 .28439 .35707 .30021 .35143 .28994 .34108 Metetra
.657 .753 .921 r
.28994 .34108 .30021 .35143 .31612 .34575 .30591 .33532 Metetra
.657 .753 .92 r
.30591 .33532 .31612 .34575 .33214 .34004 .32199 .32954 Metetra
.657 .753 .92 r
.32199 .32954 .33214 .34004 .34824 .3343 .33816 .32371 Metetra
.657 .752 .92 r
.33816 .32371 .34824 .3343 .36445 .32852 .35442 .31786 Metetra
.657 .752 .92 r
.35442 .31786 .36445 .32852 .38075 .32271 .37079 .31197 Metetra
.657 .752 .92 r
.37079 .31197 .38075 .32271 .39716 .31687 .38726 .30604 Metetra
.657 .751 .919 r
.38726 .30604 .39716 .31687 .41366 .31099 .40383 .30008 Metetra
.657 .751 .919 r
.40383 .30008 .41366 .31099 .43027 .30508 .4205 .29409 Metetra
.657 .751 .919 r
.4205 .29409 .43027 .30508 .44698 .29913 .43727 .28807 Metetra
.657 .751 .919 r
.43727 .28807 .44698 .29913 .46379 .29315 .45415 .28201 Metetra
.657 .75 .919 r
.45415 .28201 .46379 .29315 .4807 .28714 .47113 .27593 Metetra
.657 .75 .919 r
.47113 .27593 .4807 .28714 .49772 .2811 .48822 .26981 Metetra
.656 .75 .919 r
.48822 .26981 .49772 .2811 .51485 .27502 .50541 .26365 Metetra
.656 .75 .919 r
.50541 .26365 .51485 .27502 .53208 .26891 .52272 .25747 Metetra
.656 .75 .919 r
.52272 .25747 .53208 .26891 .54942 .26276 .54013 .25124 Metetra
.656 .75 .919 r
.54013 .25124 .54942 .26276 .56687 .25658 .55765 .24499 Metetra
.656 .75 .919 r
.55765 .24499 .56687 .25658 .58444 .25036 .57529 .2387 Metetra
.656 .75 .919 r
.57529 .2387 .58444 .25036 .60211 .2441 .59304 .23237 Metetra
.656 .75 .919 r
.59304 .23237 .60211 .2441 .6199 .23781 .6109 .22601 Metetra
.656 .75 .919 r
.6109 .22601 .6199 .23781 .6378 .23148 .62888 .21961 Metetra
.656 .75 .919 r
.62888 .21961 .6378 .23148 .65581 .22511 .64697 .21317 Metetra
.655 .75 .919 r
.64697 .21317 .65581 .22511 .67395 .21871 .66518 .20669 Metetra
.655 .75 .919 r
.66518 .20669 .67395 .21871 .69219 .21226 .68351 .20018 Metetra
.655 .75 .92 r
.68351 .20018 .69219 .21226 .71056 .20578 .70196 .19362 Metetra
.655 .75 .92 r
.70196 .19362 .71056 .20578 .72905 .19925 .72054 .18702 Metetra
.655 .75 .92 r
.72054 .18702 .72905 .19925 .74766 .19268 .73923 .18038 Metetra
.655 .752 .921 r
.06491 .40799 .07603 .41763 .09073 .41241 .07965 .40273 Metetra
.655 .753 .922 r
.07965 .40273 .09073 .41241 .10551 .40715 .09448 .39743 Metetra
.655 .753 .922 r
.09448 .39743 .10551 .40715 .12038 .40185 .1094 .39209 Metetra
.655 .754 .922 r
.1094 .39209 .12038 .40185 .13534 .39652 .12441 .38672 Metetra
.656 .754 .922 r
.12441 .38672 .13534 .39652 .15039 .39115 .13951 .3813 Metetra
.656 .754 .922 r
.13951 .3813 .15039 .39115 .16553 .38574 .15469 .37585 Metetra
.656 .755 .922 r
.15469 .37585 .16553 .38574 .18076 .38029 .16997 .37035 Metetra
.656 .755 .922 r
.16997 .37035 .18076 .38029 .19607 .3748 .18535 .3648 Metetra
.657 .755 .922 r
.18535 .3648 .19607 .3748 .21149 .36928 .20081 .35922 Metetra
.657 .755 .922 r
.20081 .35922 .21149 .36928 .22699 .36371 .21637 .35359 Metetra
.657 .755 .922 r
.21637 .35359 .22699 .36371 .24259 .35811 .23202 .34792 Metetra
.657 .755 .922 r
.23202 .34792 .24259 .35811 .25827 .35247 .24777 .34221 Metetra
.658 .755 .922 r
.24777 .34221 .25827 .35247 .27406 .34679 .26361 .33645 Metetra
.658 .755 .921 r
.26361 .33645 .27406 .34679 .28994 .34108 .27955 .33065 Metetra
.658 .754 .921 r
.27955 .33065 .28994 .34108 .30591 .33532 .29559 .32482 Metetra
.658 .754 .921 r
.29559 .32482 .30591 .33532 .32199 .32954 .31172 .31894 Metetra
.658 .754 .921 r
.31172 .31894 .32199 .32954 .33816 .32371 .32795 .31303 Metetra
.658 .753 .92 r
.32795 .31303 .33816 .32371 .35442 .31786 .34428 .30709 Metetra
.658 .753 .92 r
.34428 .30709 .35442 .31786 .37079 .31197 .36071 .3011 Metetra
.658 .752 .92 r
.36071 .3011 .37079 .31197 .38726 .30604 .37724 .29509 Metetra
.658 .752 .919 r
.37724 .29509 .38726 .30604 .40383 .30008 .39388 .28904 Metetra
.658 .751 .919 r
.39388 .28904 .40383 .30008 .4205 .29409 .41061 .28296 Metetra
.657 .751 .919 r
.41061 .28296 .4205 .29409 .43727 .28807 .42745 .27685 Metetra
.657 .75 .918 r
.42745 .27685 .43727 .28807 .45415 .28201 .4444 .27071 Metetra
.657 .75 .918 r
.4444 .27071 .45415 .28201 .47113 .27593 .46145 .26454 Metetra
.657 .749 .918 r
.46145 .26454 .47113 .27593 .48822 .26981 .4786 .25834 Metetra
.657 .749 .918 r
.4786 .25834 .48822 .26981 .50541 .26365 .49587 .25211 Metetra
.656 .749 .918 r
.49587 .25211 .50541 .26365 .52272 .25747 .51324 .24585 Metetra
.656 .749 .918 r
.51324 .24585 .52272 .25747 .54013 .25124 .53073 .23955 Metetra
.656 .749 .918 r
.53073 .23955 .54013 .25124 .55765 .24499 .54832 .23322 Metetra
.656 .749 .918 r
.54832 .23322 .55765 .24499 .57529 .2387 .56603 .22686 Metetra
.656 .749 .918 r
.56603 .22686 .57529 .2387 .59304 .23237 .58386 .22047 Metetra
.655 .749 .919 r
.58386 .22047 .59304 .23237 .6109 .22601 .6018 .21404 Metetra
.655 .749 .919 r
.6018 .21404 .6109 .22601 .62888 .21961 .61985 .20757 Metetra
.655 .749 .919 r
.61985 .20757 .62888 .21961 .64697 .21317 .63802 .20107 Metetra
.655 .749 .919 r
.63802 .20107 .64697 .21317 .66518 .20669 .65632 .19452 Metetra
.655 .75 .919 r
.65632 .19452 .66518 .20669 .68351 .20018 .67473 .18794 Metetra
.655 .75 .919 r
.67473 .18794 .68351 .20018 .70196 .19362 .69326 .18131 Metetra
.655 .75 .92 r
.69326 .18131 .70196 .19362 .72054 .18702 .71192 .17464 Metetra
.655 .75 .92 r
.71192 .17464 .72054 .18702 .73923 .18038 .7307 .16793 Metetra
.654 .751 .921 r
.05368 .39819 .06491 .40799 .07965 .40273 .06847 .3929 Metetra
.654 .752 .921 r
.06847 .3929 .07965 .40273 .09448 .39743 .08334 .38758 Metetra
.654 .752 .922 r
.08334 .38758 .09448 .39743 .1094 .39209 .0983 .38222 Metetra
.654 .753 .922 r
.0983 .38222 .1094 .39209 .12441 .38672 .11335 .37682 Metetra
.655 .754 .923 r
.11335 .37682 .12441 .38672 .13951 .3813 .1285 .37138 Metetra
.655 .755 .923 r
.1285 .37138 .13951 .3813 .15469 .37585 .14373 .36589 Metetra
.655 .756 .923 r
.14373 .36589 .15469 .37585 .16997 .37035 .15906 .36036 Metetra
.656 .756 .924 r
.15906 .36036 .16997 .37035 .18535 .3648 .17449 .35478 Metetra
.656 .757 .924 r
.17449 .35478 .18535 .3648 .20081 .35922 .19 .34915 Metetra
.657 .757 .924 r
.19 .34915 .20081 .35922 .21637 .35359 .20562 .34346 Metetra
.657 .758 .924 r
.20562 .34346 .21637 .35359 .23202 .34792 .22133 .33773 Metetra
.658 .758 .924 r
.22133 .33773 .23202 .34792 .24777 .34221 .23713 .33194 Metetra
.658 .758 .923 r
.23713 .33194 .24777 .34221 .26361 .33645 .25304 .3261 Metetra
.659 .757 .923 r
.25304 .3261 .26361 .33645 .27955 .33065 .26904 .32021 Metetra
.659 .757 .922 r
.26904 .32021 .27955 .33065 .29559 .32482 .28513 .31428 Metetra
.659 .756 .922 r
.28513 .31428 .29559 .32482 .31172 .31894 .30133 .3083 Metetra
.659 .755 .921 r
.30133 .3083 .31172 .31894 .32795 .31303 .31763 .30228 Metetra
.659 .755 .921 r
.31763 .30228 .32795 .31303 .34428 .30709 .33402 .29622 Metetra
.659 .754 .92 r
.33402 .29622 .34428 .30709 .36071 .3011 .35052 .29013 Metetra
.659 .753 .919 r
.35052 .29013 .36071 .3011 .37724 .29509 .36712 .284 Metetra
.659 .752 .919 r
.36712 .284 .37724 .29509 .39388 .28904 .38381 .27784 Metetra
.658 .751 .918 r
.38381 .27784 .39388 .28904 .41061 .28296 .40062 .27166 Metetra
.658 .75 .918 r
.40062 .27166 .41061 .28296 .42745 .27685 .41752 .26544 Metetra
.658 .749 .917 r
.41752 .26544 .42745 .27685 .4444 .27071 .43454 .2592 Metetra
.657 .748 .917 r
.43454 .2592 .4444 .27071 .46145 .26454 .45165 .25294 Metetra
.657 .748 .917 r
.45165 .25294 .46145 .26454 .4786 .25834 .46888 .24665 Metetra
.657 .747 .917 r
.46888 .24665 .4786 .25834 .49587 .25211 .48621 .24034 Metetra
.656 .747 .917 r
.48621 .24034 .49587 .25211 .51324 .24585 .50366 .234 Metetra
.656 .747 .917 r
.50366 .234 .51324 .24585 .53073 .23955 .52122 .22764 Metetra
.656 .747 .917 r
.52122 .22764 .53073 .23955 .54832 .23322 .53888 .22125 Metetra
.655 .747 .917 r
.53888 .22125 .54832 .23322 .56603 .22686 .55667 .21482 Metetra
.655 .747 .918 r
.55667 .21482 .56603 .22686 .58386 .22047 .57457 .20837 Metetra
.655 .748 .918 r
.57457 .20837 .58386 .22047 .6018 .21404 .59258 .20188 Metetra
.655 .748 .918 r
.59258 .20188 .6018 .21404 .61985 .20757 .61072 .19536 Metetra
.655 .748 .919 r
.61072 .19536 .61985 .20757 .63802 .20107 .62897 .18879 Metetra
.655 .749 .919 r
.62897 .18879 .63802 .20107 .65632 .19452 .64734 .18219 Metetra
.655 .749 .919 r
.64734 .18219 .65632 .19452 .67473 .18794 .66584 .17555 Metetra
.655 .749 .92 r
.66584 .17555 .67473 .18794 .69326 .18131 .68445 .16886 Metetra
.655 .75 .92 r
.68445 .16886 .69326 .18131 .71192 .17464 .7032 .16212 Metetra
.655 .75 .92 r
.7032 .16212 .71192 .17464 .7307 .16793 .72206 .15534 Metetra
.653 .748 .919 r
.04235 .38815 .05368 .39819 .06847 .3929 .05718 .38283 Metetra
.653 .748 .92 r
.05718 .38283 .06847 .3929 .08334 .38758 .07209 .3775 Metetra
.653 .749 .921 r
.07209 .3775 .08334 .38758 .0983 .38222 .08709 .37213 Metetra
.653 .751 .922 r
.08709 .37213 .0983 .38222 .11335 .37682 .10219 .36674 Metetra
.653 .752 .922 r
.10219 .36674 .11335 .37682 .1285 .37138 .11737 .36131 Metetra
.653 .754 .923 r
.11737 .36131 .1285 .37138 .14373 .36589 .13265 .35584 Metetra
.653 .755 .925 r
.13265 .35584 .14373 .36589 .15906 .36036 .14802 .35032 Metetra
.654 .757 .925 r
.14802 .35032 .15906 .36036 .17449 .35478 .16349 .34475 Metetra
.655 .759 .926 r
.16349 .34475 .17449 .35478 .19 .34915 .17906 .33912 Metetra
.655 .76 .927 r
.17906 .33912 .19 .34915 .20562 .34346 .19472 .33343 Metetra
.656 .762 .927 r
.19472 .33343 .20562 .34346 .22133 .33773 .21049 .32766 Metetra
.657 .763 .927 r
.21049 .32766 .22133 .33773 .23713 .33194 .22635 .32182 Metetra
.658 .763 .927 r
.22635 .32182 .23713 .33194 .25304 .3261 .24231 .31591 Metetra
.659 .763 .927 r
.24231 .31591 .25304 .3261 .26904 .32021 .25838 .30992 Metetra
.66 .763 .926 r
.25838 .30992 .26904 .32021 .28513 .31428 .27454 .30386 Metetra
.661 .762 .925 r
.27454 .30386 .28513 .31428 .30133 .3083 .29081 .29774 Metetra
.662 .76 .923 r
.29081 .29774 .30133 .3083 .31763 .30228 .30717 .29157 Metetra
.662 .759 .922 r
.30717 .29157 .31763 .30228 .33402 .29622 .32364 .28534 Metetra
.662 .757 .92 r
.32364 .28534 .33402 .29622 .35052 .29013 .3402 .27907 Metetra
.662 .754 .919 r
.3402 .27907 .35052 .29013 .36712 .284 .35687 .27276 Metetra
.661 .752 .917 r
.35687 .27276 .36712 .284 .38381 .27784 .37364 .26643 Metetra
.661 .75 .916 r
.37364 .26643 .38381 .27784 .40062 .27166 .39051 .26009 Metetra
.66 .748 .915 r
.39051 .26009 .40062 .27166 .41752 .26544 .40749 .25373 Metetra
.659 .746 .914 r
.40749 .25373 .41752 .26544 .43454 .2592 .42457 .24736 Metetra
.658 .745 .914 r
.42457 .24736 .43454 .2592 .45165 .25294 .44175 .24099 Metetra
.657 .744 .913 r
.44175 .24099 .45165 .25294 .46888 .24665 .45905 .2346 Metetra
.656 .743 .913 r
.45905 .2346 .46888 .24665 .48621 .24034 .47645 .22822 Metetra
.656 .743 .914 r
.47645 .22822 .48621 .24034 .50366 .234 .49396 .22182 Metetra
.655 .743 .914 r
.49396 .22182 .50366 .234 .52122 .22764 .51159 .21541 Metetra
.655 .743 .915 r
.51159 .21541 .52122 .22764 .53888 .22125 .52933 .20898 Metetra
.654 .744 .915 r
.52933 .20898 .53888 .22125 .55667 .21482 .54719 .20253 Metetra
.654 .744 .916 r
.54719 .20253 .55667 .21482 .57457 .20837 .56516 .19604 Metetra
.654 .745 .917 r
.56516 .19604 .57457 .20837 .59258 .20188 .58326 .18953 Metetra
.654 .746 .918 r
.58326 .18953 .59258 .20188 .61072 .19536 .60147 .18298 Metetra
.654 .747 .919 r
.60147 .18298 .61072 .19536 .62897 .18879 .6198 .17638 Metetra
.654 .748 .919 r
.6198 .17638 .62897 .18879 .64734 .18219 .63826 .16974 Metetra
.654 .749 .92 r
.63826 .16974 .64734 .18219 .66584 .17555 .65684 .16306 Metetra
.654 .75 .921 r
.65684 .16306 .66584 .17555 .68445 .16886 .67554 .15632 Metetra
.654 .751 .921 r
.67554 .15632 .68445 .16886 .7032 .16212 .69437 .14952 Metetra
.654 .752 .921 r
.69437 .14952 .7032 .16212 .72206 .15534 .71332 .14268 Metetra
.654 .744 .916 r
.03093 .37779 .04235 .38815 .05718 .38283 .04581 .37236 Metetra
.654 .743 .915 r
.04581 .37236 .05718 .38283 .07209 .3775 .06077 .36693 Metetra
.653 .742 .915 r
.06077 .36693 .07209 .3775 .08709 .37213 .07582 .3615 Metetra
.652 .741 .915 r
.07582 .3615 .08709 .37213 .10219 .36674 .09096 .35607 Metetra
.651 .742 .916 r
.09096 .35607 .10219 .36674 .11737 .36131 .10618 .35065 Metetra
.649 .743 .918 r
.10618 .35065 .11737 .36131 .13265 .35584 .12149 .34525 Metetra
.648 .745 .92 r
.12149 .34525 .13265 .35584 .14802 .35032 .1369 .33984 Metetra
.648 .748 .923 r
.1369 .33984 .14802 .35032 .16349 .34475 .15239 .33444 Metetra
.647 .753 .927 r
.15239 .33444 .16349 .34475 .17906 .33912 .16798 .32901 Metetra
.647 .759 .93 r
.16798 .32901 .17906 .33912 .19472 .33343 .18367 .32354 Metetra
.648 .765 .934 r
.18367 .32354 .19472 .33343 .21049 .32766 .19947 .31799 Metetra
.65 .772 .937 r
.19947 .31799 .21049 .32766 .22635 .32182 .21537 .31233 Metetra
.653 .778 .94 r
.21537 .31233 .22635 .32182 .24231 .31591 .23138 .30652 Metetra
.657 .783 .941 r
.23138 .30652 .24231 .31591 .25838 .30992 .2475 .30055 Metetra
.661 .785 .94 r
.2475 .30055 .25838 .30992 .27454 .30386 .26374 .29439 Metetra
.665 .786 .938 r
.26374 .29439 .27454 .30386 .29081 .29774 .28008 .28804 Metetra
.669 .783 .935 r
.28008 .28804 .29081 .29774 .30717 .29157 .29654 .28153 Metetra
.671 .778 .93 r
.29654 .28153 .30717 .29157 .32364 .28534 .31309 .27488 Metetra
.673 .771 .924 r
.31309 .27488 .32364 .28534 .3402 .27907 .32975 .26813 Metetra
.672 .763 .919 r
.32975 .26813 .3402 .27907 .35687 .27276 .34651 .26133 Metetra
.671 .754 .913 r
.34651 .26133 .35687 .27276 .37364 .26643 .36337 .25453 Metetra
.668 .745 .908 r
.36337 .25453 .37364 .26643 .39051 .26009 .38032 .24777 Metetra
.665 .738 .904 r
.38032 .24777 .39051 .26009 .40749 .25373 .39737 .24108 Metetra
.662 .732 .902 r
.39737 .24108 .40749 .25373 .42457 .24736 .41451 .23449 Metetra
.658 .727 .9 r
.41451 .23449 .42457 .24736 .44175 .24099 .43177 .22799 Metetra
.655 .725 .901 r
.43177 .22799 .44175 .24099 .45905 .2346 .44912 .22157 Metetra
.653 .725 .902 r
.44912 .22157 .45905 .2346 .47645 .22822 .46659 .21523 Metetra
.651 .727 .905 r
.46659 .21523 .47645 .22822 .49396 .22182 .48416 .20893 Metetra
.65 .73 .908 r
.48416 .20893 .49396 .22182 .51159 .21541 .50186 .20265 Metetra
.649 .733 .911 r
.50186 .20265 .51159 .21541 .52933 .20898 .51967 .19636 Metetra
.649 .738 .914 r
.51967 .19636 .52933 .20898 .54719 .20253 .5376 .19003 Metetra
.649 .742 .917 r
.5376 .19003 .54719 .20253 .56516 .19604 .55565 .18366 Metetra
.65 .746 .92 r
.55565 .18366 .56516 .19604 .58326 .18953 .57382 .17722 Metetra
.651 .749 .922 r
.57382 .17722 .58326 .18953 .60147 .18298 .59212 .1707 Metetra
.652 .752 .923 r
.59212 .1707 .60147 .18298 .6198 .17638 .61054 .16411 Metetra
.652 .754 .924 r
.61054 .16411 .6198 .17638 .63826 .16974 .62908 .15744 Metetra
.653 .755 .924 r
.62908 .15744 .63826 .16974 .65684 .16306 .64774 .15069 Metetra
.654 .756 .925 r
.64774 .15069 .65684 .16306 .67554 .15632 .66653 .14387 Metetra
.654 .756 .925 r
.66653 .14387 .67554 .15632 .69437 .14952 .68545 .13699 Metetra
.655 .756 .924 r
.68545 .13699 .69437 .14952 .71332 .14268 .70449 .13003 Metetra
.656 .75 .919 r
.01934 .36758 .03093 .37779 .04581 .37236 .03427 .36205 Metetra
.656 .749 .918 r
.03427 .36205 .04581 .37236 .06077 .36693 .0493 .35648 Metetra
.656 .747 .917 r
.0493 .35648 .06077 .36693 .07582 .3615 .06441 .35089 Metetra
.656 .745 .916 r
.06441 .35089 .07582 .3615 .09096 .35607 .07962 .34525 Metetra
.655 .742 .914 r
.07962 .34525 .09096 .35607 .10618 .35065 .09491 .33959 Metetra
.655 .738 .911 r
.09491 .33959 .10618 .35065 .12149 .34525 .1103 .33388 Metetra
.655 .734 .908 r
.1103 .33388 .12149 .34525 .1369 .33984 .12578 .32815 Metetra
.654 .729 .904 r
.12578 .32815 .1369 .33984 .15239 .33444 .14136 .32238 Metetra
.654 .723 .899 r
.14136 .32238 .15239 .33444 .16798 .32901 .15704 .31657 Metetra
.653 .717 .895 r
.15704 .31657 .16798 .32901 .18367 .32354 .17281 .31073 Metetra
.653 .711 .89 r
.17281 .31073 .18367 .32354 .19947 .31799 .18867 .30485 Metetra
.652 .706 .887 r
.18867 .30485 .19947 .31799 .21537 .31233 .20464 .29894 Metetra
.652 .703 .884 r
.20464 .29894 .21537 .31233 .23138 .30652 .2207 .29299 Metetra
.652 .702 .883 r
.2207 .29299 .23138 .30652 .2475 .30055 .23687 .287 Metetra
.652 .703 .884 r
.23687 .287 .2475 .30055 .26374 .29439 .25313 .28097 Metetra
.652 .706 .887 r
.25313 .28097 .26374 .29439 .28008 .28804 .2695 .27491 Metetra
.653 .713 .892 r
.2695 .27491 .28008 .28804 .29654 .28153 .28597 .26881 Metetra
.653 .722 .899 r
.28597 .26881 .29654 .28153 .31309 .27488 .30254 .26269 Metetra
.96 .63 .339 r
.30254 .26269 .31309 .27488 .32975 .26813 .31127 .37549 Metetra
.31127 .37549 .32975 .26813 .34651 .26133 .33638 .24405 Metetra
.945 .712 .554 r
.33638 .24405 .34651 .26133 .36337 .25453 .35905 .13428 Metetra
0 0 .335 r
.35905 .13428 .36337 .25453 .38032 .24777 .36989 .2377 Metetra
.655 .783 .942 r
.36989 .2377 .38032 .24777 .39737 .24108 .387 .23138 Metetra
.655 .792 .948 r
.387 .23138 .39737 .24108 .41451 .23449 .40421 .22501 Metetra
.654 .799 .952 r
.40421 .22501 .41451 .23449 .43177 .22799 .42154 .21859 Metetra
.654 .802 .954 r
.42154 .21859 .43177 .22799 .44912 .22157 .43898 .21213 Metetra
.654 .803 .954 r
.43898 .21213 .44912 .22157 .46659 .21523 .45653 .20562 Metetra
.654 .801 .953 r
.45653 .20562 .46659 .21523 .48416 .20893 .4742 .19908 Metetra
.654 .797 .951 r
.4742 .19908 .48416 .20893 .50186 .20265 .49198 .19249 Metetra
.655 .792 .948 r
.49198 .19249 .50186 .20265 .51967 .19636 .50988 .18586 Metetra
.655 .787 .944 r
.50988 .18586 .51967 .19636 .5376 .19003 .5279 .17919 Metetra
.656 .781 .941 r
.5279 .17919 .5376 .19003 .55565 .18366 .54603 .17247 Metetra
.656 .775 .937 r
.54603 .17247 .55565 .18366 .57382 .17722 .56429 .1657 Metetra
.656 .77 .934 r
.56429 .1657 .57382 .17722 .59212 .1707 .58266 .1589 Metetra
.656 .766 .931 r
.58266 .1589 .59212 .1707 .61054 .16411 .60116 .15204 Metetra
.656 .762 .928 r
.60116 .15204 .61054 .16411 .62908 .15744 .61978 .14515 Metetra
.656 .76 .926 r
.61978 .14515 .62908 .15744 .64774 .15069 .63853 .1382 Metetra
.656 .757 .924 r
.63853 .1382 .64774 .15069 .66653 .14387 .6574 .13121 Metetra
.656 .755 .923 r
.6574 .13121 .66653 .14387 .68545 .13699 .6764 .12417 Metetra
.656 .754 .922 r
.6764 .12417 .68545 .13699 .70449 .13003 .69553 .11708 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(90)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(95)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(100)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(105)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(110)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakBelow->False,
  ImageSize->{167.562, 135.75},
  ImageMargins->{{33.125, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002W0000Qa000`40O003h00OYgoo002WOol00:Mo
o`00Ygoo002WOol00:Moo`00Ygoo002WOol00:Moo`00I7oo0`000goo0`0017oo0P00=7oo001UOol0
0`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`00<goo001UOol00`00Oomoo`03Ool00`00Oomo
o`02Ool01000Oomoo`00<goo001UOol00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`00<goo
001UOol00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`00<goo001TOol20004Ool20005Ool2
000dOol00:Moo`00Ygoo001bOol2000cOol004]oo`<000Aoo`8000=oo`<001Eoo`<00003Ool007oo
039oo`00C7oo00<007ooOol00Woo00@007ooOol000Eoo`03001oogoo011oo`8000Aoo`80039oo`00
C7oo00<007ooOol00Woo00@007ooOol000Eoo`03001oogoo00eoo`<00003Ool007oo00=oo`03001o
o`00035oo`00C7oo00<007ooOol00Woo00@007ooOol0009oo`<000ioo`8000Uoo`03001oo`00035o
o`00C7oo00<007ooOol00Woo00@007ooOol0009oo`03001oogoo00]oo`<00003Ool007oo00Qoo`04
001oogoo000`Ool004]oo`8000Eoo`8000=oo`@000Qoo`8000ioo`05001oogooOol0000_Ool0061o
o`<00003Ool007oo00eoo`05001oogooOol0000_Ool005ioo`<0019oo`04001oogooOol2000^Ool0
05]oo`<001Ioo`05001oogooOol0000^Ool003=oo`<000Aoo`8000Aoo`8001Moo`<001Qoo`03001o
ogoo009oo`03001oogoo00=oo`8000Aoo`8000=oo`@001Uoo`00=7oo00<007ooOol00Woo00@007oo
Ool0009oo`04001oogoo000COol30002Ool00`00Oomoo`0FOol00`00Oomoo`02Ool20003Ool01000
Oomoo`000goo0P0017oo00<007ooOol06Goo000dOol00`00Oomoo`02Ool01000Oomoo`000Woo00@0
07ooOol0015oo`<001eoo`03001oogoo00=oo`05001oogooOol00002Ool00`00Oomoo`08Ool00`00
Oomoo`0HOol003Aoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`003Woo0`0087oo00<0
07ooOol017oo00@007ooOol0009oo`03001oogoo00Uoo`03001oogoo01Moo`00=7oo00<007ooOol0
0Woo00@007ooOol0009oo`04001oogoo000<Ool4000QOol00`00Oomoo`04Ool200001Goo001oogoo
000027oo00@007ooOol001Uoo`00<goo0P001Goo0P0017oo0P002Woo0`009Goo00<007ooOol01Goo
00<007ooOol00P002Woo0P006Woo001:Ool200000goo001oo`0TOol20008Ool00`00Oomoo`0WOol0
04Moo`<002Ioo`<00003Dad007oo00Qoo`03001oogoo02Ioo`00A7oo0`009goo0P0000AC7@00DaeC
7@8000Moo`8002Qoo`008goo0`000goo0`005Woo1@0097oo0`0000=C7@00Dad01@002Goo00<007oo
Ool09Goo000VOol00`00Oomoo`03Ool00`00Oomoo`0@Ool30004Ool00`00Oomoo`0OOol3000015<M
001C7E<M0P000U<M10002Goo00<007ooOol097oo000TOol30005Ool00`00Oomoo`0>Ool200000goo
001oo`04Ool00`00Oomoo`0MOol20002Dad00`00Dad00002000015<M001C7E<M0P0000=C7@000000
1goo0P009Woo000SOol01000Oomoo`000Woo0`003Woo0`0027oo0P0077oo0`0000AC7@00DaeC7@80
0004Dad005<MDad300001E<M000005;m00002Goo00<007ooOol017oo0P0017oo0P0017oo0`0047oo
000SOol01000Oomoo`000Woo00<007ooOol037oo0P000Woo00<007ooOol01Woo0`006Goo0P000U<M
00<005<M00000P0000=C7@0000000`0000=Bo@0000000`0000=Bo@00000027oo00<007ooOol00goo
00@007ooOol000=oo`8000Eoo`03001oogoo00moo`0097oo0P000goo100027oo0`003Goo0`005Woo
0`0000=C7@0000000`0000IC7@00000005;m0002D_d200000e;m001Bo@030002D_d00`00Oomoo`05
Ool20004Ool01000Oomoo`0027oo100047oo000cOol2000@Ool3000DOol20002Dah2000015;m001B
oE;m0P000U;m00<005;m00000P0000=Bo@00D_d00`0000ABo@00D_eBo@8000Qoo`05001oogooOol0
0002Ool00`00Oomoo`06Ool00`00Ool0000AOol0031oo`@0015oo`<0019oo`800007Dch00000001B
o@00D_d00`0000ABo@00D_eBo@800004D_d005;mD_d20002D_d500000e;m001oo`07Ool01@00Oomo
ogoo00000Woo00<007ooOol01Woo00<007oo00004Goo000>Ool30004Ool2000GOol2000DOol01000
N]4000003Goo1@0000=C?P0000000U;m00@005;m0000009Bo@03001Bo@0000800003D_d005;m00<0
0004D_d005;mD_d200000e;m000000030007Ool20003Ool2000:Ool2000AOol0015oo`04001oogoo
0002Ool00`00Oomoo`0AOol300000goo001oo`0COol01000N]5jd@002Woo0`0000=C?P00Dch00`00
00ABo@00D_eBo@800004D_d005;mD_d2000015;m001BoE;m0P000U;m1@0000EBo@00001Bo@00009B
o@8000Qoo`03001oogoo01moo`003goo0`000Woo00@007ooOol0015oo`<001Qoo`04001jdG[A0008
Ool20002Dch01000Dch000000U;m00<005;m00000P0000=Bo@00D_d00`0000=Bo@0000000`0000MB
o@00000005;m001Bo@0500001U;m00000000Dad000Qoo`03001oogoo01moo`003Woo00@007ooOol0
009oo`04001oogoo000>Ool3000KOol00`00N]5jd@020005Ool200000e<n0000000300000e;l0000
000300000e;m000000030002D_d2000015;m001BoE;m0P0000ABo@00D_eBo@800003D_d005;m00<0
0004Dad005<MDad20007Ool2000POol000ioo`04001oogoo0002Ool01000Oomoo`0037oo0P0000=o
o`00Ool06goo00<007[AN]400P000Woo0`0000=C?P0000000U;l00@005;l0000009Bo@04001Bo@00
0002D_d00`00D_d0000200000e;m001Bo@0300000e;m001Bo@0300000e;m001Bo@0300000e;m001C
7@0300000e<M00000008Ool00`00Oomoo`0MOol000moo`8000Aoo`8000Yoo`<001moo`04001jdG[A
N]440002Dch2000015Kl001Bo5;l0P0000ABo@00D_eBo@800004D_d005;mD_d2000015;m001BoE;m
0P0000ABo@00D_eBo@800004D_d005;mD_d20002D_d40002Dad200000e<M001oo`07Ool00`00Oomo
o`0MOol001ioo`<0029oo`04001jdG[AN]4200001e<n00000000E_`005Kl00<00003E_d005;m00<0
0003D_d005;m00<00003D_d005;m00<00003D_d0000000<00003D_d0000000<00003D_d0000000<0
0007Dad00000001C7@00Dad00P001goo0P007Woo000LOol4000SOol01000N]5jdG[A0`000UKl00@0
05Kl0000009Fo@04001Fo@000002E_d01000D_d000000U;m10000U;m0P0000ABo@00D_eBo@800004
D_d005;mD_d200000e<M00000003000015<M001C7E<M0P0000=C7@00Ool027oo00<007ooOol00goo
0P0017oo0P0017oo0P002Woo000KOol2000VOol200001G[A000005Km00000UKl0P0000AFo@00E_eF
o@800003E_d0000000<00007D_d00000001Bo@00D_d00`0000=Bo@00D_d00`0000=Bo@00D_d00`00
00=C7@0000000`0000=C7@00Dad00`0000=C7@0000000P0027oo0P000goo00@007ooOol000=oo`80
00=oo`04001oogoo0009Ool001Yoo`800003Ool007oo01ioo`8000Aoo`<00003Ead005Km00<00003
E_d0000000<00003E_d00000009Fo@04001Fo@000002D_d01P00D_d00000D_d0009Bo@800004D_d0
05;mD_d200000e<M0000000300000e<M0000000300001e<M00000000Dad005<M008000Moo`8000=o
o`04001oogoo0008Ool01000Oomoo`002Goo000JOol20002Ool00`00Oomoo`0JOol2000015;l0000
07oo0`0000=G7@0000000P000UKm0P0000AFo@00E_eFo@800004E_d005KmE_d2000015;m001BoE;m
0P0000=Bo@00D_d00`0000=Bo@0000001P000U<M00L005<M000005<M001C7@03000015<M001C7E<M
0P0000=C7@00Ool027oo00@007ooOol0009oo`03001oogoo00Ioo`<000Yoo`0057oo0`000goo0P00
0goo00<007ooOol05Goo10000U;L0P0000=jRP00NXX00`0000=G7@00Ead00`0000=G7@00E_d00`00
00=Fo@00E_d00`0000=Fo@00D_d00`0000=Bo@0000000P000U;m0P0000=C7@0000000`0000AC7@00
DaeC7@800003Dad005<M00D00005Dad00000001C7@040008Ool01000Oomoo`000Woo00<007ooOol0
1Woo00<007ooOol02Woo000EOol00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`0BOol20000
0e;L001Bg0050003NXX01000Ead005LM0`0000=G7@00Ead00P000ULM10000UKm100000MBo@000000
05<M001C7@0300000e<M0000000300000e<M001C7@0300000e<M001C7@020002Dad400001e<M0000
0000Dad005<M008000Moo`80009oo`8000Yoo`<000Uoo`003Woo1@000Woo00<007ooOol00Woo00<0
07ooOol017oo00<007ooOol03Woo0`0000=Bg000D]`00P000ULn00<005Lm00000gZ:00@005LM0000
009G7@@00007Ead00000001G7@00Ead00`0000=C7@00Dad00P000U<M00D005<M000005<M00D00003
Dad0000000<00003Dad0000000<00005Dad00000001C7@040002Dad01P00Dad00000Dad000Qoo`03
001oogoo01Qoo`005Goo00<007ooOol00Woo00<007ooOol01Goo00<007ooOol02goo0P000U;L00@0
05;L0000009G?P07001G?P00001G7@00NXX00`0000MG7@00000005LM001G7@020002Ead01000Ead0
00000ULM00H005<M000005<M0002Dad800000e<M00000002Dad01P00Dad00000Dad0009C7@800004
Dad005<MDad40002Dad200000e<M000000020008Ool00`00Oomoo`0GOol001Eoo`04001oogooOol2
0008Ool00`00Oomoo`08Ool20002D]`01@00D]`00000Ech01@0000EG7@00000007Z:00@0009G7@06
001G7@00001G7@000ULM0P0000=G7@0000000`0000=C7@0000001P0000AC7@00DaeC7@800004Dad0
05<MDad200000e<M001C7@0300000e<M0000000300000e<M0000000300001U<M000005<M001C7@80
00Qoo`03001oogoo01Ioo`0057oo0P000goo0P002Goo00<007ooOol00goo100000ABg000D]aBg080
0003Dah00000009G7@03001G7@0000800003Ead007Z:00800003Ead0000000<00003Ead0000000<0
0003Ead00000009C7@05001C7@00001C7@0500000e<M001C7@0300000e<M001C7@0300000e<M0000
00020002Dad200000e<M001C7@0300000e<M0000000300000e<M001C7@0200000e<M001oo`07Ool0
0`00Oomoo`0FOol000Qoo`8000Aoo`8000=oo`<000=oo`03001oogoo00Uoo`03001oogoo00800003
D_`005;L00D00007Dad00000001G7@00Ead00P000ULM0`0000AjRP000000009G7@04001G7@000002
Ead01P00Ead00000Ead0009C7@P00003Dad000000080009C7@@00007Dad00000001C7@00Dad00`00
00=C7@00Dad00P0000AC7@00DaeC7@@0009C7@800003Dad00000008000Moo`8001Moo`001goo00@0
07ooOol000=oo`8000Ioo`04001oogoo000<Ool200000e;l001Bo00300000e<M001C7@020002Ead4
00001ELM00000000Ead00P0000IjRP00EaeG7@00Ead2000015LM001G7ELM0P0000=G7@0000002000
0U<M0P0000=C7@00Dad00`0000=C7@00Dad00P000U<M00H005<M000005<M0002Dad200000e<M0000
000300000e<M0000000300001U<M000005<M001C7@8000Qoo`03001oogoo00=oo`8000Aoo`8000Ao
o`8000=oo`0000=oo`0000000`0000Eoo`00Oomoo`0000]oo`04001oogoo0007Ool500000e;m0000
000300000e<M0000000200001e<M00000000Ead005LM00<00003Ead005LM00800003NXX0000000<0
0003Ead0000000<00004Ead00000Dad500000e<M001C7@0300000e<M001C7@020002Dad01@00Dad0
0000Dad01@0000=C7@0000000`0000EC7@00001C7@00009C7@800003Dad0000000<00003Dad005<M
00800003Dad0000000Moo`8000=oo`04001oogoo0003Ool20003Ool01000Oomoo`000Woo0007Ool0
1000Oomoo`0027oo0`000goo00<007ooOol00goo0P000U;m00@005;m0000009Bo@07001Bo@00001C
7@00Dad00`0000=G7@00Ead00P000ULM100000MG7@00NXX005LM001G7@02000015LM001G7ELM0P00
00=C7@0000000`0000=C7@00Dad00P000U<M100000=C7@0000000`0000EC7@00001C7@00009C7@80
0003Dad005<M00<00003Dad005<M00<00003Dad0000000P00005Dad00000Dad00008Ool01@00Oomo
ogoo00000Woo00<007ooOol01Woo00@007ooOol0009oo`001goo00@007ooOol000Qoo`03001oogoo
00=oo`04001oogooOol300000e;m001Bo@02000015;m001BoE;m0P0000=C7@0000000`0000=G7@00
00000P0000MG7@00000005LM001G7@0500001ULM000005LM001G7@P0009C7@@00007Dad00000001C
7@00Dad00P0000AC7@00DaeC7@800003Dad005<M00<00003Dad000000080009C7@@00007Dad00000
001C7@00Dad010000U<M0P0000=C7@00Dad00P0027oo00@007ooOol0009oo`03001oogoo00Moo`80
00=oo`0027oo0P002Goo100000=oo`000000100000=Bo@0000000P0000=Bo@0000000`0000=C7@00
00000U<M00L005<M000005LM001G7@0300000eLM001G7@020002Ead400000eLM0000000600000e<M
001C7@0300000e<M001C7@020002Dad01@00Dad00000Dad01@0000=C7@0000000P000U<M0P0000=C
7@00Dad00`0000=C7@00Dad00P0000AC7@00DaeC7@800003Dad005<M00<00003Dad0000000800003
Dad0000000Moo`04001oogoo0002Ool00`00Oomoo`06Ool01000Oomoo`000Woo000HOol01@00D_eB
o@00D_d00P0000=Bo@00D_d00`0000=C7@00Dad00P0000AC7@00DaeC7@800003Ead0000000<00003
Ead0000000800007Ead00000001G7@00Ead00P0000=G7@0000000`0000=C7@00Dad00`0000=C7@00
00000P0000=C7@0000000`0000EC7@00001C7@00009C7@800003Dad005<M00<00003Dad005<M0080
0003Dad0000000<00003Dad0000000<00003Dad0000000H00003Dad005<M00800003Dad007oo00Mo
o`03001oogoo008000Yoo`8000=oo`004Woo0P0017oo0P0000IBo@00001Bo@00D_d300000e<M0000
000200000e<M0000000300000e<M00000002Ead01`00Ead00000Ead005LM00<00003Ead005LM00P0
0003Dad000000080009C7@800003Dad005<M0080009C7@07001C7@00001C7@00Dad00`0000=C7@00
00000P0000=C7@0000000`0000EC7@00001C7@00009C7@800003Dad005<M00<00003Dad005<M00@0
009C7@800003Dad00000008000Qoo`03001oogoo011oo`004Goo00@007ooOol000=oo`04001oo`00
D_d500001U;m000005<M001C7@<00003Dad005<M00800004Ead005LMEad200000eLM000000030000
0eLM000000050002Dad01`00Dad00000Dad005<M00<00003Dad005<M00800003Dad0000000<00003
Dad000000080009C7@800003Dad005<M0080009C7@07001C7@00001C7@00Dad00`0000=C7@000000
0P0000=C7@0000000`0000=C7@0000000`0000IC7@00001C7@00Dad20007Ool00`00Oomoo`0@Ool0
015oo`04001oogoo0003Ool00`00Oomoo`0200000e;m001Bo@0200000e<M001C7@0300000e<M0000
000200000e<M0000000300001ELM000005LM00000ULM1`000U<M00D005<M000005<M00D00003Dad0
000000800003Dad00000009C7@07001C7@00001C7@00Dad00`0000=C7@00Dad00P0000=C7@000000
0`0000=C7@0000000P0000MC7@00000005<M001C7@02000015<M001C7E<M0P0000=C7@0000000`00
00=C7@0000000P0000=C7@0000001goo00<007ooOol03goo000AOol01000Oomoo`000Woo0P0017oo
0`0000=C7@0000000`0000IC7@00001C7@00Dad300000e<M001G7@0200000eLM0000000800000e<M
0000000300001E<M000005<M00000U<M0P0000=C7@00Dad00P0000AC7@00DaeC7@800003Dad00000
0080009C7@@00005Dad00000Dad00002Dad200000e<M001C7@020002Dad01@00Dad00000Dad01@00
00=C7@0000001P0000=C7@00Dad00P0000=C7@00Ool01Woo0P0047oo000AOol01000Oomoo`000Woo
00<007ooOol00goo00<005<M00000P0000=C7@00Dad00P0000=C7@0000000`0000=C7@0000000P00
00AG7@00001G7@D00003Dad005<M00<00003Dad005<M00800003Dad0000000<00003Dad000000080
0003Dad0000000<00006Dad00000Dad005<M0`0000=C7@00Dad00P0000=C7@0000000`0000=C7@00
00000P0000=C7@0000000`0000IC7@00001C7@00Dad300000e<M001C7@040002Dad200000e<M0000
00020007Ool00`00Oomoo`02Ool30009Ool0019oo`8000=oo`8000Eoo`800003Dad0000000800007
Dad00000001C7@00Dad00P0000=C7@00Dad01@0000AG7@00001C7@D00003Dad0000000800003Dad0
0000009C7@07001C7@00001C7@00Dad00P000U<M00D005<M000005<M00@0009C7@@00005Dad00000
Dad00002Dad200000e<M001C7@02000015<M001C7E<M0P0000=C7@0000000P0000=C7@0000000`00
00=C7@0000001P0000=C7@00Dad00P001goo00<007ooOol00Woo00<007ooOol027oo000GOol20007
Ool01000Dad005<M0`0000=C7@0000000P0000=C7@000000200000=C7@0000000U<M00L005<M0000
05<M001C7@0200000e<M0000000300000e<M0000000200000e<M0000000300001U<M000005<M001C
7@<00003Dad005<M00800003Dad005<M00<00003Dad0000000800003Dad0000000<00006Dad00000
Dad005<M0P0000AC7@00DaeC7@800003Dad0000000800003Dad0000000<00003Dad0000000Ioo`03
001oogoo009oo`03001oogoo00Qoo`001Goo0P0017oo0P000goo0`0017oo00<007ooOol01goo1000
00IC7@00001C7@00Dad800000e<M001C7@0200000e<M0000000300000e<M0000000200001e<M0000
0000Dad005<M00800003Dad005<M0080009C7@05001C7@00001C7@040002Dad400000e<M00000002
00001e<M00000000Dad005<M00800003Dad005<M00<00003Dad0000000800003Dad0000000<00003
Dad0000000D00004Dad005<MDad200000e<M001oo`06Ool01@00Oomoogoo00002Woo0004Ool01000
Oomoo`000goo0P001Woo00D007ooOomoo`0000Yoo`05001C7@00001C7@0:00000e<M000000020000
1e<M00000000Dad005<M00800003Dad005<M00<00003Dad0000000800003Dad0000000800003Dad0
000000<00006Dad00000Dad005<M0`0000=C7@00Dad00P0000=C7@00Dad00P000U<M100000=C7@00
00000P0000MC7@00000005<M001C7@0200000e<M001C7@0300000e<M001C7@0=0005Ool30002Ool0
0`00Oomoo`08Ool000Aoo`04001oogoo000;Ool00`00Oomoo`02000;Ool00`00Dad0000700000e<M
001C7@0200000e<M0000000300000e<M0000000200000e<M0000000300001U<M000005<M001C7@80
0003Dad005<M00<00003Dad005<M00800003Dad000000080009C7@@00003Dad0000000800007Dad0
0000001C7@00Dad00P0000=C7@00Dad00P0000=C7@0000000`0000=C7@0000000P0000=C7@000000
0`0000=C7@0000000P0000EC7@00001C7@00009C7@800004Ool000000002Ool01000Ool000002Woo
0004Ool01000Oomoo`0027oo0`000goo0P0037oo1`0000=C7@0000000P0000MC7@00000005<M001C
7@0200000e<M001C7@02000015<M001C7E<M0P0000=C7@0000000P0000=C7@0000000`0000=C7@00
00000P0000IC7@00001C7@00Dad300000e<M001C7@0200000e<M0000000200000e<M000000030000
0e<M0000000200001E<M000005<M00000U<M0P0000=C7@00Dad00P0000AC7@00DaeC7@800003Dad0
000000800003Dad0000000<00003Dad0000000Eoo`03001oogoo00]oo`0017oo00@007ooOol000Qo
o`03001oogoo00=oo`03001oogoo00Uoo`<00005Ool005<M001C7@0200000e<M001C7@0300000e<M
0000000200000e<M0000000200000e<M0000000300001U<M000005<M001C7@800004Dad005<MDad2
00000e<M0000000200000e<M000000020002Dad400001U<M000005<M001C7@80009C7@07001C7@00
001C7@00Dad00P0000=C7@0000000`0000=C7@0000000P0000=C7@0000001P0000=C7@0000000P00
00=C7@00Dad01@001Goo00<007ooOol02goo0005Ool20009Ool40002Ool20007Ool30005Ool30000
0e<M0000000300001U<M000005<M001C7@800003Dad005<M00<00003Dad0000000800003Dad00000
00800003Dad0000000<00006Dad00000Dad005<M0P0000=C7@00Dad00`0000=C7@0000000P0000=C
7@0000000P0000=C7@0000000`0000=C7@0000000P0000IC7@00001C7@00Dad300000e<M001C7@02
00000e<M0000000200000e<M0000000800001E<M000005<M00001Goo00<007ooOol02goo000FOol2
0004Ool30008Ool00`00Dad0000200000e<M0000000200000e<M0000000200000e<M000000030000
1U<M000005<M001C7@800003Dad005<M00<00003Dad0000000800003Dad0000000800003Dad00000
00<00006Dad00000Dad005<M0P0000=C7@00Dad00P0000AC7@00DaeC7@800003Dad0000000800003
Dad0000000800003Dad0000000<00006Dad00000Dad005<M0P0000AC7@00DaeC7@800003Dad00000
00800003Dad00000008000Eoo`03001oogoo00Yoo`003goo0`000goo00@007ooOomoo`<000aoo`80
0006Dad00000Dad005<M0P0000=C7@00Dad00`0000=C7@0000000P0000=C7@0000000P0000=C7@00
00000`0000IC7@00001C7@00Dad200000e<M001C7@0200000e<M0000000300000e<M000000020000
0e<M0000000200000e<M0000000300001U<M000005<M001C7@800003Dad005<M00800003Dad00000
00<00003Dad0000000800003Dad0000000H00003Dad005<M00800003Dad005<M008000Aoo`03001o
ogoo00Yoo`0047oo00<007ooOol00Woo10004Goo00<005<M00000`0000=C7@0000000`0000IC7@00
001C7@00Dad200000e<M001C7@0300000e<M0000000200000e<M0000000200000e<M000000020000
1e<M00000000Dad005<M00800003Dad005<M00800003Dad0000000800003Dad0000000<00003Dad0
000000800003Dad0000000800005Dad00000Dad00002Dad200000e<M001C7@0200000e<M00000002
00000e<M0000000300000e<M0000000200000e<M00000003Ool00`00Oomoo`0:Ool0011oo`03001o
ogoo009oo`8001Aoo`800003Dad005<M00800003Dad0000000800003Dad0000000800003Dad00000
00<00006Dad00000Dad005<M0P0000=C7@00Dad00P0000=C7@0000000P000U<M100000=C7@000000
0P0000IC7@00001C7@00Dad2000015<M001C7E<M0P0000=C7@0000000P0000=C7@0000000P0000=C
7@0000000`0000=C7@0000000P0000IC7@00001C7@00Dad200000e<M001C7@0800000e<M00000002
Ool00`00Oomoo`0:Ool0011oo`03001oogoo00Aoo`03001oogoo01=oo`<00006Dad00000Dad005<M
0P0000=C7@00Dad00`0000=C7@0000000P0000=C7@0000000P0000=C7@0000000P0000IC7@00001C
7@00Dad300000e<M001C7@0200000e<M0000000200000e<M0000000200000e<M0000000300001U<M
000005<M001C7@800003Dad005<M00800003Dad0000000P00003Dad0000000800003Dad0000000D0
0004Dad005<MDad200001E<M001oogoo000037oo000@Ool00`00Oomoo`05Ool00`00Oomoo`0COol3
00000e<M0000000200000e<M0000000300001U<M000005<M001C7@800003Dad0000000800003Dad0
000000800003Dad0000000<00003Dad0000000800006Dad00000Dad005<M0P0000=C7@00Dad00P00
00=C7@0000000`0000=C7@0000000P0000=C7@0000000P0000IC7@00001C7@00Dad2000015<M001C
7E<M0P0000=C7@0000000P0000=C7@0000003@0000=oo`00Ool02goo000?Ool20008Ool00`00Oomo
o`0BOol00`00Dad0000200000e<M0000000200000e<M0000000200000e<M0000000200001U<M0000
05<M001C7@800003Dad005<M00<00003Dad0000000800003Dad0000000800003Dad0000000800003
Dad0000000800005Dad00000Dad00002Dad200000e<M0000000200000e<M0000000200000e<M0000
000200000e<M0000000600000e<M001C7@0200000e<M001C7@0700000e<M0000000200000goo001o
o`0;Ool001Yoo`8001=oo`800006Dad00000Dad005<M0P0000=C7@0000000P0000=C7@0000000P00
00=C7@0000000P0000=C7@0000000`0000IC7@00001C7@00Dad200000e<M001C7@0200000e<M0000
000200000e<M0000000200000e<M0000000300001U<M000005<M001C7@800003Dad005<M00800003
Dad0000000800003Dad0000000800003Dad0000000<00003Dad0000000800003Dad0000000800006
Dad00000Dad005<M0P0000=oo`00Ool02Woo000LOol00`00Oomoo`0BOol00`00Dad0000300001U<M
000005<M001C7@800003Dad005<M00<00003Dad0000000800003Dad0000000800003Dad000000080
0003Dad0000000800006Dad00000Dad005<M0P0000=C7@0000000P0000=C7@0000000`0000=C7@00
00000P0000=C7@0000000P0000IC7@00001C7@00Dad200000e<M001C7@0200000e<M000000080000
0e<M0000000200000e<M00000002000015<M000000002goo000MOol00`00Oomoo`0BOol00`00Dad0
000300000e<M0000000200000e<M0000000300001U<M000005<M001C7@800003Dad0000000800003
Dad0000000800003Dad0000000800003Dad0000000800006Dad00000Dad005<M0P0000AC7@00DaeC
7@800003Dad0000000800003Dad0000000800003Dad0000000800003Dad0000000800006Dad00000
Dad005<M0P0000=C7@000000200000=C7@0000000P0000=C7@0000002goo000NOol00`00Oomoo`0B
Ool200000e<M0000000200000e<M0000000200000e<M0000000200000e<M0000000200001U<M0000
05<M001C7@800003Dad005<M00800003Dad0000000800003Dad0000000800003Dad0000000800003
Dad0000000<00006Dad00000Dad005<M0P0000=C7@0000000P0000=C7@0000000P0000=C7@000000
0P0000=C7@0000000P0000IC7@00001C7@00Dad200000e<M0000000800000goo001oo`0:Ool001mo
o`8001Aoo`04001C7@00Dad200000e<M0000000200000e<M0000000200000e<M0000000200000e<M
0000000200000e<M0000000200001U<M000005<M001C7@800003Dad0000000800003Dad000000080
0003Dad0000000<00003Dad0000000800006Dad00000Dad005<M0P0000=C7@0000000P0000=C7@00
00000P0000=C7@0000000P0000=C7@0000000P0000AC7@00001C7@@000Ioo`03001oogoo00Uoo`00
8Goo00<007ooOol04Woo0`0000IC7@00001C7@00Dad200000e<M001C7@0200000e<M000000020000
0e<M0000000200000e<M0000000200000e<M0000000200001U<M000005<M001C7@800003Dad00000
00800003Dad0000000<00003Dad0000000800003Dad0000000800006Dad00000Dad005<M0P0000=C
7@0000000P0000=C7@0000000P0000=C7@0000000P0000=C7@0000000`001goo00<007ooOol02Goo
000ROol00`00Oomoo`0BOol300000e<M0000000200000e<M0000000200001U<M000005<M001C7@80
0003Dad0000000800003Dad0000000800003Dad0000000800003Dad0000000800006Dad00000Dad0
05<M0P0000=C7@0000000`0000=C7@0000000P0000=C7@0000000P0000=C7@0000000P0000IC7@00
001C7@00Dad200000e<M001C7@0200000e<M000000020005Ool00`00Oomoo`04Ool00`00Oomoo`09
Ool002=oo`03001oogoo019oo`<00003Dad0000000800003Dad0000000800003Dad0000000800006
Dad00000Dad005<M0P0000=C7@0000000P0000=C7@0000000P0000=C7@0000000P0000=C7@000000
0P0000EC7@00001C7@00009C7@800003Dad0000000800003Dad0000000800003Dad0000000800003
Dad0000000800003Dad0000000800003Dad0000000Yoo`03001oogoo00=oo`03001oogoo00Uoo`00
97oo0P0057oo0`0000=C7@0000000P0000=C7@0000000P0000=C7@0000000P0000=C7@0000000P00
00AC7@00001C7@@00003Dad0000000800003Dad0000000800005Dad00000001C7@0800000e<M0000
000F000<Ool00`00Oomoo`04Ool00`00Oomoo`08Ool002Ioo`03001oogoo015oo`05001C7@00001C
7@0400000e<M0000000200000e<M0000000200000e<M0000000200001U<M000005<M001C7@800003
Dad0000000800003Dad0000000800005Dad00000001C7@0400000e<M0000000200001U<M000005<M
001C7@800003Dad0000000800003Dad000000080019oo`03001oogoo00=oo`03001oogoo00Qoo`00
9goo00<007ooOol04Goo00L005<M000005<M001C7@0200000e<M0000000200000e<M000000020000
0e<M0000000200000e<M0000000200001U<M000005<M001C7@800003Dad0000000800005Dad00000
001C7@0400000e<M0000000200000e<M00000002000015<M000005<M10005goo00<007ooOol00Woo
00<007ooOol027oo000XOol00`00Oomoo`0AOol00`00Dad00003000015<M000005<M100000=C7@00
00000P0000=C7@0000000P0000=C7@0000000P0000=C7@0000000P0000IC7@00001C7@00Dad20000
1E<M00000000Dad0100000=C7@0000000P0000=C7@0000000P0000=C7@00000077oo00D007ooOomo
o`0000Yoo`00:Goo0P004Woo00<005<M00000`0000IC7@00001C7@00Dad200000e<M000000020000
0e<M0000000200000e<M0000000200000e<M0000000200000e<M00000002000015<M000005<M0`00
00=C7@0000000P0000=C7@0000000P0000=C7@0000000P007Woo00D007ooOomoo`0000Yoo`00:goo
00<007ooOol047oo0P0000EC7@00000005<M00@00004Dad00000Dad400000e<M0000000200000e<M
0000000200000e<M0000000200000e<M00000002000015<M000005<M0`0000=C7@0000000P0000=C
7@0000000P008goo00@007ooOol000Yoo`00;7oo00<007ooOol04Goo00<005<M00000`0000EC7@00
000005<M00800003Dad0000000800003Dad0000000800003Dad0000000800003Dad0000000800003
Dad0000000800005Dad00000001C7@0200000e<M00000002000XOol00`00Ool0000:Ool002eoo`03
001oogoo015oo`03001C7@0000<00003Dad0000000800004Dad00000Dad300000e<M000000020000
0e<M0000000200000e<M0000000200000e<M0000000200001E<M00000000Dad00P00;7oo00@007oo
Ool000Uoo`00;Woo0P004Woo00<005<M00000`0000=C7@0000000P0000AC7@00001C7@<00003Dad0
000000800003Dad0000000800003Dad0000000800003Dad0000000800004Dad00000000_Ool00`00
Ool00009Ool0031oo`03001oogoo011oo`03001C7@0000<00003Dad0000000800004Dad00000Dad3
00000e<M0000000200000e<M0000000200000e<M0000000200000e<M00000002000cOol20009Ool0
035oo`03001oogoo011oo`03001C7@0000<00003Dad0000000800004Dad00000Dad300000e<M0000
000200000e<M0000000200000e<M00000002000fOol30009Ool0039oo`80015oo`03001C7@0000<0
0003Dad0000000800004Dad00000Dad400001E<M00000000Dad01000=Woo100037oo000dOol00`00
Oomoo`0?Ool200001E<M00000000Dad01`0000=C7@0000000P0000AC7@00000003Ioo`@0011oo`00
=Goo00<007ooOol047oo00<005<M00000P0000AC7@00001C7@@00003Dad00000008003Eoo`@001Ao
o`00=Woo00<007ooOol047oo00<005<M00001@0000=C7@0000000P00=Goo100067oo000gOol2000A
Ool4000015<M00000000=Woo0`0077oo000iOol00`00Oomoo`0@Ool3000eOol4000OOol003Yoo`03
001oogoo00moo`03001oogoo035oo`@002=oo`00>goo00<007ooOol03Goo00<007ooOol0;Woo1000
9goo000lOol2000=Ool00`00Oomoo`0ZOol4000[Ool003ioo`03001oogoo00Yoo`03001oogoo02Io
o`@002moo`00?goo00<007ooOol02Goo00<007ooOol08Woo1000<goo0010Ool00`00Oomoo`08Ool0
0`00Oomoo`0NOol4000gOol0045oo`8000Qoo`03001oogoo01Yoo`@003]oo`00@goo00<007ooOol0
1Goo00<007ooOol05Woo1000?goo0014Ool00`00Oomoo`04Ool00`00Oomoo`0BOol40013Ool004Eo
o`03001oogoo00=oo`03001oogoo00moo`<004Moo`00AWoo0P000goo00<007ooOol02goo1000BWoo
0018Ool01000Oomoo`002Goo1000CWoo0019Ool00`00Ool00005Ool4001BOol004Yoo`800003Ool0
0000008005Ioo`00Bgoo0P00FWoo002WOol00:Moo`00Ygoo002WOol00:Moo`00Ygoo002WOol00001
\
\>"],
  ImageRangeCache->{{{0, 166.562}, {134.75, 0}} -> {-0.15452, -0.0597142, 
  0.00703392, 0.00703392}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["6.5 Computing the Cumulative Normal Distribution", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We have been able to define, compute and plot our Binary options within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " without any extensive coding. This relied heavily on the existence of \
accurate built-in routines for the cumulative normal distribution, expressed \
in terms of the Error function ",
  StyleBox["Erf",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ":"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Erf\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("Erf[z] gives the error function erf(z). Erf[z0, z1] gives the \
generalized error function erf(z1) - erf(z0)."\)], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Erf[{\(-Infinity\), \ 0, \ Infinity}]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\(-1\), 0, 1}\)], "Output"]
}, Open  ]],

Cell["\<\
This was mapped onto the cumulative normal distribution thus:\
\>", 
  "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["Norm[x_] := (1 + Erf[x/Sqrt[2]])/2", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Let's check that this is right, by differentiating within ",
  StyleBox["Mathematica:",
    FontSlant->"Italic"]
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Norm'\)[x]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(-\(x\^2\/2\)\)\/\@\(2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "We get the correct probability density function. This highlights one of \
the nice features about ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " - we do not have to worry about coding up \"special functions\". Such \
special functions occur in many different cases - for example, hypergeometric \
functions appear in Asian option pricing; Bessel and chi-square functions \
appear in bond option pricing, and so on.\n\nIf you are using this text to \
verify numerical models coded up in other systems, you may wonder how to \
treat such functions. In general, there is quite an art to doing so. "
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Common Rational Approximations and Issues Raised", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "There are some standard approximations for the Error function or \
cumulative normal distribution that have passed into common usage. They are \
given by Abramowitz and Stegun (1972, Chapter 26) and have been made popular \
in derivatives modelling as a result of their inclusion in the classic text \
by John Hull (3rd edition, 1996). Originally due to C. Hastings, and \
published in 1955, they are presented here as ",
  StyleBox["HFuncOne",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["HFuncTwo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". This pair of functions are familiar to many workers in derivatives."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Approximation Number One", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["We first introduce the basic function:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(HFuncOneBase[x_]\  := \n\t
      Module[{\[Gamma]\  = \ 0.33267, \ a = 0.4361836, \ 
          b = \ \(-0.1201676\), \ c = \ 0.9372980, \ k}, \n\t\tk\  = \ 
          1/\((1\  + \ \[Gamma]\ x)\); \n\t\tExp[\(-x^2\)/2]/
            Sqrt[2\ Pi]*\((a*k\  + \ b*k*k\  + \ c*k*k*k)\)]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["Then the first approximation is given by", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(HFuncOne[x_]\  := \n\t
      If[x >= 0, \ 1\  - \ HFuncOneBase[x], \ HFuncOneBase[\(-x\)]]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Note first that we have defined the negative case very carefully. Hull \
gives the formula for ",
  Cell[BoxData[
      \(TraditionalForm\`x > 0\)]],
  " as the function ",
  Cell[BoxData[
      \(TraditionalForm\`N(x)\)]],
  ", and then suggests using"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`1 - N(\(-x\))\)], "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "for ",
  Cell[BoxData[
      \(TraditionalForm\`x < 0\)]],
  ". This must emphatically NOT be done, for very basic reasons - as ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " becomes large and positive the ",
  Cell[BoxData[
      \(TraditionalForm\`N\)]],
  " function approaches unity, so that defining the value for negative ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " this way entails a massive loss of precision as we subtract two nearly \
identical numbers, both close to unity. The way we have written it, in terms \
of the \"Base\" function, nearby numbers are never subtracted. Furthermore, \
in our discussions below, we are using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s arbitrary-precision arithmetic to evaluate these approximations. If you \
use the C programming language, the answers for the errors may be worse than \
those we present. There is, however, one further potentially nasty issue.\n\n\
Now we know already what the derivatives of the cumulative normal \
distribution are, so it may be the case that these are coded up separately. \
However, if the modelling system is organized in such a way that, for \
example, central differences of values are used everywhere to compute Delta \
and Gamma, these approximations may themselves end up being effectively \
differentiated. Let's do this symbolically:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(HFuncOneBaseD[x_]\  = D[HFuncOneBase[x], \ x]; \n
    HFuncOneBaseD2[x_]\  = D[HFuncOneBase[x], \ {x, 2}]; \)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(HFuncOneD[x_]\  := \n\t
      If[x >= 0, \ \(-\ HFuncOneBaseD[x]\), \ \(-HFuncOneBaseD[\(-x\)]\)]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(HFuncOneD2[x_]\  := 
      If[x >= 0, \ \(-\ HFuncOneBaseD2[x]\), \ HFuncOneBaseD2[\(-x\)]]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["The function itself looks just fine:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[HFuncOne[x], \ {x, \ \(-3\), \ 3}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.15873 0.0147151 0.589402 [
[.02381 .00222 -6 -9 ]
[.02381 .00222 6 0 ]
[.18254 .00222 -6 -9 ]
[.18254 .00222 6 0 ]
[.34127 .00222 -6 -9 ]
[.34127 .00222 6 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.81746 .00222 -3 -9 ]
[.81746 .00222 3 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.4875 .1326 -18 -4.5 ]
[.4875 .1326 0 4.5 ]
[.4875 .25048 -18 -4.5 ]
[.4875 .25048 0 4.5 ]
[.4875 .36836 -18 -4.5 ]
[.4875 .36836 0 4.5 ]
[.4875 .48624 -18 -4.5 ]
[.4875 .48624 0 4.5 ]
[.4875 .60412 -6 -4.5 ]
[.4875 .60412 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .01472 m
.02381 .02097 L
s
[(-3)] .02381 .00222 0 1 Mshowa
.18254 .01472 m
.18254 .02097 L
s
[(-2)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(-1)] .34127 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(1)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(2)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.5 .1326 m
.50625 .1326 L
s
[(0.2)] .4875 .1326 1 0 Mshowa
.5 .25048 m
.50625 .25048 L
s
[(0.4)] .4875 .25048 1 0 Mshowa
.5 .36836 m
.50625 .36836 L
s
[(0.6)] .4875 .36836 1 0 Mshowa
.5 .48624 m
.50625 .48624 L
s
[(0.8)] .4875 .48624 1 0 Mshowa
.5 .60412 m
.50625 .60412 L
s
[(1)] .4875 .60412 1 0 Mshowa
.125 Mabswid
.5 .04419 m
.50375 .04419 L
s
.5 .07366 m
.50375 .07366 L
s
.5 .10313 m
.50375 .10313 L
s
.5 .16207 m
.50375 .16207 L
s
.5 .19154 m
.50375 .19154 L
s
.5 .22101 m
.50375 .22101 L
s
.5 .27995 m
.50375 .27995 L
s
.5 .30942 m
.50375 .30942 L
s
.5 .33889 m
.50375 .33889 L
s
.5 .39783 m
.50375 .39783 L
s
.5 .4273 m
.50375 .4273 L
s
.5 .45677 m
.50375 .45677 L
s
.5 .51571 m
.50375 .51571 L
s
.5 .54518 m
.50375 .54518 L
s
.5 .57465 m
.50375 .57465 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01551 m
.04262 .01589 L
.06244 .01644 L
.08255 .01724 L
.10458 .01847 L
.12415 .01999 L
.14509 .02219 L
.16639 .02521 L
.18653 .02895 L
.20651 .03371 L
.22495 .03921 L
.24527 .0467 L
.2643 .05525 L
.30213 .07735 L
.34091 .1079 L
.38213 .14961 L
.42184 .19815 L
.46003 .25083 L
.50068 .31042 L
.53981 .36777 L
.58139 .4249 L
.62146 .47322 L
.66001 .51175 L
.70101 .5436 L
.72143 .55608 L
.7405 .56589 L
.77847 .58073 L
.79882 .5865 L
.81738 .59069 L
.83646 .59408 L
.85722 .59692 L
.877 .59894 L
.898 .60053 L
.91748 .6016 L
.93576 .60233 L
.95594 .60291 L
.97444 .60329 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{165.75, 102.438},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002U0000IQ000`40O003h00OYGoo002UOol000mo
o`<001Aoo`@001=oo`<002Qoo`<001Eoo`@001=oo`<000]oo`004Woo00<007ooOol04Woo00<007oo
Ool057oo00<007ooOol0:7oo00<007ooOol05Goo00<007ooOol05Woo00<007ooOol027oo0009Ool5
0004Ool00`00Oomoo`0;Ool50003Ool00`00Oomoo`0<Ool50002Ool00`00Oomoo`0XOol00`00Oomo
o`0FOol00`00Oomoo`0EOol00`00Oomoo`08Ool0011oo`8001Moo`03001oogoo019oo`03001oogoo
02Qoo`03001oogoo01Moo`03001oogoo019oo`8000]oo`004Woo00<007ooOol04Goo00@007ooOol0
01Aoo`03001oogoo02Qoo`03001oogoo01Aoo`04001oogoo000FOol00`00Oomoo`08Ool000moo`<0
01Eoo`8001Aoo`8002Uoo`8001Moo`8001Aoo`<000]oo`00YGoo001COol00`00Oomoo`1?Ool005=o
o`03001oogoo04moo`002WooT`0027oo000>Ool00`00Oomoo`0;Ool:000FOol00`00Oomoo`0DOol0
0`00Oomoo`0DOol00`00Oomoo`0EOol00`00Oomoo`0DOol00`00Oomoo`09Ool002Eoo`<002]oo`03
001oogoo04moo`00:7oo1@009Woo00<007ooOol0Cgoo000]Ool00`00Oomoo`0SOol2001@Ool002io
o`<0029oo`03001oogoo04moo`00<Goo0P0087oo00<007ooOol0Cgoo000cOol2000NOol00`00Oomo
o`1?Ool003Eoo`03001oogoo01]oo`80051oo`00=Woo0P006goo00<007ooOol0Cgoo000hOol00`00
Oomoo`0HOol00`00Oomoo`1?Ool003Uoo`03001oogoo01Moo`03001oogoo04moo`00>Woo00<007oo
Ool05Woo00<007ooOol0Cgoo000kOol00`00Oomoo`0EOol2001@Ool003aoo`03001oogoo01Aoo`03
001oogoo04moo`00?Goo00@007ooOomoo`8000Aoo`8000=oo`@000=oo`03001oogoo04moo`00?Woo
00<007oo00000Woo00@007ooOomoo`8000Aoo`03001oogoo00=oo`03001oogoo04moo`00?goo0P00
0Woo00<007ooOol027oo00<007ooOol00Woo0P00D7oo0010Ool01000Oomoo`002goo00D007ooOomo
o`00055oo`00@7oo0P0000=oo`00Ool01goo00@007ooOol000=oo`03001oogoo04moo`00@Goo0P00
2Woo0P0017oo00<007ooOol0Cgoo0012Ool00`00Oomoo`0>Ool2001@Ool004=oo`03001oogoo00eo
o`03001oogoo04moo`00A7oo00<007ooOol037oo00<007ooOol0Cgoo0015Ool00`00Oomoo`0;Ool0
0`00Oomoo`1?Ool004Ioo`03001oogoo00Yoo`03001oogoo04moo`00Agoo00<007ooOol02Goo0P00
D7oo0018Ool00`00Oomoo`08Ool00`00Oomoo`1?Ool004Qoo`03001oogoo00Qoo`03001oogoo04mo
o`00BGoo00<007ooOol01goo00<007ooOol0Cgoo001:Ool00`00Oomoo`06Ool2001@Ool004]oo`03
001oogoo00Eoo`03001oogoo04moo`00@Goo0P0017oo0P000Woo00<007oo00000P000goo00<007oo
Ool0Cgoo0010Ool01000Oomoo`000goo0P000goo00<007oo000017oo00<007ooOol0Cgoo0010Ool0
1000Oomoo`0027oo10000goo0P00D7oo0010Ool01000Oomoo`0027oo0`0017oo00<007ooOol0Cgoo
0010Ool01000Oomoo`0027oo00<007oo000017oo00<007ooOol0Cgoo0011Ool2000:Ool30003Ool0
0`00Oomoo`1?Ool004moo`05001oogooOol0001AOol0051oo`03001oogoo0080051oo`00DGoo00<0
07oo0000DGoo001AOol00`00Ool0001AOol0059oo`80055oo`00Dgoo0P00D7oo001COol00`00Oomo
o`1?Ool005=oo`80051oo`00Dgoo00<007oo0000Cgoo001COol200000goo001oo`1=Ool005=oo`04
001oogoo001>Ool005=oo`05001oogooOol0001=Ool0045oo`8000Aoo`8000Aoo`8000Aoo`03001o
ogoo009oo`03001oogoo04Yoo`00@7oo00@007ooOol000=oo`8000=oo`04001oogoo0003Ool00`00
Oomoo`03Ool00`00Oomoo`19Ool0041oo`04001oogoo0008Ool01000Oomoo`000goo0P0017oo00<0
07ooOol0BGoo0010Ool01000Oomoo`0027oo0`0017oo00<007ooOol017oo00<007ooOol0B7oo0010
Ool01000Oomoo`0027oo00<007ooOol017oo00<007ooOol01Goo00<007ooOol0Agoo0011Ool2000:
Ool30003Ool00`00Oomoo`05Ool00`00Oomoo`17Ool005=oo`8000Moo`03001oogoo04Ioo`00Dgoo
00<007ooOol01goo00<007ooOol0AGoo001COol00`00Oomoo`07Ool00`00Oomoo`15Ool005=oo`03
001oogoo00Qoo`03001oogoo04Aoo`00Dgoo0P002Woo00<007ooOol0@goo001COol00`00Oomoo`09
Ool00`00Oomoo`13Ool005=oo`03001oogoo00Yoo`03001oogoo049oo`00Dgoo00<007ooOol02goo
00<007ooOol0@Goo001COol2000=Ool00`00Oomoo`10Ool005=oo`03001oogoo00eoo`03001oogoo
03moo`00Dgoo00<007ooOol03Woo00<007ooOol0?Woo0011Ool20004Ool20004Ool20004Ool00`00
Oomoo`0?Ool00`00Oomoo`0mOol0041oo`04001oogoo0003Ool20003Ool01000Oomoo`000goo00<0
07ooOol047oo00<007ooOol0?7oo0010Ool01000Oomoo`0027oo00@007ooOol000=oo`80019oo`03
001oogoo03]oo`00@7oo00@007ooOol000Uoo`8000Aoo`03001oogoo019oo`03001oogoo03Yoo`00
@7oo00@007ooOol000Qoo`04001oogoo0003Ool00`00Oomoo`0COol00`00Oomoo`0iOol0045oo`80
00Yoo`8000Aoo`03001oogoo01Aoo`03001oogoo03Qoo`00Dgoo0P005Woo0P00>7oo001COol00`00
Oomoo`0GOol00`00Oomoo`0eOol005=oo`03001oogoo01Qoo`8003Eoo`00Dgoo00<007ooOol06Woo
00<007ooOol0<Woo001COol2000LOol00`00Oomoo`0aOol005=oo`03001oogoo01aoo`03001oogoo
031oo`00Dgoo00<007ooOol07Goo0`00;goo001COol00`00Oomoo`0POol2000]Ool005=oo`03001o
ogoo029oo`8002]oo`00Dgoo0P009Goo0P00:Goo001COol00`00Oomoo`0VOol6000SOol004aoo`<0
00Aoo`03001oogoo02aoo`80025oo`00CGoo00<007ooOol00goo00<007ooOol0;Woo30005Goo001=
Ool00`00Oomoo`03Ool2000kOol:000;Ool004eoo`03001oogoo00=oo`03001oogoo04moo`00CGoo
00<007ooOol00goo00<007ooOol0Cgoo001<Ool2001GOol00:Eoo`00\
\>"],
  ImageRangeCache->{{{0, 164.75}, {101.438, 0}} -> {-3.60651, -0.125721, 
  0.0431217, 0.011613}}]
}, Open  ]],

Cell["\<\
However, the percentage error approaches 4% at large negative \
arguments (this may be a larger error if single-precision C is used):\
\>", 
  "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[100*\((HFuncOne[x]/Norm[x] - 1)\), \ {x, \ \(-7\), \ 7}, \ 
      PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0680272 0.0162384 0.14949 [
[.09184 .00374 -6 -9 ]
[.09184 .00374 6 0 ]
[.22789 .00374 -6 -9 ]
[.22789 .00374 6 0 ]
[.36395 .00374 -6 -9 ]
[.36395 .00374 6 0 ]
[.63605 .00374 -3 -9 ]
[.63605 .00374 3 0 ]
[.77211 .00374 -3 -9 ]
[.77211 .00374 3 0 ]
[.90816 .00374 -3 -9 ]
[.90816 .00374 3 0 ]
[.4875 .16573 -6 -4.5 ]
[.4875 .16573 0 4.5 ]
[.4875 .31522 -6 -4.5 ]
[.4875 .31522 0 4.5 ]
[.4875 .46471 -6 -4.5 ]
[.4875 .46471 0 4.5 ]
[.4875 .6142 -6 -4.5 ]
[.4875 .6142 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.09184 .01624 m
.09184 .02249 L
s
[(-6)] .09184 .00374 0 1 Mshowa
.22789 .01624 m
.22789 .02249 L
s
[(-4)] .22789 .00374 0 1 Mshowa
.36395 .01624 m
.36395 .02249 L
s
[(-2)] .36395 .00374 0 1 Mshowa
.63605 .01624 m
.63605 .02249 L
s
[(2)] .63605 .00374 0 1 Mshowa
.77211 .01624 m
.77211 .02249 L
s
[(4)] .77211 .00374 0 1 Mshowa
.90816 .01624 m
.90816 .02249 L
s
[(6)] .90816 .00374 0 1 Mshowa
.125 Mabswid
.12585 .01624 m
.12585 .01999 L
s
.15986 .01624 m
.15986 .01999 L
s
.19388 .01624 m
.19388 .01999 L
s
.2619 .01624 m
.2619 .01999 L
s
.29592 .01624 m
.29592 .01999 L
s
.32993 .01624 m
.32993 .01999 L
s
.39796 .01624 m
.39796 .01999 L
s
.43197 .01624 m
.43197 .01999 L
s
.46599 .01624 m
.46599 .01999 L
s
.53401 .01624 m
.53401 .01999 L
s
.56803 .01624 m
.56803 .01999 L
s
.60204 .01624 m
.60204 .01999 L
s
.67007 .01624 m
.67007 .01999 L
s
.70408 .01624 m
.70408 .01999 L
s
.7381 .01624 m
.7381 .01999 L
s
.80612 .01624 m
.80612 .01999 L
s
.84014 .01624 m
.84014 .01999 L
s
.87415 .01624 m
.87415 .01999 L
s
.05782 .01624 m
.05782 .01999 L
s
.02381 .01624 m
.02381 .01999 L
s
.94218 .01624 m
.94218 .01999 L
s
.97619 .01624 m
.97619 .01999 L
s
.25 Mabswid
0 .01624 m
1 .01624 L
s
.5 .16573 m
.50625 .16573 L
s
[(1)] .4875 .16573 1 0 Mshowa
.5 .31522 m
.50625 .31522 L
s
[(2)] .4875 .31522 1 0 Mshowa
.5 .46471 m
.50625 .46471 L
s
[(3)] .4875 .46471 1 0 Mshowa
.5 .6142 m
.50625 .6142 L
s
[(4)] .4875 .6142 1 0 Mshowa
.125 Mabswid
.5 .04614 m
.50375 .04614 L
s
.5 .07603 m
.50375 .07603 L
s
.5 .10593 m
.50375 .10593 L
s
.5 .13583 m
.50375 .13583 L
s
.5 .19563 m
.50375 .19563 L
s
.5 .22552 m
.50375 .22552 L
s
.5 .25542 m
.50375 .25542 L
s
.5 .28532 m
.50375 .28532 L
s
.5 .34512 m
.50375 .34512 L
s
.5 .37501 m
.50375 .37501 L
s
.5 .40491 m
.50375 .40491 L
s
.5 .43481 m
.50375 .43481 L
s
.5 .49461 m
.50375 .49461 L
s
.5 .52451 m
.50375 .52451 L
s
.5 .5544 m
.50375 .5544 L
s
.5 .5843 m
.50375 .5843 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
.5 Mabswid
.02381 .60332 m
.06244 .51383 L
.10458 .41801 L
.14415 .33129 L
.18221 .25284 L
.22272 .17727 L
.26171 .11516 L
.28158 .08852 L
.30316 .06398 L
.32216 .04646 L
.33296 .03827 L
.34309 .03178 L
.35344 .02631 L
.36292 .02232 L
.37305 .01907 L
.37864 .0177 L
.38395 .01667 L
.38898 .01591 L
.39354 .0154 L
.39607 .01518 L
.3988 .015 L
.40006 .01493 L
.40138 .01487 L
.40262 .01482 L
.40376 .01479 L
.40488 .01476 L
.40595 .01474 L
.40707 .01472 L
.40829 .01472 L
.40936 .01472 L
.41049 .01472 L
.41146 .01473 L
.41252 .01474 L
.41377 .01477 L
.41491 .01479 L
.41749 .01487 L
.42214 .01506 L
.43207 .01562 L
.44267 .01626 L
.44782 .01651 L
.45035 .01661 L
.45268 .01669 L
.4548 .01675 L
.45588 .01677 L
.45704 .01679 L
.4582 .01681 L
.45929 .01682 L
.46059 .01683 L
.46177 .01683 L
.463 .01682 L
Mistroke
.46412 .01681 L
.46541 .0168 L
.46662 .01678 L
.46879 .01673 L
.47112 .01666 L
.47644 .01646 L
.48132 .01623 L
.48358 .01613 L
.48599 .01603 L
.48716 .01599 L
.48827 .01596 L
.48928 .01593 L
.49035 .01591 L
.49148 .0159 L
.49253 .01589 L
.49365 .0159 L
.49489 .01592 L
.49613 .01597 L
.49683 .016 L
.49747 .01603 L
.49864 .01611 L
.49988 .01622 L
.50112 .01634 L
.50182 .01639 L
.50246 .01643 L
.50364 .01648 L
.50489 .01651 L
.50602 .01653 L
.50706 .01653 L
.50821 .01652 L
.50945 .0165 L
.51076 .01648 L
.51219 .01644 L
.51479 .01636 L
.51966 .01621 L
.52428 .0161 L
.52662 .01605 L
.52797 .01603 L
.5292 .01602 L
.53027 .01601 L
.53139 .016 L
.53252 .016 L
.53376 .01599 L
.53483 .01599 L
.53597 .01599 L
.53705 .016 L
.53804 .016 L
.53923 .01601 L
.54034 .01602 L
.54288 .01604 L
Mistroke
.548 .0161 L
.5572 .01623 L
.56234 .0163 L
.56722 .01635 L
.56991 .01637 L
.57284 .01639 L
.57413 .0164 L
.57551 .0164 L
.57679 .01641 L
.57799 .01641 L
.57923 .01641 L
.57989 .01641 L
.5806 .01641 L
.58134 .01641 L
.58204 .01641 L
.58335 .01641 L
.58458 .01641 L
.58571 .01641 L
.58696 .0164 L
.58828 .0164 L
.59101 .01639 L
.59358 .01638 L
.59941 .01634 L
.61959 .01619 L
.62943 .01613 L
.63438 .01611 L
.63721 .0161 L
.63979 .01609 L
.64201 .01609 L
.64443 .01608 L
.64697 .01608 L
.64812 .01608 L
.64938 .01608 L
.65047 .01607 L
.65163 .01607 L
.65287 .01607 L
.65402 .01607 L
.65517 .01607 L
.65626 .01607 L
.65723 .01607 L
.6583 .01607 L
.65963 .01607 L
.66086 .01608 L
.66363 .01608 L
.66608 .01608 L
.66865 .01608 L
.67802 .0161 L
.69901 .01615 L
.70853 .01617 L
.71871 .01619 L
Mistroke
.72825 .0162 L
.73706 .01621 L
.74727 .01622 L
.75265 .01623 L
.75835 .01623 L
.76794 .01623 L
.77338 .01623 L
.77849 .01623 L
.78345 .01624 L
.78887 .01624 L
.79398 .01624 L
.79873 .01624 L
.80343 .01624 L
.80769 .01624 L
.81261 .01624 L
.81725 .01624 L
.8219 .01624 L
.82701 .01624 L
.83234 .01624 L
.83732 .01624 L
.84198 .01624 L
.84623 .01624 L
.85114 .01624 L
.85579 .01624 L
.86037 .01624 L
.86544 .01624 L
.86798 .01624 L
.87072 .01624 L
.8731 .01624 L
.87569 .01624 L
.88037 .01624 L
.88292 .01624 L
.8853 .01624 L
.88956 .01624 L
.89411 .01624 L
.89669 .01624 L
.89943 .01624 L
.90434 .01624 L
.90707 .01624 L
.90963 .01624 L
.91261 .01624 L
.91544 .01624 L
.91787 .01624 L
.92052 .01624 L
.92516 .01624 L
.92778 .01624 L
.93057 .01624 L
.93559 .01624 L
.93814 .01624 L
.94083 .01624 L
Mistroke
.94564 .01624 L
.94833 .01624 L
.95083 .01624 L
.95357 .01624 L
.95651 .01624 L
.95904 .01624 L
.9618 .01624 L
.9644 .01624 L
.96681 .01624 L
.97136 .01624 L
.97366 .01624 L
.97619 .01624 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{165.75, 102.438},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002U0000IQ000`40O003h00OYGoo002UOol001Uo
o`80019oo`<000moo`@0021oo`@0015oo`<0015oo`8001Moo`0067oo00@007ooOol0019oo`03001o
ogoo00moo`03001oogoo025oo`03001oogoo019oo`03001oogoo00moo`04001oogoo000FOol0019o
o`D00005Ool007ooOol0000:Ool500000goo000000020009Ool50003Ool00`00Oomoo`0QOol00`00
Oomoo`0?Ool4000@Ool01000Oomoo`005Woo000HOol3000AOol00`00Ool0000COol00`00Oomoo`0Q
Ool00`00Oomoo`0>Ool00`00Ool0000AOol3000GOol001Qoo`03001oogoo015oo`03001oo`00011o
o`04001oogoo000POol01000Oomoo`0047oo00<007oo00004Goo00<007ooOol05goo000IOol3000A
Ool2000AOol2000ROol2000BOol2000BOol3000FOol00:Eoo`00DWoo00<007ooOol0D7oo001BOol0
0`00Oomoo`1@Ool000Yooi4000Yoo`005goo00<007ooOol04Goo00<007ooOol03Woo0`004goo00<0
07ooOol047oo00<007ooOol04Goo00<007ooOol04Goo00<007ooOol05Goo000kOol00`00Oomoo`0D
Ool00`00Oomoo`1@Ool003Uoo`8001Moo`03001oogoo051oo`00>7oo00<007ooOol05goo00<007oo
Ool0D7oo000hOol00`00Oomoo`0GOol00`00Oomoo`1@Ool003Moo`03001oogoo01Qoo`03001oogoo
051oo`00=Goo0P006goo00<007ooOol0D7oo000dOol00`00Oomoo`0KOol00`00Oomoo`1@Ool003=o
o`03001oogoo01aoo`03001oogoo051oo`00<Woo00<007ooOol07Goo00<007ooOol0D7oo000bOol0
0`00Oomoo`0MOol00`00Oomoo`1@Ool0035oo`03001oogoo01ioo`03001oogoo051oo`00<Goo00<0
07ooOol07Woo00<007ooOol0D7oo000`Ool00`00Oomoo`0OOol00`00Oomoo`1@Ool0031oo`03001o
ogoo01moo`03001oogoo051oo`00;goo00<007ooOol087oo00<007ooOol0D7oo000^Ool00`00Oomo
o`0QOol00`00Oomoo`1@Ool002ioo`03001oogoo025oo`03001oogoo051oo`00;Goo00<007ooOol0
6goo0`0017oo00<007ooOol0D7oo000/Ool00`00Oomoo`0MOol00`00Oomoo`03Ool00`00Oomoo`1@
Ool002aoo`03001oogoo01eoo`03001oogoo00=oo`80055oo`00:goo00<007ooOol07Woo00<007oo
Ool00goo00<007ooOol0D7oo000ZOol00`00Oomoo`0OOol00`00Oomoo`03Ool00`00Oomoo`1@Ool0
02Yoo`03001oogoo01ioo`8000Eoo`03001oogoo051oo`00:Goo00<007ooOol09Woo00<007ooOol0
D7oo000YOol00`00Oomoo`0VOol00`00Oomoo`1@Ool002Qoo`03001oogoo02Moo`03001oogoo051o
o`00:7oo00<007ooOol09goo00<007ooOol0D7oo000WOol00`00Oomoo`0XOol00`00Oomoo`1@Ool0
02Ioo`03001oogoo02Uoo`03001oogoo051oo`009Woo00<007ooOol0:Goo00<007ooOol0D7oo000U
Ool00`00Oomoo`0ZOol00`00Oomoo`1@Ool002Eoo`03001oogoo02Yoo`03001oogoo051oo`0097oo
00<007ooOol0:goo00<007ooOol0D7oo000TOol00`00Oomoo`0[Ool00`00Oomoo`1@Ool002=oo`03
001oogoo02aoo`03001oogoo051oo`008goo00<007ooOol0;7oo00<007ooOol0D7oo000ROol00`00
Oomoo`0]Ool00`00Oomoo`1@Ool0029oo`03001oogoo02eoo`03001oogoo051oo`008Goo00<007oo
Ool0;Woo00<007ooOol0D7oo000QOol00`00Oomoo`0WOol40003Ool00`00Oomoo`1@Ool0025oo`03
001oogoo02Qoo`03001oogoo00=oo`03001oogoo051oo`0087oo00<007ooOol0:Woo00<007ooOol0
0Woo0P00DGoo000POol00`00Oomoo`0[Ool01@00Oomoogoo0000DWoo000OOol00`00Oomoo`0YOol0
1000Oomoo`000goo00<007ooOol0D7oo000OOol00`00Oomoo`0ZOol20004Ool00`00Oomoo`1@Ool0
01ioo`03001oogoo035oo`03001oogoo051oo`007Woo00<007ooOol0<Goo00<007ooOol0D7oo000M
Ool00`00Oomoo`0bOol00`00Oomoo`1@Ool001eoo`03001oogoo039oo`03001oogoo051oo`0077oo
00<007ooOol0<goo00<007ooOol0D7oo000LOol00`00Oomoo`0cOol00`00Oomoo`1@Ool001aoo`03
001oogoo03=oo`03001oogoo051oo`006goo00<007ooOol0=7oo00<007ooOol0D7oo000KOol00`00
Oomoo`0dOol00`00Oomoo`1@Ool001Yoo`03001oogoo03Eoo`03001oogoo051oo`006Woo00<007oo
Ool0=Goo00<007ooOol0D7oo000IOol00`00Oomoo`0fOol00`00Oomoo`1@Ool001Uoo`03001oogoo
03Ioo`03001oogoo051oo`0067oo00<007ooOol0=goo00<007ooOol0D7oo000HOol00`00Oomoo`0g
Ool00`00Oomoo`1@Ool001Moo`03001oogoo035oo`<000Aoo`03001oogoo051oo`005goo00<007oo
Ool0=7oo00D007ooOomoo`00059oo`005goo00<007ooOol0=7oo00@007ooOomoo`80055oo`005Woo
00<007ooOol0<goo0P0017oo00<007ooOol0D7oo000FOol00`00Oomoo`0eOol01@00Oomoogoo0000
DWoo000EOol00`00Oomoo`0cOol30004Ool00`00Oomoo`1@Ool001Eoo`03001oogoo03Yoo`03001o
ogoo051oo`0057oo00<007ooOol0>goo00<007ooOol0D7oo000DOol00`00Oomoo`0kOol00`00Oomo
o`1@Ool001Aoo`03001oogoo03]oo`03001oogoo051oo`004goo00<007ooOol0?7oo00<007ooOol0
D7oo000COol00`00Oomoo`0lOol00`00Oomoo`1@Ool0019oo`03001oogoo03eoo`03001oogoo051o
o`004Woo00<007ooOol0?Goo00<007ooOol0D7oo000AOol00`00Oomoo`0nOol00`00Oomoo`1@Ool0
015oo`03001oogoo03ioo`03001oogoo051oo`0047oo00<007ooOol0?goo00<007ooOol0D7oo000@
Ool00`00Oomoo`0oOol00`00Oomoo`1@Ool000moo`03001oogoo041oo`03001oogoo051oo`003goo
00<007ooOol0@7oo00<007ooOol0D7oo000>Ool00`00Oomoo`11Ool00`00Oomoo`1@Ool000ioo`03
001oogoo045oo`03001oogoo051oo`003Goo00<007ooOol0?7oo0`000goo00<007ooOol0D7oo001=
Ool00`00Oomoo`02Ool00`00Oomoo`1@Ool004]oo`@000=oo`80055oo`00Bgoo00<007oo000017oo
00<007ooOol0D7oo001;Ool00`00Ool0001GOol004aoo`8005Moo`00YGoo0000\
\>"],
  ImageRangeCache->{{{0, 164.75}, {101.438, 0}} -> {-8.40621, -0.501571, 
  0.102048, 0.0464382}}]
}, Open  ]],

Cell["\<\
If we look at the difference error, we see that the discrepancy is \
less than about 0.00001 everywhere:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[HFuncOne[x] - Norm[x], \ {x, \ \(-7\), \ 7}, \ 
      PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0680272 0.308997 25536.2 [
[.09184 .2965 -6 -9 ]
[.09184 .2965 6 0 ]
[.22789 .2965 -6 -9 ]
[.22789 .2965 6 0 ]
[.36395 .2965 -6 -9 ]
[.36395 .2965 6 0 ]
[.63605 .2965 -3 -9 ]
[.63605 .2965 3 0 ]
[.77211 .2965 -3 -9 ]
[.77211 .2965 3 0 ]
[.90816 .2965 -3 -9 ]
[.90816 .2965 3 0 ]
[.4875 .05364 -48 -4.5 ]
[.4875 .05364 0 4.5 ]
[.4875 .18132 -39 -6.25 ]
[.4875 .18132 0 6.25 ]
[.4875 .43668 -33 -6.25 ]
[.4875 .43668 0 6.25 ]
[.4875 .56436 -42 -4.5 ]
[.4875 .56436 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.09184 .309 m
.09184 .31525 L
s
[(-6)] .09184 .2965 0 1 Mshowa
.22789 .309 m
.22789 .31525 L
s
[(-4)] .22789 .2965 0 1 Mshowa
.36395 .309 m
.36395 .31525 L
s
[(-2)] .36395 .2965 0 1 Mshowa
.63605 .309 m
.63605 .31525 L
s
[(2)] .63605 .2965 0 1 Mshowa
.77211 .309 m
.77211 .31525 L
s
[(4)] .77211 .2965 0 1 Mshowa
.90816 .309 m
.90816 .31525 L
s
[(6)] .90816 .2965 0 1 Mshowa
.125 Mabswid
.12585 .309 m
.12585 .31275 L
s
.15986 .309 m
.15986 .31275 L
s
.19388 .309 m
.19388 .31275 L
s
.2619 .309 m
.2619 .31275 L
s
.29592 .309 m
.29592 .31275 L
s
.32993 .309 m
.32993 .31275 L
s
.39796 .309 m
.39796 .31275 L
s
.43197 .309 m
.43197 .31275 L
s
.46599 .309 m
.46599 .31275 L
s
.53401 .309 m
.53401 .31275 L
s
.56803 .309 m
.56803 .31275 L
s
.60204 .309 m
.60204 .31275 L
s
.67007 .309 m
.67007 .31275 L
s
.70408 .309 m
.70408 .31275 L
s
.7381 .309 m
.7381 .31275 L
s
.80612 .309 m
.80612 .31275 L
s
.84014 .309 m
.84014 .31275 L
s
.87415 .309 m
.87415 .31275 L
s
.05782 .309 m
.05782 .31275 L
s
.02381 .309 m
.02381 .31275 L
s
.94218 .309 m
.94218 .31275 L
s
.97619 .309 m
.97619 .31275 L
s
.25 Mabswid
0 .309 m
1 .309 L
s
.5 .05364 m
.50625 .05364 L
s
[(-0.00001)] .4875 .05364 1 0 Mshowa
.5 .18132 m
.50625 .18132 L
s
gsave
.4875 .18132 -39 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(-5) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-6) show
grestore
.5 .43668 m
.50625 .43668 L
s
gsave
.4875 .43668 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(5) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-6) show
grestore
.5 .56436 m
.50625 .56436 L
s
[(0.00001)] .4875 .56436 1 0 Mshowa
.125 Mabswid
.5 .07917 m
.50375 .07917 L
s
.5 .10471 m
.50375 .10471 L
s
.5 .13024 m
.50375 .13024 L
s
.5 .15578 m
.50375 .15578 L
s
.5 .20685 m
.50375 .20685 L
s
.5 .23239 m
.50375 .23239 L
s
.5 .25792 m
.50375 .25792 L
s
.5 .28346 m
.50375 .28346 L
s
.5 .33453 m
.50375 .33453 L
s
.5 .36007 m
.50375 .36007 L
s
.5 .38561 m
.50375 .38561 L
s
.5 .41114 m
.50375 .41114 L
s
.5 .46221 m
.50375 .46221 L
s
.5 .48775 m
.50375 .48775 L
s
.5 .51329 m
.50375 .51329 L
s
.5 .53882 m
.50375 .53882 L
s
.5 .0281 m
.50375 .0281 L
s
.5 .00256 m
.50375 .00256 L
s
.5 .5899 m
.50375 .5899 L
s
.5 .61543 m
.50375 .61543 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
.5 Mabswid
.02381 .309 m
.02605 .309 L
.02846 .309 L
.03279 .309 L
.03543 .309 L
.03793 .309 L
.04262 .309 L
.04502 .309 L
.04756 .309 L
.05212 .309 L
.05466 .309 L
.05704 .309 L
.05964 .309 L
.06244 .309 L
.06524 .309 L
.06784 .309 L
.07069 .309 L
.07373 .309 L
.07662 .309 L
.07935 .309 L
.08183 .309 L
.08445 .309 L
.08685 .309 L
.08948 .309 L
.09412 .309 L
.09673 .309 L
.09953 .309 L
.10458 .309 L
.10714 .309 L
.10986 .309 L
.11475 .309 L
.11747 .309 L
.12001 .309 L
.12284 .30901 L
.12582 .30901 L
.12823 .30901 L
.13087 .30901 L
.1355 .30902 L
.13812 .30903 L
.14091 .30903 L
.14594 .30905 L
.14853 .30906 L
.15127 .30907 L
.15618 .3091 L
.16147 .30915 L
.16448 .30919 L
.16729 .30923 L
.17236 .30932 L
.177 .30943 L
.18236 .30961 L
Mistroke
.18744 .30984 L
.19248 .31014 L
.19705 .3105 L
.20232 .31104 L
.20728 .31171 L
.21189 .31251 L
.21613 .31343 L
.22568 .31635 L
.2304 .31834 L
.23545 .32099 L
.24455 .32746 L
.25011 .33274 L
.25537 .33885 L
.26522 .35378 L
.2749 .37365 L
.28517 .40109 L
.30361 .46608 L
.31322 .50488 L
.32334 .54411 L
.32752 .55834 L
.33197 .57131 L
.33454 .57745 L
.33686 .58204 L
.33799 .58388 L
.3392 .58557 L
.34027 .5868 L
.34141 .58782 L
.34262 .58857 L
.34375 .58895 L
.34443 .58902 L
.34509 .58897 L
.34631 .58856 L
.3476 .58769 L
.3488 .58645 L
.35011 .5846 L
.3515 .58205 L
.35378 .57657 L
.35592 .56987 L
.36079 .54872 L
.36603 .51671 L
.37083 .47895 L
.38166 .36827 L
.39157 .24881 L
.39702 .18408 L
.40217 .12914 L
.40696 .08747 L
.40967 .06945 L
.41096 .0625 L
.41216 .05703 L
.41326 .05297 L
Mistroke
.41426 .05005 L
.41538 .04772 L
.41658 .04634 L
.41772 .04613 L
.41898 .04718 L
.42016 .04942 L
.42126 .05262 L
.4225 .05756 L
.42365 .06334 L
.42624 .0808 L
.42852 .10119 L
.43093 .12765 L
.43969 .26048 L
.44997 .45094 L
.45445 .52435 L
.45699 .55783 L
.45933 .58167 L
.46046 .59026 L
.46169 .59734 L
.46299 .60188 L
.4642 .60332 L
.46536 .602 L
.46643 .5984 L
.46759 .59172 L
.46882 .58154 L
.47004 .56819 L
.47134 .55037 L
.47368 .50908 L
.47898 .37634 L
.48393 .22343 L
.48636 .15041 L
.4886 .09285 L
.48965 .07124 L
.49063 .0553 L
.49166 .044 L
.49278 .03909 L
.49393 .04366 L
.49497 .05772 L
.49618 .08763 L
.49732 .13167 L
.49971 .2827 L
.50096 .38659 L
.50228 .46717 L
.50344 .51757 L
.50471 .55397 L
.50587 .57238 L
.50694 .57876 L
.50764 .57806 L
.5083 .57431 L
.50954 .56025 L
Mistroke
.51023 .54893 L
.51087 .5366 L
.51232 .50324 L
.51718 .35956 L
.52179 .21932 L
.52612 .11316 L
.52829 .0735 L
.5294 .05704 L
.53062 .04218 L
.53131 .03522 L
.53195 .02968 L
.53322 .02134 L
.53436 .0167 L
.53559 .01472 L
.53683 .01572 L
.53753 .01758 L
.53819 .02013 L
.53888 .02364 L
.53952 .02756 L
.54094 .03869 L
.54347 .06567 L
.54587 .09835 L
.55699 .29676 L
.56677 .46052 L
.57171 .519 L
.57452 .54291 L
.57578 .55131 L
.57716 .55885 L
.57842 .56422 L
.57959 .56794 L
.58083 .57053 L
.58152 .5714 L
.58215 .57182 L
.58328 .57174 L
.58451 .57045 L
.58567 .56812 L
.58673 .56505 L
.58797 .56043 L
.58912 .55511 L
.59175 .53968 L
.59706 .49633 L
.61548 .28334 L
.62588 .16927 L
.631 .12372 L
.63575 .08931 L
.64005 .06504 L
.64224 .05513 L
.64462 .04631 L
.64723 .03878 L
.64843 .03608 L
Mistroke
.64969 .03373 L
.65083 .03202 L
.65209 .03059 L
.65323 .02969 L
.65429 .02918 L
.65555 .02898 L
.65619 .02903 L
.6569 .02922 L
.65814 .02985 L
.65931 .03078 L
.66068 .03227 L
.66193 .034 L
.66476 .03905 L
.66959 .05093 L
.67467 .06689 L
.69324 .13931 L
.70353 .17913 L
.71323 .21204 L
.72268 .2385 L
.73138 .25788 L
.74081 .27394 L
.74568 .28046 L
.75102 .28643 L
.76121 .29491 L
.76678 .29825 L
.77198 .30072 L
.77698 .3026 L
.78241 .3042 L
.78755 .30537 L
.7923 .30622 L
.79657 .30682 L
.80129 .30734 L
.80622 .30776 L
.81085 .30807 L
.81553 .3083 L
.82066 .30849 L
.82601 .30864 L
.831 .30874 L
.83568 .30881 L
.83993 .30886 L
.84485 .3089 L
.84949 .30893 L
.85411 .30895 L
.85919 .30896 L
.86175 .30897 L
.8645 .30897 L
.86688 .30898 L
.86947 .30898 L
.87203 .30898 L
.87441 .30899 L
Mistroke
.8791 .30899 L
.88337 .30899 L
.88791 .30899 L
.89052 .30899 L
.89327 .30899 L
.8982 .309 L
.90095 .309 L
.90351 .309 L
.9065 .309 L
.90933 .309 L
.91177 .309 L
.91442 .309 L
.91907 .309 L
.9217 .309 L
.92449 .309 L
.92693 .309 L
.9295 .309 L
.93184 .309 L
.93439 .309 L
.93888 .309 L
.94141 .309 L
.94411 .309 L
.94898 .309 L
.95134 .309 L
.95391 .309 L
.95617 .309 L
.95856 .309 L
.96072 .309 L
.96309 .309 L
.96726 .309 L
.96951 .309 L
.97157 .309 L
.97619 .309 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{165.75, 102.438},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002U0000IQ000`40O003h00ODWoo00<007ooOol0
D7oo001BOol00`00Oomoo`1@Ool0059oo`03001oogoo00=oo`03001oogoo04Yoo`00DWoo00<007oo
Ool00goo00<007ooOol0BWoo001BOol00`00Oomoo`02Ool2001<Ool0059oo`03001oogoo009oo`80
019oo`8003Qoo`00DGoo0P0017oo00<007oo000047oo00@007oo000003Moo`00:7oo0P0017oo0P00
17oo0P0017oo0P0017oo0P0017oo0P000goo0`000goo0P0017oo00<007oo000047oo00@007ooOol0
03Moo`009goo00@007ooOol000=oo`8000=oo`04001oogoo0002Ool01000Oomoo`000Woo00D007oo
Ool007oo0080009oo`05001oogooOol00004Ool20004Ool00`00Ool0000@Ool01@00Oomoogoo0000
=Woo000QOol500001Goo001oogoo000027oo00@007ooOol0009oo`04001oogoo0002Ool00`00Oomo
o`040002Ool01@00Oomoogoo00000goo10000goo00<007oo00003goo00<007ooOol00Woo00<007oo
Ool0=7oo000WOol01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0002Ool00`00Oomoo`02
00001Goo001oogoo00000goo00@007ooOomoo`<000Aoo`03001oo`0000moo`03001oogoo00=oo`03
001oogoo03=oo`009goo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00<007oo
Ool00P0000Eoo`00Oomoo`0000=oo`04001oogooOol30004Ool00`00Ool0000?Ool00`00Oomoo`03
Ool00`00Oomoo`0cOol002Qoo`8000Yoo`8000Aoo`8000Aoo`800004Ool007ooOol20003Ool20003
Ool30004Ool00`00Ool0000?Ool00`00Oomoo`03Ool00`00Oomoo`0cOol004=oo`04001oogoo0009
Ool30004Ool00`00Ool0000?Ool00`00Oomoo`03Ool00`00Oomoo`0cOol004=oo`04001oogoo0009
Ool30004Ool00`00Ool0000>Ool00`00Oomoo`05Ool00`00Oomoo`0bOol004=oo`04001oogoo0009
Ool30004Ool00`00Ool0000>Ool00`00Oomoo`05Ool00`00Oomoo`0bOol004=oo`04001oogoo0009
Ool30003Ool01000Oomoo`003Woo00<007ooOol01Goo00<007ooOol0<Woo0013Ool01000Oomoo`00
2Goo0`000goo00@007ooOol000ioo`03001oogoo00Eoo`03001oogoo039oo`00@Woo00D007ooOomo
o`0000Uoo`<000=oo`04001oogoo000=Ool00`00Oomoo`07Ool00`00Oomoo`0aOol0049oo`05001o
ogooOol00009Ool30003Ool01000Oomoo`003Goo00<007ooOol01goo00<007ooOol0<Goo0012Ool0
0`00Oomoo`02Ool00`00Oomoo`06Ool30003Ool01000Oomoo`003Goo00<007ooOol01goo00<007oo
Ool0<Goo0012Ool00`00Oomoo`02Ool00`00Oomoo`06Ool30003Ool01000Oomoo`003Goo00<007oo
Ool01goo00<007ooOol0<Goo0012Ool00`00Oomoo`02Ool00`00Oomoo`06Ool30003Ool01000Oomo
o`003Goo00<007ooOol027oo00<007ooOol0<7oo0012Ool00`00Oomoo`02Ool00`00Oomoo`06Ool3
0003Ool01000Oomoo`003Goo00<007ooOol027oo00<007ooOol0<7oo0012Ool00`00Oomoo`02Ool0
0`00Oomoo`06Ool30003Ool01000Oomoo`003Goo00<007ooOol027oo00<007ooOol0<7oo0012Ool0
0`00Oomoo`02Ool00`00Oomoo`06Ool30003Ool01@00Oomoogoo000037oo00<007ooOol027oo00<0
07ooOol0<7oo0011Ool00`00Oomoo`03Ool00`00Oomoo`06Ool30003Ool01@00Oomoogoo000037oo
00<007ooOol027oo00<007ooOol0<7oo000`Ool30003Ool01@00Oomoogoo00000Woo0`0000Aoo`00
Oomoo`800003Ool007oo00Moo`<000=oo`05001oogooOol0000<Ool00`00Oomoo`09Ool00`00Oomo
o`0_Ool003=oo`07001oogooOol007oo000017oo00H007ooOol007oo0002Ool20008Ool30002Ool0
0`00Oomoo`02Ool00`00Oomoo`0:Ool00`00Oomoo`09Ool00`00Oomoo`0_Ool002Yoo`D000Aoo`03
001oogoo009oo`03001oogoo00=oo`06001oogoo001oo`000Woo0P0027oo0`000Woo00<007ooOol0
0Woo00<007ooOol02Woo00<007ooOol02Goo00<007ooOol0;goo000`Ool30004Ool00`00Ool00004
Ool01P00Oomoo`00Ool0009oo`8000Qoo`@00003Ool007oo00=oo`03001oogoo00Yoo`03001oogoo
00Uoo`03001oogoo02moo`00<7oo00<007ooOol00goo00D007ooOomoo`0000=oo`06001oogoo001o
o`000Woo0P0027oo0`000Woo00<007ooOol00Woo00<007ooOol02Woo00<007ooOol02Woo00<007oo
Ool0;Woo000`Ool40009Ool20002Ool00`00Oomoo`0200000goo001oo`06Ool40002Ool00`00Oomo
o`02Ool00`00Oomoo`0:Ool00`00Oomoo`0:Ool00`00Oomoo`0^Ool0045oo`03001oogoo00Aoo`D0
0003Ool0000000<0009oo`03001oogoo009oo`03001oogoo00Yoo`03001oogoo00Yoo`03001oogoo
02ioo`00@Goo00<007ooOol017oo00<007ooOol00goo1@000Woo00<007ooOol00Woo00<007ooOol0
2Goo00<007ooOol037oo00<007ooOol0;Goo0011Ool00`00Oomoo`04Ool00`00Oomoo`03Ool20002
Ool01000Oomoo`0017oo00<007ooOol02Goo00<007ooOol037oo00<007ooOol0;Goo0010Ool00`00
Oomoo`05Ool00`00Oomoo`04Ool200001Goo001oogoo000017oo00<007ooOol02Goo00<007ooOol0
37oo00<007ooOol0;Goo0010Ool00`00Oomoo`05Ool00`00Oomoo`04Ool01000Oomoo`000Woo00<0
07ooOol00Woo00<007ooOol02Goo00<007ooOol037oo00<007ooOol0;Goo0010Ool00`00Oomoo`05
Ool00`00Oomoo`04Ool01000Oomoo`000Woo00<007ooOol00Woo00<007ooOol02Goo00<007ooOol0
3Goo00<007ooOol0;7oo0010Ool00`00Oomoo`05Ool00`00Oomoo`04Ool01000Oomoo`000Woo00<0
07ooOol00Woo00<007ooOol02Goo00<007ooOol03Goo00<007ooOol0;7oo0010Ool00`00Oomoo`05
Ool00`00Oomoo`04Ool01000Oomoo`000Woo00<007ooOol00Woo00<007ooOol02Goo00<007ooOol0
3Goo00<007ooOol0;7oo0010Ool00`00Oomoo`05Ool00`00Oomoo`04Ool01000Oomoo`000Woo00<0
07ooOol00Woo00<007ooOol02Goo00<007ooOol03Woo00<007ooOol0:goo000AOol2000DOol3000C
Ool40007Ool00`00Oomoo`04Ool01000Oomoo`000Woo00<007ooOol00goo00<007ooOol027oo1000
3Woo00<007ooOol00goo0`004goo0P003goo000@Ool01000Oomoo`0057oo00<007ooOol04goo00<0
07oo00001goo00<007ooOol017oo00@007ooOol0009oo`03001oogoo00=oo`03001oogoo00Moo`03
001oo`00011oo`03001oogoo00Aoo`03001oogoo015oo`04001oogoo000>Ool000Yoo`D00005Ool0
07ooOol0000<Ool500000goo00000002000=Ool50003Ool20007Ool00`00Oomoo`04Ool01000Oomo
o`000Woo00<007ooOol00goo00<007ooOol01goo00@007ooOol0011oo`04001oogooOol4000BOol0
1000Oomoo`003Woo000@Ool3000COol00`00Ool0000GOol00`00Oomoo`05Ool00`00Oomoo`04Ool0
1000Oomoo`000Woo00<007ooOol00goo00<007ooOol01goo00D007ooOomoo`0000moo`07001oogoo
Ool007oo00004goo0`003goo000@Ool00`00Oomoo`0COol00`00Ool0000DOol01000Oomoo`001goo
00<007ooOol017oo00@007ooOol0009oo`03001oogoo00=oo`03001oogoo00Moo`80009oo`03001o
ogoo00ioo`06001oogoo001oo`004goo00<007ooOol03goo000AOol3000COol2000EOol20008Ool0
0`00Oomoo`04Ool01000Oomoo`000Woo00<007ooOol00goo00<007ooOol01goo00@007oo0000019o
o`800003Ool0000001Aoo`<000ioo`00?goo00<007ooOol01Woo00<007ooOol017oo00@007ooOol0
009oo`03001oogoo00=oo`03001oogoo00Moo`03001oogoo01Eoo`03001oogoo02Eoo`00?goo00<0
07ooOol01Woo00<007ooOol017oo00@007ooOol0009oo`03001oogoo00=oo`03001oogoo00Moo`03
001oogoo01Ioo`8002Eoo`00?goo00<007ooOol01Woo00<007ooOol00goo00D007ooOomoo`00009o
o`03001oogoo00=oo`03001oogoo00Moo`03001oogoo01Moo`D0025oo`00Y@00000?Ool00`00Oomo
o`0AOol4000EOol01000Oomoo`002Goo00<007ooOol00Woo00D007ooOomoo`00009oo`03001oogoo
00=oo`03001oogoo00Moo`04001oogoo000FOol00`00Oomoo`0COol00`00Oomoo`0=Ool002Ioo`80
01Moo`03001oogoo00Moo`03001oogoo009oo`05001oogooOol00002Ool00`00Oomoo`03Ool00`00
Oomoo`07Ool00`00Oomoo`0mOol002Qoo`03001oogoo01Aoo`03001oogoo00Moo`03001oogoo009o
o`05001oogooOol00002Ool00`00Oomoo`03Ool00`00Oomoo`06Ool00`00Oomoo`0nOol002Uoo`03
001oogoo01=oo`03001oogoo00Moo`03001oogoo009oo`05001oogooOol00002Ool00`00Oomoo`03
Ool00`00Oomoo`06Ool00`00Oomoo`0nOol002Yoo`03001oogoo019oo`03001oogoo00Moo`03001o
ogoo009oo`05001oogooOol00002Ool00`00Oomoo`03Ool00`00Oomoo`06Ool00`00Oomoo`0nOol0
02Yoo`03001oogoo015oo`03001oogoo00Qoo`03001oogoo009oo`05001oogooOol00002Ool00`00
Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`0nOol002]oo`03001oogoo011oo`03001oogoo00Qo
o`03001oogoo009oo`05001oogooOol00002Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomo
o`0nOol002]oo`03001oogoo011oo`03001oogoo00Qoo`03001oogoo009oo`05001oogooOol00002
Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`0nOol002aoo`03001oogoo00moo`03001o
ogoo00Qoo`03001oogoo009oo`05001oogooOol00002Ool00`00Oomoo`04Ool00`00Oomoo`05Ool0
0`00Oomoo`0nOol002eoo`03001oogoo00ioo`03001oogoo00Qoo`03001oogoo009oo`05001oogoo
Ool00002Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`0nOol002eoo`03001oogoo00io
o`03001oogoo00Qoo`03001oogoo009oo`05001oogooOol00002Ool00`00Oomoo`04Ool00`00Oomo
o`05Ool00`00Oomoo`0nOol002ioo`03001oogoo00eoo`03001oogoo00Qoo`03001oogoo009oo`05
001oogooOol00002Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`0nOol002ioo`03001o
ogoo00eoo`03001oogoo00Qoo`03001oogoo009oo`05001oogooOol00002Ool00`00Oomoo`04Ool0
0`00Oomoo`05Ool00`00Oomoo`0nOol002ioo`03001oogoo00eoo`03001oogoo00Qoo`03001oogoo
009oo`05001oogooOol00002Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`0oOol002mo
o`03001oogoo00aoo`03001oogoo00Qoo`03001oogoo009oo`05001oogooOol00002Ool00`00Oomo
o`04Ool00`00Oomoo`04Ool00`00Oomoo`0oOol002moo`03001oogoo00aoo`03001oogoo00Uoo`05
001oogooOol00003Ool01000Oomoo`001Woo00<007ooOol017oo00<007ooOol0?goo000_Ool40003
Ool01@00Oomoogoo00000Woo0`0017oo0P0017oo00D007ooOomoo`0000=oo`04001oogoo0006Ool0
0`00Oomoo`04Ool00`00Oomoo`0oOol0031oo`04001oogoo0003Ool00`00Ool00004Ool00`00Oomo
o`02Ool01000Oomoo`000goo00D007ooOomoo`0000=oo`04001oogoo0006Ool00`00Oomoo`04Ool0
0`00Oomoo`0oOol0031oo`04001oogoo0004Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomo
o`000goo00D007ooOomoo`0000=oo`04001oogoo0007Ool00`00Oomoo`03Ool00`00Oomoo`0oOol0
031oo`<000Aoo`03001oo`0000=oo`8000Aoo`04001oogoo0003Ool01@00Oomoogoo00000goo00@0
07ooOol000Moo`03001oogoo00=oo`03001oogoo03moo`00<7oo0P0017oo00D007ooOomoo`00009o
o`8000Aoo`04001oogoo0003Ool01@00Oomoogoo00000goo0`0027oo00<007ooOol00goo00<007oo
Ool0?goo000`Ool40009Ool20005Ool20004Ool01@00Oomoogoo00000goo00<007oo000027oo00<0
07ooOol00goo00<007ooOol0?goo000aOol00`00Oomoo`09Ool00`00Oomoo`0:Ool01000Oomoogoo
0`0000Aoo`00Ool000Qoo`03001oogoo00=oo`03001oogoo03moo`00<Goo00<007ooOol02Goo00<0
07ooOol02Goo100000Qoo`00Ool007oo001oo`0027oo00<007ooOol00goo00<007ooOol0?goo000b
Ool00`00Oomoo`08Ool00`00Oomoo`0:Ool01000Oomoogoo0`0000Aoo`00Ool000Qoo`03001oogoo
00=oo`03001oogoo03moo`00<Woo00<007ooOol027oo00<007ooOol02Woo00D007ooOomoo`0000=o
o`03001oo`0000Qoo`03001oogoo009oo`03001oogoo041oo`00<Woo00<007ooOol027oo00<007oo
Ool02Woo00@007ooOomoo`<00004Ool007oo0008Ool00`00Oomoo`02Ool00`00Oomoo`10Ool0039o
o`03001oogoo00Qoo`03001oogoo00Yoo`05001oogooOol00003Ool00`00Ool00008Ool00`00Oomo
o`02Ool00`00Oomoo`10Ool0039oo`03001oogoo00Qoo`03001oogoo00Yoo`05001oogooOol00003
Ool00`00Ool00008Ool00`00Oomoo`02Ool00`00Oomoo`10Ool003=oo`03001oogoo00Moo`03001o
ogoo00Yoo`05001oogooOol00003Ool00`00Ool00009Ool01@00Oomoogoo0000@Woo000cOol00`00
Oomoo`07Ool00`00Oomoo`0:Ool01@00Oomoogoo00000goo00<007oo00002Goo00D007ooOomoo`00
049oo`00<goo00<007ooOol01Woo00<007ooOol02goo00D007ooOomoo`0000=oo`03001oo`0000Uo
o`05001oogooOol00012Ool003=oo`03001oogoo00Ioo`03001oogoo00]oo`05001oogooOol00003
Ool00`00Ool00009Ool01@00Oomoogoo0000@Woo000dOol00`00Oomoo`05Ool00`00Oomoo`0;Ool0
1@00Oomoogoo00000goo00<007oo00002Goo00D007ooOomoo`00049oo`00=7oo00<007ooOol01Goo
00<007ooOol02goo00D007ooOomoo`0000=oo`03001oo`0000Uoo`04001oogoo0013Ool003Aoo`03
001oogoo00Eoo`03001oogoo00]oo`05001oogooOol00003Ool00`00Ool00009Ool01000Oomoo`00
@goo000eOol00`00Oomoo`03Ool00`00Oomoo`0<Ool01000Oomoo`0017oo00<007oo00002Goo00@0
07ooOol004=oo`00=Goo00<007ooOol00goo00<007ooOol037oo00@007ooOol000Aoo`<000Uoo`04
001oogoo0013Ool003Eoo`03001oogoo00=oo`03001oogoo00eoo`03001oo`0000Aoo`<000Uoo`04
001oogoo0013Ool002Qoo`8000Aoo`8000Aoo`<000=oo`8000Aoo`8000Aoo`8000=oo`<000Aoo`<0
00Uoo`04001oogoo0013Ool002Moo`04001oogoo0003Ool20003Ool01000Oomoo`000Woo00@007oo
0000009oo`04001oogoo0002Ool01000Oomoo`000Woo0`0017oo0P002goo0P00A7oo000WOol01000
Oomoo`0027oo00@007ooOol0009oo`800005Ool007ooOol00002Ool01000Oomoo`000Woo00<007oo
Ool00`0017oo0P002goo0P00A7oo000WOol01000Oomoo`0027oo00@007ooOol0009oo`800005Ool0
07ooOol00002Ool01000Oomoo`000Woo00<007ooOol00`0017oo0P002goo0P00A7oo000WOol01000
Oomoo`0027oo00<007ooOol00P0000Eoo`00001oo`00009oo`04001oogoo0002Ool01000Oomoo`00
0Woo0`0017oo0P00DGoo000XOol2000:Ool200000goo000000030004Ool20004Ool20003Ool30004
Ool00`00Oomoo`1@Ool003Qoo`80015oo`03001oo`0000Aoo`03001oogoo051oo`00C7oo00<007oo
Ool00goo00<007ooOol0D7oo001<Ool00`00Oomoo`03Ool00`00Oomoo`1@Ool0059oo`03001oogoo
051oo`00DWoo00<007ooOol0D7oo0000\
\>"],
  ImageRangeCache->{{{0, 164.75}, {101.438, 0}} -> {-7.37782, -1.21005*^-5, 
  0.0895639, 2.38594*^-7}}]
}, Open  ]],

Cell["\<\
But note the oscillation - it becomes quite severe, and is \
amplified when we differentiate.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[HFuncOneD[x] - \(Norm'\)[x], \ {x, \ \(-7\), \ 7}, \ 
      PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0680272 0.177538 1935.79 [
[.09184 .16504 -6 -9 ]
[.09184 .16504 6 0 ]
[.22789 .16504 -6 -9 ]
[.22789 .16504 6 0 ]
[.36395 .16504 -6 -9 ]
[.36395 .16504 6 0 ]
[.63605 .16504 -3 -9 ]
[.63605 .16504 3 0 ]
[.77211 .16504 -3 -9 ]
[.77211 .16504 3 0 ]
[.90816 .16504 -3 -9 ]
[.90816 .16504 3 0 ]
[.4875 .08075 -48 -4.5 ]
[.4875 .08075 0 4.5 ]
[.4875 .27433 -42 -4.5 ]
[.4875 .27433 0 4.5 ]
[.4875 .37112 -36 -4.5 ]
[.4875 .37112 0 4.5 ]
[.4875 .46791 -42 -4.5 ]
[.4875 .46791 0 4.5 ]
[.4875 .5647 -36 -4.5 ]
[.4875 .5647 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.09184 .17754 m
.09184 .18379 L
s
[(-6)] .09184 .16504 0 1 Mshowa
.22789 .17754 m
.22789 .18379 L
s
[(-4)] .22789 .16504 0 1 Mshowa
.36395 .17754 m
.36395 .18379 L
s
[(-2)] .36395 .16504 0 1 Mshowa
.63605 .17754 m
.63605 .18379 L
s
[(2)] .63605 .16504 0 1 Mshowa
.77211 .17754 m
.77211 .18379 L
s
[(4)] .77211 .16504 0 1 Mshowa
.90816 .17754 m
.90816 .18379 L
s
[(6)] .90816 .16504 0 1 Mshowa
.125 Mabswid
.12585 .17754 m
.12585 .18129 L
s
.15986 .17754 m
.15986 .18129 L
s
.19388 .17754 m
.19388 .18129 L
s
.2619 .17754 m
.2619 .18129 L
s
.29592 .17754 m
.29592 .18129 L
s
.32993 .17754 m
.32993 .18129 L
s
.39796 .17754 m
.39796 .18129 L
s
.43197 .17754 m
.43197 .18129 L
s
.46599 .17754 m
.46599 .18129 L
s
.53401 .17754 m
.53401 .18129 L
s
.56803 .17754 m
.56803 .18129 L
s
.60204 .17754 m
.60204 .18129 L
s
.67007 .17754 m
.67007 .18129 L
s
.70408 .17754 m
.70408 .18129 L
s
.7381 .17754 m
.7381 .18129 L
s
.80612 .17754 m
.80612 .18129 L
s
.84014 .17754 m
.84014 .18129 L
s
.87415 .17754 m
.87415 .18129 L
s
.05782 .17754 m
.05782 .18129 L
s
.02381 .17754 m
.02381 .18129 L
s
.94218 .17754 m
.94218 .18129 L
s
.97619 .17754 m
.97619 .18129 L
s
.25 Mabswid
0 .17754 m
1 .17754 L
s
.5 .08075 m
.50625 .08075 L
s
[(-0.00005)] .4875 .08075 1 0 Mshowa
.5 .27433 m
.50625 .27433 L
s
[(0.00005)] .4875 .27433 1 0 Mshowa
.5 .37112 m
.50625 .37112 L
s
[(0.0001)] .4875 .37112 1 0 Mshowa
.5 .46791 m
.50625 .46791 L
s
[(0.00015)] .4875 .46791 1 0 Mshowa
.5 .5647 m
.50625 .5647 L
s
[(0.0002)] .4875 .5647 1 0 Mshowa
.125 Mabswid
.5 .10011 m
.50375 .10011 L
s
.5 .11946 m
.50375 .11946 L
s
.5 .13882 m
.50375 .13882 L
s
.5 .15818 m
.50375 .15818 L
s
.5 .1969 m
.50375 .1969 L
s
.5 .21625 m
.50375 .21625 L
s
.5 .23561 m
.50375 .23561 L
s
.5 .25497 m
.50375 .25497 L
s
.5 .29369 m
.50375 .29369 L
s
.5 .31304 m
.50375 .31304 L
s
.5 .3324 m
.50375 .3324 L
s
.5 .35176 m
.50375 .35176 L
s
.5 .39047 m
.50375 .39047 L
s
.5 .40983 m
.50375 .40983 L
s
.5 .42919 m
.50375 .42919 L
s
.5 .44855 m
.50375 .44855 L
s
.5 .48726 m
.50375 .48726 L
s
.5 .50662 m
.50375 .50662 L
s
.5 .52598 m
.50375 .52598 L
s
.5 .54534 m
.50375 .54534 L
s
.5 .06139 m
.50375 .06139 L
s
.5 .04203 m
.50375 .04203 L
s
.5 .02267 m
.50375 .02267 L
s
.5 .00332 m
.50375 .00332 L
s
.5 .58405 m
.50375 .58405 L
s
.5 .60341 m
.50375 .60341 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
.5 Mabswid
.02381 .17754 m
.02605 .17754 L
.02846 .17754 L
.03279 .17754 L
.03543 .17754 L
.03793 .17754 L
.04262 .17754 L
.04502 .17754 L
.04756 .17754 L
.05212 .17754 L
.05466 .17754 L
.05704 .17754 L
.06244 .17754 L
.06504 .17754 L
.06788 .17754 L
.07038 .17754 L
.07305 .17754 L
.07568 .17754 L
.07807 .17754 L
.08274 .17754 L
.08556 .17754 L
.08814 .17754 L
.09093 .17754 L
.09399 .17754 L
.09914 .17754 L
.10195 .17754 L
.10458 .17754 L
.10959 .17754 L
.11241 .17754 L
.11503 .17754 L
.11983 .17754 L
.12226 .17754 L
.12493 .17754 L
.12954 .17754 L
.1345 .17755 L
.13919 .17755 L
.14349 .17755 L
.14823 .17756 L
.15329 .17757 L
.15807 .17758 L
.16241 .1776 L
.16752 .17762 L
.17306 .17765 L
.17829 .17769 L
.18311 .17775 L
.18779 .17781 L
.19281 .1779 L
.19755 .17801 L
.20188 .17813 L
.20694 .17832 L
Mistroke
.21246 .17857 L
.22252 .17922 L
.22785 .17969 L
.23286 .18023 L
.24223 .18153 L
.25232 .18343 L
.26321 .18611 L
.2828 .19223 L
.29317 .1955 L
.29887 .19699 L
.30168 .19757 L
.30427 .19801 L
.30555 .19817 L
.30676 .1983 L
.30802 .1984 L
.30868 .19844 L
.3094 .19847 L
.31008 .19849 L
.31082 .1985 L
.31159 .19849 L
.31231 .19847 L
.3136 .19839 L
.31432 .19832 L
.31498 .19825 L
.31623 .19808 L
.31739 .19787 L
.32 .19725 L
.3223 .19649 L
.32473 .19547 L
.32943 .1928 L
.33382 .18942 L
.33864 .1847 L
.34381 .17843 L
.3636 .14566 L
.37404 .12773 L
.37931 .12061 L
.38231 .11756 L
.38366 .11649 L
.38514 .11554 L
.38644 .11491 L
.38766 .11453 L
.38895 .11433 L
.38967 .11431 L
.39032 .11437 L
.3915 .11462 L
.39277 .11512 L
.39397 .11583 L
.39508 .11669 L
.39756 .11934 L
.40029 .12348 L
Mistroke
.40314 .12918 L
.40581 .13581 L
.41503 .16764 L
.42495 .21237 L
.42985 .23477 L
.43506 .25561 L
.43807 .26497 L
.43949 .26848 L
.44079 .27112 L
.4421 .27315 L
.44279 .27394 L
.44352 .27457 L
.44475 .2751 L
.44545 .2751 L
.4461 .2749 L
.44735 .27393 L
.44805 .27304 L
.44871 .27198 L
.4499 .26946 L
.45118 .26593 L
.45337 .25768 L
.4558 .24535 L
.45838 .22852 L
.46116 .20628 L
.46619 .15685 L
.47083 .10518 L
.47578 .05277 L
.47724 .03986 L
.47862 .02955 L
.47986 .02224 L
.48056 .01909 L
.4812 .01688 L
.48237 .01478 L
.48366 .01572 L
.4843 .01764 L
.485 .02089 L
.48625 .03023 L
.48756 .04536 L
.48831 .05657 L
.48898 .06872 L
.49156 .13296 L
.493 .18352 L
.49452 .25003 L
.4972 .40559 L
.49841 .49419 L
.49971 .60332 L
.50093 .54709 L
.50205 .45893 L
.50477 .28585 L
.50603 .22442 L
Mistroke
.50735 .17006 L
.50856 .12946 L
.50988 .09327 L
.51109 .06735 L
.51222 .04846 L
.5129 .03938 L
.51354 .03222 L
.51476 .02236 L
.51607 .01641 L
.51681 .01496 L
.5175 .01472 L
.51819 .01547 L
.51894 .01731 L
.51965 .01994 L
.5203 .02306 L
.52154 .0306 L
.52284 .04052 L
.52854 .09813 L
.53825 .20105 L
.54104 .22424 L
.54369 .24232 L
.54606 .25513 L
.5474 .26089 L
.54865 .26537 L
.54988 .26891 L
.551 .27143 L
.55225 .27345 L
.5529 .27419 L
.55359 .27473 L
.55489 .27513 L
.5561 .2748 L
.5568 .27432 L
.55743 .27371 L
.55816 .2728 L
.55884 .27175 L
.5612 .26687 L
.56337 .26078 L
.56828 .24277 L
.58661 .16106 L
.59128 .14443 L
.59385 .13674 L
.5963 .13047 L
.59856 .1256 L
.60104 .12131 L
.60328 .11833 L
.60536 .11632 L
.60666 .11543 L
.60739 .11504 L
.60807 .11476 L
.6093 .11443 L
Mistroke
.61061 .11431 L
.61178 .11441 L
.61287 .11467 L
.61407 .11514 L
.61536 .11583 L
.61673 .11678 L
.61822 .11804 L
.62093 .12089 L
.626 .12779 L
.64542 .16183 L
.65572 .1778 L
.66113 .18445 L
.66682 .18997 L
.67188 .19364 L
.67475 .19522 L
.67741 .19638 L
.67987 .19721 L
.6821 .19777 L
.68334 .19801 L
.68468 .19821 L
.68594 .19835 L
.68711 .19844 L
.6882 .19848 L
.68938 .1985 L
.6905 .19848 L
.69151 .19843 L
.6927 .19835 L
.694 .19823 L
.69635 .19791 L
.69881 .19748 L
.70143 .19692 L
.70613 .1957 L
.72579 .18941 L
.74384 .1843 L
.75324 .18232 L
.76315 .18074 L
.7731 .1796 L
.77874 .17913 L
.78399 .17877 L
.78903 .1785 L
.79449 .17826 L
.79965 .17809 L
.80442 .17796 L
.80907 .17786 L
.81344 .17779 L
.81798 .17773 L
.82299 .17768 L
.82796 .17764 L
.83338 .17761 L
.8385 .17759 L
Mistroke
.84325 .17758 L
.84795 .17757 L
.85222 .17756 L
.85714 .17755 L
.86177 .17755 L
.86643 .17755 L
.87154 .17754 L
.87412 .17754 L
.87688 .17754 L
.87926 .17754 L
.88186 .17754 L
.88444 .17754 L
.88682 .17754 L
.89152 .17754 L
.89579 .17754 L
.90033 .17754 L
.90271 .17754 L
.90522 .17754 L
.90971 .17754 L
.91221 .17754 L
.91453 .17754 L
.91725 .17754 L
.91981 .17754 L
.92203 .17754 L
.92444 .17754 L
.92865 .17754 L
.93104 .17754 L
.93356 .17754 L
.93809 .17754 L
.94062 .17754 L
.94302 .17754 L
.94753 .17754 L
.95005 .17754 L
.95238 .17754 L
.95511 .17754 L
.95767 .17754 L
.95991 .17754 L
.96233 .17754 L
.96654 .17754 L
.96894 .17754 L
.97146 .17754 L
.97599 .17754 L
.97619 .17754 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{171.438, 105.938},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002[0000JA000`40O003h00OEGoo00<007ooOol0
Dgoo001EOol00`00Oomoo`1COol0059oo`05001oogoo001oo`02001BOol0055oo`80009oo`04001o
o`00001BOol0055oo`80009oo`04001oo`00001BOol0055oo`80009oo`04001oo`00001BOol0055o
o`80009oo`05001oo`00Ool0001AOol0055oo`09001oo`00Ool007oo001oo`00055oo`00DGoo00T0
07oo001oo`00Ool007oo0000DGoo001AOol02@00Ool007oo001oo`00Ool0001AOol0055oo`09001o
o`00Ool007oo001oo`00055oo`00:goo0P0017oo0P0017oo0P0017oo0P0017oo0P0017oo0P000goo
100000Qoo`00Ool007oo001oo`00DGoo000ZOol01000Oomoo`000goo0P000goo00@007ooOol0009o
o`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol000Eoo`07001oo`00Ool007oo00000Woo
00<007ooOol0CWoo000TOol500001Goo001oogoo000027oo00@007ooOol0009oo`04001oogoo0002
Ool01000Oomoo`000Woo00@007ooOol000Eoo`04001oo`00Ool30002Ool00`00Oomoo`1>Ool002Yo
o`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0002Ool01000Oomo
o`000Woo0`000Woo00@007oo000000=oo`03001oogoo04ioo`00:Woo00@007ooOol000Qoo`04001o
ogoo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0002Ool00`00Ool00002Ool0
1000Ool000000goo00<007ooOol0CWoo000[Ool2000:Ool20004Ool20004Ool20004Ool20003Ool4
00000goo001oo`020003Ool00`00Oomoo`1>Ool0051oo`05001oogoo001oo`020003Ool00`00Oomo
o`1>Ool0051oo`05001oogoo001oo`020003Ool00`00Oomoo`1>Ool0045oo`<000aoo`05001oogoo
001oo`020003Ool00`00Oomoo`0:Ool30011Ool0041oo`800003Ool007oo00]oo`05001oogoo001o
o`020003Ool00`00Oomoo`09Ool01000Oomoo`00@Goo000AOol2000FOol3000COol400000goo001o
o`0:Ool01@00Oomoo`00Ool00P000goo00<007ooOol027oo00<007ooOol00Woo100057oo0`0057oo
0P0047oo000@Ool01000Oomoo`005Woo00<007ooOol04Woo0P000goo00<007ooOol027oo00H007oo
Oomoo`00Ool20003Ool00`00Oomoo`08Ool00`00Oomoo`02Ool2000GOol00`00Oomoo`0BOol01000
Oomoo`003goo000:Ool500001Goo001oogoo00003Woo1@0000=oo`0000000P003Goo1P000Woo00D0
07ooOomoo`0000Uoo`06001oogooOol007oo0P000goo00<007ooOol027oo00<007ooOol00goo0P00
57oo10004goo00@007ooOol000moo`0047oo0`005Goo00<007oo00004goo00D007ooOomoo`00009o
o`03001oogoo00Moo`06001oogooOol007oo0P000goo00<007ooOol01goo00<007ooOol017oo00<0
07oo00004goo00<007oo000057oo0`0047oo000@Ool00`00Oomoo`0EOol00`00Ool0000COol20002
Ool01000Oomoo`002Goo00H007ooOomoo`00Ool20003Ool00`00Oomoo`07Ool00`00Oomoo`03Ool0
1000Ool000004goo00<007oo000057oo00<007ooOol047oo000AOol3000EOol2000BOol00`00Oomo
o`020004Ool00`00Oomoo`06Ool01P00Oomoogoo001oo`8000Aoo`03001oogoo00Eoo`03001oogoo
00Eoo`8001Eoo`8001Eoo`<000moo`00?7oo00<007ooOol01goo00<007ooOol01Woo00<007ooOol0
0Woo0`0017oo00<007ooOol01Goo00<007ooOol01goo00<007ooOol0>goo000lOol00`00Oomoo`07
Ool00`00Oomoo`06Ool00`00Oomoo`02Ool30004Ool00`00Oomoo`05Ool00`00Oomoo`07Ool00`00
Oomoo`0kOol003]oo`03001oogoo00Qoo`03001oogoo00Ioo`03001oogoo009oo`<000Aoo`03001o
ogoo00Eoo`03001oogoo00Qoo`03001oogoo03Yoo`00Z`00000?Ool00`00Oomoo`0EOol01000Oomo
ogoo1@002Goo00<007ooOol00Woo00<007ooOol01Woo00<007ooOol017oo00<007ooOol00goo0`00
17oo00<007ooOol017oo00<007ooOol01goo00D007ooOomoo`0000aoo`@0009oo`03001oogoo01Ao
o`03001oogoo00ioo`00<7oo0P001Woo0P003Goo00<007ooOol017oo00<007ooOol00goo0`0017oo
00<007ooOol017oo00<007ooOol037oo0P001goo0`00;Woo000bOol6000?Ool00`00Oomoo`04Ool0
0`00Oomoo`03Ool30004Ool00`00Oomoo`04Ool00`00Oomoo`0=Ool8000aOol004Moo`03001oogoo
00Aoo`03001oogoo00=oo`<000Aoo`03001oogoo00Aoo`03001oogoo04Ioo`00B7oo00<007ooOol0
0goo00<007ooOol00goo0`0017oo00<007ooOol00goo00<007ooOol0Agoo0018Ool00`00Oomoo`03
Ool00`00Oomoo`03Ool30004Ool00`00Oomoo`03Ool00`00Oomoo`17Ool004Qoo`03001oogoo00=o
o`03001oogoo00=oo`<000Eoo`03001oogoo009oo`03001oogoo04Moo`00BGoo00<007ooOol00Woo
00<007ooOol00goo0`001Goo00<007ooOol00Woo00<007ooOol0Agoo0019Ool00`00Oomoo`02Ool0
0`00Oomoo`03Ool30005Ool00`00Oomoo`02Ool00`00Oomoo`17Ool004Uoo`05001oogooOol00006
Ool30005Ool01@00Oomoogoo0000BWoo0019Ool01@00Oomoogoo00001Woo0`001Goo00D007ooOomo
o`0004Yoo`00BWoo00@007ooOol000Ioo`<000Eoo`05001oogooOol0001:Ool004Yoo`04001oogoo
0006Ool30006Ool00`00Ool0001;Ool002]oo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`80
009oo`@000=oo`<000Ioo`03001oo`0004]oo`00:Woo00@007ooOol000=oo`8000=oo`04001oogoo
0002Ool01000Oomoo`000Woo00@007ooOol0009oo`03001oo`00008000Aoo`03001oogoo00<000Io
o`03001oo`0004]oo`00:Woo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@0
07ooOol0009oo`03001oogoo008000Aoo`03001oogoo00<000Ioo`<004]oo`00:Woo00@007ooOol0
00Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0002Ool30003
Ool3001DOol002Yoo`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo
0002Ool01000Oomoo`000Woo00<007ooOol00goo0`00E7oo000[Ool2000:Ool20004Ool20004Ool2
0004Ool20003Ool40002Ool3001DOol005Aoo`<005Aoo`00E7oo0`00E7oo001DOol3001DOol005Ao
o`<005Aoo`00E7oo0`00E7oo001DOol3001DOol005Aoo`<005Aoo`00E7oo0`00E7oo001DOol3001D
Ool005Aoo`<005Aoo`00E7oo0`00E7oo000aOol20004Ool20004Ool20004Ool20004Ool20003Ool3
0003Ool3001DOol0031oo`04001oogoo0003Ool20003Ool01000Oomoo`000Woo00@007ooOol0009o
o`04001oogoo0003Ool00`00Oomoo`02Ool3001DOol0031oo`04001oogoo0008Ool01000Oomoo`00
0Woo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`02Ool3001DOol0031oo`04001oogoo
0008Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`02Ool2001E
Ool0031oo`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0003Ool0
0`00Oomoo`02Ool2001EOol0035oo`8000Yoo`8000Aoo`8000Aoo`8000=oo`8000Aoo`8005Eoo`00
E7oo0P00EGoo001DOol2001EOol005Aoo`8005Eoo`00E7oo0P00EGoo001DOol2001EOol005Aoo`80
05Eoo`00E7oo0P00EGoo001DOol2001EOol005Aoo`8005Eoo`00E7oo0P00EGoo000[Ool20004Ool2
0004Ool20004Ool20004Ool20003Ool30003Ool30003Ool2001EOol002Yoo`04001oogoo0003Ool2
0003Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`05Ool00`00
Oomoo`02001EOol002Yoo`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol0009oo`04001o
ogoo0003Ool00`00Oomoo`05Ool00`00Oomoo`03001DOol002Yoo`04001oogoo0008Ool01000Oomo
o`000Woo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`02Ool30003Ool2001EOol002Yo
o`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomo
o`02Ool00`00Oomoo`03Ool2001EOol002]oo`8000Yoo`8000Aoo`8000Aoo`8000=oo`8000Aoo`@0
009oo`8005Eoo`00E7oo0P00EGoo001DOol2001EOol005Aoo`8005Eoo`00E7oo0P00EGoo001DOol2
001EOol005Aoo`8005Eoo`00E7oo0P00EGoo001DOol2001EOol005Aoo`8005Eoo`00E7oo0P00EGoo
000aOol20004Ool20004Ool20004Ool20004Ool20003Ool40003Ool00`00Oomoo`1COol0031oo`04
001oogoo0003Ool20003Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0003Ool00`00
Oomoo`03Ool00`00Oomoo`1COol0031oo`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol0
009oo`04001oogoo0004Ool00`00Oomoo`02Ool2001DOol0031oo`04001oogoo0008Ool01000Oomo
o`000Woo00@007ooOol0009oo`04001oogoo0005Ool01@00Oomoogoo0000EGoo000`Ool01000Oomo
o`0027oo00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol000=oo`03
001oogoo05=oo`00<Goo0P002Woo0P0017oo0P0017oo0P0017oo0P0017oo00<007ooOol0Dgoo001E
Ool00`00Oomoo`1COol005Eoo`03001oogoo05=oo`00EGoo00<007ooOol0Dgoo001EOol00`00Oomo
o`1COol005Eoo`03001oogoo05=oo`00EGoo00<007ooOol0Dgoo0000\
\>"],
  ImageRangeCache->{{{0, 170.438}, {104.938, 0}} -> {-7.37795, -9.17151*^-5, 
  0.0865766, 3.04246*^-6}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[HFuncOneD2[x] - \(\(Norm'\)'\)[x], \ {x, \ \(-7\), \ 7}, \ 
      PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0680272 0.317272 96.1267 [
[.09184 .30477 -6 -9 ]
[.09184 .30477 6 0 ]
[.22789 .30477 -6 -9 ]
[.22789 .30477 6 0 ]
[.36395 .30477 -6 -9 ]
[.36395 .30477 6 0 ]
[.63605 .30477 -3 -9 ]
[.63605 .30477 3 0 ]
[.77211 .30477 -3 -9 ]
[.77211 .30477 3 0 ]
[.90816 .30477 -3 -9 ]
[.90816 .30477 3 0 ]
[.4875 .02889 -36 -4.5 ]
[.4875 .02889 0 4.5 ]
[.4875 .12502 -36 -4.5 ]
[.4875 .12502 0 4.5 ]
[.4875 .22115 -36 -4.5 ]
[.4875 .22115 0 4.5 ]
[.4875 .4134 -30 -4.5 ]
[.4875 .4134 0 4.5 ]
[.4875 .50953 -30 -4.5 ]
[.4875 .50953 0 4.5 ]
[.4875 .60565 -30 -4.5 ]
[.4875 .60565 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.09184 .31727 m
.09184 .32352 L
s
[(-6)] .09184 .30477 0 1 Mshowa
.22789 .31727 m
.22789 .32352 L
s
[(-4)] .22789 .30477 0 1 Mshowa
.36395 .31727 m
.36395 .32352 L
s
[(-2)] .36395 .30477 0 1 Mshowa
.63605 .31727 m
.63605 .32352 L
s
[(2)] .63605 .30477 0 1 Mshowa
.77211 .31727 m
.77211 .32352 L
s
[(4)] .77211 .30477 0 1 Mshowa
.90816 .31727 m
.90816 .32352 L
s
[(6)] .90816 .30477 0 1 Mshowa
.125 Mabswid
.12585 .31727 m
.12585 .32102 L
s
.15986 .31727 m
.15986 .32102 L
s
.19388 .31727 m
.19388 .32102 L
s
.2619 .31727 m
.2619 .32102 L
s
.29592 .31727 m
.29592 .32102 L
s
.32993 .31727 m
.32993 .32102 L
s
.39796 .31727 m
.39796 .32102 L
s
.43197 .31727 m
.43197 .32102 L
s
.46599 .31727 m
.46599 .32102 L
s
.53401 .31727 m
.53401 .32102 L
s
.56803 .31727 m
.56803 .32102 L
s
.60204 .31727 m
.60204 .32102 L
s
.67007 .31727 m
.67007 .32102 L
s
.70408 .31727 m
.70408 .32102 L
s
.7381 .31727 m
.7381 .32102 L
s
.80612 .31727 m
.80612 .32102 L
s
.84014 .31727 m
.84014 .32102 L
s
.87415 .31727 m
.87415 .32102 L
s
.05782 .31727 m
.05782 .32102 L
s
.02381 .31727 m
.02381 .32102 L
s
.94218 .31727 m
.94218 .32102 L
s
.97619 .31727 m
.97619 .32102 L
s
.25 Mabswid
0 .31727 m
1 .31727 L
s
.5 .02889 m
.50625 .02889 L
s
[(-0.003)] .4875 .02889 1 0 Mshowa
.5 .12502 m
.50625 .12502 L
s
[(-0.002)] .4875 .12502 1 0 Mshowa
.5 .22115 m
.50625 .22115 L
s
[(-0.001)] .4875 .22115 1 0 Mshowa
.5 .4134 m
.50625 .4134 L
s
[(0.001)] .4875 .4134 1 0 Mshowa
.5 .50953 m
.50625 .50953 L
s
[(0.002)] .4875 .50953 1 0 Mshowa
.5 .60565 m
.50625 .60565 L
s
[(0.003)] .4875 .60565 1 0 Mshowa
.125 Mabswid
.5 .04812 m
.50375 .04812 L
s
.5 .06734 m
.50375 .06734 L
s
.5 .08657 m
.50375 .08657 L
s
.5 .10579 m
.50375 .10579 L
s
.5 .14424 m
.50375 .14424 L
s
.5 .16347 m
.50375 .16347 L
s
.5 .18269 m
.50375 .18269 L
s
.5 .20192 m
.50375 .20192 L
s
.5 .24037 m
.50375 .24037 L
s
.5 .2596 m
.50375 .2596 L
s
.5 .27882 m
.50375 .27882 L
s
.5 .29805 m
.50375 .29805 L
s
.5 .3365 m
.50375 .3365 L
s
.5 .35572 m
.50375 .35572 L
s
.5 .37495 m
.50375 .37495 L
s
.5 .39417 m
.50375 .39417 L
s
.5 .43262 m
.50375 .43262 L
s
.5 .45185 m
.50375 .45185 L
s
.5 .47107 m
.50375 .47107 L
s
.5 .4903 m
.50375 .4903 L
s
.5 .52875 m
.50375 .52875 L
s
.5 .54798 m
.50375 .54798 L
s
.5 .5672 m
.50375 .5672 L
s
.5 .58643 m
.50375 .58643 L
s
.5 .00967 m
.50375 .00967 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
.5 Mabswid
.02381 .31727 m
.02605 .31727 L
.02846 .31727 L
.03279 .31727 L
.03543 .31727 L
.03793 .31727 L
.04262 .31727 L
.04502 .31727 L
.04756 .31727 L
.05212 .31727 L
.05466 .31727 L
.05704 .31727 L
.06244 .31727 L
.06504 .31727 L
.06788 .31727 L
.07038 .31727 L
.07305 .31727 L
.07568 .31727 L
.07807 .31727 L
.08274 .31727 L
.08556 .31727 L
.08814 .31727 L
.09093 .31727 L
.09399 .31727 L
.09914 .31727 L
.10195 .31727 L
.10458 .31727 L
.10959 .31727 L
.11241 .31727 L
.11503 .31727 L
.11983 .31727 L
.12493 .31727 L
.12948 .31727 L
.13375 .31727 L
.13844 .31727 L
.14349 .31728 L
.14845 .31728 L
.15306 .31728 L
.15812 .31728 L
.16353 .31729 L
.16922 .31729 L
.17458 .3173 L
.1794 .31731 L
.18463 .31732 L
.19409 .31735 L
.19947 .31737 L
.2045 .31739 L
.21465 .31747 L
.22034 .31752 L
.22555 .31757 L
Mistroke
.24457 .31786 L
.25545 .31806 L
.26531 .31824 L
.27019 .31831 L
.27276 .31834 L
.27416 .31836 L
.27546 .31837 L
.27671 .31838 L
.27784 .31838 L
.2791 .31839 L
.27979 .31839 L
.28044 .31839 L
.28158 .31839 L
.28264 .31839 L
.28379 .31839 L
.28502 .31838 L
.28631 .31837 L
.2875 .31835 L
.29017 .31831 L
.29253 .31826 L
.295 .31819 L
.30026 .31798 L
.30324 .31782 L
.30601 .31764 L
.31107 .31724 L
.31656 .3167 L
.3265 .31544 L
.3451 .31259 L
.34968 .31198 L
.3522 .31169 L
.35459 .31146 L
.35594 .31136 L
.35741 .31126 L
.35875 .31119 L
.35998 .31114 L
.36114 .31111 L
.36242 .3111 L
.36307 .3111 L
.36376 .31111 L
.36502 .31115 L
.36634 .31121 L
.36757 .31129 L
.36892 .3114 L
.37036 .31156 L
.37331 .312 L
.376 .31253 L
.38118 .31396 L
.38607 .31577 L
.40437 .3256 L
.40938 .32845 L
Mistroke
.41214 .32986 L
.41477 .33104 L
.4172 .33194 L
.41849 .33233 L
.41986 .33267 L
.42112 .3329 L
.42227 .33305 L
.42342 .33313 L
.42451 .33313 L
.42566 .33306 L
.42692 .33289 L
.42825 .3326 L
.42948 .33222 L
.43168 .33127 L
.43405 .32984 L
.43682 .3276 L
.43944 .32492 L
.44436 .31841 L
.45456 .30016 L
.46015 .28956 L
.46289 .28507 L
.46408 .28338 L
.46539 .28179 L
.46647 .28071 L
.46762 .27983 L
.46883 .27925 L
.46995 .27907 L
.47059 .27914 L
.47128 .27937 L
.47253 .28022 L
.47375 .28164 L
.47487 .28353 L
.47612 .28637 L
.47729 .28978 L
.47868 .2949 L
.47995 .30068 L
.48254 .31621 L
.48532 .33936 L
.48767 .36506 L
.48984 .39472 L
.49243 .43838 L
.49477 .48685 L
.49686 .53822 L
.49796 .56859 L
.49912 .60332 L
.50036 .01472 L
.5015 .05023 L
.50374 .11208 L
.5063 .17111 L
.50872 .21686 L
Mistroke
.51326 .28033 L
.51579 .30533 L
.51814 .32289 L
.52071 .33698 L
.52201 .34231 L
.52344 .34696 L
.52466 .35003 L
.52582 .35224 L
.52686 .35369 L
.528 .35476 L
.52923 .35536 L
.53055 .35544 L
.53179 .35505 L
.53291 .35435 L
.53405 .35334 L
.53511 .35218 L
.53748 .34892 L
.54179 .34144 L
.55087 .32407 L
.55587 .31579 L
.5606 .30958 L
.56328 .30685 L
.56619 .30454 L
.56747 .30374 L
.56884 .30302 L
.57002 .30251 L
.57129 .30208 L
.57256 .30176 L
.57373 .30155 L
.57485 .30144 L
.57603 .30141 L
.57732 .30146 L
.57853 .30159 L
.57985 .30182 L
.58126 .30215 L
.58385 .30297 L
.58659 .30409 L
.5915 .30658 L
.6026 .31304 L
.61231 .31807 L
.61776 .32023 L
.62035 .32106 L
.62275 .32171 L
.62527 .32228 L
.62755 .32269 L
.63008 .32303 L
.63147 .32318 L
.63278 .32328 L
.63394 .32335 L
.63517 .3234 L
Mistroke
.63622 .32343 L
.63736 .32344 L
.6386 .32344 L
.63929 .32342 L
.63992 .32341 L
.64115 .32336 L
.64231 .3233 L
.64453 .32315 L
.64687 .32294 L
.65109 .32247 L
.66062 .32108 L
.67024 .31958 L
.68035 .3182 L
.68898 .31729 L
.69387 .31691 L
.69841 .31663 L
.70094 .31651 L
.70333 .31641 L
.70549 .31634 L
.70779 .31628 L
.71029 .31622 L
.71259 .31619 L
.71387 .31618 L
.71507 .31617 L
.71638 .31616 L
.71712 .31616 L
.71779 .31615 L
.71908 .31615 L
.71973 .31615 L
.72045 .31615 L
.72172 .31616 L
.72292 .31617 L
.72432 .31618 L
.7256 .31619 L
.7285 .31622 L
.73344 .31629 L
.73866 .31638 L
.75859 .31674 L
.76728 .31688 L
.77672 .317 L
.78679 .31709 L
.7963 .31715 L
.80158 .31718 L
.80719 .3172 L
.81244 .31722 L
.81724 .31723 L
.82174 .31724 L
.82665 .31725 L
.83184 .31725 L
.83678 .31726 L
Mistroke
.84137 .31726 L
.84627 .31726 L
.85089 .31727 L
.85508 .31727 L
.85996 .31727 L
.86511 .31727 L
.86951 .31727 L
.87433 .31727 L
.87953 .31727 L
.88244 .31727 L
.8851 .31727 L
.88757 .31727 L
.89026 .31727 L
.89262 .31727 L
.89515 .31727 L
.9 .31727 L
.90456 .31727 L
.90719 .31727 L
.90997 .31727 L
.91493 .31727 L
.91745 .31727 L
.92013 .31727 L
.92497 .31727 L
.92793 .31727 L
.93072 .31727 L
.93342 .31727 L
.93594 .31727 L
.94058 .31727 L
.94303 .31727 L
.94567 .31727 L
.94843 .31727 L
.95105 .31727 L
.95343 .31727 L
.95596 .31727 L
.95837 .31727 L
.96055 .31727 L
.96479 .31727 L
.96718 .31727 L
.96975 .31727 L
.97442 .31727 L
.97619 .31727 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{171.438, 105.938},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002[0000JA000`40O003h00OEGoo00<007ooOol0
Dgoo001EOol00`00Oomoo`1COol003Moo`8000Aoo`8000Aoo`8000Aoo`8000=oo`<000Aoo`03001o
ogoo05=oo`00=Woo00@007ooOol000=oo`8000=oo`04001oogoo0002Ool01000Oomoo`001Goo00D0
07ooOomoo`0005Eoo`00<7oo1@0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool01000Oomoo`00
1Goo00@007ooOomoo`8005Aoo`00=Woo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`00
0goo0P0017oo00<007ooOol0Dgoo000fOol01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo
0005Ool01@00Oomoogoo0000EGoo000gOol2000:Ool20004Ool20003Ool30004Ool00`00Oomoo`1C
Ool005Eoo`03001oogoo05=oo`00EGoo00<007ooOol0Dgoo001EOol00`00Oomoo`1COol005Eoo`03
001oogoo05=oo`00EGoo00<007ooOol0Dgoo001EOol00`00Oomoo`1COol005Eoo`03001oogoo05=o
o`00EGoo00<007ooOol0Dgoo001EOol00`00Oomoo`1COol005Eoo`03001oogoo05=oo`00=goo0P00
17oo0P0017oo0P0017oo0P000goo10000goo00<007ooOol0Dgoo000fOol01000Oomoo`000goo0P00
0goo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`03Ool00`00Oomoo`1COol0031oo`D0
0005Ool007ooOol00008Ool01000Oomoo`000Woo00@007ooOol000Aoo`03001oogoo009oo`8005Ao
o`00=Woo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`001Goo00D007ooOomoo`0005Eo
o`00=Woo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol000=oo`03
001oogoo05=oo`00=goo0P002Woo0P0017oo0P0017oo0P0017oo00<007ooOol0Dgoo001EOol2001D
Ool005Eoo`8005Aoo`00EGoo0P00E7oo001EOol2001DOol005Eoo`8005Aoo`00EGoo0P00E7oo001E
Ool2001DOol005Eoo`8005Aoo`00EGoo0P00E7oo001EOol2001DOol003Moo`8000Aoo`8000Aoo`80
00Aoo`8000=oo`<000Aoo`8005Aoo`00=Woo00@007ooOol000=oo`8000=oo`04001oogoo0002Ool0
1000Oomoo`000goo00<007ooOol00goo0P00E7oo000`Ool500001Goo001oogoo000027oo00@007oo
Ool0009oo`04001oogoo0003Ool00`00Oomoo`03Ool2001DOol003Ioo`04001oogoo0008Ool01000
Oomoo`000Woo00@007ooOol000=oo`03001oogoo00=oo`8005Aoo`00=Woo00@007ooOol000Qoo`04
001oogoo0002Ool01000Oomoo`000goo00<007ooOol00goo0P00E7oo000gOol2000:Ool20004Ool2
0003Ool20005Ool2001DOol005Eoo`8005Aoo`00EGoo0P00E7oo001EOol00`00Ool0001COol001=o
o`8001Eoo`<0019oo`@0019oo`03001oo`00015oo`@001Aoo`<001=oo`80019oo`004Woo00@007oo
Ool001Eoo`03001oogoo019oo`03001oogoo019oo`03001oo`00019oo`03001oogoo01Eoo`03001o
ogoo015oo`04001oogoo000AOol000aoo`D00005Ool007ooOol0000=Ool500000goo00000002000<
Ool50003Ool00`00Oomoo`0AOol00`00Ool0000COol00`00Oomoo`0BOol4000BOol01000Oomoo`00
4Goo000BOol3000DOol00`00Ool0000FOol00`00Oomoo`0:Ool20004Ool00`00Ool0000DOol00`00
Oomoo`0AOol00`00Ool0000COol3000BOol0019oo`03001oogoo01Aoo`03001oo`0001=oo`04001o
ogoo000<Ool00`00Ool00003Ool00`00Ool0000AOol01000Oomoo`004goo00<007oo00004goo00<0
07ooOol04Woo000COol3000DOol2000DOol2000<Ool01000Oomoo`000goo00<007oo00004Woo0P00
5Goo0P0057oo0`004Goo001=Ool01@00Oomoogoo00000goo00<007oo0000Dgoo001=Ool01@00Oomo
ogoo00000goo00<007oo00001goo1`00AGoo000jOol7000;Ool00`00Oomoo`03Ool01P00Oomoo`00
Ool000Moo`03001oogoo00Aoo`03001oogoo049oo`000WooY`000Woo0006Ool00`00Oomoo`02Ool0
0`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`03Ool0
0`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool0
0`00Oomoo`02Ool00`00Ool00003Ool01000Oomoo`000Woo00H007ooOol007oo0002Ool01000Oomo
o`001Goo00<007oo00002@001Goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<0
07ooOol00goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00goo00<0
07ooOol017oo0015Ool300000goo00000007Ool01000Oomoo`000Woo00<007ooOol00Woo00<007oo
Ool0Bgoo0017Ool30008Ool01000Oomoo`000Woo00D007ooOomoo`0004ioo`00DWoo00@007ooOol0
009oo`05001oogooOol0001>Ool0059oo`04001oogoo0003Ool00`00Ool0001?Ool0059oo`04001o
ogoo0003Ool3001?Ool005=oo`03001oo`0000Aoo`03001oogoo04ioo`00Dgoo00<007oo0000EGoo
001COol00`00Ool0001EOol005=oo`03001oo`0005Eoo`00Dgoo00<007oo0000EGoo001COol00`00
Ool0001EOol005=oo`03001oo`0005Eoo`00=goo0P0017oo0P0017oo0P0017oo0P000goo0`000Woo
00<007oo0000EGoo000fOol01000Oomoo`000goo0P000goo00@007ooOol0009oo`04001oogoo0003
Ool01`00Oomoogoo001oo`0005Eoo`00=Woo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomo
o`000goo00H007ooOomoo`00Ool2001DOol003Ioo`04001oogoo0008Ool01000Oomoo`000Woo00@0
07ooOol000=oo`07001oogooOol007oo0000EGoo000fOol01000Oomoo`0027oo00@007ooOol0009o
o`04001oogoo0003Ool01`00Oomoogoo001oo`0005Eoo`00=goo0P002Woo0P0017oo0P000goo0P00
0goo00<007oo0000EGoo001COol00`00Ool0001EOol005=oo`03001oo`0005Eoo`00Dgoo00<007oo
0000EGoo001COol00`00Ool0001EOol005=oo`03001oo`0005Eoo`00E7oo0P00EGoo001DOol2001E
Ool005Aoo`8005Eoo`00E7oo0P00EGoo001DOol2001EOol003Moo`8000Aoo`8000Aoo`8000Aoo`80
00=oo`@0009oo`8005Eoo`00=Woo00@007ooOol000=oo`8000=oo`04001oogoo0002Ool01000Oomo
o`000goo00<007ooOol00Woo0P00EGoo000fOol01000Oomoo`0027oo00@007ooOol0009oo`04001o
ogoo0004Ool01000Oomoogoo0`00E7oo000fOol01000Oomoo`0027oo00@007ooOol0009oo`04001o
ogoo0005Ool00`00Oomoo`02001EOol003Ioo`04001oogoo0008Ool01000Oomoo`000Woo00@007oo
Ool0009oo`04001oogoo0002Ool2001EOol003Moo`8000Yoo`8000Aoo`8000Aoo`8000=oo`8005Eo
o`00E7oo0P00EGoo001DOol2001EOol005Aoo`8005Eoo`00E7oo0P00EGoo001DOol2001EOol005Ao
o`8005Eoo`00E7oo0P00EGoo001DOol2001EOol005Aoo`8005Eoo`00E7oo0P00EGoo000gOol20004
Ool20004Ool20004Ool20003Ool30004Ool00`00Oomoo`1COol003Ioo`04001oogoo0003Ool20003
Ool01000Oomoo`000Woo00@007ooOol000Eoo`05001oogooOol0001EOol003Ioo`04001oogoo0008
Ool01000Oomoo`000Woo00@007ooOol000Eoo`04001oogooOol2001DOol003Ioo`04001oogoo0008
Ool01000Oomoo`000Woo00@007ooOol000=oo`8000Aoo`03001oogoo05=oo`00=Woo00@007ooOol0
00Qoo`04001oogoo0002Ool01000Oomoo`001Goo00D007ooOomoo`0005Eoo`00=goo0P002Woo0P00
17oo0P000goo0`00FWoo002[Ool00001\
\>"],
  ImageRangeCache->{{{0, 170.438}, {104.938, 0}} -> {-7.5541, -0.00330059, 
  0.0886436, 6.27316*^-5}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({HFuncOneD2[0], \ \(\(Norm'\)'\)[0]}\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\(-0.00327083774166059271`\), 0}\)], "Output",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell["\<\
So this type of rational approximation cannot be used to get \
precise answers for large negative arguments, and if differences are taken, \
the error in \[CapitalGamma] may be significant near the strike and at \
maturity.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Approximation Number Two", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["We define the corresponding functions by", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(HFuncTwoBase[x_]\  := \n\t
      Module[{\[Gamma]\  = \ 0.2316419, \ a = 0.319381530, \ 
          b = \ \(-0.356563782\), \ c = \ 1.781477937, \ 
          d\  = \ \(-1.821255978\), \ e\  = \ 1.330274429, \ 
          k}, \n\t\tk\  = \ 1/\((1\  + \ \[Gamma]\ x)\); \n\t\tu\  = \ 
          Exp[\(-x^2\)/2]/
              Sqrt[2\ Pi]*\((a*k\  + \ b*k^2\  + \ c*k^3\  + \ d\ k^4\  + \ 
                e\ k^5)\)]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(HFuncTwo[x_]\  := \n\t
      If[x >= 0, \ 1\  - \ HFuncTwoBase[x], \ HFuncTwoBase[\(-x\)]]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(HFuncTwoBaseD[x_]\  = D[HFuncTwoBase[x], \ x]; \n
    HFuncTwoBaseD2[x_]\  = D[HFuncTwoBase[x], \ {x, 2}]; \)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(HFuncTwoD[x_]\  := \n\t
      If[x >= 0, \ \(-\ HFuncTwoBaseD[x]\), \ \(-HFuncTwoBaseD[\(-x\)]\)]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(HFuncTwoD2[x_]\  := \n\t
      If[x >= 0, \ \(-\ HFuncTwoBaseD2[x]\), \ HFuncTwoBaseD2[\(-x\)]]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["The function itself looks just fine:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[HFuncTwo[x], \ {x, \ \(-3\), \ 3}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.15873 0.0147151 0.589399 [
[.02381 .00222 -6 -9 ]
[.02381 .00222 6 0 ]
[.18254 .00222 -6 -9 ]
[.18254 .00222 6 0 ]
[.34127 .00222 -6 -9 ]
[.34127 .00222 6 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.81746 .00222 -3 -9 ]
[.81746 .00222 3 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.4875 .13259 -18 -4.5 ]
[.4875 .13259 0 4.5 ]
[.4875 .25047 -18 -4.5 ]
[.4875 .25047 0 4.5 ]
[.4875 .36835 -18 -4.5 ]
[.4875 .36835 0 4.5 ]
[.4875 .48623 -18 -4.5 ]
[.4875 .48623 0 4.5 ]
[.4875 .60411 -6 -4.5 ]
[.4875 .60411 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .01472 m
.02381 .02097 L
s
[(-3)] .02381 .00222 0 1 Mshowa
.18254 .01472 m
.18254 .02097 L
s
[(-2)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(-1)] .34127 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(1)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(2)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.5 .13259 m
.50625 .13259 L
s
[(0.2)] .4875 .13259 1 0 Mshowa
.5 .25047 m
.50625 .25047 L
s
[(0.4)] .4875 .25047 1 0 Mshowa
.5 .36835 m
.50625 .36835 L
s
[(0.6)] .4875 .36835 1 0 Mshowa
.5 .48623 m
.50625 .48623 L
s
[(0.8)] .4875 .48623 1 0 Mshowa
.5 .60411 m
.50625 .60411 L
s
[(1)] .4875 .60411 1 0 Mshowa
.125 Mabswid
.5 .04419 m
.50375 .04419 L
s
.5 .07366 m
.50375 .07366 L
s
.5 .10313 m
.50375 .10313 L
s
.5 .16206 m
.50375 .16206 L
s
.5 .19153 m
.50375 .19153 L
s
.5 .221 m
.50375 .221 L
s
.5 .27994 m
.50375 .27994 L
s
.5 .30941 m
.50375 .30941 L
s
.5 .33888 m
.50375 .33888 L
s
.5 .39782 m
.50375 .39782 L
s
.5 .42729 m
.50375 .42729 L
s
.5 .45676 m
.50375 .45676 L
s
.5 .5157 m
.50375 .5157 L
s
.5 .54517 m
.50375 .54517 L
s
.5 .57464 m
.50375 .57464 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01551 m
.04262 .01588 L
.06244 .01644 L
.08255 .01723 L
.10458 .01847 L
.12415 .01999 L
.14509 .02219 L
.16639 .0252 L
.18653 .02895 L
.20651 .03371 L
.22495 .03921 L
.24527 .0467 L
.2643 .05526 L
.30213 .07736 L
.34091 .1079 L
.38213 .14961 L
.42184 .19814 L
.46003 .25083 L
.50068 .31042 L
.53981 .36777 L
.58139 .4249 L
.62146 .47322 L
.66001 .51175 L
.70101 .54359 L
.72143 .55608 L
.7405 .56588 L
.77847 .58072 L
.79882 .5865 L
.81738 .59069 L
.83646 .59409 L
.85722 .59692 L
.877 .59894 L
.898 .60053 L
.91748 .6016 L
.93576 .60233 L
.95594 .60291 L
.97444 .60329 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{171.438, 105.938},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002[0000JA000`40O003h00OZgoo002[Ool000mo
o`<001Eoo`@001Aoo`<002Yoo`<001Eoo`@001Aoo`<000aoo`004Woo00<007ooOol04goo00<007oo
Ool05Goo00<007ooOol0:Woo00<007ooOol05Goo00<007ooOol05goo00<007ooOol02Goo0009Ool5
0004Ool00`00Oomoo`0<Ool50003Ool00`00Oomoo`0=Ool50002Ool00`00Oomoo`0ZOol00`00Oomo
o`0FOol00`00Oomoo`0FOol00`00Oomoo`09Ool0011oo`8001Qoo`03001oogoo01=oo`03001oogoo
02Yoo`03001oogoo01Moo`03001oogoo01=oo`8000aoo`004Woo00<007ooOol04Woo00@007ooOol0
01Eoo`03001oogoo02Yoo`03001oogoo01Aoo`04001oogoo000GOol00`00Oomoo`09Ool000moo`<0
01Ioo`8001Eoo`8002]oo`8001Moo`8001Eoo`<000aoo`00EWoo00<007ooOol0DWoo001FOol00`00
Oomoo`1BOol000YooiT000Qoo`003Woo4P0000=oo`00Ool00goo00<007ooOol00Woo00D007ooOomo
o`0000Aoo`03001oogoo009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool0
0`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo0000
17oo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001o
ogoo009oo`03001oogoo009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool0
0`00Oomoo`0:Ool0021oo`H0031oo`03001oogoo059oo`009Woo1P00:Woo00<007ooOol0DWoo000/
Ool3000WOol00`00Oomoo`1BOol002moo`8002Eoo`03001oogoo059oo`00<Goo00<007ooOol08Woo
00<007ooOol0DWoo000bOol2000ROol00`00Oomoo`1BOol003Aoo`80021oo`03001oogoo059oo`00
=Woo0P007Woo00<007ooOol0DWoo000hOol2000LOol00`00Oomoo`1BOol003Yoo`03001oogoo01Uo
o`03001oogoo059oo`00>goo00<007ooOol067oo00<007ooOol0DWoo000lOol00`00Oomoo`0GOol0
0`00Oomoo`1BOol003eoo`03001oogoo01Ioo`03001oogoo059oo`00?Woo00<007ooOol05Goo00<0
07ooOol0DWoo000oOol00`00Oomoo`02Ool20004Ool20003Ool40003Ool00`00Oomoo`1BOol0041o
o`04001oogoo0002Ool01000Oomoogoo0P0017oo00<007ooOol00goo00<007ooOol0DWoo0011Ool0
0`00Ool00002Ool00`00Oomoo`08Ool00`00Oomoo`02Ool2001COol0049oo`80009oo`03001oogoo
00Uoo`05001oogooOol0001DOol004=oo`04001oogoo0008Ool01000Oomoo`000goo00<007ooOol0
DWoo0014Ool2000:Ool20004Ool00`00Oomoo`1BOol004Aoo`03001oogoo00moo`03001oogoo059o
o`00AGoo00<007ooOol03Woo00<007ooOol0DWoo0016Ool00`00Oomoo`0=Ool00`00Oomoo`1BOol0
04Moo`03001oogoo00aoo`03001oogoo059oo`00B7oo00<007ooOol02goo00<007ooOol0DWoo0019
Ool00`00Oomoo`0:Ool00`00Oomoo`1BOol004Yoo`03001oogoo00Uoo`03001oogoo059oo`00BWoo
00<007ooOol02Goo00<007ooOol0DWoo001;Ool00`00Oomoo`08Ool00`00Oomoo`1BOol004aoo`03
001oogoo00Moo`03001oogoo059oo`00CGoo00<007ooOol01Woo00<007ooOol0DWoo001=Ool00`00
Oomoo`06Ool00`00Oomoo`1BOol004Aoo`8000Aoo`80009oo`03001oo`00008000=oo`03001oogoo
059oo`00@goo00@007ooOol000=oo`8000=oo`03001oo`0000Aoo`03001oogoo059oo`00@goo00@0
07ooOol000Qoo`@000=oo`8005=oo`00@goo00@007ooOol000Qoo`<000Aoo`03001oogoo059oo`00
@goo00@007ooOol000Qoo`03001oo`0000Aoo`03001oogoo059oo`00A7oo0P002Woo0`000goo00<0
07ooOol0DWoo001BOol01@00Oomoogoo0000E7oo001COol01000Oomoo`00E7oo001DOol00`00Ool0
001DOol005Aoo`03001oo`0005Aoo`00EGoo0P00E7oo001FOol00`00Oomoo`1BOol005Ioo`03001o
ogoo059oo`00EWoo0P00Dgoo001FOol00`00Ool0001BOol005Ioo`03001oo`00059oo`00EWoo00@0
07ooOol0055oo`00EWoo00D007ooOomoo`00051oo`00A7oo0P0017oo0P0017oo0P0017oo00D007oo
Oomoo`00051oo`00@goo00@007ooOol000=oo`8000=oo`04001oogoo0003Ool00`00Oomoo`02Ool0
0`00Oomoo`1=Ool004=oo`04001oogoo0008Ool01000Oomoo`000goo0P0017oo00<007ooOol0C7oo
0013Ool01000Oomoo`0027oo0`0017oo00<007ooOol00goo00<007ooOol0C7oo0013Ool01000Oomo
o`0027oo00<007ooOol017oo00<007ooOol017oo00<007ooOol0Bgoo0014Ool2000:Ool30003Ool0
0`00Oomoo`05Ool00`00Oomoo`1:Ool005Ioo`03001oogoo00Eoo`03001oogoo04Yoo`00EWoo00<0
07ooOol01Woo00<007ooOol0BGoo001FOol00`00Oomoo`07Ool00`00Oomoo`18Ool005Ioo`03001o
ogoo00Moo`03001oogoo04Qoo`00EWoo00<007ooOol027oo00<007ooOol0Agoo001FOol00`00Oomo
o`09Ool00`00Oomoo`16Ool005Ioo`03001oogoo00Uoo`03001oogoo04Ioo`00EWoo00<007ooOol0
2Woo00<007ooOol0AGoo001FOol00`00Oomoo`0;Ool00`00Oomoo`14Ool005Ioo`03001oogoo00ao
o`03001oogoo04=oo`00EWoo00<007ooOol03Goo00<007ooOol0@Woo001FOol00`00Oomoo`0>Ool0
0`00Oomoo`11Ool004Aoo`8000Aoo`8000Aoo`8000Aoo`03001oogoo00moo`03001oogoo041oo`00
@goo00@007ooOol000=oo`8000=oo`04001oogoo0003Ool00`00Oomoo`0@Ool00`00Oomoo`0oOol0
04=oo`04001oogoo0008Ool01000Oomoo`000goo0P004Woo00<007ooOol0?Woo0013Ool01000Oomo
o`002Goo0P0017oo00<007ooOol04Woo00<007ooOol0?Goo0013Ool01000Oomoo`0027oo00@007oo
Ool000=oo`03001oogoo01=oo`03001oogoo03aoo`00A7oo0P002Woo0P0017oo00<007ooOol057oo
00<007ooOol0>goo001FOol00`00Oomoo`0EOol2000kOol005Ioo`03001oogoo01Moo`03001oogoo
03Qoo`00EWoo00<007ooOol067oo00<007ooOol0=goo001FOol00`00Oomoo`0IOol00`00Oomoo`0f
Ool005Ioo`03001oogoo01Yoo`03001oogoo03Eoo`00EWoo00<007ooOol06goo0P00=Goo001FOol0
0`00Oomoo`0MOol2000cOol005Ioo`03001oogoo01moo`80035oo`00EWoo00<007ooOol08Goo0`00
;Woo001FOol00`00Oomoo`0TOol3000[Ool005Ioo`03001oogoo02Moo`<002Qoo`00EWoo00<007oo
Ool0:Woo0`009Goo001?Ool30004Ool00`00Oomoo`0]Ool6000OOol0051oo`03001oogoo00=oo`03
001oogoo03=oo`T001Ioo`00D7oo00<007ooOol00goo0P00?Goo2P0037oo001@Ool00`00Oomoo`03
Ool00`00Oomoo`1BOol0051oo`03001oogoo00=oo`03001oogoo059oo`00Cgoo0P00FWoo002[Ool0
0001\
\>"],
  ImageRangeCache->{{{0, 170.438}, {104.938, 0}} -> {-3.58346, -0.12172, 
  0.0414704, 0.0111683}}]
}, Open  ]],

Cell["\<\
The percentage error is now confined to less than 0.7% at large \
negative arguments:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[100*\((HFuncTwo[x]/Norm[x] - 1)\), \ {x, \ \(-7\), \ 7}, \ 
      PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0680272 0.0150401 0.904658 [
[.09184 .00254 -6 -9 ]
[.09184 .00254 6 0 ]
[.22789 .00254 -6 -9 ]
[.22789 .00254 6 0 ]
[.36395 .00254 -6 -9 ]
[.36395 .00254 6 0 ]
[.63605 .00254 -3 -9 ]
[.63605 .00254 3 0 ]
[.77211 .00254 -3 -9 ]
[.77211 .00254 3 0 ]
[.90816 .00254 -3 -9 ]
[.90816 .00254 3 0 ]
[.4875 .10551 -18 -4.5 ]
[.4875 .10551 0 4.5 ]
[.4875 .19597 -18 -4.5 ]
[.4875 .19597 0 4.5 ]
[.4875 .28644 -18 -4.5 ]
[.4875 .28644 0 4.5 ]
[.4875 .3769 -18 -4.5 ]
[.4875 .3769 0 4.5 ]
[.4875 .46737 -18 -4.5 ]
[.4875 .46737 0 4.5 ]
[.4875 .55783 -18 -4.5 ]
[.4875 .55783 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.09184 .01504 m
.09184 .02129 L
s
[(-6)] .09184 .00254 0 1 Mshowa
.22789 .01504 m
.22789 .02129 L
s
[(-4)] .22789 .00254 0 1 Mshowa
.36395 .01504 m
.36395 .02129 L
s
[(-2)] .36395 .00254 0 1 Mshowa
.63605 .01504 m
.63605 .02129 L
s
[(2)] .63605 .00254 0 1 Mshowa
.77211 .01504 m
.77211 .02129 L
s
[(4)] .77211 .00254 0 1 Mshowa
.90816 .01504 m
.90816 .02129 L
s
[(6)] .90816 .00254 0 1 Mshowa
.125 Mabswid
.12585 .01504 m
.12585 .01879 L
s
.15986 .01504 m
.15986 .01879 L
s
.19388 .01504 m
.19388 .01879 L
s
.2619 .01504 m
.2619 .01879 L
s
.29592 .01504 m
.29592 .01879 L
s
.32993 .01504 m
.32993 .01879 L
s
.39796 .01504 m
.39796 .01879 L
s
.43197 .01504 m
.43197 .01879 L
s
.46599 .01504 m
.46599 .01879 L
s
.53401 .01504 m
.53401 .01879 L
s
.56803 .01504 m
.56803 .01879 L
s
.60204 .01504 m
.60204 .01879 L
s
.67007 .01504 m
.67007 .01879 L
s
.70408 .01504 m
.70408 .01879 L
s
.7381 .01504 m
.7381 .01879 L
s
.80612 .01504 m
.80612 .01879 L
s
.84014 .01504 m
.84014 .01879 L
s
.87415 .01504 m
.87415 .01879 L
s
.05782 .01504 m
.05782 .01879 L
s
.02381 .01504 m
.02381 .01879 L
s
.94218 .01504 m
.94218 .01879 L
s
.97619 .01504 m
.97619 .01879 L
s
.25 Mabswid
0 .01504 m
1 .01504 L
s
.5 .10551 m
.50625 .10551 L
s
[(0.1)] .4875 .10551 1 0 Mshowa
.5 .19597 m
.50625 .19597 L
s
[(0.2)] .4875 .19597 1 0 Mshowa
.5 .28644 m
.50625 .28644 L
s
[(0.3)] .4875 .28644 1 0 Mshowa
.5 .3769 m
.50625 .3769 L
s
[(0.4)] .4875 .3769 1 0 Mshowa
.5 .46737 m
.50625 .46737 L
s
[(0.5)] .4875 .46737 1 0 Mshowa
.5 .55783 m
.50625 .55783 L
s
[(0.6)] .4875 .55783 1 0 Mshowa
.125 Mabswid
.5 .03313 m
.50375 .03313 L
s
.5 .05123 m
.50375 .05123 L
s
.5 .06932 m
.50375 .06932 L
s
.5 .08741 m
.50375 .08741 L
s
.5 .1236 m
.50375 .1236 L
s
.5 .14169 m
.50375 .14169 L
s
.5 .15979 m
.50375 .15979 L
s
.5 .17788 m
.50375 .17788 L
s
.5 .21406 m
.50375 .21406 L
s
.5 .23216 m
.50375 .23216 L
s
.5 .25025 m
.50375 .25025 L
s
.5 .26834 m
.50375 .26834 L
s
.5 .30453 m
.50375 .30453 L
s
.5 .32262 m
.50375 .32262 L
s
.5 .34072 m
.50375 .34072 L
s
.5 .35881 m
.50375 .35881 L
s
.5 .395 m
.50375 .395 L
s
.5 .41309 m
.50375 .41309 L
s
.5 .43118 m
.50375 .43118 L
s
.5 .44928 m
.50375 .44928 L
s
.5 .48546 m
.50375 .48546 L
s
.5 .50356 m
.50375 .50356 L
s
.5 .52165 m
.50375 .52165 L
s
.5 .53974 m
.50375 .53974 L
s
.5 .57593 m
.50375 .57593 L
s
.5 .59402 m
.50375 .59402 L
s
.5 .61211 m
.50375 .61211 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
.5 Mabswid
.02381 .60332 m
.06244 .4431 L
.10458 .29884 L
.14415 .19253 L
.18221 .11667 L
.20178 .08725 L
.22272 .06243 L
.24402 .04362 L
.2538 .03695 L
.26416 .03109 L
.27367 .02671 L
.28414 .02289 L
.29366 .02021 L
.30258 .01831 L
.30754 .01747 L
.31296 .01672 L
.31773 .01618 L
.32282 .01572 L
.32786 .01537 L
.33243 .01513 L
.33464 .01504 L
.33702 .01495 L
.34135 .01484 L
.34372 .01479 L
.34503 .01477 L
.34626 .01476 L
.34741 .01474 L
.34866 .01473 L
.3498 .01473 L
.35085 .01472 L
.35209 .01472 L
.35326 .01472 L
.35454 .01472 L
.35525 .01472 L
.35591 .01472 L
.35716 .01472 L
.35787 .01472 L
.35853 .01473 L
.36129 .01474 L
.36403 .01476 L
.36662 .01479 L
.37147 .01484 L
.38235 .01496 L
.38737 .01501 L
.39207 .01504 L
.39697 .01507 L
.39957 .01508 L
.40102 .01509 L
.40236 .01509 L
.40361 .0151 L
Mistroke
.40473 .0151 L
.40598 .0151 L
.40667 .0151 L
.40731 .0151 L
.40849 .0151 L
.40957 .0151 L
.41076 .0151 L
.41201 .0151 L
.41306 .0151 L
.41418 .0151 L
.41621 .01509 L
.42082 .01508 L
.43102 .01505 L
.43639 .01503 L
.43934 .01502 L
.44078 .01502 L
.4421 .01502 L
.44324 .01502 L
.44449 .01501 L
.44575 .01501 L
.44708 .01501 L
.4478 .01501 L
.44845 .01501 L
.44921 .01501 L
.44992 .01501 L
.45119 .01501 L
.4519 .01501 L
.45256 .01501 L
.4548 .01502 L
.45726 .01502 L
.46223 .01503 L
.46746 .01504 L
.47004 .01505 L
.47242 .01505 L
.47353 .01505 L
.47458 .01505 L
.47567 .01505 L
.47687 .01506 L
.47809 .01506 L
.4792 .01506 L
.48043 .01505 L
.48172 .01505 L
.48302 .01505 L
.48421 .01505 L
.48688 .01504 L
.48939 .01504 L
.4917 .01503 L
.49301 .01503 L
.49367 .01503 L
.49439 .01503 L
Mistroke
.49512 .01503 L
.4959 .01503 L
.49664 .01503 L
.49731 .01503 L
.49862 .01503 L
.49935 .01504 L
.50004 .01504 L
.50129 .01505 L
.50199 .01505 L
.50262 .01505 L
.50383 .01505 L
.50495 .01505 L
.50618 .01505 L
.5075 .01505 L
.51015 .01504 L
.51266 .01504 L
.51497 .01503 L
.51628 .01503 L
.51748 .01503 L
.5185 .01503 L
.51962 .01503 L
.52079 .01503 L
.52188 .01503 L
.5231 .01503 L
.52379 .01503 L
.52443 .01503 L
.52712 .01503 L
.53191 .01504 L
.53709 .01504 L
.54002 .01505 L
.54141 .01505 L
.54271 .01505 L
.54388 .01505 L
.54515 .01505 L
.54587 .01505 L
.54655 .01505 L
.54783 .01505 L
.54848 .01505 L
.54917 .01505 L
.55042 .01505 L
.55118 .01505 L
.5519 .01505 L
.55324 .01505 L
.55573 .01505 L
.55802 .01505 L
.56318 .01504 L
.57216 .01504 L
.57475 .01504 L
.5772 .01504 L
.57939 .01503 L
Mistroke
.58063 .01503 L
.58177 .01503 L
.58294 .01503 L
.58403 .01503 L
.58523 .01503 L
.58652 .01503 L
.58716 .01503 L
.58786 .01503 L
.58914 .01503 L
.59039 .01503 L
.59155 .01503 L
.59386 .01503 L
.59595 .01504 L
.60065 .01504 L
.61142 .01504 L
.61615 .01504 L
.61865 .01504 L
.62133 .01504 L
.62386 .01505 L
.62617 .01505 L
.62728 .01505 L
.62849 .01505 L
.62962 .01505 L
.63069 .01505 L
.63187 .01505 L
.63318 .01505 L
.63445 .01505 L
.63516 .01505 L
.63582 .01505 L
.63693 .01505 L
.63813 .01505 L
.6394 .01505 L
.64057 .01505 L
.64328 .01505 L
.64584 .01505 L
.65063 .01504 L
.66139 .01504 L
.6711 .01504 L
.67642 .01504 L
.68139 .01504 L
.68598 .01504 L
.69018 .01503 L
.69241 .01503 L
.69484 .01503 L
.69617 .01503 L
.6974 .01503 L
.69857 .01503 L
.69984 .01503 L
.70113 .01503 L
.70184 .01503 L
Mistroke
.70249 .01503 L
.70367 .01503 L
.70493 .01503 L
.70564 .01503 L
.70641 .01503 L
.70714 .01503 L
.70781 .01503 L
.70908 .01503 L
.71043 .01503 L
.71288 .01503 L
.71517 .01503 L
.7204 .01503 L
.73901 .01504 L
.75752 .01504 L
.768 .01504 L
.77774 .01504 L
.78745 .01504 L
.79288 .01504 L
.79781 .01504 L
.80302 .01504 L
.80867 .01504 L
.81401 .01504 L
.81892 .01504 L
.8236 .01504 L
.82863 .01504 L
.83338 .01504 L
.83771 .01504 L
.84278 .01504 L
.84831 .01504 L
.85353 .01504 L
.85836 .01504 L
.86298 .01504 L
.86797 .01504 L
.87267 .01504 L
.87699 .01504 L
.88178 .01504 L
.88702 .01504 L
.88966 .01504 L
.89249 .01504 L
.89758 .01504 L
.90016 .01504 L
.9029 .01504 L
.9078 .01504 L
.91053 .01504 L
.91308 .01504 L
.91606 .01504 L
.91889 .01504 L
.92132 .01504 L
.92396 .01504 L
.9286 .01504 L
Mistroke
.93122 .01504 L
.93401 .01504 L
.93646 .01504 L
.93904 .01504 L
.94132 .01504 L
.94382 .01504 L
.94823 .01504 L
.95071 .01504 L
.95337 .01504 L
.95817 .01504 L
.96049 .01504 L
.96301 .01504 L
.96523 .01504 L
.9676 .01504 L
.96972 .01504 L
.97206 .01504 L
.97419 .01504 L
.97619 .01504 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{171.438, 105.938},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002[0000JA000`40O003h00OZgoo002[Ool001Mo
o`8001=oo`<0019oo`@002=oo`@0019oo`<0019oo`8001Ioo`005Woo00@007ooOol001=oo`03001o
ogoo019oo`03001oogoo02Aoo`03001oogoo01=oo`03001oogoo011oo`04001oogoo000EOol0011o
o`D00005Ool007ooOol0000;Ool500000goo00000002000<Ool50003Ool00`00Oomoo`0TOol00`00
Oomoo`0@Ool4000AOol01000Oomoo`005Goo000FOol3000BOol00`00Ool0000FOol00`00Oomoo`0T
Ool00`00Oomoo`0?Ool00`00Ool0000BOol3000FOol001Ioo`03001oogoo019oo`03001oo`0001=o
o`04001oogoo000SOol01000Oomoo`004Goo00<007oo00004Woo00<007ooOol05Woo000GOol3000B
Ool2000DOol2000UOol2000COol2000COol3000EOol005Eoo`03001oogoo05=oo`00EGoo00<007oo
Ool0Dgoo000kOol8000BOol00`00Oomoo`1COol000Mooid000Moo`00<Goo0`008Goo00<007ooOol0
Dgoo000_Ool2000TOol00`00Oomoo`1COol002ioo`03001oogoo02Aoo`03001oogoo05=oo`00;Goo
00<007ooOol09Goo00<007ooOol0Dgoo000/Ool00`00Oomoo`0VOol00`00Oomoo`1COol002]oo`03
001oogoo02Moo`03001oogoo05=oo`00:Woo00<007ooOol0:7oo00<007ooOol0Dgoo000YOol00`00
Oomoo`0YOol00`00Oomoo`1COol002Qoo`03001oogoo02Yoo`03001oogoo05=oo`009goo00<007oo
Ool0:goo00<007ooOol0Dgoo000VOol00`00Oomoo`0/Ool00`00Oomoo`1COol002Ioo`03001oogoo
01Yoo`8000Aoo`8000=oo`<000Aoo`03001oogoo05=oo`009Goo00<007ooOol06Woo00@007ooOol0
00=oo`8000Aoo`03001oogoo00=oo`03001oogoo05=oo`0097oo00<007ooOol06goo00@007ooOol0
00Uoo`03001oogoo00=oo`8005Aoo`008goo00<007ooOol077oo00@007ooOol000Uoo`03001oogoo
00=oo`03001oogoo05=oo`008goo00<007ooOol077oo00@007ooOol000Uoo`03001oogoo00=oo`03
001oogoo05=oo`008Woo00<007ooOol07Woo0P002Goo0P001Goo00<007ooOol0Dgoo000ROol00`00
Oomoo`0`Ool00`00Oomoo`1COol0025oo`03001oogoo035oo`03001oogoo05=oo`008Goo00<007oo
Ool0<Goo00<007ooOol0Dgoo000POol00`00Oomoo`0bOol00`00Oomoo`1COol0021oo`03001oogoo
039oo`03001oogoo05=oo`007goo00<007ooOol0<goo00<007ooOol0Dgoo000OOol00`00Oomoo`0c
Ool00`00Oomoo`1COol001ioo`03001oogoo03Aoo`03001oogoo05=oo`007Woo00<007ooOol08Woo
0P0017oo0P000goo10000goo00<007ooOol0Dgoo000MOol00`00Oomoo`0ROol01000Oomoo`000goo
0P0017oo00<007ooOol00goo00<007ooOol0Dgoo000MOol00`00Oomoo`0ROol01000Oomoo`002Woo
00<007ooOol00Woo0P00E7oo000LOol00`00Oomoo`0SOol01000Oomoo`002goo00D007ooOomoo`00
05Eoo`0077oo00<007ooOol08goo00@007ooOol000Qoo`04001oogoo0003Ool00`00Oomoo`1COol0
01aoo`03001oogoo02Aoo`8000Yoo`8000Aoo`03001oogoo05=oo`006goo00<007ooOol0=goo00<0
07ooOol0Dgoo000KOol00`00Oomoo`0gOol00`00Oomoo`1COol001]oo`03001oogoo03Moo`03001o
ogoo05=oo`006Woo00<007ooOol0>7oo00<007ooOol0Dgoo000JOol00`00Oomoo`0hOol00`00Oomo
o`1COol001Yoo`03001oogoo03Qoo`03001oogoo05=oo`006Goo00<007ooOol0>Goo00<007ooOol0
Dgoo000IOol00`00Oomoo`0iOol00`00Oomoo`1COol001Uoo`03001oogoo02Moo`8000Aoo`8000=o
o`<000Aoo`03001oogoo05=oo`0067oo00<007ooOol09goo00@007ooOol000=oo`8000Ioo`05001o
ogooOol0001EOol001Qoo`03001oogoo02Moo`04001oogoo000;Ool01000Oomoogoo0P00E7oo000H
Ool00`00Oomoo`0WOol01000Oomoo`002Goo0P0017oo00<007ooOol0Dgoo000GOol00`00Oomoo`0X
Ool01000Oomoo`002goo00D007ooOomoo`0005Eoo`005goo00<007ooOol0:Goo0P002Goo0`0017oo
00<007ooOol0Dgoo000GOol00`00Oomoo`0kOol00`00Oomoo`1COol001Ioo`03001oogoo03aoo`03
001oogoo05=oo`005Woo00<007ooOol0?7oo00<007ooOol0Dgoo000FOol00`00Oomoo`0lOol00`00
Oomoo`1COol001Eoo`03001oogoo03eoo`03001oogoo05=oo`005Goo00<007ooOol0?Goo00<007oo
Ool0Dgoo000EOol00`00Oomoo`0mOol00`00Oomoo`1COol001Eoo`03001oogoo03eoo`03001oogoo
05=oo`0057oo00<007ooOol0;7oo0P0017oo0P0017oo0`000goo00<007ooOol0Dgoo000DOol00`00
Oomoo`0[Ool01000Oomoo`000goo0P001Goo00<007ooOol00Woo00<007ooOol0Dgoo000DOol00`00
Oomoo`0[Ool01000Oomoo`0027oo10000goo0P00E7oo000DOol00`00Oomoo`0[Ool01000Oomoo`00
27oo00<007oo000017oo00<007ooOol0Dgoo000COol00`00Oomoo`0/Ool01000Oomoo`0027oo00<0
07oo000017oo00<007ooOol0Dgoo000COol00`00Oomoo`0]Ool2000:Ool20004Ool00`00Oomoo`1C
Ool001=oo`03001oogoo03moo`03001oogoo05=oo`004Woo00<007ooOol0@7oo00<007ooOol0Dgoo
000BOol00`00Oomoo`10Ool00`00Oomoo`1COol0019oo`03001oogoo041oo`03001oogoo05=oo`00
4Woo00<007ooOol0@7oo00<007ooOol0Dgoo000AOol00`00Oomoo`11Ool00`00Oomoo`1COol0015o
o`03001oogoo045oo`03001oogoo05=oo`004Goo00<007ooOol0@Goo00<007ooOol0Dgoo000AOol0
0`00Oomoo`0_Ool20004Ool20003Ool30004Ool00`00Oomoo`1COol0011oo`03001oogoo02moo`04
001oogoo0003Ool20006Ool01@00Oomoogoo0000EGoo000@Ool00`00Oomoo`0_Ool01000Oomoo`00
2goo00@007ooOomoo`8005Aoo`0047oo00<007ooOol0;goo00@007ooOol000Qoo`<000Aoo`03001o
ogoo05=oo`003goo00<007ooOol0<7oo00@007ooOol000Qoo`03001oogoo00Aoo`03001oogoo05=o
o`003goo00<007ooOol0<Goo0P002Goo10000goo00<007ooOol0Dgoo000?Ool00`00Oomoo`13Ool0
0`00Oomoo`1COol000moo`03001oogoo04=oo`03001oogoo05=oo`003Woo00<007ooOol0A7oo00<0
07ooOol0Dgoo000>Ool00`00Oomoo`14Ool00`00Oomoo`1COol000ioo`03001oogoo04Aoo`03001o
ogoo05=oo`003Goo00<007ooOol0AGoo00<007ooOol0Dgoo000=Ool00`00Oomoo`15Ool00`00Oomo
o`1COol000eoo`03001oogoo04Eoo`03001oogoo05=oo`003Goo00<007ooOol0<goo0P0017oo0P00
17oo0P0017oo00<007ooOol0Dgoo000<Ool00`00Oomoo`0cOol01000Oomoo`000goo0P000goo00@0
07ooOol000=oo`03001oogoo05=oo`0037oo00<007ooOol0<goo00@007ooOol000Qoo`04001oogoo
0003Ool2001DOol000aoo`03001oogoo03=oo`04001oogoo0008Ool30004Ool00`00Oomoo`1COol0
00aoo`03001oogoo03=oo`04001oogoo0008Ool00`00Oomoo`04Ool00`00Oomoo`1COol000]oo`03
001oogoo03Eoo`8000Yoo`<000=oo`03001oogoo05=oo`002goo00<007ooOol0Agoo00<007ooOol0
Dgoo000;Ool00`00Oomoo`17Ool00`00Oomoo`1COol000Yoo`03001oogoo04Qoo`03001oogoo05=o
o`002Woo00<007ooOol0B7oo00<007ooOol0Dgoo001EOol00`00Oomoo`1COol005Eoo`03001oogoo
05=oo`00\
\>"],
  ImageRangeCache->{{{0, 170.438}, {104.938, 0}} -> {-8.03692, -0.0776465, 
  0.0943093, 0.00709174}}]
}, Open  ]],

Cell["\<\
If we look at the difference error, we see that the discrepancy is \
less than about 0.0000001 everywhere:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[HFuncTwo[x] - Norm[x], \ {x, \ \(-7\), \ 7}, \ 
      PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0680272 0.309062 3.95013e+06 [
[.09184 .29656 -6 -9 ]
[.09184 .29656 6 0 ]
[.22789 .29656 -6 -9 ]
[.22789 .29656 6 0 ]
[.36395 .29656 -6 -9 ]
[.36395 .29656 6 0 ]
[.63605 .29656 -3 -9 ]
[.63605 .29656 3 0 ]
[.77211 .29656 -3 -9 ]
[.77211 .29656 3 0 ]
[.90816 .29656 -3 -9 ]
[.90816 .29656 3 0 ]
[.4875 .07205 -39 -6.25 ]
[.4875 .07205 0 6.25 ]
[.4875 .15106 -39 -6.25 ]
[.4875 .15106 0 6.25 ]
[.4875 .23006 -39 -6.25 ]
[.4875 .23006 0 6.25 ]
[.4875 .38806 -33 -6.25 ]
[.4875 .38806 0 6.25 ]
[.4875 .46707 -33 -6.25 ]
[.4875 .46707 0 6.25 ]
[.4875 .54607 -33 -6.25 ]
[.4875 .54607 0 6.25 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.09184 .30906 m
.09184 .31531 L
s
[(-6)] .09184 .29656 0 1 Mshowa
.22789 .30906 m
.22789 .31531 L
s
[(-4)] .22789 .29656 0 1 Mshowa
.36395 .30906 m
.36395 .31531 L
s
[(-2)] .36395 .29656 0 1 Mshowa
.63605 .30906 m
.63605 .31531 L
s
[(2)] .63605 .29656 0 1 Mshowa
.77211 .30906 m
.77211 .31531 L
s
[(4)] .77211 .29656 0 1 Mshowa
.90816 .30906 m
.90816 .31531 L
s
[(6)] .90816 .29656 0 1 Mshowa
.125 Mabswid
.12585 .30906 m
.12585 .31281 L
s
.15986 .30906 m
.15986 .31281 L
s
.19388 .30906 m
.19388 .31281 L
s
.2619 .30906 m
.2619 .31281 L
s
.29592 .30906 m
.29592 .31281 L
s
.32993 .30906 m
.32993 .31281 L
s
.39796 .30906 m
.39796 .31281 L
s
.43197 .30906 m
.43197 .31281 L
s
.46599 .30906 m
.46599 .31281 L
s
.53401 .30906 m
.53401 .31281 L
s
.56803 .30906 m
.56803 .31281 L
s
.60204 .30906 m
.60204 .31281 L
s
.67007 .30906 m
.67007 .31281 L
s
.70408 .30906 m
.70408 .31281 L
s
.7381 .30906 m
.7381 .31281 L
s
.80612 .30906 m
.80612 .31281 L
s
.84014 .30906 m
.84014 .31281 L
s
.87415 .30906 m
.87415 .31281 L
s
.05782 .30906 m
.05782 .31281 L
s
.02381 .30906 m
.02381 .31281 L
s
.94218 .30906 m
.94218 .31281 L
s
.97619 .30906 m
.97619 .31281 L
s
.25 Mabswid
0 .30906 m
1 .30906 L
s
.5 .07205 m
.50625 .07205 L
s
gsave
.4875 .07205 -39 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(-6) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-8) show
grestore
.5 .15106 m
.50625 .15106 L
s
gsave
.4875 .15106 -39 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(-4) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-8) show
grestore
.5 .23006 m
.50625 .23006 L
s
gsave
.4875 .23006 -39 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(-2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-8) show
grestore
.5 .38806 m
.50625 .38806 L
s
gsave
.4875 .38806 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-8) show
grestore
.5 .46707 m
.50625 .46707 L
s
gsave
.4875 .46707 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(4) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-8) show
grestore
.5 .54607 m
.50625 .54607 L
s
gsave
.4875 .54607 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(6) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-8) show
grestore
.125 Mabswid
.5 .0918 m
.50375 .0918 L
s
.5 .11156 m
.50375 .11156 L
s
.5 .13131 m
.50375 .13131 L
s
.5 .17081 m
.50375 .17081 L
s
.5 .19056 m
.50375 .19056 L
s
.5 .21031 m
.50375 .21031 L
s
.5 .24981 m
.50375 .24981 L
s
.5 .26956 m
.50375 .26956 L
s
.5 .28931 m
.50375 .28931 L
s
.5 .32881 m
.50375 .32881 L
s
.5 .34856 m
.50375 .34856 L
s
.5 .36831 m
.50375 .36831 L
s
.5 .40782 m
.50375 .40782 L
s
.5 .42757 m
.50375 .42757 L
s
.5 .44732 m
.50375 .44732 L
s
.5 .48682 m
.50375 .48682 L
s
.5 .50657 m
.50375 .50657 L
s
.5 .52632 m
.50375 .52632 L
s
.5 .0523 m
.50375 .0523 L
s
.5 .03255 m
.50375 .03255 L
s
.5 .0128 m
.50375 .0128 L
s
.5 .56582 m
.50375 .56582 L
s
.5 .58557 m
.50375 .58557 L
s
.5 .60532 m
.50375 .60532 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
.5 Mabswid
.02381 .30906 m
.02605 .30906 L
.02846 .30906 L
.03279 .30906 L
.03543 .30906 L
.03793 .30906 L
.04262 .30906 L
.04502 .30906 L
.04756 .30906 L
.05212 .30906 L
.05466 .30906 L
.05704 .30906 L
.06244 .30906 L
.06504 .30906 L
.06788 .30906 L
.07038 .30906 L
.07305 .30906 L
.07568 .30907 L
.07807 .30907 L
.08274 .30907 L
.08556 .30907 L
.08814 .30907 L
.09093 .30907 L
.09399 .30908 L
.09914 .30909 L
.10195 .30909 L
.10458 .3091 L
.10959 .30912 L
.11241 .30913 L
.11503 .30915 L
.11983 .30918 L
.12493 .30924 L
.12948 .3093 L
.13375 .30938 L
.13844 .30951 L
.14349 .30969 L
.14845 .30993 L
.15306 .31023 L
.15812 .31067 L
.16353 .3113 L
.16922 .31222 L
.17458 .31338 L
.1794 .31474 L
.18463 .31664 L
.18952 .31891 L
.19409 .32156 L
.19953 .32552 L
.2045 .33004 L
.21465 .34258 L
.22024 .35178 L
Mistroke
.22555 .36227 L
.23516 .38611 L
.24539 .41866 L
.26379 .49203 L
.2741 .5344 L
.27952 .55384 L
.28248 .56292 L
.28524 .57013 L
.28787 .57565 L
.29026 .57938 L
.29092 .58016 L
.29163 .58089 L
.29292 .58187 L
.29364 .58222 L
.2944 .58244 L
.29505 .5825 L
.29576 .58241 L
.29702 .58189 L
.29816 .58099 L
.29942 .57951 L
.30075 .57738 L
.30298 .5724 L
.30543 .56491 L
.30797 .55472 L
.31073 .54087 L
.31573 .50816 L
.32508 .42199 L
.33507 .30256 L
.34604 .16445 L
.35101 .11032 L
.35374 .08507 L
.35633 .06497 L
.35874 .05016 L
.36001 .044 L
.36136 .03877 L
.3625 .03549 L
.36375 .03308 L
.36489 .03202 L
.36595 .03203 L
.36718 .03327 L
.36788 .03456 L
.36853 .03616 L
.36984 .04053 L
.37124 .04691 L
.37377 .06295 L
.37608 .08245 L
.3817 .14796 L
.38701 .22891 L
.39672 .39681 L
.40203 .4795 L
Mistroke
.40462 .51262 L
.407 .53709 L
.40804 .54578 L
.40917 .55374 L
.41024 .55974 L
.41123 .56392 L
.41239 .56706 L
.41343 .56822 L
.41464 .56755 L
.41578 .56489 L
.41696 .56005 L
.4182 .55263 L
.41926 .54443 L
.42042 .53346 L
.42302 .50182 L
.42544 .46398 L
.43591 .24032 L
.44105 .12777 L
.44332 .08628 L
.44452 .0673 L
.44581 .04967 L
.44697 .03663 L
.44804 .02717 L
.44905 .02052 L
.45013 .01619 L
.45126 .01472 L
.45231 .01632 L
.45347 .02147 L
.4547 .03093 L
.45596 .0447 L
.45731 .06429 L
.45977 .11162 L
.46437 .23372 L
.46894 .37626 L
.47146 .45045 L
.47385 .50917 L
.47519 .53459 L
.4759 .54523 L
.47666 .5542 L
.47736 .56016 L
.47799 .56349 L
.47923 .56399 L
.48048 .5559 L
.48118 .54739 L
.48182 .53699 L
.48301 .51135 L
.48426 .47535 L
.48678 .37798 L
.48946 .24784 L
.49179 .13386 L
Mistroke
.49301 .08303 L
.49431 .04412 L
.49504 .03203 L
.49585 .02953 L
.49658 .03926 L
.49727 .06077 L
.49797 .09735 L
.49862 .14571 L
.49936 .22185 L
.50007 .32122 L
.50137 .47114 L
.50258 .55058 L
.50327 .57528 L
.5039 .58664 L
.50463 .58864 L
.50531 .58133 L
.50647 .55289 L
.50772 .50618 L
.50996 .39938 L
.51241 .27774 L
.51507 .16555 L
.51651 .11954 L
.51714 .10304 L
.51784 .08757 L
.51911 .06687 L
.51981 .05959 L
.52046 .05531 L
.5217 .05374 L
.52283 .05935 L
.52407 .07257 L
.52538 .09358 L
.52759 .1425 L
.52995 .2081 L
.5342 .34061 L
.53889 .47486 L
.54125 .52793 L
.54261 .55253 L
.54385 .57092 L
.54518 .58606 L
.54591 .59239 L
.5466 .59707 L
.54783 .6022 L
.54853 .60332 L
.54918 .60323 L
.55042 .60007 L
.55155 .59392 L
.55286 .58316 L
.55409 .5697 L
.55687 .52877 L
.55942 .48085 L
Mistroke
.56519 .3524 L
.57026 .23755 L
.57499 .14682 L
.57737 .1109 L
.57992 .08117 L
.58138 .06855 L
.58209 .06352 L
.58276 .05958 L
.58399 .05401 L
.58464 .05202 L
.58534 .05061 L
.58648 .04988 L
.58772 .05128 L
.58903 .05515 L
.59024 .06086 L
.59137 .06801 L
.59241 .07591 L
.59473 .0982 L
.59993 .16715 L
.60484 .2479 L
.61355 .39843 L
.61817 .46839 L
.62313 .52795 L
.62576 .5516 L
.62816 .56793 L
.62932 .57395 L
.63058 .57914 L
.63177 .58273 L
.63287 .58493 L
.63409 .58612 L
.63524 .58605 L
.6365 .58467 L
.63721 .58332 L
.63786 .58171 L
.63909 .57774 L
.64045 .57204 L
.64317 .55652 L
.64779 .51939 L
.65273 .46791 L
.66163 .35797 L
.67185 .23058 L
.67724 .17202 L
.68294 .12028 L
.68793 .08508 L
.69057 .07034 L
.69342 .05753 L
.69602 .04849 L
.69841 .04241 L
.69956 .0402 L
.70081 .03831 L
Mistroke
.70199 .03699 L
.70309 .03617 L
.70437 .03568 L
.70502 .03563 L
.70573 .03571 L
.70699 .0362 L
.70816 .03705 L
.70888 .03774 L
.70955 .03851 L
.71082 .04026 L
.71366 .04551 L
.71843 .05786 L
.72361 .07517 L
.74222 .15143 L
.76161 .22254 L
.77265 .25187 L
.7777 .26248 L
.78301 .27192 L
.78772 .27893 L
.79287 .28531 L
.80191 .29375 L
.80702 .29726 L
.81247 .3002 L
.81761 .30236 L
.82229 .3039 L
.82714 .30515 L
.83227 .30617 L
.83666 .30684 L
.84148 .30741 L
.84666 .30787 L
.85222 .30823 L
.85748 .30848 L
.86228 .30864 L
.86706 .30876 L
.87215 .30885 L
.87696 .30891 L
.8813 .30895 L
.88644 .30899 L
.88933 .309 L
.89199 .30901 L
.89688 .30903 L
.89935 .30903 L
.90204 .30904 L
.90457 .30904 L
.9073 .30905 L
.90972 .30905 L
.91228 .30905 L
.91458 .30905 L
.91712 .30906 L
.9216 .30906 L
Mistroke
.92431 .30906 L
.9268 .30906 L
.92948 .30906 L
.9324 .30906 L
.93509 .30906 L
.93759 .30906 L
.94016 .30906 L
.94253 .30906 L
.94689 .30906 L
.94919 .30906 L
.95161 .30906 L
.9557 .30906 L
.95801 .30906 L
.96018 .30906 L
.96422 .30906 L
.9679 .30906 L
.96994 .30906 L
.97216 .30906 L
.97619 .30906 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{163.438, 101},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002S0000IA000`40O003h00ODGoo00<007ooOol0
Cgoo001AOol00`00Oomoo`1?Ool0055oo`03001oogoo04moo`00BGoo00<007ooOol01Goo00<007oo
Ool0Cgoo0018Ool20007Ool00`00Oomoo`1?Ool004Qoo`8000Ioo`80055oo`00>Woo0P0037oo0P00
1Woo0P007goo0P00<7oo000jOol00`00Ool0000;Ool20006Ool2000OOol00`00Ool0000_Ool003Yo
o`03001oo`0000]oo`03001oo`0000Eoo`8001ioo`04001oogoo000_Ool0031oo`8000=oo`04001o
ogooOol2000017oo0000000017oo0P000goo00<007oo00001Goo0P000Woo00<007ooOol01goo0P00
47oo00@007ooOol002moo`00;goo00@007ooOol000=oo`06001oo`00Ool007oo0P0017oo00@007oo
Ool0009oo`03001oo`0000Eoo`80009oo`03001oogoo00Moo`80011oo`05001oogooOol0000^Ool0
02Uoo`D00005Ool007ooOol00004Ool00`00Ool00002Ool20004Ool01000Oomoo`000Woo00<007oo
00001Goo0P000Woo00<007ooOol01goo0P0047oo00D007ooOomoo`0002ioo`00;goo0`0017oo00@0
07oo0000009oo`8000Aoo`04001oogoo0002Ool00`00Ool00005Ool300000goo00000008Ool00`00
Ool0000?Ool00`00Oomoo`02Ool00`00Oomoo`0[Ool002moo`03001oogoo00=oo`05001oogooOol0
0003Ool00`00Oomoo`02Ool01000Oomoo`000Woo00<007oo00001Goo0P000Woo0P0027oo00<007oo
00003goo00<007ooOol00Woo00<007ooOol0:goo000`Ool30006Ool00`00Oomoo`020005Ool20003
Ool00`00Ool00002Ool50002Ool20008Ool00`00Ool0000>Ool00`00Oomoo`03Ool00`00Oomoo`0[
Ool003Uoo`05001oogooOol0000:Ool400000goo001oo`030002Ool20008Ool00`00Ool0000>Ool0
0`00Oomoo`03Ool00`00Oomoo`0[Ool003Uoo`05001oogooOol0000:Ool00`00Ool00003Ool01000
Ool000000Woo0P0027oo00<007oo00003Woo00<007ooOol017oo00<007ooOol0:Woo000iOol01@00
Oomoogoo00002Woo00<007oo00000Woo00<007oo00000P0000Aoo`00Ool000Moo`04001oogoo000>
Ool00`00Oomoo`04Ool00`00Oomoo`0ZOol003Uoo`05001oogooOol00009Ool01000Oomoo`000Woo
1@0000Aoo`00Ool000Moo`04001oogoo000=Ool00`00Oomoo`05Ool00`00Oomoo`0ZOol003Uoo`05
001oogooOol00009Ool01000Oomoo`0017oo0`0000Aoo`00Ool000Moo`04001oogoo000=Ool00`00
Oomoo`05Ool00`00Oomoo`0ZOol003Uoo`03001oogoo009oo`03001oogoo00Ioo`04001oogoo0004
Ool3000017oo001oo`001goo00@007ooOol000eoo`03001oogoo00Ioo`03001oogoo02Uoo`00>Goo
00<007ooOol00Woo00<007ooOol01Woo00@007ooOol000Aoo`<00004Ool007oo0007Ool01000Oomo
o`003Goo00<007ooOol01Woo00<007ooOol0:Goo000`Ool30002Ool01@00Oomoogoo00000Woo0`00
17oo0P000Woo00@007ooOol000Aoo`<00004Ool007oo0007Ool01000Oomoo`003Goo00<007ooOol0
1Woo00<007ooOol0:Goo000aOol00`00Oomoo`02Ool00`00Ool00004Ool20003Ool01P00Oomoo`00
Ool0009oo`03001oogoo009oo`<00004Ool007oo0007Ool01@00Oomoogoo000037oo00<007ooOol0
1Woo00<007ooOol0:Goo000YOol500000goo000000020004Ool20004Ool20003Ool01P00Oomoo`00
Ool0009oo`03001oogoo009oo`<00004Ool007oo0007Ool01@00Oomoogoo000037oo00<007ooOol0
1goo00<007ooOol0:7oo000_Ool00`00Ool00004Ool00`00Ool00004Ool20003Ool01P00Oomoo`00
Ool0009oo`03001oogoo009oo`D00003Ool007oo00Ioo`05001oogooOol0000;Ool00`00Oomoo`08
Ool00`00Oomoo`0XOol002moo`03001oo`0000=oo`03001oogoo008000=oo`8000=oo`06001oogoo
001oo`000Woo00<007ooOol00Woo0`0000Aoo`00Ool000Moo`05001oogooOol0000;Ool00`00Oomo
o`08Ool00`00Oomoo`0XOol0031oo`8000Ioo`04001oogooOol30004Ool20002Ool01000Oomoo`00
0Woo1@0000Aoo`00Ool000Moo`05001oogooOol0000;Ool00`00Oomoo`08Ool00`00Oomoo`0XOol0
03Qoo`03001oogoo00=oo`03001oogoo00Ioo`D00003Ool007oo00<00004Ool007oo0007Ool01@00
Oomoogoo00002goo00<007ooOol02Goo00<007ooOol09goo000hOol00`00Oomoo`03Ool00`00Oomo
o`06Ool01@00Oomoogoo00000Woo100000Aoo`00Ool000Moo`05001oogooOol0000;Ool00`00Oomo
o`09Ool00`00Oomoo`0WOol003Qoo`03001oogoo00=oo`03001oogoo00Ioo`08001oogooOol007oo
001oo`<00005Ool007ooOol00006Ool01@00Oomoogoo00002goo00<007ooOol02Goo00<007ooOol0
9goo000hOol00`00Oomoo`03Ool00`00Oomoo`05Ool00`00Oomoo`02Ool00`00Ool00002000017oo
001oo`000Woo00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Uoo`03001oogoo02Mo
o`00>7oo00<007ooOol00goo00<007ooOol01Goo00<007ooOol00Woo00T007ooOomoo`00Ool007oo
00000Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol02Goo00<007ooOol02Woo00<007oo
Ool09Woo000gOol00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`02Ool02@00Oomoogoo001o
o`00Ool00002Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`09Ool00`00Oomoo`0:Ool0
0`00Oomoo`0VOol002moo`@0009oo`05001oo`00Ool00002Ool30004Ool200000goo001oo`03Ool0
2@00Oomoogoo001oo`00Ool00002Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`09Ool0
0`00Oomoo`0:Ool00`00Oomoo`0VOol0031oo`03001oogoo00=oo`<000Aoo`8000=oo`03001oogoo
008000Aoo`09001oogooOol007oo001oo`00009oo`03001oogoo00=oo`03001oogoo00=oo`03001o
ogoo00Qoo`03001oogoo00Yoo`03001oogoo02Ioo`00:Goo1@000goo00<007ooOol00goo00<007oo
Ool00goo0P000goo00<007ooOol00P0017oo00T007ooOomoo`00Ool007oo00000Woo00<007ooOol0
0goo00<007ooOol00goo00<007ooOol027oo00<007ooOol02goo00<007ooOol09Goo000bOol01000
Oomoogoo0`0017oo0P000goo00<007ooOol00P0017oo00T007ooOomoo`00Ool007oo00000Woo00<0
07ooOol00goo00<007ooOol00goo00<007ooOol027oo00<007ooOol02goo00<007ooOol09Goo000_
Ool01000Oomoo`000Woo00D007oo001oo`0000=oo`8000=oo`03001oogoo008000Aoo`06001oogoo
Ool007oo0`000Woo00<007ooOol00goo00<007ooOol00goo00<007ooOol027oo00<007ooOol02goo
00<007ooOol09Goo000`Ool20005Ool00`00Oomoo`02Ool30004Ool200000goo001oo`03Ool02@00
Oomoogoo001oo`00Ool00002Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`08Ool00`00
Oomoo`0<Ool00`00Oomoo`0TOol003Moo`03001oogoo00Aoo`03001oogoo00Eoo`03001oogoo009o
o`03001oo`0000800004Ool007oo0002Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`08
Ool00`00Oomoo`0<Ool00`00Oomoo`0TOol0015oo`8001Aoo`<000eoo`03001oogoo009oo`@000Io
o`03001oo`0000<00008Ool007oo001oo`00Ool0009oo`03001oogoo00=oo`03001oogoo00=oo`03
001oogoo00@000Aoo`03001oogoo00aoo`<001=oo`8000moo`0047oo00@007ooOol001Aoo`03001o
ogoo00aoo`03001oogoo00=oo`8000Moo`03001oogoo009oo`03001oogoo00800004Ool007oo0002
Ool00`00Oomoo`03Ool00`00Oomoo`03Ool01@00Oomoogoo00001Woo00<007ooOol03Goo00<007oo
Ool04Goo00@007ooOol000ioo`002Woo1@0000Eoo`00Oomoo`0000aoo`D00003Ool00000008000ao
o`D000=oo`03001oogoo00Eoo`03001oogoo009oo`09001oo`00Ool007oo001oo`00009oo`03001o
ogoo00=oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo00]oo`@0019o
o`04001oogoo000>Ool0011oo`<001=oo`03001oo`0000eoo`03001oogoo00Eoo`8000Ioo`03001o
ogoo009oo`03001oo`0000800004Ool007oo0002Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00
Oomoo`03Ool01@00Oomoogoo00003Woo00D007oo001oo`00015oo`<000moo`0047oo00<007ooOol0
4goo00<007oo00003Goo00<007ooOol00goo00@007oo000000Ioo`03001oogoo009oo`09001oogoo
Ool007oo001oo`00009oo`03001oogoo00=oo`03001oogoo00=oo`04001oogoo0002Ool01@00Oomo
ogoo00003Woo00D007oo001oo`00015oo`03001oogoo00moo`004Goo0`004goo0P003Goo00<007oo
Ool017oo0`001Woo00<007ooOol00Woo00T007ooOomoo`00Ool007oo00000Woo00<007ooOol00goo
00<007ooOol00goo00@007ooOomoo`8000Aoo`03001oogoo00eoo`800003Ool007oo015oo`<000io
o`00=Woo00<007ooOol01Woo00<007ooOol017oo00<007ooOol00Woo00T007ooOomoo`00Ool007oo
00000Woo00<007ooOol00goo00<007ooOol00goo00<007ooOol01goo00<007ooOol04Goo0P008Goo
000fOol00`00Oomoo`06Ool00`00Oomoo`04Ool00`00Oomoo`02Ool02@00Oomoogoo001oo`00Ool0
0002Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`07Ool00`00Oomoo`0COol00`00Oomo
o`0NOol003Ioo`03001oogoo00Ioo`03001oogoo00Aoo`03001oogoo009oo`09001oogooOol007oo
001oo`00009oo`03001oogoo009oo`03001oogoo00Aoo`03001oogoo00Moo`03001oogoo01=oo`<0
01ioo`00=Woo00<007ooOol01Woo00<007ooOol017oo00<007ooOol00goo00P007ooOol007oo001o
o`000Woo00<007ooOol00Woo00<007ooOol017oo00<007ooOol01goo00<007ooOol05Woo30004Woo
002S000000moo`03001oogoo00aoo`03001oogoo00Aoo`03001oogoo00ioo`03001oogoo009oo`05
001oogooOol00006Ool00`00Oomoo`03Ool02000Oomoo`00Ool007oo0002Ool00`00Oomoo`02Ool0
0`00Oomoo`04Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`0>Ool00`00Oomoo`0COol0
0`00Oomoo`0=Ool001moo`8001Eoo`03001oogoo00Ioo`03001oogoo00Aoo`03001oogoo00=oo`08
001oogoo001oo`00Ool0009oo`03001oogoo009oo`03001oogoo00Aoo`03001oogoo00Moo`03001o
ogoo03Aoo`008Goo00<007ooOol04Woo00<007ooOol01Woo00<007ooOol017oo00<007ooOol00goo
00P007ooOol007oo001oo`000Woo00<007ooOol00Woo00<007ooOol01Goo00<007ooOol01Goo00<0
07ooOol0=Goo000ROol00`00Oomoo`0@Ool00`00Oomoo`07Ool00`00Oomoo`04Ool00`00Oomoo`03
Ool02000Oomoo`00Ool007oo0002Ool00`00Oomoo`02Ool00`00Oomoo`05Ool00`00Oomoo`05Ool0
0`00Oomoo`0eOol002=oo`03001oogoo00moo`03001oogoo00Moo`03001oogoo00Aoo`03001oogoo
00=oo`05001oogoo001oo`020003Ool00`00Oomoo`02Ool00`00Oomoo`05Ool00`00Oomoo`05Ool0
0`00Oomoo`0eOol002=oo`03001oogoo00moo`03001oogoo00Moo`03001oogoo00Aoo`03001oogoo
00=oo`05001oogoo001oo`020003Ool00`00Oomoo`02Ool00`00Oomoo`05Ool00`00Oomoo`05Ool0
0`00Oomoo`0eOol002=oo`03001oogoo00moo`03001oogoo00Moo`03001oogoo00=oo`03001oogoo
00Aoo`05001oogoo001oo`020003Ool00`00Oomoo`02Ool00`00Oomoo`05Ool00`00Oomoo`05Ool0
0`00Oomoo`0eOol002Aoo`03001oogoo00ioo`03001oogoo00Qoo`03001oogoo009oo`03001oogoo
00Aoo`05001oogoo001oo`020003Ool00`00Oomoo`02Ool00`00Oomoo`05Ool00`00Oomoo`05Ool0
0`00Oomoo`0eOol002Aoo`03001oogoo00Qoo`@0009oo`05001oogooOol00002Ool3000017oo001o
ogoo0`001Woo00D007ooOol007oo008000=oo`03001oogoo009oo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo03Eoo`009Goo00<007ooOol027oo00<007ooOol00Woo0P0000=oo`00Ool00goo
00H007ooOol007oo0002Ool00`00Oomoo`04Ool01@00Oomoo`00Ool00P000goo00<007ooOol00Woo
00<007ooOol01Goo00<007ooOol01Goo00<007ooOol0=Goo000UOol00`00Oomoo`09Ool01`00Oomo
ogoo001oo`0000Eoo`06001oogoo001oo`000Woo00<007ooOol017oo00D007ooOol007oo008000=o
o`03001oogoo009oo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo03Eoo`009Woo00<007oo
Ool02Goo00<007ooOol00P0000=oo`00Ool00goo00H007ooOol007oo0002Ool00`00Oomoo`04Ool0
1@00Oomoo`00Ool00P000goo00<007ooOol00Woo00<007ooOol01Goo00<007ooOol017oo00<007oo
Ool0=Woo000VOol00`00Oomoo`06Ool01000Oomoo`000Woo00D007ooOomoo`0000=oo`06001oogoo
001oo`000Woo00<007ooOol017oo00D007ooOol007oo008000=oo`03001oogoo009oo`03001oogoo
00Eoo`03001oogoo00Aoo`03001oogoo03Ioo`009goo00<007ooOol01Woo0P000Woo00<007ooOol0
1Goo0P000Woo00<007ooOol00`001Woo100000=oo`0000000goo00<007ooOol00Woo00<007ooOol0
1Goo00<007ooOol017oo00<007ooOol0=Woo000WOol00`00Oomoo`0:Ool00`00Oomoo`09Ool00`00
Oomoo`02Ool00`00Oomoo`0600000goo001oo`020003Ool00`00Oomoo`02Ool00`00Oomoo`05Ool0
0`00Oomoo`04Ool00`00Oomoo`0fOol002Moo`03001oogoo00Yoo`03001oogoo00Uoo`03001oogoo
009oo`03001oogoo00Aoo`05001oo`00001oo`020004Ool01@00Oomoogoo00001goo00<007ooOol0
17oo00<007ooOol0=Woo000XOol00`00Oomoo`09Ool00`00Oomoo`09Ool00`00Oomoo`02Ool00`00
Oomoo`04Ool200000goo001oo`020004Ool01@00Oomoogoo00001goo00<007ooOol017oo00<007oo
Ool0=Woo000XOol00`00Oomoo`09Ool00`00Oomoo`09Ool00`00Oomoo`02Ool00`00Oomoo`04Ool4
00000goo00000004Ool01000Oomoo`0027oo00<007ooOol017oo00<007ooOol0=Woo000XOol00`00
Oomoo`09Ool00`00Oomoo`09Ool00`00Oomoo`02Ool00`00Oomoo`04Ool00`00Ool00002Ool20004
Ool01000Oomoo`0027oo00<007ooOol017oo00<007ooOol0=Woo000YOol00`00Oomoo`08Ool00`00
Oomoo`09Ool00`00Oomoo`02Ool00`00Oomoo`04Ool00`00Ool00002Ool20004Ool01000Oomoo`00
2Goo00<007ooOol00goo00<007ooOol0=Woo000YOol00`00Oomoo`08Ool00`00Oomoo`0:Ool01@00
Oomoogoo00001Woo00<007oo00000Woo0P0017oo00@007ooOol000Uoo`03001oogoo009oo`03001o
ogoo03Moo`00:Goo00<007ooOol017oo0`0000=oo`0000000goo00<007ooOol00`000Woo00<007oo
00000P001Woo00<007oo00000Woo0P0017oo00@007ooOol000Uoo`03001oogoo009oo`03001oogoo
03Moo`00:Goo00<007ooOol01Goo00P007ooOol007oo001oo`0017oo00@007ooOomoo`80009oo`03
001oogoo00Aoo`03001oo`00009oo`8000Aoo`04001oogoo0009Ool00`00Oomoo`02Ool00`00Oomo
o`0gOol002Yoo`03001oogoo009oo`@00005Ool007ooOol00005Ool01000Oomoogoo0P000Woo00<0
07ooOol017oo00<007oo00000Woo0P0017oo00@007ooOol000Uoo`03001oogoo009oo`03001oogoo
03Moo`00:Woo00<007ooOol00Woo00<007oo00000Woo00D007oo001oo`0000Aoo`04001oogooOol2
0002Ool00`00Oomoo`04Ool00`00Ool00002Ool20004Ool01000Oomoo`002Goo00<007ooOol00Woo
00<007ooOol0=goo000ZOol00`00Oomoo`02Ool01`00Ool007oo001oo`0000=oo`05001oogooOol0
0003Ool20002Ool00`00Oomoo`04Ool00`00Ool00002Ool20004Ool01000Oomoo`002Goo00<007oo
Ool00Woo00<007ooOol0=goo000ZOol00`00Oomoo`03Ool200000goo001oo`07Ool20003Ool00`00
Ool000020006Ool400000goo00000004Ool01000Oomoo`002Goo00<007ooOol00Woo00<007ooOol0
=goo000[Ool00`00Oomoo`05Ool00`00Oomoo`0;Ool01@00Oomoogoo00000Woo200000=oo`000000
17oo00@007ooOol000Uoo`03001oogoo009oo`03001oogoo03Moo`00:goo00<007ooOol01Goo00<0
07ooOol02goo00@007ooOol000Moo`03001oo`00009oo`8000Aoo`04001oogoo0009Ool00`00Oomo
o`02Ool00`00Oomoo`0gOol002]oo`03001oogoo00Eoo`03001oogoo00]oo`04001oogoo0007Ool4
00000goo00000004Ool01000Oomoo`002Goo00<007ooOol00Woo00<007ooOol0=goo000[Ool00`00
Oomoo`05Ool00`00Oomoo`0;Ool01000Oomoo`001goo100000=oo`00000017oo00@007ooOol000Yo
o`05001oogooOol0000iOol002]oo`03001oogoo00Eoo`03001oogoo00]oo`04001oogoo0007Ool0
0`00Ool00002Ool20004Ool01000Oomoo`002Woo00D007ooOomoo`0003Uoo`00;7oo00<007ooOol0
17oo00<007ooOol02goo00@007ooOol000Moo`03001oo`00009oo`8000Aoo`04001oogoo000:Ool0
1@00Oomoogoo0000>Goo000/Ool00`00Oomoo`04Ool00`00Oomoo`0;Ool01000Oomoo`001goo00<0
07oo00000Woo0P0017oo00@007ooOol000Yoo`05001oogooOol0000iOol002aoo`04001oogooOol3
0002Ool01@00Oomoogoo00000Woo0`000goo0`0027oo0P000Woo0P0017oo00@007ooOol000Yoo`05
001oogooOol0000iOol002aoo`04001oogoo0002Ool01`00Oomoogoo001oo`0000Aoo`03001oogoo
009oo`04001oo`000007Ool20002Ool20004Ool01000Oomoo`002Woo00D007ooOomoo`0003Uoo`00
;7oo00@007ooOol0009oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`04001oo`00
0007Ool20002Ool20004Ool01000Oomoo`002Woo00D007ooOomoo`0003Uoo`00;Goo00<007oo0000
0`000goo00<007oo000017oo00<007ooOol00Woo00@007oo000000Moo`80009oo`8000Eoo`03001o
o`0000Yoo`04001oogoo000jOol002eoo`03001oo`00009oo`04001oogoo0003Ool01@00Oomoogoo
000017oo0P0000=oo`00Ool01Woo0P000Woo0P001Goo0P002goo00@007ooOol003Yoo`00;Goo00<0
07ooOol00`002Goo0P0017oo0`0027oo0`0000=oo`0000001Goo0P002goo00@007ooOol003Yoo`00
;Woo00@007ooOol0011oo`8000Aoo`@00005Ool007oo001oo`020005Ool2000;Ool01000Oomoo`00
>Woo000^Ool01000Oomoo`0077oo00<007ooOol00P001Goo0P0037oo0P00>goo000^Ool3000LOol0
1000Ool007oo0P001Goo0P0037oo0P00>goo000_Ool2000LOol300000goo00000005Ool2000<Ool2
000kOol0055oo`03001oogoo00Aoo`8004Uoo`00DGoo00<007ooOol017oo0P00BGoo001AOol00`00
Oomoo`04Ool20019Ool0055oo`03001oogoo04moo`00DGoo00<007ooOol0Cgoo0000\
\>"],
  ImageRangeCache->{{{0, 162.438}, {100, 0}} -> {-7.37876, -7.8241*^-8, 
  0.0908504, 1.56458*^-9}}]
}, Open  ]],

Cell["\<\
But note the oscillation - it becomes quite severe as we \
differentiate.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[HFuncTwoD[x] - \(Norm'\)[x], \ {x, \ \(-7\), \ 7}, \ 
      PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0680272 0.166276 173748 [
[.09184 .15378 -6 -9 ]
[.09184 .15378 6 0 ]
[.22789 .15378 -6 -9 ]
[.22789 .15378 6 0 ]
[.36395 .15378 -6 -9 ]
[.36395 .15378 6 0 ]
[.63605 .15378 -3 -9 ]
[.63605 .15378 3 0 ]
[.77211 .15378 -3 -9 ]
[.77211 .15378 3 0 ]
[.90816 .15378 -3 -9 ]
[.90816 .15378 3 0 ]
[.4875 .0794 -39 -6.25 ]
[.4875 .0794 0 6.25 ]
[.4875 .25315 -33 -6.25 ]
[.4875 .25315 0 6.25 ]
[.4875 .34002 -33 -6.25 ]
[.4875 .34002 0 6.25 ]
[.4875 .4269 -45 -6.25 ]
[.4875 .4269 0 6.25 ]
[.4875 .51377 -33 -6.25 ]
[.4875 .51377 0 6.25 ]
[.4875 .60064 -45 -6.25 ]
[.4875 .60064 0 6.25 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.09184 .16628 m
.09184 .17253 L
s
[(-6)] .09184 .15378 0 1 Mshowa
.22789 .16628 m
.22789 .17253 L
s
[(-4)] .22789 .15378 0 1 Mshowa
.36395 .16628 m
.36395 .17253 L
s
[(-2)] .36395 .15378 0 1 Mshowa
.63605 .16628 m
.63605 .17253 L
s
[(2)] .63605 .15378 0 1 Mshowa
.77211 .16628 m
.77211 .17253 L
s
[(4)] .77211 .15378 0 1 Mshowa
.90816 .16628 m
.90816 .17253 L
s
[(6)] .90816 .15378 0 1 Mshowa
.125 Mabswid
.12585 .16628 m
.12585 .17003 L
s
.15986 .16628 m
.15986 .17003 L
s
.19388 .16628 m
.19388 .17003 L
s
.2619 .16628 m
.2619 .17003 L
s
.29592 .16628 m
.29592 .17003 L
s
.32993 .16628 m
.32993 .17003 L
s
.39796 .16628 m
.39796 .17003 L
s
.43197 .16628 m
.43197 .17003 L
s
.46599 .16628 m
.46599 .17003 L
s
.53401 .16628 m
.53401 .17003 L
s
.56803 .16628 m
.56803 .17003 L
s
.60204 .16628 m
.60204 .17003 L
s
.67007 .16628 m
.67007 .17003 L
s
.70408 .16628 m
.70408 .17003 L
s
.7381 .16628 m
.7381 .17003 L
s
.80612 .16628 m
.80612 .17003 L
s
.84014 .16628 m
.84014 .17003 L
s
.87415 .16628 m
.87415 .17003 L
s
.05782 .16628 m
.05782 .17003 L
s
.02381 .16628 m
.02381 .17003 L
s
.94218 .16628 m
.94218 .17003 L
s
.97619 .16628 m
.97619 .17003 L
s
.25 Mabswid
0 .16628 m
1 .16628 L
s
.5 .0794 m
.50625 .0794 L
s
gsave
.4875 .0794 -39 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(-5) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-7) show
grestore
.5 .25315 m
.50625 .25315 L
s
gsave
.4875 .25315 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(5) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-7) show
grestore
.5 .34002 m
.50625 .34002 L
s
gsave
.4875 .34002 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(1) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-6) show
grestore
.5 .4269 m
.50625 .4269 L
s
gsave
.4875 .4269 -45 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(1.5) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-6) show
grestore
.5 .51377 m
.50625 .51377 L
s
gsave
.4875 .51377 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-6) show
grestore
.5 .60064 m
.50625 .60064 L
s
gsave
.4875 .60064 -45 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(2.5) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-6) show
grestore
.125 Mabswid
.5 .09678 m
.50375 .09678 L
s
.5 .11415 m
.50375 .11415 L
s
.5 .13153 m
.50375 .13153 L
s
.5 .1489 m
.50375 .1489 L
s
.5 .18365 m
.50375 .18365 L
s
.5 .20103 m
.50375 .20103 L
s
.5 .2184 m
.50375 .2184 L
s
.5 .23577 m
.50375 .23577 L
s
.5 .27052 m
.50375 .27052 L
s
.5 .2879 m
.50375 .2879 L
s
.5 .30527 m
.50375 .30527 L
s
.5 .32265 m
.50375 .32265 L
s
.5 .3574 m
.50375 .3574 L
s
.5 .37477 m
.50375 .37477 L
s
.5 .39215 m
.50375 .39215 L
s
.5 .40952 m
.50375 .40952 L
s
.5 .44427 m
.50375 .44427 L
s
.5 .46165 m
.50375 .46165 L
s
.5 .47902 m
.50375 .47902 L
s
.5 .4964 m
.50375 .4964 L
s
.5 .53115 m
.50375 .53115 L
s
.5 .54852 m
.50375 .54852 L
s
.5 .5659 m
.50375 .5659 L
s
.5 .58327 m
.50375 .58327 L
s
.5 .06203 m
.50375 .06203 L
s
.5 .04465 m
.50375 .04465 L
s
.5 .02728 m
.50375 .02728 L
s
.5 .0099 m
.50375 .0099 L
s
.5 .61802 m
.50375 .61802 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
.5 Mabswid
.02381 .16628 m
.02605 .16628 L
.02846 .16628 L
.03279 .16628 L
.03543 .16628 L
.03793 .16628 L
.04262 .16628 L
.04502 .16628 L
.04756 .16628 L
.05212 .16628 L
.05466 .16628 L
.05704 .16628 L
.06244 .16628 L
.06504 .16628 L
.06788 .16628 L
.07038 .16628 L
.07305 .16628 L
.07568 .16628 L
.07807 .16628 L
.08274 .16628 L
.08556 .16628 L
.08814 .16628 L
.09093 .16628 L
.09399 .16628 L
.09914 .16628 L
.10195 .16628 L
.10458 .16628 L
.10959 .16629 L
.11241 .16629 L
.11503 .16629 L
.11983 .1663 L
.12493 .16631 L
.12948 .16633 L
.13375 .16634 L
.13844 .16637 L
.14349 .1664 L
.14845 .16644 L
.15306 .1665 L
.15812 .16658 L
.16353 .16668 L
.16922 .16683 L
.17458 .16702 L
.17937 .16723 L
.18463 .16751 L
.19435 .16822 L
.19967 .16873 L
.20464 .1693 L
.22309 .17223 L
.24167 .17609 L
.24676 .1771 L
Mistroke
.25218 .17803 L
.25481 .17839 L
.25729 .17866 L
.25837 .17875 L
.25951 .17883 L
.26077 .1789 L
.26192 .17894 L
.26324 .17896 L
.26444 .17895 L
.26511 .17894 L
.26583 .17891 L
.26715 .17883 L
.26838 .17873 L
.26968 .17858 L
.27202 .17822 L
.27474 .17762 L
.27729 .17689 L
.28309 .1745 L
.28815 .17156 L
.29277 .16816 L
.3032 .1582 L
.31315 .14679 L
.31861 .14053 L
.32376 .13521 L
.32857 .13122 L
.33128 .12954 L
.33257 .12892 L
.33378 .12845 L
.3348 .12815 L
.33589 .12792 L
.3371 .12778 L
.3382 .12776 L
.33935 .12787 L
.3406 .12813 L
.34178 .12852 L
.34287 .12901 L
.34505 .13036 L
.3474 .13238 L
.35166 .13756 L
.35672 .14622 L
.36137 .15627 L
.37155 .18263 L
.37652 .19559 L
.38103 .20596 L
.38328 .21028 L
.38573 .21413 L
.38685 .21556 L
.38804 .21682 L
.38905 .21768 L
.39017 .21838 L
Mistroke
.39144 .21885 L
.3926 .21896 L
.39324 .21889 L
.39394 .21871 L
.39521 .21806 L
.3964 .21711 L
.39766 .2157 L
.39993 .21217 L
.40262 .20633 L
.40552 .19808 L
.41079 .17868 L
.42062 .13458 L
.42522 .11582 L
.42775 .1075 L
.43014 .10151 L
.43149 .09909 L
.43226 .09807 L
.43296 .09736 L
.43421 .09666 L
.43484 .0966 L
.43554 .09676 L
.43679 .09768 L
.43749 .09856 L
.43814 .0996 L
.43933 .10211 L
.44058 .10558 L
.4431 .11517 L
.44579 .12906 L
.45064 .162 L
.45637 .20746 L
.45888 .2264 L
.46165 .24405 L
.46289 .25035 L
.46357 .25327 L
.46422 .25565 L
.46548 .25907 L
.46663 .26067 L
.46794 .26053 L
.46915 .25838 L
.46985 .2562 L
.47048 .25362 L
.4719 .24567 L
.47333 .2345 L
.47465 .22146 L
.47763 .18284 L
.48301 .09118 L
.48527 .05349 L
.48651 .03616 L
.48769 .02356 L
.48836 .01858 L
Mistroke
.48908 .01536 L
.49035 .01635 L
.49163 .02812 L
.49282 .0511 L
.4978 .33901 L
.49849 .41207 L
.49924 .50392 L
.49995 .60332 L
.5006 .52525 L
.50192 .36715 L
.50316 .25222 L
.50434 .16927 L
.50562 .10356 L
.50678 .06208 L
.50784 .03674 L
.509 .02085 L
.51027 .01472 L
.5109 .01532 L
.51159 .01826 L
.51283 .02856 L
.51399 .04278 L
.51526 .06175 L
.51756 .10131 L
.52188 .17532 L
.52456 .21242 L
.52707 .23801 L
.52834 .24721 L
.5297 .25441 L
.53043 .25716 L
.53121 .25922 L
.53187 .26032 L
.53259 .26085 L
.53389 .26013 L
.53507 .25777 L
.53573 .25582 L
.53644 .25323 L
.53773 .24738 L
.54021 .23265 L
.5425 .21618 L
.55154 .14608 L
.55415 .12941 L
.55662 .11645 L
.55884 .10749 L
.56009 .10361 L
.56126 .10078 L
.56193 .09948 L
.56265 .09836 L
.56397 .09703 L
.56525 .0966 L
.56642 .09692 L
Mistroke
.5677 .09802 L
.56842 .09896 L
.56909 .10005 L
.57046 .10281 L
.57192 .10653 L
.57714 .12499 L
.58189 .14598 L
.59117 .18647 L
.59585 .20221 L
.59851 .209 L
.60101 .21379 L
.60233 .21569 L
.60372 .21722 L
.60491 .21814 L
.60621 .21875 L
.60694 .21892 L
.60761 .21897 L
.60838 .21889 L
.60911 .21869 L
.6104 .21806 L
.61113 .21754 L
.6118 .21697 L
.61411 .21435 L
.61665 .2104 L
.62178 .19972 L
.63261 .17146 L
.63795 .15789 L
.64289 .14699 L
.64736 .13904 L
.64965 .13576 L
.65211 .13287 L
.65457 .13064 L
.65689 .12914 L
.65822 .12852 L
.6589 .12828 L
.65964 .12807 L
.66094 .12783 L
.66216 .12776 L
.66342 .12782 L
.66458 .128 L
.66586 .12834 L
.66721 .12883 L
.66949 .12997 L
.67199 .13162 L
.67661 .13557 L
.68094 .14003 L
.70102 .16258 L
.70599 .16714 L
.71141 .17126 L
.71653 .1743 L
Mistroke
.71897 .17546 L
.72128 .17639 L
.72354 .17715 L
.72598 .1778 L
.7282 .17826 L
.73024 .17857 L
.73139 .17871 L
.73246 .1788 L
.73363 .17888 L
.73489 .17894 L
.73621 .17896 L
.73746 .17896 L
.73858 .17893 L
.7398 .17887 L
.74102 .1788 L
.74214 .17871 L
.74465 .17845 L
.74921 .17781 L
.75958 .17583 L
.78059 .17154 L
.78975 .17006 L
.79952 .16882 L
.80514 .16827 L
.81025 .16785 L
.82022 .16725 L
.82507 .16703 L
.83021 .16685 L
.83465 .16673 L
.83941 .16662 L
.84453 .16653 L
.84932 .16647 L
.85501 .16641 L
.86021 .16637 L
.865 .16635 L
.87009 .16633 L
.87489 .16631 L
.87924 .1663 L
.88438 .1663 L
.88727 .16629 L
.88993 .16629 L
.89482 .16628 L
.89747 .16628 L
.89998 .16628 L
.90496 .16628 L
.90949 .16628 L
.91437 .16628 L
.91712 .16628 L
.91967 .16628 L
.9239 .16628 L
.92857 .16628 L
Mistroke
.93092 .16628 L
.93344 .16628 L
.93801 .16628 L
.94036 .16628 L
.94286 .16628 L
.94737 .16628 L
.95012 .16628 L
.95272 .16628 L
.95508 .16628 L
.95758 .16628 L
.9598 .16628 L
.96223 .16628 L
.96652 .16628 L
.96893 .16628 L
.97152 .16628 L
.97619 .16628 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{163.438, 101},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002S0000IA000`40O003h00ODGoo00<007ooOol0
Cgoo001AOol00`00Oomoo`1?Ool0055oo`03001oogoo04moo`00Cgoo00<007oo00000P00Cgoo001?
Ool00`00Ool00002001?Ool004moo`03001oo`00008004moo`00Cgoo00<007oo00000P00Cgoo001?
Ool5001?Ool004ioo`03001oo`0000<004moo`00CWoo00<007oo00000`00Cgoo000_Ool30003Ool0
1@00Oomoogoo00000Woo0`0017oo0P002Goo00<007oo00000`00Cgoo000bOol01`00Oomoogoo001o
o`0000Aoo`03001oogoo009oo`04001oogoo0008Ool00`00Ool00003001?Ool002Uoo`D000Aoo`03
001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`04001oogoo0008Ool00`00Ool00003001?
Ool002moo`<000Aoo`03001oo`0000Aoo`03001oogoo009oo`04001oogoo0008Ool00`00Ool00003
001?Ool002moo`03001oogoo00=oo`05001oogooOol00003Ool00`00Oomoo`02Ool01000Oomoo`00
27oo00<007oo00000`00Cgoo000_Ool40009Ool20005Ool20009Ool00`00Ool00003001?Ool004Io
o`H0009oo`03001oo`0000<000Ioo`<004Ioo`00AGoo00<007oo00001Woo00<007oo00000`001Woo
00<007oo0000AWoo000COol2000COol3000AOol40005Ool01000Oomoo`001Goo1P001Woo00<007oo
00001goo10004Woo0`004Woo0P004Woo000BOol01000Oomoo`004goo00<007ooOol04Goo00<007oo
Ool01Goo00@007ooOol000Aoo`L000Ioo`04001oogoo0007Ool00`00Oomoo`0COol00`00Oomoo`0@
Ool01000Oomoo`004Goo000<Ool500001Goo001oogoo00002goo1@0000=oo`0000000P002goo1@00
0goo00<007ooOol017oo00@007ooOol000Eoo`03001oo`0000<000Ioo`04001oogoo0008Ool01@00
Oomoogoo00003Woo10004Goo00@007ooOol0015oo`004Woo0`004Woo00<007oo000037oo10001Goo
00<007ooOol00Woo00D007ooOomoo`0000Eoo`03001oo`0000<000Eoo`05001oogooOol00009Ool6
000<Ool00`00Ool0000BOol3000BOol0019oo`03001oogoo019oo`03001oo`0000Yoo`8000Aoo`03
001oo`00009oo`03001oogoo009oo`05001oogooOol00005Ool00`00Ool000030005Ool00`00Oomo
o`02Ool00`00Oomoo`03Ool00`00Oomoo`020003Ool2000;Ool00`00Ool0000BOol00`00Oomoo`0B
Ool001=oo`<0019oo`8000Yoo`03001oogoo00=oo`03001oogoo008000Eoo`05001oogooOol00004
Ool00`00Oomoo`020002Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`04Ool30005Ool0
0`00Oomoo`0:Ool2000COol3000AOol003=oo`03001oogoo00Aoo`03001oogoo00Moo`03001oogoo
009oo`05001oogooOol00002Ool20002Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`06
Ool00`00Oomoo`04Ool00`00Oomoo`0bOol0039oo`03001oogoo00Ioo`03001oogoo00Ioo`03001o
ogoo009oo`05001oogooOol00002Ool20002Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomo
o`05Ool00`00Oomoo`06Ool00`00Oomoo`0aOol000=ooid000=oo`001Woo00<007ooOol00goo00<0
07ooOol00Woo00<007ooOol00Woo00<007ooOol00goo300017oo00@007ooOomoo`8000Aoo`03001o
ogoo009oo`8000Aoo`03001oo`00009oo`08001oogoo001oo`00Ool0009oo`80009oo`05001oo`00
Ool00002Ool01`00Oomoogoo001oo`0000=oo`8000Aoo`03001oogoo00=oo`8000=oo`03001oogoo
009oo``000Aoo`03001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00Eoo`00
:7oo200037oo00<007ooOol017oo00<007ooOol00goo00D007ooOomoo`00009oo`80009oo`05001o
ogooOol00006Ool00`00Oomoo`03Ool00`00Oomoo`0;Ool8000XOol003eoo`03001oogoo00=oo`03
001oogoo00=oo`05001oogooOol00002Ool20002Ool01@00Oomoogoo00001Woo00<007ooOol00goo
00<007ooOol0>goo000mOol00`00Oomoo`02Ool00`00Oomoo`05Ool01000Oomoo`000Woo0P000Woo
00D007ooOomoo`0000Ioo`03001oogoo009oo`03001oogoo03aoo`00?Goo00<007ooOol00Woo00<0
07ooOol01Goo00@007ooOol0009oo`80009oo`05001oogooOol00006Ool00`00Oomoo`02Ool00`00
Oomoo`0lOol003eoo`03001oogoo009oo`03001oogoo00Eoo`04001oogoo0002Ool20003Ool00`00
Ool00008Ool01@00Oomoogoo0000?Woo000nOol20002Ool00`00Oomoo`05Ool01000Oomoo`000Woo
0P000goo00<007oo000027oo00@007ooOol003moo`00?goo00<007oo000027oo00@007ooOol0009o
o`8000=oo`03001oo`0000Qoo`@003moo`00@7oo00<007ooOol01goo00@007ooOol0009oo`8000=o
o`03001oo`0000Uoo`80041oo`00BWoo00@007ooOol0009oo`8000=oo`03001oo`0004]oo`00;goo
0`000goo00D007ooOomoo`00009oo`<000Aoo`8000Eoo`04001oogoo0002Ool20003Ool00`00Ool0
001;Ool0039oo`07001oogooOol007oo000017oo00<007ooOol00Woo00@007ooOol000Eoo`03001o
o`00009oo`8000=oo`03001oo`0004]oo`00<Woo00<007ooOol00Woo00<007ooOol00goo00<007oo
Ool00Woo00@007ooOol000Eoo`8000=oo`8000=oo`03001oo`0004]oo`00;goo0`0017oo00<007oo
000017oo00<007ooOol00Woo00@007ooOol000Eoo`8000=oo`<0009oo`8004aoo`00;goo00<007oo
Ool00goo00D007ooOomoo`0000=oo`03001oogoo009oo`04001oogoo0005Ool20003Ool20003Ool2
001<Ool002moo`@000Uoo`8000Eoo`8000Uoo`04001oo`00001AOol004Qoo`@0009oo`04001oo`00
001AOol004ioo`04001oo`00001AOol004moo`<0055oo`00CGoo1@00DGoo001@Ool2001AOol0051o
o`80055oo`00D7oo0P00DGoo000_Ool30003Ool01@00Oomoogoo00000Woo0`0017oo0P002goo0P00
DGoo000`Ool00`00Oomoo`03Ool00`00Ool00004Ool00`00Oomoo`02Ool01000Oomoo`002Woo0P00
DGoo000`Ool00`00Oomoo`04Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`002Woo0P00
DGoo000`Ool00`00Oomoo`03Ool00`00Ool00004Ool00`00Oomoo`02Ool01000Oomoo`002Woo0P00
DGoo000`Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Yo
o`<0051oo`00;goo0P002goo0P001Goo0P002goo0P00DGoo001>Ool4001AOol004Qoo`@00003Ool0
07oo00<0055oo`00CGoo1@00DGoo001=Ool00`00Oomoo`02001AOol004ioo`@0055oo`00D7oo0P00
DGoo001@Ool2001AOol0051oo`80055oo`008goo0`0017oo0P000goo0`000goo00D007ooOomoo`00
009oo`<000Aoo`8000]oo`80055oo`0097oo00<007ooOol00goo0P001Woo00L007ooOomoo`00Ool0
0004Ool00`00Oomoo`02Ool01000Oomoo`002Woo0P00DGoo000TOol00`00Oomoo`0;Ool00`00Oomo
o`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`002Woo0P00DGoo000TOol00`00Oomo
o`08Ool30004Ool00`00Ool00004Ool00`00Oomoo`02Ool01000Oomoo`002Woo0`00D7oo000TOol0
0`00Oomoo`08Ool00`00Oomoo`03Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol0
00Yoo`80055oo`008goo0P002Woo10002Goo0P001Goo0P002Goo1000DGoo0018Ool400000goo001o
o`03001AOol004eoo`D0055oo`00CGoo00<007ooOol00P00DGoo001>Ool200000goo001oo`1@Ool0
055oo`03001oogoo04moo`00DGoo00<007ooOol0Cgoo001AOol00`00Oomoo`1?Ool002moo`@0009o
o`05001oogooOol00002Ool30004Ool2000<Ool00`00Oomoo`1?Ool0031oo`03001oogoo00=oo`03
001oo`0000Aoo`03001oogoo009oo`04001oogoo000;Ool00`00Oomoo`1?Ool0035oo`03001oogoo
00=oo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000;Ool00`00Oomoo`1?Ool0039oo`07
001oogooOol007oo000017oo00<007ooOol00Woo00@007ooOol000]oo`03001oogoo04moo`00;goo
00@007ooOol0009oo`05001oogooOol00003Ool00`00Oomoo`02Ool01000Oomoo`002goo0P00D7oo
000`Ool2000:Ool20005Ool2000<Ool00`00Oomoo`1?Ool004ioo`800003Ool007oo051oo`00B7oo
100000Ioo`00Ool007oo001AOol004eoo`<00003Ool007oo051oo`00CGoo00D007ooOomoo`00055o
o`00CWoo0P0000=oo`00Ool0D7oo001AOol00`00Oomoo`1?Ool0055oo`03001oogoo04moo`00DGoo
00<007ooOol0Cgoo000SOol40003Ool20003Ool30003Ool01@00Oomoogoo00000Woo0`0017oo0P00
37oo00<007ooOol0Cgoo000TOol00`00Oomoo`03Ool20006Ool01`00Oomoogoo001oo`0000Aoo`03
001oogoo009oo`04001oogoo000;Ool00`00Oomoo`1?Ool002Eoo`03001oogoo00Yoo`03001oogoo
009oo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000;Ool00`00Oomoo`1?Ool002Ioo`03
001oogoo00Ioo`<000Aoo`03001oo`0000Aoo`03001oogoo009oo`04001oogoo000;Ool2001@Ool0
02=oo`04001oogoo0008Ool00`00Oomoo`03Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@0
07ooOol000]oo`03001oogoo04moo`0097oo0P002Goo10002Goo0P001Goo0P002Goo0P0000=oo`00
Ool0D7oo0018Ool400001Woo001oo`00Ool0055oo`00CGoo0`00Dgoo001=Ool00`00Oomoo`1COol0
04ioo`8005=oo`00\
\>"],
  ImageRangeCache->{{{0, 162.438}, {100, 0}} -> {-7.64929, -9.57012*^-7, 
  0.0941813, 3.68746*^-8}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[HFuncTwoD2[x] - \(\(Norm'\)'\)[x], \ {x, \ \(-7\), \ 7}, \ 
      PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0680272 0.315042 5513.71 [
[.09184 .30254 -6 -9 ]
[.09184 .30254 6 0 ]
[.22789 .30254 -6 -9 ]
[.22789 .30254 6 0 ]
[.36395 .30254 -6 -9 ]
[.36395 .30254 6 0 ]
[.63605 .30254 -3 -9 ]
[.63605 .30254 3 0 ]
[.77211 .30254 -3 -9 ]
[.77211 .30254 3 0 ]
[.90816 .30254 -3 -9 ]
[.90816 .30254 3 0 ]
[.4875 .09449 -48 -4.5 ]
[.4875 .09449 0 4.5 ]
[.4875 .20477 -48 -4.5 ]
[.4875 .20477 0 4.5 ]
[.4875 .42532 -42 -4.5 ]
[.4875 .42532 0 4.5 ]
[.4875 .53559 -42 -4.5 ]
[.4875 .53559 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.09184 .31504 m
.09184 .32129 L
s
[(-6)] .09184 .30254 0 1 Mshowa
.22789 .31504 m
.22789 .32129 L
s
[(-4)] .22789 .30254 0 1 Mshowa
.36395 .31504 m
.36395 .32129 L
s
[(-2)] .36395 .30254 0 1 Mshowa
.63605 .31504 m
.63605 .32129 L
s
[(2)] .63605 .30254 0 1 Mshowa
.77211 .31504 m
.77211 .32129 L
s
[(4)] .77211 .30254 0 1 Mshowa
.90816 .31504 m
.90816 .32129 L
s
[(6)] .90816 .30254 0 1 Mshowa
.125 Mabswid
.12585 .31504 m
.12585 .31879 L
s
.15986 .31504 m
.15986 .31879 L
s
.19388 .31504 m
.19388 .31879 L
s
.2619 .31504 m
.2619 .31879 L
s
.29592 .31504 m
.29592 .31879 L
s
.32993 .31504 m
.32993 .31879 L
s
.39796 .31504 m
.39796 .31879 L
s
.43197 .31504 m
.43197 .31879 L
s
.46599 .31504 m
.46599 .31879 L
s
.53401 .31504 m
.53401 .31879 L
s
.56803 .31504 m
.56803 .31879 L
s
.60204 .31504 m
.60204 .31879 L
s
.67007 .31504 m
.67007 .31879 L
s
.70408 .31504 m
.70408 .31879 L
s
.7381 .31504 m
.7381 .31879 L
s
.80612 .31504 m
.80612 .31879 L
s
.84014 .31504 m
.84014 .31879 L
s
.87415 .31504 m
.87415 .31879 L
s
.05782 .31504 m
.05782 .31879 L
s
.02381 .31504 m
.02381 .31879 L
s
.94218 .31504 m
.94218 .31879 L
s
.97619 .31504 m
.97619 .31879 L
s
.25 Mabswid
0 .31504 m
1 .31504 L
s
.5 .09449 m
.50625 .09449 L
s
[(-0.00004)] .4875 .09449 1 0 Mshowa
.5 .20477 m
.50625 .20477 L
s
[(-0.00002)] .4875 .20477 1 0 Mshowa
.5 .42532 m
.50625 .42532 L
s
[(0.00002)] .4875 .42532 1 0 Mshowa
.5 .53559 m
.50625 .53559 L
s
[(0.00004)] .4875 .53559 1 0 Mshowa
.125 Mabswid
.5 .12206 m
.50375 .12206 L
s
.5 .14963 m
.50375 .14963 L
s
.5 .1772 m
.50375 .1772 L
s
.5 .23234 m
.50375 .23234 L
s
.5 .2599 m
.50375 .2599 L
s
.5 .28747 m
.50375 .28747 L
s
.5 .34261 m
.50375 .34261 L
s
.5 .37018 m
.50375 .37018 L
s
.5 .39775 m
.50375 .39775 L
s
.5 .45288 m
.50375 .45288 L
s
.5 .48045 m
.50375 .48045 L
s
.5 .50802 m
.50375 .50802 L
s
.5 .06692 m
.50375 .06692 L
s
.5 .03936 m
.50375 .03936 L
s
.5 .01179 m
.50375 .01179 L
s
.5 .56316 m
.50375 .56316 L
s
.5 .59073 m
.50375 .59073 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
.5 Mabswid
.02381 .31504 m
.02605 .31504 L
.02846 .31504 L
.03279 .31504 L
.03543 .31504 L
.03793 .31504 L
.04262 .31504 L
.04502 .31504 L
.04756 .31504 L
.05212 .31504 L
.05466 .31504 L
.05704 .31504 L
.06244 .31504 L
.06504 .31504 L
.06788 .31504 L
.07038 .31504 L
.07305 .31504 L
.07568 .31504 L
.07807 .31504 L
.08274 .31504 L
.08556 .31504 L
.08814 .31504 L
.09093 .31504 L
.09399 .31504 L
.09914 .31504 L
.10195 .31504 L
.10458 .31504 L
.10959 .31504 L
.11503 .31504 L
.12017 .31505 L
.12493 .31505 L
.12921 .31505 L
.13393 .31505 L
.13886 .31505 L
.14349 .31506 L
.14818 .31506 L
.15332 .31507 L
.15868 .31508 L
.16367 .31509 L
.17262 .31512 L
.18218 .31516 L
.19242 .31522 L
.20352 .3153 L
.21312 .31538 L
.21821 .31542 L
.22367 .31546 L
.22643 .31548 L
.22904 .31549 L
.23139 .3155 L
.23203 .3155 L
Mistroke
.23271 .3155 L
.23394 .3155 L
.23519 .3155 L
.23589 .3155 L
.23654 .3155 L
.23773 .3155 L
.23898 .3155 L
.24011 .3155 L
.24116 .31549 L
.24355 .31548 L
.24486 .31547 L
.24628 .31545 L
.24887 .31542 L
.25139 .31538 L
.25371 .31533 L
.25899 .3152 L
.26459 .315 L
.27399 .31454 L
.2841 .31389 L
.29358 .31324 L
.2982 .31296 L
.3024 .31274 L
.30478 .31264 L
.30704 .31257 L
.30828 .31254 L
.30942 .31252 L
.31067 .31251 L
.3113 .31251 L
.31199 .31251 L
.31269 .31251 L
.31336 .31251 L
.31461 .31253 L
.3153 .31254 L
.31595 .31256 L
.31741 .31261 L
.31858 .31266 L
.31987 .31272 L
.32247 .3129 L
.32521 .31314 L
.32771 .31342 L
.33336 .31423 L
.34359 .31629 L
.35272 .31841 L
.35774 .31948 L
.36249 .32028 L
.36518 .3206 L
.36592 .32066 L
.3667 .32073 L
.36737 .32077 L
.3681 .32081 L
Mistroke
.36935 .32085 L
.37053 .32086 L
.37124 .32085 L
.37192 .32083 L
.3732 .32076 L
.3744 .32067 L
.37568 .32053 L
.37797 .32017 L
.3807 .31958 L
.38318 .31888 L
.39334 .31459 L
.39888 .31163 L
.40406 .30891 L
.40652 .30774 L
.40885 .30678 L
.41112 .30602 L
.41212 .30574 L
.41322 .30549 L
.41435 .3053 L
.41554 .30516 L
.41685 .3051 L
.41807 .30512 L
.41937 .30526 L
.42057 .30547 L
.42188 .30581 L
.42327 .3063 L
.42569 .30745 L
.42788 .30884 L
.43281 .31306 L
.44169 .32308 L
.44667 .32844 L
.44803 .32964 L
.44947 .33073 L
.4507 .33147 L
.4514 .33181 L
.45206 .33206 L
.45333 .33238 L
.45449 .33244 L
.45575 .33223 L
.45645 .33199 L
.45711 .33167 L
.45831 .33087 L
.4594 .32988 L
.46066 .32841 L
.46184 .3267 L
.46426 .32222 L
.46651 .31688 L
.47164 .30148 L
.47442 .2923 L
.47705 .28431 L
Mistroke
.47819 .28139 L
.47941 .27887 L
.48004 .27786 L
.48074 .27704 L
.48197 .27644 L
.48313 .27709 L
.48419 .27892 L
.48542 .28278 L
.48658 .28851 L
.48765 .29583 L
.48865 .30465 L
.48974 .31687 L
.4909 .33323 L
.49214 .35495 L
.49348 .38415 L
.49591 .45504 L
.49706 .49804 L
.49828 .55112 L
.49932 .60332 L
.50046 .01472 L
.50176 .08092 L
.50297 .13329 L
.50426 .18107 L
.5057 .22472 L
.5081 .27975 L
.50942 .30173 L
.51066 .31797 L
.51179 .3296 L
.51299 .33891 L
.51401 .34477 L
.51513 .34928 L
.51634 .35224 L
.51763 .35356 L
.51885 .35337 L
.51996 .35222 L
.52126 .34991 L
.52264 .34661 L
.52512 .33928 L
.53071 .32109 L
.53305 .31437 L
.53555 .30828 L
.53699 .3054 L
.5383 .30317 L
.53962 .30132 L
.54087 .29994 L
.54208 .29892 L
.54338 .29817 L
.54456 .29777 L
.54565 .29764 L
.54683 .29773 L
Mistroke
.54812 .29808 L
.54878 .29835 L
.54948 .29871 L
.55074 .2995 L
.5532 .30151 L
.55552 .30386 L
.55989 .30886 L
.56471 .31443 L
.56982 .31945 L
.57249 .3215 L
.57391 .3224 L
.57543 .32321 L
.57683 .32382 L
.57811 .32427 L
.57933 .32459 L
.58062 .32483 L
.58186 .32495 L
.58256 .32498 L
.5832 .32499 L
.58438 .32493 L
.58563 .32479 L
.58675 .3246 L
.5878 .32436 L
.59018 .32365 L
.59511 .32158 L
.60039 .31884 L
.60993 .3139 L
.61413 .31213 L
.61877 .31064 L
.6213 .31005 L
.62369 .30964 L
.625 .30948 L
.6262 .30936 L
.62753 .30928 L
.62819 .30925 L
.62892 .30923 L
.62966 .30923 L
.63037 .30923 L
.63168 .30927 L
.63242 .3093 L
.6331 .30934 L
.63464 .30947 L
.63587 .3096 L
.63723 .30977 L
.63997 .31019 L
.65036 .31238 L
.66008 .3146 L
.66505 .31557 L
.67047 .31643 L
.67295 .31674 L
Mistroke
.67559 .31702 L
.67806 .31723 L
.68034 .31737 L
.68148 .31743 L
.68274 .31748 L
.68401 .31752 L
.6847 .31754 L
.68534 .31755 L
.68652 .31757 L
.68778 .31758 L
.68895 .31758 L
.69005 .31757 L
.69112 .31755 L
.69225 .31753 L
.69427 .31747 L
.69646 .31739 L
.69886 .31728 L
.71984 .31593 L
.72903 .31538 L
.73389 .31515 L
.73907 .31495 L
.74419 .3148 L
.74884 .3147 L
.75108 .31466 L
.75351 .31463 L
.75581 .31461 L
.75793 .3146 L
.75907 .31459 L
.7603 .31459 L
.76146 .31458 L
.76252 .31458 L
.76376 .31458 L
.76446 .31458 L
.76511 .31458 L
.7663 .31458 L
.76757 .31458 L
.76826 .31458 L
.76889 .31459 L
.77033 .31459 L
.77292 .3146 L
.77777 .31463 L
.79824 .3148 L
.80822 .31487 L
.81732 .31492 L
.82676 .31496 L
.83569 .31499 L
.84059 .315 L
.84511 .31501 L
.85004 .31502 L
.85535 .31502 L
Mistroke
.86039 .31503 L
.865 .31503 L
.86964 .31503 L
.87406 .31504 L
.87916 .31504 L
.88391 .31504 L
.88912 .31504 L
.89182 .31504 L
.89467 .31504 L
.90002 .31504 L
.90495 .31504 L
.91025 .31504 L
.91326 .31504 L
.91607 .31504 L
.91868 .31504 L
.92115 .31504 L
.9258 .31504 L
.92839 .31504 L
.93079 .31504 L
.93363 .31504 L
.93623 .31504 L
.94086 .31504 L
.94329 .31504 L
.94585 .31504 L
.95016 .31504 L
.9526 .31504 L
.95488 .31504 L
.95756 .31504 L
.96009 .31504 L
.96483 .31504 L
.96749 .31504 L
.96994 .31504 L
.97547 .31504 L
.97619 .31504 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{163.438, 101},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002S0000IA000`40O003h00ODGoo00<007ooOol0
Cgoo001AOol00`00Oomoo`1?Ool0055oo`03001oogoo04moo`00DGoo00<007ooOol0Cgoo001AOol0
0`00Oomoo`1?Ool0055oo`03001oogoo04moo`00DGoo00<007ooOol0Cgoo001AOol00`00Oomoo`1?
Ool0055oo`03001oogoo04moo`00DGoo00<007ooOol0Cgoo001AOol00`00Oomoo`1?Ool0055oo`03
001oogoo04moo`00DGoo00<007ooOol0Cgoo001AOol00`00Oomoo`1?Ool002Moo`8000Aoo`8000Ao
o`8000Aoo`8000Aoo`8000Aoo`8000Aoo`<000=oo`03001oogoo04moo`009Woo00@007ooOol000=o
o`8000=oo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0004Ool0
0`00Oomoo`02Ool00`00Oomoo`1?Ool0021oo`D00005Ool007ooOol00008Ool01000Oomoo`000Woo
00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`000Woo10000goo0P00D7oo000VOol01000
Oomoo`0027oo00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0009o
o`03001oo`0000Aoo`03001oogoo04moo`009Woo00@007ooOol000Qoo`04001oogoo0002Ool01000
Oomoo`000Woo00@007ooOol0009oo`04001oogoo0002Ool00`00Ool00004Ool00`00Oomoo`1?Ool0
02Moo`8000Yoo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`03001oogoo04moo`00DGoo00<0
07ooOol0Cgoo001AOol00`00Oomoo`1?Ool0055oo`03001oogoo04moo`00DGoo00<007ooOol0Cgoo
001AOol00`00Oomoo`1?Ool0055oo`03001oogoo04moo`00DGoo00<007ooOol0Cgoo001AOol00`00
Oomoo`1?Ool0055oo`03001oogoo04moo`00DGoo00<007ooOol0Cgoo001AOol00`00Oomoo`1?Ool0
055oo`03001oogoo04moo`009goo0P0017oo0P0017oo0P0017oo0P0017oo0P0017oo0P000goo1000
0goo00<007ooOol0Cgoo000VOol01000Oomoo`000goo0P000goo00@007ooOol0009oo`04001oogoo
0002Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oogoo00=oo`03001oogoo04moo`0087oo
1@0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`04
001oogoo0004Ool00`00Oomoo`02Ool2001@Ool002Ioo`04001oogoo0008Ool01000Oomoo`000Woo
00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`001Goo00@007ooOomoo`80051oo`009Woo
00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo
0002Ool01000Oomoo`000goo0P00D7oo000WOol2000:Ool20004Ool20004Ool20004Ool20004Ool2
0004Ool2001@Ool0055oo`80051oo`00DGoo0P00D7oo001AOol2001@Ool0055oo`80051oo`004Goo
0P0057oo0`004Woo10004Goo0P004Woo10004goo0`004goo0P003goo000@Ool01000Oomoo`0057oo
00<007ooOol04Woo00<007ooOol04Goo0P004goo00<007ooOol057oo00<007ooOol04Goo00@007oo
Ool000ioo`002Woo1@0000Eoo`00Oomoo`0000aoo`D00003Ool00000008000aoo`D000=oo`03001o
ogoo011oo`8001Aoo`03001oogoo015oo`@0019oo`04001oogoo000>Ool0011oo`<001=oo`03001o
o`0001Ioo`03001oogoo00]oo`80009oo`8001Eoo`03001oogoo011oo`03001oo`0001=oo`<000mo
o`0047oo00<007ooOol04goo00<007oo00004goo00@007ooOol000eoo`80009oo`80019oo`04001o
ogoo000BOol00`00Ool0000COol00`00Oomoo`0?Ool0015oo`<001=oo`8001Aoo`8000ioo`04001o
o`00Ool2000COol2000DOol2000DOol3000>Ool004eoo`04001oo`00Ool2001@Ool004aoo`05001o
ogoo001oo`020004Ool30019Ool0045oo`D000Ioo`05001oogoo001oo`020003Ool20002Ool00`00
Oomoo`16Ool00:<0000017oo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007oo
Ool00goo00<007ooOol00Woo30001Goo00<007ooOol00Woo00<007oo00001`0000=oo`00Ool017oo
00<007ooOol00Woo00H007ooOomoo`00Ool20003Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00
Oomoo`03Ool01000Oomoogoo3@001Goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo
00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo0017Ool01@00Oomo
ogoo00000goo00@007oo000000=oo`03001oogoo00=oo`D0049oo`00B7oo0`0017oo00L007oo001o
o`00Ool0001=Ool004moo`07001oo`00Ool007oo0000CGoo001?Ool01`00Ool007oo001oo`0004eo
o`00Cgoo00@007oo001oo`8004ioo`00Cgoo00<007oo00000Woo00<007ooOol0C7oo001?Ool00`00
Ool0001AOol004moo`03001oo`00055oo`00D7oo0P00DGoo001@Ool2001AOol0051oo`80055oo`00
D7oo0P00DGoo001@Ool2001AOol0051oo`80055oo`009goo0P0017oo0P0017oo0P0017oo0P0017oo
0P0017oo0P000goo10000Woo0P00DGoo000VOol01000Oomoo`000goo0P000goo00@007ooOol0009o
o`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oogoo009oo`80055oo`00
9Woo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`04001o
ogoo0004Ool01000Oomoogoo0`00D7oo000VOol01000Oomoo`0027oo00@007ooOol0009oo`04001o
ogoo0002Ool01000Oomoo`000Woo00@007ooOol000Eoo`03001oogoo0080055oo`009Woo00@007oo
Ool000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0002Ool0
1000Oomoo`000Woo0P00DGoo000WOol2000:Ool20004Ool20004Ool20004Ool20004Ool20003Ool2
001AOol0051oo`80055oo`00D7oo0P00DGoo001@Ool2001AOol0051oo`80055oo`00D7oo0P00DGoo
001@Ool2001AOol0051oo`80055oo`00D7oo0P00DGoo001@Ool2001AOol0051oo`80055oo`00D7oo
0P00DGoo001@Ool2001AOol002Moo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`<0
009oo`80055oo`009Woo00@007ooOol000=oo`8000=oo`04001oogoo0002Ool01000Oomoo`000Woo
00@007ooOol0009oo`04001oogoo0004Ool01000Oomoogoo0P00DGoo000VOol01000Oomoo`0027oo
00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`@0009oo`<0
051oo`009Woo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0009o
o`04001oogoo0002Ool00`00Ool00003Ool2001AOol002Ioo`04001oogoo0008Ool01000Oomoo`00
0Woo00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`000Woo00<007oo00000goo0P00DGoo
000WOol2000:Ool20004Ool20004Ool20004Ool20004Ool20003Ool2001AOol0051oo`80055oo`00
D7oo0P00DGoo001@Ool2001AOol0051oo`80055oo`00DGoo00<007ooOol0Cgoo001AOol00`00Oomo
o`1?Ool0055oo`03001oogoo04moo`00DGoo00<007ooOol0Cgoo001AOol00`00Oomoo`1?Ool0055o
o`03001oogoo04moo`00\
\>"],
  ImageRangeCache->{{{0, 162.438}, {100, 0}} -> {-7.37876, -5.71379*^-5, 
  0.0908504, 1.1209*^-6}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({HFuncTwoD2[0], \ \(\(Norm'\)'\)[0]}\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\(-0.0000591722558631602169`\), 0}\)], "Output"]
}, Open  ]],

Cell["\<\
So the second approximation behaves better than the first, with \
regard to both large negative arguments and differentiation.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Other Approximations", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "If you are working in C you may be interested to note that there is an \
accurate function for the cumulative normal distribution given in the book by \
Press ",
  StyleBox["et al",
    FontSlant->"Italic"],
  " (",
  StyleBox["Numerical Recipes in C",
    FontSlant->"Italic"],
  ", 2nd edition). The % error plot over the range ",
  Cell[BoxData[
      \(TraditionalForm\`\(-7\) < x < 7\)]],
  " is dead flat for this function when double-precision C is used with care. \
The code can be embedded in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " using ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " should you wish to confirm this directly. The algorithm at the heart of \
that approximation is a continued fraction expansion. We shall explore this \
presently. Other approaches can be based on the series method, but it needs \
to be applied piecewise. First we can develop a power series about the \
origin. Note that in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " the ",
  StyleBox["Normal",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function is used to convert a truncated power series to a polynomial."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(zerotrunc[x_, \ n_]\  := Normal[\ Series[Norm[x], \ {x, \ 0, \ n}]]; \n
    zerotrunc[x, \ 15]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1\/2 + x\/\@\(2\ \[Pi]\) - x\^3\/\(6\ \@\(2\ \[Pi]\)\) + 
      x\^5\/\(40\ \@\(2\ \[Pi]\)\) - x\^7\/\(336\ \@\(2\ \[Pi]\)\) + 
      x\^9\/\(3456\ \@\(2\ \[Pi]\)\) - x\^11\/\(42240\ \@\(2\ \[Pi]\)\) + 
      x\^13\/\(599040\ \@\(2\ \[Pi]\)\) - 
      x\^15\/\(9676800\ \@\(2\ \[Pi]\)\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "For large values of the argument we can use asymptotic series valid near ",
  
  Cell[BoxData[
      \(TraditionalForm\`\(+\[Infinity]\)\)]],
  ":"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(inftrunc[x_, \ n_]\  := \ 
      Simplify[1/2\  + \ 
            1/2\ Normal[Series[Erf[y], \ {y, \ Infinity, \ n}]]\  /. \ 
          y\  -> \ x/Sqrt[2]]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(inftrunc[x, \ 8]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1\/2\ \((2 - \(E\^\(-\(x\^2\/2\)\)\ \@\(2\/\[Pi]\)\ \((\(-15\) + 3\ \
x\^2 - x\^4 + x\^6)\)\)\/x\^7)\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Or near ",
  Cell[BoxData[
      \(TraditionalForm\`\(-\[Infinity]\)\)]],
  ":"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(ninftrunc[x_, \ n_]\  := \ 
      Simplify[1/2\  + \ 
            1/2\ Normal[Series[Erf[y], \ {y, \ \(-Infinity\), \ n}]]\  /. \ 
          y\  -> \ x/Sqrt[2]]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ninftrunc[x, 8]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(\(E\^\(-\(x\^2\/2\)\)\ \((\(-15\) + 3\ x\^2 - x\^4 + 
                x\^6)\)\)\/\(\@\(2\ \[Pi]\)\ x\^7\)\)\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The power series can give excellent results if sufficiently many terms are \
taken, and works well up to about ",
  Cell[BoxData[
      \(TraditionalForm\`x = 2\)]],
  " with 30 terms:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(diff[x_]\  = \ zerotrunc[x, \ 30] - Norm[x]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[diff[x], \ {x, \ \(-2\), \ 2}, \ PlotRange\  -> \ All]; \)\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.238095 0.309017 5.10217e+08 [
[.02381 .29652 -6 -9 ]
[.02381 .29652 6 0 ]
[.2619 .29652 -6 -9 ]
[.2619 .29652 6 0 ]
[.7381 .29652 -3 -9 ]
[.7381 .29652 3 0 ]
[.97619 .29652 -3 -9 ]
[.97619 .29652 3 0 ]
[.4875 .00289 -43.5 -6.25 ]
[.4875 .00289 0 6.25 ]
[.4875 .10493 -43.5 -6.25 ]
[.4875 .10493 0 6.25 ]
[.4875 .20697 -43.5 -6.25 ]
[.4875 .20697 0 6.25 ]
[.4875 .41106 -37.5 -6.25 ]
[.4875 .41106 0 6.25 ]
[.4875 .5131 -37.5 -6.25 ]
[.4875 .5131 0 6.25 ]
[.4875 .61515 -37.5 -6.25 ]
[.4875 .61515 0 6.25 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .30902 m
.02381 .31527 L
s
[(-2)] .02381 .29652 0 1 Mshowa
.2619 .30902 m
.2619 .31527 L
s
[(-1)] .2619 .29652 0 1 Mshowa
.7381 .30902 m
.7381 .31527 L
s
[(1)] .7381 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(2)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.07143 .30902 m
.07143 .31277 L
s
.11905 .30902 m
.11905 .31277 L
s
.16667 .30902 m
.16667 .31277 L
s
.21429 .30902 m
.21429 .31277 L
s
.30952 .30902 m
.30952 .31277 L
s
.35714 .30902 m
.35714 .31277 L
s
.40476 .30902 m
.40476 .31277 L
s
.45238 .30902 m
.45238 .31277 L
s
.54762 .30902 m
.54762 .31277 L
s
.59524 .30902 m
.59524 .31277 L
s
.64286 .30902 m
.64286 .31277 L
s
.69048 .30902 m
.69048 .31277 L
s
.78571 .30902 m
.78571 .31277 L
s
.83333 .30902 m
.83333 .31277 L
s
.88095 .30902 m
.88095 .31277 L
s
.92857 .30902 m
.92857 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.5 .00289 m
.50625 .00289 L
s
gsave
.4875 .00289 -43.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(-6) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-10) show
grestore
.5 .10493 m
.50625 .10493 L
s
gsave
.4875 .10493 -43.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(-4) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-10) show
grestore
.5 .20697 m
.50625 .20697 L
s
gsave
.4875 .20697 -43.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(-2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-10) show
grestore
.5 .41106 m
.50625 .41106 L
s
gsave
.4875 .41106 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-10) show
grestore
.5 .5131 m
.50625 .5131 L
s
gsave
.4875 .5131 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(4) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-10) show
grestore
.5 .61515 m
.50625 .61515 L
s
gsave
.4875 .61515 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(6) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-10) show
grestore
.125 Mabswid
.5 .0284 m
.50375 .0284 L
s
.5 .05391 m
.50375 .05391 L
s
.5 .07942 m
.50375 .07942 L
s
.5 .13044 m
.50375 .13044 L
s
.5 .15595 m
.50375 .15595 L
s
.5 .18146 m
.50375 .18146 L
s
.5 .23248 m
.50375 .23248 L
s
.5 .258 m
.50375 .258 L
s
.5 .28351 m
.50375 .28351 L
s
.5 .33453 m
.50375 .33453 L
s
.5 .36004 m
.50375 .36004 L
s
.5 .38555 m
.50375 .38555 L
s
.5 .43657 m
.50375 .43657 L
s
.5 .46208 m
.50375 .46208 L
s
.5 .48759 m
.50375 .48759 L
s
.5 .53861 m
.50375 .53861 L
s
.5 .56413 m
.50375 .56413 L
s
.5 .58964 m
.50375 .58964 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
.5 Mabswid
.02381 .01472 m
.02846 .09141 L
.03279 .14521 L
.03754 .18944 L
.04262 .22393 L
.04795 .24975 L
.053 .26707 L
.05753 .27837 L
.06244 .2873 L
.06751 .29384 L
.07038 .29666 L
.07308 .29884 L
.07851 .30216 L
.08156 .30354 L
.08435 .30456 L
.08931 .30594 L
.09212 .30652 L
.09472 .30697 L
.09949 .3076 L
.10216 .30786 L
.10458 .30806 L
.10945 .30836 L
.11191 .30848 L
.1146 .30858 L
.11691 .30866 L
.11937 .30872 L
.12383 .30881 L
.12651 .30885 L
.12942 .30889 L
.13199 .30891 L
.13472 .30893 L
.13717 .30895 L
.13987 .30896 L
.14465 .30898 L
.14699 .30899 L
.14956 .30899 L
.15207 .309 L
.15474 .309 L
.15725 .30901 L
.15953 .30901 L
.1617 .30901 L
.16399 .30901 L
.16667 .30901 L
.16914 .30901 L
.1718 .30901 L
.17472 .30901 L
.17724 .30902 L
.17995 .30902 L
.18485 .30902 L
.1874 .30902 L
Mistroke
.18813 .30902 L
.18882 .30902 L
.19011 .30902 L
.19133 .30902 L
.19248 .30902 L
.19367 .30902 L
.19498 .30902 L
.19775 .30902 L
.20029 .30902 L
.20171 .30902 L
.20241 .30902 L
.20306 .30902 L
.2038 .30902 L
.20461 .30902 L
.2053 .30902 L
.20602 .30902 L
.20726 .30902 L
.2084 .30902 L
.20965 .30902 L
.21096 .30902 L
.21211 .30902 L
.21317 .30902 L
.21434 .30902 L
.21559 .30902 L
.2163 .30902 L
.21695 .30902 L
.21767 .30902 L
.21843 .30902 L
.21971 .30902 L
.22043 .30902 L
.22111 .30902 L
.22235 .30902 L
.22353 .30902 L
.22418 .30902 L
.22489 .30902 L
.22613 .30902 L
.22734 .30902 L
.22845 .30902 L
.22968 .30902 L
.23096 .30902 L
.23209 .30902 L
.23313 .30902 L
.23428 .30902 L
.23551 .30902 L
.23616 .30902 L
.23685 .30902 L
.2376 .30902 L
.2383 .30902 L
.23956 .30902 L
.24027 .30902 L
Mistroke
.24094 .30902 L
.24218 .30902 L
.24332 .30902 L
.24463 .30902 L
.24588 .30902 L
.24656 .30902 L
.2472 .30902 L
.2484 .30902 L
.24969 .30902 L
.25042 .30902 L
.25109 .30902 L
.25221 .30902 L
.25344 .30902 L
.25472 .30902 L
.25592 .30902 L
.2566 .30902 L
.25734 .30902 L
.25804 .30902 L
.25869 .30902 L
.25991 .30902 L
.26121 .30902 L
.26188 .30902 L
.2626 .30902 L
.2639 .30902 L
.26466 .30902 L
.26545 .30902 L
.26613 .30902 L
.26688 .30902 L
.26811 .30902 L
.26942 .30902 L
.2701 .30902 L
.27085 .30902 L
.27217 .30902 L
.27341 .30902 L
.2741 .30902 L
.27475 .30902 L
.27601 .30902 L
.27716 .30902 L
.27828 .30902 L
.27948 .30902 L
.28061 .30902 L
.28163 .30902 L
.28284 .30902 L
.28414 .30902 L
.28537 .30902 L
.2865 .30902 L
.28772 .30902 L
.28902 .30902 L
.29025 .30902 L
.29137 .30902 L
Mistroke
.29268 .30902 L
.29337 .30902 L
.29411 .30902 L
.29475 .30902 L
.29546 .30902 L
.2967 .30902 L
.29742 .30902 L
.2981 .30902 L
.29938 .30902 L
.30009 .30902 L
.30075 .30902 L
.30152 .30902 L
.30225 .30902 L
.30357 .30902 L
.30477 .30902 L
.30606 .30902 L
.30679 .30902 L
.30746 .30902 L
.30872 .30902 L
.30991 .30902 L
.31098 .30902 L
.31215 .30902 L
.31342 .30902 L
.31406 .30902 L
.31477 .30902 L
.31603 .30902 L
.31719 .30902 L
.31828 .30902 L
.31944 .30902 L
.3206 .30902 L
.32187 .30902 L
.32297 .30902 L
.32415 .30902 L
.32526 .30902 L
.32627 .30902 L
.32756 .30902 L
.32821 .30902 L
.32892 .30902 L
.33019 .30902 L
.33137 .30902 L
.33208 .30902 L
.33276 .30902 L
.33403 .30902 L
.33475 .30902 L
.3354 .30902 L
.33613 .30902 L
.3369 .30902 L
.3382 .30902 L
.33894 .30902 L
.33961 .30902 L
Mistroke
.34085 .30902 L
.34216 .30902 L
.34334 .30902 L
.34444 .30902 L
.34564 .30902 L
.34693 .30902 L
.34815 .30902 L
.34885 .30902 L
.3495 .30902 L
.35082 .30902 L
.35155 .30902 L
.35223 .30902 L
.35337 .30902 L
.35462 .30902 L
.35592 .30902 L
.35713 .30902 L
.35782 .30902 L
.35857 .30902 L
.35928 .30902 L
.35993 .30902 L
.36117 .30902 L
.36248 .30902 L
.36316 .30902 L
.36389 .30902 L
.36521 .30902 L
.36601 .30902 L
.36677 .30902 L
.36746 .30902 L
.3682 .30902 L
.36944 .30902 L
.37061 .30902 L
.37189 .30902 L
.37261 .30902 L
.37327 .30902 L
.37452 .30902 L
.37589 .30902 L
.37658 .30902 L
.37732 .30902 L
.37802 .30902 L
.37865 .30902 L
.37989 .30902 L
.38104 .30902 L
.3823 .30902 L
.38293 .30902 L
.38363 .30902 L
.3848 .30902 L
.38606 .30902 L
.38725 .30902 L
.38833 .30902 L
.38962 .30902 L
Mistroke
.39083 .30902 L
.39192 .30902 L
.3931 .30902 L
.39439 .30902 L
.39509 .30902 L
.39575 .30902 L
.39691 .30902 L
.39818 .30902 L
.39937 .30902 L
.40049 .30902 L
.4015 .30902 L
.40259 .30902 L
.40379 .30902 L
.40505 .30902 L
.40613 .30902 L
.40732 .30902 L
.40863 .30902 L
.40934 .30902 L
.41001 .30902 L
.41118 .30902 L
.41247 .30902 L
.41319 .30902 L
.41387 .30902 L
.41515 .30902 L
.41587 .30902 L
.41652 .30902 L
.41724 .30902 L
.41801 .30902 L
.41921 .30902 L
.42034 .30902 L
.42102 .30902 L
.42167 .30902 L
.42289 .30902 L
.42417 .30902 L
.42536 .30902 L
.42666 .30902 L
.42738 .30902 L
.42805 .30902 L
.4293 .30902 L
.42996 .30902 L
.43067 .30902 L
.43141 .30902 L
.43211 .30902 L
.43343 .30902 L
.43466 .30902 L
.43579 .30902 L
.43704 .30902 L
.43767 .30902 L
.43836 .30902 L
.44373 .30902 L
Mistroke
.44951 .30902 L
.45084 .30902 L
.45159 .30902 L
.45229 .30902 L
.45358 .30902 L
.45429 .30902 L
.45495 .30902 L
.45618 .30902 L
.45687 .30902 L
.45751 .30902 L
.45868 .30902 L
.45993 .30902 L
.46109 .30902 L
.46233 .30902 L
.46351 .30902 L
.46458 .30902 L
.46584 .30902 L
.46655 .30902 L
.4672 .30902 L
.46841 .30902 L
.46969 .30902 L
.47083 .30902 L
.47207 .30902 L
.47336 .30902 L
.47457 .30902 L
.47571 .30902 L
.47674 .30902 L
.47793 .30902 L
.47905 .30902 L
.48027 .30902 L
.48157 .30902 L
.48267 .30902 L
.48389 .30902 L
.48914 .30902 L
.49369 .30902 L
.4962 .30902 L
.49858 .30902 L
.50079 .30902 L
.50204 .30902 L
.5032 .30902 L
.50427 .30902 L
.50524 .30902 L
.50636 .30902 L
.50742 .30902 L
.50858 .30902 L
.50983 .30902 L
.51208 .30902 L
.51453 .30902 L
.51719 .30902 L
.51789 .30902 L
Mistroke
.51864 .30902 L
.51932 .30902 L
.51997 .30902 L
.52122 .30902 L
.52188 .30902 L
.5226 .30902 L
.52385 .30902 L
.52516 .30902 L
.52629 .30902 L
.52752 .30902 L
.52877 .30902 L
.52994 .30902 L
.531 .30902 L
.53215 .30902 L
.5334 .30902 L
.53403 .30902 L
.53472 .30902 L
.53595 .30902 L
.53709 .30902 L
.546 .30902 L
.58485 .30902 L
.62615 .30902 L
.63616 .30902 L
.63744 .30902 L
.63809 .30902 L
.6388 .30902 L
.63957 .30902 L
.64028 .30902 L
.64094 .30902 L
.64165 .30902 L
.64231 .30902 L
.64303 .30902 L
.64435 .30902 L
.64562 .30902 L
.6468 .30902 L
.64792 .30902 L
.64912 .30902 L
.65044 .30902 L
.65168 .30902 L
.65234 .30902 L
.65306 .30902 L
.65438 .30902 L
.65566 .30902 L
.65685 .30902 L
.65804 .30902 L
.65916 .30902 L
.66016 .30902 L
.66125 .30902 L
.66244 .30902 L
.66369 .30902 L
Mistroke
.66476 .30902 L
.66594 .30902 L
.66717 .30902 L
.66786 .30902 L
.6685 .30902 L
.66968 .30902 L
.67094 .30902 L
.67205 .30902 L
.6731 .30902 L
.67425 .30902 L
.67549 .30902 L
.67673 .30902 L
.67744 .30902 L
.67809 .30902 L
.67879 .30902 L
.67942 .30902 L
.68015 .30902 L
.68083 .30902 L
.68197 .30902 L
.68318 .30902 L
.68382 .30902 L
.68451 .30902 L
.68573 .30902 L
.68697 .30902 L
.68814 .30902 L
.68919 .30902 L
.69033 .30902 L
.69158 .30902 L
.69227 .30902 L
.6929 .30902 L
.69414 .30902 L
.69528 .30902 L
.69636 .30902 L
.69749 .30902 L
.69863 .30902 L
.69988 .30902 L
.70096 .30902 L
.70212 .30902 L
.70322 .30902 L
.70421 .30902 L
.7054 .30902 L
.70651 .30902 L
.70772 .30902 L
.70903 .30902 L
.71415 .30902 L
.72333 .30902 L
.72401 .30902 L
.72466 .30902 L
.7254 .30902 L
.72607 .30902 L
Mistroke
.72729 .30902 L
.72862 .30902 L
.72991 .30902 L
.73061 .30902 L
.73127 .30902 L
.73199 .30902 L
.73278 .30902 L
.73345 .30902 L
.73418 .30902 L
.73545 .30902 L
.73684 .30902 L
.73815 .30902 L
.73937 .30902 L
.74046 .30902 L
.74166 .30902 L
.74411 .30902 L
.76342 .30902 L
.76475 .30902 L
.76549 .30902 L
.76617 .30902 L
.76748 .30902 L
.76872 .30902 L
.76999 .30902 L
.77065 .30902 L
.77138 .30902 L
.77212 .30902 L
.7729 .30902 L
.77362 .30902 L
.7743 .30902 L
.77558 .30902 L
.77681 .30902 L
.7779 .30902 L
.7791 .30902 L
.78179 .30902 L
.78427 .30902 L
.78551 .30902 L
.78683 .30902 L
.78814 .30902 L
.7889 .30902 L
.78959 .30902 L
.79084 .30902 L
.79153 .30902 L
.79218 .30902 L
.79335 .30902 L
.7946 .30902 L
.7957 .30902 L
.79693 .30902 L
.79804 .30902 L
.79908 .30902 L
.8003 .30902 L
Mistroke
.80143 .30902 L
.80266 .30902 L
.80396 .30902 L
.81398 .30902 L
.81465 .30902 L
.81539 .30902 L
.81608 .30902 L
.81672 .30902 L
.81747 .30902 L
.81826 .30902 L
.81892 .30902 L
.81965 .30902 L
.82097 .30902 L
.82171 .30902 L
.82238 .30902 L
.82493 .30902 L
.82972 .30902 L
.83239 .30902 L
.83481 .30902 L
.83708 .30902 L
.83953 .30903 L
.84397 .30903 L
.84648 .30903 L
.84923 .30904 L
.85191 .30904 L
.85478 .30905 L
.85747 .30906 L
.85992 .30907 L
.86471 .3091 L
.867 .30911 L
.86954 .30914 L
.872 .30916 L
.87465 .3092 L
.87713 .30924 L
.8794 .30928 L
.88384 .3094 L
.88648 .30949 L
.88893 .30959 L
.89185 .30974 L
.89449 .30991 L
.89939 .31032 L
.90208 .31062 L
.90462 .31096 L
.90927 .31178 L
.91191 .31239 L
.91438 .31307 L
.91935 .31488 L
.92214 .31621 L
.92469 .31767 L
.92923 .32103 L
Mistroke
.93417 .32611 L
.93883 .33278 L
.94315 .34115 L
.94705 .3511 L
.95125 .36516 L
.95522 .38252 L
.95885 .40295 L
.96289 .43199 L
.9673 .47372 L
.9719 .53179 L
.97619 .60332 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{192.688, 119.062},
  ImageMargins->{{33.0625, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000300000Ma000`40O003h00O`7oo0030Ool00<1o
o`00>goo0P000goo00D007ooOomoo`00009oo`<000Aoo`80071oo`00>Woo00@007ooOol000=oo`03
001oo`0000Aoo`03001oogoo009oo`04001oogoo001_Ool003Aoo`D00005Ool007ooOol00004Ool0
0`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`00Kgoo000jOol30004Ool00`00Ool00004Ool0
0`00Oomoo`02Ool01000Oomoo`003goo00<007ooOol0GGoo000jOol00`00Oomoo`03Ool01@00Oomo
ogoo00000goo00<007ooOol00Woo00@007ooOol000moo`8005ioo`00>goo0`002Goo0P001Goo0P00
47oo00<007ooOol0GGoo000>Ool00`00Oomoo`17Ool30003Ool00`00Ool0001OOol000ioo`03001o
ogoo049oo`@0009oo`06001oogooOol007oo0P00Ggoo000>Ool00`00Oomoo`18Ool01P00Oomoogoo
001oo`<005ioo`003Woo00<007ooOol0B7oo00H007ooOomoo`00Ool2001OOol000ioo`03001oogoo
04Moo`8000Aoo`03001oo`0005moo`003Woo00<007ooOol0Cgoo00<007ooOol0GGoo000>Ool00`00
Oomoo`1?Ool2001NOol000moo`03001oogoo04ioo`03001oogoo05eoo`003goo00<007ooOol0CWoo
00<007ooOol0GGoo000?Ool00`00Oomoo`1>Ool00`00Oomoo`1MOol000moo`03001oogoo04ioo`03
001oogoo05eoo`003goo00<007ooOol0CWoo0P00GWoo000?Ool00`00Oomoo`0YOol30002Ool01@00
Oomoogoo00000Woo0`0017oo0P0047oo00<007ooOol0GGoo000?Ool00`00Oomoo`0ZOol00`00Oomo
o`02Ool00`00Ool00004Ool00`00Oomoo`02Ool01000Oomoo`003goo00<007ooOol0GGoo000?Ool0
0`00Oomoo`0ROol500000goo000000020004Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomo
o`003goo00<007ooOol0GGoo000?Ool00`00Oomoo`0XOol00`00Ool00004Ool00`00Ool00004Ool0
0`00Oomoo`02Ool01000Oomoo`003goo0P00GWoo000?Ool00`00Oomoo`0XOol00`00Ool00003Ool0
1@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000moo`03001oogoo05eoo`003goo00<0
07ooOol0:Goo0P002Woo0P001Goo0P0027oo0`000goo00<007oo0000Ggoo000@Ool00`00Oomoo`10
Ool40002Ool01P00Oomoogoo001oo`8005moo`0047oo00<007ooOol0AWoo00H007ooOomoo`00Ool2
001OOol0011oo`03001oogoo04Ioo`06001oogooOol007oo0`00GWoo000@Ool00`00Oomoo`15Ool2
0004Ool00`00Ool0001OOol0011oo`03001oogoo04eoo`03001oogoo05eoo`0047oo00<007ooOol0
CGoo00<007ooOol0GGoo000@Ool00`00Oomoo`1=Ool2001NOol0011oo`03001oogoo04eoo`03001o
ogoo05eoo`0047oo00<007ooOol0CGoo00<007ooOol0GGoo000AOol00`00Oomoo`1<Ool00`00Oomo
o`1MOol0015oo`03001oogoo04aoo`8005ioo`004Goo00<007ooOol0C7oo00<007ooOol0GGoo000A
Ool00`00Oomoo`0VOol40002Ool01@00Oomoogoo00000Woo0`0017oo0P0047oo00<007ooOol0GGoo
000AOol00`00Oomoo`0WOol00`00Oomoo`03Ool00`00Ool00004Ool00`00Oomoo`02Ool01000Oomo
o`003goo00<007ooOol0GGoo000AOol00`00Oomoo`0POol50003Ool00`00Oomoo`03Ool00`00Oomo
o`03Ool00`00Oomoo`02Ool01000Oomoo`003goo00<007ooOol0GGoo000AOol00`00Oomoo`0YOol0
1`00Oomoogoo001oo`0000Aoo`03001oogoo009oo`04001oogoo000?Ool2001NOol0015oo`03001o
ogoo02Ioo`04001oogoo0002Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000mo
o`03001oogoo05eoo`004Goo00<007ooOol09goo0P002Woo0P001Goo0P0027oo0`000goo00<007oo
0000Ggoo000AOol00`00Oomoo`0oOol40002Ool01P00Oomoogoo001oo`8005moo`004Goo00<007oo
Ool0AGoo00H007ooOomoo`00Ool3001NOol0015oo`03001oogoo04Eoo`06001oogooOol007oo0P00
Ggoo000BOol00`00Oomoo`13Ool20004Ool00`00Ool0001OOol0019oo`03001oogoo04]oo`03001o
ogoo05eoo`003goo10009Goo0`009Goo00<007ooOol097oo0`009Woo100037oo000@Ool01000Oomo
o`009Goo00<007ooOol097oo0P009Woo00<007ooOol09Woo00<007ooOol037oo0009Ool50003Ool0
0`00Ool0000NOol50002Ool00`00Oomoo`0TOol00`00Oomoo`0UOol00`00Oomoo`0WOol00`00Oomo
o`0;Ool0019oo`8002Eoo`03001oogoo02Aoo`03001oogoo02Eoo`03001oogoo02Qoo`03001oogoo
00Yoo`003goo00H007ooOol007oo000TOol00`00Oomoo`0TOol00`00Oomoo`0UOol00`00Oomoo`0U
Ool01000Oomoo`0037oo000@Ool20002Ool00`00Oomoo`0QOol2000VOol2000UOol2000XOol2000=
Ool001Eoo`03001oogoo04Qoo`03001oogoo05eoo`005Woo00<007ooOol0Agoo00<007ooOol0GGoo
000GOol20017Ool00`00Oomoo`1MOol000Yoojd000Uoo`003Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol0
1Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol0
1Goo00<007ooOol017oo2P001goo00<007ooOol02goo000>Ool00`00Oomoo`0VOol00`00Oomoo`0V
Ool00`00Oomoo`0VOol00`00Oomoo`0MOol20007Ool00`00Oomoo`0;Ool0061oo`03001oogoo04Mo
o`8001Aoo`00H7oo00<007ooOol0B7oo00<007ooOol04Woo001POol2001:Ool00`00Oomoo`0AOol0
061oo`03001oogoo04Yoo`03001oogoo011oo`00H7oo00<007ooOol0BWoo00<007ooOol047oo001P
Ool00`00Oomoo`1:Ool00`00Oomoo`0@Ool0061oo`8004]oo`03001oogoo011oo`00H7oo00<007oo
Ool0Bgoo00<007ooOol03goo001POol00`00Oomoo`1;Ool00`00Oomoo`0?Ool0061oo`03001oogoo
04aoo`03001oogoo00ioo`00H7oo00<007ooOol0C7oo00<007ooOol03Woo001POol2001=Ool00`00
Oomoo`0>Ool003Yoo`@0009oo`05001oogooOol00002Ool30004Ool2000@Ool00`00Oomoo`1<Ool0
0`00Oomoo`0>Ool003]oo`03001oogoo00=oo`03001oo`0000Aoo`03001oogoo009oo`04001oogoo
000?Ool00`00Oomoo`1<Ool00`00Oomoo`0>Ool003aoo`03001oogoo00=oo`03001oogoo00=oo`03
001oogoo009oo`04001oogoo000?Ool00`00Oomoo`1<Ool00`00Oomoo`0>Ool003eoo`07001oogoo
Ool007oo000017oo00<007ooOol00Woo00@007ooOol000moo`8004eoo`03001oogoo00ioo`00>Woo
00@007ooOol0009oo`05001oogooOol00003Ool00`00Oomoo`02Ool01000Oomoo`003goo00<007oo
Ool0C7oo00<007ooOol03Woo000kOol2000:Ool20005Ool20008Ool30003Ool00`00Ool0001?Ool0
0`00Oomoo`0=Ool005=oo`@0009oo`06001oogooOol007oo0P00Cgoo00<007ooOol03Goo001IOol0
1P00Oomoogoo001oo`<004ioo`03001oogoo00eoo`00FGoo00H007ooOomoo`00Ool2001?Ool00`00
Oomoo`0=Ool005Qoo`8000Aoo`03001oo`0004moo`03001oogoo00eoo`00H7oo00<007ooOol0CGoo
00<007ooOol03Goo001POol00`00Oomoo`1>Ool00`00Oomoo`0<Ool0061oo`8004moo`03001oogoo
00aoo`00H7oo00<007ooOol0CWoo00<007ooOol037oo001POol00`00Oomoo`1>Ool00`00Oomoo`0<
Ool0061oo`03001oogoo04ioo`03001oogoo00aoo`00H7oo0P00Cgoo00<007ooOol037oo000kOol3
0002Ool01@00Oomoogoo00000Woo0`0017oo0P0047oo00<007ooOol0CWoo00<007ooOol037oo000l
Ool00`00Oomoo`02Ool00`00Ool00004Ool00`00Oomoo`02Ool01000Oomoo`003goo00<007ooOol0
CWoo00<007ooOol037oo000jOol40004Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`00
3goo00<007ooOol0CWoo00<007ooOol037oo000jOol00`00Ool00004Ool00`00Ool00004Ool00`00
Oomoo`02Ool01000Oomoo`003goo00<007ooOol0Cgoo00<007ooOol02goo000jOol00`00Ool00003
Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000moo`80051oo`03001oogoo00]o
o`00>goo0P002Woo0P001Goo0P0047oo00<007ooOol0Cgoo00<007ooOol02goo001HOol30003Ool0
0`00Ool0001AOol00`00Oomoo`0;Ool005=oo`@0009oo`06001oogooOol007oo0P00DGoo00<007oo
Ool02goo001IOol01P00Oomoogoo001oo`<0051oo`03001oogoo00]oo`00FGoo00H007ooOomoo`00
Ool2001AOol00`00Oomoo`0;Ool005Qoo`8000Aoo`03001oo`00055oo`03001oogoo00]oo`00H7oo
00<007ooOol0Cgoo00<007ooOol02goo001POol2001@Ool00`00Oomoo`0;Ool0061oo`03001oogoo
04moo`03001oogoo00]oo`00H7oo00<007ooOol0Cgoo00<007ooOol02goo001POol00`00Oomoo`1?
Ool00`00Oomoo`0;Ool0061oo`03001oogoo04moo`03001oogoo00]oo`00H7oo0P00D7oo00<007oo
Ool02goo000kOol20003Ool01@00Oomoogoo00000Woo0`0017oo0P0047oo00<007ooOol0Cgoo00<0
07ooOol02goo000jOol01000Oomoo`000goo00<007oo000017oo00<007ooOol00Woo00@007ooOol0
00moo`03001oogoo04moo`03001oogoo00]oo`00>Woo00@007ooOol000Aoo`03001oogoo00=oo`03
001oogoo009oo`04001oogoo000?Ool00`00Oomoo`1MOol003Yoo`<000Aoo`03001oo`0000Aoo`03
001oogoo009oo`04001oogoo000?Ool2001NOol003Yoo`03001oogoo00=oo`05001oogooOol00003
Ool00`00Oomoo`02Ool01000Oomoo`003goo00<007ooOol0GGoo000kOol30009Ool20005Ool20008
Ool30003Ool00`00Oomoo`1OOol005=oo`@0009oo`07001oogooOol007oo0000H7oo001IOol01`00
Oomoogoo001oo`00061oo`00FGoo00L007ooOomoo`00Ool0001POol005Qoo`8000Aoo`03001oogoo
05moo`00\
\>"],
  ImageRangeCache->{{{0, 191.688}, {118.062, 0}} -> {-2.35785, -6.71048*^-10, 
  0.0243602, 1.13678*^-11}}]
}, Open  ]],

Cell["\<\
The asymptotic series can be a little hard to handle and control, \
and so I cannot recommend that combination of power and asymptotic series. An \
alternative for large positive arguments (large negative dealt with in the \
usual way) that is both accurate and mind-bogglingly cute is the continued \
fraction expansion, also from Abramowitz and Stegun. It has the merit that \
apart from the exponential function, only integer coefficients are used.\
\>",
   "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(ContinuedFractionApprox[x_, \ n_]\  := \n
      Module[{u\  = \ Range[n], v, \ w}, v\  = \ Join[Reverse[u], \ {1}]; \n
        w = \ Fold[x\  + \ #2/#1 &, \ 1, v] - x]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm[ContinuedFractionApprox[x, 15]]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`1\/\(x + 1\/\(x + 2\/\(x + 3\/\(x + 4\/\(x + 5\/\(x + \
6\/\(x + 7\/\(x + 8\/\(x + 9\/\(x + 10\/\(x + 11\/\(x + 12\/\(x + 13\/\(x + \
14\/\(x + 15\)\)\)\)\)\)\)\)\)\)\)\)\)\)\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "For large positive ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " we define the following approximation:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(CFAOne[x_]\  = \ 
      1 - Exp[\(-x^2\)/2]/Sqrt[2\ Pi] ContinuedFractionApprox[x, 35]; \)\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(diffCF[x_]\  = \ CFAOne[x] - Norm[x]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[diffCF[x], \ {x, \ 1.5, \ 4}, \ PlotRange\  -> \ All]; \)\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.547619 0.380952 0.0147151 3.88063e+07 [
[.02381 .00222 -9 -9 ]
[.02381 .00222 9 0 ]
[.40476 .00222 -9 -9 ]
[.40476 .00222 9 0 ]
[.59524 .00222 -3 -9 ]
[.59524 .00222 3 0 ]
[.78571 .00222 -9 -9 ]
[.78571 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.20179 .09233 -33 -6.25 ]
[.20179 .09233 0 6.25 ]
[.20179 .16994 -33 -6.25 ]
[.20179 .16994 0 6.25 ]
[.20179 .24755 -33 -6.25 ]
[.20179 .24755 0 6.25 ]
[.20179 .32517 -33 -6.25 ]
[.20179 .32517 0 6.25 ]
[.20179 .40278 -33 -6.25 ]
[.20179 .40278 0 6.25 ]
[.20179 .48039 -45 -6.25 ]
[.20179 .48039 0 6.25 ]
[.20179 .558 -45 -6.25 ]
[.20179 .558 0 6.25 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .01472 m
.02381 .02097 L
s
[(1.5)] .02381 .00222 0 1 Mshowa
.40476 .01472 m
.40476 .02097 L
s
[(2.5)] .40476 .00222 0 1 Mshowa
.59524 .01472 m
.59524 .02097 L
s
[(3)] .59524 .00222 0 1 Mshowa
.78571 .01472 m
.78571 .02097 L
s
[(3.5)] .78571 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(4)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.0619 .01472 m
.0619 .01847 L
s
.1 .01472 m
.1 .01847 L
s
.1381 .01472 m
.1381 .01847 L
s
.17619 .01472 m
.17619 .01847 L
s
.25238 .01472 m
.25238 .01847 L
s
.29048 .01472 m
.29048 .01847 L
s
.32857 .01472 m
.32857 .01847 L
s
.36667 .01472 m
.36667 .01847 L
s
.44286 .01472 m
.44286 .01847 L
s
.48095 .01472 m
.48095 .01847 L
s
.51905 .01472 m
.51905 .01847 L
s
.55714 .01472 m
.55714 .01847 L
s
.63333 .01472 m
.63333 .01847 L
s
.67143 .01472 m
.67143 .01847 L
s
.70952 .01472 m
.70952 .01847 L
s
.74762 .01472 m
.74762 .01847 L
s
.82381 .01472 m
.82381 .01847 L
s
.8619 .01472 m
.8619 .01847 L
s
.9 .01472 m
.9 .01847 L
s
.9381 .01472 m
.9381 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.21429 .09233 m
.22054 .09233 L
s
gsave
.20179 .09233 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-9) show
grestore
.21429 .16994 m
.22054 .16994 L
s
gsave
.20179 .16994 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(4) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-9) show
grestore
.21429 .24755 m
.22054 .24755 L
s
gsave
.20179 .24755 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(6) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-9) show
grestore
.21429 .32517 m
.22054 .32517 L
s
gsave
.20179 .32517 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(8) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-9) show
grestore
.21429 .40278 m
.22054 .40278 L
s
gsave
.20179 .40278 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(1) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-8) show
grestore
.21429 .48039 m
.22054 .48039 L
s
gsave
.20179 .48039 -45 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(1.2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-8) show
grestore
.21429 .558 m
.22054 .558 L
s
gsave
.20179 .558 -45 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(1.4) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-8) show
grestore
.125 Mabswid
.21429 .03412 m
.21804 .03412 L
s
.21429 .05352 m
.21804 .05352 L
s
.21429 .07292 m
.21804 .07292 L
s
.21429 .11173 m
.21804 .11173 L
s
.21429 .13113 m
.21804 .13113 L
s
.21429 .15054 m
.21804 .15054 L
s
.21429 .18934 m
.21804 .18934 L
s
.21429 .20875 m
.21804 .20875 L
s
.21429 .22815 m
.21804 .22815 L
s
.21429 .26696 m
.21804 .26696 L
s
.21429 .28636 m
.21804 .28636 L
s
.21429 .30576 m
.21804 .30576 L
s
.21429 .34457 m
.21804 .34457 L
s
.21429 .36397 m
.21804 .36397 L
s
.21429 .38338 m
.21804 .38338 L
s
.21429 .42218 m
.21804 .42218 L
s
.21429 .44158 m
.21804 .44158 L
s
.21429 .46099 m
.21804 .46099 L
s
.21429 .49979 m
.21804 .49979 L
s
.21429 .5192 m
.21804 .5192 L
s
.21429 .5386 m
.21804 .5386 L
s
.21429 .57741 m
.21804 .57741 L
s
.21429 .59681 m
.21804 .59681 L
s
.21429 .61621 m
.21804 .61621 L
s
.25 Mabswid
.21429 0 m
.21429 .61803 L
s
.5 Mabswid
.02381 .60332 m
.03279 .45999 L
.04262 .34269 L
.05288 .25304 L
.06244 .1917 L
.0731 .14173 L
.0828 .10862 L
.094 .08098 L
.10458 .06237 L
.11428 .04994 L
.12455 .04028 L
.13333 .03416 L
.14299 .0291 L
.15313 .0252 L
.1624 .02256 L
.17311 .02033 L
.18328 .0188 L
.19287 .01775 L
.20309 .01692 L
.21273 .01634 L
.22147 .01595 L
.2315 .01562 L
.24072 .01539 L
.25085 .01521 L
.26154 .01507 L
.27102 .01498 L
.28118 .01491 L
.29077 .01486 L
.29951 .01482 L
.30943 .01479 L
.3186 .01477 L
.32866 .01476 L
.33936 .01475 L
.34873 .01474 L
.35883 .01473 L
.36837 .01473 L
.37711 .01472 L
.38691 .01472 L
.39603 .01472 L
.40604 .01472 L
.41673 .01472 L
.42698 .01472 L
.43808 .01472 L
.44856 .01472 L
.45823 .01472 L
.46792 .01472 L
.47699 .01472 L
.48694 .01472 L
.49249 .01472 L
.49763 .01472 L
Mistroke
.50776 .01472 L
.51881 .01472 L
.52854 .01472 L
.53398 .01472 L
.53891 .01472 L
.54829 .01472 L
.54957 .01472 L
.55077 .01472 L
.55143 .01472 L
.55216 .01472 L
.55342 .01472 L
.55462 .01472 L
.5559 .01472 L
.55699 .01472 L
.5582 .01472 L
.55948 .01472 L
.56021 .01472 L
.56087 .01472 L
.56209 .01472 L
.56337 .01472 L
.56469 .01472 L
.56541 .01472 L
.56609 .01472 L
.56689 .01472 L
.56762 .01472 L
.56905 .01472 L
.57026 .01472 L
.57155 .01472 L
.57264 .01472 L
.57384 .01472 L
.57654 .01472 L
.57903 .01472 L
.58024 .01472 L
.58153 .01472 L
.58283 .01472 L
.58426 .01472 L
.58542 .01472 L
.58664 .01472 L
.58728 .01472 L
.58799 .01472 L
.58922 .01472 L
.59042 .01472 L
.5917 .01472 L
.59291 .01472 L
.59401 .01472 L
.59512 .01472 L
.59633 .01472 L
.59851 .01472 L
.60379 .01472 L
.60668 .01472 L
Mistroke
.60741 .01472 L
.6081 .01472 L
.60939 .01472 L
.61049 .01472 L
.61169 .01472 L
.61416 .01472 L
.61548 .01472 L
.61689 .01472 L
.61757 .01472 L
.61822 .01472 L
.61942 .01472 L
.62058 .01472 L
.62183 .01472 L
.62408 .01472 L
.62919 .01472 L
.63951 .01472 L
.65799 .01472 L
.66293 .01472 L
.66564 .01472 L
.66817 .01472 L
.66949 .01472 L
.67023 .01472 L
.67091 .01472 L
.67393 .01472 L
.67925 .01472 L
.69937 .01472 L
.73902 .01472 L
.77714 .01472 L
.81773 .01472 L
.85679 .01472 L
.89831 .01472 L
.93831 .01472 L
.97619 .01472 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{206.438, 127.562},
  ImageMargins->{{33.0625, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003>0000Oa000`40O003h00OcWoo003>Ool000Mo
o`<000Aoo`8000=oo`<003Uoo`@000=oo`8000=oo`<001aoo`<001]oo`<000Aoo`8000=oo`<001ao
o`<000Qoo`0027oo00<007ooOol00goo0P001Woo00<007ooOol0=goo00<007ooOol00goo0P001Woo
00<007ooOol077oo00<007ooOol06goo00@007ooOomoo`8000Ioo`03001oogoo01Yoo`03001oogoo
00Moo`0027oo00<007ooOol02goo00<007ooOol0>7oo00<007ooOol02Woo00<007ooOol077oo00<0
07ooOol06goo00<007ooOol02Goo00<007ooOol067oo100027oo0008Ool00`00Oomoo`08Ool3000l
Ool00`00Oomoo`06Ool3000MOol2000LOol20009Ool3000KOol00`00Ool00009Ool000Qoo`03001o
ogoo00Qoo`03001oogoo03Uoo`04001oogoo0008Ool00`00Oomoo`0OOol00`00Oomoo`0KOol00`00
Oomoo`06Ool00`00Oomoo`0KOol00`00Ool00009Ool000Moo`8000Yoo`@003Uoo`8000Uoo`@001]o
o`<001]oo`<000Uoo`@001]oo`8000Uoo`00cWoo000cOol00`00Oomoo`2HOol003=oo`03001oogoo
09Qoo`002Woo`00017oo000?Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00
Oomoo`03Ool;0005Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`04Ool00`00Oomoo`04
Ool00`00Oomoo`05Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`05
Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`04
Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04
Ool00`00Oomoo`07Ool000moo`03001oogoo01Aoo`@000Uoo`03001oogoo025oo`03001oogoo029o
o`03001oogoo025oo`03001oogoo025oo`03001oogoo00Moo`0097oo0P003Goo00<007ooOol0V7oo
000SOol00`00Oomoo`0=Ool2002IOol0029oo`03001oogoo00ioo`03001oogoo09Qoo`008Goo00<0
07ooOol03goo00<007ooOol0V7oo000POol00`00Oomoo`0@Ool00`00Oomoo`2HOol001moo`03001o
ogoo015oo`8009Uoo`007Woo00<007ooOol04Woo00<007ooOol0V7oo000NOol00`00Oomoo`0BOol0
0`00Oomoo`2HOol001eoo`03001oogoo01=oo`8009Uoo`004Goo10000Woo00H007ooOomoo`00Ool4
0004Ool2000<Ool00`00Oomoo`2HOol0019oo`03001oogoo00=oo`05001oo`00Ool00002Ool00`00
Oomoo`02Ool01000Oomoo`002goo00<007ooOol0V7oo000COol00`00Oomoo`03Ool01000Oomoo`00
0Woo00<007ooOol00Woo00@007ooOol000]oo`03001oogoo09Qoo`0057oo00H007ooOomoo`00Ool2
0003Ool00`00Oomoo`02Ool01000Oomoo`002goo0P00VGoo000AOol01000Oomoo`000Woo00D007oo
Oomoo`0000=oo`03001oogoo009oo`04001oogoo000;Ool00`00Oomoo`2HOol0019oo`8000Moo`03
001oogoo008000Eoo`8000Qoo`80009oo`03001oogoo09Qoo`006Woo00<007ooOol03Goo10000goo
00<007oo0000VWoo000JOol00`00Oomoo`0BOol300000goo0000002IOol001Yoo`03001oogoo019o
o`05001oo`00Ool0002JOol001Uoo`03001oogoo01=oo`80009oo`03001oogoo09Qoo`006Goo00<0
07ooOol05goo00<007ooOol0V7oo000IOol00`00Oomoo`0GOol2002IOol001Qoo`03001oogoo01Qo
o`03001oogoo09Qoo`0067oo00<007ooOol067oo00<007ooOol0V7oo000HOol00`00Oomoo`0HOol2
002IOol0019oo`<0009oo`80009oo`03001oogoo00<000Aoo`8000aoo`03001oogoo09Qoo`004goo
00@007ooOomoo`800003Ool007oo00=oo`03001oogoo009oo`04001oogoo000;Ool00`00Oomoo`2H
Ool0015oo`@0009oo`03001oo`0000Eoo`03001oogoo009oo`04001oogoo000;Ool00`00Oomoo`2H
Ool0015oo`03001oo`0000=oo`800003Ool007oo00=oo`03001oogoo009oo`04001oogoo000;Ool2
002IOol0015oo`03001oo`0000=oo`05001oogooOol00003Ool00`00Oomoo`02Ool01000Oomoo`00
2goo00<007ooOol0V7oo000BOol20003Ool00`00Oomoo`04Ool20005Ool20008Ool20002Ool00`00
Oomoo`2HOol001Ioo`03001oogoo015oo`@000=oo`03001oo`0009Yoo`005Woo00<007ooOol05Woo
0`0000=oo`000000VGoo000FOol00`00Oomoo`0FOol01@00Ool007oo0000VWoo000FOol00`00Oomo
o`0FOol20002Ool00`00Oomoo`2HOol001Ioo`03001oogoo01Yoo`8009Uoo`005Goo00<007ooOol0
6goo00<007ooOol0V7oo000EOol00`00Oomoo`0KOol00`00Oomoo`2HOol001Eoo`03001oogoo01]o
o`03001oogoo09Qoo`005Goo00<007ooOol06goo0P00VGoo000BOol2000017oo001oo`000goo00<0
07ooOol00`0017oo0P0037oo00<007ooOol0V7oo000AOol00`00Oomoo`020002Ool00`00Ool00004
Ool00`00Oomoo`02Ool01000Oomoo`002goo00<007ooOol0V7oo000AOol01000Oomoo`0017oo00<0
07ooOol00goo00<007ooOol00Woo00@007ooOol000]oo`03001oogoo09Qoo`004Goo10000goo00<0
07oo000017oo00<007ooOol00Woo00@007ooOol000]oo`8009Uoo`004Goo00@007ooOol0009oo`05
001oogooOol00003Ool00`00Oomoo`02Ool01000Oomoo`002goo00<007ooOol0V7oo000BOol30009
Ool20005Ool20008Ool20002Ool00`00Oomoo`2HOol001Aoo`03001oogoo01=oo`@000=oo`04001o
o`00002IOol001Aoo`03001oogoo01Qoo`<00003Ool007oo09Uoo`0057oo00<007ooOol067oo00D0
07oo001oo`0009Yoo`004goo00<007ooOol06Goo0P000Woo00<007ooOol0V7oo000COol00`00Oomo
o`0MOol2002IOol001=oo`03001oogoo01eoo`03001oogoo09Qoo`004goo00<007ooOol07Goo00<0
07ooOol0V7oo000COol00`00Oomoo`0MOol00`00Oomoo`2HOol0019oo`8000=oo`05001oogooOol0
0002Ool30004Ool2000<Ool2002IOol0015oo`04001oo`000003Ool00`00Ool00004Ool00`00Oomo
o`02Ool01000Oomoo`002goo00<007ooOol0V7oo000AOol01000Ool0000017oo00<007ooOol00goo
00<007ooOol00Woo00@007ooOol000]oo`03001oogoo09Qoo`004Woo0P0017oo00<007oo000017oo
00<007ooOol00Woo00@007ooOol000]oo`03001oogoo09Qoo`004Goo0P0000Eoo`00Oomoo`0000=o
o`05001oogooOol00004Ool01000Oomoo`002goo0P00VGoo000BOol2000:Ool20005Ool2000<Ool0
0`00Oomoo`2HOol0019oo`03001oogoo01Yoo`80009oo`03001oogoo09Qoo`004Woo00<007ooOol0
5Goo10000goo00@007oo000009Uoo`004Woo00<007ooOol06Woo0`0000=oo`00Ool0VGoo000BOol0
0`00Oomoo`0JOol01@00Ool007oo0000VWoo000BOol00`00Oomoo`0JOol20002Ool00`00Oomoo`2H
Ool0019oo`03001oogoo01ioo`8009Uoo`004Woo00<007ooOol07Woo00<007ooOol0V7oo000AOol0
0`00Oomoo`0OOol00`00Oomoo`2HOol0015oo`03001oogoo01moo`03001oogoo09Qoo`004Goo0`00
0goo00D007ooOomoo`00009oo`<000Aoo`8000aoo`8009Uoo`004Goo0P001Goo00<007oo000017oo
00<007ooOol00Woo00@007ooOol000]oo`03001oogoo09Qoo`004Goo0P001Woo00<007ooOol00goo
00<007ooOol00Woo00@007ooOol000]oo`03001oogoo09Qoo`004Goo0P001Goo00<007oo000017oo
00<007ooOol00Woo00@007ooOol000]oo`8009Uoo`004Goo0P0017oo00D007ooOomoo`0000=oo`03
001oogoo009oo`04001oogoo000;Ool00`00Oomoo`2HOol0015oo`8000]oo`8000Eoo`8000Qoo`<0
0003Ool007oo09Uoo`004Goo00<007ooOol05Woo100000Ioo`00Ool007oo002JOol0015oo`03001o
ogoo01aoo`03001oogoo008009Uoo`004Goo00<007ooOol06goo00D007oo001oo`0009Yoo`0047oo
00<007ooOol077oo0`0000=oo`00Ool0VGoo000@Ool00`00Oomoo`0POol00`00Oomoo`2HOol0011o
o`03001oogoo021oo`8009Uoo`0047oo00<007ooOol087oo00<007ooOol0V7oo000@Ool00`00Oomo
o`0POol00`00Oomoo`2HOol0011oo`03001oogoo021oo`03001oogoo09Qoo`001Goo0`0017oo0P00
0Woo1@000Woo00D007ooOomoo`00009oo`<000Aoo`8000aoo`8009Uoo`001Woo00<007ooOol00goo
0P000Woo00<007oo00001Goo00<007oo000017oo00<007ooOol00Woo00@007ooOol000]oo`03001o
ogoo09Qoo`001Woo00<007ooOol01goo00@007ooOol000Eoo`03001oogoo00=oo`03001oogoo009o
o`04001oogoo000;Ool00`00Oomoo`2HOol000Ioo`03001oogoo00Moo`05001oogooOol00003Ool0
0`00Ool00004Ool00`00Oomoo`02Ool01000Oomoo`002goo0P00VGoo0006Ool00`00Oomoo`07Ool2
0002Ool01000Oomoo`000goo00D007ooOomoo`0000Aoo`04001oogoo000;Ool00`00Oomoo`2HOol0
00Eoo`8000Uoo`04001oo`00000:Ool20005Ool20008Ool300000goo001oo`2IOol0011oo`03001o
ogoo01Moo`@00006Ool007oo001oo`00VWoo000@Ool00`00Oomoo`0MOol00`00Oomoo`02002IOol0
011oo`03001oogoo01aoo`05001oo`00Ool0002JOol0011oo`03001oogoo01aoo`<00003Ool007oo
09Uoo`0047oo00<007ooOol087oo00<007ooOol0V7oo000@Ool00`00Oomoo`0POol2002IOol0011o
o`03001oogoo021oo`03001oogoo09Qoo`0047oo00<007ooOol087oo00<007ooOol0V7oo000?Ool0
0`00Oomoo`0QOol2002IOol000Eoo`<000Aoo`800004Ool007ooOol30002Ool01@00Oomoogoo0000
0Woo0`0017oo0P0037oo00<007ooOol0V7oo0006Ool00`00Oomoo`03Ool200000goo001oo`02Ool0
0`00Oomoo`02Ool00`00Ool00004Ool00`00Oomoo`02Ool01000Oomoo`002goo00<007ooOol0V7oo
0006Ool00`00Oomoo`06Ool00`00Ool000030004Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01000
Oomoo`002goo00<007ooOol0V7oo0006Ool00`00Oomoo`06Ool01@00Ool007oo000017oo00<007oo
000017oo00<007ooOol00Woo00@007ooOol000]oo`8009Uoo`001Woo00<007ooOol01Woo00D007oo
001oo`0000=oo`05001oogooOol00003Ool00`00Oomoo`02Ool01000Oomoo`002goo00<007ooOol0
V7oo0005Ool20008Ool00`00Oomoo`02000:Ool20005Ool20008Ool300000goo001oo`2IOol000mo
o`03001oogoo01Qoo`@00006Ool007oo001oo`00VWoo000?Ool00`00Oomoo`0NOol00`00Oomoo`02
002IOol000moo`03001oogoo01eoo`05001oo`00Ool0002JOol000moo`03001oogoo01eoo`<00003
Ool007oo09Uoo`003goo00<007ooOol08Goo0P00VGoo000?Ool00`00Oomoo`0QOol00`00Oomoo`2H
Ool000moo`03001oogoo025oo`03001oogoo09Qoo`00<goo00<007ooOol0V7oo000cOol2002IOol0
0001\
\>"],
  ImageRangeCache->{{{0, 205.438}, {126.562, 0}} -> {1.29306, -1.53692*^-9, 
  0.0137503, 1.34983*^-10}}]
}, Open  ]],

Cell["\<\
So a careful join of the power series with the continued fraction \
expansion can give excellent results. In fact we can do rather better still \
if we replace the power series by another continued fraction. This second \
series is good in a large neighbourhood of the origin. It is based on the \
function\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(ContinuedFractionApproxTwo[x_, \ n_]\  := \n
      Module[{u\  = \ Range[n], \ v, \ w}, \nv\  = \ Reverse[u]; \n
        Fold[\((2*#2 - 1)\) + x^2 \((\(-1\))\)^#2\ #2/#1\  &, \ 1, v]]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm[ContinuedFractionApproxTwo[x, 15]]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`1 - 
      x\^2\/\(\(2\ x\^2\)\/\(5 - \(3\ x\^2\)\/\(\(4\ x\^2\)\/\(9 - \(5\ \
x\^2\)\/\(\(6\ x\^2\)\/\(13 - \(7\ x\^2\)\/\(\(8\ x\^2\)\/\(17 - \(9\ \
x\^2\)\/\(\(10\ x\^2\)\/\(21 - \(11\ x\^2\)\/\(\(12\ x\^2\)\/\(25 - \(13\ \
x\^2\)\/\(\(14\ x\^2\)\/\(29 - 15\ x\^2\) + 27\)\) + 23\)\) + 19\)\) + 15\)\) \
+ 11\)\) + 7\)\) + 3\)\)], "Output"]
}, Open  ]],

Cell["The approximation to the normal distribution is given by", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(CFATwo[x_]\  = \ 
      1/2 + Exp[\(-x^2\)/2]/Sqrt[2\ Pi] 
          x/ContinuedFractionApproxTwo[x, 20]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(diffCFTwo[x_]\  = \ CFATwo[x] - Norm[x]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Let's look at the error for ",
  Cell[BoxData[
      \(TraditionalForm\`0 < x < 3\)]],
  ":"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[diffCFTwo[x], \ {x, \ 0, \ 3}, \ PlotRange\  -> \ All]; \)\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.0147169 8.32771e+09 [
[.18254 .00222 -9 -9 ]
[.18254 .00222 9 0 ]
[.34127 .00222 -3 -9 ]
[.34127 .00222 3 0 ]
[.5 .00222 -9 -9 ]
[.5 .00222 9 0 ]
[.65873 .00222 -3 -9 ]
[.65873 .00222 3 0 ]
[.81746 .00222 -9 -9 ]
[.81746 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .09799 -37.5 -6.25 ]
[.01131 .09799 0 6.25 ]
[.01131 .18127 -37.5 -6.25 ]
[.01131 .18127 0 6.25 ]
[.01131 .26455 -37.5 -6.25 ]
[.01131 .26455 0 6.25 ]
[.01131 .34783 -37.5 -6.25 ]
[.01131 .34783 0 6.25 ]
[.01131 .4311 -37.5 -6.25 ]
[.01131 .4311 0 6.25 ]
[.01131 .51438 -37.5 -6.25 ]
[.01131 .51438 0 6.25 ]
[.01131 .59766 -37.5 -6.25 ]
[.01131 .59766 0 6.25 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(0.5)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(1)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1.5)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(2)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(2.5)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(3)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .09799 m
.03006 .09799 L
s
gsave
.01131 .09799 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(1) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-11) show
grestore
.02381 .18127 m
.03006 .18127 L
s
gsave
.01131 .18127 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-11) show
grestore
.02381 .26455 m
.03006 .26455 L
s
gsave
.01131 .26455 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(3) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-11) show
grestore
.02381 .34783 m
.03006 .34783 L
s
gsave
.01131 .34783 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(4) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-11) show
grestore
.02381 .4311 m
.03006 .4311 L
s
gsave
.01131 .4311 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(5) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-11) show
grestore
.02381 .51438 m
.03006 .51438 L
s
gsave
.01131 .51438 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(6) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-11) show
grestore
.02381 .59766 m
.03006 .59766 L
s
gsave
.01131 .59766 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(7) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-11) show
grestore
.125 Mabswid
.02381 .03137 m
.02756 .03137 L
s
.02381 .04803 m
.02756 .04803 L
s
.02381 .06468 m
.02756 .06468 L
s
.02381 .08134 m
.02756 .08134 L
s
.02381 .11465 m
.02756 .11465 L
s
.02381 .1313 m
.02756 .1313 L
s
.02381 .14796 m
.02756 .14796 L
s
.02381 .16462 m
.02756 .16462 L
s
.02381 .19793 m
.02756 .19793 L
s
.02381 .21458 m
.02756 .21458 L
s
.02381 .23124 m
.02756 .23124 L
s
.02381 .24789 m
.02756 .24789 L
s
.02381 .2812 m
.02756 .2812 L
s
.02381 .29786 m
.02756 .29786 L
s
.02381 .31451 m
.02756 .31451 L
s
.02381 .33117 m
.02756 .33117 L
s
.02381 .36448 m
.02756 .36448 L
s
.02381 .38114 m
.02756 .38114 L
s
.02381 .39779 m
.02756 .39779 L
s
.02381 .41445 m
.02756 .41445 L
s
.02381 .44776 m
.02756 .44776 L
s
.02381 .46441 m
.02756 .46441 L
s
.02381 .48107 m
.02756 .48107 L
s
.02381 .49772 m
.02756 .49772 L
s
.02381 .53104 m
.02756 .53104 L
s
.02381 .54769 m
.02756 .54769 L
s
.02381 .56435 m
.02756 .56435 L
s
.02381 .581 m
.02756 .581 L
s
.02381 .61431 m
.02756 .61431 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
.5 Mabswid
.02381 .01472 m
.06244 .01472 L
.08255 .01472 L
.10458 .01472 L
.11448 .01472 L
.12507 .01472 L
.1302 .01472 L
.13273 .01472 L
.13384 .01472 L
.13506 .01472 L
.13615 .01472 L
.13718 .01472 L
.13826 .01472 L
.13942 .01472 L
.14069 .01472 L
.14189 .01472 L
.14297 .01472 L
.14415 .01472 L
.1448 .01472 L
.14548 .01472 L
.1467 .01472 L
.14802 .01472 L
.14946 .01472 L
.15447 .01472 L
.16382 .01472 L
.16906 .01472 L
.17194 .01472 L
.17467 .01472 L
.17721 .01472 L
.1779 .01472 L
.17864 .01472 L
.17927 .01472 L
.17997 .01472 L
.18117 .01472 L
.18246 .01472 L
.18368 .01472 L
.1848 .01472 L
.18607 .01472 L
.18728 .01472 L
.18954 .01472 L
.1946 .01472 L
.20372 .01472 L
.22442 .01472 L
.26289 .01472 L
.30381 .01472 L
.34321 .01472 L
.3811 .01472 L
.42144 .01472 L
.46026 .01472 L
.47039 .01472 L
Mistroke
.48144 .01472 L
.48649 .01472 L
.49187 .01472 L
.49448 .01472 L
.49694 .01472 L
.49802 .01472 L
.49915 .01472 L
.50039 .01472 L
.50154 .01472 L
.50267 .01472 L
.50388 .01472 L
.5052 .01472 L
.50642 .01472 L
.50755 .01472 L
.50862 .01472 L
.50978 .01472 L
.51104 .01472 L
.51169 .01472 L
.5124 .01472 L
.51313 .01472 L
.51382 .01472 L
.51505 .01472 L
.51637 .01472 L
.51767 .01472 L
.51891 .01472 L
.52002 .01472 L
.52121 .01472 L
.52236 .01472 L
.52343 .01472 L
.5246 .01472 L
.52586 .01472 L
.52717 .01472 L
.52842 .01472 L
.52955 .01472 L
.53077 .01472 L
.53182 .01472 L
.53298 .01472 L
.53419 .01472 L
.53533 .01472 L
.53639 .01472 L
.53737 .01472 L
.53844 .01472 L
.53956 .01472 L
.54079 .01472 L
.54149 .01472 L
.54213 .01472 L
.5433 .01472 L
.54455 .01472 L
.54567 .01472 L
.54671 .01472 L
Mistroke
.54786 .01472 L
.54909 .01472 L
.55431 .01472 L
.55689 .01472 L
.55814 .01472 L
.55928 .01472 L
.56042 .01472 L
.5615 .01472 L
.56264 .01472 L
.56386 .01472 L
.56513 .01472 L
.56582 .01472 L
.56647 .01472 L
.56761 .01472 L
.56887 .01472 L
.57001 .01472 L
.57108 .01472 L
.57205 .01472 L
.5731 .01472 L
.57424 .01472 L
.57546 .01472 L
.57649 .01472 L
.57764 .01472 L
.58786 .01472 L
.59035 .01472 L
.59172 .01472 L
.59301 .01472 L
.59374 .01472 L
.59453 .01472 L
.59525 .01472 L
.59592 .01472 L
.59723 .01472 L
.5979 .01472 L
.59863 .01472 L
.59991 .01472 L
.60113 .01472 L
.60223 .01472 L
.60342 .01472 L
.60857 .01472 L
.61786 .01472 L
.62759 .01472 L
.62825 .01472 L
.62894 .01472 L
.6297 .01472 L
.6304 .01472 L
.63165 .01472 L
.63237 .01472 L
.63303 .01472 L
.63427 .01472 L
.63541 .01472 L
Mistroke
.63671 .01472 L
.63795 .01472 L
.63902 .01472 L
.6402 .01472 L
.64144 .01472 L
.64259 .01472 L
.64367 .01472 L
.64465 .01472 L
.64579 .01472 L
.64688 .01472 L
.64804 .01472 L
.64928 .01472 L
.65045 .01472 L
.65151 .01472 L
.65276 .01472 L
.65346 .01472 L
.65411 .01472 L
.65531 .01472 L
.65657 .01472 L
.65779 .01472 L
.65847 .01472 L
.65911 .01472 L
.66028 .01472 L
.66152 .01472 L
.66263 .01472 L
.66367 .01472 L
.66481 .01472 L
.66604 .01472 L
.66727 .01472 L
.66797 .01472 L
.66862 .01472 L
.66935 .01472 L
.67004 .01472 L
.67134 .01472 L
.67246 .01472 L
.67369 .01472 L
.67499 .01472 L
.6762 .01472 L
.67725 .01472 L
.67839 .01472 L
.67946 .01472 L
.68043 .01472 L
.68158 .01472 L
.68283 .01472 L
.684 .01472 L
.6851 .01472 L
.68634 .01472 L
.68747 .01472 L
.68868 .01472 L
.68999 .01472 L
Mistroke
.69114 .01472 L
.69235 .01472 L
.69339 .01472 L
.69453 .01472 L
.69575 .01472 L
.69704 .01472 L
.69772 .01472 L
.69847 .01472 L
.69978 .01472 L
.70102 .01472 L
.70171 .01472 L
.70235 .01472 L
.70352 .01472 L
.70475 .01472 L
.70587 .01472 L
.70706 .01472 L
.70818 .01472 L
.7092 .01472 L
.7104 .01472 L
.71171 .01472 L
.71293 .01472 L
.71407 .01472 L
.71527 .01472 L
.71657 .01472 L
.71779 .01472 L
.71891 .01472 L
.72022 .01472 L
.72096 .01472 L
.72164 .01472 L
.7229 .01472 L
.72353 .01472 L
.72424 .01472 L
.72491 .01472 L
.72563 .01472 L
.7269 .01473 L
.72761 .01472 L
.72828 .01472 L
.729 .01472 L
.72978 .01472 L
.73045 .01472 L
.73118 .01472 L
.73187 .01472 L
.73251 .01472 L
.73371 .01472 L
.73499 .01472 L
.7362 .01472 L
.73751 .01472 L
.73875 .01472 L
.73991 .01472 L
.74105 .01472 L
Mistroke
.74209 .01473 L
.74328 .01472 L
.74441 .01473 L
.74564 .01473 L
.74694 .01473 L
.74805 .01473 L
.74927 .01473 L
.75058 .01473 L
.75132 .01473 L
.75199 .01473 L
.7533 .01473 L
.75453 .01473 L
.75558 .01473 L
.75674 .01473 L
.75795 .01474 L
.75909 .01473 L
.76014 .01474 L
.76111 .01474 L
.76218 .01474 L
.7633 .01474 L
.76444 .01475 L
.76566 .01474 L
.76681 .01475 L
.76785 .01475 L
.76908 .01475 L
.76978 .01475 L
.77042 .01475 L
.77159 .01475 L
.77283 .01476 L
.78227 .01478 L
.78499 .01479 L
.78564 .0148 L
.78633 .01479 L
.78757 .0148 L
.78989 .01481 L
.79241 .01483 L
.79737 .01486 L
.80203 .0149 L
.80621 .01494 L
.81073 .015 L
.81566 .01509 L
.82026 .01518 L
.82531 .01532 L
.83071 .0155 L
.83638 .01574 L
.84175 .01606 L
.84653 .01641 L
.85179 .01689 L
.85679 .01748 L
.86148 .01818 L
Mistroke
.86684 .01919 L
.87175 .02034 L
.88063 .02326 L
.88552 .02543 L
.89018 .02802 L
.89976 .03534 L
.90511 .04107 L
.90997 .0475 L
.91451 .05494 L
.91943 .06486 L
.92835 .08928 L
.93326 .10732 L
.93784 .12797 L
.9433 .15843 L
.94827 .19311 L
.95843 .2912 L
.96402 .36602 L
.96933 .45498 L
.97619 .60332 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{206.438, 127.562},
  ImageMargins->{{33.0625, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003>0000Oa000`40O003h00OcWoo003>Ool00<io
o`00cWoo003>Ool00<ioo`00cWoo000kOol20004Ool20003Ool3000BOol3000BOol30004Ool20003
Ool3000BOol4000AOol40003Ool20003Ool3000BOol30004Ool003Yoo`04001oogoo0003Ool20006
Ool00`00Oomoo`0@Ool00`00Oomoo`0BOol00`00Oomoo`03Ool20006Ool00`00Oomoo`0@Ool00`00
Oomoo`0BOol00`00Oomoo`03Ool20006Ool00`00Oomoo`0BOol00`00Oomoo`01Ool003Yoo`04001o
ogoo000;Ool00`00Oomoo`0@Ool00`00Oomoo`0BOol00`00Oomoo`0;Ool00`00Oomoo`0AOol00`00
Oomoo`0BOol00`00Oomoo`0:Ool00`00Oomoo`0BOol00`00Oomoo`01Ool003Yoo`04001oogoo0008
Ool3000COol00`00Oomoo`0BOol00`00Oomoo`08Ool3000EOol00`00Oomoo`0BOol00`00Oomoo`06
Ool3000COol20004Ool003Yoo`04001oogoo0008Ool00`00Oomoo`0COol00`00Oomoo`0BOol00`00
Oomoo`08Ool00`00Oomoo`0BOol01000Oomoo`004Goo00@007ooOol000Qoo`03001oogoo01Eoo`03
001oogoo005oo`00>goo0P002Goo10004Goo0P004goo0P002Woo10004Woo0P004goo0P002Goo1000
4Goo0`0017oo003>Ool002Moo`03001oogoo0:Aoo`009goo00<007ooOol0Y7oo000SOon[000002Mo
o`03001oogoo00=oo`03001oogoo009oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo009o
o`03001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo009o
o`03001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo009o
o`03001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo009o
o`03001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo009o
o`03001oogoo009oo`L000Aoo`03001oogoo00=oo`03001oogoo009oo`03001oogoo009oo`009goo
00<007ooOol0Sgoo0P004goo000WOol2002BOol00`00Oomoo`0@Ool002Moo`03001oogoo099oo`03
001oogoo00moo`009goo00<007ooOol0Tgoo00<007ooOol03Woo000WOol2002EOol00`00Oomoo`0=
Ool002Moo`03001oogoo09Aoo`03001oogoo00eoo`009goo00<007ooOol0U7oo00<007ooOol03Goo
000WOol2002FOol00`00Oomoo`0<Ool002Moo`03001oogoo09Eoo`03001oogoo00aoo`0000Aoo`00
000000=oo`05001oogooOol00002Ool30004Ool2000@Ool2002GOol00`00Oomoo`0;Ool0009oo`03
001oogoo00=oo`03001oo`0000Aoo`03001oogoo009oo`04001oogoo000?Ool00`00Oomoo`2FOol0
0`00Oomoo`0;Ool0009oo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo009oo`04001oogoo
000?Ool00`00Oomoo`2GOol00`00Oomoo`0:Ool0009oo`03001oogoo00=oo`03001oo`0000Aoo`03
001oogoo009oo`04001oogoo000?Ool2002HOol00`00Oomoo`0:Ool0009oo`03001oogoo009oo`05
001oogooOol00003Ool00`00Oomoo`02Ool01000Oomoo`003goo00<007ooOol0V7oo00<007ooOol0
2Goo00000goo0000000;Ool20005Ool20008Ool30002Ool4002JOol00`00Oomoo`09Ool001Yoo`@0
009oo`03001oogoo009oo`04001oo`00002IOol00`00Oomoo`09Ool0021oo`03001oogoo009oo`03
001oo`0009Yoo`03001oogoo00Uoo`0087oo00<007ooOol00Woo00<007oo0000VWoo00<007ooOol0
2Goo000OOol20003Ool200000goo0000002IOol00`00Oomoo`09Ool002Moo`03001oogoo09Qoo`03
001oogoo00Uoo`009goo00<007ooOol0V7oo00<007ooOol02Goo000WOol2002JOol00`00Oomoo`08
Ool002Moo`03001oogoo09Uoo`03001oogoo00Qoo`0000=oo`0000000P000Woo00D007ooOomoo`00
009oo`<000Aoo`80011oo`03001oogoo09Uoo`03001oogoo00Qoo`000Woo00<007ooOol00goo00<0
07oo000017oo00<007ooOol00Woo00@007ooOol000moo`8009Yoo`03001oogoo00Qoo`000goo00<0
07ooOol00goo00<007ooOol00goo00<007ooOol00Woo00@007ooOol000moo`03001oogoo09Uoo`03
001oogoo00Qoo`0017oo00L007ooOomoo`00Ool00004Ool00`00Oomoo`02Ool01000Oomoo`003goo
0P00Vgoo00<007ooOol01goo00001Goo001oogoo00000Woo00D007ooOomoo`0000=oo`03001oogoo
009oo`04001oogoo000?Ool00`00Oomoo`2JOol00`00Oomoo`07Ool0009oo`8000Yoo`8000Eoo`80
00Qoo`<0009oo`@009aoo`03001oogoo00Moo`006Woo10000Woo00<007ooOol00Woo00@007oo0000
09]oo`03001oogoo00Moo`0087oo00<007ooOol00Woo00<007oo0000W7oo00<007ooOol01goo000P
Ool00`00Oomoo`02Ool00`00Ool0002LOol00`00Oomoo`07Ool001moo`8000=oo`800003Ool00000
09]oo`03001oogoo00Moo`009goo00<007ooOol0Vgoo00<007ooOol01Woo000WOol00`00Oomoo`2K
Ool00`00Oomoo`06Ool002Moo`8009aoo`03001oogoo00Ioo`009goo00<007ooOol0Vgoo00<007oo
Ool01Woo000017oo000000000goo00D007ooOomoo`00009oo`<000Aoo`80011oo`03001oogoo09]o
o`03001oogoo00Ioo`0017oo00L007ooOomoo`00Ool00004Ool00`00Oomoo`02Ool01000Oomoo`00
3goo0P00W7oo00<007ooOol01Woo0004Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02
Ool01000Oomoo`003goo00<007ooOol0Vgoo00<007ooOol01Woo0002Ool20004Ool00`00Ool00004
Ool00`00Oomoo`02Ool01000Oomoo`003goo00<007ooOol0Vgoo00<007ooOol01Woo0004Ool01000
Oomoo`000goo00D007ooOomoo`0000Aoo`04001oogoo000?Ool2002LOol00`00Oomoo`06Ool00004
Ool00000000:Ool20005Ool2000@Ool00`00Oomoo`2LOol00`00Oomoo`05Ool001moo`<0009oo`D0
09eoo`03001oogoo00Eoo`006Woo10000Woo00<007ooOol00Woo00<007oo0000WWoo00<007ooOol0
1Goo000POol00`00Oomoo`02Ool00`00Ool0002NOol00`00Oomoo`05Ool0021oo`03001oogoo009o
o`04001oo`00002MOol00`00Oomoo`05Ool001moo`8000=oo`800003Ool007oo09eoo`03001oogoo
00Eoo`009goo00<007ooOol0W7oo00<007ooOol01Goo000WOol2002MOol00`00Oomoo`05Ool002Mo
o`03001oogoo09aoo`03001oogoo00Eoo`000Woo0`000Woo00D007ooOomoo`00009oo`<000Aoo`80
011oo`03001oogoo09aoo`03001oogoo00Eoo`000goo00<007ooOol00Woo00<007oo000017oo00<0
07ooOol00Woo00@007ooOol000moo`8009ioo`03001oogoo00Aoo`0000=oo`0000000P0017oo00<0
07ooOol00goo00<007ooOol00Woo00@007ooOol000moo`03001oogoo09eoo`03001oogoo00Aoo`00
00Aoo`00Ool000Aoo`03001oo`0000Aoo`03001oogoo009oo`04001oogoo000?Ool00`00Oomoo`2M
Ool00`00Oomoo`04Ool00004Ool007oo0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@0
07ooOol000moo`8009ioo`03001oogoo00Aoo`000Woo0P002Woo0P001Goo0P0047oo00<007ooOol0
WGoo00<007ooOol017oo000OOol30002Ool4002OOol00`00Oomoo`04Ool001Yoo`@0009oo`03001o
ogoo009oo`04001oo`00002NOol00`00Oomoo`04Ool0021oo`03001oogoo009oo`03001oo`0009mo
o`03001oogoo00Aoo`0087oo00<007ooOol00Woo00@007oo000009ioo`03001oogoo00Aoo`007goo
0P000goo0P0000=oo`00Ool0WWoo00<007ooOol017oo000WOol00`00Oomoo`2MOol00`00Oomoo`04
Ool002Moo`8009ioo`03001oogoo00Aoo`009goo00<007ooOol0WGoo00<007ooOol017oo000WOol0
0`00Oomoo`2MOol00`00Oomoo`04Ool00004Ool000000003Ool01@00Oomoogoo00000Woo0`0017oo
0P0047oo0P00Wgoo00<007ooOol00goo0004Ool01`00Oomoogoo001oo`0000Aoo`03001oogoo009o
o`04001oogoo000?Ool00`00Oomoo`2NOol00`00Oomoo`03Ool000Aoo`03001oogoo009oo`03001o
ogoo00=oo`03001oogoo009oo`04001oogoo000?Ool00`00Oomoo`2NOol00`00Oomoo`03Ool00004
Ool000000004Ool00`00Ool00004Ool00`00Oomoo`02Ool01000Oomoo`003goo0P00Wgoo00<007oo
Ool00goo00000goo001oo`04Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000mo
o`03001oogoo09ioo`03001oogoo00=oo`0000=oo`0000000P002Goo0P001Goo0P0027oo0`000Woo
1000X7oo00<007ooOol00goo000JOol40002Ool00`00Oomoo`02Ool01000Ool00000Wgoo00<007oo
Ool00goo000POol00`00Oomoo`02Ool00`00Ool0002POol00`00Oomoo`03Ool0021oo`03001oogoo
009oo`03001oo`000:1oo`03001oogoo00=oo`007goo0P000goo0P0000=oo`000000Wgoo00<007oo
Ool00goo000WOol00`00Oomoo`2NOol00`00Oomoo`03Ool002Moo`8009moo`03001oogoo00=oo`00
9goo00<007ooOol0WWoo00<007ooOol00goo000WOol00`00Oomoo`2NOol00`00Oomoo`03Ool0009o
o`8000=oo`05001oogooOol00002Ool30004Ool2000@Ool2002OOol00`00Oomoo`03Ool00005Ool0
07ooOol00003Ool00`00Ool00004Ool00`00Oomoo`02Ool01000Oomoo`003goo00<007ooOol0WWoo
00<007ooOol00goo00001Goo001oogoo000017oo00<007ooOol00goo00<007ooOol00Woo00@007oo
Ool000moo`03001oogoo09ioo`03001oogoo00=oo`0000Aoo`00000000Aoo`03001oo`0000Aoo`03
001oogoo009oo`04001oogoo000?Ool2002OOol00`00Oomoo`03Ool00003Ool007oo00Aoo`05001o
ogooOol00003Ool00`00Oomoo`02Ool01000Oomoo`003goo00<007ooOol0WWoo00<007ooOol00goo
0002Ool30009Ool20005Ool20008Ool30002Ool4002POol00`00Oomoo`03Ool001Yoo`@0009oo`03
001oogoo009oo`04001oo`00002POol00`00Oomoo`02Ool0021oo`03001oogoo009oo`03001oo`00
0:5oo`03001oogoo009oo`0087oo00<007ooOol00Woo00<007oo0000XGoo00<007ooOol00Woo000O
Ool20003Ool200000goo0000002POol00`00Oomoo`02Ool002Moo`03001oogoo09moo`03001oogoo
009oo`009goo00<007ooOol0Wgoo00<007ooOol00Woo000WOol2002POol00`00Oomoo`02Ool002Mo
o`03001oogoo09moo`03001oogoo009oo`000Woo00<007ooOol00Woo00D007ooOomoo`00009oo`<0
00Aoo`80011oo`800:1oo`03001oogoo009oo`000Woo00<007ooOol00goo00<007oo000017oo00<0
07ooOol00Woo00@007ooOol000moo`03001oogoo09moo`03001oogoo009oo`000goo00<007ooOol0
0goo00<007ooOol00goo00<007ooOol00Woo00@007ooOol000moo`03001oogoo09moo`03001oogoo
009oo`000goo00<007ooOol00Woo00<007oo000017oo00<007ooOol00Woo00@007ooOol000moo`80
0:1oo`03001oogoo009oo`0017oo00@007ooOol000=oo`05001oogooOol00004Ool01000Oomoo`00
3goo00<007ooOol0Wgoo00<007ooOol00Woo00000goo000000020009Ool20005Ool20008Ool30002
Ool4002VOol001Yoo`@0009oo`03001oogoo009oo`04001oo`00002UOol0021oo`03001oogoo009o
o`03001oo`000:Ioo`0087oo00<007ooOol00Woo00<007ooOol0YWoo000OOol20003Ool2002XOol0
0<ioo`00cWoo003>Ool00<ioo`00cWoo0000\
\>"],
  ImageRangeCache->{{{0, 205.438}, {126.562, 
  0}} -> {-0.734828, -1.14591*^-11, 0.018545, 7.06954*^-13}}]
}, Open  ]],

Cell[TextData[{
  "These two continued fraction expansions are what I recommend for accurate \
computation of the cumulative normal distribution, if you are not using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". They are given by equations (26.2.14) and (26.2.15) of Abramowitz and \
Stegun (1972). They are easy to code in any language, and give errors \
significantly smaller than the popular rational approximations."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 6 Bibliography", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "\nAbramowitz, M. and Stegun, I.A., 1972, ",
  StyleBox["Handbook of Mathematical Functions, ",
    FontSlant->"Italic"],
  "Dover 1972 edition. \n\nHull, J.C., 1996, ",
  StyleBox["Options, Futures, and other Derivatives",
    FontSlant->"Italic"],
  ", 3rd edition, Prentice-Hall.\n\n",
  "Press, W.H., Teukolsky, S.A., Vetterling, W.T. and Flannery, B.P., 1992, ",
  
  StyleBox["Numerical Recipes in C, the Art of Scientific Computing",
    FontSlant->"Italic"],
  ", 2nd edition, Cambridge University Press.",
  "\n\nRubenstein, M.  and Reiner, E., 1991, Unscrambling the binary code, ",
  StyleBox["RISK Magazine",
    FontSlant->"Italic"],
  ", October.\n"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{626, 659},
WindowMargins->{{14, Automatic}, {Automatic, 2}},
PrintingCopies->1,
PrintingStartingPageNumber->136,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"6. Binaries and Normal Distribution"}]]}, {Cell[ 
        TextData[ {"6. Binaries and Normal Distribution"}]], Inherited, 
      Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
PrivateFontOptions->{"FontType"->"Outline"},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003EL9@?oNomL=TPV60b85N`?P0080004/0B`000006G<AP001
0000I00000400@410?l00BL?00400@S0Qc0000000000074/01T1T00000008000
00000000007oo@S0V73oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 233, 8, 171, "Title"],
Cell[1967, 61, 207, 4, 60, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2199, 69, 140, 3, 49, "Section"],
Cell[2342, 74, 1508, 26, 284, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3887, 105, 122, 1, 49, "Section"],
Cell[4012, 108, 837, 19, 89, "Text"],
Cell[4852, 129, 168, 4, 37, "Input"],
Cell[5023, 135, 360, 9, 44, "Text"],
Cell[5386, 146, 191, 5, 37, "Input"],
Cell[5580, 153, 128, 1, 20, "Text"],
Cell[5711, 156, 280, 6, 48, "Input"],

Cell[CellGroupData[{
Cell[6016, 166, 235, 4, 20, "Input"],
Cell[6254, 172, 146227, 3709, 152, 127243, 3470, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[152518, 3886, 234, 4, 20, "Input"],
Cell[152755, 3892, 153669, 3800, 152, 127111, 3467, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[306439, 7695, 306, 6, 33, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[306782, 7706, 110, 1, 42, "Section"],
Cell[306895, 7709, 236, 6, 28, "Input"],
Cell[307134, 7717, 337, 6, 62, "NumberedEquation"],
Cell[307474, 7725, 239, 6, 37, "Input"],
Cell[307716, 7733, 715, 12, 69, "NumberedEquation"],
Cell[308434, 7747, 222, 5, 28, "Input"],
Cell[308659, 7754, 648, 11, 81, "NumberedEquation"],
Cell[309310, 7767, 215, 5, 28, "Input"],
Cell[309528, 7774, 518, 9, 74, "NumberedEquation"],
Cell[310049, 7785, 244, 6, 28, "Input"],
Cell[310296, 7793, 406, 7, 69, "NumberedEquation"],

Cell[CellGroupData[{
Cell[310727, 7804, 115, 1, 28, "Subsubsection"],
Cell[310845, 7807, 242, 5, 33, "Text"],

Cell[CellGroupData[{
Cell[311112, 7816, 302, 6, 29, "Input"],
Cell[311417, 7824, 35, 1, 20, "Output"]
}, Open  ]],
Cell[311467, 7828, 128, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[311620, 7833, 232, 4, 20, "Input"],
Cell[311855, 7839, 40, 1, 20, "Output"]
}, Open  ]],
Cell[311910, 7843, 297, 6, 33, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[312244, 7854, 94, 1, 28, "Subsubsection"],
Cell[312341, 7857, 190, 4, 20, "Text"],
Cell[312534, 7863, 586, 12, 108, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[313169, 7881, 99, 1, 42, "Section"],
Cell[313271, 7884, 252, 5, 33, "Text"],

Cell[CellGroupData[{
Cell[313548, 7893, 271, 5, 20, "Input"],
Cell[313822, 7900, 157417, 3927, 142, 133854, 3630, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False,
  PageBreakBelow->False]
}, Open  ]],
Cell[471254, 11830, 139, 2, 20, "Text",
  PageBreakBelow->False],

Cell[CellGroupData[{
Cell[471418, 11836, 141, 3, 20, "Input",
  PageBreakBelow->False],
Cell[471562, 11841, 341, 5, 62, "Output"]
}, Open  ]],
Cell[471918, 11849, 960, 16, 85, "Text",
  PageBreakAbove->False,
  PageBreakBelow->False],

Cell[CellGroupData[{
Cell[472903, 11869, 296, 6, 20, "Input",
  PageBreakBelow->False],
Cell[473202, 11877, 156210, 3929, 142, 132867, 3635, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakBelow->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[629461, 15812, 124, 1, 42, "Section"],
Cell[629588, 15815, 479, 12, 33, "Text"],

Cell[CellGroupData[{
Cell[630092, 15831, 98, 2, 20, "Input"],
Cell[630193, 15835, 144, 2, 20, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[630374, 15842, 127, 2, 20, "Input"],
Cell[630504, 15846, 48, 1, 20, "Output"]
}, Open  ]],
Cell[630567, 15850, 145, 4, 20, "Text"],
Cell[630715, 15856, 108, 1, 20, "Input"],
Cell[630826, 15859, 200, 5, 20, "Text"],

Cell[CellGroupData[{
Cell[631051, 15868, 102, 2, 20, "Input"],
Cell[631156, 15872, 69, 1, 49, "Output"]
}, Open  ]],
Cell[631240, 15876, 695, 12, 85, "Text"],

Cell[CellGroupData[{
Cell[631960, 15892, 127, 1, 33, "Subsection"],
Cell[632090, 15895, 744, 16, 60, "Text"],

Cell[CellGroupData[{
Cell[632859, 15915, 106, 1, 28, "Subsubsection"],
Cell[632968, 15918, 111, 1, 20, "Text"],
Cell[633082, 15921, 369, 6, 52, "Input"],
Cell[633454, 15929, 113, 1, 20, "Text"],
Cell[633570, 15932, 183, 4, 29, "Input"],
Cell[633756, 15938, 341, 10, 33, "Text"],
Cell[634100, 15950, 131, 2, 21, "NumberedEquation"],
Cell[634234, 15954, 1455, 29, 137, "Text"],
Cell[635692, 15985, 197, 3, 29, "Input"],
Cell[635892, 15990, 190, 4, 29, "Input"],
Cell[636085, 15996, 184, 4, 20, "Input"],
Cell[636272, 16002, 109, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[636406, 16007, 135, 2, 20, "Input"],
Cell[636544, 16011, 8379, 332, 109, 3724, 270, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[644938, 16346, 217, 5, 20, "Text"],

Cell[CellGroupData[{
Cell[645180, 16355, 188, 3, 20, "Input"],
Cell[645371, 16360, 11025, 501, 109, 6362, 439, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[656411, 16864, 185, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[656621, 16872, 176, 3, 20, "Input"],
Cell[656800, 16877, 19303, 692, 109, 7943, 547, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[676118, 17572, 174, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[676317, 17580, 182, 3, 20, "Input"],
Cell[676502, 17585, 15179, 649, 112, 8086, 557, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[691718, 18239, 188, 3, 20, "Input"],
Cell[691909, 18244, 13833, 633, 112, 8143, 558, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[705779, 18882, 126, 2, 20, "Input"],
Cell[705908, 18886, 124, 2, 20, "Output"]
}, Open  ]],
Cell[706047, 18891, 304, 6, 33, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[706400, 18903, 103, 1, 33, "Subsection"],
Cell[706506, 18906, 113, 1, 20, "Text"],
Cell[706622, 18909, 501, 9, 52, "Input"],
Cell[707126, 18920, 183, 4, 29, "Input"],
Cell[707312, 18926, 197, 3, 29, "Input"],
Cell[707512, 18931, 190, 4, 29, "Input"],
Cell[707705, 18937, 188, 4, 29, "Input"],
Cell[707896, 18943, 109, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[708030, 18948, 135, 2, 20, "Input"],
Cell[708168, 18952, 8897, 339, 112, 3725, 270, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[717080, 19294, 166, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[717271, 19302, 188, 3, 20, "Input"],
Cell[717462, 19307, 13417, 615, 112, 7833, 541, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[730894, 19925, 187, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[731106, 19933, 176, 3, 20, "Input"],
Cell[731285, 19938, 23876, 831, 108, 10057, 656, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[755176, 20772, 154, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[755355, 20780, 182, 3, 20, "Input"],
Cell[755540, 20785, 17178, 760, 108, 10133, 668, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[772755, 21550, 188, 3, 20, "Input"],
Cell[772946, 21555, 13798, 638, 108, 8370, 566, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[786781, 22198, 126, 2, 20, "Input"],
Cell[786910, 22202, 69, 1, 20, "Output"]
}, Open  ]],
Cell[786994, 22206, 207, 4, 20, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[787238, 22215, 99, 1, 33, "Subsection"],
Cell[787340, 22218, 1248, 32, 85, "Text"],

Cell[CellGroupData[{
Cell[788613, 22254, 184, 3, 29, "Input"],
Cell[788800, 22259, 320, 5, 40, "Output"]
}, Open  ]],
Cell[789135, 22267, 235, 7, 20, "Text"],
Cell[789373, 22276, 244, 5, 20, "Input"],

Cell[CellGroupData[{
Cell[789642, 22285, 106, 2, 20, "Input"],
Cell[789751, 22289, 136, 2, 76, "Output"]
}, Open  ]],
Cell[789902, 22294, 167, 6, 20, "Text"],
Cell[790072, 22302, 250, 5, 20, "Input"],

Cell[CellGroupData[{
Cell[790347, 22311, 105, 2, 20, "Input"],
Cell[790455, 22315, 148, 2, 50, "Output"]
}, Open  ]],
Cell[790618, 22320, 273, 7, 20, "Text"],
Cell[790894, 22329, 139, 2, 20, "Input"],

Cell[CellGroupData[{
Cell[791058, 22335, 158, 3, 20, "Input"],
Cell[791219, 22340, 20824, 952, 126, 13367, 855, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[812058, 23295, 535, 9, 60, "Text"],
Cell[812596, 23306, 257, 4, 40, "Input"],

Cell[CellGroupData[{
Cell[812878, 23314, 137, 2, 20, "Input"],
Cell[813018, 23318, 227, 3, 160, "Output"]
}, Open  ]],
Cell[813260, 23324, 201, 6, 20, "Text"],
Cell[813464, 23332, 185, 4, 20, "Input"],
Cell[813652, 23338, 132, 2, 20, "Input"],

Cell[CellGroupData[{
Cell[813809, 23344, 157, 3, 20, "Input"],
Cell[813969, 23349, 15416, 576, 134, 7398, 472, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[829400, 23928, 387, 7, 46, "Text"],
Cell[829790, 23937, 276, 5, 52, "Input"],

Cell[CellGroupData[{
Cell[830091, 23946, 140, 2, 20, "Input"],
Cell[830234, 23950, 380, 6, 188, "Output"]
}, Open  ]],
Cell[830629, 23959, 129, 1, 20, "Text"],
Cell[830761, 23962, 200, 4, 20, "Input"],
Cell[830964, 23968, 135, 2, 20, "Input"],
Cell[831102, 23972, 180, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[831307, 23982, 158, 3, 20, "Input"],
Cell[831468, 23987, 19289, 804, 134, 10832, 695, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[850772, 24794, 504, 9, 46, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[851325, 24809, 98, 1, 42, "Section"],
Cell[851426, 24812, 750, 18, 137, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

