(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   1437962,      36237]*)
(*NotebookOutlinePosition[   1439794,      36293]*)
(*  CellTagsIndexPosition[   1439750,      36289]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 8.
Barrier Options - a Case Study in Rapid Development\
\>", "Title",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["\<\
Doing it Quickly, and the Curious Properties of Options with \
Barriers\
\>", "Subsubtitle",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["8.1 Introduction", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We can use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " in various ways to gain insight into the properties of an option. This \
chapter illustrates one way of proceeding. Suppose you have done the basic \
mathematical and financial research to model the option, you have the \
valuation formula, and what you need is an implementation to get numbers out. \
What you want is a tool to allow the extraction of results as quickly as \
possible. When the formulae are in closed form, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is an outstanding tool for this. You literally just have to copy the \
formulae into a Notebook, and enter the desired calculations. In reality one \
should be a little more careful - the process consists of:\n\n(a) copying the \
formulae from the research material into ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ";\n(b) use of symbolic differentiation to get the Greeks;\n(c) checking \
any results given in the basic research against the output from ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ";",
  StyleBox["\n(",
    FontSlant->"Italic"],
  "d) debugging as necessary;\n(e) visualization of the results to check that \
they are intuitive;\n(f) investigation of any odd phenomena;\n(g) \
application.\n\nLater on, when one is required to develop a large numerically \
based system, perhaps in C/C++, to carry out similar calculations (we have to \
ask, why bother, for this type of calculation - it is all easy in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "!), one can refer back to this type of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " implementation to check the results, and supply any test documentation \
for internal or external approval processes. \n\n",
  StyleBox[
  "The Barrier options are sufficiently complex that reliable computation of \
the Greeks is analytically quite burdensome - we invite you to notice how \
this is trivialized using the symbolic power of Mathematica.",
    FontSlant->"Italic"],
  "\n\nThe material presented in this chapter arose out of a real-world \
desire to create a benchmark model as quickly as possible, to test other \
models of Barrier options. It was created literally in the form presented \
here, by making direct use of known results for European Barrier options. \n\n\
A good reason for investigating Barrier options is that they represent a very \
simple situation where it is possible for Gamma and/or Vega to be negative. \
In simple European vanilla Call or Put options these quantities are strictly \
non-negative. Roughly speaking, the concave structure of the payoffs is \
preserved by the time evolution of such vanilla options, so that Gamma is \
positive, and an increase in volatility increase the chances of a positive \
payout at maturity. In the case of a Barrier option, the payoff need not be \
concave, and an increase in volatility can enhance the probability of a \
knock-out, thereby potentially causing a decrease in value. We shall use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to exhibit these phenomena directly.\n\nThe fact that Gamma can have \
either sign is of considerable significance, especially when one comes to \
consider the incorporation of transaction costs, for the simple model of \
costs requiring only an adjustment of volatility fails to work.\n\nThe \
formulae for Barrier options with no rebate are given by Hull in the latest \
version of his book (Hull, 1996). However, here we shall work with the model \
with rebates, as given by Rubenstein and Reiner in an article in ",
  StyleBox["RISK Magazine",
    FontSlant->"Italic"],
  " (1991) and various conferences. A detailed derivation of these results \
for the case of a down-and-out Call was given in Chapter 4, based on the PDE \
approach. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["8.2 Mathematica Implementation", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Loading the Standard Euro-options Package", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
All the Barrier options involve adding a function to Black-Scholes \
in order to satisfy the appropriate Barrier condition, and use the cumulative \
normal distribution. We therefore load the standard European option tools \
first:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Needs[\"Derivatives`BlackScholes`\"]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["?Norm", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \("Norm[x] returns the cumulative normal distribution function evaluated \
at x."\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Modifications for Barriers", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "The following allow a rebate and are based on Rubenstein and Reiner \
(1991). We use continuously compounded variables. In all cases, we have:\n\n",
  
  Cell[BoxData[
      \(TraditionalForm\`r\)]],
  " = risk-free rate;\n",
  Cell[BoxData[
      \(TraditionalForm\`q\)]],
  " = continuous dividend yield;\n",
  Cell[BoxData[
      \(TraditionalForm\`sd\)]],
  " = volatility;\n",
  Cell[BoxData[
      \(TraditionalForm\`t\)]],
  " = expiry time in years;\n",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  " = stock price;\n",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " = strike;\n",
  Cell[BoxData[
      \(TraditionalForm\`h\)]],
  " = barrier."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "The Barrier options are all functions of the following basic functions",
  ". The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " inpu",
  "t",
  " ",
  "cells which follow have been converted to Standard Form in order to make \
them easier to read."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(\[Lambda][r_, q_, sd_] := 1\/2 + \(r - q\)\/sd\^2; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(x[p_, k_, sd_, r_, q_, t_] := 
      Log[p\/k]\/\(sd\ \@t\) + \[Lambda][r, q, sd]\ sd\ \@t; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(xone[p_, h_, sd_, r_, q_, t_] := x[p, h, sd, r, q, t]; \)\)], "Input",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(y[p_, k_, h_, sd_, r_, q_, t_] := 
      Log[h\^2\/\(p\ k\)]\/\(sd\ \@t\) + \[Lambda][r, q, sd]\ sd\ \@t; \)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(yone[p_, h_, sd_, r_, q_, t_] := x[h, p, sd, r, q, t]; \)\)], "Input",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(a[r_, q_, sd_] := \(r - q\)\/sd\^2 - 1\/2; \)\), 
    \(\(b[r_, q_, 
        sd_] := \@\(\((r - q - sd\^2\/2)\)\^2 + 2\ r\ sd\^2\)\/sd\^2; \)\), 
    \(\(z[p_, h_, sd_, r_, q_, t_] := 
      Log[h\/p]\/\(sd\ \@t\) + b[r, q, sd]\ sd\ \@t; \)\)}], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(intone[p_, k_, sd_, r_, q_, 
        t_, \[Phi]_] := \[Phi]\ p\ Exp[\(-q\)\ t]\ Norm[\[Phi]\ x[p, k, sd, 
                r, q, t]] - \[Phi]\ k\ Exp[\(-r\)\ t]\ Norm[\[Phi]\ x[p, k, 
                  sd, r, q, t] - \[Phi]\ sd\ \@t]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(inttwo[p_, k_, h_, sd_, r_, q_, 
        t_, \[Phi]_] := \[Phi]\ p\ Exp[\(-q\)\ t]\ Norm[\[Phi]\ xone[p, h, 
                sd, r, q, 
                t]] - \[Phi]\ k\ Exp[\(-r\)\ t]\ Norm[\[Phi]\ xone[p, h, sd, 
                  r, q, t] - \[Phi]\ sd\ \@t]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(intthree[p_, k_, h_, sd_, r_, q_, 
        t_, \[Phi]_, \[Eta]_] := \[Phi]\ p\ Exp[\(-q\)\ t]\ \((h\/p)\)\^\(2\ \
\[Lambda][r, q, sd]\)\ Norm[\[Eta]\ y[p, k, h, sd, r, q, 
                t]] - \[Phi]\ k\ Exp[\(-r\)\ t]\ \((h\/p)\)\^\(2\ \
\[Lambda][r, q, sd] - 2\)\ Norm[\[Eta]\ y[p, k, h, sd, r, q, 
                  t] - \[Eta]\ sd\ \@t]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(intfour[p_, k_, h_, sd_, r_, q_, 
        t_, \[Phi]_, \[Eta]_] := \[Phi]\ p\ Exp[\(-q\)\ t]\ \((h\/p)\)\^\(2\ \
\[Lambda][r, q, sd]\)\ Norm[\[Eta]\ yone[p, h, sd, r, q, 
                t]] - \[Phi]\ k\ Exp[\(-r\)\ t]\ \((h\/p)\)\^\(2\ \
\[Lambda][r, q, sd] - 2\)\ Norm[\[Eta]\ yone[p, h, sd, r, q, 
                  t] - \[Eta]\ sd\ \@t]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(intfive[reb_, p_, h_, sd_, r_, q_, t_, \[Eta]_] := 
      reb\ Exp[\(-r\)\ t]\ \((Norm[\[Eta]\ xone[p, h, sd, r, q, 
                    t] - \[Eta]\ sd\ \@t] - \((h\/p)\)\^\(2\ \[Lambda][r, q, \
sd] - 2\)\ Norm[\[Eta]\ yone[p, h, sd, r, q, t] - \[Eta]\ sd\ \@t])\)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(intsix[reb_, p_, h_, sd_, r_, q_, t_, \[Eta]_] := 
      reb\ \((\((h\/p)\)\^\(a[r, q, sd] + b[r, q, sd]\)\ Norm[\[Eta]\ z[p, h, 
                    sd, r, q, 
                    t]] + \((h\/p)\)\^\(a[r, q, sd] - b[r, q, sd]\)\ Norm[\
\[Eta]\ z[p, h, sd, r, q, t] - 2\ \[Eta]\ b[r, q, sd]\ sd\ \@t])\)\)], "Input",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["8.3 Up/Down and In (Knock-In) Calls", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Our first task is to define our functions and to do numerical and \
graphical checks against the original Rubenstein and Reiner paper. First we \
introduce the value and price sensitivity functions. Readers should note the \
use of ",
  StyleBox["Evaluate",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " here. It forces the action of the ",
  StyleBox["D",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " operator. Note also that the ",
  StyleBox["D",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " has to be applied within the ",
  StyleBox["If",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " statement."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Down and In Calls", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
DownAndInCall[reb_, p_, k_, h_, sd_, r_, q_, t_] :=
If[k >= h, intthree[p, k, h, sd, r, q, t, 1, 1] + intfive[reb, p, h, sd, r, \
q, t, 1], 
intone[p, k, sd, r, q, t, 1] - inttwo[p, k, h, sd, r, q, t, 1] + intfour[p, \
k, h, sd, r, q, t, 1, 1] + intfive[reb, p, h, sd, r, q, t, 1]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndInCallDelta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intthree[p, k, h, sd, r, q, t, 1, 1] + intfive[reb, p, \
h, sd, r, q, t, 1], p]],
Evaluate[D[intone[p, k, sd, r, q, t, 1] - inttwo[p, k, h, sd, r, q, t, 1] + \
intfour[p, k, h, sd, r, q, t, 1, 1] + intfive[reb, p, h, sd, r, q, t, 1], \
p]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndInCallGamma[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intthree[p, k, h, sd, r, q, t, 1, 1] + intfive[reb, p, \
h, sd, r, q, t, 1], {p,2}]],
Evaluate[D[intone[p, k, sd, r, q, t, 1] - inttwo[p, k, h, sd, r, q, t, 1] + \
intfour[p, k, h, sd, r, q, t, 1, 1] + intfive[reb, p, h, sd, r, q, t, 1], \
{p,2}]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndInCallTheta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[-D[intthree[p, k, h, sd, r, q, t, 1, 1] + intfive[reb, p, \
h, sd, r, q, t, 1], t]],
Evaluate[-D[intone[p, k, sd, r, q, t, 1] - inttwo[p, k, h, sd, r, q, t, 1] + \
intfour[p, k, h, sd, r, q, t, 1, 1] + intfive[reb, p, h, sd, r, q, t, 1], \
t]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndInCallVega[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intthree[p, k, h, sd, r, q, t, 1, 1] + intfive[reb, p, \
h, sd, r, q, t, 1], sd]],
Evaluate[D[intone[p, k, sd, r, q, t, 1] - inttwo[p, k, h, sd, r, q, t, 1] + \
intfour[p, k, h, sd, r, q, t, 1, 1] + intfive[reb, p, h, sd, r, q, t, 1], \
sd]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndInCallRho[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intthree[p, k, h, sd, r, q, t, 1, 1] + intfive[reb, p, \
h, sd, r, q, t, 1], r]],
Evaluate[D[intone[p, k, sd, r, q, t, 1] - inttwo[p, k, h, sd, r, q, t, 1] + \
intfour[p, k, h, sd, r, q, t, 1, 1] + intfive[reb, p, h, sd, r, q, t, 1], \
r]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9]
}, Open  ]],

Cell[CellGroupData[{

Cell["Up and In Calls", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
UpAndInCall[reb_, p_, k_, h_, sd_, r_, q_, t_] :=
If[k >= h, intone[p, k, sd, r, q, t, 1] + intfive[reb, p, h, sd, r, q, t, \
-1],
inttwo[p, k, h, sd, r, q, t, 1] - intthree[p, k, h, sd, r, q, t, 1, \
-1]+intfour[p, k, h, sd, r, q, t, 1, -1] + intfive[reb, p, h, sd, r, q, t, \
-1]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndInCallDelta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, 1] + intfive[reb, p, h, sd, \
r, q, t, -1], p]],
Evaluate[D[inttwo[p, k, h, sd, r, q, t, 1] - intthree[p, k, h, sd, r, q, t, \
1, -1]+ intfour[p, k, h, sd, r, q, t, 1, -1] + intfive[reb, p, h, sd, r, q, \
t, -1], p]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndInCallGamma[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, 1] + intfive[reb, p, h, sd, \
r, q, t, -1], {p,2}]],
Evaluate[D[inttwo[p, k, h, sd, r, q, t, 1] - intthree[p, k, h, sd, r, q, t, \
1, -1]+ intfour[p, k, h, sd, r, q, t, 1, -1] + intfive[reb, p, h, sd, r, q, \
t, -1], {p,2}]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndInCallTheta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[-D[intone[p, k, sd, r, q, t, 1] + intfive[reb, p, h, sd, \
r, q, t, -1], t]],
Evaluate[-D[inttwo[p, k, h, sd, r, q, t, 1] - intthree[p, k, h, sd, r, q, t, \
1, -1]+ intfour[p, k, h, sd, r, q, t, 1, -1] + intfive[reb, p, h, sd, r, q, \
t, -1], t]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndInCallVega[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, 1] + intfive[reb, p, h, sd, \
r, q, t, -1], sd]],
Evaluate[D[inttwo[p, k, h, sd, r, q, t, 1] - intthree[p, k, h, sd, r, q, t, \
1, -1]+ intfour[p, k, h, sd, r, q, t, 1, -1] + intfive[reb, p, h, sd, r, q, \
t, -1], sd]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndInCallRho[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, 1] + intfive[reb, p, h, sd, \
r, q, t, -1], r]],
Evaluate[D[inttwo[p, k, h, sd, r, q, t, 1] - intthree[p, k, h, sd, r, q, t, \
1, -1]+ intfour[p, k, h, sd, r, q, t, 1, -1] + intfive[reb, p, h, sd, r, q, \
t, -1], r]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
The following tables of values can be checked directly against a \
table given by Rubenstein and Reiner. Note the conversion to annual rates for \
the interest rates and yield.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
TableForm[Table[N[DownAndInCall[2,100,k,97,0.20,Log[1.1],Log[1.05],\
t]], {k, 90, 110, 10}, {t, 0.5, 1.5, 0.25}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"8.54546926978186327`", "9.9673104623274682`", 
            "11.2000626826833449`", "12.2995178236062496`", 
            "13.2959732729216706`"},
          {"3.91066603931580836`", "5.30465658565594111`", 
            "6.55242612523746625`", "7.6874264763783815`", 
            "8.73126371436038795`"},
          {"1.59872052903291201`", "2.5771335831727562`", 
            "3.57656631394136637`", "4.55321055028629562`", 
            "5.49370529561058074`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
TableForm[Table[N[DownAndInCall[2,100,k,100,0.20,Log[1.1],Log[1.05],\
t]], {k, 90, 110, 10}, {t, 0.5, 1.5, 0.25}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"13.0537212816941949`", "14.4510837771139932`", 
            "15.6916425807581632`", "16.8086696860359623`", 
            "17.8250050156922634`"},
          {"6.63323601474891688`", "8.32391479627339236`", 
            "9.77728793659991879`", "11.0687051348176845`", 
            "12.2375813699729363`"},
          {"2.78482997209708571`", "4.27027631942470264`", 
            "5.6243747348147437`", "6.87002319587264054`", 
            "8.02475655589035596`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
TableForm[Table[N[UpAndInCall[2,100,k,103,0.20,Log[1.1],Log[1.05],t]\
], {k, 90, 110, 10}, {t, 0.5, 1.5, 0.25}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"13.0850939714896608`", "14.5316501582285635`", 
            "15.7834954878938527`", "16.9008877837898596`", 
            "17.9138957041031581`"},
          {"6.91517019832690849`", "8.54393429492127864`", 
            "9.95959655184614334`", "11.2249377872408873`", 
            "12.3743988096560553`"},
          {"3.07013391439043914`", "4.49209982720444855`", 
            "5.80783037153994996`", "7.02705776867246356`", 
            "8.16216948695430666`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell["\<\
We can also plot the time-price plots of the value and Delta and \
Gamma:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[If[S>95, 
DownAndInCall[2,S,100,95,0.2,Log[1.1],Log[1.05],t],
UpAndInCall[2,S,100,95,0.2,Log[1.1],Log[1.05],t]], {S, 80, 120}, {t, 0.0001, \
1}, BoxRatios -> {1,1,1}, PlotPoints -> 40];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.0855 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0567234 1.16608 2.22045e-16 1.16608 [
[.07825 .23514 -11.8839 -9 ]
[.07825 .23514 .11606 0 ]
[.21247 .17844 -11.3335 -9 ]
[.21247 .17844 .66648 0 ]
[.3545 .11847 -16.1747 -9 ]
[.3545 .11847 1.82534 0 ]
[.50502 .05496 -15.349 -9 ]
[.50502 .05496 2.65097 0 ]
[.66483 -0.01242 -14.5234 -9 ]
[.66483 -0.01242 3.4766 0 ]
[.09301 .88495 -10.1462 0 ]
[.09301 .88495 7.85383 9 ]
[.17991 .94587 -9.90019 0 ]
[.17991 .94587 8.09981 9 ]
[.25944 1.00163 -9.68252 0 ]
[.25944 1.00163 8.31748 9 ]
[.33251 1.05285 -9.48854 0 ]
[.33251 1.05285 8.51146 9 ]
[.39987 1.10006 -3.10486 0 ]
[.39987 1.10006 2.89514 9 ]
[.07325 .26228 -6 -2.6177 ]
[.07325 .26228 0 6.3823 ]
[.05137 .39926 -6 -2.73944 ]
[.05137 .39926 0 6.26056 ]
[.02721 .55049 -6 -2.87518 ]
[.02721 .55049 0 6.12482 ]
[.0004 .71833 -6 -3.02745 ]
[.0004 .71833 0 5.97255 ]
[ 0 0 0 0 ]
[ 1 1.0855 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
[(80)] .07825 .23514 .98066 1 Mshowa
.22216 .18933 m
.227 .19478 L
s
[(90)] .21247 .17844 .88892 1 Mshowa
.36358 .12987 m
.36813 .13557 L
s
[(100)] .3545 .11847 .79718 1 Mshowa
.51343 .06687 m
.51763 .07282 L
s
[(110)] .50502 .05496 .70545 1 Mshowa
.67245 0 m
.67627 .00621 L
s
[(120)] .66483 -0.01242 .61371 1 Mshowa
.125 Mabswid
.11461 .23456 m
.11764 .23771 L
s
.14105 .22344 m
.14405 .22662 L
s
.16779 .2122 m
.17076 .21541 L
s
.19482 .20083 m
.19776 .20407 L
s
.2498 .17771 m
.25268 .18101 L
s
.27776 .16595 m
.2806 .16928 L
s
.30604 .15406 m
.30885 .15742 L
s
.33465 .14204 m
.33741 .14542 L
s
.39286 .11756 m
.39554 .12101 L
s
.42247 .10511 m
.42512 .10859 L
s
.45243 .09251 m
.45504 .09602 L
s
.48275 .07977 m
.48531 .08331 L
s
.54447 .05381 m
.54695 .05742 L
s
.57589 .0406 m
.57832 .04424 L
s
.60769 .02723 m
.61007 .0309 L
s
.63987 .0137 m
.64221 .0174 L
s
.25 Mabswid
0 .80374 m
.40037 1.0855 L
s
.09485 .87049 m
.09577 .86326 L
s
[(0.2)] .09301 .88495 .12735 -1 Mshowa
.18136 .93137 m
.18209 .92412 L
s
[(0.4)] .17991 .94587 .10002 -1 Mshowa
.26055 .98709 m
.2611 .97983 L
s
[(0.6)] .25944 1.00163 .07584 -1 Mshowa
.3333 1.03829 m
.33369 1.03102 L
s
[(0.8)] .33251 1.05285 .05428 -1 Mshowa
.40037 1.0855 m
.40063 1.07821 L
s
[(1)] .39987 1.10006 .03495 -1 Mshowa
.125 Mabswid
.02453 .821 m
.02518 .81668 L
s
.04852 .83789 m
.04914 .83356 L
s
.07196 .85438 m
.07254 .85004 L
s
.11722 .88623 m
.11774 .88189 L
s
.13908 .90162 m
.13958 .89727 L
s
.16046 .91666 m
.16092 .91231 L
s
.20181 .94576 m
.20221 .9414 L
s
.22181 .95983 m
.22219 .95548 L
s
.24138 .97361 m
.24174 .96925 L
s
.27931 1.0003 m
.27961 .99593 L
s
.29768 1.01322 m
.29796 1.00886 L
s
.31567 1.02589 m
.31593 1.02152 L
s
.35057 1.05045 m
.35079 1.04608 L
s
.3675 1.06236 m
.3677 1.058 L
s
.3841 1.07404 m
.38427 1.06967 L
s
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
.0867 .25665 m
.09342 .25384 L
s
[(0)] .07325 .26228 1 -0.41829 Mshowa
.06494 .39395 m
.07173 .39129 L
s
[(2)] .05137 .39926 1 -0.39123 Mshowa
.04092 .54554 m
.04777 .54307 L
s
[(4)] .02721 .55049 1 -0.36107 Mshowa
.01425 .71379 m
.02118 .71153 L
s
[(6)] .0004 .71833 1 -0.32723 Mshowa
.125 Mabswid
.08145 .28975 m
.0855 .28808 L
s
.07608 .32364 m
.08013 .322 L
s
.07058 .35837 m
.07464 .35675 L
s
.05916 .43042 m
.06324 .42886 L
s
.05323 .46782 m
.05732 .46628 L
s
.04715 .50618 m
.05126 .50467 L
s
.03451 .58594 m
.03864 .58449 L
s
.02794 .62742 m
.03208 .626 L
s
.02119 .67002 m
.02534 .66863 L
s
.00712 .75878 m
.01129 .75746 L
s
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
.41799 .58158 m
.08846 .24555 L
s
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.41799 .58158 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
0 0 m
1 0 L
1 1.0855 L
0 1.0855 L
closepath
clip
newpath
.5 Mabswid
.242 .454 .866 r
.40407 .75813 .41144 .76884 .42371 .77577 .41635 .76463 Metetra
.181 .407 .846 r
.41635 .76463 .42371 .77577 .43609 .78399 .42874 .7724 Metetra
.114 .357 .821 r
.42874 .7724 .43609 .78399 .44859 .79357 .44124 .78153 Metetra
.044 .303 .791 r
.44124 .78153 .44859 .79357 .46123 .80461 .45388 .79212 Metetra
0 .246 .757 r
.45388 .79212 .46123 .80461 .47403 .81722 .46666 .80426 Metetra
0 .187 .719 r
.46666 .80426 .47403 .81722 .487 .83148 .47962 .81805 Metetra
0 .127 .676 r
.47962 .81805 .487 .83148 .50016 .84751 .49276 .8336 Metetra
0 .068 .631 r
.49276 .8336 .50016 .84751 .51355 .86541 .50612 .85101 Metetra
0 .01 .585 r
.50612 .85101 .51355 .86541 .52718 .88528 .51971 .8704 Metetra
0 0 .537 r
.51971 .8704 .52718 .88528 .54108 .90724 .53357 .89188 Metetra
0 0 .49 r
.53357 .89188 .54108 .90724 .55528 .9314 .54772 .91557 Metetra
0 0 .444 r
.54772 .91557 .55528 .9314 .56981 .9579 .5622 .94159 Metetra
0 0 .4 r
.5622 .94159 .56981 .9579 .58472 .98686 .57704 .97007 Metetra
0 0 .358 r
.57704 .97007 .58472 .98686 .60002 1.01842 .59228 1.00117 Metetra
.505 .517 .803 r
.59228 1.00117 .60002 1.01842 .61501 1.02196 .60725 1.00462 Metetra
.866 .684 .654 r
.60725 1.00462 .61501 1.02196 .62875 .97827 .6211 .96144 Metetra
.861 .685 .662 r
.6211 .96144 .62875 .97827 .64237 .93873 .63483 .92238 Metetra
.855 .686 .671 r
.63483 .92238 .64237 .93873 .65591 .90287 .64846 .887 Metetra
.85 .687 .68 r
.64846 .887 .65591 .90287 .66938 .87029 .66203 .85487 Metetra
.844 .688 .69 r
.66203 .85487 .66938 .87029 .68283 .84065 .67556 .82566 Metetra
.838 .69 .699 r
.67556 .82566 .68283 .84065 .69626 .81363 .68908 .79905 Metetra
.831 .691 .709 r
.68908 .79905 .69626 .81363 .70969 .78894 .7026 .77476 Metetra
.825 .693 .718 r
.7026 .77476 .70969 .78894 .72315 .76636 .71614 .75255 Metetra
.818 .695 .728 r
.71614 .75255 .72315 .76636 .73664 .74566 .72971 .73221 Metetra
.811 .696 .738 r
.72971 .73221 .73664 .74566 .75017 .72665 .74333 .71354 Metetra
.804 .698 .748 r
.74333 .71354 .75017 .72665 .76376 .70916 .757 .69637 Metetra
.797 .7 .758 r
.757 .69637 .76376 .70916 .77742 .69303 .77073 .68055 Metetra
.79 .702 .768 r
.77073 .68055 .77742 .69303 .79114 .67812 .78453 .66592 Metetra
.783 .704 .777 r
.78453 .66592 .79114 .67812 .80493 .66431 .79841 .65238 Metetra
.775 .706 .786 r
.79841 .65238 .80493 .66431 .81881 .65148 .81236 .63981 Metetra
.768 .708 .795 r
.81236 .63981 .81881 .65148 .83278 .63954 .8264 .62811 Metetra
.761 .71 .804 r
.8264 .62811 .83278 .63954 .84683 .62839 .84053 .61718 Metetra
.754 .712 .812 r
.84053 .61718 .84683 .62839 .86097 .61794 .85475 .60695 Metetra
.748 .713 .82 r
.85475 .60695 .86097 .61794 .87521 .60814 .86907 .59733 Metetra
.741 .715 .827 r
.86907 .59733 .87521 .60814 .88954 .5989 .88348 .58827 Metetra
.735 .717 .834 r
.88348 .58827 .88954 .5989 .90396 .59017 .89798 .57971 Metetra
.729 .719 .841 r
.89798 .57971 .90396 .59017 .91849 .58188 .91259 .57158 Metetra
.723 .721 .847 r
.91259 .57158 .91849 .58188 .93311 .57401 .92729 .56384 Metetra
.718 .722 .853 r
.92729 .56384 .93311 .57401 .94784 .56649 .94209 .55644 Metetra
.266 .469 .871 r
.39664 .74739 .40407 .75813 .41635 .76463 .40894 .75347 Metetra
.206 .424 .852 r
.40894 .75347 .41635 .76463 .42874 .7724 .42134 .76079 Metetra
.141 .375 .829 r
.42134 .76079 .42874 .7724 .44124 .78153 .43384 .76946 Metetra
.072 .322 .801 r
.43384 .76946 .44124 .78153 .45388 .79212 .44648 .77957 Metetra
.001 .265 .769 r
.44648 .77957 .45388 .79212 .46666 .80426 .45925 .79123 Metetra
0 .207 .732 r
.45925 .79123 .46666 .80426 .47962 .81805 .4722 .80454 Metetra
0 .147 .69 r
.4722 .80454 .47962 .81805 .49276 .8336 .48532 .8196 Metetra
0 .087 .646 r
.48532 .8196 .49276 .8336 .50612 .85101 .49865 .83652 Metetra
0 .029 .6 r
.49865 .83652 .50612 .85101 .51971 .8704 .51221 .85541 Metetra
0 0 .553 r
.51221 .85541 .51971 .8704 .53357 .89188 .52602 .8764 Metetra
0 0 .505 r
.52602 .8764 .53357 .89188 .54772 .91557 .54013 .89959 Metetra
0 0 .459 r
.54013 .89959 .54772 .91557 .5622 .94159 .55455 .92512 Metetra
0 0 .414 r
.55455 .92512 .5622 .94159 .57704 .97007 .56933 .95312 Metetra
0 0 .371 r
.56933 .95312 .57704 .97007 .59228 1.00117 .58449 .98373 Metetra
.506 .517 .802 r
.58449 .98373 .59228 1.00117 .60725 1.00462 .59946 .98709 Metetra
.864 .683 .655 r
.59946 .98709 .60725 1.00462 .6211 .96144 .61342 .94443 Metetra
.859 .684 .664 r
.61342 .94443 .6211 .96144 .63483 .92238 .62724 .90588 Metetra
.853 .685 .672 r
.62724 .90588 .63483 .92238 .64846 .887 .64097 .87097 Metetra
.848 .686 .681 r
.64097 .87097 .64846 .887 .66203 .85487 .65464 .83931 Metetra
.842 .687 .691 r
.65464 .83931 .66203 .85487 .67556 .82566 .66826 .81054 Metetra
.835 .689 .701 r
.66826 .81054 .67556 .82566 .68908 .79905 .68186 .78435 Metetra
.829 .69 .71 r
.68186 .78435 .68908 .79905 .7026 .77476 .69547 .76047 Metetra
.822 .692 .72 r
.69547 .76047 .7026 .77476 .71614 .75255 .70909 .73865 Metetra
.815 .694 .73 r
.70909 .73865 .71614 .75255 .72971 .73221 .72274 .71867 Metetra
.808 .695 .74 r
.72274 .71867 .72971 .73221 .74333 .71354 .73644 .70035 Metetra
.801 .697 .75 r
.73644 .70035 .74333 .71354 .757 .69637 .75019 .68351 Metetra
.794 .699 .76 r
.75019 .68351 .757 .69637 .77073 .68055 .764 .66799 Metetra
.787 .701 .77 r
.764 .66799 .77073 .68055 .78453 .66592 .77788 .65367 Metetra
.78 .703 .779 r
.77788 .65367 .78453 .66592 .79841 .65238 .79183 .6404 Metetra
.772 .705 .789 r
.79183 .6404 .79841 .65238 .81236 .63981 .80587 .62809 Metetra
.765 .707 .797 r
.80587 .62809 .81236 .63981 .8264 .62811 .81999 .61663 Metetra
.758 .709 .806 r
.81999 .61663 .8264 .62811 .84053 .61718 .83419 .60593 Metetra
.751 .711 .814 r
.83419 .60593 .84053 .61718 .85475 .60695 .84849 .59591 Metetra
.745 .713 .822 r
.84849 .59591 .85475 .60695 .86907 .59733 .86289 .58649 Metetra
.738 .715 .83 r
.86289 .58649 .86907 .59733 .88348 .58827 .87737 .57761 Metetra
.732 .717 .837 r
.87737 .57761 .88348 .58827 .89798 .57971 .89196 .56921 Metetra
.726 .719 .843 r
.89196 .56921 .89798 .57971 .91259 .57158 .90664 .56124 Metetra
.72 .721 .85 r
.90664 .56124 .91259 .57158 .92729 .56384 .92142 .55364 Metetra
.715 .722 .855 r
.92142 .55364 .92729 .56384 .94209 .55644 .9363 .54637 Metetra
.29 .484 .875 r
.38916 .73665 .39664 .74739 .40894 .75347 .40148 .74228 Metetra
.232 .44 .858 r
.40148 .74228 .40894 .75347 .42134 .76079 .41388 .74914 Metetra
.168 .392 .837 r
.41388 .74914 .42134 .76079 .43384 .76946 .4264 .75734 Metetra
.101 .34 .811 r
.4264 .75734 .43384 .76946 .44648 .77957 .43903 .76697 Metetra
.03 .285 .78 r
.43903 .76697 .44648 .77957 .45925 .79123 .4518 .77814 Metetra
0 .227 .744 r
.4518 .77814 .45925 .79123 .4722 .80454 .46473 .79095 Metetra
0 .167 .704 r
.46473 .79095 .4722 .80454 .48532 .8196 .47784 .80551 Metetra
0 .107 .661 r
.47784 .80551 .48532 .8196 .49865 .83652 .49114 .82192 Metetra
0 .048 .616 r
.49114 .82192 .49865 .83652 .51221 .85541 .50467 .8403 Metetra
0 0 .568 r
.50467 .8403 .51221 .85541 .52602 .8764 .51845 .86078 Metetra
0 0 .521 r
.51845 .86078 .52602 .8764 .54013 .89959 .5325 .88346 Metetra
0 0 .474 r
.5325 .88346 .54013 .89959 .55455 .92512 .54687 .90849 Metetra
0 0 .428 r
.54687 .90849 .55455 .92512 .56933 .95312 .56158 .93599 Metetra
0 0 .385 r
.56158 .93599 .56933 .95312 .58449 .98373 .57668 .9661 Metetra
.507 .516 .801 r
.57668 .9661 .58449 .98373 .59946 .98709 .59164 .96936 Metetra
.863 .682 .656 r
.59164 .96936 .59946 .98709 .61342 .94443 .6057 .92724 Metetra
.857 .683 .665 r
.6057 .92724 .61342 .94443 .62724 .90588 .61962 .88921 Metetra
.852 .684 .674 r
.61962 .88921 .62724 .90588 .64097 .87097 .63345 .8548 Metetra
.846 .685 .683 r
.63345 .8548 .64097 .87097 .65464 .83931 .6472 .82361 Metetra
.84 .686 .692 r
.6472 .82361 .65464 .83931 .66826 .81054 .66091 .79529 Metetra
.833 .688 .702 r
.66091 .79529 .66826 .81054 .68186 .78435 .6746 .76954 Metetra
.827 .689 .712 r
.6746 .76954 .68186 .78435 .69547 .76047 .68829 .74607 Metetra
.82 .691 .722 r
.68829 .74607 .69547 .76047 .70909 .73865 .70199 .72464 Metetra
.813 .693 .732 r
.70199 .72464 .70909 .73865 .72274 .71867 .71573 .70504 Metetra
.806 .694 .742 r
.71573 .70504 .72274 .71867 .73644 .70035 .7295 .68708 Metetra
.798 .696 .752 r
.7295 .68708 .73644 .70035 .75019 .68351 .74333 .67057 Metetra
.791 .698 .762 r
.74333 .67057 .75019 .68351 .764 .66799 .75722 .65538 Metetra
.784 .7 .772 r
.75722 .65538 .764 .66799 .77788 .65367 .77118 .64135 Metetra
.776 .703 .782 r
.77118 .64135 .77788 .65367 .79183 .6404 .78521 .62837 Metetra
.769 .705 .791 r
.78521 .62837 .79183 .6404 .80587 .62809 .79932 .61632 Metetra
.762 .707 .8 r
.79932 .61632 .80587 .62809 .81999 .61663 .81352 .60511 Metetra
.755 .709 .809 r
.81352 .60511 .81999 .61663 .83419 .60593 .82781 .59464 Metetra
.748 .711 .817 r
.82781 .59464 .83419 .60593 .84849 .59591 .84219 .58483 Metetra
.741 .713 .825 r
.84219 .58483 .84849 .59591 .86289 .58649 .85666 .57561 Metetra
.735 .715 .832 r
.85666 .57561 .86289 .58649 .87737 .57761 .87122 .56692 Metetra
.729 .717 .839 r
.87122 .56692 .87737 .57761 .89196 .56921 .88589 .55869 Metetra
.723 .719 .846 r
.88589 .55869 .89196 .56921 .90664 .56124 .90065 .55086 Metetra
.717 .721 .852 r
.90065 .55086 .90664 .56124 .92142 .55364 .91551 .5434 Metetra
.712 .722 .858 r
.91551 .5434 .92142 .55364 .9363 .54637 .93047 .53626 Metetra
.313 .498 .879 r
.38163 .72589 .38916 .73665 .40148 .74228 .39396 .73107 Metetra
.257 .456 .864 r
.39396 .73107 .40148 .74228 .41388 .74914 .40638 .73747 Metetra
.195 .41 .844 r
.40638 .73747 .41388 .74914 .4264 .75734 .41891 .74519 Metetra
.129 .359 .82 r
.41891 .74519 .4264 .75734 .43903 .76697 .43154 .75432 Metetra
.059 .305 .791 r
.43154 .75432 .43903 .76697 .4518 .77814 .44431 .76499 Metetra
0 .247 .757 r
.44431 .76499 .4518 .77814 .46473 .79095 .45723 .77729 Metetra
0 .188 .718 r
.45723 .77729 .46473 .79095 .47784 .80551 .47032 .79133 Metetra
0 .127 .676 r
.47032 .79133 .47784 .80551 .49114 .82192 .4836 .80722 Metetra
0 .067 .631 r
.4836 .80722 .49114 .82192 .50467 .8403 .49709 .82508 Metetra
0 .009 .584 r
.49709 .82508 .50467 .8403 .51845 .86078 .51083 .84503 Metetra
0 0 .537 r
.51083 .84503 .51845 .86078 .5325 .88346 .52484 .86719 Metetra
0 0 .49 r
.52484 .86719 .5325 .88346 .54687 .90849 .53915 .89169 Metetra
0 0 .443 r
.53915 .89169 .54687 .90849 .56158 .93599 .5538 .91868 Metetra
0 0 .399 r
.5538 .91868 .56158 .93599 .57668 .9661 .56883 .94828 Metetra
.508 .516 .799 r
.56883 .94828 .57668 .9661 .59164 .96936 .58378 .95144 Metetra
.861 .681 .657 r
.58378 .95144 .59164 .96936 .6057 .92724 .59795 .90987 Metetra
.855 .682 .666 r
.59795 .90987 .6057 .92724 .61962 .88921 .61197 .87237 Metetra
.85 .683 .675 r
.61197 .87237 .61962 .88921 .63345 .8548 .62588 .83847 Metetra
.844 .684 .684 r
.62588 .83847 .63345 .8548 .6472 .82361 .63973 .80777 Metetra
.837 .685 .694 r
.63973 .80777 .6472 .82361 .66091 .79529 .65352 .77992 Metetra
.831 .687 .704 r
.65352 .77992 .66091 .79529 .6746 .76954 .6673 .75461 Metetra
.824 .688 .714 r
.6673 .75461 .6746 .76954 .68829 .74607 .68107 .73156 Metetra
.817 .69 .724 r
.68107 .73156 .68829 .74607 .70199 .72464 .69485 .71054 Metetra
.81 .692 .734 r
.69485 .71054 .70199 .72464 .71573 .70504 .70867 .69132 Metetra
.803 .694 .744 r
.70867 .69132 .71573 .70504 .7295 .68708 .72252 .67372 Metetra
.795 .696 .755 r
.72252 .67372 .7295 .68708 .74333 .67057 .73643 .65757 Metetra
.788 .698 .765 r
.73643 .65757 .74333 .67057 .75722 .65538 .7504 .6427 Metetra
.781 .7 .774 r
.7504 .6427 .75722 .65538 .77118 .64135 .76443 .62898 Metetra
.773 .702 .784 r
.76443 .62898 .77118 .64135 .78521 .62837 .77854 .61629 Metetra
.766 .704 .794 r
.77854 .61629 .78521 .62837 .79932 .61632 .79274 .60451 Metetra
.759 .706 .803 r
.79274 .60451 .79932 .61632 .81352 .60511 .80701 .59355 Metetra
.752 .708 .811 r
.80701 .59355 .81352 .60511 .82781 .59464 .82138 .58331 Metetra
.745 .711 .82 r
.82138 .58331 .82781 .59464 .84219 .58483 .83583 .57372 Metetra
.738 .713 .827 r
.83583 .57372 .84219 .58483 .85666 .57561 .85038 .5647 Metetra
.732 .715 .835 r
.85038 .5647 .85666 .57561 .87122 .56692 .86503 .55619 Metetra
.726 .717 .842 r
.86503 .55619 .87122 .56692 .88589 .55869 .87977 .54813 Metetra
.72 .719 .848 r
.87977 .54813 .88589 .55869 .90065 .55086 .89461 .54046 Metetra
.714 .721 .854 r
.89461 .54046 .90065 .55086 .91551 .5434 .90955 .53314 Metetra
.709 .722 .86 r
.90955 .53314 .91551 .5434 .93047 .53626 .92459 .52613 Metetra
.336 .512 .883 r
.37404 .71513 .38163 .72589 .39396 .73107 .3864 .71985 Metetra
.282 .472 .869 r
.3864 .71985 .39396 .73107 .40638 .73747 .39883 .72577 Metetra
.222 .427 .851 r
.39883 .72577 .40638 .73747 .41891 .74519 .41136 .733 Metetra
.157 .378 .828 r
.41136 .733 .41891 .74519 .43154 .75432 .42401 .74163 Metetra
.088 .324 .801 r
.42401 .74163 .43154 .75432 .44431 .76499 .43677 .75178 Metetra
.016 .268 .768 r
.43677 .75178 .44431 .76499 .45723 .77729 .44968 .76355 Metetra
0 .208 .731 r
.44968 .76355 .45723 .77729 .47032 .79133 .46276 .77706 Metetra
0 .148 .691 r
.46276 .77706 .47032 .79133 .4836 .80722 .47601 .79241 Metetra
0 .088 .646 r
.47601 .79241 .4836 .80722 .49709 .82508 .48948 .80974 Metetra
0 .028 .6 r
.48948 .80974 .49709 .82508 .51083 .84503 .50318 .82915 Metetra
0 0 .553 r
.50318 .82915 .51083 .84503 .52484 .86719 .51714 .85077 Metetra
0 0 .505 r
.51714 .85077 .52484 .86719 .53915 .89169 .5314 .87474 Metetra
0 0 .459 r
.5314 .87474 .53915 .89169 .5538 .91868 .54599 .90119 Metetra
0 0 .413 r
.54599 .90119 .5538 .91868 .56883 .94828 .56094 .93026 Metetra
.509 .515 .798 r
.56094 .93026 .56883 .94828 .58378 .95144 .57589 .93332 Metetra
.859 .679 .658 r
.57589 .93332 .58378 .95144 .59795 .90987 .59016 .89232 Metetra
.853 .68 .667 r
.59016 .89232 .59795 .90987 .61197 .87237 .60427 .85536 Metetra
.847 .681 .676 r
.60427 .85536 .61197 .87237 .62588 .83847 .61828 .82198 Metetra
.841 .683 .686 r
.61828 .82198 .62588 .83847 .63973 .80777 .63221 .79178 Metetra
.835 .684 .695 r
.63221 .79178 .63973 .80777 .65352 .77992 .6461 .76441 Metetra
.828 .685 .705 r
.6461 .76441 .65352 .77992 .6673 .75461 .65995 .73956 Metetra
.821 .687 .715 r
.65995 .73956 .6673 .75461 .68107 .73156 .6738 .71695 Metetra
.814 .689 .726 r
.6738 .71695 .68107 .73156 .69485 .71054 .68767 .69634 Metetra
.807 .691 .736 r
.68767 .69634 .69485 .71054 .70867 .69132 .70156 .67752 Metetra
.8 .693 .747 r
.70156 .67752 .70867 .69132 .72252 .67372 .7155 .6603 Metetra
.792 .695 .757 r
.7155 .6603 .72252 .67372 .73643 .65757 .72948 .64449 Metetra
.785 .697 .767 r
.72948 .64449 .73643 .65757 .7504 .6427 .74353 .62996 Metetra
.777 .699 .777 r
.74353 .62996 .7504 .6427 .76443 .62898 .75764 .61655 Metetra
.77 .701 .787 r
.75764 .61655 .76443 .62898 .77854 .61629 .77183 .60415 Metetra
.763 .703 .796 r
.77183 .60415 .77854 .61629 .79274 .60451 .7861 .59265 Metetra
.755 .706 .805 r
.7861 .59265 .79274 .60451 .80701 .59355 .80046 .58195 Metetra
.748 .708 .814 r
.80046 .58195 .80701 .59355 .82138 .58331 .8149 .57195 Metetra
.741 .71 .822 r
.8149 .57195 .82138 .58331 .83583 .57372 .82943 .56258 Metetra
.735 .712 .83 r
.82943 .56258 .83583 .57372 .85038 .5647 .84406 .55376 Metetra
.728 .715 .838 r
.84406 .55376 .85038 .5647 .86503 .55619 .85879 .54544 Metetra
.722 .717 .844 r
.85879 .54544 .86503 .55619 .87977 .54813 .87361 .53755 Metetra
.716 .719 .851 r
.87361 .53755 .87977 .54813 .89461 .54046 .88853 .53003 Metetra
.711 .721 .857 r
.88853 .53003 .89461 .54046 .90955 .53314 .90355 .52285 Metetra
.706 .722 .862 r
.90355 .52285 .90955 .53314 .92459 .52613 .91867 .51596 Metetra
.359 .526 .886 r
.3664 .70437 .37404 .71513 .3864 .71985 .37877 .70862 Metetra
.306 .488 .873 r
.37877 .70862 .3864 .71985 .39883 .72577 .39123 .71405 Metetra
.248 .444 .857 r
.39123 .71405 .39883 .72577 .41136 .733 .40377 .72078 Metetra
.185 .396 .836 r
.40377 .72078 .41136 .733 .42401 .74163 .41642 .72889 Metetra
.118 .344 .81 r
.41642 .72889 .42401 .74163 .43677 .75178 .42919 .73851 Metetra
.046 .288 .78 r
.42919 .73851 .43677 .75178 .44968 .76355 .4421 .74975 Metetra
0 .229 .744 r
.4421 .74975 .44968 .76355 .46276 .77706 .45516 .76271 Metetra
0 .169 .705 r
.45516 .76271 .46276 .77706 .47601 .79241 .46839 .77751 Metetra
0 .108 .662 r
.46839 .77751 .47601 .79241 .48948 .80974 .48183 .79428 Metetra
0 .048 .616 r
.48183 .79428 .48948 .80974 .50318 .82915 .49549 .81313 Metetra
0 0 .569 r
.49549 .81313 .50318 .82915 .51714 .85077 .50941 .8342 Metetra
0 0 .521 r
.50941 .8342 .51714 .85077 .5314 .87474 .52362 .85762 Metetra
0 0 .474 r
.52362 .85762 .5314 .87474 .54599 .90119 .53814 .88352 Metetra
0 0 .428 r
.53814 .88352 .54599 .90119 .56094 .93026 .55303 .91205 Metetra
.511 .515 .796 r
.55303 .91205 .56094 .93026 .57589 .93332 .56796 .91499 Metetra
.857 .678 .659 r
.56796 .91499 .57589 .93332 .59016 .89232 .58233 .87458 Metetra
.851 .679 .668 r
.58233 .87458 .59016 .89232 .60427 .85536 .59654 .83818 Metetra
.845 .68 .677 r
.59654 .83818 .60427 .85536 .61828 .82198 .61064 .80534 Metetra
.839 .681 .687 r
.61064 .80534 .61828 .82198 .63221 .79178 .62466 .77566 Metetra
.832 .683 .697 r
.62466 .77566 .63221 .79178 .6461 .76441 .63863 .74878 Metetra
.826 .684 .707 r
.63863 .74878 .6461 .76441 .65995 .73956 .65256 .72439 Metetra
.819 .686 .717 r
.65256 .72439 .65995 .73956 .6738 .71695 .6665 .70223 Metetra
.812 .688 .728 r
.6665 .70223 .6738 .71695 .68767 .69634 .68044 .68206 Metetra
.804 .69 .738 r
.68044 .68206 .68767 .69634 .70156 .67752 .69441 .66364 Metetra
.797 .692 .749 r
.69441 .66364 .70156 .67752 .7155 .6603 .70843 .6468 Metetra
.789 .694 .759 r
.70843 .6468 .7155 .6603 .72948 .64449 .72249 .63136 Metetra
.782 .696 .769 r
.72249 .63136 .72948 .64449 .74353 .62996 .73661 .61716 Metetra
.774 .698 .78 r
.73661 .61716 .74353 .62996 .75764 .61655 .7508 .60408 Metetra
.766 .701 .789 r
.7508 .60408 .75764 .61655 .77183 .60415 .76507 .59198 Metetra
.759 .703 .799 r
.76507 .59198 .77183 .60415 .7861 .59265 .77942 .58076 Metetra
.752 .705 .808 r
.77942 .58076 .7861 .59265 .80046 .58195 .79385 .57031 Metetra
.745 .708 .817 r
.79385 .57031 .80046 .58195 .8149 .57195 .80837 .56056 Metetra
.738 .71 .825 r
.80837 .56056 .8149 .57195 .82943 .56258 .82299 .55141 Metetra
.731 .712 .833 r
.82299 .55141 .82943 .56258 .84406 .55376 .83769 .5428 Metetra
.725 .714 .84 r
.83769 .5428 .84406 .55376 .85879 .54544 .8525 .53466 Metetra
.719 .717 .847 r
.8525 .53466 .85879 .54544 .87361 .53755 .8674 .52694 Metetra
.713 .719 .854 r
.8674 .52694 .87361 .53755 .88853 .53003 .8824 .51958 Metetra
.708 .721 .86 r
.8824 .51958 .88853 .53003 .90355 .52285 .8975 .51254 Metetra
.703 .723 .865 r
.8975 .51254 .90355 .52285 .91867 .51596 .9127 .50577 Metetra
.381 .54 .888 r
.35869 .69361 .3664 .70437 .37877 .70862 .37109 .69738 Metetra
.331 .503 .877 r
.37109 .69738 .37877 .70862 .39123 .71405 .38357 .70232 Metetra
.275 .461 .862 r
.38357 .70232 .39123 .71405 .40377 .72078 .39613 .70853 Metetra
.213 .415 .843 r
.39613 .70853 .40377 .72078 .41642 .72889 .40879 .71612 Metetra
.147 .364 .819 r
.40879 .71612 .41642 .72889 .42919 .73851 .42156 .7252 Metetra
.077 .309 .79 r
.42156 .7252 .42919 .73851 .4421 .74975 .43446 .73588 Metetra
.004 .251 .757 r
.43446 .73588 .4421 .74975 .45516 .76271 .44751 .74827 Metetra
0 .19 .719 r
.44751 .74827 .45516 .76271 .46839 .77751 .46073 .76251 Metetra
0 .129 .677 r
.46073 .76251 .46839 .77751 .48183 .79428 .47414 .77871 Metetra
0 .069 .632 r
.47414 .77871 .48183 .79428 .49549 .81313 .48777 .79699 Metetra
0 .01 .585 r
.48777 .79699 .49549 .81313 .50941 .8342 .50165 .81748 Metetra
0 0 .537 r
.50165 .81748 .50941 .8342 .52362 .85762 .5158 .84033 Metetra
0 0 .49 r
.5158 .84033 .52362 .85762 .53814 .88352 .53027 .86566 Metetra
0 0 .443 r
.53027 .86566 .53814 .88352 .55303 .91205 .54508 .89364 Metetra
.512 .514 .795 r
.54508 .89364 .55303 .91205 .56796 .91499 .56001 .89646 Metetra
.855 .677 .66 r
.56001 .89646 .56796 .91499 .58233 .87458 .57447 .85665 Metetra
.849 .678 .669 r
.57447 .85665 .58233 .87458 .59654 .83818 .58878 .82083 Metetra
.843 .679 .679 r
.58878 .82083 .59654 .83818 .61064 .80534 .60296 .78854 Metetra
.837 .68 .688 r
.60296 .78854 .61064 .80534 .62466 .77566 .61707 .75939 Metetra
.83 .681 .699 r
.61707 .75939 .62466 .77566 .63863 .74878 .63111 .73301 Metetra
.823 .683 .709 r
.63111 .73301 .63863 .74878 .65256 .72439 .64513 .70912 Metetra
.816 .685 .719 r
.64513 .70912 .65256 .72439 .6665 .70223 .65915 .68742 Metetra
.809 .687 .73 r
.65915 .68742 .6665 .70223 .68044 .68206 .67317 .66768 Metetra
.801 .689 .741 r
.67317 .66768 .68044 .68206 .69441 .66364 .68722 .64968 Metetra
.793 .691 .751 r
.68722 .64968 .69441 .66364 .70843 .6468 .70131 .63323 Metetra
.786 .693 .762 r
.70131 .63323 .70843 .6468 .72249 .63136 .71545 .61816 Metetra
.778 .695 .772 r
.71545 .61816 .72249 .63136 .73661 .61716 .72965 .60431 Metetra
.77 .698 .782 r
.72965 .60431 .73661 .61716 .7508 .60408 .74392 .59155 Metetra
.763 .7 .792 r
.74392 .59155 .7508 .60408 .76507 .59198 .75826 .57976 Metetra
.755 .702 .802 r
.75826 .57976 .76507 .59198 .77942 .58076 .77269 .56883 Metetra
.748 .705 .811 r
.77269 .56883 .77942 .58076 .79385 .57031 .7872 .55865 Metetra
.741 .707 .82 r
.7872 .55865 .79385 .57031 .80837 .56056 .8018 .54914 Metetra
.734 .71 .828 r
.8018 .54914 .80837 .56056 .82299 .55141 .81649 .54021 Metetra
.727 .712 .836 r
.81649 .54021 .82299 .55141 .83769 .5428 .83128 .53181 Metetra
.721 .714 .843 r
.83128 .53181 .83769 .5428 .8525 .53466 .84616 .52386 Metetra
.715 .717 .85 r
.84616 .52386 .8525 .53466 .8674 .52694 .86114 .51631 Metetra
.71 .719 .856 r
.86114 .51631 .8674 .52694 .8824 .51958 .87622 .5091 Metetra
.705 .721 .862 r
.87622 .5091 .8824 .51958 .8975 .51254 .8914 .5022 Metetra
.7 .723 .868 r
.8914 .5022 .8975 .51254 .9127 .50577 .90668 .49556 Metetra
.403 .553 .891 r
.35093 .68286 .35869 .69361 .37109 .69738 .36336 .68615 Metetra
.355 .518 .88 r
.36336 .68615 .37109 .69738 .38357 .70232 .37586 .69058 Metetra
.301 .478 .867 r
.37586 .69058 .38357 .70232 .39613 .70853 .38844 .69627 Metetra
.241 .433 .849 r
.38844 .69627 .39613 .70853 .40879 .71612 .40111 .70331 Metetra
.177 .383 .827 r
.40111 .70331 .40879 .71612 .42156 .7252 .41389 .71183 Metetra
.108 .329 .8 r
.41389 .71183 .42156 .7252 .43446 .73588 .42679 .72194 Metetra
.035 .272 .769 r
.42679 .72194 .43446 .73588 .44751 .74827 .43983 .73376 Metetra
0 .212 .732 r
.43983 .73376 .44751 .74827 .46073 .76251 .45303 .74741 Metetra
0 .151 .692 r
.45303 .74741 .46073 .76251 .47414 .77871 .46642 .76302 Metetra
0 .09 .648 r
.46642 .76302 .47414 .77871 .48777 .79699 .48001 .78071 Metetra
0 .03 .601 r
.48001 .78071 .48777 .79699 .50165 .81748 .49385 .80061 Metetra
0 0 .554 r
.49385 .80061 .50165 .81748 .5158 .84033 .50795 .82287 Metetra
0 0 .506 r
.50795 .82287 .5158 .84033 .53027 .86566 .52236 .84762 Metetra
0 0 .459 r
.52236 .84762 .53027 .86566 .54508 .89364 .5371 .87502 Metetra
.513 .513 .793 r
.5371 .87502 .54508 .89364 .56001 .89646 .55202 .87773 Metetra
.853 .676 .661 r
.55202 .87773 .56001 .89646 .57447 .85665 .56658 .83853 Metetra
.847 .676 .67 r
.56658 .83853 .57447 .85665 .58878 .82083 .58097 .8033 Metetra
.841 .677 .68 r
.58097 .8033 .58878 .82083 .60296 .78854 .59525 .77158 Metetra
.834 .679 .69 r
.59525 .77158 .60296 .78854 .61707 .75939 .60943 .74297 Metetra
.827 .68 .7 r
.60943 .74297 .61707 .75939 .63111 .73301 .62356 .71712 Metetra
.82 .682 .711 r
.62356 .71712 .63111 .73301 .64513 .70912 .63766 .69372 Metetra
.813 .683 .721 r
.63766 .69372 .64513 .70912 .65915 .68742 .65175 .6725 Metetra
.805 .685 .732 r
.65175 .6725 .65915 .68742 .67317 .66768 .66585 .65321 Metetra
.798 .687 .743 r
.66585 .65321 .67317 .66768 .68722 .64968 .67998 .63564 Metetra
.79 .69 .754 r
.67998 .63564 .68722 .64968 .70131 .63323 .69415 .61959 Metetra
.782 .692 .764 r
.69415 .61959 .70131 .63323 .71545 .61816 .70836 .6049 Metetra
.774 .694 .775 r
.70836 .6049 .71545 .61816 .72965 .60431 .72264 .59141 Metetra
.767 .697 .785 r
.72264 .59141 .72965 .60431 .74392 .59155 .73699 .57899 Metetra
.759 .699 .795 r
.73699 .57899 .74392 .59155 .75826 .57976 .75141 .56751 Metetra
.752 .702 .805 r
.75141 .56751 .75826 .57976 .77269 .56883 .76591 .55687 Metetra
.744 .704 .814 r
.76591 .55687 .77269 .56883 .7872 .55865 .7805 .54696 Metetra
.737 .707 .823 r
.7805 .54696 .7872 .55865 .8018 .54914 .79518 .53769 Metetra
.73 .709 .831 r
.79518 .53769 .8018 .54914 .81649 .54021 .80995 .529 Metetra
.724 .712 .839 r
.80995 .529 .81649 .54021 .83128 .53181 .82481 .5208 Metetra
.718 .714 .846 r
.82481 .5208 .83128 .53181 .84616 .52386 .83978 .51304 Metetra
.712 .717 .853 r
.83978 .51304 .84616 .52386 .86114 .51631 .85484 .50566 Metetra
.706 .719 .859 r
.85484 .50566 .86114 .51631 .87622 .5091 .87 .49861 Metetra
.701 .721 .865 r
.87 .49861 .87622 .5091 .8914 .5022 .88526 .49185 Metetra
.697 .723 .87 r
.88526 .49185 .8914 .5022 .90668 .49556 .90062 .48533 Metetra
.424 .566 .892 r
.3431 .67213 .35093 .68286 .36336 .68615 .35556 .67493 Metetra
.378 .532 .883 r
.35556 .67493 .36336 .68615 .37586 .69058 .36809 .67884 Metetra
.327 .494 .871 r
.36809 .67884 .37586 .69058 .38844 .69627 .38069 .68399 Metetra
.269 .45 .855 r
.38069 .68399 .38844 .69627 .40111 .70331 .39338 .69048 Metetra
.206 .402 .835 r
.39338 .69048 .40111 .70331 .41389 .71183 .40617 .69842 Metetra
.139 .35 .81 r
.40617 .69842 .41389 .71183 .42679 .72194 .41907 .70795 Metetra
.067 .293 .78 r
.41907 .70795 .42679 .72194 .43983 .73376 .4321 .71918 Metetra
0 .234 .745 r
.4321 .71918 .43983 .73376 .45303 .74741 .44529 .73222 Metetra
0 .173 .706 r
.44529 .73222 .45303 .74741 .46642 .76302 .45866 .74722 Metetra
0 .111 .663 r
.45866 .74722 .46642 .76302 .48001 .78071 .47222 .7643 Metetra
0 .051 .618 r
.47222 .7643 .48001 .78071 .49385 .80061 .48602 .7836 Metetra
0 0 .57 r
.48602 .7836 .49385 .80061 .50795 .82287 .50007 .80524 Metetra
0 0 .522 r
.50007 .80524 .50795 .82287 .52236 .84762 .51442 .82939 Metetra
0 0 .475 r
.51442 .82939 .52236 .84762 .5371 .87502 .52909 .85619 Metetra
.515 .513 .792 r
.52909 .85619 .5371 .87502 .55202 .87773 .544 .85877 Metetra
.851 .674 .662 r
.544 .85877 .55202 .87773 .56658 .83853 .55865 .82022 Metetra
.845 .675 .672 r
.55865 .82022 .56658 .83853 .58097 .8033 .57313 .7856 Metetra
.838 .676 .681 r
.57313 .7856 .58097 .8033 .59525 .77158 .58749 .75447 Metetra
.831 .677 .692 r
.58749 .75447 .59525 .77158 .60943 .74297 .60176 .72642 Metetra
.824 .679 .702 r
.60176 .72642 .60943 .74297 .62356 .71712 .61597 .70111 Metetra
.817 .68 .713 r
.61597 .70111 .62356 .71712 .63766 .69372 .63015 .67822 Metetra
.81 .682 .723 r
.63015 .67822 .63766 .69372 .65175 .6725 .64431 .65748 Metetra
.802 .684 .734 r
.64431 .65748 .65175 .6725 .66585 .65321 .65849 .63866 Metetra
.794 .686 .745 r
.65849 .63866 .66585 .65321 .67998 .63564 .67269 .62152 Metetra
.786 .689 .756 r
.67269 .62152 .67998 .63564 .69415 .61959 .68694 .60589 Metetra
.779 .691 .767 r
.68694 .60589 .69415 .61959 .70836 .6049 .70123 .59159 Metetra
.771 .694 .778 r
.70123 .59159 .70836 .6049 .72264 .59141 .71558 .57847 Metetra
.763 .696 .788 r
.71558 .57847 .72264 .59141 .73699 .57899 .73 .56639 Metetra
.755 .699 .798 r
.73 .56639 .73699 .57899 .75141 .56751 .7445 .55523 Metetra
.748 .701 .808 r
.7445 .55523 .75141 .56751 .76591 .55687 .75908 .54488 Metetra
.74 .704 .817 r
.75908 .54488 .76591 .55687 .7805 .54696 .77375 .53524 Metetra
.733 .707 .826 r
.77375 .53524 .7805 .54696 .79518 .53769 .78851 .52623 Metetra
.726 .709 .834 r
.78851 .52623 .79518 .53769 .80995 .529 .80336 .51777 Metetra
.72 .712 .842 r
.80336 .51777 .80995 .529 .82481 .5208 .8183 .50978 Metetra
.714 .714 .849 r
.8183 .50978 .82481 .5208 .83978 .51304 .83334 .50221 Metetra
.708 .717 .856 r
.83334 .50221 .83978 .51304 .85484 .50566 .84848 .495 Metetra
.703 .719 .862 r
.84848 .495 .85484 .50566 .87 .49861 .86372 .4881 Metetra
.698 .721 .868 r
.86372 .4881 .87 .49861 .88526 .49185 .87906 .48147 Metetra
.694 .724 .873 r
.87906 .48147 .88526 .49185 .90062 .48533 .8945 .47507 Metetra
.445 .579 .894 r
.33521 .66142 .3431 .67213 .35556 .67493 .34771 .66372 Metetra
.401 .546 .886 r
.34771 .66372 .35556 .67493 .36809 .67884 .36027 .6671 Metetra
.352 .509 .875 r
.36027 .6671 .36809 .67884 .38069 .68399 .37289 .6717 Metetra
.297 .468 .86 r
.37289 .6717 .38069 .68399 .39338 .69048 .3856 .67762 Metetra
.236 .421 .842 r
.3856 .67762 .39338 .69048 .40617 .69842 .3984 .68498 Metetra
.17 .37 .819 r
.3984 .68498 .40617 .69842 .41907 .70795 .41131 .6939 Metetra
.099 .315 .791 r
.41131 .6939 .41907 .70795 .4321 .71918 .42434 .70452 Metetra
.026 .256 .758 r
.42434 .70452 .4321 .71918 .44529 .73222 .43751 .71694 Metetra
0 .196 .72 r
.43751 .71694 .44529 .73222 .45866 .74722 .45086 .73132 Metetra
0 .134 .678 r
.45086 .73132 .45866 .74722 .47222 .7643 .46439 .74776 Metetra
0 .072 .634 r
.46439 .74776 .47222 .7643 .48602 .7836 .47815 .76643 Metetra
0 .012 .587 r
.47815 .76643 .48602 .7836 .50007 .80524 .49216 .78745 Metetra
0 0 .539 r
.49216 .78745 .50007 .80524 .51442 .82939 .50645 .81097 Metetra
0 0 .491 r
.50645 .81097 .51442 .82939 .52909 .85619 .52106 .83715 Metetra
.516 .512 .79 r
.52106 .83715 .52909 .85619 .544 .85877 .53595 .83961 Metetra
.849 .673 .664 r
.53595 .83961 .544 .85877 .55865 .82022 .55069 .8017 Metetra
.842 .674 .673 r
.55069 .8017 .55865 .82022 .57313 .7856 .56526 .76772 Metetra
.836 .675 .683 r
.56526 .76772 .57313 .7856 .58749 .75447 .5797 .73719 Metetra
.829 .676 .693 r
.5797 .73719 .58749 .75447 .60176 .72642 .59405 .70972 Metetra
.821 .677 .704 r
.59405 .70972 .60176 .72642 .61597 .70111 .60834 .68496 Metetra
.814 .679 .715 r
.60834 .68496 .61597 .70111 .63015 .67822 .62259 .6626 Metetra
.806 .681 .726 r
.62259 .6626 .63015 .67822 .64431 .65748 .63683 .64237 Metetra
.799 .683 .737 r
.63683 .64237 .64431 .65748 .65849 .63866 .65108 .62402 Metetra
.791 .685 .748 r
.65108 .62402 .65849 .63866 .67269 .62152 .66536 .60734 Metetra
.783 .688 .759 r
.66536 .60734 .67269 .62152 .68694 .60589 .67968 .59214 Metetra
.775 .69 .77 r
.67968 .59214 .68694 .60589 .70123 .59159 .69405 .57824 Metetra
.767 .693 .781 r
.69405 .57824 .70123 .59159 .71558 .57847 .70848 .56549 Metetra
.759 .695 .791 r
.70848 .56549 .71558 .57847 .73 .56639 .72297 .55376 Metetra
.751 .698 .801 r
.72297 .55376 .73 .56639 .7445 .55523 .73755 .54292 Metetra
.743 .701 .811 r
.73755 .54292 .7445 .55523 .75908 .54488 .75221 .53287 Metetra
.736 .704 .82 r
.75221 .53287 .75908 .54488 .77375 .53524 .76695 .52351 Metetra
.729 .707 .829 r
.76695 .52351 .77375 .53524 .78851 .52623 .78179 .51475 Metetra
.722 .709 .838 r
.78179 .51475 .78851 .52623 .80336 .51777 .79671 .50652 Metetra
.716 .712 .845 r
.79671 .50652 .80336 .51777 .8183 .50978 .81174 .49875 Metetra
.71 .715 .852 r
.81174 .49875 .8183 .50978 .83334 .50221 .82686 .49137 Metetra
.704 .717 .859 r
.82686 .49137 .83334 .50221 .84848 .495 .84208 .48433 Metetra
.699 .72 .865 r
.84208 .48433 .84848 .495 .86372 .4881 .8574 .47758 Metetra
.695 .722 .871 r
.8574 .47758 .86372 .4881 .87906 .48147 .87282 .47109 Metetra
.69 .724 .876 r
.87282 .47109 .87906 .48147 .8945 .47507 .88834 .46481 Metetra
.465 .591 .894 r
.32725 .65074 .33521 .66142 .34771 .66372 .33979 .65253 Metetra
.424 .56 .887 r
.33979 .65253 .34771 .66372 .36027 .6671 .35238 .65538 Metetra
.377 .525 .878 r
.35238 .65538 .36027 .6671 .37289 .6717 .36504 .65941 Metetra
.324 .485 .865 r
.36504 .65941 .37289 .6717 .3856 .67762 .37777 .66475 Metetra
.265 .44 .848 r
.37777 .66475 .3856 .67762 .3984 .68498 .39058 .67151 Metetra
.201 .39 .827 r
.39058 .67151 .3984 .68498 .41131 .6939 .4035 .67981 Metetra
.132 .336 .8 r
.4035 .67981 .41131 .6939 .42434 .70452 .41653 .68979 Metetra
.059 .279 .769 r
.41653 .68979 .42434 .70452 .43751 .71694 .4297 .70158 Metetra
0 .218 .734 r
.4297 .70158 .43751 .71694 .45086 .73132 .44302 .7153 Metetra
0 .156 .693 r
.44302 .7153 .45086 .73132 .46439 .74776 .45653 .7311 Metetra
0 .094 .65 r
.45653 .7311 .46439 .74776 .47815 .76643 .47025 .74911 Metetra
0 .033 .603 r
.47025 .74911 .47815 .76643 .49216 .78745 .48421 .76948 Metetra
0 0 .556 r
.48421 .76948 .49216 .78745 .50645 .81097 .49845 .79235 Metetra
0 0 .508 r
.49845 .79235 .50645 .81097 .52106 .83715 .51299 .8179 Metetra
.518 .512 .788 r
.51299 .8179 .52106 .83715 .53595 .83961 .52787 .82022 Metetra
.846 .671 .665 r
.52787 .82022 .53595 .83961 .55069 .8017 .5427 .78299 Metetra
.84 .672 .674 r
.5427 .78299 .55069 .8017 .56526 .76772 .55735 .74965 Metetra
.833 .673 .685 r
.55735 .74965 .56526 .76772 .5797 .73719 .57187 .71975 Metetra
.826 .674 .695 r
.57187 .71975 .5797 .73719 .59405 .70972 .5863 .69288 Metetra
.818 .676 .706 r
.5863 .69288 .59405 .70972 .60834 .68496 .60066 .6687 Metetra
.811 .677 .717 r
.60066 .6687 .60834 .68496 .62259 .6626 .61499 .64688 Metetra
.803 .679 .728 r
.61499 .64688 .62259 .6626 .63683 .64237 .62931 .62717 Metetra
.795 .682 .739 r
.62931 .62717 .63683 .64237 .65108 .62402 .64363 .60931 Metetra
.787 .684 .751 r
.64363 .60931 .65108 .62402 .66536 .60734 .65798 .59309 Metetra
.779 .686 .762 r
.65798 .59309 .66536 .60734 .67968 .59214 .67237 .57832 Metetra
.771 .689 .773 r
.67237 .57832 .67968 .59214 .69405 .57824 .68682 .56484 Metetra
.763 .692 .784 r
.68682 .56484 .69405 .57824 .70848 .56549 .70132 .55247 Metetra
.755 .695 .795 r
.70132 .55247 .70848 .56549 .72297 .55376 .71589 .5411 Metetra
.747 .698 .805 r
.71589 .5411 .72297 .55376 .73755 .54292 .73055 .53059 Metetra
.739 .701 .815 r
.73055 .53059 .73755 .54292 .75221 .53287 .74528 .52085 Metetra
.732 .703 .824 r
.74528 .52085 .75221 .53287 .76695 .52351 .7601 .51177 Metetra
.725 .706 .833 r
.7601 .51177 .76695 .52351 .78179 .51475 .77501 .50327 Metetra
.718 .709 .841 r
.77501 .50327 .78179 .51475 .79671 .50652 .79002 .49527 Metetra
.712 .712 .849 r
.79002 .49527 .79671 .50652 .81174 .49875 .80512 .48771 Metetra
.706 .715 .856 r
.80512 .48771 .81174 .49875 .82686 .49137 .82032 .48052 Metetra
.701 .717 .862 r
.82032 .48052 .82686 .49137 .84208 .48433 .83562 .47365 Metetra
.696 .72 .868 r
.83562 .47365 .84208 .48433 .8574 .47758 .85102 .46706 Metetra
.691 .722 .874 r
.85102 .46706 .8574 .47758 .87282 .47109 .86653 .46069 Metetra
.687 .725 .878 r
.86653 .46069 .87282 .47109 .88834 .46481 .88213 .45453 Metetra
.485 .603 .895 r
.31923 .6401 .32725 .65074 .33979 .65253 .33181 .64137 Metetra
.446 .574 .889 r
.33181 .64137 .33979 .65253 .35238 .65538 .34444 .64368 Metetra
.401 .54 .88 r
.34444 .64368 .35238 .65538 .36504 .65941 .35712 .64713 Metetra
.351 .502 .869 r
.35712 .64713 .36504 .65941 .37777 .66475 .36988 .65187 Metetra
.294 .459 .853 r
.36988 .65187 .37777 .66475 .39058 .67151 .38271 .65801 Metetra
.232 .411 .834 r
.38271 .65801 .39058 .67151 .4035 .67981 .39564 .66568 Metetra
.165 .358 .81 r
.39564 .66568 .4035 .67981 .41653 .68979 .40867 .67501 Metetra
.093 .301 .781 r
.40867 .67501 .41653 .68979 .4297 .70158 .42184 .68613 Metetra
.018 .242 .747 r
.42184 .68613 .4297 .70158 .44302 .7153 .43515 .69918 Metetra
0 .18 .708 r
.43515 .69918 .44302 .7153 .45653 .7311 .44864 .7143 Metetra
0 .117 .666 r
.44864 .7143 .45653 .7311 .47025 .74911 .46232 .73164 Metetra
0 .055 .62 r
.46232 .73164 .47025 .74911 .48421 .76948 .47624 .75133 Metetra
0 0 .573 r
.47624 .75133 .48421 .76948 .49845 .79235 .49042 .77354 Metetra
0 0 .525 r
.49042 .77354 .49845 .79235 .51299 .8179 .5049 .79842 Metetra
.52 .511 .787 r
.5049 .79842 .51299 .8179 .52787 .82022 .51977 .8006 Metetra
.844 .669 .666 r
.51977 .8006 .52787 .82022 .5427 .78299 .53468 .76407 Metetra
.837 .67 .676 r
.53468 .76407 .5427 .78299 .55735 .74965 .54941 .73141 Metetra
.83 .671 .686 r
.54941 .73141 .55735 .74965 .57187 .71975 .56401 .70215 Metetra
.823 .673 .697 r
.56401 .70215 .57187 .71975 .5863 .69288 .57851 .6759 Metetra
.815 .674 .708 r
.57851 .6759 .5863 .69288 .60066 .6687 .59294 .65231 Metetra
.807 .676 .719 r
.59294 .65231 .60066 .6687 .61499 .64688 .60735 .63105 Metetra
.799 .678 .73 r
.60735 .63105 .61499 .64688 .62931 .62717 .62173 .61188 Metetra
.791 .68 .742 r
.62173 .61188 .62931 .62717 .64363 .60931 .63613 .59452 Metetra
.783 .683 .753 r
.63613 .59452 .64363 .60931 .65798 .59309 .65056 .57878 Metetra
.775 .685 .765 r
.65056 .57878 .65798 .59309 .67237 .57832 .66502 .56447 Metetra
.766 .688 .776 r
.66502 .56447 .67237 .57832 .68682 .56484 .67954 .5514 Metetra
.758 .691 .787 r
.67954 .5514 .68682 .56484 .70132 .55247 .69412 .53943 Metetra
.75 .694 .798 r
.69412 .53943 .70132 .55247 .71589 .5411 .70877 .52842 Metetra
.742 .697 .808 r
.70877 .52842 .71589 .5411 .73055 .53059 .72349 .51825 Metetra
.735 .7 .818 r
.72349 .51825 .73055 .53059 .74528 .52085 .7383 .50882 Metetra
.727 .703 .828 r
.7383 .50882 .74528 .52085 .7601 .51177 .7532 .50003 Metetra
.72 .706 .836 r
.7532 .50003 .7601 .51177 .77501 .50327 .76819 .49179 Metetra
.714 .709 .845 r
.76819 .49179 .77501 .50327 .79002 .49527 .78328 .48402 Metetra
.708 .712 .852 r
.78328 .48402 .79002 .49527 .80512 .48771 .79846 .47667 Metetra
.702 .715 .859 r
.79846 .47667 .80512 .48771 .82032 .48052 .81374 .46967 Metetra
.697 .718 .866 r
.81374 .46967 .82032 .48052 .83562 .47365 .82912 .46297 Metetra
.692 .721 .871 r
.82912 .46297 .83562 .47365 .85102 .46706 .8446 .45653 Metetra
.688 .723 .877 r
.8446 .45653 .85102 .46706 .86653 .46069 .86018 .45029 Metetra
.684 .725 .881 r
.86018 .45029 .86653 .46069 .88213 .45453 .87587 .44424 Metetra
.504 .615 .895 r
.31114 .62951 .31923 .6401 .33181 .64137 .32376 .63025 Metetra
.467 .587 .89 r
.32376 .63025 .33181 .64137 .34444 .64368 .33643 .632 Metetra
.425 .555 .882 r
.33643 .632 .34444 .64368 .35712 .64713 .34915 .63487 Metetra
.377 .518 .872 r
.34915 .63487 .35712 .64713 .36988 .65187 .36194 .639 Metetra
.323 .477 .858 r
.36194 .639 .36988 .65187 .38271 .65801 .37479 .6445 Metetra
.263 .43 .84 r
.37479 .6445 .38271 .65801 .39564 .66568 .38773 .65151 Metetra
.197 .379 .818 r
.38773 .65151 .39564 .66568 .40867 .67501 .40078 .66016 Metetra
.127 .324 .791 r
.40078 .66016 .40867 .67501 .42184 .68613 .41394 .6706 Metetra
.053 .265 .759 r
.41394 .6706 .42184 .68613 .43515 .69918 .42724 .68296 Metetra
0 .203 .722 r
.42724 .68296 .43515 .69918 .44864 .7143 .4407 .69738 Metetra
0 .14 .681 r
.4407 .69738 .44864 .7143 .46232 .73164 .45436 .71402 Metetra
0 .078 .637 r
.45436 .71402 .46232 .73164 .47624 .75133 .46824 .73301 Metetra
0 .016 .59 r
.46824 .73301 .47624 .75133 .49042 .77354 .48236 .75453 Metetra
0 0 .542 r
.48236 .75453 .49042 .77354 .5049 .79842 .49678 .77872 Metetra
.521 .511 .785 r
.49678 .77872 .5049 .79842 .51977 .8006 .51163 .78076 Metetra
.841 .668 .667 r
.51163 .78076 .51977 .8006 .53468 .76407 .52663 .74495 Metetra
.834 .668 .677 r
.52663 .74495 .53468 .76407 .54941 .73141 .54144 .71298 Metetra
.827 .67 .688 r
.54144 .71298 .54941 .73141 .56401 .70215 .55611 .68439 Metetra
.819 .671 .699 r
.55611 .68439 .56401 .70215 .57851 .6759 .57068 .65878 Metetra
.811 .672 .71 r
.57068 .65878 .57851 .6759 .59294 .65231 .58519 .63579 Metetra
.803 .674 .721 r
.58519 .63579 .59294 .65231 .60735 .63105 .59966 .61513 Metetra
.795 .676 .733 r
.59966 .61513 .60735 .63105 .62173 .61188 .61412 .5965 Metetra
.787 .679 .745 r
.61412 .5965 .62173 .61188 .63613 .59452 .62859 .57967 Metetra
.779 .681 .756 r
.62859 .57967 .63613 .59452 .65056 .57878 .64308 .56442 Metetra
.77 .684 .768 r
.64308 .56442 .65056 .57878 .66502 .56447 .65762 .55057 Metetra
.762 .687 .78 r
.65762 .55057 .66502 .56447 .67954 .5514 .67221 .53793 Metetra
.754 .69 .791 r
.67221 .53793 .67954 .5514 .69412 .53943 .68686 .52637 Metetra
.745 .693 .802 r
.68686 .52637 .69412 .53943 .70877 .52842 .70158 .51573 Metetra
.737 .697 .812 r
.70158 .51573 .70877 .52842 .72349 .51825 .71639 .50591 Metetra
.73 .7 .822 r
.71639 .50591 .72349 .51825 .7383 .50882 .73128 .49679 Metetra
.723 .703 .831 r
.73128 .49679 .7383 .50882 .7532 .50003 .74625 .48829 Metetra
.716 .706 .84 r
.74625 .48829 .7532 .50003 .76819 .49179 .76132 .48031 Metetra
.709 .71 .848 r
.76132 .48031 .76819 .49179 .78328 .48402 .77648 .47278 Metetra
.703 .713 .856 r
.77648 .47278 .78328 .48402 .79846 .47667 .79174 .46564 Metetra
.698 .716 .863 r
.79174 .46564 .79846 .47667 .81374 .46967 .8071 .45883 Metetra
.693 .718 .869 r
.8071 .45883 .81374 .46967 .82912 .46297 .82256 .4523 Metetra
.688 .721 .875 r
.82256 .4523 .82912 .46297 .8446 .45653 .83812 .44599 Metetra
.684 .724 .88 r
.83812 .44599 .8446 .45653 .86018 .45029 .85379 .43989 Metetra
.681 .726 .884 r
.85379 .43989 .86018 .45029 .87587 .44424 .86955 .43394 Metetra
.523 .626 .895 r
.30297 .61897 .31114 .62951 .32376 .63025 .31564 .61918 Metetra
.488 .6 .891 r
.31564 .61918 .32376 .63025 .33643 .632 .32836 .62036 Metetra
.449 .569 .884 r
.32836 .62036 .33643 .632 .34915 .63487 .34112 .62264 Metetra
.403 .534 .875 r
.34112 .62264 .34915 .63487 .36194 .639 .35394 .62613 Metetra
.351 .495 .862 r
.35394 .62613 .36194 .639 .37479 .6445 .36682 .63098 Metetra
.294 .45 .846 r
.36682 .63098 .37479 .6445 .38773 .65151 .37978 .63731 Metetra
.23 .401 .826 r
.37978 .63731 .38773 .65151 .40078 .66016 .39283 .64527 Metetra
.161 .346 .801 r
.39283 .64527 .40078 .66016 .41394 .6706 .406 .655 Metetra
.088 .288 .771 r
.406 .655 .41394 .6706 .42724 .68296 .41929 .66664 Metetra
.012 .227 .736 r
.41929 .66664 .42724 .68296 .4407 .69738 .43274 .68034 Metetra
0 .164 .696 r
.43274 .68034 .4407 .69738 .45436 .71402 .44637 .69625 Metetra
0 .101 .653 r
.44637 .69625 .45436 .71402 .46824 .73301 .4602 .71452 Metetra
0 .038 .607 r
.4602 .71452 .46824 .73301 .48236 .75453 .47428 .73531 Metetra
0 0 .559 r
.47428 .73531 .48236 .75453 .49678 .77872 .48863 .75879 Metetra
.523 .51 .783 r
.48863 .75879 .49678 .77872 .51163 .78076 .50347 .76067 Metetra
.838 .666 .669 r
.50347 .76067 .51163 .78076 .52663 .74495 .51854 .72562 Metetra
.831 .667 .679 r
.51854 .72562 .52663 .74495 .54144 .71298 .53343 .69437 Metetra
.823 .668 .69 r
.53343 .69437 .54144 .71298 .55611 .68439 .54817 .66647 Metetra
.816 .669 .701 r
.54817 .66647 .55611 .68439 .57068 .65878 .56281 .64152 Metetra
.808 .671 .712 r
.56281 .64152 .57068 .65878 .58519 .63579 .57739 .61917 Metetra
.799 .673 .724 r
.57739 .61917 .58519 .63579 .59966 .61513 .59193 .5991 Metetra
.791 .675 .736 r
.59193 .5991 .59966 .61513 .61412 .5965 .60646 .58104 Metetra
.783 .677 .748 r
.60646 .58104 .61412 .5965 .62859 .57967 .621 .56475 Metetra
.774 .68 .76 r
.621 .56475 .62859 .57967 .64308 .56442 .63556 .55002 Metetra
.766 .683 .772 r
.63556 .55002 .64308 .56442 .65762 .55057 .65017 .53664 Metetra
.757 .686 .783 r
.65017 .53664 .65762 .55057 .67221 .53793 .66483 .52445 Metetra
.749 .689 .795 r
.66483 .52445 .67221 .53793 .68686 .52637 .67955 .51329 Metetra
.74 .693 .806 r
.67955 .51329 .68686 .52637 .70158 .51573 .69435 .50304 Metetra
.733 .696 .816 r
.69435 .50304 .70158 .51573 .71639 .50591 .70923 .49357 Metetra
.725 .7 .826 r
.70923 .49357 .71639 .50591 .73128 .49679 .72419 .48477 Metetra
.718 .703 .835 r
.72419 .48477 .73128 .49679 .74625 .48829 .73925 .47656 Metetra
.711 .707 .844 r
.73925 .47656 .74625 .48829 .76132 .48031 .75439 .46884 Metetra
.705 .71 .852 r
.75439 .46884 .76132 .48031 .77648 .47278 .76964 .46155 Metetra
.699 .713 .86 r
.76964 .46155 .77648 .47278 .79174 .46564 .78498 .45462 Metetra
.694 .716 .866 r
.78498 .45462 .79174 .46564 .8071 .45883 .80042 .448 Metetra
.689 .719 .873 r
.80042 .448 .8071 .45883 .82256 .4523 .81596 .44163 Metetra
.684 .722 .878 r
.81596 .44163 .82256 .4523 .83812 .44599 .8316 .43547 Metetra
.681 .725 .883 r
.8316 .43547 .83812 .44599 .85379 .43989 .84734 .42948 Metetra
.677 .727 .887 r
.84734 .42948 .85379 .43989 .86955 .43394 .86318 .42363 Metetra
.541 .637 .895 r
.29473 .6085 .30297 .61897 .31564 .61918 .30745 .60818 Metetra
.509 .612 .891 r
.30745 .60818 .31564 .61918 .32836 .62036 .32021 .60878 Metetra
.472 .584 .885 r
.32021 .60878 .32836 .62036 .34112 .62264 .33302 .61044 Metetra
.428 .55 .877 r
.33302 .61044 .34112 .62264 .35394 .62613 .34588 .61329 Metetra
.379 .512 .866 r
.34588 .61329 .35394 .62613 .36682 .63098 .35879 .61746 Metetra
.324 .469 .851 r
.35879 .61746 .36682 .63098 .37978 .63731 .37177 .6231 Metetra
.263 .422 .833 r
.37177 .6231 .37978 .63731 .39283 .64527 .38484 .63034 Metetra
.196 .369 .81 r
.38484 .63034 .39283 .64527 .406 .655 .39801 .63934 Metetra
.124 .312 .782 r
.39801 .63934 .406 .655 .41929 .66664 .4113 .65023 Metetra
.048 .252 .748 r
.4113 .65023 .41929 .66664 .43274 .68034 .42474 .66318 Metetra
0 .189 .711 r
.42474 .66318 .43274 .68034 .44637 .69625 .43834 .67833 Metetra
0 .125 .669 r
.43834 .67833 .44637 .69625 .4602 .71452 .45214 .69585 Metetra
0 .062 .623 r
.45214 .69585 .4602 .71452 .47428 .73531 .46617 .71589 Metetra
0 0 .576 r
.46617 .71589 .47428 .73531 .48863 .75879 .48046 .73863 Metetra
.525 .51 .781 r
.48046 .73863 .48863 .75879 .50347 .76067 .49529 .74035 Metetra
.835 .664 .67 r
.49529 .74035 .50347 .76067 .51854 .72562 .51043 .70607 Metetra
.828 .665 .681 r
.51043 .70607 .51854 .72562 .53343 .69437 .52539 .67557 Metetra
.82 .666 .692 r
.52539 .67557 .53343 .69437 .54817 .66647 .5402 .64838 Metetra
.812 .667 .703 r
.5402 .64838 .54817 .66647 .56281 .64152 .55491 .62412 Metetra
.804 .669 .715 r
.55491 .62412 .56281 .64152 .57739 .61917 .56955 .60242 Metetra
.795 .671 .727 r
.56955 .60242 .57739 .61917 .59193 .5991 .58416 .58298 Metetra
.787 .673 .739 r
.58416 .58298 .59193 .5991 .60646 .58104 .59875 .56552 Metetra
.778 .676 .751 r
.59875 .56552 .60646 .58104 .621 .56475 .61336 .54978 Metetra
.769 .679 .763 r
.61336 .54978 .621 .56475 .63556 .55002 .62799 .53557 Metetra
.761 .682 .775 r
.62799 .53557 .63556 .55002 .65017 .53664 .64267 .52268 Metetra
.752 .685 .787 r
.64267 .52268 .65017 .53664 .66483 .52445 .6574 .51095 Metetra
.743 .689 .799 r
.6574 .51095 .66483 .52445 .67955 .51329 .6722 .50021 Metetra
.735 .692 .81 r
.6722 .50021 .67955 .51329 .69435 .50304 .68707 .49035 Metetra
.727 .696 .82 r
.68707 .49035 .69435 .50304 .70923 .49357 .70202 .48123 Metetra
.72 .7 .83 r
.70202 .48123 .70923 .49357 .72419 .48477 .71706 .47276 Metetra
.713 .703 .84 r
.71706 .47276 .72419 .48477 .73925 .47656 .73219 .46484 Metetra
.706 .707 .848 r
.73219 .46484 .73925 .47656 .75439 .46884 .74741 .45739 Metetra
.7 .71 .856 r
.74741 .45739 .75439 .46884 .76964 .46155 .76274 .45034 Metetra
.694 .714 .864 r
.76274 .45034 .76964 .46155 .78498 .45462 .77815 .44362 Metetra
.689 .717 .87 r
.77815 .44362 .78498 .45462 .80042 .448 .79367 .43718 Metetra
.685 .72 .876 r
.79367 .43718 .80042 .448 .81596 .44163 .80929 .43097 Metetra
.681 .723 .881 r
.80929 .43097 .81596 .44163 .8316 .43547 .82501 .42495 Metetra
.677 .726 .886 r
.82501 .42495 .8316 .43547 .84734 .42948 .84084 .41908 Metetra
.674 .728 .89 r
.84084 .41908 .84734 .42948 .86318 .42363 .85676 .41333 Metetra
.558 .648 .894 r
.28641 .59811 .29473 .6085 .30745 .60818 .29919 .59724 Metetra
.529 .625 .891 r
.29919 .59724 .30745 .60818 .32021 .60878 .312 .59726 Metetra
.494 .597 .886 r
.312 .59726 .32021 .60878 .33302 .61044 .32486 .5983 Metetra
.453 .566 .878 r
.32486 .5983 .33302 .61044 .34588 .61329 .33775 .60048 Metetra
.407 .53 .869 r
.33775 .60048 .34588 .61329 .35879 .61746 .3507 .60397 Metetra
.354 .488 .856 r
.3507 .60397 .35879 .61746 .37177 .6231 .36372 .60888 Metetra
.295 .442 .839 r
.36372 .60888 .37177 .6231 .38484 .63034 .37681 .61538 Metetra
.231 .391 .818 r
.37681 .61538 .38484 .63034 .39801 .63934 .38999 .62361 Metetra
.16 .336 .792 r
.38999 .62361 .39801 .63934 .4113 .65023 .40328 .63374 Metetra
.086 .276 .761 r
.40328 .63374 .4113 .65023 .42474 .66318 .4167 .6459 Metetra
.008 .214 .725 r
.4167 .6459 .42474 .66318 .43834 .67833 .43028 .66027 Metetra
0 .15 .684 r
.43028 .66027 .43834 .67833 .45214 .69585 .44405 .677 Metetra
0 .086 .64 r
.44405 .677 .45214 .69585 .46617 .71589 .45804 .69626 Metetra
0 .022 .593 r
.45804 .69626 .46617 .71589 .48046 .73863 .47227 .71823 Metetra
.527 .509 .779 r
.47227 .71823 .48046 .73863 .49529 .74035 .48708 .71979 Metetra
.832 .662 .672 r
.48708 .71979 .49529 .74035 .51043 .70607 .50229 .68631 Metetra
.824 .662 .682 r
.50229 .68631 .51043 .70607 .52539 .67557 .51731 .65658 Metetra
.816 .663 .694 r
.51731 .65658 .52539 .67557 .5402 .64838 .53219 .63014 Metetra
.808 .665 .705 r
.53219 .63014 .5402 .64838 .55491 .62412 .54696 .60659 Metetra
.799 .667 .717 r
.54696 .60659 .55491 .62412 .56955 .60242 .56167 .58557 Metetra
.791 .669 .729 r
.56167 .58557 .56955 .60242 .58416 .58298 .57634 .56678 Metetra
.782 .671 .742 r
.57634 .56678 .58416 .58298 .59875 .56552 .591 .54992 Metetra
.773 .674 .754 r
.591 .54992 .59875 .56552 .61336 .54978 .60567 .53477 Metetra
.764 .677 .767 r
.60567 .53477 .61336 .54978 .62799 .53557 .62037 .52109 Metetra
.755 .681 .779 r
.62037 .52109 .62799 .53557 .64267 .52268 .63512 .50871 Metetra
.747 .684 .791 r
.63512 .50871 .64267 .52268 .6574 .51095 .64992 .49744 Metetra
.738 .688 .803 r
.64992 .49744 .6574 .51095 .6722 .50021 .66479 .48714 Metetra
.73 .692 .814 r
.66479 .48714 .6722 .50021 .68707 .49035 .67973 .47768 Metetra
.722 .696 .825 r
.67973 .47768 .68707 .49035 .70202 .48123 .69476 .46893 Metetra
.714 .7 .835 r
.69476 .46893 .70202 .48123 .71706 .47276 .70987 .46078 Metetra
.707 .703 .844 r
.70987 .46078 .71706 .47276 .73219 .46484 .72508 .45316 Metetra
.701 .707 .853 r
.72508 .45316 .73219 .46484 .74741 .45739 .74038 .44598 Metetra
.695 .711 .861 r
.74038 .44598 .74741 .45739 .76274 .45034 .75578 .43916 Metetra
.69 .714 .868 r
.75578 .43916 .76274 .45034 .77815 .44362 .77128 .43265 Metetra
.685 .718 .874 r
.77128 .43265 .77815 .44362 .79367 .43718 .78688 .42639 Metetra
.681 .721 .88 r
.78688 .42639 .79367 .43718 .80929 .43097 .80258 .42033 Metetra
.677 .724 .885 r
.80258 .42033 .80929 .43097 .82501 .42495 .81838 .41444 Metetra
.674 .727 .89 r
.81838 .41444 .82501 .42495 .84084 .41908 .83428 .40868 Metetra
.671 .729 .893 r
.83428 .40868 .84084 .41908 .85676 .41333 .85029 .40302 Metetra
.575 .659 .894 r
.27801 .58781 .28641 .59811 .29919 .59724 .29085 .58639 Metetra
.548 .637 .891 r
.29085 .58639 .29919 .59724 .312 .59726 .30372 .58582 Metetra
.516 .611 .886 r
.30372 .58582 .312 .59726 .32486 .5983 .31662 .58622 Metetra
.478 .581 .88 r
.31662 .58622 .32486 .5983 .33775 .60048 .32957 .58773 Metetra
.434 .546 .871 r
.32957 .58773 .33775 .60048 .3507 .60397 .34256 .5905 Metetra
.384 .507 .859 r
.34256 .5905 .3507 .60397 .36372 .60888 .35561 .59467 Metetra
.328 .463 .844 r
.35561 .59467 .36372 .60888 .37681 .61538 .36872 .6004 Metetra
.265 .414 .825 r
.36872 .6004 .37681 .61538 .38999 .62361 .38192 .60784 Metetra
.197 .359 .801 r
.38192 .60784 .38999 .62361 .40328 .63374 .39521 .61716 Metetra
.123 .301 .772 r
.39521 .61716 .40328 .63374 .4167 .6459 .40863 .62851 Metetra
.046 .239 .738 r
.40863 .62851 .4167 .6459 .43028 .66027 .4222 .64206 Metetra
0 .175 .699 r
.4222 .64206 .43028 .66027 .44405 .677 .43593 .65797 Metetra
0 .111 .657 r
.43593 .65797 .44405 .677 .45804 .69626 .44988 .67642 Metetra
0 .046 .611 r
.44988 .67642 .45804 .69626 .47227 .71823 .46406 .69759 Metetra
.529 .509 .777 r
.46406 .69759 .47227 .71823 .48708 .71979 .47885 .69897 Metetra
.828 .659 .673 r
.47885 .69897 .48708 .71979 .50229 .68631 .49413 .66632 Metetra
.82 .66 .684 r
.49413 .66632 .50229 .68631 .51731 .65658 .50921 .6374 Metetra
.812 .661 .696 r
.50921 .6374 .51731 .65658 .53219 .63014 .52415 .61174 Metetra
.804 .663 .707 r
.52415 .61174 .53219 .63014 .54696 .60659 .53898 .58893 Metetra
.795 .665 .72 r
.53898 .58893 .54696 .60659 .56167 .58557 .55375 .56862 Metetra
.786 .667 .732 r
.55375 .56862 .56167 .58557 .57634 .56678 .56848 .55049 Metetra
.777 .67 .745 r
.56848 .55049 .57634 .56678 .591 .54992 .5832 .53427 Metetra
.768 .673 .758 r
.5832 .53427 .591 .54992 .60567 .53477 .59794 .51972 Metetra
.759 .676 .771 r
.59794 .51972 .60567 .53477 .62037 .52109 .6127 .5066 Metetra
.75 .68 .783 r
.6127 .5066 .62037 .52109 .63512 .50871 .62752 .49474 Metetra
.741 .683 .795 r
.62752 .49474 .63512 .50871 .64992 .49744 .64239 .48395 Metetra
.732 .687 .807 r
.64239 .48395 .64992 .49744 .66479 .48714 .65733 .47409 Metetra
.724 .691 .819 r
.65733 .47409 .66479 .48714 .67973 .47768 .67234 .46503 Metetra
.716 .696 .829 r
.67234 .46503 .67973 .47768 .69476 .46893 .68744 .45665 Metetra
.709 .7 .839 r
.68744 .45665 .69476 .46893 .70987 .46078 .70263 .44884 Metetra
.702 .704 .849 r
.70263 .44884 .70987 .46078 .72508 .45316 .71791 .44152 Metetra
.696 .708 .857 r
.71791 .44152 .72508 .45316 .74038 .44598 .73329 .4346 Metetra
.69 .712 .865 r
.73329 .4346 .74038 .44598 .75578 .43916 .74877 .42801 Metetra
.685 .715 .872 r
.74877 .42801 .75578 .43916 .77128 .43265 .76435 .42171 Metetra
.68 .719 .878 r
.76435 .42171 .77128 .43265 .78688 .42639 .78002 .41562 Metetra
.676 .722 .884 r
.78002 .41562 .78688 .42639 .80258 .42033 .7958 .40971 Metetra
.673 .725 .889 r
.7958 .40971 .80258 .42033 .81838 .41444 .81168 .40395 Metetra
.67 .728 .893 r
.81168 .40395 .81838 .41444 .83428 .40868 .82767 .39829 Metetra
.668 .731 .897 r
.82767 .39829 .83428 .40868 .85029 .40302 .84376 .39271 Metetra
.592 .669 .893 r
.26952 .57761 .27801 .58781 .29085 .58639 .28242 .57564 Metetra
.567 .648 .89 r
.28242 .57564 .29085 .58639 .30372 .58582 .29535 .57447 Metetra
.537 .624 .886 r
.29535 .57447 .30372 .58582 .31662 .58622 .30832 .57422 Metetra
.502 .596 .88 r
.30832 .57422 .31662 .58622 .32957 .58773 .32132 .57504 Metetra
.461 .563 .872 r
.32132 .57504 .32957 .58773 .34256 .5905 .33435 .57707 Metetra
.413 .526 .862 r
.33435 .57707 .34256 .5905 .35561 .59467 .34744 .58048 Metetra
.36 .483 .849 r
.34744 .58048 .35561 .59467 .36872 .6004 .36058 .58541 Metetra
.3 .436 .831 r
.36058 .58541 .36872 .6004 .38192 .60784 .3738 .59204 Metetra
.234 .383 .809 r
.3738 .59204 .38192 .60784 .39521 .61716 .38711 .60051 Metetra
.162 .326 .782 r
.38711 .60051 .39521 .61716 .40863 .62851 .40053 .61101 Metetra
.086 .265 .75 r
.40053 .61101 .40863 .62851 .4222 .64206 .41408 .62371 Metetra
.007 .201 .714 r
.41408 .62371 .4222 .64206 .43593 .65797 .42779 .63877 Metetra
0 .136 .672 r
.42779 .63877 .43593 .65797 .44988 .67642 .4417 .65637 Metetra
0 .071 .628 r
.4417 .65637 .44988 .67642 .46406 .69759 .45582 .67669 Metetra
.532 .508 .775 r
.45582 .67669 .46406 .69759 .47885 .69897 .4706 .67789 Metetra
.824 .657 .675 r
.4706 .67789 .47885 .69897 .49413 .66632 .48594 .64611 Metetra
.816 .658 .686 r
.48594 .64611 .49413 .66632 .50921 .6374 .50108 .61803 Metetra
.808 .659 .698 r
.50108 .61803 .50921 .6374 .52415 .61174 .51607 .59317 Metetra
.799 .66 .71 r
.51607 .59317 .52415 .61174 .53898 .58893 .53096 .57114 Metetra
.79 .662 .723 r
.53096 .57114 .53898 .58893 .55375 .56862 .54578 .55156 Metetra
.781 .665 .736 r
.54578 .55156 .55375 .56862 .56848 .55049 .56057 .53414 Metetra
.771 .668 .749 r
.56057 .53414 .56848 .55049 .5832 .53427 .57536 .51858 Metetra
.762 .671 .762 r
.57536 .51858 .5832 .53427 .59794 .51972 .59015 .50464 Metetra
.753 .675 .775 r
.59015 .50464 .59794 .51972 .6127 .5066 .60498 .4921 Metetra
.744 .678 .788 r
.60498 .4921 .6127 .5066 .62752 .49474 .61986 .48077 Metetra
.735 .683 .8 r
.61986 .48077 .62752 .49474 .64239 .48395 .6348 .47048 Metetra
.726 .687 .812 r
.6348 .47048 .64239 .48395 .65733 .47409 .64981 .46108 Metetra
.718 .691 .824 r
.64981 .46108 .65733 .47409 .67234 .46503 .6649 .45243 Metetra
.71 .696 .834 r
.6649 .45243 .67234 .46503 .68744 .45665 .68007 .44442 Metetra
.703 .7 .844 r
.68007 .44442 .68744 .45665 .70263 .44884 .69534 .43695 Metetra
.696 .704 .854 r
.69534 .43695 .70263 .44884 .71791 .44152 .71069 .42992 Metetra
.69 .709 .862 r
.71069 .42992 .71791 .44152 .73329 .4346 .72615 .42327 Metetra
.685 .713 .87 r
.72615 .42327 .73329 .4346 .74877 .42801 .7417 .41691 Metetra
.68 .716 .876 r
.7417 .41691 .74877 .42801 .76435 .42171 .75736 .4108 Metetra
.676 .72 .882 r
.75736 .4108 .76435 .42171 .78002 .41562 .77311 .40489 Metetra
.672 .723 .888 r
.77311 .40489 .78002 .41562 .7958 .40971 .78897 .39913 Metetra
.669 .727 .892 r
.78897 .39913 .7958 .40971 .81168 .40395 .80494 .39348 Metetra
.667 .729 .896 r
.80494 .39348 .81168 .40395 .82767 .39829 .821 .38792 Metetra
.665 .732 .9 r
.821 .38792 .82767 .39829 .84376 .39271 .83717 .38241 Metetra
.607 .679 .892 r
.26094 .56752 .26952 .57761 .28242 .57564 .27391 .56501 Metetra
.585 .66 .889 r
.27391 .56501 .28242 .57564 .29535 .57447 .28691 .56323 Metetra
.557 .637 .886 r
.28691 .56323 .29535 .57447 .30832 .57422 .29994 .56232 Metetra
.525 .61 .88 r
.29994 .56232 .30832 .57422 .32132 .57504 .31299 .56243 Metetra
.487 .579 .874 r
.31299 .56243 .32132 .57504 .33435 .57707 .32608 .56371 Metetra
.442 .544 .864 r
.32608 .56371 .33435 .57707 .34744 .58048 .33921 .56633 Metetra
.392 .503 .852 r
.33921 .56633 .34744 .58048 .36058 .58541 .3524 .57043 Metetra
.334 .457 .836 r
.3524 .57043 .36058 .58541 .3738 .59204 .36564 .5762 Metetra
.27 .407 .816 r
.36564 .5762 .3738 .59204 .38711 .60051 .37897 .58381 Metetra
.201 .351 .792 r
.37897 .58381 .38711 .60051 .40053 .61101 .39239 .59342 Metetra
.126 .291 .762 r
.39239 .59342 .40053 .61101 .41408 .62371 .40593 .60522 Metetra
.047 .228 .727 r
.40593 .60522 .41408 .62371 .42779 .63877 .41962 .61938 Metetra
0 .163 .688 r
.41962 .61938 .42779 .63877 .4417 .65637 .43349 .63609 Metetra
0 .097 .644 r
.43349 .63609 .4417 .65637 .45582 .67669 .44757 .65554 Metetra
.534 .508 .773 r
.44757 .65554 .45582 .67669 .4706 .67789 .46233 .65654 Metetra
.82 .654 .677 r
.46233 .65654 .4706 .67789 .48594 .64611 .47772 .62567 Metetra
.812 .655 .688 r
.47772 .62567 .48594 .64611 .50108 .61803 .49292 .59847 Metetra
.803 .656 .7 r
.49292 .59847 .50108 .61803 .51607 .59317 .50796 .57445 Metetra
.794 .658 .713 r
.50796 .57445 .51607 .59317 .53096 .57114 .5229 .55323 Metetra
.784 .66 .726 r
.5229 .55323 .53096 .57114 .54578 .55156 .53778 .53442 Metetra
.775 .663 .739 r
.53778 .53442 .54578 .55156 .56057 .53414 .55262 .51772 Metetra
.766 .666 .752 r
.55262 .51772 .56057 .53414 .57536 .51858 .56746 .50285 Metetra
.756 .669 .766 r
.56746 .50285 .57536 .51858 .59015 .50464 .58232 .48955 Metetra
.747 .673 .779 r
.58232 .48955 .59015 .50464 .60498 .4921 .59721 .47761 Metetra
.737 .677 .792 r
.59721 .47761 .60498 .4921 .61986 .48077 .61215 .46683 Metetra
.728 .682 .805 r
.61215 .46683 .61986 .48077 .6348 .47048 .62716 .45705 Metetra
.72 .686 .817 r
.62716 .45705 .6348 .47048 .64981 .46108 .64224 .44811 Metetra
.711 .691 .829 r
.64224 .44811 .64981 .46108 .6649 .45243 .6574 .43989 Metetra
.704 .696 .84 r
.6574 .43989 .6649 .45243 .68007 .44442 .67264 .43225 Metetra
.697 .701 .85 r
.67264 .43225 .68007 .44442 .69534 .43695 .68798 .42512 Metetra
.69 .705 .859 r
.68798 .42512 .69534 .43695 .71069 .42992 .70341 .41839 Metetra
.684 .71 .867 r
.70341 .41839 .71069 .42992 .72615 .42327 .71895 .412 Metetra
.679 .714 .875 r
.71895 .412 .72615 .42327 .7417 .41691 .73458 .40587 Metetra
.675 .718 .881 r
.73458 .40587 .7417 .41691 .75736 .4108 .75031 .39995 Metetra
.671 .721 .887 r
.75031 .39995 .75736 .4108 .77311 .40489 .76615 .3942 Metetra
.668 .725 .892 r
.76615 .3942 .77311 .40489 .78897 .39913 .78209 .38857 Metetra
.666 .728 .896 r
.78209 .38857 .78897 .39913 .80494 .39348 .79813 .38303 Metetra
.664 .731 .9 r
.79813 .38303 .80494 .39348 .821 .38792 .81427 .37756 Metetra
.662 .734 .903 r
.81427 .37756 .821 .38792 .83717 .38241 .83052 .37212 Metetra
.623 .689 .891 r
.25227 .55757 .26094 .56752 .27391 .56501 .26532 .55451 Metetra
.602 .671 .889 r
.26532 .55451 .27391 .56501 .28691 .56323 .27838 .55212 Metetra
.577 .65 .885 r
.27838 .55212 .28691 .56323 .29994 .56232 .29148 .55054 Metetra
.548 .625 .88 r
.29148 .55054 .29994 .56232 .31299 .56243 .3046 .54993 Metetra
.512 .595 .874 r
.3046 .54993 .31299 .56243 .32608 .56371 .31775 .55044 Metetra
.471 .562 .866 r
.31775 .55044 .32608 .56371 .33921 .56633 .33093 .55223 Metetra
.423 .523 .855 r
.33093 .55223 .33921 .56633 .3524 .57043 .34415 .55548 Metetra
.368 .479 .841 r
.34415 .55548 .3524 .57043 .36564 .5762 .35743 .56036 Metetra
.307 .43 .823 r
.35743 .56036 .36564 .5762 .37897 .58381 .37078 .56706 Metetra
.24 .376 .8 r
.37078 .56706 .37897 .58381 .39239 .59342 .38421 .57574 Metetra
.166 .317 .773 r
.38421 .57574 .39239 .59342 .40593 .60522 .39775 .5866 Metetra
.088 .255 .74 r
.39775 .5866 .40593 .60522 .41962 .61938 .41143 .59982 Metetra
.008 .19 .703 r
.41143 .59982 .41962 .61938 .43349 .63609 .42527 .6156 Metetra
0 .123 .661 r
.42527 .6156 .43349 .63609 .44757 .65554 .4393 .63413 Metetra
.536 .508 .771 r
.4393 .63413 .44757 .65554 .46233 .65654 .45405 .63492 Metetra
.816 .651 .678 r
.45405 .63492 .46233 .65654 .47772 .62567 .46949 .60501 Metetra
.807 .652 .69 r
.46949 .60501 .47772 .62567 .49292 .59847 .48472 .57872 Metetra
.798 .653 .703 r
.48472 .57872 .49292 .59847 .50796 .57445 .49982 .55558 Metetra
.788 .655 .716 r
.49982 .55558 .50796 .57445 .5229 .55323 .51481 .5352 Metetra
.779 .657 .729 r
.51481 .5352 .5229 .55323 .53778 .53442 .52973 .5172 Metetra
.769 .66 .743 r
.52973 .5172 .53778 .53442 .55262 .51772 .54463 .50126 Metetra
.759 .664 .757 r
.54463 .50126 .55262 .51772 .56746 .50285 .55952 .4871 Metetra
.749 .668 .77 r
.55952 .4871 .56746 .50285 .58232 .48955 .57444 .47447 Metetra
.74 .672 .784 r
.57444 .47447 .58232 .48955 .59721 .47761 .58939 .46314 Metetra
.73 .676 .798 r
.58939 .46314 .59721 .47761 .61215 .46683 .60439 .45294 Metetra
.721 .681 .811 r
.60439 .45294 .61215 .46683 .62716 .45705 .61946 .44368 Metetra
.713 .686 .823 r
.61946 .44368 .62716 .45705 .64224 .44811 .63461 .43521 Metetra
.704 .691 .835 r
.63461 .43521 .64224 .44811 .6574 .43989 .64984 .42741 Metetra
.697 .696 .846 r
.64984 .42741 .6574 .43989 .67264 .43225 .66516 .42016 Metetra
.69 .701 .855 r
.66516 .42016 .67264 .43225 .68798 .42512 .68057 .41337 Metetra
.684 .706 .864 r
.68057 .41337 .68798 .42512 .70341 .41839 .69608 .40694 Metetra
.679 .711 .873 r
.69608 .40694 .70341 .41839 .71895 .412 .71169 .4008 Metetra
.674 .715 .88 r
.71169 .4008 .71895 .412 .73458 .40587 .72739 .39489 Metetra
.67 .719 .886 r
.72739 .39489 .73458 .40587 .75031 .39995 .74321 .38916 Metetra
.667 .723 .891 r
.74321 .38916 .75031 .39995 .76615 .3942 .75912 .38356 Metetra
.664 .727 .896 r
.75912 .38356 .76615 .3942 .78209 .38857 .77514 .37806 Metetra
.662 .73 .9 r
.77514 .37806 .78209 .38857 .79813 .38303 .79126 .37262 Metetra
.661 .733 .903 r
.79126 .37262 .79813 .38303 .81427 .37756 .80748 .36721 Metetra
.659 .735 .906 r
.80748 .36721 .81427 .37756 .83052 .37212 .82381 .36183 Metetra
.637 .699 .891 r
.24351 .54777 .25227 .55757 .26532 .55451 .25662 .54416 Metetra
.619 .682 .888 r
.25662 .54416 .26532 .55451 .27838 .55212 .26977 .54116 Metetra
.597 .662 .884 r
.26977 .54116 .27838 .55212 .29148 .55054 .28293 .5389 Metetra
.57 .639 .88 r
.28293 .5389 .29148 .55054 .3046 .54993 .29612 .53755 Metetra
.537 .611 .874 r
.29612 .53755 .3046 .54993 .31775 .55044 .30934 .53727 Metetra
.498 .579 .867 r
.30934 .53727 .31775 .55044 .33093 .55223 .32258 .53821 Metetra
.454 .542 .857 r
.32258 .53821 .33093 .55223 .34415 .55548 .33585 .54058 Metetra
.402 .5 .845 r
.33585 .54058 .34415 .55548 .35743 .56036 .34917 .54453 Metetra
.344 .453 .828 r
.34917 .54453 .35743 .56036 .37078 .56706 .36255 .55027 Metetra
.279 .401 .808 r
.36255 .55027 .37078 .56706 .38421 .57574 .376 .55798 Metetra
.207 .344 .782 r
.376 .55798 .38421 .57574 .39775 .5866 .38955 .56786 Metetra
.131 .282 .752 r
.38955 .56786 .39775 .5866 .41143 .59982 .40321 .58009 Metetra
.05 .217 .717 r
.40321 .58009 .41143 .59982 .42527 .6156 .41702 .59489 Metetra
0 .151 .676 r
.41702 .59489 .42527 .6156 .4393 .63413 .43102 .61244 Metetra
.539 .507 .769 r
.43102 .61244 .4393 .63413 .45405 .63492 .44574 .61302 Metetra
.811 .648 .68 r
.44574 .61302 .45405 .63492 .46949 .60501 .46123 .5841 Metetra
.802 .649 .692 r
.46123 .5841 .46949 .60501 .48472 .57872 .47651 .55877 Metetra
.792 .65 .705 r
.47651 .55877 .48472 .57872 .49982 .55558 .49164 .53657 Metetra
.782 .652 .719 r
.49164 .53657 .49982 .55558 .51481 .5352 .50667 .51707 Metetra
.772 .655 .733 r
.50667 .51707 .51481 .5352 .52973 .5172 .52164 .49991 Metetra
.762 .658 .747 r
.52164 .49991 .52973 .5172 .54463 .50126 .53659 .48476 Metetra
.752 .661 .761 r
.53659 .48476 .54463 .50126 .55952 .4871 .55153 .47134 Metetra
.742 .666 .775 r
.55153 .47134 .55952 .4871 .57444 .47447 .5665 .45941 Metetra
.732 .67 .789 r
.5665 .45941 .57444 .47447 .58939 .46314 .58151 .44873 Metetra
.723 .675 .803 r
.58151 .44873 .58939 .46314 .60439 .45294 .59658 .43911 Metetra
.714 .681 .817 r
.59658 .43911 .60439 .45294 .61946 .44368 .61171 .43038 Metetra
.705 .686 .829 r
.61171 .43038 .61946 .44368 .63461 .43521 .62693 .4224 Metetra
.697 .691 .841 r
.62693 .4224 .63461 .43521 .64984 .42741 .64222 .41504 Metetra
.69 .697 .852 r
.64222 .41504 .64984 .42741 .66516 .42016 .65761 .40817 Metetra
.683 .702 .862 r
.65761 .40817 .66516 .42016 .68057 .41337 .6731 .40171 Metetra
.677 .707 .87 r
.6731 .40171 .68057 .41337 .69608 .40694 .68868 .39557 Metetra
.673 .712 .878 r
.68868 .39557 .69608 .40694 .71169 .4008 .70437 .38968 Metetra
.668 .717 .885 r
.70437 .38968 .71169 .4008 .72739 .39489 .72015 .38399 Metetra
.665 .721 .891 r
.72015 .38399 .72739 .39489 .74321 .38916 .73604 .37843 Metetra
.662 .725 .896 r
.73604 .37843 .74321 .38916 .75912 .38356 .75203 .37298 Metetra
.66 .729 .9 r
.75203 .37298 .75912 .38356 .77514 .37806 .76813 .36758 Metetra
.659 .732 .904 r
.76813 .36758 .77514 .37806 .79126 .37262 .78433 .36223 Metetra
.658 .735 .907 r
.78433 .36223 .79126 .37262 .80748 .36721 .80063 .35689 Metetra
.657 .737 .909 r
.80063 .35689 .80748 .36721 .82381 .36183 .81704 .35154 Metetra
.651 .709 .89 r
.23463 .53814 .24351 .54777 .25662 .54416 .24783 .53399 Metetra
.635 .693 .887 r
.24783 .53399 .25662 .54416 .26977 .54116 .26105 .53038 Metetra
.615 .675 .884 r
.26105 .53038 .26977 .54116 .28293 .5389 .2743 .52744 Metetra
.591 .653 .879 r
.2743 .52744 .28293 .5389 .29612 .53755 .28756 .52534 Metetra
.561 .627 .874 r
.28756 .52534 .29612 .53755 .30934 .53727 .30085 .52423 Metetra
.526 .596 .867 r
.30085 .52423 .30934 .53727 .32258 .53821 .31416 .52431 Metetra
.484 .561 .859 r
.31416 .52431 .32258 .53821 .33585 .54058 .32749 .52574 Metetra
.436 .521 .847 r
.32749 .52574 .33585 .54058 .34917 .54453 .34086 .52873 Metetra
.38 .476 .833 r
.34086 .52873 .34917 .54453 .36255 .55027 .35427 .53347 Metetra
.318 .426 .814 r
.35427 .53347 .36255 .55027 .376 .55798 .36775 .54016 Metetra
.249 .37 .791 r
.36775 .54016 .376 .55798 .38955 .56786 .38131 .549 Metetra
.174 .31 .763 r
.38131 .549 .38955 .56786 .40321 .58009 .39497 .56019 Metetra
.094 .246 .73 r
.39497 .56019 .40321 .58009 .41702 .59489 .40876 .57395 Metetra
.011 .179 .691 r
.40876 .57395 .41702 .59489 .43102 .61244 .42272 .59048 Metetra
.542 .507 .767 r
.42272 .59048 .43102 .61244 .44574 .61302 .43743 .59083 Metetra
.806 .645 .682 r
.43743 .59083 .44574 .61302 .46123 .5841 .45295 .56296 Metetra
.796 .645 .695 r
.45295 .56296 .46123 .5841 .47651 .55877 .46826 .53864 Metetra
.786 .647 .708 r
.46826 .53864 .47651 .55877 .49164 .53657 .48343 .51741 Metetra
.776 .649 .722 r
.48343 .51741 .49164 .53657 .50667 .51707 .4985 .49884 Metetra
.765 .652 .736 r
.4985 .49884 .50667 .51707 .52164 .49991 .51351 .48256 Metetra
.755 .655 .751 r
.51351 .48256 .52164 .49991 .53659 .48476 .5285 .46825 Metetra
.745 .659 .766 r
.5285 .46825 .53659 .48476 .55153 .47134 .54349 .45561 Metetra
.734 .664 .781 r
.54349 .45561 .55153 .47134 .5665 .45941 .55851 .44439 Metetra
.724 .669 .795 r
.55851 .44439 .5665 .45941 .58151 .44873 .57358 .43438 Metetra
.715 .674 .809 r
.57358 .43438 .58151 .44873 .59658 .43911 .58871 .42537 Metetra
.706 .68 .823 r
.58871 .42537 .59658 .43911 .61171 .43038 .6039 .41719 Metetra
.697 .686 .836 r
.6039 .41719 .61171 .43038 .62693 .4224 .61918 .4097 Metetra
.689 .692 .848 r
.61918 .4097 .62693 .4224 .64222 .41504 .63455 .40277 Metetra
.682 .698 .858 r
.63455 .40277 .64222 .41504 .65761 .40817 .65001 .39629 Metetra
.676 .704 .868 r
.65001 .39629 .65761 .40817 .6731 .40171 .66557 .39017 Metetra
.671 .709 .877 r
.66557 .39017 .6731 .40171 .68868 .39557 .68122 .38431 Metetra
.666 .714 .884 r
.68122 .38431 .68868 .39557 .70437 .38968 .69698 .37867 Metetra
.663 .719 .891 r
.69698 .37867 .70437 .38968 .72015 .38399 .71285 .37317 Metetra
.66 .724 .896 r
.71285 .37317 .72015 .38399 .73604 .37843 .72881 .36778 Metetra
.658 .728 .901 r
.72881 .36778 .73604 .37843 .75203 .37298 .74488 .36245 Metetra
.656 .731 .904 r
.74488 .36245 .75203 .37298 .76813 .36758 .76106 .35716 Metetra
.655 .734 .908 r
.76106 .35716 .76813 .36758 .78433 .36223 .77734 .35187 Metetra
.655 .737 .91 r
.77734 .35187 .78433 .36223 .80063 .35689 .79372 .34658 Metetra
.655 .74 .912 r
.79372 .34658 .80063 .35689 .81704 .35154 .81021 .34126 Metetra
.664 .718 .889 r
.22566 .52869 .23463 .53814 .24783 .53399 .23893 .52402 Metetra
.651 .704 .886 r
.23893 .52402 .24783 .53399 .26105 .53038 .25224 .5198 Metetra
.633 .687 .883 r
.25224 .5198 .26105 .53038 .2743 .52744 .26557 .51617 Metetra
.612 .666 .879 r
.26557 .51617 .2743 .52744 .28756 .52534 .27892 .51331 Metetra
.585 .642 .874 r
.27892 .51331 .28756 .52534 .30085 .52423 .29228 .51137 Metetra
.552 .613 .867 r
.29228 .51137 .30085 .52423 .31416 .52431 .30566 .51054 Metetra
.514 .58 .86 r
.30566 .51054 .31416 .52431 .32749 .52574 .31906 .51101 Metetra
.469 .542 .849 r
.31906 .51101 .32749 .52574 .34086 .52873 .33249 .51299 Metetra
.416 .499 .836 r
.33249 .51299 .34086 .52873 .35427 .53347 .34595 .51668 Metetra
.357 .45 .819 r
.34595 .51668 .35427 .53347 .36775 .54016 .35946 .52229 Metetra
.291 .396 .798 r
.35946 .52229 .36775 .54016 .38131 .549 .37304 .53003 Metetra
.218 .337 .772 r
.37304 .53003 .38131 .549 .39497 .56019 .3867 .54013 Metetra
.14 .274 .741 r
.3867 .54013 .39497 .56019 .40876 .57395 .40049 .55278 Metetra
.057 .208 .706 r
.40049 .55278 .40876 .57395 .42272 .59048 .41442 .56823 Metetra
.545 .507 .764 r
.41442 .56823 .42272 .59048 .43743 .59083 .42911 .56834 Metetra
.8 .641 .684 r
.42911 .56834 .43743 .59083 .45295 .56296 .44465 .54157 Metetra
.79 .642 .697 r
.44465 .54157 .45295 .56296 .46826 .53864 .45999 .51832 Metetra
.779 .643 .711 r
.45999 .51832 .46826 .53864 .48343 .51741 .47518 .49812 Metetra
.769 .645 .726 r
.47518 .49812 .48343 .51741 .4985 .49884 .49029 .48053 Metetra
.758 .648 .741 r
.49029 .48053 .4985 .49884 .51351 .48256 .50534 .46518 Metetra
.747 .652 .756 r
.50534 .46518 .51351 .48256 .5285 .46825 .52037 .45174 Metetra
.736 .657 .771 r
.52037 .45174 .5285 .46825 .54349 .45561 .5354 .43992 Metetra
.726 .662 .787 r
.5354 .43992 .54349 .45561 .55851 .44439 .55047 .42945 Metetra
.715 .668 .802 r
.55047 .42945 .55851 .44439 .57358 .43438 .56559 .42013 Metetra
.706 .674 .816 r
.56559 .42013 .57358 .43438 .58871 .42537 .58077 .41174 Metetra
.697 .68 .83 r
.58077 .41174 .58871 .42537 .6039 .41719 .59603 .40413 Metetra
.688 .687 .843 r
.59603 .40413 .6039 .41719 .61918 .4097 .61138 .39714 Metetra
.681 .693 .855 r
.61138 .39714 .61918 .4097 .63455 .40277 .62681 .39065 Metetra
.674 .699 .865 r
.62681 .39065 .63455 .40277 .65001 .39629 .64234 .38455 Metetra
.669 .705 .875 r
.64234 .38455 .65001 .39629 .66557 .39017 .65797 .37875 Metetra
.664 .711 .883 r
.65797 .37875 .66557 .39017 .68122 .38431 .6737 .37318 Metetra
.66 .717 .89 r
.6737 .37318 .68122 .38431 .69698 .37867 .68954 .36776 Metetra
.657 .722 .896 r
.68954 .36776 .69698 .37867 .71285 .37317 .70548 .36246 Metetra
.655 .726 .901 r
.70548 .36246 .71285 .37317 .72881 .36778 .72152 .35721 Metetra
.654 .73 .905 r
.72152 .35721 .72881 .36778 .74488 .36245 .73767 .352 Metetra
.653 .734 .909 r
.73767 .352 .74488 .36245 .76106 .35716 .75392 .34678 Metetra
.653 .737 .911 r
.75392 .34678 .76106 .35716 .77734 .35187 .77028 .34155 Metetra
.653 .74 .913 r
.77028 .34155 .77734 .35187 .79372 .34658 .78674 .33629 Metetra
.653 .742 .915 r
.78674 .33629 .79372 .34658 .81021 .34126 .80331 .33097 Metetra
.676 .728 .889 r
.21656 .51944 .22566 .52869 .23893 .52402 .22992 .51426 Metetra
.665 .715 .886 r
.22992 .51426 .23893 .52402 .25224 .5198 .24332 .50945 Metetra
.651 .699 .882 r
.24332 .50945 .25224 .5198 .26557 .51617 .25674 .50514 Metetra
.632 .68 .878 r
.25674 .50514 .26557 .51617 .27892 .51331 .27017 .5015 Metetra
.608 .657 .873 r
.27017 .5015 .27892 .51331 .29228 .51137 .28362 .49871 Metetra
.578 .63 .867 r
.28362 .49871 .29228 .51137 .30566 .51054 .29709 .49695 Metetra
.543 .599 .86 r
.29709 .49695 .30566 .51054 .31906 .51101 .31056 .49642 Metetra
.501 .563 .85 r
.31056 .49642 .31906 .51101 .33249 .51299 .32406 .49734 Metetra
.452 .521 .838 r
.32406 .49734 .33249 .51299 .34595 .51668 .33758 .49993 Metetra
.396 .475 .823 r
.33758 .49993 .34595 .51668 .35946 .52229 .35113 .5044 Metetra
.333 .423 .804 r
.35113 .5044 .35946 .52229 .37304 .53003 .36474 .51098 Metetra
.262 .365 .781 r
.36474 .51098 .37304 .53003 .3867 .54013 .37842 .5199 Metetra
.186 .303 .752 r
.37842 .5199 .3867 .54013 .40049 .55278 .39219 .53139 Metetra
.104 .238 .719 r
.39219 .53139 .40049 .55278 .41442 .56823 .4061 .54569 Metetra
.548 .507 .762 r
.4061 .54569 .41442 .56823 .42911 .56834 .42077 .54553 Metetra
.794 .636 .687 r
.42077 .54553 .42911 .56834 .44465 .54157 .43633 .51993 Metetra
.783 .637 .7 r
.43633 .51993 .44465 .54157 .45999 .51832 .45169 .49783 Metetra
.772 .639 .715 r
.45169 .49783 .45999 .51832 .47518 .49812 .46691 .47871 Metetra
.761 .642 .73 r
.46691 .47871 .47518 .49812 .49029 .48053 .48204 .46216 Metetra
.749 .645 .745 r
.48204 .46216 .49029 .48053 .50534 .46518 .49712 .4478 Metetra
.738 .649 .761 r
.49712 .4478 .50534 .46518 .52037 .45174 .51218 .43527 Metetra
.727 .654 .777 r
.51218 .43527 .52037 .45174 .5354 .43992 .52726 .4243 Metetra
.716 .66 .793 r
.52726 .4243 .5354 .43992 .55047 .42945 .54237 .41462 Metetra
.706 .667 .809 r
.54237 .41462 .55047 .42945 .56559 .42013 .55754 .40601 Metetra
.696 .673 .824 r
.55754 .40601 .56559 .42013 .58077 .41174 .57278 .39827 Metetra
.687 .68 .838 r
.57278 .39827 .58077 .41174 .59603 .40413 .5881 .39123 Metetra
.679 .687 .851 r
.5881 .39123 .59603 .40413 .61138 .39714 .60351 .38475 Metetra
.672 .694 .863 r
.60351 .38475 .61138 .39714 .62681 .39065 .61901 .3787 Metetra
.666 .701 .873 r
.61901 .3787 .62681 .39065 .64234 .38455 .63461 .37298 Metetra
.661 .708 .882 r
.63461 .37298 .64234 .38455 .65797 .37875 .65031 .3675 Metetra
.657 .714 .89 r
.65031 .3675 .65797 .37875 .6737 .37318 .66612 .36219 Metetra
.654 .72 .897 r
.66612 .36219 .6737 .37318 .68954 .36776 .68203 .35699 Metetra
.652 .725 .902 r
.68203 .35699 .68954 .36776 .70548 .36246 .69804 .35185 Metetra
.651 .729 .907 r
.69804 .35185 .70548 .36246 .72152 .35721 .71416 .34673 Metetra
.65 .733 .91 r
.71416 .34673 .72152 .35721 .73767 .352 .73039 .34161 Metetra
.65 .737 .913 r
.73039 .34161 .73767 .352 .75392 .34678 .74672 .33646 Metetra
.65 .74 .915 r
.74672 .33646 .75392 .34678 .77028 .34155 .76316 .33126 Metetra
.651 .742 .916 r
.76316 .33126 .77028 .34155 .78674 .33629 .77969 .32601 Metetra
.651 .744 .918 r
.77969 .32601 .78674 .33629 .80331 .33097 .79634 .32068 Metetra
.687 .738 .889 r
.20735 .51042 .21656 .51944 .22992 .51426 .2208 .50475 Metetra
.679 .726 .885 r
.2208 .50475 .22992 .51426 .24332 .50945 .23428 .49936 Metetra
.667 .712 .881 r
.23428 .49936 .24332 .50945 .25674 .50514 .24779 .49438 Metetra
.651 .694 .877 r
.24779 .49438 .25674 .50514 .27017 .5015 .26133 .48997 Metetra
.63 .673 .872 r
.26133 .48997 .27017 .5015 .28362 .49871 .27487 .4863 Metetra
.604 .647 .867 r
.27487 .4863 .28362 .49871 .29709 .49695 .28843 .48358 Metetra
.572 .618 .86 r
.28843 .48358 .29709 .49695 .31056 .49642 .30199 .48202 Metetra
.533 .583 .851 r
.30199 .48202 .31056 .49642 .32406 .49734 .31556 .48183 Metetra
.488 .544 .84 r
.31556 .48183 .32406 .49734 .33758 .49993 .32915 .48325 Metetra
.435 .499 .826 r
.32915 .48325 .33758 .49993 .35113 .5044 .34276 .48651 Metetra
.375 .449 .809 r
.34276 .48651 .35113 .5044 .36474 .51098 .3564 .49186 Metetra
.307 .393 .788 r
.3564 .49186 .36474 .51098 .37842 .5199 .37011 .49953 Metetra
.233 .333 .761 r
.37011 .49953 .37842 .5199 .39219 .53139 .38389 .50978 Metetra
.152 .268 .73 r
.38389 .50978 .39219 .53139 .4061 .54569 .39778 .52284 Metetra
.551 .506 .759 r
.39778 .52284 .4061 .54569 .42077 .54553 .41243 .5224 Metetra
.786 .632 .689 r
.41243 .5224 .42077 .54553 .43633 .51993 .428 .49805 Metetra
.775 .633 .703 r
.428 .49805 .43633 .51993 .45169 .49783 .44337 .47715 Metetra
.763 .634 .718 r
.44337 .47715 .45169 .49783 .46691 .47871 .4586 .4592 Metetra
.751 .637 .734 r
.4586 .4592 .46691 .47871 .48204 .46216 .47375 .44376 Metetra
.74 .641 .751 r
.47375 .44376 .48204 .46216 .49712 .4478 .48885 .43043 Metetra
.728 .646 .767 r
.48885 .43043 .49712 .4478 .51218 .43527 .50394 .41888 Metetra
.716 .652 .784 r
.50394 .41888 .51218 .43527 .52726 .4243 .51906 .4088 Metetra
.705 .658 .801 r
.51906 .4088 .52726 .4243 .54237 .41462 .53422 .39994 Metetra
.695 .666 .817 r
.53422 .39994 .54237 .41462 .55754 .40601 .54944 .39207 Metetra
.685 .673 .832 r
.54944 .39207 .55754 .40601 .57278 .39827 .56473 .38499 Metetra
.677 .681 .847 r
.56473 .38499 .57278 .39827 .5881 .39123 .5801 .37853 Metetra
.669 .689 .86 r
.5801 .37853 .5881 .39123 .60351 .38475 .59557 .37256 Metetra
.662 .696 .871 r
.59557 .37256 .60351 .38475 .61901 .3787 .61114 .36694 Metetra
.657 .704 .881 r
.61114 .36694 .61901 .3787 .63461 .37298 .62681 .36159 Metetra
.653 .711 .89 r
.62681 .36159 .63461 .37298 .65031 .3675 .64258 .35642 Metetra
.65 .717 .897 r
.64258 .35642 .65031 .3675 .66612 .36219 .65846 .35135 Metetra
.648 .723 .903 r
.65846 .35135 .66612 .36219 .68203 .35699 .67445 .34635 Metetra
.647 .728 .908 r
.67445 .34635 .68203 .35699 .69804 .35185 .69054 .34136 Metetra
.646 .733 .912 r
.69054 .34136 .69804 .35185 .71416 .34673 .70674 .33635 Metetra
.646 .737 .915 r
.70674 .33635 .71416 .34673 .73039 .34161 .72304 .3313 Metetra
.647 .74 .917 r
.72304 .3313 .73039 .34161 .74672 .33646 .73945 .32618 Metetra
.648 .743 .918 r
.73945 .32618 .74672 .33646 .76316 .33126 .75596 .321 Metetra
.649 .745 .919 r
.75596 .321 .76316 .33126 .77969 .32601 .77258 .31573 Metetra
.65 .747 .92 r
.77258 .31573 .77969 .32601 .79634 .32068 .7893 .31038 Metetra
.698 .747 .89 r
.19801 .50163 .20735 .51042 .2208 .50475 .21154 .49552 Metetra
.692 .737 .886 r
.21154 .49552 .2208 .50475 .23428 .49936 .22512 .48957 Metetra
.683 .724 .881 r
.22512 .48957 .23428 .49936 .24779 .49438 .23873 .48393 Metetra
.669 .708 .877 r
.23873 .48393 .24779 .49438 .26133 .48997 .25237 .47875 Metetra
.651 .688 .872 r
.25237 .47875 .26133 .48997 .27487 .4863 .26601 .4742 Metetra
.629 .664 .866 r
.26601 .4742 .27487 .4863 .28843 .48358 .27967 .4705 Metetra
.6 .636 .859 r
.27967 .4705 .28843 .48358 .30199 .48202 .29333 .46785 Metetra
.565 .604 .851 r
.29333 .46785 .30199 .48202 .31556 .48183 .307 .4665 Metetra
.523 .566 .841 r
.307 .4665 .31556 .48183 .32915 .48325 .32066 .46669 Metetra
.474 .523 .828 r
.32066 .46669 .32915 .48325 .34276 .48651 .33434 .46866 Metetra
.417 .475 .813 r
.33434 .46866 .34276 .48651 .3564 .49186 .34804 .47269 Metetra
.352 .421 .793 r
.34804 .47269 .3564 .49186 .37011 .49953 .36177 .47903 Metetra
.28 .362 .769 r
.36177 .47903 .37011 .49953 .38389 .50978 .37557 .48794 Metetra
.202 .298 .74 r
.37557 .48794 .38389 .50978 .39778 .52284 .38946 .49968 Metetra
.555 .506 .756 r
.38946 .49968 .39778 .52284 .41243 .5224 .40409 .49894 Metetra
.778 .626 .692 r
.40409 .49894 .41243 .5224 .428 .49805 .41965 .47592 Metetra
.766 .627 .707 r
.41965 .47592 .428 .49805 .44337 .47715 .43502 .45632 Metetra
.754 .629 .722 r
.43502 .45632 .44337 .47715 .4586 .4592 .45026 .43961 Metetra
.741 .633 .739 r
.45026 .43961 .4586 .4592 .47375 .44376 .46541 .42534 Metetra
.729 .637 .756 r
.46541 .42534 .47375 .44376 .48885 .43043 .48053 .41312 Metetra
.717 .643 .774 r
.48053 .41312 .48885 .43043 .50394 .41888 .49565 .40259 Metetra
.705 .649 .792 r
.49565 .40259 .50394 .41888 .51906 .4088 .5108 .39346 Metetra
.693 .657 .809 r
.5108 .39346 .51906 .4088 .53422 .39994 .526 .38545 Metetra
.683 .665 .826 r
.526 .38545 .53422 .39994 .54944 .39207 .54126 .37835 Metetra
.673 .673 .842 r
.54126 .37835 .54944 .39207 .56473 .38499 .55661 .37195 Metetra
.665 .682 .856 r
.55661 .37195 .56473 .38499 .5801 .37853 .57204 .36608 Metetra
.658 .691 .869 r
.57204 .36608 .5801 .37853 .59557 .37256 .58757 .36061 Metetra
.652 .699 .881 r
.58757 .36061 .59557 .37256 .61114 .36694 .60321 .35543 Metetra
.648 .707 .89 r
.60321 .35543 .61114 .36694 .62681 .36159 .61895 .35043 Metetra
.645 .714 .898 r
.61895 .35043 .62681 .36159 .64258 .35642 .63479 .34554 Metetra
.643 .721 .905 r
.63479 .34554 .64258 .35642 .65846 .35135 .65074 .3407 Metetra
.642 .727 .91 r
.65074 .3407 .65846 .35135 .67445 .34635 .6668 .33586 Metetra
.642 .732 .914 r
.6668 .33586 .67445 .34635 .69054 .34136 .68297 .33099 Metetra
.642 .737 .917 r
.68297 .33099 .69054 .34136 .70674 .33635 .69924 .32606 Metetra
.643 .74 .919 r
.69924 .32606 .70674 .33635 .72304 .3313 .71562 .32105 Metetra
.645 .743 .921 r
.71562 .32105 .72304 .3313 .73945 .32618 .7321 .31595 Metetra
.647 .746 .922 r
.7321 .31595 .73945 .32618 .75596 .321 .74869 .31075 Metetra
.648 .748 .922 r
.74869 .31075 .75596 .321 .77258 .31573 .76539 .30545 Metetra
.65 .749 .922 r
.76539 .30545 .77258 .31573 .7893 .31038 .78218 .30004 Metetra
.707 .756 .891 r
.18853 .49309 .19801 .50163 .21154 .49552 .20215 .48658 Metetra
.704 .748 .886 r
.20215 .48658 .21154 .49552 .22512 .48957 .21583 .48012 Metetra
.697 .737 .881 r
.21583 .48012 .22512 .48957 .23873 .48393 .22954 .47384 Metetra
.687 .722 .876 r
.22954 .47384 .23873 .48393 .25237 .47875 .24328 .4679 Metetra
.672 .704 .871 r
.24328 .4679 .25237 .47875 .26601 .4742 .25704 .46247 Metetra
.653 .682 .865 r
.25704 .46247 .26601 .4742 .27967 .4705 .27081 .45776 Metetra
.627 .655 .858 r
.27081 .45776 .27967 .4705 .29333 .46785 .28458 .45399 Metetra
.596 .624 .85 r
.28458 .45399 .29333 .46785 .307 .4665 .29835 .45142 Metetra
.558 .589 .841 r
.29835 .45142 .307 .4665 .32066 .46669 .31211 .4503 Metetra
.512 .548 .829 r
.31211 .4503 .32066 .46669 .33434 .46866 .32587 .45091 Metetra
.459 .501 .815 r
.32587 .45091 .33434 .46866 .34804 .47269 .33963 .45352 Metetra
.398 .449 .797 r
.33963 .45352 .34804 .47269 .36177 .47903 .35342 .45841 Metetra
.329 .392 .776 r
.35342 .45841 .36177 .47903 .37557 .48794 .36724 .46588 Metetra
.252 .329 .749 r
.36724 .46588 .37557 .48794 .38946 .49968 .38113 .47619 Metetra
.559 .507 .753 r
.38113 .47619 .38946 .49968 .40409 .49894 .39575 .47513 Metetra
.769 .62 .694 r
.39575 .47513 .40409 .49894 .41965 .47592 .4113 .45355 Metetra
.756 .621 .71 r
.4113 .45355 .41965 .47592 .43502 .45632 .42665 .43533 Metetra
.743 .624 .727 r
.42665 .43533 .43502 .45632 .45026 .43961 .44188 .41996 Metetra
.729 .627 .745 r
.44188 .41996 .45026 .43961 .46541 .42534 .45704 .40695 Metetra
.716 .633 .763 r
.45704 .40695 .46541 .42534 .48053 .41312 .47217 .39591 Metetra
.704 .639 .782 r
.47217 .39591 .48053 .41312 .49565 .40259 .48731 .38648 Metetra
.691 .647 .801 r
.48731 .38648 .49565 .40259 .5108 .39346 .50248 .37834 Metetra
.68 .655 .819 r
.50248 .37834 .5108 .39346 .526 .38545 .51772 .37123 Metetra
.669 .665 .836 r
.51772 .37123 .526 .38545 .54126 .37835 .53302 .36491 Metetra
.66 .674 .853 r
.53302 .36491 .54126 .37835 .55661 .37195 .54842 .3592 Metetra
.652 .684 .867 r
.54842 .3592 .55661 .37195 .57204 .36608 .56391 .35393 Metetra
.646 .694 .88 r
.56391 .35393 .57204 .36608 .58757 .36061 .5795 .34896 Metetra
.641 .703 .891 r
.5795 .34896 .58757 .36061 .60321 .35543 .5952 .34418 Metetra
.638 .711 .9 r
.5952 .34418 .60321 .35543 .61895 .35043 .61101 .33952 Metetra
.636 .719 .907 r
.61101 .33952 .61895 .35043 .63479 .34554 .62692 .33489 Metetra
.636 .726 .913 r
.62692 .33489 .63479 .34554 .65074 .3407 .64295 .33024 Metetra
.636 .732 .917 r
.64295 .33024 .65074 .3407 .6668 .33586 .65908 .32554 Metetra
.637 .737 .92 r
.65908 .32554 .6668 .33586 .68297 .33099 .67532 .32076 Metetra
.639 .741 .922 r
.67532 .32076 .68297 .33099 .69924 .32606 .69167 .31587 Metetra
.641 .744 .924 r
.69167 .31587 .69924 .32606 .71562 .32105 .70812 .31087 Metetra
.643 .747 .924 r
.70812 .31087 .71562 .32105 .7321 .31595 .72468 .30575 Metetra
.646 .749 .924 r
.72468 .30575 .7321 .31595 .74869 .31075 .74135 .30051 Metetra
.648 .751 .924 r
.74135 .30051 .74869 .31075 .76539 .30545 .75812 .29515 Metetra
.65 .752 .924 r
.75812 .29515 .76539 .30545 .78218 .30004 .77499 .28967 Metetra
.715 .765 .892 r
.17892 .48481 .18853 .49309 .20215 .48658 .19262 .47796 Metetra
.714 .759 .888 r
.19262 .47796 .20215 .48658 .21583 .48012 .20639 .47104 Metetra
.711 .749 .882 r
.20639 .47104 .21583 .48012 .22954 .47384 .22021 .46417 Metetra
.704 .737 .877 r
.22021 .46417 .22954 .47384 .24328 .4679 .23406 .45749 Metetra
.692 .72 .871 r
.23406 .45749 .24328 .4679 .25704 .46247 .24794 .45118 Metetra
.676 .7 .864 r
.24794 .45118 .25704 .46247 .27081 .45776 .26183 .44544 Metetra
.654 .675 .857 r
.26183 .44544 .27081 .45776 .28458 .45399 .27573 .44052 Metetra
.627 .646 .849 r
.27573 .44052 .28458 .45399 .29835 .45142 .28961 .43667 Metetra
.592 .611 .84 r
.28961 .43667 .29835 .45142 .31211 .4503 .30348 .43417 Metetra
.55 .572 .829 r
.30348 .43417 .31211 .4503 .32587 .45091 .31734 .4333 Metetra
.5 .527 .816 r
.31734 .4333 .32587 .45091 .33963 .45352 .33118 .43439 Metetra
.443 .477 .8 r
.33118 .43439 .33963 .45352 .35342 .45841 .34503 .43772 Metetra
.377 .422 .78 r
.34503 .43772 .35342 .45841 .36724 .46588 .3589 .44361 Metetra
.304 .361 .756 r
.3589 .44361 .36724 .46588 .38113 .47619 .37281 .45237 Metetra
.563 .507 .75 r
.37281 .45237 .38113 .47619 .39575 .47513 .38742 .45095 Metetra
.758 .613 .698 r
.38742 .45095 .39575 .47513 .4113 .45355 .40293 .43093 Metetra
.744 .614 .714 r
.40293 .43093 .4113 .45355 .42665 .43533 .41825 .41423 Metetra
.73 .617 .732 r
.41825 .41423 .42665 .43533 .44188 .41996 .43346 .4003 Metetra
.716 .622 .751 r
.43346 .4003 .44188 .41996 .45704 .40695 .44861 .38865 Metetra
.702 .628 .771 r
.44861 .38865 .45704 .40695 .47217 .39591 .46375 .37888 Metetra
.688 .635 .791 r
.46375 .37888 .47217 .39591 .48731 .38648 .4789 .3706 Metetra
.676 .644 .811 r
.4789 .3706 .48731 .38648 .50248 .37834 .4941 .36351 Metetra
.664 .654 .83 r
.4941 .36351 .50248 .37834 .51772 .37123 .50936 .35733 Metetra
.654 .665 .848 r
.50936 .35733 .51772 .37123 .53302 .36491 .52471 .35183 Metetra
.645 .676 .865 r
.52471 .35183 .53302 .36491 .54842 .3592 .54016 .34682 Metetra
.638 .687 .879 r
.54016 .34682 .54842 .3592 .56391 .35393 .5557 .34213 Metetra
.633 .698 .892 r
.5557 .34213 .56391 .35393 .5795 .34896 .57136 .33765 Metetra
.63 .708 .902 r
.57136 .33765 .5795 .34896 .5952 .34418 .58712 .33326 Metetra
.629 .717 .91 r
.58712 .33326 .5952 .34418 .61101 .33952 .60299 .32889 Metetra
.628 .725 .916 r
.60299 .32889 .61101 .33952 .62692 .33489 .61898 .32448 Metetra
.63 .732 .921 r
.61898 .32448 .62692 .33489 .64295 .33024 .63508 .31999 Metetra
.631 .737 .924 r
.63508 .31999 .64295 .33024 .65908 .32554 .65128 .31538 Metetra
.634 .742 .926 r
.65128 .31538 .65908 .32554 .67532 .32076 .6676 .31065 Metetra
.637 .746 .927 r
.6676 .31065 .67532 .32076 .69167 .31587 .68402 .30577 Metetra
.64 .749 .928 r
.68402 .30577 .69167 .31587 .70812 .31087 .70055 .30074 Metetra
.643 .751 .927 r
.70055 .30074 .70812 .31087 .72468 .30575 .71718 .29557 Metetra
.646 .752 .927 r
.71718 .29557 .72468 .30575 .74135 .30051 .73392 .29025 Metetra
.648 .753 .926 r
.73392 .29025 .74135 .30051 .75812 .29515 .75077 .2848 Metetra
.651 .754 .925 r
.75077 .2848 .75812 .29515 .77499 .28967 .76772 .27923 Metetra
.72 .774 .895 r
.16916 .47678 .17892 .48481 .19262 .47796 .18294 .46967 Metetra
.723 .77 .89 r
.18294 .46967 .19262 .47796 .20639 .47104 .1968 .46236 Metetra
.723 .762 .884 r
.1968 .46236 .20639 .47104 .22021 .46417 .21072 .45496 Metetra
.719 .751 .878 r
.21072 .45496 .22021 .46417 .23406 .45749 .22469 .44759 Metetra
.712 .737 .871 r
.22469 .44759 .23406 .45749 .24794 .45118 .2387 .44042 Metetra
.699 .718 .864 r
.2387 .44042 .24794 .45118 .26183 .44544 .25272 .43366 Metetra
.681 .695 .856 r
.25272 .43366 .26183 .44544 .27573 .44052 .26675 .42754 Metetra
.657 .667 .848 r
.26675 .42754 .27573 .44052 .28961 .43667 .28077 .42235 Metetra
.626 .635 .839 r
.28077 .42235 .28961 .43667 .30348 .43417 .29476 .41838 Metetra
.588 .597 .829 r
.29476 .41838 .30348 .43417 .31734 .4333 .30874 .41594 Metetra
.542 .554 .816 r
.30874 .41594 .31734 .4333 .33118 .43439 .32269 .41536 Metetra
.488 .505 .801 r
.32269 .41536 .33118 .43439 .34503 .43772 .33662 .41699 Metetra
.426 .451 .783 r
.33662 .41699 .34503 .43772 .3589 .44361 .35055 .42116 Metetra
.356 .392 .761 r
.35055 .42116 .3589 .44361 .37281 .45237 .3645 .42821 Metetra
.568 .507 .747 r
.3645 .42821 .37281 .45237 .38742 .45095 .37909 .42639 Metetra
.746 .605 .701 r
.37909 .42639 .38742 .45095 .40293 .43093 .39455 .40808 Metetra
.73 .606 .719 r
.39455 .40808 .40293 .43093 .41825 .41423 .40983 .39303 Metetra
.715 .61 .738 r
.40983 .39303 .41825 .41423 .43346 .4003 .42501 .38067 Metetra
.7 .615 .759 r
.42501 .38067 .43346 .4003 .44861 .38865 .44014 .37051 Metetra
.685 .622 .78 r
.44014 .37051 .44861 .38865 .46375 .37888 .45527 .3621 Metetra
.671 .631 .802 r
.45527 .3621 .46375 .37888 .4789 .3706 .47042 .35507 Metetra
.658 .642 .823 r
.47042 .35507 .4789 .3706 .4941 .36351 .48564 .34908 Metetra
.646 .654 .844 r
.48564 .34908 .4941 .36351 .50936 .35733 .50093 .34387 Metetra
.636 .667 .862 r
.50093 .34387 .50936 .35733 .52471 .35183 .51632 .3392 Metetra
.628 .679 .879 r
.51632 .3392 .52471 .35183 .54016 .34682 .53181 .33488 Metetra
.623 .692 .893 r
.53181 .33488 .54016 .34682 .5557 .34213 .54742 .33077 Metetra
.62 .703 .905 r
.54742 .33077 .5557 .34213 .57136 .33765 .56313 .32674 Metetra
.618 .714 .914 r
.56313 .32674 .57136 .33765 .58712 .33326 .57896 .3227 Metetra
.619 .724 .921 r
.57896 .3227 .58712 .33326 .60299 .32889 .5949 .31858 Metetra
.621 .732 .926 r
.5949 .31858 .60299 .32889 .61898 .32448 .61096 .31435 Metetra
.624 .738 .929 r
.61096 .31435 .61898 .32448 .63508 .31999 .62713 .30996 Metetra
.628 .744 .931 r
.62713 .30996 .63508 .31999 .65128 .31538 .64341 .3054 Metetra
.632 .748 .932 r
.64341 .3054 .65128 .31538 .6676 .31065 .6598 .30066 Metetra
.636 .751 .932 r
.6598 .30066 .6676 .31065 .68402 .30577 .67629 .29573 Metetra
.64 .753 .931 r
.67629 .29573 .68402 .30577 .70055 .30074 .69289 .29064 Metetra
.643 .755 .93 r
.69289 .29064 .70055 .30074 .71718 .29557 .7096 .28537 Metetra
.647 .755 .929 r
.7096 .28537 .71718 .29557 .73392 .29025 .72641 .27995 Metetra
.65 .756 .927 r
.72641 .27995 .73392 .29025 .75077 .2848 .74333 .27439 Metetra
.652 .756 .926 r
.74333 .27439 .75077 .2848 .76772 .27923 .76036 .26869 Metetra
.724 .782 .898 r
.15926 .469 .16916 .47678 .18294 .46967 .17311 .46172 Metetra
.73 .78 .893 r
.17311 .46172 .18294 .46967 .1968 .46236 .18705 .45412 Metetra
.733 .775 .887 r
.18705 .45412 .1968 .46236 .21072 .45496 .20107 .44627 Metetra
.733 .767 .88 r
.20107 .44627 .21072 .45496 .22469 .44759 .21516 .43827 Metetra
.73 .754 .872 r
.21516 .43827 .22469 .44759 .2387 .44042 .22929 .43029 Metetra
.721 .738 .864 r
.22929 .43029 .2387 .44042 .25272 .43366 .24346 .42252 Metetra
.707 .716 .856 r
.24346 .42252 .25272 .43366 .26675 .42754 .25763 .4152 Metetra
.687 .69 .847 r
.25763 .4152 .26675 .42754 .28077 .42235 .2718 .40861 Metetra
.66 .659 .838 r
.2718 .40861 .28077 .42235 .29476 .41838 .28595 .40307 Metetra
.626 .622 .827 r
.28595 .40307 .29476 .41838 .30874 .41594 .30006 .39892 Metetra
.584 .581 .815 r
.30006 .39892 .30874 .41594 .32269 .41536 .31414 .39654 Metetra
.534 .534 .801 r
.31414 .39654 .32269 .41536 .33662 .41699 .32818 .39628 Metetra
.476 .481 .784 r
.32818 .39628 .33662 .41699 .35055 .42116 .34219 .39854 Metetra
.409 .423 .764 r
.34219 .39854 .35055 .42116 .3645 .42821 .35619 .4037 Metetra
.573 .508 .744 r
.35619 .4037 .3645 .42821 .37909 .42639 .37078 .40143 Metetra
.73 .595 .705 r
.37078 .40143 .37909 .42639 .39455 .40808 .38616 .38502 Metetra
.713 .596 .724 r
.38616 .38502 .39455 .40808 .40983 .39303 .40138 .3718 Metetra
.696 .601 .745 r
.40138 .3718 .40983 .39303 .42501 .38067 .41651 .36118 Metetra
.68 .607 .768 r
.41651 .36118 .42501 .38067 .44014 .37051 .43161 .35263 Metetra
.664 .616 .791 r
.43161 .35263 .44014 .37051 .45527 .3621 .44672 .3457 Metetra
.649 .627 .815 r
.44672 .3457 .45527 .3621 .47042 .35507 .46188 .33999 Metetra
.636 .64 .838 r
.46188 .33999 .47042 .35507 .48564 .34908 .4771 .33516 Metetra
.624 .655 .86 r
.4771 .33516 .48564 .34908 .50093 .34387 .49242 .33095 Metetra
.615 .67 .879 r
.49242 .33095 .50093 .34387 .51632 .3392 .50785 .32712 Metetra
.609 .684 .895 r
.50785 .32712 .51632 .3392 .53181 .33488 .52339 .32348 Metetra
.606 .698 .909 r
.52339 .32348 .53181 .33488 .54742 .33077 .53905 .31991 Metetra
.605 .711 .919 r
.53905 .31991 .54742 .33077 .56313 .32674 .55482 .31629 Metetra
.607 .722 .927 r
.55482 .31629 .56313 .32674 .57896 .3227 .57071 .31254 Metetra
.61 .732 .932 r
.57071 .31254 .57896 .3227 .5949 .31858 .58673 .30862 Metetra
.615 .74 .935 r
.58673 .30862 .5949 .31858 .61096 .31435 .60285 .30449 Metetra
.62 .746 .937 r
.60285 .30449 .61096 .31435 .62713 .30996 .61909 .30014 Metetra
.625 .751 .937 r
.61909 .30014 .62713 .30996 .64341 .3054 .63545 .29556 Metetra
.631 .754 .936 r
.63545 .29556 .64341 .3054 .6598 .30066 .65191 .29076 Metetra
.636 .756 .935 r
.65191 .29076 .6598 .30066 .67629 .29573 .66848 .28574 Metetra
.641 .758 .933 r
.66848 .28574 .67629 .29573 .69289 .29064 .68515 .28053 Metetra
.645 .758 .931 r
.68515 .28053 .69289 .29064 .7096 .28537 .70193 .27514 Metetra
.648 .758 .929 r
.70193 .27514 .7096 .28537 .72641 .27995 .71882 .26958 Metetra
.652 .758 .927 r
.71882 .26958 .72641 .27995 .74333 .27439 .73581 .26387 Metetra
.654 .757 .925 r
.73581 .26387 .74333 .27439 .76036 .26869 .75291 .25803 Metetra
.725 .789 .903 r
.1492 .46142 .15926 .469 .17311 .46172 .16311 .45409 Metetra
.734 .79 .897 r
.16311 .45409 .17311 .46172 .18705 .45412 .17713 .44632 Metetra
.741 .788 .891 r
.17713 .44632 .18705 .45412 .20107 .44627 .19124 .43815 Metetra
.745 .783 .883 r
.19124 .43815 .20107 .44627 .21516 .43827 .20544 .42964 Metetra
.746 .773 .875 r
.20544 .42964 .21516 .43827 .22929 .43029 .2197 .42092 Metetra
.742 .758 .866 r
.2197 .42092 .22929 .43029 .24346 .42252 .23401 .41218 Metetra
.732 .739 .857 r
.23401 .41218 .24346 .42252 .25763 .4152 .24835 .40365 Metetra
.717 .714 .847 r
.24835 .40365 .25763 .4152 .2718 .40861 .26269 .39561 Metetra
.694 .684 .836 r
.26269 .39561 .2718 .40861 .28595 .40307 .27701 .38841 Metetra
.664 .649 .825 r
.27701 .38841 .28595 .40307 .30006 .39892 .29129 .38242 Metetra
.626 .609 .813 r
.29129 .38242 .30006 .39892 .31414 .39654 .30552 .37804 Metetra
.58 .563 .799 r
.30552 .37804 .31414 .39654 .32818 .39628 .3197 .3757 Metetra
.525 .512 .784 r
.3197 .3757 .32818 .39628 .34219 .39854 .33382 .37582 Metetra
.462 .455 .765 r
.33382 .37582 .34219 .39854 .35619 .4037 .3479 .37884 Metetra
.58 .51 .741 r
.3479 .37884 .35619 .4037 .37078 .40143 .36248 .37607 Metetra
.712 .582 .709 r
.36248 .37607 .37078 .40143 .38616 .38502 .37777 .36178 Metetra
.693 .585 .73 r
.37777 .36178 .38616 .38502 .40138 .3718 .3929 .3506 Metetra
.674 .59 .753 r
.3929 .3506 .40138 .3718 .41651 .36118 .40797 .34191 Metetra
.655 .598 .779 r
.40797 .34191 .41651 .36118 .43161 .35263 .42302 .33515 Metetra
.638 .61 .805 r
.42302 .33515 .43161 .35263 .44672 .3457 .4381 .32982 Metetra
.622 .624 .831 r
.4381 .32982 .44672 .3457 .46188 .33999 .45324 .32553 Metetra
.608 .64 .856 r
.45324 .32553 .46188 .33999 .4771 .33516 .46848 .32193 Metetra
.598 .657 .879 r
.46848 .32193 .4771 .33516 .49242 .33095 .48382 .31873 Metetra
.59 .675 .899 r
.48382 .31873 .49242 .33095 .50785 .32712 .49929 .31573 Metetra
.587 .692 .914 r
.49929 .31573 .50785 .32712 .52339 .32348 .51487 .31274 Metetra
.587 .707 .926 r
.51487 .31274 .52339 .32348 .53905 .31991 .53058 .30965 Metetra
.59 .721 .935 r
.53058 .30965 .53905 .31991 .55482 .31629 .54642 .30636 Metetra
.595 .733 .94 r
.54642 .30636 .55482 .31629 .57071 .31254 .56238 .30283 Metetra
.602 .742 .943 r
.56238 .30283 .57071 .31254 .58673 .30862 .57846 .29901 Metetra
.61 .75 .944 r
.57846 .29901 .58673 .30862 .60285 .30449 .59466 .29491 Metetra
.618 .755 .944 r
.59466 .29491 .60285 .30449 .61909 .30014 .61097 .29051 Metetra
.625 .758 .942 r
.61097 .29051 .61909 .30014 .63545 .29556 .6274 .28584 Metetra
.632 .76 .94 r
.6274 .28584 .63545 .29556 .65191 .29076 .64393 .28091 Metetra
.638 .761 .937 r
.64393 .28091 .65191 .29076 .66848 .28574 .66057 .27575 Metetra
.643 .761 .935 r
.66057 .27575 .66848 .28574 .68515 .28053 .67732 .27038 Metetra
.648 .761 .932 r
.67732 .27038 .68515 .28053 .70193 .27514 .69417 .26481 Metetra
.651 .76 .929 r
.69417 .26481 .70193 .27514 .71882 .26958 .71113 .25909 Metetra
.654 .759 .927 r
.71113 .25909 .71882 .26958 .73581 .26387 .72819 .25321 Metetra
.656 .758 .925 r
.72819 .25321 .73581 .26387 .75291 .25803 .74536 .24721 Metetra
.722 .794 .908 r
.139 .45397 .1492 .46142 .16311 .45409 .15295 .44674 Metetra
.734 .798 .903 r
.15295 .44674 .16311 .45409 .17713 .44632 .16703 .43896 Metetra
.745 .8 .897 r
.16703 .43896 .17713 .44632 .19124 .43815 .18122 .43062 Metetra
.754 .799 .889 r
.18122 .43062 .19124 .43815 .20544 .42964 .19552 .42175 Metetra
.76 .792 .879 r
.19552 .42175 .20544 .42964 .2197 .42092 .20991 .41243 Metetra
.761 .781 .869 r
.20991 .41243 .2197 .42092 .23401 .41218 .22437 .40282 Metetra
.757 .764 .858 r
.22437 .40282 .23401 .41218 .24835 .40365 .23887 .39311 Metetra
.746 .741 .847 r
.23887 .39311 .24835 .40365 .26269 .39561 .2534 .38361 Metetra
.728 .713 .835 r
.2534 .38361 .26269 .39561 .27701 .38841 .26791 .37466 Metetra
.703 .678 .823 r
.26791 .37466 .27701 .38841 .29129 .38242 .28239 .36666 Metetra
.669 .639 .811 r
.28239 .36666 .29129 .38242 .30552 .37804 .29681 .36007 Metetra
.627 .594 .797 r
.29681 .36007 .30552 .37804 .3197 .3757 .31116 .35538 Metetra
.576 .543 .781 r
.31116 .35538 .3197 .3757 .33382 .37582 .32543 .35307 Metetra
.516 .488 .764 r
.32543 .35307 .33382 .37582 .3479 .37884 .33963 .35364 Metetra
.587 .512 .737 r
.33963 .35364 .3479 .37884 .36248 .37607 .35421 .3503 Metetra
.688 .567 .713 r
.35421 .3503 .36248 .37607 .37777 .36178 .36936 .33842 Metetra
.666 .57 .737 r
.36936 .33842 .37777 .36178 .3929 .3506 .38438 .32957 Metetra
.644 .577 .764 r
.38438 .32957 .3929 .3506 .40797 .34191 .39936 .32306 Metetra
.623 .587 .793 r
.39936 .32306 .40797 .34191 .42302 .33515 .41434 .31827 Metetra
.604 .602 .823 r
.41434 .31827 .42302 .33515 .4381 .32982 .42938 .31471 Metetra
.587 .62 .852 r
.42938 .31471 .4381 .32982 .45324 .32553 .44451 .31193 Metetra
.573 .64 .879 r
.44451 .31193 .45324 .32553 .46848 .32193 .45975 .30959 Metetra
.564 .662 .903 r
.45975 .30959 .46848 .32193 .48382 .31873 .47512 .30742 Metetra
.56 .683 .922 r
.47512 .30742 .48382 .31873 .49929 .31573 .49062 .3052 Metetra
.561 .703 .936 r
.49062 .3052 .49929 .31573 .51487 .31274 .50625 .30278 Metetra
.567 .72 .945 r
.50625 .30278 .51487 .31274 .53058 .30965 .52202 .30008 Metetra
.575 .735 .951 r
.52202 .30008 .53058 .30965 .54642 .30636 .53792 .29702 Metetra
.586 .746 .953 r
.53792 .29702 .54642 .30636 .56238 .30283 .55395 .29358 Metetra
.597 .755 .953 r
.55395 .29358 .56238 .30283 .57846 .29901 .5701 .28976 Metetra
.608 .761 .952 r
.5701 .28976 .57846 .29901 .59466 .29491 .58637 .28556 Metetra
.619 .764 .949 r
.58637 .28556 .59466 .29491 .61097 .29051 .60276 .28103 Metetra
.628 .766 .946 r
.60276 .28103 .61097 .29051 .6274 .28584 .61925 .27618 Metetra
.635 .766 .942 r
.61925 .27618 .6274 .28584 .64393 .28091 .63585 .27106 Metetra
.642 .766 .938 r
.63585 .27106 .64393 .28091 .66057 .27575 .65257 .26569 Metetra
.647 .764 .934 r
.65257 .26569 .66057 .27575 .67732 .27038 .66938 .2601 Metetra
.651 .763 .931 r
.66938 .2601 .67732 .27038 .69417 .26481 .68631 .25434 Metetra
.654 .761 .928 r
.68631 .25434 .69417 .26481 .71113 .25909 .70334 .24842 Metetra
.657 .759 .925 r
.70334 .24842 .71113 .25909 .72819 .25321 .72048 .24236 Metetra
.659 .757 .923 r
.72048 .24236 .72819 .25321 .74536 .24721 .73772 .2362 Metetra
.716 .796 .914 r
.12866 .44654 .139 .45397 .15295 .44674 .14264 .43955 Metetra
.73 .804 .91 r
.14264 .43955 .15295 .44674 .16703 .43896 .15675 .43195 Metetra
.745 .811 .904 r
.15675 .43195 .16703 .43896 .18122 .43062 .17099 .42367 Metetra
.759 .814 .896 r
.17099 .42367 .18122 .43062 .19552 .42175 .18537 .41466 Metetra
.771 .813 .886 r
.18537 .41466 .19552 .42175 .20991 .41243 .19987 .40496 Metetra
.778 .805 .875 r
.19987 .40496 .20991 .41243 .22437 .40282 .21447 .39464 Metetra
.78 .792 .862 r
.21447 .39464 .22437 .40282 .23887 .39311 .22915 .38388 Metetra
.776 .772 .849 r
.22915 .38388 .23887 .39311 .2534 .38361 .24388 .37294 Metetra
.763 .745 .836 r
.24388 .37294 .2534 .38361 .26791 .37466 .25861 .36216 Metetra
.743 .711 .822 r
.25861 .36216 .26791 .37466 .28239 .36666 .27332 .35199 Metetra
.713 .672 .808 r
.27332 .35199 .28239 .36666 .29681 .36007 .28798 .34294 Metetra
.675 .627 .793 r
.28798 .34294 .29681 .36007 .31116 .35538 .30255 .33555 Metetra
.627 .576 .777 r
.30255 .33555 .31116 .35538 .32543 .35307 .31701 .33042 Metetra
.571 .521 .76 r
.31701 .33042 .32543 .35307 .33963 .35364 .33137 .32814 Metetra
.595 .515 .734 r
.33137 .32814 .33963 .35364 .35421 .3503 .34596 .32411 Metetra
.657 .547 .718 r
.34596 .32411 .35421 .3503 .36936 .33842 .36094 .31506 Metetra
.631 .551 .746 r
.36094 .31506 .36936 .33842 .38438 .32957 .37581 .3089 Metetra
.605 .559 .777 r
.37581 .3089 .38438 .32957 .39936 .32306 .39067 .30488 Metetra
.579 .574 .811 r
.39067 .30488 .39936 .32306 .41434 .31827 .40557 .30232 Metetra
.556 .593 .846 r
.40557 .30232 .41434 .31827 .42938 .31471 .42055 .30068 Metetra
.538 .617 .879 r
.42055 .30068 .42938 .31471 .44451 .31193 .43566 .29951 Metetra
.525 .643 .908 r
.43566 .29951 .44451 .31193 .45975 .30959 .4509 .29845 Metetra
.52 .67 .931 r
.4509 .29845 .45975 .30959 .47512 .30742 .46629 .29725 Metetra
.523 .696 .948 r
.46629 .29725 .47512 .30742 .49062 .3052 .48183 .29572 Metetra
.532 .718 .958 r
.48183 .29572 .49062 .3052 .50625 .30278 .49752 .29375 Metetra
.546 .737 .964 r
.49752 .29375 .50625 .30278 .52202 .30008 .51335 .29127 Metetra
.563 .752 .966 r
.51335 .29127 .52202 .30008 .53792 .29702 .52932 .28828 Metetra
.58 .762 .965 r
.52932 .28828 .53792 .29702 .55395 .29358 .54542 .28477 Metetra
.597 .769 .962 r
.54542 .28477 .55395 .29358 .5701 .28976 .56164 .2808 Metetra
.611 .772 .957 r
.56164 .2808 .5701 .28976 .58637 .28556 .57798 .27639 Metetra
.623 .773 .952 r
.57798 .27639 .58637 .28556 .60276 .28103 .59444 .2716 Metetra
.633 .772 .947 r
.59444 .2716 .60276 .28103 .61925 .27618 .611 .26648 Metetra
.641 .77 .942 r
.611 .26648 .61925 .27618 .63585 .27106 .62768 .26109 Metetra
.647 .768 .937 r
.62768 .26109 .63585 .27106 .65257 .26569 .64446 .25546 Metetra
.652 .765 .932 r
.64446 .25546 .65257 .26569 .66938 .2601 .66135 .24963 Metetra
.655 .763 .929 r
.66135 .24963 .66938 .2601 .68631 .25434 .67834 .24364 Metetra
.658 .76 .925 r
.67834 .24364 .68631 .25434 .70334 .24842 .69545 .23752 Metetra
.66 .758 .923 r
.69545 .23752 .70334 .24842 .72048 .24236 .71266 .23128 Metetra
.661 .756 .921 r
.71266 .23128 .72048 .24236 .73772 .2362 .72997 .22494 Metetra
.704 .793 .919 r
.11821 .43895 .12866 .44654 .14264 .43955 .13219 .43236 Metetra
.72 .805 .918 r
.13219 .43236 .14264 .43955 .15675 .43195 .1463 .42515 Metetra
.738 .817 .914 r
.1463 .42515 .15675 .43195 .17099 .42367 .16057 .41719 Metetra
.757 .827 .907 r
.16057 .41719 .17099 .42367 .18537 .41466 .175 .40836 Metetra
.775 .833 .897 r
.175 .40836 .18537 .41466 .19987 .40496 .18958 .39859 Metetra
.79 .832 .884 r
.18958 .39859 .19987 .40496 .21447 .39464 .2043 .38787 Metetra
.801 .824 .869 r
.2043 .38787 .21447 .39464 .22915 .38388 .21914 .37629 Metetra
.804 .807 .854 r
.21914 .37629 .22915 .38388 .24388 .37294 .23407 .36403 Metetra
.799 .782 .837 r
.23407 .36403 .24388 .37294 .25861 .36216 .24905 .35144 Metetra
.784 .75 .821 r
.24905 .35144 .25861 .36216 .27332 .35199 .26403 .33894 Metetra
.76 .71 .805 r
.26403 .33894 .27332 .35199 .28798 .34294 .27897 .32712 Metetra
.726 .664 .789 r
.27897 .32712 .28798 .34294 .30255 .33555 .29382 .31661 Metetra
.681 .612 .772 r
.29382 .31661 .30255 .33555 .31701 .33042 .30855 .30814 Metetra
.628 .556 .754 r
.30855 .30814 .31701 .33042 .33137 .32814 .32313 .30242 Metetra
.606 .519 .731 r
.32313 .30242 .33137 .32814 .34596 .32411 .33773 .29756 Metetra
.614 .52 .724 r
.33773 .29756 .34596 .32411 .36094 .31506 .35248 .2919 Metetra
.58 .524 .757 r
.35248 .2919 .36094 .31506 .37581 .3089 .36717 .28894 Metetra
.546 .535 .795 r
.36717 .28894 .37581 .3089 .39067 .30488 .38187 .28781 Metetra
.513 .555 .836 r
.38187 .28781 .39067 .30488 .40557 .30232 .39666 .28778 Metetra
.484 .582 .877 r
.39666 .28778 .40557 .30232 .42055 .30068 .41158 .28824 Metetra
.464 .614 .913 r
.41158 .28824 .42055 .30068 .43566 .29951 .42666 .28873 Metetra
.456 .649 .942 r
.42666 .28873 .43566 .29951 .4509 .29845 .4419 .2889 Metetra
.46 .683 .962 r
.4419 .2889 .4509 .29845 .46629 .29725 .45732 .28854 Metetra
.477 .715 .974 r
.45732 .28854 .46629 .29725 .48183 .29572 .47291 .2875 Metetra
.501 .741 .98 r
.47291 .2875 .48183 .29572 .49752 .29375 .48866 .28574 Metetra
.53 .76 .981 r
.48866 .28574 .49752 .29375 .51335 .29127 .50456 .28326 Metetra
.557 .773 .979 r
.50456 .28326 .51335 .29127 .52932 .28828 .5206 .28009 Metetra
.582 .781 .974 r
.5206 .28009 .52932 .28828 .54542 .28477 .53678 .27632 Metetra
.603 .783 .967 r
.53678 .27632 .54542 .28477 .56164 .2808 .55307 .272 Metetra
.62 .783 .959 r
.55307 .272 .56164 .2808 .57798 .27639 .56949 .26722 Metetra
.632 .78 .952 r
.56949 .26722 .57798 .27639 .59444 .2716 .58601 .26207 Metetra
.642 .776 .945 r
.58601 .26207 .59444 .2716 .611 .26648 .60265 .2566 Metetra
.649 .772 .939 r
.60265 .2566 .611 .26648 .62768 .26109 .61939 .25088 Metetra
.654 .768 .933 r
.61939 .25088 .62768 .26109 .64446 .25546 .63625 .24496 Metetra
.657 .764 .929 r
.63625 .24496 .64446 .25546 .66135 .24963 .6532 .23888 Metetra
.659 .761 .925 r
.6532 .23888 .66135 .24963 .67834 .24364 .67027 .23266 Metetra
.661 .758 .922 r
.67027 .23266 .67834 .24364 .69545 .23752 .68745 .22634 Metetra
.662 .756 .92 r
.68745 .22634 .69545 .23752 .71266 .23128 .70473 .21992 Metetra
.663 .754 .918 r
.70473 .21992 .71266 .23128 .72997 .22494 .72213 .21344 Metetra
.69 .784 .923 r
.10768 .43098 .11821 .43895 .13219 .43236 .12164 .42487 Metetra
.703 .798 .924 r
.12164 .42487 .13219 .43236 .1463 .42515 .13573 .41825 Metetra
.721 .816 .924 r
.13573 .41825 .1463 .42515 .16057 .41719 .14998 .41091 Metetra
.744 .834 .92 r
.14998 .41091 .16057 .41719 .175 .40836 .1644 .40266 Metetra
.768 .849 .912 r
.1644 .40266 .175 .40836 .18958 .39859 .17901 .39331 Metetra
.793 .859 .899 r
.17901 .39331 .18958 .39859 .2043 .38787 .1938 .38269 Metetra
.814 .86 .882 r
.1938 .38269 .2043 .38787 .21914 .37629 .20877 .37073 Metetra
.829 .85 .863 r
.20877 .37073 .21914 .37629 .23407 .36403 .22389 .35751 Metetra
.834 .83 .843 r
.22389 .35751 .23407 .36403 .24905 .35144 .23913 .34325 Metetra
.829 .798 .823 r
.23913 .34325 .24905 .35144 .26403 .33894 .25442 .32836 Metetra
.811 .757 .803 r
.25442 .32836 .26403 .33894 .27897 .32712 .2697 .31344 Metetra
.781 .709 .784 r
.2697 .31344 .27897 .32712 .29382 .31661 .28491 .29926 Metetra
.74 .654 .765 r
.28491 .29926 .29382 .31661 .30855 .30814 .29999 .28669 Metetra
.688 .595 .746 r
.29999 .28669 .30855 .30814 .32313 .30242 .31489 .27668 Metetra
.619 .527 .728 r
.31489 .27668 .32313 .30242 .33773 .29756 .32954 .27076 Metetra
.549 .479 .729 r
.32954 .27076 .33773 .29756 .35248 .2919 .34397 .26933 Metetra
.499 .482 .77 r
.34397 .26933 .35248 .2919 .36717 .28894 .3584 .2703 Metetra
.447 .497 .818 r
.3584 .2703 .36717 .28894 .38187 .28781 .3729 .27262 Metetra
.396 .523 .868 r
.3729 .27262 .38187 .28781 .39666 .28778 .38756 .27546 Metetra
.357 .561 .913 r
.38756 .27546 .39666 .28778 .41158 .28824 .4024 .27816 Metetra
.337 .606 .947 r
.4024 .27816 .41158 .28824 .42666 .28873 .41746 .28026 Metetra
.344 .654 .97 r
.41746 .28026 .42666 .28873 .4419 .2889 .43272 .28146 Metetra
.376 .701 .984 r
.43272 .28146 .4419 .2889 .45732 .28854 .44818 .28163 Metetra
.424 .741 .992 r
.44818 .28163 .45732 .28854 .47291 .2875 .46384 .28072 Metetra
.476 .771 .996 r
.46384 .28072 .47291 .2875 .48866 .28574 .47966 .27879 Metetra
.525 .789 .993 r
.47966 .27879 .48866 .28574 .50456 .28326 .49564 .27594 Metetra
.565 .798 .987 r
.49564 .27594 .50456 .28326 .5206 .28009 .51176 .27228 Metetra
.596 .799 .977 r
.51176 .27228 .5206 .28009 .53678 .27632 .52801 .26797 Metetra
.618 .795 .967 r
.52801 .26797 .53678 .27632 .55307 .272 .54439 .26311 Metetra
.634 .789 .956 r
.54439 .26311 .55307 .272 .56949 .26722 .56087 .25783 Metetra
.645 .782 .947 r
.56087 .25783 .56949 .26722 .58601 .26207 .57747 .25222 Metetra
.652 .775 .939 r
.57747 .25222 .58601 .26207 .60265 .2566 .59418 .24635 Metetra
.656 .769 .933 r
.59418 .24635 .60265 .2566 .61939 .25088 .611 .24029 Metetra
.659 .764 .927 r
.611 .24029 .61939 .25088 .63625 .24496 .62792 .23408 Metetra
.661 .76 .923 r
.62792 .23408 .63625 .24496 .6532 .23888 .64495 .22776 Metetra
.663 .757 .92 r
.64495 .22776 .6532 .23888 .67027 .23266 .66209 .22134 Metetra
.663 .755 .918 r
.66209 .22134 .67027 .23266 .68745 .22634 .67934 .21485 Metetra
.664 .753 .917 r
.67934 .21485 .68745 .22634 .70473 .21992 .6967 .20829 Metetra
.664 .752 .915 r
.6967 .20829 .70473 .21992 .72213 .21344 .71417 .20168 Metetra
.675 .769 .922 r
.0971 .42241 .10768 .43098 .12164 .42487 .11103 .41676 Metetra
.684 .783 .926 r
.11103 .41676 .12164 .42487 .13573 .41825 .12509 .41078 Metetra
.697 .802 .93 r
.12509 .41078 .13573 .41825 .14998 .41091 .13928 .4043 Metetra
.718 .826 .932 r
.13928 .4043 .14998 .41091 .1644 .40266 .15364 .39705 Metetra
.745 .853 .93 r
.15364 .39705 .1644 .40266 .17901 .39331 .16819 .38872 Metetra
.777 .879 .921 r
.16819 .38872 .17901 .39331 .1938 .38269 .18297 .37899 Metetra
.811 .897 .903 r
.18297 .37899 .1938 .38269 .20877 .37073 .19798 .36752 Metetra
.842 .902 .88 r
.19798 .36752 .20877 .37073 .22389 .35751 .21322 .35413 Metetra
.864 .891 .853 r
.21322 .35413 .22389 .35751 .23913 .34325 .22868 .33877 Metetra
.874 .864 .826 r
.22868 .33877 .23913 .34325 .25442 .32836 .2443 .32169 Metetra
.867 .822 .802 r
.2443 .32169 .25442 .32836 .2697 .31344 .25999 .30343 Metetra
.845 .769 .779 r
.25999 .30343 .2697 .31344 .28491 .29926 .27567 .28486 Metetra
.806 .708 .757 r
.27567 .28486 .28491 .29926 .29999 .28669 .29124 .26708 Metetra
.755 .642 .735 r
.29124 .26708 .29999 .28669 .31489 .27668 .30661 .25141 Metetra
.639 .541 .728 r
.30661 .25141 .31489 .27668 .32954 .27076 .32134 .24406 Metetra
.433 .407 .731 r
.32134 .24406 .32954 .27076 .34397 .26933 .33534 .24823 Metetra
.341 .403 .781 r
.33534 .24823 .34397 .26933 .3584 .2703 .34941 .25422 Metetra
.238 .415 .836 r
.34941 .25422 .3584 .2703 .3729 .27262 .36367 .26074 Metetra
.14 .444 .878 r
.36367 .26074 .3729 .27262 .38756 .27546 .37817 .26679 Metetra
.085 .491 .898 r
.37817 .26679 .38756 .27546 .4024 .27816 .39294 .2717 Metetra
.096 .557 .91 r
.39294 .2717 .4024 .27816 .41746 .28026 .40799 .27506 Metetra
.166 .635 .931 r
.40799 .27506 .41746 .28026 .43272 .28146 .4233 .27674 Metetra
.271 .713 .962 r
.4233 .27674 .43272 .28146 .44818 .28163 .43885 .27677 Metetra
.382 .775 .988 r
.43885 .27677 .44818 .28163 .46384 .28072 .45459 .27532 Metetra
.477 .81 .999 r
.45459 .27532 .46384 .28072 .47966 .27879 .47051 .27262 Metetra
.547 .822 .997 r
.47051 .27262 .47966 .27879 .49564 .27594 .48659 .2689 Metetra
.594 .819 .986 r
.48659 .2689 .49564 .27594 .51176 .27228 .5028 .26438 Metetra
.623 .809 .972 r
.5028 .26438 .51176 .27228 .52801 .26797 .51913 .25927 Metetra
.64 .797 .958 r
.51913 .25927 .52801 .26797 .54439 .26311 .53558 .25373 Metetra
.651 .786 .946 r
.53558 .25373 .54439 .26311 .56087 .25783 .55214 .24787 Metetra
.657 .776 .937 r
.55214 .24787 .56087 .25783 .57747 .25222 .56882 .2418 Metetra
.66 .768 .929 r
.56882 .2418 .57747 .25222 .59418 .24635 .58559 .23556 Metetra
.662 .762 .924 r
.58559 .23556 .59418 .24635 .611 .24029 .60248 .22921 Metetra
.663 .758 .921 r
.60248 .22921 .611 .24029 .62792 .23408 .61947 .22278 Metetra
.664 .755 .918 r
.61947 .22278 .62792 .23408 .64495 .22776 .63658 .21627 Metetra
.664 .753 .916 r
.63658 .21627 .64495 .22776 .66209 .22134 .65379 .2097 Metetra
.664 .751 .915 r
.65379 .2097 .66209 .22134 .67934 .21485 .67112 .20309 Metetra
.664 .75 .914 r
.67112 .20309 .67934 .21485 .6967 .20829 .68856 .19642 Metetra
.664 .75 .914 r
.68856 .19642 .6967 .20829 .71417 .20168 .70611 .18971 Metetra
.666 .755 .917 r
.08647 .41321 .0971 .42241 .11103 .41676 .10041 .40779 Metetra
.668 .762 .921 r
.10041 .40779 .11103 .41676 .12509 .41078 .11444 .40227 Metetra
.674 .775 .926 r
.11444 .40227 .12509 .41078 .13928 .4043 .12858 .39654 Metetra
.684 .796 .934 r
.12858 .39654 .13928 .4043 .15364 .39705 .14285 .39045 Metetra
.702 .828 .943 r
.14285 .39045 .15364 .39705 .16819 .38872 .15727 .38369 Metetra
.73 .87 .947 r
.15727 .38369 .16819 .38872 .18297 .37899 .17189 .37583 Metetra
.768 .915 .937 r
.17189 .37583 .18297 .37899 .19798 .36752 .18678 .36628 Metetra
.813 .95 .909 r
.18678 .36628 .19798 .36752 .21322 .35413 .20197 .35439 Metetra
.859 .964 .868 r
.20197 .35439 .21322 .35413 .22868 .33877 .2175 .33956 Metetra
.899 .953 .827 r
.2175 .33956 .22868 .33877 .2443 .32169 .23336 .32145 Metetra
.922 .918 .794 r
.23336 .32145 .2443 .32169 .25999 .30343 .2495 .30018 Metetra
.919 .861 .767 r
.2495 .30018 .25999 .30343 .27567 .28486 .26579 .2765 Metetra
.889 .788 .743 r
.26579 .2765 .27567 .28486 .29124 .26708 .28207 .25178 Metetra
.837 .707 .72 r
.28207 .25178 .29124 .26708 .30661 .25141 .29818 .22792 Metetra
.672 .572 .735 r
.29818 .22792 .30661 .25141 .32134 .24406 .31306 .21844 Metetra
.171 .246 .699 r
.31306 .21844 .32134 .24406 .33534 .24823 .3264 .23074 Metetra
0 .184 .714 r
.3264 .23074 .33534 .24823 .34941 .25422 .33998 .24359 Metetra
0 .138 .652 r
.33998 .24359 .34941 .25422 .36367 .26074 .35393 .25522 Metetra
.417 0 0 r
.35393 .25522 .36367 .26074 .37817 .26679 .36829 .26448 Metetra
.366 0 0 r
.36829 .26448 .37817 .26679 .39294 .2717 .38305 .27084 Metetra
.224 0 0 r
.38305 .27084 .39294 .2717 .40799 .27506 .39818 .27425 Metetra
0 .563 .731 r
.39818 .27425 .40799 .27506 .4233 .27674 .41361 .275 Metetra
.221 .733 .881 r
.41361 .275 .4233 .27674 .43885 .27677 .4293 .27358 Metetra
.419 .83 .973 r
.4293 .27358 .43885 .27677 .45459 .27532 .44518 .2705 Metetra
.543 .854 .998 r
.44518 .2705 .45459 .27532 .47051 .27262 .46123 .26623 Metetra
.607 .842 .988 r
.46123 .26623 .47051 .27262 .48659 .2689 .47741 .26114 Metetra
.638 .819 .97 r
.47741 .26114 .48659 .2689 .5028 .26438 .49371 .25552 Metetra
.653 .798 .952 r
.49371 .25552 .5028 .26438 .51913 .25927 .51013 .24956 Metetra
.66 .782 .939 r
.51013 .24956 .51913 .25927 .53558 .25373 .52666 .24338 Metetra
.663 .77 .93 r
.52666 .24338 .53558 .25373 .55214 .24787 .5433 .23707 Metetra
.664 .762 .923 r
.5433 .23707 .55214 .24787 .56882 .2418 .56004 .23067 Metetra
.664 .757 .919 r
.56004 .23067 .56882 .2418 .58559 .23556 .57689 .2242 Metetra
.664 .754 .917 r
.57689 .2242 .58559 .23556 .60248 .22921 .59385 .21767 Metetra
.664 .751 .915 r
.59385 .21767 .60248 .22921 .61947 .22278 .61092 .2111 Metetra
.664 .75 .914 r
.61092 .2111 .61947 .22278 .63658 .21627 .6281 .20448 Metetra
.664 .749 .914 r
.6281 .20448 .63658 .21627 .65379 .2097 .64539 .19782 Metetra
.664 .749 .913 r
.64539 .19782 .65379 .2097 .67112 .20309 .6628 .19111 Metetra
.664 .749 .913 r
.6628 .19111 .67112 .20309 .68856 .19642 .68032 .18435 Metetra
.664 .749 .913 r
.68032 .18435 .68856 .19642 .70611 .18971 .69795 .17756 Metetra
.664 .749 .913 r
.07576 .40364 .08647 .41321 .10041 .40779 .08974 .39822 Metetra
.664 .75 .914 r
.08974 .39822 .10041 .40779 .11444 .40227 .10381 .39275 Metetra
.665 .753 .916 r
.10381 .39275 .11444 .40227 .12858 .39654 .11795 .38726 Metetra
.665 .759 .92 r
.11795 .38726 .12858 .39654 .14285 .39045 .13219 .38171 Metetra
.668 .774 .929 r
.13219 .38171 .14285 .39045 .15727 .38369 .14651 .37608 Metetra
.673 .803 .945 r
.14651 .37608 .15727 .38369 .17189 .37583 .16095 .37023 Metetra
.683 .855 .965 r
.16095 .37023 .17189 .37583 .18678 .36628 .17553 .36383 Metetra
.698 .928 .966 r
.17553 .36383 .18678 .36628 .20197 .35439 .19033 .35616 Metetra
.708 .982 .896 r
.19033 .35616 .20197 .35439 .2175 .33956 .20548 .34596 Metetra
.726 .98 .76 r
.20548 .34596 .2175 .33956 .23336 .32145 .22113 .33147 Metetra
.8 .971 .669 r
.22113 .33147 .23336 .32145 .2495 .30018 .23741 .31084 Metetra
.91 .967 .658 r
.23741 .31084 .2495 .30018 .26579 .2765 .25432 .28304 Metetra
.974 .92 .676 r
.25432 .28304 .26579 .2765 .28207 .25178 .27169 .24882 Metetra
.951 .82 .679 r
.27169 .24882 .28207 .25178 .29818 .22792 .28915 .21107 Metetra
.753 .668 .767 r
.28915 .21107 .29818 .22792 .31306 .21844 .30438 .19766 Metetra
0 0 .337 r
.30438 .19766 .31306 .21844 .3264 .23074 .31657 .22414 Metetra
.817 .338 0 r
.31657 .22414 .3264 .23074 .33998 .24359 .32944 .24705 Metetra
.741 .255 .087 r
.32944 .24705 .33998 .24359 .35393 .25522 .34311 .26376 Metetra
.616 .131 .08 r
.34311 .26376 .35393 .25522 .36829 .26448 .35754 .27365 Metetra
.467 0 0 r
.35754 .27365 .36829 .26448 .38305 .27084 .37255 .2776 Metetra
.251 0 0 r
.37255 .2776 .38305 .27084 .39818 .27425 .38799 .27719 Metetra
.38799 .27719 .39818 .27425 .41361 .275 .40371 .27398 Metetra
.437 .868 .923 r
.40371 .27398 .41361 .275 .4293 .27358 .41963 .26917 Metetra
.6 .885 .991 r
.41963 .26917 .4293 .27358 .44518 .2705 .43568 .26353 Metetra
.648 .841 .974 r
.43568 .26353 .44518 .2705 .46123 .26623 .45186 .25748 Metetra
.661 .803 .951 r
.45186 .25748 .46123 .26623 .47741 .26114 .46814 .25124 Metetra
.665 .779 .934 r
.46814 .25124 .47741 .26114 .49371 .25552 .48453 .24491 Metetra
.665 .765 .925 r
.48453 .24491 .49371 .25552 .51013 .24956 .50103 .23851 Metetra
.665 .757 .919 r
.50103 .23851 .51013 .24956 .52666 .24338 .51763 .23207 Metetra
.665 .753 .916 r
.51763 .23207 .52666 .24338 .5433 .23707 .53434 .22558 Metetra
.665 .751 .914 r
.53434 .22558 .5433 .23707 .56004 .23067 .55115 .21905 Metetra
.664 .75 .914 r
.55115 .21905 .56004 .23067 .57689 .2242 .56808 .21248 Metetra
.664 .749 .913 r
.56808 .21248 .57689 .2242 .59385 .21767 .58511 .20586 Metetra
.664 .749 .913 r
.58511 .20586 .59385 .21767 .61092 .2111 .60226 .19921 Metetra
.664 .749 .913 r
.60226 .19921 .61092 .2111 .6281 .20448 .61951 .1925 Metetra
.664 .749 .913 r
.61951 .1925 .6281 .20448 .64539 .19782 .63689 .18576 Metetra
.664 .749 .913 r
.63689 .18576 .64539 .19782 .6628 .19111 .65437 .17897 Metetra
.664 .749 .913 r
.65437 .17897 .6628 .19111 .68032 .18435 .67197 .17213 Metetra
.664 .749 .913 r
.67197 .17213 .68032 .18435 .69795 .17756 .68969 .16525 Metetra
.664 .749 .913 r
.06494 .39395 .07576 .40364 .08974 .39822 .07897 .38846 Metetra
.664 .749 .913 r
.07897 .38846 .08974 .39822 .10381 .39275 .09308 .38294 Metetra
.664 .749 .913 r
.09308 .38294 .10381 .39275 .11795 .38726 .10727 .37739 Metetra
.664 .749 .913 r
.10727 .37739 .11795 .38726 .13219 .38171 .12155 .3718 Metetra
.664 .749 .913 r
.12155 .3718 .13219 .38171 .14651 .37608 .13591 .36618 Metetra
.665 .751 .914 r
.13591 .36618 .14651 .37608 .16095 .37023 .15037 .36053 Metetra
.665 .756 .918 r
.15037 .36053 .16095 .37023 .17553 .36383 .1649 .35484 Metetra
.665 .774 .93 r
.1649 .35484 .17553 .36383 .19033 .35616 .17953 .34912 Metetra
.662 .821 .96 r
.17953 .34912 .19033 .35616 .20548 .34596 .19424 .34336 Metetra
.606 .908 .983 r
.19424 .34336 .20548 .34596 .22113 .33147 .20905 .33757 Metetra
.20905 .33757 .22113 .33147 .23741 .31084 .22394 .33174 Metetra
.258 0 .065 r
.22394 .33174 .23741 .31084 .25432 .28304 .23893 .32588 Metetra
.389 .061 .283 r
.23893 .32588 .25432 .28304 .27169 .24882 .25401 .31998 Metetra
.434 .141 .366 r
.25401 .31998 .27169 .24882 .28915 .21107 .26918 .31394 Metetra
.422 .15 .393 r
.26918 .31394 .28915 .21107 .30438 .19766 .28498 .30176 Metetra
.486 .206 .414 r
.28498 .30176 .30438 .19766 .31657 .22414 .2998 .30206 Metetra
.441 .152 .377 r
.2998 .30206 .31657 .22414 .32944 .24705 .31525 .29602 Metetra
.404 .086 .31 r
.31525 .29602 .32944 .24705 .34311 .26376 .3308 .28993 Metetra
.312 0 .152 r
.3308 .28993 .34311 .26376 .35754 .27365 .34645 .28381 Metetra
.032 0 0 r
.34645 .28381 .35754 .27365 .37255 .2776 .3622 .27765 Metetra
.486 .899 .902 r
.3622 .27765 .37255 .2776 .38799 .27719 .37804 .27145 Metetra
.647 .868 .982 r
.37804 .27145 .38799 .27719 .40371 .27398 .39399 .26521 Metetra
.665 .802 .948 r
.39399 .26521 .40371 .27398 .41963 .26917 .41003 .25894 Metetra
.665 .77 .928 r
.41003 .25894 .41963 .26917 .43568 .26353 .42618 .25262 Metetra
.665 .757 .919 r
.42618 .25262 .43568 .26353 .45186 .25748 .44243 .24626 Metetra
.665 .751 .915 r
.44243 .24626 .45186 .25748 .46814 .25124 .45879 .23986 Metetra
.664 .75 .914 r
.45879 .23986 .46814 .25124 .48453 .24491 .47524 .23342 Metetra
.664 .749 .913 r
.47524 .23342 .48453 .24491 .50103 .23851 .49181 .22694 Metetra
.664 .749 .913 r
.49181 .22694 .50103 .23851 .51763 .23207 .50848 .22042 Metetra
.664 .749 .913 r
.50848 .22042 .51763 .23207 .53434 .22558 .52526 .21385 Metetra
.664 .749 .913 r
.52526 .21385 .53434 .22558 .55115 .21905 .54215 .20725 Metetra
.664 .749 .913 r
.54215 .20725 .55115 .21905 .56808 .21248 .55915 .2006 Metetra
.664 .749 .913 r
.55915 .2006 .56808 .21248 .58511 .20586 .57626 .1939 Metetra
.664 .749 .913 r
.57626 .1939 .58511 .20586 .60226 .19921 .59349 .18716 Metetra
.664 .749 .913 r
.59349 .18716 .60226 .19921 .61951 .1925 .61082 .18038 Metetra
.664 .749 .913 r
.61082 .18038 .61951 .1925 .63689 .18576 .62827 .17355 Metetra
.664 .749 .913 r
.62827 .17355 .63689 .18576 .65437 .17897 .64584 .16668 Metetra
.664 .749 .913 r
.64584 .16668 .65437 .17897 .67197 .17213 .66352 .15976 Metetra
.664 .749 .913 r
.66352 .15976 .67197 .17213 .68969 .16525 .68132 .1528 Metetra
0 g
.25 Mabswid
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
[(80)] .07825 .23514 .98066 1 Mshowa
.22216 .18933 m
.227 .19478 L
s
[(90)] .21247 .17844 .88892 1 Mshowa
.36358 .12987 m
.36813 .13557 L
s
[(100)] .3545 .11847 .79718 1 Mshowa
.51343 .06687 m
.51763 .07282 L
s
[(110)] .50502 .05496 .70545 1 Mshowa
.67245 0 m
.67627 .00621 L
s
[(120)] .66483 -0.01242 .61371 1 Mshowa
.125 Mabswid
.11461 .23456 m
.11764 .23771 L
s
.14105 .22344 m
.14405 .22662 L
s
.16779 .2122 m
.17076 .21541 L
s
.19482 .20083 m
.19776 .20407 L
s
.2498 .17771 m
.25268 .18101 L
s
.27776 .16595 m
.2806 .16928 L
s
.30604 .15406 m
.30885 .15742 L
s
.33465 .14204 m
.33741 .14542 L
s
.39286 .11756 m
.39554 .12101 L
s
.42247 .10511 m
.42512 .10859 L
s
.45243 .09251 m
.45504 .09602 L
s
.48275 .07977 m
.48531 .08331 L
s
.54447 .05381 m
.54695 .05742 L
s
.57589 .0406 m
.57832 .04424 L
s
.60769 .02723 m
.61007 .0309 L
s
.63987 .0137 m
.64221 .0174 L
s
.25 Mabswid
0 .80374 m
.40037 1.0855 L
s
.09485 .87049 m
.09577 .86326 L
s
[(0.2)] .09301 .88495 .12735 -1 Mshowa
.18136 .93137 m
.18209 .92412 L
s
[(0.4)] .17991 .94587 .10002 -1 Mshowa
.26055 .98709 m
.2611 .97983 L
s
[(0.6)] .25944 1.00163 .07584 -1 Mshowa
.3333 1.03829 m
.33369 1.03102 L
s
[(0.8)] .33251 1.05285 .05428 -1 Mshowa
.40037 1.0855 m
.40063 1.07821 L
s
[(1)] .39987 1.10006 .03495 -1 Mshowa
.125 Mabswid
.02453 .821 m
.02518 .81668 L
s
.04852 .83789 m
.04914 .83356 L
s
.07196 .85438 m
.07254 .85004 L
s
.11722 .88623 m
.11774 .88189 L
s
.13908 .90162 m
.13958 .89727 L
s
.16046 .91666 m
.16092 .91231 L
s
.20181 .94576 m
.20221 .9414 L
s
.22181 .95983 m
.22219 .95548 L
s
.24138 .97361 m
.24174 .96925 L
s
.27931 1.0003 m
.27961 .99593 L
s
.29768 1.01322 m
.29796 1.00886 L
s
.31567 1.02589 m
.31593 1.02152 L
s
.35057 1.05045 m
.35079 1.04608 L
s
.3675 1.06236 m
.3677 1.058 L
s
.3841 1.07404 m
.38427 1.06967 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{185.688, 201.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002i0000bA000`40O003h00O^Goo002iOol006ao
o`<000=oo`@000=oo`8003ioo`00KGoo00<007ooOol00goo00<007ooOol00Woo00@007ooOol003eo
o`00KGoo00<007ooOol017oo00D007ooOomoo`00009oo`03001oogoo03]oo`00KGoo00<007ooOol0
1Goo00@007ooOol0009oo`03001oogoo03]oo`00KGoo00<007ooOol00Woo00@007ooOol0009oo`04
001oogoo000mOol006aoo`8000Eoo`8000Aoo`8003ioo`00^Goo002iOol00;Uoo`00NWoo0P00?Goo
001gOol300000goo0000000lOol0055oo`<000=oo`<000Aoo`8001Eoo`8000Aoo`8003aoo`00DWoo
00<007ooOol00goo00<007ooOol00Woo00@007ooOol0019oo`800003Ool007oo00=oo`03001oo`00
03]oo`00DWoo00<007ooOol00goo00<007ooOol00Woo00@007ooOol000moo`<000Qoo`03001oo`00
03]oo`00DWoo00<007ooOol00goo00<007ooOol00Woo00@007ooOol000eoo`@000Uoo`04001oogoo
000jOol0059oo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000:Ool3000=Ool01000Oomo
o`00>Woo001AOol20004Ool20005Ool20009Ool4000>Ool01@00Oomoogoo0000>Goo001WOol2000B
Ool00`00Oomoo`02Ool00`00Oomoo`0fOol006Aoo`@001Aoo`05001oogooOol0000hOol0069oo`80
01Qoo`03001oogoo009oo`03001oogoo03Eoo`00Ggoo0`006Woo00<007ooOol00Woo00<007ooOol0
=Goo000hOol30004Ool20004Ool2000FOol20003Ool00`00Oomoo`0GOol00`00Oomoo`03Ool00`00
Oomoo`0dOol003Uoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`004goo0P007goo00<0
07ooOol017oo00<007ooOol0<goo000iOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol0
011oo`<00003Ool007oo01ioo`03001oogoo00Aoo`03001oogoo03=oo`00>Goo00<007ooOol00Woo
00@007ooOol0009oo`04001oogoo000>Ool2000TOol00`00Oomoo`05Ool00`00Oomoo`0bOol003Uo
o`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`002goo0`0000=oo`00Ool08goo00<007oo
Ool01Woo00<007ooOol0<Goo000hOol20005Ool20004Ool2000:Ool2000YOol00`00Oomoo`06Ool0
0`00Oomoo`0aOol004moo`80009oo`03001oogoo02Ioo`03001oogoo00Moo`03001oogoo031oo`00
C7oo0`00;Goo00<007ooOol01goo00<007ooOol0<7oo001:Ool20002Ool00`00Oomoo`0[Ool00`00
Oomoo`08Ool00`00Oomoo`0_Ool004Moo`<0039oo`03001oogoo00Uoo`03001oogoo02ioo`009Goo
0`0017oo0P005goo0P000Woo00<007ooOol0<7oo00<007ooOol02Goo00<007ooOol0;Woo000XOol0
1000Oomoo`000Woo00<007ooOol04Woo0P00=goo00<007ooOol02Woo00<007ooOol0;Goo000VOol3
0002Ool01000Oomoo`004Goo0`0000=oo`00Ool0=Woo00<007ooOol02goo00<007ooOol0;7oo000U
Ool01000Oomoo`000Woo00@007ooOol000moo`8003aoo`8000aoo`03001oogoo02aoo`009Goo00@0
07ooOol0009oo`04001oogoo000<Ool300000goo001oo`0hOol300000eKm001oo`0<Ool00`00Oomo
o`0[Ool002Ioo`8000Aoo`8000]oo`8003]oo`<00004E_d005KmE_d2000<Ool00`00Oomoo`0[Ool0
03Moo`80009oo`03001oogoo03Moo`03001Fo@00009Fo@D000eoo`03001oogoo02Yoo`00=7oo0`00
?7oo0P000UKm0`0000=Fo@00E_d00`003Goo00<007ooOol0:Goo000bOol20002Ool00`00Oomoo`0g
Ool3000015Km001Fo@000UKm00<005Km00000P000UKm00<007ooOol02Woo00<007ooOol0:Goo000_
Ool3000kOol3000015Km001FoEKm0`000UKm0P000UKm00<005KmE_d00P0037oo00<007ooOol0:7oo
000@Ool20004Ool2000EOol20003Ool00`00Oomoo`0:Ool00`00Oomoo`0XOol30002E_d00`00E_d0
00020002E_d3000015Km001FoEKm0`0000=Fo@00Ool02goo00<007ooOol0:7oo000?Ool01000Oomo
o`000Woo00@007ooOol0019oo`<0011oo`8002Qoo`80009Fo@03001FoEKm0080009Fo@05001Fo@00
E_d00002E_d50002E_d2000<Ool00`00Oomoo`0WOol000moo`04001oogoo0002Ool01000Oomoo`00
3goo0`000Woo00<007ooOol03Goo00@003B<000002Aoo`<00004E_d005KmE_d3000015Km001FoEKm
0P000UKm0`0000=G7@00E_d0100000=G7@00Ool037oo00<007ooOol09Woo000@Ool20003Ool01000
Oomoo`003Goo10004Woo00<003B<=8`00P008Goo0`0000AFo@00E_eFo@<00004E_d005KmE_d30000
1UKm001Fo@00Ead0009G7@<00004Ead005LMEad2000<Ool00`00Oomoo`0VOol000moo`04001oogoo
0002Ool01000Oomoo`002Woo0`005Woo00D003B<000lc@0001ioo`<00004E_d005KmE_d3000015Km
001FoEKm0`0000AG7@00E_eFo@<00004Ead000000002Ead00`00Ead000020002Ead00`00Oomoo`0:
Ool00`00Oomoo`0UOol0011oo`8000Aoo`8000Uoo`800003Ool007oo01Eoo`800005=8`003c=?<d0
000JOol30002E_d00`00E_d000020002Ead00`00E_d000020002Ead00`00Ead000020002Ead20002
Ead00`00EaeG7@020002Ead00`00EaeG7@02000<Ool00`00Oomoo`0TOol001moo`8001Yoo`800004
=8`003c=?<d3000FOol20002E_d00`00E_eFo@020002Ead00`00EaeG7@020002Ead00`00EaeG7@02
0002Ead3000015LM001G7ELM1@000ULM0`0000=G7@00Ool02goo00<007ooOol097oo000LOol30000
0goo001oo`0HOol20002=8`00`00?<dlc@0200000fU0001oo`0AOol3000015Km001FoEKm0`0000AG
7@00EaeG7@<00004Ead005LMEad300001ULM00000000Ead0009G7@<00003Ead005LM00@00004Ead0
05LMEad2000<Ool00`00Oomoo`0SOol001]oo`03001oogoo01aoo`80008dS006000lc@00>8`006U0
0P003Woo0`000ULM00<005Km00000P0000AG7@00EaeG7@<00004Ead005LMEad300001ULM00000000
Dad0009G7@<00004Ead005LMEad3000015LM001G7ELM0`0000=G7@00Ool02goo00<007ooOol08goo
000KOol2000LOol00`00=8/00002=8`00`00?<d00002>8`01000G@9M0P002goo0P000ULM00<005LM
Ead00P000ULM00<005LM00000P000ULn00<005LM00000P000ULM0P000U<N00<005<M00000P000U<M
00<005LM00000P0000=C7@00Ead0100000AG7@00EaeG7@8000aoo`03001oogoo029oo`006goo0P00
77oo00D003B;000dS000008lc@03000hS3R<00800004G@8000000008Ool00`00Ead00002Ead30000
15Ln001G7ELM0P000U<n00<005LnEch00P000U<n0`0000AC?P00DaiC7P80009C7P03001C7E<M0080
009C7@03001C7@0000800004Dad005LMEad300000e<M0000000<Ool00`00Oomoo`0QOol001Eoo`80
00Aoo`03001oo`0001Yoo`800004=8/003B<0002?<d01P00>8`hS000<4T000=<PP<000=oo`<00006
Ead00000001CG`000ULn0`0000=CG`000000100000IC?P00000005<n0002Dch3000015<N001C7U<N
0`0000AC7@00DaeC7@80009C7@D00004Dad000000002Dad00`00Oomoo`09Ool00`00Oomoo`0QOol0
01Aoo`04001oogoo0002Ool01@00Oomoogoo00006Goo00<003B;00000SB<00<003c=00000SR<00<0
0319<4T00P0000=<PP00>0000SP00`000ULn0P000TnO1@000TmO0P000TmO00<004lo00000P000Tln
00<005<n00000P0000AC7P00DaiC7P<00004Dah005<MDad300000e<M000000030002Dad20002Dad0
0`00DaeC7@02000;Ool00`00Oomoo`0POol001Aoo`04001oogoo0002Ool00`00Oomoo`02Ool00`00
Oomoo`0FOol00`00=8/00002=8`00`00?<d00002>8`00`00<4T0000290@50002Dgl>00000dMO0000
00030002Ccl50002Cch500000e<N0000000400001e<N00000000Dad005<M00@00004Dad005<MDad3
00000e<M001oo`0:Ool00`00Oomoo`0POol001Aoo`04001oogoo0002Ool00`00Oomoo`03Ool00`00
Oomoo`0DOol200001cB;000dS3B<000lc@00008hS003000`B@00008T10L00003;Sd0000000`00093
?`03001;?`0000800004Bal004lnCch3000014lN001?7TlN0P000U<N00<005<N00000P0000AC7@00
DaeC7@<00004Dad005<MDad3000015<M001C7E<M0P002goo00<007ooOol07goo000DOol01000Oomo
o`000Woo0P001Goo00<007ooOol04goo0P0000LdR`00=8`dS000?<d00002>8`00`00<4T000020000
0cgh0000000G0002Aal5000014lN001?7TlN0`0000A?7P00DaiC7P80009C7P03001C7@0000800004
Dad00000Dad3000015<M001C7E<M0`0000=C7@00Ool02goo00<007ooOol07Woo000EOol20003Ool0
0`00Oomoo`05Ool00`00Oomoo`0AOol20002=8/00`00=8`00002?<d00`00>8`0000300001dGg0000
04XH0016>@000092>P06000nG3iL000jO@000SZN00<003Zn00000SkO00<004;o00001@0000A;7P00
Bai;7PD00003Cah0000000@00004Cah005<NDah20002Dad00`00Dad00002000015<M001C7E<M0`00
00=C7@0000000`002goo00<007ooOol07Woo000JOol00`00Oomoo`06Ool00`00Oomoo`0?Ool30002
=8/01P00=8`003c=000hS0800008CQL00000001>5`00BQP00096>@050012>T8j000nG00<0002A]h0
0`00A_h00005000014[n001?7TlN0P000Tkn00<004lN00001@0000=C7@0000000`0000=C7@000000
100000IC7@00000005<M0002Dad2000;Ool00`00Oomoo`0MOol001Yoo`03001oogoo00Moo`03001o
ogoo00ioo`06000`:@00=8/003B<1@0000EJM`00001>5`00009>5`05001:64XH0016>@0500001dIK
0016O4Il0012W@000092_@L0009:oP03001:oP0000D00004C_h004knC_h20002C_d00`00Cad00002
000015;m001C7E<M0`0000ABo@00DaeC7@80009Bo@03001C7@0000800003D_d007oo00]oo`03001o
ogoo01aoo`006Goo00<007ooOol02Goo00<007ooOol037oo0P0000<`:@00=8/00P000V;I100000EB
>000CQM>5`00009>5`@00005CSP004YIBUT00002AU/00`00AW`0000400001TZl0016_DJm0016g@D0
009:o@03001>oP0000D00004C_d004kmC_d20002D_d00`00D_d00002000015;m000005;m0`0000AB
o@00D_eBo@80009Bo@04001Bo@00000;Ool00`00Oomoo`0LOol001Uoo`03001oogoo00Yoo`03001o
ogoo00Yoo`@00003=8/0000000=Rf@03001FN000009B>005001>5dhG001>5`0200000e8g001>>002
CSP00`00BUT0000400001TYk001:W4ZL001:_0D0009:g@03001:g@00009:o@80009>o@03001>o@00
00D00004C_d005;mD_d20002D_d00`00D_d00002000015;m001BoE;m0`0000ABo@00D_eBo@800003
D_d007oo00]oo`03001oogoo01]oo`006Goo0P0037oo00<007ooOol027oo1`0000=W6@00H]T00P00
0UIh00D0058hDSP004hG00800006ESH0058gDSL004hh0`0000I>F@00CWY>NP00BW/50002C[`00`00
B[`00002B]d5000014kM001>oDkm0P000Tkm00<004km00001@0000=Bo@0000000`0000ABo@00D_eB
o@<00004D_d005;mD_d300000e;m001Bo@03000:Ool00`00Oomoo`0KOol001Uoo`03001oogoo00ao
o`03001oogoo00Eoo`<00003LmX006mj0080009W6@80009N^@03001FN00000@00004CQL005Hf0002
DSL200000e9H001>F@02CUT01P00CWX00000CWX0009>V`03001>_4jl00D00004C[`004kMC]d50000
14kM001>oDkm0P000U;m00<005;m00000P0000ABo@00D_eBo@D00003D_d0000000@00003D_d00000
00<0009Bo@8000Yoo`03001oogoo01Yoo`006Goo00<007ooOol03Goo00<007ooOol00Woo100000]c
fP00KgX006/j001W6@00H]T005ji00<00004DUP004XH0002CQL00`00ESIF=P0200001UIG001BF59H
001>F@<00006DWT004ijCWX004jK0`0000A>V`00C[`0009>_0<00004C]`004kLC]`5000015;M001B
oE;m0P000U;m00<005;m00000P0000ABo@00001Bo@80009Bo@03001Bo@0000800004D_d005;mD_d3
00000e;m001oo`0:Ool00`00Oomoo`0IOol001Uoo`03001oogoo00ioo`05001oo`00001?_`060003
JcX01`00H]URfF;I001FV@00009BF004001:64XH0002CQL00`00ESH00002EUL00`00DUQBF0020000
0e9h001BN@02DWT01P00CWX00000DYX0009>V`D00004D[`004kLC]`300000e;L001Bg006000015;m
001BoE;m0P0000ABo@00D_eBo@<00004D_d005;mD_d2000015;m001BoE;m0`0000ABo@00D_eBo@80
00Yoo`03001oogoo01Uoo`0067oo00<007ooOol03Woo0`0000IGGP00000006oK0002Ki/200001F/j
001VnP00H]T00P000UJI00H0059HDUP004XH0002CQL30003EUL200001UIH001BN59h001BN@<00003
DYX005:J00<00006DY/005:kD[/005:l0`0000AB_000D]aBg080009Bg003001Bg000009Bo@<00003
D_`005;m00<00004D_d00000D_d300000e;m001Bo@03000015;m001BoE;m0`0000=Bo@00Ool02Woo
00<007ooOol067oo000HOol00`00Oomoo`0<Ool20003Ead600000fnK001[FP020002I_X20002F[T0
0`00EYT000080002FUH01000EUL000000eIH00H0059h000005Ii0002DYX200000eJJ001BV`02DY/0
1000D[/000000U:k00<005:l00000P0000ABg000D]aBg0D00004D]`005;lD_`20002D_`00`00D_d0
0002000015;m001BoE;m0P000U;m00<005;m00000P0000ABo@00D_eBo@8000]oo`03001oogoo01Mo
o`0067oo0P002Woo0`000ULM00H005LM000006_M0002Ji`2000016]J001W>VLj0P0000=NfP00F[T0
0`0000ABN@00BSP00091n003001:5`0000=JEP03001JE`0000=FF0800004EWP005IiEWT200001UJI
001FVUJJ001BV`<00004D[/005:kD[/20002D[/00`00D]`00002000015;L001Bg5;L1@0000=Bg000
00000`0000=Bo0000000100000=Bo00000000`0000=Bo@000000100000=Fo@0000002Woo00<007oo
Ool05goo000HOol00`00Oomoo`07Ool20002Ead00`00EaeG7@06000016^L001[Nf]k0P0000EW>P00
H_X005kJ00800005EYT0059iDWT00002BSP01@00@OP004XG00000UYF0P000UYG00@005IH000000=F
N0<000=FV@04001FVP000002EYX00`00D[/00002000015Jk001B^e:k0P0000AFf`00D]aBg0<00003
E]`005;L00<00004E]`005;lD_`300000eKl001Bo003000015Kl001Bo5;l0P000UKl00<005Km0000
0P0000=Fo000Ool02Woo00<007ooOol05Woo000HOol00`00Oomoo`05Ool20002E_d00`00Ead00002
000015Mn001S_@000VNL0P0000=[N`00Ic/00P000V;j0P0000EJ^P00EYUFV@00009BN@03001:>000
0091n006001:5`00001JEP000UYG0P000UYg00@005Ih0000009FN003001FV@0000800004EYT005JJ
EYX20002E[X00`00E[/0000200000eJk001Ff`03000015KK001Fg5KL0P000UKL00<005KL00000P00
00AFg000001Fo080009Fo003001Fo00000800004E_`005KlE_`2000015Kl001Fo5Kl0P002Woo00<0
07ooOol05Woo000HOol00`00Oomoo`02Ool3000015Km001FoEKm0`0000=G?P0000000`0000=WW000
Jg`00P0000QW>`00Ha/006;j001NfP000UZj00<005JI00002P000eYF0P0000=JM`00FWL00UYg00<0
05Yh00000UIh0P000UZI00<005JI00000P0000AFVP00E[YF^P800004E[X005JkE[/20002E]/00`00
E]/00002000015KK001Fg5KL0P000UKL00<005KL00000P0000AFo000E_aFo0800004E_`005KlE_`3
00000eKl001Fo00300000eKl001oo`0:Ool00`00Oomoo`0EOol001Qoo`03001oogoo00<00003E_d0
05Km00<0009G7@800004Ggh006>MHid200001F]l001WFfMK00000f<K00H005kJG]YNfP00F[X20000
159j001:F@000T8I00D003WH001:5`0000=JEP04001JMeYg0002FWL20002FWP2000015ZH001JVEZI
0P0000=JV@00EYX00UJJ0P0000=F^P00E[X00`0000AF^`00E]]Ff`80009Ff`03001Ff`0000800004
E]/005KLE]`2000015KL001Fo5Kl0`0000=Fo000E_`00`0000AFo000E_aFo0800004E_`005KlE_`2
000;Ool00`00Oomoo`0DOol001Moo`D00003E_d0000000<00005Ead00000001KGP0300000f>M001W
O00300002FMK001S>`00Ha/005kk001NfP0300001eJJ001BNU9j001:F@0000926@80009:5`04001J
EUYF0003FWL200001EYg001JN5Yh00000eZH0P0000IJV@00FYUJV@00EYX20002E[X20002E[X00`00
E[/00002000015Jk001FfeKK0P0000AFf`00E]]Ff`<00003E]`005KL00<00004E]`005KlE_`20000
15Kl001Fo5Kl0`0000=Fo000E_`00`0000=Fo00000002Woo00<007ooOol057oo000GOol20002E_d2
0002E_d00`00Ead0000200001ELm00000000Ggd01@0000EWG000000006<k00<00003G_/00000009J
^P03001FVP0000X00006BQL005YF001NMP000UYg00@005YgFWMJM`<000=JV006001JVEZIFYT005ZI
0P000UZi0P0000AJ^P00E[YF^P80009F^P03001F^`0000800003E]/005KK00<00004E]/005KLE]`3
00000eKL001Fg00300000eKl001Fo003000015Kl001Fo5Kl0P000UKl0P0000=Fo000Ool02Woo00<0
07ooOol04goo000@Ool40003Ool01P00Ool00000E_d0009Fo@80009G7@80009KG@800004Hgd006=L
He`20002Hc/20002Ha/2000015[K001J^UZj0P0000ABNP00BUX00096>@04000mn@00=KP40002GWH0
1000FWL000000UYg0P000UYh00@005ZH000000=JV@800003FYT005Zi008000=J^P800004F[X005Jj
E[X2000015Zj001FfeKK0P000UKK00<005KK00000P0000AFf`00001Fg0800004E]`005KlE_`20002
E_`500001eKl00000000E_`005Kl008000Yoo`03001oogoo01=oo`004Goo00<007ooOol00goo00D0
07ooOol005Km00<00004Ead000000002Fcd200004EmM00000000He`006=L001S>`00Ha/006<K001N
n`00F]/00`0000MF^P00DWYBNP00BUX00002AST200001CFh001:5dXG00000eif00<005igGWL00P00
00ANM`00FWQJN080009JV004001JV@000002FYT20002F[T01000F[X000000eZj0P0000=J^P00F[X0
0`0000AJf`00E]]Ff`800004E]/005KKE]/300000eKK001Fg00300001eKL00000000E_`005Kl00<0
0004E_`005KlE_`200000eKl001oo`0:Ool00`00Oomoo`0BOol0019oo`03001oogoo009oo`04001o
ogooOol200000eLM00000002Ecd00`00Fcd00002Ged01000HeeSGF=M0P0000ESG000000006<K00<0
0003G_/00000009Ff`03001F^UJj00X00004=KP004XG0002GWH20002GWL01000GWMNMeig0P0000AN
U`00FYQJV080009JV0800004FYT005ZiF[T20002F[T01000F[X000000UZj0P000UZj00<005[K0000
0P0000=Jf`00E]/00`0000AFf`00E]]Ff`800006E]/00000001Fg0000UKl0P0000AFo000E_aFo0<0
0003E_`005Kl008000Uoo`<001=oo`004goo00D007ooOomoo`0000Eoo`05001G7ELM001G?@030000
1UmM001OG@00Hed0009SG003001S?6<l0080009O6`80009Jf`03001Ff`0000<00008BWX004HjASX0
03hI000ef@<0009:5`800005GWH005igGWL00003GWL01000GYMNUejG0P0000ENV000FYQJV00000=J
V@800004F[T005ZiF[T200001UZj001J^P00F[X0009J^P800004F]X005[KF]/20002F]/00`00E]/0
000200000eKK00000002E]/00`00E]`0000200000eKl001Fo003000015Kl001Fo5Kl0P0000=Fo000
Ool01Goo0`000Woo00<007ooOol04Goo000@Ool01000Oomoo`000goo00<007ooOol00goo00<005LM
Ead00P000U/m0`0000EOG@00000006=L00800008Hc`005lK001O6`00G_/0009Jf`800007E[/005:J
DYX004Yj00000THj0P0000@ef@00;IP]V0<000=NMP800003GWH005ig008000=NU`04001NV5jHGYP2
00000ejH001JV@020003F[T200000eZi001J^P020003F[X200000eZj001JfP03000015[K001Jfe[K
100000AFf`00F]/0009Ff`800004E]/005KlE_`20002E_`00`00E_`0000200000eKl000000020002
Ool30005Ool00`00Oomoo`0AOol0015oo`8000=oo`03001oogoo00Eoo`800004Fcd000000002Gcd2
0002Gc`200001F<l00000000Ga/00`0000INn`00000005KK0002E[/00`00DYX0000700001SGI000]
V000BQL000=NMP05001NMUifGWH00002GWL01000GYL000000ejH00<005jHGYP00P000UjH00@005Zi
0000009J^@80009J^@04001J^P000002F[X20002F]X00`00F]/0000200000e[K0000000300001e[K
00000000F]/005KK00<00004E]/005KlE_`200001eKl00000000E_`005Kl00<000Qoo`03001oogoo
015oo`005Woo00<007ooOol01goo0P0000EK?@00000005lm00<00005Gc`00000001O?00300000elK
00000002F]/00`00E]/0000400001dYj0016FP00?QX003Wi00<00004;IP004XG0002GWH20002GWH2
0003GWL01@00GYMNU`00GYP00P0000ANV000GYP0009NV080009N^@800004G[T005ZiF[T200001UZj
001J^P00F[X0009JfP800004F]X005[KF]/20002F]/2000015[K001Jfe[K0P0000AJf`00E]]Ff`<0
0005E_`00000E_`00002E_`200000eKl001oo`08Ool00`00Oomoo`0AOol001Ioo`03001oogoo00Uo
o`05001K?E/mFcd00003Gc`00`00GcaO?0020002Ga`2000015[k001Jfe[K0P0000EB^`00DY]BV`00
009:NP040016FP00?QX20002<KP300000dXG00000002HWH20000169f001NMeig0P000UjG0P000UjH
00@005jHGYQNV0800005G[P005jiG[T00003G[T200000eji001J^P020003F[X200000e[J001JfP03
00001e[K00000000F]/005[K00<00003F]/005[K00<00007F]/005Kl001Jn`00E_`00`0000=Fo000
E_`00P002Goo00<007ooOol047oo000FOol00`00Oomoo`0:Ool01P00Fcd005lm001O?0800009Gc`0
05lL001O7000Ga`005[k00<00006E]/005:kD[/005:K2P0000<a^0009GP00`000f9f00@0069fHWIR
MP80009RU`800005HYL005jHGYP00003GYP01000G[QN^5jh0P0000=N^000G[T00P000Uji0P000Uji
00@005Zj0000009JfP80009JfP04001Jf`000002F]/2000015[K001Jfe[K0P000U[K00@005[K0000
009Jn`800004F_/005KlE_`200000eKl001oo`08Ool00`00Oomoo`0@Ool001Ioo`03001oogoo00]o
o`03001O?Elm00800005Gc`00000001O700300000elL00000002F_/00`00E]/0000400001Tik0016
FP00@SX0008j6P03000af@0000<00003BQL0069f009RMP05001RMV9fHWH00003HYL01000HYMRUf:G
0P0000=NU`00GYP00P000Ujh0P000Ujh0P0000IN^@00G[T005ji0002G[T200001Ujj001JfP00F]X0
009JfP800007F]X005[K001Jf`00F]/00`0000AJf`00F]]Jf`800007F]/00000001Jf`00F_/00`00
00=Jo000E_`00`001goo00<007ooOol047oo000FOol00`00Oomoo`0<Ool200001Ell00000000Ga`0
0`0000=O700000000U[l00@005[k0000009B^`03001BV`00009>N`0:0016FP00@SX003XJ000af@00
:IP20002BQL01@00HWH0069f00000V9f00D006:GHYMRU`0000=RU`05001NUejGGYL00002GYP20000
1Ujh001N^000G[P0009N^@04001N^EjiG[T200000eji001N^P020003F]X200000e[J001JfP020002
F]/20002F]/00`00F]/0000200001E[K000005[K00000U[K0P0000AJn`00F_aJo080009Fo08000Io
o`03001oogoo011oo`005Goo0P0047oo0P0000=O?00000000UlL0P0000AJo000F_aJo0800004E]/0
05:kD[/>00003bVH00000000BQL0069f001RMP00HWH006:F001RU`020002HYL20002GYL20002GYP0
1@00G[QN^5jh00000ejh0P0000=N^@00G[T00P000Uji0P000Uji00@005[J0000009JfP@00004F]/0
05[J0002F]/200001e[K001Jf`00F]/005[K00<00003F]/005[k00<00006F_/00000001Jo0001Woo
00<007ooOol047oo000EOol00`00Oomoo`0AOol00`00GaaO700200000e/L001Jo00300000eKl001F
f`03000015:k001>V`000TYk00H0049J000n6P00=OX400000b5G000000020003HWH01@00HYIRUV:F
00000V:F0P000V:G0P000V:G0`000ejh00D005jhG[QN^000009N^@800006G[T005ji001N^@000Uji
0P000UkJ0P0000=NfP0000000P000e[J0P0000=Jf`00F]/00P000U[K0P0000AJf`00F]]Jf`800006
F_/00000001Jn`000U[l0P001Goo00<007ooOol047oo000EOol00`00Oomoo`0AOol01P00Ga`005/L
001K70<00003F_`0000000<00003D]/005:k00`00003;KT00000008000966005001RMV9fHWH00003
HYH01000HYIRUP000f:G00D006:GHYMRU`00009RV006001N^000H[P005jh0P0000AN^@00G[P0009N
^@04001N^EjiG[T200001eji001NfP00G]X005kJ0080009NfP04001JfP000002F]/20002F]/20000
0e[K001Jf`0300001E[K000005[k00000U[k0`0000=Jn`00Ool017oo00<007ooOol047oo000EOol0
0`00Oomoo`0BOol200000e/L00000002F_`2000015Kl001Fg5KL100000Y>V`00BW]:N`00AU/003hj
000enP@000068GP00000001660000V9f0P000V:F0P000V:F00D006:GHYMRU`0000=RU`03001RV6:H
0080009R^080009N^004001N^5jhG[P200000eji001N^@020002G[T20002G]X2000015kJ001NfUkJ
0P000U[J0P0000=JfP00F]/00P000U[K0P0000MJf`00F]/005[K001Jn`03000015[k001Jne[k0P00
1Goo00<007ooOol03goo000EOol00`00Oomoo`0DOol00`00Fa`0000200000e[l001Fo003000015;L
001B^`000TjK00<004Yk00000`0000@n>P00000]f@@000046CL004HHAQP20002HYH20002HYH20000
2F:F001RU`00HYL006:G001RU`0200001V:g001R^000H[P0009N^005001N^5jhG[P00002G[T20002
G[T200001ekI001NfP00G]T005kJ00<00003G]X005[J0080009JfP80009Jf`800003F]/005[K0080
009Jf`80009Jn`03001Jn`0000800003F_/0000000Aoo`03001oogoo00moo`005Goo00<007ooOol0
5Goo00D005[lF_`005[l0080009Fg003001Bg00000H00003AW/0049K00H0008UV@8000046CL00000
0003HYH01@00HYIRUV:F00000f:F00D006:GHYMRU`00009RU`05001R]f:gH[L00003H[P200000f:h
001N^00200001Eji001N^@00G[T00P000ekI00@005kIG]UNf@<0009NfP80009NfP800003G]X005[K
0080009Jf`80009Jf`800004F]/005[kF_/20002F_/200000e[k001oo`03Ool00`00Oomoo`0?Ool0
01Eoo`03001oogoo01Ioo`03001Jo00000800003E_`005KL0080009B_005001>^`00BY/00002AW/2
00000cXJ000anP0300001bFI000001Tg0016600000=RUP03001RUV:F0080009RUP80009RU`80009R
U`05001R]f:gH[L00002H[P20002H[P20002H[P01@00G[UN^Eji00000Uji00@005kI0000009Nf@<0
0006G]X00000G]X005kJ0P000UkJ0P0000]JfP00000005[K001Jf`00F]/005[K00<00003F_/005[k
0080009Jn`8000=oo`03001oogoo00moo`0057oo00<007ooOol067oo00<005KlE_`0100000ABg000
D[aB_0d0008Y^@D00006AQP006:FHYH006JF0P000VJF0P000V:F0P000V:G0P0000ERU`00H[L006:g
00800007H[P006:h001R^000H[P00P000eji00<005jiG[T00P000UkI0P0000=Nf@0000000P000UkJ
0P0000ANfP000000009NfP05001NfUkJ001Jf`020002F]/2000015[K001Jne[k0P0000IJn`000000
05[k0003Ool00`00Oomoo`0?Ool001Aoo`03001oogoo01Uoo`800003E_`005KL00D00006C[`004ZK
0016N`000T9K0P000SGj0P0000<Y^@008GP0100000=6600000000VJF00D006JFIYIVUP0000=RUP05
001RUf:GHYL00003HYL01000H[MR]`000f:h00D006:hH[QR^000009R^004001N^@00H[P200000f:i
001Nf@020002G]T20002G]T20002G]X01@00G]YNfP00G]X00P000UkJ0P000UkJ0P0000=Jf`00F]/0
0`0000IJn`00001Jn`00F_/20003Ool00`00Oomoo`0?Ool001Aoo`03001oogoo01Yoo`04001Fo000
0002D]`01000D[`004jl2P0000<enP00;MX02@000fJF00D006JFIYIVUP0000=RUP03001RUf:G0080
009RU`80009R]`05001R^6:hH[P00002H[P20002H[P20002H[P00`00H[UR^@020002G]T200001ekI
001Nf@00G]T005kJ0080009NfP80009NfP800003G]X005kJ0080009Jf`80009Jf`800003F_/005[k
00800003F_/00000009oo`03001oogoo00moo`0057oo00<007ooOol06goo00<005KL00000U;L1000
00Y:W000AY]6V`00@W/003hk000f6`@000049IT000005CP200003THH001VUVJF001VU@00IYH006JF
001RUP00IYH20002IYL20002H[L200001F:g001R^000H[L00P000V:h0P0000=R^000H[P00P0000ER
^@00H[T005kI0080009Nf@80009Nf@80009NfP800003G]X005kJ0080009NfP80009NfP800003G]X0
05[K0080009Jn`80009Jn`03001Jn`0000=oo`03001oogoo00ioo`0057oo00<007ooOol077oo1000
00=Bg000C[`0300000<anP0000001P000THH00L006JF001VU@00IYH00002IYH01000IYIVUP000fJG
00D006:gH[MR]`0000=R]`04001R]f:g0003H[P00`00H[QR^0020002H[P01@00H[UR^F:i00000V;I
00T005kI000005kI001Nf@00G]X00P000UkJ0P000UkJ0P0000=NfP00G]X00P000UkJ0P000UkK0P00
00=Jn`00F_/00P0000=Jn`0000000Woo00<007ooOol03Woo000DOol2000NOol00`00D]`000030002
C[`02000BY`004Ik000nF`00>S/400000bVj00000005000014HH001VUP000VJE00D006JFIYIVUP00
009VUP05001VUfJGIYL00003H[L00`00H[MR]`0200001F:g001R]`00H[P00P0000UR^000H[P006:h
001R^000H[T00P000V;I0P0000=Nf@0000000UkI0P000UkI0P0000=NfP00G]X00P000UkJ0P0000MN
fP00000005kJ001Nf`020002F_/200001E[k000005[k00000Woo00<007ooOol03Woo000=Ool30003
Ool2000POol01@00D]`004jl00000Tjl2@0000<an`00;MX00P0000<MN@0000001P000fJE00<006JF
IYH00P0000]VUP00IYH006JG001VUP00H[L006Jg0080009R]`05001R]f:gH[L00002H[P01@00H[QR
^6:h00000f:h00<006:hH[P00P0000=Rf@00G]T00P0000ENf@00H]T005kI0080009NfP80009NfP80
0005G]X00000G]X00002G]X200001EkK00000000G_/00P0000=Jn`00F_/00P0000=oo`00Ool03goo
000>Ool00`00Oomoo`02Ool00`00Oomoo`0OOol00`00D]`0000400001dZL0016O000@W/003hk00T0
00045CP000000002AQP01@00IYEVU@00IYD00P000VJF0P000VJF00D006JFIYIVUP00009V]`05001R
]f:gH[L00002H[L20002H[P01@00H[QR^6:h00000V:h0P000V:h00<006;IH]T00`0000=Rf@00H]T0
0V;I00<005kIG]T00P0000=NfP00G]X00P0000ENfP00G]X005kJ00<0000:G]X00000G]X005kk001N
n`00F_/20002F_/2000@Ool000aoo`@000=oo`03001oogoo021oo`800003C]`004jl00T00003=Q/0
02gj00H000045CP000CF0002AQP20002IYD01@00IYIVUVJF00000fJF00<006JFIYH00P000VJg0P00
00=R]`00I[L00P000VJg0P0000ER^000H[P006:h0080009R^0800004H]P00000H]T300000f;I001R
f@0200001EkI001Rf@00G]X00P000UkJ00<005kJG]X00P0000MNfP00G]X005kJ001NfP020002G_/2
00000ekk001Jn`03000@Ool000aoo`03001oo`0000Aoo`03001oogoo029oo`<00004B[`004JL0002
@W`700029KX200001AUi000000001=H00`000fJE00D006JFIYIVUP00009VUP04001VUP000002I[H2
0002I[L01000I[MV]`000fJg00<006:hH[P00P000V:h00D006:hH[QR^000009Rf005001RfF;IH]T0
0002H]T20002H]T00`00H]URf@0200000ekI001NfP0200001EkJ001NfP00G]X00P000UkJ0P000UkJ
0P0000=NnP00G_/00P0000=Jn`00000047oo000<Ool00`00Ool00004Ool00`00Oomoo`0ROol00`00
C[`0000900000cXk000b6`0200000bFj000MV@06000014HH001VUFJE0P000VJF00@006JF0000009V
UP05001V]VJfI[H00003I[L01000I[MV]`000fJg00X006:h001V]`00H[P006:h001R^0800005H]P0
06;H001Rf@0200001F;I001Rf@00H]T00P0000ERf@00H]T005kI0080009NfP05001NfUkJG]X00002
G]X200000ekJ001NfP0200001Ekj00000000G_/00P0000=oo`00Ool03goo000=Ool20004Ool00`00
Oomoo`0SOol02`00B[a:_000BY`004Il0012O000>U/0100000<YfP0000001`000THH00@006JE001V
U@80009VUP05001VUVJFIYH00002I[H20002I[L02P00I[L00000I[L006Jg001V]`000fJg00D006:h
H[QR^000009R^003001Rf6;H00800003H]P00000009Rf@80009Rf@03001RfF;I00800007H]T005kJ
001NfP00G]X00P0000=NfP0000000UkJ0P000UkJ00<005kjG_X00P0017oo00<007ooOol03Goo000B
Ool2000VOol;00000cHK000]n`0200000b6j0000000600001THH001VU@00IYD000=VUP05001VUVJF
001VUP020002I[H20002I[L01@00I[MV]fJg00000VJg00D006JgI[MV]`00009R^0800006H[P00000
H]P006;H100000ERf@00H]T006;I00800005H]T006;I001Rf@020002G]X20002G]X00`00G]YNfP02
00000ekJ001NfP0200000ekj001oo`05Ool00`00Oomoo`0=Ool0019oo`03001oogoo02Ioo`0:001:
_000AY`0049l000nG000=S/4000012Fj000001Ui2@000VJE00X006JFIYH006JF001VUP00IYH000=V
]P05001V]fJgI[L00002I[L200001fJg001V]`00I[L006Jg0080009V^003001R^6:h00H00004H]P0
06;IH]T20002H]T00`00H]URf@020002H]T200000f;I001NfP0200000ekJ00000002G]X200000ekJ
00000007Ool00`00Oomoo`0=Ool0019oo`03001oogoo02Moo`X00003<Q/0000000X000966007001V
U@00IYD006JF00000VJF00@006JFIYH000=V]P05001V]fJg001V]P020002I[L01@00I[MV]fJg0000
0VJg00<006JhI[P01@0000ERf000H]P006;H00800005H]T006;I001Rf@0200001F;I001Rf@00H]T0
0P000V;I0P0000=NfP0000000UkJ0P0000=NfP0000002Goo00<007ooOol03Goo000BOol00`00Oomo
o`0WOol02000AY`0049l000nO000>U`400000bWK000000080002AQP00`00IYD00002IYD04@00IYIV
UVJF001VUP00I[H006Jf001V]P00I[L006Jf00000fJg00<006JgI[L00P0000=V]`0000000`0000EV
^000001Vf00000=Rf004001Rf0000003H]T01000H]URf@000f;I00<006;IH]T00P0000IRf@00001R
fP00G]X200000ekJ001oo`0:Ool00`00Oomoo`0=Ool0019oo`03001oogoo02Qoo`T00003=Q/00000
00/000966080009VU@05001VUVJF001VUP020002I[H01@00I[IV]VJf00000VJf00D006JgI[L006Jg
00800003I[L0000000<000=V^004001Vf6KH0003H]P00`00H]P00002000026;I000006;I001Rf@00
H]T2000026;I000006;I001Rf@00H]X200000ekJ0000000<Ool00`00Oomoo`0=Ool0019oo`03001o
ogoo02Qoo`090016W4JL0012O000?U`003Hl00@000069MX000006GT000TG0P0000<0eP0000000P00
0VJE0P0000EVUP00IYIVUP0000=V]P03001V]VJf00800007I[H006Jf001V]`00I[L00`0000AV]`00
I[MV]`80000;I[P00000I]P006KH001Rf000I]P00002H]P20002H]P00`00H]URf@020002H]T01000
H]URf@000V;I0P000V;I00<007ooOol037oo00<007ooOol03Goo000BOol00`00Oomoo`0YOol90000
0c8K000Yn`0200001AfJ000000002AL00`000THH00<006JE00000VZE0P000VJF00D006JfI[H006Jf
0080009V]P04001V]VJfI[H40002I[L01000I[L000000VJg00@006JgI[L0009Vf080009Vf003001R
f6;H00800008H]P00000H]T006;I001Rf@800009H]T00000H]T006;I001Rf@00011oo`03001oogoo
00aoo`004Goo0P00;Goo100000<jG000<S/0100000<Q^P0000001`000THH0P000VZE00<006JFIYH0
0P0000EV]P00I[IV]P0000=V]PD00003I[L006Jg0080009V]`03001V]fJg0080009V]`08001Vf000
I]P006KH001Vf0800003I]P00000009Rf003001RfF;I0080009Rf@04001RfF;I0002H]T2000AOol0
0`00Oomoo`0<Ool0015oo`03001oogoo02aoo`040012O000>U`400000bgk0000000=0003JYD01@00
IYIVUP00J[D00P000VJf00<006Jf000010000VJf00@006JgI[L000=V]`03001V]`00009V]`800003
I[L006KH009Vf004001Vf6KH0002I]P2000026;H000006;I001Rf@00H]T200001F;I000006;I0000
4goo00<007ooOol037oo000AOol00`00Oomoo`0]Ool400000cHl000b6`0200000bGK000000090000
34HH001ZUFZE001ZU@00IYH006Ze001V]PD0009V]P05001V]VJfI[H00002I[L2000016Jg001V]`00
0fJg00@006JgI[L000=Vf006001Vf000001Vf0000VKH00@006;HH]P0009Rf@80009Rf@04001RfF;I
000DOol00`00Oomoo`0<Ool0015oo`03001oogoo02eoo`03000nO3il00D00004:O/000007IX90002
AQP00`00JYD00002JYD00`00J[D00002000016Jf001Z]P000fJf00D006JfI[H006Jf0080009V]`04
001V]fJg0002I[L2000026Jg001V]`00I]P006KH0002I]P01000I]QVf6KH0P0000MRf000H]T006;H
001Rf@0200000f;I001oo`0DOol00`00Oomoo`0<Ool0015oo`03001oogoo02ioo`D00003<S`00000
00d00005AQP006ZE001ZU@0400001VZe001V]P00J[H0009V]P04001Z]P000002I[H02000I[MV]fJg
001V]`00I[L20002I[L01@00I[MV]fJg00000VKH00P006KH001Vf000I]P006KH0P0000AVf000H]QR
f080009Rf@03001oogoo01Aoo`03001oogoo00aoo`004Goo00<007ooOol0;goo00@003YL000f?0@0
00069M/000006IX000Th300000=Z]@0000000VZe00T006Zf001Z]P00I[H006Zf00000fJf00D006Jg
I[L006Jf00800003I[L006Jg009V]`03001V]fJg00800004I]P006KG0003I]P00`00I]QVf0020000
26KH001Rf000I]P006;I000GOol00`00Oomoo`0<Ool0015oo`03001oogoo031oo`H00004:O/00000
7KX300000`Th0000000600001FZE001Z]FZe00000fZe00<006Zf00000VZf00@006ZfJ[H0009V]P04
001Z]P000002I[H00`00I[MV]`0200000fJg001V]`020002I]L01000I]MVe`000VKH0P0000MVf000
001Vf000I]P00003I]P00`00Oomoo`0GOol00`00Oomoo`0;Ool0011oo`8003=oo`800003<S`002hK
008000038M/0000000T000966080009Z]@05001Z]FZe001Z]@020002J[H02@00J[H006Zf001V]P00
J[H00003I[H00`00I[L00002I[L01000I[MV]`000VKG0P0000=Ve`00I]L00P000VKH00D006KHI]P0
06KH00<00003I]P007oo01Uoo`03001oogoo00]oo`0047oo00<007ooOol0<Woo00<003IL00000`00
00<Uf`0000001P0000<0eP00000010000VZe0P0000EZ]@00J[EZ]@00009Z]P04001Z]P000002J[H0
0`00J[IZ]P02000016Jf001Z]P000fJg00H006JgI[L006KG0002I]L01000I]MVe`000VKH0P0000AV
f000001Vf0800003I]P007oo01Yoo`03001oogoo00]oo`0047oo00<007ooOol0<goo0P0000<b?000
:O/03P0000I66000J[EZ]@00J[D20002J[D00`00J[H00002J[H01`00J[IZ]VZf001Z]P00009Z]P04
001Z]VZf0002I[L01P00J[L00000I[L000=Ve`06001Ve`00001Vf0000VKH00D006KHI]P006KH0080
01aoo`03001oogoo00]oo`0047oo00<007ooOol0=7oo2@0000<AF@000000200000EZ]@00J[EZ]@00
009Z]@04001Z]@000002J[H01@00J[IZ]P00J[H00P000VZf00@006Zf001Z]P80009Z]`03001V]fJg
00800004I]L006KG0002I]L01P00I]QVf6KH001Vf0800003I]P0000001aoo`03001oogoo00]oo`00
47oo00<007ooOol0<Woo1`0000<Un`0000003P000VZe00H006ZeJ[D006Ze0002J[H01000J[H00000
0VZf00D006ZfJ[H006Zf00800003J[H006Zg009Z]`03001V]`00009Ve`04001VefKG0002I]L20000
16KH001Ve`000VKH00@007ooOol001]oo`03001oogoo00]oo`0047oo00<007ooOol0;goo0`000goo
00@0038l000Z708000037K/0000000T0000;@QP006ZeJ[D006Ze001Z]@00J[D00002J[H00`00J[H0
0003J[H00`00J[H00002J[H00`00J[IZ]P02000016Zg001Z]`000fKG00@006KG001Ve`80009Ve`06
001VefKG001Vf00017oo00<007ooOol067oo00<007ooOol02goo000@Ool00`00Oomoo`0/Ool30007
Ool400001B7K000000004GT0200000E26000J[EZ]@00009Z]@09001Z]@00J[D006Zf001Z]P00009Z
]P03001Z]P0000=Z]P03001Z]P00009Z]P04001Z]fZg0002I]L01P00J]L00000I]L0009Ve`800004
I]L006KG0006Ool00`00Oomoo`0GOol00`00Oomoo`0;Ool000moo`8002]oo`<000]oo`8000039O/0
000000`00004@QP006Ze0002J[D00`00J[D00002J[D02@00J[H006Ze001Z]P00J[H00002J[H01000
J[H000000VZf00@006Zg001Z]P80009Ze`06001VefKG001Ve`000VKG00@006KGI]L000Ioo`03001o
ogoo01Qoo`03001oogoo00Yoo`002Goo0P0017oo00<007ooOol09Woo10003Woo00D002XL000001gK
00<000033ET0000000T0009Z]@04001Z]@000002J[D00`00J[H00002J[D02@00J[H006Zf001Z]P00
J[H00002J[H01P00J[H00000J[H000=Ze`05001Ve`00J]L00002I]L200000fKG001oo`07Ool00`00
Oomoo`0GOol00`00Oomoo`0:Ool000Qoo`04001oogoo0003Ool00`00Oomoo`0SOol3000COol20000
1BGk000000004GX02@000T8H00<006Ze00000VZe00D006ZeJ[D006Ze0080009Z]@03001Z]P00009Z
]P06001Z]VZf001Z]P000VZf00<006ZfJ[H00P0000AZe`00J]L0009Ze`03001Ve`00009Ve`03001o
ogoo00Qoo`03001oogoo01Ioo`03001oogoo00Yoo`0027oo00@007ooOol000=oo`03001oogoo021o
o`<001Moo`<000037K/0000000X000926080009Z]@03001Z]@00009Z]@05001Z]FZe001Z]@020002
J[H01@00J[H006Zf00000fZf00<006Zf00000VZf00P006[GJ]L006[G001Ze`000fKG00<007ooOol0
2Goo00<007ooOol05Goo00<007ooOol02Woo0008Ool30004Ool00`00Oomoo`0MOol3000KOol50000
0`ei0000000:0002J[D01000J[D000000VZe00<006Ze00000VZe00D006ZfJ[H006Zf00800007J[H0
06ZfJ[H006Zf00000fZf00D006[G001Ze`00009Ze`800003I]L007oo00aoo`03001oogoo01Aoo`03
001oogoo00Yoo`0027oo00<007ooOol017oo00<007ooOol06Woo0`007Woo00D002Gk000001Vk00/0
0007@QP006ZeJ[D006Ze00000VZe00@006Ze0000009Z]@03001Z]P00009Z]P09001Z]P00J[H006Zf
001Z]P00009Z]P03001ZeP00009Ze`03001Ze`00009Ze`03001oogoo00aoo`03001oogoo01Aoo`03
001oogoo00Yoo`002Goo0`000Woo00<007ooOol05goo10008Woo0P0000<Mf`0000002P0000A26000
J[D0009Z]@06001Z]FZe001Z]@000VZe00@006Ze0000009Z]P03001Z]P00009Z]P04001Z]P00J[H2
0002J]H01@00J]L006[G00000f[G00<007ooOol03Goo00<007ooOol04goo00<007ooOol02Woo000>
Ool2000EOol3000WOol3000011FJ0000004H200000]26000001Z]@00J[D006Ze001Z]@00009Z]@03
001Z]@00009Z]P04001Z]P000002J[H01@00J[H006Zf00000V[F00H006[F000006[G0002J]L00`00
Oomoo`0?Ool00`00Oomoo`0COol00`00Oomoo`09Ool000ioo`03001oogoo015oo`<002Yoo`03000M
f`0000@000030AP0000000L0009Z]@0>001Z]FZe001Z]@00J[D006Ze001Z]@00J[H0009Z]P06001Z
]VZf001Z]P000VZf00<006[F00000V[F00@006[G001ZeP8001=oo`03001oogoo019oo`03001oogoo
00Uoo`003Woo00<007ooOol03Woo0`00;Woo0`0000<ANP000000200000E26@00J[EZ]@00009Z]@03
001Z]@00009Z]@04001Z]@00J[D20002J[H00`00J[H00002J[H01@00J[H006Zf00000f[F00D006[G
001ZeP0001Eoo`03001oogoo015oo`03001oogoo00Uoo`003Woo00<007ooOol02Woo1000<Goo00<0
07oo0000300000=26@0000000VZe00@006je0000009Z]@05001Z]@00J[D00002J[H00`00J[H00002
J[H01P00J[H00000J[H0009ZeP03001ZeP00009ZeP03001oogoo01=oo`03001oogoo015oo`03001o
ogoo00Uoo`003Woo00<007ooOol01goo0`00=Goo00D007ooOol000ej00d0009Z]@03001^]@00009Z
]@05001Z]@00001Z]@02000016Zf001Z]P000VZf00d006Zf001Z]P00J]H006[F001ZeP00J]H0000G
Ool00`00Oomoo`0@Ool00`00Oomoo`09Ool000ioo`03001oogoo00Aoo`<003Qoo`04001oogooOol;
0002@QT00`00J[D00002K[D02000J[EZ]@00J[D006Ze0002J[D02`00J[H006Ze001Z]P00J[H006Zf
00000V[F00D006[F001ZeP0001Uoo`03001oogoo00moo`03001oogoo00Uoo`003Goo00<007ooOol0
0Woo0`00>goo00<007ooOol00goo2P000T8I0P0000E^]@00001Z]@00009Z]@03001Z]@00009Z]@03
001Z]P00009Z]@04001Z]P00J[H20002J]H01@00J]H006[F00006Woo00<007ooOol03Woo00<007oo
Ool02Goo000=Ool00`00Ool00002000nOol00`00Oomoo`04Ool<0002K[D01`00K[D00000J[D006je
00800003J[D006Ze0080009Z]@05001Z]P00J[H00002J]H01@00J]H006[F000077oo00<007ooOol0
3Woo00<007ooOol027oo000=Ool20011Ool00`00Oomoo`05Ool90000148I001^]@000Vje00H006Ze
J[D006je0002J[D00`00J[D00002J[D02000J[D00000J[H006[F0002J]H00`00J]H0000LOol00`00
Oomoo`0>Ool00`00Oomoo`08Ool000moo`03001oogoo03ioo`03001oogoo00Moo`T00007K[D006je
K[D006Ze00000Vje00<006Ze00000VZe00<006Ze00000VZe00D006Zf001Z]P00009ZeP04001ZeP00
000MOol00`00Oomoo`0=Ool00`00Oomoo`08Ool0011oo`8003ioo`03001oogoo00Qoo`X00003K[D0
06je00800003K[D006je00800003J[D006je0080009Z]@04001Z]P00J[H2000016[F001ZeP007goo
00<007ooOol037oo00<007ooOol027oo000AOol2000mOol00`00Oomoo`09Ool600001d8I001^]Fje
001^]@00009^]@03001^]@00009Z]@03001^]@00009Z]@05001^]@00J[H00002J[H01000J]IZeP00
87oo00<007ooOol02goo00<007ooOol027oo000COol00`00Oomoo`0jOol00`00Oomoo`0:Ool70002
K[D00`00K[D00002K[D00`00K[D00002J[D01@00K[D006Ze00000Vje00<006Zf00000VZf00<006[F
00008Woo00<007ooOol02Woo00<007ooOol027oo000DOol2000iOol00`00Oomoo`0;Ool900000fje
001^]@0200000fje001^]@0200000fje001^]@0200000fje001^]@0200000fZf001^eP02000ROol0
0`00Oomoo`0:Ool00`00Oomoo`08Ool001Ioo`03001oogoo03Ioo`03001oogoo00aoo`D000926@03
001^]@00009^]@03001^]@00009^]@05001^]@00K[D00002K[D01@00K[D006Zf00000VkF00<007oo
Ool08Woo00<007ooOol02Goo00<007ooOol027oo000GOol3000eOol00`00Oomoo`0=Ool50000148I
001^]@000Vje00<006je00000Vje00@006je001^]@800006K[D006je001Z]P000VkF00<007ooOol0
8goo00<007ooOol02Goo00<007ooOol01goo000IOol00`00Oomoo`0cOol00`00Oomoo`0=Ool40000
100?000000000Vje00<006je00000Vje00D006je001^]@00009^]@05001^]@00K[D00002K]H00`00
Oomoo`0UOol00`00Oomoo`08Ool00`00Oomoo`07Ool001Yoo`03001oogoo039oo`03001oogoo00io
o`D00007@QT006jeK[D006je00000Vje00D006je001^]@00009^]@05001^]@00K[D00002K]H00`00
Oomoo`0VOol00`00Oomoo`07Ool00`00Oomoo`07Ool001]oo`<0035oo`03001oogoo00moo`@00004
@QT006je0002K[D01P00K[D006je001^]@800003K[D006je00<00004K[D006kF000XOol00`00Oomo
o`07Ool00`00Oomoo`07Ool001aoo`80035oo`03001oogoo00moo`D00006@QT00000K[D006je0P00
00A^]@00K[D0009^]@05001^]@00K[D00002K[D00`00Oomoo`0XOol00`00Oomoo`06Ool00`00Oomo
o`07Ool001ioo`8002moo`03001oogoo011oo`H0009^]@03001^]@00009^]@05001^]@00K[D00002
K[D00`00K[D00002K[D00`00Oomoo`0YOol00`00Oomoo`05Ool00`00Oomoo`07Ool0021oo`03001o
ogoo02aoo`03001oogoo015oo`<00006@QT006jeK[D006je0`0000=^]@00K[D00P0000A^]@00K[D0
009^]@03001oogoo02]oo`03001oogoo00Aoo`03001oogoo00Moo`008Goo00<007ooOol0:goo00<0
07ooOol04Goo100000E26@00001^]@00009^]@05001^]@00K[D00002K[D01@00K[D006je0000;goo
00<007ooOol017oo00<007ooOol01Woo000ROol3000ZOol00`00Oomoo`0BOol600002Fje001^]Fje
001^]@00K[D00002K[D01@00K[D006je0000;goo00<007ooOol017oo00<007ooOol01Woo000DOol2
0004Ool20003Ool400000goo001oo`0YOol00`00Oomoo`0COol20002@QT02P00K[D006je001^]@00
K[D006je0P0000A^]@00K[D0035oo`03001oogoo00=oo`03001oogoo00Ioo`004goo00@007ooOol0
00=oo`8000Aoo`03001oogoo009oo`8002Qoo`03001oogoo01=oo`<000926@08001^]Fje001^]@00
K[D0009^]@05001^]@00K[D0000bOol00`00Oomoo`02Ool00`00Oomoo`06Ool001=oo`04001oogoo
000:Ool00`00Oomoo`03Ool00`00Oomoo`0UOol00`00Oomoo`0DOol500001fje001^]Fje001^]@00
009^]@04001^]@00000dOol01@00Oomoogoo000027oo000COol01000Oomoo`002goo00<007ooOol0
0goo0P009Goo00<007ooOol05Goo0P0000A26@00K[D0009^]@80009^]@05001^]@00K[D0000eOol0
1000Oomoo`0027oo000COol01000Oomoo`0027oo00@007ooOol000Moo`8002=oo`03001oogoo01Eo
o`<00007@QT00000K[D006je00000Vje00D006je001^]@0003Eoo`04001oogoo0008Ool001Aoo`80
00Yoo`8000Uoo`03001oogoo025oo`03001oogoo01Ioo`@0009^]@05001^]@00K[D00002K[D2000g
Ool01000Oomoo`001goo000/Ool3000POol00`00Oomoo`0FOol200001T8I001^]Fje001^]@<00004
K[D006je000hOol00`00Ool00007Ool002ioo`03001oogoo01ioo`03001oogoo01Moo`800007@QT0
0000K[D006jd00000Vje00<006je0000>Goo0P001goo000_Ool2000NOol00`00Oomoo`0GOol50000
16je001^]0000Vje00<006je0000>7oo0`001goo000aOol00`00Oomoo`0KOol00`00Oomoo`0HOol0
1`00@QU26@00K[D006jd00<00003K[D007oo03=oo`D000Yoo`008Woo0P0017oo0P0017oo0`0000=o
o`00Ool06goo00<007ooOol067oo0P000T8I00<006jd00000Vjd00<006je0000<7oo10003goo000Q
Ool01000Oomoo`000goo0P001Goo00@007ooOomoo`8001Yoo`03001oogoo01Uoo`@00007K[@006jd
K[@006je0000:goo1@004goo000QOol01000Oomoo`0027oo10000goo0P006Goo00<007ooOol06Goo
0P0000Q26@00K[@006jd001^]000:7oo100067oo000QOol01000Oomoo`0027oo00<007oo00001Woo
0`005Woo00<007ooOol06Woo10000Vjd00<006jd00008goo1@0077oo000QOol01000Oomoo`0027oo
00<007oo000027oo00<007ooOol057oo00<007ooOol06goo00D0048I001^]000009^]003001oogoo
01eoo`@0025oo`008Woo0P002Woo0P002Goo0`004goo00<007ooOol06goo0`0000E^]000K[A^]000
01Yoo`D002Eoo`00>goo00<007ooOol04Goo00<007ooOol077oo100000=^]000Ool05Woo1000:Woo
000lOol00`00Oomoo`0?Ool00`00Oomoo`0MOol00`00@QT00002K[@00`00Oomoo`0@Ool5000^Ool0
03eoo`8000moo`03001oogoo01eoo`<0009^]003001oogoo00aoo`@003=oo`00<7oo0P0017oo0P00
17oo0P0000=oo`00Ool03Goo00<007ooOol07Woo10002Woo1@00=goo000_Ool01000Oomoo`000goo
0P000goo00D007ooOol007oo008000aoo`03001oogoo025oo`03001oogoo00Aoo`@003aoo`00;goo
00@007ooOol000Qoo`04001oogoo0003Ool00`00Ool00009Ool00`00Oomoo`0SOol50010Ool002mo
o`04001oogoo0008Ool30005Ool00`00Oomoo`08Ool00`00Oomoo`0OOol40015Ool002moo`04001o
ogoo0008Ool00`00Oomoo`06Ool20008Ool00`00Oomoo`0JOol50019Ool0031oo`8000Yoo`<000Mo
o`03001oogoo00Eoo`03001oogoo01Ioo`@004ioo`00Agoo0P001Goo00<007ooOol04Goo1@00DWoo
0019Ool00`00Oomoo`02Ool00`00Oomoo`0=Ool4001GOol004Yoo`05001oogooOol0000:Ool5001K
Ool003aoo`8000Aoo`8000Aoo`800005Ool00000Ool00006Ool4001POol003]oo`04001oogoo0003
Ool20003Ool01000Oomoo`000Woo2000I7oo000kOol01000Oomoo`0027oo00@007ooOol000=oo`80
06Uoo`00>goo00@007ooOol000Uoo`8006moo`00>goo00@007ooOol000Qoo`04001oogoo001^Ool0
03aoo`8000Yoo`8006moo`00^Goo001<Ool3001ZOol004eoo`03001oogoo06Uoo`00CGoo00<007oo
Ool0JGoo001=Ool00`00Oomoo`1YOol004eoo`03001oogoo06Uoo`00C7oo0P00Jgoo002iOol00001
\
\>"],
  ImageRangeCache->{{{0, 184.688}, {200.5, 0}} -> {-0.0208193, -0.0577046, 
  0.00522764, 0.00522764}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Plot3D[If[S>95, 
DownAndInCallDelta[2,S,100,95,0.2,Log[1.1],Log[1.05],t], 
UpAndInCallDelta[2,S,100,95,0.2,Log[1.1],Log[1.05],t]], {S, 80, 120}, {t, \
0.0001, 1}, BoxRatios -> {1, 1, 1}, PlotPoints -> 40];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.0855 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0567234 1.16608 0 1.16608 [
[.07825 .23514 -11.8839 -9 ]
[.07825 .23514 .11606 0 ]
[.21247 .17844 -11.3335 -9 ]
[.21247 .17844 .66648 0 ]
[.3545 .11847 -16.1747 -9 ]
[.3545 .11847 1.82534 0 ]
[.50502 .05496 -15.349 -9 ]
[.50502 .05496 2.65097 0 ]
[.66483 -0.01242 -14.5234 -9 ]
[.66483 -0.01242 3.4766 0 ]
[.09301 .88495 -10.1462 0 ]
[.09301 .88495 7.85383 9 ]
[.17991 .94587 -9.90019 0 ]
[.17991 .94587 8.09981 9 ]
[.25944 1.00163 -9.68252 0 ]
[.25944 1.00163 8.31748 9 ]
[.33251 1.05285 -9.48854 0 ]
[.33251 1.05285 8.51146 9 ]
[.39987 1.10006 -3.10486 0 ]
[.39987 1.10006 2.89514 9 ]
[.07351 .26066 -24 -2.61627 ]
[.07351 .26066 0 6.38373 ]
[.05582 .37136 -30 -2.71456 ]
[.05582 .37136 0 6.28544 ]
[.03666 .49129 -6 -2.82187 ]
[.03666 .49129 0 6.17813 ]
[.01584 .62164 -24 -2.93951 ]
[.01584 .62164 0 6.06049 ]
[-0.00687 .76383 -18 -3.06904 ]
[-0.00687 .76383 0 5.93096 ]
[ 0 0 0 0 ]
[ 1 1.0855 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
[(80)] .07825 .23514 .98066 1 Mshowa
.22216 .18933 m
.227 .19478 L
s
[(90)] .21247 .17844 .88892 1 Mshowa
.36358 .12987 m
.36813 .13557 L
s
[(100)] .3545 .11847 .79718 1 Mshowa
.51343 .06687 m
.51763 .07282 L
s
[(110)] .50502 .05496 .70545 1 Mshowa
.67245 0 m
.67627 .00621 L
s
[(120)] .66483 -0.01242 .61371 1 Mshowa
.125 Mabswid
.11461 .23456 m
.11764 .23771 L
s
.14105 .22344 m
.14405 .22662 L
s
.16779 .2122 m
.17076 .21541 L
s
.19482 .20083 m
.19776 .20407 L
s
.2498 .17771 m
.25268 .18101 L
s
.27776 .16595 m
.2806 .16928 L
s
.30604 .15406 m
.30885 .15742 L
s
.33465 .14204 m
.33741 .14542 L
s
.39286 .11756 m
.39554 .12101 L
s
.42247 .10511 m
.42512 .10859 L
s
.45243 .09251 m
.45504 .09602 L
s
.48275 .07977 m
.48531 .08331 L
s
.54447 .05381 m
.54695 .05742 L
s
.57589 .0406 m
.57832 .04424 L
s
.60769 .02723 m
.61007 .0309 L
s
.63987 .0137 m
.64221 .0174 L
s
.25 Mabswid
0 .80374 m
.40037 1.0855 L
s
.09485 .87049 m
.09577 .86326 L
s
[(0.2)] .09301 .88495 .12735 -1 Mshowa
.18136 .93137 m
.18209 .92412 L
s
[(0.4)] .17991 .94587 .10002 -1 Mshowa
.26055 .98709 m
.2611 .97983 L
s
[(0.6)] .25944 1.00163 .07584 -1 Mshowa
.3333 1.03829 m
.33369 1.03102 L
s
[(0.8)] .33251 1.05285 .05428 -1 Mshowa
.40037 1.0855 m
.40063 1.07821 L
s
[(1)] .39987 1.10006 .03495 -1 Mshowa
.125 Mabswid
.02453 .821 m
.02518 .81668 L
s
.04852 .83789 m
.04914 .83356 L
s
.07196 .85438 m
.07254 .85004 L
s
.11722 .88623 m
.11774 .88189 L
s
.13908 .90162 m
.13958 .89727 L
s
.16046 .91666 m
.16092 .91231 L
s
.20181 .94576 m
.20221 .9414 L
s
.22181 .95983 m
.22219 .95548 L
s
.24138 .97361 m
.24174 .96925 L
s
.27931 1.0003 m
.27961 .99593 L
s
.29768 1.01322 m
.29796 1.00886 L
s
.31567 1.02589 m
.31593 1.02152 L
s
.35057 1.05045 m
.35079 1.04608 L
s
.3675 1.06236 m
.3677 1.058 L
s
.3841 1.07404 m
.38427 1.06967 L
s
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
.08695 .25503 m
.09368 .25222 L
s
[(-0.5)] .07351 .26066 1 -0.41861 Mshowa
.06937 .36599 m
.07615 .3633 L
s
[(-0.25)] .05582 .37136 1 -0.39676 Mshowa
.05032 .4862 m
.05715 .48365 L
s
[(0)] .03666 .49129 1 -0.37292 Mshowa
.02961 .61686 m
.0365 .61447 L
s
[(0.25)] .01584 .62164 1 -0.34678 Mshowa
.00702 .75941 m
.01397 .7572 L
s
[(0.5)] -0.00687 .76383 1 -0.31799 Mshowa
.125 Mabswid
.08355 .27653 m
.08759 .27486 L
s
.08009 .29837 m
.08413 .29671 L
s
.07657 .32056 m
.08062 .31891 L
s
.073 .34309 m
.07706 .34146 L
s
.06568 .38925 m
.06975 .38766 L
s
.06194 .4129 m
.06601 .41132 L
s
.05813 .43693 m
.06221 .43537 L
s
.05426 .46136 m
.05835 .45981 L
s
.04632 .51145 m
.05042 .50994 L
s
.04225 .53713 m
.04636 .53564 L
s
.03811 .56325 m
.04223 .56178 L
s
.0339 .58983 m
.03802 .58837 L
s
.02526 .64437 m
.02939 .64296 L
s
.02082 .67237 m
.02496 .67098 L
s
.0163 .70086 m
.02046 .69949 L
s
.01171 .72987 m
.01587 .72853 L
s
.00226 .78949 m
.00643 .78819 L
s
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
.41799 .58158 m
.08846 .24555 L
s
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.41799 .58158 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
0 0 m
1 0 L
1 1.0855 L
0 1.0855 L
closepath
clip
newpath
.5 Mabswid
.324 .549 .914 r
.40027 .86005 .40794 .86915 .42067 .87342 .41301 .86424 Metetra
.297 .533 .91 r
.41301 .86424 .42067 .87342 .43352 .87815 .42588 .86891 Metetra
.27 .517 .906 r
.42588 .86891 .43352 .87815 .44649 .88331 .43886 .87403 Metetra
.244 .502 .901 r
.43886 .87403 .44649 .88331 .45958 .88889 .45196 .87959 Metetra
.219 .488 .896 r
.45196 .87959 .45958 .88889 .47281 .89486 .4652 .88556 Metetra
.194 .474 .891 r
.4652 .88556 .47281 .89486 .48618 .9012 .47858 .89192 Metetra
.171 .461 .886 r
.47858 .89192 .48618 .9012 .49969 .90787 .49211 .89863 Metetra
.149 .449 .881 r
.49211 .89863 .49969 .90787 .51336 .91485 .50579 .90566 Metetra
.128 .439 .876 r
.50579 .90566 .51336 .91485 .52718 .92209 .51963 .91299 Metetra
.11 .43 .872 r
.51963 .91299 .52718 .92209 .54117 .92957 .53363 .92056 Metetra
.093 .422 .868 r
.53363 .92056 .54117 .92957 .55533 .93725 .5478 .92835 Metetra
.078 .416 .865 r
.5478 .92835 .55533 .93725 .56965 .94509 .56214 .93631 Metetra
.065 .412 .862 r
.56214 .93631 .56965 .94509 .58415 .95305 .57667 .94441 Metetra
.054 .409 .86 r
.57667 .94441 .58415 .95305 .59883 .96109 .59137 .95261 Metetra
.957 .696 .5 r
.59137 .95261 .59883 .96109 .60272 .53174 .59626 .52621 Metetra
.576 .054 0 r
.59626 .52621 .60272 .53174 .61606 .54511 .60961 .53964 Metetra
.508 0 0 r
.60961 .53964 .61606 .54511 .62956 .55737 .62314 .55194 Metetra
0 .091 .563 r
.62314 .55194 .62956 .55737 .64324 .56855 .63684 .56315 Metetra
0 .167 .635 r
.63684 .56315 .64324 .56855 .65707 .5787 .6507 .5733 Metetra
0 .242 .702 r
.6507 .5733 .65707 .5787 .67104 .58784 .6647 .58244 Metetra
0 .313 .763 r
.6647 .58244 .67104 .58784 .68516 .59604 .67885 .59062 Metetra
0 .38 .816 r
.67885 .59062 .68516 .59604 .6994 .60332 .69313 .59786 Metetra
0 .441 .859 r
.69313 .59786 .6994 .60332 .71376 .60973 .70753 .60423 Metetra
.068 .495 .895 r
.70753 .60423 .71376 .60973 .72824 .61533 .72205 .60977 Metetra
.139 .541 .922 r
.72205 .60977 .72824 .61533 .74282 .62014 .73668 .61452 Metetra
.203 .58 .942 r
.73668 .61452 .74282 .62014 .75751 .62422 .75141 .61852 Metetra
.261 .614 .957 r
.75141 .61852 .75751 .62422 .7723 .62761 .76624 .62182 Metetra
.311 .641 .966 r
.76624 .62182 .7723 .62761 .78718 .63035 .78117 .62447 Metetra
.355 .663 .972 r
.78117 .62447 .78718 .63035 .80215 .63248 .79618 .6265 Metetra
.394 .682 .976 r
.79618 .6265 .80215 .63248 .8172 .63404 .81129 .62796 Metetra
.428 .697 .977 r
.81129 .62796 .8172 .63404 .83234 .63508 .82648 .62888 Metetra
.457 .709 .976 r
.82648 .62888 .83234 .63508 .84756 .63561 .84175 .6293 Metetra
.482 .719 .975 r
.84175 .6293 .84756 .63561 .86286 .63569 .8571 .62926 Metetra
.504 .727 .973 r
.8571 .62926 .86286 .63569 .87824 .63534 .87253 .6288 Metetra
.524 .733 .97 r
.87253 .6288 .87824 .63534 .8937 .6346 .88804 .62794 Metetra
.54 .738 .967 r
.88804 .62794 .8937 .6346 .90923 .63349 .90362 .62671 Metetra
.555 .743 .964 r
.90362 .62671 .90923 .63349 .92484 .63204 .91929 .62515 Metetra
.567 .746 .961 r
.91929 .62515 .92484 .63204 .94052 .63028 .93502 .62327 Metetra
.579 .748 .958 r
.93502 .62327 .94052 .63028 .95628 .62824 .95084 .62111 Metetra
.33 .55 .914 r
.39254 .85082 .40027 .86005 .41301 .86424 .4053 .85493 Metetra
.302 .534 .91 r
.4053 .85493 .41301 .86424 .42588 .86891 .41817 .85953 Metetra
.275 .518 .905 r
.41817 .85953 .42588 .86891 .43886 .87403 .43117 .86461 Metetra
.248 .502 .9 r
.43117 .86461 .43886 .87403 .45196 .87959 .44428 .87014 Metetra
.221 .487 .895 r
.44428 .87014 .45196 .87959 .4652 .88556 .45753 .87611 Metetra
.196 .472 .89 r
.45753 .87611 .4652 .88556 .47858 .89192 .47093 .88248 Metetra
.171 .459 .884 r
.47093 .88248 .47858 .89192 .49211 .89863 .48446 .88923 Metetra
.148 .446 .879 r
.48446 .88923 .49211 .89863 .50579 .90566 .49815 .89632 Metetra
.127 .435 .874 r
.49815 .89632 .50579 .90566 .51963 .91299 .51201 .90372 Metetra
.107 .425 .869 r
.51201 .90372 .51963 .91299 .53363 .92056 .52602 .91139 Metetra
.089 .417 .865 r
.52602 .91139 .53363 .92056 .5478 .92835 .54021 .91929 Metetra
.073 .411 .862 r
.54021 .91929 .5478 .92835 .56214 .93631 .55457 .92738 Metetra
.059 .405 .859 r
.55457 .92738 .56214 .93631 .57667 .94441 .56911 .93562 Metetra
.047 .402 .857 r
.56911 .93562 .57667 .94441 .59137 .95261 .58383 .94398 Metetra
.957 .696 .5 r
.58383 .94398 .59137 .95261 .59626 .52621 .58973 .52068 Metetra
.582 .059 0 r
.58973 .52068 .59626 .52621 .60961 .53964 .6031 .53417 Metetra
.514 0 0 r
.6031 .53417 .60961 .53964 .62314 .55194 .61665 .54652 Metetra
0 .087 .557 r
.61665 .54652 .62314 .55194 .63684 .56315 .63037 .55775 Metetra
0 .164 .63 r
.63037 .55775 .63684 .56315 .6507 .5733 .64426 .56791 Metetra
0 .24 .699 r
.64426 .56791 .6507 .5733 .6647 .58244 .6583 .57705 Metetra
0 .312 .761 r
.6583 .57705 .6647 .58244 .67885 .59062 .67248 .5852 Metetra
0 .38 .814 r
.67248 .5852 .67885 .59062 .69313 .59786 .68679 .59241 Metetra
0 .442 .859 r
.68679 .59241 .69313 .59786 .70753 .60423 .70123 .59873 Metetra
.068 .497 .895 r
.70123 .59873 .70753 .60423 .72205 .60977 .71579 .60421 Metetra
.14 .544 .923 r
.71579 .60421 .72205 .60977 .73668 .61452 .73046 .60888 Metetra
.205 .583 .943 r
.73046 .60888 .73668 .61452 .75141 .61852 .74524 .6128 Metetra
.263 .617 .958 r
.74524 .6128 .75141 .61852 .76624 .62182 .76012 .61601 Metetra
.314 .644 .967 r
.76012 .61601 .76624 .62182 .78117 .62447 .77509 .61856 Metetra
.359 .667 .973 r
.77509 .61856 .78117 .62447 .79618 .6265 .79016 .62048 Metetra
.398 .685 .976 r
.79016 .62048 .79618 .6265 .81129 .62796 .80531 .62183 Metetra
.431 .7 .977 r
.80531 .62183 .81129 .62796 .82648 .62888 .82055 .62264 Metetra
.461 .712 .977 r
.82055 .62264 .82648 .62888 .84175 .6293 .83587 .62295 Metetra
.486 .722 .975 r
.83587 .62295 .84175 .6293 .8571 .62926 .85127 .62279 Metetra
.508 .729 .973 r
.85127 .62279 .8571 .62926 .87253 .6288 .86676 .6222 Metetra
.527 .736 .97 r
.86676 .6222 .87253 .6288 .88804 .62794 .88232 .62122 Metetra
.544 .741 .967 r
.88232 .62122 .88804 .62794 .90362 .62671 .89796 .61987 Metetra
.558 .744 .964 r
.89796 .61987 .90362 .62671 .91929 .62515 .91367 .61818 Metetra
.57 .747 .961 r
.91367 .61818 .91929 .62515 .93502 .62327 .92947 .61618 Metetra
.581 .75 .957 r
.92947 .61618 .93502 .62327 .95084 .62111 .94534 .6139 Metetra
.336 .552 .913 r
.38475 .84147 .39254 .85082 .4053 .85493 .39753 .84549 Metetra
.308 .536 .909 r
.39753 .84549 .4053 .85493 .41817 .85953 .41041 .85001 Metetra
.279 .519 .905 r
.41041 .85001 .41817 .85953 .43117 .86461 .42342 .85504 Metetra
.251 .502 .899 r
.42342 .85504 .43117 .86461 .44428 .87014 .43655 .86054 Metetra
.224 .486 .894 r
.43655 .86054 .44428 .87014 .45753 .87611 .44981 .86649 Metetra
.197 .471 .888 r
.44981 .86649 .45753 .87611 .47093 .88248 .46321 .87288 Metetra
.172 .457 .883 r
.46321 .87288 .47093 .88248 .48446 .88923 .47676 .87966 Metetra
.148 .444 .877 r
.47676 .87966 .48446 .88923 .49815 .89632 .49046 .88681 Metetra
.125 .432 .872 r
.49046 .88681 .49815 .89632 .51201 .90372 .50432 .89428 Metetra
.104 .421 .867 r
.50432 .89428 .51201 .90372 .52602 .91139 .51835 .90205 Metetra
.085 .412 .862 r
.51835 .90205 .52602 .91139 .54021 .91929 .53255 .91007 Metetra
.067 .405 .858 r
.53255 .91007 .54021 .91929 .55457 .92738 .54693 .91829 Metetra
.052 .399 .855 r
.54693 .91829 .55457 .92738 .56911 .93562 .56149 .92668 Metetra
.039 .395 .852 r
.56149 .92668 .56911 .93562 .58383 .94398 .57623 .93519 Metetra
.957 .696 .5 r
.57623 .93519 .58383 .94398 .58973 .52068 .58313 .51516 Metetra
.588 .064 0 r
.58313 .51516 .58973 .52068 .6031 .53417 .59652 .52871 Metetra
.52 0 0 r
.59652 .52871 .6031 .53417 .61665 .54652 .61009 .5411 Metetra
0 .083 .551 r
.61009 .5411 .61665 .54652 .63037 .55775 .62384 .55236 Metetra
0 .16 .625 r
.62384 .55236 .63037 .55775 .64426 .56791 .63776 .56253 Metetra
0 .237 .695 r
.63776 .56253 .64426 .56791 .6583 .57705 .65183 .57166 Metetra
0 .311 .758 r
.65183 .57166 .6583 .57705 .67248 .5852 .66604 .57979 Metetra
0 .381 .813 r
.66604 .57979 .67248 .5852 .68679 .59241 .68039 .58696 Metetra
0 .443 .859 r
.68039 .58696 .68679 .59241 .70123 .59873 .69487 .59323 Metetra
.068 .499 .895 r
.69487 .59323 .70123 .59873 .71579 .60421 .70947 .59864 Metetra
.141 .546 .924 r
.70947 .59864 .71579 .60421 .73046 .60888 .72419 .60323 Metetra
.207 .587 .944 r
.72419 .60323 .73046 .60888 .74524 .6128 .73901 .60707 Metetra
.266 .62 .959 r
.73901 .60707 .74524 .6128 .76012 .61601 .75393 .61018 Metetra
.317 .648 .969 r
.75393 .61018 .76012 .61601 .77509 .61856 .76895 .61262 Metetra
.362 .67 .974 r
.76895 .61262 .77509 .61856 .79016 .62048 .78406 .61444 Metetra
.401 .688 .977 r
.78406 .61444 .79016 .62048 .80531 .62183 .79927 .61567 Metetra
.435 .703 .978 r
.79927 .61567 .80531 .62183 .82055 .62264 .81456 .61636 Metetra
.465 .715 .977 r
.81456 .61636 .82055 .62264 .83587 .62295 .82993 .61654 Metetra
.49 .724 .976 r
.82993 .61654 .83587 .62295 .85127 .62279 .84539 .61626 Metetra
.512 .732 .973 r
.84539 .61626 .85127 .62279 .86676 .6222 .86092 .61555 Metetra
.531 .738 .97 r
.86092 .61555 .86676 .6222 .88232 .62122 .87654 .61443 Metetra
.547 .743 .967 r
.87654 .61443 .88232 .62122 .89796 .61987 .89223 .61296 Metetra
.561 .746 .964 r
.89223 .61296 .89796 .61987 .91367 .61818 .908 .61114 Metetra
.574 .749 .96 r
.908 .61114 .91367 .61818 .92947 .61618 .92385 .60902 Metetra
.584 .751 .957 r
.92385 .60902 .92947 .61618 .94534 .6139 .93978 .60662 Metetra
.342 .555 .913 r
.3769 .832 .38475 .84147 .39753 .84549 .38969 .83591 Metetra
.314 .537 .909 r
.38969 .83591 .39753 .84549 .41041 .85001 .40259 .84035 Metetra
.285 .52 .904 r
.40259 .84035 .41041 .85001 .42342 .85504 .41561 .84531 Metetra
.256 .503 .899 r
.41561 .84531 .42342 .85504 .43655 .86054 .42875 .85078 Metetra
.227 .486 .893 r
.42875 .85078 .43655 .86054 .44981 .86649 .44202 .85672 Metetra
.2 .47 .887 r
.44202 .85672 .44981 .86649 .46321 .87288 .45543 .86311 Metetra
.173 .455 .882 r
.45543 .86311 .46321 .87288 .47676 .87966 .46899 .86992 Metetra
.147 .441 .876 r
.46899 .86992 .47676 .87966 .49046 .88681 .4827 .87712 Metetra
.123 .428 .87 r
.4827 .87712 .49046 .88681 .50432 .89428 .49657 .88467 Metetra
.101 .417 .864 r
.49657 .88467 .50432 .89428 .51835 .90205 .51061 .89253 Metetra
.081 .407 .859 r
.51061 .89253 .51835 .90205 .53255 .91007 .52483 .90067 Metetra
.062 .399 .855 r
.52483 .90067 .53255 .91007 .54693 .91829 .53922 .90902 Metetra
.046 .392 .851 r
.53922 .90902 .54693 .91829 .56149 .92668 .55379 .91757 Metetra
.032 .387 .848 r
.55379 .91757 .56149 .92668 .57623 .93519 .56856 .92624 Metetra
.956 .696 .501 r
.56856 .92624 .57623 .93519 .58313 .51516 .57646 .50964 Metetra
.594 .069 0 r
.57646 .50964 .58313 .51516 .59652 .52871 .58987 .52326 Metetra
.526 0 0 r
.58987 .52326 .59652 .52871 .61009 .5411 .60346 .5357 Metetra
.451 0 0 r
.60346 .5357 .61009 .5411 .62384 .55236 .61724 .54699 Metetra
0 .157 .62 r
.61724 .54699 .62384 .55236 .63776 .56253 .63118 .55717 Metetra
0 .235 .691 r
.63118 .55717 .63776 .56253 .65183 .57166 .64528 .56629 Metetra
0 .31 .755 r
.64528 .56629 .65183 .57166 .66604 .57979 .65953 .57439 Metetra
0 .381 .811 r
.65953 .57439 .66604 .57979 .68039 .58696 .67392 .58152 Metetra
0 .445 .858 r
.67392 .58152 .68039 .58696 .69487 .59323 .68844 .58773 Metetra
.068 .501 .896 r
.68844 .58773 .69487 .59323 .70947 .59864 .70308 .59307 Metetra
.142 .549 .924 r
.70308 .59307 .70947 .59864 .72419 .60323 .71784 .59758 Metetra
.209 .59 .946 r
.71784 .59758 .72419 .60323 .73901 .60707 .73271 .60132 Metetra
.269 .624 .96 r
.73271 .60132 .73901 .60707 .75393 .61018 .74768 .60433 Metetra
.321 .651 .97 r
.74768 .60433 .75393 .61018 .76895 .61262 .76274 .60667 Metetra
.366 .674 .976 r
.76274 .60667 .76895 .61262 .78406 .61444 .77791 .60837 Metetra
.406 .692 .978 r
.77791 .60837 .78406 .61444 .79927 .61567 .79316 .60947 Metetra
.44 .707 .979 r
.79316 .60947 .79927 .61567 .81456 .61636 .8085 .61004 Metetra
.469 .718 .978 r
.8085 .61004 .81456 .61636 .82993 .61654 .82392 .61009 Metetra
.494 .727 .976 r
.82392 .61009 .82993 .61654 .84539 .61626 .83943 .60968 Metetra
.516 .734 .973 r
.83943 .60968 .84539 .61626 .86092 .61555 .85502 .60883 Metetra
.535 .74 .97 r
.85502 .60883 .86092 .61555 .87654 .61443 .87069 .60759 Metetra
.551 .745 .967 r
.87069 .60759 .87654 .61443 .89223 .61296 .88644 .60598 Metetra
.565 .748 .963 r
.88644 .60598 .89223 .61296 .908 .61114 .90227 .60403 Metetra
.577 .751 .96 r
.90227 .60403 .908 .61114 .92385 .60902 .91817 .60178 Metetra
.587 .753 .957 r
.91817 .60178 .92385 .60902 .93978 .60662 .93416 .59925 Metetra
.349 .557 .913 r
.36899 .8224 .3769 .832 .38969 .83591 .3818 .8262 Metetra
.32 .539 .908 r
.3818 .8262 .38969 .83591 .40259 .84035 .39471 .83055 Metetra
.29 .521 .904 r
.39471 .83055 .40259 .84035 .41561 .84531 .40774 .83544 Metetra
.26 .503 .898 r
.40774 .83544 .41561 .84531 .42875 .85078 .42089 .84085 Metetra
.231 .486 .892 r
.42089 .84085 .42875 .85078 .44202 .85672 .43417 .84677 Metetra
.202 .469 .886 r
.43417 .84677 .44202 .85672 .45543 .86311 .44759 .85316 Metetra
.174 .453 .88 r
.44759 .85316 .45543 .86311 .46899 .86992 .46116 .86 Metetra
.147 .438 .874 r
.46116 .86 .46899 .86992 .4827 .87712 .47488 .86725 Metetra
.122 .425 .868 r
.47488 .86725 .4827 .87712 .49657 .88467 .48876 .87488 Metetra
.098 .412 .862 r
.48876 .87488 .49657 .88467 .51061 .89253 .50281 .88284 Metetra
.077 .402 .856 r
.50281 .88284 .51061 .89253 .52483 .90067 .51703 .89109 Metetra
.057 .393 .852 r
.51703 .89109 .52483 .90067 .53922 .90902 .53144 .89958 Metetra
.039 .385 .847 r
.53144 .89958 .53922 .90902 .55379 .91757 .54603 .90828 Metetra
.024 .38 .844 r
.54603 .90828 .55379 .91757 .56856 .92624 .56081 .91713 Metetra
.956 .696 .501 r
.56081 .91713 .56856 .92624 .57646 .50964 .56972 .50414 Metetra
.6 .074 0 r
.56972 .50414 .57646 .50964 .58987 .52326 .58314 .51783 Metetra
.533 .002 0 r
.58314 .51783 .58987 .52326 .60346 .5357 .59676 .53032 Metetra
.457 0 0 r
.59676 .53032 .60346 .5357 .61724 .54699 .61056 .54163 Metetra
0 .154 .614 r
.61056 .54163 .61724 .54699 .63118 .55717 .62453 .55182 Metetra
0 .233 .686 r
.62453 .55182 .63118 .55717 .64528 .56629 .63867 .56093 Metetra
0 .309 .752 r
.63867 .56093 .64528 .56629 .65953 .57439 .65295 .569 Metetra
0 .381 .81 r
.65295 .569 .65953 .57439 .67392 .58152 .66738 .57608 Metetra
0 .446 .858 r
.66738 .57608 .67392 .58152 .68844 .58773 .68194 .58223 Metetra
.068 .503 .896 r
.68194 .58223 .68844 .58773 .70308 .59307 .69662 .58749 Metetra
.144 .552 .925 r
.69662 .58749 .70308 .59307 .71784 .59758 .71143 .59192 Metetra
.212 .594 .947 r
.71143 .59192 .71784 .59758 .73271 .60132 .72634 .59556 Metetra
.272 .628 .962 r
.72634 .59556 .73271 .60132 .74768 .60433 .74135 .59847 Metetra
.325 .655 .971 r
.74135 .59847 .74768 .60433 .76274 .60667 .75647 .60068 Metetra
.37 .678 .977 r
.75647 .60068 .76274 .60667 .77791 .60837 .77168 .60226 Metetra
.41 .696 .979 r
.77168 .60226 .77791 .60837 .79316 .60947 .78699 .60324 Metetra
.444 .71 .979 r
.78699 .60324 .79316 .60947 .8085 .61004 .80238 .60367 Metetra
.473 .721 .978 r
.80238 .60367 .8085 .61004 .82392 .61009 .81785 .60359 Metetra
.498 .73 .976 r
.81785 .60359 .82392 .61009 .83943 .60968 .83341 .60304 Metetra
.52 .737 .973 r
.83341 .60304 .83943 .60968 .85502 .60883 .84906 .60205 Metetra
.539 .743 .97 r
.84906 .60205 .85502 .60883 .87069 .60759 .86478 .60067 Metetra
.555 .747 .966 r
.86478 .60067 .87069 .60759 .88644 .60598 .88059 .59892 Metetra
.568 .75 .963 r
.88059 .59892 .88644 .60598 .90227 .60403 .89647 .59684 Metetra
.58 .752 .959 r
.89647 .59684 .90227 .60403 .91817 .60178 .91243 .59445 Metetra
.591 .754 .956 r
.91243 .59445 .91817 .60178 .93416 .59925 .92848 .59179 Metetra
.357 .56 .912 r
.36102 .81268 .36899 .8224 .3818 .8262 .37385 .81635 Metetra
.327 .541 .908 r
.37385 .81635 .3818 .8262 .39471 .83055 .38677 .82059 Metetra
.296 .523 .903 r
.38677 .82059 .39471 .83055 .40774 .83544 .39982 .8254 Metetra
.266 .504 .898 r
.39982 .8254 .40774 .83544 .42089 .84085 .41298 .83076 Metetra
.235 .486 .892 r
.41298 .83076 .42089 .84085 .43417 .84677 .42626 .83665 Metetra
.205 .469 .885 r
.42626 .83665 .43417 .84677 .44759 .85316 .43969 .84304 Metetra
.176 .452 .879 r
.43969 .84304 .44759 .85316 .46116 .86 .45326 .8499 Metetra
.148 .436 .872 r
.45326 .8499 .46116 .86 .47488 .86725 .46699 .8572 Metetra
.121 .421 .866 r
.46699 .8572 .47488 .86725 .48876 .87488 .48088 .86489 Metetra
.096 .408 .859 r
.48088 .86489 .48876 .87488 .50281 .88284 .49494 .87295 Metetra
.073 .396 .853 r
.49494 .87295 .50281 .88284 .51703 .89109 .50917 .88132 Metetra
.052 .386 .848 r
.50917 .88132 .51703 .89109 .53144 .89958 .52359 .88996 Metetra
.033 .378 .843 r
.52359 .88996 .53144 .89958 .54603 .90828 .5382 .89881 Metetra
.017 .372 .839 r
.5382 .89881 .54603 .90828 .56081 .91713 .55299 .90784 Metetra
.956 .696 .501 r
.55299 .90784 .56081 .91713 .56972 .50414 .56291 .49866 Metetra
.607 .08 0 r
.56291 .49866 .56972 .50414 .58314 .51783 .57635 .51242 Metetra
.54 .007 0 r
.57635 .51242 .58314 .51783 .59676 .53032 .58999 .52495 Metetra
.464 0 0 r
.58999 .52495 .59676 .53032 .61056 .54163 .60381 .5363 Metetra
0 .15 .608 r
.60381 .5363 .61056 .54163 .62453 .55182 .61782 .54649 Metetra
0 .23 .682 r
.61782 .54649 .62453 .55182 .63867 .56093 .63198 .55559 Metetra
0 .308 .749 r
.63198 .55559 .63867 .56093 .65295 .569 .6463 .56363 Metetra
0 .381 .808 r
.6463 .56363 .65295 .569 .66738 .57608 .66077 .57066 Metetra
0 .448 .858 r
.66077 .57066 .66738 .57608 .68194 .58223 .67537 .57674 Metetra
.068 .506 .897 r
.67537 .57674 .68194 .58223 .69662 .58749 .6901 .58192 Metetra
.145 .556 .927 r
.6901 .58192 .69662 .58749 .71143 .59192 .70494 .58626 Metetra
.215 .598 .948 r
.70494 .58626 .71143 .59192 .72634 .59556 .7199 .58979 Metetra
.276 .632 .963 r
.7199 .58979 .72634 .59556 .74135 .59847 .73496 .59258 Metetra
.329 .66 .972 r
.73496 .59258 .74135 .59847 .75647 .60068 .75013 .59468 Metetra
.375 .682 .978 r
.75013 .59468 .75647 .60068 .77168 .60226 .76539 .59612 Metetra
.415 .7 .98 r
.76539 .59612 .77168 .60226 .78699 .60324 .78074 .59697 Metetra
.449 .714 .98 r
.78074 .59697 .78699 .60324 .80238 .60367 .79619 .59726 Metetra
.478 .725 .979 r
.79619 .59726 .80238 .60367 .81785 .60359 .81172 .59704 Metetra
.503 .733 .976 r
.81172 .59704 .81785 .60359 .83341 .60304 .82733 .59634 Metetra
.524 .74 .973 r
.82733 .59634 .83341 .60304 .84906 .60205 .84303 .59521 Metetra
.543 .745 .97 r
.84303 .59521 .84906 .60205 .86478 .60067 .8588 .59368 Metetra
.559 .749 .966 r
.8588 .59368 .86478 .60067 .88059 .59892 .87466 .59179 Metetra
.572 .752 .962 r
.87466 .59179 .88059 .59892 .89647 .59684 .8906 .58957 Metetra
.584 .754 .959 r
.8906 .58957 .89647 .59684 .91243 .59445 .90663 .58705 Metetra
.594 .756 .955 r
.90663 .58705 .91243 .59445 .92848 .59179 .92273 .58424 Metetra
.365 .562 .912 r
.35299 .80283 .36102 .81268 .37385 .81635 .36583 .80636 Metetra
.334 .544 .907 r
.36583 .80636 .37385 .81635 .38677 .82059 .37878 .81049 Metetra
.303 .525 .903 r
.37878 .81049 .38677 .82059 .39982 .8254 .39183 .81521 Metetra
.271 .505 .897 r
.39183 .81521 .39982 .8254 .41298 .83076 .405 .8205 Metetra
.24 .487 .891 r
.405 .8205 .41298 .83076 .42626 .83665 .4183 .82635 Metetra
.209 .468 .884 r
.4183 .82635 .42626 .83665 .43969 .84304 .43173 .83272 Metetra
.178 .45 .878 r
.43173 .83272 .43969 .84304 .45326 .8499 .44531 .8396 Metetra
.149 .433 .871 r
.44531 .8396 .45326 .8499 .46699 .8572 .45904 .84694 Metetra
.121 .418 .864 r
.45904 .84694 .46699 .8572 .48088 .86489 .47294 .85471 Metetra
.094 .404 .857 r
.47294 .85471 .48088 .86489 .49494 .87295 .487 .86286 Metetra
.07 .391 .85 r
.487 .86286 .49494 .87295 .50917 .88132 .50125 .87135 Metetra
.047 .38 .844 r
.50125 .87135 .50917 .88132 .52359 .88996 .51568 .88014 Metetra
.027 .371 .839 r
.51568 .88014 .52359 .88996 .5382 .89881 .53029 .88916 Metetra
.009 .364 .834 r
.53029 .88916 .5382 .89881 .55299 .90784 .54511 .89838 Metetra
.956 .696 .502 r
.54511 .89838 .55299 .90784 .56291 .49866 .55603 .4932 Metetra
.613 .085 0 r
.55603 .4932 .56291 .49866 .57635 .51242 .56948 .50704 Metetra
.547 .013 0 r
.56948 .50704 .57635 .51242 .58999 .52495 .58314 .51962 Metetra
.471 0 0 r
.58314 .51962 .58999 .52495 .60381 .5363 .59699 .53099 Metetra
0 .146 .601 r
.59699 .53099 .60381 .5363 .61782 .54649 .61102 .54119 Metetra
0 .228 .677 r
.61102 .54119 .61782 .54649 .63198 .55559 .62522 .55027 Metetra
0 .307 .746 r
.62522 .55027 .63198 .55559 .6463 .56363 .63958 .55827 Metetra
0 .382 .807 r
.63958 .55827 .6463 .56363 .66077 .57066 .65408 .56525 Metetra
0 .45 .857 r
.65408 .56525 .66077 .57066 .67537 .57674 .66873 .57127 Metetra
.069 .509 .897 r
.66873 .57127 .67537 .57674 .6901 .58192 .68349 .57636 Metetra
.147 .56 .928 r
.68349 .57636 .6901 .58192 .70494 .58626 .69839 .58059 Metetra
.218 .602 .95 r
.69839 .58059 .70494 .58626 .7199 .58979 .71339 .58402 Metetra
.28 .636 .964 r
.71339 .58402 .7199 .58979 .73496 .59258 .7285 .58668 Metetra
.333 .664 .974 r
.7285 .58668 .73496 .59258 .75013 .59468 .74372 .58865 Metetra
.38 .686 .979 r
.74372 .58865 .75013 .59468 .76539 .59612 .75903 .58996 Metetra
.42 .704 .981 r
.75903 .58996 .76539 .59612 .78074 .59697 .77443 .59066 Metetra
.454 .717 .981 r
.77443 .59066 .78074 .59697 .79619 .59726 .78993 .5908 Metetra
.483 .728 .979 r
.78993 .5908 .79619 .59726 .81172 .59704 .80551 .59043 Metetra
.508 .736 .976 r
.80551 .59043 .81172 .59704 .82733 .59634 .82118 .58958 Metetra
.529 .743 .973 r
.82118 .58958 .82733 .59634 .84303 .59521 .83693 .5883 Metetra
.547 .747 .969 r
.83693 .5883 .84303 .59521 .8588 .59368 .85276 .58663 Metetra
.563 .751 .966 r
.85276 .58663 .8588 .59368 .87466 .59179 .86868 .58459 Metetra
.576 .754 .962 r
.86868 .58459 .87466 .59179 .8906 .58957 .88467 .58222 Metetra
.587 .756 .958 r
.88467 .58222 .8906 .58957 .90663 .58705 .90075 .57955 Metetra
.597 .757 .955 r
.90075 .57955 .90663 .58705 .92273 .58424 .91691 .5766 Metetra
.373 .566 .911 r
.3449 .79285 .35299 .80283 .36583 .80636 .35776 .79623 Metetra
.342 .546 .907 r
.35776 .79623 .36583 .80636 .37878 .81049 .37072 .80023 Metetra
.31 .527 .902 r
.37072 .80023 .37878 .81049 .39183 .81521 .38378 .80485 Metetra
.278 .507 .896 r
.38378 .80485 .39183 .81521 .405 .8205 .39696 .81006 Metetra
.245 .487 .89 r
.39696 .81006 .405 .8205 .4183 .82635 .41027 .81586 Metetra
.213 .468 .883 r
.41027 .81586 .4183 .82635 .43173 .83272 .42371 .82221 Metetra
.181 .449 .876 r
.42371 .82221 .43173 .83272 .44531 .8396 .43729 .8291 Metetra
.15 .431 .869 r
.43729 .8291 .44531 .8396 .45904 .84694 .45103 .83648 Metetra
.12 .415 .862 r
.45103 .83648 .45904 .84694 .47294 .85471 .46493 .84432 Metetra
.092 .399 .854 r
.46493 .84432 .47294 .85471 .487 .86286 .479 .85257 Metetra
.066 .386 .847 r
.479 .85257 .487 .86286 .50125 .87135 .49326 .86118 Metetra
.042 .374 .841 r
.49326 .86118 .50125 .87135 .51568 .88014 .50769 .87011 Metetra
.02 .364 .835 r
.50769 .87011 .51568 .88014 .53029 .88916 .52232 .87931 Metetra
.001 .356 .83 r
.52232 .87931 .53029 .88916 .54511 .89838 .53715 .88871 Metetra
.956 .696 .502 r
.53715 .88871 .54511 .89838 .55603 .4932 .54907 .48777 Metetra
.62 .091 0 r
.54907 .48777 .55603 .4932 .56948 .50704 .56254 .50168 Metetra
.554 .018 0 r
.56254 .50168 .56948 .50704 .58314 .51962 .57622 .51432 Metetra
.478 0 0 r
.57622 .51432 .58314 .51962 .59699 .53099 .59009 .52572 Metetra
0 .142 .595 r
.59009 .52572 .59699 .53099 .61102 .54119 .60415 .53593 Metetra
0 .225 .672 r
.60415 .53593 .61102 .54119 .62522 .55027 .61839 .54498 Metetra
0 .306 .743 r
.61839 .54498 .62522 .55027 .63958 .55827 .63278 .55295 Metetra
0 .383 .805 r
.63278 .55295 .63958 .55827 .65408 .56525 .64732 .55987 Metetra
0 .452 .857 r
.64732 .55987 .65408 .56525 .66873 .57127 .66201 .5658 Metetra
.07 .512 .898 r
.66201 .5658 .66873 .57127 .68349 .57636 .67682 .57081 Metetra
.15 .564 .929 r
.67682 .57081 .68349 .57636 .69839 .58059 .69176 .57493 Metetra
.221 .606 .951 r
.69176 .57493 .69839 .58059 .71339 .58402 .70681 .57823 Metetra
.284 .641 .966 r
.70681 .57823 .71339 .58402 .7285 .58668 .72197 .58077 Metetra
.338 .669 .975 r
.72197 .58077 .7285 .58668 .74372 .58865 .73723 .5826 Metetra
.385 .691 .98 r
.73723 .5826 .74372 .58865 .75903 .58996 .75259 .58376 Metetra
.425 .708 .982 r
.75259 .58376 .75903 .58996 .77443 .59066 .76805 .58431 Metetra
.459 .721 .981 r
.76805 .58431 .77443 .59066 .78993 .5908 .7836 .5843 Metetra
.488 .732 .979 r
.7836 .5843 .78993 .5908 .80551 .59043 .79923 .58377 Metetra
.513 .739 .976 r
.79923 .58377 .80551 .59043 .82118 .58958 .81495 .58277 Metetra
.534 .745 .973 r
.81495 .58277 .82118 .58958 .83693 .5883 .83076 .58133 Metetra
.552 .75 .969 r
.83076 .58133 .83693 .5883 .85276 .58663 .84665 .57949 Metetra
.567 .753 .965 r
.84665 .57949 .85276 .58663 .86868 .58459 .86262 .5773 Metetra
.58 .756 .961 r
.86262 .5773 .86868 .58459 .88467 .58222 .87867 .57478 Metetra
.591 .757 .958 r
.87867 .57478 .88467 .58222 .90075 .57955 .89481 .57196 Metetra
.601 .758 .954 r
.89481 .57196 .90075 .57955 .91691 .5766 .91103 .56887 Metetra
.383 .569 .911 r
.33674 .78275 .3449 .79285 .35776 .79623 .34962 .78596 Metetra
.351 .549 .906 r
.34962 .78596 .35776 .79623 .37072 .80023 .3626 .78982 Metetra
.318 .529 .902 r
.3626 .78982 .37072 .80023 .38378 .80485 .37568 .79431 Metetra
.285 .509 .896 r
.37568 .79431 .38378 .80485 .39696 .81006 .38887 .79944 Metetra
.251 .488 .89 r
.38887 .79944 .39696 .81006 .41027 .81586 .40218 .80518 Metetra
.217 .468 .883 r
.40218 .80518 .41027 .81586 .42371 .82221 .41563 .8115 Metetra
.184 .448 .875 r
.41563 .8115 .42371 .82221 .43729 .8291 .42922 .81839 Metetra
.152 .429 .867 r
.42922 .81839 .43729 .8291 .45103 .83648 .44296 .82581 Metetra
.121 .411 .859 r
.44296 .82581 .45103 .83648 .46493 .84432 .45686 .83371 Metetra
.091 .395 .852 r
.45686 .83371 .46493 .84432 .479 .85257 .47094 .84205 Metetra
.063 .38 .844 r
.47094 .84205 .479 .85257 .49326 .86118 .4852 .85079 Metetra
.038 .367 .837 r
.4852 .85079 .49326 .86118 .50769 .87011 .49964 .85987 Metetra
.014 .356 .83 r
.49964 .85987 .50769 .87011 .52232 .87931 .51428 .86925 Metetra
0 .347 .824 r
.51428 .86925 .52232 .87931 .53715 .88871 .52911 .87885 Metetra
.956 .697 .502 r
.52911 .87885 .53715 .88871 .54907 .48777 .54203 .48237 Metetra
.627 .098 0 r
.54203 .48237 .54907 .48777 .56254 .50168 .55551 .49637 Metetra
.561 .024 0 r
.55551 .49637 .56254 .50168 .57622 .51432 .56921 .50906 Metetra
.486 0 0 r
.56921 .50906 .57622 .51432 .59009 .52572 .58312 .52049 Metetra
0 .138 .587 r
.58312 .52049 .59009 .52572 .60415 .53593 .59721 .5307 Metetra
0 .223 .666 r
.59721 .5307 .60415 .53593 .61839 .54498 .61147 .53973 Metetra
0 .305 .739 r
.61147 .53973 .61839 .54498 .63278 .55295 .6259 .54766 Metetra
0 .383 .803 r
.6259 .54766 .63278 .55295 .64732 .55987 .64049 .55451 Metetra
0 .454 .856 r
.64049 .55451 .64732 .55987 .66201 .5658 .65521 .56036 Metetra
.071 .516 .899 r
.65521 .56036 .66201 .5658 .67682 .57081 .67007 .56526 Metetra
.152 .568 .93 r
.67007 .56526 .67682 .57081 .69176 .57493 .68505 .56927 Metetra
.225 .611 .953 r
.68505 .56927 .69176 .57493 .70681 .57823 .70015 .57245 Metetra
.289 .646 .968 r
.70015 .57245 .70681 .57823 .72197 .58077 .71536 .57485 Metetra
.344 .674 .977 r
.71536 .57485 .72197 .58077 .73723 .5826 .73067 .57652 Metetra
.391 .695 .981 r
.73067 .57652 .73723 .5826 .75259 .58376 .74609 .57753 Metetra
.431 .712 .983 r
.74609 .57753 .75259 .58376 .76805 .58431 .76159 .57792 Metetra
.465 .725 .982 r
.76159 .57792 .76805 .58431 .7836 .5843 .7772 .57775 Metetra
.494 .735 .98 r
.7772 .57775 .7836 .5843 .79923 .58377 .79288 .57705 Metetra
.518 .743 .976 r
.79288 .57705 .79923 .58377 .81495 .58277 .80866 .57588 Metetra
.539 .748 .973 r
.80866 .57588 .81495 .58277 .83076 .58133 .82452 .57428 Metetra
.557 .752 .969 r
.82452 .57428 .83076 .58133 .84665 .57949 .84047 .57228 Metetra
.571 .755 .965 r
.84047 .57228 .84665 .57949 .86262 .5773 .85649 .56993 Metetra
.584 .757 .961 r
.85649 .56993 .86262 .5773 .87867 .57478 .87261 .56724 Metetra
.595 .759 .957 r
.87261 .56724 .87867 .57478 .89481 .57196 .8888 .56427 Metetra
.604 .76 .953 r
.8888 .56427 .89481 .57196 .91103 .56887 .90508 .56103 Metetra
.392 .573 .91 r
.32853 .77252 .33674 .78275 .34962 .78596 .34143 .77555 Metetra
.36 .553 .906 r
.34143 .77555 .34962 .78596 .3626 .78982 .35442 .77925 Metetra
.327 .532 .901 r
.35442 .77925 .3626 .78982 .37568 .79431 .36751 .78361 Metetra
.292 .511 .895 r
.36751 .78361 .37568 .79431 .38887 .79944 .38071 .78863 Metetra
.258 .489 .889 r
.38071 .78863 .38887 .79944 .40218 .80518 .39403 .7943 Metetra
.223 .468 .882 r
.39403 .7943 .40218 .80518 .41563 .8115 .40749 .80058 Metetra
.188 .447 .874 r
.40749 .80058 .41563 .8115 .42922 .81839 .42108 .80747 Metetra
.154 .427 .866 r
.42108 .80747 .42922 .81839 .44296 .82581 .43482 .81491 Metetra
.121 .409 .857 r
.43482 .81491 .44296 .82581 .45686 .83371 .44873 .82287 Metetra
.09 .391 .849 r
.44873 .82287 .45686 .83371 .47094 .84205 .46281 .83131 Metetra
.061 .375 .841 r
.46281 .83131 .47094 .84205 .4852 .85079 .47707 .84018 Metetra
.033 .361 .833 r
.47707 .84018 .4852 .85079 .49964 .85987 .49152 .84941 Metetra
.008 .349 .826 r
.49152 .84941 .49964 .85987 .51428 .86925 .50616 .85897 Metetra
0 .338 .819 r
.50616 .85897 .51428 .86925 .52911 .87885 .52101 .86878 Metetra
.956 .697 .503 r
.52101 .86878 .52911 .87885 .54203 .48237 .53491 .47702 Metetra
.634 .104 0 r
.53491 .47702 .54203 .48237 .55551 .49637 .54841 .4911 Metetra
.569 .03 0 r
.54841 .4911 .55551 .49637 .56921 .50906 .56213 .50385 Metetra
.493 0 0 r
.56213 .50385 .56921 .50906 .58312 .52049 .57606 .5153 Metetra
0 .134 .58 r
.57606 .5153 .58312 .52049 .59721 .5307 .59018 .52551 Metetra
0 .22 .661 r
.59018 .52551 .59721 .5307 .61147 .53973 .60448 .53453 Metetra
0 .305 .736 r
.60448 .53453 .61147 .53973 .6259 .54766 .61895 .5424 Metetra
0 .384 .801 r
.61895 .5424 .6259 .54766 .64049 .55451 .63357 .54919 Metetra
0 .457 .856 r
.63357 .54919 .64049 .55451 .65521 .56036 .64834 .55494 Metetra
.072 .52 .899 r
.64834 .55494 .65521 .56036 .67007 .56526 .66324 .55974 Metetra
.155 .573 .932 r
.66324 .55974 .67007 .56526 .68505 .56927 .67827 .56362 Metetra
.229 .616 .954 r
.67827 .56362 .68505 .56927 .70015 .57245 .69342 .56666 Metetra
.294 .651 .969 r
.69342 .56666 .70015 .57245 .71536 .57485 .70868 .56891 Metetra
.349 .679 .978 r
.70868 .56891 .71536 .57485 .73067 .57652 .72404 .57043 Metetra
.397 .7 .982 r
.72404 .57043 .73067 .57652 .74609 .57753 .73951 .57127 Metetra
.437 .717 .984 r
.73951 .57127 .74609 .57753 .76159 .57792 .75507 .57149 Metetra
.471 .729 .982 r
.75507 .57149 .76159 .57792 .7772 .57775 .77072 .57114 Metetra
.5 .739 .98 r
.77072 .57114 .7772 .57775 .79288 .57705 .78646 .57028 Metetra
.524 .746 .976 r
.78646 .57028 .79288 .57705 .80866 .57588 .80229 .56893 Metetra
.544 .751 .972 r
.80229 .56893 .80866 .57588 .82452 .57428 .81821 .56715 Metetra
.562 .755 .968 r
.81821 .56715 .82452 .57428 .84047 .57228 .83421 .56499 Metetra
.576 .758 .964 r
.83421 .56499 .84047 .57228 .85649 .56993 .8503 .56246 Metetra
.589 .759 .96 r
.8503 .56246 .85649 .56993 .87261 .56724 .86647 .55962 Metetra
.599 .76 .956 r
.86647 .55962 .87261 .56724 .8888 .56427 .88272 .55648 Metetra
.608 .761 .952 r
.88272 .55648 .8888 .56427 .90508 .56103 .89906 .55309 Metetra
.402 .577 .909 r
.32025 .76216 .32853 .77252 .34143 .77555 .33317 .76499 Metetra
.37 .556 .905 r
.33317 .76499 .34143 .77555 .35442 .77925 .34618 .76851 Metetra
.336 .535 .9 r
.34618 .76851 .35442 .77925 .36751 .78361 .35929 .77273 Metetra
.301 .513 .895 r
.35929 .77273 .36751 .78361 .38071 .78863 .3725 .77763 Metetra
.265 .491 .888 r
.3725 .77763 .38071 .78863 .39403 .7943 .38583 .78321 Metetra
.229 .469 .881 r
.38583 .78321 .39403 .7943 .40749 .80058 .39928 .78945 Metetra
.193 .447 .873 r
.39928 .78945 .40749 .80058 .42108 .80747 .41288 .79631 Metetra
.157 .426 .864 r
.41288 .79631 .42108 .80747 .43482 .81491 .42663 .80378 Metetra
.123 .406 .855 r
.42663 .80378 .43482 .81491 .44873 .82287 .44054 .8118 Metetra
.09 .387 .846 r
.44054 .8118 .44873 .82287 .46281 .83131 .45462 .82033 Metetra
.058 .37 .838 r
.45462 .82033 .46281 .83131 .47707 .84018 .46888 .82932 Metetra
.029 .354 .829 r
.46888 .82932 .47707 .84018 .49152 .84941 .48333 .83871 Metetra
.002 .341 .821 r
.48333 .83871 .49152 .84941 .50616 .85897 .49798 .84846 Metetra
0 .33 .814 r
.49798 .84846 .50616 .85897 .52101 .86878 .51283 .85849 Metetra
.956 .697 .503 r
.51283 .85849 .52101 .86878 .53491 .47702 .52771 .47171 Metetra
.641 .111 0 r
.52771 .47171 .53491 .47702 .54841 .4911 .54123 .48588 Metetra
.577 .037 0 r
.54123 .48588 .54841 .4911 .56213 .50385 .55497 .49869 Metetra
.501 0 0 r
.55497 .49869 .56213 .50385 .57606 .5153 .56892 .51017 Metetra
.415 0 0 r
.56892 .51017 .57606 .5153 .59018 .52551 .58307 .52038 Metetra
0 .217 .655 r
.58307 .52038 .59018 .52551 .60448 .53453 .59741 .52937 Metetra
0 .304 .732 r
.59741 .52937 .60448 .53453 .61895 .5424 .61191 .53719 Metetra
0 .386 .799 r
.61191 .53719 .61895 .5424 .63357 .54919 .62658 .5439 Metetra
0 .46 .856 r
.62658 .5439 .63357 .54919 .64834 .55494 .64139 .54956 Metetra
.074 .524 .9 r
.64139 .54956 .64834 .55494 .66324 .55974 .65634 .55423 Metetra
.159 .578 .933 r
.65634 .55423 .66324 .55974 .67827 .56362 .67141 .55798 Metetra
.234 .622 .956 r
.67141 .55798 .67827 .56362 .69342 .56666 .68661 .56087 Metetra
.3 .657 .971 r
.68661 .56087 .69342 .56666 .70868 .56891 .70192 .56296 Metetra
.356 .685 .98 r
.70192 .56296 .70868 .56891 .72404 .57043 .71733 .56431 Metetra
.403 .706 .984 r
.71733 .56431 .72404 .57043 .73951 .57127 .73285 .56498 Metetra
.444 .722 .984 r
.73285 .56498 .73951 .57127 .75507 .57149 .74846 .56502 Metetra
.477 .734 .983 r
.74846 .56502 .75507 .57149 .77072 .57114 .76417 .56449 Metetra
.506 .743 .98 r
.76417 .56449 .77072 .57114 .78646 .57028 .77997 .56344 Metetra
.53 .749 .976 r
.77997 .56344 .78646 .57028 .80229 .56893 .79585 .56191 Metetra
.55 .754 .972 r
.79585 .56191 .80229 .56893 .81821 .56715 .81183 .55995 Metetra
.567 .757 .968 r
.81183 .55995 .81821 .56715 .83421 .56499 .82788 .5576 Metetra
.581 .76 .963 r
.82788 .5576 .83421 .56499 .8503 .56246 .84403 .5549 Metetra
.593 .761 .959 r
.84403 .5549 .8503 .56246 .86647 .55962 .86025 .55189 Metetra
.603 .762 .955 r
.86025 .55189 .86647 .55962 .88272 .55648 .87657 .54859 Metetra
.612 .762 .951 r
.87657 .54859 .88272 .55648 .89906 .55309 .89296 .54503 Metetra
.413 .581 .909 r
.31191 .75169 .32025 .76216 .33317 .76499 .32485 .75429 Metetra
.38 .56 .905 r
.32485 .75429 .33317 .76499 .34618 .76851 .33788 .75762 Metetra
.346 .538 .9 r
.33788 .75762 .34618 .76851 .35929 .77273 .351 .76167 Metetra
.31 .516 .894 r
.351 .76167 .35929 .77273 .3725 .77763 .36422 .76643 Metetra
.273 .493 .887 r
.36422 .76643 .3725 .77763 .38583 .78321 .37756 .77191 Metetra
.235 .47 .88 r
.37756 .77191 .38583 .78321 .39928 .78945 .39102 .77808 Metetra
.198 .447 .872 r
.39102 .77808 .39928 .78945 .41288 .79631 .40462 .78493 Metetra
.161 .425 .863 r
.40462 .78493 .41288 .79631 .42663 .80378 .41837 .7924 Metetra
.125 .403 .853 r
.41837 .7924 .42663 .80378 .44054 .8118 .43228 .80047 Metetra
.09 .383 .844 r
.43228 .80047 .44054 .8118 .45462 .82033 .44636 .80909 Metetra
.056 .364 .834 r
.44636 .80909 .45462 .82033 .46888 .82932 .46062 .81821 Metetra
.025 .348 .825 r
.46062 .81821 .46888 .82932 .48333 .83871 .47507 .82777 Metetra
0 .333 .816 r
.47507 .82777 .48333 .83871 .49798 .84846 .48972 .8377 Metetra
0 .32 .808 r
.48972 .8377 .49798 .84846 .51283 .85849 .50458 .84796 Metetra
.956 .697 .504 r
.50458 .84796 .51283 .85849 .52771 .47171 .52043 .46646 Metetra
.649 .118 0 r
.52043 .46646 .52771 .47171 .54123 .48588 .53396 .48073 Metetra
.585 .043 0 r
.53396 .48073 .54123 .48588 .55497 .49869 .54772 .4936 Metetra
.509 0 0 r
.54772 .4936 .55497 .49869 .56892 .51017 .5617 .50511 Metetra
.423 0 0 r
.5617 .50511 .56892 .51017 .58307 .52038 .57588 .51532 Metetra
0 .214 .648 r
.57588 .51532 .58307 .52038 .59741 .52937 .59025 .52427 Metetra
0 .303 .727 r
.59025 .52427 .59741 .52937 .61191 .53719 .6048 .53203 Metetra
0 .387 .797 r
.6048 .53203 .61191 .53719 .62658 .5439 .6195 .53865 Metetra
0 .463 .856 r
.6195 .53865 .62658 .5439 .64139 .54956 .63436 .5442 Metetra
.076 .529 .901 r
.63436 .5442 .64139 .54956 .65634 .55423 .64935 .54875 Metetra
.163 .584 .935 r
.64935 .54875 .65634 .55423 .67141 .55798 .66448 .55235 Metetra
.24 .628 .958 r
.66448 .55235 .67141 .55798 .68661 .56087 .67972 .55508 Metetra
.306 .663 .973 r
.67972 .55508 .68661 .56087 .70192 .56296 .69508 .557 Metetra
.363 .69 .981 r
.69508 .557 .70192 .56296 .71733 .56431 .71055 .55817 Metetra
.411 .711 .985 r
.71055 .55817 .71733 .56431 .73285 .56498 .72612 .55865 Metetra
.451 .727 .985 r
.72612 .55865 .73285 .56498 .74846 .56502 .74178 .5585 Metetra
.484 .738 .983 r
.74178 .5585 .74846 .56502 .76417 .56449 .75754 .55778 Metetra
.513 .747 .98 r
.75754 .55778 .76417 .56449 .77997 .56344 .7734 .55653 Metetra
.536 .753 .976 r
.7734 .55653 .77997 .56344 .79585 .56191 .78934 .55481 Metetra
.556 .757 .971 r
.78934 .55481 .79585 .56191 .81183 .55995 .80537 .55266 Metetra
.572 .76 .967 r
.80537 .55266 .81183 .55995 .82788 .5576 .82148 .55013 Metetra
.586 .762 .962 r
.82148 .55013 .82788 .5576 .84403 .5549 .83768 .54725 Metetra
.598 .763 .958 r
.83768 .54725 .84403 .5549 .86025 .55189 .85397 .54406 Metetra
.607 .763 .954 r
.85397 .54406 .86025 .55189 .87657 .54859 .87034 .54058 Metetra
.616 .763 .95 r
.87034 .54058 .87657 .54859 .89296 .54503 .8868 .53686 Metetra
.425 .586 .908 r
.30351 .74108 .31191 .75169 .32485 .75429 .31648 .74345 Metetra
.391 .564 .904 r
.31648 .74345 .32485 .75429 .33788 .75762 .32952 .74656 Metetra
.356 .542 .899 r
.32952 .74656 .33788 .75762 .351 .76167 .34266 .75042 Metetra
.32 .519 .893 r
.34266 .75042 .351 .76167 .36422 .76643 .35589 .75503 Metetra
.282 .495 .887 r
.35589 .75503 .36422 .76643 .37756 .77191 .36924 .76039 Metetra
.243 .471 .879 r
.36924 .76039 .37756 .77191 .39102 .77808 .3827 .76649 Metetra
.204 .447 .87 r
.3827 .76649 .39102 .77808 .40462 .78493 .39631 .77329 Metetra
.165 .424 .861 r
.39631 .77329 .40462 .78493 .41837 .7924 .41005 .78077 Metetra
.127 .401 .851 r
.41005 .78077 .41837 .7924 .43228 .80047 .42396 .78888 Metetra
.09 .379 .841 r
.42396 .78888 .43228 .80047 .44636 .80909 .43804 .79759 Metetra
.055 .359 .831 r
.43804 .79759 .44636 .80909 .46062 .81821 .45229 .80683 Metetra
.021 .341 .821 r
.45229 .80683 .46062 .81821 .47507 .82777 .46674 .81655 Metetra
0 .325 .811 r
.46674 .81655 .47507 .82777 .48972 .8377 .48139 .82669 Metetra
0 .311 .802 r
.48139 .82669 .48972 .8377 .50458 .84796 .49625 .83718 Metetra
.955 .697 .504 r
.49625 .83718 .50458 .84796 .52043 .46646 .51305 .46128 Metetra
.657 .125 0 r
.51305 .46128 .52043 .46646 .53396 .48073 .5266 .47565 Metetra
.593 .05 0 r
.5266 .47565 .53396 .48073 .54772 .4936 .54038 .48858 Metetra
.518 0 0 r
.54038 .48858 .54772 .4936 .5617 .50511 .55439 .50012 Metetra
.431 0 0 r
.55439 .50012 .5617 .50511 .57588 .51532 .56861 .51032 Metetra
0 .212 .641 r
.56861 .51032 .57588 .51532 .59025 .52427 .58301 .51924 Metetra
0 .302 .723 r
.58301 .51924 .59025 .52427 .6048 .53203 .59759 .52693 Metetra
0 .389 .795 r
.59759 .52693 .6048 .53203 .6195 .53865 .61234 .53345 Metetra
0 .467 .856 r
.61234 .53345 .6195 .53865 .63436 .5442 .62724 .53889 Metetra
.079 .534 .903 r
.62724 .53889 .63436 .5442 .64935 .54875 .64228 .54329 Metetra
.168 .59 .937 r
.64228 .54329 .64935 .54875 .66448 .55235 .65746 .54674 Metetra
.246 .635 .96 r
.65746 .54674 .66448 .55235 .67972 .55508 .67275 .5493 Metetra
.313 .67 .975 r
.67275 .5493 .67972 .55508 .69508 .557 .68816 .55103 Metetra
.37 .697 .983 r
.68816 .55103 .69508 .557 .71055 .55817 .70368 .55201 Metetra
.418 .717 .986 r
.70368 .55201 .71055 .55817 .72612 .55865 .7193 .55229 Metetra
.458 .732 .986 r
.7193 .55229 .72612 .55865 .74178 .5585 .73502 .55194 Metetra
.492 .743 .984 r
.73502 .55194 .74178 .5585 .75754 .55778 .75084 .55101 Metetra
.52 .751 .98 r
.75084 .55101 .75754 .55778 .7734 .55653 .76675 .54956 Metetra
.543 .756 .976 r
.76675 .54956 .7734 .55653 .78934 .55481 .78275 .54764 Metetra
.562 .76 .971 r
.78275 .54764 .78934 .55481 .80537 .55266 .79883 .54529 Metetra
.578 .762 .966 r
.79883 .54529 .80537 .55266 .82148 .55013 .815 .54256 Metetra
.591 .764 .961 r
.815 .54256 .82148 .55013 .83768 .54725 .83126 .53949 Metetra
.602 .764 .957 r
.83126 .53949 .83768 .54725 .85397 .54406 .84761 .53611 Metetra
.612 .765 .953 r
.84761 .53611 .85397 .54406 .87034 .54058 .86404 .53246 Metetra
.62 .764 .949 r
.86404 .53246 .87034 .54058 .8868 .53686 .88056 .52857 Metetra
.437 .591 .907 r
.29504 .73037 .30351 .74108 .31648 .74345 .30803 .73247 Metetra
.403 .569 .903 r
.30803 .73247 .31648 .74345 .32952 .74656 .3211 .73534 Metetra
.368 .546 .898 r
.3211 .73534 .32952 .74656 .34266 .75042 .33426 .73899 Metetra
.33 .522 .893 r
.33426 .73899 .34266 .75042 .35589 .75503 .3475 .74343 Metetra
.292 .498 .886 r
.3475 .74343 .35589 .75503 .36924 .76039 .36086 .74865 Metetra
.252 .473 .878 r
.36086 .74865 .36924 .76039 .3827 .76649 .37433 .75464 Metetra
.211 .448 .869 r
.37433 .75464 .3827 .76649 .39631 .77329 .38793 .76139 Metetra
.171 .423 .86 r
.38793 .76139 .39631 .77329 .41005 .78077 .40168 .76886 Metetra
.131 .399 .849 r
.40168 .76886 .41005 .78077 .42396 .78888 .41558 .77701 Metetra
.092 .376 .839 r
.41558 .77701 .42396 .78888 .43804 .79759 .42965 .7858 Metetra
.054 .354 .828 r
.42965 .7858 .43804 .79759 .45229 .80683 .44391 .79517 Metetra
.018 .334 .817 r
.44391 .79517 .45229 .80683 .46674 .81655 .45835 .80506 Metetra
0 .317 .806 r
.45835 .80506 .46674 .81655 .48139 .82669 .473 .8154 Metetra
0 .301 .796 r
.473 .8154 .48139 .82669 .49625 .83718 .48786 .82613 Metetra
.955 .697 .505 r
.48786 .82613 .49625 .83718 .51305 .46128 .50559 .45619 Metetra
.664 .132 0 r
.50559 .45619 .51305 .46128 .5266 .47565 .51915 .47065 Metetra
.601 .058 0 r
.51915 .47065 .5266 .47565 .54038 .48858 .53296 .48365 Metetra
.526 0 0 r
.53296 .48365 .54038 .48858 .55439 .50012 .54699 .49522 Metetra
.439 0 0 r
.54699 .49522 .55439 .50012 .56861 .51032 .56124 .50541 Metetra
0 .209 .634 r
.56124 .50541 .56861 .51032 .58301 .51924 .57568 .51428 Metetra
0 .302 .718 r
.57568 .51428 .58301 .51924 .59759 .52693 .5903 .5219 Metetra
0 .391 .793 r
.5903 .5219 .59759 .52693 .61234 .53345 .60509 .52832 Metetra
0 .471 .856 r
.60509 .52832 .61234 .53345 .62724 .53889 .62004 .53362 Metetra
.082 .54 .904 r
.62004 .53362 .62724 .53889 .64228 .54329 .63513 .53787 Metetra
.173 .597 .939 r
.63513 .53787 .64228 .54329 .65746 .54674 .65035 .54115 Metetra
.253 .642 .963 r
.65035 .54115 .65746 .54674 .67275 .5493 .6657 .54352 Metetra
.321 .677 .977 r
.6657 .54352 .67275 .5493 .68816 .55103 .68116 .54506 Metetra
.379 .704 .985 r
.68116 .54506 .68816 .55103 .70368 .55201 .69673 .54583 Metetra
.427 .723 .987 r
.69673 .54583 .70368 .55201 .7193 .55229 .71241 .54589 Metetra
.467 .737 .987 r
.71241 .54589 .7193 .55229 .73502 .55194 .72819 .54532 Metetra
.5 .748 .984 r
.72819 .54532 .73502 .55194 .75084 .55101 .74406 .54418 Metetra
.527 .755 .98 r
.74406 .54418 .75084 .55101 .76675 .54956 .76002 .54251 Metetra
.55 .76 .975 r
.76002 .54251 .76675 .54956 .78275 .54764 .77607 .54037 Metetra
.568 .763 .97 r
.77607 .54037 .78275 .54764 .79883 .54529 .79222 .53782 Metetra
.584 .765 .965 r
.79222 .53782 .79883 .54529 .815 .54256 .80845 .53488 Metetra
.597 .766 .96 r
.80845 .53488 .815 .54256 .83126 .53949 .82476 .53161 Metetra
.607 .766 .956 r
.82476 .53161 .83126 .53949 .84761 .53611 .84117 .52805 Metetra
.616 .766 .951 r
.84117 .52805 .84761 .53611 .86404 .53246 .85766 .52421 Metetra
.624 .765 .947 r
.85766 .52421 .86404 .53246 .88056 .52857 .87424 .52015 Metetra
.449 .596 .906 r
.2865 .71953 .29504 .73037 .30803 .73247 .29953 .72135 Metetra
.416 .574 .902 r
.29953 .72135 .30803 .73247 .3211 .73534 .31262 .72395 Metetra
.38 .551 .898 r
.31262 .72395 .3211 .73534 .33426 .73899 .32579 .72737 Metetra
.342 .526 .892 r
.32579 .72737 .33426 .73899 .3475 .74343 .33906 .73161 Metetra
.302 .501 .885 r
.33906 .73161 .3475 .74343 .36086 .74865 .35242 .73667 Metetra
.261 .475 .877 r
.35242 .73667 .36086 .74865 .37433 .75464 .3659 .74255 Metetra
.219 .449 .868 r
.3659 .74255 .37433 .75464 .38793 .76139 .3795 .74923 Metetra
.177 .423 .858 r
.3795 .74923 .38793 .76139 .40168 .76886 .39324 .75667 Metetra
.135 .397 .848 r
.39324 .75667 .40168 .76886 .41558 .77701 .40714 .76485 Metetra
.094 .373 .836 r
.40714 .76485 .41558 .77701 .42965 .7858 .42121 .77371 Metetra
.054 .349 .824 r
.42121 .77371 .42965 .7858 .44391 .79517 .43545 .78321 Metetra
.016 .328 .813 r
.43545 .78321 .44391 .79517 .45835 .80506 .44989 .79327 Metetra
0 .308 .801 r
.44989 .79327 .45835 .80506 .473 .8154 .46453 .80382 Metetra
0 .291 .79 r
.46453 .80382 .473 .8154 .48786 .82613 .47939 .81481 Metetra
.955 .697 .505 r
.47939 .81481 .48786 .82613 .50559 .45619 .49804 .45118 Metetra
.672 .14 0 r
.49804 .45118 .50559 .45619 .51915 .47065 .51162 .46576 Metetra
.61 .065 0 r
.51162 .46576 .51915 .47065 .53296 .48365 .52544 .47882 Metetra
.535 0 0 r
.52544 .47882 .53296 .48365 .54699 .49522 .5395 .49042 Metetra
.447 0 0 r
.5395 .49042 .54699 .49522 .56124 .50541 .55378 .5006 Metetra
0 .206 .626 r
.55378 .5006 .56124 .50541 .57568 .51428 .56826 .50942 Metetra
0 .302 .713 r
.56826 .50942 .57568 .51428 .5903 .5219 .58292 .51694 Metetra
0 .393 .791 r
.58292 .51694 .5903 .5219 .60509 .52832 .59776 .52325 Metetra
0 .476 .856 r
.59776 .52325 .60509 .52832 .62004 .53362 .61275 .52841 Metetra
.086 .546 .906 r
.61275 .52841 .62004 .53362 .63513 .53787 .62789 .5325 Metetra
.179 .604 .941 r
.62789 .5325 .63513 .53787 .65035 .54115 .64316 .53558 Metetra
.261 .65 .965 r
.64316 .53558 .65035 .54115 .6657 .54352 .65856 .53775 Metetra
.33 .685 .979 r
.65856 .53775 .6657 .54352 .68116 .54506 .67408 .53907 Metetra
.388 .711 .986 r
.67408 .53907 .68116 .54506 .69673 .54583 .6897 .53962 Metetra
.436 .73 .988 r
.6897 .53962 .69673 .54583 .71241 .54589 .70543 .53946 Metetra
.476 .743 .987 r
.70543 .53946 .71241 .54589 .72819 .54532 .72126 .53866 Metetra
.508 .753 .984 r
.72126 .53866 .72819 .54532 .74406 .54418 .73719 .53728 Metetra
.535 .759 .979 r
.73719 .53728 .74406 .54418 .76002 .54251 .75321 .53538 Metetra
.557 .763 .974 r
.75321 .53538 .76002 .54251 .77607 .54037 .76932 .53302 Metetra
.575 .766 .969 r
.76932 .53302 .77607 .54037 .79222 .53782 .78552 .53024 Metetra
.59 .767 .964 r
.78552 .53024 .79222 .53782 .80845 .53488 .80181 .5271 Metetra
.602 .768 .959 r
.80181 .5271 .80845 .53488 .82476 .53161 .81818 .52362 Metetra
.612 .767 .954 r
.81818 .52362 .82476 .53161 .84117 .52805 .83465 .51986 Metetra
.621 .767 .95 r
.83465 .51986 .84117 .52805 .85766 .52421 .8512 .51584 Metetra
.628 .766 .946 r
.8512 .51584 .85766 .52421 .87424 .52015 .86784 .51159 Metetra
.463 .602 .905 r
.2779 .7086 .2865 .71953 .29953 .72135 .29096 .71009 Metetra
.429 .58 .901 r
.29096 .71009 .29953 .72135 .31262 .72395 .30408 .7124 Metetra
.393 .556 .897 r
.30408 .7124 .31262 .72395 .32579 .72737 .31727 .71556 Metetra
.355 .531 .891 r
.31727 .71556 .32579 .72737 .33906 .73161 .33055 .71957 Metetra
.314 .505 .884 r
.33055 .71957 .33906 .73161 .35242 .73667 .34393 .72445 Metetra
.272 .478 .876 r
.34393 .72445 .35242 .73667 .3659 .74255 .35741 .73019 Metetra
.229 .45 .867 r
.35741 .73019 .3659 .74255 .3795 .74923 .37101 .73677 Metetra
.184 .423 .857 r
.37101 .73677 .3795 .74923 .39324 .75667 .38475 .74418 Metetra
.14 .396 .846 r
.38475 .74418 .39324 .75667 .40714 .76485 .39865 .75238 Metetra
.097 .37 .834 r
.39865 .75238 .40714 .76485 .42121 .77371 .4127 .76131 Metetra
.054 .345 .821 r
.4127 .76131 .42121 .77371 .43545 .78321 .42694 .77092 Metetra
.014 .321 .808 r
.42694 .77092 .43545 .78321 .44989 .79327 .44137 .78115 Metetra
0 .3 .796 r
.44137 .78115 .44989 .79327 .46453 .80382 .456 .79193 Metetra
0 .281 .783 r
.456 .79193 .46453 .80382 .47939 .81481 .47085 .80318 Metetra
.955 .697 .506 r
.47085 .80318 .47939 .81481 .49804 .45118 .4904 .44629 Metetra
.681 .148 0 r
.4904 .44629 .49804 .45118 .51162 .46576 .50398 .46098 Metetra
.619 .073 0 r
.50398 .46098 .51162 .46576 .52544 .47882 .51783 .47412 Metetra
.544 0 0 r
.51783 .47412 .52544 .47882 .5395 .49042 .53192 .48574 Metetra
.456 0 0 r
.53192 .48574 .5395 .49042 .55378 .5006 .54623 .49589 Metetra
0 .203 .617 r
.54623 .49589 .55378 .5006 .56826 .50942 .56075 .50465 Metetra
0 .302 .708 r
.56075 .50465 .56826 .50942 .58292 .51694 .57545 .51209 Metetra
0 .396 .789 r
.57545 .51209 .58292 .51694 .59776 .52325 .59033 .51826 Metetra
0 .481 .856 r
.59033 .51826 .59776 .52325 .61275 .52841 .60537 .52326 Metetra
.091 .554 .907 r
.60537 .52326 .61275 .52841 .62789 .5325 .62056 .52716 Metetra
.186 .613 .944 r
.62056 .52716 .62789 .5325 .64316 .53558 .63588 .53005 Metetra
.269 .658 .968 r
.63588 .53005 .64316 .53558 .65856 .53775 .65134 .53199 Metetra
.34 .693 .982 r
.65134 .53199 .65856 .53775 .67408 .53907 .66691 .53308 Metetra
.398 .718 .988 r
.66691 .53308 .67408 .53907 .6897 .53962 .68259 .53339 Metetra
.446 .736 .989 r
.68259 .53339 .6897 .53962 .70543 .53946 .69837 .53298 Metetra
.485 .749 .987 r
.69837 .53298 .70543 .53946 .72126 .53866 .71426 .53193 Metetra
.517 .758 .984 r
.71426 .53193 .72126 .53866 .73719 .53728 .73024 .53031 Metetra
.543 .763 .979 r
.73024 .53031 .73719 .53728 .75321 .53538 .74632 .52817 Metetra
.565 .767 .973 r
.74632 .52817 .75321 .53538 .76932 .53302 .76249 .52557 Metetra
.582 .769 .968 r
.76249 .52557 .76932 .53302 .78552 .53024 .77874 .52256 Metetra
.596 .769 .962 r
.77874 .52256 .78552 .53024 .80181 .5271 .79509 .51919 Metetra
.608 .769 .957 r
.79509 .51919 .80181 .5271 .81818 .52362 .81153 .5155 Metetra
.618 .769 .952 r
.81153 .5155 .81818 .52362 .83465 .51986 .82805 .51153 Metetra
.625 .768 .948 r
.82805 .51153 .83465 .51986 .8512 .51584 .84466 .50732 Metetra
.632 .767 .944 r
.84466 .50732 .8512 .51584 .86784 .51159 .86137 .50289 Metetra
.477 .608 .903 r
.26923 .69756 .2779 .7086 .29096 .71009 .28232 .69871 Metetra
.443 .586 .9 r
.28232 .69871 .29096 .71009 .30408 .7124 .29548 .7007 Metetra
.407 .562 .896 r
.29548 .7007 .30408 .7124 .31727 .71556 .3087 .70356 Metetra
.368 .536 .89 r
.3087 .70356 .31727 .71556 .33055 .71957 .32199 .70731 Metetra
.327 .509 .883 r
.32199 .70731 .33055 .71957 .34393 .72445 .33538 .71197 Metetra
.284 .481 .875 r
.33538 .71197 .34393 .72445 .35741 .73019 .34887 .71755 Metetra
.239 .453 .866 r
.34887 .71755 .35741 .73019 .37101 .73677 .36247 .72402 Metetra
.193 .424 .856 r
.36247 .72402 .37101 .73677 .38475 .74418 .37621 .73137 Metetra
.147 .395 .844 r
.37621 .73137 .38475 .74418 .39865 .75238 .3901 .73957 Metetra
.101 .367 .831 r
.3901 .73957 .39865 .75238 .4127 .76131 .40414 .74856 Metetra
.056 .34 .818 r
.40414 .74856 .4127 .76131 .42694 .77092 .41837 .75829 Metetra
.012 .315 .804 r
.41837 .75829 .42694 .77092 .44137 .78115 .43278 .76869 Metetra
0 .292 .79 r
.43278 .76869 .44137 .78115 .456 .79193 .44741 .7797 Metetra
0 .271 .777 r
.44741 .7797 .456 .79193 .47085 .80318 .46224 .79122 Metetra
.955 .697 .507 r
.46224 .79122 .47085 .80318 .4904 .44629 .48265 .44152 Metetra
.689 .157 0 r
.48265 .44152 .4904 .44629 .50398 .46098 .49625 .45634 Metetra
.628 .081 0 r
.49625 .45634 .50398 .46098 .51783 .47412 .51012 .46954 Metetra
.553 0 0 r
.51012 .46954 .51783 .47412 .53192 .48574 .52424 .48119 Metetra
.464 0 0 r
.52424 .48119 .53192 .48574 .54623 .49589 .53858 .49132 Metetra
0 .2 .608 r
.53858 .49132 .54623 .49589 .56075 .50465 .55314 .50001 Metetra
0 .302 .702 r
.55314 .50001 .56075 .50465 .57545 .51209 .56789 .50733 Metetra
0 .4 .786 r
.56789 .50733 .57545 .51209 .59033 .51826 .58281 .51336 Metetra
0 .487 .856 r
.58281 .51336 .59033 .51826 .60537 .52326 .5979 .51818 Metetra
.097 .562 .909 r
.5979 .51818 .60537 .52326 .62056 .52716 .61314 .52188 Metetra
.195 .622 .947 r
.61314 .52188 .62056 .52716 .63588 .53005 .62852 .52454 Metetra
.279 .668 .971 r
.62852 .52454 .63588 .53005 .65134 .53199 .64402 .52625 Metetra
.35 .702 .984 r
.64402 .52625 .65134 .53199 .66691 .53308 .65965 .52708 Metetra
.409 .726 .99 r
.65965 .52708 .66691 .53308 .68259 .53339 .67538 .52713 Metetra
.457 .743 .99 r
.67538 .52713 .68259 .53339 .69837 .53298 .69123 .52646 Metetra
.495 .755 .988 r
.69123 .52646 .69837 .53298 .71426 .53193 .70717 .52514 Metetra
.527 .763 .983 r
.70717 .52514 .71426 .53193 .73024 .53031 .72321 .52326 Metetra
.552 .767 .978 r
.72321 .52326 .73024 .53031 .74632 .52817 .73934 .52086 Metetra
.573 .77 .972 r
.73934 .52086 .74632 .52817 .76249 .52557 .75557 .51801 Metetra
.589 .771 .966 r
.75557 .51801 .76249 .52557 .77874 .52256 .77188 .51476 Metetra
.603 .771 .961 r
.77188 .51476 .77874 .52256 .79509 .51919 .78829 .51116 Metetra
.614 .771 .955 r
.78829 .51116 .79509 .51919 .81153 .5155 .80478 .50724 Metetra
.623 .77 .951 r
.80478 .50724 .81153 .5155 .82805 .51153 .82137 .50306 Metetra
.63 .769 .946 r
.82137 .50306 .82805 .51153 .84466 .50732 .83804 .49865 Metetra
.636 .767 .942 r
.83804 .49865 .84466 .50732 .86137 .50289 .85481 .49403 Metetra
.491 .615 .902 r
.26049 .68644 .26923 .69756 .28232 .69871 .27362 .68721 Metetra
.458 .593 .899 r
.27362 .68721 .28232 .69871 .29548 .7007 .28681 .68884 Metetra
.422 .568 .894 r
.28681 .68884 .29548 .7007 .3087 .70356 .30006 .69137 Metetra
.383 .542 .889 r
.30006 .69137 .3087 .70356 .32199 .70731 .31338 .69483 Metetra
.341 .514 .882 r
.31338 .69483 .32199 .70731 .33538 .71197 .32678 .69925 Metetra
.297 .485 .874 r
.32678 .69925 .33538 .71197 .34887 .71755 .34027 .70462 Metetra
.251 .455 .865 r
.34027 .70462 .34887 .71755 .36247 .72402 .35388 .71096 Metetra
.203 .425 .854 r
.35388 .71096 .36247 .72402 .37621 .73137 .36761 .71823 Metetra
.155 .395 .842 r
.36761 .71823 .37621 .73137 .3901 .73957 .38149 .72641 Metetra
.106 .365 .829 r
.38149 .72641 .3901 .73957 .40414 .74856 .39553 .73544 Metetra
.058 .336 .815 r
.39553 .73544 .40414 .74856 .41837 .75829 .40974 .74529 Metetra
.012 .309 .8 r
.40974 .74529 .41837 .75829 .43278 .76869 .42414 .75586 Metetra
0 .284 .785 r
.42414 .75586 .43278 .76869 .44741 .7797 .43874 .7671 Metetra
0 .261 .77 r
.43874 .7671 .44741 .7797 .46224 .79122 .45357 .77892 Metetra
.954 .697 .508 r
.45357 .77892 .46224 .79122 .48265 .44152 .4748 .4369 Metetra
.697 .165 0 r
.4748 .4369 .48265 .44152 .49625 .45634 .48842 .45185 Metetra
.637 .089 0 r
.48842 .45185 .49625 .45634 .51012 .46954 .50231 .46513 Metetra
.563 .003 0 r
.50231 .46513 .51012 .46954 .52424 .48119 .51645 .47679 Metetra
.473 0 0 r
.51645 .47679 .52424 .48119 .53858 .49132 .53083 .48689 Metetra
.369 0 0 r
.53083 .48689 .53858 .49132 .55314 .50001 .54543 .4955 Metetra
0 .303 .696 r
.54543 .4955 .55314 .50001 .56789 .50733 .56022 .5027 Metetra
0 .404 .784 r
.56022 .5027 .56789 .50733 .58281 .51336 .5752 .50856 Metetra
0 .495 .857 r
.5752 .50856 .58281 .51336 .5979 .51818 .59033 .51319 Metetra
.104 .571 .912 r
.59033 .51319 .5979 .51818 .61314 .52188 .60563 .51666 Metetra
.205 .632 .95 r
.60563 .51666 .61314 .52188 .62852 .52454 .62106 .51907 Metetra
.291 .678 .974 r
.62106 .51907 .62852 .52454 .64402 .52625 .63662 .52052 Metetra
.363 .711 .986 r
.63662 .52052 .64402 .52625 .65965 .52708 .6523 .52108 Metetra
.421 .735 .991 r
.6523 .52108 .65965 .52708 .67538 .52713 .66809 .52084 Metetra
.468 .751 .991 r
.66809 .52084 .67538 .52713 .69123 .52646 .68399 .51988 Metetra
.507 .761 .988 r
.68399 .51988 .69123 .52646 .70717 .52514 .69999 .51828 Metetra
.537 .768 .983 r
.69999 .51828 .70717 .52514 .72321 .52326 .71609 .51612 Metetra
.562 .771 .977 r
.71609 .51612 .72321 .52326 .73934 .52086 .73228 .51345 Metetra
.581 .773 .971 r
.73228 .51345 .73934 .52086 .75557 .51801 .74856 .51033 Metetra
.597 .774 .964 r
.74856 .51033 .75557 .51801 .77188 .51476 .76493 .50683 Metetra
.61 .773 .959 r
.76493 .50683 .77188 .51476 .78829 .51116 .7814 .50298 Metetra
.62 .772 .953 r
.7814 .50298 .78829 .51116 .80478 .50724 .79795 .49884 Metetra
.628 .771 .949 r
.79795 .49884 .80478 .50724 .82137 .50306 .8146 .49444 Metetra
.635 .769 .944 r
.8146 .49444 .82137 .50306 .83804 .49865 .83134 .48982 Metetra
.64 .768 .94 r
.83134 .48982 .83804 .49865 .85481 .49403 .84817 .48501 Metetra
.507 .622 .9 r
.25168 .67524 .26049 .68644 .27362 .68721 .26486 .6756 Metetra
.474 .6 .897 r
.26486 .6756 .27362 .68721 .28681 .68884 .27808 .67683 Metetra
.438 .575 .893 r
.27808 .67683 .28681 .68884 .30006 .69137 .29136 .67899 Metetra
.399 .548 .888 r
.29136 .67899 .30006 .69137 .31338 .69483 .3047 .68212 Metetra
.357 .52 .881 r
.3047 .68212 .31338 .69483 .32678 .69925 .31812 .68625 Metetra
.311 .49 .873 r
.31812 .68625 .32678 .69925 .34027 .70462 .33163 .6914 Metetra
.264 .459 .864 r
.33163 .6914 .34027 .70462 .35388 .71096 .34524 .69756 Metetra
.214 .427 .853 r
.34524 .69756 .35388 .71096 .36761 .71823 .35897 .70473 Metetra
.164 .395 .84 r
.35897 .70473 .36761 .71823 .38149 .72641 .37283 .71287 Metetra
.113 .363 .826 r
.37283 .71287 .38149 .72641 .39553 .73544 .38686 .72194 Metetra
.062 .333 .811 r
.38686 .72194 .39553 .73544 .40974 .74529 .40105 .73188 Metetra
.013 .303 .796 r
.40105 .73188 .40974 .74529 .42414 .75586 .41543 .74264 Metetra
0 .275 .779 r
.41543 .74264 .42414 .75586 .43874 .7671 .43002 .75412 Metetra
0 .25 .763 r
.43002 .75412 .43874 .7671 .45357 .77892 .44482 .76623 Metetra
.954 .697 .509 r
.44482 .76623 .45357 .77892 .4748 .4369 .46685 .43245 Metetra
.705 .174 0 r
.46685 .43245 .4748 .4369 .48842 .45185 .48048 .44754 Metetra
.646 .098 0 r
.48048 .44754 .48842 .45185 .50231 .46513 .49439 .4609 Metetra
.572 .01 0 r
.49439 .4609 .50231 .46513 .51645 .47679 .50856 .47257 Metetra
.482 0 0 r
.50856 .47257 .51645 .47679 .53083 .48689 .52298 .48263 Metetra
.376 0 0 r
.52298 .48263 .53083 .48689 .54543 .4955 .53762 .49115 Metetra
0 .304 .689 r
.53762 .49115 .54543 .4955 .56022 .5027 .55246 .4982 Metetra
0 .409 .781 r
.55246 .4982 .56022 .5027 .5752 .50856 .56748 .50388 Metetra
0 .503 .858 r
.56748 .50388 .5752 .50856 .59033 .51319 .58267 .50828 Metetra
.112 .582 .915 r
.58267 .50828 .59033 .51319 .60563 .51666 .59802 .5115 Metetra
.216 .643 .953 r
.59802 .5115 .60563 .51666 .62106 .51907 .6135 .51364 Metetra
.304 .689 .977 r
.6135 .51364 .62106 .51907 .63662 .52052 .62912 .5148 Metetra
.376 .721 .989 r
.62912 .5148 .63662 .52052 .6523 .52108 .64485 .51506 Metetra
.434 .744 .993 r
.64485 .51506 .6523 .52108 .66809 .52084 .6607 .51451 Metetra
.481 .758 .991 r
.6607 .51451 .66809 .52084 .68399 .51988 .67666 .51325 Metetra
.518 .767 .987 r
.67666 .51325 .68399 .51988 .69999 .51828 .69272 .51135 Metetra
.548 .773 .982 r
.69272 .51135 .69999 .51828 .71609 .51612 .70887 .50888 Metetra
.571 .775 .975 r
.70887 .50888 .71609 .51612 .73228 .51345 .72512 .50592 Metetra
.59 .776 .969 r
.72512 .50592 .73228 .51345 .74856 .51033 .74146 .50252 Metetra
.605 .776 .962 r
.74146 .50252 .74856 .51033 .76493 .50683 .7579 .49875 Metetra
.617 .775 .956 r
.7579 .49875 .76493 .50683 .7814 .50298 .77442 .49466 Metetra
.626 .773 .951 r
.77442 .49466 .7814 .50298 .79795 .49884 .79104 .49028 Metetra
.633 .771 .946 r
.79104 .49028 .79795 .49884 .8146 .49444 .80774 .48566 Metetra
.639 .77 .942 r
.80774 .48566 .8146 .49444 .83134 .48982 .82454 .48083 Metetra
.644 .768 .938 r
.82454 .48083 .83134 .48982 .84817 .48501 .84144 .47582 Metetra
.523 .63 .899 r
.24279 .66398 .25168 .67524 .26486 .6756 .25602 .66389 Metetra
.491 .607 .896 r
.25602 .66389 .26486 .6756 .27808 .67683 .26929 .66468 Metetra
.455 .582 .892 r
.26929 .66468 .27808 .67683 .29136 .67899 .28261 .66643 Metetra
.416 .555 .887 r
.28261 .66643 .29136 .67899 .3047 .68212 .29598 .66919 Metetra
.373 .526 .88 r
.29598 .66919 .3047 .68212 .31812 .68625 .30941 .67299 Metetra
.327 .495 .872 r
.30941 .67299 .31812 .68625 .33163 .6914 .32293 .67787 Metetra
.279 .463 .863 r
.32293 .67787 .33163 .6914 .34524 .69756 .33654 .68382 Metetra
.227 .43 .851 r
.33654 .68382 .34524 .69756 .35897 .70473 .35027 .69085 Metetra
.175 .396 .838 r
.35027 .69085 .35897 .70473 .37283 .71287 .36413 .69893 Metetra
.121 .362 .824 r
.36413 .69893 .37283 .71287 .38686 .72194 .37814 .70801 Metetra
.067 .329 .808 r
.37814 .70801 .38686 .72194 .40105 .73188 .39231 .71805 Metetra
.015 .297 .791 r
.39231 .71805 .40105 .73188 .41543 .74264 .40667 .72897 Metetra
0 .267 .774 r
.40667 .72897 .41543 .74264 .43002 .75412 .42123 .7407 Metetra
0 .239 .756 r
.42123 .7407 .43002 .75412 .44482 .76623 .43602 .75313 Metetra
.954 .697 .51 r
.43602 .75313 .44482 .76623 .46685 .43245 .45878 .42821 Metetra
.714 .184 0 r
.45878 .42821 .46685 .43245 .48048 .44754 .47243 .44344 Metetra
.655 .107 0 r
.47243 .44344 .48048 .44754 .49439 .4609 .48636 .45688 Metetra
.582 .018 0 r
.48636 .45688 .49439 .4609 .50856 .47257 .50057 .46856 Metetra
.491 0 0 r
.50057 .46856 .50856 .47257 .52298 .48263 .51502 .47857 Metetra
.383 0 0 r
.51502 .47857 .52298 .48263 .53762 .49115 .5297 .48697 Metetra
0 .306 .682 r
.5297 .48697 .53762 .49115 .55246 .4982 .54459 .49386 Metetra
0 .415 .779 r
.54459 .49386 .55246 .4982 .56748 .50388 .55966 .49933 Metetra
.001 .513 .859 r
.55966 .49933 .56748 .50388 .58267 .50828 .5749 .50348 Metetra
.122 .593 .918 r
.5749 .50348 .58267 .50828 .59802 .5115 .59031 .50642 Metetra
.229 .656 .957 r
.59031 .50642 .59802 .5115 .6135 .51364 .60585 .50826 Metetra
.318 .701 .98 r
.60585 .50826 .6135 .51364 .62912 .5148 .62152 .50909 Metetra
.391 .732 .991 r
.62152 .50909 .62912 .5148 .64485 .51506 .63732 .50902 Metetra
.449 .753 .994 r
.63732 .50902 .64485 .51506 .6607 .51451 .65323 .50814 Metetra
.495 .766 .992 r
.65323 .50814 .6607 .51451 .67666 .51325 .66924 .50654 Metetra
.531 .774 .987 r
.66924 .50654 .67666 .51325 .69272 .51135 .68536 .50431 Metetra
.559 .778 .98 r
.68536 .50431 .69272 .51135 .70887 .50888 .70157 .50153 Metetra
.582 .779 .973 r
.70157 .50153 .70887 .50888 .72512 .50592 .71788 .49826 Metetra
.599 .779 .966 r
.71788 .49826 .72512 .50592 .74146 .50252 .73428 .49457 Metetra
.613 .778 .96 r
.73428 .49457 .74146 .50252 .7579 .49875 .75077 .49052 Metetra
.624 .776 .954 r
.75077 .49052 .7579 .49875 .77442 .49466 .76735 .48616 Metetra
.632 .774 .949 r
.76735 .48616 .77442 .49466 .79104 .49028 .78403 .48154 Metetra
.639 .772 .944 r
.78403 .48154 .79104 .49028 .80774 .48566 .8008 .47669 Metetra
.644 .77 .939 r
.8008 .47669 .80774 .48566 .82454 .48083 .81766 .47165 Metetra
.648 .767 .936 r
.81766 .47165 .82454 .48083 .84144 .47582 .83462 .46644 Metetra
.539 .638 .897 r
.23383 .65269 .24279 .66398 .25602 .66389 .24711 .6521 Metetra
.508 .616 .894 r
.24711 .6521 .25602 .66389 .26929 .66468 .26043 .65242 Metetra
.473 .59 .89 r
.26043 .65242 .26929 .66468 .28261 .66643 .27379 .65371 Metetra
.434 .563 .885 r
.27379 .65371 .28261 .66643 .29598 .66919 .28719 .65604 Metetra
.391 .533 .879 r
.28719 .65604 .29598 .66919 .30941 .67299 .30066 .65946 Metetra
.345 .502 .871 r
.30066 .65946 .30941 .67299 .32293 .67787 .31419 .66402 Metetra
.295 .468 .861 r
.31419 .66402 .32293 .67787 .33654 .68382 .32781 .66972 Metetra
.242 .434 .85 r
.32781 .66972 .33654 .68382 .35027 .69085 .34153 .67657 Metetra
.187 .398 .837 r
.34153 .67657 .35027 .69085 .36413 .69893 .35538 .68455 Metetra
.131 .362 .822 r
.35538 .68455 .36413 .69893 .37814 .70801 .36937 .69363 Metetra
.074 .327 .805 r
.36937 .69363 .37814 .70801 .39231 .71805 .38352 .70375 Metetra
.018 .292 .787 r
.38352 .70375 .39231 .71805 .40667 .72897 .39786 .71484 Metetra
0 .259 .768 r
.39786 .71484 .40667 .72897 .42123 .7407 .41239 .72681 Metetra
0 .229 .749 r
.41239 .72681 .42123 .7407 .43602 .75313 .42715 .73957 Metetra
.953 .697 .511 r
.42715 .73957 .43602 .75313 .45878 .42821 .4506 .4242 Metetra
.722 .193 0 r
.4506 .4242 .45878 .42821 .47243 .44344 .46426 .43959 Metetra
.664 .116 0 r
.46426 .43959 .47243 .44344 .48636 .45688 .47822 .45311 Metetra
.591 .027 0 r
.47822 .45311 .48636 .45688 .50057 .46856 .49245 .46479 Metetra
.5 0 0 r
.49245 .46479 .50057 .46856 .51502 .47857 .50695 .47473 Metetra
.39 0 0 r
.50695 .47473 .51502 .47857 .5297 .48697 .52167 .483 Metetra
0 .308 .675 r
.52167 .483 .5297 .48697 .54459 .49386 .53661 .48969 Metetra
0 .422 .777 r
.53661 .48969 .54459 .49386 .55966 .49933 .55173 .49492 Metetra
.009 .524 .86 r
.55173 .49492 .55966 .49933 .5749 .50348 .56703 .49879 Metetra
.135 .607 .922 r
.56703 .49879 .5749 .50348 .59031 .50642 .58249 .50142 Metetra
.244 .67 .961 r
.58249 .50142 .59031 .50642 .60585 .50826 .59809 .50292 Metetra
.335 .714 .983 r
.59809 .50292 .60585 .50826 .62152 .50909 .61383 .50339 Metetra
.408 .744 .993 r
.61383 .50339 .62152 .50909 .63732 .50902 .62968 .50296 Metetra
.465 .763 .995 r
.62968 .50296 .63732 .50902 .65323 .50814 .64565 .50172 Metetra
.51 .774 .991 r
.64565 .50172 .65323 .50814 .66924 .50654 .66172 .49976 Metetra
.545 .78 .985 r
.66172 .49976 .66924 .50654 .68536 .50431 .6779 .49718 Metetra
.572 .783 .978 r
.6779 .49718 .68536 .50431 .70157 .50153 .69417 .49405 Metetra
.592 .783 .971 r
.69417 .49405 .70157 .50153 .71788 .49826 .71054 .49046 Metetra
.609 .782 .964 r
.71054 .49046 .71788 .49826 .73428 .49457 .727 .48646 Metetra
.621 .78 .957 r
.727 .48646 .73428 .49457 .75077 .49052 .74355 .48212 Metetra
.631 .777 .951 r
.74355 .48212 .75077 .49052 .76735 .48616 .7602 .47749 Metetra
.638 .775 .946 r
.7602 .47749 .76735 .48616 .78403 .48154 .77693 .47261 Metetra
.644 .772 .941 r
.77693 .47261 .78403 .48154 .8008 .47669 .79377 .46753 Metetra
.649 .769 .937 r
.79377 .46753 .8008 .47669 .81766 .47165 .81069 .46227 Metetra
.652 .767 .933 r
.81069 .46227 .81766 .47165 .83462 .46644 .82772 .45687 Metetra
.557 .647 .895 r
.22478 .64139 .23383 .65269 .24711 .6521 .23813 .64027 Metetra
.527 .625 .892 r
.23813 .64027 .24711 .6521 .26043 .65242 .25151 .64004 Metetra
.492 .599 .888 r
.25151 .64004 .26043 .65242 .27379 .65371 .26491 .64082 Metetra
.454 .571 .883 r
.26491 .64082 .27379 .65371 .28719 .65604 .27836 .64266 Metetra
.411 .541 .877 r
.27836 .64266 .28719 .65604 .30066 .65946 .29185 .64565 Metetra
.364 .509 .869 r
.29185 .64565 .30066 .65946 .31419 .66402 .3054 .64983 Metetra
.313 .474 .86 r
.3054 .64983 .31419 .66402 .32781 .66972 .31903 .65523 Metetra
.259 .438 .848 r
.31903 .65523 .32781 .66972 .34153 .67657 .33275 .66186 Metetra
.202 .401 .835 r
.33275 .66186 .34153 .67657 .35538 .68455 .34659 .66971 Metetra
.143 .363 .819 r
.34659 .66971 .35538 .68455 .36937 .69363 .36056 .67875 Metetra
.083 .325 .802 r
.36056 .67875 .36937 .69363 .38352 .70375 .37469 .68893 Metetra
.023 .288 .783 r
.37469 .68893 .38352 .70375 .39786 .71484 .389 .70018 Metetra
0 .252 .763 r
.389 .70018 .39786 .71484 .41239 .72681 .4035 .71241 Metetra
0 .218 .741 r
.4035 .71241 .41239 .72681 .42715 .73957 .41822 .72552 Metetra
.953 .697 .512 r
.41822 .72552 .42715 .73957 .4506 .4242 .4423 .42048 Metetra
.73 .203 0 r
.4423 .42048 .4506 .4242 .46426 .43959 .45597 .43603 Metetra
.673 .126 0 r
.45597 .43603 .46426 .43959 .47822 .45311 .46995 .44962 Metetra
.601 .035 0 r
.46995 .44962 .47822 .45311 .49245 .46479 .48422 .4613 Metetra
.509 0 0 r
.48422 .4613 .49245 .46479 .50695 .47473 .49875 .47115 Metetra
.396 0 0 r
.49875 .47115 .50695 .47473 .52167 .483 .51353 .47926 Metetra
0 .312 .667 r
.51353 .47926 .52167 .483 .53661 .48969 .52851 .48573 Metetra
0 .431 .775 r
.52851 .48573 .53661 .48969 .55173 .49492 .5437 .49068 Metetra
.019 .537 .863 r
.5437 .49068 .55173 .49492 .56703 .49879 .55906 .49423 Metetra
.15 .622 .926 r
.55906 .49423 .56703 .49879 .58249 .50142 .57457 .4965 Metetra
.262 .685 .966 r
.57457 .4965 .58249 .50142 .59809 .50292 .59024 .49762 Metetra
.354 .728 .987 r
.59024 .49762 .59809 .50292 .61383 .50339 .60603 .49771 Metetra
.427 .756 .995 r
.60603 .49771 .61383 .50339 .62968 .50296 .62195 .49687 Metetra
.483 .773 .995 r
.62195 .49687 .62968 .50296 .64565 .50172 .63797 .49523 Metetra
.526 .782 .991 r
.63797 .49523 .64565 .50172 .66172 .49976 .65411 .49288 Metetra
.559 .786 .984 r
.65411 .49288 .66172 .49976 .6779 .49718 .67034 .48992 Metetra
.584 .787 .976 r
.67034 .48992 .6779 .49718 .69417 .49405 .68667 .48643 Metetra
.604 .786 .968 r
.68667 .48643 .69417 .49405 .71054 .49046 .7031 .48249 Metetra
.618 .784 .961 r
.7031 .48249 .71054 .49046 .727 .48646 .71962 .47817 Metetra
.63 .781 .954 r
.71962 .47817 .727 .48646 .74355 .48212 .73623 .47353 Metetra
.638 .778 .948 r
.73623 .47353 .74355 .48212 .7602 .47749 .75294 .46862 Metetra
.645 .775 .942 r
.75294 .46862 .7602 .47749 .77693 .47261 .76974 .46348 Metetra
.649 .772 .938 r
.76974 .46348 .77693 .47261 .79377 .46753 .78664 .45816 Metetra
.653 .769 .934 r
.78664 .45816 .79377 .46753 .81069 .46227 .80363 .45269 Metetra
.656 .766 .931 r
.80363 .45269 .81069 .46227 .82772 .45687 .82072 .44709 Metetra
.575 .657 .893 r
.21564 .63012 .22478 .64139 .23813 .64027 .22907 .62841 Metetra
.546 .634 .89 r
.22907 .62841 .23813 .64027 .25151 .64004 .24251 .62759 Metetra
.513 .609 .886 r
.24251 .62759 .25151 .64004 .26491 .64082 .25597 .62779 Metetra
.475 .581 .881 r
.25597 .62779 .26491 .64082 .27836 .64266 .26946 .62909 Metetra
.432 .55 .875 r
.26946 .62909 .27836 .64266 .29185 .64565 .28299 .63157 Metetra
.385 .517 .867 r
.28299 .63157 .29185 .64565 .3054 .64983 .29657 .63531 Metetra
.334 .481 .858 r
.29657 .63531 .3054 .64983 .31903 .65523 .31021 .64034 Metetra
.278 .444 .846 r
.31021 .64034 .31903 .65523 .33275 .66186 .32394 .64669 Metetra
.219 .405 .833 r
.32394 .64669 .33275 .66186 .34659 .66971 .33777 .65436 Metetra
.158 .364 .817 r
.33777 .65436 .34659 .66971 .36056 .67875 .35172 .66333 Metetra
.094 .324 .799 r
.35172 .66333 .36056 .67875 .37469 .68893 .36582 .67354 Metetra
.03 .284 .779 r
.36582 .67354 .37469 .68893 .389 .70018 .38009 .68494 Metetra
0 .245 .757 r
.38009 .68494 .389 .70018 .4035 .71241 .39456 .69743 Metetra
0 .208 .734 r
.39456 .69743 .4035 .71241 .41822 .72552 .40924 .7109 Metetra
.952 .697 .514 r
.40924 .7109 .41822 .72552 .4423 .42048 .43387 .41708 Metetra
.738 .213 0 r
.43387 .41708 .4423 .42048 .45597 .43603 .44755 .43281 Metetra
.682 .136 0 r
.44755 .43281 .45597 .43603 .46995 .44962 .46156 .44648 Metetra
.61 .044 0 r
.46156 .44648 .46995 .44962 .48422 .4613 .47586 .45813 Metetra
.517 0 0 r
.47586 .45813 .48422 .4613 .49875 .47115 .49044 .46787 Metetra
.402 0 0 r
.49044 .46787 .49875 .47115 .51353 .47926 .50526 .47578 Metetra
0 .316 .659 r
.50526 .47578 .51353 .47926 .52851 .48573 .52031 .48199 Metetra
0 .442 .773 r
.52031 .48199 .52851 .48573 .5437 .49068 .53555 .48662 Metetra
.032 .552 .865 r
.53555 .48662 .5437 .49068 .55906 .49423 .55097 .4898 Metetra
.168 .64 .931 r
.55097 .4898 .55906 .49423 .57457 .4965 .56655 .49167 Metetra
.283 .702 .971 r
.56655 .49167 .57457 .4965 .59024 .49762 .58227 .49237 Metetra
.376 .744 .99 r
.58227 .49237 .59024 .49762 .60603 .49771 .59813 .49202 Metetra
.448 .769 .997 r
.59813 .49202 .60603 .49771 .62195 .49687 .61411 .49074 Metetra
.502 .783 .995 r
.61411 .49074 .62195 .49687 .63797 .49523 .6302 .48866 Metetra
.543 .79 .989 r
.6302 .48866 .63797 .49523 .65411 .49288 .64639 .48589 Metetra
.574 .792 .981 r
.64639 .48589 .65411 .49288 .67034 .48992 .66269 .48252 Metetra
.598 .791 .973 r
.66269 .48252 .67034 .48992 .68667 .48643 .67908 .47864 Metetra
.615 .788 .965 r
.67908 .47864 .68667 .48643 .7031 .48249 .69557 .47434 Metetra
.628 .785 .957 r
.69557 .47434 .7031 .48249 .71962 .47817 .71215 .46968 Metetra
.638 .782 .95 r
.71215 .46968 .71962 .47817 .73623 .47353 .72882 .46472 Metetra
.645 .778 .944 r
.72882 .46472 .73623 .47353 .75294 .46862 .74559 .45952 Metetra
.651 .774 .939 r
.74559 .45952 .75294 .46862 .76974 .46348 .76245 .45412 Metetra
.655 .771 .934 r
.76245 .45412 .76974 .46348 .78664 .45816 .77941 .44856 Metetra
.657 .768 .931 r
.77941 .44856 .78664 .45816 .80363 .45269 .79647 .44287 Metetra
.659 .765 .928 r
.79647 .44287 .80363 .45269 .82072 .44709 .81363 .43708 Metetra
.593 .667 .891 r
.20642 .61891 .21564 .63012 .22907 .62841 .21992 .61656 Metetra
.566 .645 .888 r
.21992 .61656 .22907 .62841 .24251 .62759 .23344 .6151 Metetra
.534 .62 .884 r
.23344 .6151 .24251 .62759 .25597 .62779 .24697 .61465 Metetra
.497 .591 .879 r
.24697 .61465 .25597 .62779 .26946 .62909 .26052 .61532 Metetra
.455 .56 .873 r
.26052 .61532 .26946 .62909 .28299 .63157 .27409 .61722 Metetra
.408 .526 .865 r
.27409 .61722 .28299 .63157 .29657 .63531 .2877 .62044 Metetra
.356 .49 .856 r
.2877 .62044 .29657 .63531 .31021 .64034 .30136 .62503 Metetra
.3 .451 .844 r
.30136 .62503 .31021 .64034 .32394 .64669 .31509 .63104 Metetra
.239 .41 .831 r
.31509 .63104 .32394 .64669 .33777 .65436 .32891 .63847 Metetra
.175 .367 .815 r
.32891 .63847 .33777 .65436 .35172 .66333 .34285 .64731 Metetra
.108 .324 .796 r
.34285 .64731 .35172 .66333 .36582 .67354 .35692 .65753 Metetra
.04 .281 .775 r
.35692 .65753 .36582 .67354 .38009 .68494 .37115 .66906 Metetra
0 .238 .752 r
.37115 .66906 .38009 .68494 .39456 .69743 .38558 .68181 Metetra
0 .198 .727 r
.38558 .68181 .39456 .69743 .40924 .7109 .40021 .69566 Metetra
.951 .697 .515 r
.40021 .69566 .40924 .7109 .43387 .41708 .4253 .41407 Metetra
.745 .223 0 r
.4253 .41407 .43387 .41708 .44755 .43281 .439 .42999 Metetra
.69 .146 0 r
.439 .42999 .44755 .43281 .46156 .44648 .45303 .44373 Metetra
.618 .052 0 r
.45303 .44373 .46156 .44648 .47586 .45813 .46737 .45535 Metetra
.524 0 0 r
.46737 .45535 .47586 .45813 .49044 .46787 .482 .46494 Metetra
.406 0 0 r
.482 .46494 .49044 .46787 .50526 .47578 .49687 .47262 Metetra
.264 0 0 r
.49687 .47262 .50526 .47578 .52031 .48199 .51197 .47852 Metetra
0 .455 .771 r
.51197 .47852 .52031 .48199 .53555 .48662 .52728 .48277 Metetra
.048 .571 .869 r
.52728 .48277 .53555 .48662 .55097 .4898 .54276 .48553 Metetra
.19 .66 .937 r
.54276 .48553 .55097 .4898 .56655 .49167 .55841 .48694 Metetra
.308 .721 .976 r
.55841 .48694 .56655 .49167 .58227 .49237 .5742 .48715 Metetra
.401 .76 .994 r
.5742 .48715 .58227 .49237 .59813 .49202 .59012 .48631 Metetra
.471 .782 .998 r
.59012 .48631 .59813 .49202 .61411 .49074 .60616 .48455 Metetra
.523 .793 .995 r
.60616 .48455 .61411 .49074 .6302 .48866 .62231 .48199 Metetra
.562 .797 .987 r
.62231 .48199 .6302 .48866 .64639 .48589 .63857 .47875 Metetra
.59 .797 .978 r
.63857 .47875 .64639 .48589 .66269 .48252 .65493 .47494 Metetra
.611 .794 .969 r
.65493 .47494 .66269 .48252 .67908 .47864 .67138 .47065 Metetra
.627 .79 .961 r
.67138 .47065 .67908 .47864 .69557 .47434 .68793 .46596 Metetra
.638 .786 .953 r
.68793 .46596 .69557 .47434 .71215 .46968 .70457 .46095 Metetra
.646 .782 .946 r
.70457 .46095 .71215 .46968 .72882 .46472 .72131 .45567 Metetra
.652 .777 .94 r
.72131 .45567 .72882 .46472 .74559 .45952 .73814 .45018 Metetra
.656 .773 .935 r
.73814 .45018 .74559 .45952 .76245 .45412 .75507 .44451 Metetra
.659 .769 .931 r
.75507 .44451 .76245 .45412 .77941 .44856 .77209 .43872 Metetra
.661 .766 .927 r
.77209 .43872 .77941 .44856 .79647 .44287 .78921 .43282 Metetra
.663 .763 .925 r
.78921 .43282 .79647 .44287 .81363 .43708 .80644 .42684 Metetra
.612 .678 .889 r
.19709 .60781 .20642 .61891 .21992 .61656 .21068 .60478 Metetra
.586 .656 .885 r
.21068 .60478 .21992 .61656 .23344 .6151 .22429 .6026 Metetra
.556 .631 .881 r
.22429 .6026 .23344 .6151 .24697 .61465 .23789 .60143 Metetra
.521 .603 .876 r
.23789 .60143 .24697 .61465 .26052 .61532 .25151 .60139 Metetra
.48 .571 .87 r
.25151 .60139 .26052 .61532 .27409 .61722 .26513 .60262 Metetra
.433 .537 .863 r
.26513 .60262 .27409 .61722 .2877 .62044 .27879 .60523 Metetra
.381 .499 .853 r
.27879 .60523 .2877 .62044 .30136 .62503 .29247 .60929 Metetra
.324 .459 .842 r
.29247 .60929 .30136 .62503 .31509 .63104 .30622 .61486 Metetra
.261 .416 .828 r
.30622 .61486 .31509 .63104 .32891 .63847 .32003 .62198 Metetra
.195 .371 .812 r
.32003 .62198 .32891 .63847 .34285 .64731 .33395 .63065 Metetra
.125 .325 .793 r
.33395 .63065 .34285 .64731 .35692 .65753 .34799 .64083 Metetra
.052 .279 .771 r
.34799 .64083 .35692 .65753 .37115 .66906 .36219 .65247 Metetra
0 .233 .746 r
.36219 .65247 .37115 .66906 .38558 .68181 .37656 .66547 Metetra
0 .188 .719 r
.37656 .66547 .38558 .68181 .40021 .69566 .39115 .67972 Metetra
.95 .696 .518 r
.39115 .67972 .40021 .69566 .4253 .41407 .41659 .41152 Metetra
.752 .234 0 r
.41659 .41152 .4253 .41407 .439 .42999 .4303 .42764 Metetra
.698 .156 0 r
.4303 .42764 .439 .42999 .45303 .44373 .44436 .44145 Metetra
.626 .061 0 r
.44436 .44145 .45303 .44373 .46737 .45535 .45874 .453 Metetra
.531 0 0 r
.45874 .453 .46737 .45535 .482 .46494 .47341 .46241 Metetra
.409 0 0 r
.47341 .46241 .482 .46494 .49687 .47262 .48835 .46981 Metetra
.261 0 0 r
.48835 .46981 .49687 .47262 .51197 .47852 .50351 .47534 Metetra
0 .472 .771 r
.50351 .47534 .51197 .47852 .52728 .48277 .51888 .47915 Metetra
.069 .592 .874 r
.51888 .47915 .52728 .48277 .54276 .48553 .53444 .48142 Metetra
.217 .683 .943 r
.53444 .48142 .54276 .48553 .55841 .48694 .55015 .4823 Metetra
.337 .742 .981 r
.55015 .4823 .55841 .48694 .5742 .48715 .56601 .48197 Metetra
.429 .777 .997 r
.56601 .48197 .5742 .48715 .59012 .48631 .582 .48057 Metetra
.497 .795 .998 r
.582 .48057 .59012 .48631 .60616 .48455 .59811 .47827 Metetra
.546 .803 .993 r
.59811 .47827 .60616 .48455 .62231 .48199 .61433 .47518 Metetra
.582 .804 .984 r
.61433 .47518 .62231 .48199 .63857 .47875 .63065 .47144 Metetra
.607 .801 .974 r
.63065 .47144 .63857 .47875 .65493 .47494 .64707 .46716 Metetra
.625 .797 .964 r
.64707 .46716 .65493 .47494 .67138 .47065 .66358 .46243 Metetra
.639 .791 .956 r
.66358 .46243 .67138 .47065 .68793 .46596 .68019 .45734 Metetra
.648 .786 .948 r
.68019 .45734 .68793 .46596 .70457 .46095 .69689 .45196 Metetra
.654 .781 .941 r
.69689 .45196 .70457 .46095 .72131 .45567 .71369 .44635 Metetra
.659 .776 .935 r
.71369 .44635 .72131 .45567 .73814 .45018 .73059 .44056 Metetra
.662 .771 .931 r
.73059 .44056 .73814 .45018 .75507 .44451 .74758 .43464 Metetra
.664 .767 .927 r
.74758 .43464 .75507 .44451 .77209 .43872 .76467 .42861 Metetra
.665 .764 .924 r
.76467 .42861 .77209 .43872 .78921 .43282 .78186 .42251 Metetra
.665 .761 .921 r
.78186 .42251 .78921 .43282 .80644 .42684 .79915 .41635 Metetra
.631 .69 .887 r
.18765 .59689 .19709 .60781 .21068 .60478 .20134 .59312 Metetra
.608 .669 .883 r
.20134 .59312 .21068 .60478 .22429 .6026 .21504 .59016 Metetra
.579 .644 .879 r
.21504 .59016 .22429 .6026 .23789 .60143 .22874 .58818 Metetra
.546 .616 .873 r
.22874 .58818 .23789 .60143 .25151 .60139 .24244 .58734 Metetra
.506 .584 .867 r
.24244 .58734 .25151 .60139 .26513 .60262 .25613 .58779 Metetra
.46 .549 .86 r
.25613 .58779 .26513 .60262 .27879 .60523 .26983 .58967 Metetra
.408 .51 .851 r
.26983 .58967 .27879 .60523 .29247 .60929 .28356 .59309 Metetra
.35 .468 .839 r
.28356 .59309 .29247 .60929 .30622 .61486 .29732 .59813 Metetra
.287 .424 .826 r
.29732 .59813 .30622 .61486 .32003 .62198 .31114 .60485 Metetra
.218 .377 .809 r
.31114 .60485 .32003 .62198 .33395 .63065 .32504 .61326 Metetra
.145 .328 .79 r
.32504 .61326 .33395 .63065 .34799 .64083 .33905 .62335 Metetra
.068 .278 .767 r
.33905 .62335 .34799 .64083 .36219 .65247 .3532 .63506 Metetra
0 .228 .741 r
.3532 .63506 .36219 .65247 .37656 .66547 .36753 .6483 Metetra
0 .179 .712 r
.36753 .6483 .37656 .66547 .39115 .67972 .38205 .66296 Metetra
.949 .696 .52 r
.38205 .66296 .39115 .67972 .41659 .41152 .40772 .40954 Metetra
.758 .244 0 r
.40772 .40954 .41659 .41152 .4303 .42764 .42144 .42587 Metetra
.704 .166 0 r
.42144 .42587 .4303 .42764 .44436 .44145 .43553 .43973 Metetra
.632 .069 0 r
.43553 .43973 .44436 .44145 .45874 .453 .44996 .45119 Metetra
.536 0 0 r
.44996 .45119 .45874 .453 .47341 .46241 .46469 .46037 Metetra
.41 0 0 r
.46469 .46037 .47341 .46241 .48835 .46981 .47969 .46741 Metetra
.254 0 0 r
.47969 .46741 .48835 .46981 .50351 .47534 .49492 .47249 Metetra
0 .492 .771 r
.49492 .47249 .50351 .47534 .51888 .47915 .51036 .47578 Metetra
.096 .618 .881 r
.51036 .47578 .51888 .47915 .53444 .48142 .52599 .47747 Metetra
.25 .709 .951 r
.52599 .47747 .53444 .48142 .55015 .4823 .54178 .47775 Metetra
.371 .765 .987 r
.54178 .47775 .55015 .4823 .56601 .48197 .55771 .47679 Metetra
.461 .795 .999 r
.55771 .47679 .56601 .48197 .582 .48057 .57377 .47478 Metetra
.525 .809 .998 r
.57377 .47478 .582 .48057 .59811 .47827 .58994 .47186 Metetra
.57 .812 .99 r
.58994 .47186 .59811 .47827 .61433 .47518 .60623 .4682 Metetra
.602 .809 .979 r
.60623 .4682 .61433 .47518 .63065 .47144 .62261 .46391 Metetra
.624 .804 .969 r
.62261 .46391 .63065 .47144 .64707 .46716 .63909 .45912 Metetra
.64 .798 .959 r
.63909 .45912 .64707 .46716 .66358 .46243 .65567 .45393 Metetra
.65 .791 .95 r
.65567 .45393 .66358 .46243 .68019 .45734 .67234 .44842 Metetra
.657 .785 .942 r
.67234 .44842 .68019 .45734 .69689 .45196 .68911 .44266 Metetra
.662 .779 .936 r
.68911 .44266 .69689 .45196 .71369 .44635 .70597 .43672 Metetra
.665 .774 .93 r
.70597 .43672 .71369 .44635 .73059 .44056 .72293 .43064 Metetra
.667 .769 .926 r
.72293 .43064 .73059 .44056 .74758 .43464 .73998 .42447 Metetra
.668 .765 .923 r
.73998 .42447 .74758 .43464 .76467 .42861 .75714 .41822 Metetra
.668 .761 .92 r
.75714 .41822 .76467 .42861 .78186 .42251 .7744 .41192 Metetra
.668 .758 .918 r
.7744 .41192 .78186 .42251 .79915 .41635 .79177 .4056 Metetra
.651 .704 .886 r
.17808 .58622 .18765 .59689 .20134 .59312 .19189 .58167 Metetra
.63 .683 .881 r
.19189 .58167 .20134 .59312 .21504 .59016 .2057 .57786 Metetra
.604 .658 .876 r
.2057 .57786 .21504 .59016 .22874 .58818 .2195 .57498 Metetra
.572 .63 .871 r
.2195 .57498 .22874 .58818 .24244 .58734 .2333 .57323 Metetra
.534 .598 .864 r
.2333 .57323 .24244 .58734 .25613 .58779 .24708 .57278 Metetra
.489 .562 .856 r
.24708 .57278 .25613 .58779 .26983 .58967 .26085 .5738 Metetra
.438 .523 .847 r
.26085 .5738 .26983 .58967 .28356 .59309 .27462 .57644 Metetra
.38 .48 .836 r
.27462 .57644 .28356 .59309 .29732 .59813 .28841 .58082 Metetra
.316 .434 .822 r
.28841 .58082 .29732 .59813 .31114 .60485 .30224 .58701 Metetra
.245 .384 .806 r
.30224 .58701 .31114 .60485 .32504 .61326 .31613 .59507 Metetra
.169 .333 .786 r
.31613 .59507 .32504 .61326 .33905 .62335 .33011 .60499 Metetra
.088 .279 .762 r
.33011 .60499 .33905 .62335 .3532 .63506 .34422 .61672 Metetra
.004 .225 .735 r
.34422 .61672 .3532 .63506 .36753 .6483 .35848 .63019 Metetra
0 .171 .704 r
.35848 .63019 .36753 .6483 .38205 .66296 .37293 .64528 Metetra
.947 .695 .523 r
.37293 .64528 .38205 .66296 .40772 .40954 .39867 .40825 Metetra
.762 .254 0 r
.39867 .40825 .40772 .40954 .42144 .42587 .41241 .4248 Metetra
.709 .176 0 r
.41241 .4248 .42144 .42587 .43553 .43973 .42654 .43869 Metetra
.637 .077 0 r
.42654 .43869 .43553 .43973 .44996 .45119 .44101 .45 Metetra
.539 0 0 r
.44101 .45 .44996 .45119 .46469 .46037 .45581 .45888 Metetra
.408 0 0 r
.45581 .45888 .46469 .46037 .47969 .46741 .47087 .46549 Metetra
.243 0 0 r
.47087 .46549 .47969 .46741 .49492 .47249 .48618 .47003 Metetra
0 .517 .773 r
.48618 .47003 .49492 .47249 .51036 .47578 .50171 .47269 Metetra
.131 .648 .889 r
.50171 .47269 .51036 .47578 .52599 .47747 .51741 .4737 Metetra
.289 .738 .96 r
.51741 .4737 .52599 .47747 .54178 .47775 .53328 .47327 Metetra
.41 .79 .992 r
.53328 .47327 .54178 .47775 .55771 .47679 .54928 .4716 Metetra
.496 .813 1 r
.54928 .4716 .55771 .47679 .57377 .47478 .56541 .46888 Metetra
.556 .821 .995 r
.56541 .46888 .57377 .47478 .58994 .47186 .58166 .46528 Metetra
.596 .819 .985 r
.58166 .46528 .58994 .47186 .60623 .4682 .59801 .46098 Metetra
.623 .813 .973 r
.59801 .46098 .60623 .4682 .62261 .46391 .61446 .4561 Metetra
.641 .806 .962 r
.61446 .4561 .62261 .46391 .63909 .45912 .63101 .45077 Metetra
.654 .798 .952 r
.63101 .45077 .63909 .45912 .65567 .45393 .64765 .4451 Metetra
.661 .79 .943 r
.64765 .4451 .65567 .45393 .67234 .44842 .66439 .43916 Metetra
.666 .782 .936 r
.66439 .43916 .67234 .44842 .68911 .44266 .68122 .43302 Metetra
.669 .776 .93 r
.68122 .43302 .68911 .44266 .70597 .43672 .69814 .42675 Metetra
.671 .77 .925 r
.69814 .42675 .70597 .43672 .72293 .43064 .71516 .42039 Metetra
.671 .766 .921 r
.71516 .42039 .72293 .43064 .73998 .42447 .73229 .41397 Metetra
.671 .762 .918 r
.73229 .41397 .73998 .42447 .75714 .41822 .74951 .40752 Metetra
.67 .758 .916 r
.74951 .40752 .75714 .41822 .7744 .41192 .76684 .40105 Metetra
.669 .756 .915 r
.76684 .40105 .7744 .41192 .79177 .4056 .78428 .39458 Metetra
.67 .718 .884 r
.16838 .57588 .17808 .58622 .19189 .58167 .18231 .57052 Metetra
.653 .698 .879 r
.18231 .57052 .19189 .58167 .2057 .57786 .19625 .5658 Metetra
.629 .674 .874 r
.19625 .5658 .2057 .57786 .2195 .57498 .21017 .56193 Metetra
.6 .646 .868 r
.21017 .56193 .2195 .57498 .2333 .57323 .22408 .55913 Metetra
.564 .614 .861 r
.22408 .55913 .2333 .57323 .24708 .57278 .23796 .55763 Metetra
.521 .577 .853 r
.23796 .55763 .24708 .57278 .26085 .5738 .25182 .55764 Metetra
.471 .537 .843 r
.25182 .55764 .26085 .5738 .27462 .57644 .26566 .55934 Metetra
.413 .493 .832 r
.26566 .55934 .27462 .57644 .28841 .58082 .2795 .56289 Metetra
.349 .445 .818 r
.2795 .56289 .28841 .58082 .30224 .58701 .29334 .56841 Metetra
.277 .394 .802 r
.29334 .56841 .30224 .58701 .31613 .59507 .30723 .57598 Metetra
.198 .339 .782 r
.30723 .57598 .31613 .59507 .33011 .60499 .32119 .58562 Metetra
.113 .282 .758 r
.32119 .58562 .33011 .60499 .34422 .61672 .33524 .59731 Metetra
.023 .224 .73 r
.33524 .59731 .34422 .61672 .35848 .63019 .34944 .61098 Metetra
0 .165 .697 r
.34944 .61098 .35848 .63019 .37293 .64528 .36381 .6265 Metetra
.945 .695 .527 r
.36381 .6265 .37293 .64528 .39867 .40825 .38944 .40779 Metetra
.765 .264 .02 r
.38944 .40779 .39867 .40825 .41241 .4248 .40319 .42458 Metetra
.712 .185 0 r
.40319 .42458 .41241 .4248 .42654 .43869 .41736 .43847 Metetra
.639 .084 0 r
.41736 .43847 .42654 .43869 .44101 .45 .4319 .44957 Metetra
.539 0 0 r
.4319 .44957 .44101 .45 .45581 .45888 .44676 .45806 Metetra
.402 0 0 r
.44676 .45806 .45581 .45888 .47087 .46549 .46191 .46412 Metetra
.225 0 0 r
.46191 .46412 .47087 .46549 .48618 .47003 .4773 .46798 Metetra
0 .549 .778 r
.4773 .46798 .48618 .47003 .50171 .47269 .49291 .46989 Metetra
.175 .685 .9 r
.49291 .46989 .50171 .47269 .51741 .4737 .5087 .47009 Metetra
.337 .771 .969 r
.5087 .47009 .51741 .4737 .53328 .47327 .52465 .46883 Metetra
.456 .815 .996 r
.52465 .46883 .53328 .47327 .54928 .4716 .54073 .46633 Metetra
.536 .831 .999 r
.54073 .46633 .54928 .4716 .56541 .46888 .55694 .46281 Metetra
.588 .831 .991 r
.55694 .46281 .56541 .46888 .58166 .46528 .57326 .45846 Metetra
.623 .825 .978 r
.57326 .45846 .58166 .46528 .59801 .46098 .58968 .45346 Metetra
.645 .815 .965 r
.58968 .45346 .59801 .46098 .61446 .4561 .6062 .44795 Metetra
.658 .805 .953 r
.6062 .44795 .61446 .4561 .63101 .45077 .62281 .44205 Metetra
.667 .795 .943 r
.62281 .44205 .63101 .45077 .64765 .4451 .63952 .43587 Metetra
.672 .786 .935 r
.63952 .43587 .64765 .4451 .66439 .43916 .65632 .42949 Metetra
.675 .779 .928 r
.65632 .42949 .66439 .43916 .68122 .43302 .67321 .42299 Metetra
.676 .772 .923 r
.67321 .42299 .68122 .43302 .69814 .42675 .6902 .4164 Metetra
.675 .766 .919 r
.6902 .4164 .69814 .42675 .71516 .42039 .70729 .40978 Metetra
.675 .762 .916 r
.70729 .40978 .71516 .42039 .73229 .41397 .72448 .40314 Metetra
.674 .758 .914 r
.72448 .40314 .73229 .41397 .74951 .40752 .74178 .3965 Metetra
.672 .755 .913 r
.74178 .3965 .74951 .40752 .76684 .40105 .75918 .38988 Metetra
.671 .752 .912 r
.75918 .38988 .76684 .40105 .78428 .39458 .7767 .38328 Metetra
.69 .734 .884 r
.15852 .56599 .16838 .57588 .18231 .57052 .17258 .5598 Metetra
.675 .715 .878 r
.17258 .5598 .18231 .57052 .19625 .5658 .18666 .55411 Metetra
.655 .691 .872 r
.18666 .55411 .19625 .5658 .21017 .56193 .20072 .54915 Metetra
.629 .664 .865 r
.20072 .54915 .21017 .56193 .22408 .55913 .21477 .54518 Metetra
.595 .631 .857 r
.21477 .54518 .22408 .55913 .23796 .55763 .22878 .54246 Metetra
.555 .595 .849 r
.22878 .54246 .23796 .55763 .25182 .55764 .24275 .54126 Metetra
.506 .554 .839 r
.24275 .54126 .25182 .55764 .26566 .55934 .25668 .54182 Metetra
.45 .509 .828 r
.25668 .54182 .26566 .55934 .2795 .56289 .27058 .54433 Metetra
.385 .459 .814 r
.27058 .54433 .2795 .56289 .29334 .56841 .28446 .54898 Metetra
.313 .406 .797 r
.28446 .54898 .29334 .56841 .30723 .57598 .29836 .55589 Metetra
.232 .348 .777 r
.29836 .55589 .30723 .57598 .32119 .58562 .31229 .56511 Metetra
.143 .288 .753 r
.31229 .56511 .32119 .58562 .33524 .59731 .3263 .57665 Metetra
.048 .225 .724 r
.3263 .57665 .33524 .59731 .34944 .61098 .34043 .59046 Metetra
0 .16 .69 r
.34043 .59046 .34944 .61098 .36381 .6265 .35471 .60641 Metetra
.942 .693 .533 r
.35471 .60641 .36381 .6265 .38944 .40779 .37999 .4084 Metetra
.766 .274 .055 r
.37999 .4084 .38944 .40779 .40319 .42458 .39377 .42541 Metetra
.712 .193 0 r
.39377 .42541 .40319 .42458 .41736 .43847 .40798 .43925 Metetra
.638 .089 0 r
.40798 .43925 .41736 .43847 .4319 .44957 .42259 .45006 Metetra
.534 0 0 r
.42259 .45006 .4319 .44957 .44676 .45806 .43753 .45802 Metetra
.389 0 0 r
.43753 .45802 .44676 .45806 .46191 .46412 .45277 .46338 Metetra
.198 0 0 r
.45277 .46338 .46191 .46412 .4773 .46798 .46826 .46641 Metetra
.023 .589 .786 r
.46826 .46641 .4773 .46798 .49291 .46989 .48396 .46739 Metetra
.232 .728 .913 r
.48396 .46739 .49291 .46989 .5087 .47009 .49985 .46662 Metetra
.395 .807 .978 r
.49985 .46662 .5087 .47009 .52465 .46883 .51589 .46438 Metetra
.507 .84 .998 r
.51589 .46438 .52465 .46883 .54073 .46633 .53206 .46092 Metetra
.578 .846 .995 r
.53206 .46092 .54073 .46633 .55694 .46281 .54835 .4565 Metetra
.622 .839 .983 r
.54835 .4565 .55694 .46281 .57326 .45846 .56474 .45131 Metetra
.649 .827 .969 r
.56474 .45131 .57326 .45846 .58968 .45346 .58124 .44554 Metetra
.665 .814 .955 r
.58124 .44554 .58968 .45346 .6062 .44795 .59782 .43935 Metetra
.674 .802 .943 r
.59782 .43935 .6062 .44795 .62281 .44205 .6145 .43286 Metetra
.679 .791 .934 r
.6145 .43286 .62281 .44205 .63952 .43587 .63127 .42617 Metetra
.681 .781 .926 r
.63127 .42617 .63952 .43587 .65632 .42949 .64813 .41937 Metetra
.681 .773 .92 r
.64813 .41937 .65632 .42949 .67321 .42299 .66509 .41251 Metetra
.681 .766 .916 r
.66509 .41251 .67321 .42299 .6902 .4164 .68215 .40563 Metetra
.679 .761 .913 r
.68215 .40563 .6902 .4164 .70729 .40978 .69931 .39877 Metetra
.677 .757 .911 r
.69931 .39877 .70729 .40978 .72448 .40314 .71657 .39194 Metetra
.675 .753 .91 r
.71657 .39194 .72448 .40314 .74178 .3965 .73394 .38515 Metetra
.673 .751 .909 r
.73394 .38515 .74178 .3965 .75918 .38988 .75142 .37841 Metetra
.671 .749 .909 r
.75142 .37841 .75918 .38988 .7767 .38328 .76901 .37172 Metetra
.709 .751 .885 r
.14848 .55665 .15852 .56599 .17258 .5598 .16268 .54965 Metetra
.698 .734 .878 r
.16268 .54965 .17258 .5598 .18666 .55411 .17691 .54295 Metetra
.682 .711 .87 r
.17691 .54295 .18666 .55411 .20072 .54915 .19114 .53682 Metetra
.659 .684 .862 r
.19114 .53682 .20072 .54915 .21477 .54518 .20534 .53154 Metetra
.629 .652 .854 r
.20534 .53154 .21477 .54518 .22878 .54246 .21951 .52742 Metetra
.591 .615 .845 r
.21951 .52742 .22878 .54246 .24275 .54126 .23362 .52478 Metetra
.545 .574 .834 r
.23362 .52478 .24275 .54126 .25668 .54182 .24767 .52393 Metetra
.49 .527 .822 r
.24767 .52393 .25668 .54182 .27058 .54433 .26166 .52514 Metetra
.427 .476 .808 r
.26166 .52514 .27058 .54433 .28446 .54898 .27561 .52865 Metetra
.354 .42 .792 r
.27561 .52865 .28446 .54898 .29836 .55589 .28953 .53465 Metetra
.271 .36 .772 r
.28953 .53465 .29836 .55589 .31229 .56511 .30346 .54326 Metetra
.18 .296 .747 r
.30346 .54326 .31229 .56511 .3263 .57665 .31742 .55451 Metetra
.079 .228 .718 r
.31742 .55451 .3263 .57665 .34043 .59046 .33147 .56838 Metetra
0 .158 .682 r
.33147 .56838 .34043 .59046 .35471 .60641 .34565 .58475 Metetra
.937 .691 .54 r
.34565 .58475 .35471 .60641 .37999 .4084 .3703 .41034 Metetra
.764 .282 .092 r
.3703 .41034 .37999 .4084 .39377 .42541 .38411 .42758 Metetra
.708 .2 .02 r
.38411 .42758 .39377 .42541 .40798 .43925 .39838 .4413 Metetra
.632 .092 0 r
.39838 .4413 .40798 .43925 .42259 .45006 .41306 .45167 Metetra
.524 0 0 r
.41306 .45167 .42259 .45006 .43753 .45802 .4281 .45892 Metetra
.368 0 0 r
.4281 .45892 .43753 .45802 .45277 .46338 .44345 .46337 Metetra
.158 0 0 r
.44345 .46337 .45277 .46338 .46826 .46641 .45905 .46534 Metetra
.084 .64 .799 r
.45905 .46534 .46826 .46641 .48396 .46739 .47486 .46518 Metetra
.304 .778 .928 r
.47486 .46518 .48396 .46739 .49985 .46662 .49085 .46323 Metetra
.464 .845 .985 r
.49085 .46323 .49985 .46662 .51589 .46438 .50699 .45982 Metetra
.564 .863 .996 r
.50699 .45982 .51589 .46438 .53206 .46092 .52325 .45525 Metetra
.623 .857 .987 r
.52325 .45525 .53206 .46092 .54835 .4565 .53963 .44979 Metetra
.656 .842 .972 r
.53963 .44979 .54835 .4565 .56474 .45131 .5561 .44367 Metetra
.675 .826 .956 r
.5561 .44367 .56474 .45131 .58124 .44554 .57267 .43708 Metetra
.684 .81 .942 r
.57267 .43708 .58124 .44554 .59782 .43935 .58933 .43018 Metetra
.689 .796 .931 r
.58933 .43018 .59782 .43935 .6145 .43286 .60607 .42309 Metetra
.69 .784 .923 r
.60607 .42309 .6145 .43286 .63127 .42617 .62291 .41591 Metetra
.689 .774 .916 r
.62291 .41591 .63127 .42617 .64813 .41937 .63984 .4087 Metetra
.687 .766 .912 r
.63984 .4087 .64813 .41937 .66509 .41251 .65686 .40152 Metetra
.684 .759 .909 r
.65686 .40152 .66509 .41251 .68215 .40563 .67399 .3944 Metetra
.681 .755 .907 r
.67399 .3944 .68215 .40563 .69931 .39877 .69121 .38734 Metetra
.678 .751 .906 r
.69121 .38734 .69931 .39877 .71657 .39194 .70855 .38037 Metetra
.675 .748 .906 r
.70855 .38037 .71657 .39194 .73394 .38515 .72599 .37348 Metetra
.673 .747 .906 r
.72599 .37348 .73394 .38515 .75142 .37841 .74355 .36666 Metetra
.67 .746 .907 r
.74355 .36666 .75142 .37841 .76901 .37172 .76122 .35991 Metetra
.726 .77 .888 r
.13824 .54799 .14848 .55665 .16268 .54965 .15258 .54024 Metetra
.721 .755 .879 r
.15258 .54024 .16268 .54965 .17691 .54295 .16697 .53255 Metetra
.709 .734 .87 r
.16697 .53255 .17691 .54295 .19114 .53682 .18137 .52519 Metetra
.691 .708 .861 r
.18137 .52519 .19114 .53682 .20534 .53154 .19577 .51846 Metetra
.665 .676 .851 r
.19577 .51846 .20534 .53154 .21951 .52742 .21012 .51274 Metetra
.63 .639 .84 r
.21012 .51274 .21951 .52742 .23362 .52478 .22442 .50839 Metetra
.587 .597 .829 r
.22442 .50839 .23362 .52478 .24767 .52393 .23863 .50581 Metetra
.535 .549 .816 r
.23863 .50581 .24767 .52393 .26166 .52514 .25276 .50536 Metetra
.472 .496 .802 r
.25276 .50536 .26166 .52514 .27561 .52865 .2668 .50738 Metetra
.4 .439 .785 r
.2668 .50738 .27561 .52865 .28953 .53465 .28077 .51214 Metetra
.318 .376 .765 r
.28077 .51214 .28953 .53465 .30346 .54326 .29471 .51984 Metetra
.224 .308 .741 r
.29471 .51984 .30346 .54326 .31742 .55451 .30864 .53058 Metetra
.119 .236 .711 r
.30864 .53058 .31742 .55451 .33147 .56838 .32261 .54437 Metetra
.004 .16 .675 r
.32261 .54437 .33147 .56838 .34565 .58475 .33667 .56113 Metetra
.93 .688 .551 r
.33667 .56113 .34565 .58475 .3703 .41034 .36033 .41404 Metetra
.758 .288 .131 r
.36033 .41404 .3703 .41034 .38411 .42758 .37417 .43147 Metetra
.7 .204 .062 r
.37417 .43147 .38411 .42758 .39838 .4413 .38852 .44494 Metetra
.62 .093 0 r
.38852 .44494 .39838 .4413 .41306 .45167 .4033 .45467 Metetra
.505 0 0 r
.4033 .45467 .41306 .45167 .4281 .45892 .41846 .46096 Metetra
.334 0 0 r
.41846 .46096 .4281 .45892 .44345 .46337 .43393 .4642 Metetra
.098 0 0 r
.43393 .4642 .44345 .46337 .45905 .46534 .44966 .4648 Metetra
.169 .705 .819 r
.44966 .4648 .45905 .46534 .47486 .46518 .4656 .4632 Metetra
.395 .834 .945 r
.4656 .4632 .47486 .46518 .49085 .46323 .48171 .4598 Metetra
.542 .88 .989 r
.48171 .4598 .49085 .46323 .50699 .45982 .49796 .455 Metetra
.624 .879 .989 r
.49796 .455 .50699 .45982 .52325 .45525 .51432 .44914 Metetra
.667 .861 .973 r
.51432 .44914 .52325 .45525 .53963 .44979 .53078 .44252 Metetra
.689 .84 .956 r
.53078 .44252 .53963 .44979 .5561 .44367 .54734 .43538 Metetra
.698 .819 .94 r
.54734 .43538 .5561 .44367 .57267 .43708 .56398 .42792 Metetra
.701 .801 .927 r
.56398 .42792 .57267 .43708 .58933 .43018 .58071 .4203 Metetra
.7 .786 .917 r
.58071 .4203 .58933 .43018 .60607 .42309 .59752 .41263 Metetra
.698 .773 .91 r
.59752 .41263 .60607 .42309 .62291 .41591 .61442 .40499 Metetra
.694 .764 .906 r
.61442 .40499 .62291 .41591 .63984 .4087 .63142 .39743 Metetra
.69 .757 .903 r
.63142 .39743 .63984 .4087 .65686 .40152 .64851 .38999 Metetra
.686 .751 .901 r
.64851 .38999 .65686 .40152 .67399 .3944 .66571 .38268 Metetra
.682 .748 .901 r
.66571 .38268 .67399 .3944 .69121 .38734 .68301 .3755 Metetra
.678 .745 .901 r
.68301 .3755 .69121 .38734 .70855 .38037 .70042 .36844 Metetra
.675 .743 .902 r
.70042 .36844 .70855 .38037 .72599 .37348 .71794 .36149 Metetra
.671 .743 .904 r
.71794 .36149 .72599 .37348 .74355 .36666 .73558 .35464 Metetra
.669 .742 .905 r
.73558 .35464 .74355 .36666 .76122 .35991 .75334 .34787 Metetra
.74 .79 .893 r
.12777 .5401 .13824 .54799 .15258 .54024 .14224 .53175 Metetra
.741 .779 .883 r
.14224 .53175 .15258 .54024 .16697 .53255 .15679 .52316 Metetra
.736 .761 .872 r
.15679 .52316 .16697 .53255 .18137 .52519 .17139 .51457 Metetra
.724 .736 .861 r
.17139 .51457 .18137 .52519 .19577 .51846 .186 .50631 Metetra
.703 .705 .849 r
.186 .50631 .19577 .51846 .21012 .51274 .20058 .49876 Metetra
.673 .668 .837 r
.20058 .49876 .21012 .51274 .22442 .50839 .2151 .4924 Metetra
.633 .624 .824 r
.2151 .4924 .22442 .50839 .23863 .50581 .22953 .48769 Metetra
.583 .575 .81 r
.22953 .48769 .23863 .50581 .25276 .50536 .24385 .48512 Metetra
.523 .52 .794 r
.24385 .48512 .25276 .50536 .2668 .50738 .25804 .48516 Metetra
.453 .461 .777 r
.25804 .48516 .2668 .50738 .28077 .51214 .27211 .48821 Metetra
.37 .395 .756 r
.27211 .48821 .28077 .51214 .29471 .51984 .28608 .49457 Metetra
.276 .325 .733 r
.28608 .49457 .29471 .51984 .30864 .53058 .29999 .50446 Metetra
.168 .248 .703 r
.29999 .50446 .30864 .53058 .32261 .54437 .31389 .51795 Metetra
.047 .166 .666 r
.31389 .51795 .32261 .54437 .33667 .56113 .32784 .53501 Metetra
.916 .68 .568 r
.32784 .53501 .33667 .56113 .36033 .41404 .35002 .42007 Metetra
.747 .292 .172 r
.35002 .42007 .36033 .41404 .37417 .43147 .36391 .43764 Metetra
.685 .205 .105 r
.36391 .43764 .37417 .43147 .38852 .44494 .37835 .45065 Metetra
.6 .088 .005 r
.37835 .45065 .38852 .44494 .4033 .45467 .39326 .45941 Metetra
.473 0 0 r
.39326 .45941 .4033 .45467 .41846 .46096 .40857 .46434 Metetra
.281 0 0 r
.40857 .46434 .41846 .46096 .43393 .4642 .42421 .46594 Metetra
.01 0 0 r
.42421 .46594 .43393 .4642 .44966 .4648 .44009 .46475 Metetra
.285 .785 .847 r
.44009 .46475 .44966 .4648 .4656 .4632 .45618 .46131 Metetra
.504 .89 .959 r
.45618 .46131 .4656 .4632 .48171 .4598 .47242 .45613 Metetra
.626 .906 .984 r
.47242 .45613 .48171 .4598 .49796 .455 .48879 .44966 Metetra
.684 .885 .973 r
.48879 .44966 .49796 .455 .51432 .44914 .50526 .44231 Metetra
.708 .856 .953 r
.50526 .44231 .51432 .44914 .53078 .44252 .52182 .43439 Metetra
.716 .828 .934 r
.52182 .43439 .53078 .44252 .54734 .43538 .53846 .42616 Metetra
.717 .805 .919 r
.53846 .42616 .54734 .43538 .56398 .42792 .55517 .41783 Metetra
.713 .786 .909 r
.55517 .41783 .56398 .42792 .58071 .4203 .57197 .40951 Metetra
.708 .771 .901 r
.57197 .40951 .58071 .4203 .59752 .41263 .58885 .40132 Metetra
.702 .76 .897 r
.58885 .40132 .59752 .41263 .61442 .40499 .60582 .39331 Metetra
.696 .752 .895 r
.60582 .39331 .61442 .40499 .63142 .39743 .62289 .3855 Metetra
.69 .746 .894 r
.62289 .3855 .63142 .39743 .64851 .38999 .64005 .37789 Metetra
.685 .742 .895 r
.64005 .37789 .64851 .38999 .66571 .38268 .65732 .37048 Metetra
.68 .74 .896 r
.65732 .37048 .66571 .38268 .68301 .3755 .6747 .36326 Metetra
.676 .739 .898 r
.6747 .36326 .68301 .3755 .70042 .36844 .69219 .35619 Metetra
.672 .739 .9 r
.69219 .35619 .70042 .36844 .71794 .36149 .70979 .34924 Metetra
.669 .739 .903 r
.70979 .34924 .71794 .36149 .73558 .35464 .72751 .34241 Metetra
.667 .74 .905 r
.72751 .34241 .73558 .35464 .75334 .34787 .74536 .33565 Metetra
.749 .811 .901 r
.11706 .53302 .12777 .5401 .14224 .53175 .13164 .52434 Metetra
.758 .805 .89 r
.13164 .52434 .14224 .53175 .15679 .52316 .14633 .51507 Metetra
.762 .792 .878 r
.14633 .51507 .15679 .52316 .17139 .51457 .16112 .50539 Metetra
.757 .77 .864 r
.16112 .50539 .17139 .51457 .186 .50631 .17596 .49557 Metetra
.743 .741 .85 r
.17596 .49557 .186 .50631 .20058 .49876 .19081 .48605 Metetra
.719 .703 .835 r
.19081 .48605 .20058 .49876 .2151 .4924 .20562 .47733 Metetra
.684 .659 .819 r
.20562 .47733 .2151 .4924 .22953 .48769 .22033 .47 Metetra
.638 .607 .803 r
.22033 .47 .22953 .48769 .24385 .48512 .23491 .46471 Metetra
.58 .55 .786 r
.23491 .46471 .24385 .48512 .25804 .48516 .24932 .46209 Metetra
.512 .488 .767 r
.24932 .46209 .25804 .48516 .27211 .48821 .26356 .46272 Metetra
.431 .42 .746 r
.26356 .46272 .27211 .48821 .28608 .49457 .27763 .4671 Metetra
.337 .346 .722 r
.27763 .4671 .28608 .49457 .29999 .50446 .29156 .4756 Metetra
.227 .265 .693 r
.29156 .4756 .29999 .50446 .31389 .51795 .3054 .48842 Metetra
.101 .177 .657 r
.3054 .48842 .31389 .51795 .32784 .53501 .31921 .5056 Metetra
.882 .661 .598 r
.31921 .5056 .32784 .53501 .35002 .42007 .33928 .42935 Metetra
.729 .293 .214 r
.33928 .42935 .35002 .42007 .36391 .43764 .35324 .4469 Metetra
.661 .201 .15 r
.35324 .4469 .36391 .43764 .37835 .45065 .36782 .4591 Metetra
.566 .075 .049 r
.36782 .4591 .37835 .45065 .39326 .45941 .38291 .46638 Metetra
.423 0 0 r
.38291 .46638 .39326 .45941 .40857 .46434 .39842 .46933 Metetra
.196 0 0 r
.39842 .46933 .40857 .46434 .42421 .46594 .41425 .46865 Metetra
.41425 .46865 .42421 .46594 .44009 .46475 .43033 .46505 Metetra
.438 .876 .88 r
.43033 .46505 .44009 .46475 .45618 .46131 .44659 .45923 Metetra
.627 .935 .964 r
.44659 .45923 .45618 .46131 .47242 .45613 .46299 .45183 Metetra
.708 .915 .966 r
.46299 .45183 .47242 .45613 .48879 .44966 .4795 .44337 Metetra
.736 .875 .945 r
.4795 .44337 .48879 .44966 .50526 .44231 .49608 .43432 Metetra
.741 .838 .924 r
.49608 .43432 .50526 .44231 .52182 .43439 .51273 .42501 Metetra
.737 .807 .907 r
.51273 .42501 .52182 .43439 .53846 .42616 .52946 .41568 Metetra
.729 .783 .896 r
.52946 .41568 .53846 .42616 .55517 .41783 .54625 .40652 Metetra
.72 .765 .889 r
.54625 .40652 .55517 .41783 .57197 .40951 .56312 .39763 Metetra
.711 .752 .885 r
.56312 .39763 .57197 .40951 .58885 .40132 .58007 .38905 Metetra
.702 .744 .884 r
.58007 .38905 .58885 .40132 .60582 .39331 .59711 .38081 Metetra
.694 .738 .884 r
.59711 .38081 .60582 .39331 .62289 .3855 .61424 .37288 Metetra
.688 .735 .887 r
.61424 .37288 .62289 .3855 .64005 .37789 .63148 .36525 Metetra
.682 .733 .89 r
.63148 .36525 .64005 .37789 .65732 .37048 .64882 .35786 Metetra
.677 .733 .893 r
.64882 .35786 .65732 .37048 .6747 .36326 .66628 .35069 Metetra
.672 .734 .896 r
.66628 .35069 .6747 .36326 .69219 .35619 .68385 .34369 Metetra
.669 .735 .9 r
.68385 .34369 .69219 .35619 .70979 .34924 .70154 .33682 Metetra
.666 .737 .903 r
.70154 .33682 .70979 .34924 .72751 .34241 .71935 .33004 Metetra
.664 .739 .906 r
.71935 .33004 .72751 .34241 .74536 .33565 .73728 .32333 Metetra
.748 .829 .914 r
.10611 .52662 .11706 .53302 .13164 .52434 .12074 .51805 Metetra
.767 .832 .902 r
.12074 .51805 .13164 .52434 .14633 .51507 .13554 .50852 Metetra
.781 .827 .888 r
.13554 .50852 .14633 .51507 .16112 .50539 .15049 .49809 Metetra
.788 .812 .872 r
.15049 .49809 .16112 .50539 .17596 .49557 .16556 .48693 Metetra
.784 .786 .854 r
.16556 .48693 .17596 .49557 .19081 .48605 .18071 .47541 Metetra
.769 .749 .835 r
.18071 .47541 .19081 .48605 .20562 .47733 .19585 .46406 Metetra
.74 .703 .816 r
.19585 .46406 .20562 .47733 .22033 .47 .21094 .45357 Metetra
.699 .649 .797 r
.21094 .45357 .22033 .47 .23491 .46471 .22589 .44476 Metetra
.645 .589 .777 r
.22589 .44476 .23491 .46471 .24932 .46209 .24064 .4385 Metetra
.578 .523 .756 r
.24064 .4385 .24932 .46209 .26356 .46272 .25515 .43566 Metetra
.5 .451 .734 r
.25515 .43566 .26356 .46272 .27763 .4671 .26942 .43702 Metetra
.407 .374 .71 r
.26942 .43702 .27763 .4671 .29156 .4756 .28343 .44323 Metetra
.298 .289 .681 r
.28343 .44323 .29156 .4756 .3054 .48842 .29725 .45472 Metetra
.169 .196 .646 r
.29725 .45472 .3054 .48842 .31921 .5056 .31093 .47167 Metetra
.76 .583 .657 r
.31093 .47167 .31921 .5056 .33928 .42935 .32797 .44332 Metetra
.702 .288 .258 r
.32797 .44332 .33928 .42935 .35324 .4469 .34205 .46055 Metetra
.625 .19 .195 r
.34205 .46055 .35324 .4469 .36782 .4591 .35683 .47128 Metetra
.515 .052 .091 r
.35683 .47128 .36782 .4591 .38291 .46638 .37217 .47619 Metetra
.344 0 0 r
.37217 .47619 .38291 .46638 .39842 .46933 .38795 .47615 Metetra
.06 0 0 r
.38795 .47615 .39842 .46933 .41425 .46865 .40406 .47219 Metetra
.324 .797 .708 r
.40406 .47219 .41425 .46865 .43033 .46505 .42038 .46528 Metetra
.621 .956 .908 r
.42038 .46528 .43033 .46505 .44659 .45923 .43687 .45635 Metetra
.743 .949 .946 r
.43687 .45635 .44659 .45923 .46299 .45183 .45345 .44618 Metetra
.774 .895 .929 r
.45345 .44618 .46299 .45183 .4795 .44337 .47009 .43541 Metetra
.772 .844 .906 r
.47009 .43541 .4795 .44337 .49608 .43432 .4868 .42451 Metetra
.76 .804 .888 r
.4868 .42451 .49608 .43432 .51273 .42501 .50355 .41381 Metetra
.745 .774 .876 r
.50355 .41381 .51273 .42501 .52946 .41568 .52035 .40351 Metetra
.731 .753 .87 r
.52035 .40351 .52946 .41568 .54625 .40652 .53722 .39372 Metetra
.718 .739 .868 r
.53722 .39372 .54625 .40652 .56312 .39763 .55415 .38447 Metetra
.707 .73 .869 r
.55415 .38447 .56312 .39763 .58007 .38905 .57117 .37575 Metetra
.697 .725 .872 r
.57117 .37575 .58007 .38905 .59711 .38081 .58828 .36751 Metetra
.689 .724 .876 r
.58828 .36751 .59711 .38081 .61424 .37288 .60549 .35968 Metetra
.682 .724 .882 r
.60549 .35968 .61424 .37288 .63148 .36525 .6228 .35219 Metetra
.676 .726 .887 r
.6228 .35219 .63148 .36525 .64882 .35786 .64022 .34496 Metetra
.671 .728 .893 r
.64022 .34496 .64882 .35786 .66628 .35069 .65777 .33795 Metetra
.667 .731 .897 r
.65777 .33795 .66628 .35069 .68385 .34369 .67542 .33109 Metetra
.665 .734 .902 r
.67542 .33109 .68385 .34369 .70154 .33682 .6932 .32433 Metetra
.662 .737 .905 r
.6932 .32433 .70154 .33682 .71935 .33004 .7111 .31764 Metetra
.661 .74 .909 r
.7111 .31764 .71935 .33004 .73728 .32333 .72912 .31098 Metetra
.733 .837 .929 r
.09497 .52049 .10611 .52662 .12074 .51805 .10957 .5126 Metetra
.76 .854 .92 r
.10957 .5126 .12074 .51805 .13554 .50852 .12439 .50351 Metetra
.787 .864 .906 r
.12439 .50351 .13554 .50852 .15049 .49809 .13942 .49305 Metetra
.81 .861 .887 r
.13942 .49305 .15049 .49809 .16556 .48693 .15467 .48118 Metetra
.822 .844 .865 r
.15467 .48118 .16556 .48693 .18071 .47541 .17009 .46804 Metetra
.821 .812 .841 r
.17009 .46804 .18071 .47541 .19585 .46406 .18562 .45405 Metetra
.804 .766 .817 r
.18562 .45405 .19585 .46406 .21094 .45357 .20116 .43991 Metetra
.77 .708 .794 r
.20116 .43991 .21094 .45357 .22589 .44476 .21662 .42661 Metetra
.72 .641 .769 r
.21662 .42661 .22589 .44476 .24064 .4385 .23189 .41532 Metetra
.656 .569 .745 r
.23189 .41532 .24064 .4385 .25515 .43566 .24687 .40735 Metetra
.578 .492 .72 r
.24687 .40735 .25515 .43566 .26942 .43702 .26149 .40396 Metetra
.487 .41 .695 r
.26149 .40396 .26942 .43702 .28343 .44323 .27574 .40629 Metetra
.38 .321 .666 r
.27574 .40629 .28343 .44323 .29725 .45472 .28962 .41518 Metetra
.252 .224 .632 r
.28962 .41518 .29725 .45472 .31093 .47167 .3032 .43115 Metetra
0 0 .479 r
.3032 .43115 .31093 .47167 .32797 .44332 .31586 .46456 Metetra
.663 .275 .301 r
.31586 .46456 .32797 .44332 .34205 .46055 .33014 .48079 Metetra
.571 .167 .237 r
.33014 .48079 .34205 .46055 .35683 .47128 .34524 .48875 Metetra
.436 .008 .125 r
.34524 .48875 .35683 .47128 .37217 .47619 .36097 .48962 Metetra
.212 0 0 r
.36097 .48962 .37217 .47619 .38795 .47615 .37716 .48485 Metetra
.37716 .48485 .38795 .47615 .40406 .47219 .39364 .47597 Metetra
.598 .943 .782 r
.39364 .47597 .40406 .47219 .42038 .46528 .41029 .46441 Metetra
.793 .981 .902 r
.41029 .46441 .42038 .46528 .43687 .45635 .42704 .4514 Metetra
.823 .91 .898 r
.42704 .4514 .43687 .45635 .45345 .44618 .44382 .4379 Metetra
.808 .84 .875 r
.44382 .4379 .45345 .44618 .47009 .43541 .46062 .42459 Metetra
.784 .789 .857 r
.46062 .42459 .47009 .43541 .4868 .42451 .47744 .41191 Metetra
.76 .753 .847 r
.47744 .41191 .4868 .42451 .50355 .41381 .49428 .40008 Metetra
.74 .73 .843 r
.49428 .40008 .50355 .41381 .52035 .40351 .51115 .38919 Metetra
.722 .716 .844 r
.51115 .38919 .52035 .40351 .53722 .39372 .52808 .37921 Metetra
.708 .709 .849 r
.52808 .37921 .53722 .39372 .55415 .38447 .54508 .37003 Metetra
.696 .707 .856 r
.54508 .37003 .55415 .38447 .57117 .37575 .56217 .36155 Metetra
.686 .708 .865 r
.56217 .36155 .57117 .37575 .58828 .36751 .57935 .35363 Metetra
.679 .712 .874 r
.57935 .35363 .58828 .36751 .60549 .35968 .59663 .34614 Metetra
.673 .717 .882 r
.59663 .34614 .60549 .35968 .6228 .35219 .61402 .33897 Metetra
.668 .722 .89 r
.61402 .33897 .6228 .35219 .64022 .34496 .63153 .33203 Metetra
.665 .728 .896 r
.63153 .33203 .64022 .34496 .65777 .33795 .64916 .32524 Metetra
.662 .733 .902 r
.64916 .32524 .65777 .33795 .67542 .33109 .66691 .31854 Metetra
.66 .737 .906 r
.66691 .31854 .67542 .33109 .6932 .32433 .68477 .3119 Metetra
.659 .74 .91 r
.68477 .3119 .6932 .32433 .7111 .31764 .70276 .30528 Metetra
.658 .743 .912 r
.70276 .30528 .7111 .31764 .72912 .31098 .72087 .29866 Metetra
.7 .823 .941 r
.08375 .5138 .09497 .52049 .10957 .5126 .09825 .5071 Metetra
.727 .855 .941 r
.09825 .5071 .10957 .5126 .12439 .50351 .11295 .49933 Metetra
.763 .887 .933 r
.11295 .49933 .12439 .50351 .13942 .49305 .12792 .49005 Metetra
.803 .91 .914 r
.12792 .49005 .13942 .49305 .15467 .48118 .14318 .47882 Metetra
.84 .915 .886 r
.14318 .47882 .15467 .48118 .17009 .46804 .15875 .46534 Metetra
.866 .898 .854 r
.15875 .46534 .17009 .46804 .18562 .45405 .1746 .44957 Metetra
.873 .858 .823 r
.1746 .44957 .18562 .45405 .20116 .43991 .19065 .43189 Metetra
.855 .797 .794 r
.19065 .43189 .20116 .43991 .21662 .42661 .20678 .41323 Metetra
.813 .722 .764 r
.20678 .41323 .21662 .42661 .23189 .41532 .22281 .39506 Metetra
.751 .637 .735 r
.22281 .39506 .23189 .41532 .24687 .40735 .23857 .37928 Metetra
.673 .549 .706 r
.23857 .37928 .24687 .40735 .26149 .40396 .2539 .36801 Metetra
.581 .459 .678 r
.2539 .36801 .26149 .40396 .27574 .40629 .26866 .36332 Metetra
.475 .365 .648 r
.26866 .36332 .27574 .40629 .28962 .41518 .28281 .36699 Metetra
.349 .263 .615 r
.28281 .36699 .28962 .41518 .3032 .43115 .29638 .38027 Metetra
.937 .668 .197 r
.29638 .38027 .3032 .43115 .31586 .46456 .30247 .49816 Metetra
.606 .25 .341 r
.30247 .49816 .31586 .46456 .33014 .48079 .31715 .51162 Metetra
.489 .124 .273 r
.31715 .51162 .33014 .48079 .34524 .48875 .33283 .51394 Metetra
.307 0 .142 r
.33283 .51394 .34524 .48875 .36097 .48962 .34923 .50738 Metetra
.34923 .50738 .36097 .48962 .37716 .48485 .36605 .49456 Metetra
.55 .861 .563 r
.36605 .49456 .37716 .48485 .39364 .47597 .38308 .47802 Metetra
.858 .995 .818 r
.38308 .47802 .39364 .47597 .41029 .46441 .40016 .45988 Metetra
.88 .904 .842 r
.40016 .45988 .41029 .46441 .42704 .4514 .41722 .44174 Metetra
.84 .813 .823 r
.41722 .44174 .42704 .4514 .44382 .4379 .43422 .42462 Metetra
.799 .751 .809 r
.43422 .42462 .44382 .4379 .46062 .42459 .45116 .40908 Metetra
.765 .712 .803 r
.45116 .40908 .46062 .42459 .47744 .41191 .46806 .3953 Metetra
.738 .69 .805 r
.46806 .3953 .47744 .41191 .49428 .40008 .48496 .38321 Metetra
.716 .679 .813 r
.48496 .38321 .49428 .40008 .51115 .38919 .50188 .37261 Metetra
.7 .677 .824 r
.50188 .37261 .51115 .38919 .52808 .37921 .51886 .36322 Metetra
.688 .681 .838 r
.51886 .36322 .52808 .37921 .54508 .37003 .53592 .35478 Metetra
.678 .689 .853 r
.53592 .35478 .54508 .37003 .56217 .36155 .55307 .34701 Metetra
.672 .699 .867 r
.55307 .34701 .56217 .36155 .57935 .35363 .57032 .33974 Metetra
.667 .709 .88 r
.57032 .33974 .57935 .35363 .59663 .34614 .58769 .33278 Metetra
.663 .719 .89 r
.58769 .33278 .59663 .34614 .61402 .33897 .60516 .32601 Metetra
.661 .727 .898 r
.60516 .32601 .61402 .33897 .63153 .33203 .62276 .31936 Metetra
.659 .734 .905 r
.62276 .31936 .63153 .33203 .64916 .32524 .64047 .31277 Metetra
.658 .739 .909 r
.64047 .31277 .64916 .32524 .66691 .31854 .6583 .30619 Metetra
.658 .742 .912 r
.6583 .30619 .66691 .31854 .68477 .3119 .67626 .29961 Metetra
.657 .745 .915 r
.67626 .29961 .68477 .3119 .70276 .30528 .69433 .293 Metetra
.657 .747 .916 r
.69433 .293 .70276 .30528 .72087 .29866 .71253 .28636 Metetra
.666 .784 .937 r
.0726 .5056 .08375 .5138 .09825 .5071 .08698 .50002 Metetra
.678 .814 .948 r
.08698 .50002 .09825 .5071 .11295 .49933 .10151 .49398 Metetra
.7 .859 .958 r
.10151 .49398 .11295 .49933 .12792 .49005 .11623 .48705 Metetra
.734 .916 .955 r
.11623 .48705 .12792 .49005 .14318 .47882 .13123 .47855 Metetra
.78 .965 .923 r
.13123 .47855 .14318 .47882 .15875 .46534 .14659 .46759 Metetra
.834 .988 .867 r
.14659 .46759 .15875 .46534 .1746 .44957 .16242 .45314 Metetra
.891 .98 .815 r
.16242 .45314 .1746 .44957 .19065 .43189 .17875 .43441 Metetra
.932 .94 .78 r
.17875 .43441 .19065 .43189 .20678 .41323 .19554 .41127 Metetra
.93 .862 .753 r
.19554 .41127 .20678 .41323 .22281 .39506 .21262 .3847 Metetra
.88 .759 .723 r
.21262 .3847 .22281 .39506 .23857 .37928 .2297 .35701 Metetra
.799 .646 .692 r
.2297 .35701 .23857 .37928 .2539 .36801 .2464 .33171 Metetra
.7 .535 .66 r
.2464 .33171 .2539 .36801 .26866 .36332 .26239 .31298 Metetra
.59 .427 .629 r
.26239 .31298 .26866 .36332 .28281 .36699 .27738 .30494 Metetra
.463 .317 .596 r
.27738 .30494 .28281 .36699 .29638 .38027 .29128 .3109 Metetra
.984 .716 .339 r
.29128 .3109 .29638 .38027 .30247 .49816 .28674 .55611 Metetra
.52 .205 .373 r
.28674 .55611 .30247 .49816 .31715 .51162 .30234 .56157 Metetra
.354 .042 .291 r
.30234 .56157 .31715 .51162 .33283 .51394 .31927 .55052 Metetra
.058 0 .106 r
.31927 .55052 .33283 .51394 .34923 .50738 .33695 .5284 Metetra
.33695 .5284 .34923 .50738 .36605 .49456 .35487 .50073 Metetra
.933 .968 .697 r
.35487 .50073 .36605 .49456 .38308 .47802 .37271 .47201 Metetra
.921 .849 .756 r
.37271 .47201 .38308 .47802 .40016 .45988 .39033 .44522 Metetra
.846 .74 .747 r
.39033 .44522 .40016 .45988 .41722 .44174 .40768 .42194 Metetra
.786 .674 .741 r
.40768 .42194 .41722 .44174 .43422 .42462 .42481 .40259 Metetra
.742 .638 .744 r
.42481 .40259 .43422 .42462 .45116 .40908 .44179 .3869 Metetra
.712 .623 .756 r
.44179 .3869 .45116 .40908 .46806 .3953 .4587 .37422 Metetra
.691 .623 .775 r
.4587 .37422 .46806 .3953 .48496 .38321 .4756 .36381 Metetra
.677 .634 .799 r
.4756 .36381 .48496 .38321 .50188 .37261 .49254 .35499 Metetra
.669 .652 .824 r
.49254 .35499 .50188 .37261 .51886 .36322 .50956 .34721 Metetra
.664 .673 .848 r
.50956 .34721 .51886 .36322 .53592 .35478 .52667 .34006 Metetra
.661 .693 .869 r
.52667 .34006 .53592 .35478 .55307 .34701 .54389 .33327 Metetra
.659 .71 .886 r
.54389 .33327 .55307 .34701 .57032 .33974 .56122 .32666 Metetra
.658 .723 .897 r
.56122 .32666 .57032 .33974 .58769 .33278 .57866 .32014 Metetra
.657 .733 .905 r
.57866 .32014 .58769 .33278 .60516 .32601 .59622 .31363 Metetra
.657 .74 .911 r
.59622 .31363 .60516 .32601 .62276 .31936 .6139 .30712 Metetra
.657 .744 .914 r
.6139 .30712 .62276 .31936 .64047 .31277 .63169 .30057 Metetra
.657 .747 .916 r
.63169 .30057 .64047 .31277 .6583 .30619 .64961 .29399 Metetra
.657 .748 .917 r
.64961 .29399 .6583 .30619 .67626 .29961 .66765 .28737 Metetra
.657 .749 .918 r
.66765 .28737 .67626 .29961 .69433 .293 .68581 .2807 Metetra
.657 .75 .918 r
.68581 .2807 .69433 .293 .71253 .28636 .70409 .27399 Metetra
.657 .754 .922 r
.06151 .49604 .0726 .5056 .08698 .50002 .07589 .49072 Metetra
.657 .761 .926 r
.07589 .49072 .08698 .50002 .10151 .49398 .09036 .48536 Metetra
.658 .778 .937 r
.09036 .48536 .10151 .49398 .11623 .48705 .10493 .47993 Metetra
.659 .815 .958 r
.10493 .47993 .11623 .48705 .13123 .47855 .1196 .47433 Metetra
.656 .883 .981 r
.1196 .47433 .13123 .47855 .14659 .46759 .13442 .46821 Metetra
.613 .948 .93 r
.13442 .46821 .14659 .46759 .16242 .45314 .1495 .46066 Metetra
.496 .876 .675 r
.1495 .46066 .16242 .45314 .17875 .43441 .16506 .44977 Metetra
.16506 .44977 .17875 .43441 .19554 .41127 .18142 .43234 Metetra
.655 .838 .427 r
.18142 .43234 .19554 .41127 .21262 .3847 .19891 .40457 Metetra
.912 .926 .546 r
.19891 .40457 .21262 .3847 .2297 .35701 .21762 .36423 Metetra
.986 .857 .628 r
.21762 .36423 .2297 .35701 .2464 .33171 .23713 .31354 Metetra
.892 .696 .631 r
.23713 .31354 .2464 .33171 .26239 .31298 .25648 .26078 Metetra
.754 .541 .607 r
.25648 .26078 .26239 .31298 .27738 .30494 .27447 .21853 Metetra
.607 .401 .576 r
.27447 .21853 .27738 .30494 .29128 .3109 .29015 .19954 Metetra
.99 .731 .38 r
.29015 .19954 .29128 .3109 .28674 .55611 .26503 .67906 Metetra
.373 .113 .385 r
.26503 .67906 .28674 .55611 .30234 .56157 .28375 .65308 Metetra
.063 0 .245 r
.28375 .65308 .30234 .56157 .31927 .55052 .30438 .59895 Metetra
.30438 .59895 .31927 .55052 .33695 .5284 .32515 .53652 Metetra
.994 .868 .594 r
.32515 .53652 .33695 .5284 .35487 .50073 .34505 .48013 Metetra
.879 .697 .652 r
.34505 .48013 .35487 .50073 .37271 .47201 .36382 .43629 Metetra
.769 .586 .651 r
.36382 .43629 .37271 .47201 .39033 .44522 .38164 .40541 Metetra
.7 .532 .656 r
.38164 .40541 .39033 .44522 .40768 .42194 .39884 .38482 Metetra
.661 .515 .674 r
.39884 .38482 .40768 .42194 .42481 .40259 .41572 .37107 Metetra
.642 .526 .706 r
.41572 .37107 .42481 .40259 .44179 .3869 .4325 .36129 Metetra
.638 .558 .749 r
.4325 .36129 .44179 .3869 .4587 .37422 .44928 .35353 Metetra
.642 .603 .795 r
.44928 .35353 .4587 .37422 .4756 .36381 .46614 .34668 Metetra
.647 .649 .838 r
.46614 .34668 .4756 .36381 .49254 .35499 .48309 .34018 Metetra
.652 .688 .871 r
.48309 .34018 .49254 .35499 .50956 .34721 .50015 .3338 Metetra
.655 .714 .892 r
.50015 .3338 .50956 .34721 .52667 .34006 .51733 .32742 Metetra
.656 .731 .905 r
.51733 .32742 .52667 .34006 .54389 .33327 .53461 .32102 Metetra
.656 .74 .912 r
.53461 .32102 .54389 .33327 .56122 .32666 .55202 .31458 Metetra
.657 .745 .915 r
.55202 .31458 .56122 .32666 .57866 .32014 .56954 .3081 Metetra
.657 .748 .917 r
.56954 .3081 .57866 .32014 .59622 .31363 .58718 .30157 Metetra
.657 .749 .918 r
.58718 .30157 .59622 .31363 .6139 .30712 .60493 .295 Metetra
.657 .75 .918 r
.60493 .295 .6139 .30712 .63169 .30057 .62281 .28839 Metetra
.657 .75 .919 r
.62281 .28839 .63169 .30057 .64961 .29399 .64081 .28173 Metetra
.657 .75 .919 r
.64081 .28173 .64961 .29399 .66765 .28737 .65893 .27502 Metetra
.657 .75 .919 r
.65893 .27502 .66765 .28737 .68581 .2807 .67718 .26827 Metetra
.657 .75 .919 r
.67718 .26827 .68581 .2807 .70409 .27399 .69555 .26148 Metetra
.657 .75 .919 r
.05032 .4862 .06151 .49604 .07589 .49072 .06475 .48082 Metetra
.657 .75 .919 r
.06475 .48082 .07589 .49072 .09036 .48536 .07927 .4754 Metetra
.657 .75 .919 r
.07927 .4754 .09036 .48536 .10493 .47993 .09387 .46996 Metetra
.657 .751 .919 r
.09387 .46996 .10493 .47993 .1196 .47433 .10857 .46447 Metetra
.657 .755 .922 r
.10857 .46447 .1196 .47433 .13442 .46821 .12335 .45896 Metetra
.657 .77 .932 r
.12335 .45896 .13442 .46821 .1495 .46066 .13823 .45341 Metetra
.653 .816 .962 r
.13823 .45341 .1495 .46066 .16506 .44977 .1532 .44783 Metetra
.573 .908 .976 r
.1532 .44783 .16506 .44977 .18142 .43234 .16827 .44221 Metetra
.008 0 0 r
.16827 .44221 .18142 .43234 .19891 .40457 .18343 .43656 Metetra
.348 0 .205 r
.18343 .43656 .19891 .40457 .21762 .36423 .19868 .43087 Metetra
.431 .131 .354 r
.19868 .43087 .21762 .36423 .23713 .31354 .21403 .42515 Metetra
.459 .184 .408 r
.21403 .42515 .23713 .31354 .25648 .26078 .22948 .41939 Metetra
.471 .207 .431 r
.22948 .41939 .25648 .26078 .27447 .21853 .24502 .41359 Metetra
.439 .185 .43 r
.24502 .41359 .27447 .21853 .29015 .19954 .26193 .39377 Metetra
.804 .516 .501 r
.26193 .39377 .29015 .19954 .26503 .67906 .24858 .72693 Metetra
.989 .748 .473 r
.24858 .72693 .26503 .67906 .28375 .65308 .29225 .39598 Metetra
.514 .293 .516 r
.29225 .39598 .28375 .65308 .30438 .59895 .3082 .39003 Metetra
.517 .3 .524 r
.3082 .39003 .30438 .59895 .32515 .53652 .32425 .38404 Metetra
.524 .315 .538 r
.32425 .38404 .32515 .53652 .34505 .48013 .3404 .37802 Metetra
.536 .341 .563 r
.3404 .37802 .34505 .48013 .36382 .43629 .35666 .37196 Metetra
.556 .386 .606 r
.35666 .37196 .36382 .43629 .38164 .40541 .37302 .36586 Metetra
.585 .457 .673 r
.37302 .36586 .38164 .40541 .39884 .38482 .38949 .35971 Metetra
.617 .55 .757 r
.38949 .35971 .39884 .38482 .41572 .37107 .40607 .35353 Metetra
.641 .64 .834 r
.40607 .35353 .41572 .37107 .4325 .36129 .42276 .34731 Metetra
.652 .7 .882 r
.42276 .34731 .4325 .36129 .44928 .35353 .43955 .34104 Metetra
.656 .731 .905 r
.43955 .34104 .44928 .35353 .46614 .34668 .45646 .33474 Metetra
.657 .743 .914 r
.45646 .33474 .46614 .34668 .48309 .34018 .47348 .32839 Metetra
.657 .748 .917 r
.47348 .32839 .48309 .34018 .50015 .3338 .49062 .322 Metetra
.657 .749 .918 r
.49062 .322 .50015 .3338 .51733 .32742 .50786 .31557 Metetra
.657 .75 .919 r
.50786 .31557 .51733 .32742 .53461 .32102 .52523 .3091 Metetra
.657 .75 .919 r
.52523 .3091 .53461 .32102 .55202 .31458 .54271 .30258 Metetra
.657 .75 .919 r
.54271 .30258 .55202 .31458 .56954 .3081 .56031 .29601 Metetra
.657 .75 .919 r
.56031 .29601 .56954 .3081 .58718 .30157 .57802 .28941 Metetra
.657 .75 .919 r
.57802 .28941 .58718 .30157 .60493 .295 .59586 .28275 Metetra
.657 .75 .919 r
.59586 .28275 .60493 .295 .62281 .28839 .61382 .27606 Metetra
.657 .75 .919 r
.61382 .27606 .62281 .28839 .64081 .28173 .6319 .26931 Metetra
.657 .75 .919 r
.6319 .26931 .64081 .28173 .65893 .27502 .65011 .26252 Metetra
.657 .75 .919 r
.65011 .26252 .65893 .27502 .67718 .26827 .66844 .25569 Metetra
.657 .75 .919 r
.66844 .25569 .67718 .26827 .69555 .26148 .6869 .24881 Metetra
0 g
.25 Mabswid
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
[(80)] .07825 .23514 .98066 1 Mshowa
.22216 .18933 m
.227 .19478 L
s
[(90)] .21247 .17844 .88892 1 Mshowa
.36358 .12987 m
.36813 .13557 L
s
[(100)] .3545 .11847 .79718 1 Mshowa
.51343 .06687 m
.51763 .07282 L
s
[(110)] .50502 .05496 .70545 1 Mshowa
.67245 0 m
.67627 .00621 L
s
[(120)] .66483 -0.01242 .61371 1 Mshowa
.125 Mabswid
.11461 .23456 m
.11764 .23771 L
s
.14105 .22344 m
.14405 .22662 L
s
.16779 .2122 m
.17076 .21541 L
s
.19482 .20083 m
.19776 .20407 L
s
.2498 .17771 m
.25268 .18101 L
s
.27776 .16595 m
.2806 .16928 L
s
.30604 .15406 m
.30885 .15742 L
s
.33465 .14204 m
.33741 .14542 L
s
.39286 .11756 m
.39554 .12101 L
s
.42247 .10511 m
.42512 .10859 L
s
.45243 .09251 m
.45504 .09602 L
s
.48275 .07977 m
.48531 .08331 L
s
.54447 .05381 m
.54695 .05742 L
s
.57589 .0406 m
.57832 .04424 L
s
.60769 .02723 m
.61007 .0309 L
s
.63987 .0137 m
.64221 .0174 L
s
.25 Mabswid
0 .80374 m
.40037 1.0855 L
s
.09485 .87049 m
.09577 .86326 L
s
[(0.2)] .09301 .88495 .12735 -1 Mshowa
.18136 .93137 m
.18209 .92412 L
s
[(0.4)] .17991 .94587 .10002 -1 Mshowa
.26055 .98709 m
.2611 .97983 L
s
[(0.6)] .25944 1.00163 .07584 -1 Mshowa
.3333 1.03829 m
.33369 1.03102 L
s
[(0.8)] .33251 1.05285 .05428 -1 Mshowa
.40037 1.0855 m
.40063 1.07821 L
s
[(1)] .39987 1.10006 .03495 -1 Mshowa
.125 Mabswid
.02453 .821 m
.02518 .81668 L
s
.04852 .83789 m
.04914 .83356 L
s
.07196 .85438 m
.07254 .85004 L
s
.11722 .88623 m
.11774 .88189 L
s
.13908 .90162 m
.13958 .89727 L
s
.16046 .91666 m
.16092 .91231 L
s
.20181 .94576 m
.20221 .9414 L
s
.22181 .95983 m
.22219 .95548 L
s
.24138 .97361 m
.24174 .96925 L
s
.27931 1.0003 m
.27961 .99593 L
s
.29768 1.01322 m
.29796 1.00886 L
s
.31567 1.02589 m
.31593 1.02152 L
s
.35057 1.05045 m
.35079 1.04608 L
s
.3675 1.06236 m
.3677 1.058 L
s
.3841 1.07404 m
.38427 1.06967 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{185.688, 201.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002i0000bA000`40O003h00O^Goo002iOol007Ao
o`<000=oo`@000=oo`8003Ioo`00MGoo00<007ooOol00goo00<007ooOol00Woo00@007ooOol003Eo
o`00MGoo00<007ooOol017oo00D007ooOomoo`00009oo`03001oogoo03=oo`00MGoo00<007ooOol0
1Goo00@007ooOol0009oo`03001oogoo03=oo`00MGoo00<007ooOol00Woo00@007ooOol0009oo`04
001oogoo000eOol007Aoo`8000Eoo`8000Aoo`8003Ioo`00^Goo002iOol00;Uoo`00PWoo0P00=Goo
001oOol300000goo001oo`0dOol005Uoo`<000=oo`<000Aoo`8001Eoo`8000Aoo`8003Aoo`00FWoo
00<007ooOol00goo00<007ooOol00Woo00@007ooOol0019oo`@000Aoo`03001oo`0003=oo`00FWoo
00<007ooOol00goo00<007ooOol00Woo00@007ooOol000moo`<000Qoo`03001oo`0003=oo`00FWoo
00<007ooOol00goo00<007ooOol00Woo00@007ooOol000eoo`<000Yoo`04001oogoo000bOol005Yo
o`03001oogoo00=oo`03001oogoo009oo`04001oogoo000;Ool20003Ool00`00Oomoo`07Ool01000
Oomoo`00<Woo001IOol20004Ool20005Ool20009Ool3000?Ool01@00Oomoogoo0000<Goo001_Ool2
0002Ool00`00Oomoo`0=Ool00`00Oomoo`02Ool00`00Oomoo`0^Ool006eoo`8001Aoo`03001oogoo
009oo`03001oogoo02ioo`00JWoo0`0000=oo`00Ool04goo00<007ooOol00goo00<007ooOol0;Goo
001XOol2000JOol00`00Oomoo`02Ool00`00Oomoo`0]Ool006Ioo`800003Ool007oo01Uoo`03001o
ogoo00=oo`03001oogoo02aoo`00@7oo0`0017oo0P0017oo0P0057oo0`007Woo00<007ooOol017oo
00<007ooOol0:goo0011Ool00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol0015oo`800003
Ool007oo01ioo`03001oogoo00Aoo`03001oogoo02]oo`00@Goo00<007ooOol00Woo00@007ooOol0
009oo`04001oogoo000?Ool2000SOol00`00Oomoo`05Ool00`00Oomoo`0ZOol0045oo`03001oogoo
009oo`04001oogoo0002Ool01000Oomoo`0037oo100097oo00<007ooOol01Goo00<007ooOol0:Woo
0011Ool00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol000Yoo`8002Qoo`03001oogoo00Io
o`03001oogoo02Uoo`00@7oo0P001Goo0P0017oo0P002Goo0`00:Goo00<007ooOol01goo00<007oo
Ool0:7oo001EOol3000/Ool00`00Oomoo`07Ool00`00Oomoo`0XOol005=oo`<002ioo`03001oogoo
00Qoo`03001oogoo02Moo`00DGoo0P00<Goo00<007ooOol027oo00<007ooOol09goo001>Ool3000c
Ool00`00Oomoo`09Ool00`00Oomoo`0VOol002eoo`<000Aoo`8001Ioo`8000=oo`03001oogoo031o
o`03001oogoo00Yoo`03001oogoo02Eoo`00<7oo00@007ooOol0009oo`03001oogoo015oo`<003Mo
o`03001oogoo00Yoo`03001oogoo02Eoo`00;Woo0`000Woo00@007ooOol0011oo`<003Yoo`03001o
ogoo00]oo`03001oogoo02Aoo`00;Goo00@007ooOol0009oo`04001oogoo000>Ool3000lOol00`00
Oomoo`0;Ool00`00Oomoo`0TOol002eoo`04001oogoo0002Ool01000Oomoo`0037oo0P00?goo00<0
07ooOol037oo00<007ooOol08goo000^Ool20004Ool2000:Ool40010Ool00`00Oomoo`0=Ool00`00
Oomoo`0ROol003ioo`8004Aoo`03001oogoo00eoo`03001oogoo029oo`00?7oo0`00AWoo00<007oo
Ool03Goo00<007ooOol08Goo000iOol30019Ool00`00Oomoo`0=Ool00`00Oomoo`0QOol001Qoo`80
00Aoo`8001Moo`800003Ool007oo04Uoo`03001oogoo00ioo`03001oogoo021oo`005goo00@007oo
Ool0009oo`04001oogoo000DOol2000=Ool00`00Oomoo`0nOol00`00Oomoo`0?Ool00`00Oomoo`0O
Ool001Moo`04001oogoo0002Ool01000Oomoo`004Goo10003Woo00<007ooOol0?Woo00<007ooOol0
3goo00<007ooOol07goo000HOol20003Ool01000Oomoo`003goo0P004Goo0P00@7oo00<007ooOol0
47oo00<007ooOol07Woo000GOol01000Oomoo`000Woo00@007ooOol000eoo`800003Ool007oo00mo
o`03000h[@00041oo`03001oogoo011oo`03001oogoo01ioo`0067oo0P0017oo0P002goo0`0057oo
00<003R]0000@7oo00<007ooOol04Goo00<007ooOol07Goo000YOol2000FOol40010Ool00`00Oomo
o`0BOol00`00Oomoo`0LOol002Moo`8001Qoo`@0041oo`03001oogoo019oo`03001oogoo01aoo`00
97oo0`006Goo0P0000<h[@000000@7oo00<007ooOol04goo00<007ooOol06goo000SOol2000KOol2
00000cR]0000000oOol2000EOol00`00Oomoo`0KOol002=oo`8001]oo`800003>:d0000003aoo`<0
0003E_d007oo01Eoo`03001oogoo01Yoo`004Goo0P0017oo0P000goo0`0017oo00<007oo00006Goo
00@003c=000h[@8003Uoo`<00004E_d005KmE_d2000FOol00`00Oomoo`0IOol0011oo`04001oogoo
0003Ool20006Ool01@00Oomoogoo00000Woo00<007ooOol05Woo00@003c=000h[@8003Ioo`<0009F
o@03001Fo@0000<00003E_d007oo01Aoo`03001oogoo01Uoo`002Woo1@0000Eoo`00Oomoo`0000]o
o`03001oogoo008000=oo`03001oogoo01Aoo`800003?<d003R]008003Eoo`04001FoEKm0002E_d2
0002E_d200000eKm0000000EOol00`00Oomoo`0HOol0011oo`04001oogoo0008Ool30003Ool20004
Ool00`00Oomoo`0COol200000cc=00000002000cOol20003E_d3000015Km001FoEKm0`0000=Fo@00
Ool057oo00<007ooOol067oo000@Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol017oo
00<007ooOol04Woo00@003c=000h[@<0031oo`<0009Fo@05001Fo@00E_d00002E_d3000015Km001F
oEKm0P005Goo00<007ooOol05goo000AOol20009Ool40002Ool00`00Oomoo`05Ool00`00Oomoo`0@
Ool200000cc=000h[@03000]Ool30002E_d00`00E_eFo@020002E_d3000015Km001FoEKm0`000UKm
00<007ooOol04goo00<007ooOol05Woo000ROol20007Ool00`00Oomoo`0?Ool200000cc=000h[@03
000ZOol30002E_d00`00E_eFo@0300001UKm001Fo@00E_d0009Fo@<00003E_d00000009Fo@8001Ao
o`03001oogoo01Ioo`008Woo00<007ooOol01goo00<007ooOol03Woo0P0000<lc@00>:d00`00:7oo
0P000UKm00<005KmE_d00`0000AFo@00E_eFo@80009Fo@<0009Fo@03001Fo@0000<00003E_d007oo
01Aoo`03001oogoo01Eoo`008Woo00<007ooOol027oo00<007ooOol037oo00H003R]000lc@00>:d3
000UOol3000015Km001FoEKm0`0000AFo@00E_eFo@<00004E_d005Km0002E_d00`00E_eFo@030000
15Km001FoEKm0P0057oo00<007ooOol05Goo000QOol00`00Oomoo`0:Ool00`00Oomoo`0;Ool20002
?<d00`00>:d00002000ROol3000015Km001FoEKm0`0000AFo@00E_eFo@<0009Fo@04001Fo@000003
E_d3000015Kl000005Km0`0000=Fo@00Ool057oo00<007ooOol057oo000QOol2000<Ool00`00Oomo
o`0:Ool2000013c=000h[CR]0`007goo0`0000AFo@00E_eFo@<0009Bo@03001Fo@000080009Fo003
001FoEKm0080009Fo005001Fo@00E_`00002E_`3000015Kl001FoEKm0P005Goo00<007ooOol04goo
000QOol00`00Oomoo`0<Ool00`00Oomoo`08Ool300001Cc=000h[@00IQ000P0077oo0`000UKm00<0
05Km00000P000U;l00<005;mD_d00P000UKl00<005KlE_`00`0000AFo000E_aFo08000=Fo0800004
E_`005KlE_`30002E_`00`00Oomoo`0BOol00`00Oomoo`0COol0025oo`03001oogoo00eoo`03001o
ogoo00Moo`08000h[@00?<d003R]001V408001Uoo`<0009Fo@03001FoEKm0080009Bg003001Bo5;l
00<00004E]`005KlE_`3000015Kl001Fo5Kl0`0000AFo000E_aFo080009Fo0<0009Fo003001Fo5Kl
008001Aoo`03001oogoo019oo`008Goo00<007ooOol03Woo00<007ooOol01Goo0P0000Lh[@00?<d0
03R]001V4002000GOol20002D_`00`00E_eFo@03000015;K001Bg5;L0`0000AFf`00E]aFg0<00004
E]`005KlE_`3000015Kl001Fo5Kl0`000UKl00D005Kl001Fo000009Fo0<00003E_`007oo01=oo`03
001oogoo019oo`008Goo00<007ooOol03goo00<007ooOol017oo0`000Sc=00@003R]001V408001Ao
o`<00004D]`005;lD_`3000015:J001Bfe;K0`0000AF^`00E]]Ff`<0009Ff`03001Fg0000080009F
g003001Fo0000080009Fo003001Fo5Kl00<00006E_`00000001Fo0000UKl0P0057oo00<007ooOol0
4Goo000POol00`00Oomoo`0AOol00`00Oomoo`02Ool01000=8/000000Sc=00@003R]001V40<0011o
o`<0009BVP03001Bg00000800003DWT005:J009BVP800003EYX005Jk009F^`80009Ff`03001FfeKK
0080009Ff`03001Fg5KL0080009Fo003001Fo5Kl00<00006E_`00000001Fo0000UKl0`0000=Fo000
Ool057oo00<007ooOol047oo000POol00`00Oomoo`0BOol01P00Oomoogoo000dR`800006?<d003R]
>:d006H@0`003Woo0P0000=>>000DYX00U:J0P0000=B=`00DWT00U9i00@005:J000000=FVP04001F
^`000003E]/3000015KK001FfeKK0`0000AFo000E_aFo0<00004E_`005KlE_`20002E_`3000015Kl
001Fo5Kl0P0057oo00<007ooOol047oo000POol2000DOol02@00Ool003B;000h[@00?<d00002>:d0
0`00IQ000002000;Ool300002DWE001>>4hh001BVP00DQH00002DSL00`00DWUBN@020002EYT00`00
EYYFVP020002F[X01@00E]/005[K00000UKK0`0000AJo000E_aFo0<00004E_`005KlE_`300001UKl
001Fo000E_`0009Fo0<00003E_d007oo01Aoo`03001oogoo00moo`0027oo0P0017oo0P000goo1000
0Woo0`0017oo00<007ooOol057oo0P0000HdR`00>:d003c=0002>:d00`00IQ0000020008Ool30000
1DFC0019eDWE00000Thh0P000U8F00P0058gDSL0059i001JN0000UJI0`0000AJ^P00F[YJ^P8000=J
f`800004F_/005[lF_`3000015[l001Fo5Kl0`0000AFo000E_aFo08000=Fo080009Fo004001Fo@00
000DOol00`00Oomoo`0?Ool000Moo`04001oogoo0003Ool20004Ool00`00Oomoo`05Ool01@00Oomo
ogoo00005Woo0P0000<dR`0000000Sc=00<003R]00000VH@0`001Goo0`0000E5DP00AI=5T`0000=9
e@800003EQD0058F009B5P04001B=`000003FWP2000015Zj001J^UZj0`0000AJf`00F]]Jf`8000=J
n`80009Jo003001Jo0000080009Fo003001Fo00000<00006E_`005Kl001Fo0000UKl0P000UKm00<0
07ooOol04Woo00<007ooOol03Woo00000goo0000000300001Goo001oogoo00002Woo00<007ooOol0
17oo00D007ooOomoo`0001Ioo`03000dRcB;0080008lc@06000h[@00IQ0007k/0P000goo0P0000E1
D@00AE95DP000095T`050019eDWEBMD00002EQD01000DQIB5U8F0P0000AJN000FWQJN08000=J^P80
0003G]/005[K009Jf`80009Jn`04001Jn`000003F_`20002F_`00`00E_aFo0020002E_`200000eKl
00000002E_`3000015LM001FoEKm0P0057oo00<007ooOol03Goo0007Ool01000Oomoo`002goo00<0
07ooOol00`0017oo00<007ooOol04goo0P0000@dR`00>:d0008lc@06000h[@00IQ0007k/1@0000A1
<000@E400095DP050015TdFCAI<00002BMD01P00EQEF5EHE001B5P8000=JN0800003G]T005Zj009J
^P09001NfekKG]/005[K001Jn`00009Jn`80009Jo005001Jo000Fa`00002F_`3000015LL001Fo5Kl
0`0000IG7000000005LM0002Ead300000eLM001oo`0COol00`00Oomoo`0=Ool000Moo`04001oogoo
0008Ool01000Oomoo`000Woo00<007ooOol00goo00<007ooOol057oo0P0000@dR`00>:d0008lc@06
000h[@00IQ0007k/0P0000M1<000@C11<000@E400002AE801P00AI=5TdFC0019e@8000=F5@800003
GYL005Yh009JN009001NfEkIG]T005Zj001NnP00009Nf`80009Jn`<00004Fa`005[lF_`20002Fa`3
000015/L001G75LL0`0000IG7@00Ea`005LM0002Ead3000015LM001G7ELM0P0057oo00<007ooOol0
37oo0008Ool2000:Ool20003Ool40002Ool00`00Oomoo`0DOol02@00=8/dR`00>:d003c=000h[@02
00000fH@001nk00200001d4`0011<44`0011D@000095DP040015TdFCAI<200000eXd001F5@02EQD0
1P00GYMNUejG001JN08000=Nf@80009NnP<00006Ga/005[k001O70000U/L0`0000IK7000Fa`005/L
0002Fa`3000015LM001G7ELM0P000ULM0`0000AG7@00EaeG7@<00003Ead007oo01=oo`03001oogoo
00aoo`007goo00<007ooOol04goo0P000SB;0P000Sc=0P0000E9m`00IQ0007k/00800007@C00044`
@C00045A00000dEB00@004FCAI<000=J=004001F5@000004GYL03000H]URf@00G]T006<K001NnP00
Ga`0009O6`8000=O70800004Fa`005/LFa`3000015/L000000000ULM00<005LM00001P0000AG7@00
EaeG7@<00004Ead005LMEad2000DOol00`00Oomoo`0;Ool001moo`8001Aoo`80008dR`800003?<d0
00000099m`04001V4000O^`200001d4`0011<44`0011D@0000=5DP040015TdFC0003FS@20002I[L0
0`00GYMNU`020002H]T20002Ha/20003Ga`20002Gc`01P00Ga`005/m001K70<0009K7@80009K7@03
001G7ELM0080009G7@<00003Ead005LM00<0009G7@03001G7@0000800003Ead00000019oo`<000ao
o`007goo00<007ooOol04Woo00@002`6000dR`@0009BE`060019m`00IQ0007k/0P0000M1<000@C11
<000@E400003AE801@00AI<0069D00000UXd00L006JgI[MV]`00GYL00002IaT2000016Lk001S6`00
0V<l00D005lL001O?@00009O?080009K?@80009K?@05001K7@00Fcd00002Fad300001ELM001G7@00
Ead00P0000=G7@0000000ULM00<005LMEad00P000ULM0P000ULM00<007ooOol037oo0`000Woo00<0
07ooOol02Woo000OOol00`00Oomoo`0AOol300000cB;00000002GYP02@00DUMBE`00BOL006H@001n
k00200001d4`0011<44`0011D@0000=5DP80009RE004001J=5Xd0004I[L00`00IaUW6@020003Ic/2
0002Hc`20002Gcd300001U/m00000000Fcd0009K?@80009K?@<00004Ead005LMEad20002Ead30000
15LM001G7ELM0`0000IG7@00000005LM0002Ead2000:Ool30005Ool00`00Oomoo`0:Ool001ioo`03
001oogoo015oo`L000=NV008001BE`00BOL006H@001nk0800005@C00044`@C000002@E400`00AE95
DP020002HU@01@00FS@006kg00000VJg0P0000QW6@00Ie]WF`00Ic/006=l0`0000YOG@00000005]M
001K?@00Feh0009K?@<00006Fcd00000001G?@000ULM0`0000IG7@00000005LM0002Ead300001ULM
00000000Ead0009G7@<00003Ead007oo00Ioo`<000Qoo`03001oogoo00Yoo`007Woo0P004Goo0`00
0Vmj00D006;IH]T005jH00H00003IQ0007k/00800005@C00044`@C000002@E401000AE95DP00169D
0P0000A^m`00I[L0009[FP80009WF`800006Hg`005mn001OG@000U]n00<005]M00000P0000AKGP00
0000009G?P80009G?P03001G?@0000800006Ecd00000001G?@000ULM0`0000IG7@00000005LM0002
Ead3000015LM001G7ELM0P000goo0`002goo00<007ooOol02Woo000NOol00`00Oomoo`0>Ool40002
KgX20002H]T20000159H001:64XH0P0000=V4000O^`00P0000E1<000@C11<0000091D@040015DTEB
0004HU@01000K_M^mfkg0P0000=[FP00Jk`00`0000ESW@00000005mn00<0009KOP80009KGP80009G
GP<00004Eeh005LnEch20002Ech20002Ech00`00Ecd00006000015LM001G7ELM0`0000AG7@00EaeG
7@<00003Ead00000008000moo`03001oogoo00Uoo`007Woo00<007ooOol02Goo2@0000A_NP00IaYW
6P800006FYT0059HDUP004XH0P0000=V4000O^`00P0000E1<000@C11<0000091D@050015DP00L]@0
0003HU@01@00K_M^m`00LiX00P0000A[_000GmiOgP80009K_P80009GW`<00005Egl00000001CG`03
00000e=O0000000400001ULn00000000Dch0009G?P80009G?P80009C7P03001G7@0000800004Dad0
05LMEad300001ULM00000000Dad0015oo`03001oogoo00Uoo`007Woo00<007ooOol01goo0P000ULM
00<005=N00000P000VnK0P0000=W6P0000000eZI00<0059H00000TXH00@006H@001nk0800005@C00
044`@C000002@E401@00AE8007;D00000V9D00L007Mh000007>J001Wo00500001E>n00000000Dil0
0`0000E?W`00000004mo00<0009?G`80009?G`03001CGP0000800003Dch0000000@00006Dch00000
001C?P000U<N0P000U<N1@0000IC7@00000005<M0002Dad2000@Ool00`00Oomoo`09Ool001ioo`03
001oogoo00Eoo`80009G7@03001G7@0000@00003Ki/006/j0080009NfP03001JV@0000H00004IQ00
07k/0002@C001000@C11<0000T5A0P000W;D00<0069D00000WMh5`000T]o0`0000=;G`0000000`00
0TmO0P000Tlo00<005<n00000P0000AC?P00DciC?P<00006Dch00000001C7P000U<N0P000U<N00<0
05<M00000P0000=C7@00Ool03goo00<007ooOol02Goo000NOol00`00Oomoo`02Ool3000015LM001G
7ELM1@0000M[_000001[>P00H_X00002G]X200002e9i001:>DXi0011n000IQ0007k/00000T4`00@0
044`@C000091D@8000=be0800004MgP00000CmT500000g[@001jd00C00000dMO0000000400000dlo
000000030002Cch9000015<n001C7U<N0`0000AC7P00DaiC7P800003Dad0000000<000moo`03001o
ogoo00Uoo`007Goo00@007ooOomoo`80009Fo@03001G7@0000800006Dil006?M001[_0000V]k00@0
06;jH_YRnP80000:E[X0059iDWT004Xi0011n000IQ030002@C001000@C11<0000T5A0P000g;D00<0
07Mh00001@0000Ajd000001jd0800004N]000000N]0@0002@cl00`00Bcl00002000014/o001??dlo
0`0000=??`00Dch00`0000AC?P00DciC?P<00003Dah0000000@00003Dah00000009C7P04001C7@00
000>Ool00`00Oomoo`09Ool001eoo`@0009Fo@03001Fo@0000800003Eeh0000000D00005Jg/006Lk
001RnP020003E[X00`00DWT00002BST00`00IQ1V40030002@C001000@C11<0000T5A00<007;DL]@0
0W;D0P0000Ajd000001jd0800007N]000000001jd000N]000P0000=jd00000000P0000=jd0000000
3@000TLo1@0000=;?`0000000`000Tln1@0000IC7P00000005<N0002Dah3000015<N001C7U<N0P00
00=C7P00Ool03Goo00<007ooOol02Goo000MOol2000015Km001FoEKm0P000ULM1@0000=W_@00Ig`0
0`0000=W>`0000000UkJ00<005Jj00001P000VH@0`000T4`00@0044`@C000091D@04001be7;DL]@3
000017[@000007[@0P0000Mjd000000007[@001jd00200000g[@0000000200000g[@0000000?0000
0d<o0000000300000d/o000000060002Cch2000014lN001C7U<N0`0000AC7P00DaiC7P<00003Dah0
0000008000ioo`03001oogoo00Qoo`005goo0P0017oo00<007oo00000`0000IG7@00000005]n0002
Hid2000016Ml001W>fLk0P000Ukk00H005kJ0000059j0002BUT00`00@QT00002IQ030002@C002@00
@C11<000@E4007mc001be00200000g[A001jd002000017[@000007[@0`0000Ajd000001jd0800005
N]000000001jd00B00000d<o000000030002Acl00`00Bcl0000500000dln0000000300000dlN0000
000400001e<N00000000Dah005<N008000eoo`03001oogoo00Qoo`005Woo00@007ooOol000=oo`05
001oogoo001G7@0300001eMN00000000Hid006Ml00<00004Ic/006<K0002G_/20002E[X00`00DWX0
0002BUT20002IQ030002@C001P00@C11<000@E40009oL`06001fd@00N]4007[@0P0000Ajd000001j
d0<00007N]000000001jd000N]000`0000=jd00000000`0000=jd00000002`0000<o7`0000001P00
00=;?`000000100000A?7P00Cai?7P800004Cah005<NDah3000015<N001C7U<N0P000U<N00<007oo
Ool02Woo00<007ooOol027oo000FOol01000Oomoo`000goo00@007ooOomoo`800005Ecd00000001O
OP040003He`200001V<K00000000F]/0009F^PL00003IQ0007k/00800091<0060011<44`0011D@00
0Wmc00@007KA001jd@800003N]0007[@00800005N]000000001jd00300000g[@001jd00300000g[@
0000000300000g[@0000000<0002>al7000014LO000004/O1P0000=?7P0000000`0000=?7P000000
100000=C7P0000000`002goo00<007ooOol027oo000FOol01000Oomoo`000Woo0P001Goo00<005]N
Feh00`0000ISO@00He`006=L0002Hc`01@00G_]Nn`00F]/00`0000EBVP00BWX004Hj00800003IQ00
07k/00800091<0030011<44`0080009oL`800003M]4007[A00<0009jd0800005N]000000001jd003
000017[@000007[@0P0000=jd00000000`0000=jd00000003P0000<k7`000000200000=;7`000000
100000A?7P00Cai?7P800004Cah004lNCah3000014lN001C7U<N0P002Woo00<007ooOol027oo000F
Ool01000Oomoo`000Woo00<007ooOol01Goo0P000UmM0P000V=L0`0000ES?000Ga/005kk00800003
E[/005:J009BVP06001:NP00ASX006H@0P0000Enk`00@C11<0000091<0800007Og<007KA001fd@00
N]400`0000Ejd000N]0007[@00<00005N]000000001jd00300000g[@001jd00500000g[@0000000?
00000cKo000000080002Aal800000dlN0000000300000dlN0000000400000e<N00000009Ool00`00
Oomoo`08Ool001Moo`8000=oo`03001oogoo00Moo`800004Ged000000002Hc`20002Ga/200001U[K
001F^eJk001BVPH00005IQ000000O^l00002@C000`00@C11<00200000gmc00000002M]42000017[@
000007[@0P000W[@0`0000Ejd000000007[@00<00003N]0007[@00D00003N]00000001000004=_l0
03/O>al800000dLO001;7`06000014lN001?7TlN0P0000A?7P00Cai?7P<00003Cah007oo00Qoo`03
001oogoo00Qoo`0077oo00<007ooOol02Goo00@005llGcaO?080009O70800003F_/005[K00D00007
BW/004IJ000n6P00IQ000P0000Enk`00@C11<0000091<0D0009fd@800004N]000000N]020002N]03
00001G[@00000000N]000`0000Ajd000001jd0@00003N]00000001@00003>_l0000000P0009;7`P0
0003Cah0000000<00003Cah0000000<000Uoo`03001oogoo00Moo`0077oo00<007ooOol02Woo00X0
05ll001O?000Ga`005lL001Jn`80009F^`03001BV`00009:N`050016FP00001V400200001Gk_0011
<44`00000T4`0P0000Qf/P00001fd@00N]4007[@0P0000Ijd000001jd000N]0200001G[@00000000
N]000`0000Ejd000000007[@00<00003N]00000001D00004>_l00000?_l800000dLO001;7`060000
0dlN001?7P03000014lN001?7TlN0P0027oo00<007ooOol01goo000LOol00`00Oomoo`0;Ool00`00
GcaO?0020002Ga`20002E]/00`00E[/0000800001FH@000007k_00000T4`00@0044`@C00009f/P80
0005M]4007[A001jd002000017[@000007[@0P0000=jd000N]000`0000Ejd000000007[@00<00005
N]000000001jd00K00000cko0000000800000d/O0000000600000dlN0000000400000dlN00000007
Ool00`00Oomoo`07Ool001aoo`03001oogoo00aoo`800006Ga`00000001Jo0000UKK0P0000AB^`00
CY/00096N`040012>P00IQ0200001Gk_0011<44`00000T4`0`0000Mfd@00M]4007[A001jd0020000
17[@000007[@0P0000=jd000N]000`0000Ejd000000007[@00<00005N]000000001jd0090002OolA
000013[o000nocko1`0000A77`00Bai;7PD00004Cah004lNCah300000dlN001oo`06Ool00`00Oomo
o`07Ool001]oo`03001oogoo00moo`05001K75/L001Jo00300001e;K001B^e:k001>V`000096N`80
0005IQ000000O^l00002@C000`00@C00000300001gKA001fd@00N]4007[@00800004N]000000N]02
000017[@000007[@0P0000=jd00000000P0000Ajd000001jd0<00003N]00000000L000=ooa@00003
?_l004<O00P00003Bah0000000H00003Cah0000000<000Ioo`03001oogoo00Moo`006goo0P004Goo
00@005/L0000009Fg003001Bf`0000T00005IQ000000O^l00002@C000`00@C00000300001GKA001f
d@00N]400P0000=jd000N]000P0000Ajd000001jd0800004N]000000N]04000017[@000007[@0`00
00=jd00000001`0017oo5P0000=2o`0000001P0000A;7P00Bai;7PD00004Cah004lNCah20005Ool0
0`00Oomoo`07Ool001]oo`03001oogoo015oo`05001Jo5[l001Fg0020002D[/01@00CY/004Yk0000
0T9K00<006H@00000Wk_00L0044`@C00044`0008mP030002M]4200001G[@001jd000N]000P0000Aj
d000001jd0800004N]000000N]04000017[@000007[@0`0000=jd00000001`001Woo5@0000A2o`00
@_m2o`P00003Bah004/N00H00003Cah0000000Aoo`03001oogoo00Moo`006goo00<007ooOol04Woo
10000U;L00<005:k00001`0000EV4000O^mnk`000091<0L0009fd@800003N]000000009jd0800004
N]000000N]02000017[@000007[@100000Ejd000000007[@00800003N]00000000L000MooaP00003
@_l0000000P00003Bah0000000@00003Cah007oo00Aoo`03001oogoo00Ioo`006goo00<007ooOol0
4goo00<005KL0000100000I>_000BY]:V`00AW/200001FH@001nkgk_00000T4`1`000WKA0P0000=j
d00000000W[@0P0000Ajd000001jd0800005N]000000001jd00300000g[@0000000200000g[@001j
d0090008OolH000014;o0016odKo1`0000A;7P00Bai;7PD000Aoo`03001oogoo00Ioo`006goo00<0
07ooOol057oo00L005;LD]`005:l001>_00700001FH@001nkgk_00000T4`1`0000Efd@00N]4007[@
00800005N]0007[@001jd00300000g[@001jd00300001G[@00000000N]001@000W[@2@002Goo6`00
00=6o`00A_l0200000=;7P0000000`000goo00<007ooOol01Woo000JOol2000GOol00`00D]`00003
00002dZL0016NdIk0012F`00IQ0007k_00000d4`0`0000T=>000000007KA001jd@00N]000P0000Ej
d000N]0007[@00<00003N]0007[@00<00005N]000000001jd00500000g[@001jd008000;OolL0000
0dKo00000006000014/N001?7TlN0P000Woo00<007ooOol01Woo000JOol00`00Oomoo`0GOol00`00
D]`00002C[`00`00BY`00002AW/2000016H@001nk`000d4`00L001fI000000dh0000]P0200000g[A
001jd00200000g[@00000002N]0300000g[@001jd00300001G[@00000000N]001@0000Ajd000001j
d0L000Qoo`<0009ooa/00004A_l004KnA_h:0002Ool00`00Oomoo`06Ool001Yoo`03001oogoo01Mo
o`03001Bg00000T00004IQ0007k_0003@C0300002AEI000000000;H007[A001jd00200000g[@0000
0002N]03000017[@000007[@0P0000Ejd000000007[@00D00004N]000000N]070005Ool30007OolM
00000dKn0000000500001D/N001oogoo000027oo000JOol00`00Oomoo`0HOol02@00C[a>_000BY`0
04JL0012O002000016H@001nk`000T4`00<002Fj000020000W[@0P0000Ijd000001jd000N]020000
17[@000007[@0P0000Ejd000000007[@00D00005N]000000001jd0060002Ool3000<OolL00000dKn
001:oP0700000goo001oo`07Ool001Yoo`8001Yoo`03001>_00000P0009nk`030011<44`00/0009j
d0800004N]000000N]0200000g[@001jd00300000g[@001jd00300000g[@0000000300000g[@0000
000200000g[@00000005000BOolL00000dKn000000060008Ool0009oo`8000Aoo`8000=oo`@0009o
o`<000Aoo`03001oogoo01Yoo`03001:_000009:W0030016O0000080009nk`030011<00000800004
8KX000004ET500001G[@001jd000N]000P0000Ijd000001jd000N]0300000g[@001jd00300000g[@
0000000300000g[@0000000200000g[@00000003000DOol00`00Ool0000J000014Kn001:oT[n1@00
1goo00001Goo001oogoo00000goo0P0017oo00<007ooOol01Goo00D007ooOomoo`0001eoo`T0009n
k`030011<00000@000046GT000002AP300001G[@001jd000N]000P0000Ijd000001jd000N]030000
17[@000007[@0P0000=jd00000000`0000=jd000000020005Goo00<007ooOol00goo7`001goo0000
1Goo001oogoo00002Woo00<007ooOol017oo00@007ooOol001moo`06001:_000AY`0049l0P000Wk_
00<0044`00000`0000HM^P00000=F@002AP200001G[@001jd000N]000P0000Ajd000001jd0800004
N]000000N]0300000g[@001jd00:00000g[@00000003000FOol00`00Oomoo`07OolE0004Ool00`00
Oomoo`05Ool00005Ool007ooOol0000;Ool00`00Oomoo`030003Ool2000OOol70002O^l30002:O/2
000011fj000000eI1P000W[@0P0000Ajd000001jd0800004N]000000N]0300000g[@001jd0050000
0g[@0000000200000g[@00000003000GOol00`00Oomoo`0OOol00`00Oomoo`05Ool00005Ool007oo
Ool00008Ool01000Oomoo`000Woo00<007ooOol00goo00<007ooOol07Woo00L004JL0012O000IQ00
0002O^l400000bWk000Qf`0200000aEj000000050002N]02000017[@000007[@0P0000Ajd000001j
d0<00003N]00000000D00003N]00000000800003N]00000000<001Qoo`03001oogoo01ioo`03001o
ogoo00Eoo`000Woo0P002Woo0P000goo10000Woo00<007ooOol07goo100000QV4000O^mnk`00=S`0
02hK100000<MVP0000001P0000Ejd000N]0007[@00800004N]000000N]0200000g[@000000040000
0g[@0000000200000g[@0000000200000g[@00000003000IOol00`00Oomoo`0MOol00`00Oomoo`05
Ool001Uoo`03001oogoo021oo`040016W0000003O^l500000bWk0000000900001G[@001jd000N]00
0P0000Ajd000001jd0H00004N]000000N]0400000g[@0000000200000g[@00000003000IOol00`00
Oomoo`0MOol00`00Oomoo`05Ool001Uoo`03001oogoo025oo`<0009nk`<00003<S`002hK00`00003
N]000000009jd0800003N]00000000<00005N]000000001jd00300001G[@00000000N]00100000=j
d00000000`006Woo00<007ooOol07Goo00<007ooOol017oo000HOol2000SOol300001Gk_000jG000
=S`0200000@IVP00000=F@8000040=L000000002N]0500001G[@00000000N]000P0000Ejd0000000
07[@00<00003N]00000000800003N]00000000<001]oo`03001oogoo01aoo`03001oogoo00Aoo`00
67oo00<007ooOol08goo2P000RWk0P0000@M^`00000ANP@000030=L0000000@00004N]000000N]03
000017[@000007[@0`0000Ejd000000007[@00@00003N]00000000<001aoo`03001oogoo01]oo`03
001oogoo00Aoo`0067oo00<007ooOol08goo100000DnO000>U`0038l00800003:O/0027K00800003
5IX0000000H00005N]0007[@001jd00200001G[@00000000N]000P0000Ejd000000007[@00<00003
N]00000000800003N]00000000<001eoo`03001oogoo01Yoo`03001oogoo00Aoo`0067oo00<007oo
Ool08goo2P0000<^700000000P0000<M^`0000001`0000=jd000N]000P0000=jd000N]000`0000Aj
d000001jd0<00005N]000000001jd00400000g[@00000003000MOol00`00Oomoo`0JOol00`00Oomo
o`04Ool001Qoo`03001oogoo02=oo`800008Ool00000>U`003IL000b?08000039O/0027K00X00008
N]000000N]0007[@001jd0<00004N]000000N]0300001G[@00000000N]00100000=jd00000000`00
7Woo00<007ooOol06Goo00<007ooOol017oo000HOol00`00Oomoo`0SOol200000goo000nO0060000
0bXL0000000200000aVk0000000200001@UI00000000N]000P0000Ejd000N]0007[@00<00004N]00
0000N]0300001G[@00000000N]00100000=jd00000000`007goo00<007ooOol067oo00<007ooOol0
17oo000HOol00`00Oomoo`0SOol20002Ool=000011Vk000000ej1`0000Ejd000N]0007[@00<00004
N]000000N]0300001G[@00000000N]00100000=jd00000000`0087oo00<007ooOol067oo00<007oo
Ool00goo000GOol00`00Oomoo`0TOol20003Ool8000012XL0000027K0P0000<EVP0000001P0000=j
d00000000W[@0`0000Ajd000001jd0<00005N]000000001jd00400000g[@00000003000QOol00`00
Oomoo`0GOol00`00Oomoo`03Ool001Moo`03001oogoo02Aoo`03001oo`0000800003Ool000000080
0003<S`002hL008000049O/000006K/200000`ej0000000400001W[@000007[@001jd0800004N]00
0000N]0300001G[@00000000N]00100000=jd00000000`008Goo00<007ooOol05goo00<007ooOol0
0goo000GOol2000TOol30004Ool01000>W`003IL100000@Z7000000Mf`8000054IX000000001>@03
00001W[@000007[@001jd0800004N]000000N]0300001G[@00000000N]001@0000=jd00000000P00
8Woo00<007ooOol05Woo00<007ooOol00goo000GOol00`00Oomoo`0POol30008Ool400000c8l000Z
70020000127l000001Vk1P0000<1600000000P0000Ijd000001jd000N]0200001G[@00000000N]00
0`0000=jd0000000100000Ajc`00000002=oo`03001oogoo01Eoo`03001oogoo00=oo`005goo00<0
07ooOol07Goo0`0037oo3P0000<AV`0000000P0000D16000000007[@00800003N]0007[@00<00004
N]000000N]0300000g[@0000000500000g[?0000000TOol00`00Oomoo`0DOol00`00Oomoo`03Ool0
01Moo`03001oogoo01Uoo`@0011oo`03000fG@0000<00004:Q`000008O`200001AFk000000001EX0
1@0000Ijd000001jd000N]03000017[@000007[@0`0000=jd00000001P0000=jc`00Ool097oo00<0
07ooOol04goo00<007ooOol00goo000FOol00`00Oomoo`0GOol3000EOol200000c9L000^?0020000
12Gl000001gK0P0000<AV`0000001@0000Ajd000001jd0800004N]000000N]0200001G[@00000000
N]00200000=jc`00Ool09Goo00<007ooOol04goo00<007ooOol00Woo000FOol2000EOol3000HOol0
0`00=Ud00003000012XL0000027l0P0000DE^`00000000EJ00@00005N]000000001jd00200000g[@
001jd00200001G[@00000000N]00200000=jc`00Ool09Goo00<007ooOol04goo00<007ooOol00Woo
0004Ool20004Ool20003Ool30004Ool00`00Oomoo`0AOol3000LOol4000012hl000002Gl0P0000@I
f`00000AV`X00006N]000000N]0007[@0P0000Ejd000000007[@00P00003N/l007oo02Ioo`03001o
ogoo019oo`03001oogoo009oo`000goo00@007ooOol000=oo`8000Ioo`05001oogooOol0000@Ool3
000POol00`00<Ud0000=00000`Ej0000000400001W[@000007[@001jd0<00004N]000000N]080000
0g[?001oo`0WOol00`00Oomoo`0AOol00`00Oomoo`02Ool000=oo`04001oogoo000;Ool01@00Oomo
ogoo000037oo100097oo100000<Z?00000000P0000DMo0000000016K00P00006N]000000N]0007[@
100000=jd000N]00200000=jc`00Ool0:7oo00<007ooOol047oo00<007ooOol00Woo0003Ool01000
Oomoo`0027oo0`0017oo00<007ooOol01goo0`00:Goo0P0000<^?00000000P0000DQo000000001Fk
00<000031GX0000000<00004N]000000N]0200001g[@00000000N]0007[@00P00003N/l007oo02Uo
o`03001oogoo00moo`03001oogoo009oo`000goo00@007ooOol000Qoo`03001oogoo00=oo`8000Io
o`<002aoo`03000bG@0000<000049Q`000007O`300000a6K00000005000017[@000007[@0P0000=j
d00000000P000W[@200000=jc`00Ool0:Goo00<007ooOol03goo00<007ooOol00Woo0004Ool20009
Ool40002Ool00`00Oomoo`02Ool3000`Ool4000012XL0000027l0P0000<If`0000000P0000<5NP00
00000P0000Ajd000001jd0<00007N]000000001jd000N]001`0000=jc`00Ool0:Woo00<007ooOol0
3Woo00<007ooOol00Woo000EOol00`00Ool00002000dOol00`00;Ud0000C00001G[@00000000N]00
0`0000Ajd000001jd0H00003N/l007oo02]oo`03001oogoo00ioo`03001oogoo005oo`005Goo0P00
>7oo100000@V7000000Qo08000036M/00000008000031GX0000000<00005N]000000001jd0030000
17[@000007[@1P0000=jc`00Ool0;7oo00<007ooOol03Goo00<007ooOol00Goo000GOol00`00Oomo
o`0fOol2000012Xm0000028L0P0000DIg000000000fK00D000030CT007[@00<00005N]000000001j
d00300000g[@0000000300000g[?001oo`0]Ool00`00Oomoo`0<Ool00`00Oomoo`01Ool001Qoo`80
03Ioo`03000^?@0000L000055M/000000009V`0900001G[@00000000N]00100000=jd00000000P00
00=jc`00Ool0;Woo00<007ooOol02goo00<007ooOol00Goo000IOol2000fOol4000012HL000001gl
0`0000<A^`0000000`000P5I100000Ejd000000007[@00@00003N]00000000800003N/l007oo02io
o`03001oogoo00]oo`03001oogoo005oo`006goo00<007ooOol0=7oo1@0000<R700000003P0000Ej
d000000007[@00@00003N]00000000800003N/l007oo02moo`03001oogoo00Yoo`03001oogoo005o
o`0077oo0P00=Goo00<002Xm0000100000DMo0000000016k00@000030GX0000000800005N]000000
001jd00400000g[@0000000200000g[?001oo`0`Ool00`00Oomoo`09Ool00`00Oomoo`01Ool001io
o`03001oogoo03=oo`800003:Sd0000000<000056O`00000000=^`0700001G[@00000000N]001000
00=jd00000000P0000=jc`00Ool0<Goo00<007ooOol02Goo0@000Goo0Goo000OOol3000cOol50000
1B8L000000005M`0100000<9V`0000000P0000Ejd000000007[@00@00003N]00000000800003N/l0
07oo039oo`03001oogoo00Qoo`40005oo`5oo`008Goo00<007ooOol0<Goo00D002Xm0000028L0080
00057O`00000000A^`0500000`5j0000000300000g[@0000000200000g[@0000000200000g[?001o
o`0bOol00`00Oomoo`08Ool10001Ool1Ool0029oo`04001oogoo000`Ool2000012Hm0000028L0P00
00<Io00000000P0000<=V`0000001P0000=jd00000000P0000=jd00000000P0000=jc`00Ool0<goo
00<007ooOol01goo0@000Goo0Goo000SOol3000aOol600000agl0000000800001@5j00000000N]00
100000=jd00000000P0000=jc`00Ool0=7oo00<007ooOol01Woo0@000Goo0Goo000TOol2000aOol0
0`00Oomoo`0200001B8L000000006O`00`0000<=^`0000001@0000=jd00000000P0000=jd0000000
0P0000=jc`00Ool0=Goo00<007ooOol01Goo0@000Goo0Goo000VOol3000^Ool00`00Oomoo`02Ool2
00001AhL000000005M`0100000<9V`0000000P0000=jd00000000P0000=jd00000000P0000=jc`00
Ool0=Woo00<007ooOol017oo0@000Goo0Goo000XOol00`00Oomoo`0/Ool00`00Oomoo`04Ool20000
1Agl000000004M`01P0000<1NP0000000`0000=jd00000000P0000=jc`00Ool0=Woo00<007ooOol0
1Goo0@000Goo000YOol00`00Oomoo`0[Ool00`00Oomoo`06Ool200001AWl000000003K/02P0000=j
d00000000P0000=jc`00Ool0=goo00<007ooOol017oo0@000Goo000ZOol3000ZOol00`00Oomoo`08
Ool200000aGL0000000400000`FK0000000300000g[@0000000200000g[?001oo`0hOol00`00Oomo
o`03Ool10001Ool001aoo`8000Aoo`8000=oo`@00003Ool007oo02Uoo`03001oogoo00Yoo`d00003
N]00000000800003N/l007oo03Uoo`03001oogoo009oo`40005oo`006goo00@007ooOol000=oo`80
00Aoo`03001oogoo009oo`<002Moo`03001oogoo00eoo`<000033K/0000000@00003N]0000000080
0003N/l007oo03Yoo`05001oogooOol00001Ool001]oo`04001oogoo000:Ool00`00Oomoo`03Ool0
0`00Oomoo`0UOol00`00Oomoo`0?Ool400001@VK00000000N]00100000=jc`00Ool0>goo00@007oo
Ool0005oo`006goo00@007ooOol000]oo`03001oogoo00=oo`800003Ool007oo029oo`03001oogoo
015oo`@000031I/0000000@00003N/l007oo03]oo`04001oogoo0001Ool001]oo`04001oogoo0008
Ool01000Oomoo`001goo0P008goo00<007ooOol04goo2@0000=jc`00Ool0?7oo00<007ooOol00@00
000LOol2000:Ool20009Ool00`00Oomoo`0QOol00`00Oomoo`0FOol600000g[?001oo`0mOol10001
Ool1000003Aoo`<0021oo`03001oogoo01Moo`D00003N/l007oo03ioo`800000=Woo00<007ooOol0
7Woo00<007ooOol067oo100000=jc`00Ool0?Goo0`00000gOol3000MOol00`00Oomoo`0JOol4000j
Ool40003Ool003Uoo`03001oogoo01]oo`03001oogoo01eoo`03001oogoo03=oo`D000Moo`00:Woo
0P0017oo0P0017oo0`0000=oo`00Ool06goo00<007ooOol0Cgoo100037oo000YOol01000Oomoo`00
0goo0P001Goo00@007ooOomoo`8001Yoo`03001oogoo04]oo`@0011oo`00:Goo00@007ooOol000Qo
o`@000Aoo`04001oogoo000FOol00`00Oomoo`17Ool4000DOol002Uoo`04001oogoo0008Ool00`00
Ool00006Ool3000FOol00`00Oomoo`12Ool5000HOol002Uoo`04001oogoo0008Ool00`00Ool00007
Ool2000FOol00`00Oomoo`0nOol4000MOol002Yoo`8000Yoo`8000Uoo`<001=oo`03001oogoo03Yo
o`@0025oo`00@goo00<007ooOol04Goo00<007ooOol0=Woo10009Goo0014Ool00`00Ool0000?Ool0
0`00Oomoo`0bOol5000YOol004Eoo`8000moo`03001oogoo02ioo`@002ioo`00>7oo0P0017oo0P00
17oo0P0000=oo`0000003Goo00<007ooOol0:Woo1000<Woo000gOol01000Oomoo`000goo0P000goo
00D007ooOol007oo008000aoo`03001oogoo02Eoo`D003Ioo`00=goo00@007ooOol000Qoo`04001o
ogoo0003Ool2000:Ool00`00Oomoo`0QOol4000kOol003Moo`04001oogoo0008Ool30005Ool00`00
Oomoo`08Ool00`00Oomoo`0MOol4000oOol003Moo`04001oogoo0008Ool00`00Oomoo`06Ool30007
Ool00`00Oomoo`0IOol40013Ool003Qoo`8000Yoo`<000Moo`03001oogoo00Eoo`03001oogoo01Ao
o`D004Moo`00Cgoo0`0017oo00<007ooOol047oo1000C7oo001AOol00`00Oomoo`02Ool00`00Oomo
o`0<Ool4001@Ool004Aoo`8000Aoo`8000Aoo`<00004Ool007oo000:Ool4001DOol004=oo`04001o
ogoo0003Ool20003Ool00`00Oomoo`0300000goo001oo`04Ool5001HOol004=oo`04001oogoo0008
Ool01000Oomoo`000Woo0P0000=oo`0000000P00GGoo0013Ool01000Oomoo`002Goo0P0017oo0P00
HGoo0013Ool01000Oomoo`0027oo00@007ooOol006Ioo`00A7oo0P002Woo0P00Igoo002iOol00;Uo
o`00E7oo0`00HWoo001EOol00`00Oomoo`1QOol005Eoo`03001oogoo065oo`00EGoo00<007ooOol0
HGoo001EOol00`00Oomoo`1QOol005Aoo`8006=oo`00^Goo0000\
\>"],
  ImageRangeCache->{{{0, 184.688}, {200.5, 0}} -> {-0.0638339, -0.0601894, 
  0.00525242, 0.00525242}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Plot3D[If[S>95,
DownAndInCallGamma[2,S,100,95,0.2,Log[1.1],Log[1.05],t],
UpAndInCallGamma[2,S,100,95,0.2,Log[1.1],Log[1.05],t]], {S, 80, 120}, {t, \
0.0001, 1},
BoxRatios -> {1, 1, 1}, PlotPoints -> 40];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.0855 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0567234 1.16608 0 1.16608 [
[.07825 .23514 -11.8839 -9 ]
[.07825 .23514 .11606 0 ]
[.21247 .17844 -11.3335 -9 ]
[.21247 .17844 .66648 0 ]
[.3545 .11847 -16.1747 -9 ]
[.3545 .11847 1.82534 0 ]
[.50502 .05496 -15.349 -9 ]
[.50502 .05496 2.65097 0 ]
[.66483 -0.01242 -14.5234 -9 ]
[.66483 -0.01242 3.4766 0 ]
[.09301 .88495 -10.1462 0 ]
[.09301 .88495 7.85383 9 ]
[.17991 .94587 -9.90019 0 ]
[.17991 .94587 8.09981 9 ]
[.25944 1.00163 -9.68252 0 ]
[.25944 1.00163 8.31748 9 ]
[.33251 1.05285 -9.48854 0 ]
[.33251 1.05285 8.51146 9 ]
[.39987 1.10006 -3.10486 0 ]
[.39987 1.10006 2.89514 9 ]
[.07292 .26435 -30 -2.61953 ]
[.07292 .26435 0 6.38047 ]
[.05684 .365 -30 -2.70889 ]
[.05684 .365 0 6.29111 ]
[.03955 .47323 -6 -2.80566 ]
[.03955 .47323 0 6.19434 ]
[.02091 .58994 -24 -2.91081 ]
[.02091 .58994 0 6.08919 ]
[.00075 .71616 -24 -3.02547 ]
[.00075 .71616 0 5.97453 ]
[ 0 0 0 0 ]
[ 1 1.0855 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
[(80)] .07825 .23514 .98066 1 Mshowa
.22216 .18933 m
.227 .19478 L
s
[(90)] .21247 .17844 .88892 1 Mshowa
.36358 .12987 m
.36813 .13557 L
s
[(100)] .3545 .11847 .79718 1 Mshowa
.51343 .06687 m
.51763 .07282 L
s
[(110)] .50502 .05496 .70545 1 Mshowa
.67245 0 m
.67627 .00621 L
s
[(120)] .66483 -0.01242 .61371 1 Mshowa
.125 Mabswid
.11461 .23456 m
.11764 .23771 L
s
.14105 .22344 m
.14405 .22662 L
s
.16779 .2122 m
.17076 .21541 L
s
.19482 .20083 m
.19776 .20407 L
s
.2498 .17771 m
.25268 .18101 L
s
.27776 .16595 m
.2806 .16928 L
s
.30604 .15406 m
.30885 .15742 L
s
.33465 .14204 m
.33741 .14542 L
s
.39286 .11756 m
.39554 .12101 L
s
.42247 .10511 m
.42512 .10859 L
s
.45243 .09251 m
.45504 .09602 L
s
.48275 .07977 m
.48531 .08331 L
s
.54447 .05381 m
.54695 .05742 L
s
.57589 .0406 m
.57832 .04424 L
s
.60769 .02723 m
.61007 .0309 L
s
.63987 .0137 m
.64221 .0174 L
s
.25 Mabswid
0 .80374 m
.40037 1.0855 L
s
.09485 .87049 m
.09577 .86326 L
s
[(0.2)] .09301 .88495 .12735 -1 Mshowa
.18136 .93137 m
.18209 .92412 L
s
[(0.4)] .17991 .94587 .10002 -1 Mshowa
.26055 .98709 m
.2611 .97983 L
s
[(0.6)] .25944 1.00163 .07584 -1 Mshowa
.3333 1.03829 m
.33369 1.03102 L
s
[(0.8)] .33251 1.05285 .05428 -1 Mshowa
.40037 1.0855 m
.40063 1.07821 L
s
[(1)] .39987 1.10006 .03495 -1 Mshowa
.125 Mabswid
.02453 .821 m
.02518 .81668 L
s
.04852 .83789 m
.04914 .83356 L
s
.07196 .85438 m
.07254 .85004 L
s
.11722 .88623 m
.11774 .88189 L
s
.13908 .90162 m
.13958 .89727 L
s
.16046 .91666 m
.16092 .91231 L
s
.20181 .94576 m
.20221 .9414 L
s
.22181 .95983 m
.22219 .95548 L
s
.24138 .97361 m
.24174 .96925 L
s
.27931 1.0003 m
.27961 .99593 L
s
.29768 1.01322 m
.29796 1.00886 L
s
.31567 1.02589 m
.31593 1.02152 L
s
.35057 1.05045 m
.35079 1.04608 L
s
.3675 1.06236 m
.3677 1.058 L
s
.3841 1.07404 m
.38427 1.06967 L
s
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
.08637 .25873 m
.09309 .25592 L
s
[(-0.04)] .07292 .26435 1 -0.41788 Mshowa
.07038 .35961 m
.07715 .35691 L
s
[(-0.02)] .05684 .365 1 -0.39802 Mshowa
.05319 .4681 m
.06001 .46553 L
s
[(0)] .03955 .47323 1 -0.37652 Mshowa
.03465 .58509 m
.04152 .58266 L
s
[(0.02)] .02091 .58994 1 -0.35315 Mshowa
.0146 .71162 m
.02152 .70935 L
s
[(0.04)] .00075 .71616 1 -0.32767 Mshowa
.125 Mabswid
.08248 .28328 m
.08652 .28161 L
s
.07852 .30827 m
.08257 .30662 L
s
.07449 .33371 m
.07854 .33207 L
s
.0662 .38599 m
.07027 .38439 L
s
.06194 .41285 m
.06602 .41127 L
s
.05761 .44021 m
.06169 .43865 L
s
.04869 .49651 m
.05279 .49499 L
s
.0441 .52547 m
.04821 .52397 L
s
.03942 .55499 m
.04354 .55351 L
s
.02979 .61578 m
.03392 .61435 L
s
.02482 .64709 m
.02896 .64568 L
s
.01976 .67903 m
.02391 .67764 L
s
.00933 .74488 m
.01349 .74354 L
s
.00395 .77883 m
.00812 .77752 L
s
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
.41799 .58158 m
.08846 .24555 L
s
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.41799 .58158 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
0 0 m
1 0 L
1 1.0855 L
0 1.0855 L
closepath
clip
newpath
.5 Mabswid
.437 .662 .956 r
.40023 .86105 .40797 .86821 .4208 .86934 .41308 .86236 Metetra
.446 .672 .96 r
.41308 .86236 .4208 .86934 .43373 .87023 .42604 .86343 Metetra
.458 .684 .963 r
.42604 .86343 .43373 .87023 .44678 .87081 .43911 .8642 Metetra
.471 .697 .966 r
.43911 .8642 .44678 .87081 .45993 .87106 .45229 .86464 Metetra
.486 .71 .969 r
.45229 .86464 .45993 .87106 .47318 .87095 .46558 .86471 Metetra
.503 .723 .971 r
.46558 .86471 .47318 .87095 .48654 .87043 .47897 .86438 Metetra
.521 .737 .973 r
.47897 .86438 .48654 .87043 .49999 .8695 .49246 .86362 Metetra
.539 .75 .974 r
.49246 .86362 .49999 .8695 .51354 .86813 .50605 .86241 Metetra
.557 .763 .974 r
.50605 .86241 .51354 .86813 .52717 .86631 .51973 .86074 Metetra
.576 .775 .973 r
.51973 .86074 .52717 .86631 .54089 .86404 .5335 .8586 Metetra
.593 .786 .972 r
.5335 .8586 .54089 .86404 .55469 .86132 .54735 .85598 Metetra
.611 .796 .97 r
.54735 .85598 .55469 .86132 .56856 .85814 .56128 .85289 Metetra
.627 .805 .968 r
.56128 .85289 .56856 .85814 .58251 .85452 .57528 .84934 Metetra
.643 .812 .965 r
.57528 .84934 .58251 .85452 .59652 .85047 .58935 .84534 Metetra
.954 .641 .398 r
.58935 .84534 .59652 .85047 .61383 .97484 .60643 .96887 Metetra
.894 .803 .751 r
.60643 .96887 .61383 .97484 .62797 .95183 .62067 .94556 Metetra
.888 .802 .759 r
.62067 .94556 .62797 .95183 .64208 .93008 .63486 .92351 Metetra
.88 .8 .766 r
.63486 .92351 .64208 .93008 .65616 .9095 .64902 .90266 Metetra
.873 .798 .774 r
.64902 .90266 .65616 .9095 .67022 .89004 .66316 .88295 Metetra
.865 .796 .781 r
.66316 .88295 .67022 .89004 .68427 .87163 .67729 .86431 Metetra
.857 .794 .788 r
.67729 .86431 .68427 .87163 .69833 .85421 .69142 .84667 Metetra
.849 .792 .795 r
.69142 .84667 .69833 .85421 .7124 .83772 .70556 .82999 Metetra
.841 .789 .802 r
.70556 .82999 .7124 .83772 .72649 .82211 .71972 .8142 Metetra
.832 .787 .809 r
.71972 .8142 .72649 .82211 .74061 .80733 .73391 .79926 Metetra
.824 .784 .815 r
.73391 .79926 .74061 .80733 .75477 .79332 .74813 .78511 Metetra
.815 .781 .821 r
.74813 .78511 .75477 .79332 .76898 .78004 .7624 .7717 Metetra
.807 .779 .827 r
.7624 .7717 .76898 .78004 .78324 .76745 .77672 .75899 Metetra
.798 .776 .832 r
.77672 .75899 .78324 .76745 .79755 .75549 .7911 .74693 Metetra
.79 .773 .838 r
.7911 .74693 .79755 .75549 .81192 .74412 .80554 .73547 Metetra
.782 .771 .843 r
.80554 .73547 .81192 .74412 .82637 .73331 .82004 .72459 Metetra
.774 .769 .848 r
.82004 .72459 .82637 .73331 .84088 .72302 .83462 .71422 Metetra
.767 .766 .852 r
.83462 .71422 .84088 .72302 .85547 .7132 .84927 .70435 Metetra
.759 .764 .857 r
.84927 .70435 .85547 .7132 .87014 .70383 .864 .69492 Metetra
.752 .762 .861 r
.864 .69492 .87014 .70383 .88489 .69486 .87882 .68592 Metetra
.745 .76 .865 r
.87882 .68592 .88489 .69486 .89972 .68628 .89371 .6773 Metetra
.739 .758 .869 r
.89371 .6773 .89972 .68628 .91464 .67804 .9087 .66903 Metetra
.732 .757 .872 r
.9087 .66903 .91464 .67804 .92965 .67013 .92378 .66109 Metetra
.726 .755 .875 r
.92378 .66109 .92965 .67013 .94476 .66251 .93895 .65344 Metetra
.721 .754 .878 r
.93895 .65344 .94476 .66251 .95995 .65515 .95421 .64607 Metetra
.427 .655 .955 r
.39242 .85374 .40023 .86105 .41308 .86236 .40529 .85524 Metetra
.435 .665 .958 r
.40529 .85524 .41308 .86236 .42604 .86343 .41827 .8565 Metetra
.446 .677 .962 r
.41827 .8565 .42604 .86343 .43911 .8642 .43137 .85747 Metetra
.46 .69 .966 r
.43137 .85747 .43911 .8642 .45229 .86464 .44457 .85812 Metetra
.475 .703 .969 r
.44457 .85812 .45229 .86464 .46558 .86471 .45789 .85838 Metetra
.492 .718 .972 r
.45789 .85838 .46558 .86471 .47897 .86438 .47132 .85825 Metetra
.51 .732 .974 r
.47132 .85825 .47897 .86438 .49246 .86362 .48485 .85767 Metetra
.529 .746 .975 r
.48485 .85767 .49246 .86362 .50605 .86241 .49848 .85663 Metetra
.548 .76 .975 r
.49848 .85663 .50605 .86241 .51973 .86074 .51221 .85512 Metetra
.567 .773 .975 r
.51221 .85512 .51973 .86074 .5335 .8586 .52603 .85312 Metetra
.586 .784 .974 r
.52603 .85312 .5335 .8586 .54735 .85598 .53993 .85062 Metetra
.605 .795 .972 r
.53993 .85062 .54735 .85598 .56128 .85289 .55392 .84763 Metetra
.622 .805 .97 r
.55392 .84763 .56128 .85289 .57528 .84934 .56798 .84416 Metetra
.638 .813 .967 r
.56798 .84416 .57528 .84934 .58935 .84534 .58211 .84021 Metetra
.954 .641 .397 r
.58211 .84021 .58935 .84534 .60643 .96887 .59896 .96287 Metetra
.896 .803 .75 r
.59896 .96287 .60643 .96887 .62067 .94556 .61329 .93923 Metetra
.889 .802 .757 r
.61329 .93923 .62067 .94556 .63486 .92351 .62757 .91688 Metetra
.881 .8 .765 r
.62757 .91688 .63486 .92351 .64902 .90266 .64181 .89575 Metetra
.874 .798 .772 r
.64181 .89575 .64902 .90266 .66316 .88295 .65603 .87577 Metetra
.866 .796 .78 r
.65603 .87577 .66316 .88295 .67729 .86431 .67023 .85688 Metetra
.858 .793 .787 r
.67023 .85688 .67729 .86431 .69142 .84667 .68444 .83903 Metetra
.85 .791 .794 r
.68444 .83903 .69142 .84667 .70556 .82999 .69865 .82214 Metetra
.841 .788 .801 r
.69865 .82214 .70556 .82999 .71972 .8142 .71288 .80617 Metetra
.833 .786 .807 r
.71288 .80617 .71972 .8142 .73391 .79926 .72714 .79106 Metetra
.824 .783 .814 r
.72714 .79106 .73391 .79926 .74813 .78511 .74143 .77676 Metetra
.816 .78 .82 r
.74143 .77676 .74813 .78511 .7624 .7717 .75576 .76323 Metetra
.807 .778 .826 r
.75576 .76323 .7624 .7717 .77672 .75899 .77015 .7504 Metetra
.798 .775 .831 r
.77015 .7504 .77672 .75899 .7911 .74693 .78458 .73823 Metetra
.79 .772 .837 r
.78458 .73823 .7911 .74693 .80554 .73547 .79909 .72669 Metetra
.782 .77 .842 r
.79909 .72669 .80554 .73547 .82004 .72459 .81365 .71572 Metetra
.774 .767 .847 r
.81365 .71572 .82004 .72459 .83462 .71422 .82829 .70529 Metetra
.766 .765 .852 r
.82829 .70529 .83462 .71422 .84927 .70435 .84301 .69536 Metetra
.759 .763 .856 r
.84301 .69536 .84927 .70435 .864 .69492 .8578 .68589 Metetra
.751 .761 .86 r
.8578 .68589 .864 .69492 .87882 .68592 .87268 .67684 Metetra
.744 .759 .864 r
.87268 .67684 .87882 .68592 .89371 .6773 .88765 .66819 Metetra
.738 .757 .868 r
.88765 .66819 .89371 .6773 .9087 .66903 .9027 .65989 Metetra
.732 .755 .872 r
.9027 .65989 .9087 .66903 .92378 .66109 .91784 .65192 Metetra
.726 .754 .875 r
.91784 .65192 .92378 .66109 .93895 .65344 .93308 .64426 Metetra
.72 .753 .878 r
.93308 .64426 .93895 .65344 .95421 .64607 .94841 .63687 Metetra
.416 .647 .953 r
.38454 .84627 .39242 .85374 .40529 .85524 .39743 .84796 Metetra
.424 .657 .957 r
.39743 .84796 .40529 .85524 .41827 .8565 .41043 .84943 Metetra
.434 .669 .961 r
.41043 .84943 .41827 .8565 .43137 .85747 .42355 .85061 Metetra
.447 .682 .965 r
.42355 .85061 .43137 .85747 .44457 .85812 .43679 .85147 Metetra
.463 .696 .969 r
.43679 .85147 .44457 .85812 .45789 .85838 .45014 .85195 Metetra
.48 .711 .972 r
.45014 .85195 .45789 .85838 .47132 .85825 .4636 .85202 Metetra
.498 .726 .974 r
.4636 .85202 .47132 .85825 .48485 .85767 .47716 .85164 Metetra
.518 .741 .976 r
.47716 .85164 .48485 .85767 .49848 .85663 .49084 .85079 Metetra
.538 .756 .977 r
.49084 .85079 .49848 .85663 .51221 .85512 .50461 .84945 Metetra
.558 .77 .977 r
.50461 .84945 .51221 .85512 .52603 .85312 .51848 .8476 Metetra
.578 .783 .976 r
.51848 .8476 .52603 .85312 .53993 .85062 .53243 .84524 Metetra
.598 .794 .975 r
.53243 .84524 .53993 .85062 .55392 .84763 .54647 .84236 Metetra
.616 .805 .972 r
.54647 .84236 .55392 .84763 .56798 .84416 .56059 .83898 Metetra
.634 .814 .969 r
.56059 .83898 .56798 .84416 .58211 .84021 .57478 .83509 Metetra
.954 .64 .396 r
.57478 .83509 .58211 .84021 .59896 .96287 .5914 .95683 Metetra
.897 .803 .748 r
.5914 .95683 .59896 .96287 .61329 .93923 .60583 .93285 Metetra
.89 .802 .756 r
.60583 .93285 .61329 .93923 .62757 .91688 .6202 .91018 Metetra
.882 .8 .763 r
.6202 .91018 .62757 .91688 .64181 .89575 .63453 .88875 Metetra
.875 .798 .771 r
.63453 .88875 .64181 .89575 .65603 .87577 .64883 .8685 Metetra
.867 .795 .778 r
.64883 .8685 .65603 .87577 .67023 .85688 .66311 .84936 Metetra
.859 .793 .785 r
.66311 .84936 .67023 .85688 .68444 .83903 .67739 .83127 Metetra
.85 .79 .792 r
.67739 .83127 .68444 .83903 .69865 .82214 .69167 .81418 Metetra
.842 .788 .799 r
.69167 .81418 .69865 .82214 .71288 .80617 .70597 .79802 Metetra
.833 .785 .806 r
.70597 .79802 .71288 .80617 .72714 .79106 .72029 .78274 Metetra
.824 .782 .812 r
.72029 .78274 .72714 .79106 .74143 .77676 .73465 .76829 Metetra
.816 .779 .819 r
.73465 .76829 .74143 .77676 .75576 .76323 .74905 .75462 Metetra
.807 .777 .825 r
.74905 .75462 .75576 .76323 .77015 .7504 .7635 .74167 Metetra
.798 .774 .83 r
.7635 .74167 .77015 .7504 .78458 .73823 .778 .7294 Metetra
.79 .771 .836 r
.778 .7294 .78458 .73823 .79909 .72669 .79257 .71777 Metetra
.782 .769 .841 r
.79257 .71777 .79909 .72669 .81365 .71572 .8072 .70673 Metetra
.774 .766 .846 r
.8072 .70673 .81365 .71572 .82829 .70529 .8219 .69623 Metetra
.766 .764 .851 r
.8219 .69623 .82829 .70529 .84301 .69536 .83668 .68624 Metetra
.758 .762 .855 r
.83668 .68624 .84301 .69536 .8578 .68589 .85154 .67672 Metetra
.751 .76 .86 r
.85154 .67672 .8578 .68589 .87268 .67684 .86648 .66763 Metetra
.744 .758 .864 r
.86648 .66763 .87268 .67684 .88765 .66819 .88151 .65895 Metetra
.737 .756 .868 r
.88151 .65895 .88765 .66819 .9027 .65989 .89663 .65062 Metetra
.731 .754 .871 r
.89663 .65062 .9027 .65989 .91784 .65192 .91184 .64263 Metetra
.725 .753 .875 r
.91184 .64263 .91784 .65192 .93308 .64426 .92715 .63495 Metetra
.719 .751 .878 r
.92715 .63495 .93308 .64426 .94841 .63687 .94255 .62754 Metetra
.405 .639 .951 r
.3766 .83863 .38454 .84627 .39743 .84796 .3895 .84052 Metetra
.412 .649 .955 r
.3895 .84052 .39743 .84796 .41043 .84943 .40253 .8422 Metetra
.422 .661 .96 r
.40253 .8422 .41043 .84943 .42355 .85061 .41567 .84361 Metetra
.434 .674 .964 r
.41567 .84361 .42355 .85061 .43679 .85147 .42893 .8447 Metetra
.45 .689 .968 r
.42893 .8447 .43679 .85147 .45014 .85195 .4423 .84541 Metetra
.467 .704 .972 r
.4423 .84541 .45014 .85195 .4636 .85202 .4558 .8457 Metetra
.486 .72 .975 r
.4558 .8457 .4636 .85202 .47716 .85164 .4694 .84554 Metetra
.506 .736 .977 r
.4694 .84554 .47716 .85164 .49084 .85079 .48311 .84489 Metetra
.527 .752 .978 r
.48311 .84489 .49084 .85079 .50461 .84945 .49693 .84373 Metetra
.548 .767 .979 r
.49693 .84373 .50461 .84945 .51848 .8476 .51084 .84205 Metetra
.57 .781 .978 r
.51084 .84205 .51848 .8476 .53243 .84524 .52485 .83984 Metetra
.59 .793 .977 r
.52485 .83984 .53243 .84524 .54647 .84236 .53895 .83708 Metetra
.61 .804 .974 r
.53895 .83708 .54647 .84236 .56059 .83898 .55312 .8338 Metetra
.629 .814 .971 r
.55312 .8338 .56059 .83898 .57478 .83509 .56738 .82999 Metetra
.954 .639 .396 r
.56738 .82999 .57478 .83509 .5914 .95683 .58377 .95075 Metetra
.898 .803 .747 r
.58377 .95075 .5914 .95683 .60583 .93285 .59829 .92642 Metetra
.891 .802 .754 r
.59829 .92642 .60583 .93285 .6202 .91018 .61275 .90342 Metetra
.883 .799 .762 r
.61275 .90342 .6202 .91018 .63453 .88875 .62717 .88168 Metetra
.876 .797 .769 r
.62717 .88168 .63453 .88875 .64883 .8685 .64155 .86114 Metetra
.868 .795 .777 r
.64155 .86114 .64883 .8685 .66311 .84936 .65591 .84174 Metetra
.86 .792 .784 r
.65591 .84174 .66311 .84936 .67739 .83127 .67026 .82341 Metetra
.851 .79 .791 r
.67026 .82341 .67739 .83127 .69167 .81418 .68462 .80609 Metetra
.842 .787 .798 r
.68462 .80609 .69167 .81418 .70597 .79802 .69899 .78974 Metetra
.834 .784 .805 r
.69899 .78974 .70597 .79802 .72029 .78274 .71338 .77428 Metetra
.825 .781 .811 r
.71338 .77428 .72029 .78274 .73465 .76829 .72781 .75967 Metetra
.816 .778 .817 r
.72781 .75967 .73465 .76829 .74905 .75462 .74227 .74586 Metetra
.807 .775 .823 r
.74227 .74586 .74905 .75462 .7635 .74167 .75679 .7328 Metetra
.798 .773 .829 r
.75679 .7328 .7635 .74167 .778 .7294 .77135 .72042 Metetra
.79 .77 .835 r
.77135 .72042 .778 .7294 .79257 .71777 .78598 .7087 Metetra
.781 .767 .84 r
.78598 .7087 .79257 .71777 .8072 .70673 .80068 .69758 Metetra
.773 .765 .845 r
.80068 .69758 .8072 .70673 .8219 .69623 .81545 .68702 Metetra
.765 .762 .85 r
.81545 .68702 .8219 .69623 .83668 .68624 .83029 .67698 Metetra
.758 .76 .855 r
.83029 .67698 .83668 .68624 .85154 .67672 .84521 .66741 Metetra
.75 .758 .859 r
.84521 .66741 .85154 .67672 .86648 .66763 .86022 .65829 Metetra
.743 .756 .863 r
.86022 .65829 .86648 .66763 .88151 .65895 .87532 .64957 Metetra
.736 .755 .867 r
.87532 .64957 .88151 .65895 .89663 .65062 .8905 .64122 Metetra
.73 .753 .871 r
.8905 .64122 .89663 .65062 .91184 .64263 .90578 .63322 Metetra
.724 .752 .874 r
.90578 .63322 .91184 .64263 .92715 .63495 .92116 .62552 Metetra
.718 .75 .878 r
.92116 .62552 .92715 .63495 .94255 .62754 .93663 .6181 Metetra
.394 .63 .949 r
.36859 .83082 .3766 .83863 .3895 .84052 .38151 .83292 Metetra
.4 .64 .953 r
.38151 .83292 .3895 .84052 .40253 .8422 .39455 .83482 Metetra
.409 .652 .958 r
.39455 .83482 .40253 .8422 .41567 .84361 .40771 .83646 Metetra
.421 .665 .962 r
.40771 .83646 .41567 .84361 .42893 .8447 .42099 .83779 Metetra
.435 .68 .967 r
.42099 .83779 .42893 .8447 .4423 .84541 .43439 .83874 Metetra
.453 .697 .971 r
.43439 .83874 .4423 .84541 .4558 .8457 .44792 .83927 Metetra
.472 .713 .975 r
.44792 .83927 .4558 .8457 .4694 .84554 .46156 .83934 Metetra
.493 .73 .978 r
.46156 .83934 .4694 .84554 .48311 .84489 .47531 .83892 Metetra
.515 .747 .979 r
.47531 .83892 .48311 .84489 .49693 .84373 .48917 .83796 Metetra
.537 .763 .98 r
.48917 .83796 .49693 .84373 .51084 .84205 .50313 .83647 Metetra
.56 .778 .98 r
.50313 .83647 .51084 .84205 .52485 .83984 .51719 .83442 Metetra
.582 .792 .979 r
.51719 .83442 .52485 .83984 .53895 .83708 .53134 .8318 Metetra
.603 .804 .977 r
.53134 .8318 .53895 .83708 .55312 .8338 .54557 .82863 Metetra
.623 .815 .974 r
.54557 .82863 .55312 .8338 .56738 .82999 .55988 .82491 Metetra
.953 .638 .395 r
.55988 .82491 .56738 .82999 .58377 .95075 .57605 .94464 Metetra
.899 .803 .745 r
.57605 .94464 .58377 .95075 .59829 .92642 .59067 .91993 Metetra
.892 .801 .752 r
.59067 .91993 .59829 .92642 .61275 .90342 .60523 .89658 Metetra
.885 .799 .76 r
.60523 .89658 .61275 .90342 .62717 .88168 .61973 .87452 Metetra
.877 .797 .767 r
.61973 .87452 .62717 .88168 .64155 .86114 .63419 .85368 Metetra
.869 .794 .775 r
.63419 .85368 .64155 .86114 .65591 .84174 .64864 .834 Metetra
.86 .791 .782 r
.64864 .834 .65591 .84174 .67026 .82341 .66306 .81542 Metetra
.852 .789 .789 r
.66306 .81542 .67026 .82341 .68462 .80609 .6775 .79788 Metetra
.843 .786 .796 r
.6775 .79788 .68462 .80609 .69899 .78974 .69194 .78132 Metetra
.834 .783 .803 r
.69194 .78132 .69899 .78974 .71338 .77428 .7064 .76568 Metetra
.825 .78 .81 r
.7064 .76568 .71338 .77428 .72781 .75967 .72089 .75091 Metetra
.816 .777 .816 r
.72089 .75091 .72781 .75967 .74227 .74586 .73542 .73696 Metetra
.807 .774 .822 r
.73542 .73696 .74227 .74586 .75679 .7328 .75 .72377 Metetra
.798 .771 .828 r
.75 .72377 .75679 .7328 .77135 .72042 .76464 .7113 Metetra
.79 .768 .834 r
.76464 .7113 .77135 .72042 .78598 .7087 .77933 .69948 Metetra
.781 .766 .839 r
.77933 .69948 .78598 .7087 .80068 .69758 .79409 .68829 Metetra
.773 .763 .844 r
.79409 .68829 .80068 .69758 .81545 .68702 .80892 .67766 Metetra
.765 .761 .849 r
.80892 .67766 .81545 .68702 .83029 .67698 .82383 .66757 Metetra
.757 .759 .854 r
.82383 .66757 .83029 .67698 .84521 .66741 .83882 .65796 Metetra
.749 .757 .859 r
.83882 .65796 .84521 .66741 .86022 .65829 .85389 .64881 Metetra
.742 .755 .863 r
.85389 .64881 .86022 .65829 .87532 .64957 .86906 .64006 Metetra
.735 .753 .867 r
.86906 .64006 .87532 .64957 .8905 .64122 .88431 .63169 Metetra
.729 .752 .871 r
.88431 .63169 .8905 .64122 .90578 .63322 .89966 .62367 Metetra
.723 .75 .874 r
.89966 .62367 .90578 .63322 .92116 .62552 .9151 .61596 Metetra
.717 .749 .877 r
.9151 .61596 .92116 .62552 .93663 .6181 .93064 .60853 Metetra
.383 .621 .946 r
.36052 .82282 .36859 .83082 .38151 .83292 .37344 .82513 Metetra
.387 .63 .951 r
.37344 .82513 .38151 .83292 .39455 .83482 .3865 .82726 Metetra
.395 .642 .956 r
.3865 .82726 .39455 .83482 .40771 .83646 .39967 .82915 Metetra
.406 .656 .961 r
.39967 .82915 .40771 .83646 .42099 .83779 .41298 .83073 Metetra
.42 .671 .966 r
.41298 .83073 .42099 .83779 .43439 .83874 .4264 .83195 Metetra
.438 .688 .97 r
.4264 .83195 .43439 .83874 .44792 .83927 .43996 .83274 Metetra
.457 .705 .974 r
.43996 .83274 .44792 .83927 .46156 .83934 .45363 .83306 Metetra
.479 .723 .978 r
.45363 .83306 .46156 .83934 .47531 .83892 .46742 .83287 Metetra
.502 .741 .98 r
.46742 .83287 .47531 .83892 .48917 .83796 .48132 .83214 Metetra
.525 .759 .982 r
.48132 .83214 .48917 .83796 .50313 .83647 .49533 .83085 Metetra
.549 .775 .982 r
.49533 .83085 .50313 .83647 .51719 .83442 .50944 .82897 Metetra
.572 .79 .981 r
.50944 .82897 .51719 .83442 .53134 .8318 .52364 .82651 Metetra
.595 .803 .979 r
.52364 .82651 .53134 .8318 .54557 .82863 .53794 .82347 Metetra
.616 .815 .977 r
.53794 .82347 .54557 .82863 .55988 .82491 .55231 .81984 Metetra
.953 .637 .394 r
.55231 .81984 .55988 .82491 .57605 .94464 .56825 .9385 Metetra
.9 .803 .743 r
.56825 .9385 .57605 .94464 .59067 .91993 .58297 .9134 Metetra
.893 .801 .751 r
.58297 .9134 .59067 .91993 .60523 .89658 .59762 .88968 Metetra
.886 .799 .758 r
.59762 .88968 .60523 .89658 .61973 .87452 .61221 .86727 Metetra
.878 .796 .766 r
.61221 .86727 .61973 .87452 .63419 .85368 .62676 .84611 Metetra
.87 .794 .773 r
.62676 .84611 .63419 .85368 .64864 .834 .64129 .82615 Metetra
.861 .791 .78 r
.64129 .82615 .64864 .834 .66306 .81542 .65579 .8073 Metetra
.852 .788 .788 r
.65579 .8073 .66306 .81542 .6775 .79788 .6703 .78953 Metetra
.844 .785 .795 r
.6703 .78953 .6775 .79788 .69194 .78132 .68481 .77276 Metetra
.835 .782 .801 r
.68481 .77276 .69194 .78132 .7064 .76568 .69934 .75693 Metetra
.825 .779 .808 r
.69934 .75693 .7064 .76568 .72089 .75091 .7139 .742 Metetra
.816 .776 .815 r
.7139 .742 .72089 .75091 .73542 .73696 .7285 .72791 Metetra
.807 .773 .821 r
.7285 .72791 .73542 .73696 .75 .72377 .74315 .71459 Metetra
.798 .77 .827 r
.74315 .71459 .75 .72377 .76464 .7113 .75785 .70201 Metetra
.789 .767 .833 r
.75785 .70201 .76464 .7113 .77933 .69948 .77261 .69011 Metetra
.781 .764 .838 r
.77261 .69011 .77933 .69948 .79409 .68829 .78743 .67884 Metetra
.772 .762 .844 r
.78743 .67884 .79409 .68829 .80892 .67766 .80233 .66815 Metetra
.764 .76 .849 r
.80233 .66815 .80892 .67766 .82383 .66757 .8173 .65801 Metetra
.756 .757 .853 r
.8173 .65801 .82383 .66757 .83882 .65796 .83236 .64836 Metetra
.749 .755 .858 r
.83236 .64836 .83882 .65796 .85389 .64881 .8475 .63918 Metetra
.741 .753 .862 r
.8475 .63918 .85389 .64881 .86906 .64006 .86273 .63041 Metetra
.734 .752 .866 r
.86273 .63041 .86906 .64006 .88431 .63169 .87805 .62202 Metetra
.728 .75 .87 r
.87805 .62202 .88431 .63169 .89966 .62367 .89347 .61399 Metetra
.722 .749 .874 r
.89347 .61399 .89966 .62367 .9151 .61596 .90898 .60627 Metetra
.716 .748 .877 r
.90898 .60627 .9151 .61596 .93064 .60853 .92459 .59884 Metetra
.371 .612 .943 r
.35237 .81461 .36052 .82282 .37344 .82513 .36531 .81714 Metetra
.374 .62 .948 r
.36531 .81714 .37344 .82513 .3865 .82726 .37838 .81952 Metetra
.38 .631 .953 r
.37838 .81952 .3865 .82726 .39967 .82915 .39157 .82167 Metetra
.39 .645 .958 r
.39157 .82167 .39967 .82915 .41298 .83073 .40489 .82352 Metetra
.404 .661 .964 r
.40489 .82352 .41298 .83073 .4264 .83195 .41834 .82501 Metetra
.421 .678 .969 r
.41834 .82501 .4264 .83195 .43996 .83274 .43192 .82608 Metetra
.441 .697 .974 r
.43192 .82608 .43996 .83274 .45363 .83306 .44562 .82667 Metetra
.463 .716 .978 r
.44562 .82667 .45363 .83306 .46742 .83287 .45945 .82674 Metetra
.487 .735 .981 r
.45945 .82674 .46742 .83287 .48132 .83214 .47339 .82626 Metetra
.512 .753 .983 r
.47339 .82626 .48132 .83214 .49533 .83085 .48744 .82519 Metetra
.537 .771 .984 r
.48744 .82519 .49533 .83085 .50944 .82897 .5016 .82351 Metetra
.562 .787 .984 r
.5016 .82351 .50944 .82897 .52364 .82651 .51586 .82123 Metetra
.586 .802 .982 r
.51586 .82123 .52364 .82651 .53794 .82347 .53021 .81832 Metetra
.609 .815 .979 r
.53021 .81832 .53794 .82347 .55231 .81984 .54465 .8148 Metetra
.953 .637 .393 r
.54465 .8148 .55231 .81984 .56825 .9385 .56036 .93232 Metetra
.902 .803 .741 r
.56036 .93232 .56825 .9385 .58297 .9134 .57519 .90681 Metetra
.895 .801 .749 r
.57519 .90681 .58297 .9134 .59762 .88968 .58993 .8827 Metetra
.887 .798 .756 r
.58993 .8827 .59762 .88968 .61221 .86727 .60462 .85993 Metetra
.879 .796 .764 r
.60462 .85993 .61221 .86727 .62676 .84611 .61926 .83844 Metetra
.871 .793 .771 r
.61926 .83844 .62676 .84611 .64129 .82615 .63386 .81817 Metetra
.862 .79 .779 r
.63386 .81817 .64129 .82615 .65579 .8073 .64845 .79906 Metetra
.853 .787 .786 r
.64845 .79906 .65579 .8073 .6703 .78953 .66303 .78103 Metetra
.844 .784 .793 r
.66303 .78103 .6703 .78953 .68481 .77276 .67761 .76404 Metetra
.835 .78 .8 r
.67761 .76404 .68481 .77276 .69934 .75693 .69222 .74803 Metetra
.826 .777 .807 r
.69222 .74803 .69934 .75693 .7139 .742 .70685 .73293 Metetra
.816 .774 .813 r
.70685 .73293 .7139 .742 .7285 .72791 .72151 .71868 Metetra
.807 .771 .82 r
.72151 .71868 .7285 .72791 .74315 .71459 .73622 .70525 Metetra
.798 .768 .826 r
.73622 .70525 .74315 .71459 .75785 .70201 .75099 .69256 Metetra
.789 .765 .832 r
.75099 .69256 .75785 .70201 .77261 .69011 .76581 .68057 Metetra
.78 .763 .837 r
.76581 .68057 .77261 .69011 .78743 .67884 .7807 .66922 Metetra
.772 .76 .843 r
.7807 .66922 .78743 .67884 .80233 .66815 .79567 .65848 Metetra
.763 .758 .848 r
.79567 .65848 .80233 .66815 .8173 .65801 .81071 .64829 Metetra
.755 .756 .853 r
.81071 .64829 .8173 .65801 .83236 .64836 .82583 .63861 Metetra
.748 .754 .857 r
.82583 .63861 .83236 .64836 .8475 .63918 .84104 .62939 Metetra
.74 .752 .862 r
.84104 .62939 .8475 .63918 .86273 .63041 .85633 .62061 Metetra
.733 .75 .866 r
.85633 .62061 .86273 .63041 .87805 .62202 .87173 .61221 Metetra
.727 .749 .87 r
.87173 .61221 .87805 .62202 .89347 .61399 .88721 .60417 Metetra
.721 .748 .874 r
.88721 .60417 .89347 .61399 .90898 .60627 .90279 .59645 Metetra
.715 .747 .877 r
.90279 .59645 .90898 .60627 .92459 .59884 .91848 .58901 Metetra
.359 .602 .94 r
.34417 .80619 .35237 .81461 .36531 .81714 .35711 .80895 Metetra
.36 .609 .945 r
.35711 .80895 .36531 .81714 .37838 .81952 .37019 .81158 Metetra
.365 .62 .95 r
.37019 .81158 .37838 .81952 .39157 .82167 .38339 .814 Metetra
.374 .634 .955 r
.38339 .814 .39157 .82167 .40489 .82352 .39673 .81614 Metetra
.387 .649 .961 r
.39673 .81614 .40489 .82352 .41834 .82501 .4102 .81793 Metetra
.403 .667 .967 r
.4102 .81793 .41834 .82501 .43192 .82608 .4238 .81929 Metetra
.423 .687 .973 r
.4238 .81929 .43192 .82608 .44562 .82667 .43753 .82018 Metetra
.446 .707 .978 r
.43753 .82018 .44562 .82667 .45945 .82674 .45139 .82053 Metetra
.47 .727 .982 r
.45139 .82053 .45945 .82674 .47339 .82626 .46537 .82031 Metetra
.496 .747 .984 r
.46537 .82031 .47339 .82626 .48744 .82519 .47947 .81949 Metetra
.523 .767 .986 r
.47947 .81949 .48744 .82519 .5016 .82351 .49368 .81803 Metetra
.55 .785 .986 r
.49368 .81803 .5016 .82351 .51586 .82123 .50799 .81594 Metetra
.576 .801 .985 r
.50799 .81594 .51586 .82123 .53021 .81832 .5224 .81319 Metetra
.601 .815 .982 r
.5224 .81319 .53021 .81832 .54465 .8148 .5369 .8098 Metetra
.952 .636 .392 r
.5369 .8098 .54465 .8148 .56036 .93232 .55239 .92612 Metetra
.903 .803 .739 r
.55239 .92612 .56036 .93232 .57519 .90681 .56732 .90016 Metetra
.896 .801 .747 r
.56732 .90016 .57519 .90681 .58993 .8827 .58216 .87564 Metetra
.888 .798 .754 r
.58216 .87564 .58993 .8827 .60462 .85993 .59694 .8525 Metetra
.88 .795 .762 r
.59694 .8525 .60462 .85993 .61926 .83844 .61167 .83066 Metetra
.872 .792 .769 r
.61167 .83066 .61926 .83844 .63386 .81817 .62636 .81007 Metetra
.863 .789 .777 r
.62636 .81007 .63386 .81817 .64845 .79906 .64102 .79067 Metetra
.854 .786 .784 r
.64102 .79067 .64845 .79906 .66303 .78103 .65568 .77239 Metetra
.845 .782 .791 r
.65568 .77239 .66303 .78103 .67761 .76404 .67034 .75517 Metetra
.835 .779 .798 r
.67034 .75517 .67761 .76404 .69222 .74803 .68501 .73896 Metetra
.826 .776 .805 r
.68501 .73896 .69222 .74803 .70685 .73293 .69971 .72368 Metetra
.817 .773 .812 r
.69971 .72368 .70685 .73293 .72151 .71868 .71445 .70929 Metetra
.807 .769 .818 r
.71445 .70929 .72151 .71868 .73622 .70525 .72923 .69573 Metetra
.798 .766 .824 r
.72923 .69573 .73622 .70525 .75099 .69256 .74406 .68293 Metetra
.789 .764 .83 r
.74406 .68293 .75099 .69256 .76581 .68057 .75895 .67085 Metetra
.78 .761 .836 r
.75895 .67085 .76581 .68057 .7807 .66922 .7739 .65944 Metetra
.771 .758 .842 r
.7739 .65944 .7807 .66922 .79567 .65848 .78893 .64864 Metetra
.763 .756 .847 r
.78893 .64864 .79567 .65848 .81071 .64829 .80404 .63841 Metetra
.754 .754 .852 r
.80404 .63841 .81071 .64829 .82583 .63861 .81923 .6287 Metetra
.747 .752 .857 r
.81923 .6287 .82583 .63861 .84104 .62939 .83451 .61946 Metetra
.739 .75 .861 r
.83451 .61946 .84104 .62939 .85633 .62061 .84987 .61066 Metetra
.732 .749 .865 r
.84987 .61066 .85633 .62061 .87173 .61221 .86533 .60225 Metetra
.725 .747 .869 r
.86533 .60225 .87173 .61221 .88721 .60417 .88089 .59421 Metetra
.719 .746 .873 r
.88089 .59421 .88721 .60417 .90279 .59645 .89654 .58649 Metetra
.713 .745 .877 r
.89654 .58649 .90279 .59645 .91848 .58901 .9123 .57906 Metetra
.348 .591 .936 r
.3359 .79754 .34417 .80619 .35711 .80895 .34885 .80053 Metetra
.346 .598 .941 r
.34885 .80053 .35711 .80895 .37019 .81158 .36193 .80343 Metetra
.349 .608 .946 r
.36193 .80343 .37019 .81158 .38339 .814 .37514 .80614 Metetra
.357 .621 .952 r
.37514 .80614 .38339 .814 .39673 .81614 .38849 .80858 Metetra
.368 .637 .958 r
.38849 .80858 .39673 .81614 .4102 .81793 .40197 .81068 Metetra
.384 .655 .965 r
.40197 .81068 .4102 .81793 .4238 .81929 .4156 .81236 Metetra
.404 .675 .971 r
.4156 .81236 .4238 .81929 .43753 .82018 .42936 .81357 Metetra
.427 .697 .977 r
.42936 .81357 .43753 .82018 .45139 .82053 .44325 .81423 Metetra
.452 .718 .982 r
.44325 .81423 .45139 .82053 .46537 .82031 .45727 .8143 Metetra
.479 .74 .985 r
.45727 .8143 .46537 .82031 .47947 .81949 .47141 .81375 Metetra
.508 .761 .987 r
.47141 .81375 .47947 .81949 .49368 .81803 .48566 .81254 Metetra
.536 .781 .988 r
.48566 .81254 .49368 .81803 .50799 .81594 .50003 .81065 Metetra
.564 .799 .987 r
.50003 .81065 .50799 .81594 .5224 .81319 .51449 .80808 Metetra
.591 .814 .985 r
.51449 .80808 .5224 .81319 .5369 .8098 .52905 .80484 Metetra
.952 .635 .391 r
.52905 .80484 .5369 .8098 .55239 .92612 .54432 .91989 Metetra
.905 .803 .737 r
.54432 .91989 .55239 .92612 .56732 .90016 .55936 .89346 Metetra
.897 .8 .745 r
.55936 .89346 .56732 .90016 .58216 .87564 .57431 .86851 Metetra
.889 .797 .752 r
.57431 .86851 .58216 .87564 .59694 .8525 .58918 .84496 Metetra
.881 .794 .76 r
.58918 .84496 .59694 .8525 .61167 .83066 .604 .82276 Metetra
.873 .791 .767 r
.604 .82276 .61167 .83066 .62636 .81007 .61878 .80184 Metetra
.864 .788 .775 r
.61878 .80184 .62636 .81007 .64102 .79067 .63352 .78213 Metetra
.855 .784 .782 r
.63352 .78213 .64102 .79067 .65568 .77239 .64826 .76358 Metetra
.845 .781 .789 r
.64826 .76358 .65568 .77239 .67034 .75517 .66299 .74613 Metetra
.836 .778 .796 r
.66299 .74613 .67034 .75517 .68501 .73896 .67774 .72971 Metetra
.826 .774 .803 r
.67774 .72971 .68501 .73896 .69971 .72368 .69251 .71426 Metetra
.817 .771 .81 r
.69251 .71426 .69971 .72368 .71445 .70929 .70731 .69971 Metetra
.807 .768 .817 r
.70731 .69971 .71445 .70929 .72923 .69573 .72216 .68602 Metetra
.798 .765 .823 r
.72216 .68602 .72923 .69573 .74406 .68293 .73705 .67312 Metetra
.788 .762 .829 r
.73705 .67312 .74406 .68293 .75895 .67085 .75201 .66096 Metetra
.779 .759 .835 r
.75201 .66096 .75895 .67085 .7739 .65944 .76703 .64948 Metetra
.77 .757 .841 r
.76703 .64948 .7739 .65944 .78893 .64864 .78213 .63863 Metetra
.762 .754 .846 r
.78213 .63863 .78893 .64864 .80404 .63841 .7973 .62836 Metetra
.753 .752 .851 r
.7973 .62836 .80404 .63841 .81923 .6287 .81256 .61862 Metetra
.746 .75 .856 r
.81256 .61862 .81923 .6287 .83451 .61946 .82791 .60936 Metetra
.738 .749 .861 r
.82791 .60936 .83451 .61946 .84987 .61066 .84334 .60055 Metetra
.731 .747 .865 r
.84334 .60055 .84987 .61066 .86533 .60225 .85887 .59215 Metetra
.724 .746 .869 r
.85887 .59215 .86533 .60225 .88089 .59421 .8745 .5841 Metetra
.718 .745 .873 r
.8745 .5841 .88089 .59421 .89654 .58649 .89023 .57639 Metetra
.712 .744 .877 r
.89023 .57639 .89654 .58649 .9123 .57906 .90605 .56897 Metetra
.336 .581 .932 r
.32756 .78864 .3359 .79754 .34885 .80053 .34051 .79187 Metetra
.332 .586 .937 r
.34051 .79187 .34885 .80053 .36193 .80343 .3536 .79504 Metetra
.333 .595 .942 r
.3536 .79504 .36193 .80343 .37514 .80614 .36682 .79805 Metetra
.338 .608 .948 r
.36682 .79805 .37514 .80614 .38849 .80858 .38017 .80081 Metetra
.349 .623 .955 r
.38017 .80081 .38849 .80858 .40197 .81068 .39368 .80325 Metetra
.364 .642 .962 r
.39368 .80325 .40197 .81068 .4156 .81236 .40732 .80528 Metetra
.383 .662 .969 r
.40732 .80528 .4156 .81236 .42936 .81357 .4211 .80682 Metetra
.406 .685 .975 r
.4211 .80682 .42936 .81357 .44325 .81423 .43502 .80782 Metetra
.432 .708 .981 r
.43502 .80782 .44325 .81423 .45727 .8143 .44907 .80821 Metetra
.46 .732 .986 r
.44907 .80821 .45727 .8143 .47141 .81375 .46325 .80796 Metetra
.49 .755 .989 r
.46325 .80796 .47141 .81375 .48566 .81254 .47756 .80702 Metetra
.521 .776 .99 r
.47756 .80702 .48566 .81254 .50003 .81065 .49197 .80537 Metetra
.551 .796 .99 r
.49197 .80537 .50003 .81065 .51449 .80808 .50649 .80301 Metetra
.58 .814 .988 r
.50649 .80301 .51449 .80808 .52905 .80484 .52112 .79992 Metetra
.952 .634 .39 r
.52112 .79992 .52905 .80484 .54432 .91989 .53616 .91363 Metetra
.906 .803 .735 r
.53616 .91363 .54432 .91989 .55936 .89346 .55132 .88671 Metetra
.899 .8 .742 r
.55132 .88671 .55936 .89346 .57431 .86851 .56637 .8613 Metetra
.891 .797 .75 r
.56637 .8613 .57431 .86851 .58918 .84496 .58134 .83732 Metetra
.882 .794 .757 r
.58134 .83732 .58918 .84496 .604 .82276 .59625 .81473 Metetra
.874 .79 .765 r
.59625 .81473 .604 .82276 .61878 .80184 .61111 .79346 Metetra
.865 .787 .773 r
.61111 .79346 .61878 .80184 .63352 .78213 .62594 .77344 Metetra
.855 .783 .78 r
.62594 .77344 .63352 .78213 .64826 .76358 .64076 .75461 Metetra
.846 .779 .787 r
.64076 .75461 .64826 .76358 .66299 .74613 .65557 .73691 Metetra
.836 .776 .794 r
.65557 .73691 .66299 .74613 .67774 .72971 .67039 .72027 Metetra
.826 .772 .801 r
.67039 .72027 .67774 .72971 .69251 .71426 .68523 .70464 Metetra
.816 .769 .808 r
.68523 .70464 .69251 .71426 .70731 .69971 .7001 .68994 Metetra
.807 .766 .815 r
.7001 .68994 .70731 .69971 .72216 .68602 .71501 .67612 Metetra
.797 .763 .821 r
.71501 .67612 .72216 .68602 .73705 .67312 .72998 .66312 Metetra
.788 .76 .828 r
.72998 .66312 .73705 .67312 .75201 .66096 .745 .65087 Metetra
.778 .757 .834 r
.745 .65087 .75201 .66096 .76703 .64948 .76009 .63933 Metetra
.769 .754 .839 r
.76009 .63933 .76703 .64948 .78213 .63863 .77525 .62843 Metetra
.761 .752 .845 r
.77525 .62843 .78213 .63863 .7973 .62836 .79049 .61813 Metetra
.752 .75 .85 r
.79049 .61813 .7973 .62836 .81256 .61862 .80582 .60837 Metetra
.744 .748 .855 r
.80582 .60837 .81256 .61862 .82791 .60936 .82123 .5991 Metetra
.737 .747 .86 r
.82123 .5991 .82791 .60936 .84334 .60055 .83674 .59029 Metetra
.729 .745 .865 r
.83674 .59029 .84334 .60055 .85887 .59215 .85234 .58189 Metetra
.723 .744 .869 r
.85234 .58189 .85887 .59215 .8745 .5841 .86804 .57385 Metetra
.716 .743 .873 r
.86804 .57385 .8745 .5841 .89023 .57639 .88384 .56615 Metetra
.71 .742 .877 r
.88384 .56615 .89023 .57639 .90605 .56897 .89974 .55875 Metetra
.325 .569 .928 r
.31916 .77949 .32756 .78864 .34051 .79187 .33212 .78295 Metetra
.318 .574 .932 r
.33212 .78295 .34051 .79187 .3536 .79504 .3452 .7864 Metetra
.316 .581 .937 r
.3452 .7864 .3536 .79504 .36682 .79805 .35842 .78972 Metetra
.319 .593 .943 r
.35842 .78972 .36682 .79805 .38017 .80081 .37179 .79283 Metetra
.328 .608 .95 r
.37179 .79283 .38017 .80081 .39368 .80325 .3853 .79563 Metetra
.341 .627 .957 r
.3853 .79563 .39368 .80325 .40732 .80528 .39895 .79802 Metetra
.36 .648 .965 r
.39895 .79802 .40732 .80528 .4211 .80682 .41276 .79994 Metetra
.383 .672 .973 r
.41276 .79994 .4211 .80682 .43502 .80782 .4267 .8013 Metetra
.409 .696 .98 r
.4267 .8013 .43502 .80782 .44907 .80821 .44079 .80205 Metetra
.439 .722 .985 r
.44079 .80205 .44907 .80821 .46325 .80796 .45501 .80212 Metetra
.47 .747 .989 r
.45501 .80212 .46325 .80796 .47756 .80702 .46935 .80148 Metetra
.503 .771 .992 r
.46935 .80148 .47756 .80702 .49197 .80537 .48382 .8001 Metetra
.536 .793 .992 r
.48382 .8001 .49197 .80537 .50649 .80301 .4984 .79796 Metetra
.567 .812 .991 r
.4984 .79796 .50649 .80301 .52112 .79992 .51308 .79506 Metetra
.951 .632 .389 r
.51308 .79506 .52112 .79992 .53616 .91363 .52791 .90735 Metetra
.908 .803 .732 r
.52791 .90735 .53616 .91363 .55132 .88671 .54318 .8799 Metetra
.9 .8 .74 r
.54318 .8799 .55132 .88671 .56637 .8613 .55834 .854 Metetra
.892 .796 .748 r
.55834 .854 .56637 .8613 .58134 .83732 .57342 .82957 Metetra
.883 .793 .755 r
.57342 .82957 .58134 .83732 .59625 .81473 .58842 .80657 Metetra
.875 .789 .763 r
.58842 .80657 .59625 .81473 .61111 .79346 .60337 .78492 Metetra
.865 .785 .77 r
.60337 .78492 .61111 .79346 .62594 .77344 .61829 .76457 Metetra
.856 .782 .778 r
.61829 .76457 .62594 .77344 .64076 .75461 .63318 .74545 Metetra
.846 .778 .785 r
.63318 .74545 .64076 .75461 .65557 .73691 .64807 .7275 Metetra
.836 .774 .792 r
.64807 .7275 .65557 .73691 .67039 .72027 .66296 .71064 Metetra
.826 .77 .8 r
.66296 .71064 .67039 .72027 .68523 .70464 .67787 .69482 Metetra
.816 .767 .807 r
.67787 .69482 .68523 .70464 .7001 .68994 .69281 .67996 Metetra
.806 .764 .813 r
.69281 .67996 .7001 .68994 .71501 .67612 .70779 .66602 Metetra
.797 .76 .82 r
.70779 .66602 .71501 .67612 .72998 .66312 .72283 .65291 Metetra
.787 .758 .826 r
.72283 .65291 .72998 .66312 .745 .65087 .73792 .64058 Metetra
.777 .755 .832 r
.73792 .64058 .745 .65087 .76009 .63933 .75307 .62898 Metetra
.768 .752 .838 r
.75307 .62898 .76009 .63933 .77525 .62843 .7683 .61804 Metetra
.759 .75 .844 r
.7683 .61804 .77525 .62843 .79049 .61813 .78361 .60771 Metetra
.751 .748 .849 r
.78361 .60771 .79049 .61813 .80582 .60837 .79901 .59794 Metetra
.743 .746 .855 r
.79901 .59794 .80582 .60837 .82123 .5991 .81449 .58867 Metetra
.735 .745 .86 r
.81449 .58867 .82123 .5991 .83674 .59029 .83007 .57986 Metetra
.728 .743 .864 r
.83007 .57986 .83674 .59029 .85234 .58189 .84574 .57147 Metetra
.721 .742 .869 r
.84574 .57147 .85234 .58189 .86804 .57385 .86151 .56345 Metetra
.715 .741 .873 r
.86151 .56345 .86804 .57385 .88384 .56615 .87739 .55577 Metetra
.709 .741 .877 r
.87739 .55577 .88384 .56615 .89974 .55875 .89337 .54839 Metetra
.314 .558 .923 r
.31071 .77004 .31916 .77949 .33212 .78295 .32366 .77375 Metetra
.304 .56 .927 r
.32366 .77375 .33212 .78295 .3452 .7864 .33674 .77749 Metetra
.299 .567 .931 r
.33674 .77749 .3452 .7864 .35842 .78972 .34996 .78114 Metetra
.299 .577 .937 r
.34996 .78114 .35842 .78972 .37179 .79283 .36333 .7846 Metetra
.306 .592 .944 r
.36333 .7846 .37179 .79283 .3853 .79563 .37684 .78778 Metetra
.317 .61 .952 r
.37684 .78778 .3853 .79563 .39895 .79802 .39051 .79058 Metetra
.335 .632 .961 r
.39051 .79058 .39895 .79802 .41276 .79994 .40433 .7929 Metetra
.357 .656 .969 r
.40433 .7929 .41276 .79994 .4267 .8013 .4183 .79466 Metetra
.384 .683 .977 r
.4183 .79466 .4267 .8013 .44079 .80205 .43241 .79579 Metetra
.415 .71 .984 r
.43241 .79579 .44079 .80205 .45501 .80212 .44667 .79623 Metetra
.448 .737 .989 r
.44667 .79623 .45501 .80212 .46935 .80148 .46106 .79592 Metetra
.483 .764 .993 r
.46106 .79592 .46935 .80148 .48382 .8001 .47557 .79484 Metetra
.518 .788 .994 r
.47557 .79484 .48382 .8001 .4984 .79796 .49021 .79296 Metetra
.553 .81 .993 r
.49021 .79296 .4984 .79796 .51308 .79506 .50495 .79027 Metetra
.951 .631 .388 r
.50495 .79027 .51308 .79506 .52791 .90735 .51957 .90105 Metetra
.909 .803 .73 r
.51957 .90105 .52791 .90735 .54318 .8799 .53496 .87304 Metetra
.902 .799 .737 r
.53496 .87304 .54318 .8799 .55834 .854 .55023 .84661 Metetra
.893 .796 .745 r
.55023 .84661 .55834 .854 .57342 .82957 .56541 .82171 Metetra
.885 .792 .753 r
.56541 .82171 .57342 .82957 .58842 .80657 .58051 .79826 Metetra
.876 .788 .76 r
.58051 .79826 .58842 .80657 .60337 .78492 .59555 .77623 Metetra
.866 .784 .768 r
.59555 .77623 .60337 .78492 .61829 .76457 .61055 .75552 Metetra
.856 .78 .775 r
.61055 .75552 .61829 .76457 .63318 .74545 .62552 .7361 Metetra
.847 .776 .783 r
.62552 .7361 .63318 .74545 .64807 .7275 .64049 .71787 Metetra
.836 .772 .79 r
.64049 .71787 .64807 .7275 .66296 .71064 .65545 .70079 Metetra
.826 .768 .798 r
.65545 .70079 .66296 .71064 .67787 .69482 .67044 .68478 Metetra
.816 .765 .805 r
.67044 .68478 .67787 .69482 .69281 .67996 .68545 .66977 Metetra
.806 .761 .812 r
.68545 .66977 .69281 .67996 .70779 .66602 .7005 .65569 Metetra
.796 .758 .818 r
.7005 .65569 .70779 .66602 .72283 .65291 .7156 .64248 Metetra
.786 .755 .825 r
.7156 .64248 .72283 .65291 .73792 .64058 .73076 .63008 Metetra
.776 .752 .831 r
.73076 .63008 .73792 .64058 .75307 .62898 .74598 .61843 Metetra
.767 .75 .837 r
.74598 .61843 .75307 .62898 .7683 .61804 .76128 .60745 Metetra
.758 .748 .843 r
.76128 .60745 .7683 .61804 .78361 .60771 .77666 .5971 Metetra
.749 .746 .849 r
.77666 .5971 .78361 .60771 .79901 .59794 .79212 .58732 Metetra
.741 .744 .854 r
.79212 .58732 .79901 .59794 .81449 .58867 .80768 .57806 Metetra
.733 .743 .859 r
.80768 .57806 .81449 .58867 .83007 .57986 .82332 .56927 Metetra
.726 .741 .864 r
.82332 .56927 .83007 .57986 .84574 .57147 .83907 .5609 Metetra
.719 .74 .868 r
.83907 .5609 .84574 .57147 .86151 .56345 .85492 .5529 Metetra
.713 .74 .873 r
.85492 .5529 .86151 .56345 .87739 .55577 .87086 .54525 Metetra
.707 .739 .877 r
.87086 .54525 .87739 .55577 .89337 .54839 .88692 .53789 Metetra
.303 .547 .918 r
.30219 .7603 .31071 .77004 .32366 .77375 .31514 .76425 Metetra
.29 .547 .921 r
.31514 .76425 .32366 .77375 .33674 .77749 .32821 .76828 Metetra
.282 .551 .925 r
.32821 .76828 .33674 .77749 .34996 .78114 .34143 .77227 Metetra
.279 .56 .931 r
.34143 .77227 .34996 .78114 .36333 .7846 .35479 .77611 Metetra
.282 .574 .938 r
.35479 .77611 .36333 .7846 .37684 .78778 .36831 .7797 Metetra
.292 .591 .946 r
.36831 .7797 .37684 .78778 .39051 .79058 .38199 .78293 Metetra
.307 .613 .955 r
.38199 .78293 .39051 .79058 .40433 .7929 .39582 .78568 Metetra
.329 .639 .964 r
.39582 .78568 .40433 .7929 .4183 .79466 .4098 .78788 Metetra
.356 .667 .974 r
.4098 .78788 .4183 .79466 .43241 .79579 .42394 .78943 Metetra
.387 .696 .982 r
.42394 .78943 .43241 .79579 .44667 .79623 .43823 .79027 Metetra
.422 .726 .989 r
.43823 .79027 .44667 .79623 .46106 .79592 .45266 .79034 Metetra
.459 .755 .993 r
.45266 .79034 .46106 .79592 .47557 .79484 .46722 .78959 Metetra
.498 .782 .996 r
.46722 .78959 .47557 .79484 .49021 .79296 .48191 .788 Metetra
.535 .807 .996 r
.48191 .788 .49021 .79296 .50495 .79027 .49672 .78555 Metetra
.95 .63 .387 r
.49672 .78555 .50495 .79027 .51957 .90105 .51113 .89473 Metetra
.911 .803 .727 r
.51113 .89473 .51957 .90105 .53496 .87304 .52664 .86612 Metetra
.903 .799 .735 r
.52664 .86612 .53496 .87304 .55023 .84661 .54203 .83913 Metetra
.895 .795 .742 r
.54203 .83913 .55023 .84661 .56541 .82171 .55731 .81372 Metetra
.886 .791 .75 r
.55731 .81372 .56541 .82171 .58051 .79826 .57251 .78981 Metetra
.877 .787 .758 r
.57251 .78981 .58051 .79826 .59555 .77623 .58764 .76735 Metetra
.867 .782 .765 r
.58764 .76735 .59555 .77623 .61055 .75552 .60273 .74628 Metetra
.857 .778 .773 r
.60273 .74628 .61055 .75552 .62552 .7361 .61779 .72653 Metetra
.847 .774 .781 r
.61779 .72653 .62552 .7361 .64049 .71787 .63283 .70803 Metetra
.837 .77 .788 r
.63283 .70803 .64049 .71787 .65545 .70079 .64787 .69071 Metetra
.826 .766 .795 r
.64787 .69071 .65545 .70079 .67044 .68478 .66292 .6745 Metetra
.816 .762 .803 r
.66292 .6745 .67044 .68478 .68545 .66977 .678 .65933 Metetra
.805 .759 .81 r
.678 .65933 .68545 .66977 .7005 .65569 .69312 .64513 Metetra
.795 .755 .817 r
.69312 .64513 .7005 .65569 .7156 .64248 .70829 .63183 Metetra
.785 .752 .823 r
.70829 .63183 .7156 .64248 .73076 .63008 .72352 .61936 Metetra
.775 .75 .83 r
.72352 .61936 .73076 .63008 .74598 .61843 .73881 .60765 Metetra
.766 .747 .836 r
.73881 .60765 .74598 .61843 .76128 .60745 .75418 .59665 Metetra
.757 .745 .842 r
.75418 .59665 .76128 .60745 .77666 .5971 .76963 .58629 Metetra
.748 .743 .848 r
.76963 .58629 .77666 .5971 .79212 .58732 .78516 .57652 Metetra
.739 .742 .853 r
.78516 .57652 .79212 .58732 .80768 .57806 .80079 .56727 Metetra
.732 .74 .859 r
.80079 .56727 .80768 .57806 .82332 .56927 .81651 .5585 Metetra
.724 .739 .864 r
.81651 .5585 .82332 .56927 .83907 .5609 .83233 .55016 Metetra
.717 .738 .868 r
.83233 .55016 .83907 .5609 .85492 .5529 .84825 .5422 Metetra
.711 .738 .873 r
.84825 .5422 .85492 .5529 .87086 .54525 .86427 .53458 Metetra
.705 .737 .877 r
.86427 .53458 .87086 .54525 .88692 .53789 .8804 .52726 Metetra
.294 .535 .912 r
.29362 .75023 .30219 .7603 .31514 .76425 .30656 .75442 Metetra
.277 .533 .915 r
.30656 .75442 .31514 .76425 .32821 .76828 .31963 .75874 Metetra
.265 .535 .918 r
.31963 .75874 .32821 .76828 .34143 .77227 .33283 .76309 Metetra
.258 .542 .923 r
.33283 .76309 .34143 .77227 .35479 .77611 .34619 .76733 Metetra
.258 .554 .93 r
.34619 .76733 .35479 .77611 .36831 .7797 .35971 .77135 Metetra
.264 .571 .938 r
.35971 .77135 .36831 .7797 .38199 .78293 .37338 .77504 Metetra
.278 .593 .948 r
.37338 .77504 .38199 .78293 .39582 .78568 .38722 .77827 Metetra
.298 .618 .958 r
.38722 .77827 .39582 .78568 .4098 .78788 .40122 .78094 Metetra
.324 .648 .968 r
.40122 .78094 .4098 .78788 .42394 .78943 .41538 .78296 Metetra
.356 .679 .978 r
.41538 .78296 .42394 .78943 .43823 .79027 .4297 .78425 Metetra
.393 .712 .986 r
.4297 .78425 .43823 .79027 .45266 .79034 .44416 .78473 Metetra
.432 .744 .993 r
.44416 .78473 .45266 .79034 .46722 .78959 .45877 .78435 Metetra
.474 .775 .997 r
.45877 .78435 .46722 .78959 .48191 .788 .47352 .78309 Metetra
.515 .802 .998 r
.47352 .78309 .48191 .788 .49672 .78555 .48838 .78092 Metetra
.95 .628 .385 r
.48838 .78092 .49672 .78555 .51113 .89473 .50258 .88841 Metetra
.913 .802 .724 r
.50258 .88841 .51113 .89473 .52664 .86612 .51822 .85915 Metetra
.905 .798 .732 r
.51822 .85915 .52664 .86612 .54203 .83913 .53373 .83156 Metetra
.896 .794 .739 r
.53373 .83156 .54203 .83913 .55731 .81372 .54912 .80559 Metetra
.887 .789 .747 r
.54912 .80559 .55731 .81372 .57251 .78981 .56443 .78118 Metetra
.878 .785 .755 r
.56443 .78118 .57251 .78981 .58764 .76735 .57965 .75829 Metetra
.868 .78 .763 r
.57965 .75829 .58764 .76735 .60273 .74628 .59483 .73683 Metetra
.858 .776 .77 r
.59483 .73683 .60273 .74628 .61779 .72653 .60997 .71674 Metetra
.847 .772 .778 r
.60997 .71674 .61779 .72653 .63283 .70803 .62509 .69795 Metetra
.837 .767 .786 r
.62509 .69795 .63283 .70803 .64787 .69071 .6402 .68039 Metetra
.826 .763 .793 r
.6402 .68039 .64787 .69071 .66292 .6745 .65533 .66398 Metetra
.815 .76 .801 r
.65533 .66398 .66292 .6745 .678 .65933 .67049 .64865 Metetra
.805 .756 .808 r
.67049 .64865 .678 .65933 .69312 .64513 .68567 .63432 Metetra
.794 .753 .815 r
.68567 .63432 .69312 .64513 .70829 .63183 .70091 .62093 Metetra
.784 .75 .822 r
.70091 .62093 .70829 .63183 .72352 .61936 .71621 .6084 Metetra
.774 .747 .828 r
.71621 .6084 .72352 .61936 .73881 .60765 .73157 .59665 Metetra
.764 .745 .835 r
.73157 .59665 .73881 .60765 .75418 .59665 .74701 .58563 Metetra
.755 .742 .841 r
.74701 .58563 .75418 .59665 .76963 .58629 .76252 .57527 Metetra
.746 .741 .847 r
.76252 .57527 .76963 .58629 .78516 .57652 .77813 .56551 Metetra
.737 .739 .853 r
.77813 .56551 .78516 .57652 .80079 .56727 .79382 .55629 Metetra
.729 .738 .858 r
.79382 .55629 .80079 .56727 .81651 .5585 .80962 .54755 Metetra
.722 .737 .863 r
.80962 .54755 .81651 .5585 .83233 .55016 .82551 .53925 Metetra
.715 .736 .868 r
.82551 .53925 .83233 .55016 .84825 .5422 .84151 .53133 Metetra
.708 .736 .873 r
.84151 .53133 .84825 .5422 .86427 .53458 .85761 .52376 Metetra
.702 .736 .877 r
.85761 .52376 .86427 .53458 .8804 .52726 .87382 .51649 Metetra
.286 .524 .906 r
.285 .73981 .29362 .75023 .30656 .75442 .29793 .74423 Metetra
.265 .519 .908 r
.29793 .74423 .30656 .75442 .31963 .75874 .31098 .74885 Metetra
.248 .518 .911 r
.31098 .74885 .31963 .75874 .33283 .76309 .32418 .75356 Metetra
.237 .523 .915 r
.32418 .75356 .33283 .76309 .34619 .76733 .33752 .75822 Metetra
.233 .533 .921 r
.33752 .75822 .34619 .76733 .35971 .77135 .35103 .76271 Metetra
.235 .548 .929 r
.35103 .76271 .35971 .77135 .37338 .77504 .3647 .76688 Metetra
.246 .569 .939 r
.3647 .76688 .37338 .77504 .38722 .77827 .37854 .77063 Metetra
.264 .595 .95 r
.37854 .77063 .38722 .77827 .40122 .78094 .39255 .77383 Metetra
.289 .626 .961 r
.39255 .77383 .40122 .78094 .41538 .78296 .40672 .77636 Metetra
.321 .659 .972 r
.40672 .77636 .41538 .78296 .4297 .78425 .42107 .77814 Metetra
.359 .695 .983 r
.42107 .77814 .4297 .78425 .44416 .78473 .43557 .77909 Metetra
.401 .73 .991 r
.43557 .77909 .44416 .78473 .45877 .78435 .45022 .77914 Metetra
.445 .765 .996 r
.45022 .77914 .45877 .78435 .47352 .78309 .46501 .77825 Metetra
.491 .796 .999 r
.46501 .77825 .47352 .78309 .48838 .78092 .47993 .77639 Metetra
.949 .626 .384 r
.47993 .77639 .48838 .78092 .50258 .88841 .49394 .88207 Metetra
.915 .802 .721 r
.49394 .88207 .50258 .88841 .51822 .85915 .50971 .85211 Metetra
.907 .798 .729 r
.50971 .85211 .51822 .85915 .53373 .83156 .52534 .82388 Metetra
.898 .793 .736 r
.52534 .82388 .53373 .83156 .54912 .80559 .54085 .79732 Metetra
.888 .788 .744 r
.54085 .79732 .54912 .80559 .56443 .78118 .55626 .77238 Metetra
.879 .783 .752 r
.55626 .77238 .56443 .78118 .57965 .75829 .57158 .74901 Metetra
.868 .778 .76 r
.57158 .74901 .57965 .75829 .59483 .73683 .58685 .72714 Metetra
.858 .774 .768 r
.58685 .72714 .59483 .73683 .60997 .71674 .60207 .70669 Metetra
.847 .769 .775 r
.60207 .70669 .60997 .71674 .62509 .69795 .61727 .6876 Metetra
.836 .765 .783 r
.61727 .6876 .62509 .69795 .6402 .68039 .63246 .66979 Metetra
.826 .76 .791 r
.63246 .66979 .6402 .68039 .65533 .66398 .64766 .65318 Metetra
.815 .757 .798 r
.64766 .65318 .65533 .66398 .67049 .64865 .66289 .63769 Metetra
.804 .753 .806 r
.66289 .63769 .67049 .64865 .68567 .63432 .67815 .62324 Metetra
.793 .75 .813 r
.67815 .62324 .68567 .63432 .70091 .62093 .69345 .60977 Metetra
.782 .746 .82 r
.69345 .60977 .70091 .62093 .71621 .6084 .70882 .59718 Metetra
.772 .744 .827 r
.70882 .59718 .71621 .6084 .73157 .59665 .72425 .58541 Metetra
.762 .741 .834 r
.72425 .58541 .73157 .59665 .74701 .58563 .73975 .57438 Metetra
.753 .739 .84 r
.73975 .57438 .74701 .58563 .76252 .57527 .75534 .56403 Metetra
.744 .738 .846 r
.75534 .56403 .76252 .57527 .77813 .56551 .77102 .5543 Metetra
.735 .736 .852 r
.77102 .5543 .77813 .56551 .79382 .55629 .78679 .54512 Metetra
.727 .735 .858 r
.78679 .54512 .79382 .55629 .80962 .54755 .80265 .53643 Metetra
.72 .735 .863 r
.80265 .53643 .80962 .54755 .82551 .53925 .81862 .52818 Metetra
.712 .734 .868 r
.81862 .52818 .82551 .53925 .84151 .53133 .8347 .52031 Metetra
.706 .734 .873 r
.8347 .52031 .84151 .53133 .85761 .52376 .85088 .5128 Metetra
.7 .734 .877 r
.85088 .5128 .85761 .52376 .87382 .51649 .86716 .50558 Metetra
.279 .513 .9 r
.27633 .72902 .285 .73981 .29793 .74423 .28924 .73366 Metetra
.253 .505 .901 r
.28924 .73366 .29793 .74423 .31098 .74885 .30228 .73858 Metetra
.232 .501 .902 r
.30228 .73858 .31098 .74885 .32418 .75356 .31546 .74365 Metetra
.216 .503 .906 r
.31546 .74365 .32418 .75356 .33752 .75822 .32879 .74875 Metetra
.207 .51 .911 r
.32879 .74875 .33752 .75822 .35103 .76271 .34228 .75372 Metetra
.205 .524 .919 r
.34228 .75372 .35103 .76271 .3647 .76688 .35594 .75843 Metetra
.211 .543 .928 r
.35594 .75843 .3647 .76688 .37854 .77063 .36977 .76274 Metetra
.226 .569 .939 r
.36977 .76274 .37854 .77063 .39255 .77383 .38379 .76651 Metetra
.25 .6 .952 r
.38379 .76651 .39255 .77383 .40672 .77636 .39797 .76961 Metetra
.281 .635 .964 r
.39797 .76961 .40672 .77636 .42107 .77814 .41234 .77195 Metetra
.32 .674 .976 r
.41234 .77195 .42107 .77814 .43557 .77909 .42687 .77341 Metetra
.364 .713 .987 r
.42687 .77341 .43557 .77909 .45022 .77914 .44155 .77394 Metetra
.412 .752 .994 r
.44155 .77394 .45022 .77914 .46501 .77825 .45639 .77347 Metetra
.462 .788 .999 r
.45639 .77347 .46501 .77825 .47993 .77639 .47137 .77197 Metetra
.948 .625 .382 r
.47137 .77197 .47993 .77639 .49394 .88207 .48519 .87574 Metetra
.917 .802 .718 r
.48519 .87574 .49394 .88207 .50971 .85211 .5011 .84502 Metetra
.908 .797 .725 r
.5011 .84502 .50971 .85211 .52534 .82388 .51686 .81608 Metetra
.899 .792 .733 r
.51686 .81608 .52534 .82388 .54085 .79732 .53249 .78888 Metetra
.89 .787 .741 r
.53249 .78888 .54085 .79732 .55626 .77238 .548 .76338 Metetra
.88 .781 .749 r
.548 .76338 .55626 .77238 .57158 .74901 .56342 .7395 Metetra
.869 .776 .757 r
.56342 .7395 .57158 .74901 .58685 .72714 .57878 .71719 Metetra
.858 .771 .765 r
.57878 .71719 .58685 .72714 .60207 .70669 .59409 .69637 Metetra
.847 .766 .772 r
.59409 .69637 .60207 .70669 .61727 .6876 .60937 .67697 Metetra
.836 .762 .78 r
.60937 .67697 .61727 .6876 .63246 .66979 .62464 .6589 Metetra
.825 .757 .788 r
.62464 .6589 .63246 .66979 .64766 .65318 .63991 .64208 Metetra
.814 .753 .796 r
.63991 .64208 .64766 .65318 .66289 .63769 .65521 .62644 Metetra
.803 .749 .804 r
.65521 .62644 .66289 .63769 .67815 .62324 .67054 .61188 Metetra
.792 .746 .811 r
.67054 .61188 .67815 .62324 .69345 .60977 .68591 .59832 Metetra
.781 .743 .818 r
.68591 .59832 .69345 .60977 .70882 .59718 .70135 .58569 Metetra
.77 .74 .826 r
.70135 .58569 .70882 .59718 .72425 .58541 .71685 .57391 Metetra
.76 .738 .832 r
.71685 .57391 .72425 .58541 .73975 .57438 .73242 .56289 Metetra
.751 .736 .839 r
.73242 .56289 .73975 .57438 .75534 .56403 .74808 .55257 Metetra
.741 .735 .845 r
.74808 .55257 .75534 .56403 .77102 .5543 .76383 .54287 Metetra
.733 .734 .852 r
.76383 .54287 .77102 .5543 .78679 .54512 .77967 .53374 Metetra
.724 .733 .857 r
.77967 .53374 .78679 .54512 .80265 .53643 .79561 .52511 Metetra
.717 .732 .863 r
.79561 .52511 .80265 .53643 .81862 .52818 .81166 .51693 Metetra
.71 .732 .868 r
.81166 .51693 .81862 .52818 .8347 .52031 .82781 .50914 Metetra
.703 .732 .873 r
.82781 .50914 .8347 .52031 .85088 .5128 .84407 .50169 Metetra
.697 .732 .878 r
.84407 .50169 .85088 .5128 .86716 .50558 .86044 .49453 Metetra
.275 .503 .894 r
.26761 .71782 .27633 .72902 .28924 .73366 .28051 .72266 Metetra
.244 .491 .893 r
.28051 .72266 .28924 .73366 .30228 .73858 .29353 .72787 Metetra
.217 .484 .894 r
.29353 .72787 .30228 .73858 .31546 .74365 .30668 .73332 Metetra
.195 .482 .896 r
.30668 .73332 .31546 .74365 .32879 .74875 .31999 .73887 Metetra
.181 .486 .9 r
.31999 .73887 .32879 .74875 .34228 .75372 .33346 .74437 Metetra
.173 .497 .906 r
.33346 .74437 .34228 .75372 .35594 .75843 .34711 .74965 Metetra
.175 .514 .915 r
.34711 .74965 .35594 .75843 .36977 .76274 .36093 .75456 Metetra
.186 .539 .926 r
.36093 .75456 .36977 .76274 .38379 .76651 .37494 .75896 Metetra
.206 .57 .939 r
.37494 .75896 .38379 .76651 .39797 .76961 .38913 .76269 Metetra
.236 .607 .953 r
.38913 .76269 .39797 .76961 .41234 .77195 .4035 .76564 Metetra
.275 .648 .967 r
.4035 .76564 .41234 .77195 .42687 .77341 .41806 .7677 Metetra
.321 .691 .98 r
.41806 .7677 .42687 .77341 .44155 .77394 .43278 .76877 Metetra
.373 .735 .99 r
.43278 .76877 .44155 .77394 .45639 .77347 .44767 .76878 Metetra
.427 .776 .997 r
.44767 .76878 .45639 .77347 .47137 .77197 .4627 .76769 Metetra
.947 .622 .38 r
.4627 .76769 .47137 .77197 .48519 .87574 .47634 .8694 Metetra
.919 .801 .714 r
.47634 .8694 .48519 .87574 .5011 .84502 .4924 .83786 Metetra
.91 .796 .722 r
.4924 .83786 .5011 .84502 .51686 .81608 .50829 .80816 Metetra
.901 .79 .73 r
.50829 .80816 .51686 .81608 .53249 .78888 .52403 .78028 Metetra
.891 .785 .737 r
.52403 .78028 .53249 .78888 .548 .76338 .53965 .75416 Metetra
.88 .779 .745 r
.53965 .75416 .548 .76338 .56342 .7395 .55518 .72974 Metetra
.87 .774 .753 r
.55518 .72974 .56342 .7395 .57878 .71719 .57063 .70696 Metetra
.859 .768 .761 r
.57063 .70696 .57878 .71719 .59409 .69637 .58603 .68575 Metetra
.847 .763 .769 r
.58603 .68575 .59409 .69637 .60937 .67697 .60139 .66602 Metetra
.836 .758 .777 r
.60139 .66602 .60937 .67697 .62464 .6589 .61673 .64768 Metetra
.824 .754 .785 r
.61673 .64768 .62464 .6589 .63991 .64208 .63208 .63066 Metetra
.813 .75 .793 r
.63208 .63066 .63991 .64208 .65521 .62644 .64745 .61486 Metetra
.801 .746 .801 r
.64745 .61486 .65521 .62644 .67054 .61188 .66285 .6002 Metetra
.79 .742 .809 r
.66285 .6002 .67054 .61188 .68591 .59832 .67829 .58658 Metetra
.779 .739 .817 r
.67829 .58658 .68591 .59832 .70135 .58569 .6938 .57392 Metetra
.768 .737 .824 r
.6938 .57392 .70135 .58569 .71685 .57391 .70937 .56213 Metetra
.758 .735 .831 r
.70937 .56213 .71685 .57391 .73242 .56289 .72501 .55114 Metetra
.748 .733 .838 r
.72501 .55114 .73242 .56289 .74808 .55257 .74074 .54086 Metetra
.739 .732 .845 r
.74074 .54086 .74808 .55257 .76383 .54287 .75656 .53123 Metetra
.73 .731 .851 r
.75656 .53123 .76383 .54287 .77967 .53374 .77248 .52217 Metetra
.722 .73 .857 r
.77248 .52217 .77967 .53374 .79561 .52511 .7885 .51362 Metetra
.714 .73 .863 r
.7885 .51362 .79561 .52511 .81166 .51693 .80462 .50551 Metetra
.707 .73 .868 r
.80462 .50551 .81166 .51693 .82781 .50914 .82085 .4978 Metetra
.7 .73 .873 r
.82085 .4978 .82781 .50914 .84407 .50169 .83719 .49043 Metetra
.694 .73 .878 r
.83719 .49043 .84407 .50169 .86044 .49453 .85364 .48336 Metetra
.273 .493 .888 r
.25885 .7062 .26761 .71782 .28051 .72266 .27173 .71121 Metetra
.236 .478 .886 r
.27173 .71121 .28051 .72266 .29353 .72787 .28473 .7167 Metetra
.204 .466 .884 r
.28473 .7167 .29353 .72787 .30668 .73332 .29786 .72253 Metetra
.176 .46 .885 r
.29786 .72253 .30668 .73332 .31999 .73887 .31114 .72854 Metetra
.155 .461 .887 r
.31114 .72854 .31999 .73887 .33346 .74437 .32458 .73458 Metetra
.141 .468 .892 r
.32458 .73458 .33346 .74437 .34711 .74965 .3382 .74048 Metetra
.136 .482 .9 r
.3382 .74048 .34711 .74965 .36093 .75456 .352 .74605 Metetra
.142 .505 .91 r
.352 .74605 .36093 .75456 .37494 .75896 .366 .75113 Metetra
.158 .536 .923 r
.366 .75113 .37494 .75896 .38913 .76269 .38019 .75557 Metetra
.185 .573 .938 r
.38019 .75557 .38913 .76269 .4035 .76564 .39457 .75921 Metetra
.223 .617 .954 r
.39457 .75921 .4035 .76564 .41806 .7677 .40914 .76193 Metetra
.271 .664 .969 r
.40914 .76193 .41806 .7677 .43278 .76877 .42389 .76361 Metetra
.325 .712 .982 r
.42389 .76361 .43278 .76877 .44767 .76878 .43882 .76417 Metetra
.385 .759 .991 r
.43882 .76417 .44767 .76878 .4627 .76769 .45391 .76356 Metetra
.947 .62 .379 r
.45391 .76356 .4627 .76769 .47634 .8694 .46737 .86308 Metetra
.921 .801 .711 r
.46737 .86308 .47634 .8694 .4924 .83786 .48358 .83063 Metetra
.912 .795 .718 r
.48358 .83063 .4924 .83786 .50829 .80816 .49961 .80011 Metetra
.902 .789 .726 r
.49961 .80011 .50829 .80816 .52403 .78028 .51548 .77148 Metetra
.892 .783 .734 r
.51548 .77148 .52403 .78028 .53965 .75416 .53122 .74469 Metetra
.881 .777 .742 r
.53122 .74469 .53965 .75416 .55518 .72974 .54685 .7197 Metetra
.87 .771 .75 r
.54685 .7197 .55518 .72974 .57063 .70696 .5624 .69642 Metetra
.859 .765 .758 r
.5624 .69642 .57063 .70696 .58603 .68575 .57788 .67479 Metetra
.847 .76 .766 r
.57788 .67479 .58603 .68575 .60139 .66602 .59332 .65472 Metetra
.835 .755 .774 r
.59332 .65472 .60139 .66602 .61673 .64768 .60875 .63612 Metetra
.823 .75 .783 r
.60875 .63612 .61673 .64768 .63208 .63066 .62417 .61889 Metetra
.811 .745 .791 r
.62417 .61889 .63208 .63066 .64745 .61486 .63961 .60294 Metetra
.799 .742 .799 r
.63961 .60294 .64745 .61486 .66285 .6002 .65508 .58818 Metetra
.788 .738 .807 r
.65508 .58818 .66285 .6002 .67829 .58658 .67059 .57451 Metetra
.777 .735 .815 r
.67059 .57451 .67829 .58658 .6938 .57392 .68617 .56184 Metetra
.766 .733 .822 r
.68617 .56184 .6938 .57392 .70937 .56213 .70181 .55007 Metetra
.755 .731 .83 r
.70181 .55007 .70937 .56213 .72501 .55114 .71752 .53912 Metetra
.745 .729 .837 r
.71752 .53912 .72501 .55114 .74074 .54086 .73333 .52891 Metetra
.736 .728 .844 r
.73333 .52891 .74074 .54086 .75656 .53123 .74922 .51936 Metetra
.727 .727 .851 r
.74922 .51936 .75656 .53123 .77248 .52217 .76521 .51039 Metetra
.718 .727 .857 r
.76521 .51039 .77248 .52217 .7885 .51362 .78131 .50193 Metetra
.711 .727 .863 r
.78131 .50193 .7885 .51362 .80462 .50551 .79751 .49392 Metetra
.704 .727 .869 r
.79751 .49392 .80462 .50551 .82085 .4978 .81382 .48631 Metetra
.697 .728 .874 r
.81382 .48631 .82085 .4978 .83719 .49043 .83024 .47904 Metetra
.691 .729 .879 r
.83024 .47904 .83719 .49043 .85364 .48336 .84678 .47205 Metetra
.273 .485 .882 r
.25005 .69412 .25885 .7062 .27173 .71121 .26292 .69927 Metetra
.231 .465 .878 r
.26292 .69927 .27173 .71121 .28473 .7167 .27589 .70501 Metetra
.192 .45 .875 r
.27589 .70501 .28473 .7167 .29786 .72253 .28899 .71121 Metetra
.158 .439 .873 r
.28899 .71121 .29786 .72253 .31114 .72854 .30224 .7177 Metetra
.129 .434 .873 r
.30224 .7177 .31114 .72854 .32458 .73458 .31565 .72431 Metetra
.108 .437 .876 r
.31565 .72431 .32458 .73458 .3382 .74048 .32923 .73086 Metetra
.096 .448 .882 r
.32923 .73086 .3382 .74048 .352 .74605 .34301 .73715 Metetra
.095 .467 .891 r
.34301 .73715 .352 .74605 .366 .75113 .35698 .74299 Metetra
.106 .496 .903 r
.35698 .74299 .366 .75113 .38019 .75557 .37115 .74821 Metetra
.129 .533 .918 r
.37115 .74821 .38019 .75557 .39457 .75921 .38553 .75262 Metetra
.164 .578 .935 r
.38553 .75262 .39457 .75921 .40914 .76193 .40012 .75609 Metetra
.212 .629 .952 r
.40012 .75609 .40914 .76193 .42389 .76361 .41489 .75847 Metetra
.269 .683 .968 r
.41489 .75847 .42389 .76361 .43882 .76417 .42985 .75967 Metetra
.334 .737 .981 r
.42985 .75967 .43882 .76417 .45391 .76356 .44499 .7596 Metetra
.946 .618 .376 r
.44499 .7596 .45391 .76356 .46737 .86308 .4583 .85677 Metetra
.923 .8 .706 r
.4583 .85677 .46737 .86308 .48358 .83063 .47467 .82333 Metetra
.914 .794 .714 r
.47467 .82333 .48358 .83063 .49961 .80011 .49084 .7919 Metetra
.904 .787 .722 r
.49084 .7919 .49961 .80011 .51548 .77148 .50684 .76245 Metetra
.893 .78 .73 r
.50684 .76245 .51548 .77148 .53122 .74469 .5227 .73495 Metetra
.882 .774 .738 r
.5227 .73495 .53122 .74469 .54685 .7197 .53844 .70933 Metetra
.87 .768 .746 r
.53844 .70933 .54685 .7197 .5624 .69642 .55408 .68553 Metetra
.859 .761 .754 r
.55408 .68553 .5624 .69642 .57788 .67479 .56965 .66346 Metetra
.846 .756 .763 r
.56965 .66346 .57788 .67479 .59332 .65472 .58518 .64303 Metetra
.834 .75 .771 r
.58518 .64303 .59332 .65472 .60875 .63612 .60068 .62415 Metetra
.822 .745 .78 r
.60068 .62415 .60875 .63612 .62417 .61889 .61618 .60672 Metetra
.809 .741 .788 r
.61618 .60672 .62417 .61889 .63961 .60294 .63169 .59064 Metetra
.797 .737 .796 r
.63169 .59064 .63961 .60294 .65508 .58818 .64723 .5758 Metetra
.785 .733 .805 r
.64723 .5758 .65508 .58818 .67059 .57451 .66281 .5621 Metetra
.774 .731 .813 r
.66281 .5621 .67059 .57451 .68617 .56184 .67846 .54943 Metetra
.763 .728 .821 r
.67846 .54943 .68617 .56184 .70181 .55007 .69417 .53771 Metetra
.752 .726 .829 r
.69417 .53771 .70181 .55007 .71752 .53912 .70996 .52683 Metetra
.742 .725 .836 r
.70996 .52683 .71752 .53912 .73333 .52891 .72583 .5167 Metetra
.732 .724 .843 r
.72583 .5167 .73333 .52891 .74922 .51936 .7418 .50725 Metetra
.723 .724 .85 r
.7418 .50725 .74922 .51936 .76521 .51039 .75787 .49839 Metetra
.715 .724 .857 r
.75787 .49839 .76521 .51039 .78131 .50193 .77404 .49005 Metetra
.707 .724 .863 r
.77404 .49005 .78131 .50193 .79751 .49392 .79032 .48217 Metetra
.7 .725 .869 r
.79032 .48217 .79751 .49392 .81382 .48631 .80671 .47467 Metetra
.694 .726 .875 r
.80671 .47467 .81382 .48631 .83024 .47904 .82322 .46751 Metetra
.688 .727 .88 r
.82322 .46751 .83024 .47904 .84678 .47205 .83984 .46063 Metetra
.277 .478 .875 r
.24122 .68154 .25005 .69412 .26292 .69927 .25408 .68679 Metetra
.23 .454 .87 r
.25408 .68679 .26292 .69927 .27589 .70501 .26702 .69275 Metetra
.184 .433 .865 r
.26702 .69275 .27589 .70501 .28899 .71121 .28009 .69931 Metetra
.142 .417 .861 r
.28009 .69931 .28899 .71121 .30224 .7177 .2933 .70628 Metetra
.106 .407 .858 r
.2933 .70628 .30224 .7177 .31565 .72431 .30666 .71349 Metetra
.076 .404 .858 r
.30666 .71349 .31565 .72431 .32923 .73086 .3202 .72074 Metetra
.056 .41 .861 r
.3202 .72074 .32923 .73086 .34301 .73715 .33394 .7278 Metetra
.046 .425 .868 r
.33394 .7278 .34301 .73715 .35698 .74299 .34788 .73448 Metetra
.049 .451 .878 r
.34788 .73448 .35698 .74299 .37115 .74821 .36203 .74056 Metetra
.065 .487 .892 r
.36203 .74056 .37115 .74821 .38553 .75262 .3764 .74585 Metetra
.097 .532 .909 r
.3764 .74585 .38553 .75262 .40012 .75609 .39098 .75016 Metetra
.143 .586 .928 r
.39098 .75016 .40012 .75609 .41489 .75847 .40577 .75334 Metetra
.202 .645 .947 r
.40577 .75334 .41489 .75847 .42985 .75967 .42076 .75527 Metetra
.271 .706 .964 r
.42076 .75527 .42985 .75967 .44499 .7596 .43594 .75585 Metetra
.944 .615 .374 r
.43594 .75585 .44499 .7596 .4583 .85677 .4491 .85048 Metetra
.926 .8 .702 r
.4491 .85048 .4583 .85677 .47467 .82333 .46565 .81595 Metetra
.916 .792 .71 r
.46565 .81595 .47467 .82333 .49084 .7919 .48197 .78352 Metetra
.905 .785 .717 r
.48197 .78352 .49084 .7919 .50684 .76245 .49811 .75318 Metetra
.894 .778 .725 r
.49811 .75318 .50684 .76245 .5227 .73495 .51409 .72489 Metetra
.883 .771 .734 r
.51409 .72489 .5227 .73495 .53844 .70933 .52994 .6986 Metetra
.871 .764 .742 r
.52994 .6986 .53844 .70933 .55408 .68553 .54568 .67423 Metetra
.858 .757 .75 r
.54568 .67423 .55408 .68553 .56965 .66346 .56134 .6517 Metetra
.846 .751 .759 r
.56134 .6517 .56965 .66346 .58518 .64303 .57695 .6309 Metetra
.833 .745 .768 r
.57695 .6309 .58518 .64303 .60068 .62415 .59253 .61175 Metetra
.82 .74 .776 r
.59253 .61175 .60068 .62415 .61618 .60672 .60811 .59413 Metetra
.807 .736 .785 r
.60811 .59413 .61618 .60672 .63169 .59064 .62369 .57792 Metetra
.795 .732 .794 r
.62369 .57792 .63169 .59064 .64723 .5758 .6393 .56301 Metetra
.783 .728 .802 r
.6393 .56301 .64723 .5758 .66281 .5621 .65495 .5493 Metetra
.771 .726 .811 r
.65495 .5493 .66281 .5621 .67846 .54943 .67066 .53667 Metetra
.759 .723 .819 r
.67066 .53667 .67846 .54943 .69417 .53771 .68645 .52502 Metetra
.748 .722 .827 r
.68645 .52502 .69417 .53771 .70996 .52683 .70231 .51424 Metetra
.738 .721 .835 r
.70231 .51424 .70996 .52683 .72583 .5167 .71826 .50423 Metetra
.728 .72 .843 r
.71826 .50423 .72583 .5167 .7418 .50725 .7343 .49491 Metetra
.719 .72 .85 r
.7343 .49491 .7418 .50725 .75787 .49839 .75045 .48619 Metetra
.711 .72 .857 r
.75045 .48619 .75787 .49839 .77404 .49005 .7667 .47799 Metetra
.703 .721 .864 r
.7667 .47799 .77404 .49005 .79032 .48217 .78306 .47025 Metetra
.696 .722 .87 r
.78306 .47025 .79032 .48217 .80671 .47467 .79954 .46289 Metetra
.69 .724 .876 r
.79954 .46289 .80671 .47467 .82322 .46751 .81612 .45585 Metetra
.684 .725 .881 r
.81612 .45585 .82322 .46751 .83984 .46063 .83283 .44909 Metetra
.285 .474 .869 r
.23237 .66846 .24122 .68154 .25408 .68679 .24521 .67373 Metetra
.232 .445 .862 r
.24521 .67373 .25408 .68679 .26702 .69275 .25813 .67987 Metetra
.18 .419 .855 r
.25813 .67987 .26702 .69275 .28009 .69931 .27116 .68676 Metetra
.13 .397 .848 r
.27116 .68676 .28009 .69931 .2933 .70628 .28432 .69421 Metetra
.084 .38 .842 r
.28432 .69421 .2933 .70628 .30666 .71349 .29763 .70203 Metetra
.045 .371 .839 r
.29763 .70203 .30666 .71349 .3202 .72074 .31112 .71002 Metetra
.015 .37 .838 r
.31112 .71002 .3202 .72074 .33394 .7278 .32481 .71793 Metetra
0 .379 .841 r
.32481 .71793 .33394 .7278 .34788 .73448 .3387 .72552 Metetra
0 .4 .848 r
.3387 .72552 .34788 .73448 .36203 .74056 .35282 .73257 Metetra
0 .432 .86 r
.35282 .73257 .36203 .74056 .3764 .74585 .36716 .73884 Metetra
.021 .477 .876 r
.36716 .73884 .3764 .74585 .39098 .75016 .38173 .74412 Metetra
.063 .532 .895 r
.38173 .74412 .39098 .75016 .40577 .75334 .39653 .74822 Metetra
.122 .596 .917 r
.39653 .74822 .40577 .75334 .42076 .75527 .41154 .751 Metetra
.194 .664 .938 r
.41154 .751 .42076 .75527 .43594 .75585 .42676 .75232 Metetra
.943 .611 .371 r
.42676 .75232 .43594 .75585 .4491 .85048 .43979 .84423 Metetra
.928 .799 .697 r
.43979 .84423 .4491 .85048 .46565 .81595 .45652 .80848 Metetra
.918 .791 .705 r
.45652 .80848 .46565 .81595 .48197 .78352 .47301 .77494 Metetra
.907 .783 .713 r
.47301 .77494 .48197 .78352 .49811 .75318 .48929 .74362 Metetra
.895 .775 .721 r
.48929 .74362 .49811 .75318 .51409 .72489 .50539 .71448 Metetra
.883 .767 .729 r
.50539 .71448 .51409 .72489 .52994 .6986 .52135 .68745 Metetra
.87 .76 .738 r
.52135 .68745 .52994 .6986 .54568 .67423 .5372 .66248 Metetra
.857 .753 .746 r
.5372 .66248 .54568 .67423 .56134 .6517 .55295 .63946 Metetra
.844 .746 .755 r
.55295 .63946 .56134 .6517 .57695 .6309 .56865 .61829 Metetra
.831 .74 .764 r
.56865 .61829 .57695 .6309 .59253 .61175 .58431 .59886 Metetra
.818 .735 .773 r
.58431 .59886 .59253 .61175 .60811 .59413 .59995 .58105 Metetra
.805 .73 .782 r
.59995 .58105 .60811 .59413 .62369 .57792 .6156 .56473 Metetra
.792 .726 .791 r
.6156 .56473 .62369 .57792 .6393 .56301 .63128 .54979 Metetra
.779 .723 .8 r
.63128 .54979 .6393 .56301 .65495 .5493 .64701 .5361 Metetra
.767 .72 .809 r
.64701 .5361 .65495 .5493 .67066 .53667 .66279 .52354 Metetra
.755 .718 .818 r
.66279 .52354 .67066 .53667 .68645 .52502 .67864 .51199 Metetra
.744 .717 .826 r
.67864 .51199 .68645 .52502 .70231 .51424 .69458 .50134 Metetra
.733 .716 .835 r
.69458 .50134 .70231 .51424 .71826 .50423 .7106 .49149 Metetra
.724 .716 .843 r
.7106 .49149 .71826 .50423 .7343 .49491 .72672 .48233 Metetra
.714 .716 .85 r
.72672 .48233 .7343 .49491 .75045 .48619 .74295 .47378 Metetra
.706 .717 .858 r
.74295 .47378 .75045 .48619 .7667 .47799 .75928 .46575 Metetra
.698 .718 .865 r
.75928 .46575 .7667 .47799 .78306 .47025 .77572 .45817 Metetra
.692 .72 .871 r
.77572 .45817 .78306 .47025 .79954 .46289 .79228 .45097 Metetra
.685 .721 .877 r
.79228 .45097 .79954 .46289 .81612 .45585 .80896 .44408 Metetra
.68 .723 .882 r
.80896 .44408 .81612 .45585 .83283 .44909 .82575 .43745 Metetra
.297 .472 .863 r
.2235 .65483 .23237 .66846 .24521 .67373 .23633 .66005 Metetra
.239 .438 .855 r
.23633 .66005 .24521 .67373 .25813 .67987 .24922 .66631 Metetra
.18 .406 .845 r
.24922 .66631 .25813 .67987 .27116 .68676 .26221 .67348 Metetra
.122 .378 .835 r
.26221 .67348 .27116 .68676 .28432 .69421 .27532 .68139 Metetra
.067 .354 .826 r
.27532 .68139 .28432 .69421 .29763 .70203 .28857 .68984 Metetra
.017 .337 .818 r
.28857 .68984 .29763 .70203 .31112 .71002 .302 .69861 Metetra
0 .328 .813 r
.302 .69861 .31112 .71002 .32481 .71793 .31562 .70742 Metetra
0 .33 .811 r
.31562 .70742 .32481 .71793 .3387 .72552 .32946 .71603 Metetra
0 .344 .813 r
.32946 .71603 .3387 .72552 .35282 .73257 .34352 .72416 Metetra
0 .371 .82 r
.34352 .72416 .35282 .73257 .36716 .73884 .35783 .73153 Metetra
0 .412 .833 r
.35783 .73153 .36716 .73884 .38173 .74412 .37237 .73792 Metetra
0 .466 .851 r
.37237 .73792 .38173 .74412 .39653 .74822 .38716 .74309 Metetra
.028 .532 .873 r
.38716 .74309 .39653 .74822 .41154 .751 .40219 .74685 Metetra
.102 .607 .897 r
.40219 .74685 .41154 .751 .42676 .75232 .41744 .74906 Metetra
.941 .608 .369 r
.41744 .74906 .42676 .75232 .43979 .84423 .43036 .83801 Metetra
.931 .798 .692 r
.43036 .83801 .43979 .84423 .45652 .80848 .44728 .8009 Metetra
.92 .789 .7 r
.44728 .8009 .45652 .80848 .47301 .77494 .46394 .76614 Metetra
.908 .78 .708 r
.46394 .76614 .47301 .77494 .48929 .74362 .48037 .73373 Metetra
.896 .771 .716 r
.48037 .73373 .48929 .74362 .50539 .71448 .49661 .70364 Metetra
.883 .763 .724 r
.49661 .70364 .50539 .71448 .52135 .68745 .51269 .67582 Metetra
.87 .755 .733 r
.51269 .67582 .52135 .68745 .5372 .66248 .52863 .6502 Metetra
.856 .747 .742 r
.52863 .6502 .5372 .66248 .55295 .63946 .54448 .62666 Metetra
.843 .74 .751 r
.54448 .62666 .55295 .63946 .56865 .61829 .56026 .60511 Metetra
.829 .734 .76 r
.56026 .60511 .56865 .61829 .58431 .59886 .576 .58541 Metetra
.815 .728 .769 r
.576 .58541 .58431 .59886 .59995 .58105 .59172 .56743 Metetra
.801 .723 .778 r
.59172 .56743 .59995 .58105 .6156 .56473 .60744 .55103 Metetra
.788 .719 .788 r
.60744 .55103 .6156 .56473 .63128 .54979 .62319 .53609 Metetra
.775 .716 .797 r
.62319 .53609 .63128 .54979 .64701 .5361 .63898 .52246 Metetra
.762 .714 .807 r
.63898 .52246 .64701 .5361 .66279 .52354 .65483 .51 Metetra
.75 .712 .816 r
.65483 .51 .66279 .52354 .67864 .51199 .67076 .4986 Metetra
.739 .711 .825 r
.67076 .4986 .67864 .51199 .69458 .50134 .68676 .48813 Metetra
.728 .711 .834 r
.68676 .48813 .69458 .50134 .7106 .49149 .70286 .47847 Metetra
.718 .711 .843 r
.70286 .47847 .7106 .49149 .72672 .48233 .71906 .46951 Metetra
.709 .712 .851 r
.71906 .46951 .72672 .48233 .74295 .47378 .73537 .46117 Metetra
.701 .714 .858 r
.73537 .46117 .74295 .47378 .75928 .46575 .75178 .45334 Metetra
.693 .715 .866 r
.75178 .45334 .75928 .46575 .77572 .45817 .76831 .44596 Metetra
.687 .717 .872 r
.76831 .44596 .77572 .45817 .79228 .45097 .78495 .43894 Metetra
.681 .72 .879 r
.78495 .43894 .79228 .45097 .80896 .44408 .80172 .43221 Metetra
.676 .722 .884 r
.80172 .43221 .80896 .44408 .82575 .43745 .8186 .42574 Metetra
.314 .472 .857 r
.21461 .64064 .2235 .65483 .23633 .66005 .22743 .6457 Metetra
.251 .434 .847 r
.22743 .6457 .23633 .66005 .24922 .66631 .24031 .65199 Metetra
.186 .396 .836 r
.24031 .65199 .24922 .66631 .26221 .67348 .25326 .65938 Metetra
.119 .361 .823 r
.25326 .65938 .26221 .67348 .27532 .68139 .26631 .66773 Metetra
.054 .329 .81 r
.26631 .66773 .27532 .68139 .28857 .68984 .2795 .67681 Metetra
0 .304 .798 r
.2795 .67681 .28857 .68984 .302 .69861 .29285 .68638 Metetra
0 .286 .786 r
.29285 .68638 .302 .69861 .31562 .70742 .30639 .69618 Metetra
0 .278 .778 r
.30639 .69618 .31562 .70742 .32946 .71603 .32015 .70589 Metetra
0 .283 .773 r
.32015 .70589 .32946 .71603 .34352 .72416 .33415 .71522 Metetra
0 .302 .773 r
.33415 .71522 .34352 .72416 .35783 .73153 .3484 .72387 Metetra
0 .336 .78 r
.3484 .72387 .35783 .73153 .37237 .73792 .3629 .73152 Metetra
0 .387 .793 r
.3629 .73152 .37237 .73792 .38716 .74309 .37767 .73793 Metetra
0 .453 .813 r
.37767 .73793 .38716 .74309 .40219 .74685 .3927 .74285 Metetra
0 .532 .837 r
.3927 .74285 .40219 .74685 .41744 .74906 .40797 .74609 Metetra
.939 .603 .365 r
.40797 .74609 .41744 .74906 .43036 .83801 .4208 .83184 Metetra
.933 .797 .686 r
.4208 .83184 .43036 .83801 .44728 .8009 .43793 .79319 Metetra
.922 .787 .694 r
.43793 .79319 .44728 .8009 .46394 .76614 .45477 .75705 Metetra
.91 .777 .702 r
.45477 .75705 .46394 .76614 .48037 .73373 .47136 .72343 Metetra
.897 .767 .71 r
.47136 .72343 .48037 .73373 .49661 .70364 .48774 .69231 Metetra
.883 .758 .719 r
.48774 .69231 .49661 .70364 .51269 .67582 .50394 .66363 Metetra
.869 .749 .728 r
.50394 .66363 .51269 .67582 .52863 .6502 .51999 .6373 Metetra
.855 .741 .737 r
.51999 .6373 .52863 .6502 .54448 .62666 .53594 .61323 Metetra
.84 .733 .746 r
.53594 .61323 .54448 .62666 .56026 .60511 .5518 .59128 Metetra
.826 .727 .755 r
.5518 .59128 .56026 .60511 .576 .58541 .56761 .57132 Metetra
.812 .721 .765 r
.56761 .57132 .576 .58541 .59172 .56743 .5834 .5532 Metetra
.797 .716 .775 r
.5834 .5532 .59172 .56743 .60744 .55103 .5992 .53677 Metetra
.784 .712 .785 r
.5992 .53677 .60744 .55103 .62319 .53609 .61502 .52186 Metetra
.77 .709 .795 r
.61502 .52186 .62319 .53609 .63898 .52246 .63088 .50834 Metetra
.757 .707 .805 r
.63088 .50834 .63898 .52246 .65483 .51 .6468 .49604 Metetra
.745 .706 .814 r
.6468 .49604 .65483 .51 .67076 .4986 .66279 .48484 Metetra
.733 .705 .824 r
.66279 .48484 .67076 .4986 .68676 .48813 .67887 .47459 Metetra
.723 .705 .834 r
.67887 .47459 .68676 .48813 .70286 .47847 .69505 .46517 Metetra
.712 .706 .843 r
.69505 .46517 .70286 .47847 .71906 .46951 .71133 .45647 Metetra
.703 .708 .851 r
.71133 .45647 .71906 .46951 .73537 .46117 .72771 .44837 Metetra
.695 .71 .86 r
.72771 .44837 .73537 .46117 .75178 .45334 .74421 .44078 Metetra
.688 .712 .867 r
.74421 .44078 .75178 .45334 .76831 .44596 .76082 .43362 Metetra
.681 .715 .874 r
.76082 .43362 .76831 .44596 .78495 .43894 .77756 .4268 Metetra
.676 .718 .881 r
.77756 .4268 .78495 .43894 .80172 .43221 .79441 .42027 Metetra
.671 .721 .887 r
.79441 .42027 .80172 .43221 .8186 .42574 .81138 .41396 Metetra
.337 .476 .852 r
.20571 .62589 .21461 .64064 .22743 .6457 .21854 .63066 Metetra
.269 .433 .84 r
.21854 .63066 .22743 .6457 .24031 .65199 .23141 .63685 Metetra
.198 .39 .827 r
.23141 .63685 .24031 .65199 .25326 .65938 .24432 .64438 Metetra
.123 .347 .811 r
.24432 .64438 .25326 .65938 .26631 .66773 .25731 .65309 Metetra
.048 .307 .795 r
.25731 .65309 .26631 .66773 .2795 .67681 .27042 .66278 Metetra
0 .272 .777 r
.27042 .66278 .2795 .67681 .29285 .68638 .28369 .67319 Metetra
0 .244 .759 r
.28369 .67319 .29285 .68638 .30639 .69618 .29713 .68403 Metetra
0 .226 .743 r
.29713 .68403 .30639 .69618 .32015 .70589 .3108 .69497 Metetra
0 .219 .729 r
.3108 .69497 .32015 .70589 .33415 .71522 .32471 .70565 Metetra
0 .227 .72 r
.32471 .70565 .33415 .71522 .3484 .72387 .33888 .71573 Metetra
0 .253 .717 r
.33888 .71573 .3484 .72387 .3629 .73152 .35333 .72485 Metetra
0 .296 .721 r
.35333 .72485 .3629 .73152 .37767 .73793 .36805 .7327 Metetra
0 .358 .734 r
.36805 .7327 .37767 .73793 .3927 .74285 .38306 .73898 Metetra
0 .438 .756 r
.38306 .73898 .3927 .74285 .40797 .74609 .39834 .74346 Metetra
.937 .598 .361 r
.39834 .74346 .40797 .74609 .4208 .83184 .41112 .82572 Metetra
.936 .795 .68 r
.41112 .82572 .4208 .83184 .43793 .79319 .42847 .78533 Metetra
.924 .784 .688 r
.42847 .78533 .43793 .79319 .45477 .75705 .44551 .74764 Metetra
.911 .773 .696 r
.44551 .74764 .45477 .75705 .47136 .72343 .46227 .71266 Metetra
.897 .762 .705 r
.46227 .71266 .47136 .72343 .48774 .69231 .47879 .68038 Metetra
.883 .752 .713 r
.47879 .68038 .48774 .69231 .50394 .66363 .49512 .65075 Metetra
.868 .742 .722 r
.49512 .65075 .50394 .66363 .51999 .6373 .51128 .62369 Metetra
.853 .734 .731 r
.51128 .62369 .51999 .6373 .53594 .61323 .52732 .59905 Metetra
.837 .726 .741 r
.52732 .59905 .53594 .61323 .5518 .59128 .54326 .57672 Metetra
.822 .719 .751 r
.54326 .57672 .5518 .59128 .56761 .57132 .55915 .55652 Metetra
.807 .713 .761 r
.55915 .55652 .56761 .57132 .5834 .5532 .57501 .53829 Metetra
.792 .708 .771 r
.57501 .53829 .5834 .5532 .5992 .53677 .59088 .52186 Metetra
.778 .704 .781 r
.59088 .52186 .5992 .53677 .61502 .52186 .60676 .50705 Metetra
.764 .701 .792 r
.60676 .50705 .61502 .52186 .63088 .50834 .62269 .4937 Metetra
.751 .699 .803 r
.62269 .4937 .63088 .50834 .6468 .49604 .63868 .48163 Metetra
.738 .698 .813 r
.63868 .48163 .6468 .49604 .66279 .48484 .65475 .47069 Metetra
.727 .699 .823 r
.65475 .47069 .66279 .48484 .67887 .47459 .6709 .46073 Metetra
.716 .7 .834 r
.6709 .46073 .67887 .47459 .69505 .46517 .68715 .45161 Metetra
.706 .701 .843 r
.68715 .45161 .69505 .46517 .71133 .45647 .70351 .44321 Metetra
.697 .704 .853 r
.70351 .44321 .71133 .45647 .72771 .44837 .71998 .4354 Metetra
.688 .706 .861 r
.71998 .4354 .72771 .44837 .74421 .44078 .73656 .42809 Metetra
.681 .71 .869 r
.73656 .42809 .74421 .44078 .76082 .43362 .75326 .42118 Metetra
.675 .713 .877 r
.75326 .42118 .76082 .43362 .77756 .4268 .77008 .4146 Metetra
.67 .717 .884 r
.77008 .4146 .77756 .4268 .79441 .42027 .78703 .40827 Metetra
.666 .72 .889 r
.78703 .40827 .79441 .42027 .81138 .41396 .80409 .40214 Metetra
.365 .484 .847 r
.1968 .61058 .20571 .62589 .21854 .63066 .20966 .61489 Metetra
.294 .436 .834 r
.20966 .61489 .21854 .63066 .23141 .63685 .22252 .62083 Metetra
.217 .387 .819 r
.22252 .62083 .23141 .63685 .24432 .64438 .2354 .62837 Metetra
.135 .338 .801 r
.2354 .62837 .24432 .64438 .25731 .65309 .24834 .63735 Metetra
.05 .289 .78 r
.24834 .63735 .25731 .65309 .27042 .66278 .26137 .6476 Metetra
0 .244 .757 r
.26137 .6476 .27042 .66278 .28369 .67319 .27453 .65886 Metetra
0 .204 .732 r
.27453 .65886 .28369 .67319 .29713 .68403 .28787 .6708 Metetra
0 .173 .707 r
.28787 .6708 .29713 .68403 .3108 .69497 .30142 .68306 Metetra
0 .154 .682 r
.30142 .68306 .3108 .69497 .32471 .70565 .31522 .69526 Metetra
0 .149 .661 r
.31522 .69526 .32471 .70565 .33888 .71573 .32929 .70698 Metetra
0 .162 .645 r
.32929 .70698 .33888 .71573 .35333 .72485 .34365 .71781 Metetra
0 .195 .637 r
.34365 .71781 .35333 .72485 .36805 .7327 .35831 .72736 Metetra
0 .248 .638 r
.35831 .72736 .36805 .7327 .38306 .73898 .37328 .73526 Metetra
.263 0 0 r
.37328 .73526 .38306 .73898 .39834 .74346 .38855 .74123 Metetra
.934 .592 .357 r
.38855 .74123 .39834 .74346 .41112 .82572 .4013 .81966 Metetra
.939 .793 .673 r
.4013 .81966 .41112 .82572 .42847 .78533 .4189 .77728 Metetra
.926 .781 .681 r
.4189 .77728 .42847 .78533 .44551 .74764 .43615 .7378 Metetra
.912 .768 .69 r
.43615 .7378 .44551 .74764 .46227 .71266 .45309 .70129 Metetra
.897 .757 .698 r
.45309 .70129 .46227 .71266 .47879 .68038 .46977 .66774 Metetra
.881 .745 .707 r
.46977 .66774 .47879 .68038 .49512 .65075 .48622 .63707 Metetra
.866 .735 .716 r
.48622 .63707 .49512 .65075 .51128 .62369 .50249 .60921 Metetra
.85 .725 .726 r
.50249 .60921 .51128 .62369 .52732 .59905 .51862 .584 Metetra
.834 .717 .735 r
.51862 .584 .52732 .59905 .54326 .57672 .53465 .56128 Metetra
.818 .709 .746 r
.53465 .56128 .54326 .57672 .55915 .55652 .55062 .54088 Metetra
.802 .703 .756 r
.55062 .54088 .55915 .55652 .57501 .53829 .56655 .5226 Metetra
.786 .698 .767 r
.56655 .5226 .57501 .53829 .59088 .52186 .58248 .50624 Metetra
.772 .694 .778 r
.58248 .50624 .59088 .52186 .60676 .50705 .59843 .49161 Metetra
.757 .692 .789 r
.59843 .49161 .60676 .50705 .62269 .4937 .61442 .47851 Metetra
.744 .691 .801 r
.61442 .47851 .62269 .4937 .63868 .48163 .63048 .46675 Metetra
.731 .691 .812 r
.63048 .46675 .63868 .48163 .65475 .47069 .64662 .45615 Metetra
.719 .692 .823 r
.64662 .45615 .65475 .47069 .6709 .46073 .66285 .44655 Metetra
.708 .693 .834 r
.66285 .44655 .6709 .46073 .68715 .45161 .67918 .4378 Metetra
.698 .696 .845 r
.67918 .4378 .68715 .45161 .70351 .44321 .69562 .42975 Metetra
.689 .699 .855 r
.69562 .42975 .70351 .44321 .71998 .4354 .71217 .42229 Metetra
.681 .703 .864 r
.71217 .42229 .71998 .4354 .73656 .42809 .72884 .4153 Metetra
.674 .707 .872 r
.72884 .4153 .73656 .42809 .75326 .42118 .74563 .40869 Metetra
.669 .711 .88 r
.74563 .40869 .75326 .42118 .77008 .4146 .76254 .40237 Metetra
.664 .716 .887 r
.76254 .40237 .77008 .4146 .78703 .40827 .77958 .39626 Metetra
.66 .72 .893 r
.77958 .39626 .78703 .40827 .80409 .40214 .79673 .39032 Metetra
.399 .495 .841 r
.18788 .59475 .1968 .61058 .20966 .61489 .2008 .59838 Metetra
.326 .444 .828 r
.2008 .59838 .20966 .61489 .22252 .62083 .21367 .60387 Metetra
.245 .39 .811 r
.21367 .60387 .22252 .62083 .2354 .62837 .22653 .61123 Metetra
.157 .334 .791 r
.22653 .61123 .2354 .62837 .24834 .63735 .23942 .62036 Metetra
.062 .276 .767 r
.23942 .62036 .24834 .63735 .26137 .6476 .25237 .63108 Metetra
0 .221 .739 r
.25237 .63108 .26137 .6476 .27453 .65886 .26542 .64315 Metetra
0 .169 .707 r
.26542 .64315 .27453 .65886 .28787 .6708 .27863 .65624 Metetra
0 .124 .672 r
.27863 .65624 .28787 .6708 .30142 .68306 .29204 .66995 Metetra
0 .089 .635 r
.29204 .66995 .30142 .68306 .31522 .69526 .3057 .68385 Metetra
0 .07 .6 r
.3057 .68385 .31522 .69526 .32929 .70698 .31963 .69744 Metetra
0 .068 .568 r
.31963 .69744 .32929 .70698 .34365 .71781 .33387 .71026 Metetra
.45 0 0 r
.33387 .71026 .34365 .71781 .35831 .72736 .34844 .72182 Metetra
.44 0 0 r
.34844 .72182 .35831 .72736 .37328 .73526 .36334 .73167 Metetra
.397 0 0 r
.36334 .73167 .37328 .73526 .38855 .74123 .37858 .73943 Metetra
.931 .584 .351 r
.37858 .73943 .38855 .74123 .4013 .81966 .39134 .81366 Metetra
.942 .791 .665 r
.39134 .81366 .4013 .81966 .4189 .77728 .40921 .76897 Metetra
.927 .777 .674 r
.40921 .76897 .4189 .77728 .43615 .7378 .4267 .72745 Metetra
.912 .763 .683 r
.4267 .72745 .43615 .7378 .45309 .70129 .44384 .6892 Metetra
.896 .75 .691 r
.44384 .6892 .45309 .70129 .46977 .66774 .46068 .6542 Metetra
.88 .737 .701 r
.46068 .6542 .46977 .66774 .48622 .63707 .47726 .6224 Metetra
.863 .726 .71 r
.47726 .6224 .48622 .63707 .50249 .60921 .49365 .59369 Metetra
.846 .715 .719 r
.49365 .59369 .50249 .60921 .51862 .584 .50987 .56789 Metetra
.828 .706 .73 r
.50987 .56789 .51862 .584 .53465 .56128 .52598 .54482 Metetra
.812 .698 .74 r
.52598 .54482 .53465 .56128 .55062 .54088 .54202 .52428 Metetra
.795 .692 .751 r
.54202 .52428 .55062 .54088 .56655 .5226 .55801 .50602 Metetra
.779 .687 .763 r
.55801 .50602 .56655 .5226 .58248 .50624 .574 .48983 Metetra
.763 .684 .774 r
.574 .48983 .58248 .50624 .59843 .49161 .59002 .47547 Metetra
.749 .682 .786 r
.59002 .47547 .59843 .49161 .61442 .47851 .60608 .46272 Metetra
.735 .681 .799 r
.60608 .46272 .61442 .47851 .63048 .46675 .62221 .45137 Metetra
.722 .682 .811 r
.62221 .45137 .63048 .46675 .64662 .45615 .63841 .44122 Metetra
.71 .684 .823 r
.63841 .44122 .64662 .45615 .66285 .44655 .65472 .43207 Metetra
.699 .687 .835 r
.65472 .43207 .66285 .44655 .67918 .4378 .67113 .42377 Metetra
.689 .691 .847 r
.67113 .42377 .67918 .4378 .69562 .42975 .68765 .41615 Metetra
.68 .695 .857 r
.68765 .41615 .69562 .42975 .71217 .42229 .70429 .40909 Metetra
.673 .7 .867 r
.70429 .40909 .71217 .42229 .72884 .4153 .72105 .40247 Metetra
.666 .705 .876 r
.72105 .40247 .72884 .4153 .74563 .40869 .73793 .39618 Metetra
.661 .71 .884 r
.73793 .39618 .74563 .40869 .76254 .40237 .75493 .39014 Metetra
.657 .716 .891 r
.75493 .39014 .76254 .40237 .77958 .39626 .77206 .38428 Metetra
.654 .72 .897 r
.77206 .38428 .77958 .39626 .79673 .39032 .78931 .37854 Metetra
.439 .511 .836 r
.17895 .57849 .18788 .59475 .2008 .59838 .19195 .58116 Metetra
.365 .457 .822 r
.19195 .58116 .2008 .59838 .21367 .60387 .20487 .58594 Metetra
.282 .399 .804 r
.20487 .58594 .21367 .60387 .22653 .61123 .21773 .59287 Metetra
.188 .336 .782 r
.21773 .59287 .22653 .61123 .23942 .62036 .23058 .60193 Metetra
.086 .271 .755 r
.23058 .60193 .23942 .62036 .25237 .63108 .24344 .61298 Metetra
0 .204 .723 r
.24344 .61298 .25237 .63108 .26542 .64315 .25638 .6258 Metetra
0 .139 .684 r
.25638 .6258 .26542 .64315 .27863 .65624 .26945 .64005 Metetra
0 .079 .639 r
.26945 .64005 .27863 .65624 .29204 .66995 .28269 .65531 Metetra
0 .029 .59 r
.28269 .65531 .29204 .66995 .3057 .68385 .29617 .6711 Metetra
0 0 .539 r
.29617 .6711 .3057 .68385 .31963 .69744 .30993 .68686 Metetra
0 0 .489 r
.30993 .68686 .31963 .69744 .33387 .71026 .32402 .702 Metetra
.548 .021 0 r
.32402 .702 .33387 .71026 .34844 .72182 .33845 .71596 Metetra
.553 0 0 r
.33845 .71596 .34844 .72182 .36334 .73167 .35325 .72817 Metetra
.522 0 0 r
.35325 .72817 .36334 .73167 .37858 .73943 .36843 .73815 Metetra
.927 .575 .344 r
.36843 .73815 .37858 .73943 .39134 .81366 .38124 .80772 Metetra
.944 .788 .657 r
.38124 .80772 .39134 .81366 .40921 .76897 .39942 .76031 Metetra
.929 .772 .666 r
.39942 .76031 .40921 .76897 .4267 .72745 .41716 .71642 Metetra
.912 .757 .675 r
.41716 .71642 .4267 .72745 .44384 .6892 .43451 .67617 Metetra
.895 .742 .684 r
.43451 .67617 .44384 .6892 .46068 .6542 .45153 .63956 Metetra
.877 .728 .693 r
.45153 .63956 .46068 .6542 .47726 .6224 .46825 .60651 Metetra
.858 .715 .703 r
.46825 .60651 .47726 .6224 .49365 .59369 .48475 .5769 Metetra
.84 .704 .713 r
.48475 .5769 .49365 .59369 .50987 .56789 .50106 .55052 Metetra
.822 .694 .723 r
.50106 .55052 .50987 .56789 .52598 .54482 .51725 .52715 Metetra
.804 .686 .734 r
.51725 .52715 .52598 .54482 .54202 .52428 .53335 .50654 Metetra
.787 .679 .746 r
.53335 .50654 .54202 .52428 .55801 .50602 .54941 .48843 Metetra
.77 .675 .758 r
.54941 .48843 .55801 .50602 .574 .48983 .56546 .47253 Metetra
.754 .672 .771 r
.56546 .47253 .574 .48983 .59002 .47547 .58154 .45858 Metetra
.738 .67 .784 r
.58154 .45858 .59002 .47547 .60608 .46272 .59766 .44632 Metetra
.724 .671 .797 r
.59766 .44632 .60608 .46272 .62221 .45137 .61385 .43551 Metetra
.711 .673 .811 r
.61385 .43551 .62221 .45137 .63841 .44122 .63013 .42591 Metetra
.699 .676 .824 r
.63013 .42591 .63841 .44122 .65472 .43207 .64651 .41733 Metetra
.688 .68 .837 r
.64651 .41733 .65472 .43207 .67113 .42377 .663 .40956 Metetra
.678 .686 .85 r
.663 .40956 .67113 .42377 .68765 .41615 .67961 .40246 Metetra
.67 .692 .861 r
.67961 .40246 .68765 .41615 .70429 .40909 .69634 .39586 Metetra
.663 .698 .872 r
.69634 .39586 .70429 .40909 .72105 .40247 .71319 .38965 Metetra
.658 .704 .881 r
.71319 .38965 .72105 .40247 .73793 .39618 .73016 .38372 Metetra
.653 .711 .89 r
.73016 .38372 .73793 .39618 .75493 .39014 .74726 .37799 Metetra
.65 .717 .897 r
.74726 .37799 .75493 .39014 .77206 .38428 .76448 .37239 Metetra
.648 .722 .903 r
.76448 .37239 .77206 .38428 .78931 .37854 .78183 .36685 Metetra
.484 .533 .831 r
.17 .56194 .17895 .57849 .19195 .58116 .18311 .56332 Metetra
.412 .476 .817 r
.18311 .56332 .19195 .58116 .20487 .58594 .19612 .56702 Metetra
.328 .414 .798 r
.19612 .56702 .20487 .58594 .21773 .59287 .20901 .57319 Metetra
.231 .346 .775 r
.20901 .57319 .21773 .59287 .23058 .60193 .22184 .58188 Metetra
.123 .273 .746 r
.22184 .58188 .23058 .60193 .24344 .61298 .23464 .59303 Metetra
.003 .196 .71 r
.23464 .59303 .24344 .61298 .25638 .6258 .24746 .60645 Metetra
0 .119 .666 r
.24746 .60645 .25638 .6258 .26945 .64005 .26037 .62183 Metetra
0 .043 .612 r
.26037 .62183 .26945 .64005 .28269 .65531 .27342 .63873 Metetra
0 0 .55 r
.27342 .63873 .28269 .65531 .29617 .6711 .28669 .65662 Metetra
0 0 .482 r
.28669 .65662 .29617 .6711 .30993 .68686 .30023 .67485 Metetra
.58 .114 0 r
.30023 .67485 .30993 .68686 .32402 .702 .3141 .69275 Metetra
.634 .125 0 r
.3141 .69275 .32402 .702 .33845 .71596 .32835 .7096 Metetra
.651 .11 0 r
.32835 .7096 .33845 .71596 .35325 .72817 .34301 .72471 Metetra
.631 .068 0 r
.34301 .72471 .35325 .72817 .36843 .73815 .35808 .73743 Metetra
.921 .563 .336 r
.35808 .73743 .36843 .73815 .38124 .80772 .371 .80182 Metetra
.947 .785 .648 r
.371 .80182 .38124 .80772 .39942 .76031 .38953 .75117 Metetra
.93 .766 .658 r
.38953 .75117 .39942 .76031 .41716 .71642 .40755 .7045 Metetra
.912 .749 .667 r
.40755 .7045 .41716 .71642 .43451 .67617 .42514 .66193 Metetra
.892 .732 .676 r
.42514 .66193 .43451 .67617 .45153 .63956 .44234 .62349 Metetra
.873 .716 .685 r
.44234 .62349 .45153 .63956 .46825 .60651 .45921 .58908 Metetra
.853 .702 .695 r
.45921 .58908 .46825 .60651 .48475 .5769 .47581 .55853 Metetra
.833 .69 .705 r
.47581 .55853 .48475 .5769 .50106 .55052 .49222 .53161 Metetra
.813 .68 .716 r
.49222 .53161 .50106 .55052 .51725 .52715 .50847 .50803 Metetra
.794 .671 .728 r
.50847 .50803 .51725 .52715 .53335 .50654 .52464 .48749 Metetra
.776 .664 .74 r
.52464 .48749 .53335 .50654 .54941 .48843 .54075 .46967 Metetra
.758 .66 .753 r
.54075 .46967 .54941 .48843 .56546 .47253 .55685 .45423 Metetra
.741 .658 .767 r
.55685 .45423 .56546 .47253 .58154 .45858 .57298 .44087 Metetra
.726 .657 .782 r
.57298 .44087 .58154 .45858 .59766 .44632 .58917 .42928 Metetra
.711 .659 .797 r
.58917 .42928 .59766 .44632 .61385 .43551 .60543 .41917 Metetra
.698 .663 .812 r
.60543 .41917 .61385 .43551 .63013 .42591 .62178 .41029 Metetra
.686 .668 .827 r
.62178 .41029 .63013 .42591 .64651 .41733 .63824 .40239 Metetra
.675 .674 .841 r
.63824 .40239 .64651 .41733 .663 .40956 .65481 .39528 Metetra
.666 .681 .855 r
.65481 .39528 .663 .40956 .67961 .40246 .6715 .38876 Metetra
.659 .689 .867 r
.6715 .38876 .67961 .40246 .69634 .39586 .68832 .38269 Metetra
.653 .697 .878 r
.68832 .38269 .69634 .39586 .71319 .38965 .70526 .37694 Metetra
.648 .705 .888 r
.70526 .37694 .71319 .38965 .73016 .38372 .72233 .3714 Metetra
.645 .712 .896 r
.72233 .3714 .73016 .38372 .74726 .37799 .73952 .36599 Metetra
.643 .719 .903 r
.73952 .36599 .74726 .37799 .76448 .37239 .75684 .36064 Metetra
.642 .725 .909 r
.75684 .36064 .76448 .37239 .78183 .36685 .77428 .35529 Metetra
.534 .559 .827 r
.16098 .54535 .17 .56194 .18311 .56332 .17428 .54504 Metetra
.465 .501 .811 r
.17428 .54504 .18311 .56332 .19612 .56702 .18742 .54722 Metetra
.383 .436 .792 r
.18742 .54722 .19612 .56702 .20901 .57319 .2004 .55216 Metetra
.286 .364 .768 r
.2004 .55216 .20901 .57319 .22184 .58188 .21325 .56006 Metetra
.174 .285 .738 r
.21325 .56006 .22184 .58188 .23464 .59303 .22601 .57094 Metetra
.048 .2 .7 r
.22601 .57094 .23464 .59303 .24746 .60645 .23872 .5847 Metetra
0 .11 .652 r
.23872 .5847 .24746 .60645 .26037 .62183 .25145 .60108 Metetra
0 .019 .592 r
.25145 .60108 .26037 .62183 .27342 .63873 .26429 .61966 Metetra
0 0 .518 r
.26429 .61966 .27342 .63873 .28669 .65662 .2773 .63984 Metetra
0 0 .434 r
.2773 .63984 .28669 .65662 .30023 .67485 .29057 .66092 Metetra
.634 .195 0 r
.29057 .66092 .30023 .67485 .3141 .69275 .30417 .68208 Metetra
.705 .222 0 r
.30417 .68208 .3141 .69275 .32835 .7096 .31817 .70245 Metetra
.733 .218 0 r
.31817 .70245 .32835 .7096 .34301 .72471 .33261 .72115 Metetra
.719 .186 0 r
.33261 .72115 .34301 .72471 .35808 .73743 .34752 .73734 Metetra
.913 .548 .325 r
.34752 .73734 .35808 .73743 .371 .80182 .36061 .79591 Metetra
.949 .78 .638 r
.36061 .79591 .371 .80182 .38953 .75117 .37954 .74135 Metetra
.93 .759 .648 r
.37954 .74135 .38953 .75117 .40755 .7045 .39789 .69136 Metetra
.91 .739 .658 r
.39789 .69136 .40755 .7045 .42514 .66193 .41573 .64609 Metetra
.888 .72 .667 r
.41573 .64609 .42514 .66193 .44234 .62349 .43313 .60556 Metetra
.867 .703 .677 r
.43313 .60556 .44234 .62349 .45921 .58908 .45015 .56967 Metetra
.845 .687 .687 r
.45015 .56967 .45921 .58908 .47581 .55853 .46686 .53818 Metetra
.823 .674 .697 r
.46686 .53818 .47581 .55853 .49222 .53161 .48335 .51079 Metetra
.802 .662 .709 r
.48335 .51079 .49222 .53161 .50847 .50803 .49966 .48715 Metetra
.782 .653 .721 r
.49966 .48715 .50847 .50803 .52464 .48749 .51588 .46688 Metetra
.762 .647 .734 r
.51588 .46688 .52464 .48749 .54075 .46967 .53204 .44958 Metetra
.744 .643 .748 r
.53204 .44958 .54075 .46967 .55685 .45423 .54819 .43486 Metetra
.726 .642 .764 r
.54819 .43486 .55685 .45423 .57298 .44087 .56437 .42233 Metetra
.71 .643 .78 r
.56437 .42233 .57298 .44087 .58917 .42928 .58061 .41163 Metetra
.695 .646 .797 r
.58061 .41163 .58917 .42928 .60543 .41917 .59693 .40244 Metetra
.682 .652 .814 r
.59693 .40244 .60543 .41917 .62178 .41029 .61335 .39444 Metetra
.67 .659 .831 r
.61335 .39444 .62178 .41029 .63824 .40239 .62989 .38738 Metetra
.66 .668 .847 r
.62989 .38738 .63824 .40239 .65481 .39528 .64654 .38104 Metetra
.652 .678 .862 r
.64654 .38104 .65481 .39528 .6715 .38876 .66333 .3752 Metetra
.645 .687 .875 r
.66333 .3752 .6715 .38876 .68832 .38269 .68024 .36972 Metetra
.641 .697 .887 r
.68024 .36972 .68832 .38269 .70526 .37694 .69727 .36446 Metetra
.637 .707 .897 r
.69727 .36446 .70526 .37694 .72233 .3714 .71444 .35932 Metetra
.636 .716 .905 r
.71444 .35932 .72233 .3714 .73952 .36599 .73173 .35422 Metetra
.635 .723 .911 r
.73173 .35422 .73952 .36599 .75684 .36064 .74914 .3491 Metetra
.636 .73 .916 r
.74914 .3491 .75684 .36064 .77428 .35529 .76667 .34392 Metetra
.589 .592 .823 r
.15187 .5291 .16098 .54535 .17428 .54504 .16541 .52662 Metetra
.525 .533 .806 r
.16541 .52662 .17428 .54504 .18742 .54722 .17876 .52672 Metetra
.446 .466 .787 r
.17876 .52672 .18742 .54722 .2004 .55216 .1919 .52983 Metetra
.351 .391 .763 r
.1919 .52983 .2004 .55216 .21325 .56006 .20484 .53632 Metetra
.24 .308 .732 r
.20484 .53632 .21325 .56006 .22601 .57094 .2176 .54638 Metetra
.111 .216 .694 r
.2176 .54638 .22601 .57094 .23872 .5847 .23022 .56005 Metetra
0 .116 .644 r
.23022 .56005 .23872 .5847 .25145 .60108 .24279 .57716 Metetra
0 .01 .579 r
.24279 .57716 .25145 .60108 .26429 .61966 .25538 .59733 Metetra
0 0 .497 r
.25538 .59733 .26429 .61966 .2773 .63984 .2681 .61997 Metetra
0 0 .399 r
.2681 .61997 .2773 .63984 .29057 .66092 .28103 .64429 Metetra
.669 .265 0 r
.28103 .64429 .29057 .66092 .30417 .68208 .29428 .66934 Metetra
.759 .307 0 r
.29428 .66934 .30417 .68208 .31817 .70245 .30793 .69404 Metetra
.797 .315 0 r
.30793 .69404 .31817 .70245 .33261 .72115 .32207 .71727 Metetra
.789 .29 0 r
.32207 .71727 .33261 .72115 .34752 .73734 .33673 .73792 Metetra
.903 .527 .309 r
.33673 .73792 .34752 .73734 .36061 .79591 .35008 .7899 Metetra
.951 .775 .627 r
.35008 .7899 .36061 .79591 .37954 .74135 .36948 .73053 Metetra
.93 .75 .638 r
.36948 .73053 .37954 .74135 .39789 .69136 .38821 .67651 Metetra
.906 .727 .648 r
.38821 .67651 .39789 .69136 .41573 .64609 .40634 .62806 Metetra
.882 .706 .658 r
.40634 .62806 .41573 .64609 .43313 .60556 .42394 .58517 Metetra
.858 .686 .667 r
.42394 .58517 .43313 .60556 .45015 .56967 .44111 .54768 Metetra
.834 .669 .678 r
.44111 .54768 .45015 .56967 .46686 .53818 .45792 .51528 Metetra
.811 .654 .688 r
.45792 .51528 .46686 .53818 .48335 .51079 .47447 .48759 Metetra
.788 .642 .7 r
.47447 .48759 .48335 .51079 .49966 .48715 .49084 .46414 Metetra
.766 .633 .713 r
.49084 .46414 .49966 .48715 .51588 .46688 .50709 .44444 Metetra
.745 .626 .728 r
.50709 .44444 .51588 .46688 .53204 .44958 .52328 .428 Metetra
.725 .623 .744 r
.52328 .428 .53204 .44958 .54819 .43486 .53947 .41433 Metetra
.707 .623 .761 r
.53947 .41433 .54819 .43486 .56437 .42233 .55569 .40296 Metetra
.69 .626 .78 r
.55569 .40296 .56437 .42233 .58061 .41163 .57198 .39345 Metetra
.675 .632 .799 r
.57198 .39345 .58061 .41163 .59693 .40244 .58837 .38543 Metetra
.662 .641 .818 r
.58837 .38543 .59693 .40244 .61335 .39444 .60486 .37854 Metetra
.651 .652 .838 r
.60486 .37854 .61335 .39444 .62989 .38738 .62148 .37249 Metetra
.642 .664 .856 r
.62148 .37249 .62989 .38738 .64654 .38104 .63822 .36703 Metetra
.635 .676 .872 r
.63822 .36703 .64654 .38104 .66333 .3752 .65509 .36196 Metetra
.63 .689 .886 r
.65509 .36196 .66333 .3752 .68024 .36972 .67209 .35711 Metetra
.627 .701 .898 r
.67209 .35711 .68024 .36972 .69727 .36446 .68923 .35235 Metetra
.626 .712 .908 r
.68923 .35235 .69727 .36446 .71444 .35932 .70649 .3476 Metetra
.627 .722 .915 r
.70649 .3476 .71444 .35932 .73173 .35422 .72387 .34278 Metetra
.628 .73 .921 r
.72387 .34278 .73173 .35422 .74914 .3491 .74138 .33785 Metetra
.631 .737 .924 r
.74138 .33785 .74914 .3491 .76667 .34392 .75901 .33278 Metetra
.648 .632 .82 r
.14257 .5138 .15187 .5291 .16541 .52662 .15645 .50863 Metetra
.59 .573 .802 r
.15645 .50863 .16541 .52662 .17876 .52672 .1701 .50595 Metetra
.517 .505 .782 r
.1701 .50595 .17876 .52672 .1919 .52983 .1835 .50644 Metetra
.427 .428 .757 r
.1835 .50644 .1919 .52983 .20484 .53632 .19662 .51066 Metetra
.32 .342 .727 r
.19662 .51066 .20484 .53632 .2176 .54638 .20946 .51907 Metetra
.192 .246 .69 r
.20946 .51907 .2176 .54638 .23022 .56005 .22207 .53192 Metetra
.042 .139 .64 r
.22207 .53192 .23022 .56005 .24279 .57716 .2345 .54923 Metetra
0 .022 .575 r
.2345 .54923 .24279 .57716 .25538 .59733 .24684 .57073 Metetra
0 0 .489 r
.24684 .57073 .25538 .59733 .2681 .61997 .25921 .59588 Metetra
0 0 .379 r
.25921 .59588 .2681 .61997 .28103 .64429 .27172 .62385 Metetra
.681 .319 0 r
.27172 .62385 .28103 .64429 .29428 .66934 .28451 .65354 Metetra
.795 .38 0 r
.28451 .65354 .29428 .66934 .30793 .69404 .29771 .68362 Metetra
.847 .398 0 r
.29771 .68362 .30793 .69404 .32207 .71727 .31141 .71266 Metetra
.843 .379 .07 r
.31141 .71266 .32207 .71727 .33673 .73792 .32571 .73916 Metetra
.888 .498 .285 r
.32571 .73916 .33673 .73792 .35008 .7899 .33941 .78361 Metetra
.952 .767 .616 r
.33941 .78361 .35008 .7899 .36948 .73053 .35938 .71819 Metetra
.928 .739 .628 r
.35938 .71819 .36948 .73053 .38821 .67651 .37855 .65922 Metetra
.901 .712 .638 r
.37855 .65922 .38821 .67651 .40634 .62806 .397 .60696 Metetra
.874 .688 .648 r
.397 .60696 .40634 .62806 .42394 .58517 .41482 .56138 Metetra
.846 .665 .657 r
.41482 .56138 .42394 .58517 .44111 .54768 .43213 .52223 Metetra
.82 .646 .667 r
.43213 .52223 .44111 .54768 .45792 .51528 .44904 .48908 Metetra
.794 .63 .678 r
.44904 .48908 .45792 .51528 .47447 .48759 .46564 .46138 Metetra
.769 .617 .691 r
.46564 .46138 .47447 .48759 .49084 .46414 .48203 .43853 Metetra
.745 .607 .705 r
.48203 .43853 .49084 .46414 .50709 .44444 .49829 .41988 Metetra
.723 .602 .721 r
.49829 .41988 .50709 .44444 .52328 .428 .5145 .40479 Metetra
.702 .6 .739 r
.5145 .40479 .52328 .428 .53947 .41433 .53071 .39264 Metetra
.683 .602 .759 r
.53071 .39264 .53947 .41433 .55569 .40296 .54697 .38287 Metetra
.666 .608 .781 r
.54697 .38287 .55569 .40296 .57198 .39345 .5633 .37494 Metetra
.65 .618 .804 r
.5633 .37494 .57198 .39345 .58837 .38543 .57975 .3684 Metetra
.638 .631 .827 r
.57975 .3684 .58837 .38543 .60486 .37854 .59631 .36286 Metetra
.627 .646 .849 r
.59631 .36286 .60486 .37854 .62148 .37249 .61301 .358 Metetra
.62 .662 .869 r
.61301 .358 .62148 .37249 .63822 .36703 .62984 .35354 Metetra
.615 .679 .887 r
.62984 .35354 .63822 .36703 .65509 .36196 .6468 .34928 Metetra
.613 .694 .902 r
.6468 .34928 .65509 .36196 .67209 .35711 .6639 .34507 Metetra
.613 .709 .913 r
.6639 .34507 .67209 .35711 .68923 .35235 .68113 .34079 Metetra
.615 .721 .922 r
.68113 .34079 .68923 .35235 .70649 .3476 .69849 .33637 Metetra
.618 .732 .927 r
.69849 .33637 .70649 .3476 .72387 .34278 .71596 .33177 Metetra
.622 .74 .931 r
.71596 .33177 .72387 .34278 .74138 .33785 .73356 .32695 Metetra
.627 .746 .933 r
.73356 .32695 .74138 .33785 .75901 .33278 .75128 .32191 Metetra
.71 .68 .819 r
.13296 .5003 .14257 .5138 .15645 .50863 .14726 .49195 Metetra
.66 .622 .8 r
.14726 .49195 .15645 .50863 .1701 .50595 .16134 .48573 Metetra
.595 .553 .778 r
.16134 .48573 .1701 .50595 .1835 .50644 .17514 .48257 Metetra
.512 .476 .753 r
.17514 .48257 .1835 .50644 .19662 .51066 .18858 .48335 Metetra
.412 .388 .723 r
.18858 .48335 .19662 .51066 .20946 .51907 .20165 .48887 Metetra
.29 .29 .687 r
.20165 .48887 .20946 .51907 .22207 .53192 .21435 .49973 Metetra
.144 .18 .64 r
.21435 .49973 .22207 .53192 .2345 .54923 .22671 .51626 Metetra
0 .056 .578 r
.22671 .51626 .2345 .54923 .24684 .57073 .23883 .53846 Metetra
0 0 .493 r
.23883 .53846 .24684 .57073 .25921 .59588 .25082 .56593 Metetra
0 0 .379 r
.25082 .56593 .25921 .59588 .27172 .62385 .26283 .59787 Metetra
0 0 .238 r
.26283 .59787 .27172 .62385 .28451 .65354 .27504 .63306 Metetra
.81 .438 0 r
.27504 .63306 .28451 .65354 .29771 .68362 .2876 .66991 Metetra
.883 .47 .044 r
.2876 .66991 .29771 .68362 .31141 .71266 .30069 .70655 Metetra
.885 .456 .143 r
.30069 .70655 .31141 .71266 .32571 .73916 .31444 .74095 Metetra
.863 .45 .245 r
.31444 .74095 .32571 .73916 .33941 .78361 .32862 .77668 Metetra
.953 .758 .603 r
.32862 .77668 .33941 .78361 .35938 .71819 .3493 .70343 Metetra
.923 .725 .616 r
.3493 .70343 .35938 .71819 .37855 .65922 .369 .63824 Metetra
.892 .693 .627 r
.369 .63824 .37855 .65922 .397 .60696 .38781 .5814 Metetra
.861 .665 .636 r
.38781 .5814 .397 .60696 .41482 .56138 .40587 .53282 Metetra
.83 .64 .646 r
.40587 .53282 .41482 .56138 .43213 .52223 .42331 .49206 Metetra
.8 .618 .656 r
.42331 .49206 .43213 .52223 .44904 .48908 .44027 .4585 Metetra
.771 .6 .667 r
.44027 .4585 .44904 .48908 .46564 .46138 .45688 .43135 Metetra
.743 .586 .68 r
.45688 .43135 .46564 .46138 .48203 .43853 .47326 .40977 Metetra
.717 .577 .696 r
.47326 .40977 .48203 .43853 .49829 .41988 .48951 .3929 Metetra
.693 .572 .714 r
.48951 .3929 .49829 .41988 .5145 .40479 .5057 .37989 Metetra
.671 .572 .736 r
.5057 .37989 .5145 .40479 .53071 .39264 .52192 .36995 Metetra
.65 .578 .76 r
.52192 .36995 .53071 .39264 .54697 .38287 .53819 .36236 Metetra
.633 .588 .786 r
.53819 .36236 .54697 .38287 .5633 .37494 .55457 .35649 Metetra
.618 .604 .814 r
.55457 .35649 .5633 .37494 .57975 .3684 .57107 .35181 Metetra
.606 .623 .842 r
.57107 .35181 .57975 .3684 .59631 .36286 .58771 .34788 Metetra
.597 .644 .868 r
.58771 .34788 .59631 .36286 .61301 .358 .60449 .34435 Metetra
.592 .666 .89 r
.60449 .34435 .61301 .358 .62984 .35354 .62141 .34095 Metetra
.591 .688 .909 r
.62141 .34095 .62984 .35354 .6468 .34928 .63846 .3375 Metetra
.593 .707 .922 r
.63846 .3375 .6468 .34928 .6639 .34507 .65566 .33386 Metetra
.598 .723 .932 r
.65566 .33386 .6639 .34507 .68113 .34079 .67298 .32996 Metetra
.604 .736 .938 r
.67298 .32996 .68113 .34079 .69849 .33637 .69043 .32575 Metetra
.611 .746 .941 r
.69043 .32575 .69849 .33637 .71596 .33177 .708 .32123 Metetra
.619 .752 .941 r
.708 .32123 .71596 .33177 .73356 .32695 .72568 .3164 Metetra
.626 .757 .941 r
.72568 .3164 .73356 .32695 .75128 .32191 .74348 .31127 Metetra
.775 .741 .822 r
.12287 .48977 .13296 .5003 .14726 .49195 .13765 .47796 Metetra
.736 .684 .8 r
.13765 .47796 .14726 .49195 .16134 .48573 .1523 .46748 Metetra
.68 .615 .776 r
.1523 .46748 .16134 .48573 .17514 .48257 .16667 .45947 Metetra
.606 .536 .75 r
.16667 .45947 .17514 .48257 .18858 .48335 .18065 .45524 Metetra
.514 .448 .72 r
.18065 .45524 .18858 .48335 .20165 .48887 .19416 .45606 Metetra
.402 .35 .685 r
.19416 .45606 .20165 .48887 .21435 .49973 .20715 .46305 Metetra
.265 .239 .642 r
.20715 .46305 .21435 .49973 .22671 .51626 .21961 .47709 Metetra
.099 .114 .586 r
.21961 .47709 .22671 .51626 .23883 .53846 .23159 .49864 Metetra
0 0 .509 r
.23159 .49864 .23883 .53846 .25082 .56593 .24323 .52769 Metetra
0 0 .4 r
.24323 .52769 .25082 .56593 .26283 .59787 .25467 .56363 Metetra
0 0 .252 r
.25467 .56363 .26283 .59787 .27504 .63306 .26612 .60523 Metetra
.793 .475 0 r
.26612 .60523 .27504 .63306 .2876 .66991 .27783 .65063 Metetra
.903 .531 .079 r
.27783 .65063 .2876 .66991 .30069 .70655 .29004 .69744 Metetra
.919 .524 .197 r
.29004 .69744 .30069 .70655 .31444 .74095 .30296 .74286 Metetra
.814 .363 .164 r
.30296 .74286 .31444 .74095 .32862 .77668 .31776 .76839 Metetra
.951 .746 .59 r
.31776 .76839 .32862 .77668 .3493 .70343 .33936 .68467 Metetra
.916 .706 .604 r
.33936 .68467 .3493 .70343 .369 .63824 .3597 .61149 Metetra
.879 .669 .615 r
.3597 .61149 .369 .63824 .38781 .5814 .37893 .54914 Metetra
.842 .636 .624 r
.37893 .54914 .38781 .5814 .40587 .53282 .39722 .49731 Metetra
.806 .607 .633 r
.39722 .49731 .40587 .53282 .42331 .49206 .41475 .45527 Metetra
.772 .583 .643 r
.41475 .45527 .42331 .49206 .44027 .4585 .43171 .42204 Metetra
.739 .563 .654 r
.43171 .42204 .44027 .4585 .45688 .43135 .44828 .39645 Metetra
.708 .548 .669 r
.44828 .39645 .45688 .43135 .47326 .40977 .46458 .37729 Metetra
.679 .538 .686 r
.46458 .37729 .47326 .40977 .48951 .3929 .48076 .36336 Metetra
.652 .535 .708 r
.48076 .36336 .48951 .3929 .5057 .37989 .49691 .35352 Metetra
.627 .539 .735 r
.49691 .35352 .5057 .37989 .52192 .36995 .5131 .34674 Metetra
.605 .549 .765 r
.5131 .34674 .52192 .36995 .53819 .36236 .52938 .3421 Metetra
.586 .567 .799 r
.52938 .3421 .53819 .36236 .55457 .35649 .54579 .33887 Metetra
.572 .592 .835 r
.54579 .33887 .55457 .35649 .57107 .35181 .56234 .33643 Metetra
.561 .621 .869 r
.56234 .33643 .57107 .35181 .58771 .34788 .57905 .3343 Metetra
.556 .652 .899 r
.57905 .3343 .58771 .34788 .60449 .34435 .59592 .33216 Metetra
.557 .681 .922 r
.59592 .33216 .60449 .34435 .62141 .34095 .61293 .32976 Metetra
.563 .707 .938 r
.61293 .32976 .62141 .34095 .63846 .3375 .63008 .32698 Metetra
.573 .729 .948 r
.63008 .32698 .63846 .3375 .65566 .33386 .64737 .32372 Metetra
.584 .745 .953 r
.64737 .32372 .65566 .33386 .67298 .32996 .66478 .31999 Metetra
.597 .756 .954 r
.66478 .31999 .67298 .32996 .69043 .32575 .68232 .31578 Metetra
.609 .763 .953 r
.68232 .31578 .69043 .32575 .708 .32123 .69997 .31114 Metetra
.62 .767 .95 r
.69997 .31114 .708 .32123 .72568 .3164 .71773 .30612 Metetra
.629 .768 .946 r
.71773 .30612 .72568 .3164 .74348 .31127 .73561 .30076 Metetra
.837 .818 .831 r
.11206 .48355 .12287 .48977 .13765 .47796 .12734 .46861 Metetra
.816 .764 .805 r
.12734 .46861 .13765 .47796 .1523 .46748 .14264 .45352 Metetra
.773 .696 .778 r
.14264 .45352 .1523 .46748 .16667 .45947 .15778 .43956 Metetra
.709 .615 .75 r
.15778 .43956 .16667 .45947 .18065 .45524 .17259 .42842 Metetra
.626 .524 .719 r
.17259 .42842 .18065 .45524 .19416 .45606 .18688 .42197 Metetra
.524 .425 .684 r
.18688 .42197 .19416 .45606 .20715 .46305 .2005 .42213 Metetra
.401 .316 .645 r
.2005 .42213 .20715 .46305 .21961 .47709 .21337 .43064 Metetra
.251 .194 .596 r
.21337 .43064 .21961 .47709 .23159 .49864 .22547 .44887 Metetra
.064 .052 .531 r
.22547 .44887 .23159 .49864 .24323 .52769 .23686 .47757 Metetra
0 0 .436 r
.23686 .47757 .24323 .52769 .25467 .56363 .24771 .51672 Metetra
0 0 .296 r
.24771 .51672 .25467 .56363 .26612 .60523 .25825 .56538 Metetra
.722 .476 0 r
.25825 .56538 .26612 .60523 .27783 .65063 .2688 .62159 Metetra
.899 .575 .084 r
.2688 .62159 .27783 .65063 .29004 .69744 .27971 .68234 Metetra
.945 .584 .231 r
.27971 .68234 .29004 .69744 .30296 .74286 .29133 .74374 Metetra
.672 .138 0 r
.29133 .74374 .30296 .74286 .31776 .76839 .30694 .75721 Metetra
.947 .729 .576 r
.30694 .75721 .31776 .76839 .33936 .68467 .32977 .65891 Metetra
.903 .682 .592 r
.32977 .65891 .33936 .68467 .3597 .61149 .35093 .57522 Metetra
.858 .638 .602 r
.35093 .57522 .3597 .61149 .37893 .54914 .37062 .50626 Metetra
.815 .599 .61 r
.37062 .50626 .37893 .54914 .39722 .49731 .3891 .45128 Metetra
.773 .565 .618 r
.3891 .45128 .39722 .49731 .41475 .45527 .40664 .40895 Metetra
.733 .537 .628 r
.40664 .40895 .41475 .45527 .43171 .42204 .4235 .37762 Metetra
.695 .515 .639 r
.4235 .37762 .43171 .42204 .44828 .39645 .4399 .35549 Metetra
.659 .498 .655 r
.4399 .35549 .44828 .39645 .46458 .37729 .45604 .34073 Metetra
.625 .489 .676 r
.45604 .34073 .46458 .37729 .48076 .36336 .47207 .3316 Metetra
.593 .488 .704 r
.47207 .3316 .48076 .36336 .49691 .35352 .48811 .32655 Metetra
.563 .497 .739 r
.48811 .32655 .49691 .35352 .5131 .34674 .50424 .32422 Metetra
.536 .517 .782 r
.50424 .32422 .5131 .34674 .52938 .3421 .52051 .32347 Metetra
.514 .548 .829 r
.52051 .32347 .52938 .3421 .54579 .33887 .53695 .32344 Metetra
.499 .589 .876 r
.53695 .32344 .54579 .33887 .56234 .33643 .55357 .32349 Metetra
.492 .634 .916 r
.55357 .32349 .56234 .33643 .57905 .3343 .57035 .32317 Metetra
.497 .677 .945 r
.57035 .32317 .57905 .3343 .59592 .33216 .5873 .32222 Metetra
.511 .714 .964 r
.5873 .32222 .59592 .33216 .61293 .32976 .60441 .32051 Metetra
.531 .743 .972 r
.60441 .32051 .61293 .32976 .63008 .32698 .62165 .31801 Metetra
.555 .763 .975 r
.62165 .31801 .63008 .32698 .64737 .32372 .63902 .31476 Metetra
.578 .775 .972 r
.63902 .31476 .64737 .32372 .66478 .31999 .65652 .31083 Metetra
.599 .78 .967 r
.65652 .31083 .66478 .31999 .68232 .31578 .67413 .30632 Metetra
.615 .781 .961 r
.67413 .30632 .68232 .31578 .69997 .31114 .69185 .30131 Metetra
.628 .779 .954 r
.69185 .30131 .69997 .31114 .71773 .30612 .70968 .29589 Metetra
.637 .776 .947 r
.70968 .29589 .71773 .30612 .73561 .30076 .72763 .29016 Metetra
.88 .91 .845 r
.10036 .48254 .11206 .48355 .12734 .46861 .11597 .46609 Metetra
.889 .871 .813 r
.11597 .46609 .12734 .46861 .14264 .45352 .13186 .4473 Metetra
.872 .807 .783 r
.13186 .4473 .14264 .45352 .15778 .43956 .14789 .42715 Metetra
.826 .724 .753 r
.14789 .42715 .15778 .43956 .17259 .42842 .16382 .40739 Metetra
.754 .628 .721 r
.16382 .40739 .17259 .42842 .18688 .42197 .17935 .39045 Metetra
.661 .524 .686 r
.17935 .39045 .18688 .42197 .2005 .42213 .19419 .37929 Metetra
.549 .413 .648 r
.19419 .37929 .2005 .42213 .21337 .43064 .20808 .37703 Metetra
.417 .295 .606 r
.20808 .37703 .21337 .43064 .22547 .44887 .22083 .38661 Metetra
.254 .162 .553 r
.22083 .38661 .22547 .44887 .23686 .47757 .23238 .4104 Metetra
.046 .003 .479 r
.23238 .4104 .23686 .47757 .24771 .51672 .24281 .44982 Metetra
0 0 .365 r
.24281 .44982 .24771 .51672 .25825 .56538 .25234 .50499 Metetra
0 0 .186 r
.25234 .50499 .25825 .56538 .2688 .62159 .26133 .5744 Metetra
.842 .587 .043 r
.26133 .5744 .2688 .62159 .27971 .68234 .27029 .65467 Metetra
.959 .637 .238 r
.27029 .65467 .27971 .68234 .29133 .74374 .27979 .74046 Metetra
.252 .735 .929 r
.27979 .74046 .29133 .74374 .30694 .75721 .29645 .73958 Metetra
.938 .707 .563 r
.29645 .73958 .30694 .75721 .32977 .65891 .32103 .6199 Metetra
.882 .648 .579 r
.32103 .6199 .32977 .65891 .35093 .57522 .34323 .5221 Metetra
.827 .595 .588 r
.34323 .5221 .35093 .57522 .37062 .50626 .36339 .44563 Metetra
.774 .549 .594 r
.36339 .44563 .37062 .50626 .3891 .45128 .38194 .38867 Metetra
.724 .51 .601 r
.38194 .38867 .3891 .45128 .40664 .40895 .39928 .34864 Metetra
.676 .477 .609 r
.39928 .34864 .40664 .40895 .4235 .37762 .41581 .3226 Metetra
.63 .45 .621 r
.41581 .3226 .4235 .37762 .4399 .35549 .43185 .30757 Metetra
.584 .431 .639 r
.43185 .30757 .4399 .35549 .45604 .34073 .44765 .30069 Metetra
.539 .421 .667 r
.44765 .30069 .45604 .34073 .47207 .3316 .46342 .29938 Metetra
.493 .422 .706 r
.46342 .29938 .47207 .3316 .48811 .32655 .47928 .30145 Metetra
.446 .44 .759 r
.47928 .30145 .48811 .32655 .50424 .32422 .49532 .30513 Metetra
.402 .479 .824 r
.49532 .30513 .50424 .32422 .52051 .32347 .51157 .30911 Metetra
.366 .536 .891 r
.51157 .30911 .52051 .32347 .53695 .32344 .52804 .3125 Metetra
.35 .603 .943 r
.52804 .3125 .53695 .32344 .55357 .32349 .54472 .31478 Metetra
.362 .668 .974 r
.54472 .31478 .55357 .32349 .57035 .32317 .5616 .31569 Metetra
.401 .724 .989 r
.5616 .31569 .57035 .32317 .5873 .32222 .57864 .31521 Metetra
.455 .766 .996 r
.57864 .31521 .5873 .32222 .60441 .32051 .59583 .31343 Metetra
.509 .792 .997 r
.59583 .31343 .60441 .32051 .62165 .31801 .61315 .31052 Metetra
.555 .804 .991 r
.61315 .31052 .62165 .31801 .63902 .31476 .6306 .30666 Metetra
.59 .805 .982 r
.6306 .30666 .63902 .31476 .65652 .31083 .64816 .30206 Metetra
.615 .8 .97 r
.64816 .30206 .65652 .31083 .67413 .30632 .66583 .29688 Metetra
.631 .792 .959 r
.66583 .29688 .67413 .30632 .69185 .30131 .68362 .29126 Metetra
.641 .783 .949 r
.68362 .29126 .69185 .30131 .70968 .29589 .70151 .28532 Metetra
.648 .775 .941 r
.70151 .28532 .70968 .29589 .72763 .29016 .71952 .27913 Metetra
.84 .986 .865 r
.08791 .48534 .10036 .48254 .11597 .46609 .10336 .47099 Metetra
.892 .983 .807 r
.10336 .47099 .11597 .46609 .13186 .4473 .11941 .45223 Metetra
.935 .951 .769 r
.11941 .45223 .13186 .4473 .14789 .42715 .13605 .42867 Metetra
.944 .885 .744 r
.13605 .42867 .14789 .42715 .16382 .40739 .15314 .40096 Metetra
.904 .785 .72 r
.15314 .40096 .16382 .40739 .17935 .39045 .17036 .37115 Metetra
.823 .667 .689 r
.17036 .37115 .17935 .39045 .19419 .37929 .18724 .3429 Metetra
.718 .544 .654 r
.18724 .3429 .19419 .37929 .20808 .37703 .20325 .32117 Metetra
.595 .421 .616 r
.20325 .32117 .20808 .37703 .22083 .38661 .21786 .31163 Metetra
.455 .294 .573 r
.21786 .31163 .22083 .38661 .23238 .4104 .23065 .3199 Metetra
.282 .151 .518 r
.23065 .3199 .23238 .4104 .24281 .44982 .24141 .35079 Metetra
.054 0 .438 r
.24141 .35079 .24281 .44982 .25234 .50499 .25017 .4075 Metetra
0 0 .301 r
.25017 .4075 .25234 .50499 .26133 .5744 .25725 .49088 Metetra
0 0 .07 r
.25725 .49088 .26133 .5744 .27029 .65467 .26326 .59855 Metetra
.936 .671 .197 r
.26326 .59855 .27029 .65467 .27979 .74046 .26911 .724 Metetra
.788 .709 .777 r
.26911 .724 .27979 .74046 .29645 .73958 .28702 .70638 Metetra
.92 .676 .552 r
.28702 .70638 .29645 .73958 .32103 .6199 .31428 .55317 Metetra
.847 .601 .566 r
.31428 .55317 .32103 .6199 .34323 .5221 .33779 .43637 Metetra
.777 .537 .572 r
.33779 .43637 .34323 .5221 .36339 .44563 .35826 .35315 Metetra
.711 .481 .574 r
.35826 .35315 .36339 .44563 .38194 .38867 .37646 .29881 Metetra
.649 .433 .578 r
.37646 .29881 .38194 .38867 .39928 .34864 .39312 .2679 Metetra
.587 .391 .584 r
.39312 .2679 .39928 .34864 .41581 .3226 .40885 .25481 Metetra
.522 .355 .596 r
.40885 .25481 .41581 .3226 .43185 .30757 .42413 .25424 Metetra
.45 .325 .618 r
.42413 .25424 .43185 .30757 .44765 .30069 .43933 .26144 Metetra
.362 .303 .657 r
.43933 .26144 .44765 .30069 .46342 .29938 .45466 .27248 Metetra
.245 .297 .718 r
.45466 .27248 .46342 .29938 .47928 .30145 .47028 .28439 Metetra
.093 .315 .792 r
.47028 .28439 .47928 .30145 .49532 .30513 .48621 .29513 Metetra
0 .363 .825 r
.48621 .29513 .49532 .30513 .51157 .30911 .50247 .30354 Metetra
0 .435 .805 r
.50247 .30354 .51157 .30911 .52804 .3125 .519 .30916 Metetra
0 .537 .812 r
.519 .30916 .52804 .3125 .54472 .31478 .53578 .31202 Metetra
.129 .658 .87 r
.53578 .31202 .54472 .31478 .5616 .31569 .55275 .31243 Metetra
.298 .764 .943 r
.55275 .31243 .5616 .31569 .57864 .31521 .56988 .31083 Metetra
.441 .825 .988 r
.56988 .31083 .57864 .31521 .59583 .31343 .58715 .30769 Metetra
.538 .842 .999 r
.58715 .30769 .59583 .31343 .61315 .31052 .60454 .30341 Metetra
.594 .833 .99 r
.60454 .30341 .61315 .31052 .6306 .30666 .62204 .29833 Metetra
.625 .816 .974 r
.62204 .29833 .6306 .30666 .64816 .30206 .63966 .29269 Metetra
.641 .799 .958 r
.63966 .29269 .64816 .30206 .66583 .29688 .65739 .28668 Metetra
.65 .784 .946 r
.65739 .28668 .66583 .29688 .68362 .29126 .67524 .28042 Metetra
.654 .773 .936 r
.67524 .28042 .68362 .29126 .70151 .28532 .6932 .27399 Metetra
.656 .765 .93 r
.6932 .27399 .70151 .28532 .71952 .27913 .71128 .26743 Metetra
.692 .962 .942 r
.0756 .48566 .08791 .48534 .10336 .47099 .0903 .47723 Metetra
.66 .965 .783 r
.0903 .47723 .10336 .47099 .11941 .45223 .10555 .46531 Metetra
.645 .905 .588 r
.10555 .46531 .11941 .45223 .13605 .42867 .12163 .44751 Metetra
.736 .9 .516 r
.12163 .44751 .13605 .42867 .15314 .40096 .1388 .42117 Metetra
.898 .934 .561 r
.1388 .42117 .15314 .40096 .17036 .37115 .15717 .38442 Metetra
.987 .896 .627 r
.15717 .38442 .17036 .37115 .18724 .3429 .17654 .33791 Metetra
.936 .764 .643 r
.17654 .33791 .18724 .3429 .20325 .32117 .1962 .2862 Metetra
.816 .61 .623 r
.1962 .2862 .20325 .32117 .21786 .31163 .2151 .23815 Metetra
.641 .436 .59 r
.2151 .23815 .21786 .31163 .23065 .3199 .23093 .21534 Metetra
.539 .339 .558 r
.23093 .21534 .23065 .3199 .24141 .35079 .24491 .20954 Metetra
.378 .2 .509 r
.24491 .20954 .24141 .35079 .25017 .4075 .25548 .23933 Metetra
.099 0 .416 r
.25548 .23933 .25017 .4075 .25725 .49088 .2611 .32676 Metetra
0 0 .251 r
.2611 .32676 .25725 .49088 .26326 .59855 .26289 .46926 Metetra
.776 .621 .057 r
.26289 .46926 .26326 .59855 .26911 .724 .26193 .66405 Metetra
.711 .51 .615 r
.26193 .66405 .26911 .724 .28702 .70638 .28101 .62953 Metetra
.887 .629 .542 r
.28101 .62953 .28702 .70638 .31428 .55317 .31271 .41899 Metetra
.787 .532 .552 r
.31271 .41899 .31428 .55317 .33779 .43637 .33758 .27924 Metetra
.695 .451 .55 r
.33758 .27924 .33779 .43637 .35826 .35315 .35746 .19845 Metetra
.607 .38 .546 r
.35746 .19845 .35826 .35315 .37646 .29881 .37402 .16348 Metetra
.518 .313 .542 r
.37402 .16348 .37646 .29881 .39312 .2679 .38868 .16134 Metetra
.414 .245 .54 r
.38868 .16134 .39312 .2679 .40885 .25481 .4025 .17999 Metetra
.271 .163 .542 r
.4025 .17999 .40885 .25481 .42413 .25424 .41628 .20896 Metetra
.018 .04 .54 r
.41628 .20896 .42413 .25424 .43933 .26144 .43046 .24002 Metetra
0 0 .438 r
.43046 .24002 .43933 .26144 .45466 .27248 .44526 .26764 Metetra
.755 .244 0 r
.44526 .26764 .45466 .27248 .47028 .28439 .46069 .28896 Metetra
.678 .163 .005 r
.46069 .28896 .47028 .28439 .48621 .29513 .47667 .30326 Metetra
.554 .042 0 r
.47667 .30326 .48621 .29513 .50247 .30354 .49306 .31122 Metetra
.404 0 0 r
.49306 .31122 .50247 .30354 .519 .30916 .50976 .31417 Metetra
.184 0 0 r
.50976 .31417 .519 .30916 .53578 .31202 .52667 .3135 Metetra
.133 .677 .707 r
.52667 .3135 .53578 .31202 .55275 .31243 .54375 .31044 Metetra
.425 .856 .938 r
.54375 .31044 .55275 .31243 .56988 .31083 .56096 .30587 Metetra
.573 .875 .994 r
.56096 .30587 .56988 .31083 .58715 .30769 .5783 .30041 Metetra
.628 .843 .982 r
.5783 .30041 .58715 .30769 .60454 .30341 .59575 .29443 Metetra
.648 .81 .961 r
.59575 .29443 .60454 .30341 .62204 .29833 .61331 .28816 Metetra
.655 .786 .944 r
.61331 .28816 .62204 .29833 .63966 .29269 .63099 .28172 Metetra
.657 .771 .933 r
.63099 .28172 .63966 .29269 .65739 .28668 .64878 .27517 Metetra
.658 .762 .926 r
.64878 .27517 .65739 .28668 .67524 .28042 .6667 .26855 Metetra
.658 .756 .922 r
.6667 .26855 .67524 .28042 .6932 .27399 .68474 .26188 Metetra
.658 .752 .919 r
.68474 .26188 .6932 .27399 .71128 .26743 .70289 .25515 Metetra
.658 .796 .948 r
.0643 .47798 .0756 .48566 .0903 .47723 .0786 .47261 Metetra
.644 .867 .983 r
.0786 .47261 .0903 .47723 .10555 .46531 .09302 .46708 Metetra
.486 .901 .879 r
.09302 .46708 .10555 .46531 .12163 .44751 .10759 .46095 Metetra
.10759 .46095 .12163 .44751 .1388 .42117 .12253 .45274 Metetra
.125 0 .089 r
.12253 .45274 .1388 .42117 .15717 .38442 .13832 .43847 Metetra
.057 0 .162 r
.13832 .43847 .15717 .38442 .17654 .33791 .15594 .4098 Metetra
0 0 .066 r
.15594 .4098 .17654 .33791 .1962 .2862 .17667 .35442 Metetra
.78 .77 .261 r
.17667 .35442 .1962 .2862 .2151 .23815 .20125 .26273 Metetra
.973 .892 .527 r
.20125 .26273 .2151 .23815 .23093 .21534 .22132 .20461 Metetra
.673 .727 .89 r
.22132 .20461 .23093 .21534 .24491 .20954 .23535 .19875 Metetra
.673 .727 .89 r
.23535 .19875 .24491 .20954 .25548 .23933 .24946 .19286 Metetra
.575 .42 .633 r
.24946 .19286 .25548 .23933 .2611 .32676 .26365 .18694 Metetra
.529 .319 .54 r
.26365 .18694 .2611 .32676 .26289 .46926 .27793 .18098 Metetra
0 0 .23 r
.27793 .18098 .26289 .46926 .26193 .66405 .27041 .42667 Metetra
.57 .342 .53 r
.27041 .42667 .26193 .66405 .28101 .62953 .28828 .39465 Metetra
.786 .523 .538 r
.28828 .39465 .28101 .62953 .31271 .41899 .32126 .16289 Metetra
.517 .295 .517 r
.32126 .16289 .31271 .41899 .33758 .27924 .33587 .1568 Metetra
.533 .327 .547 r
.33587 .1568 .33758 .27924 .35746 .19845 .35056 .15066 Metetra
.574 .418 .631 r
.35056 .15066 .35746 .19845 .37402 .16348 .36535 .14449 Metetra
.642 .598 .789 r
.36535 .14449 .37402 .16348 .38868 .16134 .38022 .13828 Metetra
0 0 .396 r
.38022 .13828 .38868 .16134 .4025 .17999 .39346 .16668 Metetra
.951 .599 .244 r
.39346 .16668 .4025 .17999 .41628 .20896 .40509 .2425 Metetra
.775 .423 .389 r
.40509 .2425 .41628 .20896 .43046 .24002 .41881 .29036 Metetra
.636 .302 .388 r
.41881 .29036 .43046 .24002 .44526 .26764 .43402 .31463 Metetra
.544 .209 .351 r
.43402 .31463 .44526 .26764 .46069 .28896 .45009 .32343 Metetra
.46 .103 .272 r
.45009 .32343 .46069 .28896 .47667 .30326 .46662 .32377 Metetra
.336 0 .102 r
.46662 .32377 .47667 .30326 .49306 .31122 .48339 .32008 Metetra
.061 0 0 r
.48339 .32008 .49306 .31122 .50976 .31417 .50033 .31468 Metetra
.4 .856 .848 r
.50033 .31468 .50976 .31417 .52667 .3135 .51739 .30861 Metetra
.611 .89 .989 r
.51739 .30861 .52667 .3135 .54375 .31044 .53456 .30228 Metetra
.651 .829 .968 r
.53456 .30228 .54375 .31044 .56096 .30587 .55185 .29585 Metetra
.658 .789 .944 r
.55185 .29585 .56096 .30587 .5783 .30041 .56925 .28936 Metetra
.659 .768 .93 r
.56925 .28936 .5783 .30041 .59575 .29443 .58677 .28281 Metetra
.659 .757 .923 r
.58677 .28281 .59575 .29443 .61331 .28816 .60441 .27623 Metetra
.658 .752 .919 r
.60441 .27623 .61331 .28816 .63099 .28172 .62217 .26959 Metetra
.658 .75 .918 r
.62217 .26959 .63099 .28172 .64878 .27517 .64004 .26292 Metetra
.658 .749 .917 r
.64004 .26292 .64878 .27517 .6667 .26855 .65804 .25619 Metetra
.658 .749 .917 r
.65804 .25619 .6667 .26855 .68474 .26188 .67616 .24942 Metetra
.658 .748 .917 r
.67616 .24942 .68474 .26188 .70289 .25515 .6944 .24261 Metetra
.658 .748 .917 r
.05319 .4681 .0643 .47798 .0786 .47261 .06754 .46269 Metetra
.658 .749 .917 r
.06754 .46269 .0786 .47261 .09302 .46708 .08198 .45726 Metetra
.658 .752 .919 r
.08198 .45726 .09302 .46708 .10759 .46095 .0965 .45179 Metetra
.659 .768 .93 r
.0965 .45179 .10759 .46095 .12253 .45274 .11112 .44629 Metetra
.652 .831 .969 r
.11112 .44629 .12253 .45274 .13832 .43847 .12582 .44075 Metetra
.396 .853 .827 r
.12582 .44075 .13832 .43847 .15594 .4098 .14061 .43518 Metetra
.303 0 .131 r
.14061 .43518 .15594 .4098 .17667 .35442 .1555 .42957 Metetra
.439 .147 .37 r
.1555 .42957 .17667 .35442 .20125 .26273 .17048 .42394 Metetra
.472 .208 .432 r
.17048 .42394 .20125 .26273 .22132 .20461 .18555 .41826 Metetra
.478 .221 .445 r
.18555 .41826 .22132 .20461 .23535 .19875 .20072 .41255 Metetra
.478 .221 .445 r
.20072 .41255 .23535 .19875 .24946 .19286 .21598 .4068 Metetra
.478 .221 .445 r
.21598 .4068 .24946 .19286 .26365 .18694 .23133 .40102 Metetra
.478 .221 .445 r
.23133 .40102 .26365 .18694 .27793 .18098 .24679 .3952 Metetra
0 0 .015 r
.24679 .3952 .27793 .18098 .27041 .42667 .28282 .16383 Metetra
.514 .291 .513 r
.28282 .16383 .27041 .42667 .28828 .39465 .29731 .15774 Metetra
0 0 .206 r
.29731 .15774 .28828 .39465 .32126 .16289 .29374 .37753 Metetra
.478 .221 .445 r
.29374 .37753 .32126 .16289 .33587 .1568 .30959 .37156 Metetra
.478 .221 .445 r
.30959 .37156 .33587 .1568 .35056 .15066 .32554 .36555 Metetra
.478 .221 .445 r
.32554 .36555 .35056 .15066 .36535 .14449 .34159 .35951 Metetra
.478 .221 .445 r
.34159 .35951 .36535 .14449 .38022 .13828 .35775 .35342 Metetra
.478 .221 .445 r
.35775 .35342 .38022 .13828 .39346 .16668 .37401 .3473 Metetra
.474 .213 .438 r
.37401 .3473 .39346 .16668 .40509 .2425 .39038 .34114 Metetra
.454 .174 .398 r
.39038 .34114 .40509 .2425 .41881 .29036 .40685 .33494 Metetra
.394 .067 .288 r
.40685 .33494 .41881 .29036 .43402 .31463 .42343 .32869 Metetra
.153 0 0 r
.42343 .32869 .43402 .31463 .45009 .32343 .44012 .32241 Metetra
.505 .903 .928 r
.44012 .32241 .45009 .32343 .46662 .32377 .45692 .31608 Metetra
.651 .835 .971 r
.45692 .31608 .46662 .32377 .48339 .32008 .47383 .30972 Metetra
.659 .778 .937 r
.47383 .30972 .48339 .32008 .50033 .31468 .49085 .30331 Metetra
.659 .757 .923 r
.49085 .30331 .50033 .31468 .51739 .30861 .50799 .29686 Metetra
.658 .751 .918 r
.50799 .29686 .51739 .30861 .53456 .30228 .52523 .29036 Metetra
.658 .749 .917 r
.52523 .29036 .53456 .30228 .55185 .29585 .5426 .28382 Metetra
.658 .748 .917 r
.5426 .28382 .55185 .29585 .56925 .28936 .56008 .27724 Metetra
.658 .748 .917 r
.56008 .27724 .56925 .28936 .58677 .28281 .57768 .27062 Metetra
.658 .748 .916 r
.57768 .27062 .58677 .28281 .60441 .27623 .59539 .26395 Metetra
.658 .748 .916 r
.59539 .26395 .60441 .27623 .62217 .26959 .61323 .25723 Metetra
.658 .748 .916 r
.61323 .25723 .62217 .26959 .64004 .26292 .63119 .25047 Metetra
.658 .748 .916 r
.63119 .25047 .64004 .26292 .65804 .25619 .64927 .24366 Metetra
.658 .748 .916 r
.64927 .24366 .65804 .25619 .67616 .24942 .66747 .23681 Metetra
.658 .748 .916 r
.66747 .23681 .67616 .24942 .6944 .24261 .6858 .22991 Metetra
0 g
.25 Mabswid
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
[(80)] .07825 .23514 .98066 1 Mshowa
.22216 .18933 m
.227 .19478 L
s
[(90)] .21247 .17844 .88892 1 Mshowa
.36358 .12987 m
.36813 .13557 L
s
[(100)] .3545 .11847 .79718 1 Mshowa
.51343 .06687 m
.51763 .07282 L
s
[(110)] .50502 .05496 .70545 1 Mshowa
.67245 0 m
.67627 .00621 L
s
[(120)] .66483 -0.01242 .61371 1 Mshowa
.125 Mabswid
.11461 .23456 m
.11764 .23771 L
s
.14105 .22344 m
.14405 .22662 L
s
.16779 .2122 m
.17076 .21541 L
s
.19482 .20083 m
.19776 .20407 L
s
.2498 .17771 m
.25268 .18101 L
s
.27776 .16595 m
.2806 .16928 L
s
.30604 .15406 m
.30885 .15742 L
s
.33465 .14204 m
.33741 .14542 L
s
.39286 .11756 m
.39554 .12101 L
s
.42247 .10511 m
.42512 .10859 L
s
.45243 .09251 m
.45504 .09602 L
s
.48275 .07977 m
.48531 .08331 L
s
.54447 .05381 m
.54695 .05742 L
s
.57589 .0406 m
.57832 .04424 L
s
.60769 .02723 m
.61007 .0309 L
s
.63987 .0137 m
.64221 .0174 L
s
.25 Mabswid
0 .80374 m
.40037 1.0855 L
s
.09485 .87049 m
.09577 .86326 L
s
[(0.2)] .09301 .88495 .12735 -1 Mshowa
.18136 .93137 m
.18209 .92412 L
s
[(0.4)] .17991 .94587 .10002 -1 Mshowa
.26055 .98709 m
.2611 .97983 L
s
[(0.6)] .25944 1.00163 .07584 -1 Mshowa
.3333 1.03829 m
.33369 1.03102 L
s
[(0.8)] .33251 1.05285 .05428 -1 Mshowa
.40037 1.0855 m
.40063 1.07821 L
s
[(1)] .39987 1.10006 .03495 -1 Mshowa
.125 Mabswid
.02453 .821 m
.02518 .81668 L
s
.04852 .83789 m
.04914 .83356 L
s
.07196 .85438 m
.07254 .85004 L
s
.11722 .88623 m
.11774 .88189 L
s
.13908 .90162 m
.13958 .89727 L
s
.16046 .91666 m
.16092 .91231 L
s
.20181 .94576 m
.20221 .9414 L
s
.22181 .95983 m
.22219 .95548 L
s
.24138 .97361 m
.24174 .96925 L
s
.27931 1.0003 m
.27961 .99593 L
s
.29768 1.01322 m
.29796 1.00886 L
s
.31567 1.02589 m
.31593 1.02152 L
s
.35057 1.05045 m
.35079 1.04608 L
s
.3675 1.06236 m
.3677 1.058 L
s
.3841 1.07404 m
.38427 1.06967 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{185.688, 201.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002i0000bA000`40O003h00O^Goo002iOol00;Uo
o`00^Goo001eOol30003Ool40003Ool2000eOol007Ioo`03001oogoo00=oo`03001oogoo009oo`04
001oogoo000dOol007Ioo`03001oogoo00Aoo`05001oogooOol00002Ool00`00Oomoo`0bOol007Io
o`03001oogoo00Eoo`04001oogoo0002Ool00`00Oomoo`0bOol007Ioo`03001oogoo009oo`04001o
ogoo0002Ool01000Oomoo`00=7oo001eOol20005Ool20004Ool2000eOol00;Uoo`00^Goo002iOol0
08=oo`8003Aoo`00P7oo0`0000=oo`00Ool0<goo001JOol30003Ool30004Ool2000EOol20004Ool2
000cOol005]oo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000BOol200000goo001oo`03
Ool2000cOol005]oo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000?Ool30008Ool00`00
Ool0000bOol005]oo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000=Ool3000:Ool01000
Oomoo`00<Goo001KOol00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`002Woo0`003Goo00@0
07ooOol0035oo`00FWoo0P0017oo0P001Goo0P002Goo0`003goo00D007ooOomoo`00031oo`00Kgoo
0`004Woo00D007ooOomoo`00031oo`00KGoo0`0057oo00<007ooOol00Woo00<007ooOol0;Goo001[
Ool2000HOol00`00Oomoo`02Ool00`00Oomoo`0/Ool006Qoo`<001Yoo`03001oogoo009oo`03001o
ogoo02aoo`00@Goo0`0017oo0P0017oo0P005Woo0P000goo00<007ooOol05goo00<007ooOol00goo
00<007ooOol0:goo0012Ool00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol0019oo`<001mo
o`03001oogoo00=oo`03001oogoo02]oo`00@Woo00<007ooOol00Woo00@007ooOol0009oo`04001o
ogoo000@Ool2000ROol00`00Oomoo`04Ool00`00Oomoo`0ZOol0049oo`03001oogoo009oo`04001o
ogoo0002Ool01000Oomoo`003Woo0`008goo00<007ooOol01Goo00<007ooOol0:Goo0012Ool00`00
Oomoo`02Ool01000Oomoo`000Woo00@007ooOol000]oo`<002Ioo`03001oogoo00Eoo`03001oogoo
02Uoo`00@Goo0P001Goo0P0017oo0P002Woo0`00:7oo00<007ooOol01Woo00<007ooOol0:7oo001G
Ool3000[Ool00`00Oomoo`06Ool00`00Oomoo`0XOol005Eoo`<002eoo`03001oogoo00Moo`03001o
ogoo02Moo`00Dgoo0P00<7oo00<007ooOol027oo00<007ooOol09Woo000_Ool30004Ool2000HOol3
000bOol00`00Oomoo`08Ool00`00Oomoo`0VOol0039oo`04001oogoo0002Ool00`00Oomoo`0COol2
0003Ool3000_Ool00`00Oomoo`09Ool00`00Oomoo`0UOol0031oo`<0009oo`04001oogoo000BOol4
0002Ool300000cc^001oo`0^Ool00`00Oomoo`09Ool00`00Oomoo`0UOol002moo`04001oogoo0002
Ool01000Oomoo`0047oo0P0017oo0P0000@lkP00?>hlkP8002ioo`03001oogoo00Yoo`03001oogoo
02Aoo`00;goo00@007ooOol0009oo`04001oogoo000=Ool40002Ool3000013c^000lkP000Sc^0P00
;Woo00<007ooOol02goo00<007ooOol08goo000`Ool20004Ool2000<Ool40003Ool00`00?>h00002
?>h00`00?>h00002?>h2000^Ool00`00Oomoo`0;Ool00`00Oomoo`0SOol0049oo`800004Ool0044`
0003Ool00`00?>h00002?>h01@00?>h003c^00000Sc^00<007ooOol0:goo00<007ooOol037oo00<0
07ooOol08Woo000oOol30003Ool00`00@C000003Ool00`00?>h00002?>h20002?>h01000?>hlkP00
;Goo00<007ooOol037oo00<007ooOol08Woo000mOol2000017oo001oogoo0`0000=1<000Ool00Woo
00L003c^000lkP00?>h00002?>h00`00?>hlkP02000]Ool00`00Oomoo`0<Ool00`00Oomoo`0QOol0
03Yoo`<000Aoo`800005?>h00000@C000003Ool20002?>h00`00?>h00002?>h00`00?>hlkP02000]
Ool00`00Oomoo`0=Ool00`00Oomoo`0POol001Yoo`8000Aoo`8001Ioo`80009oo`03001oo`000080
0003?>h003c^00800007@C0007ooOol003c^00000Sc^00<003c^00000Sc^00<003c^?>h00`00;7oo
00<007ooOol03Goo00<007ooOol087oo000IOol01000Oomoo`000Woo00@007ooOol001=oo`8000Ao
o`800006?>h003c^?>h003c^0P0000M1<000Oomoo`00?>h00002?>h00`00?>h00002?>h00`00?>hl
kP03000/Ool00`00Oomoo`0>Ool00`00Oomoo`0OOol001Uoo`04001oogoo0002Ool01000Oomoo`00
47oo0`0000=oo`00Ool00Woo00D003c^000lkP00008lkP@00007@C0007ooOol003c^00000Sc^00<0
03c^00000Sc^00L003c^000lcP00NVL0000[Ool00`00Oomoo`0>Ool00`00Oomoo`0OOol001Yoo`80
00=oo`04001oogoo000>Ool30007Ool02000?>h003c^000lkP00?>h300001d4`001oogoo000lkP00
008lkP03000lkP00008lkP07000lkP00?<h007YW0000:goo00<007ooOol03goo00<007ooOol07Woo
000IOol01000Oomoo`000Woo00@007ooOol000]oo`<000Uoo`800003?>h00000008lkP03000lkP00
00800007@C0007ooOol003c^00000Sc^00<003c^00000Sc^00H003c^000lcP00NVL2000ZOol00`00
Oomoo`0@Ool00`00Oomoo`0MOol001Yoo`8000Aoo`8000Yoo`@000Yoo`<00006?>h003c^?>h003c^
0`0000M1<000Oomoo`00?>h00002?>h01@00?>h003c^00000Sc^00L003c>001jI`00HJ`0000YOol0
0`00Oomoo`0@Ool00`00Oomoo`0MOol002Yoo`8000ioo`05000lkP00?>h00002?>h01000?>h00000
0T4`00H007ooOol003c^0002?>h20002?>h01000?>hlkP000Sc>00@0066/HJ`002Qoo`8001=oo`03
001oogoo01aoo`009goo10003Woo0P0000@lkP00?>h0008lkP04000lkP000002@C002P00Ool00000
?>h003c^000lkP000Sc^00@003c^?>h0008lcP05001Q[66/HJ`0000TOol300000eKm001oo`0BOol0
0`00Oomoo`0LOol002Ioo`03001oogoo011oo`800004?>h003c^0002?>h01000?>h000000T4`00<0
07oo00000P000Sc^00<003c^00000Sc^00@003c^?>h0008lcP05001Q[66/HJ`0000QOol3000015Km
001FoEKm0P004goo00<007ooOol06goo000VOol00`00Oomoo`0@Ool2000013c^000lkP000Sc^1000
0T4`00D007oo000lkP00008lkP03000lkP00008lkP04000lkSc^0002?<h01000HJaQ[66/0P007Goo
0`0000AFo@00E_eFo@D001Aoo`03001oogoo01Yoo`009Goo0`0047oo00X003c=000lkP00?>h003c^
000lkP<00091<005001oo`00?>h00002?>h00`00?>h00002?>h01000?>hlkP000Sc>00H0066/HJ`0
054/000JOol3000015Km001FoEKm0`0000=Fo@00E_d00`004goo00<007ooOol06Woo000=Ool20004
Ool20004Ool20004Ool30004Ool00`00Ool0000?Ool20002?>h20002?>h00`00?>h000020002@C00
1@00Ool003c^00000Sc^00<003c^00000Sc^00<003c^00000cc>0P0000AQ[000DB`001Moo`<00004
E_d005KmE_d30002E_d00`00E_d000020002Ead00`00Oomoo`0AOol00`00Oomoo`0IOol000aoo`04
001oogoo0003Ool20003Ool01000Oomoo`0017oo00<007ooOol00Woo00D007ooOomoo`0000ioo`80
0004?>h003c^0002?>h00`00?>h000020002@C001@00Ool003c^00000Sc^00<003c^00000Sc^00<0
03c^00000Sc>00D003R/001Q[000009A;003001oogoo015oo`<0009Fo@03001Fo@000080009Fo@03
001FoEKm0080009G7@03001G7ELM0080019oo`03001oogoo01Uoo`001Woo1@0000Eoo`00Oomoo`00
00Qoo`04001oogoo0002Ool40003Ool00`00Oomoo`02Ool00`00Oomoo`0;Ool2000013c^000lkP00
0Sc^00<003c^00000P000T4`00D007oo000lkP00008lkP03000lkP00008lkP03000lkP00008lcP05
000h[000HJ`00002DB`3000>Ool30002E_d00`00E_eFo@020002Ead00`00E_eFo@03000015LM001G
7ELM0`0000=C7@00Ool04Woo00<007ooOol067oo000<Ool01000Oomoo`0027oo00@007ooOol0009o
o`03001oo`0000Aoo`03001oogoo00=oo`03001oogoo00Uoo`<00004?>h003c^0002?>h00`00?>h0
000200000d4`0000000200002Cc^000lkSc^000lkP00?>h00002?>h01000?<hlcP000SR/00@0054/
DBaA;0800003H>0007oo00Yoo`80009Fo@03001FoEKm00<00004Ead005LMEad3000015LM001G7ELM
1@000U<M0P004goo00<007ooOol05goo000<Ool01000Oomoo`0027oo00@007ooOol0009oo`03001o
o`0000Aoo`03001oogoo00Aoo`03001oogoo00Qoo`07000lc@00?>h003c^00000Sc^00<003c^0000
0P0000=1<00000000P0000@lkP00?>hlkP80008lkP04000lkSc^0002?<h01000>:`h[0000U4/00<0
04C;A</01@001Goo0`0000AFo@00E_eFo@<00004Ead005LMEad3000015<n001G7ELM0`0000=C?P00
Dad0100000=C7P0000004Woo00<007ooOol05goo000=Ool2000:Ool20004Ool20004Ool00`00Oomo
o`05Ool00`00Oomoo`07Ool01`00?<d003c^000lkP00008lkPD00003@C00000000800006?>h003c^
000lkP000Sc^00@003c^?>h0008lcP04000h[3R/0002DB`00`00A<]4b`020003E:001`00Oomoogoo
001G7@00009Fo@<00004Ech005LMEad3000015<n001C?U<n0`0000AC?P00DciC?P<0009C?P80009C
7P03001oogoo011oo`03001oogoo01Ioo`009Goo00<007ooOol01Woo00<007ooOol01Goo0`000Sc^
00H003c^000lkP00?>h400000d4`000000030002?>h00`00?>h00002?>h01P00?>hlkP00?<h000<h
[080000:DB`004C;A</003QX001DX000A204000015LM000000000U=N00<005Ln00000P000U=O00<0
05<n00000P000TmO1@000U<n0`0000AC7P00DaiC7P80015oo`03001oogoo01Ioo`0097oo0P002Goo
00<007ooOol017oo0`000Sc^0P000Sc^00@003c^0011D@800006@C000000000lkP000Sc^00<003c^
00000Sc^00H003c^?>h003c>0002>:`01000<4T000000dC;00<003QX>6P010000ULM0P000TnO00<0
05=NDeh00P000T]o1@000TmO0`0000A;?`00Ccm??`<00004Cah005<NDah300000e<N001oo`0AOol0
0`00Oomoo`0EOol002Aoo`03001oogoo00Uoo`03001oogoo00=oo`<00004?>h003c^0002?>h01000
?>h0045A0P0000I1<000000003c^0002?>h00`00?>h00002?>h01P00?>hlkP00?<h0008h[003000`
BC1900800004A</003QX>6P20002EadE00000dLo00000004000014lN001?7TlN0`0000A?7P00DaiC
7P80019oo`03001oogoo01Aoo`0097oo00<007ooOol02Woo00@007ooOomoo`800006?<d003c^000l
kP000Sc^00@003c^0011D@800006@C000000000lkP000Sc^00<003c^00000Sc^00H003c^?>h003c>
0002>:`01@00<4T`B@004000100000QCG`00000003WH000anP000Rhl00<002in00000Rjo00@003;o
<_l0008k7`D00004Aal004/OBal=00000e<N001oo`0AOol00`00Oomoo`0DOol002Aoo`03001oogoo
00]oo`03001oogoo00800006?<d003c^000lkP000Sc^00@003c^0011D@800006@C000000000lkP00
0Sc^00<003c^00000Sc^00<003c^00000Sc>00D003R/>:`0031900P00003?KH0000001400004@]h0
04;o@_l50002B_h00`00Cah000020002C_h00`00Cah00004000BOol00`00Oomoo`0COol002=oo`80
00ioo`0:001oo`00000lc@00?>h003c^0002?>h01000?>h0045A0P0000I1<000000003c^0002?>h0
0`00?>h00002?>h01@00?>h003c>00000SR/1@000TVd00<004FeAKD00P0000A9mP00BOL000966@04
0012>T8j0002?U`00`00?Yd00002?[h6000014[N001:oT[n1@000Tkn00<004knC_h00P000Tkm00@0
05;m0000015oo`03001oogoo01=oo`0097oo00<007ooOol03Goo10000Sc^00<003c^00000Sc^0`00
00A1D@000011<0<00009?>h003c^?>h003c^000lkP00008lkP04000lcP00>:`30003DM<00`00BKA9
]0030002BOH01000BOM9m`000THI00<0048j00001`000TJL00<004Jm00000T[N0`0000E:g@00C_e>
o@00009>oP<00004C_d004kmC_d20002D_d00`00Oomoo`0?Ool00`00Oomoo`0BOol002=oo`03001o
ogoo00ioo`@0008lkP06000lkP00?>h003c^0P0000A1D@000011<0<00004?>h003c^?>h20002?>h0
0`00?>hlkP070003DM<01000BK@000000Tge00H004WfBOH004Wg0002AQT200001DYI001:FTYJ0000
0TIk00<004JLAY`01@0000A:g@00B]e:g@P0009>o@<00004D_d005;mD_d2000AOol00`00Oomoo`0A
Ool002=oo`03001oogoo00eoo`<00004?<d003c^?>h20002?>h01@00?>h00000@E400P0000Y1<000
000003c^000lkP00?>h0008lkP03000lkP000080009A/P03001El`0000=Ad`800005EO@004geCOD0
0002BOH00`00BOL0000200001Thh001:FDYI001:FPH00006CY/004ZlB[`004[M0`0000A>g000C]e>
g@H00004D_d005;mD_d300000e;m001oo`0@Ool00`00Oomoo`0AOol002=oo`8000ioo`<00006?<d0
03c^000lkP000Sc^00D003c^0000045A00800003@C0000000080008lkP04000lkP00?>h30002FO80
1000DK9A/P000UGc00@0057CDM<0009Em005001=mDge0019mP0300000e8g001>>002CSP01P00BUT0
0000CWX0009>NP03001>VdjK00D0009>_003001>g0000080009Bg003001Bg@00009Bo@<00004D_d0
05;mD_d2000AOol00`00Oomoo`0@Ool000]oo`8000Aoo`8000Aoo`8000=oo`@000=oo`03001oogoo
00eoo`<00006?<d003c^000lkP000Sc^00H003c^0000045A0002@C02000013c^000lkSc^100000=I
d@00FO800UWb00@0056b0000009El`04001Ade7C0003EO@01P00COD00000DSH000=B=`03001>>000
00800006DUT004ijCWX004ij0`0000EBV`00C[]>^`00009>_080009Bg003001Bg5;L00D00004D]`0
05;mD_d300000e;m001oo`0@Ool00`00Oomoo`0@Ool000Yoo`04001oogoo0003Ool20003Ool01000
Oomoo`000goo00<007ooOol00goo00<007ooOol037oo0P0000<hR`0000000Sc^00<003c^00000Sc^
00H003c^0000045A0002@C0200000cc^000lkP0200001FHa001IdEWA00000eWb00@0056b0000009E
l`04001Ad`000003EO@200001EHe001B=U8f00000U8g0P0000=BF000DUT00U9I00H004ij000005:J
0003DY/00`00C[/00002000015:k001Bg5;L1@000U;L00<005;L00000P0000ABo000D_eBo@80015o
o`03001oogoo00moo`0017oo1@0000Eoo`00Oomoo`0000Qoo`04001oogoo0004Ool00`00Oomoo`02
Ool00`00Oomoo`0<Ool200000cR;00000002?>h00`00?>h00002?>h01P00?>h00000@E400091<0H0
009V<@04001Id@000003FO82000015hC001EleGc0P0000=J5000EO@00UGd00@005HeESD0009B=P04
001B=`000003DUP00`00DUT0000200001U9i001BVU:J001BV`<00004D[/005:kD[/300000e:k001B
g0060002D]`00`00D_`0000200000e;l001oo`0AOol00`00Oomoo`0>Ool000Yoo`04001oogoo0008
Ool01000Oomoo`001Goo00@007ooOol000moo`<00007?<d003c^?>h003c^00000Sc^100000A1D@00
@C11<080009V4@80009V<@04001Id@000003FO8200001EhC001EleGc00000eXD00@005GdEO@000=F
=@04001B=P000002EUL00`00DUQBF00200000eIh001BN@02DWT01000DYX000000U:J00<005:k0000
0P0000AB^`00D[]B^`<00005D]`005;LD]`00002D]`20002D]`01000D_`000004Goo00<007ooOol0
3Woo000:Ool01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0002Ool2000=Ool2000013R;
000lc@000Sc^00H003c^000lkP00?>h300001D5A0011<44`00000fHA0P000VHa00@005WA000000=I
lP07001N4ehCGQ<005Gc00000eXD00@005Gd000000=F=@800006FUH005IGEUL0059H0P000eIh00<0
059i00000P0000AFVP00DYYBVP80009F^`03001B^`0000800004E]/00000D]`6000015;L001Bg5;L
0P000U;l00<007ooOol03goo00<007ooOol03Goo000;Ool2000:Ool20004Ool20003Ool00`00Oomo
o`0<Ool2000013R;000lc@000Sc^0P000Sc^1@0000E1D@00@C11<00000=V4@80009V<@04001Id@00
0003FO801000GQ=N4ehC0`000eXD0P0000=J=@00ESD00UHe00@005YFFUH0009FE`80009FN003001F
N5Ih00800004EYT005JJEYX3000015Jj001F^eJk0P0000=F^`00E]/00UKK0P0000EFf`00E]aFg000
009Bg0<00004E]`005;lD_`2000@Ool00`00Oomoo`0=Ool0029oo`03001oogoo00]oo`<00009>8/0
03c=000lkP00?>h003c^00<0000310h0000000800091<004001V4FHAIQ420002IS420000168b001I
lUWb0P000ehC0P0000EN=000FQAJ500000=J=@04001F=@000003FUH200001UYg001FN5Ih001FN080
00=FV@800003EYX005Jj009F^P800006E[X005JkE[/005KK0P000eKK00<005KL00000P0000AFg000
E]aFg0<00003E_`00000011oo`03001oogoo00aoo`008Woo00<007ooOol02goo0P000SR;0P000Sc^
1`0000<43P0000000P0000=1<00000000fHA00@006HaIS5V<@80009R<P04001IlP000003GQ<20000
1Ehd001J55XD00000eXe0P000UYF00@005YFFUH000=JM`04001FN0000002FYP01000EYT000000eJJ
00@005Jj000000=F^P05001F^`00001Ff`0200001UKK00000000E]`0009Fg0<00006E]`00000001F
o0004Goo00<007ooOol02goo000QOol2000<Ool300001CR;000lc@00?>h00P0000A5]000000P/@80
000310h0000000<00003BE0006HA009V4@04001V<FHaIS420002HS801000FO8000000ehC00P005hd
GS@005XD001NE0000UXe00H005YFFUIJEP00FUH20003FWL2000015Yh001JV5ZH0P000UZI00<005JJ
00000P0000AF^P00E[YF^P<00003E[/005KK00<0009Ff`03001Fg0000080009Fg0800004E]`005Kl
E_`2000@Ool00`00Oomoo`0;Ool0025oo`03001oogoo00Yoo`@00003>8/0000000800003EQD004Fd
008000048;40000010h500000dU@001V4@02IQ401000IS5V<FHa0P000V8b0P0000=R<`00GQ<00UhC
00@005hdGSAN=0800005GU@005XeFSD00004FUH01000GWMNM`000UYg00@005YhFWQJN0800004FYP0
05ZIFYT20002F[T00`00E[X00002000015Zj001F^eJk1@000UKK0P000UKK00@005KL0000009Fg0<0
0003E_`007oo00ioo`<000aoo`008Goo00<007ooOol02Woo0`0000<hR`0000001`0000De<`000000
00@>00@00099D005001V4FHAIQ400003IS420002HS820002HS<01000GQ<000000ehd00P005iDGU@0
05Xe001NE@000UYF0P000Uig00D005Yg001NM`00009JN004001JV5ZHFYP2000015Zi001J^EZi0P00
0UZj00<005Zj00000P0000=Jf`00E]/0100000AFf`00E]]Ff`80009Fg0800004E]`005KlE_`2000;
Ool30002Ool00`00Oomoo`0:Ool0025oo`03001oogoo00Uoo`L00008DQL0046e@KD0035D000e<`<0
000310h0000000800099D005001V4FHAIQ400003IS401000HS9R<V8b0P000V8c00@005hC000000=N
=005001NE5iDGU@00002GUD01@00FUH005if00000Uig0P000Uig00@005Yh000000=JV004001J^EZi
F[T2000015Zi001J^UZj0P000UZj00<005[K00000P0000AJf`00E]]Ff`<00007E]/005KL001Ff`00
E]`00`0000=Fo000Ool01goo0`001Goo00<007ooOol02Woo000QOol00`00Oomoo`09Ool20002KcT0
0`00FWL00002DQL01P00@KD0035D000e<`<0000310h0000000800099D005001V4FHAIQ400003IS40
1@00HS9R<V8b00000f8c0P000V9C00@005hd000000=NE005001NEEiEGUD00003GWH2000015jG001N
Meig0P000UjH00@005ZH0000009J^@8000=J^@800004F[X005ZjF[X20002F]/00`00F]/000020000
15[K001FfeKK0P000UKK0P0000AFo000E_aFo08000Aoo`<000Qoo`03001oogoo00Yoo`008Goo00<0
07ooOol01goo100000E_>@00001JM`00009B5`040011]@00<E@500000`@>000000020002BE000`00
IQ5V4@020003IS401@00HS9R<V8b00000f8c0P000V9C0P0000ARE000GUANE08000=NE@05001NMUif
GWH00003GYL2000015jG001NV5jH0P000Uji0P0000AN^@00F[UJ^@8000=J^P800004F]X005[KF]/2
000015[K001Jfe[K0`0000MJf`00E]/005KK001Fo00300000eKl001oo`03000;Ool00`00Oomoo`0:
Ool0025oo`03001oogoo009oo`D00007Del00000KcT006;H00000UYg00<0058G00000T6e0P0000DP
d`00000000@>00@00099D003001V4FHA008000=V<@05001R<V8bHS800003HS<01000HU=RDf9C0P00
0V9D0P000V9e00@005iE0000009NMP8000=NU`04001NUejGGYL200001Ejh001N^Eji00000eji0P00
00AN^@00F[YJ^P80009JfPD00006F]/00000001Jf`000U[K0P000UKK0P000UKl00@005Kl000000mo
o`03001oogoo00Uoo`0087oo00@007ooOomoo`80009G7@03001G7@0000800003LgX006;H009Rf0T0
00058=<0000000043P040002BE000`00IQ5V4@020002IS420003HS801@00HS=R<f8c00000f9C00@0
069DHUARE080009RM@80009RMP800004HYH005jGGYL20002GYL20003G[P200000eji001N^@020003
G[T200001ekJ001JfP00F]X005[J00<0009Jf`03001Jf`0000800006F]/00000001Jn`000UKl0P00
00=Fo000Ool03Woo00<007ooOol02Goo000POol4000015Km001G7ELM100000EcNP00JaT006;H0080
0008CSL0047G@ML0035e000aE0@0000310h0000000800099D003001V4FHA0080009V<@80009R<P80
009R<`8000=RD`05001RE69DHU@00003HWD01@00HWIRMV9f00000f:F0P000V:G0P0000AR^000G[QN
^080009N^@80009N^@06001N^@00G]T005kJ0P000e[J0P0000AJfP00F]]Jf`800006F]/005[K001J
f`000U[k0`0000=Jo000E_`00P003Woo00<007ooOol02Goo000POol01@00E_eFo@00E_d020000f/I
00<005Ih00000Thg00H0047G000aM@00<E@90002BE000`00IQ5V4@020002IS420002HS820002HS<2
0002HU<20003HU@01@00HWERMF9e00000f9f00D006:FHYIRUP0000=RU`04001R^6:hH[P200001V:h
001N^@00G[T0009N^@80009Nf@80009NfP03001JfP0000800007F]X00000001Jf`00F]/00P000e[K
0P0000AJn`00F_aJo080009Jo003001oogoo00]oo`03001oogoo00Uoo`006Woo0P0017oo0P0000EF
o@00000005=o00<00005Lk/006]J001[6@020002EWP00`00CSL00002@ML<0002BE000`00IQ5V4@02
0002IS420002HS820002HS<20002HU<20002HU@02`00IW@0069e001VU@00HWH006JF00000V:F00D0
06:GHYMRU`0000=R^005001R^6:hH[P00003G[T200001UkI001Nf@00G]X0009NfP800006G]X00000
001JfP000U[K0P000U[K00@005[K0000009Jn`<00004F_/005[lF_`2000<Ool00`00Oomoo`09Ool0
01Uoo`04001oogoo0003Ool01@00Ool00000Ech0100000Ac^`00JeY[FP80009N^@03001FN0000080
0091e`8000038?@0023C00L00099D003001V4FHA0080009V<@80009R<P04001ZLP000002JW<20002
IW@20002IW@04P00HWD006JE001RMP00IYH006:F001VU`00HYL006:g001R^08000=R^005001N^Eji
G[T00003G]T01000G]YNfUkJ0P0000=NfP00G]X00P000e[J0P0000AJf`00F]]Jf`80009Jn`800004
F_/005[kF_/300000e[l001oo`0;Ool00`00Oomoo`09Ool001Uoo`04001oogoo0003Ool01000Oomo
ogoo0`000Woo0P0000E[FP00H_X005ji00<00003CSP0047h00<00006<GD000008=<000@?1@000TU@
00L006HA001bT@00IS400002KW820002JW820002JW<20002IW@20003IW@01@00IYEVUFJE00000fJF
00D006JGIYMVU`0000=R]`0=001R^6:h001R^000H[P005ji001Rf@00G]T00P000UkJ0P000UkJ0P00
00ANfP00001JfP80009Jf`<00007F_/005[k001Jn`00F_/00`0000AJn`00F_aJo08000]oo`03001o
ogoo00Uoo`006Goo00@007ooOol0009oo`8000Yoo`800005H_X00000EWT00002CSP01000@OP003FG
30000TU@00L006HA001bT@00IS400002KW820002JW820002JW<20002IW@00`00IWAVM002IW@01@00
IYEVUFJE00000fJF00<006JGIYL00P000V:g0P000V:h0P000V:h0P000V;I0P000V;I0P0000MNfP00
G]X005kJ001NfP030002G]X2000015kk001Jne[k0P000U[k0P0000AJn`00F_]Jn`<00003F_`00000
00]oo`03001oogoo00Qoo`006Goo00@007ooOol0009oo`03001oogoo00]oo`03001J^P00009FN@03
001>>4hh00800003=IL002De008000033780000000D00099D007001V4@00LY4006Ha00000Vib0P00
0VYb00D006YcJW=ZL`0000=VM004001VM6IdIW@20002IYD20002IYH01000I[H000000VJg0P000VJg
0P0000YR^000H[P006;H001Rf@00H]T0009Rf@05001NfUkJG]X00003G]X200001ekJ001NfP00G_X0
05kk00<00007F_/005[k001Jn`00F_/00`0000IJn`00000005[l000;Ool00`00Oomoo`08Ool001Yo
o`8000=oo`03001oogoo00]oo`05001J^UZj001FN@06000013FG0000023d20000TU@00L006HA001b
T@00IS400002KW820002JW801@00JW=ZLfYc00000VId00<006ZD00000VId00@006ZE0000009VUP80
009V]P05001V]fJgI[L00003I[L01@00H[QR^6:h00000f;H00@006;IH]URf@800007H]T005kJ001N
fP00G]X00`0000=NfP0000000Ukj0P0000ANn`00F_]Jn`80009Jn`800006F_/005[k001Jn`000U[l
0P002Woo00<007ooOol027oo000OOol00`00Oomoo`0<Ool400001DYI00126@00=KP03@000TU@00L0
06HA001bT@00IS400002KW820002JW803000JW=ZL`00JY<006Id001ZU000IW@0009ZU@80009VUP05
001V]VJfI[H00003I[L01@00I[MV]fJg00000V:h0P000V;H0P000V;I0P000V;I0P000UkJ0P000UkJ
0P0000MNnP00G_X005kj001Nn`030002F_/200000e[k001Jn`0300001U[k00000000F_`000Yoo`03
001oogoo00Qoo`007goo00<007ooOol037oo00@005:JDYX0009:F@800005=KP002UG000U=@090000
1TU@001V4FHA001bT@8000=^LP05001ZLVYbJW800002JW<02@00JY<006Id001ZU000IW@00002JYD0
1@00IYIVUVJF00000VJf00X006Zf001V]`00I[L006Jg001Vf080009Vf080009Rf080000;H]T006;I
001Rf@00G]X006;J001NfP020003G_X00`00G_YNnP020002G_/01000F_/000000U[k0P000U[k0P00
00AJn`00F_aJo08000Uoo`03001oogoo00Qoo`007Woo0P003goo00<005:J00001P0000<YE`000000
0P0000<<LP000000100000I9D000IQ5V4@00LY420003KW800`00JW9ZLP020002JW<01000JY<00000
0fZD00D006ZEJYEZU@00009VUP09001Z]P00I[H006Zf001V]`00009V]`04001Vf6KH0003I]P01@00
H]QRf6;H00000f;I00<006;IH]T00P000V;J0P0000ANnP00001NnP800004G_X005kk0002G_/20002
G_/200001e[k001Jn`00F_/005[k00<00003F_`007oo00Qoo`03001oogoo00Qoo`007Woo00<007oo
Ool03goo00H004YJ0012>@00>MT400000aSf00000007000014U@001V4@000W:A0P000fib00D006Yb
JW8006jC008000=ZT`05001ZU6ZDJY@00002JYD01000J[D000000fZf00D006ZfJ[IZ]P00009V]`80
009Vf0800009I]P006KH001Rf000I]T006;I00800009H]T006;i001RfP00H_X005kj00<00003G_X0
0000009Nn`<00003G_/005kk0080009Jn`80009Jn`800004F_/005[lF_`20008Ool00`00Oomoo`08
Ool001ioo`03001oogoo00moo`05001:FTYJ0012>@0200000bfH000YE`0200000a2d000000050000
14U@001V4@000W:A0P000fib00D006YbJW8006jC008000=ZT`03001ZU6ZD00800005JYD006ZeJ[D0
0004J[H00`00J[IZ]P02000016Jg001VefKG0P000VKH00D006KHI]QVf000009Vf@80009Rf@05001R
nF;iH_T00002H_X600001Ukj001Nn`00G_/0009Nn`80000;G_/00000001Nn`00F_/005[k001Jn`03
00000e[l00000008Ool00`00Oomoo`07Ool001ioo`03001oogoo011oo`D00003>MT0000000/00004
BE0006HA0002LY420002KW82000016Yb001^TfjC0P000VZC0P000VZD00@006je0000009Z]@06001Z
]VZfJ[H006[F0P000V[G00D006KGI]MVe`00009Vf080009Vf0800009I]T006Ki001Rf@00H_T006;i
00800009H_X006;j001RnP00H_X005kj008000=Nn`800003G_/00000009Nn`04001Nn`000002F_/2
0002F_/200000e[l001oo`07Ool00`00Oomoo`07Ool001ioo`03001oogoo011oo`030016>P00008j
6@800004;IP000006?H70002BE000`00IQ400002LY401000KW9^LVib0P0000EZLP00KY=^T`0000=Z
T`04001^]0000002K[D02000J[EZ]@00K[D006Zf0002J]H01@00J]MZef[G00000VKG0P0000AVf000
I]QVf080009Vf005001VnFKiI_T00002H_T20002H_T01000H_YRnV;j0P0000=RnP0000000Ukj00P0
05kk000005kk001Nn`000Ukk0P000Ukk0P0000=Jn`00F_/00P000U[l0P001goo00<007ooOol01goo
000NOol00`00Oomoo`0AOol01P00ASX003XI000a^@/00099D003001V4@00009bT@06001^LVibKW80
07:B0P000VjC00H006ZCJY=ZT`00K[@20002K[D02000J[EZ]@00K[D006Zf0002J]H01@00J]MZe`00
J]L00P000V[G00D006KHI]QVf00000=Vf003001VnFKi00800009H_T006Ki001Rn@00H_X006;j00@0
0004H_X005kjG_X200000ekk00000003G_/200000ekk001Nn`020002F_/200001U[k00000000F_`0
00Moo`03001oogoo00Moo`007Woo00<007ooOol04Woo1P0000<UN00000001`000TU@00<006HA0000
0W:A00H006ibKW9^LP00LY820002KY<200001FZC001^]6jd00000Vje00D006je001Z]@00009^]@05
001ZeV[F001ZeP020002J]L01@00J]MZef[G00000fKH0P0000=Vf000I_P00P000VKi00<006KiI_T0
0P000V;j0P000V;j00@006;jH_YRnP800003H_X00000009Nn`04001Nn`000002G_/200001Ekk0000
0000G_/00P000U[k00@005[l000000Ioo`03001oogoo00Moo`007Goo00<007ooOol04goo00@003hj
000enPH0000319@0000000<00099D003001V4@00009bT@04001^LVib0002LY802000KY=^T`00L[<0
06ZC0002K[@01000K[E^]@000Vje00L006jeK[E^]@00J]H00002J]H01@00J]MZef[G00000V[G00D0
06[h001Vf000009Vn004001Vn6Kh0003I_T02P00I_T006Ki001RnP00H_X006;j0`0000MRnP00001R
nP00H_X00002G_/20002G_/200002Ekk000005kk001Nn`00G_/00002F_/200000e[l00000005Ool0
0`00Oomoo`07Ool001eoo`03001oogoo01Aoo`05000n>P00000Y^@0:0002BE020003LY401000KW9^
LP000W:B00h006jCKY<007:c001ZT`00K[@006kD001^]@000Vje00D006jeK[D006kF0080009ZeP08
001Zef[G001Ze`00J]L0009Zn003001Vn6Kh00800008I_P006Ki001Vn@00I_T000=Vn@03001RnV;j
00800004H_X000000002H_X01000H_YRnV;j0P0000=Nn`00G_/00P0000ENn`00G_/005kk0080009N
n`<00005F_/00000F_`00005Ool00`00Oomoo`07Ool001eoo`8001Eoo`H000037GP0000000H00099
D080009bT@80009^LP04001bTW:B0002KY<01000L[=b/`000Vjd00@006kD0000009^]@800005K[D0
06kFK]H00002J]H01@00K]L006[G00000V[G00<006[hJ_P00P0000EVn000J_QZn000009Vn@80009V
n@80000@I_T00000H_X006;j001RnP00H_X006;j001RnP00H_X20002G_/200000ekk00000002G_/2
0002G_/200001ekk001Jn`00F_/005[l008000Eoo`03001oogoo00Ioo`001Goo0P0017oo0P0017oo
0P000goo10000goo00<007ooOol05Goo0P0000<]fP0000002@000TU@0P000W:A0P0000=^LP00LY80
0W:B00<006jC00000g:c00<006jd00000fkD00D006jeK[D006kE0080009^eP800005J]H006kGK]L0
0002J]L200001F[h001Zn6[h00000V[h0P0000IVn@00I_T006Ki0002I_T01000I_UVn@000V;j0`00
00=RnP0000000V;j0P000V;j0P0000=Nn`0000000Ukk0P0000=Nn`00G_/00P000Ukk0P000U[k0P00
00=Jo000Ool017oo00<007ooOol01Woo0004Ool01000Oomoo`000goo0P000goo00@007ooOol000=o
o`03001oogoo00=oo`03001oogoo01Ioo`<000049IT000003AL60002BE020002LY401000M[400000
0W:B0P0000A^T`00L[=b/`800003K[@006kD009^e003001^]@00009^e@04001^eVkF0002K]H01`00
K]M^efkG001Ze`00009Zm`03001Zn6[h00800005J_P006[hJ_P00002I_T20002I_T200000fKi0000
000600001f;j001RnP00H_X006;j00800005G_/005kk001Nn`020002G_/2000015kk000000000Ukk
0P0000=Nn`00F_`00P0017oo00<007ooOol01Woo0004Ool01000Oomoo`0027oo00@007ooOol000Ao
o`03001oogoo009oo`03001oogoo01Ioo`03000anP0000X00099D080009bT@04001f/@000002LY80
1000L[8000000W:c00@007;D0000009^e004001be@000002K]D200001FkF001^eVkF00000VkG00<0
06kg00000f[g00<006[h00000V[h00`006[hJ_QZn000I_T006Ki001Vn@00I_T300000fKi00000002
H_X20002H_X00`00H_YRnP0200000f;j00000002G_/20002G_/20002G_/01@00G_]Nn`00G_/00P00
0Ukk0P0000=Jo00000000goo00<007ooOol01Woo0004Ool01000Oomoo`0027oo00@007ooOol000Eo
o`04001oogoo000JOol200000bWJ000000070002BE020002LY401000M[4000000W:B00<007:b0000
0g:c00<007;D00000fkD00<007;E00000VkE00@006kEK]D0009^eP04001^mP000002K_L2000016[g
001Zm`000f[h0P0000AZn000J_P000=Vn@D00007I_T006Ki001RnP00I_X0100000=RnP00H_X00P00
00]RnP00G_/006;k001Nn`00G_/005kk0080009Nn`800005G_/00000001Nn`0200000e[l001oo`02
Ool00`00Oomoo`06Ool000Aoo`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol0009oo`03
001oogoo01Qoo`@000047IT000TH2AP40002BE020002LY400`00M[400002LY801000L[9b/P000W:c
00@007;DL]@0009^e004001beG;E0002K]D01P00K]E^e@00K]H0009^mP04001^mfkg0002K_L00`00
J_MZm`0200001F[h001Zn6[h00000V[h1@0000EVn@00I_UVn@00009Vn@80009VnPD00005H_X006;j
001RnP020002H_/20002G_/200001Ekk00000000G_/00P0000MNn`00G_/005kk001Jo0020002Ool0
0`00Oomoo`06Ool000Eoo`8000Yoo`8000Aoo`8000=oo`8001Uoo`03000]n`0000@000032AP00000
00800099D004001bTG:A0002M[401000LY9bTP000W:b01<007:cL[<007;D001be000K]@007;E001b
m@00K]D006kE001bmP020002K_H200001fkg001^mfkg001Zm`00009Zn003001Zn6[h00@0009Zn004
001VnFKi0003I_T02000I_T006Kj001VnP00I_X3000016;j001RnV;j0P000V;k0P000V;k0P0000YN
n`00001Nn`00G_/005kk001Nn`80009Nn`800005F_`00000Ool00008Ool001aoo`03001oogoo01Uo
o`/00099D004001bTG:A0002M[401000LY9bTP000W:b0P0000Yb/`00L]@007;D001^e000L]D0009b
m@06001^eFkE001bmP000Vkf00@006kfK_H0009^m`04001^m`000002J_P4000016[h001Zn6[h0P00
00QVn@00J_T006Ki001Vn@000fKj00@006Kj001W6P@00009H_X006<J001Rn`00Ha/006;k00800005
G_/005lK001Nn`020002G_/200001Ekk00000000G_/00P0000EJo000Oomoo`0000Moo`0077oo00<0
07ooOol06Goo00<002Wk:O/0100000@5600000000099D004001bTG:A0002M[401`00LY9bTP00L[80
07KC0080009be006001be7;D001be@000g;e00<006kE00000W;f0P0000M^mP00K_I^mP00K_L00002
K_L00`00J_P000030002J_P200001F[h001ZnF[i00000V[i0P0000QVn@00IaT006Kj001W6P000VLJ
0`0000=S6P00HaX00V<J00<006<KHa/00P0000=S6`0000000UlK0P000UlK0P0000YO6`00001O6`00
G_/005lK001Jo0800003Ool007oo00Ioo`0077oo00<007ooOol06Woo00@002Wk000Q^PH00099D005
001bTG:A001f/@0200001G:B001b/W:b00000WKC00<007;D00000g;D00<007;E00000W;e00H007;e
000007;f0002L_H01`00K_I^mP00K_L006kg00D0009^n003001Zn000009Zn007001ZnF[iJ_T006[i
00000V/I00<006LIIaT00P000VLJ00`006LJ001W6P00HaX006<J001S6P00HaX200000f<K00000002
Ha/200000elK001O6`0200001ElK001O6`00Ga/00P0000EO6`00000005lL00<000Moo`006goo00<0
07ooOol077oo100000<IVP0000000P0000=9D00000000W:A00@007Ja001jdP80009b/P06001fdgKC
001be0000W;D00@007;d0000009bm@05001bm@00L_H00003L_H01000K_H006kg10000Vkg00H006kh
K_P006lH0003J_P2000016[i001[6@000f/I00T006LI001W6@00IaX006LJ00000fLJ00<006<J0000
0P0000QS6P00001S6`00Ha/006<K0P000UlK0P0000=O6`0000000UlK0P0000=O6`0000000UlK0P00
00=O700000001goo000KOol2000MOol00`00:O/0000600000dU@00000002LY401000M[4007[B0P00
0W:b00H007KCM]<007;D0002L]@00`00L_@00002L_D01P00L_Ebm@00L_H0009bmP03001bmP0000<0
0003K_L006kg009^m`800003K_P006lH00800005J_P006/HJaP00002JaT2000016/I001[6@000VLI
0P0000MW6P00IaX006LJ001W6P050002HaX00`00Ha]S6`020002Ha/200000elK00000002Ga/20000
1UlK000005lK001O6`80009O708000Moo`006goo00<007ooOol07Goo0P0000<Q^`000000100000AJ
4`00LY40009f/@03001jdP00009b/P06001fdgKC001fl`000g;D00X007;d000007;e001bm@00L_D0
009bmP@00004K_L006kgK_L200001fkg001_5flG001_6000009_6003001[66/H00800005JaT006/I
JaT00002JaT00`00IaT00002IaT00`00IaYW6P02000016LJ000006<J0`0000QS6P00001S6`00Ha/0
06<K0P0000EO6`00Ga/005lK00800005Ga/005lK001O6`0200001ElK00000000Ga`00P001Woo000K
Ool00`00Oomoo`0NOol300000aFJ00000002000015XC001bT@000WJa00<007[B00000W:b00H007KC
M]<007Kc0002L]@00`00L_@00002M_D00`00L_Ebm@0600000g;f001c5P0200001Fkg001_5flG0000
0VlG00<006lG00000VlH0P0000E[6000JaQ[6000009[6@800004JaT006/I0003IaT00`00IaX00002
IaX01@00IaYW6P00HaX00`0000=S6P0000000V<K0P0000=S6`0000000UlK0P0000=O6`0000000UlK
0P0000=O6`0000000UlK0`001goo000KOol00`00Oomoo`0OOol60002FQ<01@00LY4007Ja00000W[B
00@007:bL[80009fd`03001fl`00009be003001bm000009fm@D0009bmP03001bmP00009c5P800007
K_L006lGKaL006lG00000VlG00<006lH00000VlH00<006/HJaP00P0000=[6@00JaT00P0000Q[6@00
JaT006LI001W6@000VLJ0P0000AW6P00001S6P800004HaX006<JHaX200000f<K00000002Ha/20000
0elK00000002Ga/200001elK000005lK001O6`0000=oo`03001oogoo00Aoo`006goo00<007ooOol0
7goo00H002Gk000001FK0003FQ<2000017Ja001jdW[B0P0000Ub/P00M]<007Kc001fm000L]@00P00
00=fm@0000000`0000Mbm@00L_IbmP00L_H00002LaH00`00LaL00002KaL00`00KaL00002KaL00`00
KaP00003KaP00`00JaP00002JaP00`00JaT00002JaT00`00JaU[6@0200001FLI000006LJ00000VLJ
00<006LJIaX00P0000AS6P00001S6P800005Ha/006<K001S6`0200001ElK001S6`00Ga/00P0000EO
6`00Ga/005lK008000Aoo`03001oogoo00Aoo`006Woo0P008Woo0P0000<Mf`0000000eXC0P0000Af
/@00N]9jdP<0009fd`03001fl`00009fm0@00004M_D007Ke0002L_D01@00L_H007<F00000W<F00H0
07<GLaL006lG0002KaL00`00KaL00002KaP200000flH001_6002000016/H001[6@000f/I00<006/I
00000V/I00<006LJIaX00P0000IW6P00001W6P00IaX2000016<J000006<K0`0000AS6`00001S6`80
009S6`800004Ga/00000Ga/30006Ool00`00Oomoo`04Ool001Yoo`03001oogoo029oo`@000=J4`80
0004M[4007[B0002N_801@00M]=fd`00M_<0100000Abm000M_D0009fm@<0009bmP0:001c5P00001c
5P00LaL007<G0002KaL00`00KaL00002KaP01000KaQ_60000VlH00<006lH00000V/I0P0000A[6@00
JaT0009[6@800003IaX006LJ0080009W6P06001W6P00IaX006<J0`000V<K0P000V<K00<006<KHa/0
0P0000=S6`0000000UlK00<007ooOol01goo00<007ooOol017oo000JOol00`00Oomoo`0SOol30003
FQ<01P00M[5f/@00N]80009jlP03001fd`000080009fm003001fm000009fm@03001fm@00009g5@05
001bmP00LaH00002LaH00`00LaL00002LaL200000flG001_5`0200000flH001_600200000flH001_
600200001V/I001[6F/I001[6@80000=JaT006/IJaT006LJ001W6P00IaX006LJ00000VLJ0P0000QS
6P00001S6`00Ha/006<K0P0000US6`00001S6`00Ha/005lK00002Woo00<007ooOol017oo000JOol0
0`00Oomoo`0SOol010008O`000000UXC00<007Ja00000W[B00<007[b00000`0000Ifl`00M_Afm000
M_@2000017Ke001fm@000WLE00D007;f001c5P00009c5P05001c5`00LaL00002LaL00`00KaL00002
KaL00`00KaP00002KaP00`00KaP00002KaP01P00JaU[6@00JaT0009[6@800004JaT006/I0002IaX2
00001VLJ000006LJ001W6P800003HaX00000009S6`800004Ha/00000Ha/20002Ha/2000;Ool00`00
Oomoo`04Ool001Yoo`03001oogoo02Aoo`<0009J4`03001f/@0000D0009jlP03001fl`00009fm080
009g5003001fm@00009g5@0:001g5@00001c5P00LaH007<F0002LaL01P00LaMc5`00KaL0009_5`80
009_6003001_6000009_60800003JaT006/I00800005JaT006/IJaT00002JaT01000IaX006LJ0P00
0VLJ00H006LJ001W6P00HaX400000f<K00000002Ha/00`00Ha/00002000=Ool00`00Oomoo`04Ool0
01Yoo`8002Eoo`04000Qo000FQ<40002N]800`00N_800002N_820002M_@00`00M_@00002Ma@00`00
M_D00002MaD01`00MaD007LF001c5P00009c5P800006LaL007<G001c5`000VlG00D006lG001_6000
009_6003001_6000009_6003001[6@00009[6@03001[6F/I0080000<JaT006/I001W6P00IaX006LJ
001W6P000VLJ0P0000]S6P00001S6`00Ha/006<K001S6`00015oo`03001oogoo00=oo`000Woo0P00
17oo0P0017oo0P0017oo0`000Woo00<007ooOol09Woo1@0000Af/@00N]80009jlP05001jlP00Na<0
0002M_@01`00M_@007LD001g5@00009g5@03001g5@00009g5P05001c5P00LaH00002LaH00`00LaL0
0002LaL01`00KaL006lG001_6000009_6003001_600000=_608000=[6@04001[6@00JaT2000016/I
001[6@000fLJ00L006LJ000006LJ001W6P020002IaX00`00Ha]S6`0200000f<K001oo`0AOol00`00
Oomoo`03Ool00005Ool007ooOol00003Ool20003Ool01000Oomoo`0017oo00D007ooOomoo`0002Io
o`@0009Vf007001f/@00N]8007[b00000W[b00D007/C000007Kd0080009g500<001g5@00001g5@00
MaD007LF001g5P000W<F00<007<FLaH00P000W<G00L007<G000006lG001_600200000flH001_6002
00000flH001_600200001V/I001[6@00JaT0009[6@03001[6F/I00800004IaX006LJ0002IaX01P00
IaX006LJ001W6P@00003Ha/0000001=oo`03001oogoo00=oo`0000Eoo`00Oomoo`0000Qoo`04001o
ogoo0002Ool40002Ool00`00Oomoo`0POol40003Ool01@00I]QVf6KH00000W[B00<007[b00000W[b
00L007/C001k4`00M_@00002Ma@01000MaD007LE0P0000Eg5@00MaH007LF00800008LaH007<F001c
5`00LaL0009c5`05001c5`00KaP00002KaP00`00KaP00002KaP00`00KaP00002KaT01@00JaT006/I
00000f/I00@006/I001[6P800007IaX00000IaX006LJ00000fLJ00H006LJ000006<K000EOol00`00
Oomoo`03Ool00005Ool007ooOol00008Ool01000Oomoo`000Woo00<007oo00000goo00<007ooOol0
7Goo0`0027oo00@006KHI]P0009jdP07001jlP00N_8007/C00000W/C00L007Kd001g5000MaD00002
MaD01`00MaD007LF001g5P00009g5P07001c5P00LaL007<G00000W<G00X007<GLaL006lH001_6000
KaP0009_6003001_6000009_6@03001_6@00009[6@800006JaT006/I001[6P000V/J00d006LJIaX0
06LJ001W6P00IaX006LJ001W6P02000FOol00`00Oomoo`03Ool00005Ool007ooOol00008Ool01000
Oomoo`000Woo00<007oo00000goo0P006goo0`002goo10000W[B0P000W[b00<007/C00000W/C0P00
0WLD00D007LE001g5@00009g5@05001g5P00MaH00002MaH2000017<G001c5g<G0P0000Qc5`00LaL0
07<G001_60000VlH00<006lH00000VlH00@006lI001_6@800003JaT006/I00800004JaT006/I0002
JaX2000016LJ001W6P000VLJ0P0000AW6P00IaX0009oo`03001oogoo01=oo`03001oogoo00=oo`00
0Woo0P002Woo0P0017oo0P000goo00<007ooOol05goo0`004Goo00D007[B001jlP00009jlP06001k
4`00Na<007/D0P0000Yg5000MaD007LE001g5@00MaD0009g5P09001g5P00MaH007<G001c5`00009c
5`03001c5`00009c5`80009_600:001_6000001_6000KaP006lI0002KaT00`00JaT00002JaT01`00
JaU[6@00JaX006/J00800003IaX006LJ00800005IaX00000IaX00004Ool00`00Oomoo`0BOol00`00
Oomoo`03Ool001Qoo`03001oogoo01Aoo`@001Aoo`<00006N_8007[b001k4`000W/C0P000W/D0`00
00Ig5@00MaEg5@00MaD300001GLF001g5P00McH00P0000Ac5`00LaMc5`80009c5`0:001c5`00001_
6000KaP006lh0002KaP20003KaT20002JaT200001V/I001[6@00JaX0009[6P07001W6VLJ001W6P00
IaX00P001Goo00<007ooOol04goo00<007ooOol00Woo000HOol00`00Oomoo`0AOol3000IOol30002
N_800`00Na<00002Na<20002Na@02@00Nc@007LE001g5@00MaD00002McH01000MaH000000WLf00<0
07<f00000W<G00D007<g000007<G0080009c>003001_6000009_>004001_6000KcP200001VlI001_
6@00JaT0009[6@04001[6@00JaT200002F/J000006LJ001W6P00IaX00008Ool00`00Oomoo`0BOol0
0`00Oomoo`02Ool001Qoo`03001oogoo00ioo`<001aoo`03000V?@00009jlP09001k4`00Na<007/D
001k5000009k=080009g5@03001g5@00009g=P800004MaH007Lf0002LcH01@00LaL007<g00000W<g
00L007<G001c>000LcP00002KcP01@00KcP006lh00000Vlh00@006lI001_>@800006JaT006/i001[
6@000V/I00P006/JJaX006LJ001W6P002Woo00<007ooOol04Goo00<007ooOol00Woo000HOol00`00
Oomoo`0;Ool3000POol40002Na<01000Na<000000W/D00@007/d001k=@800003MaD00000009g=P04
001g=P000002McH01000LcH007<g0P0000Ac=`00LcMc=`80000;LcP007<hLcP006lh001_>000KcP0
0002KcP01@00KcT006li00000Vli00D006/i001[>@00009[6@800006JaX006/J001W6P0037oo00<0
07ooOol047oo00<007ooOol00Woo000HOol20008Ool4000UOol40002Na<20002Na@01000Nc@00000
0W/e00P007LE001g=P00McH007Lf0P0000=g=P0000000W<g00<007<g00000W<g00@007<g0000009c
>005001c>000KcP00002KcP01`00KcP006li001_>@00009_>@03001_>@00009[>@04001[6@00JcT2
000016/J001[6P003Woo00<007ooOol03goo00<007ooOol00Woo000HOol00`00Oomoo`04Ool3000Y
Ool00`00:Ud00003000017/D001k50000W/d0P000W/e0`0000=g=P00McH00P000WLf00D007<f001c
=`00009c=`09001c=`00LcL007<g001c>000009c>0<00006KcP006lh001_>@000Vli00@006li001_
>@800007JcT006/i001[>@00JcX00P0000=[6P00Ool03goo00<007ooOol03Woo00<007ooOol00Woo
000GOol00`00Oomoo`02Ool3000]Ool60002Na@03`00Nc@007/e001k=@00NcD007Le001g=P00McH0
0002McH300002g<g001c=`00LcL007<g001c=`00LcL00P0000Ic>000LcP006lh0002KcP300001fli
001_>@00KcT006/i00800006JcT006/i001[>P000V/j00<007ooOol03goo00<007ooOol03Woo00<0
07ooOol00Woo000GOol00`00Ool00002000aOol200001bYM00000000Na@007/d0080009k=@80009g
=@0=001g=P00McH007Lf001g=P00McH007<g00000W<g0`0000Qc=`00LcL007<g001c>0000Vlh00L0
06lh001_>000KcP00002KcT200002F/i001[>F/i001[>@00JcT00002JcX00`00Oomoo`0AOol00`00
Oomoo`0>Ool00`00Oomoo`01Ool001Moo`8003Ioo`H0000:Nc@00000NcD007/e001g=@00McD30000
17Lf001g=P000WLf00/007<g001c=`00LcL007<g001c=`00009c=`80009_>006001_>000KcP006lh
0P0000=_>@00KcT00P000f/i0P000V/i00<006/j00005Goo00<007ooOol03Goo00<007ooOol00Goo
000IOol00`00Oomoo`0cOol00`00;Wh0000400000g/e001k=@020002McD04@00McD007Lf001g=P00
McH007Lf001g=P00LcL007<g00000W<g00T007<g001c=`00LcP006lh00000Vlh0P0000Q_>000KcQ_
>000KcT006li0P0000Q[>@00JcT006/i001[>@005goo00<007ooOol037oo00<007ooOol00Goo000J
Ool2000dOol400001Bin00000000NcD00P0000Ag=@00McD0009g=P<00003McH00000009g=P<0000;
LcL007<g001c=`00LcL007<h001c>00200000flh001_>0020003KcP200000fli001_>@0200001V/i
001[>@00JcT001Uoo`03001oogoo00]oo`03001oogoo005oo`006goo0P00=7oo200001Ik=@00McD0
07Le001g=P00McH007Lf001g=P00McH007Lg001g=`00LcL0009c=`<00009LcP007<h001c>000KcP0
06lh00800003KcP006lh00800009KcT006liKcT006/i001[>@0001Yoo`03001oogoo00]oo`03001o
ogoo005oo`007Goo0P00=7oo0P0000<bWP0000000`0000Eg=@00McD007Lf0080009g=P80009g=P80
009g=`0;001c=`00LcL007<g001c=`00LcP00002LcP2000026lh001_>6lh001_>000KcT20002KcT0
0`00KcT00002JcT00`00Oomoo`0IOol00`00Oomoo`0:Ool00`00Oomoo`01Ool001moo`03001oogoo
035oo`03000bWP0000P00007McH007Lf001g=P00McH00P0000Ig=P00001g=`00McL200000g<g0000
0002LcL20002LcP03`00LcP006lh001_>000KcP006li001_>@00KcT00002KcT00`00JcT0000MOol0
0`00Oomoo`09Ool00`00Oomoo`01Ool001moo`8003=oo`X00005McH007Lf001g=P020002McH01000
McL000000WLg01P007<g001c=`00LcL007<g001c>000LcP007<h001_>000KcP006lh001_>@00KcT2
000016li001_>@007goo00<007ooOol02Goo0@000Goo0Goo000QOol3000aOol600000cJn00000003
000017Lf001g=P000WLf0P0000Ag=`00McL0009c=`800008LcL007<g001c=`00LcP0009c>0800004
KcP006lhKcP20002KcT01@00KcT006li00008Goo00<007ooOol027oo0@000Goo0Goo000SOol00`00
Oomoo`0`Ool;00001gLf001g=P00McH007Lf00<00005McL007<g001c=`02000017<g001c=g<g0`00
00=c>000LcP00`0000Y_>000KcP006li001_>@00KcT0029oo`03001oogoo00Qoo`40005oo`5oo`00
97oo0P00<Woo300000eg=P00McH007Lf001g=`00LcL007<g001c=`03000057<g001c>000LcP007<h
001c>000LcP006lh001_>000KcT006li000TOol00`00Oomoo`07Ool10001Ool1Ool002Ioo`80035o
o`800003>[h0000000H00005McH00000McH00002McL200002W<g001c=`00LcL007<g001c>0000W<h
0P0000Ac>000LcQc>080009_>0800003KcT007oo02Eoo`03001oogoo00Ioo`40005oo`5oo`009goo
0P00<7oo00<007oo00002@0000Mg=P00McH007Lg001g=`0300000g<g00000002LcL01P00LcP007<h
001c>080009c>006001c>000KcP006lh0P00:7oo00<007ooOol01Goo0@000Goo0Goo000YOol00`00
Oomoo`0]Ool00`00Oomoo`09Ool200007gLf000007Lg001g=`00McL007<g001c=`00LcL007<h001c
>000LcP007<h001c>000KcP006lh001_>00002Uoo`03001oogoo00Aoo`40005oo`5oo`00:Woo0P00
;Goo00<007ooOol02goo00@007Lg001g=`80009g=`800004LcL007<gLcL2000027<h001c>000LcP0
07<h0002KcP200000flh001oo`0YOol00`00Oomoo`05Ool10001Ool002aoo`03001oogoo02Yoo`03
001oogoo00aoo`800006McL00000McL007Lg0P000W<g00<007<g00000P0000=c>000LcP00`0000=_
>000KcP00P00;7oo00<007ooOol017oo0@000Goo000]Ool2000ZOol00`00Oomoo`0>Ool01`00McL0
0000McL007<g0080000ALcL00000LcL007<h001c>000LcP007<h001_>000KcP0000]Ool00`00Oomo
o`03Ool10001Ool001ioo`8000Aoo`8000=oo`@00003Ool0000002Uoo`03001oogoo00moo`04001g
=`00McL20002LcL01000LcL000000W<h0P0000Qc>000LcP007<h001_>000;goo00<007ooOol00Woo
0@000Goo000MOol01000Oomoo`000goo0P0017oo00<007ooOol00goo00<007ooOol09Woo00<007oo
Ool047oo0P0000=g=`0000000W<g0P0000Ec=`00LcP007<h00800006LcP007<h001c>000<Goo00D0
07ooOomoo`00005oo`007Goo00@007ooOol000Yoo`03001oogoo00=oo`8002Ioo`03001oogoo019o
o`06001c=`00LcL007<g0P0000Ec>000LcP007<h00800005LcP007<hLcP0000bOol01000Oomoo`00
0Goo000MOol01000Oomoo`002goo00<007ooOol017oo00<007oo00008goo00<007ooOol04goo00H0
07<g001c=`00LcL3000017<h000007<h0`0000=c>000Ool0<Woo00@007ooOol0005oo`007Goo00@0
07ooOol000Qoo`04001oogoo0007Ool2000SOol00`00Oomoo`0DOol02P00LcL007<g001c=`00LcL0
07<h0P0000Ac>000LcP003Eoo`03001oogoo004000007Woo0P002Woo0P002Goo0P008Woo00<007oo
Ool05Goo0P0000=c=`0000000W<g0P0000=c>00000000W<h00<007ooOol0=7oo0@000Goo0@00000g
Ool2000POol00`00Oomoo`0GOol01P00LcL007<g001c=`800004LcP007<h000hOol2000003Qoo`80
01moo`03001oogoo01Qoo`06001c=`00LcL007<g0P0000=c>000Ool0=goo0`00000jOol2000MOol0
0`00Oomoo`0IOol01P00LcL007<g001c=`8003Aoo`D000=oo`00>goo0P0077oo00<007ooOol06Woo
0P0000Ec=`00001c>000031oo`@000Qoo`00;7oo0P0017oo0P0017oo0`000Woo00<007ooOol06Goo
00<007ooOol077oo00@007<g000002eoo`@000aoo`00:goo00@007ooOol000=oo`8000Eoo`03001o
ogoo009oo`8001Uoo`03001oogoo01eoo`03001c>00002Qoo`D0011oo`00:goo00@007ooOol000Qo
o`@000Eoo`8001Moo`03001oogoo01ioo`03001oogoo02=oo`@001Eoo`00:goo00@007ooOol000Qo
o`03001oo`0000Moo`8001Ioo`03001oogoo041oo`@001Uoo`00:goo00@007ooOol000Qoo`03001o
o`0000Moo`03001oo`0001Eoo`03001oogoo03]oo`D001eoo`00;7oo0P002Woo0P002Woo0P004goo
00<007ooOol0=goo10008Woo0014Ool00`00Ool0000AOol00`00Oomoo`0dOol4000VOol004Moo`03
001oogoo00ioo`03001oogoo02moo`D002Yoo`00Agoo0`003Woo00<007ooOol0:goo1000;goo000i
Ool20004Ool20004Ool20003Ool00`00Oomoo`0;Ool00`00Oomoo`0VOol5000cOol003Qoo`04001o
ogoo0003Ool20003Ool01000Oomoo`000Woo0`002goo00<007ooOol08Woo1000>7oo000hOol01000
Oomoo`0027oo00@007ooOol000Eoo`03001oogoo00Qoo`03001oogoo01ioo`@003aoo`00>7oo00@0
07ooOol000Qoo`<000Ioo`<000Qoo`03001oogoo01Uoo`D0041oo`00>7oo00@007ooOol000Qoo`03
001oogoo00Uoo`03001oogoo00Eoo`03001oogoo01Eoo`@004Eoo`00>Goo0P002Woo0`0027oo0`00
1Goo00<007ooOol04Goo1000BGoo001COol00`00Oomoo`02Ool00`00Oomoo`0<Ool5001=Ool0059o
o`03001oo`0000=oo`03001oogoo00Qoo`@0059oo`00AGoo0P0017oo0P0017oo0P000Woo0P0000=o
o`00Ool01Goo1000EWoo0014Ool01000Oomoo`000goo0P000goo00L007ooOol007oo001oo`020000
0goo00000003001JOol004Aoo`04001oogoo0008Ool01000Oomoo`0017oo0P00Ggoo0014Ool01000
Oomoo`002Goo0P00IWoo0014Ool01000Oomoo`0027oo00@007ooOol006Eoo`00AGoo0P002Woo0P00
IWoo002iOol00;Uoo`00EWoo0`00H7oo001GOol00`00Oomoo`1OOol005Moo`03001oogoo05moo`00
Egoo00<007ooOol0Ggoo001GOol00`00Oomoo`1OOol005Ioo`80065oo`00^Goo002iOol00001\
\>"],
  ImageRangeCache->{{{0, 184.688}, {200.5, 0}} -> {-0.0787075, -0.068263, 
  0.00533296, 0.00533296}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["8.4 Up/Down and Out (Knock-Out) Calls", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
As before, we can do numerical and graphical checks against the \
original Rubenstein and Reiner paper. First we introduce the value and price \
sensitivity functions:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Down and Out Calls", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
DownAndOutCall[reb_, p_, k_, h_, sd_, r_, q_, t_] :=
If[k >= h, intone[p, k, sd, r, q, t, 1] - intthree[p, k, h, sd, r, q, t, 1, \
1] + intsix[reb, p, h, sd, r, q, t, 1],
inttwo[p, k, h, sd, r, q, t, 1] - intfour[p, k, h, sd, r, q, t, 1, 1] + 
intsix[reb, p, h, sd, r, q, t, 1]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndOutCallDelta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, 1] - intthree[p, k, h, sd, r, \
q, t, 1, 1] + intsix[reb, p, h, sd, r, q, t, 1], p]],
Evaluate[D[inttwo[p, k, h, sd, r, q, t, 1] - intfour[p, k, h, sd, r, q, t, 1, \
1] + intsix[reb, p, h, sd, r, q, t, 1], p]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndOutCallGamma[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, 1] - intthree[p, k, h, sd, r, \
q, t, 1, 1] + intsix[reb, p, h, sd, r, q, t, 1], {p,2}]],
Evaluate[D[inttwo[p, k, h, sd, r, q, t, 1] - intfour[p, k, h, sd, r, q, t, 1, \
1] + intsix[reb, p, h, sd, r, q, t, 1], {p,2}]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndOutCallTheta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[-D[intone[p, k, sd, r, q, t, 1] - intthree[p, k, h, sd, \
r, q, t, 1, 1] + intsix[reb, p, h, sd, r, q, t, 1], t]],
Evaluate[-D[inttwo[p, k, h, sd, r, q, t, 1] - intfour[p, k, h, sd, r, q, t, \
1, 1] + intsix[reb, p, h, sd, r, q, t, 1], t]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndOutCallVega[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, 1] - intthree[p, k, h, sd, r, \
q, t, 1, 1] + intsix[reb, p, h, sd, r, q, t, 1], sd]],
Evaluate[D[inttwo[p, k, h, sd, r, q, t, 1] - intfour[p, k, h, sd, r, q, t, 1, \
1] + intsix[reb, p, h, sd, r, q, t, 1], sd]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndOutCallRho[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, 1] - intthree[p, k, h, sd, r, \
q, t, 1, 1] + intsix[reb, p, h, sd, r, q, t, 1], r]],
Evaluate[D[inttwo[p, k, h, sd, r, q, t, 1] - intfour[p, k, h, sd, r, q, t, 1, \
1] + intsix[reb, p, h, sd, r, q, t, 1], r]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9]
}, Open  ]],

Cell[CellGroupData[{

Cell["Up and Out Calls", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
UpAndOutCall[reb_, p_, k_, h_, sd_, r_, q_, t_] :=
If[k >= h, intsix[reb, p, h, sd, r, q, t, -1],
intone[p, k, sd, r, q, t, 1]- inttwo[p, k, h, sd, r, q, t, 1] +
intthree[p, k, h, sd, r, q, t, 1, -1] - intfour[p, k, h, sd, r, q, t, 1, -1] \
+ intsix[reb, p, h, sd, r, q, t, -1]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndOutCallDelta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intsix[reb, p, h, sd, r, q, t, -1], p]],
Evaluate[D[intone[p, k, sd, r, q, t, 1]- inttwo[p, k, h, sd, r, q, t, 1] + \
intthree[p, k, h, sd, r, q, t, 1, -1] - intfour[p, k, h, sd, r, q, t, 1, -1] \
+ intsix[reb, p, h, sd, r, q, t, -1], p]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndOutCallGamma[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intsix[reb, p, h, sd, r, q, t, -1], {p,2}]],
Evaluate[D[intone[p, k, sd, r, q, t, 1]- inttwo[p, k, h, sd, r, q, t, 1] + \
intthree[p, k, h, sd, r, q, t, 1, -1] - intfour[p, k, h, sd, r, q, t, 1, -1] \
+ intsix[reb, p, h, sd, r, q, t, -1], {p,2}]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  FontSize->9],

Cell["\<\
UpAndOutCallTheta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[-D[intsix[reb, p, h, sd, r, q, t, -1], t]],
Evaluate[-D[intone[p, k, sd, r, q, t, 1]- inttwo[p, k, h, sd, r, q, t, 1] + \
intthree[p, k, h, sd, r, q, t, 1, -1] - intfour[p, k, h, sd, r, q, t, 1, -1] \
+ intsix[reb, p, h, sd, r, q, t, -1], t]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  FontSize->9],

Cell["\<\
UpAndOutCallVega[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intsix[reb, p, h, sd, r, q, t, -1], sd]],
Evaluate[D[intone[p, k, sd, r, q, t, 1]- inttwo[p, k, h, sd, r, q, t, 1] + \
intthree[p, k, h, sd, r, q, t, 1, -1] - intfour[p, k, h, sd, r, q, t, 1, -1] \
+  intsix[reb, p, h, sd, r, q, t, -1], sd]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndOutCallRho[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intsix[reb, p, h, sd, r, q, t, -1], r]],
Evaluate[D[intone[p, k, sd, r, q, t, 1]- inttwo[p, k, h, sd, r, q, t, 1] + \
intthree[p, k, h, sd, r, q, t, 1, -1] - intfour[p, k, h, sd, r, q, t, 1, -1] \
+ intsix[reb, p, h, sd, r, q, t, -1], r]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
The following tables of values can be checked directly against a \
further sample table of Rubenstein and Reiner. Note again the conversion to \
annual rates for the interest rates and yield.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
TableForm[Table[N[DownAndOutCall[2,100,k,97,0.20,Log[1.1],Log[1.05],\
t]], {k, 90, 110, 10}, {t, 0.5, 1.5, 0.25}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"6.47883622469529818`", "6.44667324238167438`", 
            "6.44799605647323303`", "6.45987430435200415`", 
            "6.47463666966280193`"},
          {"4.69315418821607543`", "4.98215813821260056`", 
            "5.18127796976086685`", "5.3320011003615928`", 
            "5.4519225825047588`"},
          {"3.15669365584714078`", "3.65604266384709619`", 
            "4.00422457927179209`", "4.26753508750863552`", 
            "4.47665618717198476`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
TableForm[Table[N[UpAndOutCall[2,100,k,103,0.20,Log[1.1],Log[1.05],\
t]], {k, 90, 110, 10}, {t, 0.5, 1.5, 0.25}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"1.94252869793152491`", "1.88736751573220082`", 
            "1.87128459223356902`", "1.86688004076394982`", 
            "1.86670993063602601`"},
          {"1.69196720414899948`", "1.74791439819888472`", 
            "1.78082888412303308`", "1.80286548609464176`", 
            "1.81878317936380433`"},
          {"1.68859744543363765`", "1.74611038906702509`", 
            "1.77968186264405137`", "1.80206356571802146`", 
            "1.81818768798297263`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell["\<\
We can also plot the time-price plots of the value and Delta and \
Gamma:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[If[S>95,DownAndOutCall[2,S,100,95,0.2,Log[1.1],Log[1.05],t],
UpAndOutCall[2,S,100,95,0.2,Log[1.1],Log[1.05],t]], {S, 80, 120}, {t, 0.0001, \
1}, BoxRatios -> {1, 1, 1}, PlotPoints -> 40];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.0855 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0567234 1.16608 1.11022e-16 1.16608 [
[.07825 .23514 -11.8839 -9 ]
[.07825 .23514 .11606 0 ]
[.21247 .17844 -11.3335 -9 ]
[.21247 .17844 .66648 0 ]
[.3545 .11847 -16.1747 -9 ]
[.3545 .11847 1.82534 0 ]
[.50502 .05496 -15.349 -9 ]
[.50502 .05496 2.65097 0 ]
[.66483 -0.01242 -14.5234 -9 ]
[.66483 -0.01242 3.4766 0 ]
[.09301 .88495 -10.1462 0 ]
[.09301 .88495 7.85383 9 ]
[.17991 .94587 -9.90019 0 ]
[.17991 .94587 8.09981 9 ]
[.25944 1.00163 -9.68252 0 ]
[.25944 1.00163 8.31748 9 ]
[.33251 1.05285 -9.48854 0 ]
[.33251 1.05285 8.51146 9 ]
[.39987 1.10006 -3.10486 0 ]
[.39987 1.10006 2.89514 9 ]
[.07325 .26228 -6 -2.6177 ]
[.07325 .26228 0 6.3823 ]
[.04204 .45766 -12 -2.7917 ]
[.04204 .45766 0 6.2083 ]
[.00609 .6827 -12 -2.99498 ]
[.00609 .6827 0 6.00502 ]
[ 0 0 0 0 ]
[ 1 1.0855 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
[(80)] .07825 .23514 .98066 1 Mshowa
.22216 .18933 m
.227 .19478 L
s
[(90)] .21247 .17844 .88892 1 Mshowa
.36358 .12987 m
.36813 .13557 L
s
[(100)] .3545 .11847 .79718 1 Mshowa
.51343 .06687 m
.51763 .07282 L
s
[(110)] .50502 .05496 .70545 1 Mshowa
.67245 0 m
.67627 .00621 L
s
[(120)] .66483 -0.01242 .61371 1 Mshowa
.125 Mabswid
.11461 .23456 m
.11764 .23771 L
s
.14105 .22344 m
.14405 .22662 L
s
.16779 .2122 m
.17076 .21541 L
s
.19482 .20083 m
.19776 .20407 L
s
.2498 .17771 m
.25268 .18101 L
s
.27776 .16595 m
.2806 .16928 L
s
.30604 .15406 m
.30885 .15742 L
s
.33465 .14204 m
.33741 .14542 L
s
.39286 .11756 m
.39554 .12101 L
s
.42247 .10511 m
.42512 .10859 L
s
.45243 .09251 m
.45504 .09602 L
s
.48275 .07977 m
.48531 .08331 L
s
.54447 .05381 m
.54695 .05742 L
s
.57589 .0406 m
.57832 .04424 L
s
.60769 .02723 m
.61007 .0309 L
s
.63987 .0137 m
.64221 .0174 L
s
.25 Mabswid
0 .80374 m
.40037 1.0855 L
s
.09485 .87049 m
.09577 .86326 L
s
[(0.2)] .09301 .88495 .12735 -1 Mshowa
.18136 .93137 m
.18209 .92412 L
s
[(0.4)] .17991 .94587 .10002 -1 Mshowa
.26055 .98709 m
.2611 .97983 L
s
[(0.6)] .25944 1.00163 .07584 -1 Mshowa
.3333 1.03829 m
.33369 1.03102 L
s
[(0.8)] .33251 1.05285 .05428 -1 Mshowa
.40037 1.0855 m
.40063 1.07821 L
s
[(1)] .39987 1.10006 .03495 -1 Mshowa
.125 Mabswid
.02453 .821 m
.02518 .81668 L
s
.04852 .83789 m
.04914 .83356 L
s
.07196 .85438 m
.07254 .85004 L
s
.11722 .88623 m
.11774 .88189 L
s
.13908 .90162 m
.13958 .89727 L
s
.16046 .91666 m
.16092 .91231 L
s
.20181 .94576 m
.20221 .9414 L
s
.22181 .95983 m
.22219 .95548 L
s
.24138 .97361 m
.24174 .96925 L
s
.27931 1.0003 m
.27961 .99593 L
s
.29768 1.01322 m
.29796 1.00886 L
s
.31567 1.02589 m
.31593 1.02152 L
s
.35057 1.05045 m
.35079 1.04608 L
s
.3675 1.06236 m
.3677 1.058 L
s
.3841 1.07404 m
.38427 1.06967 L
s
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
.0867 .25665 m
.09342 .25384 L
s
[(0)] .07325 .26228 1 -0.41829 Mshowa
.05566 .45249 m
.06248 .4499 L
s
[(10)] .04204 .45766 1 -0.37962 Mshowa
.01991 .67808 m
.02683 .67577 L
s
[(20)] .00609 .6827 1 -0.33445 Mshowa
.125 Mabswid
.08082 .29373 m
.08487 .29206 L
s
.07479 .3318 m
.07884 .33017 L
s
.06859 .37093 m
.07265 .36932 L
s
.06222 .41114 m
.06629 .40956 L
s
.04892 .49502 m
.05302 .4935 L
s
.04199 .53879 m
.0461 .5373 L
s
.03485 .58385 m
.03897 .58239 L
s
.02749 .63026 m
.03163 .62884 L
s
.0121 .72737 m
.01626 .72602 L
s
.00405 .77821 m
.00822 .7769 L
s
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
.41799 .58158 m
.08846 .24555 L
s
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.41799 .58158 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
0 0 m
1 0 L
1 1.0855 L
0 1.0855 L
closepath
clip
newpath
.5 Mabswid
.634 .713 .904 r
.41001 .59883 .41713 .6062 .42908 .60357 .422 .59618 Metetra
.633 .713 .904 r
.422 .59618 .42908 .60357 .44109 .60096 .43404 .59355 Metetra
.632 .713 .905 r
.43404 .59355 .44109 .60096 .45317 .59837 .44616 .59094 Metetra
.631 .713 .906 r
.44616 .59094 .45317 .59837 .46532 .5958 .45834 .58835 Metetra
.63 .713 .906 r
.45834 .58835 .46532 .5958 .47754 .59324 .47059 .58577 Metetra
.629 .713 .907 r
.47059 .58577 .47754 .59324 .48982 .59069 .4829 .5832 Metetra
.628 .713 .907 r
.4829 .5832 .48982 .59069 .50218 .58814 .49529 .58063 Metetra
.628 .713 .908 r
.49529 .58063 .50218 .58814 .5146 .58559 .50775 .57807 Metetra
.627 .714 .908 r
.50775 .57807 .5146 .58559 .5271 .58304 .52029 .57549 Metetra
.627 .714 .909 r
.52029 .57549 .5271 .58304 .53967 .58047 .53289 .57291 Metetra
.627 .714 .909 r
.53289 .57291 .53967 .58047 .55231 .57789 .54557 .57031 Metetra
.627 .715 .91 r
.54557 .57031 .55231 .57789 .56502 .57528 .55832 .5677 Metetra
.627 .716 .91 r
.55832 .5677 .56502 .57528 .57781 .57266 .57115 .56506 Metetra
.628 .716 .91 r
.57115 .56506 .57781 .57266 .59068 .57001 .58405 .56239 Metetra
.334 .582 .933 r
.58405 .56239 .59068 .57001 .60377 .57361 .59717 .56593 Metetra
0 .041 .567 r
.59717 .56593 .60377 .57361 .6173 .58754 .6107 .57972 Metetra
0 .068 .595 r
.6107 .57972 .6173 .58754 .63104 .60133 .62445 .59338 Metetra
0 .092 .619 r
.62445 .59338 .63104 .60133 .645 .61501 .63841 .60694 Metetra
0 .112 .64 r
.63841 .60694 .645 .61501 .65918 .62864 .65259 .62043 Metetra
0 .13 .658 r
.65259 .62043 .65918 .62864 .67358 .64224 .667 .63391 Metetra
0 .146 .673 r
.667 .63391 .67358 .64224 .68821 .65585 .68164 .6474 Metetra
0 .159 .685 r
.68164 .6474 .68821 .65585 .70307 .6695 .69651 .66094 Metetra
0 .17 .695 r
.69651 .66094 .70307 .6695 .71817 .68323 .71161 .67456 Metetra
0 .18 .704 r
.71161 .67456 .71817 .68323 .73351 .69707 .72696 .68829 Metetra
0 .187 .711 r
.72696 .68829 .73351 .69707 .7491 .71104 .74256 .70216 Metetra
0 .193 .716 r
.74256 .70216 .7491 .71104 .76494 .72517 .75842 .71619 Metetra
0 .198 .721 r
.75842 .71619 .76494 .72517 .78105 .73948 .77454 .73042 Metetra
0 .202 .724 r
.77454 .73042 .78105 .73948 .79743 .754 .79093 .74487 Metetra
0 .204 .727 r
.79093 .74487 .79743 .754 .81408 .76876 .8076 .75955 Metetra
0 .206 .728 r
.8076 .75955 .81408 .76876 .83102 .78377 .82456 .7745 Metetra
0 .207 .729 r
.82456 .7745 .83102 .78377 .84826 .79905 .84182 .78972 Metetra
0 .208 .73 r
.84182 .78972 .84826 .79905 .8658 .81462 .85938 .80524 Metetra
0 .207 .73 r
.85938 .80524 .8658 .81462 .88366 .8305 .87726 .82108 Metetra
0 .207 .73 r
.87726 .82108 .88366 .8305 .90184 .84671 .89547 .83725 Metetra
0 .206 .729 r
.89547 .83725 .90184 .84671 .92035 .86326 .91401 .85377 Metetra
0 .204 .728 r
.91401 .85377 .92035 .86326 .93921 .88017 .9329 .87066 Metetra
0 .203 .727 r
.9329 .87066 .93921 .88017 .95842 .89745 .95214 .88792 Metetra
0 .201 .725 r
.95214 .88792 .95842 .89745 .978 .91513 .97176 .90559 Metetra
0 .199 .724 r
.97176 .90559 .978 .91513 .99796 .9332 .99176 .92366 Metetra
.634 .713 .904 r
.40284 .5914 .41001 .59883 .422 .59618 .41485 .58871 Metetra
.633 .713 .904 r
.41485 .58871 .422 .59618 .43404 .59355 .42693 .58606 Metetra
.631 .712 .905 r
.42693 .58606 .43404 .59355 .44616 .59094 .43908 .58343 Metetra
.63 .712 .906 r
.43908 .58343 .44616 .59094 .45834 .58835 .45129 .58082 Metetra
.629 .712 .906 r
.45129 .58082 .45834 .58835 .47059 .58577 .46357 .57822 Metetra
.628 .712 .907 r
.46357 .57822 .47059 .58577 .4829 .5832 .47592 .57563 Metetra
.628 .713 .907 r
.47592 .57563 .4829 .5832 .49529 .58063 .48835 .57305 Metetra
.627 .713 .908 r
.48835 .57305 .49529 .58063 .50775 .57807 .50084 .57047 Metetra
.627 .713 .909 r
.50084 .57047 .50775 .57807 .52029 .57549 .51341 .56788 Metetra
.627 .714 .909 r
.51341 .56788 .52029 .57549 .53289 .57291 .52605 .56528 Metetra
.626 .714 .909 r
.52605 .56528 .53289 .57291 .54557 .57031 .53877 .56267 Metetra
.626 .715 .91 r
.53877 .56267 .54557 .57031 .55832 .5677 .55156 .56004 Metetra
.627 .715 .91 r
.55156 .56004 .55832 .5677 .57115 .56506 .56442 .55738 Metetra
.627 .716 .91 r
.56442 .55738 .57115 .56506 .58405 .56239 .57736 .5547 Metetra
.337 .583 .934 r
.57736 .5547 .58405 .56239 .59717 .56593 .59051 .55818 Metetra
0 .049 .574 r
.59051 .55818 .59717 .56593 .6107 .57972 .60404 .57183 Metetra
0 .075 .602 r
.60404 .57183 .6107 .57972 .62445 .59338 .61779 .58535 Metetra
0 .099 .626 r
.61779 .58535 .62445 .59338 .63841 .60694 .63176 .59877 Metetra
0 .12 .647 r
.63176 .59877 .63841 .60694 .65259 .62043 .64595 .61214 Metetra
0 .137 .664 r
.64595 .61214 .65259 .62043 .667 .63391 .66036 .62548 Metetra
0 .153 .679 r
.66036 .62548 .667 .63391 .68164 .6474 .67501 .63885 Metetra
0 .166 .691 r
.67501 .63885 .68164 .6474 .69651 .66094 .68988 .65227 Metetra
0 .176 .701 r
.68988 .65227 .69651 .66094 .71161 .67456 .705 .66577 Metetra
0 .185 .709 r
.705 .66577 .71161 .67456 .72696 .68829 .72036 .67939 Metetra
0 .193 .715 r
.72036 .67939 .72696 .68829 .74256 .70216 .73597 .69316 Metetra
0 .198 .721 r
.73597 .69316 .74256 .70216 .75842 .71619 .75184 .7071 Metetra
0 .203 .725 r
.75184 .7071 .75842 .71619 .77454 .73042 .76797 .72124 Metetra
0 .206 .728 r
.76797 .72124 .77454 .73042 .79093 .74487 .78438 .73561 Metetra
0 .208 .73 r
.78438 .73561 .79093 .74487 .8076 .75955 .80106 .75022 Metetra
0 .21 .731 r
.80106 .75022 .8076 .75955 .82456 .7745 .81804 .76509 Metetra
0 .21 .732 r
.81804 .76509 .82456 .7745 .84182 .78972 .83532 .78025 Metetra
0 .21 .732 r
.83532 .78025 .84182 .78972 .85938 .80524 .8529 .79572 Metetra
0 .21 .732 r
.8529 .79572 .85938 .80524 .87726 .82108 .8708 .81152 Metetra
0 .209 .731 r
.8708 .81152 .87726 .82108 .89547 .83725 .88903 .82765 Metetra
0 .208 .731 r
.88903 .82765 .89547 .83725 .91401 .85377 .9076 .84414 Metetra
0 .206 .73 r
.9076 .84414 .91401 .85377 .9329 .87066 .92652 .861 Metetra
0 .204 .728 r
.92652 .861 .9329 .87066 .95214 .88792 .9458 .87826 Metetra
0 .202 .727 r
.9458 .87826 .95214 .88792 .97176 .90559 .96546 .89591 Metetra
0 .2 .725 r
.96546 .89591 .97176 .90559 .99176 .92366 .98549 .91398 Metetra
.634 .713 .904 r
.3956 .58389 .40284 .5914 .41485 .58871 .40765 .58118 Metetra
.632 .712 .904 r
.40765 .58118 .41485 .58871 .42693 .58606 .41976 .5785 Metetra
.631 .712 .905 r
.41976 .5785 .42693 .58606 .43908 .58343 .43194 .57585 Metetra
.63 .712 .906 r
.43194 .57585 .43908 .58343 .45129 .58082 .44418 .57322 Metetra
.629 .712 .906 r
.44418 .57322 .45129 .58082 .46357 .57822 .4565 .5706 Metetra
.628 .712 .907 r
.4565 .5706 .46357 .57822 .47592 .57563 .46888 .568 Metetra
.627 .712 .908 r
.46888 .568 .47592 .57563 .48835 .57305 .48134 .5654 Metetra
.627 .713 .908 r
.48134 .5654 .48835 .57305 .50084 .57047 .49387 .5628 Metetra
.626 .713 .909 r
.49387 .5628 .50084 .57047 .51341 .56788 .50647 .56019 Metetra
.626 .713 .909 r
.50647 .56019 .51341 .56788 .52605 .56528 .51915 .55758 Metetra
.626 .714 .91 r
.51915 .55758 .52605 .56528 .53877 .56267 .5319 .55495 Metetra
.626 .714 .91 r
.5319 .55495 .53877 .56267 .55156 .56004 .54473 .55231 Metetra
.626 .715 .91 r
.54473 .55231 .55156 .56004 .56442 .55738 .55764 .54964 Metetra
.626 .716 .911 r
.55764 .54964 .56442 .55738 .57736 .5547 .57062 .54694 Metetra
.339 .584 .934 r
.57062 .54694 .57736 .5547 .59051 .55818 .58379 .55035 Metetra
0 .057 .582 r
.58379 .55035 .59051 .55818 .60404 .57183 .59733 .56387 Metetra
0 .083 .61 r
.59733 .56387 .60404 .57183 .61779 .58535 .61108 .57724 Metetra
0 .107 .633 r
.61108 .57724 .61779 .58535 .63176 .59877 .62505 .59052 Metetra
0 .127 .654 r
.62505 .59052 .63176 .59877 .64595 .61214 .63925 .60375 Metetra
0 .145 .671 r
.63925 .60375 .64595 .61214 .66036 .62548 .65367 .61696 Metetra
0 .16 .685 r
.65367 .61696 .66036 .62548 .67501 .63885 .66831 .63019 Metetra
0 .172 .696 r
.66831 .63019 .67501 .63885 .68988 .65227 .6832 .64349 Metetra
0 .183 .706 r
.6832 .64349 .68988 .65227 .705 .66577 .69832 .65687 Metetra
0 .191 .714 r
.69832 .65687 .705 .66577 .72036 .67939 .71369 .67038 Metetra
0 .198 .72 r
.71369 .67038 .72036 .67939 .73597 .69316 .72931 .68405 Metetra
0 .203 .725 r
.72931 .68405 .73597 .69316 .75184 .7071 .74519 .69789 Metetra
0 .207 .728 r
.74519 .69789 .75184 .7071 .76797 .72124 .76134 .71194 Metetra
0 .21 .731 r
.76134 .71194 .76797 .72124 .78438 .73561 .77776 .72622 Metetra
0 .212 .733 r
.77776 .72622 .78438 .73561 .80106 .75022 .79446 .74075 Metetra
0 .213 .734 r
.79446 .74075 .80106 .75022 .81804 .76509 .81145 .75556 Metetra
0 .214 .735 r
.81145 .75556 .81804 .76509 .83532 .78025 .82875 .77066 Metetra
0 .213 .735 r
.82875 .77066 .83532 .78025 .8529 .79572 .84635 .78607 Metetra
0 .213 .734 r
.84635 .78607 .8529 .79572 .8708 .81152 .86428 .80182 Metetra
0 .211 .733 r
.86428 .80182 .8708 .81152 .88903 .82765 .88253 .81791 Metetra
0 .21 .732 r
.88253 .81791 .88903 .82765 .9076 .84414 .90113 .83437 Metetra
0 .208 .731 r
.90113 .83437 .9076 .84414 .92652 .861 .92008 .85121 Metetra
0 .206 .729 r
.92008 .85121 .92652 .861 .9458 .87826 .93939 .86845 Metetra
0 .203 .728 r
.93939 .86845 .9458 .87826 .96546 .89591 .95908 .88609 Metetra
0 .201 .726 r
.95908 .88609 .96546 .89591 .98549 .91398 .97916 .90417 Metetra
.634 .712 .904 r
.3883 .57631 .3956 .58389 .40765 .58118 .40038 .57357 Metetra
.632 .712 .904 r
.40038 .57357 .40765 .58118 .41976 .5785 .41252 .57087 Metetra
.631 .712 .905 r
.41252 .57087 .41976 .5785 .43194 .57585 .42473 .56819 Metetra
.629 .712 .906 r
.42473 .56819 .43194 .57585 .44418 .57322 .43701 .56554 Metetra
.628 .712 .906 r
.43701 .56554 .44418 .57322 .4565 .5706 .44936 .56291 Metetra
.627 .712 .907 r
.44936 .56291 .4565 .5706 .46888 .568 .46178 .56028 Metetra
.627 .712 .908 r
.46178 .56028 .46888 .568 .48134 .5654 .47427 .55767 Metetra
.626 .712 .908 r
.47427 .55767 .48134 .5654 .49387 .5628 .48684 .55505 Metetra
.626 .713 .909 r
.48684 .55505 .49387 .5628 .50647 .56019 .49948 .55243 Metetra
.625 .713 .909 r
.49948 .55243 .50647 .56019 .51915 .55758 .51219 .54981 Metetra
.625 .713 .91 r
.51219 .54981 .51915 .55758 .5319 .55495 .52498 .54717 Metetra
.625 .714 .91 r
.52498 .54717 .5319 .55495 .54473 .55231 .53784 .54451 Metetra
.625 .715 .91 r
.53784 .54451 .54473 .55231 .55764 .54964 .55079 .54182 Metetra
.626 .715 .911 r
.55079 .54182 .55764 .54964 .57062 .54694 .56381 .53911 Metetra
.342 .586 .934 r
.56381 .53911 .57062 .54694 .58379 .55035 .57701 .54246 Metetra
0 .065 .59 r
.57701 .54246 .58379 .55035 .59733 .56387 .59055 .55582 Metetra
0 .092 .617 r
.59055 .55582 .59733 .56387 .61108 .57724 .6043 .56904 Metetra
0 .115 .641 r
.6043 .56904 .61108 .57724 .62505 .59052 .61828 .58218 Metetra
0 .135 .661 r
.61828 .58218 .62505 .59052 .63925 .60375 .63248 .59526 Metetra
0 .152 .677 r
.63248 .59526 .63925 .60375 .65367 .61696 .6469 .60833 Metetra
0 .167 .691 r
.6469 .60833 .65367 .61696 .66831 .63019 .66156 .62143 Metetra
0 .179 .702 r
.66156 .62143 .66831 .63019 .6832 .64349 .67645 .6346 Metetra
0 .189 .711 r
.67645 .6346 .6832 .64349 .69832 .65687 .69158 .64786 Metetra
0 .197 .719 r
.69158 .64786 .69832 .65687 .71369 .67038 .70696 .66126 Metetra
0 .204 .725 r
.70696 .66126 .71369 .67038 .72931 .68405 .72259 .67481 Metetra
0 .208 .729 r
.72259 .67481 .72931 .68405 .74519 .69789 .73848 .68855 Metetra
0 .212 .732 r
.73848 .68855 .74519 .69789 .76134 .71194 .75464 .7025 Metetra
0 .215 .735 r
.75464 .7025 .76134 .71194 .77776 .72622 .77107 .7167 Metetra
0 .216 .736 r
.77107 .7167 .77776 .72622 .79446 .74075 .78779 .73115 Metetra
0 .217 .737 r
.78779 .73115 .79446 .74075 .81145 .75556 .8048 .74589 Metetra
0 .217 .737 r
.8048 .74589 .81145 .75556 .82875 .77066 .82211 .76092 Metetra
0 .216 .737 r
.82211 .76092 .82875 .77066 .84635 .78607 .83974 .77628 Metetra
0 .215 .736 r
.83974 .77628 .84635 .78607 .86428 .80182 .85769 .79198 Metetra
0 .214 .735 r
.85769 .79198 .86428 .80182 .88253 .81791 .87597 .80804 Metetra
0 .212 .734 r
.87597 .80804 .88253 .81791 .90113 .83437 .89459 .82447 Metetra
0 .21 .732 r
.89459 .82447 .90113 .83437 .92008 .85121 .91357 .84128 Metetra
0 .207 .731 r
.91357 .84128 .92008 .85121 .93939 .86845 .93292 .8585 Metetra
0 .205 .729 r
.93292 .8585 .93939 .86845 .95908 .88609 .95264 .87614 Metetra
0 .202 .727 r
.95264 .87614 .95908 .88609 .97916 .90417 .97276 .89421 Metetra
.634 .712 .903 r
.38094 .56865 .3883 .57631 .40038 .57357 .39305 .56589 Metetra
.632 .712 .904 r
.39305 .56589 .40038 .57357 .41252 .57087 .40522 .56316 Metetra
.63 .711 .905 r
.40522 .56316 .41252 .57087 .42473 .56819 .41747 .56046 Metetra
.629 .711 .906 r
.41747 .56046 .42473 .56819 .43701 .56554 .42978 .55779 Metetra
.628 .711 .906 r
.42978 .55779 .43701 .56554 .44936 .56291 .44216 .55513 Metetra
.627 .711 .907 r
.44216 .55513 .44936 .56291 .46178 .56028 .45461 .55249 Metetra
.626 .712 .908 r
.45461 .55249 .46178 .56028 .47427 .55767 .46714 .54986 Metetra
.625 .712 .908 r
.46714 .54986 .47427 .55767 .48684 .55505 .47974 .54723 Metetra
.625 .712 .909 r
.47974 .54723 .48684 .55505 .49948 .55243 .49241 .5446 Metetra
.625 .713 .909 r
.49241 .5446 .49948 .55243 .51219 .54981 .50516 .54196 Metetra
.624 .713 .91 r
.50516 .54196 .51219 .54981 .52498 .54717 .51799 .5393 Metetra
.624 .714 .91 r
.51799 .5393 .52498 .54717 .53784 .54451 .53089 .53663 Metetra
.625 .714 .911 r
.53089 .53663 .53784 .54451 .55079 .54182 .54388 .53393 Metetra
.625 .715 .911 r
.54388 .53393 .55079 .54182 .56381 .53911 .55694 .53121 Metetra
.345 .587 .934 r
.55694 .53121 .56381 .53911 .57701 .54246 .57016 .53448 Metetra
0 .074 .598 r
.57016 .53448 .57701 .54246 .59055 .55582 .5837 .54769 Metetra
0 .1 .625 r
.5837 .54769 .59055 .55582 .6043 .56904 .59746 .56076 Metetra
0 .123 .648 r
.59746 .56076 .6043 .56904 .61828 .58218 .61145 .57374 Metetra
0 .143 .668 r
.61145 .57374 .61828 .58218 .63248 .59526 .62565 .58667 Metetra
0 .16 .684 r
.62565 .58667 .63248 .59526 .6469 .60833 .64008 .5996 Metetra
0 .174 .697 r
.64008 .5996 .6469 .60833 .66156 .62143 .65474 .61256 Metetra
0 .186 .708 r
.65474 .61256 .66156 .62143 .67645 .6346 .66964 .6256 Metetra
0 .196 .717 r
.66964 .6256 .67645 .6346 .69158 .64786 .68478 .63873 Metetra
0 .203 .724 r
.68478 .63873 .69158 .64786 .70696 .66126 .70016 .65201 Metetra
0 .209 .729 r
.70016 .65201 .70696 .66126 .72259 .67481 .7158 .66545 Metetra
0 .214 .733 r
.7158 .66545 .72259 .67481 .73848 .68855 .7317 .67908 Metetra
0 .217 .736 r
.7317 .67908 .73848 .68855 .75464 .7025 .74787 .69294 Metetra
0 .219 .738 r
.74787 .69294 .75464 .7025 .77107 .7167 .76432 .70704 Metetra
0 .22 .739 r
.76432 .70704 .77107 .7167 .78779 .73115 .78105 .72141 Metetra
0 .22 .74 r
.78105 .72141 .78779 .73115 .8048 .74589 .79808 .73607 Metetra
0 .22 .74 r
.79808 .73607 .8048 .74589 .82211 .76092 .81541 .75105 Metetra
0 .219 .739 r
.81541 .75105 .82211 .76092 .83974 .77628 .83306 .76635 Metetra
0 .218 .738 r
.83306 .76635 .83974 .77628 .85769 .79198 .85103 .782 Metetra
0 .216 .737 r
.85103 .782 .85769 .79198 .87597 .80804 .86933 .79802 Metetra
0 .214 .736 r
.86933 .79802 .87597 .80804 .89459 .82447 .88798 .81441 Metetra
0 .211 .734 r
.88798 .81441 .89459 .82447 .91357 .84128 .90699 .83121 Metetra
0 .209 .732 r
.90699 .83121 .91357 .84128 .93292 .8585 .92637 .84841 Metetra
0 .206 .73 r
.92637 .84841 .93292 .8585 .95264 .87614 .94613 .86604 Metetra
0 .203 .728 r
.94613 .86604 .95264 .87614 .97276 .89421 .96628 .88411 Metetra
.633 .712 .903 r
.37352 .56092 .38094 .56865 .39305 .56589 .38566 .55813 Metetra
.632 .711 .904 r
.38566 .55813 .39305 .56589 .40522 .56316 .39786 .55538 Metetra
.63 .711 .905 r
.39786 .55538 .40522 .56316 .41747 .56046 .41014 .55265 Metetra
.629 .711 .906 r
.41014 .55265 .41747 .56046 .42978 .55779 .42248 .54996 Metetra
.627 .711 .906 r
.42248 .54996 .42978 .55779 .44216 .55513 .4349 .54729 Metetra
.626 .711 .907 r
.4349 .54729 .44216 .55513 .45461 .55249 .44738 .54463 Metetra
.625 .711 .908 r
.44738 .54463 .45461 .55249 .46714 .54986 .45994 .54198 Metetra
.625 .711 .908 r
.45994 .54198 .46714 .54986 .47974 .54723 .47258 .53933 Metetra
.624 .712 .909 r
.47258 .53933 .47974 .54723 .49241 .5446 .48529 .53669 Metetra
.624 .712 .909 r
.48529 .53669 .49241 .5446 .50516 .54196 .49808 .53403 Metetra
.624 .713 .91 r
.49808 .53403 .50516 .54196 .51799 .5393 .51094 .53137 Metetra
.624 .713 .91 r
.51094 .53137 .51799 .5393 .53089 .53663 .52388 .52868 Metetra
.624 .714 .911 r
.52388 .52868 .53089 .53663 .54388 .53393 .5369 .52597 Metetra
.624 .715 .911 r
.5369 .52597 .54388 .53393 .55694 .53121 .55 .52323 Metetra
.348 .589 .935 r
.55 .52323 .55694 .53121 .57016 .53448 .56325 .52644 Metetra
0 .083 .607 r
.56325 .52644 .57016 .53448 .5837 .54769 .5768 .53948 Metetra
0 .109 .634 r
.5768 .53948 .5837 .54769 .59746 .56076 .59056 .55239 Metetra
0 .132 .656 r
.59056 .55239 .59746 .56076 .61145 .57374 .60455 .56521 Metetra
0 .151 .675 r
.60455 .56521 .61145 .57374 .62565 .58667 .61876 .57799 Metetra
0 .168 .691 r
.61876 .57799 .62565 .58667 .64008 .5996 .63319 .59077 Metetra
0 .182 .704 r
.63319 .59077 .64008 .5996 .65474 .61256 .64786 .60358 Metetra
0 .193 .714 r
.64786 .60358 .65474 .61256 .66964 .6256 .66276 .61648 Metetra
0 .202 .723 r
.66276 .61648 .66964 .6256 .68478 .63873 .67791 .62948 Metetra
0 .21 .729 r
.67791 .62948 .68478 .63873 .70016 .65201 .6933 .64263 Metetra
0 .215 .734 r
.6933 .64263 .70016 .65201 .7158 .66545 .70895 .65595 Metetra
0 .219 .737 r
.70895 .65595 .7158 .66545 .7317 .67908 .72486 .66948 Metetra
0 .222 .74 r
.72486 .66948 .7317 .67908 .74787 .69294 .74104 .68323 Metetra
0 .223 .742 r
.74104 .68323 .74787 .69294 .76432 .70704 .7575 .69724 Metetra
0 .224 .742 r
.7575 .69724 .76432 .70704 .78105 .72141 .77425 .71153 Metetra
0 .224 .743 r
.77425 .71153 .78105 .72141 .79808 .73607 .79129 .72612 Metetra
0 .223 .742 r
.79129 .72612 .79808 .73607 .81541 .75105 .80864 .74103 Metetra
0 .222 .742 r
.80864 .74103 .81541 .75105 .83306 .76635 .82631 .75627 Metetra
0 .22 .74 r
.82631 .75627 .83306 .76635 .85103 .782 .8443 .77187 Metetra
0 .218 .739 r
.8443 .77187 .85103 .782 .86933 .79802 .86263 .78785 Metetra
0 .216 .737 r
.86263 .78785 .86933 .79802 .88798 .81441 .8813 .80421 Metetra
0 .213 .735 r
.8813 .80421 .88798 .81441 .90699 .83121 .90034 .82098 Metetra
0 .21 .733 r
.90034 .82098 .90699 .83121 .92637 .84841 .91975 .83817 Metetra
0 .207 .731 r
.91975 .83817 .92637 .84841 .94613 .86604 .93955 .85579 Metetra
0 .204 .729 r
.93955 .85579 .94613 .86604 .96628 .88411 .95974 .87386 Metetra
.633 .711 .903 r
.36602 .55311 .37352 .56092 .38566 .55813 .3782 .55029 Metetra
.631 .711 .904 r
.3782 .55029 .38566 .55813 .39786 .55538 .39043 .54751 Metetra
.63 .711 .905 r
.39043 .54751 .39786 .55538 .41014 .55265 .40274 .54477 Metetra
.628 .711 .906 r
.40274 .54477 .41014 .55265 .42248 .54996 .41512 .54205 Metetra
.627 .711 .906 r
.41512 .54205 .42248 .54996 .4349 .54729 .42757 .53936 Metetra
.626 .711 .907 r
.42757 .53936 .4349 .54729 .44738 .54463 .44009 .53668 Metetra
.625 .711 .908 r
.44009 .53668 .44738 .54463 .45994 .54198 .45268 .53402 Metetra
.624 .711 .908 r
.45268 .53402 .45994 .54198 .47258 .53933 .46535 .53136 Metetra
.624 .711 .909 r
.46535 .53136 .47258 .53933 .48529 .53669 .4781 .5287 Metetra
.623 .712 .91 r
.4781 .5287 .48529 .53669 .49808 .53403 .49092 .52603 Metetra
.623 .712 .91 r
.49092 .52603 .49808 .53403 .51094 .53137 .50382 .52335 Metetra
.623 .713 .911 r
.50382 .52335 .51094 .53137 .52388 .52868 .5168 .52065 Metetra
.623 .714 .911 r
.5168 .52065 .52388 .52868 .5369 .52597 .52986 .51793 Metetra
.623 .714 .911 r
.52986 .51793 .5369 .52597 .55 .52323 .543 .51518 Metetra
.351 .591 .935 r
.543 .51518 .55 .52323 .56325 .52644 .55628 .51831 Metetra
0 .092 .615 r
.55628 .51831 .56325 .52644 .5768 .53948 .56983 .53118 Metetra
0 .118 .642 r
.56983 .53118 .5768 .53948 .59056 .55239 .58359 .54392 Metetra
0 .141 .664 r
.58359 .54392 .59056 .55239 .60455 .56521 .59758 .55658 Metetra
0 .16 .683 r
.59758 .55658 .60455 .56521 .61876 .57799 .6118 .5692 Metetra
0 .176 .698 r
.6118 .5692 .61876 .57799 .63319 .59077 .62624 .58182 Metetra
0 .19 .711 r
.62624 .58182 .63319 .59077 .64786 .60358 .64091 .59449 Metetra
0 .201 .72 r
.64091 .59449 .64786 .60358 .66276 .61648 .65582 .60723 Metetra
0 .209 .728 r
.65582 .60723 .66276 .61648 .67791 .62948 .67098 .6201 Metetra
0 .216 .734 r
.67098 .6201 .67791 .62948 .6933 .64263 .68638 .63312 Metetra
0 .221 .739 r
.68638 .63312 .6933 .64263 .70895 .65595 .70204 .64632 Metetra
0 .224 .742 r
.70204 .64632 .70895 .65595 .72486 .66948 .71796 .65974 Metetra
0 .227 .744 r
.71796 .65974 .72486 .66948 .74104 .68323 .73415 .67339 Metetra
0 .228 .745 r
.73415 .67339 .74104 .68323 .7575 .69724 .75062 .6873 Metetra
0 .228 .746 r
.75062 .6873 .7575 .69724 .77425 .71153 .76738 .70151 Metetra
0 .228 .746 r
.76738 .70151 .77425 .71153 .79129 .72612 .78444 .71602 Metetra
0 .227 .745 r
.78444 .71602 .79129 .72612 .80864 .74103 .8018 .73086 Metetra
0 .225 .744 r
.8018 .73086 .80864 .74103 .82631 .75627 .81949 .74604 Metetra
0 .223 .742 r
.81949 .74604 .82631 .75627 .8443 .77187 .8375 .7616 Metetra
0 .22 .741 r
.8375 .7616 .8443 .77187 .86263 .78785 .85585 .77753 Metetra
0 .218 .739 r
.85585 .77753 .86263 .78785 .8813 .80421 .87456 .79386 Metetra
0 .215 .737 r
.87456 .79386 .8813 .80421 .90034 .82098 .89362 .81061 Metetra
0 .211 .734 r
.89362 .81061 .90034 .82098 .91975 .83817 .91306 .82778 Metetra
0 .208 .732 r
.91306 .82778 .91975 .83817 .93955 .85579 .93289 .84539 Metetra
0 .205 .729 r
.93289 .84539 .93955 .85579 .95974 .87386 .95312 .86346 Metetra
.633 .711 .903 r
.35847 .54522 .36602 .55311 .3782 .55029 .37067 .54238 Metetra
.631 .711 .904 r
.37067 .54238 .3782 .55029 .39043 .54751 .38294 .53957 Metetra
.629 .71 .905 r
.38294 .53957 .39043 .54751 .40274 .54477 .39528 .5368 Metetra
.628 .71 .905 r
.39528 .5368 .40274 .54477 .41512 .54205 .40769 .53406 Metetra
.626 .71 .906 r
.40769 .53406 .41512 .54205 .42757 .53936 .42017 .53135 Metetra
.625 .71 .907 r
.42017 .53135 .42757 .53936 .44009 .53668 .43273 .52865 Metetra
.624 .71 .908 r
.43273 .52865 .44009 .53668 .45268 .53402 .44536 .52597 Metetra
.623 .711 .908 r
.44536 .52597 .45268 .53402 .46535 .53136 .45806 .5233 Metetra
.623 .711 .909 r
.45806 .5233 .46535 .53136 .4781 .5287 .47084 .52063 Metetra
.622 .711 .91 r
.47084 .52063 .4781 .5287 .49092 .52603 .4837 .51795 Metetra
.622 .712 .91 r
.4837 .51795 .49092 .52603 .50382 .52335 .49664 .51526 Metetra
.622 .713 .911 r
.49664 .51526 .50382 .52335 .5168 .52065 .50966 .51255 Metetra
.622 .713 .911 r
.50966 .51255 .5168 .52065 .52986 .51793 .52275 .50981 Metetra
.623 .714 .912 r
.52275 .50981 .52986 .51793 .543 .51518 .53593 .50705 Metetra
.354 .592 .935 r
.53593 .50705 .543 .51518 .55628 .51831 .54924 .51011 Metetra
0 .101 .624 r
.54924 .51011 .55628 .51831 .56983 .53118 .56279 .5228 Metetra
0 .128 .651 r
.56279 .5228 .56983 .53118 .58359 .54392 .57656 .53537 Metetra
0 .15 .673 r
.57656 .53537 .58359 .54392 .59758 .55658 .59056 .54785 Metetra
0 .169 .691 r
.59056 .54785 .59758 .55658 .6118 .5692 .60478 .5603 Metetra
0 .185 .706 r
.60478 .5603 .6118 .5692 .62624 .58182 .61922 .57276 Metetra
0 .198 .717 r
.61922 .57276 .62624 .58182 .64091 .59449 .6339 .58527 Metetra
0 .208 .727 r
.6339 .58527 .64091 .59449 .65582 .60723 .64882 .59787 Metetra
0 .216 .734 r
.64882 .59787 .65582 .60723 .67098 .6201 .66398 .61059 Metetra
0 .222 .739 r
.66398 .61059 .67098 .6201 .68638 .63312 .67939 .62348 Metetra
0 .227 .743 r
.67939 .62348 .68638 .63312 .70204 .64632 .69505 .63656 Metetra
0 .23 .746 r
.69505 .63656 .70204 .64632 .71796 .65974 .71098 .64985 Metetra
0 .232 .748 r
.71098 .64985 .71796 .65974 .73415 .67339 .72719 .6634 Metetra
0 .232 .749 r
.72719 .6634 .73415 .67339 .75062 .6873 .74367 .67722 Metetra
0 .232 .749 r
.74367 .67722 .75062 .6873 .76738 .70151 .76044 .69133 Metetra
0 .231 .748 r
.76044 .69133 .76738 .70151 .78444 .71602 .77751 .70576 Metetra
0 .23 .747 r
.77751 .70576 .78444 .71602 .8018 .73086 .7949 .72053 Metetra
0 .228 .746 r
.7949 .72053 .8018 .73086 .81949 .74604 .8126 .73566 Metetra
0 .225 .744 r
.8126 .73566 .81949 .74604 .8375 .7616 .83063 .75116 Metetra
0 .222 .742 r
.83063 .75116 .8375 .7616 .85585 .77753 .84901 .76706 Metetra
0 .219 .74 r
.84901 .76706 .85585 .77753 .87456 .79386 .86774 .78336 Metetra
0 .216 .738 r
.86774 .78336 .87456 .79386 .89362 .81061 .88683 .80008 Metetra
0 .213 .735 r
.88683 .80008 .89362 .81061 .91306 .82778 .9063 .81723 Metetra
0 .209 .733 r
.9063 .81723 .91306 .82778 .93289 .84539 .92617 .83484 Metetra
0 .206 .73 r
.92617 .83484 .93289 .84539 .95312 .86346 .94643 .85291 Metetra
.633 .711 .903 r
.35085 .53726 .35847 .54522 .37067 .54238 .36308 .53438 Metetra
.631 .71 .904 r
.36308 .53438 .37067 .54238 .38294 .53957 .37538 .53155 Metetra
.629 .71 .905 r
.37538 .53155 .38294 .53957 .39528 .5368 .38776 .52875 Metetra
.627 .71 .905 r
.38776 .52875 .39528 .5368 .40769 .53406 .4002 .52599 Metetra
.626 .71 .906 r
.4002 .52599 .40769 .53406 .42017 .53135 .41271 .52326 Metetra
.625 .71 .907 r
.41271 .52326 .42017 .53135 .43273 .52865 .4253 .52055 Metetra
.624 .71 .908 r
.4253 .52055 .43273 .52865 .44536 .52597 .43796 .51785 Metetra
.623 .71 .908 r
.43796 .51785 .44536 .52597 .45806 .5233 .4507 .51516 Metetra
.622 .71 .909 r
.4507 .51516 .45806 .5233 .47084 .52063 .46352 .51248 Metetra
.622 .711 .91 r
.46352 .51248 .47084 .52063 .4837 .51795 .47642 .50979 Metetra
.621 .711 .91 r
.47642 .50979 .4837 .51795 .49664 .51526 .48939 .50708 Metetra
.621 .712 .911 r
.48939 .50708 .49664 .51526 .50966 .51255 .50245 .50436 Metetra
.621 .713 .911 r
.50245 .50436 .50966 .51255 .52275 .50981 .51558 .50162 Metetra
.622 .714 .912 r
.51558 .50162 .52275 .50981 .53593 .50705 .5288 .49884 Metetra
.357 .594 .936 r
.5288 .49884 .53593 .50705 .54924 .51011 .54213 .50182 Metetra
0 .112 .634 r
.54213 .50182 .54924 .51011 .56279 .5228 .55569 .51433 Metetra
0 .138 .66 r
.55569 .51433 .56279 .5228 .57656 .53537 .56946 .52672 Metetra
0 .16 .681 r
.56946 .52672 .57656 .53537 .59056 .54785 .58346 .53902 Metetra
0 .178 .699 r
.58346 .53902 .59056 .54785 .60478 .5603 .59769 .5513 Metetra
0 .194 .713 r
.59769 .5513 .60478 .5603 .61922 .57276 .61214 .56358 Metetra
0 .206 .724 r
.61214 .56358 .61922 .57276 .6339 .58527 .62682 .57593 Metetra
0 .216 .733 r
.62682 .57593 .6339 .58527 .64882 .59787 .64175 .58838 Metetra
0 .224 .74 r
.64175 .58838 .64882 .59787 .66398 .61059 .65691 .60095 Metetra
0 .229 .745 r
.65691 .60095 .66398 .61059 .67939 .62348 .67233 .6137 Metetra
0 .233 .748 r
.67233 .6137 .67939 .62348 .69505 .63656 .688 .62665 Metetra
0 .236 .75 r
.688 .62665 .69505 .63656 .71098 .64985 .70394 .63983 Metetra
0 .237 .752 r
.70394 .63983 .71098 .64985 .72719 .6634 .72015 .65326 Metetra
0 .237 .752 r
.72015 .65326 .72719 .6634 .74367 .67722 .73665 .66698 Metetra
0 .236 .752 r
.73665 .66698 .74367 .67722 .76044 .69133 .75343 .681 Metetra
0 .235 .751 r
.75343 .681 .76044 .69133 .77751 .70576 .77052 .69536 Metetra
0 .233 .75 r
.77052 .69536 .77751 .70576 .7949 .72053 .78792 .71006 Metetra
0 .231 .748 r
.78792 .71006 .7949 .72053 .8126 .73566 .80564 .72512 Metetra
0 .228 .746 r
.80564 .72512 .8126 .73566 .83063 .75116 .82369 .74057 Metetra
0 .224 .744 r
.82369 .74057 .83063 .75116 .84901 .76706 .84209 .75642 Metetra
0 .221 .742 r
.84209 .75642 .84901 .76706 .86774 .78336 .86084 .77269 Metetra
0 .218 .739 r
.86084 .77269 .86774 .78336 .88683 .80008 .87997 .78939 Metetra
0 .214 .736 r
.87997 .78939 .88683 .80008 .9063 .81723 .89947 .80653 Metetra
0 .21 .734 r
.89947 .80653 .9063 .81723 .92617 .83484 .91937 .82413 Metetra
0 .206 .731 r
.91937 .82413 .92617 .83484 .94643 .85291 .93967 .8422 Metetra
.633 .711 .903 r
.34316 .52921 .35085 .53726 .36308 .53438 .35542 .5263 Metetra
.631 .71 .904 r
.35542 .5263 .36308 .53438 .37538 .53155 .36776 .52344 Metetra
.629 .71 .904 r
.36776 .52344 .37538 .53155 .38776 .52875 .38016 .52062 Metetra
.627 .709 .905 r
.38016 .52062 .38776 .52875 .4002 .52599 .39264 .51783 Metetra
.625 .709 .906 r
.39264 .51783 .4002 .52599 .41271 .52326 .40519 .51508 Metetra
.624 .709 .907 r
.40519 .51508 .41271 .52326 .4253 .52055 .41781 .51235 Metetra
.623 .709 .908 r
.41781 .51235 .4253 .52055 .43796 .51785 .43051 .50964 Metetra
.622 .71 .908 r
.43051 .50964 .43796 .51785 .4507 .51516 .44328 .50694 Metetra
.621 .71 .909 r
.44328 .50694 .4507 .51516 .46352 .51248 .45613 .50424 Metetra
.621 .71 .91 r
.45613 .50424 .46352 .51248 .47642 .50979 .46906 .50154 Metetra
.62 .711 .91 r
.46906 .50154 .47642 .50979 .48939 .50708 .48208 .49883 Metetra
.62 .712 .911 r
.48208 .49883 .48939 .50708 .50245 .50436 .49517 .4961 Metetra
.621 .712 .912 r
.49517 .4961 .50245 .50436 .51558 .50162 .50834 .49334 Metetra
.621 .713 .912 r
.50834 .49334 .51558 .50162 .5288 .49884 .5216 .49056 Metetra
.36 .595 .936 r
.5216 .49056 .5288 .49884 .54213 .50182 .53496 .49346 Metetra
0 .122 .643 r
.53496 .49346 .54213 .50182 .55569 .51433 .54852 .50578 Metetra
0 .148 .669 r
.54852 .50578 .55569 .51433 .56946 .52672 .5623 .51797 Metetra
0 .17 .69 r
.5623 .51797 .56946 .52672 .58346 .53902 .5763 .53008 Metetra
0 .188 .707 r
.5763 .53008 .58346 .53902 .59769 .5513 .59053 .54218 Metetra
0 .203 .721 r
.59053 .54218 .59769 .5513 .61214 .56358 .60499 .55429 Metetra
0 .215 .731 r
.60499 .55429 .61214 .56358 .62682 .57593 .61968 .56647 Metetra
0 .224 .739 r
.61968 .56647 .62682 .57593 .64175 .58838 .63461 .57875 Metetra
0 .231 .746 r
.63461 .57875 .64175 .58838 .65691 .60095 .64978 .59118 Metetra
0 .236 .75 r
.64978 .59118 .65691 .60095 .67233 .6137 .6652 .60378 Metetra
0 .239 .753 r
.6652 .60378 .67233 .6137 .688 .62665 .68088 .61659 Metetra
0 .241 .755 r
.68088 .61659 .688 .62665 .70394 .63983 .69683 .62965 Metetra
0 .242 .756 r
.69683 .62965 .70394 .63983 .72015 .65326 .71305 .64297 Metetra
0 .242 .756 r
.71305 .64297 .72015 .65326 .73665 .66698 .72956 .65658 Metetra
0 .24 .755 r
.72956 .65658 .73665 .66698 .75343 .681 .74636 .67052 Metetra
0 .239 .754 r
.74636 .67052 .75343 .681 .77052 .69536 .76346 .68479 Metetra
0 .236 .752 r
.76346 .68479 .77052 .69536 .78792 .71006 .78087 .69941 Metetra
0 .233 .75 r
.78087 .69941 .78792 .71006 .80564 .72512 .79861 .71442 Metetra
0 .23 .748 r
.79861 .71442 .80564 .72512 .82369 .74057 .81668 .72982 Metetra
0 .227 .746 r
.81668 .72982 .82369 .74057 .84209 .75642 .8351 .74563 Metetra
0 .223 .743 r
.8351 .74563 .84209 .75642 .86084 .77269 .85388 .76187 Metetra
0 .219 .74 r
.85388 .76187 .86084 .77269 .87997 .78939 .87303 .77854 Metetra
0 .215 .737 r
.87303 .77854 .87997 .78939 .89947 .80653 .89256 .79567 Metetra
0 .211 .734 r
.89256 .79567 .89947 .80653 .91937 .82413 .91249 .81327 Metetra
0 .207 .731 r
.91249 .81327 .91937 .82413 .93967 .8422 .93283 .83134 Metetra
.633 .71 .903 r
.3354 .52108 .34316 .52921 .35542 .5263 .3477 .51814 Metetra
.63 .71 .903 r
.3477 .51814 .35542 .5263 .36776 .52344 .36006 .51525 Metetra
.628 .709 .904 r
.36006 .51525 .36776 .52344 .38016 .52062 .3725 .5124 Metetra
.627 .709 .905 r
.3725 .5124 .38016 .52062 .39264 .51783 .38501 .50959 Metetra
.625 .709 .906 r
.38501 .50959 .39264 .51783 .40519 .51508 .39759 .50682 Metetra
.623 .709 .907 r
.39759 .50682 .40519 .51508 .41781 .51235 .41025 .50407 Metetra
.622 .709 .908 r
.41025 .50407 .41781 .51235 .43051 .50964 .42298 .50135 Metetra
.621 .709 .908 r
.42298 .50135 .43051 .50964 .44328 .50694 .43579 .49863 Metetra
.62 .709 .909 r
.43579 .49863 .44328 .50694 .45613 .50424 .44867 .49592 Metetra
.62 .71 .91 r
.44867 .49592 .45613 .50424 .46906 .50154 .46164 .49321 Metetra
.62 .71 .911 r
.46164 .49321 .46906 .50154 .48208 .49883 .47469 .49049 Metetra
.619 .711 .911 r
.47469 .49049 .48208 .49883 .49517 .4961 .48782 .48775 Metetra
.62 .712 .912 r
.48782 .48775 .49517 .4961 .50834 .49334 .50104 .48499 Metetra
.62 .713 .912 r
.50104 .48499 .50834 .49334 .5216 .49056 .51433 .48219 Metetra
.363 .597 .936 r
.51433 .48219 .5216 .49056 .53496 .49346 .52772 .48501 Metetra
0 .133 .653 r
.52772 .48501 .53496 .49346 .54852 .50578 .54128 .49713 Metetra
0 .159 .679 r
.54128 .49713 .54852 .50578 .5623 .51797 .55507 .50912 Metetra
0 .18 .699 r
.55507 .50912 .5623 .51797 .5763 .53008 .56908 .52104 Metetra
0 .198 .715 r
.56908 .52104 .5763 .53008 .59053 .54218 .58331 .53294 Metetra
0 .212 .728 r
.58331 .53294 .59053 .54218 .60499 .55429 .59777 .54487 Metetra
0 .224 .738 r
.59777 .54487 .60499 .55429 .61968 .56647 .61247 .55687 Metetra
0 .232 .746 r
.61247 .55687 .61968 .56647 .63461 .57875 .6274 .56899 Metetra
0 .239 .752 r
.6274 .56899 .63461 .57875 .64978 .59118 .64258 .58125 Metetra
0 .243 .755 r
.64258 .58125 .64978 .59118 .6652 .60378 .65801 .59371 Metetra
0 .246 .758 r
.65801 .59371 .6652 .60378 .68088 .61659 .6737 .60638 Metetra
0 .247 .759 r
.6737 .60638 .68088 .61659 .69683 .62965 .68965 .61931 Metetra
0 .247 .76 r
.68965 .61931 .69683 .62965 .71305 .64297 .70588 .63252 Metetra
0 .246 .759 r
.70588 .63252 .71305 .64297 .72956 .65658 .7224 .64603 Metetra
0 .245 .758 r
.7224 .64603 .72956 .65658 .74636 .67052 .73921 .65986 Metetra
0 .242 .757 r
.73921 .65986 .74636 .67052 .76346 .68479 .75632 .67405 Metetra
0 .239 .755 r
.75632 .67405 .76346 .68479 .78087 .69941 .77375 .68861 Metetra
0 .236 .753 r
.77375 .68861 .78087 .69941 .79861 .71442 .7915 .70355 Metetra
0 .232 .75 r
.7915 .70355 .79861 .71442 .81668 .72982 .8096 .7189 Metetra
0 .228 .747 r
.8096 .7189 .81668 .72982 .8351 .74563 .82804 .73467 Metetra
0 .224 .744 r
.82804 .73467 .8351 .74563 .85388 .76187 .84684 .75087 Metetra
0 .22 .741 r
.84684 .75087 .85388 .76187 .87303 .77854 .86601 .76753 Metetra
0 .216 .738 r
.86601 .76753 .87303 .77854 .89256 .79567 .88558 .78464 Metetra
0 .212 .735 r
.88558 .78464 .89256 .79567 .91249 .81327 .90554 .80224 Metetra
0 .207 .732 r
.90554 .80224 .91249 .81327 .93283 .83134 .92592 .82032 Metetra
.633 .71 .902 r
.32757 .51286 .3354 .52108 .3477 .51814 .3399 .50989 Metetra
.63 .709 .903 r
.3399 .50989 .3477 .51814 .36006 .51525 .3523 .50697 Metetra
.628 .709 .904 r
.3523 .50697 .36006 .51525 .3725 .5124 .36477 .50409 Metetra
.626 .708 .905 r
.36477 .50409 .3725 .5124 .38501 .50959 .37731 .50126 Metetra
.624 .708 .906 r
.37731 .50126 .38501 .50959 .39759 .50682 .38993 .49847 Metetra
.623 .708 .907 r
.38993 .49847 .39759 .50682 .41025 .50407 .40262 .49571 Metetra
.621 .708 .908 r
.40262 .49571 .41025 .50407 .42298 .50135 .41538 .49297 Metetra
.62 .708 .908 r
.41538 .49297 .42298 .50135 .43579 .49863 .42823 .49024 Metetra
.619 .709 .909 r
.42823 .49024 .43579 .49863 .44867 .49592 .44115 .48752 Metetra
.619 .709 .91 r
.44115 .48752 .44867 .49592 .46164 .49321 .45415 .4848 Metetra
.619 .71 .911 r
.45415 .4848 .46164 .49321 .47469 .49049 .46724 .48207 Metetra
.618 .711 .911 r
.46724 .48207 .47469 .49049 .48782 .48775 .48041 .47932 Metetra
.619 .712 .912 r
.48041 .47932 .48782 .48775 .50104 .48499 .49366 .47655 Metetra
.619 .713 .913 r
.49366 .47655 .50104 .48499 .51433 .48219 .507 .47375 Metetra
.367 .599 .936 r
.507 .47375 .51433 .48219 .52772 .48501 .52041 .47648 Metetra
0 .145 .664 r
.52041 .47648 .52772 .48501 .54128 .49713 .53398 .48839 Metetra
0 .17 .688 r
.53398 .48839 .54128 .49713 .55507 .50912 .54777 .50017 Metetra
0 .191 .708 r
.54777 .50017 .55507 .50912 .56908 .52104 .56178 .51189 Metetra
0 .208 .724 r
.56178 .51189 .56908 .52104 .58331 .53294 .57602 .52359 Metetra
0 .222 .736 r
.57602 .52359 .58331 .53294 .59777 .54487 .59049 .53533 Metetra
0 .233 .746 r
.59049 .53533 .59777 .54487 .61247 .55687 .60519 .54714 Metetra
0 .241 .753 r
.60519 .54714 .61247 .55687 .6274 .56899 .62013 .55908 Metetra
0 .246 .758 r
.62013 .55908 .6274 .56899 .64258 .58125 .63531 .57118 Metetra
0 .25 .761 r
.63531 .57118 .64258 .58125 .65801 .59371 .65075 .58349 Metetra
0 .252 .763 r
.65075 .58349 .65801 .59371 .6737 .60638 .66644 .59602 Metetra
0 .253 .764 r
.66644 .59602 .6737 .60638 .68965 .61931 .6824 .60881 Metetra
0 .252 .764 r
.6824 .60881 .68965 .61931 .70588 .63252 .69864 .6219 Metetra
0 .251 .763 r
.69864 .6219 .70588 .63252 .7224 .64603 .71517 .6353 Metetra
0 .249 .761 r
.71517 .6353 .7224 .64603 .73921 .65986 .73199 .64904 Metetra
0 .246 .759 r
.73199 .64904 .73921 .65986 .75632 .67405 .74911 .66314 Metetra
0 .243 .757 r
.74911 .66314 .75632 .67405 .77375 .68861 .76656 .67763 Metetra
0 .239 .755 r
.76656 .67763 .77375 .68861 .7915 .70355 .78433 .69251 Metetra
0 .235 .752 r
.78433 .69251 .7915 .70355 .8096 .7189 .80244 .70781 Metetra
0 .23 .749 r
.80244 .70781 .8096 .7189 .82804 .73467 .8209 .72354 Metetra
0 .226 .746 r
.8209 .72354 .82804 .73467 .84684 .75087 .83972 .73971 Metetra
0 .221 .742 r
.83972 .73971 .84684 .75087 .86601 .76753 .85893 .75635 Metetra
0 .217 .739 r
.85893 .75635 .86601 .76753 .88558 .78464 .87852 .77345 Metetra
0 .212 .736 r
.87852 .77345 .88558 .78464 .90554 .80224 .89852 .79104 Metetra
0 .208 .732 r
.89852 .79104 .90554 .80224 .92592 .82032 .91893 .80913 Metetra
.632 .71 .902 r
.31968 .50456 .32757 .51286 .3399 .50989 .33204 .50155 Metetra
.63 .709 .903 r
.33204 .50155 .3399 .50989 .3523 .50697 .34447 .4986 Metetra
.628 .708 .904 r
.34447 .4986 .3523 .50697 .36477 .50409 .35697 .4957 Metetra
.626 .708 .905 r
.35697 .4957 .36477 .50409 .37731 .50126 .36955 .49284 Metetra
.624 .708 .906 r
.36955 .49284 .37731 .50126 .38993 .49847 .38219 .49003 Metetra
.622 .708 .907 r
.38219 .49003 .38993 .49847 .40262 .49571 .39492 .48725 Metetra
.621 .708 .908 r
.39492 .48725 .40262 .49571 .41538 .49297 .40772 .48449 Metetra
.619 .708 .909 r
.40772 .48449 .41538 .49297 .42823 .49024 .42059 .48176 Metetra
.619 .708 .909 r
.42059 .48176 .42823 .49024 .44115 .48752 .43355 .47903 Metetra
.618 .709 .91 r
.43355 .47903 .44115 .48752 .45415 .4848 .44659 .4763 Metetra
.617 .709 .911 r
.44659 .4763 .45415 .4848 .46724 .48207 .45971 .47356 Metetra
.617 .71 .912 r
.45971 .47356 .46724 .48207 .48041 .47932 .47292 .47081 Metetra
.617 .711 .912 r
.47292 .47081 .48041 .47932 .49366 .47655 .48621 .46803 Metetra
.618 .712 .913 r
.48621 .46803 .49366 .47655 .507 .47375 .49959 .46522 Metetra
.37 .601 .937 r
.49959 .46522 .507 .47375 .52041 .47648 .51303 .46787 Metetra
0 .157 .675 r
.51303 .46787 .52041 .47648 .53398 .48839 .5266 .47955 Metetra
0 .182 .699 r
.5266 .47955 .53398 .48839 .54777 .50017 .5404 .49111 Metetra
0 .203 .718 r
.5404 .49111 .54777 .50017 .56178 .51189 .55442 .50262 Metetra
0 .219 .733 r
.55442 .50262 .56178 .51189 .57602 .52359 .56866 .51411 Metetra
0 .232 .744 r
.56866 .51411 .57602 .52359 .59049 .53533 .58313 .52565 Metetra
0 .242 .753 r
.58313 .52565 .59049 .53533 .60519 .54714 .59784 .53727 Metetra
0 .25 .759 r
.59784 .53727 .60519 .54714 .62013 .55908 .61278 .54903 Metetra
0 .254 .764 r
.61278 .54903 .62013 .55908 .63531 .57118 .62797 .56096 Metetra
0 .257 .766 r
.62797 .56096 .63531 .57118 .65075 .58349 .64341 .5731 Metetra
0 .259 .768 r
.64341 .5731 .65075 .58349 .66644 .59602 .65911 .58549 Metetra
0 .259 .768 r
.65911 .58549 .66644 .59602 .6824 .60881 .67508 .59815 Metetra
0 .258 .767 r
.67508 .59815 .6824 .60881 .69864 .6219 .69133 .61111 Metetra
0 .256 .766 r
.69133 .61111 .69864 .6219 .71517 .6353 .70786 .6244 Metetra
0 .253 .764 r
.70786 .6244 .71517 .6353 .73199 .64904 .72469 .63805 Metetra
0 .249 .762 r
.72469 .63805 .73199 .64904 .74911 .66314 .74183 .65206 Metetra
0 .246 .76 r
.74183 .65206 .74911 .66314 .76656 .67763 .75929 .66647 Metetra
0 .241 .757 r
.75929 .66647 .76656 .67763 .78433 .69251 .77708 .68129 Metetra
0 .237 .754 r
.77708 .68129 .78433 .69251 .80244 .70781 .7952 .69654 Metetra
0 .232 .75 r
.7952 .69654 .80244 .70781 .8209 .72354 .81369 .71223 Metetra
0 .227 .747 r
.81369 .71223 .8209 .72354 .83972 .73971 .83253 .72838 Metetra
0 .222 .743 r
.83253 .72838 .83972 .73971 .85893 .75635 .85176 .74499 Metetra
0 .218 .74 r
.85176 .74499 .85893 .75635 .87852 .77345 .87138 .76209 Metetra
0 .213 .736 r
.87138 .76209 .87852 .77345 .89852 .79104 .89141 .77968 Metetra
0 .208 .733 r
.89141 .77968 .89852 .79104 .91893 .80913 .91186 .79778 Metetra
.632 .709 .902 r
.31171 .49617 .31968 .50456 .33204 .50155 .32411 .49312 Metetra
.63 .708 .903 r
.32411 .49312 .33204 .50155 .34447 .4986 .33657 .49014 Metetra
.627 .708 .904 r
.33657 .49014 .34447 .4986 .35697 .4957 .34911 .48721 Metetra
.625 .707 .905 r
.34911 .48721 .35697 .4957 .36955 .49284 .36171 .48433 Metetra
.623 .707 .906 r
.36171 .48433 .36955 .49284 .38219 .49003 .37439 .4815 Metetra
.621 .707 .907 r
.37439 .4815 .38219 .49003 .39492 .48725 .38715 .4787 Metetra
.62 .707 .908 r
.38715 .4787 .39492 .48725 .40772 .48449 .39998 .47593 Metetra
.619 .707 .909 r
.39998 .47593 .40772 .48449 .42059 .48176 .41289 .47318 Metetra
.618 .708 .909 r
.41289 .47318 .42059 .48176 .43355 .47903 .42589 .47044 Metetra
.617 .708 .91 r
.42589 .47044 .43355 .47903 .44659 .4763 .43896 .46771 Metetra
.616 .709 .911 r
.43896 .46771 .44659 .4763 .45971 .47356 .45212 .46496 Metetra
.616 .71 .912 r
.45212 .46496 .45971 .47356 .47292 .47081 .46536 .46221 Metetra
.616 .71 .912 r
.46536 .46221 .47292 .47081 .48621 .46803 .47869 .45942 Metetra
.617 .712 .913 r
.47869 .45942 .48621 .46803 .49959 .46522 .49211 .45661 Metetra
.374 .603 .937 r
.49211 .45661 .49959 .46522 .51303 .46787 .50557 .45917 Metetra
0 .17 .686 r
.50557 .45917 .51303 .46787 .5266 .47955 .51916 .47062 Metetra
0 .194 .709 r
.51916 .47062 .5266 .47955 .5404 .49111 .53296 .48195 Metetra
0 .215 .728 r
.53296 .48195 .5404 .49111 .55442 .50262 .54698 .49323 Metetra
0 .231 .742 r
.54698 .49323 .55442 .50262 .56866 .51411 .56123 .5045 Metetra
0 .243 .753 r
.56123 .5045 .56866 .51411 .58313 .52565 .57571 .51583 Metetra
0 .252 .761 r
.57571 .51583 .58313 .52565 .59784 .53727 .59042 .52726 Metetra
0 .259 .766 r
.59042 .52726 .59784 .53727 .61278 .54903 .60537 .53883 Metetra
0 .263 .77 r
.60537 .53883 .61278 .54903 .62797 .56096 .62056 .55058 Metetra
0 .265 .772 r
.62056 .55058 .62797 .56096 .64341 .5731 .63601 .56255 Metetra
0 .266 .772 r
.63601 .56255 .64341 .5731 .65911 .58549 .65172 .57479 Metetra
0 .265 .772 r
.65172 .57479 .65911 .58549 .67508 .59815 .66769 .58731 Metetra
0 .263 .771 r
.66769 .58731 .67508 .59815 .69133 .61111 .68395 .60014 Metetra
0 .26 .77 r
.68395 .60014 .69133 .61111 .70786 .6244 .70049 .61332 Metetra
0 .257 .767 r
.70049 .61332 .70786 .6244 .72469 .63805 .71733 .62687 Metetra
0 .253 .765 r
.71733 .62687 .72469 .63805 .74183 .65206 .73448 .6408 Metetra
0 .249 .762 r
.73448 .6408 .74183 .65206 .75929 .66647 .75195 .65514 Metetra
0 .244 .759 r
.75195 .65514 .75929 .66647 .77708 .68129 .76975 .66989 Metetra
0 .239 .755 r
.76975 .66989 .77708 .68129 .7952 .69654 .78789 .68509 Metetra
0 .234 .752 r
.78789 .68509 .7952 .69654 .81369 .71223 .8064 .70075 Metetra
0 .229 .748 r
.8064 .70075 .81369 .71223 .83253 .72838 .82527 .71687 Metetra
0 .223 .744 r
.82527 .71687 .83253 .72838 .85176 .74499 .84452 .73346 Metetra
0 .218 .74 r
.84452 .73346 .85176 .74499 .87138 .76209 .86417 .75056 Metetra
0 .213 .737 r
.86417 .75056 .87138 .76209 .89141 .77968 .88423 .76815 Metetra
0 .208 .733 r
.88423 .76815 .89141 .77968 .91186 .79778 .90472 .78626 Metetra
.632 .709 .902 r
.30368 .48769 .31171 .49617 .32411 .49312 .3161 .48461 Metetra
.63 .708 .903 r
.3161 .48461 .32411 .49312 .33657 .49014 .3286 .48159 Metetra
.627 .707 .904 r
.3286 .48159 .33657 .49014 .34911 .48721 .34117 .47863 Metetra
.625 .707 .905 r
.34117 .47863 .34911 .48721 .36171 .48433 .3538 .47573 Metetra
.623 .706 .906 r
.3538 .47573 .36171 .48433 .37439 .4815 .36652 .47287 Metetra
.621 .706 .907 r
.36652 .47287 .37439 .4815 .38715 .4787 .3793 .47006 Metetra
.619 .706 .908 r
.3793 .47006 .38715 .4787 .39998 .47593 .39217 .46727 Metetra
.618 .706 .909 r
.39217 .46727 .39998 .47593 .41289 .47318 .40512 .46451 Metetra
.617 .707 .909 r
.40512 .46451 .41289 .47318 .42589 .47044 .41815 .46177 Metetra
.616 .707 .91 r
.41815 .46177 .42589 .47044 .43896 .46771 .43126 .45903 Metetra
.615 .708 .911 r
.43126 .45903 .43896 .46771 .45212 .46496 .44445 .45628 Metetra
.615 .709 .912 r
.44445 .45628 .45212 .46496 .46536 .46221 .45773 .45352 Metetra
.615 .71 .913 r
.45773 .45352 .46536 .46221 .47869 .45942 .4711 .45073 Metetra
.615 .711 .913 r
.4711 .45073 .47869 .45942 .49211 .45661 .48456 .44791 Metetra
.378 .605 .937 r
.48456 .44791 .49211 .45661 .50557 .45917 .49805 .45038 Metetra
0 .183 .697 r
.49805 .45038 .50557 .45917 .51916 .47062 .51164 .46158 Metetra
0 .207 .72 r
.51164 .46158 .51916 .47062 .53296 .48195 .52545 .47267 Metetra
0 .227 .738 r
.52545 .47267 .53296 .48195 .54698 .49323 .53948 .48372 Metetra
0 .242 .751 r
.53948 .48372 .54698 .49323 .56123 .5045 .55373 .49476 Metetra
0 .254 .761 r
.55373 .49476 .56123 .5045 .57571 .51583 .56821 .50587 Metetra
0 .262 .768 r
.56821 .50587 .57571 .51583 .59042 .52726 .58293 .51709 Metetra
0 .268 .773 r
.58293 .51709 .59042 .52726 .60537 .53883 .59788 .52846 Metetra
0 .271 .776 r
.59788 .52846 .60537 .53883 .62056 .55058 .61308 .54003 Metetra
0 .273 .777 r
.61308 .54003 .62056 .55058 .63601 .56255 .62853 .55183 Metetra
0 .272 .777 r
.62853 .55183 .63601 .56255 .65172 .57479 .64425 .56391 Metetra
0 .271 .777 r
.64425 .56391 .65172 .57479 .66769 .58731 .66023 .57629 Metetra
0 .268 .775 r
.66023 .57629 .66769 .58731 .68395 .60014 .67649 .58899 Metetra
0 .265 .773 r
.67649 .58899 .68395 .60014 .70049 .61332 .69304 .60206 Metetra
0 .261 .77 r
.69304 .60206 .70049 .61332 .71733 .62687 .70989 .6155 Metetra
0 .257 .767 r
.70989 .6155 .71733 .62687 .73448 .6408 .72705 .62935 Metetra
0 .252 .764 r
.72705 .62935 .73448 .6408 .75195 .65514 .74453 .64361 Metetra
0 .246 .76 r
.74453 .64361 .75195 .65514 .76975 .66989 .76235 .65831 Metetra
0 .241 .757 r
.76235 .65831 .76975 .66989 .78789 .68509 .78051 .67346 Metetra
0 .235 .753 r
.78051 .67346 .78789 .68509 .8064 .70075 .79903 .68908 Metetra
0 .23 .749 r
.79903 .68908 .8064 .70075 .82527 .71687 .81792 .70517 Metetra
0 .224 .745 r
.81792 .70517 .82527 .71687 .84452 .73346 .8372 .72176 Metetra
0 .219 .741 r
.8372 .72176 .84452 .73346 .86417 .75056 .85688 .73884 Metetra
0 .213 .737 r
.85688 .73884 .86417 .75056 .88423 .76815 .87697 .75645 Metetra
0 .208 .733 r
.87697 .75645 .88423 .76815 .90472 .78626 .89749 .77457 Metetra
.632 .709 .901 r
.29557 .47912 .30368 .48769 .3161 .48461 .30803 .47599 Metetra
.63 .708 .902 r
.30803 .47599 .3161 .48461 .3286 .48159 .32056 .47294 Metetra
.627 .707 .903 r
.32056 .47294 .3286 .48159 .34117 .47863 .33315 .46995 Metetra
.624 .706 .904 r
.33315 .46995 .34117 .47863 .3538 .47573 .34583 .46702 Metetra
.622 .706 .905 r
.34583 .46702 .3538 .47573 .36652 .47287 .35857 .46414 Metetra
.62 .706 .906 r
.35857 .46414 .36652 .47287 .3793 .47006 .37139 .46131 Metetra
.618 .706 .907 r
.37139 .46131 .3793 .47006 .39217 .46727 .38429 .45852 Metetra
.617 .706 .908 r
.38429 .45852 .39217 .46727 .40512 .46451 .39727 .45575 Metetra
.615 .706 .909 r
.39727 .45575 .40512 .46451 .41815 .46177 .41034 .453 Metetra
.615 .707 .91 r
.41034 .453 .41815 .46177 .43126 .45903 .42348 .45025 Metetra
.614 .707 .911 r
.42348 .45025 .43126 .45903 .44445 .45628 .43671 .4475 Metetra
.614 .708 .912 r
.43671 .4475 .44445 .45628 .45773 .45352 .45003 .44474 Metetra
.614 .709 .913 r
.45003 .44474 .45773 .45352 .4711 .45073 .46344 .44195 Metetra
.614 .71 .914 r
.46344 .44195 .4711 .45073 .48456 .44791 .47693 .43913 Metetra
.381 .607 .937 r
.47693 .43913 .48456 .44791 .49805 .45038 .49045 .4415 Metetra
0 .197 .709 r
.49045 .4415 .49805 .45038 .51164 .46158 .50405 .45244 Metetra
0 .221 .731 r
.50405 .45244 .51164 .46158 .52545 .47267 .51786 .46328 Metetra
0 .24 .748 r
.51786 .46328 .52545 .47267 .53948 .48372 .5319 .47408 Metetra
0 .254 .76 r
.5319 .47408 .53948 .48372 .55373 .49476 .54616 .48489 Metetra
0 .265 .769 r
.54616 .48489 .55373 .49476 .56821 .50587 .56065 .49577 Metetra
0 .273 .776 r
.56065 .49577 .56821 .50587 .58293 .51709 .57537 .50676 Metetra
0 .277 .78 r
.57537 .50676 .58293 .51709 .59788 .52846 .59033 .51793 Metetra
0 .28 .782 r
.59033 .51793 .59788 .52846 .61308 .54003 .60553 .52931 Metetra
0 .28 .783 r
.60553 .52931 .61308 .54003 .62853 .55183 .62099 .54093 Metetra
0 .279 .782 r
.62099 .54093 .62853 .55183 .64425 .56391 .6367 .55285 Metetra
0 .277 .781 r
.6367 .55285 .64425 .56391 .66023 .57629 .65269 .56508 Metetra
0 .274 .779 r
.65269 .56508 .66023 .57629 .67649 .58899 .66896 .57765 Metetra
0 .27 .776 r
.66896 .57765 .67649 .58899 .69304 .60206 .68552 .5906 Metetra
0 .265 .773 r
.68552 .5906 .69304 .60206 .70989 .6155 .70238 .60394 Metetra
0 .26 .77 r
.70238 .60394 .70989 .6155 .72705 .62935 .71955 .6177 Metetra
0 .254 .766 r
.71955 .6177 .72705 .62935 .74453 .64361 .73704 .63189 Metetra
0 .249 .762 r
.73704 .63189 .74453 .64361 .76235 .65831 .75487 .64653 Metetra
0 .243 .758 r
.75487 .64653 .76235 .65831 .78051 .67346 .77305 .66164 Metetra
0 .237 .754 r
.77305 .66164 .78051 .67346 .79903 .68908 .79159 .67722 Metetra
0 .231 .75 r
.79159 .67722 .79903 .68908 .81792 .70517 .8105 .69329 Metetra
0 .225 .745 r
.8105 .69329 .81792 .70517 .8372 .72176 .8298 .70987 Metetra
0 .219 .741 r
.8298 .70987 .8372 .72176 .85688 .73884 .84951 .72696 Metetra
0 .213 .737 r
.84951 .72696 .85688 .73884 .87697 .75645 .86963 .74457 Metetra
0 .208 .733 r
.86963 .74457 .87697 .75645 .89749 .77457 .89019 .76271 Metetra
.633 .708 .901 r
.28739 .47045 .29557 .47912 .30803 .47599 .29988 .46729 Metetra
.63 .707 .902 r
.29988 .46729 .30803 .47599 .32056 .47294 .31244 .46419 Metetra
.627 .706 .903 r
.31244 .46419 .32056 .47294 .33315 .46995 .32507 .46117 Metetra
.624 .706 .904 r
.32507 .46117 .33315 .46995 .34583 .46702 .33777 .45822 Metetra
.622 .705 .905 r
.33777 .45822 .34583 .46702 .35857 .46414 .35055 .45532 Metetra
.619 .705 .906 r
.35055 .45532 .35857 .46414 .37139 .46131 .36341 .45247 Metetra
.617 .705 .907 r
.36341 .45247 .37139 .46131 .38429 .45852 .37634 .44966 Metetra
.616 .705 .908 r
.37634 .44966 .38429 .45852 .39727 .45575 .38935 .44688 Metetra
.614 .705 .91 r
.38935 .44688 .39727 .45575 .41034 .453 .40245 .44413 Metetra
.613 .706 .911 r
.40245 .44413 .41034 .453 .42348 .45025 .41563 .44138 Metetra
.613 .707 .912 r
.41563 .44138 .42348 .45025 .43671 .4475 .4289 .43863 Metetra
.612 .707 .912 r
.4289 .43863 .43671 .4475 .45003 .44474 .44226 .43587 Metetra
.612 .709 .913 r
.44226 .43587 .45003 .44474 .46344 .44195 .4557 .43308 Metetra
.613 .71 .914 r
.4557 .43308 .46344 .44195 .47693 .43913 .46923 .43026 Metetra
.386 .609 .937 r
.46923 .43026 .47693 .43913 .49045 .4415 .48278 .43253 Metetra
0 .212 .721 r
.48278 .43253 .49045 .4415 .50405 .45244 .49639 .4432 Metetra
0 .235 .742 r
.49639 .4432 .50405 .45244 .51786 .46328 .51021 .45377 Metetra
0 .253 .758 r
.51021 .45377 .51786 .46328 .5319 .47408 .52425 .46431 Metetra
0 .267 .77 r
.52425 .46431 .5319 .47408 .54616 .48489 .53852 .47487 Metetra
0 .277 .778 r
.53852 .47487 .54616 .48489 .56065 .49577 .55301 .4855 Metetra
0 .283 .783 r
.55301 .4855 .56065 .49577 .57537 .50676 .56773 .49627 Metetra
0 .287 .787 r
.56773 .49627 .57537 .50676 .59033 .51793 .5827 .50722 Metetra
0 .289 .788 r
.5827 .50722 .59033 .51793 .60553 .52931 .59791 .5184 Metetra
0 .288 .788 r
.59791 .5184 .60553 .52931 .62099 .54093 .61337 .52985 Metetra
0 .286 .787 r
.61337 .52985 .62099 .54093 .6367 .55285 .62909 .54159 Metetra
0 .283 .785 r
.62909 .54159 .6367 .55285 .65269 .56508 .64508 .55367 Metetra
0 .279 .782 r
.64508 .55367 .65269 .56508 .66896 .57765 .66136 .56611 Metetra
0 .275 .779 r
.66136 .56611 .66896 .57765 .68552 .5906 .67792 .57894 Metetra
0 .269 .776 r
.67792 .57894 .68552 .5906 .70238 .60394 .69479 .59218 Metetra
0 .263 .772 r
.69479 .59218 .70238 .60394 .71955 .6177 .71197 .60585 Metetra
0 .257 .768 r
.71197 .60585 .71955 .6177 .73704 .63189 .72947 .61997 Metetra
0 .251 .764 r
.72947 .61997 .73704 .63189 .75487 .64653 .74731 .63455 Metetra
0 .244 .759 r
.74731 .63455 .75487 .64653 .77305 .66164 .76551 .64962 Metetra
0 .238 .755 r
.76551 .64962 .77305 .66164 .79159 .67722 .78406 .66517 Metetra
0 .232 .75 r
.78406 .66517 .79159 .67722 .8105 .69329 .803 .68122 Metetra
0 .225 .746 r
.803 .68122 .8105 .69329 .8298 .70987 .82232 .69779 Metetra
0 .219 .741 r
.82232 .69779 .8298 .70987 .84951 .72696 .84206 .71488 Metetra
0 .213 .737 r
.84206 .71488 .84951 .72696 .86963 .74457 .86221 .73251 Metetra
0 .207 .733 r
.86221 .73251 .86963 .74457 .89019 .76271 .8828 .75067 Metetra
.633 .708 .901 r
.27914 .46169 .28739 .47045 .29988 .46729 .29166 .45848 Metetra
.63 .707 .902 r
.29166 .45848 .29988 .46729 .31244 .46419 .30425 .45535 Metetra
.627 .706 .903 r
.30425 .45535 .31244 .46419 .32507 .46117 .31692 .45229 Metetra
.624 .705 .904 r
.31692 .45229 .32507 .46117 .33777 .45822 .32965 .44931 Metetra
.621 .704 .905 r
.32965 .44931 .33777 .45822 .35055 .45532 .34246 .44639 Metetra
.619 .704 .906 r
.34246 .44639 .35055 .45532 .36341 .45247 .35535 .44352 Metetra
.616 .704 .907 r
.35535 .44352 .36341 .45247 .37634 .44966 .36831 .4407 Metetra
.615 .704 .908 r
.36831 .4407 .37634 .44966 .38935 .44688 .38136 .43792 Metetra
.613 .704 .91 r
.38136 .43792 .38935 .44688 .40245 .44413 .39449 .43516 Metetra
.612 .705 .911 r
.39449 .43516 .40245 .44413 .41563 .44138 .40771 .43241 Metetra
.611 .706 .912 r
.40771 .43241 .41563 .44138 .4289 .43863 .42101 .42966 Metetra
.611 .707 .913 r
.42101 .42966 .4289 .43863 .44226 .43587 .43441 .4269 Metetra
.611 .708 .914 r
.43441 .4269 .44226 .43587 .4557 .43308 .44789 .42412 Metetra
.611 .709 .914 r
.44789 .42412 .4557 .43308 .46923 .43026 .46146 .42131 Metetra
.39 .611 .938 r
.46146 .42131 .46923 .43026 .48278 .43253 .47504 .42346 Metetra
0 .228 .734 r
.47504 .42346 .48278 .43253 .49639 .4432 .48865 .43385 Metetra
0 .25 .754 r
.48865 .43385 .49639 .4432 .51021 .45377 .50248 .44414 Metetra
0 .267 .769 r
.50248 .44414 .51021 .45377 .52425 .46431 .51653 .4544 Metetra
0 .28 .78 r
.51653 .4544 .52425 .46431 .53852 .47487 .5308 .46469 Metetra
0 .289 .787 r
.5308 .46469 .53852 .47487 .55301 .4855 .5453 .47508 Metetra
0 .294 .791 r
.5453 .47508 .55301 .4855 .56773 .49627 .56003 .48561 Metetra
0 .297 .793 r
.56003 .48561 .56773 .49627 .5827 .50722 .575 .49634 Metetra
0 .298 .794 r
.575 .49634 .5827 .50722 .59791 .5184 .59021 .50731 Metetra
0 .296 .793 r
.59021 .50731 .59791 .5184 .61337 .52985 .60568 .51856 Metetra
0 .294 .792 r
.60568 .51856 .61337 .52985 .62909 .54159 .6214 .53013 Metetra
0 .29 .789 r
.6214 .53013 .62909 .54159 .64508 .55367 .6374 .54206 Metetra
0 .285 .786 r
.6374 .54206 .64508 .55367 .66136 .56611 .65368 .55436 Metetra
0 .279 .782 r
.65368 .55436 .66136 .56611 .67792 .57894 .67025 .56707 Metetra
0 .273 .778 r
.67025 .56707 .67792 .57894 .69479 .59218 .68712 .58021 Metetra
0 .267 .774 r
.68712 .58021 .69479 .59218 .71197 .60585 .70431 .5938 Metetra
0 .26 .77 r
.70431 .5938 .71197 .60585 .72947 .61997 .72182 .60785 Metetra
0 .253 .765 r
.72182 .60785 .72947 .61997 .74731 .63455 .73968 .62237 Metetra
0 .246 .76 r
.73968 .62237 .74731 .63455 .76551 .64962 .75789 .6374 Metetra
0 .239 .756 r
.75789 .6374 .76551 .64962 .78406 .66517 .77646 .65292 Metetra
0 .232 .751 r
.77646 .65292 .78406 .66517 .803 .68122 .79542 .66896 Metetra
0 .226 .746 r
.79542 .66896 .803 .68122 .82232 .69779 .81477 .68553 Metetra
0 .219 .741 r
.81477 .68553 .82232 .69779 .84206 .71488 .83453 .70263 Metetra
0 .213 .737 r
.83453 .70263 .84206 .71488 .86221 .73251 .85471 .72027 Metetra
0 .207 .732 r
.85471 .72027 .86221 .73251 .8828 .75067 .87533 .73846 Metetra
.633 .707 .9 r
.27081 .45283 .27914 .46169 .29166 .45848 .28337 .44957 Metetra
.63 .706 .901 r
.28337 .44957 .29166 .45848 .30425 .45535 .29599 .4464 Metetra
.626 .705 .902 r
.29599 .4464 .30425 .45535 .31692 .45229 .30869 .44331 Metetra
.623 .704 .903 r
.30869 .44331 .31692 .45229 .32965 .44931 .32145 .44029 Metetra
.62 .704 .905 r
.32145 .44029 .32965 .44931 .34246 .44639 .3343 .43735 Metetra
.618 .703 .906 r
.3343 .43735 .34246 .44639 .35535 .44352 .34721 .43447 Metetra
.615 .703 .907 r
.34721 .43447 .35535 .44352 .36831 .4407 .36021 .43164 Metetra
.613 .703 .908 r
.36021 .43164 .36831 .4407 .38136 .43792 .37329 .42885 Metetra
.612 .703 .91 r
.37329 .42885 .38136 .43792 .39449 .43516 .38646 .42608 Metetra
.611 .704 .911 r
.38646 .42608 .39449 .43516 .40771 .43241 .39971 .42333 Metetra
.61 .705 .912 r
.39971 .42333 .40771 .43241 .42101 .42966 .41305 .42059 Metetra
.609 .706 .913 r
.41305 .42059 .42101 .42966 .43441 .4269 .42648 .41784 Metetra
.609 .707 .914 r
.42648 .41784 .43441 .4269 .44789 .42412 .44 .41506 Metetra
.609 .708 .915 r
.44 .41506 .44789 .42412 .46146 .42131 .45361 .41226 Metetra
.394 .613 .938 r
.45361 .41226 .46146 .42131 .47504 .42346 .46722 .4143 Metetra
0 .244 .747 r
.46722 .4143 .47504 .42346 .48865 .43385 .48084 .42439 Metetra
0 .266 .766 r
.48084 .42439 .48865 .43385 .50248 .44414 .49467 .43438 Metetra
0 .282 .78 r
.49467 .43438 .50248 .44414 .51653 .4544 .50873 .44435 Metetra
0 .294 .789 r
.50873 .44435 .51653 .4544 .5308 .46469 .52301 .45436 Metetra
0 .301 .795 r
.52301 .45436 .5308 .46469 .5453 .47508 .53752 .46448 Metetra
0 .306 .799 r
.53752 .46448 .5453 .47508 .56003 .48561 .55225 .47476 Metetra
0 .307 .8 r
.55225 .47476 .56003 .48561 .575 .49634 .56722 .48526 Metetra
0 .307 .8 r
.56722 .48526 .575 .49634 .59021 .50731 .58244 .49601 Metetra
0 .304 .799 r
.58244 .49601 .59021 .50731 .60568 .51856 .59791 .50707 Metetra
0 .301 .796 r
.59791 .50707 .60568 .51856 .6214 .53013 .61364 .51846 Metetra
0 .296 .793 r
.61364 .51846 .6214 .53013 .6374 .54206 .62964 .53023 Metetra
0 .29 .789 r
.62964 .53023 .6374 .54206 .65368 .55436 .64592 .54239 Metetra
0 .284 .785 r
.64592 .54239 .65368 .55436 .67025 .56707 .6625 .55498 Metetra
0 .277 .781 r
.6625 .55498 .67025 .56707 .68712 .58021 .67938 .56802 Metetra
0 .27 .776 r
.67938 .56802 .68712 .58021 .70431 .5938 .69657 .58152 Metetra
0 .262 .771 r
.69657 .58152 .70431 .5938 .72182 .60785 .7141 .59551 Metetra
0 .255 .766 r
.7141 .59551 .72182 .60785 .73968 .62237 .73197 .60998 Metetra
0 .247 .761 r
.73197 .60998 .73968 .62237 .75789 .6374 .75019 .62497 Metetra
0 .24 .756 r
.75019 .62497 .75789 .6374 .77646 .65292 .76878 .64047 Metetra
0 .233 .751 r
.76878 .64047 .77646 .65292 .79542 .66896 .78775 .65651 Metetra
0 .226 .746 r
.78775 .65651 .79542 .66896 .81477 .68553 .80713 .67307 Metetra
0 .219 .741 r
.80713 .67307 .81477 .68553 .83453 .70263 .82691 .69019 Metetra
0 .212 .737 r
.82691 .69019 .83453 .70263 .85471 .72027 .84713 .70785 Metetra
0 .206 .732 r
.84713 .70785 .85471 .72027 .87533 .73846 .86778 .72608 Metetra
.633 .707 .9 r
.26241 .44388 .27081 .45283 .28337 .44957 .275 .44056 Metetra
.63 .706 .901 r
.275 .44056 .28337 .44957 .29599 .4464 .28765 .43734 Metetra
.626 .705 .902 r
.28765 .43734 .29599 .4464 .30869 .44331 .30038 .43422 Metetra
.623 .704 .903 r
.30038 .43422 .30869 .44331 .32145 .44029 .31318 .43117 Metetra
.62 .703 .904 r
.31318 .43117 .32145 .44029 .3343 .43735 .32606 .4282 Metetra
.617 .702 .906 r
.32606 .4282 .3343 .43735 .34721 .43447 .33901 .4253 Metetra
.614 .702 .907 r
.33901 .4253 .34721 .43447 .36021 .43164 .35204 .42246 Metetra
.612 .702 .908 r
.35204 .42246 .36021 .43164 .37329 .42885 .36515 .41966 Metetra
.61 .702 .909 r
.36515 .41966 .37329 .42885 .38646 .42608 .37835 .4169 Metetra
.609 .703 .911 r
.37835 .4169 .38646 .42608 .39971 .42333 .39164 .41416 Metetra
.608 .704 .912 r
.39164 .41416 .39971 .42333 .41305 .42059 .40501 .41142 Metetra
.607 .705 .913 r
.40501 .41142 .41305 .42059 .42648 .41784 .41847 .40868 Metetra
.607 .706 .914 r
.41847 .40868 .42648 .41784 .44 .41506 .43203 .40591 Metetra
.608 .707 .915 r
.43203 .40591 .44 .41506 .45361 .41226 .44568 .40312 Metetra
.399 .615 .938 r
.44568 .40312 .45361 .41226 .46722 .4143 .45932 .40505 Metetra
0 .261 .761 r
.45932 .40505 .46722 .4143 .48084 .42439 .47295 .41481 Metetra
0 .282 .779 r
.47295 .41481 .48084 .42439 .49467 .43438 .4868 .42449 Metetra
0 .298 .791 r
.4868 .42449 .49467 .43438 .50873 .44435 .50086 .43415 Metetra
0 .308 .799 r
.50086 .43415 .50873 .44435 .52301 .45436 .51515 .44386 Metetra
0 .314 .804 r
.51515 .44386 .52301 .45436 .53752 .46448 .52966 .4537 Metetra
0 .317 .807 r
.52966 .4537 .53752 .46448 .55225 .47476 .5444 .46372 Metetra
0 .318 .807 r
.5444 .46372 .55225 .47476 .56722 .48526 .55938 .47397 Metetra
0 .316 .806 r
.55938 .47397 .56722 .48526 .58244 .49601 .5746 .4845 Metetra
0 .313 .804 r
.5746 .4845 .58244 .49601 .59791 .50707 .59007 .49535 Metetra
0 .308 .801 r
.59007 .49535 .59791 .50707 .61364 .51846 .6058 .50657 Metetra
0 .302 .797 r
.6058 .50657 .61364 .51846 .62964 .53023 .62181 .51817 Metetra
0 .295 .793 r
.62181 .51817 .62964 .53023 .64592 .54239 .63809 .5302 Metetra
0 .288 .788 r
.63809 .5302 .64592 .54239 .6625 .55498 .65467 .54267 Metetra
0 .28 .783 r
.65467 .54267 .6625 .55498 .67938 .56802 .67156 .5556 Metetra
0 .272 .778 r
.67156 .5556 .67938 .56802 .69657 .58152 .68876 .56902 Metetra
0 .264 .773 r
.68876 .56902 .69657 .58152 .7141 .59551 .70629 .58295 Metetra
0 .256 .767 r
.70629 .58295 .7141 .59551 .73197 .60998 .72417 .59738 Metetra
0 .248 .762 r
.72417 .59738 .73197 .60998 .75019 .62497 .74241 .61233 Metetra
0 .24 .757 r
.74241 .61233 .75019 .62497 .76878 .64047 .76102 .62782 Metetra
0 .233 .751 r
.76102 .62782 .76878 .64047 .78775 .65651 .78001 .64385 Metetra
0 .225 .746 r
.78001 .64385 .78775 .65651 .80713 .67307 .7994 .66043 Metetra
0 .218 .741 r
.7994 .66043 .80713 .67307 .82691 .69019 .81921 .67756 Metetra
0 .211 .736 r
.81921 .67756 .82691 .69019 .84713 .70785 .83946 .69525 Metetra
0 .205 .731 r
.83946 .69525 .84713 .70785 .86778 .72608 .86015 .71352 Metetra
.634 .707 .899 r
.25393 .43482 .26241 .44388 .275 .44056 .26655 .43145 Metetra
.63 .705 .9 r
.26655 .43145 .275 .44056 .28765 .43734 .27924 .42818 Metetra
.626 .704 .901 r
.27924 .42818 .28765 .43734 .30038 .43422 .292 .42501 Metetra
.623 .703 .903 r
.292 .42501 .30038 .43422 .31318 .43117 .30483 .42193 Metetra
.619 .702 .904 r
.30483 .42193 .31318 .43117 .32606 .4282 .31774 .41894 Metetra
.616 .701 .905 r
.31774 .41894 .32606 .4282 .33901 .4253 .33072 .41602 Metetra
.613 .701 .907 r
.33072 .41602 .33901 .4253 .35204 .42246 .34379 .41317 Metetra
.611 .701 .908 r
.34379 .41317 .35204 .42246 .36515 .41966 .35693 .41037 Metetra
.609 .701 .909 r
.35693 .41037 .36515 .41966 .37835 .4169 .37016 .40761 Metetra
.607 .702 .911 r
.37016 .40761 .37835 .4169 .39164 .41416 .38348 .40487 Metetra
.606 .703 .912 r
.38348 .40487 .39164 .41416 .40501 .41142 .39689 .40214 Metetra
.606 .704 .913 r
.39689 .40214 .40501 .41142 .41847 .40868 .41039 .39941 Metetra
.605 .705 .914 r
.41039 .39941 .41847 .40868 .43203 .40591 .42399 .39666 Metetra
.606 .707 .915 r
.42399 .39666 .43203 .40591 .44568 .40312 .43767 .39388 Metetra
.403 .617 .938 r
.43767 .39388 .44568 .40312 .45932 .40505 .45134 .39569 Metetra
0 .28 .775 r
.45134 .39569 .45932 .40505 .47295 .41481 .46498 .40511 Metetra
0 .3 .791 r
.46498 .40511 .47295 .41481 .4868 .42449 .47884 .41445 Metetra
0 .314 .802 r
.47884 .41445 .4868 .42449 .50086 .43415 .49292 .42379 Metetra
0 .323 .809 r
.49292 .42379 .50086 .43415 .51515 .44386 .50721 .43319 Metetra
0 .328 .813 r
.50721 .43319 .51515 .44386 .52966 .4537 .52173 .44273 Metetra
0 .329 .814 r
.52173 .44273 .52966 .4537 .5444 .46372 .53648 .45248 Metetra
.002 .329 .814 r
.53648 .45248 .5444 .46372 .55938 .47397 .55146 .46247 Metetra
.005 .326 .812 r
.55146 .46247 .55938 .47397 .5746 .4845 .56668 .47277 Metetra
.005 .321 .809 r
.56668 .47277 .5746 .4845 .59007 .49535 .58216 .48341 Metetra
.002 .315 .805 r
.58216 .48341 .59007 .49535 .6058 .50657 .59789 .49443 Metetra
0 .308 .8 r
.59789 .49443 .6058 .50657 .62181 .51817 .6139 .50587 Metetra
0 .301 .796 r
.6139 .50587 .62181 .51817 .63809 .5302 .63019 .51776 Metetra
0 .292 .79 r
.63019 .51776 .63809 .5302 .65467 .54267 .64677 .53011 Metetra
0 .284 .785 r
.64677 .53011 .65467 .54267 .67156 .5556 .66366 .54295 Metetra
0 .275 .78 r
.66366 .54295 .67156 .5556 .68876 .56902 .68087 .55629 Metetra
0 .266 .774 r
.68087 .55629 .68876 .56902 .70629 .58295 .69841 .57016 Metetra
0 .258 .768 r
.69841 .57016 .70629 .58295 .72417 .59738 .7163 .58455 Metetra
0 .249 .763 r
.7163 .58455 .72417 .59738 .74241 .61233 .73455 .59948 Metetra
0 .241 .757 r
.73455 .59948 .74241 .61233 .76102 .62782 .75317 .61496 Metetra
0 .232 .751 r
.75317 .61496 .76102 .62782 .78001 .64385 .77218 .63099 Metetra
0 .225 .746 r
.77218 .63099 .78001 .64385 .7994 .66043 .7916 .64758 Metetra
0 .217 .74 r
.7916 .64758 .7994 .66043 .81921 .67756 .81143 .66474 Metetra
0 .21 .735 r
.81143 .66474 .81921 .67756 .83946 .69525 .83171 .68247 Metetra
0 .203 .73 r
.83171 .68247 .83946 .69525 .86015 .71352 .85243 .70078 Metetra
.634 .706 .899 r
.24537 .42565 .25393 .43482 .26655 .43145 .25803 .42222 Metetra
.63 .705 .9 r
.25803 .42222 .26655 .43145 .27924 .42818 .27075 .4189 Metetra
.626 .703 .901 r
.27075 .4189 .27924 .42818 .292 .42501 .28355 .41569 Metetra
.622 .702 .902 r
.28355 .41569 .292 .42501 .30483 .42193 .29641 .41258 Metetra
.619 .701 .904 r
.29641 .41258 .30483 .42193 .31774 .41894 .30935 .40956 Metetra
.615 .7 .905 r
.30935 .40956 .31774 .41894 .33072 .41602 .32236 .40662 Metetra
.612 .7 .906 r
.32236 .40662 .33072 .41602 .34379 .41317 .33546 .40376 Metetra
.61 .7 .908 r
.33546 .40376 .34379 .41317 .35693 .41037 .34864 .40096 Metetra
.607 .7 .909 r
.34864 .40096 .35693 .41037 .37016 .40761 .3619 .3982 Metetra
.606 .7 .911 r
.3619 .3982 .37016 .40761 .38348 .40487 .37525 .39547 Metetra
.604 .701 .912 r
.37525 .39547 .38348 .40487 .39689 .40214 .3887 .39276 Metetra
.604 .702 .913 r
.3887 .39276 .39689 .40214 .41039 .39941 .40223 .39005 Metetra
.603 .704 .915 r
.40223 .39005 .41039 .39941 .42399 .39666 .41586 .38732 Metetra
.603 .706 .916 r
.41586 .38732 .42399 .39666 .43767 .39388 .42959 .38455 Metetra
.408 .62 .938 r
.42959 .38455 .43767 .39388 .45134 .39569 .44328 .38624 Metetra
0 .299 .789 r
.44328 .38624 .45134 .39569 .46498 .40511 .45694 .39529 Metetra
0 .318 .804 r
.45694 .39529 .46498 .40511 .47884 .41445 .47081 .40427 Metetra
0 .33 .814 r
.47081 .40427 .47884 .41445 .49292 .42379 .4849 .41326 Metetra
0 .338 .819 r
.4849 .41326 .49292 .42379 .50721 .43319 .4992 .42233 Metetra
.006 .342 .822 r
.4992 .42233 .50721 .43319 .52173 .44273 .51373 .43156 Metetra
.017 .342 .822 r
.51373 .43156 .52173 .44273 .53648 .45248 .52848 .44101 Metetra
.022 .34 .82 r
.52848 .44101 .53648 .45248 .55146 .46247 .54347 .45074 Metetra
.023 .335 .817 r
.54347 .45074 .55146 .46247 .56668 .47277 .55869 .46079 Metetra
.021 .329 .813 r
.55869 .46079 .56668 .47277 .58216 .48341 .57417 .47122 Metetra
.016 .322 .809 r
.57417 .47122 .58216 .48341 .59789 .49443 .58991 .48205 Metetra
.009 .314 .804 r
.58991 .48205 .59789 .49443 .6139 .50587 .60592 .49332 Metetra
0 .306 .798 r
.60592 .49332 .6139 .50587 .63019 .51776 .62221 .50507 Metetra
0 .296 .793 r
.62221 .50507 .63019 .51776 .64677 .53011 .63879 .5173 Metetra
0 .287 .787 r
.63879 .5173 .64677 .53011 .66366 .54295 .65569 .53005 Metetra
0 .278 .781 r
.65569 .53005 .66366 .54295 .68087 .55629 .6729 .54332 Metetra
0 .268 .775 r
.6729 .54332 .68087 .55629 .69841 .57016 .69045 .55713 Metetra
0 .259 .769 r
.69045 .55713 .69841 .57016 .7163 .58455 .70835 .57149 Metetra
0 .249 .763 r
.70835 .57149 .7163 .58455 .73455 .59948 .72661 .5864 Metetra
0 .241 .757 r
.72661 .5864 .73455 .59948 .75317 .61496 .74524 .60188 Metetra
0 .232 .751 r
.74524 .60188 .75317 .61496 .77218 .63099 .76427 .61793 Metetra
0 .224 .745 r
.76427 .61793 .77218 .63099 .7916 .64758 .78371 .63454 Metetra
0 .216 .74 r
.78371 .63454 .7916 .64758 .81143 .66474 .80357 .65174 Metetra
0 .208 .734 r
.80357 .65174 .81143 .66474 .83171 .68247 .82387 .66951 Metetra
0 .201 .729 r
.82387 .66951 .83171 .68247 .85243 .70078 .84463 .68787 Metetra
.635 .706 .898 r
.23674 .41638 .24537 .42565 .25803 .42222 .24943 .41289 Metetra
.631 .704 .899 r
.24943 .41289 .25803 .42222 .27075 .4189 .26219 .40951 Metetra
.626 .703 .9 r
.26219 .40951 .27075 .4189 .28355 .41569 .27502 .40625 Metetra
.622 .701 .902 r
.27502 .40625 .28355 .41569 .29641 .41258 .28791 .4031 Metetra
.618 .7 .903 r
.28791 .4031 .29641 .41258 .30935 .40956 .30088 .40005 Metetra
.615 .699 .904 r
.30088 .40005 .30935 .40956 .32236 .40662 .31393 .3971 Metetra
.611 .699 .906 r
.31393 .3971 .32236 .40662 .33546 .40376 .32705 .39422 Metetra
.608 .698 .908 r
.32705 .39422 .33546 .40376 .34864 .40096 .34026 .39142 Metetra
.606 .699 .909 r
.34026 .39142 .34864 .40096 .3619 .3982 .35356 .38867 Metetra
.604 .699 .911 r
.35356 .38867 .3619 .3982 .37525 .39547 .36694 .38595 Metetra
.602 .7 .912 r
.36694 .38595 .37525 .39547 .3887 .39276 .38042 .38326 Metetra
.601 .701 .914 r
.38042 .38326 .3887 .39276 .40223 .39005 .39399 .38057 Metetra
.601 .703 .915 r
.39399 .38057 .40223 .39005 .41586 .38732 .40766 .37786 Metetra
.601 .704 .916 r
.40766 .37786 .41586 .38732 .42959 .38455 .42142 .37513 Metetra
.414 .622 .938 r
.42142 .37513 .42959 .38455 .44328 .38624 .43514 .37668 Metetra
0 .32 .804 r
.43514 .37668 .44328 .38624 .45694 .39529 .44882 .38534 Metetra
0 .337 .817 r
.44882 .38534 .45694 .39529 .47081 .40427 .46271 .39393 Metetra
0 .348 .825 r
.46271 .39393 .47081 .40427 .4849 .41326 .47681 .40255 Metetra
.017 .354 .829 r
.47681 .40255 .4849 .41326 .4992 .42233 .49112 .41127 Metetra
.031 .356 .83 r
.49112 .41127 .4992 .42233 .51373 .43156 .50565 .42017 Metetra
.039 .355 .829 r
.50565 .42017 .51373 .43156 .52848 .44101 .52041 .42931 Metetra
.043 .351 .826 r
.52041 .42931 .52848 .44101 .54347 .45074 .5354 .43876 Metetra
.042 .345 .822 r
.5354 .43876 .54347 .45074 .55869 .46079 .55063 .44856 Metetra
.038 .338 .818 r
.55063 .44856 .55869 .46079 .57417 .47122 .56611 .45876 Metetra
.031 .329 .813 r
.56611 .45876 .57417 .47122 .58991 .48205 .58185 .4694 Metetra
.023 .32 .807 r
.58185 .4694 .58991 .48205 .60592 .49332 .59785 .48051 Metetra
.012 .31 .801 r
.59785 .48051 .60592 .49332 .62221 .50507 .61415 .49211 Metetra
0 .3 .795 r
.61415 .49211 .62221 .50507 .63879 .5173 .63073 .50423 Metetra
0 .29 .788 r
.63073 .50423 .63879 .5173 .65569 .53005 .64763 .51689 Metetra
0 .28 .782 r
.64763 .51689 .65569 .53005 .6729 .54332 .66485 .5301 Metetra
0 .269 .776 r
.66485 .5301 .6729 .54332 .69045 .55713 .6824 .54386 Metetra
0 .259 .769 r
.6824 .54386 .69045 .55713 .70835 .57149 .70031 .5582 Metetra
0 .249 .763 r
.70031 .5582 .70835 .57149 .72661 .5864 .71858 .57311 Metetra
0 .24 .756 r
.71858 .57311 .72661 .5864 .74524 .60188 .73723 .58859 Metetra
0 .231 .75 r
.73723 .58859 .74524 .60188 .76427 .61793 .75628 .60466 Metetra
0 .222 .744 r
.75628 .60466 .76427 .61793 .78371 .63454 .77574 .62131 Metetra
0 .214 .738 r
.77574 .62131 .78371 .63454 .80357 .65174 .79562 .63855 Metetra
0 .206 .733 r
.79562 .63855 .80357 .65174 .82387 .66951 .81595 .65637 Metetra
0 .199 .727 r
.81595 .65637 .82387 .66951 .84463 .68787 .83674 .67479 Metetra
.636 .706 .897 r
.22803 .407 .23674 .41638 .24943 .41289 .24076 .40344 Metetra
.631 .704 .898 r
.24076 .40344 .24943 .41289 .26219 .40951 .25355 .4 Metetra
.627 .702 .9 r
.25355 .4 .26219 .40951 .27502 .40625 .26641 .39669 Metetra
.622 .7 .901 r
.26641 .39669 .27502 .40625 .28791 .4031 .27934 .39349 Metetra
.618 .699 .902 r
.27934 .39349 .28791 .4031 .30088 .40005 .29234 .39041 Metetra
.614 .698 .904 r
.29234 .39041 .30088 .40005 .31393 .3971 .30542 .38744 Metetra
.61 .697 .906 r
.30542 .38744 .31393 .3971 .32705 .39422 .31857 .38455 Metetra
.607 .697 .907 r
.31857 .38455 .32705 .39422 .34026 .39142 .33181 .38175 Metetra
.604 .697 .909 r
.33181 .38175 .34026 .39142 .35356 .38867 .34514 .37901 Metetra
.602 .697 .911 r
.34514 .37901 .35356 .38867 .36694 .38595 .35856 .37631 Metetra
.6 .698 .912 r
.35856 .37631 .36694 .38595 .38042 .38326 .37206 .37364 Metetra
.599 .7 .914 r
.37206 .37364 .38042 .38326 .39399 .38057 .38567 .37098 Metetra
.598 .701 .915 r
.38567 .37098 .39399 .38057 .40766 .37786 .39937 .3683 Metetra
.598 .703 .917 r
.39937 .3683 .40766 .37786 .42142 .37513 .41317 .3656 Metetra
.419 .625 .938 r
.41317 .3656 .42142 .37513 .43514 .37668 .42692 .36701 Metetra
0 .342 .818 r
.42692 .36701 .43514 .37668 .44882 .38534 .44062 .37525 Metetra
0 .357 .83 r
.44062 .37525 .44882 .38534 .46271 .39393 .45452 .38343 Metetra
.025 .366 .836 r
.45452 .38343 .46271 .39393 .47681 .40255 .46863 .39165 Metetra
.044 .37 .839 r
.46863 .39165 .47681 .40255 .49112 .41127 .48296 .39999 Metetra
.057 .371 .838 r
.48296 .39999 .49112 .41127 .50565 .42017 .4975 .40854 Metetra
.063 .367 .836 r
.4975 .40854 .50565 .42017 .52041 .42931 .51227 .41736 Metetra
.064 .362 .832 r
.51227 .41736 .52041 .42931 .5354 .43876 .52726 .42651 Metetra
.062 .355 .827 r
.52726 .42651 .5354 .43876 .55063 .44856 .54249 .43605 Metetra
.056 .346 .822 r
.54249 .43605 .55063 .44856 .56611 .45876 .55797 .44602 Metetra
.047 .336 .816 r
.55797 .44602 .56611 .45876 .58185 .4694 .57371 .45646 Metetra
.036 .326 .81 r
.57371 .45646 .58185 .4694 .59785 .48051 .58972 .4674 Metetra
.024 .315 .803 r
.58972 .4674 .59785 .48051 .61415 .49211 .60601 .47887 Metetra
.01 .304 .797 r
.60601 .47887 .61415 .49211 .63073 .50423 .6226 .49089 Metetra
0 .293 .79 r
.6226 .49089 .63073 .50423 .64763 .51689 .6395 .50346 Metetra
0 .281 .783 r
.6395 .50346 .64763 .51689 .66485 .5301 .65672 .51661 Metetra
0 .27 .776 r
.65672 .51661 .66485 .5301 .6824 .54386 .67428 .53035 Metetra
0 .259 .769 r
.67428 .53035 .6824 .54386 .70031 .5582 .69219 .54467 Metetra
0 .249 .762 r
.69219 .54467 .70031 .5582 .71858 .57311 .71048 .55958 Metetra
0 .239 .756 r
.71048 .55958 .71858 .57311 .73723 .58859 .72914 .57508 Metetra
0 .229 .749 r
.72914 .57508 .73723 .58859 .75628 .60466 .7482 .59118 Metetra
0 .22 .743 r
.7482 .59118 .75628 .60466 .77574 .62131 .76768 .60788 Metetra
0 .212 .737 r
.76768 .60788 .77574 .62131 .79562 .63855 .78759 .62517 Metetra
0 .204 .731 r
.78759 .62517 .79562 .63855 .81595 .65637 .80795 .64306 Metetra
0 .197 .726 r
.80795 .64306 .81595 .65637 .83674 .67479 .82877 .66155 Metetra
.637 .706 .896 r
.21924 .39752 .22803 .407 .24076 .40344 .232 .39388 Metetra
.632 .704 .898 r
.232 .39388 .24076 .40344 .25355 .4 .24483 .39037 Metetra
.627 .701 .899 r
.24483 .39037 .25355 .4 .26641 .39669 .25772 .387 Metetra
.622 .7 .9 r
.25772 .387 .26641 .39669 .27934 .39349 .27068 .38375 Metetra
.618 .698 .902 r
.27068 .38375 .27934 .39349 .29234 .39041 .28372 .38064 Metetra
.613 .697 .903 r
.28372 .38064 .29234 .39041 .30542 .38744 .29683 .37764 Metetra
.609 .696 .905 r
.29683 .37764 .30542 .38744 .31857 .38455 .31001 .37475 Metetra
.605 .695 .907 r
.31001 .37475 .31857 .38455 .33181 .38175 .32328 .37194 Metetra
.602 .695 .909 r
.32328 .37194 .33181 .38175 .34514 .37901 .33664 .36921 Metetra
.6 .696 .91 r
.33664 .36921 .34514 .37901 .35856 .37631 .35009 .36654 Metetra
.597 .697 .912 r
.35009 .36654 .35856 .37631 .37206 .37364 .36363 .3639 Metetra
.596 .698 .914 r
.36363 .3639 .37206 .37364 .38567 .37098 .37726 .36127 Metetra
.595 .7 .916 r
.37726 .36127 .38567 .37098 .39937 .3683 .391 .35864 Metetra
.596 .702 .917 r
.391 .35864 .39937 .3683 .41317 .3656 .40483 .35597 Metetra
.425 .628 .938 r
.40483 .35597 .41317 .3656 .42692 .36701 .41862 .35723 Metetra
0 .365 .834 r
.41862 .35723 .42692 .36701 .44062 .37525 .43234 .36501 Metetra
.03 .378 .843 r
.43234 .36501 .44062 .37525 .45452 .38343 .44626 .37275 Metetra
.056 .386 .847 r
.44626 .37275 .45452 .38343 .46863 .39165 .46039 .38054 Metetra
.073 .388 .848 r
.46039 .38054 .46863 .39165 .48296 .39999 .47473 .38848 Metetra
.084 .386 .846 r
.47473 .38848 .48296 .39999 .4975 .40854 .48928 .39666 Metetra
.088 .381 .842 r
.48928 .39666 .4975 .40854 .51227 .41736 .50405 .40513 Metetra
.087 .373 .838 r
.50405 .40513 .51227 .41736 .52726 .42651 .51905 .41398 Metetra
.082 .364 .832 r
.51905 .41398 .52726 .42651 .54249 .43605 .53428 .42324 Metetra
.074 .354 .826 r
.53428 .42324 .54249 .43605 .55797 .44602 .54976 .43298 Metetra
.063 .343 .819 r
.54976 .43298 .55797 .44602 .57371 .45646 .5655 .44322 Metetra
.05 .331 .812 r
.5655 .44322 .57371 .45646 .58972 .4674 .58151 .454 Metetra
.035 .319 .805 r
.58151 .454 .58972 .4674 .60601 .47887 .5978 .46533 Metetra
.019 .307 .798 r
.5978 .46533 .60601 .47887 .6226 .49089 .61439 .47725 Metetra
.003 .295 .791 r
.61439 .47725 .6226 .49089 .6395 .50346 .63128 .48975 Metetra
0 .282 .783 r
.63128 .48975 .6395 .50346 .65672 .51661 .64851 .50286 Metetra
0 .271 .776 r
.64851 .50286 .65672 .51661 .67428 .53035 .66607 .51657 Metetra
0 .259 .769 r
.66607 .51657 .67428 .53035 .69219 .54467 .68399 .53089 Metetra
0 .248 .762 r
.68399 .53089 .69219 .54467 .71048 .55958 .70229 .54582 Metetra
0 .237 .755 r
.70229 .54582 .71048 .55958 .72914 .57508 .72096 .56136 Metetra
0 .228 .748 r
.72096 .56136 .72914 .57508 .7482 .59118 .74004 .57751 Metetra
0 .218 .741 r
.74004 .57751 .7482 .59118 .76768 .60788 .75954 .59426 Metetra
0 .209 .735 r
.75954 .59426 .76768 .60788 .78759 .62517 .77947 .61162 Metetra
0 .201 .729 r
.77947 .61162 .78759 .62517 .80795 .64306 .79986 .62958 Metetra
0 .194 .724 r
.79986 .62958 .80795 .64306 .82877 .66155 .82071 .64815 Metetra
.638 .706 .896 r
.21037 .38792 .21924 .39752 .232 .39388 .22317 .3842 Metetra
.633 .703 .897 r
.22317 .3842 .232 .39388 .24483 .39037 .23603 .38061 Metetra
.628 .701 .898 r
.23603 .38061 .24483 .39037 .25772 .387 .24896 .37717 Metetra
.623 .699 .899 r
.24896 .37717 .25772 .387 .27068 .38375 .26195 .37387 Metetra
.617 .697 .901 r
.26195 .37387 .27068 .38375 .28372 .38064 .27502 .37072 Metetra
.613 .695 .903 r
.27502 .37072 .28372 .38064 .29683 .37764 .28816 .3677 Metetra
.608 .694 .904 r
.28816 .3677 .29683 .37764 .31001 .37475 .30138 .36479 Metetra
.604 .694 .906 r
.30138 .36479 .31001 .37475 .32328 .37194 .31468 .36199 Metetra
.6 .693 .908 r
.31468 .36199 .32328 .37194 .33664 .36921 .32806 .35928 Metetra
.597 .694 .91 r
.32806 .35928 .33664 .36921 .35009 .36654 .34154 .35663 Metetra
.595 .695 .912 r
.34154 .35663 .35009 .36654 .36363 .3639 .35511 .35403 Metetra
.593 .696 .914 r
.35511 .35403 .36363 .3639 .37726 .36127 .36878 .35144 Metetra
.592 .698 .916 r
.36878 .35144 .37726 .36127 .391 .35864 .38255 .34885 Metetra
.592 .7 .918 r
.38255 .34885 .391 .35864 .40483 .35597 .39642 .34624 Metetra
.431 .631 .938 r
.39642 .34624 .40483 .35597 .41862 .35723 .41024 .34734 Metetra
.029 .389 .849 r
.41024 .34734 .41862 .35723 .43234 .36501 .42398 .35462 Metetra
.065 .401 .856 r
.42398 .35462 .43234 .36501 .44626 .37275 .43792 .36188 Metetra
.089 .405 .858 r
.43792 .36188 .44626 .37275 .46039 .38054 .45207 .36921 Metetra
.105 .405 .857 r
.45207 .36921 .46039 .38054 .47473 .38848 .46642 .37672 Metetra
.112 .401 .853 r
.46642 .37672 .47473 .38848 .48928 .39666 .48098 .38449 Metetra
.114 .394 .848 r
.48098 .38449 .48928 .39666 .50405 .40513 .49576 .39261 Metetra
.111 .385 .842 r
.49576 .39261 .50405 .40513 .51905 .41398 .51076 .40113 Metetra
.103 .374 .836 r
.51076 .40113 .51905 .41398 .53428 .42324 .526 .41011 Metetra
.093 .362 .829 r
.526 .41011 .53428 .42324 .54976 .43298 .54148 .41961 Metetra
.079 .349 .822 r
.54148 .41961 .54976 .43298 .5655 .44322 .55721 .42965 Metetra
.064 .336 .814 r
.55721 .42965 .5655 .44322 .58151 .454 .57322 .44026 Metetra
.047 .323 .806 r
.57322 .44026 .58151 .454 .5978 .46533 .58951 .45147 Metetra
.029 .309 .799 r
.58951 .45147 .5978 .46533 .61439 .47725 .60609 .4633 Metetra
.01 .296 .791 r
.60609 .4633 .61439 .47725 .63128 .48975 .62299 .47575 Metetra
0 .283 .783 r
.62299 .47575 .63128 .48975 .64851 .50286 .64022 .48883 Metetra
0 .27 .776 r
.64022 .48883 .64851 .50286 .66607 .51657 .65779 .50253 Metetra
0 .258 .768 r
.65779 .50253 .66607 .51657 .68399 .53089 .67571 .51687 Metetra
0 .246 .76 r
.67571 .51687 .68399 .53089 .70229 .54582 .69401 .53183 Metetra
0 .235 .753 r
.69401 .53183 .70229 .54582 .72096 .56136 .7127 .54742 Metetra
0 .225 .746 r
.7127 .54742 .72096 .56136 .74004 .57751 .7318 .56363 Metetra
0 .215 .739 r
.7318 .56363 .74004 .57751 .75954 .59426 .75131 .58046 Metetra
0 .206 .733 r
.75131 .58046 .75954 .59426 .77947 .61162 .77127 .59789 Metetra
0 .198 .727 r
.77127 .59789 .77947 .61162 .79986 .62958 .79168 .61594 Metetra
0 .19 .721 r
.79168 .61594 .79986 .62958 .82071 .64815 .81256 .6346 Metetra
.639 .706 .895 r
.20142 .37822 .21037 .38792 .22317 .3842 .21426 .3744 Metetra
.634 .703 .896 r
.21426 .3744 .22317 .3842 .23603 .38061 .22716 .37072 Metetra
.629 .7 .897 r
.22716 .37072 .23603 .38061 .24896 .37717 .24012 .36721 Metetra
.623 .698 .898 r
.24012 .36721 .24896 .37717 .26195 .37387 .25315 .36385 Metetra
.617 .696 .9 r
.25315 .36385 .26195 .37387 .27502 .37072 .26624 .36065 Metetra
.612 .694 .902 r
.26624 .36065 .27502 .37072 .28816 .3677 .27941 .35759 Metetra
.607 .692 .904 r
.27941 .35759 .28816 .3677 .30138 .36479 .29266 .35468 Metetra
.602 .691 .906 r
.29266 .35468 .30138 .36479 .31468 .36199 .30599 .35188 Metetra
.598 .691 .908 r
.30599 .35188 .31468 .36199 .32806 .35928 .31941 .34919 Metetra
.595 .691 .91 r
.31941 .34919 .32806 .35928 .34154 .35663 .33291 .34657 Metetra
.592 .692 .912 r
.33291 .34657 .34154 .35663 .35511 .35403 .34651 .34401 Metetra
.59 .694 .914 r
.34651 .34401 .35511 .35403 .36878 .35144 .36021 .34148 Metetra
.589 .696 .916 r
.36021 .34148 .36878 .35144 .38255 .34885 .37401 .33895 Metetra
.588 .698 .918 r
.37401 .33895 .38255 .34885 .39642 .34624 .38791 .3364 Metetra
.438 .634 .938 r
.38791 .3364 .39642 .34624 .41024 .34734 .40177 .33733 Metetra
.069 .415 .864 r
.40177 .33733 .41024 .34734 .42398 .35462 .41553 .34407 Metetra
.102 .424 .869 r
.41553 .34407 .42398 .35462 .43792 .36188 .4295 .3508 Metetra
.125 .426 .868 r
.4295 .3508 .43792 .36188 .45207 .36921 .44367 .35763 Metetra
.138 .423 .865 r
.44367 .35763 .45207 .36921 .46642 .37672 .45804 .36467 Metetra
.143 .417 .86 r
.45804 .36467 .46642 .37672 .48098 .38449 .47261 .37202 Metetra
.142 .407 .854 r
.47261 .37202 .48098 .38449 .49576 .39261 .4874 .37975 Metetra
.136 .396 .847 r
.4874 .37975 .49576 .39261 .51076 .40113 .5024 .38793 Metetra
.125 .383 .839 r
.5024 .38793 .51076 .40113 .526 .41011 .51764 .39662 Metetra
.112 .37 .831 r
.51764 .39662 .526 .41011 .54148 .41961 .53312 .40587 Metetra
.096 .355 .823 r
.53312 .40587 .54148 .41961 .55721 .42965 .54885 .41571 Metetra
.078 .341 .815 r
.54885 .41571 .55721 .42965 .57322 .44026 .56485 .42618 Metetra
.058 .326 .807 r
.56485 .42618 .57322 .44026 .58951 .45147 .58114 .43728 Metetra
.038 .311 .799 r
.58114 .43728 .58951 .45147 .60609 .4633 .59772 .44903 Metetra
.017 .297 .791 r
.59772 .44903 .60609 .4633 .62299 .47575 .61462 .46144 Metetra
0 .283 .783 r
.61462 .46144 .62299 .47575 .64022 .48883 .63185 .47451 Metetra
0 .269 .775 r
.63185 .47451 .64022 .48883 .65779 .50253 .64942 .48823 Metetra
0 .256 .767 r
.64942 .48823 .65779 .50253 .67571 .51687 .66735 .5026 Metetra
0 .244 .759 r
.66735 .5026 .67571 .51687 .69401 .53183 .68566 .51762 Metetra
0 .232 .751 r
.68566 .51762 .69401 .53183 .7127 .54742 .70436 .53328 Metetra
0 .222 .744 r
.70436 .53328 .7127 .54742 .7318 .56363 .72347 .54957 Metetra
0 .211 .737 r
.72347 .54957 .7318 .56363 .75131 .58046 .743 .56648 Metetra
0 .202 .73 r
.743 .56648 .75131 .58046 .77127 .59789 .76298 .58401 Metetra
0 .194 .724 r
.76298 .58401 .77127 .59789 .79168 .61594 .78342 .60215 Metetra
0 .186 .718 r
.78342 .60215 .79168 .61594 .81256 .6346 .80433 .62091 Metetra
.641 .706 .894 r
.19239 .36841 .20142 .37822 .21426 .3744 .20527 .36447 Metetra
.635 .703 .895 r
.20527 .36447 .21426 .3744 .22716 .37072 .2182 .3607 Metetra
.63 .7 .896 r
.2182 .3607 .22716 .37072 .24012 .36721 .2312 .3571 Metetra
.624 .697 .897 r
.2312 .3571 .24012 .36721 .25315 .36385 .24426 .35367 Metetra
.618 .694 .899 r
.24426 .35367 .25315 .36385 .26624 .36065 .25739 .35041 Metetra
.612 .692 .901 r
.25739 .35041 .26624 .36065 .27941 .35759 .27059 .34733 Metetra
.606 .69 .902 r
.27059 .34733 .27941 .35759 .29266 .35468 .28387 .34439 Metetra
.601 .689 .905 r
.28387 .34439 .29266 .35468 .30599 .35188 .29723 .3416 Metetra
.596 .689 .907 r
.29723 .3416 .30599 .35188 .31941 .34919 .31067 .33893 Metetra
.592 .689 .909 r
.31067 .33893 .31941 .34919 .33291 .34657 .3242 .33636 Metetra
.588 .69 .912 r
.3242 .33636 .33291 .34657 .34651 .34401 .33783 .33385 Metetra
.586 .691 .914 r
.33783 .33385 .34651 .34401 .36021 .34148 .35156 .33138 Metetra
.585 .693 .917 r
.35156 .33138 .36021 .34148 .37401 .33895 .36539 .32892 Metetra
.584 .696 .919 r
.36539 .32892 .37401 .33895 .38791 .3364 .37932 .32644 Metetra
.445 .637 .938 r
.37932 .32644 .38791 .3364 .40177 .33733 .39321 .3272 Metetra
.111 .442 .879 r
.39321 .3272 .40177 .33733 .41553 .34407 .40701 .33334 Metetra
.143 .448 .881 r
.40701 .33334 .41553 .34407 .4295 .3508 .42101 .33949 Metetra
.163 .447 .878 r
.42101 .33949 .4295 .3508 .44367 .35763 .4352 .34577 Metetra
.173 .441 .872 r
.4352 .34577 .44367 .35763 .45804 .36467 .44959 .35231 Metetra
.175 .432 .866 r
.44959 .35231 .45804 .36467 .47261 .37202 .46417 .35919 Metetra
.171 .42 .858 r
.46417 .35919 .47261 .37202 .4874 .37975 .47897 .36651 Metetra
.161 .407 .85 r
.47897 .36651 .4874 .37975 .5024 .38793 .49398 .37434 Metetra
.148 .392 .842 r
.49398 .37434 .5024 .38793 .51764 .39662 .50921 .38274 Metetra
.131 .377 .833 r
.50921 .38274 .51764 .39662 .53312 .40587 .52469 .39174 Metetra
.112 .361 .825 r
.52469 .39174 .53312 .40587 .54885 .41571 .54042 .40139 Metetra
.091 .345 .816 r
.54042 .40139 .54885 .41571 .56485 .42618 .55642 .41172 Metetra
.069 .328 .807 r
.55642 .41172 .56485 .42618 .58114 .43728 .5727 .42272 Metetra
.046 .312 .799 r
.5727 .42272 .58114 .43728 .59772 .44903 .58928 .43442 Metetra
.023 .297 .79 r
.58928 .43442 .59772 .44903 .61462 .46144 .60617 .44681 Metetra
0 .282 .782 r
.60617 .44681 .61462 .46144 .63185 .47451 .6234 .45989 Metetra
0 .267 .773 r
.6234 .45989 .63185 .47451 .64942 .48823 .64097 .47365 Metetra
0 .254 .765 r
.64097 .47365 .64942 .48823 .66735 .5026 .6589 .48809 Metetra
0 .241 .757 r
.6589 .48809 .66735 .5026 .68566 .51762 .67722 .50319 Metetra
0 .229 .749 r
.67722 .50319 .68566 .51762 .70436 .53328 .69593 .51894 Metetra
0 .217 .741 r
.69593 .51894 .70436 .53328 .72347 .54957 .71505 .53532 Metetra
0 .207 .734 r
.71505 .53532 .72347 .54957 .743 .56648 .73461 .55234 Metetra
0 .198 .727 r
.73461 .55234 .743 .56648 .76298 .58401 .75461 .56998 Metetra
0 .189 .721 r
.75461 .56998 .76298 .58401 .78342 .60215 .77507 .58823 Metetra
0 .181 .715 r
.77507 .58823 .78342 .60215 .80433 .62091 .79602 .60709 Metetra
.643 .706 .893 r
.18327 .35849 .19239 .36841 .20527 .36447 .19619 .35443 Metetra
.637 .703 .894 r
.19619 .35443 .20527 .36447 .2182 .3607 .20917 .35055 Metetra
.631 .699 .895 r
.20917 .35055 .2182 .3607 .2312 .3571 .2222 .34684 Metetra
.625 .696 .896 r
.2222 .34684 .2312 .3571 .24426 .35367 .2353 .34333 Metetra
.618 .693 .897 r
.2353 .34333 .24426 .35367 .25739 .35041 .24846 .34001 Metetra
.612 .69 .899 r
.24846 .34001 .25739 .35041 .27059 .34733 .26169 .33688 Metetra
.605 .688 .901 r
.26169 .33688 .27059 .34733 .28387 .34439 .275 .33392 Metetra
.599 .687 .903 r
.275 .33392 .28387 .34439 .29723 .3416 .28838 .33114 Metetra
.593 .686 .906 r
.28838 .33114 .29723 .3416 .31067 .33893 .30185 .32849 Metetra
.589 .686 .909 r
.30185 .32849 .31067 .33893 .3242 .33636 .31541 .32596 Metetra
.585 .686 .911 r
.31541 .32596 .3242 .33636 .33783 .33385 .32907 .32352 Metetra
.582 .688 .914 r
.32907 .32352 .33783 .33385 .35156 .33138 .34282 .32114 Metetra
.58 .69 .917 r
.34282 .32114 .35156 .33138 .36539 .32892 .35668 .31877 Metetra
.579 .693 .92 r
.35668 .31877 .36539 .32892 .37932 .32644 .37064 .31638 Metetra
.452 .64 .937 r
.37064 .31638 .37932 .32644 .39321 .3272 .38457 .31694 Metetra
.158 .471 .893 r
.38457 .31694 .39321 .3272 .40701 .33334 .39841 .32241 Metetra
.187 .473 .891 r
.39841 .32241 .40701 .33334 .42101 .33949 .41244 .32792 Metetra
.204 .469 .886 r
.41244 .32792 .42101 .33949 .4352 .34577 .42666 .3336 Metetra
.21 .46 .878 r
.42666 .3336 .4352 .34577 .44959 .35231 .44106 .33959 Metetra
.209 .448 .87 r
.44106 .33959 .44959 .35231 .46417 .35919 .45566 .34597 Metetra
.201 .433 .861 r
.45566 .34597 .46417 .35919 .47897 .36651 .47047 .35286 Metetra
.188 .417 .852 r
.47047 .35286 .47897 .36651 .49398 .37434 .48548 .36032 Metetra
.171 .401 .843 r
.48548 .36032 .49398 .37434 .50921 .38274 .50072 .36841 Metetra
.151 .383 .834 r
.50072 .36841 .50921 .38274 .52469 .39174 .51619 .37717 Metetra
.128 .365 .825 r
.51619 .37717 .52469 .39174 .54042 .40139 .53191 .38665 Metetra
.104 .348 .816 r
.53191 .38665 .54042 .40139 .55642 .41172 .54791 .39685 Metetra
.079 .33 .807 r
.54791 .39685 .55642 .41172 .5727 .42272 .56418 .40778 Metetra
.053 .313 .798 r
.56418 .40778 .5727 .42272 .58928 .43442 .58075 .41945 Metetra
.027 .296 .789 r
.58075 .41945 .58928 .43442 .60617 .44681 .59764 .43186 Metetra
.002 .28 .78 r
.59764 .43186 .60617 .44681 .6234 .45989 .61486 .44498 Metetra
0 .264 .771 r
.61486 .44498 .6234 .45989 .64097 .47365 .63244 .45882 Metetra
0 .25 .762 r
.63244 .45882 .64097 .47365 .6589 .48809 .65037 .47334 Metetra
0 .236 .754 r
.65037 .47334 .6589 .48809 .67722 .50319 .66869 .48854 Metetra
0 .224 .745 r
.66869 .48854 .67722 .50319 .69593 .51894 .68742 .50441 Metetra
0 .212 .737 r
.68742 .50441 .69593 .51894 .71505 .53532 .70655 .52092 Metetra
0 .202 .73 r
.70655 .52092 .71505 .53532 .73461 .55234 .72613 .53806 Metetra
0 .192 .723 r
.72613 .53806 .73461 .55234 .75461 .56998 .74615 .55582 Metetra
0 .184 .717 r
.74615 .55582 .75461 .56998 .77507 .58823 .76664 .5742 Metetra
0 .176 .711 r
.76664 .5742 .77507 .58823 .79602 .60709 .78761 .59317 Metetra
.646 .707 .892 r
.17407 .34847 .18327 .35849 .19619 .35443 .18703 .34428 Metetra
.64 .703 .892 r
.18703 .34428 .19619 .35443 .20917 .35055 .20005 .34026 Metetra
.633 .699 .893 r
.20005 .34026 .20917 .35055 .2222 .34684 .21313 .33644 Metetra
.626 .696 .894 r
.21313 .33644 .2222 .34684 .2353 .34333 .22626 .33282 Metetra
.619 .692 .896 r
.22626 .33282 .2353 .34333 .24846 .34001 .23946 .32942 Metetra
.612 .689 .898 r
.23946 .32942 .24846 .34001 .26169 .33688 .25272 .32623 Metetra
.604 .686 .9 r
.25272 .32623 .26169 .33688 .275 .33392 .26606 .32325 Metetra
.597 .684 .902 r
.26606 .32325 .275 .33392 .28838 .33114 .27947 .32046 Metetra
.591 .682 .905 r
.27947 .32046 .28838 .33114 .30185 .32849 .29296 .31785 Metetra
.585 .682 .908 r
.29296 .31785 .30185 .32849 .31541 .32596 .30654 .31538 Metetra
.58 .683 .911 r
.30654 .31538 .31541 .32596 .32907 .32352 .32022 .31302 Metetra
.577 .684 .914 r
.32022 .31302 .32907 .32352 .34282 .32114 .334 .31072 Metetra
.574 .687 .917 r
.334 .31072 .34282 .32114 .35668 .31877 .34788 .30846 Metetra
.573 .69 .92 r
.34788 .30846 .35668 .31877 .37064 .31638 .36187 .30619 Metetra
.461 .644 .937 r
.36187 .30619 .37064 .31638 .38457 .31694 .37583 .30654 Metetra
.208 .502 .906 r
.37583 .30654 .38457 .31694 .39841 .32241 .38972 .31127 Metetra
.235 .5 .901 r
.38972 .31127 .39841 .32241 .41244 .32792 .40379 .31606 Metetra
.247 .491 .892 r
.40379 .31606 .41244 .32792 .42666 .3336 .41804 .32107 Metetra
.25 .478 .883 r
.41804 .32107 .42666 .3336 .44106 .33959 .43248 .32645 Metetra
.244 .463 .873 r
.43248 .32645 .44106 .33959 .45566 .34597 .44709 .3323 Metetra
.232 .446 .862 r
.44709 .3323 .45566 .34597 .47047 .35286 .4619 .33872 Metetra
.215 .427 .853 r
.4619 .33872 .47047 .35286 .48548 .36032 .47692 .34579 Metetra
.194 .408 .843 r
.47692 .34579 .48548 .36032 .50072 .36841 .49216 .35358 Metetra
.171 .388 .833 r
.49216 .35358 .50072 .36841 .51619 .37717 .50762 .36212 Metetra
.144 .369 .824 r
.50762 .36212 .51619 .37717 .53191 .38665 .52334 .37143 Metetra
.117 .349 .815 r
.52334 .37143 .53191 .38665 .54791 .39685 .53932 .38154 Metetra
.088 .33 .805 r
.53932 .38154 .54791 .39685 .56418 .40778 .55559 .39243 Metetra
.059 .311 .796 r
.55559 .39243 .56418 .40778 .58075 .41945 .57215 .40412 Metetra
.03 .293 .786 r
.57215 .40412 .58075 .41945 .59764 .43186 .58903 .41657 Metetra
.002 .276 .777 r
.58903 .41657 .59764 .43186 .61486 .44498 .60625 .42978 Metetra
0 .26 .768 r
.60625 .42978 .61486 .44498 .63244 .45882 .62382 .44372 Metetra
0 .245 .759 r
.62382 .44372 .63244 .45882 .65037 .47334 .64176 .45837 Metetra
0 .231 .75 r
.64176 .45837 .65037 .47334 .66869 .48854 .66009 .47371 Metetra
0 .218 .741 r
.66009 .47371 .66869 .48854 .68742 .50441 .67882 .48972 Metetra
0 .206 .733 r
.67882 .48972 .68742 .50441 .70655 .52092 .69797 .50638 Metetra
0 .195 .726 r
.69797 .50638 .70655 .52092 .72613 .53806 .71756 .52366 Metetra
0 .186 .718 r
.71756 .52366 .72613 .53806 .74615 .55582 .7376 .54156 Metetra
0 .178 .712 r
.7376 .54156 .74615 .55582 .76664 .5742 .75812 .56007 Metetra
0 .171 .706 r
.75812 .56007 .76664 .5742 .78761 .59317 .77912 .57917 Metetra
.649 .708 .89 r
.16478 .33837 .17407 .34847 .18703 .34428 .17779 .33401 Metetra
.643 .704 .891 r
.17779 .33401 .18703 .34428 .20005 .34026 .19085 .32984 Metetra
.636 .699 .892 r
.19085 .32984 .20005 .34026 .21313 .33644 .20397 .32588 Metetra
.629 .695 .893 r
.20397 .32588 .21313 .33644 .22626 .33282 .21714 .32213 Metetra
.621 .691 .894 r
.21714 .32213 .22626 .33282 .23946 .32942 .23038 .31863 Metetra
.612 .687 .896 r
.23038 .31863 .23946 .32942 .25272 .32623 .24367 .31537 Metetra
.604 .683 .898 r
.24367 .31537 .25272 .32623 .26606 .32325 .25704 .31235 Metetra
.596 .681 .9 r
.25704 .31235 .26606 .32325 .27947 .32046 .27048 .30956 Metetra
.588 .679 .903 r
.27048 .30956 .27947 .32046 .29296 .31785 .28399 .30697 Metetra
.581 .678 .906 r
.28399 .30697 .29296 .31785 .30654 .31538 .2976 .30457 Metetra
.576 .678 .91 r
.2976 .30457 .30654 .31538 .32022 .31302 .31129 .3023 Metetra
.571 .68 .913 r
.31129 .3023 .32022 .31302 .334 .31072 .32509 .30013 Metetra
.568 .682 .917 r
.32509 .30013 .334 .31072 .34788 .30846 .339 .29801 Metetra
.566 .686 .921 r
.339 .29801 .34788 .30846 .36187 .30619 .35301 .29588 Metetra
.47 .648 .936 r
.35301 .29588 .36187 .30619 .37583 .30654 .36702 .296 Metetra
.264 .534 .917 r
.36702 .296 .37583 .30654 .38972 .31127 .38096 .29987 Metetra
.286 .526 .908 r
.38096 .29987 .38972 .31127 .40379 .31606 .39508 .30385 Metetra
.294 .513 .897 r
.39508 .30385 .40379 .31606 .41804 .32107 .40937 .30812 Metetra
.292 .496 .885 r
.40937 .30812 .41804 .32107 .43248 .32645 .42382 .31282 Metetra
.281 .477 .873 r
.42382 .31282 .43248 .32645 .44709 .3323 .43846 .31808 Metetra
.265 .457 .862 r
.43846 .31808 .44709 .3323 .4619 .33872 .45328 .32401 Metetra
.243 .436 .851 r
.45328 .32401 .4619 .33872 .47692 .34579 .4683 .33069 Metetra
.218 .414 .841 r
.4683 .33069 .47692 .34579 .49216 .35358 .48353 .33818 Metetra
.19 .392 .831 r
.48353 .33818 .49216 .35358 .50762 .36212 .49899 .34651 Metetra
.159 .371 .822 r
.49899 .34651 .50762 .36212 .52334 .37143 .5147 .3557 Metetra
.128 .349 .812 r
.5147 .3557 .52334 .37143 .53932 .38154 .53067 .36575 Metetra
.095 .328 .803 r
.53067 .36575 .53932 .38154 .55559 .39243 .54692 .37666 Metetra
.063 .308 .793 r
.54692 .37666 .55559 .39243 .57215 .40412 .56348 .3884 Metetra
.03 .289 .783 r
.56348 .3884 .57215 .40412 .58903 .41657 .58035 .40096 Metetra
0 .271 .773 r
.58035 .40096 .58903 .41657 .60625 .42978 .59756 .4143 Metetra
0 .254 .764 r
.59756 .4143 .60625 .42978 .62382 .44372 .61513 .4284 Metetra
0 .238 .754 r
.61513 .4284 .62382 .44372 .64176 .45837 .63307 .44322 Metetra
0 .223 .745 r
.63307 .44322 .64176 .45837 .66009 .47371 .6514 .45873 Metetra
0 .21 .736 r
.6514 .45873 .66009 .47371 .67882 .48972 .67014 .47491 Metetra
0 .198 .728 r
.67014 .47491 .67882 .48972 .69797 .50638 .68931 .49173 Metetra
0 .188 .72 r
.68931 .49173 .69797 .50638 .71756 .52366 .70891 .50918 Metetra
0 .179 .713 r
.70891 .50918 .71756 .52366 .7376 .54156 .72898 .52723 Metetra
0 .171 .707 r
.72898 .52723 .7376 .54156 .75812 .56007 .74952 .54588 Metetra
0 .164 .701 r
.74952 .54588 .75812 .56007 .77912 .57917 .77055 .56511 Metetra
.652 .709 .889 r
.1554 .32818 .16478 .33837 .17779 .33401 .16846 .32366 Metetra
.646 .705 .89 r
.16846 .32366 .17779 .33401 .19085 .32984 .18157 .3193 Metetra
.639 .7 .89 r
.18157 .3193 .19085 .32984 .20397 .32588 .19473 .31516 Metetra
.632 .695 .891 r
.19473 .31516 .20397 .32588 .21714 .32213 .20795 .31126 Metetra
.623 .69 .892 r
.20795 .31126 .21714 .32213 .23038 .31863 .22122 .30763 Metetra
.614 .685 .893 r
.22122 .30763 .23038 .31863 .24367 .31537 .23456 .30427 Metetra
.605 .681 .895 r
.23456 .30427 .24367 .31537 .25704 .31235 .24795 .30119 Metetra
.595 .677 .898 r
.24795 .30119 .25704 .31235 .27048 .30956 .26141 .29838 Metetra
.586 .674 .901 r
.26141 .29838 .27048 .30956 .28399 .30697 .27495 .29583 Metetra
.577 .673 .904 r
.27495 .29583 .28399 .30697 .2976 .30457 .28858 .2935 Metetra
.57 .672 .908 r
.28858 .2935 .2976 .30457 .31129 .3023 .30229 .29135 Metetra
.564 .674 .912 r
.30229 .29135 .31129 .3023 .32509 .30013 .31611 .28933 Metetra
.56 .677 .917 r
.31611 .28933 .32509 .30013 .339 .29801 .33003 .28738 Metetra
.558 .681 .921 r
.33003 .28738 .339 .29801 .35301 .29588 .34406 .28544 Metetra
.48 .652 .935 r
.34406 .28544 .35301 .29588 .36702 .296 .35811 .28529 Metetra
.323 .566 .926 r
.35811 .28529 .36702 .296 .38096 .29987 .37212 .28818 Metetra
.34 .553 .912 r
.37212 .28818 .38096 .29987 .39508 .30385 .3863 .29123 Metetra
.343 .534 .898 r
.3863 .29123 .39508 .30385 .40937 .30812 .40063 .29464 Metetra
.335 .513 .884 r
.40063 .29464 .40937 .30812 .42382 .31282 .41512 .29859 Metetra
.32 .49 .871 r
.41512 .29859 .42382 .31282 .43846 .31808 .42977 .30322 Metetra
.298 .467 .859 r
.42977 .30322 .43846 .31808 .45328 .32401 .44461 .30863 Metetra
.271 .443 .848 r
.44461 .30863 .45328 .32401 .4683 .33069 .45963 .31492 Metetra
.241 .418 .838 r
.45963 .31492 .4683 .33069 .48353 .33818 .47485 .32212 Metetra
.208 .394 .828 r
.47485 .32212 .48353 .33818 .49899 .34651 .4903 .33028 Metetra
.173 .371 .818 r
.4903 .33028 .49899 .34651 .5147 .3557 .50599 .33939 Metetra
.137 .347 .808 r
.50599 .33939 .5147 .3557 .53067 .36575 .52195 .34945 Metetra
.1 .325 .799 r
.52195 .34945 .53067 .36575 .54692 .37666 .53818 .36044 Metetra
.063 .303 .789 r
.53818 .36044 .54692 .37666 .56348 .3884 .55473 .37231 Metetra
.027 .282 .778 r
.55473 .37231 .56348 .3884 .58035 .40096 .57159 .38503 Metetra
0 .263 .768 r
.57159 .38503 .58035 .40096 .59756 .4143 .58879 .39857 Metetra
0 .245 .758 r
.58879 .39857 .59756 .4143 .61513 .4284 .60636 .41287 Metetra
0 .229 .748 r
.60636 .41287 .61513 .4284 .63307 .44322 .6243 .42791 Metetra
0 .214 .739 r
.6243 .42791 .63307 .44322 .6514 .45873 .64263 .44363 Metetra
0 .201 .73 r
.64263 .44363 .6514 .45873 .67014 .47491 .66138 .46002 Metetra
0 .19 .721 r
.66138 .46002 .67014 .47491 .68931 .49173 .68056 .47703 Metetra
0 .18 .714 r
.68056 .47703 .68931 .49173 .70891 .50918 .70018 .49465 Metetra
0 .171 .707 r
.70018 .49465 .70891 .50918 .72898 .52723 .72026 .51286 Metetra
0 .164 .701 r
.72026 .51286 .72898 .52723 .74952 .54588 .74083 .53165 Metetra
0 .158 .696 r
.74083 .53165 .74952 .54588 .77055 .56511 .76189 .55101 Metetra
.656 .712 .889 r
.14592 .31795 .1554 .32818 .16846 .32366 .15903 .31322 Metetra
.651 .707 .888 r
.15903 .31322 .16846 .32366 .18157 .3193 .1722 .30866 Metetra
.644 .701 .888 r
.1722 .30866 .18157 .3193 .19473 .31516 .18541 .30431 Metetra
.636 .696 .888 r
.18541 .30431 .19473 .31516 .20795 .31126 .19868 .30022 Metetra
.627 .69 .889 r
.19868 .30022 .20795 .31126 .22122 .30763 .212 .2964 Metetra
.617 .683 .89 r
.212 .2964 .22122 .30763 .23456 .30427 .22537 .2929 Metetra
.606 .678 .892 r
.22537 .2929 .23456 .30427 .24795 .30119 .2388 .28973 Metetra
.595 .673 .894 r
.2388 .28973 .24795 .30119 .26141 .29838 .25229 .28689 Metetra
.584 .669 .897 r
.25229 .28689 .26141 .29838 .27495 .29583 .26585 .28436 Metetra
.573 .666 .901 r
.26585 .28436 .27495 .29583 .28858 .2935 .27949 .28211 Metetra
.564 .665 .906 r
.27949 .28211 .28858 .2935 .30229 .29135 .29321 .2801 Metetra
.556 .666 .911 r
.29321 .2801 .30229 .29135 .31611 .28933 .30704 .27827 Metetra
.55 .669 .916 r
.30704 .27827 .31611 .28933 .33003 .28738 .32097 .27654 Metetra
.547 .674 .921 r
.32097 .27654 .33003 .28738 .34406 .28544 .33502 .27484 Metetra
.491 .657 .934 r
.33502 .27484 .34406 .28544 .35811 .28529 .34911 .27441 Metetra
.389 .6 .931 r
.34911 .27441 .35811 .28529 .37212 .28818 .36321 .27614 Metetra
.399 .579 .912 r
.36321 .27614 .37212 .28818 .3863 .29123 .37745 .27809 Metetra
.395 .554 .895 r
.37745 .27809 .3863 .29123 .40063 .29464 .39184 .28052 Metetra
.381 .528 .879 r
.39184 .28052 .40063 .29464 .41512 .29859 .40637 .28362 Metetra
.359 .501 .865 r
.40637 .28362 .41512 .29859 .42977 .30322 .42105 .28755 Metetra
.331 .474 .853 r
.42105 .28755 .42977 .30322 .44461 .30863 .43589 .29243 Metetra
.299 .447 .842 r
.43589 .29243 .44461 .30863 .45963 .31492 .4509 .29833 Metetra
.263 .42 .831 r
.4509 .29833 .45963 .31492 .47485 .32212 .46612 .3053 Metetra
.224 .394 .822 r
.46612 .3053 .47485 .32212 .4903 .33028 .48155 .31335 Metetra
.184 .368 .812 r
.48155 .31335 .4903 .33028 .50599 .33939 .49723 .32246 Metetra
.142 .342 .803 r
.49723 .32246 .50599 .33939 .52195 .34945 .51316 .33262 Metetra
.1 .318 .793 r
.51316 .33262 .52195 .34945 .53818 .36044 .52938 .34377 Metetra
.059 .295 .783 r
.52938 .34377 .53818 .36044 .55473 .37231 .5459 .35586 Metetra
.019 .273 .772 r
.5459 .35586 .55473 .37231 .57159 .38503 .56275 .36884 Metetra
0 .253 .762 r
.56275 .36884 .57159 .38503 .58879 .39857 .57995 .38264 Metetra
0 .234 .751 r
.57995 .38264 .58879 .39857 .60636 .41287 .59751 .39722 Metetra
0 .218 .741 r
.59751 .39722 .60636 .41287 .6243 .42791 .61545 .41252 Metetra
0 .203 .731 r
.61545 .41252 .6243 .42791 .64263 .44363 .63379 .42849 Metetra
0 .19 .722 r
.63379 .42849 .64263 .44363 .66138 .46002 .65254 .44511 Metetra
0 .18 .714 r
.65254 .44511 .66138 .46002 .68056 .47703 .67173 .46233 Metetra
0 .17 .707 r
.67173 .46233 .68056 .47703 .70018 .49465 .69136 .48013 Metetra
0 .162 .7 r
.69136 .48013 .70018 .49465 .72026 .51286 .71147 .4985 Metetra
0 .156 .695 r
.71147 .4985 .72026 .51286 .74083 .53165 .73205 .51742 Metetra
0 .151 .69 r
.73205 .51742 .74083 .53165 .76189 .55101 .75314 .53689 Metetra
.661 .714 .888 r
.13633 .3077 .14592 .31795 .15903 .31322 .1495 .30275 Metetra
.656 .71 .887 r
.1495 .30275 .15903 .31322 .1722 .30866 .16273 .29796 Metetra
.649 .704 .887 r
.16273 .29796 .1722 .30866 .18541 .30431 .17601 .29336 Metetra
.641 .697 .886 r
.17601 .29336 .18541 .30431 .19868 .30022 .18933 .289 Metetra
.632 .69 .886 r
.18933 .289 .19868 .30022 .212 .2964 .2027 .28496 Metetra
.621 .683 .887 r
.2027 .28496 .212 .2964 .22537 .2929 .21612 .28125 Metetra
.609 .675 .888 r
.21612 .28125 .22537 .2929 .2388 .28973 .22958 .27793 Metetra
.596 .668 .89 r
.22958 .27793 .2388 .28973 .25229 .28689 .2431 .27501 Metetra
.582 .663 .893 r
.2431 .27501 .25229 .28689 .26585 .28436 .25668 .27248 Metetra
.569 .658 .897 r
.25668 .27248 .26585 .28436 .27949 .28211 .27034 .27032 Metetra
.557 .656 .902 r
.27034 .27032 .27949 .28211 .29321 .2801 .28407 .26848 Metetra
.546 .656 .908 r
.28407 .26848 .29321 .2801 .30704 .27827 .2979 .26689 Metetra
.538 .659 .914 r
.2979 .26689 .30704 .27827 .32097 .27654 .31184 .26546 Metetra
.533 .665 .921 r
.31184 .26546 .32097 .27654 .33502 .27484 .32589 .26409 Metetra
.505 .661 .931 r
.32589 .26409 .33502 .27484 .34911 .27441 .34003 .26332 Metetra
.459 .633 .93 r
.34003 .26332 .34911 .27441 .36321 .27614 .35423 .26363 Metetra
.461 .603 .906 r
.35423 .26363 .36321 .27614 .37745 .27809 .36856 .26428 Metetra
.449 .572 .886 r
.36856 .26428 .37745 .27809 .39184 .28052 .38302 .26555 Metetra
.427 .54 .869 r
.38302 .26555 .39184 .28052 .40637 .28362 .39759 .26769 Metetra
.398 .509 .854 r
.39759 .26769 .40637 .28362 .42105 .28755 .41229 .27086 Metetra
.364 .478 .842 r
.41229 .27086 .42105 .28755 .43589 .29243 .42714 .27519 Metetra
.325 .448 .831 r
.42714 .27519 .43589 .29243 .4509 .29833 .44215 .28075 Metetra
.283 .418 .822 r
.44215 .28075 .4509 .29833 .46612 .3053 .45735 .28756 Metetra
.237 .389 .813 r
.45735 .28756 .46612 .3053 .48155 .31335 .47276 .29561 Metetra
.19 .361 .804 r
.47276 .29561 .48155 .31335 .49723 .32246 .4884 .30486 Metetra
.142 .334 .795 r
.4884 .30486 .49723 .32246 .51316 .33262 .50431 .31524 Metetra
.095 .308 .785 r
.50431 .31524 .51316 .33262 .52938 .34377 .52051 .32669 Metetra
.048 .283 .775 r
.52051 .32669 .52938 .34377 .5459 .35586 .53701 .33912 Metetra
.004 .26 .764 r
.53701 .33912 .5459 .35586 .56275 .36884 .55384 .35245 Metetra
0 .239 .753 r
.55384 .35245 .56275 .36884 .57995 .38264 .57103 .36661 Metetra
0 .22 .742 r
.57103 .36661 .57995 .38264 .59751 .39722 .58858 .38153 Metetra
0 .204 .732 r
.58858 .38153 .59751 .39722 .61545 .41252 .60652 .39714 Metetra
0 .19 .722 r
.60652 .39714 .61545 .41252 .63379 .42849 .62486 .41339 Metetra
0 .178 .713 r
.62486 .41339 .63379 .42849 .65254 .44511 .64362 .43025 Metetra
0 .168 .705 r
.64362 .43025 .65254 .44511 .67173 .46233 .66282 .44768 Metetra
0 .16 .698 r
.66282 .44768 .67173 .46233 .69136 .48013 .68247 .46566 Metetra
0 .154 .693 r
.68247 .46566 .69136 .48013 .71147 .4985 .70259 .48417 Metetra
0 .148 .688 r
.70259 .48417 .71147 .4985 .73205 .51742 .72319 .50321 Metetra
0 .144 .684 r
.72319 .50321 .73205 .51742 .75314 .53689 .74431 .52277 Metetra
.665 .718 .888 r
.12664 .29746 .13633 .3077 .1495 .30275 .13987 .2923 Metetra
.661 .714 .887 r
.13987 .2923 .1495 .30275 .16273 .29796 .15316 .28724 Metetra
.656 .708 .886 r
.15316 .28724 .16273 .29796 .17601 .29336 .1665 .28234 Metetra
.648 .701 .884 r
.1665 .28234 .17601 .29336 .18933 .289 .17989 .27768 Metetra
.639 .692 .883 r
.17989 .27768 .18933 .289 .2027 .28496 .19332 .27331 Metetra
.628 .683 .883 r
.19332 .27331 .2027 .28496 .21612 .28125 .2068 .26931 Metetra
.614 .673 .883 r
.2068 .26931 .21612 .28125 .22958 .27793 .22031 .26575 Metetra
.599 .664 .884 r
.22031 .26575 .22958 .27793 .2431 .27501 .23387 .26266 Metetra
.583 .655 .887 r
.23387 .26266 .2431 .27501 .25668 .27248 .24747 .26009 Metetra
.566 .648 .891 r
.24747 .26009 .25668 .27248 .27034 .27032 .26114 .258 Metetra
.55 .644 .896 r
.26114 .258 .27034 .27032 .28407 .26848 .27487 .25635 Metetra
.535 .643 .903 r
.27487 .25635 .28407 .26848 .2979 .26689 .2887 .25507 Metetra
.522 .645 .911 r
.2887 .25507 .2979 .26689 .31184 .26546 .30262 .25404 Metetra
.514 .652 .92 r
.30262 .25404 .31184 .26546 .32589 .26409 .31667 .25313 Metetra
.52 .666 .928 r
.31667 .25313 .32589 .26409 .34003 .26332 .33086 .25196 Metetra
.534 .665 .92 r
.33086 .25196 .34003 .26332 .35423 .26363 .34521 .25049 Metetra
.525 .624 .892 r
.34521 .25049 .35423 .26363 .36856 .26428 .35965 .24952 Metetra
.503 .585 .87 r
.35965 .24952 .36856 .26428 .38302 .26555 .37418 .24941 Metetra
.473 .547 .851 r
.37418 .24941 .38302 .26555 .39759 .26769 .3888 .25044 Metetra
.436 .511 .837 r
.3888 .25044 .39759 .26769 .41229 .27086 .40353 .2528 Metetra
.394 .476 .826 r
.40353 .2528 .41229 .27086 .42714 .27519 .41838 .25662 Metetra
.347 .443 .817 r
.41838 .25662 .42714 .27519 .44215 .28075 .43337 .26194 Metetra
.297 .411 .809 r
.43337 .26194 .44215 .28075 .45735 .28756 .44854 .26875 Metetra
.244 .379 .801 r
.44854 .26875 .45735 .28756 .47276 .29561 .46392 .27699 Metetra
.189 .349 .794 r
.46392 .27699 .47276 .29561 .4884 .30486 .47953 .28657 Metetra
.134 .319 .785 r
.47953 .28657 .4884 .30486 .50431 .31524 .49541 .29738 Metetra
.079 .291 .775 r
.49541 .29738 .50431 .31524 .52051 .32669 .51157 .3093 Metetra
.027 .266 .765 r
.51157 .3093 .52051 .32669 .53701 .33912 .52805 .32222 Metetra
0 .242 .753 r
.52805 .32222 .53701 .33912 .55384 .35245 .54486 .33603 Metetra
0 .221 .742 r
.54486 .33603 .55384 .35245 .57103 .36661 .56203 .35063 Metetra
0 .203 .731 r
.56203 .35063 .57103 .36661 .58858 .38153 .57957 .36594 Metetra
0 .187 .72 r
.57957 .36594 .58858 .38153 .60652 .39714 .59751 .3819 Metetra
0 .175 .711 r
.59751 .3819 .60652 .39714 .62486 .41339 .61585 .39844 Metetra
0 .164 .702 r
.61585 .39844 .62486 .41339 .64362 .43025 .63462 .41554 Metetra
0 .156 .696 r
.63462 .41554 .64362 .43025 .66282 .44768 .65382 .43316 Metetra
0 .15 .69 r
.65382 .43316 .66282 .44768 .68247 .46566 .67349 .45128 Metetra
0 .145 .685 r
.67349 .45128 .68247 .46566 .70259 .48417 .69362 .46991 Metetra
0 .141 .682 r
.69362 .46991 .70259 .48417 .72319 .50321 .71425 .48903 Metetra
0 .138 .679 r
.71425 .48903 .72319 .50321 .74431 .52277 .73538 .50865 Metetra
.669 .722 .889 r
.11683 .28727 .12664 .29746 .13987 .2923 .13012 .28191 Metetra
.667 .719 .888 r
.13012 .28191 .13987 .2923 .15316 .28724 .14347 .2766 Metetra
.663 .714 .886 r
.14347 .2766 .15316 .28724 .1665 .28234 .15689 .27139 Metetra
.657 .707 .884 r
.15689 .27139 .1665 .28234 .17989 .27768 .17035 .26634 Metetra
.649 .698 .881 r
.17035 .26634 .17989 .27768 .19332 .27331 .18386 .26154 Metetra
.638 .687 .879 r
.18386 .26154 .19332 .27331 .2068 .26931 .19741 .25711 Metetra
.624 .674 .878 r
.19741 .25711 .2068 .26931 .22031 .26575 .211 .25314 Metetra
.607 .661 .877 r
.211 .25314 .22031 .26575 .23387 .26266 .22461 .24973 Metetra
.587 .648 .878 r
.22461 .24973 .23387 .26266 .24747 .26009 .23825 .24697 Metetra
.565 .636 .881 r
.23825 .24697 .24747 .26009 .26114 .258 .25192 .24489 Metetra
.542 .628 .886 r
.25192 .24489 .26114 .258 .27487 .25635 .26565 .24345 Metetra
.521 .623 .894 r
.26565 .24345 .27487 .25635 .2887 .25507 .27946 .24258 Metetra
.501 .623 .904 r
.27946 .24258 .2887 .25507 .30262 .25404 .29335 .24213 Metetra
.486 .63 .916 r
.29335 .24213 .30262 .25404 .31667 .25313 .30736 .2419 Metetra
.538 .67 .923 r
.30736 .2419 .31667 .25313 .33086 .25196 .32162 .24026 Metetra
.613 .692 .9 r
.32162 .24026 .33086 .25196 .34521 .25049 .33615 .23642 Metetra
.59 .639 .868 r
.33615 .23642 .34521 .25049 .35965 .24952 .35074 .23334 Metetra
.557 .59 .843 r
.35074 .23334 .35965 .24952 .37418 .24941 .36538 .23151 Metetra
.516 .546 .825 r
.36538 .23151 .37418 .24941 .3888 .25044 .38006 .23126 Metetra
.47 .505 .812 r
.38006 .23126 .3888 .25044 .40353 .2528 .3948 .23282 Metetra
.418 .467 .803 r
.3948 .23282 .40353 .2528 .41838 .25662 .40964 .23627 Metetra
.363 .43 .797 r
.40964 .23627 .41838 .25662 .43337 .26194 .4246 .24159 Metetra
.303 .395 .791 r
.4246 .24159 .43337 .26194 .44854 .26875 .43972 .24871 Metetra
.24 .361 .786 r
.43972 .24871 .44854 .26875 .46392 .27699 .45505 .25747 Metetra
.175 .328 .78 r
.45505 .25747 .46392 .27699 .47953 .28657 .47061 .2677 Metetra
.111 .297 .772 r
.47061 .2677 .47953 .28657 .49541 .29738 .48644 .27921 Metetra
.048 .268 .762 r
.48644 .27921 .49541 .29738 .51157 .3093 .50257 .29184 Metetra
0 .241 .751 r
.50257 .29184 .51157 .3093 .52805 .32222 .51901 .30542 Metetra
0 .218 .739 r
.51901 .30542 .52805 .32222 .54486 .33603 .5358 .31982 Metetra
0 .198 .727 r
.5358 .31982 .54486 .33603 .56203 .35063 .55296 .33493 Metetra
0 .181 .716 r
.55296 .33493 .56203 .35063 .57957 .36594 .57049 .35066 Metetra
0 .168 .706 r
.57049 .35066 .57957 .36594 .59751 .3819 .58842 .36694 Metetra
0 .158 .698 r
.58842 .36694 .59751 .3819 .61585 .39844 .60676 .38374 Metetra
0 .151 .691 r
.60676 .38374 .61585 .39844 .63462 .41554 .62553 .40102 Metetra
0 .145 .686 r
.62553 .40102 .63462 .41554 .65382 .43316 .64475 .41877 Metetra
0 .141 .682 r
.64475 .41877 .65382 .43316 .67349 .45128 .66442 .43699 Metetra
0 .138 .679 r
.66442 .43699 .67349 .45128 .69362 .46991 .68456 .45567 Metetra
0 .136 .677 r
.68456 .45567 .69362 .46991 .71425 .48903 .7052 .47483 Metetra
0 .134 .675 r
.7052 .47483 .71425 .48903 .73538 .50865 .72635 .49447 Metetra
.672 .726 .89 r
.10689 .27712 .11683 .28727 .13012 .28191 .12023 .27163 Metetra
.671 .724 .889 r
.12023 .27163 .13012 .28191 .14347 .2766 .13365 .26613 Metetra
.67 .721 .888 r
.13365 .26613 .14347 .2766 .15689 .27139 .14713 .26064 Metetra
.667 .716 .885 r
.14713 .26064 .15689 .27139 .17035 .26634 .16068 .25521 Metetra
.662 .708 .882 r
.16068 .25521 .17035 .26634 .18386 .26154 .17429 .24991 Metetra
.653 .696 .878 r
.17429 .24991 .18386 .26154 .19741 .25711 .18794 .24486 Metetra
.641 .682 .873 r
.18794 .24486 .19741 .25711 .211 .25314 .20162 .24021 Metetra
.623 .664 .869 r
.20162 .24021 .211 .25314 .22461 .24973 .21532 .23614 Metetra
.6 .644 .865 r
.21532 .23614 .22461 .24973 .23825 .24697 .22903 .23287 Metetra
.572 .624 .865 r
.22903 .23287 .23825 .24697 .25192 .24489 .24275 .23053 Metetra
.54 .605 .868 r
.24275 .23053 .25192 .24489 .26565 .24345 .25647 .22921 Metetra
.505 .592 .875 r
.25647 .22921 .26565 .24345 .27946 .24258 .27023 .22886 Metetra
.471 .586 .887 r
.27023 .22886 .27946 .24258 .29335 .24213 .28406 .2293 Metetra
.441 .59 .905 r
.28406 .2293 .29335 .24213 .30736 .2419 .29799 .23023 Metetra
.559 .672 .914 r
.29799 .23023 .30736 .2419 .32162 .24026 .31231 .228 Metetra
.694 .713 .863 r
.31231 .228 .32162 .24026 .33615 .23642 .32713 .22073 Metetra
.653 .643 .828 r
.32713 .22073 .33615 .23642 .35074 .23334 .34193 .21473 Metetra
.604 .584 .803 r
.34193 .21473 .35074 .23334 .36538 .23151 .3567 .2107 Metetra
.551 .532 .787 r
.3567 .2107 .36538 .23151 .38006 .23126 .37144 .20905 Metetra
.493 .485 .777 r
.37144 .20905 .38006 .23126 .3948 .23282 .38618 .20999 Metetra
.43 .443 .772 r
.38618 .20999 .3948 .23282 .40964 .23627 .40097 .21349 Metetra
.363 .404 .77 r
.40097 .21349 .40964 .23627 .4246 .24159 .41586 .21939 Metetra
.29 .366 .768 r
.41586 .21939 .4246 .24159 .43972 .24871 .4309 .22743 Metetra
.214 .33 .766 r
.4309 .22743 .43972 .24871 .45505 .25747 .44615 .23728 Metetra
.136 .295 .762 r
.44615 .23728 .45505 .25747 .47061 .2677 .46164 .24864 Metetra
.06 .262 .755 r
.46164 .24864 .47061 .2677 .48644 .27921 .47741 .26123 Metetra
0 .232 .744 r
.47741 .26123 .48644 .27921 .50257 .29184 .49349 .2748 Metetra
0 .207 .732 r
.49349 .2748 .50257 .29184 .51901 .30542 .5099 .28917 Metetra
0 .186 .719 r
.5099 .28917 .51901 .30542 .5358 .31982 .52667 .3042 Metetra
0 .17 .708 r
.52667 .3042 .5358 .31982 .55296 .33493 .54381 .31978 Metetra
0 .158 .698 r
.54381 .31978 .55296 .33493 .57049 .35066 .56133 .33584 Metetra
0 .15 .69 r
.56133 .33584 .57049 .35066 .58842 .36694 .57925 .35236 Metetra
0 .144 .685 r
.57925 .35236 .58842 .36694 .60676 .38374 .59759 .36931 Metetra
0 .139 .681 r
.59759 .36931 .60676 .38374 .62553 .40102 .61636 .38669 Metetra
0 .137 .678 r
.61636 .38669 .62553 .40102 .64475 .41877 .63558 .40449 Metetra
0 .135 .676 r
.63558 .40449 .64475 .41877 .66442 .43699 .65526 .42272 Metetra
0 .133 .674 r
.65526 .42272 .66442 .43699 .68456 .45567 .67541 .44141 Metetra
0 .132 .673 r
.67541 .44141 .68456 .45567 .7052 .47483 .69606 .46056 Metetra
0 .132 .672 r
.69606 .46056 .7052 .47483 .72635 .49447 .71723 .48018 Metetra
.672 .728 .891 r
.09685 .26694 .10689 .27712 .12023 .27163 .11023 .26138 Metetra
.672 .727 .891 r
.11023 .26138 .12023 .27163 .13365 .26613 .12369 .25579 Metetra
.672 .727 .891 r
.12369 .25579 .13365 .26613 .14713 .26064 .13723 .25017 Metetra
.672 .725 .889 r
.13723 .25017 .14713 .26064 .16068 .25521 .15084 .24452 Metetra
.671 .722 .887 r
.15084 .24452 .16068 .25521 .17429 .24991 .16452 .23886 Metetra
.669 .715 .883 r
.16452 .23886 .17429 .24991 .18794 .24486 .17828 .23321 Metetra
.664 .703 .876 r
.17828 .23321 .18794 .24486 .20162 .24021 .1921 .22766 Metetra
.653 .684 .866 r
.1921 .22766 .20162 .24021 .21532 .23614 .20597 .22239 Metetra
.634 .657 .854 r
.20597 .22239 .21532 .23614 .22903 .23287 .21985 .21772 Metetra
.604 .623 .843 r
.21985 .21772 .22903 .23287 .24275 .23053 .2337 .21413 Metetra
.561 .584 .835 r
.2337 .21413 .24275 .23053 .25647 .22921 .24749 .21212 Metetra
.503 .546 .833 r
.24749 .21212 .25647 .22921 .27023 .22886 .26121 .21209 Metetra
.433 .514 .842 r
.26121 .21209 .27023 .22886 .28406 .2293 .27489 .21402 Metetra
.354 .497 .863 r
.27489 .21402 .28406 .2293 .29799 .23023 .2886 .21742 Metetra
.579 .66 .893 r
.2886 .21742 .29799 .23023 .31231 .228 .303 .21462 Metetra
.774 .719 .801 r
.303 .21462 .31231 .228 .32713 .22073 .3183 .20154 Metetra
.705 .626 .766 r
.3183 .20154 .32713 .22073 .34193 .21473 .33343 .19105 Metetra
.635 .553 .745 r
.33343 .19105 .34193 .21473 .3567 .2107 .34835 .18421 Metetra
.564 .492 .733 r
.34835 .18421 .3567 .2107 .37144 .20905 .36311 .18149 Metetra
.491 .441 .729 r
.36311 .18149 .37144 .20905 .38618 .20999 .37777 .18279 Metetra
.413 .395 .73 r
.37777 .18279 .38618 .20999 .40097 .21349 .39242 .18767 Metetra
.328 .354 .735 r
.39242 .18767 .40097 .21349 .41586 .21939 .40716 .19551 Metetra
.236 .314 .741 r
.40716 .19551 .41586 .21939 .4309 .22743 .42206 .20566 Metetra
.139 .276 .743 r
.42206 .20566 .4309 .22743 .44615 .23728 .43718 .2175 Metetra
.044 .24 .739 r
.43718 .2175 .44615 .23728 .46164 .24864 .45258 .23056 Metetra
0 .209 .73 r
.45258 .23056 .46164 .24864 .47741 .26123 .46828 .24447 Metetra
0 .184 .717 r
.46828 .24447 .47741 .26123 .49349 .2748 .48432 .259 Metetra
0 .166 .705 r
.48432 .259 .49349 .2748 .5099 .28917 .5007 .27403 Metetra
0 .153 .694 r
.5007 .27403 .5099 .28917 .52667 .3042 .51745 .28946 Metetra
0 .145 .687 r
.51745 .28946 .52667 .3042 .54381 .31978 .53457 .30528 Metetra
0 .14 .682 r
.53457 .30528 .54381 .31978 .56133 .33584 .55208 .32148 Metetra
0 .137 .678 r
.55208 .32148 .56133 .33584 .57925 .35236 .57 .33805 Metetra
0 .135 .676 r
.57 .33805 .57925 .35236 .59759 .36931 .58833 .35501 Metetra
0 .133 .675 r
.58833 .35501 .59759 .36931 .61636 .38669 .6071 .37238 Metetra
0 .133 .674 r
.6071 .37238 .61636 .38669 .63558 .40449 .62631 .39015 Metetra
0 .132 .673 r
.62631 .39015 .63558 .40449 .65526 .42272 .64599 .40836 Metetra
0 .131 .672 r
.64599 .40836 .65526 .42272 .67541 .44141 .66615 .42702 Metetra
0 .131 .672 r
.66615 .42702 .67541 .44141 .69606 .46056 .68681 .44613 Metetra
0 .13 .671 r
.68681 .44613 .69606 .46056 .71723 .48018 .70799 .46573 Metetra
.672 .728 .891 r
.0867 .25665 .09685 .26694 .11023 .26138 .10012 .25104 Metetra
.672 .728 .891 r
.10012 .25104 .11023 .26138 .12369 .25579 .11362 .24539 Metetra
.672 .728 .891 r
.11362 .24539 .12369 .25579 .13723 .25017 .1272 .23971 Metetra
.672 .728 .891 r
.1272 .23971 .13723 .25017 .15084 .24452 .14085 .234 Metetra
.672 .728 .891 r
.14085 .234 .15084 .24452 .16452 .23886 .15459 .22825 Metetra
.672 .727 .891 r
.15459 .22825 .16452 .23886 .17828 .23321 .1684 .22248 Metetra
.672 .726 .89 r
.1684 .22248 .17828 .23321 .1921 .22766 .18229 .21667 Metetra
.671 .722 .887 r
.18229 .21667 .1921 .22766 .20597 .22239 .19626 .21082 Metetra
.67 .711 .879 r
.19626 .21082 .20597 .22239 .21985 .21772 .21031 .20495 Metetra
.665 .688 .862 r
.21031 .20495 .21985 .21772 .2337 .21413 .22444 .19904 Metetra
.656 .648 .831 r
.22444 .19904 .2337 .21413 .24749 .21212 .23866 .19309 Metetra
.64 .593 .785 r
.23866 .19309 .24749 .21212 .26121 .21209 .25295 .18711 Metetra
.619 .531 .733 r
.25295 .18711 .26121 .21209 .27489 .21402 .26733 .18109 Metetra
.598 .476 .685 r
.26733 .18109 .27489 .21402 .2886 .21742 .2818 .17507 Metetra
.565 .423 .647 r
.2818 .17507 .2886 .21742 .303 .21462 .29621 .17056 Metetra
.593 .438 .641 r
.29621 .17056 .303 .21462 .3183 .20154 .31098 .16284 Metetra
.587 .449 .66 r
.31098 .16284 .3183 .20154 .33343 .19105 .32571 .15668 Metetra
.596 .47 .679 r
.32571 .15668 .33343 .19105 .34835 .18421 .34051 .15048 Metetra
.597 .474 .683 r
.34051 .15048 .34835 .18421 .36311 .18149 .35541 .14426 Metetra
.466 .374 .667 r
.35541 .14426 .36311 .18149 .37777 .18279 .36984 .14755 Metetra
.272 .252 .654 r
.36984 .14755 .37777 .18279 .39242 .18767 .38394 .16056 Metetra
.17 .239 .693 r
.38394 .16056 .39242 .18767 .40716 .19551 .39833 .17385 Metetra
.051 .217 .716 r
.39833 .17385 .40716 .19551 .42206 .20566 .41303 .18742 Metetra
0 .19 .717 r
.41303 .18742 .42206 .20566 .43718 .2175 .42804 .20127 Metetra
0 .167 .706 r
.42804 .20127 .43718 .2175 .45258 .23056 .44337 .21542 Metetra
0 .152 .693 r
.44337 .21542 .45258 .23056 .46828 .24447 .45903 .22988 Metetra
0 .142 .685 r
.45903 .22988 .46828 .24447 .48432 .259 .47504 .24466 Metetra
0 .138 .68 r
.47504 .24466 .48432 .259 .5007 .27403 .4914 .25976 Metetra
0 .135 .677 r
.4914 .25976 .5007 .27403 .51745 .28946 .50813 .27521 Metetra
0 .134 .676 r
.50813 .27521 .51745 .28946 .53457 .30528 .52523 .291 Metetra
0 .133 .675 r
.52523 .291 .53457 .30528 .55208 .32148 .54273 .30716 Metetra
0 .133 .674 r
.54273 .30716 .55208 .32148 .57 .33805 .56064 .32369 Metetra
0 .132 .674 r
.56064 .32369 .57 .33805 .58833 .35501 .57897 .34061 Metetra
0 .132 .673 r
.57897 .34061 .58833 .35501 .6071 .37238 .59773 .35793 Metetra
0 .131 .673 r
.59773 .35793 .6071 .37238 .62631 .39015 .61694 .37566 Metetra
0 .131 .672 r
.61694 .37566 .62631 .39015 .64599 .40836 .63662 .39383 Metetra
0 .131 .672 r
.63662 .39383 .64599 .40836 .66615 .42702 .65678 .41245 Metetra
0 .13 .671 r
.65678 .41245 .66615 .42702 .68681 .44613 .67745 .43153 Metetra
0 .13 .671 r
.67745 .43153 .68681 .44613 .70799 .46573 .69864 .45109 Metetra
0 g
.25 Mabswid
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
[(80)] .07825 .23514 .98066 1 Mshowa
.22216 .18933 m
.227 .19478 L
s
[(90)] .21247 .17844 .88892 1 Mshowa
.36358 .12987 m
.36813 .13557 L
s
[(100)] .3545 .11847 .79718 1 Mshowa
.51343 .06687 m
.51763 .07282 L
s
[(110)] .50502 .05496 .70545 1 Mshowa
.67245 0 m
.67627 .00621 L
s
[(120)] .66483 -0.01242 .61371 1 Mshowa
.125 Mabswid
.11461 .23456 m
.11764 .23771 L
s
.14105 .22344 m
.14405 .22662 L
s
.16779 .2122 m
.17076 .21541 L
s
.19482 .20083 m
.19776 .20407 L
s
.2498 .17771 m
.25268 .18101 L
s
.27776 .16595 m
.2806 .16928 L
s
.30604 .15406 m
.30885 .15742 L
s
.33465 .14204 m
.33741 .14542 L
s
.39286 .11756 m
.39554 .12101 L
s
.42247 .10511 m
.42512 .10859 L
s
.45243 .09251 m
.45504 .09602 L
s
.48275 .07977 m
.48531 .08331 L
s
.54447 .05381 m
.54695 .05742 L
s
.57589 .0406 m
.57832 .04424 L
s
.60769 .02723 m
.61007 .0309 L
s
.63987 .0137 m
.64221 .0174 L
s
.25 Mabswid
0 .80374 m
.40037 1.0855 L
s
.09485 .87049 m
.09577 .86326 L
s
[(0.2)] .09301 .88495 .12735 -1 Mshowa
.18136 .93137 m
.18209 .92412 L
s
[(0.4)] .17991 .94587 .10002 -1 Mshowa
.26055 .98709 m
.2611 .97983 L
s
[(0.6)] .25944 1.00163 .07584 -1 Mshowa
.3333 1.03829 m
.33369 1.03102 L
s
[(0.8)] .33251 1.05285 .05428 -1 Mshowa
.40037 1.0855 m
.40063 1.07821 L
s
[(1)] .39987 1.10006 .03495 -1 Mshowa
.125 Mabswid
.02453 .821 m
.02518 .81668 L
s
.04852 .83789 m
.04914 .83356 L
s
.07196 .85438 m
.07254 .85004 L
s
.11722 .88623 m
.11774 .88189 L
s
.13908 .90162 m
.13958 .89727 L
s
.16046 .91666 m
.16092 .91231 L
s
.20181 .94576 m
.20221 .9414 L
s
.22181 .95983 m
.22219 .95548 L
s
.24138 .97361 m
.24174 .96925 L
s
.27931 1.0003 m
.27961 .99593 L
s
.29768 1.01322 m
.29796 1.00886 L
s
.31567 1.02589 m
.31593 1.02152 L
s
.35057 1.05045 m
.35079 1.04608 L
s
.3675 1.06236 m
.3677 1.058 L
s
.3841 1.07404 m
.38427 1.06967 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{185.688, 201.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002i0000bA000`40O003h00O^Goo002iOol006mo
o`<000=oo`@000=oo`8003]oo`00L7oo00<007ooOol00goo00<007ooOol00Woo00@007ooOol003Yo
o`00L7oo00<007ooOol017oo00D007ooOomoo`00009oo`03001oogoo03Qoo`00L7oo00<007ooOol0
1Goo00@007ooOol0009oo`03001oogoo03Qoo`00L7oo00<007ooOol00Woo00@007ooOol0009oo`04
001oogoo000jOol006moo`8000Eoo`8000Aoo`8003]oo`00^Goo002iOol00;Uoo`00OGoo0P00>Woo
001jOol300000goo001oo`0iOol005Aoo`<000=oo`<000Aoo`8001Eoo`8000Aoo`8003Uoo`00EGoo
00<007ooOol00goo00<007ooOol00Woo00@007ooOol0019oo`800003Ool007oo00=oo`03001oo`00
03Qoo`00EGoo00<007ooOol00goo00<007ooOol00Woo00@007ooOol000moo`<000Qoo`03001oo`00
03Qoo`00EGoo00<007ooOol00goo00<007ooOol00Woo00@007ooOol000eoo`<000Yoo`04001oogoo
000gOol005Eoo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000:Ool3000=Ool01000Oomo
o`00=goo001DOol20004Ool20005Ool20009Ool3000?Ool01@00Oomoogoo0000=Woo001ZOol2000B
Ool00`00Oomoo`02Ool00`00Oomoo`0cOol006Moo`<001Aoo`03001oogoo009oo`03001oogoo03=o
o`00IGoo0P005goo00<007ooOol00goo00<007ooOol0<Woo001ROol3000JOol00`00Oomoo`02Ool0
0`00Oomoo`0bOol003Yoo`<000Aoo`8000Aoo`8001Moo`80009oo`03001oogoo01Qoo`03001oogoo
00=oo`03001oogoo035oo`00>goo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo000DOol2
000OOol00`00Oomoo`04Ool00`00Oomoo`0`Ool003]oo`03001oogoo009oo`04001oogoo0002Ool0
1000Oomoo`004Goo0`0000=oo`00Ool07Woo00<007ooOol017oo00<007ooOol0<7oo000kOol00`00
Oomoo`02Ool01000Oomoo`000Woo00@007ooOol000moo`8002Aoo`03001oogoo00Eoo`03001oogoo
02moo`00>goo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo000<Ool300000goo001oo`0S
Ool00`00Oomoo`05Ool00`00Oomoo`0_Ool003Yoo`8000Eoo`8000Aoo`8000]oo`8002Uoo`03001o
ogoo00Ioo`03001oogoo02ioo`00DWoo0P0000=oo`00Ool0:7oo00<007ooOol01goo00<007ooOol0
;Goo001?Ool3000]Ool00`00Oomoo`07Ool00`00Oomoo`0]Ool004eoo`800003Ool007oo02eoo`03
001oogoo00Qoo`03001oogoo02aoo`00BWoo0`00<Woo00<007ooOol027oo00<007ooOol0;7oo000X
Ool30004Ool2000GOol20002Ool00`00Oomoo`0`Ool00`00Oomoo`09Ool00`00Oomoo`0[Ool002]o
o`04001oogoo0002Ool00`00Oomoo`0BOol2000gOol00`00Oomoo`0:Ool00`00Oomoo`0ZOol002Uo
o`<0009oo`04001oogoo000AOol300000goo001oo`04000bOol00`00Oomoo`0:Ool00`00Oomoo`0Z
Ool002Qoo`04001oogoo0002Ool01000Oomoo`003goo0P000goo0`0000==m@00>GD00P00<Goo00<0
07ooOol02goo00<007ooOol0:Goo000XOol01000Oomoo`000Woo00@007ooOol000aoo`<00003Ool0
07oo00800009COD004geCOD003Ue000Q5000031oo`03001oogoo00]oo`03001oogoo02Uoo`00:Goo
0P0017oo0P002goo0P000goo0`0000A9e@00COD0009=m@04000iM@008A@2000_Ool00`00Oomoo`0<
Ool00`00Oomoo`0XOol003Yoo`800003Ool007oo00800005BM@004WEBMD00002COD01`00COE=m@00
>GD0024D008002ioo`03001oogoo00eoo`03001oogoo02Moo`00=goo0`000goo0P0000E9]000BMA9
e0000099e@03001=m@00009=m@07000iM@008A@001Cf0000;Goo00<007ooOol03Goo00<007ooOol0
9goo000eOol20002Ool4000014ge0019]0000TWD00<004WEBMD02P0000<DmP000000;Goo00<007oo
Ool03Goo00<007ooOol09Woo000bOol30003Ool200001dhG001=mDge0019]0000099e0060019eDWE
001B=`000TWg00<003gG00000SFG1@00:goo00<007ooOol03Goo00<007ooOol09Woo000BOol20004
Ool2000FOol20003Ool30000159I001>5`000Tge00@004VdBK@00099e0<0009B=`030019m`00008m
e`04000eU`00:GL200000`3F001oo`0YOol00`00Oomoo`0>Ool00`00Oomoo`0UOol0015oo`04001o
ogoo0002Ool01000Oomoo`004goo0P000goo0P0000EBVP00DUUBF@00009>5`07001=mDge0019]000
BM@00P0000EJV000DSMB=`000099m`0;000me`00=IL002Ug000ME`000=H0000YOol00`00Oomoo`0?
Ool00`00Oomoo`0TOol0015oo`04001oogoo0002Ool01000Oomoo`0047oo0`000Woo0`000UKK00<0
05:J00000U9I00<004hG00000Tge00H004Vd000006;i0002FYP01000DSL004Wg1`0000<YM`007EL0
0`00:7oo00<007ooOol03goo00<007ooOol097oo000BOol20003Ool01000Oomoo`003Woo0`000goo
0P0000AFg000E]]Ff`d000=Rn@H00006AST003gh?OP003GH0`0000<ME`0000000P009goo00<007oo
Ool047oo00<007ooOol08goo000AOol01000Oomoo`000Woo00@007ooOol000]oo`<000=oo`<00003
E_`005KL00<00008CWX004IJ0012>P00=QX0008]n`05001:_4Zl001Rn@0200001E:J001>FDiI0000
0THi00D003gh000ef000008]V0H002Ioo`03001oogoo011oo`03001oogoo02=oo`004Woo0P0017oo
0P002Woo2@0000=Fo00000000P000U:k00`004ijCWX004IJ0012>P00=QX002gk0002B[`200001E[K
001BVU:J00000TiI00<004Hi00000Sgh00H003GH000]V0009GP200000`Cg0000000UOol00`00Oomo
o`0AOol00`00Oomoo`0ROol0029oo`800006Ool00000001Fo0000UKl0P0000IFg000D[]B^`00D[/>
00001TJm001Jfe[K001BVP/00007;IP002Eh000IF0001?L00P0097oo00<007ooOol04Woo00<007oo
Ool08Goo000OOol60002E_`00`00E_`0000200000eKL001Fg00500002djK001:NdYk0016N`00@U/0
03iL00000SYL00<004Jm0000100000A:V`00AUX00092>P03000n6@00008ef@@0008IF0@002=oo`03
001oogoo019oo`03001oogoo025oo`007Woo00<007ooOol00P000UKl00@005Kl0000009Fo006001F
g000001B^`000Tjk00D004jKCY/004Yk00/00096_@03001>g000009:V`030016FP000092>P06000n
6@00=MT002fi0`0000<IF00000000`008Woo00<007ooOol04goo00<007ooOol087oo000NOol30002
E_`01000E_`000000UKl00@005Kl0000009Bg004001B^`00C[/5000014ZL0016W0000T9l00D0049l
000nW@000096_@d0008ef@8000039IT0000000H0025oo`03001oogoo01=oo`03001oogoo021oo`00
7Woo00<005Kl00000UKl0P0000AFo000E_aFo0800004E]`005;LD]`40002C[`01000BY`004ZL3000
00A2_@00A[d00092O0050012F`00?S/00002=QX600027GT2000010DH0000000087oo00<007ooOol0
57oo00<007ooOol07goo000GOol20005Ool00`00E_`0000200000eKl001Fo00300000eKL001Fg003
000015:l001>_0000Tjl10000TZL00L004JL0016W000@Yd00002@Yd;00001chk000f6P00<OX002gJ
00<000057GT001EH00056004000NOol00`00Oomoo`0EOol00`00Oomoo`0NOol001Ioo`04001oogoo
0003Ool01`00Ool00000E_`005Kl00<00003E_`005KL0080009Bg003001B_00000@00004C[`004ZL
0002BY`900002TJm0012_D:m000jW@00>W`003YL2@000RFi0`0000<EF00000000P0000<0U@00Ool0
77oo00<007ooOol05Goo00<007ooOol07Woo000FOol01000Oomoo`000goo00@007ooOomoo`@00003
E_`0000000<00003E]`005;L0080009B_004001>_000C[`5000014Zl0016W0000TJM00<004Jm0000
2P0000LjG000=S/0038K000]nP0300000bFi000MV@0700000`2E001oo`0KOol00`00Oomoo`0FOol0
0`00Oomoo`0MOol001Ioo`04001oogoo0003Ool01@00Oomoogoo00000UKl0P0000AFo000E]aFg0@0
009Bg003001B_00000<0009>_004001:_000B[`700002TJm0016_DJm000n_@00<Wd0039M1P0000<b
6`0000000RWJ100000<IN@0000000P0000<0e`00000077oo00<007ooOol05Woo00<007ooOol07Goo
000FOol01000Oomoo`000goo00<007ooOol00Woo0P0000=Fo000E_`00`0000=Bg000D]`01@0000A>
_000C[`0009>_0@0009:_004001:_000A[d=00000c8l000b700500000bWJ000U^P0300001QUi0000
00DH0000e`8001]oo`03001oogoo01Moo`03001oogoo01aoo`005goo0P0017oo00<007ooOol00goo
00@005Kl0000009Fg004001Bg0000002D]`01000D]`004jl1@0000A>_000B[`0009:_0@0009:_@05
0016_@00A[d00002?Yd01000:Ud002Xm100000@^6`00:O/Yn`@000077IX001Ui000AF@001AP01@00
6Goo00<007ooOol067oo00<007ooOol06goo000MOol00`00Oomoo`04Ool01@00E]aFg000E]`00P00
0U;L00<005;L00000`000Tjl00<004jl00001P0000A:_000B[d0009:_@`00003:S`002XL00<00003
:O/002GJ00<000057IX00000000AF@07000IOol00`00Oomoo`0GOol00`00Oomoo`0KOol001aoo`03
001oogoo00Ioo`04001Fg0000002D]`01P00D]`00000D]`0009Bg006001>_000001>_0000Tjl00<0
04Zl00001P0000A:_@00B[d0009:_@D000079Qd00000000Z7000:O/010000R6j0P0000<IN@000000
100000@0]`00000001Qoo`03001oogoo01Qoo`03001oogoo01Yoo`0077oo00<007ooOol01goo00<0
05KL0000100000=Bg000D]`01@0000=>_000C[`01@0000I:_000B[a:_000B[d700001D[M000jW@00
6Q`0200000LUf`000000026j000MVP0200000a5I0000000400000`2f0000000GOol00`00Oomoo`0H
Ool00`00Oomoo`0JOol001aoo`03001oogoo00Qoo`05001Bg5;L001Bg00500000e;L001>g0040002
C[`01000C[`004Zl1@0000I:_@00B[e:_@00B[d800001Qgl000002Gl000Qn`<000038M/000000080
00035GT0015I008000050AP000000000]P02000FOol00`00Oomoo`0IOol00`00Oomoo`0IOol001ao
o`03001oogoo00Uoo`04001Bg0000002D]`00`00D]`000030002C]`01000C[`004jl1@0000E:_000
B[d004Zm00D00008B]d004[MB]d003ZM000Eo08000037O`000000080008Qf`8000057KX00000000E
N@0200000`di00016006000EOol00`00Oomoo`0JOol00`00Oomoo`0HOol001aoo`03001oogoo00Yo
o`03001Bg000009Bg0800004D]`005;L0002C]`4000014jl001>_4jl1`0000=:_@00B]d02P0000@E
o000000Mo0<000038M/001fk0080008IVP8000034ET000di008000060?P000000000U@0057oo00<0
07ooOol06Woo00<007ooOol067oo000LOol00`00Oomoo`0:Ool00`00D]`0000200000e;L001Bg005
00000dkL001>g005000014jl001:_0000TZm100000I:g@00B]e:g@00B]d600001AWl000000007M/0
100000<I^P006IX00`0000<AF@0000000P0000H0n0000000002E000COol00`00Oomoo`0KOol00`00
Oomoo`0GOol001]oo`03001oogoo00aoo`800003D]`005;L0080009Bg003001Bg00000<0009>g004
001>g000C[`500000d[M001:g@07000014[M000jWSZN1@0000@Ig000000Mf`<000036KX0000000@0
00033ET0000000L0019oo`03001oogoo01]oo`03001oogoo01Moo`006goo00<007ooOol03Woo1000
00=Bg000D]`00P0000I>g000C]a>g000C]`40002C]`00`00C]d000030002B]d01000B]d004[M2@00
00@Eg000000If`<000046K/000006IX30000115i000000Di20004Goo00<007ooOol077oo00<007oo
Ool05Woo000KOol00`00Oomoo`0>Ool00`00D]aBg00400000e;L001>g00500000dkL001>g0050000
14kM001>g@000T[M100000A:g@00B]e:g@<000050I/00000000Ef`0200026K/200001AVj00000000
5GX00P0000<=F@001CT01P0000<0U@00Ool03goo00<007ooOol07Goo00<007ooOol05Goo000KOol0
0`00Oomoo`0?Ool200000e;L001Bg00500000dkL001>g00500000dkL001>g00700000dkM001:g@05
00000dkM000fWP0200001@VK000000005K/00`0000<I^`0000000QFJ100000D=F@000000003h0080
00030;H00000008000ioo`03001oogoo01eoo`03001oogoo01Eoo`006goo00<007ooOol04Goo0`00
0U;L00<005;L00000`0000A>g000C]a>g0@00006C]`004kMC]d004kM10000TkM00<004kM00002000
00DA^`00000001Fk008000035IX0000000T000040?P000000;H4000=Ool00`00Oomoo`0NOol00`00
Oomoo`0DOol001]oo`03001oogoo015oo`03001Bg00000@00003D]`004kL00D00005C]`004kL001>
g00500000dkM001>g@05000014kM001>g@000SIn0P0000D1VP000000016k008000045K/000004IX4
00000`eI0000000;000<Ool00`00Oomoo`0NOol00`00Oomoo`0DOol001Yoo`03001oogoo01=oo`05
001Bg5;L001Bg0040002C]`00`00C]`0000600000dkM001>g@0500000dkM001>g@0;00001@FK0000
00004K/00P0000<AVP004IX0100000@=F@000001>0X000]oo`03001oogoo01moo`03001oogoo01=o
o`006Woo00<007ooOol057oo00H005;L000005;L0002D]`02000C]`00000C]`004kL0002C]`40000
14kM001>gDkM100000I>g@00C]e>g@00C]d800000`fK0000000200000a6J000000024GX300000`Ti
0001>00:000:Ool00`00Oomoo`0POol00`00Oomoo`0BOol001Yoo`03001oogoo01Eoo`04001Bg000
D]`500000dkL001>g00500000dkL001>g@0500000dkM001>g@0700000dkM000fOP0300000`5j0000
000200001@fK000000004IX0100000<9F@0000000P0000@160000000e`<0000309D0000000Uoo`03
001oogoo021oo`03001oogoo019oo`006Woo00<007ooOol05Woo100000=Bg000D]`01@0000=>g000
C]`010000TkM00<004kM00000`0000I>g@00C]e>g@00C]d;00001@VK000000003IX00P0000<=NP00
00000P0000D5>@000CT0004H008000050=L000000000U@020008Ool00`00Oomoo`0QOol00`00Oomo
o`0AOol001Yoo`03001oogoo01Moo`03001Bg00000<00004D]`004kLC]`400000dkL001>g0050000
0dkM001>g@0700000dkM001>g@0:00000`VJ0000000300001Pej000000UI0005>@X0000309D00000
00Moo`03001oogoo025oo`03001oogoo015oo`006Woo00<007ooOol067oo00@005;L001Bg0D00003
C]`004kL00D00003C]`004kM00@0009>g@04001>g@00C]d500000dkM000bOP05000010Ej00000000
0Pej0`0000<9NP002ET00P0000<1>@0000002P001Woo00<007ooOol08Woo00<007ooOol047oo000J
Ool00`00Oomoo`0IOol400000e;L001Bg004000014kL001>g4kL100000=>g@00C]d01`0000A>g@00
C]e>gA4000033GX00000008000031ET0004i00`000Ioo`03001oogoo021oo`<0015oo`006Goo00<0
07ooOol06goo00<005;L00000`0000=Bg000C]`01@0000=>g000C]d01@0000=>g@00C]d01`0000=>
g@00C]d01P0000<1FP0000000`0000L9NP00000000UJ0009F@0500000`4H0000m`0300000`2F0000
U@020005Ool00`00Oomoo`0MOol30002Ool00`00Oomoo`0?Ool001Uoo`03001oogoo01aoo`04001B
g000D]`500000dkL001>g004000014kM001>gDkM100000A>g@00C]d0009>g@@00003C]d0039n00L0
00051GX000000009NP02000010UJ000000Di100000<160000?L00`0000<0UP0000000P0017oo00<0
07ooOol06Woo0`001Goo00<007ooOol03goo000IOol00`00Oomoo`0MOol30002D]`00`00D]`00003
00000dkL001>g00500000dkM001>g@0700000dkM001>g@0@00001`Uj000000001ET000Di0100009o
o`03001oogoo01Moo`<000Qoo`03001oogoo00moo`006Goo00<007ooOol07Goo00<005;L00001000
00=Bg000C]`01@0000=>g000C]d0100000A>g@00C]e>g@H00004C]d004kMC]d800000`5J00000004
00000`UJ000000020000104i0000004H1@0000<0]P000000100000=oo`00Ool05Goo0`002goo00<0
07ooOol03goo000IOol00`00Oomoo`0NOol200000e;L001Bg004000014kL001>g4kL100000=>g@00
C]d01@0000=>g@00C]d01`0000=>g@00<Wd0200000<5NP001GX00`0000D5F@000000004i010001=o
o`<000moo`03001oogoo00ioo`0067oo0`008Goo0`0000=Bg000D]`01@0000=>g000C]`0100000A>
g@00C]e>g@@00003C]d004kM00`000030ET0000000D000031ET000EI00@000040CT000000?P30000
0`2f00000005000@Ool3000BOol00`00Oomoo`0>Ool000]oo`<000Aoo`8000Aoo`03001oogoo025o
o`03001Bg00000@00003D]`004kL00@00003C]`004kM00D00003C]d004kM00@0009>g@03001>g@00
00d000031EX0000000<000050CT000000001>@0200001@3h000000000;H01P003Goo0`005Goo00<0
07ooOol03Woo000<Ool00`00Oomoo`02Ool01000Oomoo`000goo00<007ooOol08Woo00D005;LD]`0
05;L00@00003C]`004kL00D00003C]d004kM00@00003C]d004kM00D00003C]d004kM010000031ET0
0000014000Uoo`<001Qoo`03001oogoo00ioo`0037oo00<007ooOol00Woo00@007ooOol000=oo`03
001oogoo02=oo`@00003D]`005;L00@00004C]`004kLC]`400000dkM001>g@0500000dkM001>g@04
00000dkM000^O@0900020EX400000`4i0001>@080000103G0000002F0`001Woo0`006goo00<007oo
Ool03Woo000<Ool00`00Oomoo`02Ool01000Oomoo`000goo00<007ooOol097oo00<005;L00000`00
00=Bg000C]`01@0000=>g@00C]d0100000A>g@00C]e>g@@00003C]d004kM018000030ET0004i00X0
00040=L0000009H30002Ool3000NOol00`00Oomoo`0>Ool000aoo`03001oogoo009oo`04001oogoo
0003Ool00`00Oomoo`0UOol01000D]`005;L1@0000=>g0000000100000=>g@00C]d01@0000=>g@00
C]d010000TkM00<004kM0000200000<1F@000ET01P0000@1>@000001608000030?P0003G00T0025o
o`03001oogoo00ioo`002goo0P001Goo0P0017oo00<007ooOol09Woo0`000U;L100000=>g000C]`0
1P000TkM1P0000=>g@0000001@0000=>g@00;Wd02`0000<1F@0000000P0000<1>@0000000`0000<0
n0000=L01`008goo00<007ooOol03Goo000GOol2000YOol400000e;L001Bg00400000dkL001>g@05
00000dkM001>g@04000014kM001>gDkM6@0000<160000AP02@0000<0U@00Ool08Goo00<007ooOol0
3Goo000GOol00`00Oomoo`0YOol00`00D]`0000300000e;L001Bg00500000dkM001>g@0400000dkM
001>g@0500000dkM001>g@0800000`4H0000000300001@4i000000000CT0100000<160000AP02@00
00<0U@00Ool087oo00<007ooOol03Goo000GOol00`00Oomoo`0ZOol01000D]`005;L100000ABg000
C]a>g0@00003C]d004kM00@00004C]d004kMC]d400000dkM001>g@0Q00000`2E001oo`0OOol00`00
Oomoo`0=Ool001Moo`03001oogoo02]oo`<00003D]`005;L00D00003C]`004kM00@00003C]d004kM
00D00003C]d004kM00@00003C]d002iM00P000050CT000000001>@020000104i0000004H1@0000<0
e`0000001@0087oo00<007ooOol03Goo000GOol00`00Oomoo`0[Ool00`00D]`0000400000e;L0000
000400000dkM000000040002C]d600000dkM0000000Q0000102f000000007goo00<007ooOol03Goo
000GOol00`00Oomoo`0/Ool200000e;L0000000200000e;L001Bg00400000dkM001>g@0500000dkM
001>g@0400000dkM001>g@0;00001@4I000000000CT00`0000H1>@00000160000AP800000`2f0000
000OOol00`00Oomoo`0=Ool001Ioo`03001oogoo02moo`04001Bg000D]`400000e;L001Bg0050000
0dkM001>g@0400000dkM001>g@040002C]d00`00C]d0000M00000`2f0000000OOol00`00Oomoo`0<
Ool001Ioo`80035oo`<00003D]`005;L00@00004D]`004kMC]d400000dkM001>g@0400000dkM001>
g@0500000dkM0000000800001@4i000000000CT00P0000@16000000160`001ioo`03001oogoo00ao
o`005Woo00<007ooOol0<Goo1@0000=Bg00000001@0000=Bg@000000100000=>g@00000010000TkM
100000=Bg@00:Ud04P0000D160000000003g008000030=L00000008001ioo`03001oogoo00aoo`00
5Woo00<007ooOol0<Goo00<005;L0000100000=Bg000D]`0100000=Bg@00C]d0100000=>g@00C]d0
1@0000=>g@00C]d0300000@16@000001>@<000030AP0004H00P000030=L00000008001eoo`03001o
ogoo00aoo`005Woo00<007ooOol0<Woo0P0000=Bg000D]`0100000=Bg000D]`0100000=Bg@00C]d0
100000A>g@00C]e>g@@00003D]d0000001<000050AP000000000n0040000103G000000007Goo00<0
07ooOol037oo000FOol00`00Oomoo`0dOol300000e;L0000000400000e;L0000000400000e;M0000
000500000e;M0000000200000e;M001Bg@0400000`2e000000050000104I0000004H400077oo00<0
07ooOol037oo000EOol00`00Oomoo`0fOol300000e;L001Bg00400000e;L001Bg@04000015;M001B
gE;M100000=Bg@00D]d0100000=Bg@00:Ud0500000@0n0000000e`H001]oo`03001oogoo00aoo`00
5Goo00<007ooOol0=Woo00<005;L00000`0000=Bg000D]`0100000=Bg@0000001@0000=Bg@000000
100000=Bg@0000002P0000<0n00000000`0000@16000000160D000040?P000000=L200000`2f0000
000KOol00`00Oomoo`0<Ool001Eoo`03001oogoo03Moo`03001Bg00000D00003D]`0000000<00003
D]d005;M00@00003D]d005;M00@000Uoo`L000030AP0000000L000030?P00000008000060=L00000
0000]P006goo00<007ooOol02goo000EOol2000iOol01000D]`005;L1@0000=Bg000D]`0100000=B
g@00D]d010003Woo2@0000<1600000003P0000<0]P00Ool06Goo00<007ooOol02goo000EOol00`00
Oomoo`0iOol400000e;L0000000400000e;L00000004000BOol300000`3G0000000900000`4H0000
n00900000`2f001oo`0HOol00`00Oomoo`0;Ool001Eoo`03001oogoo03]oo`<00003D]`005;L00@0
01Uoo`/000030AP0000000h001Uoo`03001oogoo00]oo`0057oo00<007ooOol0?Goo1@007Woo2000
00<1600000001@0000<0n00000000`0000<0e`0000000`0067oo00<007ooOol02goo000DOol00`00
Oomoo`0mOol00`00Oomoo`0MOol>00000`4H000000030000103h0000003h1`0067oo00<007ooOol0
2goo000DOol00`00Oomoo`0mOol00`00Oomoo`0JOol30005Ool40000103g0000004H2`0000D0n000
0000003G008001Qoo`03001oogoo00]oo`0057oo00<007ooOol0?Goo00<007ooOol05goo0`002Goo
2`0000<1600000002@0000<0e`00000067oo00<007ooOol02Woo000DOol2000nOol00`00Oomoo`0D
Ool3000=Ool800000`4H0000000<00000`3G0000000GOol00`00Oomoo`0:Ool001Aoo`03001oogoo
03eoo`03001oogoo015oo`<0019oo`T000030AP0000000d001Ioo`03001oogoo00Yoo`004goo00<0
07ooOol0?Woo00<007ooOol03Goo10005Woo00<0003F00000`0000<0n00000000P0000<0n0000000
0`0000<0m`00000010005Woo00<007ooOol02Woo000COol00`00Oomoo`0nOol00`00Oomoo`0:Ool3
000JOol900000`3h0000000300000`3h0000000500000`3G001oo`0DOol00`00Oomoo`0:Ool001=o
o`03001oogoo03ioo`03001oogoo00Moo`<001iooaP001Eoo`03001oogoo00Yoo`004goo00<007oo
Ool0?Woo00<007ooOol017oo0`008Woo0`0000<0e`000000100000D0n0000000003h00T001Aoo`03
001oogoo00Yoo`004goo00<007ooOol0?Woo00<007ooOol010009goo400000H0m`000000003G000E
Ool00`00Oomoo`09Ool001=oo`03001oogoo03eoo`@002aoo`P000050?P000000000n009000DOol0
0`00Oomoo`09Ool001=oo`8003aoo`<0031oo`l000030?L0000000@001=oo`03001oogoo00Uoo`00
1Goo10000goo0P0017oo00<007ooOol0>Goo0`000Woo00<007ooOol0;goo1@0000<0m`0000001`00
00<0m`0000000`004goo00<007ooOol02Goo0006Ool00`00Oomoo`02Ool01000Oomoo`000goo00<0
07ooOol0=Woo0`001Goo00<007ooOol0<Goo50004Woo00<007ooOol02Goo0007Ool01@00Oomoogoo
00000Woo00D007ooOomoo`0003Aoo`@000Qoo`03001oogoo039oo`L000030?P0000000T0019oo`03
001oogoo00Uoo`0027oo00@007ooOol0009oo`05001oogooOol0000aOol3000<Ool00`00Oomoo`0b
Ool=00000`3g00000004000AOol00`00Oomoo`09Ool000Eoo`04001oogoo0002Ool01000Oomoo`00
0goo00<007ooOol0;7oo0`003goo00<007ooOol0<goo1@0000<0m`000000200000<0e`0000004Goo
00<007ooOol02Goo0006Ool20004Ool20004Ool00`00Oomoo`0YOol3000BOol00`00Oomoo`0dOolC
000AOol00`00Oomoo`08Ool0015oo`03001oogoo02Ioo`@001Eoo`03001oogoo03Iooa80011oo`03
001oogoo00Qoo`004Goo0P0097oo0`006Goo00<007ooOol0=goo2`0000<0m`00000010003goo00<0
07ooOol027oo000AOol00`00Oomoo`0POol3000LOol00`00Oomoo`0hOol900000`3g00000005000?
Ool00`00Oomoo`08Ool0015oo`03001oogoo01eoo`<001moo`03001oogoo03Uoo`D000030?L0003g
00T000ioo`03001oogoo00Qoo`004Goo00<007ooOol06Woo0`008Woo00<007ooOol0>Woo40003Woo
00<007ooOol027oo000AOol00`00Oomoo`0FOol4000UOol00`00Oomoo`0kOol@000=Ool00`00Oomo
o`08Ool0011oo`03001oogoo01Aoo`<002Uoo`03001oogoo03aoo`l000ioo`03001oogoo00Moo`00
47oo00<007ooOol04Goo0`00;7oo00<007ooOol0?Woo300000<0e`00Ool037oo00<007ooOol01goo
000@Ool00`00Oomoo`0>Ool3000_Ool00`00Oomoo`0oOol:0000103G0000000037oo00<007ooOol0
1goo000@Ool2000;Ool4000bOol00`00Oomoo`10Ool>000;Ool00`00Oomoo`07Ool0011oo`03001o
ogoo00Moo`<003Ioo`03001oogoo045oo`d000]oo`03001oogoo00Moo`0047oo00<007ooOol017oo
0`00>Goo00<007ooOol0@Woo3@002Woo00<007ooOol01goo000?Ool00`00Oomoo`02Ool3000lOol0
0`00Oomoo`13Ool<000:Ool00`00Oomoo`07Ool000moo`03001oo`00008003moo`03001oogoo04Ao
o``000Yoo`03001oogoo00Ioo`003goo0P00@Woo00<007ooOol0AGoo2`002Woo00<007ooOol01Woo
000AOol00`00Oomoo`0oOol00`00Oomoo`16Ool;0009Ool00`00Oomoo`06Ool0019oo`8003moo`03
001oogoo04Moo`X000Uoo`03001oogoo00Ioo`004goo0P00?Woo00<007ooOol0BGoo2@0027oo00<0
07ooOol01Woo000EOol00`00Oomoo`0kOol00`00Oomoo`1:Ool90007Ool00`00Oomoo`06Ool001Io
o`8003Yoo`03001oogoo04aoo`P000Moo`03001oogoo00Ioo`0067oo00<007ooOol0=goo00<007oo
Ool0CGoo1P0000<0e`00Ool01Goo00<007ooOol01Woo000IOol3000fOol00`00Oomoo`1>Ool80006
Ool00`00Oomoo`05Ool001]oo`03001oogoo03Aoo`03001oogoo04moo`@000040=L000000005Ool0
0`00Oomoo`05Ool001aoo`8003Aoo`03001oogoo051oo`L000Eoo`03001oogoo00Eoo`007Woo0P00
<Woo00<007ooOol0DGoo0`0000@0e`00000000Aoo`03001oogoo00Eoo`007goo0P00<Goo00<007oo
Ool0Dgoo1P000goo00<007ooOol01Goo000QOol00`00Oomoo`0^Ool00`00Oomoo`1DOol50003Ool0
0`00Oomoo`05Ool0029oo`8002ioo`03001oogoo05Eoo`D0009oo`03001oogoo00Eoo`008goo0P00
;Goo00<007ooOol0EWoo10000goo00<007ooOol017oo000UOol2000[Ool00`00Oomoo`1FOol00`00
Ool000020002Ool00`00Oomoo`04Ool001Ioo`8000Aoo`8000=oo`@00003Ool0000002Yoo`03001o
ogoo05Moo`03001oo`0000800003Ool007oo00Eoo`005Goo00@007ooOol000=oo`8000Aoo`03001o
ogoo00=oo`03001oogoo02Moo`03001oogoo05Qoo`06001oo`00001oo`001Woo000EOol01000Oomo
o`002Woo00<007ooOol00goo0P009goo00<007ooOol0FGoo00<007oo00000P001Woo000EOol01000
Oomoo`002goo00<007ooOol017oo00<007ooOol097oo00<007ooOol0FWoo00@007oo000000Ioo`00
5Goo00@007ooOol000Qoo`04001oogoo0007Ool2000TOol00`00Oomoo`1JOol01000Oomoo`001Woo
000FOol2000:Ool20009Ool2000SOol00`00Oomoo`1KOol01000Oomoo`001Goo000_Ool2000QOol0
0`00Oomoo`1LOol00`00Ool00005Ool0031oo`80021oo`03001oogoo05eoo`8000Eoo`00<Woo00<0
07ooOol07Goo00<007ooOol0G7oo0`001Goo000cOol2000MOol00`00Oomoo`1GOol50008Ool002Eo
o`8000Aoo`8000Aoo`<00003Ool007oo01]oo`03001oogoo05=oo`@000eoo`0097oo00@007ooOol0
00=oo`8000Eoo`04001oogooOol2000JOol00`00Oomoo`1>Ool5000AOol002Aoo`04001oogoo0008
Ool40003Ool2000IOol00`00Oomoo`1:Ool4000FOol002Aoo`04001oogoo0008Ool00`00Ool00006
Ool2000GOol00`00Oomoo`15Ool5000JOol002Aoo`04001oogoo0008Ool00`00Ool00007Ool2000F
Ool00`00Oomoo`11Ool4000OOol002Eoo`8000Yoo`8000Uoo`8001Aoo`03001oogoo03eoo`@002=o
o`00?Goo0P004goo00<007ooOol0>7oo1@009goo000oOol00`00Oomoo`0?Ool00`00Oomoo`0eOol4
000/Ool0041oo`8000moo`03001oogoo031oo`D0031oo`00<Woo0P0017oo0P0017oo0P000Woo00<0
07ooOol037oo00<007ooOol0;7oo1000=Goo000aOol01000Oomoo`000goo0P000goo00@007ooOol0
009oo`03001oogoo00]oo`03001oogoo02Moo`D003Uoo`00<Goo00@007ooOol000Qoo`04001oogoo
0003Ool3000:Ool00`00Oomoo`0SOol4000nOol0035oo`04001oogoo0008Ool30006Ool00`00Oomo
o`08Ool00`00Oomoo`0NOol50012Ool0035oo`04001oogoo0008Ool00`00Oomoo`07Ool20008Ool0
0`00Oomoo`0JOol40017Ool0039oo`8000Yoo`<000Qoo`03001oogoo00Eoo`03001oogoo01Eoo`D0
04]oo`00BWoo0P001Goo00<007ooOol04Goo1000D7oo001<Ool00`00Oomoo`02Ool00`00Oomoo`0=
Ool4001DOol004eoo`05001oogooOol0000:Ool5001HOol003ioo`8000Aoo`8000Aoo`80009oo`80
0003Ool007oo00Eoo`@005eoo`00?Goo00@007ooOol000=oo`8000=oo`04001oogoo0003Ool20000
0goo00000003001QOol003eoo`04001oogoo0008Ool01000Oomoo`0017oo0P00IWoo000mOol01000
Oomoo`002Goo0P00KGoo000mOol01000Oomoo`0027oo00@007ooOol006aoo`00?Woo0P002Woo0P00
KGoo002iOol004moo`<006Moo`00D7oo00<007ooOol0IWoo001@Ool00`00Oomoo`1VOol0051oo`03
001oogoo06Ioo`00D7oo00<007ooOol0IWoo001?Ool2001XOol00;Uoo`00\
\>"],
  ImageRangeCache->{{{0, 184.688}, {200.5, 0}} -> {-0.0340624, -0.0577046, 
  0.00522764, 0.00522764}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Plot3D[If[S>95, 
DownAndOutCallDelta[2,S,100,95,0.2,Log[1.1],Log[1.05],t], 
UpAndOutCallDelta[2,S,100,95,0.2,Log[1.1],Log[1.05],t]], {S, 80, 120}, {t, \
0.0001, 1}, BoxRatios -> {1, 1, 1}, PlotPoints -> 40];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.0855 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0567234 1.16608 0 1.16608 [
[.07825 .23514 -11.8839 -9 ]
[.07825 .23514 .11606 0 ]
[.21247 .17844 -11.3335 -9 ]
[.21247 .17844 .66648 0 ]
[.3545 .11847 -16.1747 -9 ]
[.3545 .11847 1.82534 0 ]
[.50502 .05496 -15.349 -9 ]
[.50502 .05496 2.65097 0 ]
[.66483 -0.01242 -14.5234 -9 ]
[.66483 -0.01242 3.4766 0 ]
[.09301 .88495 -10.1462 0 ]
[.09301 .88495 7.85383 9 ]
[.17991 .94587 -9.90019 0 ]
[.17991 .94587 8.09981 9 ]
[.25944 1.00163 -9.68252 0 ]
[.25944 1.00163 8.31748 9 ]
[.33251 1.05285 -9.48854 0 ]
[.33251 1.05285 8.51146 9 ]
[.39987 1.10006 -3.10486 0 ]
[.39987 1.10006 2.89514 9 ]
[.07034 .28047 -12 -2.6338 ]
[.07034 .28047 0 6.3662 ]
[.05238 .39292 -24 -2.73378 ]
[.05238 .39292 0 6.26622 ]
[.0329 .51482 -6 -2.84304 ]
[.0329 .51482 0 6.15696 ]
[.01172 .64742 -18 -2.9629 ]
[.01172 .64742 0 6.0371 ]
[-0.01139 .79219 -6 -3.09502 ]
[-0.01139 .79219 0 5.90498 ]
[ 0 0 0 0 ]
[ 1 1.0855 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
[(80)] .07825 .23514 .98066 1 Mshowa
.22216 .18933 m
.227 .19478 L
s
[(90)] .21247 .17844 .88892 1 Mshowa
.36358 .12987 m
.36813 .13557 L
s
[(100)] .3545 .11847 .79718 1 Mshowa
.51343 .06687 m
.51763 .07282 L
s
[(110)] .50502 .05496 .70545 1 Mshowa
.67245 0 m
.67627 .00621 L
s
[(120)] .66483 -0.01242 .61371 1 Mshowa
.125 Mabswid
.11461 .23456 m
.11764 .23771 L
s
.14105 .22344 m
.14405 .22662 L
s
.16779 .2122 m
.17076 .21541 L
s
.19482 .20083 m
.19776 .20407 L
s
.2498 .17771 m
.25268 .18101 L
s
.27776 .16595 m
.2806 .16928 L
s
.30604 .15406 m
.30885 .15742 L
s
.33465 .14204 m
.33741 .14542 L
s
.39286 .11756 m
.39554 .12101 L
s
.42247 .10511 m
.42512 .10859 L
s
.45243 .09251 m
.45504 .09602 L
s
.48275 .07977 m
.48531 .08331 L
s
.54447 .05381 m
.54695 .05742 L
s
.57589 .0406 m
.57832 .04424 L
s
.60769 .02723 m
.61007 .0309 L
s
.63987 .0137 m
.64221 .0174 L
s
.25 Mabswid
0 .80374 m
.40037 1.0855 L
s
.09485 .87049 m
.09577 .86326 L
s
[(0.2)] .09301 .88495 .12735 -1 Mshowa
.18136 .93137 m
.18209 .92412 L
s
[(0.4)] .17991 .94587 .10002 -1 Mshowa
.26055 .98709 m
.2611 .97983 L
s
[(0.6)] .25944 1.00163 .07584 -1 Mshowa
.3333 1.03829 m
.33369 1.03102 L
s
[(0.8)] .33251 1.05285 .05428 -1 Mshowa
.40037 1.0855 m
.40063 1.07821 L
s
[(1)] .39987 1.10006 .03495 -1 Mshowa
.125 Mabswid
.02453 .821 m
.02518 .81668 L
s
.04852 .83789 m
.04914 .83356 L
s
.07196 .85438 m
.07254 .85004 L
s
.11722 .88623 m
.11774 .88189 L
s
.13908 .90162 m
.13958 .89727 L
s
.16046 .91666 m
.16092 .91231 L
s
.20181 .94576 m
.20221 .9414 L
s
.22181 .95983 m
.22219 .95548 L
s
.24138 .97361 m
.24174 .96925 L
s
.27931 1.0003 m
.27961 .99593 L
s
.29768 1.01322 m
.29796 1.00886 L
s
.31567 1.02589 m
.31593 1.02152 L
s
.35057 1.05045 m
.35079 1.04608 L
s
.3675 1.06236 m
.3677 1.058 L
s
.3841 1.07404 m
.38427 1.06967 L
s
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
.08381 .27488 m
.09054 .27209 L
s
[(-1)] .07034 .28047 1 -0.41471 Mshowa
.06595 .3876 m
.07273 .38493 L
s
[(-0.5)] .05238 .39292 1 -0.39249 Mshowa
.04658 .50979 m
.05342 .50727 L
s
[(0)] .0329 .51482 1 -0.36821 Mshowa
.02552 .64271 m
.03242 .64035 L
s
[(0.5)] .01172 .64742 1 -0.34158 Mshowa
.00252 .78784 m
.00948 .78567 L
s
[(1)] -0.01139 .79219 1 -0.31222 Mshowa
.125 Mabswid
.08035 .29672 m
.08439 .29506 L
s
.07683 .3189 m
.08088 .31726 L
s
.07326 .34144 m
.07732 .33981 L
s
.06963 .36433 m
.0737 .36272 L
s
.0622 .41124 m
.06628 .40966 L
s
.05839 .43527 m
.06248 .4337 L
s
.05452 .4597 m
.05861 .45815 L
s
.05058 .48453 m
.05468 .483 L
s
.04251 .53547 m
.04662 .53398 L
s
.03837 .56159 m
.04249 .56012 L
s
.03416 .58816 m
.03829 .58671 L
s
.02988 .6152 m
.03401 .61377 L
s
.02108 .67071 m
.02523 .66932 L
s
.01657 .69921 m
.02072 .69783 L
s
.01197 .72822 m
.01613 .72687 L
s
.00729 .75776 m
.01145 .75643 L
s
.08722 .25338 m
.09125 .25169 L
s
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
.41799 .58158 m
.08846 .24555 L
s
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.41799 .58158 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
0 0 m
1 0 L
1 1.0855 L
0 1.0855 L
closepath
clip
newpath
.5 Mabswid
.634 .752 .933 r
.40125 .83369 .40895 .8402 .4218 .83718 .41414 .83065 Metetra
.636 .753 .933 r
.41414 .83065 .4218 .83718 .43472 .83408 .4271 .82755 Metetra
.638 .754 .932 r
.4271 .82755 .43472 .83408 .44772 .83091 .44014 .82437 Metetra
.64 .755 .932 r
.44014 .82437 .44772 .83091 .46078 .82767 .45324 .82111 Metetra
.642 .756 .932 r
.45324 .82111 .46078 .82767 .47391 .82435 .46642 .81778 Metetra
.644 .757 .931 r
.46642 .81778 .47391 .82435 .48712 .82096 .47967 .81436 Metetra
.646 .758 .931 r
.47967 .81436 .48712 .82096 .50039 .81748 .49299 .81087 Metetra
.648 .759 .931 r
.49299 .81087 .50039 .81748 .51374 .81393 .50638 .80729 Metetra
.649 .76 .93 r
.50638 .80729 .51374 .81393 .52716 .8103 .51984 .80364 Metetra
.651 .761 .93 r
.51984 .80364 .52716 .8103 .54064 .8066 .53337 .79991 Metetra
.653 .761 .929 r
.53337 .79991 .54064 .8066 .5542 .80283 .54698 .7961 Metetra
.655 .762 .929 r
.54698 .7961 .5542 .80283 .56782 .79899 .56065 .79222 Metetra
.656 .762 .928 r
.56065 .79222 .56782 .79899 .58152 .79508 .57439 .78828 Metetra
.657 .763 .927 r
.57439 .78828 .58152 .79508 .59529 .7911 .58821 .78426 Metetra
.959 .665 .432 r
.58821 .78426 .59529 .7911 .61496 1.01997 .60741 1.01156 Metetra
.756 .761 .857 r
.60741 1.01156 .61496 1.01997 .6297 1.01064 .62221 1.00224 Metetra
.745 .76 .865 r
.62221 1.00224 .6297 1.01064 .6445 1.00204 .63707 .99366 Metetra
.734 .759 .873 r
.63707 .99366 .6445 1.00204 .65936 .9941 .652 .98574 Metetra
.723 .758 .881 r
.652 .98574 .65936 .9941 .6743 .98676 .667 .97844 Metetra
.712 .758 .888 r
.667 .97844 .6743 .98676 .68932 .97996 .68208 .97168 Metetra
.702 .757 .895 r
.68208 .97168 .68932 .97996 .70442 .97365 .69725 .96541 Metetra
.692 .756 .901 r
.69725 .96541 .70442 .97365 .71961 .96778 .7125 .9596 Metetra
.682 .755 .907 r
.7125 .9596 .71961 .96778 .73489 .96231 .72784 .95418 Metetra
.673 .755 .912 r
.72784 .95418 .73489 .96231 .75027 .95719 .74329 .94911 Metetra
.664 .755 .917 r
.74329 .94911 .75027 .95719 .76575 .95238 .75883 .94436 Metetra
.656 .755 .922 r
.75883 .94436 .76575 .95238 .78133 .94785 .77448 .93988 Metetra
.649 .755 .926 r
.77448 .93988 .78133 .94785 .79701 .94356 .79023 .93565 Metetra
.642 .755 .93 r
.79023 .93565 .79701 .94356 .8128 .93948 .80608 .93163 Metetra
.636 .755 .934 r
.80608 .93163 .8128 .93948 .82869 .93558 .82204 .92778 Metetra
.631 .756 .937 r
.82204 .92778 .82869 .93558 .84469 .93183 .83812 .92409 Metetra
.626 .756 .94 r
.83812 .92409 .84469 .93183 .8608 .92822 .8543 .92052 Metetra
.621 .757 .943 r
.8543 .92052 .8608 .92822 .87702 .92471 .87059 .91706 Metetra
.617 .758 .946 r
.87059 .91706 .87702 .92471 .89335 .92129 .887 .91368 Metetra
.614 .759 .948 r
.887 .91368 .89335 .92129 .90979 .91794 .90351 .91037 Metetra
.611 .76 .95 r
.90351 .91037 .90979 .91794 .92634 .91464 .92014 .9071 Metetra
.609 .761 .952 r
.92014 .9071 .92634 .91464 .943 .91138 .93688 .90387 Metetra
.607 .762 .954 r
.93688 .90387 .943 .91138 .95977 .90815 .95374 .90066 Metetra
.605 .763 .955 r
.95374 .90066 .95977 .90815 .97666 .90492 .9707 .89746 Metetra
.604 .765 .956 r
.9707 .89746 .97666 .90492 .99365 .9017 .98778 .89425 Metetra
.633 .751 .933 r
.39348 .82711 .40125 .83369 .41414 .83065 .40641 .82407 Metetra
.635 .753 .933 r
.40641 .82407 .41414 .83065 .4271 .82755 .41941 .82095 Metetra
.637 .754 .933 r
.41941 .82095 .4271 .82755 .44014 .82437 .43248 .81777 Metetra
.639 .755 .932 r
.43248 .81777 .44014 .82437 .45324 .82111 .44563 .8145 Metetra
.641 .756 .932 r
.44563 .8145 .45324 .82111 .46642 .81778 .45885 .81115 Metetra
.643 .757 .932 r
.45885 .81115 .46642 .81778 .47967 .81436 .47214 .80772 Metetra
.645 .758 .931 r
.47214 .80772 .47967 .81436 .49299 .81087 .48551 .8042 Metetra
.647 .759 .931 r
.48551 .8042 .49299 .81087 .50638 .80729 .49894 .8006 Metetra
.649 .76 .93 r
.49894 .8006 .50638 .80729 .51984 .80364 .51245 .79692 Metetra
.651 .761 .93 r
.51245 .79692 .51984 .80364 .53337 .79991 .52603 .79316 Metetra
.653 .762 .929 r
.52603 .79316 .53337 .79991 .54698 .7961 .53968 .78932 Metetra
.654 .762 .929 r
.53968 .78932 .54698 .7961 .56065 .79222 .55341 .78541 Metetra
.656 .763 .928 r
.55341 .78541 .56065 .79222 .57439 .78828 .5672 .78142 Metetra
.658 .763 .928 r
.5672 .78142 .57439 .78828 .58821 .78426 .58106 .77736 Metetra
.959 .665 .432 r
.58106 .77736 .58821 .78426 .60741 1.01156 .5998 1.00301 Metetra
.755 .76 .857 r
.5998 1.00301 .60741 1.01156 .62221 1.00224 .61466 .99371 Metetra
.743 .759 .865 r
.61466 .99371 .62221 1.00224 .63707 .99366 .62958 .98514 Metetra
.732 .758 .873 r
.62958 .98514 .63707 .99366 .652 .98574 .64457 .97726 Metetra
.721 .757 .881 r
.64457 .97726 .652 .98574 .667 .97844 .65963 .96998 Metetra
.71 .756 .888 r
.65963 .96998 .667 .97844 .68208 .97168 .67477 .96327 Metetra
.699 .755 .895 r
.67477 .96327 .68208 .97168 .69725 .96541 .69 .95705 Metetra
.689 .754 .901 r
.69 .95705 .69725 .96541 .7125 .9596 .70531 .95129 Metetra
.68 .754 .907 r
.70531 .95129 .7125 .9596 .72784 .95418 .72072 .94593 Metetra
.67 .753 .913 r
.72072 .94593 .72784 .95418 .74329 .94911 .73623 .94092 Metetra
.662 .753 .918 r
.73623 .94092 .74329 .94911 .75883 .94436 .75184 .93623 Metetra
.654 .753 .923 r
.75184 .93623 .75883 .94436 .77448 .93988 .76755 .93182 Metetra
.646 .753 .927 r
.76755 .93182 .77448 .93988 .79023 .93565 .78337 .92765 Metetra
.639 .753 .931 r
.78337 .92765 .79023 .93565 .80608 .93163 .7993 .92368 Metetra
.633 .754 .935 r
.7993 .92368 .80608 .93163 .82204 .92778 .81533 .9199 Metetra
.628 .754 .938 r
.81533 .9199 .82204 .92778 .83812 .92409 .83148 .91626 Metetra
.623 .755 .941 r
.83148 .91626 .83812 .92409 .8543 .92052 .84773 .91274 Metetra
.618 .756 .944 r
.84773 .91274 .8543 .92052 .87059 .91706 .8641 .90933 Metetra
.615 .757 .946 r
.8641 .90933 .87059 .91706 .887 .91368 .88058 .906 Metetra
.611 .758 .949 r
.88058 .906 .887 .91368 .90351 .91037 .89717 .90272 Metetra
.609 .759 .951 r
.89717 .90272 .90351 .91037 .92014 .9071 .91388 .89949 Metetra
.606 .76 .953 r
.91388 .89949 .92014 .9071 .93688 .90387 .9307 .89629 Metetra
.605 .762 .954 r
.9307 .89629 .93688 .90387 .95374 .90066 .94764 .89311 Metetra
.603 .763 .956 r
.94764 .89311 .95374 .90066 .9707 .89746 .96469 .88992 Metetra
.602 .764 .957 r
.96469 .88992 .9707 .89746 .98778 .89425 .98185 .88673 Metetra
.632 .751 .933 r
.38563 .82046 .39348 .82711 .40641 .82407 .3986 .81742 Metetra
.634 .752 .933 r
.3986 .81742 .40641 .82407 .41941 .82095 .41164 .8143 Metetra
.636 .754 .933 r
.41164 .8143 .41941 .82095 .43248 .81777 .42476 .8111 Metetra
.638 .755 .933 r
.42476 .8111 .43248 .81777 .44563 .8145 .43795 .80782 Metetra
.64 .756 .933 r
.43795 .80782 .44563 .8145 .45885 .81115 .45121 .80446 Metetra
.642 .757 .932 r
.45121 .80446 .45885 .81115 .47214 .80772 .46455 .80101 Metetra
.645 .758 .932 r
.46455 .80101 .47214 .80772 .48551 .8042 .47796 .79747 Metetra
.647 .759 .931 r
.47796 .79747 .48551 .8042 .49894 .8006 .49144 .79385 Metetra
.649 .76 .931 r
.49144 .79385 .49894 .8006 .51245 .79692 .50499 .79014 Metetra
.651 .761 .93 r
.50499 .79014 .51245 .79692 .52603 .79316 .51862 .78635 Metetra
.653 .762 .93 r
.51862 .78635 .52603 .79316 .53968 .78932 .53232 .78248 Metetra
.654 .763 .929 r
.53232 .78248 .53968 .78932 .55341 .78541 .54609 .77853 Metetra
.656 .763 .928 r
.54609 .77853 .55341 .78541 .5672 .78142 .55993 .77451 Metetra
.658 .763 .928 r
.55993 .77451 .5672 .78142 .58106 .77736 .57385 .77041 Metetra
.959 .665 .431 r
.57385 .77041 .58106 .77736 .5998 1.00301 .59211 .99432 Metetra
.754 .758 .856 r
.59211 .99432 .5998 1.00301 .61466 .99371 .60703 .98503 Metetra
.742 .757 .865 r
.60703 .98503 .61466 .99371 .62958 .98514 .62201 .97648 Metetra
.73 .756 .873 r
.62201 .97648 .62958 .98514 .64457 .97726 .63706 .96863 Metetra
.719 .755 .881 r
.63706 .96863 .64457 .97726 .65963 .96998 .65219 .9614 Metetra
.708 .754 .888 r
.65219 .9614 .65963 .96998 .67477 .96327 .66739 .95473 Metetra
.697 .753 .895 r
.66739 .95473 .67477 .96327 .69 .95705 .68268 .94857 Metetra
.687 .753 .901 r
.68268 .94857 .69 .95705 .70531 .95129 .69806 .94287 Metetra
.677 .752 .908 r
.69806 .94287 .70531 .95129 .72072 .94593 .71353 .93757 Metetra
.668 .752 .913 r
.71353 .93757 .72072 .94593 .73623 .94092 .7291 .93263 Metetra
.659 .751 .918 r
.7291 .93263 .73623 .94092 .75184 .93623 .74478 .928 Metetra
.651 .751 .923 r
.74478 .928 .75184 .93623 .76755 .93182 .76056 .92366 Metetra
.643 .751 .928 r
.76056 .92366 .76755 .93182 .78337 .92765 .77644 .91955 Metetra
.637 .752 .932 r
.77644 .91955 .78337 .92765 .7993 .92368 .79244 .91565 Metetra
.63 .752 .935 r
.79244 .91565 .7993 .92368 .81533 .9199 .80855 .91192 Metetra
.625 .753 .939 r
.80855 .91192 .81533 .9199 .83148 .91626 .82476 .90834 Metetra
.62 .754 .942 r
.82476 .90834 .83148 .91626 .84773 .91274 .84109 .90488 Metetra
.616 .755 .945 r
.84109 .90488 .84773 .91274 .8641 .90933 .85754 .90152 Metetra
.612 .756 .947 r
.85754 .90152 .8641 .90933 .88058 .906 .8741 .89824 Metetra
.609 .757 .949 r
.8741 .89824 .88058 .906 .89717 .90272 .89077 .895 Metetra
.606 .758 .951 r
.89077 .895 .89717 .90272 .91388 .89949 .90755 .89181 Metetra
.604 .76 .953 r
.90755 .89181 .91388 .89949 .9307 .89629 .92446 .88865 Metetra
.602 .761 .955 r
.92446 .88865 .9307 .89629 .94764 .89311 .94147 .88549 Metetra
.601 .762 .956 r
.94147 .88549 .94764 .89311 .96469 .88992 .9586 .88233 Metetra
.6 .764 .958 r
.9586 .88233 .96469 .88992 .98185 .88673 .97585 .87915 Metetra
.631 .751 .934 r
.37771 .81375 .38563 .82046 .3986 .81742 .39072 .8107 Metetra
.633 .752 .934 r
.39072 .8107 .3986 .81742 .41164 .8143 .4038 .80758 Metetra
.635 .753 .933 r
.4038 .80758 .41164 .8143 .42476 .8111 .41696 .80437 Metetra
.637 .755 .933 r
.41696 .80437 .42476 .8111 .43795 .80782 .43019 .80108 Metetra
.64 .756 .933 r
.43019 .80108 .43795 .80782 .45121 .80446 .44349 .79771 Metetra
.642 .757 .932 r
.44349 .79771 .45121 .80446 .46455 .80101 .45687 .79424 Metetra
.644 .758 .932 r
.45687 .79424 .46455 .80101 .47796 .79747 .47033 .79069 Metetra
.646 .76 .932 r
.47033 .79069 .47796 .79747 .49144 .79385 .48386 .78704 Metetra
.648 .761 .931 r
.48386 .78704 .49144 .79385 .50499 .79014 .49746 .78331 Metetra
.65 .761 .93 r
.49746 .78331 .50499 .79014 .51862 .78635 .51113 .77949 Metetra
.652 .762 .93 r
.51113 .77949 .51862 .78635 .53232 .78248 .52488 .77559 Metetra
.654 .763 .929 r
.52488 .77559 .53232 .78248 .54609 .77853 .5387 .7716 Metetra
.656 .763 .929 r
.5387 .7716 .54609 .77853 .55993 .77451 .55259 .76753 Metetra
.658 .764 .928 r
.55259 .76753 .55993 .77451 .57385 .77041 .56656 .76339 Metetra
.959 .664 .431 r
.56656 .76339 .57385 .77041 .59211 .99432 .58435 .98548 Metetra
.752 .756 .856 r
.58435 .98548 .59211 .99432 .60703 .98503 .59933 .9762 Metetra
.74 .755 .865 r
.59933 .9762 .60703 .98503 .62201 .97648 .61438 .96768 Metetra
.729 .754 .873 r
.61438 .96768 .62201 .97648 .63706 .96863 .62949 .95986 Metetra
.717 .753 .881 r
.62949 .95986 .63706 .96863 .65219 .9614 .64467 .95268 Metetra
.706 .752 .888 r
.64467 .95268 .65219 .9614 .66739 .95473 .65993 .94606 Metetra
.695 .751 .895 r
.65993 .94606 .66739 .95473 .68268 .94857 .67528 .93996 Metetra
.684 .751 .902 r
.67528 .93996 .68268 .94857 .69806 .94287 .69073 .93432 Metetra
.674 .75 .908 r
.69073 .93432 .69806 .94287 .71353 .93757 .70626 .92909 Metetra
.665 .75 .914 r
.70626 .92909 .71353 .93757 .7291 .93263 .7219 .92422 Metetra
.656 .75 .919 r
.7219 .92422 .7291 .93263 .74478 .928 .73764 .91966 Metetra
.648 .75 .924 r
.73764 .91966 .74478 .928 .76056 .92366 .75349 .91539 Metetra
.64 .75 .928 r
.75349 .91539 .76056 .92366 .77644 .91955 .76945 .91135 Metetra
.633 .75 .932 r
.76945 .91135 .77644 .91955 .79244 .91565 .78551 .90752 Metetra
.627 .751 .936 r
.78551 .90752 .79244 .91565 .80855 .91192 .80169 .90386 Metetra
.622 .752 .939 r
.80169 .90386 .80855 .91192 .82476 .90834 .81798 .90034 Metetra
.617 .753 .942 r
.81798 .90034 .82476 .90834 .84109 .90488 .83439 .89694 Metetra
.613 .754 .945 r
.83439 .89694 .84109 .90488 .85754 .90152 .85091 .89364 Metetra
.609 .755 .948 r
.85091 .89364 .85754 .90152 .8741 .89824 .86754 .8904 Metetra
.606 .756 .95 r
.86754 .8904 .8741 .89824 .89077 .895 .88429 .88721 Metetra
.603 .757 .952 r
.88429 .88721 .89077 .895 .90755 .89181 .90116 .88406 Metetra
.601 .759 .954 r
.90116 .88406 .90755 .89181 .92446 .88865 .91814 .88093 Metetra
.6 .76 .956 r
.91814 .88093 .92446 .88865 .94147 .88549 .93524 .8778 Metetra
.598 .762 .957 r
.93524 .8778 .94147 .88549 .9586 .88233 .95246 .87466 Metetra
.598 .764 .958 r
.95246 .87466 .9586 .88233 .97585 .87915 .96979 .87151 Metetra
.63 .75 .934 r
.36972 .80697 .37771 .81375 .39072 .8107 .38276 .80392 Metetra
.632 .752 .934 r
.38276 .80392 .39072 .8107 .4038 .80758 .39588 .80079 Metetra
.634 .753 .934 r
.39588 .80079 .4038 .80758 .41696 .80437 .40908 .79758 Metetra
.636 .755 .934 r
.40908 .79758 .41696 .80437 .43019 .80108 .42235 .79428 Metetra
.639 .756 .933 r
.42235 .79428 .43019 .80108 .44349 .79771 .4357 .7909 Metetra
.641 .757 .933 r
.4357 .7909 .44349 .79771 .45687 .79424 .44913 .78742 Metetra
.643 .758 .932 r
.44913 .78742 .45687 .79424 .47033 .79069 .46263 .78384 Metetra
.646 .76 .932 r
.46263 .78384 .47033 .79069 .48386 .78704 .4762 .78018 Metetra
.648 .761 .931 r
.4762 .78018 .48386 .78704 .49746 .78331 .48985 .77642 Metetra
.65 .762 .931 r
.48985 .77642 .49746 .78331 .51113 .77949 .50357 .77257 Metetra
.652 .762 .93 r
.50357 .77257 .51113 .77949 .52488 .77559 .51737 .76864 Metetra
.654 .763 .93 r
.51737 .76864 .52488 .77559 .5387 .7716 .53124 .76461 Metetra
.656 .764 .929 r
.53124 .76461 .5387 .7716 .55259 .76753 .54518 .76051 Metetra
.658 .764 .928 r
.54518 .76051 .55259 .76753 .56656 .76339 .5592 .75632 Metetra
.959 .664 .43 r
.5592 .75632 .56656 .76339 .58435 .98548 .57651 .97649 Metetra
.751 .755 .856 r
.57651 .97649 .58435 .98548 .59933 .9762 .59156 .96722 Metetra
.739 .753 .864 r
.59156 .96722 .59933 .9762 .61438 .96768 .60666 .95873 Metetra
.727 .752 .873 r
.60666 .95873 .61438 .96768 .62949 .95986 .62183 .95095 Metetra
.715 .751 .881 r
.62183 .95095 .62949 .95986 .64467 .95268 .63708 .94381 Metetra
.703 .75 .888 r
.63708 .94381 .64467 .95268 .65993 .94606 .6524 .93725 Metetra
.692 .749 .895 r
.6524 .93725 .65993 .94606 .67528 .93996 .66782 .93122 Metetra
.682 .749 .902 r
.66782 .93122 .67528 .93996 .69073 .93432 .68332 .92565 Metetra
.671 .748 .908 r
.68332 .92565 .69073 .93432 .70626 .92909 .69892 .92049 Metetra
.662 .748 .914 r
.69892 .92049 .70626 .92909 .7219 .92422 .71463 .91569 Metetra
.653 .748 .919 r
.71463 .91569 .7219 .92422 .73764 .91966 .73043 .91122 Metetra
.645 .748 .924 r
.73043 .91122 .73764 .91966 .75349 .91539 .74635 .90702 Metetra
.637 .748 .929 r
.74635 .90702 .75349 .91539 .76945 .91135 .76237 .90306 Metetra
.63 .749 .933 r
.76237 .90306 .76945 .91135 .78551 .90752 .77851 .8993 Metetra
.624 .749 .937 r
.77851 .8993 .78551 .90752 .80169 .90386 .79476 .89571 Metetra
.618 .75 .94 r
.79476 .89571 .80169 .90386 .81798 .90034 .81113 .89226 Metetra
.614 .751 .943 r
.81113 .89226 .81798 .90034 .83439 .89694 .82761 .88892 Metetra
.609 .752 .946 r
.82761 .88892 .83439 .89694 .85091 .89364 .8442 .88567 Metetra
.606 .754 .949 r
.8442 .88567 .85091 .89364 .86754 .8904 .86092 .88249 Metetra
.603 .755 .951 r
.86092 .88249 .86754 .8904 .88429 .88721 .87775 .87935 Metetra
.601 .757 .953 r
.87775 .87935 .88429 .88721 .90116 .88406 .89469 .87624 Metetra
.599 .758 .955 r
.89469 .87624 .90116 .88406 .91814 .88093 .91176 .87315 Metetra
.597 .76 .956 r
.91176 .87315 .91814 .88093 .93524 .8778 .92894 .87005 Metetra
.596 .762 .958 r
.92894 .87005 .93524 .8778 .95246 .87466 .94624 .86694 Metetra
.595 .763 .959 r
.94624 .86694 .95246 .87466 .96979 .87151 .96366 .8638 Metetra
.629 .75 .934 r
.36164 .80013 .36972 .80697 .38276 .80392 .37473 .79707 Metetra
.631 .751 .934 r
.37473 .79707 .38276 .80392 .39588 .80079 .38789 .79394 Metetra
.633 .753 .934 r
.38789 .79394 .39588 .80079 .40908 .79758 .40113 .79072 Metetra
.635 .754 .934 r
.40113 .79072 .40908 .79758 .42235 .79428 .41444 .78742 Metetra
.638 .756 .934 r
.41444 .78742 .42235 .79428 .4357 .7909 .42783 .78402 Metetra
.64 .757 .933 r
.42783 .78402 .4357 .7909 .44913 .78742 .4413 .78053 Metetra
.643 .758 .933 r
.4413 .78053 .44913 .78742 .46263 .78384 .45485 .77694 Metetra
.645 .76 .932 r
.45485 .77694 .46263 .78384 .4762 .78018 .46847 .77326 Metetra
.647 .761 .932 r
.46847 .77326 .4762 .78018 .48985 .77642 .48216 .76947 Metetra
.65 .762 .931 r
.48216 .76947 .48985 .77642 .50357 .77257 .49593 .7656 Metetra
.652 .763 .931 r
.49593 .7656 .50357 .77257 .51737 .76864 .50978 .76163 Metetra
.654 .763 .93 r
.50978 .76163 .51737 .76864 .53124 .76461 .5237 .75757 Metetra
.656 .764 .929 r
.5237 .75757 .53124 .76461 .54518 .76051 .53769 .75342 Metetra
.658 .765 .928 r
.53769 .75342 .54518 .76051 .5592 .75632 .55176 .74919 Metetra
.959 .664 .429 r
.55176 .74919 .5592 .75632 .57651 .97649 .5686 .96733 Metetra
.749 .753 .855 r
.5686 .96733 .57651 .97649 .59156 .96722 .58371 .95808 Metetra
.737 .751 .864 r
.58371 .95808 .59156 .96722 .60666 .95873 .59887 .94962 Metetra
.725 .75 .873 r
.59887 .94962 .60666 .95873 .62183 .95095 .61411 .94188 Metetra
.713 .749 .881 r
.61411 .94188 .62183 .95095 .63708 .94381 .62941 .9348 Metetra
.701 .748 .888 r
.62941 .9348 .63708 .94381 .6524 .93725 .6448 .9283 Metetra
.69 .747 .895 r
.6448 .9283 .6524 .93725 .66782 .93122 .66027 .92234 Metetra
.679 .747 .902 r
.66027 .92234 .66782 .93122 .68332 .92565 .67584 .91684 Metetra
.668 .746 .909 r
.67584 .91684 .68332 .92565 .69892 .92049 .69151 .91176 Metetra
.659 .746 .914 r
.69151 .91176 .69892 .92049 .71463 .91569 .70728 .90705 Metetra
.65 .746 .92 r
.70728 .90705 .71463 .91569 .73043 .91122 .72315 .90266 Metetra
.641 .746 .925 r
.72315 .90266 .73043 .91122 .74635 .90702 .73913 .89854 Metetra
.634 .746 .929 r
.73913 .89854 .74635 .90702 .76237 .90306 .75523 .89466 Metetra
.627 .747 .934 r
.75523 .89466 .76237 .90306 .77851 .8993 .77144 .89098 Metetra
.62 .748 .937 r
.77144 .89098 .77851 .8993 .79476 .89571 .78776 .88747 Metetra
.615 .749 .941 r
.78776 .88747 .79476 .89571 .81113 .89226 .8042 .88409 Metetra
.61 .75 .944 r
.8042 .88409 .81113 .89226 .82761 .88892 .82075 .88082 Metetra
.606 .751 .947 r
.82075 .88082 .82761 .88892 .8442 .88567 .83743 .87764 Metetra
.603 .753 .95 r
.83743 .87764 .8442 .88567 .86092 .88249 .85422 .87451 Metetra
.6 .754 .952 r
.85422 .87451 .86092 .88249 .87775 .87935 .87113 .87142 Metetra
.598 .756 .954 r
.87113 .87142 .87775 .87935 .89469 .87624 .88816 .86836 Metetra
.596 .758 .956 r
.88816 .86836 .89469 .87624 .91176 .87315 .90531 .8653 Metetra
.594 .759 .957 r
.90531 .8653 .91176 .87315 .92894 .87005 .92257 .86223 Metetra
.594 .761 .959 r
.92257 .86223 .92894 .87005 .94624 .86694 .93996 .85915 Metetra
.593 .763 .96 r
.93996 .85915 .94624 .86694 .96366 .8638 .95746 .85603 Metetra
.628 .749 .935 r
.35349 .79321 .36164 .80013 .37473 .79707 .36662 .79015 Metetra
.63 .751 .935 r
.36662 .79015 .37473 .79707 .38789 .79394 .37982 .78702 Metetra
.632 .753 .935 r
.37982 .78702 .38789 .79394 .40113 .79072 .3931 .7838 Metetra
.634 .754 .934 r
.3931 .7838 .40113 .79072 .41444 .78742 .40645 .78049 Metetra
.637 .756 .934 r
.40645 .78049 .41444 .78742 .42783 .78402 .41989 .77709 Metetra
.639 .757 .934 r
.41989 .77709 .42783 .78402 .4413 .78053 .4334 .77359 Metetra
.642 .759 .933 r
.4334 .77359 .4413 .78053 .45485 .77694 .44699 .76998 Metetra
.644 .76 .933 r
.44699 .76998 .45485 .77694 .46847 .77326 .46066 .76628 Metetra
.647 .761 .932 r
.46066 .76628 .46847 .77326 .48216 .76947 .4744 .76247 Metetra
.649 .762 .932 r
.4744 .76247 .48216 .76947 .49593 .7656 .48822 .75857 Metetra
.652 .763 .931 r
.48822 .75857 .49593 .7656 .50978 .76163 .50211 .75456 Metetra
.654 .764 .93 r
.50211 .75456 .50978 .76163 .5237 .75757 .51608 .75047 Metetra
.656 .765 .929 r
.51608 .75047 .5237 .75757 .53769 .75342 .53013 .74628 Metetra
.658 .765 .929 r
.53013 .74628 .53769 .75342 .55176 .74919 .54425 .742 Metetra
.959 .664 .429 r
.54425 .742 .55176 .74919 .5686 .96733 .56062 .958 Metetra
.747 .751 .855 r
.56062 .958 .5686 .96733 .58371 .95808 .57579 .94877 Metetra
.735 .749 .864 r
.57579 .94877 .58371 .95808 .59887 .94962 .59101 .94035 Metetra
.722 .748 .872 r
.59101 .94035 .59887 .94962 .61411 .94188 .6063 .93266 Metetra
.71 .747 .881 r
.6063 .93266 .61411 .94188 .62941 .9348 .62167 .92563 Metetra
.698 .746 .888 r
.62167 .92563 .62941 .9348 .6448 .9283 .63712 .91921 Metetra
.687 .745 .896 r
.63712 .91921 .6448 .9283 .66027 .92234 .65265 .91332 Metetra
.676 .744 .902 r
.65265 .91332 .66027 .92234 .67584 .91684 .66829 .9079 Metetra
.665 .744 .909 r
.66829 .9079 .67584 .91684 .69151 .91176 .68402 .90291 Metetra
.655 .744 .915 r
.68402 .90291 .69151 .91176 .70728 .90705 .69985 .89828 Metetra
.646 .744 .92 r
.69985 .89828 .70728 .90705 .72315 .90266 .71579 .89398 Metetra
.638 .744 .925 r
.71579 .89398 .72315 .90266 .73913 .89854 .73184 .88995 Metetra
.63 .744 .93 r
.73184 .88995 .73913 .89854 .75523 .89466 .74801 .88616 Metetra
.623 .745 .934 r
.74801 .88616 .75523 .89466 .77144 .89098 .76429 .88257 Metetra
.617 .746 .938 r
.76429 .88257 .77144 .89098 .78776 .88747 .78068 .87914 Metetra
.611 .747 .942 r
.78068 .87914 .78776 .88747 .8042 .88409 .7972 .87584 Metetra
.607 .748 .945 r
.7972 .87584 .8042 .88409 .82075 .88082 .81383 .87264 Metetra
.603 .75 .948 r
.81383 .87264 .82075 .88082 .83743 .87764 .83058 .86952 Metetra
.599 .752 .95 r
.83058 .86952 .83743 .87764 .85422 .87451 .84745 .86645 Metetra
.597 .753 .953 r
.84745 .86645 .85422 .87451 .87113 .87142 .86444 .86342 Metetra
.594 .755 .955 r
.86444 .86342 .87113 .87142 .88816 .86836 .88155 .86041 Metetra
.593 .757 .957 r
.88155 .86041 .88816 .86836 .90531 .8653 .89878 .85739 Metetra
.592 .759 .958 r
.89878 .85739 .90531 .8653 .92257 .86223 .91614 .85435 Metetra
.591 .761 .96 r
.91614 .85435 .92257 .86223 .93996 .85915 .93361 .85129 Metetra
.591 .763 .961 r
.93361 .85129 .93996 .85915 .95746 .85603 .9512 .8482 Metetra
.627 .749 .935 r
.34526 .78622 .35349 .79321 .36662 .79015 .35842 .78317 Metetra
.629 .751 .935 r
.35842 .78317 .36662 .79015 .37982 .78702 .37166 .78003 Metetra
.631 .752 .935 r
.37166 .78003 .37982 .78702 .3931 .7838 .38498 .77682 Metetra
.633 .754 .935 r
.38498 .77682 .3931 .7838 .40645 .78049 .39838 .7735 Metetra
.636 .755 .935 r
.39838 .7735 .40645 .78049 .41989 .77709 .41186 .77009 Metetra
.638 .757 .934 r
.41186 .77009 .41989 .77709 .4334 .77359 .42542 .76658 Metetra
.641 .759 .934 r
.42542 .76658 .4334 .77359 .44699 .76998 .43905 .76296 Metetra
.644 .76 .933 r
.43905 .76296 .44699 .76998 .46066 .76628 .45277 .75924 Metetra
.646 .761 .933 r
.45277 .75924 .46066 .76628 .4744 .76247 .46656 .75541 Metetra
.649 .762 .932 r
.46656 .75541 .4744 .76247 .48822 .75857 .48042 .75148 Metetra
.651 .763 .931 r
.48042 .75148 .48822 .75857 .50211 .75456 .49437 .74744 Metetra
.654 .764 .931 r
.49437 .74744 .50211 .75456 .51608 .75047 .50839 .74331 Metetra
.656 .765 .93 r
.50839 .74331 .51608 .75047 .53013 .74628 .52249 .73908 Metetra
.658 .766 .929 r
.52249 .73908 .53013 .74628 .54425 .742 .53666 .73476 Metetra
.959 .663 .428 r
.53666 .73476 .54425 .742 .56062 .958 .55256 .9485 Metetra
.746 .749 .854 r
.55256 .9485 .56062 .958 .57579 .94877 .56779 .93929 Metetra
.733 .747 .863 r
.56779 .93929 .57579 .94877 .59101 .94035 .58307 .93091 Metetra
.72 .746 .872 r
.58307 .93091 .59101 .94035 .6063 .93266 .59842 .92327 Metetra
.708 .745 .88 r
.59842 .92327 .6063 .93266 .62167 .92563 .61385 .91631 Metetra
.695 .744 .888 r
.61385 .91631 .62167 .92563 .63712 .91921 .62936 .90996 Metetra
.684 .743 .896 r
.62936 .90996 .63712 .91921 .65265 .91332 .64496 .90415 Metetra
.672 .742 .903 r
.64496 .90415 .65265 .91332 .66829 .9079 .66065 .89883 Metetra
.662 .742 .909 r
.66065 .89883 .66829 .9079 .68402 .90291 .67645 .89393 Metetra
.652 .741 .915 r
.67645 .89393 .68402 .90291 .69985 .89828 .69235 .8894 Metetra
.642 .741 .921 r
.69235 .8894 .69985 .89828 .71579 .89398 .70835 .88519 Metetra
.634 .742 .926 r
.70835 .88519 .71579 .89398 .73184 .88995 .72447 .88126 Metetra
.626 .742 .931 r
.72447 .88126 .73184 .88995 .74801 .88616 .74071 .87756 Metetra
.619 .743 .935 r
.74071 .87756 .74801 .88616 .76429 .88257 .75706 .87406 Metetra
.613 .744 .939 r
.75706 .87406 .76429 .88257 .78068 .87914 .77353 .87072 Metetra
.607 .746 .943 r
.77353 .87072 .78068 .87914 .7972 .87584 .79012 .8675 Metetra
.603 .747 .946 r
.79012 .8675 .7972 .87584 .81383 .87264 .80683 .86438 Metetra
.599 .749 .949 r
.80683 .86438 .81383 .87264 .83058 .86952 .82366 .86133 Metetra
.596 .75 .951 r
.82366 .86133 .83058 .86952 .84745 .86645 .84061 .85833 Metetra
.593 .752 .954 r
.84061 .85833 .84745 .86645 .86444 .86342 .85768 .85535 Metetra
.591 .754 .956 r
.85768 .85535 .86444 .86342 .88155 .86041 .87488 .85239 Metetra
.59 .756 .958 r
.87488 .85239 .88155 .86041 .89878 .85739 .89219 .84941 Metetra
.589 .758 .959 r
.89219 .84941 .89878 .85739 .91614 .85435 .90963 .84641 Metetra
.588 .76 .961 r
.90963 .84641 .91614 .85435 .93361 .85129 .92718 .84338 Metetra
.588 .762 .962 r
.92718 .84338 .93361 .85129 .9512 .8482 .94486 .8403 Metetra
.625 .748 .935 r
.33695 .77915 .34526 .78622 .35842 .78317 .35015 .7761 Metetra
.627 .75 .935 r
.35015 .7761 .35842 .78317 .37166 .78003 .36343 .77298 Metetra
.63 .752 .935 r
.36343 .77298 .37166 .78003 .38498 .77682 .37679 .76976 Metetra
.632 .754 .935 r
.37679 .76976 .38498 .77682 .39838 .7735 .39023 .76645 Metetra
.635 .755 .935 r
.39023 .76645 .39838 .7735 .41186 .77009 .40375 .76303 Metetra
.637 .757 .935 r
.40375 .76303 .41186 .77009 .42542 .76658 .41736 .75951 Metetra
.64 .759 .934 r
.41736 .75951 .42542 .76658 .43905 .76296 .43104 .75588 Metetra
.643 .76 .934 r
.43104 .75588 .43905 .76296 .45277 .75924 .4448 .75214 Metetra
.646 .761 .933 r
.4448 .75214 .45277 .75924 .46656 .75541 .45863 .74829 Metetra
.649 .763 .932 r
.45863 .74829 .46656 .75541 .48042 .75148 .47255 .74433 Metetra
.651 .764 .932 r
.47255 .74433 .48042 .75148 .49437 .74744 .48654 .74026 Metetra
.654 .765 .931 r
.48654 .74026 .49437 .74744 .50839 .74331 .50062 .73609 Metetra
.656 .765 .93 r
.50062 .73609 .50839 .74331 .52249 .73908 .51476 .73182 Metetra
.658 .766 .929 r
.51476 .73182 .52249 .73908 .53666 .73476 .52899 .72746 Metetra
.959 .663 .427 r
.52899 .72746 .53666 .73476 .55256 .9485 .54442 .93881 Metetra
.744 .746 .854 r
.54442 .93881 .55256 .9485 .56779 .93929 .55971 .92963 Metetra
.731 .745 .863 r
.55971 .92963 .56779 .93929 .58307 .93091 .57506 .92129 Metetra
.718 .743 .872 r
.57506 .92129 .58307 .93091 .59842 .92327 .59047 .91371 Metetra
.705 .742 .88 r
.59047 .91371 .59842 .92327 .61385 .91631 .60596 .90683 Metetra
.693 .741 .888 r
.60596 .90683 .61385 .91631 .62936 .90996 .62153 .90056 Metetra
.681 .74 .896 r
.62153 .90056 .62936 .90996 .64496 .90415 .63719 .89484 Metetra
.669 .739 .903 r
.63719 .89484 .64496 .90415 .66065 .89883 .65294 .88961 Metetra
.658 .739 .91 r
.65294 .88961 .66065 .89883 .67645 .89393 .6688 .88481 Metetra
.648 .739 .916 r
.6688 .88481 .67645 .89393 .69235 .8894 .68476 .88038 Metetra
.638 .739 .921 r
.68476 .88038 .69235 .8894 .70835 .88519 .70084 .87628 Metetra
.63 .74 .927 r
.70084 .87628 .70835 .88519 .72447 .88126 .71703 .87245 Metetra
.622 .74 .931 r
.71703 .87245 .72447 .88126 .74071 .87756 .73333 .86886 Metetra
.615 .741 .936 r
.73333 .86886 .74071 .87756 .75706 .87406 .74976 .86545 Metetra
.609 .742 .94 r
.74976 .86545 .75706 .87406 .77353 .87072 .7663 .86221 Metetra
.603 .744 .944 r
.7663 .86221 .77353 .87072 .79012 .8675 .78297 .85908 Metetra
.599 .746 .947 r
.78297 .85908 .79012 .8675 .80683 .86438 .79975 .85604 Metetra
.595 .747 .95 r
.79975 .85604 .80683 .86438 .82366 .86133 .81666 .85307 Metetra
.592 .749 .952 r
.81666 .85307 .82366 .86133 .84061 .85833 .83369 .85013 Metetra
.59 .751 .955 r
.83369 .85013 .84061 .85833 .85768 .85535 .85085 .84722 Metetra
.588 .753 .957 r
.85085 .84722 .85768 .85535 .87488 .85239 .86812 .84431 Metetra
.587 .756 .959 r
.86812 .84431 .87488 .85239 .89219 .84941 .88552 .84137 Metetra
.586 .758 .96 r
.88552 .84137 .89219 .84941 .90963 .84641 .90305 .83841 Metetra
.586 .76 .962 r
.90305 .83841 .90963 .84641 .92718 .84338 .92069 .83541 Metetra
.586 .762 .963 r
.92069 .83541 .92718 .84338 .94486 .8403 .93846 .83235 Metetra
.624 .748 .936 r
.32856 .77201 .33695 .77915 .35015 .7761 .3418 .76897 Metetra
.626 .75 .936 r
.3418 .76897 .35015 .7761 .36343 .77298 .35512 .76585 Metetra
.628 .751 .936 r
.35512 .76585 .36343 .77298 .37679 .76976 .36852 .76263 Metetra
.631 .753 .936 r
.36852 .76263 .37679 .76976 .39023 .76645 .382 .75932 Metetra
.633 .755 .935 r
.382 .75932 .39023 .76645 .40375 .76303 .39557 .75591 Metetra
.636 .757 .935 r
.39557 .75591 .40375 .76303 .41736 .75951 .40921 .75238 Metetra
.639 .759 .935 r
.40921 .75238 .41736 .75951 .43104 .75588 .42294 .74874 Metetra
.642 .76 .934 r
.42294 .74874 .43104 .75588 .4448 .75214 .43674 .74498 Metetra
.645 .762 .934 r
.43674 .74498 .4448 .75214 .45863 .74829 .45063 .74111 Metetra
.648 .763 .933 r
.45063 .74111 .45863 .74829 .47255 .74433 .4646 .73713 Metetra
.651 .764 .932 r
.4646 .73713 .47255 .74433 .48654 .74026 .47864 .73303 Metetra
.653 .765 .931 r
.47864 .73303 .48654 .74026 .50062 .73609 .49276 .72882 Metetra
.656 .766 .93 r
.49276 .72882 .50062 .73609 .51476 .73182 .50696 .72451 Metetra
.658 .767 .93 r
.50696 .72451 .51476 .73182 .52899 .72746 .52124 .7201 Metetra
.959 .662 .427 r
.52124 .7201 .52899 .72746 .54442 .93881 .53621 .92894 Metetra
.742 .744 .853 r
.53621 .92894 .54442 .93881 .55971 .92963 .55156 .91978 Metetra
.729 .742 .863 r
.55156 .91978 .55971 .92963 .57506 .92129 .56697 .91149 Metetra
.715 .741 .872 r
.56697 .91149 .57506 .92129 .59047 .91371 .58244 .90398 Metetra
.702 .739 .88 r
.58244 .90398 .59047 .91371 .60596 .90683 .59798 .89717 Metetra
.689 .738 .888 r
.59798 .89717 .60596 .90683 .62153 .90056 .61361 .89099 Metetra
.677 .737 .896 r
.61361 .89099 .62153 .90056 .63719 .89484 .62933 .88537 Metetra
.665 .737 .903 r
.62933 .88537 .63719 .89484 .65294 .88961 .64515 .88025 Metetra
.654 .736 .91 r
.64515 .88025 .65294 .88961 .6688 .88481 .66107 .87556 Metetra
.644 .736 .916 r
.66107 .87556 .6688 .88481 .68476 .88038 .6771 .87124 Metetra
.634 .737 .922 r
.6771 .87124 .68476 .88038 .70084 .87628 .69325 .86725 Metetra
.625 .737 .927 r
.69325 .86725 .70084 .87628 .71703 .87245 .7095 .86354 Metetra
.618 .738 .932 r
.7095 .86354 .71703 .87245 .73333 .86886 .72588 .86005 Metetra
.61 .739 .937 r
.72588 .86005 .73333 .86886 .74976 .86545 .74238 .85675 Metetra
.604 .741 .941 r
.74238 .85675 .74976 .86545 .7663 .86221 .75899 .85361 Metetra
.599 .742 .945 r
.75899 .85361 .7663 .86221 .78297 .85908 .77573 .85057 Metetra
.595 .744 .948 r
.77573 .85057 .78297 .85908 .79975 .85604 .7926 .84762 Metetra
.591 .746 .951 r
.7926 .84762 .79975 .85604 .81666 .85307 .80959 .84473 Metetra
.588 .748 .953 r
.80959 .84473 .81666 .85307 .83369 .85013 .8267 .84187 Metetra
.586 .75 .956 r
.8267 .84187 .83369 .85013 .85085 .84722 .84394 .83902 Metetra
.584 .753 .958 r
.84394 .83902 .85085 .84722 .86812 .84431 .8613 .83616 Metetra
.583 .755 .96 r
.8613 .83616 .86812 .84431 .88552 .84137 .87878 .83328 Metetra
.583 .757 .961 r
.87878 .83328 .88552 .84137 .90305 .83841 .89639 .83036 Metetra
.583 .76 .963 r
.89639 .83036 .90305 .83841 .92069 .83541 .91413 .82738 Metetra
.583 .762 .964 r
.91413 .82738 .92069 .83541 .93846 .83235 .93199 .82435 Metetra
.622 .747 .936 r
.32009 .76479 .32856 .77201 .3418 .76897 .33336 .76176 Metetra
.624 .749 .936 r
.33336 .76176 .3418 .76897 .35512 .76585 .34672 .75864 Metetra
.627 .751 .936 r
.34672 .75864 .35512 .76585 .36852 .76263 .36016 .75544 Metetra
.629 .753 .936 r
.36016 .75544 .36852 .76263 .382 .75932 .37369 .75213 Metetra
.632 .755 .936 r
.37369 .75213 .382 .75932 .39557 .75591 .38729 .74871 Metetra
.635 .757 .936 r
.38729 .74871 .39557 .75591 .40921 .75238 .40098 .74518 Metetra
.638 .758 .935 r
.40098 .74518 .40921 .75238 .42294 .74874 .41476 .74154 Metetra
.641 .76 .935 r
.41476 .74154 .42294 .74874 .43674 .74498 .42861 .73777 Metetra
.644 .762 .934 r
.42861 .73777 .43674 .74498 .45063 .74111 .44254 .73388 Metetra
.647 .763 .933 r
.44254 .73388 .45063 .74111 .4646 .73713 .45656 .72987 Metetra
.65 .764 .933 r
.45656 .72987 .4646 .73713 .47864 .73303 .47065 .72574 Metetra
.653 .766 .932 r
.47065 .72574 .47864 .73303 .49276 .72882 .48483 .7215 Metetra
.656 .766 .931 r
.48483 .7215 .49276 .72882 .50696 .72451 .49908 .71714 Metetra
.658 .767 .93 r
.49908 .71714 .50696 .72451 .52124 .7201 .51341 .71268 Metetra
.959 .662 .426 r
.51341 .71268 .52124 .7201 .53621 .92894 .52792 .91885 Metetra
.74 .741 .853 r
.52792 .91885 .53621 .92894 .55156 .91978 .54333 .90973 Metetra
.726 .74 .862 r
.54333 .90973 .55156 .91978 .56697 .91149 .5588 .9015 Metetra
.713 .738 .871 r
.5588 .9015 .56697 .91149 .58244 .90398 .57433 .89406 Metetra
.699 .737 .88 r
.57433 .89406 .58244 .90398 .59798 .89717 .58993 .88733 Metetra
.686 .735 .888 r
.58993 .88733 .59798 .89717 .61361 .89099 .60562 .88125 Metetra
.674 .735 .896 r
.60562 .88125 .61361 .89099 .62933 .88537 .6214 .87574 Metetra
.661 .734 .903 r
.6214 .87574 .62933 .88537 .64515 .88025 .63728 .87073 Metetra
.65 .734 .91 r
.63728 .87073 .64515 .88025 .66107 .87556 .65327 .86616 Metetra
.639 .734 .917 r
.65327 .86616 .66107 .87556 .6771 .87124 .66936 .86197 Metetra
.63 .734 .923 r
.66936 .86197 .6771 .87124 .69325 .86725 .68557 .8581 Metetra
.621 .735 .928 r
.68557 .8581 .69325 .86725 .7095 .86354 .7019 .85451 Metetra
.613 .736 .933 r
.7019 .85451 .7095 .86354 .72588 .86005 .71835 .85114 Metetra
.606 .737 .938 r
.71835 .85114 .72588 .86005 .74238 .85675 .73491 .84796 Metetra
.6 .739 .942 r
.73491 .84796 .74238 .85675 .75899 .85361 .75161 .84492 Metetra
.594 .74 .946 r
.75161 .84492 .75899 .85361 .77573 .85057 .76842 .84199 Metetra
.59 .742 .949 r
.76842 .84199 .77573 .85057 .7926 .84762 .78537 .83913 Metetra
.586 .745 .952 r
.78537 .83913 .7926 .84762 .80959 .84473 .80244 .83633 Metetra
.584 .747 .955 r
.80244 .83633 .80959 .84473 .8267 .84187 .81963 .83354 Metetra
.582 .749 .957 r
.81963 .83354 .8267 .84187 .84394 .83902 .83695 .83076 Metetra
.58 .752 .959 r
.83695 .83076 .84394 .83902 .8613 .83616 .8544 .82796 Metetra
.58 .754 .961 r
.8544 .82796 .8613 .83616 .87878 .83328 .87197 .82513 Metetra
.579 .757 .962 r
.87197 .82513 .87878 .83328 .89639 .83036 .88967 .82224 Metetra
.58 .759 .964 r
.88967 .82224 .89639 .83036 .91413 .82738 .90749 .8193 Metetra
.58 .762 .965 r
.90749 .8193 .91413 .82738 .93199 .82435 .92544 .81628 Metetra
.621 .746 .936 r
.31153 .75748 .32009 .76479 .33336 .76176 .32484 .75446 Metetra
.623 .748 .937 r
.32484 .75446 .33336 .76176 .34672 .75864 .33824 .75136 Metetra
.625 .75 .937 r
.33824 .75136 .34672 .75864 .36016 .75544 .35172 .74817 Metetra
.628 .752 .937 r
.35172 .74817 .36016 .75544 .37369 .75213 .36529 .74487 Metetra
.631 .754 .937 r
.36529 .74487 .37369 .75213 .38729 .74871 .37894 .74146 Metetra
.634 .756 .936 r
.37894 .74146 .38729 .74871 .40098 .74518 .39267 .73793 Metetra
.637 .758 .936 r
.39267 .73793 .40098 .74518 .41476 .74154 .40649 .73427 Metetra
.64 .76 .935 r
.40649 .73427 .41476 .74154 .42861 .73777 .42039 .73049 Metetra
.644 .762 .935 r
.42039 .73049 .42861 .73777 .44254 .73388 .43437 .72659 Metetra
.647 .764 .934 r
.43437 .72659 .44254 .73388 .45656 .72987 .44844 .72256 Metetra
.65 .765 .933 r
.44844 .72256 .45656 .72987 .47065 .72574 .46258 .7184 Metetra
.653 .766 .932 r
.46258 .7184 .47065 .72574 .48483 .7215 .47681 .71412 Metetra
.656 .767 .931 r
.47681 .71412 .48483 .7215 .49908 .71714 .49112 .70972 Metetra
.658 .768 .93 r
.49112 .70972 .49908 .71714 .51341 .71268 .5055 .70521 Metetra
.959 .661 .425 r
.5055 .70521 .51341 .71268 .52792 .91885 .51955 .90856 Metetra
.738 .739 .852 r
.51955 .90856 .52792 .91885 .54333 .90973 .53502 .89948 Metetra
.724 .737 .862 r
.53502 .89948 .54333 .90973 .5588 .9015 .55055 .8913 Metetra
.71 .735 .871 r
.55055 .8913 .5588 .9015 .57433 .89406 .56614 .88394 Metetra
.696 .734 .88 r
.56614 .88394 .57433 .89406 .58993 .88733 .5818 .87731 Metetra
.683 .732 .888 r
.5818 .87731 .58993 .88733 .60562 .88125 .59755 .87134 Metetra
.67 .732 .896 r
.59755 .87134 .60562 .88125 .6214 .87574 .61339 .86594 Metetra
.657 .731 .904 r
.61339 .86594 .6214 .87574 .63728 .87073 .62933 .86106 Metetra
.646 .731 .911 r
.62933 .86106 .63728 .87073 .65327 .86616 .64538 .85662 Metetra
.635 .731 .917 r
.64538 .85662 .65327 .86616 .66936 .86197 .66154 .85256 Metetra
.625 .731 .923 r
.66154 .85256 .66936 .86197 .68557 .8581 .67782 .84883 Metetra
.616 .732 .929 r
.67782 .84883 .68557 .8581 .7019 .85451 .69421 .84537 Metetra
.608 .733 .934 r
.69421 .84537 .7019 .85451 .71835 .85114 .71073 .84213 Metetra
.601 .735 .939 r
.71073 .84213 .71835 .85114 .73491 .84796 .72737 .83907 Metetra
.595 .736 .943 r
.72737 .83907 .73491 .84796 .75161 .84492 .74414 .83615 Metetra
.589 .738 .947 r
.74414 .83615 .75161 .84492 .76842 .84199 .76103 .83332 Metetra
.585 .741 .95 r
.76103 .83332 .76842 .84199 .78537 .83913 .77806 .83057 Metetra
.582 .743 .953 r
.77806 .83057 .78537 .83913 .80244 .83633 .79521 .82785 Metetra
.579 .746 .956 r
.79521 .82785 .80244 .83633 .81963 .83354 .81248 .82515 Metetra
.578 .748 .958 r
.81248 .82515 .81963 .83354 .83695 .83076 .82989 .82244 Metetra
.576 .751 .96 r
.82989 .82244 .83695 .83076 .8544 .82796 .84742 .8197 Metetra
.576 .754 .962 r
.84742 .8197 .8544 .82796 .87197 .82513 .86508 .81692 Metetra
.576 .756 .963 r
.86508 .81692 .87197 .82513 .88967 .82224 .88287 .81408 Metetra
.577 .759 .965 r
.88287 .81408 .88967 .82224 .90749 .8193 .90078 .81117 Metetra
.578 .762 .966 r
.90078 .81117 .90749 .8193 .92544 .81628 .91882 .80817 Metetra
.619 .746 .937 r
.30289 .75009 .31153 .75748 .32484 .75446 .31624 .74709 Metetra
.621 .747 .937 r
.31624 .74709 .32484 .75446 .33824 .75136 .32968 .744 Metetra
.623 .75 .937 r
.32968 .744 .33824 .75136 .35172 .74817 .3432 .74082 Metetra
.626 .752 .937 r
.3432 .74082 .35172 .74817 .36529 .74487 .3568 .73753 Metetra
.629 .754 .937 r
.3568 .73753 .36529 .74487 .37894 .74146 .3705 .73413 Metetra
.632 .756 .937 r
.3705 .73413 .37894 .74146 .39267 .73793 .38427 .7306 Metetra
.636 .758 .937 r
.38427 .7306 .39267 .73793 .40649 .73427 .39814 .72695 Metetra
.639 .76 .936 r
.39814 .72695 .40649 .73427 .42039 .73049 .41208 .72316 Metetra
.643 .762 .935 r
.41208 .72316 .42039 .73049 .43437 .72659 .42612 .71924 Metetra
.646 .764 .935 r
.42612 .71924 .43437 .72659 .44844 .72256 .44023 .71519 Metetra
.649 .765 .934 r
.44023 .71519 .44844 .72256 .46258 .7184 .45443 .711 Metetra
.653 .767 .933 r
.45443 .711 .46258 .7184 .47681 .71412 .46871 .70669 Metetra
.656 .768 .932 r
.46871 .70669 .47681 .71412 .49112 .70972 .48307 .70225 Metetra
.658 .769 .931 r
.48307 .70225 .49112 .70972 .5055 .70521 .4975 .69769 Metetra
.959 .661 .424 r
.4975 .69769 .5055 .70521 .51955 .90856 .51111 .89804 Metetra
.736 .736 .851 r
.51111 .89804 .51955 .90856 .53502 .89948 .52664 .889 Metetra
.721 .734 .861 r
.52664 .889 .53502 .89948 .55055 .8913 .54222 .88089 Metetra
.707 .732 .87 r
.54222 .88089 .55055 .8913 .56614 .88394 .55787 .87362 Metetra
.693 .73 .879 r
.55787 .87362 .56614 .88394 .5818 .87731 .57359 .86709 Metetra
.679 .729 .888 r
.57359 .86709 .5818 .87731 .59755 .87134 .5894 .86124 Metetra
.665 .728 .896 r
.5894 .86124 .59755 .87134 .61339 .86594 .6053 .85597 Metetra
.653 .728 .904 r
.6053 .85597 .61339 .86594 .62933 .86106 .6213 .85123 Metetra
.641 .727 .911 r
.6213 .85123 .62933 .86106 .64538 .85662 .63741 .84693 Metetra
.63 .728 .918 r
.63741 .84693 .64538 .85662 .66154 .85256 .65364 .84302 Metetra
.62 .728 .924 r
.65364 .84302 .66154 .85256 .67782 .84883 .66998 .83943 Metetra
.61 .729 .93 r
.66998 .83943 .67782 .84883 .69421 .84537 .68645 .83611 Metetra
.602 .731 .935 r
.68645 .83611 .69421 .84537 .71073 .84213 .70303 .83301 Metetra
.595 .732 .94 r
.70303 .83301 .71073 .84213 .72737 .83907 .71975 .83008 Metetra
.589 .734 .944 r
.71975 .83008 .72737 .83907 .74414 .83615 .73659 .82729 Metetra
.584 .736 .948 r
.73659 .82729 .74414 .83615 .76103 .83332 .75356 .82458 Metetra
.58 .739 .951 r
.75356 .82458 .76103 .83332 .77806 .83057 .77066 .82194 Metetra
.577 .742 .954 r
.77066 .82194 .77806 .83057 .79521 .82785 .7879 .81932 Metetra
.575 .744 .957 r
.7879 .81932 .79521 .82785 .81248 .82515 .80526 .8167 Metetra
.573 .747 .959 r
.80526 .8167 .81248 .82515 .82989 .82244 .82275 .81407 Metetra
.572 .75 .961 r
.82275 .81407 .82989 .82244 .84742 .8197 .84037 .8114 Metetra
.572 .753 .963 r
.84037 .8114 .84742 .8197 .86508 .81692 .85811 .80867 Metetra
.573 .756 .965 r
.85811 .80867 .86508 .81692 .88287 .81408 .87599 .80586 Metetra
.573 .759 .966 r
.87599 .80586 .88287 .81408 .90078 .81117 .89399 .80298 Metetra
.575 .762 .967 r
.89399 .80298 .90078 .81117 .91882 .80817 .91212 .80001 Metetra
.617 .745 .937 r
.29416 .74261 .30289 .75009 .31624 .74709 .30755 .73963 Metetra
.619 .747 .938 r
.30755 .73963 .31624 .74709 .32968 .744 .32103 .73656 Metetra
.621 .749 .938 r
.32103 .73656 .32968 .744 .3432 .74082 .33459 .7334 Metetra
.624 .751 .938 r
.33459 .7334 .3432 .74082 .3568 .73753 .34823 .73013 Metetra
.627 .754 .938 r
.34823 .73013 .3568 .73753 .3705 .73413 .36197 .72674 Metetra
.631 .756 .938 r
.36197 .72674 .3705 .73413 .38427 .7306 .37579 .72322 Metetra
.634 .758 .937 r
.37579 .72322 .38427 .7306 .39814 .72695 .3897 .71956 Metetra
.638 .76 .937 r
.3897 .71956 .39814 .72695 .41208 .72316 .40369 .71577 Metetra
.642 .762 .936 r
.40369 .71577 .41208 .72316 .42612 .71924 .41777 .71184 Metetra
.645 .764 .935 r
.41777 .71184 .42612 .71924 .44023 .71519 .43194 .70777 Metetra
.649 .766 .934 r
.43194 .70777 .44023 .71519 .45443 .711 .44618 .70355 Metetra
.652 .767 .933 r
.44618 .70355 .45443 .711 .46871 .70669 .46052 .6992 Metetra
.656 .768 .932 r
.46052 .6992 .46871 .70669 .48307 .70225 .47493 .69472 Metetra
.659 .769 .931 r
.47493 .69472 .48307 .70225 .4975 .69769 .48942 .69011 Metetra
.959 .66 .423 r
.48942 .69011 .4975 .69769 .51111 .89804 .50259 .88729 Metetra
.733 .733 .85 r
.50259 .88729 .51111 .89804 .52664 .889 .51818 .8783 Metetra
.718 .73 .86 r
.51818 .8783 .52664 .889 .54222 .88089 .53382 .87026 Metetra
.703 .728 .87 r
.53382 .87026 .54222 .88089 .55787 .87362 .54952 .86308 Metetra
.689 .727 .879 r
.54952 .86308 .55787 .87362 .57359 .86709 .5653 .85667 Metetra
.675 .726 .888 r
.5653 .85667 .57359 .86709 .5894 .86124 .58116 .85095 Metetra
.661 .725 .896 r
.58116 .85095 .5894 .86124 .6053 .85597 .59712 .84582 Metetra
.648 .724 .904 r
.59712 .84582 .6053 .85597 .6213 .85123 .61319 .84123 Metetra
.636 .724 .912 r
.61319 .84123 .6213 .85123 .63741 .84693 .62936 .83709 Metetra
.624 .724 .919 r
.62936 .83709 .63741 .84693 .65364 .84302 .64565 .83333 Metetra
.614 .725 .925 r
.64565 .83333 .65364 .84302 .66998 .83943 .66206 .8299 Metetra
.605 .726 .931 r
.66206 .8299 .66998 .83943 .68645 .83611 .67859 .82674 Metetra
.596 .728 .936 r
.67859 .82674 .68645 .83611 .70303 .83301 .69525 .82379 Metetra
.589 .73 .941 r
.69525 .82379 .70303 .83301 .71975 .83008 .71204 .82101 Metetra
.583 .732 .945 r
.71204 .82101 .71975 .83008 .73659 .82729 .72896 .81835 Metetra
.578 .734 .949 r
.72896 .81835 .73659 .82729 .75356 .82458 .74601 .81577 Metetra
.574 .737 .953 r
.74601 .81577 .75356 .82458 .77066 .82194 .76319 .81324 Metetra
.571 .74 .956 r
.76319 .81324 .77066 .82194 .7879 .81932 .78051 .81072 Metetra
.569 .743 .958 r
.78051 .81072 .7879 .81932 .80526 .8167 .79795 .8082 Metetra
.568 .746 .961 r
.79795 .8082 .80526 .8167 .82275 .81407 .81553 .80565 Metetra
.568 .75 .963 r
.81553 .80565 .82275 .81407 .84037 .8114 .83323 .80304 Metetra
.568 .753 .965 r
.83323 .80304 .84037 .8114 .85811 .80867 .85107 .80036 Metetra
.569 .756 .966 r
.85107 .80036 .85811 .80867 .87599 .80586 .86904 .79761 Metetra
.57 .759 .967 r
.86904 .79761 .87599 .80586 .89399 .80298 .88713 .79476 Metetra
.572 .762 .968 r
.88713 .79476 .89399 .80298 .91212 .80001 .90535 .7918 Metetra
.615 .744 .938 r
.28535 .73505 .29416 .74261 .30755 .73963 .29878 .73208 Metetra
.617 .746 .938 r
.29878 .73208 .30755 .73963 .32103 .73656 .31229 .72904 Metetra
.619 .748 .938 r
.31229 .72904 .32103 .73656 .33459 .7334 .32589 .7259 Metetra
.622 .751 .939 r
.32589 .7259 .33459 .7334 .34823 .73013 .33957 .72265 Metetra
.625 .753 .939 r
.33957 .72265 .34823 .73013 .36197 .72674 .35335 .71927 Metetra
.629 .756 .938 r
.35335 .71927 .36197 .72674 .37579 .72322 .36721 .71576 Metetra
.633 .758 .938 r
.36721 .71576 .37579 .72322 .3897 .71956 .38117 .71212 Metetra
.636 .76 .938 r
.38117 .71212 .3897 .71956 .40369 .71577 .39521 .70832 Metetra
.64 .763 .937 r
.39521 .70832 .40369 .71577 .41777 .71184 .40934 .70438 Metetra
.644 .765 .936 r
.40934 .70438 .41777 .71184 .43194 .70777 .42355 .70029 Metetra
.648 .766 .935 r
.42355 .70029 .43194 .70777 .44618 .70355 .43785 .69605 Metetra
.652 .768 .934 r
.43785 .69605 .44618 .70355 .46052 .6992 .45224 .69167 Metetra
.655 .769 .933 r
.45224 .69167 .46052 .6992 .47493 .69472 .4667 .68715 Metetra
.659 .77 .932 r
.4667 .68715 .47493 .69472 .48942 .69011 .48125 .68249 Metetra
.959 .66 .422 r
.48125 .68249 .48942 .69011 .50259 .88729 .49399 .87628 Metetra
.731 .729 .849 r
.49399 .87628 .50259 .88729 .51818 .8783 .50964 .86735 Metetra
.715 .727 .859 r
.50964 .86735 .51818 .8783 .53382 .87026 .52533 .85939 Metetra
.7 .725 .869 r
.52533 .85939 .53382 .87026 .54952 .86308 .54109 .85232 Metetra
.685 .723 .879 r
.54109 .85232 .54952 .86308 .5653 .85667 .55693 .84604 Metetra
.67 .722 .888 r
.55693 .84604 .5653 .85667 .58116 .85095 .57285 .84045 Metetra
.656 .721 .896 r
.57285 .84045 .58116 .85095 .59712 .84582 .58887 .83549 Metetra
.643 .72 .905 r
.58887 .83549 .59712 .84582 .61319 .84123 .60499 .83106 Metetra
.63 .72 .912 r
.60499 .83106 .61319 .84123 .62936 .83709 .62123 .82708 Metetra
.618 .721 .919 r
.62123 .82708 .62936 .83709 .64565 .83333 .63758 .8235 Metetra
.608 .722 .926 r
.63758 .8235 .64565 .83333 .66206 .8299 .65406 .82025 Metetra
.598 .723 .932 r
.65406 .82025 .66206 .8299 .67859 .82674 .67066 .81725 Metetra
.59 .725 .937 r
.67066 .81725 .67859 .82674 .69525 .82379 .68739 .81447 Metetra
.583 .727 .942 r
.68739 .81447 .69525 .82379 .71204 .82101 .70425 .81184 Metetra
.577 .729 .947 r
.70425 .81184 .71204 .82101 .72896 .81835 .72125 .80933 Metetra
.572 .732 .951 r
.72125 .80933 .72896 .81835 .74601 .81577 .73837 .80688 Metetra
.568 .735 .954 r
.73837 .80688 .74601 .81577 .76319 .81324 .75564 .80448 Metetra
.566 .739 .957 r
.75564 .80448 .76319 .81324 .78051 .81072 .77303 .80208 Metetra
.564 .742 .96 r
.77303 .80208 .78051 .81072 .79795 .8082 .79056 .79965 Metetra
.563 .745 .962 r
.79056 .79965 .79795 .8082 .81553 .80565 .80822 .79718 Metetra
.563 .749 .964 r
.80822 .79718 .81553 .80565 .83323 .80304 .82602 .79464 Metetra
.564 .752 .966 r
.82602 .79464 .83323 .80304 .85107 .80036 .84394 .79202 Metetra
.565 .756 .967 r
.84394 .79202 .85107 .80036 .86904 .79761 .862 .78931 Metetra
.567 .759 .969 r
.862 .78931 .86904 .79761 .88713 .79476 .88019 .78649 Metetra
.569 .762 .969 r
.88019 .78649 .88713 .79476 .90535 .7918 .89851 .78356 Metetra
.612 .742 .938 r
.27645 .72738 .28535 .73505 .29878 .73208 .28991 .72444 Metetra
.614 .745 .938 r
.28991 .72444 .29878 .73208 .31229 .72904 .30346 .72143 Metetra
.617 .747 .939 r
.30346 .72143 .31229 .72904 .32589 .7259 .3171 .71831 Metetra
.62 .75 .939 r
.3171 .71831 .32589 .7259 .33957 .72265 .33082 .71509 Metetra
.623 .752 .939 r
.33082 .71509 .33957 .72265 .35335 .71927 .34464 .71173 Metetra
.627 .755 .939 r
.34464 .71173 .35335 .71927 .36721 .71576 .35855 .70824 Metetra
.631 .758 .939 r
.35855 .70824 .36721 .71576 .38117 .71212 .37255 .70461 Metetra
.635 .76 .938 r
.37255 .70461 .38117 .71212 .39521 .70832 .38664 .70082 Metetra
.639 .763 .938 r
.38664 .70082 .39521 .70832 .40934 .70438 .40081 .69687 Metetra
.643 .765 .937 r
.40081 .69687 .40934 .70438 .42355 .70029 .41508 .69277 Metetra
.647 .767 .936 r
.41508 .69277 .42355 .70029 .43785 .69605 .42943 .6885 Metetra
.651 .769 .935 r
.42943 .6885 .43785 .69605 .45224 .69167 .44387 .68409 Metetra
.655 .77 .934 r
.44387 .68409 .45224 .69167 .4667 .68715 .45839 .67952 Metetra
.659 .771 .932 r
.45839 .67952 .4667 .68715 .48125 .68249 .473 .67481 Metetra
.959 .659 .421 r
.473 .67481 .48125 .68249 .49399 .87628 .48532 .865 Metetra
.728 .725 .848 r
.48532 .865 .49399 .87628 .50964 .86735 .50102 .85613 Metetra
.712 .723 .858 r
.50102 .85613 .50964 .86735 .52533 .85939 .51677 .84827 Metetra
.696 .721 .869 r
.51677 .84827 .52533 .85939 .54109 .85232 .53258 .84131 Metetra
.681 .719 .878 r
.53258 .84131 .54109 .85232 .55693 .84604 .54847 .83517 Metetra
.665 .718 .888 r
.54847 .83517 .55693 .84604 .57285 .84045 .56445 .82975 Metetra
.651 .717 .896 r
.56445 .82975 .57285 .84045 .58887 .83549 .58053 .82495 Metetra
.637 .716 .905 r
.58053 .82495 .58887 .83549 .60499 .83106 .59671 .82071 Metetra
.624 .716 .913 r
.59671 .82071 .60499 .83106 .62123 .82708 .61301 .81692 Metetra
.612 .717 .92 r
.61301 .81692 .62123 .82708 .63758 .8235 .62942 .81353 Metetra
.601 .718 .927 r
.62942 .81353 .63758 .8235 .65406 .82025 .64597 .81046 Metetra
.591 .719 .933 r
.64597 .81046 .65406 .82025 .67066 .81725 .66264 .80765 Metetra
.583 .722 .938 r
.66264 .80765 .67066 .81725 .68739 .81447 .67944 .80505 Metetra
.576 .724 .943 r
.67944 .80505 .68739 .81447 .70425 .81184 .69638 .80259 Metetra
.57 .727 .948 r
.69638 .80259 .70425 .81184 .72125 .80933 .71345 .80023 Metetra
.565 .73 .952 r
.71345 .80023 .72125 .80933 .73837 .80688 .73065 .79794 Metetra
.562 .733 .956 r
.73065 .79794 .73837 .80688 .75564 .80448 .748 .79566 Metetra
.56 .737 .959 r
.748 .79566 .75564 .80448 .77303 .80208 .76548 .79338 Metetra
.558 .741 .962 r
.76548 .79338 .77303 .80208 .79056 .79965 .78309 .79106 Metetra
.558 .745 .964 r
.78309 .79106 .79056 .79965 .80822 .79718 .80084 .78867 Metetra
.558 .748 .966 r
.80084 .78867 .80822 .79718 .82602 .79464 .81872 .78621 Metetra
.559 .752 .968 r
.81872 .78621 .82602 .79464 .84394 .79202 .83674 .78365 Metetra
.561 .756 .969 r
.83674 .78365 .84394 .79202 .862 .78931 .85489 .78098 Metetra
.563 .759 .97 r
.85489 .78098 .862 .78931 .88019 .78649 .87317 .77819 Metetra
.566 .762 .971 r
.87317 .77819 .88019 .78649 .89851 .78356 .89158 .77527 Metetra
.61 .741 .938 r
.26747 .71961 .27645 .72738 .28991 .72444 .28096 .71671 Metetra
.612 .743 .939 r
.28096 .71671 .28991 .72444 .30346 .72143 .29454 .71372 Metetra
.614 .746 .939 r
.29454 .71372 .30346 .72143 .3171 .71831 .30822 .71064 Metetra
.617 .749 .94 r
.30822 .71064 .3171 .71831 .33082 .71509 .32198 .70745 Metetra
.621 .752 .94 r
.32198 .70745 .33082 .71509 .34464 .71173 .33584 .70412 Metetra
.625 .755 .94 r
.33584 .70412 .34464 .71173 .35855 .70824 .34979 .70066 Metetra
.629 .758 .94 r
.34979 .70066 .35855 .70824 .37255 .70461 .36384 .69703 Metetra
.633 .76 .939 r
.36384 .69703 .37255 .70461 .38664 .70082 .37797 .69325 Metetra
.638 .763 .939 r
.37797 .69325 .38664 .70082 .40081 .69687 .3922 .68931 Metetra
.642 .765 .938 r
.3922 .68931 .40081 .69687 .41508 .69277 .40651 .68519 Metetra
.647 .768 .937 r
.40651 .68519 .41508 .69277 .42943 .6885 .42092 .68091 Metetra
.651 .769 .936 r
.42092 .68091 .42943 .6885 .44387 .68409 .43541 .67646 Metetra
.655 .771 .934 r
.43541 .67646 .44387 .68409 .45839 .67952 .44999 .67185 Metetra
.659 .772 .933 r
.44999 .67185 .45839 .67952 .473 .67481 .46465 .66709 Metetra
.959 .658 .42 r
.46465 .66709 .473 .67481 .48532 .865 .47657 .85344 Metetra
.725 .721 .847 r
.47657 .85344 .48532 .865 .50102 .85613 .49233 .84464 Metetra
.708 .719 .857 r
.49233 .84464 .50102 .85613 .51677 .84827 .50813 .83688 Metetra
.692 .716 .868 r
.50813 .83688 .51677 .84827 .53258 .84131 .524 .83006 Metetra
.676 .715 .878 r
.524 .83006 .53258 .84131 .54847 .83517 .53994 .82407 Metetra
.66 .713 .887 r
.53994 .82407 .54847 .83517 .56445 .82975 .55597 .81882 Metetra
.645 .712 .896 r
.55597 .81882 .56445 .82975 .58053 .82495 .5721 .81422 Metetra
.631 .712 .905 r
.5721 .81422 .58053 .82495 .59671 .82071 .58834 .81017 Metetra
.617 .712 .913 r
.58834 .81017 .59671 .82071 .61301 .81692 .6047 .80659 Metetra
.605 .713 .921 r
.6047 .80659 .61301 .81692 .62942 .81353 .62118 .80341 Metetra
.594 .714 .927 r
.62118 .80341 .62942 .81353 .64597 .81046 .63779 .80055 Metetra
.584 .716 .934 r
.63779 .80055 .64597 .81046 .66264 .80765 .65453 .79794 Metetra
.575 .718 .94 r
.65453 .79794 .66264 .80765 .67944 .80505 .6714 .79553 Metetra
.568 .721 .945 r
.6714 .79553 .67944 .80505 .69638 .80259 .68841 .79326 Metetra
.562 .724 .95 r
.68841 .79326 .69638 .80259 .71345 .80023 .70556 .79107 Metetra
.558 .728 .954 r
.70556 .79107 .71345 .80023 .73065 .79794 .72285 .78893 Metetra
.555 .731 .957 r
.72285 .78893 .73065 .79794 .748 .79566 .74027 .7868 Metetra
.553 .735 .961 r
.74027 .7868 .748 .79566 .76548 .79338 .75784 .78464 Metetra
.552 .74 .963 r
.75784 .78464 .76548 .79338 .78309 .79106 .77554 .78243 Metetra
.552 .744 .966 r
.77554 .78243 .78309 .79106 .80084 .78867 .79337 .78014 Metetra
.553 .748 .968 r
.79337 .78014 .80084 .78867 .81872 .78621 .81135 .77775 Metetra
.555 .752 .969 r
.81135 .77775 .81872 .78621 .83674 .78365 .82945 .77525 Metetra
.557 .756 .97 r
.82945 .77525 .83674 .78365 .85489 .78098 .8477 .77262 Metetra
.559 .759 .971 r
.8477 .77262 .85489 .78098 .87317 .77819 .86607 .76986 Metetra
.562 .763 .972 r
.86607 .76986 .87317 .77819 .89158 .77527 .88458 .76695 Metetra
.607 .74 .939 r
.25839 .71174 .26747 .71961 .28096 .71671 .27192 .70887 Metetra
.609 .742 .939 r
.27192 .70887 .28096 .71671 .29454 .71372 .28554 .70592 Metetra
.611 .745 .94 r
.28554 .70592 .29454 .71372 .30822 .71064 .29925 .70288 Metetra
.614 .748 .941 r
.29925 .70288 .30822 .71064 .32198 .70745 .31305 .69972 Metetra
.618 .751 .941 r
.31305 .69972 .32198 .70745 .33584 .70412 .32695 .69644 Metetra
.622 .754 .941 r
.32695 .69644 .33584 .70412 .34979 .70066 .34094 .693 Metetra
.626 .757 .941 r
.34094 .693 .34979 .70066 .36384 .69703 .35503 .6894 Metetra
.631 .76 .94 r
.35503 .6894 .36384 .69703 .37797 .69325 .36921 .68563 Metetra
.636 .763 .94 r
.36921 .68563 .37797 .69325 .3922 .68931 .38349 .68169 Metetra
.641 .766 .939 r
.38349 .68169 .3922 .68931 .40651 .68519 .39785 .67757 Metetra
.646 .768 .938 r
.39785 .67757 .40651 .68519 .42092 .68091 .41231 .67327 Metetra
.65 .77 .937 r
.41231 .67327 .42092 .68091 .43541 .67646 .42685 .66879 Metetra
.655 .772 .935 r
.42685 .66879 .43541 .67646 .44999 .67185 .44149 .66414 Metetra
.659 .773 .934 r
.44149 .66414 .44999 .67185 .46465 .66709 .45621 .65933 Metetra
.959 .657 .418 r
.45621 .65933 .46465 .66709 .47657 .85344 .46775 .84157 Metetra
.722 .717 .845 r
.46775 .84157 .47657 .85344 .49233 .84464 .48356 .83285 Metetra
.705 .714 .856 r
.48356 .83285 .49233 .84464 .50813 .83688 .49941 .8252 Metetra
.688 .712 .867 r
.49941 .8252 .50813 .83688 .524 .83006 .51533 .81853 Metetra
.671 .71 .877 r
.51533 .81853 .524 .83006 .53994 .82407 .53132 .81271 Metetra
.654 .708 .887 r
.53132 .81271 .53994 .82407 .55597 .81882 .54741 .80766 Metetra
.639 .707 .896 r
.54741 .80766 .55597 .81882 .5721 .81422 .5636 .80327 Metetra
.624 .707 .905 r
.5636 .80327 .5721 .81422 .58834 .81017 .57989 .79944 Metetra
.61 .707 .914 r
.57989 .79944 .58834 .81017 .6047 .80659 .59631 .7961 Metetra
.597 .708 .921 r
.59631 .7961 .6047 .80659 .62118 .80341 .61285 .79314 Metetra
.586 .71 .928 r
.61285 .79314 .62118 .80341 .63779 .80055 .62953 .79051 Metetra
.576 .712 .935 r
.62953 .79051 .63779 .80055 .65453 .79794 .64633 .78812 Metetra
.567 .714 .941 r
.64633 .78812 .65453 .79794 .6714 .79553 .66328 .78592 Metetra
.56 .718 .946 r
.66328 .78592 .6714 .79553 .68841 .79326 .68036 .78385 Metetra
.554 .721 .951 r
.68036 .78385 .68841 .79326 .70556 .79107 .69759 .78185 Metetra
.55 .725 .955 r
.69759 .78185 .70556 .79107 .72285 .78893 .71495 .77988 Metetra
.547 .729 .959 r
.71495 .77988 .72285 .78893 .74027 .7868 .73246 .7779 Metetra
.546 .734 .962 r
.73246 .7779 .74027 .7868 .75784 .78464 .75011 .77587 Metetra
.545 .738 .965 r
.75011 .77587 .75784 .78464 .77554 .78243 .7679 .77377 Metetra
.546 .743 .967 r
.7679 .77377 .77554 .78243 .79337 .78014 .78582 .77158 Metetra
.547 .747 .969 r
.78582 .77158 .79337 .78014 .81135 .77775 .80389 .76927 Metetra
.55 .752 .971 r
.80389 .76927 .81135 .77775 .82945 .77525 .82209 .76682 Metetra
.552 .756 .972 r
.82209 .76682 .82945 .77525 .8477 .77262 .84042 .76424 Metetra
.556 .76 .973 r
.84042 .76424 .8477 .77262 .86607 .76986 .85889 .7615 Metetra
.559 .764 .974 r
.85889 .7615 .86607 .76986 .88458 .76695 .87749 .75861 Metetra
.605 .738 .939 r
.24922 .70376 .25839 .71174 .27192 .70887 .26278 .70093 Metetra
.606 .74 .94 r
.26278 .70093 .27192 .70887 .28554 .70592 .27644 .69802 Metetra
.608 .743 .941 r
.27644 .69802 .28554 .70592 .29925 .70288 .29018 .69503 Metetra
.611 .746 .941 r
.29018 .69503 .29925 .70288 .31305 .69972 .30402 .69192 Metetra
.615 .75 .942 r
.30402 .69192 .31305 .69972 .32695 .69644 .31796 .68867 Metetra
.619 .753 .942 r
.31796 .68867 .32695 .69644 .34094 .693 .332 .68527 Metetra
.624 .757 .942 r
.332 .68527 .34094 .693 .35503 .6894 .34613 .6817 Metetra
.629 .76 .941 r
.34613 .6817 .35503 .6894 .36921 .68563 .36036 .67796 Metetra
.634 .763 .941 r
.36036 .67796 .36921 .68563 .38349 .68169 .37468 .67403 Metetra
.639 .766 .94 r
.37468 .67403 .38349 .68169 .39785 .67757 .3891 .6699 Metetra
.644 .769 .939 r
.3891 .6699 .39785 .67757 .41231 .67327 .4036 .66559 Metetra
.649 .771 .938 r
.4036 .66559 .41231 .67327 .42685 .66879 .4182 .66109 Metetra
.654 .773 .936 r
.4182 .66109 .42685 .66879 .44149 .66414 .43289 .6564 Metetra
.659 .774 .935 r
.43289 .6564 .44149 .66414 .45621 .65933 .44767 .65153 Metetra
.958 .657 .417 r
.44767 .65153 .45621 .65933 .46775 .84157 .45885 .82937 Metetra
.718 .712 .844 r
.45885 .82937 .46775 .84157 .48356 .83285 .47471 .82074 Metetra
.701 .709 .855 r
.47471 .82074 .48356 .83285 .49941 .8252 .49061 .81322 Metetra
.683 .707 .866 r
.49061 .81322 .49941 .8252 .51533 .81853 .50658 .80671 Metetra
.665 .704 .876 r
.50658 .80671 .51533 .81853 .53132 .81271 .52263 .80109 Metetra
.648 .703 .886 r
.52263 .80109 .53132 .81271 .54741 .80766 .53876 .79625 Metetra
.632 .702 .896 r
.53876 .79625 .54741 .80766 .5636 .80327 .555 .7921 Metetra
.616 .701 .905 r
.555 .7921 .5636 .80327 .57989 .79944 .57136 .78852 Metetra
.602 .702 .914 r
.57136 .78852 .57989 .79944 .59631 .7961 .58783 .78542 Metetra
.588 .703 .922 r
.58783 .78542 .59631 .7961 .61285 .79314 .60444 .78272 Metetra
.577 .705 .93 r
.60444 .78272 .61285 .79314 .62953 .79051 .62117 .78034 Metetra
.566 .707 .936 r
.62117 .78034 .62953 .79051 .64633 .78812 .63805 .7782 Metetra
.558 .71 .943 r
.63805 .7782 .64633 .78812 .66328 .78592 .65506 .77623 Metetra
.551 .714 .948 r
.65506 .77623 .66328 .78592 .68036 .78385 .67222 .77437 Metetra
.545 .718 .953 r
.67222 .77437 .68036 .78385 .69759 .78185 .68953 .77257 Metetra
.541 .723 .957 r
.68953 .77257 .69759 .78185 .71495 .77988 .70697 .77078 Metetra
.539 .727 .961 r
.70697 .77078 .71495 .77988 .73246 .7779 .72456 .76896 Metetra
.538 .732 .964 r
.72456 .76896 .73246 .7779 .75011 .77587 .7423 .76708 Metetra
.538 .737 .967 r
.7423 .76708 .75011 .77587 .7679 .77377 .76017 .7651 Metetra
.539 .742 .969 r
.76017 .7651 .7679 .77377 .78582 .77158 .77819 .76301 Metetra
.542 .747 .971 r
.77819 .76301 .78582 .77158 .80389 .76927 .79634 .76077 Metetra
.544 .752 .973 r
.79634 .76077 .80389 .76927 .82209 .76682 .81463 .75839 Metetra
.548 .756 .974 r
.81463 .75839 .82209 .76682 .84042 .76424 .83306 .75584 Metetra
.552 .761 .975 r
.83306 .75584 .84042 .76424 .85889 .7615 .85163 .75313 Metetra
.556 .765 .975 r
.85163 .75313 .85889 .7615 .87749 .75861 .87033 .75024 Metetra
.602 .736 .939 r
.23997 .69565 .24922 .70376 .26278 .70093 .25356 .69287 Metetra
.603 .739 .94 r
.25356 .69287 .26278 .70093 .27644 .69802 .26725 .69001 Metetra
.605 .742 .941 r
.26725 .69001 .27644 .69802 .29018 .69503 .28103 .68707 Metetra
.608 .745 .942 r
.28103 .68707 .29018 .69503 .30402 .69192 .2949 .68402 Metetra
.612 .749 .942 r
.2949 .68402 .30402 .69192 .31796 .68867 .30888 .68082 Metetra
.616 .752 .943 r
.30888 .68082 .31796 .68867 .332 .68527 .32296 .67747 Metetra
.621 .756 .943 r
.32296 .67747 .332 .68527 .34613 .6817 .33713 .67394 Metetra
.626 .76 .943 r
.33713 .67394 .34613 .6817 .36036 .67796 .3514 .67023 Metetra
.632 .763 .942 r
.3514 .67023 .36036 .67796 .37468 .67403 .36577 .66632 Metetra
.637 .767 .941 r
.36577 .66632 .37468 .67403 .3891 .6699 .38024 .6622 Metetra
.643 .77 .94 r
.38024 .6622 .3891 .6699 .4036 .66559 .3948 .65787 Metetra
.649 .772 .939 r
.3948 .65787 .4036 .66559 .4182 .66109 .40946 .65335 Metetra
.654 .774 .937 r
.40946 .65335 .4182 .66109 .43289 .6564 .4242 .64862 Metetra
.659 .776 .936 r
.4242 .64862 .43289 .6564 .44767 .65153 .43904 .6437 Metetra
.958 .655 .415 r
.43904 .6437 .44767 .65153 .45885 .82937 .44988 .81682 Metetra
.715 .707 .842 r
.44988 .81682 .45885 .82937 .47471 .82074 .46579 .80828 Metetra
.696 .704 .853 r
.46579 .80828 .47471 .82074 .49061 .81322 .48174 .8009 Metetra
.677 .701 .864 r
.48174 .8009 .49061 .81322 .50658 .80671 .49775 .79458 Metetra
.659 .698 .875 r
.49775 .79458 .50658 .80671 .52263 .80109 .51385 .78917 Metetra
.641 .697 .886 r
.51385 .78917 .52263 .80109 .53876 .79625 .53003 .78458 Metetra
.624 .696 .896 r
.53003 .78458 .53876 .79625 .555 .7921 .54633 .78069 Metetra
.608 .695 .906 r
.54633 .78069 .555 .7921 .57136 .78852 .56273 .77739 Metetra
.593 .696 .915 r
.56273 .77739 .57136 .78852 .58783 .78542 .57926 .77457 Metetra
.579 .697 .923 r
.57926 .77457 .58783 .78542 .60444 .78272 .59593 .77216 Metetra
.567 .699 .931 r
.59593 .77216 .60444 .78272 .62117 .78034 .61273 .77005 Metetra
.556 .702 .938 r
.61273 .77005 .62117 .78034 .63805 .7782 .62967 .76818 Metetra
.547 .706 .944 r
.62967 .76818 .63805 .7782 .65506 .77623 .64676 .76646 Metetra
.54 .71 .95 r
.64676 .76646 .65506 .77623 .67222 .77437 .66399 .76484 Metetra
.535 .715 .955 r
.66399 .76484 .67222 .77437 .68953 .77257 .68137 .76326 Metetra
.532 .72 .959 r
.68137 .76326 .68953 .77257 .70697 .77078 .6989 .76166 Metetra
.53 .725 .963 r
.6989 .76166 .70697 .77078 .72456 .76896 .71657 .76002 Metetra
.53 .731 .966 r
.71657 .76002 .72456 .76896 .7423 .76708 .73439 .75828 Metetra
.53 .736 .969 r
.73439 .75828 .7423 .76708 .76017 .7651 .75236 .75643 Metetra
.532 .742 .971 r
.75236 .75643 .76017 .7651 .77819 .76301 .77046 .75443 Metetra
.535 .747 .973 r
.77046 .75443 .77819 .76301 .79634 .76077 .78871 .75228 Metetra
.539 .752 .975 r
.78871 .75228 .79634 .76077 .81463 .75839 .8071 .74995 Metetra
.543 .757 .976 r
.8071 .74995 .81463 .75839 .83306 .75584 .82562 .74744 Metetra
.548 .761 .976 r
.82562 .74744 .83306 .75584 .85163 .75313 .84428 .74475 Metetra
.553 .766 .977 r
.84428 .74475 .85163 .75313 .87033 .75024 .86308 .74186 Metetra
.599 .734 .939 r
.23062 .68742 .23997 .69565 .25356 .69287 .24424 .68468 Metetra
.6 .737 .94 r
.24424 .68468 .25356 .69287 .26725 .69001 .25796 .68189 Metetra
.601 .74 .942 r
.25796 .68189 .26725 .69001 .28103 .68707 .27178 .67901 Metetra
.604 .743 .943 r
.27178 .67901 .28103 .68707 .2949 .68402 .28569 .67602 Metetra
.608 .747 .943 r
.28569 .67602 .2949 .68402 .30888 .68082 .2997 .67289 Metetra
.612 .751 .944 r
.2997 .67289 .30888 .68082 .32296 .67747 .31382 .66959 Metetra
.617 .755 .944 r
.31382 .66959 .32296 .67747 .33713 .67394 .32803 .66612 Metetra
.623 .76 .944 r
.32803 .66612 .33713 .67394 .3514 .67023 .34235 .66244 Metetra
.629 .763 .943 r
.34235 .66244 .3514 .67023 .36577 .66632 .35677 .65856 Metetra
.635 .767 .943 r
.35677 .65856 .36577 .66632 .38024 .6622 .37128 .65445 Metetra
.641 .77 .941 r
.37128 .65445 .38024 .6622 .3948 .65787 .3859 .65013 Metetra
.648 .773 .94 r
.3859 .65013 .3948 .65787 .40946 .65335 .40061 .64558 Metetra
.653 .776 .938 r
.40061 .64558 .40946 .65335 .4242 .64862 .41541 .64081 Metetra
.659 .777 .937 r
.41541 .64081 .4242 .64862 .43904 .6437 .43031 .63584 Metetra
.958 .654 .413 r
.43031 .63584 .43904 .6437 .44988 .81682 .44083 .80387 Metetra
.711 .702 .84 r
.44083 .80387 .44988 .81682 .46579 .80828 .45679 .79544 Metetra
.691 .698 .851 r
.45679 .79544 .46579 .80828 .48174 .8009 .47279 .78823 Metetra
.672 .694 .863 r
.47279 .78823 .48174 .8009 .49775 .79458 .48885 .78211 Metetra
.652 .692 .874 r
.48885 .78211 .49775 .79458 .51385 .78917 .50499 .77695 Metetra
.633 .69 .885 r
.50499 .77695 .51385 .78917 .53003 .78458 .52122 .77263 Metetra
.615 .689 .896 r
.52122 .77263 .53003 .78458 .54633 .78069 .53756 .76904 Metetra
.598 .689 .906 r
.53756 .76904 .54633 .78069 .56273 .77739 .55402 .76604 Metetra
.582 .69 .915 r
.55402 .76604 .56273 .77739 .57926 .77457 .57061 .76354 Metetra
.568 .691 .924 r
.57061 .76354 .57926 .77457 .59593 .77216 .58733 .76144 Metetra
.555 .694 .932 r
.58733 .76144 .59593 .77216 .61273 .77005 .6042 .75964 Metetra
.545 .697 .939 r
.6042 .75964 .61273 .77005 .62967 .76818 .6212 .75806 Metetra
.536 .701 .946 r
.6212 .75806 .62967 .76818 .64676 .76646 .63836 .75662 Metetra
.529 .706 .952 r
.63836 .75662 .64676 .76646 .66399 .76484 .65567 .75526 Metetra
.524 .711 .957 r
.65567 .75526 .66399 .76484 .68137 .76326 .67313 .75391 Metetra
.521 .717 .962 r
.67313 .75391 .68137 .76326 .6989 .76166 .69074 .75253 Metetra
.52 .723 .965 r
.69074 .75253 .6989 .76166 .71657 .76002 .7085 .75107 Metetra
.52 .729 .969 r
.7085 .75107 .71657 .76002 .73439 .75828 .7264 .74949 Metetra
.522 .735 .971 r
.7264 .74949 .73439 .75828 .75236 .75643 .74445 .74777 Metetra
.525 .741 .974 r
.74445 .74777 .75236 .75643 .77046 .75443 .76265 .74588 Metetra
.529 .747 .975 r
.76265 .74588 .77046 .75443 .78871 .75228 .78099 .7438 Metetra
.533 .753 .977 r
.78099 .7438 .78871 .75228 .8071 .74995 .79947 .74153 Metetra
.539 .758 .978 r
.79947 .74153 .8071 .74995 .82562 .74744 .81809 .73905 Metetra
.544 .763 .978 r
.81809 .73905 .82562 .74744 .84428 .74475 .83685 .73636 Metetra
.55 .767 .978 r
.83685 .73636 .84428 .74475 .86308 .74186 .85574 .73346 Metetra
.595 .732 .939 r
.22118 .67905 .23062 .68742 .24424 .68468 .23484 .67637 Metetra
.596 .734 .941 r
.23484 .67637 .24424 .68468 .25796 .68189 .24858 .67364 Metetra
.597 .737 .942 r
.24858 .67364 .25796 .68189 .27178 .67901 .26243 .67083 Metetra
.6 .741 .943 r
.26243 .67083 .27178 .67901 .28569 .67602 .27638 .66792 Metetra
.604 .745 .944 r
.27638 .66792 .28569 .67602 .2997 .67289 .29042 .66486 Metetra
.608 .75 .945 r
.29042 .66486 .2997 .67289 .31382 .66959 .30458 .66164 Metetra
.614 .754 .945 r
.30458 .66164 .31382 .66959 .32803 .66612 .31883 .65823 Metetra
.62 .759 .945 r
.31883 .65823 .32803 .66612 .34235 .66244 .33319 .65461 Metetra
.626 .763 .945 r
.33319 .65461 .34235 .66244 .35677 .65856 .34766 .65076 Metetra
.633 .768 .944 r
.34766 .65076 .35677 .65856 .37128 .65445 .36223 .64668 Metetra
.64 .771 .943 r
.36223 .64668 .37128 .65445 .3859 .65013 .37689 .64236 Metetra
.646 .775 .942 r
.37689 .64236 .3859 .65013 .40061 .64558 .39166 .63779 Metetra
.653 .777 .94 r
.39166 .63779 .40061 .64558 .41541 .64081 .40652 .63299 Metetra
.659 .779 .938 r
.40652 .63299 .41541 .64081 .43031 .63584 .42147 .62797 Metetra
.958 .653 .41 r
.42147 .62797 .43031 .63584 .44083 .80387 .43172 .79049 Metetra
.706 .695 .837 r
.43172 .79049 .44083 .80387 .45679 .79544 .44772 .78219 Metetra
.686 .691 .849 r
.44772 .78219 .45679 .79544 .47279 .78823 .46377 .77516 Metetra
.665 .687 .861 r
.46377 .77516 .47279 .78823 .48885 .78211 .47987 .76928 Metetra
.645 .684 .873 r
.47987 .76928 .48885 .78211 .50499 .77695 .49605 .7644 Metetra
.625 .682 .884 r
.49605 .7644 .50499 .77695 .52122 .77263 .51233 .76039 Metetra
.606 .681 .895 r
.51233 .76039 .52122 .77263 .53756 .76904 .52871 .75712 Metetra
.588 .681 .906 r
.52871 .75712 .53756 .76904 .55402 .76604 .54522 .75447 Metetra
.571 .682 .916 r
.54522 .75447 .55402 .76604 .57061 .76354 .56186 .75233 Metetra
.556 .684 .925 r
.56186 .75233 .57061 .76354 .58733 .76144 .57864 .75058 Metetra
.543 .687 .933 r
.57864 .75058 .58733 .76144 .6042 .75964 .59557 .74912 Metetra
.532 .691 .941 r
.59557 .74912 .6042 .75964 .6212 .75806 .61264 .74787 Metetra
.523 .696 .948 r
.61264 .74787 .6212 .75806 .63836 .75662 .62987 .74674 Metetra
.516 .701 .954 r
.62987 .74674 .63836 .75662 .65567 .75526 .64726 .74566 Metetra
.512 .707 .959 r
.64726 .74566 .65567 .75526 .67313 .75391 .66479 .74456 Metetra
.509 .714 .964 r
.66479 .74456 .67313 .75391 .69074 .75253 .68248 .74341 Metetra
.509 .721 .968 r
.68248 .74341 .69074 .75253 .7085 .75107 .70033 .74214 Metetra
.51 .727 .971 r
.70033 .74214 .7085 .75107 .7264 .74949 .71832 .74073 Metetra
.513 .734 .974 r
.71832 .74073 .7264 .74949 .74445 .74777 .73646 .73914 Metetra
.517 .741 .976 r
.73646 .73914 .74445 .74777 .76265 .74588 .75475 .73735 Metetra
.522 .747 .978 r
.75475 .73735 .76265 .74588 .78099 .7438 .77318 .73535 Metetra
.528 .753 .979 r
.77318 .73535 .78099 .7438 .79947 .74153 .79176 .73313 Metetra
.534 .759 .98 r
.79176 .73313 .79947 .74153 .81809 .73905 .81048 .73067 Metetra
.54 .764 .98 r
.81048 .73067 .81809 .73905 .83685 .73636 .82933 .72799 Metetra
.547 .769 .98 r
.82933 .72799 .83685 .73636 .85574 .73346 .84832 .72507 Metetra
.592 .729 .939 r
.21166 .67054 .22118 .67905 .23484 .67637 .22534 .66791 Metetra
.592 .731 .941 r
.22534 .66791 .23484 .67637 .24858 .67364 .23911 .66526 Metetra
.593 .735 .943 r
.23911 .66526 .24858 .67364 .26243 .67083 .25299 .66253 Metetra
.595 .739 .944 r
.25299 .66253 .26243 .67083 .27638 .66792 .26697 .65971 Metetra
.599 .743 .945 r
.26697 .65971 .27638 .66792 .29042 .66486 .28105 .65674 Metetra
.603 .748 .946 r
.28105 .65674 .29042 .66486 .30458 .66164 .29524 .65361 Metetra
.609 .753 .947 r
.29524 .65361 .30458 .66164 .31883 .65823 .30953 .65028 Metetra
.616 .758 .947 r
.30953 .65028 .31883 .65823 .33319 .65461 .32394 .64672 Metetra
.623 .763 .947 r
.32394 .64672 .33319 .65461 .34766 .65076 .33845 .64293 Metetra
.63 .768 .946 r
.33845 .64293 .34766 .65076 .36223 .64668 .35306 .63888 Metetra
.637 .772 .945 r
.35306 .63888 .36223 .64668 .37689 .64236 .36778 .63457 Metetra
.645 .776 .943 r
.36778 .63457 .37689 .64236 .39166 .63779 .3826 .63 Metetra
.652 .779 .941 r
.3826 .63 .39166 .63779 .40652 .63299 .39752 .62517 Metetra
.658 .781 .939 r
.39752 .62517 .40652 .63299 .42147 .62797 .41253 .62009 Metetra
.958 .651 .408 r
.41253 .62009 .42147 .62797 .43172 .79049 .42254 .77664 Metetra
.701 .688 .834 r
.42254 .77664 .43172 .79049 .44772 .78219 .43859 .76848 Metetra
.68 .683 .847 r
.43859 .76848 .44772 .78219 .46377 .77516 .45467 .76166 Metetra
.658 .679 .859 r
.45467 .76166 .46377 .77516 .47987 .76928 .47081 .75605 Metetra
.636 .676 .871 r
.47081 .75605 .47987 .76928 .49605 .7644 .48703 .75148 Metetra
.615 .674 .883 r
.48703 .75148 .49605 .7644 .51233 .76039 .50335 .74783 Metetra
.595 .673 .895 r
.50335 .74783 .51233 .76039 .52871 .75712 .51978 .74494 Metetra
.576 .673 .906 r
.51978 .74494 .52871 .75712 .54522 .75447 .53633 .74268 Metetra
.558 .674 .916 r
.53633 .74268 .54522 .75447 .56186 .75233 .55303 .74093 Metetra
.542 .677 .926 r
.55303 .74093 .56186 .75233 .57864 .75058 .56986 .73957 Metetra
.528 .68 .935 r
.56986 .73957 .57864 .75058 .59557 .74912 .58685 .7385 Metetra
.517 .685 .943 r
.58685 .7385 .59557 .74912 .61264 .74787 .60399 .73761 Metetra
.508 .69 .95 r
.60399 .73761 .61264 .74787 .62987 .74674 .62129 .73681 Metetra
.502 .696 .956 r
.62129 .73681 .62987 .74674 .64726 .74566 .63875 .73604 Metetra
.498 .703 .962 r
.63875 .73604 .64726 .74566 .66479 .74456 .65636 .73523 Metetra
.496 .711 .966 r
.65636 .73523 .66479 .74456 .68248 .74341 .67413 .73431 Metetra
.497 .718 .97 r
.67413 .73431 .68248 .74341 .70033 .74214 .69206 .73326 Metetra
.499 .726 .974 r
.69206 .73326 .70033 .74214 .71832 .74073 .71014 .73201 Metetra
.503 .734 .976 r
.71014 .73201 .71832 .74073 .73646 .73914 .72838 .73056 Metetra
.508 .741 .979 r
.72838 .73056 .73646 .73914 .75475 .73735 .74676 .72888 Metetra
.515 .748 .98 r
.74676 .72888 .75475 .73735 .77318 .73535 .76529 .72695 Metetra
.521 .755 .981 r
.76529 .72695 .77318 .73535 .79176 .73313 .78396 .72477 Metetra
.529 .761 .982 r
.78396 .72477 .79176 .73313 .81048 .73067 .80277 .72233 Metetra
.536 .766 .982 r
.80277 .72233 .81048 .73067 .82933 .72799 .82172 .71963 Metetra
.544 .771 .982 r
.82172 .71963 .82933 .72799 .84832 .72507 .84082 .71668 Metetra
.588 .726 .939 r
.20204 .66186 .21166 .67054 .22534 .66791 .21574 .6593 Metetra
.587 .728 .941 r
.21574 .6593 .22534 .66791 .23911 .66526 .22955 .65672 Metetra
.588 .732 .943 r
.22955 .65672 .23911 .66526 .25299 .66253 .24345 .65409 Metetra
.59 .736 .945 r
.24345 .65409 .25299 .66253 .26697 .65971 .25746 .65137 Metetra
.593 .741 .946 r
.25746 .65137 .26697 .65971 .28105 .65674 .27157 .64851 Metetra
.598 .746 .947 r
.27157 .64851 .28105 .65674 .29524 .65361 .28579 .64549 Metetra
.604 .752 .948 r
.28579 .64549 .29524 .65361 .30953 .65028 .30013 .64225 Metetra
.611 .757 .949 r
.30013 .64225 .30953 .65028 .32394 .64672 .31457 .63878 Metetra
.618 .763 .949 r
.31457 .63878 .32394 .64672 .33845 .64293 .32912 .63506 Metetra
.626 .768 .948 r
.32912 .63506 .33845 .64293 .35306 .63888 .34379 .63106 Metetra
.635 .773 .947 r
.34379 .63106 .35306 .63888 .36778 .63457 .35856 .62677 Metetra
.643 .777 .945 r
.35856 .62677 .36778 .63457 .3826 .63 .37344 .6222 Metetra
.651 .781 .943 r
.37344 .6222 .3826 .63 .39752 .62517 .38841 .61734 Metetra
.658 .784 .941 r
.38841 .61734 .39752 .62517 .41253 .62009 .40349 .61221 Metetra
.957 .649 .404 r
.40349 .61221 .41253 .62009 .42254 .77664 .4133 .76226 Metetra
.696 .68 .831 r
.4133 .76226 .42254 .77664 .43859 .76848 .42939 .75427 Metetra
.673 .675 .844 r
.42939 .75427 .43859 .76848 .45467 .76166 .44551 .74769 Metetra
.65 .67 .857 r
.44551 .74769 .45467 .76166 .47081 .75605 .46168 .74238 Metetra
.627 .667 .869 r
.46168 .74238 .47081 .75605 .48703 .75148 .47793 .73818 Metetra
.604 .665 .882 r
.47793 .73818 .48703 .75148 .50335 .74783 .49429 .73492 Metetra
.582 .663 .894 r
.49429 .73492 .50335 .74783 .51978 .74494 .51076 .73246 Metetra
.562 .664 .905 r
.51076 .73246 .51978 .74494 .53633 .74268 .52736 .73065 Metetra
.543 .665 .916 r
.52736 .73065 .53633 .74268 .55303 .74093 .5441 .72935 Metetra
.526 .668 .927 r
.5441 .72935 .55303 .74093 .56986 .73957 .56099 .72844 Metetra
.512 .672 .936 r
.56099 .72844 .56986 .73957 .58685 .7385 .57803 .72779 Metetra
.5 .677 .944 r
.57803 .72779 .58685 .7385 .60399 .73761 .59524 .7273 Metetra
.491 .684 .952 r
.59524 .7273 .60399 .73761 .62129 .73681 .61261 .72688 Metetra
.485 .691 .959 r
.61261 .72688 .62129 .73681 .63875 .73604 .63014 .72645 Metetra
.482 .699 .964 r
.63014 .72645 .63875 .73604 .65636 .73523 .64783 .72594 Metetra
.481 .707 .969 r
.64783 .72594 .65636 .73523 .67413 .73431 .66569 .72528 Metetra
.483 .716 .973 r
.66569 .72528 .67413 .73431 .69206 .73326 .6837 .72444 Metetra
.487 .725 .976 r
.6837 .72444 .69206 .73326 .71014 .73201 .70187 .72338 Metetra
.492 .733 .979 r
.70187 .72338 .71014 .73201 .72838 .73056 .7202 .72206 Metetra
.499 .741 .981 r
.7202 .72206 .72838 .73056 .74676 .72888 .73868 .72048 Metetra
.507 .749 .983 r
.73868 .72048 .74676 .72888 .76529 .72695 .7573 .71861 Metetra
.515 .756 .983 r
.7573 .71861 .76529 .72695 .78396 .72477 .77607 .71646 Metetra
.524 .763 .984 r
.77607 .71646 .78396 .72477 .80277 .72233 .79498 .71402 Metetra
.533 .768 .984 r
.79498 .71402 .80277 .72233 .82172 .71963 .81403 .7113 Metetra
.541 .773 .984 r
.81403 .7113 .82172 .71963 .84082 .71668 .83322 .7083 Metetra
.585 .723 .938 r
.19233 .653 .20204 .66186 .21574 .6593 .20606 .65051 Metetra
.583 .725 .941 r
.20606 .65051 .21574 .6593 .22955 .65672 .21989 .64802 Metetra
.583 .728 .943 r
.21989 .64802 .22955 .65672 .24345 .65409 .23382 .6455 Metetra
.584 .732 .945 r
.23382 .6455 .24345 .65409 .25746 .65137 .24785 .6429 Metetra
.587 .737 .947 r
.24785 .6429 .25746 .65137 .27157 .64851 .26199 .64017 Metetra
.592 .743 .949 r
.26199 .64017 .27157 .64851 .28579 .64549 .27625 .63727 Metetra
.598 .749 .95 r
.27625 .63727 .28579 .64549 .30013 .64225 .29061 .63416 Metetra
.605 .756 .951 r
.29061 .63416 .30013 .64225 .31457 .63878 .3051 .6308 Metetra
.613 .763 .951 r
.3051 .6308 .31457 .63878 .32912 .63506 .31969 .62716 Metetra
.622 .769 .95 r
.31969 .62716 .32912 .63506 .34379 .63106 .3344 .62323 Metetra
.631 .774 .949 r
.3344 .62323 .34379 .63106 .35856 .62677 .34923 .61898 Metetra
.64 .779 .947 r
.34923 .61898 .35856 .62677 .37344 .6222 .36416 .61442 Metetra
.649 .783 .945 r
.36416 .61442 .37344 .6222 .38841 .61734 .37919 .60954 Metetra
.658 .786 .943 r
.37919 .60954 .38841 .61734 .40349 .61221 .39433 .60436 Metetra
.957 .647 .401 r
.39433 .60436 .40349 .61221 .4133 .76226 .40401 .74729 Metetra
.69 .672 .827 r
.40401 .74729 .4133 .76226 .42939 .75427 .42013 .73948 Metetra
.665 .665 .84 r
.42013 .73948 .42939 .75427 .44551 .74769 .43628 .73319 Metetra
.64 .66 .854 r
.43628 .73319 .44551 .74769 .46168 .74238 .45248 .72823 Metetra
.616 .656 .867 r
.45248 .72823 .46168 .74238 .47793 .73818 .46876 .72445 Metetra
.591 .654 .88 r
.46876 .72445 .47793 .73818 .49429 .73492 .48515 .72165 Metetra
.568 .652 .893 r
.48515 .72165 .49429 .73492 .51076 .73246 .50165 .71969 Metetra
.546 .653 .905 r
.50165 .71969 .51076 .73246 .52736 .73065 .51829 .71838 Metetra
.525 .654 .917 r
.51829 .71838 .52736 .73065 .5441 .72935 .53508 .7176 Metetra
.508 .658 .928 r
.53508 .7176 .5441 .72935 .56099 .72844 .55202 .71718 Metetra
.492 .663 .937 r
.55202 .71718 .56099 .72844 .57803 .72779 .56912 .71702 Metetra
.48 .669 .946 r
.56912 .71702 .57803 .72779 .59524 .7273 .58639 .71698 Metetra
.472 .676 .954 r
.58639 .71698 .59524 .7273 .61261 .72688 .60383 .71697 Metetra
.466 .685 .961 r
.60383 .71697 .61261 .72688 .63014 .72645 .62143 .71691 Metetra
.464 .694 .967 r
.62143 .71691 .63014 .72645 .64783 .72594 .63921 .71672 Metetra
.464 .704 .972 r
.63921 .71672 .64783 .72594 .66569 .72528 .65715 .71635 Metetra
.468 .714 .976 r
.65715 .71635 .66569 .72528 .6837 .72444 .67525 .71573 Metetra
.473 .724 .979 r
.67525 .71573 .6837 .72444 .70187 .72338 .69351 .71485 Metetra
.481 .733 .982 r
.69351 .71485 .70187 .72338 .7202 .72206 .71193 .71367 Metetra
.489 .742 .984 r
.71193 .71367 .7202 .72206 .73868 .72048 .7305 .71217 Metetra
.499 .751 .985 r
.7305 .71217 .73868 .72048 .7573 .71861 .74922 .71036 Metetra
.509 .758 .986 r
.74922 .71036 .7573 .71861 .77607 .71646 .76808 .70822 Metetra
.52 .765 .986 r
.76808 .70822 .77607 .71646 .79498 .71402 .78709 .70576 Metetra
.53 .771 .986 r
.78709 .70576 .79498 .71402 .81403 .7113 .80624 .703 Metetra
.54 .776 .985 r
.80624 .703 .81403 .7113 .83322 .7083 .82553 .69993 Metetra
.581 .719 .937 r
.18253 .64395 .19233 .653 .20606 .65051 .19629 .64153 Metetra
.578 .721 .94 r
.19629 .64153 .20606 .65051 .21989 .64802 .21014 .63914 Metetra
.577 .724 .943 r
.21014 .63914 .21989 .64802 .23382 .6455 .22409 .63674 Metetra
.577 .728 .945 r
.22409 .63674 .23382 .6455 .24785 .6429 .23814 .63427 Metetra
.58 .733 .948 r
.23814 .63427 .24785 .6429 .26199 .64017 .25231 .63169 Metetra
.584 .74 .95 r
.25231 .63169 .26199 .64017 .27625 .63727 .26659 .62895 Metetra
.59 .747 .952 r
.26659 .62895 .27625 .63727 .29061 .63416 .28099 .62598 Metetra
.598 .754 .953 r
.28099 .62598 .29061 .63416 .3051 .6308 .29551 .62276 Metetra
.607 .762 .953 r
.29551 .62276 .3051 .6308 .31969 .62716 .31015 .61924 Metetra
.617 .769 .953 r
.31015 .61924 .31969 .62716 .3344 .62323 .3249 .6154 Metetra
.627 .775 .952 r
.3249 .6154 .3344 .62323 .34923 .61898 .33978 .61121 Metetra
.637 .781 .95 r
.33978 .61121 .34923 .61898 .36416 .61442 .35476 .60667 Metetra
.648 .786 .948 r
.35476 .60667 .36416 .61442 .37919 .60954 .36986 .60178 Metetra
.657 .79 .945 r
.36986 .60178 .37919 .60954 .39433 .60436 .38506 .59655 Metetra
.956 .644 .396 r
.38506 .59655 .39433 .60436 .40401 .74729 .39466 .73166 Metetra
.683 .662 .823 r
.39466 .73166 .40401 .74729 .42013 .73948 .41081 .72406 Metetra
.657 .655 .836 r
.41081 .72406 .42013 .73948 .43628 .73319 .42698 .71809 Metetra
.63 .649 .85 r
.42698 .71809 .43628 .73319 .45248 .72823 .44321 .71355 Metetra
.603 .644 .864 r
.44321 .71355 .45248 .72823 .46876 .72445 .45952 .71024 Metetra
.577 .641 .878 r
.45952 .71024 .46876 .72445 .48515 .72165 .47593 .70799 Metetra
.551 .64 .891 r
.47593 .70799 .48515 .72165 .50165 .71969 .49246 .70659 Metetra
.527 .64 .905 r
.49246 .70659 .50165 .71969 .51829 .71838 .50914 .70587 Metetra
.505 .642 .917 r
.50914 .70587 .51829 .71838 .53508 .7176 .52596 .70567 Metetra
.486 .646 .928 r
.52596 .70567 .53508 .7176 .55202 .71718 .54295 .70583 Metetra
.47 .652 .939 r
.54295 .70583 .55202 .71718 .56912 .71702 .56011 .70621 Metetra
.457 .659 .948 r
.56011 .70621 .56912 .71702 .58639 .71698 .57744 .70668 Metetra
.448 .668 .956 r
.57744 .70668 .58639 .71698 .60383 .71697 .59495 .70713 Metetra
.444 .678 .964 r
.59495 .70713 .60383 .71697 .62143 .71691 .61263 .70748 Metetra
.443 .689 .97 r
.61263 .70748 .62143 .71691 .63921 .71672 .63048 .70764 Metetra
.445 .7 .975 r
.63048 .70764 .63921 .71672 .65715 .71635 .64851 .70755 Metetra
.45 .711 .979 r
.64851 .70755 .65715 .71635 .67525 .71573 .6667 .70717 Metetra
.458 .723 .982 r
.6667 .70717 .67525 .71573 .69351 .71485 .68505 .70647 Metetra
.468 .734 .985 r
.68505 .70647 .69351 .71485 .71193 .71367 .70356 .70541 Metetra
.479 .744 .987 r
.70356 .70541 .71193 .71367 .7305 .71217 .72222 .70399 Metetra
.491 .753 .988 r
.72222 .70399 .7305 .71217 .74922 .71036 .74104 .70221 Metetra
.503 .761 .988 r
.74104 .70221 .74922 .71036 .76808 .70822 .76 .70007 Metetra
.515 .768 .988 r
.76 .70007 .76808 .70822 .78709 .70576 .77911 .69757 Metetra
.527 .774 .988 r
.77911 .69757 .78709 .70576 .80624 .703 .79835 .69474 Metetra
.538 .779 .987 r
.79835 .69474 .80624 .703 .82553 .69993 .81774 .69158 Metetra
.577 .715 .936 r
.17265 .63469 .18253 .64395 .19629 .64153 .18643 .63233 Metetra
.573 .716 .939 r
.18643 .63233 .19629 .64153 .21014 .63914 .2003 .63004 Metetra
.571 .719 .942 r
.2003 .63004 .21014 .63914 .22409 .63674 .21427 .62778 Metetra
.57 .723 .945 r
.21427 .62778 .22409 .63674 .23814 .63427 .22834 .62547 Metetra
.572 .728 .948 r
.22834 .62547 .23814 .63427 .25231 .63169 .24253 .62307 Metetra
.576 .735 .951 r
.24253 .62307 .25231 .63169 .26659 .62895 .25684 .62051 Metetra
.582 .743 .953 r
.25684 .62051 .26659 .62895 .28099 .62598 .27126 .61773 Metetra
.59 .752 .955 r
.27126 .61773 .28099 .62598 .29551 .62276 .28581 .61467 Metetra
.599 .76 .956 r
.28581 .61467 .29551 .62276 .31015 .61924 .30049 .61131 Metetra
.61 .769 .956 r
.30049 .61131 .31015 .61924 .3249 .6154 .31529 .60758 Metetra
.622 .777 .955 r
.31529 .60758 .3249 .6154 .33978 .61121 .33021 .60348 Metetra
.634 .783 .953 r
.33021 .60348 .33978 .61121 .35476 .60667 .34525 .59899 Metetra
.645 .789 .951 r
.34525 .59899 .35476 .60667 .36986 .60178 .3604 .59409 Metetra
.656 .793 .948 r
.3604 .59409 .36986 .60178 .38506 .59655 .37567 .58882 Metetra
.955 .64 .39 r
.37567 .58882 .38506 .59655 .39466 .73166 .38528 .71525 Metetra
.675 .65 .817 r
.38528 .71525 .39466 .73166 .41081 .72406 .40145 .70791 Metetra
.647 .642 .831 r
.40145 .70791 .41081 .72406 .42698 .71809 .41764 .70231 Metetra
.618 .635 .846 r
.41764 .70231 .42698 .71809 .44321 .71355 .43388 .69825 Metetra
.589 .63 .86 r
.43388 .69825 .44321 .71355 .45952 .71024 .4502 .69552 Metetra
.559 .626 .875 r
.4502 .69552 .45952 .71024 .47593 .70799 .46663 .69389 Metetra
.531 .625 .89 r
.46663 .69389 .47593 .70799 .49246 .70659 .48319 .69316 Metetra
.505 .625 .904 r
.48319 .69316 .49246 .70659 .50914 .70587 .49989 .69313 Metetra
.481 .628 .917 r
.49989 .69313 .50914 .70587 .52596 .70567 .51676 .6936 Metetra
.459 .633 .929 r
.51676 .6936 .52596 .70567 .54295 .70583 .53379 .69442 Metetra
.442 .639 .94 r
.53379 .69442 .54295 .70583 .56011 .70621 .551 .69541 Metetra
.429 .648 .95 r
.551 .69541 .56011 .70621 .57744 .70668 .56839 .69645 Metetra
.421 .658 .958 r
.56839 .69645 .57744 .70668 .59495 .70713 .58597 .69741 Metetra
.417 .67 .966 r
.58597 .69741 .59495 .70713 .61263 .70748 .60372 .6982 Metetra
.418 .683 .972 r
.60372 .6982 .61263 .70748 .63048 .70764 .62166 .69874 Metetra
.423 .696 .977 r
.62166 .69874 .63048 .70764 .64851 .70755 .63976 .69896 Metetra
.431 .709 .982 r
.63976 .69896 .64851 .70755 .6667 .70717 .65804 .69882 Metetra
.442 .722 .985 r
.65804 .69882 .6667 .70717 .68505 .70647 .67649 .69828 Metetra
.454 .735 .987 r
.67649 .69828 .68505 .70647 .70356 .70541 .69509 .69734 Metetra
.468 .746 .989 r
.69509 .69734 .70356 .70541 .72222 .70399 .71385 .69597 Metetra
.483 .756 .99 r
.71385 .69597 .72222 .70399 .74104 .70221 .73276 .6942 Metetra
.498 .765 .991 r
.73276 .6942 .74104 .70221 .76 .70007 .75182 .69202 Metetra
.512 .772 .99 r
.75182 .69202 .76 .70007 .77911 .69757 .77103 .68945 Metetra
.525 .778 .99 r
.77103 .68945 .77911 .69757 .79835 .69474 .79037 .68652 Metetra
.538 .783 .989 r
.79037 .68652 .79835 .69474 .81774 .69158 .80986 .68324 Metetra
.574 .71 .935 r
.16268 .62518 .17265 .63469 .18643 .63233 .17648 .62288 Metetra
.568 .711 .938 r
.17648 .62288 .18643 .63233 .2003 .63004 .19037 .6207 Metetra
.564 .713 .941 r
.19037 .6207 .2003 .63004 .21427 .62778 .20436 .61858 Metetra
.562 .717 .945 r
.20436 .61858 .21427 .62778 .22834 .62547 .21845 .61646 Metetra
.562 .722 .948 r
.21845 .61646 .22834 .62547 .24253 .62307 .23265 .61426 Metetra
.565 .73 .952 r
.23265 .61426 .24253 .62307 .25684 .62051 .24697 .61192 Metetra
.571 .738 .955 r
.24697 .61192 .25684 .62051 .27126 .61773 .26142 .60937 Metetra
.579 .748 .957 r
.26142 .60937 .27126 .61773 .28581 .61467 .276 .60653 Metetra
.59 .758 .959 r
.276 .60653 .28581 .61467 .30049 .61131 .29071 .60336 Metetra
.602 .768 .959 r
.29071 .60336 .30049 .61131 .31529 .60758 .30554 .5998 Metetra
.615 .778 .959 r
.30554 .5998 .31529 .60758 .33021 .60348 .32051 .59581 Metetra
.629 .786 .957 r
.32051 .59581 .33021 .60348 .34525 .59899 .33561 .59139 Metetra
.642 .793 .955 r
.33561 .59139 .34525 .59899 .3604 .59409 .35082 .58651 Metetra
.655 .798 .951 r
.35082 .58651 .3604 .59409 .37567 .58882 .36615 .58119 Metetra
.954 .635 .383 r
.36615 .58119 .37567 .58882 .38528 .71525 .37586 .69794 Metetra
.667 .637 .811 r
.37586 .69794 .38528 .71525 .40145 .70791 .39205 .6909 Metetra
.636 .628 .825 r
.39205 .6909 .40145 .70791 .41764 .70231 .40825 .68575 Metetra
.604 .62 .84 r
.40825 .68575 .41764 .70231 .43388 .69825 .42449 .68227 Metetra
.571 .613 .856 r
.42449 .68227 .43388 .69825 .4502 .69552 .44082 .68021 Metetra
.539 .609 .871 r
.44082 .68021 .4502 .69552 .46663 .69389 .45726 .67932 Metetra
.508 .607 .887 r
.45726 .67932 .46663 .69389 .48319 .69316 .47383 .67938 Metetra
.478 .608 .902 r
.47383 .67938 .48319 .69316 .49989 .69313 .49056 .68015 Metetra
.451 .611 .916 r
.49056 .68015 .49989 .69313 .51676 .6936 .50745 .68142 Metetra
.427 .616 .929 r
.50745 .68142 .51676 .6936 .53379 .69442 .52453 .68299 Metetra
.408 .624 .941 r
.52453 .68299 .53379 .69442 .551 .69541 .54179 .68469 Metetra
.395 .634 .951 r
.54179 .68469 .551 .69541 .56839 .69645 .55924 .68637 Metetra
.387 .647 .96 r
.55924 .68637 .56839 .69645 .58597 .69741 .57688 .6879 Metetra
.385 .661 .967 r
.57688 .6879 .58597 .69741 .60372 .6982 .59471 .68917 Metetra
.388 .676 .974 r
.59471 .68917 .60372 .6982 .62166 .69874 .61272 .6901 Metetra
.396 .692 .979 r
.61272 .6901 .62166 .69874 .63976 .69896 .63092 .69063 Metetra
.408 .707 .984 r
.63092 .69063 .63976 .69896 .65804 .69882 .64929 .69072 Metetra
.423 .722 .987 r
.64929 .69072 .65804 .69882 .67649 .69828 .66782 .69034 Metetra
.44 .736 .99 r
.66782 .69034 .67649 .69828 .69509 .69734 .68652 .68948 Metetra
.458 .749 .992 r
.68652 .68948 .69509 .69734 .71385 .69597 .70538 .68815 Metetra
.476 .76 .993 r
.70538 .68815 .71385 .69597 .73276 .6942 .72439 .68634 Metetra
.493 .769 .993 r
.72439 .68634 .73276 .6942 .75182 .69202 .74355 .68409 Metetra
.51 .777 .992 r
.74355 .68409 .75182 .69202 .77103 .68945 .76285 .68142 Metetra
.525 .783 .991 r
.76285 .68142 .77103 .68945 .79037 .68652 .78229 .67835 Metetra
.539 .788 .99 r
.78229 .67835 .79037 .68652 .80986 .68324 .80187 .67491 Metetra
.572 .705 .932 r
.15263 .61541 .16268 .62518 .17648 .62288 .16645 .61316 Metetra
.564 .705 .936 r
.16645 .61316 .17648 .62288 .19037 .6207 .18036 .61107 Metetra
.557 .706 .94 r
.18036 .61107 .19037 .6207 .20436 .61858 .19436 .60911 Metetra
.553 .709 .944 r
.19436 .60911 .20436 .61858 .21845 .61646 .20846 .60719 Metetra
.552 .715 .948 r
.20846 .60719 .21845 .61646 .23265 .61426 .22267 .60524 Metetra
.553 .722 .952 r
.22267 .60524 .23265 .61426 .24697 .61192 .23701 .60317 Metetra
.558 .732 .956 r
.23701 .60317 .24697 .61192 .26142 .60937 .25147 .60089 Metetra
.566 .743 .96 r
.25147 .60089 .26142 .60937 .276 .60653 .26607 .59833 Metetra
.577 .755 .962 r
.26607 .59833 .276 .60653 .29071 .60336 .2808 .59541 Metetra
.591 .767 .963 r
.2808 .59541 .29071 .60336 .30554 .5998 .29568 .59206 Metetra
.606 .779 .963 r
.29568 .59206 .30554 .5998 .32051 .59581 .31069 .58824 Metetra
.622 .789 .962 r
.31069 .58824 .32051 .59581 .33561 .59139 .32583 .58392 Metetra
.638 .797 .959 r
.32583 .58392 .33561 .59139 .35082 .58651 .34111 .57908 Metetra
.654 .804 .955 r
.34111 .57908 .35082 .58651 .36615 .58119 .3565 .57373 Metetra
.953 .629 .373 r
.3565 .57373 .36615 .58119 .37586 .69794 .36643 .67957 Metetra
.657 .622 .803 r
.36643 .67957 .37586 .69794 .39205 .6909 .38262 .67288 Metetra
.623 .611 .818 r
.38262 .67288 .39205 .6909 .40825 .68575 .39882 .66828 Metetra
.587 .601 .834 r
.39882 .66828 .40825 .68575 .42449 .68227 .41506 .66549 Metetra
.551 .594 .85 r
.41506 .66549 .42449 .68227 .44082 .68021 .43138 .66424 Metetra
.514 .588 .867 r
.43138 .66424 .44082 .68021 .45726 .67932 .44781 .66425 Metetra
.478 .586 .883 r
.44781 .66425 .45726 .67932 .47383 .67938 .46439 .66525 Metetra
.444 .586 .9 r
.46439 .66525 .47383 .67938 .49056 .68015 .48113 .66696 Metetra
.413 .589 .915 r
.48113 .66696 .49056 .68015 .50745 .68142 .49805 .66916 Metetra
.387 .596 .928 r
.49805 .66916 .50745 .68142 .52453 .68299 .51516 .67161 Metetra
.366 .605 .94 r
.51516 .67161 .52453 .68299 .54179 .68469 .53247 .67412 Metetra
.352 .618 .951 r
.53247 .67412 .54179 .68469 .55924 .68637 .54998 .67653 Metetra
.345 .633 .96 r
.54998 .67653 .55924 .68637 .57688 .6879 .56769 .67868 Metetra
.346 .65 .968 r
.56769 .67868 .57688 .6879 .59471 .68917 .58559 .68047 Metetra
.353 .668 .974 r
.58559 .68047 .59471 .68917 .61272 .6901 .60369 .68182 Metetra
.365 .687 .98 r
.60369 .68182 .61272 .6901 .63092 .69063 .62197 .68267 Metetra
.383 .705 .985 r
.62197 .68267 .63092 .69063 .64929 .69072 .64043 .68297 Metetra
.403 .723 .989 r
.64043 .68297 .64929 .69072 .66782 .69034 .65906 .68272 Metetra
.425 .739 .992 r
.65906 .68272 .66782 .69034 .68652 .68948 .67785 .6819 Metetra
.447 .753 .994 r
.67785 .6819 .68652 .68948 .70538 .68815 .69681 .68054 Metetra
.469 .765 .995 r
.69681 .68054 .70538 .68815 .72439 .68634 .71591 .67866 Metetra
.49 .775 .995 r
.71591 .67866 .72439 .68634 .74355 .68409 .73516 .67629 Metetra
.509 .783 .994 r
.73516 .67629 .74355 .68409 .76285 .68142 .75456 .67345 Metetra
.527 .789 .993 r
.75456 .67345 .76285 .68142 .78229 .67835 .7741 .6702 Metetra
.542 .793 .991 r
.7741 .6702 .78229 .67835 .80187 .67491 .79378 .66656 Metetra
.57 .7 .93 r
.1425 .60535 .15263 .61541 .16645 .61316 .15635 .60311 Metetra
.56 .698 .933 r
.15635 .60311 .16645 .61316 .18036 .61107 .17027 .60111 Metetra
.551 .698 .937 r
.17027 .60111 .18036 .61107 .19436 .60911 .18428 .5993 Metetra
.544 .7 .942 r
.18428 .5993 .19436 .60911 .20846 .60719 .19839 .59761 Metetra
.54 .705 .947 r
.19839 .59761 .20846 .60719 .22267 .60524 .2126 .59594 Metetra
.539 .713 .952 r
.2126 .59594 .22267 .60524 .23701 .60317 .22694 .59419 Metetra
.542 .723 .957 r
.22694 .59419 .23701 .60317 .25147 .60089 .24141 .59226 Metetra
.55 .736 .962 r
.24141 .59226 .25147 .60089 .26607 .59833 .25602 .59005 Metetra
.561 .75 .966 r
.25602 .59005 .26607 .59833 .2808 .59541 .27077 .58746 Metetra
.576 .765 .968 r
.27077 .58746 .2808 .59541 .29568 .59206 .28567 .5844 Metetra
.594 .779 .969 r
.28567 .5844 .29568 .59206 .31069 .58824 .30072 .5808 Metetra
.613 .792 .967 r
.30072 .5808 .31069 .58824 .32583 .58392 .31592 .57663 Metetra
.632 .803 .965 r
.31592 .57663 .32583 .58392 .34111 .57908 .33125 .57186 Metetra
.651 .811 .96 r
.33125 .57186 .34111 .57908 .3565 .57373 .34671 .56649 Metetra
.95 .619 .359 r
.34671 .56649 .3565 .57373 .36643 .67957 .357 .65992 Metetra
.645 .604 .794 r
.357 .65992 .36643 .67957 .38262 .67288 .37319 .65366 Metetra
.607 .591 .809 r
.37319 .65366 .38262 .67288 .39882 .66828 .38937 .64972 Metetra
.567 .579 .825 r
.38937 .64972 .39882 .66828 .41506 .66549 .40558 .64778 Metetra
.526 .57 .842 r
.40558 .64778 .41506 .66549 .43138 .66424 .42188 .64752 Metetra
.484 .563 .86 r
.42188 .64752 .43138 .66424 .44781 .66425 .4383 .64862 Metetra
.442 .559 .878 r
.4383 .64862 .44781 .66425 .46439 .66525 .45487 .65076 Metetra
.402 .559 .896 r
.45487 .65076 .46439 .66525 .48113 .66696 .47161 .65362 Metetra
.366 .563 .911 r
.47161 .65362 .48113 .66696 .49805 .66916 .48855 .65692 Metetra
.335 .57 .926 r
.48855 .65692 .49805 .66916 .51516 .67161 .50569 .6604 Metetra
.312 .581 .938 r
.50569 .6604 .51516 .67161 .53247 .67412 .52304 .66385 Metetra
.297 .596 .948 r
.52304 .66385 .53247 .67412 .54998 .67653 .5406 .66707 Metetra
.292 .615 .957 r
.5406 .66707 .54998 .67653 .56769 .67868 .55838 .66991 Metetra
.297 .636 .965 r
.55838 .66991 .56769 .67868 .58559 .68047 .57636 .67226 Metetra
.309 .658 .972 r
.57636 .67226 .58559 .68047 .60369 .68182 .59454 .67403 Metetra
.329 .681 .978 r
.59454 .67403 .60369 .68182 .62197 .68267 .61291 .67516 Metetra
.353 .704 .984 r
.61291 .67516 .62197 .68267 .64043 .68297 .63146 .67564 Metetra
.381 .725 .989 r
.63146 .67564 .64043 .68297 .65906 .68272 .65018 .67547 Metetra
.409 .743 .993 r
.65018 .67547 .65906 .68272 .67785 .6819 .66908 .67464 Metetra
.438 .759 .996 r
.66908 .67464 .67785 .6819 .69681 .68054 .68812 .67319 Metetra
.465 .772 .997 r
.68812 .67319 .69681 .68054 .71591 .67866 .70733 .67117 Metetra
.489 .782 .997 r
.70733 .67117 .71591 .67866 .73516 .67629 .72667 .6686 Metetra
.511 .79 .996 r
.72667 .6686 .73516 .67629 .75456 .67345 .74617 .66555 Metetra
.53 .795 .994 r
.74617 .66555 .75456 .67345 .7741 .6702 .7658 .66205 Metetra
.547 .798 .991 r
.7658 .66205 .7741 .6702 .79378 .66656 .78557 .65816 Metetra
.571 .696 .926 r
.13229 .59497 .1425 .60535 .15635 .60311 .14617 .5927 Metetra
.558 .691 .929 r
.14617 .5927 .15635 .60311 .17027 .60111 .16011 .59075 Metetra
.545 .689 .933 r
.16011 .59075 .17027 .60111 .18428 .5993 .17413 .58909 Metetra
.535 .689 .938 r
.17413 .58909 .18428 .5993 .19839 .59761 .18824 .58763 Metetra
.527 .693 .944 r
.18824 .58763 .19839 .59761 .2126 .59594 .20245 .58628 Metetra
.523 .7 .951 r
.20245 .58628 .2126 .59594 .22694 .59419 .21678 .58492 Metetra
.523 .711 .957 r
.21678 .58492 .22694 .59419 .24141 .59226 .23124 .58343 Metetra
.529 .725 .964 r
.23124 .58343 .24141 .59226 .25602 .59005 .24585 .58166 Metetra
.54 .742 .969 r
.24585 .58166 .25602 .59005 .27077 .58746 .26061 .5795 Metetra
.556 .76 .973 r
.26061 .5795 .27077 .58746 .28567 .5844 .27553 .57683 Metetra
.577 .778 .975 r
.27553 .57683 .28567 .5844 .30072 .5808 .29061 .57356 Metetra
.6 .795 .974 r
.29061 .57356 .30072 .5808 .31592 .57663 .30585 .56961 Metetra
.623 .809 .971 r
.30585 .56961 .31592 .57663 .33125 .57186 .32124 .56496 Metetra
.647 .82 .966 r
.32124 .56496 .33125 .57186 .34671 .56649 .33677 .55959 Metetra
.946 .604 .338 r
.33677 .55959 .34671 .56649 .357 .65992 .34761 .63869 Metetra
.631 .583 .782 r
.34761 .63869 .357 .65992 .37319 .65366 .36378 .63296 Metetra
.588 .567 .797 r
.36378 .63296 .37319 .65366 .38937 .64972 .37992 .62984 Metetra
.543 .552 .814 r
.37992 .62984 .38937 .64972 .40558 .64778 .39609 .62896 Metetra
.495 .54 .832 r
.39609 .62896 .40558 .64778 .42188 .64752 .41234 .62995 Metetra
.445 .531 .852 r
.41234 .62995 .42188 .64752 .4383 .64862 .42872 .63241 Metetra
.395 .526 .871 r
.42872 .63241 .4383 .64862 .45487 .65076 .44526 .63594 Metetra
.347 .524 .889 r
.44526 .63594 .45487 .65076 .47161 .65362 .462 .64019 Metetra
.303 .528 .905 r
.462 .64019 .47161 .65362 .48855 .65692 .47894 .64482 Metetra
.266 .536 .918 r
.47894 .64482 .48855 .65692 .50569 .6604 .4961 .64953 Metetra
.239 .549 .93 r
.4961 .64953 .50569 .6604 .52304 .66385 .51349 .65405 Metetra
.225 .568 .939 r
.51349 .65405 .52304 .66385 .5406 .66707 .53111 .65819 Metetra
.224 .59 .947 r
.53111 .65819 .5406 .66707 .55838 .66991 .54895 .66178 Metetra
.235 .617 .955 r
.54895 .66178 .55838 .66991 .57636 .67226 .56701 .6647 Metetra
.256 .645 .964 r
.56701 .6647 .57636 .67226 .59454 .67403 .58527 .66687 Metetra
.286 .674 .972 r
.58527 .66687 .59454 .67403 .61291 .67516 .60373 .66826 Metetra
.321 .702 .98 r
.60373 .66826 .61291 .67516 .63146 .67564 .62238 .66885 Metetra
.358 .728 .988 r
.62238 .66885 .63146 .67564 .65018 .67547 .6412 .66866 Metetra
.396 .749 .993 r
.6412 .66866 .65018 .67547 .66908 .67464 .66019 .66773 Metetra
.431 .767 .997 r
.66019 .66773 .66908 .67464 .68812 .67319 .67933 .66611 Metetra
.463 .781 .998 r
.67933 .66611 .68812 .67319 .70733 .67117 .69863 .66385 Metetra
.492 .791 .998 r
.69863 .66385 .70733 .67117 .72667 .6686 .71807 .66101 Metetra
.517 .797 .997 r
.71807 .66101 .72667 .6686 .74617 .66555 .73766 .65766 Metetra
.537 .802 .994 r
.73766 .65766 .74617 .66555 .7658 .66205 .75738 .65385 Metetra
.555 .804 .991 r
.75738 .65385 .7658 .66205 .78557 .65816 .77725 .64966 Metetra
.574 .692 .921 r
.12202 .58426 .13229 .59497 .14617 .5927 .13593 .58188 Metetra
.558 .685 .924 r
.13593 .58188 .14617 .5927 .16011 .59075 .1499 .57993 Metetra
.541 .679 .928 r
.1499 .57993 .16011 .59075 .17413 .58909 .16392 .57837 Metetra
.526 .677 .933 r
.16392 .57837 .17413 .58909 .18824 .58763 .17803 .57714 Metetra
.513 .678 .94 r
.17803 .57714 .18824 .58763 .20245 .58628 .19223 .57614 Metetra
.504 .683 .947 r
.19223 .57614 .20245 .58628 .21678 .58492 .20654 .57524 Metetra
.5 .694 .955 r
.20654 .57524 .21678 .58492 .23124 .58343 .22098 .57428 Metetra
.502 .709 .964 r
.22098 .57428 .23124 .58343 .24585 .58166 .23557 .5731 Metetra
.512 .729 .972 r
.23557 .5731 .24585 .58166 .26061 .5795 .25032 .57153 Metetra
.528 .752 .978 r
.25032 .57153 .26061 .5795 .27553 .57683 .26524 .5694 Metetra
.551 .775 .982 r
.26524 .5694 .27553 .57683 .29061 .57356 .28034 .56658 Metetra
.579 .798 .982 r
.28034 .56658 .29061 .57356 .30585 .56961 .29561 .56297 Metetra
.609 .817 .98 r
.29561 .56297 .30585 .56961 .32124 .56496 .31105 .5585 Metetra
.639 .832 .974 r
.31105 .5585 .32124 .56496 .33677 .55959 .32665 .55317 Metetra
.937 .576 .3 r
.32665 .55317 .33677 .55959 .34761 .63869 .33829 .61545 Metetra
.615 .558 .767 r
.33829 .61545 .34761 .63869 .36378 .63296 .35441 .61039 Metetra
.566 .538 .783 r
.35441 .61039 .36378 .63296 .37992 .62984 .37049 .60833 Metetra
.513 .519 .8 r
.37049 .60833 .37992 .62984 .39609 .62896 .38659 .60882 Metetra
.456 .503 .819 r
.38659 .60882 .39609 .62896 .41234 .62995 .40277 .61139 Metetra
.395 .491 .839 r
.40277 .61139 .41234 .62995 .42872 .63241 .41908 .61557 Metetra
.333 .482 .859 r
.41908 .61557 .42872 .63241 .44526 .63594 .43558 .62087 Metetra
.271 .478 .876 r
.43558 .62087 .44526 .63594 .462 .64019 .45228 .62684 Metetra
.215 .479 .891 r
.45228 .62684 .462 .64019 .47894 .64482 .46921 .63309 Metetra
.17 .488 .901 r
.46921 .63309 .47894 .64482 .4961 .64953 .48639 .63924 Metetra
.139 .503 .909 r
.48639 .63924 .4961 .64953 .51349 .65405 .50382 .64502 Metetra
.127 .526 .916 r
.50382 .64502 .51349 .65405 .53111 .65819 .52149 .65018 Metetra
.133 .556 .923 r
.52149 .65018 .53111 .65819 .54895 .66178 .5394 .65455 Metetra
.155 .59 .933 r
.5394 .65455 .54895 .66178 .56701 .6647 .55753 .65803 Metetra
.191 .628 .945 r
.55753 .65803 .56701 .6647 .58527 .66687 .57589 .66055 Metetra
.236 .666 .959 r
.57589 .66055 .58527 .66687 .60373 .66826 .59444 .66209 Metetra
.286 .702 .972 r
.59444 .66209 .60373 .66826 .62238 .66885 .61318 .66268 Metetra
.337 .733 .983 r
.61318 .66268 .62238 .66885 .6412 .66866 .6321 .66236 Metetra
.386 .758 .992 r
.6321 .66236 .6412 .66866 .66019 .66773 .65118 .6612 Metetra
.43 .778 .997 r
.65118 .6612 .66019 .66773 .67933 .66611 .67042 .65927 Metetra
.468 .792 .999 r
.67042 .65927 .67933 .66611 .69863 .66385 .68981 .65666 Metetra
.5 .801 .999 r
.68981 .65666 .69863 .66385 .71807 .66101 .70934 .65344 Metetra
.527 .806 .997 r
.70934 .65344 .71807 .66101 .73766 .65766 .72902 .64971 Metetra
.548 .808 .994 r
.72902 .64971 .73766 .65766 .75738 .65385 .74884 .64553 Metetra
.565 .808 .99 r
.74884 .64553 .75738 .65385 .77725 .64966 .7688 .64097 Metetra
.581 .69 .916 r
.11167 .57323 .12202 .58426 .13593 .58188 .12563 .57063 Metetra
.562 .679 .918 r
.12563 .57063 .13593 .58188 .1499 .57993 .13963 .56855 Metetra
.541 .67 .921 r
.13963 .56855 .1499 .57993 .16392 .57837 .15368 .56701 Metetra
.521 .663 .925 r
.15368 .56701 .16392 .57837 .17803 .57714 .16778 .56597 Metetra
.501 .66 .932 r
.16778 .56597 .17803 .57714 .19223 .57614 .18196 .56533 Metetra
.484 .662 .94 r
.18196 .56533 .19223 .57614 .20654 .57524 .19624 .56496 Metetra
.472 .67 .95 r
.19624 .56496 .20654 .57524 .22098 .57428 .21064 .56467 Metetra
.467 .685 .961 r
.21064 .56467 .22098 .57428 .23557 .5731 .22518 .56425 Metetra
.472 .708 .972 r
.22518 .56425 .23557 .5731 .25032 .57153 .2399 .56348 Metetra
.487 .736 .982 r
.2399 .56348 .25032 .57153 .26524 .5694 .2548 .56212 Metetra
.512 .767 .988 r
.2548 .56212 .26524 .5694 .28034 .56658 .26989 .55997 Metetra
.547 .798 .991 r
.26989 .55997 .28034 .56658 .29561 .56297 .28519 .55687 Metetra
.586 .826 .99 r
.28519 .55687 .29561 .56297 .31105 .5585 .30068 .55271 Metetra
.626 .847 .984 r
.30068 .55271 .31105 .5585 .32665 .55317 .31635 .54747 Metetra
.91 .51 .215 r
.31635 .54747 .32665 .55317 .33829 .61545 .32911 .58958 Metetra
.596 .528 .749 r
.32911 .58958 .33829 .61545 .35441 .61039 .34515 .58539 Metetra
.538 .502 .764 r
.34515 .58539 .35441 .61039 .37049 .60833 .36113 .58473 Metetra
.475 .478 .781 r
.36113 .58473 .37049 .60833 .38659 .60882 .37711 .58703 Metetra
.404 .456 .8 r
.37711 .58703 .38659 .60882 .40277 .61139 .39318 .5917 Metetra
.327 .436 .82 r
.39318 .5917 .40277 .61139 .41908 .61557 .40939 .59812 Metetra
.245 .421 .839 r
.40939 .59812 .41908 .61557 .43558 .62087 .4258 .60569 Metetra
.163 .411 .853 r
.4258 .60569 .43558 .62087 .45228 .62684 .44245 .61385 Metetra
.089 .409 .86 r
.44245 .61385 .45228 .62684 .46921 .63309 .45936 .62209 Metetra
.032 .415 .862 r
.45936 .62209 .46921 .63309 .48639 .63924 .47655 .62998 Metetra
0 .433 .861 r
.47655 .62998 .48639 .63924 .50382 .64502 .494 .63718 Metetra
0 .463 .862 r
.494 .63718 .50382 .64502 .52149 .65018 .51173 .64345 Metetra
.013 .503 .87 r
.51173 .64345 .52149 .65018 .5394 .65455 .52971 .64861 Metetra
.055 .551 .886 r
.52971 .64861 .5394 .65455 .55753 .65803 .54793 .65256 Metetra
.114 .604 .908 r
.54793 .65256 .55753 .65803 .57589 .66055 .56637 .65529 Metetra
.182 .656 .933 r
.56637 .65529 .57589 .66055 .59444 .66209 .58502 .65682 Metetra
.253 .703 .956 r
.58502 .65682 .59444 .66209 .61318 .66268 .60386 .65722 Metetra
.322 .742 .976 r
.60386 .65722 .61318 .66268 .6321 .66236 .62287 .65658 Metetra
.384 .772 .989 r
.62287 .65658 .6321 .66236 .65118 .6612 .64205 .655 Metetra
.437 .792 .997 r
.64205 .655 .65118 .6612 .67042 .65927 .66138 .6526 Metetra
.481 .805 1 r
.66138 .6526 .67042 .65927 .68981 .65666 .68086 .6495 Metetra
.515 .811 .999 r
.68086 .6495 .68981 .65666 .70934 .65344 .70049 .64579 Metetra
.542 .814 .996 r
.70049 .64579 .70934 .65344 .72902 .64971 .72025 .64159 Metetra
.563 .813 .992 r
.72025 .64159 .72902 .64971 .74884 .64553 .74016 .63696 Metetra
.578 .812 .988 r
.74016 .63696 .74884 .64553 .7688 .64097 .7602 .632 Metetra
.593 .691 .911 r
.10123 .56197 .11167 .57323 .12563 .57063 .11527 .55897 Metetra
.572 .677 .911 r
.11527 .55897 .12563 .57063 .13963 .56855 .12933 .55659 Metetra
.548 .662 .912 r
.12933 .55659 .13963 .56855 .15368 .56701 .14341 .5549 Metetra
.521 .649 .915 r
.14341 .5549 .15368 .56701 .16778 .56597 .15752 .55391 Metetra
.493 .639 .92 r
.15752 .55391 .16778 .56597 .18196 .56533 .17168 .55358 Metetra
.465 .635 .928 r
.17168 .55358 .18196 .56533 .19624 .56496 .18591 .55377 Metetra
.441 .637 .939 r
.18591 .55377 .19624 .56496 .21064 .56467 .20025 .55429 Metetra
.423 .649 .952 r
.20025 .55429 .21064 .56467 .22518 .56425 .21472 .55488 Metetra
.416 .672 .967 r
.21472 .55488 .22518 .56425 .2399 .56348 .22936 .55522 Metetra
.423 .705 .981 r
.22936 .55522 .2399 .56348 .2548 .56212 .2442 .55498 Metetra
.448 .746 .992 r
.2442 .55498 .2548 .56212 .26989 .55997 .25926 .55385 Metetra
.489 .79 .998 r
.25926 .55385 .26989 .55997 .28519 .55687 .27455 .55156 Metetra
.542 .832 .999 r
.27455 .55156 .28519 .55687 .30068 .55271 .29007 .54794 Metetra
.599 .865 .993 r
.29007 .54794 .30068 .55271 .31635 .54747 .3058 .54289 Metetra
.733 .207 0 r
.3058 .54289 .31635 .54747 .32911 .58958 .32015 .56011 Metetra
.572 .492 .726 r
.32015 .56011 .32911 .58958 .34515 .58539 .33607 .5571 Metetra
.503 .458 .739 r
.33607 .5571 .34515 .58539 .36113 .58473 .35189 .55837 Metetra
.425 .425 .755 r
.35189 .55837 .36113 .58473 .37711 .58703 .36769 .56317 Metetra
.334 .392 .773 r
.36769 .56317 .37711 .58703 .39318 .5917 .38358 .57071 Metetra
.23 .361 .79 r
.38358 .57071 .39318 .5917 .40939 .59812 .39964 .58018 Metetra
.115 .332 .801 r
.39964 .58018 .40939 .59812 .4258 .60569 .41593 .59077 Metetra
0 .309 .801 r
.41593 .59077 .4258 .60569 .44245 .61385 .43249 .60174 Metetra
0 .297 .788 r
.43249 .60174 .44245 .61385 .45936 .62209 .44936 .61247 Metetra
0 .3 .767 r
.44936 .61247 .45936 .62209 .47655 .62998 .46653 .62243 Metetra
0 .323 .75 r
.46653 .62243 .47655 .62998 .494 .63718 .48402 .63122 Metetra
0 .364 .748 r
.48402 .63122 .494 .63718 .51173 .64345 .5018 .63861 Metetra
0 .423 .765 r
.5018 .63861 .51173 .64345 .52971 .64861 .51986 .64443 Metetra
0 .495 .799 r
.51986 .64443 .52971 .64861 .54793 .65256 .53817 .64866 Metetra
.031 .572 .845 r
.53817 .64866 .54793 .65256 .56637 .65529 .55671 .65133 Metetra
.132 .646 .893 r
.55671 .65133 .56637 .65529 .58502 .65682 .57546 .65256 Metetra
.232 .71 .936 r
.57546 .65256 .58502 .65682 .60386 .65722 .5944 .65247 Metetra
.322 .758 .968 r
.5944 .65247 .60386 .65722 .62287 .65658 .61351 .65122 Metetra
.398 .791 .988 r
.61351 .65122 .62287 .65658 .64205 .655 .63278 .64899 Metetra
.458 .81 .998 r
.63278 .64899 .64205 .655 .66138 .6526 .6522 .64592 Metetra
.504 .818 1 r
.6522 .64592 .66138 .6526 .68086 .6495 .67177 .64217 Metetra
.538 .821 .998 r
.67177 .64217 .68086 .6495 .70049 .64579 .69148 .63786 Metetra
.562 .819 .994 r
.69148 .63786 .70049 .64579 .72025 .64159 .71133 .63311 Metetra
.58 .816 .989 r
.71133 .63311 .72025 .64159 .74016 .63696 .73133 .628 Metetra
.593 .812 .983 r
.73133 .628 .74016 .63696 .7602 .632 .75146 .62261 Metetra
.61 .699 .907 r
.09069 .55066 .10123 .56197 .11527 .55897 .10483 .54705 Metetra
.59 .68 .904 r
.10483 .54705 .11527 .55897 .12933 .55659 .11899 .5441 Metetra
.564 .66 .902 r
.11899 .5441 .12933 .55659 .14341 .5549 .13314 .54195 Metetra
.532 .639 .901 r
.13314 .54195 .14341 .5549 .15752 .55391 .14729 .54072 Metetra
.495 .619 .903 r
.14729 .54072 .15752 .55391 .17168 .55358 .16145 .54047 Metetra
.455 .603 .909 r
.16145 .54047 .17168 .55358 .18591 .55377 .17563 .54115 Metetra
.413 .594 .919 r
.17563 .54115 .18591 .55377 .20025 .55429 .18988 .54259 Metetra
.372 .595 .932 r
.18988 .54259 .20025 .55429 .21472 .55488 .20423 .54447 Metetra
.34 .611 .949 r
.20423 .54447 .21472 .55488 .22936 .55522 .21873 .5464 Metetra
.323 .643 .967 r
.21873 .5464 .22936 .55522 .2442 .55498 .23344 .54787 Metetra
.332 .693 .981 r
.23344 .54787 .2442 .55498 .25926 .55385 .2484 .54839 Metetra
.373 .755 .989 r
.2484 .54839 .25926 .55385 .27455 .55156 .26364 .54748 Metetra
.444 .821 .991 r
.26364 .54748 .27455 .55156 .29007 .54794 .27916 .5448 Metetra
.534 .878 .988 r
.27916 .5448 .29007 .54794 .3058 .54289 .29495 .54015 Metetra
.846 .931 .884 r
.29495 .54015 .3058 .54289 .32015 .56011 .31156 .52541 Metetra
.542 .447 .697 r
.31156 .52541 .32015 .56011 .33607 .5571 .3273 .52411 Metetra
.458 .402 .707 r
.3273 .52411 .33607 .5571 .35189 .55837 .34286 .52819 Metetra
.357 .354 .719 r
.34286 .52819 .35189 .55837 .36769 .56317 .35839 .53665 Metetra
.233 .303 .731 r
.35839 .53665 .36769 .56317 .38358 .57071 .374 .54836 Metetra
.082 .248 .736 r
.374 .54836 .38358 .57071 .39964 .58018 .38982 .56213 Metetra
0 .192 .722 r
.38982 .56213 .39964 .58018 .41593 .59077 .40592 .57687 Metetra
0 .144 .679 r
.40592 .57687 .41593 .59077 .43249 .60174 .42235 .59158 Metetra
0 .119 .617 r
.42235 .59158 .43249 .60174 .44936 .61247 .43915 .60541 Metetra
.425 0 0 r
.43915 .60541 .44936 .61247 .46653 .62243 .45631 .61776 Metetra
.419 0 0 r
.45631 .61776 .46653 .62243 .48402 .63122 .47383 .6282 Metetra
.368 0 0 r
.47383 .6282 .48402 .63122 .5018 .63861 .49168 .63651 Metetra
.284 0 0 r
.49168 .63651 .5018 .63861 .51986 .64443 .50983 .64264 Metetra
.171 0 0 r
.50983 .64264 .51986 .64443 .53817 .64866 .52825 .64669 Metetra
0 .54 .756 r
.52825 .64669 .53817 .64866 .55671 .65133 .54689 .64881 Metetra
.105 .647 .847 r
.54689 .64881 .55671 .65133 .57546 .65256 .56575 .64924 Metetra
.239 .731 .919 r
.56575 .64924 .57546 .65256 .5944 .65247 .58479 .64822 Metetra
.352 .786 .967 r
.58479 .64822 .5944 .65247 .61351 .65122 .60399 .64601 Metetra
.437 .816 .991 r
.60399 .64601 .61351 .65122 .63278 .64899 .62336 .64283 Metetra
.498 .828 .999 r
.62336 .64283 .63278 .64899 .6522 .64592 .64287 .63889 Metetra
.539 .829 .999 r
.64287 .63889 .6522 .64592 .67177 .64217 .66252 .63436 Metetra
.567 .825 .994 r
.66252 .63436 .67177 .64217 .69148 .63786 .68231 .62938 Metetra
.586 .819 .988 r
.68231 .62938 .69148 .63786 .71133 .63311 .70225 .62405 Metetra
.598 .813 .982 r
.70225 .62405 .71133 .63311 .73133 .628 .72234 .61846 Metetra
.607 .808 .977 r
.72234 .61846 .73133 .628 .75146 .62261 .74256 .61267 Metetra
.632 .714 .906 r
.07998 .53966 .09069 .55066 .10483 .54705 .09426 .53527 Metetra
.616 .694 .9 r
.09426 .53527 .10483 .54705 .11899 .5441 .10856 .53141 Metetra
.592 .67 .893 r
.10856 .53141 .11899 .5441 .13314 .54195 .12285 .52831 Metetra
.56 .64 .887 r
.12285 .52831 .13314 .54195 .14729 .54072 .13711 .52627 Metetra
.518 .609 .882 r
.13711 .52627 .14729 .54072 .16145 .54047 .15133 .52553 Metetra
.466 .576 .881 r
.15133 .52553 .16145 .54047 .17563 .54115 .16551 .52626 Metetra
.404 .546 .884 r
.16551 .52626 .17563 .54115 .18988 .54259 .17966 .52846 Metetra
.331 .523 .893 r
.17966 .52846 .18988 .54259 .20423 .54447 .19384 .53188 Metetra
.251 .513 .906 r
.19384 .53188 .20423 .54447 .21873 .5464 .20812 .53606 Metetra
.173 .522 .918 r
.20812 .53606 .21873 .5464 .23344 .54787 .22258 .54028 Metetra
.119 .557 .919 r
.22258 .54028 .23344 .54787 .2484 .54839 .23731 .54367 Metetra
.12 .622 .903 r
.23731 .54367 .2484 .54839 .26364 .54748 .25238 .54534 Metetra
.196 .712 .884 r
.25238 .54534 .26364 .54748 .27916 .5448 .26782 .54454 Metetra
.339 .817 .883 r
.26782 .54454 .27916 .5448 .29495 .54015 .28364 .54079 Metetra
.974 .811 .62 r
.28364 .54079 .29495 .54015 .31156 .52541 .30364 .48249 Metetra
.505 .393 .659 r
.30364 .48249 .31156 .52541 .3273 .52411 .31907 .48385 Metetra
.398 .329 .663 r
.31907 .48385 .3273 .52411 .34286 .52819 .33421 .49244 Metetra
.261 .257 .666 r
.33421 .49244 .34286 .52819 .35839 .53665 .34927 .50671 Metetra
.076 .17 .66 r
.34927 .50671 .35839 .53665 .374 .54836 .36444 .52488 Metetra
0 .066 .625 r
.36444 .52488 .374 .54836 .38982 .56213 .37987 .54515 Metetra
0 0 .534 r
.37987 .54515 .38982 .56213 .40592 .57687 .39567 .56585 Metetra
.591 .103 0 r
.39567 .56585 .40592 .57687 .42235 .59158 .41192 .58554 Metetra
.652 .111 0 r
.41192 .58554 .42235 .59158 .43915 .60541 .42863 .60316 Metetra
.636 .074 0 r
.42863 .60316 .43915 .60541 .45631 .61776 .44579 .618 Metetra
.579 .005 0 r
.44579 .618 .45631 .61776 .47383 .6282 .46337 .6297 Metetra
.493 0 0 r
.46337 .6297 .47383 .6282 .49168 .63651 .48132 .63824 Metetra
.378 0 0 r
.48132 .63824 .49168 .63651 .50983 .64264 .49959 .6438 Metetra
.229 0 0 r
.49959 .6438 .50983 .64264 .52825 .64669 .51813 .64674 Metetra
.048 0 0 r
.51813 .64674 .52825 .64669 .54689 .64881 .5369 .64745 Metetra
.142 .683 .826 r
.5369 .64745 .54689 .64881 .56575 .64924 .55587 .64636 Metetra
.306 .78 .928 r
.55587 .64636 .56575 .64924 .58479 .64822 .57501 .64386 Metetra
.426 .827 .98 r
.57501 .64386 .58479 .64822 .60399 .64601 .59431 .64029 Metetra
.503 .841 .997 r
.59431 .64029 .60399 .64601 .62336 .64283 .61376 .63593 Metetra
.551 .838 .998 r
.61376 .63593 .62336 .64283 .64287 .63889 .63335 .631 Metetra
.579 .83 .992 r
.63335 .631 .64287 .63889 .66252 .63436 .65309 .62567 Metetra
.596 .82 .985 r
.65309 .62567 .66252 .63436 .68231 .62938 .67297 .62004 Metetra
.607 .812 .979 r
.67297 .62004 .68231 .62938 .70225 .62405 .693 .61421 Metetra
.613 .805 .974 r
.693 .61421 .70225 .62405 .72234 .61846 .71318 .60823 Metetra
.617 .8 .97 r
.71318 .60823 .72234 .61846 .74256 .61267 .7335 .60214 Metetra
.65 .736 .912 r
.06904 .52935 .07998 .53966 .09426 .53527 .08345 .52429 Metetra
.643 .721 .905 r
.08345 .52429 .09426 .53527 .10856 .53141 .09792 .51941 Metetra
.63 .698 .894 r
.09792 .51941 .10856 .53141 .12285 .52831 .11242 .51493 Metetra
.608 .666 .881 r
.11242 .51493 .12285 .52831 .13711 .52627 .12692 .51123 Metetra
.572 .624 .866 r
.12692 .51123 .13711 .52627 .15133 .52553 .14136 .50877 Metetra
.52 .574 .851 r
.14136 .50877 .15133 .52553 .16551 .52626 .15568 .50815 Metetra
.447 .518 .838 r
.15568 .50815 .16551 .52626 .17966 .52846 .16986 .50992 Metetra
.347 .457 .83 r
.16986 .50992 .17966 .52846 .19384 .53188 .1839 .51437 Metetra
.209 .393 .826 r
.1839 .51437 .19384 .53188 .20812 .53606 .19785 .52135 Metetra
.018 .325 .81 r
.19785 .52135 .20812 .53606 .22258 .54028 .21184 .53005 Metetra
0 .26 .738 r
.21184 .53005 .22258 .54028 .23731 .54367 .22604 .53902 Metetra
.362 0 0 r
.22604 .53902 .23731 .54367 .25238 .54534 .24063 .54635 Metetra
.329 0 0 r
.24063 .54635 .25238 .54534 .26782 .54454 .25576 .55009 Metetra
.153 0 0 r
.25576 .55009 .26782 .54454 .28364 .54079 .27151 .54876 Metetra
.982 .766 .538 r
.27151 .54876 .28364 .54079 .30364 .48249 .29696 .42501 Metetra
.459 .325 .611 r
.29696 .42501 .30364 .48249 .31907 .48385 .31182 .43115 Metetra
.315 .233 .602 r
.31182 .43115 .31907 .48385 .33421 .49244 .32619 .44797 Metetra
.11 .114 .581 r
.32619 .44797 .33421 .49244 .34927 .50671 .34041 .47266 Metetra
0 0 .519 r
.34041 .47266 .34927 .50671 .36444 .52488 .35479 .50201 Metetra
0 0 .361 r
.35479 .50201 .36444 .52488 .37987 .54515 .36959 .53285 Metetra
.784 .331 0 r
.36959 .53285 .37987 .54515 .39567 .56585 .38495 .56241 Metetra
.81 .324 .003 r
.38495 .56241 .39567 .56585 .41192 .58554 .40096 .58859 Metetra
.76 .269 .065 r
.40096 .58859 .41192 .58554 .42863 .60316 .4176 .6101 Metetra
.688 .194 .068 r
.4176 .6101 .42863 .60316 .44579 .618 .43482 .62642 Metetra
.605 .099 .024 r
.43482 .62642 .44579 .618 .46337 .6297 .45254 .6377 Metetra
.504 0 0 r
.45254 .6377 .46337 .6297 .48132 .63824 .47066 .6445 Metetra
.367 0 0 r
.47066 .6445 .48132 .63824 .49959 .6438 .48911 .64759 Metetra
.173 0 0 r
.48911 .64759 .49959 .6438 .51813 .64674 .5078 .6478 Metetra
.5078 .6478 .51813 .64674 .5369 .64745 .52671 .64589 Metetra
.299 .788 .89 r
.52671 .64589 .5369 .64745 .55587 .64636 .5458 .64248 Metetra
.452 .849 .976 r
.5458 .64248 .55587 .64636 .57501 .64386 .56504 .63807 Metetra
.535 .854 .997 r
.56504 .63807 .57501 .64386 .59431 .64029 .58443 .63299 Metetra
.577 .842 .995 r
.58443 .63299 .59431 .64029 .61376 .63593 .60397 .62749 Metetra
.599 .827 .986 r
.60397 .62749 .61376 .63593 .63335 .631 .62365 .62172 Metetra
.61 .814 .978 r
.62365 .62172 .63335 .631 .65309 .62567 .64348 .61578 Metetra
.616 .805 .972 r
.64348 .61578 .65309 .62567 .67297 .62004 .66346 .60973 Metetra
.619 .799 .968 r
.66346 .60973 .67297 .62004 .693 .61421 .68358 .60359 Metetra
.621 .795 .965 r
.68358 .60359 .693 .61421 .71318 .60823 .70386 .59738 Metetra
.622 .792 .963 r
.70386 .59738 .71318 .60823 .7335 .60214 .72429 .59112 Metetra
.655 .75 .92 r
.05788 .51958 .06904 .52935 .08345 .52429 .07236 .51429 Metetra
.655 .747 .918 r
.07236 .51429 .08345 .52429 .09792 .51941 .08693 .50897 Metetra
.654 .739 .912 r
.08693 .50897 .09792 .51941 .11242 .51493 .10159 .50364 Metetra
.651 .722 .901 r
.10159 .50364 .11242 .51493 .12692 .51123 .11633 .49834 Metetra
.642 .691 .88 r
.11633 .49834 .12692 .51123 .14136 .50877 .13113 .49326 Metetra
.619 .64 .85 r
.13113 .49326 .14136 .50877 .15568 .50815 .14594 .48886 Metetra
.575 .571 .812 r
.14594 .48886 .15568 .50815 .16986 .50992 .16064 .48612 Metetra
.498 .484 .772 r
.16064 .48612 .16986 .50992 .1839 .51437 .17506 .48674 Metetra
.37 .374 .733 r
.17506 .48674 .1839 .51437 .19785 .52135 .18902 .49289 Metetra
.145 .221 .685 r
.18902 .49289 .19785 .52135 .21184 .53005 .20241 .50627 Metetra
0 0 .549 r
.20241 .50627 .21184 .53005 .22604 .53902 .21537 .52661 Metetra
.774 .313 0 r
.21537 .52661 .22604 .53902 .24063 .54635 .22836 .5503 Metetra
.77 .309 .152 r
.22836 .5503 .24063 .54635 .25576 .55009 .24207 .57052 Metetra
.579 .169 .229 r
.24207 .57052 .25576 .55009 .27151 .54876 .25712 .5796 Metetra
.987 .752 .49 r
.25712 .5796 .27151 .54876 .29696 .42501 .29311 .33561 Metetra
.395 .24 .55 r
.29311 .33561 .29696 .42501 .31182 .43115 .30671 .35301 Metetra
.18 .092 .512 r
.30671 .35301 .31182 .43115 .32619 .44797 .31931 .38921 Metetra
0 0 .419 r
.31931 .38921 .32619 .44797 .34041 .47266 .33169 .43711 Metetra
0 0 .172 r
.33169 .43711 .34041 .47266 .35479 .50201 .34451 .48905 Metetra
.913 .518 .124 r
.34451 .48905 .35479 .50201 .36959 .53285 .35821 .53831 Metetra
.861 .452 .257 r
.35821 .53831 .36959 .53285 .38495 .56241 .373 .5801 Metetra
.768 .367 .293 r
.373 .5801 .38495 .56241 .40096 .58859 .38887 .61196 Metetra
.683 .284 .283 r
.38887 .61196 .40096 .58859 .4176 .6101 .40567 .63359 Metetra
.601 .192 .239 r
.40567 .63359 .4176 .6101 .43482 .62642 .42321 .64621 Metetra
.509 .075 .151 r
.42321 .64621 .43482 .62642 .45254 .6377 .44128 .65181 Metetra
.382 0 0 r
.44128 .65181 .45254 .6377 .47066 .6445 .45971 .65251 Metetra
.171 0 0 r
.45971 .65251 .47066 .6445 .48911 .64759 .4784 .65012 Metetra
.4784 .65012 .48911 .64759 .5078 .6478 .49729 .64594 Metetra
.417 .855 .929 r
.49729 .64594 .5078 .6478 .52671 .64589 .51634 .6408 Metetra
.544 .871 .993 r
.51634 .6408 .52671 .64589 .5458 .64248 .53553 .63517 Metetra
.592 .847 .993 r
.53553 .63517 .5458 .64248 .56504 .63807 .55487 .62931 Metetra
.61 .823 .982 r
.55487 .62931 .56504 .63807 .58443 .63299 .57436 .62332 Metetra
.617 .808 .973 r
.57436 .62332 .58443 .63299 .60397 .62749 .59399 .61726 Metetra
.621 .799 .968 r
.59399 .61726 .60397 .62749 .62365 .62172 .61377 .61115 Metetra
.622 .794 .964 r
.61377 .61115 .62365 .62172 .64348 .61578 .6337 .60498 Metetra
.623 .791 .963 r
.6337 .60498 .64348 .61578 .66346 .60973 .65378 .59876 Metetra
.623 .789 .962 r
.65378 .59876 .66346 .60973 .68358 .60359 .67401 .5925 Metetra
.623 .789 .961 r
.67401 .5925 .68358 .60359 .70386 .59738 .6944 .58619 Metetra
.623 .788 .961 r
.6944 .58619 .70386 .59738 .72429 .59112 .71494 .57983 Metetra
.655 .752 .922 r
.04658 .50979 .05788 .51958 .07236 .51429 .06111 .50444 Metetra
.655 .752 .922 r
.06111 .50444 .07236 .51429 .08693 .50897 .07573 .49905 Metetra
.655 .752 .922 r
.07573 .49905 .08693 .50897 .10159 .50364 .09044 .49364 Metetra
.655 .752 .921 r
.09044 .49364 .10159 .50364 .11633 .49834 .10525 .48819 Metetra
.655 .75 .92 r
.10525 .48819 .11633 .49834 .13113 .49326 .12014 .4827 Metetra
.655 .743 .915 r
.12014 .4827 .13113 .49326 .14594 .48886 .13513 .47718 Metetra
.653 .721 .899 r
.13513 .47718 .14594 .48886 .16064 .48612 .15021 .47163 Metetra
.646 .671 .859 r
.15021 .47163 .16064 .48612 .17506 .48674 .16538 .46604 Metetra
.626 .585 .788 r
.16538 .46604 .17506 .48674 .18902 .49289 .18066 .46042 Metetra
.595 .488 .702 r
.18066 .46042 .18902 .49289 .20241 .50627 .19602 .45476 Metetra
.566 .412 .631 r
.19602 .45476 .20241 .50627 .21537 .52661 .21149 .44907 Metetra
.546 .364 .586 r
.21149 .44907 .21537 .52661 .22836 .5503 .22706 .44333 Metetra
.534 .337 .559 r
.22706 .44333 .22836 .5503 .24207 .57052 .24272 .43757 Metetra
.499 .3 .541 r
.24272 .43757 .24207 .57052 .25712 .5796 .25783 .43893 Metetra
.859 .594 .536 r
.25783 .43893 .25712 .5796 .29311 .33561 .29635 .16896 Metetra
.762 .623 .059 r
.29635 .16896 .29311 .33561 .30671 .35301 .29032 .42004 Metetra
.431 .131 .354 r
.29032 .42004 .30671 .35301 .31931 .38921 .30639 .41412 Metetra
.302 0 .125 r
.30639 .41412 .31931 .38921 .33169 .43711 .32257 .40817 Metetra
.587 .506 .73 r
.32257 .40817 .33169 .43711 .34451 .48905 .33876 .40351 Metetra
.968 .706 .279 r
.33876 .40351 .34451 .48905 .35821 .53831 .33849 .68133 Metetra
.474 .202 .421 r
.33849 .68133 .35821 .53831 .373 .5801 .35646 .67733 Metetra
.454 .167 .388 r
.35646 .67733 .373 .5801 .38887 .61196 .37466 .67165 Metetra
.423 .109 .325 r
.37466 .67165 .38887 .61196 .40567 .63359 .39299 .66593 Metetra
.353 0 .189 r
.39299 .66593 .40567 .63359 .42321 .64621 .41145 .66016 Metetra
.162 0 0 r
.41145 .66016 .42321 .64621 .44128 .65181 .43004 .65436 Metetra
.26 .764 .734 r
.43004 .65436 .44128 .65181 .45971 .65251 .44878 .64851 Metetra
.535 .884 .984 r
.44878 .64851 .45971 .65251 .4784 .65012 .46765 .64262 Metetra
.601 .843 .99 r
.46765 .64262 .4784 .65012 .49729 .64594 .48666 .63668 Metetra
.616 .813 .976 r
.48666 .63668 .49729 .64594 .51634 .6408 .50581 .6307 Metetra
.621 .798 .967 r
.50581 .6307 .51634 .6408 .53553 .63517 .52511 .62468 Metetra
.623 .792 .963 r
.52511 .62468 .53553 .63517 .55487 .62931 .54454 .61861 Metetra
.623 .789 .962 r
.54454 .61861 .55487 .62931 .57436 .62332 .56413 .6125 Metetra
.623 .789 .961 r
.56413 .6125 .57436 .62332 .59399 .61726 .58386 .60633 Metetra
.623 .788 .961 r
.58386 .60633 .59399 .61726 .61377 .61115 .60374 .60013 Metetra
.623 .788 .961 r
.60374 .60013 .61377 .61115 .6337 .60498 .62377 .59387 Metetra
.623 .788 .961 r
.62377 .59387 .6337 .60498 .65378 .59876 .64396 .58757 Metetra
.623 .788 .961 r
.64396 .58757 .65378 .59876 .67401 .5925 .6643 .58122 Metetra
.623 .788 .961 r
.6643 .58122 .67401 .5925 .6944 .58619 .6848 .57482 Metetra
.623 .788 .961 r
.6848 .57482 .6944 .58619 .71494 .57983 .70545 .56837 Metetra
0 g
.25 Mabswid
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
[(80)] .07825 .23514 .98066 1 Mshowa
.22216 .18933 m
.227 .19478 L
s
[(90)] .21247 .17844 .88892 1 Mshowa
.36358 .12987 m
.36813 .13557 L
s
[(100)] .3545 .11847 .79718 1 Mshowa
.51343 .06687 m
.51763 .07282 L
s
[(110)] .50502 .05496 .70545 1 Mshowa
.67245 0 m
.67627 .00621 L
s
[(120)] .66483 -0.01242 .61371 1 Mshowa
.125 Mabswid
.11461 .23456 m
.11764 .23771 L
s
.14105 .22344 m
.14405 .22662 L
s
.16779 .2122 m
.17076 .21541 L
s
.19482 .20083 m
.19776 .20407 L
s
.2498 .17771 m
.25268 .18101 L
s
.27776 .16595 m
.2806 .16928 L
s
.30604 .15406 m
.30885 .15742 L
s
.33465 .14204 m
.33741 .14542 L
s
.39286 .11756 m
.39554 .12101 L
s
.42247 .10511 m
.42512 .10859 L
s
.45243 .09251 m
.45504 .09602 L
s
.48275 .07977 m
.48531 .08331 L
s
.54447 .05381 m
.54695 .05742 L
s
.57589 .0406 m
.57832 .04424 L
s
.60769 .02723 m
.61007 .0309 L
s
.63987 .0137 m
.64221 .0174 L
s
.25 Mabswid
0 .80374 m
.40037 1.0855 L
s
.09485 .87049 m
.09577 .86326 L
s
[(0.2)] .09301 .88495 .12735 -1 Mshowa
.18136 .93137 m
.18209 .92412 L
s
[(0.4)] .17991 .94587 .10002 -1 Mshowa
.26055 .98709 m
.2611 .97983 L
s
[(0.6)] .25944 1.00163 .07584 -1 Mshowa
.3333 1.03829 m
.33369 1.03102 L
s
[(0.8)] .33251 1.05285 .05428 -1 Mshowa
.40037 1.0855 m
.40063 1.07821 L
s
[(1)] .39987 1.10006 .03495 -1 Mshowa
.125 Mabswid
.02453 .821 m
.02518 .81668 L
s
.04852 .83789 m
.04914 .83356 L
s
.07196 .85438 m
.07254 .85004 L
s
.11722 .88623 m
.11774 .88189 L
s
.13908 .90162 m
.13958 .89727 L
s
.16046 .91666 m
.16092 .91231 L
s
.20181 .94576 m
.20221 .9414 L
s
.22181 .95983 m
.22219 .95548 L
s
.24138 .97361 m
.24174 .96925 L
s
.27931 1.0003 m
.27961 .99593 L
s
.29768 1.01322 m
.29796 1.00886 L
s
.31567 1.02589 m
.31593 1.02152 L
s
.35057 1.05045 m
.35079 1.04608 L
s
.3675 1.06236 m
.3677 1.058 L
s
.3841 1.07404 m
.38427 1.06967 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True,
  ImageSize->{185.688, 201.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002i0000bA000`40O003h00O^Goo002iOol0075o
o`<000=oo`@000=oo`8003Uoo`00LWoo00<007ooOol00goo00<007ooOol00Woo00@007ooOol003Qo
o`00LWoo00<007ooOol017oo00D007ooOomoo`00009oo`03001oogoo03Ioo`00LWoo00<007ooOol0
1Goo00@007ooOol0009oo`03001oogoo03Ioo`00LWoo00<007ooOol00Woo00@007ooOol0009oo`04
001oogoo000hOol0075oo`8000Eoo`8000Aoo`8003Uoo`00^Goo002iOol00;Uoo`00Ogoo0P00>7oo
001lOol300000goo0000000gOol005Ioo`<000=oo`<000Aoo`8001Eoo`8000Aoo`8003Moo`00Egoo
00<007ooOol00goo00<007ooOol00Woo00@007ooOol0019oo`800003Ool007oo00=oo`03001oo`00
03Ioo`00Egoo00<007ooOol00goo00<007ooOol00Woo00@007ooOol000moo`<000Qoo`03001oo`00
03Ioo`00Egoo00<007ooOol00goo00<007ooOol00Woo00@007ooOol000eoo`800003Ool007oo00Qo
o`04001oogoo000eOol005Moo`03001oogoo00=oo`03001oogoo009oo`04001oogoo000:Ool3000=
Ool01000Oomoo`00=Goo001FOol20004Ool20005Ool20009Ool4000>Ool01@00Oomoogoo0000=7oo
001/Ool2000BOol00`00Oomoo`02Ool00`00Oomoo`0aOol006Uoo`@001Aoo`05001oogooOol0000c
Ool006Moo`8001Qoo`03001oogoo009oo`03001oogoo031oo`00I7oo0`006Woo00<007ooOol00Woo
00<007ooOol0<7oo000mOol30004Ool20004Ool2000FOol20003Ool00`00Oomoo`0GOol00`00Oomo
o`03Ool00`00Oomoo`0_Ool003ioo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`004goo
0P007goo00<007ooOol017oo00<007ooOol0;Woo000nOol00`00Oomoo`02Ool01000Oomoo`000Woo
00@007ooOol0011oo`<00003Ool007oo01ioo`03001oogoo00Aoo`03001oogoo02ioo`00?Woo00<0
07ooOol00Woo00@007ooOol0009oo`04001oogoo000>Ool2000TOol00`00Oomoo`05Ool00`00Oomo
o`0]Ool003ioo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`002goo0`000Woo00<007oo
Ool08Goo00<007ooOol01Woo00<007ooOol0;7oo000mOol20005Ool20004Ool2000:Ool2000YOol0
0`00Oomoo`06Ool00`00Oomoo`0/Ool005Aoo`80009oo`03001oogoo02Ioo`03001oogoo00Moo`03
001oogoo02]oo`00DGoo0`00;Goo00<007ooOol01goo00<007ooOol0:goo001?Ool20002Ool00`00
Oomoo`0[Ool00`00Oomoo`08Ool00`00Oomoo`0ZOol004aoo`<0039oo`03001oogoo00Uoo`03001o
ogoo02Uoo`00:Woo0`0017oo0P005goo0P000Woo00<007ooOol0<7oo00<007ooOol02Goo00<007oo
Ool0:Goo000]Ool01000Oomoo`000Woo00<007ooOol04Woo0P00=goo00<007ooOol02Woo00<007oo
Ool0:7oo000[Ool30002Ool01000Oomoo`004Goo0`0000=oo`00Ool0=Woo00<007ooOol02goo00<0
07ooOol09goo000ZOol01000Oomoo`000Woo00@007ooOol000moo`8003eoo`03001oogoo00Yoo`03
001oogoo02Moo`00:Woo00@007ooOol0009oo`04001oogoo000<Ool30002Ool00`00Oomoo`0jOol0
0`00Oomoo`0;Ool00`00Oomoo`0VOol002]oo`8000Aoo`8000]oo`80049oo`03001oogoo00]oo`03
001oogoo02Ioo`00?7oo0P000Woo00<007oo0000?goo00<007ooOol037oo00<007ooOol09Goo000i
Ool30006Ool00`00Oomoo`0mOol00`00Oomoo`0=Ool00`00Oomoo`0TOol003Moo`80009oo`03001o
ogoo00Aoo`03001oogoo03eoo`03001oogoo00eoo`03001oogoo02Aoo`00=7oo0`002goo00<007oo
Ool0?Goo00<007ooOol03Woo00<007ooOol08goo000EOol20004Ool2000EOol20003Ool00`00Oomo
o`08Ool00`00Oomoo`0mOol00`00Oomoo`0>Ool00`00Oomoo`0SOol001Aoo`04001oogoo0002Ool0
1000Oomoo`004Woo0`003Woo0P00?goo00<007ooOol03goo00<007ooOol08Woo000DOol01000Oomo
o`000Woo00@007ooOol000moo`<0009oo`03001oogoo00aoo`8003moo`03001oogoo011oo`03001o
ogoo025oo`005Goo0P000goo00@007ooOol000eoo`@0019oo`8003moo`03001oogoo011oo`03001o
ogoo025oo`0057oo00@007ooOol0009oo`04001oogoo000:Ool3000FOol2000oOol00`00Oomoo`0A
Ool00`00Oomoo`0POol001Eoo`8000Aoo`8000Uoo`800003Ool007oo01Ioo`8003moo`03001oogoo
019oo`03001oogoo01moo`0097oo0P006goo0P00?goo00<007ooOol04Woo00<007ooOol07goo000Q
Ool300000goo001oo`0JOol2000oOol00`00Oomoo`0COol00`00Oomoo`0NOol0021oo`03001oogoo
01eoo`03001^L@0003moo`03001oogoo01=oo`03001oogoo01ioo`0087oo0P007Woo00<006ia0000
@7oo00<007ooOol04goo00<007ooOol07Goo000POol2000NOol00`00KW400010Ool00`00Oomoo`0D
Ool00`00Oomoo`0LOol0021oo`03001oo`0001eoo`03001^L@00041oo`03001oogoo01Aoo`03001o
ogoo01aoo`007goo00D007ooOomoo`0001aoo`03001^L@00041oo`03001oogoo01Eoo`03001oogoo
01]oo`007goo0`000Woo00<007ooOol06Goo00<006ia0000@7oo00<007ooOol05Goo00<007ooOol0
6goo000HOol30004Ool00`00Oomoo`03Ool00`00Oomoo`0HOol00`00KW400010Ool00`00Oomoo`0F
Ool00`00Oomoo`0JOol001Uoo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo01Ioo`04001^
LFia0010Ool00`00Oomoo`0GOol00`00Oomoo`0IOol0019oo`D0009oo`03001oogoo00=oo`8000Io
o`03001oogoo01Eoo`04001^LFia0010Ool00`00Oomoo`0GOol00`00Oomoo`0IOol001Uoo`03001o
ogoo00=oo`03001oogoo00Ioo`03001oogoo01Aoo`04001^L@000010Ool00`00Oomoo`0HOol00`00
Oomoo`0HOol001Uoo`03001oogoo00=oo`03001oogoo00Moo`03001oogoo01=oo`04001^L@000010
Ool00`00Oomoo`0IOol00`00Oomoo`0GOol001Qoo`8000Aoo`03001oogoo00Uoo`03001oogoo019o
o`04001^L@000010Ool00`00Oomoo`0IOol00`00Oomoo`0GOol001ioo`8000]oo`03001oogoo015o
o`04001^L@000010Ool00`00Oomoo`0JOol00`00Oomoo`0FOol001ioo`03001oogoo00]oo`03001o
ogoo011oo`04001^L@000010Ool00`00Oomoo`0JOol00`00Oomoo`0FOol001ioo`03001oogoo00ao
o`03001oogoo00moo`04001^L@000010Ool00`00Oomoo`0KOol00`00Oomoo`0EOol001ioo`03001o
ogoo00eoo`03001oogoo00eoo`03001^LFia0080041oo`03001oogoo01aoo`03001oogoo01Aoo`00
7Woo0P003goo00<007ooOol037oo00<006iaKW400P00@7oo00<007ooOol077oo00<007ooOol057oo
000MOol00`00Oomoo`0@Ool00`00Oomoo`0;Ool01P00KW5^L@00HV40041oo`03001oogoo01aoo`03
001oogoo01=oo`007Goo00<007ooOol04Goo00<007ooOol02Woo00@006iaKW40009RH@03001oogoo
03eoo`03001oogoo01eoo`03001oogoo019oo`007Goo00<007ooOol04Woo00<007ooOol02Goo00@0
06iaKW40009RH@03001oogoo03eoo`03001oogoo01eoo`03001oogoo019oo`007Goo0P0057oo00<0
07ooOol027oo00@006iaKW40009RH@8003ioo`03001oogoo01ioo`03001oogoo015oo`007Goo00<0
07ooOol057oo00<007ooOol01goo00P006iaKW40069Q000dR`00?Woo00<007ooOol07Woo00<007oo
Ool04Goo000MOol00`00Oomoo`0EOol00`00Oomoo`05Ool02@00KW5^LFia001RH@00=8/0000nOol0
0`00Oomoo`0OOol00`00Oomoo`0@Ool001eoo`8001Moo`03001oogoo00Aoo`07001^LFiaKW40069Q
00000SB;00<007ooOol0>goo00<007ooOol087oo00<007ooOol03goo000:Ool20004Ool20003Ool3
0004Ool2000IOol00`00Oomoo`03Ool01000KW5^LFia0P000cB;00<007ooOol0>goo00<007ooOol0
87oo00<007ooOol03goo0009Ool01000Oomoo`000goo0P001Woo00D007ooOomoo`0001]oo`03001o
ogoo009oo`04001^LFiaKW420003=8/00`00Oomoo`0kOol00`00Oomoo`0QOol00`00Oomoo`0>Ool0
00=oo`D00005Ool007ooOol0000;Ool01@00Oomoogoo000077oo00D007ooOomoo`00009^L@<0008d
R`800003Ool0000003Yoo`03001oogoo025oo`03001oogoo00ioo`002Goo00@007ooOol000Qoo`<0
00Aoo`8001aoo`04001oogoo0002KW430002=8/300000dXG001oo`0iOol00`00Oomoo`0ROol00`00
Oomoo`0=Ool000Uoo`04001oogoo0008Ool00`00Oomoo`04Ool00`00Oomoo`0LOol00`00Ool00002
KW4200000cB;0000000300000dXG001oo`0iOol00`00Oomoo`0QOol3000>Ool000Yoo`8000Uoo`@0
00=oo`03001oogoo01eoo`04001^LFiaKW4400001S3a00000000BQL003Yoo`03001oogoo01ioo`<0
009oo`03001oogoo00aoo`0077oo00<007ooOol07Goo00D006iaKW5^L@00009o3`<0000455000000
000jOol00`00Oomoo`0KOol30005Ool00`00Oomoo`0<Ool001]oo`8001moo`05001^LFiaKW400002
O`l01000>E<001A@0`00>Woo00<007ooOol067oo0`0027oo00<007ooOol037oo000KOol00`00Oomo
o`0KOol40003KW401`00O`mo3`00>E<001A@00<003Yoo`03001oogoo01Eoo`<000]oo`03001oogoo
00aoo`006goo00<007ooOol067oo0`0000A5D@00?C40009^L@03001o3gl?00800003>E<0000000<0
03]oo`03001oogoo015oo`<000moo`03001oogoo00]oo`006goo00<007ooOol05Woo0P0000M5LP00
AE55D@00?C400002KW400`00O`mo3`0200000cUC000Xl`04000jOol00`00Oomoo`0>Ool3000BOol0
0`00Oomoo`0;Ool001]oo`03001oogoo01=oo`<00004BK@004Eb0002AE400`00?C400002KW401@00
O`mo3`00Oa400P0000<Xl`0000d00P00>Woo00<007ooOol02goo0`005Goo00<007ooOol02goo000K
Ool00`00Oomoo`0>Ool500001dgf0019]4Vd0015LP000095D@03000m<@00009^L@07001o3gl?001o
4@00>E<01@00>Woo00<007ooOol027oo0`0067oo00<007ooOol02goo000JOol00`00Oomoo`0=Ool2
000015:k001BF@000Tgf00H004VdBK@004Eb0002AE400`00?C400002KW400`00O`l00002Oa401000
>E<002Sc0`00>Woo00<007ooOol017oo10006goo00<007ooOol02goo000JOol00`00Oomoo`0:Ool3
00002E;l001B^e:k001BF@00COH00002BK@00`00AG800002AE400`00?C400002KW400`00O`l00002
Oa401000>E<002Sc0`00>Woo00@007ooOomoo`<001moo`03001oogoo00]oo`006Woo0P002Goo0P00
0U;m00X005;lD_`005:k001BF@00COH00099]0030015LP000095D@05000m<@00KW400002O`l00`00
Oa5o4@07000jOol4000ROol00`00Oomoo`0;Ool001Yoo`03001oogoo00Eoo`<00004D_d005;mD_d8
0002COH01@00BK@004Eb00000TEA00D003da001^L@00009o3`07001o4GlA0011U@00<ED01000=goo
0`009Woo00<007ooOol02Woo000JOol00`00Oomoo`03Ool20002Dad00`00D_d00002000014jK001:
F@000Sgh00T002eg001=mP00BK@004Eb00000TEA00L003da001^L@00O`l00002Oa420000146E000a
EC5E0`00=7oo0`000Woo00<007ooOol097oo00<007ooOol02Woo000JOol00`00Oomoo`030002Dad0
1000Dad000000U;L00T004jKCY/004YI000mn000;GL00`0000A9]000AG800095D@07000m<@00KW40
07l?00000WlA0P000T6E00@0035E001jb080031oo`<000Eoo`03001oogoo02Aoo`03001oogoo00Yo
o`006Woo100000AC7@00DaeC7@80009Bo003001Bg00000D0008mn08000064?D004Vd0015LP000TEA
00L003da001^L@00O`l00002Oa402000Oc<0046E000aE@00N/P2000]Ool30008Ool00`00Oomoo`0T
Ool00`00Oomoo`0:Ool001=oo`8000Aoo`80009C7@D00004D_d005;lD_`2000014Yk0012F`000SXJ
0`0000L]M`000019]000AG800002AE402@00?C4006ia001o3`00Oa400002Oc<02@00@ID0035E001j
b000MP<0000YOol3000;Ool00`00Oomoo`0TOol00`00Oomoo`0:Ool0019oo`04001oogoo0003Ool0
1000Ool000000U;m00<005;m0000100000Y>_000BW]:N`00@U/003XJ000]fP<000054?D00000AG80
0002AE401000?C4006ia0P000WlA00@007lcOc<00091U@80009jb08002Ioo`<000ioo`03001oogoo
02Aoo`03001oogoo00Yoo`004Woo00@007ooOol000=oo`04001oogoo0002D_d20002D_`01000D]`0
04jl2@0000<]fP0000000`0000I5LP00AE55D@00?C42000017l?001o4@000glc1@000W[80`008Goo
10004Goo00<007ooOol097oo00<007ooOol02Woo000BOol01000Oomoo`000goo00@007ooOomoo`80
0004D_d005;lD_`4000024JL0012O49l000jG000<S`50000105I0015LP000TEA00P003da000007l?
001o4@000Wlc0P000TGF0P000W[800<007H300007Woo0`005Goo00<007ooOol09Goo00<007ooOol0
2Goo000BOol01000Oomoo`000goo0P000goo00<005;m00000P0000M>g000B[a:_000AY`00002@W`0
2000>U`0038l000Z70008Q`40002AE403000?C400000O`l007lA001o<`00Kk`00095eP8000=jb0<0
01Yoo`<001Qoo`03001oogoo02Eoo`03001oogoo00Uoo`004goo0P0017oo00<007ooOol00goo0P00
00=Bg000C]`050000TEA00<003da00002@000dGF00H007[8N/Qjb000KLP2000FOol3000LOol00`00
Oomoo`0TOol00`00Oomoo`09Ool001Qoo`03001oogoo00Ioo`<0009:_003001:_0000096W005000n
O000>Wd00002=Wd01000;Wd002Yn0P000TEA00<003da00000P0000PkG`00000004mo001__0000TGF
00D007[8N/Qjb000009]b080019oo`<001moo`03001oogoo02Aoo`03001oogoo00Uoo`0067oo00<0
07ooOol01Woo00D004kMC]d004Zl01800095D@03000m<@0000X00003AMH0000000=jb0800004KLP0
0000000>Ool3000ROol00`00Oomoo`0TOol00`00Oomoo`09Ool001Qoo`8000Qoo`@0009:_@050016
_@00@Yd00002?Yd01@00>Yd003ZN00000SJN00H003Jn0015D@00?C4400000dLo0000000500001dWg
001jb7[8000lc@00009]b0<000Yoo`<002Eoo`03001oogoo02Aoo`03001oogoo00Uoo`0067oo00<0
07ooOol027oo00<004[M00000TZm4`0000<m<@000000100000=;?`0000000P000TWg00H007[8N/P0
03c=0002KLP01000ID0000001Woo0`009Woo0`009Woo00<007ooOol02Goo000HOol00`00Oomoo`09
Ool400002DJm0016_DJm0012_@00@[d00002?[h<0002Aal20002Ccl01@00Ccl004Wg00000W[800<0
03c=?<d00P0000EQJ@00001oo`00009oo`<002Ioo`<0009??P03001oogoo02Aoo`03001oogoo00Uo
o`0067oo00<007ooOol02Woo00<004[M00002`0000I2_P00?]hngP00@]h800000d/O000000040000
1DWg001jb7[800000Sc=00@0065YHFUQJ@D002Ioo`<0009??P03001??Tln008002Ioo`03001oogoo
00Qoo`0067oo00<007ooOol02goo00P004[MB]d004Jm0016_@000T:m2@000T;n00<004Ko00001000
0TlO00H004ln000004Wg0002N/P01000?<dlc@000f5Y00@006E0000002=oo`@00004Cch004lnCch5
00000dln001oo`0TOol00`00Oomoo`08Ool001Moo`03001oogoo00eoo`03001:g@0000H00008A]h0
04;N@]h004;N0012gPH00003A_h004/N00D0009??P80009jb003000lcCc=008000=QJ@03001Q0P00
008001ioo`<0009??P03001??Tln00H00003Cch00000008002Aoo`03001oogoo00Qoo`005goo00<0
07ooOol03Woo00L004[MB]d004KM0016g@0:000014Kn0016oTKn1@0000=;7P0000001@0000=jb000
?<d00Sc=00<003R/00000V5Y00H00642H@80051P000JOol30002Cch00`00Cci??P060002Cch40000
0dln001oo`0SOol00`00Oomoo`08Ool001Moo`03001oogoo00moo`H00096g@040016gP00A]h80000
14[n001;7T/N0P000TlN00<004lN00000P0000Ejb000?<dlc@00008h[006001QJ@00EBT006421@00
5goo00@004lnCch0009??PH0009??P<0009??P03001??P00008002=oo`03001oogoo00Qoo`005goo
00<007ooOol047oo00@004[M0016g@P00003A]h004Kn00P00003Bah0000000<0009?7P03001jb000
008lc@05000h[3R/>:`00003EBT01`00F<9H`US2001@@00001=oo`<0009??PH0009??P@0009??P03
001??Tln00@00003Ccl007oo025oo`03001oogoo00Qoo`005goo00<007ooOol04Goo100000A:g@00
A]h00096gPD00003B_h004[n00D0009?7P03001?7P0000<00005N/P003c=?<d00003>:`01000EBUE
:EDY0P000eS210003Woo0`000Tln1P000Tln0`000Tlo00<004lnCch0100000=??`00000010008Goo
00<007ooOol027oo000GOol00`00Oomoo`0BOol00`00B]d00006000014KN0016oTKn1@0000=:oP00
Bah01@000TlN0P0000Ejb000?<dlc@0000<h[0800007EBT004c7C<L005S200000TaP00<004P00000
2Woo0`000Tln1P000Tln0`000TmO00<004loCcl00`000Tlo0`000TmO0P0000=??`0000008Goo00<0
07ooOol01goo000FOol00`00Oomoo`0DOol00`00B]d00002B]h7000014[n001:oT[n1@0000=?7P00
Cah00`0000A?7P00N/P0008lc@08000h[3R/000dJP00EBT000=<a`8000=<H0D000Aoo`<0009??PH0
009??`<0009?G`03001?GdmO00<0009?G`<0009;G`03001?GdmO00<0009;?`03001oogoo01ioo`03
001oogoo00Moo`005Woo00<007ooOol05Goo100000A:gP00B_i:oPL00004B_h004[nB_h40002Cah4
0002?<d01@00>:`h[3R/00000SAZ0P000dc700D00414001<H000009000L0009??PH0009?G`<00003
Bel0000000D00003Bel0000000800004Bel004]OBel400000d]O001;?`04000OOol00`00Oomoo`07
Ool001Ioo`8001Moo`03001:g@0000@00006B_h004[nB_h004[n1@0000=>oP0000000`0000A?7P00
0000008lc@05000h[3R/>:`00002=6X01@00;0H004c700000d141@0000A?G`00Cem?G``00004Agl0
00000002@el60002Ael60002Ael700000d/o0000000NOol00`00Oomoo`07Ool000Aoo`8000Aoo`80
00=oo`<000Aoo`03001oogoo01Moo`05001:oD[m001:oP0700000d[n000000030002Cah00`00Cah0
00040002?<d01000>:`h[0000SAZ00@002`6;0H/1PH00097W`03001?GdmO01D0008gG`03000oGcmO
00<00093G`030017GdMO00<00004Ael004]OBel400000d/o001oo`0MOol00`00Oomoo`07Ool000=o
o`04001oogoo0003Ool20006Ool01@00Oomoogoo00006Woo10000T[n00<004[n0000100000=:oP00
C_h01@000TlN0`000Sc=00@003R/>:`0008dJP03000/1R`602h00003@cl0000000@00097?`H001ao
o`03001oogoo00Moo`000goo00@007ooOol000]oo`05001oogooOol0000KOol00`00B_h000040000
14[n001:oT[n1@0000=>oP00Cah00`0000=?7P0000000Sc=00@003R/>:`0008dJP@00006@UX003iK
?U/003Xl100000<^G@000000100000DNGP000000019M00d0008c7`03000g?cLo00H00003?cl00000
00D00004Acl00000Acl3000KOol00`00Oomoo`07Ool000=oo`04001oogoo0008Ool30004Ool00`00
Oomoo`0JOol200000d[n001:oP05000014kn001>oTkn10000TlN100000Hlc@00>:`h[000=6X30000
0dYJ0000000=000012YM000VGRIN5`0000<g7`00000010000Slo00<004<o@cl01P000TLo0P006goo
00<007ooOol01Woo0003Ool01000Oomoo`0027oo00<007ooOol00goo0P007Woo0`000T[n00<004[n
0000100000=>oP0000000`0000=?7P0000000Sc=00<003R/00001P0000=:NP0000000P000T9K00H0
03iL?U`003YL0002=Ud500029Wh00`009Wh0000500029[l;0002=al90002@cl00`00@cm3?`030000
0dLo001oo`0JOol00`00Oomoo`06Ool000Aoo`8000Uoo`@0009oo`03001oogoo01ioo`H00003C_h0
000000<0009>oP03001?7P0000<00009?<d00000001BN@00EWT004ij00D00096N`030012F`0000H0
0006=Ud0039m<Wd002in2@0000<V_`0000003@0000@k7`00>alk7`X00003@cl0000000@001Uoo`03
001oogoo00Ioo`005Goo00<007ooOol07goo00<004kn0000100000=>oP00C_h01@0000=?7P000000
0`0000=C7P0000000UIi100000=:NP000000100000I6N`00@Wa2O000?W`<000012jN000^WRjN5000
0SlO00<003lO?al01P0000=3?`0000000TLo0P0067oo00<007ooOol01Woo000EOol00`00Oomoo`0P
Ool00`00C_h00002C_h40002C_h01000C_h000000TlN00<004lN00001@0000=FN@0000000P0000=>
NP00BW/00TYk00@004IkAW/00092O003000nO000008jO@03000fO@00008fOP03000bWS:N01d00004
?al004<O@al700000dLo0000000GOol00`00Oomoo`06Ool001Eoo`03001oogoo025oo`@00003C_h0
04kn00D00003C_h0000000P00003EWT00000009BVP04001>NTij0002BW/J0002<[l01000<]lbg`00
0SKO00@003Ko=_l0008jo``000937`H00003Aal007oo01Ioo`03001oogoo00Ioo`005Goo00<007oo
Ool08goo1P0000=>oP0000002@0000AC7P00Dah0009FV@03001BVP0000D00005BY/004JKAY/00002
@W`00`00@W`00002?Yd01000>YdjW@000SZN00H003JN=Yh003Jn0002=[h>00001C[o000joc[o0000
0SlO1`0000A37`00@am37`H001Eoo`03001oogoo00Ioo`005Goo00<007ooOol08goo00<004kn0000
100000=>oP0000001@000TlN1@000UJI0P0000=BVP00CY/00TjK00D004ZKBY/004JK02/0008o7`03
00137d<O00H00004@al004LOAal3000DOol00`00Oomoo`06Ool001Aoo`03001oogoo02Eoo`800003
C_h004kn00D00003C_h0000000<00003Dah0000000@00005EYX005:JDYX00002CY/500001DZL0016
W4JL00000T:L00H004:M@Yd003jM0002?Yh01000>[hj_P000SZn00@003Zn>[h0008jg`04000jgc[O
0002>]l?000014<O00137d<O1P000TLO00<007ooOol04goo00<007ooOol01Goo000DOol00`00Oomo
o`0VOol40002C_h40002C_h01000Cah000000U<N00<005<N00000eJJ00<005:J00000P0000IBV`00
CY]>V`00BY`P00001C[O000nocko00000Sko30000T<O00<004LO00001@0057oo00<007ooOol01Goo
000DOol2000TOol30002Ool400000dkn001>oP0500000dlN0000000300000e<M001FVP03000015JJ
001BVe:K1@0000=:W000BY`00TZL00<004JL00000T:M00@004:m@[d0008n_AL00005?_l003ko?_l0
0002@_l@000COol00`00Oomoo`05Ool001Aoo`03001oogoo021oo`<000Ioo`03001>oP0000@00003
C_h000000080009C7P@00006Dad005JjE[X005JJ0`0000EB^`00CY]>V`00009:W003001:W4ZL00/0
0005@[d003jn?[h00002?[h01000?]hngP000SkN00<003kO00003P000T;o00<004<O@al020000TLO
0P004Woo00<007ooOol01Goo000DOol00`00Oomoo`0MOol3000:Ool00`00C_h0000400000dkn001?
7P0300000e<M001C7@030002E[X200000eJj001B^`02D[/00`00CY/0000500001DZl0016_4Jl0000
0TJm00<004:m000040000SkN00@003kO?]l0008no`030012od;o00T00004@al004LOAal600000dLO
001oo`0AOol00`00Oomoo`05Ool001Aoo`03001oogoo01Yoo`<000ioo`800003C_d004kn00D00003
Dad000000080009C7@<00005F[X005JjE[X00002D[/20002D[/00`00C[`00002B[`80002A[d01P00
@[e2_@00@[h00092gP030012gT;N00l00004@_l004;o@_l900000dLO00177`07000@Ool00`00Oomo
o`05Ool001=oo`03001oogoo01Moo`@001=oo`<00003C_d0000000<00003Dad0000000<00003Dad0
05Zj009J^P03001F^P0000800004D[/005:kD[/50002C[`00`00B[`00002A[d>00001D;N0012gT;N
00000T;N300000A2o`00@_m2o`X00004Aal004LOAal2000?Ool00`00Oomoo`05Ool001=oo`8001Eo
o`<001Qoo`@00003C_d000000080009C7@07001C7@00001C7@00F[X00`0000AF^`00D[]B^`<00005
D[/004jlC[`00002C[`500001DZm0016_DJm00000TJm00<004KNA]h02`000T;N00<004;n00000T;o
2@0000A6o`00Aam77`X00003Aal0000000moo`03001oogoo00Aoo`004goo00<007ooOol04Goo0`00
77oo100000=>o@00Dad01@0000=C7@0000000P000UZj00<005Jk00000P000UJk00<005:k00001@00
00I>_000B[a:_000B[d900001DKN0016gTKN00000T;N00<004;N@]h020000T;n00@004;o@_l00096
o`D000977`0300177dLO00H00003Aal007oo00ioo`03001oogoo00Aoo`004goo00<007ooOol03Woo
0`0087oo00<004km00000`000U<M100000AC7@000000009J^P80009F^`03001F^eJk0080009B^`03
001>_000009>_0D0009:g@05001:gD[M0016g@0<000014KN0016oTKn300000A6o`00A_m6o`H00003
Aal004LO00H000ioo`03001oogoo00Aoo`004goo00<007ooOol02Woo100097oo00<004km00001000
00=C7@0000000`0000IC7@00000005[J0002E[/3000015Jk001B^e:k1@0000E>g000C]a>g000009:
g@D0009:g@050016gTKN0016gP0900001DKn0016oTKn00000TKn2@0000A6o`00Aam77`H00004Aal0
04LOAal2000=Ool00`00Oomoo`04Ool001=oo`03001oogoo00Moo`<002Uoo`05001>oDkm001Bo@02
000015<M001C7E<M10000e[J0P0000AFf`00E[]F^`<00006D]/005;LD]`004kL1P0000A:g@00B]e:
g@P00096gP030016gP000096gPT00004A_h004KnA_h9000014LO00177dLO1P0000=;7`00Ool037oo
00<007ooOol017oo000BOol20006Ool3000]Ool400000e<M0000000300000e<M0000000200000e[J
00000003E]/20002E]/00`00D]/000040002C]`00`00C]a>g00500001D[M001:gD[M00000T[M2000
0TKn00<004Kn0000200000A6oP00A_i6oPT00003Aal004/O00H000aoo`03001oogoo00Aoo`002goo
0`0017oo00<007ooOol00Woo0`00<Woo0`0000=C7@0000000`0000=C7@0000000P000U[J00D005KK
001Jf`00009Ff`80009Ff`03001Bg000009>g0<00006C]`004kMC]d004[M1P0000E:g@00B]i:gP00
0096gPD00096oP050016oTKn0016oP06000014Kn0016oTKn3`002goo00<007ooOol017oo000<Ool0
0`00Oomoo`03Ool00`00Ool00002000fOol400000e<M00000002000015<M000000000U[J0P000U[K
0`0000AFf`00E]]Ff`D00004D]`004kLC]`50002C]d00`00B]d00008000014[n0016oTKn20000TKn
00<004KnA_h01@000T[n00<004/N00001P0000=;7P00Bah00P002goo00<007ooOol00goo000<Ool0
0`00Oomoo`03Ool2000kOol300000e<M001C7@0300002U<M00000000F]X005[K001Jf`000UKK0`00
00IFg000D]aBg000D]`3000014kL001>gDkM1@000TkM00D004[MB]d004[N00H00004B_h004[nB_h9
000014[n001:oT[n1@000T/N00<004/NBah01@000T/N00<007ooOol027oo00<007ooOol00goo000<
Ool00`00Oomoo`05Ool00`00Oomoo`0hOol00`00D_d000030002Dad500000e[J00000003F]/20000
15KK001Fg5KL1@000U;L00<004kL00000P0000A>g@00C]e>g@P0009:oP03001:oP0000D00004B_h0
04[nB_h9000014[n001:oT[n1P0000A;7P00001;7PD000Uoo`03001oogoo00=oo`0037oo00<007oo
Ool01Woo0P00>Goo00<005<M0000100000=C7@0000000`000U[J00H005[K001Jf`00E]/3000015KL
001Bg5;L1@000U;L00<004kM00000P0000E>g@00C]e>g@00009:o@D0009:oP03001:oP0000D00004
B_h004[nB_h9000014[n001;7T/N1P0000=;7P0000000`0027oo00<007ooOol00goo000;Ool20009
Ool2000iOol00`00DaeC7@0400000e<M000000020002F]X20002F]/20002F]/00`00E]`000020000
15KL001Bg5;L1@000TkM00<004kM00001@0000A>o@00B_e:o@D0009:oP03001:oP0000D00004B_h0
04[nB_h900000d/N00000004000014/N001;7T/N0P001goo00<007ooOol00goo000HOol00`00Oomo
o`0gOol200000e<M0000000200000e<M0000000300001U[K001Jf`00F]/0009Jf`80009Fg003001F
g00000800004D]`005;LD]`50002C_d00`00C_d00005000014km001:oT[n1@000T[n00<004[n0000
1@0000A:oP00B_i:oPH00004Bah004/NBah600000d/N001oo`06Ool00`00Oomoo`03Ool001Uoo`80
03Uoo`03001C7@0000<000Aoo`04001Jf`000003F]/2000015[K001Fg5KL0P000UKL00<005;L0000
0P0000ABo000D_eBo@D0009>o@03001>o@0000D00004C_h004[nB_h50002B_h00`00B_h000050000
14[n001;7T/N1P0000A;7P00001;7PD000Ioo`03001oogoo00=oo`006goo00<007ooOol0=goo00<0
07ooOol01goo00T005kKG]/005[K001Jn`00F]/00`0000AFo000E]aFg080009Bo003001Bo0000080
0004D_d004kmC_d50002C_d00`00C_d00005000014kn001:oT[n1@000T[n00<004[n00001@0000A;
7P00Bai;7PH00003Bah0000000<000Eoo`03001oogoo00=oo`0077oo0`00=Woo00<007ooOol01goo
00<005kKG]/00P000U[k0P000U[k00<005Kl00000P0000AFo000D_aBo080009Bo003001Bo@000080
0004D_d004kmC_d50002C_d00`00C_h00005000014kn001:oT[n1@000T[n200000=;7P0000001000
00A;7P00Bai;7P8000Eoo`03001oogoo009oo`007Woo00<007ooOol0=7oo00<007ooOol027oo0P00
00INn`00000005[k0002F_/20002E_`00`00E_`0000200000eKl000000030002D_d00`00D_d00002
000014km001>oDkm1@000Tkn00<004kn00001@0000=>oP000000100000A:oP00Bai;7PD00004Bah0
04/NBah600000d/N00000004Ool00`00Oomoo`02Ool001moo`8003Aoo`03001oogoo00Qoo`05001N
n`00G_/00002F_/3000015[k001Fo5Kl0P000UKl0P0000AFo000D_aBo0D0009Bo@D00004C_d004km
C_d50002C_h5000014kn001>oTkn1P0000=;7P00Bah01P0000=;7P000000100000=;7P00Ool00goo
00<007ooOol00Woo000QOol2000bOol00`00Oomoo`09Ool01000G_]Nnekk0P0000AJn`00F_]Jn`<0
0007F_`00000001Fo000E_`00`0000AFo000D_eBo@<00004D_d005;mD_d5000014km001>oDkm0`00
00=>oP00C_h01P0000A>oP00C_i>oPD0009?7PD00004Cah00000Cah50003Ool00`00Oomoo`02Ool0
025oo`<0035oo`03001oogoo00Yoo`06001Nn`00G_/005[k0`000U[k0P0000AJo000E_aFo080009F
o003001Fo00000800004D_d005;mD_d300000e;m001>o@0600000dkm000000030002C_h00`00C_h0
000500000dlN0000000400000dlN001?7P06000014lN001?7TlN0P000Woo00<007ooOol00Woo000T
Ool00`00Oomoo`0^Ool00`00Oomoo`0;Ool00`00G_]Nn`020002F_/01P00F_/005[k001Jo0<00004
E_`005KlE_`20002E_`500000e;m000000030002D_d00`00C_d00002000014km001>oTkn1@000Tkn
00<004kn00000P0000A?7P00Cai?7PD00004Cah004lNCah600001DlN000007oo000017oo000UOol0
0`00Oomoo`0]Ool00`00Oomoo`0;Ool01@00G_/005kk00000U[k0P000U[k0P000U[l00<005Kl0000
0P0000IFo000000005Kl0002D_d2000015;m001BoE;m1@000U;m1@0000A>oP00C_i>oPD00003Cah0
000000@00004Cah004lNCah500000dlN00000004000014lN001oo`0017oo000VOol2000]Ool00`00
Oomoo`0<Ool2000015kk000000000Ukk0P0000AJn`00F_aJo080009Fo0800004E_`005KlE_`30000
0eKm001Bo@03000015;m001BoE;m0`0000=Bo@00C_d01P0000=>oP0000000`0000A?7P00Cai?7PH0
0003Cah0000000<00003Cah004lN00H00003Ool007oo009oo`00:7oo0P00:goo00<007ooOol03Woo
00T005kkG_/005kk001Nn`00F_/00`0000MJo000000005[l001Fo00300000eKl000000030002D_d5
00000e;m000000030002D_d00`00C_d00002000014kn001?7TlN1@0000=?7P000000100000A?7P00
Cai?7PD00003Cah0000000@000=oo`006Goo0P0017oo0P000goo100000=oo`000000:Woo00<007oo
Ool03Woo00<005kkG_/00P000Ukk0P000U[k0P0000AJo000F_aJo080009Fo0800004E_`005KmE_d3
00000e;m001Bo@03000015;m001BoE;m1@0000=>o@000000100000=?7P0000000`0000A?7P00Cai?
7PH00003Cah0000000<00003Cah0000000<000=oo`0067oo00@007ooOol000=oo`8000Aoo`03001o
ogoo00=oo`03001oogoo02Moo`03001oogoo00moo`80000;G_/00000001Nn`00F_/005[k001Jo003
00001e[l00000000E_`005Kl00<00004E_d005;mD_d20002D_d00`00D_d00002000015;m001BoE;m
0P000U<M00<004lN00000P0000A?7P00Cai?7PD00003Cah0000000@00003Cah004lN00<000Aoo`03
001oogoo005oo`0067oo00@007ooOol000Yoo`03001oogoo00=oo`8002Moo`03001oogoo00moo`05
001Nn`00G_/00002G_/20002F_/20002F_`2000015[l001Fo5Kl0P000UKl1@0000=Fo@0000000`00
00=Bo@000000100000=Bo@0000000`0000=?7P000000100000=?7P0000000`0000A?7P00Cai?7PD0
00=oo`05001oogooOol00003Ool001Qoo`04001oogoo000;Ool00`00Oomoo`04Ool00`00Oomoo`0T
Ool00`00Oomoo`0@Ool2000015kk000000000Ukk0P000U[k0P0000=Jo000F_`00`0000IFo0000000
05Kl0002E_d2000015Km001BoE;m0P000U;m00<005;m00000P0000AC7@00DaeC7@800004Dah004lN
Cah300000dlN001?7P060009Ool01000Oomoo`000goo000HOol01000Oomoo`0027oo00@007ooOol0
00Moo`8002Aoo`03001oogoo019oo`09001Nnekk001Nn`00G_/005[k0080009Jo080009Jo0800004
F_`005KlE_`300000eKm0000000300000e;m0000000300000e<M0000000400000e<M000000030000
0e<N00000003000?Ool01000Oomoo`000goo000IOol2000:Ool20009Ool2000SOol00`00Oomoo`0B
Ool00`00G_]Nn`020002G_/2000015kk000000000U[l0P0000=Jo000F_`00`0000MFo000000005Km
001Fo@03000015LM001C7E<M0P0000AC7@00DaeC7@<00003Dad005<M00<001Eoo`04001oogoo0002
Ool0039oo`80025oo`03001oogoo01=oo`80009Nn`800003G_/00000009Nn`04001Jo0000002F_`2
0002F_`2000015LL001FoEKm0P000ULM1@0000=C7@0000000`0000=C7@0000000`006Goo00<007oo
00000Woo000cOol2000POol00`00Oomoo`0DOol02000G_/00000G_/005kk0002G_/200001E[l0000
0000F_`00`0000=K7000Ea`00`0000MG7@00000005LM001G7@03000015<M001C7E<M0P007goo0P00
0Woo000eOol00`00Oomoo`0MOol00`00Oomoo`0EOol00`00G_/0000200001Ekk00000000G_/00P00
0U/L0P0000AK7000FaaK7080009G70800004Ead005LMEad200000eLM00000004000POol30002Ool0
03Ioo`8001eoo`03001oogoo01Ioo`03001O6elK0080009O6`80009O6`800003Fa`005/L00<00005
Fa`00000001K700500000eLM0000000QOol50005Ool002Moo`8000Aoo`8000Aoo`<0009oo`03001o
ogoo01Uoo`03001oogoo01Moo`0:001O6`00Ga/005lK001O6`00Ga/20002Fa`20002Fa`200001e/L
00000000Fa`005LM00<001moo`@000Yoo`009Woo00@007ooOol000=oo`8000Eoo`03001oogoo009o
o`8001Uoo`03001oogoo01Qoo`03001O6elK0080009O6`800005Ga/00000001O700300001E/L0000
05/L00000U/L0P007Goo1@003Woo000VOol01000Oomoo`0027oo10000goo0`0067oo00<007ooOol0
6Goo0P000V<K0P0000=O6`0000000UlK0P0000MO7000Fa`005/L001K7003000KOol4000COol002Io
o`04001oogoo0008Ool00`00Ool00007Ool2000FOol00`00Oomoo`0JOol03@00Ha/00000Ga/005lK
001O6`00Ga/005lL0080009K708001Uoo`D001Moo`009Woo00@007ooOol000Qoo`03001oo`0000Qo
o`8001Eoo`03001oogoo01]oo`03001S6`0000800003Ga/00000009O6`800004Ga`00000000GOol4
000LOol002Moo`8000Yoo`8000Yoo`8001=oo`03001oogoo01aoo`03001S6f<K00800005Ga/00000
001O6`02000FOol4000POol0041oo`80019oo`03001oogoo01aoo`03001S6`00009S6`03001O6elK
008001=oo`D002Aoo`00@Woo00<007ooOol03goo00<007ooOol07Goo0P0000ES6`00001O6`00015o
o`@002Uoo`00@goo0P003goo00<007ooOol07goo00@006<K000000eoo`D002eoo`00=Goo0P0017oo
0P0017oo0P000Woo00<007ooOol037oo00<007ooOol087oo00<007ooOol02Goo1000<Woo000dOol0
1000Oomoo`000goo0P000goo00@007ooOol0009oo`03001oogoo00]oo`03001oogoo02Moo`D003Io
o`00=7oo00@007ooOol000Qoo`04001oogoo0003Ool3000:Ool00`00Oomoo`0SOol4000kOol003Ao
o`04001oogoo0008Ool30006Ool00`00Oomoo`08Ool00`00Oomoo`0NOol5000oOol003Aoo`04001o
ogoo0008Ool00`00Oomoo`07Ool20008Ool00`00Oomoo`0JOol40014Ool003Eoo`8000Yoo`<000Mo
o`8000Moo`03001oogoo01Eoo`D004Qoo`00CGoo0P001Goo00<007ooOol04Goo1000CGoo001>Ool2
0004Ool00`00Oomoo`0=Ool4001AOol0051oo`05001oogooOol0000:Ool5001EOol0045oo`8000Ao
o`8000Aoo`80009oo`800003Ool007oo00Eoo`@005Yoo`00@7oo00@007ooOol000=oo`8000=oo`04
001oogoo0003Ool200000goo00000003001NOol0041oo`04001oogoo0008Ool01000Oomoo`0017oo
0P00Hgoo0010Ool01000Oomoo`002Goo0P00JWoo0010Ool01000Oomoo`0027oo00@007ooOol006Uo
o`00@Goo0P002Woo0P00JWoo002iOol0055oo`<006Eoo`00DWoo00<007ooOol0I7oo001BOol00`00
Oomoo`1TOol0059oo`03001oogoo06Aoo`00DWoo00<007ooOol0I7oo001AOol2001VOol00;Uoo`00
\
\>"],
  ImageRangeCache->{{{0, 184.688}, {200.5, 0}} -> {-0.0473313, -0.0577046, 
  0.00522764, 0.00522764}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Plot3D[If[S>95,
DownAndOutCallGamma[2,S,100,95,0.2,Log[1.1],Log[1.05],t],
UpAndOutCallGamma[2,S,100,95,0.2,Log[1.1],Log[1.05],t]], {S, 80, 120}, {t, \
0.0001, 1},
BoxRatios -> {1, 1, 1}, PlotPoints -> 40];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.0855 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0567234 1.16608 0 1.16608 [
[.07825 .23514 -11.8839 -9 ]
[.07825 .23514 .11606 0 ]
[.21247 .17844 -11.3335 -9 ]
[.21247 .17844 .66648 0 ]
[.3545 .11847 -16.1747 -9 ]
[.3545 .11847 1.82534 0 ]
[.50502 .05496 -15.349 -9 ]
[.50502 .05496 2.65097 0 ]
[.66483 -0.01242 -14.5234 -9 ]
[.66483 -0.01242 3.4766 0 ]
[.09301 .88495 -10.1462 0 ]
[.09301 .88495 7.85383 9 ]
[.17991 .94587 -9.90019 0 ]
[.17991 .94587 8.09981 9 ]
[.25944 1.00163 -9.68252 0 ]
[.25944 1.00163 8.31748 9 ]
[.33251 1.05285 -9.48854 0 ]
[.33251 1.05285 8.51146 9 ]
[.39987 1.10006 -3.10486 0 ]
[.39987 1.10006 2.89514 9 ]
[.07023 .28119 -30 -2.63444 ]
[.07023 .28119 0 6.36556 ]
[.04288 .45239 -6 -2.78697 ]
[.04288 .45239 0 6.21303 ]
[.01196 .64598 -24 -2.96159 ]
[.01196 .64598 0 6.03841 ]
[ 0 0 0 0 ]
[ 1 1.0855 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
[(80)] .07825 .23514 .98066 1 Mshowa
.22216 .18933 m
.227 .19478 L
s
[(90)] .21247 .17844 .88892 1 Mshowa
.36358 .12987 m
.36813 .13557 L
s
[(100)] .3545 .11847 .79718 1 Mshowa
.51343 .06687 m
.51763 .07282 L
s
[(110)] .50502 .05496 .70545 1 Mshowa
.67245 0 m
.67627 .00621 L
s
[(120)] .66483 -0.01242 .61371 1 Mshowa
.125 Mabswid
.11461 .23456 m
.11764 .23771 L
s
.14105 .22344 m
.14405 .22662 L
s
.16779 .2122 m
.17076 .21541 L
s
.19482 .20083 m
.19776 .20407 L
s
.2498 .17771 m
.25268 .18101 L
s
.27776 .16595 m
.2806 .16928 L
s
.30604 .15406 m
.30885 .15742 L
s
.33465 .14204 m
.33741 .14542 L
s
.39286 .11756 m
.39554 .12101 L
s
.42247 .10511 m
.42512 .10859 L
s
.45243 .09251 m
.45504 .09602 L
s
.48275 .07977 m
.48531 .08331 L
s
.54447 .05381 m
.54695 .05742 L
s
.57589 .0406 m
.57832 .04424 L
s
.60769 .02723 m
.61007 .0309 L
s
.63987 .0137 m
.64221 .0174 L
s
.25 Mabswid
0 .80374 m
.40037 1.0855 L
s
.09485 .87049 m
.09577 .86326 L
s
[(0.2)] .09301 .88495 .12735 -1 Mshowa
.18136 .93137 m
.18209 .92412 L
s
[(0.4)] .17991 .94587 .10002 -1 Mshowa
.26055 .98709 m
.2611 .97983 L
s
[(0.6)] .25944 1.00163 .07584 -1 Mshowa
.3333 1.03829 m
.33369 1.03102 L
s
[(0.8)] .33251 1.05285 .05428 -1 Mshowa
.40037 1.0855 m
.40063 1.07821 L
s
[(1)] .39987 1.10006 .03495 -1 Mshowa
.125 Mabswid
.02453 .821 m
.02518 .81668 L
s
.04852 .83789 m
.04914 .83356 L
s
.07196 .85438 m
.07254 .85004 L
s
.11722 .88623 m
.11774 .88189 L
s
.13908 .90162 m
.13958 .89727 L
s
.16046 .91666 m
.16092 .91231 L
s
.20181 .94576 m
.20221 .9414 L
s
.22181 .95983 m
.22219 .95548 L
s
.24138 .97361 m
.24174 .96925 L
s
.27931 1.0003 m
.27961 .99593 L
s
.29768 1.01322 m
.29796 1.00886 L
s
.31567 1.02589 m
.31593 1.02152 L
s
.35057 1.05045 m
.35079 1.04608 L
s
.3675 1.06236 m
.3677 1.058 L
s
.3841 1.07404 m
.38427 1.06967 L
s
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
.08369 .27561 m
.09043 .27282 L
s
[(-0.02)] .07023 .28119 1 -0.41457 Mshowa
.0565 .4472 m
.06331 .44461 L
s
[(0)] .04288 .45239 1 -0.38067 Mshowa
.02575 .64126 m
.03264 .63891 L
s
[(0.02)] .01196 .64598 1 -0.34187 Mshowa
.125 Mabswid
.07719 .31662 m
.08125 .31497 L
s
.0705 .35885 m
.07456 .35723 L
s
.06361 .40236 m
.06768 .40077 L
s
.04917 .49345 m
.05327 .49193 L
s
.04161 .54116 m
.04572 .53967 L
s
.03381 .59041 m
.03793 .58895 L
s
.01742 .69381 m
.02157 .69244 L
s
.00881 .74814 m
.01297 .74681 L
s
.25 Mabswid
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
.41799 .58158 m
.08846 .24555 L
s
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.41799 .58158 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.40037 1.0855 L
s
.40037 1.0855 m
.41799 .58158 L
s
0 0 m
1 0 L
1 1.0855 L
0 1.0855 L
closepath
clip
newpath
.5 Mabswid
.71 .783 .909 r
.4032 .78137 .4108 .78725 .42353 .78179 .41598 .77584 Metetra
.713 .782 .907 r
.41598 .77584 .42353 .78179 .4363 .7762 .4288 .77017 Metetra
.715 .781 .904 r
.4288 .77017 .4363 .7762 .44913 .77052 .44168 .76439 Metetra
.716 .78 .902 r
.44168 .76439 .44913 .77052 .46201 .76476 .45461 .75853 Metetra
.717 .778 .901 r
.45461 .75853 .46201 .76476 .47494 .75895 .46759 .75261 Metetra
.717 .776 .899 r
.46759 .75261 .47494 .75895 .48792 .75312 .48062 .74665 Metetra
.716 .774 .898 r
.48062 .74665 .48792 .75312 .50094 .74729 .49369 .7407 Metetra
.715 .771 .897 r
.49369 .7407 .50094 .74729 .51401 .74148 .50681 .73476 Metetra
.713 .768 .896 r
.50681 .73476 .51401 .74148 .52714 .73571 .51999 .72886 Metetra
.711 .766 .895 r
.51999 .72886 .52714 .73571 .54031 .73 .53321 .72302 Metetra
.709 .763 .895 r
.53321 .72302 .54031 .73 .55354 .72436 .54648 .71725 Metetra
.706 .76 .894 r
.54648 .71725 .55354 .72436 .56682 .7188 .55981 .71157 Metetra
.703 .757 .894 r
.55981 .71157 .56682 .7188 .58016 .71334 .5732 .706 Metetra
.699 .754 .894 r
.5732 .706 .58016 .71334 .59355 .70798 .58664 .70053 Metetra
.953 .708 .527 r
.58664 .70053 .59355 .70798 .60272 .53174 .59624 .5255 Metetra
.69 .193 0 r
.59624 .5255 .60272 .53174 .61616 .54874 .6097 .5428 Metetra
.642 .134 0 r
.6097 .5428 .61616 .54874 .6298 .56432 .62335 .55864 Metetra
.586 .068 0 r
.62335 .55864 .6298 .56432 .64362 .57849 .6372 .57305 Metetra
.521 0 0 r
.6372 .57305 .64362 .57849 .6576 .59128 .65121 .58603 Metetra
0 .082 .548 r
.65121 .58603 .6576 .59128 .67174 .60271 .66538 .59764 Metetra
0 .164 .622 r
.66538 .59764 .67174 .60271 .68601 .61283 .6797 .6079 Metetra
0 .248 .695 r
.6797 .6079 .68601 .61283 .70042 .62169 .69415 .61685 Metetra
0 .332 .762 r
.69415 .61685 .70042 .62169 .71494 .62933 .70871 .62456 Metetra
0 .412 .822 r
.70871 .62456 .71494 .62933 .72956 .6358 .72338 .63107 Metetra
.015 .485 .873 r
.72338 .63107 .72956 .6358 .74428 .64117 .73815 .63645 Metetra
.106 .55 .914 r
.73815 .63645 .74428 .64117 .75908 .6455 .753 .64076 Metetra
.19 .606 .945 r
.753 .64076 .75908 .6455 .77396 .64884 .76794 .64405 Metetra
.266 .652 .966 r
.76794 .64405 .77396 .64884 .78891 .65127 .78295 .6464 Metetra
.333 .69 .98 r
.78295 .6464 .78891 .65127 .80393 .65283 .79802 .64788 Metetra
.39 .719 .988 r
.79802 .64788 .80393 .65283 .81901 .6536 .81315 .64853 Metetra
.44 .742 .992 r
.81315 .64853 .81901 .6536 .83415 .65364 .82834 .64844 Metetra
.481 .759 .992 r
.82834 .64844 .83415 .65364 .84934 .653 .84359 .64766 Metetra
.517 .772 .989 r
.84359 .64766 .84934 .653 .86458 .65175 .85889 .64625 Metetra
.546 .781 .985 r
.85889 .64625 .86458 .65175 .87987 .64993 .87423 .64428 Metetra
.571 .787 .981 r
.87423 .64428 .87987 .64993 .89521 .64761 .88963 .64178 Metetra
.592 .791 .975 r
.88963 .64178 .89521 .64761 .91061 .64483 .90508 .63882 Metetra
.609 .794 .97 r
.90508 .63882 .91061 .64483 .92605 .64163 .92057 .63545 Metetra
.623 .795 .964 r
.92057 .63545 .92605 .64163 .94154 .63807 .93612 .63171 Metetra
.635 .795 .959 r
.93612 .63171 .94154 .63807 .95709 .63418 .95172 .62764 Metetra
.711 .785 .909 r
.39553 .77549 .4032 .78137 .41598 .77584 .40835 .76989 Metetra
.714 .784 .907 r
.40835 .76989 .41598 .77584 .4288 .77017 .42123 .76414 Metetra
.717 .783 .905 r
.42123 .76414 .4288 .77017 .44168 .76439 .43416 .75826 Metetra
.718 .782 .902 r
.43416 .75826 .44168 .76439 .45461 .75853 .44714 .75228 Metetra
.719 .78 .9 r
.44714 .75228 .45461 .75853 .46759 .75261 .46017 .74624 Metetra
.719 .778 .899 r
.46017 .74624 .46759 .75261 .48062 .74665 .47324 .74016 Metetra
.718 .775 .897 r
.47324 .74016 .48062 .74665 .49369 .7407 .48637 .73407 Metetra
.717 .772 .896 r
.48637 .73407 .49369 .7407 .50681 .73476 .49954 .728 Metetra
.716 .77 .895 r
.49954 .728 .50681 .73476 .51999 .72886 .51277 .72197 Metetra
.713 .767 .894 r
.51277 .72197 .51999 .72886 .53321 .72302 .52604 .71599 Metetra
.711 .764 .894 r
.52604 .71599 .53321 .72302 .54648 .71725 .53936 .71009 Metetra
.708 .761 .893 r
.53936 .71009 .54648 .71725 .55981 .71157 .55274 .70428 Metetra
.705 .758 .893 r
.55274 .70428 .55981 .71157 .5732 .706 .56617 .69858 Metetra
.701 .755 .893 r
.56617 .69858 .5732 .706 .58664 .70053 .57966 .69299 Metetra
.953 .708 .528 r
.57966 .69299 .58664 .70053 .59624 .5255 .5897 .51925 Metetra
.701 .205 0 r
.5897 .51925 .59624 .5255 .6097 .5428 .60316 .53686 Metetra
.655 .147 0 r
.60316 .53686 .6097 .5428 .62335 .55864 .61684 .55298 Metetra
.601 .083 0 r
.61684 .55298 .62335 .55864 .6372 .57305 .63071 .56764 Metetra
.538 .011 0 r
.63071 .56764 .6372 .57305 .65121 .58603 .64475 .58084 Metetra
.465 0 0 r
.64475 .58084 .65121 .58603 .66538 .59764 .65896 .59263 Metetra
0 .15 .605 r
.65896 .59263 .66538 .59764 .6797 .6079 .67332 .60303 Metetra
0 .235 .679 r
.67332 .60303 .6797 .6079 .69415 .61685 .68781 .61209 Metetra
0 .321 .75 r
.68781 .61209 .69415 .61685 .70871 .62456 .70242 .61987 Metetra
0 .404 .813 r
.70242 .61987 .70871 .62456 .72338 .63107 .71714 .62643 Metetra
.002 .48 .867 r
.71714 .62643 .72338 .63107 .73815 .63645 .73196 .63181 Metetra
.096 .548 .91 r
.73196 .63181 .73815 .63645 .753 .64076 .74687 .63609 Metetra
.183 .606 .943 r
.74687 .63609 .753 .64076 .76794 .64405 .76186 .63933 Metetra
.262 .654 .966 r
.76186 .63933 .76794 .64405 .78295 .6464 .77692 .64161 Metetra
.331 .693 .981 r
.77692 .64161 .78295 .6464 .79802 .64788 .79205 .64298 Metetra
.391 .723 .989 r
.79205 .64298 .79802 .64788 .81315 .64853 .80724 .64352 Metetra
.441 .747 .993 r
.80724 .64352 .81315 .64853 .82834 .64844 .82248 .64328 Metetra
.484 .764 .993 r
.82248 .64328 .82834 .64844 .84359 .64766 .83778 .64235 Metetra
.52 .777 .99 r
.83778 .64235 .84359 .64766 .85889 .64625 .85314 .64078 Metetra
.551 .786 .986 r
.85314 .64078 .85889 .64625 .87423 .64428 .86854 .63863 Metetra
.576 .792 .981 r
.86854 .63863 .87423 .64428 .88963 .64178 .88399 .63595 Metetra
.597 .796 .976 r
.88399 .63595 .88963 .64178 .90508 .63882 .89949 .6328 Metetra
.614 .798 .97 r
.89949 .6328 .90508 .63882 .92057 .63545 .91504 .62924 Metetra
.628 .799 .964 r
.91504 .62924 .92057 .63545 .93612 .63171 .93064 .62531 Metetra
.64 .798 .959 r
.93064 .62531 .93612 .63171 .95172 .62764 .9463 .62105 Metetra
.712 .786 .91 r
.38778 .76961 .39553 .77549 .40835 .76989 .40065 .76394 Metetra
.715 .786 .907 r
.40065 .76394 .40835 .76989 .42123 .76414 .41358 .75809 Metetra
.718 .785 .905 r
.41358 .75809 .42123 .76414 .43416 .75826 .42656 .75211 Metetra
.72 .783 .902 r
.42656 .75211 .43416 .75826 .44714 .75228 .43959 .74602 Metetra
.721 .781 .9 r
.43959 .74602 .44714 .75228 .46017 .74624 .45267 .73986 Metetra
.721 .779 .898 r
.45267 .73986 .46017 .74624 .47324 .74016 .4658 .73365 Metetra
.721 .777 .897 r
.4658 .73365 .47324 .74016 .48637 .73407 .47898 .72742 Metetra
.72 .774 .895 r
.47898 .72742 .48637 .73407 .49954 .728 .4922 .72121 Metetra
.718 .771 .894 r
.4922 .72121 .49954 .728 .51277 .72197 .50548 .71503 Metetra
.716 .768 .894 r
.50548 .71503 .51277 .72197 .52604 .71599 .5188 .70891 Metetra
.713 .765 .893 r
.5188 .70891 .52604 .71599 .53936 .71009 .53217 .70287 Metetra
.71 .761 .892 r
.53217 .70287 .53936 .71009 .55274 .70428 .5456 .69693 Metetra
.707 .758 .892 r
.5456 .69693 .55274 .70428 .56617 .69858 .55908 .6911 Metetra
.703 .755 .892 r
.55908 .6911 .56617 .69858 .57966 .69299 .57262 .68538 Metetra
.953 .708 .528 r
.57262 .68538 .57966 .69299 .5897 .51925 .58309 .51298 Metetra
.712 .217 0 r
.58309 .51298 .5897 .51925 .60316 .53686 .59657 .53093 Metetra
.668 .161 0 r
.59657 .53093 .60316 .53686 .61684 .55298 .61026 .54735 Metetra
.616 .097 0 r
.61026 .54735 .61684 .55298 .63071 .56764 .62415 .56228 Metetra
.555 .026 0 r
.62415 .56228 .63071 .56764 .64475 .58084 .63823 .57571 Metetra
.483 0 0 r
.63823 .57571 .64475 .58084 .65896 .59263 .65248 .58769 Metetra
0 .135 .587 r
.65248 .58769 .65896 .59263 .67332 .60303 .66687 .59824 Metetra
0 .222 .663 r
.66687 .59824 .67332 .60303 .68781 .61209 .68141 .60742 Metetra
0 .31 .736 r
.68141 .60742 .68781 .61209 .70242 .61987 .69607 .61528 Metetra
0 .395 .802 r
.69607 .61528 .70242 .61987 .71714 .62643 .71084 .62187 Metetra
0 .475 .859 r
.71084 .62187 .71714 .62643 .73196 .63181 .72571 .62726 Metetra
.085 .546 .905 r
.72571 .62726 .73196 .63181 .74687 .63609 .74067 .63151 Metetra
.176 .606 .94 r
.74067 .63151 .74687 .63609 .76186 .63933 .75571 .6347 Metetra
.257 .657 .965 r
.75571 .6347 .76186 .63933 .77692 .64161 .77083 .63689 Metetra
.329 .697 .981 r
.77083 .63689 .77692 .64161 .79205 .64298 .78602 .63815 Metetra
.391 .728 .99 r
.78602 .63815 .79205 .64298 .80724 .64352 .80126 .63855 Metetra
.443 .752 .994 r
.80126 .63855 .80724 .64352 .82248 .64328 .81656 .63818 Metetra
.488 .77 .994 r
.81656 .63818 .82248 .64328 .83778 .64235 .83192 .63708 Metetra
.525 .782 .991 r
.83192 .63708 .83778 .64235 .85314 .64078 .84733 .63533 Metetra
.555 .791 .987 r
.84733 .63533 .85314 .64078 .86854 .63863 .86278 .63299 Metetra
.581 .797 .981 r
.86278 .63299 .86854 .63863 .88399 .63595 .87829 .63012 Metetra
.602 .8 .976 r
.87829 .63012 .88399 .63595 .89949 .6328 .89385 .62677 Metetra
.619 .802 .97 r
.89385 .62677 .89949 .6328 .91504 .62924 .90945 .62301 Metetra
.633 .802 .964 r
.90945 .62301 .91504 .62924 .93064 .62531 .92511 .61887 Metetra
.645 .802 .958 r
.92511 .61887 .93064 .62531 .9463 .62105 .94081 .61441 Metetra
.713 .788 .911 r
.37994 .76373 .38778 .76961 .40065 .76394 .39287 .75798 Metetra
.717 .788 .908 r
.39287 .75798 .40065 .76394 .41358 .75809 .40585 .75205 Metetra
.72 .787 .905 r
.40585 .75205 .41358 .75809 .42656 .75211 .41888 .74596 Metetra
.722 .785 .902 r
.41888 .74596 .42656 .75211 .43959 .74602 .43196 .73976 Metetra
.723 .783 .9 r
.43196 .73976 .43959 .74602 .45267 .73986 .4451 .73347 Metetra
.723 .781 .898 r
.4451 .73347 .45267 .73986 .4658 .73365 .45828 .72712 Metetra
.723 .778 .896 r
.45828 .72712 .4658 .73365 .47898 .72742 .47151 .72075 Metetra
.722 .775 .895 r
.47151 .72075 .47898 .72742 .4922 .72121 .48479 .71439 Metetra
.72 .772 .894 r
.48479 .71439 .4922 .72121 .50548 .71503 .49812 .70806 Metetra
.718 .769 .893 r
.49812 .70806 .50548 .71503 .5188 .70891 .51149 .70179 Metetra
.715 .766 .892 r
.51149 .70179 .5188 .70891 .53217 .70287 .52491 .6956 Metetra
.712 .762 .892 r
.52491 .6956 .53217 .70287 .5456 .69693 .53839 .68951 Metetra
.709 .759 .891 r
.53839 .68951 .5456 .69693 .55908 .6911 .55192 .68354 Metetra
.705 .756 .891 r
.55192 .68354 .55908 .6911 .57262 .68538 .56551 .67769 Metetra
.953 .708 .529 r
.56551 .67769 .57262 .68538 .58309 .51298 .57641 .50671 Metetra
.724 .23 0 r
.57641 .50671 .58309 .51298 .59657 .53093 .5899 .52501 Metetra
.681 .175 0 r
.5899 .52501 .59657 .53093 .61026 .54735 .60361 .54176 Metetra
.631 .112 0 r
.60361 .54176 .61026 .54735 .62415 .56228 .61753 .55697 Metetra
.572 .043 0 r
.61753 .55697 .62415 .56228 .63823 .57571 .63164 .57065 Metetra
.503 0 0 r
.63164 .57065 .63823 .57571 .65248 .58769 .64592 .58283 Metetra
.423 0 0 r
.64592 .58283 .65248 .58769 .66687 .59824 .66036 .59355 Metetra
0 .207 .645 r
.66036 .59355 .66687 .59824 .68141 .60742 .67494 .60285 Metetra
0 .297 .72 r
.67494 .60285 .68141 .60742 .69607 .61528 .68965 .61079 Metetra
0 .386 .79 r
.68965 .61079 .69607 .61528 .71084 .62187 .70447 .61742 Metetra
0 .469 .85 r
.70447 .61742 .71084 .62187 .72571 .62726 .71939 .62282 Metetra
.074 .543 .9 r
.71939 .62282 .72571 .62726 .74067 .63151 .73441 .62704 Metetra
.168 .607 .938 r
.73441 .62704 .74067 .63151 .75571 .6347 .74951 .63016 Metetra
.253 .659 .964 r
.74951 .63016 .75571 .6347 .77083 .63689 .76468 .63225 Metetra
.327 .701 .981 r
.76468 .63225 .77083 .63689 .78602 .63815 .77992 .63339 Metetra
.392 .733 .991 r
.77992 .63339 .78602 .63815 .80126 .63855 .79522 .63366 Metetra
.446 .758 .995 r
.79522 .63366 .80126 .63855 .81656 .63818 .81058 .63312 Metetra
.491 .775 .995 r
.81058 .63312 .81656 .63818 .83192 .63708 .82599 .63184 Metetra
.529 .788 .992 r
.82599 .63184 .83192 .63708 .84733 .63533 .84146 .6299 Metetra
.56 .796 .987 r
.84146 .6299 .84733 .63533 .86278 .63299 .85697 .62736 Metetra
.586 .802 .982 r
.85697 .62736 .86278 .63299 .87829 .63012 .87253 .62429 Metetra
.607 .805 .976 r
.87253 .62429 .87829 .63012 .89385 .62677 .88814 .62073 Metetra
.624 .806 .97 r
.88814 .62073 .89385 .62677 .90945 .62301 .9038 .61675 Metetra
.638 .806 .963 r
.9038 .61675 .90945 .62301 .92511 .61887 .91951 .6124 Metetra
.65 .805 .958 r
.91951 .6124 .92511 .61887 .94081 .61441 .93527 .60773 Metetra
.714 .79 .911 r
.37203 .75784 .37994 .76373 .39287 .75798 .385 .75203 Metetra
.718 .79 .908 r
.385 .75203 .39287 .75798 .40585 .75205 .39804 .74601 Metetra
.721 .789 .905 r
.39804 .74601 .40585 .75205 .41888 .74596 .41112 .73982 Metetra
.724 .787 .903 r
.41112 .73982 .41888 .74596 .43196 .73976 .42426 .73349 Metetra
.725 .785 .9 r
.42426 .73349 .43196 .73976 .4451 .73347 .43745 .72707 Metetra
.726 .783 .898 r
.43745 .72707 .4451 .73347 .45828 .72712 .45069 .72058 Metetra
.726 .78 .896 r
.45069 .72058 .45828 .72712 .47151 .72075 .46397 .71406 Metetra
.725 .777 .894 r
.46397 .71406 .47151 .72075 .48479 .71439 .4773 .70754 Metetra
.723 .774 .893 r
.4773 .70754 .48479 .71439 .49812 .70806 .49068 .70105 Metetra
.721 .77 .892 r
.49068 .70105 .49812 .70806 .51149 .70179 .50411 .69462 Metetra
.718 .767 .891 r
.50411 .69462 .51149 .70179 .52491 .6956 .51759 .68828 Metetra
.715 .763 .89 r
.51759 .68828 .52491 .6956 .53839 .68951 .53111 .68203 Metetra
.711 .76 .89 r
.53111 .68203 .53839 .68951 .55192 .68354 .54469 .67591 Metetra
.707 .756 .89 r
.54469 .67591 .55192 .68354 .56551 .67769 .55833 .66992 Metetra
.953 .708 .53 r
.55833 .66992 .56551 .67769 .57641 .50671 .56967 .50043 Metetra
.735 .243 0 r
.56967 .50043 .57641 .50671 .5899 .52501 .58317 .51911 Metetra
.695 .189 0 r
.58317 .51911 .5899 .52501 .60361 .54176 .5969 .53621 Metetra
.647 .128 0 r
.5969 .53621 .60361 .54176 .61753 .55697 .61084 .55172 Metetra
.59 .059 0 r
.61084 .55172 .61753 .55697 .63164 .57065 .62498 .56567 Metetra
.522 0 0 r
.62498 .56567 .63164 .57065 .64592 .58283 .63929 .57807 Metetra
.443 0 0 r
.63929 .57807 .64592 .58283 .66036 .59355 .65377 .58897 Metetra
0 .192 .625 r
.65377 .58897 .66036 .59355 .67494 .60285 .6684 .5984 Metetra
0 .284 .703 r
.6684 .5984 .67494 .60285 .68965 .61079 .68316 .60642 Metetra
0 .375 .776 r
.68316 .60642 .68965 .61079 .70447 .61742 .69803 .6131 Metetra
0 .462 .84 r
.69803 .6131 .70447 .61742 .71939 .62282 .71301 .61849 Metetra
.061 .54 .893 r
.71301 .61849 .71939 .62282 .73441 .62704 .72808 .62267 Metetra
.159 .607 .934 r
.72808 .62267 .73441 .62704 .74951 .63016 .74324 .62572 Metetra
.248 .662 .963 r
.74324 .62572 .74951 .63016 .76468 .63225 .75847 .62772 Metetra
.326 .706 .982 r
.75847 .62772 .76468 .63225 .77992 .63339 .77376 .62873 Metetra
.392 .739 .992 r
.77376 .62873 .77992 .63339 .79522 .63366 .78912 .62884 Metetra
.449 .764 .996 r
.78912 .62884 .79522 .63366 .81058 .63312 .80454 .62812 Metetra
.495 .782 .996 r
.80454 .62812 .81058 .63312 .82599 .63184 .82001 .62666 Metetra
.534 .794 .993 r
.82001 .62666 .82599 .63184 .84146 .6299 .83553 .62451 Metetra
.566 .802 .988 r
.83553 .62451 .84146 .6299 .85697 .62736 .8511 .62176 Metetra
.592 .807 .982 r
.8511 .62176 .85697 .62736 .87253 .62429 .86671 .61846 Metetra
.613 .81 .976 r
.86671 .61846 .87253 .62429 .88814 .62073 .88238 .61468 Metetra
.63 .81 .969 r
.88238 .61468 .88814 .62073 .9038 .61675 .89809 .61047 Metetra
.644 .81 .963 r
.89809 .61047 .9038 .61675 .91951 .6124 .91385 .6059 Metetra
.655 .808 .957 r
.91385 .6059 .91951 .6124 .93527 .60773 .92967 .601 Metetra
.714 .792 .912 r
.36403 .75197 .37203 .75784 .385 .75203 .37706 .74609 Metetra
.719 .792 .909 r
.37706 .74609 .385 .75203 .39804 .74601 .39014 .73998 Metetra
.723 .791 .906 r
.39014 .73998 .39804 .74601 .41112 .73982 .40329 .73368 Metetra
.726 .79 .903 r
.40329 .73368 .41112 .73982 .42426 .73349 .41648 .72723 Metetra
.728 .788 .9 r
.41648 .72723 .42426 .73349 .43745 .72707 .42972 .72067 Metetra
.729 .785 .898 r
.42972 .72067 .43745 .72707 .45069 .72058 .44302 .71403 Metetra
.728 .782 .895 r
.44302 .71403 .45069 .72058 .46397 .71406 .45636 .70735 Metetra
.728 .779 .894 r
.45636 .70735 .46397 .71406 .4773 .70754 .46974 .70066 Metetra
.726 .775 .892 r
.46974 .70066 .4773 .70754 .49068 .70105 .48318 .69401 Metetra
.724 .772 .891 r
.48318 .69401 .49068 .70105 .50411 .69462 .49666 .68741 Metetra
.721 .768 .89 r
.49666 .68741 .50411 .69462 .51759 .68828 .51019 .6809 Metetra
.718 .764 .889 r
.51019 .6809 .51759 .68828 .53111 .68203 .52376 .67449 Metetra
.714 .76 .889 r
.52376 .67449 .53111 .68203 .54469 .67591 .5374 .66821 Metetra
.71 .757 .889 r
.5374 .66821 .54469 .67591 .55833 .66992 .55108 .66207 Metetra
.953 .709 .53 r
.55108 .66207 .55833 .66992 .56967 .50043 .56286 .49416 Metetra
.746 .256 0 r
.56286 .49416 .56967 .50043 .58317 .51911 .57636 .51324 Metetra
.708 .204 0 r
.57636 .51324 .58317 .51911 .5969 .53621 .59011 .53071 Metetra
.663 .144 0 r
.59011 .53071 .5969 .53621 .61084 .55172 .60407 .54655 Metetra
.608 .076 0 r
.60407 .54655 .61084 .55172 .62498 .56567 .61824 .56078 Metetra
.542 0 0 r
.61824 .56078 .62498 .56567 .63929 .57807 .6326 .57342 Metetra
.465 0 0 r
.6326 .57342 .63929 .57807 .65377 .58897 .64712 .5845 Metetra
0 .175 .603 r
.64712 .5845 .65377 .58897 .6684 .5984 .66179 .59407 Metetra
0 .27 .684 r
.66179 .59407 .6684 .5984 .68316 .60642 .67659 .60219 Metetra
0 .364 .761 r
.67659 .60219 .68316 .60642 .69803 .6131 .69152 .6089 Metetra
0 .454 .829 r
.69152 .6089 .69803 .6131 .71301 .61849 .70656 .6143 Metetra
.048 .536 .886 r
.70656 .6143 .71301 .61849 .72808 .62267 .72168 .61844 Metetra
.15 .607 .93 r
.72168 .61844 .72808 .62267 .74324 .62572 .7369 .62141 Metetra
.243 .665 .961 r
.7369 .62141 .74324 .62572 .75847 .62772 .75219 .62329 Metetra
.324 .711 .981 r
.75219 .62329 .75847 .62772 .77376 .62873 .76754 .62416 Metetra
.394 .745 .993 r
.76754 .62416 .77376 .62873 .78912 .62884 .78296 .6241 Metetra
.452 .771 .997 r
.78296 .6241 .78912 .62884 .80454 .62812 .79843 .6232 Metetra
.5 .789 .997 r
.79843 .6232 .80454 .62812 .82001 .62666 .81396 .62152 Metetra
.54 .801 .994 r
.81396 .62152 .82001 .62666 .83553 .62451 .82954 .61916 Metetra
.572 .808 .988 r
.82954 .61916 .83553 .62451 .8511 .62176 .84516 .61618 Metetra
.598 .813 .982 r
.84516 .61618 .8511 .62176 .86671 .61846 .86083 .61264 Metetra
.619 .815 .975 r
.86083 .61264 .86671 .61846 .88238 .61468 .87655 .60862 Metetra
.636 .815 .969 r
.87655 .60862 .88238 .61468 .89809 .61047 .89232 .60417 Metetra
.65 .814 .962 r
.89232 .60417 .89809 .61047 .91385 .6059 .90813 .59935 Metetra
.661 .812 .956 r
.90813 .59935 .91385 .6059 .92967 .601 .924 .59421 Metetra
.715 .794 .913 r
.35595 .74611 .36403 .75197 .37706 .74609 .36903 .74016 Metetra
.721 .794 .91 r
.36903 .74016 .37706 .74609 .39014 .73998 .38217 .73397 Metetra
.725 .793 .906 r
.38217 .73397 .39014 .73998 .40329 .73368 .39537 .72756 Metetra
.728 .792 .903 r
.39537 .72756 .40329 .73368 .41648 .72723 .40862 .72098 Metetra
.73 .79 .9 r
.40862 .72098 .41648 .72723 .42972 .72067 .42192 .71427 Metetra
.731 .787 .897 r
.42192 .71427 .42972 .72067 .44302 .71403 .43527 .70748 Metetra
.731 .784 .895 r
.43527 .70748 .44302 .71403 .45636 .70735 .44866 .70063 Metetra
.731 .781 .893 r
.44866 .70063 .45636 .70735 .46974 .70066 .46211 .69377 Metetra
.729 .777 .891 r
.46211 .69377 .46974 .70066 .48318 .69401 .47559 .68694 Metetra
.727 .773 .89 r
.47559 .68694 .48318 .69401 .49666 .68741 .48913 .68016 Metetra
.724 .769 .889 r
.48913 .68016 .49666 .68741 .51019 .6809 .50271 .67347 Metetra
.72 .765 .888 r
.50271 .67347 .51019 .6809 .52376 .67449 .51634 .66688 Metetra
.716 .761 .888 r
.51634 .66688 .52376 .67449 .5374 .66821 .53003 .66044 Metetra
.712 .757 .888 r
.53003 .66044 .5374 .66821 .55108 .66207 .54376 .65414 Metetra
.953 .709 .531 r
.54376 .65414 .55108 .66207 .56286 .49416 .55598 .48788 Metetra
.758 .27 0 r
.55598 .48788 .56286 .49416 .57636 .51324 .56949 .50741 Metetra
.722 .219 0 r
.56949 .50741 .57636 .51324 .59011 .53071 .58325 .52527 Metetra
.678 .161 0 r
.58325 .52527 .59011 .53071 .60407 .54655 .59723 .54147 Metetra
.626 .094 0 r
.59723 .54147 .60407 .54655 .61824 .56078 .61143 .55601 Metetra
.562 .019 0 r
.61143 .55601 .61824 .56078 .6326 .57342 .62582 .5689 Metetra
.487 0 0 r
.62582 .5689 .6326 .57342 .64712 .5845 .64039 .58018 Metetra
.399 0 0 r
.64039 .58018 .64712 .5845 .66179 .59407 .65511 .5899 Metetra
0 .254 .663 r
.65511 .5899 .66179 .59407 .67659 .60219 .66996 .5981 Metetra
0 .352 .744 r
.66996 .5981 .67659 .60219 .69152 .6089 .68494 .60486 Metetra
0 .446 .816 r
.68494 .60486 .69152 .6089 .70656 .6143 .70003 .61025 Metetra
.034 .532 .877 r
.70003 .61025 .70656 .6143 .72168 .61844 .71522 .61435 Metetra
.141 .607 .925 r
.71522 .61435 .72168 .61844 .7369 .62141 .73049 .61723 Metetra
.238 .668 .959 r
.73049 .61723 .7369 .62141 .75219 .62329 .74584 .61899 Metetra
.323 .716 .981 r
.74584 .61899 .75219 .62329 .76754 .62416 .76125 .6197 Metetra
.395 .752 .993 r
.76125 .6197 .76754 .62416 .78296 .6241 .77673 .61946 Metetra
.455 .778 .998 r
.77673 .61946 .78296 .6241 .79843 .6232 .79226 .61835 Metetra
.505 .796 .998 r
.79226 .61835 .79843 .6232 .81396 .62152 .80785 .61645 Metetra
.546 .808 .994 r
.80785 .61645 .81396 .62152 .82954 .61916 .82348 .61385 Metetra
.578 .815 .989 r
.82348 .61385 .82954 .61916 .84516 .61618 .83916 .61062 Metetra
.605 .819 .982 r
.83916 .61062 .84516 .61618 .86083 .61264 .85489 .60682 Metetra
.626 .82 .975 r
.85489 .60682 .86083 .61264 .87655 .60862 .87066 .60254 Metetra
.643 .819 .968 r
.87066 .60254 .87655 .60862 .89232 .60417 .88648 .59784 Metetra
.656 .818 .961 r
.88648 .59784 .89232 .60417 .90813 .59935 .90235 .59276 Metetra
.667 .815 .955 r
.90235 .59276 .90813 .59935 .924 .59421 .91827 .58737 Metetra
.716 .796 .915 r
.34778 .74026 .35595 .74611 .36903 .74016 .36091 .73426 Metetra
.722 .797 .911 r
.36091 .73426 .36903 .74016 .38217 .73397 .37411 .72798 Metetra
.727 .796 .907 r
.37411 .72798 .38217 .73397 .39537 .72756 .38736 .72146 Metetra
.73 .794 .903 r
.38736 .72146 .39537 .72756 .40862 .72098 .40067 .71475 Metetra
.733 .792 .9 r
.40067 .71475 .40862 .72098 .42192 .71427 .41403 .70789 Metetra
.734 .79 .897 r
.41403 .70789 .42192 .71427 .43527 .70748 .42744 .70093 Metetra
.734 .787 .894 r
.42744 .70093 .43527 .70748 .44866 .70063 .44089 .69391 Metetra
.734 .783 .892 r
.44089 .69391 .44866 .70063 .46211 .69377 .45439 .68686 Metetra
.732 .779 .89 r
.45439 .68686 .46211 .69377 .47559 .68694 .46794 .67984 Metetra
.73 .775 .889 r
.46794 .67984 .47559 .68694 .48913 .68016 .48153 .67287 Metetra
.727 .771 .888 r
.48153 .67287 .48913 .68016 .50271 .67347 .49517 .66598 Metetra
.723 .767 .887 r
.49517 .66598 .50271 .67347 .51634 .66688 .50885 .65922 Metetra
.719 .762 .886 r
.50885 .65922 .51634 .66688 .53003 .66044 .52259 .65259 Metetra
.715 .758 .886 r
.52259 .65259 .53003 .66044 .54376 .65414 .53637 .64612 Metetra
.953 .709 .531 r
.53637 .64612 .54376 .65414 .55598 .48788 .54902 .48161 Metetra
.769 .284 0 r
.54902 .48161 .55598 .48788 .56949 .50741 .56254 .50161 Metetra
.735 .235 0 r
.56254 .50161 .56949 .50741 .58325 .52527 .57631 .51991 Metetra
.694 .178 0 r
.57631 .51991 .58325 .52527 .59723 .54147 .59032 .53649 Metetra
.644 .113 0 r
.59032 .53649 .59723 .54147 .61143 .55601 .60455 .55136 Metetra
.583 .038 0 r
.60455 .55136 .61143 .55601 .62582 .5689 .61897 .56452 Metetra
.509 0 0 r
.61897 .56452 .62582 .5689 .64039 .58018 .63358 .57602 Metetra
.423 0 0 r
.63358 .57602 .64039 .58018 .65511 .5899 .64835 .58589 Metetra
0 .237 .64 r
.64835 .58589 .65511 .5899 .66996 .5981 .66326 .59419 Metetra
0 .338 .724 r
.66326 .59419 .66996 .5981 .68494 .60486 .67829 .601 Metetra
0 .437 .801 r
.67829 .601 .68494 .60486 .70003 .61025 .69344 .60638 Metetra
.019 .528 .867 r
.69344 .60638 .70003 .61025 .71522 .61435 .70868 .61042 Metetra
.13 .607 .919 r
.70868 .61042 .71522 .61435 .73049 .61723 .72402 .6132 Metetra
.232 .671 .956 r
.72402 .6132 .73049 .61723 .74584 .61899 .73942 .61482 Metetra
.321 .722 .98 r
.73942 .61482 .74584 .61899 .76125 .6197 .7549 .61536 Metetra
.397 .759 .993 r
.7549 .61536 .76125 .6197 .77673 .61946 .77043 .61492 Metetra
.46 .786 .999 r
.77043 .61492 .77673 .61946 .79226 .61835 .78603 .61359 Metetra
.511 .804 .998 r
.78603 .61359 .79226 .61835 .80785 .61645 .80167 .61145 Metetra
.552 .815 .995 r
.80167 .61145 .80785 .61645 .82348 .61385 .81736 .60858 Metetra
.586 .822 .989 r
.81736 .60858 .82348 .61385 .83916 .61062 .83309 .60508 Metetra
.612 .825 .982 r
.83309 .60508 .83916 .61062 .85489 .60682 .84888 .60102 Metetra
.633 .825 .974 r
.84888 .60102 .85489 .60682 .87066 .60254 .8647 .59646 Metetra
.65 .824 .967 r
.8647 .59646 .87066 .60254 .88648 .59784 .88058 .59147 Metetra
.663 .822 .96 r
.88058 .59147 .88648 .59784 .90235 .59276 .8965 .58612 Metetra
.673 .818 .953 r
.8965 .58612 .90235 .59276 .91827 .58737 .91247 .58046 Metetra
.716 .799 .916 r
.33952 .73444 .34778 .74026 .36091 .73426 .3527 .72838 Metetra
.723 .799 .912 r
.3527 .72838 .36091 .73426 .37411 .72798 .36596 .72201 Metetra
.728 .799 .907 r
.36596 .72201 .37411 .72798 .38736 .72146 .37927 .71539 Metetra
.733 .797 .904 r
.37927 .71539 .38736 .72146 .40067 .71475 .39263 .70855 Metetra
.735 .795 .9 r
.39263 .70855 .40067 .71475 .41403 .70789 .40605 .70153 Metetra
.737 .792 .897 r
.40605 .70153 .41403 .70789 .42744 .70093 .41952 .6944 Metetra
.738 .789 .894 r
.41952 .6944 .42744 .70093 .44089 .69391 .43304 .68719 Metetra
.737 .785 .892 r
.43304 .68719 .44089 .69391 .45439 .68686 .4466 .67995 Metetra
.736 .781 .889 r
.4466 .67995 .45439 .68686 .46794 .67984 .4602 .67272 Metetra
.733 .777 .888 r
.4602 .67272 .46794 .67984 .48153 .67287 .47385 .66554 Metetra
.73 .772 .887 r
.47385 .66554 .48153 .67287 .49517 .66598 .48754 .65846 Metetra
.727 .768 .886 r
.48754 .65846 .49517 .66598 .50885 .65922 .50128 .65149 Metetra
.722 .763 .885 r
.50128 .65149 .50885 .65922 .52259 .65259 .51507 .64467 Metetra
.718 .759 .885 r
.51507 .64467 .52259 .65259 .53637 .64612 .52891 .63801 Metetra
.953 .709 .532 r
.52891 .63801 .53637 .64612 .54902 .48161 .54199 .47535 Metetra
.781 .298 0 r
.54199 .47535 .54902 .48161 .56254 .50161 .55551 .49587 Metetra
.749 .25 0 r
.55551 .49587 .56254 .50161 .57631 .51991 .56929 .51463 Metetra
.71 .195 0 r
.56929 .51463 .57631 .51991 .59032 .53649 .58333 .53163 Metetra
.662 .132 0 r
.58333 .53163 .59032 .53649 .60455 .55136 .59758 .54685 Metetra
.603 .058 0 r
.59758 .54685 .60455 .55136 .61897 .56452 .61205 .56031 Metetra
.532 0 0 r
.61205 .56031 .61897 .56452 .63358 .57602 .6267 .57203 Metetra
.447 0 0 r
.6267 .57203 .63358 .57602 .64835 .58589 .64151 .58207 Metetra
.349 0 0 r
.64151 .58207 .64835 .58589 .66326 .59419 .65647 .59048 Metetra
0 .323 .702 r
.65647 .59048 .66326 .59419 .67829 .601 .67157 .59733 Metetra
0 .427 .784 r
.67157 .59733 .67829 .601 .69344 .60638 .68677 .6027 Metetra
.003 .522 .855 r
.68677 .6027 .69344 .60638 .70868 .61042 .70208 .60668 Metetra
.119 .606 .912 r
.70208 .60668 .70868 .61042 .72402 .6132 .71747 .60935 Metetra
.226 .675 .952 r
.71747 .60935 .72402 .6132 .73942 .61482 .73294 .61081 Metetra
.32 .728 .979 r
.73294 .61081 .73942 .61482 .7549 .61536 .74848 .61116 Metetra
.399 .767 .993 r
.74848 .61116 .7549 .61536 .77043 .61492 .76407 .6105 Metetra
.465 .794 .999 r
.76407 .6105 .77043 .61492 .78603 .61359 .77972 .60892 Metetra
.518 .812 .999 r
.77972 .60892 .78603 .61359 .80167 .61145 .79542 .60652 Metetra
.56 .823 .995 r
.79542 .60652 .80167 .61145 .81736 .60858 .81117 .60337 Metetra
.594 .829 .989 r
.81117 .60337 .81736 .60858 .83309 .60508 .82696 .59958 Metetra
.62 .831 .981 r
.82696 .59958 .83309 .60508 .84888 .60102 .8428 .59522 Metetra
.641 .831 .973 r
.8428 .59522 .84888 .60102 .8647 .59646 .85868 .59036 Metetra
.657 .829 .966 r
.85868 .59036 .8647 .59646 .88058 .59147 .87461 .58508 Metetra
.67 .826 .958 r
.87461 .58508 .88058 .59147 .8965 .58612 .89058 .57944 Metetra
.68 .822 .952 r
.89058 .57944 .8965 .58612 .91247 .58046 .90661 .57349 Metetra
.716 .801 .918 r
.33117 .72864 .33952 .73444 .3527 .72838 .34441 .72254 Metetra
.724 .802 .913 r
.34441 .72254 .3527 .72838 .36596 .72201 .35771 .71609 Metetra
.73 .802 .908 r
.35771 .71609 .36596 .72201 .37927 .71539 .37108 .70936 Metetra
.735 .8 .904 r
.37108 .70936 .37927 .71539 .39263 .70855 .38451 .70238 Metetra
.738 .798 .9 r
.38451 .70238 .39263 .70855 .40605 .70153 .39799 .6952 Metetra
.74 .795 .897 r
.39799 .6952 .40605 .70153 .41952 .6944 .41152 .68789 Metetra
.741 .792 .893 r
.41152 .68789 .41952 .6944 .43304 .68719 .4251 .68048 Metetra
.741 .788 .891 r
.4251 .68048 .43304 .68719 .4466 .67995 .43872 .67303 Metetra
.739 .784 .888 r
.43872 .67303 .4466 .67995 .4602 .67272 .45238 .66558 Metetra
.737 .779 .887 r
.45238 .66558 .4602 .67272 .47385 .66554 .46609 .65819 Metetra
.734 .774 .885 r
.46609 .65819 .47385 .66554 .48754 .65846 .47985 .65088 Metetra
.73 .769 .884 r
.47985 .65088 .48754 .65846 .50128 .65149 .49364 .6437 Metetra
.726 .764 .883 r
.49364 .6437 .50128 .65149 .51507 .64467 .50749 .63667 Metetra
.721 .759 .883 r
.50749 .63667 .51507 .64467 .52891 .63801 .52138 .62982 Metetra
.953 .71 .533 r
.52138 .62982 .52891 .63801 .54199 .47535 .53489 .46912 Metetra
.792 .312 0 r
.53489 .46912 .54199 .47535 .55551 .49587 .5484 .49019 Metetra
.762 .266 0 r
.5484 .49019 .55551 .49587 .56929 .51463 .5622 .50946 Metetra
.725 .213 0 r
.5622 .50946 .56929 .51463 .58333 .53163 .57625 .5269 Metetra
.68 .151 0 r
.57625 .5269 .58333 .53163 .59758 .54685 .59054 .54251 Metetra
.624 .079 0 r
.59054 .54251 .59758 .54685 .61205 .56031 .60504 .55628 Metetra
.555 0 0 r
.60504 .55628 .61205 .56031 .6267 .57203 .61973 .56826 Metetra
.472 0 0 r
.61973 .56826 .6267 .57203 .64151 .58207 .6346 .57848 Metetra
.374 0 0 r
.6346 .57848 .64151 .58207 .65647 .59048 .64961 .587 Metetra
0 .307 .677 r
.64961 .587 .65647 .59048 .67157 .59733 .66476 .59389 Metetra
0 .415 .765 r
.66476 .59389 .67157 .59733 .68677 .6027 .68003 .59924 Metetra
0 .516 .841 r
.68003 .59924 .68677 .6027 .70208 .60668 .6954 .60314 Metetra
.108 .605 .903 r
.6954 .60314 .70208 .60668 .71747 .60935 .71085 .60569 Metetra
.22 .678 .948 r
.71085 .60569 .71747 .60935 .73294 .61081 .72638 .60698 Metetra
.319 .734 .977 r
.72638 .60698 .73294 .61081 .74848 .61116 .74198 .60712 Metetra
.402 .775 .993 r
.74198 .60712 .74848 .61116 .76407 .6105 .75764 .60621 Metetra
.47 .803 .999 r
.75764 .60621 .76407 .6105 .77972 .60892 .77335 .60436 Metetra
.525 .821 .999 r
.77335 .60436 .77972 .60892 .79542 .60652 .78911 .60167 Metetra
.568 .831 .995 r
.78911 .60167 .79542 .60652 .81117 .60337 .80491 .59822 Metetra
.602 .837 .988 r
.80491 .59822 .81117 .60337 .82696 .59958 .82076 .59411 Metetra
.629 .838 .98 r
.82076 .59411 .82696 .59958 .8428 .59522 .83665 .58943 Metetra
.649 .837 .972 r
.83665 .58943 .8428 .59522 .85868 .59036 .85259 .58425 Metetra
.665 .834 .964 r
.85259 .58425 .85868 .59036 .87461 .58508 .86857 .57865 Metetra
.677 .83 .957 r
.86857 .57865 .87461 .58508 .89058 .57944 .8846 .5727 Metetra
.687 .825 .95 r
.8846 .5727 .89058 .57944 .90661 .57349 .90068 .56644 Metetra
.716 .804 .919 r
.32272 .72289 .33117 .72864 .34441 .72254 .33601 .71674 Metetra
.725 .805 .914 r
.33601 .71674 .34441 .72254 .35771 .71609 .34938 .71022 Metetra
.732 .805 .909 r
.34938 .71022 .35771 .71609 .37108 .70936 .36281 .70337 Metetra
.737 .804 .905 r
.36281 .70337 .37108 .70936 .38451 .70238 .3763 .69625 Metetra
.741 .801 .9 r
.3763 .69625 .38451 .70238 .39799 .6952 .38984 .68891 Metetra
.744 .798 .896 r
.38984 .68891 .39799 .6952 .41152 .68789 .40343 .68141 Metetra
.745 .795 .893 r
.40343 .68141 .41152 .68789 .4251 .68048 .41707 .67379 Metetra
.745 .791 .89 r
.41707 .67379 .4251 .68048 .43872 .67303 .43076 .66612 Metetra
.743 .786 .887 r
.43076 .66612 .43872 .67303 .45238 .66558 .44449 .65844 Metetra
.741 .781 .885 r
.44449 .65844 .45238 .66558 .46609 .65819 .45826 .65081 Metetra
.738 .776 .884 r
.45826 .65081 .46609 .65819 .47985 .65088 .47207 .64327 Metetra
.734 .771 .883 r
.47207 .64327 .47985 .65088 .49364 .6437 .48593 .63585 Metetra
.729 .766 .882 r
.48593 .63585 .49364 .6437 .50749 .63667 .49983 .6286 Metetra
.724 .76 .881 r
.49983 .6286 .50749 .63667 .52138 .62982 .51378 .62153 Metetra
.952 .71 .534 r
.51378 .62153 .52138 .62982 .53489 .46912 .52771 .4629 Metetra
.803 .327 0 r
.52771 .4629 .53489 .46912 .5484 .49019 .54122 .48458 Metetra
.775 .283 0 r
.54122 .48458 .5484 .49019 .5622 .50946 .55502 .5044 Metetra
.74 .231 0 r
.55502 .5044 .5622 .50946 .57625 .5269 .56909 .52233 Metetra
.698 .171 0 r
.56909 .52233 .57625 .5269 .59054 .54251 .58341 .53836 Metetra
.644 .1 0 r
.58341 .53836 .59054 .54251 .60504 .55628 .59795 .55248 Metetra
.578 .019 0 r
.59795 .55248 .60504 .55628 .61973 .56826 .61269 .56472 Metetra
.497 0 0 r
.61269 .56472 .61973 .56826 .6346 .57848 .6276 .57513 Metetra
.401 0 0 r
.6276 .57513 .6346 .57848 .64961 .587 .64267 .58376 Metetra
0 .289 .649 r
.64267 .58376 .64961 .587 .66476 .59389 .65788 .5907 Metetra
0 .402 .742 r
.65788 .5907 .66476 .59389 .68003 .59924 .67321 .59603 Metetra
0 .509 .825 r
.67321 .59603 .68003 .59924 .6954 .60314 .68864 .59984 Metetra
.096 .604 .892 r
.68864 .59984 .6954 .60314 .71085 .60569 .70416 .60224 Metetra
.214 .682 .942 r
.70416 .60224 .71085 .60569 .72638 .60698 .71975 .60334 Metetra
.318 .742 .974 r
.71975 .60334 .72638 .60698 .74198 .60712 .73541 .60325 Metetra
.406 .784 .992 r
.73541 .60325 .74198 .60712 .75764 .60621 .75113 .60207 Metetra
.477 .813 .999 r
.75113 .60207 .75764 .60621 .77335 .60436 .76691 .59993 Metetra
.533 .831 .999 r
.76691 .59993 .77335 .60436 .78911 .60167 .78272 .59691 Metetra
.578 .84 .995 r
.78272 .59691 .78911 .60167 .80491 .59822 .79859 .59313 Metetra
.612 .845 .988 r
.79859 .59313 .80491 .59822 .82076 .59411 .81449 .58868 Metetra
.638 .845 .979 r
.81449 .58868 .82076 .59411 .83665 .58943 .83044 .58365 Metetra
.659 .843 .971 r
.83044 .58365 .83665 .58943 .85259 .58425 .84643 .57813 Metetra
.674 .839 .962 r
.84643 .57813 .85259 .58425 .86857 .57865 .86246 .57218 Metetra
.686 .834 .954 r
.86246 .57218 .86857 .57865 .8846 .5727 .87854 .56589 Metetra
.694 .828 .947 r
.87854 .56589 .8846 .5727 .90068 .56644 .89467 .55932 Metetra
.716 .807 .921 r
.31417 .71717 .32272 .72289 .33601 .71674 .32752 .71099 Metetra
.726 .808 .916 r
.32752 .71099 .33601 .71674 .34938 .71022 .34094 .7044 Metetra
.734 .808 .911 r
.34094 .7044 .34938 .71022 .36281 .70337 .35443 .69745 Metetra
.74 .807 .905 r
.35443 .69745 .36281 .70337 .3763 .69625 .36799 .69019 Metetra
.744 .805 .901 r
.36799 .69019 .3763 .69625 .38984 .68891 .3816 .68268 Metetra
.747 .802 .896 r
.3816 .68268 .38984 .68891 .40343 .68141 .39526 .67497 Metetra
.749 .798 .893 r
.39526 .67497 .40343 .68141 .41707 .67379 .40896 .66714 Metetra
.749 .794 .889 r
.40896 .66714 .41707 .67379 .43076 .66612 .42271 .65923 Metetra
.748 .789 .886 r
.42271 .65923 .43076 .66612 .44449 .65844 .43651 .6513 Metetra
.745 .784 .884 r
.43651 .6513 .44449 .65844 .45826 .65081 .45034 .64342 Metetra
.742 .778 .882 r
.45034 .64342 .45826 .65081 .47207 .64327 .46421 .63562 Metetra
.738 .773 .881 r
.46421 .63562 .47207 .64327 .48593 .63585 .47813 .62795 Metetra
.733 .767 .88 r
.47813 .62795 .48593 .63585 .49983 .6286 .49209 .62046 Metetra
.728 .761 .879 r
.49209 .62046 .49983 .6286 .51378 .62153 .50609 .61316 Metetra
.952 .71 .535 r
.50609 .61316 .51378 .62153 .52771 .4629 .52044 .45673 Metetra
.814 .341 0 r
.52044 .45673 .52771 .4629 .54122 .48458 .53395 .47906 Metetra
.788 .299 0 r
.53395 .47906 .54122 .48458 .55502 .5044 .54776 .49949 Metetra
.755 .249 0 r
.54776 .49949 .55502 .5044 .56909 .52233 .56185 .51795 Metetra
.715 .191 0 r
.56185 .51795 .56909 .52233 .58341 .53836 .5762 .53443 Metetra
.664 .122 0 r
.5762 .53443 .58341 .53836 .59795 .55248 .59077 .54892 Metetra
.601 .042 0 r
.59077 .54892 .59795 .55248 .61269 .56472 .60555 .56145 Metetra
.523 0 0 r
.60555 .56145 .61269 .56472 .6276 .57513 .62052 .57207 Metetra
.427 0 0 r
.62052 .57207 .6276 .57513 .64267 .58376 .63565 .58082 Metetra
.315 0 0 r
.63565 .58082 .64267 .58376 .65788 .5907 .65092 .5878 Metetra
0 .388 .716 r
.65092 .5878 .65788 .5907 .67321 .59603 .66631 .5931 Metetra
0 .501 .806 r
.66631 .5931 .67321 .59603 .68864 .59984 .6818 .59681 Metetra
.082 .602 .88 r
.6818 .59681 .68864 .59984 .70416 .60224 .69739 .59904 Metetra
.208 .686 .935 r
.69739 .59904 .70416 .60224 .71975 .60334 .71305 .59993 Metetra
.318 .749 .971 r
.71305 .59993 .71975 .60334 .73541 .60325 .72877 .59957 Metetra
.41 .794 .991 r
.72877 .59957 .73541 .60325 .75113 .60207 .74456 .5981 Metetra
.484 .823 .999 r
.74456 .5981 .75113 .60207 .76691 .59993 .76039 .59562 Metetra
.543 .841 .999 r
.76039 .59562 .76691 .59993 .78272 .59691 .77627 .59225 Metetra
.588 .85 .994 r
.77627 .59225 .78272 .59691 .79859 .59313 .79219 .58811 Metetra
.622 .853 .986 r
.79219 .58811 .79859 .59313 .81449 .58868 .80815 .58328 Metetra
.649 .852 .978 r
.80815 .58328 .81449 .58868 .83044 .58365 .82415 .57788 Metetra
.668 .849 .969 r
.82415 .57788 .83044 .58365 .84643 .57813 .8402 .57198 Metetra
.683 .844 .96 r
.8402 .57198 .84643 .57813 .86246 .57218 .85628 .56567 Metetra
.694 .838 .952 r
.85628 .56567 .86246 .57218 .87854 .56589 .87241 .55902 Metetra
.702 .831 .944 r
.87241 .55902 .87854 .56589 .89467 .55932 .8886 .5521 Metetra
.716 .81 .924 r
.30553 .7115 .31417 .71717 .32752 .71099 .31893 .70531 Metetra
.726 .812 .918 r
.31893 .70531 .32752 .71099 .34094 .7044 .33241 .69866 Metetra
.735 .812 .912 r
.33241 .69866 .34094 .7044 .35443 .69745 .34596 .69161 Metetra
.742 .811 .906 r
.34596 .69161 .35443 .69745 .36799 .69019 .35958 .6842 Metetra
.748 .809 .901 r
.35958 .6842 .36799 .69019 .3816 .68268 .37326 .67651 Metetra
.751 .806 .896 r
.37326 .67651 .3816 .68268 .39526 .67497 .38699 .6686 Metetra
.753 .802 .892 r
.38699 .6686 .39526 .67497 .40896 .66714 .40076 .66053 Metetra
.753 .797 .888 r
.40076 .66053 .40896 .66714 .42271 .65923 .41458 .65236 Metetra
.752 .792 .885 r
.41458 .65236 .42271 .65923 .43651 .6513 .42844 .64417 Metetra
.75 .786 .883 r
.42844 .64417 .43651 .6513 .45034 .64342 .44234 .63601 Metetra
.747 .78 .88 r
.44234 .63601 .45034 .64342 .46421 .63562 .45628 .62794 Metetra
.742 .774 .879 r
.45628 .62794 .46421 .63562 .47813 .62795 .47025 .62 Metetra
.737 .768 .878 r
.47025 .62 .47813 .62795 .49209 .62046 .48427 .61224 Metetra
.732 .762 .877 r
.48427 .61224 .49209 .62046 .50609 .61316 .49834 .60469 Metetra
.952 .71 .536 r
.49834 .60469 .50609 .61316 .52044 .45673 .5131 .45059 Metetra
.824 .356 0 r
.5131 .45059 .52044 .45673 .53395 .47906 .5266 .47365 Metetra
.8 .316 0 r
.5266 .47365 .53395 .47906 .54776 .49949 .54041 .49473 Metetra
.77 .268 0 r
.54041 .49473 .54776 .49949 .56185 .51795 .55452 .51378 Metetra
.732 .211 0 r
.55452 .51378 .56185 .51795 .5762 .53443 .56889 .53075 Metetra
.684 .144 0 r
.56889 .53075 .5762 .53443 .59077 .54892 .5835 .54565 Metetra
.623 .066 0 r
.5835 .54565 .59077 .54892 .60555 .56145 .59833 .5585 Metetra
.548 0 0 r
.59833 .5585 .60555 .56145 .62052 .57207 .61335 .56933 Metetra
.454 0 0 r
.61335 .56933 .62052 .57207 .63565 .58082 .62854 .57821 Metetra
.342 0 0 r
.62854 .57821 .63565 .58082 .65092 .5878 .64387 .58523 Metetra
0 .371 .686 r
.64387 .58523 .65092 .5878 .66631 .5931 .65933 .59048 Metetra
0 .492 .783 r
.65933 .59048 .66631 .5931 .6818 .59681 .67489 .59407 Metetra
.069 .6 .864 r
.67489 .59407 .6818 .59681 .69739 .59904 .69054 .59612 Metetra
.202 .69 .926 r
.69054 .59612 .69739 .59904 .71305 .59993 .70627 .59676 Metetra
.319 .758 .966 r
.70627 .59676 .71305 .59993 .72877 .59957 .72206 .59611 Metetra
.416 .805 .988 r
.72206 .59611 .72877 .59957 .74456 .5981 .73791 .5943 Metetra
.493 .835 .998 r
.73791 .5943 .74456 .5981 .76039 .59562 .7538 .59146 Metetra
.553 .852 .998 r
.7538 .59146 .76039 .59562 .77627 .59225 .76974 .58771 Metetra
.599 .86 .993 r
.76974 .58771 .77627 .59225 .79219 .58811 .78572 .58316 Metetra
.634 .861 .985 r
.78572 .58316 .79219 .58811 .80815 .58328 .80174 .57793 Metetra
.66 .859 .976 r
.80174 .57793 .80815 .58328 .82415 .57788 .81779 .57211 Metetra
.679 .855 .966 r
.81779 .57211 .82415 .57788 .8402 .57198 .83389 .56581 Metetra
.693 .849 .957 r
.83389 .56581 .8402 .57198 .85628 .56567 .85003 .5591 Metetra
.703 .842 .949 r
.85003 .5591 .85628 .56567 .87241 .55902 .86621 .55207 Metetra
.71 .834 .941 r
.86621 .55207 .87241 .55902 .8886 .5521 .88244 .54478 Metetra
.714 .813 .927 r
.29678 .70589 .30553 .7115 .31893 .70531 .31023 .6997 Metetra
.727 .816 .92 r
.31023 .6997 .31893 .70531 .33241 .69866 .32377 .693 Metetra
.737 .816 .914 r
.32377 .693 .33241 .69866 .34596 .69161 .33739 .68585 Metetra
.745 .816 .908 r
.33739 .68585 .34596 .69161 .35958 .6842 .35107 .67831 Metetra
.751 .813 .902 r
.35107 .67831 .35958 .6842 .37326 .67651 .36482 .67043 Metetra
.755 .81 .896 r
.36482 .67043 .37326 .67651 .38699 .6686 .37862 .6623 Metetra
.757 .806 .892 r
.37862 .6623 .38699 .6686 .40076 .66053 .39246 .65398 Metetra
.758 .801 .888 r
.39246 .65398 .40076 .66053 .41458 .65236 .40635 .64555 Metetra
.757 .795 .884 r
.40635 .64555 .41458 .65236 .42844 .64417 .42028 .63707 Metetra
.755 .789 .881 r
.42028 .63707 .42844 .64417 .44234 .63601 .43425 .62861 Metetra
.752 .783 .879 r
.43425 .62861 .44234 .63601 .45628 .62794 .44826 .62023 Metetra
.747 .776 .877 r
.44826 .62023 .45628 .62794 .47025 .62 .4623 .612 Metetra
.742 .77 .876 r
.4623 .612 .47025 .62 .48427 .61224 .47638 .60395 Metetra
.736 .763 .875 r
.47638 .60395 .48427 .61224 .49834 .60469 .4905 .59613 Metetra
.952 .711 .537 r
.4905 .59613 .49834 .60469 .5131 .45059 .50568 .44451 Metetra
.835 .371 0 r
.50568 .44451 .5131 .45059 .5266 .47365 .51916 .46836 Metetra
.812 .332 0 r
.51916 .46836 .5266 .47365 .54041 .49473 .53297 .49017 Metetra
.784 .286 0 r
.53297 .49017 .54041 .49473 .55452 .51378 .54709 .50985 Metetra
.748 .232 0 r
.54709 .50985 .55452 .51378 .56889 .53075 .56149 .52737 Metetra
.703 .167 0 r
.56149 .52737 .56889 .53075 .5835 .54565 .57614 .54272 Metetra
.645 .09 0 r
.57614 .54272 .5835 .54565 .59833 .5585 .59102 .5559 Metetra
.572 0 0 r
.59102 .5559 .59833 .5585 .61335 .56933 .60609 .56697 Metetra
.481 0 0 r
.60609 .56697 .61335 .56933 .62854 .57821 .62134 .57599 Metetra
.37 0 0 r
.62134 .57599 .62854 .57821 .64387 .58523 .63674 .58303 Metetra
.239 0 0 r
.63674 .58303 .64387 .58523 .65933 .59048 .65226 .58822 Metetra
0 .481 .757 r
.65226 .58822 .65933 .59048 .67489 .59407 .66789 .59167 Metetra
.054 .597 .846 r
.66789 .59167 .67489 .59407 .69054 .59612 .68361 .59351 Metetra
.196 .694 .915 r
.68361 .59351 .69054 .59612 .70627 .59676 .69941 .59387 Metetra
.32 .766 .96 r
.69941 .59387 .70627 .59676 .72206 .59611 .71527 .59289 Metetra
.422 .816 .985 r
.71527 .59289 .72206 .59611 .73791 .5943 .73118 .59071 Metetra
.503 .847 .996 r
.73118 .59071 .73791 .5943 .7538 .59146 .74713 .58746 Metetra
.565 .863 .996 r
.74713 .58746 .7538 .59146 .76974 .58771 .76313 .58328 Metetra
.612 .87 .991 r
.76313 .58328 .76974 .58771 .78572 .58316 .77917 .57829 Metetra
.647 .87 .983 r
.77917 .57829 .78572 .58316 .80174 .57793 .79524 .57261 Metetra
.672 .867 .973 r
.79524 .57261 .80174 .57793 .81779 .57211 .81135 .56634 Metetra
.69 .861 .963 r
.81135 .56634 .81779 .57211 .83389 .56581 .8275 .5596 Metetra
.703 .853 .954 r
.8275 .5596 .83389 .56581 .85003 .5591 .84369 .55246 Metetra
.712 .845 .945 r
.84369 .55246 .85003 .5591 .86621 .55207 .85993 .54502 Metetra
.718 .837 .938 r
.85993 .54502 .86621 .55207 .88244 .54478 .87621 .53734 Metetra
.713 .817 .93 r
.28792 .70035 .29678 .70589 .31023 .6997 .30143 .69417 Metetra
.726 .82 .923 r
.30143 .69417 .31023 .6997 .32377 .693 .31503 .68744 Metetra
.738 .821 .916 r
.31503 .68744 .32377 .693 .33739 .68585 .32871 .6802 Metetra
.747 .82 .909 r
.32871 .6802 .33739 .68585 .35107 .67831 .34246 .67252 Metetra
.754 .818 .903 r
.34246 .67252 .35107 .67831 .36482 .67043 .35628 .66446 Metetra
.759 .815 .897 r
.35628 .66446 .36482 .67043 .37862 .6623 .37015 .6561 Metetra
.762 .81 .891 r
.37015 .6561 .37862 .6623 .39246 .65398 .38407 .64751 Metetra
.763 .805 .887 r
.38407 .64751 .39246 .65398 .40635 .64555 .39804 .63879 Metetra
.762 .799 .883 r
.39804 .63879 .40635 .64555 .42028 .63707 .41204 .63 Metetra
.76 .793 .879 r
.41204 .63 .42028 .63707 .43425 .62861 .42608 .62122 Metetra
.757 .786 .877 r
.42608 .62122 .43425 .62861 .44826 .62023 .44015 .61251 Metetra
.752 .779 .875 r
.44015 .61251 .44826 .62023 .4623 .612 .45426 .60395 Metetra
.747 .772 .873 r
.45426 .60395 .4623 .612 .47638 .60395 .46841 .59559 Metetra
.74 .764 .872 r
.46841 .59559 .47638 .60395 .4905 .59613 .48259 .58747 Metetra
.952 .711 .538 r
.48259 .58747 .4905 .59613 .50568 .44451 .49816 .43849 Metetra
.845 .386 .002 r
.49816 .43849 .50568 .44451 .51916 .46836 .51163 .46322 Metetra
.824 .349 0 r
.51163 .46322 .51916 .46836 .53297 .49017 .52544 .48582 Metetra
.797 .305 0 r
.52544 .48582 .53297 .49017 .54709 .50985 .53957 .50621 Metetra
.763 .252 0 r
.53957 .50621 .54709 .50985 .56149 .52737 .554 .52433 Metetra
.721 .19 0 r
.554 .52433 .56149 .52737 .57614 .54272 .56869 .54017 Metetra
.666 .114 0 r
.56869 .54017 .57614 .54272 .59102 .5559 .58361 .55372 Metetra
.596 .025 0 r
.58361 .55372 .59102 .5559 .60609 .56697 .59874 .56504 Metetra
.507 0 0 r
.59874 .56504 .60609 .56697 .62134 .57599 .61405 .57419 Metetra
.397 0 0 r
.61405 .57419 .62134 .57599 .63674 .58303 .62951 .58127 Metetra
.265 0 0 r
.62951 .58127 .63674 .58303 .65226 .58822 .64511 .58638 Metetra
0 .468 .726 r
.64511 .58638 .65226 .58822 .66789 .59167 .66081 .58966 Metetra
.04 .593 .824 r
.66081 .58966 .66789 .59167 .68361 .59351 .6766 .59125 Metetra
.189 .698 .901 r
.6766 .59125 .68361 .59351 .69941 .59387 .69246 .59129 Metetra
.321 .776 .952 r
.69246 .59129 .69941 .59387 .71527 .59289 .70839 .58993 Metetra
.43 .828 .981 r
.70839 .58993 .71527 .59289 .73118 .59071 .72437 .58733 Metetra
.515 .86 .993 r
.72437 .58733 .73118 .59071 .74713 .58746 .74039 .58363 Metetra
.579 .875 .994 r
.74039 .58363 .74713 .58746 .76313 .58328 .75645 .57897 Metetra
.626 .881 .988 r
.75645 .57897 .76313 .58328 .77917 .57829 .77255 .5735 Metetra
.66 .879 .98 r
.77255 .5735 .77917 .57829 .79524 .57261 .78867 .56732 Metetra
.685 .874 .97 r
.78867 .56732 .79524 .57261 .81135 .56634 .80484 .56057 Metetra
.702 .866 .96 r
.80484 .56057 .81135 .56634 .8275 .5596 .82104 .55335 Metetra
.714 .858 .95 r
.82104 .55335 .8275 .5596 .84369 .55246 .83728 .54575 Metetra
.722 .848 .941 r
.83728 .54575 .84369 .55246 .85993 .54502 .85356 .53786 Metetra
.727 .839 .934 r
.85356 .53786 .85993 .54502 .87621 .53734 .86989 .52976 Metetra
.71 .82 .934 r
.27896 .69488 .28792 .70035 .30143 .69417 .29251 .68875 Metetra
.726 .824 .926 r
.29251 .68875 .30143 .69417 .31503 .68744 .30617 .682 Metetra
.739 .826 .918 r
.30617 .682 .31503 .68744 .32871 .6802 .31992 .67469 Metetra
.75 .826 .911 r
.31992 .67469 .32871 .6802 .34246 .67252 .33374 .66687 Metetra
.758 .824 .904 r
.33374 .66687 .34246 .67252 .35628 .66446 .34763 .65862 Metetra
.764 .82 .897 r
.34763 .65862 .35628 .66446 .37015 .6561 .36158 .65002 Metetra
.767 .816 .891 r
.36158 .65002 .37015 .6561 .38407 .64751 .37558 .64115 Metetra
.769 .81 .886 r
.37558 .64115 .38407 .64751 .39804 .63879 .38962 .63211 Metetra
.768 .803 .881 r
.38962 .63211 .39804 .63879 .41204 .63 .4037 .62298 Metetra
.766 .796 .878 r
.4037 .62298 .41204 .63 .42608 .62122 .41781 .61385 Metetra
.763 .789 .874 r
.41781 .61385 .42608 .62122 .44015 .61251 .43196 .60479 Metetra
.758 .781 .872 r
.43196 .60479 .44015 .61251 .45426 .60395 .44614 .59587 Metetra
.752 .773 .87 r
.44614 .59587 .45426 .60395 .46841 .59559 .46035 .58716 Metetra
.745 .766 .869 r
.46035 .58716 .46841 .59559 .48259 .58747 .4746 .57871 Metetra
.951 .712 .539 r
.4746 .57871 .48259 .58747 .49816 .43849 .49056 .43256 Metetra
.854 .401 .026 r
.49056 .43256 .49816 .43849 .51163 .46322 .504 .45825 Metetra
.835 .365 .001 r
.504 .45825 .51163 .46322 .52544 .48582 .51781 .48174 Metetra
.81 .323 0 r
.51781 .48174 .52544 .48582 .53957 .50621 .53195 .50291 Metetra
.778 .273 0 r
.53195 .50291 .53957 .50621 .554 .52433 .5464 .52169 Metetra
.737 .212 0 r
.5464 .52169 .554 .52433 .56869 .54017 .56113 .53806 Metetra
.685 .139 0 r
.56113 .53806 .56869 .54017 .58361 .55372 .5761 .55202 Metetra
.618 .051 0 r
.5761 .55202 .58361 .55372 .59874 .56504 .59129 .56361 Metetra
.532 0 0 r
.59129 .56361 .59874 .56504 .61405 .57419 .60666 .5729 Metetra
.424 0 0 r
.60666 .5729 .61405 .57419 .62951 .58127 .6222 .57999 Metetra
.292 0 0 r
.6222 .57999 .62951 .58127 .64511 .58638 .63786 .58501 Metetra
.139 0 0 r
.63786 .58501 .64511 .58638 .66081 .58966 .65364 .58809 Metetra
.025 .588 .798 r
.65364 .58809 .66081 .58966 .6766 .59125 .6695 .58939 Metetra
.184 .701 .884 r
.6695 .58939 .6766 .59125 .69246 .59129 .68544 .58906 Metetra
.324 .786 .942 r
.68544 .58906 .69246 .59129 .70839 .58993 .70144 .58728 Metetra
.44 .841 .975 r
.70144 .58728 .70839 .58993 .72437 .58733 .71748 .5842 Metetra
.528 .873 .989 r
.71748 .5842 .72437 .58733 .74039 .58363 .73357 .57999 Metetra
.594 .888 .991 r
.73357 .57999 .74039 .58363 .75645 .57897 .74969 .5748 Metetra
.642 .892 .985 r
.74969 .5748 .75645 .57897 .77255 .5735 .76584 .56879 Metetra
.675 .888 .976 r
.76584 .56879 .77255 .5735 .78867 .56732 .78202 .56207 Metetra
.699 .881 .965 r
.78202 .56207 .78867 .56732 .80484 .56057 .79824 .55479 Metetra
.715 .872 .955 r
.79824 .55479 .80484 .56057 .82104 .55335 .81449 .54704 Metetra
.726 .862 .945 r
.81449 .54704 .82104 .55335 .83728 .54575 .83078 .53894 Metetra
.733 .851 .937 r
.83078 .53894 .83728 .54575 .85356 .53786 .84711 .53058 Metetra
.736 .841 .929 r
.84711 .53058 .85356 .53786 .86989 .52976 .86349 .52203 Metetra
.707 .824 .938 r
.26987 .68949 .27896 .69488 .29251 .68875 .28348 .68343 Metetra
.724 .829 .93 r
.28348 .68343 .29251 .68875 .30617 .682 .2972 .6767 Metetra
.74 .832 .921 r
.2972 .6767 .30617 .682 .31992 .67469 .31101 .66932 Metetra
.752 .832 .913 r
.31101 .66932 .31992 .67469 .33374 .66687 .3249 .66138 Metetra
.761 .83 .905 r
.3249 .66138 .33374 .66687 .34763 .65862 .33887 .65293 Metetra
.768 .826 .898 r
.33887 .65293 .34763 .65862 .36158 .65002 .3529 .64408 Metetra
.773 .821 .891 r
.3529 .64408 .36158 .65002 .37558 .64115 .36698 .63492 Metetra
.774 .815 .885 r
.36698 .63492 .37558 .64115 .38962 .63211 .3811 .62554 Metetra
.774 .808 .88 r
.3811 .62554 .38962 .63211 .4037 .62298 .39526 .61605 Metetra
.772 .8 .876 r
.39526 .61605 .4037 .62298 .41781 .61385 .40946 .60653 Metetra
.769 .792 .872 r
.40946 .60653 .41781 .61385 .43196 .60479 .42368 .59707 Metetra
.764 .784 .869 r
.42368 .59707 .43196 .60479 .44614 .59587 .43793 .58776 Metetra
.758 .776 .868 r
.43793 .58776 .44614 .59587 .46035 .58716 .45222 .57867 Metetra
.751 .767 .866 r
.45222 .57867 .46035 .58716 .4746 .57871 .46653 .56985 Metetra
.951 .712 .54 r
.46653 .56985 .4746 .57871 .49056 .43256 .48287 .42672 Metetra
.864 .416 .05 r
.48287 .42672 .49056 .43256 .504 .45825 .49628 .45349 Metetra
.845 .382 .029 r
.49628 .45349 .504 .45825 .51781 .48174 .51008 .47796 Metetra
.821 .341 .001 r
.51008 .47796 .51781 .48174 .53195 .50291 .52422 .49999 Metetra
.791 .293 0 r
.52422 .49999 .53195 .50291 .5464 .52169 .5387 .51951 Metetra
.753 .234 0 r
.5387 .51951 .5464 .52169 .56113 .53806 .55346 .53647 Metetra
.703 .163 0 r
.55346 .53647 .56113 .53806 .5761 .55202 .56848 .55087 Metetra
.639 .077 0 r
.56848 .55087 .5761 .55202 .59129 .56361 .58373 .56275 Metetra
.556 0 0 r
.58373 .56275 .59129 .56361 .60666 .5729 .59917 .57219 Metetra
.45 0 0 r
.59917 .57219 .60666 .5729 .6222 .57999 .61478 .57928 Metetra
.317 0 0 r
.61478 .57928 .6222 .57999 .63786 .58501 .63052 .58417 Metetra
.161 0 0 r
.63052 .58417 .63786 .58501 .65364 .58809 .64637 .58701 Metetra
.01 .581 .767 r
.64637 .58701 .65364 .58809 .6695 .58939 .66231 .58797 Metetra
.178 .704 .863 r
.66231 .58797 .6695 .58939 .68544 .58906 .67833 .58722 Metetra
.329 .796 .93 r
.67833 .58722 .68544 .58906 .70144 .58728 .69439 .58495 Metetra
.451 .855 .968 r
.69439 .58495 .70144 .58728 .71748 .5842 .71051 .58134 Metetra
.544 .888 .984 r
.71051 .58134 .71748 .5842 .73357 .57999 .72666 .57656 Metetra
.611 .901 .986 r
.72666 .57656 .73357 .57999 .74969 .5748 .74284 .57078 Metetra
.659 .903 .98 r
.74284 .57078 .74969 .5748 .76584 .56879 .75905 .56415 Metetra
.692 .897 .971 r
.75905 .56415 .76584 .56879 .78202 .56207 .77529 .55684 Metetra
.714 .888 .96 r
.77529 .55684 .78202 .56207 .79824 .55479 .79156 .54897 Metetra
.729 .877 .95 r
.79156 .54897 .79824 .55479 .81449 .54704 .80786 .54066 Metetra
.738 .865 .94 r
.80786 .54066 .81449 .54704 .83078 .53894 .82419 .53202 Metetra
.743 .853 .931 r
.82419 .53202 .83078 .53894 .84711 .53058 .84057 .52314 Metetra
.746 .842 .923 r
.84057 .52314 .84711 .53058 .86349 .52203 .857 .51411 Metetra
.702 .828 .943 r
.26068 .68419 .26987 .68949 .28348 .68343 .27433 .67825 Metetra
.722 .834 .934 r
.27433 .67825 .28348 .68343 .2972 .6767 .2881 .67155 Metetra
.74 .838 .925 r
.2881 .67155 .2972 .6767 .31101 .66932 .30198 .66414 Metetra
.754 .838 .916 r
.30198 .66414 .31101 .66932 .3249 .66138 .31594 .65607 Metetra
.765 .837 .907 r
.31594 .65607 .3249 .66138 .33887 .65293 .32999 .64744 Metetra
.773 .833 .898 r
.32999 .64744 .33887 .65293 .3529 .64408 .3441 .63833 Metetra
.778 .828 .891 r
.3441 .63833 .3529 .64408 .36698 .63492 .35826 .62885 Metetra
.781 .821 .884 r
.35826 .62885 .36698 .63492 .3811 .62554 .37247 .61911 Metetra
.781 .813 .878 r
.37247 .61911 .3811 .62554 .39526 .61605 .38672 .60921 Metetra
.779 .805 .873 r
.38672 .60921 .39526 .61605 .40946 .60653 .401 .59927 Metetra
.776 .796 .869 r
.401 .59927 .40946 .60653 .42368 .59707 .41531 .58938 Metetra
.771 .787 .866 r
.41531 .58938 .42368 .59707 .43793 .58776 .42964 .57963 Metetra
.764 .778 .864 r
.42964 .57963 .43793 .58776 .45222 .57867 .444 .57011 Metetra
.757 .769 .863 r
.444 .57011 .45222 .57867 .46653 .56985 .45838 .56088 Metetra
.951 .713 .542 r
.45838 .56088 .46653 .56985 .48287 .42672 .47508 .42101 Metetra
.872 .43 .074 r
.47508 .42101 .48287 .42672 .49628 .45349 .48846 .44898 Metetra
.855 .398 .057 r
.48846 .44898 .49628 .45349 .51008 .47796 .50223 .47454 Metetra
.832 .359 .033 r
.50223 .47454 .51008 .47796 .52422 .49999 .51639 .49754 Metetra
.803 .313 .002 r
.51639 .49754 .52422 .49999 .5387 .51951 .53088 .51788 Metetra
.766 .256 0 r
.53088 .51788 .5387 .51951 .55346 .53647 .54568 .53549 Metetra
.719 .187 0 r
.54568 .53549 .55346 .53647 .56848 .55087 .56076 .55038 Metetra
.657 .104 0 r
.56076 .55038 .56848 .55087 .58373 .56275 .57607 .56257 Metetra
.577 .002 0 r
.57607 .56257 .58373 .56275 .59917 .57219 .59158 .57214 Metetra
.473 0 0 r
.59158 .57214 .59917 .57219 .61478 .57928 .60726 .57922 Metetra
.342 0 0 r
.60726 .57922 .61478 .57928 .63052 .58417 .62308 .58396 Metetra
.183 0 0 r
.62308 .58396 .63052 .58417 .64637 .58701 .63901 .58651 Metetra
0 .573 .731 r
.63901 .58651 .64637 .58701 .66231 .58797 .65503 .58707 Metetra
.174 .707 .839 r
.65503 .58707 .66231 .58797 .67833 .58722 .67112 .58583 Metetra
.335 .806 .915 r
.67112 .58583 .67833 .58722 .69439 .58495 .68727 .583 Metetra
.465 .87 .958 r
.68727 .583 .69439 .58495 .71051 .58134 .70345 .57877 Metetra
.562 .903 .977 r
.70345 .57877 .71051 .58134 .72666 .57656 .71966 .57334 Metetra
.63 .915 .98 r
.71966 .57334 .72666 .57656 .74284 .57078 .73591 .56689 Metetra
.678 .914 .975 r
.73591 .56689 .74284 .57078 .75905 .56415 .75218 .5596 Metetra
.709 .906 .965 r
.75218 .5596 .75905 .56415 .77529 .55684 .76847 .55162 Metetra
.73 .894 .954 r
.76847 .55162 .77529 .55684 .79156 .54897 .78479 .5431 Metetra
.743 .881 .943 r
.78479 .5431 .79156 .54897 .80786 .54066 .80114 .53418 Metetra
.75 .868 .933 r
.80114 .53418 .80786 .54066 .82419 .53202 .81752 .52495 Metetra
.754 .854 .925 r
.81752 .52495 .82419 .53202 .84057 .52314 .83394 .51552 Metetra
.755 .842 .917 r
.83394 .51552 .84057 .52314 .857 .51411 .85041 .50598 Metetra
.695 .832 .948 r
.25135 .67899 .26068 .68419 .27433 .67825 .26505 .67321 Metetra
.718 .84 .939 r
.26505 .67321 .27433 .67825 .2881 .67155 .27888 .66659 Metetra
.739 .844 .929 r
.27888 .66659 .2881 .67155 .30198 .66414 .29282 .65916 Metetra
.756 .846 .919 r
.29282 .65916 .30198 .66414 .31594 .65607 .30686 .651 Metetra
.769 .844 .909 r
.30686 .651 .31594 .65607 .32999 .64744 .32098 .64218 Metetra
.778 .841 .899 r
.32098 .64218 .32999 .64744 .3441 .63833 .33518 .63281 Metetra
.785 .835 .891 r
.33518 .63281 .3441 .63833 .35826 .62885 .34944 .62299 Metetra
.788 .828 .883 r
.34944 .62299 .35826 .62885 .37247 .61911 .36374 .61285 Metetra
.789 .819 .877 r
.36374 .61285 .37247 .61911 .38672 .60921 .37808 .60252 Metetra
.787 .81 .871 r
.37808 .60252 .38672 .60921 .401 .59927 .39245 .5921 Metetra
.783 .801 .867 r
.39245 .5921 .401 .59927 .41531 .58938 .40684 .58173 Metetra
.778 .791 .863 r
.40684 .58173 .41531 .58938 .42964 .57963 .42125 .57149 Metetra
.771 .78 .861 r
.42125 .57149 .42964 .57963 .444 .57011 .43569 .56149 Metetra
.763 .77 .859 r
.43569 .56149 .444 .57011 .45838 .56088 .45015 .55182 Metetra
.951 .713 .543 r
.45015 .55182 .45838 .56088 .47508 .42101 .46719 .41543 Metetra
.88 .445 .099 r
.46719 .41543 .47508 .42101 .48846 .44898 .48052 .44475 Metetra
.863 .414 .085 r
.48052 .44475 .48846 .44898 .50223 .47454 .49428 .47155 Metetra
.842 .377 .066 r
.49428 .47155 .50223 .47454 .51639 .49754 .50843 .49563 Metetra
.814 .332 .038 r
.50843 .49563 .51639 .49754 .53088 .51788 .52294 .51689 Metetra
.778 .277 .002 r
.52294 .51689 .53088 .51788 .54568 .53549 .53778 .53523 Metetra
.732 .211 0 r
.53778 .53523 .54568 .53549 .56076 .55038 .55291 .55064 Metetra
.673 .129 0 r
.55291 .55064 .56076 .55038 .57607 .56257 .56829 .56316 Metetra
.595 .03 0 r
.56829 .56316 .57607 .56257 .59158 .57214 .58387 .57288 Metetra
.494 0 0 r
.58387 .57288 .59158 .57214 .60726 .57922 .59963 .57992 Metetra
.364 0 0 r
.59963 .57992 .60726 .57922 .62308 .58396 .61554 .58445 Metetra
.203 0 0 r
.61554 .58445 .62308 .58396 .63901 .58651 .63155 .58665 Metetra
.018 0 0 r
.63155 .58665 .63901 .58651 .65503 .58707 .64766 .58674 Metetra
.172 .708 .809 r
.64766 .58674 .65503 .58707 .67112 .58583 .66383 .58493 Metetra
.343 .817 .896 r
.66383 .58493 .67112 .58583 .68727 .583 .68004 .58145 Metetra
.481 .885 .946 r
.68004 .58145 .68727 .583 .70345 .57877 .6963 .57652 Metetra
.582 .918 .968 r
.6963 .57652 .70345 .57877 .71966 .57334 .71258 .57035 Metetra
.652 .928 .973 r
.71258 .57035 .71966 .57334 .73591 .56689 .72889 .56315 Metetra
.698 .925 .967 r
.72889 .56315 .73591 .56689 .75218 .5596 .74521 .55511 Metetra
.728 .914 .958 r
.74521 .55511 .75218 .5596 .76847 .55162 .76156 .5464 Metetra
.747 .9 .947 r
.76156 .5464 .76847 .55162 .78479 .5431 .77792 .53717 Metetra
.757 .885 .936 r
.77792 .53717 .78479 .5431 .80114 .53418 .79432 .52756 Metetra
.763 .869 .926 r
.79432 .52756 .80114 .53418 .81752 .52495 .81075 .5177 Metetra
.765 .855 .917 r
.81075 .5177 .81752 .52495 .83394 .51552 .82722 .50768 Metetra
.764 .841 .91 r
.82722 .50768 .83394 .51552 .85041 .50598 .84373 .4976 Metetra
.685 .835 .955 r
.24191 .67388 .25135 .67899 .26505 .67321 .25564 .66833 Metetra
.713 .845 .945 r
.25564 .66833 .26505 .67321 .27888 .66659 .26952 .66183 Metetra
.737 .852 .934 r
.26952 .66183 .27888 .66659 .29282 .65916 .28352 .65444 Metetra
.756 .854 .923 r
.28352 .65444 .29282 .65916 .30686 .651 .29764 .6462 Metetra
.772 .853 .911 r
.29764 .6462 .30686 .651 .32098 .64218 .31185 .6372 Metetra
.783 .849 .901 r
.31185 .6372 .32098 .64218 .33518 .63281 .32613 .62755 Metetra
.791 .843 .891 r
.32613 .62755 .33518 .63281 .34944 .62299 .34048 .61738 Metetra
.795 .835 .882 r
.34048 .61738 .34944 .62299 .36374 .61285 .35488 .60682 Metetra
.797 .826 .875 r
.35488 .60682 .36374 .61285 .37808 .60252 .36932 .596 Metetra
.795 .816 .868 r
.36932 .596 .37808 .60252 .39245 .5921 .38378 .58506 Metetra
.792 .806 .863 r
.38378 .58506 .39245 .5921 .40684 .58173 .39827 .57415 Metetra
.786 .795 .859 r
.39827 .57415 .40684 .58173 .42125 .57149 .41277 .56336 Metetra
.779 .783 .856 r
.41277 .56336 .42125 .57149 .43569 .56149 .42729 .55283 Metetra
.77 .772 .854 r
.42729 .55283 .43569 .56149 .45015 .55182 .44183 .54264 Metetra
.95 .714 .545 r
.44183 .54264 .45015 .55182 .46719 .41543 .4592 .41003 Metetra
.888 .46 .124 r
.4592 .41003 .46719 .41543 .48052 .44475 .47247 .44087 Metetra
.871 .43 .114 r
.47247 .44087 .48052 .44475 .49428 .47155 .4862 .46906 Metetra
.85 .394 .098 r
.4862 .46906 .49428 .47155 .50843 .49563 .50034 .49437 Metetra
.823 .35 .075 r
.50034 .49437 .50843 .49563 .52294 .51689 .51488 .51665 Metetra
.788 .297 .043 r
.51488 .51665 .52294 .51689 .53778 .53523 .52975 .53581 Metetra
.743 .233 0 r
.52975 .53581 .53778 .53523 .55291 .55064 .54494 .5518 Metetra
.685 .154 0 r
.54494 .5518 .55291 .55064 .56829 .56316 .56038 .56468 Metetra
.61 .057 0 r
.56038 .56468 .56829 .56316 .58387 .57288 .57605 .57453 Metetra
.512 0 0 r
.57605 .57453 .58387 .57288 .59963 .57992 .59189 .5815 Metetra
.383 0 0 r
.59189 .5815 .59963 .57992 .61554 .58445 .60789 .58577 Metetra
.221 0 0 r
.60789 .58577 .61554 .58445 .63155 .58665 .62399 .58754 Metetra
.03 0 0 r
.62399 .58754 .63155 .58665 .64766 .58674 .64018 .58707 Metetra
.171 .709 .774 r
.64018 .58707 .64766 .58674 .66383 .58493 .65643 .58459 Metetra
.355 .828 .873 r
.65643 .58459 .66383 .58493 .68004 .58145 .67273 .58035 Metetra
.501 .9 .931 r
.67273 .58035 .68004 .58145 .6963 .57652 .68906 .57461 Metetra
.606 .934 .957 r
.68906 .57461 .6963 .57652 .71258 .57035 .7054 .56761 Metetra
.676 .942 .963 r
.7054 .56761 .71258 .57035 .72889 .56315 .72177 .55956 Metetra
.721 .935 .958 r
.72177 .55956 .72889 .56315 .74521 .55511 .73815 .55067 Metetra
.748 .921 .949 r
.73815 .55067 .74521 .55511 .76156 .5464 .75455 .54114 Metetra
.764 .904 .938 r
.75455 .54114 .76156 .5464 .77792 .53717 .77096 .53112 Metetra
.772 .887 .927 r
.77096 .53112 .77792 .53717 .79432 .52756 .7874 .52078 Metetra
.776 .87 .918 r
.7874 .52078 .79432 .52756 .81075 .5177 .80387 .51023 Metetra
.776 .854 .909 r
.80387 .51023 .81075 .5177 .82722 .50768 .82039 .49958 Metetra
.773 .839 .902 r
.82039 .49958 .82722 .50768 .84373 .4976 .83694 .48892 Metetra
.673 .838 .962 r
.23233 .66888 .24191 .67388 .25564 .66833 .2461 .66362 Metetra
.705 .851 .952 r
.2461 .66362 .25564 .66833 .26952 .66183 .26002 .65731 Metetra
.733 .86 .94 r
.26002 .65731 .26952 .66183 .28352 .65444 .27409 .64999 Metetra
.756 .863 .928 r
.27409 .64999 .28352 .65444 .29764 .6462 .28827 .64171 Metetra
.775 .863 .915 r
.28827 .64171 .29764 .6462 .31185 .6372 .30257 .63255 Metetra
.789 .859 .902 r
.30257 .63255 .31185 .6372 .32613 .62755 .31695 .62263 Metetra
.798 .853 .891 r
.31695 .62263 .32613 .62755 .34048 .61738 .3314 .61208 Metetra
.804 .844 .881 r
.3314 .61208 .34048 .61738 .35488 .60682 .3459 .60106 Metetra
.806 .834 .873 r
.3459 .60106 .35488 .60682 .36932 .596 .36044 .58971 Metetra
.805 .823 .866 r
.36044 .58971 .36932 .596 .38378 .58506 .37501 .57819 Metetra
.801 .811 .86 r
.37501 .57819 .38378 .58506 .39827 .57415 .3896 .56667 Metetra
.795 .799 .855 r
.3896 .56667 .39827 .57415 .41277 .56336 .4042 .55527 Metetra
.787 .787 .852 r
.4042 .55527 .41277 .56336 .42729 .55283 .41881 .54413 Metetra
.778 .774 .85 r
.41881 .54413 .42729 .55283 .44183 .54264 .43343 .53336 Metetra
.95 .715 .548 r
.43343 .53336 .44183 .54264 .4592 .41003 .4511 .40483 Metetra
.895 .474 .149 r
.4511 .40483 .4592 .41003 .47247 .44087 .4643 .4374 Metetra
.878 .445 .143 r
.4643 .4374 .47247 .44087 .4862 .46906 .47799 .46718 Metetra
.857 .41 .13 r
.47799 .46718 .4862 .46906 .50034 .49437 .49212 .49389 Metetra
.83 .367 .111 r
.49212 .49389 .50034 .49437 .51488 .51665 .50667 .51733 Metetra
.795 .316 .083 r
.50667 .51733 .51488 .51665 .52975 .53581 .52159 .53739 Metetra
.751 .254 .045 r
.52159 .53739 .52975 .53581 .54494 .5518 .53683 .55404 Metetra
.695 .177 0 r
.53683 .55404 .54494 .5518 .56038 .56468 .55234 .56729 Metetra
.622 .082 0 r
.55234 .56729 .56038 .56468 .57605 .57453 .56809 .57726 Metetra
.526 0 0 r
.56809 .57726 .57605 .57453 .59189 .5815 .58403 .58411 Metetra
.399 0 0 r
.58403 .58411 .59189 .5815 .60789 .58577 .60012 .58804 Metetra
.236 0 0 r
.60012 .58804 .60789 .58577 .62399 .58754 .61632 .58929 Metetra
.038 0 0 r
.61632 .58929 .62399 .58754 .64018 .58707 .6326 .58814 Metetra
.174 .708 .735 r
.6326 .58814 .64018 .58707 .65643 .58459 .64894 .58487 Metetra
.369 .838 .846 r
.64894 .58487 .65643 .58459 .67273 .58035 .66531 .57975 Metetra
.524 .916 .913 r
.66531 .57975 .67273 .58035 .68906 .57461 .68171 .57308 Metetra
.632 .949 .944 r
.68171 .57308 .68906 .57461 .7054 .56761 .69813 .5651 Metetra
.702 .954 .952 r
.69813 .5651 .7054 .56761 .72177 .55956 .71456 .55608 Metetra
.745 .944 .947 r
.71456 .55608 .72177 .55956 .73815 .55067 .73099 .54625 Metetra
.769 .927 .938 r
.73099 .54625 .73815 .55067 .75455 .54114 .74744 .5358 Metetra
.782 .907 .927 r
.74744 .5358 .75455 .54114 .77096 .53112 .7639 .52492 Metetra
.788 .887 .917 r
.7639 .52492 .77096 .53112 .7874 .52078 .78038 .51377 Metetra
.788 .869 .908 r
.78038 .51377 .7874 .52078 .80387 .51023 .7969 .50248 Metetra
.786 .851 .9 r
.7969 .50248 .80387 .51023 .82039 .49958 .81345 .49115 Metetra
.782 .836 .894 r
.81345 .49115 .82039 .49958 .83694 .48892 .83005 .47989 Metetra
.655 .839 .971 r
.22262 .66397 .23233 .66888 .2461 .66362 .23641 .65909 Metetra
.693 .856 .96 r
.23641 .65909 .2461 .66362 .26002 .65731 .25038 .65306 Metetra
.727 .868 .948 r
.25038 .65306 .26002 .65731 .27409 .64999 .26449 .64588 Metetra
.755 .874 .933 r
.26449 .64588 .27409 .64999 .28827 .64171 .27875 .6376 Metetra
.777 .874 .919 r
.27875 .6376 .28827 .64171 .30257 .63255 .29313 .62831 Metetra
.794 .871 .905 r
.29313 .62831 .30257 .63255 .31695 .62263 .30761 .61812 Metetra
.806 .864 .892 r
.30761 .61812 .31695 .62263 .3314 .61208 .32217 .60718 Metetra
.812 .855 .88 r
.32217 .60718 .3314 .61208 .3459 .60106 .33678 .59565 Metetra
.815 .843 .871 r
.33678 .59565 .3459 .60106 .36044 .58971 .35144 .58371 Metetra
.815 .831 .862 r
.35144 .58371 .36044 .58971 .37501 .57819 .36612 .57155 Metetra
.811 .818 .856 r
.36612 .57155 .37501 .57819 .3896 .56667 .38082 .55933 Metetra
.805 .804 .851 r
.38082 .55933 .3896 .56667 .4042 .55527 .39552 .54723 Metetra
.796 .79 .847 r
.39552 .54723 .4042 .55527 .41881 .54413 .41023 .5354 Metetra
.786 .776 .844 r
.41023 .5354 .41881 .54413 .43343 .53336 .42494 .52397 Metetra
.949 .716 .55 r
.42494 .52397 .43343 .53336 .4511 .40483 .44289 .39988 Metetra
.901 .488 .174 r
.44289 .39988 .4511 .40483 .4643 .4374 .45601 .43444 Metetra
.884 .459 .171 r
.45601 .43444 .4643 .4374 .47799 .46718 .46964 .46603 Metetra
.863 .425 .162 r
.46964 .46603 .47799 .46718 .49212 .49389 .48375 .49433 Metetra
.835 .384 .147 r
.48375 .49433 .49212 .49389 .50667 .51733 .49831 .5191 Metetra
.801 .334 .124 r
.49831 .5191 .50667 .51733 .52159 .53739 .51327 .54019 Metetra
.757 .273 .09 r
.51327 .54019 .52159 .53739 .53683 .55404 .52857 .55755 Metetra
.701 .199 .042 r
.52857 .55755 .53683 .55404 .55234 .56729 .54416 .5712 Metetra
.63 .106 0 r
.54416 .5712 .55234 .56729 .56809 .57726 .56 .58127 Metetra
.535 0 0 r
.56 .58127 .56809 .57726 .58403 .58411 .57604 .58793 Metetra
.41 0 0 r
.57604 .58793 .58403 .58411 .60012 .58804 .59223 .59143 Metetra
.246 0 0 r
.59223 .59143 .60012 .58804 .61632 .58929 .60852 .59204 Metetra
.043 0 0 r
.60852 .59204 .61632 .58929 .6326 .58814 .6249 .59007 Metetra
.18 .706 .69 r
.6249 .59007 .6326 .58814 .64894 .58487 .64133 .58585 Metetra
.388 .848 .815 r
.64133 .58585 .64894 .58487 .66531 .57975 .65779 .57969 Metetra
.551 .931 .892 r
.65779 .57969 .66531 .57975 .68171 .57308 .67427 .57192 Metetra
.662 .964 .928 r
.67427 .57192 .68171 .57308 .69813 .5651 .69075 .56284 Metetra
.731 .965 .938 r
.69075 .56284 .69813 .5651 .71456 .55608 .70724 .55272 Metetra
.77 .951 .934 r
.70724 .55272 .71456 .55608 .73099 .54625 .72373 .54181 Metetra
.791 .931 .925 r
.72373 .54181 .73099 .54625 .74744 .5358 .74022 .53034 Metetra
.8 .908 .915 r
.74022 .53034 .74744 .5358 .7639 .52492 .75673 .51851 Metetra
.803 .886 .905 r
.75673 .51851 .7639 .52492 .78038 .51377 .77325 .50647 Metetra
.801 .866 .897 r
.77325 .50647 .78038 .51377 .7969 .50248 .7898 .49438 Metetra
.796 .847 .89 r
.7898 .49438 .7969 .50248 .81345 .49115 .8064 .48233 Metetra
.79 .831 .884 r
.8064 .48233 .81345 .49115 .83005 .47989 .82304 .47043 Metetra
.631 .838 .98 r
.21278 .65914 .22262 .66397 .23641 .65909 .22658 .65475 Metetra
.676 .861 .97 r
.22658 .65475 .23641 .65909 .25038 .65306 .24057 .6491 Metetra
.717 .877 .956 r
.24057 .6491 .25038 .65306 .26449 .64588 .25474 .64215 Metetra
.751 .885 .94 r
.25474 .64215 .26449 .64588 .27875 .6376 .26907 .63394 Metetra
.778 .887 .924 r
.26907 .63394 .27875 .6376 .29313 .62831 .28353 .62455 Metetra
.799 .884 .907 r
.28353 .62455 .29313 .62831 .30761 .61812 .29811 .6141 Metetra
.813 .877 .892 r
.29811 .6141 .30761 .61812 .32217 .60718 .31278 .60275 Metetra
.822 .867 .879 r
.31278 .60275 .32217 .60718 .33678 .59565 .32752 .59068 Metetra
.826 .854 .868 r
.32752 .59068 .33678 .59565 .35144 .58371 .3423 .57809 Metetra
.826 .84 .859 r
.3423 .57809 .35144 .58371 .36612 .57155 .35711 .5652 Metetra
.822 .825 .851 r
.35711 .5652 .36612 .57155 .38082 .55933 .37192 .5522 Metetra
.816 .81 .845 r
.37192 .5522 .38082 .55933 .39552 .54723 .38674 .5393 Metetra
.807 .794 .841 r
.38674 .5393 .39552 .54723 .41023 .5354 .40156 .52667 Metetra
.796 .779 .838 r
.40156 .52667 .41023 .5354 .42494 .52397 .41637 .51447 Metetra
.949 .717 .553 r
.41637 .51447 .42494 .52397 .44289 .39988 .43456 .39523 Metetra
.907 .501 .199 r
.43456 .39523 .44289 .39988 .45601 .43444 .44757 .43208 Metetra
.889 .473 .2 r
.44757 .43208 .45601 .43444 .46964 .46603 .46114 .46577 Metetra
.867 .439 .194 r
.46114 .46577 .46964 .46603 .48375 .49433 .47522 .49591 Metetra
.839 .399 .183 r
.47522 .49591 .48375 .49433 .49831 .5191 .48979 .5222 Metetra
.803 .35 .163 r
.48979 .5222 .49831 .5191 .51327 .54019 .50478 .54445 Metetra
.759 .291 .133 r
.50478 .54445 .51327 .54019 .52857 .55755 .52014 .5626 Metetra
.704 .218 .091 r
.52014 .5626 .52857 .55755 .54416 .5712 .53582 .57667 Metetra
.633 .128 .032 r
.53582 .57667 .54416 .5712 .56 .58127 .55176 .5868 Metetra
.54 .015 0 r
.55176 .5868 .56 .58127 .57604 .58793 .5679 .59319 Metetra
.415 0 0 r
.5679 .59319 .57604 .58793 .59223 .59143 .5842 .59612 Metetra
.251 0 0 r
.5842 .59612 .59223 .59143 .60852 .59204 .60061 .59593 Metetra
.043 0 0 r
.60061 .59593 .60852 .59204 .6249 .59007 .61709 .59296 Metetra
.61709 .59296 .6249 .59007 .64133 .58585 .63362 .5876 Metetra
.412 .858 .781 r
.63362 .5876 .64133 .58585 .65779 .57969 .65016 .58022 Metetra
.583 .945 .868 r
.65016 .58022 .65779 .57969 .67427 .57192 .66672 .57117 Metetra
.696 .977 .909 r
.66672 .57117 .67427 .57192 .69075 .56284 .68327 .5608 Metetra
.761 .974 .921 r
.68327 .5608 .69075 .56284 .70724 .55272 .69981 .54941 Metetra
.796 .956 .919 r
.69981 .54941 .70724 .55272 .72373 .54181 .71635 .53729 Metetra
.813 .932 .911 r
.71635 .53729 .72373 .54181 .74022 .53034 .73289 .52468 Metetra
.818 .906 .901 r
.73289 .52468 .74022 .53034 .75673 .51851 .74944 .51179 Metetra
.817 .882 .892 r
.74944 .51179 .75673 .51851 .77325 .50647 .766 .4988 Metetra
.812 .861 .884 r
.766 .4988 .77325 .50647 .7898 .49438 .78259 .48584 Metetra
.805 .841 .878 r
.78259 .48584 .7898 .49438 .8064 .48233 .79922 .47303 Metetra
.797 .824 .873 r
.79922 .47303 .8064 .48233 .82304 .47043 .8159 .46047 Metetra
.598 .833 .989 r
.2028 .65435 .21278 .65914 .22658 .65475 .21659 .6506 Metetra
.651 .863 .98 r
.21659 .6506 .22658 .65475 .24057 .6491 .2306 .64546 Metetra
.7 .885 .966 r
.2306 .64546 .24057 .6491 .25474 .64215 .24481 .63886 Metetra
.743 .898 .948 r
.24481 .63886 .25474 .64215 .26907 .63394 .2592 .63081 Metetra
.777 .902 .929 r
.2592 .63081 .26907 .63394 .28353 .62455 .27375 .62139 Metetra
.803 .9 .911 r
.27375 .62139 .28353 .62455 .29811 .6141 .28844 .6107 Metetra
.821 .892 .893 r
.28844 .6107 .29811 .6141 .31278 .60275 .30323 .59893 Metetra
.832 .881 .878 r
.30323 .59893 .31278 .60275 .32752 .59068 .31809 .58627 Metetra
.838 .867 .865 r
.31809 .58627 .32752 .59068 .3423 .57809 .33301 .57296 Metetra
.838 .851 .854 r
.33301 .57296 .3423 .57809 .35711 .5652 .34795 .55923 Metetra
.835 .834 .846 r
.34795 .55923 .35711 .5652 .37192 .5522 .3629 .54534 Metetra
.828 .816 .839 r
.3629 .54534 .37192 .5522 .38674 .5393 .37785 .5315 Metetra
.818 .799 .834 r
.37785 .5315 .38674 .5393 .40156 .52667 .39279 .51795 Metetra
.806 .782 .83 r
.39279 .51795 .40156 .52667 .41637 .51447 .4077 .50486 Metetra
.948 .718 .556 r
.4077 .50486 .41637 .51447 .43456 .39523 .4261 .39096 Metetra
.912 .514 .224 r
.4261 .39096 .43456 .39523 .44757 .43208 .43898 .43048 Metetra
.893 .486 .228 r
.43898 .43048 .44757 .43208 .46114 .46577 .45246 .46661 Metetra
.87 .453 .225 r
.45246 .46661 .46114 .46577 .47522 .49591 .46651 .49888 Metetra
.84 .412 .217 r
.46651 .49888 .47522 .49591 .48979 .5222 .48107 .52692 Metetra
.804 .364 .201 r
.48107 .52692 .48979 .5222 .50478 .54445 .4961 .5505 Metetra
.759 .306 .176 r
.4961 .5505 .50478 .54445 .52014 .5626 .51154 .56953 Metetra
.703 .235 .138 r
.51154 .56953 .52014 .5626 .53582 .57667 .52731 .58403 Metetra
.632 .147 .084 r
.52731 .58403 .53582 .57667 .55176 .5868 .54336 .59415 Metetra
.539 .036 .009 r
.54336 .59415 .55176 .5868 .5679 .59319 .55962 .60015 Metetra
.415 0 0 r
.55962 .60015 .5679 .59319 .5842 .59612 .57603 .60235 Metetra
.249 0 0 r
.57603 .60235 .5842 .59612 .60061 .59593 .59256 .60115 Metetra
.036 0 0 r
.59256 .60115 .60061 .59593 .61709 .59296 .60915 .59695 Metetra
.60915 .59695 .61709 .59296 .63362 .5876 .62578 .59022 Metetra
.443 .867 .743 r
.62578 .59022 .63362 .5876 .65016 .58022 .64242 .58136 Metetra
.621 .959 .841 r
.64242 .58136 .65016 .58022 .66672 .57117 .65905 .57081 Metetra
.732 .988 .888 r
.65905 .57081 .66672 .57117 .68327 .5608 .67567 .55894 Metetra
.793 .981 .902 r
.67567 .55894 .68327 .5608 .69981 .54941 .69227 .5461 Metetra
.823 .958 .901 r
.69227 .5461 .69981 .54941 .71635 .53729 .70886 .53261 Metetra
.834 .93 .894 r
.70886 .53261 .71635 .53729 .73289 .52468 .72544 .51873 Metetra
.835 .902 .886 r
.72544 .51873 .73289 .52468 .74944 .51179 .74203 .50467 Metetra
.83 .876 .878 r
.74203 .50467 .74944 .51179 .766 .4988 .75863 .49063 Metetra
.822 .853 .871 r
.75863 .49063 .766 .4988 .78259 .48584 .77525 .47676 Metetra
.812 .833 .865 r
.77525 .47676 .78259 .48584 .79922 .47303 .79192 .46315 Metetra
.802 .815 .862 r
.79192 .46315 .79922 .47303 .8159 .46047 .80863 .4499 Metetra
.551 .821 .996 r
.19269 .64955 .2028 .65435 .21659 .6506 .20645 .64662 Metetra
.614 .861 .99 r
.20645 .64662 .21659 .6506 .2306 .64546 .22046 .64215 Metetra
.675 .892 .976 r
.22046 .64215 .2306 .64546 .24481 .63886 .23469 .63606 Metetra
.728 .911 .957 r
.23469 .63606 .24481 .63886 .2592 .63081 .24914 .6283 Metetra
.772 .919 .936 r
.24914 .6283 .2592 .63081 .27375 .62139 .26377 .61893 Metetra
.805 .918 .914 r
.26377 .61893 .27375 .62139 .28844 .6107 .27856 .60806 Metetra
.828 .91 .894 r
.27856 .60806 .28844 .6107 .30323 .59893 .29348 .59587 Metetra
.843 .897 .877 r
.29348 .59587 .30323 .59893 .31809 .58627 .30848 .58258 Metetra
.85 .881 .862 r
.30848 .58258 .31809 .58627 .33301 .57296 .32355 .56846 Metetra
.852 .863 .849 r
.32355 .56846 .33301 .57296 .34795 .55923 .33865 .55378 Metetra
.848 .844 .839 r
.33865 .55378 .34795 .55923 .3629 .54534 .35375 .53884 Metetra
.841 .824 .832 r
.35375 .53884 .3629 .54534 .37785 .5315 .36884 .52393 Metetra
.83 .804 .826 r
.36884 .52393 .37785 .5315 .39279 .51795 .38391 .50928 Metetra
.817 .785 .822 r
.38391 .50928 .39279 .51795 .4077 .50486 .39895 .49514 Metetra
.947 .719 .56 r
.39895 .49514 .4077 .50486 .4261 .39096 .4175 .38713 Metetra
.916 .527 .249 r
.4175 .38713 .4261 .39096 .43898 .43048 .43023 .42979 Metetra
.896 .499 .255 r
.43023 .42979 .43898 .43048 .45246 .46661 .4436 .4688 Metetra
.871 .465 .256 r
.4436 .4688 .45246 .46661 .46651 .49888 .45759 .50358 Metetra
.84 .425 .251 r
.45759 .50358 .46651 .49888 .48107 .52692 .47215 .53367 Metetra
.802 .377 .238 r
.47215 .53367 .48107 .52692 .4961 .5505 .48722 .55878 Metetra
.755 .319 .216 r
.48722 .55878 .4961 .5505 .51154 .56953 .50273 .57878 Metetra
.698 .25 .182 r
.50273 .57878 .51154 .56953 .52731 .58403 .5186 .5937 Metetra
.627 .163 .134 r
.5186 .5937 .52731 .58403 .54336 .59415 .53477 .60373 Metetra
.533 .054 .064 r
.53477 .60373 .54336 .59415 .55962 .60015 .55116 .60916 Metetra
.409 0 0 r
.55116 .60916 .55962 .60015 .57603 .60235 .56771 .6104 Metetra
.241 0 0 r
.56771 .6104 .57603 .60235 .59256 .60115 .58437 .60791 Metetra
.021 0 0 r
.58437 .60791 .59256 .60115 .60915 .59695 .60108 .60219 Metetra
.60108 .60219 .60915 .59695 .62578 .59022 .61781 .59377 Metetra
.481 .877 .705 r
.61781 .59377 .62578 .59022 .64242 .58136 .63455 .58315 Metetra
.663 .971 .813 r
.63455 .58315 .64242 .58136 .65905 .57081 .65126 .57081 Metetra
.772 .996 .864 r
.65126 .57081 .65905 .57081 .67567 .55894 .66795 .55719 Metetra
.826 .983 .88 r
.66795 .55719 .67567 .55894 .69227 .5461 .6846 .54269 Metetra
.849 .955 .881 r
.6846 .54269 .69227 .5461 .70886 .53261 .70124 .52764 Metetra
.854 .923 .875 r
.70124 .52764 .70886 .53261 .72544 .51873 .71786 .51233 Metetra
.85 .893 .868 r
.71786 .51233 .72544 .51873 .74203 .50467 .73448 .497 Metetra
.841 .866 .861 r
.73448 .497 .74203 .50467 .75863 .49063 .75111 .48184 Metetra
.83 .842 .856 r
.75111 .48184 .75863 .49063 .77525 .47676 .76777 .46699 Metetra
.818 .821 .852 r
.76777 .46699 .77525 .47676 .79192 .46315 .78447 .45256 Metetra
.806 .804 .849 r
.78447 .45256 .79192 .46315 .80863 .4499 .80122 .43863 Metetra
.485 .796 .999 r
.18245 .64465 .19269 .64955 .20645 .64662 .19616 .64275 Metetra
.559 .85 .998 r
.19616 .64275 .20645 .64662 .22046 .64215 .21013 .63919 Metetra
.634 .893 .986 r
.21013 .63919 .22046 .64215 .23469 .63606 .22437 .6338 Metetra
.703 .922 .967 r
.22437 .6338 .23469 .63606 .24914 .6283 .23885 .62651 Metetra
.759 .936 .943 r
.23885 .62651 .24914 .6283 .26377 .61893 .25356 .61733 Metetra
.803 .938 .918 r
.25356 .61733 .26377 .61893 .27856 .60806 .26846 .60635 Metetra
.834 .931 .894 r
.26846 .60635 .27856 .60806 .29348 .59587 .28351 .59376 Metetra
.853 .917 .874 r
.28351 .59376 .29348 .59587 .30848 .58258 .29867 .5798 Metetra
.864 .899 .857 r
.29867 .5798 .30848 .58258 .32355 .56846 .3139 .56477 Metetra
.867 .878 .843 r
.3139 .56477 .32355 .56846 .33865 .55378 .32917 .54901 Metetra
.864 .856 .832 r
.32917 .54901 .33865 .55378 .35375 .53884 .34445 .53285 Metetra
.856 .833 .823 r
.34445 .53285 .35375 .53884 .36884 .52393 .35971 .51665 Metetra
.844 .81 .817 r
.35971 .51665 .36884 .52393 .38391 .50928 .37493 .50072 Metetra
.83 .788 .812 r
.37493 .50072 .38391 .50928 .39895 .49514 .39011 .48533 Metetra
.946 .721 .565 r
.39011 .48533 .39895 .49514 .4175 .38713 .40876 .38387 Metetra
.919 .539 .273 r
.40876 .38387 .4175 .38713 .43023 .42979 .42128 .43027 Metetra
.898 .51 .282 r
.42128 .43027 .43023 .42979 .4436 .4688 .43452 .47271 Metetra
.87 .476 .286 r
.43452 .47271 .4436 .4688 .45759 .50358 .44843 .51047 Metetra
.837 .435 .283 r
.44843 .51047 .45759 .50358 .47215 .53367 .46298 .54297 Metetra
.797 .387 .273 r
.46298 .54297 .47215 .53367 .48722 .55878 .47809 .56984 Metetra
.749 .33 .254 r
.47809 .56984 .48722 .55878 .50273 .57878 .49368 .59092 Metetra
.69 .261 .224 r
.49368 .59092 .50273 .57878 .5186 .5937 .50967 .60624 Metetra
.617 .176 .18 r
.50967 .60624 .5186 .5937 .53477 .60373 .52597 .61603 Metetra
.522 .068 .115 r
.52597 .61603 .53477 .60373 .55116 .60916 .54251 .62066 Metetra
.396 0 .021 r
.54251 .62066 .55116 .60916 .56771 .6104 .55921 .62061 Metetra
.225 0 0 r
.55921 .62061 .56771 .6104 .58437 .60791 .57601 .61647 Metetra
.57601 .61647 .58437 .60791 .60108 .60219 .59286 .60884 Metetra
.59286 .60884 .60108 .60219 .61781 .59377 .60971 .59834 Metetra
.528 .888 .667 r
.60971 .59834 .61781 .59377 .63455 .58315 .62654 .58558 Metetra
.712 .98 .783 r
.62654 .58558 .63455 .58315 .65126 .57081 .64334 .57111 Metetra
.814 .999 .838 r
.64334 .57111 .65126 .57081 .66795 .55719 .66009 .55544 Metetra
.859 .98 .857 r
.66009 .55544 .66795 .55719 .6846 .54269 .6768 .53901 Metetra
.873 .947 .859 r
.6768 .53901 .6846 .54269 .70124 .52764 .69348 .5222 Metetra
.872 .912 .854 r
.69348 .5222 .70124 .52764 .71786 .51233 .71014 .5053 Metetra
.863 .88 .849 r
.71014 .5053 .71786 .51233 .73448 .497 .72679 .48858 Metetra
.85 .852 .844 r
.72679 .48858 .73448 .497 .75111 .48184 .74345 .47222 Metetra
.836 .828 .84 r
.74345 .47222 .75111 .48184 .76777 .46699 .76014 .45636 Metetra
.822 .807 .837 r
.76014 .45636 .76777 .46699 .78447 .45256 .77687 .4411 Metetra
.808 .79 .836 r
.77687 .4411 .78447 .45256 .80122 .43863 .79367 .4265 Metetra
.392 .752 .993 r
.1721 .63952 .18245 .64465 .19616 .64275 .18571 .63892 Metetra
.475 .822 .998 r
.18571 .63892 .19616 .64275 .21013 .63919 .19962 .63654 Metetra
.568 .884 .991 r
.19962 .63654 .21013 .63919 .22437 .6338 .21383 .63214 Metetra
.657 .928 .973 r
.21383 .63214 .22437 .6338 .23885 .62651 .22833 .62557 Metetra
.733 .953 .947 r
.22833 .62557 .23885 .62651 .25356 .61733 .2431 .61676 Metetra
.793 .96 .919 r
.2431 .61676 .25356 .61733 .26846 .60635 .2581 .6058 Metetra
.835 .954 .893 r
.2581 .6058 .26846 .60635 .28351 .59376 .27328 .59286 Metetra
.862 .94 .869 r
.27328 .59286 .28351 .59376 .29867 .5798 .28861 .5782 Metetra
.878 .919 .85 r
.28861 .5782 .29867 .5798 .3139 .56477 .30403 .56216 Metetra
.883 .896 .835 r
.30403 .56216 .3139 .56477 .32917 .54901 .3195 .54514 Metetra
.881 .87 .823 r
.3195 .54514 .32917 .54901 .34445 .53285 .33498 .52755 Metetra
.873 .844 .813 r
.33498 .52755 .34445 .53285 .35971 .51665 .35043 .50981 Metetra
.86 .818 .806 r
.35043 .50981 .35971 .51665 .37493 .50072 .36583 .49232 Metetra
.844 .792 .801 r
.36583 .49232 .37493 .50072 .39011 .48533 .38117 .47542 Metetra
.945 .723 .572 r
.38117 .47542 .39011 .48533 .40876 .38387 .39985 .38132 Metetra
.922 .551 .297 r
.39985 .38132 .40876 .38387 .42128 .43027 .41212 .43222 Metetra
.898 .52 .308 r
.41212 .43222 .42128 .43027 .43452 .47271 .42518 .47881 Metetra
.868 .485 .314 r
.42518 .47881 .43452 .47271 .44843 .51047 .439 .52015 Metetra
.832 .443 .314 r
.439 .52015 .44843 .51047 .46298 .54297 .45352 .55553 Metetra
.79 .395 .306 r
.45352 .55553 .46298 .54297 .47809 .56984 .46866 .58445 Metetra
.739 .338 .29 r
.46866 .58445 .47809 .56984 .49368 .59092 .48435 .60672 Metetra
.678 .269 .263 r
.48435 .60672 .49368 .59092 .50967 .60624 .50047 .62238 Metetra
.603 .185 .222 r
.50047 .62238 .50967 .60624 .52597 .61603 .51694 .63171 Metetra
.506 .078 .161 r
.51694 .63171 .52597 .61603 .54251 .62066 .53365 .63518 Metetra
.376 0 .072 r
.53365 .63518 .54251 .62066 .55921 .62061 .55052 .63341 Metetra
.199 0 0 r
.55052 .63341 .55921 .62061 .57601 .61647 .56749 .62712 Metetra
.56749 .62712 .57601 .61647 .59286 .60884 .58448 .61706 Metetra
.58448 .61706 .59286 .60884 .60971 .59834 .60145 .60398 Metetra
.586 .9 .634 r
.60145 .60398 .60971 .59834 .62654 .58558 .61839 .5886 Metetra
.765 .987 .755 r
.61839 .5886 .62654 .58558 .64334 .57111 .63527 .57158 Metetra
.856 .997 .812 r
.63527 .57158 .64334 .57111 .66009 .55544 .65209 .5535 Metetra
.889 .97 .831 r
.65209 .5535 .66009 .55544 .6768 .53901 .66885 .53485 Metetra
.894 .933 .834 r
.66885 .53485 .6768 .53901 .69348 .5222 .68557 .51604 Metetra
.886 .896 .832 r
.68557 .51604 .69348 .5222 .71014 .5053 .70226 .4974 Metetra
.872 .862 .828 r
.70226 .4974 .71014 .5053 .72679 .48858 .71894 .47918 Metetra
.855 .834 .825 r
.71894 .47918 .72679 .48858 .74345 .47222 .73563 .46157 Metetra
.839 .81 .822 r
.73563 .46157 .74345 .47222 .76014 .45636 .75235 .44468 Metetra
.822 .79 .822 r
.75235 .44468 .76014 .45636 .77687 .4411 .76912 .42861 Metetra
.807 .773 .822 r
.76912 .42861 .77687 .4411 .79367 .4265 .78595 .41337 Metetra
.265 .677 .967 r
.16166 .63394 .1721 .63952 .18571 .63892 .17513 .63497 Metetra
.35 .763 .98 r
.17513 .63497 .18571 .63892 .19962 .63654 .18893 .63414 Metetra
.459 .848 .981 r
.18893 .63414 .19962 .63654 .21383 .63214 .20307 .6311 Metetra
.575 .916 .968 r
.20307 .6311 .21383 .63214 .22833 .62557 .21755 .62558 Metetra
.681 .96 .943 r
.21755 .62558 .22833 .62557 .2431 .61676 .23235 .61745 Metetra
.765 .979 .914 r
.23235 .61745 .2431 .61676 .2581 .6058 .24744 .60672 Metetra
.827 .979 .885 r
.24744 .60672 .2581 .6058 .27328 .59286 .26276 .59353 Metetra
.867 .965 .86 r
.26276 .59353 .27328 .59286 .28861 .5782 .27827 .57815 Metetra
.89 .943 .84 r
.27827 .57815 .28861 .5782 .30403 .56216 .2939 .56099 Metetra
.9 .916 .823 r
.2939 .56099 .30403 .56216 .3195 .54514 .3096 .5425 Metetra
.899 .887 .81 r
.3096 .5425 .3195 .54514 .33498 .52755 .32531 .5232 Metetra
.891 .856 .801 r
.32531 .5232 .33498 .52755 .35043 .50981 .34099 .5036 Metetra
.877 .826 .793 r
.34099 .5036 .35043 .50981 .36583 .49232 .35661 .4842 Metetra
.859 .796 .788 r
.35661 .4842 .36583 .49232 .38117 .47542 .37214 .46544 Metetra
.943 .726 .58 r
.37214 .46544 .38117 .47542 .39985 .38132 .39075 .37967 Metetra
.924 .562 .32 r
.39075 .37967 .39985 .38132 .41212 .43222 .40271 .4361 Metetra
.897 .53 .334 r
.40271 .4361 .41212 .43222 .42518 .47881 .41554 .48777 Metetra
.864 .492 .342 r
.41554 .48777 .42518 .47881 .439 .52015 .42923 .5335 Metetra
.825 .45 .343 r
.42923 .5335 .439 .52015 .45352 .55553 .4437 .57234 Metetra
.78 .4 .337 r
.4437 .57234 .45352 .55553 .46866 .58445 .45889 .60368 Metetra
.726 .343 .323 r
.45889 .60368 .46866 .58445 .48435 .60672 .47469 .62725 Metetra
.663 .274 .298 r
.47469 .62725 .48435 .60672 .50047 .62238 .49097 .6431 Metetra
.584 .189 .26 r
.49097 .6431 .50047 .62238 .51694 .63171 .50763 .65162 Metetra
.484 .082 .203 r
.50763 .65162 .51694 .63171 .53365 .63518 .52454 .65342 Metetra
.349 0 .117 r
.52454 .65342 .53365 .63518 .55052 .63341 .54162 .6493 Metetra
.164 0 0 r
.54162 .6493 .55052 .63341 .56749 .62712 .55876 .64018 Metetra
.55876 .64018 .56749 .62712 .58448 .61706 .57592 .62699 Metetra
.57592 .62699 .58448 .61706 .60145 .60398 .59304 .61067 Metetra
.654 .914 .606 r
.59304 .61067 .60145 .60398 .61839 .5886 .61008 .59207 Metetra
.822 .989 .729 r
.61008 .59207 .61839 .5886 .63527 .57158 .62705 .57198 Metetra
.896 .987 .784 r
.62705 .57198 .63527 .57158 .65209 .5535 .64393 .55106 Metetra
.915 .952 .804 r
.64393 .55106 .65209 .5535 .66885 .53485 .66074 .52986 Metetra
.91 .911 .809 r
.66074 .52986 .66885 .53485 .68557 .51604 .6775 .50882 Metetra
.895 .873 .808 r
.6775 .50882 .68557 .51604 .70226 .4974 .69422 .48827 Metetra
.876 .839 .806 r
.69422 .48827 .70226 .4974 .71894 .47918 .71092 .46847 Metetra
.857 .811 .805 r
.71092 .46847 .71894 .47918 .73563 .46157 .72763 .44958 Metetra
.838 .788 .804 r
.72763 .44958 .73563 .46157 .75235 .44468 .74438 .43169 Metetra
.82 .77 .806 r
.74438 .43169 .75235 .44468 .76912 .42861 .76118 .41486 Metetra
.803 .754 .808 r
.76118 .41486 .76912 .42861 .78595 .41337 .77806 .39909 Metetra
.105 .563 .913 r
.15117 .62758 .16166 .63394 .17513 .63497 .16444 .63062 Metetra
.171 .653 .927 r
.16444 .63062 .17513 .63497 .18893 .63414 .17807 .63181 Metetra
.285 .76 .934 r
.17807 .63181 .18893 .63414 .20307 .6311 .19207 .63063 Metetra
.428 .861 .929 r
.19207 .63063 .20307 .6311 .21755 .62558 .20648 .62667 Metetra
.574 .938 .912 r
.20648 .62667 .21755 .62558 .23235 .61745 .22127 .61964 Metetra
.7 .982 .888 r
.22127 .61964 .23235 .61745 .24744 .60672 .23642 .60946 Metetra
.795 .996 .862 r
.23642 .60946 .24744 .60672 .26276 .59353 .25188 .59622 Metetra
.859 .989 .839 r
.25188 .59622 .26276 .59353 .27827 .57815 .26758 .58018 Metetra
.898 .969 .821 r
.26758 .58018 .27827 .57815 .2939 .56099 .28345 .56178 Metetra
.916 .939 .806 r
.28345 .56178 .2939 .56099 .3096 .5425 .29942 .54157 Metetra
.919 .906 .794 r
.29942 .54157 .3096 .5425 .32531 .5232 .31541 .52019 Metetra
.912 .871 .785 r
.31541 .52019 .32531 .5232 .34099 .5036 .33137 .49829 Metetra
.896 .836 .778 r
.33137 .49829 .34099 .5036 .35661 .4842 .34724 .4765 Metetra
.876 .801 .773 r
.34724 .4765 .35661 .4842 .37214 .46544 .363 .45542 Metetra
.94 .73 .591 r
.363 .45542 .37214 .46544 .39075 .37967 .38145 .37918 Metetra
.925 .572 .342 r
.38145 .37918 .39075 .37967 .40271 .4361 .393 .44251 Metetra
.895 .538 .359 r
.393 .44251 .40271 .4361 .41554 .48777 .40553 .50055 Metetra
.859 .498 .368 r
.40553 .50055 .41554 .48777 .42923 .5335 .41903 .55174 Metetra
.816 .454 .37 r
.41903 .55174 .42923 .5335 .4437 .57234 .43345 .59484 Metetra
.767 .403 .365 r
.43345 .59484 .4437 .57234 .45889 .60368 .44868 .62904 Metetra
.71 .344 .353 r
.44868 .62904 .45889 .60368 .47469 .62725 .46461 .65398 Metetra
.643 .275 .33 r
.46461 .65398 .47469 .62725 .49097 .6431 .48109 .66977 Metetra
.561 .189 .294 r
.48109 .66977 .49097 .6431 .50763 .65162 .49798 .67692 Metetra
.455 .081 .239 r
.49798 .67692 .50763 .65162 .52454 .65342 .51514 .67628 Metetra
.313 0 .154 r
.51514 .67628 .52454 .65342 .54162 .6493 .53246 .6689 Metetra
.115 0 .025 r
.53246 .6689 .54162 .6493 .55876 .64018 .54983 .65598 Metetra
.54983 .65598 .55876 .64018 .57592 .62699 .56717 .63873 Metetra
.56717 .63873 .57592 .62699 .59304 .61067 .58444 .61829 Metetra
.733 .929 .588 r
.58444 .61829 .59304 .61067 .61008 .59207 .6016 .59572 Metetra
.879 .984 .706 r
.6016 .59572 .61008 .59207 .62705 .57198 .61866 .57192 Metetra
.93 .967 .758 r
.61866 .57192 .62705 .57198 .64393 .55106 .6356 .54767 Metetra
.934 .926 .776 r
.6356 .54767 .64393 .55106 .66074 .52986 .65246 .52356 Metetra
.919 .882 .782 r
.65246 .52356 .66074 .52986 .6775 .50882 .66924 .50005 Metetra
.897 .843 .783 r
.66924 .50005 .6775 .50882 .69422 .48827 .68598 .47747 Metetra
.875 .811 .783 r
.68598 .47747 .69422 .48827 .71092 .46847 .70271 .45605 Metetra
.853 .784 .784 r
.70271 .45605 .71092 .46847 .72763 .44958 .71945 .43591 Metetra
.832 .763 .786 r
.71945 .43591 .72763 .44958 .74438 .43169 .73623 .41711 Metetra
.813 .746 .789 r
.73623 .41711 .74438 .43169 .76118 .41486 .75307 .39965 Metetra
.796 .733 .794 r
.75307 .39965 .76118 .41486 .77806 .39909 .77 .38348 Metetra
0 .41 .827 r
.1407 .61995 .15117 .62758 .16444 .63062 .1537 .62543 Metetra
0 .48 .822 r
.1537 .62543 .16444 .63062 .17807 .63181 .16706 .62922 Metetra
.031 .587 .819 r
.16706 .62922 .17807 .63181 .19207 .63063 .18085 .63058 Metetra
.182 .715 .817 r
.18085 .63058 .19207 .63063 .20648 .62667 .19509 .62887 Metetra
.37 .837 .812 r
.19509 .62887 .20648 .62667 .22127 .61964 .20981 .62361 Metetra
.555 .929 .806 r
.20981 .62361 .22127 .61964 .23642 .60946 .22498 .61451 Metetra
.709 .982 .799 r
.22498 .61451 .23642 .60946 .25188 .59622 .24056 .60157 Metetra
.82 .999 .791 r
.24056 .60157 .25188 .59622 .26758 .58018 .25646 .58499 Metetra
.889 .99 .783 r
.25646 .58499 .26758 .58018 .28345 .56178 .2726 .56526 Metetra
.926 .963 .776 r
.2726 .56526 .28345 .56178 .29942 .54157 .28888 .54304 Metetra
.938 .928 .769 r
.28888 .54304 .29942 .54157 .31541 .52019 .30522 .5191 Metetra
.933 .888 .763 r
.30522 .5191 .31541 .52019 .33137 .49829 .32151 .4943 Metetra
.918 .847 .758 r
.32151 .4943 .33137 .49829 .34724 .4765 .33771 .46948 Metetra
.895 .807 .754 r
.33771 .46948 .34724 .4765 .363 .45542 .35376 .44541 Metetra
.936 .736 .607 r
.35376 .44541 .363 .45542 .38145 .37918 .37191 .38024 Metetra
.925 .581 .364 r
.37191 .38024 .38145 .37918 .393 .44251 .38291 .45238 Metetra
.891 .544 .382 r
.38291 .45238 .393 .44251 .40553 .50055 .39504 .51857 Metetra
.851 .502 .392 r
.39504 .51857 .40553 .50055 .41903 .55174 .40829 .57672 Metetra
.804 .455 .395 r
.40829 .57672 .41903 .55174 .43345 .59484 .42262 .62515 Metetra
.751 .402 .391 r
.42262 .62515 .43345 .59484 .44868 .62904 .43791 .66275 Metetra
.691 .342 .379 r
.43791 .66275 .44868 .62904 .46461 .65398 .454 .68907 Metetra
.62 .271 .358 r
.454 .68907 .46461 .65398 .48109 .66977 .47074 .70429 Metetra
.532 .184 .323 r
.47074 .70429 .48109 .66977 .49798 .67692 .48792 .70918 Metetra
.42 .073 .268 r
.48792 .70918 .49798 .67692 .51514 .67628 .5054 .7049 Metetra
.266 0 .182 r
.5054 .7049 .51514 .67628 .53246 .6689 .52301 .69293 Metetra
.05 0 .049 r
.52301 .69293 .53246 .6689 .54983 .65598 .54064 .67485 Metetra
.54064 .67485 .54983 .65598 .56717 .63873 .5582 .65222 Metetra
.5582 .65222 .56717 .63873 .58444 .61829 .57564 .62651 Metetra
.818 .941 .58 r
.57564 .62651 .58444 .61829 .6016 .59572 .59293 .599 Metetra
.93 .968 .687 r
.59293 .599 .6016 .59572 .61866 .57192 .61008 .57075 Metetra
.954 .936 .732 r
.61008 .57075 .61866 .57192 .6356 .54767 .62708 .54262 Metetra
.943 .889 .748 r
.62708 .54262 .6356 .54767 .65246 .52356 .64397 .51524 Metetra
.919 .845 .755 r
.64397 .51524 .65246 .52356 .66924 .50005 .66078 .48906 Metetra
.893 .807 .758 r
.66078 .48906 .66924 .50005 .68598 .47747 .67754 .4644 Metetra
.867 .777 .76 r
.67754 .4644 .68598 .47747 .70271 .45605 .69428 .44141 Metetra
.843 .753 .763 r
.69428 .44141 .70271 .45605 .71945 .43591 .71105 .42015 Metetra
.822 .734 .768 r
.71105 .42015 .71945 .43591 .73623 .41711 .72787 .40062 Metetra
.802 .72 .774 r
.72787 .40062 .73623 .41711 .75307 .39965 .74476 .38275 Metetra
.785 .709 .78 r
.74476 .38275 .75307 .39965 .77 .38348 .76176 .36642 Metetra
0 .243 .723 r
.13035 .61034 .1407 .61995 .1537 .62543 .14299 .61871 Metetra
0 .262 .676 r
.14299 .61871 .1537 .62543 .16706 .62922 .15599 .62577 Metetra
.264 0 0 r
.15599 .62577 .16706 .62922 .18085 .63058 .16943 .63055 Metetra
.151 0 0 r
.16943 .63055 .18085 .63058 .19509 .62887 .1834 .63209 Metetra
.1834 .63209 .19509 .62887 .20981 .62361 .19793 .62959 Metetra
.19793 .62959 .20981 .62361 .22498 .61451 .21303 .62245 Metetra
.51 .872 .643 r
.21303 .62245 .22498 .61451 .24056 .60157 .22867 .61045 Metetra
.703 .952 .677 r
.22867 .61045 .24056 .60157 .25646 .58499 .24476 .59367 Metetra
.838 .984 .704 r
.24476 .59367 .25646 .58499 .2726 .56526 .2612 .57259 Metetra
.916 .978 .721 r
.2612 .57259 .2726 .56526 .28888 .54304 .27787 .54799 Metetra
.95 .949 .729 r
.27787 .54799 .28888 .54304 .30522 .5191 .29463 .52086 Metetra
.955 .907 .733 r
.29463 .52086 .30522 .5191 .32151 .4943 .31137 .49232 Metetra
.941 .86 .733 r
.31137 .49232 .32151 .4943 .33771 .46948 .32799 .46352 Metetra
.915 .813 .731 r
.32799 .46352 .33771 .46948 .35376 .44541 .34441 .43549 Metetra
.928 .745 .634 r
.34441 .43549 .35376 .44541 .37191 .38024 .36207 .38341 Metetra
.925 .59 .385 r
.36207 .38341 .37191 .38024 .38291 .45238 .37233 .46711 Metetra
.886 .549 .404 r
.37233 .46711 .38291 .45238 .39504 .51857 .38391 .54405 Metetra
.841 .503 .415 r
.38391 .54405 .39504 .51857 .40829 .57672 .39682 .61131 Metetra
.79 .453 .419 r
.39682 .61131 .40829 .57672 .42262 .62515 .41102 .66654 Metetra
.732 .398 .415 r
.41102 .66654 .42262 .62515 .43791 .66275 .42637 .70821 Metetra
.668 .336 .403 r
.42637 .70821 .43791 .66275 .454 .68907 .4427 .73573 Metetra
.591 .263 .381 r
.4427 .73573 .454 .68907 .47074 .70429 .45977 .74946 Metetra
.497 .173 .346 r
.45977 .74946 .47074 .70429 .48792 .70918 .47735 .75056 Metetra
.375 .056 .291 r
.47735 .75056 .48792 .70918 .5054 .7049 .49523 .74076 Metetra
.205 0 .201 r
.49523 .74076 .5054 .7049 .52301 .69293 .51322 .72215 Metetra
0 0 .056 r
.51322 .72215 .52301 .69293 .54064 .67485 .53117 .69691 Metetra
.53117 .69691 .54064 .67485 .5582 .65222 .54899 .66712 Metetra
.698 .837 .404 r
.54899 .66712 .5582 .65222 .57564 .62651 .56662 .63462 Metetra
.901 .943 .583 r
.56662 .63462 .57564 .62651 .59293 .599 .58405 .60098 Metetra
.967 .937 .671 r
.58405 .60098 .59293 .599 .61008 .57075 .60128 .56741 Metetra
.964 .891 .706 r
.60128 .56741 .61008 .57075 .62708 .54262 .61834 .53483 Metetra
.939 .842 .721 r
.61834 .53483 .62708 .54262 .64397 .51524 .63526 .50389 Metetra
.909 .8 .728 r
.63526 .50389 .64397 .51524 .66078 .48906 .65209 .47498 Metetra
.879 .765 .733 r
.65209 .47498 .66078 .48906 .67754 .4644 .66886 .44832 Metetra
.852 .738 .738 r
.66886 .44832 .67754 .4644 .69428 .44141 .68564 .42396 Metetra
.828 .718 .743 r
.68564 .42396 .69428 .44141 .71105 .42015 .70244 .40188 Metetra
.806 .702 .751 r
.70244 .40188 .71105 .42015 .72787 .40062 .7193 .38195 Metetra
.787 .691 .759 r
.7193 .38195 .72787 .40062 .74476 .38275 .73626 .364 Metetra
.77 .684 .768 r
.73626 .364 .74476 .38275 .76176 .36642 .75334 .34785 Metetra
0 .098 .633 r
.12026 .59773 .13035 .61034 .14299 .61871 .13248 .60931 Metetra
0 .051 .533 r
.13248 .60931 .14299 .61871 .15599 .62577 .14498 .62039 Metetra
.52 0 0 r
.14498 .62039 .15599 .62577 .16943 .63055 .15791 .62971 Metetra
.471 0 0 r
.15791 .62971 .16943 .63055 .1834 .63209 .17141 .6359 Metetra
.324 0 0 r
.17141 .6359 .1834 .63209 .19793 .62959 .18558 .63768 Metetra
.103 0 0 r
.18558 .63768 .19793 .62959 .21303 .62245 .20047 .63395 Metetra
.20047 .63395 .21303 .62245 .22867 .61045 .21606 .62405 Metetra
.21606 .62405 .22867 .61045 .24476 .59367 .23231 .6078 Metetra
.684 .895 .536 r
.23231 .6078 .24476 .59367 .2612 .57259 .24907 .58555 Metetra
.851 .955 .61 r
.24907 .58555 .2612 .57259 .27787 .54799 .26619 .55819 Metetra
.94 .958 .659 r
.26619 .55819 .27787 .54799 .29463 .52086 .2835 .52701 Metetra
.97 .924 .686 r
.2835 .52701 .29463 .52086 .31137 .49232 .30081 .49351 Metetra
.964 .875 .699 r
.30081 .49351 .31137 .49232 .32799 .46352 .318 .45928 Metetra
.937 .82 .704 r
.318 .45928 .32799 .46352 .34441 .43549 .33493 .42585 Metetra
.911 .762 .683 r
.33493 .42585 .34441 .43549 .36207 .38341 .35185 .38955 Metetra
.924 .597 .405 r
.35185 .38955 .36207 .38341 .37233 .46711 .36109 .48896 Metetra
.879 .552 .426 r
.36109 .48896 .37233 .46711 .38391 .54405 .37187 .58058 Metetra
.828 .502 .437 r
.37187 .58058 .38391 .54405 .39682 .61131 .38429 .66019 Metetra
.772 .449 .44 r
.38429 .66019 .39682 .61131 .41102 .66654 .39831 .72435 Metetra
.703 .384 .433 r
.39831 .72435 .41102 .66654 .42637 .70821 .41385 .76849 Metetra
.456 .157 .368 r
.41385 .76849 .42637 .70821 .4427 .73573 .43146 .76337 Metetra
.374 .004 .202 r
.43146 .76337 .4427 .73573 .45977 .74946 .44919 .75822 Metetra
.082 0 0 r
.44919 .75822 .45977 .74946 .47735 .75056 .46704 .75302 Metetra
.229 .742 .717 r
.46704 .75302 .47735 .75056 .49523 .74076 .48502 .7478 Metetra
.016 0 0 r
.48502 .7478 .49523 .74076 .51322 .72215 .50312 .74253 Metetra
.50312 .74253 .51322 .72215 .53117 .69691 .52138 .7219 Metetra
.52138 .7219 .53117 .69691 .54899 .66712 .53951 .68243 Metetra
.831 .878 .442 r
.53951 .68243 .54899 .66712 .56662 .63462 .55737 .64118 Metetra
.965 .924 .59 r
.55737 .64118 .56662 .63462 .58405 .60098 .57494 .59999 Metetra
.982 .887 .656 r
.57494 .59999 .58405 .60098 .60128 .56741 .59226 .56021 Metetra
.956 .834 .682 r
.59226 .56021 .60128 .56741 .61834 .53483 .60936 .52274 Metetra
.921 .786 .694 r
.60936 .52274 .61834 .53483 .63526 .50389 .6263 .48814 Metetra
.887 .748 .702 r
.6263 .48814 .63526 .50389 .65209 .47498 .64315 .45666 Metetra
.856 .718 .709 r
.64315 .45666 .65209 .47498 .66886 .44832 .65995 .42835 Metetra
.829 .695 .716 r
.65995 .42835 .66886 .44832 .68564 .42396 .67675 .4031 Metetra
.806 .679 .725 r
.67675 .4031 .68564 .42396 .70244 .40188 .6936 .38071 Metetra
.785 .668 .735 r
.6936 .38071 .70244 .40188 .7193 .38195 .71054 .3609 Metetra
.767 .662 .746 r
.71054 .3609 .7193 .38195 .73626 .364 .72758 .34338 Metetra
.751 .659 .759 r
.72758 .34338 .73626 .364 .75334 .34785 .74476 .32786 Metetra
0 .013 .584 r
.11065 .58081 .12026 .59773 .13248 .60931 .1224 .59555 Metetra
0 0 .444 r
.1224 .59555 .13248 .60931 .14498 .62039 .13428 .61122 Metetra
.681 .167 0 r
.13428 .61122 .14498 .62039 .15791 .62971 .14651 .62635 Metetra
.698 .157 0 r
.14651 .62635 .15791 .62971 .17141 .6359 .15928 .63909 Metetra
.605 .078 0 r
.15928 .63909 .17141 .6359 .18558 .63768 .17278 .64747 Metetra
.436 0 0 r
.17278 .64747 .18558 .63768 .20047 .63395 .18717 .64959 Metetra
.21 0 0 r
.18717 .64959 .20047 .63395 .21606 .62405 .20251 .64398 Metetra
.20251 .64398 .21606 .62405 .23231 .6078 .21878 .6298 Metetra
.21878 .6298 .23231 .6078 .24907 .58555 .23585 .60707 Metetra
.662 .826 .4 r
.23585 .60707 .24907 .58555 .26619 .55819 .25352 .57668 Metetra
.866 .92 .525 r
.25352 .57668 .26619 .55819 .2835 .52701 .27154 .54025 Metetra
.964 .93 .606 r
.27154 .54025 .2835 .52701 .30081 .49351 .28966 .49991 Metetra
.983 .889 .649 r
.28966 .49991 .30081 .49351 .318 .45928 .30764 .45801 Metetra
.961 .829 .668 r
.30764 .45801 .318 .45928 .33493 .42585 .3253 .41681 Metetra
.839 .793 .807 r
.3253 .41681 .33493 .42585 .35185 .38955 .34114 .40007 Metetra
.922 .604 .424 r
.34114 .40007 .35185 .38955 .36109 .48896 .34887 .52181 Metetra
.87 .552 .445 r
.34887 .52181 .36109 .48896 .37187 .58058 .35848 .63447 Metetra
.813 .497 .456 r
.35848 .63447 .37187 .58058 .38429 .66019 .37016 .7316 Metetra
.644 .336 .436 r
.37016 .7316 .38429 .66019 .39831 .72435 .38579 .76416 Metetra
.416 .082 .288 r
.38579 .76416 .39831 .72435 .41385 .76849 .40335 .75901 Metetra
.623 .782 .958 r
.40335 .75901 .41385 .76849 .43146 .76337 .42104 .75383 Metetra
.623 .782 .958 r
.42104 .75383 .43146 .76337 .44919 .75822 .43884 .7486 Metetra
.623 .782 .958 r
.43884 .7486 .44919 .75822 .46704 .75302 .45678 .74334 Metetra
.623 .782 .958 r
.45678 .74334 .46704 .75302 .48502 .7478 .47484 .73805 Metetra
.623 .782 .958 r
.47484 .73805 .48502 .7478 .50312 .74253 .49302 .73272 Metetra
.623 .782 .958 r
.49302 .73272 .50312 .74253 .52138 .7219 .51134 .72734 Metetra
.929 .964 .667 r
.51134 .72734 .52138 .7219 .53951 .68243 .52976 .69588 Metetra
.945 .892 .493 r
.52976 .69588 .53951 .68243 .55737 .64118 .54786 .64345 Metetra
.993 .874 .598 r
.54786 .64345 .55737 .64118 .57494 .59999 .56559 .59323 Metetra
.969 .819 .64 r
.56559 .59323 .57494 .59999 .59226 .56021 .58298 .54653 Metetra
.929 .766 .659 r
.58298 .54653 .59226 .56021 .60936 .52274 .60012 .50409 Metetra
.889 .723 .669 r
.60012 .50409 .60936 .52274 .6263 .48814 .61708 .46618 Metetra
.854 .69 .678 r
.61708 .46618 .6263 .48814 .64315 .45666 .63394 .43278 Metetra
.825 .667 .687 r
.63394 .43278 .64315 .45666 .65995 .42835 .65077 .40362 Metetra
.799 .65 .697 r
.65077 .40362 .65995 .42835 .67675 .4031 .66763 .37833 Metetra
.777 .639 .709 r
.66763 .37833 .67675 .4031 .6936 .38071 .68456 .35646 Metetra
.759 .634 .722 r
.68456 .35646 .6936 .38071 .71054 .3609 .70159 .33756 Metetra
.743 .632 .737 r
.70159 .33756 .71054 .3609 .72758 .34338 .71875 .32116 Metetra
.729 .635 .753 r
.71875 .32116 .72758 .34338 .74476 .32786 .73606 .30684 Metetra
0 .015 .586 r
.10176 .55818 .11065 .58081 .1224 .59555 .11311 .57511 Metetra
0 0 .437 r
.11311 .57511 .1224 .59555 .13428 .61122 .12434 .59525 Metetra
.721 .299 0 r
.12434 .59525 .13428 .61122 .14651 .62635 .13565 .61718 Metetra
.827 .356 0 r
.13565 .61718 .14651 .62635 .15928 .63909 .14733 .63874 Metetra
.79 .32 .116 r
.14733 .63874 .15928 .63909 .17278 .64747 .15972 .65714 Metetra
.666 .222 .186 r
.15972 .65714 .17278 .64747 .18717 .64959 .17311 .66925 Metetra
.493 .086 .196 r
.17311 .66925 .18717 .64959 .20251 .64398 .18774 .6721 Metetra
.278 0 .154 r
.18774 .6721 .20251 .64398 .21878 .6298 .20371 .66344 Metetra
.008 0 .059 r
.20371 .66344 .21878 .6298 .23585 .60707 .22095 .64225 Metetra
.22095 .64225 .23585 .60707 .25352 .57668 .23923 .60903 Metetra
.653 .765 .289 r
.23923 .60903 .25352 .57668 .27154 .54025 .25822 .56578 Metetra
.892 .888 .463 r
.25822 .56578 .27154 .54025 .28966 .49991 .2775 .5156 Metetra
.986 .894 .57 r
.2775 .5156 .28966 .49991 .30764 .45801 .29668 .46215 Metetra
.983 .838 .621 r
.29668 .46215 .30764 .45801 .3253 .41681 .31544 .40909 Metetra
0 .26 .709 r
.31544 .40909 .3253 .41681 .34114 .40007 .3297 .41747 Metetra
.919 .609 .441 r
.3297 .41747 .34114 .40007 .34887 .52181 .3351 .5729 Metetra
.859 .55 .464 r
.3351 .5729 .34887 .52181 .35848 .63447 .34285 .71774 Metetra
.657 .355 .449 r
.34285 .71774 .35848 .63447 .37016 .7316 .35757 .75981 Metetra
.365 0 .191 r
.35757 .75981 .37016 .7316 .38579 .76416 .37508 .75463 Metetra
.623 .782 .958 r
.37508 .75463 .38579 .76416 .40335 .75901 .39272 .74941 Metetra
.623 .782 .958 r
.39272 .74941 .40335 .75901 .42104 .75383 .41048 .74416 Metetra
.623 .782 .958 r
.41048 .74416 .42104 .75383 .43884 .7486 .42837 .73887 Metetra
.623 .782 .958 r
.42837 .73887 .43884 .7486 .45678 .74334 .44638 .73354 Metetra
.623 .782 .958 r
.44638 .73354 .45678 .74334 .47484 .73805 .46452 .72817 Metetra
.623 .782 .958 r
.46452 .72817 .47484 .73805 .49302 .73272 .48279 .72277 Metetra
.623 .782 .958 r
.48279 .72277 .49302 .73272 .51134 .72734 .50119 .71733 Metetra
.805 .827 .868 r
.50119 .71733 .51134 .72734 .52976 .69588 .51974 .70257 Metetra
.998 .853 .539 r
.51974 .70257 .52976 .69588 .54786 .64345 .53812 .63639 Metetra
.974 .795 .599 r
.53812 .63639 .54786 .64345 .56559 .59323 .55599 .57607 Metetra
.928 .736 .623 r
.55599 .57607 .56559 .59323 .58298 .54653 .57344 .52251 Metetra
.882 .689 .637 r
.57344 .52251 .58298 .54653 .60012 .50409 .59061 .4759 Metetra
.844 .655 .647 r
.59061 .4759 .60012 .50409 .61708 .46618 .60758 .43595 Metetra
.811 .63 .657 r
.60758 .43595 .61708 .46618 .63394 .43278 .62448 .40208 Metetra
.784 .613 .668 r
.62448 .40208 .63394 .43278 .65077 .40362 .64136 .37357 Metetra
.762 .604 .682 r
.64136 .37357 .65077 .40362 .66763 .37833 .6583 .34964 Metetra
.743 .6 .697 r
.6583 .34964 .66763 .37833 .68456 .35646 .67534 .32953 Metetra
.728 .601 .715 r
.67534 .32953 .68456 .35646 .70159 .33756 .69251 .31256 Metetra
.715 .607 .735 r
.69251 .31256 .70159 .33756 .71875 .32116 .70982 .29809 Metetra
.705 .617 .756 r
.70982 .29809 .71875 .32116 .73606 .30684 .7273 .2856 Metetra
.012 .112 .623 r
.09373 .52927 .10176 .55818 .11311 .57511 .10503 .54562 Metetra
0 0 .502 r
.10503 .54562 .11311 .57511 .12434 .59525 .11579 .56815 Metetra
0 0 .312 r
.11579 .56815 .12434 .59525 .13565 .61718 .12615 .59632 Metetra
.828 .457 0 r
.12615 .59632 .13565 .61718 .14733 .63874 .13642 .62839 Metetra
.906 .495 .158 r
.13642 .62839 .14733 .63874 .15972 .65714 .14705 .66115 Metetra
.833 .43 .283 r
.14705 .66115 .15972 .65714 .17311 .66925 .15857 .68999 Metetra
.694 .318 .33 r
.15857 .68999 .17311 .66925 .18774 .6721 .17153 .70945 Metetra
.525 .181 .327 r
.17153 .70945 .18774 .6721 .20371 .66344 .18637 .71419 Metetra
.32 .015 .281 r
.18637 .71419 .20371 .66344 .22095 .64225 .20328 .70033 Metetra
.058 0 .184 r
.20328 .70033 .22095 .64225 .23923 .60903 .2221 .66657 Metetra
0 0 .017 r
.2221 .66657 .23923 .60903 .25822 .56578 .24238 .61477 Metetra
.673 .727 .218 r
.24238 .61477 .25822 .56578 .2775 .5156 .26346 .54956 Metetra
.934 .863 .434 r
.26346 .54956 .2775 .5156 .29668 .46215 .28461 .47722 Metetra
.998 .844 .554 r
.28461 .47722 .29668 .46215 .31544 .40909 .30523 .40435 Metetra
.911 .516 .235 r
.30523 .40435 .31544 .40909 .3297 .41747 .31712 .44662 Metetra
.915 .612 .458 r
.31712 .44662 .3297 .41747 .3351 .5729 .31858 .65741 Metetra
.762 .458 .47 r
.31858 .65741 .3351 .5729 .34285 .71774 .32918 .75542 Metetra
.399 .055 .262 r
.32918 .75542 .34285 .71774 .35757 .75981 .34664 .75021 Metetra
.623 .782 .958 r
.34664 .75021 .35757 .75981 .37508 .75463 .36423 .74497 Metetra
.623 .782 .958 r
.36423 .74497 .37508 .75463 .39272 .74941 .38195 .73968 Metetra
.623 .782 .958 r
.38195 .73968 .39272 .74941 .41048 .74416 .39979 .73436 Metetra
.623 .782 .958 r
.39979 .73436 .41048 .74416 .42837 .73887 .41775 .729 Metetra
.623 .782 .958 r
.41775 .729 .42837 .73887 .44638 .73354 .43585 .7236 Metetra
.623 .782 .958 r
.43585 .7236 .44638 .73354 .46452 .72817 .45407 .71817 Metetra
.623 .782 .958 r
.45407 .71817 .46452 .72817 .48279 .72277 .47242 .71269 Metetra
.623 .782 .958 r
.47242 .71269 .48279 .72277 .50119 .71733 .49091 .70718 Metetra
.802 .827 .871 r
.49091 .70718 .50119 .71733 .51974 .70257 .50957 .69258 Metetra
.97 .76 .564 r
.50957 .69258 .51974 .70257 .53812 .63639 .52822 .61119 Metetra
.914 .695 .592 r
.52822 .61119 .53812 .63639 .55599 .57607 .5462 .54149 Metetra
.863 .645 .606 r
.5462 .54149 .55599 .57607 .57344 .52251 .56368 .48311 Metetra
.82 .609 .617 r
.56368 .48311 .57344 .52251 .59061 .4759 .58085 .43503 Metetra
.786 .584 .627 r
.58085 .43503 .59061 .4759 .60758 .43595 .59785 .39591 Metetra
.759 .568 .64 r
.59785 .39591 .60758 .43595 .62448 .40208 .6148 .3643 Metetra
.737 .561 .655 r
.6148 .3643 .62448 .40208 .64136 .37357 .63178 .33883 Metetra
.719 .56 .673 r
.63178 .33883 .64136 .37357 .6583 .34964 .64886 .31823 Metetra
.706 .565 .694 r
.64886 .31823 .6583 .34964 .67534 .32953 .66606 .30141 Metetra
.695 .576 .718 r
.66606 .30141 .67534 .32953 .69251 .31256 .6834 .28748 Metetra
.687 .592 .744 r
.6834 .28748 .69251 .31256 .70982 .29809 .70091 .27569 Metetra
.681 .611 .771 r
.70091 .27569 .70982 .29809 .7273 .2856 .71858 .26545 Metetra
.283 .273 .671 r
.08623 .49687 .09373 .52927 .10503 .54562 .09821 .50736 Metetra
.1 .115 .588 r
.09821 .50736 .10503 .54562 .11579 .56815 .10927 .5262 Metetra
0 0 .467 r
.10927 .5262 .11579 .56815 .12615 .59632 .11929 .55523 Metetra
0 0 .267 r
.11929 .55523 .12615 .59632 .13642 .62839 .1283 .59517 Metetra
.849 .555 .02 r
.1283 .59517 .13642 .62839 .14705 .66115 .13666 .64461 Metetra
.952 .6 .265 r
.13666 .64461 .14705 .66115 .15857 .68999 .14503 .69903 Metetra
.865 .514 .385 r
.14503 .69903 .15857 .68999 .17153 .70945 .15441 .75033 Metetra
.719 .39 .42 r
.15441 .75033 .17153 .70945 .18637 .71419 .16598 .7874 Metetra
.482 .185 .385 r
.16598 .7874 .18637 .71419 .20328 .70033 .182 .78671 Metetra
.419 .137 .377 r
.182 .78671 .20328 .70033 .2221 .66657 .19926 .77511 Metetra
.092 0 .269 r
.19926 .77511 .2221 .66657 .24238 .61477 .2211 .71541 Metetra
0 0 .088 r
.2211 .71541 .24238 .61477 .26346 .54956 .24519 .62571 Metetra
.738 .724 .198 r
.24519 .62571 .26346 .54956 .28461 .47722 .27004 .51829 Metetra
.984 .836 .447 r
.27004 .51829 .28461 .47722 .30523 .40435 .29423 .40733 Metetra
.926 .595 .391 r
.29423 .40733 .30523 .40435 .31712 .44662 .30245 .49845 Metetra
.898 .599 .472 r
.30245 .49845 .31712 .44662 .31858 .65741 .30062 .75102 Metetra
.464 .182 .399 r
.30062 .75102 .31858 .65741 .32918 .75542 .31804 .74578 Metetra
.623 .782 .958 r
.31804 .74578 .32918 .75542 .34664 .75021 .33558 .7405 Metetra
.623 .782 .958 r
.33558 .7405 .34664 .75021 .36423 .74497 .35325 .73518 Metetra
.623 .782 .958 r
.35325 .73518 .36423 .74497 .38195 .73968 .37104 .72983 Metetra
.623 .782 .958 r
.37104 .72983 .38195 .73968 .39979 .73436 .38895 .72444 Metetra
.623 .782 .958 r
.38895 .72444 .39979 .73436 .41775 .729 .407 .71901 Metetra
.623 .782 .958 r
.407 .71901 .41775 .729 .43585 .7236 .42517 .71354 Metetra
.623 .782 .958 r
.42517 .71354 .43585 .7236 .45407 .71817 .44348 .70803 Metetra
.623 .782 .958 r
.44348 .70803 .45407 .71817 .47242 .71269 .46191 .70248 Metetra
.623 .782 .958 r
.46191 .70248 .47242 .71269 .49091 .70718 .48048 .69689 Metetra
.95 .789 .647 r
.48048 .69689 .49091 .70718 .50957 .69258 .49956 .64748 Metetra
.881 .638 .567 r
.49956 .64748 .50957 .69258 .52822 .61119 .51837 .55433 Metetra
.824 .586 .579 r
.51837 .55433 .52822 .61119 .5462 .54149 .53634 .48087 Metetra
.78 .551 .589 r
.53634 .48087 .5462 .54149 .56368 .48311 .55379 .42395 Metetra
.745 .528 .6 r
.55379 .42395 .56368 .48311 .58085 .43503 .57095 .38039 Metetra
.719 .515 .613 r
.57095 .38039 .58085 .43503 .59785 .39591 .588 .34726 Metetra
.699 .511 .63 r
.588 .34726 .59785 .39591 .6148 .3643 .60505 .32204 Metetra
.684 .515 .651 r
.60505 .32204 .6148 .3643 .63178 .33883 .62219 .30267 Metetra
.674 .527 .678 r
.62219 .30267 .63178 .33883 .64886 .31823 .63945 .28749 Metetra
.668 .547 .708 r
.63945 .28749 .64886 .31823 .66606 .30141 .65686 .27523 Metetra
.665 .572 .741 r
.65686 .27523 .66606 .30141 .6834 .28748 .67441 .26498 Metetra
.663 .601 .775 r
.67441 .26498 .6834 .28748 .70091 .27569 .69212 .25603 Metetra
.663 .631 .808 r
.69212 .25603 .70091 .27569 .71858 .26545 .70998 .24793 Metetra
.521 .467 .736 r
.07789 .4704 .08623 .49687 .09821 .50736 .09132 .47023 Metetra
.425 .347 .665 r
.09132 .47023 .09821 .50736 .10927 .5262 .10407 .47582 Metetra
.292 .211 .59 r
.10407 .47582 .10927 .5262 .11929 .55523 .11562 .49148 Metetra
.098 .041 .498 r
.11562 .49148 .11929 .55523 .1283 .59517 .12539 .52283 Metetra
0 0 .352 r
.12539 .52283 .1283 .59517 .13666 .64461 .13285 .57566 Metetra
0 0 .077 r
.13285 .57566 .13666 .64461 .14503 .69903 .13786 .65367 Metetra
.966 .687 .264 r
.13786 .65367 .14503 .69903 .15441 .75033 .14105 .75463 Metetra
.82 .403 .252 r
.14105 .75463 .15441 .75033 .16598 .7874 .15362 .78249 Metetra
.579 .862 .996 r
.15362 .78249 .16598 .7874 .182 .78671 .17017 .77746 Metetra
.623 .782 .958 r
.17017 .77746 .182 .78671 .19926 .77511 .18683 .77241 Metetra
.522 .883 .98 r
.18683 .77241 .19926 .77511 .2211 .71541 .20361 .76731 Metetra
.422 .102 .315 r
.20361 .76731 .2211 .71541 .24519 .62571 .2205 .76218 Metetra
0 0 .115 r
.2205 .76218 .24519 .62571 .27004 .51829 .2477 .64146 Metetra
.862 .76 .251 r
.2477 .64146 .27004 .51829 .29423 .40733 .28088 .43604 Metetra
.903 .597 .455 r
.28088 .43604 .29423 .40733 .30245 .49845 .28301 .60391 Metetra
.778 .479 .481 r
.28301 .60391 .30245 .49845 .30062 .75102 .28926 .74131 Metetra
.623 .782 .958 r
.28926 .74131 .30062 .75102 .31804 .74578 .30675 .736 Metetra
.623 .782 .958 r
.30675 .736 .31804 .74578 .33558 .7405 .32437 .73065 Metetra
.623 .782 .958 r
.32437 .73065 .33558 .7405 .35325 .73518 .34211 .72527 Metetra
.623 .782 .958 r
.34211 .72527 .35325 .73518 .37104 .72983 .35998 .71984 Metetra
.623 .782 .958 r
.35998 .71984 .37104 .72983 .38895 .72444 .37798 .71438 Metetra
.623 .782 .958 r
.37798 .71438 .38895 .72444 .407 .71901 .3961 .70888 Metetra
.623 .782 .958 r
.3961 .70888 .407 .71901 .42517 .71354 .41436 .70334 Metetra
.623 .782 .958 r
.41436 .70334 .42517 .71354 .44348 .70803 .43275 .69776 Metetra
.623 .782 .958 r
.43275 .69776 .44348 .70803 .46191 .70248 .45127 .69213 Metetra
.9 .818 .757 r
.45127 .69213 .46191 .70248 .48048 .69689 .47031 .66428 Metetra
.93 .681 .539 r
.47031 .66428 .48048 .69689 .49956 .64748 .49036 .54079 Metetra
.758 .51 .557 r
.49036 .54079 .49956 .64748 .51837 .55433 .50894 .45284 Metetra
.715 .478 .564 r
.50894 .45284 .51837 .55433 .53634 .48087 .52668 .3913 Metetra
.684 .459 .575 r
.52668 .3913 .53634 .48087 .55379 .42395 .54398 .34872 Metetra
.661 .451 .59 r
.54398 .34872 .55379 .42395 .57095 .38039 .56113 .31929 Metetra
.646 .455 .611 r
.56113 .31929 .57095 .38039 .588 .34726 .57826 .29868 Metetra
.638 .47 .639 r
.57826 .29868 .588 .34726 .60505 .32204 .59548 .28376 Metetra
.636 .496 .675 r
.59548 .28376 .60505 .32204 .62219 .30267 .61282 .27239 Metetra
.639 .532 .717 r
.61282 .27239 .62219 .30267 .63945 .28749 .63029 .26312 Metetra
.644 .575 .762 r
.63029 .26312 .63945 .28749 .65686 .27523 .6479 .25505 Metetra
.65 .618 .805 r
.6479 .25505 .65686 .27523 .67441 .26498 .66565 .24763 Metetra
.654 .657 .84 r
.66565 .24763 .67441 .26498 .69212 .25603 .68352 .24055 Metetra
.657 .687 .867 r
.68352 .24055 .69212 .25603 .70998 .24793 .70152 .23361 Metetra
.651 .678 .863 r
.06751 .45715 .07789 .4704 .09132 .47023 .08172 .45183 Metetra
.629 .599 .801 r
.08172 .45183 .09132 .47023 .10407 .47582 .09598 .44668 Metetra
.587 .492 .714 r
.09598 .44668 .10407 .47582 .11562 .49148 .1102 .44241 Metetra
.526 .384 .631 r
.1102 .44241 .11562 .49148 .12539 .52283 .12407 .44144 Metetra
.445 .281 .563 r
.12407 .44144 .12539 .52283 .13285 .57566 .13676 .4504 Metetra
.321 .16 .5 r
.13676 .4504 .13285 .57566 .13786 .65367 .14655 .48404 Metetra
.083 0 .403 r
.14655 .48404 .13786 .65367 .14105 .75463 .15057 .5688 Metetra
0 0 .143 r
.15057 .5688 .14105 .75463 .15362 .78249 .1452 .74211 Metetra
0 .013 .545 r
.1452 .74211 .15362 .78249 .17017 .77746 .15818 .7681 Metetra
.623 .782 .958 r
.15818 .7681 .17017 .77746 .18683 .77241 .17491 .76298 Metetra
.623 .782 .958 r
.17491 .76298 .18683 .77241 .20361 .76731 .19175 .75782 Metetra
.623 .782 .958 r
.19175 .75782 .20361 .76731 .2205 .76218 .20871 .75262 Metetra
.623 .782 .958 r
.20871 .75262 .2205 .76218 .2477 .64146 .22579 .74739 Metetra
.22579 .74739 .2477 .64146 .28088 .43604 .25333 .62318 Metetra
.68 .397 .486 r
.25333 .62318 .28088 .43604 .28301 .60391 .26031 .73682 Metetra
.47 .198 .419 r
.26031 .73682 .28301 .60391 .28926 .74131 .27776 .73148 Metetra
.623 .782 .958 r
.27776 .73148 .28926 .74131 .30675 .736 .29532 .7261 Metetra
.623 .782 .958 r
.29532 .7261 .30675 .736 .32437 .73065 .31301 .72068 Metetra
.623 .782 .958 r
.31301 .72068 .32437 .73065 .34211 .72527 .33083 .71522 Metetra
.623 .782 .958 r
.33083 .71522 .34211 .72527 .35998 .71984 .34878 .70973 Metetra
.623 .782 .958 r
.34878 .70973 .35998 .71984 .37798 .71438 .36685 .70419 Metetra
.623 .782 .958 r
.36685 .70419 .37798 .71438 .3961 .70888 .38506 .69862 Metetra
.623 .782 .958 r
.38506 .69862 .3961 .70888 .41436 .70334 .4034 .693 Metetra
.623 .782 .958 r
.4034 .693 .41436 .70334 .43275 .69776 .42187 .68735 Metetra
.968 .744 .547 r
.42187 .68735 .43275 .69776 .45127 .69213 .44305 .58557 Metetra
.82 .558 .543 r
.44305 .58557 .45127 .69213 .47031 .66428 .46399 .4487 Metetra
.652 .406 .535 r
.46399 .4487 .47031 .66428 .49036 .54079 .48264 .37004 Metetra
.615 .382 .541 r
.48264 .37004 .49036 .54079 .50894 .45284 .50022 .32592 Metetra
.592 .374 .554 r
.50022 .32592 .50894 .45284 .52668 .3913 .51741 .30111 Metetra
.582 .382 .576 r
.51741 .30111 .52668 .3913 .54398 .34872 .53451 .28634 Metetra
.583 .408 .611 r
.53451 .28634 .54398 .34872 .56113 .31929 .55167 .27637 Metetra
.594 .454 .66 r
.55167 .27637 .56113 .31929 .57826 .29868 .56893 .26849 Metetra
.613 .517 .722 r
.56893 .26849 .57826 .29868 .59548 .28376 .58631 .26143 Metetra
.632 .586 .784 r
.58631 .26143 .59548 .28376 .61282 .27239 .60381 .25466 Metetra
.646 .647 .836 r
.60381 .25466 .61282 .27239 .63029 .26312 .62142 .24795 Metetra
.654 .69 .871 r
.62142 .24795 .63029 .26312 .6479 .25505 .63916 .24124 Metetra
.657 .717 .892 r
.63916 .24124 .6479 .25505 .66565 .24763 .65701 .23449 Metetra
.659 .731 .903 r
.65701 .23449 .66565 .24763 .68352 .24055 .67498 .2277 Metetra
.659 .739 .909 r
.67498 .2277 .68352 .24055 .70152 .23361 .69308 .22087 Metetra
.66 .746 .914 r
.0565 .4472 .06751 .45715 .08172 .45183 .07076 .44178 Metetra
.66 .745 .913 r
.07076 .44178 .08172 .45183 .09598 .44668 .0851 .43632 Metetra
.659 .74 .909 r
.0851 .43632 .09598 .44668 .1102 .44241 .09954 .43082 Metetra
.657 .716 .891 r
.09954 .43082 .1102 .44241 .12407 .44144 .11406 .42529 Metetra
.643 .637 .83 r
.11406 .42529 .12407 .44144 .13676 .4504 .12866 .41973 Metetra
.6 .494 .706 r
.12866 .41973 .13676 .4504 .14655 .48404 .14336 .41414 Metetra
.551 .372 .593 r
.14336 .41414 .14655 .48404 .15057 .5688 .15815 .40851 Metetra
.523 .311 .534 r
.15815 .40851 .15057 .5688 .1452 .74211 .17303 .40285 Metetra
.51 .284 .508 r
.17303 .40285 .1452 .74211 .15818 .7681 .188 .39715 Metetra
.509 .282 .506 r
.188 .39715 .15818 .7681 .17491 .76298 .20306 .39141 Metetra
.509 .282 .506 r
.20306 .39141 .17491 .76298 .19175 .75782 .21822 .38564 Metetra
.509 .282 .506 r
.21822 .38564 .19175 .75782 .20871 .75262 .23347 .37984 Metetra
.509 .282 .506 r
.23347 .37984 .20871 .75262 .22579 .74739 .24882 .37399 Metetra
0 0 .154 r
.24882 .37399 .22579 .74739 .25333 .62318 .23119 .7323 Metetra
.459 .178 .399 r
.23119 .7323 .25333 .62318 .26031 .73682 .24858 .72693 Metetra
.959 .701 .501 r
.24858 .72693 .26031 .73682 .27776 .73148 .29545 .35624 Metetra
.509 .282 .506 r
.29545 .35624 .27776 .73148 .29532 .7261 .31118 .35025 Metetra
.509 .282 .506 r
.31118 .35025 .29532 .7261 .31301 .72068 .32702 .34422 Metetra
0 0 .209 r
.32702 .34422 .31301 .72068 .33083 .71522 .3194 .70505 Metetra
.623 .782 .958 r
.3194 .70505 .33083 .71522 .34878 .70973 .33743 .69948 Metetra
.967 .702 .479 r
.33743 .69948 .34878 .70973 .36685 .70419 .37515 .3259 Metetra
.509 .282 .506 r
.37515 .3259 .36685 .70419 .38506 .69862 .39139 .31972 Metetra
.509 .282 .506 r
.39139 .31972 .38506 .69862 .4034 .693 .40775 .31349 Metetra
.509 .282 .506 r
.40775 .31349 .4034 .693 .42187 .68735 .42421 .30723 Metetra
.509 .282 .506 r
.42421 .30723 .42187 .68735 .44305 .58557 .44077 .30092 Metetra
.513 .29 .513 r
.44077 .30092 .44305 .58557 .46399 .4487 .45745 .29458 Metetra
.524 .314 .537 r
.45745 .29458 .46399 .4487 .48264 .37004 .47423 .28819 Metetra
.545 .36 .581 r
.47423 .28819 .48264 .37004 .50022 .32592 .49112 .28176 Metetra
.579 .44 .656 r
.49112 .28176 .50022 .32592 .51741 .30111 .50813 .27528 Metetra
.618 .547 .753 r
.50813 .27528 .51741 .30111 .53451 .28634 .52524 .26877 Metetra
.645 .646 .836 r
.52524 .26877 .53451 .28634 .55167 .27637 .54247 .26221 Metetra
.656 .705 .883 r
.54247 .26221 .55167 .27637 .56893 .26849 .55982 .25561 Metetra
.659 .731 .903 r
.55982 .25561 .56893 .26849 .58631 .26143 .57728 .24896 Metetra
.66 .741 .91 r
.57728 .24896 .58631 .26143 .60381 .25466 .59485 .24227 Metetra
.66 .745 .913 r
.59485 .24227 .60381 .25466 .62142 .24795 .61255 .23553 Metetra
.66 .746 .914 r
.61255 .23553 .62142 .24795 .63916 .24124 .63036 .22875 Metetra
.66 .746 .914 r
.63036 .22875 .63916 .24124 .65701 .23449 .6483 .22192 Metetra
.66 .746 .914 r
.6483 .22192 .65701 .23449 .67498 .2277 .66636 .21505 Metetra
.66 .746 .914 r
.66636 .21505 .67498 .2277 .69308 .22087 .68454 .20813 Metetra
0 g
.25 Mabswid
.67245 0 m
.92713 .41497 L
s
.92713 .41497 m
1 .94814 L
s
1 .94814 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.08846 .24555 m
0 .80374 L
s
0 .80374 m
.70644 .58546 L
s
.70644 .58546 m
.67245 0 L
s
.67245 0 m
.08846 .24555 L
s
.08846 .24555 m
.67245 0 L
s
.08846 .24555 m
.09356 .25076 L
s
[(80)] .07825 .23514 .98066 1 Mshowa
.22216 .18933 m
.227 .19478 L
s
[(90)] .21247 .17844 .88892 1 Mshowa
.36358 .12987 m
.36813 .13557 L
s
[(100)] .3545 .11847 .79718 1 Mshowa
.51343 .06687 m
.51763 .07282 L
s
[(110)] .50502 .05496 .70545 1 Mshowa
.67245 0 m
.67627 .00621 L
s
[(120)] .66483 -0.01242 .61371 1 Mshowa
.125 Mabswid
.11461 .23456 m
.11764 .23771 L
s
.14105 .22344 m
.14405 .22662 L
s
.16779 .2122 m
.17076 .21541 L
s
.19482 .20083 m
.19776 .20407 L
s
.2498 .17771 m
.25268 .18101 L
s
.27776 .16595 m
.2806 .16928 L
s
.30604 .15406 m
.30885 .15742 L
s
.33465 .14204 m
.33741 .14542 L
s
.39286 .11756 m
.39554 .12101 L
s
.42247 .10511 m
.42512 .10859 L
s
.45243 .09251 m
.45504 .09602 L
s
.48275 .07977 m
.48531 .08331 L
s
.54447 .05381 m
.54695 .05742 L
s
.57589 .0406 m
.57832 .04424 L
s
.60769 .02723 m
.61007 .0309 L
s
.63987 .0137 m
.64221 .0174 L
s
.25 Mabswid
0 .80374 m
.40037 1.0855 L
s
.09485 .87049 m
.09577 .86326 L
s
[(0.2)] .09301 .88495 .12735 -1 Mshowa
.18136 .93137 m
.18209 .92412 L
s
[(0.4)] .17991 .94587 .10002 -1 Mshowa
.26055 .98709 m
.2611 .97983 L
s
[(0.6)] .25944 1.00163 .07584 -1 Mshowa
.3333 1.03829 m
.33369 1.03102 L
s
[(0.8)] .33251 1.05285 .05428 -1 Mshowa
.40037 1.0855 m
.40063 1.07821 L
s
[(1)] .39987 1.10006 .03495 -1 Mshowa
.125 Mabswid
.02453 .821 m
.02518 .81668 L
s
.04852 .83789 m
.04914 .83356 L
s
.07196 .85438 m
.07254 .85004 L
s
.11722 .88623 m
.11774 .88189 L
s
.13908 .90162 m
.13958 .89727 L
s
.16046 .91666 m
.16092 .91231 L
s
.20181 .94576 m
.20221 .9414 L
s
.22181 .95983 m
.22219 .95548 L
s
.24138 .97361 m
.24174 .96925 L
s
.27931 1.0003 m
.27961 .99593 L
s
.29768 1.01322 m
.29796 1.00886 L
s
.31567 1.02589 m
.31593 1.02152 L
s
.35057 1.05045 m
.35079 1.04608 L
s
.3675 1.06236 m
.3677 1.058 L
s
.3841 1.07404 m
.38427 1.06967 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True,
  ImageSize->{185.688, 201.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002i0000bA000`40O003h00O^Goo002iOol00;Uo
o`00M7oo0`000goo10000goo0P00=Woo001eOol00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomo
o`00=Goo001eOol00`00Oomoo`04Ool01@00Oomoogoo00000Woo00<007ooOol0<goo001eOol00`00
Oomoo`05Ool01000Oomoo`000Woo00<007ooOol0<goo001eOol00`00Oomoo`02Ool01000Oomoo`00
0Woo00@007ooOol003Eoo`00M7oo0P001Goo0P0017oo0P00=Woo002iOol00;Uoo`00^Goo0022Ool2
000eOol007moo`<00003Ool0000003Aoo`00FGoo0`000goo0`0017oo0P005Goo0P0017oo0P00=7oo
001JOol00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`004Woo0P0000=oo`00Ool00goo00<0
07oo0000<goo001JOol00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`003goo0`0027oo00<0
07oo0000<goo001JOol00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`003Goo0P0000=oo`00
Ool027oo00@007ooOol0039oo`00FWoo00<007ooOol00goo00<007ooOol00Woo00@007ooOol000Yo
o`<000eoo`04001oogoo000bOol005Uoo`8000Aoo`8000Eoo`8000Uoo`@000ioo`05001oogooOol0
000aOol006moo`80019oo`03001oogoo009oo`03001oogoo02ioo`00K7oo10004goo00<007ooOol0
0Woo00<007ooOol0;Woo001ZOol2000HOol00`00Oomoo`02Ool00`00Oomoo`0]Ool006Qoo`8001Yo
o`03001oogoo009oo`03001oogoo02eoo`00@7oo0`0017oo0P0017oo0P005Woo0`000Woo00<007oo
Ool05goo00<007ooOol00goo00<007ooOol0;7oo0011Ool00`00Oomoo`02Ool01000Oomoo`000Woo
00@007ooOol001=oo`8001moo`03001oogoo00Aoo`03001oogoo02]oo`00@Goo00<007ooOol00Woo
00@007ooOol0009oo`04001oogoo000AOol2000QOol00`00Oomoo`04Ool00`00Oomoo`0[Ool0045o
o`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`003Woo0`008goo00<007ooOol01Goo00<0
07ooOol0:Woo0011Ool00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol000aoo`8002Ioo`03
001oogoo00Eoo`03001oogoo02Yoo`00@7oo0P001Goo0P0017oo0P002Woo0`00:7oo00<007ooOol0
1Woo00<007ooOol0:Goo001GOol2000[Ool00`00Oomoo`07Ool00`00Oomoo`0XOol005Eoo`8002eo
o`03001oogoo00Moo`03001oogoo02Qoo`00DWoo0`00;goo00<007ooOol027oo00<007ooOol09goo
001@Ool2000bOol00`00Oomoo`09Ool00`00Oomoo`0VOol002ioo`<000Aoo`8001Moo`800003Ool0
07oo035oo`03001oogoo00Uoo`03001oogoo02Ioo`00<Goo00@007ooOol0009oo`03001oogoo015o
o`<003Ioo`03001oogoo00Yoo`03001oogoo02Eoo`00;goo0`000Woo00@007ooOol0015oo`8003Uo
o`03001oogoo00Yoo`03001oogoo02Eoo`00;Woo00@007ooOol0009oo`04001oogoo000>Ool3000k
Ool00`00Oomoo`0;Ool00`00Oomoo`0TOol002ioo`04001oogoo0002Ool01000Oomoo`0037oo0P00
?Woo00<007ooOol037oo00<007ooOol08goo000_Ool20004Ool2000;Ool20010Ool00`00Oomoo`0<
Ool00`00Oomoo`0SOol003moo`<0049oo`03001oogoo00eoo`03001oogoo029oo`00?Goo0`00AGoo
00<007ooOol037oo00<007ooOol08Woo000kOol20018Ool00`00Oomoo`0=Ool00`00Oomoo`0QOol0
01Qoo`8000Aoo`8001Qoo`<004Yoo`03001oogoo00ioo`03001oogoo021oo`005goo00@007ooOol0
009oo`04001oogoo000DOol3001=Ool00`00Oomoo`0>Ool00`00Oomoo`0POol001Moo`04001oogoo
0002Ool01000Oomoo`004Woo1000CWoo00<007ooOol03goo00<007ooOol07goo000HOol20003Ool0
1000Oomoo`0047oo0P00DGoo0P004Woo00<007ooOol07Woo000GOol01000Oomoo`000Woo00@007oo
Ool000ioo`<004moo`<00003E_d007oo015oo`03001oogoo01ioo`0067oo0P0017oo0P0037oo0`00
Cgoo0`0000AFo@00E_eFo@80019oo`03001oogoo01eoo`00:Woo0P0000=oo`00Ool0C7oo0`0000AF
o@00E_eFo@@00003E_d007oo011oo`03001oogoo01eoo`00:7oo0P00CWoo0`000UKm00<005Km0000
0P000UKl0P0000=Fo@0000004Goo00<007ooOol077oo000UOol3001=Ool30002E_d00`00E_eFo@02
0002E]`00`00E_aFo00300000eJk001oo`0AOol00`00Oomoo`0KOol002Aoo`03001oogoo04]oo`<0
009Fo@03001FoEKm00<00004D]/005KLE]`3000015:j001F^eJk0P004Goo00<007ooOol06goo000T
Ool2001:Ool20002E_`00`00E_eFo@03000015:J001Bfe;K0`0000ABN@00D[YB^P<00003EYT007oo
015oo`03001oogoo01Yoo`0097oo00<007oo0000AWoo0`0000ABg000E_aFo0<00003DUT005:J009B
VP800003DUP0059i009BN@@000=FV@03001oogoo00moo`03001oogoo01Yoo`0097oo00@007ooOol0
04Aoo`03001BVP00009Bg0<00003CQL0059I009BF@04001BVP000002DUP00`00DWUBN@02000015Ih
001FN0000UJI00<007ooOol047oo00<007ooOol06Goo000SOol20003Ool00`00Oomoo`0oOol20003
DYX200001DgE001>5dhG00000U9I0`0000=B=P00DUP00U9H0P0000MFE`00EWP005Ih001FV@02000C
Ool00`00Oomoo`0HOol002=oo`03001oogoo00=oo`03001oogoo03]oo`<00004CSP005:JDYX20003
CMD00`00CQM>5`0200000e7e001B=P02DSH01000DUQBF0000eIG0P0000=FN00000000UIh00<007oo
Ool047oo00<007ooOol067oo000;Ool20004Ool20004Ool20003Ool40003Ool00`00Oomoo`04Ool0
0`00Oomoo`0gOol300000dWD001>>002CSP200000dVc001=e@02CMD01000CQM>5`000e7e00D0058f
DSH0059H008000=FE`<00004EUL005IhEWP2000BOol00`00Oomoo`0GOol000Yoo`04001oogoo0003
Ool20003Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo00<007ooOol0=7oo0P00
00=5LP00BM@00TWD00@004hhCSP00099/`06001=eDgECMD004hG0P000e7e00<0058fDSH00P000UHf
00<005IGEUL00P000eIG0P0000=JN000Ool04Goo00<007ooOol05goo0004Ool500001Goo001oogoo
000027oo00@007ooOol000Aoo`03001oogoo009oo`03001oogoo00Ioo`03001oogoo031oo`<00004
@E4004Eb0003BM@01000CSP000000dVc00D004gECME=e@00009Am008001AmE7e001B=P00EQD0009F
=P04001FE`00FUH20002EUL01000FWQJN0004goo00<007ooOol05Woo000:Ool01000Oomoo`0027oo
00@007ooOol000Eoo`05001oogooOol00009Ool00`00Oomoo`0/Ool300001D4`0011DD5A00000TEb
00L004WDBMA9e000BI800003BK<01@00CME=eDgE00000U7d00D0057eDOEAm@00009F5@05001F=UHf
ESH00002FUH200000eIG00000003FWP00`00Oomoo`0AOol00`00Oomoo`0EOol000Yoo`04001oogoo
0008Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oogoo00Qoo`03001oogoo02Uoo`800005
@C00044`@C000002@E401000AG95LP000dWD00@004VBBI800099/`03001=eDgE0080009Am003001A
mE7e008000=F5@04001F=P000003FUH2000015Yg001JN5Yh0P004goo00<007ooOol05Goo000;Ool2
000:Ool20004Ool20003Ool2000;Ool00`00Oomoo`0UOol30000144`0011<0000T4`00@0045A@E40
0095LP030019e4WD00800099TP040019/dVcBK<200000e7C001Am002DO@200000eHD001F5@02EQD2
000015YF001JEUYF0P000eYg0P0000=NV000Ool04goo00<007ooOol057oo000ROol00`00Oomoo`0;
Ool00`00Oomoo`0QOol300001d4`0011<44`0011<0000091<0040011DD5A0003AG8200001DUa0019
TTVB00000dVc00@0057CDM<000=Am004001F55HD0003EQD01000FUIJEUYF0P0000=NMP00FWL00UYg
00D005jHGYQNV00001Aoo`03001oogoo01=oo`008Woo00<007ooOol037oo00<007ooOol07goo00<0
044`00000T4`00H0044`@C00044`0002@C001000@E51D@000dEb00@004UaBG400099TP050019/dVc
BK<00003DM<01000DOAAm0000eHD00@005HE000000=JEP03001NMUif0080009JM`03001NV5jH009N
V003001oogoo015oo`03001oogoo01=oo`008Woo00<007ooOol03Goo00<007ooOol07Woo00<0044`
00000T4`00H0044`@C00044`0002@C001@00@E51DD5A00000TEb00@004UaBG400099TP030019/dVc
008000=Ad`04001Am0000003EQ@20002GSD01000FUIJEP000Uif0P0000AJM`00GYL0009NV08001Ao
o`03001oogoo019oo`008Woo00<007ooOol03Woo00<007ooOol07Goo00<0044`00000T4`00H0044`
@C00044`0002@C001P00@E51DD5A0015LP800099L@040019TTVBBI8200000eGB001Ad`02DM<20002
FQ@01000EQAF50000ehe00T005YF001NMP00GWH005if00000UjG00<005jH00000V:h00<007ooOol0
4Goo00<007ooOol04Woo000ROol00`00Oomoo`0?Ool00`00Oomoo`0COol20007Ool00`00@C000002
@C001P00@C11<000@C000091<0060011DD5A@E4004Eb0P000TUa00@004VBBI99TP800003EM80057C
009Ad`05001J55XDFQ@00002EQ@02P00GSEN=Ehe001JEP00GWH005if0`000UjG0P000f:h00<007oo
Ool04Woo00<007ooOol04Goo000QOol00`00Oomoo`0AOol00`00Oomoo`0?Ool300000d4`001oo`06
Ool00`00@C000002@C001P00@C11<000@C000091<0040011DD5A@E4200000dfA0019L@02BG401000
BI99TP000UGB00D0057CDM=Ad`0000=J5005001F5000GS@00003GSD01P00GWINMP00GWH0009RU`80
0004H[L006:hH[P2000EOol00`00Oomoo`0@Ool0025oo`8001=oo`03001oogoo00eoo`030011<000
0091<003001oogoo00Aoo`800005@C00044`@C000002@C000`00@C000002@C001000@E51DD5A0P00
00==T@00BG400TUa00@004VBBI8000=EdP04001Ad`000004FQ@200000ehd001N=@02GSD01000GWIN
MUif0P000V:G00@006:gH[MR]`80009R^003001oogoo019oo`03001oogoo011oo`008Goo00<007oo
Ool04goo00<007ooOol037oo00<0044`00000T4`00<007ooOol017oo0P0000E1<000@C11<0000091
<0030011<0000091<0040011DD5A@E4200000dfA0019L@02BG401000BI8000000eGB00D0057C001N
4`0000=J5004001N=5hd0002GSD20002GWH30002HYL01@00H[MR]f:g00000f:h00<007ooOol04goo
00<007ooOol03goo000QOol00`00Oomoo`0DOol00`00Oomoo`0;Ool00`00@C000002@C000`00Oomo
o`04Ool200001D4`0011<44`00000T4`00<0044`00000T4`00D0045A@E51D@00009=T@060019LDUa
BG4004VB0P000eGB0P000UhC00D005XDFQAJ500000=N=080009RM@800007HYH00000HYL006Jg0000
0V:g00D006:hH[QR^00001Eoo`03001oogoo00moo`008Goo00<007ooOol05Goo00@007ooOomoo`80
00Moo`030011<0000091<003001oogoo00Aoo`800005@C00044`@C000002@C000`00@C000003@C00
1000@E51D@000TfA00@004UaBG59L@80009EdP03001EdUGB0080009N4`03001J55XD008000=N=005
001RMF9eHWD00003HYH20002I[L02@00H[L006KH001R^000I]P0000EOol00`00Oomoo`0>Ool0025o
o`03001oogoo01Ioo`<00003@C0007oo00Ioo`030011<0000091<003001oogoo009oo`@00005@C00
044`@C000002@C000`00@C000003@C001000@E51D@000TfA00@004UaBG59L@80009EdP04001EdUGB
0003GQ<01@00FQ@0069D00000ehd00D0069eHWERM@0000=RUP03001V]fJg009V]`05001Vf6KHI]P0
0003I]P00`00Oomoo`0COol00`00Oomoo`0=Ool0025oo`03001oogoo019oo`D00004@C00044`0006
Ool200001D4`0011<44`00000goo0P0000EfC0000011<0000091<0060011<44`0011<0000d4`00@0
045A0000009=T@050019LDUaBG400003EM801000EM9EdP0015hC0P000V9D00@005hd0000009RM@03
001VUP00009RUP04001V]fJgI[L20002I]P20002I]P2000EOol00`00Oomoo`0=Ool0021oo`03001o
ogoo011oo`<00004@C00044`0002@C000`00@C000006Ool200001D4`0011<44`00000Woo0`0000Ef
C0000011<0000091<0060011<44`0011<0000d4`00@0045A000000==T@040019LDUa0003EM801000
EM9EdP0015hC00@0069DHUARE080009VM@80009VUP03001RUP0000800004I[L006KHI]P20002I_P2
0002I_T00`00Oomoo`0COol00`00Oomoo`0<Ool0021oo`8000moo`800007@C00044`@C00044`0000
0T4`00<0044`00001Woo0P0000Q1<000@C11<000Ool007I]0P0000EfC0000011<0000091<0060011
<44`0011<0000d4`00@0045A000000==T@040019LDUa0003EM801@00EM8005hC00000ehC00D0069D
HUARE00000=VM@03001VUVJF009VUP05001V]`00I[L00003I]P01@00I_QVn6Kh00000VKi0P005Goo
00<007ooOol037oo000POol00`00Oomoo`0;Ool30000144a0011<0000T4`00<0044`00000T4`00<0
044`00001Woo0P0000A1<000@C11<0<00003MVd007I/00<00005@C00044`@C000002@C000`00@C00
0003@C001000@E4000000dfA00@004Ua000000=EdP05001EdP00GQ<00002GQ<20003HU@01@00IWEV
MFIe000016JF00D006Jg001V]`00009Vf080009Vn0800004I_T006LI000FOol00`00Oomoo`0;Ool0
021oo`03001oogoo00Uoo`800007AG80044a@C40044`00000T4`00<0044`00000T4`00@0044`0000
00Eoo`800003@C00044`00@00008MVd007I/001jc`00@C000091<0060011<44`0011<0000d4`0P00
00=AT@00CI400TfA00@004Ua000000AEdP800004GQ<005hCGQ<20002HU@01`00IYD006Ie001Z]@00
009VUP80009V]`04001Ze`000002J_P20002J_T01@00IaUW6FLI000057oo0`0037oo000POol00`00
Oomoo`06Ool3000014gf0015LP000T4a00<0044`00000T4`00<0044`00000T4`00D0044`001oo`00
00=oo`<00003@C00044`0080000:MVh007I]001fK000N/l0044`0002@C001P00@C11<000@C0000=1
<0800003DI4004fA009=T@050019L@00FO800003EM801P00GQ=N4`00GQ<0009VE00;001RE000IYD0
06Ie001Z]@00IYH00002J]H300000f[G001Zn002J_P01@00J_UZnF[i00000fLI0P0047oo0`000Woo
00<007ooOol02Woo000POol00`00Oomoo`04Ool200001e:J001=mTgf0015LP000091<@030011<000
0091<0030011<0000091<0<0009oo`03001oogoo00800091<0090011<000001fKP00MVd007I/0080
0091<0040011<44`0002@C000`00@C000003@C0200000e6A001=T@02CI420002FO801@00EM9EdUGB
00000UhC00<005hC00000VID0P000VJE00<006Ie00000VZe00T006[FJ]IZeP00J]L006[G00000f[h
0P0000IZn@00JaT006LI0002IaX00`00Oomoo`0:Ool30005Ool00`00Oomoo`0:Ool001moo`03001o
ogoo00=oo`800009E]`005:JDYX004gf0015LP000091<@050011<000@C000002@C001000@C00044`
0P000goo00@007oo00000091<0090011<000001fKP00MVd007I/00800091<0040011<44`0002@C00
0`00@C000003@C0200000e6A001=T@02CI420002FO801@00EM9EdUGB00000ehC0P000VID0P000fJE
00D006ZeJ[EZ]@0000=ZeP08001Ze`00001[6000J_P0009[6@03001[6F/I0080009W6P8000Uoo`<0
00Qoo`03001oogoo00Yoo`007goo00<007ooOol00`0000mFo@00E]aFg000DYX004gf0015LP00@C40
044`00000T4`00<0044`00000T4`00@0044`000000Aoo`<00091<0090011<000001fKP00MVd007I/
00800091<0040011<44`0002@C000`00@C000003@C0200000e6A001=T@02CI420002FO800`00EM9E
dP020003GQ<01@00IUAVE6ID000016JE00<006ZeJ[D00P000V[F00@006kg000000=[6003001[6F/I
00800007JaT006/j001W6P00IcX00P001Goo0`002goo00<007ooOol02Woo000OOol400000eKm0000
0006000024gf0015LP00@C40044`0002@C000`00@C000002@C000`00@C000005Ool30002@C001P00
@C000000MVh0009fK@<00091<0040011<44`0002@C000`00@C000003@C001000DI5AT@000dfA00D0
05WbFO9IlP00009EdP8000=N4`05001VE6IDIU@00003IYD2000016Ze001^eVkF0P000Vkg0P000V/H
0P0000E[6@00JcU[>@00009[>P800004IcX006Lk0002Ool3000?Ool00`00Oomoo`09Ool001Uoo`80
00Aoo`800007E_d00000DWT004Wf00000T6D00X003TB000004Eb0011<@00@C000091<0030011<000
0091<0030011<00000Eoo`<00004@C00044`@C02000017I^001fKGI]0`000T4`00@0044`@C000091
<0040011<44`0002@C001000DI5AT@000dfA00D005WbFO9IlP00009EdP80009N4`07001ZL`00IU@0
06jD00000VJE00@006kE0000009^eP04001^mfkgK_L200001F/H001_>Fli00000V/i0P0000E[>P00
JcY[>P00009W>`<0019oo`03001oogoo00Uoo`0067oo00@007ooOol000=oo`09001oo`00D[/0059i
0019mP000091U00:000i4P000015LP00@C40044`0002@C000`00@C000002@C000`00@C000005Ool3
0000144`0011<44`0P0000AfKP00MVefK@<00091<0040011<44`0002@C001000@C11<0000T4`00@0
056ADI4000==T@07001IlUWbFO8005GB00000V8b00/005hC001ZL`00IU@006jD001VU@00009^e@04
001^eVkFK]H200000fkg001_60020002KcT200001F/i001[>V/j00000V/j0P0000AW>`00Ie/001=o
o`03001oogoo00Uoo`0067oo00@007ooOol000=oo`03001oo`00009B^`04001BN@000002@I@02P00
>A800000AG80044a0011<0000T4`00<0044`00000T4`00<0044`000017oo100000A1<000@C11<080
0004MVh007I]MVd30002@C001000@C11<0000T4`00@0044`@C000091<005001ATE6ADI400002CI40
0`00FO9IlP02FO820002HS820002JW<01`00IU@006jD001VU@00009^e@04001^eVkF0002LaL00`00
KaQ_600200001Fli001_>Fli00000V/j0P0000E[>P00Je][F`0000=WF`03001oogoo011oo`03001o
ogoo00Uoo`0067oo00@007ooOol000=oo`03001oogoo00H00091U00:000i4P000015LP00@C40044`
0002@C000`00@C000002@C000`00@C000004Ool40000144`0011<44`0P000WI^00D007I]001jc`00
0091<0040011<44`0002@C001000@C11<0000T4`00H0056ADI5AT@00CI420004FO820002HS820003
JW<01@00KYA^U6jD00000VkE00D007;f001^eP00009c5`06001_6000001_>@000Vli0P0000A[>P00
JeY[FP800008Je/006MK001WF`00Ie/0015oo`03001oogoo00Uoo`0067oo00@007ooOol0009oo`0>
001oogooOol0047G000eM@00BOH0046D000i4P800006AG80044a0011<0000T4`00<0044`00000T4`
00<0044`000017oo100000A1<000@C11<080009fKP05001fK@00N/l00002@C001000@C11<0000T4`
00@0044`@C000091<006001ATE6ADI4004fA0P0015Wb00@0068bHS9R<P8000=ZL`07001^U6jDKY@0
06kE00000g;f00L007<GLaL007<h001_>@03000016li001_FVmJ0P0000E[FP00Je][F`00009WF`80
0004Ie/006ML000@Ool00`00Oomoo`09Ool001Uoo`8000=oo`03001oogoo009oo`090011e`000019
mP00@I@003TB00800006AG80044a0011<0000T4`00<0044`00000T4`00<0044`000017oo0P0000Aj
d000@C000091<080009fKP05001fK@00N/l00002@C001000@C11<0000T4`00D0044`@C00044`0080
00=AT@05001=T@00HQ400003FO801@00HS9R<V8b000016Yc00D006jDKY@007;e008000=bmP03001c
5`00009c>003001_>@00009cF@800005KeX006mkKg/00002Je/200000fMK001WG0020002Ie`2000?
Ool00`00Oomoo`09Ool001ioo`03001oogoo009oo`030011e`0000<00003@I@003TB00800006AG80
044a0011<0000T4`00<0044`00000T4`00<0044`000017oo0P0000Ajd000@C000091<080009fKP05
001fK@00N/l00002@C001000@C11<0000T4`00D0044`@C00044`008000=AT@05001=T@00HQ400003
FO801@00HS9R<V8b00000fYc0P000VjD00<007;e00000g;f00<007Lg00000W<h00@007=I0000009c
F@03001_NP00009_N`800003Je/006]k00800007Ie`006Ml001WG000Ie`00P003goo00<007ooOol0
27oo000NOol00`00Oomoo`02Ool02@00@ML000009=80046D000i4P030002@C400`00@C000002@C00
0`00@C000002@C000`00@C000004Ool2000017[@0011<0000T4`0P000WI^00D007I]001jc`000091
<0040011<44`0002@C001@00@C11<000@C000P000e6A00D004fA001R4@0000=IlP05001R<V8bHS80
0003JW<01000L]@000000W;e00@007;fL_H0009g=`03001c>000009cF@800003LeT006mj00800003
Kg/006]k00800003Jg/006Ml0080009WO0800004Ie`006=L000?Ool00`00Oomoo`08Ool001ioo`03
001oogoo00=oo`<000059=80046D000i4P030002@C400`00@C000002@C000`00@C000002@C000`00
@C000004Ool2000017[@0011<0000T4`0`0000IfKP00000007[?0002@C001000@C11<0000T4`00D0
044`@C00044`008000=AT@80009R4@03001IlUWb0080009R<P03001^T`00009ZL`0>001be000001b
m@00McH007;f001g=`00MgP0009cF@07001cN@00001_NP00Ki/00P0000=[N`00Ji`00P0000MWO000
Ig`006Ml001WO0020002He`2000>Ool00`00Oomoo`08Ool001ioo`03001oogoo00=oo`03000U5@00
00@00006>A8000000011<@000T4`00<0044`00000T4`00H0044`@C00044`0003Ool3000017[@0011
<0000T4`0`0000IfKP00000007[?0002@C001P00@C11<000@C0000=1<0040011<0000003DI420002
HQ400`00FO9IlP020002HS801@00KY<006Yc00000W;D00@007;eL_D0009g=P03001g=`00009gN080
0008LeT007=i001cVP00Ki/0009_V`03001[W000009[W003001WO6Ml00800003Ig`006=m00<00003
Hed0000000eoo`03001oogoo00Qoo`007Goo00<007ooOol017oo00<002DE0000100000Pi4P000000
044a0011<0000T4`00@0044`@C000091<0030011<00000=oo`07001ES`00N]00044`00000T4`0`00
00QfKP00000007[?0011<0000T4`00@0044`@C0000=1<0040011<0000003DI420002HQ400`00FO9I
lP020002HS801@00KY<006Yc00000W;D00@007;eL_D0009g=P05001g=`00MgP00002MiT00`00LgT0
000200001Vnk000006nK001[W0800003Ji`006NL0080009WO@08001SO@00Hgd006=M001SG@8000ao
o`03001oogoo00Qoo`007Goo00<007ooOol01Goo1`0000LX[`000011<@00@C000002@C001000@C11
<0000T4`0`000goo00L005F?001jd000@C000002@C0300000gI^00000002N/l00`00@C000002@C00
1000@C11<0000d4`00@0044`000000=AT@80009R4@05001IlUWb001ZDP020003KY<200000g;D001g
5@020002McH03000NeL00000MgP007NI001g^@00LiX0009_^`<00003Ji`006^L00800007Ii`006NM
001WO@00Hgd00P0000MSO@00001SG@00Ged0000<Ool00`00Oomoo`08Ool001eoo`03001oogoo00Eo
o`@0000432l00000::l20000144a0011<0000T4`00L0044`@C00044`0011<0020003Ool20002N]00
0`00@C000002@C0300000gI^00000002N/l00`00@C000002@C001000@C11<0000d4`0P0000=Z<@00
DI400U6A0P000V8A00<005Wb00000VYB0P000fjC00@007;DL]@0009g5@03001g=P00009kE`08001k
U`00MiT007Ni001c^P800004Kk/00000Jk`200000f^L001W_@020002Iid200000f=m001SO@020000
0f=m00000002Ged2000;Ool00`00Oomoo`08Ool001eoo`03001oogoo00Eoo`04000U5@009=830000
1bR_0000044a0011<0000091<0030011<0000091<0040011<0000003Ool20002N]000`00@C000002
@C001000MVh007I^0P000W[?00<0044`00000T4`00@0044`@C0000=1<0800003JS40056A009AT@07
001R4F8AHQ4005Wb00000VYB0P000fjC00<007;D00000gLE00D007Lf001kE`00009kU`800005MkT0
07>j001_f`0300000f^l001[_@0200000fNm001W_@0300001V>M000006=m001OO@800005Ggd00000
001OG@02000;Ool00`00Oomoo`07Ool001eoo`03001oogoo00Eoo`03000U5@0000@00007::l00000
@C40044`00000T4`00<0044`00000T4`00@0044`0000009oo`<0009jd0040011<000@C0200000gI^
001fKP020002N/l00`00@C000002@C001000@C11<0000d4`0P0000=Z<@00DI400U6A00<0068AHQ40
0V8A0P000VYB00@006jCKY=^T`800006L]@007LEMaD007]E0P0000=kE`00NiL00P0000Mgf@000000
06oK001_g00300000f^m001W_@0200000fNm001SWP0300001V>M000005mm001OOP80009OO@800003
Ged0000000Yoo`03001oogoo00Moo`0077oo0P001goo1`0000LX[`000011<@00@C000002@C000`00
@C000002@C001000@C0000000Woo0`000W[@00@0044`0011<0800003MVh000000080009jc`030011
<0000091<0040011<44`0003@C0200000fXa001AT@02DI400`00HQ5R4@02HQ420002JU801P00KY=^
TfjC001fl`80009g5@06001kE@00NiH007^G0P000WOI100000=_g00000000VOM00@006Nm001S_P80
0004Hih005nNGih200002UnN000005mn001OOP00Ggd005mn0P0000=KGP0000002Goo00<007ooOol0
1goo000MOol00`00Oomoo`06Ool600001bR_0000044a0011<0000091<0030011<0000091<0040011
<0000002Ool300001W[@0011<44`0011<0800003MVh000000080009jc`030011<0000091<0040011
<44`0003@C0200000fXa001AT@02DI400`00HQ5R4@02HQ401@00JU9ZDVYB00000fjC00<007Kc0000
0WLE00<007]E00000W^F100000Egf@00000006_l00@00003Imd0000000<00005Gkh00000001OWP04
00000enN00000002Ggh200001emn000005]N001KGP0000Uoo`03001oogoo00Moo`0077oo00<007oo
Ool01goo1P0000LX[`000011<@00@C000002@C000`00@C000002@C001000@C0000000Woo0`0000Ij
d000@C11<000@C0200001GI^000006h^00000W[?00<0044`00000T4`00@0044`@C0000=1<0800003
JS40056A009AT@03001R4F8A009R4@04001ZDVYBJU82000016jC001flgKc0P0000=g5@0000000W^F
00D007^g000007?i00<00005Jo`006Ol001So@0300000eoN001O_P020002Gkh300001E^N00000000
Fgh00`0000AKOP00001KOP80009KGP8000Qoo`03001oogoo00Moo`0077oo00<007ooOol01goo1P00
00LX[`000011<@00@C000002@C000`00@C000002@C000`00@C000003Ool01@00EHl007[@00000T4`
00D0044`000007I^00800007KRh007[?N/l0044`00000T4`00@0044`@C0000=1<080009Z<@04001A
TE6A0004HQ402000JU9ZDP00M]8006jC0002M_<02000Oe@007]E001kUP00NkL200001G?i00000000
Ho/01P0000=KgP0000000P0000=K_P0000000P0000AKW`00001KWP<00005Fgh00000001KOP020000
1E]N00000000Feh00P001goo00<007ooOol01goo000LOol00`00Oomoo`07Ool600001bR_0000044a
0011<0000091<0030011<0000091<0030011<000009oo`800004EHl007[@0002@C001P00@C000000
MVh0009^;P06001jcg[?0011<0000T4`00@0044`@C0000=1<006001Z<FXaJS40056A0P00168A00X0
06YBJU8007KB001^T`00M_<0009oE0<00004NiH00000MmP400001F?k00000000Emd01@0000EC_`00
000005No00@00003Eil00000009GO`800005Fgh00000001KOP0200001E]N00000000Feh00P001Woo
00<007ooOol01goo000LOol00`00Oomoo`08Ool600000`P<00000002@C001P00@C11<000@C000091
<0030011<000009oo`<0009jd0050011<44`0011<002000017I^001^;P000g[?00<0044`00000T4`
00@0044`@C0000=1<009001Z<FXaJS40056A001f/@00009R4@80009ZDP04001fdWKB0002M_<01P00
Oe@007nD001k]@h00003Ckh0000000800003Dkl0000000800005Dil00000001GW`03000015Mo0000
05Mn0`0000AGOP00001GGP<00003Eeh0000000Ioo`03001oogoo00Ioo`0077oo00<007ooOol02Goo
1@0000<8300000000T4`00<0044`00000T4`00H0044`@C00044`0002Ool30002N]001@00@C11<000
@C000P0000AfKP00KRh000=jc`030011<0000091<0040011<44`0003@C002@00JS5Z<FXa001AT@00
M[400002HQ4200000fYB001fdP02M]801@00M_=fl`00Oe@00P000W^e100000=Kn@0000003P000TnO
0P0000ECW`00000005=o00<00004Egl00000Egl3000015MN000005MN0`0000=GGP0000001Goo00<0
07ooOol01Woo000LOol00`00Oomoo`09Ool500001@P<0000044`00000T4`00@0044`@C000091<004
0011<000Ool40002N]001@00@C11<000@C000P0000AfKP00KRh000=jc`030011<0000091<0040011
<44`0003@C002@00JS5Z<FXa001AT@00M[400002HQ401000LY8000000gKB0`0000EoE000Oi@007^e
02<00004Dgl00000Eel3000015MN000005MN0`0000=GGP00000017oo00<007ooOol01Woo000KOol2
000<Ool400001@P<0000044`00000T4`00@0044`@C000091<0030011<00000D0009jd0050011<44`
0011<00400000fh^001jc`02N/l00`00@C000002@C001000@C11<0000d4`00T006XaJS5Z<@00DI40
07Ja00000V8A00@007:B0000009fdP07001o<glc001oE000Oi@01`000goo4P0000A7O`00001;O`T0
0005Del00000001GGP0200001EMN00000000Eeh00P000goo00<007ooOol01Woo000KOol00`00Oomo
o`0;Ool400001@P<0000044`00000T4`00@0044`@C000091<0030011<00000D0009jd0050011<44`
0011<00400000fh^001jc`02N/l00`00@C000002@C001000@C11<0000d4`00T006XaJS5Z<@00DI40
07Ja00000V8A00@007:B0000009fdP03001o<glc00P000IooaH00005Bel00000001?G`0500001E=O
00000000Deh00P0000ICGP00000005Ln0003Ool00`00Oomoo`06Ool001]oo`03001oogoo00aoo`<0
000520`00000@C000002@C001000@C11<0000T4`00<0044`00000`0000EES`00N]1jd0000091<003
0011<00000<00003KRh007[?009jc`030011<0000091<0040011<44`0002@C020003JS401`00DI40
07Ja001R4@00009bTP0:001fdWKBM]8007lc001oL`00Nk830005Ool30003OolH00000dmO00000007
000015=N000005=N1@000Woo00<007ooOol01Woo000KOol00`00Oomoo`0<Ool300001@P<0000044`
00000T4`00@0044`@C000091<0H00005EHl007[@N]000002@C000`00@C00000300000fh^001jc`02
N/l00`00@C000002@C001000@C11<0000T4`0P000fXa0P000WJa00<0068A00000W:B00@007KBM]80
009o<`80009k/P<0009oo`<000Mooa800003@cl0000000<00005Bel00000001?G`05000015=O0000
05<n1@0000=oo`00Ool01goo000KOol00`00Oomoo`0=Ool200001@P<0000044`00000T4`00L0044`
@C00044`0011<0060003N]001@00@C11<000@C00100000E^;P00N/mjc`000091<0040011<44`0002
@C000`00@C11<0020003JS420002M[400`00HQ400002LY801000M]9fdP000Wlc0P0000=k/P000000
100037oo5`0000E;G`00000004mO00D00004Dcl00000Dch60008Ool001]oo`03001oogoo00eoo`D0
0007@C00044`@C00044`00000T4`00<0044`00000P0000<00`0000000g[@00D0044`@C00044`0080
0004HLl006h^0002N/l01000@C11<0000T4`00@0044`@C000091<08000=Z<@80009f/@03001R4@00
009bTP800003M]8007lc00800003Nk80000000<000moob@00003Dch005<n00800003Ool007oo00Io
o`006goo00<007ooOol03Goo1@0000M1<000@C11<000@C000002@C000`00@C0000050003N]001@00
@C11<000@C000P0000=Qc`0000000g[?00@0044`@C000091<0080011<44`0011<000N_4000=Z<@80
00=f/@800003LY8007lB0080009o<`D00003N]000000019ooa800003Acl0000000<00003Bcl00000
00<00003Ccl0000000<00004Dch000000007Ool001Yoo`03001oogoo00ioo`D00007@C00044`@C00
044`00000T4`00<0044`00001@000g[@00D0044`@C00044`00800003HLl0000000=jc`040011<44`
0002@C002000@C11<000@C0007[a0003JS420003M[401000LY9bTP000WlB200000=jd000000057oo
5`0000=;?`0000000`0000=??`0000000`0000=C?P0000001goo000JOol2000@Ool30002@C001P00
@C11<000@C000091<0030011<00000D000=jd0030011<0000091<0800003HLl0000000=jc`040011
<44`0002@C001P00@C11<000@C00009jl@03001Z<FXa008000=f/@03001bTW:B00@00004Og400000
Jhh300000g[@0000000GOolE00000d/o0000000300000dlo0000000300000goo001oo`06Ool0009o
o`8000Aoo`8000Aoo`8000=oo`@000=oo`03001oogoo00moo`<00091<0060011<44`0011<0000T4`
00<0044`00001@000g[@00<0044`00000T4`0P0000=Qc`0000000g[?00@0044`@C000091<0060011
<44`0011<0000W[a00<006XaJS400P000gJa10000WlB00<007ma00001@0000=jd000000067oo5@00
00=;?`0000000`001Woo00<007ooOol01Goo00001Goo001oogoo00000goo0P000goo00@007ooOol0
00=oo`03001oogoo00=oo`03001oogoo00moo`<00091<0060011<44`0011<0000T4`00D0044`0000
000300<000=jd0030011<0000091<080009Qc`05001jcg[?N/l00002@C001000@C11<0000T4`00<0
044`00000W[a00D006XaJS4007Ja00@00006LY8007lBOa8007ma0`0000Ejd000000007[@008001Uo
o`03001oogoo009ooa0000]oo`03001oogoo00Eoo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0004
Ool00`00Oomoo`02Ool00`00Oomoo`0?Ool30002@C000`00@C000002@C001@00@C11<000@C001P00
0g[@00<0044`00000T4`0P000V7?00D007[?N/mjc`000091<0040011<44`0002@C020003N_450002
M[4200001W:B001o4WlB001oL@<00005N]000000001jd002000IOol00`00Oomoo`0MOol00`00Oomo
o`05Ool00005Ool007ooOol00008Ool01000Oomoo`001Goo00D007ooOomoo`00015oo`<00005@C00
044`@C000002@C001@00@C11<000@C001@0017[@00L0044`0011<00000H00002HLl01@00N/mjcg[?
00000T4`00@0044`@C000091<0800004N_4000000002JS401000M[5f/GJa0P0000IbTP00Oa9o4P00
Og4300000g[@0000000200000g[@001oo`0IOol00`00Oomoo`0MOol00`00Oomoo`04Ool00005Ool0
07ooOol00008Ool01000Oomoo`000Woo00@007ooOol0009oo`03001oogoo011oo`<00005@C00044`
@C000002@C001@00@C11<000@C000`0000<h[000N]000g[@00L0044`0011<00000H00002HLl01@00
N/mjcg[?00000T4`00@0044`@C000091<0<0009jl@04001Z<FXa0003M[401`00Nc@00000Oa8007ma
00X001aoo`03001oogoo01aoo`03001oogoo00Aoo`000Woo0P002Woo0P0017oo0P000goo00<007oo
Ool047oo0`0000E1<000@C11<0000091<0050011<44`0011<00300000cR/001jd002N]002000@C11
<000@C0000060002HLl01@00N/mjcg[?00000T4`00<0044`@C001@000g[a00L006Xa001cF000M[40
0002Nc@200000glB001oL@020002Fa`6000MOol00`00Oomoo`0KOol00`00Oomoo`04Ool001Uoo`03
001oogoo011oo`<00005@C00044`@C000002@C001@00@C11<000@C000`0000<h[000N]000W[@00X0
044`@C00044`00001P00HLl000Ajc`030011<44`00@00091<003001jlG[a009jl@80009cF003001f
/@00009k=005001o4WlB001oL@0400000e/L00000003000NOol00`00Oomoo`0JOol00`00Oomoo`04
Ool001Uoo`03001oogoo011oo`<00005@C00044`@C000002@C001@00@C11<000@C000`0000<h[000
N]000W[@00X0044`@C00044`00001P00HLl000Ajc`@00091<0030011<0000080009jl@8000=cF003
001f/@00009k=003001o4WlB00<00005Fa`00000001K7004000NOol00`00Oomoo`0JOol00`00Oomo
o`04Ool001Qoo`80019oo`<00005@C00044`@C000002@C001@00@C11<000@C000P000SR/00D007[@
N]1jd0000091<0060011<00000H0067?1P0000=1<000000010000Tln0`000Tln00<007=HLeP00P00
0W/d00<006MK00001@000U/L1P007goo00<007ooOol06Goo00<007ooOol017oo000IOol00`00Oomo
o`0@Ool300001D4`0011<44`00000T4`00<0044`@C0010000SR/00D007[@N]1jd0000091<0040011
<00000H40002N/l30002Cch00`00Cci??P030002Cch00`00Cci??P03000014ln001k=0000VMK0P00
00=WF`00Fa`00`0000=K7000Fa`00`0087oo00<007ooOol067oo00<007ooOol017oo000IOol00`00
Oomoo`0@Ool20002@C001000@C11<0000T4`00@0044`0011<0<00003>:`007[@00=jd0030011<44`
00D0009??P<0009??P03001??Tln00<0009??P03001??Tln00<00004Cch004lnCch3000016MK001W
FfMK0`000U/L00D005/L000005/L0080025oo`03001oogoo01Qoo`03001oogoo00=oo`0067oo00<0
07ooOol04Goo0P000T4`00H0044`@C00044`0002@C000`00@C00000200000cR/001jd003N]040000
0d4`00001P030002Cch00`00Cci??P030002Cch00`00Cci??P03000014ln001??Tln0`000Tln0P00
0VMK0P0000=K700000000`0000=K7000000097oo00<007ooOol05goo00<007ooOol00goo000HOol0
0`00Oomoo`0AOol20002@C001P00@C11<000@C000091<0030011<00000800004>:`007[@N]030002
@C040002Cch00`00Cci??P030002Cch00`00Cci??P0300000dln000000040002Cch30002Cch00`00
Ie/000020002Fa`01000Fa`000009Woo00<007ooOol05goo00<007ooOol00goo000HOol00`00Oomo
o`0AOol20002@C001P00@C11<000@C000091<0050011<000000h[0040002N]001000@C0000000Tln
00<004lnCch00`000Tln1P0000A??P000000009??P03001??Tln00<0009??P03001??Tln00<00004
Fa`005/LFa`2000YOol00`00Oomoo`0FOol00`00Oomoo`03Ool001Qoo`03001oogoo015oo`800091
<0030011<0000091<0050011<44`0011<004000017oo001jd7[@0`0000A??P00Cci??P<0009??P<0
009??P03001??Tln0080009??P03001??Tln00<0009??P03001??Tln00<00003Cch0000000@002ao
o`03001oogoo01Eoo`03001oogoo00=oo`0067oo00<007ooOol04Goo0P0000E1<000@C11<0000091
<0030011<00000<000Eoo`8000=oo`@0009??P<0009??P03001??Tln00<00004Cch004lnCch30002
Cch00`00Cci??P0300000dln001??P02000bOol00`00Oomoo`0DOol00`00Oomoo`03Ool001Moo`80
01=oo`800006@C00044`@C00044`0`0000A1<000000000Yoo`03001??Tln00<0009??P03001??Tln
00<00004Cch004lnCch30002Cch00`00Cci??P0300000dln00000003000eOol00`00Oomoo`0COol0
0`00Oomoo`03Ool001Moo`03001oogoo019oo`@00007@C0000000011<000@C000P0000=??P00Ool0
2Woo0P000Woo00<004lnCch00`000Tln00<004ln00000P000Tln1P000Tln1@00>7oo00<007ooOol0
4goo00<007ooOol00Woo000GOol00`00Oomoo`0BOol00`0000@0000200000d4`0011<00300000dln
0000000AOol20003Ool40002Cch30002Cch00`00Cci??P0300000e/L0000000jOol00`00Oomoo`0C
Ool00`00Oomoo`02Ool001Moo`03001oogoo015oo`D00004@C0000000002Cch01000Cch000006Goo
1P000Tln100000=K70000000?Goo00<007ooOol04Woo00<007ooOol00Woo000GOol00`00Oomoo`0=
Ool40002Ool40002Cch5000MOol00`00Fcd0000500000e/L00000010Ool00`00Oomoo`0AOol00`00
Oomoo`02Ool001Moo`03001oogoo00Yoo`<000Moo`D0009??P8001moo`04001K?@000002Fcd20013
Ool00`00Oomoo`0@Ool00`00Oomoo`02Ool001Moo`03001oogoo00Moo`<000]oo`800004Bgl004ln
000ROol00`00Fcd000020016Ool00`00Oomoo`0?Ool00`00Oomoo`02Ool001Moo`03001oogoo00Ao
o`<0011oo`<002Aoo`8004Qoo`03001oogoo00moo`03001oogoo009oo`005Woo00<007ooOol00Woo
0`00>goo00<007ooOol0Agoo00<007ooOol03Woo00<007ooOol00Woo000FOol00`00Ool00002000n
Ool00`00Oomoo`18Ool00`00Oomoo`0>Ool00`00Oomoo`01Ool001Ioo`80045oo`03001oogoo04Uo
o`03001oogoo00eoo`03001oogoo005oo`0067oo00<007ooOol0?Woo00<007ooOol0BWoo00<007oo
Ool037oo00<007ooOol00Goo000IOol2000nOol00`00Oomoo`1;Ool00`00Oomoo`0;Ool00`00Oomo
o`01Ool001]oo`03001oogoo03]oo`03001oogoo04]oo`03001oogoo00]oo`03001oogoo005oo`00
77oo0`00>Woo00<007ooOol0C7oo00<007ooOol02Woo00<007ooOol00Goo000MOol2000iOol00`00
Oomoo`1>Ool00`00Oomoo`09Ool00`00Oomoo`01Ool001moo`03001oogoo03Ioo`03001oogoo04mo
o`03001oogoo00Uoo`40005oo`5oo`0087oo0P00=Woo00<007ooOol0D7oo00<007ooOol027oo0@00
0Goo0Goo000QOol2000eOol00`00Oomoo`1@Ool00`00Oomoo`08Ool10001Ool1Ool002=oo`<0039o
o`03001oogoo055oo`03001oogoo00Moo`40005oo`5oo`009Goo00<007ooOol0<7oo00<007ooOol0
DWoo00<007ooOol01Woo0@000Goo0Goo000VOol2000`Ool00`00Oomoo`1COol00`00Oomoo`05Ool1
0001Ool1Ool002Qoo`8002ioo`03001oogoo05Aoo`03001oogoo00Aoo`40005oo`5oo`00:Goo0P00
;Goo00<007ooOol0E7oo00<007ooOol01Goo0@000Goo000[Ool00`00Oomoo`0ZOol00`00Oomoo`1E
Ool00`00Oomoo`04Ool10001Ool002aoo`03001oogoo02Uoo`03001oogoo05Ioo`03001oogoo00=o
o`40005oo`0077oo0P0017oo0P000goo10000Woo0P00:Goo00<007ooOol0Egoo00<007ooOol00Woo
0@000Goo000KOol01000Oomoo`000goo0P0017oo00<007ooOol017oo0P009goo00<007ooOol0F7oo
00D007ooOomoo`00005oo`006goo00@007ooOol000Yoo`03001oogoo00Aoo`8002Ioo`03001oogoo
05Uoo`04001oogoo0001Ool001]oo`04001oogoo000;Ool00`00Oomoo`05Ool2000TOol00`00Oomo
o`1IOol01000Oomoo`000Goo000KOol01000Oomoo`0027oo00@007ooOol000Qoo`8002=oo`03001o
ogoo05Yoo`03001oogoo0040000077oo0P002Woo0P002goo00<007ooOol087oo00<007ooOol0Fgoo
0@000Goo0@00000fOol2000POol00`00Oomoo`1LOol2000003Ioo`03001oo`0001moo`03001oogoo
05]oo`<00000>Goo0P007Goo00<007ooOol0EWoo1@000goo000jOol2000LOol00`00Oomoo`1BOol4
0008Ool002]oo`8000Aoo`8000Aoo`<0009oo`8001Yoo`03001oogoo04ioo`@000aoo`00:Woo00@0
07ooOol000=oo`8000Eoo`03001oogoo009oo`8001Uoo`03001oogoo04Uoo`D0011oo`00:Woo00@0
07ooOol000Qoo`@000Eoo`8001Moo`03001oogoo04Eoo`@001Eoo`00:Woo00@007ooOol000Qoo`03
001oo`0000Moo`8001Ioo`03001oogoo041oo`D001Uoo`00:Woo00@007ooOol000Qoo`03001oo`00
00Uoo`03001oogoo01=oo`03001oogoo03aoo`@001ioo`00:goo0P002Woo0P002Woo0P004goo00<0
07ooOol0>7oo10008Woo0015Ool00`00Oomoo`0?Ool00`00Oomoo`0dOol5000VOol004Ioo`03001o
ogoo00ioo`03001oogoo031oo`@002]oo`00AWoo0`003Woo00<007ooOol0:goo1@00;goo000hOol2
0004Ool20004Ool20003Ool00`00Oomoo`0;Ool00`00Oomoo`0WOol4000dOol003Moo`04001oogoo
0003Ool20003Ool01000Oomoo`000Woo10002Woo00<007ooOol08Woo1@00>7oo000gOol01000Oomo
o`0027oo00@007ooOol000Eoo`03001oogoo00Qoo`03001oogoo01ioo`@003eoo`00=goo00@007oo
Ool000Qoo`<000Ioo`<000Qoo`03001oogoo01Yoo`@0045oo`00=goo00@007ooOol000Qoo`03001o
ogoo00Uoo`03001oogoo00Eoo`03001oogoo01Eoo`D004Eoo`00>7oo0P002Woo0`0027oo0`001Goo
00<007ooOol04Goo1000BWoo001AOol20004Ool00`00Oomoo`0<Ool5001>Ool0055oo`03001oo`00
00=oo`03001oogoo00Qoo`@005=oo`00A7oo0P0017oo0P0017oo0P000Woo0P0000=oo`00Ool01Goo
1000Egoo0013Ool01000Oomoo`000goo0P000goo00@007ooOol000=oo`800003Ool0000000<005]o
o`00@goo00@007ooOol000Qoo`04001oogoo0004Ool2001POol004=oo`04001oogoo0009Ool2001W
Ool004=oo`04001oogoo0008Ool01000Oomoo`00IWoo0014Ool2000:Ool2001WOol00;Uoo`00^Goo
001DOol3001ROol005Eoo`03001oogoo065oo`00EGoo00<007ooOol0HGoo001EOol00`00Oomoo`1Q
Ool005Eoo`03001oogoo065oo`00E7oo0P00Hgoo002iOol00001\
\>"],
  ImageRangeCache->{{{0, 184.688}, {200.5, 0}} -> {-0.0676582, -0.0622653, 
  0.00527313, 0.00527313}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["8.5 Up/Down and In (Knock-In) Puts", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We can do numerical and graphical checks against the original \
Rubenstein and Reiner paper. First we introduce the value and price \
sensitivity functions:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Down and In Puts", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
DownAndInPut[reb_, p_, k_, h_, sd_, r_, q_, t_] :=
If[k >= h, inttwo[p, k, h, sd, r, q, t, -1]-intthree[p, k, h, sd, r, q, t, \
-1, 1]+ intfour[p, k, h, sd, r, q, t, -1, 1]+ intfive[reb, p, h, sd, r, q, t, \
1],
intone[p, k, sd, r, q, t, -1]  + intfive[reb, p, h, sd, r, q, t, 1]];\
\>", 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  FontSize->9],

Cell["\<\
DownAndInPutDelta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[inttwo[p, k, h, sd, r, q, t, -1]-intthree[p, k, h, sd, \
r, q, t, -1, 1]+ intfour[p, k, h, sd, r, q, t, -1, 1]+intfive[reb, p, h, sd, \
r, q, t, 1], p]],
Evaluate[D[intone[p, k, sd, r, q, t, -1]  + intfive[reb, p, h, sd, r, q, t, \
1], p]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  FontSize->9],

Cell["\<\
DownAndInPutGamma[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[inttwo[p, k, h, sd, r, q, t, -1]-intthree[p, k, h, sd, \
r, q, t, -1, 1]+ intfour[p, k, h, sd, r, q, t, -1, 1]+intfive[reb, p, h, sd, \
r, q, t, 1], {p,2}]],
Evaluate[D[intone[p, k, sd, r, q, t, -1]  + intfive[reb, p, h, sd, r, q, t, \
1], {p,2}]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  FontSize->9],

Cell["\<\
DownAndInPutTheta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[-D[inttwo[p, k, h, sd, r, q, t, -1]-intthree[p, k, h, sd, \
r, q, t, -1, 1]+ 
intfour[p, k, h, sd, r, q, t, -1, 1]+intfive[reb, p, h, sd, r, q, t, 1], t]],
Evaluate[-D[intone[p, k, sd, r, q, t, -1]  + intfive[reb, p, h, sd, r, q, t, \
1], t]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  FontSize->9],

Cell["\<\
DownAndInPutVega[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[inttwo[p, k, h, sd, r, q, t, -1]-intthree[p, k, h, sd, \
r, q, t, -1, 1]+ 
intfour[p, k, h, sd, r, q, t, -1, 1]+intfive[reb, p, h, sd, r, q, t, 1], \
sd]],
Evaluate[D[intone[p, k, sd, r, q, t, -1]  + intfive[reb, p, h, sd, r, q, t, \
1], sd]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  FontSize->9],

Cell["\<\
DownAndInPutRho[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[inttwo[p, k, h, sd, r, q, t, -1]-intthree[p, k, h, sd, \
r, q, t, -1, 1]+ 
intfour[p, k, h, sd, r, q, t, -1, 1]+intfive[reb, p, h, sd, r, q, t, 1], r]],
Evaluate[D[intone[p, k, sd, r, q, t, -1]  + intfive[reb, p, h, sd, r, q, t, \
1], r]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  FontSize->9]
}, Open  ]],

Cell[CellGroupData[{

Cell["Up and In Puts", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True],

Cell["\<\
UpAndInPut[reb_, p_, k_, h_, sd_, r_, q_, t_] :=
If[k >= h, intone[p, k, sd, r, q, t, -1] - inttwo[p, k, h, sd, r, q, t, -1] + \
intfour[p, k, h, sd, r, q, t, -1, -1]+intfive[reb, p, h, sd, r, q, t, -1],
intthree[p, k, h, sd, r, q, t, -1, -1]+intfive[reb, p, h, sd, r, q, t, \
-1]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndInPutDelta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, -1] - inttwo[p, k, h, sd, r, \
q, t, -1] + intfour[p, k, h, sd, r, q, t, -1, -1]+intfive[reb, p, h, sd, r, \
q, t, -1], p]],
Evaluate[D[intthree[p, k, h, sd, r, q, t, -1, -1]+intfive[reb, p, h, sd, r, \
q, t, -1], p]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndInPutGamma[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, -1] - inttwo[p, k, h, sd, r, \
q, t, -1] + intfour[p, k, h, sd, r, q, t, -1, -1]+intfive[reb, p, h, sd, r, \
q, t, -1], {p,2}]],
Evaluate[D[intthree[p, k, h, sd, r, q, t, -1, -1]+intfive[reb, p, h, sd, r, \
q, t, -1], {p,2}]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndInPutTheta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[-D[intone[p, k, sd, r, q, t, -1] - inttwo[p, k, h, sd, r, \
q, t, -1] + intfour[p, k, h, sd, r, q, t, -1, -1]+intfive[reb, p, h, sd, r, \
q, t, -1], t]],
Evaluate[-D[intthree[p, k, h, sd, r, q, t, -1, -1]+intfive[reb, p, h, sd, r, \
q, t, -1], t]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndInPutVega[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, -1] - inttwo[p, k, h, sd, r, \
q, t, -1] + intfour[p, k, h, sd, r, q, t, -1, -1]+intfive[reb, p, h, sd, r, \
q, t, -1], sd]],
Evaluate[D[intthree[p, k, h, sd, r, q, t, -1, -1]+intfive[reb, p, h, sd, r, \
q, t, -1], sd]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndInPutRho[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, -1] - inttwo[p, k, h, sd, r, \
q, t, -1] + intfour[p, k, h, sd, r, q, t, -1, -1]+intfive[reb, p, h, sd, r, \
q, t, -1], r]],
Evaluate[D[intthree[p, k, h, sd, r, q, t, -1, -1]+intfive[reb, p, h, sd, r, \
q, t, -1], r]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
The following tables of values can be checked directly against a \
further table of Rubenstein and Reiner. Note the conversion to annual rates \
for the interest rates and yield.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
TableForm[Table[N[DownAndInPut[2,100,k,97,0.20,Log[1.1],Log[1.05],t]\
], {k, 90, 110, 10}, {t, 0.5, 1.5, 0.25}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"1.63321592925009912`", "2.12861972858359038`", 
            "2.52532233862337873`", "2.84263637379557554`", 
            "3.09738882848003438`"},
          {"4.74369403971030401`", "5.30961327362496504`", 
            "5.70062898319346`", "5.97865464488403919`", 
            "6.1771588933633259`"},
          {"10.1770298703536554`", "10.4257376928546463`", 
            "10.5477123739133316`", "10.5925483371082673`", 
            "10.5840800980581084`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
TableForm[Table[N[DownAndInPut[2,100,k,100,0.20,Log[1.1],Log[1.05],\
t]], {k, 90, 110, 10}, {t, 0.5, 1.5, 0.25}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"1.27534701894416535`", "1.83532432368172138`", 
            "2.27172916084475318`", "2.61688722532009165`", 
            "2.89271640908605576`"},
          {"4.38948764445481032`", "5.01827978906336369`", 
            "5.4482836075955987`", "5.75377803479555893`", 
            "5.97313448378118749`"},
          {"10.0757074942589008`", "10.2747657584368923`", 
            "10.3862794967195171`", "10.431951456544255`", 
            "10.4281513901130939`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
TableForm[Table[N[UpAndInPut[2,100,k,103,0.20,Log[1.1],Log[1.05],t]]\
, {k, 90, 110, 10}, {t, 0.5, 1.5, 0.25}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"0.879386506227545616`", "1.25265178612509009`", 
            "1.59628302040083251`", "1.89473184373291818`", 
            "2.14931082053955879`"},
          {"2.81756891405395393`", "3.46594283014130821`", 
            "3.94601499163619351`", "4.31046434387857857`", 
            "4.59059099118718538`"},
          {"7.08063881110663828`", "7.61511526974798158`", 
            "7.96787971861305521`", "8.2042668220047723`", 
            "8.35913873358017234`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["8.6 Up/Down and Out (Knock-Out) Puts", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
We confine ourselves to numerical checks against the original \
Rubenstein and Reiner paper. First we introduce the value and price \
sensitivity functions:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Down and Out Puts", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
DownAndOutPut[reb_, p_, k_, h_, sd_, r_, q_, t_] :=
If[k >= h, intone[p, k, sd, r, q, t, -1]-inttwo[p, k, h, sd, r, q, t, \
-1]+intthree[p, k, h, sd, r, q, t, -1, 1]-intfour[p, k, h, sd, r, q, t, -1, \
1]+intsix[reb, p, h, sd, r, q, t, 1],
intsix[reb, p, h, sd, r, q, t, 1]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  FontSize->9],

Cell["\<\
DownAndOutPutDelta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, -1]-inttwo[p, k, h, sd, r, q, \
t, -1]+
intthree[p, k, h, sd, r, q, t, -1, 1]-intfour[p, k, h, sd, r, q, t, -1, 1]+ \
intsix[reb, p, h, sd, r, q, t, 1],p]],
Evaluate[D[intsix[reb, p, h, sd, r, q, t, 1], p]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  FontSize->9],

Cell["\<\
DownAndOutPutGamma[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, -1]-inttwo[p, k, h, sd, r, q, \
t, -1]+
intthree[p, k, h, sd, r, q, t, -1, 1]-intfour[p, k, h, sd, r, q, t, -1, 1]+ \
intsix[reb, p, h, sd, r, q, t, 1],{p,2}]],
Evaluate[D[intsix[reb, p, h, sd, r, q, t, 1], {p,2}]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndOutPutTheta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[-D[intone[p, k, sd, r, q, t, -1]-inttwo[p, k, h, sd, r, \
q, t, -1]+intthree[p, k, h, sd, r, q, t, -1, 1]-intfour[p, k, h, sd, r, q, t, \
-1, 1]+
intsix[reb, p, h, sd, r, q, t, 1],t]],
Evaluate[-D[intsix[reb, p, h, sd, r, q, t, 1], t]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndOutPutVega[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, -1]-inttwo[p, k, h, sd, r, q, \
t, -1]+intthree[p, k, h, sd, r, q, t, -1, 1]-intfour[p, k, h, sd, r, q, t, \
-1, 1]+intsix[reb, p, h, sd, r, q, t, 1],sd]],
Evaluate[D[intsix[reb, p, h, sd, r, q, t, 1], sd]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
DownAndOutPutRho[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[intone[p, k, sd, r, q, t, -1]-inttwo[p, k, h, sd, r, q, \
t, -1]+ intthree[p, k, h, sd, r, q, t, -1, 1]-intfour[p, k, h, sd, r, q, t, \
-1, 1]+intsix[reb, p, h, sd, r, q, t, 1],r]],
Evaluate[D[intsix[reb, p, h, sd, r, q, t, 1], r]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9]
}, Open  ]],

Cell[CellGroupData[{

Cell["Up and Out Puts", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
UpAndOutPut[reb_, p_, k_, h_, sd_, r_, q_, t_] :=
If[k >= h, inttwo[p, k, h, sd, r, q, t, -1] -
intfour[p, k, h, sd, r, q, t, -1, -1] + intsix[reb, p, h, sd, r, q, t, -1],
intone[p, k, sd, r, q, t, -1] - intthree[p, k, h, sd, r, q, t, -1, -1] + \
intsix[reb, p, h, sd, r, q, t, -1]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndOutPutDelta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[inttwo[p, k, h, sd, r, q, t, -1] - intfour[p, k, h, sd, \
r, q, t, -1, -1] + intsix[reb, p, h, sd, r, q, t, -1], p]],
Evaluate[D[intone[p, k, sd, r, q, t, -1] - intthree[p, k, h, sd, r, q, t, -1, \
-1] + intsix[reb, p, h, sd, r, q, t, -1],p]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndOutPutTheta[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[-D[inttwo[p, k, h, sd, r, q, t, -1] - intfour[p, k, h, \
sd, r, q, t, -1, -1] + intsix[reb, p, h, sd, r, q, t, -1], t]],
Evaluate[-D[intone[p, k, sd, r, q, t, -1] - intthree[p, k, h, sd, r, q, t, \
-1, -1] + intsix[reb, p, h, sd, r, q, t, -1],t]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndOutPutGamma[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[inttwo[p, k, h, sd, r, q, t, -1] - intfour[p, k, h, sd, \
r, q, t, -1, -1] + intsix[reb, p, h, sd, r, q, t, -1], {p,2}]],
Evaluate[D[intone[p, k, sd, r, q, t, -1] - intthree[p, k, h, sd, r, q, t, -1, \
-1] + intsix[reb, p, h, sd, r, q, t, -1],{p,2}]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndOutPutVega[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[inttwo[p, k, h, sd, r, q, t, -1] - intfour[p, k, h, sd, \
r, q, t, -1, -1] + intsix[reb, p, h, sd, r, q, t, -1], sd]],
Evaluate[D[intone[p, k, sd, r, q, t, -1] - intthree[p, k, h, sd, r, q, t, -1, \
-1] + intsix[reb, p, h, sd, r, q, t, -1],sd]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
UpAndOutPutRho[reb_, p_, k_, h_, sd_, r_, q_, t_] =
If[k >= h, Evaluate[D[inttwo[p, k, h, sd, r, q, t, -1] - intfour[p, k, h, sd, \
r, q, t, -1, -1] + intsix[reb, p, h, sd, r, q, t, -1], r]],
Evaluate[D[intone[p, k, sd, r, q, t, -1] - intthree[p, k, h, sd, r, q, t, -1, \
-1] + intsix[reb, p, h, sd, r, q, t, -1],r]]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True,
  FontSize->9],

Cell["\<\
The following tables of values can also be checked directly against \
Rubenstein and Reiner. Note the conversion to annual rates for the interest \
rates and yield.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
TableForm[
Table[N[DownAndOutPut[2,100,k,97,0.20,Log[1.1],Log[1.05],t]], {k, 90, 110, \
10}, {t, 0.5, 1.5, 0.25}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"1.61271530247703349`", "1.66960452269328074`", 
            "1.70282298061978876`", "1.72497329344680627`", 
            "1.74093250749823074`"},
          {"1.6163778175274734`", "1.67156644303354839`", 
            "1.70407078280055257`", "1.7258458318338099`", 
            "1.74158051731007095`"},
          {"1.86926183668821916`", "1.81192799317739563`", 
            "1.79498328120460071`", "1.79012556135828537`", 
            "1.78967621894720156`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
TableForm[
Table[N[DownAndOutPut[2,100,k,100,0.20,Log[1.1],Log[1.05],t]], {k, 90, 110, \
10}, {t, 0.5, 1.5, 0.25}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"2.`", "2.`", "2.`", "2.`", "2.`"},
          {"2.`", "2.`", "2.`", "2.`", "2.`"},
          {"2.0000000000000071`", "2.`", "2.`", "2.0000000000000071`", 
            "2.0000000000000071`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
TableForm[
Table[N[UpAndOutPut[2,100,k,103,0.20,Log[1.1],Log[1.05],t]], {k, 90, 110, \
10}, {t, 0.5, 1.5, 0.25}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"2.369861900443611`", "2.55060643440340184`", 
            "2.63858363981317811`", "2.6812535201050176`", 
            "2.69900620759341958`"},
          {"3.54582011812784747`", "3.52027085576882603`", 
            "3.46540611532866282`", "3.40241182943482467`", 
            "3.33814411164092472`"},
          {"4.96897007087926034`", "4.62758438553568218`", 
            "4.38153727747571952`", "4.1867827730573337`", 
            "4.02461327557985182`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["8.7 An Investigation of Up and Out Puts: ", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
What Happens When k =h (Barrier = Strike) and q > r (Dividend Yield \
Greater than Risk-Free Rate)?\
\>", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In the following plots we investigate the structure of the valuation \
function and associated Delta, Gamma, and Vega. In the first plot we note the \
almost flat structure of the value, here viewed as a surface ",
  Cell[BoxData[
      \(TraditionalForm\`V(S, t)\)]],
  ". The value is zero at the Barrier, here set to 50. The strike is also 50, \
the volatility is 0.25 (25%), and the continuously compounded interest rate ",
  
  Cell[BoxData[
      \(TraditionalForm\`r\  = 10\)]],
  "%. The dividend yield is 15%. The time is measured in years and varies \
from zero to one year. The stock price varies from 30  to 50 (barrier). "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[UpAndOutPut[0, S, 50, 50, 0.25,0.1, 0.15, t],
{S, 30, 50}, {t, 0.001, 1}, PlotRange -> All];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.7672 .09664 0 -6.16225 ]
[.7672 .09664 18 2.83775 ]
[.82794 .18966 0 -6.07272 ]
[.82794 .18966 18 2.92728 ]
[.88358 .27486 0 -5.99233 ]
[.88358 .27486 18 3.00767 ]
[.93473 .3532 0 -5.91977 ]
[.93473 .3532 18 3.08023 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.02411 .26511 -6 -2.74232 ]
[.02411 .26511 0 6.25768 ]
[.01568 .31795 -6 -2.787 ]
[.01568 .31795 0 6.213 ]
[.00696 .37264 -12 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00208 .42927 -12 -2.88161 ]
[-0.00208 .42927 0 6.11839 ]
[-0.01144 .48796 -12 -2.93176 ]
[-0.01144 .48796 0 6.06824 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.75489 .10119 m
.74873 .10346 L
s
[(0.2)] .7672 .09664 -1 .36939 Mshowa
.81555 .19399 m
.80936 .19615 L
s
[(0.4)] .82794 .18966 -1 .34949 Mshowa
.87113 .27899 m
.8649 .28106 L
s
[(0.6)] .88358 .27486 -1 .33163 Mshowa
.92222 .35714 m
.91596 .35912 L
s
[(0.8)] .93473 .3532 -1 .3155 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(1)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.70561 .0258 m
.70193 .02722 L
s
.72242 .05152 m
.71874 .05292 L
s
.73884 .07664 m
.73515 .07802 L
s
.77057 .12518 m
.76687 .12652 L
s
.7859 .14863 m
.7822 .14996 L
s
.80089 .17156 m
.79718 .17287 L
s
.8299 .21593 m
.82618 .21721 L
s
.84394 .2374 m
.84021 .23867 L
s
.85767 .25842 m
.85394 .25967 L
s
.8843 .29914 m
.88056 .30036 L
s
.8972 .31887 m
.89345 .32008 L
s
.90983 .3382 m
.90608 .3394 L
s
.93435 .37571 m
.93059 .37688 L
s
.94625 .3939 m
.94249 .39506 L
s
.95791 .41174 m
.95414 .41289 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
[(0)] .02411 .26511 1 -0.3906 Mshowa
.02795 .31328 m
.03408 .31095 L
s
[(5)] .01568 .31795 1 -0.38067 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(10)] .00696 .37264 1 -0.37036 Mshowa
.01027 .42483 m
.01644 .42261 L
s
[(15)] -0.00208 .42927 1 -0.35964 Mshowa
.00095 .48364 m
.00715 .48148 L
s
[(20)] -0.01144 .48796 1 -0.3485 Mshowa
.125 Mabswid
.03468 .27078 m
.03835 .26935 L
s
.03301 .2813 m
.03669 .27988 L
s
.03134 .29189 m
.03501 .29047 L
s
.02965 .30255 m
.03332 .30114 L
s
.02623 .32409 m
.02991 .3227 L
s
.02451 .33498 m
.02819 .33359 L
s
.02277 .34593 m
.02646 .34455 L
s
.02102 .35697 m
.02471 .3556 L
s
.01749 .37927 m
.02118 .37791 L
s
.0157 .39054 m
.0194 .38919 L
s
.0139 .40189 m
.0176 .40055 L
s
.01209 .41332 m
.01579 .41198 L
s
.00843 .43643 m
.01214 .4351 L
s
.00658 .4481 m
.01029 .44678 L
s
.00472 .45986 m
.00843 .45855 L
s
.00284 .47171 m
.00655 .4704 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.757 .783 .875 r
.3749 .78096 .39828 .79947 .43743 .77601 .4145 .75699 Metetra
.757 .782 .874 r
.4145 .75699 .43743 .77601 .47687 .75231 .45439 .73279 Metetra
.757 .781 .873 r
.45439 .73279 .47687 .75231 .51658 .72835 .49457 .70833 Metetra
.757 .781 .872 r
.49457 .70833 .51658 .72835 .55657 .70411 .53503 .6836 Metetra
.758 .78 .871 r
.53503 .6836 .55657 .70411 .59684 .67953 .57577 .65854 Metetra
.759 .779 .87 r
.57577 .65854 .59684 .67953 .63738 .65458 .6168 .63311 Metetra
.76 .779 .869 r
.6168 .63311 .63738 .65458 .67819 .62919 .6581 .60726 Metetra
.762 .779 .867 r
.6581 .60726 .67819 .62919 .71926 .60331 .69966 .58093 Metetra
.764 .779 .866 r
.69966 .58093 .71926 .60331 .76057 .57688 .74148 .55405 Metetra
.766 .78 .864 r
.74148 .55405 .76057 .57688 .80211 .54984 .78353 .52657 Metetra
.769 .78 .862 r
.78353 .52657 .80211 .54984 .84385 .52212 .8258 .4984 Metetra
.772 .781 .86 r
.8258 .4984 .84385 .52212 .88577 .49365 .86826 .4695 Metetra
.776 .783 .858 r
.86826 .4695 .88577 .49365 .92784 .46438 .91088 .43978 Metetra
.78 .784 .856 r
.91088 .43978 .92784 .46438 .97003 .43424 .95362 .4092 Metetra
.757 .783 .874 r
.35087 .76193 .3749 .78096 .4145 .75699 .39094 .73743 Metetra
.757 .782 .874 r
.39094 .73743 .4145 .75699 .45439 .73279 .4313 .7127 Metetra
.757 .781 .873 r
.4313 .7127 .45439 .73279 .49457 .70833 .47195 .68774 Metetra
.758 .78 .872 r
.47195 .68774 .49457 .70833 .53503 .6836 .51289 .66249 Metetra
.758 .78 .871 r
.51289 .66249 .53503 .6836 .57577 .65854 .55412 .63694 Metetra
.759 .779 .87 r
.55412 .63694 .57577 .65854 .6168 .63311 .59564 .61102 Metetra
.76 .779 .868 r
.59564 .61102 .6168 .63311 .6581 .60726 .63744 .58469 Metetra
.762 .779 .867 r
.63744 .58469 .6581 .60726 .69966 .58093 .67952 .55789 Metetra
.764 .779 .866 r
.67952 .55789 .69966 .58093 .74148 .55405 .72185 .53055 Metetra
.766 .78 .864 r
.72185 .53055 .74148 .55405 .78353 .52657 .76444 .50261 Metetra
.769 .78 .862 r
.76444 .50261 .78353 .52657 .8258 .4984 .80725 .474 Metetra
.772 .781 .86 r
.80725 .474 .8258 .4984 .86826 .4695 .85025 .44465 Metetra
.776 .782 .858 r
.85025 .44465 .86826 .4695 .91088 .43978 .89343 .41448 Metetra
.78 .784 .856 r
.89343 .41448 .91088 .43978 .95362 .4092 .93675 .38345 Metetra
.758 .783 .874 r
.32617 .74235 .35087 .76193 .39094 .73743 .36672 .7173 Metetra
.758 .782 .873 r
.36672 .7173 .39094 .73743 .4313 .7127 .40756 .69204 Metetra
.758 .781 .872 r
.40756 .69204 .4313 .7127 .47195 .68774 .4487 .66654 Metetra
.758 .78 .871 r
.4487 .66654 .47195 .68774 .51289 .66249 .49013 .64077 Metetra
.759 .78 .87 r
.49013 .64077 .51289 .66249 .55412 .63694 .53186 .6147 Metetra
.759 .779 .869 r
.53186 .6147 .55412 .63694 .59564 .61102 .57389 .58827 Metetra
.76 .779 .868 r
.57389 .58827 .59564 .61102 .63744 .58469 .6162 .56145 Metetra
.762 .779 .867 r
.6162 .56145 .63744 .58469 .67952 .55789 .6588 .53416 Metetra
.764 .779 .865 r
.6588 .53416 .67952 .55789 .72185 .53055 .70167 .50635 Metetra
.766 .779 .864 r
.70167 .50635 .72185 .53055 .76444 .50261 .74479 .47795 Metetra
.769 .78 .862 r
.74479 .47795 .76444 .50261 .80725 .474 .78816 .44888 Metetra
.772 .781 .86 r
.78816 .44888 .80725 .474 .85025 .44465 .83173 .41907 Metetra
.775 .782 .858 r
.83173 .41907 .85025 .44465 .89343 .41448 .87549 .38845 Metetra
.779 .784 .856 r
.87549 .38845 .89343 .41448 .93675 .38345 .91939 .35695 Metetra
.759 .783 .873 r
.30078 .72221 .32617 .74235 .36672 .7173 .34182 .6966 Metetra
.759 .782 .872 r
.34182 .6966 .36672 .7173 .40756 .69204 .38315 .67077 Metetra
.759 .781 .872 r
.38315 .67077 .40756 .69204 .4487 .66654 .42479 .64471 Metetra
.759 .78 .871 r
.42479 .64471 .4487 .66654 .49013 .64077 .46673 .6184 Metetra
.759 .779 .87 r
.46673 .6184 .49013 .64077 .53186 .6147 .50897 .59178 Metetra
.76 .779 .869 r
.50897 .59178 .53186 .6147 .57389 .58827 .55151 .56484 Metetra
.761 .778 .868 r
.55151 .56484 .57389 .58827 .6162 .56145 .59435 .5375 Metetra
.762 .778 .867 r
.59435 .5375 .6162 .56145 .6588 .53416 .63748 .50972 Metetra
.764 .778 .865 r
.63748 .50972 .6588 .53416 .70167 .50635 .6809 .48142 Metetra
.766 .779 .864 r
.6809 .48142 .70167 .50635 .74479 .47795 .72458 .45254 Metetra
.768 .78 .862 r
.72458 .45254 .74479 .47795 .78816 .44888 .76851 .423 Metetra
.771 .781 .86 r
.76851 .423 .78816 .44888 .83173 .41907 .81266 .39273 Metetra
.775 .782 .859 r
.81266 .39273 .83173 .41907 .87549 .38845 .85701 .36164 Metetra
.779 .783 .856 r
.85701 .36164 .87549 .38845 .91939 .35695 .90152 .32967 Metetra
.759 .783 .872 r
.27466 .70147 .30078 .72221 .34182 .6966 .3162 .67528 Metetra
.759 .782 .872 r
.3162 .67528 .34182 .6966 .38315 .67077 .35804 .64887 Metetra
.759 .781 .871 r
.35804 .64887 .38315 .67077 .42479 .64471 .40019 .62223 Metetra
.759 .78 .87 r
.40019 .62223 .42479 .64471 .46673 .6184 .44265 .59534 Metetra
.759 .779 .869 r
.44265 .59534 .46673 .6184 .50897 .59178 .48541 .56817 Metetra
.76 .779 .868 r
.48541 .56817 .50897 .59178 .55151 .56484 .52849 .54068 Metetra
.761 .778 .867 r
.52849 .54068 .55151 .56484 .59435 .5375 .57187 .51281 Metetra
.762 .778 .866 r
.57187 .51281 .59435 .5375 .63748 .50972 .61555 .48452 Metetra
.764 .778 .865 r
.61555 .48452 .63748 .50972 .6809 .48142 .65952 .45572 Metetra
.766 .779 .864 r
.65952 .45572 .6809 .48142 .72458 .45254 .70377 .42635 Metetra
.768 .779 .862 r
.70377 .42635 .72458 .45254 .76851 .423 .74828 .39633 Metetra
.771 .78 .861 r
.74828 .39633 .76851 .423 .81266 .39273 .79303 .36559 Metetra
.775 .782 .859 r
.79303 .36559 .81266 .39273 .85701 .36164 .83799 .33403 Metetra
.779 .783 .857 r
.83799 .33403 .85701 .36164 .90152 .32967 .88312 .30158 Metetra
.76 .783 .872 r
.24778 .68012 .27466 .70147 .3162 .67528 .28984 .65332 Metetra
.76 .782 .871 r
.28984 .65332 .3162 .67528 .35804 .64887 .3322 .6263 Metetra
.76 .781 .87 r
.3322 .6263 .35804 .64887 .40019 .62223 .37488 .59907 Metetra
.76 .78 .87 r
.37488 .59907 .40019 .62223 .44265 .59534 .41787 .57159 Metetra
.76 .779 .869 r
.41787 .57159 .44265 .59534 .48541 .56817 .46117 .54383 Metetra
.76 .778 .868 r
.46117 .54383 .48541 .56817 .52849 .54068 .50479 .51577 Metetra
.761 .778 .867 r
.50479 .51577 .52849 .54068 .57187 .51281 .54872 .48735 Metetra
.762 .778 .866 r
.54872 .48735 .57187 .51281 .61555 .48452 .59296 .45852 Metetra
.763 .778 .865 r
.59296 .45852 .61555 .48452 .65952 .45572 .63751 .42921 Metetra
.765 .778 .864 r
.63751 .42921 .65952 .45572 .70377 .42635 .68235 .39934 Metetra
.768 .779 .862 r
.68235 .39934 .70377 .42635 .74828 .39633 .72745 .36883 Metetra
.771 .78 .861 r
.72745 .36883 .74828 .39633 .79303 .36559 .77281 .33761 Metetra
.774 .781 .859 r
.77281 .33761 .79303 .36559 .83799 .33403 .81839 .30557 Metetra
.778 .783 .857 r
.81839 .30557 .83799 .33403 .88312 .30158 .86416 .27265 Metetra
.761 .783 .871 r
.2201 .65813 .24778 .68012 .28984 .65332 .2627 .6307 Metetra
.761 .782 .87 r
.2627 .6307 .28984 .65332 .3322 .6263 .3056 .60305 Metetra
.76 .781 .87 r
.3056 .60305 .3322 .6263 .37488 .59907 .34882 .57519 Metetra
.76 .78 .869 r
.34882 .57519 .37488 .59907 .41787 .57159 .39236 .54709 Metetra
.76 .779 .868 r
.39236 .54709 .41787 .57159 .46117 .54383 .43621 .51873 Metetra
.761 .778 .867 r
.43621 .51873 .46117 .54383 .50479 .51577 .48039 .49008 Metetra
.761 .778 .866 r
.48039 .49008 .50479 .51577 .54872 .48735 .52489 .46108 Metetra
.762 .777 .866 r
.52489 .46108 .54872 .48735 .59296 .45852 .56971 .4317 Metetra
.763 .777 .865 r
.56971 .4317 .59296 .45852 .63751 .42921 .61484 .40185 Metetra
.765 .778 .863 r
.61484 .40185 .63751 .42921 .68235 .39934 .66027 .37147 Metetra
.767 .778 .862 r
.66027 .37147 .68235 .39934 .72745 .36883 .70599 .34046 Metetra
.77 .78 .861 r
.70599 .34046 .72745 .36883 .77281 .33761 .75198 .30875 Metetra
.774 .781 .859 r
.75198 .30875 .77281 .33761 .81839 .30557 .7982 .27623 Metetra
.777 .783 .857 r
.7982 .27623 .81839 .30557 .86416 .27265 .84462 .24283 Metetra
.762 .783 .87 r
.1916 .63547 .2201 .65813 .2627 .6307 .23475 .60738 Metetra
.761 .782 .87 r
.23475 .60738 .2627 .6307 .3056 .60305 .2782 .57908 Metetra
.761 .781 .869 r
.2782 .57908 .3056 .60305 .34882 .57519 .32198 .55057 Metetra
.761 .78 .868 r
.32198 .55057 .34882 .57519 .39236 .54709 .36608 .52182 Metetra
.761 .779 .868 r
.36608 .52182 .39236 .54709 .43621 .51873 .4105 .49283 Metetra
.761 .778 .867 r
.4105 .49283 .43621 .51873 .48039 .49008 .45525 .46355 Metetra
.761 .777 .866 r
.45525 .46355 .48039 .49008 .52489 .46108 .50033 .43396 Metetra
.762 .777 .865 r
.50033 .43396 .52489 .46108 .56971 .4317 .54574 .40399 Metetra
.763 .777 .864 r
.54574 .40399 .56971 .4317 .61484 .40185 .59147 .37359 Metetra
.765 .777 .863 r
.59147 .37359 .61484 .40185 .66027 .37147 .63752 .34268 Metetra
.767 .778 .862 r
.63752 .34268 .66027 .37147 .70599 .34046 .68387 .31117 Metetra
.77 .779 .861 r
.68387 .31117 .70599 .34046 .75198 .30875 .73049 .27896 Metetra
.773 .781 .859 r
.73049 .27896 .75198 .30875 .7982 .27623 .77737 .24596 Metetra
.777 .782 .857 r
.77737 .24596 .7982 .27623 .84462 .24283 .82447 .21208 Metetra
.763 .783 .87 r
.16224 .61211 .1916 .63547 .23475 .60738 .20595 .58335 Metetra
.762 .782 .869 r
.20595 .58335 .23475 .60738 .2782 .57908 .24997 .55437 Metetra
.762 .781 .868 r
.24997 .55437 .2782 .57908 .32198 .55057 .29432 .52517 Metetra
.762 .779 .868 r
.29432 .52517 .32198 .55057 .36608 .52182 .339 .49575 Metetra
.762 .778 .867 r
.339 .49575 .36608 .52182 .4105 .49283 .38401 .46609 Metetra
.761 .778 .866 r
.38401 .46609 .4105 .49283 .45525 .46355 .42935 .43616 Metetra
.762 .777 .865 r
.42935 .43616 .45525 .46355 .50033 .43396 .47502 .40594 Metetra
.762 .776 .865 r
.47502 .40594 .50033 .43396 .54574 .40399 .52104 .37537 Metetra
.763 .776 .864 r
.52104 .37537 .54574 .40399 .59147 .37359 .56738 .34439 Metetra
.764 .777 .863 r
.56738 .34439 .59147 .37359 .63752 .34268 .61406 .31293 Metetra
.766 .777 .862 r
.61406 .31293 .63752 .34268 .68387 .31117 .66105 .2809 Metetra
.769 .778 .861 r
.66105 .2809 .68387 .31117 .73049 .27896 .70834 .2482 Metetra
.772 .78 .859 r
.70834 .2482 .73049 .27896 .77737 .24596 .75589 .21472 Metetra
.776 .782 .858 r
.75589 .21472 .77737 .24596 .82447 .21208 .80369 .18035 Metetra
.763 .783 .869 r
.13197 .58802 .16224 .61211 .20595 .58335 .17626 .55855 Metetra
.763 .782 .868 r
.17626 .55855 .20595 .58335 .24997 .55437 .22087 .52887 Metetra
.763 .78 .868 r
.22087 .52887 .24997 .55437 .29432 .52517 .26581 .49897 Metetra
.762 .779 .867 r
.26581 .49897 .29432 .52517 .339 .49575 .31109 .46884 Metetra
.762 .778 .866 r
.31109 .46884 .339 .49575 .38401 .46609 .35669 .43847 Metetra
.762 .777 .866 r
.35669 .43847 .38401 .46609 .42935 .43616 .40264 .40786 Metetra
.762 .776 .865 r
.40264 .40786 .42935 .43616 .47502 .40594 .44893 .37697 Metetra
.762 .776 .864 r
.44893 .37697 .47502 .40594 .52104 .37537 .49557 .34576 Metetra
.763 .776 .864 r
.49557 .34576 .52104 .37537 .56738 .34439 .54254 .31419 Metetra
.764 .776 .863 r
.54254 .31419 .56738 .34439 .61406 .31293 .58986 .28216 Metetra
.766 .777 .862 r
.58986 .28216 .61406 .31293 .66105 .2809 .63751 .24961 Metetra
.768 .778 .861 r
.63751 .24961 .66105 .2809 .70834 .2482 .68547 .2164 Metetra
.771 .779 .86 r
.68547 .2164 .70834 .2482 .75589 .21472 .73373 .18245 Metetra
.775 .782 .858 r
.73373 .18245 .75589 .21472 .80369 .18035 .78223 .14761 Metetra
.764 .783 .868 r
.10075 .56316 .13197 .58802 .17626 .55855 .14564 .53297 Metetra
.764 .782 .868 r
.14564 .53297 .17626 .55855 .22087 .52887 .19086 .50255 Metetra
.764 .78 .867 r
.19086 .50255 .22087 .52887 .26581 .49897 .23641 .47192 Metetra
.763 .779 .866 r
.23641 .47192 .26581 .49897 .31109 .46884 .2823 .44105 Metetra
.763 .778 .866 r
.2823 .44105 .31109 .46884 .35669 .43847 .32853 .40995 Metetra
.763 .777 .865 r
.32853 .40995 .35669 .43847 .40264 .40786 .3751 .37861 Metetra
.762 .776 .864 r
.3751 .37861 .40264 .40786 .44893 .37697 .42202 .34701 Metetra
.762 .775 .864 r
.42202 .34701 .44893 .37697 .49557 .34576 .46929 .31513 Metetra
.763 .775 .863 r
.46929 .31513 .49557 .34576 .54254 .31419 .51692 .28291 Metetra
.764 .775 .862 r
.51692 .28291 .54254 .31419 .58986 .28216 .56489 .2503 Metetra
.765 .776 .862 r
.56489 .2503 .58986 .28216 .63751 .24961 .61321 .2172 Metetra
.767 .777 .861 r
.61321 .2172 .63751 .24961 .68547 .2164 .66187 .1835 Metetra
.77 .779 .86 r
.66187 .1835 .68547 .2164 .73373 .18245 .71084 .14908 Metetra
.774 .781 .859 r
.71084 .14908 .73373 .18245 .78223 .14761 .76008 .1138 Metetra
.765 .783 .868 r
.06854 .53751 .10075 .56316 .14564 .53297 .11405 .50656 Metetra
.765 .782 .867 r
.11405 .50656 .14564 .53297 .19086 .50255 .15989 .47539 Metetra
.764 .78 .866 r
.15989 .47539 .19086 .50255 .23641 .47192 .20607 .44399 Metetra
.764 .779 .866 r
.20607 .44399 .23641 .47192 .2823 .44105 .25259 .41235 Metetra
.764 .778 .865 r
.25259 .41235 .2823 .44105 .32853 .40995 .29946 .38048 Metetra
.763 .777 .864 r
.29946 .38048 .32853 .40995 .3751 .37861 .34668 .34837 Metetra
.763 .776 .864 r
.34668 .34837 .3751 .37861 .42202 .34701 .39425 .31602 Metetra
.763 .775 .863 r
.39425 .31602 .42202 .34701 .46929 .31513 .44218 .2834 Metetra
.763 .774 .862 r
.44218 .2834 .46929 .31513 .51692 .28291 .49046 .2505 Metetra
.763 .774 .862 r
.49046 .2505 .51692 .28291 .56489 .2503 .53911 .21726 Metetra
.764 .774 .861 r
.53911 .21726 .56489 .2503 .61321 .2172 .58813 .1836 Metetra
.766 .775 .861 r
.58813 .1836 .61321 .2172 .66187 .1835 .63749 .14941 Metetra
.768 .777 .86 r
.63749 .14941 .66187 .1835 .71084 .14908 .68719 .11455 Metetra
.772 .78 .859 r
.68719 .11455 .71084 .14908 .76008 .1138 .73719 .07886 Metetra
.766 .783 .867 r
.03529 .51101 .06854 .53751 .11405 .50656 .08144 .47929 Metetra
.766 .782 .866 r
.08144 .47929 .11405 .50656 .15989 .47539 .12792 .44733 Metetra
.765 .78 .865 r
.12792 .44733 .15989 .47539 .20607 .44399 .17475 .41514 Metetra
.765 .779 .865 r
.17475 .41514 .20607 .44399 .25259 .41235 .22193 .3827 Metetra
.764 .778 .864 r
.22193 .3827 .25259 .41235 .29946 .38048 .26946 .35003 Metetra
.764 .777 .864 r
.26946 .35003 .29946 .38048 .34668 .34837 .31734 .31712 Metetra
.764 .776 .863 r
.31734 .31712 .34668 .34837 .39425 .31602 .36558 .28395 Metetra
.763 .775 .862 r
.36558 .28395 .39425 .31602 .44218 .2834 .41418 .25054 Metetra
.763 .774 .862 r
.41418 .25054 .44218 .2834 .49046 .2505 .46315 .21687 Metetra
.763 .773 .861 r
.46315 .21687 .49046 .2505 .53911 .21726 .51249 .18292 Metetra
.763 .773 .861 r
.51249 .18292 .53911 .21726 .58813 .1836 .56221 .14866 Metetra
.764 .774 .861 r
.56221 .14866 .58813 .1836 .63749 .14941 .61229 .11398 Metetra
.766 .775 .86 r
.61229 .11398 .63749 .14941 .68719 .11455 .66274 .07875 Metetra
.77 .779 .86 r
.66274 .07875 .68719 .11455 .73719 .07886 .71353 .04274 Metetra
.767 .783 .866 r
.00095 .48363 .03529 .51101 .08144 .47929 .04776 .4511 Metetra
.766 .781 .865 r
.04776 .4511 .08144 .47929 .12792 .44733 .09491 .41833 Metetra
.766 .78 .865 r
.09491 .41833 .12792 .44733 .17475 .41514 .14241 .38532 Metetra
.766 .779 .864 r
.14241 .38532 .17475 .41514 .22193 .3827 .19026 .35206 Metetra
.765 .778 .863 r
.19026 .35206 .22193 .3827 .26946 .35003 .23847 .31856 Metetra
.765 .777 .863 r
.23847 .31856 .26946 .35003 .31734 .31712 .28704 .28481 Metetra
.765 .776 .862 r
.28704 .28481 .31734 .31712 .36558 .28395 .33597 .2508 Metetra
.764 .775 .862 r
.33597 .2508 .36558 .28395 .41418 .25054 .38527 .21654 Metetra
.764 .774 .861 r
.38527 .21654 .41418 .25054 .46315 .21687 .43494 .18201 Metetra
.764 .772 .86 r
.43494 .18201 .46315 .21687 .51249 .18292 .48499 .14723 Metetra
.763 .771 .86 r
.48499 .14723 .51249 .18292 .56221 .14866 .53543 .11218 Metetra
.763 .771 .859 r
.53543 .11218 .56221 .14866 .61229 .11398 .58624 .07686 Metetra
.763 .771 .86 r
.58624 .07686 .61229 .11398 .66274 .07875 .63745 .04128 Metetra
.764 .774 .861 r
.63745 .04128 .66274 .07875 .71353 .04274 .68905 .00539 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{174.375, 141.312},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002^0000SA000`40O003h00O[Woo002^Ool00:io
o`00[Woo002^Ool00:ioo`00K7oo0`0017oo0P00>Goo001_Ool01000Oomoo`000Woo00<007ooOol0
=Woo001_Ool01000Oomoo`000Woo00<007ooOol0=Woo001/Ool30003Ool01000Oomoo`00>7oo001/
Ool00`00Oomoo`03Ool01000Oomoo`00>7oo001/Ool40003Ool2000iOol00:ioo`00[Woo001eOol2
000gOol0079oo`D003Moo`00DGoo0`000Woo0`005goo0P000Woo00@006<K000003Ioo`00DWoo00<0
07ooOol017oo00<007ooOol04Goo100000=oo`0000000V<K0`00=Goo001@Ool40005Ool00`00Oomo
o`0?Ool20003Ool20004Ha/3000eOol0051oo`03001oo`0000=oo`<000moo`@000=oo`03001S6f<K
00AS6`04001S6`00000dOol0051oo`03001oo`0000=oo`03001oogoo00aoo`<000Eoo`8000MS6`05
001S6`00Ool0000cOol0055oo`8000=oo`@000Uoo`@000Eoo`03001S6`0000MS6`03001S6`000080
03=oo`00H7oo0`0027oo00D006<KHa]S6`0000IS6`800004Ha/00000000bOol005ioo`<000Qoo`80
00ES6`03001S6f<K009S6`8000=S6`80039oo`00Fgoo0`002Woo00<006<KHa/01F<K00D006<KHa]S
6`00009S6`04001S6f<KHa/2000aOol005Uoo`<000]oo`03001S6f<K00MS6`03001S6`0000=S6`03
001S6f<K009S6`8000Eoo`8000Aoo`8000=oo`@001aoo`00=goo0`000goo0P005goo0`000Woo00<0
07ooOol01goo0P002V<K0P0016<K00<006<KHa/00V<K0P0017oo00@007ooOol000=oo`8000Aoo`03
001oogoo01aoo`00>7oo00D007ooOomoo`00009oo`03001oogoo019oo`<000eoo`03001S6`0000US
6`03001S6`0000AS6`06001S6f<KHa/006<K0P000goo00@007ooOol000Yoo`03001oogoo01]oo`00
=Woo10000Woo00@007ooOol0015oo`<000ioo`8000=S6`03001S6f<K00AS6`8000=S6`05001S6f<K
Ha/00002Ha/01000Ha]S6f<K0P000Woo00@007ooOol000]oo`03001oogoo01Yoo`00=Woo00<007oo
00000goo00@007ooOol000moo`<000ioo`8000IS6`04001S6f<KHa/20005Ha/01000Ha]S6f<K0`00
1F<K00@007ooOol0009oo`03001oogoo00Ioo`04001oogoo000LOol003Ioo`03001oo`0000=oo`04
001oogoo000<Ool3000@Ool00`00Ha]S6`07Ha/00`00Ha/00008Ha/01000Ha]S6`001f<K0P000Woo
0P002Woo0P007Goo000gOol20004Ool2000;Ool3000@Ool2000:Ha/20009Ha/01000Ha/000001f<K
0`00;7oo0017Ool3000BOol00`00Ha/00009Ha/00`00Ha/00009Ha/200000f<K001S6`05Ha/01000
Ha/00000;7oo0014Ool4000COol01@00Ha]S6f<K00001f<K00D006<KHa]S6`0000MS6`05001S6`00
Ha/00005Ha/01000Ha]S6f<K0P00:goo0012Ool2000FOol00`00Ha]S6`03Ha/00`00Ha]S6`02Ha/2
0005Ha/00`00Ha]S6`02Ha/20003Ha/20004Ha/00`00Ha]S6`03Ha/2000ZOol003moo`@001Ioo`03
001S6f<K00AS6`05001S6f<KHa/00007Ha/01@00Ha]S6f<K00001V<K00D006<KHa]S6`0000IS6`80
02Yoo`007Goo0`000goo0`005goo0P006Goo00<006<KHa/01V<K00<006<K00002F<K00<006<K0000
1f<K0P0000=S6`00Ha/01f<K0P00:Goo000POol00`00Oomoo`03Ool00`00Oomoo`0AOol4000HOol2
0009Ha/2000:Ha/20008Ha/30009Ha/3000XOol0021oo`03001oogoo00=oo`03001oogoo00moo`80
01]oo`03001S6`0000QS6`03001S6`0000US6`03001S6`0000QS6`03001S6`0000QS6`04001S>`00
000XOol001ioo`8000=oo`<000moo`@001Yoo`8000=S6`03001S6f<K00=S6`8000=S6`03001S6f<K
00AS6`8000=S6`03001S6f<K00=S6`<0009S6`03001S6f<K00=S6`8000=S>`8000Ioo`8000Aoo`80
00Aoo`<0019oo`0087oo00@007ooOol000moo`8001aoo`8000IS6`05001S6f<KHa/00006Ha/01000
Ha]S6f<K0P001V<K00D006<KHa]S6`00009S6`05001S6f<KHa/00002Ha/20006Hc/20004Ool01000
Oomoo`000goo0P001Goo00<007ooOol04Goo000MOol30003Ool40009Ool4000LOol00`00Ha]S6`07
Ha/00`00Ha/00008Ha/00`00Ha/00009Ha/00`00Ha/00003Ha/01`00Ha]S6f<K001S6`0000QS>`80
00Aoo`04001oogoo0008Ool4000BOol002ioo`8001ioo`8000YS6`8000US6`8000YS6`8000AS6`04
001S6f<KHa/20009Hc/30003Ool01000Oomoo`0027oo00<007oo00004goo000[Ool4000NOol00`00
Ha/00009Ha/00`00Ha/00008Ha/00`00Ha/00009Ha/00`00Ha/00005Ha/01@00Ha/006<K000026<k
00@006<k000000=oo`04001oogoo0008Ool00`00Ool0000COol002Uoo`80021oo`8000=S6`03001S
6f<K00AS6`8000=S6`03001S6f<K00=S6`8000=S6`03001S6f<K00AS6`8000=S6`03001S6f<K009S
6`8000=S6`03001S>f<k00=S>`8000=S>`8000=oo`8000Yoo`8001=oo`001Woo0`0017oo0P005goo
0`0000=oo`00Ool07Woo00<006<KHa/016<K00@006<KHa]S6`8000IS6`05001S6f<KHa/00006Ha/0
1000Ha]S6f<K0P001V<K00<006<KHa/00P001F<K00D006<kHc]S>`0000IS>`8002=oo`002Goo00@0
07ooOol0009oo`03001oogoo015oo`<002=oo`03001S6f<K00IS6`03001S6`0000US6`03001S6`00
00QS6`03001S6`0000QS6`05001S6`00Ha/00006Ha/00`00Hc/00007Hc/2000SOol000Uoo`04001o
ogoo0002Ool00`00Oomoo`0?Ool2000TOol20009Ha/2000:Ha/20009Ha/20009Ha/20002Ha/00`00
Ha]S6`04Ha/20007Hc/01000Hc/000008Woo0007Ool20003Ool01000Oomoo`003Woo100097oo00<0
06<K000026<K00@006<KHa/000QS6`03001S6`0000QS6`03001S6`0000QS6`03001S6`00009S6`03
001S6f<K00=S6`03001S6`0000ES>`8000=S>`80025oo`002Goo00@007ooOol0009oo`03001oogoo
00Yoo`8002Ioo`8000=S6`03001S6f<K00=S6`8000AS6`03001S6f<K00=S6`8000=S6`03001S6f<K
00=S6`8000=S6`03001S6f<K00=S6`8000=S6`03001S6`0000=S6`8000=S6`05001S>f<kHc/00006
Hc/00`00Oomoo`0OOol000Ioo`<000Aoo`8000Yoo`@002Eoo`8000IS6`05001S6f<KHa/00007Ha/0
1@00Ha]S6f<K00001V<K00D006<KHa]S6`0000IS6`05001S6f<KHa/00006Ha/20002Ha/00`00Ha]S
6`04Ha/00`00Hc/00007Hc/2000POol001Moo`8002Qoo`03001S6f<K00MS6`03001S6`0000US6`03
001S6`0000QS6`03001S6`0000QS6`03001S6`0000MS6`800003Ha/006<K00IS6`8000QS>`<001mo
o`0057oo0`0000=oo`00Ool09Goo0P002V<K0P002V<K0P002F<K0P002F<K0P001F<K0`0000=S6`00
00001f<K00<006<K00001f<k00@006<k000000Ioo`8000Aoo`8000Aoo`8000]oo`004goo0P00:Goo
00@006<KHa/000QS6`03001S6`0000US6`03001S6`0000QS6`03001S6`0000QS6`03001S6`00009S
6`<000=S6`03001S6`0000ES6`8000=S6`03001S>f<k009S>`8000=S>`8000Aoo`04001oogoo0003
Ool20003Ool01000Oomoo`002Woo000=Ool20004Ool3000VOol20004Ha/00`00Ha]S6`03Ha/20003
Ha/00`00Ha]S6`04Ha/20003Ha/00`00Ha]S6`03Ha/20003Ha/00`00Ha]S6`03Ha/20002Ha/30004
Ha/20003Ha/01@00Ha]S6f<K00001V<K00@006<kHc/000IS>`8000=oo`04001oogoo0008Ool01000
Oomoo`002Woo000<Ool01000Oomoo`000Woo0P000Woo00<007ooOol08Woo00<006<KHa/01F<K00D0
06<KHa]S6`0000IS6`04001S6f<KHa/20006Ha/01@00Ha]S6f<K00001V<K00H006<KHa]S6`00Ha/3
0002Ha/00`00Ha]S6`020006Ha/00`00Ha/00007Ha/00`00Hc/00007Hc/20003Ool01000Oomoo`00
27oo0`002goo000<Ool01000Oomoo`000goo00D007ooOomoo`00029oo`03001S6f<K00MS6`03001S
6`0000QS6`03001S6`0000US6`03001S6`0000QS6`03001S6`00008000ES6`03001S6`0000QS6`80
00QS6`8000QS>`<0009oo`04001oogoo0008Ool00`00Oomoo`0;Ool000aoo`04001oogoo0002Ool2
0004Ool00`00Oomoo`0MOol2000:Ha/20009Ha/2000:Ha/20008Ha/30008Ha/20008Ha/00`00Ha/0
0007Ha/00`00Ha/00007Hc/01000Hc/000000goo0P002Woo0`002Woo000<Ool01000Oomoo`000Woo
0P001Goo00<007ooOol06goo00<006<K00002F<K00<006<K000026<K00<006<K00002F<K00<006<K
00001F<K100000=S6`00Ha/01V<K00@006<KHa/000ES6`8000=S6`03001S6f<K009S6`8000=S6`03
001S>f<k009S>`8000=S>`8001]oo`003Goo0P000goo00<007ooOol01Goo00<007ooOol067oo0P00
0f<K00<006<KHa/016<K0P000f<K00<006<KHa/00f<K0P000f<K00<006<KHa/016<K0P000f<K00<0
06<K00000P0000=S6`0000000f<K00<006<KHa/00V<K0P0016<K00D006<KHa]S6`0000ES6`80009S
6`03001S6f<K00AS6`04001S>f<k0006Hc/2000JOol0019oo`03001oogoo00Ioo`03001oogoo01Eo
o`8000IS6`04001S6f<KHa/20006Ha/01@00Ha]S6f<K00001V<K00@006<KHa]S6`8000ES6`80009S
6`8000IS6`04001S6f<K0007Ha/00`00Ha/00007Ha/00`00Ha/00007Ha/00`00Hc/00007Hc/2000J
Ool0019oo`03001oogoo00Moo`03001oogoo01=oo`03001S6f<K00MS6`03001S6`0000US6`03001S
6`0000QS6`03001S6`0000AS6`<00004Ha/006<K0008Ha/00`00Ha/00008Ha/20008Ha/20008Ha/2
0007Hc/01000Hc/000006Goo000;Ool30003Ool2000:Ool00`00Oomoo`0@Ool2000:Ha/2000:Ha/2
0009Ha/20002Ha/30004Ha/20009Ha/20008Ha/00`00Ha/00007Ha/01000Ha/000001F<K0P0000=S
6`00Hc/016<k0P000f<k0P0067oo000>Ool00`00Oomoo`02000;Ool00`00Oomoo`0>Ool01000Ha]S
6`0026<K00@006<KHa/000QS6`03001S6`0000QS6`03001S6`00008000IS6`04001S6f<K0006Ha/2
0002Ha/00`00Ha]S6`03Ha/20003Ha/00`00Ha]S6`02Ha/20003Ha/00`00Ha]S6`02Ha/00`00Ha]S
6`02Ha/01@00Hc]S>f<k00001V<k00<007ooOol05Woo000>Ool01000Oomoo`003Goo00<007ooOol0
37oo00<006<KHa/00V<K00<006<KHa/00f<K0P0016<K00<006<KHa/00f<K0P000f<K00<006<KHa/0
0f<K100000=S6`00Ha/016<K0P0016<K00<006<KHa/00V<K00<006<KHa/00f<K00D006<KHa]S6`00
00IS6`04001S6f<K0006Ha/01000Ha/000001V<K00<006<k00001f<k0P001Woo0P0017oo0P0017oo
0P000goo000;Ool30003Ool2000=Ool00`00Oomoo`0:Ool00`00Ha]S6`04Ha/01@00Ha]S6f<K0000
1f<K00D006<KHa]S6`0000IS6`03001S6f<K00<000ES6`05001S6f<KHa/00007Ha/01000Ha/00000
1f<K00<006<K00001f<K00<006<K00001f<K0P0026<K0P001f<k00@006<k000000Aoo`04001oogoo
0003Ool20003Ool01000Oomoo`000Woo000;Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool20009Ool0
0`00Ha]S6`06Ha/00`00Ha/00009Ha/00`00Ha/00007Ha/40008Ha/00`00Ha/00008Ha/20009Ha/2
0008Ha/20007Ha/01000Ha/000001V<K00<006<k00001F<k0P000f<k00<007ooOol00Woo00@007oo
Ool000Qoo`04001oogoo0002Ool000]oo`@0009oo`03001oogoo00moo`03001oogoo00Eoo`03001S
6f<K00MS6`8000YS6`8000ES6`<00003Ha/0000000US6`8000QS6`03001S6`0000QS6`03001S6`00
00MS6`03001S6`0000ES6`8000=S6`03001S6f<K009S6`8000=S>`05001S>f<kHc/00006Hc/01@00
Oomoogoo00000Woo00<007ooOol01goo0P000goo000@Ool2000BOol00`00Oomoo`02Ool30008Ha/0
1000Ha]S6`0026<K00<006<K00000V<K0`000f<K00@006<KHa/000IS6`80009S6`03001S6f<K00=S
6`8000=S6`03001S6f<K00=S6`8000=S6`03001S6f<K009S6`8000=S6`05001S6f<KHa/00006Ha/0
1000Ha]S6`001V<k00<006<k00001f<k0P000Woo00@007ooOol000Qoo`04001oogoo0002Ool0011o
o`03001oogoo019oo`04001oogoo0003Ha/00`00Ha]S6`03Ha/20004Ha/00`00Ha]S6`03Ha/20002
Ha/30004Ha/20004Ha/00`00Ha]S6`02Ha/00`00Ha]S6`03Ha/01@00Ha]S6f<K00001V<K00D006<K
Ha]S6`0000IS6`04001S6f<K0006Ha/00`00Ha/00007Ha/30007Hc/20007Hc/00`00Hc/00003Ool2
000:Ool20003Ool0011oo`8001Aoo`8000ES6`05001S6f<KHa/00007Ha/01@00Ha]S6f<K00000V<K
0P000V<K00D006<KHa]S6`0000MS6`04001S6`000007Ha/00`00Ha/00008Ha/00`00Ha/00007Ha/0
0`00Ha/00007Ha/20008Ha/20007Hc/00`00Hc/00005Hc/20003Hc/00`00Oomoo`0AOol000=oo`<0
00Aoo`8000Aoo`03001oogoo019oo`03001S6f<K00IS6`03001S6`0000US6`03001S6`0000<000ES
6`03001S6`0000QS6`8000US6`8000US6`8000QS6`8000MS6`04001S6f<K0006Ha/01000Ha/00000
16<k0P000f<k00D006<kHc]S>`0000ES>`80019oo`0017oo00<007ooOol00Woo00@007ooOol000=o
o`03001oogoo015oo`03001S6f<K00MS6`8000YS6`<000QS6`8000QS6`03001S6`0000QS6`04001S
6f<K0006Ha/200000f<K001S6`05Ha/20002Ha/00`00Ha]S6`03Ha/00`00Ha]S6`02Ha/01000Ha]S
6f<K0P000f<K0P000V<k00<006<kHc/016<k00<006<k00001V<k0P004Woo0004Ool00`00Oomoo`02
Ool01000Oomoo`000goo00<007ooOol03goo0`0026<K00@006<KHa/000IS6`<0009S6`03001S6f<K
00AS6`80009S6`03001S6f<K00=S6`8000=S6`03001S6f<K00=S6`8000AS6`03001S6f<K009S6`03
001S6f<K009S6`03001S6f<K009S6`03001S6f<K00=S6`05001S6f<KHa/00006Ha/00`00Ha/00006
Ha/30007Hc/20006Hc/01000Hc/000004Goo0004Ool00`00Oomoo`02Ool01000Oomoo`000Woo00<0
07ooOol03goo00D006<KHa]S6`0000ES6`8000AS6`03001S6f<K00D000AS6`03001S6f<K009S6`03
001S6f<K00=S6`05001S6f<KHa/00006Ha/01@00Ha]S6f<K00001f<K00@006<K000000IS6`04001S
6`000007Ha/00`00Ha/00007Ha/20007Ha/20007Hc/00`00Hc/00005Hc/01000Hc]S>f<k0P0047oo
0004Ool00`00Oomoo`02Ool01000Oomoo`000Woo0P003Woo0P001F<K00D006<KHa]S6`0000IS6`<0
009S6`03001S6f<K00ES6`04001S6`000007Ha/00`00Ha/00008Ha/00`00Ha/00008Ha/20008Ha/2
0009Ha/20007Ha/00`00Ha/00006Ha/01000Ha/0000016<k0P000f<k00D006<kHc]S>`0000ES>`03
001oogoo00=oo`<000Qoo`000goo0P001Goo0P000goo00<007ooOol037oo00<006<KHa/01V<K00<0
06<K000016<K0`000V<K00<006<K000026<K0P002F<K0P002F<K0P0026<K00<006<K00001f<K00@0
06<KHa/000IS6`800003Ha/006<K00AS6`8000=S6`04001S6f<KHa/20003Ha/20002Hc/00`00Hc]S
>`04Hc/00`00Hc/00006Hc/20005Ool00`00Oomoo`07Ool000moo`8000aoo`03001S6f<K00MS6`80
009S6`<000ES6`8000QS6`03001S6`0000QS6`04001S6f<K0006Ha/20002Ha/00`00Ha]S6`03Ha/2
0003Ha/00`00Ha]S6`03Ha/00`00Ha]S6`02Ha/00`00Ha]S6`02Ha/00`00Ha]S6`02Ha/01@00Ha]S
6f<K00001V<K00<006<K00001V<K0`001f<k0P001V<k00<006<k00001Goo00<007ooOol01goo000?
Ool2000:Ool30008Ha/00`00Ha/000020006Ha/20002Ha/00`00Ha]S6`03Ha/20003Ha/00`00Ha]S
6`03Ha/20004Ha/00`00Ha]S6`02Ha/00`00Ha]S6`03Ha/01@00Ha]S6f<K00001V<K00D006<KHa]S
6`0000IS6`04001S6`000006Ha/00`00Ha/00007Ha/20007Ha/20007Hc/00`00Hc/00004Hc/20003
Hc/00`00Oomoo`02Ool00`00Oomoo`07Ool0009oo`<000=oo`<000Aoo`03001oogoo00Qoo`05001S
6f<KHa/00005Ha/40002Ha/00`00Ha]S6`02Ha/20005Ha/01@00Ha]S6f<K00001V<K00D006<KHa]S
6`0000MS6`04001S6`000007Ha/00`00Ha/00008Ha/00`00Ha/00007Ha/20008Ha/20007Ha/00`00
Ha/00006Ha/01000Ha/0000016<k0P000f<k00@006<kHc/000ES>`8000=oo`03001oogoo00Moo`00
0goo00<007ooOol01Goo00<007ooOol00P0027oo0P001F<K00@006<KHa]S6`<000ES6`04001S6`00
0008Ha/00`00Ha/00008Ha/00`00Ha/00008Ha/20009Ha/20009Ha/20007Ha/00`00Ha/00006Ha/2
00000f<K001S6`04Ha/20003Ha/01000Ha]S6f<K0P000f<K0P000V<k00<006<kHc/00f<k00<006<k
00001V<k0P000Woo0P002Goo0003Ool00`00Oomoo`05Ool01000Oomoo`001goo0P0026<K0`0026<K
0P002V<K0P002F<K0P0026<K00@006<KHa/000IS6`80009S6`03001S6f<K00AS6`800003Ha/006<K
00AS6`8000=S6`03001S6f<K009S6`03001S6f<K009S6`05001S6f<KHa/00006Ha/00`00Ha/00006
Ha/00`00Hc/00006Hc/20006Hc/00`00Ool0000=Ool000=oo`03001oogoo009oo`<000=oo`8000Eo
o`03001S6f<K00ES6`D000QS6`04001S6f<K0008Ha/01000Ha]S6`001V<K0P000V<K00<006<KHa/0
0f<K0P0016<K00<006<KHa/00V<K00<006<KHa/00f<K00<006<KHa/00V<K00<006<KHa/00V<K00D0
06<KHa]S6`0000IS6`04001S6`000006Ha/00`00Ha/00007Ha/20007Ha/20006Hc/00`00Hc/00005
Hc/01000Oomoo`003Goo0003Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool30004
Ha/30002Ha/01000Hc]S>`001F<K0P0016<K00<006<KHa/00f<K0P0016<K00<006<KHa/00V<K00<0
06<KHa/00f<K00D006<KHa]S6`0000MS6`04001S6`000007Ha/01000Ha/000001V<K00<006<K0000
1f<K0P0026<K0P001f<K00<006<K00001V<K00@006<KHa/000=S>`8000=S>`05001S>f<kHc/00004
Ool00`00Oomoo`0:Ool0009oo`8000Aoo`@0009oo`80009oo`04001S>f<kHc/40003Ha/20004Hc/0
1@00Ha]S6f<K00001f<K00D006<KHa]S6`0000MS6`04001S6`000007Ha/00`00Ha/00008Ha/20009
Ha/20008Ha/20007Ha/00`00Ha/00006Ha/200000f<K001S6`04Ha/20003Ha/01000Ha]S6f<K0P00
16<K00<006<k00001V<k00<006<k00001Goo00<007ooOol02Woo000>Ool01000Ool000000V<k1@00
0f<K00<006<kHc/01F<k00<006<K00002F<K00<006<K000026<K0`0026<K0P0026<K00@006<KHa/0
00IS6`800003Ha/006<K00ES6`80009S6`03001S6f<K00=S6`05001S6f<KHa/00004Ha/00`00Ha]S
6`02Ha/01@00Ha]S6f<K00001V<K00<006<K00001V<K0P001f<k0P001Woo00<007ooOol02Woo000=
Ool3000016<k000000001F<k00<006<K000026<k0`002F<K0`001f<K00D006<KHa]S6`0000IS6`04
001S6f<K0005Ha/20004Ha/00`00Ha]S6`02Ha/00`00Ha]S6`02Ha/00`00Ha]S6`02Ha/00`00Ha]S
6`03Ha/01@00Ha]S6f<K00001F<K00@006<K000000IS6`03001S6`0000MS6`8000IS6`03001S6`00
00ES>`8000Qoo`03001oogoo00Yoo`003Goo100026<k0`001f<k00D006<kHc]S>`0000IS6`8000=S
6`03001S6f<K00=S6`03001S6f<K00=S6`04001S6f<KHa/20004Ha/01@00Ha]S6f<K00001f<K00@0
06<K000000IS6`04001S6`000007Ha/00`00Ha/00006Ha/20008Ha/20006Ha/200000f<K001S6`03
Ha/20003Ha/01@00Hc]S>f<k00002Woo00<007ooOol02Woo00000goo000000020003Ool20003Ool2
0009Hc/01@00Hc]S>f<k00001F<k00<006<kHc/00f<k00<006<KHa/00V<K00<006<KHa/016<K00D0
06<KHa]S6`0000MS6`03001S6`0000MS6`03001S6`0000QS6`8000QS6`8000US6`8000IS6`04001S
6f<K0006Ha/01000Ha]S6`0016<K00<006<KHa/00V<K00@006<KHa/000ES6`04001S>`00000;Ool0
0`00Oomoo`0:Ool0009oo`03001oogoo009oo`04001oogoo0004Ool00`00Hc]S>`04Hc/20005Hc/0
1@00Hc]S>f<k00001f<k00@006<K000000QS6`03001S6`0000QS6`8000QS6`8000QS6`04001S6f<K
0006Ha/01000Ha]S6`001V<K0P000V<K00@006<KHa]S6`8000AS6`04001S6f<KHa/20004Ha/01000
Ha/000001F<K00<006<K00001V<K0P003Goo00<007ooOol02Woo0003Ool01@00Oomoogoo00000Woo
00<007ooOol00goo0P000f<k00<006<kHc/01V<k00<006<k000026<k0`002F<K0P0026<K00@006<K
Ha/000IS6`04001S6f<K0005Ha/20004Ha/01000Ha]S6f<K0P0016<K00<006<KHa/00V<K00<006<K
Ha/00f<K00<006<K00001f<K00<006<K00001V<K0P001f<K0P001V<K00@007ooOol000eoo`03001o
ogoo00Uoo`0017oo00@007ooOol0009oo`03001oogoo00Aoo`800003Hc/006<k00QS>`8000QS>`05
001S>f<kHc/00006Ha/20002Ha/00`00Ha]S6`03Ha/20004Ha/01000Ha]S6f<K0P0016<K00D006<K
Ha]S6`0000MS6`03001S6`0000MS6`04001S6`000006Ha/20008Ha/20006Ha/00`00Ha/00006Ha/0
1000Ha]S6`000f<K0P0017oo00<007ooOol02Woo00<007ooOol02Goo00001Goo001oogoo00000Woo
00@007ooOol000Moo`<000QS>`04001S>f<k0005Hc/20005Hc/00`00Ha]S6`02Ha/00`00Ha]S6`03
Ha/01@00Ha]S6f<K00001f<K00<006<K00001f<K00<006<K000026<K0P0026<K0P001f<K00<006<K
00001V<K0P0000=S6`00Ha/016<K00D006<KHa]S6`0000=S6`8000AS6`03001S6`0000Ioo`03001o
ogoo00Yoo`03001oogoo00Uoo`000Woo0P0017oo0P002Woo0P001F<k0P0016<k00D006<kHc]S>`00
00QS>`04001S6`000007Ha/00`00Ha/00008Ha/20008Ha/20008Ha/00`00Ha/00006Ha/200000f<K
001S6`04Ha/20003Ha/00`00Ha]S6`02Ha/00`00Ha]S6`02Ha/01@00Ha]S6f<K00001F<K00<006<K
00001V<K0P0027oo00<007ooOol02Goo00<007ooOol02Goo000EOol20003Hc/00`00Hc]S>`05Hc/0
0`00Hc/00009Hc/20009Ha/20008Ha/01000Ha]S6`001V<K00@006<KHa/000ES6`8000=S6`03001S
6f<K009S6`03001S6f<K009S6`05001S6f<KHa/00006Ha/01000Ha/000001V<K00<006<K00001V<K
0P001V<K00<007ooOol02Goo00<007ooOol027oo00<007ooOol02Goo000FOol200000f<k001S>`07
Hc/30008Hc/01000Hc]S>`001V<K0P000V<K00<006<KHa/00f<K0P0016<K00@006<KHa]S6`8000AS
6`05001S6f<KHa/00006Ha/01000Ha/000001V<K00<006<K00001f<K0P0026<K0P001V<K00<006<K
00001F<K00<007ooOol02goo00<007ooOol01goo00<007ooOol02Goo000GOol30007Hc/01@00Hc]S
>f<k00001F<k0P0016<k00<006<KHa/00V<K00<006<KHa/00f<K00D006<KHa]S6`0000MS6`03001S
6`0000MS6`03001S6`0000MS6`8000QS6`<000IS6`03001S6`0000IS6`80009S6`04001S6f<KHa/2
0003Ha/01@00Ha]S6f<K00003Woo00<007ooOol01goo00<007ooOol02Goo000IOol20005Hc/00`00
Hc]S>`03Hc/01@00Hc]S>f<k00001f<k00@006<K000000MS6`03001S6`0000QS6`8000QS6`<000IS
6`04001S6f<K0006Ha/01@00Ha]S6f<K00000f<K0P000f<K00<006<KHa/00V<K00<006<KHa/00f<K
00<006<K00001V<K00<006<K000047oo00<007ooOol01goo00<007ooOol027oo000JOol20003Hc/0
0`00Hc]S>`05Hc/00`00Hc/00008Hc/20009Ha/30006Ha/20002Ha/00`00Ha]S6`04Ha/01@00Ha]S
6f<K00000f<K0P0016<K00@006<KHa]S6`8000ES6`03001S6`0000IS6`04001S6`000006Ha/20007
Ha/2000BOol00`00Oomoo`06Ool00`00Oomoo`08Ool001]oo`800003Hc/006<k00MS>`8000QS>`04
001S>f<k0006Ha/20003Ha/00`00Ha]S6`02Ha/00`00Ha]S6`03Ha/01000Ha]S6f<K0P001F<K00<0
06<K00001f<K00<006<K00001f<K0P001f<K0P001V<K0P0000=S6`00Ha/016<K0P0057oo00<007oo
Ool01Woo00<007ooOol027oo000LOol30006Hc/20002Hc/00`00Hc]S>`03Hc/20004Hc/00`00Ha]S
6`02Ha/00`00Ha]S6`04Ha/01000Ha/000001f<K00<006<K00001f<K0P0026<K0P001V<K0P0000=S
6`00Ha/01F<K00@006<KHa/000AS6`03001S6f<K009S6`05001S6f<KHa/0000GOol00`00Oomoo`05
Ool00`00Oomoo`08Ool001ioo`8000AS>`03001S>f<k00=S>`05001S>f<kHc/00007Hc/01000Ha/0
00001f<K0P002F<K0P001V<K0P0000=S6`00Ha/01F<K0P000V<K00<006<KHa/00V<K00<006<KHa/0
0V<K00@006<KHa]S6`8000AS6`04001S6`000005Ha/01000Ha/000006Goo00<007ooOol017oo00<0
07ooOol027oo000OOol200000f<k00000007Hc/00`00Hc/00008Hc/30007Ha/01000Ha]S6`001V<K
0P000V<K00<006<KHa/00V<K00<006<KHa/00V<K00<006<KHa/00V<K00<006<KHa/00f<K00@006<K
000000IS6`03001S6`0000IS6`8000MS6`8001aoo`03001oogoo00=oo`03001oogoo00Qoo`0087oo
0`0026<k0`001f<k00D006<kHc]S>`0000ES6`03001S6f<K009S6`03001S6f<K009S6`03001S6f<K
00=S6`04001S6`000006Ha/01000Ha/000001V<K0P0026<K0P001V<K00<006<K00001V<K00<007oo
Ool077oo00<007ooOol00goo00<007ooOol027oo000QOol30006Hc/01000Hc]S>f<k0P000f<k0P00
1F<k00D006<KHa]S6`0000IS6`04001S6`000006Ha/20008Ha/30006Ha/00`00Ha/00006Ha/20002
Ha/01000Ha]S6f<K0P000f<K00@006<KHa]S6`80021oo`03001oogoo00=oo`03001oogoo00Moo`00
8goo0P000f<k0P001V<k00<006<k000026<k00<006<K00001f<K0P001f<K00@006<KHa/000IS6`05
001S6f<KHa/00003Ha/20003Ha/00`00Ha]S6`02Ha/00`00Ha]S6`03Ha/00`00Ha/00006Ha/00`00
Ha/0000SOol00`00Oomoo`02Ool00`00Oomoo`07Ool002Aoo`800003Hc/006<k00MS>`8000US>`<0
00IS6`04001S6f<K0005Ha/00`00Ha]S6`02Ha/01000Ha]S6f<K0P001F<K00<006<K00001V<K00@0
06<K000000IS6`8000MS6`8002Eoo`05001oogooOol00009Ool002Eoo`8000QS>`04001S>f<k0006
Hc/20003Hc/01000Ha]S6f<K0P0016<K00D006<KHa]S6`0000IS6`03001S6`0000MS6`8000MS6`<0
00IS6`03001S6`0000IS6`03001oogoo02Eoo`05001oogooOol00009Ool002Ioo`8000ES>`8000AS
>`03001S>f<k009S>`03001S>f<k00AS>`03001S6`0000MS6`03001S6`0000MS6`<000IS6`03001S
6`0000IS6`05001S6f<KHa/00003Ha/20003Ha/01000Ha]S6f<K0P00:Goo00@007ooOol000Uoo`00
:7oo00D006<kHc]S>`0000MS>`04001S>`000007Hc/20008Ha/30006Ha/01@00Ha]S6f<K00000f<K
0P000f<K00@006<KHa]S6`8000ES6`03001S6`0000IS6`03001S6`0002aoo`03001oo`0000Uoo`00
:Goo00<006<k000026<k0`001V<k0P0000=S>`00Ha/01F<K0P000f<K00@006<KHa]S6`8000ES6`03
001S6`0000IS6`03001S6`0000MS6`8000MS6`8002eoo`03001oo`0000Uoo`00:Woo0P001f<k00D0
06<kHc]S>`0000AS>`03001S>f<k009S>`03001S6f<K009S6`03001S6f<K00AS6`03001S6`0000MS
6`8000MS6`8000IS6`80009S6`03001S6f<K009S6`80031oo`03001oo`0000Qoo`00:goo0`000f<k
0P001F<k00@006<k000000IS>`04001S6`000007Ha/20006Ha/200000f<K001S6`05Ha/01000Ha]S
6`0016<K00<006<KHa/00f<K00@006<K000003=oo`8000Qoo`00;Goo0P0000=S>`00Hc/01V<k0P00
26<k0`001V<K0P000V<K00<006<KHa/00V<K00<006<KHa/00V<K00@006<KHa]S6`8000AS6`04001S
6`000006Ha/2000dOol30008Ool002ioo`<000IS>`04001S>f<k0006Hc/01@00Hc]S>f<k000016<K
00<006<KHa/00f<K00@006<K000000IS6`03001S6`0000IS6`<000IS6`03001oogoo031oo`@000]o
o`00;goo00<007oo00000f<k0P0016<k00@006<kHc]S>`8000ES>`04001S6`000006Ha/30007Ha/2
0006Ha/01@00Ha]S6f<K00000f<K0P00<7oo0`003goo000`Ool01@00Ool006<k00001f<k00<006<k
00001f<k0`001V<K00D006<KHa]S6`0000AS6`80009S6`04001S6f<KHa/20005Ha/00`00Ha/0000^
Ool4000BOol0035oo`<000QS>`<000IS>`05001S>f<kHc/00003Ha/20005Ha/01000Ha/000001F<K
00<006<K00001f<K0P00:goo10005Woo000cOol20006Hc/01@00Hc]S>f<k00000f<k0P001F<k00<0
06<K00001f<K0P001f<K0`001V<K00<007ooOol09goo10006Woo000dOol20003Hc/20005Hc/00`00
Hc/00007Hc/30006Ha/00`00Ha/00006Ha/01@00Ha]S6f<K00000f<K0P009Woo10007Woo000eOol2
00000f<k001S>`06Hc/30006Hc/01@00Hc]S>f<k00000f<K0P000f<K00@006<KHa]S6`8000ES6`03
001S6`0002Eoo`<0029oo`00=Woo0`001V<k00D006<kHc]S>`0000=S>`8000ES>`03001S6`0000IS
6`03001S6`0000MS6`80029oo`@002Eoo`00>7oo0P000f<k0P001F<k00<006<k00001f<k0P001f<K
0`001V<K00<007ooOol07Woo1000:Goo000iOol200000f<k001S>`06Hc/20006Hc/20002Hc/00`00
Ha]S6`02Ha/20003Ha/01000Ha]S6f<K0P007Goo1000;Goo000jOol30006Hc/01000Hc]S>`0016<k
00<006<kHc/00f<k00@006<K000000IS6`03001S6`0001]oo`@0035oo`00>goo0`000f<k0P0016<k
00@006<k000000IS>`8000QS6`8001Uoo`<003Eoo`00?Goo0P0000=S>`00Hc/01F<k0`001V<k00@0
06<kHc/000AS6`<001Moo`@003Qoo`00?Woo0P001V<k00D006<kHc]S>`0000=S>`8000AS>`04001S
6`00000FOol4000lOol003moo`8000=S>`8000ES>`03001S>`0000IS>`8001Aoo`@0041oo`00@7oo
0P0000=S>`00Hc/01V<k0`001F<k00<007ooOol047oo1000A7oo0012Ool20006Hc/01@00Hc]S>f<k
00000f<k00<007ooOol03Woo0`00B7oo0013Ool20003Hc/20005Hc/00`00Hc/0000=Ool4001;Ool0
04Aoo`800003Hc/006<k00IS>`8000Yoo`@004moo`00AGoo0`001F<k0P0027oo1000Dgoo0017Ool3
00000f<k00000006Ool4001GOol004Qoo`03001oo`0000Eoo`<005]oo`00BGoo0P0000=oo`000000
0P00GWoo001:Ool2001ROol00:ioo`00[Woo002^Ool00001\
\>"],
  ImageRangeCache->{{{0, 173.375}, {140.312, 0}} -> {-0.0863396, -0.0234745, 
  0.00628646, 0.00628646}}]
}, Open  ]],

Cell[TextData[{
  "In the following we show Vega, computed symbolically by the computer. ",
  StyleBox["It is nowhere positive.",
    FontSlant->"Italic"]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[UpAndOutPutVega[0, S, 50, 50, 0.25,0.1, 0.15, t],
{S, 30, 50}, {t, 0.001, 1}, PlotRange -> All];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.7672 .09664 0 -6.16225 ]
[.7672 .09664 18 2.83775 ]
[.82794 .18966 0 -6.07272 ]
[.82794 .18966 18 2.92728 ]
[.88358 .27486 0 -5.99233 ]
[.88358 .27486 18 3.00767 ]
[.93473 .3532 0 -5.91977 ]
[.93473 .3532 18 3.08023 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.01849 .30037 -12 -2.77211 ]
[.01849 .30037 0 6.22789 ]
[.0089 .36048 -12 -2.82307 ]
[.0089 .36048 0 6.17693 ]
[-0.00107 .42296 -12 -2.87623 ]
[-0.00107 .42296 0 6.12377 ]
[-0.01144 .48795 -6 -2.93176 ]
[-0.01144 .48795 0 6.06824 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.75489 .10119 m
.74873 .10346 L
s
[(0.2)] .7672 .09664 -1 .36939 Mshowa
.81555 .19399 m
.80936 .19615 L
s
[(0.4)] .82794 .18966 -1 .34949 Mshowa
.87113 .27899 m
.8649 .28106 L
s
[(0.6)] .88358 .27486 -1 .33163 Mshowa
.92222 .35714 m
.91596 .35912 L
s
[(0.8)] .93473 .3532 -1 .3155 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(1)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.70561 .0258 m
.70193 .02722 L
s
.72242 .05152 m
.71874 .05292 L
s
.73884 .07664 m
.73515 .07802 L
s
.77057 .12518 m
.76687 .12652 L
s
.7859 .14863 m
.7822 .14996 L
s
.80089 .17156 m
.79718 .17287 L
s
.8299 .21593 m
.82618 .21721 L
s
.84394 .2374 m
.84021 .23867 L
s
.85767 .25842 m
.85394 .25967 L
s
.8843 .29914 m
.88056 .30036 L
s
.8972 .31887 m
.89345 .32008 L
s
.90983 .3382 m
.90608 .3394 L
s
.93435 .37571 m
.93059 .37688 L
s
.94625 .3939 m
.94249 .39506 L
s
.95791 .41174 m
.95414 .41289 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03074 .29567 m
.03686 .29332 L
s
[(-3)] .01849 .30037 1 -0.38397 Mshowa
.02119 .3559 m
.02734 .35361 L
s
[(-2)] .0089 .36048 1 -0.37265 Mshowa
.01127 .41851 m
.01744 .41628 L
s
[(-1)] -0.00107 .42296 1 -0.36084 Mshowa
.00095 .48363 m
.00715 .48147 L
s
[(0)] -0.01144 .48795 1 -0.3485 Mshowa
.125 Mabswid
.02886 .30753 m
.03254 .30613 L
s
.02696 .31949 m
.03064 .31809 L
s
.02505 .33153 m
.02874 .33014 L
s
.02313 .34367 m
.02682 .34229 L
s
.01924 .36823 m
.02293 .36686 L
s
.01727 .38065 m
.02096 .37929 L
s
.01529 .39317 m
.01898 .39182 L
s
.01329 .40579 m
.01699 .40445 L
s
.00924 .43133 m
.01294 .43 L
s
.00719 .44425 m
.0109 .44293 L
s
.00513 .45727 m
.00884 .45596 L
s
.00305 .4704 m
.00676 .46909 L
s
.0326 .28389 m
.03628 .28248 L
s
.03445 .27221 m
.03812 .27078 L
s
.03629 .26061 m
.03996 .25918 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.734 .814 .914 r
.37526 .77217 .39871 .78725 .43767 .76607 .41465 .7523 Metetra
.763 .832 .905 r
.41465 .7523 .43767 .76607 .47702 .74085 .45448 .72858 Metetra
.792 .85 .894 r
.45448 .72858 .47702 .74085 .51662 .71137 .49463 .70065 Metetra
.818 .868 .883 r
.49463 .70065 .51662 .71137 .55634 .67789 .53499 .6686 Metetra
.839 .884 .873 r
.53499 .6686 .55634 .67789 .59605 .64126 .5754 .63309 Metetra
.853 .898 .868 r
.5754 .63309 .59605 .64126 .63571 .60298 .6158 .59545 Metetra
.855 .911 .87 r
.6158 .59545 .63571 .60298 .67535 .56519 .65618 .55773 Metetra
.842 .922 .886 r
.65618 .55773 .67535 .56519 .71514 .53072 .69667 .52269 Metetra
.799 .926 .92 r
.69667 .52269 .71514 .53072 .75545 .50302 .73761 .49375 Metetra
.693 .904 .971 r
.73761 .49375 .75545 .50302 .79688 .48619 .77956 .47507 Metetra
.442 .789 .998 r
.77956 .47507 .79688 .48619 .84033 .48516 .82338 .47164 Metetra
0 .465 .859 r
.82338 .47164 .84033 .48516 .88712 .50601 .87036 .48959 Metetra
0 .059 .554 r
.87036 .48959 .88712 .50601 .93915 .55659 .92236 .53684 Metetra
.672 .209 0 r
.92236 .53684 .93915 .55659 .99922 .6478 .98216 .62429 Metetra
.721 .808 .919 r
.35115 .75614 .37526 .77217 .41465 .7523 .39094 .73756 Metetra
.749 .827 .912 r
.39094 .73756 .41465 .7523 .45448 .72858 .43123 .71537 Metetra
.778 .846 .902 r
.43123 .71537 .45448 .72858 .49463 .70065 .47193 .68909 Metetra
.806 .865 .892 r
.47193 .68909 .49463 .70065 .53499 .6686 .5129 .65862 Metetra
.829 .883 .882 r
.5129 .65862 .53499 .6686 .5754 .63309 .55402 .62442 Metetra
.845 .898 .875 r
.55402 .62442 .5754 .63309 .6158 .59545 .59516 .58764 Metetra
.851 .912 .875 r
.59516 .58764 .6158 .59545 .65618 .55773 .6363 .55017 Metetra
.841 .924 .887 r
.6363 .55017 .65618 .55773 .69667 .52269 .67754 .51467 Metetra
.803 .929 .917 r
.67754 .51467 .69667 .52269 .73761 .49375 .71915 .48456 Metetra
.709 .912 .965 r
.71915 .48456 .73761 .49375 .77956 .47507 .76166 .46402 Metetra
.482 .814 1 r
.76166 .46402 .77956 .47507 .82338 .47164 .80588 .45809 Metetra
.049 .518 .889 r
.80588 .45809 .82338 .47164 .87036 .48959 .85308 .47298 Metetra
0 .105 .594 r
.85308 .47298 .87036 .48959 .92236 .53684 .90507 .51666 Metetra
.65 .183 0 r
.90507 .51666 .92236 .53684 .98216 .62429 .96458 .60007 Metetra
.707 .802 .924 r
.32635 .73911 .35115 .75614 .39094 .73756 .36652 .72175 Metetra
.733 .82 .918 r
.36652 .72175 .39094 .73756 .43123 .71537 .40726 .7011 Metetra
.763 .84 .91 r
.40726 .7011 .43123 .71537 .47193 .68909 .44848 .67655 Metetra
.792 .861 .901 r
.44848 .67655 .47193 .68909 .5129 .65862 .49006 .64782 Metetra
.817 .88 .891 r
.49006 .64782 .5129 .65862 .55402 .62442 .53186 .61514 Metetra
.836 .898 .883 r
.53186 .61514 .55402 .62442 .59516 .58764 .57376 .57945 Metetra
.845 .913 .881 r
.57376 .57945 .59516 .58764 .6363 .55017 .61569 .54244 Metetra
.838 .925 .89 r
.61569 .54244 .6363 .55017 .67754 .51467 .6577 .50666 Metetra
.807 .932 .915 r
.6577 .50666 .67754 .51467 .71915 .48456 .70003 .47546 Metetra
.723 .92 .96 r
.70003 .47546 .71915 .48456 .76166 .46402 .74313 .45306 Metetra
.522 .838 .999 r
.74313 .45306 .76166 .46402 .80588 .45809 .78781 .44454 Metetra
.115 .571 .917 r
.78781 .44454 .80588 .45809 .85308 .47298 .83524 .45619 Metetra
0 .156 .637 r
.83524 .45619 .85308 .47298 .90507 .51666 .88723 .49606 Metetra
.624 .153 0 r
.88723 .49606 .90507 .51666 .96458 .60007 .94646 .5751 Metetra
.693 .795 .928 r
.30085 .72103 .32635 .73911 .36652 .72175 .34139 .7048 Metetra
.718 .812 .924 r
.34139 .7048 .36652 .72175 .40726 .7011 .38255 .68566 Metetra
.746 .833 .918 r
.38255 .68566 .40726 .7011 .44848 .67655 .42427 .66289 Metetra
.776 .855 .91 r
.42427 .66289 .44848 .67655 .49006 .64782 .46643 .63603 Metetra
.803 .876 .9 r
.46643 .63603 .49006 .64782 .53186 .61514 .50891 .6051 Metetra
.825 .896 .892 r
.50891 .6051 .53186 .61514 .57376 .57945 .55156 .57075 Metetra
.837 .913 .888 r
.55156 .57075 .57376 .57945 .61569 .54244 .59429 .53447 Metetra
.835 .926 .893 r
.59429 .53447 .61569 .54244 .6577 .50666 .63711 .49861 Metetra
.809 .934 .914 r
.63711 .49861 .6577 .50666 .70003 .47546 .68019 .46646 Metetra
.736 .926 .954 r
.68019 .46646 .70003 .47546 .74313 .45306 .72395 .44221 Metetra
.559 .859 .997 r
.72395 .44221 .74313 .45306 .78781 .44454 .76911 .43102 Metetra
.184 .624 .943 r
.76911 .43102 .78781 .44454 .83524 .45619 .81683 .43925 Metetra
0 .214 .683 r
.81683 .43925 .83524 .45619 .88723 .49606 .86884 .47502 Metetra
.594 .118 0 r
.86884 .47502 .88723 .49606 .94646 .5751 .92776 .54934 Metetra
.681 .788 .931 r
.27463 .70185 .30085 .72103 .34139 .7048 .31552 .68661 Metetra
.702 .804 .928 r
.31552 .68661 .34139 .7048 .38255 .68566 .35708 .66889 Metetra
.728 .824 .924 r
.35708 .66889 .38255 .68566 .42427 .66289 .39926 .64792 Metetra
.757 .847 .918 r
.39926 .64792 .42427 .66289 .46643 .63603 .44198 .62306 Metetra
.787 .87 .91 r
.44198 .62306 .46643 .63603 .50891 .6051 .48511 .5941 Metetra
.811 .892 .901 r
.48511 .5941 .50891 .6051 .55156 .57075 .52851 .56139 Metetra
.827 .911 .895 r
.52851 .56139 .55156 .57075 .59429 .53447 .57206 .52614 Metetra
.829 .927 .897 r
.57206 .52614 .59429 .53447 .63711 .49861 .61571 .49048 Metetra
.809 .936 .913 r
.61571 .49048 .63711 .49861 .68019 .46646 .6596 .45756 Metetra
.748 .932 .948 r
.6596 .45756 .68019 .46646 .72395 .44221 .70406 .43152 Metetra
.594 .878 .993 r
.70406 .43152 .72395 .44221 .76911 .43102 .74976 .41758 Metetra
.254 .676 .964 r
.74976 .41758 .76911 .43102 .81683 .43925 .79779 .42219 Metetra
0 .278 .733 r
.79779 .42219 .81683 .43925 .86884 .47502 .84985 .45357 Metetra
.557 .078 0 r
.84985 .45357 .86884 .47502 .92776 .54934 .90848 .52276 Metetra
.669 .781 .933 r
.24766 .68155 .27463 .70185 .31552 .68661 .2889 .66713 Metetra
.686 .795 .932 r
.2889 .66713 .31552 .68661 .35708 .66889 .33084 .65069 Metetra
.709 .813 .93 r
.33084 .65069 .35708 .66889 .39926 .64792 .37346 .63146 Metetra
.737 .836 .926 r
.37346 .63146 .39926 .64792 .44198 .62306 .4167 .60869 Metetra
.767 .862 .919 r
.4167 .60869 .44198 .62306 .48511 .5941 .46045 .5819 Metetra
.795 .886 .912 r
.46045 .5819 .48511 .5941 .52851 .56139 .50458 .55115 Metetra
.814 .908 .905 r
.50458 .55115 .52851 .56139 .57206 .52614 .54894 .51729 Metetra
.822 .926 .903 r
.54894 .51729 .57206 .52614 .61571 .49048 .59346 .48217 Metetra
.808 .938 .914 r
.59346 .48217 .61571 .49048 .6596 .45756 .63819 .44872 Metetra
.757 .937 .944 r
.63819 .44872 .6596 .45756 .70406 .43152 .6834 .42101 Metetra
.626 .896 .987 r
.6834 .42101 .70406 .43152 .74976 .41758 .7297 .40427 Metetra
.325 .726 .98 r
.7297 .40427 .74976 .41758 .79779 .42219 .7781 .40506 Metetra
0 .349 .784 r
.7781 .40506 .79779 .42219 .84985 .45357 .83023 .4317 Metetra
0 0 .486 r
.83023 .4317 .84985 .45357 .90848 .52276 .88857 .49533 Metetra
.66 .774 .934 r
.21993 .66014 .24766 .68155 .2889 .66713 .26151 .6463 Metetra
.672 .785 .934 r
.26151 .6463 .2889 .66713 .33084 .65069 .30382 .63094 Metetra
.691 .802 .934 r
.30382 .63094 .33084 .65069 .37346 .63146 .34684 .61332 Metetra
.716 .824 .932 r
.34684 .61332 .37346 .63146 .4167 .60869 .39056 .59263 Metetra
.745 .85 .928 r
.39056 .59263 .4167 .60869 .46045 .5819 .4349 .56819 Metetra
.774 .878 .922 r
.4349 .56819 .46045 .5819 .50458 .55115 .47972 .53973 Metetra
.798 .903 .915 r
.47972 .53973 .50458 .55115 .54894 .51729 .52489 .5077 Metetra
.811 .924 .911 r
.52489 .5077 .54894 .51729 .59346 .48217 .57028 .47355 Metetra
.804 .938 .917 r
.57028 .47355 .59346 .48217 .63819 .44872 .6159 .43992 Metetra
.764 .941 .94 r
.6159 .43992 .63819 .44872 .6834 .42101 .66193 .41072 Metetra
.655 .91 .98 r
.66193 .41072 .6834 .42101 .7297 .40427 .70889 .39117 Metetra
.395 .774 .992 r
.70889 .39117 .7297 .40427 .7781 .40506 .75772 .38793 Metetra
0 .427 .836 r
.75772 .38793 .7781 .40506 .83023 .4317 .80995 .40945 Metetra
0 .027 .54 r
.80995 .40945 .83023 .4317 .88857 .49533 .868 .46699 Metetra
.653 .769 .934 r
.19139 .63765 .21993 .66014 .26151 .6463 .23333 .62413 Metetra
.661 .777 .935 r
.23333 .62413 .26151 .6463 .30382 .63094 .27599 .60956 Metetra
.675 .79 .936 r
.27599 .60956 .30382 .63094 .34684 .61332 .31941 .59332 Metetra
.695 .81 .936 r
.31941 .59332 .34684 .61332 .39056 .59263 .36357 .57461 Metetra
.721 .835 .935 r
.36357 .57461 .39056 .59263 .4349 .56819 .40844 .55261 Metetra
.751 .864 .932 r
.40844 .55261 .4349 .56819 .47972 .53973 .45391 .52675 Metetra
.778 .894 .926 r
.45391 .52675 .47972 .53973 .52489 .5077 .49986 .49702 Metetra
.796 .919 .921 r
.49986 .49702 .52489 .5077 .57028 .47355 .54613 .46439 Metetra
.797 .937 .922 r
.54613 .46439 .57028 .47355 .6159 .43992 .59267 .43106 Metetra
.768 .944 .937 r
.59267 .43106 .6159 .43992 .66193 .41072 .63958 .40067 Metetra
.68 .923 .972 r
.63958 .40067 .66193 .41072 .70889 .39117 .68728 .37837 Metetra
.462 .817 .997 r
.68728 .37837 .70889 .39117 .75772 .38793 .73659 .37088 Metetra
.043 .511 .886 r
.73659 .37088 .75772 .38793 .80995 .40945 .78897 .38686 Metetra
0 .096 .601 r
.78897 .38686 .80995 .40945 .868 .46699 .84674 .4377 Metetra
.649 .766 .934 r
.16202 .61416 .19139 .63765 .23333 .62413 .20432 .60068 Metetra
.653 .77 .935 r
.20432 .60068 .23333 .62413 .27599 .60956 .24735 .58655 Metetra
.661 .779 .936 r
.24735 .58655 .27599 .60956 .31941 .59332 .29114 .57132 Metetra
.675 .794 .938 r
.29114 .57132 .31941 .59332 .36357 .57461 .33572 .55433 Metetra
.696 .817 .94 r
.33572 .55433 .36357 .57461 .40844 .55261 .38107 .53471 Metetra
.723 .846 .94 r
.38107 .53471 .40844 .55261 .45391 .52675 .42713 .51167 Metetra
.752 .879 .937 r
.42713 .51167 .45391 .52675 .49986 .49702 .47381 .48475 Metetra
.775 .909 .932 r
.47381 .48475 .49986 .49702 .54613 .46439 .52094 .4543 Metetra
.784 .933 .929 r
.52094 .4543 .54613 .46439 .59267 .43106 .56843 .42194 Metetra
.767 .945 .937 r
.56843 .42194 .59267 .43106 .63958 .40067 .61628 .39085 Metetra
.7 .933 .964 r
.61628 .39085 .63958 .40067 .68728 .37837 .66478 .36597 Metetra
.525 .855 .996 r
.66478 .36597 .68728 .37837 .73659 .37088 .71466 .35407 Metetra
.152 .598 .931 r
.71466 .35407 .73659 .37088 .78897 .38686 .76725 .364 Metetra
0 .179 .671 r
.76725 .364 .78897 .38686 .84674 .4377 .82477 .40742 Metetra
.647 .763 .934 r
.13177 .58973 .16202 .61416 .20432 .60068 .17445 .57606 Metetra
.648 .765 .935 r
.17445 .57606 .20432 .60068 .24735 .58655 .21787 .562 Metetra
.652 .77 .936 r
.21787 .562 .24735 .58655 .29114 .57132 .26204 .54732 Metetra
.659 .78 .938 r
.26204 .54732 .29114 .57132 .33572 .55433 .30701 .53156 Metetra
.673 .797 .941 r
.30701 .53156 .33572 .55433 .38107 .53471 .3528 .51403 Metetra
.694 .823 .945 r
.3528 .51403 .38107 .53471 .42713 .51167 .39939 .49383 Metetra
.721 .857 .946 r
.39939 .49383 .42713 .51167 .47381 .48475 .44672 .47014 Metetra
.748 .892 .944 r
.44672 .47014 .47381 .48475 .52094 .4543 .49467 .44267 Metetra
.765 .923 .94 r
.49467 .44267 .52094 .4543 .56843 .42194 .5431 .41219 Metetra
.761 .942 .941 r
.5431 .41219 .56843 .42194 .61628 .39085 .59193 .38118 Metetra
.715 .941 .958 r
.59193 .38118 .61628 .39085 .66478 .36597 .64134 .35411 Metetra
.581 .887 .99 r
.64134 .35411 .66478 .36597 .71466 .35407 .69188 .33769 Metetra
.267 .685 .967 r
.69188 .33769 .71466 .35407 .76725 .364 .74475 .34101 Metetra
0 .28 .75 r
.74475 .34101 .76725 .364 .82477 .40742 .80203 .37608 Metetra
.646 .763 .934 r
.10059 .56446 .13177 .58973 .17445 .57606 .14367 .55042 Metetra
.646 .763 .934 r
.14367 .55042 .17445 .57606 .21787 .562 .18749 .53611 Metetra
.647 .765 .935 r
.18749 .53611 .21787 .562 .26204 .54732 .23207 .52146 Metetra
.65 .769 .936 r
.23207 .52146 .26204 .54732 .30701 .53156 .27745 .50628 Metetra
.656 .779 .939 r
.27745 .50628 .30701 .53156 .3528 .51403 .32365 .49017 Metetra
.667 .798 .944 r
.32365 .49017 .3528 .51403 .39939 .49383 .37071 .47243 Metetra
.687 .826 .95 r
.37071 .47243 .39939 .49383 .44672 .47014 .41861 .45212 Metetra
.712 .864 .954 r
.41861 .45212 .44672 .47014 .49467 .44267 .46728 .42839 Metetra
.736 .904 .953 r
.46728 .42839 .49467 .44267 .5431 .41219 .51661 .40102 Metetra
.745 .934 .95 r
.51661 .40102 .5431 .41219 .59193 .38118 .56647 .37133 Metetra
.72 .944 .955 r
.56647 .37133 .59193 .38118 .64134 .35411 .61687 .3429 Metetra
.628 .913 .981 r
.61687 .3429 .64134 .35411 .69188 .33769 .66818 .32205 Metetra
.385 .769 .99 r
.66818 .32205 .69188 .33769 .74475 .34101 .72141 .3181 Metetra
0 .403 .835 r
.72141 .3181 .74475 .34101 .80203 .37608 .77849 .34365 Metetra
.646 .762 .934 r
.06842 .53837 .10059 .56446 .14367 .55042 .11191 .52389 Metetra
.646 .762 .934 r
.11191 .52389 .14367 .55042 .18749 .53611 .15616 .50915 Metetra
.646 .763 .934 r
.15616 .50915 .18749 .53611 .23207 .52146 .20118 .49416 Metetra
.646 .764 .934 r
.20118 .49416 .23207 .52146 .27745 .50628 .247 .47887 Metetra
.647 .767 .936 r
.247 .47887 .27745 .50628 .32365 .49017 .29364 .46318 Metetra
.65 .775 .94 r
.29364 .46318 .32365 .49017 .37071 .47243 .34114 .44685 Metetra
.658 .792 .946 r
.34114 .44685 .37071 .47243 .41861 .45212 .38953 .42929 Metetra
.673 .822 .955 r
.38953 .42929 .41861 .45212 .46728 .42839 .43882 .40958 Metetra
.695 .865 .963 r
.43882 .40958 .46728 .42839 .51661 .40102 .48894 .38667 Metetra
.714 .909 .963 r
.48894 .38667 .51661 .40102 .56647 .37133 .53978 .36031 Metetra
.712 .937 .96 r
.53978 .36031 .56647 .37133 .61687 .3429 .59125 .33227 Metetra
.661 .931 .971 r
.59125 .33227 .61687 .3429 .66818 .32205 .64349 .30767 Metetra
.498 .844 .996 r
.64349 .30767 .66818 .32205 .72141 .3181 .6972 .29575 Metetra
.12 .548 .918 r
.6972 .29575 .72141 .3181 .77849 .34365 .7541 .31005 Metetra
.646 .762 .934 r
.03523 .51144 .06842 .53837 .11191 .52389 .07913 .49649 Metetra
.646 .762 .934 r
.07913 .49649 .11191 .52389 .15616 .50915 .1238 .48127 Metetra
.646 .762 .934 r
.1238 .48127 .15616 .50915 .20118 .49416 .16928 .46579 Metetra
.646 .762 .934 r
.16928 .46579 .20118 .49416 .247 .47887 .21557 .45002 Metetra
.646 .763 .934 r
.21557 .45002 .247 .47887 .29364 .46318 .2627 .43397 Metetra
.646 .764 .935 r
.2627 .43397 .29364 .46318 .34114 .44685 .3107 .41762 Metetra
.646 .768 .937 r
.3107 .41762 .34114 .44685 .38953 .42929 .35958 .40092 Metetra
.647 .779 .944 r
.35958 .40092 .38953 .42929 .43882 .40958 .40939 .38368 Metetra
.653 .804 .956 r
.40939 .38368 .43882 .40958 .48894 .38667 .46014 .36532 Metetra
.664 .848 .97 r
.46014 .36532 .48894 .38667 .53978 .36031 .51183 .34469 Metetra
.676 .901 .976 r
.51183 .34469 .53978 .36031 .59125 .33227 .56437 .32083 Metetra
.667 .931 .97 r
.56437 .32083 .59125 .33227 .64349 .30767 .61769 .29523 Metetra
.592 .903 .984 r
.61769 .29523 .64349 .30767 .6972 .29575 .67207 .2751 Metetra
.337 .714 .984 r
.67207 .2751 .6972 .29575 .7541 .31005 .72883 .27522 Metetra
.646 .762 .934 r
.00095 .48363 .03523 .51144 .07913 .49649 .04527 .46819 Metetra
.646 .762 .934 r
.04527 .46819 .07913 .49649 .1238 .48127 .09038 .45247 Metetra
.646 .762 .934 r
.09038 .45247 .1238 .48127 .16928 .46579 .13631 .43646 Metetra
.646 .762 .934 r
.13631 .43646 .16928 .46579 .21557 .45002 .18308 .42016 Metetra
.646 .762 .934 r
.18308 .42016 .21557 .45002 .2627 .43397 .23072 .40356 Metetra
.646 .762 .934 r
.23072 .40356 .2627 .43397 .3107 .41762 .27924 .38665 Metetra
.646 .762 .934 r
.27924 .38665 .3107 .41762 .35958 .40092 .32868 .36942 Metetra
.646 .763 .934 r
.32868 .36942 .35958 .40092 .40939 .38368 .37905 .35186 Metetra
.646 .765 .935 r
.37905 .35186 .40939 .38368 .46014 .36532 .43039 .33397 Metetra
.645 .772 .941 r
.43039 .33397 .46014 .36532 .51183 .34469 .48273 .31573 Metetra
.643 .794 .955 r
.48273 .31573 .51183 .34469 .56437 .32083 .53609 .29714 Metetra
.632 .836 .978 r
.53609 .29714 .56437 .32083 .61769 .29523 .5905 .27817 Metetra
.602 .88 .992 r
.5905 .27817 .61769 .29523 .67207 .2751 .646 .25883 Metetra
.592 .886 .991 r
.646 .25883 .67207 .2751 .72883 .27522 .70262 .2391 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{174.375, 141.313},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002^0000SA000`40O003h00O[Woo002^Ool00:io
o`00[Woo002^Ool006]oo`<000Aoo`8003Yoo`00KWoo00@007ooOol0009oo`03001oogoo03Moo`00
KWoo00@007ooOol0009oo`03001oogoo03Moo`00Jgoo0`000goo00@007ooOol003Uoo`00Jgoo00<0
07ooOol00goo00@007ooOol003Uoo`00Jgoo10000goo0P00>Woo002^Ool00:ioo`00[Woo001dOol3
000gOol0075oo`<00003Ool007oo03Moo`00D7oo0`000Woo0`005goo0P0017oo0P00=goo001AOol0
0`00Oomoo`04Ool00`00Oomoo`0AOol30006Ool01000Ool00000=Goo001?Ool40005Ool00`00Oomo
o`0>Ool30009Ool00`00Ool0000fOol004moo`03001oo`0000=oo`<000moo`8000aoo`04001oogoo
000eOol004moo`03001oo`0000=oo`03001oogoo00aoo`<000ioo`05001oogooOol0000dOol0051o
o`8000=oo`@000Uoo`800003Ool007oo00ioo`04001oogooOol2000cOol005moo`<001=oo`03001o
ogoo009oo`03001oogoo035oo`00GGoo0P0000=oo`00Ool04goo00<007ooOol00goo00<007ooOol0
<7oo001JOol3000IOol00`00Oomoo`02Ool00`00Oomoo`0`Ool005Qoo`<001]oo`03001oogoo00=o
o`8000Eoo`8000Aoo`8000=oo`@001aoo`00=Woo0`000goo0P005goo0`007Woo00<007ooOol017oo
00<007ooOol00Woo00@007ooOol000=oo`8000Aoo`03001oogoo01aoo`00=goo00D007ooOomoo`00
009oo`03001oogoo015oo`@0021oo`03001oogoo00Aoo`03001oogoo009oo`04001oogoo000:Ool0
0`00Oomoo`0KOol003Eoo`@0009oo`04001oogoo000@Ool3000TOol00`00Oomoo`05Ool01@00Oomo
ogoo00000Woo00<007ooOol02Goo00<007ooOol06Woo000eOol00`00Ool00003Ool01000Oomoo`00
3Woo10009Goo00<007ooOol01Goo0P000Woo00@007ooOol000Qoo`04001oogoo000LOol003Eoo`03
001oo`0000=oo`04001oogoo000;Ool3000YOol00`00Oomoo`06Ool01000Oomoogoo0P002Woo0P00
7Goo000fOol20004Ool2000:Ool4000ZOol00`00Oomoo`07Ool00`00Oomoo`0[Ool004Eoo`<002io
o`03001oogoo00Qoo`03001oogoo02Yoo`00@goo1000;goo00<007ooOol027oo0P00:goo0010Ool3
000cOol00`00Oomoo`09Ool00`00Oomoo`0YOol001aoo`<000=oo`<001Uoo`800003Ool007oo03=o
o`03001oogoo00Yoo`03001oogoo02Qoo`007goo00<007ooOol00goo00<007ooOol04goo0`00>7oo
00<007ooOol02Woo0P00:Goo000OOol00`00Oomoo`03Ool00`00Oomoo`0AOol200000goo001oo`0h
Ool00`00Oomoo`0;Ool00`00Oomoo`0WOol001eoo`8000=oo`<0015oo`<003eoo`03001oogoo00ao
o`03001oogoo02Ioo`007goo00@007ooOol0015oo`800003Ool007oo03eoo`03001oogoo00aoo`03
001oogoo00Eoo`8000Aoo`8000Aoo`<0019oo`0077oo0`000goo10002goo0`00@Woo00<007ooOol0
3Goo00<007ooOol00goo00@007ooOol000=oo`8000Eoo`03001oogoo015oo`00;Woo0`00AGoo00<0
07ooOol03Woo00<007ooOol00Woo00@007ooOol000Qoo`@0019oo`00;7oo0P00B7oo00<007ooOol0
3Woo00<007ooOol00Woo00@007ooOol000Qoo`03001oo`0001=oo`00:Goo0`00Bgoo00<007ooOol0
3Woo0P000Woo00@007ooOol000Qoo`03001oo`0001=oo`009goo0`00CGoo00<007ooOol03Woo0P00
0goo0P002Woo0P004goo0005Ool30004Ool2000FOol3001@Ool00`00Oomoo`0?Ool00`00Oomoo`0R
Ool000Qoo`04001oogoo0002Ool00`00Oomoo`0AOol3001BOol00`00Oomoo`0@Ool00`00Oomoo`0Q
Ool000Qoo`04001oogoo0002Ool00`00Oomoo`0>Ool3001EOol00`00Oomoo`0@Ool2000ROol000Io
o`8000=oo`04001oogoo000>Ool3001GOol00`00Oomoo`0AOol00`00Oomoo`0POol000Qoo`04001o
ogoo0002Ool00`00Oomoo`09Ool3001JOol00`00Oomoo`0BOol00`00Oomoo`0OOol000Eoo`<000Ao
o`8000Uoo`<00003Ool007oo05Uoo`8001Aoo`80021oo`005Goo0P00G7oo1@005Goo00<007ooOol0
7Woo000BOol3001KOol600000d^O0000000EOol00`00Oomoo`04Ool20004Ool20004Ool2000;Ool0
015oo`8005]oo`D000E;W`8001Aoo`8000Aoo`04001oogoo0003Ool20003Ool01000Oomoo`002Woo
000AOol00`00Ool0001GOol500000dnO001;W`06Bil00`00Oomoo`0COol00`00Oomoo`02Ool01000
Oomoo`0027oo00@007ooOol000Yoo`004Goo0P0000=oo`00Ool0DWoo1@001DnO00<004^OBil01D^O
00<007ooOol04goo00D007ooOomoo`00009oo`03001oogoo00Ioo`<000]oo`004Goo00D007ooOomo
o`0004moo`D000Q?W`X001Aoo`80009oo`04001oogoo0008Ool00`00Oomoo`0;Ool0015oo`03001o
ogoo009oo`03001oogoo04Uoo`L000I?W`<0009;W`03000Zgb[O00@Zg`03001oogoo019oo`8000=o
o`8000Yoo`<000Yoo`0047oo0P001Goo00<007ooOol0AGoo1P0015=O00<004nOCil00`001T^O00<0
02[O:]l012[O00<007ooOol04goo00<007ooOol06Goo000@Ool00`00Oomoo`05Ool00`00Oomoo`12
Ool50007Del30008Bil50003:]l2000DOol2000JOol000Uoo`<000Aoo`8000Moo`03001oogoo03io
o`H000MCG`8000=G_`80009;W`@000@oO`L001Aoo`03001oogoo01Qoo`0037oo00D007ooOomoo`00
00Uoo`03001oogoo03Yoo`D000AC?P03001CGe=O009CG`8000IG_`<000ToO`03000>?@hm008>?@80
01Aoo`03001oogoo01Moo`000goo1@0017oo00D007ooOomoo`0000Yoo`03001oogoo03Moo`@000QC
?P04001CG`000005Ekl30003Ekl20004?gl600033Sd00`00Oomoo`0BOol2000HOol000Yoo`8000Ao
o`03001oogoo00Uoo`8003Aoo`H000UC?P@0009G_`<000MG_`D000Dc7`<000033Sd0000001Aoo`03
001oogoo01Ioo`0037oo00D007ooOomoo`0000eoo`03001oogoo02ioo`D000AC7P03001C?U<n00=C
?P<000AGW`<000IG_`<000=C_`8000Hc7`030001VP0000<001Aoo`03001oogoo00Aoo`8000Aoo`80
00Aoo`8000=oo`002Goo0`000goo00<007ooOol03Goo00<007ooOol0:Woo1@0025<N00<005<n0000
0P001UNO00@005^nFkiK_P800004Ekl000000007Dkl60002<al010000IX1VP6J0P004goo0P0017oo
00@007ooOol000=oo`8000=oo`04001oogoo0002Ool000moo`8000moo`03001oogoo02Eoo`L000YC
7P<000IGW`8000EK_P<000IC_`<0009;W`04000R_R:n8[h500020IX2000BOol20004Ool01000Oomo
o`0027oo00@007ooOol0009oo`003goo00<007ooOol03goo00<007ooOol08Goo0`0000=C7@000000
15<M00<005<NDah01E<N0P000eMo00@005NOEimGW`8000EK_P8000=OgP800004Dkl000000006Bil0
0`008[hR_P048[h5000DOol01@00Oomoogoo00000Woo00<007ooOol01goo0P000goo000?Ool2000A
Ool00`00Oomoo`0MOol3000015<M000000001e<M00<005<NDah00U<N0P001UMo00<005NO00001E^n
0P001UoN0`001d^O1`000R:n00<0004G00000P004goo0P000Woo00@007ooOol000Qoo`04001oogoo
0002Ool000Qoo`@000=oo`03001oogoo015oo`03001oogoo01Uoo`<00006Dad00000001C7@002E<M
00@005<N000000QGO`@0009K_P8000IOgP8000=KgP80009;W`<000A3O`8000034Wd0000000<00003
0AL00000019oo`8000=oo`8000Yoo`8000=oo`002Goo00<007ooOol00goo00<007ooOol04Woo00<0
07ooOol05Woo0P0000AC7@00000000EC7@03001C7E<M00EC7@@000MGO`8000AK_P<000EOgP8000IK
gP<000M3O`@000<BO@8000030AL0000001=oo`03001oogoo015oo`000Woo1@000goo00<007ooOol0
0Woo00<007ooOol04goo00<007ooOol04Woo1P002E<M00<005<MDad0100015<n0P0015Mo00<005^n
Fkh00e^n00@005nnGkiO_P800003Gmh0000000EKgP<000=K_P030013Od=o00@000<kG`@000064Wd0
000000015`004Woo0P004Woo000;Ool00`00Oomoo`02000FOol00`00Oomoo`0>Ool60003Dad00`00
DaeC7@06Dad3000:Dch01000Egl0000015^n0P001Enn10000U_N0`001U^n10001c]O00<000HL1Q`0
1P004goo00<007ooOol047oo0008Ool01000Oomoo`000Woo00<007ooOol05Woo00<007ooOol02Woo
0`0000=C7@0000001e<M00<005<MDad00U<M0`000e<N00<005<nDch01e<n0`0015^n00<005nnGkh0
15nn00<006?NHmh00V?N0`001E^n0`000eNo2`000`HL00@0002e000001=oo`03001oogoo00moo`00
2Goo0P000goo0P0067oo0P001goo0`0000AC7@00000000YC7@04001C7@000007Dah00`00DciC?P03
Dch30003Fgh200000e^n001O_P04Gkh20004Hmh01000HmeSgF?M0P0000=K_P0000000eNo0`000e>o
00<003<O<al013<O100000<6700000000P004Woo0P0047oo000>Ool00`00Oomoo`0IOol01000Oomo
ogoo0`0000AC7@00000000=C7@03001C7E<M00MC7@8000YC7P03001C?P00008000MKOP<000AO_P03
001SgV?N009SgP8000ESg@@00003Ekl0000000IC_`H0008c7`030001^P6j00D001=oo`03001oogoo
00=oo`<000Qoo`003Woo00<007ooOol06Woo0`0000AC7@00000000MC7@03001C7E<M00=C7@<0009C
7@03001C7U<N00MC7P<000MKOP8000=OWP800003Gkh006?N00=SgP03001SgF?M00ASg@04001SgF?M
Hmd30004Dkl30003Dil00`00:_lZo`0500030KX3000COol00`00Oomoo`03Ool00`00Oomoo`07Ool0
00ioo`03001oogoo01Qoo`800004Dad000000002Dad00`00DaeC7@06Dad00`00Dad000020006Dad0
0`00DaiC7P03Dah30003Eeh00`00FgiKOP02Fgh20006Gih40002Hmh00`00HmeSg@03Hmd20003Hmd0
1P00HmiSgV?N001C_`8000MCW`03000Zob[o00@Zo`8000040KX00000000COol00`00Oomoo`03Ool0
0`00Oomoo`07Ool000Moo`<000=oo`8001Moo`H000IC7@03001C7E<M00AC7@@000UC7@03001C7P00
008000MGGP04001KOP000006Gih20004Hkh30003Hmd20003Hmd20004Hmh30007Dil?000COol00`00
Oomoo`02Ool00`00Oomoo`07Ool000Qoo`03001oogoo009oo`8001Aoo`<00003Dad00000008000QC
7@03001C7E<M00<000AC7@03001C7E<M00IC7@<000UGGP<000EOWP8000ES_P07001W_FNmIkd006?M
000016?M00<006?NHmh00V?N0P000f>n0P000U>O0`0014nO00<002:n8[h00b:n1P004Goo0`000goo
00<007ooOol01goo00000goo000000030002Ool00`00Oomoo`02Ool00`00Oomoo`0@Ool3000015<M
0000000015<M00<005<MDad015<M100025<M00<005<MDad00U<M0`000eLn00<005MNEeh015MN0P00
0e]n0P0000=OWP0000001F>n0P0016Nm10000V?M00<006?NHmh00V?N00<006>nHkh016>n100014nO
200000<R_P0000000`003Woo0`0000=oo`0000000Woo0P002Goo0008Ool00`00Oomoo`02Ool00`00
Oomoo`0=Ool3000015<M0000000025<M00<005<MDad00`000e<M00<005<MDad01e<M00@005<M0000
00MG?P03001GGUMN009GGP03001KOU]n00AKOP<000AS_P8000EW_@03001W_FNm009W_@@00003Hmh0
000000ES_P8000=O_PD000A7O`04000FOQIn5Wh7000:Ool30005Ool00`00Oomoo`0;Ool000Qoo`03
001oogoo009oo`03001oogoo00]oo`800004Dad000000002Dad00`00DaeC7@06Dad30007Dad00`00
DaeC7@06Dad2000:Ech01000Eeh000001U]n00@005nNGiiOWP800003Hkh006Nm00AW_@8000AW_@04
001WgFOMImd30005Hkh01000GkiO_Unn0P000enn00@004MoAgm7O`P0008FOP0400015`000005Ool4
0008Ool00`00Oomoo`0;Ool000Moo`8000Aoo`03001oogoo00Qoo`H000IC7@03001C7E<M009C7@<0
009C7@03001C7E<M00IC7@03001C7E<M009C7@<0009C7P03001G?ULn00MG?P<000EKOP8000EOWP<0
00AW_@03001W_FNm009W_@8000=Wg@04001WgFOMImd200000f>n00000003Gkh00`00GkiO_P03Gkh4
0005@el00`003Ud>G@07000017oo000000003Goo00<007ooOol02Woo000<Ool20007Ool300001E<M
000005<M000025<M00@005<M000000IC7@03001C7E<M00IC7@04001C7@000006Dah00`00EciG?P03
Ech30003Eeh01@00FgiKOU]n00001UnN00@006>mHkeS_@800003Ikd006Nm00=W_@03001WgFOM009W
g@03001WgFOM00=Wg@@00003Gkh0000000=O_P<0009O_P030013Gd=O0093G`H0008>G@030000U000
0080011oo`03001oogoo00Yoo`003Goo00<007ooOol00Woo0`0000AC7@00000000AC7@03001C7E<M
00AC7@<000UC7@03001C7E<M00AC7@<000UC7P03001G?P00008000MGGP03001KOP0000EOWP8000ES
_@@0009W_@03001WgFOM009Wg@03001WgFOM00=Wg@04001W_FNmIkd400000enn00000005Gkh50005
?el50000102D000000003goo00<007ooOol02Woo000<Ool00`00Oomoo`03000015<M0000000025<M
00<005<MDad00`000e<M00<005<MDad01e<M00<005<MDad00`000e<M00<005<NDah01U<N0`002EMN
0`0015nN00<006>mHkd016>m00<006NmIkd00VNm10000VOM00<006OMImd00VOM0P000fNm00@006Nm
IkeW_@<000AO_P8000=K_PX000031Q`0000000<000moo`03001oogoo00Yoo`0037oo0`0000AC7@00
0000009C7@03001C7E<M00IC7@<000MC7@03001C7E<M00EC7@@000MC7@03001C7U<N009C7P<000=G
?P8000AGGP<000=OOP800003Gih006>m00AS_@8000AW_@03001W_FNm009W_@<000=Wg@04001W_FNm
Ikd20003Ikd01P00IkeW_FNm001O_P8000=K_P8000=KW`03000k?c/o00<k?`P000ioo`03001oogoo
00Yoo`001Woo0P0017oo10001U<M00<005<MDad00U<M0`000U<M00<005<MDad01U<M00<005<MDad0
10000e<M00<005<MDad01U<M00@005<N000000QG?P04001GGP000007Ggh30004Hkd00`00IkeW_@02
Ikd20004Ikd01`00Jka[_6^l001Wg@0000=W_@04001W_FNmIkd20004Ikd400000e^n00000005Fil9
00000`7K00000003000=Ool00`00Oomoo`0:Ool000Eoo`04001oogoo0003Ool30008Dad01000Dad0
00001U<M00<005<MDad01E<M0`0025<M00<005<MDad015<M10002ELn0`001Umn0P000f>M0P0000=S
_@00Ikd00fNm00<006NmIkd00fNm00<006^lJk`00V^l100000=W_@00Ikd00VNm00<006NmIkd00VNm
0P000fNm0P0015^O0`002Goo1`0037oo00<007ooOol02Woo0005Ool01000Oomoo`001Goo0P001U<M
100025<M00<005<MDad00U<M0P000e<M00<005<MDad01U<M00<005<MDad00`0015<N0P0015Ln0`00
0e]N00<005mnGgh00Umn00<006>MHid016>M10000VNm00<006NmIkd00fNm00<006^lJk`00V^l00@0
06^lJka[_0@00003Ikd006Nm00=W_@03001W_FNm009W_@05001W_FNm001KW`02000?Ool4000=Ool0
0`00Oomoo`09Ool000Eoo`04001oogoo0006Ool20002Dad30004Dad00`00DaeC7@06Dad01000Dad0
00001U<M00<005<MDad015<M10002E<N00@005Ln000000MKGP04001OOP000005Hid20004Hid30004
Ikd01000Jka[_6^l0P000f^l00<006^lJk`00V^l0`0000=W_@0000000fNm0P000fNm0P0057oo0P00
37oo00<007ooOol02Goo0005Ool01000Oomoo`001goo0`0025<M00<005<MDad015<M0`002E<M00<0
05<MDad00`0015<M00<005<NDah01U<N0`0025]N0`001F>M00<006>MHid00f>M00@006NMIieWW@80
0003Ikd006^l009[_003001[_6^l009[_003001[_6^l009[_004001[_6^lJk`20003Ikd00`00IkeW
_@02Ikd00`00Oomoo`0EOol2000;Ool00`00Oomoo`09Ool000Ioo`8000Uoo`<000QC7@03001C7E<M
00<000=C7@03001C7E<M00IC7@<000QC7@03001C7U<N009C7P<000=G?P03001KGU]N009KGP<000=O
OP800003Hid0000000ASW@8000EWW@@00003Jk`006^l00=[_003001[_6^l009[_005001[_6^lJk`0
0002Jk`00`00Ikd00003Ikd2000HOol2000;Ool00`00Oomoo`09Ool001=oo`8000IC7@@000MC7@03
001C7E<M009C7@<000=C7@03001C7E<M00IC7@04001C7P000007Ech01000Feh000001emn0`0016>M
00<006NMIid016NM00<006^lJk`00V^l0`000V^l00<006^lJk`00V^l00D006^lJka[_00000=[_0@0
0003Ikd007oo01Yoo`8000Yoo`03001oogoo00Uoo`0057oo0P000U<M0`0015<M00<005<MDad01E<M
00@005<M000000MC7@03001C7E<M00AC7@@000QG?P<000IOOP8000=SO@800003Hid006NM00AWW@80
00A[_004001[_6^lJk`30003Jk`01@00Jka[_6^l00000f^l00@006^lJka[_08001eoo`8000Uoo`03
001oogoo00Uoo`005Goo0`0025<M00<005<MDad015<M0`002E<M00<005<MDad00`0015LN00<005Ln
Ech00ULn0`000e]N00<005mnGgh00Umn00<006=mHgd016=m0`0016NM00<006^lJk`00f^l00D006^l
Jka[_00000=[_003001[_00000=[_005001[_6^lJk`00003Jk`00`00Oomoo`0MOol30008Ool00`00
Oomoo`09Ool001Ioo`<000QC7@03001C7E<M00<000=C7@03001C7E<M00IC7@<000QG7P04001G?P00
0007Feh01000Ggh000001V=m00@006>MHieSW@800003Iid006^l00A[_005001[_6^lJk`00004Jk`4
00000f^l001[_002Jk`01@00Jka[_6^l00008Goo0`0027oo00<007ooOol027oo000HOol20006Dad4
0007Dad00`00DaeC7@02Dad30002Dad00`00EaiG7P06Eah20009Feh30005Hgd20005Hid30004Jk`0
1@00Jka[_6^l000016^l00<006nlKk`00Vnl0`0000=[_000Jk`00V^l00<007ooOol08Goo0`001goo
00<007ooOol027oo000IOol20002Dad30003Dad00`00DaeC7@06Dad01000Dad000001U<M00<005LN
Eah00ULN0`000ULn00<005]NFeh015]N0P000emM00D006=mHgeSO@0000ISW@04001WWFNMIid20000
0f^l001[_002Jk`00`00Jka[_002Jk`00`00Kka__002Kk`01000Kk]_^fnk0`0000=[_000Ool097oo
0`001Woo00<007ooOol027oo000JOol30007Dad00`00DaeC7@04Dad40008Dad01000Eah000001ULn
00<005]NFeh00U]N00<005mMGed015mM00<006=m00001F>M0P001FNM100000=[_000Jk`00f^l00<0
06nlKk`00Vnl00D006nkKk]_^`0000=_^`03001oogoo02Aoo`<000Ioo`03001oogoo00Qoo`006goo
0`001e<M00<005<MDad00`0015<M0P001U<M0`0025Ln00@005]N000000MOG@<000ASW@03001WWFNM
00AWW@04001WW6NLIi`40002Jk`00`00Kka__002Kk`01@00Kk]_^fnk00000fnk00<007ooOol09Woo
0`001Goo00<007ooOol027oo000MOol20006Dad30009Dad00`00DaeC7@030003Ead20006Ech30006
Ged20003Ggd200000f>M001WW@04Iid20003Ii`00`00Jia[W002Ji`30003Kk`01@00Kk]_^fnk0000
0fnk00<007ooOol0:7oo0`0017oo00<007ooOol027oo000NOol20002Dad30003Dad00`00DaeC7@06
Dad30008Ead00`00EciG?P030003Fcd300000emM00000006Ggd30004Iid00`00IiaWW002Ii`00`00
Jia[W002Ji`01`00Jia[W6^L001__00000=_^`05001_^fnkKk/0000/Ool20004Ool00`00Oomoo`08
Ool001moo`<000MC7@03001C7E<M009C7@<0009C7@03001G7ELM00IG7@<000QK?@<000IOO@04001S
OF=mHgd200000fNM001WW003Ii`00`00Jia[W002Ji`00`00Jia[W002Ji`400000fnk001_^`02Kk/0
0`00Oomoo`0[Ool30004Ool00`00Oomoo`07Ool0021oo`8000QC7@04001C7@000006Dad00`00EaeG
7@02Ead30002Ecd00`00FceK?@04Fcd20003Fed01000GgeOOEmm0P001F=m10000VNL00<006^LJi`0
0V^L00<006^LJi`00V^L00<006nKKi/00VnK0`0000=_^`00Ool0;Woo0`000goo00<007ooOol01goo
000ROol00`00DaeC7@04Dad40008Dad01000Ead000001ULm00<005/mFcd00U/m00<005]MFed015]M
00<005mm00001F=m0P0016Mm0`000f^L00<006^LJi`00V^L00<006nKKi/00VnK00D006nKKi]_V`00
035oo`8000=oo`03001oogoo00Moo`008goo00<005<MDad00`0015<M00<005<MDad015<M100025Lm
00@005/m000000MKG@<000ASO@03001WOFMm00=WO@07001WW6NLIi`006^L000016^L00<006nKKi/0
0VnK00D006nKKi]_V`0003=oo`80009oo`03001oogoo00Moo`0097oo0P0025<M00<005<MDad00`00
15LM00<005LmEcd015Lm10001E]M0`000emM00@006=m000000AWO@8000AWW0@0009[W003001_VfnK
009_V`05001_VfnKKi/0000dOol20002Ool00`00Oomoo`07Ool002Eoo`<000IC7@<000QG7@03001G
?ELm00<000AK?@800003Fed0000000IOG@<000AWO@03001WW6NL00=WW003001[W6^L009[W0<000=_
V`05001_VfnKKi/0000fOol200000goo001oo`08Ool002Moo`80009C7@<000=C7@03001G7ELM00EG
7@<000QK?@<000IOG@04001SOF=mHgd200000fMm001WW004Ii`00`00Jia[W002Ji`01`00Jia[W6^L
001_V`0000=_V`03001oogoo03Ioo`<000Uoo`00:7oo0`001e<M00<005LMEad00ULM0P000eLm00<0
05/mFcd00U/m0`000e]M00@005mMGeeOG@8000ESO@<000AWW004001[W6^LJi`20004Ji`400000fnK
001oo`0hOol200000goo001oo`07Ool002Uoo`8000QC7@04001G7@000006Ecd01000Fcd000001e]M
00<005mM00001V=m00@006=mHgeSO@800003Ii`006^L009[W003001[W6^L00=[W004001[Vf^KJi/2
000kOol30008Ool002Yoo`8000IC7@@000MG?@<000QKG@<000ASO@8000ESO@@00003Ji`006^L00A[
W005001[Vf^KJi/0000nOol20008Ool002aoo`03001C7E<M00<000AG7@03001G?ELm009G?@8000=G
?@03001KGE]M009KG@<000=OG@800003Hgd006=m00ASO@8000=WO0<000A[W005001[Vf^KJi/0000n
Ool30008Ool002eoo`8000QG7@04001G?@000006Ecd01000Fed000001emM0`0016=m00<006MlIg`0
0VMl00@006MlIgaWO0800003Ji`006^K009[V`03001oogoo03Uoo`@000]oo`00;Woo0`001ULM0`00
1eLm0`001UmM0P000emM00@006=m000000AWO003001WO6Ml00=WO0@00003Ji/007oo03Moo`@000mo
o`00;goo00<007oo00000ULM0`000eLM00<005LmEcd00ULm0P000e/m00<005mMGed00UmM00<005mM
Ged00emM0`0016Ml00<006MlIg`00fMl00<006]lJg`00V]l00<007ooOol0=7oo0`004goo000`Ool4
0007Ead01000Ecd000001U/m00@005mM000000EOG@04001SGF=MHed200000fMl001WO003Ig`20004
Jg`00`00Oomoo`0aOol4000FOol0039oo`03001oo`0000IG7@<000MK?@<000AOG@8000ESG@<000=W
O003001[O6]l00=[O003001oogoo02ioo`@001Yoo`00<goo00<007oo00000ULM0`000eLm00<005/m
Fcd00U/m0P000e/m0P0000=OG@00Hed01F=M00L006=lHgaSO000Ig`00005Jg`00`00Oomoo`0[Ool4
000NOol003Aoo`04001oo`000007Ecd01000Fcd000001U/m0`0016=M0P0016=l0`0016]l00<007oo
Ool0:7oo10008Woo000eOol200000goo001G?@05Ecd30005Fcd20003Ged200000f=M001SO004Hg`0
1000IgaWO6Ml0P0000=[O000Ool09Woo10009Woo000gOol00`00Ool00002Ecd30003Fcd200000e/m
00000006Ged30003Hg`20005Ig`2000UOol3000ZOol003Qoo`04001oo`000007Fcd30006Ged01`00
GeeOGEmM001SO00000IWO003001oogoo025oo`@002eoo`00>Goo00@007ooOol000AK?@8000=K?@04
001OGEmMGed20004Ged30004Ig`2000POol4000aOol003Yoo`800003Ool005/m008000IK?@03001O
G@0000EOG@04001SG6=LHe`200000fMl001oo`0MOol4000eOol003aoo`04001oo`000006Fcd40003
Ged20005He`2000KOol4000iOol003eoo`04001oogoo0002Fcd30004Gcd00`00Ged00006He`00`00
Oomoo`0GOol4000mOol003ioo`03001oogoo008000MO?@<000ASG08001Moo`<0045oo`00?goo0P00
0Woo00<005lmGcd00Ulm0P000emM0P0000=SG000Ool057oo1000A7oo0011Ool01@00Oomoo`00Gcd0
0P001UmM0P004Woo1000B7oo0012Ool00`00Oomoo`020005Ged2000@Ool4001<Ool004=oo`04001o
ogooOol200000emM0000000>Ool4001@Ool004Aoo`8000=oo`03001oogoo00Yoo`@005Aoo`00AWoo
00@007ooOol000Uoo`<005Qoo`00Agoo00<007oo00001Goo1000Fgoo0018Ool200000goo00000002
001OOol004Uoo`8006=oo`00[Woo002^Ool00001\
\>"],
  ImageRangeCache->{{{0, 173.375}, {140.313, 0}} -> {-0.0756344, -0.0183307, 
  0.00621756, 0.00621756}}]
}, Open  ]],

Cell[TextData[{
  "Because of the flatness of the value surface, Delta is close to ",
  Cell[BoxData[
      \(TraditionalForm\`\(-1\)\)]],
  " everywhere, as revealed in the next plot."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[UpAndOutPutDelta[0, S, 50, 50, 0.25,0.1, 0.15, t],
{S, 30, 50}, {t, 0.001, 1}, PlotRange -> All];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.7672 .09664 0 -6.16225 ]
[.7672 .09664 18 2.83775 ]
[.82794 .18966 0 -6.07272 ]
[.82794 .18966 18 2.92728 ]
[.88358 .27486 0 -5.99233 ]
[.88358 .27486 18 3.00767 ]
[.93473 .3532 0 -5.91977 ]
[.93473 .3532 18 3.08023 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.02388 .26658 -24 -2.74356 ]
[.02388 .26658 0 6.25644 ]
[.01382 .32962 -24 -2.79688 ]
[.01382 .32962 0 6.20312 ]
[.00334 .3953 -12 -2.85267 ]
[.00334 .3953 0 6.14733 ]
[-0.00759 .46378 -24 -2.91108 ]
[-0.00759 .46378 0 6.08892 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.75489 .10119 m
.74873 .10346 L
s
[(0.2)] .7672 .09664 -1 .36939 Mshowa
.81555 .19399 m
.80936 .19615 L
s
[(0.4)] .82794 .18966 -1 .34949 Mshowa
.87113 .27899 m
.8649 .28106 L
s
[(0.6)] .88358 .27486 -1 .33163 Mshowa
.92222 .35714 m
.91596 .35912 L
s
[(0.8)] .93473 .3532 -1 .3155 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(1)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.70561 .0258 m
.70193 .02722 L
s
.72242 .05152 m
.71874 .05292 L
s
.73884 .07664 m
.73515 .07802 L
s
.77057 .12518 m
.76687 .12652 L
s
.7859 .14863 m
.7822 .14996 L
s
.80089 .17156 m
.79718 .17287 L
s
.8299 .21593 m
.82618 .21721 L
s
.84394 .2374 m
.84021 .23867 L
s
.85767 .25842 m
.85394 .25967 L
s
.8843 .29914 m
.88056 .30036 L
s
.8972 .31887 m
.89345 .32008 L
s
.90983 .3382 m
.90608 .3394 L
s
.93435 .37571 m
.93059 .37688 L
s
.94625 .3939 m
.94249 .39506 L
s
.95791 .41174 m
.95414 .41289 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.0361 .26181 m
.04222 .25942 L
s
[(-1.2)] .02388 .26658 1 -0.39032 Mshowa
.02609 .32498 m
.03223 .32265 L
s
[(-1.1)] .01382 .32962 1 -0.37847 Mshowa
.01566 .39079 m
.02182 .38853 L
s
[(-1)] .00334 .3953 1 -0.36607 Mshowa
.00479 .45941 m
.01097 .45723 L
s
[(-0.9)] -0.00759 .46378 1 -0.35309 Mshowa
.125 Mabswid
.03413 .27424 m
.0378 .27281 L
s
.03215 .28677 m
.03582 .28535 L
s
.03015 .2994 m
.03382 .29799 L
s
.02813 .31214 m
.03181 .31074 L
s
.02404 .33792 m
.02773 .33654 L
s
.02197 .35098 m
.02566 .3496 L
s
.01989 .36414 m
.02358 .36277 L
s
.01778 .37741 m
.02148 .37605 L
s
.01352 .40428 m
.01722 .40294 L
s
.01137 .41789 m
.01507 .41656 L
s
.00919 .43162 m
.0129 .43029 L
s
.007 .44546 m
.01071 .44414 L
s
.00256 .47349 m
.00627 .47219 L
s
.00031 .48769 m
.00403 .4864 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.667 .721 .889 r
.37487 .78156 .39806 .80578 .43701 .79343 .41412 .76923 Metetra
.674 .725 .888 r
.41412 .76923 .43701 .79343 .47649 .77992 .45394 .75582 Metetra
.684 .731 .886 r
.45394 .75582 .47649 .77992 .51649 .76495 .49429 .74102 Metetra
.696 .739 .884 r
.49429 .74102 .51649 .76495 .55697 .74821 .53515 .72452 Metetra
.71 .747 .881 r
.53515 .72452 .55697 .74821 .59787 .7294 .57647 .70599 Metetra
.725 .756 .877 r
.57647 .70599 .59787 .7294 .63912 .70824 .61818 .6851 Metetra
.741 .765 .872 r
.61818 .6851 .63912 .70824 .68065 .68445 .66021 .66158 Metetra
.758 .774 .866 r
.66021 .66158 .68065 .68445 .72236 .65783 .70245 .63518 Metetra
.775 .782 .858 r
.70245 .63518 .72236 .65783 .76413 .62823 .7448 .60572 Metetra
.792 .79 .85 r
.7448 .60572 .76413 .62823 .80587 .59554 .78713 .57307 Metetra
.808 .797 .841 r
.78713 .57307 .80587 .59554 .84744 .55974 .82932 .53718 Metetra
.823 .803 .832 r
.82932 .53718 .84744 .55974 .88873 .52085 .87124 .49808 Metetra
.837 .807 .822 r
.87124 .49808 .88873 .52085 .92963 .47897 .91278 .45586 Metetra
.849 .811 .812 r
.91278 .45586 .92963 .47897 .97003 .43424 .95382 .41065 Metetra
.664 .719 .89 r
.35113 .7566 .37487 .78156 .41412 .76923 .39068 .74428 Metetra
.671 .723 .889 r
.39068 .74428 .41412 .76923 .45394 .75582 .43082 .73096 Metetra
.68 .729 .887 r
.43082 .73096 .45394 .75582 .49429 .74102 .47152 .71634 Metetra
.691 .736 .885 r
.47152 .71634 .49429 .74102 .53515 .72452 .51276 .70008 Metetra
.705 .744 .882 r
.51276 .70008 .53515 .72452 .57647 .70599 .5545 .68185 Metetra
.72 .754 .879 r
.5545 .68185 .57647 .70599 .61818 .6851 .59668 .66129 Metetra
.737 .763 .874 r
.59668 .66129 .61818 .6851 .66021 .66158 .63921 .63809 Metetra
.754 .773 .868 r
.63921 .63809 .66021 .66158 .70245 .63518 .68199 .61196 Metetra
.772 .782 .861 r
.68199 .61196 .70245 .63518 .7448 .60572 .72492 .58269 Metetra
.79 .791 .853 r
.72492 .58269 .7448 .60572 .78713 .57307 .76786 .55013 Metetra
.807 .798 .844 r
.76786 .55013 .78713 .57307 .82932 .53718 .81069 .51421 Metetra
.822 .805 .834 r
.81069 .51421 .82932 .53718 .87124 .49808 .85327 .47493 Metetra
.837 .81 .824 r
.85327 .47493 .87124 .49808 .91278 .45586 .89547 .43237 Metetra
.85 .813 .813 r
.89547 .43237 .91278 .45586 .95382 .41065 .93717 .38669 Metetra
.661 .717 .89 r
.32681 .73088 .35113 .7566 .39068 .74428 .36667 .71854 Metetra
.667 .721 .889 r
.36667 .71854 .39068 .74428 .43082 .73096 .40712 .70529 Metetra
.676 .726 .888 r
.40712 .70529 .43082 .73096 .47152 .71634 .44817 .69083 Metetra
.686 .733 .886 r
.44817 .69083 .47152 .71634 .51276 .70008 .48979 .67484 Metetra
.7 .742 .884 r
.48979 .67484 .51276 .70008 .5545 .68185 .53195 .65693 Metetra
.715 .751 .881 r
.53195 .65693 .5545 .68185 .59668 .66129 .57458 .63674 Metetra
.732 .761 .876 r
.57458 .63674 .59668 .66129 .63921 .63809 .61762 .61392 Metetra
.75 .772 .871 r
.61762 .61392 .63921 .63809 .68199 .61196 .66095 .58813 Metetra
.768 .782 .864 r
.66095 .58813 .68199 .61196 .72492 .58269 .70447 .55913 Metetra
.787 .791 .856 r
.70447 .55913 .72492 .58269 .76786 .55013 .74804 .52672 Metetra
.805 .8 .847 r
.74804 .52672 .76786 .55013 .81069 .51421 .79153 .49081 Metetra
.821 .806 .837 r
.79153 .49081 .81069 .51421 .85327 .47493 .83479 .45139 Metetra
.837 .812 .826 r
.83479 .45139 .85327 .47493 .89547 .43237 .87768 .40852 Metetra
.851 .816 .815 r
.87768 .40852 .89547 .43237 .93717 .38669 .92006 .36237 Metetra
.659 .715 .89 r
.30191 .70437 .32681 .73088 .36667 .71854 .34206 .69197 Metetra
.664 .719 .889 r
.34206 .69197 .36667 .71854 .40712 .70529 .38283 .67876 Metetra
.671 .724 .889 r
.38283 .67876 .40712 .70529 .44817 .69083 .42422 .66446 Metetra
.681 .73 .887 r
.42422 .66446 .44817 .69083 .48979 .67484 .46622 .64873 Metetra
.694 .739 .886 r
.46622 .64873 .48979 .67484 .53195 .65693 .50879 .63118 Metetra
.709 .749 .883 r
.50879 .63118 .53195 .65693 .57458 .63674 .55188 .61141 Metetra
.726 .759 .879 r
.55188 .61141 .57458 .63674 .61762 .61392 .59542 .58902 Metetra
.745 .77 .874 r
.59542 .58902 .61762 .61392 .66095 .58813 .63931 .56366 Metetra
.764 .781 .867 r
.63931 .56366 .66095 .58813 .70447 .55913 .68343 .535 Metetra
.783 .792 .859 r
.68343 .535 .70447 .55913 .74804 .52672 .72765 .50283 Metetra
.802 .801 .85 r
.72765 .50283 .74804 .52672 .79153 .49081 .77181 .467 Metetra
.82 .809 .84 r
.77181 .467 .79153 .49081 .83479 .45139 .81577 .42748 Metetra
.837 .815 .829 r
.81577 .42748 .83479 .45139 .87768 .40852 .85938 .38434 Metetra
.851 .819 .817 r
.85938 .38434 .87768 .40852 .92006 .36237 .90248 .33771 Metetra
.656 .713 .89 r
.27639 .67704 .30191 .70437 .34206 .69197 .31684 .66453 Metetra
.661 .716 .889 r
.31684 .66453 .34206 .69197 .38283 .67876 .35793 .65133 Metetra
.667 .721 .889 r
.35793 .65133 .38283 .67876 .42422 .66446 .39965 .63717 Metetra
.676 .727 .888 r
.39965 .63717 .42422 .66446 .46622 .64873 .44202 .6217 Metetra
.688 .735 .887 r
.44202 .6217 .46622 .64873 .50879 .63118 .48499 .60453 Metetra
.703 .745 .885 r
.48499 .60453 .50879 .63118 .55188 .61141 .52854 .58522 Metetra
.72 .757 .882 r
.52854 .58522 .55188 .61141 .59542 .58902 .57258 .56335 Metetra
.739 .768 .877 r
.57258 .56335 .59542 .58902 .63931 .56366 .61703 .53849 Metetra
.759 .781 .871 r
.61703 .53849 .63931 .56366 .68343 .535 .66176 .51028 Metetra
.779 .792 .863 r
.66176 .51028 .68343 .535 .72765 .50283 .70664 .47844 Metetra
.8 .802 .854 r
.70664 .47844 .72765 .50283 .77181 .467 .75151 .44278 Metetra
.819 .811 .843 r
.75151 .44278 .77181 .467 .81577 .42748 .7962 .40323 Metetra
.836 .818 .831 r
.7962 .40323 .81577 .42748 .85938 .38434 .84055 .35985 Metetra
.852 .823 .819 r
.84055 .35985 .85938 .38434 .90248 .33771 .8844 .31277 Metetra
.654 .711 .89 r
.25025 .64886 .27639 .67704 .31684 .66453 .29099 .63621 Metetra
.658 .714 .89 r
.29099 .63621 .31684 .66453 .35793 .65133 .33239 .62297 Metetra
.663 .718 .889 r
.33239 .62297 .35793 .65133 .39965 .63717 .37445 .6089 Metetra
.671 .724 .889 r
.37445 .6089 .39965 .63717 .44202 .6217 .41717 .59367 Metetra
.682 .732 .888 r
.41717 .59367 .44202 .6217 .48499 .60453 .46055 .57689 Metetra
.696 .742 .887 r
.46055 .57689 .48499 .60453 .52854 .58522 .50454 .5581 Metetra
.712 .753 .884 r
.50454 .5581 .52854 .58522 .57258 .56335 .54908 .53682 Metetra
.732 .766 .88 r
.54908 .53682 .57258 .56335 .61703 .53849 .59409 .51258 Metetra
.753 .779 .875 r
.59409 .51258 .61703 .53849 .66176 .51028 .63944 .48494 Metetra
.774 .792 .867 r
.63944 .48494 .66176 .51028 .70664 .47844 .68499 .45355 Metetra
.796 .804 .858 r
.68499 .45355 .70664 .47844 .75151 .44278 .73059 .41816 Metetra
.817 .814 .847 r
.73059 .41816 .75151 .44278 .7962 .40323 .77603 .37868 Metetra
.836 .821 .835 r
.77603 .37868 .7962 .40323 .84055 .35985 .82116 .3351 Metetra
.853 .827 .822 r
.82116 .3351 .84055 .35985 .8844 .31277 .86579 .28758 Metetra
.653 .71 .889 r
.22346 .61982 .25025 .64886 .29099 .63621 .2645 .60696 Metetra
.655 .712 .889 r
.2645 .60696 .29099 .63621 .33239 .62297 .30621 .59361 Metetra
.659 .715 .889 r
.30621 .59361 .33239 .62297 .37445 .6089 .34859 .57958 Metetra
.666 .72 .889 r
.34859 .57958 .37445 .6089 .41717 .59367 .39167 .56457 Metetra
.675 .728 .889 r
.39167 .56457 .41717 .59367 .46055 .57689 .43543 .54818 Metetra
.688 .737 .888 r
.43543 .54818 .46055 .57689 .50454 .5581 .47985 .52995 Metetra
.704 .749 .887 r
.47985 .52995 .50454 .5581 .54908 .53682 .52489 .50935 Metetra
.723 .763 .884 r
.52489 .50935 .54908 .53682 .59409 .51258 .57045 .48584 Metetra
.745 .777 .879 r
.57045 .48584 .59409 .51258 .63944 .48494 .61643 .45892 Metetra
.768 .792 .872 r
.61643 .45892 .63944 .48494 .68499 .45355 .66267 .42814 Metetra
.792 .805 .863 r
.66267 .42814 .68499 .45355 .73059 .41816 .70901 .39317 Metetra
.814 .816 .852 r
.70901 .39317 .73059 .41816 .77603 .37868 .75525 .35386 Metetra
.835 .825 .839 r
.75525 .35386 .77603 .37868 .82116 .3351 .80119 .31016 Metetra
.853 .832 .825 r
.80119 .31016 .82116 .3351 .86579 .28758 .84664 .26222 Metetra
.652 .708 .889 r
.196 .5899 .22346 .61982 .2645 .60696 .23734 .57676 Metetra
.653 .71 .889 r
.23734 .57676 .2645 .60696 .30621 .59361 .27935 .56324 Metetra
.656 .712 .889 r
.27935 .56324 .30621 .59361 .34859 .57958 .32206 .54917 Metetra
.661 .716 .89 r
.32206 .54917 .34859 .57958 .39167 .56457 .36548 .53431 Metetra
.669 .723 .89 r
.36548 .53431 .39167 .56457 .43543 .54818 .40962 .51828 Metetra
.68 .732 .89 r
.40962 .51828 .43543 .54818 .47985 .52995 .45447 .50063 Metetra
.695 .744 .889 r
.45447 .50063 .47985 .52995 .52489 .50935 .49998 .4808 Metetra
.714 .758 .887 r
.49998 .4808 .52489 .50935 .57045 .48584 .54609 .45818 Metetra
.736 .774 .884 r
.54609 .45818 .57045 .48584 .61643 .45892 .59269 .43216 Metetra
.761 .791 .877 r
.59269 .43216 .61643 .45892 .66267 .42814 .63964 .40219 Metetra
.786 .806 .869 r
.63964 .40219 .66267 .42814 .70901 .39317 .68674 .36784 Metetra
.811 .82 .857 r
.68674 .36784 .70901 .39317 .75525 .35386 .7338 .32885 Metetra
.834 .83 .843 r
.7338 .32885 .75525 .35386 .80119 .31016 .7806 .28514 Metetra
.854 .837 .828 r
.7806 .28514 .80119 .31016 .84664 .26222 .82691 .23681 Metetra
.651 .707 .889 r
.16784 .55907 .196 .5899 .23734 .57676 .20948 .54559 Metetra
.652 .708 .889 r
.20948 .54559 .23734 .57676 .27935 .56324 .25181 .53181 Metetra
.653 .71 .889 r
.25181 .53181 .27935 .56324 .32206 .54917 .29484 .51761 Metetra
.657 .713 .889 r
.29484 .51761 .32206 .54917 .36548 .53431 .3386 .5028 Metetra
.662 .718 .89 r
.3386 .5028 .36548 .53431 .40962 .51828 .38311 .48708 Metetra
.671 .726 .891 r
.38311 .48708 .40962 .51828 .45447 .50063 .42835 .47 Metetra
.685 .738 .891 r
.42835 .47 .45447 .50063 .49998 .4808 .47433 .45101 Metetra
.703 .753 .891 r
.47433 .45101 .49998 .4808 .54609 .45818 .52097 .42943 Metetra
.725 .77 .888 r
.52097 .42943 .54609 .45818 .59269 .43216 .56819 .40454 Metetra
.751 .789 .884 r
.56819 .40454 .59269 .43216 .63964 .40219 .61584 .37566 Metetra
.779 .807 .875 r
.61584 .37566 .63964 .40219 .68674 .36784 .66374 .3422 Metetra
.807 .823 .864 r
.66374 .3422 .68674 .36784 .7338 .32885 .71166 .30377 Metetra
.832 .836 .849 r
.71166 .30377 .7338 .32885 .7806 .28514 .75935 .2602 Metetra
.855 .844 .833 r
.75935 .2602 .7806 .28514 .82691 .23681 .80658 .21154 Metetra
.651 .707 .888 r
.13896 .52734 .16784 .55907 .20948 .54559 .18091 .51346 Metetra
.651 .707 .888 r
.18091 .51346 .20948 .54559 .25181 .53181 .22355 .49933 Metetra
.652 .708 .888 r
.22355 .49933 .25181 .53181 .29484 .51761 .26691 .48487 Metetra
.653 .71 .889 r
.26691 .48487 .29484 .51761 .3386 .5028 .31101 .46998 Metetra
.657 .713 .89 r
.31101 .46998 .3386 .5028 .38311 .48708 .35587 .45443 Metetra
.663 .72 .891 r
.35587 .45443 .38311 .48708 .42835 .47 .40151 .43786 Metetra
.674 .73 .892 r
.40151 .43786 .42835 .47 .47433 .45101 .44792 .41974 Metetra
.69 .745 .893 r
.44792 .41974 .47433 .45101 .52097 .42943 .49507 .39936 Metetra
.711 .763 .893 r
.49507 .39936 .52097 .42943 .56819 .40454 .54289 .3759 Metetra
.738 .785 .89 r
.54289 .3759 .56819 .40454 .61584 .37566 .59125 .34848 Metetra
.769 .807 .884 r
.59125 .34848 .61584 .37566 .66374 .3422 .63996 .31631 Metetra
.8 .827 .872 r
.63996 .31631 .66374 .3422 .71166 .30377 .68877 .27879 Metetra
.83 .843 .857 r
.68877 .27879 .71166 .30377 .75935 .2602 .73742 .23559 Metetra
.856 .854 .838 r
.73742 .23559 .75935 .2602 .80658 .21154 .78562 .1867 Metetra
.65 .706 .888 r
.10933 .49468 .13896 .52734 .18091 .51346 .15159 .48036 Metetra
.651 .706 .888 r
.15159 .48036 .18091 .51346 .22355 .49933 .19455 .4658 Metetra
.651 .706 .888 r
.19455 .4658 .22355 .49933 .26691 .48487 .23824 .45096 Metetra
.651 .707 .888 r
.23824 .45096 .26691 .48487 .31101 .46998 .28269 .43581 Metetra
.653 .709 .889 r
.28269 .43581 .31101 .46998 .35587 .45443 .3279 .42024 Metetra
.656 .713 .89 r
.3279 .42024 .35587 .45443 .40151 .43786 .37391 .40399 Metetra
.663 .721 .892 r
.37391 .40399 .40151 .43786 .44792 .41974 .42073 .38667 Metetra
.675 .734 .895 r
.42073 .38667 .44792 .41974 .49507 .39936 .46836 .3676 Metetra
.694 .753 .897 r
.46836 .3676 .49507 .39936 .54289 .3759 .51675 .34589 Metetra
.721 .777 .897 r
.51675 .34589 .54289 .3759 .59125 .34848 .56581 .32046 Metetra
.754 .804 .893 r
.56581 .32046 .59125 .34848 .63996 .31631 .61534 .29021 Metetra
.79 .831 .884 r
.61534 .29021 .63996 .31631 .68877 .27879 .66509 .25419 Metetra
.826 .853 .867 r
.66509 .25419 .68877 .27879 .73742 .23559 .71476 .21178 Metetra
.857 .866 .846 r
.71476 .21178 .73742 .23559 .78562 .1867 .764 .16281 Metetra
.65 .705 .887 r
.07892 .46108 .10933 .49468 .15159 .48036 .12149 .44629 Metetra
.65 .705 .887 r
.12149 .44629 .15159 .48036 .19455 .4658 .16478 .43124 Metetra
.65 .705 .887 r
.16478 .43124 .19455 .4658 .23824 .45096 .20881 .41593 Metetra
.65 .706 .887 r
.20881 .41593 .23824 .45096 .28269 .43581 .2536 .40036 Metetra
.651 .706 .888 r
.2536 .40036 .28269 .43581 .3279 .42024 .29918 .38448 Metetra
.652 .708 .889 r
.29918 .38448 .3279 .42024 .37391 .40399 .34556 .36822 Metetra
.654 .712 .89 r
.34556 .36822 .37391 .40399 .42073 .38667 .39278 .35138 Metetra
.661 .721 .894 r
.39278 .35138 .42073 .38667 .46836 .3676 .44084 .33352 Metetra
.674 .737 .898 r
.44084 .33352 .46836 .3676 .51675 .34589 .48975 .31382 Metetra
.696 .763 .903 r
.48975 .31382 .51675 .34589 .56581 .32046 .53946 .2911 Metetra
.73 .796 .905 r
.53946 .2911 .56581 .32046 .61534 .29021 .58981 .26381 Metetra
.772 .833 .899 r
.58981 .26381 .61534 .29021 .66509 .25419 .64056 .2304 Metetra
.817 .865 .883 r
.64056 .2304 .66509 .25419 .71476 .21178 .69134 .18962 Metetra
.857 .886 .857 r
.69134 .18962 .71476 .21178 .764 .16281 .74173 .14092 Metetra
.65 .705 .887 r
.0477 .4265 .07892 .46108 .12149 .44629 .09058 .4112 Metetra
.65 .705 .887 r
.09058 .4112 .12149 .44629 .16478 .43124 .1342 .39565 Metetra
.65 .705 .887 r
.1342 .39565 .16478 .43124 .20881 .41593 .17857 .37982 Metetra
.65 .705 .887 r
.17857 .37982 .20881 .41593 .2536 .40036 .22372 .36372 Metetra
.65 .705 .887 r
.22372 .36372 .2536 .40036 .29918 .38448 .26967 .34733 Metetra
.65 .705 .887 r
.26967 .34733 .29918 .38448 .34556 .36822 .31644 .33065 Metetra
.651 .706 .888 r
.31644 .33065 .34556 .36822 .39278 .35138 .36406 .31366 Metetra
.652 .709 .89 r
.36406 .31366 .39278 .35138 .44084 .33352 .41253 .29628 Metetra
.655 .717 .894 r
.41253 .29628 .44084 .33352 .48975 .31382 .46191 .27824 Metetra
.665 .735 .902 r
.46191 .27824 .48975 .31382 .53946 .2911 .51218 .25878 Metetra
.689 .77 .914 r
.51218 .25878 .53946 .2911 .58981 .26381 .56331 .23624 Metetra
.732 .823 .921 r
.56331 .23624 .58981 .26381 .64056 .2304 .61509 .20803 Metetra
.792 .88 .91 r
.61509 .20803 .64056 .2304 .69134 .18962 .66714 .17122 Metetra
.853 .92 .875 r
.66714 .17122 .69134 .18962 .74173 .14092 .71886 .12377 Metetra
.65 .704 .886 r
.01565 .39089 .0477 .4265 .09058 .4112 .05884 .37508 Metetra
.65 .704 .886 r
.05884 .37508 .09058 .4112 .1342 .39565 .10278 .359 Metetra
.65 .704 .886 r
.10278 .359 .1342 .39565 .17857 .37982 .1475 .34263 Metetra
.65 .704 .886 r
.1475 .34263 .17857 .37982 .22372 .36372 .19301 .32597 Metetra
.65 .704 .886 r
.19301 .32597 .22372 .36372 .26967 .34733 .23934 .30901 Metetra
.65 .704 .886 r
.23934 .30901 .26967 .34733 .31644 .33065 .2865 .29174 Metetra
.65 .704 .886 r
.2865 .29174 .31644 .33065 .36406 .31366 .33453 .27416 Metetra
.65 .705 .887 r
.33453 .27416 .36406 .31366 .41253 .29628 .38344 .25626 Metetra
.65 .705 .887 r
.38344 .25626 .41253 .29628 .46191 .27824 .43327 .23802 Metetra
.651 .708 .889 r
.43327 .23802 .46191 .27824 .51218 .25878 .48402 .21944 Metetra
.652 .718 .897 r
.48402 .21944 .51218 .25878 .56331 .23624 .53574 .20051 Metetra
.654 .746 .918 r
.53574 .20051 .56331 .23624 .61509 .20803 .58845 .18121 Metetra
.649 .816 .963 r
.58845 .18121 .61509 .20803 .66714 .17122 .64218 .16155 Metetra
.652 .946 .957 r
.64218 .16155 .66714 .17122 .71886 .12377 .69654 .13436 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  GroupPageBreakWithin->True,
  AspectRatioFixed->True,
  ImageSize->{174.375, 141.312},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002^0000SA000`40O003h00O[Woo002^Ool00:io
o`00[Woo002^Ool00:ioo`00[Woo002^Ool00:ioo`00[Woo002^Ool006moo`<000Aoo`8003Ioo`00
LWoo00@007ooOol0009oo`03001oogoo03=oo`00LWoo00@007ooOol0009oo`03001oogoo03=oo`00
Kgoo0`000goo00@007ooOol003Eoo`00Kgoo00<007ooOol00goo00@007ooOol003Eoo`00Kgoo1000
0goo0P00=Woo002^Ool00:ioo`00N7oo0P00=7oo001FOol30002Ool3000GOol300000goo001oo`0c
Ool005Moo`03001oogoo00Aoo`03001oogoo019oo`8000Aoo`8003=oo`00EGoo10001Goo00<007oo
Ool03goo0`0000=oo`00Ool00goo00<007oo0000<Woo001EOol00`00Ool00003Ool3000@Ool20009
Ool3000bOol005Eoo`03001oo`0000=oo`03001oogoo00eoo`<000]oo`04001oogoo000aOol005Io
o`8000=oo`@000Yoo`8000ioo`05001oogooOol0000`Ool006Ioo`<0011oo`05001oogooOol0000`
Ool006Aoo`8001=oo`03001oogoo009oo`03001oogoo02eoo`00HGoo0`005Woo00<007ooOol00Woo
00<007ooOol0;7oo001NOol3000IOol00`00Oomoo`02Ool00`00Oomoo`04Ool20004Ool20003Ool4
000IOol003ioo`<000=oo`8001Ioo`80009oo`03001oogoo01Moo`03001oogoo009oo`8000Aoo`04
001oogoo0003Ool20004Ool00`00Oomoo`0IOol003moo`05001oogooOol00002Ool00`00Oomoo`0@
Ool3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`002Woo00<007ooOol067oo000m
Ool40002Ool01000Oomoo`0047oo0P008Goo00<007ooOol017oo00D007ooOomoo`00009oo`03001o
ogoo00Uoo`03001oogoo01Moo`00?Goo00<007oo00000goo00@007ooOol000eoo`@0029oo`03001o
ogoo00Aoo`80009oo`04001oogoo0008Ool01000Oomoo`006Goo000mOol00`00Ool00003Ool01000
Oomoo`002goo0P009Woo00<007ooOol01Goo00@007ooOomoo`8000Yoo`8001Yoo`00?Woo0P0017oo
0P002Goo10009goo00<007ooOol01Woo00<007ooOol0:7oo001=Ool2000[Ool01000Ool000001Woo
00<007ooOol09goo001:Ool4000/Ool300000fnk001oo`03Ool2000YOol002Ioo`<000=oo`<001Uo
o`8002moo`<000=_^`03001oogoo00=oo`03001oogoo02Ioo`00:Goo00<007ooOol00goo00<007oo
Ool04goo0`0000=oo`00Ool0;7oo0P0000=CgP00Kk/016nk00<007ooOol00goo00<007ooOol09Goo
000YOol00`00Oomoo`03Ool00`00Oomoo`0AOol2000`Ool20002Dmh20004Kk/00`00Ki/00004Ool0
0`00Oomoo`0UOol002Moo`8000=oo`<0011oo`@0031oo`8000=CgP03001_^`0000=_^`05001_VfnK
Ki/00004Ool00`00Oomoo`0TOol002Uoo`04001oogoo000AOol3000_Ool50002Dmh01000Kk]_^`00
0Vnk00<006nKKi/00fnK00<007ooOol00Woo00<007ooOol00goo0P0017oo0P0017oo0`004Goo000V
Ool30003Ool4000;Ool3000_Ool30005Deh20003Kk/00`00Kk/00005Ki/00`00Kg/00003Ool00`00
Oomoo`02Ool01000Oomoo`000goo0P001Goo00<007ooOol047oo000iOol3000^Ool30007Deh01000
IieWW@000Vnk0P001FnK00D006mkKg]_N`0000=oo`05001oogooOol00002Ool00`00Oomoo`06Ool4
000AOol003Ioo`<002ioo`<00003D_d005=N00ECGP8000AWW@04001_^`000004Ki/00`00Kg]_N`03
Kg/01@00Oomoogoo00000Woo00@007ooOol000Qoo`03001oo`00019oo`00<goo0`00;Woo0`001E;m
00@005=NDeiCGP8000IWW@800003Ki/006nK009_V`03001_Nfmk00=_N`80009oo`80009oo`04001o
ogoo0008Ool00`00Ool0000BOol0015oo`<000Aoo`8001Moo`<002ioo`8000UBo@03001CGP0000MW
W@03001[O6]l0080009_V`03001_Nfmk00=_N`04001_NVmj0003Ool00`00Oomoo`02000:Ool2000B
Ool001Aoo`04001oogoo0002Ool00`00Oomoo`0AOol3000^Ool300000e;L001Bo@08D_d30005Iid2
0004Jg`00`00Ki/00005Kg/00`00KgY_NP02KgX01@00Oomoogoo00008Goo000DOol01000Oomoo`00
0Woo00<007ooOol03goo0P0000=oo`00Ool0;7oo0P001E;L00<005;mD_d015;m0P000emM00D006NM
IieWW@0000I[O08000E_N`03001_NVmj00=_NP80009oo`03001oogoo01moo`004Woo0P000goo00@0
07ooOol000ioo`<002ioo`<000QBg003001BoE;m009Bo@03001OGEmM00AOG@03001WW@0000I[O080
0003Jg/006mk009_N`03001_NVmj00=_NP04001_NVmj0002Ool00`00Oomoo`0NOol001Aoo`04001o
ogoo0002Ool00`00Oomoo`09Ool3000^Ool30002D]`00`00D]aBg007D]`01000D_d000001emM0P00
1V]l00<006]k00000V]k00<006mk00001Fmj00<006mjKgX00Vmj00<007oo000087oo000AOol30004
Ool2000:Ool3000]Ool30006D]`00`00D]aBg006D]`20008Ged00`00He`00005Jg`01000Jg][N`00
0f]k00<006mjKgX00fmj00<006mjKgX016mj00<007oo00007goo000ROol2000]Ool300000e;L001B
g007D]`00`00D]aBg002D]`30002Fad00`00GeeOG@03Ged20003He`01@00Jga[O6]l00000V]k0P00
0V]k00<006]K00000fmj00<006mjKgX016mj0P000Woo00<007ooOol077oo000OOol3000]Ool20005
D]`00`00D]aBg006D]`01000D]`000001U/M00D005mMGeeOG@0000ISG003001[O00000<00004Jg/0
06]k0003Je/00`00KgX00006KgX01P00KeY_FP00Ool000Ioo`8000Aoo`8000Aoo`8000Yoo`007Woo
0P00;7oo0`0025;L00<005;LD]`01E;L0P002E/M00<005mM00001f=L0`001F]k00<006]KJe/00f]K
00<006mjKgX016mj00<006mJKeX00VmJ00<007oo000017oo00@007ooOol000=oo`8000=oo`04001o
ogoo0009Ool000]oo`<000Aoo`8000=oo`@000=oo`03001oo`0002Qoo`<0009Bg003001Bg5;L00IB
g003001Bg5;L009Bg0<00003E_`005/M00QK7@8000ESG0<000=WG007001[Nf]kJg/006]K00000f]K
0P001Fmj00<006mJKeX00fmJ0P0000=oo`00Ool00Woo00@007ooOol000Qoo`04001oogoo0009Ool0
00aoo`03001oogoo00=oo`8000Aoo`03001oogoo009oo`80009oo`03001oogoo029oo`<000IBg003
001Bg5;L00IBg004001Bg0000005E_`00`00FaeK7@04Fad2000015ll001SG6=L1@001FML00<006]k
00000f]K00<006]K00000V]K00D006mjKgY_NP0000E_FP05001_FP00Ool00003Ool01000Oomoo`00
27oo0`002Woo0005Ool50002Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool01@00Oomoogoo00008Goo
0P0000=Bg000D]`01e;L00<005;LD]`015;L0`0025Kl00<005/MFad00U/M00@005llGcaO?0<0009S
G003001WG6ML00EWG08000A[F`8000A[F`03001_NP0000E_FP04001_FVmJKeX30002Ool01000Oomo
o`0027oo00<007ooOol02Woo000<Ool00`00Oomoo`0;Ool00`00Oomoo`020004Ool00`00Oomoo`0L
Ool20004D]`00`00D]aBg006D]`00`00D]aBg0030002D]`00`00E_aFo007E_`01@00Fad00000Gc`0
0`000Ull00<006=L00001fML00@006MKIe/000=[F`8000E[F`03001_FVmJ00=_FP03001_FVmJ00=_
FP03001oo`00009oo`8000Yoo`<000Uoo`0037oo00<007ooOol027oo00@007ooOol0009oo`03001o
ogoo00Aoo`03001oogoo01Qoo`<000MBg003001Bg5;L00EBg0<000IBg003001Fo5Kl00IFo0D000EO
?0<000EWG08000AWF`03001[F`00009[F`07001[Ff]KJe/006]J00000fmJ00<006mJKeX016mJ00<0
07oo00006Woo000;Ool2000;Ool20003Ool00`00Oomoo`05Ool00`00Oomoo`0DOol30002D]`00`00
D]aBg005D]`00`00D]aBg002D]`300000e;L001Bg007D]`00`00E_aFo002E_`50006Gc`20003Hc`0
1@00IeaWG6ML00001VMK0P000f]K00@006]KJe/000=[FP03001_FP0000I_FP03001_FP00008001Uo
o`007Goo00<007ooOol01Woo00<007ooOol047oo0`001U;L00<005;LD]`01E;L00@005;L000000EB
g003001Bg5;L00IBg003001Fo00000<000=K7003001O?5ll009O?003001S?6<l00AS?003001WG000
00IWF`03001WF`0000A[F`8000E[FP03001_FVmJ00A_FP07001_FVmJKeX007oo000067oo000LOol2
0009Ool00`00Oomoo`0<Ool300000e;L001Bg007D]`00`00D]aBg004D]`20008D]`00`00D]aBg005
D]`40006Fa`01000Gc`000001f<l0P001VMK00D006MKIe]WF`0000=[F`8000A[FP8000E_FP03001_
FVmJ00=_FP8001Qoo`0077oo0P002Woo00<007ooOol02Goo0P001E;L00<005;LD]`01U;L00<005;L
D]`00`0000=Bg000D]`01e;L00<005;LD]`00U;L100000=Fo000Fa`01U/L0P0026<l00<006<l0000
1FMK00<006MKIe/00fMK00<006]K00000V]J00@006]JJeX0009[FP05001_FVmJKeX00005KeX4000G
Ool000Uoo`<000Aoo`8000=oo`<000Eoo`03001oogoo00Uoo`03001oogoo00Ioo`8000QBg003001B
g5;L00ABg0@000EBg003001Bg5;L00IBg003001Bg000008000EFo003001K75/L009K7080009O7003
001S?6<l00=S?08000=S?005001WFfMKIe/00006Ie/20003JeX00`00JeX00004JeX00`00KeX00005
KeX01P00KeY_FP00Ool000Eoo`8000Aoo`8000Aoo`8000=oo`002Woo00<007ooOol00goo0P0017oo
00<007ooOol00goo00<007ooOol02goo00<007ooOol00Woo0`000U;L00<005;LD]`01U;L00<005;L
D]`00`000U;L00<005;LD]`01E;L00<005;LD]`015;L0`002EKl00@005/L000000EO7005001S?6<l
Hc`00006Hc`00`00Ie/00006Ie/01000Ie]WF`000f]J00<006]JJeX016]J00<006mJKeX00fmJ00<0
06mJKeX00VmJ0P0017oo00@007ooOol000=oo`8000=oo`04001oogoo0002Ool000=oo`D0009oo`03
001oogoo00Uoo`03001oogoo00=oo`03001oogoo00aoo`@000IBg003001Bg5;L00EBg0<000IBg003
001Bg5;L00EBg003001Bg5;L00<000=Fo003001Fo5Kl00IFo08000QO7003001S?00000MS?08000EW
F`8000AWF`05001[FP00JeX00004JeX00`00JeX00003KeX00`00KeY_FP03KeX00`00Ool00003Ool0
1000Oomoo`0027oo00@007ooOol0009oo`002Woo00<007ooOol02Goo00<007ooOol00goo00<007oo
Ool02Woo0`0000=Bg000D]`01e;L00<005;LD]`00U;L0P0000=Bg000D]`01e;L00<005;LD]`015;L
0`001eKl00<005KlE_`00UKl0`0000=K7000Ga`01elL0P001V<l0P000V<k00D006MKIe]WF`0000IW
F`80009[FP05001[FV]JJeX00003JeX00`00KeX00005KeX01@00KeX007oo00000Woo00@007ooOol0
00Uoo`8000=oo`002Woo00<007ooOol02Goo00<007ooOol00goo00<007ooOol01goo0`001E;L00<0
05;LD]`01U;L00@005;L000000ABg003001Bg5;L00IBg003001Bg5;L00<00003D]`005Kl00QFo004
001Fo0000005Fa`00`00GaaO7002Ga`30002Gc`00`00HcaS?002Hc`00`00Hc]S>`03Hc/00`00Ie/0
0006Ie/00`00Je/00003JeX00`00JeX00005JeX00`00KeY_FP03KeX01000KeY_FVmJ0P000Woo00@0
07ooOol000Qoo`04001oogoo0002Ool000Uoo`8000Yoo`8000Aoo`8000Ioo`<00003D]`005;L00IB
g003001Bg5;L00ABg0<000MBg003001Bg5;L00EBg0<000EBg003001Fo5Kl00IFo08000QK7004001O
70000006Gc`01000Hc`000001V<k0P001VMK00D006]KJe][F`0000=[FP03001[FV]J00=[FP8000A_
FP03001_FVmJ00=_FP80009oo`8000Yoo`8000=oo`0077oo00@007ooOomoo`<000EBg003001Bg5;L
00EBg003001Bg5;L00<0009Bg003001Bg5;L00EBg003001Bg5;L009Bg0D000MBg003001Fo5Kl009F
o0<00003E_`005/L00MK708000QO?08000MS>`03001W>`0000EWF`03001[Ff]K00=[F`05001[FP00
JeX00003JeX01000JeY[FP000VmJ00<006mJKeX00fmJ10004Woo000KOol00`00Oomoo`0200000e;L
001Bg006D]`00`00D]aBg004D]`30006D]`00`00D]aBg005D]`00`00D]`000020004D]`00`00D]aB
g005D]`01000E_`000001EKl00<005/LFa`015/L00@005/LFa`000IO?004001S?6<l0005Hc/01@00
Ic]W>fLk00000fMK00<006]KJe/016]K0P000V]J00@006]JJeX000=[FP03001_FP0000E_FP03001_
FVmJ0080019oo`006goo0`0015;L00<005;LD]`01E;L00<005;LD]`00`000U;L00<005;LD]`01E;L
00<005;LD]`00e;L0`0025;L00<005;LD]`015;L0P0025Kl00@005/LFaaK708000AK7004001O?5ll
Gc`20004Hc`01@00Hc]S>f<k00001FLk00<006MK00001V]K00@006]KJe/0009[FP8000E[FP03001_
FVmJ00=_FP03001_FVmJ009_FP80015oo`0057oo0`0017oo00<005;LD]`01E;L00<005;LD]`015;L
0`001U;L00<005;LD]`01E;L00<005;L00000`0000=Bg000D]`025;L00<005;LD]`00`0000=Fo000
E_`025Kl00<005/L00001e/L00<005ll00001f<l00<006<k00001VLk0P001F]K0P0016]K00@006]J
000000A[FP03001[FP0000=_FP03001_FVmJ009_FP03001_FP00015oo`005Goo00<007ooOol00Woo
0P0025;L00<005;LD]`00`000U;L00<005;LD]`01E;L00<005;LD]`00e;L10001E;L00<005;LD]`0
1U;L0`001EKl00<005KlE_`01UKl0P0025/L0P0026<l0P001VLk00@006LkIc/000=[F`03001[Ff]K
00A[F`800003JeX006]J009[FP07001[FV]JJeX006mJ000016mJ00D006mJKeY_FP0000Aoo`<000Uo
o`003Woo1@000Woo00<007ooOol00goo0P001e;L0`001U;L00<005;LD]`01E;L00<005;L00000`00
00=Bg000D]`01U;L00<005;LD]`00U;L0`0000=Bg000E_`01UKl00<005KlE_`00UKl0`0000=Jo000
Fa`01E/L0P0000=O7000Hc`01F<l0P0000=S?000Ic/016Lk00<006LkIc/00VLk00<006]K00001V]K
00<006/j00000V]J00<006]J00001F]J00<006mJKeX00VmJ00<006mJKeX00fmJ00<007ooOol00Woo
00<007ooOol027oo000EOol00`00Oomoo`02Ool200000goo001Bg003D]`200000e;L001Bg007D]`0
0`00D]aBg003D]`40005D]`00`00D]aBg005D]`01000D]`000001E;L00<005KlE_`01EKl00@005Kl
000000EJo003001K75/L009K7003001O75lL009O7003001S?6<l009S?003001S?6<l009S?005001W
>fLkIc/00005Ic/20006Je/01@00JcY[>V/j00000V]J00<006]JJeX00f]J0P000fmJ00<006mJKeX0
16mJ00<007ooOol00Woo00<007ooOol027oo000EOol00`00Oomoo`02Ool01P00Oomoogoo001Bg080
00ABg003001Bg5;L00IBg003001Bg00000<00003D]`005;L00IBg003001Bg5;L00ABg08000QBg003
001Fo5Kl00AFo08000QJo004001K70000006Ga`01000Hc`000001V<l00<006Lk00001FLk00<006Lk
00001F]K00<006/jJcX00V/j00D006]J001[FP0000=[FP06001[>V/j001_FP001VmJ00<006li0000
0goo00<007ooOol027oo000DOol20004Ool00`00Oomoo`02Ool20006D]`00`00D]aBg004D]`40005
D]`00`00D]aBg005D]`00`00D]aBg0030002D]`00`00D]aBg006D]`00`00E_aFo0030002E_`00`00
F_aJo005F_`20008Ga`20008Hc`20005Ic/01@00Ic]W>fLk00000f]K00<006/jJcX00f/j0P000V]J
00@006]JJeX000A[>P03001_FVmJ00A_FP03001_>Fli0080009oo`03001oogoo00Qoo`006Woo00<0
07ooOol017oo00<005;LD]`015;L00<005;LD]`0100000=Bg000D]`01U;L00<005;LD]`015;L0`00
1U;L00<005;LD]`015;L10001UKl00<005[lF_`00U[l0P000U/L00<005lLGa`015lL00@005lLGa`0
00IS?004001S>f<k0003Ic/00`00Ic]W>`03Ic/00`00Je/00005JcX01000JcY[>P000V]J0P001F/j
00<006mJKeX00fmJ00<006liKcT00Vli00@007oo000000Yoo`006Woo00<007ooOol01Goo00<005;L
D]`00e;L10001E;L00<005;LD]`01E;L00<005;LD]`00`0000=Bg000D]`01e;L00<005;LD]`00`00
0UKl00<005KlE_`01UKl00@005[l000000EK7004001O75lLGa`20004Ga`01000HcaS?6<l0P0016<k
00<006Lk00001VLk0P001F/j00<006/jJcX00V/j00@006]J000000A[>P03001[>P0000=_FP03001_
>Fli009_>@8000eoo`006Woo00<007ooOol01Woo00<005;L00000`0000=Bg000D]`01U;L00<005;L
D]`015;L0`001E;L00<005;LD]`01E;L0`001UKl00<005KlE_`01EKl0P0025/L00<005lL00001elL
00<006<l00001V<k0P001VLk00@006LkIc/000=[>P03001[>V/j00=[>P80009[>P04001[>V/j0003
JcX00`00KeX00004KcT00`00Ool0000=Ool001Uoo`8000Qoo`<000EBg003001Bg5;L00EBg003001B
g5;L00<00003D]`005;L00IBg003001Bg5;L009Bg0800003E]`005Kl00MFo004001Fo5KlE_`30000
0eKl001K7007Fa`20008Ga`20006Hc/00`00Hc/00005Ic/00`00Ic]W>`02Ic/00`00JcX00005JcX0
2000JcY[>P00JcX006/j0005JcX00`00KcU_>@02KcT01000Oomoo`003Goo0008Ool20004Ool20003
Ool30004Ool00`00Oomoo`03Ool300000goo001Bg006D]`00`00D]aBg004D]`30005D]`00`00D]aB
g005D]`01000D]`0000015KL00<005KlE_`01EKl00@005Kl000000EFo003001K75/L009K70<00003
Fa`005lL00EO70800003Ga`006<k00AS>`05001S>f<kHc/00003Ic/00`00Ic]W>`03Ic/20005JcX0
0`00JcY[>P02JcX00`00JcX00005JcX20003KcT00`00Oomoo`02Ool00`00Oomoo`0:Ool000Moo`04
001oogoo0003Ool20006Ool01000Oomoo`000goo0`001Goo00<005;LD]`01E;L00<005;LD]`00`00
00=Bg000D]`01U;L00<005;LD]`015;L0P001eKL00<005KlE_`015Kl0P0025Kl00@005/L000000EK
7003001O75lL009O7003001O75lL009O7005001S>f<kHc/00005Hc/00`00Ic/00005Ic/01000Ic]W
>`000f/j00<006/jJcX016/j00<006/j00000f/j00H006/jJcX006li0005Ool00`00Oomoo`0:Ool0
0003Ool0000000<00005Ool007ooOol00009Ool30002Ool40009Ool00`00D]aBg004D]`30005D]`0
0`00D]aBg005D]`00`00D]aBg0030002D]`00`00E]aFg005E]`00`00E_aFo0030002E_`00`00E_aF
o005E_`20008Fa`01000Ga`000001UlL00<006<k00001V<k0P001FLk00<006LkIc/00VLk00<006/j
00001V/j0P0000E[>P00JcY[>P0000A[>P03001oogoo00Aoo`03001oogoo00Yoo`001goo00@007oo
Ool000Qoo`04001oogoo0002Ool2000<Ool00`00D]aBg00300000e;L001Bg006D]`00`00D]aBg004
D]`30006D]`00`00E]aFg004E]`30006E_`00`00E_aFo002E_`20002F_`00`00FaaK7005Fa`20008
Ga`20006Hc/01000Hc]S>`000fLk00<006LkIc/00fLk0P001V/j00D006/jJcX006/j008000A[>P03
001oogoo00Eoo`03001oogoo00Yoo`001goo00@007ooOol000Qoo`04001oogoo0004Ool00`00Oomo
o`0:Ool20005D]`00`00D]aBg005D]`00`00D]aBg0030002D]`00`00D]aBg005D]`00`00E]aFg003
0002E]`00`00E_aFo005E_`01000E_`000001E[l00<005/LFa`00U/L0P0000=O7000Ga`01ElL0P00
0V<K00@006<kHc]S>`8000AS>`03001W>`0000EW>`03001W>`0000E[>P06001[>V/jJcX006/j0P00
0f/j00<007ooOol01Woo00<007ooOol02Woo0008Ool2000:Ool20006Ool00`00Oomoo`09Ool00`00
D]aBg005D]`00`00D]aBg004D]`30006D]`00`00D]aBg004D]`30006E]`00`00E_aFo004E_`20008
F_`01000Fa`0000015lL00<005lLGa`00UlL00<006<KHa/00f<K00<006<k00001V<k0P001FLk00D0
06LkIc]W>`0000=[>P03001[>V/j009[>P80009[>P03001[>P0000Uoo`03001oogoo00Yoo`007Goo
00<007ooOol02Goo00<005;LD]`01E;L00<005;LD]`00`000U;L00<005;LD]`01E;L00<005;LD]`0
0`0000=Fg000E]`01eKL00<005KlE_`00`0000=Fo000F_`01e[l0P001elL00@005lL000000IS6`80
00IS>`04001W>fLk0003Ic/00`00Ic]W>`03Ic/00`00JcX00004JcX00`00JcX00002JcX2000:Ool0
0`00Oomoo`0:Ool001ioo`8000Yoo`03001Bg5;L00ABg0<000IBg003001Bg5;L00ABg0<000EFg003
001Fg5KL00EFg0<000EFo003001Jo5[l009Jo0<00003F_`005lL00IO70<000IS6`03001S6`0000ES
>`03001W>fLk009W>`03001W>`0000IW>`8000A[>P06001[>V/jJcX006/j0P002Woo00<007ooOol0
2Woo000POol00`00Oomoo`08Ool00`00D]aBg0030002D]`00`00D]aBg005D]`00`00D]aBg0030000
0e;L001Fg006E]`00`00E]aFg002E]`200000eKl001Fo006E_`01000F_`000001E[l00<005lLGa`0
0UlL0P000elL00@006<KHa]S6`8000=S6`05001S>f<kHc/00005Ic/20006Ic/00`00Ic/00003JcX0
0`00JcY[>P03JcX01000Oomoo`002Woo00<007ooOol02Goo000QOol00`00Oomoo`08Ool20006D]`0
0`00D]aBg004D]`30005D]`00`00E]aFg005E]`01000E]`0000015Kl00<005KlE_`015Kl0P0025[l
00@005lL000000IO7003001S6`0000IS6`03001S>`0000EW>`03001W>`0000EW>`07001W>fLkIc/0
06/j00001F/j00D007ooOomoo`0000Yoo`03001oogoo00Uoo`008Woo00<007ooOol027oo00<005;L
D]`01E;L00<005;LD]`00`0000=Bg000D]`01U;L00<005KLE]`00eKL0`001eKl00<005KlE_`00`00
0U[l00<005[lF_`01E[l0P0025lL0P001f<K0P001FLk00D006LkIc]W>`0000=W>`03001W>fLk009W
>`8000E[>P03001oogoo00=oo`03001oogoo00Moo`03001oogoo00Uoo`008goo00<007ooOol027oo
00<005;LD]`015;L0`001E;L00<005;LD]`015;L00<005KLE]`00`000UKL00<005KlE_`01EKl0`00
1U[l00<005[lF_`00U[l0P0000=K7000Ga`01ElL0P0000=O7000Ha/01F<K00@006<KHa/000=W>`03
001W>fLk00=W>`03001W>`0000AW>`04001W>VLj0003JcX00`00Oomoo`05Ool00`00Oomoo`06Ool0
0`00Oomoo`09Ool002Aoo`8000Uoo`03001Bg5;L00<0009Bg003001Bg5;L00ABg003001Bg5;L00=B
g0<000IFg003001Fo5Kl009Fo080009Fo003001Jo5[l00EJo004001Jo0000004Fa`00`00GaaO7002
Ga`00`00GaaO7002Ga`01000Ha]S6f<K0P0016<K00<006Lk00001VLk0P0016Lk00<006LjIcX00VLj
00<006/j00002Goo00<007ooOol01Goo00<007ooOol02Goo000VOol00`00Oomoo`07Ool20006D]`0
0`00D]aBg004D]`00`00D]`000020002D]`00`00E]aFg005E]`01000E_`000001EKl00<005[lF_`0
15[l0P001e/L00@005lL000000IO7003001S6`0000IS6`8000IW>`03001W>`0000=W>`03001W>VLj
00=W>P8000Yoo`03001oogoo00Eoo`03001oogoo00Uoo`009goo00<007ooOol01goo00<005;LD]`0
1E;L00@005;LD]aBg0@000IBg003001Fg5KL00AFg08000QFo003001Jo5[l00<00003F_`005/L00IK
70<000MO708000IS6`04001S>f<k0003Ic/20003Ic/00`00Ic/00005IcX00`00Oomoo`0;Ool00`00
Oomoo`04Ool00`00Oomoo`09Ool002Qoo`03001oogoo00Moo`03001Bg5;L00ABg0@000=Bg003001B
g5;L00EBg003001Fg5KL00<0009Fo003001Fo5Kl00EFo0<000EJo003001K75/L009K708000=O7003
001O75lL009O7080009O6`04001S6f<KHa/20004Hc/00`00Ic/00005Ic/20005IcX00`00Oomoo`0=
Ool00`00Oomoo`04Ool00`00Oomoo`08Ool002Uoo`8000Qoo`03001Bg5;L00<00003D]`005;L00EB
g003001Bg5;L00ABg0<000IFo003001Fo5Kl009Fo0800003E_`005[l00IJo004001K70000006Ga`0
1000Ga`000001ElK00<006<K00001V<k0P001FLk00@006LkIc/000=W>P03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo00Qoo`00:goo00<007ooOol01Woo0P001E;L00<005;LD]`015;L00<005;L
D]`00`000UKL00<005KlE_`01EKl00@005Kl000000AFo003001Jo5[l00AJo0<000MO708000MO6`80
00IS>`03001S>`0000AW>`03001W>fLk009W>`03001W>P00019oo`03001oogoo00=oo`03001oogoo
00Qoo`00;7oo00<007ooOol01Woo00<005;LD]`015;L00<005;LD]`00e;L0`001UKL00<005KlE_`0
15Kl0P001eKl00<005[lF_`00`000e[l00<005lLGa`00UlL0P0000=O7000Ga/01ElK00@005lKGa/0
00=S>`8000=S>`04001W>fLk0005Ic/2000DOol00`00Oomoo`02Ool00`00Oomoo`08Ool002eoo`03
001oogoo00Ioo`03001Bg5;L00=Bg003001Bg5;L00<0009Bg003001Fg5KL00EFg003001Fo5Kl00<0
0003E_`005Kl00IFo0<000MJo004001O70000004Ga`01000Ga]O6elK0P0015lK00<006<k00001F<k
00<006Lk00001FLk00<007ooOol05Goo00D007ooOomoo`0000Yoo`00;Woo00<007ooOol01Woo00@0
05;LD]aBg0@000IBg003001Fg5KL00AFg0<000EFo003001Fo5Kl009Fo0800003F_`005[l00MJo080
00MO7003001O6`0000IO6`8000IS>`8000EW>`03001oogoo01Ioo`05001oogooOol0000:Ool002mo
o`8000Moo`<000=Bg003001Bg5;L00EBg003001Fg5KL00<00003E]`005Kl00IFo004001Fo0000004
F_`00`00F_aJo002F_`300000e[l001O7006Ga`20006Ga/00`00Ha/00005Hc/01000Hc]S>`000fLk
00<007ooOol067oo00@007ooOol000Yoo`00<Goo00<007ooOol017oo00<005;LD]`015;L00<005;L
D]`015;L0`001EKL00<005KlE_`015Kl0P001e[l00@005[l000000EJo003001O75lL009O7080009O
7004001O6elKGa/20003Ha/00`00Hc]S>`020004Hc/00`00Ic/0000LOol00`00Ool0000:Ool0039o
o`03001oogoo00Aoo`03001Bg5;L00ABg003001Bg5;L00<00003E]`005KL00IFg003001Fo5Kl00<0
009Fo003001Jo5[l00AJo08000QJo004001O70000005Ga`00`00Ga/00005Ha/00`00Hc/00006Hc/2
000MOol01000Oomoo`002Goo000cOol00`00Oomoo`04Ool00`00D]aBg003D]`30005E]`00`00E]aF
g004E]`30006E_`01000F_aJo5[l0P000U[l00<005[lF_`01E[l0P001elL0P001V<K0P001V<k00<0
07ooOol07Woo00<007oo00002Goo000dOol20004Ool00`00D]aBg0030002D]`00`00E]aFg004E]`0
0`00E]aFg0030002E_`00`00E_aFo005E_`00`00F_`00005F_`00`00F_aJo002F_`20002Fa`00`00
GaaO7002Ga`20002Ga/01000Ha]S6f<K0P000V<K00@006<kHc]S>`80029oo`8000Uoo`00=Woo00<0
07ooOol00Woo0P001U;L00<005KLE]`00eKL0`001UKl00<005KlE_`015Kl0P001e[l00@005[l0000
00EK7004001O70000005Ga/00`00Ha/00005Ha/00`00Hc/0000SOol30009Ool003Moo`03001oogoo
00=oo`03001Bg5;L00ABg003001Fg0000080009Fg003001Fo5Kl00EFo003001Fo5Kl00<0009Fo003
001Jo5[l00AJo08000MK708000MO6`8000IS6`80021oo`@000aoo`00>7oo00<007ooOol00goo00@0
05;LD]aBg0@000IFg003001Fo5Kl00AFo0<000IFo003001Jo5[l00<0009Jo003001K75/L009K7080
0003Ga`005lK00EO6`03001O6`0000AS6`8001ioo`@0011oo`00>Goo00<007ooOol00goo0`000e;L
00<005KLE]`01EKL00<005KlE_`00`000UKl00<005KlE_`015Kl0`001U[l00@005/L000000AO7004
001O6elKGa/20003Ga/01000Ha/000007Goo0`0057oo000jOol20004Ool00`00D]aBg003D]`00`00
E]aFg004E]`30006E_`00`00E_aFo00300000eKl001Jo006F_`20007Ga`00`00Ga/00005Ga/2000K
Ool4000GOol003aoo`03001oogoo009oo`03001Bg5;L00=Bg003001Fg5KL00<0009Fo003001Fo5Kl
00AFo0<000EFo003001Jo5[l009Jo080009Jo003001O75lL00AO708000EO6`03001oogoo01Moo`@0
01]oo`00?Goo00<007ooOol00Woo00@005;LD]aBg0@000IFo003001Fo5Kl00<00003E_`005Kl00IF
o004001Jo0000005F_`00`00GaaO70030002Ga`01@00Ga]O6elK00005Woo10007goo000nOol00`00
Oomoo`02Ool30004E]`00`00E_aFo003E_`30005E_`00`00E_aFo004E_`20007F_`30006Ga`00`00
Ga/0000COol4000SOol003moo`80009oo`03001Fg5KL00EFg003001Fo0000080009Fo003001Fo5Kl
00AFo003001Fo5Kl00<0009Jo003001Jo5[l009Jo0800003F_`005lL00IO7080015oo`<002Moo`00
@Goo00@007ooOol000IFg08000IFo003001Fo5Kl00=Fo0<000IJo004001Jo0000004F_`00`00GaaO
7002Ga`2000?Ool4000ZOol0049oo`04001oogoo0002E]`300000eKL001Fo006E_`00`00E_`00002
0002E_`00`00F_aJo004F_`20007F_`01000Ga`000003Goo1000;Woo0013Ool00`00Oomoo`020005
E]`00`00E_aFo004E_`20006E_`00`00F_aJo0030002F_`00`00F_aJo004F_`2000;Ool4000bOol0
04Aoo`04001oogoo0006E]`00`00E_aFo0030002E_`00`00E_aFo003E_`30006F_`00`00F_aJo003
0009Ool4000fOol004Eoo`800003Ool005KL00AFg0<000IFo003001Fo0000080009Fo003001Jo5[l
00AJo0<000Qoo`@003Yoo`00Agoo00@007oo001Fg0<0009Fo003001Fo5Kl00AFo08000IFo004001J
o5[lF_`20008Ool3000nOol004Qoo`03001oo`0000IFo003001Fo5Kl00<00003E_`005Kl00IFo003
001Jo00000Ioo`@0045oo`00BGoo00<007oo00001EKl0`001EKl00<005KlE_`015Kl0P000goo1000
AGoo001:Ool300001UKl00000000E_`000MFo003001Fo5Kl00<00003Ool00000008004Uoo`00C7oo
0P001EKl00<005KlE_`015Kl1`00CGoo001=Ool20005E_`00`00E_aFo006001AOol004ioo`8000=F
o0L005Aoo`00Cgoo1`00F7oo001@Ool2001LOol00:ioo`00[Woo002^Ool00:ioo`00[Woo002^Ool0
0:ioo`00[Woo0000\
\>"],
  ImageRangeCache->{{{0, 173.375}, {140.312, 0}} -> {-0.171221, -0.0642591, 
  0.00683276, 0.00683276}}]
}, Open  ]],

Cell["\<\
Gamma is very small and negative, as is shown in the following plot \
(note the vertical axes scale).\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[UpAndOutPutGamma[0, S, 50, 50, 0.25,0.1, 0.15, t],
{S, 30, 50}, {t, 0.001, 1}, PlotRange -> All, PlotPoints -> 30];\
\>", "Input",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.7672 .09664 0 -6.16225 ]
[.7672 .09664 18 2.83775 ]
[.82794 .18966 0 -6.07272 ]
[.82794 .18966 18 2.92728 ]
[.88358 .27486 0 -5.99233 ]
[.88358 .27486 18 3.00767 ]
[.93473 .3532 0 -5.91977 ]
[.93473 .3532 18 3.08023 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.0167 .31158 -30 -2.7816 ]
[.0167 .31158 0 6.2184 ]
[.00766 .36824 -30 -2.82965 ]
[.00766 .36824 0 6.17035 ]
[-0.00171 .42699 -30 -2.87966 ]
[-0.00171 .42699 0 6.12034 ]
[-0.01144 .48795 -6 -2.93176 ]
[-0.01144 .48795 0 6.06824 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.75489 .10119 m
.74873 .10346 L
s
[(0.2)] .7672 .09664 -1 .36939 Mshowa
.81555 .19399 m
.80936 .19615 L
s
[(0.4)] .82794 .18966 -1 .34949 Mshowa
.87113 .27899 m
.8649 .28106 L
s
[(0.6)] .88358 .27486 -1 .33163 Mshowa
.92222 .35714 m
.91596 .35912 L
s
[(0.8)] .93473 .3532 -1 .3155 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(1)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.70561 .0258 m
.70193 .02722 L
s
.72242 .05152 m
.71874 .05292 L
s
.73884 .07664 m
.73515 .07802 L
s
.77057 .12518 m
.76687 .12652 L
s
.7859 .14863 m
.7822 .14996 L
s
.80089 .17156 m
.79718 .17287 L
s
.8299 .21593 m
.82618 .21721 L
s
.84394 .2374 m
.84021 .23867 L
s
.85767 .25842 m
.85394 .25967 L
s
.8843 .29914 m
.88056 .30036 L
s
.8972 .31887 m
.89345 .32008 L
s
.90983 .3382 m
.90608 .3394 L
s
.93435 .37571 m
.93059 .37688 L
s
.94625 .3939 m
.94249 .39506 L
s
.95791 .41174 m
.95414 .41289 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.02896 .3069 m
.03509 .30456 L
s
[(-0.03)] .0167 .31158 1 -0.38187 Mshowa
.01996 .36367 m
.02611 .36139 L
s
[(-0.02)] .00766 .36824 1 -0.37119 Mshowa
.01063 .42254 m
.0168 .42032 L
s
[(-0.01)] -0.00171 .42699 1 -0.36007 Mshowa
.00095 .48363 m
.00715 .48147 L
s
[(0)] -0.01144 .48795 1 -0.3485 Mshowa
.125 Mabswid
.02718 .31809 m
.03086 .31669 L
s
.0254 .32936 m
.02908 .32797 L
s
.0236 .34071 m
.02728 .33933 L
s
.02179 .35215 m
.02547 .35077 L
s
.01812 .37527 m
.02182 .37391 L
s
.01627 .38696 m
.01997 .38561 L
s
.0144 .39873 m
.0181 .39739 L
s
.01252 .41059 m
.01623 .40925 L
s
.00872 .43458 m
.01243 .43325 L
s
.0068 .4467 m
.01051 .44539 L
s
.00487 .45892 m
.00858 .45761 L
s
.00292 .47123 m
.00663 .46993 L
s
.03072 .29578 m
.03439 .29437 L
s
.03247 .28475 m
.03614 .28333 L
s
.0342 .27379 m
.03787 .27237 L
s
.03593 .26291 m
.0396 .26148 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.693 .788 .924 r
.3872 .78709 .39842 .79536 .41715 .78807 .40602 .77984 Metetra
.7 .792 .922 r
.40602 .77984 .41715 .78807 .43599 .78038 .42495 .77219 Metetra
.707 .795 .919 r
.42495 .77219 .43599 .78038 .45494 .77227 .444 .76413 Metetra
.714 .799 .917 r
.444 .76413 .45494 .77227 .474 .76372 .46316 .75562 Metetra
.722 .802 .915 r
.46316 .75562 .474 .76372 .49316 .75472 .48242 .74665 Metetra
.729 .806 .912 r
.48242 .74665 .49316 .75472 .51241 .74526 .50178 .73721 Metetra
.736 .809 .909 r
.50178 .73721 .51241 .74526 .53175 .73533 .52123 .72729 Metetra
.743 .811 .906 r
.52123 .72729 .53175 .73533 .55117 .72494 .54077 .71689 Metetra
.75 .814 .903 r
.54077 .71689 .55117 .72494 .57066 .71408 .56038 .70602 Metetra
.756 .816 .9 r
.56038 .70602 .57066 .71408 .59023 .70278 .58007 .69467 Metetra
.762 .817 .896 r
.58007 .69467 .59023 .70278 .60986 .69103 .59982 .68287 Metetra
.768 .819 .893 r
.59982 .68287 .60986 .69103 .62954 .67886 .61963 .67062 Metetra
.773 .82 .89 r
.61963 .67062 .62954 .67886 .64928 .6663 .63949 .65796 Metetra
.777 .82 .886 r
.63949 .65796 .64928 .6663 .66907 .65336 .65941 .6449 Metetra
.781 .82 .883 r
.65941 .6449 .66907 .65336 .68889 .64007 .67936 .63147 Metetra
.785 .82 .88 r
.67936 .63147 .68889 .64007 .70876 .62646 .69935 .61771 Metetra
.788 .819 .877 r
.69935 .61771 .70876 .62646 .72866 .61256 .71938 .60363 Metetra
.79 .818 .875 r
.71938 .60363 .72866 .61256 .74859 .5984 .73944 .58928 Metetra
.792 .817 .872 r
.73944 .58928 .74859 .5984 .76856 .58401 .75954 .57469 Metetra
.793 .816 .87 r
.75954 .57469 .76856 .58401 .78855 .56943 .77966 .55989 Metetra
.794 .814 .867 r
.77966 .55989 .78855 .56943 .80858 .55469 .79981 .54492 Metetra
.795 .812 .865 r
.79981 .54492 .80858 .55469 .82864 .53981 .81998 .5298 Metetra
.795 .809 .863 r
.81998 .5298 .82864 .53981 .84872 .52483 .84019 .51457 Metetra
.795 .807 .861 r
.84019 .51457 .84872 .52483 .86884 .50977 .86043 .49927 Metetra
.794 .804 .86 r
.86043 .49927 .86884 .50977 .889 .49467 .88071 .4839 Metetra
.794 .801 .858 r
.88071 .4839 .889 .49467 .90919 .47954 .90102 .46852 Metetra
.792 .799 .857 r
.90102 .46852 .90919 .47954 .92943 .46442 .92136 .45312 Metetra
.791 .796 .856 r
.92136 .45312 .92943 .46442 .94971 .44931 .94176 .43775 Metetra
.789 .793 .855 r
.94176 .43775 .94971 .44931 .97003 .43424 .96219 .42242 Metetra
.691 .788 .924 r
.37583 .77868 .3872 .78709 .40602 .77984 .39474 .77148 Metetra
.698 .791 .923 r
.39474 .77148 .40602 .77984 .42495 .77219 .41377 .76389 Metetra
.705 .795 .921 r
.41377 .76389 .42495 .77219 .444 .76413 .43291 .75587 Metetra
.712 .799 .918 r
.43291 .75587 .444 .76413 .46316 .75562 .45217 .74741 Metetra
.72 .802 .916 r
.45217 .74741 .46316 .75562 .48242 .74665 .47153 .73848 Metetra
.727 .806 .913 r
.47153 .73848 .48242 .74665 .50178 .73721 .491 .72907 Metetra
.735 .809 .91 r
.491 .72907 .50178 .73721 .52123 .72729 .51057 .71917 Metetra
.742 .812 .907 r
.51057 .71917 .52123 .72729 .54077 .71689 .53022 .70877 Metetra
.749 .814 .904 r
.53022 .70877 .54077 .71689 .56038 .70602 .54995 .69788 Metetra
.755 .817 .901 r
.54995 .69788 .56038 .70602 .58007 .69467 .56976 .6865 Metetra
.762 .818 .897 r
.56976 .6865 .58007 .69467 .59982 .68287 .58964 .67465 Metetra
.767 .82 .894 r
.58964 .67465 .59982 .68287 .61963 .67062 .60957 .66233 Metetra
.773 .821 .891 r
.60957 .66233 .61963 .67062 .63949 .65796 .62957 .64957 Metetra
.777 .822 .887 r
.62957 .64957 .63949 .65796 .65941 .6449 .64961 .6364 Metetra
.782 .822 .884 r
.64961 .6364 .65941 .6449 .67936 .63147 .66969 .62283 Metetra
.785 .822 .881 r
.66969 .62283 .67936 .63147 .69935 .61771 .68982 .60891 Metetra
.789 .821 .878 r
.68982 .60891 .69935 .61771 .71938 .60363 .70998 .59466 Metetra
.791 .82 .875 r
.70998 .59466 .71938 .60363 .73944 .58928 .73017 .58012 Metetra
.793 .819 .872 r
.73017 .58012 .73944 .58928 .75954 .57469 .75039 .56532 Metetra
.795 .817 .869 r
.75039 .56532 .75954 .57469 .77966 .55989 .77064 .5503 Metetra
.796 .815 .867 r
.77064 .5503 .77966 .55989 .79981 .54492 .79091 .53509 Metetra
.797 .813 .865 r
.79091 .53509 .79981 .54492 .81998 .5298 .81122 .51972 Metetra
.797 .811 .863 r
.81122 .51972 .81998 .5298 .84019 .51457 .83155 .50424 Metetra
.797 .808 .861 r
.83155 .50424 .84019 .51457 .86043 .49927 .85191 .48867 Metetra
.796 .806 .859 r
.85191 .48867 .86043 .49927 .88071 .4839 .8723 .47304 Metetra
.796 .803 .858 r
.8723 .47304 .88071 .4839 .90102 .46852 .89273 .45737 Metetra
.794 .8 .856 r
.89273 .45737 .90102 .46852 .92136 .45312 .91319 .44171 Metetra
.793 .797 .855 r
.91319 .44171 .92136 .45312 .94176 .43775 .9337 .42607 Metetra
.791 .794 .854 r
.9337 .42607 .94176 .43775 .96219 .42242 .95425 .41046 Metetra
.688 .787 .925 r
.36431 .77014 .37583 .77868 .39474 .77148 .38331 .763 Metetra
.695 .791 .924 r
.38331 .763 .39474 .77148 .41377 .76389 .40243 .75546 Metetra
.702 .794 .922 r
.40243 .75546 .41377 .76389 .43291 .75587 .42167 .7475 Metetra
.71 .798 .92 r
.42167 .7475 .43291 .75587 .45217 .74741 .44103 .73908 Metetra
.717 .802 .917 r
.44103 .73908 .45217 .74741 .47153 .73848 .4605 .7302 Metetra
.725 .806 .914 r
.4605 .7302 .47153 .73848 .491 .72907 .48007 .72083 Metetra
.733 .809 .912 r
.48007 .72083 .491 .72907 .51057 .71917 .49975 .71095 Metetra
.74 .812 .909 r
.49975 .71095 .51057 .71917 .53022 .70877 .51952 .70057 Metetra
.747 .815 .905 r
.51952 .70057 .53022 .70877 .54995 .69788 .53937 .68967 Metetra
.754 .817 .902 r
.53937 .68967 .54995 .69788 .56976 .6865 .5593 .67827 Metetra
.761 .82 .898 r
.5593 .67827 .56976 .6865 .58964 .67465 .57931 .66637 Metetra
.767 .821 .895 r
.57931 .66637 .58964 .67465 .60957 .66233 .59937 .65398 Metetra
.773 .822 .891 r
.59937 .65398 .60957 .66233 .62957 .64957 .6195 .64114 Metetra
.778 .823 .888 r
.6195 .64114 .62957 .64957 .64961 .6364 .63967 .62785 Metetra
.782 .823 .885 r
.63967 .62785 .64961 .6364 .66969 .62283 .65989 .61416 Metetra
.786 .823 .881 r
.65989 .61416 .66969 .62283 .68982 .60891 .68015 .60008 Metetra
.79 .823 .878 r
.68015 .60008 .68982 .60891 .70998 .59466 .70044 .58565 Metetra
.793 .822 .875 r
.70044 .58565 .70998 .59466 .73017 .58012 .72076 .57091 Metetra
.795 .821 .872 r
.72076 .57091 .73017 .58012 .75039 .56532 .74111 .5559 Metetra
.797 .819 .869 r
.74111 .5559 .75039 .56532 .77064 .5503 .76149 .54065 Metetra
.798 .817 .867 r
.76149 .54065 .77064 .5503 .79091 .53509 .7819 .52519 Metetra
.799 .815 .865 r
.7819 .52519 .79091 .53509 .81122 .51972 .80233 .50957 Metetra
.799 .813 .862 r
.80233 .50957 .81122 .51972 .83155 .50424 .82279 .49382 Metetra
.799 .81 .86 r
.82279 .49382 .83155 .50424 .85191 .48867 .84327 .47798 Metetra
.798 .807 .859 r
.84327 .47798 .85191 .48867 .8723 .47304 .86379 .46207 Metetra
.798 .804 .857 r
.86379 .46207 .8723 .47304 .89273 .45737 .88433 .44612 Metetra
.796 .801 .856 r
.88433 .44612 .89273 .45737 .91319 .44171 .90492 .43018 Metetra
.795 .798 .854 r
.90492 .43018 .91319 .44171 .9337 .42607 .92554 .41425 Metetra
.793 .795 .853 r
.92554 .41425 .9337 .42607 .95425 .41046 .9462 .39836 Metetra
.686 .786 .926 r
.35263 .76146 .36431 .77014 .38331 .763 .37172 .75437 Metetra
.693 .79 .925 r
.37172 .75437 .38331 .763 .40243 .75546 .39093 .74689 Metetra
.7 .794 .923 r
.39093 .74689 .40243 .75546 .42167 .7475 .41027 .73899 Metetra
.707 .798 .921 r
.41027 .73899 .42167 .7475 .44103 .73908 .42973 .73064 Metetra
.715 .801 .918 r
.42973 .73064 .44103 .73908 .4605 .7302 .4493 .72181 Metetra
.723 .805 .916 r
.4493 .72181 .4605 .7302 .48007 .72083 .46899 .71249 Metetra
.731 .809 .913 r
.46899 .71249 .48007 .72083 .49975 .71095 .48878 .70264 Metetra
.739 .812 .91 r
.48878 .70264 .49975 .71095 .51952 .70057 .50866 .69228 Metetra
.746 .816 .907 r
.50866 .69228 .51952 .70057 .53937 .68967 .52864 .68139 Metetra
.753 .818 .903 r
.52864 .68139 .53937 .68967 .5593 .67827 .5487 .66997 Metetra
.76 .821 .9 r
.5487 .66997 .5593 .67827 .57931 .66637 .56883 .65803 Metetra
.767 .822 .896 r
.56883 .65803 .57931 .66637 .59937 .65398 .58902 .64559 Metetra
.773 .824 .892 r
.58902 .64559 .59937 .65398 .6195 .64114 .60928 .63266 Metetra
.778 .825 .889 r
.60928 .63266 .6195 .64114 .63967 .62785 .62959 .61927 Metetra
.783 .825 .885 r
.62959 .61927 .63967 .62785 .65989 .61416 .64994 .60544 Metetra
.787 .825 .882 r
.64994 .60544 .65989 .61416 .68015 .60008 .67033 .59121 Metetra
.791 .825 .878 r
.67033 .59121 .68015 .60008 .70044 .58565 .69076 .5766 Metetra
.794 .824 .875 r
.69076 .5766 .70044 .58565 .72076 .57091 .71122 .56167 Metetra
.797 .823 .872 r
.71122 .56167 .72076 .57091 .74111 .5559 .73171 .54643 Metetra
.798 .821 .869 r
.73171 .54643 .74111 .5559 .76149 .54065 .75222 .53095 Metetra
.8 .819 .867 r
.75222 .53095 .76149 .54065 .7819 .52519 .77276 .51524 Metetra
.801 .817 .864 r
.77276 .51524 .7819 .52519 .80233 .50957 .79332 .49936 Metetra
.801 .814 .862 r
.79332 .49936 .80233 .50957 .82279 .49382 .81391 .48333 Metetra
.801 .812 .86 r
.81391 .48333 .82279 .49382 .84327 .47798 .83452 .4672 Metetra
.801 .809 .858 r
.83452 .4672 .84327 .47798 .86379 .46207 .85516 .451 Metetra
.8 .806 .856 r
.85516 .451 .86379 .46207 .88433 .44612 .87582 .43476 Metetra
.799 .802 .855 r
.87582 .43476 .88433 .44612 .90492 .43018 .89652 .41852 Metetra
.797 .799 .853 r
.89652 .41852 .90492 .43018 .92554 .41425 .91726 .40229 Metetra
.795 .796 .852 r
.91726 .40229 .92554 .41425 .9462 .39836 .93804 .38611 Metetra
.683 .785 .927 r
.3408 .75263 .35263 .76146 .37172 .75437 .35997 .7456 Metetra
.69 .789 .926 r
.35997 .7456 .37172 .75437 .39093 .74689 .37927 .73819 Metetra
.697 .793 .924 r
.37927 .73819 .39093 .74689 .41027 .73899 .39871 .73036 Metetra
.705 .797 .922 r
.39871 .73036 .41027 .73899 .42973 .73064 .41827 .72207 Metetra
.712 .801 .92 r
.41827 .72207 .42973 .73064 .4493 .72181 .43795 .71331 Metetra
.72 .805 .917 r
.43795 .71331 .4493 .72181 .46899 .71249 .45774 .70403 Metetra
.729 .809 .914 r
.45774 .70403 .46899 .71249 .48878 .70264 .47765 .69424 Metetra
.737 .813 .911 r
.47765 .69424 .48878 .70264 .50866 .69228 .49765 .68391 Metetra
.744 .816 .908 r
.49765 .68391 .50866 .69228 .52864 .68139 .51775 .67303 Metetra
.752 .819 .905 r
.51775 .67303 .52864 .68139 .5487 .66997 .53793 .6616 Metetra
.759 .822 .901 r
.53793 .6616 .5487 .66997 .56883 .65803 .55819 .64964 Metetra
.766 .824 .897 r
.55819 .64964 .56883 .65803 .58902 .64559 .57852 .63715 Metetra
.773 .825 .894 r
.57852 .63715 .58902 .64559 .60928 .63266 .59891 .62414 Metetra
.778 .826 .89 r
.59891 .62414 .60928 .63266 .62959 .61927 .61936 .61065 Metetra
.784 .827 .886 r
.61936 .61065 .62959 .61927 .64994 .60544 .63985 .59669 Metetra
.788 .827 .882 r
.63985 .59669 .64994 .60544 .67033 .59121 .66038 .5823 Metetra
.792 .827 .879 r
.66038 .5823 .67033 .59121 .69076 .5766 .68095 .56752 Metetra
.795 .826 .876 r
.68095 .56752 .69076 .5766 .71122 .56167 .70155 .55238 Metetra
.798 .825 .872 r
.70155 .55238 .71122 .56167 .73171 .54643 .72217 .53693 Metetra
.8 .823 .869 r
.72217 .53693 .73171 .54643 .75222 .53095 .74282 .5212 Metetra
.802 .821 .866 r
.74282 .5212 .75222 .53095 .77276 .51524 .76349 .50524 Metetra
.803 .819 .864 r
.76349 .50524 .77276 .51524 .79332 .49936 .78419 .48908 Metetra
.803 .816 .861 r
.78419 .48908 .79332 .49936 .81391 .48333 .8049 .47277 Metetra
.803 .813 .859 r
.8049 .47277 .81391 .48333 .83452 .4672 .82564 .45634 Metetra
.803 .81 .857 r
.82564 .45634 .83452 .4672 .85516 .451 .84641 .43984 Metetra
.802 .807 .855 r
.84641 .43984 .85516 .451 .87582 .43476 .8672 .42329 Metetra
.801 .804 .854 r
.8672 .42329 .87582 .43476 .89652 .41852 .88802 .40674 Metetra
.799 .8 .852 r
.88802 .40674 .89652 .41852 .91726 .40229 .90888 .3902 Metetra
.797 .797 .851 r
.90888 .3902 .91726 .40229 .93804 .38611 .92977 .37372 Metetra
.68 .784 .928 r
.3288 .74365 .3408 .75263 .35997 .7456 .34806 .73668 Metetra
.687 .788 .927 r
.34806 .73668 .35997 .7456 .37927 .73819 .36746 .72934 Metetra
.694 .792 .925 r
.36746 .72934 .37927 .73819 .39871 .73036 .38698 .72158 Metetra
.702 .796 .923 r
.38698 .72158 .39871 .73036 .41827 .72207 .40664 .71337 Metetra
.71 .8 .921 r
.40664 .71337 .41827 .72207 .43795 .71331 .42643 .70467 Metetra
.718 .805 .919 r
.42643 .70467 .43795 .71331 .45774 .70403 .44633 .69547 Metetra
.726 .809 .916 r
.44633 .69547 .45774 .70403 .47765 .69424 .46635 .68573 Metetra
.734 .813 .913 r
.46635 .68573 .47765 .69424 .49765 .68391 .48647 .67544 Metetra
.743 .816 .91 r
.48647 .67544 .49765 .68391 .51775 .67303 .5067 .66459 Metetra
.751 .82 .906 r
.5067 .66459 .51775 .67303 .53793 .6616 .52701 .65317 Metetra
.758 .823 .903 r
.52701 .65317 .53793 .6616 .55819 .64964 .5474 .64119 Metetra
.766 .825 .899 r
.5474 .64119 .55819 .64964 .57852 .63715 .56786 .62865 Metetra
.772 .827 .895 r
.56786 .62865 .57852 .63715 .59891 .62414 .58839 .61558 Metetra
.778 .828 .891 r
.58839 .61558 .59891 .62414 .61936 .61065 .60898 .60199 Metetra
.784 .829 .887 r
.60898 .60199 .61936 .61065 .63985 .59669 .62961 .58791 Metetra
.789 .829 .883 r
.62961 .58791 .63985 .59669 .66038 .5823 .65028 .57337 Metetra
.793 .829 .88 r
.65028 .57337 .66038 .5823 .68095 .56752 .67099 .55842 Metetra
.797 .828 .876 r
.67099 .55842 .68095 .56752 .70155 .55238 .69173 .54307 Metetra
.8 .827 .873 r
.69173 .54307 .70155 .55238 .72217 .53693 .7125 .52739 Metetra
.802 .825 .869 r
.7125 .52739 .72217 .53693 .74282 .5212 .73329 .51142 Metetra
.804 .823 .866 r
.73329 .51142 .74282 .5212 .76349 .50524 .7541 .49518 Metetra
.805 .821 .864 r
.7541 .49518 .76349 .50524 .78419 .48908 .77493 .47874 Metetra
.806 .818 .861 r
.77493 .47874 .78419 .48908 .8049 .47277 .79577 .46213 Metetra
.806 .815 .859 r
.79577 .46213 .8049 .47277 .82564 .45634 .81664 .4454 Metetra
.805 .812 .857 r
.81664 .4454 .82564 .45634 .84641 .43984 .83754 .42858 Metetra
.804 .809 .855 r
.83754 .42858 .84641 .43984 .8672 .42329 .85845 .41172 Metetra
.803 .805 .853 r
.85845 .41172 .8672 .42329 .88802 .40674 .8794 .39484 Metetra
.801 .802 .851 r
.8794 .39484 .88802 .40674 .90888 .3902 .90037 .37798 Metetra
.8 .798 .85 r
.90037 .37798 .90888 .3902 .92977 .37372 .92139 .36117 Metetra
.678 .783 .929 r
.31664 .7345 .3288 .74365 .34806 .73668 .33599 .7276 Metetra
.684 .786 .928 r
.33599 .7276 .34806 .73668 .36746 .72934 .35547 .72033 Metetra
.691 .791 .926 r
.35547 .72033 .36746 .72934 .38698 .72158 .3751 .71265 Metetra
.699 .795 .925 r
.3751 .71265 .38698 .72158 .40664 .71337 .39486 .70452 Metetra
.707 .8 .923 r
.39486 .70452 .40664 .71337 .42643 .70467 .41475 .69591 Metetra
.715 .804 .92 r
.41475 .69591 .42643 .70467 .44633 .69547 .43476 .68678 Metetra
.723 .809 .918 r
.43476 .68678 .44633 .69547 .46635 .68573 .45489 .67711 Metetra
.732 .813 .915 r
.45489 .67711 .46635 .68573 .48647 .67544 .47513 .66688 Metetra
.741 .817 .911 r
.47513 .66688 .48647 .67544 .5067 .66459 .49548 .65607 Metetra
.749 .82 .908 r
.49548 .65607 .5067 .66459 .52701 .65317 .51592 .64467 Metetra
.757 .824 .904 r
.51592 .64467 .52701 .65317 .5474 .64119 .53644 .63268 Metetra
.765 .826 .9 r
.53644 .63268 .5474 .64119 .56786 .62865 .55704 .62012 Metetra
.772 .828 .896 r
.55704 .62012 .56786 .62865 .58839 .61558 .57771 .60698 Metetra
.778 .83 .892 r
.57771 .60698 .58839 .61558 .60898 .60199 .59844 .59331 Metetra
.784 .831 .888 r
.59844 .59331 .60898 .60199 .62961 .58791 .61921 .57911 Metetra
.79 .831 .884 r
.61921 .57911 .62961 .58791 .65028 .57337 .64003 .56442 Metetra
.794 .831 .88 r
.64003 .56442 .65028 .57337 .67099 .55842 .66089 .54929 Metetra
.798 .831 .876 r
.66089 .54929 .67099 .55842 .69173 .54307 .68177 .53374 Metetra
.802 .829 .873 r
.68177 .53374 .69173 .54307 .7125 .52739 .70268 .51783 Metetra
.804 .828 .869 r
.70268 .51783 .7125 .52739 .73329 .51142 .72361 .5016 Metetra
.806 .826 .866 r
.72361 .5016 .73329 .51142 .7541 .49518 .74456 .48509 Metetra
.807 .823 .863 r
.74456 .48509 .7541 .49518 .77493 .47874 .76553 .46836 Metetra
.808 .82 .86 r
.76553 .46836 .77493 .47874 .79577 .46213 .78652 .45144 Metetra
.808 .817 .858 r
.78652 .45144 .79577 .46213 .81664 .4454 .80752 .43439 Metetra
.808 .814 .856 r
.80752 .43439 .81664 .4454 .83754 .42858 .82854 .41724 Metetra
.807 .811 .854 r
.82854 .41724 .83754 .42858 .85845 .41172 .84959 .40004 Metetra
.806 .807 .852 r
.84959 .40004 .85845 .41172 .8794 .39484 .87066 .38282 Metetra
.804 .803 .85 r
.87066 .38282 .8794 .39484 .90037 .37798 .89176 .36562 Metetra
.802 .799 .849 r
.89176 .36562 .90037 .37798 .92139 .36117 .91289 .34847 Metetra
.675 .781 .93 r
.30431 .72519 .31664 .7345 .33599 .7276 .32375 .71836 Metetra
.681 .785 .929 r
.32375 .71836 .33599 .7276 .35547 .72033 .34332 .71116 Metetra
.688 .789 .928 r
.34332 .71116 .35547 .72033 .3751 .71265 .36304 .70357 Metetra
.695 .794 .926 r
.36304 .70357 .3751 .71265 .39486 .70452 .3829 .69553 Metetra
.703 .799 .924 r
.3829 .69553 .39486 .70452 .41475 .69591 .40289 .68701 Metetra
.712 .803 .922 r
.40289 .68701 .41475 .69591 .43476 .68678 .42301 .67797 Metetra
.721 .808 .919 r
.42301 .67797 .43476 .68678 .45489 .67711 .44326 .66838 Metetra
.729 .813 .916 r
.44326 .66838 .45489 .67711 .47513 .66688 .46362 .65821 Metetra
.738 .817 .913 r
.46362 .65821 .47513 .66688 .49548 .65607 .48409 .64745 Metetra
.747 .821 .91 r
.48409 .64745 .49548 .65607 .51592 .64467 .50466 .63609 Metetra
.756 .824 .906 r
.50466 .63609 .51592 .64467 .53644 .63268 .52532 .62411 Metetra
.764 .827 .902 r
.52532 .62411 .53644 .63268 .55704 .62012 .54606 .61153 Metetra
.771 .83 .898 r
.54606 .61153 .55704 .62012 .57771 .60698 .56687 .59835 Metetra
.778 .832 .894 r
.56687 .59835 .57771 .60698 .59844 .59331 .58774 .5846 Metetra
.785 .833 .889 r
.58774 .5846 .59844 .59331 .61921 .57911 .60866 .57029 Metetra
.79 .834 .885 r
.60866 .57029 .61921 .57911 .64003 .56442 .62963 .55546 Metetra
.795 .834 .881 r
.62963 .55546 .64003 .56442 .66089 .54929 .65063 .54015 Metetra
.8 .833 .877 r
.65063 .54015 .66089 .54929 .68177 .53374 .67167 .5244 Metetra
.803 .832 .873 r
.67167 .5244 .68177 .53374 .70268 .51783 .69272 .50826 Metetra
.806 .83 .869 r
.69272 .50826 .70268 .51783 .72361 .5016 .7138 .49176 Metetra
.808 .828 .866 r
.7138 .49176 .72361 .5016 .74456 .48509 .7349 .47497 Metetra
.81 .826 .863 r
.7349 .47497 .74456 .48509 .76553 .46836 .75601 .45793 Metetra
.811 .823 .86 r
.75601 .45793 .76553 .46836 .78652 .45144 .77713 .44069 Metetra
.811 .82 .857 r
.77713 .44069 .78652 .45144 .80752 .43439 .79827 .4233 Metetra
.81 .816 .855 r
.79827 .4233 .80752 .43439 .82854 .41724 .81942 .40581 Metetra
.81 .812 .853 r
.81942 .40581 .82854 .41724 .84959 .40004 .8406 .38825 Metetra
.808 .809 .851 r
.8406 .38825 .84959 .40004 .87066 .38282 .86179 .37068 Metetra
.807 .804 .849 r
.86179 .37068 .87066 .38282 .89176 .36562 .88302 .35312 Metetra
.804 .8 .848 r
.88302 .35312 .89176 .36562 .91289 .34847 .90427 .33562 Metetra
.672 .78 .931 r
.29182 .71571 .30431 .72519 .32375 .71836 .31134 .70894 Metetra
.678 .784 .93 r
.31134 .70894 .32375 .71836 .34332 .71116 .331 .70182 Metetra
.684 .788 .929 r
.331 .70182 .34332 .71116 .36304 .70357 .35081 .69431 Metetra
.692 .793 .927 r
.35081 .69431 .36304 .70357 .3829 .69553 .37077 .68637 Metetra
.7 .798 .925 r
.37077 .68637 .3829 .69553 .40289 .68701 .39086 .67795 Metetra
.708 .803 .923 r
.39086 .67795 .40289 .68701 .42301 .67797 .4111 .66901 Metetra
.717 .808 .921 r
.4111 .66901 .42301 .67797 .44326 .66838 .43145 .65952 Metetra
.726 .812 .918 r
.43145 .65952 .44326 .66838 .46362 .65821 .45194 .64944 Metetra
.736 .817 .915 r
.45194 .64944 .46362 .65821 .48409 .64745 .47253 .63875 Metetra
.745 .821 .911 r
.47253 .63875 .48409 .64745 .50466 .63609 .49323 .62743 Metetra
.754 .825 .908 r
.49323 .62743 .50466 .63609 .52532 .62411 .51402 .61548 Metetra
.762 .829 .904 r
.51402 .61548 .52532 .62411 .54606 .61153 .5349 .6029 Metetra
.77 .832 .899 r
.5349 .6029 .54606 .61153 .56687 .59835 .55585 .58969 Metetra
.778 .834 .895 r
.55585 .58969 .56687 .59835 .58774 .5846 .57687 .57587 Metetra
.785 .835 .891 r
.57687 .57587 .58774 .5846 .60866 .57029 .59795 .56146 Metetra
.791 .836 .886 r
.59795 .56146 .60866 .57029 .62963 .55546 .61906 .54649 Metetra
.796 .836 .882 r
.61906 .54649 .62963 .55546 .65063 .54015 .64022 .53101 Metetra
.801 .836 .878 r
.64022 .53101 .65063 .54015 .67167 .5244 .66141 .51506 Metetra
.805 .835 .873 r
.66141 .51506 .67167 .5244 .69272 .50826 .68261 .49867 Metetra
.808 .833 .87 r
.68261 .49867 .69272 .50826 .7138 .49176 .70384 .48191 Metetra
.811 .831 .866 r
.70384 .48191 .7138 .49176 .7349 .47497 .72509 .46483 Metetra
.812 .828 .863 r
.72509 .46483 .7349 .47497 .75601 .45793 .74634 .44747 Metetra
.813 .825 .859 r
.74634 .44747 .75601 .45793 .77713 .44069 .76761 .4299 Metetra
.814 .822 .857 r
.76761 .4299 .77713 .44069 .79827 .4233 .78889 .41216 Metetra
.813 .818 .854 r
.78889 .41216 .79827 .4233 .81942 .40581 .81018 .3943 Metetra
.812 .814 .852 r
.81018 .3943 .81942 .40581 .8406 .38825 .83148 .37637 Metetra
.811 .81 .85 r
.83148 .37637 .8406 .38825 .86179 .37068 .85281 .35842 Metetra
.809 .806 .848 r
.85281 .35842 .86179 .37068 .88302 .35312 .87416 .34049 Metetra
.807 .802 .846 r
.87416 .34049 .88302 .35312 .90427 .33562 .89553 .32261 Metetra
.669 .778 .932 r
.27915 .70605 .29182 .71571 .31134 .70894 .29875 .69934 Metetra
.674 .782 .931 r
.29875 .69934 .31134 .70894 .331 .70182 .31851 .6923 Metetra
.681 .787 .93 r
.31851 .6923 .331 .70182 .35081 .69431 .33841 .68489 Metetra
.688 .791 .928 r
.33841 .68489 .35081 .69431 .37077 .68637 .35846 .67705 Metetra
.696 .796 .927 r
.35846 .67705 .37077 .68637 .39086 .67795 .37866 .66874 Metetra
.705 .801 .925 r
.37866 .66874 .39086 .67795 .4111 .66901 .399 .65991 Metetra
.714 .807 .923 r
.399 .65991 .4111 .66901 .43145 .65952 .41947 .65052 Metetra
.723 .812 .92 r
.41947 .65052 .43145 .65952 .45194 .64944 .44007 .64054 Metetra
.733 .817 .917 r
.44007 .64054 .45194 .64944 .47253 .63875 .46079 .62994 Metetra
.742 .822 .914 r
.46079 .62994 .47253 .63875 .49323 .62743 .48162 .6187 Metetra
.752 .826 .91 r
.48162 .6187 .49323 .62743 .51402 .61548 .50255 .60679 Metetra
.761 .83 .906 r
.50255 .60679 .51402 .61548 .5349 .6029 .52357 .59422 Metetra
.769 .833 .901 r
.52357 .59422 .5349 .6029 .55585 .58969 .54467 .58099 Metetra
.777 .836 .897 r
.54467 .58099 .55585 .58969 .57687 .57587 .56584 .56712 Metetra
.785 .837 .892 r
.56584 .56712 .57687 .57587 .59795 .56146 .58706 .55262 Metetra
.792 .839 .887 r
.58706 .55262 .59795 .56146 .61906 .54649 .60834 .53753 Metetra
.797 .839 .883 r
.60834 .53753 .61906 .54649 .64022 .53101 .62965 .52188 Metetra
.803 .839 .878 r
.62965 .52188 .64022 .53101 .66141 .51506 .65099 .50572 Metetra
.807 .838 .874 r
.65099 .50572 .66141 .51506 .68261 .49867 .67235 .4891 Metetra
.81 .836 .87 r
.67235 .4891 .68261 .49867 .70384 .48191 .69374 .47207 Metetra
.813 .834 .866 r
.69374 .47207 .70384 .48191 .72509 .46483 .71513 .45468 Metetra
.815 .831 .862 r
.71513 .45468 .72509 .46483 .74634 .44747 .73654 .43699 Metetra
.816 .828 .859 r
.73654 .43699 .74634 .44747 .76761 .4299 .75795 .41907 Metetra
.816 .825 .856 r
.75795 .41907 .76761 .4299 .78889 .41216 .77937 .40096 Metetra
.816 .821 .853 r
.77937 .40096 .78889 .41216 .81018 .3943 .8008 .38272 Metetra
.815 .817 .851 r
.8008 .38272 .81018 .3943 .83148 .37637 .82224 .3644 Metetra
.814 .812 .849 r
.82224 .3644 .83148 .37637 .85281 .35842 .84369 .34605 Metetra
.812 .808 .847 r
.84369 .34605 .85281 .35842 .87416 .34049 .86517 .32772 Metetra
.81 .803 .845 r
.86517 .32772 .87416 .34049 .89553 .32261 .88667 .30945 Metetra
.666 .777 .932 r
.2663 .6962 .27915 .70605 .29875 .69934 .28599 .68955 Metetra
.671 .781 .932 r
.28599 .68955 .29875 .69934 .31851 .6923 .30584 .68259 Metetra
.677 .785 .931 r
.30584 .68259 .31851 .6923 .33841 .68489 .32583 .67527 Metetra
.684 .79 .93 r
.32583 .67527 .33841 .68489 .35846 .67705 .34598 .66754 Metetra
.692 .795 .928 r
.34598 .66754 .35846 .67705 .37866 .66874 .36628 .65935 Metetra
.701 .8 .927 r
.36628 .65935 .37866 .66874 .399 .65991 .38672 .65064 Metetra
.71 .806 .924 r
.38672 .65064 .399 .65991 .41947 .65052 .40731 .64138 Metetra
.719 .811 .922 r
.40731 .64138 .41947 .65052 .44007 .64054 .42802 .63152 Metetra
.729 .817 .919 r
.42802 .63152 .44007 .64054 .46079 .62994 .44887 .62103 Metetra
.739 .822 .916 r
.44887 .62103 .46079 .62994 .48162 .6187 .46983 .60987 Metetra
.749 .827 .912 r
.46983 .60987 .48162 .6187 .50255 .60679 .4909 .59803 Metetra
.759 .831 .908 r
.4909 .59803 .50255 .60679 .52357 .59422 .51206 .5855 Metetra
.768 .835 .903 r
.51206 .5855 .52357 .59422 .54467 .58099 .53331 .57227 Metetra
.777 .838 .899 r
.53331 .57227 .54467 .58099 .56584 .56712 .55463 .55836 Metetra
.785 .84 .894 r
.55463 .55836 .56584 .56712 .58706 .55262 .57601 .54379 Metetra
.792 .841 .889 r
.57601 .54379 .58706 .55262 .60834 .53753 .59744 .52858 Metetra
.798 .842 .884 r
.59744 .52858 .60834 .53753 .62965 .52188 .61891 .51277 Metetra
.804 .842 .879 r
.61891 .51277 .62965 .52188 .65099 .50572 .64041 .49641 Metetra
.809 .841 .875 r
.64041 .49641 .65099 .50572 .67235 .4891 .66194 .47955 Metetra
.813 .839 .87 r
.66194 .47955 .67235 .4891 .69374 .47207 .68348 .46224 Metetra
.816 .837 .866 r
.68348 .46224 .69374 .47207 .71513 .45468 .70503 .44454 Metetra
.818 .834 .862 r
.70503 .44454 .71513 .45468 .73654 .43699 .72658 .42651 Metetra
.819 .831 .858 r
.72658 .42651 .73654 .43699 .75795 .41907 .74815 .40821 Metetra
.82 .827 .855 r
.74815 .40821 .75795 .41907 .77937 .40096 .76971 .38971 Metetra
.819 .823 .852 r
.76971 .38971 .77937 .40096 .8008 .38272 .79128 .37107 Metetra
.819 .819 .85 r
.79128 .37107 .8008 .38272 .82224 .3644 .81286 .35234 Metetra
.817 .814 .847 r
.81286 .35234 .82224 .3644 .84369 .34605 .83445 .33357 Metetra
.815 .81 .845 r
.83445 .33357 .84369 .34605 .86517 .32772 .85606 .31482 Metetra
.813 .805 .843 r
.85606 .31482 .86517 .32772 .88667 .30945 .87768 .29612 Metetra
.663 .775 .933 r
.25328 .68615 .2663 .6962 .28599 .68955 .27305 .67956 Metetra
.668 .779 .932 r
.27305 .67956 .28599 .68955 .30584 .68259 .29298 .67268 Metetra
.674 .783 .932 r
.29298 .67268 .30584 .68259 .32583 .67527 .31307 .66546 Metetra
.681 .788 .931 r
.31307 .66546 .32583 .67527 .34598 .66754 .33332 .65784 Metetra
.688 .793 .93 r
.33332 .65784 .34598 .66754 .36628 .65935 .35371 .64977 Metetra
.697 .798 .928 r
.35371 .64977 .36628 .65935 .38672 .65064 .37426 .6412 Metetra
.706 .804 .926 r
.37426 .6412 .38672 .65064 .40731 .64138 .39496 .63208 Metetra
.715 .81 .924 r
.39496 .63208 .40731 .64138 .42802 .63152 .41579 .62236 Metetra
.726 .816 .921 r
.41579 .62236 .42802 .63152 .44887 .62103 .43676 .61199 Metetra
.736 .822 .918 r
.43676 .61199 .44887 .62103 .46983 .60987 .45785 .60094 Metetra
.746 .827 .914 r
.45785 .60094 .46983 .60987 .4909 .59803 .47906 .58919 Metetra
.756 .832 .91 r
.47906 .58919 .4909 .59803 .51206 .5855 .50036 .57672 Metetra
.766 .836 .906 r
.50036 .57672 .51206 .5855 .53331 .57227 .52176 .56352 Metetra
.776 .84 .901 r
.52176 .56352 .53331 .57227 .55463 .55836 .54323 .5496 Metetra
.784 .842 .896 r
.54323 .5496 .55463 .55836 .57601 .54379 .56477 .53496 Metetra
.792 .844 .891 r
.56477 .53496 .57601 .54379 .59744 .52858 .58637 .51965 Metetra
.799 .845 .885 r
.58637 .51965 .59744 .52858 .61891 .51277 .608 .5037 Metetra
.805 .845 .88 r
.608 .5037 .61891 .51277 .64041 .49641 .62967 .48714 Metetra
.811 .844 .875 r
.62967 .48714 .64041 .49641 .66194 .47955 .65136 .47003 Metetra
.815 .843 .87 r
.65136 .47003 .66194 .47955 .68348 .46224 .67306 .45244 Metetra
.818 .841 .866 r
.67306 .45244 .68348 .46224 .70503 .44454 .69477 .43442 Metetra
.821 .838 .862 r
.69477 .43442 .70503 .44454 .72658 .42651 .71649 .41603 Metetra
.822 .834 .858 r
.71649 .41603 .72658 .42651 .74815 .40821 .7382 .39735 Metetra
.823 .831 .854 r
.7382 .39735 .74815 .40821 .76971 .38971 .75992 .37844 Metetra
.823 .826 .851 r
.75992 .37844 .76971 .38971 .79128 .37107 .78163 .35937 Metetra
.822 .822 .848 r
.78163 .35937 .79128 .37107 .81286 .35234 .80335 .3402 Metetra
.821 .817 .846 r
.80335 .3402 .81286 .35234 .83445 .33357 .82508 .32098 Metetra
.819 .812 .844 r
.82508 .32098 .83445 .33357 .85606 .31482 .84681 .30178 Metetra
.816 .806 .842 r
.84681 .30178 .85606 .31482 .87768 .29612 .86856 .28264 Metetra
.66 .774 .933 r
.24008 .6759 .25328 .68615 .27305 .67956 .25994 .66937 Metetra
.665 .777 .933 r
.25994 .66937 .27305 .67956 .29298 .67268 .27995 .66256 Metetra
.67 .781 .932 r
.27995 .66256 .29298 .67268 .31307 .66546 .30013 .65543 Metetra
.677 .786 .932 r
.30013 .65543 .31307 .66546 .33332 .65784 .32047 .64793 Metetra
.684 .791 .931 r
.32047 .64793 .33332 .65784 .35371 .64977 .34096 .63999 Metetra
.692 .797 .93 r
.34096 .63999 .35371 .64977 .37426 .6412 .36161 .63157 Metetra
.701 .803 .928 r
.36161 .63157 .37426 .6412 .39496 .63208 .38242 .6226 Metetra
.711 .809 .926 r
.38242 .6226 .39496 .63208 .41579 .62236 .40337 .61304 Metetra
.721 .815 .923 r
.40337 .61304 .41579 .62236 .43676 .61199 .42446 .60282 Metetra
.732 .822 .92 r
.42446 .60282 .43676 .61199 .45785 .60094 .44568 .59191 Metetra
.743 .827 .917 r
.44568 .59191 .45785 .60094 .47906 .58919 .46702 .58028 Metetra
.754 .833 .913 r
.46702 .58028 .47906 .58919 .50036 .57672 .48848 .56789 Metetra
.764 .838 .908 r
.48848 .56789 .50036 .57672 .52176 .56352 .51002 .55474 Metetra
.774 .842 .903 r
.51002 .55474 .52176 .56352 .54323 .5496 .53165 .54083 Metetra
.783 .845 .898 r
.53165 .54083 .54323 .5496 .56477 .53496 .55336 .52616 Metetra
.792 .847 .892 r
.55336 .52616 .56477 .53496 .58637 .51965 .57511 .51077 Metetra
.8 .848 .887 r
.57511 .51077 .58637 .51965 .608 .5037 .59692 .49467 Metetra
.807 .849 .882 r
.59692 .49467 .608 .5037 .62967 .48714 .61875 .47793 Metetra
.813 .848 .876 r
.61875 .47793 .62967 .48714 .65136 .47003 .64061 .46058 Metetra
.817 .847 .871 r
.64061 .46058 .65136 .47003 .67306 .45244 .66248 .4427 Metetra
.821 .845 .866 r
.66248 .4427 .67306 .45244 .69477 .43442 .68436 .42434 Metetra
.824 .842 .862 r
.68436 .42434 .69477 .43442 .71649 .41603 .70623 .40559 Metetra
.826 .838 .857 r
.70623 .40559 .71649 .41603 .7382 .39735 .72811 .3865 Metetra
.827 .834 .854 r
.72811 .3865 .7382 .39735 .75992 .37844 .74998 .36716 Metetra
.827 .829 .85 r
.74998 .36716 .75992 .37844 .78163 .35937 .77184 .34763 Metetra
.826 .824 .847 r
.77184 .34763 .78163 .35937 .80335 .3402 .7937 .32799 Metetra
.824 .819 .844 r
.7937 .32799 .80335 .3402 .82508 .32098 .81556 .3083 Metetra
.822 .814 .842 r
.81556 .3083 .82508 .32098 .84681 .30178 .83743 .28862 Metetra
.82 .808 .84 r
.83743 .28862 .84681 .30178 .86856 .28264 .85931 .269 Metetra
.658 .772 .934 r
.22669 .66545 .24008 .6759 .25994 .66937 .24663 .65896 Metetra
.662 .775 .933 r
.24663 .65896 .25994 .66937 .27995 .66256 .26673 .65221 Metetra
.666 .779 .933 r
.26673 .65221 .27995 .66256 .30013 .65543 .287 .64518 Metetra
.672 .783 .933 r
.287 .64518 .30013 .65543 .32047 .64793 .30743 .63779 Metetra
.679 .789 .932 r
.30743 .63779 .32047 .64793 .34096 .63999 .32802 .63 Metetra
.687 .794 .931 r
.32802 .63 .34096 .63999 .36161 .63157 .34877 .62173 Metetra
.696 .801 .93 r
.34877 .62173 .36161 .63157 .38242 .6226 .36969 .61293 Metetra
.706 .807 .928 r
.36969 .61293 .38242 .6226 .40337 .61304 .39075 .60354 Metetra
.716 .814 .926 r
.39075 .60354 .40337 .61304 .42446 .60282 .41197 .5935 Metetra
.727 .821 .923 r
.41197 .5935 .42446 .60282 .44568 .59191 .43332 .58276 Metetra
.739 .827 .919 r
.43332 .58276 .44568 .59191 .46702 .58028 .4548 .57127 Metetra
.75 .834 .915 r
.4548 .57127 .46702 .58028 .48848 .56789 .47639 .559 Metetra
.761 .839 .911 r
.47639 .559 .48848 .56789 .51002 .55474 .49809 .54594 Metetra
.772 .844 .906 r
.49809 .54594 .51002 .55474 .53165 .54083 .51988 .53206 Metetra
.782 .847 .9 r
.51988 .53206 .53165 .54083 .55336 .52616 .54175 .51739 Metetra
.792 .85 .895 r
.54175 .51739 .55336 .52616 .57511 .51077 .56367 .50193 Metetra
.8 .852 .889 r
.56367 .50193 .57511 .51077 .59692 .49467 .58565 .48571 Metetra
.808 .853 .883 r
.58565 .48571 .59692 .49467 .61875 .47793 .60766 .46879 Metetra
.814 .852 .877 r
.60766 .46879 .61875 .47793 .64061 .46058 .62969 .45121 Metetra
.82 .851 .872 r
.62969 .45121 .64061 .46058 .66248 .4427 .65173 .43304 Metetra
.824 .849 .866 r
.65173 .43304 .66248 .4427 .68436 .42434 .67378 .41434 Metetra
.827 .846 .861 r
.67378 .41434 .68436 .42434 .70623 .40559 .69583 .3952 Metetra
.829 .842 .857 r
.69583 .3952 .70623 .40559 .72811 .3865 .71786 .37569 Metetra
.83 .838 .853 r
.71786 .37569 .72811 .3865 .74998 .36716 .73989 .35589 Metetra
.831 .833 .849 r
.73989 .35589 .74998 .36716 .77184 .34763 .76191 .33587 Metetra
.83 .828 .846 r
.76191 .33587 .77184 .34763 .7937 .32799 .78391 .31573 Metetra
.828 .822 .843 r
.78391 .31573 .7937 .32799 .81556 .3083 .80592 .29552 Metetra
.826 .816 .84 r
.80592 .29552 .81556 .3083 .83743 .28862 .82792 .27533 Metetra
.823 .81 .838 r
.82792 .27533 .83743 .28862 .85931 .269 .84993 .25519 Metetra
.655 .77 .934 r
.21312 .65479 .22669 .66545 .24663 .65896 .23314 .64832 Metetra
.659 .773 .934 r
.23314 .64832 .24663 .65896 .26673 .65221 .25333 .64164 Metetra
.663 .777 .934 r
.25333 .64164 .26673 .65221 .287 .64518 .27368 .63469 Metetra
.668 .781 .934 r
.27368 .63469 .287 .64518 .30743 .63779 .2942 .62742 Metetra
.675 .786 .933 r
.2942 .62742 .30743 .63779 .32802 .63 .31489 .61976 Metetra
.682 .792 .932 r
.31489 .61976 .32802 .63 .34877 .62173 .33574 .61166 Metetra
.691 .798 .931 r
.33574 .61166 .34877 .62173 .36969 .61293 .35676 .60305 Metetra
.701 .805 .93 r
.35676 .60305 .36969 .61293 .39075 .60354 .37794 .59385 Metetra
.711 .812 .928 r
.37794 .59385 .39075 .60354 .41197 .5935 .39927 .58401 Metetra
.722 .82 .925 r
.39927 .58401 .41197 .5935 .43332 .58276 .42075 .57346 Metetra
.734 .827 .922 r
.42075 .57346 .43332 .58276 .4548 .57127 .44237 .56215 Metetra
.746 .834 .918 r
.44237 .56215 .4548 .57127 .47639 .559 .46411 .55004 Metetra
.758 .84 .914 r
.46411 .55004 .47639 .559 .49809 .54594 .48596 .5371 Metetra
.77 .846 .909 r
.48596 .5371 .49809 .54594 .51988 .53206 .50791 .5233 Metetra
.781 .85 .903 r
.50791 .5233 .51988 .53206 .54175 .51739 .52994 .50865 Metetra
.791 .854 .897 r
.52994 .50865 .54175 .51739 .56367 .50193 .55204 .49316 Metetra
.801 .856 .891 r
.55204 .49316 .56367 .50193 .58565 .48571 .57419 .47685 Metetra
.809 .857 .885 r
.57419 .47685 .58565 .48571 .60766 .46879 .59638 .45976 Metetra
.816 .857 .879 r
.59638 .45976 .60766 .46879 .62969 .45121 .61859 .44195 Metetra
.822 .856 .873 r
.61859 .44195 .62969 .45121 .65173 .43304 .64082 .42349 Metetra
.827 .854 .867 r
.64082 .42349 .65173 .43304 .67378 .41434 .66304 .40445 Metetra
.831 .851 .861 r
.66304 .40445 .67378 .41434 .69583 .3952 .68526 .3849 Metetra
.833 .847 .856 r
.68526 .3849 .69583 .3952 .71786 .37569 .70746 .36494 Metetra
.835 .842 .852 r
.70746 .36494 .71786 .37569 .73989 .35589 .72965 .34465 Metetra
.835 .837 .848 r
.72965 .34465 .73989 .35589 .76191 .33587 .75182 .32412 Metetra
.834 .831 .844 r
.75182 .32412 .76191 .33587 .78391 .31573 .77398 .30343 Metetra
.833 .825 .841 r
.77398 .30343 .78391 .31573 .80592 .29552 .79613 .28267 Metetra
.83 .819 .838 r
.79613 .28267 .80592 .29552 .82792 .27533 .81826 .26192 Metetra
.827 .812 .836 r
.81826 .26192 .82792 .27533 .84993 .25519 .8404 .24123 Metetra
.653 .769 .934 r
.19935 .64391 .21312 .65479 .23314 .64832 .21946 .63747 Metetra
.656 .771 .934 r
.21946 .63747 .23314 .64832 .25333 .64164 .23973 .63083 Metetra
.66 .775 .934 r
.23973 .63083 .25333 .64164 .27368 .63469 .26017 .62396 Metetra
.664 .779 .934 r
.26017 .62396 .27368 .63469 .2942 .62742 .28078 .61679 Metetra
.67 .783 .934 r
.28078 .61679 .2942 .62742 .31489 .61976 .30156 .60928 Metetra
.677 .789 .934 r
.30156 .60928 .31489 .61976 .33574 .61166 .32252 .60135 Metetra
.685 .796 .933 r
.32252 .60135 .33574 .61166 .35676 .60305 .34364 .59292 Metetra
.695 .803 .932 r
.34364 .59292 .35676 .60305 .37794 .59385 .36492 .58394 Metetra
.705 .81 .93 r
.36492 .58394 .37794 .59385 .39927 .58401 .38637 .57433 Metetra
.717 .818 .928 r
.38637 .57433 .39927 .58401 .42075 .57346 .40798 .56401 Metetra
.729 .826 .925 r
.40798 .56401 .42075 .57346 .44237 .56215 .42973 .55292 Metetra
.741 .834 .922 r
.42973 .55292 .44237 .56215 .46411 .55004 .45161 .541 Metetra
.754 .841 .917 r
.45161 .541 .46411 .55004 .48596 .5371 .47362 .52822 Metetra
.766 .847 .912 r
.47362 .52822 .48596 .5371 .50791 .5233 .49573 .51455 Metetra
.778 .853 .906 r
.49573 .51455 .50791 .5233 .52994 .50865 .51793 .49996 Metetra
.79 .857 .9 r
.51793 .49996 .52994 .50865 .55204 .49316 .54021 .48447 Metetra
.8 .86 .894 r
.54021 .48447 .55204 .49316 .57419 .47685 .56254 .46809 Metetra
.81 .862 .887 r
.56254 .46809 .57419 .47685 .59638 .45976 .58491 .45087 Metetra
.818 .862 .88 r
.58491 .45087 .59638 .45976 .61859 .44195 .60731 .43285 Metetra
.825 .861 .874 r
.60731 .43285 .61859 .44195 .64082 .42349 .62972 .4141 Metetra
.83 .859 .867 r
.62972 .4141 .64082 .42349 .66304 .40445 .65212 .39469 Metetra
.835 .856 .861 r
.65212 .39469 .66304 .40445 .68526 .3849 .67452 .37473 Metetra
.837 .852 .856 r
.67452 .37473 .68526 .3849 .70746 .36494 .6969 .35429 Metetra
.839 .847 .851 r
.6969 .35429 .70746 .36494 .72965 .34465 .71926 .33348 Metetra
.84 .841 .846 r
.71926 .33348 .72965 .34465 .75182 .32412 .74159 .31239 Metetra
.839 .835 .842 r
.74159 .31239 .75182 .32412 .77398 .30343 .7639 .29112 Metetra
.837 .828 .839 r
.7639 .29112 .77398 .30343 .79613 .28267 .78619 .26976 Metetra
.835 .821 .836 r
.78619 .26976 .79613 .28267 .81826 .26192 .80847 .24839 Metetra
.832 .814 .833 r
.80847 .24839 .81826 .26192 .8404 .24123 .83074 .2271 Metetra
.651 .767 .934 r
.18539 .63281 .19935 .64391 .21946 .63747 .20559 .62637 Metetra
.653 .769 .934 r
.20559 .62637 .21946 .63747 .23973 .63083 .22594 .61977 Metetra
.656 .772 .935 r
.22594 .61977 .23973 .63083 .26017 .62396 .24647 .61296 Metetra
.66 .776 .935 r
.24647 .61296 .26017 .62396 .28078 .61679 .26717 .6059 Metetra
.666 .78 .935 r
.26717 .6059 .28078 .61679 .30156 .60928 .28804 .59851 Metetra
.672 .786 .935 r
.28804 .59851 .30156 .60928 .32252 .60135 .30909 .59075 Metetra
.68 .792 .934 r
.30909 .59075 .32252 .60135 .34364 .59292 .33031 .58253 Metetra
.689 .8 .934 r
.33031 .58253 .34364 .59292 .36492 .58394 .35171 .57378 Metetra
.699 .807 .932 r
.35171 .57378 .36492 .58394 .38637 .57433 .37327 .56442 Metetra
.71 .816 .931 r
.37327 .56442 .38637 .57433 .40798 .56401 .395 .55436 Metetra
.722 .825 .928 r
.395 .55436 .40798 .56401 .42973 .55292 .41688 .54353 Metetra
.735 .833 .925 r
.41688 .54353 .42973 .55292 .45161 .541 .43891 .53186 Metetra
.749 .841 .921 r
.43891 .53186 .45161 .541 .47362 .52822 .46106 .51929 Metetra
.762 .849 .916 r
.46106 .51929 .47362 .52822 .49573 .51455 .48334 .50579 Metetra
.775 .855 .91 r
.48334 .50579 .49573 .51455 .51793 .49996 .50571 .49132 Metetra
.788 .86 .904 r
.50571 .49132 .51793 .49996 .54021 .48447 .52817 .47588 Metetra
.8 .864 .897 r
.52817 .47588 .54021 .48447 .56254 .46809 .55068 .45948 Metetra
.81 .867 .89 r
.55068 .45948 .56254 .46809 .58491 .45087 .57324 .44214 Metetra
.819 .868 .882 r
.57324 .44214 .58491 .45087 .60731 .43285 .59583 .42393 Metetra
.827 .867 .875 r
.59583 .42393 .60731 .43285 .62972 .4141 .61843 .4049 Metetra
.834 .865 .868 r
.61843 .4049 .62972 .4141 .65212 .39469 .64103 .38513 Metetra
.839 .862 .861 r
.64103 .38513 .65212 .39469 .67452 .37473 .66361 .36473 Metetra
.842 .858 .855 r
.66361 .36473 .67452 .37473 .6969 .35429 .68617 .34379 Metetra
.844 .852 .85 r
.68617 .34379 .6969 .35429 .71926 .33348 .7087 .32242 Metetra
.845 .846 .845 r
.7087 .32242 .71926 .33348 .74159 .31239 .7312 .30073 Metetra
.844 .839 .84 r
.7312 .30073 .74159 .31239 .7639 .29112 .75367 .27882 Metetra
.843 .832 .836 r
.75367 .27882 .7639 .29112 .78619 .26976 .77611 .2568 Metetra
.84 .825 .833 r
.77611 .2568 .78619 .26976 .80847 .24839 .79853 .23477 Metetra
.836 .817 .83 r
.79853 .23477 .80847 .24839 .83074 .2271 .82093 .21282 Metetra
.649 .766 .934 r
.17124 .6215 .18539 .63281 .20559 .62637 .19151 .61505 Metetra
.651 .768 .934 r
.19151 .61505 .20559 .62637 .22594 .61977 .21196 .60846 Metetra
.653 .77 .935 r
.21196 .60846 .22594 .61977 .24647 .61296 .23257 .6017 Metetra
.657 .773 .935 r
.23257 .6017 .24647 .61296 .26717 .6059 .25336 .59471 Metetra
.661 .777 .935 r
.25336 .59471 .26717 .6059 .28804 .59851 .27432 .58746 Metetra
.667 .783 .935 r
.27432 .58746 .28804 .59851 .30909 .59075 .29546 .57986 Metetra
.674 .789 .935 r
.29546 .57986 .30909 .59075 .33031 .58253 .31678 .57185 Metetra
.682 .796 .935 r
.31678 .57185 .33031 .58253 .35171 .57378 .33828 .56334 Metetra
.692 .804 .934 r
.33828 .56334 .35171 .57378 .37327 .56442 .35996 .55424 Metetra
.703 .813 .933 r
.35996 .55424 .37327 .56442 .395 .55436 .3818 .54448 Metetra
.715 .822 .931 r
.3818 .54448 .395 .55436 .41688 .54353 .40382 .53395 Metetra
.729 .832 .929 r
.40382 .53395 .41688 .54353 .43891 .53186 .42598 .52258 Metetra
.743 .841 .925 r
.42598 .52258 .43891 .53186 .46106 .51929 .44829 .51029 Metetra
.757 .85 .92 r
.44829 .51029 .46106 .51929 .48334 .50579 .47073 .49702 Metetra
.771 .857 .914 r
.47073 .49702 .48334 .50579 .50571 .49132 .49327 .48273 Metetra
.785 .864 .908 r
.49327 .48273 .50571 .49132 .52817 .47588 .51591 .4674 Metetra
.798 .869 .9 r
.51591 .4674 .52817 .47588 .55068 .45948 .53861 .45102 Metetra
.81 .872 .893 r
.53861 .45102 .55068 .45948 .57324 .44214 .56137 .43362 Metetra
.821 .874 .885 r
.56137 .43362 .57324 .44214 .59583 .42393 .58416 .41525 Metetra
.83 .874 .877 r
.58416 .41525 .59583 .42393 .61843 .4049 .60696 .39595 Metetra
.837 .872 .869 r
.60696 .39595 .61843 .4049 .64103 .38513 .62975 .37583 Metetra
.843 .869 .862 r
.62975 .37583 .64103 .38513 .66361 .36473 .65253 .35498 Metetra
.847 .864 .855 r
.65253 .35498 .66361 .36473 .68617 .34379 .67528 .3335 Metetra
.85 .859 .849 r
.67528 .3335 .68617 .34379 .7087 .32242 .69799 .31153 Metetra
.851 .852 .843 r
.69799 .31153 .7087 .32242 .7312 .30073 .72066 .28918 Metetra
.85 .845 .838 r
.72066 .28918 .7312 .30073 .75367 .27882 .74329 .26657 Metetra
.849 .837 .834 r
.74329 .26657 .75367 .27882 .77611 .2568 .76588 .24383 Metetra
.846 .828 .83 r
.76588 .24383 .77611 .2568 .79853 .23477 .78844 .22107 Metetra
.842 .819 .827 r
.78844 .22107 .79853 .23477 .82093 .21282 .81097 .19838 Metetra
.648 .765 .934 r
.15688 .60997 .17124 .6215 .19151 .61505 .17724 .60349 Metetra
.649 .766 .934 r
.17724 .60349 .19151 .61505 .21196 .60846 .19777 .5969 Metetra
.651 .768 .935 r
.19777 .5969 .21196 .60846 .23257 .6017 .21847 .59016 Metetra
.654 .771 .935 r
.21847 .59016 .23257 .6017 .25336 .59471 .23935 .58324 Metetra
.657 .774 .935 r
.23935 .58324 .25336 .59471 .27432 .58746 .2604 .57609 Metetra
.662 .779 .936 r
.2604 .57609 .27432 .58746 .29546 .57986 .28164 .56864 Metetra
.668 .785 .936 r
.28164 .56864 .29546 .57986 .31678 .57185 .30305 .56083 Metetra
.675 .792 .936 r
.30305 .56083 .31678 .57185 .33828 .56334 .32465 .55257 Metetra
.684 .8 .936 r
.32465 .55257 .33828 .56334 .35996 .55424 .34643 .54377 Metetra
.695 .809 .936 r
.34643 .54377 .35996 .55424 .3818 .54448 .3684 .53432 Metetra
.707 .819 .934 r
.3684 .53432 .3818 .54448 .40382 .53395 .39053 .52415 Metetra
.721 .829 .932 r
.39053 .52415 .40382 .53395 .42598 .52258 .41283 .51313 Metetra
.735 .84 .929 r
.41283 .51313 .42598 .52258 .44829 .51029 .43529 .50118 Metetra
.75 .85 .925 r
.43529 .50118 .44829 .51029 .47073 .49702 .45788 .48822 Metetra
.766 .859 .919 r
.45788 .48822 .47073 .49702 .49327 .48273 .4806 .47419 Metetra
.781 .867 .912 r
.4806 .47419 .49327 .48273 .51591 .4674 .50342 .45905 Metetra
.796 .874 .905 r
.50342 .45905 .51591 .4674 .53861 .45102 .52632 .44277 Metetra
.809 .878 .897 r
.52632 .44277 .53861 .45102 .56137 .43362 .54928 .42536 Metetra
.821 .881 .888 r
.54928 .42536 .56137 .43362 .58416 .41525 .57227 .40687 Metetra
.832 .881 .879 r
.57227 .40687 .58416 .41525 .60696 .39595 .59528 .38734 Metetra
.84 .88 .87 r
.59528 .38734 .60696 .39595 .62975 .37583 .61828 .36686 Metetra
.847 .877 .862 r
.61828 .36686 .62975 .37583 .65253 .35498 .64126 .34555 Metetra
.852 .872 .854 r
.64126 .34555 .65253 .35498 .67528 .3335 .66421 .32351 Metetra
.856 .866 .847 r
.66421 .32351 .67528 .3335 .69799 .31153 .68711 .30089 Metetra
.857 .859 .841 r
.68711 .30089 .69799 .31153 .72066 .28918 .70995 .27781 Metetra
.857 .851 .836 r
.70995 .27781 .72066 .28918 .74329 .26657 .73275 .25443 Metetra
.855 .842 .831 r
.73275 .25443 .74329 .26657 .76588 .24383 .75549 .23088 Metetra
.852 .832 .827 r
.75549 .23088 .76588 .24383 .78844 .22107 .7782 .2073 Metetra
.848 .822 .823 r
.7782 .2073 .78844 .22107 .81097 .19838 .80086 .18379 Metetra
.647 .764 .934 r
.14231 .59823 .15688 .60997 .17724 .60349 .16276 .5917 Metetra
.648 .765 .934 r
.16276 .5917 .17724 .60349 .19777 .5969 .18338 .58508 Metetra
.649 .766 .935 r
.18338 .58508 .19777 .5969 .21847 .59016 .20417 .57834 Metetra
.651 .768 .935 r
.20417 .57834 .21847 .59016 .23935 .58324 .22514 .57146 Metetra
.654 .771 .935 r
.22514 .57146 .23935 .58324 .2604 .57609 .24628 .56439 Metetra
.657 .775 .936 r
.24628 .56439 .2604 .57609 .28164 .56864 .26761 .55707 Metetra
.662 .78 .937 r
.26761 .55707 .28164 .56864 .30305 .56083 .28912 .54945 Metetra
.669 .787 .937 r
.28912 .54945 .30305 .56083 .32465 .55257 .31081 .54143 Metetra
.677 .795 .938 r
.31081 .54143 .32465 .55257 .34643 .54377 .3327 .53293 Metetra
.686 .804 .938 r
.3327 .53293 .34643 .54377 .3684 .53432 .35477 .52384 Metetra
.698 .814 .937 r
.35477 .52384 .3684 .53432 .39053 .52415 .37703 .51405 Metetra
.711 .826 .936 r
.37703 .51405 .39053 .52415 .41283 .51313 .39946 .50344 Metetra
.726 .837 .933 r
.39946 .50344 .41283 .51313 .43529 .50118 .42206 .49192 Metetra
.742 .849 .93 r
.42206 .49192 .43529 .50118 .45788 .48822 .44481 .47936 Metetra
.759 .86 .925 r
.44481 .47936 .45788 .48822 .4806 .47419 .4677 .46568 Metetra
.775 .87 .918 r
.4677 .46568 .4806 .47419 .50342 .45905 .4907 .45082 Metetra
.792 .878 .91 r
.4907 .45082 .50342 .45905 .52632 .44277 .51379 .43473 Metetra
.807 .885 .901 r
.51379 .43473 .52632 .44277 .54928 .42536 .53696 .4174 Metetra
.821 .889 .892 r
.53696 .4174 .54928 .42536 .57227 .40687 .56016 .39885 Metetra
.833 .89 .882 r
.56016 .39885 .57227 .40687 .59528 .38734 .58339 .37914 Metetra
.844 .889 .872 r
.58339 .37914 .59528 .38734 .61828 .36686 .60661 .35833 Metetra
.852 .886 .863 r
.60661 .35833 .61828 .36686 .64126 .34555 .6298 .33655 Metetra
.858 .882 .854 r
.6298 .33655 .64126 .34555 .66421 .32351 .65295 .31391 Metetra
.862 .875 .846 r
.65295 .31391 .66421 .32351 .68711 .30089 .67605 .29058 Metetra
.864 .867 .839 r
.67605 .29058 .68711 .30089 .70995 .27781 .69908 .26671 Metetra
.864 .858 .833 r
.69908 .26671 .70995 .27781 .73275 .25443 .72205 .24247 Metetra
.862 .848 .827 r
.72205 .24247 .73275 .25443 .75549 .23088 .74496 .21801 Metetra
.859 .837 .823 r
.74496 .21801 .75549 .23088 .7782 .2073 .7678 .19349 Metetra
.854 .826 .819 r
.7678 .19349 .7782 .2073 .80086 .18379 .7906 .16906 Metetra
.646 .763 .934 r
.12753 .58628 .14231 .59823 .16276 .5917 .14807 .57969 Metetra
.647 .764 .934 r
.14807 .57969 .16276 .5917 .18338 .58508 .16878 .57302 Metetra
.648 .765 .934 r
.16878 .57302 .18338 .58508 .20417 .57834 .18966 .56626 Metetra
.649 .766 .935 r
.18966 .56626 .20417 .57834 .22514 .57146 .21072 .55938 Metetra
.651 .768 .935 r
.21072 .55938 .22514 .57146 .24628 .56439 .23195 .55235 Metetra
.653 .772 .936 r
.23195 .55235 .24628 .56439 .26761 .55707 .25337 .54514 Metetra
.657 .776 .937 r
.25337 .54514 .26761 .55707 .28912 .54945 .27497 .53767 Metetra
.662 .782 .937 r
.27497 .53767 .28912 .54945 .31081 .54143 .29677 .52988 Metetra
.669 .789 .938 r
.29677 .52988 .31081 .54143 .3327 .53293 .31875 .52167 Metetra
.677 .798 .939 r
.31875 .52167 .3327 .53293 .35477 .52384 .34093 .51295 Metetra
.688 .808 .939 r
.34093 .51295 .35477 .52384 .37703 .51405 .3633 .50358 Metetra
.701 .82 .939 r
.3633 .50358 .37703 .51405 .39946 .50344 .38585 .49345 Metetra
.715 .833 .938 r
.38585 .49345 .39946 .50344 .42206 .49192 .40859 .48242 Metetra
.732 .847 .935 r
.40859 .48242 .42206 .49192 .44481 .47936 .43149 .47036 Metetra
.749 .86 .93 r
.43149 .47036 .44481 .47936 .4677 .46568 .45455 .45716 Metetra
.767 .872 .924 r
.45455 .45716 .4677 .46568 .4907 .45082 .47773 .44271 Metetra
.786 .883 .917 r
.47773 .44271 .4907 .45082 .51379 .43473 .50103 .42694 Metetra
.803 .891 .907 r
.50103 .42694 .51379 .43473 .53696 .4174 .5244 .4098 Metetra
.819 .897 .897 r
.5244 .4098 .53696 .4174 .56016 .39885 .54783 .3913 Metetra
.834 .9 .886 r
.54783 .3913 .56016 .39885 .58339 .37914 .57128 .37146 Metetra
.846 .9 .875 r
.57128 .37146 .58339 .37914 .60661 .35833 .59473 .35037 Metetra
.857 .898 .864 r
.59473 .35037 .60661 .35833 .6298 .33655 .61814 .32812 Metetra
.864 .893 .854 r
.61814 .32812 .6298 .33655 .65295 .31391 .64151 .30486 Metetra
.869 .886 .844 r
.64151 .30486 .65295 .31391 .67605 .29058 .66482 .28077 Metetra
.872 .877 .836 r
.66482 .28077 .67605 .29058 .69908 .26671 .68805 .25601 Metetra
.873 .866 .829 r
.68805 .25601 .69908 .26671 .72205 .24247 .71119 .23078 Metetra
.871 .855 .823 r
.71119 .23078 .72205 .24247 .74496 .21801 .73426 .20528 Metetra
.867 .843 .818 r
.73426 .20528 .74496 .21801 .7678 .19349 .75725 .17969 Metetra
.862 .83 .814 r
.75725 .17969 .7678 .19349 .7906 .16906 .78018 .15419 Metetra
.646 .763 .934 r
.11253 .57414 .12753 .58628 .14807 .57969 .13316 .56746 Metetra
.646 .763 .934 r
.13316 .56746 .14807 .57969 .16878 .57302 .15397 .56072 Metetra
.647 .764 .934 r
.15397 .56072 .16878 .57302 .18966 .56626 .17494 .5539 Metetra
.647 .765 .934 r
.17494 .5539 .18966 .56626 .21072 .55938 .19609 .547 Metetra
.648 .766 .935 r
.19609 .547 .21072 .55938 .23195 .55235 .21742 .53998 Metetra
.65 .768 .935 r
.21742 .53998 .23195 .55235 .25337 .54514 .23893 .53282 Metetra
.653 .772 .936 r
.23893 .53282 .25337 .54514 .27497 .53767 .26062 .52547 Metetra
.656 .776 .937 r
.26062 .52547 .27497 .53767 .29677 .52988 .28251 .51787 Metetra
.661 .783 .938 r
.28251 .51787 .29677 .52988 .31875 .52167 .30459 .50994 Metetra
.668 .791 .94 r
.30459 .50994 .31875 .52167 .34093 .51295 .32687 .50157 Metetra
.677 .801 .941 r
.32687 .50157 .34093 .51295 .3633 .50358 .34935 .49265 Metetra
.689 .813 .941 r
.34935 .49265 .3633 .50358 .38585 .49345 .37202 .48304 Metetra
.703 .827 .941 r
.37202 .48304 .38585 .49345 .40859 .48242 .39488 .47259 Metetra
.719 .842 .94 r
.39488 .47259 .40859 .48242 .43149 .47036 .41793 .46114 Metetra
.737 .857 .937 r
.41793 .46114 .43149 .47036 .45455 .45716 .44115 .44855 Metetra
.757 .872 .932 r
.44115 .44855 .45455 .45716 .47773 .44271 .46451 .43467 Metetra
.777 .886 .924 r
.46451 .43467 .47773 .44271 .50103 .42694 .488 .41938 Metetra
.797 .898 .914 r
.488 .41938 .50103 .42694 .5244 .4098 .51159 .4026 Metetra
.816 .906 .903 r
.51159 .4026 .5244 .4098 .54783 .3913 .53524 .38429 Metetra
.833 .911 .891 r
.53524 .38429 .54783 .3913 .57128 .37146 .55893 .36446 Metetra
.848 .913 .878 r
.55893 .36446 .57128 .37146 .59473 .35037 .58262 .34315 Metetra
.861 .911 .865 r
.58262 .34315 .59473 .35037 .61814 .32812 .60628 .32047 Metetra
.871 .906 .853 r
.60628 .32047 .61814 .32812 .64151 .30486 .62988 .29657 Metetra
.877 .899 .843 r
.62988 .29657 .64151 .30486 .66482 .28077 .65341 .27163 Metetra
.881 .889 .833 r
.65341 .27163 .66482 .28077 .68805 .25601 .67684 .24588 Metetra
.882 .877 .825 r
.67684 .24588 .68805 .25601 .71119 .23078 .70017 .21952 Metetra
.881 .864 .818 r
.70017 .21952 .71119 .23078 .73426 .20528 .7234 .19281 Metetra
.877 .849 .812 r
.7234 .19281 .73426 .20528 .75725 .17969 .74654 .16596 Metetra
.871 .835 .808 r
.74654 .16596 .75725 .17969 .78018 .15419 .76959 .1392 Metetra
.646 .763 .934 r
.09731 .5618 .11253 .57414 .13316 .56746 .11803 .55503 Metetra
.646 .763 .934 r
.11803 .55503 .13316 .56746 .15397 .56072 .13893 .5482 Metetra
.646 .763 .934 r
.13893 .5482 .15397 .56072 .17494 .5539 .16 .54131 Metetra
.646 .763 .934 r
.16 .54131 .17494 .5539 .19609 .547 .18125 .53434 Metetra
.647 .764 .934 r
.18125 .53434 .19609 .547 .21742 .53998 .20267 .5273 Metetra
.648 .766 .935 r
.20267 .5273 .21742 .53998 .23893 .53282 .22428 .52014 Metetra
.649 .768 .936 r
.22428 .52014 .23893 .53282 .26062 .52547 .24607 .51286 Metetra
.652 .771 .937 r
.24607 .51286 .26062 .52547 .28251 .51787 .26805 .50539 Metetra
.655 .776 .938 r
.26805 .50539 .28251 .51787 .30459 .50994 .29023 .49767 Metetra
.66 .783 .939 r
.29023 .49767 .30459 .50994 .32687 .50157 .3126 .48963 Metetra
.667 .792 .941 r
.3126 .48963 .32687 .50157 .34935 .49265 .33518 .48114 Metetra
.677 .803 .943 r
.33518 .48114 .34935 .49265 .37202 .48304 .35796 .47207 Metetra
.689 .817 .944 r
.35796 .47207 .37202 .48304 .39488 .47259 .38094 .46227 Metetra
.704 .834 .945 r
.38094 .46227 .39488 .47259 .41793 .46114 .40412 .45155 Metetra
.722 .851 .943 r
.40412 .45155 .41793 .46114 .44115 .44855 .42749 .43972 Metetra
.742 .87 .94 r
.42749 .43972 .44115 .44855 .46451 .43467 .45102 .4266 Metetra
.764 .887 .933 r
.45102 .4266 .46451 .43467 .488 .41938 .47471 .41202 Metetra
.786 .903 .924 r
.47471 .41202 .488 .41938 .51159 .4026 .49851 .39583 Metetra
.808 .916 .911 r
.49851 .39583 .51159 .4026 .53524 .38429 .5224 .37793 Metetra
.829 .924 .897 r
.5224 .37793 .53524 .38429 .55893 .36446 .54633 .35828 Metetra
.848 .928 .882 r
.54633 .35828 .55893 .36446 .58262 .34315 .57028 .3369 Metetra
.864 .928 .867 r
.57028 .3369 .58262 .34315 .60628 .32047 .59419 .31387 Metetra
.876 .923 .853 r
.59419 .31387 .60628 .32047 .62988 .29657 .61805 .28933 Metetra
.886 .915 .84 r
.61805 .28933 .62988 .29657 .65341 .27163 .64181 .26349 Metetra
.892 .904 .829 r
.64181 .26349 .65341 .27163 .67684 .24588 .66546 .23659 Metetra
.894 .89 .819 r
.66546 .23659 .67684 .24588 .70017 .21952 .68899 .20892 Metetra
.893 .875 .812 r
.68899 .20892 .70017 .21952 .7234 .19281 .71239 .18076 Metetra
.888 .858 .805 r
.71239 .18076 .7234 .19281 .74654 .16596 .73567 .1524 Metetra
.881 .84 .8 r
.73567 .1524 .74654 .16596 .76959 .1392 .75885 .12412 Metetra
.646 .762 .934 r
.08186 .54926 .09731 .5618 .11803 .55503 .10268 .54239 Metetra
.646 .762 .934 r
.10268 .54239 .11803 .55503 .13893 .5482 .12367 .53547 Metetra
.646 .763 .934 r
.12367 .53547 .13893 .5482 .16 .54131 .14483 .52848 Metetra
.646 .763 .934 r
.14483 .52848 .16 .54131 .18125 .53434 .16618 .52143 Metetra
.646 .763 .934 r
.16618 .52143 .18125 .53434 .20267 .5273 .1877 .51431 Metetra
.646 .764 .934 r
.1877 .51431 .20267 .5273 .22428 .52014 .20941 .50712 Metetra
.647 .765 .935 r
.20941 .50712 .22428 .52014 .24607 .51286 .2313 .49983 Metetra
.648 .767 .936 r
.2313 .49983 .24607 .51286 .26805 .50539 .25338 .49242 Metetra
.65 .77 .937 r
.25338 .49242 .26805 .50539 .29023 .49767 .27565 .48485 Metetra
.653 .775 .938 r
.27565 .48485 .29023 .49767 .3126 .48963 .29813 .47705 Metetra
.658 .782 .94 r
.29813 .47705 .3126 .48963 .33518 .48114 .3208 .46893 Metetra
.664 .792 .943 r
.3208 .46893 .33518 .48114 .35796 .47207 .34368 .46039 Metetra
.674 .805 .945 r
.34368 .46039 .35796 .47207 .38094 .46227 .36678 .45125 Metetra
.687 .821 .948 r
.36678 .45125 .38094 .46227 .40412 .45155 .39008 .44134 Metetra
.703 .841 .949 r
.39008 .44134 .40412 .45155 .42749 .43972 .41358 .43044 Metetra
.722 .862 .948 r
.41358 .43044 .42749 .43972 .45102 .4266 .43727 .41831 Metetra
.745 .884 .943 r
.43727 .41831 .45102 .4266 .47471 .41202 .46114 .40472 Metetra
.77 .906 .935 r
.46114 .40472 .47471 .41202 .49851 .39583 .48515 .38944 Metetra
.795 .924 .922 r
.48515 .38944 .49851 .39583 .5224 .37793 .50927 .37229 Metetra
.82 .937 .906 r
.50927 .37229 .5224 .37793 .54633 .35828 .53346 .35314 Metetra
.843 .945 .888 r
.53346 .35314 .54633 .35828 .57028 .3369 .55768 .33193 Metetra
.863 .947 .869 r
.55768 .33193 .57028 .3369 .59419 .31387 .58187 .30871 Metetra
.88 .944 .851 r
.58187 .30871 .59419 .31387 .61805 .28933 .606 .28359 Metetra
.894 .936 .835 r
.606 .28359 .61805 .28933 .64181 .26349 .63003 .25679 Metetra
.903 .924 .822 r
.63003 .25679 .64181 .26349 .66546 .23659 .65392 .22859 Metetra
.907 .908 .811 r
.65392 .22859 .66546 .23659 .68899 .20892 .67765 .19934 Metetra
.907 .889 .803 r
.67765 .19934 .68899 .20892 .71239 .18076 .70123 .1694 Metetra
.902 .869 .796 r
.70123 .1694 .71239 .18076 .73567 .1524 .72465 .13916 Metetra
.894 .847 .79 r
.72465 .13916 .73567 .1524 .75885 .12412 .74793 .10897 Metetra
.646 .762 .934 r
.06617 .53654 .08186 .54926 .10268 .54239 .08708 .52956 Metetra
.646 .762 .934 r
.08708 .52956 .10268 .54239 .12367 .53547 .10817 .52253 Metetra
.646 .762 .934 r
.10817 .52253 .12367 .53547 .14483 .52848 .12944 .51544 Metetra
.646 .762 .934 r
.12944 .51544 .14483 .52848 .16618 .52143 .15088 .50829 Metetra
.646 .763 .934 r
.15088 .50829 .16618 .52143 .1877 .51431 .1725 .50107 Metetra
.646 .763 .934 r
.1725 .50107 .1877 .51431 .20941 .50712 .19431 .49379 Metetra
.646 .763 .934 r
.19431 .49379 .20941 .50712 .2313 .49983 .21631 .48644 Metetra
.646 .764 .935 r
.21631 .48644 .2313 .49983 .25338 .49242 .23849 .47901 Metetra
.647 .766 .935 r
.23849 .47901 .25338 .49242 .27565 .48485 .26087 .47148 Metetra
.648 .769 .937 r
.26087 .47148 .27565 .48485 .29813 .47705 .28344 .46381 Metetra
.651 .773 .938 r
.28344 .46381 .29813 .47705 .3208 .46893 .30622 .45595 Metetra
.654 .78 .941 r
.30622 .45595 .3208 .46893 .34368 .46039 .3292 .44782 Metetra
.66 .791 .944 r
.3292 .44782 .34368 .46039 .36678 .45125 .35239 .43929 Metetra
.669 .805 .948 r
.35239 .43929 .36678 .45125 .39008 .44134 .3758 .4302 Metetra
.682 .824 .951 r
.3758 .4302 .39008 .44134 .41358 .43044 .39942 .42033 Metetra
.698 .847 .954 r
.39942 .42033 .41358 .43044 .43727 .41831 .42326 .40941 Metetra
.719 .873 .953 r
.42326 .40941 .43727 .41831 .46114 .40472 .44728 .39715 Metetra
.744 .901 .948 r
.44728 .39715 .46114 .40472 .48515 .38944 .47149 .38322 Metetra
.772 .927 .937 r
.47149 .38322 .48515 .38944 .50927 .37229 .49584 .36732 Metetra
.8 .948 .918 r
.49584 .36732 .50927 .37229 .53346 .35314 .52029 .34918 Metetra
.828 .963 .895 r
.52029 .34918 .53346 .35314 .55768 .33193 .5448 .32862 Metetra
.854 .97 .87 r
.5448 .32862 .55768 .33193 .58187 .30871 .5693 .30557 Metetra
.877 .969 .847 r
.5693 .30557 .58187 .30871 .606 .28359 .59373 .28007 Metetra
.897 .962 .826 r
.59373 .28007 .606 .28359 .63003 .25679 .61805 .25232 Metetra
.912 .949 .81 r
.61805 .25232 .63003 .25679 .65392 .22859 .6422 .22264 Metetra
.921 .931 .798 r
.6422 .22264 .65392 .22859 .67765 .19934 .66617 .19144 Metetra
.923 .909 .789 r
.66617 .19144 .67765 .19934 .70123 .1694 .68992 .15923 Metetra
.919 .883 .782 r
.68992 .15923 .70123 .1694 .72465 .13916 .71348 .12652 Metetra
.91 .856 .776 r
.71348 .12652 .72465 .13916 .74793 .10897 .73685 .09381 Metetra
.646 .762 .934 r
.05024 .52362 .06617 .53654 .08708 .52956 .07125 .51653 Metetra
.646 .762 .934 r
.07125 .51653 .08708 .52956 .10817 .52253 .09243 .50939 Metetra
.646 .762 .934 r
.09243 .50939 .10817 .52253 .12944 .51544 .1138 .50219 Metetra
.646 .762 .934 r
.1138 .50219 .12944 .51544 .15088 .50829 .13534 .49493 Metetra
.646 .762 .934 r
.13534 .49493 .15088 .50829 .1725 .50107 .15707 .4876 Metetra
.646 .762 .934 r
.15707 .4876 .1725 .50107 .19431 .49379 .17898 .48021 Metetra
.646 .763 .934 r
.17898 .48021 .19431 .49379 .21631 .48644 .20109 .47276 Metetra
.646 .763 .934 r
.20109 .47276 .21631 .48644 .23849 .47901 .22338 .46524 Metetra
.646 .763 .934 r
.22338 .46524 .23849 .47901 .26087 .47148 .24586 .45765 Metetra
.646 .764 .935 r
.24586 .45765 .26087 .47148 .28344 .46381 .26855 .44998 Metetra
.647 .766 .936 r
.26855 .44998 .28344 .46381 .30622 .45595 .29143 .44221 Metetra
.648 .77 .938 r
.29143 .44221 .30622 .45595 .3292 .44782 .31451 .4343 Metetra
.65 .776 .94 r
.31451 .4343 .3292 .44782 .35239 .43929 .33781 .42618 Metetra
.654 .786 .944 r
.33781 .42618 .35239 .43929 .3758 .4302 .36132 .41776 Metetra
.661 .8 .949 r
.36132 .41776 .3758 .4302 .39942 .42033 .38505 .40887 Metetra
.672 .821 .955 r
.38505 .40887 .39942 .42033 .42326 .40941 .409 .39926 Metetra
.688 .849 .96 r
.409 .39926 .42326 .40941 .44728 .39715 .43316 .38863 Metetra
.709 .883 .961 r
.43316 .38863 .44728 .39715 .47149 .38322 .45754 .37657 Metetra
.735 .919 .955 r
.45754 .37657 .47149 .38322 .49584 .36732 .4821 .36263 Metetra
.764 .952 .937 r
.4821 .36263 .49584 .36732 .52029 .34918 .50681 .34637 Metetra
.794 .976 .907 r
.50681 .34637 .52029 .34918 .5448 .32862 .53161 .32734 Metetra
.824 .99 .87 r
.53161 .32734 .5448 .32862 .5693 .30557 .55644 .30524 Metetra
.854 .994 .833 r
.55644 .30524 .5693 .30557 .59373 .28007 .58122 .27993 Metetra
.883 .989 .803 r
.58122 .27993 .59373 .28007 .61805 .25232 .60588 .25147 Metetra
.91 .978 .782 r
.60588 .25147 .61805 .25232 .6422 .22264 .63034 .22016 Metetra
.93 .96 .77 r
.63034 .22016 .6422 .22264 .66617 .19144 .65456 .18652 Metetra
.941 .935 .763 r
.65456 .18652 .66617 .19144 .68992 .15923 .6785 .15122 Metetra
.941 .904 .759 r
.6785 .15122 .68992 .15923 .71348 .12652 .70217 .11503 Metetra
.93 .868 .755 r
.70217 .11503 .71348 .12652 .73685 .09381 .72559 .07874 Metetra
.646 .762 .934 r
.03406 .51049 .05024 .52362 .07125 .51653 .05517 .5033 Metetra
.646 .762 .934 r
.05517 .5033 .07125 .51653 .09243 .50939 .07645 .49605 Metetra
.646 .762 .934 r
.07645 .49605 .09243 .50939 .1138 .50219 .09792 .48873 Metetra
.646 .762 .934 r
.09792 .48873 .1138 .50219 .13534 .49493 .11956 .48135 Metetra
.646 .762 .934 r
.11956 .48135 .13534 .49493 .15707 .4876 .14139 .47391 Metetra
.646 .762 .934 r
.14139 .47391 .15707 .4876 .17898 .48021 .16341 .46641 Metetra
.646 .762 .934 r
.16341 .46641 .17898 .48021 .20109 .47276 .18563 .45884 Metetra
.646 .762 .934 r
.18563 .45884 .20109 .47276 .22338 .46524 .20803 .4512 Metetra
.646 .762 .934 r
.20803 .4512 .22338 .46524 .24586 .45765 .23063 .4435 Metetra
.646 .763 .934 r
.23063 .4435 .24586 .45765 .26855 .44998 .25342 .43573 Metetra
.646 .763 .934 r
.25342 .43573 .26855 .44998 .29143 .44221 .27642 .42789 Metetra
.646 .764 .935 r
.27642 .42789 .29143 .44221 .31451 .4343 .29962 .41997 Metetra
.646 .766 .936 r
.29962 .41997 .31451 .4343 .33781 .42618 .32303 .41197 Metetra
.647 .77 .938 r
.32303 .41197 .33781 .42618 .36132 .41776 .34665 .40385 Metetra
.648 .777 .942 r
.34665 .40385 .36132 .41776 .38505 .40887 .37048 .39556 Metetra
.652 .789 .948 r
.37048 .39556 .38505 .40887 .409 .39926 .39454 .38698 Metetra
.658 .81 .956 r
.39454 .38698 .409 .39926 .43316 .38863 .41882 .37791 Metetra
.668 .84 .966 r
.41882 .37791 .43316 .38863 .45754 .37657 .44332 .36802 Metetra
.684 .882 .972 r
.44332 .36802 .45754 .37657 .4821 .36263 .46805 .35684 Metetra
.704 .93 .964 r
.46805 .35684 .4821 .36263 .50681 .34637 .49298 .34372 Metetra
.725 .971 .931 r
.49298 .34372 .50681 .34637 .53161 .32734 .51807 .32788 Metetra
.743 .992 .868 r
.51807 .32788 .53161 .32734 .55644 .30524 .54326 .30855 Metetra
.764 .993 .793 r
.54326 .30855 .55644 .30524 .58122 .27993 .56844 .28505 Metetra
.797 .987 .73 r
.56844 .28505 .58122 .27993 .60588 .25147 .59352 .25696 Metetra
.844 .983 .696 r
.59352 .25696 .60588 .25147 .63034 .22016 .61838 .22431 Metetra
.897 .978 .689 r
.61838 .22431 .63034 .22016 .65456 .18652 .64291 .1876 Metetra
.941 .963 .697 r
.64291 .1876 .65456 .18652 .6785 .15122 .66705 .14776 Metetra
.962 .932 .71 r
.66705 .14776 .6785 .15122 .70217 .11503 .69079 .10607 Metetra
.958 .885 .718 r
.69079 .10607 .70217 .11503 .72559 .07874 .71417 .06394 Metetra
.646 .762 .934 r
.01764 .49717 .03406 .51049 .05517 .5033 .03884 .48986 Metetra
.646 .762 .934 r
.03884 .48986 .05517 .5033 .07645 .49605 .06022 .48249 Metetra
.646 .762 .934 r
.06022 .48249 .07645 .49605 .09792 .48873 .08178 .47506 Metetra
.646 .762 .934 r
.08178 .47506 .09792 .48873 .11956 .48135 .10353 .46757 Metetra
.646 .762 .934 r
.10353 .46757 .11956 .48135 .14139 .47391 .12547 .46001 Metetra
.646 .762 .934 r
.12547 .46001 .14139 .47391 .16341 .46641 .1476 .45238 Metetra
.646 .762 .934 r
.1476 .45238 .16341 .46641 .18563 .45884 .16992 .44469 Metetra
.646 .762 .934 r
.16992 .44469 .18563 .45884 .20803 .4512 .19243 .43693 Metetra
.646 .762 .934 r
.19243 .43693 .20803 .4512 .23063 .4435 .21514 .4291 Metetra
.646 .762 .934 r
.21514 .4291 .23063 .4435 .25342 .43573 .23806 .42121 Metetra
.646 .762 .934 r
.23806 .42121 .25342 .43573 .27642 .42789 .26118 .41324 Metetra
.646 .762 .934 r
.26118 .41324 .27642 .42789 .29962 .41997 .2845 .4052 Metetra
.646 .763 .934 r
.2845 .4052 .29962 .41997 .32303 .41197 .30803 .39709 Metetra
.646 .763 .934 r
.30803 .39709 .32303 .41197 .34665 .40385 .33178 .38891 Metetra
.646 .764 .935 r
.33178 .38891 .34665 .40385 .37048 .39556 .35573 .38065 Metetra
.646 .767 .937 r
.35573 .38065 .37048 .39556 .39454 .38698 .37991 .37231 Metetra
.646 .772 .94 r
.37991 .37231 .39454 .38698 .41882 .37791 .40431 .36388 Metetra
.646 .784 .948 r
.40431 .36388 .41882 .37791 .44332 .36802 .42893 .35532 Metetra
.646 .807 .96 r
.42893 .35532 .44332 .36802 .46805 .35684 .45378 .34649 Metetra
.647 .848 .977 r
.45378 .34649 .46805 .35684 .49298 .34372 .47886 .33711 Metetra
.641 .906 .983 r
.47886 .33711 .49298 .34372 .51807 .32788 .50417 .32654 Metetra
.607 .948 .923 r
.50417 .32654 .51807 .32788 .54326 .30855 .52968 .31367 Metetra
.529 .906 .74 r
.52968 .31367 .54326 .30855 .56844 .28505 .55533 .29687 Metetra
.55533 .29687 .56844 .28505 .59352 .25696 .58098 .27413 Metetra
.58098 .27413 .59352 .25696 .61838 .22431 .60644 .24364 Metetra
.60644 .24364 .61838 .22431 .64291 .1876 .63147 .20449 Metetra
.745 .859 .427 r
.63147 .20449 .64291 .1876 .66705 .14776 .65588 .15739 Metetra
.91 .92 .533 r
.65588 .15739 .66705 .14776 .69079 .10607 .67957 .10469 Metetra
.985 .904 .62 r
.67957 .10469 .69079 .10607 .71417 .06394 .7026 .0499 Metetra
.646 .762 .934 r
.00095 .48363 .01764 .49717 .03884 .48986 .02225 .47621 Metetra
.646 .762 .934 r
.02225 .47621 .03884 .48986 .06022 .48249 .04373 .46872 Metetra
.646 .762 .934 r
.04373 .46872 .06022 .48249 .08178 .47506 .06539 .46117 Metetra
.646 .762 .934 r
.06539 .46117 .08178 .47506 .10353 .46757 .08724 .45356 Metetra
.646 .762 .934 r
.08724 .45356 .10353 .46757 .12547 .46001 .10928 .44588 Metetra
.646 .762 .934 r
.10928 .44588 .12547 .46001 .1476 .45238 .13152 .43813 Metetra
.646 .762 .934 r
.13152 .43813 .1476 .45238 .16992 .44469 .15395 .43031 Metetra
.646 .762 .934 r
.15395 .43031 .16992 .44469 .19243 .43693 .17658 .42242 Metetra
.646 .762 .934 r
.17658 .42242 .19243 .43693 .21514 .4291 .19941 .41447 Metetra
.646 .762 .934 r
.19941 .41447 .21514 .4291 .23806 .42121 .22244 .40644 Metetra
.646 .762 .934 r
.22244 .40644 .23806 .42121 .26118 .41324 .24568 .39834 Metetra
.646 .762 .934 r
.24568 .39834 .26118 .41324 .2845 .4052 .26913 .39017 Metetra
.646 .762 .934 r
.26913 .39017 .2845 .4052 .30803 .39709 .29279 .38193 Metetra
.646 .762 .934 r
.29279 .38193 .30803 .39709 .33178 .38891 .31666 .37361 Metetra
.646 .762 .934 r
.31666 .37361 .33178 .38891 .35573 .38065 .34075 .36521 Metetra
.646 .762 .934 r
.34075 .36521 .35573 .38065 .37991 .37231 .36506 .35674 Metetra
.646 .762 .934 r
.36506 .35674 .37991 .37231 .40431 .36388 .38959 .34819 Metetra
.646 .763 .934 r
.38959 .34819 .40431 .36388 .42893 .35532 .41436 .33956 Metetra
.646 .764 .935 r
.41436 .33956 .42893 .35532 .45378 .34649 .43935 .33085 Metetra
.646 .768 .937 r
.43935 .33085 .45378 .34649 .47886 .33711 .46457 .32206 Metetra
.645 .779 .945 r
.46457 .32206 .47886 .33711 .50417 .32654 .49003 .31319 Metetra
.641 .806 .962 r
.49003 .31319 .50417 .32654 .52968 .31367 .51573 .30423 Metetra
.618 .863 .989 r
.51573 .30423 .52968 .31367 .55533 .29687 .54167 .29519 Metetra
.464 .886 .914 r
.54167 .29519 .55533 .29687 .58098 .27413 .56786 .28607 Metetra
.006 0 0 r
.56786 .28607 .58098 .27413 .60644 .24364 .59429 .27685 Metetra
.295 0 .103 r
.59429 .27685 .60644 .24364 .63147 .20449 .62099 .26755 Metetra
.394 .063 .281 r
.62099 .26755 .63147 .20449 .65588 .15739 .64793 .25807 Metetra
.357 .075 .342 r
.64793 .25807 .65588 .15739 .67957 .10469 .67432 .23194 Metetra
0 0 .118 r
.67432 .23194 .67957 .10469 .7026 .0499 .69108 .04035 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{174.375, 141.313},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002^0000SA000`40O003h00O[Woo002^Ool00:io
o`00[Woo002^Ool00:ioo`00[Woo002^Ool00:ioo`00[Woo002^Ool00:ioo`00[Woo001`Ool30004
Ool2000eOol007=oo`04001oogoo0002Ool00`00Oomoo`0bOol007=oo`04001oogoo0002Ool00`00
Oomoo`0bOol0071oo`<000=oo`04001oogoo000dOol0071oo`03001oogoo00=oo`04001oogoo000d
Ool0071oo`@000=oo`8003Eoo`00[Woo002^Ool007Uoo`8003=oo`00F7oo0`000Woo0`005Woo0`00
00=oo`00Ool0<Woo001IOol00`00Oomoo`04Ool00`00Oomoo`0AOol20004Ool2000bOol005Moo`@0
00Eoo`03001oogoo00ioo`<00003Ool007oo00=oo`03001oo`00035oo`00Egoo00<007oo00000goo
0`003Woo0`002Goo00<007oo0000<Goo001GOol00`00Ool00003Ool00`00Oomoo`0<Ool20003Ool0
0`00Oomoo`06Ool01000Oomoo`00<7oo001HOol20003Ool40008Ool4000=Ool20002Ool00`00Oomo
o`0]Ool006Moo`80015oo`0500000`00Ool0000_Ool006Aoo`@0019oo`0300000`00009oo`03001o
ogoo02aoo`00HWoo0P005Woo0P0000EoT`00Oomoo`0000Eoo`8000Aoo`8000=oo`@001Uoo`00@Goo
0`000goo0P005Woo1@005Woo0P0000EoT`00001oo`0000Aoo`04001oogoo0003Ool20004Ool00`00
Oomoo`0IOol0049oo`05001oogooOol00002Ool00`00Oomoo`0AOol2000KOol30002NiH30003Ool0
1000Oomoo`002Woo00<007ooOol067oo0010Ool40002Ool01000Oomoo`0047oo0`0000=oo`00Ool0
6Woo0P000g^F00@007ooOol0009oo`04001oogoo000;Ool00`00Oomoo`0GOol0041oo`03001oo`00
00=oo`04001oogoo000=Ool3000OOol30002NiH01@00MgP007oo00000Woo00@007ooOol000Qoo`04
001oogoo000IOol0041oo`03001oo`0000=oo`04001oogoo000;Ool20002Ool00`00Oomoo`0MOol3
000017^F001gN7Mh10000Woo0P002Woo0P006Woo0011Ool20004Ool20009Ool4000SOol3000017^F
001gN0000WMh00<007oo0000:Goo001@Ool2000VOol200001G^f000007Mh00000gMh00<007oo0000
:Goo001=Ool4000WOol2000017^f000000000gMh00<007=i00000P00:7oo000[Ool30003Ool3000G
Ool2000ZOol01P00;4X007^f001kV0800004MgP007=iLgT3000XOol002ioo`03001oogoo00=oo`03
001oogoo015oo`<00003Ool007oo02Uoo`03000/BP0000<0009gV@04001cNG=i0002LeT2000WOol0
02ioo`03001oogoo00=oo`03001oogoo00moo`<002ioo`03000/BP00008000=gV@03001cN@00009c
F@@002Ioo`00;7oo0P000goo0`003goo0`00<7oo00H002a:;4X007^H0002MiT01P00LgT00000LeT0
009_F@8002Ioo`00;Woo00@007ooOol000ioo`@0039oo`03000/BRa:00<00004MiT007=i0002LgT0
1@00KeU_F@00KeX00P001Woo0P0017oo0P0017oo0`0047oo000[Ool30003Ool40009Ool2000eOol2
00000ba:0000000300000g=i001cN@02LgT00`00KeT00003KeX20004Ool01000Oomoo`000goo0P00
1Goo00<007ooOol03goo000kOol300000goo001oo`0dOol200001ba:000007Ni001gV@0000=cN@03
001cN@0000=_FP<000Aoo`04001oogoo0008Ool4000@Ool003Uoo`<003Qoo`80008/BP@00005MiT0
0000LgT00002LgT2000016mJ001_FVmJ0P000goo00@007ooOol000Qoo`03001oo`00015oo`00=Woo
0`00>Woo00<0031800000Ra:0P000WNI00@007=iLgT0009cN@04001_FP000002KeX200001Goo001o
ogoo00000Woo00<007ooOol01Woo00<007oo00004Goo000FOol30004Ool2000EOol200000goo001o
o`0jOol00`00<4P00002;4X2000017NI000007=i0P0000QcN@00KeX006mJ001_FP000VmJ0P000goo
0P002Woo0P004Goo000IOol01000Oomoo`000Woo00<007ooOol03goo0`00?Woo00@00318<4P0008/
BP04001g^@00LgT20002LgX00`00KeX00003KeX01@00KeY_FP00JeX00P008Goo000IOol01000Oomo
o`000Woo00<007ooOol03Goo0P000Woo00<007ooOol0?7oo00@00318<4P0008/BP05001g^@00LgT0
0002LgX00`00KgX00003KeX200000fmJ001[FP02JeX2000POol001Moo`8000=oo`04001oogoo000<
Ool40011Ool20002<4P00`00;4X/BP0200001G>J000007=j00000Vmj0P0000E_FP00KeY_FP0000=[
FP<0021oo`006Goo00@007ooOol0009oo`03001oogoo00Moo`<004Aoo`03000T0`00008`B007000/
BRa:001cVP00LiX00P000Vmj00@006mjKgX0009_FP05001_FP00JeX00002JeX2000OOol001Ioo`<0
00Aoo`8000Qoo`80009oo`03001oogoo049oo`05000T0`00<4P00002;4X00`00LiX00002LiX20000
16mj001_NVmj0P0000=_FP00KeX00VmJ00<006]JJeX010007Woo000TOol40017Ool00`0090<00002
<4P00`00;4X000020002LiX200001Fmj000006mj00000VmJ00@006mJKeY_FP800003JeX006]J00<0
00Ioo`8000Aoo`8000Aoo`8000Yoo`008goo0P00BGoo0P0000@T0`00<4P`B0<00003LiX0000000@0
009_NP09001_FVmJ001_FP00KeX006]J00800003JeX006/j009[>P03001oogoo009oo`04001oogoo
0003Ool20003Ool01000Oomoo`002Goo000SOol00`00Ool00017Ool2000290<00`00<4P000040002
LiX01@00KgX00000KgX00P0000=_FP00KeX00VmJ00<006]J00000V]J00@006/jJcY[>P<000=oo`04
001oogoo0008Ool01000Oomoo`002Goo000SOol01000Oomoo`00AGoo2@0000IcVP00LiYcVP00KiX2
00001Fmj001_NVmj00000fmJ00<006]J00000V]J00D006]J001[>P00009[>P05001oogooOol00002
Ool00`00Oomoo`06Ool3000:Ool0029oo`8000=oo`03001oogoo045oo`P0009c^P800004LiX006nJ
0002KgX02@00KgX00000KgX006mJ001[FP0200000f]J001[FP02JeX00`00JcY[>P030002Ool01000
Oomoo`0027oo00<007ooOol02Woo000ROol20004Ool00`00Oomoo`0oOol600001G>j001c^W>j0000
0W>J00<006nJ00000fmj00<006mj00000fmj00<006]J00000V]J00@006]JJeY[FP800004JcX006/j
JcX20002Ool2000:Ool30009Ool0029oo`03001oogoo00Aoo`03001oogoo03aoo`D0009cn`<00007
LkX007>JLiX006nK00000fmj00T006mj001_NP00KgX006mJ00000V]J00D006]J001[FP00009[FP03
001[>V/j00<001Yoo`008Woo00<007ooOol01Goo00<007ooOol0>7oo1@0000A_nP00Lo]cn`80009c
^`80000?LiX006nK001_N`00KgX006mj001_NP00KgX006mJ00800003JeX006]J009[FP03001[FV]J
00800003JcX006/j00<001Uoo`008Woo00<007ooOol01Woo00<007ooOol0<goo2`0000=cn`000000
0W>k00H006nK000006nK0003Kg/00`00KgX00002KgX40002JeX01000JeY[FV]J0P0000=[FP00JeX0
0P0000I[>P00JcX007oo000HOol000Yoo`8000Aoo`8000Aoo`8000=oo`<000Aoo`03001oogoo00Mo
o`03001oogoo02moo`X00003Jo/006ok00@00004Lk/006nK0002Ki/01P00Kg]_Nfmk001_N`800004
KgX006mkKg/200001f]J001[F`00JeX006]J00800005JeX006/kJc/00003JcX2000HOol000Uoo`04
001oogoo0003Ool20003Ool01000Oomoo`001Goo00<007ooOol00P002Woo00<007ooOol0:goo1P00
00aC?P00000005ok001[n`00Ko/006oK0002Kk/00`00Ki/00002Ki/01`00Ki/006mk001_N`00009_
N`03001_Nfmk00800004JeX006]KJe/200001F]J001[Ff]K00000V/k00D006/k001[>P00009W>`03
001oogoo01Eoo`000goo1@0000Eoo`00Oomoo`0000Qoo`04001oogoo0005Ool00`00Oomoo`02000;
Ool00`00Oomoo`0YOol40002Dah8000016ok001_f`000Vnk00H006nk000006nK0003Ki/00`00Kg/0
0002Kg/200000fmk001[F`050002Je/00`00Je][F`0200000f/k001[>`02Jc/00`00Ic]W>`030005
Ool20004Ool20004Ool20003Ool000Uoo`04001oogoo0008Ool01000Oomoo`000goo0P000goo00<0
07ooOol02goo00<007ooOol09Goo10000U<M0`0000ACW`00GoeOo@80009[g0@00004Kk/006nk0002
Ki/01P00Ki]_VfnK001_N`800007Kg/006mkKg/006]K00000f]K00<006]KJe/00P0000E[F`00Je][
F`0000=[>`800005Ic/006LkIc/00004Ool01000Oomoo`000goo0P000goo00@007ooOol0009oo`00
2Goo00@007ooOol000Qoo`04001oogoo0005Ool01000Oomoo`003Woo00<007ooOol08Goo100000=C
7@00Dad00P000U=o0`0000=Oo@0000000V_L00<006_L00000Vnl00<006nk00000VnK00L006nK001_
V`00Kg/00002Kg/01P00Kg]_N`00Jg/000=[F`@0000;Je/00000001[F`00Je/006/k001W>`020002
Ic/30003Ool01000Oomoo`0027oo00@007ooOol0009oo`002Woo0P002Woo0P000goo0`000goo00<0
07ooOol03Goo00<007ooOol07Goo1P000U<M0`0000=CO`0000000U^n100000A[g000Jm`0009__006
001__000001_V`000VnK0P0000A_N`00Kg]_N`800009Kg/006]k001[F`00Je/006]K00@000=[F`03
001[Ff]K00800007Ic/006Lk001W>`00Ic/00P000goo00@007ooOol000Uoo`8000=oo`008Goo00<0
07ooOol03Woo00<007ooOol067oo1`000U<M0`000U<n0`0000AK_P000000009Wg@D00004Kk`006nl
0002Ki/2000016nK001_VfnK0P0000=_N`00Jg/00P0000=[N`00Je/00V]K00<006]K00000V]K0P00
0V]K0P0000E[F`00Jc][>`00009W>`800003Ic/006Lk0080009oo`04001oogoo0008Ool01000Oomo
o`000Woo0009Ool20004Ool20004Ool20003Ool40002Ool2000AOol2000EOol70002Dad30002Dch0
0`00Dch00002Eil500001VOM001[_6^l001[_0800006Kk`006nKKi/006nK0P000VnK00H006]kJg/0
06]k0002Jg/01`00Je][Ff]K001[F`00009[F`05001[F`00Je/00002Je/00`00Jc][>`0200001FLk
001W>fLk00000VLk0`000Woo0P002Woo0P000goo0008Ool01000Oomoo`000goo0P000goo00@007oo
Ool000=oo`03001oogoo00=oo`03001oogoo015oo`03001oogoo00moo`H0009C7@@0009C7P03001C
?U<n00<00006Eil005nnGkh006>m0`0000A[_000Jk`0009[_0800008Ki/006nK001[V`00Ki/0009[
N`03001[N`00009[N`06001[N`00Je/006]K0P0000A[F`00Je][F`80009[F`800003Jc/006Lk0080
009W>`800005Ic/006LkIc/0000COol0009oo`D00005Ool007ooOol00008Ool01000Oomoo`0017oo
00D007ooOomoo`0001Eoo`03001oogoo00]oo`H0009C7@<000=C7@03001C7U<N00<0009GGP<00004
Gkh006>m0002Ikd200001F^l001[_6^l00000V^L00<006nK00000V^K0`0000=[N`00Jg/00P0000A[
N`00Jg][N`800004Je/006]KJe/200000f]K001[F`0300002V]K000006Lk001W>`00Ic/006Lk0P00
0VLk0`004Woo0008Ool01000Oomoo`0027oo00@007ooOol000Eoo`04001oogoo000FOol00`00Oomo
o`09Ool40002Dad30002Dad00`00DaeC7@040002Ech30002Fih4000016Nm000006Nm0`0000I[_000
Jia[W000Ji`2000016^K001[Nf]k0P0000A[N`00Jg][N`800004Jg/006]KJe/200001F]K001[Ff]K
00000V]K0P0000E[F`00Ie]WF`00009W>`800006Ic/006Lk001W>`000VLk0P004Goo0008Ool01000
Oomoo`0027oo00@007ooOol0009oo`04001oogoo0002Ool00`00Oomoo`0EOol00`00Oomoo`06Ool3
0002Dad30002Dad00`00DaeC7@030002Dch01000Ech000000U]n1@0000ISW@00000006Nm0002Ji`3
000016^L001[W0000V^L0P0000A[N`00Jg][N`800005Jg/006]kJg/00002Je/01000Je][F`000V]K
0P0000=[F`00Je/00P000VMK0P0000EW>`00Ic]W>`0000=W>`800004Ic/00000000AOol000Uoo`80
00Yoo`8000Aoo`8000=oo`03001oogoo01Ioo`03001oogoo009oo`@00004Dad005<M0002Dad00`00
DaeC7@030002Dah00`00DciC?P03000015]n000005mn0`0000ASW@00IieWW@<00004Ji`006^LJi`2
00001F^L001[W6^L00000V]l00@006]kJg/0009[N`03001[Nf]k00800004Je/006]KJe/200000f]K
001[F`0200000f]K001[F`030002Ic/200001fLk001W>`00Ic/006Lk0080009W>`8000Aoo`<000Uo
o`007goo0P006Goo1@0000AC7@00DaeC7@8000=C7@<0009C7P03001C7U<N00<0009GGP<00004Ggh0
05nNGih3000016NM001WWFNM0`0000=[W000Ji`00`0000A[W000Jga[O0800004Jg/006]kJg/20000
0f]k001[N`0300000f]K001[F`0200001F]K001[Ff]K00000f]K0P0000EW>`00Ic]W>`00009W>`80
0007Ic/006Lk001W>`00Ic/00P0017oo00<007ooOol027oo000POol00`00Oomoo`0DOol5000015<M
001C7E<M0`000U<M00<005<M00000U<M00<005<NDah00`000ULn00<005MN00000U]n0`0000AOWP00
HieSW@<00004Iid006NMIid2000016^L001[W6^L0`0000=[O000Jg/00`0000=[N`00Jg/00P0000A[
N`00Je][F`800005Je/006]KJe/00003Je/200000f]K001WF`020002Ic/200000fLk001W>`020002
Ic/2000016Lk001W>`0017oo00<007ooOol027oo0008Ool20004Ool20004Ool20003Ool30003Ool2
000COol5000015<M001C7E<M0`000U<M00<005<MDad00`000U<M0`000U<n00<005LnEch00`0000AK
OP00GgiOOP<00003Hid006>M009SW@800004Iid006NLIi`2000016^L001[O6]l0P0000A[N`00Jg][
N`800004Jg/006]kJg/200001F]K001[Ff]K00000V]K00H006]K001[F`00Je/200001fMK001WF`00
Ic/006Lk00800007Ic/006Lk001W>`00Ic/00P000VLk0P000goo00<007ooOol027oo0007Ool01000
Oomoo`000goo0P000goo00@007ooOol000=oo`03001oogoo009oo`03001oogoo00moo`H0009C7@@0
009C7@03001C7E<M00<0009C7@<0009C7P03001C?U<n00<0009GGP<00004Ggh005mnGgh20003Hid0
1000Iid000000VNL00D006NLIi`006]l0080009[O0800005Jg/006]kJg/00002Jg/01000Jg][N`00
0V]K0P0000A[F`00Je][F`800005Je/006MKIe/00002Ie/200001FLk001W>fLk00000VLk0P0000MW
>`00Ic/006Lk001W>`020002Ool00`00Oomoo`08Ool00003Ool0000000<00005Ool007ooOol00008
Ool01000Oomoo`000goo00<007ooOol00Woo00<007ooOol037oo1P000U<M0`000e<M00<005<MDad0
0`000U<M0`000U<N00<005<NDah00`000ULn0`000U]N0`0000eOOP00HgeSO@00Hid006NM001WW@00
Ii`00002Ii`00`00Ig`00003Jg`01000Jga[O0000V]k0P0000A[N`00Jg][N`800003Je/006]K00<0
0005Je/006]KJe/00002Ie/200000fMK001W>`020002Ic/200001FLk001W>fLk00000VLk0P0000EW
>`00Ool007oo008000Yoo`001goo00@007ooOol000Qoo`04001oogoo0003Ool00`00Oomoo`02Ool0
0`00Oomoo`09Ool60002Dad30003Dad00`00DaeC7@030002Dad00`00Dad00002Dad00`00DaiC7P03
0002Ech00`00Ech00002Eeh00`00Feh00002Ggh30002Hgd2000016NM001WWFNM0`0000QWW000Ig`0
06Ml001[O0000V]l0P0000=[N`00Jg/00`0000=[N`00Je/00P0000=[F`00Je/00V]K00<006]KJe/0
0P0000=WF`00Ie/00P0000MW>`00Ic/006Lk001W>`020002Ic/200000fLk001W>`020002Ool00`00
Oomoo`0;Ool000Moo`04001oogoo0008Ool01000Oomoo`000goo00<007ooOol00Woo00<007ooOol0
1Woo1`0000AC7@00000000=C7@03001C7E<M00<0009C7@03001C7E<M00<0009C7@<0009C7P03001G
?ULn00<0009GGP<00004Ggh005mmGgd300001V=m00000000Iid0009WO0800004Ig`006MlIg`30000
0f]l001[O002000016]k001[Nf]k0P0000E[F`00Je][F`0000=[F`800003Je/006MK00800005Ie/0
06MKIe/00002Ic/200001fLk001W>`00Ic/006Lk00800003Ic/007oo00=oo`03001oogoo00]oo`00
27oo0P002Woo0P000goo0P000goo0P001Goo1`000U<M0P000e<M00<005<MDad00`000U<M00<005<M
Dad00`000U<M0`000U<N00<005<NDah00`000ULn0`000U]N0`0000IOO@00Ggd006=m0002Hgd30000
0fMl001WO003000016Ml001WO6Ml0P0000A[O000Jga[O0800003Jg/006]K00<00007Je/006]K001[
F`00Ie/00P0000=WF`00Ie/00P000VMK0P0000EW>`00Ic]W>`0000=W>`04001W>`00Ic/20005Ool0
0`00Oomoo`0;Ool001ioo`8000=oo`H0009C7@<0009C7@03001C7E<M00<0009C7@03001C7E<M00<0
009C7@<0009C7@03001C7U<N00<0009G?P<0009KGP03001KGP00009OGP80009OO@<00004Hgd006=m
Hgd2000016Ml001WO6Ml0`0000=WO000Ig`00`0000=[O000Jg`00P0000=[F`00Je/00V]K00<006]K
Je/00P0000=WF`00Ie/00P0000IWF`00Ie/006MK0002Ie/200002VLk00000000Ic/006Lk001W>`00
27oo00<007ooOol02Woo000NOol00`00Ool000050002Dad30002Dad00`00DaeC7@030002Dad00`00
DaeC7@030002Dad30002Dad00`00DaeC7@030002Ech00`00Ech00002Ech00`00FeiKGP02000015mN
000005mm0`000V=m0`0000=SO@00Igd00`0000AWO000IgaWO0800003Ig`006Ml009WO003001[O000
009WN`04001[Ff]KJe/200000f]K001WF`0200001FMK001WFfMK00000fMK0P0000=WF`00Ic/00P00
0VLk0P0000AW>`00Ic/000Uoo`03001oogoo00Yoo`0067oo0P0017oo1@000U<M0`000U<M00<005<M
Dad00`000U<M00<005<MDad00`000U<M0`000U<M00<005<MDad00`000U<N00<005LnEch00`000ULn
0`0000IKGP00000005mm0002Ggd00`00Hgd00002Hgd2000016Mm001WO6Ml0`0000=WO000Ig`00P00
0fMl00<006Ml00000VMk00H006MK001[F`00Ie/200001FMK001WFfMK00000fMK0P0000=WF`00Ie/0
0P0000MW>`00Ic/006Lk001W>`020002Ic/00`00Oomoo`07Ool00`00Oomoo`0:Ool001Moo`04001o
ogoo0003Ool20002Dad30002Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad30002Dad00`00
DaeC7@030002Dad00`00DaiC7P0300000eLn00000002000015]N000000000U]N0`000Umm0`0000=S
O@00Hgd00`0000AWO000IgaWO0800008Ig`006MlIg`006Ml001WO0800004Ig/006MKIe/200000fMK
001WF`0300001fMK001WF`00Ie/006MK00800005Ie/006MKIe/00002Ic/200001VLk001W>`00Ic/0
00Yoo`03001oogoo00Yoo`005goo00@007ooOol000Eoo`80009C7@03001C7E<M00<0009C7@03001C
7E<M00<0009C7@<0009C7@03001C7E<M00<0009C7@03001C7E<M00<0009G?P80009G?P03001KGP00
009KGP03001KGP00009OG@<0009OO@800004Hgd006=mHgd300001FMl001WO6Ml00000fMl0P0000AW
O000IgaWO0800003Ie/006MK00800003Ie/006MK009WF`03001WFfMK00800003Ie/006MK0080009W
F`800003Ic/006Lk0080009W>`8000]oo`03001oogoo00Yoo`005goo00@007ooOol000Ioo`03001C
7E<M00<0009C7@H0009C7@<0009C7@03001C7E<M00<0009C7@03001C7E<M00<0009C7P05001G?P00
Ech00002Ech30002Feh3000025mM001OGEmM001OO@00Hgd3000016=m001SO6=l0P000VMl00H006Ml
001WO000Ig`300000fMl001WG00200001FMK001WFfMK00000fMK0P0000=WF`00Ie/00P0000MWF`00
Ie/006MK001WF`02000026Lk001W>`00Ic/007oo000;Ool00`00Oomoo`0:Ool001Moo`04001oogoo
0007Ool20002Dad30002Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad00`00DaeC7@030002
Dad00`00DaiC7P020002Ech30002Fch30002Fed30002Ged2000016=m001SOF=m0`0000ISO000Hg`0
06Ml0002Ig`2000016Ml001WO6Ml0P0000=WG000Ie`00`0000MWF`00Ie/006MK001WF`0200000fMK
001WF`020002Ie/200001FMK001W>fLk00000VLk0P000goo00<007ooOol027oo00<007ooOol02Woo
000HOol20009Ool30002Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad00`00DaeC7@030002
Dad00`00DaeC7@03000015Ln000000000ULn0`000U]M0`000UmM0`0000IOG@00000006=m0002Hgd2
0002Hg`300000fMl001WO00300000fMl001WG00200000fML001WG002Ie`00`00Ie]WF`0200000fMK
001WF`0200001VMK001WF`00Ie/0009WF`800005Ic/00000Ic/00006Ool00`00Oomoo`08Ool00`00
Oomoo`09Ool002Eoo`800004Dad000000002Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad0
0`00DaeC7@03000015<M000000000ULn00<005Ln00000ULn00<005]M00000U]M00<005mM00000UmM
0P000UmM0`0000ISO@00Hgd006=l0002Ig`2000016Ml001WO6Ml0P0000EWG000IeaWG00000=WG080
0003Ie/006MK00800005Ie/006MKIe/00003Ie/200000fMK001WF`020002Ic/00`00Oomoo`05Ool0
0`00Oomoo`08Ool00`00Oomoo`09Ool002Ioo`80009C7@03001C7E<M00<0009C7@03001C7E<M00<0
009C7@03001C7E<M00<0009C7@80009G?@03001G?ULn00<0009G?P<0009KG@<00006Ged005mM001O
G@000V=M0P000V=m0`0000=WO000Ig`00`0000=WO000Ie`00`0000MWG000Ie`006ML001WG0020000
16MK001WFfMK0P0000MWF`00Ie/006MK001WF`0200001VMK001W>`00Ic/000Uoo`03001oogoo00Mo
o`03001oogoo00Uoo`009goo0P0000AC7@000000009C7@03001C7E<M00<0009C7@03001C7E<M00<0
009C7@<00004Ead005LmEcd30002Ech30002Fcd30002Fed20002Ged3000016=M001SGF=M0P000V=l
0P0000AWO000IeaWG0800004Ie`006MLIe`20002Ie`2000016ML001WG6ML0P0000=WF`00Ie/00P00
0VMK0P0000=WF`00Ie/00P000VLk0P002goo00<007ooOol01Woo00<007ooOol02Goo000XOol30002
Dad00`00Dad000020002Dad60002Dad30002Dad00`00Ead000020002Ecd30002Ecd30002Fed30000
15mM000000000UmM0`0000ISG@00Hed006=l0002He`200000fML001WG00300001fML001WG000Ie`0
06ML00<00003Ie`006ML00800007Ie/006MK001WF`00Ie/00P0000IWF`00Ie/006Lk000=Ool00`00
Oomoo`06Ool00`00Oomoo`09Ool002Uoo`L00004Dad000000002Dad00`00DaeC7@030002Dad00`00
DaeC7@020002Ead30002Ecd00`00Ecd00002Fcd01`00Fed005]M001OG@00009OG@03001OG@00009O
G@80009SG@<00003He`006=L00800004Ie`006MLIe`20002Ie`2000016ML001WG6ML0P0000=WG000
Ie`00P000VMK0P0000=WF`00Ie/00P000VMK0P003goo00<007ooOol01Goo00<007ooOol02Goo000[
Ool20002Dad20002Dad00`00DaeC7@030002Dad00`00DaeC7@03000015LM000000000ULm00<005Lm
Ecd00`000U/m0P000U]M0`000UmM0`0000=OG@00Ged00`000V=M0P0000ASG000HeaSG0800006Ie`0
06ML001WG0000VML0P0000=WG000Ie`00P0000MWG000Ie/006MK001WF`0200001VMK001WF`00Ie/0
019oo`03001oogoo00Aoo`03001oogoo00Uoo`00;7oo0`000U<M00<005<M00000P000U<M00<005<M
Dad00`000U<M0P000ULM00<005LmEcd00`000ULm0`0000AK?@000000009KG@<0009OG@800004Ged0
06=MHed300001f=L00000000He`006=L0080009WG0<00003Ie`006ML00800003Ie`006ML008000=W
F`03001WF`00009WF`800003Ie/0000001Aoo`03001oogoo00=oo`03001oogoo00Uoo`00;Goo0P00
0U<M0`0000AC7@00DaeC7@<0009C7@<00004Ead005LMEad30002Ecd30002Fcd20002Fcd30002Ged3
00002UmM00000000Hed006=M001SG0000V=L0P0000ISG000He`006ML0002Ie`200000fML001WG002
000026ML001WG6ML001WF`00Ie/200000fMK001WF`02000FOol00`00Oomoo`04Ool00`00Oomoo`08
Ool002ioo`<0009C7@D0009C7@<0009C7@03001G7@000080009G?@<0009G?@07001K?@00Fcd005/m
00000U/m00L005mM001OG@00Ged00002Ged20002Hed300000f=L001SG00300001f=L00000000Ie`0
06ML00800005Ie`006MLIe`00003Ie`200000fMK001WF`0200000fMK001oo`0HOol00`00Oomoo`03
Ool00`00Oomoo`08Ool0031oo`<00004Dad005<MDad30002Dad00`00DaeC7@03000015LM00000000
0ULm00<005LmEcd00P000U/m0`000U/m0`0000IOG@00000005mM0002Ged20002Hed2000016=L001S
G6=L0P000V=L0P0000AWG000IeaWG0800007Ie`006ML001WG000Ie`00P0000=WF`00Ie/00P006goo
00<007ooOol00Woo00<007ooOol027oo000aOol2000015<M000000000U<M00<005<MDad00`000ULM
0P000ULM00<005LmEcd00`0000AG?@000000009K?@<0009KG@80009OG@<00006Ged005mM001SG@00
0V=L0P0000ISG000He`006=L0002He`200000fML001WG0020002Ie`200000fML001WG00200000fMK
001oo`0MOol01@00Oomoogoo00002Woo000bOol3000015<M001C7E<M0`000U<M0`0000AG7@00EaeG
7@<0009G?@80009K?@<0009K?@<00004Ged000000002Ged20002Ged300000f=L001SG0020002He`3
00000f=L001SG00200001fML001WG000Ie`006ML00800005Ie`006MLIe`0000OOol01@00Oomoogoo
00002Woo000cOol60002Dad30002Dad00`00Ead000020002Ecd3000015Lm001K?@000U/m00L005/m
001K?@00Ged00002Ged3000015mM000000000V=M0P0000ASG000HeaSG080009SG0800003He`006=L
0080009WG0800004Ie`006MLIe`200000fML001oo`0POol01000Oomoo`002Woo000eOol2000015<M
000000000U<M00<005<MDad00P000ULM0`000ULm00<005Lm00000P000U/m0`0000YK?@00000005mM
001OG@00Ged0009OG@<00007Hed00000001SG000He`00`0000ESG000001SG000009SG0800007Ie`0
0000001WG000Ie`00P008goo00<007oo00002Woo000fOol20002Dad00`00DaeC7@03000015LM0000
00000ULM00<005LmEcd00P000ULm0`000U/m0P000U/m0`0000IOG@00000005mM0002Ged20002He`2
00001V=L001SG000He`0009SG0800003He`006=L0080009WG0800003Ie`007oo02Aoo`04001oogoo
0009Ool003Moo`H0009C7@80009G7@03001G7ELM00<00004Ecd000000002Fcd3000015/m00000000
0Ulm0P000UmM0`0000MOG@00Ged006=L001SG0020002He`300000f=L001SG00200001f=L001SG000
Ie`006=L008002Moo`03001oo`0000Uoo`00>7oo0P000U<M0`0000AG7@00EaeG7@<0009G?@80009G
?@<0009K?@80009K?@<00004Gcd000000002Ged20002Ged2000016=L001SG6=L0P000V=L0P0000=S
G000He`00P000V=L0P0000=SG000Ool0:Goo0P002Goo000jOol20002Dad00`00Ead000020002Ead3
000015Lm000000000U/m00/005/m001K?@00Fcd005/m001O?@00009O?@<00005Ged00000001OG@03
00000f=L001SG00300001F=L000006=L00000V=L0P0000=SG0000000:Woo0`002Goo000kOol60000
15LM000000000ULM0P000ULm00<005/mFcd00P000U/m0`0000YK?@00000005lm001OG@00Ged0009O
G@80009SG0800004He`006=LHe`20002He`200000f=L001SG002000XOol4000<Ool003aoo`80009G
7@80009G7@<00004Ecd005LmEcd3000015/m000000000U/m0P000U/m0`0000MOG@00000005mM001O
G@030002He`200001f=L001SG000He`006=L00800003He`007oo02Eoo`@0011oo`00?Goo0`0000AG
7@000000009G7@03001G?@000080009G?@80009K?@<00004Fcd000000002Gcd20002Ged2000025mM
001OGEmM001SG000He`20002He`200000f=L001SG002000SOol4000DOol003ioo`<0009G7@03001G
7ELM0080009G?@<00004Fcd000000002Fcd20002Fcd300001Elm00000000Ged00`000UmM0P0000=S
G000He`00`0000ESG000001SG000029oo`<001Qoo`00@7oo1P0000AG7@000000009G?@80009K?@<0
009K?@80009K?@80009O?@80009OG@<00005Ged00000He`00002He`200000f=L0000000OOol4000K
Ool0045oo`80009G7@80009G?@<00004Ecd000000002Fcd02`00Fcd005/m001K?@00Fcd005lm0000
0Ulm0P0000EO?@00000005mL00<00003He`006=L008001eoo`@001moo`00@Woo0`0000AG7@000000
009G?@80009K?@03001K?E/m0080009K?@80009K?@<00003Gcd005lm0080009O?@80009OG0800003
He`007oo01Yoo`@002=oo`00@goo0`000ULM0`0000AG?@00FceK?@<00005Fcd00000001K?@030002
Gcd200001Ulm001O?@00Gcd0009O?@800003Ge`0000001Qoo`@002Moo`00AGoo0`000ULM00<005Lm
00000P000U/m0P000U/m0P000U/m0`0000=O?@0000000Ulm0`0000=O?@00Gc`00P005Woo1000:goo
0016Ool20002Ead20002Ecd300001E/m00000000Fcd00`0000EK?@00000005lm00<0009O?@800003
Gc`007oo01=oo`@002moo`00Agoo0`0000AG?@000000009G?@80009K?@80009K?@80009K?@80009O
?@<00003Gcd00000019oo`<003=oo`00B7oo0`000ULm0`0000EK?@00000005/m00<00005Fcd00000
001O?@03000015lm0000000047oo1000=Woo001:Ool3000015Lm000000000U/m0P000U/m0P000U/m
0P000Ulm0P003goo1000>Woo001;Ool01@00Ool00000Ecd00`0000aK?@00Fcd005/m001K?@00Fcd0
05lm000=Ool4000nOol004aoo`04001oogoo0002Fcd00`00Fcd0000200001E/m00000000Fcd00P00
2Woo1000@Woo001=Ool00`00Ool00002Fcd20002Fcd20002Fcd20008Ool40016Ool004ioo`@00005
Fcd00000001K?@030006Ool4001:Ool0051oo`03001oo`00009K?@8000Ioo`<004ioo`00DGoo00@0
07oo000000Aoo`@0055oo`00DWoo0P0000=oo`0000000P00EGoo001COol2001IOol00:ioo`00[Woo
002^Ool00:ioo`00[Woo002^Ool00:ioo`00[Woo002^Ool00:ioo`00\
\>"],
  ImageRangeCache->{{{0, 173.375}, {140.313, 0}} -> {-0.215085, -0.0853357, 
  0.00711507, 0.00711507}}]
}, Open  ]],

Cell["\<\
Theta has no definite sign, as revealed by the following plot and \
subsequent evaluations:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[UpAndOutPutTheta[0, S, 50, 50, 0.25,0.1, 0.15, t],
{S, 20, 50}, {t, 0.001, 1}, PlotRange -> All];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.22808 .16727 -11.5904 -9 ]
[.22808 .16727 .40961 0 ]
[.44535 .08183 -10.7564 -9 ]
[.44535 .08183 1.24358 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.7672 .09664 0 -6.16225 ]
[.7672 .09664 18 2.83775 ]
[.82794 .18966 0 -6.07272 ]
[.82794 .18966 18 2.92728 ]
[.88358 .27486 0 -5.99233 ]
[.88358 .27486 18 3.00767 ]
[.93473 .3532 0 -5.91977 ]
[.93473 .3532 18 3.08023 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.0227 .27399 -12 -2.74981 ]
[.0227 .27399 0 6.25019 ]
[.01458 .32484 -12 -2.79283 ]
[.01458 .32484 0 6.20717 ]
[.0062 .37739 -6 -2.83743 ]
[.0062 .37739 0 6.16257 ]
[-0.00247 .43173 -6 -2.88371 ]
[-0.00247 .43173 0 6.11629 ]
[-0.01144 .48795 -6 -2.93176 ]
[-0.01144 .48795 0 6.06824 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(20)] .02757 .24618 1 .93395 Mshowa
.23703 .17687 m
.2415 .18167 L
s
[(30)] .22808 .16727 .93173 1 Mshowa
.4535 .09211 m
.45758 .09725 L
s
[(40)] .44535 .08183 .79274 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.0759 .23997 m
.07874 .24269 L
s
.11524 .22456 m
.11804 .22733 L
s
.1552 .20892 m
.15797 .21172 L
s
.19579 .19302 m
.19852 .19586 L
s
.27893 .16047 m
.28157 .16339 L
s
.32151 .14379 m
.32411 .14675 L
s
.36479 .12685 m
.36734 .12985 L
s
.40878 .10962 m
.41128 .11266 L
s
.49897 .07431 m
.50137 .07743 L
s
.54521 .0562 m
.54755 .05937 L
s
.59224 .03779 m
.59452 .041 L
s
.64007 .01906 m
.64229 .02231 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.75489 .10119 m
.74873 .10346 L
s
[(0.2)] .7672 .09664 -1 .36939 Mshowa
.81555 .19399 m
.80936 .19615 L
s
[(0.4)] .82794 .18966 -1 .34949 Mshowa
.87113 .27899 m
.8649 .28106 L
s
[(0.6)] .88358 .27486 -1 .33163 Mshowa
.92222 .35714 m
.91596 .35912 L
s
[(0.8)] .93473 .3532 -1 .3155 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(1)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.70561 .0258 m
.70193 .02722 L
s
.72242 .05152 m
.71874 .05292 L
s
.73884 .07664 m
.73515 .07802 L
s
.77057 .12518 m
.76687 .12652 L
s
.7859 .14863 m
.7822 .14996 L
s
.80089 .17156 m
.79718 .17287 L
s
.8299 .21593 m
.82618 .21721 L
s
.84394 .2374 m
.84021 .23867 L
s
.85767 .25842 m
.85394 .25967 L
s
.8843 .29914 m
.88056 .30036 L
s
.8972 .31887 m
.89345 .32008 L
s
.90983 .3382 m
.90608 .3394 L
s
.93435 .37571 m
.93059 .37688 L
s
.94625 .3939 m
.94249 .39506 L
s
.95791 .41174 m
.95414 .41289 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03493 .26923 m
.04104 .26685 L
s
[(-2)] .0227 .27399 1 -0.38893 Mshowa
.02685 .32018 m
.03299 .31785 L
s
[(-1)] .01458 .32484 1 -0.37937 Mshowa
.01851 .37284 m
.02466 .37057 L
s
[(0)] .0062 .37739 1 -0.36946 Mshowa
.00988 .42729 m
.01605 .42507 L
s
[(1)] -0.00247 .43173 1 -0.35918 Mshowa
.00095 .48363 m
.00715 .48147 L
s
[(2)] -0.01144 .48795 1 -0.3485 Mshowa
.125 Mabswid
.03333 .27929 m
.037 .27787 L
s
.03173 .28941 m
.0354 .288 L
s
.03011 .2996 m
.03379 .29819 L
s
.02849 .30986 m
.03217 .30845 L
s
.02521 .33057 m
.02889 .32918 L
s
.02355 .34103 m
.02723 .33965 L
s
.02188 .35157 m
.02557 .35019 L
s
.0202 .36217 m
.02389 .3608 L
s
.01681 .38358 m
.0205 .38223 L
s
.01509 .3944 m
.01879 .39305 L
s
.01337 .40529 m
.01707 .40395 L
s
.01163 .41625 m
.01533 .41492 L
s
.00812 .43841 m
.01182 .43708 L
s
.00634 .4496 m
.01005 .44828 L
s
.00456 .46086 m
.00827 .45955 L
s
.00276 .47221 m
.00648 .47091 L
s
.03651 .25924 m
.04018 .2578 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.761 .781 .87 r
.37477 .78405 .39816 .80282 .43737 .77847 .41444 .75914 Metetra
.76 .78 .869 r
.41444 .75914 .43737 .77847 .47684 .75404 .45437 .73413 Metetra
.759 .778 .869 r
.45437 .73413 .47684 .75404 .51658 .72967 .49456 .70913 Metetra
.756 .776 .869 r
.49456 .70913 .51658 .72967 .55658 .70558 .53503 .68437 Metetra
.751 .773 .871 r
.53503 .68437 .55658 .70558 .59689 .68211 .5758 .66016 Metetra
.744 .769 .874 r
.5758 .66016 .59689 .68211 .63755 .65966 .6169 .63696 Metetra
.733 .764 .878 r
.6169 .63696 .63755 .65966 .67862 .63866 .65841 .61521 Metetra
.718 .757 .883 r
.65841 .61521 .67862 .63866 .72018 .61945 .7004 .59534 Metetra
.701 .75 .89 r
.7004 .59534 .72018 .61945 .76233 .60226 .74299 .57763 Metetra
.682 .742 .897 r
.74299 .57763 .76233 .60226 .80517 .58714 .78629 .56218 Metetra
.662 .735 .904 r
.78629 .56218 .80517 .58714 .84879 .57394 .83038 .54887 Metetra
.645 .73 .911 r
.83038 .54887 .84879 .57394 .89325 .56237 .87534 .53739 Metetra
.63 .728 .918 r
.87534 .53739 .89325 .56237 .93858 .55196 .92123 .52725 Metetra
.621 .73 .925 r
.92123 .52725 .93858 .55196 .98479 .54218 .96804 .51787 Metetra
.762 .781 .869 r
.35073 .76475 .37477 .78405 .41444 .75914 .39087 .73927 Metetra
.761 .78 .868 r
.39087 .73927 .41444 .75914 .45437 .73413 .43127 .71367 Metetra
.76 .778 .868 r
.43127 .71367 .45437 .73413 .49456 .70913 .47195 .68804 Metetra
.758 .776 .868 r
.47195 .68804 .49456 .70913 .53503 .68437 .51289 .66258 Metetra
.753 .773 .869 r
.51289 .66258 .53503 .68437 .5758 .66016 .55413 .63762 Metetra
.746 .769 .872 r
.55413 .63762 .5758 .66016 .6169 .63696 .59569 .6136 Metetra
.735 .764 .876 r
.59569 .6136 .6169 .63696 .65841 .61521 .63765 .59105 Metetra
.72 .757 .881 r
.63765 .59105 .65841 .61521 .7004 .59534 .68009 .57045 Metetra
.702 .749 .888 r
.68009 .57045 .7004 .59534 .74299 .57763 .72312 .55216 Metetra
.682 .74 .896 r
.72312 .55216 .74299 .57763 .78629 .56218 .76687 .53635 Metetra
.66 .733 .903 r
.76687 .53635 .78629 .56218 .83038 .54887 .81144 .52292 Metetra
.64 .727 .911 r
.81144 .52292 .83038 .54887 .87534 .53739 .85692 .51156 Metetra
.624 .724 .919 r
.85692 .51156 .87534 .53739 .92123 .52725 .90337 .50175 Metetra
.613 .725 .926 r
.90337 .50175 .92123 .52725 .96804 .51787 .9508 .49283 Metetra
.763 .781 .868 r
.32603 .74492 .35073 .76475 .39087 .73927 .36665 .71885 Metetra
.762 .78 .868 r
.36665 .71885 .39087 .73927 .43127 .71367 .40754 .69264 Metetra
.761 .779 .867 r
.40754 .69264 .43127 .71367 .47195 .68804 .4487 .66637 Metetra
.759 .777 .867 r
.4487 .66637 .47195 .68804 .51289 .66258 .49014 .6402 Metetra
.755 .774 .868 r
.49014 .6402 .51289 .66258 .55413 .63762 .53186 .61445 Metetra
.748 .77 .87 r
.53186 .61445 .55413 .63762 .59569 .6136 .57391 .58958 Metetra
.738 .764 .874 r
.57391 .58958 .59569 .6136 .63765 .59105 .61633 .56616 Metetra
.723 .757 .88 r
.61633 .56616 .63765 .59105 .68009 .57045 .65922 .54475 Metetra
.704 .748 .886 r
.65922 .54475 .68009 .57045 .72312 .55216 .7027 .52581 Metetra
.682 .739 .894 r
.7027 .52581 .72312 .55216 .76687 .53635 .7469 .50958 Metetra
.658 .73 .903 r
.7469 .50958 .76687 .53635 .81144 .52292 .79195 .49602 Metetra
.636 .723 .911 r
.79195 .49602 .81144 .52292 .85692 .51156 .83795 .48482 Metetra
.617 .72 .919 r
.83795 .48482 .85692 .51156 .90337 .50175 .88498 .4754 Metetra
.604 .72 .927 r
.88498 .4754 .90337 .50175 .9508 .49283 .93305 .46705 Metetra
.764 .781 .868 r
.30063 .72453 .32603 .74492 .36665 .71885 .34175 .69786 Metetra
.763 .78 .867 r
.34175 .69786 .36665 .71885 .40754 .69264 .38314 .67103 Metetra
.762 .779 .866 r
.38314 .67103 .40754 .69264 .4487 .66637 .42481 .6441 Metetra
.761 .777 .866 r
.42481 .6441 .4487 .66637 .49014 .6402 .46675 .61721 Metetra
.757 .774 .867 r
.46675 .61721 .49014 .6402 .53186 .61445 .50897 .59065 Metetra
.751 .77 .869 r
.50897 .59065 .53186 .61445 .57391 .58958 .55151 .56488 Metetra
.741 .765 .872 r
.55151 .56488 .57391 .58958 .61633 .56616 .59441 .54051 Metetra
.726 .757 .878 r
.59441 .54051 .61633 .56616 .65922 .54475 .63777 .5182 Metetra
.706 .748 .885 r
.63777 .5182 .65922 .54475 .7027 .52581 .68171 .49851 Metetra
.682 .737 .893 r
.68171 .49851 .7027 .52581 .7469 .50958 .72636 .4818 Metetra
.656 .727 .902 r
.72636 .4818 .7469 .50958 .79195 .49602 .77189 .46809 Metetra
.631 .719 .911 r
.77189 .46809 .79195 .49602 .83795 .48482 .81842 .45708 Metetra
.609 .714 .919 r
.81842 .45708 .83795 .48482 .88498 .4754 .86603 .44815 Metetra
.593 .714 .928 r
.86603 .44815 .88498 .4754 .93305 .46705 .91476 .44049 Metetra
.765 .781 .867 r
.27451 .70355 .30063 .72453 .34175 .69786 .31614 .67626 Metetra
.764 .78 .866 r
.31614 .67626 .34175 .69786 .38314 .67103 .35805 .6488 Metetra
.764 .779 .866 r
.35805 .6488 .38314 .67103 .42481 .6441 .40023 .62121 Metetra
.762 .777 .865 r
.40023 .62121 .42481 .6441 .46675 .61721 .44269 .59359 Metetra
.759 .775 .866 r
.44269 .59359 .46675 .61721 .50897 .59065 .48544 .5662 Metetra
.754 .771 .867 r
.48544 .5662 .50897 .59065 .55151 .56488 .52849 .53949 Metetra
.744 .766 .87 r
.52849 .53949 .55151 .56488 .59441 .54051 .57189 .51409 Metetra
.729 .758 .875 r
.57189 .51409 .59441 .54051 .63777 .5182 .61572 .49077 Metetra
.709 .748 .882 r
.61572 .49077 .63777 .5182 .68171 .49851 .66012 .47022 Metetra
.683 .736 .891 r
.66012 .47022 .68171 .49851 .72636 .4818 .70524 .45293 Metetra
.654 .724 .9 r
.70524 .45293 .72636 .4818 .77189 .46809 .75125 .43904 Metetra
.625 .714 .91 r
.75125 .43904 .77189 .46809 .81842 .45708 .7983 .42825 Metetra
.6 .708 .92 r
.7983 .42825 .81842 .45708 .86603 .44815 .84651 .41992 Metetra
.581 .707 .929 r
.84651 .41992 .86603 .44815 .91476 .44049 .8959 .41311 Metetra
.765 .781 .866 r
.24763 .68196 .27451 .70355 .31614 .67626 .28979 .65403 Metetra
.765 .78 .865 r
.28979 .65403 .31614 .67626 .35805 .6488 .33223 .62593 Metetra
.765 .779 .865 r
.33223 .62593 .35805 .6488 .40023 .62121 .37494 .59766 Metetra
.764 .777 .864 r
.37494 .59766 .40023 .62121 .44269 .59359 .41794 .56931 Metetra
.761 .775 .864 r
.41794 .56931 .44269 .59359 .48544 .5662 .46122 .54108 Metetra
.757 .772 .865 r
.46122 .54108 .48544 .5662 .52849 .53949 .5048 .51339 Metetra
.748 .766 .868 r
.5048 .51339 .52849 .53949 .57189 .51409 .54872 .48688 Metetra
.733 .758 .873 r
.54872 .48688 .57189 .51409 .61572 .49077 .59305 .46242 Metetra
.712 .748 .88 r
.59305 .46242 .61572 .49077 .66012 .47022 .63791 .44086 Metetra
.684 .734 .889 r
.63791 .44086 .66012 .47022 .70524 .45293 .6835 .42288 Metetra
.652 .72 .898 r
.6835 .42288 .70524 .45293 .75125 .43904 .72999 .40874 Metetra
.619 .708 .909 r
.72999 .40874 .75125 .43904 .7983 .42825 .77757 .39822 Metetra
.589 .7 .919 r
.77757 .39822 .7983 .42825 .84651 .41992 .82637 .39062 Metetra
.566 .698 .93 r
.82637 .39062 .84651 .41992 .8959 .41311 .87646 .38487 Metetra
.766 .781 .865 r
.21996 .65973 .24763 .68196 .28979 .65403 .26266 .63115 Metetra
.766 .78 .865 r
.26266 .63115 .28979 .65403 .33223 .62593 .30565 .60238 Metetra
.765 .779 .864 r
.30565 .60238 .33223 .62593 .37494 .59766 .34891 .57343 Metetra
.765 .777 .863 r
.34891 .57343 .37494 .59766 .41794 .56931 .39246 .54434 Metetra
.763 .776 .863 r
.39246 .54434 .41794 .56931 .46122 .54108 .4363 .51528 Metetra
.759 .773 .864 r
.4363 .51528 .46122 .54108 .5048 .51339 .48043 .48658 Metetra
.752 .768 .866 r
.48043 .48658 .5048 .51339 .54872 .48688 .52489 .45889 Metetra
.738 .76 .87 r
.52489 .45889 .54872 .48688 .59305 .46242 .56972 .43314 Metetra
.717 .748 .877 r
.56972 .43314 .59305 .46242 .63791 .44086 .61507 .41038 Metetra
.687 .733 .886 r
.61507 .41038 .63791 .44086 .6835 .42288 .66112 .39152 Metetra
.651 .717 .896 r
.66112 .39152 .6835 .42288 .72999 .40874 .70809 .37705 Metetra
.612 .701 .907 r
.70809 .37705 .72999 .40874 .77757 .39822 .75619 .36684 Metetra
.576 .69 .919 r
.75619 .36684 .77757 .39822 .82637 .39062 .8056 .36015 Metetra
.548 .687 .93 r
.8056 .36015 .82637 .39062 .87646 .38487 .85639 .35573 Metetra
.767 .781 .865 r
.19147 .63684 .21996 .65973 .26266 .63115 .23473 .60757 Metetra
.767 .78 .864 r
.23473 .60757 .26266 .63115 .30565 .60238 .27827 .57812 Metetra
.766 .779 .863 r
.27827 .57812 .30565 .60238 .34891 .57343 .32211 .54847 Metetra
.766 .778 .863 r
.32211 .54847 .34891 .57343 .39246 .54434 .36623 .51865 Metetra
.765 .776 .862 r
.36623 .51865 .39246 .54434 .4363 .51528 .41064 .48876 Metetra
.762 .773 .862 r
.41064 .48876 .4363 .51528 .48043 .48658 .45535 .45906 Metetra
.756 .769 .864 r
.45535 .45906 .48043 .48658 .52489 .45889 .50036 .43012 Metetra
.743 .761 .867 r
.50036 .43012 .52489 .45889 .56972 .43314 .54573 .40293 Metetra
.722 .749 .873 r
.54573 .40293 .56972 .43314 .61507 .41038 .59158 .37872 Metetra
.691 .732 .882 r
.59158 .37872 .61507 .41038 .66112 .39152 .63809 .35872 Metetra
.651 .713 .893 r
.63809 .35872 .66112 .39152 .70809 .37705 .68552 .34376 Metetra
.606 .693 .905 r
.68552 .34376 .70809 .37705 .75619 .36684 .73414 .3339 Metetra
.561 .678 .917 r
.73414 .3339 .75619 .36684 .8056 .36015 .78415 .32836 Metetra
.526 .673 .93 r
.78415 .32836 .8056 .36015 .85639 .35573 .83568 .32565 Metetra
.768 .781 .864 r
.16212 .61325 .19147 .63684 .23473 .60757 .20595 .58328 Metetra
.768 .78 .863 r
.20595 .58328 .23473 .60757 .27827 .57812 .25007 .55312 Metetra
.767 .779 .863 r
.25007 .55312 .27827 .57812 .32211 .54847 .29449 .52276 Metetra
.767 .778 .862 r
.29449 .52276 .32211 .54847 .36623 .51865 .3392 .49221 Metetra
.766 .776 .861 r
.3392 .49221 .36623 .51865 .41064 .48876 .3842 .4615 Metetra
.764 .774 .861 r
.3842 .4615 .41064 .48876 .45535 .45906 .4295 .43082 Metetra
.76 .77 .862 r
.4295 .43082 .45535 .45906 .50036 .43012 .47511 .40061 Metetra
.75 .763 .864 r
.47511 .40061 .50036 .43012 .54573 .40293 .52104 .37181 Metetra
.73 .751 .869 r
.52104 .37181 .54573 .40293 .59158 .37872 .5674 .34584 Metetra
.698 .733 .878 r
.5674 .34584 .59158 .37872 .63809 .35872 .61438 .32434 Metetra
.653 .709 .889 r
.61438 .32434 .63809 .35872 .68552 .34376 .66227 .30861 Metetra
.599 .683 .901 r
.66227 .30861 .68552 .34376 .73414 .3339 .71138 .29908 Metetra
.543 .663 .914 r
.71138 .29908 .73414 .3339 .78415 .32836 .762 .295 Metetra
.497 .653 .929 r
.762 .295 .78415 .32836 .83568 .32565 .81429 .29459 Metetra
.769 .781 .863 r
.13186 .58892 .16212 .61325 .20595 .58328 .17629 .55824 Metetra
.768 .78 .863 r
.17629 .55824 .20595 .58328 .25007 .55312 .22101 .52735 Metetra
.768 .779 .862 r
.22101 .52735 .25007 .55312 .29449 .52276 .26602 .49625 Metetra
.768 .778 .861 r
.26602 .49625 .29449 .52276 .3392 .49221 .31134 .46495 Metetra
.767 .776 .86 r
.31134 .46495 .3392 .49221 .3842 .4615 .35695 .43346 Metetra
.766 .775 .86 r
.35695 .43346 .3842 .4615 .4295 .43082 .40287 .40185 Metetra
.764 .772 .86 r
.40287 .40185 .4295 .43082 .47511 .40061 .44909 .3704 Metetra
.756 .766 .861 r
.44909 .3704 .47511 .40061 .52104 .37181 .49562 .33989 Metetra
.739 .754 .865 r
.49562 .33989 .52104 .37181 .5674 .34584 .54253 .31179 Metetra
.708 .734 .872 r
.54253 .31179 .5674 .34584 .61438 .32434 .58999 .28821 Metetra
.658 .706 .883 r
.58999 .28821 .61438 .32434 .66227 .30861 .6383 .27121 Metetra
.593 .672 .895 r
.6383 .27121 .66227 .30861 .71138 .29908 .68788 .26189 Metetra
.521 .642 .909 r
.68788 .26189 .71138 .29908 .762 .295 .7391 .25972 Metetra
.459 .626 .924 r
.7391 .25972 .762 .295 .81429 .29459 .79219 .26248 Metetra
.77 .781 .863 r
.10067 .56384 .13186 .58892 .17629 .55824 .1457 .5324 Metetra
.769 .78 .862 r
.1457 .5324 .17629 .55824 .22101 .52735 .19104 .50076 Metetra
.769 .779 .861 r
.19104 .50076 .22101 .52735 .26602 .49625 .23667 .46891 Metetra
.769 .778 .86 r
.23667 .46891 .26602 .49625 .31134 .46495 .28261 .43684 Metetra
.768 .776 .86 r
.28261 .43684 .31134 .46495 .35695 .43346 .32885 .40457 Metetra
.768 .775 .859 r
.32885 .40457 .35695 .43346 .40287 .40185 .3754 .3721 Metetra
.766 .773 .858 r
.3754 .3721 .40287 .40185 .44909 .3704 .42226 .33954 Metetra
.762 .769 .858 r
.42226 .33954 .44909 .3704 .49562 .33989 .46943 .30734 Metetra
.75 .759 .86 r
.46943 .30734 .49562 .33989 .54253 .31179 .51693 .27677 Metetra
.722 .739 .865 r
.51693 .27677 .54253 .31179 .58999 .28821 .56489 .25026 Metetra
.67 .706 .874 r
.56489 .25026 .58999 .28821 .6383 .27121 .61362 .231 Metetra
.591 .66 .886 r
.61362 .231 .6383 .27121 .68788 .26189 .6636 .22145 Metetra
.495 .613 .899 r
.6636 .22145 .68788 .26189 .7391 .25972 .71539 .22181 Metetra
.404 .583 .914 r
.71539 .22181 .7391 .25972 .79219 .26248 .76933 .22929 Metetra
.771 .781 .862 r
.06848 .53796 .10067 .56384 .1457 .5324 .11415 .50575 Metetra
.77 .78 .861 r
.11415 .50575 .1457 .5324 .19104 .50076 .16012 .47332 Metetra
.77 .779 .86 r
.16012 .47332 .19104 .50076 .23667 .46891 .20639 .44069 Metetra
.769 .778 .86 r
.20639 .44069 .23667 .46891 .28261 .43684 .25297 .40784 Metetra
.769 .776 .859 r
.25297 .40784 .28261 .43684 .32885 .40457 .29986 .37477 Metetra
.769 .775 .858 r
.29986 .37477 .32885 .40457 .3754 .3721 .34707 .34148 Metetra
.768 .774 .858 r
.34707 .34148 .3754 .3721 .42226 .33954 .39458 .30798 Metetra
.767 .772 .857 r
.39458 .30798 .42226 .33954 .46943 .30734 .44242 .27438 Metetra
.761 .766 .856 r
.44242 .27438 .46943 .30734 .51693 .27677 .49057 .24129 Metetra
.742 .749 .857 r
.49057 .24129 .51693 .27677 .56489 .25026 .53908 .21081 Metetra
.694 .712 .863 r
.53908 .21081 .56489 .25026 .61362 .231 .5882 .18726 Metetra
.6 .65 .871 r
.5882 .18726 .61362 .231 .6636 .22145 .6385 .17598 Metetra
.465 .573 .879 r
.6385 .17598 .6636 .22145 .71539 .22181 .69078 .17962 Metetra
.318 .51 .887 r
.69078 .17962 .71539 .22181 .76933 .22929 .74568 .19495 Metetra
.771 .781 .861 r
.03526 .51123 .06848 .53796 .11415 .50575 .08158 .47822 Metetra
.771 .78 .86 r
.08158 .47822 .11415 .50575 .16012 .47332 .12821 .445 Metetra
.771 .779 .86 r
.12821 .445 .16012 .47332 .20639 .44069 .17514 .41155 Metetra
.77 .778 .859 r
.17514 .41155 .20639 .44069 .25297 .40784 .22239 .37788 Metetra
.77 .776 .858 r
.22239 .37788 .25297 .40784 .29986 .37477 .26994 .34399 Metetra
.769 .775 .858 r
.26994 .34399 .29986 .37477 .34707 .34148 .31782 .30988 Metetra
.769 .774 .857 r
.31782 .30988 .34707 .34148 .39458 .30798 .36601 .27553 Metetra
.769 .773 .856 r
.36601 .27553 .39458 .30798 .44242 .27438 .41453 .24096 Metetra
.768 .771 .855 r
.41453 .24096 .44242 .27438 .49057 .24129 .46337 .20618 Metetra
.763 .763 .853 r
.46337 .20618 .49057 .24129 .53908 .21081 .51254 .17164 Metetra
.738 .736 .849 r
.51254 .17164 .53908 .21081 .5882 .18726 .5621 .14024 Metetra
.65 .662 .848 r
.5621 .14024 .5882 .18726 .6385 .17598 .61251 .12127 Metetra
.451 .523 .84 r
.61251 .12127 .6385 .17598 .69078 .17962 .66506 .12747 Metetra
.171 .364 .813 r
.66506 .12747 .69078 .17962 .74568 .19495 .72121 .15941 Metetra
.772 .781 .86 r
.00095 .48363 .03526 .51123 .08158 .47822 .04795 .44979 Metetra
.772 .78 .86 r
.04795 .44979 .08158 .47822 .12821 .445 .09525 .41573 Metetra
.772 .779 .859 r
.09525 .41573 .12821 .445 .17514 .41155 .14287 .38145 Metetra
.771 .778 .858 r
.14287 .38145 .17514 .41155 .22239 .37788 .1908 .34694 Metetra
.771 .776 .858 r
.1908 .34694 .22239 .37788 .26994 .34399 .23905 .3122 Metetra
.77 .775 .857 r
.23905 .3122 .26994 .34399 .31782 .30988 .28762 .27723 Metetra
.77 .774 .856 r
.28762 .27723 .31782 .30988 .36601 .27553 .33651 .24202 Metetra
.77 .773 .856 r
.33651 .24202 .36601 .27553 .41453 .24096 .38573 .20659 Metetra
.769 .772 .855 r
.38573 .20659 .41453 .24096 .46337 .20618 .43528 .17091 Metetra
.769 .77 .854 r
.43528 .17091 .46337 .20618 .51254 .17164 .48516 .13499 Metetra
.767 .766 .852 r
.48516 .13499 .51254 .17164 .5621 .14024 .53538 .09883 Metetra
.758 .74 .836 r
.53538 .09883 .5621 .14024 .61251 .12127 .58594 .06243 Metetra
.721 .652 .78 r
.58594 .06243 .61251 .12127 .66506 .12747 .63684 .02578 Metetra
0 .046 .582 r
.63684 .02578 .66506 .12747 .72121 .15941 .69585 .12259 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(20)] .02757 .24618 1 .93395 Mshowa
.23703 .17687 m
.2415 .18167 L
s
[(30)] .22808 .16727 .93173 1 Mshowa
.4535 .09211 m
.45758 .09725 L
s
[(40)] .44535 .08183 .79274 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.0759 .23997 m
.07874 .24269 L
s
.11524 .22456 m
.11804 .22733 L
s
.1552 .20892 m
.15797 .21172 L
s
.19579 .19302 m
.19852 .19586 L
s
.27893 .16047 m
.28157 .16339 L
s
.32151 .14379 m
.32411 .14675 L
s
.36479 .12685 m
.36734 .12985 L
s
.40878 .10962 m
.41128 .11266 L
s
.49897 .07431 m
.50137 .07743 L
s
.54521 .0562 m
.54755 .05937 L
s
.59224 .03779 m
.59452 .041 L
s
.64007 .01906 m
.64229 .02231 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{174.375, 141.313},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002^0000SA000`40O003h00O[Woo002^Ool00:io
o`00[Woo001ZOol30004Ool2000kOol006eoo`04001oogoo0002Ool00`00Oomoo`0hOol006eoo`04
001oogoo0002Ool00`00Oomoo`0hOol006Yoo`<000=oo`04001oogoo000jOol006Yoo`03001oogoo
00=oo`04001oogoo000jOol006Yoo`@000=oo`8003]oo`00[Woo002^Ool00:ioo`00M7oo0P00>7oo
001aOol300000goo001oo`0gOol006eoo`@000Aoo`8003Moo`00K7oo0P001goo00<007oo0000=Woo
001YOol40008Ool00`00Ool0000fOol004Ioo`<000=oo`8001Qoo`<00004Ool005jH0008Ool01000
Oomoo`00=Goo0017Ool01@00Oomoogoo00000Woo00<007ooOol04goo0`0000=oo`0000000ejH00<0
07ooOol01Goo00D007ooOomoo`0003Aoo`00AGoo10000Woo00@007ooOol0019oo`<000=oo`03001N
V5jH00=NV08000Ioo`05001oogooOol0000dOol004Eoo`03001oo`0000=oo`04001oogoo000?Ool3
0005Ool00`00GYQNV004GYP20006Ool01000Oomoogoo0P00<goo0015Ool00`00Ool00003Ool01000
Oomoo`003Goo100017oo0P0025jH0P001Goo00<007ooOol00goo00<007ooOol0<7oo0016Ool20004
Ool2000<Ool20007Ool00`00H_X00008GYP00`0003800005Ool00`00Oomoo`02Ool00`00Oomoo`0`
Ool005Moo`<000Qoo`05001RnV;jH_X00007GYP00`0003800005Ool00`00Oomoo`03Ool00`00Oomo
o`04Ool20004Ool20003Ool4000LOol005Aoo`@000Qoo`8000ARnP03001NV5jH00INV0030000<P00
00Aoo`03001oogoo00=oo`03001oogoo00=oo`04001oogoo0003Ool20004Ool00`00Oomoo`0LOol0
059oo`8000]oo`03001RnV;j00ERnP03001NV5jH00ENV0040000<P0b0003Ool00`00Oomoo`04Ool0
0`00Oomoo`02Ool01000Oomoo`002Woo00<007ooOol06goo001?Ool3000<Ool00`00H_YRnP06H_X0
0`00GYQNV005GYP010000380<P000goo00<007ooOol01Goo00D007ooOomoo`00009oo`03001oogoo
00Uoo`03001oogoo01Yoo`00C7oo0`000Woo00<007ooOol027oo0P002F;j00<005jHGYP01UjH00@0
000b0380009oo`03001oogoo00Eoo`80009oo`04001oogoo0008Ool01000Oomoo`0077oo001:Ool2
000?Ool00`00Ha/0000:H_X00`00GYQNV005GYP01`000380<P0b001oo`0000Moo`8000=oo`8000Yo
o`8001eoo`00Agoo0`0047oo00D006<KHa]S6`0000URnP03001NV5jH00ENV0070000<P0b038007oo
00002Goo00<007ooOol0:goo0015Ool2000AOol20005Ha/00`00H_YRnP06H_X60003GYP010000380
<P0b0P002Goo00<007ooOol0:goo000ROol30004Ool2000GOol3000BOol00`00Ha]S6`05Ha/00`00
H_YRnP03H_X300000e:k000j6P03>QX5000303800`00Oomoo`08Ool00`00Oomoo`0ZOol002Eoo`04
001oogoo0002Ool00`00Oomoo`0AOol4000COol00`00Ha]S6`07Ha/00`00H_X000020005D[/00`00
>QXj6P04>QX01P005GX00000038000]oo`03001oogoo02Uoo`009Goo00@007ooOol0009oo`03001o
ogoo00moo`8001Eoo`8000YS6`8000QB^`03000j6SXJ00Dj6P03000ENQEj00<000Yoo`03001oogoo
02Uoo`008goo0P000goo00@007ooOol000ioo`<001Ioo`03001S6`0000US6`03001Nn`0000UB^`03
000j6SXJ00@j6P04000ENQEj5GX3000:Ool00`00Oomoo`0XOol002Eoo`04001oogoo0002Ool00`00
Oomoo`0:Ool2000GOol20003Ha/00`00Ha]S6`04Ha/20003G_/00`00D[]B^`06D[/00`00>QXj6P05
>QX01@005GXENP005GX00P002Goo0P00:Goo000ROol30004Ool2000:Ool3000GOol20006Ha/01000
Ha]S6f<K0P001Ekk00<005:kD[/01U:k00<003XJ>QX01CXJ00@001Ej5GX000<ENP03001oogoo00Mo
o`03001oogoo02Moo`00<Woo100067oo00<006<KHa/01f<K00<006<K000025kk00<005:kD[/01U:k
00<003XJ>QX01CXJ00<001Ej00000aEj00<007ooOol027oo00<007ooOol017oo0P0017oo0P0017oo
0`004goo000`Ool2000JOol2000:Ha/20009G_/00`00D[]B^`06D[/00`00>QXj6P05>QX00`005GX0
00045GX00`00Oomoo`07Ool20004Ool01000Oomoo`000goo0P001Goo00<007ooOol04Woo000]Ool3
000KOol00`00Ha/00009Ha/00`00Ha/0000:G_/00`00D[]B^`04D[/<00045GX00`00Oomoo`08Ool0
0`00Oomoo`02Ool01000Oomoo`0027oo10004goo000ZOol30002Ool00`00Oomoo`0HOol01@00Ha]S
6f<K00001f<K00D006<KHa]S6`0000UNn`03001B^e:k00@0009>V`03000jG3YL00DjG0D0008ENP03
001oogoo00Qoo`05001oogooOol00002Ool00`00Oomoo`06Ool00`00Ool0000DOol002Qoo`8001io
o`8000ES6`03001S6f<K009S6`8000ES6`03001Nnekk00INn`<000M>V`03000jG3YL00DjG080008Z
70<000]oo`04001oogoo0002Ool00`00Oomoo`06Ool00`00Ool0000DOol002Eoo`@001ioo`03001S
6f<K00ES6`05001S6f<KHa/00007Ha/00`00G_]Nn`03G_/300000e[K001>V`07CY/00`00>U`jG005
>U`20005:Q`00`00Oomoo`07Ool20003Ool2000:Ool2000DOol000=oo`@000=oo`8001Moo`80025o
o`03001S6f<K00MS6`03001S6`0000US6`03001Nn`00008000EJf`03001>VdjK00I>V`03000jG3YL
00DjG003000Z72XL00<Z7003001oogoo00Uoo`03001oogoo029oo`0017oo00<007ooOol00Woo00@0
07ooOol001=oo`<0025oo`8000YS6`8000YS6`8000UJf`03001>VdjK00I>V`03000jG3YL00@jG080
00DZ7003001oogoo00Uoo`03001oogoo025oo`001Goo00D007ooOomoo`00009oo`03001oogoo00io
o`<002=oo`04001S6f<K0008Ha/01000Ha]S6`0026<K00<005kk00002E[K00<004jKCY/01TjK00<0
03YL>U`013YL0P001RXL00<007ooOol027oo0P008Woo0006Ool01000Oomoo`000Woo00<007ooOol0
37oo0P0097oo0P0016<K00<006<KHa/00f<K0P0016<K00<006<KHa/00f<K0P000ekk00<005[KF]/0
1e[K00<004jKCY/00TjK4@000RXL00<007ooOol027oo00<007ooOol087oo0003Ool01000Oomoo`00
0Woo00@007ooOol000]oo`<00003Ool007oo029oo`03001S6f<K00ES6`05001S6f<KHa/00007Ha/0
1@00Ha]S6f<K00001Ukk00<005[KF]/01U[K1@0014Zl00<003il?W`00cil00<003il0000139M1000
2Goo00<007ooOol087oo0004Ool20004Ool2000:Ool2000WOol00`00Ha]S6`07Ha/00`00Ha/00009
Ha/00`00Ha/00007G_/00`00F]]Jf`03F]/300000eKK001:_007B[`00`00?W`nO002?W`01000?W`n
O0001S9M00<007ooOol027oo0P0087oo000COol3000WOol2000:Ha/2000:Ha/20009G_/00`00F]/0
00020005E]/00`00B[a:_005B[`00`00?W`nO002?W`01000?W`nO0001c9M00<007ooOol01goo0P00
87oo000@Ool3000YOol00`00Ha/00009Ha/00`00Ha/00009Ha/00`00Ha/00009G_/20008E]/00`00
B[a:_006B[`00`00?W`nO0020003?W`00`00<UdbG@05<Ud00`00Oomoo`07Ool00`00Oomoo`0NOol0
00moo`8002Yoo`05001S6f<KHa/00006Ha/20003Ha/00`00Ha]S6`04Ha/20003Ha/00`00G_]Nn`05
G_/00`00G_/00009E]/00`00B[a:_005B[`300000cil000nO002?W`00`00<UdbG@06<Ud00`00Oomo
o`07Ool00`00Oomoo`04Ool20004Ool20004Ool2000;Ool000moo`<002Moo`8000ES6`04001S6f<K
Ha/20006Ha/01000Ha]S6f<K0P001V<K00<005kkG_/00Ukk0P000ekk00<005KKE]/01eKK00@004Zl
B[a:_1H000Qoo`8000Aoo`04001oogoo0003Ool20003Ool01000Oomoo`002Woo000?Ool200000goo
001oo`0TOol00`00Ha]S6`06Ha/00`00Ha/00009Ha/00`00Ha/00008Ha/01@00G_]Nnekk00001Ukk
00<005KKE]/01UKK1P000TZl00<004:M@Yd00T:M00@004:M@Yd000PjW@03001oogoo00Moo`03001o
ogoo009oo`04001oogoo0008Ool01000Oomoo`002Woo0008Ool40003Ool01@00Oomoogoo00008goo
00<006<KHa/01f<K0P002V<K0P002V<K00<005kk00001ekk00<005KKE]/015KK1@001TZl00<004:M
@Yd00T:M00@004:M@Yd000PjW@03001oogoo00Moo`05001oogooOol00002Ool00`00Oomoo`06Ool3
000;Ool000Uoo`03001oogoo009oo`8000Aoo`03001oogoo01ioo`8000US6`04001S6f<K0008Ha/0
0`00Ha/0000:Ha/20009G_/00`00E]]Ff`040002E]`00`00B[a:_006B[`01@00@Ye2WD:M00000d:M
00<003ZM>Yd01SZM00<007ooOol01Woo0P000Woo00@007ooOol000Qoo`03001oogoo00]oo`000Woo
1@000goo00@007ooOomoo`8000Eoo`03001oogoo01aoo`04001S6f<K0007Ha/00`00Ha]S6`02Ha/0
0`00Ha]S6`03Ha/20003Ha/00`00Ha]S6`05Ha/200000elK001Nn`08G_/30007E]`00`00B[a:_005
B[`00`00@Ye2W@02@Yd01@00@Ye2WD:M00001cZM00<007ooOol01goo00@007ooOomoo`8000Yoo`<0
00Yoo`002goo00@007ooOol000Moo`03001oogoo01Uoo`8000AS6`03001S6f<K00=S6`03001S6f<K
00AS6`05001S6f<KHa/00006Ha/00`00Ha]S6`02Ha/20004Ga/00`00G_]Nn`03G_/300000e[k001F
g007E]`00`00B[a:_006B[`00`00@Ye2W@02@Yd=0009Ool00`00Oomoo`0IOol000Qoo`04001oogoo
0002Ool00`00Oomoo`06Ool00`00Oomoo`0FOol20007Ha/01@00Ha]S6f<K000026<K00<006<K0000
26<K00@006<K000000MO6`03001Nn`0000<000AJn`03001Fg5KL00IFg003001:_4Zl00A:_0L000=6
_@03000nWCjM00DnW@03001oogoo00Moo`8001Yoo`002Goo0P000goo00<007ooOol01goo00<007oo
Ool057oo00<006<KHa/026<K00<006<K00002F<K0P002F<K0P0025lK0`0000=Nn`0000001U[k00<0
05KLE]`01EKL00<004ZlB[`010000Tjl00<004JmA[d00TJm00D004JmA[e6_@0000LnW@03001oogoo
00Moo`03001oogoo01Qoo`003Woo00<007ooOol027oo00<007ooOol04Goo0P002f<K0P002F<K00<0
06<K000026<K00<006<K00001UlK0P000UlK0P0025[k00<005KLE]`015KL10001djl00<004JmA[d0
0TJm00@004JmA[d000PnW@03001oogoo00Moo`03001oogoo01Moo`003Goo0P002goo00<007ooOol0
3goo00@006<KHa/000US6`03001S6`0000QS6`05001S6f<KHa/00005Ha/20003Ha/00`00Ga]O6`03
0002Ga/200000elK001Jn`08F_/00`00E]aFg0030002D]`00`00C[a>_007C[`00`00A[e6_@02A[d0
1000A[e6_@001cjM00<007ooOol01goo0P0067oo0007Ool30003Ool00`00Oomoo`0;Ool00`00Oomo
o`0=Ool00`00Ha]S6`02Ha/00`00Ha]S6`04Ha/20003Ha/00`00Ha]S6`03Ha/20005Ha/01@00Ha]S
6f<K00001F<K0`000elK0P0015lK00<005[kF_/01U[k0`001U;L00<004jlC[`01Tjl00<004JmA[d0
0TJm00@004JmA[d000@nW@D000Uoo`03001oogoo01Ioo`0027oo00<007ooOol00Woo0P003Goo00<0
07ooOol02goo00<006<KHa/016<K00@006<KHa]S6`8000IS6`05001S6f<KHa/00008Ha/00`00Ha/0
0003Ha/30002Ha/01000Ga/000001elK00<005[kF_/00U[k0`0000=Jo000D]`01e;L00<004jlC[`0
1Tjl00<004JmA[d00TJm1`0014:m00<007ooOol027oo00<007ooOol017oo0P0017oo0P0017oo0P00
0goo00000goo000000030002Ool00`00Oomoo`02Ool00`00Oomoo`0=Ool00`00Oomoo`09Ool00`00
Ha]S6`06Ha/00`00Ha/00009Ha/00`00Ha/00009Ha/2000016<K000000001F<K0P002UlK00@005[k
000000EJo003001Bg5;L00IBg003001>_4jl009>_0P00004A[d004Jm0008@[d00`00Oomoo`07Ool2
0004Ool01000Oomoo`000goo0P000goo00@007ooOol0009oo`0027oo00<007ooOol00Woo00<007oo
Ool03Woo00<007ooOol01goo00<006<KHa/01f<K0P002V<K0`0026<K10001f<K00<006<K00002UlK
0P0025[l00<005;LD]`015;L1P0014kL00<004JmA[d00dJm00<004Jm000024:m00<007ooOol01goo
00<007ooOol00Woo00@007ooOol000Qoo`04001oogoo0002Ool000Qoo`03001oogoo009oo`03001o
ogoo00moo`03001oogoo00Aoo`<000QS6`04001S6f<K0008Ha/01@00Ha]S6f<K00001F<K0`000V<K
00<006<KHa/00f<K0P000f<K00<005lKGa/01ElK0P0000=O6`00F_`025[l00<005;LD]`00`000U;L
00<004kLC]`01TkL00<004JmA[d00TJm00@004JmA[d000Q2_@03001oogoo00Moo`05001oogooOol0
0002Ool00`00Oomoo`07Ool20003Ool000Moo`8000=oo`03001oogoo015oo`03001oogoo009oo`05
001S6f<KHa/00005Ha/20004Ha/00`00Ha]S6`03Ha/20005Ha/00`00Ha/000020006Ha/01@00Ha]S
6f<K00001V<K00<005lKGa/00UlK0P0015lK00<005[lF_`01U[l0`001U;L00<004kLC]`01TkL00<0
04JmA[d00TJm00@004JmA[d000A2_@D000Qoo`80009oo`04001oogoo0008Ool01000Oomoo`000Woo
000<Ool2000COol01000Ool000001F<K00D006<KHa]S6`0000MS6`05001S6f<KHa/00006Ha/30000
0f<K001S6`07Ha/00`00Ha/00008Ha/01000Ga/000001UlK00<005[lF_`015[l0P0000=Jo000D]`0
1e;L00<004kLC]`01DkL00<004JmA[d00TJm200014Jm00<007ooOol01goo00@007ooOomoo`8000Yo
o`8000=oo`0037oo0P004goo0P0026<K00<006<K00002F<K00<006<K00001F<K0P000V<K0P002F<K
0P002F<K0P002ElK00@005[lF_aJo08000AJo003001Bg5;L00IBg003001>g4kL009>g0L0009:g@03
001:g@0000Q6_@03001oogoo00Moo`03001oogoo015oo`001Woo0P0017oo00<007ooOol04Goo00<0
06<KHa/026<K0P002V<K0P000f<K0`000f<K00<006<K000026<K00<006<K000026<K00<005lK0000
2ElK00@005[l000000IJo003001Bg5;L00IBg0D000=>g@03001:gD[M00=:g@04001:gD[M0008A[d0
0`00Oomoo`06Ool2000BOol000Eoo`04001oogoo0003Ool00`00Oomoo`0?Ool30009Ha/01000Ha]S
6`0026<K00<006<K00000`001F<K00D006<KHa]S6`0000ES6`8000=S6`03001S6f<K00=S6`8000=O
6`03001O6elK00IO6`8000UJo003001Bg5;L009Bg0<00003D]`004kM00I>g@03001:gD[M00=:g@04
001:gD[M0004A[d50008Ool00`00Oomoo`0@Ool000Eoo`04001oogoo0003Ool00`00Oomoo`0>Ool0
1@00Ha]S6f<K00001f<K00<006<KHa/00V<K00<006<KHa/00f<K1P001F<K0P001F<K00D006<KHa]S
6`0000IS6`05001S6f<KHa/00006Ga/00`00Ga]O6`02Ga/30002G_/00`00F_aJo006F_`00`00D]`0
00020005D]`00`00C]e>g@05C]d00`00B]e:g@03B]d70004B]d00`00Oomoo`07Ool00`00Oomoo`0?
Ool000Eoo`04001oogoo0003Ool00`00Oomoo`0<Ool20005Ha/00`00Ha]S6`03Ha/00`00Ha]S6`04
Ha/01000Ha]S6f<K0`0016<K00D006<KHa]S6`0000QS6`03001S6`0000QS6`03001S6`0000QO6`04
001O6`000006G_/00`00F_aJo004F_`30009D]`00`00C]e>g@04C]d00`00B]e:g@040002B]d00`00
B]e:g@06B]d00`00Oomoo`06Ool2000@Ool000Eoo`04001oogoo0002Ool2000=Ool00`00Ha]S6`06
Ha/01@00Ha]S6f<K000026<K0`0026<K00<006<K00002F<K0P002F<K0P002ElK0P002Ekk00<005[l
F_`00`000eKl00<005;LD]`01U;L00<004kMC]d00TkM1@0014[M00@004[MB]d000Q:g@03001oogoo
00Ioo`03001oogoo00=oo`<000Qoo`001Woo0P000goo00<007ooOol02goo00<006<KHa/026<K00<0
06<K00001V<K1@002F<K0`0026<K00<006<K000026<K00<006<K000025lK00<005lK00002Ekk0`00
1eKl00<005;LD]`015;L1P0014kM00<004[MB]d00d[M00@004[MB]d000M:g@03001oogoo00Moo`03
001oogoo00=oo`03001oogoo00Moo`002goo0P002Woo0P002f<K0P0016<K0`000V<K00<006<K0000
26<K00D006<KHa]S6`0000ES6`8000=S6`03001S6f<K00=S6`8000=S6`03001O6elK00=O6`8000=O
6`03001Nnekk00ANn`800003F_`005Kl00QFo003001Bg5;L00<0009Bo003001>gDkM00I>g@03001:
gD[M00=:g@03001:g@0000A:g@D000Qoo`03001oogoo00=oo`03001oogoo00Moo`002goo00<007oo
Ool027oo00@006<KHa/000US6`03001S6f<K00@000=S6`8000=S6`03001S6f<K00=S6`8000ES6`05
001S6f<KHa/00006Ha/01@00Ha]S6f<K00001V<K00D005lKGa]O6`0000IO6`03001Nnekk009Nn`03
001Jo5[l009Jo003001Fo5Kl00IFo0<000IBo003001>gDkM00I>g@03001:gD[M009:g@L000A:g@03
001oogoo00Moo`03001oogoo009oo`03001oogoo00Moo`002goo0P0027oo00<006<KHa/00V<K00<0
06<KHa/016<K0P0000=S6`0000000P000f<K0P001V<K00D006<KHa]S6`0000QS6`03001S6`0000QS
6`03001S6`0000QS6`03001O6`0000QO6`04001Nn`000006F_`00`00E_aFo002E_`300000eKl001B
o007D_`00`00C]e>g@04C]d6000014kM001>g@0024[M00<007ooOol01Goo0`000goo00<007ooOol0
1goo0004Ool30004Ool00`00Oomoo`06Ool00`00Ha]S6`04Ha/01000Ha]S6f<K0P0000=S6`000000
16<K00<006<K00002F<K00<006<K00002F<K0P002F<K0P002F<K0P002ElK0P002E[l00@005Kl0000
00EFo003001Bo5;l00EBo003001>gDkM00@0009>g@03001>gDkM00=>g@03001>g@0000Q:g@04001o
ogooOol300000goo00000002Ool20009Ool000Eoo`03001oogoo009oo`8000Moo`03001S6f<K00IS
6`03001S6`0000<000IS6`8000YS6`<000QS6`04001S6f<K0006Ha/20002Ha/00`00Ha]S6`04Ha/2
0002Ha/00`00Ga]O6`04Ga/200000elK001Jo008F_`20008E_`00`00D_aBo003D_`40007C]d00`00
C]e>g@03C]d00`00C]d00004B]d70005Ool00`00Oomoo`0;Ool000Eoo`03001oogoo009oo`8000Io
o`03001S6f<K00MS6`<000QS6`04001S6f<K0008Ha/01@00Ha]S6f<K00001F<K0P0016<K00<006<K
Ha/00V<K00<006<KHa/00f<K00<006<KHa/00V<K00<006<KHa/00f<K00<005lKGa/00UlK00<005lK
Ga/00UlK00<005[lF_`015[l0P0000=Jo000E_`01eKl00<005;lD_`00`000e;l00<004kMC]d01TkM
00<004kMC]d00TkM1`0014[M00<007ooOol01Goo00<007ooOol02goo0005Ool00`00Oomoo`02Ool0
0`00Oomoo`03Ool30006Ha/30003Ha/00`00Ha]S6`03Ha/20004Ha/00`00Ha]S6`03Ha/20005Ha/0
1@00Ha]S6f<K00001f<K00@006<K000000MS6`04001S6`000007Ha/01000Ga/000001UlK00<005[l
F_`00U[l00<005[lF_`00U[l00<005KlE_`01EKl0`001e;l00<004kMC]d01DkM00<004kMC]d01000
00=>g@00B]d01d[M00<007ooOol01Goo00<007ooOol02Woo0005Ool00`00Oomoo`02Ool20003Ool0
1@00Ha]S6f<K00000V<K1@001F<K00D006<KHa]S6`0000MS6`05001S6f<KHa/00008Ha/00`00Ha/0
0008Ha/20009Ha/20009Ha/20009Ga/01000F_`000001U[l00@005KlE_aFo0<00003E_`005;l00QB
o003001>gDkM009>g@D000A>g@03001>g@0000Q:g@03001oogoo00Aoo`03001oogoo00Yoo`0017oo
0P0017oo00<007ooOol00P0016<K0`000V<K00<006<KHa/01V<K00<006<K00002F<K00<006<K0000
2F<K0P0026<K00<006<K000026<K00@006<KHa/000IS6`80009S6`03001O6elK00IO6`8000QJo004
001Fo0000005E_`00`00D_aBo006D_`50003D_d00`00C]e>g@03C]d01000C]e>g@001T[M0`001Goo
00<007ooOol02Woo000:Ool30003Ha/30002Ha/00`00Ha/00009Ha/2000:Ha/30008Ha/01000Ha]S
6`001F<K0P000f<K00<006<KHa/00f<K0P0016<K00<006<KHa/00V<K00<006<KHa/00f<K00<005lK
Ga/00UlK0`0000=O6`00F_`01e[l0P0025Kl00<005;lD_`00U;l0`0000=Bo000D_d01U;m00<004kM
C]d00dkM00<004kM00000T[M10000Tkm00<007ooOol00goo00<007ooOol02Woo0009Ool20002Ha/3
0005Ha/30008Ha/01000Ha]S6`0026<K00D006<KHa]S6`0000ES6`8000AS6`05001S6f<KHa/00006
Ha/01@00Ha]S6f<K00001f<K00@006<K000000MS6`04001O6`000005Ga/00`00F_aJo002F_`30000
0e[l001Fo007E_`00`00D_`000020005D_`00`00D_eBo@05D_d00`00C]e>g@02C]d50007C_d00`00
Oomoo`02Ool00`00Oomoo`0:Ool0009oo`@000=oo`@000MS6`05001S6f<KHa/00005Ha/20004Ha/0
0`00Ha]S6`03Ha/20005Ha/01@00Ha]S6f<K00001f<K00<006<K000026<K00<006<K000026<K0P00
2F<K0P0025lK00@005[l000000EJo003001Fo5Kl00AFo0<000UBo003001BoE;m00=Bo@H0009>o@80
00Q>o@03001oogoo009oo`03001oogoo00Yoo`000goo00<007ooOol00goo0`001V<K0P001F<K00D0
06<KHa]S6`0000MS6`05001S6f<KHa/00008Ha/00`00Ha/00008Ha/20009Ha/20008Ha/01000Ha]S
6`001V<K0P0000=O6`00Ga/01elK0P0025[l00<005KlE_`00`000eKl00<005;lD_`01U;l00<005;m
D_d00`000U;m00<004kmC_d00dkm00<004km00001Tkm0`000goo00<007ooOol02Woo0004Ool00`00
Oomoo`04Ool30003Ha/00`00Ha]S6`06Ha/00`00Ha/00009Ha/00`00Ha/00009Ha/20008Ha/01000
Ha]S6`001V<K0P000V<K00<006<KHa/00f<K0P0016<K00<006<KHa/00V<K00<005lKGa/00UlK00<0
05lKGa/00elK0P000UlK00<005[lF_`01E[l0`001eKl00<005;lD_`015;l10001U;m00<004kmC_d0
0dkm00<004km00000Tkm0`000Tkm00<007ooOol00Woo00<007ooOol02Goo0005Ool00`00Oomoo`04
Ool01@00Ool006<K00002F<K0`002F<K0`0026<K00@006<KHa/000ES6`8000AS6`03001S6f<K009S
6`03001S6f<K00=S6`05001S6f<KHa/00007Ha/01000Ha/000001UlK00D005lKGa]O6`0000EO6`03
001Jo5[l009Jo080009Jo003001Fo5Kl00IFo003001Bo5;l00<0009Fo003001BoE;m00IBo@03001>
oDkm009>o@D000I>o@05001oogooOol0000;Ool0009oo`04001oogoo0007Ool01000Ool0000026<K
00D006<KHa]S6`0000IS6`8000=S6`03001S6f<K00=S6`8000AS6`05001S6f<KHa/00007Ha/01000
Ha/000001f<K00<006<K000026<K0P0025lK00@005lK000000MO6`04001Jo0000005F_`00`00E_aF
o004E_`40006E_`00`00D_eBo@04D_d60002C_d20008C_d01000Oomoo`002goo0003Ool20009Ool2
00000goo001S6`04Ha/20005Ha/00`00Ha]S6`02Ha/00`00Ha]S6`04Ha/01@00Ha]S6f<K00001f<K
00<006<K000026<K0P002F<K0P0026<K00<006<K000025lK0P002ElK0P0025[l00<005KlE_`00`00
0UKl00<005KlE_`01UKl00<005;mD_d00`000U;m00<004kmC_d00dkm00<004km00001Tkm0`0000=o
o`00Ool02Woo000@Ool00`00Ool00003Ha/00`00Ha]S6`06Ha/01000Ha/0000026<K00<006<K0000
26<K0P0026<K00@006<KHa/000IS6`80009S6`03001S6f<K00=S6`8000=S6`03001O6elK00AO6`04
001O6elK0006Ga/200000elL001Jo007F_`30006E_`00`00E_aFo004E_`40006D_d00`00C_e>o@03
C_d00`00C_d00002C_d30002C_d00`00Ool0000;Ool0015oo`05001oo`00Ha/00009Ha/30009Ha/2
0008Ha/01000Ha]S6`001F<K0P0016<K00<006<KHa/00V<K00<006<KHa/00f<K00D006<KHa]S6`00
00IS6`04001O6elKGa/20004Ga/00`00Ga]O6`02Ga/00`00GaaO7002Ga`00`00F_aJo002F_`30000
0e[l001Fo007E_`00`00E_aFo0030002E_`00`00D_eBo@06D_d00`00C_e>o@02C_d50006C_d2000;
Ool0019oo`04001oo`000008Ha/01@00Ha]S6f<K00001V<K0P000V<K00<006<KHa/00f<K0P0016<K
00D006<KHa]S6`0000MS6`04001S6`000007Ha/00`00Ha/00008Ha/00`00Ga/00007Ga/01000Ga/0
00001UlL00@005[l000000EJo003001Fo5Kl00EFo0<000IFo003001BoE;m00ABo@H00004D_d005;m
0008C_d00`00Oomoo`09Ool001=oo`800003Ool006<K00AS6`8000ES6`03001S6f<K009S6`03001S
6f<K00=S6`05001S6f<KHa/00007Ha/00`00Ha/00008Ha/20009Ha/20009Ha/20008Ga/20008Ga`2
0008F_`01000E_aFo5Kl0`0000=Fo000E_`01eKl00<005;mD_d00`000U;m00<005;mD_d00e;m00<0
05;m00001Tkm0`002Woo000EOol00`00Ool00003Ha/00`00Ha]S6`06Ha/01000Ha/000001f<K00<0
06<K000026<K0`001f<K00@006<KHa/000IS6`80009S6`03001S6f<K00AS6`800003Ha/005lK00EO
6`800003Ga/005lL00EO70800003Ga`005[l00MJo004001Fo0000005E_`00`00E_aFo004E_`40006
D_d00`00D_eBo@03D_d20002C_d40002Ool00`00Oomoo`08Ool001Ioo`05001oo`00Ha/00009Ha/2
0009Ha/30007Ha/01@00Ha]S6f<K00001F<K00<006<KHa/00V<K00<006<KHa/00V<K00<006<KHa/0
0f<K00<006<KHa/00V<K00<006<KHa/00V<K00<005lKGa/00UlK00<005lKGa/00UlK00<005lLGa`0
0UlL00<005lLGa`00UlL00<005[lF_`015[l0`0025Kl00<005KlE_`00`000UKl00<005;mD_d01E;m
00<005;mD_d00U;m1@001Woo00<007ooOol027oo000GOol01000Ool0000026<K00@006<KHa/000IS
6`8000=S6`03001S6f<K00=S6`03001S6f<K00=S6`05001S6f<KHa/00006Ha/01000Ha/000001f<K
00@006<K000000IS6`04001O6`000006Ga/01000Ga`000001UlL00<005[lF_`00`000U[l00<005Kl
E_`01UKl0`001UKl00<005;mD_d00e;m1P000Woo00<007ooOol01Woo00<007ooOol027oo000HOol2
00000goo001S6`04Ha/20004Ha/00`00Ha]S6`02Ha/00`00Ha]S6`04Ha/01@00Ha]S6f<K00001f<K
00<006<K00001f<K0P002F<K0P0026<K0`001elK0P0025lL0`001U[l00<005KlE_`00UKl0`0000=F
o000E_`01UKl00<005;mD_d00`002Goo00<007ooOol01Goo00<007ooOol027oo000JOol00`00Ool0
0003Ha/00`00Ha]S6`05Ha/01000Ha/0000026<K00<006<K000026<K0P001f<K00@006<KHa/000IS
6`80009S6`03001S6f<K00AS6`05001S6f<KHa/00004Ga/20002Ga/00`00GaaO7004Ga`00`00Ga`0
0008F_`01000E_`000001EKl00<005KlE_`015Kl0`003Goo00<007ooOol017oo00<007ooOol027oo
000KOol01@00Ool006<K000026<K0`002F<K0`001V<K0P000V<K00<006<KHa/00f<K00<006<KHa/0
0V<K00<006<KHa/00V<K00<006<KHa/00f<K00@006<KHa]S6`8000ES6`04001O6`000005Ga/01000
GaaO75lL0P000elL00<005[lF_`015[l0`0025Kl00<005KlE_`00`004Goo00<007ooOol00goo00<0
07ooOol027oo000LOol00`00Ool00008Ha/01@00Ha]S6f<K00001V<K0P000f<K00<006<KHa/00V<K
00<006<KHa/00f<K00D006<KHa]S6`0000IS6`04001S6`000007Ha/00`00Ha/00007Ha/20008Ga/0
0`00Ga`00006Ga`00`00F_aJo0030003F_`00`00E_aFo004E_`4000DOol00`00Oomoo`04Ool00`00
Oomoo`07Ool001eoo`03001oo`0000ES6`8000ES6`03001S6f<K009S6`03001S6f<K00AS6`04001S
6`000007Ha/00`00Ha/00007Ha/20009Ha/20006Ha/200000elK001O6`07Ga/20007Ga`30007F_`0
0`00E_aFo003000IOol00`00Oomoo`03Ool00`00Oomoo`07Ool001ioo`<000=S6`03001S6f<K00IS
6`04001S6`000007Ha/20009Ha/30006Ha/01000Ha]S6`001V<K0P000V<K00<006<KHa/00V<K00<0
05lKGa/00UlK00<005lKGa/00UlK0`000UlL00<005lLGa`00UlL0P0000=K7000F_`01e[l0`007Goo
00<007ooOol00Woo00<007ooOol01goo000POol200000f<K001S6`08Ha/20008Ha/01000Ha]S6`00
1V<K0P000f<K00@006<KHa]S6`8000AS6`03001S6f<K009S6`03001S6f<K00=S6`04001S6`000006
Ga/01000Ga/000001UlL00@005lL000000AK7003001Jo5[l009Jo0<0025oo`05001oogooOol00009
Ool0025oo`<000QS6`04001S6f<K0005Ha/20004Ha/00`00Ha]S6`02Ha/00`00Ha]S6`04Ha/00`00
Ha/00007Ha/01000Ha/000001V<K0P0025lK0P0025lL0P001e/L00@005[l000002Aoo`05001oogoo
Ool00009Ool0029oo`03001oo`0000ES6`8000AS6`05001S6f<KHa/00007Ha/01000Ha/000001f<K
0P0026<K0`001V<K00@006<KHa/000IO6`03001O6`0000IO70800003Ga`005/L00IK708002Moo`04
001oogoo0009Ool002=oo`<000=S6`03001S6f<K00ES6`03001S6`0000QS6`8000QS6`03001S6`00
00IS6`8000=S6`04001S6f<KHa/20004Ha/01000Ga]O6elK0P000elK00<005lLGa`00UlL00<005lL
Ga`00UlL00<005/LFa`00U/L0P00:Woo00<007oo00002Goo000UOol200000f<K001S6`07Ha/30006
Ha/20002Ha/00`00Ha]S6`03Ha/20003Ha/00`00Ha]S6`02Ha/00`00Ha]S6`04Ha/00`00Ha/00007
Ha/00`00Ga/00006Ga/01000Ga`000001UlL00@005/L000002aoo`04001oogoo0008Ool002Ioo`<0
00MS6`05001S6f<KHa/00004Ha/00`00Ha]S6`03Ha/01@00Ha]S6f<K00001V<K00@006<K000000MS
6`8000QS6`8000MO6`<000MO708002moo`03001oo`0000Qoo`009goo00@007oo000000=S6`8000ES
6`04001S6`000007Ha/00`00Ha/00007Ha/30006Ha/200000f<K001S6`05Ha/20002Ha/00`00Ga]O
6`02Ga/20003Ga/00`00GaaO7002Ga`2000bOol20008Ool002Qoo`800004Ool006<K0007Ha/20009
Ha/30006Ha/01@00Ha]S6f<K000016<K00<006<KHa/00V<K00<006<KHa/00V<K00<006<KHa/00f<K
00@005lK000000IO6`04001O7000000cOol30008Ool002Yoo`03001oo`0000MS6`04001S6f<K0006
Ha/20003Ha/01000Ha]S6f<K0P001F<K00@006<K000000IS6`04001S6`000006Ha/20008Ga/2000a
Ool4000;Ool002]oo`03001oo`0000ES6`03001S6f<K009S6`03001S6f<K009S6`03001S6f<K00AS
6`03001S6`0000MS6`8000QS6`<000IS6`03001S6`0000IO6`80031oo`<000moo`00;7oo00<007oo
00000f<K00<006<KHa/016<K00@006<K000000MS6`<000IS6`04001S6f<K0006Ha/01@00Ha]S6f<K
00000f<K0P000f<K00<005lKGa/00UlK00<007ooOol0;7oo10004Woo000]Ool300000f<K001S6`06
Ha/30006Ha/20003Ha/01000Ha]S6f<K0P0016<K00@006<KHa]S6`8000ES6`03001S6`0000IS6`04
001O6`00000[Ool4000FOol002moo`<000IS6`05001S6f<KHa/00004Ha/00`00Ha]S6`04Ha/00`00
Ha/00007Ha/00`00Ha/00007Ha/20007Ha/2000YOol4000JOol0031oo`03001oo`0000=S6`8000ES
6`04001S6`000007Ha/20008Ha/20006Ha/20002Ha/00`00Ha]S6`02Ha/2000XOol3000NOol0035o
o`05001oo`00Ha/00007Ha/20008Ha/00`00Ha/00006Ha/20002Ha/00`00Ha]S6`02Ha/00`00Ha]S
6`03Ha/01000Ha/000009Woo10008Goo000bOol40006Ha/01000Ha]S6`001F<K0P000f<K00<006<K
Ha/00V<K00<006<KHa/00f<K00@006<K000000IS6`8002Aoo`@002Eoo`00=7oo00<007oo00000f<K
0P0016<K00D006<KHa]S6`0000IS6`04001S6`000006Ha/30006Ha/00`00Oomoo`0POol4000YOol0
03Eoo`05001oo`00Ha/00007Ha/00`00Ha/00007Ha/30006Ha/01@00Ha]S6f<K00000f<K0P0087oo
0`00;Goo000fOol00`00Ool00008Ha/30006Ha/01@00Ha]S6f<K00000f<K0P001F<K00<006<K0000
7Woo1000<7oo000gOol30006Ha/01@00Ha]S6f<K00000f<K0P001F<K00<006<K00001f<K0P006goo
1000=7oo000iOol20003Ha/20005Ha/00`00Ha/00007Ha/30006Ha/00`00Oomoo`0GOol4000hOol0
03Yoo`800003Ha/006<K00IS6`<000IS6`05001S6f<KHa/00003Ha/2000FOol4000lOol003]oo`<0
00IS6`05001S6f<KHa/00003Ha/20005Ha/00`00Ha/0000EOol30010Ool003aoo`03001oo`0000=S
6`8000ES6`03001S6`0000MS6`80019oo`@004=oo`00?Goo0`0000=S6`00Ha/01V<K0P001V<K0P00
47oo1000Agoo000oOol30006Ha/01000Ha]S6`0016<K00<007ooOol037oo1000Bgoo0010Ool00`00
Ool00003Ha/20004Ha/01000Ha/0000037oo0`00Cgoo0011Ool01@00Ool006<K00001V<K0P002Woo
1000DWoo0012Ool30006Ha/00`00Oomoo`06Ool4001FOol004Aoo`8000=S6`8000Eoo`@005Yoo`00
AGoo0P0000=S6`00Ool00goo0`00GWoo0016Ool200000goo00000002001QOol004Moo`8006Eoo`00
[Woo0000\
\>"],
  ImageRangeCache->{{{0, 173.375}, {140.313, 0}} -> {-0.0594786, -0.0105679, 
  0.00611358, 0.00611358}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["UpAndOutPutTheta[0, 30, 50, 50, 0.25,0.1, 0.15, 1]//N", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(0.695210221227661762`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["UpAndOutPutTheta[0, 40, 50, 50, 0.25,0.1, 0.15, 0.2]//N", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(-0.838731154540347745`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Case Variant: k = h (Barrier = Strike) and q = r (Dividend Yield \
Equal to Risk-Free Rate.)\
\>", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "The strike is also 50, the volatility is 0.25 (25%), and the continuously \
compounded interest rate ",
  Cell[BoxData[
      \(TraditionalForm\`r\  = 10\)]],
  "%. The dividend yield is also 10%. The time is measured in years and \
varies from zero to 3 years. The stock price varies from 30  to 50 (Barrier). \
"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[UpAndOutPut[0, S, 50, 50, 0.25,0.1, 0.1, t],
{S, 30, 50}, {t, 0.001, 3}, PlotRange -> All];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.66533e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.80843 .15978 0 -6.10127 ]
[.80843 .15978 6 2.89873 ]
[.90116 .30178 0 -5.96725 ]
[.90116 .30178 6 3.03275 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.02411 .26511 -6 -2.74232 ]
[.02411 .26511 0 6.25768 ]
[.01568 .31796 -6 -2.787 ]
[.01568 .31796 0 6.213 ]
[.00695 .37265 -12 -2.8334 ]
[.00695 .37265 0 6.1666 ]
[-0.00208 .42929 -12 -2.88162 ]
[-0.00208 .42929 0 6.11838 ]
[-0.01144 .48797 -12 -2.93178 ]
[-0.01144 .48797 0 6.06822 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.79606 .16417 m
.78988 .16637 L
s
[(1)] .80843 .15978 -1 .35584 Mshowa
.88868 .30585 m
.88245 .30788 L
s
[(2)] .90116 .30178 -1 .32606 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(3)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.71146 .03476 m
.70779 .03618 L
s
.7336 .06862 m
.72991 .07001 L
s
.75506 .10145 m
.75136 .10281 L
s
.77587 .13328 m
.77217 .13462 L
s
.81567 .19417 m
.81195 .19546 L
s
.83471 .22329 m
.83099 .22457 L
s
.85321 .25159 m
.84948 .25285 L
s
.8712 .2791 m
.86746 .28034 L
s
.90569 .33187 m
.90195 .33307 L
s
.92225 .35719 m
.91849 .35838 L
s
.93836 .38184 m
.9346 .38301 L
s
.95406 .40585 m
.95029 .407 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
[(0)] .02411 .26511 1 -0.3906 Mshowa
.02795 .31329 m
.03408 .31095 L
s
[(5)] .01568 .31796 1 -0.38067 Mshowa
.01926 .36809 m
.02541 .36581 L
s
[(10)] .00695 .37265 1 -0.37036 Mshowa
.01027 .42485 m
.01644 .42263 L
s
[(15)] -0.00208 .42929 1 -0.35964 Mshowa
.00095 .48366 m
.00714 .4815 L
s
[(20)] -0.01144 .48797 1 -0.34849 Mshowa
.125 Mabswid
.03468 .27078 m
.03835 .26935 L
s
.03301 .2813 m
.03669 .27988 L
s
.03134 .29189 m
.03501 .29048 L
s
.02965 .30255 m
.03332 .30114 L
s
.02623 .3241 m
.02991 .3227 L
s
.02451 .33498 m
.02819 .33359 L
s
.02277 .34594 m
.02646 .34456 L
s
.02102 .35698 m
.02471 .3556 L
s
.01749 .37928 m
.02118 .37792 L
s
.0157 .39055 m
.0194 .3892 L
s
.0139 .4019 m
.0176 .40056 L
s
.01209 .41333 m
.01579 .41199 L
s
.00843 .43644 m
.01214 .43511 L
s
.00658 .44812 m
.01029 .4468 L
s
.00471 .45988 m
.00843 .45857 L
s
.00284 .47172 m
.00655 .47042 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.751 .804 .895 r
.3768 .73511 .39997 .75117 .43856 .72971 .41586 .71304 Metetra
.75 .801 .894 r
.41586 .71304 .43856 .72971 .47746 .70809 .45522 .6908 Metetra
.75 .799 .892 r
.45522 .6908 .47746 .70809 .51668 .68628 .49491 .66838 Metetra
.749 .797 .891 r
.49491 .66838 .51668 .68628 .55621 .6643 .53492 .64577 Metetra
.749 .795 .89 r
.53492 .64577 .55621 .6643 .59607 .64214 .57525 .62298 Metetra
.749 .792 .888 r
.57525 .62298 .59607 .64214 .63626 .6198 .61592 .60001 Metetra
.748 .79 .887 r
.61592 .60001 .63626 .6198 .67678 .59728 .65692 .57684 Metetra
.748 .788 .885 r
.65692 .57684 .67678 .59728 .71763 .57456 .69825 .55348 Metetra
.747 .786 .884 r
.69825 .55348 .71763 .57456 .75882 .55166 .73993 .52994 Metetra
.747 .784 .883 r
.73993 .52994 .75882 .55166 .80036 .52857 .78196 .50619 Metetra
.746 .781 .881 r
.78196 .50619 .80036 .52857 .84224 .50528 .82434 .48225 Metetra
.746 .779 .88 r
.82434 .48225 .84224 .50528 .88448 .4818 .86707 .4581 Metetra
.745 .777 .879 r
.86707 .4581 .88448 .4818 .92708 .45812 .91017 .43375 Metetra
.745 .775 .877 r
.91017 .43375 .92708 .45812 .97003 .43424 .95362 .4092 Metetra
.752 .804 .894 r
.35297 .71865 .3768 .73511 .41586 .71304 .3925 .69596 Metetra
.752 .802 .893 r
.3925 .69596 .41586 .71304 .45522 .6908 .43235 .67307 Metetra
.752 .8 .892 r
.43235 .67307 .45522 .6908 .49491 .66838 .47252 .65001 Metetra
.751 .798 .89 r
.47252 .65001 .49491 .66838 .53492 .64577 .51301 .62676 Metetra
.751 .795 .889 r
.51301 .62676 .53492 .64577 .57525 .62298 .55384 .60332 Metetra
.75 .793 .887 r
.55384 .60332 .57525 .62298 .61592 .60001 .595 .57968 Metetra
.75 .791 .886 r
.595 .57968 .61592 .60001 .65692 .57684 .63649 .55586 Metetra
.749 .789 .885 r
.63649 .55586 .65692 .57684 .69825 .55348 .67833 .53183 Metetra
.749 .786 .883 r
.67833 .53183 .69825 .55348 .73993 .52994 .72051 .50761 Metetra
.748 .784 .882 r
.72051 .50761 .73993 .52994 .78196 .50619 .76304 .48319 Metetra
.748 .782 .881 r
.76304 .48319 .78196 .50619 .82434 .48225 .80592 .45857 Metetra
.748 .78 .879 r
.80592 .45857 .82434 .48225 .86707 .4581 .84917 .43374 Metetra
.747 .778 .878 r
.84917 .43374 .86707 .4581 .91017 .43375 .89278 .4087 Metetra
.747 .775 .877 r
.89278 .4087 .91017 .43375 .95362 .4092 .93675 .38345 Metetra
.754 .805 .893 r
.32843 .7018 .35297 .71865 .3925 .69596 .36846 .67844 Metetra
.754 .803 .892 r
.36846 .67844 .3925 .69596 .43235 .67307 .4088 .6549 Metetra
.753 .801 .891 r
.4088 .6549 .43235 .67307 .47252 .65001 .44947 .63117 Metetra
.753 .798 .889 r
.44947 .63117 .47252 .65001 .51301 .62676 .49047 .60724 Metetra
.752 .796 .888 r
.49047 .60724 .51301 .62676 .55384 .60332 .5318 .58312 Metetra
.752 .794 .887 r
.5318 .58312 .55384 .60332 .595 .57968 .57346 .55881 Metetra
.752 .791 .885 r
.57346 .55881 .595 .57968 .63649 .55586 .61547 .5343 Metetra
.751 .789 .884 r
.61547 .5343 .63649 .55586 .67833 .53183 .65782 .50959 Metetra
.751 .787 .882 r
.65782 .50959 .67833 .53183 .72051 .50761 .70052 .48467 Metetra
.75 .785 .881 r
.70052 .48467 .72051 .50761 .76304 .48319 .74357 .45955 Metetra
.75 .783 .88 r
.74357 .45955 .76304 .48319 .80592 .45857 .78698 .43422 Metetra
.749 .78 .878 r
.78698 .43422 .80592 .45857 .84917 .43374 .83075 .40867 Metetra
.749 .778 .877 r
.83075 .40867 .84917 .43374 .89278 .4087 .87488 .38292 Metetra
.748 .776 .876 r
.87488 .38292 .89278 .4087 .93675 .38345 .91939 .35695 Metetra
.756 .806 .893 r
.30316 .68452 .32843 .7018 .36846 .67844 .3437 .66048 Metetra
.756 .804 .891 r
.3437 .66048 .36846 .67844 .4088 .6549 .38455 .63625 Metetra
.755 .801 .89 r
.38455 .63625 .4088 .6549 .44947 .63117 .42574 .61182 Metetra
.755 .799 .889 r
.42574 .61182 .44947 .63117 .49047 .60724 .46726 .5872 Metetra
.754 .797 .887 r
.46726 .5872 .49047 .60724 .5318 .58312 .50911 .56238 Metetra
.754 .794 .886 r
.50911 .56238 .5318 .58312 .57346 .55881 .5513 .53737 Metetra
.753 .792 .884 r
.5513 .53737 .57346 .55881 .61547 .5343 .59383 .51214 Metetra
.753 .79 .883 r
.59383 .51214 .61547 .5343 .65782 .50959 .63671 .48671 Metetra
.752 .788 .882 r
.63671 .48671 .65782 .50959 .70052 .48467 .67994 .46108 Metetra
.752 .785 .88 r
.67994 .46108 .70052 .48467 .74357 .45955 .72352 .43523 Metetra
.751 .783 .879 r
.72352 .43523 .74357 .45955 .78698 .43422 .76747 .40917 Metetra
.751 .781 .878 r
.76747 .40917 .78698 .43422 .83075 .40867 .81178 .38289 Metetra
.75 .779 .876 r
.81178 .38289 .83075 .40867 .87488 .38292 .85646 .35639 Metetra
.75 .777 .875 r
.85646 .35639 .87488 .38292 .91939 .35695 .90152 .32967 Metetra
.758 .807 .892 r
.27712 .66679 .30316 .68452 .3437 .66048 .31818 .64205 Metetra
.757 .804 .89 r
.31818 .64205 .3437 .66048 .38455 .63625 .35957 .6171 Metetra
.757 .802 .889 r
.35957 .6171 .38455 .63625 .42574 .61182 .40129 .59196 Metetra
.756 .8 .888 r
.40129 .59196 .42574 .61182 .46726 .5872 .44334 .56662 Metetra
.756 .797 .886 r
.44334 .56662 .46726 .5872 .50911 .56238 .48573 .54107 Metetra
.755 .795 .885 r
.48573 .54107 .50911 .56238 .5513 .53737 .52846 .51532 Metetra
.755 .793 .884 r
.52846 .51532 .5513 .53737 .59383 .51214 .57154 .48936 Metetra
.754 .79 .882 r
.57154 .48936 .59383 .51214 .63671 .48671 .61496 .46319 Metetra
.754 .788 .881 r
.61496 .46319 .63671 .48671 .67994 .46108 .65874 .4368 Metetra
.753 .786 .879 r
.65874 .4368 .67994 .46108 .72352 .43523 .70288 .4102 Metetra
.753 .784 .878 r
.70288 .4102 .72352 .43523 .76747 .40917 .74739 .38338 Metetra
.752 .781 .877 r
.74739 .38338 .76747 .40917 .81178 .38289 .79226 .35634 Metetra
.752 .779 .875 r
.79226 .35634 .81178 .38289 .85646 .35639 .8375 .32908 Metetra
.751 .777 .874 r
.8375 .32908 .85646 .35639 .90152 .32967 .88312 .30158 Metetra
.76 .807 .891 r
.25027 .64861 .27712 .66679 .31818 .64205 .29188 .62313 Metetra
.759 .805 .889 r
.29188 .62313 .31818 .64205 .35957 .6171 .33382 .59744 Metetra
.759 .803 .888 r
.33382 .59744 .35957 .6171 .40129 .59196 .37609 .57156 Metetra
.758 .8 .887 r
.37609 .57156 .40129 .59196 .44334 .56662 .41869 .54546 Metetra
.758 .798 .885 r
.41869 .54546 .44334 .56662 .48573 .54107 .46164 .51916 Metetra
.757 .796 .884 r
.46164 .51916 .48573 .54107 .52846 .51532 .50493 .49265 Metetra
.757 .793 .883 r
.50493 .49265 .52846 .51532 .57154 .48936 .54857 .46592 Metetra
.756 .791 .881 r
.54857 .46592 .57154 .48936 .61496 .46319 .59256 .43898 Metetra
.756 .789 .88 r
.59256 .43898 .61496 .46319 .65874 .4368 .63691 .41182 Metetra
.755 .787 .879 r
.63691 .41182 .65874 .4368 .70288 .4102 .68162 .38444 Metetra
.755 .784 .877 r
.68162 .38444 .70288 .4102 .74739 .38338 .72669 .35684 Metetra
.754 .782 .876 r
.72669 .35684 .74739 .38338 .79226 .35634 .77214 .329 Metetra
.753 .78 .875 r
.77214 .329 .79226 .35634 .8375 .32908 .81796 .30094 Metetra
.753 .778 .873 r
.81796 .30094 .8375 .32908 .88312 .30158 .86416 .27265 Metetra
.762 .808 .89 r
.22257 .62994 .25027 .64861 .29188 .62313 .26475 .6037 Metetra
.761 .806 .889 r
.26475 .6037 .29188 .62313 .33382 .59744 .30725 .57724 Metetra
.761 .803 .887 r
.30725 .57724 .33382 .59744 .37609 .57156 .3501 .55058 Metetra
.76 .801 .886 r
.3501 .55058 .37609 .57156 .41869 .54546 .39328 .52371 Metetra
.76 .799 .885 r
.39328 .52371 .41869 .54546 .46164 .51916 .4368 .49662 Metetra
.759 .796 .883 r
.4368 .49662 .46164 .51916 .50493 .49265 .48067 .46932 Metetra
.758 .794 .882 r
.48067 .46932 .50493 .49265 .54857 .46592 .52489 .4418 Metetra
.758 .792 .88 r
.52489 .4418 .54857 .46592 .59256 .43898 .56946 .41406 Metetra
.757 .789 .879 r
.56946 .41406 .59256 .43898 .63691 .41182 .6144 .3861 Metetra
.757 .787 .878 r
.6144 .3861 .63691 .41182 .68162 .38444 .6597 .35791 Metetra
.756 .785 .876 r
.6597 .35791 .68162 .38444 .72669 .35684 .70536 .32949 Metetra
.756 .783 .875 r
.70536 .32949 .72669 .35684 .77214 .329 .7514 .30084 Metetra
.755 .78 .874 r
.7514 .30084 .77214 .329 .81796 .30094 .79782 .27195 Metetra
.755 .778 .872 r
.79782 .27195 .81796 .30094 .86416 .27265 .84462 .24283 Metetra
.764 .809 .889 r
.19399 .61077 .22257 .62994 .26475 .6037 .23675 .58373 Metetra
.763 .806 .888 r
.23675 .58373 .26475 .6037 .30725 .57724 .27984 .55648 Metetra
.762 .804 .886 r
.27984 .55648 .30725 .57724 .3501 .55058 .32328 .52901 Metetra
.762 .802 .885 r
.32328 .52901 .3501 .55058 .39328 .52371 .36705 .50133 Metetra
.761 .799 .884 r
.36705 .50133 .39328 .52371 .4368 .49662 .41117 .47343 Metetra
.761 .797 .882 r
.41117 .47343 .4368 .49662 .48067 .46932 .45564 .44531 Metetra
.76 .795 .881 r
.45564 .44531 .48067 .46932 .52489 .4418 .50046 .41697 Metetra
.76 .792 .88 r
.50046 .41697 .52489 .4418 .56946 .41406 .54564 .3884 Metetra
.759 .79 .878 r
.54564 .3884 .56946 .41406 .6144 .3861 .59118 .3596 Metetra
.759 .788 .877 r
.59118 .3596 .6144 .3861 .6597 .35791 .63709 .33057 Metetra
.758 .785 .876 r
.63709 .33057 .6597 .35791 .70536 .32949 .68337 .3013 Metetra
.757 .783 .874 r
.68337 .3013 .70536 .32949 .7514 .30084 .73002 .2718 Metetra
.757 .781 .873 r
.73002 .2718 .7514 .30084 .79782 .27195 .77705 .24206 Metetra
.756 .779 .872 r
.77705 .24206 .79782 .27195 .84462 .24283 .82447 .21208 Metetra
.765 .81 .888 r
.16446 .59107 .19399 .61077 .23675 .58373 .20783 .5632 Metetra
.765 .807 .887 r
.20783 .5632 .23675 .58373 .27984 .55648 .25154 .53512 Metetra
.764 .805 .885 r
.25154 .53512 .27984 .55648 .32328 .52901 .29559 .50682 Metetra
.764 .802 .884 r
.29559 .50682 .32328 .52901 .36705 .50133 .33998 .47829 Metetra
.763 .8 .883 r
.33998 .47829 .36705 .50133 .41117 .47343 .38471 .44955 Metetra
.763 .798 .881 r
.38471 .44955 .41117 .47343 .45564 .44531 .4298 .42058 Metetra
.762 .795 .88 r
.4298 .42058 .45564 .44531 .50046 .41697 .47525 .39138 Metetra
.761 .793 .879 r
.47525 .39138 .50046 .41697 .54564 .3884 .52105 .36195 Metetra
.761 .791 .877 r
.52105 .36195 .54564 .3884 .59118 .3596 .56722 .33228 Metetra
.76 .788 .876 r
.56722 .33228 .59118 .3596 .63709 .33057 .61376 .30238 Metetra
.76 .786 .875 r
.61376 .30238 .63709 .33057 .68337 .3013 .66067 .27224 Metetra
.759 .784 .873 r
.66067 .27224 .68337 .3013 .73002 .2718 .70796 .24186 Metetra
.758 .782 .872 r
.70796 .24186 .73002 .2718 .77705 .24206 .75563 .21123 Metetra
.758 .779 .871 r
.75563 .21123 .77705 .24206 .82447 .21208 .80369 .18035 Metetra
.767 .81 .887 r
.13395 .57081 .16446 .59107 .20783 .5632 .17796 .54208 Metetra
.767 .808 .886 r
.17796 .54208 .20783 .5632 .25154 .53512 .2223 .51313 Metetra
.766 .805 .884 r
.2223 .51313 .25154 .53512 .29559 .50682 .26698 .48396 Metetra
.766 .803 .883 r
.26698 .48396 .29559 .50682 .33998 .47829 .31201 .45457 Metetra
.765 .801 .882 r
.31201 .45457 .33998 .47829 .38471 .44955 .35739 .42495 Metetra
.764 .798 .88 r
.35739 .42495 .38471 .44955 .4298 .42058 .40312 .39509 Metetra
.764 .796 .879 r
.40312 .39509 .4298 .42058 .47525 .39138 .44922 .365 Metetra
.763 .794 .878 r
.44922 .365 .47525 .39138 .52105 .36195 .49567 .33468 Metetra
.763 .791 .876 r
.49567 .33468 .52105 .36195 .56722 .33228 .54249 .30411 Metetra
.762 .789 .875 r
.54249 .30411 .56722 .33228 .61376 .30238 .58968 .27331 Metetra
.761 .787 .874 r
.58968 .27331 .61376 .30238 .66067 .27224 .63724 .24226 Metetra
.761 .784 .872 r
.63724 .24226 .66067 .27224 .70796 .24186 .68519 .21096 Metetra
.76 .782 .871 r
.68519 .21096 .70796 .24186 .75563 .21123 .73351 .17941 Metetra
.76 .78 .87 r
.73351 .17941 .75563 .21123 .80369 .18035 .78223 .14761 Metetra
.769 .811 .886 r
.1024 .54997 .13395 .57081 .17796 .54208 .14707 .52034 Metetra
.769 .808 .885 r
.14707 .52034 .17796 .54208 .2223 .51313 .19207 .4905 Metetra
.768 .806 .883 r
.19207 .4905 .2223 .51313 .26698 .48396 .23742 .46043 Metetra
.768 .804 .882 r
.23742 .46043 .26698 .48396 .31201 .45457 .28311 .43012 Metetra
.767 .801 .881 r
.28311 .43012 .31201 .45457 .35739 .42495 .32916 .39958 Metetra
.766 .799 .879 r
.32916 .39958 .35739 .42495 .40312 .39509 .37556 .36881 Metetra
.766 .797 .878 r
.37556 .36881 .40312 .39509 .44922 .365 .42232 .3378 Metetra
.765 .794 .877 r
.42232 .3378 .44922 .365 .49567 .33468 .46944 .30655 Metetra
.764 .792 .875 r
.46944 .30655 .49567 .33468 .54249 .30411 .51694 .27505 Metetra
.764 .79 .874 r
.51694 .27505 .54249 .30411 .58968 .27331 .5648 .2433 Metetra
.763 .787 .873 r
.5648 .2433 .58968 .27331 .63724 .24226 .61304 .21131 Metetra
.763 .785 .871 r
.61304 .21131 .63724 .24226 .68519 .21096 .66167 .17906 Metetra
.762 .783 .87 r
.66167 .17906 .68519 .21096 .73351 .17941 .71068 .14656 Metetra
.761 .78 .869 r
.71068 .14656 .73351 .17941 .78223 .14761 .76008 .1138 Metetra
.771 .812 .885 r
.06976 .52851 .1024 .54997 .14707 .52034 .11511 .49796 Metetra
.771 .809 .884 r
.11511 .49796 .14707 .52034 .19207 .4905 .1608 .46718 Metetra
.77 .807 .882 r
.1608 .46718 .19207 .4905 .23742 .46043 .20684 .43616 Metetra
.769 .804 .881 r
.20684 .43616 .23742 .46043 .28311 .43012 .25322 .40491 Metetra
.769 .802 .88 r
.25322 .40491 .28311 .43012 .32916 .39958 .29996 .37343 Metetra
.768 .8 .878 r
.29996 .37343 .32916 .39958 .37556 .36881 .34705 .3417 Metetra
.768 .797 .877 r
.34705 .3417 .37556 .36881 .42232 .3378 .39451 .30973 Metetra
.767 .795 .876 r
.39451 .30973 .42232 .3378 .46944 .30655 .44233 .27751 Metetra
.766 .793 .874 r
.44233 .27751 .46944 .30655 .51694 .27505 .49052 .24504 Metetra
.766 .79 .873 r
.49052 .24504 .51694 .27505 .5648 .2433 .53909 .21232 Metetra
.765 .788 .872 r
.53909 .21232 .5648 .2433 .61304 .21131 .58804 .17934 Metetra
.764 .786 .87 r
.58804 .17934 .61304 .21131 .66167 .17906 .63736 .14611 Metetra
.764 .783 .869 r
.63736 .14611 .66167 .17906 .71068 .14656 .68708 .11262 Metetra
.763 .781 .868 r
.68708 .11262 .71068 .14656 .76008 .1138 .73719 .07886 Metetra
.773 .812 .884 r
.03597 .50641 .06976 .52851 .11511 .49796 .08203 .47489 Metetra
.773 .81 .882 r
.08203 .47489 .11511 .49796 .1608 .46718 .12843 .44313 Metetra
.772 .807 .881 r
.12843 .44313 .1608 .46718 .20684 .43616 .17519 .41114 Metetra
.771 .805 .88 r
.17519 .41114 .20684 .43616 .25322 .40491 .22229 .37891 Metetra
.771 .803 .879 r
.22229 .37891 .25322 .40491 .29996 .37343 .26975 .34643 Metetra
.77 .8 .877 r
.26975 .34643 .29996 .37343 .34705 .3417 .31756 .31371 Metetra
.77 .798 .876 r
.31756 .31371 .34705 .3417 .39451 .30973 .36574 .28074 Metetra
.769 .795 .875 r
.36574 .28074 .39451 .30973 .44233 .27751 .41429 .24751 Metetra
.768 .793 .873 r
.41429 .24751 .44233 .27751 .49052 .24504 .46321 .21404 Metetra
.768 .791 .872 r
.46321 .21404 .49052 .24504 .53909 .21232 .5125 .18031 Metetra
.767 .788 .871 r
.5125 .18031 .53909 .21232 .58804 .17934 .56218 .14631 Metetra
.766 .786 .869 r
.56218 .14631 .58804 .17934 .63736 .14611 .61224 .11206 Metetra
.765 .784 .868 r
.61224 .11206 .63736 .14611 .68708 .11262 .66268 .07753 Metetra
.765 .782 .867 r
.66268 .07753 .68708 .11262 .73719 .07886 .71353 .04274 Metetra
.775 .813 .883 r
.00095 .48363 .03597 .50641 .08203 .47489 .04776 .4511 Metetra
.775 .81 .881 r
.04776 .4511 .08203 .47489 .12843 .44313 .09491 .41833 Metetra
.774 .808 .88 r
.09491 .41833 .12843 .44313 .17519 .41114 .14241 .38531 Metetra
.773 .806 .879 r
.14241 .38531 .17519 .41114 .22229 .37891 .19026 .35206 Metetra
.773 .803 .877 r
.19026 .35206 .22229 .37891 .26975 .34643 .23847 .31855 Metetra
.772 .801 .876 r
.23847 .31855 .26975 .34643 .31756 .31371 .28704 .28479 Metetra
.771 .798 .875 r
.28704 .28479 .31756 .31371 .36574 .28074 .33597 .25079 Metetra
.771 .796 .874 r
.33597 .25079 .36574 .28074 .41429 .24751 .38527 .21652 Metetra
.77 .794 .872 r
.38527 .21652 .41429 .24751 .46321 .21404 .43494 .182 Metetra
.769 .791 .871 r
.43494 .182 .46321 .21404 .5125 .18031 .48499 .14721 Metetra
.769 .789 .87 r
.48499 .14721 .5125 .18031 .56218 .14631 .53543 .11216 Metetra
.768 .787 .868 r
.53543 .11216 .56218 .14631 .61224 .11206 .58624 .07684 Metetra
.767 .784 .867 r
.58624 .07684 .61224 .11206 .66268 .07753 .63745 .04125 Metetra
.767 .782 .866 r
.63745 .04125 .66268 .07753 .71353 .04274 .68905 .00539 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{174.375, 141.313},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002^0000SA000`40O003h00O[Woo002^Ool00:io
o`00[Woo001_Ool30004Ool2000fOol0079oo`04001oogoo0002Ool00`00Oomoo`0cOol0079oo`04
001oogoo0002Ool00`00Oomoo`0cOol006moo`<000=oo`04001oogoo000eOol006moo`03001oogoo
00=oo`04001oogoo000eOol006moo`@000=oo`8003Ioo`00[Woo002^Ool007Qoo`8003Aoo`00MGoo
1P00<goo001cOol20002Ool01000Hc/00000<goo001COol30002Ool3000EOol5000017oo001S>f<k
0`00<Woo001DOol00`00Oomoo`04Ool00`00Oomoo`0@Ool20004Ool20003Hc/00`00Hc/0000bOol0
059oo`@000Eoo`03001oogoo00eoo`@000Aoo`03001S>f<k00=S>`04001S>`00000aOol0059oo`03
001oo`0000=oo`<000eoo`<000Moo`03001S>f<k00AS>`03001S>f<k0080031oo`00DWoo00<007oo
00000goo00<007ooOol02goo0`001goo0P001f<k00<006<kHc/00P00<7oo001COol20003Ool40007
Ool30009Ool00`00Hc/00007Hc/01000Hc/006<k0P00;goo001QOol30009Ool20003Hc/00`00Hc]S
>`04Hc/20003Hc/2000^Ool005ioo`<0009oo`03001oogoo00Eoo`8000IS>`04001S>f<kHc/30005
Hc/00`00Oomoo`0/Ool005aoo`8000aoo`03001S>f<k00MS>`03001S>`0000=S>`03001S>f<k009S
>`8002eoo`00FGoo0`000Woo00<007ooOol01goo0P002V<k0P0016<k00<006<kHc/00f<k0P00;7oo
000hOol30003Ool2000GOol2000>Ool00`00Hc/00009Hc/00`00Hc/00004Hc/00`00Hc]S>`03Hc/2
000/Ool003Uoo`05001oogooOol00002Ool00`00Oomoo`0AOol300000goo001oo`0<Ool01@00Hc]S
>f<k00001f<k00D006<kHc]S>`0000=S>`03001S>f<k009S>`04001S6`00000[Ool003Moo`@0009o
o`04001oogoo000@Ool3000@Ool20005Hc/00`00Hc]S>`02Hc/20005Hc/01000Hc]S>`000V<k0P00
0f<K0P00:Woo000gOol00`00Ool00003Ool01000Oomoo`003Woo0P000Woo00<007ooOol03Goo00<0
06<kHc/01F<k00D006<kHc]S>`0000MS>`03001S>f<k00<000IS6`03001oogoo02Qoo`00=goo00<0
07oo00000goo00@007ooOol000]oo`<001=oo`03001S>f<k00MS>`03001S>`0000US>`03001S>`00
00QS6`8002Uoo`00>7oo0P0017oo0P002Woo0P000Woo00<007ooOol03goo0P002V<k0P002V<k0`00
26<K0P00:Goo0017Ool3000EOol00`00Hc/00009Hc/00`00Hc/00009Hc/01000Hc/000001f<K00@0
06<K000002Qoo`00AGoo0P0000=oo`00Ool04goo0P000f<k00<006<kHc/016<k0P000f<k00<006<k
Hc/016<k0P000f<k00<006<KHa/00f<K0P000f<K0P009goo0012Ool3000FOol20006Hc/01000Hc]S
>f<k0P001V<k00@006<kHc]S>`8000ES>`80009S6`8000IS6`8002Ioo`00@7oo0P000Woo00<007oo
Ool04goo00<006<kHc/01f<k00<006<k00002F<k00<006<k00001f<k0P0000=S6`00Ha/026<K00<0
07ooOol00goo0`007Woo000NOol30003Ool3000FOol3000HOol2000:Hc/2000:Hc/20008Hc/3000:
Ha/20005Ool00`00Oomoo`0MOol0025oo`03001oogoo00=oo`03001oogoo015oo`800003Ool007oo
01Moo`03001S>`0000US>`03001S>`0000US>`03001S>`0000QS>`03001S>`0000US6`03001S6`00
00Eoo`03001oogoo01eoo`008Goo00<007ooOol00goo00<007ooOol03Woo0`006goo00D006<kHc]S
>`0000IS>`8000=S>`03001S>f<k00AS>`8000=S>`03001S>f<k00=S>`<0009S>`03001S6f<K00AS
6`8000=S6`03001oogoo009oo`03001oogoo01eoo`007goo0P000goo0`003Woo0`0000=oo`00Ool0
6Goo0P001F<k00@006<kHc]S>`8000IS>`04001S>f<kHc/20006Hc/01@00Hc]S>f<k00000V<k00D0
06<kHc]S>`0000=S6`8000IS6`05001oogooOol0000OOol0025oo`04001oogoo000>Ool2000NOol0
0`00Hc]S>`06Hc/00`00Hc/00009Hc/00`00Hc/00009Hc/00`00Hc/00003Hc/00`00Hc]S>`02Hc/0
0`00Ha/00008Ha/00`00Oomoo`02000OOol001ioo`<000=oo`@000Qoo`<00003Ool007oo01aoo`03
001S>f<k00MS>`8000YS>`8000YS>`8000AS>`03001S>f<k009S>`8000US6`80029oo`00;Woo0P00
87oo0P002F<k00@006<kHc/000QS>`03001S>`0000US>`03001S>`0000AS>`07001S>f<kHc/006<k
000026<K00@006<K0000025oo`00:goo0`000Woo00<007ooOol077oo00@006<kHc/000MS>`03001S
>f<k009S>`03001S>f<k00=S>`8000=S>`03001S>f<k00AS>`8000=S>`03001S>f<k009S>`<000=S
>`03001S6f<K00=S6`8000=S6`03001oogoo01moo`00:Goo0P008Woo0P0016<k00<006<kHc/00f<k
00<006<kHc/016<k00D006<kHc]S>`0000IS>`04001S>f<kHc/20006Hc/00`00Hc]S>`020006Hc/0
0`00Ha]S6`020006Ha/00`00Oomoo`0NOol000Moo`<000Aoo`8001Ioo`<0009oo`03001oogoo01io
o`03001S>f<k00ES>`05001S>f<kHc/00008Hc/00`00Hc/00008Hc/00`00Hc/00008Hc/01@00Hc/0
06<k00001V<k00<006<K000026<K0P007goo000:Ool01000Oomoo`000Woo00<007ooOol04Goo0`00
97oo00<006<kHc/01f<k00<006<k00002F<k0P002F<k0P002F<k0P000V<k00<006<kHc/016<k0P00
2F<K0P007goo000:Ool01000Oomoo`000Woo00<007ooOol03Woo0`009Goo0P002V<k0P002F<k00<0
06<k000026<k00<006<k000026<k00<006<k00000V<k00<006<kHc/00f<k00<006<k000026<K00@0
06<K000001ioo`0027oo0P000goo00@007ooOol000eoo`@002Ioo`03001S>`0000US>`03001S>`00
00QS>`05001S>f<kHc/00005Hc/20003Hc/00`00Hc]S>`03Hc/20003Hc/00`00Hc/00003Hc/20003
Hc/00`00Ha]S6`03Ha/20003Ha/2000MOol000Yoo`04001oogoo0002Ool00`00Oomoo`09Ool2000X
Ool20003Hc/00`00Hc]S>`04Hc/20003Hc/00`00Hc]S>`03Hc/20005Hc/01@00Hc]S>f<k00001V<k
00D006<kHc]S>`0000IS>`80009S>`03001S>f<k00AS>`05001S6f<KHa/00006Ha/00`00Oomoo`0K
Ool000Moo`<000Aoo`8000Uoo`D002Ioo`8000IS>`04001S>f<kHc/20006Hc/01@00Hc]S>f<k0000
26<k00<006<k000026<k00<006<k000026<k00<006<k000026<k00<006<K00001f<K0P0077oo000G
Ool2000ZOol00`00Hc]S>`07Hc/00`00Hc/00009Hc/00`00Hc/00009Hc/20009Hc/20009Hc/20009
Hc/20008Ha/3000KOol001Aoo`<00003Ool007oo02Moo`8000YS>`8000YS>`8000US>`03001S>`00
00QS>`03001S>`0000QS>`<000QS>`03001S6`0000MS6`04001S6`00000KOol001=oo`8002]oo`03
001S>`0000US>`03001S>`0000US>`04001S>f<k0006Hc/20003Hc/00`00Hc]S>`03Hc/20003Hc/0
0`00Hc]S>`02Hc/3000016<k001S>`001F<k0P000f<K00<006<KHa/00V<K0P000f<K0P006Woo000C
Ool3000YOol01@00Hc]S>f<k00001V<k0P000f<k00<006<kHc/016<k0P0016<k00@006<kHc]S>`80
00IS>`05001S>f<kHc/00006Hc/50004Hc/00`00Hc/00002Hc/20006Ha/01000Ha]S6`001V<K0P00
6Goo000=Ool20004Ool01000Oomoo`009Woo0P001F<k00@006<kHc]S>`8000IS>`04001S>f<kHc/2
0007Hc/00`00Hc/00009Hc/00`00Hc/00005Hc/400000f<k001S>`05Hc/200000f<k001S6`07Ha/0
0`00Ha/00007Ha/2000IOol000aoo`04001oogoo0003Ool01@00Oomoogoo000097oo00<006<kHc/0
1V<k00<006<k00002F<k00<006<k00002F<k0P002V<k0P000f<k0`000f<k0P0026<k0P002F<K0P00
26<K0`0067oo000<Ool01000Oomoo`000Woo0P0017oo00<007ooOol087oo00<006<kHc/01f<k0P00
2V<k0`0026<k00<006<k00002F<k00<006<k00000`001F<k00<006<k00001V<k0`0000=S>`00Ha/0
1F<K0P0000=S6`00Ha/01F<K0P0000AS6`00000000Aoo`@000moo`0037oo00@007ooOol0009oo`03
001oogoo00Aoo`03001oogoo01eoo`<000QS>`04001S?6<l0008Hc/01@00HcaS?6<l00001F<k0P00
0f<k00<006<kHc/016<k1P001F<k0P000f<k00<006<kHc/00V<k00L006<kHc]S>`00Hc/00004Ha/0
0`00Ha]S6`02Ha/00`00Ha]S6`02Ha/00`00Ha]S6`02Ha/20005Ool00`00Oomoo`0?Ool000aoo`04
001oogoo0002Ool20006Ool00`00Oomoo`0KOol01@00HcaS?6<l00001F<k0P0016<l00<006<kHc/0
0f<k0P001F<l00D006<kHc]S>`0000IS>`04001S>f<kHc/40004Hc/00`00Hc]S>`020006Hc/01000
Hc/000001F<k00@006<k001S6`8000IS6`04001S6`000006Ha/20005Ool00`00Oomoo`0>Ool000eo
o`8000=oo`8000Moo`03001oogoo01Qoo`8000ES?005001S>f<kHc/00007Hc`01@00Hc]S>f<k0000
26<l00<006<k000026<k10001f<k00<006<k000026<k0P0026<k0`0026<K0P0026<K0`001Goo00<0
07ooOol03Goo000BOol00`00Oomoo`07Ool00`00Oomoo`0FOol00`00HcaS?006Hc`00`00Hc/00009
Hc`00`00Hc/00009Hc`20007Hc/40009Hc/20008Hc/00`00Hc`00008Hc/00`00Hc/00007Ha/00`00
Ha/00007Ha/01000Ha/000000Woo00@007ooOol000moo`004Woo0P002Goo00<007ooOol057oo00<0
06<lHc`01f<l0P002V<l0P002F<l00<006<l00001F<k0P0000ES>`00HcaS?00000IS>`80009S?003
001S>f<k00=S>`8000=S?003001S>f<k00=S>`800004Hc/006<k0004Ha/20003Ha/00`00Ha]S6`02
Ha/20003Ha/20002Ool2000@Ool000]oo`<000=oo`03001oogoo00Yoo`03001oogoo015oo`8000US
?004001S?6<l0008Hc`01000HcaS?0001V<l0P000f<l00<006<k00000P0000=S>`00000016<l00<0
06<kHc/00V<k00<006<lHc`00f<l00D006<kHc]S>`0000IS?005001S>f<kHc/00004Hc/00`00Hc/0
0002Ha/00`00Ha]S6`04Ha/01000Ha]S6`001V<K0P004goo000>Ool00`00Oomoo`02000<Ool00`00
Oomoo`0?Ool01000HcaS?0001V<l0P0016<l00<006<lHc`00f<l0P0016<l00@006<lHcaS?08000AS
?0<000=S>`03001S?6<l00ES?004001S>`000007Hc`00`00Hc/00008Hc`00`00Hc/00006Hc/20000
0f<K001S6`06Ha/00`00Ha/00007Ha/2000COol000ioo`03001oogoo008000eoo`03001oogoo00ao
o`8000AS?004001S?6<lHc`20007Hc`01@00HcaS?6<l00001f<l00<006<l00000f<l0`000f<l00<0
06<k000026<l0P002F<l0P002F<l0P001f<k0`0026<K0P0026<K0`004Woo000;Ool30003Ool00`00
Oomoo`0=Ool2000:Ool20007Hc`00`00Hc`0000:Hc`00`00Hc`00008Hc`60006Hc`20008Hc`00`00
Hc`00008Hc`00`00Hc`00007Hc`200000f<l001S>`05Hc/40006Ha/200000f<K001S6`05Ha/20000
0elK0000000BOol000]oo`03001oogoo00=oo`03001oogoo00moo`03001oogoo00Ioo`03001S?6<l
00MS?08000]S?08000QS?0D000IS?080009S?003001S?6<l00=S?08000=S?003001S?6<l00=S?080
00=S?003001S?6<l009S?08000AS?003001S>f<k009S>`04001S?6<lHc`20004Ha/00`00Ha]S6`02
Ha/00`00Ha]S6`02Ha/00`00Ga]O6`02Ga/2000AOol000]oo`@0009oo`03001oogoo011oo`03001o
ogoo00=oo`8000US?004001S?6<l0009Hc`01000HcaS?0001F<l0`0016<l00<006<lHc`00V<l00<0
06<lHc`00f<l00D006<lHcaS?00000IS?005001S?6<lHc`00006Hc`01000Hc`000001f<l00@006<k
000000ES?0800003Ha/0000000IS6`04001S6`000006Ga/2000@Ool0011oo`8001=oo`05001oogoo
Ool00002Hc`00`00HcaS?005Hc`00`00HcaS?002Hc`00`00HcaS?004Hc`20004Hc`00`00Hc`00002
0008Hc`01000Hc`000001f<l00<006<l000026<l00<006<l00001f<l0P002F<l0P001f<l0`0026<K
0P0025lK0`003goo000@Ool2000DOol00`00Ool00004Hc`00`00HcaS?003Hc`00`00HcaS?004Hc`0
1000HcaS?6<l0P001F<l0`0000=S?000Hc`026<l0P002F<l0P002F<l0P001f<l00@006<lHc`000IS
?0800003Hc`006<l00ES?0@000IS6`800003Ga`005lK00EO6`800003Ga`0000000moo`000goo0`00
17oo0P0017oo00<007ooOol057oo00<006<lHc`016<l00D006<lHcaS?00000QS?003001S?00000ES
?080009S?08000US?004001S?6<l0006Hc`20002Hc`00`00HcaS?004Hc`200000f<l001S?004Hc`2
0004Hc`00`00HcaS?002Hc`00`00HcaS?002Hc`00`00HcaS?002Hc`01000HaaS76<L0P0016<K00<0
05lLGa`00UlL00<005lKGa/00UlK00<005lLGa`00UlL0P003Woo0004Ool00`00Oomoo`02Ool01000
Oomoo`000goo0P0057oo00<006<lHc`01V<l00<006<l00002F<l0`000V<l0`000f<l00@006<lHc`0
00IS?08000AS?003001S?6<l009S?003001S?6<l00=S?003001S?6<l009S?003001S?6<l009S?005
001S?6<lHc`00007Hc`01000Hc`000001V<l00@006<l000000ES70800003Ha/0000000IO7004001O
6`000006Ga`00`00Oomoo`0<Ool000Aoo`03001oogoo009oo`04001oogoo0003Ool00`00Oomoo`0B
Ool00`00HcaS?007Hc`30008Hc`00`00HcaS?0030004Hc`20004Hc`01000HcaS?6<l0P001f<l00@0
06<l000000MS?004001S?0000006Hc`00`00Hc`00008Hc`20008Hc`20007Ha`30008Ga`20008Ga`2
000=Ool000Aoo`03001oogoo009oo`04001oogoo0003Ool00`00Oomoo`0@Ool30008Hc`01@00HcaS
?6<l00001F<l1@000V<l00D006<lHcaS?00000MS?003001S?00000US?08000US?08000QS?08000QS
?003001S?00000IS?0800003Hc`006<L00ES70@000IO70800003Ga`005lL00EO70800003Ga`00000
00aoo`0017oo00<007ooOol00Woo00@007ooOol000=oo`03001oogoo00moo`05001S?6<lHc`00005
Hc`20005Hc`01000HcaS?6<l0`001V<l00<006<l000026<l0`0026<l00@006<lHc`000IS?0800003
Hc`006<l00ES?080009S?003001S?6<l00=S?08000=S?003001S?6<l009S?003001S?6<l009S?003
001S76<L009S7004001S76<LHa`20004Ga`00`00GaaO7002Ga`00`00GaaO7002Ga`00`00GaaO7002
Ga`00`00Oomoo`0:Ool000=oo`8000Eoo`8000=oo`8000moo`8000ES?005001S?6<lHc`00008Hc`3
0009Hc`20008Hc`01@00HcaS?6<l00001F<l0P0016<l00<006<lHc`00V<l00<006<lHc`00V<l00<0
06<lHc`00V<l00<006<lHc`00f<l00D006<lHcaS?00000IS?004001S?0000006Hc`01000Ha`00000
1F<L0P0000=O700000001UlL00@005lL000000EO708000Eoo`<000=oo`003goo00<007ooOol037oo
0P0026<l00<006<l00001V<l1P0026<l00@006<lHc`000ES?08000ES?005001S?6<lHc`00007Hc`0
1000Hc`000001V<l00@006<l000000MS?003001S?00000MS?08000QS?08000MS70<000QO708000MO
70<000Moo`40005oo`5oo`003goo0P0037oo00<006<lHc`026<l0`000f<l0`0000=S?00000000f<l
00<006<lHc`00f<l0P0016<l00D006<lHcaS?00000QS?003001S?00000QS?08000QS?08000US?080
00MS?003001S?00000IS?0800003Hc`006<L00ES70@000IO70800003Ga`005lL00EO7004001O7000
0007Ool10001Ool1Ool000moo`8000Yoo`<000US?004001S?6<lHc`30003Hc`00`00HcaS?004Hc`0
1@00HcaS?6<l00001f<l00<006<l00002F<l0P0026<l00<006<l00001V<l0P000V<l00<006<lHc`0
16<l0P0000=S?000Hc`016<l0P000f<l00<006<lHc`00V<l00<006<lHc`00V<l00<006<LHa`00V<L
00@005lLGaaO708000AO7003001O75lL009O7004001O75lLGa`20003Ga`20004Ool20003Ool0009o
o`<000=oo`<000Aoo`03001oogoo00Qoo`05001S?6<lHc`00006Hc`2000016<l000000000V<l00@0
06<l000000QS?003001S?00000QS?08000US?004001S?6<l0005Hc`20003Hc`00`00HcaS?002Hc`0
0`00HcaS?003Hc`00`00HcaS?002Hc`00`00HcaS?002Hc`01@00HcaS?6<l00001V<l00@006<l0000
00IS?004001S70000005Ga`200000elL00000006Ga`00`00Ga`00006Ga`20005Ool10001Ool1Ool0
00=oo`03001oogoo00Eoo`05001oogooOol00008Ool20005Hc`01000HcaS?6<l0P0000=S?0000000
1F<l0P002V<l0`001V<l0P000V<l00<006<lHc`016<l0P0016<l00D006<lHcaS?00000IS?004001S
?0000007Hc`01000Hc`000001V<l00<006<l00001f<l0P0026<l0P001elL0`0025lL0P001elL0P00
0Woo0`000goo0003Ool00`00Oomoo`05Ool00`00Oomoo`020007Ool00`00HcaS?006Hc`00`00Hc`0
00030006Hc`01000HcaS?0001f<l0P000f<l00<006<lHc`00V<l00<006<lHc`00f<l00@006<lHcaS
?08000MS?003001S?00000MS?08000US?08000QS?0<000IS?004001S?6<l0006Hc`00`00Gc`00006
Ga`40006Ga`200000elL001O7004Ga`200000goo001oo`07Ool000=oo`03001oogoo009oo`<000=o
o`8000Ioo`03001S?6<l00MS?0<000MS?08000AS?003001S?6<l009S?08000IS?004001S?0000007
Hc`00`00Hc`00009Hc`20007Hc`01000HcaS?0001V<l0P0000=S?000Hc`01F<l0P000f<l00@006<l
HcaS?08000AS?004001S?6<lHc`20003Gc`00`00GaaO7002Ga`01000GaaO75lL0P0015lL00<005lL
Ga`00UlL00D005lLGaaO700000=oo`03001oogoo00Ioo`000goo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00goo0`001V<l0`000f<l0P000f<l0P001f<l00@006<l000000QS?08000US?0<0
00MS?0800003Hc`006<l00AS?08000AS?003001S?6<l009S?003001S?6<l009S?003001S?6<l009S
?003001S?6<l00AS?003001S?00000MS?003001S?00000IO?004001O70000005Ga`200000elL0000
0005Ga`01000Ga`000001Goo00<007ooOol01Goo0002Ool20004Ool40002Ool20003Ool01@00HeaS
G6=L00000V<l1@001V<l00<006<l00002F<l0`0026<l00@006<lHc`000IS?08000=S?08000AS?003
001S?6<l009S?005001S?6<lHc`00007Hc`01000Hc`000001V<l00@006<l000000MS?08000QS?080
00MO?08000MO70<000MO708000Moo`03001oogoo00Eoo`003Woo00<007ooOol00P0016=L0`000V<l
00<006<lHc`01V<l0`0026<l00D006<lHcaS?00000ES?08000AS?003001S?6<l009S?003001S?6<l
00ES?004001S?0000006Hc`00`00Hc`00008Hc`20008Hc`30006Hc`200000f<l001S?005Hc`20000
0ell001O?005Gc`00`00Gc`00006Ga`40006Ga`00`00Oomoo`07Ool00`00Oomoo`05Ool000ioo`<0
00=SG0<0009SG003001S?00000QS?005001S?6<lHc`00005Hc`20005Hc`01@00HcaS?6<l00001f<l
00@006<l000000QS?08000QS?0<000IS?080009S?003001S?6<l00AS?005001S?6<lHc`00004Hc`0
0`00HcaS?002Hc`00`00HcaS?002Hc`00`00GcaO?002Gc`01000GcaO?5ll0P000ell00<005lLGa`0
0UlL00@005lLGaaO708000=O708000Yoo`03001oogoo00Eoo`003Goo0P000V=L0`001F=L0`001V<l
0P001F<l00D006<lHcaS?00000QS?003001S?00000QS?0<000QS?004001S?6<l0006Hc`01@00HcaS
?6<l000016<l00<006<lHc`00f<l00@006<lHcaS?08000ES?004001S?0000006Hc`01000Hc`00000
1Ull00<005ll00001Ull00@005lL000000AO7005001O7000Ga`0000<Ool00`00Oomoo`05Ool00003
Ool00000008000=oo`8000=oo`@000ISG08000=S?08000=S?003001S?6<l00IS?003001S?00000US
?0<000MS?005001S?6<lHc`00005Hc`20004Hc`01000HcaS?6<l0P001F<l00@006<l000000MS?003
001S?00000MS?08000QS?08000QO?08000MO?08000MO70<000eoo`03001oogoo00Eoo`000Woo00<0
07ooOol00Woo00@007ooOol0009oo`@000ESG003001S?6<l00ES?003001S?00000US?0<000QS?005
001S?6<lHc`00005Hc`00`00HcaS?003Hc`01@00HcaS?6<l00001f<l00<006<l00001f<l0P002F<l
0P001V<l0P0000=S?000Hc`01F<l0P0000=S?000Gc`01Ell0P000Ull00<005llGc`00Ull0P000UlL
00<005lLGa`00UlL10003Woo00<007ooOol01Goo0003Ool01@00Oomoogoo00000Woo00<007ooOol0
0Woo00D007oo000006=L008000QS?0<000QS?005001S?6<lHc`00005Hc`20005Hc`01@00HcaS?6<l
00001f<l00<006<l000026<l0P001V<l0P000V<l00<006<lHc`016<l0P000V<l00<006<lHc`00V<l
00<006<lHc`00V<l00<006<lHc`00V<l00<006<lHc`00V<l00<005llGc`00Ull00<005llGc`00ell
00@005ll000000EO7004001O70000004Ool00`00Oomoo`0<Ool00`00Oomoo`04Ool000Aoo`04001o
ogoo0002Ool00`00Oomoo`03Ool00`00Oomoo`020008Hc`01@00HcaS?6<l00001F<l0P001F<l00D0
06<lHcaS?00000QS?003001S?00000QS?0<000IS?080009S?003001S?6<l009S?003001S?6<l00=S
?003001S?6<l009S?003001S?6<l00=S?004001S?0000006Hc`01000Hc`000001V<l00@005ll0000
00IO?08000MO708000Ioo`03001oogoo00aoo`03001oogoo00Aoo`0000Eoo`00Oomoo`00009oo`04
001oogoo0006Ool01@00Oomoogoo00001F<l0P001F<l00D006<lHcaS?00000QS?003001S?00000US
?0<000IS?08000=S?003001S?6<l009S?003001S?6<l00=S?004001S?0000007Hc`01000Hc`00000
1V<l0P0026<l0P0026<l0P001Ull0P0000=O?000Ga`015lL0P002Goo00<007ooOol02goo00<007oo
Ool017oo0002Ool20004Ool20008Ool20002Ool01@00HcaS?6<l000026<l00<006<l00002F<l0`00
1f<l0P000f<l00<006<lHc`00V<l00<006<lHc`016<l00@006<l000000IS?08000US?0<000IS?004
001S?6<l0006Hc`01000HcaS?00016<l0`000Ull00<005llGc`00Ull00<005llGc`00Ull00D005lL
GaaO700000aoo`03001oogoo00Yoo`03001oogoo00Aoo`0057oo00H007ooOol006<l0009Hc`30007
Hc`20003Hc`20004Hc`00`00HcaS?004Hc`01000Hc`000001f<l0P001f<l00@006<lHc`000IS?080
00=S?004001S?6<lHc`20004Hc`01000HcaS?6<l0P0016<l00@006<l000000IO?004001O?0000005
Gc`01000Ga`000003Woo00<007ooOol02Goo00<007ooOol017oo000EOol00`00Oomoo`020007Hc`2
0003Hc`20004Hc`00`00HcaS?005Hc`01000Hc`000001f<l0`001V<l0P000V<l00<006<lHc`00f<l
00<006<lHc`00V<l00<006<lHc`00V<l00<006<lHc`016<l00<006<l00001f<l00<006<l00001V<l
0P0025ll0P001ell0P0047oo00<007ooOol02Goo00<007ooOol017oo000FOol01000Oomoogoo0P00
16<l00<006<lHc`01F<l00@006<l000000QS?0<000IS?08000=S?003001S?6<l009S?003001S?6<l
00=S?005001S?6<lHc`00006Hc`01000Hc`000001f<l0P0026<l0P001V<l00@005llGc`000IO?003
001O?00000IO?003001oogoo015oo`03001oogoo00Qoo`03001oogoo00Aoo`005goo0P000goo00@0
06<l000000QS?0<000MS?08000=S?003001S?6<l009S?003001S?6<l00AS?004001S?0000007Hc`0
0`00Hc`00007Hc`30006Hc`200000f<l001S?005Hc`20002Hc`01000HcaS?6<l0P0015ll00@005ll
GcaO?08000=O?004001O?5llGc`2000EOol00`00Oomoo`08Ool00`00Oomoo`03Ool001Uoo`04001o
ogooOol20008Hc`01000HcaS?6<l0P0016<l00<006<lHc`016<l00@006<l000000MS?08000US?0<0
00IS?005001S?6<lHc`00004Hc`00`00HcaS?002Hc`00`00HcaS?002Hc`00`00HcaS?003Hc`00`00
Hc`00007Gc`00`00Gc`00006Gc`00`00Gc`0000HOol00`00Oomoo`07Ool00`00Oomoo`03Ool001Yo
o`03001oogoo009oo`03001S?6<l00=S?08000IS?004001S?0000007Hc`20008Hc`01000HcaS?000
1V<l0P000f<l00@006<lHcaS?08000ES?004001S?0000006Hc`01000Hc`000001V<l0P0025ll0P00
1ell0P006Goo00<007ooOol01goo00<007ooOol00goo000KOol00`00Oomoo`02Ool01@00HcaS?6<l
000026<l0P0026<l00@006<lHc`000ES?08000AS?003001S?6<l009S?003001S?6<l00AS?003001S
?00000MS?08000QS?08000IS?080009O?003001O?5ll009O?0<0009O?003001O?5ll009O?08001ao
o`03001oogoo00Ioo`03001oogoo00=oo`0077oo0P000goo00<006<l000026<l00@006<lHc`000ES
?08000AS?005001S?6<lHc`00007Hc`01000Hc`000001f<l0P001V<l0P000V<l00<006<lHc`00V<l
0`000V<l00<006<lHc`00V<l00<005llGc`00ell00@005ll000000IO?004001O?000000OOol00`00
Oomoo`05Ool00`00Oomoo`03Ool001ioo`04001oogooOol20006Hc`20004Hc`01@00HcaS?6<l0000
1f<l00<006<l000026<l0`001V<l0P000V<l00<006<lHc`00V<l00<006<lHc`00f<l00@006<l0000
00IS?004001S?0000006Gc`20008Gc`2000ROol00`00Oomoo`04Ool00`00Oomoo`03Ool001moo`03
001oogoo009oo`8000=S?003001S?6<l00ES?003001S?00000QS?0<000MS?005001S?6<lHc`00004
Hc`00`00HcaS?003Hc`01000Hc`000001V<l0P0026<l0`001Ull00<005ll00001Ull0P0097oo00<0
07ooOol017oo00<007ooOol00goo000POol20004Ool00`00Hc`00008Hc`30007Hc`01000HcaS?6<l
0P000f<l0P001F<l00@006<l000000IS?0<000IS?004001S?6<l0006Hc`01@00GcaO?5ll00000ell
0P000ell00<005llGc`00Ull00<007ooOol09Goo00<007ooOol017oo00<007ooOol00Woo000ROol0
0`00Oomoo`02Ool20007Hc`01000HcaS?6<l0P000f<l0P001V<l00<006<l00001f<l0`001V<l00D0
06<lHcaS?00000=S?08000AS?004001S?6<lHc`20005Gc`00`00Gc`00006Gc`01000Gc`00000:Goo
00<007ooOol00goo00<007ooOol00Woo000SOol00`00Oomoo`03Ool20003Hc`20006Hc`00`00Hc`0
0008Hc`30006Hc`01@00HcaS?6<l00000f<l0P001F<l00<006<l00001f<l00<006<l00001ell0P00
1ell0P00;7oo00<007ooOol00Woo00<007ooOol00Woo000TOol00`00Oomoo`04Ool00`00Hc`00008
Hc`30006Hc`20003Hc`01000HcaS?6<l0P001F<l00<006<l00001f<l0`001f<l0P001Ull0P000Ull
00<005llGc`00Ull0P00;Woo00<007ooOol00Woo00<007ooOol00Woo000UOol20005Ool20006Hc`2
0003Hc`00`00HcaS?002Hc`00`00HcaS?004Hc`00`00Hc`00007Hc`30006Hc`01@00HcaS?6<l0000
16<l0P000V<l00<005llGc`00Ull00<005llGc`00ell00@005ll0000035oo`05001oogooOol00004
Ool002Moo`03001oogoo00Aoo`03001S?6<l009S?003001S?6<l00AS?004001S?0000007Hc`20006
Hc`20003Hc`01000HcaS?6<l0P001F<l00@006<l000000ES?004001O?0000006Gc`2000dOol01000
Oomoo`0017oo000XOol00`00Oomoo`04Ool01000Hc`000001f<l0`001V<l0P000V<l00<006<lHc`0
0V<l00<006<lHc`016<l00<006<l00001f<l0P001f<l0`001Ull00<007ooOol0=Goo00<007oo0000
17oo000YOol00`00Oomoo`04Ool20006Hc`20003Hc`00`00HcaS?002Hc`00`00HcaS?003Hc`01000
Hc`000001f<l0P001V<l0P000V<l00<006<lHc`00V<l0P000ell00@005llGcaO?08003Qoo`04001o
ogoo0003Ool002Yoo`8000Ioo`03001S?6<l009S?003001S?6<l00AS?004001S?0000006Hc`30006
Hc`20002Hc`00`00HcaS?002Hc`00`00HcaS?003Hc`01000Hc`000001Ull00<005ll0000>goo00<0
07oo00000goo000/Ool00`00Oomoo`04Ool01000Hc`000001f<l0`001V<l00D006<lHcaS?00000AS
?003001S?6<l00=S?004001S?0000006Hc`20008Gc`2000mOol20003Ool002eoo`03001oogoo00Ao
o`8000MS?005001S?6<lHc`00003Hc`20005Hc`01000Hc`000001V<l0`001V<l00@005llGc`000AO
?0<003ioo`<000=oo`00;Woo00<007ooOol01Goo0P000f<l0P001F<l00<006<l00001f<l0`001V<l
00D006<lHcaS?00000=S?08000AO?004001O?000000mOol40006Ool002moo`8000Moo`03001S?000
00MS?0<000ES?08000=S?004001S?6<lHc`20005Hc`00`00Hc`00006Gc`2000lOol3000:Ool0035o
o`03001oogoo00Eoo`8000ES?08000=S?0800003Hc`0000000IS?003001S?00000MS?08000IO?003
001oogoo03Qoo`@000eoo`00<Woo00<007ooOol01Woo0P0000=S?00000001V<l0`001f<l0`0016<l
0`000Ull00@005llGcaO?08003Moo`@0015oo`00<goo00<007ooOol01goo0P001V<l00D006<lHcaS
?00000AS?08000=S?004001S?0000006Gc`00`00Gc`0000eOol4000EOol003Aoo`8000Uoo`04001S
?6<lHc`20005Hc`01000Hc`000001F<l0`001ell0P00<goo0`006Goo000fOol00`00Oomoo`07Ool0
0`00Hc`00007Hc`20006Hc`01@00GcaO?5ll000015ll0P00<Goo100077oo000gOol00`00Oomoo`07
Ool20006Hc`01000HcaS?0000f<l0P001Ell00@005ll000002moo`@0021oo`00>7oo00<007ooOol0
27oo00@006<lHcaS?08000AS?003001S?00000MO?08002eoo`@002Aoo`00>Goo0P002Goo00<006<l
00001V<l0`001Ell0P00;7oo0`00:7oo000kOol00`00Oomoo`07Ool20004Hc`20003Hc`200000ell
0000000ZOol4000[Ool003aoo`03001oogoo00Qoo`04001S?0000006Hc`2000XOol4000_Ool003eo
o`8000Uoo`8000IS?003001oogoo02Aoo`@003=oo`00?goo00<007ooOol027oo00@006<lHcaS?080
02=oo`@003Moo`00@7oo00<007ooOol027oo00<006<l00008Woo0`00>goo0011Ool00`00Oomoo`08
Ool00`00Oomoo`0MOol4000nOol0049oo`8000Qoo`03001oogoo01Uoo`@0049oo`00A7oo00<007oo
Ool01Goo00<007ooOol05Goo1000AWoo0015Ool00`00Oomoo`04Ool00`00Oomoo`0BOol3001:Ool0
04Ioo`03001oogoo00=oo`03001oogoo00ioo`@004eoo`00Agoo0P000goo00<007ooOol02Woo1000
DGoo0019Ool01000Oomoo`0027oo1000EGoo001:Ool00`00Ool00005Ool3001IOol004]oo`800003
Ool00000008005aoo`00C7oo0P00H7oo002^Ool00001\
\>"],
  ImageRangeCache->{{{0, 173.375}, {140.313, 0}} -> {-0.0839847, -0.00882247, 
  0.0060902, 0.0060902}}]
}, Open  ]],

Cell["\<\
In the following we show Vega, computed symbolically by the \
computer. It is zero everywhere (to machine precision of the plotted function \
).\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[UpAndOutPutVega[0, S, 50, 50, 0.25,0.1, 0.10, t],
{S, 30, 50}, {t, 0.001, 3}, PlotRange -> {-0.01, 0.01}];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.66533e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.80843 .15978 0 -6.10127 ]
[.80843 .15978 6 2.89873 ]
[.90116 .30178 0 -5.96725 ]
[.90116 .30178 6 3.03275 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.02494 .25992 -30 -2.73794 ]
[.02494 .25992 0 6.26206 ]
[.01611 .31527 -36 -2.78472 ]
[.01611 .31527 0 6.21528 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00254 .43215 -30 -2.88407 ]
[-0.00254 .43215 0 6.11593 ]
[-0.0124 .49393 -24 -2.93688 ]
[-0.0124 .49393 0 6.06312 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.79606 .16417 m
.78988 .16637 L
s
[(1)] .80843 .15978 -1 .35584 Mshowa
.88868 .30585 m
.88245 .30788 L
s
[(2)] .90116 .30178 -1 .32606 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(3)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.71146 .03476 m
.70779 .03618 L
s
.7336 .06862 m
.72991 .07001 L
s
.75506 .10145 m
.75136 .10281 L
s
.77587 .13328 m
.77217 .13462 L
s
.81567 .19417 m
.81195 .19546 L
s
.83471 .22329 m
.83099 .22457 L
s
.85321 .25159 m
.84948 .25285 L
s
.8712 .2791 m
.86746 .28034 L
s
.90569 .33187 m
.90195 .33307 L
s
.92225 .35719 m
.91849 .35838 L
s
.93836 .38184 m
.9346 .38301 L
s
.95406 .40585 m
.95029 .407 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03716 .25514 m
.04327 .25274 L
s
[(-0.01)] .02494 .25992 1 -0.39157 Mshowa
.02837 .31059 m
.0345 .30825 L
s
[(-0.005)] .01611 .31527 1 -0.38117 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.00981 .42772 m
.01599 .4255 L
s
[(0.005)] -0.00254 .43215 1 -0.3591 Mshowa
0 .48963 m
.0062 .48748 L
s
[(0.01)] -0.0124 .49393 1 -0.34736 Mshowa
.125 Mabswid
.03543 .26607 m
.0391 .26464 L
s
.03368 .27708 m
.03735 .27566 L
s
.03193 .28817 m
.0356 .28676 L
s
.03016 .29934 m
.03383 .29793 L
s
.02658 .32192 m
.03026 .32053 L
s
.02477 .33334 m
.02845 .33195 L
s
.02295 .34483 m
.02663 .34345 L
s
.02111 .35641 m
.0248 .35504 L
s
.0174 .37983 m
.02109 .37847 L
s
.01552 .39167 m
.01922 .39032 L
s
.01363 .4036 m
.01733 .40225 L
s
.01173 .41561 m
.01543 .41428 L
s
.00788 .43991 m
.01159 .43859 L
s
.00593 .4522 m
.00964 .45089 L
s
.00397 .46458 m
.00768 .46328 L
s
.00199 .47706 m
.00571 .47576 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.656 .751 .92 r
.37901 .68206 .40169 .70185 .43949 .69119 .41715 .67112 Metetra
.656 .751 .92 r
.41715 .67112 .43949 .69119 .47783 .68039 .45584 .66002 Metetra
.656 .751 .92 r
.45584 .66002 .47783 .68039 .51672 .66942 .49508 .64877 Metetra
.656 .751 .92 r
.49508 .64877 .51672 .66942 .55616 .6583 .5349 .63735 Metetra
.656 .751 .92 r
.5349 .63735 .55616 .6583 .59617 .64702 .57529 .62576 Metetra
.656 .751 .92 r
.57529 .62576 .59617 .64702 .63677 .63557 .61629 .614 Metetra
.656 .751 .92 r
.61629 .614 .63677 .63557 .67796 .62396 .6579 .60207 Metetra
.656 .751 .92 r
.6579 .60207 .67796 .62396 .71976 .61218 .70013 .58995 Metetra
.656 .751 .92 r
.70013 .58995 .71976 .61218 .76219 .60021 .743 .57766 Metetra
.656 .751 .92 r
.743 .57766 .76219 .60021 .80525 .58807 .78652 .56517 Metetra
.656 .751 .92 r
.78652 .56517 .80525 .58807 .84896 .57575 .8307 .5525 Metetra
.656 .751 .92 r
.8307 .5525 .84896 .57575 .89334 .56324 .87558 .53963 Metetra
.656 .751 .92 r
.87558 .53963 .89334 .56324 .9384 .55053 .92115 .52656 Metetra
.656 .751 .92 r
.92115 .52656 .9384 .55053 .98416 .53763 .96743 .51328 Metetra
.656 .751 .92 r
.35573 .66174 .37901 .68206 .41715 .67112 .39421 .65051 Metetra
.656 .751 .92 r
.39421 .65051 .41715 .67112 .45584 .66002 .43325 .63911 Metetra
.656 .751 .92 r
.43325 .63911 .45584 .66002 .49508 .64877 .47286 .62755 Metetra
.656 .751 .92 r
.47286 .62755 .49508 .64877 .5349 .63735 .51305 .61582 Metetra
.656 .751 .92 r
.51305 .61582 .5349 .63735 .57529 .62576 .55384 .60391 Metetra
.656 .751 .92 r
.55384 .60391 .57529 .62576 .61629 .614 .59525 .59183 Metetra
.656 .751 .92 r
.59525 .59183 .61629 .614 .6579 .60207 .63727 .57956 Metetra
.656 .751 .92 r
.63727 .57956 .6579 .60207 .70013 .58995 .67994 .56711 Metetra
.656 .751 .92 r
.67994 .56711 .70013 .58995 .743 .57766 .72326 .55446 Metetra
.656 .751 .92 r
.72326 .55446 .743 .57766 .78652 .56517 .76725 .54162 Metetra
.656 .751 .92 r
.76725 .54162 .78652 .56517 .8307 .5525 .81192 .52858 Metetra
.656 .751 .92 r
.81192 .52858 .8307 .5525 .87558 .53963 .85729 .51534 Metetra
.656 .751 .92 r
.85729 .51534 .87558 .53963 .92115 .52656 .90338 .50188 Metetra
.656 .751 .92 r
.90338 .50188 .92115 .52656 .96743 .51328 .95021 .48822 Metetra
.656 .751 .92 r
.33183 .64088 .35573 .66174 .39421 .65051 .37065 .62934 Metetra
.656 .751 .92 r
.37065 .62934 .39421 .65051 .43325 .63911 .41005 .61764 Metetra
.656 .751 .92 r
.41005 .61764 .43325 .63911 .47286 .62755 .45003 .60576 Metetra
.656 .751 .92 r
.45003 .60576 .47286 .62755 .51305 .61582 .49061 .5937 Metetra
.656 .751 .92 r
.49061 .5937 .51305 .61582 .55384 .60391 .53179 .58146 Metetra
.656 .751 .92 r
.53179 .58146 .55384 .60391 .59525 .59183 .57361 .56904 Metetra
.656 .751 .92 r
.57361 .56904 .59525 .59183 .63727 .57956 .61607 .55642 Metetra
.656 .751 .92 r
.61607 .55642 .63727 .57956 .67994 .56711 .65918 .54361 Metetra
.656 .751 .92 r
.65918 .54361 .67994 .56711 .72326 .55446 .70296 .5306 Metetra
.656 .751 .92 r
.70296 .5306 .72326 .55446 .76725 .54162 .74742 .51739 Metetra
.656 .751 .92 r
.74742 .51739 .76725 .54162 .81192 .52858 .79259 .50397 Metetra
.656 .751 .92 r
.79259 .50397 .81192 .52858 .85729 .51534 .83848 .49033 Metetra
.656 .751 .92 r
.83848 .49033 .85729 .51534 .90338 .50188 .8851 .47648 Metetra
.656 .751 .92 r
.8851 .47648 .90338 .50188 .95021 .48822 .93247 .4624 Metetra
.656 .751 .92 r
.30728 .61945 .33183 .64088 .37065 .62934 .34645 .6076 Metetra
.656 .751 .92 r
.34645 .6076 .37065 .62934 .41005 .61764 .38621 .59557 Metetra
.656 .751 .92 r
.38621 .59557 .41005 .61764 .45003 .60576 .42656 .58336 Metetra
.656 .751 .92 r
.42656 .58336 .45003 .60576 .49061 .5937 .46753 .57096 Metetra
.656 .751 .92 r
.46753 .57096 .49061 .5937 .53179 .58146 .50912 .55837 Metetra
.656 .751 .92 r
.50912 .55837 .53179 .58146 .57361 .56904 .55136 .54559 Metetra
.656 .751 .92 r
.55136 .54559 .57361 .56904 .61607 .55642 .59425 .53262 Metetra
.656 .751 .92 r
.59425 .53262 .61607 .55642 .65918 .54361 .63781 .51944 Metetra
.656 .751 .92 r
.63781 .51944 .65918 .54361 .70296 .5306 .68206 .50605 Metetra
.656 .751 .92 r
.68206 .50605 .70296 .5306 .74742 .51739 .72701 .49245 Metetra
.656 .751 .92 r
.72701 .49245 .74742 .51739 .79259 .50397 .77269 .47863 Metetra
.656 .751 .92 r
.77269 .47863 .79259 .50397 .83848 .49033 .81909 .46458 Metetra
.656 .751 .92 r
.81909 .46458 .83848 .49033 .8851 .47648 .86626 .45031 Metetra
.656 .751 .92 r
.86626 .45031 .8851 .47648 .93247 .4624 .9142 .43581 Metetra
.656 .751 .92 r
.28205 .59743 .30728 .61945 .34645 .6076 .32157 .58525 Metetra
.656 .751 .92 r
.32157 .58525 .34645 .6076 .38621 .59557 .3617 .57288 Metetra
.656 .751 .92 r
.3617 .57288 .38621 .59557 .42656 .58336 .40243 .56032 Metetra
.656 .751 .92 r
.40243 .56032 .42656 .58336 .46753 .57096 .4438 .54757 Metetra
.656 .751 .92 r
.4438 .54757 .46753 .57096 .50912 .55837 .48581 .53463 Metetra
.656 .751 .92 r
.48581 .53463 .50912 .55837 .55136 .54559 .52847 .52148 Metetra
.656 .751 .92 r
.52847 .52148 .55136 .54559 .59425 .53262 .5718 .50812 Metetra
.656 .751 .92 r
.5718 .50812 .59425 .53262 .63781 .51944 .61582 .49455 Metetra
.656 .751 .92 r
.61582 .49455 .63781 .51944 .68206 .50605 .66055 .48077 Metetra
.656 .751 .92 r
.66055 .48077 .68206 .50605 .72701 .49245 .706 .46676 Metetra
.656 .751 .92 r
.706 .46676 .72701 .49245 .77269 .47863 .75218 .45252 Metetra
.656 .751 .92 r
.75218 .45252 .77269 .47863 .81909 .46458 .79913 .43805 Metetra
.656 .751 .92 r
.79913 .43805 .81909 .46458 .86626 .45031 .84685 .42335 Metetra
.656 .751 .92 r
.84685 .42335 .86626 .45031 .9142 .43581 .89536 .40839 Metetra
.656 .751 .92 r
.25611 .57479 .28205 .59743 .32157 .58525 .296 .56227 Metetra
.656 .751 .92 r
.296 .56227 .32157 .58525 .3617 .57288 .33649 .54955 Metetra
.656 .751 .92 r
.33649 .54955 .3617 .57288 .40243 .56032 .37762 .53663 Metetra
.656 .751 .92 r
.37762 .53663 .40243 .56032 .4438 .54757 .41938 .52351 Metetra
.656 .751 .92 r
.41938 .52351 .4438 .54757 .48581 .53463 .46181 .51019 Metetra
.656 .751 .92 r
.46181 .51019 .48581 .53463 .52847 .52148 .50491 .49665 Metetra
.656 .751 .92 r
.50491 .49665 .52847 .52148 .5718 .50812 .54869 .4829 Metetra
.656 .751 .92 r
.54869 .4829 .5718 .50812 .61582 .49455 .59318 .46893 Metetra
.656 .751 .92 r
.59318 .46893 .61582 .49455 .66055 .48077 .63839 .45473 Metetra
.656 .751 .92 r
.63839 .45473 .66055 .48077 .706 .46676 .68435 .4403 Metetra
.656 .751 .92 r
.68435 .4403 .706 .46676 .75218 .45252 .73106 .42562 Metetra
.656 .751 .92 r
.73106 .42562 .75218 .45252 .79913 .43805 .77855 .41071 Metetra
.656 .751 .92 r
.77855 .41071 .79913 .43805 .84685 .42335 .82683 .39554 Metetra
.656 .751 .92 r
.82683 .39554 .84685 .42335 .89536 .40839 .87594 .38012 Metetra
.656 .751 .92 r
.22944 .55152 .25611 .57479 .296 .56227 .26969 .53863 Metetra
.656 .751 .92 r
.26969 .53863 .296 .56227 .33649 .54955 .31056 .52554 Metetra
.656 .751 .92 r
.31056 .52554 .33649 .54955 .37762 .53663 .35208 .51225 Metetra
.656 .751 .92 r
.35208 .51225 .37762 .53663 .41938 .52351 .39426 .49875 Metetra
.656 .751 .92 r
.39426 .49875 .41938 .52351 .46181 .51019 .43711 .48503 Metetra
.656 .751 .92 r
.43711 .48503 .46181 .51019 .50491 .49665 .48064 .47109 Metetra
.656 .751 .92 r
.48064 .47109 .50491 .49665 .54869 .4829 .52489 .45693 Metetra
.656 .751 .92 r
.52489 .45693 .54869 .4829 .59318 .46893 .56985 .44253 Metetra
.656 .751 .92 r
.56985 .44253 .59318 .46893 .63839 .45473 .61556 .4279 Metetra
.656 .751 .92 r
.61556 .4279 .63839 .45473 .68435 .4403 .66203 .41302 Metetra
.656 .751 .92 r
.66203 .41302 .68435 .4403 .73106 .42562 .70928 .39789 Metetra
.656 .751 .92 r
.70928 .39789 .73106 .42562 .77855 .41071 .75733 .38251 Metetra
.656 .751 .92 r
.75733 .38251 .77855 .41071 .82683 .39554 .80619 .36687 Metetra
.656 .751 .92 r
.80619 .36687 .82683 .39554 .87594 .38012 .8559 .35095 Metetra
.656 .751 .92 r
.202 .52757 .22944 .55152 .26969 .53863 .24262 .51431 Metetra
.656 .751 .92 r
.24262 .51431 .26969 .53863 .31056 .52554 .28388 .50084 Metetra
.656 .751 .92 r
.28388 .50084 .31056 .52554 .35208 .51225 .32579 .48716 Metetra
.656 .751 .92 r
.32579 .48716 .35208 .51225 .39426 .49875 .36838 .47325 Metetra
.656 .751 .92 r
.36838 .47325 .39426 .49875 .43711 .48503 .41166 .45912 Metetra
.656 .751 .92 r
.41166 .45912 .43711 .48503 .48064 .47109 .45565 .44476 Metetra
.656 .751 .92 r
.45565 .44476 .48064 .47109 .52489 .45693 .50036 .43016 Metetra
.656 .751 .92 r
.50036 .43016 .52489 .45693 .56985 .44253 .54581 .41533 Metetra
.656 .751 .92 r
.54581 .41533 .56985 .44253 .61556 .4279 .59203 .40024 Metetra
.656 .751 .92 r
.59203 .40024 .61556 .4279 .66203 .41302 .63902 .3849 Metetra
.656 .751 .92 r
.63902 .3849 .66203 .41302 .70928 .39789 .68682 .36929 Metetra
.656 .751 .92 r
.68682 .36929 .70928 .39789 .75733 .38251 .73543 .35342 Metetra
.656 .751 .92 r
.73543 .35342 .75733 .38251 .80619 .36687 .78489 .33727 Metetra
.656 .751 .92 r
.78489 .33727 .80619 .36687 .8559 .35095 .83521 .32084 Metetra
.656 .751 .92 r
.17377 .50293 .202 .52757 .24262 .51431 .21476 .48928 Metetra
.656 .751 .92 r
.21476 .48928 .24262 .51431 .28388 .50084 .2564 .47541 Metetra
.656 .751 .92 r
.2564 .47541 .28388 .50084 .32579 .48716 .29872 .46131 Metetra
.656 .751 .92 r
.29872 .46131 .32579 .48716 .36838 .47325 .34173 .44699 Metetra
.656 .751 .92 r
.34173 .44699 .36838 .47325 .41166 .45912 .38545 .43243 Metetra
.656 .751 .92 r
.38545 .43243 .41166 .45912 .45565 .44476 .42989 .41762 Metetra
.656 .751 .92 r
.42989 .41762 .45565 .44476 .50036 .43016 .47508 .40258 Metetra
.656 .751 .92 r
.47508 .40258 .50036 .43016 .54581 .41533 .52102 .38727 Metetra
.656 .751 .92 r
.52102 .38727 .54581 .41533 .59203 .40024 .56775 .37171 Metetra
.656 .751 .92 r
.56775 .37171 .59203 .40024 .63902 .3849 .61528 .35588 Metetra
.656 .751 .92 r
.61528 .35588 .63902 .3849 .68682 .36929 .66363 .33978 Metetra
.656 .751 .92 r
.66363 .33978 .68682 .36929 .73543 .35342 .71283 .32339 Metetra
.656 .751 .92 r
.71283 .32339 .73543 .35342 .78489 .33727 .76289 .30672 Metetra
.656 .751 .92 r
.76289 .30672 .78489 .33727 .83521 .32084 .81384 .28975 Metetra
.656 .751 .92 r
.1447 .47755 .17377 .50293 .21476 .48928 .18606 .46349 Metetra
.656 .751 .92 r
.18606 .46349 .21476 .48928 .2564 .47541 .2281 .44921 Metetra
.656 .751 .92 r
.2281 .44921 .2564 .47541 .29872 .46131 .27083 .43468 Metetra
.656 .751 .92 r
.27083 .43468 .29872 .46131 .34173 .44699 .31426 .41992 Metetra
.656 .751 .92 r
.31426 .41992 .34173 .44699 .38545 .43243 .35843 .40491 Metetra
.656 .751 .92 r
.35843 .40491 .38545 .43243 .42989 .41762 .40333 .38964 Metetra
.656 .751 .92 r
.40333 .38964 .42989 .41762 .47508 .40258 .449 .37412 Metetra
.656 .751 .92 r
.449 .37412 .47508 .40258 .52102 .38727 .49545 .35833 Metetra
.656 .751 .92 r
.49545 .35833 .52102 .38727 .56775 .37171 .5427 .34227 Metetra
.656 .751 .92 r
.5427 .34227 .56775 .37171 .61528 .35588 .59078 .32593 Metetra
.656 .751 .92 r
.59078 .32593 .61528 .35588 .66363 .33978 .6397 .3093 Metetra
.656 .751 .92 r
.6397 .3093 .66363 .33978 .71283 .32339 .68949 .29238 Metetra
.656 .751 .92 r
.68949 .29238 .71283 .32339 .76289 .30672 .74017 .27515 Metetra
.656 .751 .92 r
.74017 .27515 .76289 .30672 .81384 .28975 .79176 .25762 Metetra
.656 .751 .92 r
.11475 .45142 .1447 .47755 .18606 .46349 .15649 .43693 Metetra
.656 .751 .92 r
.15649 .43693 .18606 .46349 .2281 .44921 .19893 .4222 Metetra
.656 .751 .92 r
.19893 .4222 .2281 .44921 .27083 .43468 .24207 .40723 Metetra
.656 .751 .92 r
.24207 .40723 .27083 .43468 .31426 .41992 .28594 .39201 Metetra
.656 .751 .92 r
.28594 .39201 .31426 .41992 .35843 .40491 .33056 .37653 Metetra
.656 .751 .92 r
.33056 .37653 .35843 .40491 .40333 .38964 .37593 .36078 Metetra
.656 .751 .92 r
.37593 .36078 .40333 .38964 .449 .37412 .42209 .34476 Metetra
.656 .751 .92 r
.42209 .34476 .449 .37412 .49545 .35833 .46905 .32846 Metetra
.656 .751 .92 r
.46905 .32846 .49545 .35833 .5427 .34227 .51684 .31188 Metetra
.656 .751 .92 r
.51684 .31188 .5427 .34227 .59078 .32593 .56547 .295 Metetra
.656 .751 .92 r
.56547 .295 .59078 .32593 .6397 .3093 .61498 .27782 Metetra
.656 .751 .92 r
.61498 .27782 .6397 .3093 .68949 .29238 .66537 .26033 Metetra
.656 .751 .92 r
.66537 .26033 .68949 .29238 .74017 .27515 .71668 .24253 Metetra
.656 .751 .92 r
.71668 .24253 .74017 .27515 .79176 .25762 .76894 .22439 Metetra
.656 .751 .92 r
.08389 .42449 .11475 .45142 .15649 .43693 .12602 .40955 Metetra
.656 .751 .92 r
.12602 .40955 .15649 .43693 .19893 .4222 .16886 .39437 Metetra
.656 .751 .92 r
.16886 .39437 .19893 .4222 .24207 .40723 .21242 .37893 Metetra
.656 .751 .92 r
.21242 .37893 .24207 .40723 .28594 .39201 .25673 .36322 Metetra
.656 .751 .92 r
.25673 .36322 .28594 .39201 .33056 .37653 .3018 .34724 Metetra
.656 .751 .92 r
.3018 .34724 .33056 .37653 .37593 .36078 .34766 .33099 Metetra
.656 .751 .92 r
.34766 .33099 .37593 .36078 .42209 .34476 .39431 .31445 Metetra
.656 .751 .92 r
.39431 .31445 .42209 .34476 .46905 .32846 .4418 .29762 Metetra
.656 .751 .92 r
.4418 .29762 .46905 .32846 .51684 .31188 .49013 .28049 Metetra
.656 .751 .92 r
.49013 .28049 .51684 .31188 .56547 .295 .53933 .26304 Metetra
.656 .751 .92 r
.53933 .26304 .56547 .295 .61498 .27782 .58942 .24529 Metetra
.656 .751 .92 r
.58942 .24529 .61498 .27782 .66537 .26033 .64044 .2272 Metetra
.656 .751 .92 r
.64044 .2272 .66537 .26033 .71668 .24253 .69239 .20879 Metetra
.656 .751 .92 r
.69239 .20879 .71668 .24253 .76894 .22439 .74532 .19002 Metetra
.656 .751 .92 r
.05208 .39672 .08389 .42449 .12602 .40955 .0946 .38132 Metetra
.656 .751 .92 r
.0946 .38132 .12602 .40955 .16886 .39437 .13784 .36565 Metetra
.656 .751 .92 r
.13784 .36565 .16886 .39437 .21242 .37893 .18183 .34972 Metetra
.656 .751 .92 r
.18183 .34972 .21242 .37893 .25673 .36322 .22658 .33351 Metetra
.656 .751 .92 r
.22658 .33351 .25673 .36322 .3018 .34724 .27212 .31701 Metetra
.656 .751 .92 r
.27212 .31701 .3018 .34724 .34766 .33099 .31846 .30023 Metetra
.656 .751 .92 r
.31846 .30023 .34766 .33099 .39431 .31445 .36562 .28314 Metetra
.656 .751 .92 r
.36562 .28314 .39431 .31445 .4418 .29762 .41363 .26575 Metetra
.656 .751 .92 r
.41363 .26575 .4418 .29762 .49013 .28049 .46252 .24804 Metetra
.656 .751 .92 r
.46252 .24804 .49013 .28049 .53933 .26304 .5123 .23001 Metetra
.656 .751 .92 r
.5123 .23001 .53933 .26304 .58942 .24529 .563 .21164 Metetra
.656 .751 .92 r
.563 .21164 .58942 .24529 .64044 .2272 .61464 .19293 Metetra
.656 .751 .92 r
.61464 .19293 .64044 .2272 .69239 .20879 .66726 .17387 Metetra
.656 .751 .92 r
.66726 .17387 .69239 .20879 .74532 .19002 .72088 .15445 Metetra
.656 .751 .92 r
.01926 .36808 .05208 .39672 .0946 .38132 .06217 .35219 Metetra
.656 .751 .92 r
.06217 .35219 .0946 .38132 .13784 .36565 .10583 .33602 Metetra
.656 .751 .92 r
.10583 .33602 .13784 .36565 .18183 .34972 .15025 .31957 Metetra
.656 .751 .92 r
.15025 .31957 .18183 .34972 .22658 .33351 .19545 .30283 Metetra
.656 .751 .92 r
.19545 .30283 .22658 .33351 .27212 .31701 .24146 .28579 Metetra
.656 .751 .92 r
.24146 .28579 .27212 .31701 .31846 .30023 .28829 .26844 Metetra
.656 .751 .92 r
.28829 .26844 .31846 .30023 .36562 .28314 .33597 .25079 Metetra
.656 .751 .92 r
.33597 .25079 .36562 .28314 .41363 .26575 .38452 .2328 Metetra
.656 .751 .92 r
.38452 .2328 .41363 .26575 .46252 .24804 .43397 .21449 Metetra
.656 .751 .92 r
.43397 .21449 .46252 .24804 .5123 .23001 .48434 .19583 Metetra
.656 .751 .92 r
.48434 .19583 .5123 .23001 .563 .21164 .53566 .17683 Metetra
.656 .751 .92 r
.53566 .17683 .563 .21164 .61464 .19293 .58795 .15746 Metetra
.656 .751 .92 r
.58795 .15746 .61464 .19293 .66726 .17387 .64124 .13773 Metetra
.656 .751 .92 r
.64124 .13773 .66726 .17387 .72088 .15445 .69556 .11761 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  GroupPageBreakWithin->True,
  AspectRatioFixed->True,
  ImageSize->{169.375, 137.25},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002Y0000RA000`40O003h00OZGoo002YOol00:Uo
o`00ZGoo002YOol00:Uoo`00ZGoo002YOol00:Uoo`00ZGoo002YOol00:Uoo`00LWoo0`0017oo0P00
;Woo001eOol01000Oomoo`000Woo00<007ooOol0:goo001eOol01000Oomoo`000Woo00<007ooOol0
:goo001bOol30003Ool01000Oomoo`00;Goo001bOol00`00Oomoo`03Ool01000Oomoo`00;Goo001b
Ool40003Ool2000^Ool00:Uoo`00ZGoo002YOol005]oo`<0009oo`<001Qoo`8002aoo`00G7oo00<0
07ooOol017oo00<007ooOol04Goo100000=oo`00Ool0:goo001JOol40005Ool00`00Oomoo`0?Ool3
0004Ool2000[Ool005Yoo`03001oo`0000=oo`<000moo`<000Moo`03001oo`0002Yoo`00FWoo00<0
07oo00000goo00<007ooOol03Goo0`002Goo00<007oo0000:Woo001KOol20003Ool40009Ool3000<
Ool01000Oomoo`00:Goo001[Ool3000>Ool01@00Oomoogoo0000:7oo001XOol3000AOol01@00Oomo
ogoo0000:7oo001VOol2000DOol00`00Oomoo`02Ool00`00Oomoo`0UOol004Aoo`<000=oo`8001Mo
o`<001Ioo`03001oogoo009oo`03001oogoo02Eoo`00AGoo00D007ooOomoo`00009oo`03001oogoo
015oo`<001Uoo`03001oogoo00=oo`03001oogoo02Aoo`00@goo10000Woo00@007ooOol0015oo`<0
01]oo`03001oogoo00Aoo`03001oogoo02=oo`00@goo00<007oo00000goo00@007ooOol000ioo`<0
01ioo`03001oogoo00Aoo`03001oogoo02=oo`00@goo00<007oo00000goo00@007ooOol000aoo`80
0003Ool007oo01ioo`03001oogoo00Eoo`03001oogoo029oo`00A7oo0P0017oo0P002Woo0`008goo
00<007ooOol01Woo00<007ooOol08Goo001DOol2000UOol20008Ool00`00Oomoo`0QOol0055oo`<0
02Aoo`<00003Dad007oo00Qoo`03001oogoo021oo`00;Woo0`000goo0`005goo0`009Goo0P001E<M
00<007ooOol01Woo0P0017oo0`006Woo000aOol00`00Oomoo`03Ool00`00Oomoo`0BOol200000goo
001oo`0ROol30007Dad20008Ool00`00Oomoo`03Ool00`00Oomoo`0IOol0035oo`03001oogoo00=o
o`03001oogoo00moo`<002Aoo`<000=C7@03001C7E<M00AC7@03001C7@0000Qoo`03001oogoo009o
o`03001oogoo01Uoo`00;goo0P000goo0`0047oo0P0097oo0`001e<M00<005<MDad00e<M100027oo
00D007ooOomoo`0001]oo`00<Goo00@007ooOol000moo`<002=oo`<00003Dad005<M00QC7@03001C
7E<M00<0009C7@03001oogoo00Ioo`80009oo`03001oogoo01Uoo`00;Woo0`000goo10002Goo0`00
97oo0P001E<M00<005<MDad01U<M0`000U<M00D005<MDaeC7@0000Qoo`04001oo`00000KOol003mo
o`800003Ool007oo029oo`8000QC7@03001C7E<M009C7@<00003Dad005<M009C7@03001C7E<M009C
7@03001oogoo00Eoo`03001oogoo01aoo`00?7oo0`0097oo0`000U<M00<005<MDad01U<M00@005<M
000000EC7@04001C7E<M0004Dad20007Ool00`00Oomoo`0KOol001Yoo`<000Aoo`8001Moo`8002Ao
o`<000IC7@03001C7E<M00AC7@<000QC7@04001C7@00Dad300000e<M001oo`07Ool00`00Oomoo`0J
Ool001eoo`04001oogoo0002Ool00`00Oomoo`0AOol4000ROol300000e<M001C7@07Dad00`00DaeC
7@030002Dad00`00DaeC7@06Dad01000Dad000001E<M00<007ooOol01Goo00<007ooOol06Woo000M
Ool01000Oomoo`000Woo00<007ooOol03Woo0`0097oo0P001E<M00<005<MDad01E<M0`001U<M00<0
05<MDad015<M10001e<M00<007ooOol01Goo00<007ooOol06Goo000KOol20003Ool01000Oomoo`00
3Woo0P009Goo0P0025<M00@005<MDaeC7@<00003Dad005<M00MC7@03001C7E<M00<0009C7@8000MC
7@03001oogoo00Ioo`03001oogoo01Qoo`007Goo00@007ooOol0009oo`03001oogoo00Uoo`@002=o
o`<0009C7@03001C7E<M00EC7@04001C7@000005Dad00`00DaeC7@05Dad30006Dad00`00DaeC7@04
Dad30007Ool2000IOol001Yoo`<000Aoo`8000Yoo`8002=oo`@000IC7@03001C7E<M00AC7@8000QC
7@03001C7E<M009C7@800003Dad005<M00IC7@80009C7@<0009C7@03001oogoo00Ioo`03001oogoo
01Moo`00:Woo0`008Woo0`000e<M00<005<MDad01E<M00<005<MDad00`000U<M00<005<MDad01U<M
00@005<M000000AC7@03001C7E<M00AC7@@000IC7@03001oogoo00Ioo`03001oogoo01Ioo`009goo
10008Goo0`001e<M00<005<MDad015<M0`001U<M00<005<MDad015<M0`001e<M00@005<MDaeC7@<0
0003Dad005<M00MC7@03001oogoo00Eoo`8001Moo`009Woo0P008Woo0P0000=C7@00Dad025<M00<0
05<MDad00`000U<M00<005<MDad01E<M00<005<MDad00`000U<M00<005<MDad01E<M00<005<M0000
0P0000EC7@00DaeC7@0000MC7@8000Moo`03001oogoo01Eoo`003goo0P0017oo0P0017oo0P000goo
0`000goo0`007goo0P0015<M00<005<MDad01U<M0`001U<M00<005<MDad015<M0`001U<M00<005<M
Dad00e<M0`001E<M00@005<MDad000=C7@<00003Dad007oo00Moo`03001oogoo01Aoo`003Woo00@0
07ooOol000=oo`8000=oo`04001oogoo0003Ool00`00Oomoo`02Ool01000Oomoo`0077oo0P001e<M
00<005<MDad00U<M0`0000=C7@00Dad01e<M00<005<MDad00`0000=C7@00Dad01e<M00<005<M0000
0`000U<M00<005<MDad00e<M00@005<M001C7@8000EC7@03001oogoo00Eoo`03001oogoo00=oo`@0
00eoo`0027oo1@0000Eoo`00Oomoo`0000Qoo`04001oogoo0003Ool00`00Oomoo`02Ool01@00Oomo
ogoo000067oo0`000U<M00<005<MDad01E<M00@005<M000000EC7@03001C7E<M00AC7@@000EC7@03
001C7E<M00AC7@@000IC7@03001C7E<M00=C7@<000QC7@03001oogoo00Eoo`03001oogoo00=oo`03
001oogoo00eoo`003Woo00@007ooOol000Qoo`04001oogoo0003Ool01000Oomoogoo0P0017oo00<0
07ooOol04Woo0`001U<M00<005<MDad015<M0P0025<M00<005<MDad00`000U<M00<005<MDad01E<M
00<005<MDad00`000U<M00<005<MDad01U<M00<005<MDad00`0000=C7@00Dad01e<M0P001Woo00<0
07ooOol017oo00<007ooOol037oo000>Ool01000Oomoo`0027oo00@007ooOol000=oo`05001oogoo
Ool00006Ool00`00Oomoo`0>Ool30002Dad00`00DaeC7@05Dad00`00DaeC7@030002Dad00`00DaeC
7@05Dad30006Dad00`00DaeC7@04Dad30006Dad00`00DaeC7@04Dad40003Dad00`00Dad00004Dad3
00000e<M001oo`06Ool00`00Oomoo`04Ool00`00Oomoo`0;Ool000moo`8000Yoo`8000=oo`8000=o
o`8000Ioo`03001oogoo00Yoo`<000IC7@03001C7E<M00AC7@<000IC7@03001C7E<M009C7@800003
Dad005<M00MC7@03001C7E<M00<0009C7@03001C7E<M00EC7@03001C7E<M00<0009C7@03001C7E<M
00=C7@04001C7@00Dad20005Dad00`00Oomoo`05Ool01000Oomoo`000Woo00<007ooOol02goo000U
Ool20007Ool00`00Oomoo`06Ool30002Dad00`00DaeC7@05Dad00`00DaeC7@030002Dad00`00DaeC
7@05Dad01000Dad0000015<M00<005<MDad01E<M0`001U<M00<005<MDad015<M0`001U<M00<005<M
Dad00e<M0`0025<M00<007ooOol017oo0P000Woo0P003Woo0008Ool20004Ool20004Ool20004Ool2
0003Ool30003Ool00`00Oomoo`07Ool00`00Oomoo`02Ool30006Dad00`00DaeC7@04Dad30006Dad0
0`00DaeC7@04Dad20007Dad00`00DaeC7@02Dad40008Dad00`00DaeC7@0300000e<M001C7@07Dad0
0`00DaeC7@0300000e<M001C7@07Dad00`00Oomoo`05Ool00`00Oomoo`0@Ool000Moo`04001oogoo
0003Ool20003Ool01000Oomoo`000Woo00@007ooOol000Eoo`04001oogoo000:Ool40002Dad00`00
DaeC7@05Dad00`00DaeC7@030002Dad00`00DaeC7@05Dad00`00DaeC7@030002Dad00`00DaeC7@05
Dad40004Dad00`00DaeC7@04Dad40005Dad00`00DaeC7@04Dad40003Dad00`00Dad00006Dad30007
Ool00`00Oomoo`0?Ool00003Ool0000000<00005Ool007ooOol00008Ool01000Oomoo`000Woo00@0
07ooOol000Eoo`04001oo`000008Ool30006Dad00`00DaeC7@04Dad30006Dad00`00DaeC7@04Dad3
0006Dad00`00DaeC7@02Dad40007Dad00`00DaeC7@030002Dad00`00DaeC7@05Dad00`00DaeC7@03
0002Dad00`00DaeC7@03Dad00`00Dad00002Dad30002Dad00`00Oomoo`05Ool2000@Ool000Moo`04
001oogoo0008Ool01000Oomoo`000Woo00@007ooOol0009oo`<0009oo`8000Ioo`80009C7@03001C
7E<M00EC7@03001C7E<M00<0009C7@03001C7E<M00EC7@03001C7E<M00<0009C7@03001C7E<M00EC
7@H0009C7@03001C7E<M00AC7@<000IC7@03001C7E<M00AC7@<000IC7@03001C7E<M009C7@03001C
7@00008000IC7@03001oogoo00Eoo`03001oogoo00ioo`001goo00@007ooOol000Qoo`04001oogoo
0002Ool01000Oomoo`000Woo00<007ooOol00Woo00<007ooOol017oo00<005<MDad00e<M00<005<M
Dad015<M0`001U<M00<005<MDad015<M0`001U<M00<005<MDad00U<M1@001U<M00<005<MDad00`00
0U<M00<005<MDad01E<M00<005<MDad00`0000=C7@00Dad01e<M00<005<MDad00`0000=C7@00Dad0
1U<M0P001goo00<007ooOol03Goo0008Ool2000:Ool20004Ool20003Ool400000goo00000003Ool3
0006Dad00`00DaeC7@030002Dad00`00DaeC7@05Dad00`00DaeC7@030002Dad00`00DaeC7@05Dad6
0002Dad00`00DaeC7@04Dad30006Dad00`00DaeC7@04Dad30005Dad00`00DaeC7@04Dad40003Dad0
0`00Dad00004Dad200000e<M001oo`06Ool2000>Ool002Aoo`D000=C7@03001C7E<M00=C7@<000IC
7@03001C7E<M00AC7@<000IC7@03001C7E<M009C7@D000IC7@03001C7E<M00<0009C7@03001C7E<M
00EC7@03001C7E<M00<00003Dad005<M00IC7@03001C7E<M00<0009C7@03001C7E<M00=C7@04001C
7@00Dad20004Dad00`00Oomoo`05Ool00`00Oomoo`0<Ool001ioo`8000Aoo`8000MC7@03001C7@00
0080009C7@03001C7E<M00EC7@03001C7E<M00<0009C7@03001C7E<M00EC7@03001C7@0000<00003
Dad005<M00IC7@<000IC7@03001C7E<M00AC7@<000EC7@03001C7E<M00AC7@<000IC7@03001C7E<M
009C7@@000MC7@03001oogoo00Eoo`03001oogoo00]oo`007Goo00@007ooOol0009oo`800003Ool0
05<M00EC7@8000IC7@03001C7E<M00AC7@<000IC7@03001C7E<M00=C7@@000EC7@03001C7E<M009C
7@80009C7@03001C7E<M00EC7@03001C7E<M00<00003Dad005<M00IC7@03001C7E<M00<00003Dad0
05<M00IC7@03001C7E<M00@0009C7@03001C7E<M00AC7@8000Ioo`8000aoo`007Goo00@007ooOol0
009oo`05001oogooOol00002Dad30002Dad00`00DaeC7@04Dad00`00DaeC7@030002Dad00`00DaeC
7@05Dad00`00Dad0000300000e<M001C7@06Dad01000Dad000001E<M00<005<MDad015<M0`001E<M
00<005<MDad015<M0`001E<M00<005<MDad015<M0`0015<M00@005<MDad0009C7@<00003Dad007oo
00Ioo`03001oogoo00Yoo`007Goo00@007ooOol0009oo`8000=oo`8000IC7@03001C7E<M00=C7@<0
00IC7@03001C7E<M00=C7@@000EC7@03001C7E<M00AC7@<000MC7@03001C7E<M00<00003Dad005<M
00IC7@03001C7E<M00<00003Dad005<M00IC7@03001C7E<M00<0009C7@03001C7E<M00=C7@D000EC
7@03001oogoo00Aoo`03001oogoo00Yoo`007Goo00@007ooOol0009oo`8000Aoo`03001C7E<M00EC
7@03001C7@000080009C7@03001C7E<M00EC7@03001C7@0000<000=C7@03001C7E<M00=C7@03001C
7E<M00<000=C7@03001C7E<M00AC7@<000EC7@03001C7E<M00AC7@<000EC7@03001C7E<M00AC7@<0
00IC7@03001C7E<M00<0009C7@03001C7E<M00EC7@03001oogoo00Aoo`03001oogoo00=oo`<000=o
o`007Woo0P000goo00<007ooOol017oo00<005<MDad015<M0P001U<M00<005<MDad00e<M10001e<M
00@005<MDaeC7@@000MC7@03001C7E<M00<00003Dad005<M00IC7@03001C7E<M00<00003Dad005<M
00IC7@03001C7E<M00<0009C7@03001C7E<M00AC7@<000=C7@04001C7E<M0005Dad30006Ool00`00
Oomoo`05Ool10001Ool1Ool002=oo`03001oogoo00Eoo`03001C7E<M00<0009C7@03001C7E<M00AC
7@03001C7@0000<000=C7@03001C7E<M00AC7@@000AC7@03001C7E<M00AC7@<000EC7@03001C7E<M
00AC7@<000EC7@03001C7E<M00AC7@<000IC7@03001C7E<M00<00003Dad005<M00AC7@03001C7@00
009C7@<0009C7@03001oogoo00Aoo`03001oogoo00Eoo`40005oo`5oo`008goo00<007ooOol01Woo
0P001U<M00<005<MDad00U<M10001e<M00<005<MDad00`0000=C7@00Dad01U<M00<005<MDad00`00
00=C7@00Dad01U<M00<005<MDad00`000U<M00<005<MDad015<M00<005<MDad00`000U<M00<005<M
Dad015<M0`001E<M00<005<MDad00e<M10001U<M00<007ooOol017oo00<007ooOol00Woo0P000goo
0005Ool20004Ool20004Ool20004Ool20003Ool30003Ool00`00Oomoo`08Ool00`00DaeC7@05Dad0
0`00Dad000020003Dad00`00DaeC7@04Dad30005Dad00`00DaeC7@04Dad30005Dad00`00DaeC7@04
Dad30006Dad01000DaeC7E<M1@001U<M00<005<MDad00`0000=C7@00Dad01U<M00<005<MDad00`00
0U<M00<005<MDad015<M00<007ooOol00goo0`001Goo0@000Goo0Goo0004Ool01000Oomoo`000goo
0P000goo00@007ooOol0009oo`04001oogoo0005Ool00`00Oomoo`02000:Ool00`00DaeC7@03Dad4
0006Dad00`00DaeC7@0300000e<M001C7@06Dad00`00DaeC7@0300000e<M001C7@06Dad00`00DaeC
7@030002Dad00`00DaeC7@04Dad40004Dad00`00DaeC7@04Dad30005Dad00`00DaeC7@04Dad30003
Dad01000DaeC7@000e<M100000Aoo`000000009oo`03001oogoo00<000=oo`0017oo00@007ooOol0
00Qoo`04001oogoo0002Ool01000Oomoo`001Goo00@007ooOol000aoo`03001C7@0000<000=C7@03
001C7E<M00=C7@<000EC7@03001C7E<M00AC7@<000EC7@03001C7E<M00AC7@<000IC7@03001C7E<M
00<00003Dad005<M00IC7@03001C7E<M00<00003Dad005<M00IC7@03001C7E<M00<0009C7@03001C
7E<M009C7@03001C7@0000<000=C7@8000Eoo`03001oogoo00Ioo`0017oo00@007ooOol000Qoo`04
001oogoo0002Ool01000Oomoo`000Woo0`000goo0P002Woo10001e<M00<005<M00000P000U<M00<0
05<MDad015<M00<005<MDad00`000e<M00<005<MDad00e<M00<005<MDad00`000U<M00<005<MDad0
15<M0`001E<M00<005<MDad015<M0`001E<M00<005<MDad015<M0`001U<M00<005<MDad00`0000=C
7@00Dad01E<M00<007ooOol00goo00<007ooOol01Woo0004Ool01000Oomoo`0027oo00@007ooOol0
009oo`04001oogoo0002Ool00`00Oomoo`03Ool00`00Oomoo`06Ool30004Ool00`00DaeC7@04Dad2
0006Dad00`00DaeC7@03Dad30007Dad01000DaeC7E<M10001U<M00<005<MDad00`0000=C7@00Dad0
1U<M00<005<MDad00`000U<M00<005<MDad015<M00<005<MDad00`000U<M00<005<MDad015<M0`00
0U<M00@005<MDad000EC7@8000Aoo`03001oogoo00Ioo`001Goo0P002Woo0P0017oo0P000goo1000
0Woo00<007ooOol00goo0`0027oo00<005<MDad00`0000=C7@00Dad01U<M00<005<M00000P000e<M
00<005<MDad015<M100015<M00<005<MDad00e<M0`001E<M00<005<MDad015<M0`001U<M00@005<M
DaeC7@D000IC7@03001C7E<M00<00003Dad005<M00=C7@03001C7@00009C7@<0009C7@03001oogoo
009oo`03001oogoo00Eoo`008Goo0P000Woo0`0037oo0P001E<M00<005<MDad015<M0`001U<M00<0
05<MDad00`0000=C7@00Dad01U<M00<005<M00000P0000=C7@00Dad01U<M00<005<MDad00`000U<M
00<005<MDad015<M100015<M00<005<MDad015<M0`001E<M00<005<MDad00U<M10001U<M00D007oo
Oomoo`0000Moo`008Goo10004Goo00<005<MDad00e<M00<005<MDad00`000e<M00<005<MDad00e<M
0`001E<M00<005<MDad015<M0P001E<M00<005<MDad015<M0`001U<M00<005<MDad00`0000=C7@00
Dad01U<M00<005<MDad00`0000=C7@00Dad01E<M00<005<MDad00`000U<M00<005<MDad015<M0P00
0Woo00<007ooOol01Goo000:Ool20004Ool20004Ool20003Ool30003Ool2000DOol01000DaeC7E<M
10001e<M00<005<M00000P0000=C7@00Dad01U<M00<005<MDad00`000U<M00<005<MDad00e<M00<0
05<MDad00`000U<M00<005<MDad015<M0`001E<M00<005<MDad015<M0`001E<M00@005<MDaeC7@D0
00=C7@04001C7E<M0002Dad300001E<M001oogoo00001goo0009Ool01000Oomoo`000goo0P000goo
00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo019oo`<000AC7@03001C7E<M00AC7@8000EC
7@03001C7E<M00AC7@<000IC7@04001C7E<MDad40006Dad00`00DaeC7@030002Dad00`00DaeC7@04
Dad00`00DaeC7@0300000e<M001C7@05Dad40004Dad00`00DaeC7@02Dad00`00Dad000020005Dad0
0`00Ool00007Ool000Uoo`04001oogoo0008Ool01000Oomoo`000goo00<007ooOol01Goo00<007oo
Ool04Woo00<005<MDad015<M00<005<MDad00`000U<M00<005<MDad00e<M00<005<MDad00`000U<M
00<005<MDad015<M100015<M00<005<MDad00e<M0`001U<M00@005<MDaeC7@D000EC7@03001C7E<M
00<00003Dad005<M00IC7@03001C7E<M00<00003Dad005<M00EC7@03001oo`0000Moo`002Goo00@0
07ooOol000Qoo`04001oogoo0003Ool00`00Oomoo`06Ool00`00Oomoo`0BOol00`00DaeC7@03Dad3
0006Dad01000DaeC7E<M10001U<M00<005<MDad00`000U<M00<005<MDad015<M00<005<M00000P00
0U<M00<005<MDad015<M10000e<M00<005<MDad015<M0`001E<M00<005<MDad015<M0`000U<M00@0
05<MDad000=C7@D000Moo`002Goo00@007ooOol000Qoo`04001oogoo0003Ool00`00Oomoo`07Ool2
000COol00`00Dad000020003Dad00`00DaeC7@03Dad40004Dad00`00DaeC7@03Dad30006Dad01000
DaeC7E<M10001U<M00<005<MDad00`0000=C7@00Dad01E<M00<005<MDad00`000U<M00<005<MDad0
15<M00<005<MDad00`0000=C7@00Dad00e<M00<005<M00000`000e<M00<007oo00001Woo000:Ool2
000:Ool20003Ool2000;Ool00`00Oomoo`0AOol00`00DaeC7@05Dad00`00Dad000020002Dad00`00
DaeC7@04Dad00`00Dad000020002Dad00`00DaeC7@04Dad40004Dad00`00DaeC7@03Dad30005Dad0
0`00DaeC7@03Dad30006Dad01000DaeC7E<M1@001E<M00<005<MDad00`0000=C7@00Dad01E<M0P00
1Woo000YOol00`00Oomoo`0AOol00`00DaeC7@04Dad20006Dad01000DaeC7E<M10001U<M00<005<M
Dad00`0000=C7@00Dad01U<M00<005<M00000P000U<M00<005<MDad015<M00<005<M00000P000U<M
00<005<MDad015<M10000e<M00<005<MDad015<M0`000e<M00<005<M00001E<M0P001Woo000ZOol0
0`00Oomoo`0AOol00`00DaeC7@030002Dad00`00DaeC7@03Dad40004Dad00`00DaeC7@03Dad30005
Dad00`00DaeC7@04Dad20006Dad01000DaeC7E<M10001U<M00<005<MDad00`0000=C7@00Dad01E<M
00<005<MDad00`0000=C7@00Dad00e<M00<005<M00000U<M0`0000=oo`00Ool01Goo000[Ool2000B
Ool20006Dad00`00Dad000020003Dad00`00DaeC7@03Dad00`00Dad000020003Dad00`00DaeC7@03
Dad00`00DaeC7@0300000e<M001C7@05Dad40003Dad00`00DaeC7@04Dad30005Dad01000DaeC7E<M
1@001E<M00@005<MDaeC7@D000Aoo`03001oogoo00Aoo`00;Goo00<007ooOol04Goo00<005<MDad0
0e<M0P001e<M00@005<MDaeC7@<000MC7@04001C7E<MDad40005Dad00`00DaeC7@030002Dad00`00
DaeC7@03Dad00`00DaeC7@030002Dad00`00DaeC7@03Dad40003Dad00`00DaeC7@04Dad40002Ool0
0`00Oomoo`04Ool00`00Oomoo`04Ool002ioo`03001oogoo015oo`03001C7@000080009C7@03001C
7E<M00AC7@@000=C7@03001C7E<M00AC7@@000=C7@03001C7E<M00=C7@<000IC7@04001C7E<MDad4
0006Dad00`00Dad000020002Dad00`00DaeC7@03Dad00`00DaeC7@030007Ool00`00Oomoo`03Ool0
0`00Oomoo`04Ool002moo`03001oogoo015oo`03001C7E<M00AC7@03001C7E<M00<0009C7@03001C
7E<M00=C7@03001C7E<M00<0009C7@03001C7E<M00=C7@03001C7@000080009C7@03001C7E<M00AC
7@@000=C7@03001C7E<M00AC7@8000IC7@04001C7E<MDad4000:Ool00`00Oomoo`04Ool00`00Oomo
o`03Ool0031oo`03001oogoo015oo`03001C7E<M00=C7@<000IC7@04001C7E<MDad40006Dad01000
DaeC7E<M0`001U<M00<005<MDad00`0000=C7@00Dad01E<M00<005<MDad00`0000=C7@00Dad01E<M
10003goo00<007ooOol00goo00<007ooOol00goo000aOol2000BOol00`00Dad000020003Dad00`00
DaeC7@03Dad40003Dad00`00DaeC7@04Dad40003Dad00`00DaeC7@03Dad30005Dad01000DaeC7E<M
1@001E<M00<005<MDad00`0057oo00<007ooOol00Woo00<007ooOol00goo000cOol00`00Oomoo`0@
Ool20006Dad00`00Dad000020002Dad00`00DaeC7@03Dad00`00DaeC7@030002Dad00`00DaeC7@03
Dad00`00Dad000020002Dad00`00DaeC7@03Dad40004Dad00`00DaeC7@03Dad3000GOol00`00Oomo
o`02Ool00`00Oomoo`03Ool003Aoo`03001oogoo015oo`03001C7E<M00=C7@8000IC7@04001C7E<M
Dad40006Dad01000DaeC7E<M10001E<M00<005<M00000P000U<M00<005<MDad015<M00<005<M0000
0P006goo00D007ooOomoo`0000Eoo`00=Goo00<007ooOol04Goo00<005<M00000P000U<M00<005<M
Dad00e<M10000e<M00<005<MDad015<M100015<M00@005<MDaeC7@<000IC7@04001C7E<MDad4000O
Ool01000Oomoo`001Goo000fOol2000BOol20005Dad00`00Dad000020002Dad00`00DaeC7@03Dad0
0`00DaeC7@0300000e<M001C7@05Dad40003Dad00`00DaeC7@03Dad4000TOol00`00Ool00005Ool0
03Qoo`03001oogoo015oo`04001C7E<MDad30006Dad01000DaeC7E<M10001E<M00<005<MDad00`00
00=C7@00Dad01E<M00<005<M00000P00:7oo00@007ooOol000Aoo`00>Goo00<007ooOol04Goo0`00
0e<M00<005<MDad00e<M100015<M00@005<MDaeC7@@000EC7@04001C7E<MDad4000/Ool00`00Ool0
0004Ool003Yoo`03001oogoo015oo`03001C7E<M00=C7@03001C7@000080009C7@03001C7E<M00=C
7@@000=C7@03001C7E<M00=C7@@0035oo`8000Aoo`00>goo0P004Woo00@005<MDaeC7@@000EC7@03
001C7@000080009C7@03001C7E<M00=C7@03001C7@00008003Aoo`<000Aoo`00?Goo00<007ooOol0
47oo0`0015<M00@005<MDaeC7@<000IC7@04001C7E<MDad3000cOol40007Ool003ioo`03001oogoo
011oo`03001C7E<M00=C7@@000=C7@03001C7E<M00=C7@@003=oo`<000]oo`00?goo00<007ooOol0
47oo00<005<M00000P000U<M00<005<MDad00e<M00<005<M00000P00<goo10003Woo0010Ool2000A
Ool20005Dad01000DaeC7E<M0`00<Woo10004Woo0012Ool00`00Oomoo`0@Ool01000DaeC7E<M1@00
<Goo10005Woo0013Ool00`00Oomoo`0@Ool3000cOol3000JOol004Aoo`03001oogoo00moo`03001o
ogoo02moo`@001eoo`00AGoo0P003Woo00<007ooOol0;7oo10008Goo0017Ool00`00Oomoo`0;Ool0
0`00Oomoo`0XOol4000UOol004Qoo`03001oogoo00Yoo`03001oogoo02Aoo`@002Uoo`00BGoo00<0
07ooOol02Goo00<007ooOol08Goo0`00;Goo001:Ool00`00Oomoo`08Ool00`00Oomoo`0MOol4000`
Ool004]oo`8000Qoo`03001oogoo01Uoo`@003Aoo`00CGoo00<007ooOol01Goo00<007ooOol05Goo
1000>7oo001>Ool00`00Oomoo`04Ool00`00Oomoo`0BOol3000lOol004moo`03001oogoo00=oo`03
001oogoo00ioo`@003moo`00D7oo0P000goo00<007ooOol02Woo1000@goo001BOol01000Oomoo`00
27oo1000Agoo001COol00`00Ool00005Ool3001;Ool005Aoo`800003Ool00000008004ioo`00EGoo
0P00DWoo002YOol00:Uoo`00ZGoo002YOol00:Uoo`00ZGoo002YOol00:Uoo`00ZGoo0000\
\>"],
  ImageRangeCache->{{{0, 168.375}, {136.25, 0}} -> {-0.246741, -0.0828671, 
  0.00728008, 0.00728008}}]
}, Open  ]],

Cell[TextData[{
  "Because of the flatness of the value surface, Delta is close to ",
  Cell[BoxData[
      \(TraditionalForm\`\(-1\)\)]],
  " everywhere, as revealed in the next plot - it is constant in price of the \
underlying but not constant in time."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[UpAndOutPutDelta[0, S, 50, 50, 0.25,0.1, 0.1, t],
{S, 30, 50}, {t, 0.001, 3}, PlotRange -> All];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.80843 .15978 0 -6.10127 ]
[.80843 .15978 6 2.89873 ]
[.90116 .30178 0 -5.96725 ]
[.90116 .30178 6 3.03275 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.02413 .26503 -12 -2.74225 ]
[.02413 .26503 0 6.25775 ]
[.01099 .34738 -24 -2.81194 ]
[.01099 .34738 0 6.18806 ]
[-0.00287 .43425 -24 -2.88586 ]
[-0.00287 .43425 0 6.11414 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.79606 .16417 m
.78988 .16637 L
s
[(1)] .80843 .15978 -1 .35584 Mshowa
.88868 .30585 m
.88245 .30788 L
s
[(2)] .90116 .30178 -1 .32606 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(3)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.71146 .03476 m
.70779 .03618 L
s
.7336 .06862 m
.72991 .07001 L
s
.75506 .10145 m
.75136 .10281 L
s
.77587 .13328 m
.77217 .13462 L
s
.81567 .19417 m
.81195 .19546 L
s
.83471 .22329 m
.83099 .22457 L
s
.85321 .25159 m
.84948 .25285 L
s
.8712 .2791 m
.86746 .28034 L
s
.90569 .33187 m
.90195 .33307 L
s
.92225 .35719 m
.91849 .35838 L
s
.93836 .38184 m
.9346 .38301 L
s
.95406 .40585 m
.95029 .407 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03635 .26025 m
.04246 .25787 L
s
[(-1)] .02413 .26503 1 -0.39061 Mshowa
.02327 .34277 m
.02942 .34046 L
s
[(-0.9)] .01099 .34738 1 -0.37512 Mshowa
.00948 .42982 m
.01565 .4276 L
s
[(-0.8)] -0.00287 .43425 1 -0.3587 Mshowa
.125 Mabswid
.03379 .27641 m
.03746 .27499 L
s
.0312 .29274 m
.03488 .29132 L
s
.02859 .30924 m
.03227 .30783 L
s
.02594 .32591 m
.02963 .32452 L
s
.02057 .3598 m
.02426 .35843 L
s
.01784 .37702 m
.02154 .37566 L
s
.01509 .39443 m
.01878 .39308 L
s
.0123 .41203 m
.016 .41069 L
s
.00663 .44781 m
.01034 .44649 L
s
.00375 .466 m
.00746 .46469 L
s
.00083 .48439 m
.00455 .48309 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.649 .659 .846 r
.37521 .77348 .39806 .80578 .43696 .79556 .41432 .76293 Metetra
.649 .659 .846 r
.41432 .76293 .43696 .79556 .47642 .78519 .45401 .75221 Metetra
.649 .659 .846 r
.45401 .75221 .47642 .78519 .51647 .77466 .49428 .74134 Metetra
.649 .659 .846 r
.49428 .74134 .51647 .77466 .55711 .76398 .53516 .7303 Metetra
.649 .659 .846 r
.53516 .7303 .55711 .76398 .59835 .75314 .57666 .7191 Metetra
.649 .659 .846 r
.57666 .7191 .59835 .75314 .64022 .74214 .61878 .70773 Metetra
.649 .659 .846 r
.61878 .70773 .64022 .74214 .68272 .73097 .66155 .69619 Metetra
.649 .659 .846 r
.66155 .69619 .68272 .73097 .72586 .71964 .70498 .68446 Metetra
.649 .659 .846 r
.70498 .68446 .72586 .71964 .76967 .70812 .74908 .67256 Metetra
.649 .659 .846 r
.74908 .67256 .76967 .70812 .81416 .69643 .79388 .66047 Metetra
.649 .659 .846 r
.79388 .66047 .81416 .69643 .85934 .68456 .83937 .64818 Metetra
.649 .659 .846 r
.83937 .64818 .85934 .68456 .90523 .6725 .8856 .63571 Metetra
.649 .659 .846 r
.8856 .63571 .90523 .6725 .95186 .66025 .93256 .62303 Metetra
.649 .659 .846 r
.93256 .62303 .95186 .66025 .99922 .6478 .98028 .61015 Metetra
.649 .657 .845 r
.35192 .74027 .37521 .77348 .41432 .76293 .39124 .72936 Metetra
.649 .657 .845 r
.39124 .72936 .41432 .76293 .45401 .75221 .43115 .71829 Metetra
.649 .657 .845 r
.43115 .71829 .45401 .75221 .49428 .74134 .47166 .70706 Metetra
.649 .657 .845 r
.47166 .70706 .49428 .74134 .53516 .7303 .51278 .69565 Metetra
.649 .657 .845 r
.51278 .69565 .53516 .7303 .57666 .7191 .55452 .68408 Metetra
.649 .657 .845 r
.55452 .68408 .57666 .7191 .61878 .70773 .5969 .67232 Metetra
.649 .657 .845 r
.5969 .67232 .61878 .70773 .66155 .69619 .63994 .66038 Metetra
.649 .657 .845 r
.63994 .66038 .66155 .69619 .70498 .68446 .68365 .64826 Metetra
.649 .657 .845 r
.68365 .64826 .70498 .68446 .74908 .67256 .72805 .63595 Metetra
.649 .657 .845 r
.72805 .63595 .74908 .67256 .79388 .66047 .77314 .62344 Metetra
.649 .657 .845 r
.77314 .62344 .79388 .66047 .83937 .64818 .81896 .61073 Metetra
.649 .657 .845 r
.81896 .61073 .83937 .64818 .8856 .63571 .86551 .59782 Metetra
.649 .657 .845 r
.86551 .59782 .8856 .63571 .93256 .62303 .91281 .5847 Metetra
.649 .657 .845 r
.91281 .5847 .93256 .62303 .98028 .61015 .96089 .57136 Metetra
.649 .655 .843 r
.32819 .70611 .35192 .74027 .39124 .72936 .36773 .69484 Metetra
.649 .655 .843 r
.36773 .69484 .39124 .72936 .43115 .71829 .40785 .6834 Metetra
.649 .655 .843 r
.40785 .6834 .43115 .71829 .47166 .70706 .44858 .67179 Metetra
.649 .655 .843 r
.44858 .67179 .47166 .70706 .51278 .69565 .48994 .66 Metetra
.649 .655 .843 r
.48994 .66 .51278 .69565 .55452 .68408 .53193 .64803 Metetra
.649 .655 .843 r
.53193 .64803 .55452 .68408 .5969 .67232 .57457 .63588 Metetra
.649 .655 .843 r
.57457 .63588 .5969 .67232 .63994 .66038 .61788 .62354 Metetra
.649 .655 .843 r
.61788 .62354 .63994 .66038 .68365 .64826 .66186 .611 Metetra
.649 .655 .843 r
.66186 .611 .68365 .64826 .72805 .63595 .70655 .59826 Metetra
.649 .655 .843 r
.70655 .59826 .72805 .63595 .77314 .62344 .75195 .58532 Metetra
.649 .655 .843 r
.75195 .58532 .77314 .62344 .81896 .61073 .79808 .57217 Metetra
.649 .655 .843 r
.79808 .57217 .81896 .61073 .86551 .59782 .84496 .55881 Metetra
.649 .655 .843 r
.84496 .55881 .86551 .59782 .91281 .5847 .8926 .54523 Metetra
.649 .655 .843 r
.8926 .54523 .91281 .5847 .96089 .57136 .94104 .53142 Metetra
.648 .654 .841 r
.30402 .67097 .32819 .70611 .36773 .69484 .34376 .65933 Metetra
.648 .654 .841 r
.34376 .65933 .36773 .69484 .40785 .6834 .3841 .64751 Metetra
.648 .654 .841 r
.3841 .64751 .40785 .6834 .44858 .67179 .42506 .63551 Metetra
.648 .654 .841 r
.42506 .63551 .44858 .67179 .48994 .66 .46664 .62332 Metetra
.648 .654 .841 r
.46664 .62332 .48994 .66 .53193 .64803 .50888 .61095 Metetra
.648 .654 .841 r
.50888 .61095 .53193 .64803 .57457 .63588 .55177 .59838 Metetra
.648 .654 .841 r
.55177 .59838 .57457 .63588 .61788 .62354 .59535 .58561 Metetra
.648 .654 .841 r
.59535 .58561 .61788 .62354 .66186 .611 .63961 .57264 Metetra
.648 .654 .841 r
.63961 .57264 .66186 .611 .70655 .59826 .68458 .55947 Metetra
.648 .654 .841 r
.68458 .55947 .70655 .59826 .75195 .58532 .73028 .54608 Metetra
.648 .654 .841 r
.73028 .54608 .75195 .58532 .79808 .57217 .77673 .53247 Metetra
.648 .654 .841 r
.77673 .53247 .79808 .57217 .84496 .55881 .82393 .51864 Metetra
.648 .654 .841 r
.82393 .51864 .84496 .55881 .8926 .54523 .87192 .50458 Metetra
.648 .654 .841 r
.87192 .50458 .8926 .54523 .94104 .53142 .92071 .49028 Metetra
.648 .652 .84 r
.27939 .63484 .30402 .67097 .34376 .65933 .31933 .62281 Metetra
.648 .652 .84 r
.31933 .62281 .34376 .65933 .3841 .64751 .35988 .61059 Metetra
.648 .652 .84 r
.35988 .61059 .3841 .64751 .42506 .63551 .40106 .59819 Metetra
.648 .652 .84 r
.40106 .59819 .42506 .63551 .46664 .62332 .44288 .58559 Metetra
.648 .652 .84 r
.44288 .58559 .46664 .62332 .50888 .61095 .48536 .57279 Metetra
.648 .652 .84 r
.48536 .57279 .50888 .61095 .55177 .59838 .52851 .5598 Metetra
.648 .652 .84 r
.52851 .5598 .55177 .59838 .59535 .58561 .57234 .54659 Metetra
.648 .652 .84 r
.57234 .54659 .59535 .58561 .63961 .57264 .61688 .53317 Metetra
.648 .652 .84 r
.61688 .53317 .63961 .57264 .68458 .55947 .66214 .51954 Metetra
.648 .652 .84 r
.66214 .51954 .68458 .55947 .73028 .54608 .70814 .50568 Metetra
.648 .652 .84 r
.70814 .50568 .73028 .54608 .77673 .53247 .7549 .4916 Metetra
.648 .652 .84 r
.7549 .4916 .77673 .53247 .82393 .51864 .80243 .47728 Metetra
.648 .652 .84 r
.80243 .47728 .82393 .51864 .87192 .50458 .85075 .46272 Metetra
.648 .652 .84 r
.85075 .46272 .87192 .50458 .92071 .49028 .89989 .44792 Metetra
.648 .65 .838 r
.2543 .59769 .27939 .63484 .31933 .62281 .29444 .58525 Metetra
.648 .65 .838 r
.29444 .58525 .31933 .62281 .35988 .61059 .3352 .57263 Metetra
.648 .65 .838 r
.3352 .57263 .35988 .61059 .40106 .59819 .3766 .5598 Metetra
.648 .65 .838 r
.3766 .5598 .40106 .59819 .44288 .58559 .41865 .54677 Metetra
.648 .65 .838 r
.41865 .54677 .44288 .58559 .48536 .57279 .46137 .53354 Metetra
.648 .65 .838 r
.46137 .53354 .48536 .57279 .52851 .5598 .50476 .5201 Metetra
.648 .65 .838 r
.50476 .5201 .52851 .5598 .57234 .54659 .54886 .50643 Metetra
.648 .65 .838 r
.54886 .50643 .57234 .54659 .61688 .53317 .59367 .49255 Metetra
.648 .65 .838 r
.59367 .49255 .61688 .53317 .66214 .51954 .63921 .47844 Metetra
.648 .65 .838 r
.63921 .47844 .66214 .51954 .70814 .50568 .68551 .4641 Metetra
.648 .65 .838 r
.68551 .4641 .70814 .50568 .7549 .4916 .73258 .44952 Metetra
.648 .65 .838 r
.73258 .44952 .7549 .4916 .80243 .47728 .78043 .43469 Metetra
.648 .65 .838 r
.78043 .43469 .80243 .47728 .85075 .46272 .82909 .41962 Metetra
.648 .65 .838 r
.82909 .41962 .85075 .46272 .89989 .44792 .87859 .40428 Metetra
.648 .648 .837 r
.22874 .55949 .2543 .59769 .29444 .58525 .26908 .54663 Metetra
.648 .648 .837 r
.26908 .54663 .29444 .58525 .3352 .57263 .31005 .53358 Metetra
.648 .648 .837 r
.31005 .53358 .3352 .57263 .3766 .5598 .35166 .52031 Metetra
.648 .648 .837 r
.35166 .52031 .3766 .5598 .41865 .54677 .39394 .50684 Metetra
.648 .648 .837 r
.39394 .50684 .41865 .54677 .46137 .53354 .43689 .49316 Metetra
.648 .648 .837 r
.43689 .49316 .46137 .53354 .50476 .5201 .48053 .47925 Metetra
.648 .648 .837 r
.48053 .47925 .50476 .5201 .54886 .50643 .52489 .46511 Metetra
.648 .648 .837 r
.52489 .46511 .54886 .50643 .59367 .49255 .56997 .45075 Metetra
.648 .648 .837 r
.56997 .45075 .59367 .49255 .63921 .47844 .61579 .43615 Metetra
.648 .648 .837 r
.61579 .43615 .63921 .47844 .68551 .4641 .66238 .4213 Metetra
.648 .648 .837 r
.66238 .4213 .68551 .4641 .73258 .44952 .70975 .4062 Metetra
.648 .648 .837 r
.70975 .4062 .73258 .44952 .78043 .43469 .75793 .39085 Metetra
.648 .648 .837 r
.75793 .39085 .78043 .43469 .82909 .41962 .80693 .37524 Metetra
.648 .648 .837 r
.80693 .37524 .82909 .41962 .87859 .40428 .85677 .35935 Metetra
.647 .646 .835 r
.20271 .5202 .22874 .55949 .26908 .54663 .24325 .50691 Metetra
.647 .646 .835 r
.24325 .50691 .26908 .54663 .31005 .53358 .28442 .49342 Metetra
.647 .646 .835 r
.28442 .49342 .31005 .53358 .35166 .52031 .32624 .4797 Metetra
.647 .646 .835 r
.32624 .4797 .35166 .52031 .39394 .50684 .36874 .46577 Metetra
.647 .646 .835 r
.36874 .46577 .39394 .50684 .43689 .49316 .41192 .45161 Metetra
.647 .646 .835 r
.41192 .45161 .43689 .49316 .48053 .47925 .45581 .43722 Metetra
.647 .646 .835 r
.45581 .43722 .48053 .47925 .52489 .46511 .50042 .4226 Metetra
.647 .646 .835 r
.50042 .4226 .52489 .46511 .56997 .45075 .54576 .40773 Metetra
.647 .646 .835 r
.54576 .40773 .56997 .45075 .61579 .43615 .59187 .39262 Metetra
.647 .646 .835 r
.59187 .39262 .61579 .43615 .66238 .4213 .63875 .37725 Metetra
.647 .646 .835 r
.63875 .37725 .66238 .4213 .70975 .4062 .68643 .36162 Metetra
.647 .646 .835 r
.68643 .36162 .70975 .4062 .75793 .39085 .73492 .34572 Metetra
.647 .646 .835 r
.73492 .34572 .75793 .39085 .80693 .37524 .78425 .32954 Metetra
.647 .646 .835 r
.78425 .32954 .80693 .37524 .85677 .35935 .83444 .31309 Metetra
.647 .645 .834 r
.17621 .47981 .20271 .5202 .24325 .50691 .21693 .46608 Metetra
.647 .645 .834 r
.21693 .46608 .24325 .50691 .28442 .49342 .2583 .45212 Metetra
.647 .645 .834 r
.2583 .45212 .28442 .49342 .32624 .4797 .30033 .43794 Metetra
.647 .645 .834 r
.30033 .43794 .32624 .4797 .36874 .46577 .34305 .42353 Metetra
.647 .645 .834 r
.34305 .42353 .36874 .46577 .41192 .45161 .38646 .40888 Metetra
.647 .645 .834 r
.38646 .40888 .41192 .45161 .45581 .43722 .43058 .39399 Metetra
.647 .645 .834 r
.43058 .39399 .45581 .43722 .50042 .4226 .47544 .37886 Metetra
.647 .645 .834 r
.47544 .37886 .50042 .4226 .54576 .40773 .52105 .36347 Metetra
.647 .645 .834 r
.52105 .36347 .54576 .40773 .59187 .39262 .56743 .34782 Metetra
.647 .645 .834 r
.56743 .34782 .59187 .39262 .63875 .37725 .6146 .33191 Metetra
.647 .645 .834 r
.6146 .33191 .63875 .37725 .68643 .36162 .66258 .31572 Metetra
.647 .645 .834 r
.66258 .31572 .68643 .36162 .73492 .34572 .71139 .29925 Metetra
.647 .645 .834 r
.71139 .29925 .73492 .34572 .78425 .32954 .76105 .2825 Metetra
.647 .645 .834 r
.76105 .2825 .78425 .32954 .83444 .31309 .81159 .26545 Metetra
.647 .643 .832 r
.14922 .43829 .17621 .47981 .21693 .46608 .19012 .42409 Metetra
.647 .643 .832 r
.19012 .42409 .21693 .46608 .2583 .45212 .23169 .40966 Metetra
.647 .643 .832 r
.23169 .40966 .2583 .45212 .30033 .43794 .27392 .39499 Metetra
.647 .643 .832 r
.27392 .39499 .30033 .43794 .34305 .42353 .31685 .38008 Metetra
.647 .643 .832 r
.31685 .38008 .34305 .42353 .38646 .40888 .36049 .36493 Metetra
.647 .643 .832 r
.36049 .36493 .38646 .40888 .43058 .39399 .40485 .34952 Metetra
.647 .643 .832 r
.40485 .34952 .43058 .39399 .47544 .37886 .44995 .33386 Metetra
.647 .643 .832 r
.44995 .33386 .47544 .37886 .52105 .36347 .49582 .31793 Metetra
.647 .643 .832 r
.49582 .31793 .52105 .36347 .56743 .34782 .54247 .30173 Metetra
.647 .643 .832 r
.54247 .30173 .56743 .34782 .6146 .33191 .58993 .28525 Metetra
.647 .643 .832 r
.58993 .28525 .6146 .33191 .66258 .31572 .6382 .26849 Metetra
.647 .643 .832 r
.6382 .26849 .66258 .31572 .71139 .29925 .68732 .25143 Metetra
.647 .643 .832 r
.68732 .25143 .71139 .29925 .76105 .2825 .73731 .23407 Metetra
.647 .643 .832 r
.73731 .23407 .76105 .2825 .81159 .26545 .78819 .2164 Metetra
.646 .641 .831 r
.12174 .39561 .14922 .43829 .19012 .42409 .16282 .38093 Metetra
.646 .641 .831 r
.16282 .38093 .19012 .42409 .23169 .40966 .20458 .366 Metetra
.646 .641 .831 r
.20458 .366 .23169 .40966 .27392 .39499 .24701 .35083 Metetra
.646 .641 .831 r
.24701 .35083 .27392 .39499 .31685 .38008 .29015 .33541 Metetra
.646 .641 .831 r
.29015 .33541 .31685 .38008 .36049 .36493 .33401 .31973 Metetra
.646 .641 .831 r
.33401 .31973 .36049 .36493 .40485 .34952 .3786 .30379 Metetra
.646 .641 .831 r
.3786 .30379 .40485 .34952 .44995 .33386 .42395 .28757 Metetra
.646 .641 .831 r
.42395 .28757 .44995 .33386 .49582 .31793 .47007 .27108 Metetra
.646 .641 .831 r
.47007 .27108 .49582 .31793 .54247 .30173 .51699 .25431 Metetra
.646 .641 .831 r
.51699 .25431 .54247 .30173 .58993 .28525 .56472 .23724 Metetra
.646 .641 .831 r
.56472 .23724 .58993 .28525 .6382 .26849 .61329 .21988 Metetra
.646 .641 .831 r
.61329 .21988 .6382 .26849 .68732 .25143 .66272 .20221 Metetra
.646 .641 .831 r
.66272 .20221 .68732 .25143 .73731 .23407 .71303 .18422 Metetra
.646 .641 .831 r
.71303 .18422 .73731 .23407 .78819 .2164 .76425 .16591 Metetra
.646 .639 .829 r
.09377 .35175 .12174 .39561 .16282 .38093 .13503 .33656 Metetra
.646 .639 .829 r
.13503 .33656 .16282 .38093 .20458 .366 .17697 .32112 Metetra
.646 .639 .829 r
.17697 .32112 .20458 .366 .24701 .35083 .2196 .30543 Metetra
.646 .639 .829 r
.2196 .30543 .24701 .35083 .29015 .33541 .26294 .28947 Metetra
.646 .639 .829 r
.26294 .28947 .29015 .33541 .33401 .31973 .30701 .27325 Metetra
.646 .639 .829 r
.30701 .27325 .33401 .31973 .3786 .30379 .35183 .25675 Metetra
.646 .639 .829 r
.35183 .25675 .3786 .30379 .42395 .28757 .39741 .23996 Metetra
.646 .639 .829 r
.39741 .23996 .42395 .28757 .47007 .27108 .44379 .22289 Metetra
.646 .639 .829 r
.44379 .22289 .47007 .27108 .51699 .25431 .49097 .20552 Metetra
.646 .639 .829 r
.49097 .20552 .51699 .25431 .56472 .23724 .53898 .18785 Metetra
.646 .639 .829 r
.53898 .18785 .56472 .23724 .61329 .21988 .58784 .16986 Metetra
.646 .639 .829 r
.58784 .16986 .61329 .21988 .66272 .20221 .63757 .15155 Metetra
.646 .639 .829 r
.63757 .15155 .66272 .20221 .71303 .18422 .6882 .13291 Metetra
.646 .639 .829 r
.6882 .13291 .71303 .18422 .76425 .16591 .73975 .11394 Metetra
.646 .637 .827 r
.0653 .30666 .09377 .35175 .13503 .33656 .10673 .29095 Metetra
.646 .637 .827 r
.10673 .29095 .13503 .33656 .17697 .32112 .14885 .27499 Metetra
.646 .637 .827 r
.14885 .27499 .17697 .32112 .2196 .30543 .19167 .25875 Metetra
.646 .637 .827 r
.19167 .25875 .2196 .30543 .26294 .28947 .23521 .24224 Metetra
.646 .637 .827 r
.23521 .24224 .26294 .28947 .30701 .27325 .27949 .22545 Metetra
.646 .637 .827 r
.27949 .22545 .30701 .27325 .35183 .25675 .32453 .20837 Metetra
.646 .637 .827 r
.32453 .20837 .35183 .25675 .39741 .23996 .37035 .191 Metetra
.646 .637 .827 r
.37035 .191 .39741 .23996 .44379 .22289 .41696 .17333 Metetra
.646 .637 .827 r
.41696 .17333 .44379 .22289 .49097 .20552 .4644 .15534 Metetra
.646 .637 .827 r
.4644 .15534 .49097 .20552 .53898 .18785 .51268 .13703 Metetra
.646 .637 .827 r
.51268 .13703 .53898 .18785 .58784 .16986 .56183 .1184 Metetra
.646 .637 .827 r
.56183 .1184 .58784 .16986 .63757 .15155 .61186 .09943 Metetra
.646 .637 .827 r
.61186 .09943 .63757 .15155 .6882 .13291 .66281 .08011 Metetra
.646 .637 .827 r
.66281 .08011 .6882 .13291 .73975 .11394 .71469 .06044 Metetra
.645 .635 .826 r
.03634 .26033 .0653 .30666 .10673 .29095 .07793 .24409 Metetra
.645 .635 .826 r
.07793 .24409 .10673 .29095 .14885 .27499 .12022 .22757 Metetra
.645 .635 .826 r
.12022 .22757 .14885 .27499 .19167 .25875 .16322 .21077 Metetra
.645 .635 .826 r
.16322 .21077 .19167 .25875 .23521 .24224 .20695 .19369 Metetra
.645 .635 .826 r
.20695 .19369 .23521 .24224 .27949 .22545 .25144 .17632 Metetra
.645 .635 .826 r
.25144 .17632 .27949 .22545 .32453 .20837 .29669 .15864 Metetra
.645 .635 .826 r
.29669 .15864 .32453 .20837 .37035 .191 .34274 .14065 Metetra
.645 .635 .826 r
.34274 .14065 .37035 .191 .41696 .17333 .3896 .12235 Metetra
.645 .635 .826 r
.3896 .12235 .41696 .17333 .4644 .15534 .43729 .10372 Metetra
.645 .635 .826 r
.43729 .10372 .4644 .15534 .51268 .13703 .48583 .08476 Metetra
.645 .635 .826 r
.48583 .08476 .51268 .13703 .56183 .1184 .53526 .06546 Metetra
.645 .635 .826 r
.53526 .06546 .56183 .1184 .61186 .09943 .58558 .0458 Metetra
.645 .635 .826 r
.58558 .0458 .61186 .09943 .66281 .08011 .63684 .02578 Metetra
.645 .635 .826 r
.63684 .02578 .66281 .08011 .71469 .06044 .68905 .00539 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{169.375, 137.25},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002Y0000RA000`40O003h00OZGoo002YOol00:Uo
o`00ZGoo002YOol00:Uoo`00ZGoo002YOol006moo`<000Aoo`80035oo`00LWoo00@007ooOol0009o
o`03001oogoo02ioo`00LWoo00@007ooOol0009oo`03001oogoo02ioo`00Kgoo0`000goo00@007oo
Ool0031oo`00Kgoo00<007ooOol00goo00@007ooOol0031oo`00Kgoo10000goo0P00<Goo002YOol0
0:Uoo`00ZGoo001hOol2000_Ool005Ioo`<0009oo`<001Moo`<00003DYX007oo02ioo`00Egoo00<0
07ooOol017oo00<007ooOol04Goo0`0015:J0P00;Woo001EOol40005Ool00`00Oomoo`0?Ool30006
DYX3000]Ool005Eoo`03001oo`0000=oo`<000moo`<00003DYX005:J00IBVP<002eoo`00EGoo00<0
07oo00000goo00<007ooOol03Goo0P001E:J00<005:JDYX015:J00@005:J000002aoo`00EWoo0P00
0goo10002Goo0`001e:J00<005:JDYX015:J00D005:J001oo`0002]oo`00IWoo0`0000=BVP00DYX0
1e:J00<005:JDYX00e:J00D005:J001oo`0002]oo`00Hgoo0`001E:J00<005:JDYX01E:J00<005:J
DYX00e:J00H005:J000007oo000ZOol0061oo`@000MBVP03001BVU:J00IBVP04001BVU:JDYX30000
0e:J001oo`02000YOol003ioo`<000=oo`8001Qoo`@00003DYX005:J00MBVP03001BVU:J00EBVP04
001BVP000002DYX01P00DYYBVP00Ool002Uoo`00?goo00D007ooOomoo`00009oo`03001oogoo019o
o`@000EBVP03001BVU:J00EBVP03001BVU:J00EBVP8000ABVP04001BVU:J0002Ool00`00Oomoo`0V
Ool003eoo`@0009oo`04001oogoo000BOol40007DYX00`00DYYBVP06DYX01000DYYBVU:J0`0000=B
VP00DYX00e:J00L005:JDYYBVP00Ool0000XOol003eoo`03001oo`0000=oo`04001oogoo000?Ool4
0002DYX00`00DYYBVP06DYX00`00DYYBVP05DYX01000DYX000001E:J00D005:JDYYBVP0000=BVP04
001oogoo000WOol003eoo`03001oo`0000=oo`04001oogoo000=Ool30006DYX00`00DYYBVP06DYX0
0`00DYYBVP03DYX30007DYX01@00DYYBVU:J000015:J00@007oo000002Ioo`00?Woo0P0017oo0P00
2goo0`0000=BVP00DYX01U:J00<005:JDYX01U:J00<005:JDYX00`000U:J00<005:JDYX01U:J00@0
05:JDYX000ABVP04001oogoo000VOol004moo`<000ABVP03001BVU:J00EBVP03001BVU:J00EBVP<0
00IBVP03001BVU:J00IBVP03001BVP0000=BVP<0009oo`03001oogoo02=oo`00C7oo10001U:J00<0
05:JDYX01U:J00@005:JDYYBVP<00003DYX005:J00IBVP03001BVU:J00IBVP03001BVP0000<0009B
VP05001oogooOol0000TOol004Yoo`<0009BVP03001BVU:J00EBVP03001BVU:J00EBVP04001BVP00
0005DYX00`00DYYBVP05DYX00`00DYYBVP05DYX30006DYX00`00Oomoo`020005Ool3000KOol002Io
o`<000=oo`<001Qoo`<000IBVP03001BVU:J00EBVP03001BVU:J00=BVP<000MBVP03001BVU:J00EB
VP03001BVU:J009BVP<00003DYX0000000IBVP03001oogoo008000Ioo`03001oogoo01Yoo`00:Goo
00<007ooOol00goo00<007ooOol04Woo0`0000=BVP00DYX01U:J00<005:JDYX01E:J00<005:JDYX0
0`000U:J00<005:JDYX01U:J00<005:JDYX015:J00<005:J00000P001E:J00<005:JDYX01E:J00D0
07ooOomoo`0000Eoo`03001oogoo01Yoo`00:Goo00<007ooOol00goo00<007ooOol047oo0`0015:J
00<005:JDYX01E:J00<005:JDYX015:J0`001U:J00<005:JDYX01E:J00<005:JDYX015:J0P0025:J
00<005:JDYX01E:J00D007ooOomoo`0000Eoo`03001oogoo01Yoo`009goo0P000goo0`0047oo1000
1U:J00<005:JDYX01E:J00@005:JDYYBVP<00003DYX005:J00IBVP03001BVU:J00IBVP04001BVU:J
DYX200000e:J001BVP07DYX00`00DYYBVP05DYX00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomo
o`0JOol002Uoo`04001oogoo000@Ool30002DYX00`00DYYBVP05DYX00`00DYYBVP04DYX01000DYX0
00001E:J00<005:JDYX01E:J00<005:JDYX01E:J00@005:J000000ABVP03001BVU:J00EBVP8000IB
VP8000=oo`80009oo`8001aoo`009Woo0`000goo10002Woo0`001U:J00<005:JDYX01E:J00<005:J
DYX00e:J0P001e:J00<005:JDYX01U:J00<005:JDYX00e:J0`001U:J00<005:JDYX01E:J0P000e:J
0`0000=BVP00Ool00goo00<007ooOol07Woo000hOol200000e:J001BVP06DYX00`00DYYBVP05DYX0
0`00DYYBVP0300000e:J001BVP07DYX00`00DYYBVP05DYX00`00DYYBVP030002DYX00`00DYYBVP05
DYX00`00DYYBVP04DYX200000e:J00000005DYX00`00Oomoo`02Ool00`00Oomoo`0MOol003Eoo`<0
00ABVP03001BVU:J00EBVP03001BVU:J00ABVP<000EBVP03001BVU:J00EBVP03001BVU:J00EBVP<0
00IBVP03001BVU:J00EBVP03001BVU:J00=BVP<000MBVP03001oogoo00=oo`03001oogoo01aoo`00
<Woo10001U:J00<005:JDYX01U:J00@005:JDYYBVP800003DYX005:J00EBVP03001BVU:J00IBVP04
001BVU:JDYX300000e:J001BVP06DYX00`00DYYBVP05DYX00`00DYYBVP0300000e:J00000008DYX0
0`00Oomoo`02Ool2000MOol0015oo`<000Aoo`8001Ioo`@00003DYX005:J00IBVP03001BVU:J00EB
VP04001BVP000004DYX00`00DYYBVP04DYX00`00DYYBVP05DYX01000DYX000001E:J00<005:JDYX0
1E:J00<005:JDYX015:J0`0015:J00<005:J000025:J00<007ooOol00Woo00<007ooOol06goo000D
Ool01000Oomoo`000Woo00<007ooOol04Goo0`001E:J00<005:JDYX01E:J00<005:JDYX00e:J0`00
1U:J00<005:JDYX01E:J00<005:JDYX00e:J0`001e:J00<005:JDYX01E:J00@005:JDYYBVP<00003
DYX005:J00ABVP03001BVP0000QBVP03001oogoo00=oo`03001oogoo01Yoo`0057oo00@007ooOol0
009oo`03001oogoo00ioo`<00003DYX005:J00EBVP03001BVU:J00EBVP03001BVU:J00<0009BVP03
001BVU:J00EBVP03001BVU:J00ABVP03001BVU:J00<0009BVP03001BVU:J00IBVP03001BVU:J00AB
VP04001BVP000005DYX00`00DYYBVP02DYX01000DYYBVP001e:J0P0017oo00<007ooOol06Woo000B
Ool20003Ool01000Oomoo`003Goo100015:J00<005:JDYX015:J00<005:JDYX015:J0`001U:J00<0
05:JDYX01E:J00<005:JDYX00e:J0`001U:J00<005:JDYX01E:J00<005:JDYX00e:J0`001e:J00<0
05:JDYX00U:J00D005:JDYYBVP0000=BVP<00003DYX007oo00Aoo`03001oogoo01Uoo`0057oo00@0
07ooOol0009oo`03001oogoo00Qoo`D000IBVP03001BVU:J00EBVP04001BVU:JDYX200000e:J001B
VP06DYX00`00DYYBVP05DYX00`00DYYBVP0300000e:J001BVP07DYX00`00DYYBVP04DYX00`00DYYB
VP030002DYX00`00DYYBVP06DYX01@00DYYBVU:J00000e:J00@005:J000000EBVP03001oogoo009o
o`03001oogoo01Uoo`004Goo0`0017oo0P002Goo1@0000=BVP00DYX01U:J00<005:JDYX015:J00@0
05:J000000ABVP03001BVU:J00EBVP03001BVU:J00ABVP<000EBVP03001BVU:J00EBVP03001BVU:J
00ABVP<000IBVP03001BVU:J00EBVP05001BVU:JDYX00002DYX30007DYX00`00Oomoo`03Ool00`00
Oomoo`0HOol0025oo`@000EBVP03001BVU:J00EBVP03001BVU:J00=BVP8000IBVP03001BVU:J00EB
VP04001BVU:JDYX300000e:J001BVP05DYX00`00DYYBVP06DYX00`00DYYBVP0300000e:J001BVP07
DYX00`00DYYBVP04DYX00`00DYYBVP040002DYX00`00DYYBVP06DYX00`00Oomoo`03Ool00`00Oomo
o`0GOol001ioo`@00003DYX005:J00IBVP03001BVU:J00ABVP03001BVU:J00<00003DYX005:J00IB
VP03001BVU:J00ABVP04001BVP000005DYX00`00DYYBVP04DYX00`00DYYBVP05DYX30005DYX00`00
DYYBVP05DYX00`00DYYBVP04DYX300000e:J001BVP04DYX00`00DYYBVP05DYX00`00Oomoo`03Ool2
000HOol001eoo`8000EBVP03001BVU:J00ABVP03001BVU:J00ABVP<000EBVP03001BVU:J00EBVP03
001BVU:J00=BVP8000MBVP03001BVU:J00EBVP04001BVU:JDYX300000e:J001BVP05DYX00`00DYYB
VP06DYX00`00DYYBVP0300000e:J001BVP02DYX00`00DYYBVP02DYX00`00DYYBVP05DYX00`00Oomo
o`04Ool00`00Oomoo`0FOol001Ioo`<000Aoo`03001BVU:J00ABVP03001BVU:J00EBVP03001BVU:J
00<00003DYX005:J00IBVP03001BVU:J00ABVP03001BVU:J00<00003DYX005:J00MBVP03001BVU:J
00ABVP04001BVP000005DYX00`00DYYBVP04DYX00`00DYYBVP05DYX30005DYX01@00DYYBVU:J0000
15:J00<005:JDYX00e:J0`001Woo00<007ooOol05Goo000GOol00`00Oomoo`03Ool20006DYX00`00
DYYBVP04DYX30005DYX00`00DYYBVP04DYX00`00DYYBVP04DYX30005DYX00`00DYYBVP05DYX00`00
DYYBVP03DYX30007DYX00`00DYYBVP05DYX01000DYYBVU:J0`0000=BVP00DYX01E:J00<005:JDYX0
0U:J00D005:JDYYBVP00009BVP<0009BVP03001oogoo00Aoo`03001oogoo00Aoo`@000eoo`0047oo
1@000Woo00<007ooOol00goo0P001e:J00@005:JDYYBVP800003DYX005:J00EBVP03001BVU:J00EB
VP03001BVU:J00<00003DYX005:J00IBVP03001BVU:J00ABVP03001BVU:J00<0009BVP03001BVU:J
00IBVP03001BVU:J00ABVP@000EBVP03001BVU:J00ABVP05001BVU:JDYX00003DYX30006DYX00`00
Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`0=Ool001Moo`03001oogoo009oo`04001oogoo0006
DYX01000DYX0000015:J00<005:JDYX015:J00<005:JDYX015:J0`001E:J00<005:JDYX015:J00<0
05:JDYX015:J0`001U:J00<005:JDYX01E:J00<005:JDYX00U:J10001e:J00<005:JDYX01E:J00@0
05:JDYYBVP<00003DYX005:J00IBVP03001oogoo00Eoo`03001oogoo00Aoo`03001oogoo00aoo`00
5goo00<007ooOol00Woo0P000Woo00<005:JDYX00e:J0P001U:J00<005:JDYX01E:J00@005:JDYYB
VP800003DYX005:J00EBVP03001BVU:J00EBVP03001BVU:J00<00003DYX005:J00MBVP03001BVU:J
00ABVP03001BVP0000<0009BVP03001BVU:J00IBVP03001BVU:J00ABVP06001BVP00001BVP000e:J
00<005:JDYX01E:J00<007ooOol017oo00<007ooOol01Goo00<007ooOol02goo000FOol20004Ool0
1@00Oomoogoo00000U:J0`0000=BVP00DYX01U:J00<005:JDYX015:J00@005:J000000ABVP03001B
VU:J00ABVP03001BVU:J00ABVP<000EBVP03001BVU:J00EBVP03001BVU:J00=BVP@000IBVP03001B
VU:J00EBVP03001BVU:J00=BVP<000=BVP05001BVU:JDYX00007DYX00`00Oomoo`05Ool01@00Oomo
ogoo00000Woo00<007ooOol02goo000LOol00`00Oomoo`02Ool20005DYX00`00DYYBVP05DYX00`00
DYYBVP03DYX20006DYX00`00DYYBVP05DYX01000DYYBVU:J0P0000=BVP00DYX01E:J00<005:JDYX0
1U:J00<005:J00000`0000=BVP00DYX01U:J00<005:JDYX01E:J00<005:JDYX00`000U:J00<005:J
DYX00U:J00D005:JDYYBVP0000IBVP03001oogoo00Eoo`8000=oo`8000ioo`0077oo00<007ooOol0
0Woo00<005:JDYX01E:J00<005:JDYX015:J00<005:JDYX00`0000=BVP00DYX01U:J00<005:JDYX0
15:J00@005:J000000ABVP03001BVU:J00ABVP03001BVU:J00ABVP@000EBVP03001BVU:J00EBVP03
001BVU:J00=BVP@000IBVP03001BVU:J009BVP04001BVU:J0005DYX30007Ool00`00Oomoo`0AOol0
01aoo`03001oogoo00=oo`03001BVU:J00ABVP03001BVU:J00ABVP<000EBVP03001BVU:J00EBVP03
001BVU:J00=BVP8000IBVP03001BVU:J00EBVP04001BVU:JDYX300000e:J001BVP05DYX00`00DYYB
VP05DYX00`00DYYBVP030002DYX00`00DYYBVP05DYX00`00DYYBVP03DYX00`00DYX00002DYX30002
DYX00`00Oomoo`06Ool00`00Oomoo`0@Ool001aoo`03001oogoo00Aoo`03001BVU:J00ABVP03001B
VU:J00<00003DYX005:J00IBVP03001BVU:J00ABVP03001BVU:J00<00003DYX005:J00IBVP03001B
VU:J00ABVP@000EBVP03001BVU:J00ABVP03001BVU:J00ABVP<000IBVP03001BVU:J00EBVP03001B
VU:J009BVPD000IBVP03001oogoo00Ioo`03001oogoo00moo`006goo0P001Woo00<005:JDYX015:J
0`001E:J00<005:JDYX015:J00<005:JDYX015:J0`001E:J00<005:JDYX01E:J00@005:JDYYBVP@0
00MBVP03001BVU:J00EBVP04001BVU:JDYX200000e:J001BVP06DYX00`00DYYBVP05DYX00`00DYYB
VP0400000e:J001BVP06DYX00`00Oomoo`06Ool00`00Oomoo`0?Ool000Uoo`8000Aoo`8000=oo`<0
00Aoo`03001oogoo00Ioo`03001BVU:J00<00003DYX005:J00IBVP03001BVU:J00ABVP03001BVU:J
00<00003DYX005:J00IBVP03001BVU:J00ABVPH00003DYX005:J00MBVP03001BVU:J00ABVP04001B
VP000004DYX00`00DYYBVP05DYX00`00DYYBVP03DYX40004DYX00`00DYX00007DYX00`00Oomoo`06
Ool00`00Oomoo`0>Ool000Qoo`04001oogoo0003Ool20006Ool01@00Oomoogoo00002Goo0P001E:J
00<005:JDYX015:J00<005:JDYX015:J0`001E:J00<005:JDYX015:J00<005:JDYX00U:J1@001E:J
00<005:JDYX01E:J00<005:JDYX00e:J0`001U:J00<005:JDYX01E:J00<005:JDYX00`000U:J00<0
05:JDYX00e:J00<005:J00001e:J00<007ooOol01goo00<007ooOol03Goo0002Ool500001Goo001o
ogoo00002Goo0`000goo00<007ooOol01goo00<005:JDYX01E:J00<005:JDYX015:J00<005:JDYX0
0`0000=BVP00DYX01E:J00<005:JDYX01E:J1P0000=BVP00DYX01U:J00<005:JDYX015:J00<005:J
DYX00`000U:J00<005:JDYX01E:J00<005:JDYX015:J0`001U:J00<005:JDYX00e:J00<005:J0000
1E:J0`0027oo00<007ooOol03Goo0008Ool01000Oomoo`0027oo00@007ooOol000=oo`03001oogoo
00Qoo`03001BVU:J00ABVP03001BVU:J00ABVP<000EBVP03001BVU:J00ABVP03001BVU:J009BVPD0
00EBVP03001BVU:J00ABVP03001BVU:J00ABVP<000IBVP03001BVU:J00EBVP04001BVU:JDYX20000
0e:J001BVP06DYX00`00DYYBVP04DYX20002DYX30002DYX00`00Oomoo`07Ool00`00Oomoo`0<Ool0
00Qoo`04001oogoo0008Ool01000Oomoo`000goo00<007ooOol02Goo00<005:JDYX015:J00<005:J
DYX00`0000=BVP00DYX01E:J00<005:JDYX015:J00<005:J00000`000U:J00<005:JDYX015:J00<0
05:JDYX01E:J00<005:JDYX00`0000=BVP00DYX01U:J00<005:JDYX01E:J00@005:J000000ABVP03
001BVU:J00EBVP03001BVU:J00ABVP<000IBVP03001oogoo00Ioo`03001oogoo00aoo`002Goo0P00
2Woo0P000goo00<007ooOol02Woo00<005:JDYX015:J0`001E:J00<005:JDYX015:J00<005:JDYX0
0U:J10001U:J00<005:JDYX015:J00<005:JDYX00e:J10001E:J00<005:JDYX01E:J00<005:JDYX0
0e:J0`001U:J00<005:JDYX01E:J00@005:JDYYBVPD000MBVP03001oogoo00Moo`8000aoo`006Woo
00<007ooOol02goo00<005:JDYX00`0000=BVP00DYX01E:J00<005:JDYX015:J00<005:J00000`00
00=BVP00DYX01U:J00<005:JDYX015:J00<005:JDYX00`000U:J00<005:JDYX01E:J00<005:JDYX0
15:J00<005:JDYX00`000U:J00<005:JDYX01E:J00<005:JDYX015:J00@005:J000000ABVP8000MB
VP03001oogoo00Moo`03001oogoo00Yoo`006Woo0P003Goo0P001E:J00<005:JDYX015:J00<005:J
DYX00U:J10001E:J00<005:JDYX01E:J00<005:JDYX00e:J0`001U:J00<005:JDYX015:J00<005:J
DYX015:J0`001U:J00<005:JDYX01E:J00<005:JDYX00e:J0P001e:J00<005:JDYX01E:J00<007oo
Ool01goo00<007ooOol02Woo000JOol00`00Oomoo`0<Ool00`00DYYBVP05DYX00`00DYYBVP03DYX0
0`00DYX0000300000e:J001BVP06DYX00`00DYYBVP04DYX00`00DYYBVP0300000e:J001BVP06DYX0
0`00DYYBVP05DYX00`00DYYBVP0300000e:J001BVP06DYX00`00DYYBVP05DYX00`00DYYBVP030000
0e:J001BVP06DYX20006DYX20009Ool00`00Oomoo`03Ool30003Ool001Yoo`03001oogoo00eoo`03
001BVU:J00ABVP03001BVU:J00=BVP<000EBVP03001BVU:J00ABVP03001BVU:J00ABVP<000EBVP03
001BVU:J00EBVP03001BVU:J00=BVP@000EBVP03001BVU:J00EBVP03001BVU:J00=BVP@000EBVP03
001BVU:J00EBVP04001BVU:JDYX300000e:J001oo`08Ool20007Ool10001Ool1Ool001Yoo`03001o
ogoo00ioo`03001BVU:J00ABVP03001BVP0000800003DYX005:J00IBVP03001BVU:J00ABVP03001B
VU:J00<00003DYX005:J00IBVP03001BVU:J00ABVP03001BVU:J00<0009BVP03001BVU:J00ABVP03
001BVU:J00EBVP03001BVU:J00<0009BVP03001BVU:J00EBVP03001BVU:J00ABVP@000EBVP03001o
ogoo00Moo`03001oogoo00Eoo`40005oo`5oo`006Goo00<007ooOol03goo00<005:JDYX00e:J0`00
1E:J00<005:JDYX015:J00<005:JDYX015:J0`001E:J00<005:JDYX015:J00<005:JDYX015:J0`00
1U:J00<005:JDYX015:J00<005:JDYX00e:J10001U:J00<005:JDYX015:J00<005:JDYX015:J0`00
1U:J00<007ooOol027oo00<007ooOol00Woo0P000goo0008Ool20004Ool20004Ool20003Ool2000A
Ool00`00DYX0000300000e:J001BVP05DYX00`00DYYBVP04DYX00`00DYYBVP0300000e:J001BVP06
DYX00`00DYYBVP04DYX00`00DYYBVP0300000e:J001BVP06DYX00`00DYYBVP04DYX00`00DYYBVP03
0002DYX00`00DYYBVP05DYX00`00DYYBVP05DYX00`00DYYBVP0300000e:J00000007DYX00`00Oomo
o`06Ool30005Ool10001Ool1Ool000Moo`04001oogoo0003Ool20003Ool01000Oomoo`000Woo00<0
07ooOol03goo10001E:J00<005:JDYX00e:J00<005:JDYX015:J0`001E:J00<005:JDYX015:J00<0
05:JDYX015:J0`001E:J00<005:JDYX01E:J00<005:JDYX00e:J0`001U:J00<005:JDYX01E:J00<0
05:JDYX00e:J10001E:J0P001U:J00<007ooOol00goo0`000Woo00<007ooOol00`000goo00000goo
0000000300001Goo001oogoo000027oo00@007ooOol0009oo`03001oogoo00aoo`<0009oo`03001B
VU:J00ABVP03001BVU:J00ABVP03001BVU:J00<00003DYX005:J00IBVP03001BVU:J00ABVP03001B
VU:J00<00003DYX005:J00IBVP03001BVU:J00ABVP03001BVU:J00<00003DYX005:J00IBVP03001B
VU:J00EBVP03001BVU:J00<0009BVP03001BVU:J00ABVP03001BVP0000EBVP03001oogoo00<000Eo
o`03001oogoo00Ioo`001goo00@007ooOol000Uoo`8000=oo`03001oogoo00Uoo`<000Ioo`03001B
VU:J00ABVP03001BVU:J00=BVP<000EBVP03001BVU:J00ABVP03001BVU:J00ABVP<000EBVP03001B
VU:J00ABVP03001BVU:J00ABVP<000EBVP03001BVU:J00EBVP03001BVU:J00=BVP@000IBVP03001B
VU:J00ABVP03001BVP0000=BVP@000Qoo`03001oogoo00Ioo`001goo00@007ooOol000Qoo`04001o
ogoo0002Ool00`00Oomoo`06Ool3000:Ool00`00DYYBVP03DYX00`00DYYBVP0300000e:J001BVP06
DYX00`00DYYBVP04DYX00`00DYYBVP0300000e:J001BVP05DYX00`00DYYBVP05DYX00`00DYYBVP03
00000e:J001BVP06DYX00`00DYYBVP04DYX00`00DYYBVP030002DYX00`00DYYBVP05DYX00`00DYYB
VP04DYX00`00DYX000030002DYX00`00Oomoo`07Ool00`00Oomoo`06Ool000Qoo`8000Yoo`80009o
o`8000Eoo`<000eoo`03001BVU:J00=BVP<000EBVP03001BVU:J00ABVP03001BVU:J00ABVP<000EB
VP03001BVU:J00ABVP03001BVU:J00=BVP@000EBVP03001BVU:J00ABVP03001BVU:J00ABVP<000IB
VP03001BVU:J00EBVP03001BVU:J00=BVP@000EBVP03001oogoo00Moo`03001oogoo00Eoo`0067oo
00@007ooOomoo`<0015oo`03001BVP0000800003DYX005:J00IBVP03001BVU:J00ABVP03001BVU:J
00<00003DYX005:J00EBVP03001BVU:J00ABVP03001BVU:J00<0009BVP03001BVU:J00ABVP03001B
VU:J00EBVP03001BVU:J00<00003DYX005:J00IBVP03001BVU:J00EBVP03001BVU:J00<00005DYX0
05:JDYX00004DYX00`00Oomoo`07Ool00`00Oomoo`05Ool001Qoo`@001Eoo`03001BVU:J00=BVP03
001BVU:J00ABVP03001BVU:J00ABVP<000EBVP03001BVU:J00ABVP03001BVU:J00=BVP<000IBVP03
001BVU:J00ABVP03001BVU:J00=BVP@000EBVP03001BVU:J00EBVP03001BVU:J00=BVP@000EBVP03
001BVP0000EBVP03001oogoo00Ioo`03001oogoo00Eoo`0067oo0P005goo00<005:JDYX015:J00<0
05:JDYX015:J00<005:JDYX00`0000=BVP00DYX01E:J00<005:JDYX015:J00<005:JDYX00`0000=B
VP00DYX01U:J00<005:JDYX015:J00<005:JDYX00`000U:J00<005:JDYX015:J00<005:JDYX01E:J
00<005:JDYX00`000U:J00<005:JDYX015:J00@005:JDYX000ABVP03001oogoo00Ioo`03001oogoo
00Eoo`006Woo00<007ooOol05Goo00<005:JDYX00e:J00<005:JDYX015:J0`001E:J00<005:JDYX0
15:J00<005:JDYX00e:J0`001E:J00<005:JDYX01E:J00<005:JDYX00e:J0`001U:J00<005:JDYX0
15:J00<005:JDYX00e:J10001U:J00<005:JDYX015:J00@005:JDYX0009BVP<000Moo`03001oogoo
00Eoo`006goo00<007ooOol057oo00<005:JDYX015:J00<005:JDYX00`0000=BVP00DYX01E:J00<0
05:JDYX015:J00<005:JDYX00`0000=BVP00DYX01U:J00<005:JDYX015:J00<005:JDYX00`0000=B
VP00DYX01U:J00<005:JDYX015:J00<005:JDYX00`000U:J00<005:JDYX01E:J00<005:JDYX015:J
00<005:JDYX00`000U:J00<007ooOol01Goo00<007ooOol01Goo000LOol00`00Oomoo`0DOol00`00
DYYBVP03DYX30005DYX00`00DYYBVP04DYX00`00DYYBVP03DYX30005DYX00`00DYYBVP04DYX00`00
DYYBVP04DYX30005DYX00`00DYYBVP05DYX00`00DYYBVP03DYX30006DYX00`00DYYBVP05DYX00`00
DYYBVP03DYX300000e:J001BVP03DYX00`00Oomoo`04Ool00`00Oomoo`05Ool001eoo`03001oogoo
01=oo`03001BVU:J00<00003DYX005:J00EBVP03001BVU:J00ABVP03001BVU:J00<00003DYX005:J
00IBVP03001BVU:J00ABVP03001BVU:J00<00003DYX005:J00IBVP03001BVU:J00ABVP03001BVU:J
00<00003DYX005:J00MBVP03001BVU:J00ABVP03001BVU:J00<00003DYX005:J009BVP03001BVU:J
009BVP03001oogoo00=oo`03001oogoo00Eoo`007Woo0P0057oo0P001E:J00<005:JDYX015:J00<0
05:JDYX00e:J0`001E:J00<005:JDYX015:J00<005:JDYX015:J0`001E:J00<005:JDYX015:J00<0
05:JDYX015:J0`001E:J00<005:JDYX01E:J00<005:JDYX00U:J1@001E:J00D005:JDYYBVP0000=B
VP03001oogoo00Aoo`03001oogoo00Aoo`0087oo00<007ooOol04Goo00<005:JDYX01E:J00<005:J
DYX00e:J00<005:JDYX00`0000=BVP00DYX01U:J00<005:JDYX015:J00<005:JDYX00`0000=BVP00
DYX01U:J00<005:JDYX015:J00<005:JDYX00`0000=BVP00DYX01U:J00<005:JDYX015:J1@000e:J
00<005:JDYX01E:J00@005:JDYX0009BVP<000Eoo`03001oogoo00Aoo`008Goo00<007ooOol04Goo
00<005:JDYX01E:J00@005:JDYYBVP@000EBVP03001BVU:J00EBVP04001BVU:JDYX50005DYX00`00
DYYBVP04DYX00`00DYYBVP02DYX50005DYX00`00DYYBVP05DYX00`00DYYBVP0300000e:J001BVP06
DYX00`00DYYBVP04DYX00`00DYYBVP030002DYX00`00Oomoo`03Ool00`00Oomoo`04Ool0029oo`03
001oogoo015oo`03001BVU:J00ABVP@0009BVP03001BVU:J00EBVP03001BVU:J00ABVP@000=BVP03
001BVU:J00EBVP03001BVU:J00=BVPD000=BVP03001BVU:J00EBVP03001BVU:J00=BVP@000EBVP03
001BVU:J00EBVP03001BVU:J00=BVP<0009BVP05001BVU:JDYX00004Ool00`00Oomoo`04Ool002=o
o`80019oo`03001BVU:J00<00003DYX005:J00ABVP03001BVU:J00EBVP03001BVU:J00<00003DYX0
05:J00EBVP03001BVU:J00ABVP03001BVU:J00<00003DYX005:J00IBVP03001BVU:J00ABVP03001B
VU:J00<0009BVP03001BVU:J00ABVP03001BVU:J00EBVP03001BVU:J00<00003DYX005:J00=BVP04
001BVU:J0004Ool00`00Oomoo`04Ool002Eoo`03001oogoo011oo`8000EBVP03001BVU:J00=BVP03
001BVU:J00ABVP<000EBVP03001BVU:J00ABVP03001BVU:J00=BVP<000EBVP03001BVU:J00EBVP03
001BVU:J00=BVP<000IBVP03001BVU:J00ABVP03001BVU:J00=BVP@000EBVP05001BVU:JDYX00003
DYX01@00Oomoogoo00001Woo000VOol00`00Oomoo`0?Ool00`00DYYBVP04DYX00`00DYYBVP04DYX0
0`00DYYBVP0300000e:J001BVP05DYX00`00DYYBVP04DYX00`00DYYBVP0300000e:J001BVP06DYX0
0`00DYYBVP04DYX00`00DYYBVP0300000e:J001BVP06DYX00`00DYYBVP04DYX00`00DYYBVP030002
DYX00`00DYYBVP04DYX00`00DYYBVP02DYX01000DYYBVP000goo00<007ooOol017oo000WOol00`00
Oomoo`0?Ool00`00DYYBVP04DYX00`00DYYBVP03DYX30005DYX00`00DYYBVP04DYX00`00DYYBVP03
DYX30005DYX00`00DYYBVP04DYX00`00DYYBVP04DYX30005DYX00`00DYYBVP05DYX00`00DYYBVP03
DYX30006DYX00`00DYYBVP04DYX00`00DYYBVP02DYX40003Ool00`00Oomoo`03Ool002Qoo`03001o
ogoo00moo`03001BVU:J00=BVP03001BVU:J00<00003DYX005:J00EBVP03001BVU:J00ABVP03001B
VU:J00<00003DYX005:J00EBVP03001BVU:J00EBVP03001BVU:J00<00003DYX005:J00EBVP03001B
VU:J00EBVP03001BVU:J00<00003DYX005:J00IBVP03001BVU:J00ABVP03001BVU:J00<0009B^`05
001oogooOol00005Ool002Uoo`8000moo`03001BVU:J00=BVP<000EBVP03001BVU:J00ABVP03001B
VU:J00=BVP<000EBVP03001BVU:J00ABVP03001BVU:J00=BVP@000EBVP03001BVU:J00ABVP03001B
VU:J00=BVP@000EBVP03001BVU:J00EBVP03001BVU:J00=BVP<000=B^`04001B^e:k0002Ool00`00
Oomoo`03Ool002]oo`03001oogoo00eoo`03001BVP0000800003DYX005:J00IBVP03001BVU:J00=B
VP03001BVU:J00<00003DYX005:J00EBVP03001BVU:J00ABVP03001BVU:J00<0009BVP03001BVU:J
00ABVP03001BVU:J00ABVP03001BVU:J00<0009BVP03001BVU:J00EBVP03001BVU:J00ABVP03001B
VU:J00<00003D[/005:k00AB^`06001B^e:k001oo`001Goo000/Ool00`00Oomoo`0=Ool00`00DYYB
VP03DYX00`00DYYBVP04DYX01000DYYBVU:J1@001E:J00<005:JDYX015:J00<005:JDYX00e:J0`00
1U:J00<005:JDYX015:J00<005:JDYX00e:J0`001U:J00<005:JDYX015:J00<005:JDYX00U:J1@00
1E:k00<005:kD[/00e:k00D005:k001oo`0000Eoo`00;Goo00<007ooOol037oo00<005:JDYX015:J
00<005:JDYX00e:J10000e:J00<005:JDYX01E:J00<005:JDYX00e:J00<005:JDYX00`0000=BVP00
DYX01e:J00<005:JDYX00e:J00<005:JDYX00`0000=BVP00DYX01e:J00<005:JDYX00e:J1@000e:k
00<005:kD[/01E:k00<005:kD[/00U:k1@001Goo000^Ool2000=Ool00`00DYYBVP04DYX00`00DYX0
000200000e:J001BVP05DYX00`00DYYBVP04DYX01000DYYBVU:J1@001E:J00<005:JDYX01E:J00@0
05:JDYYBVPD000EBVP03001BVU:J00EBVP03001BVU:J00<00003D[/005:k00IB^`03001B^e:k00AB
^`03001B^e:k00@00003D[/0000000Eoo`00<7oo00<007ooOol02goo00<005:JDYX00e:J0P001E:J
00<005:JDYX015:J00<005:JDYX00e:J10000e:J00<005:JDYX01E:J00<005:JDYX015:J10000e:J
00<005:JDYX01E:J00<005:JDYX00e:J10001E:k00<005:kD[/015:k00<005:kD[/00e:k100015:k
00<005:k00001Goo000aOol00`00Oomoo`0:Ool00`00DYYBVP0300000e:J001BVP05DYX00`00DYYB
VP04DYX00`00DYX0000200000e:J001BVP05DYX00`00DYYBVP05DYX00`00DYYBVP0300000e:J001B
VP05DYX00`00DYYBVP04DYX00`00DYYBVP030002D[/00`00D[]B^`04D[/00`00D[]B^`04D[/00`00
D[]B^`030002D[/00`00D[]B^`03D[/01000D[/0000017oo000bOol00`00Oomoo`0:Ool20005DYX0
0`00DYYBVP03DYX00`00DYYBVP04DYX20005DYX00`00DYYBVP04DYX00`00DYYBVP04DYX30005DYX0
0`00DYYBVP04DYX00`00DYYBVP03DYX30006D[/00`00D[]B^`04D[/00`00D[]B^`03D[/30006D[/0
0`00D[]B^`03D[/30004Ool003=oo`03001oogoo00Uoo`03001BVU:J00EBVP03001BVU:J00=BVP03
001BVU:J00<00003DYX005:J00EBVP03001BVU:J00ABVP03001BVU:J00<00003DYX005:J00IBVP03
001BVU:J00=BVP03001BVU:J00<00003D[/005:k00IB^`03001B^e:k00AB^`03001B^e:k00<00003
D[/005:k00MB^`03001B^e:k00=B^`8000Aoo`00=7oo0P002Woo00<005:JDYX015:J00@005:JDYYB
VPD000EBVP03001BVU:J00ABVP04001BVU:JDYX50005DYX00`00DYYBVP04DYX01000DYYBVU:J1@00
1E:k00<005:kD[/015:k00<005:kD[/00U:k1@001E:k00<005:kD[/01E:k00@005:kD[]B^`@000Ao
o`00=Woo00<007ooOol027oo00<005:JDYX00e:J10000e:J00<005:JDYX01E:J00<005:JDYX00e:J
10000e:J00<005:JDYX01E:J00<005:JDYX00e:J10000e:k00<005:kD[/01E:k00<005:kD[/00e:k
1@000e:k00<005:kD[/01E:k00<005:kD[/00e:k1P001goo000gOol00`00Oomoo`08Ool00`00DYX0
000200000e:J001BVP05DYX00`00DYYBVP04DYX00`00DYYBVP0300000e:J001BVP05DYX00`00DYYB
VP04DYX00`00DYYBVP0300000e:k001B^`05D[/00`00D[]B^`04D[/00`00D[]B^`0300000e:k001B
^`06D[/00`00D[]B^`04D[/00`00D[]B^`06000:Ool003Qoo`03001oogoo00Qoo`03001BVU:J00=B
VP03001BVU:J00ABVP03001BVU:J00=BVP<000EBVP03001BVU:J00ABVP03001BVU:J00=BVP<000EB
^`03001B^e:k00AB^`03001B^e:k00=B^`<000EB^`03001B^e:k00EB^`03001B^e:k00=B^`D000io
o`00>Goo00<007ooOol01goo00<005:JDYX015:J00<005:JDYX00e:J00<005:JDYX00`0000=BVP00
DYX01U:J00<005:JDYX00e:J00<005:JDYX00`0000=B^`00D[/01U:k00<005:kD[/00e:k00<005:k
D[/00`0000=B^`00D[/01E:k00<005:kD[/01E:k00<005:kD[/010004Woo000jOol20008Ool00`00
DYYBVP04DYX01000DYYBVU:J10001E:J00<005:JDYX015:J00@005:JDYYBVPD000EB^`03001B^e:k
00AB^`04001B^e:kD[/50005D[/00`00D[]B^`04D[/00`00D[]B^`03D[/5000EOol003aoo`03001o
ogoo00Ioo`03001BVU:J00=BVP@0009BVP03001BVU:J00EBVP03001BVU:J00=BVP@000=B^`03001B
^e:k00EB^`03001B^e:k00=B^`@000=B^`03001B^e:k00EB^`03001B^e:k00=B^`03001B^e:k00@0
01Uoo`00?Goo00<007ooOol01Goo00<005:JDYX00`0000=BVP00DYX015:J00<005:JDYX015:J00<0
05:JDYX00`0000=B^`00D[/01E:k00<005:kD[/015:k00<005:kD[/00`0000=B^`00D[/01E:k00<0
05:kD[/015:k00@005:kD[]B^`D001eoo`00?Woo00<007ooOol01Goo0P001E:J00<005:JDYX00e:J
00<005:JDYX00e:J0`001E:k00<005:kD[/015:k00<005:kD[/00e:k0`001E:k00<005:kD[/015:k
00<005:kD[/00e:k1P0087oo000oOol20005Ool00`00DYYBVP05DYX00`00DYYBVP02DYX00`00DYYB
VP0300000e:k001B^`06D[/00`00D[]B^`03D[/00`00D[]B^`0300000e:k001B^`06D[/00`00D[]B
^`03D[/00`00D[]B^`05000TOol0045oo`03001oogoo00=oo`03001BVU:J00ABVP04001BVU:JDYX4
0005D[/00`00D[]B^`04D[/01000D[]B^e:k1@001E:k00<005:kD[/015:k00@005:kD[]B^`H002Qo
o`00@Woo00<007ooOol00goo00<005:JDYX00e:J10000e:k00<005:kD[/015:k00<005:kD[/00e:k
10000e:k00<005:kD[/01E:k00<005:kD[/00e:k1`00:goo0013Ool00`00Oomoo`02Ool00`00DYYB
VP0300000e:k001B^`05D[/00`00D[]B^`03D[/00`00D[]B^`0300000e:k001B^`05D[/00`00D[]B
^`04D[/00`00D[]B^`06000_Ool004Aoo`03001oogoo009oo`8000EB^`03001B^e:k00AB^`04001B
^e:kD[/40005D[/00`00D[]B^`04D[/01000D[]B^e:k1`00<goo0015Ool20003Ool00`00D[]B^`04
D[/00`00D[]B^`03D[/40002D[/00`00D[]B^`05D[/00`00D[]B^`03D[/8000fOol004Moo`05001o
ogooOol00005D[/00`00D[]B^`0300000e:k001B^`04D[/00`00D[]B^`04D[/00`00D[]B^`07000j
Ool004Qoo`05001oogooOol00003D[/40005D[/00`00D[]B^`03D[/00`00D[]B^`03D[/6000nOol0
04Uoo`04001oogooOol30002D[/00`00D[]B^`05D[/00`00D[]B^`02D[/00`00D[]B^`060011Ool0
04Yoo`800003Ool005:k00AB^`03001B^e:k00AB^`04001B^e:kD[/60015Ool004aoo`03001oo`00
00EB^`04001B^e:kD[/80019Ool004eoo`03001oo`0000AB^`@00003Ool00000008004aoo`00CWoo
0P0000EB^`00000007oo00@0051oo`00Cgoo0P0000Aoo`00000005Aoo`00D7oo0P00Egoo002YOol0
0:Uoo`00ZGoo002YOol00:Uoo`00\
\>"],
  ImageRangeCache->{{{0, 168.375}, {136.25, 0}} -> {-0.165444, -0.0465057, 
  0.00677941, 0.00677941}}]
}, Open  ]],

Cell[TextData[{
  "Gamma is zero, as is shown in the next plot, which just reveals the \
machine-precision rumblings of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "!"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[UpAndOutPutGamma[0, S, 50, 50, 0.25,0.1, 0.1, t],
{S, 30, 50}, {t, 0.001, 3}, PlotRange -> All];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.66533e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.80843 .15978 0 -6.10127 ]
[.80843 .15978 6 2.89873 ]
[.90116 .30178 0 -5.96725 ]
[.90116 .30178 6 3.03275 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.0174 .30719 -43.5 -3.85817 ]
[.0174 .30719 0 8.64183 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00393 .44089 -37.5 -4.01601 ]
[-0.00393 .44089 0 8.48399 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.79606 .16417 m
.78988 .16637 L
s
[(1)] .80843 .15978 -1 .35584 Mshowa
.88868 .30585 m
.88245 .30788 L
s
[(2)] .90116 .30178 -1 .32606 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(3)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.71146 .03476 m
.70779 .03618 L
s
.7336 .06862 m
.72991 .07001 L
s
.75506 .10145 m
.75136 .10281 L
s
.77587 .13328 m
.77217 .13462 L
s
.81567 .19417 m
.81195 .19546 L
s
.83471 .22329 m
.83099 .22457 L
s
.85321 .25159 m
.84948 .25285 L
s
.8712 .2791 m
.86746 .28034 L
s
.90569 .33187 m
.90195 .33307 L
s
.92225 .35719 m
.91849 .35838 L
s
.93836 .38184 m
.9346 .38301 L
s
.95406 .40585 m
.95029 .407 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.02966 .3025 m
.03578 .30015 L
s
gsave
.0174 .30719 -43.5 -1.85817 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(-2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-16) show
grestore
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.00842 .43648 m
.0146 .43427 L
s
gsave
-0.00393 .44089 -37.5 -2.01601 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-16) show
grestore
.125 Mabswid
.0271 .31864 m
.03078 .31724 L
s
.02451 .33495 m
.0282 .33356 L
s
.0219 .35143 m
.02559 .35005 L
s
.0166 .38491 m
.02029 .38355 L
s
.0139 .40192 m
.0176 .40057 L
s
.01118 .4191 m
.01488 .41777 L
s
.03219 .28652 m
.03586 .2851 L
s
.03469 .2707 m
.03836 .26928 L
s
.00564 .45404 m
.00935 .45272 L
s
.00283 .47179 m
.00654 .47048 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.542 .717 .954 r
.37892 .68418 .40162 .70396 .43934 .69754 .41682 .68179 Metetra
.764 .809 .889 r
.41682 .68179 .43934 .69754 .47783 .68039 .45588 .65789 Metetra
.657 .732 .905 r
.45588 .65789 .47783 .68039 .51674 .66093 .4951 .64662 Metetra
.537 .782 .988 r
.4951 .64662 .51674 .66093 .55622 .66472 .53491 .64166 Metetra
.676 .738 .898 r
.53491 .64166 .55622 .66472 .59617 .64702 .57533 .62792 Metetra
.746 .793 .891 r
.57533 .62792 .59617 .64702 .63649 .62698 .61606 .60534 Metetra
.394 .639 .954 r
.61606 .60534 .63649 .62698 .67825 .63046 .65828 .6119 Metetra
.747 .807 .9 r
.65828 .6119 .67825 .63046 .72007 .61761 .70007 .58886 Metetra
.648 .694 .879 r
.70007 .58886 .72007 .61761 .76189 .59586 .743 .57766 Metetra
.672 .801 .944 r
.743 .57766 .76189 .59586 .8048 .58261 .78635 .56297 Metetra
.651 .782 .943 r
.78635 .56297 .8048 .58261 .84891 .5752 .8305 .55029 Metetra
.652 .735 .91 r
.8305 .55029 .84891 .5752 .89304 .56048 .8754 .53796 Metetra
.559 .72 .948 r
.8754 .53796 .89304 .56048 .93861 .55219 .92201 .53382 Metetra
.717 .83 .935 r
.92201 .53382 .93861 .55219 .98397 .53624 .96743 .51328 Metetra
.677 .756 .911 r
.35563 .66388 .37892 .68418 .41682 .68179 .39421 .65051 Metetra
.669 .674 .845 r
.39421 .65051 .41682 .68179 .45588 .65789 .43331 .63696 Metetra
.696 .761 .902 r
.43331 .63696 .45588 .65789 .4951 .64662 .47296 .62109 Metetra
.438 .625 .932 r
.47296 .62109 .4951 .64662 .53491 .64166 .51301 .6267 Metetra
.693 .827 .947 r
.51301 .6267 .53491 .64166 .57533 .62792 .5539 .61046 Metetra
.777 .816 .884 r
.5539 .61046 .57533 .62792 .61606 .60534 .59507 .58311 Metetra
.524 .701 .95 r
.59507 .58311 .61606 .60534 .65828 .6119 .63735 .58176 Metetra
.6 .668 .887 r
.63735 .58176 .65828 .6119 .70007 .58886 .68034 .57594 Metetra
.751 .876 .937 r
.68034 .57594 .70007 .58886 .743 .57766 .72313 .55225 Metetra
.684 .742 .895 r
.72313 .55225 .743 .57766 .78635 .56297 .76685 .53608 Metetra
.666 .726 .894 r
.76685 .53608 .78635 .56297 .8305 .55029 .81135 .52191 Metetra
.648 .71 .893 r
.81135 .52191 .8305 .55029 .8754 .53796 .85674 .50976 Metetra
.566 .677 .914 r
.85674 .50976 .8754 .53796 .92201 .53382 .9039 .50638 Metetra
.683 .739 .893 r
.9039 .50638 .92201 .53382 .96743 .51328 .95035 .48934 Metetra
.553 .629 .881 r
.33231 .6323 .35563 .66388 .39421 .65051 .37056 .6315 Metetra
.599 .757 .954 r
.37056 .6315 .39421 .65051 .43331 .63696 .40983 .62415 Metetra
.754 .852 .923 r
.40983 .62415 .43331 .63696 .47296 .62109 .45012 .60141 Metetra
.457 .7 .972 r
.45012 .60141 .47296 .62109 .51301 .6267 .49052 .60246 Metetra
.709 .748 .882 r
.49052 .60246 .51301 .6267 .5539 .61046 .5318 .58365 Metetra
.691 .726 .877 r
.5318 .58365 .5539 .61046 .59507 .58311 .57358 .56684 Metetra
.547 .78 .985 r
.57358 .56684 .59507 .58311 .63735 .58176 .61618 .56085 Metetra
.653 .772 .936 r
.61618 .56085 .63735 .58176 .68034 .57594 .65935 .54805 Metetra
.713 .735 .868 r
.65935 .54805 .68034 .57594 .72313 .55225 .70278 .52726 Metetra
.666 .745 .909 r
.70278 .52726 .72313 .55225 .76685 .53608 .74712 .51292 Metetra
.622 .748 .937 r
.74712 .51292 .76685 .53608 .81135 .52191 .7925 .50285 Metetra
.682 .814 .946 r
.7925 .50285 .81135 .52191 .85674 .50976 .83805 .48583 Metetra
.567 .723 .947 r
.83805 .48583 .85674 .50976 .9039 .50638 .88553 .48044 Metetra
.71 .768 .898 r
.88553 .48044 .9039 .50638 .95035 .48934 .93212 .45957 Metetra
.468 .733 .984 r
.30755 .61512 .33231 .6323 .37056 .6315 .34611 .61413 Metetra
.675 .797 .94 r
.34611 .61413 .37056 .6315 .40983 .62415 .38603 .59994 Metetra
.694 .743 .889 r
.38603 .59994 .40983 .62415 .45012 .60141 .42656 .58336 Metetra
.521 .75 .979 r
.42656 .58336 .45012 .60141 .49052 .60246 .46738 .57978 Metetra
.712 .767 .895 r
.46738 .57978 .49052 .60246 .5318 .58365 .50911 .56058 Metetra
.778 .778 .853 r
.50911 .56058 .5318 .58365 .57358 .56684 .55124 .53015 Metetra
.521 .595 .869 r
.55124 .53015 .57358 .56684 .61618 .56085 .59425 .53262 Metetra
.637 .708 .898 r
.59425 .53262 .61618 .56085 .65935 .54805 .63789 .52167 Metetra
.675 .74 .899 r
.63789 .52167 .65935 .54805 .70278 .52726 .68206 .50605 Metetra
.664 .785 .939 r
.68206 .50605 .70278 .52726 .74712 .51292 .72694 .49132 Metetra
.723 .803 .914 r
.72694 .49132 .74712 .51292 .7925 .50285 .77201 .46961 Metetra
.602 .667 .885 r
.77201 .46961 .7925 .50285 .83805 .48583 .81889 .46232 Metetra
.569 .735 .953 r
.81889 .46232 .83805 .48583 .88553 .48044 .86685 .45602 Metetra
.712 .788 .911 r
.86685 .45602 .88553 .48044 .93212 .45957 .91406 .43466 Metetra
.543 .717 .953 r
.28235 .59307 .30755 .61512 .34611 .61413 .32131 .58963 Metetra
.724 .749 .872 r
.32131 .58963 .34611 .61413 .38603 .59994 .36191 .56849 Metetra
.617 .67 .879 r
.36191 .56849 .38603 .59994 .42656 .58336 .40243 .56032 Metetra
.548 .716 .95 r
.40243 .56032 .42656 .58336 .46738 .57978 .44359 .55646 Metetra
.755 .774 .869 r
.44359 .55646 .46738 .57978 .50911 .56058 .48586 .53018 Metetra
.703 .715 .858 r
.48586 .53018 .50911 .56058 .55124 .53015 .52846 .51035 Metetra
.533 .748 .974 r
.52846 .51035 .55124 .53015 .59425 .53262 .57177 .50588 Metetra
.636 .726 .913 r
.57177 .50588 .59425 .53262 .63789 .52167 .61582 .49455 Metetra
.724 .755 .877 r
.61582 .49455 .63789 .52167 .68206 .50605 .66018 .47176 Metetra
.581 .651 .884 r
.66018 .47176 .68206 .50605 .72694 .49132 .706 .46676 Metetra
.685 .771 .917 r
.706 .46676 .72694 .49132 .77201 .46961 .75195 .44911 Metetra
.694 .816 .94 r
.75195 .44911 .77201 .46961 .81889 .46232 .79846 .43008 Metetra
.534 .649 .908 r
.79846 .43008 .81889 .46232 .86685 .45602 .84742 .42909 Metetra
.665 .756 .918 r
.84742 .42909 .86685 .45602 .91406 .43466 .89589 .413 Metetra
.606 .737 .938 r
.25646 .5704 .28235 .59307 .32131 .58963 .296 .56227 Metetra
.613 .702 .908 r
.296 .56227 .32131 .58963 .36191 .56849 .33625 .55398 Metetra
.737 .846 .932 r
.33625 .55398 .36191 .56849 .40243 .56032 .37781 .53219 Metetra
.564 .683 .919 r
.37781 .53219 .40243 .56032 .44359 .55646 .41924 .52798 Metetra
.614 .702 .907 r
.41924 .52798 .44359 .55646 .48586 .53018 .46164 .51917 Metetra
.771 .896 .931 r
.46164 .51917 .48586 .53018 .52846 .51035 .50493 .49216 Metetra
.516 .764 .986 r
.50493 .49216 .52846 .51035 .57177 .50588 .54872 .48742 Metetra
.671 .818 .954 r
.54872 .48742 .57177 .50588 .61582 .49455 .59323 .4712 Metetra
.714 .788 .91 r
.59323 .4712 .61582 .49455 .66018 .47176 .63824 .45019 Metetra
.652 .79 .948 r
.63824 .45019 .66018 .47176 .706 .46676 .68412 .43574 Metetra
.674 .722 .886 r
.68412 .43574 .706 .46676 .75195 .44911 .73062 .41877 Metetra
.638 .716 .904 r
.73062 .41877 .75195 .44911 .79846 .43008 .77819 .40612 Metetra
.593 .755 .955 r
.77819 .40612 .79846 .43008 .84742 .42909 .82694 .3967 Metetra
.689 .73 .882 r
.82694 .3967 .84742 .42909 .89589 .413 .87556 .37665 Metetra
.473 .779 .997 r
.22905 .55595 .25646 .5704 .296 .56227 .26867 .55201 Metetra
.804 .881 .902 r
.26867 .55201 .296 .56227 .33625 .55398 .31085 .52109 Metetra
.616 .671 .88 r
.31085 .52109 .33625 .55398 .37781 .53219 .35208 .51225 Metetra
.595 .774 .965 r
.35208 .51225 .37781 .53219 .41924 .52798 .39408 .50325 Metetra
.549 .715 .95 r
.39408 .50325 .41924 .52798 .46164 .51917 .43674 .49862 Metetra
.733 .81 .912 r
.43674 .49862 .46164 .51917 .50493 .49216 .48058 .47563 Metetra
.495 .779 .995 r
.48058 .47563 .50493 .49216 .54872 .48742 .52489 .47062 Metetra
.735 .854 .936 r
.52489 .47062 .54872 .48742 .59323 .4712 .56992 .4471 Metetra
.746 .796 .893 r
.56992 .4471 .59323 .4712 .63824 .45019 .61537 .42104 Metetra
.615 .717 .918 r
.61537 .42104 .63824 .45019 .68412 .43574 .66193 .41072 Metetra
.632 .762 .941 r
.66193 .41072 .68412 .43574 .73062 .41877 .70928 .39789 Metetra
.715 .831 .937 r
.70928 .39789 .73062 .41877 .77819 .40612 .75682 .37558 Metetra
.493 .656 .933 r
.75682 .37558 .77819 .40612 .82694 .3967 .80701 .37619 Metetra
.712 .845 .945 r
.80701 .37619 .82694 .3967 .87556 .37665 .85614 .35329 Metetra
.745 .737 .844 r
.202 .52757 .22905 .55595 .26867 .55201 .24375 .50094 Metetra
.456 .499 .815 r
.24375 .50094 .26867 .55201 .31085 .52109 .28322 .50985 Metetra
.827 .876 .88 r
.28322 .50985 .31085 .52109 .35208 .51225 .3266 .47368 Metetra
.476 .58 .879 r
.3266 .47368 .35208 .51225 .39408 .50325 .36817 .47778 Metetra
.695 .762 .904 r
.36817 .47778 .39408 .50325 .43674 .49862 .41166 .45912 Metetra
.762 .698 .791 r
.41166 .45912 .43674 .49862 .48058 .47563 .45613 .42212 Metetra
.433 .482 .811 r
.45613 .42212 .48058 .47563 .52489 .47062 .50033 .43475 Metetra
.652 .686 .869 r
.50033 .43475 .52489 .47062 .56992 .4471 .54584 .41993 Metetra
.727 .773 .889 r
.54584 .41993 .56992 .4471 .61537 .42104 .59193 .39563 Metetra
.632 .761 .94 r
.59193 .39563 .61537 .42104 .66193 .41072 .63894 .38258 Metetra
.635 .743 .926 r
.63894 .38258 .66193 .41072 .70928 .39789 .68682 .36929 Metetra
.694 .765 .907 r
.68682 .36929 .70928 .39789 .75682 .37558 .73512 .34877 Metetra
.557 .728 .954 r
.73512 .34877 .75682 .37558 .80701 .37619 .78527 .34196 Metetra
.681 .728 .886 r
.78527 .34196 .80701 .37619 .85614 .35329 .83532 .32202 Metetra
.787 .801 .864 r
.17329 .50742 .202 .52757 .24375 .50094 .21602 .47581 Metetra
.547 .716 .951 r
.21602 .47581 .24375 .50094 .28322 .50985 .25677 .47089 Metetra
.678 .669 .834 r
.25677 .47089 .28322 .50985 .3266 .47368 .29934 .45225 Metetra
0 .417 .839 r
.29934 .45225 .3266 .47368 .36817 .47778 .34045 .46994 Metetra
.804 .925 .916 r
.34045 .46994 .36817 .47778 .41166 .45912 .38525 .43701 Metetra
.764 .811 .89 r
.38525 .43701 .41166 .45912 .45613 .42212 .43016 .40847 Metetra
.095 .588 .901 r
.43016 .40847 .45613 .42212 .50033 .43475 .47486 .41646 Metetra
.736 .851 .934 r
.47486 .41646 .50033 .43475 .54584 .41993 .52102 .39191 Metetra
.779 .785 .857 r
.52102 .39191 .54584 .41993 .59193 .39563 .56757 .35784 Metetra
.532 .63 .894 r
.56757 .35784 .59193 .39563 .63894 .38258 .61528 .35588 Metetra
.695 .783 .919 r
.61528 .35588 .63894 .38258 .68682 .36929 .66343 .33511 Metetra
.655 .717 .893 r
.66343 .33511 .68682 .36929 .73512 .34877 .71255 .31871 Metetra
.59 .723 .936 r
.71255 .31871 .73512 .34877 .78527 .34196 .76307 .30907 Metetra
.674 .742 .901 r
.76307 .30907 .78527 .34196 .83532 .32202 .81384 .28975 Metetra
.605 .738 .939 r
.14417 .48208 .17329 .50742 .21602 .47581 .18512 .4726 Metetra
.706 .946 .957 r
.18512 .4726 .21602 .47581 .25677 .47089 .2281 .44921 Metetra
.847 .796 .801 r
.2281 .44921 .25677 .47089 .29934 .45225 .27365 .39851 Metetra
0 .071 .633 r
.27365 .39851 .29934 .45225 .34045 .46994 .31246 .44769 Metetra
.853 .806 .804 r
.31246 .44769 .34045 .46994 .38525 .43701 .3589 .39572 Metetra
.745 .694 .802 r
.3589 .39572 .38525 .43701 .43016 .40847 .40437 .36214 Metetra
.389 .503 .853 r
.40437 .36214 .43016 .40847 .47486 .41646 .449 .37412 Metetra
.614 .645 .857 r
.449 .37412 .47486 .41646 .52102 .39191 .49541 .363 Metetra
.818 .789 .826 r
.49541 .363 .52102 .39191 .56757 .35784 .54257 .31908 Metetra
.292 .498 .886 r
.54257 .31908 .56757 .35784 .61528 .35588 .59098 .33534 Metetra
.735 .853 .936 r
.59098 .33534 .61528 .35588 .66343 .33511 .6397 .3093 Metetra
.778 .824 .888 r
.6397 .3093 .66343 .33511 .71255 .31871 .68862 .27594 Metetra
.473 .581 .882 r
.68862 .27594 .71255 .31871 .76307 .30907 .7405 .2799 Metetra
.695 .785 .921 r
.7405 .2799 .76307 .30907 .81384 .28975 .79176 .25762 Metetra
.654 .718 .895 r
.11475 .45142 .14417 .48208 .18512 .4726 .15649 .43693 Metetra
.651 .687 .871 r
.15649 .43693 .18512 .4726 .2281 .44921 .19893 .4222 Metetra
.755 .771 .866 r
.19893 .4222 .2281 .44921 .27365 .39851 .24328 .39348 Metetra
.679 .134 0 r
.24328 .39348 .27365 .39851 .31246 .44769 .28316 .4294 Metetra
.865 .834 .815 r
.28316 .4294 .31246 .44769 .3589 .39572 .33056 .37653 Metetra
.507 .778 .993 r
.33056 .37653 .3589 .39572 .40437 .36214 .3755 .37011 Metetra
.3755 .37011 .40437 .36214 .449 .37412 .42148 .36355 Metetra
.722 .923 .96 r
.42148 .36355 .449 .37412 .49541 .363 .46889 .33786 Metetra
.653 .789 .947 r
.46889 .33786 .49541 .363 .54257 .31908 .51682 .32131 Metetra
.878 .99 .811 r
.51682 .32131 .54257 .31908 .59098 .33534 .56511 .26694 Metetra
.49 .483 .777 r
.56511 .26694 .59098 .33534 .6397 .3093 .61484 .27309 Metetra
.746 .751 .856 r
.61484 .27309 .6397 .3093 .68862 .27594 .66451 .24145 Metetra
.192 .477 .893 r
.66451 .24145 .68862 .27594 .7405 .2799 .71788 .26171 Metetra
.803 .91 .914 r
.71788 .26171 .7405 .2799 .79176 .25762 .76894 .22439 Metetra
.667 .754 .916 r
.08389 .42449 .11475 .45142 .15649 .43693 .12616 .4084 Metetra
.703 .754 .891 r
.12616 .4084 .15649 .43693 .19893 .4222 .16962 .38746 Metetra
.588 .679 .904 r
.16962 .38746 .19893 .4222 .24328 .39348 .21242 .37893 Metetra
.462 .801 .999 r
.21242 .37893 .24328 .39348 .28316 .4294 .25594 .37255 Metetra
.717 .634 .763 r
.25594 .37255 .28316 .4294 .33056 .37653 .30246 .33793 Metetra
.712 .708 .845 r
.30246 .33793 .33056 .37653 .3755 .37011 .3487 .31236 Metetra
.574 .564 .804 r
.3487 .31236 .3755 .37011 .42148 .36355 .3947 .30508 Metetra
.308 .41 .804 r
.3947 .30508 .42148 .36355 .46889 .33786 .44104 .32618 Metetra
.942 .871 .738 r
.44104 .32618 .46889 .33786 .51682 .32131 .49055 .24308 Metetra
.317 .349 .737 r
.49055 .24308 .51682 .32131 .56511 .26694 .53937 .27257 Metetra
.31 .797 .754 r
.53937 .27257 .56511 .26694 .61484 .27309 .58962 .25485 Metetra
.888 .893 .828 r
.58962 .25485 .61484 .27309 .66451 .24145 .63936 .19879 Metetra
.127 .383 .839 r
.63936 .19879 .66451 .24145 .71788 .26171 .69346 .22809 Metetra
.782 .797 .865 r
.69346 .22809 .71788 .26171 .76894 .22439 .74532 .19002 Metetra
.656 .752 .92 r
.05207 .39674 .08389 .42449 .12616 .4084 .0946 .38132 Metetra
.672 .764 .919 r
.0946 .38132 .12616 .4084 .16962 .38746 .13805 .36391 Metetra
.661 .792 .945 r
.13805 .36391 .16962 .38746 .21242 .37893 .18208 .34739 Metetra
.554 .7 .937 r
.18208 .34739 .21242 .37893 .25594 .37255 .22592 .34055 Metetra
.762 .761 .851 r
.22592 .34055 .25594 .37255 .30246 .33793 .27287 .30764 Metetra
.408 .648 .956 r
.27287 .30764 .30246 .33793 .3487 .31236 .31783 .30968 Metetra
.27 .771 .877 r
.31783 .30968 .3487 .31236 .3947 .30508 .36464 .30218 Metetra
.782 .977 .911 r
.36464 .30218 .3947 .30508 .44104 .32618 .41363 .26575 Metetra
.817 .677 .709 r
.41363 .26575 .44104 .32618 .49055 .24308 .46366 .19194 Metetra
.5 .562 .851 r
.46366 .19194 .49055 .24308 .53937 .27257 .51245 .19226 Metetra
.391 .397 .746 r
.51245 .19226 .53937 .27257 .58962 .25485 .563 .21164 Metetra
.869 .756 .735 r
.563 .21164 .58962 .25485 .63936 .19879 .61296 .13633 Metetra
0 .098 .65 r
.61296 .13633 .63936 .19879 .69346 .22809 .66865 .20309 Metetra
.8 .87 .9 r
.66865 .20309 .69346 .22809 .74532 .19002 .72152 .16415 Metetra
.656 .751 .92 r
.01926 .36808 .05207 .39674 .0946 .38132 .06217 .35219 Metetra
.656 .751 .92 r
.06217 .35219 .0946 .38132 .13805 .36391 .10583 .33602 Metetra
.656 .764 .929 r
.10583 .33602 .13805 .36391 .18208 .34739 .15025 .31957 Metetra
.656 .769 .932 r
.15025 .31957 .18208 .34739 .22592 .34055 .19545 .30283 Metetra
.652 .703 .885 r
.19545 .30283 .22592 .34055 .27287 .30764 .24146 .28579 Metetra
.65 .822 .966 r
.24146 .28579 .27287 .30764 .31783 .30968 .28829 .26844 Metetra
.65 .689 .873 r
.28829 .26844 .31783 .30968 .36464 .30218 .33597 .25079 Metetra
.64 .637 .831 r
.33597 .25079 .36464 .30218 .41363 .26575 .38452 .2328 Metetra
.656 .751 .92 r
.38452 .2328 .41363 .26575 .46366 .19194 .43397 .21449 Metetra
.43397 .21449 .46366 .19194 .51245 .19226 .48434 .19583 Metetra
.378 .842 .814 r
.48434 .19583 .51245 .19226 .563 .21164 .53566 .17683 Metetra
.656 .751 .92 r
.53566 .17683 .563 .21164 .61296 .13633 .58795 .15746 Metetra
.809 .348 .136 r
.58795 .15746 .61296 .13633 .66865 .20309 .646 .25883 Metetra
.94 .799 .34 r
.646 .25883 .66865 .20309 .72152 .16415 .68905 .00539 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(30)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(35)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(40)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(45)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(50)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{169.375, 137.25},
  ImageMargins->{{35, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002Y0000RA000`40O003h00OZGoo002YOol00:Uo
o`00ZGoo002YOol00:Uoo`00ZGoo002YOol00:Uoo`00ZGoo002YOol00:Uoo`00ZGoo002YOol00:Uo
o`00M7oo0`0017oo0P00;7oo001gOol01000Oomoo`000Woo00<007ooOol0:Goo001gOol01000Oomo
o`000Woo00<007ooOol0:Goo001dOol30003Ool01000Oomoo`00:goo001dOol00`00Oomoo`03Ool0
1000Oomoo`00:goo001dOol40003Ool2000/Ool00:Uoo`00ZGoo002YOol005ioo`<0009oo`<001Mo
o`8002Yoo`00Ggoo00<007ooOol017oo00<007ooOol047oo100000=oo`00Ool0:Goo001MOol40005
Ool00`00Oomoo`0>Ool40003Ool2000YOol005eoo`03001oo`0000=oo`<000moo`8000Moo`<002Qo
o`00GGoo00<007oo00000goo00<007ooOol037oo0`0027oo1@009goo001NOol20003Ool40009Ool2
000;Ool300000goo001oo`0VOol006aoo`@000eoo`<0009oo`03001oogoo02Aoo`00JWoo0`0047oo
0`000Woo00<007ooOol097oo0019Ool30003Ool2000GOol2000COol200001G/Z001oogoo00009Goo
001:Ool01@00Oomoogoo00000Woo00<007ooOol04Goo10004goo00D007/Z001k:P0000=oo`03001o
ogoo029oo`00B7oo10000Woo00@007ooOol0015oo`8001Moo`05001k:P00NbX00003Ool00`00Oomo
o`0ROol004Qoo`03001oo`0000=oo`04001oogoo000=Ool4000IOol01@00NbX007/Z000017oo00<0
07ooOol08Goo0018Ool00`00Ool00003Ool01000Oomoo`002goo10006goo00D007/Z001k:P0000Ao
o`80029oo`00BGoo0P0017oo0P002Woo0P007goo00<007/Z00000W/Z00<007ooOol00Woo00<007oo
Ool087oo001HOol3000POol01000NbYk:P000W/Z00<007ooOol00goo00<007ooOol07goo001EOol3
000SOol01000NbYk:P000W/Z00<007ooOol00goo00<007ooOol07goo000dOol30003Ool3000FOol2
000OOol00`00Oomoo`04Ool01`00NbYk:W/Z001k:P0000Ioo`03001oogoo009oo`<001Uoo`00=goo
00<007ooOol00goo00<007ooOol047oo0`0000=oo`00Ool07Goo0P001Woo00L007/ZNbYk:P00NbX0
0007Ool00`00Oomoo`02Ool00`00Oomoo`0HOol003Moo`03001oogoo00=oo`03001oogoo00ioo`80
029oo`04001UI6ET0005Ool01@00NbYk:W/Z00000W/Z00<007ooOol017oo00<007ooOol00Woo00<0
07ooOol067oo000eOol20003Ool3000>Ool3000ROol20004IF@01@00Oomoogoo000017/Z00@007/Z
NbX000Moo`05001oogooOol0000JOol003Moo`04001oogoo000=Ool300000goo001oo`0OOol30000
0e<M001UI004IF@01000Oomoo`0017/Z00@007/Z000000Moo`80009oo`03001oogoo01Qoo`00=7oo
0`000goo100027oo0`008Goo0`0015<M0P001FET00<007oo000017/Z0P000VMl00<007ooOol01Goo
00@007oo000001Yoo`00@goo0`008Woo0P0000<cFP00Dad015<M0P001FET00<007oo00000g/Z0P00
16Ml00<007ooOol01Goo00<007ooOol06Woo000QOol30004Ool2000GOol200000goo001oo`0KOol9
0002<eX01@00DaeC7E<M00000VlG00<006ETIF@00fET0P000W/Z00<006Ml000016Ml0P001goo00<0
07ooOol06Goo000TOol01000Oomoo`000Woo00<007ooOol04Goo0`007goo1`0000<n>`00<IL00P00
00@cFP00Dad000A_5`03001UI6ET009UI004001k:W/Z0002Ig`01`00IgaWO6Ml001W>`0000Moo`03
001oogoo01Uoo`0097oo00@007ooOol0009oo`03001oogoo00ioo`<0025oo`D000@n>`04000aUc6G
<IL200000e<M001_5`03KaL00`00IFAUI002IF@00`00NbX00003Ig`01000Ig`000000fLk00<007oo
Ool01Goo00<007ooOol067oo000ROol20003Ool01000Oomoo`003Woo0P0000=oo`00Ool07goo1000
0VZf00<003hk?S/00chk00<0036G<IL00S6G00<006lGKaL016lG00H006ETIFAUI000NbX20002Ig`2
0006Ic/00`00Oomoo`05Ool00`00Oomoo`0GOol002Aoo`04001oogoo0002Ool00`00Oomoo`09Ool4
000POol30003Dad01@00J[IZ]VZf000013hk00<0036G<IL00S6G00<006lGKaL016lG00@006ETIFAU
I080008AVP03001WO00000MW>`8000Ioo`8001Qoo`008Goo0`0017oo0P002Woo0P008Goo0`001E<M
00<006ZfJ[H00VZf00<003hk?S/00Shk00<0036G<IL00c6G00<006lGKaL00flG0P000VET0P000a6J
0P001VLk00<006Nm00001goo00<007ooOol05Woo000aOol300000goo001oo`0MOol30002DYX00`00
DaeC7@02Dad00`00J[IZ]P03J[H00`00?S/n>`03?S/00`00<ILaU`03<IL01@00KaM_5flG00000W>J
1P0000<AVP00000016Lk0P000fNm00<007ooOol01Goo00<007ooOol05Goo000^Ool4000OOol30006
DYX01@00DaeC7E<M00001VZf0P000chk00<0036G<IL0136G00<006lG00000g>J0`0000=O6`006O`0
0`000fLk00<006NmIkd00fNm00<007ooOol01Goo00<007ooOol05Goo000]Ool2000POol200000e;K
001BVP07DYX00`00Dad00007J[H01P00:GL00000?S/000DaU`@000=cVP80009O6`<000056O`00000
Ic/00006Ikd20007Ool00`00Oomoo`0DOol002eoo`<001eoo`8000ABf`03001BVU:J00EBVP8000MZ
]P03001kM`00008YM`<00003<IL0000000@000059cP007>J001O6`030002Ga/00`006O`Io0030004
Ikd200000e/m001oo`07Ool00`00Oomoo`0COol002eoo`800003Ool007oo01Uoo`8000IBf`03001B
VU:J00ABVP03001Wo@0000IZ]P04001kMg]g0004:GL20002:GL01000?OPmn3gh1@001ElK00P001Wl
6O`Io0006O`006Nm0P0015/m00<007ooOol01Goo00<007ooOol00goo100037oo000]Ool01@00Oomo
ogoo00005Woo0`0000=CGP00D]/01U;K00<005:JDYX00U:J00D006OmIoeWo@0000EZ]P05001kMg]g
NgL00006:GL01000LoT003gh10001elK20001e/m00<007ooOol01Goo00<007ooOol00goo00<007oo
Ool037oo000]Ool00`00Oomoo`02Ool00`00Oomoo`0AOol30004Deh00`00D]]Bf`05D]/01000DYYB
VP0016Om00<006ZfJ[H00VZf00<007]gNgL00W]g00<002Ug:GL00bUg00<007?i00000P0013gh00<0
05lKGa/00elK00<006=L00000ciL00<003iL00001e/m0P001goo00<007ooOol00goo00<007ooOol0
2goo000/Ool00`00Oomoo`04Ool00`00Oomoo`0=Ool30003D]`00`00DeiCGP02Deh00`00D]]Bf`04
D]/00`00DYX00005Iod00`00J[IZ]P02J[H00`00NgMkM`02NgL00`00:GLYM`02:GL300000g?i000m
n005?OP00`00Ga]O6`020003He`01`00?U`nG3iL000nG00000AK?@800003E_`007oo00Ioo`03001o
ogoo00Aoo`03001oogoo00Yoo`001goo10000Woo00D007ooOomoo`00009oo`<000Aoo`8000moo`80
00Ioo`03001oogoo00Uoo`<000MBg0@00003Deh005;K009Bf`T0009Wo@04001Z]VZf0006NgL40005
LoT00`00?OPmn003?OP00`00Ga/00006He`02000?U`nG3iL000nG000Fcd20004E_`00`00Oomoo`05
Ool01000Oomoo`000Woo00<007ooOol02Woo0008Ool00`00Oomoo`03Ool00`00Ool00004Ool00`00
Oomoo`02Ool01000Oomoo`003Woo00<007ooOol01Woo00<007ooOol01Goo0`0000=C7@00D]`01U;L
00<006<K00000SJN2@000dYI00<002Fi00000P0000=Z]P00NgL01G]g0P0000<YM`00LoT01G?i00<0
03gh?OP00cgh0P001f=L00@003iL?U`nG0@000MFo003001oogoo00Aoo`80009oo`8000eoo`0000=o
o`0000000`000goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo00@007ooOol000io
o`8000Qoo`04001oogooOol30005Dad00`00D]aBg003D]`01@00Ha]S6f<K000013JN0P0000=Jf`00
BUT01DYI00H002Fi9KT006Zf0003NgL300000g]g000000050002LoT01000?OPmn3gh0P000Umm00<0
06=LHe`016=L100000A:o@00B_d000MFo08000Ioo`03001oogoo00moo`002Woo00L007ooOomoo`00
Ool00004Ool00`00Oomoo`02Ool01000Oomoo`003Woo00<007ooOol027oo0`000e<M00<005<MDad0
0e<M00D005;LD]aBg00000ES6`04000fWP000004F]/00`00BUU:F@03BUT010009KTU^BFi1@0000Ak
M`000000009C?P03001[>P00008Uo0H000EOO@03001SG6=L00<00003D]`004[m00=:o@03001:o@00
00AFo0800003E_`007oo00Ioo`03001oogoo00ioo`001goo00@007ooOol0009oo`05001oogooOol0
0003Ool00`00Oomoo`02Ool01000Oomoo`003Woo00<007ooOol01Woo0`001U<M00<005<MDad00e<M
00<005;L00001V<K0P001U[K00<004YIBUT014YI00<002Fi00000P0000@U^@00000000AC?P05001[
>V/jJcX000049O`20006Ggd30005D]`00`00B_e:o@02B_d01000B_d005Kl0P0015Kl00<007ooOol0
17oo00<007ooOol03Woo0008Ool2000:Ool20005Ool20008Ool30003Ool20005Ool30003Dad00`00
DaeC7@03Dad80006Ha/00`00FYP00007F]/00`00BUU:F@02BUT300029KT20002Gkh01@00DciC?U<n
00001F/j00<002Gl9O`00bGl00<005mmGgd00`0000=K?@00D]`01E;L00<004[mB_d00T[m10001UKl
00<007ooOol01Goo00<007ooOol03Goo000POol40002Ool01000Oomoogoo0`000goo0P001e<M00<0
05<M00000P000ULn00<004KMA]d00dKM00@006<KHa]S6`8000=JV003001Jfe[K00AJf`03001:F@00
00800003BUT002Fi008000EO_P03001C?P0000I[>P04000Uo2Gl9O`50005Fcd00`00D]aBg003D]`5
000014Kn0016oP001UKl0P001goo00<007ooOol037oo000VOol01000Oomoogoo1P000U<M00<005<M
Dad015<M0`001ELn00<004KMA]d00dKM00<006<K00001UZH00<005[KF]/00U[K0`000TYI1@001Unn
0P001f/j10000dJL00<005/mFcd01E/m00<005;L00000P000U/M00<004KnA_h00TKn00<004Kn0000
0eKl0P000U]n00<007ooOol017oo0P003Goo000VOol20002Ool30006Dad01000DaeC7E<M0P000eLM
00<005LnEch00eLn00@004KMA]e6g@<000QJV003001Jf`0000H00003G]T0038K00<0009O_P<00003
CY/006/j00E[>P03001S>`0000I6W003001K?E/m00AK?@8000IK7@030016oTKn0096oP800003E_`0
000000EKOP03001oogoo00Aoo`03001oogoo00]oo`009Goo0P0000Aoo`00Oomoo`<000IC7@03001C
7@0000IG7@04001G?ULnEch50002>cl00`00FYQJV005FYP50002@al20003G]T01000<Q/b6c8K0P00
1DjK00@006/jJcY[>P8000=S>`030016W4JL00A6W003001K?E/m00<00003D_d005/M00EK7@T000IK
OP8000Ioo`03001oogoo00Yoo`009Goo00@007ooOol0009oo`04001oogoo0003Dad40007Ead40004
B[`20002>cl01000FYQJV5ZH0`0000MJV000KeY_FP00@al00005G]T01@00<Q/b6c8K00001djK00<0
06/j00001V<k00@004JLAYa6W0D000EBo@04001K7E/MFad200000e]M000nW@03?Yd01000?YdnW@00
0U]n0`0000=Jo000Ool01Goo0P002goo000UOol01000Oomoo`000Woo00@007ooOomoo`<000AC7@03
001G7ELM009G7@800003Fa`004Zl00I:_0L000=JV003001_FVmJ009_FP03001NfEkI00ENf@05000b
6c8K<Q/00006CY/20007Hc/40003Dah00`00D_eBo@04D_d01000Fad0000015]M00<003jM?Yd00SjM
00<003jM00000P001E[l00<007ooOol017oo00<007ooOol02Goo000UOol01000Oomoo`000Woo00<0
07ooOol00Woo00<005<MDad015<M00@005LM000000AK7003001:_4Zl00=:_003001S6`0000<000AJ
V003001_FVmJ00=_FP03001NfEkI00ENf@03000b6c8K008b6`04001>VdjKCY/20002Ggd00`00Hc]S
>`02Hc/20002Ga`00`00DaiC7P03Dah01000D_eBoE;m1@001U]M00<003jM?Yd00`0000=;7P00F_`0
1E[l00<007ooOol01Goo00<007ooOol00Woo0`000goo000VOol20002Ool00`00Oomoo`04Ool00`00
DaeC7@03Dad30006Fa`00`00B[a:_0040003Ha/20002FYP00`00KeY_FP03KeX00`00KcT00006G]T6
00000djK00000005Ggd01000Hc/000001ElL00@005<NDaiC7PD000EC7P04001KGE]MFed50002Bah0
1000Bai;7P001E[l0P001Woo00<007ooOol01Goo0@000Goo0Goo000ZOol20006Ool00`00Dad00002
0003Ead00`00FaaK7003Fa`40005Ha/01@00KcT005ZH00001FmJ00D006liKcU_>@0000=Nf@8000=S
?003000>G000008000IOO@8000MO70@000=>g@03001C7U<N00AC7P@000=Bg003001;7T/N009;7P03
001;7P00009Jo0<0009G7@03001oogoo00Aoo`03001oogoo00Aoo`40005oo`5oo`00:Woo00<007oo
Ool01Woo0P001ULM00<005/LFa`01@001F<K00D006liKcU_>@0000E_FP03001_>Fli009_>@04001N
f@000005Hc`700001Umm00000000D[/000IO7003001O?00000I>g@03001C7U<N00<00003E_`005;L
00EBg004001;7T/NBah50006Ead00`00Oomoo`02Ool30003Ool20003Ool000Aoo`@0009oo`05001o
ogooOol00002Ool30004Ool2000@Ool20009Ool00`00EaeG7@03Ead40004D[/01@00Ha]S6f<K0000
1Fli00D006mJKeY_FP0000E_>@<000ES?003001Rf@0000<en@<000EB^`04001O75lLGa`20003Gc`0
0`00C]e>g@03C]d30005E_`00`00D]aBg003D]`40002AYd20007Ead50002Ool00`00Oomoo`03Ool1
0001Ool1Ool000Eoo`03001oogoo00=oo`03001oo`0000Aoo`03001oogoo009oo`04001oogoo000?
Ool00`00Oomoo`09Ool00`00Ead000040007D[/00`00Ha/00006KcT200000fmJ001_>@04KcT01@00
IkeW_FNm00000f<l00D006;IH]URf@0000@en@03001B^e:k00AB^`03001O700000IO?003001>g@00
00800003Dch005Kl00IFo003001Bg00000800003Feh004JM00=6W@030016W@0000AG7@800003Fcd0
07oo00Aoo`03001oogoo00<000=oo`001Woo00<007ooOol00goo00<007ooOol00goo00<007ooOol0
0Woo00@007ooOol000moo`03001oogoo00Uoo`<000ABg003001B^e:k00AB^`<000I_>@@000A_>@03
001W_FNm00=W_@03001S?00000ARf@03000enCGi00<en@03001B^e:k00=B^`8000MO?08000EC?P03
001Fo5Kl00AFo08000EKGP030016WDJM0096W@040016W@00Ead20004Fcd00`00Oomoo`02Ool00`00
Oomoo`06Ool000Moo`07001oogooOol007oo000017oo00<007ooOol00Woo00@007ooOol000moo`03
001oogoo00Ioo`<0009oo`03001Bg5;L00=Bg004001B^e:kD[/20003Fmh01000KcU_>Fli0P000UJj
100000=_>@00Ikd01FNm0P001V;I00<003Gi=OT00cGi00<005:kD[/00P000Umm00<005llGc`00Ull
0P0000=K?@00Dch01E<n00<005KlE_`00`000ULM00<005]NFeh00e]N00@004JMAYe6W@@000IK?@80
00Aoo`03001oogoo00Eoo`0017oo00@007ooOol0009oo`05001oogooOol00003Ool00`00Oomoo`02
Ool01000Oomoo`003Woo00<007ooOol017oo0`001Woo00<005;LD]`00e;L00<005:k00001U_N00<0
06li00001EJj00<0006j0KX00`000VNm0`0000=K7000H]T01F;I00<003Gi=OT00cGi00<005:k0000
1Emm00@005ll000000AK?@04001C?U<nDch50005Ead:0002B_h20004Fcd200000e/L001oo`03Ool0
0`00Oomoo`05Ool000Eoo`8000Yoo`8000Eoo`8000Qoo`<000=oo`<0009oo`<000Yoo`04001Bg5;L
D]`<0006E[X80005Fa`00`00H]URf@03H]T80006Ggd20006Fcd40003BY`00`00EaeG7@02Ead20000
0elm001>_005C[`02000B_i:oT[n001:oP00Fcd20004Fa`01@00Oomoogoo00001goo000MOol40002
Ool01P00Oomoogoo001oo`@000ioo`<000=>oP05001W>fLkIc/00003A]h00`00E[YF^P03E[X00`00
Ji`00004?U`00`00FaaK7005Fa`00`00H]URf@020003Gcd00`00?alo7`02?al00`00GgeOO@030002
Eeh01000FceK?E/m0P0000=O7000BY`01DZL00@005LM000000AO?@03001>_4jl00=>_0P000IK7005
001oogooOol00007Ool002=oo`04001oogooOol4000AOol01@00C_i>oTkn00001FLk00D004KNA]i6
gP0000=F^P05001[W6^LJi`00004?U`00`00FaaK7004Fa`00`00H]T00005Gcd80006Eeh00`00Fcd0
0004Ga`00`00BYa:W003BY`20007Gcd00`00C[a>_00200000eMN001:oP03B_h20006Fa`20002Ool0
0`00Oomoo`05Ool002=oo`05001oogooOol00003Ool00`00Oomoo`0?Ool00`00C_h00006Ic/60000
0eJj001[W004Ji`00`00?U`nG002?U`01000FaaK75/L10001Elm00D006>MHieSW@0000A37`04001G
GUMNEeh40006Ga`00`00BY`000020002Ech00`00GceO?@04Gcd00`00C[`00004Eeh00`00B_i:oP02
B_h20002Fa`300001EKl001oogoo00001goo000ROol20004Ool20002Ool00`00Oomoo`0?Ool20005
Ic/00`00G_/00003>OX30005Ji`01000?U`nG3iL1@0014KN00D005lmGceO?@0000ASW@T000=Bo@03
001O75lL00AO708000IG?P04001O?ElmGcd40005Eeh90005E_`00`00Ool00007Ool002eoo`03001o
ogoo011oo`05001W>fLkIc/00003G_/00`00>OXinP03>OX01000Jia[W6^L1@0014lN00<004KNA]h0
0dKN00<005lm00000f>M0P000U[K00<004KoA_l00TKo00<005;mD_d00e;m00<005lLGa`00`0000=F
o000Ech01ELn100014km00<005MN00000P0000=Bg000B[d00dZm0P001UKl0P0000=oo`00Ool01Goo
000^Ool2000AOol00`00Ic/00004G_/00`00>OXinP03>OX40004C[`00`00Cai?7P03Cah01000A]i6
gTKN1@0000=SW@00F]/015[K00<004KoA_l00TKo00@005;mD_eBo@@000EFo003001G?ULn00<00003
E_d004km00E>o@8000EBg003001:_DZm009:_@80009Fo0<00004Fed007oo0006Ool0031oo`03001o
ogoo00moo`03001Nnekk00ANn`07000inSWj>OX006NL00001Tjl2P0014kM0P001U[K2@000e;L00<0
05KlE_`00eKl0`001EKm00<004kmC_d00`0000=Fo000D]`01E;L00@004ZmB[e:_@@000EKG@8000Io
o`00<Goo00<007ooOol03goo00<005kkG_/00ekk00@003Wj>OX000=WW004001>_4jlC[`200000emm
001:_@05B[d01000C]e>gDkM0P0000=S6`00F]/015[K00<006<K0000149k00<005;LD]`015;L00<0
05Kl00000P000U[k00<005KmE_d00eKm0`001EKl00@005;LD]aBg0H00004A_h004Kn0005Fed20006
Ool0039oo`03001oogoo00moo`03001Nnekk009Nn`03000inP0000EWW003001>_00000AOO@03001:
_DZm00A:_@03001>g@0000AS6`05001Jfe[KF]/00003Ha/00`00@W]2N`02@W/00`00D]aBg003D]`2
0006F_/00`00E_d000020002E_`00`00E_aFo003E_`40003D_d00`00A_i6oP02A_h01000A_h005]M
0`0000=oo`00Ool01Goo000cOol00`00Oomoo`0?Ool80003Ii`40005Ggd01000B[e:_DZm10001V<K
00<005[K00001F<K00<0049k@W/00T9k00<005;L00000P0000=C7@00F_/01E[k0P001UKl00<005Kl
00000P0000=C?P00D_d01E;m00@004KnA_i6oP@000Aoo`03001oogoo00Aoo`00=7oo0P004Goo00@0
03lO?alo7`D000A>g@04001OOEmmGgd50003A]h00`00Ha]S6`04Ha/20006Ha/70005Dad01000F_]J
ne[k0P0000=SW@00E_`01EKl0P001E<n00@005;mD_eBo@H00003Ool007oo00Eoo`03001oogoo00Ao
o`00=Woo00<007ooOol03goo0`000dkn00<004kMC]d00dkM100014jl00<004KNA]h00TKN00<006<K
Ha/00`000U/L00<006<KHa/00`000U[l00<004LOAal00TLO00<005<MDad00e<M00<005[k000016>M
00<005KlE_`00`000ULn00@005<nDciC?PD000Qoo`03001oogoo00Aoo`03001oogoo00=oo`00=goo
00<007ooOol047oo00<004knC_h00Tkn00<004kM00000P0000=Nn`00C[`01Djl20001E/L0`001E[l
3P000f>M10001ELn10003Woo00<007ooOol00goo00<007ooOol00goo000hOol00`00Oomoo`0@Ool0
0`00C_i>oP040004G_/01000C[a>_4jl0P000U[l00<004;o@_l00T;o00@005/LFaaK70800003F_`0
05[l00AJo004001SG6=L0003@]h00`00C[a>_004C[`40004D]`00`00Ech00002000COol00`00Oomo
o`02Ool00`00Oomoo`03Ool003Uoo`80015oo`8000A6gP03001Nnekk009Nn`03001>_00000EJo003
0012od;o0092o`03001K700000AJo005001Jo5[lF_`00004He`01@00@]i2gT;N00000djl0P0000=O
?000D]`01E;L0P005Woo00<007ooOol00Woo00<007ooOol00goo000kOol00`00Oomoo`0@Ool00`00
A]i6gP02A]h00`00G_]Nn`040005F_`70006F_`00`00F_`00005He`600000djl001O?003Gc`01000
D]aBg5;L0P006Goo00D007ooOomoo`0000Eoo`00?7oo00<007ooOol047oo1`0015Ln00<005[l0000
0P000f=m00D003[O>]ljg`0000EJo0<000=SG0800003Gc`003:N008bWP8000IO?003001Bg00001ao
o`04001oogoo0005Ool003eoo`03001oogoo015oo`03000joc[o008jo`03001G?ULn00@000ESO@H0
0004F_`000000003Feh00`00He`00004Gc`00`00<YhbWP02<Yh01000GcaO?5ll10007Woo00<007oo
00001Goo000nOol00`00Oomoo`0AOol70004Cah00`00Hgd000020002Fa`00`00>WdjO@030005Feh2
0005Gc`9000ROol01000Oomoo`0017oo000oOol2000AOol00`00AYa6W002AY`00`00Cai?7P03Cah2
0006Fa`00`00>WdjO@02>Wd00`00Feh000020002E_`00`00Gc`00002000/Ool00`00Ool00004Ool0
045oo`03001oogoo00moo`030016W4JL0096W003001?7P000080009F^`03001K75/L00=K70L000EF
o080031oo`8000Aoo`00@Woo00<007ooOol03goo00<004JLAY`00`001UJk00<005/L00000P000eKl
00H004LoAcm7?`00E_`3000aOol30004Ool004=oo`03001oogoo00moo`80009G7@03001F^eJk00AF
^`<000EFo0H0031oo`@000Moo`00A7oo0P0047oo00D005LMEaeG7@0000=F^`8000=S?003001Fo000
008003=oo`<000]oo`00AWoo00<007ooOol03Woo00L005LMEaeG7@00E[/00005Hc`2000bOol4000>
Ool004Moo`03001oogoo00ioo`H000=S?080031oo`@0019oo`00B7oo00<007ooOol03Woo00<004KN
A]h00TKN00<006<l0000;goo0`005Woo0019Ool00`00Oomoo`0>Ool6000/Ool4000IOol004Yoo`80
00ioo`03001oogoo02]oo`@001eoo`00C7oo00<007ooOol02goo00<007ooOol0:7oo0`008Goo001=
Ool00`00Oomoo`0:Ool00`00Oomoo`0TOol4000TOol004ioo`03001oogoo00Uoo`03001oogoo021o
o`@002Qoo`00Cgoo0P002Goo00<007ooOol077oo1000;7oo001AOol00`00Oomoo`06Ool00`00Oomo
o`0IOol3000`Ool0059oo`03001oogoo00Eoo`03001oogoo01Eoo`@003=oo`00Dgoo00<007ooOol0
17oo00<007ooOol04Goo1000=goo001DOol00`00Oomoo`03Ool00`00Oomoo`0>Ool3000kOol005Eo
o`8000=oo`03001oogoo00Yoo`@003ioo`00Egoo00@007ooOol000Qoo`@0049oo`00F7oo00<007oo
00001Goo0`00AWoo001IOol200000goo000000020019Ool005Yoo`8004eoo`00ZGoo002YOol00:Uo
o`00ZGoo002YOol00:Uoo`00ZGoo002YOol00:Uoo`00ZGoo002YOol00:Uoo`00\
\>"],
  ImageRangeCache->{{{0, 168.375}, {136.25, 0}} -> {-0.319643, -0.115474, 
  0.00772905, 0.00772905}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Making Some Symbolic Checks", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Is Vega really identically zero? We can get ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to help us find out. Note first that our functions are defined by an If \
statement. We can force inspection of part of the expression as follows. \
First an example to show us the way:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(If[a > b, \ u, \ v]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(If[a > b, u, v]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FullForm[%]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[
      StyleBox[\(If[Greater[a, b], u, v]\),
        ShowSpecialCharacters->False,
        ShowStringCharacters->True],
      FullForm]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%[\([2]\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(u\)], "Output"]
}, Open  ]],

Cell["\<\
So we apply this idea to our formula for Vega - let's see it first \
in its full \"glory\":\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(UpAndOutPutVega[0, \ S, \ K, \ K, \ sd, r, \ r, \ t]\)[\([2]\)]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(E\^\(\(-r\)\ t - 1\/2\ \((\(-\(\(sd\ \@t\)\/2\)\) - Log[K\/S]\/\(sd\ \
\@t\))\)\^2\)\ K\ \((\(-\(\@t\/2\)\) + Log[K\/S]\/\(sd\^2\ \@t\))\)\)\/\@\(2\ \
\[Pi]\) - \(E\^\(\(-r\)\ t - 1\/2\ \((\(sd\ \@t\)\/2 - Log[K\/S]\/\(sd\ \
\@t\))\)\^2\)\ S\ \((\@t\/2 + Log[K\/S]\/\(sd\^2\ \@t\))\)\)\/\@\(2\ \[Pi]\) \
- \(E\^\(\(-r\)\ t - 1\/2\ \((\(-\(\(sd\ \@t\)\/2\)\) - Log[S\/K]\/\(sd\ \
\@t\))\)\^2\)\ S\ \((\(-\(\@t\/2\)\) + Log[S\/K]\/\(sd\^2\ \@t\))\)\)\/\@\(2\ \
\[Pi]\) + \(E\^\(\(-r\)\ t - 1\/2\ \((\(sd\ \@t\)\/2 - Log[S\/K]\/\(sd\ \
\@t\))\)\^2\)\ K\ \((\@t\/2 + Log[S\/K]\/\(sd\^2\ \@t\))\)\)\/\@\(2\ \
\[Pi]\)\)], "Output"]
}, Open  ]],

Cell["This is a bit of a mess, so let's simplify it:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(E\^\(\(-\(1\/8\)\)\ \((8\ r + sd\^2)\)\ t\)\ \
\((\(-E\^\(-\(Log[K\/S]\^2\/\(2\ sd\^2\ t\)\)\)\)\ \@\(K\/S\)\ S + \
E\^\(-\(Log[S\/K]\^2\/\(2\ sd\^2\ t\)\)\)\ K\ \@\(S\/K\))\)\ \@t\)\/\@\(2\ \
\[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
There are some square roots that should cancel - let's sort this \
out:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerExpand[%]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(E\^\(\(-\(1\/8\)\)\ \((8\ r + sd\^2)\)\ t\)\ \((\(-E\^\(-\(\((Log[K] \
- Log[S])\)\^2\/\(2\ sd\^2\ t\)\)\)\)\ \@K\ \@S + E\^\(-\(\((\(-Log[K]\) + \
Log[S])\)\^2\/\(2\ sd\^2\ t\)\)\)\ \@K\ \@S)\)\ \@t\)\/\@\(2\ \[Pi]\)\)], 
  "Output"]
}, Open  ]],

Cell["We are almost there:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell["\<\
So we can check that Vega is identically zero in this case. What \
about Gamma? We may as well try the sequence of steps that worked for Vega.\
\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      PowerExpand[
        Simplify[\(UpAndOutPutGamma[0, \ S, \ K, \ K, \ sd, r, \ r, \ 
              t]\)[\([2]\)]]]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell["\<\
So this particular type of option has a Vega and a Gamma which are \
identically zero!\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 8 Bibliography", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Hull, J., 1996, ",
  StyleBox["Options, Futures and Other Derivatives",
    FontSlant->"Italic"],
  ", 3rd edition, Prentice-Hall.\n\nRubenstein, M. and Reiner, E., 1991, ",
  StyleBox["Breaking down the barriers",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[",",
    FontSlant->"Italic"],
  " ",
  StyleBox["RISK Magazine",
    FontSlant->"Italic"],
  ", September."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{619, 664},
WindowMargins->{{11, Automatic}, {Automatic, 1}},
PrintingCopies->1,
PrintingStartingPageNumber->167,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"8. Barrier Options"}]]}, {Cell[ 
        TextData[ {"8. Barrier Options"}]], Inherited, Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True,
"FirstPageFace"->Right},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1.25,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8^on3ohP==PTc0b85N`?N0080004/0B`000006G<AMP01
0000I00000400@4100800BL?00400@RoSD0000000000064/01T1T00000000000
00000000007oo@S1mm3oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 239, 8, 263, "Title"],
Cell[1973, 61, 207, 6, 57, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2205, 71, 92, 1, 62, "Section"],
Cell[2300, 74, 3920, 70, 986, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6257, 149, 154, 3, 48, "Section"],

Cell[CellGroupData[{
Cell[6436, 156, 168, 3, 40, "Subsection"],
Cell[6607, 161, 360, 8, 51, "Text"],
Cell[6970, 171, 164, 3, 32, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[7159, 178, 133, 3, 32, "Input",
  InitializationCell->True],
Cell[7295, 183, 112, 2, 40, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[7456, 191, 153, 3, 40, "Subsection"],
Cell[7612, 196, 798, 28, 147, "Text"],
Cell[8413, 226, 349, 11, 37, "Text"],
Cell[8765, 239, 199, 4, 48, "Input",
  InitializationCell->True],
Cell[8967, 245, 240, 5, 57, "Input",
  InitializationCell->True],
Cell[9210, 252, 205, 5, 32, "Input",
  InitializationCell->True],
Cell[9418, 259, 257, 6, 62, "Input",
  InitializationCell->True],
Cell[9678, 267, 205, 5, 32, "Input",
  InitializationCell->True],
Cell[9886, 274, 388, 8, 168, "Input",
  InitializationCell->True],
Cell[10277, 284, 376, 7, 70, "Input",
  InitializationCell->True],
Cell[10656, 293, 403, 8, 70, "Input",
  InitializationCell->True],
Cell[11062, 303, 485, 9, 112, "Input",
  InitializationCell->True],
Cell[11550, 314, 484, 9, 112, "Input",
  InitializationCell->True],
Cell[12037, 325, 412, 8, 113, "Input",
  InitializationCell->True],
Cell[12452, 335, 449, 9, 116, "Input",
  InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[12950, 350, 159, 3, 48, "Section"],
Cell[13112, 355, 779, 24, 65, "Text"],

Cell[CellGroupData[{
Cell[13916, 383, 99, 1, 35, "Subsubsection"],
Cell[14018, 386, 432, 10, 80, "Input",
  InitializationCell->True],
Cell[14453, 398, 470, 11, 80, "Input",
  InitializationCell->True],
Cell[14926, 411, 478, 11, 92, "Input",
  InitializationCell->True],
Cell[15407, 424, 472, 11, 80, "Input",
  InitializationCell->True],
Cell[15882, 437, 471, 11, 92, "Input",
  InitializationCell->True],
Cell[16356, 450, 468, 11, 80, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[16861, 466, 97, 1, 35, "Subsubsection"],
Cell[16961, 469, 434, 11, 80, "Input",
  InitializationCell->True],
Cell[17398, 482, 471, 11, 92, "Input",
  InitializationCell->True],
Cell[17872, 495, 479, 11, 92, "Input",
  InitializationCell->True],
Cell[18354, 508, 473, 11, 92, "Input",
  InitializationCell->True],
Cell[18830, 521, 472, 11, 92, "Input",
  InitializationCell->True],
Cell[19305, 534, 469, 11, 92, "Input",
  InitializationCell->True],
Cell[19777, 547, 305, 7, 37, "Text"],

Cell[CellGroupData[{
Cell[20107, 558, 221, 5, 45, "Input"],
Cell[20331, 565, 672, 16, 77, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21040, 586, 222, 5, 45, "Input"],
Cell[21265, 593, 674, 16, 77, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21976, 614, 220, 5, 45, "Input"],
Cell[22199, 621, 675, 16, 77, "Output"]
}, Open  ]],
Cell[22889, 640, 202, 6, 25, "Text"],

Cell[CellGroupData[{
Cell[23116, 650, 300, 7, 72, "Input"],
Cell[23419, 659, 164255, 3979, 261, 127285, 3516, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[187711, 4643, 313, 7, 72, "Input"],
Cell[188027, 4652, 157605, 3919, 261, 127306, 3539, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[345669, 8576, 311, 8, 86, "Input"],
Cell[345983, 8586, 167244, 4029, 261, 127688, 3535, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[513288, 12622, 161, 3, 48, "Section"],
Cell[513452, 12627, 296, 7, 37, "Text"],

Cell[CellGroupData[{
Cell[513773, 12638, 100, 1, 35, "Subsubsection"],
Cell[513876, 12641, 430, 10, 80, "Input",
  InitializationCell->True],
Cell[514309, 12653, 467, 10, 80, "Input",
  InitializationCell->True],
Cell[514779, 12665, 475, 10, 80, "Input",
  InitializationCell->True],
Cell[515257, 12677, 469, 10, 80, "Input",
  InitializationCell->True],
Cell[515729, 12689, 468, 10, 80, "Input",
  InitializationCell->True],
Cell[516200, 12701, 465, 10, 80, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[516702, 12716, 98, 1, 35, "Subsubsection"],
Cell[516803, 12719, 430, 10, 80, "Input",
  InitializationCell->True],
Cell[517236, 12731, 468, 10, 80, "Input",
  InitializationCell->True],
Cell[517707, 12743, 450, 9, 80, "Input",
  InitializationCell->True],
Cell[518160, 12754, 444, 9, 80, "Input",
  InitializationCell->True],
Cell[518607, 12765, 470, 10, 80, "Input",
  InitializationCell->True],
Cell[519080, 12777, 466, 10, 80, "Input",
  InitializationCell->True],
Cell[519549, 12789, 320, 7, 37, "Text"],

Cell[CellGroupData[{
Cell[519894, 12800, 222, 5, 45, "Input"],
Cell[520119, 12807, 673, 16, 77, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[520829, 12828, 221, 5, 45, "Input"],
Cell[521053, 12835, 675, 16, 77, "Output"]
}, Open  ]],
Cell[521743, 12854, 202, 6, 25, "Text"],

Cell[CellGroupData[{
Cell[521970, 12864, 302, 6, 58, "Input"],
Cell[522275, 12872, 147624, 3790, 261, 125579, 3512, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[669936, 16667, 315, 7, 72, "Input"],
Cell[670254, 16676, 154772, 3880, 261, 128064, 3543, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[825063, 20561, 338, 9, 86, "Input",
  PageBreakAbove->False],
Cell[825404, 20572, 157713, 3879, 261, 126093, 3482, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[983178, 24458, 110, 1, 48, "Section"],
Cell[983291, 24461, 237, 5, 37, "Text"],

Cell[CellGroupData[{
Cell[983553, 24470, 98, 1, 35, "Subsubsection"],
Cell[983654, 24473, 409, 10, 67, "Input",
  InitializationCell->True],
Cell[984066, 24485, 443, 10, 92, "Input",
  InitializationCell->True],
Cell[984512, 24497, 451, 10, 92, "Input",
  InitializationCell->True],
Cell[984966, 24509, 444, 10, 92, "Input",
  InitializationCell->True],
Cell[985413, 24521, 445, 11, 92, "Input",
  InitializationCell->True],
Cell[985861, 24534, 440, 10, 92, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[986338, 24549, 120, 2, 35, "Subsubsection",
  PageBreakAbove->True],
Cell[986461, 24553, 433, 10, 67, "Input",
  InitializationCell->True],
Cell[986897, 24565, 471, 11, 92, "Input",
  InitializationCell->True],
Cell[987371, 24578, 479, 11, 92, "Input",
  InitializationCell->True],
Cell[987853, 24591, 473, 11, 92, "Input",
  InitializationCell->True],
Cell[988329, 24604, 472, 11, 92, "Input",
  InitializationCell->True],
Cell[988804, 24617, 469, 11, 92, "Input",
  InitializationCell->True],
Cell[989276, 24630, 307, 7, 37, "Text"],

Cell[CellGroupData[{
Cell[989608, 24641, 250, 6, 45, "Input"],
Cell[989861, 24649, 671, 16, 77, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[990569, 24670, 221, 5, 45, "Input"],
Cell[990793, 24677, 673, 16, 77, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[991503, 24698, 219, 5, 45, "Input"],
Cell[991725, 24705, 675, 16, 77, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[992461, 24728, 160, 3, 48, "Section"],
Cell[992624, 24733, 285, 7, 37, "Text"],

Cell[CellGroupData[{
Cell[992934, 24744, 99, 1, 35, "Subsubsection"],
Cell[993036, 24747, 400, 9, 80, "Input",
  InitializationCell->True],
Cell[993439, 24758, 437, 10, 92, "Input",
  InitializationCell->True],
Cell[993879, 24770, 471, 11, 92, "Input",
  InitializationCell->True],
Cell[994353, 24783, 464, 11, 92, "Input",
  InitializationCell->True],
Cell[994820, 24796, 462, 10, 80, "Input",
  InitializationCell->True],
Cell[995285, 24808, 460, 10, 80, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[995782, 24823, 97, 1, 35, "Subsubsection"],
Cell[995882, 24826, 434, 10, 80, "Input",
  InitializationCell->True],
Cell[996319, 24838, 471, 10, 80, "Input",
  InitializationCell->True],
Cell[996793, 24850, 473, 10, 80, "Input",
  InitializationCell->True],
Cell[997269, 24862, 479, 10, 80, "Input",
  InitializationCell->True],
Cell[997751, 24874, 472, 10, 80, "Input",
  InitializationCell->True],
Cell[998226, 24886, 469, 10, 80, "Input",
  InitializationCell->True],
Cell[998698, 24898, 293, 7, 37, "Text"],

Cell[CellGroupData[{
Cell[999016, 24909, 222, 6, 58, "Input"],
Cell[999241, 24917, 673, 16, 77, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[999951, 24938, 223, 6, 58, "Input"],
Cell[1000177, 24946, 403, 11, 77, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1000617, 24962, 221, 6, 58, "Input"],
Cell[1000841, 24970, 671, 16, 77, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1001573, 24993, 165, 3, 48, "Section"],

Cell[CellGroupData[{
Cell[1001763, 25000, 186, 4, 56, "Subsection"],
Cell[1001952, 25006, 771, 16, 78, "Text"],

Cell[CellGroupData[{
Cell[1002748, 25026, 207, 5, 45, "Input"],
Cell[1002958, 25033, 42962, 1093, 186, 21492, 822, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[1045935, 26129, 271, 7, 25, "Text"],

Cell[CellGroupData[{
Cell[1046231, 26140, 211, 5, 45, "Input"],
Cell[1046445, 26147, 38496, 1031, 186, 21327, 813, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[1084956, 27181, 302, 8, 25, "Text"],

Cell[CellGroupData[{
Cell[1085283, 27193, 242, 6, 45, "Input"],
Cell[1085528, 27201, 42236, 1075, 186, 21336, 810, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[1127779, 28279, 230, 6, 25, "Text"],

Cell[CellGroupData[{
Cell[1128034, 28289, 232, 6, 45, "Input"],
Cell[1128269, 28297, 93836, 2368, 186, 72845, 2103, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[1222120, 30668, 220, 6, 25, "Text"],

Cell[CellGroupData[{
Cell[1222365, 30678, 212, 5, 45, "Input"],
Cell[1222580, 30685, 43953, 1080, 186, 21065, 791, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1266570, 31770, 153, 2, 32, "Input"],
Cell[1266726, 31774, 55, 1, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1266818, 31780, 155, 2, 32, "Input"],
Cell[1266976, 31784, 60, 1, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1267073, 31790, 255, 7, 35, "Subsubsection",
  PageBreakAbove->False],
Cell[1267331, 31799, 450, 11, 51, "Text"],

Cell[CellGroupData[{
Cell[1267806, 31814, 206, 5, 45, "Input"],
Cell[1268015, 31821, 42535, 1071, 186, 21151, 801, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[1310565, 32895, 273, 7, 37, "Text"],

Cell[CellGroupData[{
Cell[1310863, 32906, 251, 6, 45, "Input"],
Cell[1311117, 32914, 38687, 1025, 181, 21054, 801, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[1349819, 33942, 373, 9, 37, "Text"],

Cell[CellGroupData[{
Cell[1350217, 33955, 211, 5, 45, "Input"],
Cell[1350431, 33962, 44678, 1075, 181, 20801, 774, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[1395124, 35040, 297, 9, 37, "Text"],

Cell[CellGroupData[{
Cell[1395446, 35053, 211, 5, 45, "Input"],
Cell[1395660, 35060, 37380, 999, 181, 21163, 793, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1433089, 36065, 109, 1, 35, "Subsubsection"],
Cell[1433201, 36068, 377, 8, 51, "Text"],

Cell[CellGroupData[{
Cell[1433603, 36080, 109, 2, 32, "Input"],
Cell[1433715, 36084, 49, 1, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1433801, 36090, 101, 2, 32, "Input"],
Cell[1433905, 36094, 172, 5, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1434114, 36104, 100, 2, 32, "Input"],
Cell[1434217, 36108, 35, 1, 45, "Output"]
}, Open  ]],
Cell[1434267, 36112, 172, 4, 27, "Text"],

Cell[CellGroupData[{
Cell[1434464, 36120, 158, 3, 32, "Input"],
Cell[1434625, 36125, 647, 9, 296, "Output"]
}, Open  ]],
Cell[1435287, 36137, 119, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[1435431, 36142, 101, 2, 25, "Input"],
Cell[1435535, 36146, 236, 4, 108, "Output"]
}, Open  ]],
Cell[1435786, 36153, 152, 4, 27, "Text"],

Cell[CellGroupData[{
Cell[1435963, 36161, 104, 2, 25, "Input"],
Cell[1436070, 36165, 256, 4, 93, "Output"]
}, Open  ]],
Cell[1436341, 36172, 93, 1, 27, "Text"],

Cell[CellGroupData[{
Cell[1436459, 36177, 101, 2, 25, "Input"],
Cell[1436563, 36181, 35, 1, 26, "Output"]
}, Open  ]],
Cell[1436613, 36185, 225, 5, 45, "Text"],

Cell[CellGroupData[{
Cell[1436863, 36194, 220, 5, 25, "Input"],
Cell[1437086, 36201, 35, 1, 26, "Output"]
}, Open  ]],
Cell[1437136, 36205, 167, 4, 27, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1437364, 36216, 98, 1, 55, "Section"],
Cell[1437465, 36219, 469, 14, 65, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

