(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    185642,       5509]*)
(*NotebookOutlinePosition[    187527,       5566]*)
(*  CellTagsIndexPosition[    187483,       5562]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 11. 
Vanilla American Options - 
Analytical Methods\
\>", "Title",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["\<\
Extending the Power of Analytical Methods to Moving Boundary Value \
Problems\
\>", "Subsubtitle",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["11.1 Introduction", "Section",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "\nIt is common to assume that there is no useful or accurate analytical \
treatment of options where early or scheduled exercise is permitted. There \
are two faults with this line of thought. Firstly, it is not correct - more \
often than not, at least with simpler linear systems, the non-existence of at \
least a good ",
  StyleBox["approximate",
    FontSlant->"Italic"],
  " analytical solution has a lot more to do with not trying hard enough to \
find it, coupled to a premature rush to purely numerical methods. Second, \
giving up on analytical models forces one to abandon the elegant methods for \
computing Greeks by ordinary differentiation. In this chapter I will present \
my current understanding of the state of play regarding the analytical \
approach to American options. I hope this understanding will be rendered \
obsolete by further efforts on this topic. I shall focus on the Put case with \
zero yield.\n\nThe general plan of this chapter is as follows. First I shall \
review one of the standard and well-known analytical approximations - the \
MacMillan, Barone-Adesi and Whaley (MBAW) models, as given by MacMillan \
(1986) and Barone-Adesi and Whaley (1987). Then I shall explore a much more \
interesting and accurate model based on relatively recent work by Carr, \
Jarrow and Myneni (1992), henceforth abbreviated as just \"CJM\". Then I \
shall offer some speculations on how these models might be related, and the \
numerical treatment of the latter improved. There are other models and \
approximations, and an excellent survey is given by CJM.\n\nThere is a host \
of general arguments relating to the early exercise of vanilla American \
options, many of which are covered in various chapters of Hull (1996). Our \
purpose here is not to regurgitate standard knowledge on the topic, but \
rather to explore analytical algorithms and their implementation. Our main \
focus will be on the problem of the American Put, but we shall also \
investigate the early exercise induced on a Call in the presence of ",
  StyleBox["discrete",
    FontSlant->"Italic"],
  " dividends. Calls and Puts with continuous dividends can be investigated \
in a similar manner to that pursued here for the Put. Later in this book we \
shall explore finite-difference and binomial models, and we shall use the \
analytical models developed here to check such numerical schemes."
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Loading the Vanilla European Package", "Subsubsection",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This chapter will assume that the standard formulae for vanilla \
European options have been loaded. \
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Derivatives`BlackScholes`\>"]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
11.2 Definition of an American Put via Moving Boundary 
Condition\
\>", "Section",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The American Put may be defined in various ways. The most sensible \
definition, at least for proper analysis, is in terms of a moving boundary \
value problem. This is much more useful than a tree-based definition, and in \
spite of being around for some time, it has been ignored by most workers. The \
matter was sorted out by McKean (1965) as follows. We have a function ",
  Cell[BoxData[
      \(TraditionalForm\`P(S, \ t)\)]],
  " satisfying"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`P(S, T) = Max[0, K - S]\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`lim\+\(S \[Rule] \[Infinity]\)\[ThinSpace]P(S, t) = 
      0\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "There is a critical stock price ",
  Cell[BoxData[
      \(TraditionalForm\`B\_t\)]],
  " such that"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`lim\+\(s \[Rule] B\_t\)\[ThinSpace]P(S, t) = 
      K - B\_t\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{\(lim\+\(S \[Rule] B\_t\)\), "\[ThinSpace]", 
          FractionBox[\(\[PartialD]\(P \((S, t)\)\)\), \(\[PartialD]S\),
            MultilineFunction->None]}], "=", \(-1\)}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and for values of ",
  Cell[BoxData[
      \(TraditionalForm\`S > B\_t\)]],
  ", the Black-Scholes differential equation is satisfied:"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`1\/2\ S\^2\ \[PartialD]\^2 P\/\[PartialD]S\^2\ \
\[Sigma]\^2 - 
        r\ P + \((r - 
              q)\)\ S\ \[PartialD]P\/\[PartialD]S + \
\[PartialD]P\/\[PartialD]t = 0\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["11.3 Definition of the Early Exercise Premium Function", "Section",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The definitions above give the full Put value. It has become commonplace, \
as it is remarkably useful, to write the problem in terms of a premium for \
early exercise. The subtraction is only explicit for values ",
  Cell[BoxData[
      \(TraditionalForm\`S > B\_t\)]],
  ", and here one writes"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`P = P\_E + \[Rho]\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`P\_E\)]],
  " is the vanilla European Put value. In terms of the premium variable ",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho]\)]],
  ", we have the new problem:"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Rho](S, T) = 0\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`lim\+\(S \[Rule] \[Infinity]\)\[ThinSpace]\[Rho](S, 
          t) = 0\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "At the critical stock price ",
  Cell[BoxData[
      \(TraditionalForm\`B\_t\)]],
  ", we now have"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`lim\+\(s \[Rule] B\_t\)\[ThinSpace]\[Rho](S, t) = 
      K - B\_t - \(P\_E\)(B\_t, t)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{\(lim\+\(S \[Rule] B\_t\)\), "\[ThinSpace]", 
          FractionBox[\(\[PartialD]\(\[Rho] \((S, t)\)\)\), \(\[PartialD]S\),
            MultilineFunction->None]}], "=", 
        RowBox[{\(-1\), "-", " ", 
          
          FractionBox[\(\[PartialD]\(\(P\_E\) \((B\_t, 
                  t)\)\)\), \(\[PartialD]S\),
            MultilineFunction->None]}]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and for values of ",
  Cell[BoxData[
      \(TraditionalForm\`S > B\_t\)]],
  ", the Black-Scholes differential equation is still satisfied:"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`1\/2\ S\^2\ \[PartialD]\^2 \[Rho]\/\[PartialD]S\^2\ \
\[Sigma]\^2 - 
        r\ \[Rho] + \((r - 
              q)\)\ S\ \[PartialD]\[Rho]\/\[PartialD]S + \[PartialD]\[Rho]\/\
\[PartialD]t = 0\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
So we have payoff conditions of zero, at the price of more \
complicated moving boundary conditions.\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["11.4 The Quasi-stationary Method of Solution and MBAW", "Section",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
There is a standard method for obtaining approximate solutions to \
differential equations of this type, called the \"quasi-stationary method\". \
This, and other, methods of treatment are very well described by Crank (1984) \
in his classic text. In its raw form, we would first seek a first solution \
where we can ignore the time-dependence, solving\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`1\/2\ S\^2\ \[PartialD]\^2 \[Rho]\/\[PartialD]S\^2\ \
\[Sigma]\^2 - r\ \[Rho] + \((r - q)\)\ S\ \[PartialD]\[Rho]\/\[PartialD]S = 
      0\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Note that the \"quadratic approximation\", of MBAW, as described, \
for example, by Hull (1996), is nothing other than the quasi-stationary \
approximation after the application of a further change of variables to make \
the method more accurate. The change of variable developed is given by\
\>", 
  "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Tau] = \(T - 
          t\ \ \ \ \ \ h\ \((\[Tau])\) = \(1 - \[ExponentialE]\^\(\(-r\)\ \
\[Tau]\)\ \ \ \ \ \ \ k\_1 = \(\(2\ r\)\/\[Sigma]\^2\ \ \ \ \ k\_2 = \(2\ \
\((r - q)\)\)\/\[Sigma]\^2\)\)\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["and setting", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Rho] = \(h(\[Tau])\)\ \(\[CapitalPsi](S, h)\)\)], 
  "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["leads to", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{" ", 
        RowBox[{
          RowBox[{
            RowBox[{\(S\^2\), " ", 
              
              FractionBox[\(\[PartialD]\^2 \[CapitalPsi]\),
                 \(\[PartialD]S\^2\),
                MultilineFunction->None]}], "+", 
            RowBox[{\(k\_2\), " ", "S", 
              FractionBox[\(\[PartialD]\[CapitalPsi]\), \(\[PartialD]S\),
                MultilineFunction->None]}], " ", 
            "-", \(\(k\_1\ \[CapitalPsi]\)\/h\), "-", 
            RowBox[{\((1 - h)\), " ", \(k\_1\), " ", 
              FractionBox[\(\[PartialD]\[CapitalPsi]\), \(\[PartialD]h\),
                MultilineFunction->None]}]}], "=", "0"}]}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["with quasi-stationary approximation", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{" ", 
        RowBox[{
          RowBox[{
            RowBox[{\(S\^2\), " ", 
              
              FractionBox[\(\[PartialD]\^2 \[CapitalPsi]\),
                 \(\[PartialD]S\^2\),
                MultilineFunction->None]}], "+", 
            RowBox[{\(k\_2\), " ", "S", 
              FractionBox[\(\[PartialD]\[CapitalPsi]\), \(\[PartialD]S\),
                MultilineFunction->None]}], " ", 
            "-", \(\(k\_1\ \[CapitalPsi]\)\/h\)}], "=", "0"}]}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This is an equation of homogeneous type, which is easily solved in terms \
of a power of ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  ". Only one solution satisfies the requirement that the result tends to \
zero for large ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  ". The answer may be written as"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[CapitalPsi] = A\_1\ \((S\/B\_t)\)\^\[Gamma]\_1\)], 
  "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["where ", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Gamma]\_1 = 
      1\/2\ \((1 - k\_2 - \@\(\((1 - k\_2)\)\^2 + \(4\ k\_1\)\/h\))\)\)], 
  "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The values of ",
  Cell[BoxData[
      \(TraditionalForm\`A\_1\)]],
  " and the critical price ",
  Cell[BoxData[
      \(TraditionalForm\`B\_t\)]],
  " are now determined by imposing the boundary conditions at the critical \
price. ",
  Cell[BoxData[
      \(TraditionalForm\`A\_1\)]],
  " is eliminated first using continuity of the derivative at the critical \
price. Then the other boundary condition is imposed. This leads to an \
implicit non-linear equation for ",
  Cell[BoxData[
      \(TraditionalForm\`B\_t\)]],
  ". To do the first step, let ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\)]],
  " be the Delta for the ordinary European Put. Then continuity of the \
derivative establishes that"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`A\_1 = \(-\(\(B\_t\ \((\[CapitalDelta] + 
                  1)\)\)\/\[Gamma]\_1\)\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The continuity of the value at the free boundary is then given by \
the equation\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`K - P(B\_t, t) - 
        B\_t = \(-\(\(B\_t\ \((\[CapitalDelta] + 1)\)\)\/\[Gamma]\_1\)\)\)], 
  "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This can now be solved in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["11.5 Mathematica Implementation of the MBAW Method", "Section",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Once the standard European option Package has been loaded, we can \
make use of some standard Put functions. Let's remind ourselves of their \
operation.\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?BlackScholesPut\)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("BlackScholesPut[price, strike, vol, riskfree, divyield, expiry]  \
returns the Black-Scholes value of a vanilla European Put."\)], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?BlackScholesPutDelta\)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("BlackScholesPutDelta[price, strike, vol, riskfree, divyield, expiry]  \
returns the delta of a vanilla European Put."\)], "Print"]
}, Open  ]],

Cell["\<\
Now we define the functions in the MBAW analysis. We begin with a \
clean-up:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Clear[r, q, t, \[Sigma], K]; \[IndentingNewLine]h[r_, t_] = 
      1 - Exp[\(-r\)\ t]; \nk\_1[r_, \[Sigma]_] := \(2\ r\)\/\[Sigma]\^2; \n
    k\_2[r_, \[Sigma]_, q_] := \(2\ \((r - q)\)\)\/\[Sigma]\^2; \n\[Gamma]\_1[
        r_, \ \[Sigma]_, \ t_, \ q_]\  := \ 
      1/2\ \((1\  - \ k\_2[r, \[Sigma], q]\  - \ 
            Sqrt[\((1 - k\_2[r, \[Sigma], q])\)^2\  + \ 
                4\ k\_1[r, \[Sigma]]/h[r, t]])\)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Just to check, we enter these expressions and ask for ",
  Cell[BoxData[
      \(TraditionalForm\`\[Gamma]\_1\)]],
  "in ordinary mathematical notation:"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm[\[Gamma]\_1[r, s, t, q]]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`1\/2\ \((\(-\(\(2\ \((r - 
                      q)\)\)\/s\^2\)\) - \@\(\((1 - \(2\ \((r - q)\)\)\/s\^2)\
\)\^2 + \(8\ r\)\/\(\((1 - \[ExponentialE]\^\(\(-r\)\ t\))\)\ s\^2\)\) + 
          1)\)\)], "Output"]
}, Open  ]],

Cell["\<\
The boundary condition governing the location of the critical price \
is equivalent to setting the following function to zero, where we use Sf to \
denote the critical price:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Msolver[Sf_, \ \[Sigma]_, \ r_, \ t_, \ K_, \ q_]\  = \n
      Simplify[K\  - \ Sf\  - \ 
          BlackScholesPut[Sf, \ K, \ \[Sigma], \ r, \ q, \ 
            t] + \(Sf\ \((BlackScholesPutDelta[Sf, \ K, \ \[Sigma], \ r, \ q, \
\ t] + 1)\)\)\/\[Gamma]\_1[r, \[Sigma], t, q]]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(K - Sf - 
      1\/2\ E\^\(\(-r\)\ t\)\ K\ \((1 + 
            Erf[\(t\ \((2\ q - 2\ r + \[Sigma]\^2)\) - 2\ Log[Sf\/K]\)\/\(2\ \
\@2\ \@t\ \[Sigma]\)])\) - 
      1\/2\ E\^\(\(-q\)\ t\)\ Sf\ \((\(-1\) + 
            Erf[\(t\ \((\(-2\)\ q + 2\ r + \[Sigma]\^2)\) + 2\ Log[Sf\/K]\)\/\
\(2\ \@2\ \@t\ \[Sigma]\)])\) + \(E\^\(\(-q\)\ t\)\ Sf\ \((\(-1\) + 2\ E\^\(q\
\ t\) + Erf[\(t\ \((\(-2\)\ q + 2\ r + \[Sigma]\^2)\) + 2\ Log[Sf\/K]\)\/\(2\ \
\@2\ \@t\ \[Sigma]\)])\)\)\/\(1 - \@\(\((1 + \(2\ \((q - r)\)\)\/\[Sigma]\^2)\
\)\^2 + \(8\ r\)\/\(\((1 - E\^\(\(-r\)\ t\))\)\ \[Sigma]\^2\)\) + \(2\ \((q - \
r)\)\)\/\[Sigma]\^2\)\)], "Output"]
}, Open  ]],

Cell["\<\
Now we can define two functions, one for the critical price, the \
other for the value of the option. Let's take a look at the front first of \
all:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(MMFront[K_, \ t_, \ \[Sigma]_, \ r_, \ q_]\  := \n\t
      Module[{Sf, \ v}, \n\t\tSf\  = \ 
          v /. \ \(FindRoot[
                Msolver[v, \ \[Sigma], \ r, \ t, \ K, \ q] == 
                  0, \n\t\t\t\t\t{v, \ K - 1}, \ 
                MaxIterations\  -> \ 40]\)[\([1]\)]; \n\t\tSf]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We can make a table of values, and plot it, for zero dividend \
yield:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(mmfrontdata\  = \ 
      Table[{t, \ MMFront[50, \ t, \ 0.4, \ 0.1, \ 0]}, \ {t, \ 0.0001, \ 
          20, \ 0.2}]; \)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(mbawplot\  = \ 
      ListPlot[mmfrontdata, \ PlotJoined\  -> \ True, \ 
        PlotRange\  -> \ {20, \ 50}]; \)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0480998 -0.412023 0.0206011 [
[.26431 -0.0125 -3 -9 ]
[.26431 -0.0125 3 0 ]
[.50481 -0.0125 -6 -9 ]
[.50481 -0.0125 6 0 ]
[.74531 -0.0125 -6 -9 ]
[.74531 -0.0125 6 0 ]
[.98581 -0.0125 -6 -9 ]
[.98581 -0.0125 6 0 ]
[.01131 .10301 -12 -4.5 ]
[.01131 .10301 0 4.5 ]
[.01131 .20601 -12 -4.5 ]
[.01131 .20601 0 4.5 ]
[.01131 .30902 -12 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .41202 -12 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .51503 -12 -4.5 ]
[.01131 .51503 0 4.5 ]
[.01131 .61803 -12 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.26431 0 m
.26431 .00625 L
s
[(5)] .26431 -0.0125 0 1 Mshowa
.50481 0 m
.50481 .00625 L
s
[(10)] .50481 -0.0125 0 1 Mshowa
.74531 0 m
.74531 .00625 L
s
[(15)] .74531 -0.0125 0 1 Mshowa
.98581 0 m
.98581 .00625 L
s
[(20)] .98581 -0.0125 0 1 Mshowa
.125 Mabswid
.07191 0 m
.07191 .00375 L
s
.12001 0 m
.12001 .00375 L
s
.16811 0 m
.16811 .00375 L
s
.21621 0 m
.21621 .00375 L
s
.31241 0 m
.31241 .00375 L
s
.36051 0 m
.36051 .00375 L
s
.40861 0 m
.40861 .00375 L
s
.45671 0 m
.45671 .00375 L
s
.55291 0 m
.55291 .00375 L
s
.60101 0 m
.60101 .00375 L
s
.64911 0 m
.64911 .00375 L
s
.69721 0 m
.69721 .00375 L
s
.79341 0 m
.79341 .00375 L
s
.84151 0 m
.84151 .00375 L
s
.88961 0 m
.88961 .00375 L
s
.93771 0 m
.93771 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .10301 m
.03006 .10301 L
s
[(25)] .01131 .10301 1 0 Mshowa
.02381 .20601 m
.03006 .20601 L
s
[(30)] .01131 .20601 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(35)] .01131 .30902 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(40)] .01131 .41202 1 0 Mshowa
.02381 .51503 m
.03006 .51503 L
s
[(45)] .01131 .51503 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(50)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0206 m
.02756 .0206 L
s
.02381 .0412 m
.02756 .0412 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .0824 m
.02756 .0824 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .16481 m
.02756 .16481 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .28842 m
.02756 .28842 L
s
.02381 .32962 m
.02756 .32962 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .45322 m
.02756 .45322 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .53563 m
.02756 .53563 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57683 m
.02756 .57683 L
s
.02381 .59743 m
.02756 .59743 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60653 m
.03343 .39513 L
.04305 .34805 L
.05267 .31934 L
.06229 .29887 L
.07191 .28313 L
.08153 .27048 L
.09115 .25999 L
.10077 .2511 L
.11039 .24344 L
.12001 .23675 L
.12963 .23085 L
.13925 .22561 L
.14887 .22091 L
.15849 .21667 L
.16811 .21283 L
.17773 .20934 L
.18735 .20615 L
.19697 .20322 L
.20659 .20052 L
.21621 .19803 L
.22583 .19573 L
.23545 .1936 L
.24507 .19162 L
.25469 .18978 L
.26431 .18806 L
.27393 .18645 L
.28355 .18495 L
.29317 .18355 L
.30279 .18223 L
.31241 .18099 L
.32203 .17983 L
.33165 .17874 L
.34127 .17771 L
.35089 .17674 L
.36051 .17583 L
.37013 .17497 L
.37975 .17416 L
.38937 .17339 L
.39899 .17267 L
.40861 .17198 L
.41823 .17133 L
.42785 .17072 L
.43747 .17014 L
.44709 .16959 L
.45671 .16906 L
.46633 .16857 L
.47595 .1681 L
.48557 .16766 L
.49519 .16723 L
Mistroke
.50481 .16683 L
.51443 .16645 L
.52405 .16609 L
.53367 .16575 L
.54329 .16542 L
.55291 .16512 L
.56253 .16482 L
.57215 .16454 L
.58177 .16428 L
.59139 .16403 L
.60101 .16379 L
.61063 .16356 L
.62025 .16334 L
.62987 .16314 L
.63949 .16294 L
.64911 .16276 L
.65873 .16258 L
.66835 .16241 L
.67797 .16226 L
.68759 .1621 L
.69721 .16196 L
.70683 .16183 L
.71645 .1617 L
.72607 .16157 L
.73569 .16146 L
.74531 .16135 L
.75493 .16124 L
.76455 .16114 L
.77417 .16105 L
.78379 .16096 L
.79341 .16087 L
.80303 .16079 L
.81265 .16072 L
.82227 .16064 L
.83189 .16058 L
.84151 .16051 L
.85113 .16045 L
.86075 .16039 L
.87037 .16034 L
.87999 .16029 L
.88961 .16024 L
.89923 .16019 L
.90885 .16015 L
.91847 .16011 L
.92809 .16007 L
.93771 .16003 L
.94733 .16 L
.95695 .15997 L
.96657 .15994 L
.97619 .15991 L
Mistroke
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{179.5, 110.812},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002c0000KQ000`40O003h00O/goo002cOol003Io
o`<001moo`<000Aoo`8001]oo`<000=oo`<001aoo`@000=oo`8000Uoo`00>Goo00<007ooOol07Goo
00<007ooOol00Woo00@007ooOol001]oo`03001oogoo00Eoo`03001oogoo01Yoo`03001oogoo009o
o`04001oogoo0008Ool003Uoo`03001oogoo01eoo`03001oogoo009oo`04001oogoo000KOol00`00
Oomoo`05Ool00`00Oomoo`0KOol01@00Oomoogoo00000Woo00<007ooOol01Woo000fOol3000POol0
0`00Oomoo`02Ool01000Oomoo`006goo00<007ooOol00Woo0`007goo00@007ooOol0009oo`03001o
ogoo00Ioo`00=Woo00<007ooOol087oo00<007ooOol00Woo00@007ooOol001]oo`03001oogoo009o
o`03001oogoo01aoo`04001oogoo0002Ool01000Oomoo`0027oo000fOol4000NOol20005Ool2000K
Ool20004Ool4000LOol20004Ool20009Ool00;=oo`00/goo002cOol0011ooiT000Yoo`004goo00<0
07ooOol08Woo00<007ooOol08Woo00<007ooOol08Goo00<007ooOol08Woo00<007ooOol02Woo000C
Ool00`00Oomoo`2MOol001=oo`8009ioo`004goo00<007ooOol0WGoo000COol00`00Oomoo`2MOol0
01=oo`8009ioo`004goo00<007ooOol0WGoo000COol00`00Oomoo`2MOol001=oo`8009ioo`004goo
00<007ooOol0WGoo000COol00`00Oomoo`2MOol001=oo`8009ioo`001Woo10000Woo0`0017oo00<0
07ooOol0WGoo0007Ool00`00Oomoo`05Ool01@00Oomoogoo0000Wgoo0008Ool00`00Oomoo`04Ool0
1000Oomoogoo0P00WWoo0009Ool00`00Oomoo`030004Ool00`00Oomoo`2MOol000Ioo`04001oogoo
0002Ool00`00Oomoo`04Ool00`00Oomoo`2MOol000Moo`8000=oo`@000=oo`8009ioo`004goo00<0
07ooOol0WGoo000COol00`00Oomoo`2MOol001=oo`03001oogoo09eoo`004goo0P00WWoo000COol0
0`00Oomoo`2MOol001=oo`03001oogoo05]oocD000eoo`004goo0P00?Woo7P00@Woo000COol00`00
Oomoo`0`Ool=001POol001=oo`03001oogoo02Moo`T006eoo`004goo0P008goo1@00MWoo0006Ool3
0004Ool20004Ool00`00Oomoo`0LOol6001kOol000Uoo`04001oogoo0002Ool01@00Oomoogoo0000
6goo0`00PGoo0009Ool01000Oomoo`000Woo00@007ooOomoo`8001Moo`<008Aoo`001goo0P000goo
00@007ooOol000=oo`03001oogoo01=oo`<008Moo`002Goo00@007ooOol0009oo`05001oogooOol0
000BOol3002:Ool000Ioo`<000Aoo`8000Aoo`8000moo`8008eoo`004goo00<007ooOol03Goo00<0
07ooOol0SGoo000COol00`00Oomoo`0;Ool2002@Ool001=oo`8000]oo`03001oogoo091oo`004goo
00<007ooOol02Goo00<007ooOol0TGoo000COol00`00Oomoo`08Ool00`00Oomoo`2BOol001=oo`03
001oogoo00Moo`03001oogoo09=oo`004goo0P001goo00<007ooOol0U7oo000COol00`00Oomoo`06
Ool00`00Oomoo`2DOol001=oo`03001oogoo00Eoo`03001oogoo09Eoo`004goo0P001Goo00<007oo
Ool0UWoo0006Ool30003Ool30004Ool00`00Oomoo`03Ool00`00Oomoo`2GOol000Uoo`03001oogoo
00=oo`05001oogooOol00005Ool00`00Oomoo`2GOol000Uoo`03001oogoo00=oo`04001oogooOol2
0003Ool00`00Oomoo`2HOol000Moo`8000=oo`<000Aoo`03001oogoo009oo`03001oogoo09Qoo`00
2Goo00@007ooOol000Ioo`03001oogoo009oo`03001oogoo09Qoo`001Woo0`000goo10000goo0P00
0Woo00<007ooOol0VGoo000COol01@00Oomoogoo0000Vgoo000COol01@00Oomoogoo0000Vgoo000C
Ool200000goo001oo`2KOol001=oo`04001oogoo002LOol001=oo`04001oogoo002LOol001=oo`80
0003Ool007oo09]oo`004goo00<007oo0000WGoo000COol00`00Ool0002MOol001=oo`<009eoo`00
4goo00<007oo0000WGoo0007Ool30003Ool20004Ool00`00Ool0002MOol000Qoo`05001oogooOol0
0002Ool01`00Oomoogoo001oo`0009eoo`001Woo10000Woo00@007ooOol000=oo`<009eoo`001Woo
00<007oo00000goo00@007ooOol000=oo`03001oo`0009eoo`001Woo00<007oo00000goo00@007oo
Ool000=oo`03001oo`0009eoo`001goo0P0017oo0P0017oo0`00WGoo000COol00`00Ool0002MOol0
01=oo`03001oo`0009eoo`004goo0P00WWoo000COol2002NOol001=oo`8009ioo`004goo0P00WWoo
000COol2002NOol001=oo`8009ioo`004goo0P00WWoo0007Ool30002Ool30004Ool2002NOol000Qo
o`03001oogoo00Aoo`04001oogooOol2002NOol000Ioo`@000Eoo`04001oogooOol2002NOol000Io
o`03001oo`0000=oo`<000Aoo`8009ioo`001Woo00<007oo00000goo00<007ooOol017oo0P00WWoo
0007Ool20003Ool40003Ool2002NOol001=oo`8009ioo`004goo0P00WWoo000COol2002NOol001=o
o`8009ioo`004goo00<007ooOol0WGoo000COol00`00Oomoo`2MOol001=oo`8009ioo`004goo00<0
07ooOol0WGoo000COol00`00Oomoo`2MOol001=oo`8009ioo`001Woo0`0017oo0P0017oo00<007oo
Ool0WGoo0009Ool01000Oomoo`000Woo00D007ooOomoo`0009moo`002Goo00@007ooOol0009oo`04
001oogooOol2002NOol000Ioo`<000=oo`04001oogoo002SOol000Ioo`03001oogoo00=oo`04001o
ogoo002SOol000Ioo`@000=oo`800:Aoo`00/goo0000\
\>"],
  ImageRangeCache->{{{0, 178.5}, {109.812, 0}} -> {-2.68586, 16.533, 
  0.136108, 0.317788}}]
}, Open  ]],

Cell[TextData[{
  "The function appears to level off. If we analyse the function ",
  StyleBox["Msolver",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " for very late times, and for ",
  Cell[BoxData[
      \(TraditionalForm\`q\)]],
  " zero, it is easy to find the late-time asymptote. It is given by (see \
Section 4.2 for another derivation)"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`B\_\[Infinity] = K\/\(1 + \[Sigma]\^2\/\(2\ r\)\)\)], 
  "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "or, in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", by"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(FrontLate[K_, r_, \[Sigma]_, t_] := 
      K\/\(1 + \[Sigma]\^2\/\(2\ r\)\)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We can overlay a plot of the late-time asymptote with the front, as \
follows:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(asymptote\  = \ 
      Plot[FrontLate[50, \ 0.1, \ 0.4, \ t], \ {t, \ 0, \ 20}, \ 
        PlotRange\  -> \ {20, \ 50}, \ 
        DisplayFunction\  -> \ Identity]; \)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[mbawplot, \ asymptote]; \)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.047619 -0.412023 0.0206011 [
[.2619 -0.0125 -3 -9 ]
[.2619 -0.0125 3 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.7381 -0.0125 -6 -9 ]
[.7381 -0.0125 6 0 ]
[.97619 -0.0125 -6 -9 ]
[.97619 -0.0125 6 0 ]
[.01131 .10301 -12 -4.5 ]
[.01131 .10301 0 4.5 ]
[.01131 .20601 -12 -4.5 ]
[.01131 .20601 0 4.5 ]
[.01131 .30902 -12 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .41202 -12 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .51503 -12 -4.5 ]
[.01131 .51503 0 4.5 ]
[.01131 .61803 -12 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 0 m
.2619 .00625 L
s
[(5)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(10)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(15)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(20)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .10301 m
.03006 .10301 L
s
[(25)] .01131 .10301 1 0 Mshowa
.02381 .20601 m
.03006 .20601 L
s
[(30)] .01131 .20601 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(35)] .01131 .30902 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(40)] .01131 .41202 1 0 Mshowa
.02381 .51503 m
.03006 .51503 L
s
[(45)] .01131 .51503 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(50)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0206 m
.02756 .0206 L
s
.02381 .0412 m
.02756 .0412 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .0824 m
.02756 .0824 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .16481 m
.02756 .16481 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .28842 m
.02756 .28842 L
s
.02381 .32962 m
.02756 .32962 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .45322 m
.02756 .45322 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .53563 m
.02756 .53563 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57683 m
.02756 .57683 L
s
.02381 .59743 m
.02756 .59743 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60653 m
.03334 .39513 L
.04286 .34805 L
.05239 .31934 L
.06191 .29887 L
.07143 .28313 L
.08096 .27048 L
.09048 .25999 L
.1 .2511 L
.10953 .24344 L
.11905 .23675 L
.12858 .23085 L
.1381 .22561 L
.14762 .22091 L
.15715 .21667 L
.16667 .21283 L
.1762 .20934 L
.18572 .20615 L
.19524 .20322 L
.20477 .20052 L
.21429 .19803 L
.22381 .19573 L
.23334 .1936 L
.24286 .19162 L
.25239 .18978 L
.26191 .18806 L
.27143 .18645 L
.28096 .18495 L
.29048 .18355 L
.3 .18223 L
.30953 .18099 L
.31905 .17983 L
.32858 .17874 L
.3381 .17771 L
.34762 .17674 L
.35715 .17583 L
.36667 .17497 L
.3762 .17416 L
.38572 .17339 L
.39524 .17267 L
.40477 .17198 L
.41429 .17133 L
.42381 .17072 L
.43334 .17014 L
.44286 .16959 L
.45239 .16906 L
.46191 .16857 L
.47143 .1681 L
.48096 .16766 L
.49048 .16723 L
Mistroke
.5 .16683 L
.50953 .16645 L
.51905 .16609 L
.52858 .16575 L
.5381 .16542 L
.54762 .16512 L
.55715 .16482 L
.56667 .16454 L
.5762 .16428 L
.58572 .16403 L
.59524 .16379 L
.60477 .16356 L
.61429 .16334 L
.62381 .16314 L
.63334 .16294 L
.64286 .16276 L
.65239 .16258 L
.66191 .16241 L
.67143 .16226 L
.68096 .1621 L
.69048 .16196 L
.7 .16183 L
.70953 .1617 L
.71905 .16157 L
.72858 .16146 L
.7381 .16135 L
.74762 .16124 L
.75715 .16114 L
.76667 .16105 L
.7762 .16096 L
.78572 .16087 L
.79524 .16079 L
.80477 .16072 L
.81429 .16064 L
.82381 .16058 L
.83334 .16051 L
.84286 .16045 L
.85239 .16039 L
.86191 .16034 L
.87143 .16029 L
.88096 .16024 L
.89048 .16019 L
.9 .16015 L
.90953 .16011 L
.91905 .16007 L
.92858 .16003 L
.9381 .16 L
.94762 .15997 L
.95715 .15994 L
.96667 .15991 L
Mistroke
Mfstroke
.02381 .16023 m
.06244 .16023 L
.10458 .16023 L
.14415 .16023 L
.18221 .16023 L
.22272 .16023 L
.26171 .16023 L
.30316 .16023 L
.34309 .16023 L
.3815 .16023 L
.42237 .16023 L
.46172 .16023 L
.49955 .16023 L
.53984 .16023 L
.57861 .16023 L
.61984 .16023 L
.65954 .16023 L
.69774 .16023 L
.73838 .16023 L
.77751 .16023 L
.81909 .16023 L
.85916 .16023 L
.89771 .16023 L
.93871 .16023 L
.97619 .16023 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  ImageSize->{179.5, 110.812},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002c0000KQ000`40O003h00O/goo002cOol003Io
o`<001moo`<000Aoo`8001]oo`<000=oo`<001]oo`@000=oo`8000Yoo`00>Goo00<007ooOol07Goo
00<007ooOol00Woo00@007ooOol001]oo`03001oogoo00Eoo`03001oogoo01Uoo`03001oogoo009o
o`04001oogoo0009Ool003Uoo`03001oogoo01eoo`03001oogoo009oo`04001oogoo000KOol00`00
Oomoo`05Ool00`00Oomoo`0JOol01@00Oomoogoo00000Woo00<007ooOol01goo000fOol3000POol0
0`00Oomoo`02Ool01000Oomoo`006goo00<007ooOol00Woo0`007Woo00@007ooOol0009oo`03001o
ogoo00Moo`00=Woo00<007ooOol087oo00<007ooOol00Woo00@007ooOol001]oo`03001oogoo009o
o`03001oogoo01]oo`04001oogoo0002Ool01000Oomoo`002Goo000fOol4000NOol20005Ool2000K
Ool20004Ool4000KOol20004Ool2000:Ool00;=oo`00/goo002cOol0011ooiX000Uoo`0057oo00<0
07ooOol08Goo00<007ooOol08Woo00<007ooOol08Goo00<007ooOol08Goo00<007ooOol02goo000D
Ool00`00Oomoo`2LOol001Aoo`8009eoo`0057oo00<007ooOol0W7oo000DOol00`00Oomoo`2LOol0
01Aoo`8009eoo`0057oo00<007ooOol0W7oo000DOol00`00Oomoo`2LOol001Aoo`8009eoo`0057oo
00<007ooOol0W7oo000DOol00`00Oomoo`2LOol001Aoo`8009eoo`001goo10000Woo0`0017oo00<0
07ooOol0W7oo0008Ool00`00Oomoo`05Ool01@00Oomoogoo0000WWoo0009Ool00`00Oomoo`04Ool0
1000Oomoogoo0P00WGoo000:Ool00`00Oomoo`030004Ool00`00Oomoo`2LOol000Moo`04001oogoo
0002Ool00`00Oomoo`04Ool00`00Oomoo`2LOol000Qoo`8000=oo`@000=oo`8009eoo`0057oo00<0
07ooOol0W7oo000DOol00`00Oomoo`2LOol001Aoo`03001oogoo09aoo`0057oo0P00WGoo000DOol0
0`00Oomoo`2LOol001Aooi8000eoo`0057oo0P00?Woo7@00@Woo000DOol00`00Oomoo`0_Ool>001O
Ool001Aoo`03001oogoo02Moo`P006eoo`0057oo0P008Woo1P00MGoo0007Ool30004Ool20004Ool0
0`00Oomoo`0KOol6001kOol000Yoo`04001oogoo0002Ool01@00Oomoogoo00006Woo0`00PGoo000:
Ool01000Oomoo`000Woo00@007ooOomoo`8001Ioo`<008Aoo`0027oo0P000goo00@007ooOol000=o
o`03001oogoo019oo`<008Moo`002Woo00@007ooOol0009oo`05001oogooOol0000AOol3002:Ool0
00Moo`<000Aoo`8000Aoo`8000moo`03001oogoo08]oo`0057oo00<007ooOol03Goo00<007ooOol0
S7oo000DOol00`00Oomoo`0;Ool2002?Ool001Aoo`8000]oo`03001oogoo08moo`0057oo00<007oo
Ool027oo0P00TWoo000DOol00`00Oomoo`07Ool00`00Oomoo`2BOol001Aoo`03001oogoo00Moo`03
001oogoo099oo`0057oo0P001goo00<007ooOol0Tgoo000DOol00`00Oomoo`05Ool00`00Oomoo`2D
Ool001Aoo`03001oogoo00Aoo`03001oogoo09Eoo`0057oo0P001Goo00<007ooOol0UGoo0007Ool3
0003Ool30004Ool00`00Oomoo`03Ool00`00Oomoo`2FOol000Yoo`03001oogoo00=oo`05001oogoo
Ool00005Ool00`00Oomoo`2FOol000Yoo`03001oogoo00=oo`04001oogooOol20003Ool00`00Oomo
o`2GOol000Qoo`8000=oo`<000Aoo`05001oogooOol0002JOol000Yoo`04001oogoo0006Ool01@00
Oomoogoo0000VWoo0007Ool30003Ool40003Ool20002Ool00`00Oomoo`2HOol001Aoo`04001oogoo
002KOol001Aoo`04001oogoo002KOol001Aoo`800003Ool007oo09Yoo`0057oo00@007ooOol009]o
o`0057oo00@007ooOol009]oo`0057oo0`00W7oo000DOol00`00Ool0002LOol001Aoo`03001oo`00
09aoo`0057oo0P00WGoo000DOol2002MOol000Qoo`<000=oo`8000Aoo`8009eoo`002Goo00D007oo
Oomoo`00009oo`04001oogooOol2002MOol000Moo`@0009oo`04001oogoo0003Ool2002MOol000Mo
o`03001oo`0000=oo`04001oogoo0003Ool2002MOol000Moo`03001oo`0000=oo`04001oogoo0003
Ool2002MOol000Qoo`8000Aoo`8000Aoo`8009eoo`0057oo0P00WGoo000DOol2002MOol001Aoo`80
09eoo`0057oo0P00WGoo000DOol2002MOol001Aoo`8009eoo`0057oo0P00WGoo000DOol2002MOol0
01Aoo`8009eoo`0027oo0`000Woo0`0017oo0P00WGoo0009Ool00`00Oomoo`04Ool01@00Oomoogoo
0000WWoo0007Ool40005Ool01000Oomoogoo0P00WGoo0007Ool00`00Ool00003Ool30004Ool00`00
Oomoo`2LOol000Moo`03001oo`0000=oo`03001oogoo00Aoo`03001oogoo09aoo`0027oo0P000goo
10000goo0P00WGoo000DOol00`00Oomoo`2LOol001Aoo`03001oogoo09aoo`0057oo00<007ooOol0
W7oo000DOol2002MOol001Aoo`03001oogoo09aoo`0057oo00<007ooOol0W7oo000DOol2002MOol0
01Aoo`03001oogoo09aoo`0057oo00<007ooOol0W7oo000DOol2002MOol000Moo`<000Aoo`8000Ao
o`03001oogoo09aoo`002Woo00@007ooOol0009oo`05001oogooOol0002NOol000Yoo`04001oogoo
0002Ool01000Oomoogoo0P00WGoo0007Ool30003Ool01000Oomoo`00XWoo0007Ool00`00Oomoo`03
Ool01000Oomoo`00XWoo0007Ool40003Ool2002SOol00;=oo`00\
\>"],
  ImageRangeCache->{{{0, 178.5}, {109.812, 0}} -> {-2.81399, 16.533, 
  0.137482, 0.317788}}]
}, Open  ]],

Cell[TextData[{
  "For the valuation itself, it is convenient to first introduce ",
  Cell[BoxData[
      \(TraditionalForm\`A\_1\)]],
  " as an explicit function:"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(aone[Sf_, r_, q_, \[Sigma]_, t_, 
        K_] = \(-\(\(Sf\ \((BlackScholesPutDelta[Sf, K, \[Sigma], r, q, t] + 
                  1)\)\)\/\[Gamma]\_1[r, \[Sigma], t, q]\)\); \)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Now we can give the valuation of the American Put in approximate \
closed form:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(AmericanPutMBAW[S_, \ K_, \ \[Sigma]_, \ r_, \ q_, \ t_] := \n\t
      Module[{Sf, \ v}, \n\t\t\tSf\  = \ 
          v /. \ \(FindRoot[
                Msolver[v, \ \[Sigma], \ r, \ t, \ K, \ q] == 
                  0, \n\t\t\t\t\t{v, \ K - 1}, \ 
                MaxIterations\  -> \ 40]\)[\([1]\)]; \n\t\tIf[S <= Sf, \ 
          K - S, \ aone[Sf, \ r, \ q, \ \[Sigma], \ t, \ 
                K]*\((S/Sf)\)^\[Gamma]\_1[r, \[Sigma], t, 
                  q] + \n\t\t\t\tBlackScholesPut[S, \ K, \ \[Sigma], \ r, \ 
              q, \ t]]]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We can now both calculate and plot the European and approximate \
American values, showing the rough level of the early exercise premium.\
\>", 
  "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(AmericanPutMBAW[10, \ 10, \ 0.4, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    \(1.20243897229961071`\)], "Output",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesPut[10, \ 10, \ 0.4, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    \(1.08022111136546428`\)], "Output",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[{AmericanPutMBAW[S, \ 10, \ 0.4, \ 0.1, \ 0, \ 
          1], \n\t\tBlackScholesPut[S, \ 10, \ 0.4, \ 0.1, \ 0, \ 
          1]}, \ {S, \ 2, \ 20}]; \)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.047619 0.0147151 0.0735755 [
[.2619 .00222 -3 -9 ]
[.2619 .00222 3 0 ]
[.5 .00222 -6 -9 ]
[.5 .00222 6 0 ]
[.7381 .00222 -6 -9 ]
[.7381 .00222 6 0 ]
[.97619 .00222 -6 -9 ]
[.97619 .00222 6 0 ]
[.01131 .16187 -6 -4.5 ]
[.01131 .16187 0 4.5 ]
[.01131 .30902 -6 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .45617 -6 -4.5 ]
[.01131 .45617 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .01472 m
.2619 .02097 L
s
[(5)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(10)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(15)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(20)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.07143 .01472 m
.07143 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.16667 .01472 m
.16667 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.35714 .01472 m
.35714 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.45238 .01472 m
.45238 .01847 L
s
.54762 .01472 m
.54762 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.64286 .01472 m
.64286 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.83333 .01472 m
.83333 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.92857 .01472 m
.92857 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .16187 m
.03006 .16187 L
s
[(2)] .01131 .16187 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(4)] .01131 .30902 1 0 Mshowa
.02381 .45617 m
.03006 .45617 L
s
[(6)] .01131 .45617 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(8)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .0515 m
.02756 .0515 L
s
.02381 .08829 m
.02756 .08829 L
s
.02381 .12508 m
.02756 .12508 L
s
.02381 .19865 m
.02756 .19865 L
s
.02381 .23544 m
.02756 .23544 L
s
.02381 .27223 m
.02756 .27223 L
s
.02381 .3458 m
.02756 .3458 L
s
.02381 .38259 m
.02756 .38259 L
s
.02381 .41938 m
.02756 .41938 L
s
.02381 .49296 m
.02756 .49296 L
s
.02381 .52974 m
.02756 .52974 L
s
.02381 .56653 m
.02756 .56653 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.11905 .60332 m
.15382 .54959 L
.19174 .491 L
.22736 .43597 L
.26161 .38305 L
.29806 .32672 L
.33316 .2725 L
.37046 .21808 L
.4064 .17638 L
.44097 .14427 L
.47775 .11692 L
.51316 .09592 L
.54722 .07973 L
.58347 .06596 L
.61837 .05546 L
.65547 .04664 L
.69121 .03997 L
.72558 .03489 L
.76216 .03062 L
.79738 .02738 L
.8348 .02468 L
.87086 .02265 L
.90556 .0211 L
.94246 .01979 L
.97619 .01885 L
s
.11905 .5333 m
.15382 .47963 L
.19174 .42149 L
.22736 .36808 L
.26161 .31894 L
.29806 .27028 L
.33316 .22797 L
.37046 .18849 L
.4064 .15594 L
.44097 .1295 L
.47775 .10616 L
.51316 .0878 L
.54722 .07342 L
.58347 .06106 L
.61837 .05155 L
.65547 .04352 L
.69121 .03743 L
.72558 .03279 L
.76216 .02888 L
.79738 .02592 L
.8348 .02346 L
.87086 .02161 L
.90556 .0202 L
.94246 .01903 L
.97619 .01818 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{204.5, 126.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003<0000OQ000`40O003h00Oc7oo003<Ool003Qo
o`<002Ioo`<000Aoo`8002Aoo`<000=oo`<002=oo`@000=oo`8000Uoo`00>goo00<007ooOol097oo
00<007ooOol00Woo00@007ooOol002Aoo`03001oogoo00Eoo`03001oogoo025oo`03001oogoo009o
o`04001oogoo0008Ool003]oo`03001oogoo02Aoo`03001oogoo009oo`04001oogoo000TOol00`00
Oomoo`05Ool00`00Oomoo`0ROol01@00Oomoogoo00000Woo00<007ooOol01Woo000hOol3000WOol0
0`00Oomoo`02Ool01000Oomoo`0097oo00<007ooOol00Woo0`009Woo00@007ooOol0009oo`03001o
ogoo00Ioo`00>7oo00<007ooOol09goo00<007ooOol00Woo00@007ooOol002Aoo`03001oogoo009o
o`03001oogoo02=oo`04001oogoo0002Ool01000Oomoo`0027oo000hOol4000UOol20005Ool2000T
Ool20004Ool4000SOol20004Ool20009Ool00<aoo`003Woo00<007ooOol0^goo000>Ool00`00Oomo
o`2kOol000ioo`03001oogoo0;]oo`002Goo^`0027oo000>Ool00`00Oomoo`0YOol00`00Oomoo`0Y
Ool00`00Oomoo`0ZOol00`00Oomoo`0<OolJ0003Ool00`00Oomoo`0:Ool000ioo`03001oogoo08=o
o`h002Yoo`003Woo00<007ooOol0O7oo3P00<Goo000>Ool00`00Oomoo`1fOol6000oOol000ioo`03
001oogoo06moo`P004Aoo`003Woo0P00Jgoo2P00Agoo000>Ool00`00Oomoo`1WOol8001<Ool000io
o`03001oogoo06=oo`L0055oo`003Woo00<007ooOol0H7oo0`0000Aoo`00000005Aoo`003Woo00<0
07ooOol0GGoo0`0000Aoo`00000005Moo`003Woo00<007ooOol0Fgoo0P000Woo0P00FWoo000>Ool0
0`00Oomoo`1IOol20002Ool2001LOol000ioo`8005Qoo`80009oo`8005ioo`003Woo00<007ooOol0
EGoo0P000Woo0P00H7oo000>Ool00`00Oomoo`1DOol00`00Oomoo`02001ROol000ioo`03001oogoo
059oo`800003Ool0000006Aoo`003Woo00<007ooOol0D7oo0P000Woo00<007ooOol0I7oo000>Ool0
0`00Oomoo`1>Ool20002Ool2001WOol000ioo`03001oogoo04eoo`03001oogoo008006Uoo`003Woo
0P00C7oo0P000Woo00<007ooOol0JGoo000>Ool00`00Oomoo`19Ool20002Ool2001/Ool000ioo`03
001oogoo04Qoo`05001oogooOol0001^Ool000ioo`03001oogoo04Moo`05001oogooOol0001_Ool0
00ioo`03001oogoo04Ioo`05001oogooOol0001`Ool000Moo`@000=oo`03001oogoo04Eoo`05001o
ogooOol0001aOol000Qoo`03001oogoo00=oo`03001oogoo04=oo`8000=oo`03001oogoo071oo`00
2Goo00<007ooOol00Woo0P00@goo00<007ooOol00Woo00<007ooOol0LGoo000:Ool01@00Oomoogoo
0000@goo00<007ooOol00Woo00<007ooOol0LWoo0007Ool01000Oomoo`000goo00<007ooOol0@7oo
00<007ooOol00Woo00<007ooOol0Lgoo0008Ool20004Ool00`00Oomoo`0oOol00`00Oomoo`02Ool0
0`00Oomoo`1dOol000ioo`03001oogoo03ioo`03001oogoo009oo`03001oogoo07Eoo`003Woo00<0
07ooOol0?Goo00<007ooOol00Woo00<007ooOol0MWoo000>Ool00`00Oomoo`0lOol00`00Oomoo`02
Ool00`00Oomoo`1gOol000ioo`8003aoo`03001oogoo009oo`03001oogoo07Qoo`003Woo00<007oo
Ool0>Woo00<007ooOol00Woo00<007ooOol0NGoo000>Ool00`00Oomoo`0iOol00`00Oomoo`02Ool0
0`00Oomoo`1jOol000ioo`03001oogoo03Qoo`03001oogoo009oo`03001oogoo07]oo`003Woo00<0
07ooOol0=goo00<007ooOol00goo00<007ooOol0Ngoo000>Ool00`00Oomoo`0fOol00`00Oomoo`03
Ool00`00Oomoo`1lOol000ioo`03001oogoo03Ioo`03001oogoo009oo`03001oogoo07eoo`003Woo
0P00=Woo00<007ooOol00goo00<007ooOol0OGoo000>Ool00`00Oomoo`0dOol00`00Oomoo`03Ool0
0`00Oomoo`1nOol000ioo`03001oogoo03=oo`03001oogoo00=oo`03001oogoo07moo`003Woo00<0
07ooOol0<goo00<007ooOol00Woo00<007ooOol0P7oo000>Ool00`00Oomoo`0bOol00`00Oomoo`03
Ool00`00Oomoo`20Ool000ioo`03001oogoo035oo`03001oogoo00=oo`03001oogoo085oo`003Woo
0P00<Goo00<007ooOol00goo00<007ooOol0PWoo000>Ool00`00Oomoo`0`Ool00`00Oomoo`03Ool0
0`00Oomoo`22Ool000ioo`03001oogoo02moo`03001oogoo00=oo`03001oogoo08=oo`003Woo00<0
07ooOol0;Woo00<007ooOol017oo00<007ooOol0Pgoo000>Ool00`00Oomoo`0]Ool00`00Oomoo`04
Ool00`00Oomoo`24Ool000Qoo`<000=oo`03001oogoo02aoo`03001oogoo00Eoo`03001oogoo08Ao
o`002Goo00<007ooOol00Woo00<007ooOol0;7oo00<007ooOol017oo00<007ooOol0QGoo0007Ool4
0003Ool2000/Ool00`00Oomoo`04Ool00`00Oomoo`26Ool000Moo`03001oo`0000Aoo`03001oogoo
02Yoo`03001oogoo00Eoo`03001oogoo08Ioo`001goo00<007oo000017oo00<007ooOol0:Goo00<0
07ooOol01Goo00<007ooOol0Qgoo0008Ool20004Ool00`00Oomoo`0YOol00`00Oomoo`05Ool00`00
Oomoo`27Ool000ioo`03001oogoo02Qoo`03001oogoo00Eoo`03001oogoo08Qoo`003Woo00<007oo
Ool09goo00<007ooOol01Woo00<007ooOol0R7oo000>Ool00`00Oomoo`0WOol00`00Oomoo`05Ool0
0`00Oomoo`29Ool000ioo`8002Moo`03001oogoo00Eoo`03001oogoo08Yoo`003Woo00<007ooOol0
9Goo00<007ooOol01Woo00<007ooOol0RWoo000>Ool00`00Oomoo`0UOol00`00Oomoo`05Ool00`00
Oomoo`2;Ool000ioo`03001oogoo02Aoo`03001oogoo00Eoo`03001oogoo08aoo`003Woo00<007oo
Ool08goo00<007ooOol01Goo00<007ooOol0SGoo000>Ool00`00Oomoo`0SOol00`00Oomoo`05Ool0
0`00Oomoo`2=Ool000ioo`03001oogoo029oo`03001oogoo00Eoo`03001oogoo08ioo`003Woo0P00
8Woo00<007ooOol01Goo00<007ooOol0Sgoo000>Ool00`00Oomoo`0QOol00`00Oomoo`05Ool00`00
Oomoo`2?Ool000ioo`03001oogoo021oo`03001oogoo00Eoo`03001oogoo091oo`003Woo00<007oo
Ool07goo00<007ooOol01Goo00<007ooOol0TGoo000>Ool00`00Oomoo`0NOol00`00Oomoo`06Ool0
0`00Oomoo`2AOol000ioo`03001oogoo01ioo`03001oogoo00Eoo`03001oogoo099oo`003Woo00<0
07ooOol07Goo00<007ooOol01Woo00<007ooOol0TWoo000>Ool2000MOol00`00Oomoo`06Ool00`00
Oomoo`2COol000ioo`03001oogoo01aoo`03001oogoo00Eoo`03001oogoo09Aoo`003Woo00<007oo
Ool06goo00<007ooOol01Woo00<007ooOol0U7oo000>Ool00`00Oomoo`0JOol00`00Oomoo`06Ool0
0`00Oomoo`2EOol000ioo`03001oogoo01Yoo`03001oogoo00Ioo`03001oogoo09Eoo`0027oo0P00
17oo00<007ooOol06Goo00<007ooOol01Woo00<007ooOol0UWoo0007Ool01000Oomoo`000goo00<0
07ooOol067oo00<007ooOol01Woo00<007ooOol0Ugoo0007Ool01000Oomoo`000goo0P006Goo00<0
07ooOol01Woo00<007ooOol0Ugoo0007Ool30004Ool00`00Oomoo`0GOol00`00Oomoo`06Ool00`00
Oomoo`2HOol000Moo`03001oogoo00Aoo`03001oogoo01Moo`03001oogoo00Eoo`03001oogoo09Uo
o`0027oo0`000goo00<007ooOol05Woo00<007ooOol01Goo00<007ooOol0VWoo000>Ool00`00Oomo
o`0EOol00`00Oomoo`06Ool00`00Oomoo`2JOol000ioo`03001oogoo01Eoo`03001oogoo00Eoo`03
001oogoo09]oo`003Woo0P005Goo00<007ooOol01Goo00<007ooOol0W7oo000>Ool00`00Oomoo`0C
Ool00`00Oomoo`06Ool00`00Oomoo`2LOol000ioo`03001oogoo01=oo`03001oogoo00Eoo`03001o
ogoo09eoo`003Woo00<007ooOol04Woo00<007ooOol01Goo00<007ooOol0WWoo000>Ool00`00Oomo
o`0BOol00`00Oomoo`05Ool00`00Oomoo`2NOol000ioo`03001oogoo015oo`03001oogoo00Eoo`03
001oogoo09moo`003Woo00<007ooOol047oo00<007ooOol01Goo00<007ooOol0X7oo000>Ool2000A
Ool00`00Oomoo`05Ool00`00Oomoo`2POol000ioo`03001oogoo00moo`03001oogoo00Eoo`03001o
ogoo0:5oo`003Woo00<007ooOol05goo00<007ooOol0XGoo000>Ool00`00Oomoo`0FOol00`00Oomo
o`2ROol000ioo`03001oogoo01Eoo`03001oogoo0:=oo`003Woo00<007ooOol05Goo00<007ooOol0
Xgoo000>Ool00`00Oomoo`0DOol00`00Oomoo`2TOol000ioo`8001Aoo`03001oogoo0:Eoo`003Woo
00<007ooOol04goo00<007ooOol0YGoo000>Ool00`00Oomoo`0BOol00`00Oomoo`2VOol000ioo`03
001oogoo019oo`03001oogoo0:Ioo`003Woo00<007ooOol04Goo00<007ooOol0Ygoo0008Ool20004
Ool00`00Oomoo`0@Ool00`00Oomoo`2XOol000Moo`04001oogoo0003Ool00`00Oomoo`0@Ool00`00
Oomoo`2XOol000Moo`04001oogoo0003Ool2000@Ool00`00Oomoo`2YOol000Qoo`8000Aoo`03001o
ogoo0;]oo`001goo00@007ooOol000=oo`03001oogoo0;]oo`0027oo0P0017oo00<007ooOol0^goo
003<Ool00001\
\>"],
  ImageRangeCache->{{{0, 203.5}, {125.25, 0}} -> {-1.62488, -0.827848, 
  0.11296, 0.0731089}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["11.6 Extrapolation of the Quasi-stationary Method?", "Section",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
The MBAW method can be shown, using a binomial, or \
finite-difference, model, to be a rather inaccurate estimator. We shall \
discuss a more accurate analytical model shortly. However, it is worth \
remarking on how one might, at least in principle, refine the method, bearing \
in mind corresponding results in heat and chemical diffusion. The MBAW method \
involves a change of variables, before invoking the quasi-stationary method. \
In heat problems, with Stefan boundary conditions (such as arise for melting \
ice), there are well-known methods for taking a quasi-stationary \
approximation and improving it. Crank (1984) gives the basic idea, which \
amounts to taking the quasi-stationary solution and adding a correction which \
satisfies the quasi-stationary heat equation (QSHE) with a source that is the \
time-derivative of the quasi-stationary solution. Once this PDE is solved, a \
third correction is added, which satisfies the QSHE with a source that is the \
time-derivative of the first correction, and so on. So the solution gets \
refined, at the price of introducing higher and higher order differential \
equations for the critical point.

This method can sometimes be added up exactly, by the use of Laplace \
transforms, to get an integral equation for the critical point. I have \
studied this myself for chemical diffusion in cylindrical domains, as \
discussed in Shaw (1989) (the relevant appendix is available direct from the \
author). Similar results are given by Hill and Dewynne (1985a, 1985b). \
Sometimes such integral equations can be derived directly, as discussed by \
Ockendon (1974) without recourse to an iterative improvement of the \
quasi-stationary solution. 

It is noteworthy that this is actually rather difficult to implement for the \
American Put problem, and I have so far not succeeded. I encourage others to \
explore the issue. However, two things have put me off investigating the \
matter further, at least for now. The full specification of the problem, in \
terms of an integral equation for the Laplace transform of the critical point \
curve, cannot in general be solved exactly, but does sometimes lead to \
formulae for the small-time and late-time asymptotes. Intermediate times \
typically remain awkward. The second point is that an integral equation for \
the critical point curve for the American Put has already been found \
directly, by Carr, Jarrow and Myneni (1992), and I shall instead pursue a \
discussion of their method, and an accurate approximate solution scheme.\
\>",
   "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  GroupPageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["11.7 The Carr-Jarrow-Myneni Formulae", "Section",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The main achievement of Carr, Jarrow and Myneni (CJM) in their 1992 paper \
was to give the early exercise premium as an ",
  StyleBox["exact",
    FontSlant->"Italic"],
  " functional of the critical price curve. It involves integration of a \
non-linear function of the critical price for all times between evaluation \
and maturity, rather than a non-linear function of just the critical price \
curve at evaluation. To write down the result (you should refer to CJM, 1992, \
for a derivation), we introduce the variables"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Rho]\_1 = r + \[Sigma]\^2\/2\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Rho]\_2 = \(r - \[Sigma]\^2\/2 = \[Rho]\_1 - \
\[Sigma]\^2\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and let the critical stock price be ",
  Cell[BoxData[
      \(TraditionalForm\`\(S\_0\)(t)\)]],
  ". We work with zero dividend yield, as with CJM. The CJM formula for the \
early exercise premium ",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho]\)]],
  " is then just"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Rho] = 
      r\ K\ \(\[Integral]\_0\%T\( \[ExponentialE]\^\(\(-r\)\ t\)\ \
\(Norm(\(log(S\_0[T - t]\/S) - \[Rho]\_2\ t\)\/\(\[Sigma]\ \@t\))\)\) \
\[DifferentialD]t\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The great advantage of this representation is that we can get the \
Delta and Gamma by symbolic differentiation. For the premium component, we \
have\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[CapitalDelta]\_\[Rho] = \(-\(\(r\ K\)\/\(S\ \[Sigma]\
\)\)\)\ \(\[Integral]\_0\%T\(\( \[ExponentialE]\^\(\(-r\)\ t\)\ \
\[ExponentialE]\^\(-\(b\^2\/2\)\)\)\/\@\(2\ \[Pi]\ t\)\) \
\[DifferentialD]t\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["where", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`b = \(log(S\_0[T - t]\/s) - \[Rho]\_2\ t\)\/\(\[Sigma]\
\ \@t\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The Gamma of the premium component is, by a further \
differentiation,\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[CapitalGamma]\_\[Rho] = \(\(r\ K\)\/\(S\^2\ \[Sigma]\
\ \@\(2\ \[Pi]\)\)\) \(\[Integral]\_0\%T\( \[ExponentialE]\^\(\(-r\)\ t\)\ \(\
\[ExponentialE]\^\(-\(b\^2\/2\)\)\)[
                1\/\@t - b\/\(\[Sigma]\ t\)]\) \[DifferentialD]t\)\)], 
  "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox["", TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "One can write down two equivalent integral equations by applying either of \
the boundary conditions at the critical price. The one involving continuity \
of Delta is the simplest, and will be used for our analysis. To express the \
boundary condition concisely, we write the critical price in terms of the \
strike ",
  Cell[BoxData[
      \(TraditionalForm\`K\)]],
  ", other known parameters, and another unknown ",
  Cell[BoxData[
      \(TraditionalForm\`k\_\(1  T\)\)]],
  ", as follows:"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`S\_0[T] = 
      K\ \[ExponentialE]\^\(\(-\[Rho]\_1\)\ T - \[Sigma]\ \@T\ k\_\(1  \
T\)\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Then we define ",
  Cell[BoxData[
      \(TraditionalForm\`b\_\(2  t\)\)]],
  ", following CJM, by the equation"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`b\_\(2\ t\) = \(k\_\(1\ T\)\ \@T - k\_\(1\ T - t\)\ \@\
\(T - t\)\)\/\@t + \[Sigma]\ \@t\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The integral equation we have to solve, arising from continuity of \
Delta at the critical price, is given by\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(r\ K\)\/\@\(2\ \[Pi]\)\ \(\[Integral]\_0\%T\(\( \
\[ExponentialE]\^\(\(-r\)\ t\)\ \[ExponentialE]\^\(-\(b\_\(2  t\)\^2\/2\)\)\)\
\/\@t\) \[DifferentialD]t\) = \ \[Sigma]\ S\_0[
          T]\ \(Norm(\(-k\_\(1\ T\)\))\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["11.8 An Accurate Semi-analytical Treatment of CJM", "Section",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The integral equation for the critical price could be solved by numerical \
methods, but we wish to try to give an accurate semi-analytical alternative. \
The key to this is the square root of ",
  Cell[BoxData[
      \(TraditionalForm\`t\)]],
  " in the denominator. The integral is therefore dominated by the behaviour \
of its integrand near the origin. The unknown under the integral is ",
  Cell[BoxData[
      \(TraditionalForm\`b\_\(2  t\)\)]],
  ", given by "
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`b\_\(2\ t\) = \(\(k\_\(1\ T\)\ \@T - k\_\(1\ T - t\)\ \
\@\(T - t\)\)\/\@t + \[Sigma]\ \@t = \(\[Beta](T, \ t)\)\ \@t\)\)], 
  "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "which now serves to define ",
  Cell[BoxData[
      \(TraditionalForm\`\[Beta](T, \ t)\)]],
  ". "
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Beta](T, \ 
        t)\  = \[Sigma] + \ \(k\_\(1\ T\)\ \@T - k\_\(1\ T - t\)\ \@\(T - t\)\
\)\/t\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "To evaluate the integral approximately we only need to know the behaviour \
of this function near ",
  Cell[BoxData[
      \(TraditionalForm\`t\  = \ 0\)]],
  ". We therefore set"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Beta](T) = 
      lim\+\(t \[Rule] 0\)\[ThinSpace]\[Beta](T, t)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["and we can evaluate this as just", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{\(\[Beta] \((T)\)\), " ", "=", 
        RowBox[{\(\[Sigma] + 
            lim\+\(t \[Rule] 0\)\ \(k\_\(1\ T\)\ \@T - k\_\(1\ T - t\)\ \@\(T \
- t\)\)\/t\), "=", 
          RowBox[{
            RowBox[{"\[Sigma]", "+", " ", \(k\_\(1\ T\)\/\(2\ \@T\)\), "-", 
              RowBox[{\(\@T\), " ", 
                FractionBox[\(\[PartialD]k\_\(T - t\)\), \(\[PartialD]t\),
                  MultilineFunction->None]}]}], \(|\_\(t = 0\)\)}]}]}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Let us defer for a moment the business of estimating ",
  Cell[BoxData[
      \(TraditionalForm\`\[Beta](T)\)]],
  ", and consider the integral equation for the critical point. The left side \
of this equation is now just"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(r\ K\)\/\@\(2\ \[Pi]\)\ \(\[Integral]\_0\%T\( \
\[ExponentialE]\^\(\(-\((\(\[Beta](T)\)\^2\/2 + r)\)\)\ t\)\/\@t\) \
\[DifferentialD]t\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "which can now be evaluated exactly in terms of the error function, to give \
a non-linear equation for ",
  Cell[BoxData[
      \(TraditionalForm\`k\_\(1  T\)\)]],
  " and hence for the critical price:"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(\((r\ K)\)\ \(erf(\@\(1\/2\ \((\(\[Beta](T)\)\^2 + 2\
\ r)\)\))\)\)\/\@\(\(\[Beta](T)\)\^2 + 2\ r\) = \ \ \[Sigma]\ K\ \
\[ExponentialE]\^\(\(-\[Rho]\_1\)\ T + \(-\[Sigma]\)\ \@T\ k\_\(1  T\)\)\ \
\(Norm(\(-k\_\(1\ T\)\))\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Given a model of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Beta](T)\)]],
  ", this can be solved by Newton-Raphson methods, just as in the case of the \
MBAW model. "
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["A Model of \[Beta]", "Subsection",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["All we know is that", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{\(\[Beta] \((T)\)\), " ", "=", " ", 
        RowBox[{\(k\_\(1\ T\)\/\(2\ \@T\)\), "-", 
          RowBox[{\(\@T\), " ", 
            FractionBox[\(\[PartialD]k\_\(T - t\)\), \(\[PartialD]t\),
              MultilineFunction->None]}], \(|\_\(t = 0\)\)}]}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and the second term on the right side is unknown - we solve the resulting \
equation for ",
  Cell[BoxData[
      \(TraditionalForm\`k\_\(1  T\)\)]],
  ", but we need an independent estimate of the derivative. It turns out that \
we can learn a great deal about the structure of this term by using the \
asymptotic behaviour of the critical price for large times, or by postulating \
a simple model for the time-dependence of the front consistent with this \
asymptotic behaviour. We have already worked out the late-time asymptote in \
the MBAW framework, but in order to re-derive it within the CJM model, we \
shall just suppose it has a value of ",
  Cell[BoxData[
      \(TraditionalForm\`S\_\[Infinity]\)]],
  ". We also know that"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`S\_0[T]\/K = \[ExponentialE]\^\(\(-\@T\)\ \[Sigma]\ \
k\_\(1\ T\) - \(\[Rho]\_1\) T\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Now letting ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  " become large, and inverting this relationship, we must have that, \
asymptotically,"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`k\_\(1\ T\) \[Tilde] \(log(K\/S\_\[Infinity])\)\/\(\
\[Sigma]\ \@T\) - \(\[Rho]\_1\ \@T\)\/\[Sigma]\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This allows us to estimate the unknown derivative and hence ",
  Cell[BoxData[
      \(TraditionalForm\`\[Beta](T)\)]],
  ". A few lines of algebra reveal that, as ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  " becomes infinite, ",
  StyleBox["irrespective of the value of",
    FontSlant->"Italic"],
  " ",
  Cell[BoxData[
      \(TraditionalForm\`S\_\[Infinity]\)]],
  ", "
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Beta](T) \[Tilde] \[Sigma] - \[Rho]\_1\/\[Sigma]\)], 
  "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "for large values of ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  ". Note, for consistency, that since then "
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`Norm(\(-k\_\(1\ T\)\)) \[Tilde] 1\)], 
  "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
our integral equation for the critical price gives, as before,\
\>",
   "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`S\_\[Infinity] = \(\(r\ K\)\/\(\[Sigma]\ \
\@\(\[Beta]\^2 + 2\ r\)\) = K\/\(1 + \[Sigma]\^2\/\(2\ r\)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["More generally", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`k\_\(1\ T\) = \(-\(\(\[Rho]\_1\ \@T\)\/\[Sigma]\)\) + \
\(log(K\/\(S(T)\))\)\/\(\[Sigma]\ \@T\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We have shown that any choice of \"constant\" ",
  Cell[BoxData[
      \(TraditionalForm\`S(T)\)]],
  " in the second term gets wiped out in the calculation of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Beta]\)]],
  ". So one family of estimates for ",
  Cell[BoxData[
      \(TraditionalForm\`\[Beta]\)]],
  " is given by just using the first term and making the replacement"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{\(-\@T\), " ", 
        RowBox[{
          FractionBox[\(\[PartialD]k\_\(T - t\)\), \(\[PartialD]t\),
            MultilineFunction->None], \(|\_\(t = 0\)\), " ", 
          "\[LongRightArrow]", " ", \(-\(\[Rho]\_1\/\(2\ \[Sigma]\)\)\)}]}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["which leads to", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Beta]\ \((T)\) = \[Sigma] - \[Rho]\_1\/\(2\ \[Sigma]\
\) + k\_\(1\ T\)\/\(2\ \@T\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["and the non-linear equation for solution is", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(\((r\ K)\)\ \(erf(\@\(1\/2\ \((\(\[Beta](T)\)\^2 + 2\
\ r)\)\))\)\)\/\@\(\(\[Beta](T)\)\^2 + 2\ r\) = \ \ \[Sigma]\ K\ \
\[ExponentialE]\^\(\(-\[Rho]\_1\)\ T + \(-\[Sigma]\)\ \@T\ k\_\(1  T\)\)\ \
\(Norm(\(-k\_\(1\ T\)\))\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "It is possible to consider more sophisticated models for \[Beta] based, \
for example, on the Van Moerbeke (1976) model for the front, but this \
involves a free parameter which must be fixed. The use of such a scheme can \
lead to extremely accurate results if the free parameter is suitably fiddled, \
but the method is somewhat unsatisfactory in requiring such tuning. Other \
models can be developed based on the observation that ",
  Cell[BoxData[
      \(TraditionalForm\`\[Beta](T, T)\  = \ \[Sigma] + k\_\(1  T\)/\@T\)]],
  " exactly, but this result only relates to the integral away from ",
  Cell[BoxData[
      \(TraditionalForm\`t = 0\)]],
  ", which is the dominating region. The author would be pleased to hear of \
anyone's experiments for more suitable models of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Beta]\)]],
  ". In particular, better independent estimates of the small-time or \
late-time asymptotes could be used to refine the model presented here."
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "11.9 ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Implementation of Approximate CJM"
}], "Section",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We can now give a very concise representation of the CJM model. We \
define a function for the critical price, or front, then three functions for \
value, Delta and Gamma:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Front[strike_, r_, sigma_, t_] :=
If[t==0,strike,Module[{l,m},
l= m/. FindRoot[Exp[-m*sigma*Sqrt[t] - (r+sigma^2/2)*t]*(1+Erf[-m/Sqrt[2]]) \
==
r*Sqrt[2/(r+(sigma - (r/(2*sigma)+sigma/4) +m/(2*Sqrt[t]))^2/2)]*
Erf[Sqrt[(r+(sigma-(r/(2*sigma)+sigma/4)+m/(2*Sqrt[t]))^2/2)*t]]/sigma,
{m,-(sigma/2+r/sigma)*Sqrt[t]},MaxIterations -> 30];
strike*Exp[-l*sigma*Sqrt[t] - (r+sigma^2/2)*t]]]\
\>", "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
AmericanPutCJM[price_,strike_,sigma_,r_,t_] :=
If[price <= Front[strike, r, sigma, t], strike - price,
BlackScholesPut[price,strike,sigma,r,0,t]+
r*strike*NIntegrate[Exp[-r*u]*Norm[(Log[
Front[strike,r,sigma,t-u]/price]-(r-sigma^2/2)*u)/(sigma*Sqrt[u])],
{u,0,t}]]\
\>", "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
AmericanPutDeltaCJM[price_,strike_,sigma_,r_,t_] :=
If[price <= Front[strike, r, sigma, t],  -1,
N[BlackScholesPutDelta[price,strike,sigma,r,0,t] - 
(r*strike/(price*sigma*Sqrt[2.*Pi]))*NIntegrate[Exp[-r*u]*Exp[-((Log[
Front[strike,r,sigma,t-u]/price]-(r-sigma^2/2)*u)/(sigma*Sqrt[u]))^2/2]/Sqrt[\
u],
{u, 0, t}]]]\
\>", "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
AmericanPutGammaCJM[price_,strike_,sigma_,r_,t_] :=
If[price <= Front[strike, r, sigma, t], 0,
N[
BlackScholesPutGamma[price,strike,sigma,r,0,t]+
(r*strike/(price^2*sigma*Sqrt[2.*Pi]))*NIntegrate[Exp[-r*u]*Exp[-((Log[
Front[strike,r,sigma,t-u]/price]-(r-sigma^2/2)*u)/(sigma*Sqrt[u]))^2/2]*
(1/Sqrt[u] - 
(Log[Front[strike,r,sigma,t-u]/price]-(r-sigma^2/2)*u)/(sigma*Sqrt[u])/(sigma*\
u)),
{u, 0, t}]]]
\
\>", "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Let's take a look at the critical price frontier, for the same \
parameters as before:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(cjmplot\  = \ 
      Plot[{Front[50, \ 0.1, \ 0.4, \ t], \ 
          FrontLate[50, \ 0.1, \ 0.4, \ t]}, \n\t\t{t, \ 0, \ 20}, \ 
        PlotRange\  -> \ {25, \ 
            50}, \n\t\tPlotStyle\  -> \ {{Thickness[0.01], \ 
              Dashing[{0.025, \ 0.025}]}, \ 
            Dashing[{0.025, \ 0.025}]}]; \)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.047619 -0.618034 0.0247214 [
[.2619 -0.0125 -3 -9 ]
[.2619 -0.0125 3 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.7381 -0.0125 -6 -9 ]
[.7381 -0.0125 6 0 ]
[.97619 -0.0125 -6 -9 ]
[.97619 -0.0125 6 0 ]
[.01131 .12361 -12 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -12 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -12 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -12 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -12 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 0 m
.2619 .00625 L
s
[(5)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(10)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(15)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(20)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(30)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(35)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(40)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(45)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(50)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .02472 m
.02756 .02472 L
s
.02381 .04944 m
.02756 .04944 L
s
.02381 .07416 m
.02756 .07416 L
s
.02381 .09889 m
.02756 .09889 L
s
.02381 .14833 m
.02756 .14833 L
s
.02381 .17305 m
.02756 .17305 L
s
.02381 .19777 m
.02756 .19777 L
s
.02381 .22249 m
.02756 .22249 L
s
.02381 .27193 m
.02756 .27193 L
s
.02381 .29666 m
.02756 .29666 L
s
.02381 .32138 m
.02756 .32138 L
s
.02381 .3461 m
.02756 .3461 L
s
.02381 .39554 m
.02756 .39554 L
s
.02381 .42026 m
.02756 .42026 L
s
.02381 .44498 m
.02756 .44498 L
s
.02381 .46971 m
.02756 .46971 L
s
.02381 .51915 m
.02756 .51915 L
s
.02381 .54387 m
.02756 .54387 L
s
.02381 .56859 m
.02756 .56859 L
s
.02381 .59331 m
.02756 .59331 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.01 w
[ .025 .025 ] 0 setdash
.02381 .61637 m
.02499 .47595 L
.02605 .43941 L
.02729 .41107 L
.02846 .39081 L
.03053 .36329 L
.03279 .3406 L
.0379 .3036 L
.04262 .27919 L
.05224 .24401 L
.0571 .23068 L
.06244 .21823 L
.07293 .19857 L
.0842 .18223 L
.10458 .16048 L
.12571 .14439 L
.1458 .1329 L
.184 .1173 L
.20367 .11138 L
.22464 .10616 L
.26622 .09822 L
.30629 .09268 L
.34484 .08866 L
.38585 .08538 L
.42533 .08293 L
.4633 .08106 L
.50373 .07946 L
.54264 .07823 L
.584 .07716 L
.62384 .07632 L
.66217 .07566 L
.70296 .07506 L
.74222 .07459 L
.77997 .0742 L
.82018 .07385 L
.85886 .07356 L
.9 .07329 L
.93963 .07307 L
.97619 .0729 L
s
.5 Mabswid
.02381 .06867 m
.06244 .06867 L
.10458 .06867 L
.14415 .06867 L
.18221 .06867 L
.22272 .06867 L
.26171 .06867 L
.30316 .06867 L
.34309 .06867 L
.3815 .06867 L
.42237 .06867 L
.46172 .06867 L
.49955 .06867 L
.53984 .06867 L
.57861 .06867 L
.61984 .06867 L
.65954 .06867 L
.69774 .06867 L
.73838 .06867 L
.77751 .06867 L
.81909 .06867 L
.85916 .06867 L
.89771 .06867 L
.93871 .06867 L
.97619 .06867 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  ImageSize->{204.5, 126.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003<0000OQ000`40O003h00Oc7oo003<Ool003eo
o`<002Aoo`<000Aoo`80029oo`<000=oo`<0025oo`@000=oo`8000Yoo`00@7oo00<007ooOol08Woo
00<007ooOol00Woo00@007ooOol0029oo`03001oogoo00Eoo`03001oogoo01moo`03001oogoo009o
o`04001oogoo0009Ool0041oo`03001oogoo029oo`03001oogoo009oo`04001oogoo000ROol00`00
Oomoo`05Ool00`00Oomoo`0POol01@00Oomoogoo00000Woo00<007ooOol01goo000mOol3000UOol0
0`00Oomoo`02Ool01000Oomoo`008Woo00<007ooOol00Woo0`0097oo00@007ooOol0009oo`03001o
ogoo00Moo`00?Goo00<007ooOol09Goo00<007ooOol00Woo00@007ooOol0029oo`03001oogoo009o
o`03001oogoo025oo`04001oogoo0002Ool01000Oomoo`002Goo000mOol4000SOol20005Ool2000R
Ool20004Ool4000QOol20004Ool2000:Ool00<aoo`00c7oo003<Ool0015ook8000Uoo`005Goo00<0
07ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<0
07ooOol01Woo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<0
07ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<0
07ooOol01Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<0
07ooOol02goo000EOol00`00Oomoo`0WOol00`00Oomoo`0WOol00`00Oomoo`0XOol00`00Oomoo`0W
Ool00`00Oomoo`0;Ool001Eoo`03001oogoo0;Aoo`005Goo00<007ooOol0]7oo000EOol2002eOol0
01Eoo`03001oogoo0;Aoo`005Goo00<007ooOol0]7oo000EOol00`00Oomoo`2dOol001Eoo`800;Eo
o`005Goo00<007ooOol0]7oo000EOol00`00Oomoo`2dOol001Eoo`03001oogoo0;Aoo`005Goo1@00
17oo1@0017oo1@0017oo1@000goo1P000goo1P000goo1P000goo1@0017oo1@0017oo1@0017oo1@00
0goo1P000goo1P000goo1P000goo1@0017oo1@0017oo1@0017oo1@000Woo1`000Woo0`003Goo000E
Ool2001GOol30004Ool50004Ool50004Ool50003Ool60003Ool60003Ool60003Ool6000LOol001Eo
o`03001oogoo049oo`H000=oo`H000=oo`8005ioo`005Goo00<007ooOol0<goo0`0017oo1@00MGoo
000EOol00`00Oomoo`0ZOol30004Ool20021Ool001Eoo`8002Uoo`8008Yoo`005Goo00<007ooOol0
87oo1@00Sgoo0008Ool30004Ool20004Ool00`00Oomoo`0OOol00`00Oomoo`2BOol000]oo`04001o
ogoo0002Ool01@00Oomoogoo00006Woo1000V7oo000;Ool01000Oomoo`000Woo00@007ooOomoo`80
01Qoo`03001oogoo09Yoo`002Goo0P000goo00@007ooOol000=oo`03001oogoo0;Aoo`002goo00@0
07ooOol0009oo`05001oogooOol0000DOol2002POol000Qoo`<000Aoo`8000Aoo`03001oogoo011o
o`800:9oo`005Goo00<007ooOol03goo00<007ooOol0XWoo000EOol2002eOol001Eoo`03001oogoo
0;Aoo`005Goo00<007ooOol02goo00<007ooOol0YWoo000EOol00`00Oomoo`0:Ool00`00Oomoo`2W
Ool001Eoo`8000Yoo`03001oogoo0:Qoo`005Goo00<007ooOol027oo00<007ooOol0ZGoo000EOol0
0`00Oomoo`2dOol001Eoo`03001oogoo0;Aoo`005Goo00<007ooOol0]7oo000EOol20006Ool2002]
Ool001Eoo`03001oogoo00Ioo`03001oogoo0:]oo`005Goo00<007ooOol01Goo00<007ooOol0[7oo
000EOol00`00Oomoo`04Ool00`00Oomoo`2]Ool001Eoo`8000Eoo`03001oogoo0:eoo`005Goo00<0
07ooOol0]7oo0008Ool30003Ool30004Ool00`00Oomoo`2dOol000]oo`03001oogoo00=oo`05001o
ogooOol0002fOol000]oo`03001oogoo00=oo`04001oogooOol20003Ool00`00Oomoo`2_Ool000Uo
o`8000=oo`<000Aoo`03001oogoo009oo`03001oogoo0:moo`002goo00@007ooOol000Ioo`03001o
ogoo009oo`03001oogoo0:moo`0027oo0`000goo10000goo00D007ooOomoo`000;9oo`005Goo00D0
07ooOomoo`000;9oo`005Goo0P00]Goo000EOol00`00Oomoo`2dOol001Eoo`03001oogoo0;Aoo`00
5Goo00@007ooOol00;=oo`005Goo0P0000=oo`00Ool0/Woo000EOol00`00Ool0002dOol001Eoo`03
001oo`000;Aoo`005Goo00<007oo0000]7oo000EOol3002dOol001Eoo`03001oogoo0;Aoo`005Goo
00<007ooOol0]7oo000EOol00`00Oomoo`2dOol001Eoo`800;Eoo`005Goo0P00]Goo000EOol2002e
Ool000Uoo`<000=oo`8000Aoo`800;Eoo`002Woo00D007ooOomoo`00009oo`04001oogooOol2002e
Ool000Qoo`@0009oo`04001oogoo0003Ool2002eOol000Qoo`03001oo`0000=oo`04001oogoo0003
Ool00`00Oomoo`2dOol000Qoo`03001oo`0000=oo`04001oogoo0003Ool00`00Oomoo`2dOol000Uo
o`8000Aoo`8000Aoo`03001oogoo0;Aoo`005Goo0P00]Goo000EOol2002eOol001Eoo`800;Eoo`00
5Goo0P00]Goo000EOol2002eOol001Eoo`800;Eoo`005Goo00<007ooOol0]7oo000EOol00`00Oomo
o`2dOol001Eoo`03001oogoo0;Aoo`005Goo0P00]Goo000EOol00`00Oomoo`2dOol001Eoo`03001o
ogoo0;Aoo`005Goo00<007ooOol0]7oo000EOol00`00Oomoo`2dOol001Eoo`800;Eoo`005Goo00<0
07ooOol0]7oo0009Ool30002Ool30004Ool00`00Oomoo`2dOol000Yoo`03001oogoo00Aoo`05001o
ogooOol0002fOol000Qoo`@000Eoo`04001oogooOol2002eOol000Qoo`03001oo`0000=oo`<000Ao
o`03001oogoo0;Aoo`0027oo00<007oo00000goo00<007ooOol017oo00<007ooOol0]7oo0009Ool2
0003Ool40003Ool00`00Oomoo`2dOol001Eoo`800;Eoo`005Goo00<007ooOol0]7oo000EOol00`00
Oomoo`2dOol001Eoo`03001oogoo0;Aoo`005Goo00<007ooOol0]7oo000EOol2002eOol001Eoo`03
001oogoo0;Aoo`005Goo00<007ooOol0]7oo000EOol00`00Oomoo`2dOol001Eoo`800;Eoo`005Goo
00<007ooOol0]7oo000EOol00`00Oomoo`2dOol001Eoo`03001oogoo0;Aoo`005Goo00<007ooOol0
]7oo000EOol2002eOol001Eoo`03001oogoo0;Aoo`0027oo0`0017oo0P0017oo00<007ooOol0]7oo
000;Ool01000Oomoo`000Woo00D007ooOomoo`000;Ioo`002goo00@007ooOol0009oo`04001oogoo
Ool2002eOol000Qoo`<000=oo`04001oogoo002jOol000Qoo`03001oogoo00=oo`04001oogoo002j
Ool000Qoo`@000=oo`800;]oo`00c7oo0000\
\>"],
  ImageRangeCache->{{{0, 203.5}, {125.25, 0}} -> {-2.54308, 22.44, 0.11849, 
  0.228239}}]
}, Open  ]],

Cell["\<\
We can overlay this with the MBAW frontier and see the approximate \
consistency:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[cjmplot, mbawplot]; \)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.047619 -0.618034 0.0247214 [
[.2619 -0.0125 -3 -9 ]
[.2619 -0.0125 3 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.7381 -0.0125 -6 -9 ]
[.7381 -0.0125 6 0 ]
[.97619 -0.0125 -6 -9 ]
[.97619 -0.0125 6 0 ]
[.01131 .12361 -12 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -12 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -12 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -12 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -12 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 0 m
.2619 .00625 L
s
[(5)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(10)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(15)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(20)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(30)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(35)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(40)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(45)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(50)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .02472 m
.02756 .02472 L
s
.02381 .04944 m
.02756 .04944 L
s
.02381 .07416 m
.02756 .07416 L
s
.02381 .09889 m
.02756 .09889 L
s
.02381 .14833 m
.02756 .14833 L
s
.02381 .17305 m
.02756 .17305 L
s
.02381 .19777 m
.02756 .19777 L
s
.02381 .22249 m
.02756 .22249 L
s
.02381 .27193 m
.02756 .27193 L
s
.02381 .29666 m
.02756 .29666 L
s
.02381 .32138 m
.02756 .32138 L
s
.02381 .3461 m
.02756 .3461 L
s
.02381 .39554 m
.02756 .39554 L
s
.02381 .42026 m
.02756 .42026 L
s
.02381 .44498 m
.02756 .44498 L
s
.02381 .46971 m
.02756 .46971 L
s
.02381 .51915 m
.02756 .51915 L
s
.02381 .54387 m
.02756 .54387 L
s
.02381 .56859 m
.02756 .56859 L
s
.02381 .59331 m
.02756 .59331 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.01 w
[ .025 .025 ] 0 setdash
.02381 .61637 m
.02499 .47595 L
.02605 .43941 L
.02729 .41107 L
.02846 .39081 L
.03053 .36329 L
.03279 .3406 L
.0379 .3036 L
.04262 .27919 L
.05224 .24401 L
.0571 .23068 L
.06244 .21823 L
.07293 .19857 L
.0842 .18223 L
.10458 .16048 L
.12571 .14439 L
.1458 .1329 L
.184 .1173 L
.20367 .11138 L
.22464 .10616 L
.26622 .09822 L
.30629 .09268 L
.34484 .08866 L
.38585 .08538 L
.42533 .08293 L
.4633 .08106 L
.50373 .07946 L
.54264 .07823 L
.584 .07716 L
.62384 .07632 L
.66217 .07566 L
.70296 .07506 L
.74222 .07459 L
.77997 .0742 L
.82018 .07385 L
.85886 .07356 L
.9 .07329 L
.93963 .07307 L
.97619 .0729 L
s
.5 Mabswid
.02381 .06867 m
.06244 .06867 L
.10458 .06867 L
.14415 .06867 L
.18221 .06867 L
.22272 .06867 L
.26171 .06867 L
.30316 .06867 L
.34309 .06867 L
.3815 .06867 L
.42237 .06867 L
.46172 .06867 L
.49955 .06867 L
.53984 .06867 L
.57861 .06867 L
.61984 .06867 L
.65954 .06867 L
.69774 .06867 L
.73838 .06867 L
.77751 .06867 L
.81909 .06867 L
.85916 .06867 L
.89771 .06867 L
.93871 .06867 L
.97619 .06867 L
s
[ ] 0 setdash
.02381 .60422 m
.03334 .35055 L
.04286 .29405 L
.05239 .2596 L
.06191 .23504 L
.07143 .21615 L
.08096 .20097 L
.09048 .18838 L
.1 .17771 L
.10953 .16852 L
.11905 .16049 L
.12858 .15342 L
.1381 .14712 L
.14762 .14148 L
.15715 .1364 L
.16667 .13179 L
.1762 .1276 L
.18572 .12377 L
.19524 .12025 L
.20477 .11702 L
.21429 .11403 L
.22381 .11127 L
.23334 .10871 L
.24286 .10634 L
.25239 .10412 L
.26191 .10206 L
.27143 .10014 L
.28096 .09834 L
.29048 .09665 L
.3 .09507 L
.30953 .09359 L
.31905 .09219 L
.32858 .09088 L
.3381 .08965 L
.34762 .08849 L
.35715 .08739 L
.36667 .08636 L
.3762 .08538 L
.38572 .08446 L
.39524 .08359 L
.40477 .08277 L
.41429 .08199 L
.42381 .08125 L
.43334 .08056 L
.44286 .0799 L
.45239 .07927 L
.46191 .07868 L
.47143 .07811 L
.48096 .07758 L
.49048 .07707 L
Mistroke
.5 .07659 L
.50953 .07614 L
.51905 .0757 L
.52858 .07529 L
.5381 .0749 L
.54762 .07453 L
.55715 .07418 L
.56667 .07384 L
.5762 .07353 L
.58572 .07322 L
.59524 .07294 L
.60477 .07266 L
.61429 .07241 L
.62381 .07216 L
.63334 .07193 L
.64286 .0717 L
.65239 .07149 L
.66191 .07129 L
.67143 .0711 L
.68096 .07092 L
.69048 .07075 L
.7 .07058 L
.70953 .07043 L
.71905 .07028 L
.72858 .07014 L
.7381 .07001 L
.74762 .06988 L
.75715 .06976 L
.76667 .06965 L
.7762 .06954 L
.78572 .06944 L
.79524 .06934 L
.80477 .06925 L
.81429 .06917 L
.82381 .06908 L
.83334 .06901 L
.84286 .06893 L
.85239 .06886 L
.86191 .0688 L
.87143 .06874 L
.88096 .06868 L
.89048 .06862 L
.9 .06857 L
.90953 .06852 L
.91905 .06848 L
.92858 .06843 L
.9381 .06839 L
.94762 .06835 L
.95715 .06832 L
.96667 .06828 L
Mistroke
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  ImageSize->{204.5, 126.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003<0000OQ000`40O003h00Oc7oo003<Ool003eo
o`<002Aoo`<000Aoo`80029oo`<000=oo`<0025oo`@000=oo`8000Yoo`00@7oo00<007ooOol08Woo
00<007ooOol00Woo00@007ooOol0029oo`03001oogoo00Eoo`03001oogoo01moo`03001oogoo009o
o`04001oogoo0009Ool0041oo`03001oogoo029oo`03001oogoo009oo`04001oogoo000ROol00`00
Oomoo`05Ool00`00Oomoo`0POol01@00Oomoogoo00000Woo00<007ooOol01goo000mOol3000UOol0
0`00Oomoo`02Ool01000Oomoo`008Woo00<007ooOol00Woo0`0097oo00@007ooOol0009oo`03001o
ogoo00Moo`00?Goo00<007ooOol09Goo00<007ooOol00Woo00@007ooOol0029oo`03001oogoo009o
o`03001oogoo025oo`04001oogoo0002Ool01000Oomoo`002Goo000mOol4000SOol20005Ool2000R
Ool20004Ool4000QOol20004Ool2000:Ool00<aoo`00c7oo003<Ool0015ook8000Uoo`005Goo00<0
07ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<0
07ooOol01Woo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<0
07ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<0
07ooOol01Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<0
07ooOol02goo000EOol00`00Oomoo`0WOol00`00Oomoo`0WOol00`00Oomoo`0XOol00`00Oomoo`0W
Ool00`00Oomoo`0;Ool001Eoo`03001oogoo0;Aoo`005Goo00<007ooOol0]7oo000EOol2002eOol0
01Eoo`03001oogoo0;Aoo`005Goo00<007ooOol0]7oo000EOol00`00Oomoo`2dOol001Eoo`800;Eo
o`005Goo00<007ooOol0]7oo000EOol00`00Oomoo`2dOol001Eoo`03001oogoo0;Aoo`005Goo1@00
17oo1@0017oo1@0017oo1@000goo1P000goo1P000goo1P000goo1@0017oo1@0017oo1@0017oo1@00
0gooB@003Goo000EOol2001;OolH0004Ool50004Ool50003Ool60003Ool60003Ool60003Ool6000L
Ool001Eoo`03001oogoo03aoo`h00003Ool0000000@000=oo`8005ioo`005Goo00<007ooOol0<goo
3000MGoo000EOol00`00Oomoo`0ZOol:0020Ool001Eoo`8002Uoo`D008Moo`005Goo00<007ooOol0
87oo2000S7oo0008Ool30004Ool20004Ool00`00Oomoo`0OOol4002AOol000]oo`04001oogoo0002
Ool01@00Oomoogoo00006Woo2000U7oo000;Ool01000Oomoo`000Woo00@007ooOomoo`8001Qoo`03
001oo`00008009Qoo`002Goo0P000goo00@007ooOol000=oo`03001oogoo01Moo`8009]oo`002goo
00@007ooOol0009oo`05001oogooOol0000DOol20002Ool00`00Oomoo`2KOol000Qoo`<000Aoo`80
00Aoo`03001oogoo011oo`80009oo`8009ioo`005Goo00<007ooOol03goo00<007ooOol00P00X7oo
000EOol2000BOol00`00Oomoo`2POol001Eoo`03001oogoo00moo`800:=oo`005Goo00<007ooOol0
2goo00@007oo00000:Eoo`005Goo00<007ooOol02Woo00<007oo0000Ygoo000EOol2000:Ool00`00
Ool0002XOol001Eoo`03001oogoo00Qoo`03001oo`000:Uoo`005Goo00<007ooOol02Woo00<007oo
Ool0Ygoo000EOol00`00Oomoo`09Ool00`00Oomoo`2XOol001Eoo`03001oogoo00Qoo`03001oogoo
0:Uoo`005Goo0P001Woo0`00[7oo000EOol00`00Oomoo`06Ool2002/Ool001Eoo`03001oogoo00Eo
o`800:eoo`005Goo00<007ooOol017oo0P00[Woo000EOol20005Ool2002^Ool001Eoo`03001oogoo
00Aoo`03001oogoo0:eoo`0027oo0`000goo0`0017oo00<007ooOol017oo00<007ooOol0[Goo000;
Ool00`00Oomoo`03Ool01@00Oomoogoo00001Woo00<007ooOol0[Goo000;Ool00`00Oomoo`03Ool0
1000Oomoogoo0P000goo0P00/7oo0009Ool20003Ool30004Ool00`00Oomoo`02Ool2002`Ool000]o
o`04001oogoo0006Ool00`00Oomoo`02Ool00`00Oomoo`2_Ool000Qoo`<000=oo`@000=oo`04001o
ogooOol2002aOol001Eoo`04001oogooOol2002aOol001Eoo`80009oo`03001oogoo0;1oo`005Goo
00D007ooOomoo`000;9oo`005Goo00D007ooOomoo`000;9oo`005Goo00@007ooOol00;=oo`005Goo
0P0000=oo`00Ool0/Woo000EOol01000Ool00000/goo000EOol01000Ool00000/goo000EOol01000
Ool00000/goo000EOol4002cOol001Eoo`04001oogoo002cOol001Eoo`03001oo`000;Aoo`005Goo
00<007oo0000]7oo000EOol3002dOol001Eoo`<00;Aoo`005Goo0`00]7oo0009Ool30003Ool20004
Ool3002dOol000Yoo`05001oogooOol00002Ool01000Oomoogoo0`00]7oo0008Ool40002Ool01000
Oomoo`000goo0`00]7oo0008Ool00`00Ool00003Ool01000Oomoo`000goo00<007oo0000]7oo0008
Ool00`00Ool00003Ool01000Oomoo`000goo00<007oo0000]7oo0009Ool20004Ool20004Ool00`00
Ool0002dOol001Eoo`<00;Aoo`005Goo0`00]7oo000EOol3002dOol001Eoo`<00;Aoo`005Goo0`00
]7oo000EOol2002eOol001Eoo`800;Eoo`005Goo0P00]Goo000EOol2002eOol001Eoo`800;Eoo`00
5Goo0P00]Goo000EOol2002eOol001Eoo`800;Eoo`005Goo0P00]Goo000EOol2002eOol001Eoo`80
0;Eoo`002Goo0`000Woo0`0017oo0P00]Goo000:Ool00`00Oomoo`04Ool01000Oomoogoo0P00]Goo
0008Ool40005Ool01000Oomoogoo0P00]Goo0008Ool00`00Ool00003Ool30004Ool2002eOol000Qo
o`03001oo`0000=oo`03001oogoo00Aoo`800;Eoo`002Goo0P000goo10000goo0P00]Goo000EOol2
002eOol001Eoo`800;Eoo`005Goo0P00]Goo000EOol2002eOol001Eoo`800;Eoo`005Goo0P00]Goo
000EOol00`00Oomoo`2dOol001Eoo`03001oogoo0;Aoo`005Goo00<007ooOol0]7oo000EOol2002e
Ool001Eoo`03001oogoo0;Aoo`005Goo00<007ooOol0]7oo000EOol00`00Oomoo`2dOol001Eoo`03
001oogoo0;Aoo`005Goo0P00]Goo000EOol00`00Oomoo`2dOol000Qoo`<000Aoo`8000Aoo`03001o
ogoo0;Aoo`002goo00@007ooOol0009oo`05001oogooOol0002fOol000]oo`04001oogoo0002Ool0
1000Oomoogoo0P00]Goo0008Ool30003Ool01000Oomoo`00^Woo0008Ool00`00Oomoo`03Ool01000
Oomoo`00^Woo0008Ool40003Ool2002kOol00<aoo`00\
\>"],
  ImageRangeCache->{{{0, 203.5}, {125.25, 0}} -> {-2.54308, 22.44, 0.11849, 
  0.228239}}]
}, Open  ]],

Cell["\<\
Let's make a few comparisons for a strike at 50, volatility 40%, \
interest rate 10% (cc) with one year to expiry:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["At-the-Money Comparisons", "Subsubsection",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["European", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesPut[50, \ 50, \ 0.4, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(5.4011055568273214`\)], "Output"]
}, Open  ]],

Cell["American MBAW", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(AmericanPutMBAW[50, \ 50, \ 0.4, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(6.01219487525652951`\)], "Output"]
}, Open  ]],

Cell["American CJM", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(AmericanPutCJM[50, \ 50, \ 0.4, \ 0.1, \ 1]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(5.97688400913239092`\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["In-the-Money Comparisons", "Subsubsection",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["European", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesPut[45, \ 50, \ 0.4, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(7.27411366730570918`\)], "Output"]
}, Open  ]],

Cell["American MBAW", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(AmericanPutMBAW[45, \ 50, \ 0.4, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(8.18143474856326591`\)], "Output"]
}, Open  ]],

Cell["American CJM", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(AmericanPutCJM[45, \ 50, \ 0.4, \ 0.1, \ 1]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(8.18064363224586266`\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Out-of-the-Money Comparisons", "Subsubsection",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["European", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesPut[55, \ 50, \ 0.4, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(3.97887543168881663`\)], "Output"]
}, Open  ]],

Cell["American MBAW", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(AmericanPutMBAW[55, \ 50, \ 0.4, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(4.40626377467003216`\)], "Output"]
}, Open  ]],

Cell["American CJM", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(AmericanPutCJM[55, \ 50, \ 0.4, \ 0.1, \ 1]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(4.34905798847774782`\)], "Output"]
}, Open  ]],

Cell["\<\
It is an interesting question how one decides which result is best. \
We shall revisit this question after developing binomial and \
finite-difference models, where it will turn out that the CJM approximation \
not only is substantially better, but will typically outperform binomial \
models unless several hundred time-steps are taken. \
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["11.10 The Roll-Geske-Whaley Model for American Calls ", "Section",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "When we turn our attention to American Calls, if the dividend payments are \
large enough early exercise may be desirable on the last dividend date. (Here \
we confine attention to discrete dividends.) This can be accounted for \
analytically by the Roll (1977), Geske (1979, 1981) and Whaley (1981) \
formulae, as described also by Hull (1996). Their formula reduces to the \
\"jump-adjusted\" Black-Scholes formula for low enough dividends, where there \
is no early exercise (see Chapter 24 for further discussion of the treatment \
of discrete dividends). The evaluation of this formula requires various \
special functions. First we need to define the cumulative normal distribution \
in two variables - we get it by loading a Package in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Statistics`MultinormalDistribution`\>"]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell["\<\
The objects loaded are associated with the following distribution \
list. \
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Statistics`MultinormalDistribution`*\)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
HotellingTSquareDistribution MultivariateTDistribution    \
WishartDistribution
MultinormalDistribution      QuadraticFormDistribution\
\>", "Print",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell["We want the two-variable case of the multinormal object:", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?MultinormalDistribution\)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("MultinormalDistribution[mu, sigma] represents the multivariate normal \
(Gaussian) distribution with mean vector mu and covariance matrix sigma. For \
a p-variate random vector to be distributed MultinormalDistribution[mu, \
sigma], mu must be a p-variate vector, and sigma must be a p x p symmetric  \
positive definite matrix."\)], "Print"]
}, Open  ]],

Cell["\<\
The RGW M-function, as described by Hull, is defined as \
follows:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(mu\  = \ {0, 0}; \nsigma[rho_]\  := \ {{1, \ rho}, \ {rho, \ 1}}\)], 
  "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["Here is a plot for zero correlation:", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Mfunc[a_, \ b_, \ rho_]\  := \ 
      CDF[MultinormalDistribution[mu, \ sigma[rho]], \ {a, \ b}]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot3D[
      Mfunc[a, \ b, \ 0], \ {a, \ \(-3\), \ 3}, \ {b, \ \(-3\), \ 3}]; \)\)], 
  "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.11022e-16 1.04977 -0.0679587 1.04977 [
[.12586 .2075 -12 -8.99448 ]
[.12586 .2075 0 .00552 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.56094 .0364 -5.16972 -9 ]
[.56094 .0364 .83028 0 ]
[.75679 .08071 0 -6.17777 ]
[.75679 .08071 12 2.82223 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.9429 .3657 0 -5.9083 ]
[.9429 .3657 6 3.0917 ]
[.02411 .26511 -6 -2.74232 ]
[.02411 .26511 0 6.25768 ]
[.01566 .3181 -24 -2.78712 ]
[.01566 .3181 0 6.21288 ]
[.00691 .37294 -18 -2.83365 ]
[.00691 .37294 0 6.16635 ]
[-0.00215 .42974 -24 -2.88201 ]
[-0.00215 .42974 0 6.11799 ]
[-0.01154 .4886 -6 -2.93231 ]
[-0.01154 .4886 0 6.06769 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.13514 .21677 m
.13978 .22141 L
s
[(-2)] .12586 .2075 1 .99877 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.56863 .04703 m
.57247 .05235 L
s
[(2)] .56094 .0364 .72324 1 Mshowa
.125 Mabswid
.18558 .19702 m
.18832 .19985 L
s
.23703 .17687 m
.23971 .17975 L
s
.28951 .15632 m
.29214 .15925 L
s
.39772 .11395 m
.40023 .11699 L
s
.4535 .09211 m
.45595 .0952 L
s
.51046 .06981 m
.51284 .07294 L
s
.08568 .23614 m
.08851 .23887 L
s
.62804 .02377 m
.63027 .02701 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.7445 .0853 m
.73835 .08759 L
s
[(-2)] .75679 .08071 -1 .37284 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.93037 .36962 m
.92411 .37158 L
s
[(2)] .9429 .3657 -1 .31296 Mshowa
.125 Mabswid
.7708 .12553 m
.7671 .12688 L
s
.79613 .16428 m
.79242 .1656 L
s
.82054 .20161 m
.81682 .20291 L
s
.86679 .27235 m
.86305 .2736 L
s
.88871 .30589 m
.88497 .30711 L
s
.9099 .3383 m
.90615 .3395 L
s
.71716 .04348 m
.71349 .04489 L
s
.95018 .39992 m
.94642 .40107 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
[(0)] .02411 .26511 1 -0.3906 Mshowa
.02792 .31343 m
.03405 .31109 L
s
[(0.25)] .01566 .3181 1 -0.38064 Mshowa
.01921 .36838 m
.02537 .3661 L
s
[(0.5)] .00691 .37294 1 -0.3703 Mshowa
.01019 .4253 m
.01637 .42308 L
s
[(0.75)] -0.00215 .42974 1 -0.35955 Mshowa
.00085 .48428 m
.00704 .48212 L
s
[(1)] -0.01154 .4886 1 -0.34838 Mshowa
.125 Mabswid
.03468 .27081 m
.03835 .26938 L
s
.03301 .28135 m
.03668 .27993 L
s
.03132 .29197 m
.035 .29056 L
s
.02963 .30266 m
.03331 .30126 L
s
.02621 .32427 m
.02989 .32287 L
s
.02448 .33518 m
.02816 .33379 L
s
.02273 .34617 m
.02642 .34479 L
s
.02098 .35724 m
.02467 .35586 L
s
.01744 .3796 m
.02113 .37824 L
s
.01564 .39091 m
.01934 .38955 L
s
.01384 .40229 m
.01754 .40094 L
s
.01202 .41375 m
.01573 .41241 L
s
.00835 .43692 m
.01206 .4356 L
s
.0065 .44864 m
.01021 .44732 L
s
.00463 .46043 m
.00834 .45912 L
s
.00274 .47231 m
.00646 .47101 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.656 .74 .911 r
.38311 .58349 .40511 .60397 .44186 .59365 .42018 .57289 Metetra
.641 .735 .917 r
.42018 .57289 .44186 .59365 .47912 .5846 .45775 .56355 Metetra
.605 .724 .929 r
.45775 .56355 .47912 .5846 .51693 .57854 .49587 .55721 Metetra
.525 .694 .945 r
.49587 .55721 .51693 .57854 .55544 .57802 .53466 .55643 Metetra
.379 .625 .949 r
.53466 .55643 .55544 .57802 .59491 .58551 .57439 .56367 Metetra
.183 .513 .914 r
.57439 .56367 .59491 .58551 .63567 .6017 .61538 .57963 Metetra
.042 .42 .865 r
.61538 .57963 .63567 .6017 .67797 .6241 .65789 .60182 Metetra
.045 .421 .866 r
.65789 .60182 .67797 .6241 .72176 .64729 .70192 .6248 Metetra
.183 .511 .913 r
.70192 .6248 .72176 .64729 .7667 .66533 .74716 .64261 Metetra
.365 .619 .949 r
.74716 .64261 .7667 .66533 .81237 .67464 .7932 .65166 Metetra
.505 .691 .952 r
.7932 .65166 .81237 .67464 .85845 .67517 .83971 .65187 Metetra
.585 .728 .942 r
.83971 .65187 .85845 .67517 .90487 .66918 .88662 .64555 Metetra
.624 .745 .934 r
.88662 .64555 .90487 .66918 .95175 .65941 .93403 .63542 Metetra
.639 .752 .93 r
.93403 .63542 .95175 .65941 .99922 .6478 .98205 .62344 Metetra
.656 .74 .911 r
.36055 .56248 .38311 .58349 .42018 .57289 .39793 .55159 Metetra
.642 .735 .917 r
.39793 .55159 .42018 .57289 .45775 .56355 .43582 .54194 Metetra
.606 .724 .929 r
.43582 .54194 .45775 .56355 .49587 .55721 .47425 .53527 Metetra
.528 .694 .945 r
.47425 .53527 .49587 .55721 .53466 .55643 .51333 .53412 Metetra
.385 .627 .949 r
.51333 .53412 .53466 .55643 .57439 .56367 .55331 .54095 Metetra
.197 .518 .916 r
.55331 .54095 .57439 .56367 .61538 .57963 .59452 .55646 Metetra
.064 .43 .872 r
.59452 .55646 .61538 .57963 .65789 .60182 .63723 .57815 Metetra
.074 .432 .873 r
.63723 .57815 .65789 .60182 .70192 .6248 .68147 .60062 Metetra
.211 .517 .915 r
.68147 .60062 .70192 .6248 .74716 .64261 .72699 .61794 Metetra
.385 .617 .943 r
.72699 .61794 .74716 .64261 .7932 .65166 .77336 .62653 Metetra
.516 .683 .941 r
.77336 .62653 .7932 .65166 .83971 .65187 .82029 .62632 Metetra
.591 .716 .931 r
.82029 .62632 .83971 .65187 .88662 .64555 .86768 .61959 Metetra
.627 .731 .922 r
.86768 .61959 .88662 .64555 .93403 .63542 .91559 .60905 Metetra
.642 .738 .918 r
.91559 .60905 .93403 .63542 .98205 .62344 .96414 .59666 Metetra
.657 .74 .911 r
.3374 .54092 .36055 .56248 .39793 .55159 .3751 .52971 Metetra
.642 .735 .916 r
.3751 .52971 .39793 .55159 .43582 .54194 .41332 .5197 Metetra
.608 .724 .928 r
.41332 .5197 .43582 .54194 .47425 .53527 .45206 .5126 Metetra
.533 .695 .943 r
.45206 .5126 .47425 .53527 .51333 .53412 .49144 .51089 Metetra
.4 .631 .948 r
.49144 .51089 .51333 .53412 .55331 .54095 .53166 .51698 Metetra
.228 .531 .921 r
.53166 .51698 .55331 .54095 .59452 .55646 .57307 .53153 Metetra
.114 .451 .884 r
.57307 .53153 .59452 .55646 .63723 .57815 .61595 .55211 Metetra
.134 .455 .885 r
.61595 .55211 .63723 .57815 .68147 .60062 .66037 .57344 Metetra
.266 .528 .914 r
.66037 .57344 .68147 .60062 .72699 .61794 .70609 .5897 Metetra
.422 .61 .927 r
.70609 .5897 .72699 .61794 .77336 .62653 .75275 .59741 Metetra
.536 .663 .919 r
.75275 .59741 .77336 .62653 .82029 .62632 .80004 .5965 Metetra
.6 .691 .906 r
.80004 .5965 .82029 .62632 .86768 .61959 .84783 .58921 Metetra
.631 .704 .898 r
.84783 .58921 .86768 .61959 .91559 .60905 .8962 .57819 Metetra
.644 .709 .895 r
.8962 .57819 .91559 .60905 .96414 .59666 .94525 .56535 Metetra
.657 .74 .911 r
.31364 .51879 .3374 .54092 .3751 .52971 .35167 .50722 Metetra
.644 .736 .916 r
.35167 .50722 .3751 .52971 .41332 .5197 .39021 .49678 Metetra
.611 .725 .926 r
.39021 .49678 .41332 .5197 .45206 .5126 .42928 .48907 Metetra
.544 .697 .94 r
.42928 .48907 .45206 .5126 .49144 .51089 .46896 .48646 Metetra
.427 .638 .944 r
.46896 .48646 .49144 .51089 .53166 .51698 .50944 .4912 Metetra
.284 .552 .925 r
.50944 .4912 .53166 .51698 .57307 .53153 .55103 .50392 Metetra
.198 .485 .897 r
.55103 .50392 .57307 .53153 .61595 .55211 .59404 .5223 Metetra
.229 .486 .893 r
.59404 .5223 .61595 .55211 .66037 .57344 .63855 .54133 Metetra
.346 .539 .9 r
.63855 .54133 .66037 .57344 .70609 .5897 .6844 .55553 Metetra
.471 .595 .895 r
.6844 .55553 .70609 .5897 .75275 .59741 .73123 .56162 Metetra
.559 .632 .88 r
.73123 .56162 .75275 .59741 .80004 .5965 .77876 .55956 Metetra
.609 .652 .868 r
.77876 .55956 .80004 .5965 .84783 .58921 .82687 .55146 Metetra
.633 .662 .86 r
.82687 .55146 .84783 .58921 .8962 .57819 .8756 .53983 Metetra
.643 .666 .857 r
.8756 .53983 .8962 .57819 .94525 .56535 .92503 .52646 Metetra
.658 .74 .91 r
.28924 .49605 .31364 .51879 .35167 .50722 .32761 .48408 Metetra
.646 .736 .915 r
.32761 .48408 .35167 .50722 .39021 .49678 .36649 .4731 Metetra
.618 .725 .923 r
.36649 .4731 .39021 .49678 .42928 .48907 .4059 .46456 Metetra
.561 .7 .934 r
.4059 .46456 .42928 .48907 .46896 .48646 .44589 .46055 Metetra
.467 .65 .938 r
.44589 .46055 .46896 .48646 .50944 .4912 .48664 .46311 Metetra
.361 .579 .924 r
.48664 .46311 .50944 .4912 .55103 .50392 .52842 .47278 Metetra
.306 .524 .901 r
.52842 .47278 .55103 .50392 .59404 .5223 .57151 .48743 Metetra
.34 .517 .885 r
.57151 .48743 .59404 .5223 .63855 .54133 .61604 .50257 Metetra
.429 .544 .871 r
.61604 .50257 .63855 .54133 .6844 .55553 .66189 .51332 Metetra
.517 .575 .853 r
.66189 .51332 .6844 .55553 .73123 .56162 .70875 .51679 Metetra
.578 .597 .835 r
.70875 .51679 .73123 .56162 .77876 .55956 .75638 .51296 Metetra
.613 .609 .823 r
.75638 .51296 .77876 .55956 .82687 .55146 .80465 .50372 Metetra
.631 .616 .817 r
.80465 .50372 .82687 .55146 .8756 .53983 .85359 .4913 Metetra
.638 .618 .814 r
.85359 .4913 .8756 .53983 .92503 .52646 .90326 .4773 Metetra
.659 .74 .91 r
.26419 .47268 .28924 .49605 .32761 .48408 .30289 .46027 Metetra
.649 .736 .913 r
.30289 .46027 .32761 .48408 .36649 .4731 .34213 .44865 Metetra
.626 .727 .919 r
.34213 .44865 .36649 .4731 .4059 .46456 .3819 .43901 Metetra
.582 .705 .927 r
.3819 .43901 .4059 .46456 .44589 .46055 .42224 .43311 Metetra
.514 .664 .929 r
.42224 .43311 .44589 .46055 .48664 .46311 .46328 .43261 Metetra
.443 .607 .916 r
.46328 .43261 .48664 .46311 .52842 .47278 .50526 .43793 Metetra
.412 .56 .892 r
.50526 .43793 .52842 .47278 .57151 .48743 .54843 .44728 Metetra
.438 .542 .865 r
.54843 .44728 .57151 .48743 .61604 .50257 .59293 .45692 Metetra
.496 .547 .838 r
.59293 .45692 .61604 .50257 .66189 .51332 .63867 .46283 Metetra
.552 .559 .815 r
.63867 .46283 .66189 .51332 .70875 .51679 .68544 .46267 Metetra
.591 .569 .797 r
.68544 .46267 .70875 .51679 .75638 .51296 .73302 .45645 Metetra
.615 .576 .787 r
.73302 .45645 .75638 .51296 .80465 .50372 .7813 .44574 Metetra
.627 .579 .781 r
.7813 .44574 .80465 .50372 .85359 .4913 .83029 .43237 Metetra
.632 .581 .779 r
.83029 .43237 .85359 .4913 .90326 .4773 .88005 .41765 Metetra
.66 .74 .909 r
.23845 .44867 .26419 .47268 .30289 .46027 .2775 .43578 Metetra
.653 .737 .911 r
.2775 .43578 .30289 .46027 .34213 .44865 .3171 .42343 Metetra
.636 .729 .915 r
.3171 .42343 .34213 .44865 .3819 .43901 .35725 .41252 Metetra
.605 .711 .919 r
.35725 .41252 .3819 .43901 .42224 .43311 .39797 .40437 Metetra
.56 .678 .918 r
.39797 .40437 .42224 .43311 .46328 .43261 .43936 .40022 Metetra
.515 .633 .905 r
.43936 .40022 .46328 .43261 .50526 .43793 .48159 .40036 Metetra
.496 .591 .879 r
.48159 .40036 .50526 .43793 .54843 .44728 .52489 .40341 Metetra
.511 .565 .847 r
.52489 .40341 .54843 .44728 .59293 .45692 .56936 .40654 Metetra
.544 .555 .816 r
.56936 .40654 .59293 .45692 .63867 .46283 .61497 .40676 Metetra
.577 .554 .792 r
.61497 .40676 .63867 .46283 .68544 .46267 .66158 .40237 Metetra
.602 .557 .776 r
.66158 .40237 .68544 .46267 .73302 .45645 .70902 .3934 Metetra
.617 .559 .767 r
.70902 .3934 .73302 .45645 .7813 .44574 .75722 .38102 Metetra
.624 .56 .762 r
.75722 .38102 .7813 .44574 .83029 .43237 .80617 .3666 Metetra
.628 .561 .761 r
.80617 .3666 .83029 .43237 .88005 .41765 .85591 .35111 Metetra
.661 .74 .909 r
.21199 .42398 .23845 .44867 .2775 .43578 .25139 .4106 Metetra
.656 .738 .91 r
.25139 .4106 .2775 .43578 .3171 .42343 .29137 .3975 Metetra
.646 .732 .912 r
.29137 .3975 .3171 .42343 .35725 .41252 .33192 .3853 Metetra
.626 .718 .913 r
.33192 .3853 .35725 .41252 .39797 .40437 .37306 .37487 Metetra
.598 .694 .909 r
.37306 .37487 .39797 .40437 .43936 .40022 .41484 .36703 Metetra
.571 .658 .896 r
.41484 .36703 .43936 .40022 .48159 .40036 .4574 .36196 Metetra
.558 .621 .871 r
.4574 .36196 .48159 .40036 .52489 .40341 .5009 .35871 Metetra
.563 .592 .84 r
.5009 .35871 .52489 .40341 .56936 .40654 .54543 .35538 Metetra
.58 .574 .811 r
.54543 .35538 .56936 .40654 .61497 .40676 .59098 .35 Metetra
.598 .566 .789 r
.59098 .35 .61497 .40676 .66158 .40237 .63747 .34145 Metetra
.612 .562 .774 r
.63747 .34145 .66158 .40237 .70902 .3934 .68481 .32977 Metetra
.621 .562 .766 r
.68481 .32977 .70902 .3934 .75722 .38102 .73293 .31575 Metetra
.626 .562 .762 r
.73293 .31575 .75722 .38102 .80617 .3666 .78184 .30028 Metetra
.628 .562 .76 r
.78184 .30028 .80617 .3666 .85591 .35111 .83157 .284 Metetra
.662 .741 .908 r
.18478 .39861 .21199 .42398 .25139 .4106 .22454 .38473 Metetra
.659 .739 .909 r
.22454 .38473 .25139 .4106 .29137 .3975 .26491 .37093 Metetra
.653 .735 .909 r
.26491 .37093 .29137 .3975 .33192 .3853 .30587 .35756 Metetra
.642 .726 .909 r
.30587 .35756 .33192 .3853 .37306 .37487 .34744 .34513 Metetra
.626 .708 .905 r
.34744 .34513 .37306 .37487 .41484 .36703 .38966 .33412 Metetra
.611 .682 .893 r
.38966 .33412 .41484 .36703 .4574 .36196 .43262 .32461 Metetra
.602 .652 .872 r
.43262 .32461 .4574 .36196 .5009 .35871 .47641 .31605 Metetra
.602 .625 .847 r
.47641 .31605 .5009 .35871 .54543 .35538 .52112 .30731 Metetra
.609 .605 .822 r
.52112 .30731 .54543 .35538 .59098 .35 .56675 .29724 Metetra
.617 .593 .803 r
.56675 .29724 .59098 .35 .63747 .34145 .61327 .28519 Metetra
.625 .586 .79 r
.61327 .28519 .63747 .34145 .68481 .32977 .66063 .27121 Metetra
.63 .583 .783 r
.66063 .27121 .68481 .32977 .73293 .31575 .70879 .25575 Metetra
.633 .582 .78 r
.70879 .25575 .73293 .31575 .78184 .30028 .75777 .23932 Metetra
.634 .582 .778 r
.75777 .23932 .78184 .30028 .83157 .284 .80758 .22229 Metetra
.663 .741 .908 r
.1568 .37252 .18478 .39861 .22454 .38473 .19692 .35817 Metetra
.661 .74 .908 r
.19692 .35817 .22454 .38473 .26491 .37093 .23767 .34374 Metetra
.658 .737 .908 r
.23767 .34374 .26491 .37093 .30587 .35756 .27904 .3294 Metetra
.653 .732 .907 r
.27904 .3294 .30587 .35756 .34744 .34513 .32105 .31543 Metetra
.645 .721 .904 r
.32105 .31543 .34744 .34513 .38966 .33412 .36373 .30205 Metetra
.636 .704 .895 r
.36373 .30205 .38966 .33412 .43262 .32461 .40713 .2893 Metetra
.631 .683 .881 r
.40713 .2893 .43262 .32461 .47641 .31605 .45131 .2769 Metetra
.629 .662 .863 r
.45131 .2769 .47641 .31605 .52112 .30731 .49632 .26426 Metetra
.632 .645 .845 r
.49632 .26426 .52112 .30731 .56675 .29724 .54219 .2508 Metetra
.635 .632 .83 r
.54219 .2508 .56675 .29724 .61327 .28519 .5889 .2362 Metetra
.639 .625 .82 r
.5889 .2362 .61327 .28519 .66063 .27121 .63644 .22047 Metetra
.641 .621 .815 r
.63644 .22047 .66063 .27121 .70879 .25575 .68481 .20384 Metetra
.643 .62 .812 r
.68481 .20384 .70879 .25575 .75777 .23932 .73401 .18658 Metetra
.643 .619 .811 r
.73401 .18658 .75777 .23932 .80758 .22229 .78407 .16885 Metetra
.663 .741 .908 r
.12799 .34569 .1568 .37252 .19692 .35817 .16848 .33087 Metetra
.663 .741 .908 r
.16848 .33087 .19692 .35817 .23767 .34374 .20961 .31588 Metetra
.661 .739 .908 r
.20961 .31588 .23767 .34374 .27904 .3294 .2514 .30078 Metetra
.659 .736 .907 r
.2514 .30078 .27904 .3294 .32105 .31543 .29385 .2857 Metetra
.655 .73 .905 r
.29385 .2857 .32105 .31543 .36373 .30205 .33698 .27074 Metetra
.651 .721 .899 r
.33698 .27074 .36373 .30205 .40713 .2893 .38084 .25589 Metetra
.648 .709 .891 r
.38084 .25589 .40713 .2893 .45131 .2769 .42546 .24104 Metetra
.647 .695 .881 r
.42546 .24104 .45131 .2769 .49632 .26426 .47087 .22592 Metetra
.648 .683 .87 r
.47087 .22592 .49632 .26426 .54219 .2508 .5171 .21028 Metetra
.649 .674 .861 r
.5171 .21028 .54219 .2508 .5889 .2362 .56416 .19397 Metetra
.651 .669 .854 r
.56416 .19397 .5889 .2362 .63644 .22047 .61205 .17699 Metetra
.652 .666 .85 r
.61205 .17699 .63644 .22047 .68481 .20384 .66077 .15945 Metetra
.653 .664 .849 r
.66077 .15945 .68481 .20384 .73401 .18658 .71036 .14145 Metetra
.653 .664 .848 r
.71036 .14145 .73401 .18658 .78407 .16885 .76082 .12305 Metetra
.663 .741 .908 r
.09834 .31807 .12799 .34569 .16848 .33087 .1392 .30279 Metetra
.663 .741 .908 r
.1392 .30279 .16848 .33087 .20961 .31588 .18071 .28728 Metetra
.663 .74 .908 r
.18071 .28728 .20961 .31588 .2514 .30078 .2229 .27157 Metetra
.662 .739 .907 r
.2229 .27157 .2514 .30078 .29385 .2857 .26578 .2557 Metetra
.66 .736 .906 r
.26578 .2557 .29385 .2857 .33698 .27074 .30937 .23969 Metetra
.659 .732 .903 r
.30937 .23969 .33698 .27074 .38084 .25589 .35369 .22356 Metetra
.658 .726 .899 r
.35369 .22356 .38084 .25589 .42546 .24104 .39878 .20724 Metetra
.657 .719 .894 r
.39878 .20724 .42546 .24104 .47087 .22592 .44465 .19064 Metetra
.657 .713 .889 r
.44465 .19064 .47087 .22592 .5171 .21028 .49132 .17365 Metetra
.658 .708 .884 r
.49132 .17365 .5171 .21028 .56416 .19397 .53883 .15622 Metetra
.658 .704 .881 r
.53883 .15622 .56416 .19397 .61205 .17699 .58717 .13834 Metetra
.659 .703 .879 r
.58717 .13834 .61205 .17699 .66077 .15945 .63638 .12004 Metetra
.659 .702 .878 r
.63638 .12004 .66077 .15945 .71036 .14145 .68646 .10135 Metetra
.659 .701 .878 r
.68646 .10135 .71036 .14145 .76082 .12305 .73744 .08231 Metetra
.664 .741 .908 r
.0678 .28964 .09834 .31807 .1392 .30279 .10903 .27388 Metetra
.663 .741 .908 r
.10903 .27388 .1392 .30279 .18071 .28728 .15092 .25787 Metetra
.663 .741 .908 r
.15092 .25787 .18071 .28728 .2229 .27157 .19352 .24161 Metetra
.663 .74 .908 r
.19352 .24161 .2229 .27157 .26578 .2557 .23683 .22511 Metetra
.663 .739 .907 r
.23683 .22511 .26578 .2557 .30937 .23969 .28086 .20837 Metetra
.662 .738 .906 r
.28086 .20837 .30937 .23969 .35369 .22356 .32565 .1914 Metetra
.662 .735 .904 r
.32565 .1914 .35369 .22356 .39878 .20724 .37121 .17415 Metetra
.661 .732 .902 r
.37121 .17415 .39878 .20724 .44465 .19064 .41757 .15661 Metetra
.661 .73 .9 r
.41757 .15661 .44465 .19064 .49132 .17365 .46474 .13873 Metetra
.661 .728 .898 r
.46474 .13873 .49132 .17365 .53883 .15622 .51275 .12048 Metetra
.662 .726 .897 r
.51275 .12048 .53883 .15622 .58717 .13834 .56162 .10187 Metetra
.662 .725 .896 r
.56162 .10187 .58717 .13834 .63638 .12004 .61137 .08288 Metetra
.662 .725 .896 r
.61137 .08288 .63638 .12004 .68646 .10135 .66202 .06354 Metetra
.662 .725 .896 r
.66202 .06354 .68646 .10135 .73744 .08231 .7136 .04383 Metetra
.664 .741 .908 r
.03634 .26033 .0678 .28964 .10903 .27388 .07793 .24409 Metetra
.664 .741 .908 r
.07793 .24409 .10903 .27388 .15092 .25787 .12022 .22757 Metetra
.663 .741 .908 r
.12022 .22757 .15092 .25787 .19352 .24161 .16322 .21079 Metetra
.663 .741 .908 r
.16322 .21079 .19352 .24161 .23683 .22511 .20695 .19372 Metetra
.663 .741 .908 r
.20695 .19372 .23683 .22511 .28086 .20837 .25143 .17637 Metetra
.663 .74 .907 r
.25143 .17637 .28086 .20837 .32565 .1914 .29669 .15874 Metetra
.663 .739 .907 r
.29669 .15874 .32565 .1914 .37121 .17415 .34273 .1408 Metetra
.663 .738 .906 r
.34273 .1408 .37121 .17415 .41757 .15661 .38959 .12254 Metetra
.663 .737 .905 r
.38959 .12254 .41757 .15661 .46474 .13873 .43728 .10396 Metetra
.663 .737 .905 r
.43728 .10396 .46474 .13873 .51275 .12048 .48583 .08502 Metetra
.663 .736 .904 r
.48583 .08502 .51275 .12048 .56162 .10187 .53526 .06573 Metetra
.663 .736 .904 r
.53526 .06573 .56162 .10187 .61137 .08288 .58559 .04609 Metetra
.663 .736 .904 r
.58559 .04609 .61137 .08288 .66202 .06354 .63685 .02607 Metetra
.663 .736 .904 r
.63685 .02607 .66202 .06354 .7136 .04383 .68907 .00568 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.13514 .21677 m
.13978 .22141 L
s
[(-2)] .12586 .2075 1 .99877 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.56863 .04703 m
.57247 .05235 L
s
[(2)] .56094 .0364 .72324 1 Mshowa
.125 Mabswid
.18558 .19702 m
.18832 .19985 L
s
.23703 .17687 m
.23971 .17975 L
s
.28951 .15632 m
.29214 .15925 L
s
.39772 .11395 m
.40023 .11699 L
s
.4535 .09211 m
.45595 .0952 L
s
.51046 .06981 m
.51284 .07294 L
s
.08568 .23614 m
.08851 .23887 L
s
.62804 .02377 m
.63027 .02701 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  ImageSize->{240, 194.438},
  ImageMargins->{{18, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003`0000`Q000`40O003h00Ol7oo003`Ool00?1o
o`00l7oo003`Ool00?1oo`00l7oo003`Ool00?1oo`00l7oo002[Ool20013Ool008aoo`@001Qoo`<0
0003Ool007oo049oo`00SGoo00<007ooOol05Woo0P000goo0`00@Woo002>Ool00`00Oomoo`0BOol3
0002Ool3000015Kl001oo`00@Goo002?Ool00`00Oomoo`0?Ool20002Ool30004E_`30011Ool008ao
o`04001oogoo000>Ool3000017oo000000001eKl00@005Kl0000041oo`00SGoo0P0037oo0`0000Ao
o`00000000YFo005001Fo000Ool0000oOol009Uoo`80009oo`<00003E_`005Kl00UFo003001Fo5Kl
008003moo`00UWoo0`0000Aoo`00000000EFo003001Fo5Kl00MFo006001Fo5Kl001oo`00?Woo002D
Ool2000017oo000000002EKl00<005KlE_`01UKl00L005KlE_aFo000Ool0000mOol0095oo`<00003
Ool0000000aFo003001Fo5Kl00MFo003001Fo5Kl00<003eoo`00Sgoo0P0000Ioo`00000005Kl000=
E_`00`00E_aFo006E_`3000015Kl001oo`00?7oo002<Ool600000eKl001Fo002E_`00`00E_aFo00:
E_`00`00E_aFo003E_`40004E_`00`00Ool0000kOol008Yoo`D000UFo003001Fo5Kl00YFo003001F
o000008000=Fo003001Fo5Kl00=Fo08003]oo`00Qgoo1@0035Kl00<005KlE_`02UKl0P001UKl00<0
05KlE_`00eKl00<007oo00002goo1000:goo0025Ool200001Goo000005Kl00003EKl00<005KlE_`0
1UKl0`0000=Fo000E_`01EKl00<005KlE_`015Kl0P0037oo00<007ooOol0:goo0022Ool60004E_`0
0`00E_aFo00:E_`00`00E_aFo003E_`30005E_`00`00E_aFo003E_`00`00E_aFo004E_`00`00Ool0
0004Ool50003Ool00`00Oomoo`0ZOol007moo`<00003Ool0000000QFo003001Fo5Kl00YFo003001F
o000008000UFo003001Fo5Kl009Fo003001Fo5Kl00EFo003001oo`0000aoo`03001oogoo02Uoo`00
OGoo0P000Woo0P002UKl00<005KlE_`02UKl0P0035Kl00<005KlE_`00UKl00<005KlE_`01EKl0`00
2Goo00@007ooOol002]oo`00NWoo0`000Woo0P003EKl00<005KlE_`01UKl0`0000=Fo000E_`035Kl
00D005KlE_aFo00000AFo0<00004E]`000000009Ool2000/Ool005aoo`8001Yoo`80009oo`<00003
E_`005Kl00aFo003001Fo5Kl00=Fo0<000EFo003001Fo5Kl00YFo006001Fo5KlE_`005Kl0`001EKL
00<007oo0000=Woo001KOol01000Oomoo`005Woo0`000Woo0P001EKl00<005KlE_`02eKl00<005Kl
00000P002EKl00<005KlE_`02UKl00<005Kl00000P0025KL00<007oo0000=Woo001KOol01000Oomo
o`0057oo0P000Woo0`0025Kl00<005KlE_`02UKl0P0035Kl00<005KlE_`02UKl0P0000=Fg000E]`0
2EKL00<007oo0000=Goo001KOol01000Oomoo`004Goo0`000Woo0P0035Kl00<005KlE_`01UKl0`00
00=Fo000E_`035Kl00<005KlE_`01UKl0`0000AFg000E]`000]Fg003001oo`0003Aoo`00Fgoo00@0
07ooOol000moo`80009oo`<00003E_`005Kl00aFo003001Fo5Kl00=Fo08000EFo003001Fo5Kl00YF
o003001Fo5Kl00=Fo0<000EFg08000]Fg003001oo`0003Aoo`00G7oo0P003Goo100000=oo`000000
1EKl00<005KlE_`02eKl00<005Kl00000P0025Kl00<005KlE_`02UKl00<005Kl00000P0025KL0P00
35KL00<007oo0000<goo001YOol20002Ool30008E_`00`00E_aFo00:E_`2000<E_`00`00E_aFo009
E_`2000<E]`00`00E]aFg00:E]`01000Oomoo`00<Woo001VOol30002Ool2000<E_`00`00E_aFo006
E_`300000eKl001Fo00;E_`00`00E_aFo006E_`300000eKL001Fg00<E]`00`00E]aFg009E]`20000
0goo001oo`0aOol006=oo`<0009oo`<00003E_`005Kl00aFo003001Fo5Kl00=Fo08000EFo003001F
o5Kl00YFo003001Fo5Kl00=Fo08000EFg003001Fg5KL00YFg003001Fg5KL00IFg0<00003D[/007oo
0080035oo`00HGoo0P000Woo0`001EKl00<005KlE_`02eKl00<005Kl00000P0025Kl00<005KlE_`0
2UKl00<005Kl00000P0025KL00<005KLE]`02EKL0P0015KL0`001E:k00@007ooOol0031oo`00GWoo
0`0000Aoo`00000000UFo003001Fo5Kl00YFo08000aFo003001Fo5Kl00UFo08000]Fg003001Fg5KL
00UFg0800004E]`000000008D[/01000Oomoo`00<7oo001LOol20002Ool2000=E_`00`00E_aFo006
E_`300000eKl001Fo00;E_`00`00E_aFo006E_`300000eKL001Fg00;E]`00`00E]aFg008E]`3000<
D[/01000Oomoo`00;goo001IOol3000017oo000000000UKm00<005KlE_`02eKl00<005KlE_`00eKl
0P001EKl00<005KlE_`02UKl00<005KlE_`00eKl0P001EKL00<005KLE]`02UKL00<005KLE]`01EKL
0`0000=B^`00D[/02e:k00D007ooOomoo`0002ioo`00Egoo0P0000Aoo`00000000IFo@03001Fo5Kl
00]Fo003001Fo000008000QFo003001Fo5Kl00YFo003001Fo000008000QFg003001Fg5KL00UFg003
001Fg5KL00=Fg080009B^`04001B^e:k000<D[/01000Oomoo`00;Woo001DOol6000:E_d00`00E_aF
o00:E_`2000;E_`00`00E_aFo00:E_`2000;E]`00`00E]aFg00:E]`00`00E]`000020004D[/01000
D[]B^`0035:k00D007ooOomoo`0002eoo`00DWoo1`0035Km00<005KlE_`01UKl0`0000=Fo000E_`0
2eKl00<005KlE_`01UKl0`0000=Fg000E]`02eKL00<005KLE]`025KL0`001e:k00D005:kD[]B^`00
00aB^`05001oogooOol0000/Ool004moo`D000EFo@03001FoEKm00YFo@03001Fo5Kl00=Fo08000EF
o003001Fo5Kl00UFo003001Fo5Kl00=Fo0<000EFg003001Fg5KL00UFg003001Fg5KL00AFg0@0009B
^`03001B^e:k00EB^`03001B^e:k009B^`03001B^e:k00UB^`03001oogoo008002aoo`00CGoo1@00
25Km00<005KmE_d02UKm00<005Kl00000P0025Kl00<005KlE_`02EKl00<005Kl00000P002EKL00<0
05KLE]`02EKL00<005KLE]`00`001e:k00<005:kD[/015:k00<005:kD[/00U:k00<005:kD[/025:k
0`000goo00<007ooOol0:Goo000^Ool4000HOol5000<E_d00`00E_eFo@08E_d3000<E_`00`00E_aF
o007E_`3000<E]`00`00E]aFg009E]`3000:D[/00`00D[]B^`04D[/00`00D[]B^`03D[/00`00D[]B
^`03D[/40002DWT01@00Oomoogoo0000:goo000_Ool00`00Oomoo`0EOol600000eKm001Fo@0<E_d0
0`00E_eFo@04E_d30002E_`00`00E_aFo00:E_`00`00E_aFo004E_`30002E]`00`00E]aFg00;E]`0
0`00E]aFg005E]`300000e:k001B^`0;D[/00`00D[]B^`03D[/00`00D[]B^`03D[/00`00D[]B^`03
0007DWT01@00Oomoogoo0000:Woo000XOol50003Ool00`00Oomoo`0BOol200000goo00000005E_d0
0`00E_eFo@0;E_d00`00E_eFo@030006E_`00`00E_aFo00:E_`00`00E_aFo0030006E]`00`00E]aF
g00:E]`00`00E]aFg003E]`20005D[/00`00D[]B^`0:D[/00`00D[]B^`02D[/00`00D[]B^`03D[/3
000:DWT00`00Oomoo`02Ool00`00Oomoo`0WOol0035oo`03001oogoo00ioo`H000QFo@03001FoEKm
00YFo@<000YFo003001Fo5Kl00UFo0<000YFg003001Fg5KL00YFg003001Fg000008000QB^`03001B
^e:k00UB^`03001B^e:k009B^`03001B^e:k00<00003DWT0059i00]BN@05001oogooOol0000YOol0
02ioo`04001oogoo000>Ool200000goo0000000<E_d00`00E_eFo@06E_d300000eKl001Fo00;E_`0
0`00E_aFo006E_`200000eKl001Fg00;E]`00`00E]aFg008E]`3000;D[/00`00D[]B^`0:D[/01000
D[]B^e:k0`001E9i00<0059iDWT02E9i00<007ooOol00Woo00<007ooOol09Woo000_Ool2000<Ool6
0002E_d00`00E_eFo@0:E_d00`00E_eFo@03E_d20005E_`00`00E_aFo00:E_`00`00E_aFo002E_`3
0004E_`00`00E]aFg00:E]`00`00E]aFg004E]`30002D[/00`00D[]B^`0:D[/00`00D[]B^`09D[/0
0`00D[/000020007DWT00`00DWUBN@0:DWT00`00Oomoo`02Ool00`00Oomoo`0UOol003]oo`800004
Ool000000005E_d00`00E_eFo@0:E_d00`00E_d000020008E_`00`00E_aFo00:E_`01000E_`00000
25Kl00<005KLE]`02UKL00<005KLE]`00`001U:k00<005:kD[/02E:k00<005:kD[/02E:k0P000U9i
00<0059iDWT01U9i00<0059iDWT02E9i00@007ooOomoo`8002Moo`00>7oo0`0000=oo`0000002EKm
00<005KmE_d02EKm0P0035Kl00<005KlE_`025Kl0`002eKl00<005KLE]`02EKL0`002E:k00<005:k
D[/02U:k00<005:kD[/01E:k0`0000=BNP00DWT00U9i00<0059iDWT01E9i00<0059iDWT02E9i00<0
07ooOol00goo00<007ooOol097oo000fOol20002Ool2000<E_d00`00E_eFo@06E_d200000eKm001F
o00<E_`00`00E_aFo004E_`30002E_`00`00E_aFo009E_`00`00E]aFg006E]`300000e:k001B^`0:
D[/00`00D[]B^`0:D[/00`00D[]B^`02D[/20005DWX01000DWUBN@00259i00<0059iDWT02E9i00<0
07ooOol00goo00<007ooOol08goo000cOol3000017oo000000000UKm00<005KmE_d02UKm00<005Km
E_d00UKm0`0015Km00<005KlE_`02eKl00<005KlE_`00`001UKl00<005KlE_`02EKl00<005KLE]`0
0eKL0P001E:k00<005:kD[/02E:k00<005:kD[/02E:k00<005:k00000P001e9j00@0059iDWT000QB
N@03001BNE9i00UBN@03001oogoo00=oo`03001oogoo00Eoo`8001aoo`00<Goo0P0000Aoo`000000
00IFo@03001FoEKm00YFo@04001Fo@000008E_d00`00E_aFo00:E_`3000:E_`00`00E_aFo009E_`0
0`00E]`000020007D[/00`00D[]B^`09D[/00`00D[]B^`08D[/3000;DWX00`00DWT00009DWT00`00
DWUBN@08DWT20005Ool00`00Oomoo`03Ool01000Oomoo`006goo000^Ool6000:E_d00`00E_eFo@08
E_d3000;E_d00`00E_aFo006E_`300000eKl001Fo00;E_`00`00E_aFo007E_`3000;D[/00`00D[]B
^`09D[/00`00D[]B^`03D[/40002DWX00`00DWYBNP09DWX00`00DWT00009DWT00`00DWUBN@05DWT3
00000e9H001oo`05Ool00`00Oomoo`02Ool01000Oomoo`006goo000[Ool300001Goo000005Km0000
35Km00<005KmE_d015Km0`000eKm00<005KmE_d02EKm00<005KlE_`00eKl0P001EKl00<005KlE_`0
2EKl00<005KlE_`015Kl0`000U;L00<005:kD[/02U:k00<005:kD[/025:k00<005:kD[/00`001e9j
00<0059jDWX02E9j0P002U9i00<0059iDWT00U9i0P001E9H00<007ooOol00goo00<007ooOol00Woo
00@007ooOol001]oo`00:Goo0P0000Aoo`00000000AFo@03001FoEKm00YFo@03001FoEKm00<000MF
o@03001FoEKm00UFo@03001Fo000008000QFo003001Fo5Kl00UFo003001Fo5Kl00<000IBg003001B
^e:k00UB^`03001B^e:k00MB^`@000YBNP03001BNU9j00UBNP8000YBN@03001BN@00008000MBF003
001oogoo00Aoo`05001oogooOol00002Ool00`00Oomoo`0IOol002Ioo`<00003Ool0000000QFo@03
001FoEKm00UFo@<000]Fo@03001FoEKm00QFo@8000aFo003001Fo5Kl00QFo0<000YBg003001B^e:k
00UB^`03001B^e:k00=B^`<0009BVP03001BNU9j00YBNP03001BNU9j00QBNP8000UBN@<000YBF003
001oogoo00=oo`03001oo`0000=oo`8001aoo`0097oo0P000Woo0P002eKm00<005KmE_d01UKm0P00
0UKm00<005KmE_d02UKm00<005KmE_d015Km0`000UKl00<005KlE_`02UKl00<005KlE_`015Kl0`00
00=Bg000D]`02U;L00<005:kD[/02E:k00<005:kD[/00`001U:J00<0059jDWX02E9j00<0059jDWX0
259j0P001E9i10000U9H00<0059HDUP02E9H00<007ooOol017oo00<007ooOol07goo000QOol30000
17oo000000000UKm00<005KmE_d02EKm00<005KmE_d00UKm0`001EKm00<005KmE_d02UKm00<005Km
E_d00`001UKl00<005KlE_`02UKl00<005KlE_`00`001E;L00<005;LD]`02E;L00<005:kD[/01e:k
10002E:J00<0059jDWX02U9j00<0059jDWX0159j0`0000ABNP00DWUBN@<000MBF003001BF59H00QB
F003001oogoo00Eoo`03001oogoo01ioo`0087oo1@001UKm00<005KmE_d02EKm00@005Km000000UF
o@03001FoEKm00UFo@<000YFo003001Fo5Kl00QFo0@000UBg003001Bg5;L00QBg003001B^e:k00AB
^`<0009BV`03001BVU:J00UBVP03001BNU9j00UBNP04001BNU9jDWX30004DWX3000:DUP00`00DUQB
F009DUP00`00Oomoo`05Ool00`00Oomoo`0MOol0021oo`8000YFo@03001FoEKm00QFo@<000]Fo@03
001FoEKm00IFo@800003E_d005Kl00]Fo003001Fo5Kl00AFo0<0009Bo003001Bg5;L00UBg003001B
g5;L00QBg003001B^e:k00<000IBV`03001BVU:J00QBVP03001BNU9j00UBNP@000ABNP<00003DUT0
000000YBF003001BF59H00UBF003001oogoo00Eoo`03001oogoo01eoo`006Goo0P001Goo0P002eKm
00<005KmE_d015Km0`000eKm00<005KmE_d02EKm00<005KmE_d00UKm0`0015Km00<005KlE_`02UKl
00<005KlE_`00`001U;l00<005;LD]`025;L00<005;LD]`025;L0`002U:K00<005:JDYX025:J00<0
059jDWX01U9j1000159j0P001E9I0P002U9H00<0059HDUP0259H00<007ooOol01Woo00<007ooOol0
77oo000HOol01000Oomoo`000goo0P0000=oo`00E_d02UKm00<005KmE_d00`001eKm00<005KmE_d0
2EKm00@005Km000000QFo@03001Fo5Kl00UFo0<000YBo003001Bg5;L00QBg003001Bg5;L00ABg0<0
0003D[/005:K00YBV`03001BVU:J00UBVP03001BNU9j009BNP<000=BNP03001BNP00008000MBF@80
00YBF003001BF59H00UBF003001oogoo00Ioo`03001oogoo01]oo`0067oo00@007ooOol000=oo`05
001oogooOol0000:E_d3000;E_d00`00E_eFo@08E_d2000;E_d00`00E_aFo006E_`200000e;l001B
o00;D_`00`00D]aBg008D]`00`00D]aBg0030005D[/00`00DY]BV`09DY/00`00DYYBVP08DYX00`00
DWX000020005DWX3000:DUT00`00DUP0000:DUP00`00DUQBF008DUP00`00Oomoo`05Ool2000MOol0
01Qoo`04001oogoo0003Ool20003Ool00`00E_eFo@05E_d20003E_d00`00E_eFo@09E_d00`00E_eF
o@04E_d30002E_d00`00E_eFo@09E_d00`00E_aFo002E_`30004D_`00`00D_aBo009D_`00`00D]aB
g007D]`40009D[/00`00DY]BV`08DY/00`00DYYBVP07DYX30005DWX30002CUT00`00DUUBF@09DUT0
0`00DUP00009DUP00`00DUQBF008DUP00`00Oomoo`07Ool00`00Oomoo`0JOol001Qoo`04001oogoo
0003Ool00`00Oomoo`03Ool00`00E_eFo@02E_d20006E_d00`00E_eFo@09E_d00`00E_eFo@030006
E_d00`00E_eFo@09E_d01000E_`0000025;l00<005;lD_`02E;l00@005;LD]aBg0D0009B_003001B
^e:k00QB^`03001BVe:K00UBV`03001BVU:J00=BVP<0009BVP03001BNU9j00<000I>F@03001BFE9I
00QBF@03001BF00000UBF003001BF59H00QBF08000Qoo`03001oogoo01Yoo`006Goo0P0017oo00<0
07ooOol017oo00@005Km000000UFo@03001FoEKm00QFo@<000YFo@03001FoEKm00QFo@8000]Bo003
001Bo5;l00QBo0@000QB_003001B^e:k00QB^`03001BVe:K00UBV`03001BVP00008000ABVP@000U>
F@03001BFE9I00UBF@03001BF00000UBF003001BF59H00ABF0<00003DSP007oo00Qoo`03001oogoo
01Uoo`007goo00<007ooOol01Goo00<005KmE_d02UKm00<005KmE_d015Km0`000UKm00<005KmE_d0
2EKm00<005KmE_d015Km0`000UKm00<005;lD_`02E;l00<005;lD_`015;l0`0000=Bg000D[`02U:l
00<005:kD[/025:k00<005:KDY/025:K0P000U:J1@000TiI00<004iICUT024iI00<0059IDUT02E9I
00@0059HDUP000QBF003001BF59H009BF08000EB>003001oogoo00Moo`03001oogoo01Qoo`007goo
00<007ooOol01Woo00<005KmE_d02UKm00<005KmE_d00`001UKm00<005KmE_d02EKm00<005KmE_d0
0`001UKm00<005;lD_`02E;l00<005;lD_`00`001E;L00<005:lD[`025:l00<005:kD[/025:k00<0
05:KDY/01E:K1`0024iI00<004iICUT024iI00<0059IDUT0259I00D0059HDUQBF00000MBF003001B
F000008000MB>003001oogoo00Moo`03001oogoo01Qoo`007Woo0P002Goo00<005KmE_d02EKm0`00
2UKm00<005KmE_d025Km0`002UKm00<005;lD_`025;l0`002E;L00<005:lD[`025:l00<005:kD[/0
25:k00@005:KDY]BV`D0009>NP03001>FDiI00Q>F@03001>FDiI00Q>F@03001BFE9I00UBF@04001B
F59H0006DUP3000:DSP00`00Oomoo`08Ool00`00Oomoo`0GOol001ioo`03001oogoo00Uoo`03001F
oEKm00IFo@80009Fo@03001FoEKm00UFo@03001FoEKm00AFo@<0009Fo@03001FoEKm00UFo@03001B
o5;l00ABo0<00003D_`005;L00YBg003001B_5:l00MB_003001B^e:k00MB^`D000Q>NP03001>FDiI
00M>F@03001>FDiI00U>F@03001BFE9I00QBF@06001BF59HDUP0059H10000U8h00<0058hDSP02E8h
00<007ooOol01goo0P0067oo000NOol2000;Ool00`00E_eFo@02E_d30005E_d00`00E_eFo@09E_d0
0`00E_eFo@030006E_d00`00E_eFo@09E_d00`00D_aBo0030005D_`00`00D]aBg008D]`00`00D[aB
_008D[`01000D[]B^e:k20002Tij00<004iICUT024iI00<004iICUT024iI00<0059IDUT0259I00<0
059HDUP00`001e8h00<0058hDSP0258h00<007ooOol027oo00<007ooOol05Woo000NOol00`00Oomo
o`0;Ool01000E_d000002EKm00<005KmE_d025Km0`002UKm00<005KmE_d025Km0`002E;l00<005;L
D]`025;L00<005:lD[`01U:l1@0000A>N`00000000A>N`03001>NTij00Q>NP03001>FDiI00M>F@03
001>FDiI00U>F@03001BFE9I00IBF@@000=B>003001B>58h00AB>003001B>58h00UB>003001oogoo
00Qoo`03001oogoo01Eoo`001Goo0P0017oo0P000goo10000Woo0`001Goo00<007ooOol037oo00<0
05KmE_d02UKm00<005KmE_d015Km0`000UKm00<005KmE_d02EKm00<005KmE_d015Km0`000U;m00<0
05;lD_`025;l00<005;LD]`025;L00<005:lD[`01@000TjK10001dik00<004ijCWX024ij00<004iI
CUT024iI00<004iICUT024iI00<0059IDUT00e9I0`000Thh00<0058hDSP00U8h00<0058hDSP0158h
00<0058hDSP02E8h00<007ooOol02Goo00<007ooOol057oo0004Ool01000Oomoo`000goo0P0017oo
00<007ooOol01Goo00<007ooOol00Woo00<007ooOol03Goo00<005KmE_d02UKm00<005KmE_d00`00
1UKm00<005KmE_d02EKm00<005KmE_d00`001U;m00<005;lD_`025;l00<005;LD]`01U;L10001DjK
10002Tik00<004ijCWX024ij00<004iICUT01diI00<004iICUT024iI00<0059IDUT00`001Thh00<0
058hDSP00U8h00<0058hDSP0158h00<0058hDSP0258h00<007ooOol02Goo00<007ooOol057oo0004
Ool01000Oomoo`002Woo00<007ooOol017oo00<007ooOol00Woo00<007ooOol03Woo00<005KmE_d0
2EKm0`002UKm00<005KmE_d025Km0`002U;m00<005;lD_`025;l00<005;LD]`01@000Tjl00@004jK
CY]>V`<000=>V`03001>Ndik00Q>N`03001>NTij00Q>NP03001>FDiI00Q>F@03001>FDiI00I>F@@0
00U>>003001B>58h00=B>003001B>58h00=B>003001B>58h00UB>003001oogoo00Uoo`03001oogoo
01=oo`0017oo00@007ooOol000]oo`03001oogoo00<000Aoo`80015oo`03001FoEKm00IFo@80009F
o@03001FoEKm00UFo@03001FoEKm00AFo@<0009Fo@03001BoE;m00UBo@03001Bo5;l00IBo0@000Q>
_0<000M>V`03001>Ndik00Q>N`03001>NTij00M>NP03001>FDiI00Q>F@03001>FDiI00=>F@<0009>
>003001>>4hh00Q>>003001B>58h00=B>003001B>58h00=B>003001B>58h00UB>003001oogoo00Yo
o`03001oogoo019oo`0017oo00@007ooOol000Qoo`04001oogoo0002Ool00`00Oomoo`04Ool00`00
Oomoo`0AOol00`00E_eFo@02E_d30005E_d00`00E_eFo@09E_d00`00E_eFo@030006E_d00`00D_eB
o@09D_d00`00D_aBo0050002D]`00`00C[a>_004C[`300000djl001>V`08CY/00`00CW]>N`07CW/0
0`00CWY>NP08CWX00`00CUU>F@07CUT00`00CUU>F@030006CSP00`00CSQ>>008CSP00`00DSQB>003
DSP00`00DSQB>003DSP00`00DSQB>009DSP00`00Oomoo`08Ool2000DOol000Eoo`8000Yoo`8000=o
o`@000=oo`8001=oo`04001Fo@000009E_d00`00E_eFo@08E_d3000:E_d00`00D_eBo@08D_d30008
D]`01000C[a>_4jl0P0014jl00<004jKCY/01djK00<004ikCW/01dik00<004ijCWX024ij00<004iI
CUT00diI1`002Dhh00<004hhCSP024hh00<0058hDSP0158h00<0058hDSP00U8h00<0058hDSP02E8h
00<007ooOol02Woo00<007ooOol04Goo000MOol00`00Oomoo`0COol2000;E_d00`00E_eFo@04E_d3
0002E_d00`00E_eFo@09E_d00`00D_eBo@04D_d300000e;m001Bg009D]`30007C[`00`00CY]>V`07
CY/00`00CW]>N`07CW/00`00CWY>NP06CWX70005CUT00`00CSQ>>008CSP00`00CSQ>>008CSP00`00
DSQB>005DSP01@00DSQB>58h00002e8h00<007ooOol02goo00<007ooOol00goo10002Goo000MOol0
0`00Oomoo`0EOol00`00E_eFo@09E_d00`00E_eFo@030006E_d00`00E_eFo@09E_d00`00D_eBo@03
0005D_d00`00D]aBg004D]`300000e;L001>_007C[`00`00CY]>V`07CY/00`00CW]>N`07CW/01000
CWY>NTij1@000TYI00<004iICUT024iI00<004hhCSP024hh00<004hhCSP024hh00<0058hDSP0158h
00<0058hDSP00U8h00<0058hDSP01e8h0`0037oo00<007ooOol017oo00<007ooOol02Goo000LOol2
000HOol20009E_d3000:E_d00`00E_eFo@08E_d30009D_d00`00D]aBg0030004D]`00`00C[a>_007
C[`00`00CY]>V`03CY/A0008BUT00`00CUU>F@07CUT00`00CSQ>>008CSP00`00CSQ>>008CSP00`00
DSQB>005DSP01@00DSQB>58h00001E8h10000U8h00<007ooOol02goo00<007ooOol017oo00<007oo
Ool027oo000LOol00`00Oomoo`0IOol00`00E_eFo@03E_d30002E_d00`00E_eFo@09E_d00`00E_eF
o@04E_d300000e;m001Bo@09D_d30008D]`00`00C[a>_003C[`90004AW/00`00BUY:FP07BUX00`00
BUU:F@08BUT00`00CUU>F@07CUT00`00CSQ>>008CSP00`00CSQ>>008CSP00`00DSQB>006DSP01000
DSQB>0000U8h0`001e8h00<007ooOol02goo00<007ooOol017oo00<007ooOol01goo000LOol2000K
Ool00`00E_d000020006E_d00`00E_eFo@09E_d00`00E_eFo@030005D_d00`00D_eBo@04D_d30000
0e;m001Bg007D]`70004B[`00`00AW]6N`07AW/00`00BUY:FP07BUX00`00BUU:F@07BUT00`00CUU>
F@08CUT00`00CSQ>>008CSP00`00CSQ>>008CSP00`00DSQB>006DSP5000:DSP00`00Oomoo`0;Ool0
1@00Oomoogoo00000Woo00<007ooOol01goo0009Ool20004Ool20003Ool30005Ool00`00Oomoo`0K
Ool00`00E_eFo@08E_d00`00E_eFo@08E_d30009D_d00`00D_eBo@030005D_d00`00D]aBg0050002
C]d00`00B[a:_007B[`00`00AW]6N`07AW/00`00BUY:FP06BUX00`00BUU:F@08BUT00`00CUU>F@07
CUT00`00CSQ>>008CSP00`00CSQ>>008CSP00`00DSQB>003DSP40002CSP00`00DSQB>009DSP00`00
Oomoo`0:Ool30003Ool2000:Ool000Qoo`04001oogoo0003Ool20006Ool00`00Oomoo`02Ool00`00
Oomoo`0LOol00`00E_eFo@08E_d00`00E_eFo@04E_d300000eKm001Bo@09D_d30007D_d40008C]d0
0`00B[a:_007B[`00`00AW]6N`06AW/00`00BUY:FP07BUX00`00BUU:F@07BUT00`00CUU>F@08CUT0
0`00CSQ>>007CSP00`00CSQ>>008CSP00`00DSQB>0030004CSP01000CSQ>>0002e8h00<007ooOol0
37oo00<007ooOol037oo0008Ool01000Oomoo`002goo00<007ooOol00Woo00<007ooOol07Goo00<0
05KmE_d025Km00<005KmE_d00`001EKm00<005;mD_d015;m0`000U;m00<005;mD_d01@000U;M00<0
04kMC]d024kM00<004ZlB[`01TZl00<004IkAW/01dIk00<004YJBUX01TYJ00<004YIBUT024YI00<0
04iICUT01diI00<004hhCSP024hh00<004hhCSP01Thh100024hh00<004hh00002e8h00<007ooOol0
37oo00<007ooOol037oo0008Ool01000Oomoo`0027oo0`001Goo00<007ooOol07Woo00<005KmE_d0
1eKm0`002EKm00<005;mD_d00`001E;m0`0025;M00<004kMC]d024kM00<004ZlB[`01TZl00<004Ik
AW/01TIk00<004YJBUX01dYJ00<004YIBUT01dYI00<004iICUT01diI00<004hhCSP024hh00<004hh
CSP00dhh0`000Thh00<004hhCSP01dhh00<004hh00002e8h00<007ooOol03Goo00<007ooOol02goo
0008Ool01000Oomoo`0027oo00<007ooOol017oo0P008Goo00<005KmE_d00eKm0`000eKm00<005Km
E_d01eKm0P001E;m0`000U;m00<005;MD]d01e;M00<004kMC]d01dkM00<004ZlB[`01TZl00<004Ik
AW/01dIk00<004YJBUX01TYJ00<004YIBUT01dYI00<004iICUT024iI00<004hhCSP01dhh00<004hh
CSP00`001Thh00<004hhCSP01dhh00<004hh00002e8h00<007ooOol037oo00<007ooOol02goo0009
Ool20009Ool40003Ool00`00Oomoo`0QOol00`00E_d000020007E_d00`00E_eFo@03E_d3000015Km
001BoE;m0`001U;m00<005;MD]d01e;M00<004kMC]d01dkM00<004ZlB[`01TZl00<004IkAW/01dIk
00<004YJBUX01DYJ00<004YIBUT024YI00<004iICUT01diI00<004hhCSP01Thh10002Dhh00<004hh
CSP024hh0P002e8h00<007ooOol03Goo00<007ooOol02Woo000KOol2000SOol00`00E_eFo@09E_d0
0`00E_d000020003E_d4000:D_d00`00D]eBg@07D]d00`00C]e>g@03C]dC0003AW/00`00BUY:FP06
BUX00`00BUU:F@07BUT00`00CUU>F@07CUT01000CSQ>>4hh1@000Thh00<004hhCSP02Dhh00<004hh
CSP024hh00<0058hDSP02E8h00<007ooOol03Goo0P002goo000KOol00`00Oomoo`0SOol00`00E_eF
o@07E_d30002E_d40004D_d00`00D_eBo@08D_d00`00D]eBg@05D]d70004A[d00`00@Ya2W006@Y`0
0`00?U`nG003?U`:0002BUX00`00BUU:F@07BUT00`00CUU>F@03CUT80008CSP00`00CSQ>>008CSP0
0`00CSQ>>008CSP2000;DSP00`00Oomoo`0=Ool00`00Oomoo`09Ool001]oo`03001oogoo02Aoo`03
001FoEKm00=Fo@<0009Fo@<000UBo@03001BoE;m00QBo@03001BgE;M00@000=>g@030016_DJm00I6
_@030012W4:L00I2W003000nG3iL00HnG0030012Fd9K00=2Fa8000E:>@03001>>4hh00Q>>003001>
>4hh00Q>>003001>>4hh00U>>003001B>58h00UB>003001oogoo00ioo`03001oogoo00Qoo`006goo
00<007ooOol09Goo00<005Km00000P000UKm0`0000=Fo@00D_d02U;m00<005;mD_d01U;m100024kM
00<004JmA[d01TJm00<004:L@Y`01T:L00<003iL?U`01SiL00<0049K@U/01D9K00<004HjASX01dHj
00<004XiBST024Xi00<004hhCSP01dhh00<004hhCSP02Dhh00<004hhCSP024hh0P002U8h00<007oo
Ool03goo00<007ooOol01goo000JOol00`00Oomoo`0UOol30002E_d30005E_d00`00D_eBo@09D_d0
0`00D_eBo@050002D_d00`00C]e>g@08C]d00`00A[e6_@06A[d00`00@Ya2W005@Y`00`00?U`nG006
?U`00`00@U]2F`06@U/00`00ASY6>P07ASX00`00BSU:>@07BST00`00CSQ>>008CSP00`00CSQ>>008
CSP00`00CSQ>>009CSP20008DSP3000@Ool00`00Oomoo`07Ool001Yoo`8002=oo`<000=oo`8000UF
o@03001BoE;m00QBo@<000QBo@03001>gDkM00Q>g@030016_DJm00E6_@030012W4:L00I2W003000n
G3iL00HnG0030012Fd9K00E2F`030016>THj00M6>P03001:>DXi00M:>@03001>>4hh00Q>>003001>
>4hh00U>>003001>>4hh00Q>>08000AB>0@0009BF003001oogoo00moo`03001oogoo00Ioo`000Woo
0P0017oo0P0017oo00<007ooOol00Woo0`0017oo00<007ooOol07goo0`001goo00<005KmE_d025Km
00<005;mD_d015;m0`0000=Bo@00D_d02E;m00<004kMC]d024kM00<004JmA[d01DJm00<004:L@Y`0
1D:L00<003iL?U`01ciL00<0049K@U/01D9K00<004HjASX01THj00<004XiBST024Xi00<004hhCSP0
24hh00<004hhCSP024hh00<004hhCSP024hh00<0058h00000`001e9H00<007ooOol03Goo0`001goo
00001Goo001oogoo00000goo0P0017oo00<007ooOol01Goo00D007ooOomoo`0001ioo`<000]oo`03
001FoEKm00QFo@03001BoE;m00<000EBo@03001BoE;m00QBo@03001>gDkM00I>gA000092W003000n
G3iL00HnG0030012Fd9K00E2F`030016>THj00M6>P03001:>DXi00M:>@03001>>4hh00Q>>003001>
>4hh00U>>003001>>4hh00I>>0D000UBF003001oogoo00Yoo`<0009oo`03001oogoo00Eoo`0000Eo
o`00Oomoo`0000Yoo`03001oogoo00Aoo`04001oogooOol2000JOol3000?Ool00`00E_eFo@07E_d3
0009D_d00`00D_eBo@08D_d00`00C]e>g@050002B]d00`00@[e2_@05@[d00`00>WdjO@03>Wd70005
?U`00`00@U]2F`05@U/00`00ASY6>P06ASX00`00BSU:>@08BST00`00CSQ>>007CSP00`00CSQ>>009
CSP00`00CSQ>>003CSP30002DUP00`00DUP00009DUP00`00Oomoo`07Ool30005Ool00`00Oomoo`05
Ool00005Ool007ooOol0000:Ool01000Oomoogoo0`0017oo00<007ooOol05goo0P004goo00<005Km
E_d00eKm0`000eKm00<005;mD_d01e;m00<005;mD_d01U;m100024[M00<004:m@[d01D:m00<003Ym
>Wd01CYm00@003Hl=S`f?0H000M2F`030016>THj00M6>P03001:>DXi00M:>@03001>>4hh00Q>>003
001>>4hh00Q>>003001>>4hh00<000IBF003001BF00000UBF003001oogoo00=oo`<000Qoo`03001o
ogoo00Eoo`0000Eoo`00Oomoo`0000]oo`04001oogoo0006Ool00`00Oomoo`0DOol3000FOol00`00
E_d000020007E_d00`00D_eBo@07D_d00`00D_eBo@050003D_d00`00B]e:g@07B]d00`00@[e2_@05
@[d00`00>WdjO@05>Wd00`00=S`f?005=S`40005@U/00`00ASY6>P06ASX00`00BSU:>@07BST00`00
CSQ>>008CSP00`00CSQ>>006CSP50009DUP01000DUQBF000259H00<007oo00000`0037oo00<007oo
Ool017oo0002Ool20009Ool40002Ool40002Ool00`00Oomoo`0BOol3000JOol2000:E_d00`00D_eB
o@05D_d40009D_d00`00B]e:g@07B]d00`00@[e2_@05@[d00`00>WdjO@05>Wd00`00=S`f?005=S`0
0`00>S/j>`060009ASX00`00BSU:>@07BST00`00CSQ>>007CSP01000CSQ>>4hh1@000diI00<0059H
DUP0259H00D0059HDUQBF00000QBF003001oogoo00ioo`03001oogoo00Aoo`006Goo00<007ooOol0
3goo0`007goo00<005KmE_d025Km00<005;mD_d0100015;m00<005;mD_d01e;m00<004[MB]d01d[M
00<004:m@[d014:m00<003Ym>Wd01CYm00<003Hl=S`01SHl00<003Xk>S/013Xk1@001DHj00<004Xi
BST01dXi00<004hhCSP01Thh1@002DiI00<0059HDUP0259H00@0059HDUP000QBF003001oogoo00io
o`03001oogoo00Aoo`006Goo0P003Goo0`008goo00<005KmE_d01eKm0`002E;m00<005;mD_d01e;m
00<004[MB]d00d[M2@000T:m00<003Ym>Wd01CYm00<003Hl=S`01CHl00<003Xk>S/01CXk00@003hj
?SXn>PH000U:>@04001>>4hhCSP50002BUT00`00CUU>F@09CUT00`00DUQBF008DUP01@00DUQBF59H
00001e9H00<007ooOol03Woo00<007ooOol017oo000IOol00`00Oomoo`09Ool3000WOol00`00E_eF
o@03E_d30002E_d00`00D_eBo@08D_d00`00D_eBo@05D_d70004A]d00`00>YhjWP02>Yh60004>Wd0
0`00=S`f?006=S`00`00>S/j>`05>S/00`00?SXn>P05?SX>0008BUT00`00CUU>F@08CUT00`00DUQB
F009DUP01@00DUQBF59H00001e9H00<007ooOol03Goo00<007ooOol017oo000IOol00`00Oomoo`06
Ool3000[Ool00`00E_d000020006E_d00`00D_eBo@08D_d00`00D_eBo@040003C_d00`00A]e6g@06
A]d00`00>YhjWP04>Yh00`00;Ud^G@060008=S`00`00>S/j>`04>S/00`00?SXn>P06?SX00`00ASY6
>P07ASX00`00BUU:F@08BUT00`00CUU>F@09CUT00`00DUQBF008DUP00`00DUQBF002DUP00`00DUQB
F004DUP00`00Oomoo`0=Ool00`00Oomoo`04Ool001Uoo`03001oogoo00=oo`<002moo`03001FoEKm
00QFo@03001BoE;m00IBo@@000Q>o@030016gDKM00I6g@03000jWSZN00@jWP03000^GBiM00@^G@@0
00Df?003000j>cXk00Dj>`03000n>Shj00Hn>P030016>THj00M6>P03001:FDYI00M:F@03001>FDiI
00U>F@03001BF59H00QBF003001BF59H009BF003001BF59H00ABF003001oogoo00eoo`03001oogoo
00Aoo`0067oo00@007ooOomoo`<003=oo`03001FoEKm00QFo@03001BoE;m00D000=Bo@03001>oDkm
00M>o@030016gDKM00I6g@03000jWSZN00@jWP03000^GBiM00@^G@03000V72HL00<000<f?003000j
>cXk00@j>`03000n>Shj00Hn>P030016>THj00M6>P03001:FDYI00Q:F@03001>FDiI00U>F@03001B
F59H00QBF003001BF59H009BF003001BF59H009BF0<000moo`03001oogoo00=oo`0067oo1000=goo
00<005KmE_d01eKm0`002E;m00<004kmC_d01dkm00<004KMA]d01TKM00<003ZN>Yh013ZN00<002iM
;Ud012iM00<002HL9Q`00RHL10001cXk00<003hj?SX01Shj00<004HjASX01dHj00<004YIBUT01dYI
00<004iICUT02DiI00<0059HDUP0259H00<0059HDUP00e9H10000U9j00<007ooOol03Goo00<007oo
Ool00goo000@Ool30005Ool2000jOol00`00E_eFo@03E_d30002E_d00`00D_eBo@08D_d00`00C_e>
o@05C_d@0002>Yh00`00;Ud^G@04;Ud00`009Q`V70049Q`40004>S/00`00?SXn>P06?SX00`00ASY6
>P07ASX00`00BUU:F@08BUT00`00CUU>F@09CUT00`00DUQBF007DUP00`00DUQBF0040006DWX00`00
Oomoo`0<Ool00`00Oomoo`03Ool0015oo`03001oogoo00Ioo`03001oogoo03Qoo`03001Fo@000080
00IFo@03001BoE;m00QBo@03001>oDkm00@000=>o@030016gTKN00E6gP03000fWSJN008fWPH000@^
G@03000V72HL00@V7003000Z72XL00<0008j>`03000n>Shj00Hn>P030016>THj00M6>P03001:FDYI
00M:F@03001>FDiI00U>F@03001BF59H00IBF0@000ABNP03001BNU9j00=BNP03001oogoo00aoo`03
001oogoo00=oo`004Goo00<007ooOol01goo00<007ooOol0>7oo00<005KmE_d025Km00<005;mD_d0
1U;m100024km00<004KNA]h01DKN00<003JN=Yh01CJN00<002Hm00001@001RHL00<002XL:Q`00RXL
0`0023hj00<004HjASX01dHj00<004YIBUT024YI00<004iICUT024iI00@0059HDUQBF0D0009BNP03
001BNU9j00ABNP03001BNU9j009BNP03001oogoo00aoo`03001oogoo00=oo`004Goo00<007ooOol0
27oo00<007ooOol0>7oo00<005KmE_d025Km00<005;mD_d01@000e;m00<004kmC_d01dkm00<004KN
A]h01DKN00<003JN=Yh01CJN00<002Hm9Sd00bHm0`0012HL00<002XL:Q`01BXL10001Chj00<004Hj
ASX01THj00<004YIBUT024YI00<004iICUT01TiI1P00259j00<0059jDWX0159j00<0059jDWX00U9j
00<007ooOol02goo00<007ooOol00goo000AOol00`00Oomoo`09Ool2000iOol00`00E_eFo@07E_d3
0009D_d00`00C_e>o@07C_d00`00A]i6gP05A]h00`00=YhfWP05=Yh00`009SdV?@039Sd010006O`0
00000bHL00<002XL:Q`012XL00@003Hk=S/f>`H000U6>P03001:FDYI00M:F@04001>FDiICUT50003
CWX00`00DWYBNP08DWX00`00DWYBNP04DWX00`00DWYBNP02DWX00`00Oomoo`0;Ool00`00Oomoo`03
Ool0011oo`8000eoo`03001oogoo03Moo`03001FoEKm00=Fo@<0009Fo@03001BoE;m00QBo@03001>
oDkm00E>oA00008fWP03000V?BHm00<V?@03000Io1Wl00<000039Q`002XL00DZ7003000f>cHk00Df
>`L000=6>P03001:FDYI00=:F@P000U>NP03001BNU9j00QBNP03001BNU9j00ABNP03001BNU9j009B
NP03001oogoo00]oo`03001oogoo009oo`0087oo00<007ooOol0=goo00<005Km00000P001UKm00<0
05;mD_d025;m00<004kmC_d010000dkm00<004KNA]h01DKN00<003:N<Yh00S:N1P000bHm00<001Wl
6O`00QWl0P001RXL00<003Hk=S/01SHk00<0049K@U/00d9K2@001DYj00<004ijCWX02Dij00<0059j
DWX0259j00<0059jDWX01E9j00D0059jDWYBNP0000eoo`03001oogoo009oo`008Goo00<007ooOol0
=goo0P002EKm00<005;mD_d01U;m100024km00<004KNA]h01DKN00<003:N<Yh01C:N00<001hM0000
0P0000<V?@006O`011Wl0P001BXL00<003Hk=S/01SHk00<0049K@U/01T9K00<004YjBWX024Yj00<0
04ijCWX024ij00<0059jDWX02E9j00<0059jDWX01E9j00@0059jDWX000eoo`03001oogoo009oo`00
8Woo00<007ooOol0>7oo00<005KmE_d01eKm00<005;mD_d01@000U;m00<004kmC_d024km00<004KN
A]h01DKN00<003:N<Yh01C:N00@001hM7QdN7@<000@Io004000Mo0000004:Q`00`00=S/f>`05=S/0
0`00@U]2F`07@U/00`00BWY:NP07BWX00`00CWY>NP09CWX00`00DWYBNP08DWX00`00DWYBNP05DWX0
0`00DWX00002000<Ool00`00Oomoo`02Ool002=oo`8003Uoo`03001FoEKm00EFo@@000QBo@03001>
oDkm00Q>o@030016gTKN00E6gP03000bWS:N00DbWP03000N7AhM00<N7@8000<Io003000Mo1gl0080
008Z7003000f>cHk00Hf>`030012Fd9K00I2F`03001:NTYj00Q:NP03001>NTij00Q>NP03001BNU9j
00UBNP03001BNU9j00=BNP<0009B^`03001oogoo00Yoo`03001oogoo009oo`009Goo00<007ooOol0
=goo00<005KmE_d010000eKm00<005;mD_d01e;m00<004kmC_d01Tkm40000S:N00<001hM7Qd00QhM
00@000gL0000008Io004000Mo1gl7O`200000bXL000f>`06=S/00`00@U]2F`07@U/00`00BWY:NP07
BWX00`00CWY>NP08CWX00`00DWYBNP09DWX00`00DWYBNP030003D[/01000D[]B^`002goo00<007oo
Ool00Woo000VOol00`00Oomoo`0gOol20008E_d00`00D_eBo@07D_d00`00C_e>o@050002C_d00`00
@]i2gP06@]h00`00<YhbWP02<Yh500037Qd00`003M`=g00200000aWl000Mo0037O`20008=S/00`00
@U]2F`06@U/00`00BWY:NP08BWX00`00CWY>NP08CWX00`00DWYBNP07DWX40006D[/01000D[]B^`00
2goo00<007ooOol00Woo000WOol00`00Oomoo`0gOol00`00E_eFo@07E_d00`00D_eBo@05D_d40008
C_d00`00@]i2gP06@]h00`00<YhbWP04<Yh010006Qd000000QhM00@000gL3M`=g08000HMo0<000Df
>`030012Fd9K00M2F`03001:NTYj00M:NP03001>NTij00Q>NP03001BNU9j00=BNP@0009B^`03001B
^e:k00IB^`04001B^e:k000;Ool00`00Oomoo`01Ool002Qoo`8003Qoo`03001FoEKm00MFo@03001B
oE;m00@000ABo@03001>oDkm00I>o@030012gT;N00I2gP03000bWS:N00@bWP03000J7AXM00@000<=
g08000DMo003000^?2hl00<000<f>`030012Fd9K00I2F`03001:NTYj00Q:NP03001>NTij00M>NP03
001BNU9j00<000MB^`03001B^e:k00IB^`04001B^e:k000:Ool00`00Oomoo`01Ool002Yoo`03001o
ogoo03Ioo`03001FoEKm00EFo@@000UBo@03001>oDkm00I>o@030012gT;N00I2gP03000bWS:N00@b
WP03000J7AXM008J7@8000@=g08000@Mo003000^?2hl008^?0@000U2F`03001:NTYj00M:NP03001>
NTij00=>NPL000YB^`03001B^e:k00MB^`03001B^`0000Yoo`03001oogoo005oo`00:goo00<007oo
Ool0=Woo00<005KmE_d0100015;m00<005;mD_d01e;m00<004kmC_d00Tkm4P000S:N00<001XM6Qd0
0aXM0P000`gL10000Qgl00<002hl;S`012hl1`000d9K00<004YjBWX00dYj2P001DjK00<005:kD[/0
2U:k00<005:kD[/01U:k00@005:kD[/000Uoo`03001oogoo005oo`00;7oo00<007ooOol0=Woo0P00
2E;m00<005;mD_d01E;m1P001Tkm00<004;N@]h01D;N00@0039n<WhbOPD000<J7@040009^`000002
3M`00`004M`Ag00200000agl000^?005;S`00`00?W`nO003?W`90005AY`00`00CY]>V`08CY/00`00
D[]B^`09D[/00`00D[]B^`07D[/00`00D[/00009Ool00`00Oomoo`01Ool002eoo`8003Moo`03001B
oE;m00QBo@03001BoE;m00@000=Bo@03001>oDkm00M>o@030012gT;N00E2gP03000bOS9n00@bOP<0
008J7@030009^`Vk00@000<Ag0<000H^?003000nO3il00HnO0030016W4JL00Q6W003001>VdjK00Q>
V`03001B^e:k00UB^`03001B^e:k00IB^`@000Qoo`03001oogoo005oo`00;goo00<007ooOol0=Goo
00<005;mD_d01e;m0`0025;m00<004kmC_d01dkm00<004;N@]h01D;N00<0039n<Wh0139n00<001HM
00000`0010Vk0P001A7L0P0012hl00<003il?W`01cil00<004JLAY`024JL00<004jKCY/01djK00<0
05:kD[/02E:k00@005:kD[]B^`D0009Bg08000Qoo`03001oogoo005oo`00<7oo00<007ooOol0=Goo
00<005;mD_d00e;m0`000U;m00<005;mD_d01e;m00<004kmC_d01Dkm3@00139n00@001HM5QdF7@80
00D9^`8000<Ag004000R7@000003;S`00`00?W`nO006?W`00`00AYa6W008AY`00`00CY]>V`08CY/0
0`00D[]B^`07D[/50007D]`00`00D]`00008Ool10001Ool1Ool0035oo`03001oogoo03Eoo`03001B
o@00008000IBo@03001BoE;m00MBo@03001>oDkm00@000eoo`H000@F7@8000<9^`040009^`000002
4M`00`008QdR7@0300000bhl000nO007?W`00`00AYa6W008AY`00`00CY]>V`07CY/00`00D[]B^`04
D[/30002D]`00`00D]aBg008D]`00`00D]`00007Ool10001Ool1Ool0039oo`03001oogoo03Eoo`80
00UBo@03001BoE;m00EBo@@001Ioo`<000<F7@<00089^`030009^`Vk008000034M`0028M00<R7@80
00PnO0030016W4JL00Q6W003001>VdjK00Q>V`03001B^e:k00<000IBg003001Bg5;L00QBg08000Mo
o`40005oo`5oo`00<goo0P00=goo00<005;mD_d01e;m00<005;mD_d010007Goo0`0000<F7@001K/0
10000`Vk0P001R8M10001Cil00<004JLAY`024JL00<004jKCY/00djK1`002E;L00<005;LD]`025;L
00<005;L00001Woo0@000Goo0Goo000eOol00`00Oomoo`0eOol00`00D_eBo@05D_d4000TOol30003
1K/200032K/200058Qd01000=WdfOCIm1P001TJL2P001DkL00<005;LD]`02E;L00<005;LD]`025;L
0`001Goo0@000Goo0Goo000fOol00`00Oomoo`0eOol00`00D_eBo@04000[Ool200031K/200032K/2
00048Qd00`00=WdfO@05=Wd70005A[d00`00C]a>g008C]`00`00D]aBg009D]`00`00D]aBg003D]`4
00000e;m00000005Ool10001Ool1Ool003Moo`03001oogoo03Eoo`80035oo`8000031K/000Fk0080
0089^`@0008R7@03000fOCIm00DfO@030016_DJm00U6_@03001>g4kL00Q>g003001Bg5;L00QBg003
001Bg5;L00<000IBo@8000Aoo`40005oo`5oo`00>7oo0P00=Goo00<007ooOol0<Woo0`000PFk0P00
00@9^`006QdJ7@8000038Qd003Im00HfO@030016_DJm00Q6_@03001>g4kL00Q>g003001Bg5;L00MB
g0@000UBo@03001Bo@0000Aoo`40005oo`00>Woo00<007ooOol0<Woo00<007ooOol0=7oo0P000`Fk
0`000aXM0`001cIm00<004JmA[d024Jm00<004kLC]`024kL00@005;LD]aBg0D0009Bo@03001BoE;m
00UBo@8000Aoo`40005oo`00>goo00<007ooOol0<Goo00<007ooOol0=Goo0P0010Fk0P001AXM1000
13Im00<004JmA[d024Jm00<004kLC]`01TkL1@0025;m00<005;mD_d02E;m0P000goo0@000Goo000l
Ool00`00Oomoo`0`Ool00`00Oomoo`0gOol200041K/200036Qd01000<WhbOS9n1P002DJm00@004kL
C]a>g0D0009:g@03001BoE;m00UBo@03001BoE;m00QBo@<0009oo`40005oo`00?Goo0P00<7oo00<0
07ooOol0>Goo0P000PFk00@001HM0000008J7@03000bOS9n00DbOPh000Q:g@03001BoE;m00QBo@03
001BoE;m00ABo@@00003Dad00000009oo`40005oo`00?goo00<007ooOol0;Goo00<007ooOol0>goo
0P0000@5^`005QdF7@@000LbOP030012_T:n00M2_P03001:gD[M00U:g@03001BoE;m00QBo@03001B
oE;m00<000IC7@80005oo`40005oo`00@7oo00<007ooOol0;7oo00<007ooOol0?Goo0`0011HM0P00
1c9n00<004:n@[h01d:n00<004[MB]d024[M00<005;mD_d01e;m10002U<M0`000Goo0011Ool00`00
Oomoo`0[Ool00`00Oomoo`0oOol200055Qd40004<Wh00`00@[i2_P07@[h00`00B]e:g@08B]d01000
D_eBoE;m1@000Tkm00<005<MDad02E<M0P000Goo0@000012Ool2000[Ool00`00Oomoo`11Ool20003
5Qd01000;Wh^ORin1P002D:n00<004[MB]d00d[M200024km00<005<MDad025<M10000014Ool00`00
Oomoo`0XOol00`00Oomoo`13Ool200025Qd00`00;Wh^OP05;Wh@0005B_h00`00C_e>o@09C_d00`00
DaeC7@04Dad30002Ool2000004Eoo`03001oogoo02Ioo`03001oogoo04Ioo`@000L^OP030012gT;N
00M2gP03001:oT[n00Q:oP03001>oDkm00U>o@03001C7E<M00<000Aoo`<00000AWoo00<007ooOol0
9Goo00<007ooOol0BGoo0P001bin00<004;N@]h01d;N00<004[nB_h024[n00<004kmC_d01dkm1000
0goo10000goo0017Ool00`00Oomoo`0TOol00`00Oomoo`1;Ool40004;Wh00`00@]i2gP07@]h00`00
B_i:oP08B_h01000C_e>oDkm1@000goo10001goo0018Ool2000TOol00`00Oomoo`1?Ool60009@]h0
0`00B_i:oP03B_h80004Ool4000;Ool004Yoo`03001oogoo025oo`03001oogoo05Aooa0000Qoo`@0
00moo`00Bgoo00<007ooOol087oo00<007ooOol0J7oo10004goo001<Ool00`00Oomoo`0OOol00`00
Oomoo`1TOol4000GOol004eoo`8001moo`03001oogoo061oo`@001]oo`00Cgoo00<007ooOol077oo
00<007ooOol0GGoo0`007goo001@Ool00`00Oomoo`0KOol00`00Oomoo`1IOol4000ROol0055oo`03
001oogoo01Yoo`03001oogoo05Eoo`@002Ioo`00DWoo0P006Woo00<007ooOol0DGoo1000:Woo001D
Ool00`00Oomoo`0GOol00`00Oomoo`1=Ool4000^Ool005Eoo`03001oogoo01Ioo`03001oogoo04Uo
o`@0039oo`00EWoo00<007ooOol05Goo00<007ooOol0AGoo1000=Woo001GOol00`00Oomoo`0DOol0
0`00Oomoo`11Ool4000jOol005Qoo`8001Aoo`03001oogoo03eoo`@003ioo`00FWoo00<007ooOol0
4Goo00<007ooOol0>Goo1000@Woo001KOol00`00Oomoo`0@Ool00`00Oomoo`0eOol40016Ool005ao
o`03001oogoo00moo`03001oogoo035oo`@004Yoo`00GGoo0P003goo00<007ooOol0;Goo1000CWoo
001OOol00`00Oomoo`0<Ool00`00Oomoo`0YOol4001BOol0061oo`03001oogoo00]oo`03001oogoo
02Eoo`@005Ioo`00HGoo00<007ooOol02Goo00<007ooOol08Woo1000FWoo001ROol20009Ool00`00
Oomoo`0NOol4001NOol006Aoo`03001oogoo00Ioo`03001oogoo01]oo`<0069oo`00IGoo00<007oo
Ool01Goo00<007ooOol05goo1000IGoo001VOol00`00Oomoo`04Ool00`00Oomoo`0COol4001YOol0
06Moo`03001oogoo00=oo`03001oogoo00moo`@006eoo`00J7oo0P000goo00<007ooOol02goo1000
LGoo001ZOol01000Oomoo`002Goo1000MGoo001[Ool00`00Ool00005Ool4001iOol006aoo`800003
Ool00000008007eoo`00KGoo0P00PGoo003`Ool00?1oo`00l7oo003`Ool00?1oo`00l7oo003`Ool0
0?1oo`00l7oo0000\
\>"],
  ImageRangeCache->{{{0, 239}, {193.438, 0}} -> {-0.108618, 0.0189822, 
  0.00444022, 0.00444022}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Critical Dividend and Price Computations", "Subsubsection",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "As discussed by Hull, there is a critical dividend level below which early \
exercise at the last dividend payment date is not optimal. If the dividend \
occurs at a point ",
  Cell[BoxData[
      \(TraditionalForm\`t\)]],
  " before expiry, the critical dividend level is given by the formula"
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`CriticalDiv(K, r, t) = 
      K\ \((1 - \[ExponentialE]\^\(\(-r\)\ t\))\)\)], "NumberedEquation",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(CriticalDiv[K_, \ r_, \ t_]\  := \ K*\((1\  - \ Exp[\(-r\)*t])\)\)], 
  "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(CriticalDiv[50, \ 0.1, \ 0.5]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(2.4385287749642992`\)], "Output"]
}, Open  ]],

Cell["\<\
For dividends above this value, the critical price governing \
exercise is given by the solution of the equation:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(CriticalPrice[\ strike_, \ vol_, \ r_, q_, div_, \ divtime_]\  := \n
      price\  /. \ 
        FindRoot[
          BlackScholesCall[price\ , strike, \ vol, \ r, \ q, \ divtime] == 
            price + div\  - \ strike, \ {price, strike}, \ 
          MaxIterations\  -> \ 30]\ \)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(CriticalPrice[50, \ 0.25, \ 0.1, \ 0, \ 3, \ 0.5]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(58.4246026319585176`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CriticalPrice[50, \ 0.25, \ 0.1, \ 0, \ 6, \ 0.5]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(47.1162794663178985`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CriticalPrice[50, \ 0.25, \ 0.1, \ 0, \ 2.4385287, \ 0.5]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(116.444532609266193`\)], "Output"]
}, Open  ]],

Cell["\<\
The algorithm for finding the critical price falls in a heap if we \
lower the dividend value below the critical dividend level:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(CriticalPrice[50, \ 0.25, \ 0.1, \ 0, \ 2.43, \ 0.5]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(FindRoot::"jsing" \( : \ \) 
      "Encountered a singular Jacobian at the point \!\(price\) = \
\!\(1310.95393224636502`\). Try perturbing the initial point(s)."\)], 
  "Message"],

Cell[BoxData[
    RowBox[{"price", "/.", 
      RowBox[{"FindRoot", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"BlackScholesCall", "[", 
              RowBox[{"price", ",", "50", ",", 
                StyleBox["0.25`",
                  StyleBoxAutoDelete->True,
                  PrintPrecision->2], ",", 
                StyleBox["0.1`",
                  StyleBoxAutoDelete->True,
                  PrintPrecision->1], ",", "0", ",", 
                StyleBox["0.5`",
                  StyleBoxAutoDelete->True,
                  PrintPrecision->1]}], "]"}], "==", 
            RowBox[{"price", "+", 
              RowBox[{
                StyleBox["2.43000000000000015`",
                  StyleBoxAutoDelete->True,
                  PrintPrecision->3], "\[InvisibleSpace]"}], "-", "50"}]}], 
          ",", \({price, 50}\), ",", \(MaxIterations \[Rule] 30\)}], 
        "]"}]}]], "Output"]
}, Open  ]],

Cell["\<\
This is because the equation is being solved for the zero of a \
function that does not attain the value zero - for example, if we plot it \
with a value of the dividend of 2.4, we can see this explicitly:\
\>", "Text",\

  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(checkplot[strike_, \ vol_, \ r_, \ q_, div_, \ divtime_]\ \  := 
      Plot[BlackScholesCall[price\ , strike, \ vol, \ r, \ q, \ 
            divtime] - \((price + div\  - strike)\), \n\t\t{price, \ 40, \ 
          100}, \ PlotRange\  -> \ {0, \ 0.5}]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(checkplot[50, \ 0.25, \ 0.1, \ 0, \ 2.40, \ 0.5]; \)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.611111 0.015873 0 1.23607 [
[.18254 -0.0125 -6 -9 ]
[.18254 -0.0125 6 0 ]
[.34127 -0.0125 -6 -9 ]
[.34127 -0.0125 6 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.65873 -0.0125 -6 -9 ]
[.65873 -0.0125 6 0 ]
[.81746 -0.0125 -6 -9 ]
[.81746 -0.0125 6 0 ]
[.97619 -0.0125 -9 -9 ]
[.97619 -0.0125 9 0 ]
[.01131 .12361 -18 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -18 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -18 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -18 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -18 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 0 m
.18254 .00625 L
s
[(50)] .18254 -0.0125 0 1 Mshowa
.34127 0 m
.34127 .00625 L
s
[(60)] .34127 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(70)] .5 -0.0125 0 1 Mshowa
.65873 0 m
.65873 .00625 L
s
[(80)] .65873 -0.0125 0 1 Mshowa
.81746 0 m
.81746 .00625 L
s
[(90)] .81746 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(100)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.15079 0 m
.15079 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.27778 0 m
.27778 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.53175 0 m
.53175 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.9127 0 m
.9127 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(0.1)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.2)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.3)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.4)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(0.5)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .02472 m
.02756 .02472 L
s
.02381 .04944 m
.02756 .04944 L
s
.02381 .07416 m
.02756 .07416 L
s
.02381 .09889 m
.02756 .09889 L
s
.02381 .14833 m
.02756 .14833 L
s
.02381 .17305 m
.02756 .17305 L
s
.02381 .19777 m
.02756 .19777 L
s
.02381 .22249 m
.02756 .22249 L
s
.02381 .27193 m
.02756 .27193 L
s
.02381 .29666 m
.02756 .29666 L
s
.02381 .32138 m
.02756 .32138 L
s
.02381 .3461 m
.02756 .3461 L
s
.02381 .39554 m
.02756 .39554 L
s
.02381 .42026 m
.02756 .42026 L
s
.02381 .44498 m
.02756 .44498 L
s
.02381 .46971 m
.02756 .46971 L
s
.02381 .51915 m
.02756 .51915 L
s
.02381 .54387 m
.02756 .54387 L
s
.02381 .56859 m
.02756 .56859 L
s
.02381 .59331 m
.02756 .59331 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.33405 .61803 m
.34157 .55994 L
.36141 .44892 L
.38244 .35562 L
.40219 .28662 L
.42027 .23635 L
.43968 .19348 L
.46056 .15767 L
.47954 .13248 L
.50027 .11126 L
.51994 .09588 L
.54091 .08342 L
.56232 .07392 L
.58249 .06722 L
.60257 .06221 L
.62105 .05872 L
.63987 .05599 L
.66053 .05376 L
.68004 .05218 L
.70096 .05094 L
.7222 .05001 L
.74232 .04937 L
.76223 .04891 L
.78065 .04859 L
.79931 .04834 L
.81991 .04814 L
.83926 .04801 L
.86012 .0479 L
.88016 .04782 L
.90126 .04777 L
.921 .04773 L
.93937 .0477 L
.95866 .04768 L
.97619 .04767 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  ImageSize->{236.375, 146},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003/0000TQ000`40O003h00Ok7oo003/Ool003Ao
o`<000Aoo`8001Uoo`8000Aoo`8001Uoo`03001oogoo00=oo`8001Uoo`8000Aoo`8001Qoo`<000Ao
o`8001Eoo`<000Aoo`8000Aoo`8000Moo`00=goo00@007ooOol0009oo`03001oogoo01Eoo`04001o
ogoo0002Ool01000Oomoo`0067oo00<007ooOol00Woo00@007ooOol001Moo`04001oogoo0002Ool0
1000Oomoo`006Woo00@007ooOol0009oo`03001oogoo01=oo`03001oogoo009oo`04001oogoo0002
Ool01000Oomoo`001Woo000gOol01000Oomoo`000Woo00<007ooOol05Goo00@007ooOol0009oo`04
001oogoo000IOol01@00Oomoogoo00000Woo00<007ooOol05Goo00@007ooOol0009oo`04001oogoo
000HOol30002Ool01000Oomoo`005Goo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo0006
Ool003Aoo`<000=oo`04001oogoo000GOol30003Ool01000Oomoo`006Goo00D007ooOomoo`00009o
o`03001oogoo01Ioo`8000=oo`04001oogoo000GOol01000Oomoo`000Woo00@007ooOol001Eoo`03
001oogoo009oo`04001oogoo0002Ool01000Oomoo`001Woo000dOol00`00Oomoo`03Ool01000Oomo
o`005goo00<007ooOol00goo00@007ooOol001Yoo`04001oogoo0002Ool00`00Oomoo`0EOol01000
Oomoo`000Woo00@007ooOol001Moo`04001oogoo0002Ool01000Oomoo`005Goo00<007ooOol00Woo
00@007ooOol0009oo`04001oogoo0006Ool003Aoo`@000=oo`8001Uoo`<000=oo`8001Qoo`@000=o
o`8001Uoo`8000Aoo`8001Uoo`8000Aoo`8001Eoo`8000Eoo`8000Aoo`8000Moo`00k7oo003/Ool0
0>aoo`00k7oo000COooA0008Ool001Qoo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Ao
o`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Ao
o`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo00Ao
o`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo00Ao
o`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo00Ao
o`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo00Ao
o`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo00]oo`0067oo00<007ooOol0dGoo000HOol0
0`00Oomoo`3AOol001Qoo`03001oogoo0=5oo`0067oo00<007ooOol0dGoo000HOol00`00Oomoo`3A
Ool001Qoo`03001oogoo0=5oo`0067oo00<007ooOol0dGoo000HOol00`00Oomoo`3AOol001Qoo`03
001oogoo095ooc<000eoo`0067oo00<007ooOol0Ogoo4P00@7oo000HOol00`00Oomoo`1gOol8001B
Ool001Qoo`03001oogoo07=oo`@005Yoo`0067oo00<007ooOol0L7oo0`00GWoo000HOol00`00Oomo
o`1^Ool2001QOol001Qoo`03001oogoo06aoo`8006=oo`0067oo00<007ooOol0JWoo0P00IGoo000H
Ool00`00Oomoo`1XOol2001WOol001Qoo`03001oogoo06Eoo`<006Uoo`0067oo00<007ooOol0Hgoo
0P00K7oo000HOol00`00Oomoo`1ROol00`00Oomoo`1/Ool001Qoo`03001oogoo065oo`03001oogoo
06eoo`0067oo00<007ooOol0H7oo00<007ooOol0KWoo0005Ool20004Ool20003Ool30005Ool00`00
Oomoo`1POol00`00Oomoo`1^Ool000Aoo`04001oogoo0003Ool20004Ool00`00Oomoo`04Ool00`00
Oomoo`1OOol00`00Oomoo`1_Ool000Aoo`04001oogoo0009Ool00`00Oomoo`04Ool2001OOol00`00
Oomoo`1`Ool000Aoo`04001oogoo0009Ool00`00Oomoo`04Ool00`00Oomoo`1MOol00`00Oomoo`1a
Ool000Aoo`04001oogoo0009Ool00`00Oomoo`04Ool00`00Oomoo`1LOol00`00Oomoo`1bOol000Eo
o`8000Uoo`8000Ioo`03001oogoo05aoo`03001oogoo079oo`0067oo00<007ooOol0Fgoo00<007oo
Ool0Lgoo000HOol00`00Oomoo`1JOol00`00Oomoo`1dOol001Qoo`03001oogoo05Uoo`03001oogoo
07Eoo`0067oo00<007ooOol0F7oo00<007ooOol0MWoo000HOol00`00Oomoo`1HOol00`00Oomoo`1f
Ool001Qoo`03001oogoo05Moo`03001oogoo07Moo`0067oo00<007ooOol0EWoo00<007ooOol0N7oo
000HOol00`00Oomoo`1EOol00`00Oomoo`1iOol001Qoo`03001oogoo05Eoo`03001oogoo07Uoo`00
67oo00<007ooOol0E7oo00<007ooOol0NWoo000HOol00`00Oomoo`1COol00`00Oomoo`1kOol001Qo
o`03001oogoo05=oo`03001oogoo07]oo`0067oo00<007ooOol0DWoo00<007ooOol0O7oo000HOol0
0`00Oomoo`1BOol00`00Oomoo`1lOol001Qoo`03001oogoo055oo`03001oogoo07eoo`0067oo00<0
07ooOol0DGoo00<007ooOol0OGoo000HOol00`00Oomoo`1AOol00`00Oomoo`1mOol001Qoo`03001o
ogoo051oo`03001oogoo07ioo`0067oo00<007ooOol0D7oo00<007ooOol0OWoo000HOol00`00Oomo
o`1?Ool00`00Oomoo`1oOol000Eoo`8000Aoo`8000=oo`@000Aoo`03001oogoo04moo`03001oogoo
07moo`0017oo00@007ooOol000=oo`8000Aoo`03001oogoo00Aoo`03001oogoo04moo`03001oogoo
07moo`0017oo00@007ooOol000Yoo`03001oogoo00=oo`8004moo`03001oogoo081oo`0017oo00@0
07ooOol000]oo`03001oogoo009oo`03001oogoo04ioo`03001oogoo081oo`0017oo00@007ooOol0
00Qoo`04001oogoo0004Ool00`00Oomoo`1>Ool00`00Oomoo`20Ool000Eoo`8000Yoo`8000Eoo`03
001oogoo04eoo`03001oogoo085oo`0067oo00<007ooOol0CGoo00<007ooOol0PGoo000HOol00`00
Oomoo`1=Ool00`00Oomoo`21Ool001Qoo`03001oogoo04eoo`03001oogoo085oo`0067oo00<007oo
Ool0C7oo00<007ooOol0PWoo000HOol00`00Oomoo`1<Ool00`00Oomoo`22Ool001Qoo`03001oogoo
04aoo`03001oogoo089oo`0067oo00<007ooOol0Bgoo00<007ooOol0Pgoo000HOol00`00Oomoo`1;
Ool00`00Oomoo`23Ool001Qoo`03001oogoo04]oo`03001oogoo08=oo`0067oo00<007ooOol0BWoo
00<007ooOol0Q7oo000HOol00`00Oomoo`1:Ool00`00Oomoo`24Ool001Qoo`03001oogoo04Yoo`03
001oogoo08Aoo`0067oo00<007ooOol0BGoo00<007ooOol0QGoo000HOol00`00Oomoo`19Ool00`00
Oomoo`25Ool001Qoo`03001oogoo04Uoo`03001oogoo08Eoo`0067oo00<007ooOol0B7oo00<007oo
Ool0QWoo000HOol00`00Oomoo`18Ool00`00Oomoo`26Ool001Qoo`03001oogoo04Qoo`03001oogoo
08Ioo`0067oo00<007ooOol0Agoo00<007ooOol0Qgoo0005Ool20004Ool20003Ool30005Ool00`00
Oomoo`17Ool00`00Oomoo`27Ool000Aoo`04001oogoo0003Ool20006Ool00`00Oomoo`02Ool00`00
Oomoo`17Ool00`00Oomoo`27Ool000Aoo`04001oogoo000;Ool00`00Oomoo`02Ool20017Ool00`00
Oomoo`28Ool000Aoo`04001oogoo0009Ool20005Ool00`00Oomoo`16Ool00`00Oomoo`28Ool000Ao
o`04001oogoo000;Ool00`00Oomoo`02Ool00`00Oomoo`16Ool00`00Oomoo`28Ool000Eoo`8000Uo
o`<000Eoo`03001oogoo04Ioo`03001oogoo08Qoo`0067oo00<007ooOol0AWoo00<007ooOol0R7oo
000HOol00`00Oomoo`15Ool00`00Oomoo`29Ool001Qoo`03001oogoo04Eoo`03001oogoo08Uoo`00
67oo00<007ooOol0AGoo00<007ooOol0RGoo000HOol00`00Oomoo`15Ool00`00Oomoo`29Ool001Qo
o`03001oogoo04Eoo`03001oogoo08Uoo`0067oo00<007ooOol0A7oo00<007ooOol0RWoo000HOol0
0`00Oomoo`14Ool00`00Oomoo`2:Ool001Qoo`03001oogoo04Aoo`03001oogoo08Yoo`0067oo00<0
07ooOol0A7oo00<007ooOol0RWoo000HOol00`00Oomoo`14Ool00`00Oomoo`2:Ool001Qoo`03001o
ogoo04=oo`03001oogoo08]oo`0067oo00<007ooOol0@goo00<007ooOol0Rgoo000HOol00`00Oomo
o`13Ool00`00Oomoo`2;Ool001Qoo`03001oogoo04=oo`03001oogoo08]oo`0067oo00<007ooOol0
@goo00<007ooOol0Rgoo000HOol00`00Oomoo`13Ool00`00Oomoo`2;Ool001Qoo`03001oogoo049o
o`03001oogoo08aoo`0067oo00<007ooOol0@Woo00<007ooOol0S7oo000HOol00`00Oomoo`12Ool0
0`00Oomoo`2<Ool000Eoo`8000Aoo`8000Aoo`<000Aoo`03001oogoo049oo`03001oogoo08aoo`00
17oo00@007ooOol000=oo`8000Eoo`03001oogoo00=oo`03001oogoo049oo`03001oogoo08aoo`00
17oo00@007ooOol000Qoo`@000Aoo`8004=oo`03001oogoo08aoo`0017oo00@007ooOol000Qoo`03
001oo`0000Eoo`03001oogoo045oo`03001oogoo08eoo`0017oo00@007ooOol000Qoo`03001oo`00
00Eoo`03001oogoo045oo`03001oogoo08eoo`001Goo0P002Woo0P001Goo00<007ooOol0@Goo00<0
07ooOol0SGoo000HOol00`00Oomoo`11Ool00`00Oomoo`2=Ool001Qoo`03001oogoo045oo`03001o
ogoo08eoo`0067oo00<007ooOol0@7oo00<007ooOol0SWoo000HOol00`00Oomoo`10Ool00`00Oomo
o`2>Ool001Qoo`03001oogoo041oo`03001oogoo08ioo`0067oo00<007ooOol0@7oo00<007ooOol0
SWoo000HOol00`00Oomoo`10Ool00`00Oomoo`2>Ool001Qoo`03001oogoo041oo`03001oogoo08io
o`0067oo00<007ooOol0?goo00<007ooOol0Sgoo000HOol00`00Oomoo`0oOol00`00Oomoo`2?Ool0
01Qoo`03001oogoo03moo`03001oogoo08moo`0067oo00<007ooOol0?goo00<007ooOol0Sgoo000H
Ool00`00Oomoo`0oOol00`00Oomoo`2?Ool001Qoo`03001oogoo03moo`03001oogoo08moo`0067oo
00<007ooOol0?Woo00<007ooOol0T7oo000HOol00`00Oomoo`0nOol00`00Oomoo`2@Ool001Qoo`03
001oogoo03ioo`03001oogoo091oo`0067oo00<007ooOol0?Woo00<007ooOol0T7oo000HOol00`00
Oomoo`0nOol00`00Oomoo`2@Ool001Qoo`03001oogoo03ioo`03001oogoo091oo`001Goo0P0017oo
0P000goo0`001Goo00<007ooOol0?Goo00<007ooOol0TGoo0004Ool01000Oomoo`000goo0P001Woo
00<007ooOol00Woo00<007ooOol0?Goo00<007ooOol0TGoo0004Ool01000Oomoo`002goo00<007oo
Ool00Woo0P00?Woo00<007ooOol0TGoo0004Ool01000Oomoo`0027oo0`00fGoo0004Ool01000Oomo
o`0027oo00<007ooOol0fGoo0005Ool20009Ool4003HOol00>aoo`00\
\>"],
  ImageRangeCache->{{{0, 235.375}, {145, 0}} -> {32.6956, -0.045028, 
  0.302082, 0.00387919}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Roll-Geske-Whaley Result", "Subsubsection",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We write a formula that uses Black-Scholes if we are past the final \
dividend date, else it checks to see if the dividend is below the critical \
dividend for early exercise. In that case the jump-adjusted Black-Scholes is \
used. If the dividend is above the critical value for early exercise, the RGW \
result is used. Note that formulae are given by both Hull (1996), and Jarrow \
and Rudd (1983) - these are contradictory. I have chosen to use Hull's result \
- there were several papers written by Roll, Geske and Whaley (see the \
Bibliography) and it took several years to get the right result - Jarrow and \
Rudd's book was written before their analysis was finished.\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(\tRGWH[price_, strike_, \ vol_, \ r_, \ div_, \ \[Tau]_, \ 
        tdiv_]\  := \n\t
      Module[{criticalprice, adjprice, \ aone, \ atwo, \ bone, \ btwo}, \n\t
        criticalprice = \ 
          CriticalPrice[\ strike, \ vol, \ r, \ 0, div, \[Tau] - tdiv]; \n\t
        adjprice\  = \ 
          price\  - \ \ div*
              Exp[\(-r\)*tdiv]; \n\t\taone\  = \ \((Log[
                  adjprice/
                    strike]\  + \ \((r\  + \ vol^2/2)\)*\[Tau])\)/\((vol*
                Sqrt[\[Tau]])\); \n\t\tatwo\  = \ 
          aone\  - \ 
            vol*Sqrt[\[Tau]]; \n\t\tbone\  = \ \((Log[
                  adjprice/criticalprice]\  + \ \((r\  + \ vol^2/2)\)*
                  tdiv)\)/\((vol*Sqrt[tdiv])\); \n\t\tbtwo\  = \ 
          bone\  - \ vol*Sqrt[tdiv]; \n\t\tChop[
          adjprice*Norm[bone]\  + \ 
            adjprice*
              Mfunc[aone, \ \(-bone\), \ \(-Sqrt[
                    tdiv/\[Tau]]\)]\  - \ \t\t\t\t\t\tstrike*
              Exp[\(-r\)*\[Tau]]*
              Mfunc[atwo, \ \(-btwo\), \ \ \(-Sqrt[
                    tdiv/\[Tau]]\)]\  - \ \((strike\  - \ div)\)*
              Exp[\(-r\)*tdiv]*Norm[btwo], \ 10^\((\(-4\))\)]]\)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
Here it is wrapped up for an American Call with one discrete \
dividend:\
\>", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    \(AmCallH[price_, \ strike_, \ vol_, \ r_, \[Tau]_, \ div_, 
        tdiv_]\  := \n\t
      Module[{divtime\  = \ \[Tau]\  - tdiv}, \n\t
        If[divtime\  <= \ 0, 
          BlackScholes[price, strike, \ vol, \ r, \ \[Tau]], \n\t\t\tIf[
            div\  <= \ CriticalDiv[strike\ , r, \ divtime], 
            BlackScholesCall[price\  - \ div*Exp[\(-r\)*tdiv], strike, \ 
              vol, \ r, 0, \[Tau]], \n\t\t\t\tRGWH[price, strike, \ \ vol, \ 
              r, \ div, \ \[Tau], tdiv]]]]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["Here is a sample evaluation:", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(AmCallH[100, 100, \ 0.2, \ Log[1.05], 1, 20, 184/365]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    \(5.86668591769450387`\)], "Output",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Packages Re-visited", "Section",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We shall need to refer to the results of the approximate form of the CJM \
model again, when we consider models of American options based on finite \
differences and binomial or trinomial trees. It is therefore desirable to put \
these particular functions into a Package. This is useful for other reasons, \
in that it is a good opportunity to show how to build a Package that uses \
another Package - the CJM model assumes that we already have a model for the \
European Put in place. This is very simply accomplished by adding the name of \
the existing required Package into the ",
  StyleBox["BeginPackage",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " statement. In the following listing we have kept things to a minimum in \
order to focus on just this point and defining just the functions we need, \
but this gives us a prototype that could be used to build Packages out of \
most of the functions in this book.\n\nWe remind the reader that ",
  StyleBox["this particular code is both experimental and approximate",
    FontSlant->"Italic"],
  " - it should not be used in isolation - our purpose here is mainly to make \
it easy to re-load the CJM functions when we need them. Do not execute the \
following listing."
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True],

Cell["\<\
(* :Title: 
\tApproximate American Options *)

(* :Context: 
\tDerivatives`AnalyticAmerican` *)

(* :Author: 
\tDr William T. Shaw *)

(* :Summary: 
\t*)

(* :Package Version: 
\t1.0 *)

(* :Mathematica Version: 
\t3.0 *)

(* :History:
\tV.1.0, January 1997, by William T. Shaw *)


BeginPackage[\"Derivatives`AnalyticAmerican`\", \
\"Derivatives`BlackScholes`\"]

(* Usage Information *)

Front::usage = 
\"Front[strike, interest, vol, t] returns the critical price frontier for a 
vanilla zero-dividend American Put according to an approximation of the Carr-
Jarrow-Myneni integral equation.\"

AmericanPutCJM::usage = 
\"AmericanPutCJM[price,strike,vol,r,t]  returns an APPROXIMATE value for a 
vanilla zero-dividend American Put according to an approximation of the Carr-
Jarrow-Myneni integral equation.\"


AmericanPutDeltaCJM::usage = 
\"AmericanPutDeltaCJM[price,strike,vol,r,t]  returns an APPROXIMATE value for \

the delta of a vanilla zero-dividend American Put according to an 
approximation of the Carr-Jarrow-Myneni integral equation.\"


AmericanPutGammaCJM::usage = 
\"AmericanPutGammaCJM[price,strike,vol,r,t]  returns an APPROXIMATE value for \

the gamma of a vanilla zero-dividend American Put according to an 
approximation of the Carr-Jarrow-Myneni integral equation.\"


(* END OF USAGE INFORMATION *)

Begin[\"`Private`\"]

Front[strike_, r_, sigma_, t_] :=
If[t==0,strike,Module[{l,m},
l= m/. FindRoot[Exp[-m*sigma*Sqrt[t] - (r+sigma^2/2)*t]*(1+Erf[-m/Sqrt[2]]) \
==
 r*Sqrt[2/(r+(sigma - (r/(2*sigma)+sigma/4) +m/(2*Sqrt[t]))^2/2)]*Erf[Sqrt[(
r+(sigma-(r/(2*sigma)+sigma/4)+m/(2*Sqrt[t]))^2/2)*t]]/sigma,
{m,-(sigma/2+r/sigma)*Sqrt[t]},MaxIterations -> 30];
strike*Exp[-l*sigma*Sqrt[t] - (r+sigma^2/2)*t]]]

AmericanPutCJM[price_,strike_,sigma_,r_,t_] :=
If[price <= Front[strike, r, sigma, t], strike - price,
BlackScholesPut[price,strike,sigma,r,0,t]+r*strike*NIntegrate[Exp[-r*u]*Norm[(\

Log[Front[strike,r,sigma,t-u]/price]-(r-sigma^2/2)*u)/(sigma*Sqrt[u])],
{u,0,t}]]

AmericanPutDeltaCJM[price_,strike_,sigma_,r_,t_] :=
If[price <= Front[strike, r, sigma, t],  -1,
N[BlackScholesPutDelta[price,strike,sigma,r,0,t] - (r*strike/(price*sigma*
Sqrt[2.*Pi]))*NIntegrate[Exp[-r*u]*Exp[-((Log[Front[strike,r,sigma,t-u]/
price]-(r-sigma^2/2)*u)/(sigma*Sqrt[u]))^2/2]/Sqrt[u],
{u, 0, t}]]]

AmericanPutGammaCJM[price_,strike_,sigma_,r_,t_] :=
If[price <= Front[strike, r, sigma, t], 0,
N[
BlackScholesPutGamma[price,strike,sigma,r,0,t]+(r*strike/(price^2*sigma*Sqrt[
2.*Pi]))*NIntegrate[Exp[-r*u]*Exp[-((Log[Front[strike,r,sigma,t-u]/price]-(r-
sigma^2/2)*u)/(sigma*Sqrt[u]))^2/2]*(1/Sqrt[u] - (Log[Front[strike,r,sigma,t-
u]/price]-(r-sigma^2/2)*u)/(sigma*Sqrt[u])/(sigma*u)),
{u, 0, t}]]]


End[]

EndPackage[]\
\>", "Input",
  PageWidth->Infinity,
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  InitializationCell->True,
  ShowSpecialCharacters->False,
  FormatType->InputForm,
  FontSize->8],

Cell[TextData[{
  "The contents of this file, called ",
  StyleBox["AnalyticAmerican.m",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", should be placed in the Derivatives folder or directory with the ",
  StyleBox["BlackScholes.m",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " Package. We now test it. First we quit the kernel, and restart a fresh \
session by loading the package."
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Quit[]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    \(Needs["\<Derivatives`AnalyticAmerican`\>"]\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Derivatives`AnalyticAmerican`*\)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
AmericanPutCJM      AmericanPutDeltaCJM AmericanPutGammaCJM Front\
\
\>", "Print",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?AmericanPutCJM\)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    \("AmericanPutCJM[price,strike,vol,r,t]  returns an APPROXIMATE value for \
a vanilla zero-dividend American Put according to an approximation of the \
Carr-Jarrow-Myneni integral equation."\)], "Print"]
}, Open  ]],

Cell["Note that the European formulae are now also loaded:", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?BlackScholesPut\)\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    \("BlackScholesPut[price, strike, vol, riskfree, divyield, expiry]  \
returns the Black-Scholes value of a vanilla European Put."\)], "Print",
  PageBreakAbove->False]
}, Open  ]],

Cell["We can check that the Package functions work as before:", "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \({AmericanPutCJM[50, \ 50, \ 0.4, \ 0.1, \ 1], \ 
      AmericanPutDeltaCJM[50, \ 50, \ 0.4, \ 0.1, \ 1], \ 
      AmericanPutGammaCJM[50, \ 50, \ 0.4, \ 0.1, \ 1]}\)], "Input",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    \({5.97688400913239092`, \(-0.377895750698134147`\), 
      0.0229193549454019818`}\)], "Output",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 11 Bibliography", "Section",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[TextData[{
  "Barone-Adesi, G. and Whaley, R., 1987, Efficient analytic approximation of \
American option values, ",
  StyleBox["Journal of Finance",
    FontSlant->"Italic"],
  ", 42, p. 301.\n\nCarr, P., Jarrow, R. and Myneni, R., 1992,  Alternative \
characterization of American put options, ",
  StyleBox["Mathematical Finance",
    FontSlant->"Italic"],
  ", 2, p. 87.\n\nCrank, J., 1984, ",
  StyleBox["Free and Moving Boundary Value Problems",
    FontSlant->"Italic"],
  ", Clarendon Press.\n\nGeske, R., 1979, A note on an analytic valuation \
formula for unprotected American call options on stocks with known dividends, \
",
  StyleBox["Journal of Financial Economics",
    FontSlant->"Italic"],
  ", 7, p. 375.\n\nGeske, R., 1981, Comments on Whaley's note, ",
  StyleBox["Journal of Financial Economics",
    FontSlant->"Italic"],
  ", 9, p. 213.\n\nGeske, R. and Johnson, H.E., 1984, The American put option \
valued analytically, ",
  StyleBox["Journal of Finance",
    FontSlant->"Italic"],
  ", 39, p. 1511.\n\nHill, J.M. and Dewynne, J.N., 1985a, A note on \
Langford's cylinder functions, ",
  StyleBox["Quarterly Journal of Applied Mathematics",
    FontSlant->"Italic"],
  ", 43, p. 179.\n\nHill, J.M. and Dewynne, J.N., 1985b, Bounds for moving \
boundary problems with two chemical reactions, ",
  StyleBox["Nonlinear Analysis, Theory, Methods and Applications",
    FontSlant->"Italic"],
  ", 9, p. 1293.\n\nHull, J.C., 1996, ",
  StyleBox["Options, Futures, and Other Derivatives",
    FontSlant->"Italic"],
  ", 3rd edition, Prentice-Hall.\n\nJarrow, R.A. and Rudd, A., 1983, ",
  StyleBox["Option Pricing",
    FontSlant->"Italic"],
  ", Irwin.\n\nMacMillan, L., 1986, Analytic approximation for the American \
put option, ",
  StyleBox["Advances in Futures and Options Research,",
    FontSlant->"Italic"],
  " 1, p. 119.\n\nMcKean, Jr, H.P., 1965, A free boundary problem for the \
heat equation arising from a problem in mathematical economics, ",
  StyleBox["Ind. Management Rev",
    FontSlant->"Italic"],
  ". 6, p. 32.\n\nOckendon, J.R., 1974, Techniques of analysis, in ",
  StyleBox["Moving Boundary Problems in Heat Flow and Diffusion",
    FontSlant->"Italic"],
  ", ed. Ockendon and Hodgkins, Oxford University Press.\n\nRoll, R., 1977, \
An analytic formula for unprotected American call options on stocks with \
known dividends, ",
  StyleBox["Journal of Financial Economics,",
    FontSlant->"Italic"],
  " 5, p. 251.\n\nShaw, W. T., 1989, Redox front motion in a system of \
bentonite and fractured rock, ",
  StyleBox["Swedish Nuclear Power Inspectorate Technical Report",
    FontSlant->"Italic"],
  ", SKI TR 89:7 (Appendix on moving boundary problems available from WTS).\n\
\nVan Moerbeke, P., 1976, On optimal stopping and free boundary problems, ",
  StyleBox["Arch. Rational Mech. Anal.",
    FontSlant->"Italic"],
  ", 60, p. 101.\n\nWhaley, R., 1981, On the valuation of American call \
options on stocks with known dividends, ",
  StyleBox["Journal of Financial Economics,",
    FontSlant->"Italic"],
  " 9, p. 207."
}], "Text",
  CellMargins->{{6.1875, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{561, 626},
WindowMargins->{{4, Automatic}, {Automatic, 1}},
PrintingCopies->1,
PrintingStartingPageNumber->215,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"11. Analytical Americans"}]]}, {Cell[ 
        TextData[ {"11. Analytical Americans"}]], Inherited, Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True,
"FirstPageFace"->Right},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
PrivateFontOptions->{"FontType"->"Outline"},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8`?n3ohP==PTh0b85N`?P0080004/0B`000006G<AP001
0000I00000400@4100800BL?00400@S</X0000000000064/01T1T000000@0000
00000000007oo@S</T3oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 241, 9, 171, "Title"],
Cell[1975, 62, 170, 4, 45, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2170, 70, 146, 3, 49, "Section"],
Cell[2319, 75, 2533, 39, 419, "Text"],

Cell[CellGroupData[{
Cell[4877, 118, 123, 1, 38, "Subsubsection"],
Cell[5003, 121, 187, 4, 28, "Text"],
Cell[5193, 127, 133, 2, 25, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5375, 135, 154, 4, 65, "Section"],
Cell[5532, 141, 542, 10, 58, "Text"],
Cell[6077, 153, 146, 2, 20, "NumberedEquation"],
Cell[6226, 157, 183, 3, 24, "NumberedEquation"],
Cell[6412, 162, 193, 6, 20, "Text"],
Cell[6608, 170, 183, 3, 25, "NumberedEquation"],
Cell[6794, 175, 335, 7, 32, "NumberedEquation"],
Cell[7132, 184, 228, 6, 20, "Text"],
Cell[7363, 192, 294, 6, 32, "NumberedEquation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7694, 203, 135, 1, 38, "Section"],
Cell[7832, 206, 389, 8, 40, "Text"],
Cell[8224, 216, 140, 2, 20, "NumberedEquation"],
Cell[8367, 220, 311, 9, 20, "Text"],
Cell[8681, 231, 139, 2, 20, "NumberedEquation"],
Cell[8823, 235, 192, 3, 24, "NumberedEquation"],
Cell[9018, 240, 192, 6, 20, "Text"],
Cell[9213, 248, 208, 3, 25, "NumberedEquation"],
Cell[9424, 253, 521, 12, 32, "NumberedEquation"],
Cell[9948, 267, 234, 6, 20, "Text"],
Cell[10185, 275, 314, 6, 32, "NumberedEquation"],
Cell[10502, 283, 186, 4, 20, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10725, 292, 134, 1, 38, "Section"],
Cell[10862, 295, 438, 7, 52, "Text"],
Cell[11303, 304, 261, 4, 32, "NumberedEquation"],
Cell[11567, 310, 380, 7, 40, "Text"],
Cell[11950, 319, 323, 5, 29, "NumberedEquation"],
Cell[12276, 326, 89, 1, 20, "Text"],
Cell[12368, 329, 173, 3, 20, "NumberedEquation"],
Cell[12544, 334, 86, 1, 20, "Text"],
Cell[12633, 337, 797, 18, 32, "NumberedEquation"],
Cell[13433, 357, 113, 1, 20, "Text"],
Cell[13549, 360, 619, 15, 32, "NumberedEquation"],
Cell[14171, 377, 409, 11, 29, "Text"],
Cell[14583, 390, 174, 3, 32, "NumberedEquation"],
Cell[14760, 395, 84, 1, 20, "Text"],
Cell[14847, 398, 210, 4, 42, "NumberedEquation"],
Cell[15060, 404, 813, 22, 52, "Text"],
Cell[15876, 428, 206, 3, 32, "NumberedEquation"],
Cell[16085, 433, 166, 4, 20, "Text"],
Cell[16254, 439, 216, 4, 32, "NumberedEquation"],
Cell[16473, 445, 179, 6, 20, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16689, 456, 131, 1, 38, "Section"],
Cell[16823, 459, 239, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[17087, 468, 115, 2, 25, "Input"],
Cell[17205, 472, 160, 2, 32, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17402, 479, 120, 2, 25, "Input"],
Cell[17525, 483, 151, 2, 32, "Print"]
}, Open  ]],
Cell[17691, 488, 163, 4, 20, "Text"],
Cell[17857, 494, 512, 8, 118, "Input"],
Cell[18372, 504, 246, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[18643, 514, 135, 2, 25, "Input"],
Cell[18781, 518, 245, 4, 69, "Output"]
}, Open  ]],
Cell[19041, 525, 260, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[19326, 534, 371, 6, 69, "Input"],
Cell[19700, 542, 657, 11, 164, "Output"]
}, Open  ]],
Cell[20372, 556, 234, 5, 29, "Text"],
Cell[20609, 563, 392, 7, 76, "Input"],
Cell[21004, 572, 156, 4, 20, "Text"],
Cell[21163, 578, 216, 4, 37, "Input"],

Cell[CellGroupData[{
Cell[21404, 586, 240, 5, 37, "Input"],
Cell[21647, 593, 8877, 388, 117, 4691, 331, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[30539, 984, 428, 11, 29, "Text"],
Cell[30970, 997, 175, 3, 36, "NumberedEquation"],
Cell[31148, 1002, 163, 6, 20, "Text"],
Cell[31314, 1010, 170, 3, 44, "Input"],
Cell[31487, 1015, 164, 4, 20, "Text"],
Cell[31654, 1021, 264, 5, 37, "Input"],

Cell[CellGroupData[{
Cell[31943, 1030, 128, 2, 25, "Input"],
Cell[32074, 1034, 9115, 412, 117, 5032, 357, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[41204, 1449, 238, 6, 20, "Text"],
Cell[41445, 1457, 273, 4, 53, "Input"],
Cell[41721, 1463, 165, 4, 20, "Text"],
Cell[41889, 1469, 635, 11, 88, "Input"],
Cell[42527, 1482, 251, 6, 29, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[42803, 1492, 169, 3, 25, "Input",
  PageBreakAbove->False],
Cell[42975, 1497, 79, 2, 36, "Output",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[43091, 1504, 169, 3, 25, "Input",
  PageBreakAbove->False],
Cell[43263, 1509, 79, 2, 36, "Output",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[43379, 1516, 278, 5, 37, "Input",
  PageBreakAbove->False],
Cell[43660, 1523, 10112, 322, 133, 3289, 232, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[53821, 1851, 156, 2, 38, "Section",
  PageBreakAbove->False],
Cell[53980, 1855, 2689, 42, 304, "Text",
  PageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[56706, 1902, 117, 1, 38, "Section"],
Cell[56826, 1905, 618, 11, 62, "Text"],
Cell[57447, 1918, 153, 2, 32, "NumberedEquation"],
Cell[57603, 1922, 185, 3, 32, "NumberedEquation"],
Cell[57791, 1927, 364, 10, 29, "Text"],
Cell[58158, 1939, 292, 5, 37, "NumberedEquation"],
Cell[58453, 1946, 235, 5, 29, "Text"],
Cell[58691, 1953, 323, 5, 38, "NumberedEquation"],
Cell[59017, 1960, 83, 1, 20, "Text"],
Cell[59103, 1963, 186, 3, 37, "NumberedEquation"],
Cell[59292, 1968, 156, 4, 20, "Text"],
Cell[59451, 1974, 364, 6, 36, "NumberedEquation"],
Cell[59818, 1982, 130, 2, 20, "NumberedEquation"],
Cell[59951, 1986, 587, 13, 52, "Text"],
Cell[60541, 2001, 212, 4, 22, "NumberedEquation"],
Cell[60756, 2007, 205, 6, 20, "Text"],
Cell[60964, 2015, 211, 3, 36, "NumberedEquation"],
Cell[61178, 2020, 195, 4, 20, "Text"],
Cell[61376, 2026, 342, 5, 38, "NumberedEquation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[61755, 2036, 130, 1, 38, "Section"],
Cell[61888, 2039, 560, 12, 52, "Text"],
Cell[62451, 2053, 245, 4, 36, "NumberedEquation"],
Cell[62699, 2059, 191, 6, 20, "Text"],
Cell[62893, 2067, 221, 4, 33, "NumberedEquation"],
Cell[63117, 2073, 272, 7, 29, "Text"],
Cell[63392, 2082, 188, 3, 24, "NumberedEquation"],
Cell[63583, 2087, 110, 1, 20, "Text"],
Cell[63696, 2090, 590, 12, 36, "NumberedEquation"],
Cell[64289, 2104, 315, 7, 29, "Text"],
Cell[64607, 2113, 261, 4, 38, "NumberedEquation"],
Cell[64871, 2119, 295, 7, 29, "Text"],
Cell[65169, 2128, 348, 5, 48, "NumberedEquation"],
Cell[65520, 2135, 263, 7, 29, "Text"],

Cell[CellGroupData[{
Cell[65808, 2146, 102, 1, 32, "Subsection"],
Cell[65913, 2149, 97, 1, 20, "Text"],
Cell[66013, 2152, 404, 8, 34, "NumberedEquation"],
Cell[66420, 2162, 830, 16, 73, "Text"],
Cell[67253, 2180, 208, 3, 29, "NumberedEquation"],
Cell[67464, 2185, 244, 7, 20, "Text"],
Cell[67711, 2194, 222, 3, 37, "NumberedEquation"],
Cell[67936, 2199, 476, 15, 29, "Text"],
Cell[68415, 2216, 176, 3, 28, "NumberedEquation"],
Cell[68594, 2221, 209, 6, 20, "Text"],
Cell[68806, 2229, 159, 3, 20, "NumberedEquation"],
Cell[68968, 2234, 151, 4, 20, "Text"],
Cell[69122, 2240, 231, 4, 36, "NumberedEquation"],
Cell[69356, 2246, 92, 1, 20, "Text"],
Cell[69451, 2249, 218, 3, 38, "NumberedEquation"],
Cell[69672, 2254, 472, 12, 29, "Text"],
Cell[70147, 2268, 389, 8, 30, "NumberedEquation"],
Cell[70539, 2278, 92, 1, 20, "Text"],
Cell[70634, 2281, 207, 3, 33, "NumberedEquation"],
Cell[70844, 2286, 121, 1, 20, "Text"],
Cell[70968, 2289, 348, 5, 48, "NumberedEquation"],
Cell[71319, 2296, 1073, 19, 97, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[72441, 2321, 194, 6, 38, "Section"],
Cell[72638, 2329, 257, 5, 29, "Text"],
Cell[72898, 2336, 496, 11, 101, "Input"],
Cell[73397, 2349, 377, 9, 80, "Input"],
Cell[73777, 2360, 427, 10, 90, "Input"],
Cell[74207, 2372, 516, 14, 145, "Input"],
Cell[74726, 2388, 172, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[74923, 2396, 410, 8, 62, "Input"],
Cell[75336, 2406, 8788, 344, 133, 3834, 278, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[84139, 2753, 167, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[84331, 2761, 124, 2, 25, "Input"],
Cell[84458, 2765, 10318, 447, 133, 5437, 382, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[94791, 3215, 200, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[95016, 3223, 111, 1, 28, "Subsubsection"],
Cell[95130, 3226, 86, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[95241, 3231, 144, 2, 25, "Input"],
Cell[95388, 3235, 53, 1, 36, "Output"]
}, Open  ]],
Cell[95456, 3239, 91, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[95572, 3244, 144, 2, 25, "Input"],
Cell[95719, 3248, 54, 1, 36, "Output"]
}, Open  ]],
Cell[95788, 3252, 90, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[95903, 3257, 138, 2, 25, "Input"],
Cell[96044, 3261, 54, 1, 36, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[96147, 3268, 111, 1, 28, "Subsubsection"],
Cell[96261, 3271, 86, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[96372, 3276, 144, 2, 25, "Input"],
Cell[96519, 3280, 54, 1, 36, "Output"]
}, Open  ]],
Cell[96588, 3284, 91, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[96704, 3289, 144, 2, 25, "Input"],
Cell[96851, 3293, 54, 1, 36, "Output"]
}, Open  ]],
Cell[96920, 3297, 90, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[97035, 3302, 138, 2, 25, "Input"],
Cell[97176, 3306, 54, 1, 36, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[97279, 3313, 115, 1, 28, "Subsubsection"],
Cell[97397, 3316, 86, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[97508, 3321, 144, 2, 25, "Input"],
Cell[97655, 3325, 54, 1, 36, "Output"]
}, Open  ]],
Cell[97724, 3329, 91, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[97840, 3334, 144, 2, 25, "Input"],
Cell[97987, 3338, 54, 1, 36, "Output"]
}, Open  ]],
Cell[98056, 3342, 90, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[98171, 3347, 138, 2, 25, "Input"],
Cell[98312, 3351, 54, 1, 36, "Output"]
}, Open  ]],
Cell[98381, 3355, 424, 7, 52, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[98854, 3368, 134, 1, 38, "Section"],
Cell[98991, 3371, 896, 15, 96, "Text"],
Cell[99890, 3388, 171, 3, 25, "Input",
  InitializationCell->True],
Cell[100064, 3393, 160, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[100249, 3401, 136, 2, 25, "Input"],
Cell[100388, 3405, 221, 5, 40, "Print"]
}, Open  ]],
Cell[100624, 3413, 134, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[100783, 3418, 123, 2, 25, "Input"],
Cell[100909, 3422, 363, 5, 81, "Print"]
}, Open  ]],
Cell[101287, 3430, 152, 4, 20, "Text"],
Cell[101442, 3436, 162, 3, 37, "Input"],
Cell[101607, 3441, 114, 1, 20, "Text"],
Cell[101724, 3444, 191, 3, 37, "Input"],

Cell[CellGroupData[{
Cell[101940, 3451, 181, 4, 25, "Input"],
Cell[102124, 3457, 53141, 1134, 201, 20092, 721, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[155302, 4596, 127, 1, 28, "Subsubsection"],
Cell[155432, 4599, 387, 8, 40, "Text"],
Cell[155822, 4609, 196, 3, 20, "NumberedEquation"],
Cell[156021, 4614, 190, 4, 25, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[156236, 4622, 124, 2, 25, "Input"],
Cell[156363, 4626, 53, 1, 36, "Output"]
}, Open  ]],
Cell[156431, 4630, 199, 4, 29, "Text"],
Cell[156633, 4636, 406, 8, 62, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[157064, 4648, 144, 2, 25, "Input"],
Cell[157211, 4652, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[157302, 4658, 144, 2, 25, "Input"],
Cell[157449, 4662, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[157540, 4668, 152, 2, 25, "Input"],
Cell[157695, 4672, 54, 1, 36, "Output"]
}, Open  ]],
Cell[157764, 4676, 214, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[158003, 4684, 147, 2, 25, "Input"],
Cell[158153, 4688, 200, 4, 44, "Message"],
Cell[158356, 4694, 913, 22, 60, "Output"]
}, Open  ]],
Cell[159284, 4719, 293, 6, 29, "Text"],
Cell[159580, 4727, 347, 5, 62, "Input"],

Cell[CellGroupData[{
Cell[159952, 4736, 149, 2, 25, "Input"],
Cell[160104, 4740, 11520, 381, 153, 3788, 281, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[171673, 5127, 115, 1, 28, "Subsubsection"],
Cell[171791, 5130, 762, 11, 84, "Text"],
Cell[172556, 5143, 1301, 26, 176, "Input",
  PageBreakAbove->False],
Cell[173860, 5171, 183, 5, 20, "Text",
  PageBreakAbove->False],
Cell[174046, 5178, 613, 11, 88, "Input",
  PageBreakAbove->False],
Cell[174662, 5191, 131, 2, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[174818, 5197, 173, 3, 25, "Input",
  PageBreakAbove->False],
Cell[174994, 5202, 79, 2, 36, "Output",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[175134, 5211, 100, 1, 38, "Section"],
Cell[175237, 5214, 1356, 23, 150, "Text",
  PageBreakWithin->True],
Cell[176596, 5239, 2977, 100, 796, "Input",
  InitializationCell->True],
Cell[179576, 5341, 478, 12, 40, "Text"],
Cell[180057, 5355, 126, 3, 25, "Input",
  PageBreakAbove->False],
Cell[180186, 5360, 162, 3, 25, "Input",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[180373, 5367, 155, 3, 25, "Input",
  PageBreakAbove->False],
Cell[180531, 5372, 179, 5, 20, "Print",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[180747, 5382, 139, 3, 25, "Input",
  PageBreakAbove->False],
Cell[180889, 5387, 221, 3, 44, "Print"]
}, Open  ]],
Cell[181125, 5393, 155, 2, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[181305, 5399, 140, 3, 25, "Input",
  PageBreakAbove->False],
Cell[181448, 5404, 185, 3, 32, "Print",
  PageBreakAbove->False]
}, Open  ]],
Cell[181648, 5410, 158, 2, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[181831, 5416, 283, 5, 50, "Input",
  PageBreakAbove->False],
Cell[182117, 5423, 140, 3, 36, "Output",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[182306, 5432, 129, 2, 38, "Section",
  PageBreakAbove->False],
Cell[182438, 5436, 3176, 69, 514, "Text",
  PageBreakWithin->True]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

