(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    272972,      10287]*)
(*NotebookOutlinePosition[    274863,      10344]*)
(*  CellTagsIndexPosition[    274819,      10340]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 14.
Finite-Difference Schemes for the 
Diffusion Equation with Smooth 
Initial Conditions\
\>", "Title",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["\<\
Explicit, Implicit, Crank-Nicholson and Douglas
Finite-Difference Schemes Compared\
\>", "Subsubtitle",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["14.1 Introduction", "Section",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell["\<\
In binomial models there is a link between the time-step and the \
price-step in the binomial tree. Accuracy requires a small price-step, and \
this implies a small time-step. For long times to expiry many thousands of \
time-steps may be needed to get an accurate result. In finite-difference (FD) \
schemes, explicit schemes have the same properties in this respect as \
binomial models, and there are finite-difference analogues of both binomial \
and trinomial schemes. Much more interesting are the implicit schemes. These \
allow much larger time-steps with no appreciable loss in accuracy, and \
therefore in principle offer much more efficient computation.

Furthermore, the nature of these schemes is such that they allow rather more \
accurate and straightforward computations of Delta, Gamma and Theta to be \
given than with (a standard implementation of) a binomial model. Also, there \
is an increasing understanding of how to map many types of option-pricing \
problems into finite-difference form. \
\>", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["14.2 Schemes Investigated", "Section",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
In this chapter we compare the accuracy of various difference \
schemes for solving the diffusion equation. This is the equation that arises \
when the Black-Scholes differential equation is transformed into a form \
suitable for treatment by finite-difference methods. We compare

(a) explicit finite-difference, with 400 time-steps (equivalent to the use of \
a binomial model, but on a grid rather than a tree);
(b) fully implicit, also with 400 time-steps;
(c) Crank-Nicholson, with 40 time-steps;
(d) Douglas, with 40 time-steps.

The solution method for type (a) is a simple updating rule, while (b), (c), \
(d) require the solution of tridiagonal systems of equations. \
\>", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "14.3 ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Solution Issues"
}], "Section",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We will be able to write down a closed-form solution and implement it in ",
  
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". Then we shall describe two types of solution method that work for FD \
schemes applied to the ordinary diffusion equation and European-style \
options. The first is an explicit and compiled solver, while the second uses \
a compiled solver for tridiagonal systems. We shall see later how to \
generalize these to compiled SOR and PSOR solvers. Another use of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is to automatically interpolate the solution using various types of \
interpolation scheme."
}], "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["14.4 A Simple Test Problem with Smooth Initial Conditions", "Section",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell["We consider the diffusion equation", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[PartialD]u\/\[PartialD]\[Tau] = \[PartialD]\^2 u\/\
\[PartialD]x\^2\)], "NumberedEquation",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell["on the region defined by", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(-2\) \[LessEqual] x \[LessEqual] 
      2\ \ \ \ \ \ \ \ \ \ \[Tau] \[GreaterEqual] 0\)], "NumberedEquation",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell["The initial condition is", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`u(x, 0) = sin(\(\[Pi]  x\)\/2)\)], "NumberedEquation",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell["and the boundary conditions are", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`u(2, \[Tau]) = \(u(\(-2\), \[Tau]) = 0\)\)], 
  "NumberedEquation",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell["This has the exact solution", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`u(
        x, \[Tau]) = \(sin(\(\[Pi]  x\)\/2)\)\ \
\[ExponentialE]\^\(-\(\(\[Pi]\^2\ \[Tau]\)\/4\)\)\)], "NumberedEquation",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "So it is a simple matter to test various difference schemes by comparing \
with this known exact solution. It should be emphasized that this type of \
smooth initial data, which also joins continuously onto the boundary \
conditions, is rather atypical of option-pricing problems. Our purpose here \
is to simplify matters to get a general feel for the relative merits of \
explicit and implicit schemes, and to introduce the idea of compiling within \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". "
}], "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["14.5 Explicit Scheme", "Section",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Here are the price- and time-steps used, together with an output that is \
the value of the parameter ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = \[CapitalDelta]\[Tau]\/\(\[CapitalDelta]\ \
x\^2\)\)]],
  "."
}], "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["dx = 0.025; dtau = 0.00025; alpha = dtau/dx^2", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(0.399999999999999964`\)], "Output",
  PageBreakWithin->True,
  GroupPageBreakWithin->True]
}, Open  ]],

Cell["\<\
The number of time-steps is 400, and there are 160 \
space-steps.\
\>", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell["M=400; nminus = 80; nplus = 80;", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Here we set initial and boundary conditions, and plot the latter:\
\
\>", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(initial = 
      Table[N[Sin[Pi*\((k - 1 - nminus)\)/nminus]\ ], \ {k, 1, \ 
          nminus + nplus + 1}]; \nlower = Table[0, \ {m, \ 1, \ M + 1}]; \n
    upper = Table[0, \ {m, \ 1, \ M + 1}]; \)], "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["ListPlot[initial];", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00591541 0.309017 0.294302 [
[.17169 .29652 -6 -9 ]
[.17169 .29652 6 0 ]
[.31958 .29652 -6 -9 ]
[.31958 .29652 6 0 ]
[.46747 .29652 -6 -9 ]
[.46747 .29652 6 0 ]
[.61535 .29652 -9 -9 ]
[.61535 .29652 9 0 ]
[.76324 .29652 -9 -9 ]
[.76324 .29652 9 0 ]
[.91112 .29652 -9 -9 ]
[.91112 .29652 9 0 ]
[.01131 .01472 -12 -4.5 ]
[.01131 .01472 0 4.5 ]
[.01131 .16187 -24 -4.5 ]
[.01131 .16187 0 4.5 ]
[.01131 .45617 -18 -4.5 ]
[.01131 .45617 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17169 .30902 m
.17169 .31527 L
s
[(25)] .17169 .29652 0 1 Mshowa
.31958 .30902 m
.31958 .31527 L
s
[(50)] .31958 .29652 0 1 Mshowa
.46747 .30902 m
.46747 .31527 L
s
[(75)] .46747 .29652 0 1 Mshowa
.61535 .30902 m
.61535 .31527 L
s
[(100)] .61535 .29652 0 1 Mshowa
.76324 .30902 m
.76324 .31527 L
s
[(125)] .76324 .29652 0 1 Mshowa
.91112 .30902 m
.91112 .31527 L
s
[(150)] .91112 .29652 0 1 Mshowa
.125 Mabswid
.05339 .30902 m
.05339 .31277 L
s
.08296 .30902 m
.08296 .31277 L
s
.11254 .30902 m
.11254 .31277 L
s
.14212 .30902 m
.14212 .31277 L
s
.20127 .30902 m
.20127 .31277 L
s
.23085 .30902 m
.23085 .31277 L
s
.26043 .30902 m
.26043 .31277 L
s
.29 .30902 m
.29 .31277 L
s
.34916 .30902 m
.34916 .31277 L
s
.37873 .30902 m
.37873 .31277 L
s
.40831 .30902 m
.40831 .31277 L
s
.43789 .30902 m
.43789 .31277 L
s
.49704 .30902 m
.49704 .31277 L
s
.52662 .30902 m
.52662 .31277 L
s
.5562 .30902 m
.5562 .31277 L
s
.58577 .30902 m
.58577 .31277 L
s
.64493 .30902 m
.64493 .31277 L
s
.6745 .30902 m
.6745 .31277 L
s
.70408 .30902 m
.70408 .31277 L
s
.73366 .30902 m
.73366 .31277 L
s
.79281 .30902 m
.79281 .31277 L
s
.82239 .30902 m
.82239 .31277 L
s
.85197 .30902 m
.85197 .31277 L
s
.88154 .30902 m
.88154 .31277 L
s
.9407 .30902 m
.9407 .31277 L
s
.97028 .30902 m
.97028 .31277 L
s
.99985 .30902 m
.99985 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.02381 .01472 m
.03006 .01472 L
s
[(-1)] .01131 .01472 1 0 Mshowa
.02381 .16187 m
.03006 .16187 L
s
[(-0.5)] .01131 .16187 1 0 Mshowa
.02381 .45617 m
.03006 .45617 L
s
[(0.5)] .01131 .45617 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(1)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .04415 m
.02756 .04415 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.02972 .30902 Mdot
.03564 .29746 Mdot
.04156 .28593 Mdot
.04747 .27443 Mdot
.05339 .26298 Mdot
.0593 .2516 Mdot
.06522 .24031 Mdot
.07113 .22913 Mdot
.07705 .21807 Mdot
.08296 .20715 Mdot
.08888 .19639 Mdot
.09479 .1858 Mdot
.10071 .17541 Mdot
.10663 .16521 Mdot
.11254 .15524 Mdot
.11846 .14551 Mdot
.12437 .13603 Mdot
.13029 .12682 Mdot
.1362 .11788 Mdot
.14212 .10924 Mdot
.14803 .10091 Mdot
.15395 .0929 Mdot
.15986 .08523 Mdot
.16578 .0779 Mdot
.17169 .07092 Mdot
.17761 .06431 Mdot
.18353 .05808 Mdot
.18944 .05224 Mdot
.19536 .04679 Mdot
.20127 .04175 Mdot
.20719 .03712 Mdot
.2131 .03291 Mdot
.21902 .02912 Mdot
.22493 .02576 Mdot
.23085 .02285 Mdot
.23676 .02037 Mdot
.24268 .01834 Mdot
.2486 .01676 Mdot
.25451 .01562 Mdot
.26043 .01494 Mdot
.26634 .01472 Mdot
.27226 .01494 Mdot
.27817 .01562 Mdot
.28409 .01676 Mdot
.29 .01834 Mdot
.29592 .02037 Mdot
.30183 .02285 Mdot
.30775 .02576 Mdot
.31366 .02912 Mdot
.31958 .03291 Mdot
.3255 .03712 Mdot
.33141 .04175 Mdot
.33733 .04679 Mdot
.34324 .05224 Mdot
.34916 .05808 Mdot
.35507 .06431 Mdot
.36099 .07092 Mdot
.3669 .0779 Mdot
.37282 .08523 Mdot
.37873 .0929 Mdot
.38465 .10091 Mdot
.39056 .10924 Mdot
.39648 .11788 Mdot
.4024 .12682 Mdot
.40831 .13603 Mdot
.41423 .14551 Mdot
.42014 .15524 Mdot
.42606 .16521 Mdot
.43197 .17541 Mdot
.43789 .1858 Mdot
.4438 .19639 Mdot
.44972 .20715 Mdot
.45563 .21807 Mdot
.46155 .22913 Mdot
.46747 .24031 Mdot
.47338 .2516 Mdot
.4793 .26298 Mdot
.48521 .27443 Mdot
.49113 .28593 Mdot
.49704 .29746 Mdot
.50296 .30902 Mdot
.50887 .32057 Mdot
.51479 .33211 Mdot
.5207 .34361 Mdot
.52662 .35506 Mdot
.53253 .36643 Mdot
.53845 .37772 Mdot
.54437 .3889 Mdot
.55028 .39996 Mdot
.5562 .41088 Mdot
.56211 .42164 Mdot
.56803 .43223 Mdot
.57394 .44263 Mdot
.57986 .45282 Mdot
.58577 .46279 Mdot
.59169 .47252 Mdot
.5976 .482 Mdot
.60352 .49122 Mdot
.60944 .50015 Mdot
.61535 .50879 Mdot
.62127 .51712 Mdot
.62718 .52513 Mdot
.6331 .53281 Mdot
.63901 .54014 Mdot
.64493 .54711 Mdot
.65084 .55372 Mdot
.65676 .55995 Mdot
.66267 .56579 Mdot
.66859 .57124 Mdot
.6745 .57629 Mdot
.68042 .58092 Mdot
.68634 .58513 Mdot
.69225 .58891 Mdot
.69817 .59227 Mdot
.70408 .59519 Mdot
.71 .59766 Mdot
.71591 .5997 Mdot
.72183 .60128 Mdot
.72774 .60241 Mdot
.73366 .60309 Mdot
.73957 .60332 Mdot
.74549 .60309 Mdot
.7514 .60241 Mdot
.75732 .60128 Mdot
.76324 .5997 Mdot
.76915 .59766 Mdot
.77507 .59519 Mdot
.78098 .59227 Mdot
.7869 .58891 Mdot
.79281 .58513 Mdot
.79873 .58092 Mdot
.80464 .57629 Mdot
.81056 .57124 Mdot
.81647 .56579 Mdot
.82239 .55995 Mdot
.82831 .55372 Mdot
.83422 .54711 Mdot
.84014 .54014 Mdot
.84605 .53281 Mdot
.85197 .52513 Mdot
.85788 .51712 Mdot
.8638 .50879 Mdot
.86971 .50015 Mdot
.87563 .49122 Mdot
.88154 .482 Mdot
.88746 .47252 Mdot
.89337 .46279 Mdot
.89929 .45282 Mdot
.90521 .44263 Mdot
.91112 .43223 Mdot
.91704 .42164 Mdot
.92295 .41088 Mdot
.92887 .39996 Mdot
.93478 .3889 Mdot
.9407 .37772 Mdot
.94661 .36643 Mdot
.95253 .35506 Mdot
.95844 .34361 Mdot
.96436 .33211 Mdot
.97028 .32057 Mdot
.97619 .30902 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{215, 132.812},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004<0000YQ000`40O003h00Oogoo3Goo003oOol=
Ool00?moo`eoo`00ogoo3Goo003oOol=Ool00?moo`eoo`00ogoo3Goo003oOol=Ool00?moo`eoo`00
8Goo00<007ooOol0j7oo000HOol50004Ool00`00Oomoo`3XOol001Yoo`03001oogoo00Aoo`03001o
ogoo0>Qoo`003goo1P001Goo00<007ooOol017oo0P00jGoo000JOol00`00Oomoo`04Ool00`00Oomo
o`0bOol00`00Ool0000200001Goo000007oo0000[7oo000JOol00`00Oomoo`04Ool00`00Oomoo`0_
Ool00`00Ool0000:Ool00`00Ool0002YOol001Yoo`03001oogoo00Aoo`03001oogoo02aoo`03001o
o`00011oo`800:Moo`006Woo00<007ooOol017oo00<007ooOol0:goo00<007ooOol057oo00<007oo
Ool0Xgoo000HOol30006Ool00`00Oomoo`0ZOol00`00Oomoo`0FOol00`00Oomoo`2ROol0025oo`03
001oogoo02Qoo`03001oogoo01Yoo`03001oogoo0:1oo`008Goo00<007ooOol09goo00<007ooOol0
77oo00<007ooOol0Wgoo000QOol00`00Oomoo`0UOol00`00Oomoo`0OOol00`00Oomoo`2NOol0025o
o`03001oogoo0>Qoo`008Goo00<007ooOol097oo00<007ooOol08Woo00<007ooOol0W7oo000QOol0
0`00Oomoo`0ROol00`00Oomoo`0UOol00`00Oomoo`2KOol0025oo`03001oogoo0>Qoo`008Goo00<0
07ooOol08Goo00<007ooOol0:7oo00<007ooOol0VGoo000QOol00`00Oomoo`0POol00`00Oomoo`0Z
Ool00`00Oomoo`2HOol0025oo`03001oogoo0>Qoo`008Goo00<007ooOol07Woo00<007ooOol0;Woo
00<007ooOol0UWoo000QOol00`00Oomoo`3XOol0025oo`03001oogoo01eoo`03001oogoo031oo`03
001oogoo09Eoo`008Goo00<007ooOol06goo00<007ooOol0<goo00<007ooOol0U7oo000QOol00`00
Oomoo`3XOol0025oo`03001oogoo01Yoo`03001oogoo03Ioo`03001oogoo099oo`008Goo00<007oo
Ool0j7oo000QOol00`00Oomoo`0IOol00`00Oomoo`0hOol00`00Oomoo`2AOol0025oo`03001oogoo
0>Qoo`008Goo00<007ooOol05goo00<007ooOol0?7oo00<007ooOol0Sgoo000QOol00`00Oomoo`3X
Ool0025oo`03001oogoo0>Qoo`008Goo00<007ooOol05Woo00<007ooOol0?Woo00<007ooOol0SWoo
000QOol00`00Oomoo`3XOol0025oo`03001oogoo01Aoo`03001oogoo049oo`03001oogoo08aoo`00
8Goo00<007ooOol0j7oo000QOol00`00Oomoo`0COol00`00Oomoo`14Ool00`00Oomoo`2;Ool000Qo
o`<000Ioo`8000Aoo`@000Ioo`03001oogoo0>Qoo`001goo00D007ooOomoo`0000Eoo`8000Qoo`03
001oogoo00=oo`03001oogoo015oo`03001oogoo04Moo`03001oogoo08Yoo`001@000Woo00D007oo
Oomoo`0000moo`03001oogoo00=oo`03001oogoo0>Qoo`001goo00D007ooOomoo`0000moo`03001o
ogoo00=oo`800>Uoo`001goo00D007ooOomoo`0000]oo`@000Ioo`03001oogoo011oo`03001oogoo
04Yoo`03001oogoo08Qoo`001goo00D007ooOomoo`0000]oo`03001oogoo00Moo`03001oogoo0>Qo
o`001goo00D007ooOomoo`0000]oo`03001oogoo00Moo`03001oogoo00moo`03001oogoo04aoo`03
001oogoo08Moo`0027oo0`0037oo10001Woo00<007ooOol0j7oo000QOol00`00Oomoo`3XOol0025o
o`03001oogoo00eoo`03001oogoo051oo`03001oogoo08Eoo`008Goo00<007ooOol0j7oo000QOol0
0`00Oomoo`0<Ool00`00Oomoo`1BOol00`00Oomoo`24Ool0025oo`03001oogoo0>Qoo`008Goo00<0
07ooOol0j7oo000QOol00`00Oomoo`0:Ool00`00Oomoo`1EOol00`00Oomoo`23Ool0025oo`03001o
ogoo0>Qoo`008Goo00<007ooOol0j7oo000QOol00`00Oomoo`09Ool00`00Oomoo`1HOol00`00Oomo
o`21Ool0025oo`03001oogoo0>Qoo`008Goo00<007ooOol01goo00<007ooOol0Fgoo00<007ooOol0
P7oo000QOol00`00Oomoo`3XOol0025oo`03001oogoo0>Qoo`008Goo00<007ooOol01Woo00<007oo
Ool0GWoo00<007ooOol0OWoo000QOol00`00Oomoo`3XOol0025oo`03001oogoo0>Qoo`008Goo00<0
07ooOol01Goo00<007ooOol0H7oo00<007ooOol0OGoo000QOol00`00Oomoo`0IOol50004Ool4000H
Ool40004Ool3000IOol00`00Oomoo`04Ool4000DOol50003Ool30005Ool3000AOol50002Ool50004
Ool4000?Ool50003Ool40004Ool3000=Ool0025oo`03001oogoo00=oo`03001oogoo01Aoo`04001o
ogoo0008Ool00`00Oomoo`0IOol01000Oomoo`000goo00<007ooOol05Woo00<007ooOol01Goo00@0
07ooOol001Eoo`03001oogoo009oo`05001oogooOol00003Ool01@00Oomoogoo00004Woo00<007oo
Ool00goo00@007ooOol000Qoo`03001oogoo00ioo`03001oogoo00Moo`04001oogoo0003Ool00`00
Oomoo`0:Ool0025oo`03001oogoo01]oo`03001oogoo00Qoo`03001oogoo01Uoo`04001oogoo0003
Ool00`00Oomoo`0GOol00`00Oomoo`07Ool00`00Oomoo`0COol00`00Oomoo`02Ool01@00Oomoogoo
00000goo00D007ooOomoo`00019oo`03001oogoo00Aoo`03001oogoo00Qoo`03001oogoo00ioo`03
001oogoo00Moo`04001oogoo0003Ool00`00Oomoo`0:Ool0025oo`03001oogoo01aoo`03001oogoo
00Moo`03001oogoo01Uoo`04001oogoo0003Ool00`00Oomoo`0GOol00`00Oomoo`07Ool00`00Oomo
o`0COol00`00Oomoo`02Ool01@00Oomoogoo00000goo00D007ooOomoo`00019oo`03001oogoo00Eo
o`03001oogoo00Moo`03001oogoo00ioo`03001oogoo00Moo`04001oogoo0003Ool00`00Oomoo`0:
Ool0025oo`03001oogoo009oo`03001oogoo01Qoo`03001oogoo009oo`@001Qoo`@000=oo`05001o
ogooOol0000IOol00`00Oomoo`03Ool4000FOol00`00Oomoo`02Ool01@00Oomoogoo00000goo00D0
07ooOomoo`00019oo`03001oogoo00Ioo`03001oogoo009oo`@0015oo`03001oogoo00=oo`@000=o
o`05001oogooOol0000<Ool0025oo`03001oogoo01eoo`03001oogoo009oo`03001oogoo01Uoo`03
001oogoo00Aoo`05001oogooOol0000JOol00`00Oomoo`02Ool00`00Oomoo`0GOol00`00Oomoo`02
Ool01@00Oomoogoo00000goo00D007ooOomoo`00019oo`03001oogoo00Ioo`03001oogoo009oo`03
001oogoo019oo`03001oogoo00=oo`03001oogoo00Aoo`05001oogooOol0000<Ool0025oo`03001o
ogoo01Uoo`05001oogooOol00004Ool00`00Oomoo`0IOol00`00Oomoo`04Ool01@00Oomoogoo0000
5Woo00D007ooOomoo`0000Aoo`03001oogoo01Moo`03001oogoo009oo`05001oogooOol00003Ool0
1@00Oomoogoo00004Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo019oo`03001o
ogoo00=oo`03001oogoo00Aoo`05001oogooOol0000<Ool0025oo`04001oogoo000IOol30005Ool4
000HOol40004Ool3000GOol50004Ool4000DOol30005Ool30005Ool3000AOol30005Ool30005Ool4
000?Ool30005Ool40004Ool3000=Ool0025oo`03001oogoo0>Qoo`008Goo00<007ooOol0j7oo000Q
Ool00`00Ool0001^Ool00`00Oomoo`1gOol0025oo`03001oogoo0>Qoo`006gool@00000QOol00`00
Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00
Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`04Ool00`00
Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00
Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00
Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00
Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00
Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`04Ool10000025oo`03001oogoo021oo`03001oogoo
025oo`03001oogoo021oo`03001oogoo025oo`03001oogoo021oo`03001oogoo025oo`03001oogoo
01=oo`008Goo00<007ooOol0LGoo00<007ooOol0K7oo00<007ooOol01Goo000QOol00`00Oomoo`3X
Ool0025oo`03001oogoo0>Qoo`008Goo00<007ooOol0LWoo00<007ooOol0JGoo00<007ooOol01goo
000QOol00`00Oomoo`3XOol0025oo`03001oogoo0>Qoo`008Goo00<007ooOol0M7oo00<007ooOol0
IWoo00<007ooOol027oo000QOol00`00Oomoo`3XOol0025oo`03001oogoo0>Qoo`008Goo00<007oo
Ool0MGoo00<007ooOol0Hgoo00<007ooOol02Woo000QOol00`00Oomoo`3XOol0025oo`03001oogoo
07Ioo`03001oogoo065oo`03001oogoo00]oo`008Goo00<007ooOol0j7oo000QOol00`00Oomoo`3X
Ool0025oo`03001oogoo07Qoo`03001oogoo05ioo`03001oogoo00aoo`008Goo00<007ooOol0j7oo
000QOol00`00Oomoo`3XOol0025oo`03001oogoo07Uoo`03001oogoo05]oo`03001oogoo00ioo`00
8Goo00<007ooOol0j7oo000QOol00`00Oomoo`1kOol00`00Oomoo`1HOol00`00Oomoo`0?Ool0025o
o`03001oogoo0>Qoo`008Goo00<007ooOol0j7oo000QOol00`00Oomoo`1lOol00`00Oomoo`1EOol0
0`00Oomoo`0AOol0025oo`03001oogoo0>Qoo`008Goo00<007ooOol0j7oo000QOol00`00Oomoo`1n
Ool00`00Oomoo`1BOol00`00Oomoo`0BOol0025oo`03001oogoo0>Qoo`008Goo00<007ooOol0Ogoo
00<007ooOol0Cgoo00<007ooOol057oo000QOol00`00Oomoo`3XOol0025oo`03001oogoo0>Qoo`00
27oo0`001Woo0P001Goo10001Goo00<007ooOol0P7oo00<007ooOol0CGoo00<007ooOol05Goo0007
Ool01@00Oomoogoo00001Goo0P002Goo00<007ooOol00Woo00<007ooOol0j7oo0007Ool01@00Oomo
ogoo000047oo00<007ooOol00Woo0P00Pgoo00<007ooOol0BWoo00<007ooOol05Woo0007Ool01@00
Oomoogoo000047oo00<007ooOol00Woo00<007ooOol0j7oo0007Ool01@00Oomoogoo000037oo1000
1Goo00<007ooOol0Pgoo00<007ooOol0Agoo00<007ooOol067oo0007Ool01@00Oomoogoo000037oo
00<007ooOol01Woo00<007ooOol0j7oo0007Ool01@00Oomoogoo000037oo00<007ooOol01Woo00<0
07ooOol0j7oo0008Ool3000=Ool40005Ool00`00Oomoo`25Ool00`00Oomoo`14Ool00`00Oomoo`0I
Ool0025oo`03001oogoo0>Qoo`008Goo00<007ooOol0QWoo00<007ooOol0@Goo00<007ooOol06goo
000QOol00`00Oomoo`3XOol0025oo`03001oogoo08Moo`03001oogoo03moo`03001oogoo01aoo`00
8Goo00<007ooOol0j7oo000QOol00`00Oomoo`29Ool00`00Oomoo`0kOol00`00Oomoo`0NOol0025o
o`03001oogoo0>Qoo`008Goo00<007ooOol0RWoo00<007ooOol0>Goo00<007ooOol07goo000QOol0
0`00Oomoo`3XOol0025oo`03001oogoo08aoo`03001oogoo03Ioo`03001oogoo021oo`008Goo00<0
07ooOol0j7oo000QOol00`00Oomoo`2=Ool00`00Oomoo`0cOol00`00Oomoo`0ROol0025oo`03001o
ogoo0>Qoo`008Goo00<007ooOol0Sgoo00<007ooOol0<7oo00<007ooOol08goo000QOol00`00Oomo
o`3XOol0025oo`03001oogoo091oo`03001oogoo02eoo`03001oogoo02Eoo`008Goo00<007ooOol0
j7oo000QOol00`00Oomoo`2AOol00`00Oomoo`0[Ool00`00Oomoo`0VOol0025oo`03001oogoo09=o
o`03001oogoo02Qoo`03001oogoo02Moo`008Goo00<007ooOol0j7oo000QOol00`00Oomoo`2DOol0
0`00Oomoo`0UOol00`00Oomoo`0YOol0025oo`03001oogoo09Ioo`03001oogoo029oo`03001oogoo
02Yoo`008Goo00<007ooOol0Ugoo00<007ooOol07goo00<007ooOol0;7oo000QOol00`00Oomoo`3X
Ool0025oo`03001oogoo09Uoo`03001oogoo01aoo`03001oogoo02eoo`008Goo00<007ooOol0VWoo
00<007ooOol06Goo00<007ooOol0;goo000QOol00`00Oomoo`2KOol00`00Oomoo`0GOol00`00Oomo
o`0`Ool001Qoo`D000Aoo`03001oogoo09eoo`03001oogoo01Aoo`03001oogoo035oo`006Woo00<0
07ooOol017oo00<007ooOol0WWoo00<007oo000047oo0P00=Goo000JOol00`00Oomoo`04Ool00`00
Oomoo`2QOol2000;Ool2000hOol001Yoo`03001oogoo00Aoo`800:Eoo`800004Ool00000Ool3000k
Ool001Yoo`03001oogoo00Aoo`03001oogoo0>Qoo`006Woo00<007ooOol017oo00<007ooOol0j7oo
000JOol00`00Oomoo`04Ool00`00Oomoo`3XOol001Qoo`<00?5oo`00ogoo3Goo003oOol=Ool00?mo
o`eoo`00ogoo3Goo003oOol=Ool00?moo`eoo`00ogoo3Goo0000\
\>"],
  ImageRangeCache->{{{0, 267.75}, {165, 0}} -> {-23.2041, -1.16574, 0.878758, 
  0.0176628}}]
}, Open  ]],

Cell[TextData[{
  "Here we define a function to solve the problem on the given grid, the \
output is a list of values of ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  " for the last three time points (for now we shall just use the final \
value). Note that this function makes use of the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3.0 vector compilation methods. The initial conditions, and upper and \
lower boundary conditions, are supplied as vectors, ",
  StyleBox["initial",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["lower",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["upper",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", as rank-1 objects that are real - note the syntax ",
  StyleBox["{initial, _Real, 1}",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to denote this. For further details see section 2.5.14 of the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " book."
}], "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(ExplicitSolver\  = \n\t
      Compile[{{initial, _Real, 1}, \ {lower, _Real, 1}, \ {upper, _Real, 1}, 
          alpha}, \n\t\tModule[{wold = initial, \ wnew = initial, \ wvold, m, 
            k, \ tsize\  = \ Length[lower], 
            xsize = Length[initial]}, \n\t\t\tFor[m = 2, \ 
            m\  <= \ tsize, \ \(m++\), \n\t\t\t\t\((wvold\  = \ wold; \ 
              wold = wnew; \n\t\t\t\t\tFor[k = 2, \ 
                k\  < \ xsize, \ \(k++\), \n\t\t\t\t\t\((wnew[\([k]\)]\  = \ 
                    alpha\ \((wold[\([k - 1]\)] + 
                            wold[\([k + 1]\)])\) + \((1 - 
                            2  alpha)\)\ wold[\([k]\)])\)]; \n\t\t\t\t\t\t\
wnew[\([1]\)]\  = \ lower[\([m]\)]; \n\t\t\t\t\t\twnew[\([xsize]\)]\  = \ 
                upper[\([m]\)])\)]; \n\t\t\t{wvold, wold, 
            wnew}]\n\t\t]; \)\)], "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True],

Cell["\<\
Now we apply this to get the solution at a time \[Tau] = 0.1 (the \
units are irrelevant for our analysis):\
\>", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(soln\  = \ ExplicitSolver[initial, lower, upper, \ alpha]; \)\)], 
  "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["Now we do the interpolation to supply a continuous function:", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell["\<\
interpoldata = 
Table[{(k-nminus-1)*dx ,soln[[3,k]]}, {k, 1, nminus+nplus+1}];\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["ufunc = Interpolation[interpoldata, InterpolationOrder -> 3]", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[\(InterpolatingFunction[{{\(-2.`\), 2.`}}, "<>"]\),
      False,
      Editable->False]], "Output",
  PageBreakAbove->False]
}, Open  ]],

Cell["Now we plot the error in the answer:", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot[ufunc[x] - Sin[Pi*x/2]*Exp[-Pi^2 0.1/4], {x, -2, 2}, 
PlotPoints -> 50];\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.238095 0.309017 8473.29 [
[.02381 .29652 -6 -9 ]
[.02381 .29652 6 0 ]
[.2619 .29652 -6 -9 ]
[.2619 .29652 6 0 ]
[.7381 .29652 -3 -9 ]
[.7381 .29652 3 0 ]
[.97619 .29652 -3 -9 ]
[.97619 .29652 3 0 ]
[.4875 .05482 -48 -4.5 ]
[.4875 .05482 0 4.5 ]
[.4875 .13955 -48 -4.5 ]
[.4875 .13955 0 4.5 ]
[.4875 .22428 -48 -4.5 ]
[.4875 .22428 0 4.5 ]
[.4875 .39375 -42 -4.5 ]
[.4875 .39375 0 4.5 ]
[.4875 .47848 -42 -4.5 ]
[.4875 .47848 0 4.5 ]
[.4875 .56322 -42 -4.5 ]
[.4875 .56322 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .30902 m
.02381 .31527 L
s
[(-2)] .02381 .29652 0 1 Mshowa
.2619 .30902 m
.2619 .31527 L
s
[(-1)] .2619 .29652 0 1 Mshowa
.7381 .30902 m
.7381 .31527 L
s
[(1)] .7381 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(2)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.07143 .30902 m
.07143 .31277 L
s
.11905 .30902 m
.11905 .31277 L
s
.16667 .30902 m
.16667 .31277 L
s
.21429 .30902 m
.21429 .31277 L
s
.30952 .30902 m
.30952 .31277 L
s
.35714 .30902 m
.35714 .31277 L
s
.40476 .30902 m
.40476 .31277 L
s
.45238 .30902 m
.45238 .31277 L
s
.54762 .30902 m
.54762 .31277 L
s
.59524 .30902 m
.59524 .31277 L
s
.64286 .30902 m
.64286 .31277 L
s
.69048 .30902 m
.69048 .31277 L
s
.78571 .30902 m
.78571 .31277 L
s
.83333 .30902 m
.83333 .31277 L
s
.88095 .30902 m
.88095 .31277 L
s
.92857 .30902 m
.92857 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.5 .05482 m
.50625 .05482 L
s
[(-0.00003)] .4875 .05482 1 0 Mshowa
.5 .13955 m
.50625 .13955 L
s
[(-0.00002)] .4875 .13955 1 0 Mshowa
.5 .22428 m
.50625 .22428 L
s
[(-0.00001)] .4875 .22428 1 0 Mshowa
.5 .39375 m
.50625 .39375 L
s
[(0.00001)] .4875 .39375 1 0 Mshowa
.5 .47848 m
.50625 .47848 L
s
[(0.00002)] .4875 .47848 1 0 Mshowa
.5 .56322 m
.50625 .56322 L
s
[(0.00003)] .4875 .56322 1 0 Mshowa
.125 Mabswid
.5 .07176 m
.50375 .07176 L
s
.5 .08871 m
.50375 .08871 L
s
.5 .10566 m
.50375 .10566 L
s
.5 .1226 m
.50375 .1226 L
s
.5 .1565 m
.50375 .1565 L
s
.5 .17344 m
.50375 .17344 L
s
.5 .19039 m
.50375 .19039 L
s
.5 .20734 m
.50375 .20734 L
s
.5 .24123 m
.50375 .24123 L
s
.5 .25818 m
.50375 .25818 L
s
.5 .27512 m
.50375 .27512 L
s
.5 .29207 m
.50375 .29207 L
s
.5 .32596 m
.50375 .32596 L
s
.5 .34291 m
.50375 .34291 L
s
.5 .35986 m
.50375 .35986 L
s
.5 .3768 m
.50375 .3768 L
s
.5 .4107 m
.50375 .4107 L
s
.5 .42764 m
.50375 .42764 L
s
.5 .44459 m
.50375 .44459 L
s
.5 .46154 m
.50375 .46154 L
s
.5 .49543 m
.50375 .49543 L
s
.5 .51238 m
.50375 .51238 L
s
.5 .52932 m
.50375 .52932 L
s
.5 .54627 m
.50375 .54627 L
s
.5 .03787 m
.50375 .03787 L
s
.5 .02092 m
.50375 .02092 L
s
.5 .00398 m
.50375 .00398 L
s
.5 .58016 m
.50375 .58016 L
s
.5 .59711 m
.50375 .59711 L
s
.5 .61406 m
.50375 .61406 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30902 m
.04273 .34565 L
.06337 .38496 L
.08275 .42052 L
.10139 .45303 L
.12123 .48543 L
.14033 .51365 L
.16063 .5398 L
.18019 .56155 L
.199 .57838 L
.20853 .585 L
.21902 .59153 L
.22381 .59411 L
.22897 .59645 L
.23829 .59948 L
.24324 .60094 L
.24586 .60166 L
.24661 .60186 L
.24698 .60194 L
.24732 .602 L
.24796 .60208 L
.24829 .60211 L
.24865 .60213 L
.24925 .60213 L
.24959 .60212 L
.2499 .6021 L
.25051 .60228 L
.25108 .60246 L
.2517 .60263 L
.25236 .60277 L
.2527 .60282 L
.25308 .60287 L
.25343 .6029 L
.25375 .60292 L
.25433 .60292 L
.25494 .60289 L
.25546 .60284 L
.25603 .6028 L
.25664 .60296 L
.2573 .60311 L
.25792 .60322 L
.25849 .60328 L
.25907 .60332 L
.25962 .60331 L
.26022 .60328 L
.26086 .6032 L
.26118 .60314 L
.26153 .60308 L
.26217 .60306 L
.2628 .60317 L
Mistroke
.26338 .60325 L
.26397 .6033 L
.26452 .60332 L
.26502 .60331 L
.26556 .60326 L
.26615 .60318 L
.26677 .60306 L
.26734 .60292 L
.26788 .60278 L
.26852 .60286 L
.2691 .6029 L
.26972 .60292 L
.27007 .60291 L
.2704 .6029 L
.27094 .60285 L
.27154 .60275 L
.27216 .60262 L
.27274 .60246 L
.27339 .60224 L
.27372 .60213 L
.27408 .60211 L
.27472 .60213 L
.27531 .60212 L
.27567 .6021 L
.27601 .60207 L
.27664 .60198 L
.277 .60191 L
.27733 .60183 L
.27807 .60162 L
.28289 .60055 L
.28806 .59898 L
.29739 .59503 L
.31598 .58459 L
.33622 .56869 L
.35572 .54868 L
.37448 .52566 L
.39444 .49778 L
.41365 .46777 L
.43213 .43645 L
.4518 .40097 L
.47074 .36544 L
.49087 .32673 L
.51026 .28911 L
.52891 .25325 L
.54876 .21599 L
.56787 .18158 L
.58818 .14736 L
.60775 .11712 L
.62657 .09087 L
.6466 .06671 L
Mistroke
.66588 .04763 L
.68442 .03325 L
.69474 .0267 L
.70417 .02207 L
.71391 .01869 L
.71877 .01716 L
.71979 .01708 L
.72031 .01704 L
.72088 .0168 L
.7215 .01656 L
.72207 .01637 L
.72263 .01621 L
.72317 .01609 L
.72378 .01599 L
.72434 .01593 L
.72493 .01591 L
.72527 .0159 L
.72559 .01591 L
.72611 .01594 L
.72669 .01576 L
.72787 .01541 L
.72852 .01527 L
.72911 .01518 L
.72942 .01515 L
.72976 .01513 L
.73012 .01511 L
.73045 .01511 L
.73106 .01514 L
.73162 .01519 L
.73227 .01522 L
.73287 .01506 L
.73344 .01493 L
.73397 .01483 L
.73452 .01476 L
.73512 .01472 L
.73565 .01472 L
.73612 .01474 L
.73668 .01479 L
.7372 .01486 L
.73776 .01496 L
.73836 .01498 L
.73893 .01487 L
.73944 .0148 L
.74005 .01474 L
.74039 .01472 L
.7407 .01472 L
.74128 .01473 L
.74188 .01479 L
.74303 .01499 L
.74364 .01514 L
Mistroke
.74428 .01523 L
.74484 .01517 L
.74546 .01512 L
.74602 .01511 L
.74654 .01513 L
.74688 .01516 L
.7472 .0152 L
.7478 .0153 L
.74845 .01544 L
.74915 .01565 L
.74975 .01585 L
.75038 .01592 L
.75098 .0159 L
.75153 .01591 L
.75209 .01596 L
.75269 .01603 L
.75319 .01613 L
.75375 .01626 L
.75491 .01664 L
.75555 .0169 L
.75616 .01707 L
.75672 .0171 L
.75734 .01715 L
.75789 .01722 L
.75841 .01732 L
.7589 .01743 L
.75943 .01759 L
.76052 .01799 L
.76552 .01949 L
.77078 .02148 L
.7802 .02622 L
.79914 .03831 L
.8178 .05445 L
.83766 .07599 L
.85678 .10059 L
.8771 .13003 L
.89668 .16161 L
.91551 .19436 L
.93555 .23108 L
.95484 .2677 L
.97339 .30363 L
.97619 .30902 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{215, 132.812},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004<0000YQ000`40O003h00Oogoo3Goo0026Ool0
0`00Oomoo`23Ool008Ioo`8008Aoo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Io
o`03001oogoo03Moo`d003moo`00QWoo0P00=7oo10003Goo0`00?7oo0026Ool00`00Oomoo`0bOol0
0`00Oomoo`0BOol00`00Oomoo`0iOol008Ioo`03001oogoo031oo`8001Ioo`8003Uoo`00QWoo00<0
07ooOol0;Goo0`006Woo0P00=goo0026Ool00`00Oomoo`0/Ool00`00Oomoo`0MOol00`00Oomoo`0d
Ool008Ioo`8002aoo`03001oogoo01moo`8003Aoo`00QWoo00<007ooOol0:Woo00<007ooOol08Woo
00<007ooOol0<Goo001>Ool30006Ool20004Ool30005Ool30005Ool30005Ool30005Ool30006Ool0
0`00Oomoo`0YOol00`00Oomoo`0TOol00`00Oomoo`0`Ool004eoo`05001oogooOol00005Ool20003
Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo
00000goo00D007ooOomoo`0000Eoo`03001oogoo02Qoo`03001oogoo02Ioo`03001oogoo02moo`00
AGoo1P000Woo00D007ooOomoo`0000Yoo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D0
07ooOomoo`0000=oo`05001oogooOol00007Ool00`00Oomoo`03Ool3000WOol00`00Oomoo`0XOol0
0`00Oomoo`0^Ool004eoo`05001oogooOol0000:Ool01@00Oomoogoo00000goo00D007ooOomoo`00
00=oo`05001oogooOol00003Ool01@00Oomoogoo00001goo00<007ooOol00goo00<007ooOol09Woo
00<007ooOol0:Goo00<007ooOol0;Woo001=Ool01@00Oomoogoo00002Woo00D007ooOomoo`0000=o
o`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Eoo`8000Ioo`03001o
ogoo02Eoo`03001oogoo02]oo`03001oogoo02eoo`00CGoo00D007ooOomoo`0000Yoo`05001oogoo
Ool00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00007Ool00`00
Oomoo`03Ool00`00Oomoo`0TOol00`00Oomoo`0]Ool00`00Oomoo`0/Ool004eoo`05001oogooOol0
000:Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomo
ogoo00000goo00D007ooOomoo`0000Eoo`03001oogoo02Aoo`03001oogoo02ioo`03001oogoo02]o
o`00CWoo0`0037oo0`001Goo0`001Goo0`001Goo0`001Goo0`001Woo0P0097oo00<007ooOol0<7oo
00<007ooOol0:Woo0026Ool00`00Oomoo`0ROol00`00Oomoo`0bOol00`00Oomoo`0YOol008Ioo`03
001oogoo025oo`03001oogoo03=oo`03001oogoo02Uoo`00QWoo00<007ooOol08Goo00<007ooOol0
=7oo00<007ooOol0:7oo0026Ool2000QOol00`00Oomoo`0fOol00`00Oomoo`0WOol008Ioo`03001o
ogoo01moo`03001oogoo03Qoo`03001oogoo02Ioo`00QWoo00<007ooOol07goo00<007ooOol0>Goo
00<007ooOol09Goo0026Ool00`00Oomoo`0NOol00`00Oomoo`0kOol00`00Oomoo`0TOol008Ioo`03
001oogoo01eoo`03001oogoo03aoo`03001oogoo02Aoo`00QWoo0P007Goo00<007ooOol0?Woo00<0
07ooOol08goo0026Ool00`00Oomoo`0LOol00`00Oomoo`0oOol00`00Oomoo`0ROol008Ioo`03001o
ogoo01]oo`03001oogoo041oo`03001oogoo029oo`00QWoo00<007ooOol06Woo00<007ooOol0@Woo
00<007ooOol08Goo0026Ool2000KOol00`00Oomoo`12Ool00`00Oomoo`0QOol008Ioo`03001oogoo
01Uoo`03001oogoo04Aoo`03001oogoo021oo`00CWoo0`001Woo0P0017oo0`001Goo0`001Goo0`00
1Goo0`0017oo1@001Goo00<007ooOol067oo00<007ooOol0AWoo00<007ooOol07goo001=Ool01@00
Oomoogoo00001Goo0P000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo
00000goo00D007ooOomoo`0000Aoo`04001oogoo0005Ool00`00Oomoo`0GOol00`00Oomoo`17Ool0
0`00Oomoo`0OOol004Eoo`H0009oo`05001oogooOol0000:Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00001Goo00<007ooOol01Goo00<007oo
Ool05goo00<007ooOol0B7oo00<007ooOol07Woo001=Ool01@00Oomoogoo00002Woo00D007ooOomo
o`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Ioo`03001o
ogoo00Aoo`<001Ioo`03001oogoo04Yoo`03001oogoo01eoo`00CGoo00D007ooOomoo`0000Yoo`05
001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00007
Ool00`00Oomoo`03Ool00`00Oomoo`0EOol00`00Oomoo`1;Ool00`00Oomoo`0MOol004eoo`05001o
ogooOol0000:Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool0
1@00Oomoogoo00001goo00<007ooOol00goo00<007ooOol057oo00<007ooOol0CGoo00<007ooOol0
77oo001=Ool01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00
Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00005Ool00`00Oomoo`0DOol00`00
Oomoo`1>Ool00`00Oomoo`0KOol004ioo`<000aoo`<000Eoo`<000Eoo`<000Eoo`<000Eoo`<000Io
o`8001Aoo`03001oogoo04moo`03001oogoo01]oo`00QWoo00<007ooOol04goo00<007ooOol0D7oo
00<007ooOol06Woo0026Ool00`00Oomoo`0BOol00`00Oomoo`1BOol00`00Oomoo`0IOol008Ioo`03
001oogoo015oo`03001oogoo05=oo`03001oogoo01Uoo`00QWoo00<007ooOol04Goo00<007ooOol0
E7oo00<007ooOol067oo0026Ool2000AOol00`00Oomoo`1FOol00`00Oomoo`0GOol008Ioo`03001o
ogoo011oo`03001oogoo05Ioo`03001oogoo01Moo`00QWoo00<007ooOol03goo00<007ooOol0F7oo
00<007ooOol05Woo0026Ool00`00Oomoo`0?Ool00`00Oomoo`1HOol00`00Oomoo`0FOol008Ioo`80
00moo`03001oogoo05Yoo`03001oogoo01Eoo`00QWoo00<007ooOol03Woo00<007ooOol0Fgoo00<0
07ooOol057oo0026Ool00`00Oomoo`0=Ool00`00Oomoo`1LOol00`00Oomoo`0DOol008Ioo`03001o
ogoo00aoo`03001oogoo05ioo`03001oogoo01=oo`00QWoo00<007ooOol037oo00<007ooOol0GWoo
00<007ooOol04goo0026Ool2000<Ool00`00Oomoo`1POol00`00Oomoo`0BOol008Ioo`03001oogoo
00]oo`03001oogoo061oo`03001oogoo019oo`00CWoo0`001Woo0P0017oo0`001Goo0`001Goo0`00
1Goo0`001Goo1@0017oo00<007ooOol02Woo00<007ooOol0HWoo00<007ooOol04Goo001=Ool01@00
Oomoogoo00001Goo0P000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo
00000goo00D007ooOomoo`0000Ioo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo069oo`03
001oogoo015oo`00AGoo1P000Woo00D007ooOomoo`0000Yoo`05001oogooOol00003Ool01@00Oomo
ogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00006Ool00`00Oomoo`04Ool30009Ool0
0`00Oomoo`1TOol00`00Oomoo`0@Ool004eoo`05001oogooOol0000:Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00001Woo00<007ooOol017oo
00<007ooOol02Goo00<007ooOol0I7oo00<007ooOol047oo001=Ool01@00Oomoogoo00002Woo00D0
07ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Io
o`03001oogoo00Aoo`03001oogoo00Qoo`03001oogoo06Ioo`03001oogoo00moo`00CGoo00D007oo
Oomoo`0000Yoo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05
001oogooOol00006Ool00`00Oomoo`04Ool00`00Oomoo`08Ool00`00Oomoo`1VOol00`00Oomoo`0?
Ool004eoo`05001oogooOol0000:Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001o
ogooOol00003Ool01@00Oomoogoo00001Woo00<007ooOol017oo00<007ooOol01goo00<007ooOol0
J7oo00<007ooOol03Woo001>Ool3000<Ool30005Ool30005Ool30005Ool30005Ool30006Ool20008
Ool00`00Oomoo`1XOol00`00Oomoo`0>Ool008Ioo`03001oogoo00Ioo`03001oogoo06Yoo`03001o
ogoo00eoo`00QWoo00<007ooOol01Woo00<007ooOol0JWoo00<007ooOol03Goo0026Ool00`00Oomo
o`05Ool00`00Oomoo`1/Ool00`00Oomoo`0<Ool008Ioo`8000Ioo`03001oogoo06aoo`03001oogoo
00aoo`00QWoo00<007ooOol017oo00<007ooOol0KWoo00<007ooOol02goo0008Ool5000lOol5000h
Ool00`00Oomoo`04Ool00`00Oomoo`0dOol5000eOol01000Oomoogoo1@001Goo0009Ool01000Oomo
o`00?Woo00<007ooOol0>7oo00<007ooOol00goo00<007ooOol0=goo00<007ooOol0=Woo00D007oo
Oomoo`00009oo`03001oogoo00=oo`001P0017oo00<007ooOol0<goo1P001Goo00<007ooOol0>7oo
00<007ooOol00goo00<007ooOol0=goo00<007ooOol0=Woo00<007ooOol00Woo00<007ooOol01Goo
000;Ool00`00Oomoo`0mOol00`00Oomoo`0hOol20003Ool00`00Oomoo`0hOol00`00Oomoo`0gOol0
0`00Oomoo`02Ool00`00Oomoo`04Ool000aoo`03001oogoo03aoo`03001oogoo03Qoo`03001oogoo
009oo`03001oogoo03Qoo`03001oogoo03Moo`03001oogoo00=oo`03001oogoo00=oo`0037oo00<0
07ooOol0?7oo00<007ooOol0>7oo00D007ooOomoo`0003]oo`03001oogoo03Qoo`03001oogoo009o
o`03001oogoo00=oo`0027oo00D007ooOomoo`0003ioo`03001oogoo03Qoo`05001oogooOol0000k
Ool00`00Oomoo`0hOol20003Ool00`00Oomoo`03Ool000Uoo`<003eoo`<003Yoo`800003Ool007oo
03Uoo`<003]oo`@000Ioo`00QWoo00@007ooOol007Qoo`03001oogoo00Moo`00QWoo00<007oo0000
NWoo00<007ooOol01Woo0026Ool00`00Ool0001jOol00`00Oomoo`06Ool008Ioo`8007aoo`03001o
ogoo00Eoo`0000=oo`000000o`002P000007Ool00`00Oomoo`0:Ool00`00Oomoo`09Ool00`00Oomo
o`0:Ool00`00Oomoo`0:Ool00`00Oomoo`09Ool00`00Oomoo`0:Ool00`00Oomoo`0:Ool00`00Oomo
o`0:Ool00`00Oomoo`09Ool00`00Oomoo`0:Ool00`00Oomoo`0:Ool00`00Oomoo`09Ool00`00Oomo
o`0:Ool00`00Oomoo`0:Ool00`00Oomoo`09Ool00`00Oomoo`0:Ool00`00Oomoo`0:Ool00`00Oomo
o`0:Ool00`00Oomoo`09Ool00`00Oomoo`0:Ool00`00Oomoo`04Ool000Moo`8003eoo`03001oogoo
03eoo`03001oogoo03aoo`03001oogoo03eoo`03001oogoo00Aoo`0027oo00<007ooOol0NWoo0P00
QGoo0009Ool00`00Oomoo`1iOol20025Ool000Uoo`03001oogoo07Qoo`04001oo`000024Ool000Yo
o`03001oogoo07Moo`03001oo`0008Eoo`002Woo00<007ooOol0MWoo00@007ooOol008Eoo`002goo
00<007ooOol0M7oo00D007ooOomoo`0008Eoo`002goo00<007ooOol0M7oo00@007ooOomoo`8008Ao
o`0037oo00<007ooOol0LWoo00<007ooOol00Woo00<007ooOol0Pgoo000<Ool00`00Oomoo`1bOol0
0`00Oomoo`02Ool00`00Oomoo`23Ool000eoo`03001oogoo071oo`03001oogoo00=oo`03001oogoo
08=oo`003Woo00<007ooOol0KWoo00<007ooOol017oo00<007ooOol0Pgoo000>Ool00`00Oomoo`1^
Ool00`00Oomoo`04Ool20024Ool000moo`03001oogoo06aoo`03001oogoo00Eoo`03001oogoo08=o
o`003goo00<007ooOol0K7oo00<007ooOol01Goo00<007ooOol0Pgoo000@Ool00`00Oomoo`1ZOol0
0`00Oomoo`06Ool00`00Oomoo`23Ool0015oo`03001oogoo06Uoo`03001oogoo00Ioo`8008Aoo`00
4Goo00<007ooOol0J7oo00<007ooOol01goo00<007ooOol0Pgoo000BOol00`00Oomoo`0iOol30006
Ool20005Ool30004Ool30005Ool30005Ool30004Ool60004Ool00`00Oomoo`23Ool0019oo`03001o
ogoo03Qoo`05001oogooOol00005Ool20004Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000=o
o`05001oogooOol00003Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000Ioo`03001oogoo08=o
o`004goo00<007ooOol0=goo00D007ooOomoo`0000]oo`05001oogooOol00002Ool01@00Oomoogoo
00000goo00D007ooOomoo`0000=oo`05001oogooOol00002Ool01@00Oomoogoo00001Woo00<007oo
Ool0Pgoo000COol00`00Oomoo`0gOol01@00Oomoogoo00002goo00D007ooOomoo`00009oo`05001o
ogooOol00003Ool01@00Oomoogoo00000goo00L007ooOomoo`00Ool00004Ool00`00Oomoo`04Ool3
0023Ool001Aoo`03001oogoo03Ioo`05001oogooOol0000;Ool01@00Oomoogoo00000Woo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool01`00Oomoogoo001oo`0000Aoo`03001oogoo00Aoo`03
001oogoo08=oo`0057oo00<007ooOol0=Woo00D007ooOomoo`0000]oo`05001oogooOol00002Ool0
1@00Oomoogoo00000goo00D007ooOomoo`0000=oo`04001oogooOol20005Ool00`00Oomoo`04Ool0
0`00Oomoo`23Ool001Eoo`03001oogoo03Eoo`05001oogooOol0000;Ool01@00Oomoogoo00000Woo
00D007ooOomoo`0000=oo`05001oogooOol00003Ool01000Oomoogoo0P001Goo00<007ooOol017oo
00<007ooOol0Pgoo000EOol00`00Oomoo`0fOol3000=Ool30004Ool30005Ool30005Ool40004Ool3
0006Ool20024Ool001Ioo`03001oogoo05moo`03001oogoo00]oo`03001oogoo08=oo`005Woo00<0
07ooOol0GWoo00<007ooOol037oo00<007ooOol0Pgoo000GOol00`00Oomoo`1LOol00`00Oomoo`0=
Ool00`00Oomoo`23Ool001Moo`03001oogoo05aoo`03001oogoo00eoo`03001oogoo08=oo`0067oo
00<007ooOol0FWoo00<007ooOol03Woo0P00Q7oo000IOol00`00Oomoo`1IOol00`00Oomoo`0>Ool0
0`00Oomoo`23Ool001Uoo`03001oogoo05Qoo`03001oogoo00moo`03001oogoo08=oo`006Woo00<0
07ooOol0Egoo00<007ooOol03goo00<007ooOol0Pgoo000JOol00`00Oomoo`1FOol00`00Oomoo`0@
Ool20024Ool001]oo`03001oogoo05Aoo`03001oogoo015oo`03001oogoo08=oo`0077oo00<007oo
Ool0Dgoo00<007ooOol04Goo00<007ooOol0Pgoo000LOol00`00Oomoo`1BOol00`00Oomoo`0BOol0
0`00Oomoo`23Ool001eoo`03001oogoo055oo`03001oogoo019oo`03001oogoo08=oo`007Goo00<0
07ooOol0D7oo00<007ooOol04goo0P00Q7oo000NOol00`00Oomoo`1>Ool00`00Oomoo`0DOol00`00
Oomoo`23Ool001moo`03001oogoo02aoo`<000Ioo`8000Eoo`<000Aoo`<000Eoo`<000Eoo`<000Ao
o`D000Eoo`03001oogoo08=oo`007goo00<007ooOol0:goo00D007ooOomoo`0000Eoo`8000Aoo`05
001oogooOol00002Ool01@00Oomoogoo00000goo00D007oo001oo`0000=oo`05001oogooOol00004
Ool01000Oomoo`001Goo00<007ooOol0Pgoo000POol00`00Oomoo`0ZOol01@00Oomoogoo00002goo
00D007ooOomoo`00009oo`05001oogooOol00003Ool20002Ool01@00Oomoogoo00000goo00<007oo
Ool00goo00<007ooOol01Goo0`00Pgoo000POol00`00Oomoo`0ZOol01@00Oomoogoo00002goo00D0
07ooOomoo`00009oo`05001oogooOol00003Ool20002Ool01@00Oomoogoo00000goo00<007ooOol0
17oo00<007ooOol017oo00<007ooOol0Pgoo000QOol00`00Oomoo`0YOol01@00Oomoogoo00002goo
00D007ooOomoo`00009oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`00
00Moo`03001oogoo00=oo`03001oogoo08=oo`008Goo00<007ooOol0:Goo00D007ooOomoo`0000]o
o`05001oogooOol00002Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol0
0007Ool00`00Oomoo`03Ool00`00Oomoo`23Ool0029oo`03001oogoo02Qoo`05001oogooOol0000;
Ool01@00Oomoogoo00000Woo00D007ooOomoo`00009oo`8000=oo`05001oogooOol00003Ool01@00
Oomoogoo00000goo00<007ooOol00goo00<007ooOol0Pgoo000SOol00`00Oomoo`0XOol3000=Ool3
0004Ool30002Ool00`00Oomoo`030005Ool30005Ool30006Ool20024Ool002=oo`03001oogoo04Ao
o`03001oogoo01Uoo`03001oogoo08=oo`0097oo00<007ooOol0@Woo00<007ooOol06Woo00<007oo
Ool0Pgoo000TOol00`00Oomoo`11Ool00`00Oomoo`0KOol00`00Oomoo`23Ool002Eoo`03001oogoo
03moo`03001oogoo01aoo`8008Aoo`009Woo00<007ooOol0?Woo00<007ooOol077oo00<007ooOol0
Pgoo000VOol00`00Oomoo`0mOol00`00Oomoo`0MOol00`00Oomoo`23Ool002Moo`03001oogoo03]o
o`03001oogoo01ioo`03001oogoo08=oo`00:7oo00<007ooOol0>Goo00<007ooOol07goo00<007oo
Ool0Pgoo000YOol00`00Oomoo`0hOol00`00Oomoo`0OOol20024Ool002Uoo`03001oogoo03Moo`03
001oogoo021oo`03001oogoo08=oo`00:Woo00<007ooOol0=Goo00<007ooOol08Goo00<007ooOol0
Pgoo000[Ool2000dOol00`00Oomoo`0ROol00`00Oomoo`23Ool002eoo`03001oogoo031oo`03001o
ogoo02=oo`8008Aoo`00;Woo00<007ooOol0;Woo00<007ooOol097oo00<007ooOol0Pgoo000_Ool0
0`00Oomoo`0LOol30006Ool20005Ool30004Ool30005Ool30005Ool30005Ool30006Ool00`00Oomo
o`23Ool0031oo`03001oogoo01Yoo`05001oogooOol00005Ool20004Ool01@00Oomoogoo00000Woo
00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`00
00Eoo`03001oogoo08=oo`00<Goo00<007ooOol06Goo00D007ooOomoo`0000Yoo`8000=oo`04001o
ogoo0003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`03001oogoo00Eoo`03001oogoo
00=oo`03001oogoo08=oo`00<Woo00<007ooOol067oo00D007ooOomoo`0000Uoo`03001oo`0000=o
o`04001oogoo0003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`03001oogoo00Eoo`03
001oogoo00=oo`<008=oo`00<goo00<007ooOol05goo00D007ooOomoo`0000Moo`80009oo`05001o
ogooOol00002Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00005Ool2
0006Ool00`00Oomoo`23Ool003Aoo`03001oogoo01Ioo`05001oogooOol00006Ool00`00Oomoo`02
Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo
00001goo00<007ooOol00goo00<007ooOol0Pgoo000eOol00`00Oomoo`0EOol01@00Oomoogoo0000
1Goo00<007ooOol00goo00D007ooOomoo`00009oo`05001oogooOol00003Ool01@00Oomoogoo0000
0goo00D007ooOomoo`0000=oo`05001oogooOol00005Ool00`00Oomoo`23Ool003Ioo`03001oogoo
01Eoo`<000Eoo`03001oogoo00Eoo`<000Aoo`<000Eoo`<000Eoo`<000Eoo`<000Ioo`8008Aoo`00
=goo00<007ooOol06Woo0P00<7oo00<007ooOol0Pgoo000hOol3000GOol2000bOol00`00Oomoo`23
Ool003]oo`03001oogoo01=oo`03001oogoo039oo`03001oogoo08=oo`00?7oo0P004Goo0P00=Goo
00<007ooOol0Pgoo000nOol4000;Ool2000gOol20024Ool0049oo`/003Uoo`03001oogoo08=oo`00
QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`8008Aoo`00QWoo00<007ooOol0Pgoo
0000\
\>"],
  ImageRangeCache->{{{0, 267.75}, {165, 0}} -> {-2.11818, -3.65218*^-5, 
  0.0196928, 5.53358*^-7}}]
}, Open  ]],

Cell["\<\
Finally we tabulate the numerical result, the exact result and the \
error in the numerical scheme. With 400 time-steps the error is manageably \
small.\
\>", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell["\<\
samples = TableForm[Join[{{\"x\", \"Exact\", \"Explicit FD\", \
\"Error\"}},
Table[Map[PaddedForm[N[Chop[#1]],{5,6}]&,
N[{x, 
ufunc[x], 
Sin[Pi*x/2]*Exp[-(Pi^2 0.1)/4], 
ufunc[x]- Sin[Pi*x/2]*Exp[-(Pi^2 0.1)/4]},5]],
{x, -2, 2, 0.25}]]]\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"x\"\>", "\<\"Exact\"\>", "\<\"Explicit FD\"\>", 
            "\<\"Error\"\>"},
          {
            TagBox[
              InterpretationBox["\<\"-2.000000\"\>",
                -2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.750000\"\>",
                -1.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.298990\"\>",
                -0.29899402337872427,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299010\"\>",
                -0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000013\"\>",
                1.3277284115542631*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.500000\"\>",
                -1.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552470\"\>",
                -0.55246891708560886,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552490\"\>",
                -0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000025\"\>",
                2.4533222083511497*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.250000\"\>",
                -1.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721840\"\>",
                -0.72183542630941444,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721870\"\>",
                -0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000032\"\>",
                3.2054199383124704*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.000000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.781310\"\>",
                -0.78130903533204532,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.781340\"\>",
                -0.78134373054744422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000035\"\>",
                3.4695215398894419*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.750000\"\>",
                -0.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721840\"\>",
                -0.72183542630941444,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721870\"\>",
                -0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000032\"\>",
                3.2054199383124704*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.500000\"\>",
                -0.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552470\"\>",
                -0.55246891708560886,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552490\"\>",
                -0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000025\"\>",
                2.4533222083511497*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.250000\"\>",
                -0.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.298990\"\>",
                -0.29899402337872427,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299010\"\>",
                -0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000013\"\>",
                1.3277284115542631*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.250000\"\>",
                0.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.298990\"\>",
                0.29899402337872427,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299010\"\>",
                0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000013\"\>",
                -1.3277284115542631*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.500000\"\>",
                0.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552470\"\>",
                0.55246891708560886,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552490\"\>",
                0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000025\"\>",
                -2.4533222083511497*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.750000\"\>",
                0.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721840\"\>",
                0.72183542630941444,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721870\"\>",
                0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000032\"\>",
                -3.2054199383124704*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.000000\"\>",
                1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.781310\"\>",
                0.78130903533204532,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.781340\"\>",
                0.78134373054744422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000035\"\>",
                -3.4695215398894419*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.250000\"\>",
                1.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721840\"\>",
                0.72183542630941444,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721870\"\>",
                0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000032\"\>",
                -3.2054199383124704*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.500000\"\>",
                1.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552470\"\>",
                0.55246891708560886,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552490\"\>",
                0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000025\"\>",
                -2.4533222083511497*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.750000\"\>",
                1.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.298990\"\>",
                0.29899402337872427,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299010\"\>",
                0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000013\"\>",
                -1.3277284115542631*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 2.000000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["14.6 Compiled Tridiagonal Solver for Implicit Schemes", "Section",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We introduce the following compiled ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " function to solve tridiagonal systems of equations. It mimics the Package \
function originally written by J. Keiper (see the ",
  StyleBox["Mathematica ",
    FontSlant->"Italic"],
  "Packages documentation) with some minor adjustments to get it through the \
compiler."
}], "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(CompTridiagSolve\  = \ 
      Compile[{{a, \ _Real, \ 1}, {b, _Real, \ 1}, {c, \ _Real, \ 
            1}, \ {r, \ _Real, \ 1}}, \n\ 
        Module[{len = Length[r], \ 
            solution = r, \ \taux\  = \ 1/\((b[\([1]\)])\), \ aux1 = r, \ 
            a1\  = \ Prepend[a, \ 0.0], \ 
            iter}, \n\t\tsolution[\([1]\)]\  = \ aux\ *r[\([1]\)]; \n\t\tDo[
            aux1[\([iter]\)]\  = \ 
              c[\([iter - 1]\)]\ aux; \n\t\t\t\t\ \ aux\  = \ 
              1/\((b[\([iter]\)]\  - \ 
                    a1[\([iter]\)]*
                      aux1[\([iter]\)])\); \n\t\t\t\t\ \ solution[\([iter]\)]\
\  = \ \n\t\t\t\t\((r[\([iter]\)] - 
                    a1[\([iter]\)]\ solution[\([iter - 
                            1]\)])\)\ aux, \n\t\ \ \ \ {iter, \ 2, \ 
              len}]; \n\t\tDo[
            solution[\([iter]\)]\  -= \ 
              aux1[\([iter + 1]\)]\ solution[\([iter + 
                      1]\)], \n\t\ \ \ {iter, \ len - 1, \ 1, \ \(-1\)}]; \n\ 
          solution]]; \)\)], "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["14.7 Fully Implicit Scheme", "Section",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Here is the matrix involved in the computations:", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell["\<\
FullyImpCMatrix[alpha_, nminus_, nplus_] :=
Sequence[Table[-alpha, {nplus+nminus-2}], 
Table[1+2*alpha, {nplus+nminus-1}],
Table[-alpha, {nplus+nminus-2}]]\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["Here are the parameters of this particular model:", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
M=400;
nminus = 80;
nplus = 80;
dx = 0.025;
dtau = 0.00025;
alpha = dtau/dx^2\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(0.399999999999999964`\)], "Output"]
}, Open  ]],

Cell["Initial and boundary conditions, and problem initialization:", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(initial = 
      Table[N[Sin[Pi*\((k - 1 - nminus)\)/nminus]\ ], \ {k, 1, \ 
          nminus + nplus + 1}]; \nlower = Table[0, \ {m, \ 1, \ M + 1}]; \n
    upper = Table[0, \ {m, \ 1, \ M + 1}]; \[IndentingNewLine]wold\  = \ 
      initial; \nwvold\  = \ wold; \nwnew\  = \ wold; \)], "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["The solution:", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell["CMat = FullyImpCMatrix[alpha,nminus,nplus];", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
For[m=2, m<=M+1, m++,
(wvold = wold; wold = wnew;
rhs = Take[wold, {2, -2}]+
Table[Which[k==1, alpha*lower[[m]],
k== nplus + nminus-1,  alpha*upper[[m]], True, 0],
{k, 1, nplus + nminus-1}];
temp = CompTridiagSolve[CMat, rhs];
wnew = Join[{lower[[m]]}, temp, {upper[[m]]}])]\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["Interpolating the answer:", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell["\<\
interpoldata = 
Table[{(k-nminus-1)*dx ,wnew[[k]]}, {k, 1, nminus+nplus+1}];
ufunc = Interpolation[interpoldata, InterpolationOrder -> 3];\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["Now we plot the error:", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot[ufunc[x] - Sin[Pi*x/2]*Exp[-Pi^2 0.1/4], {x, -2, 2}, 
PlotPoints -> 50];\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.238095 0.309021 3495.48 [
[.02381 .29652 -6 -9 ]
[.02381 .29652 6 0 ]
[.2619 .29652 -6 -9 ]
[.2619 .29652 6 0 ]
[.7381 .29652 -3 -9 ]
[.7381 .29652 3 0 ]
[.97619 .29652 -3 -9 ]
[.97619 .29652 3 0 ]
[.4875 .04686 -54 -4.5 ]
[.4875 .04686 0 4.5 ]
[.4875 .13425 -48 -4.5 ]
[.4875 .13425 0 4.5 ]
[.4875 .22163 -54 -4.5 ]
[.4875 .22163 0 4.5 ]
[.4875 .39641 -48 -4.5 ]
[.4875 .39641 0 4.5 ]
[.4875 .4838 -42 -4.5 ]
[.4875 .4838 0 4.5 ]
[.4875 .57118 -48 -4.5 ]
[.4875 .57118 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .30902 m
.02381 .31527 L
s
[(-2)] .02381 .29652 0 1 Mshowa
.2619 .30902 m
.2619 .31527 L
s
[(-1)] .2619 .29652 0 1 Mshowa
.7381 .30902 m
.7381 .31527 L
s
[(1)] .7381 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(2)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.07143 .30902 m
.07143 .31277 L
s
.11905 .30902 m
.11905 .31277 L
s
.16667 .30902 m
.16667 .31277 L
s
.21429 .30902 m
.21429 .31277 L
s
.30952 .30902 m
.30952 .31277 L
s
.35714 .30902 m
.35714 .31277 L
s
.40476 .30902 m
.40476 .31277 L
s
.45238 .30902 m
.45238 .31277 L
s
.54762 .30902 m
.54762 .31277 L
s
.59524 .30902 m
.59524 .31277 L
s
.64286 .30902 m
.64286 .31277 L
s
.69048 .30902 m
.69048 .31277 L
s
.78571 .30902 m
.78571 .31277 L
s
.83333 .30902 m
.83333 .31277 L
s
.88095 .30902 m
.88095 .31277 L
s
.92857 .30902 m
.92857 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.5 .04686 m
.50625 .04686 L
s
[(-0.000075)] .4875 .04686 1 0 Mshowa
.5 .13425 m
.50625 .13425 L
s
[(-0.00005)] .4875 .13425 1 0 Mshowa
.5 .22163 m
.50625 .22163 L
s
[(-0.000025)] .4875 .22163 1 0 Mshowa
.5 .39641 m
.50625 .39641 L
s
[(0.000025)] .4875 .39641 1 0 Mshowa
.5 .4838 m
.50625 .4838 L
s
[(0.00005)] .4875 .4838 1 0 Mshowa
.5 .57118 m
.50625 .57118 L
s
[(0.000075)] .4875 .57118 1 0 Mshowa
.125 Mabswid
.5 .06434 m
.50375 .06434 L
s
.5 .08182 m
.50375 .08182 L
s
.5 .09929 m
.50375 .09929 L
s
.5 .11677 m
.50375 .11677 L
s
.5 .15173 m
.50375 .15173 L
s
.5 .1692 m
.50375 .1692 L
s
.5 .18668 m
.50375 .18668 L
s
.5 .20416 m
.50375 .20416 L
s
.5 .23911 m
.50375 .23911 L
s
.5 .25659 m
.50375 .25659 L
s
.5 .27407 m
.50375 .27407 L
s
.5 .29154 m
.50375 .29154 L
s
.5 .3265 m
.50375 .3265 L
s
.5 .34398 m
.50375 .34398 L
s
.5 .36145 m
.50375 .36145 L
s
.5 .37893 m
.50375 .37893 L
s
.5 .41389 m
.50375 .41389 L
s
.5 .43136 m
.50375 .43136 L
s
.5 .44884 m
.50375 .44884 L
s
.5 .46632 m
.50375 .46632 L
s
.5 .50127 m
.50375 .50127 L
s
.5 .51875 m
.50375 .51875 L
s
.5 .53623 m
.50375 .53623 L
s
.5 .5537 m
.50375 .5537 L
s
.5 .02938 m
.50375 .02938 L
s
.5 .01191 m
.50375 .01191 L
s
.5 .58866 m
.50375 .58866 L
s
.5 .60614 m
.50375 .60614 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30902 m
.04273 .27238 L
.06337 .23311 L
.08275 .19745 L
.10139 .16488 L
.12123 .13269 L
.14033 .10451 L
.16063 .07799 L
.18019 .05655 L
.199 .03982 L
.20853 .03278 L
.21902 .02649 L
.22909 .02172 L
.23349 .01996 L
.23829 .01828 L
.24317 .01702 L
.2478 .01612 L
.24912 .01582 L
.24944 .01575 L
.24977 .01567 L
.25037 .01559 L
.2507 .01557 L
.25106 .01554 L
.2518 .01548 L
.2531 .01535 L
.25425 .0152 L
.25546 .01501 L
.2561 .01493 L
.25646 .01494 L
.25679 .01494 L
.25739 .01494 L
.25803 .01494 L
.25834 .01493 L
.25868 .01492 L
.25904 .01491 L
.25937 .01489 L
.26059 .01481 L
.2612 .01477 L
.26177 .01472 L
.26209 .01472 L
.26243 .01475 L
.26303 .0148 L
.26418 .01488 L
.26476 .01491 L
.2654 .01493 L
.26594 .01494 L
.26653 .01494 L
.26708 .01494 L
.26758 .01493 L
.26821 .01499 L
Mistroke
.26857 .01505 L
.26889 .0151 L
.2701 .01527 L
.27151 .01544 L
.27284 .01555 L
.27345 .01559 L
.27409 .01568 L
.27464 .01581 L
.27524 .01595 L
.27783 .01646 L
.2874 .01898 L
.29277 .02087 L
.29765 .02285 L
.30702 .02778 L
.31719 .03418 L
.33719 .05039 L
.35645 .07017 L
.37497 .09286 L
.39468 .12067 L
.41366 .15037 L
.43384 .18461 L
.45327 .21973 L
.47196 .25491 L
.49185 .29322 L
.511 .33036 L
.53135 .36944 L
.55096 .40607 L
.56983 .43981 L
.58989 .47349 L
.60922 .50312 L
.6278 .52869 L
.64759 .55233 L
.66663 .57122 L
.67632 .5791 L
.68687 .58656 L
.69614 .59211 L
.70637 .59679 L
.71604 .60011 L
.72135 .60146 L
.72373 .60187 L
.72498 .60215 L
.72568 .60231 L
.72602 .60239 L
.72633 .60244 L
.72695 .60248 L
.72763 .60253 L
.72882 .60264 L
.72994 .60277 L
.73114 .60295 L
.73174 .60305 L
Mistroke
.73238 .60311 L
.73299 .6031 L
.73355 .6031 L
.73389 .6031 L
.7342 .60311 L
.73454 .60311 L
.73491 .60312 L
.73526 .60314 L
.73559 .60315 L
.7362 .60319 L
.73731 .60327 L
.73786 .60332 L
.73847 .60331 L
.73899 .60326 L
.73955 .60322 L
.74056 .60315 L
.74112 .60313 L
.74173 .60311 L
.74231 .6031 L
.74284 .6031 L
.7434 .6031 L
.74399 .60311 L
.7446 .60302 L
.74526 .60292 L
.74785 .60259 L
.74911 .60249 L
.74966 .60245 L
.75025 .60237 L
.75061 .60228 L
.75095 .60221 L
.75161 .60205 L
.75306 .60176 L
.75441 .60152 L
.75507 .60142 L
.75567 .60134 L
.75624 .60121 L
.75677 .60104 L
.75795 .60068 L
.76046 .60003 L
.76553 .59839 L
.77526 .59443 L
.78427 .58968 L
.80347 .57638 L
.82388 .55733 L
.84354 .53486 L
.86245 .50968 L
.88257 .4794 L
.90195 .44741 L
.92058 .41454 L
.94042 .37784 L
Mistroke
.95951 .34134 L
.97619 .30902 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  GroupPageBreakWithin->True,
  AspectRatioFixed->True,
  ImageSize->{169, 104.312},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003C0000PQ000`40O003h00Odgoo001ZOol00`00
Oomoo`1VOol006Yoo`03001oogoo06Ioo`00JWoo0P00Igoo000dOol:000/Ool00`00Oomoo`1VOol0
035oo`<000Yoo`@002Qoo`03001oogoo06Ioo`00;goo0P004Goo00<007ooOol09Goo00<007ooOol0
IWoo000]Ool2000DOol00`00Oomoo`0TOol2001WOol002]oo`<000Ioo`8000Aoo`<000Eoo`@000Ao
o`<000Eoo`<000Ioo`03001oogoo00=oo`@000Eoo`03001oogoo06Ioo`00:Woo0P000Woo00<007oo
Ool00goo0P000goo00D007ooOomoo`0000=oo`04001oogooOol20002Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000Eoo`03001oogoo00Moo`03001oogoo009oo`03001oogoo06Ioo`008Woo1P00
00=oo`0000000goo00<007ooOol027oo00D007ooOomoo`0000=oo`09001oogooOol007oo001oo`00
00=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`06Ool00`00Oomoo`02Ool2001W
Ool002Qoo`03001oo`0000=oo`03001oogoo00Qoo`05001oogooOol00003Ool01@00Oomoogoo0000
0Woo0P000goo00D007ooOomoo`0000=oo`03001oogoo00Aoo`03001oogoo00Ioo`03001oogoo009o
o`03001oogoo06Ioo`009goo00@007ooOol000=oo`03001oogoo00Qoo`05001oogooOol00003Ool0
1@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00006Ool00`00Oomoo`02Ool4
0005Ool00`00Oomoo`1VOol002Moo`04001oogoo0003Ool00`00Oomoo`08Ool01@00Oomoogoo0000
0goo00D007ooOomoo`0000=oo`80009oo`05001oogooOol00003Ool00`00Oomoo`05Ool01@00Oomo
ogoo000027oo00<007ooOol0IWoo000VOol01@00Oomoogoo00000goo00<007ooOol027oo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool01@00Ool007oo00000goo00D007ooOomoo`0000=oo`05
001oogooOol00003Ool00`00Oomoo`06Ool2001WOol002Eoo`03001oogoo00=oo`<000aoo`<000Eo
o`<000Eoo`<000Eoo`<000Aoo`D000=oo`@000Eoo`03001oogoo06Ioo`0097oo00<007ooOol09Goo
00<007ooOol06goo00<007ooOol0IWoo000SOol00`00Oomoo`0WOol00`00Oomoo`0JOol00`00Oomo
o`1VOol002=oo`03001oogoo02Qoo`03001oogoo01Uoo`8006Moo`008Woo00<007ooOol0:Woo00<0
07ooOol067oo00<007ooOol0IWoo000QOol00`00Oomoo`0/Ool00`00Oomoo`0GOol00`00Oomoo`1V
Ool0021oo`03001oogoo02eoo`03001oogoo01Moo`8006Moo`007goo00<007ooOol0;goo00<007oo
Ool05Woo00<007ooOol0IWoo000OOol00`00Oomoo`0`Ool00`00Oomoo`0EOol00`00Oomoo`1VOol0
01ioo`03001oogoo035oo`03001oogoo01Eoo`03001oogoo06Ioo`007Goo00<007ooOol0<goo00<0
07ooOol057oo0P00Igoo000MOol00`00Oomoo`0COol30006Ool20004Ool30005Ool30005Ool30005
Ool30005Ool40004Ool00`00Oomoo`1VOol001aoo`03001oogoo01=oo`05001oogooOol00005Ool2
0003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`03001oogoo008000=oo`05001oogoo
Ool00008Ool01@00Oomoogoo0000J7oo000KOol00`00Oomoo`0<Ool60002Ool01@00Oomoogoo0000
2Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomo
o`0000Qoo`05001oogooOol0001XOol001]oo`03001oogoo01Aoo`05001oogooOol0000:Ool01@00
Oomoogoo00000goo00D007ooOomoo`0000=oo`04001oogooOol20002Ool01@00Oomoogoo000027oo
00@007ooOomoo`8006Moo`006Woo00<007ooOol05Goo00D007ooOomoo`0000Yoo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00@007ooOomoo`80009oo`05001oogooOol00004Ool40004Ool0
0`00Oomoo`1VOol001Yoo`03001oogoo01Eoo`05001oogooOol0000:Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`09001oogooOol007oo001oo`0000=oo`03001oogoo009oo`03001oogoo
00Eoo`03001oogoo06Ioo`006Goo00<007ooOol05Woo00D007ooOomoo`0000Yoo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00D007ooOomoo`00009oo`8000=oo`03001oogoo009oo`03001o
ogoo00Eoo`8006Moo`0067oo00<007ooOol067oo0`0037oo0`001Goo0`001Goo0`000goo00<007oo
00000P001Goo100017oo00<007ooOol0IWoo000HOol00`00Oomoo`0nOol00`00Oomoo`0>Ool00`00
Oomoo`1VOol001Moo`03001oogoo03moo`03001oogoo00ioo`03001oogoo06Ioo`005goo00<007oo
Ool0@7oo00<007ooOol03Goo0P00Igoo000FOol00`00Oomoo`12Ool00`00Oomoo`0<Ool00`00Oomo
o`1VOol001Ioo`03001oogoo049oo`03001oogoo00aoo`03001oogoo06Ioo`005Goo00<007ooOol0
A7oo00<007ooOol02goo0P00Igoo000DOol00`00Oomoo`15Ool00`00Oomoo`0;Ool00`00Oomoo`1V
Ool001Aoo`03001oogoo04Ioo`03001oogoo00Yoo`03001oogoo06Ioo`004goo00<007ooOol0Agoo
00<007ooOol02Woo00<007ooOol0IWoo000COol00`00Oomoo`18Ool00`00Oomoo`09Ool2001WOol0
019oo`03001oogoo01Ioo`<000Ioo`8000Aoo`<000Eoo`<000Eoo`<000Eoo`<000Aoo`D00003Ool0
07oo00@000Eoo`03001oogoo06Ioo`004Woo00<007ooOol05Goo00D007ooOomoo`0000Eoo`8000=o
o`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol0
0004Ool01P00Oomoo`00Ool000Eoo`03001oogoo009oo`03001oogoo06Ioo`004Goo00<007ooOol0
3Woo1P000Woo00D007ooOomoo`0000Yoo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D0
07ooOomoo`0000=oo`05001oogooOol00005Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomo
o`02Ool00`00Oomoo`1VOol0011oo`03001oogoo01Moo`05001oogooOol0000:Ool01@00Oomoogoo
00000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00001Woo00D007oo
Oomoo`0000Aoo`03001oogoo009oo`8006Moo`0047oo00<007ooOol05goo00D007ooOomoo`0000Yo
o`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol0
0007Ool01000Oomoogoo10001Goo00<007ooOol0IWoo000?Ool00`00Oomoo`0HOol01@00Oomoogoo
00002Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000Moo`05001oogooOol00008Ool00`00Oomoo`1VOol000moo`03001oogoo01Qoo`05001o
ogooOol0000:Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool0
1@00Oomoogoo00000goo00D007ooOomoo`0000=oo`8000Moo`8006Moo`003Woo00<007ooOol06Woo
0`0037oo0`001Goo0`001Goo0`001Goo0`001Goo0`0017oo10001Goo00<007ooOol0IWoo000>Ool0
0`00Oomoo`1BOol00`00Oomoo`04Ool00`00Oomoo`1VOol000Qoo`H002eoo`D002=oo`03001oogoo
00Aoo`03001oogoo02aoo`D002aoo`D000Aoo`002Goo00<007ooOol00P00;goo00<007ooOol097oo
00<007ooOol00goo0P00;goo00<007ooOol0;Goo00@007ooOol000Aoo`001P0017oo00<007oo0000
9Goo1P001Goo00<007ooOol097oo00<007ooOol00goo00<007ooOol0;Woo00<007ooOol0;Woo00<0
07ooOol017oo000;Ool2000`Ool00`00Oomoo`0UOol00`00Oomoo`02Ool00`00Oomoo`0^Ool00`00
Oomoo`0_Ool00`00Oomoo`03Ool000]oo`80031oo`03001oogoo02Eoo`03001oogoo009oo`03001o
ogoo02ioo`03001oogoo031oo`03001oogoo009oo`002goo0P00<7oo00<007ooOol09Woo00@007oo
Oomoo`8002moo`03001oogoo031oo`03001oogoo009oo`0027oo00D007oo001oo`00031oo`03001o
ogoo02Ioo`05001oogooOol0000`Ool00`00Oomoo`0/Ool01@00Oomoogoo000017oo0009Ool3000_
Ool3000YOol01000Oomoo`00;Woo0`00;goo0`001Goo0009Ool00`00Oomoo`1KOol00`00Oomoo`02
001WOol000Uoo`03001oogoo05aoo`03001oo`0006Qoo`0027oo00<007ooOol0GGoo00<007oo0000
J7oo0008Ool00`00Oomoo`1NOol2001XOol0009oom4000001goo00<007ooOol01goo00<007ooOol0
1goo00<007ooOol01goo00<007ooOol01goo00<007ooOol01Woo00<007ooOol01goo00<007ooOol0
1goo00<007ooOol01goo00<007ooOol01goo00<007ooOol01goo00<007ooOol01goo00<007ooOol0
1goo00<007ooOol01goo00<007ooOol01Woo00<007ooOol01goo00<007ooOol01goo00<007ooOol0
1goo00<007ooOol01goo00<007ooOol01goo00<007ooOol01goo00<007ooOol00goo001ZOol2001P
Ool00`00Oomoo`04Ool006Yoo`80061oo`03001oogoo00Aoo`00JWoo00<007oo0000GWoo00<007oo
Ool01Goo001ZOol00`00Ool0001MOol00`00Oomoo`06Ool006Yoo`04001oogoo001LOol00`00Oomo
o`06Ool006Yoo`800003Ool007oo05Yoo`03001oogoo00Moo`00JWoo00D007ooOomoo`0005Yoo`03
001oogoo00Moo`00JWoo00D007ooOomoo`0005Uoo`03001oogoo00Qoo`00JWoo00<007ooOol00Woo
00<007ooOol0EWoo00<007ooOol027oo001ZOol20003Ool00`00Oomoo`1EOol00`00Oomoo`09Ool0
06Yoo`03001oogoo00=oo`03001oogoo05=oo`03001oogoo00Yoo`00JWoo00<007ooOol00goo00<0
07ooOol0Dgoo00<007ooOol02Woo001ZOol20005Ool00`00Oomoo`1AOol00`00Oomoo`0;Ool002]o
o`<000Ioo`8000Eoo`<000Aoo`<000Eoo`<000Eoo`<000Aoo`D000Aoo`@000Aoo`03001oogoo00Ao
o`03001oogoo055oo`03001oogoo00]oo`00:Woo00D007ooOomoo`0000Eoo`8000Aoo`05001oogoo
Ool00002Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00004Ool01000
Oomoo`0027oo00D007ooOomoo`0000Moo`03001oogoo04moo`03001oogoo00aoo`00:Woo00D007oo
Oomoo`0000]oo`05001oogooOol00002Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05
001oogooOol00005Ool00`00Oomoo`08Ool01@00Oomoogoo00001goo00<007ooOol0Cgoo00<007oo
Ool037oo000ZOol01@00Oomoogoo00002goo00D007ooOomoo`00009oo`05001oogooOol00003Ool0
1@00Oomoogoo00000goo00D007ooOomoo`0000Ioo`03001oogoo00Moo`04001oogooOol20007Ool0
0`00Oomoo`1=Ool00`00Oomoo`0=Ool002Yoo`05001oogooOol0000;Ool01@00Oomoogoo00000Woo
00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00001goo00<007ooOol00Woo
100017oo00<007ooOol01Woo00<007ooOol0CGoo00<007ooOol03Goo000ZOol01@00Oomoogoo0000
2goo00D007ooOomoo`00009oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomo
o`0000Moo`03001oogoo009oo`03001oogoo00Eoo`03001oogoo00Moo`03001oogoo04]oo`03001o
ogoo00ioo`00:Woo00D007ooOomoo`0000]oo`05001oogooOol00002Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo000017oo00<007ooOol01Goo
00<007ooOol01goo00<007ooOol0Bgoo00<007ooOol03Woo000[Ool3000=Ool30004Ool30005Ool3
0005Ool30005Ool30005Ool40004Ool20009Ool00`00Oomoo`19Ool00`00Oomoo`0?Ool006Yoo`03
001oogoo00Qoo`03001oogoo04Uoo`03001oogoo00moo`00JWoo00<007ooOol02Goo00<007ooOol0
Agoo00<007ooOol047oo001ZOol2000;Ool00`00Oomoo`16Ool00`00Oomoo`0@Ool006Yoo`03001o
ogoo00Yoo`03001oogoo04Eoo`03001oogoo015oo`00JWoo00<007ooOol02goo00<007ooOol0@goo
00<007ooOol04Woo001ZOol00`00Oomoo`0;Ool00`00Oomoo`13Ool00`00Oomoo`0BOol006Yoo`80
00eoo`03001oogoo045oo`03001oogoo01=oo`00JWoo00<007ooOol03Goo00<007ooOol0@7oo00<0
07ooOol04goo001ZOol00`00Oomoo`0>Ool00`00Oomoo`0nOol00`00Oomoo`0DOol006Yoo`8000mo
o`03001oogoo03eoo`03001oogoo01Eoo`00JWoo00<007ooOol03goo00<007ooOol0?7oo00<007oo
Ool05Goo000cOol30006Ool20005Ool30004Ool30005Ool30005Ool30005Ool40004Ool00`00Oomo
o`0@Ool00`00Oomoo`0jOol00`00Oomoo`0FOol0039oo`05001oogooOol00005Ool20004Ool01@00
Oomoogoo00000Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo000027oo
00D007ooOomoo`00019oo`03001oogoo03Uoo`03001oogoo01Moo`00<Woo00D007ooOomoo`0000]o
o`05001oogooOol00002Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol0
0008Ool01000Oomoogoo0P004Woo00<007ooOol0>7oo00<007ooOol05goo000bOol01@00Oomoogoo
00002goo00D007ooOomoo`00009oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000Qoo`05001oogooOol0000DOol00`00Oomoo`0fOol00`00Oomoo`0HOol0039oo`05001o
ogooOol0000;Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool0
1@00Oomoogoo000017oo100017oo00<007ooOol04Woo00<007ooOol0=Goo00<007ooOol06Goo000b
Ool01@00Oomoogoo00002goo00D007ooOomoo`00009oo`05001oogooOol00003Ool01@00Oomoogoo
00000goo00D007ooOomoo`0000Aoo`03001oogoo00Eoo`03001oogoo01=oo`03001oogoo03Aoo`03
001oogoo01Uoo`00<Woo00D007ooOomoo`0000]oo`05001oogooOol00002Ool01@00Oomoogoo0000
0goo00D007ooOomoo`0000=oo`05001oogooOol00004Ool00`00Oomoo`05Ool2000EOol00`00Oomo
o`0bOol00`00Oomoo`0JOol003=oo`<000eoo`<000Aoo`<000Eoo`<000Eoo`<000Eoo`@000Aoo`03
001oogoo01Aoo`03001oogoo035oo`03001oogoo01]oo`00JWoo00<007ooOol05Goo00<007ooOol0
<7oo00<007ooOol06goo001ZOol2000GOol00`00Oomoo`0^Ool00`00Oomoo`0LOol006Yoo`03001o
ogoo01Ioo`03001oogoo02eoo`03001oogoo01eoo`00JWoo00<007ooOol05goo00<007ooOol0;7oo
00<007ooOol07Goo001ZOol00`00Oomoo`0HOol00`00Oomoo`0ZOol00`00Oomoo`0NOol006Yoo`80
01Yoo`03001oogoo02Qoo`03001oogoo01moo`00JWoo00<007ooOol06Woo00<007ooOol09Woo00<0
07ooOol087oo001ZOol00`00Oomoo`0KOol00`00Oomoo`0TOol00`00Oomoo`0QOol006Yoo`03001o
ogoo01aoo`03001oogoo029oo`03001oogoo029oo`00JWoo0P007Woo00<007ooOol087oo00<007oo
Ool08goo000[Ool30006Ool20005Ool30004Ool30005Ool30005Ool30006Ool00`00Oomoo`04Ool4
0004Ool00`00Oomoo`0NOol00`00Oomoo`0NOol00`00Oomoo`0TOol002Yoo`05001oogooOol00005
Ool20004Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00
Oomoogoo00001Goo00<007ooOol027oo00D007ooOomoo`00025oo`03001oogoo01aoo`03001oogoo
02Eoo`00:Woo00D007ooOomoo`0000]oo`05001oogooOol00002Ool01@00Oomoogoo00000goo00D0
07ooOomoo`0000=oo`05001oogooOol00006Ool00`00Oomoo`07Ool01000Oomoogoo0P008Goo00<0
07ooOol06Woo00<007ooOol09Woo000ZOol01@00Oomoogoo00002goo00D007ooOomoo`00009oo`05
001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Ioo`03001oogoo00Moo`05
001oogooOol0000SOol00`00Oomoo`0GOol2000YOol002Yoo`05001oogooOol0000;Ool01@00Oomo
ogoo00000Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00001Woo00<0
07ooOol00goo100017oo00<007ooOol08Woo0P005Woo00<007ooOol0:Goo000ZOol01@00Oomoogoo
00002goo00D007ooOomoo`00009oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000Moo`03001oogoo009oo`03001oogoo00Eoo`03001oogoo02Aoo`03001oogoo019oo`03
001oogoo02Yoo`00:Woo00D007ooOomoo`0000]oo`05001oogooOol00002Ool01@00Oomoogoo0000
0goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo000017oo00<007ooOol0
1Goo0P009Woo00<007ooOol03goo0P00;Goo000[Ool3000=Ool30004Ool30005Ool30005Ool30004
Ool50004Ool40004Ool00`00Oomoo`0VOol2000=Ool2000_Ool006Yoo`03001oogoo02Qoo`<000Mo
o`<0035oo`00JWoo00<007ooOol0:goo2000<goo001ZOol2001WOol006Yoo`03001oogoo06Ioo`00
JWoo00<007ooOol0IWoo0000\
\>"],
  ImageRangeCache->{{{0, 210.25}, {129.375, 0}} -> {-2.15169, -8.91045*^-5, 
  0.0252777, 1.72179*^-6}}]
}, Open  ]],

Cell["\<\
Finally we tabulate the numerical result, the exact result and the \
error in the numerical scheme.\
\>", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell["\<\
samples = TableForm[Join[{{\"x\", \"Exact\", \"Implicit FD\", \
\"Error\"}},
Table[Map[PaddedForm[N[Chop[#1]],{5,6}]&,
N[{x, ufunc[x], Sin[Pi*x/2]*Exp[-(Pi^2 0.1)/4], 
ufunc[x]- Sin[Pi*x/2]*Exp[-(Pi^2 0.1)/4]},5]],
{x, -2, 2, 0.25}]]]\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"x\"\>", "\<\"Exact\"\>", "\<\"Implicit FD\"\>", 
            "\<\"Error\"\>"},
          {
            TagBox[
              InterpretationBox["\<\"-2.000000\"\>",
                -2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.750000\"\>",
                -1.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299040\"\>",
                -0.2990395225540155,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299010\"\>",
                -0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000032\"\>",
                -3.2221891175687745*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.500000\"\>",
                -1.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552550\"\>",
                -0.55255298859920743,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552490\"\>",
                -0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000060\"\>",
                -5.953829151505996*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.250000\"\>",
                -1.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721950\"\>",
                -0.72194527103548389,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721870\"\>",
                -0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000078\"\>",
                -7.7790526686327688*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.000000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.781430\"\>",
                -0.78142793040678593,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.781340\"\>",
                -0.78134373054744422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000084\"\>",
                -8.4199859341715921*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.750000\"\>",
                -0.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721950\"\>",
                -0.72194527103548445,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721870\"\>",
                -0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000078\"\>",
                -7.7790526686882799*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.500000\"\>",
                -0.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552550\"\>",
                -0.55255298859920887,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552490\"\>",
                -0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000060\"\>",
                -5.953829151650325*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.250000\"\>",
                -0.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299040\"\>",
                -0.29903952255401789,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299010\"\>",
                -0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000032\"\>",
                -3.2221891178074724*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.250000\"\>",
                0.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299040\"\>",
                0.29903952255401733,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299010\"\>",
                0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000032\"\>",
                3.2221891177519613*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.500000\"\>",
                0.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552550\"\>",
                0.55255298859920832,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552490\"\>",
                0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000060\"\>",
                5.9538291515948139*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.750000\"\>",
                0.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721950\"\>",
                0.72194527103548389,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721870\"\>",
                0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000078\"\>",
                7.7790526686327688*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.000000\"\>",
                1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.781430\"\>",
                0.78142793040678593,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.781340\"\>",
                0.78134373054744422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000084\"\>",
                8.4199859341715921*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.250000\"\>",
                1.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721950\"\>",
                0.72194527103548445,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721870\"\>",
                0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000078\"\>",
                7.7790526686882799*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.500000\"\>",
                1.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552550\"\>",
                0.55255298859920865,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552490\"\>",
                0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000060\"\>",
                5.9538291516281205*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.750000\"\>",
                1.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299040\"\>",
                0.29903952255401772,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299010\"\>",
                0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000032\"\>",
                3.2221891177908191*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 2.000000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell["\<\
Note that we obtain no improvement in accuracy over the explicit \
scheme. The only advantage of the fully implicit scheme over the explicit \
scheme is the fact that we can increase \[Alpha], that is, the time-step for \
a given price-step, without the system going unstable. You might like to \
explore this, and the accuracy of the solution, by adjusting the parameters \
given above and re-running. \
\>", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["14.8 Crank-Nicholson", "Section",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->Automatic,
  AspectRatioFixed->True],

Cell["\<\
The fully implicit analysis can be repeated with a Crank-Nicholson \
scheme by making some minor changes to the difference algorithm. This time we \
also implement two different interpolation schemes - linear and cubic.\
\>", 
  "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell["\<\
CNCMatrix[alpha_, nminus_, nplus_] :=
Sequence[Table[-alpha/2, {nplus+nminus-2}],
Table[1+alpha, {nplus+nminus-1}],
Table[-alpha/2, {nplus+nminus-2}]];

CNDMatrix[alpha_, vec_List] := Module[{temp},
temp = (1 - alpha)*vec + (alpha/2)*(RotateRight[vec] + RotateLeft[vec]);
temp[[1]] = Simplify[First[temp] - \.0calpha*Last[vec]/2];
temp[[-1]] = Simplify[Last[temp] - alpha*First[vec]/2];
temp];

M=40; nminus = 80; nplus = 80;
dx = 0.025; dtau = 0.0025; alpha = dtau/dx^2;
initial=Table[N[Sin[Pi*(k-1-nminus)/nminus] ], {k,1, nminus+nplus+1}];
lower=Table[0, {m, 1, M+1}]; upper=Table[0, {m, 1, M+1}];
wold = initial; wvold = wold; wnew = wold;
CMat = CNCMatrix[alpha,nminus,nplus];

For[m=2, m<=M+1, m++,
(wvold = wold; wold = wnew;
rhs = CNDMatrix[alpha, Take[wold, {2, -2}]]+
Table[Which[k==1, alpha*(lower[[m-1]] + lower[[m]])/2,
k== nplus + nminus-1,  alpha*(upper[[m-1]] + upper[[m]])/2,
True, 0],
{k, 1, nplus + nminus-1}];
temp = CompTridiagSolve[CMat, rhs];
wnew = Join[{lower[[m]]}, temp, {upper[[m]]}])
];

interpoldata = 
Table[{(k-nminus-1)*dx ,wnew[[k]]}, {k, 1, nminus+nplus+1}];

ufunca = Interpolation[interpoldata, InterpolationOrder -> 1];
ufuncb = Interpolation[interpoldata, InterpolationOrder -> 3];\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell["\<\
Now we plot the error, with both linear and cubic \
interpolation:\
\>", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot[ufunca[x] - Sin[Pi*x/2]*Exp[-Pi^2 0.1/4], {x, -2, 2}, 
PlotPoints -> 50];\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.238095 0.309097 2328.46 [
[.02381 .2966 -6 -9 ]
[.02381 .2966 6 0 ]
[.2619 .2966 -6 -9 ]
[.2619 .2966 6 0 ]
[.7381 .2966 -3 -9 ]
[.7381 .2966 3 0 ]
[.97619 .2966 -3 -9 ]
[.97619 .2966 3 0 ]
[.4875 .07625 -42 -4.5 ]
[.4875 .07625 0 4.5 ]
[.4875 .19267 -48 -4.5 ]
[.4875 .19267 0 4.5 ]
[.4875 .42552 -42 -4.5 ]
[.4875 .42552 0 4.5 ]
[.4875 .54194 -36 -4.5 ]
[.4875 .54194 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .3091 m
.02381 .31535 L
s
[(-2)] .02381 .2966 0 1 Mshowa
.2619 .3091 m
.2619 .31535 L
s
[(-1)] .2619 .2966 0 1 Mshowa
.7381 .3091 m
.7381 .31535 L
s
[(1)] .7381 .2966 0 1 Mshowa
.97619 .3091 m
.97619 .31535 L
s
[(2)] .97619 .2966 0 1 Mshowa
.125 Mabswid
.07143 .3091 m
.07143 .31285 L
s
.11905 .3091 m
.11905 .31285 L
s
.16667 .3091 m
.16667 .31285 L
s
.21429 .3091 m
.21429 .31285 L
s
.30952 .3091 m
.30952 .31285 L
s
.35714 .3091 m
.35714 .31285 L
s
.40476 .3091 m
.40476 .31285 L
s
.45238 .3091 m
.45238 .31285 L
s
.54762 .3091 m
.54762 .31285 L
s
.59524 .3091 m
.59524 .31285 L
s
.64286 .3091 m
.64286 .31285 L
s
.69048 .3091 m
.69048 .31285 L
s
.78571 .3091 m
.78571 .31285 L
s
.83333 .3091 m
.83333 .31285 L
s
.88095 .3091 m
.88095 .31285 L
s
.92857 .3091 m
.92857 .31285 L
s
.25 Mabswid
0 .3091 m
1 .3091 L
s
.5 .07625 m
.50625 .07625 L
s
[(-0.0001)] .4875 .07625 1 0 Mshowa
.5 .19267 m
.50625 .19267 L
s
[(-0.00005)] .4875 .19267 1 0 Mshowa
.5 .42552 m
.50625 .42552 L
s
[(0.00005)] .4875 .42552 1 0 Mshowa
.5 .54194 m
.50625 .54194 L
s
[(0.0001)] .4875 .54194 1 0 Mshowa
.125 Mabswid
.5 .09954 m
.50375 .09954 L
s
.5 .12282 m
.50375 .12282 L
s
.5 .14611 m
.50375 .14611 L
s
.5 .16939 m
.50375 .16939 L
s
.5 .21596 m
.50375 .21596 L
s
.5 .23924 m
.50375 .23924 L
s
.5 .26253 m
.50375 .26253 L
s
.5 .28581 m
.50375 .28581 L
s
.5 .33238 m
.50375 .33238 L
s
.5 .35567 m
.50375 .35567 L
s
.5 .37895 m
.50375 .37895 L
s
.5 .40224 m
.50375 .40224 L
s
.5 .4488 m
.50375 .4488 L
s
.5 .47209 m
.50375 .47209 L
s
.5 .49537 m
.50375 .49537 L
s
.5 .51866 m
.50375 .51866 L
s
.5 .05297 m
.50375 .05297 L
s
.5 .02968 m
.50375 .02968 L
s
.5 .0064 m
.50375 .0064 L
s
.5 .56523 m
.50375 .56523 L
s
.5 .58851 m
.50375 .58851 L
s
.5 .6118 m
.50375 .6118 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .3091 m
.02491 .31197 L
.02551 .31329 L
.02609 .31425 L
.02662 .31479 L
.0272 .31491 L
.02772 .3145 L
.02821 .31363 L
.02878 .31189 L
.02932 .30949 L
.02991 .30861 L
.03054 .31525 L
.03086 .31804 L
.03121 .32067 L
.03184 .32424 L
.03245 .32613 L
.03302 .32636 L
.03363 .32489 L
.03419 .32183 L
.03478 .31681 L
.03542 .30901 L
.03577 .30576 L
.03608 .31195 L
.03678 .32359 L
.03742 .33133 L
.03802 .33598 L
.03859 .33788 L
.03919 .33716 L
.03971 .33424 L
.04028 .32846 L
.04089 .31918 L
.04121 .31299 L
.04154 .30547 L
.04216 .31645 L
.04273 .32949 L
.04336 .34018 L
.04371 .34442 L
.04403 .34717 L
.04465 .34944 L
.04523 .34775 L
.04557 .34511 L
.04592 .34094 L
.04655 .32991 L
.0469 .32175 L
.04723 .31278 L
.04759 .30143 L
.04797 .31347 L
.04858 .33253 L
.0489 .34067 L
.04925 .34785 L
Mistroke
.04987 .35697 L
.05045 .36066 L
.05107 .35949 L
.05164 .35352 L
.05216 .34406 L
.05272 .32939 L
.05304 .31884 L
.05339 .30579 L
.05401 .31796 L
.05439 .3327 L
.05475 .34455 L
.05507 .35335 L
.05542 .36113 L
.05577 .36676 L
.05613 .37058 L
.05647 .37212 L
.05679 .3718 L
.05716 .36923 L
.0575 .36476 L
.05788 .35752 L
.05827 .34722 L
.05888 .32577 L
.05922 .31111 L
.05953 .29658 L
.06015 .32841 L
.06071 .35114 L
.06103 .36159 L
.06137 .37045 L
.06199 .38062 L
.06233 .38308 L
.06265 .38328 L
.063 .38117 L
.06337 .37615 L
.06397 .36228 L
.0643 .35111 L
.06462 .33842 L
.06526 .30643 L
.06562 .30337 L
.06595 .32315 L
.06656 .35311 L
.06712 .3738 L
.0677 .38788 L
.06835 .3944 L
.06868 .39404 L
.06905 .39067 L
.0694 .38461 L
.06972 .37652 L
.07032 .35459 L
.07096 .32177 L
.07129 .30091 L
Mistroke
.07165 .30787 L
.07229 .3478 L
.07269 .36709 L
.07306 .38183 L
.0734 .39207 L
.07376 .39998 L
.07408 .4041 L
.07443 .40557 L
.07506 .40023 L
.07567 .38504 L
.07624 .36228 L
.07656 .34555 L
.0769 .32455 L
.07752 .30086 L
.07816 .34699 L
.07853 .36811 L
.07887 .38427 L
.07948 .40509 L
.0798 .4118 L
.08015 .41569 L
.08072 .41465 L
.08135 .40241 L
.08191 .38172 L
.08252 .34898 L
.08309 .30802 L
.08361 .31231 L
.08848 .3524 L
.09367 .4034 L
.09426 .36851 L
.09489 .31821 L
.09521 .28642 L
.09557 .3177 L
.0962 .37188 L
.09679 .40978 L
.09743 .4361 L
.09803 .44719 L
.09858 .44555 L
.09911 .4331 L
.09969 .40755 L
.10023 .3718 L
.10074 .32896 L
.10132 .29668 L
.10194 .35879 L
.10246 .39949 L
.10303 .43187 L
.10363 .45195 L
.10426 .45783 L
.10486 .44847 L
.10541 .42728 L
.10597 .39325 L
Mistroke
.10656 .34334 L
.10706 .2894 L
.10762 .33448 L
.10821 .38989 L
.10885 .43307 L
.10921 .44988 L
.10954 .46045 L
.11018 .46777 L
.11078 .45872 L
.11112 .44686 L
.11143 .43146 L
.112 .39227 L
.11261 .33475 L
.11323 .29503 L
.11379 .35995 L
.11441 .41494 L
.11472 .43614 L
.11507 .4544 L
.11565 .47321 L
.11626 .47668 L
.11685 .46401 L
.11738 .43888 L
.1179 .40219 L
.11844 .34978 L
.11905 .27631 L
.1196 .34683 L
.12008 .39667 L
.1206 .43822 L
.12117 .4692 L
.12171 .48429 L
.12232 .4848 L
.12288 .46957 L
.12352 .4333 L
.12413 .38058 L
.12466 .32049 L
.12522 .30507 L
.12583 .37988 L
.12639 .43253 L
.127 .47164 L
.12734 .48553 L
.12765 .49314 L
.12826 .494 L
.12883 .47774 L
.12915 .46128 L
.1295 .43795 L
.13012 .38087 L
.13049 .33626 L
.13085 .28756 L
.13117 .30528 L
.13152 .35193 L
Mistroke
.13216 .42258 L
.13277 .46892 L
.13311 .48697 L
.13343 .49795 L
.13379 .50415 L
.13414 .5036 L
.13447 .4971 L
.13483 .48376 L
.13546 .4431 L
.13581 .41124 L
.13615 .3753 L
.13651 .32949 L
.1369 .27146 L
.13751 .35872 L
.13814 .43029 L
.13874 .47691 L
.13928 .50323 L
.13962 .51121 L
.13993 .51311 L
.14053 .50195 L
.14116 .46843 L
.14152 .43967 L
.14185 .40687 L
.14247 .32951 L
.14305 .30072 L
.14357 .37571 L
.14414 .43979 L
.14476 .48851 L
.14508 .50485 L
.14542 .51644 L
.14603 .52025 L
.14661 .50442 L
.14715 .47206 L
.14772 .41958 L
.14821 .35982 L
.14875 .2785 L
.14932 .35 L
.14994 .42838 L
.15052 .48187 L
.15105 .51319 L
.15168 .52886 L
.152 .52787 L
.15235 .51988 L
.15297 .48797 L
.15355 .43743 L
.15422 .35414 L
.15485 .28235 L
.1552 .34013 L
.15558 .39473 L
.15593 .43644 L
Mistroke
.15625 .46912 L
.1569 .5151 L
.15749 .53473 L
.15812 .53234 L
.15848 .52022 L
.1588 .50219 L
.15944 .44835 L
.16004 .37435 L
.16058 .28878 L
.16117 .34338 L
.16174 .42428 L
.16229 .48135 L
.16285 .52135 L
.16347 .54229 L
.16409 .53896 L
.16444 .52651 L
.16475 .50839 L
.16533 .45906 L
.16594 .38418 L
.16647 .29918 L
.16704 .33105 L
.16758 .41213 L
.16807 .46971 L
.16864 .51781 L
.16926 .54587 L
.16985 .54931 L
.17039 .53259 L
.17098 .49319 L
.1716 .4261 L
.17214 .34738 L
.17273 .28447 L
.17309 .34829 L
.17342 .39988 L
.17377 .44562 L
.17408 .48028 L
.17467 .52867 L
.17531 .55428 L
.17564 .55703 L
.17598 .5522 L
.1766 .52371 L
.17694 .49678 L
.17726 .46453 L
.17799 .36561 L
.17863 .27257 L
.17921 .37643 L
.17953 .42409 L
.17988 .46793 L
.18051 .52643 L
.18111 .55658 L
.18166 .5627 L
Mistroke
.18229 .54419 L
.1829 .5011 L
.18398 .36122 L
.18458 .27206 L
.18513 .37294 L
.18544 .42098 L
.18578 .46539 L
.18639 .52577 L
.187 .56003 L
.18756 .56864 L
.18818 .55278 L
.18853 .53197 L
.18886 .50479 L
.18951 .42718 L
.18986 .37478 L
.19022 .31002 L
.19055 .27378 L
.19091 .34356 L
.19154 .44424 L
.1919 .48821 L
.19229 .5255 L
.19265 .55102 L
.19298 .56615 L
.19359 .57333 L
.19394 .56565 L
.19426 .55086 L
.1946 .52715 L
.19491 .49841 L
.19561 .40893 L
.19624 .29778 L
.19682 .33612 L
.19746 .44093 L
.19778 .48311 L
.19813 .5199 L
.19872 .56165 L
.19927 .57815 L
.1996 .57748 L
.19992 .56929 L
.20052 .53406 L
.20113 .47121 L
.20147 .42385 L
.20179 .37269 L
.20237 .25945 L
.20298 .37581 L
.2035 .45593 L
.20405 .52021 L
.20466 .56503 L
.20522 .58257 L
.20586 .57386 L
.20623 .55536 L
Mistroke
.20657 .52953 L
.20718 .46028 L
.2075 .41458 L
.20784 .35578 L
.20843 .27822 L
.20907 .40014 L
.20968 .48736 L
.21023 .54347 L
.21075 .57569 L
.21132 .58729 L
.21185 .57662 L
.21233 .54873 L
.21289 .49366 L
.21351 .40683 L
.21409 .29886 L
.21462 .32635 L
.21517 .42527 L
.21569 .49692 L
.21625 .55234 L
.21686 .58479 L
.2175 .58873 L
.2181 .56404 L
.21865 .51827 L
.21924 .44296 L
.21985 .33694 L
.2205 .31097 L
.22083 .37685 L
.2212 .43902 L
.22154 .48697 L
.22185 .52292 L
.22247 .57259 L
.22281 .58757 L
.22313 .59369 L
.22369 .58522 L
.22431 .54795 L
.22491 .48394 L
.22547 .39901 L
.22605 .28471 L
.2267 .3612 L
.22721 .44839 L
.22775 .51926 L
.22835 .57077 L
.2289 .59398 L
.22926 .59636 L
.2296 .58954 L
.23023 .55316 L
.23083 .48923 L
.23118 .4407 L
.23149 .38807 L
.23204 .27773 L
Mistroke
.23264 .35947 L
.23326 .46485 L
.23386 .53671 L
.23419 .56519 L
.23455 .58636 L
.23519 .59878 L
.23554 .59191 L
.23593 .57359 L
.23625 .54896 L
.23661 .51216 L
.23725 .4219 L
.23759 .36175 L
.23794 .2889 L
.23825 .28852 L
.23859 .3591 L
.23918 .46116 L
.23983 .54036 L
.24043 .58442 L
.24074 .59638 L
.24108 .60076 L
.24144 .59551 L
.24177 .5815 L
.24239 .5328 L
.24295 .46228 L
.24327 .4114 L
.24361 .3485 L
.24422 .29162 L
.24481 .40815 L
.24534 .49015 L
.24597 .55821 L
.24656 .59368 L
.24712 .60176 L
.24763 .58757 L
.24819 .54814 L
.24879 .47945 L
.24943 .37424 L
.25001 .25572 L
.25033 .32716 L
.25068 .39546 L
.25132 .49476 L
.25192 .55865 L
.25255 .59579 L
.25309 .60253 L
.25368 .5833 L
.25402 .55979 L
.25438 .52526 L
.25502 .43849 L
.25535 .38058 L
.2557 .30917 L
.25609 .28484 L
Mistroke
.25645 .36077 L
.25679 .42224 L
.25715 .47868 L
.25747 .51974 L
.25781 .55427 L
.25846 .59467 L
.25905 .60291 L
.25964 .5832 L
.25998 .55964 L
.26029 .52983 L
.26086 .45519 L
.26147 .34746 L
.26201 .27792 L
.26252 .38321 L
.26308 .47512 L
.26368 .54663 L
.26429 .58943 L
.26463 .60092 L
.26494 .60332 L
.26559 .58371 L
.26595 .55844 L
.26627 .52655 L
.26686 .44819 L
.2675 .33234 L
.2681 .30756 L
.26865 .41532 L
.26917 .49372 L
.26973 .55451 L
.27025 .58954 L
.27073 .60259 L
.27129 .59484 L
.2719 .55812 L
.27248 .49634 L
.27301 .41548 L
.27362 .29711 L
.27418 .33544 L
.2747 .4299 L
.27525 .50889 L
.27585 .56771 L
.27649 .59864 L
.27703 .59971 L
.27763 .57404 L
.27827 .51541 L
.27886 .43253 L
.2792 .3734 L
.27951 .30988 L
.27984 .27058 L
.2802 .34781 L
.28077 .44876 L
.28137 .52737 L
Mistroke
.28168 .55725 L
.28203 .58103 L
.28263 .60032 L
.28323 .59123 L
.28377 .55863 L
.2844 .49219 L
.28499 .40121 L
.28554 .29242 L
.28605 .32675 L
.2866 .42917 L
.28719 .51134 L
.28783 .56989 L
.28841 .59583 L
.28873 .5989 L
.28908 .59311 L
.28971 .55819 L
.29031 .49726 L
.29094 .40237 L
.29153 .28446 L
.29207 .34075 L
.29272 .45341 L
.29304 .49782 L
.2934 .53705 L
.29379 .56847 L
.29415 .58729 L
.29448 .59569 L
.29484 .59524 L
.29517 .58585 L
.29554 .5658 L
.29588 .53747 L
.2962 .50318 L
.2968 .41695 L
.29744 .29503 L
.29803 .34095 L
.29867 .4519 L
.29928 .52748 L
.29984 .57228 L
.3004 .59255 L
.30091 .59026 L
.30149 .56319 L
.30204 .51372 L
.30264 .43308 L
.30328 .31771 L
.30383 .30987 L
.30442 .41974 L
.30507 .50767 L
.30543 .54325 L
.30575 .56709 L
.30639 .58997 L
.307 .58311 L
Mistroke
.30751 .55535 L
.30807 .50284 L
.3086 .43177 L
.30908 .34827 L
.30959 .27028 L
.31013 .37699 L
.31068 .4639 L
.31129 .5326 L
.3119 .57375 L
.31224 .58473 L
.31255 .5872 L
.31315 .57155 L
.31371 .53196 L
.31427 .47004 L
.3148 .38998 L
.3154 .27206 L
.31596 .35408 L
.31652 .44557 L
.31713 .51889 L
.3177 .56238 L
.31822 .5814 L
.31886 .57716 L
.31943 .54782 L
.31975 .5211 L
.32009 .48385 L
.32072 .39423 L
.32133 .27912 L
.32167 .30768 L
.32199 .368 L
.32265 .46666 L
.32301 .50844 L
.32334 .53806 L
.32394 .57098 L
.32458 .57769 L
.32519 .55626 L
.32576 .51251 L
.32625 .45594 L
.32676 .3778 L
.32732 .27021 L
.32784 .34823 L
.32841 .43863 L
.32903 .5111 L
.3296 .55389 L
.33014 .57237 L
.33075 .56856 L
.33132 .54093 L
.33183 .49629 L
.33239 .42679 L
.333 .32592 L
.33364 .3197 L
Mistroke
.33419 .41176 L
.33479 .4885 L
.33543 .54177 L
.33602 .56583 L
.33666 .56433 L
.33699 .55249 L
.33735 .53075 L
.33792 .47816 L
.33852 .39846 L
.33883 .34683 L
.33918 .28248 L
.33978 .35222 L
.34037 .4403 L
.34099 .50828 L
.34159 .54761 L
.34212 .5625 L
.34267 .5574 L
.34326 .52905 L
.34382 .48052 L
.34433 .417 L
.34492 .32238 L
.34555 .32026 L
.34615 .4148 L
.34669 .48026 L
.34733 .53136 L
.34769 .54819 L
.34803 .55599 L
.34864 .55081 L
.34899 .53697 L
.3493 .51763 L
.34963 .49011 L
.34998 .4532 L
.3506 .36699 L
.35093 .31205 L
.35127 .27845 L
.35166 .34572 L
.35201 .39988 L
.35234 .44251 L
.3527 .48083 L
.35304 .50935 L
.35335 .52909 L
.35394 .54904 L
.35457 .54535 L
.35513 .52049 L
.35573 .47152 L
.35629 .40413 L
.3568 .32511 L
.35738 .30694 L
.35793 .39227 L
.35852 .46311 L
Mistroke
.35916 .51486 L
.35947 .53063 L
.35981 .54092 L
.36017 .54378 L
.3605 .53917 L
.3611 .51344 L
.36174 .46086 L
.36237 .3835 L
.36298 .28682 L
.36352 .33659 L
.36411 .41848 L
.36471 .48051 L
.36529 .51802 L
.3658 .53452 L
.36636 .53416 L
.36696 .51216 L
.3676 .46459 L
.36815 .40423 L
.36875 .31725 L
.36932 .31162 L
.36983 .38649 L
.37042 .4533 L
.37098 .49721 L
.3715 .52116 L
.37198 .52914 L
.3725 .52231 L
.37306 .49727 L
.37362 .45371 L
.37423 .38514 L
.37487 .29026 L
.37546 .34063 L
.37601 .41058 L
.37659 .46705 L
.37715 .50205 L
.37766 .51861 L
.37827 .51876 L
.37892 .4958 L
.37946 .45814 L
.38006 .39753 L
.3807 .31059 L
.38105 .28583 L
.38138 .33476 L
.38198 .40904 L
.38262 .46614 L
.38316 .49713 L
.38376 .51244 L
.38431 .50917 L
.38482 .49123 L
.38542 .45193 L
.38597 .39858 L
Mistroke
.38658 .32104 L
.38722 .31743 L
.38774 .38376 L
.38829 .43863 L
.38882 .47528 L
.3893 .49588 L
.38986 .50467 L
.39047 .49542 L
.39104 .46881 L
.39158 .42892 L
.39219 .3648 L
.39277 .28595 L
.39329 .33328 L
.39386 .39825 L
.39448 .44998 L
.39482 .47046 L
.39513 .48383 L
.39574 .49571 L
.39632 .48987 L
.39686 .46929 L
.39743 .43178 L
.39792 .38696 L
.39845 .32431 L
.39903 .30422 L
.39964 .37708 L
.40023 .42885 L
.40076 .46172 L
.40138 .48305 L
.4017 .48666 L
.40205 .48508 L
.40266 .46812 L
.40324 .43582 L
.40357 .41023 L
.40392 .37774 L
.40454 .30552 L
.40489 .29331 L
.40521 .33294 L
.40557 .37061 L
.40595 .40498 L
.40659 .44776 L
.40719 .47088 L
.40778 .4774 L
.40832 .46952 L
.40891 .44605 L
.40956 .40243 L
.40988 .37315 L
.41023 .33676 L
.41086 .29626 L
.4115 .36572 L
.41208 .41305 L
Mistroke
.41242 .43341 L
.41277 .45009 L
.41342 .46635 L
.41403 .46521 L
.41434 .45854 L
.41468 .44656 L
.41528 .41339 L
.41584 .36883 L
.41644 .30756 L
.417 .31716 L
.4175 .36582 L
.41804 .40696 L
.41864 .43796 L
.41927 .45526 L
.4198 .45742 L
.42039 .44644 L
.42072 .43446 L
.42108 .41659 L
.42172 .37124 L
.42235 .31133 L
.42294 .31583 L
.4233 .34929 L
.42364 .37631 L
.42427 .41519 L
.42462 .43039 L
.42494 .4402 L
.4253 .44639 L
.42568 .44773 L
.42627 .43923 L
.4269 .41585 L
.4275 .38035 L
.42804 .33667 L
.42868 .29435 L
.42904 .32832 L
.42937 .35562 L
.42998 .39546 L
.43029 .41055 L
.43063 .42333 L
.43123 .43592 L
.43178 .43658 L
.43239 .42514 L
.43273 .41322 L
.43304 .39897 L
.43366 .36027 L
.43399 .33461 L
.43434 .30348 L
.43498 .32608 L
.43557 .36824 L
.43611 .39669 L
.43669 .41685 L
Mistroke
.43723 .42605 L
.43773 .42618 L
.43831 .41638 L
.43895 .39358 L
.43955 .36054 L
.4401 .32001 L
.44075 .31012 L
.44111 .33693 L
.44144 .3581 L
.44205 .38845 L
.4427 .40847 L
.44328 .41604 L
.44382 .41433 L
.44441 .40278 L
.44505 .37932 L
.4454 .3614 L
.44573 .34175 L
.44608 .31731 L
.44647 .29258 L
.4468 .31721 L
.44712 .33752 L
.44745 .35627 L
.44781 .3727 L
.44844 .39372 L
.44902 .40374 L
.44964 .40477 L
.44999 .40092 L
.45032 .39448 L
.45094 .37452 L
.45126 .3607 L
.45161 .34263 L
.45222 .30333 L
.4528 .31883 L
.45314 .33812 L
.45352 .35587 L
.45385 .36872 L
.4542 .37952 L
.45452 .38673 L
.45487 .39198 L
.45548 .39434 L
.45607 .38857 L
.45669 .37373 L
.45729 .35162 L
.45783 .32449 L
.45842 .29894 L
.45906 .33275 L
.45938 .34649 L
.45973 .35876 L
.46035 .3746 L
.46096 .38238 L
Mistroke
.46153 .38287 L
.46215 .37612 L
.46246 .36985 L
.46281 .36076 L
.46338 .34043 L
.464 .31154 L
.46459 .31096 L
.46512 .33341 L
.46568 .35112 L
.46619 .36272 L
.46675 .37016 L
.46735 .3721 L
.46792 .3683 L
.46846 .35975 L
.46894 .34804 L
.46946 .33115 L
.47007 .30604 L
.47063 .31374 L
.47127 .33456 L
.47187 .34864 L
.47243 .35686 L
.47295 .36047 L
.47352 .3601 L
.47412 .35485 L
.47472 .3447 L
.47506 .33688 L
.47537 .3284 L
.47595 .30954 L
.47655 .31181 L
.47687 .32073 L
.47721 .32889 L
.47783 .34009 L
.47821 .34477 L
.47857 .3477 L
.47889 .34912 L
.47923 .34939 L
.47987 .34647 L
.48047 .33983 L
.48082 .33437 L
.48113 .32834 L
.4815 .32007 L
.48184 .31124 L
.48217 .30314 L
.48252 .31122 L
.4829 .31877 L
.48326 .32469 L
.4836 .32926 L
.48396 .3331 L
.4846 .33711 L
.4852 .3379 L
Mistroke
.48554 .33711 L
.48586 .33559 L
.4865 .33027 L
.48686 .32602 L
.48718 .32139 L
.48777 .31134 L
.4884 .30919 L
.489 .31646 L
.48955 .32134 L
.49016 .3248 L
.49074 .3263 L
.49126 .32627 L
.49183 .32483 L
.49244 .32172 L
.49279 .31932 L
.4931 .31675 L
.49372 .31069 L
.49428 .30833 L
.49481 .31103 L
.49538 .31304 L
.49594 .31429 L
.49656 .31489 L
.4971 .31484 L
.49768 .31431 L
.49822 .31344 L
.49871 .3124 L
.50003 .30903 L
.50126 .30586 L
.50187 .30458 L
.50222 .30402 L
.50254 .30363 L
.50287 .30337 L
.50323 .30327 L
.50361 .3034 L
.50396 .30376 L
.50459 .30507 L
.50519 .30718 L
.50572 .3099 L
.50631 .30728 L
.50688 .3016 L
.5075 .29684 L
.50785 .29477 L
.50818 .29334 L
.50881 .29184 L
.50939 .29209 L
.51002 .29424 L
.51062 .29818 L
.51117 .30359 L
.51181 .31202 L
.51238 .30409 L
Mistroke
.51305 .29338 L
.51367 .28623 L
.51421 .28221 L
.51478 .28031 L
.51535 .28093 L
.51598 .28461 L
.51652 .29044 L
.51711 .2995 L
.51765 .31082 L
.51815 .3071 L
.51878 .29159 L
.51913 .28474 L
.51945 .27939 L
.52003 .27257 L
.52066 .269 L
.52099 .26883 L
.52134 .26996 L
.52197 .27539 L
.52232 .28036 L
.52264 .28625 L
.52299 .29408 L
.52338 .30424 L
.52402 .30981 L
.52463 .28969 L
.52522 .27465 L
.52577 .26487 L
.52636 .25877 L
.52669 .25743 L
.52701 .25759 L
.52764 .26214 L
.528 .2672 L
.52832 .27342 L
.52895 .29009 L
.52955 .31129 L
.53009 .30507 L
.53067 .28249 L
.5313 .26387 L
.53162 .25696 L
.53197 .2513 L
.53259 .24625 L
.53316 .24736 L
.53375 .2543 L
.5343 .26632 L
.53479 .28172 L
.53533 .30343 L
.53591 .31106 L
.53653 .2812 L
.53706 .26145 L
.53765 .24593 L
.53797 .24017 L
Mistroke
.53832 .23623 L
.53897 .23542 L
.53959 .24263 L
.54018 .2565 L
.54053 .26842 L
.54087 .28207 L
.54149 .31394 L
.54184 .31301 L
.54217 .29386 L
.54252 .27558 L
.54289 .25899 L
.54348 .23939 L
.54403 .22811 L
.54437 .22459 L
.54469 .22363 L
.54528 .22813 L
.54591 .24183 L
.54649 .26281 L
.54713 .29531 L
.54746 .3154 L
.54781 .31259 L
.54842 .27386 L
.54899 .24621 L
.54952 .228 L
.55008 .21629 L
.55039 .21323 L
.55074 .21284 L
.55136 .21995 L
.55197 .23667 L
.55252 .26074 L
.55305 .29126 L
.55355 .3267 L
.55409 .28861 L
.55467 .25261 L
.55528 .22513 L
.55586 .20877 L
.55638 .20223 L
.55695 .20398 L
.55728 .20928 L
.55764 .21857 L
.55828 .24492 L
.55891 .28244 L
.55949 .32735 L
.56011 .2813 L
.5608 .23719 L
.56115 .22047 L
.56152 .20671 L
.56187 .19766 L
.56219 .19278 L
.56281 .19249 L
Mistroke
.56338 .20317 L
.56401 .22709 L
.56433 .24405 L
.56468 .26658 L
.56527 .31327 L
.5659 .29289 L
.56649 .24693 L
.56704 .21532 L
.56767 .19144 L
.56826 .1813 L
.56879 .18249 L
.56937 .19487 L
.57 .22164 L
.57032 .2404 L
.57066 .26512 L
.57128 .31942 L
.57185 .2913 L
.57238 .24653 L
.57296 .20942 L
.57351 .18559 L
.57401 .17331 L
.5746 .17094 L
.57522 .18255 L
.57575 .20367 L
.57633 .23917 L
.57697 .29316 L
.57729 .32625 L
.57764 .30749 L
.57826 .24804 L
.57884 .20633 L
.5792 .18745 L
.57953 .17413 L
.58016 .16109 L
.58051 .1606 L
.58083 .1645 L
.58118 .17362 L
.58156 .18881 L
.58214 .2234 L
.58269 .2689 L
.58332 .33673 L
.5839 .2743 L
.58453 .21785 L
.58489 .19341 L
.58523 .1756 L
.58583 .1554 L
.58647 .15087 L
.58708 .16288 L
.58743 .17679 L
.58775 .19443 L
.5884 .24379 L
Mistroke
.58874 .2763 L
.5891 .31721 L
.58944 .32069 L
.58975 .28338 L
.59036 .22244 L
.59092 .18163 L
.59152 .15357 L
.59216 .14101 L
.59252 .14218 L
.59285 .14846 L
.59346 .17319 L
.59411 .2177 L
.59477 .28325 L
.59514 .32875 L
.59547 .31031 L
.59587 .26236 L
.59625 .22391 L
.59662 .19239 L
.59697 .1688 L
.59754 .14249 L
.59817 .13146 L
.59878 .13882 L
.59912 .15083 L
.59944 .16676 L
.60001 .20805 L
.60062 .26942 L
.60119 .3439 L
.60171 .27365 L
.60236 .20442 L
.60297 .15842 L
.60332 .14068 L
.60363 .12962 L
.60398 .12324 L
.60435 .12307 L
.60496 .13809 L
.60554 .16948 L
.60617 .2233 L
.60652 .26177 L
.60685 .30348 L
.60745 .30055 L
.60777 .25878 L
.60811 .21969 L
.60847 .18536 L
.60881 .15889 L
.60945 .12552 L
.61004 .11369 L
.6106 .11936 L
.6112 .14415 L
.61185 .19206 L
.61255 .26955 L
Mistroke
.61295 .3249 L
.61332 .31299 L
.61364 .2677 L
.61399 .22433 L
.61436 .18578 L
.6147 .15714 L
.61534 .11986 L
.6157 .10901 L
.61603 .10508 L
.61663 .1134 L
.61697 .12679 L
.61728 .14479 L
.61781 .18792 L
.61837 .25038 L
.61899 .33902 L
.61955 .27111 L
.62017 .19516 L
.62052 .16167 L
.62085 .13673 L
.62143 .10698 L
.62206 .09688 L
.6224 .10083 L
.62272 .1104 L
.62306 .12745 L
.62343 .15301 L
.62404 .21337 L
.6244 .25798 L
.62472 .3051 L
.62532 .2967 L
.62597 .20794 L
.62657 .14757 L
.62714 .10979 L
.62775 .09053 L
.6281 .0894 L
.62841 .09455 L
.629 .11956 L
.62961 .16785 L
.63016 .23044 L
.63068 .30527 L
.63124 .30208 L
.63185 .21422 L
.63249 .14487 L
.63286 .11688 L
.63319 .0982 L
.63381 .08194 L
.63412 .08261 L
.63447 .09028 L
.63506 .12058 L
.6357 .17747 L
.6363 .25433 L
Mistroke
.63686 .34454 L
.63738 .27113 L
.63794 .19283 L
.63847 .13701 L
.63896 .10158 L
.63952 .0785 L
.64014 .07631 L
.64071 .0961 L
.64125 .13315 L
.64181 .19145 L
.64241 .27623 L
.64298 .33233 L
.64349 .24535 L
.64407 .16827 L
.64462 .11522 L
.64522 .07992 L
.64556 .07011 L
.64587 .06757 L
.64644 .07942 L
.64705 .11518 L
.64756 .16464 L
.64813 .23888 L
.64874 .34259 L
.64908 .30444 L
.64939 .25178 L
.65001 .16685 L
.65057 .11048 L
.6511 .07691 L
.65166 .06166 L
.65222 .06744 L
.65284 .0983 L
.65338 .14558 L
.65395 .21783 L
.65446 .29928 L
.65499 .31219 L
.65534 .25077 L
.65566 .20231 L
.65628 .12749 L
.65663 .09701 L
.65701 .07335 L
.65737 .05989 L
.65769 .05515 L
.65829 .06529 L
.65863 .08183 L
.65894 .10432 L
.65958 .1705 L
.65993 .21979 L
.66026 .27237 L
.66087 .32586 L
.66144 .22554 L
Mistroke
.66207 .14131 L
.66239 .10908 L
.66273 .08155 L
.66332 .05441 L
.66394 .05159 L
.66453 .07415 L
.66508 .1164 L
.6657 .1908 L
.66629 .28631 L
.66682 .32681 L
.6674 .22344 L
.66803 .13739 L
.66834 .10456 L
.66869 .07665 L
.6693 .0483 L
.66988 .04614 L
.67046 .06769 L
.67101 .1102 L
.6715 .16666 L
.67203 .24797 L
.67262 .35948 L
.67323 .24136 L
.67382 .15387 L
.67435 .09591 L
.67498 .05294 L
.67531 .04242 L
.67566 .03946 L
.67628 .05622 L
.67686 .0972 L
.67721 .13406 L
.67755 .17726 L
.67817 .28 L
.67852 .34993 L
.67884 .30449 L
.67957 .17875 L
.68016 .1062 L
.68071 .06105 L
.68131 .03704 L
.68165 .03527 L
.68196 .04105 L
.68255 .07163 L
.68319 .13438 L
.68352 .17775 L
.68386 .23265 L
.68449 .35396 L
.68509 .24729 L
.68576 .14412 L
.6861 .10468 L
.68646 .07171 L
.6868 .04948 L
Mistroke
.68711 .03673 L
.6877 .03238 L
.68835 .05754 L
.68895 .10995 L
.68952 .18333 L
.69012 .28828 L
.69046 .35937 L
.69077 .29914 L
.69112 .23323 L
.69149 .17366 L
.69184 .12605 L
.69216 .09086 L
.69249 .06224 L
.69285 .04104 L
.6935 .02741 L
.69386 .03359 L
.69425 .0515 L
.69462 .07915 L
.69496 .11391 L
.69553 .19134 L
.69614 .30197 L
.69672 .30007 L
.69725 .20244 L
.69786 .11521 L
.69821 .07871 L
.69854 .05316 L
.69913 .02714 L
.69976 .02901 L
.7001 .04288 L
.70042 .06363 L
.70077 .09524 L
.70113 .13864 L
.7018 .24447 L
.70243 .3535 L
.70298 .23957 L
.70358 .14396 L
.70392 .10171 L
.70424 .06991 L
.70484 .03183 L
.7054 .02157 L
.706 .03758 L
.70657 .07847 L
.70709 .13793 L
.70767 .22855 L
.7083 .35727 L
.70862 .30067 L
.70896 .2339 L
.70958 .13641 L
.70991 .09514 L
.71023 .065 L
Mistroke
.71057 .04065 L
.71093 .02484 L
.71154 .02131 L
.71189 .0322 L
.71221 .05083 L
.71281 .10734 L
.71336 .18375 L
.714 .3014 L
.7146 .29471 L
.71517 .1886 L
.71569 .11452 L
.71628 .05494 L
.71684 .02428 L
.71737 .01785 L
.71784 .03068 L
.71837 .06542 L
.71892 .12549 L
.71948 .21104 L
.72008 .33001 L
.72065 .27479 L
.72117 .18112 L
.72177 .09785 L
.72212 .06362 L
.72243 .0402 L
.72301 .01769 L
.72363 .02279 L
.72422 .05569 L
.72486 .12292 L
.72547 .21644 L
.72603 .3294 L
.72659 .27699 L
.7271 .18406 L
.72768 .10201 L
.72824 .04944 L
.72884 .01943 L
.72948 .01894 L
.73002 .04392 L
.73062 .09829 L
.73126 .18835 L
.73161 .25189 L
.73195 .32125 L
.73226 .33805 L
.7326 .2659 L
.73319 .16151 L
.73375 .08929 L
.73427 .04352 L
.73484 .01789 L
.73545 .01906 L
.73606 .04973 L
.7367 .11381 L
Mistroke
.73725 .19431 L
.73785 .30961 L
.73841 .29436 L
.73893 .19672 L
.73952 .10999 L
.74008 .05364 L
.7406 .02358 L
.74108 .01472 L
.7416 .0258 L
.74216 .06129 L
.74275 .12605 L
.74339 .22831 L
.744 .35522 L
.74457 .25313 L
.74512 .15831 L
.74564 .09098 L
.74624 .0397 L
.74678 .01758 L
.74738 .02023 L
.74802 .0544 L
.74857 .10907 L
.74916 .19598 L
.74981 .32126 L
.75017 .32687 L
.7505 .25832 L
.75114 .14947 L
.75174 .07617 L
.75226 .03646 L
.75282 .01697 L
.75342 .02379 L
.75397 .05541 L
.75449 .10685 L
.75497 .17256 L
.75549 .2644 L
.75604 .34493 L
.7566 .22951 L
.75713 .14466 L
.75775 .07183 L
.75832 .03213 L
.75891 .01745 L
.75955 .03251 L
.76015 .07626 L
.76071 .14191 L
.76133 .24281 L
.76191 .36343 L
.76243 .25256 L
.76301 .1563 L
.76363 .08063 L
.76394 .05381 L
.76428 .03307 L
Mistroke
.7649 .01925 L
.76547 .03276 L
.76601 .06916 L
.76658 .13238 L
.76707 .20652 L
.76761 .30899 L
.76819 .29273 L
.7688 .18086 L
.76939 .10243 L
.76992 .05396 L
.77055 .02467 L
.77086 .02154 L
.77121 .02712 L
.77183 .0601 L
.77241 .11776 L
.77276 .16505 L
.77309 .21871 L
.77371 .34236 L
.77406 .30847 L
.77439 .245 L
.77474 .185 L
.77512 .1308 L
.77576 .06444 L
.77637 .03094 L
.77695 .0253 L
.7775 .0436 L
.77808 .08887 L
.77873 .16896 L
.77906 .22188 L
.77941 .28708 L
.78003 .30503 L
.78062 .19782 L
.78126 .10976 L
.78159 .07678 L
.78195 .05063 L
.78229 .03487 L
.7826 .02804 L
.78317 .03435 L
.78379 .06903 L
.78443 .13616 L
.78503 .22691 L
.78563 .34342 L
.78619 .26516 L
.7867 .17931 L
.78724 .10973 L
.78783 .05877 L
.78837 .03476 L
.78895 .03348 L
.78959 .06143 L
.78992 .08756 L
Mistroke
.79028 .12514 L
.79093 .2179 L
.79156 .33838 L
.79215 .26507 L
.79247 .20833 L
.79283 .15608 L
.79347 .08524 L
.79386 .05742 L
.79421 .0416 L
.79454 .03527 L
.79489 .03713 L
.79553 .06371 L
.79589 .09197 L
.79622 .12656 L
.79683 .21127 L
.79714 .26512 L
.79748 .33106 L
.79806 .27303 L
.7986 .18356 L
.79925 .10516 L
.79984 .06003 L
.80043 .0403 L
.80098 .04474 L
.80158 .07479 L
.80223 .13583 L
.80284 .22179 L
.80342 .32759 L
.80393 .28764 L
.8045 .19412 L
.8051 .11898 L
.80544 .08784 L
.80576 .06628 L
.8064 .04491 L
.80676 .04585 L
.80709 .0548 L
.80768 .08991 L
.80832 .15595 L
.80892 .24491 L
.80948 .35022 L
.80999 .26879 L
.81055 .18234 L
.81107 .12053 L
.81155 .08071 L
.81208 .05542 L
.81266 .0504 L
.81327 .07038 L
.81384 .11206 L
.81439 .17422 L
.81499 .26554 L
.81555 .34243 L
Mistroke
.81608 .24807 L
.81668 .1617 L
.81733 .098 L
.81787 .06649 L
.81847 .05509 L
.8191 .06958 L
.81969 .10742 L
.82033 .17592 L
.82069 .22647 L
.82102 .27984 L
.82162 .32039 L
.82196 .26119 L
.82227 .21355 L
.82284 .14281 L
.82344 .09191 L
.82398 .06697 L
.82456 .06198 L
.82511 .07786 L
.82561 .10967 L
.8262 .16922 L
.82676 .2464 L
.82734 .3488 L
.82799 .25013 L
.82862 .16575 L
.82897 .12974 L
.8293 .10395 L
.82989 .07467 L
.83051 .06829 L
.83083 .07501 L
.83118 .08987 L
.83181 .1366 L
.8322 .17783 L
.83256 .2255 L
.83289 .27514 L
.83324 .33747 L
.83385 .26598 L
.83449 .17921 L
.83508 .12204 L
.83563 .08872 L
.83597 .07799 L
.83628 .07435 L
.83688 .08468 L
.83752 .12074 L
.83788 .15255 L
.83822 .18935 L
.83879 .2686 L
.8394 .33332 L
.83991 .25043 L
.84048 .17862 L
.84109 .12333 L
Mistroke
.84143 .1024 L
.84174 .08961 L
.84235 .08174 L
.84292 .09492 L
.84346 .1253 L
.84402 .17652 L
.84451 .23569 L
.84504 .31726 L
.84561 .28965 L
.84623 .20566 L
.84681 .14719 L
.84734 .11137 L
.84796 .09082 L
.84831 .08929 L
.84862 .09405 L
.84924 .12009 L
.84981 .16483 L
.85014 .19899 L
.85049 .24192 L
.85111 .33635 L
.85145 .30722 L
.85178 .25995 L
.85213 .21533 L
.85251 .17491 L
.85315 .12625 L
.85375 .10166 L
.85434 .09774 L
.85488 .11165 L
.85547 .14545 L
.85612 .20532 L
.85644 .24471 L
.85679 .29347 L
.85743 .30464 L
.85803 .22502 L
.85858 .17022 L
.85922 .12688 L
.85983 .10718 L
.86047 .10843 L
.86105 .12902 L
.86169 .173 L
.86201 .20363 L
.86237 .24385 L
.86296 .32568 L
.8635 .28805 L
.86412 .21357 L
.86471 .16198 L
.86525 .13086 L
.86576 .11599 L
.86631 .11511 L
.8669 .13185 L
Mistroke
.86724 .14957 L
.86759 .17467 L
.86823 .23597 L
.86856 .27611 L
.86891 .32532 L
.8693 .30891 L
.86966 .26235 L
.87 .22523 L
.87036 .1914 L
.87068 .16717 L
.87102 .14717 L
.87167 .12538 L
.87226 .12381 L
.87284 .13911 L
.87345 .17395 L
.87404 .22423 L
.87457 .28502 L
.87516 .32093 L
.87581 .24329 L
.87614 .21184 L
.87649 .18402 L
.87712 .1488 L
.87772 .13283 L
.87807 .13162 L
.87839 .1356 L
.87873 .14515 L
.87904 .15864 L
.8794 .17985 L
.87974 .20554 L
.88034 .26419 L
.88089 .33349 L
.8815 .27457 L
.88215 .21242 L
.88279 .16989 L
.88315 .15426 L
.88348 .14514 L
.88409 .14139 L
.88473 .15566 L
.88508 .1708 L
.88544 .19275 L
.8861 .24733 L
.8865 .29016 L
.88688 .33719 L
.88722 .30174 L
.88759 .26227 L
.88821 .21007 L
.88878 .17598 L
.88941 .15495 L
.88976 .15061 L
.89009 .15132 L
Mistroke
.89042 .15678 L
.89073 .16614 L
.89108 .18115 L
.89144 .20257 L
.89212 .25699 L
.89273 .32273 L
.89334 .28499 L
.8939 .23478 L
.89452 .19453 L
.89483 .18013 L
.89517 .16882 L
.89574 .16034 L
.89636 .16591 L
.89694 .18499 L
.89747 .21397 L
.89805 .25824 L
.89859 .31165 L
.89918 .29701 L
.89983 .24187 L
.90046 .2027 L
.90078 .18882 L
.90113 .17806 L
.90175 .17031 L
.90233 .17599 L
.90292 .1944 L
.90347 .22336 L
.90396 .25889 L
.9045 .30749 L
.90509 .3023 L
.90571 .25155 L
.90624 .21906 L
.90682 .19478 L
.90715 .1864 L
.9075 .18147 L
.90815 .18365 L
.90878 .19952 L
.90936 .22621 L
.90972 .24824 L
.91005 .27285 L
.91068 .32889 L
.91103 .30406 L
.91135 .27807 L
.9117 .25365 L
.91208 .232 L
.91267 .20731 L
.91322 .19443 L
.91356 .19133 L
.91388 .19172 L
.91447 .20104 L
.9151 .22296 L
Mistroke
.91568 .2541 L
.91632 .30023 L
.91664 .32821 L
.91699 .30349 L
.91758 .26294 L
.91821 .23109 L
.91857 .21804 L
.9189 .20953 L
.91954 .2019 L
.92014 .20547 L
.92078 .22085 L
.92139 .24626 L
.92196 .27888 L
.9225 .3188 L
.92301 .29985 L
.92361 .26349 L
.92416 .2389 L
.92474 .2218 L
.92529 .21373 L
.92587 .2137 L
.9265 .22346 L
.92713 .2434 L
.92772 .27125 L
.92804 .28972 L
.92837 .31185 L
.92874 .31523 L
.92907 .2944 L
.9294 .2767 L
.92975 .26063 L
.93038 .23903 L
.93076 .23044 L
.93113 .22551 L
.93145 .22369 L
.93181 .2245 L
.9324 .23223 L
.93273 .24014 L
.93305 .24983 L
.93368 .27604 L
.93401 .29295 L
.93436 .31401 L
.935 .29779 L
.9356 .27084 L
.93618 .25185 L
.93672 .24054 L
.93735 .23491 L
.93767 .23512 L
.93802 .23766 L
.93864 .24803 L
.93921 .26439 L
.93956 .27742 L
Mistroke
.9399 .29189 L
.94052 .32019 L
.94086 .30338 L
.94118 .2898 L
.94153 .27713 L
.94191 .26593 L
.94249 .25356 L
.94303 .24732 L
.94335 .24604 L
.94368 .24654 L
.94428 .25203 L
.9449 .26405 L
.94548 .28086 L
.94611 .30545 L
.94647 .31843 L
.94679 .30545 L
.94736 .28639 L
.94796 .27151 L
.94848 .26303 L
.94904 .25811 L
.94965 .25785 L
.94999 .25995 L
.9503 .26325 L
.95091 .27337 L
.95148 .28742 L
.95201 .30426 L
.95258 .31128 L
.95307 .29737 L
.9536 .28527 L
.95417 .27604 L
.95479 .27029 L
.95537 .26876 L
.95591 .2706 L
.96038 .2833 L
.96095 .2806 L
.96156 .28057 L
.96187 .28169 L
.96222 .28378 L
.96284 .2896 L
.96342 .29748 L
.96404 .30844 L
.96462 .30852 L
.96516 .30202 L
.96549 .29888 L
.96583 .29619 L
.96622 .294 L
.96657 .29263 L
.96693 .29189 L
.96727 .29177 L
.96791 .29297 L
Mistroke
.96853 .29586 L
.96911 .29991 L
.96943 .30273 L
.96978 .30617 L
.97041 .31024 L
.971 .30719 L
.97154 .30523 L
.97218 .30383 L
.97278 .3033 L
.97341 .30342 L
.97399 .30405 L
.97462 .30517 L
.97529 .30673 L
.97619 .3091 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->Automatic,
  AspectRatioFixed->True,
  ImageSize->{163, 100.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003;0000OA000`40O003h00OIWoo00<007ooOol0
HWoo001VOol00`00Oomoo`1ROol006Ioo`8006=oo`00IWoo00<007ooOol0;Goo00<007ooOol0<Woo
001VOol00`00Oomoo`0YOol500000goo00000002000_Ool006Ioo`03001oogoo02Ioo`h00003Ool0
07oo02]oo`00IWoo0P009Goo400000=oo`000000:goo001VOol00`00Oomoo`0ROolB00000goo0000
0002000YOol006Ioo`03001oogoo021oo`03001oo`0001400003Ool0000000<002Qoo`00IWoo00<0
07ooOol087oo00<007oo00004@0000=oo`0000000`0000=oo`00Ool09Goo001VOol00`00Oomoo`0O
Ool200000goo0000000F00000goo0000000UOol006Ioo`8001iooad00004Ool00000000TOol006Io
o`03001oogoo01eoob4002Aoo`00=goo0`001Woo0P0017oo0`001Goo0`001Goo0`001Goo1@000goo
00<007ooOol077oo8`008goo000fOol01@00Oomoogoo00001Goo0P000goo00D007ooOomoo`0000=o
o`05001oogooOol00003Ool01@00Oomoogoo00001Woo00<007ooOol00goo00<007ooOol06Woo00<0
07oo00008`008Woo000^Ool60002Ool01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05001o
ogooOol00003Ool01@00Oomoogoo00001Woo00<007ooOol00goo00<007ooOol06Woo9P0000=oo`00
Ool07goo000fOol01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool0
1@00Oomoogoo00001Woo00<007ooOol00goo0P006Woo:@0087oo000fOol01@00Oomoogoo00002Woo
00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00001Woo00<007ooOol00goo
00<007ooOol067oo:`007goo000fOol01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05001o
ogooOol00003Ool01@00Oomoogoo00001Woo00<007ooOol00goo00<007ooOol05goo;@007Woo000f
Ool01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo
00001Woo00<007ooOol00goo00<007ooOol05goo;@007Woo000gOol3000<Ool30005Ool30005Ool3
0005Ool30005Ool2000GOol_000MOol006Ioo`03001oogoo01Aoo`03001oo`0002h001eoo`00IWoo
00<007ooOol057oo00<007oo0000;`0077oo001VOol00`00Oomoo`0COolc00000goo001oo`0IOol0
06Ioo`03001oogoo01=oocD001Yoo`00IWoo0P004goo=`006Goo001VOol00`00Oomoo`0BOolg000I
Ool006Ioo`03001oogoo015oocT001Qoo`00IWoo00<007ooOol04Goo>@0067oo001VOol00`00Oomo
o`0@Oolk000GOol006Ioo`80015ooc`001Ioo`00IWoo00<007ooOol03Woo?P005Woo001VOol00`00
Oomoo`0=Oom0000EOol006Ioo`03001oogoo00eood0001Eoo`00IWoo0P003Woo@P004goo001VOol0
0`00Oomoo`0<Oom3000COol002moo`<000Ioo`8000Aoo`<000Eoo`<000Eoo`<000Eoo`<000Eoo`@0
00Aoo`03001oogoo00aood<001=oo`00;Woo00D007ooOomoo`0000Eoo`8000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00008Ool01@00Oomo
ogoo00003GooA@004Woo000VOol60002Ool01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05
001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Qoo`05001oogooOol0000=
Oom5000BOol002ioo`05001oogooOol0000:Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=o
o`05001oogooOol00003Ool01@00Oomoogoo000027oo00@007ooOomoo`8000]oodL0015oo`00;Woo
00D007ooOomoo`0000Yoo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`00
00=oo`05001oogooOol00004Ool40004Ool00`00Oomoo`0:Oom7000AOol002ioo`05001oogooOol0
000:Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomo
ogoo000017oo00<007ooOol01Goo00<007ooOol02GooB@0047oo000^Ool01@00Oomoogoo00002Woo
00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`00
00Aoo`03001oogoo00Eoo`03001oogoo00QoodX0011oo`00;goo0`0037oo0`001Goo0`001Goo0`00
1Goo0`001Goo100017oo00<007ooOol01gooC0003goo001VOol20008Oom<000?Ool006Ioo`03001o
ogoo00Ioodl000eoo`00IWoo00<007ooOol01WooC`003Goo001VOol00`00Oomoo`05OomA000<Ool0
06Ioo`8000Iooe4000aoo`00IWoo00<007ooOol01GooD@0037oo001VOol00`00Oomoo`04OomC000;
Ool000Qoo`D002Moo`03001oogoo009oo`D002Qoo`03001oogoo00Aooe<000=oo`D000=oo`002Goo
00@007ooOol001ioo`03001oogoo00Ioo`03001oogoo00Aoo`03001oogoo02Qoo`03001oogoo00=o
oeD000=oo`04001oogoo0003Ool000H000Aoo`03001oogoo01ioo`03001oogoo009oo`H000Eoo`03
001oogoo02Qoo`8000=ooaH00003Ool0000003d000Aoo`03001oogoo00=oo`002goo00<007ooOol0
7Goo00<007ooOol01Woo00<007ooOol00goo0P0017oo00D007ooOomoo`00025oo`04001oogooOolG
00000goo0000000n0004Ool00`00Oomoo`02Ool000aoo`03001oogoo00moo`03001oogoo00Eoo`<0
009oo`03001oogoo00Ioo`03001oogoo00=oo`8000Aoo`80009oo`03001oogoo01moo`04001oogoo
OolG00000goo0000000n0005Ool00`00Oomoo`01Ool000aoo`03001oogoo00]oo`07001oogooOol0
07oo00001Goo0`000Woo0P0017oo00@007ooOol0009oo`03001oogoo008000Aoo`800003Ool00000
00=oo`03001oogoo01]oo`03001oogoo01P00003Ool0000000<00003Ool0000003X000=oo`03001o
ogoo005oo`0027oo00D007ooOomoo`0000eoo`06001oogooOol007oo0P000goo10000Woo0P000Woo
00<007oo00000Woo0P0000Aoo`00Oomoo`800004Ool007ooOol200001goo000007oo001oo`0001eo
o`03001oogoo01P00003Ool0000000<00003Ool0000001H00003Ool00000024000=oo`03001oogoo
005oo`002Goo0`002Goo00/007ooOol007oo001oo`00Ool007oo00<0009oo`@0009oo`80009oo`05
001oo`00001oo`0200000goo001oo`03000017oo001oogoo0P0000Moo`00001oo`00Ool00002Ool0
1000Ool000000Woo00<007ooOol04Woo0P0000=oo`0000005P0000=oo`0000000`0000=oo`000000
400000=oo`0000000`0000=oo`0000007`0000=oo`0000000P0017oo000EOol00`00Oomoo`030000
1Goo001oo`00Ool00`000Woo10000Woo0P000Woo00<007oo0000100000Aoo`00Oomoo`800004Ool0
0000Ool200001goo000007oo001oo`00009oo`04001oo`000002Ool00`00Ool00004Ool01@00Oomo
ogoo00002Goo00<007oo00002`0000=oo`0000002P0000=oo`0000000`0000=oo`000000400000=o
o`0000000`0000=oo`0000005P0000=oo`0000001P0000=oo`0000001Woo000>Ool00`00Oomoo`04
Ool00`00Ool0000300000goo001oo`0500000goo0000000400001goo00000000Ool007oo00D00003
Ool007oo00<00004Ool00000Ool200001goo000007oo001oo`00009oo`04001oo`000002Ool01000
Ool000000goo00L007ooOomoo`00Ool00007Ool00`00Ool0000;00000goo0000000:00000goo0000
000300000goo0000000@00000goo0000000300000goo0000000F00000goo0000000600000goo0000
0006Ool000]oo`800006Ool00000001oo`000Woo00<007oo00001@0000=oo`0000000`0000=oo`00
0000100000Moo`00000007oo001oo`0500000goo001oo`03000017oo000007oo0P0000Moo`00001o
o`00Ool00002Ool01000Ool000000Woo10000goo0P000Woo0`0017oo00<007ooOol0200000=oo`00
00000`0000=oo`0000002P0000=oo`0000000`0000=oo`000000100000=oo`0000002@0000=oo`00
00000`0000=oo`0000000`0000=oo`000000400000=oo`0000001P0000Aoo`00000000Eoo`000Woo
b@000007Ool=00000goo001oo`0600000goo0000000300000goo0000000400000goo000000030000
0goo0000000300000goo0000000300000goo0000000300001Woo000007oo001oo`P00003Ool00000
00800003Ool0000000l0009oo`800004Ool00000Ool500000goo000000040002Ool20003Ool50000
0goo000000030002Ool2000017oo000007oo1@0000Moo`00Ool007oo001oo`0500000goo00000005
00000goo0000000200001goo00000000Ool007oo00D00003Ool0000000=oo`03001oogoo00=oo`00
27oo300000=oo`0000001P0000=oo`0000000`0000=oo`000000100000=oo`0000000`0000=oo`00
00000`0000=oo`0000000`0000=oo`0000000`0000=oo`000000400000=oo`0000003`002Woo0P00
0Woo1P000Woo0P000goo0`0000=oo`00Ool01@000Woo0P0000Ioo`00001oo`00Ool300001goo001o
o`00Ool007oo00D00003Ool0000000D00003Ool0000000800003Ool00000009oo`04001oo`000002
Ool00`00Ool0000:Ool000Uoo`/00003Ool0000000H00003Ool0000000<00003Ool0000000X00003
Ool0000000<00003Ool0000000<00003Ool0000000<00003Ool0000001000003Ool0000000l000Yo
o`8000=oo`800003Ool00000009oo`8000=oo`<00003Ool007oo00D0009oo`800006Ool00000Ool0
07oo0`0000Moo`00Ool007oo001oo`050002Ool3000017oo000007oo100000=oo`0000000Woo00<0
07oo000047oo0009Ool;00000goo0000000600000goo0000000300000goo0000000F00000goo0000
000300000goo0000000@00000goo0000000=00000goo001oo`09Ool20003Ool200000goo00000003
Ool01000Oomoogoo0`0000=oo`00Ool01@000Woo0P0000Ioo`00001oo`00Ool300001goo001oo`00
Ool007oo00D0009oo`<00003Ool007oo00Aoo`03001oo`0000=oo`03001oo`00011oo`002Woo2P00
00=oo`000000300000=oo`0000005P0000=oo`0000000`0000=oo`0000007`000Woo0P003goo00@0
07ooOol000=oo`04001oogooOol30004Ool40002Ool20004Ool03000Ool007oo001oo`00Ool007oo
001oo`800003Ool00000009oo`<00003Ool007oo00Aoo`03001oo`0001Ioo`002Woo6@0000=oo`00
00005P0000=oo`0000009@000Woo00<007ooOol03Woo00@007ooOol000=oo`07001oogooOol007oo
000017oo10000Woo0P0017oo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool20002Ool20002
Ool00`00Oomoo`05Ool00`00Oomoo`0DOol000]ooc400003Ool0000002@000=oo`03001oogoo015o
o`03001oogoo00Eoo`03001oo`0000Aoo`@0009oo`8000Ioo`09001oo`00Ool007oo001oo`0000Ao
o`80009oo`03001oogoo021oo`002gooF0000goo00<007ooOol087oo10000Woo0P001Woo00D007oo
Oomoo`0000Qoo`05001oogooOol0000ROol000aooeH000Aoo`80025oo`04001oo`000002Ool20006
Ool01@00Oomoogoo0000;goo000<OomF0004Ool00`00Oomoo`0POol01000Oomoo`000goo00<007oo
Ool027oo00<007ooOol0;Goo000=OomD0005Ool00`00Oomoo`0bOol00`00Oomoo`0]Ool000eooe@0
00Eoo`03001oogoo069oo`003WooD`001Goo00<007ooOol0HWoo000?OomA0006Ool2001SOol000mo
o`D00003Ool0000004T000Ioo`03001oogoo069oo`0047oo100000=oo`000000A`0027oo00<007oo
Ool0HWoo000@Ool400000goo000000170008Ool00`00Oomoo`1ROol0015oo`<00003Ool0000004H0
00Uoo`03001oogoo069oo`004Goo0`0000=oo`000000AP002Goo0P00Hgoo000BOol20002Oom6000:
Ool00`00Oomoo`1ROol0019oo`04001oogooOom60002Ool40004Ool00`00Oomoo`1ROol001IoodH0
00Ioo`05001oogooOol0001TOol001IoodD000Moo`04001oogooOol2001SOol001IoodD000Moo`05
001oogooOol0001TOol001Iood@000Aoo`@000Aoo`03001oogoo069oo`005WooA00017oo00<007oo
Ool01Goo00<007ooOol0HWoo000FOom200000goo001oo`03Ool00`00Oomoo`05Ool00`00Oomoo`1R
Ool001Iood<000Eoo`@000Aoo`8006=oo`005goo@0003goo00<007ooOol0HWoo000GOom0000?Ool0
0`00Oomoo`1ROol001Qooch0011oo`03001oogoo069oo`0067oo?P0047oo00<007ooOol0HWoo000I
Ooll000AOol2001SOol001Uoo`03001oo`0003T0015oo`03001oogoo069oo`006goo>@004Woo00<0
07ooOol0HWoo000LOolh000BOol00`00Oomoo`1ROol001aoocH001Aoo`8006=oo`007Goo=0005Goo
00<007ooOol0HWoo000MOold000EOol00`00Oomoo`1ROol001iooc8001Ioo`03001oogoo069oo`00
7Woo<P005Woo00<007ooOol0HWoo000OOol`000GOol2001SOol0025oobd001Qoo`03001oogoo069o
o`008Goo<0001Goo0`001Goo1@000goo00<007ooOol0HWoo000ROol/0003Ool01@00Oomoogoo0000
0goo00<007ooOol017oo00<007ooOol00goo00<007ooOol0HWoo000SOolX0002Ool01@00Oomoogoo
00000goo00D007ooOomoo`0000Ioo`03001oogoo00=oo`03001oogoo069oo`008goo:0000Woo00D0
07ooOomoo`0000=oo`05001oogooOol00006Ool00`00Oomoo`03Ool2001SOol002AoobL0009oo`05
001oogooOol00003Ool01@00Oomoogoo00001Woo00<007ooOol00goo00<007ooOol0HWoo000UOolT
00001Goo001oogoo00000goo00D007ooOomoo`0000=oo`03001oogoo00Aoo`03001oogoo00=oo`03
001oogoo069oo`009Goo900000Eoo`00Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04
Ool00`00Oomoo`03Ool00`00Oomoo`1ROol002Moob<000Aoo`<000Eoo`<000Eoo`<000Eoo`8006=o
o`00:7oo7P0087oo00<007ooOol0HWoo000YOolL000QOol00`00Oomoo`1ROol002YooaX0029oo`03
001oogoo069oo`00:goo00<007oo00004@0000Aoo`00000002=oo`03001oogoo069oo`00;Goo4P00
00=oo`00000097oo0P00Hgoo000_Ool@000WOol00`00Oomoo`1ROol0031oo`d002Uoo`03001oogoo
069oo`00<Woo00<007oo00001000;Goo00<007ooOol0HWoo001VOol00`00Oomoo`1ROol006Ioo`80
06=oo`00IWoo00<007ooOol0HWoo0000\
\>"],
  ImageRangeCache->{{{0, 202.75}, {124.688, 0}} -> {-2.15751, -0.000133902, 
  0.0262487, 2.68404*^-6}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Plot[ufuncb[x] - Sin[Pi*x/2]*Exp[-Pi^2 0.1/4], {x, -2, 2}, 
PlotPoints -> 50];\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.238095 0.309041 12180.8 [
[.02381 .29654 -6 -9 ]
[.02381 .29654 6 0 ]
[.2619 .29654 -6 -9 ]
[.2619 .29654 6 0 ]
[.7381 .29654 -3 -9 ]
[.7381 .29654 3 0 ]
[.97619 .29654 -3 -9 ]
[.97619 .29654 3 0 ]
[.4875 .06543 -48 -4.5 ]
[.4875 .06543 0 4.5 ]
[.4875 .18723 -48 -4.5 ]
[.4875 .18723 0 4.5 ]
[.4875 .43085 -42 -4.5 ]
[.4875 .43085 0 4.5 ]
[.4875 .55266 -42 -4.5 ]
[.4875 .55266 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .30904 m
.02381 .31529 L
s
[(-2)] .02381 .29654 0 1 Mshowa
.2619 .30904 m
.2619 .31529 L
s
[(-1)] .2619 .29654 0 1 Mshowa
.7381 .30904 m
.7381 .31529 L
s
[(1)] .7381 .29654 0 1 Mshowa
.97619 .30904 m
.97619 .31529 L
s
[(2)] .97619 .29654 0 1 Mshowa
.125 Mabswid
.07143 .30904 m
.07143 .31279 L
s
.11905 .30904 m
.11905 .31279 L
s
.16667 .30904 m
.16667 .31279 L
s
.21429 .30904 m
.21429 .31279 L
s
.30952 .30904 m
.30952 .31279 L
s
.35714 .30904 m
.35714 .31279 L
s
.40476 .30904 m
.40476 .31279 L
s
.45238 .30904 m
.45238 .31279 L
s
.54762 .30904 m
.54762 .31279 L
s
.59524 .30904 m
.59524 .31279 L
s
.64286 .30904 m
.64286 .31279 L
s
.69048 .30904 m
.69048 .31279 L
s
.78571 .30904 m
.78571 .31279 L
s
.83333 .30904 m
.83333 .31279 L
s
.88095 .30904 m
.88095 .31279 L
s
.92857 .30904 m
.92857 .31279 L
s
.25 Mabswid
0 .30904 m
1 .30904 L
s
.5 .06543 m
.50625 .06543 L
s
[(-0.00002)] .4875 .06543 1 0 Mshowa
.5 .18723 m
.50625 .18723 L
s
[(-0.00001)] .4875 .18723 1 0 Mshowa
.5 .43085 m
.50625 .43085 L
s
[(0.00001)] .4875 .43085 1 0 Mshowa
.5 .55266 m
.50625 .55266 L
s
[(0.00002)] .4875 .55266 1 0 Mshowa
.125 Mabswid
.5 .08979 m
.50375 .08979 L
s
.5 .11415 m
.50375 .11415 L
s
.5 .13851 m
.50375 .13851 L
s
.5 .16287 m
.50375 .16287 L
s
.5 .21159 m
.50375 .21159 L
s
.5 .23596 m
.50375 .23596 L
s
.5 .26032 m
.50375 .26032 L
s
.5 .28468 m
.50375 .28468 L
s
.5 .3334 m
.50375 .3334 L
s
.5 .35776 m
.50375 .35776 L
s
.5 .38213 m
.50375 .38213 L
s
.5 .40649 m
.50375 .40649 L
s
.5 .45521 m
.50375 .45521 L
s
.5 .47957 m
.50375 .47957 L
s
.5 .50393 m
.50375 .50393 L
s
.5 .52829 m
.50375 .52829 L
s
.5 .04106 m
.50375 .04106 L
s
.5 .0167 m
.50375 .0167 L
s
.5 .57702 m
.50375 .57702 L
s
.5 .60138 m
.50375 .60138 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30904 m
.04273 .27243 L
.06337 .23321 L
.08275 .19751 L
.10139 .16491 L
.12123 .13292 L
.14033 .10477 L
.16063 .07802 L
.18019 .05681 L
.199 .04017 L
.20853 .03283 L
.21902 .02673 L
.22909 .0221 L
.23349 .02022 L
.23406 .02011 L
.23469 .01995 L
.23581 .01955 L
.23712 .01891 L
.23768 .01859 L
.23829 .01834 L
.23893 .01832 L
.23928 .01831 L
.23961 .01828 L
.2402 .0182 L
.24082 .01808 L
.24145 .0179 L
.24205 .0177 L
.24317 .01721 L
.24373 .01692 L
.24432 .01677 L
.24483 .0168 L
.24539 .01681 L
.24599 .01679 L
.24663 .01672 L
.24724 .01661 L
.2478 .01647 L
.24895 .01608 L
.24958 .01581 L
.25018 .01565 L
.25052 .0157 L
.25089 .01574 L
.25153 .01579 L
.25185 .01581 L
.25218 .0158 L
.25278 .01577 L
.2534 .01569 L
.25375 .01563 L
.25407 .01556 L
.25528 .01519 L
.25561 .01507 L
Mistroke
.25595 .01494 L
.25634 .01503 L
.25669 .0151 L
.25705 .01517 L
.25739 .01522 L
.25803 .01528 L
.25855 .0153 L
.2591 .01529 L
.25971 .01523 L
.26027 .01514 L
.26079 .01503 L
.26136 .01488 L
.26189 .01472 L
.26238 .01486 L
.26294 .01501 L
.26354 .01514 L
.26417 .01524 L
.26451 .01528 L
.26482 .0153 L
.26539 .0153 L
.2659 .01528 L
.26649 .01521 L
.26705 .01512 L
.26767 .01498 L
.26798 .01499 L
.26833 .01512 L
.2695 .0155 L
.27017 .01565 L
.27054 .01571 L
.27088 .01576 L
.27119 .01579 L
.27154 .0158 L
.27216 .0158 L
.27273 .01576 L
.27336 .01569 L
.27368 .01564 L
.27402 .01572 L
.27464 .01599 L
.27522 .01622 L
.27584 .01642 L
.27693 .01668 L
.28672 .01893 L
.28794 .01953 L
.2891 .01995 L
.28974 .02011 L
.29008 .02018 L
.29044 .02024 L
.29107 .02032 L
.29165 .02037 L
.29198 .02058 L
Mistroke
.29233 .02082 L
.29298 .02122 L
.29373 .02164 L
.29444 .02198 L
.29509 .02224 L
.2958 .02247 L
.29645 .02264 L
.29704 .02275 L
.29767 .02289 L
.29799 .02312 L
.29834 .02338 L
.29953 .02418 L
.30093 .02492 L
.30222 .02541 L
.30458 .02657 L
.3071 .02819 L
.31622 .03361 L
.33649 .05009 L
.35601 .0699 L
.3748 .09269 L
.39478 .12104 L
.41402 .15118 L
.43252 .1825 L
.45222 .21781 L
.47118 .25347 L
.49134 .29227 L
.51076 .3299 L
.52944 .36582 L
.54931 .40298 L
.56845 .43725 L
.58878 .47166 L
.60837 .50178 L
.62722 .52771 L
.64728 .55178 L
.66659 .5712 L
.68515 .58542 L
.69468 .59097 L
.70492 .59584 L
.70989 .5979 L
.7153 .59977 L
.71591 .59982 L
.71626 .59986 L
.71657 .59991 L
.71716 .60004 L
.71778 .60022 L
.71916 .60078 L
.71976 .60108 L
.72041 .60132 L
.72102 .60128 L
.72158 .60127 L
Mistroke
.72214 .60129 L
.72267 .60134 L
.72329 .60144 L
.72386 .60157 L
.72448 .60176 L
.72515 .60201 L
.72574 .60226 L
.72638 .60243 L
.72699 .60235 L
.72755 .6023 L
.72811 .60228 L
.72862 .60229 L
.7292 .60233 L
.72975 .60242 L
.73036 .60255 L
.73099 .60273 L
.73154 .60292 L
.73213 .60314 L
.73277 .603 L
.73336 .60289 L
.73372 .60284 L
.73407 .60281 L
.73471 .60278 L
.73522 .60279 L
.73579 .60284 L
.73638 .60293 L
.73694 .60304 L
.73742 .60317 L
.73793 .60332 L
.73849 .60325 L
.739 .60311 L
.73956 .60297 L
.74016 .60287 L
.74072 .6028 L
.74123 .60278 L
.74183 .60279 L
.74217 .60281 L
.74249 .60285 L
.74306 .60293 L
.74367 .60306 L
.74426 .60306 L
.74491 .60282 L
.74548 .60265 L
.74609 .60249 L
.74669 .60238 L
.74723 .60231 L
.74778 .60228 L
.74829 .60228 L
.7489 .60232 L
.74953 .60239 L
Mistroke
.75008 .60243 L
.75068 .60216 L
.75132 .6019 L
.75201 .60166 L
.75266 .60149 L
.75326 .60138 L
.75761 .60052 L
.75816 .60031 L
.75875 .60012 L
.7594 .59996 L
.75999 .59985 L
.76055 .59979 L
.76115 .59975 L
.76171 .59975 L
.76222 .59955 L
.76279 .59922 L
.76333 .59893 L
.76454 .59839 L
.76517 .59817 L
.76576 .59801 L
.76641 .59788 L
.76673 .59783 L
.76709 .59778 L
.76767 .59773 L
.76821 .59747 L
.76941 .59672 L
.77159 .59573 L
.78176 .5908 L
.80125 .57773 L
.82 .56111 L
.83996 .53922 L
.85917 .51404 L
.87764 .48689 L
.89731 .4552 L
.91623 .4224 L
.93636 .38538 L
.95575 .34855 L
.97439 .31258 L
.97619 .30904 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True,
  ImageSize->{163, 100.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003;0000OA000`40O003h00OIWoo00<007ooOol0
HWoo001VOol00`00Oomoo`1ROol006Ioo`03001oogoo069oo`00=Goo1000;Goo00<007ooOol0HWoo
000aOol50002Ool6000XOol2001SOol002moo`8000aoo`<002Ioo`03001oogoo069oo`00;Goo0P00
47oo0P009Goo00<007ooOol0HWoo000[Ool2000DOol2000SOol00`00Oomoo`1ROol002Yoo`03001o
ogoo01Ioo`80025oo`03001oogoo069oo`00:Goo00<007ooOol06Goo00<007ooOol07Woo0P00Hgoo
000XOol00`00Oomoo`0KOol00`00Oomoo`0MOol00`00Oomoo`1ROol002Moo`03001oogoo00Eoo`<0
00Ioo`8000Aoo`<000Eoo`<000Eoo`<000Eoo`<000Aoo`D000Aoo`03001oogoo069oo`009Woo00<0
07ooOol01Goo00D007ooOomoo`0000Eoo`8000=oo`05001oogooOol00003Ool01@00Ool007oo0000
0goo00D007ooOomoo`0000=oo`05001oogooOol00004Ool01000Oomoo`0017oo00<007ooOol0HWoo
000UOol70002Ool01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`03001oogoo008000=oo`05
001oogooOol00003Ool01@00Oomoogoo00001Goo00<007ooOol017oo0P00Hgoo000TOol00`00Oomo
o`07Ool01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomo
ogoo00000goo00D007ooOomoo`0000Ioo`03001oogoo00=oo`03001oogoo069oo`008goo00<007oo
Ool027oo00D007ooOomoo`0000Yoo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00007Ool00`00Oomoo`02Ool00`00Oomoo`1ROol0029oo`03001o
ogoo00Uoo`05001oogooOol0000:Ool01@00Oomoogoo00000goo00@007ooOomoo`80009oo`05001o
ogooOol00003Ool01@00Oomoogoo00001goo00<007ooOol00Woo00<007ooOol0HWoo000ROol00`00
Oomoo`09Ool01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`09001oogooOol007oo001oo`00
00=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00<007ooOol0HWoo
000QOol00`00Oomoo`0;Ool3000<Ool30005Ool30003Ool00`00Ool000020005Ool30005Ool30005
Ool2001SOol0021oo`03001oogoo02Yoo`03001oogoo01Ioo`03001oogoo069oo`007goo00<007oo
Ool0:goo00<007ooOol05Woo00<007ooOol0HWoo000NOol00`00Oomoo`0]Ool00`00Oomoo`0EOol0
0`00Oomoo`1ROol001ioo`03001oogoo02ioo`03001oogoo01Aoo`03001oogoo069oo`007Goo00<0
07ooOol0;goo00<007ooOol057oo0P00Hgoo000LOol00`00Oomoo`0aOol00`00Oomoo`0COol00`00
Oomoo`1ROol001aoo`03001oogoo039oo`03001oogoo019oo`03001oogoo069oo`006goo00<007oo
Ool0<goo00<007ooOol04Woo00<007ooOol0HWoo000JOol00`00Oomoo`0eOol00`00Oomoo`0AOol0
0`00Oomoo`1ROol001Yoo`03001oogoo03Ioo`03001oogoo011oo`8006=oo`006Goo00<007ooOol0
=goo00<007ooOol047oo00<007ooOol0HWoo000IOol00`00Oomoo`0hOol00`00Oomoo`0?Ool00`00
Oomoo`1ROol001Qoo`03001oogoo03Yoo`03001oogoo00ioo`03001oogoo069oo`0067oo00<007oo
Ool0>Woo00<007ooOol03Woo00<007ooOol0HWoo000GOol00`00Oomoo`0lOol00`00Oomoo`0=Ool2
001SOol001Moo`03001oogoo03eoo`03001oogoo00aoo`03001oogoo069oo`005Woo00<007ooOol0
5Woo0`001Woo0P0017oo0`001Goo0`001Goo0`001Goo0`001Goo1@000goo00<007ooOol0HWoo000F
Ool00`00Oomoo`0EOol01@00Oomoogoo00001Goo0P000goo00D007ooOomoo`0000=oo`05001oogoo
Ool00003Ool01@00Oomoogoo00000goo00<007ooOol00P001Woo00<007ooOol00goo00<007ooOol0
HWoo000EOol00`00Oomoo`0>Ool60002Ool01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05
001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Ioo`03001oogoo00=oo`03
001oogoo069oo`0057oo00<007ooOol05goo00D007ooOomoo`0000Yoo`05001oogooOol00003Ool0
1@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00006Ool00`00Oomoo`03Ool2
001SOol001Aoo`03001oogoo01Moo`05001oogooOol0000:Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool01000Oomoogoo0P001Goo00<007ooOol00goo00<007oo
Ool0HWoo000COol00`00Oomoo`0HOol01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05001o
ogooOol00003Ool01@00Oomoogoo00000goo00@007ooOomoo`8000Eoo`03001oogoo00=oo`03001o
ogoo069oo`004goo00<007ooOol067oo00D007ooOomoo`0000Yoo`05001oogooOol00003Ool01@00
Oomoogoo00000goo00D007ooOomoo`0000=oo`07001oogooOol007oo000017oo00<007ooOol00goo
00<007ooOol0HWoo000BOol00`00Oomoo`0JOol3000<Ool30005Ool30005Ool30005Ool30003Ool0
0`00Ool000020005Ool00`00Oomoo`1ROol0019oo`03001oogoo04Moo`03001oogoo00Moo`8006=o
o`004Goo00<007ooOol0BGoo00<007ooOol01Woo00<007ooOol0HWoo000@Ool00`00Oomoo`1:Ool0
0`00Oomoo`06Ool00`00Oomoo`1ROol0011oo`03001oogoo04]oo`03001oogoo00Eoo`03001oogoo
069oo`003goo00<007ooOol0C7oo00<007ooOol01Goo00<007ooOol0HWoo000?Ool00`00Oomoo`1=
Ool00`00Oomoo`04Ool2001SOol000ioo`03001oogoo04ioo`03001oogoo00Aoo`03001oogoo069o
o`003Woo00<007ooOol0CWoo00<007ooOol017oo00<007ooOol0HWoo0008Ool6000[Ool5000ROol0
0`00Oomoo`03Ool00`00Oomoo`0[Ool5000ZOol50003Ool000Uoo`03001oogoo008002eoo`03001o
ogoo029oo`03001oogoo00=oo`8002ioo`03001oogoo02]oo`04001oogoo0003Ool000H000Aoo`03
001oo`0002=oo`H000Eoo`03001oogoo02=oo`03001oogoo009oo`03001oogoo02eoo`03001oogoo
02aoo`03001oogoo00=oo`002goo0P00;Woo00<007ooOol097oo00D007ooOomoo`0002moo`03001o
ogoo02eoo`03001oogoo009oo`002goo0P00;Woo00<007ooOol097oo00D007ooOomoo`0002moo`03
001oogoo02ioo`03001oogoo005oo`002goo0P00;Woo00<007ooOol09Goo00@007ooOol002moo`03
001oogoo02ioo`03001oogoo005oo`0027oo00D007oo001oo`0002ioo`03001oogoo02Eoo`03001o
ogoo008002ioo`03001oogoo02Yoo`05001oogooOol00003Ool000Uoo`<002eoo`<002Qoo`03001o
o`0002eoo`<002eoo`<000Aoo`002Goo00<007ooOol0FGoo0P00I7oo0008Ool00`00Oomoo`1JOol2
001TOol000Qoo`03001oogoo05]oo`03001oogoo069oo`000Woob@000007Ool00`00Oomoo`07Ool0
0`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`06Ool0
0`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`06Ool20008Ool00`00Oomo
o`06Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomo
o`07Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`03Ool006Ioo`80
05eoo`03001oogoo00=oo`00IWoo0P00G7oo00<007ooOol017oo001VOol00`00Ool0001KOol00`00
Oomoo`04Ool006Ioo`<005Yoo`03001oogoo00Eoo`00IWoo00@007ooOol005Qoo`03001oogoo00Io
o`00IWoo00@007ooOol005Qoo`03001oogoo00Ioo`00IWoo00D007ooOomoo`0005Ioo`03001oogoo
00Moo`00IWoo00D007ooOomoo`0005Ioo`03001oogoo00Moo`00IWoo0P000goo00<007ooOol0DWoo
00<007ooOol027oo001VOol00`00Oomoo`02Ool00`00Oomoo`1BOol00`00Oomoo`08Ool006Ioo`03
001oogoo00=oo`03001oogoo051oo`03001oogoo00Uoo`00IWoo00<007ooOol00goo00<007ooOol0
D7oo00<007ooOol02Goo001VOol00`00Oomoo`04Ool00`00Oomoo`1?Ool00`00Oomoo`09Ool006Io
o`8000Eoo`03001oogoo04ioo`03001oogoo00Yoo`00IWoo00<007ooOol01Goo00<007ooOol0CGoo
00<007ooOol02Woo001VOol00`00Oomoo`06Ool00`00Oomoo`1;Ool00`00Oomoo`0;Ool006Ioo`03
001oogoo00Ioo`03001oogoo04Yoo`03001oogoo00aoo`00IWoo00<007ooOol01goo00<007ooOol0
BGoo00<007ooOol037oo001VOol20008Ool00`00Oomoo`18Ool00`00Oomoo`0=Ool006Ioo`03001o
ogoo00Qoo`03001oogoo04Moo`03001oogoo00eoo`00;goo0`001Woo0P001Goo0`0017oo0`001Goo
0`001Goo0`001Goo1@000goo00<007ooOol027oo00<007ooOol0AWoo00<007ooOol03Woo000^Ool0
1@00Oomoogoo00001Goo0P0017oo00D007ooOomoo`00009oo`05001oogooOol00003Ool01@00Oomo
ogoo00000goo00D007ooOomoo`0000Ioo`03001oogoo00=oo`03001oogoo00Uoo`03001oogoo04Ao
o`03001oogoo00moo`00;Woo00D007ooOomoo`0000]oo`05001oogooOol00002Ool01@00Oomoogoo
00000goo00D007ooOomoo`0000=oo`05001oogooOol00006Ool00`00Oomoo`03Ool00`00Oomoo`0:
Ool00`00Oomoo`13Ool00`00Oomoo`0?Ool002ioo`05001oogooOol0000;Ool01@00Oomoogoo0000
0Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00001Woo00<007ooOol0
0goo0P002goo00<007ooOol0@Woo00<007ooOol047oo000^Ool01@00Oomoogoo00002goo00D007oo
Oomoo`00009oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Ioo`03
001oogoo00=oo`03001oogoo00]oo`03001oogoo045oo`03001oogoo011oo`00;Woo00D007ooOomo
o`0000]oo`05001oogooOol00002Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001o
ogooOol00006Ool00`00Oomoo`03Ool00`00Oomoo`0;Ool00`00Oomoo`10Ool00`00Oomoo`0AOol0
02ioo`05001oogooOol0000;Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000=oo`05001oogoo
Ool00003Ool01@00Oomoogoo00001Woo00<007ooOol00goo00<007ooOol037oo00<007ooOol0?goo
00<007ooOol04Goo000_Ool3000=Ool30004Ool30005Ool30005Ool30005Ool30005Ool2000=Ool0
0`00Oomoo`0nOol00`00Oomoo`0BOol006Ioo`03001oogoo00eoo`03001oogoo03eoo`03001oogoo
019oo`00IWoo00<007ooOol03Woo00<007ooOol0>goo00<007ooOol04goo001VOol00`00Oomoo`0>
Ool00`00Oomoo`0kOol00`00Oomoo`0COol006Ioo`03001oogoo00moo`03001oogoo03Uoo`03001o
ogoo01Aoo`00IWoo0P0047oo00<007ooOol0>7oo00<007ooOol05Goo001VOol00`00Oomoo`0@Ool0
0`00Oomoo`0gOol00`00Oomoo`0EOol006Ioo`03001oogoo015oo`03001oogoo03Eoo`03001oogoo
01Ioo`00IWoo00<007ooOol04Goo00<007ooOol0=Goo00<007ooOol05Woo001VOol00`00Oomoo`0B
Ool00`00Oomoo`0cOol00`00Oomoo`0GOol006Ioo`8001Aoo`03001oogoo035oo`03001oogoo01Qo
o`00IWoo00<007ooOol04goo00<007ooOol0<7oo00<007ooOol06Goo001VOol00`00Oomoo`0DOol0
0`00Oomoo`0^Ool00`00Oomoo`0JOol006Ioo`03001oogoo01Eoo`03001oogoo02eoo`03001oogoo
01Yoo`00IWoo00<007ooOol05Woo00<007ooOol0:goo00<007ooOol06goo001VOol2000HOol00`00
Oomoo`0YOol00`00Oomoo`0LOol006Ioo`03001oogoo01Moo`03001oogoo02Qoo`03001oogoo01eo
o`00;goo0`001Woo0P001Goo0`0017oo0`001Goo0`001Goo0`0017oo1@0017oo00<007ooOol067oo
00<007ooOol09Woo00<007ooOol07Woo000^Ool01@00Oomoogoo00001Goo0P0017oo00D007ooOomo
o`00009oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Aoo`04001o
ogoo0004Ool00`00Oomoo`0IOol00`00Oomoo`0UOol00`00Oomoo`0NOol002ioo`05001oogooOol0
000;Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomo
ogoo00001Goo00<007ooOol017oo00<007ooOol06Woo00<007ooOol08goo00<007ooOol07goo000^
Ool01@00Oomoogoo00002goo00D007ooOomoo`00009oo`05001oogooOol00003Ool01@00Oomoogoo
00000goo00D007ooOomoo`0000Ioo`03001oogoo00=oo`8001aoo`03001oogoo025oo`03001oogoo
021oo`00;Woo00D007ooOomoo`0000]oo`05001oogooOol00002Ool01@00Oomoogoo00000goo00D0
07ooOomoo`0000=oo`05001oogooOol00007Ool00`00Oomoo`02Ool00`00Oomoo`0LOol00`00Oomo
o`0OOol00`00Oomoo`0QOol002ioo`05001oogooOol0000;Ool01@00Oomoogoo00000Woo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00001goo00<007ooOol00Woo00<007oo
Ool07Goo00<007ooOol077oo0P0097oo000^Ool01@00Oomoogoo00002goo00D007ooOomoo`00009o
o`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol0
0004Ool00`00Oomoo`0NOol00`00Oomoo`0JOol00`00Oomoo`0TOol002moo`<000eoo`<000Aoo`<0
00Eoo`<000Eoo`<000Eoo`<000Eoo`03001oogoo01moo`8001Uoo`03001oogoo02Eoo`00IWoo0P00
8Woo00<007ooOol05Goo00<007ooOol09Woo001VOol00`00Oomoo`0ROol00`00Oomoo`0COol00`00
Oomoo`0WOol006Ioo`03001oogoo02=oo`03001oogoo015oo`03001oogoo02Qoo`00IWoo00<007oo
Ool097oo0P003goo0P00:goo001VOol00`00Oomoo`0VOol40007Ool4000]Ool006Ioo`8002Yoo`P0
035oo`00IWoo00<007ooOol0HWoo001VOol00`00Oomoo`1ROol006Ioo`03001oogoo069oo`00\
\>"],
  ImageRangeCache->{{{0, 202.75}, {124.625, 0}} -> {-2.15751, -2.5579*^-5, 
  0.0262487, 5.13076*^-7}}]
}, Open  ]],

Cell["\<\
It is clear that linear is hopeless, especially if we wish to \
differentiate. Finally we tabulate the numerical result with cubic \
interpolation, the exact result and the error in the numerical scheme.\
\>", 
  "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell["\<\
samples = TableForm[Join[{{\"x\", \"Exact\", \"Crank-Nich\", \
\"Error\"}},
Table[Map[PaddedForm[N[Chop[#1]],{5,6}]&,
N[{x, ufuncb[x], Sin[Pi*x/2]*Exp[-(Pi^2 0.1)/4], 
ufuncb[x]- Sin[Pi*x/2]*Exp[-(Pi^2 0.1)/4]},5]],{x, -2, 2, 0.25}]]]\
\>", 
  "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakBelow->False,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"x\"\>", "\<\"Exact\"\>", "\<\"Crank-Nich\"\>", 
            "\<\"Error\"\>"},
          {
            TagBox[
              InterpretationBox["\<\"-2.000000\"\>",
                -2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.750000\"\>",
                -1.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299020\"\>",
                -0.29901654759915974,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299010\"\>",
                -0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\("-9.246900"\[Times]10\^"-6"\),
                -9.2469363199287891*^-06,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.500000\"\>",
                -1.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552510\"\>",
                -0.55251053641810111,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552490\"\>",
                -0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000017\"\>",
                -1.7086110408737198*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.250000\"\>",
                -1.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721890\"\>",
                -0.72188980458787277,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721870\"\>",
                -0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000022\"\>",
                -2.2324079075208836*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.000000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.781370\"\>",
                -0.78136789395651451,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.781340\"\>",
                -0.78134373054744422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000024\"\>",
                -2.4163409070299124*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.750000\"\>",
                -0.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721890\"\>",
                -0.72188980458787277,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721870\"\>",
                -0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000022\"\>",
                -2.2324079075208836*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.500000\"\>",
                -0.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552510\"\>",
                -0.55251053641810099,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552490\"\>",
                -0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000017\"\>",
                -1.7086110408626176*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.250000\"\>",
                -0.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299020\"\>",
                -0.29901654759915941,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299010\"\>",
                -0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\("-9.246900"\[Times]10\^"-6"\),
                -9.2469363195957222*^-06,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.250000\"\>",
                0.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299020\"\>",
                0.2990165475991593,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299010\"\>",
                0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\(" 9.246900"\[Times]10\^"-6"\),
                9.2469363194846999*^-06,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.500000\"\>",
                0.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552510\"\>",
                0.55251053641810066,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552490\"\>",
                0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000017\"\>",
                1.7086110408293109*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.750000\"\>",
                0.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721890\"\>",
                0.72188980458787255,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721870\"\>",
                0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000022\"\>",
                2.2324079074986791*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.000000\"\>",
                1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.781370\"\>",
                0.78136789395651451,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.781340\"\>",
                0.78134373054744422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000024\"\>",
                2.4163409070299124*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.250000\"\>",
                1.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721890\"\>",
                0.72188980458787277,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721870\"\>",
                0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000022\"\>",
                2.2324079075208836*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.500000\"\>",
                1.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552510\"\>",
                0.55251053641810088,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552490\"\>",
                0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000017\"\>",
                1.7086110408515154*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.750000\"\>",
                1.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299020\"\>",
                0.29901654759915952,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299010\"\>",
                0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\(" 9.246900"\[Times]10\^"-6"\),
                9.2469363197067445*^-06,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 2.000000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell["\<\
Note that we obtain comparable accuracy or better with a tenth the \
number of time-steps that where used for the explicit case. This example \
nicely illustrates the power of the implicit approach and the Crank-Nicholson \
scheme in particular. As we shall see, we can do rather better.\
\>", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["14.9 Douglas", "Section",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
A further small change to the matrices takes us to the Douglas \
algorithm. This time we just stick with cubic interpolation.\
\>", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Douglas Algorithm", "Subsubsection",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell["\<\
DougCMatrix[alpha_, nminus_, nplus_] :=
Sequence[Table[1-6*alpha, {nplus+nminus-2}], Table[10+12*alpha, \
{nplus+nminus-1}],
Table[1-6*alpha, {nplus+nminus-2}]];

DougDMatrix[alpha_, vec_List] := Module[{temp},
temp = (10 - 12*alpha)*vec + (1+6*alpha)*(RotateRight[vec] + \
RotateLeft[vec]);
temp[[1]] = Simplify[First[temp] - \.0c(1+6*alpha)*Last[vec]];
temp[[-1]] = Simplify[Last[temp] - (1 + 6*alpha)*First[vec]];
temp];

M=40;nminus = 80;nplus = 80;dx = 0.025;
dtau = 0.0025;alpha = dtau/dx^2;
CMat = DougCMatrix[alpha,nminus,nplus];
initial = Table[N[Sin[Pi*(k-1-nminus)/nminus] ], {k,1, nminus+nplus+1}];
lower=Table[0, {m, 1, M+1}];
upper=Table[0, {m, 1, M+1}];
wold = initial; wvold = wold; wnew = wold;

For[m=2, m<=M+1, m++,
(wvold = wold;
wold = wnew;
rhs = DougDMatrix[alpha, Take[wold, {2, -2}]]+
Table[
Which[
k==1, (6*alpha+1)*lower[[m-1]] +(6 alpha - 1)*lower[[m]],
k== nplus + nminus-1,
(6*alpha+1)*upper[[m-1]] + (6 alpha - 1)*upper[[m]],
True, 0],
{k, 1, nplus + nminus-1}];
temp = CompTridiagSolve[CMat, rhs];
wnew = Join[{lower[[m]]}, temp, {upper[[m]]}])];
interpoldata = 
Table[{(k-nminus-1)*dx ,wnew[[k]]}, {k, 1, nminus+nplus+1}];
ufunc = Interpolation[interpoldata, InterpolationOrder -> 3];\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell["Now we plot the error - note the vertical scale!", "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot[ufunc[x] - Sin[Pi*x/2]*Exp[-Pi^2 0.1/4], {x, -2, 2}, 
PlotPoints -> 50];\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.238095 0.309023 450838 [
[.02381 .29652 -6 -9 ]
[.02381 .29652 6 0 ]
[.2619 .29652 -6 -9 ]
[.2619 .29652 6 0 ]
[.7381 .29652 -3 -9 ]
[.7381 .29652 3 0 ]
[.97619 .29652 -3 -9 ]
[.97619 .29652 3 0 ]
[.4875 .03852 -39 -6.25 ]
[.4875 .03852 0 6.25 ]
[.4875 .12869 -39 -6.25 ]
[.4875 .12869 0 6.25 ]
[.4875 .21886 -39 -6.25 ]
[.4875 .21886 0 6.25 ]
[.4875 .39919 -33 -6.25 ]
[.4875 .39919 0 6.25 ]
[.4875 .48936 -33 -6.25 ]
[.4875 .48936 0 6.25 ]
[.4875 .57953 -33 -6.25 ]
[.4875 .57953 0 6.25 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .30902 m
.02381 .31527 L
s
[(-2)] .02381 .29652 0 1 Mshowa
.2619 .30902 m
.2619 .31527 L
s
[(-1)] .2619 .29652 0 1 Mshowa
.7381 .30902 m
.7381 .31527 L
s
[(1)] .7381 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(2)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.07143 .30902 m
.07143 .31277 L
s
.11905 .30902 m
.11905 .31277 L
s
.16667 .30902 m
.16667 .31277 L
s
.21429 .30902 m
.21429 .31277 L
s
.30952 .30902 m
.30952 .31277 L
s
.35714 .30902 m
.35714 .31277 L
s
.40476 .30902 m
.40476 .31277 L
s
.45238 .30902 m
.45238 .31277 L
s
.54762 .30902 m
.54762 .31277 L
s
.59524 .30902 m
.59524 .31277 L
s
.64286 .30902 m
.64286 .31277 L
s
.69048 .30902 m
.69048 .31277 L
s
.78571 .30902 m
.78571 .31277 L
s
.83333 .30902 m
.83333 .31277 L
s
.88095 .30902 m
.88095 .31277 L
s
.92857 .30902 m
.92857 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.5 .03852 m
.50625 .03852 L
s
gsave
.4875 .03852 -39 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(-6) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-7) show
grestore
.5 .12869 m
.50625 .12869 L
s
gsave
.4875 .12869 -39 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(-4) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-7) show
grestore
.5 .21886 m
.50625 .21886 L
s
gsave
.4875 .21886 -39 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(-2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-7) show
grestore
.5 .39919 m
.50625 .39919 L
s
gsave
.4875 .39919 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-7) show
grestore
.5 .48936 m
.50625 .48936 L
s
gsave
.4875 .48936 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(4) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-7) show
grestore
.5 .57953 m
.50625 .57953 L
s
gsave
.4875 .57953 -33 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(6) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-7) show
grestore
.125 Mabswid
.5 .06106 m
.50375 .06106 L
s
.5 .0836 m
.50375 .0836 L
s
.5 .10615 m
.50375 .10615 L
s
.5 .15123 m
.50375 .15123 L
s
.5 .17377 m
.50375 .17377 L
s
.5 .19631 m
.50375 .19631 L
s
.5 .2414 m
.50375 .2414 L
s
.5 .26394 m
.50375 .26394 L
s
.5 .28648 m
.50375 .28648 L
s
.5 .33156 m
.50375 .33156 L
s
.5 .35411 m
.50375 .35411 L
s
.5 .37665 m
.50375 .37665 L
s
.5 .42173 m
.50375 .42173 L
s
.5 .44427 m
.50375 .44427 L
s
.5 .46682 m
.50375 .46682 L
s
.5 .5119 m
.50375 .5119 L
s
.5 .53444 m
.50375 .53444 L
s
.5 .55698 m
.50375 .55698 L
s
.5 .01598 m
.50375 .01598 L
s
.5 .60207 m
.50375 .60207 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30902 m
.04273 .34472 L
.06337 .38443 L
.08275 .41555 L
.10139 .44475 L
.12123 .48461 L
.14033 .5133 L
.14534 .52015 L
.14567 .52097 L
.14599 .52138 L
.14636 .5214 L
.1467 .521 L
.14731 .51932 L
.14765 .51786 L
.14797 .5162 L
.14862 .51215 L
.14898 .51241 L
.1493 .51548 L
.15007 .5217 L
.1504 .52392 L
.15077 .52594 L
.15138 .52828 L
.15172 .52898 L
.15204 .52926 L
.15238 .52914 L
.1527 .52866 L
.15306 .52767 L
.15341 .52633 L
.15403 .52303 L
.1546 .51915 L
.15523 .52252 L
.15555 .52537 L
.1559 .52822 L
.15647 .53208 L
.15707 .53496 L
.15739 .53593 L
.15773 .53657 L
.15834 .53659 L
.15891 .53535 L
.15953 .53277 L
.16011 .52929 L
.16063 .52543 L
.16122 .5298 L
.16176 .5346 L
.16236 .53886 L
.16301 .54206 L
.16333 .54307 L
.16368 .54372 L
.16432 .54365 L
.16494 .54208 L
Mistroke
.16552 .53941 L
.16611 .53562 L
.16667 .53129 L
.16717 .53638 L
.1677 .54121 L
.16829 .5455 L
.16892 .5487 L
.16945 .55024 L
.17003 .55063 L
.18021 .55805 L
.18085 .56133 L
.18145 .56288 L
.18208 .56288 L
.18241 .56221 L
.18277 .56099 L
.18314 .55923 L
.18352 .55686 L
.18388 .55425 L
.18421 .55151 L
.18485 .5523 L
.18519 .55571 L
.18555 .55901 L
.1862 .56391 L
.18657 .56596 L
.18691 .56736 L
.18753 .56866 L
.18785 .56868 L
.18819 .5682 L
.18879 .5662 L
.18943 .56251 L
.18999 .55818 L
.19059 .55512 L
.19122 .56168 L
.19181 .56673 L
.19243 .57079 L
.19279 .57239 L
.19313 .57339 L
.19373 .57394 L
.19428 .57303 L
.19489 .57054 L
.19553 .5664 L
.19607 .56191 L
.19665 .56098 L
.19719 .56662 L
.19768 .57099 L
.19827 .57499 L
.1989 .57776 L
.19949 .57877 L
.20004 .57828 L
.20061 .57636 L
Mistroke
.20114 .57343 L
.20174 .56883 L
.20232 .56348 L
.20285 .56802 L
.20334 .57286 L
.20387 .57721 L
.20445 .58067 L
.20509 .5828 L
.20568 .58312 L
.20627 .58188 L
.2068 .57943 L
.20738 .57551 L
.20802 .56992 L
.20834 .56687 L
.20868 .57076 L
.20931 .57701 L
.20962 .57972 L
.20996 .58219 L
.21057 .58543 L
.21091 .58653 L
.21122 .58708 L
.21157 .58715 L
.21194 .5866 L
.21258 .58422 L
.2129 .58238 L
.21325 .57992 L
.21387 .57448 L
.21444 .57213 L
.21505 .57858 L
.21562 .58363 L
.21622 .58761 L
.21656 .58917 L
.21687 .59014 L
.21739 .59075 L
.21796 .58994 L
.21856 .58749 L
.21912 .58384 L
.21974 .57855 L
.22008 .57512 L
.22039 .57523 L
.221 .58173 L
.22156 .5868 L
.22189 .58913 L
.22223 .59112 L
.22284 .59336 L
.22319 .59384 L
.2235 .59379 L
.22385 .59319 L
.22423 .59191 L
.22485 .58841 L
Mistroke
.2252 .58575 L
.22553 .5829 L
.22613 .57687 L
.22676 .58256 L
.22729 .58766 L
.22786 .59199 L
.22848 .59516 L
.22907 .59649 L
.22967 .59615 L
.23023 .59429 L
.23084 .59065 L
.2315 .58517 L
.23212 .57872 L
.23271 .58485 L
.23324 .59 L
.23381 .5943 L
.23433 .59704 L
.23489 .59862 L
.23541 .5987 L
.23589 .59761 L
.23646 .59495 L
.23698 .59131 L
.23758 .58585 L
.23815 .58101 L
.23869 .58701 L
.23927 .59249 L
.23991 .59708 L
.2405 .59968 L
.24108 .60062 L
.24172 .59976 L
.24204 .59859 L
.24238 .59682 L
.243 .59238 L
.24358 .58693 L
.24421 .58377 L
.24481 .59016 L
.24535 .59504 L
.24585 .59842 L
.24639 .60089 L
.24696 .60196 L
.24749 .60153 L
.24812 .59926 L
.24848 .59718 L
.24881 .59478 L
.24946 .58881 L
.25005 .58353 L
.25064 .58997 L
.25119 .59508 L
.25177 .5992 L
.25239 .60199 L
Mistroke
.25303 .60288 L
.25339 .60251 L
.25373 .60158 L
.25408 .60002 L
.25441 .5981 L
.25514 .59222 L
.25576 .58578 L
.25611 .58536 L
.25643 .58895 L
.25703 .59474 L
.25766 .59938 L
.25823 .6021 L
.25885 .60332 L
.25941 .60279 L
.25993 .60091 L
.26059 .59676 L
.26122 .59125 L
.26178 .58524 L
.26238 .58911 L
.26292 .59435 L
.26343 .59828 L
.26395 .60125 L
.26453 .60304 L
.2651 .60321 L
.26572 .60159 L
.26629 .59849 L
.26681 .59443 L
.26744 .58829 L
.26775 .58476 L
.2681 .58634 L
.26872 .59263 L
.2693 .59737 L
.26962 .59948 L
.26997 .60117 L
.27059 .60277 L
.27094 .60284 L
.27126 .60238 L
.27162 .60129 L
.272 .59948 L
.27263 .59502 L
.27297 .59203 L
.27332 .58845 L
.27363 .58502 L
.27397 .58474 L
.27457 .59098 L
.27515 .59591 L
.27578 .5997 L
.27637 .60163 L
.27691 .60192 L
.27754 .60045 L
Mistroke
.27814 .59735 L
.27867 .59328 L
.27925 .58768 L
.27988 .58315 L
.28019 .58658 L
.28054 .59002 L
.28117 .59514 L
.28173 .59844 L
.28231 .60028 L
.28285 .60056 L
.28343 .59929 L
.28405 .59616 L
.28459 .59212 L
.28517 .58656 L
.28571 .58041 L
.28621 .58571 L
.28679 .59103 L
.28743 .59541 L
.28777 .59705 L
.28808 .59811 L
.28841 .59871 L
.28877 .59877 L
.28912 .59822 L
.2895 .59696 L
.28986 .59515 L
.29019 .59298 L
.29082 .58766 L
.29117 .58413 L
.29149 .5806 L
.29206 .58273 L
.29269 .58849 L
.29326 .5926 L
.29381 .59522 L
.29444 .59654 L
.29503 .59604 L
.29568 .5936 L
.29604 .59142 L
.29636 .58898 L
.29692 .58394 L
.29751 .57749 L
.29785 .57861 L
.29816 .58171 L
.29876 .58697 L
.29938 .59097 L
.29973 .5925 L
.30006 .59342 L
.30066 .59382 L
.30129 .5924 L
.30182 .58975 L
.3024 .58554 L
Mistroke
.3029 .5809 L
.30344 .57498 L
.30407 .57858 L
.30468 .58378 L
.30527 .58759 L
.30581 .58984 L
.30645 .59075 L
.30681 .59041 L
.30714 .58956 L
.30776 .58669 L
.3084 .58194 L
.30894 .57679 L
.30952 .57036 L
.31016 .57653 L
.31075 .5813 L
.31108 .58339 L
.31143 .58514 L
.31181 .58646 L
.31216 .58709 L
.3128 .58676 L
.31316 .58573 L
.3135 .58422 L
.31413 .58013 L
.31481 .57411 L
.31544 .5672 L
.31602 .57204 L
.31658 .57668 L
.3172 .58043 L
.31778 .58256 L
.31831 .5832 L
.31889 .5824 L
.31953 .57977 L
.31986 .57778 L
.3202 .57519 L
.32083 .56944 L
.32114 .5661 L
.32148 .56332 L
.32184 .56685 L
.32218 .56981 L
.32253 .57248 L
.32286 .57461 L
.32347 .57747 L
.32408 .57872 L
.32464 .57843 L
.32525 .57651 L
.32589 .57274 L
.32646 .56814 L
.32697 .56305 L
.32757 .56031 L
.32813 .56524 L
Mistroke
.32861 .56875 L
.32913 .57157 L
.32969 .57344 L
.33021 .57395 L
.33077 .57316 L
.33138 .57078 L
.33195 .56716 L
.33247 .5628 L
.33305 .55699 L
.33368 .557 L
.334 .5597 L
.33434 .56226 L
.33495 .56589 L
.33556 .56807 L
.3359 .56863 L
.33622 .56869 L
.33656 .56828 L
.33687 .56746 L
.33723 .56601 L
.33757 .56416 L
.33874 .55459 L
.33907 .55109 L
.33939 .54967 L
.33998 .55463 L
.34063 .55898 L
.34099 .5608 L
.34133 .56202 L
.34194 .56307 L
.34258 .56247 L
.34292 .56146 L
.34328 .55984 L
.34363 .55785 L
.34394 .55566 L
.34466 .54936 L
.34502 .54565 L
.34542 .54487 L
.34573 .54748 L
.34607 .55001 L
.34668 .55372 L
.34729 .55612 L
.34785 .55704 L
.3485 .55649 L
.34887 .55542 L
.34921 .55394 L
.34983 .55008 L
.35049 .54453 L
.35112 .53825 L
.35147 .53979 L
.35179 .54229 L
.35237 .54612 L
Mistroke
.35301 .54908 L
.35358 .55045 L
.35412 .55057 L
.35469 .54943 L
.35529 .54681 L
.35655 .5373 L
.35718 .53161 L
.35754 .53447 L
.35788 .53689 L
.35848 .54042 L
.35903 .54264 L
.35964 .54381 L
.36027 .54351 L
.36061 .5427 L
.36092 .54158 L
.36128 .53984 L
.36162 .53778 L
.36222 .53315 L
.36286 .52716 L
.3632 .5256 L
.36357 .52836 L
.36422 .53244 L
.36459 .53421 L
.36498 .53561 L
.36535 .53644 L
.36569 .53674 L
.3663 .53621 L
.36695 .5341 L
.36756 .53072 L
.36811 .52663 L
.36843 .52389 L
.36877 .52075 L
.36913 .51859 L
.36947 .52101 L
.37009 .5248 L
.37043 .52645 L
.37075 .52768 L
.37137 .52908 L
.37193 .52916 L
.37246 .52818 L
.37304 .52595 L
.37425 .51776 L
.37458 .51482 L
.37493 .51149 L
.37555 .5145 L
.3761 .51758 L
.37669 .51996 L
.37733 .52131 L
.37793 .52128 L
.37853 .51996 L
Mistroke
.3791 .51757 L
.37961 .51453 L
.38016 .5104 L
.38078 .50497 L
.38113 .50446 L
.38145 .50652 L
.38211 .50999 L
.38248 .51144 L
.38281 .51242 L
.38341 .51328 L
.38375 .51323 L
.38406 .51284 L
.38464 .51125 L
.38525 .50836 L
.38588 .50421 L
.38646 .49952 L
.3871 .49677 L
.38742 .49867 L
.38777 .50052 L
.38837 .50297 L
.389 .50451 L
.3896 .50482 L
.39014 .50409 L
.39555 .49605 L
.41511 .46291 L
.43393 .42783 L
.45396 .39556 L
.47324 .36073 L
.49178 .32498 L
.51152 .28789 L
.53052 .25236 L
.55072 .21238 L
.57017 .18081 L
.58889 .15455 L
.6088 .11808 L
.60938 .1154 L
.60992 .11382 L
.61055 .1132 L
.61112 .11374 L
.61166 .11516 L
.61216 .11719 L
.61269 .12003 L
.61328 .12079 L
.61385 .11592 L
.61438 .11199 L
.61494 .10867 L
.61554 .10619 L
.61617 .10489 L
.61648 .10474 L
.61683 .10497 L
Mistroke
.61744 .10634 L
.61801 .10862 L
.61834 .1103 L
.61869 .11239 L
.61933 .11208 L
.61995 .10673 L
.62052 .10263 L
.62114 .09931 L
.6215 .09799 L
.62182 .09715 L
.62217 .09667 L
.62249 .09661 L
.62286 .09699 L
.6232 .09776 L
.62377 .09986 L
.62431 .10268 L
.62493 .10675 L
.62549 .10263 L
.62611 .09744 L
.62643 .09513 L
.62678 .09304 L
.6274 .09028 L
.62797 .08898 L
.62831 .08877 L
.62863 .08896 L
.62923 .09029 L
.62987 .09302 L
.63023 .09509 L
.63057 .09729 L
.63112 .09838 L
.63174 .09266 L
.63238 .08775 L
.63298 .08427 L
.63332 .08288 L
.63369 .08185 L
.63403 .08137 L
.63435 .08131 L
.63472 .08173 L
.63506 .08255 L
.63545 .08399 L
.63581 .08579 L
.63643 .08971 L
.63675 .09202 L
.63709 .09137 L
.63746 .08775 L
.63781 .08466 L
.63847 .07972 L
.63908 .07645 L
.63943 .07522 L
.63975 .0745 L
Mistroke
.64034 .07422 L
.64096 .07538 L
.64152 .07762 L
.64204 .08064 L
.64262 .0848 L
.64324 .08285 L
.64386 .07715 L
.64453 .07223 L
.64488 .07035 L
.64524 .06884 L
.64559 .0679 L
.6459 .06746 L
.64651 .06772 L
.64685 .06851 L
.64716 .06963 L
.64774 .07258 L
.64835 .07681 L
.64894 .07917 L
.64959 .07277 L
.65016 .06806 L
.65076 .06426 L
.65136 .06189 L
.65191 .06101 L
.65245 .06136 L
.65297 .06278 L
.65357 .0657 L
.65413 .06949 L
.65471 .07432 L
.65535 .0686 L
.656 .06287 L
.65636 .06031 L
.6567 .05831 L
.65734 .05576 L
.65793 .05496 L
.65845 .05546 L
.65901 .05728 L
.65961 .06047 L
.66016 .06452 L
.66068 .06901 L
.66116 .06454 L
.66167 .05958 L
.66223 .05522 L
.66279 .05193 L
.66332 .05004 L
.66394 .04933 L
.6645 .05013 L
.6651 .0524 L
.66574 .05633 L
.66634 .06124 L
.6669 .06165 L
Mistroke
.66749 .05561 L
.66806 .05087 L
.66865 .04712 L
.6693 .0447 L
.66986 .04409 L
.67046 .04499 L
.67098 .04696 L
.67154 .05033 L
.67216 .05521 L
.67247 .05806 L
.67281 .05739 L
.67318 .05345 L
.67352 .05012 L
.67386 .04721 L
.67417 .04484 L
.6748 .04133 L
.67537 .03961 L
.67598 .03937 L
.67633 .03999 L
.67665 .04101 L
.67723 .04401 L
.67779 .04805 L
.67844 .05387 L
.67903 .05012 L
.67968 .04389 L
.68002 .04116 L
.68038 .03875 L
.68073 .03698 L
.68104 .03583 L
.68165 .03485 L
.68198 .03504 L
.68233 .03579 L
.68296 .03846 L
.68331 .04064 L
.68364 .04308 L
.68402 .04636 L
.68438 .04983 L
.68469 .0493 L
.68503 .04559 L
.68563 .03989 L
.68596 .0372 L
.68632 .03482 L
.68696 .03191 L
.68751 .03089 L
.68812 .03143 L
.68876 .03378 L
.68935 .03748 L
.68991 .0421 L
.69041 .04707 L
.691 .0419 L
Mistroke
.69155 .03656 L
.69215 .03197 L
.69278 .02874 L
.69332 .02743 L
.69392 .02759 L
.69456 .0296 L
.69492 .03152 L
.69525 .03373 L
.69585 .03879 L
.69649 .04382 L
.69709 .03725 L
.69764 .03222 L
.69818 .02828 L
.69869 .02572 L
.69929 .02427 L
.69984 .02444 L
.70045 .02628 L
.70109 .02996 L
.70161 .03414 L
.70216 .03956 L
.70269 .03831 L
.70317 .03327 L
.70373 .02832 L
.70434 .02436 L
.70492 .02213 L
.70545 .02147 L
.70603 .0223 L
.70664 .02486 L
.70716 .02831 L
.70773 .03324 L
.70834 .03948 L
.70868 .03552 L
.709 .0322 L
.70961 .02656 L
.71018 .02267 L
.71071 .0203 L
.71127 .01925 L
.71184 .01974 L
.71246 .02202 L
.713 .02538 L
.71357 .03022 L
.71411 .03576 L
.71461 .03392 L
.71523 .02752 L
.71558 .02457 L
.7159 .02222 L
.71647 .01917 L
.71709 .01754 L
.71742 .01745 L
.71777 .01793 L
Mistroke
.71839 .02022 L
.71874 .02225 L
.71906 .0246 L
.71941 .02761 L
.71979 .03134 L
.72043 .03391 L
.72104 .02748 L
.72158 .02276 L
.72217 .01891 L
.72281 .01654 L
.72316 .01608 L
.72349 .0162 L
.72408 .01772 L
.72471 .02114 L
.72504 .02366 L
.72536 .02642 L
.72595 .03244 L
.72659 .03058 L
.72695 .02686 L
.72728 .0238 L
.72783 .01961 L
.72844 .01654 L
.72907 .01519 L
.72966 .01571 L
.73 .01676 L
.73036 .01851 L
.73071 .0207 L
.73103 .02317 L
.73167 .02925 L
.73227 .03298 L
.73292 .0261 L
.73329 .02273 L
.73363 .02011 L
.73396 .01802 L
.73432 .01629 L
.73469 .01519 L
.73507 .01472 L
.73539 .01487 L
.73573 .01561 L
.73636 .0184 L
.73697 .02278 L
.73753 .02808 L
.73814 .03371 L
.73849 .02982 L
.73881 .02648 L
.73947 .02082 L
.73983 .01838 L
.74017 .01667 L
.74077 .01492 L
.74133 .01488 L
Mistroke
.74195 .01662 L
.74229 .01836 L
.7426 .02039 L
.74317 .02515 L
.74379 .03155 L
.74438 .0308 L
.74491 .02538 L
.74553 .02034 L
.74612 .01705 L
.74665 .01546 L
.74722 .01528 L
.74785 .01691 L
.74816 .01843 L
.74851 .02062 L
.74912 .0257 L
.7497 .03164 L
.75027 .03207 L
.75082 .02651 L
.75132 .02233 L
.75185 .01896 L
.75242 .01673 L
.75295 .01606 L
.75358 .01708 L
.75416 .01965 L
.75538 .02972 L
.75599 .03577 L
.75634 .03196 L
.75666 .02869 L
.75734 .023 L
.75773 .0206 L
.75807 .01898 L
.75839 .01799 L
.75874 .01746 L
.75937 .01802 L
.75996 .02026 L
.7603 .02225 L
.76061 .02455 L
.7613 .03087 L
.76193 .03743 L
.76248 .03157 L
.76306 .02639 L
.76365 .02247 L
.76429 .01989 L
.76493 .01926 L
.76553 .02046 L
.76584 .02176 L
.76618 .0237 L
.76688 .02911 L
.76748 .03527 L
.76783 .03918 L
Mistroke
.76814 .03653 L
.76875 .03063 L
.76932 .02629 L
.76986 .02336 L
.77044 .02171 L
.77108 .02174 L
.77166 .02346 L
.77221 .02653 L
.77281 .03127 L
.77333 .03646 L
.77389 .041 L
.77453 .03459 L
.77513 .02976 L
.7755 .02749 L
.77584 .02585 L
.77615 .02484 L
.77649 .02426 L
.77682 .02421 L
.77718 .02475 L
.77756 .02598 L
.77792 .02773 L
.77855 .0322 L
.77924 .03869 L
.77983 .04383 L
.78017 .04037 L
.78049 .03743 L
.78112 .03245 L
.78148 .03034 L
.7818 .02888 L
.78238 .02744 L
.78301 .02765 L
.7836 .02954 L
.78415 .03268 L
.78477 .03767 L
.78535 .04358 L
.78587 .04607 L
.78645 .04064 L
.78707 .0359 L
.78738 .03405 L
.78772 .03247 L
.78833 .03098 L
.78891 .03114 L
.78945 .0327 L
.79002 .03576 L
.79051 .03944 L
.79105 .04445 L
.79162 .05078 L
.79224 .04569 L
.79278 .0413 L
.79336 .03778 L
Mistroke
.79368 .03637 L
.79403 .03534 L
.79466 .03491 L
.79501 .03546 L
.79539 .03669 L
.79575 .03843 L
.79608 .04049 L
.79671 .04562 L
.79704 .04888 L
.7974 .05269 L
.79774 .05399 L
.79805 .05106 L
.79865 .04607 L
.79927 .04222 L
.79984 .04004 L
.80047 .03927 L
.80079 .03955 L
.80113 .04037 L
.80172 .04295 L
.80234 .04727 L
.80293 .05263 L
.80347 .05837 L
.80405 .05507 L
.80459 .05074 L
.80519 .04711 L
.80583 .04473 L
.80646 .04411 L
.80678 .04447 L
.80713 .04539 L
.80775 .04829 L
.80833 .05238 L
.80891 .0577 L
.80947 .06359 L
.80996 .06023 L
.8105 .05599 L
.81109 .05243 L
.81171 .05008 L
.81224 .04933 L
.81282 .04991 L
.82299 .06376 L
.82358 .06169 L
.82413 .06098 L
.82476 .06171 L
.82533 .06378 L
.82597 .06758 L
.82665 .07323 L
.82729 .07964 L
.82788 .0765 L
.82845 .07262 L
.82905 .06954 L
Mistroke
.82962 .06784 L
.83014 .06739 L
.83072 .06817 L
.83128 .07017 L
.83187 .07364 L
.83252 .07876 L
.83316 .08496 L
.83353 .08517 L
.83387 .08256 L
.83456 .0782 L
.83495 .07639 L
.8353 .07517 L
.83593 .07418 L
.83629 .07429 L
.83662 .07484 L
.83722 .07695 L
.83787 .0807 L
.8385 .08568 L
.83884 .08875 L
.83919 .09225 L
.83984 .08916 L
.84043 .0855 L
.841 .08297 L
.84154 .08162 L
.84213 .08135 L
.84276 .08251 L
.85245 .09971 L
.85757 .11198 L
.85822 .1084 L
.85855 .10701 L
.85891 .10585 L
.85931 .10504 L
.85967 .10475 L
.86001 .10487 L
.86037 .10544 L
.86071 .10637 L
.86108 .10783 L
.86175 .11156 L
.8624 .11638 L
.863 .12167 L
.86358 .11958 L
.86412 .11682 L
.86471 .11461 L
.86534 .11335 L
.86566 .1132 L
.866 .1134 L
.86663 .11477 L
.86725 .11733 L
.86782 .12069 L
.86851 .12592 L
Mistroke
.86885 .12884 L
.86917 .12988 L
.86975 .12667 L
.87039 .12402 L
.87103 .1224 L
.87139 .122 L
.87172 .12199 L
.87208 .12236 L
.8724 .12304 L
.87278 .12423 L
.87313 .12574 L
.87426 .13285 L
.87484 .1375 L
.87547 .13643 L
.876 .13403 L
.87658 .13215 L
.87713 .13118 L
.87763 .13102 L
.87821 .13179 L
.87876 .13342 L
.87936 .13618 L
.88 .14015 L
.88033 .1425 L
.88068 .14526 L
.88132 .1457 L
.88194 .14307 L
.88251 .14135 L
.88312 .1404 L
.88371 .14044 L
.88423 .14129 L
.8848 .14309 L
.88542 .14601 L
.88608 .15012 L
.8867 .15475 L
.88726 .15472 L
.88784 .15247 L
.88837 .15095 L
.88894 .15002 L
.88956 .14995 L
.89009 .15069 L
.89058 .15206 L
.89113 .15427 L
.89171 .15741 L
.89234 .16161 L
.89269 .16418 L
.89301 .16479 L
.89337 .16338 L
.89375 .16204 L
.8941 .16106 L
.89443 .16035 L
Mistroke
.89508 .15968 L
.89567 .15995 L
.89631 .16121 L
.89667 .16236 L
.897 .16371 L
.89758 .16661 L
.89821 .17048 L
.89879 .17467 L
.89934 .17283 L
.89997 .17102 L
.90029 .17038 L
.90064 .16991 L
.90125 .16972 L
.90182 .17032 L
.90244 .17181 L
.90275 .1729 L
.9031 .17434 L
.90427 .18096 L
.9046 .18313 L
.90494 .18369 L
.90531 .18251 L
.90565 .18159 L
.90627 .18041 L
.90662 .18004 L
.90694 .17992 L
.90757 .18031 L
.90817 .18147 L
.90871 .18315 L
.90929 .18562 L
.90992 .18898 L
.91027 .19108 L
.91059 .19316 L
.91116 .19271 L
.91179 .1913 L
.91235 .19054 L
.91297 .19035 L
.91354 .19083 L
.91408 .19186 L
.91461 .19343 L
.91509 .19531 L
.91617 .20086 L
.91678 .20359 L
.91734 .20232 L
.91795 .20134 L
.9186 .20092 L
.91916 .20112 L
.91969 .20181 L
.92026 .20314 L
.92088 .2052 L
.92214 .21117 L
Mistroke
.92277 .2136 L
.92314 .21291 L
.92348 .21237 L
.92408 .21177 L
.92463 .21165 L
.92524 .21206 L
.92587 .21313 L
.9313 .22318 L
.94116 .24447 L
.95985 .27801 L
.97619 .30902 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True,
  ImageSize->{215, 132.812},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004<0000YQ000`40O003h00Oogoo3Goo0026Ool0
0`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23
Ool008Ioo`8003Qoo`L00003Ool000000080041oo`00QWoo00<007ooOol0=7oo4P00?Goo001LOol3
0003Ool01@00Oomoogoo00001Woo1@000goo0`003Woo00<007ooOol0<Woo500000=oo`00Ool0>Woo
001KOol01@00Oomoogoo00000goo00<007oo00002Goo00<007ooOol00Woo00D007ooOomoo`0000eo
o`03001oogoo031oo`X00003Ool0000000d003Uoo`00Dgoo1P000Woo00D007ooOomoo`0000Aoo`03
001oogoo00Qoo`03001oogoo009oo`05001oogooOol0000=Ool00`00Oomoo`0_Ool8000017oo0000
07oo100000Aoo`00001oo`P003Qoo`00Fgoo00D007ooOomoo`0000=oo`03001oo`0000Uoo`03001o
ogoo009oo`05001oogooOol0000=Ool00`00Oomoo`0]Ool:00000goo001oo`02Ool00`00Oomoo`02
Ool01@00Ool00000Ool01`00=Woo001KOol40003Ool01@00Oomoogoo000027oo00<007ooOol00Woo
00D007ooOomoo`0000eoo`<002aoo`H00003Ool0000000ioo`800003Ool0000000H003Eoo`00Fgoo
00<007ooOol04Goo00<007ooOol00Woo00D007ooOomoo`0000eoo`03001oogoo02Yoo`03001oo`00
00800003Ool0000001Aoo`05001oo`00001oo`0300000goo001oo`0bOol005aoo`03001oogoo011o
o`03001oogoo009oo`05001oogooOol00008Ool00`00Oomoo`02Ool00`00Oomoo`0ZOol500000goo
001oo`0GOol00`00Ool00004000cOol005eoo`<000eoo`<000Eoo`<000Uoo`03001oogoo009oo`03
001oogoo02Qoo`L001aoo`800004Ool00000000bOol007]oo`@000=oo`05001oogooOol0000YOol5
00000goo001oo`0MOol00`00Ool00002000bOol0089oo`05001oogooOol0000YOol5000ROol4000a
Ool008=oo`03001oogoo008002Ioo`D002Eoo`800004Ool00000000^Ool0081oo`@0009oo`03001o
ogoo02Eoo`@002Ioo`04001oogooOol2000^Ool008Ioo`03001oogoo02Aoo`<00003Ool007oo02Yo
o`8002eoo`00QWoo00<007ooOol08Woo1@00;Goo0P0000=oo`00Ool0:Woo0026Ool00`00Oomoo`0R
Ool200000goo001oo`0]Ool4000[Ool008Ioo`03001oogoo025oo`<00003Ool007oo02eoo`03001o
o`00008002Yoo`00QWoo0P008Goo1000<Woo0`00:Woo0026Ool00`00Oomoo`0OOol4000dOol2000Z
Ool008Ioo`03001oogoo01moo`@003Aoo`03001oo`0002Uoo`00QWoo00<007ooOol07Woo0`00>Goo
00<007ooOol09Woo0026Ool00`00Oomoo`0MOol4000jOol00`00Oomoo`0UOol008Ioo`03001oogoo
01eoo`8003aoo`03001oogoo02Eoo`00QWoo0P007Goo0`00?Goo0P009Goo0026Ool00`00Oomoo`0K
Ool3000nOol2000UOol005eoo`<0009oo`05001oogooOol00006Ool50003Ool3000>Ool00`00Oomo
o`0JOol200000goo001oo`0nOol3000SOol005ioo`03001oogoo009oo`03001oo`0000Uoo`03001o
ogoo009oo`05001oogooOol0000=Ool00`00Oomoo`0JOol20012Ool2000SOol005=oo`H0009oo`D0
00Aoo`03001oogoo00Qoo`03001oogoo009oo`05001oogooOol0000=Ool00`00Oomoo`0JOol20012
Ool3000ROol005]oo`04001oogoo0004Ool00`00Ool00009Ool00`00Oomoo`02Ool01@00Oomoogoo
00003Goo00<007ooOol06Goo00<007ooOol0@goo0P008Woo001LOol00`00Ool00003Ool01@00Oomo
ogoo000027oo00<007ooOol00Woo00D007ooOomoo`0000eoo`<001Uoo`03001oogoo04=oo`80029o
o`00G7oo00<007oo000047oo00<007ooOol00Woo00D007ooOomoo`0000eoo`03001oogoo01Qoo`03
001oogoo04Ioo`80021oo`00GGoo0P0047oo00<007ooOol00Woo00D007ooOomoo`0000Qoo`03001o
ogoo009oo`03001oogoo01Qoo`03001oogoo04Ioo`80021oo`00GWoo00<007ooOol037oo0`001Goo
0`002Goo00<007ooOol00Woo00<007ooOol05goo00<007ooOol0B7oo0P007goo001kOol40003Ool0
1@00Oomoogoo00006Goo00<007ooOol0B7oo0`007Woo0022Ool01@00Oomoogoo000067oo00<007oo
Ool0Bgoo00<007ooOol077oo0023Ool00`00Oomoo`02000GOol00`00Oomoo`1;Ool2000MOol0081o
o`@0009oo`03001oogoo01Eoo`03001oogoo04aoo`8001eoo`00QWoo00<007ooOol05Goo00<007oo
Ool0CGoo0P0077oo0026Ool00`00Oomoo`0DOol00`00Oomoo`1?Ool2000KOol008Ioo`03001oogoo
01=oo`03001oogoo051oo`8001]oo`00QWoo00<007ooOol04Woo00<007ooOol0DWoo0P006Woo0026
Ool2000COol00`00Oomoo`1BOol2000JOol008Ioo`03001oogoo015oo`03001oogoo05Aoo`03001o
ogoo01Qoo`00QWoo00<007ooOol047oo00<007ooOol0EWoo0P0067oo0026Ool00`00Oomoo`0@Ool0
0`00Oomoo`1FOol2000HOol008Ioo`03001oogoo00moo`03001oogoo05Qoo`8001Moo`00QWoo00<0
07ooOol03Woo00<007ooOol0FGoo0P005goo0026Ool2000>Ool00`00Oomoo`1LOol00`00Oomoo`0D
Ool008Ioo`03001oogoo00eoo`03001oogoo05aoo`8001Eoo`00Fgoo1@000Woo00D007ooOomoo`00
00Ioo`D000=oo`<000ioo`03001oogoo00aoo`03001oogoo05eoo`8001Eoo`00G7oo00@007ooOol0
00=oo`03001oo`0000Uoo`03001oogoo009oo`05001oogooOol0000=Ool00`00Oomoo`0;Ool00`00
Oomoo`1OOol00`00Oomoo`0COol005=oo`H000Aoo`03001oogoo00Aoo`03001oogoo00Qoo`03001o
ogoo009oo`05001oogooOol0000=Ool00`00Oomoo`0:Ool00`00Oomoo`1POol2000DOol005ioo`03
001oogoo009oo`03001oo`0000Uoo`03001oogoo009oo`05001oogooOol0000=Ool00`00Oomoo`0:
Ool00`00Oomoo`1QOol00`00Oomoo`0BOol005moo`04001oogoo0003Ool00`00Oomoo`06Ool00`00
Oomoo`02Ool01@00Oomoogoo00003Goo0`002Goo00<007ooOol0Hgoo00<007ooOol04Goo001OOol0
0`00Oomoo`0=Ool00`00Oomoo`02Ool01@00Oomoogoo00003Goo00<007ooOol02Goo00<007ooOol0
I7oo00<007ooOol047oo001KOol01@00Oomoogoo00003goo00<007ooOol00Woo00D007ooOomoo`00
00Qoo`03001oogoo009oo`03001oogoo00Qoo`03001oogoo06Eoo`03001oogoo011oo`00G7oo0`00
3Woo0`001Goo0`002Goo00<007ooOol00Woo00<007ooOol027oo00<007ooOol0IWoo00<007ooOol0
3goo001kOol40003Ool01@00Oomoogoo00002Goo00<007ooOol0Igoo00<007ooOol03goo0022Ool0
1@00Oomoogoo00002Goo00<007ooOol0J7oo00<007ooOol03Woo0023Ool00`00Oomoo`020008Ool0
0`00Oomoo`1XOol00`00Oomoo`0>Ool0081oo`@0009oo`03001oogoo00Ioo`03001oogoo06Yoo`03
001oogoo00eoo`00QWoo00<007ooOol01Woo00<007ooOol0JWoo00<007ooOol03Goo0026Ool00`00
Oomoo`05Ool00`00Oomoo`1/Ool00`00Oomoo`0<Ool008Ioo`03001oogoo00Eoo`03001oogoo06ao
o`03001oogoo00aoo`00QWoo00<007ooOol017oo00<007ooOol0KWoo00<007ooOol02goo0008Ool5
000lOol5000hOol20005Ool00`00Oomoo`0dOol5000fOol00`00Oomoo`050005Ool000Uoo`04001o
ogoo000nOol00`00Oomoo`0hOol00`00Oomoo`03Ool00`00Oomoo`0gOol00`00Oomoo`0fOol01@00
Oomoogoo00000Woo00<007ooOol00goo00060004Ool00`00Oomoo`0cOol60005Ool00`00Oomoo`0h
Ool00`00Oomoo`02Ool00`00Oomoo`0hOol00`00Oomoo`0gOol01@00Oomoogoo00001goo000;Ool0
0`00Oomoo`0mOol00`00Oomoo`0hOol00`00Oomoo`02Ool00`00Oomoo`0hOol00`00Oomoo`0gOol0
0`00Oomoo`02Ool00`00Oomoo`04Ool000aoo`03001oogoo03aoo`03001oogoo03Qoo`05001oogoo
Ool0000kOol00`00Oomoo`0hOol00`00Oomoo`02Ool00`00Oomoo`03Ool000aoo`03001oogoo03ao
o`03001oogoo03Qoo`05001oogooOol0000kOol00`00Oomoo`0hOol00`00Oomoo`02Ool00`00Oomo
o`03Ool000Qoo`05001oogooOol0000nOol00`00Oomoo`0hOol200000goo001oo`0kOol00`00Oomo
o`0iOol01@00Oomoogoo00001Goo0009Ool3000mOol3000jOol01000Oomoo`00>Woo0`00>goo1000
1Woo0026Ool00`00Ool0001jOol00`00Oomoo`06Ool008Ioo`03001oo`0007Yoo`03001oogoo00Io
o`00QWoo0P00O7oo00<007ooOol01Goo0026Ool2001lOol00`00Oomoo`05Ool00003Ool000000?l0
00X000001goo00<007ooOol02Woo00<007ooOol02Goo00<007ooOol02Woo00<007ooOol02Woo00<0
07ooOol02Goo00<007ooOol02Woo00<007ooOol02Woo00<007ooOol02Woo00<007ooOol02Goo00<0
07ooOol02Woo00<007ooOol02Woo00<007ooOol02Goo00<007ooOol02Woo00<007ooOol02Woo00<0
07ooOol02Goo00<007ooOol02Woo00<007ooOol02Woo00<007ooOol02Woo00<007ooOol02Goo00<0
07ooOol02Woo00<007ooOol017oo0007Ool2000mOol00`00Oomoo`0lOol2000nOol00`00Oomoo`0m
Ool00`00Oomoo`04Ool000Qoo`03001oogoo07Yoo`8008Eoo`002Goo00<007ooOol0N7oo00<007oo
0000QGoo0009Ool00`00Oomoo`1hOol00`00Ool00025Ool000Yoo`03001oogoo07Ioo`03001oogoo
008008Aoo`002Woo00<007ooOol0MWoo00@007ooOol008Eoo`002goo00<007ooOol0M7oo00D007oo
Oomoo`0008Eoo`002goo00<007ooOol0M7oo00D007ooOomoo`0008Eoo`0037oo00<007ooOol0LWoo
00<007ooOol00Woo00<007ooOol0Pgoo000<Ool00`00Oomoo`1bOol00`00Oomoo`02Ool00`00Oomo
o`23Ool000eoo`03001oogoo071oo`03001oogoo00=oo`8008Aoo`003Woo00<007ooOol0Kgoo00<0
07ooOol00goo00<007ooOol0Pgoo000>Ool00`00Oomoo`1^Ool00`00Oomoo`04Ool00`00Oomoo`23
Ool000moo`03001oogoo06eoo`03001oogoo00Aoo`03001oogoo08=oo`003goo00<007ooOol0K7oo
00<007ooOol01Goo00<007ooOol0Pgoo000@Ool00`00Oomoo`1[Ool00`00Oomoo`05Ool00`00Oomo
o`23Ool0015oo`03001oogoo06Uoo`03001oogoo00Ioo`8008Aoo`004Goo00<007ooOol0J7oo00<0
07ooOol01goo00<007ooOol0Pgoo000BOol00`00Oomoo`15Ool50003Ool01@00Oomoogoo00001Goo
1@000goo0`001Goo00<007ooOol01goo00<007ooOol0Pgoo000BOol00`00Oomoo`16Ool01000Oomo
o`0017oo00<007oo000027oo00<007ooOol00Woo00D007ooOomoo`0000=oo`03001oogoo00Qoo`03
001oogoo08=oo`004goo00<007ooOol0AWoo00<007ooOol01Goo00<007ooOol01goo00<007ooOol0
0Woo00D007ooOomoo`0000=oo`03001oogoo00Qoo`03001oogoo08=oo`0057oo00<007ooOol0AWoo
00<007ooOol00goo00<007oo000027oo00<007ooOol00Woo00D007ooOomoo`00009oo`03001oogoo
00Uoo`03001oogoo08=oo`0057oo00<007ooOol0Agoo00D007ooOomoo`0000=oo`03001oogoo00Eo
o`03001oogoo009oo`05001oogooOol00002Ool00`00Oomoo`09Ool30023Ool001Eoo`03001oogoo
04Ioo`03001oogoo00eoo`03001oogoo009oo`07001oogooOol007oo000037oo00<007ooOol0Pgoo
000EOol00`00Oomoo`12Ool01@00Oomoogoo00003goo00<007ooOol00Woo00@007ooOomoo`8000Qo
o`03001oogoo009oo`03001oogoo08=oo`005Woo00<007ooOol0@Woo0`003Woo0`001Goo0`0000=o
o`00Ool01goo00<007ooOol00Woo00<007ooOol0Pgoo000GOol00`00Oomoo`1MOol01000Oomoogoo
10000goo00D007ooOomoo`0008Eoo`005goo00<007ooOol0G7oo00<007ooOol02Goo00D007ooOomo
o`0008Eoo`0067oo00<007ooOol0Fgoo00<007ooOol02Woo00<007ooOol00P00Q7oo000IOol00`00
Oomoo`1IOol00`00Oomoo`08Ool40002Ool00`00Oomoo`23Ool001Uoo`03001oogoo05Qoo`03001o
ogoo00moo`03001oogoo08=oo`006Woo00<007ooOol0Egoo00<007ooOol03goo00<007ooOol0Pgoo
000JOol00`00Oomoo`1FOol00`00Oomoo`0@Ool00`00Oomoo`23Ool001]oo`03001oogoo05Eoo`03
001oogoo011oo`03001oogoo08=oo`0077oo00<007ooOol0Dgoo00<007ooOol04Goo0P00Q7oo000L
Ool00`00Oomoo`1BOol00`00Oomoo`0BOol00`00Oomoo`23Ool001eoo`03001oogoo055oo`03001o
ogoo019oo`03001oogoo08=oo`007Goo00<007ooOol0D7oo00<007ooOol04goo00<007ooOol0Pgoo
000NOol00`00Oomoo`1?Ool00`00Oomoo`0COol00`00Oomoo`23Ool001ioo`03001oogoo04ioo`03
001oogoo01Aoo`03001oogoo08=oo`007goo00<007ooOol0CGoo00<007ooOol057oo0P00Q7oo000O
Ool00`00Oomoo`1<Ool00`00Oomoo`0EOol00`00Oomoo`23Ool001moo`03001oogoo03Yoo`<000=o
o`05001oogooOol00005Ool50003Ool3000?Ool00`00Oomoo`23Ool0021oo`03001oogoo03Yoo`03
001oogoo00=oo`03001oo`0000Moo`8000Aoo`05001oogooOol0000>Ool00`00Oomoo`23Ool0021o
o`03001oogoo03Moo`D000Eoo`03001oogoo00Eoo`03001oo`0000Aoo`05001oogooOol0000>Ool0
0`00Oomoo`23Ool0025oo`03001oogoo03Ioo`04001oogoo0005Ool00`00Ool00006Ool00`00Ool0
0004Ool01@00Oomoogoo00003Woo00<007ooOol0Pgoo000QOol00`00Oomoo`0gOol00`00Ool00004
Ool01@00Oomoogoo000017oo00@007ooOol000Aoo`05001oogooOol0000>Ool30023Ool0029oo`03
001oogoo03Ioo`03001oo`0000eoo`04001oogoo0004Ool01@00Oomoogoo00003Woo00<007ooOol0
Pgoo000SOol00`00Oomoo`0fOol2000:Ool00`00Ool00003Ool00`00Oomoo`02Ool01@00Oomoogoo
00002Goo00<007ooOol00Woo00<007ooOol0Pgoo000TOol00`00Oomoo`0fOol00`00Oomoo`08Ool0
1000Ool007oo0`001Goo0`002Woo00<007ooOol00Woo00<007ooOol0Pgoo000TOol00`00Oomoo`0o
Ool01000Ool000004Goo10000goo00D007ooOomoo`0008Eoo`009Goo00<007ooOol0?Woo0P006Woo
00D007ooOomoo`0008Eoo`009Woo00<007oo0000?7oo0`006goo00<007ooOol00P00Q7oo000VOol0
0`00Ool0000lOol2000IOol40002Ool00`00Oomoo`23Ool002Moo`800003Ool007oo03Moo`@001mo
o`03001oogoo08=oo`009goo1000>7oo0`0087oo00<007ooOol0Pgoo000YOol3000eOol4000QOol0
0`00Oomoo`23Ool002Uoo`D0039oo`D0025oo`03001oogoo08=oo`00:goo0`00<Woo0`008goo0P00
Q7oo000[Ool3000`Ool00`00Ool00002000SOol00`00Oomoo`23Ool002aoo`80031oo`04001oo`00
000TOol00`00Oomoo`23Ool002aoo`03001oo`0002ioo`<00003Ool007oo02=oo`03001oogoo08=o
o`00;Woo0P00:goo00<007oo00000P009Woo00<007ooOol0Pgoo000`Ool00`00Ool0000XOol01000
Ool000009goo00<007ooOol0Pgoo000aOol3000UOol6000WOol20024Ool0035oo`D0029oo`D002Uo
o`03001oogoo08=oo`00<Goo1@0000=oo`00Ool07Goo00<007oo00001@0017oo00D007ooOomoo`00
00Eoo`D000=oo`<000moo`03001oogoo08=oo`00<Woo1`0077oo0P0000=oo`0000000P000Woo00<0
07ooOol00Woo00<007oo000027oo00<007ooOol00Woo00D007ooOomoo`0000ioo`03001oogoo08=o
o`00=7oo1@0000=oo`00Ool05goo00<007oo00001@000goo00<007ooOol00goo00<007ooOol01goo
00<007ooOol00Woo00D007ooOomoo`0000ioo`03001oogoo08=oo`00=7oo00<007oo00000P0000Eo
o`00001oo`0001Aoo`800005Ool00000001oo`020003Ool00`00Oomoo`02Ool00`00Ool00008Ool0
0`00Oomoo`02Ool01@00Oomoogoo00003Woo00<007ooOol0Pgoo000fOol6000017oo001oo`003Goo
0P0000=oo`0000001P000Woo100017oo00D007ooOomoo`0000Moo`03001oogoo009oo`05001oogoo
Ool0000>Ool30023Ool003Moo`L00004Ool00000Ool20007Ool00`00Ool000090003Ool00`00Oomo
o`0AOol00`00Oomoo`02Ool01@00Oomoogoo00003Woo00<007ooOol0Pgoo000iOol8000017oo0000
07oo0P0000=oo`00Ool00P0000=oo`0000001P001Woo00<007ooOol047oo00<007ooOol00Woo00D0
07ooOomoo`0000Uoo`03001oogoo009oo`03001oogoo08=oo`00>Woo1`0000Aoo`00001oo`L00003
Ool0000000D000Qoo`<000eoo`<000Eoo`<000Yoo`03001oogoo009oo`03001oogoo08=oo`00?7oo
00<007oo00004@0000=oo`00Ool0:7oo10000goo00D007ooOomoo`0008Eoo`00?Woo4P00<Woo00D0
07ooOomoo`0008Eoo`00@Goo1`0000=oo`0000000P00=Woo00<007ooOol00P00Q7oo0020Ool40002
Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomo
o`23Ool00001\
\>"],
  ImageRangeCache->{{{0, 267.75}, {165, 0}} -> {-2.11818, -6.86424*^-7, 
  0.0196928, 1.04001*^-8}}]
}, Open  ]],

Cell[TextData[{
  "Finally we tabulate the ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "-value, the numerical result, the exact result and the error in the \
numerical scheme."
}], "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell["\<\
samples = TableForm[Join[{{\"x\", \"Exact\", \"Douglas\", \
\"Error\"}},
Table[Map[PaddedForm[N[Chop[#1]],{5,6}]&,
N[{x, ufunc[x], Sin[Pi*x/2]*Exp[-(Pi^2 0.1)/4], 
ufunc[x]- Sin[Pi*x/2]*Exp[-(Pi^2 0.1)/4]},5]],
{x, -2, 2, 0.25}]]]\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"x\"\>", "\<\"Exact\"\>", "\<\"Douglas\"\>", "\<\"Error\"\>"},
          {
            TagBox[
              InterpretationBox["\<\"-2.000000\"\>",
                -2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.750000\"\>",
                -1.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299010\"\>",
                -0.29900706745521188,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299010\"\>",
                -0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\(" 2.332100"\[Times]10\^"-7"\),
                2.3320762793099803*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.500000\"\>",
                -1.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552490\"\>",
                -0.55249301939618367,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552490\"\>",
                -0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\(" 4.309100"\[Times]10\^"-7"\),
                4.3091150869756234*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.250000\"\>",
                -1.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721870\"\>",
                -0.72186691749577858,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721870\"\>",
                -0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\(" 5.630100"\[Times]10\^"-7"\),
                5.6301301898376011*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.000000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.781340\"\>",
                -0.78134312114654381,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.781340\"\>",
                -0.78134373054744422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\(" 6.094000"\[Times]10\^"-7"\),
                6.0940090040872974*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.750000\"\>",
                -0.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721870\"\>",
                -0.72186691749577858,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721870\"\>",
                -0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\(" 5.630100"\[Times]10\^"-7"\),
                5.6301301898376011*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.500000\"\>",
                -0.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552490\"\>",
                -0.55249301939618278,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552490\"\>",
                -0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\(" 4.309100"\[Times]10\^"-7"\),
                4.3091150958574076*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.250000\"\>",
                -0.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299010\"\>",
                -0.29900706745521144,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299010\"\>",
                -0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\(" 2.332100"\[Times]10\^"-7"\),
                2.3320762837508724*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.250000\"\>",
                0.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299010\"\>",
                0.29900706745521077,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299010\"\>",
                0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\("-2.332100"\[Times]10\^"-7"\),
                -2.3320762904122105*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.500000\"\>",
                0.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552490\"\>",
                0.55249301939618234,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552490\"\>",
                0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\("-4.309100"\[Times]10\^"-7"\),
                -4.3091151002982997*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.750000\"\>",
                0.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721870\"\>",
                0.7218669174957778,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721870\"\>",
                0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\("-5.630100"\[Times]10\^"-7"\),
                -5.6301301976091622*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.000000\"\>",
                1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.781340\"\>",
                0.78134312114654292,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.781340\"\>",
                0.78134373054744422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\("-6.094000"\[Times]10\^"-7"\),
                -6.0940090129690816*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.250000\"\>",
                1.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721870\"\>",
                0.72186691749577847,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721870\"\>",
                0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\("-5.630100"\[Times]10\^"-7"\),
                -5.6301301909478241*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.500000\"\>",
                1.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552490\"\>",
                0.55249301939618312,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552490\"\>",
                0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\("-4.309100"\[Times]10\^"-7"\),
                -4.3091150925267385*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.750000\"\>",
                1.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299010\"\>",
                0.29900706745521138,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299010\"\>",
                0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox[\("-2.332100"\[Times]10\^"-7"\),
                -2.3320762843059839*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 2.000000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  PageBreakAbove->False,
  FontSize->8]
}, Open  ]],

Cell[TextData[{
  StyleBox["Note that we obtain errors of about ",
    FontSlant->"Italic",
    FontVariations->{"Underline"->False}],
  StyleBox["1/40",
    FontVariations->{"Underline"->False,
    "CompatibilityType"->0}],
  StyleBox[
  " of those obtained with Crank-Nicholson, with identical time-step \
parameters",
    FontSlant->"Italic",
    FontVariations->{"Underline"->False}],
  StyleBox[
  ". Some oscillations have started to appear in our error plot, though it \
should be appreciated that our plot is now given on a much finer scale than \
was the case for CN - if we plot the Douglas errors on the same scale as the \
CN error plot the difference is clearer:",
    FontVariations->{"Underline"->False}]
}], "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True,
  FontVariations->{"Underline"->True}],

Cell[CellGroupData[{

Cell["\<\
Plot[ufunc[x] - Sin[Pi*x/2]*Exp[-Pi^2 0.1/4], {x, -2, 2}, 
PlotPoints -> 50, PlotRange -> {-0.000025, 0.000025}];\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.238095 0.309017 12360.7 [
[.02381 .29652 -6 -9 ]
[.02381 .29652 6 0 ]
[.2619 .29652 -6 -9 ]
[.2619 .29652 6 0 ]
[.7381 .29652 -3 -9 ]
[.7381 .29652 3 0 ]
[.97619 .29652 -3 -9 ]
[.97619 .29652 3 0 ]
[.4875 .0618 -48 -4.5 ]
[.4875 .0618 0 4.5 ]
[.4875 .18541 -48 -4.5 ]
[.4875 .18541 0 4.5 ]
[.4875 .43262 -42 -4.5 ]
[.4875 .43262 0 4.5 ]
[.4875 .55623 -42 -4.5 ]
[.4875 .55623 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .30902 m
.02381 .31527 L
s
[(-2)] .02381 .29652 0 1 Mshowa
.2619 .30902 m
.2619 .31527 L
s
[(-1)] .2619 .29652 0 1 Mshowa
.7381 .30902 m
.7381 .31527 L
s
[(1)] .7381 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(2)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.07143 .30902 m
.07143 .31277 L
s
.11905 .30902 m
.11905 .31277 L
s
.16667 .30902 m
.16667 .31277 L
s
.21429 .30902 m
.21429 .31277 L
s
.30952 .30902 m
.30952 .31277 L
s
.35714 .30902 m
.35714 .31277 L
s
.40476 .30902 m
.40476 .31277 L
s
.45238 .30902 m
.45238 .31277 L
s
.54762 .30902 m
.54762 .31277 L
s
.59524 .30902 m
.59524 .31277 L
s
.64286 .30902 m
.64286 .31277 L
s
.69048 .30902 m
.69048 .31277 L
s
.78571 .30902 m
.78571 .31277 L
s
.83333 .30902 m
.83333 .31277 L
s
.88095 .30902 m
.88095 .31277 L
s
.92857 .30902 m
.92857 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.5 .0618 m
.50625 .0618 L
s
[(-0.00002)] .4875 .0618 1 0 Mshowa
.5 .18541 m
.50625 .18541 L
s
[(-0.00001)] .4875 .18541 1 0 Mshowa
.5 .43262 m
.50625 .43262 L
s
[(0.00001)] .4875 .43262 1 0 Mshowa
.5 .55623 m
.50625 .55623 L
s
[(0.00002)] .4875 .55623 1 0 Mshowa
.125 Mabswid
.5 .08652 m
.50375 .08652 L
s
.5 .11125 m
.50375 .11125 L
s
.5 .13597 m
.50375 .13597 L
s
.5 .16069 m
.50375 .16069 L
s
.5 .21013 m
.50375 .21013 L
s
.5 .23485 m
.50375 .23485 L
s
.5 .25957 m
.50375 .25957 L
s
.5 .2843 m
.50375 .2843 L
s
.5 .33374 m
.50375 .33374 L
s
.5 .35846 m
.50375 .35846 L
s
.5 .38318 m
.50375 .38318 L
s
.5 .4079 m
.50375 .4079 L
s
.5 .45735 m
.50375 .45735 L
s
.5 .48207 m
.50375 .48207 L
s
.5 .50679 m
.50375 .50679 L
s
.5 .53151 m
.50375 .53151 L
s
.5 .03708 m
.50375 .03708 L
s
.5 .01236 m
.50375 .01236 L
s
.5 .58095 m
.50375 .58095 L
s
.5 .60567 m
.50375 .60567 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30902 m
.04273 .31 L
.06337 .31108 L
.08275 .31194 L
.10139 .31274 L
.12123 .31383 L
.14033 .31462 L
.14534 .31481 L
.14567 .31483 L
.14599 .31484 L
.14636 .31484 L
.1467 .31483 L
.14731 .31478 L
.14765 .31474 L
.14797 .3147 L
.14862 .31459 L
.14898 .31459 L
.1493 .31468 L
.15007 .31485 L
.1504 .31491 L
.15077 .31496 L
.15138 .31503 L
.15172 .31505 L
.15204 .31506 L
.15238 .31505 L
.1527 .31504 L
.15306 .31501 L
.15341 .31497 L
.15403 .31488 L
.1546 .31478 L
.15523 .31487 L
.15555 .31495 L
.1559 .31503 L
.15647 .31513 L
.15707 .31521 L
.15739 .31524 L
.15773 .31526 L
.15834 .31526 L
.15891 .31522 L
.15953 .31515 L
.16011 .31506 L
.16063 .31495 L
.16122 .31507 L
.16176 .3152 L
.16236 .31532 L
.16301 .31541 L
.16333 .31543 L
.16368 .31545 L
.16432 .31545 L
.16494 .31541 L
Mistroke
.16552 .31533 L
.16611 .31523 L
.16667 .31511 L
.16717 .31525 L
.1677 .31538 L
.16829 .3155 L
.16892 .31559 L
.16945 .31563 L
.17003 .31564 L
.18021 .31584 L
.18085 .31593 L
.18145 .31598 L
.18208 .31598 L
.18241 .31596 L
.18277 .31593 L
.18314 .31588 L
.18352 .31581 L
.18388 .31574 L
.18421 .31567 L
.18485 .31569 L
.18519 .31578 L
.18555 .31587 L
.1862 .31601 L
.18657 .31606 L
.18691 .3161 L
.18753 .31614 L
.18785 .31614 L
.18819 .31612 L
.18879 .31607 L
.18943 .31597 L
.18999 .31585 L
.19059 .31576 L
.19122 .31594 L
.19181 .31608 L
.19243 .31619 L
.19279 .31624 L
.19313 .31627 L
.19373 .31628 L
.19428 .31626 L
.19489 .31619 L
.19553 .31607 L
.19607 .31595 L
.19665 .31593 L
.19719 .31608 L
.19768 .3162 L
.19827 .31631 L
.1989 .31639 L
.19949 .31641 L
.20004 .3164 L
.20061 .31635 L
Mistroke
.20114 .31627 L
.20174 .31614 L
.20232 .31599 L
.20285 .31612 L
.20334 .31625 L
.20387 .31637 L
.20445 .31646 L
.20509 .31652 L
.20568 .31653 L
.20627 .3165 L
.2068 .31643 L
.20738 .31632 L
.20802 .31617 L
.20834 .31609 L
.20868 .31619 L
.20931 .31636 L
.20962 .31644 L
.20996 .31651 L
.21057 .3166 L
.21091 .31663 L
.21122 .31664 L
.21157 .31664 L
.21194 .31663 L
.21258 .31656 L
.2129 .31651 L
.21325 .31644 L
.21387 .3163 L
.21444 .31623 L
.21505 .31641 L
.21562 .31655 L
.21622 .31666 L
.21656 .3167 L
.21687 .31672 L
.21739 .31674 L
.21796 .31672 L
.21856 .31665 L
.21912 .31655 L
.21974 .31641 L
.22008 .31631 L
.22039 .31632 L
.221 .31649 L
.22156 .31663 L
.22189 .3167 L
.22223 .31675 L
.22284 .31681 L
.22319 .31683 L
.2235 .31682 L
.22385 .31681 L
.22423 .31677 L
.22485 .31668 L
Mistroke
.2252 .3166 L
.22553 .31653 L
.22613 .31636 L
.22676 .31652 L
.22729 .31666 L
.22786 .31678 L
.22848 .31686 L
.22907 .3169 L
.22967 .31689 L
.23023 .31684 L
.23084 .31674 L
.2315 .31659 L
.23212 .31641 L
.23271 .31658 L
.23324 .31672 L
.23381 .31684 L
.23433 .31691 L
.23489 .31696 L
.23541 .31696 L
.23589 .31693 L
.23646 .31686 L
.23698 .31676 L
.23758 .31661 L
.23815 .31647 L
.23869 .31664 L
.23927 .31679 L
.23991 .31691 L
.2405 .31699 L
.24108 .31701 L
.24172 .31699 L
.24204 .31696 L
.24238 .31691 L
.243 .31679 L
.24358 .31664 L
.24421 .31655 L
.24481 .31673 L
.24535 .31686 L
.24585 .31695 L
.24639 .31702 L
.24696 .31705 L
.24749 .31704 L
.24812 .31697 L
.24848 .31692 L
.24881 .31685 L
.24946 .31669 L
.25005 .31654 L
.25064 .31672 L
.25119 .31686 L
.25177 .31697 L
.25239 .31705 L
Mistroke
.25303 .31707 L
.25339 .31706 L
.25373 .31704 L
.25408 .317 L
.25441 .31694 L
.25514 .31678 L
.25576 .3166 L
.25611 .31659 L
.25643 .31669 L
.25703 .31685 L
.25766 .31698 L
.25823 .31705 L
.25885 .31709 L
.25941 .31707 L
.25993 .31702 L
.26059 .31691 L
.26122 .31675 L
.26178 .31659 L
.26238 .3167 L
.26292 .31684 L
.26343 .31695 L
.26395 .31703 L
.26453 .31708 L
.2651 .31708 L
.26572 .31704 L
.26629 .31695 L
.26681 .31684 L
.26744 .31667 L
.26775 .31658 L
.2681 .31662 L
.26872 .31679 L
.2693 .31692 L
.26962 .31698 L
.26997 .31703 L
.27059 .31707 L
.27094 .31707 L
.27126 .31706 L
.27162 .31703 L
.272 .31698 L
.27263 .31686 L
.27297 .31678 L
.27332 .31668 L
.27363 .31658 L
.27397 .31658 L
.27457 .31675 L
.27515 .31688 L
.27578 .31699 L
.27637 .31704 L
.27691 .31705 L
.27754 .31701 L
Mistroke
.27814 .31692 L
.27867 .31681 L
.27925 .31666 L
.27988 .31653 L
.28019 .31663 L
.28054 .31672 L
.28117 .31686 L
.28173 .31695 L
.28231 .317 L
.28285 .31701 L
.28343 .31698 L
.28405 .31689 L
.28459 .31678 L
.28517 .31663 L
.28571 .31646 L
.28621 .3166 L
.28679 .31675 L
.28743 .31687 L
.28777 .31691 L
.28808 .31694 L
.28841 .31696 L
.28877 .31696 L
.28912 .31695 L
.2895 .31691 L
.28986 .31686 L
.29019 .3168 L
.29082 .31666 L
.29117 .31656 L
.29149 .31646 L
.29206 .31652 L
.29269 .31668 L
.29326 .31679 L
.29381 .31686 L
.29444 .3169 L
.29503 .31689 L
.29568 .31682 L
.29604 .31676 L
.29636 .31669 L
.29692 .31655 L
.29751 .31638 L
.29785 .31641 L
.29816 .31649 L
.29876 .31664 L
.29938 .31675 L
.29973 .31679 L
.30006 .31681 L
.30066 .31683 L
.30129 .31679 L
.30182 .31671 L
.3024 .3166 L
Mistroke
.3029 .31647 L
.30344 .31631 L
.30407 .31641 L
.30468 .31655 L
.30527 .31665 L
.30581 .31672 L
.30645 .31674 L
.30681 .31673 L
.30714 .31671 L
.30776 .31663 L
.3084 .3165 L
.30894 .31636 L
.30952 .31618 L
.31016 .31635 L
.31075 .31648 L
.31108 .31654 L
.31143 .31659 L
.31181 .31662 L
.31216 .31664 L
.3128 .31663 L
.31316 .3166 L
.3135 .31656 L
.31413 .31645 L
.31481 .31629 L
.31544 .3161 L
.31602 .31623 L
.31658 .31636 L
.3172 .31646 L
.31778 .31652 L
.31831 .31653 L
.31889 .31651 L
.31953 .31644 L
.31986 .31639 L
.3202 .31631 L
.32083 .31616 L
.32114 .31607 L
.32148 .31599 L
.32184 .31609 L
.32218 .31617 L
.32253 .31624 L
.32286 .3163 L
.32347 .31638 L
.32408 .31641 L
.32464 .3164 L
.32525 .31635 L
.32589 .31625 L
.32646 .31612 L
.32697 .31598 L
.32757 .31591 L
.32813 .31604 L
Mistroke
.32861 .31614 L
.32913 .31622 L
.32969 .31627 L
.33021 .31628 L
.33077 .31626 L
.33138 .31619 L
.33195 .31609 L
.33247 .31597 L
.33305 .31582 L
.33368 .31582 L
.334 .31589 L
.33434 .31596 L
.33495 .31606 L
.33556 .31612 L
.3359 .31613 L
.33622 .31614 L
.33656 .31612 L
.33687 .3161 L
.33723 .31606 L
.33757 .31601 L
.33874 .31575 L
.33907 .31565 L
.33939 .31561 L
.33998 .31575 L
.34063 .31587 L
.34099 .31592 L
.34133 .31595 L
.34194 .31598 L
.34258 .31597 L
.34292 .31594 L
.34328 .31589 L
.34363 .31584 L
.34394 .31578 L
.34466 .31561 L
.34502 .3155 L
.34542 .31548 L
.34573 .31555 L
.34607 .31562 L
.34668 .31573 L
.34729 .31579 L
.34785 .31582 L
.3485 .3158 L
.34887 .31577 L
.34921 .31573 L
.34983 .31563 L
.35049 .31547 L
.35112 .3153 L
.35147 .31534 L
.35179 .31541 L
.35237 .31552 L
Mistroke
.35301 .3156 L
.35358 .31564 L
.35412 .31564 L
.35469 .31561 L
.35529 .31554 L
.35655 .31528 L
.35718 .31512 L
.35754 .3152 L
.35788 .31526 L
.35848 .31536 L
.35903 .31542 L
.35964 .31545 L
.36027 .31545 L
.36061 .31542 L
.36092 .31539 L
.36128 .31535 L
.36162 .31529 L
.36222 .31516 L
.36286 .315 L
.3632 .31496 L
.36357 .31503 L
.36422 .31514 L
.36459 .31519 L
.36498 .31523 L
.36535 .31525 L
.36569 .31526 L
.3663 .31525 L
.36695 .31519 L
.36756 .3151 L
.36811 .31498 L
.36843 .31491 L
.36877 .31482 L
.36913 .31476 L
.36947 .31483 L
.37009 .31493 L
.37043 .31498 L
.37075 .31501 L
.37137 .31505 L
.37193 .31505 L
.37246 .31503 L
.37304 .31496 L
.37425 .31474 L
.37458 .31466 L
.37493 .31457 L
.37555 .31465 L
.3761 .31474 L
.37669 .3148 L
.37733 .31484 L
.37793 .31484 L
.37853 .3148 L
Mistroke
.3791 .31473 L
.37961 .31465 L
.38016 .31454 L
.38078 .31439 L
.38113 .31438 L
.38145 .31443 L
.38211 .31453 L
.38248 .31457 L
.38281 .31459 L
.38341 .31462 L
.38375 .31462 L
.38406 .3146 L
.38464 .31456 L
.38525 .31448 L
.38588 .31437 L
.38646 .31424 L
.3871 .31416 L
.38742 .31422 L
.38777 .31427 L
.38837 .31433 L
.389 .31438 L
.3896 .31439 L
.39014 .31437 L
.39555 .31414 L
.41511 .31324 L
.43393 .31227 L
.45396 .31139 L
.47324 .31043 L
.49178 .30945 L
.51152 .30844 L
.53052 .30746 L
.55072 .30637 L
.57017 .3055 L
.58889 .30478 L
.6088 .30378 L
.60938 .30371 L
.60992 .30367 L
.61055 .30365 L
.61112 .30366 L
.61166 .3037 L
.61216 .30376 L
.61269 .30384 L
.61328 .30386 L
.61385 .30372 L
.61438 .30361 L
.61494 .30352 L
.61554 .30346 L
.61617 .30342 L
.61648 .30342 L
.61683 .30342 L
Mistroke
.61744 .30346 L
.61801 .30352 L
.61834 .30357 L
.61869 .30363 L
.61933 .30362 L
.61995 .30347 L
.62052 .30336 L
.62114 .30327 L
.6215 .30323 L
.62182 .30321 L
.62217 .30319 L
.62249 .30319 L
.62286 .3032 L
.6232 .30322 L
.62377 .30328 L
.62431 .30336 L
.62493 .30347 L
.62549 .30336 L
.62611 .30322 L
.62643 .30315 L
.62678 .3031 L
.6274 .30302 L
.62797 .30298 L
.62831 .30298 L
.62863 .30298 L
.62923 .30302 L
.62987 .30309 L
.63023 .30315 L
.63057 .30321 L
.63112 .30324 L
.63174 .30308 L
.63238 .30295 L
.63298 .30285 L
.63332 .30282 L
.63369 .30279 L
.63403 .30278 L
.63435 .30277 L
.63472 .30279 L
.63506 .30281 L
.63545 .30285 L
.63581 .3029 L
.63643 .303 L
.63675 .30307 L
.63709 .30305 L
.63746 .30295 L
.63781 .30287 L
.63847 .30273 L
.63908 .30264 L
.63943 .30261 L
.63975 .30259 L
Mistroke
.64034 .30258 L
.64096 .30261 L
.64152 .30267 L
.64204 .30276 L
.64262 .30287 L
.64324 .30282 L
.64386 .30266 L
.64453 .30252 L
.64488 .30247 L
.64524 .30243 L
.64559 .30241 L
.6459 .30239 L
.64651 .3024 L
.64685 .30242 L
.64716 .30245 L
.64774 .30253 L
.64835 .30265 L
.64894 .30272 L
.64959 .30254 L
.65016 .30241 L
.65076 .30231 L
.65136 .30224 L
.65191 .30222 L
.65245 .30223 L
.65297 .30227 L
.65357 .30235 L
.65413 .30245 L
.65471 .30258 L
.65535 .30243 L
.656 .30227 L
.65636 .3022 L
.6567 .30214 L
.65734 .30207 L
.65793 .30205 L
.65845 .30207 L
.65901 .30211 L
.65961 .3022 L
.66016 .30231 L
.66068 .30244 L
.66116 .30231 L
.66167 .30218 L
.66223 .30206 L
.66279 .30197 L
.66332 .30192 L
.66394 .3019 L
.6645 .30192 L
.6651 .30198 L
.66574 .30209 L
.66634 .30222 L
.6669 .30223 L
Mistroke
.66749 .30207 L
.66806 .30194 L
.66865 .30184 L
.6693 .30177 L
.66986 .30175 L
.67046 .30178 L
.67098 .30183 L
.67154 .30192 L
.67216 .30206 L
.67247 .30214 L
.67281 .30212 L
.67318 .30201 L
.67352 .30192 L
.67386 .30184 L
.67417 .30177 L
.6748 .30168 L
.67537 .30163 L
.67598 .30162 L
.67633 .30164 L
.67665 .30167 L
.67723 .30175 L
.67779 .30186 L
.67844 .30202 L
.67903 .30192 L
.67968 .30175 L
.68002 .30167 L
.68038 .30161 L
.68073 .30156 L
.68104 .30153 L
.68165 .3015 L
.68198 .30151 L
.68233 .30153 L
.68296 .3016 L
.68331 .30166 L
.68364 .30173 L
.68402 .30182 L
.68438 .30191 L
.68469 .3019 L
.68503 .30179 L
.68563 .30164 L
.68596 .30156 L
.68632 .3015 L
.68696 .30142 L
.68751 .30139 L
.68812 .30141 L
.68876 .30147 L
.68935 .30157 L
.68991 .3017 L
.69041 .30183 L
.691 .30169 L
Mistroke
.69155 .30155 L
.69215 .30142 L
.69278 .30133 L
.69332 .3013 L
.69392 .3013 L
.69456 .30136 L
.69492 .30141 L
.69525 .30147 L
.69585 .30161 L
.69649 .30175 L
.69709 .30157 L
.69764 .30143 L
.69818 .30132 L
.69869 .30125 L
.69929 .30121 L
.69984 .30121 L
.70045 .30126 L
.70109 .30137 L
.70161 .30148 L
.70216 .30163 L
.70269 .30159 L
.70317 .30146 L
.70373 .30132 L
.70434 .30121 L
.70492 .30115 L
.70545 .30113 L
.70603 .30116 L
.70664 .30123 L
.70716 .30132 L
.70773 .30146 L
.70834 .30163 L
.70868 .30152 L
.709 .30143 L
.70961 .30127 L
.71018 .30117 L
.71071 .3011 L
.71127 .30107 L
.71184 .30109 L
.71246 .30115 L
.713 .30124 L
.71357 .30137 L
.71411 .30152 L
.71461 .30147 L
.71523 .3013 L
.71558 .30122 L
.7159 .30115 L
.71647 .30107 L
.71709 .30103 L
.71742 .30102 L
.71777 .30104 L
Mistroke
.71839 .3011 L
.71874 .30115 L
.71906 .30122 L
.71941 .3013 L
.71979 .3014 L
.72043 .30147 L
.72104 .3013 L
.72158 .30117 L
.72217 .30106 L
.72281 .301 L
.72316 .30099 L
.72349 .30099 L
.72408 .30103 L
.72471 .30112 L
.72504 .30119 L
.72536 .30127 L
.72595 .30143 L
.72659 .30138 L
.72695 .30128 L
.72728 .3012 L
.72783 .30108 L
.72844 .301 L
.72907 .30096 L
.72966 .30098 L
.73 .301 L
.73036 .30105 L
.73071 .30111 L
.73103 .30118 L
.73167 .30135 L
.73227 .30145 L
.73292 .30126 L
.73329 .30117 L
.73363 .3011 L
.73396 .30104 L
.73432 .30099 L
.73469 .30096 L
.73507 .30095 L
.73539 .30095 L
.73573 .30097 L
.73636 .30105 L
.73697 .30117 L
.73753 .30131 L
.73814 .30147 L
.73849 .30136 L
.73881 .30127 L
.73947 .30112 L
.73983 .30105 L
.74017 .301 L
.74077 .30095 L
.74133 .30095 L
Mistroke
.74195 .301 L
.74229 .30105 L
.7426 .3011 L
.74317 .30123 L
.74379 .30141 L
.74438 .30139 L
.74491 .30124 L
.74553 .3011 L
.74612 .30101 L
.74665 .30097 L
.74722 .30096 L
.74785 .30101 L
.74816 .30105 L
.74851 .30111 L
.74912 .30125 L
.7497 .30141 L
.75027 .30142 L
.75082 .30127 L
.75132 .30116 L
.75185 .30106 L
.75242 .301 L
.75295 .30098 L
.75358 .30101 L
.75416 .30108 L
.75538 .30136 L
.75599 .30153 L
.75634 .30142 L
.75666 .30133 L
.75734 .30117 L
.75773 .30111 L
.75807 .30106 L
.75839 .30104 L
.75874 .30102 L
.75937 .30104 L
.75996 .3011 L
.7603 .30115 L
.76061 .30122 L
.7613 .30139 L
.76193 .30157 L
.76248 .30141 L
.76306 .30127 L
.76365 .30116 L
.76429 .30109 L
.76493 .30107 L
.76553 .30111 L
.76584 .30114 L
.76618 .30119 L
.76688 .30134 L
.76748 .30151 L
.76783 .30162 L
Mistroke
.76814 .30155 L
.76875 .30138 L
.76932 .30127 L
.76986 .30118 L
.77044 .30114 L
.77108 .30114 L
.77166 .30119 L
.77221 .30127 L
.77281 .3014 L
.77333 .30154 L
.77389 .30167 L
.77453 .30149 L
.77513 .30136 L
.7755 .3013 L
.77584 .30125 L
.77615 .30123 L
.77649 .30121 L
.77682 .30121 L
.77718 .30122 L
.77756 .30126 L
.77792 .3013 L
.77855 .30143 L
.77924 .30161 L
.77983 .30175 L
.78017 .30165 L
.78049 .30157 L
.78112 .30143 L
.78148 .30138 L
.7818 .30134 L
.78238 .3013 L
.78301 .3013 L
.7836 .30135 L
.78415 .30144 L
.78477 .30158 L
.78535 .30174 L
.78587 .30181 L
.78645 .30166 L
.78707 .30153 L
.78738 .30148 L
.78772 .30143 L
.78833 .30139 L
.78891 .3014 L
.78945 .30144 L
.79002 .30152 L
.79051 .30163 L
.79105 .30176 L
.79162 .30194 L
.79224 .3018 L
.79278 .30168 L
.79336 .30158 L
Mistroke
.79368 .30154 L
.79403 .30151 L
.79466 .3015 L
.79501 .30152 L
.79539 .30155 L
.79575 .3016 L
.79608 .30165 L
.79671 .3018 L
.79704 .30188 L
.7974 .30199 L
.79774 .30202 L
.79805 .30194 L
.79865 .30181 L
.79927 .3017 L
.79984 .30164 L
.80047 .30162 L
.80079 .30163 L
.80113 .30165 L
.80172 .30172 L
.80234 .30184 L
.80293 .30199 L
.80347 .30214 L
.80405 .30205 L
.80459 .30194 L
.80519 .30184 L
.80583 .30177 L
.80646 .30175 L
.80678 .30176 L
.80713 .30179 L
.80775 .30187 L
.80833 .30198 L
.80891 .30213 L
.80947 .30229 L
.80996 .3022 L
.8105 .30208 L
.81109 .30198 L
.81171 .30192 L
.81224 .3019 L
.81282 .30191 L
.82299 .30229 L
.82358 .30224 L
.82413 .30222 L
.82476 .30224 L
.82533 .30229 L
.82597 .3024 L
.82665 .30255 L
.82729 .30273 L
.82788 .30264 L
.82845 .30254 L
.82905 .30245 L
Mistroke
.82962 .3024 L
.83014 .30239 L
.83072 .30241 L
.83128 .30247 L
.83187 .30256 L
.83252 .3027 L
.83316 .30287 L
.83353 .30288 L
.83387 .30281 L
.83456 .30269 L
.83495 .30264 L
.8353 .30261 L
.83593 .30258 L
.83629 .30258 L
.83662 .3026 L
.83722 .30265 L
.83787 .30276 L
.8385 .30289 L
.83884 .30298 L
.83919 .30307 L
.83984 .30299 L
.84043 .30289 L
.841 .30282 L
.84154 .30278 L
.84213 .30277 L
.84276 .30281 L
.85245 .30328 L
.85757 .30361 L
.85822 .30352 L
.85855 .30348 L
.85891 .30345 L
.85931 .30342 L
.85967 .30342 L
.86001 .30342 L
.86037 .30344 L
.86071 .30346 L
.86108 .3035 L
.86175 .3036 L
.8624 .30374 L
.863 .30388 L
.86358 .30382 L
.86412 .30375 L
.86471 .30369 L
.86534 .30365 L
.86566 .30365 L
.866 .30365 L
.86663 .30369 L
.86725 .30376 L
.86782 .30385 L
.86851 .304 L
Mistroke
.86885 .30408 L
.86917 .30411 L
.86975 .30402 L
.87039 .30394 L
.87103 .3039 L
.87139 .30389 L
.87172 .30389 L
.87208 .3039 L
.8724 .30392 L
.87278 .30395 L
.87313 .30399 L
.87426 .30419 L
.87484 .30431 L
.87547 .30428 L
.876 .30422 L
.87658 .30417 L
.87713 .30414 L
.87763 .30414 L
.87821 .30416 L
.87876 .3042 L
.87936 .30428 L
.88 .30439 L
.88033 .30445 L
.88068 .30453 L
.88132 .30454 L
.88194 .30447 L
.88251 .30442 L
.88312 .30439 L
.88371 .30439 L
.88423 .30442 L
.8848 .30447 L
.88542 .30455 L
.88608 .30466 L
.8867 .30479 L
.88726 .30479 L
.88784 .30472 L
.88837 .30468 L
.88894 .30466 L
.88956 .30466 L
.89009 .30468 L
.89058 .30471 L
.89113 .30477 L
.89171 .30486 L
.89234 .30498 L
.89269 .30505 L
.89301 .30506 L
.89337 .30502 L
.89375 .30499 L
.8941 .30496 L
.89443 .30494 L
Mistroke
.89508 .30492 L
.89567 .30493 L
.89631 .30496 L
.89667 .305 L
.897 .30503 L
.89758 .30511 L
.89821 .30522 L
.89879 .30533 L
.89934 .30528 L
.89997 .30523 L
.90029 .30522 L
.90064 .3052 L
.90125 .3052 L
.90182 .30521 L
.90244 .30526 L
.90275 .30528 L
.9031 .30532 L
.90427 .30551 L
.9046 .30557 L
.90494 .30558 L
.90531 .30555 L
.90565 .30552 L
.90627 .30549 L
.90662 .30548 L
.90694 .30548 L
.90757 .30549 L
.90817 .30552 L
.90871 .30557 L
.90929 .30563 L
.90992 .30573 L
.91027 .30578 L
.91059 .30584 L
.91116 .30583 L
.91179 .30579 L
.91235 .30577 L
.91297 .30576 L
.91354 .30578 L
.91408 .3058 L
.91461 .30585 L
.91509 .3059 L
.91617 .30605 L
.91678 .30613 L
.91734 .30609 L
.91795 .30606 L
.9186 .30605 L
.91916 .30606 L
.91969 .30608 L
.92026 .30611 L
.92088 .30617 L
.92214 .30633 L
Mistroke
.92277 .3064 L
.92314 .30638 L
.92348 .30637 L
.92408 .30635 L
.92463 .30635 L
.92524 .30636 L
.92587 .30639 L
.9313 .30666 L
.94116 .30725 L
.95985 .30817 L
.97619 .30902 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True,
  ImageSize->{215, 132.812},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004<0000YQ000`40O003h00Oogoo3Goo0026Ool0
0`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool20024Ool008Io
o`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo
08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`8008Aoo`00QWoo00<007oo
Ool0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00CWoo0`001Woo0P0017oo0`00
1Goo0`001Goo0`001Goo0`0017oo1@001Goo00<007ooOol0Pgoo001=Ool01@00Oomoogoo00001Goo
0P000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000Aoo`04001oogoo0005Ool00`00Oomoo`23Ool004Eoo`H0009oo`05001oogooOol0000:
Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo
00001Goo00<007ooOol01Goo0`00Pgoo001=Ool01@00Oomoogoo00002Woo00D007ooOomoo`0000=o
o`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Ioo`03001oogoo00Ao
o`03001oogoo08=oo`00CGoo00D007ooOomoo`0000Yoo`05001oogooOol00003Ool01@00Oomoogoo
00000goo00D007ooOomoo`0000=oo`05001oogooOol00007Ool00`00Oomoo`03Ool00`00Oomoo`23
Ool004eoo`05001oogooOol0000:Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001o
ogooOol00003Ool01@00Oomoogoo00001goo00<007ooOol00goo00<007ooOol0Pgoo001=Ool01@00
Oomoogoo00002Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`05001oogooOol00005Ool00`00Oomoo`23Ool004ioo`<000aoo`<000Eo
o`<000Eoo`<000Eoo`<000Eoo`<000Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool2
0024Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Io
o`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool20024Ool008Ioo`03001oogoo08=oo`00
QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo00<007oo
Ool0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`8008Aoo`00QWoo00<007ooOol0Pgoo0026Ool00`00
Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool0
08Ioo`8008Aoo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=o
o`00QWoo00<007ooOol0Pgoo001>Ool30006Ool20004Ool30005Ool30005Ool30005Ool30005Ool5
0004Ool00`00Oomoo`23Ool004eoo`05001oogooOol00005Ool20003Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00001Woo00<007ooOol017oo
00<007ooOol0Pgoo0015Ool60002Ool01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05001o
ogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Ioo`03001oogoo00Aoo`<008=o
o`00CGoo00D007ooOomoo`0000Yoo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00006Ool00`00Oomoo`04Ool00`00Oomoo`23Ool004eoo`05001o
ogooOol0000:Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool0
1@00Oomoogoo00001Woo00<007ooOol017oo00<007ooOol0Pgoo001=Ool01@00Oomoogoo00002Woo
00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`00
00Ioo`03001oogoo00Aoo`03001oogoo08=oo`00CGoo00D007ooOomoo`0000Yoo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00006Ool00`00Oomo
o`04Ool00`00Oomoo`23Ool004ioo`<000aoo`<000Eoo`<000Eoo`<000Eoo`<000Eoo`<000Ioo`03
001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool20024Ool008Ioo`03001oogoo08=oo`00QWoo
00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0
Pgoo0026Ool20024Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomo
o`23Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Io
o`8008Aoo`0027oo1@00?7oo1@00>7oo00<007ooOol0>goo1@00>Goo1@001Goo0009Ool01000Oomo
o`00?Woo00<007ooOol0>7oo00<007ooOol0?Goo00<007ooOol0>Woo00@007ooOol000Eoo`001P00
17oo00<007ooOol0<goo1P001Goo00<007ooOol0>7oo00<007ooOol0?Goo00<007ooOol0>goo00<0
07ooOol01Goo000;Ool00`00Oomoo`0mOol00`00Oomoo`0hOol00`00Oomoo`0mOol00`00Oomoo`0l
Ool00`00Oomoo`04Ool000aoo`03001oogoo03aoo`03001oogoo03Qoo`03001oogoo03eoo`03001o
ogoo03eoo`03001oogoo00=oo`0037oo00<007ooOol0?7oo00<007ooOol0>7oo0P00?Woo00<007oo
Ool0?Goo00<007ooOol00goo0008Ool01@00Oomoogoo0000?Woo00<007ooOol0>7oo00<007ooOol0
?Goo00<007ooOol0>Goo00D007ooOomoo`0000Eoo`002Goo0`00?Goo0`00>Woo00<007ooOol0>goo
0`00?7oo0`001Woo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0
7Goo?`009goo0026Ool00`00Oomoo`08OolG000mOolG000@Ool00003Ool000000?l000X000001goo
00<007ooOol01Woo5`0000=oo`00Ool00goo00<007ooOol02Woo00<007ooOol02Goo00<007ooOol0
2Woo00<007ooOol02Woo00<007ooOol00Woo60002Goo00<007ooOol02Woo00<007ooOol02Goo00<0
07ooOol02Woo00<007ooOol02Woo00<007ooOol02Goo00<007ooOol02Woo00<007ooOol02Woo00<0
07ooOol02Woo00<007ooOol02Goo00<007ooOol02Woo00<007ooOol017oo0007Ool00`00Oomoo`0M
Oom0000OOol00`00Oomoo`0lOol00`00Oomoo`0mOol00`00Oomoo`04Ool008Ioo`03001oogoo08=o
o`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo0P00
Q7oo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool0
0`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo0P00Q7oo0026Ool00`00Oomoo`23Ool008Io
o`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo
08=oo`00QWoo00<007ooOol0Pgoo0026Ool20024Ool008Ioo`03001oogoo08=oo`00QWoo00<007oo
Ool0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026
Ool20024Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool0
04ioo`<000Ioo`8000Eoo`<000Aoo`<000Eoo`<000Eoo`<000Eoo`D000Aoo`03001oogoo08=oo`00
CGoo00D007ooOomoo`0000Eoo`8000Aoo`05001oogooOol00002Ool01@00Oomoogoo00000goo00D0
07ooOomoo`0000=oo`05001oogooOol00006Ool00`00Oomoo`04Ool00`00Oomoo`23Ool004eoo`05
001oogooOol0000;Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000=oo`05001oogooOol00003
Ool01@00Oomoogoo00001Woo00<007ooOol017oo00<007ooOol0Pgoo001=Ool01@00Oomoogoo0000
2goo00D007ooOomoo`00009oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomo
o`0000Ioo`03001oogoo00Aoo`<008=oo`00CGoo00D007ooOomoo`0000]oo`05001oogooOol00002
Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00006Ool00`00Oomoo`04
Ool00`00Oomoo`23Ool004eoo`05001oogooOol0000;Ool01@00Oomoogoo00000Woo00D007ooOomo
o`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00001Woo00<007ooOol017oo00<007ooOol0
Pgoo001=Ool01@00Oomoogoo00002goo00D007ooOomoo`00009oo`05001oogooOol00003Ool01@00
Oomoogoo00000goo00D007ooOomoo`0000Ioo`03001oogoo00Aoo`03001oogoo08=oo`00CWoo0`00
3Goo0`0017oo0`001Goo0`001Goo0`001Goo0`001Woo00<007ooOol0Pgoo0026Ool00`00Oomoo`23
Ool008Ioo`03001oogoo08=oo`00QWoo0P00Q7oo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo
08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo
0P00Q7oo0026Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026
Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool20024Ool0
08Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`03001o
ogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool20024Ool008Ioo`03001oogoo08=oo`00QWoo00<0
07ooOol0Pgoo0026Ool00`00Oomoo`23Ool004ioo`<000Ioo`8000Eoo`<000Aoo`<000Eoo`<000Eo
o`<000Aoo`D000Eoo`03001oogoo08=oo`00CGoo00D007ooOomoo`0000Eoo`8000Aoo`05001oogoo
Ool00002Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00004Ool01000
Oomoo`001Goo00<007ooOol0Pgoo001=Ool01@00Oomoogoo00002goo00D007ooOomoo`00009oo`05
001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Eoo`03001oogoo00Eoo`03
001oogoo08=oo`00CGoo00D007ooOomoo`0000]oo`05001oogooOol00002Ool01@00Oomoogoo0000
0goo00D007ooOomoo`0000=oo`05001oogooOol00006Ool00`00Oomoo`04Ool30023Ool004eoo`05
001oogooOol0000;Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000=oo`05001oogooOol00003
Ool01@00Oomoogoo00001goo00<007ooOol00goo00<007ooOol0Pgoo001=Ool01@00Oomoogoo0000
2goo00D007ooOomoo`00009oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomo
o`0000Moo`03001oogoo00=oo`03001oogoo08=oo`00CGoo00D007ooOomoo`0000]oo`05001oogoo
Ool00002Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00
Oomoogoo00001Goo00<007ooOol0Pgoo001>Ool3000=Ool30004Ool30005Ool30005Ool30005Ool3
0006Ool00`00Oomoo`23Ool008Ioo`03001oogoo08=oo`00QWoo0P00Q7oo0026Ool00`00Oomoo`23
Ool008Ioo`03001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool008Ioo`03
001oogoo08=oo`00QWoo00<007ooOol0Pgoo0026Ool20024Ool008Ioo`03001oogoo08=oo`00QWoo
00<007ooOol0Pgoo0026Ool00`00Oomoo`23Ool00001\
\>"],
  ImageRangeCache->{{{0, 267.75}, {165, 0}} -> {-2.11818, -2.50358*^-5, 
  0.0196928, 3.79328*^-7}}]
}, Open  ]],

Cell[TextData[{
  "It is interesting to re-run the Douglas scheme with ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 32\)]],
  " - do it yourself with the code - here are the results. The errors are now \
similar in scale to the CN results, but with an \[Alpha] that is eight times \
as large - we have only used five time-steps!"
}], "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell["\<\
samples = TableForm[Join[{{\"x\", \"Exact\", \"Douglas\", \
\"Error\"}},
Table[Map[PaddedForm[N[Chop[#1]],{5,6}]&,
N[{x, ufunc[x], Sin[Pi*x/2]*Exp[-(Pi^2 0.1)/4], 
ufunc[x]- Sin[Pi*x/2]*Exp[-(Pi^2 0.1)/4]},5]],
{x, -2, 2, 0.25}]]]\
\>", "Input",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"x\"\>", "\<\"Exact\"\>", "\<\"Douglas\"\>", "\<\"Error\"\>"},
          {
            TagBox[
              InterpretationBox["\<\"-2.000000\"\>",
                -2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.750000\"\>",
                -1.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.298990\"\>",
                -0.29899232429868694,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299010\"\>",
                -0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000015\"\>",
                1.49763641528744*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.500000\"\>",
                -1.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552470\"\>",
                -0.55246577759506754,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552490\"\>",
                -0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000028\"\>",
                2.7672712624826623*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.250000\"\>",
                -1.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721830\"\>",
                -0.72183132436734587,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721870\"\>",
                -0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000036\"\>",
                3.6156141451693102*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-1.000000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.781300\"\>",
                -0.78130459542194419,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.781340\"\>",
                -0.78134373054744422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000039\"\>",
                3.9135125500022738*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.750000\"\>",
                -0.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721830\"\>",
                -0.72183132436734665,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.721870\"\>",
                -0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000036\"\>",
                3.6156141450915946*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.500000\"\>",
                -0.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552470\"\>",
                -0.5524657775950701,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.552490\"\>",
                -0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000028\"\>",
                2.767271262227311*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"-0.250000\"\>",
                -0.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.298990\"\>",
                -0.29899232429868811,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.299010\"\>",
                -0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000015\"\>",
                1.4976364151708665*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.250000\"\>",
                0.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.298990\"\>",
                0.29899232429868638,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299010\"\>",
                0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000015\"\>",
                -1.4976364153429511*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.500000\"\>",
                0.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552470\"\>",
                0.5524657775950671,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552490\"\>",
                0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000028\"\>",
                -2.7672712625270712*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 0.750000\"\>",
                0.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721830\"\>",
                0.7218313243673421,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721870\"\>",
                0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000036\"\>",
                -3.6156141455467861*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.000000\"\>",
                1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.781300\"\>",
                0.78130459542194086,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.781340\"\>",
                0.78134373054744422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000039\"\>",
                -3.9135125503353407*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.250000\"\>",
                1.25,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721830\"\>",
                0.72183132436734376,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.721870\"\>",
                0.72186748050879757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000036\"\>",
                -3.6156141453802526*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.500000\"\>",
                1.5,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552470\"\>",
                0.55246577759506765,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.552490\"\>",
                0.55249345030769237,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000028\"\>",
                -2.7672712624715601*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 1.750000\"\>",
                1.75,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.298990\"\>",
                0.2989923242986865,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.299010\"\>",
                0.29900730066283981,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.000015\"\>",
                -1.4976364153318489*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 2.000000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.000000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 6}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  CellMargins->{{42, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakWithin->True,
  FontSize->8]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["14.10 Summary", "Section",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True,
  GroupPageBreakWithin->True],

Cell[TextData[{
  "We have shown how to use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s compiler to build efficient solvers for explicit, fully implicit, \
Crank-Nicholson and Douglas two-time-level difference schemes. Using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s built-in functions, we have compared the results with the exact \
solution of the diffusion equation using a simple smooth initial condition. \
These comparisons reveal the advantages of implicit schemes over explicit \
schemes, and the advantage in accuracy of the Douglas scheme over the \
Crank-Nicholson scheme. Within a general ",
  Cell[BoxData[
      \(TraditionalForm\`\[Theta]\)]],
  "-method approach, the Douglas scheme is given by setting"
}], "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \(TraditionalForm\`\[Theta] = 1\/2 - 1\/\(12\ \[Alpha]\)\)], 
  "NumberedEquation",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[TextData[{
  "and this method is the natural implicit generalization of the trinomial \
scheme, to which it reduces when ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 1/6\)]],
  ". On problems with smooth initial data, it is frequently the method of \
choice, and can give results nearly two orders of magnitude more accurate \
than a Crank-Nicholson scheme on such problems, for similar numerical \
parameters, or results of comparable accuracy with much larger time-steps. \n\
\nWe have not tried to optimize the Douglas scheme for accuracy by making the \
optimal choice of the parameter \[Alpha]. If the choice \[Alpha] = ",
  Cell[BoxData[
      \(TraditionalForm\`1/\@20\)]],
  " is made the accuracy gains over Crank-Nicholson can be quite spectacular. \
Mitchell and Griffiths (1980) give an example where the errors in Douglas are \
about 1/400000 of those in Crank-Nicholson with otherwise identical numerical \
parameters.\n\nWe emphasize that this is ",
  StyleBox["not",
    FontSlant->"Italic"],
  " the end of the story. There are additional complications in the presence \
of initial and boundary conditions that are not smooth, which is almost \
always the case in option-pricing problems. These will be considered in the \
next chapter."
}], "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 14 Bibliography", "Section",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3.0 ",
  StyleBox["Standard Add-on Packages",
    FontSlant->"Italic"],
  ", 1996, Wolfram Media-Cambridge University Press.\n\nMitchell, A.R. and \
Griffiths, D.F., 1980, ",
  StyleBox["The Finite Difference Method in Partial Differential Equations",
    FontSlant->"Italic"],
  ", John Wiley  (Corrected reprinted edition, 1994).\n\nWolfram, S., 1996, ",
  
  StyleBox["The Mathematica Book",
    FontSlant->"Italic"],
  " (for version 3.0),  Wolfram Media-Cambridge University Press."
}], "Text",
  CellMargins->{{6.25, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{643, 608},
WindowMargins->{{16, Automatic}, {Automatic, 1}},
PrintingCopies->1,
PrintingStartingPageNumber->266,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"14. Diffusion by Finite-Differences"}]]}, {Cell[ 
        TextData[ {"14. Diffusion by Finite-Differences"}]], Inherited, 
      Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
PrivateFontOptions->{"FontType"->"Outline"},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8^on3ohP==PTc0b85N`?N0080004/0B`000006G<AMP01
0000I00000400@4100800BL?00400@S1ll0000000000064/01T1T00000000000
00000000007oo@S23^3oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 277, 10, 208, "Title"],
Cell[2011, 63, 221, 6, 60, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2257, 73, 96, 1, 49, "Section"],
Cell[2356, 76, 1098, 17, 164, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3491, 98, 152, 3, 49, "Section"],
Cell[3646, 103, 808, 17, 194, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4491, 125, 174, 6, 38, "Section"],
Cell[4668, 133, 737, 15, 73, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5442, 153, 136, 1, 38, "Section"],
Cell[5581, 156, 110, 1, 20, "Text"],
Cell[5694, 159, 190, 3, 32, "NumberedEquation"],
Cell[5887, 164, 100, 1, 20, "Text"],
Cell[5990, 167, 208, 3, 20, "NumberedEquation"],
Cell[6201, 172, 100, 1, 20, "Text"],
Cell[6304, 175, 151, 2, 28, "NumberedEquation"],
Cell[6458, 179, 107, 1, 20, "Text"],
Cell[6568, 182, 164, 3, 20, "NumberedEquation"],
Cell[6735, 187, 103, 1, 20, "Text"],
Cell[6841, 190, 222, 4, 28, "NumberedEquation"],
Cell[7066, 196, 605, 12, 62, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7708, 213, 147, 3, 38, "Section"],
Cell[7858, 218, 309, 8, 20, "Text"],

Cell[CellGroupData[{
Cell[8192, 230, 203, 4, 25, "Input",
  PageBreakWithin->True],
Cell[8398, 236, 110, 3, 36, "Output",
  PageBreakWithin->True]
}, Open  ]],
Cell[8523, 242, 149, 4, 20, "Text"],
Cell[8675, 248, 134, 2, 25, "Input"],
Cell[8812, 252, 151, 4, 20, "Text"],
Cell[8966, 258, 315, 6, 62, "Input"],

Cell[CellGroupData[{
Cell[9306, 268, 121, 2, 25, "Input"],
Cell[9430, 272, 15584, 519, 140, 6273, 398, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[25029, 794, 1062, 31, 73, "Text"],
Cell[26094, 827, 960, 16, 188, "Input",
  PageBreakWithin->True],
Cell[27057, 845, 191, 4, 20, "Text"],
Cell[27251, 851, 184, 4, 25, "Input",
  PageBreakAbove->False],
Cell[27438, 857, 209, 4, 20, "Text",
  PageBreakAbove->False],
Cell[27650, 863, 214, 6, 36, "Input",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[27889, 873, 188, 3, 25, "Input",
  PageBreakAbove->False],
Cell[28080, 878, 149, 4, 36, "Output",
  PageBreakAbove->False]
}, Open  ]],
Cell[28244, 885, 185, 4, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[28454, 893, 213, 6, 36, "Input",
  PageBreakAbove->False],
Cell[28670, 901, 20571, 625, 140, 6279, 442, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[49256, 1529, 261, 6, 29, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[49542, 1539, 372, 12, 90, "Input",
  PageBreakAbove->False],
Cell[49917, 1553, 12946, 367, 212, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[62912, 1926, 132, 1, 38, "Section"],
Cell[63047, 1929, 457, 11, 40, "Text"],
Cell[63507, 1942, 1133, 22, 176, "Input",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[64677, 1969, 153, 3, 38, "Section"],
Cell[64833, 1974, 124, 1, 20, "Text"],
Cell[64960, 1977, 266, 7, 57, "Input"],
Cell[65229, 1986, 125, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[65379, 1991, 188, 9, 80, "Input"],
Cell[65570, 2002, 55, 1, 36, "Output"]
}, Open  ]],
Cell[65640, 2006, 136, 1, 20, "Text"],
Cell[65779, 2009, 400, 7, 100, "Input"],
Cell[66182, 2018, 89, 1, 20, "Text"],
Cell[66274, 2021, 146, 2, 25, "Input"],
Cell[66423, 2025, 385, 11, 101, "Input"],
Cell[66811, 2038, 101, 1, 20, "Text"],
Cell[66915, 2041, 249, 6, 46, "Input"],
Cell[67167, 2049, 146, 3, 20, "Text"],

Cell[CellGroupData[{
Cell[67338, 2056, 243, 7, 36, "Input",
  PageBreakAbove->False],
Cell[67584, 2065, 18104, 558, 112, 5596, 396, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]],
Cell[85703, 2626, 208, 5, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[85936, 2635, 370, 10, 68, "Input",
  PageBreakAbove->False],
Cell[86309, 2647, 12943, 367, 212, "Output"]
}, Open  ]],
Cell[99267, 3017, 560, 11, 52, "Text",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[99864, 3033, 176, 4, 38, "Section",
  PageBreakAbove->Automatic],
Cell[100043, 3039, 306, 6, 40, "Text"],
Cell[100352, 3047, 1381, 39, 398, "Input",
  PageBreakAbove->False,
  PageBreakWithin->True],
Cell[101736, 3088, 223, 7, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[101984, 3099, 214, 6, 36, "Input",
  PageBreakAbove->False],
Cell[102201, 3107, 43007, 2249, 108, 32461, 2112, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->Automatic]
}, Open  ]],

Cell[CellGroupData[{
Cell[145245, 5361, 214, 6, 36, "Input",
  PageBreakAbove->False],
Cell[145462, 5369, 16854, 582, 108, 6514, 448, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]],
Cell[162331, 5954, 315, 7, 29, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[162671, 5965, 398, 11, 57, "Input",
  PageBreakAbove->False,
  PageBreakBelow->False],
Cell[163072, 5978, 12992, 367, 220, "Output"]
}, Open  ]],
Cell[176079, 6348, 419, 8, 40, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[176535, 6361, 139, 3, 42, "Section"],
Cell[176677, 6366, 209, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[176911, 6374, 175, 4, 28, "Subsubsection",
  PageBreakAbove->False],
Cell[177089, 6380, 1378, 41, 420, "Input",
  PageBreakAbove->False,
  PageBreakWithin->True],
Cell[178470, 6423, 197, 4, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[178692, 6431, 213, 6, 36, "Input",
  PageBreakAbove->False],
Cell[178908, 6439, 33737, 1531, 140, 21356, 1371, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]],
Cell[212660, 7973, 282, 8, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[212967, 7985, 366, 10, 68, "Input",
  PageBreakAbove->False],
Cell[213336, 7997, 13121, 367, 240, "Output",
  PageBreakAbove->False]
}, Open  ]],
Cell[226472, 8367, 903, 23, 52, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[227400, 8394, 249, 6, 36, "Input",
  PageBreakAbove->False],
Cell[227652, 8402, 28010, 1397, 140, 19647, 1287, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]],
Cell[255677, 9802, 436, 9, 40, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[256138, 9815, 366, 10, 68, "Input",
  PageBreakAbove->False],
Cell[256507, 9827, 13030, 369, 212, "Output",
  PageBreakAbove->False,
  PageBreakWithin->True]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[269598, 10203, 146, 3, 42, "Section",
  PageBreakAbove->True],
Cell[269747, 10208, 856, 18, 85, "Text"],
Cell[270606, 10228, 191, 4, 34, "NumberedEquation"],
Cell[270800, 10234, 1371, 25, 180, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[272208, 10264, 102, 1, 42, "Section"],
Cell[272313, 10267, 631, 16, 85, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

