(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    394631,      12953]*)
(*NotebookOutlinePosition[    396555,      13011]*)
(*  CellTagsIndexPosition[    396511,      13007]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 15.
Finite-Difference Schemes for the Black-Scholes Equation with Non-smooth \
Payoff Initial Conditions\
\>", "Title",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["\<\
Errors in Greeks in Crank-Nicholson Schemes, and Resolution by
Three-Time-Level Douglas Finite-Difference Scheme\
\>", "Subsubtitle",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["15.1 Introduction", "Section",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In Chapter 14 I gave some examples which show how various commonly used \
finite-difference schemes behave with simple smooth \"payoffs\". With smooth \
payoffs the Douglas finite-difference scheme gives much better results than \
Crank-Nicholson. In this chapter we will look at how these methods behave \
when applied to real-world option-pricing problems. This will highlight the \
potentially nasty behaviour of both Crank-Nicholson and Douglas \
finite-difference schemes when applied to simple option-pricing problems. The \
discontinuous nature of the payoff or its derivative in the neighbourhood of \
the strike induces slowly decaying oscillations into the solution of the \
finite-difference equations, when we use a larger time-step (which is the \
main point of introducing FD schemes in the first place.)  These introduce \
small errors into the valuation itself, and undermine attempts to compute \
\[Delta], \[CapitalGamma] and other \"derivative\" quantities in the \
neighbourhood of the strike. Note that, even with the oscillatory components \
small in the valuation error, the effect on the slope of the function is \
larger (generating larger errors in \[Delta]), and there are even larger \
errors in \[CapitalGamma].\n\nIn this chapter I also look at the \
three-time-level version of the Douglas scheme. This chapter shows how the \
problem of larger errors in the Greeks can be cured, in that the oscillations \
can be removed, at least for our test payoffs, and the method \
(\"three-time-level Douglas\") will be used as the basis of our programme to \
define benchmark numerical algorithms. \n\nThe problems with the \
Crank-Nicholson scheme, and its resolution, are exemplified by a test problem \
with \[Alpha] = 8. Although the error in the valuation is small, we get",
  StyleBox[" an error in \[CapitalGamma] near the strike of ",
    FontSlant->"Italic"],
  StyleBox["12%",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" of its exact value, and an error in \[Theta] of ",
    FontSlant->"Italic"],
  StyleBox["200%",
    FontVariations->{"CompatibilityType"->0}],
  ". In the corresponding three-time-level Douglas solution, the error in \
\[CapitalGamma] near the strike is reduced to 0.2%, and that in \[Theta] to \
2.6%.\n\nUsers of implicit finite-difference models may wish to evaluate \
their choices of difference scheme in the light of these results, and at \
least to look at the effect of changing the difference scheme to a \
three-time-level one such as that presented here. "
}], "Text",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["15.2 Finite-Difference Schemes with Three Time-Levels", "Section",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True],

Cell[TextData[{
  "Various schemes have been proposed for treating problems introduced by \
considering discontinuous or non-smooth boundary/initial conditions. These \
schemes aim to damp fast oscillations more effectively, by adjusting the \
spectrum of eigenvalues of the difference matrix. Richtmyer and Morton (1957) \
give a list of 14 difference schemes for the diffusion equation, and \
recommend schemes (their numbering) 9, 11 and 13 for non-smooth initial data. \
Schemes 9 and 13 are also recommended for these purposes by Smith (1985). \
These two schemes may be regarded as the three-time-level versions of the \
Crank-Nicholson scheme and the Douglas scheme, since they have truncation \
errors of a similar character to their two-time-level counterparts. We \
already know that the truncation error characteristics of the Douglas scheme \
make it preferable, so we shall use this. As in Chapter 14, we let ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " denote the time-step, and ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " denote the ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "-step. The three time-level Douglas scheme is then given by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\((1\/8\  - \ \[Alpha])\) \((u\_\(n - 1\)\%\(m + 1\) + 
              u\_\(n + 1\)\%\(m + 1\))\)\  + \ \((5\/4\  + \ 2  \[Alpha])\) 
          u\_n\%\(m + 1\)\[IndentingNewLine] = \(1\/6\) \((u\_\(n - 1\)\%m + 
              u\_\(n + 1\)\%m + 
              10  u\_n\%m)\)\  - \ \(1\/24\) \((u\_\(n - 1\)\%\(m - 1\) + 
              u\_\(n + 1\)\%\(m - 1\) + 10  u\_n\%\(m - 1\))\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This type of process requires a kick-off procedure, since initially we \
only know ",
  Cell[BoxData[
      \(TraditionalForm\`u\^1\)]],
  ". We use the ordinary Douglas two-time-level scheme:\[AutoOperand]"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\((1\  - \ 6  \[Alpha])\) \((u\_\(n - 1\)\%\(m + 1\) + 
              u\_\(n + 1\)\%\(m + 1\))\)\  + \ \((10\  + \ 12  \[Alpha])\) 
          u\_n\%\(m + 1\) = \((1\  + \ 6  \[Alpha])\) \((u\_\(n - 1\)\%m + 
              u\_\(n + 1\)\%m)\)\  + \(\((10 - 12  \[Alpha])\) 
          u\_n\%m\ \)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "once with \[Alpha] \[RightArrow]\[AutoOperand] ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Alpha]/4\ \)\)]],
  ", then the three-time-level scheme once with ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]/4\)]],
  " and then again with ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]/2\)]],
  ". "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This gives us our vector ",
  Cell[BoxData[
      \(TraditionalForm\`u\^2\)]],
  " to allow the three-time-level iteration to proceed normally thereafter. \
We shall develop SOR and PSOR variations of this scheme in Chapter 16 to \
treat American-style options."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Some Required Functions from Chapter 14", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(ExplicitSolver\  = \n\t
      Compile[{{initial, _Real, 1}, \ {lower, _Real, 1}, \ {upper, _Real, 1}, 
          alpha}, \n\t\tModule[{wold = initial, \ wnew = initial, \ wvold, m, 
            k, \ tsize\  = \ Length[lower], 
            xsize = Length[initial]}, \n\t\t\tFor[m = 2, \ 
            m\  <= \ tsize, \ \(m++\), \n\t\t\t\t\((wvold\  = \ wold; \ 
              wold = wnew; \n\t\t\t\t\tFor[k = 2, \ 
                k\  < \ xsize, \ \(k++\), \n\t\t\t\t\t\((wnew[\([k]\)]\  = \ 
                    alpha\ \((wold[\([k - 1]\)] + 
                            wold[\([k + 1]\)])\) + \((1 - 
                            2  alpha)\)\ wold[\([k]\)])\)]; \n\t\t\t\t\t\t\
wnew[\([1]\)]\  = \ lower[\([m]\)]; \n\t\t\t\t\t\twnew[\([xsize]\)]\  = \ 
                upper[\([m]\)])\)]; \n\t\t\t{wvold, wold, 
            wnew}]\n\t\t]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True],

Cell[BoxData[
    \(\(CompTridiagSolve\  = \ \t\t\tCompile[{{a, \ _Real, \ 1}, {b, _Real, \ 
            1}, {c, \ _Real, \ 1}, {r, \ _Real, \ 
            1}}, \n\ \ \ \ Module[{len = Length[r], \ solution = r, \ 
            aux\  = \ 1/\((b[\([1]\)])\), \ aux1 = r, \ 
            a1\  = \ Prepend[a, \ 0.0], \ iter}, \ \tsolution[\([1]\)]\  = \ 
            aux\ *r[\([1]\)]; \[IndentingNewLine]Do[
            aux1[\([iter]\)]\  = \ 
              c[\([iter - 1]\)]\ aux; \n\t\t\t\ \ aux\  = \ 
              1/\((b[\([iter]\)]\  - \ 
                    a1[\([iter]\)]*
                      aux1[\([iter]\)])\); \n\t\t\t\tsolution[\([iter]\)]\  = \
\ \((r[\([iter]\)] - 
                    a1[\([iter]\)]\ solution[\([iter - 
                            1]\)])\)\ aux, \n\t\ \ \ \ {iter, \ 2, \ 
              len}]; \ \t\t\t\t\[IndentingNewLine]Do[
            solution[\([iter]\)]\  -= \ 
              aux1[\([iter + 1]\)]\ solution[\([iter + 
                      1]\)], \n\t\ \ \ \ {iter, \ len - 1, \ 
              1, \ \(-1\)}]; \[IndentingNewLine]\t\ \ \ \ solution]]; \)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
FullyImpCMatrix[alpha_, nminus_, nplus_] :=
Sequence[Table[-alpha, {nplus+nminus-2}], 
Table[1+2*alpha, {nplus+nminus-1}],
Table[-alpha, {nplus+nminus-2}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["15.3 Case Study 1: the Vanilla European Put Option", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We now begin a detailed study of the first of two examples that we \
have picked for detailed investigation. The reader might wonder why we are \
considering such a trivial case for which there is a known analytic solution. \
Our goal here is to try out various difference schemes and find out what \
works well, by testing them on a case for which the solution is known and \
where the errors can be precisely described. In this way we can see what is \
happening without all the complications of other real-world effects.\
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Factors Common to All Our FD Schemes", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["The Black-Scholes differential equation", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`1\/2\ S\^2\ \[PartialD]\^2 V\/\[PartialD]S\^2\ \
\[Sigma]\^2 - 
        r\ V + \((r - 
              q)\)\ S\ \[PartialD]V\/\[PartialD]S + \
\[PartialD]V\/\[PartialD]t = 0\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "with constant coefficients (in particular ",
  Cell[BoxData[
      \(TraditionalForm\`r\)]],
  " is a constant) can be transformed into the diffusion equation (1) by a \
standard change of variables, as given by Wilmott ",
  StyleBox["et al",
    FontSlant->"Italic"],
  " (Chapter 17, equations 17.1, 17.2). We can therefore implement our four \
standard schemes on the problem, and use solutions of (3) with known payoffs. \
In the following example we use the vanilla European Put. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Standardization of Variables", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "With a strike ",
  Cell[BoxData[
      \(TraditionalForm\`K\)]],
  " and constant parameters ",
  Cell[BoxData[
      \(TraditionalForm\`r, \ q, \ \[Sigma]\)]],
  ", we make the changes of variables"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(TraditionalForm\`\[Tau] = \(\(\[Sigma]\^2\ \((T - t)\)\)\/2\ \ \ \ \ \ \
\ k\_1 = \(\(2\ r\)\/\[Sigma]\^2\ \ \ \ \ \ \ k\_2 = \(2\ \((r - q)\)\)\/\
\[Sigma]\^2\)\)\), 
    \(TraditionalForm\`V(S, \ t) = 
      K\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \((k\_2 - 1)\)\ x - \((1\/4\ \
\((k\_2 - 1)\)\^2 + k\_1)\)\ \[Tau]\)\ \(u(x, \ \[Tau])\)\)}], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[TextData[{
  "The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " implementation of this requires the following functions."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(\(NonDimExpiry[T_, \[Sigma]_] := \(\[Sigma]\^2\ T\)\/2; \)\), 
    \(kone[r_, \[Sigma]_] := \(2  r\)\/\[Sigma]\^2; \ \ \ \ ktwo[r_, q_, 
        sd_] := \(2\ \((r - q)\)\)\/sd\^2; \n
    ValuationMultiplier[strike_, r_, q_, x_, tau_, sd_] := 
      strike\ Exp[\(-\(1\/2\)\)\ \((ktwo[r, q, sd] - 
                  1)\)\ x - \((1\/4\ \((ktwo[r, q, sd] - 1)\)\^2 + 
                  kone[r, sd])\)\ tau]\)}], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initial (Expiry) Conditions", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
The initial conditions for a Call and a Put, in the transformed \
coordinates, are\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(\(CallExercise[x_, r_, q_, sd_] := 
      Max[Exp[1\/2\ \((ktwo[r, q, sd] - 1)\)\ x]\ \((Exp[x] - 1)\), 
        0]; \)\), 
    \(\(PutExercise[x_, r_, q_, sd_] := 
      Max[Exp[1\/2\ \((ktwo[r, q, sd] - 1)\)\ x]\ \((1 - Exp[x])\), 
        0]; \)\)}], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Black-Scholes Model for Verification", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["We load our standard Package for vanilla European options:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Derivatives`BlackScholes`\>"]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["\<\
We test the loading of the Package by executing the function we \
shall use in our test problem:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesPut[8, \ 10, \ 0.2, \ 0.05, \ 0, \ 3]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1.47044996714742204`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Put Boundary Conditions", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
For our boundary conditions the upper boundary condition is to set \
the function to zero, while the lower takes the asymptotic Put value as the \
stock price approaches zero.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(\(g[x_, tau_, r_, q_, sd_] := 0; \)\), 
    \(\(f[x_, tau_, r_, q_, sd_] := 
      Exp[1\/2\ \((ktwo[r, q, sd] - 1)\)\ x + 
            1\/4\ \((\((ktwo[r, q, sd] - 1)\)\^2 + 4\ kone[r, sd])\)\ tau] - 
        Exp[1\/2\ \((ktwo[r, q, sd] + 1)\)\ x + 
            1\/4\ \((\((ktwo[r, q, sd] - 1)\)\^2 + 
                  4\ kone[r, sd])\)\ tau]; \)\)}], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["15.4 Explicit Scheme for Put", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(dx = 0.025; \ dtau = 0.00025; \ alpha = dtau\/dx\^2; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(M = 400; \ nminus = 160; \ nplus = 160; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Setting the Initial (Expiry) Condition and Boundary Conditions\
\>",
   "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(initial = 
      Table[PutExercise[\((k - 1 - nminus)\)\ dx, 0.05, 0, 0.2], \ {k, 
          nminus + nplus + 1}]; \n
    lower = Table[
        f[\(-nminus\)\ dx, \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \n
    upper = Table[
        g[\(+nplus\)\ dx, \ \ \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Note that the payoff looks slightly odd in these coordinates - the \
main point to notice is the discontinuity in slope at the strike.\
\>", "Text",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[initial]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00296692 0.0147151 1.94467 [
[.17216 .00222 -6 -9 ]
[.17216 .00222 6 0 ]
[.3205 .00222 -9 -9 ]
[.3205 .00222 9 0 ]
[.46885 .00222 -9 -9 ]
[.46885 .00222 9 0 ]
[.61719 .00222 -9 -9 ]
[.61719 .00222 9 0 ]
[.76554 .00222 -9 -9 ]
[.76554 .00222 9 0 ]
[.91389 .00222 -9 -9 ]
[.91389 .00222 9 0 ]
[.01131 .11195 -24 -4.5 ]
[.01131 .11195 0 4.5 ]
[.01131 .20918 -18 -4.5 ]
[.01131 .20918 0 4.5 ]
[.01131 .30642 -24 -4.5 ]
[.01131 .30642 0 4.5 ]
[.01131 .40365 -18 -4.5 ]
[.01131 .40365 0 4.5 ]
[.01131 .50088 -24 -4.5 ]
[.01131 .50088 0 4.5 ]
[.01131 .59812 -18 -4.5 ]
[.01131 .59812 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17216 .01472 m
.17216 .02097 L
s
[(50)] .17216 .00222 0 1 Mshowa
.3205 .01472 m
.3205 .02097 L
s
[(100)] .3205 .00222 0 1 Mshowa
.46885 .01472 m
.46885 .02097 L
s
[(150)] .46885 .00222 0 1 Mshowa
.61719 .01472 m
.61719 .02097 L
s
[(200)] .61719 .00222 0 1 Mshowa
.76554 .01472 m
.76554 .02097 L
s
[(250)] .76554 .00222 0 1 Mshowa
.91389 .01472 m
.91389 .02097 L
s
[(300)] .91389 .00222 0 1 Mshowa
.125 Mabswid
.05348 .01472 m
.05348 .01847 L
s
.08315 .01472 m
.08315 .01847 L
s
.11282 .01472 m
.11282 .01847 L
s
.14249 .01472 m
.14249 .01847 L
s
.20182 .01472 m
.20182 .01847 L
s
.23149 .01472 m
.23149 .01847 L
s
.26116 .01472 m
.26116 .01847 L
s
.29083 .01472 m
.29083 .01847 L
s
.35017 .01472 m
.35017 .01847 L
s
.37984 .01472 m
.37984 .01847 L
s
.40951 .01472 m
.40951 .01847 L
s
.43918 .01472 m
.43918 .01847 L
s
.49852 .01472 m
.49852 .01847 L
s
.52819 .01472 m
.52819 .01847 L
s
.55785 .01472 m
.55785 .01847 L
s
.58752 .01472 m
.58752 .01847 L
s
.64686 .01472 m
.64686 .01847 L
s
.67653 .01472 m
.67653 .01847 L
s
.7062 .01472 m
.7062 .01847 L
s
.73587 .01472 m
.73587 .01847 L
s
.79521 .01472 m
.79521 .01847 L
s
.82488 .01472 m
.82488 .01847 L
s
.85455 .01472 m
.85455 .01847 L
s
.88422 .01472 m
.88422 .01847 L
s
.94355 .01472 m
.94355 .01847 L
s
.97322 .01472 m
.97322 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .11195 m
.03006 .11195 L
s
[(0.05)] .01131 .11195 1 0 Mshowa
.02381 .20918 m
.03006 .20918 L
s
[(0.1)] .01131 .20918 1 0 Mshowa
.02381 .30642 m
.03006 .30642 L
s
[(0.15)] .01131 .30642 1 0 Mshowa
.02381 .40365 m
.03006 .40365 L
s
[(0.2)] .01131 .40365 1 0 Mshowa
.02381 .50088 m
.03006 .50088 L
s
[(0.25)] .01131 .50088 1 0 Mshowa
.02381 .59812 m
.03006 .59812 L
s
[(0.3)] .01131 .59812 1 0 Mshowa
.125 Mabswid
.02381 .03416 m
.02756 .03416 L
s
.02381 .05361 m
.02756 .05361 L
s
.02381 .07306 m
.02756 .07306 L
s
.02381 .0925 m
.02756 .0925 L
s
.02381 .1314 m
.02756 .1314 L
s
.02381 .15084 m
.02756 .15084 L
s
.02381 .17029 m
.02756 .17029 L
s
.02381 .18974 m
.02756 .18974 L
s
.02381 .22863 m
.02756 .22863 L
s
.02381 .24808 m
.02756 .24808 L
s
.02381 .26752 m
.02756 .26752 L
s
.02381 .28697 m
.02756 .28697 L
s
.02381 .32586 m
.02756 .32586 L
s
.02381 .34531 m
.02756 .34531 L
s
.02381 .36476 m
.02756 .36476 L
s
.02381 .3842 m
.02756 .3842 L
s
.02381 .4231 m
.02756 .4231 L
s
.02381 .44254 m
.02756 .44254 L
s
.02381 .46199 m
.02756 .46199 L
s
.02381 .48144 m
.02756 .48144 L
s
.02381 .52033 m
.02756 .52033 L
s
.02381 .53978 m
.02756 .53978 L
s
.02381 .55922 m
.02756 .55922 L
s
.02381 .57867 m
.02756 .57867 L
s
.02381 .61756 m
.02756 .61756 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.02678 .10976 Mdot
.02974 .11151 Mdot
.03271 .1133 Mdot
.03568 .11511 Mdot
.03864 .11696 Mdot
.04161 .11884 Mdot
.04458 .12076 Mdot
.04754 .1227 Mdot
.05051 .12469 Mdot
.05348 .1267 Mdot
.05645 .12876 Mdot
.05941 .13084 Mdot
.06238 .13297 Mdot
.06535 .13513 Mdot
.06831 .13733 Mdot
.07128 .13956 Mdot
.07425 .14184 Mdot
.07721 .14415 Mdot
.08018 .1465 Mdot
.08315 .1489 Mdot
.08611 .15133 Mdot
.08908 .15381 Mdot
.09205 .15633 Mdot
.09502 .15889 Mdot
.09798 .16149 Mdot
.10095 .16414 Mdot
.10392 .16683 Mdot
.10688 .16957 Mdot
.10985 .17235 Mdot
.11282 .17518 Mdot
.11578 .17805 Mdot
.11875 .18097 Mdot
.12172 .18394 Mdot
.12468 .18696 Mdot
.12765 .19002 Mdot
.13062 .19314 Mdot
.13359 .19631 Mdot
.13655 .19952 Mdot
.13952 .20279 Mdot
.14249 .20611 Mdot
.14545 .20948 Mdot
.14842 .2129 Mdot
.15139 .21638 Mdot
.15435 .21991 Mdot
.15732 .22349 Mdot
.16029 .22713 Mdot
.16325 .23082 Mdot
.16622 .23457 Mdot
.16919 .23837 Mdot
.17216 .24223 Mdot
.17512 .24615 Mdot
.17809 .25012 Mdot
.18106 .25415 Mdot
.18402 .25824 Mdot
.18699 .26238 Mdot
.18996 .26658 Mdot
.19292 .27084 Mdot
.19589 .27516 Mdot
.19886 .27953 Mdot
.20182 .28397 Mdot
.20479 .28846 Mdot
.20776 .29301 Mdot
.21073 .29762 Mdot
.21369 .30228 Mdot
.21666 .30701 Mdot
.21963 .31179 Mdot
.22259 .31662 Mdot
.22556 .32152 Mdot
.22853 .32646 Mdot
.23149 .33147 Mdot
.23446 .33653 Mdot
.23743 .34164 Mdot
.24039 .34681 Mdot
.24336 .35202 Mdot
.24633 .35729 Mdot
.2493 .36261 Mdot
.25226 .36798 Mdot
.25523 .37339 Mdot
.2582 .37886 Mdot
.26116 .38436 Mdot
.26413 .38991 Mdot
.2671 .39549 Mdot
.27006 .40112 Mdot
.27303 .40678 Mdot
.276 .41247 Mdot
.27896 .41819 Mdot
.28193 .42395 Mdot
.2849 .42972 Mdot
.28787 .43552 Mdot
.29083 .44133 Mdot
.2938 .44716 Mdot
.29677 .453 Mdot
.29973 .45885 Mdot
.3027 .46469 Mdot
.30567 .47053 Mdot
.30863 .47636 Mdot
.3116 .48218 Mdot
.31457 .48798 Mdot
.31753 .49375 Mdot
.3205 .49949 Mdot
.32347 .50519 Mdot
.32644 .51083 Mdot
.3294 .51643 Mdot
.33237 .52196 Mdot
.33534 .52742 Mdot
.3383 .53279 Mdot
.34127 .53807 Mdot
.34424 .54326 Mdot
.3472 .54832 Mdot
.35017 .55327 Mdot
.35314 .55807 Mdot
.3561 .56273 Mdot
.35907 .56722 Mdot
.36204 .57154 Mdot
.36501 .57566 Mdot
.36797 .57957 Mdot
.37094 .58326 Mdot
.37391 .5867 Mdot
.37687 .58988 Mdot
.37984 .59278 Mdot
.38281 .59538 Mdot
.38577 .59765 Mdot
.38874 .59958 Mdot
.39171 .60113 Mdot
.39467 .60229 Mdot
.39764 .60303 Mdot
.40061 .60332 Mdot
.40358 .60313 Mdot
.40654 .60242 Mdot
.40951 .60118 Mdot
.41248 .59935 Mdot
.41544 .59692 Mdot
.41841 .59383 Mdot
.42138 .59006 Mdot
.42434 .58555 Mdot
.42731 .58028 Mdot
.43028 .57418 Mdot
.43324 .56721 Mdot
.43621 .55933 Mdot
.43918 .55048 Mdot
.44215 .54061 Mdot
.44511 .52965 Mdot
.44808 .51755 Mdot
.45105 .50425 Mdot
.45401 .48967 Mdot
.45698 .47375 Mdot
.45995 .45641 Mdot
.46291 .43758 Mdot
.46588 .41719 Mdot
.46885 .39513 Mdot
.47181 .37133 Mdot
.47478 .34569 Mdot
.47775 .31812 Mdot
.48072 .28852 Mdot
.48368 .25677 Mdot
.48665 .22277 Mdot
.48962 .1864 Mdot
.49258 .14754 Mdot
.49555 .10607 Mdot
.49852 .06184 Mdot
.50148 .01472 Mdot
.50445 .01472 Mdot
.50742 .01472 Mdot
.51038 .01472 Mdot
.51335 .01472 Mdot
.51632 .01472 Mdot
.51928 .01472 Mdot
.52225 .01472 Mdot
.52522 .01472 Mdot
.52819 .01472 Mdot
.53115 .01472 Mdot
.53412 .01472 Mdot
.53709 .01472 Mdot
.54005 .01472 Mdot
.54302 .01472 Mdot
.54599 .01472 Mdot
.54895 .01472 Mdot
.55192 .01472 Mdot
.55489 .01472 Mdot
.55785 .01472 Mdot
.56082 .01472 Mdot
.56379 .01472 Mdot
.56676 .01472 Mdot
.56972 .01472 Mdot
.57269 .01472 Mdot
.57566 .01472 Mdot
.57862 .01472 Mdot
.58159 .01472 Mdot
.58456 .01472 Mdot
.58752 .01472 Mdot
.59049 .01472 Mdot
.59346 .01472 Mdot
.59642 .01472 Mdot
.59939 .01472 Mdot
.60236 .01472 Mdot
.60533 .01472 Mdot
.60829 .01472 Mdot
.61126 .01472 Mdot
.61423 .01472 Mdot
.61719 .01472 Mdot
.62016 .01472 Mdot
.62313 .01472 Mdot
.62609 .01472 Mdot
.62906 .01472 Mdot
.63203 .01472 Mdot
.63499 .01472 Mdot
.63796 .01472 Mdot
.64093 .01472 Mdot
.6439 .01472 Mdot
.64686 .01472 Mdot
.64983 .01472 Mdot
.6528 .01472 Mdot
.65576 .01472 Mdot
.65873 .01472 Mdot
.6617 .01472 Mdot
.66466 .01472 Mdot
.66763 .01472 Mdot
.6706 .01472 Mdot
.67356 .01472 Mdot
.67653 .01472 Mdot
.6795 .01472 Mdot
.68247 .01472 Mdot
.68543 .01472 Mdot
.6884 .01472 Mdot
.69137 .01472 Mdot
.69433 .01472 Mdot
.6973 .01472 Mdot
.70027 .01472 Mdot
.70323 .01472 Mdot
.7062 .01472 Mdot
.70917 .01472 Mdot
.71213 .01472 Mdot
.7151 .01472 Mdot
.71807 .01472 Mdot
.72104 .01472 Mdot
.724 .01472 Mdot
.72697 .01472 Mdot
.72994 .01472 Mdot
.7329 .01472 Mdot
.73587 .01472 Mdot
.73884 .01472 Mdot
.7418 .01472 Mdot
.74477 .01472 Mdot
.74774 .01472 Mdot
.7507 .01472 Mdot
.75367 .01472 Mdot
.75664 .01472 Mdot
.75961 .01472 Mdot
.76257 .01472 Mdot
.76554 .01472 Mdot
.76851 .01472 Mdot
.77147 .01472 Mdot
.77444 .01472 Mdot
.77741 .01472 Mdot
.78037 .01472 Mdot
.78334 .01472 Mdot
.78631 .01472 Mdot
.78927 .01472 Mdot
.79224 .01472 Mdot
.79521 .01472 Mdot
.79818 .01472 Mdot
.80114 .01472 Mdot
.80411 .01472 Mdot
.80708 .01472 Mdot
.81004 .01472 Mdot
.81301 .01472 Mdot
.81598 .01472 Mdot
.81894 .01472 Mdot
.82191 .01472 Mdot
.82488 .01472 Mdot
.82784 .01472 Mdot
.83081 .01472 Mdot
.83378 .01472 Mdot
.83675 .01472 Mdot
.83971 .01472 Mdot
.84268 .01472 Mdot
.84565 .01472 Mdot
.84861 .01472 Mdot
.85158 .01472 Mdot
.85455 .01472 Mdot
.85751 .01472 Mdot
.86048 .01472 Mdot
.86345 .01472 Mdot
.86641 .01472 Mdot
.86938 .01472 Mdot
.87235 .01472 Mdot
.87532 .01472 Mdot
.87828 .01472 Mdot
.88125 .01472 Mdot
.88422 .01472 Mdot
.88718 .01472 Mdot
.89015 .01472 Mdot
.89312 .01472 Mdot
.89608 .01472 Mdot
.89905 .01472 Mdot
.90202 .01472 Mdot
.90498 .01472 Mdot
.90795 .01472 Mdot
.91092 .01472 Mdot
.91389 .01472 Mdot
.91685 .01472 Mdot
.91982 .01472 Mdot
.92279 .01472 Mdot
.92575 .01472 Mdot
.92872 .01472 Mdot
.93169 .01472 Mdot
.93465 .01472 Mdot
.93762 .01472 Mdot
.94059 .01472 Mdot
.94355 .01472 Mdot
.94652 .01472 Mdot
.94949 .01472 Mdot
.95246 .01472 Mdot
.95542 .01472 Mdot
.95839 .01472 Mdot
.96136 .01472 Mdot
.96432 .01472 Mdot
.96729 .01472 Mdot
.97026 .01472 Mdot
.97322 .01472 Mdot
.97619 .01472 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  GroupPageBreakWithin->True,
  AspectRatioFixed->True,
  ImageSize->{222.938, 137.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003N0000RA000`40O003h00OgWoo003NOol00=io
o`00gWoo000cOol30004Ool2000AOol30004Ool20004Ool2000?Ool30003Ool30004Ool2000?Ool4
0003Ool20004Ool2000>Ool40002Ool30004Ool2000?Ool30004Ool20004Ool2000;Ool003Ioo`04
001oogoo0002Ool00`00Oomoo`0?Ool00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol000mo
o`03001oogoo00Eoo`04001oogoo0002Ool00`00Oomoo`0=Ool00`00Oomoo`02Ool01000Oomoo`00
0Woo00@007ooOol000ioo`03001oogoo00Eoo`04001oogoo0002Ool00`00Oomoo`0?Ool01000Oomo
o`000Woo00@007ooOol0009oo`03001oogoo00Qoo`00=Woo00@007ooOol0009oo`03001oogoo00mo
o`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`003goo00<007ooOol01Goo00@007ooOol0
009oo`03001oogoo00ioo`05001oogooOol00002Ool01000Oomoo`000Woo00<007ooOol03Goo00<0
07ooOol017oo00@007ooOol0009oo`03001oogoo00moo`04001oogoo0002Ool01000Oomoo`000Woo
00<007ooOol027oo000cOol30003Ool01000Oomoo`004Goo00<007ooOol00Woo00@007ooOol0009o
o`04001oogoo000?Ool00`00Oomoo`02Ool30003Ool01000Oomoo`004Goo00@007ooOol0009oo`04
001oogoo0002Ool00`00Oomoo`0>Ool00`00Oomoo`030003Ool01000Oomoo`003goo0P000goo00@0
07ooOol0009oo`04001oogoo000:Ool003=oo`03001oogoo00=oo`04001oogoo000AOol00`00Oomo
o`02Ool01000Oomoo`000Woo00@007ooOol000moo`03001oogoo009oo`03001oogoo00=oo`04001o
ogoo000>Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo000=Ool01000Oomoo`000Woo
00<007ooOol00goo00@007ooOol0015oo`04001oogoo0002Ool01000Oomoo`000Woo00<007ooOol0
27oo000cOol40003Ool2000AOol20005Ool20004Ool2000?Ool20004Ool40003Ool2000@Ool20004
Ool20004Ool2000?Ool20003Ool40003Ool2000?Ool30004Ool20004Ool2000;Ool001Yoo`03001o
ogoo0<5oo`006Woo00<007ooOol0`Goo000JOol00`00Oomoo`31Ool001EoolT000006Woo00<007oo
Ool00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007oo
Ool00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007oo
Ool00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007oo
Ool00gooH0001Goo000JOol00`00Oomoo`0KOol00`00Oomoo`0JOol00`00Oomoo`0KOol00`00Oomo
o`0KOol00`00Oomoo`0JOol00`00Oomoo`0KOol00`00Oomoo`0?Ool001Yoo`03001oogoo0<5oo`00
6Woo0P00`Woo000JOol00`00Oomoo`31Ool001Yoo`03001oogoo0<5oo`006Woo00<007ooOol0`Goo
000JOol20032Ool001Yoo`03001oogoo0<5oo`006Woo00<007ooOol0G7oo00<007ooOol0HWoo000J
Ool00`00Oomoo`31Ool001Yoo`800<9oo`006Woo00<007ooOol0`Goo000JOol00`00Oomoo`31Ool0
01Yoo`03001oogoo0<5oo`006Woo0P00`Woo000JOol00`00Oomoo`31Ool0009oo`8000Aoo`8000Ao
o`8000=oo`<000Aoo`03001oogoo0<5oo`0000Eoo`00Oomoo`0000=oo`8000=oo`04001oogoo0005
Ool01@00Oomoogoo0000GGoo00<007ooOol0Hgoo00001Goo001oogoo000027oo00@007ooOol000Eo
o`04001oogooOol30031Ool00005Ool007ooOol00008Ool01000Oomoo`000Woo0`0017oo00@007oo
00000<1oo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool00`00Oomoo`04Ool01000Oomoogoo
0P00_Woo0002Ool2000:Ool20003Ool40003Ool00`00Oomoo`02Ool2002mOol001Yoo`8000Eoo`03
001oogoo0;Yoo`006Woo00<007ooOol01Goo0P00^Woo000JOol00`00Oomoo`06Ool2002iOol001Yo
o`03001oogoo00Moo`<0055oo`03001oogoo06=oo`006Woo0P002Woo0P00]Woo000JOol00`00Oomo
o`0:Ool2002eOol001Yoo`03001oogoo00aoo`03001oogoo0;9oo`006Woo00<007ooOol037oo0P00
/goo000JOol2000>Ool2002bOol001Yoo`03001oogoo00moo`03001oogoo0:moo`006Woo00<007oo
Ool047oo00<007ooOol0[Woo000JOol2000AOol20018Ool00`00Oomoo`1TOol001Yoo`03001oogoo
019oo`03001oogoo0:aoo`0027oo0P0017oo0P000goo0`0017oo00<007ooOol04goo00<007ooOol0
Zgoo0007Ool01000Oomoo`000goo0P0017oo00<007ooOol00goo00<007ooOol04goo0P00[7oo0007
Ool01000Oomoo`002Goo00<007ooOol00goo0P005Woo00<007ooOol0ZGoo0007Ool01000Oomoo`00
2Goo00<007ooOol00goo00<007ooOol05Goo0P00ZWoo0007Ool01000Oomoo`002Goo00<007ooOol0
0goo00<007ooOol05Woo00<007ooOol0Z7oo0008Ool20009Ool20005Ool00`00Oomoo`0GOol00`00
Oomoo`0oOol00`00Oomoo`1UOol001Yoo`8001Uoo`03001oogoo0:Ioo`006Woo00<007ooOol06Goo
00<007ooOol0YGoo000JOol00`00Oomoo`0IOol2002VOol001Yoo`03001oogoo01Yoo`03001oogoo
0:Aoo`006Woo0P0077oo00<007ooOol0Xgoo000JOol00`00Oomoo`0LOol00`00Oomoo`2ROol001Yo
o`03001oogoo01aoo`8003]oo`03001oogoo06Eoo`006Woo00<007ooOol07Goo00<007ooOol0XGoo
000JOol2000OOol00`00Oomoo`2POol001Yoo`03001oogoo01moo`03001oogoo09moo`006Woo00<0
07ooOol07goo00<007ooOol0Wgoo000JOol00`00Oomoo`0POol00`00Oomoo`2NOol001Yoo`80025o
o`8003Ioo`03001oogoo06Ioo`006Woo00<007ooOol08Woo00<007ooOol0W7oo0002Ool20004Ool2
0003Ool30003Ool30004Ool00`00Oomoo`0ROol00`00Oomoo`2LOol00005Ool007ooOol00003Ool2
0004Ool00`00Oomoo`05Ool01@00Oomoogoo00009Goo00<007ooOol0Vgoo00001Goo001oogoo0000
2Goo00<007ooOol01Goo00@007ooOomoo`8002Aoo`03001oogoo09]oo`0000Eoo`00Oomoo`0000Uo
o`03001oogoo009oo`<000Aoo`03001oogoo02Aoo`03001oogoo09Yoo`0000Eoo`00Oomoo`0000Uo
o`03001oogoo009oo`03001oogoo00Aoo`03001oogoo02Eoo`03001oogoo031oo`03001oogoo06Io
o`000Woo0P002Goo0P0017oo10000goo00<007ooOol09Goo00<007ooOol0VGoo000JOol2000WOol0
0`00Oomoo`2HOol001Yoo`03001oogoo02Ioo`03001oogoo09Qoo`006Woo00<007ooOol09goo00<0
07ooOol0Ugoo000JOol00`00Oomoo`0XOol00`00Oomoo`2FOol001Yoo`8002Uoo`03001oogoo02ao
o`03001oogoo06Moo`006Woo00<007ooOol0:Goo00<007ooOol0UGoo000JOol00`00Oomoo`0YOol0
0`00Oomoo`2EOol001Yoo`8002]oo`03001oogoo09Aoo`006Woo00<007ooOol0:Woo00<007ooOol0
U7oo000JOol00`00Oomoo`0[Ool00`00Oomoo`0XOol00`00Oomoo`1XOol001Yoo`03001oogoo0<5o
o`006Woo0P00;Goo00<007ooOol0TWoo000JOol00`00Oomoo`0/Ool00`00Oomoo`2BOol000Qoo`80
00Aoo`8000=oo`@000=oo`03001oogoo02eoo`03001oogoo095oo`001goo00@007ooOol000=oo`80
00Aoo`03001oogoo00=oo`03001oogoo02eoo`03001oogoo02Ioo`03001oogoo06Qoo`001goo00@0
07ooOol000Yoo`03001oogoo009oo`8002moo`03001oogoo091oo`001goo00@007ooOol000]oo`05
001oogooOol0000aOol00`00Oomoo`2?Ool000Moo`04001oogoo0008Ool01000Oomoo`000goo00<0
07ooOol0;goo00<007ooOol0Sgoo0008Ool2000:Ool20004Ool00`00Oomoo`0`Ool00`00Oomoo`0R
Ool00`00Oomoo`1YOol001Yoo`800<9oo`006Woo00<007ooOol0<7oo00<007ooOol0SWoo000JOol0
0`00Oomoo`0aOol00`00Oomoo`2=Ool001Yoo`03001oogoo039oo`03001oogoo021oo`03001oogoo
06Uoo`006Woo0P00<goo00<007ooOol0S7oo000JOol00`00Oomoo`0cOol00`00Oomoo`2;Ool001Yo
o`03001oogoo03=oo`03001oogoo08]oo`006Woo00<007ooOol0E7oo00<007ooOol0JWoo000JOol2
000eOol00`00Oomoo`2:Ool001Yoo`03001oogoo03Eoo`03001oogoo08Uoo`006Woo00<007ooOol0
=Goo00<007ooOol0RGoo000JOol00`00Oomoo`0fOol00`00Oomoo`0JOol00`00Oomoo`1[Ool001Yo
o`8003Moo`03001oogoo08Qoo`006Woo00<007ooOol0=goo00<007ooOol0Qgoo0002Ool20004Ool2
0003Ool40002Ool30004Ool00`00Oomoo`1COol00`00Oomoo`1[Ool00005Ool007ooOol00003Ool2
0004Ool00`00Oomoo`05Ool01@00Oomoogoo0000>Woo00<007ooOol0QWoo00001Goo001oogoo0000
2Woo00<007ooOol017oo00@007ooOomoo`8003Uoo`03001oogoo08Ioo`0000Eoo`00Oomoo`0000]o
o`03001oogoo00<000Aoo`03001oogoo03Uoo`03001oogoo01Ioo`03001oogoo06aoo`0000Eoo`00
Oomoo`0000Qoo`04001oogoo0002Ool00`00Oomoo`04Ool00`00Oomoo`0iOol00`00Oomoo`25Ool0
009oo`8000Yoo`8000=oo`@000=oo`03001oogoo03Yoo`03001oogoo01Eoo`03001oogoo06aoo`00
6Woo0P00?7oo00<007ooOol0Pgoo000JOol00`00Oomoo`0kOol00`00Oomoo`23Ool001Yoo`03001o
ogoo03aoo`03001oogoo019oo`03001oogoo06eoo`006Woo0P00?Goo00<007ooOol0PWoo000JOol0
0`00Oomoo`0mOol00`00Oomoo`0AOol00`00Oomoo`1]Ool001Yoo`03001oogoo03eoo`03001oogoo
085oo`006Woo00<007ooOol0?Woo00<007ooOol03goo00<007ooOol0KWoo000JOol20010Ool00`00
Oomoo`1oOol001Yoo`03001oogoo03moo`03001oogoo00eoo`03001oogoo06moo`006Woo00<007oo
Ool0@7oo00<007ooOol037oo00<007ooOol0Kgoo000JOol00`00Oomoo`10Ool00`00Oomoo`1nOol0
01Yoo`80049oo`03001oogoo00Yoo`03001oogoo071oo`006Woo00<007ooOol0@Woo00<007ooOol0
2Goo00<007ooOol0L7oo0008Ool20004Ool20003Ool30004Ool00`00Oomoo`13Ool00`00Oomoo`07
Ool00`00Oomoo`1aOol000Moo`04001oogoo0003Ool20006Ool01@00Oomoogoo0000AGoo0P001goo
00<007ooOol0LWoo0007Ool01000Oomoo`002goo00@007ooOomoo`8004Ioo`03001oogoo00=oo`80
07Aoo`001goo00@007ooOol000Uoo`8000Aoo`03001oogoo04Ioo`H007Eoo`001goo00@007ooOol0
00]oo`05001oogooOol00033Ool000Qoo`8000Uoo`<000Aoo`03001oogoo0<5oo`006Woo0P00`Woo
003NOol00=ioo`00\
\>"],
  ImageRangeCache->{{{0, 221.938}, {136.438, 0}} -> {-44.6202, -0.0342152, 
  1.68357, 0.00256856}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solving the PDE (Explicit Method)", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We use the same functions as we used for our smooth test problem in \
Chapter 14 - load the function ",
  StyleBox["ExplicitSolver",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " now if you are using the electronic form:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(soln\  = \ ExplicitSolver[initial, lower, upper, \ alpha\ ]; \)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Interpolating to Supply a Continuous Function", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(interpoldata = 
      Table[{\((k - nminus - 1)\)\ dx, soln[\([3, k]\)]}, {k, 1, 
          nminus + nplus + 1}]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(ufunc = 
      Interpolation[interpoldata, InterpolationOrder \[Rule] 3]; \)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(Valuation[strike_, r_, q_, S_, T_, sd_] := 
      ValuationMultiplier[strike, r, q, Log[S\/strike], \(sd\^2\ T\)\/2, 
          sd]\ ufunc[Log[S\/strike]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Error Plot", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      Valuation[10, 0.05, 0, S, 5, 0.2] - 
        BlackScholesPut[S, 10, 0.2, 0.05, 0, 5], {S, 1, 20}, 
      PlotPoints \[Rule] 50, PlotRange \[Rule] All]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0263158 0.0501253 0.505679 1015.91 [
[.099 .49318 -9 -9 ]
[.099 .49318 9 0 ]
[.34962 .49318 -9 -9 ]
[.34962 .49318 9 0 ]
[.47494 .49318 -6 -9 ]
[.47494 .49318 6 0 ]
[.60025 .49318 -12 -9 ]
[.60025 .49318 12 0 ]
[.72556 .49318 -6 -9 ]
[.72556 .49318 6 0 ]
[.85088 .49318 -12 -9 ]
[.85088 .49318 12 0 ]
[.97619 .49318 -6 -9 ]
[.97619 .49318 6 0 ]
[.21181 .09932 -42 -4.5 ]
[.21181 .09932 0 4.5 ]
[.21181 .20091 -42 -4.5 ]
[.21181 .20091 0 4.5 ]
[.21181 .3025 -42 -4.5 ]
[.21181 .3025 0 4.5 ]
[.21181 .40409 -42 -4.5 ]
[.21181 .40409 0 4.5 ]
[.21181 .60727 -36 -4.5 ]
[.21181 .60727 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.099 .50568 m
.099 .51193 L
s
[(2.5)] .099 .49318 0 1 Mshowa
.34962 .50568 m
.34962 .51193 L
s
[(7.5)] .34962 .49318 0 1 Mshowa
.47494 .50568 m
.47494 .51193 L
s
[(10)] .47494 .49318 0 1 Mshowa
.60025 .50568 m
.60025 .51193 L
s
[(12.5)] .60025 .49318 0 1 Mshowa
.72556 .50568 m
.72556 .51193 L
s
[(15)] .72556 .49318 0 1 Mshowa
.85088 .50568 m
.85088 .51193 L
s
[(17.5)] .85088 .49318 0 1 Mshowa
.97619 .50568 m
.97619 .51193 L
s
[(20)] .97619 .49318 0 1 Mshowa
.125 Mabswid
.12406 .50568 m
.12406 .50943 L
s
.14912 .50568 m
.14912 .50943 L
s
.17419 .50568 m
.17419 .50943 L
s
.19925 .50568 m
.19925 .50943 L
s
.24937 .50568 m
.24937 .50943 L
s
.27444 .50568 m
.27444 .50943 L
s
.2995 .50568 m
.2995 .50943 L
s
.32456 .50568 m
.32456 .50943 L
s
.37469 .50568 m
.37469 .50943 L
s
.39975 .50568 m
.39975 .50943 L
s
.42481 .50568 m
.42481 .50943 L
s
.44987 .50568 m
.44987 .50943 L
s
.5 .50568 m
.5 .50943 L
s
.52506 .50568 m
.52506 .50943 L
s
.55013 .50568 m
.55013 .50943 L
s
.57519 .50568 m
.57519 .50943 L
s
.62531 .50568 m
.62531 .50943 L
s
.65038 .50568 m
.65038 .50943 L
s
.67544 .50568 m
.67544 .50943 L
s
.7005 .50568 m
.7005 .50943 L
s
.75063 .50568 m
.75063 .50943 L
s
.77569 .50568 m
.77569 .50943 L
s
.80075 .50568 m
.80075 .50943 L
s
.82581 .50568 m
.82581 .50943 L
s
.87594 .50568 m
.87594 .50943 L
s
.901 .50568 m
.901 .50943 L
s
.92607 .50568 m
.92607 .50943 L
s
.95113 .50568 m
.95113 .50943 L
s
.07393 .50568 m
.07393 .50943 L
s
.04887 .50568 m
.04887 .50943 L
s
.02381 .50568 m
.02381 .50943 L
s
.25 Mabswid
0 .50568 m
1 .50568 L
s
.22431 .09932 m
.23056 .09932 L
s
[(-0.0004)] .21181 .09932 1 0 Mshowa
.22431 .20091 m
.23056 .20091 L
s
[(-0.0003)] .21181 .20091 1 0 Mshowa
.22431 .3025 m
.23056 .3025 L
s
[(-0.0002)] .21181 .3025 1 0 Mshowa
.22431 .40409 m
.23056 .40409 L
s
[(-0.0001)] .21181 .40409 1 0 Mshowa
.22431 .60727 m
.23056 .60727 L
s
[(0.0001)] .21181 .60727 1 0 Mshowa
.125 Mabswid
.22431 .01804 m
.22806 .01804 L
s
.22431 .03836 m
.22806 .03836 L
s
.22431 .05868 m
.22806 .05868 L
s
.22431 .079 m
.22806 .079 L
s
.22431 .11963 m
.22806 .11963 L
s
.22431 .13995 m
.22806 .13995 L
s
.22431 .16027 m
.22806 .16027 L
s
.22431 .18059 m
.22806 .18059 L
s
.22431 .22122 m
.22806 .22122 L
s
.22431 .24154 m
.22806 .24154 L
s
.22431 .26186 m
.22806 .26186 L
s
.22431 .28218 m
.22806 .28218 L
s
.22431 .32282 m
.22806 .32282 L
s
.22431 .34313 m
.22806 .34313 L
s
.22431 .36345 m
.22806 .36345 L
s
.22431 .38377 m
.22806 .38377 L
s
.22431 .42441 m
.22806 .42441 L
s
.22431 .44472 m
.22806 .44472 L
s
.22431 .46504 m
.22806 .46504 L
s
.22431 .48536 m
.22806 .48536 L
s
.22431 .526 m
.22806 .526 L
s
.22431 .54632 m
.22806 .54632 L
s
.22431 .56663 m
.22806 .56663 L
s
.22431 .58695 m
.22806 .58695 L
s
.25 Mabswid
.22431 0 m
.22431 .61803 L
s
.5 Mabswid
.02381 .56483 m
.04273 .58188 L
.05258 .59246 L
.05779 .59709 L
.06066 .59906 L
.06337 .60071 L
.06459 .60135 L
.06589 .60191 L
.06731 .60237 L
.06794 .60258 L
.06829 .60272 L
.06862 .60282 L
.06924 .60294 L
.0698 .60298 L
.07032 .6031 L
.0709 .60323 L
.07153 .60328 L
.07188 .60327 L
.0722 .60324 L
.07282 .60328 L
.0734 .60332 L
.07395 .60328 L
.07454 .60316 L
.07516 .60306 L
.07574 .60302 L
.07629 .60292 L
.07678 .60276 L
.0779 .60239 L
.07847 .60223 L
.07909 .60197 L
.07977 .6016 L
.08039 .60129 L
.08275 .59979 L
.08734 .59586 L
.08992 .59288 L
.09229 .58965 L
.10124 .57333 L
.11051 .54959 L
.12065 .51625 L
.14052 .433 L
.15964 .34006 L
.17997 .24114 L
.19955 .15686 L
.21839 .09229 L
.22813 .06671 L
.23843 .04512 L
.24305 .0376 L
.24802 .03071 L
.25267 .02521 L
.25699 .02144 L
Mistroke
.26195 .01814 L
.26457 .01678 L
.26591 .01617 L
.26662 .01591 L
.26736 .0157 L
.26802 .01555 L
.26837 .01549 L
.26875 .01543 L
.26909 .01539 L
.26941 .01536 L
.27002 .01532 L
.27064 .01519 L
.27131 .01502 L
.27189 .0149 L
.27253 .0148 L
.27314 .01474 L
.27371 .01472 L
.27431 .01472 L
.27465 .01473 L
.27496 .01475 L
.27548 .01481 L
.27605 .01489 L
.27665 .01501 L
.27721 .01514 L
.27841 .01531 L
.27873 .01534 L
.27907 .01537 L
.27969 .01546 L
.28035 .01558 L
.28109 .01576 L
.28237 .01617 L
.28725 .01835 L
.29201 .02127 L
.29636 .02447 L
.30569 .03349 L
.31586 .04596 L
.33582 .0764 L
.35503 .11027 L
.37351 .14468 L
.39318 .18123 L
.41212 .21487 L
.43225 .24783 L
.45164 .2761 L
.47029 .30014 L
.49014 .32195 L
.50925 .33968 L
.52956 .35505 L
.54913 .36718 L
.56795 .37636 L
.58798 .38409 L
Mistroke
.60726 .38995 L
.6258 .39427 L
.64555 .39776 L
.66455 .40032 L
.68475 .40242 L
.70421 .4041 L
.72292 .40547 L
.74284 .40678 L
.76201 .40799 L
.78239 .40931 L
.80203 .41064 L
.82092 .41203 L
.84101 .41365 L
.86036 .41534 L
.87897 .41709 L
.89878 .41909 L
.91785 .42113 L
.93811 .42342 L
.95764 .42574 L
.97619 .42799 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  GroupPageBreakWithin->True,
  AspectRatioFixed->True,
  ImageSize->{222.938, 137.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003N0000RA000`40O003h00O<Woo00<007ooOol0
ZGoo000bOol00`00Oomoo`2YOol0039oo`03001oogoo0:Uoo`00<Woo00<007ooOol01goo0`00Wgoo
000bOol20006Ool20003Ool2002MOol0039oo`03001oogoo00Aoo`03001oogoo00Eoo`8009]oo`00
<Woo00<007ooOol00goo00<007ooOol027oo00<007ooOol0V7oo000bOol00`00Oomoo`02Ool00`00
Oomoo`0:Ool00`00Oomoo`2GOol0039oo`80009oo`03001oogoo00]oo`03001oogoo09Moo`00<Woo
00D007ooOomoo`0000ioo`03001oogoo09Ioo`00<Woo00@007ooOol0011oo`03001oogoo09Eoo`00
<Woo00@007ooOol0011oo`03001oogoo09Eoo`00<Woo00<007oo00004Woo00<007ooOol0U7oo000b
Ool3000BOol00`00Oomoo`2DOol0039oo`03001oo`0001=oo`03001oogoo09=oo`00<Woo0P005Goo
00<007ooOol0TWoo000bOol2000EOol00`00Oomoo`2BOol0039oo`8001Ioo`03001oogoo095oo`00
<Woo00<007ooOol05Goo00<007ooOol0TGoo000bOol00`00Oomoo`0FOol00`00Oomoo`2@Ool000eo
o`8000Aoo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`<000=oo`8001Uoo`03001oogoo08moo`0037oo
00@007ooOol000=oo`8000=oo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol000Aoo`03
001oogoo009oo`8001Yoo`03001oogoo08ioo`001Woo1@0000Eoo`00Oomoo`0000Qoo`04001oogoo
0002Ool01000Oomoo`000Woo00@007ooOol0009oo`@000=oo`@001Qoo`03001oogoo08ioo`0037oo
00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`03001oo`00
00=oo`03001oo`0001]oo`03001oogoo08eoo`0037oo00@007ooOol000Qoo`04001oogoo0002Ool0
1000Oomoo`000Woo00@007ooOol0009oo`03001oo`0000=oo`03001oo`0001aoo`03001oogoo08ao
o`003Goo0P002Woo0P0017oo0P0017oo0P0017oo0P000goo00<007oo000077oo00<007ooOol0S7oo
000_Ool00`00Oomoo`02000LOol00`00Oomoo`2;Ool002moo`04001oogoo000MOol00`00Oomoo`2;
Ool002moo`04001oogoo000NOol00`00Oomoo`2:Ool002moo`04001oogoo000OOol00`00Oomoo`29
Ool002ioo`04001oogooOol2000NOol00`00Oomoo`29Ool002ioo`05001oogooOol0000POol00`00
Oomoo`28Ool002ioo`05001oogooOol0000POol00`00Oomoo`28Ool002eoo`03001oogoo009oo`03
001oogoo01moo`03001oogoo08Moo`00;Goo00<007ooOol00Woo00<007ooOol07goo00<007ooOol0
Qgoo000]Ool00`00Oomoo`02Ool2000QOol00`00Oomoo`26Ool002eoo`03001oogoo009oo`03001o
ogoo021oo`03001oogoo08Ioo`00;7oo00<007ooOol00goo00<007ooOol08Goo00<007ooOol0QGoo
000/Ool00`00Oomoo`03Ool00`00Oomoo`0QOol00`00Oomoo`25Ool002aoo`03001oogoo00=oo`80
02=oo`03001oogoo08Aoo`00;7oo00<007ooOol00goo00<007ooOol08Woo00<007ooOol0Q7oo000/
Ool00`00Oomoo`03Ool00`00Oomoo`0SOol00`00Oomoo`23Ool000eoo`8000Aoo`8000Aoo`8000Ao
o`8000Aoo`8000=oo`<000Eoo`03001oogoo02=oo`03001oogoo08=oo`0037oo00@007ooOol000=o
o`8000=oo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oo`0000Aoo`03
001oogoo02Aoo`03001oogoo089oo`001Woo1@0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool0
1000Oomoo`000Woo00@007ooOol000=oo`03001oo`0000Aoo`<002Aoo`03001oogoo089oo`0037oo
00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol000=oo`8000Eoo`03
001oogoo02Eoo`03001oogoo085oo`0037oo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomo
o`000Woo00@007ooOol0009oo`04001oogoo0004Ool00`00Oomoo`0UOol00`00Oomoo`21Ool000eo
o`8000Yoo`8000Aoo`8000Aoo`8000=oo`<000Eoo`03001oogoo02Ioo`03001oogoo081oo`00:Woo
00<007ooOol01Goo0P009goo00<007ooOol0P7oo000ZOol00`00Oomoo`05Ool00`00Oomoo`0WOol0
0`00Oomoo`1oOol002Yoo`03001oogoo00Eoo`03001oogoo02Moo`03001oogoo07moo`00:Goo00<0
07ooOol01Woo00<007ooOol0:7oo00<007ooOol0OWoo000YOol00`00Oomoo`06Ool00`00Oomoo`0Y
Ool00`00Oomoo`1mOol002Uoo`03001oogoo00Ioo`8002Yoo`03001oogoo07eoo`00:Goo00<007oo
Ool01Woo00<007ooOol0:Woo00<007ooOol0O7oo000XOol00`00Oomoo`07Ool00`00Oomoo`0ZOol0
0`00Oomoo`1lOol002Qoo`03001oogoo00Moo`03001oogoo02]oo`03001oogoo07]oo`00:7oo00<0
07ooOol01goo0P00;Goo00<007ooOol0NWoo000XOol00`00Oomoo`07Ool00`00Oomoo`0]Ool00`00
Oomoo`1iOol002Qoo`03001oogoo00Moo`03001oogoo02ioo`03001oogoo07Qoo`009goo00<007oo
Ool027oo00<007ooOol0;goo00<007ooOol0Mgoo000WOol00`00Oomoo`08Ool2000`Ool00`00Oomo
o`1gOol002Moo`03001oogoo00Qoo`03001oogoo031oo`03001oogoo07Ioo`009goo00<007ooOol0
27oo00<007ooOol0<Goo00<007ooOol0MGoo000=Ool20004Ool20004Ool20004Ool20004Ool20003
Ool40004Ool00`00Oomoo`0bOol00`00Oomoo`1dOol000aoo`04001oogoo0003Ool20003Ool01000
Oomoo`000Woo00@007ooOol0009oo`04001oo`000003Ool00`00Oomoo`04Ool00`00Oomoo`0cOol0
0`00Oomoo`1cOol000Ioo`D00005Ool007ooOol00008Ool01000Oomoo`000Woo00@007ooOol0009o
o`04001oo`000004Ool00`00Oomoo`03Ool3000cOol00`00Oomoo`1cOol000aoo`04001oogoo0008
Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oo`000005Ool00`00Oomoo`02Ool00`00Oomo
o`0dOol00`00Oomoo`1bOol000aoo`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol0009o
o`800005Ool007ooOol00002Ool00`00Oomoo`02Ool00`00Oomoo`0eOol00`00Oomoo`1aOol000eo
o`8000Yoo`8000Aoo`8000Aoo`8000Aoo`8000Eoo`03001oogoo03Ioo`03001oogoo071oo`009Goo
00<007ooOol02Woo0P00>7oo00<007ooOol0Kgoo000UOol00`00Oomoo`0:Ool00`00Oomoo`0hOol0
0`00Oomoo`1^Ool002Eoo`03001oogoo00Yoo`03001oogoo03Uoo`03001oogoo06eoo`0097oo00<0
07ooOol02goo00<007ooOol0>Woo00<007ooOol0K7oo000TOol00`00Oomoo`0;Ool00`00Oomoo`0k
Ool2001/Ool002Aoo`03001oogoo00]oo`8003ioo`03001oogoo06Uoo`0097oo00<007ooOol02goo
00<007ooOol0?Woo00<007ooOol0J7oo000SOol00`00Oomoo`0<Ool00`00Oomoo`0oOol2001XOol0
02=oo`03001oogoo00aoo`03001oogoo045oo`8006Ioo`008goo00<007ooOol037oo0P00A7oo00<0
07ooOol0Hgoo000SOol00`00Oomoo`0<Ool00`00Oomoo`14Ool2001SOol002=oo`03001oogoo00ao
o`03001oogoo04Ioo`80065oo`008Woo00<007ooOol03Goo00<007ooOol0B7oo0P00Ggoo000ROol0
0`00Oomoo`0=Ool2001;Ool2001MOol0029oo`03001oogoo00eoo`03001oogoo04aoo`<005Yoo`00
8Woo00<007ooOol03Goo00<007ooOol0Cgoo1000EWoo000=Ool20004Ool20004Ool20004Ool20000
17oo001oogoo0P000goo0`001Goo00<007ooOol0Dgoo2@00CGoo000<Ool01000Oomoo`000goo0P00
0goo00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`000goo00<007ooOol017oo00<007oo
Ool0G7oo2000AGoo0006Ool500001Goo001oogoo000027oo00@007ooOol0009oo`04001oogoo0002
Ool01000Oomoo`000goo00<007ooOol017oo0`00I7oo3@00>7oo000<Ool01000Oomoo`0027oo00@0
07ooOol0009oo`04001oogoo0002Ool01000Oomoo`000goo00<007ooOol017oo00<007ooOol0LGoo
3@00:goo000<Ool01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`00
0goo00<007ooOol017oo00<007ooOol0OWoo30007goo000=Ool2000:Ool20004Ool30003Ool20003
Ool20006Ool00`00Oomoo`2:Ool9000FOol0021oo`03001oogoo00moo`8009Aoo`P000ioo`0087oo
00<007ooOol03goo00<007ooOol0Vgoo1`001goo000POol00`00Oomoo`0?Ool00`00Oomoo`2YOol0
021oo`03001oogoo00moo`03001oogoo0:Uoo`0087oo00<007ooOol03goo00<007ooOol0ZGoo000O
Ool00`00Oomoo`0@Ool2002ZOol001moo`03001oogoo011oo`03001oogoo0:Uoo`007goo00<007oo
Ool047oo00<007ooOol0ZGoo000?Ool40003Ool20003Ool300000goo001oo`0AOol00`00Oomoo`0A
Ool00`00Oomoo`03Ool20003Ool3000@Ool30004Ool2000<Ool30003Ool40003Ool20003Ool3000<
Ool30003Ool3000<Ool30004Ool00`00Oomoo`03Ool20003Ool3000=Ool40003Ool20004Ool0011o
o`03001oogoo00=oo`8000Ioo`80019oo`80019oo`03001oogoo00=oo`8000Ioo`03001oogoo00io
o`03001oogoo009oo`04001oogoo000<Ool00`00Oomoo`03Ool00`00Oomoo`03Ool20006Ool00`00
Oomoo`0:Ool00`00Oomoo`05Ool00`00Oomoo`0:Ool00`00Oomoo`03Ool00`00Oomoo`03Ool20006
Ool00`00Oomoo`0;Ool00`00Oomoo`02Ool01000Oomoo`000goo000AOol00`00Oomoo`0:Ool00`00
Oomoo`0AOol00`00Oomoo`0BOol00`00Oomoo`0:Ool00`00Oomoo`0>Ool00`00Oomoo`02Ool01000
Oomoo`0037oo00<007ooOol017oo00<007ooOol02Woo00<007ooOol02Woo00<007ooOol01Goo00<0
07ooOol02Woo00<007ooOol017oo00<007ooOol02Woo00<007ooOol037oo00D007ooOomoo`00009o
o`03001oogoo005oo`004Woo00<007ooOol01Woo10004goo00<007ooOol04Woo00<007ooOol01goo
0`004Goo00<007ooOol00Woo00@007ooOol000aoo`03001oogoo00Eoo`03001oogoo00Ioo`<000eo
o`03001oogoo009oo`<000eoo`03001oogoo00Aoo`03001oogoo00Moo`<0011oo`04001oogoo0002
Ool00`00Oomoo`01Ool000moo`04001oogoo0008Ool01000Oomoo`004goo00<007ooOol04goo00<0
07ooOol01Woo00<007ooOol04Goo00<007ooOol00Woo00@007ooOol000aoo`03001oogoo009oo`04
001oogoo0008Ool00`00Oomoo`0=Ool00`00Oomoo`02Ool00`00Oomoo`0=Ool00`00Oomoo`05Ool0
0`00Oomoo`06Ool00`00Oomoo`0=Ool01000Oomoo`000Woo00@007ooOol000=oo`0047oo0P002Goo
10004goo00<007ooOol047oo100027oo10003goo0P001Goo0P0037oo0P001Goo0P002Goo10002goo
0P0017oo10002goo0P0017oo100027oo10003Goo0P0017oo0P0017oo000MOol00`00Oomoo`0BOol2
002ZOol001eoo`03001oogoo019oo`03001oogoo0:Uoo`007Goo00<007ooOol04Woo00<007ooOol0
ZGoo000MOol00`00Oomoo`0BOol00`00Oomoo`2YOol00003Ool000000=T0009oo`001goo00<007oo
Ool00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007oo
Ool00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007oo
Ool00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007oo
Ool00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007oo
Ool00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007oo
Ool00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007oo
Ool00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007oo
Ool00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol01Goo000LOol00`00Oomoo`0COol0
0`00Oomoo`2YOol001aoo`03001oogoo01=oo`03001oogoo0:Uoo`006goo00<007ooOol057oo0P00
ZWoo000KOol00`00Oomoo`0DOol00`00Oomoo`2YOol001Yoo`03001oogoo01Eoo`03001oogoo0:Uo
o`006Woo00<007ooOol05Goo00<007ooOol0ZGoo000JOol00`00Oomoo`0EOol00`00Oomoo`2YOol0
01Uoo`03001oogoo01Ioo`800:Yoo`006Goo00<007ooOol05Woo00<007ooOol0ZGoo000HOol00`00
Oomoo`0GOol00`00Oomoo`2YOol001Qoo`03001oogoo01Moo`03001oogoo0:Uoo`001goo0P003goo
00<007ooOol05goo0P00ZWoo0009Ool00`00Oomoo`0;Ool00`00Oomoo`0HOol00`00Oomoo`2YOol0
00Yoo`03001oogoo00Yoo`03001oogoo01Qoo`03001oogoo0:Uoo`002goo00<007ooOol02Goo00<0
07ooOol067oo00<007ooOol0ZGoo000;Ool00`00Oomoo`08Ool00`00Oomoo`0IOol00`00Oomoo`2Y
Ool000aoo`03001oogoo00Ioo`03001oogoo01Yoo`800:Yoo`003Goo00<007ooOol01Goo00<007oo
Ool06Woo00<007ooOol0ZGoo000=Ool30003Ool20004Ool20004Ool20004Ool20003Ool30005Ool0
0`00Oomoo`2YOol000aoo`03001oogoo00H000=oo`04001oogoo0002Ool01000Oomoo`000Woo00@0
07ooOol000=oo`03001oogoo00Aoo`03001oogoo0:Uoo`0037oo00@007ooOol000Qoo`04001oogoo
0002Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oogoo00Aoo`<00:Uoo`0037oo00@007oo
Ool000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oogoo00Aoo`03
001oogoo0:Uoo`0037oo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007oo
Ool000=oo`03001oogoo00Aoo`03001oogoo0:Uoo`003Goo0P002Woo0P0017oo0P0017oo0P000goo
0P00/Woo003NOol00001\
\>"],
  ImageRangeCache->{{{0, 221.938}, {136.438, 0}} -> {0.348268, -0.000497763, 
  0.091825, 4.53067*^-6}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(samples = 
      TableForm[
        Join[{{"\<S\>", \ "\<Exact\>", "\<Explicit FD\>", \ 
              "\<Error\>"}}, \n\t
          Table[\((PaddedForm[N[#1], {5, 5}] &)\)/@\n\t\t{S, 
                Valuation[10, 0.05, 0, S, 5, 
                  0.2], \[IndentingNewLine]BlackScholesPut[S, 10, 0.2, 0.05, 
                  0, 5], \n\t\t\t\t\t\tValuation[10, 0.05, 0, S, 5, 0.2] - 
                  BlackScholesPut[S, 10, 0.2, 0.05, 0, 
                    5]}, \[IndentingNewLine]\t\t{S, 2, 16, 
              1}\[IndentingNewLine]]]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"Exact\"\>", "\<\"Explicit FD\"\>", 
            "\<\"Error\"\>"},
          {
            TagBox[
              InterpretationBox["\<\" 2.00000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 5.78870\"\>",
                5.788676974698868,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 5.78860\"\>",
                5.7885809005343702,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.00010\"\>",
                9.6074164497750303*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.00000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 4.80050\"\>",
                4.8005065545399717,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 4.80050\"\>",
                4.800508718641141,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox[\("-2.16410"\[Times]10\^"-6"\),
                -2.1641011693773748*^-06,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.00000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.86130\"\>",
                3.8612946253871723,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.86150\"\>",
                3.8615280263060248,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00023\"\>",
                -0.00023340091885248881,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.00000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.02040\"\>",
                3.0204377862722906,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.02090\"\>",
                3.0208607031816879,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00042\"\>",
                -0.0004229169093972196,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.00000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 2.31040\"\>",
                2.3103622592028312,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 2.31080\"\>",
                2.310845529420245,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00048\"\>",
                -0.00048327021741378928,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.00000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.73820\"\>",
                1.7382214088037049,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.73870\"\>",
                1.738661710302452,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00044\"\>",
                -0.00044030149874707547,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.00000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.29290\"\>",
                1.292865894567176,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.29320\"\>",
                1.2932190592768538,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00035\"\>",
                -0.0003531647096777224,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.00000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.95451\"\>",
                0.95451498699447379,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.95478\"\>",
                0.95478044388596572,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00027\"\>",
                -0.00026545689149193308,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.00000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.70167\"\>",
                0.70167268417318651,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.70187\"\>",
                0.70186980510265284,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00020\"\>",
                -0.00019712092946633231,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.00000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.51477\"\>",
                0.51476923439059974,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.51492\"\>",
                0.51492057347081155,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00015\"\>",
                -0.00015133908021180975,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.00000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.37754\"\>",
                0.37753648554593283,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.37766\"\>",
                0.37766075385942499,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00012\"\>",
                -0.00012426831349215961,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.00000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.27715\"\>",
                0.27715254328459443,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.27726\"\>",
                0.27726230863712598,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00011\"\>",
                -0.0001097653525315434,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.00000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.20384\"\>",
                0.20384041393715283,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.20394\"\>",
                0.2039428604577227,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00010\"\>",
                -0.00010244652056987102,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.00000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.15030\"\>",
                0.15030079478366329,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.15040\"\>",
                0.15039922861977628,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00010\"\>",
                -9.8433836112993278*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.00000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.11116\"\>",
                0.11115734419249142,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.11125\"\>",
                0.11125262488724719,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00010\"\>",
                -9.5280694755769946*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["15.5 Fully Implicit Scheme for Put", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
These algorithms are by now self-explanatory - first the \
initialization:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(dx = 0.025; \ dtau = 0.00025; \ alpha = dtau\/dx\^2; \nM = 400; \ 
    nminus = 160; \ nplus = 160; \n\n
    initial = 
      Table[PutExercise[\((k - 1 - nminus)\)\ dx, 0.05, 0, 0.2], \ {k, 
          nminus + nplus + 1}]; \n
    lower = Table[
        f[\(-nminus\)\ dx, \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \n
    upper = Table[
        g[\(+nplus\)\ dx, \ \ \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \nwold\  = \ initial; \nwvold\  = \ wold; \ 
    wnew\  = \ wold; \n\n
    CMat\  = \ FullyImpCMatrix[alpha, nminus, nplus]; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell["Evolving the solution (this may take some time):", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
For[m=2, m<=M+1, m++,
(wvold = wold;
wold = wnew;
rhs = Take[wold, {2, -2}]+
Table[
Which[
k==1, alpha*lower[[m]],
k== nplus + nminus-1,  alpha*upper[[m]],
True, 0],
{k, 1, nplus + nminus-1}];
temp = CompTridiagSolve[CMat, rhs];
wnew = Join[{lower[[m]]}, temp, {upper[[m]]}])
]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["Interpolation and construction of valuation function:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
interpoldata = Table[{(k - nminus - 1)*dx, wnew[[k]]}, 
    {k, 1, nminus + nplus + 1}]; \
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
ufuncb = Interpolation[interpoldata, InterpolationOrder -> 3];\
\>",
   "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Valuation[strike_, r_, q_, S_, T_, sd_] := 
  ValuationMultiplier[strike, r, q, Log[S/strike], (sd^2*T)/2, sd]*
   ufuncb[Log[S/strike]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Error Plot", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      Valuation[10, 0.05, 0, S, 5, 0.2] - 
        BlackScholesPut[S, 10, 0.2, 0.05, 0, 5], {S, 1, 20}, 
      PlotPoints \[Rule] 50, PlotRange \[Rule] All]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0263158 0.0501253 0.380159 516.593 [
[.099 .36766 -9 -9 ]
[.099 .36766 9 0 ]
[.34962 .36766 -9 -9 ]
[.34962 .36766 9 0 ]
[.47494 .36766 -6 -9 ]
[.47494 .36766 6 0 ]
[.60025 .36766 -12 -9 ]
[.60025 .36766 12 0 ]
[.72556 .36766 -6 -9 ]
[.72556 .36766 6 0 ]
[.85088 .36766 -12 -9 ]
[.85088 .36766 12 0 ]
[.97619 .36766 -6 -9 ]
[.97619 .36766 6 0 ]
[.21181 .0702 -42 -4.5 ]
[.21181 .0702 0 4.5 ]
[.21181 .17352 -42 -4.5 ]
[.21181 .17352 0 4.5 ]
[.21181 .27684 -42 -4.5 ]
[.21181 .27684 0 4.5 ]
[.21181 .48348 -36 -4.5 ]
[.21181 .48348 0 4.5 ]
[.21181 .5868 -36 -4.5 ]
[.21181 .5868 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.099 .38016 m
.099 .38641 L
s
[(2.5)] .099 .36766 0 1 Mshowa
.34962 .38016 m
.34962 .38641 L
s
[(7.5)] .34962 .36766 0 1 Mshowa
.47494 .38016 m
.47494 .38641 L
s
[(10)] .47494 .36766 0 1 Mshowa
.60025 .38016 m
.60025 .38641 L
s
[(12.5)] .60025 .36766 0 1 Mshowa
.72556 .38016 m
.72556 .38641 L
s
[(15)] .72556 .36766 0 1 Mshowa
.85088 .38016 m
.85088 .38641 L
s
[(17.5)] .85088 .36766 0 1 Mshowa
.97619 .38016 m
.97619 .38641 L
s
[(20)] .97619 .36766 0 1 Mshowa
.125 Mabswid
.12406 .38016 m
.12406 .38391 L
s
.14912 .38016 m
.14912 .38391 L
s
.17419 .38016 m
.17419 .38391 L
s
.19925 .38016 m
.19925 .38391 L
s
.24937 .38016 m
.24937 .38391 L
s
.27444 .38016 m
.27444 .38391 L
s
.2995 .38016 m
.2995 .38391 L
s
.32456 .38016 m
.32456 .38391 L
s
.37469 .38016 m
.37469 .38391 L
s
.39975 .38016 m
.39975 .38391 L
s
.42481 .38016 m
.42481 .38391 L
s
.44987 .38016 m
.44987 .38391 L
s
.5 .38016 m
.5 .38391 L
s
.52506 .38016 m
.52506 .38391 L
s
.55013 .38016 m
.55013 .38391 L
s
.57519 .38016 m
.57519 .38391 L
s
.62531 .38016 m
.62531 .38391 L
s
.65038 .38016 m
.65038 .38391 L
s
.67544 .38016 m
.67544 .38391 L
s
.7005 .38016 m
.7005 .38391 L
s
.75063 .38016 m
.75063 .38391 L
s
.77569 .38016 m
.77569 .38391 L
s
.80075 .38016 m
.80075 .38391 L
s
.82581 .38016 m
.82581 .38391 L
s
.87594 .38016 m
.87594 .38391 L
s
.901 .38016 m
.901 .38391 L
s
.92607 .38016 m
.92607 .38391 L
s
.95113 .38016 m
.95113 .38391 L
s
.07393 .38016 m
.07393 .38391 L
s
.04887 .38016 m
.04887 .38391 L
s
.02381 .38016 m
.02381 .38391 L
s
.25 Mabswid
0 .38016 m
1 .38016 L
s
.22431 .0702 m
.23056 .0702 L
s
[(-0.0006)] .21181 .0702 1 0 Mshowa
.22431 .17352 m
.23056 .17352 L
s
[(-0.0004)] .21181 .17352 1 0 Mshowa
.22431 .27684 m
.23056 .27684 L
s
[(-0.0002)] .21181 .27684 1 0 Mshowa
.22431 .48348 m
.23056 .48348 L
s
[(0.0002)] .21181 .48348 1 0 Mshowa
.22431 .5868 m
.23056 .5868 L
s
[(0.0004)] .21181 .5868 1 0 Mshowa
.125 Mabswid
.22431 .09603 m
.22806 .09603 L
s
.22431 .12186 m
.22806 .12186 L
s
.22431 .14769 m
.22806 .14769 L
s
.22431 .19935 m
.22806 .19935 L
s
.22431 .22518 m
.22806 .22518 L
s
.22431 .25101 m
.22806 .25101 L
s
.22431 .30267 m
.22806 .30267 L
s
.22431 .3285 m
.22806 .3285 L
s
.22431 .35433 m
.22806 .35433 L
s
.22431 .40599 m
.22806 .40599 L
s
.22431 .43182 m
.22806 .43182 L
s
.22431 .45765 m
.22806 .45765 L
s
.22431 .50931 m
.22806 .50931 L
s
.22431 .53514 m
.22806 .53514 L
s
.22431 .56097 m
.22806 .56097 L
s
.22431 .04437 m
.22806 .04437 L
s
.22431 .01854 m
.22806 .01854 L
s
.22431 .61263 m
.22806 .61263 L
s
.25 Mabswid
.22431 0 m
.22431 .61803 L
s
.5 Mabswid
.02381 .32183 m
.03302 .30239 L
.04273 .28644 L
.05342 .27178 L
.05863 .266 L
.06337 .26188 L
.06559 .26034 L
.068 .25898 L
.0691 .25849 L
.07028 .25801 L
.07137 .25769 L
.07185 .25754 L
.07238 .25738 L
.07296 .2573 L
.07352 .25723 L
.07411 .25715 L
.07474 .25705 L
.07535 .25706 L
.07589 .25708 L
.07641 .2571 L
.07696 .25712 L
.07727 .25713 L
.07761 .25717 L
.07821 .2573 L
.07886 .25745 L
.07956 .25761 L
.0799 .25768 L
.08022 .25778 L
.08083 .25803 L
.08201 .25853 L
.08426 .25977 L
.08674 .26158 L
.09176 .26668 L
.09642 .2732 L
.10078 .2808 L
.11067 .3032 L
.11973 .32932 L
.13914 .39753 L
.15781 .46822 L
.17767 .5349 L
.18685 .55942 L
.1968 .58026 L
.20224 .58892 L
.20725 .5951 L
.21205 .59942 L
.21446 .60095 L
.21574 .60168 L
.21608 .60186 L
.21641 .60203 L
.21678 .60216 L
Mistroke
.21712 .60226 L
.21845 .60262 L
.21968 .60289 L
.22094 .60312 L
.22149 .6032 L
.2221 .60327 L
.22265 .60332 L
.22316 .60326 L
.22429 .60309 L
.22665 .6026 L
.22798 .60224 L
.22829 .60214 L
.22863 .60204 L
.22923 .60177 L
.23036 .60114 L
.23159 .6004 L
.23688 .59635 L
.24148 .59144 L
.24644 .58457 L
.2554 .56901 L
.27557 .51988 L
.29501 .45965 L
.3137 .39509 L
.33359 .32462 L
.35274 .25915 L
.37309 .19559 L
.3927 .14244 L
.41156 .10002 L
.43163 .06468 L
.44162 .05082 L
.45096 .04002 L
.4605 .03116 L
.46954 .02465 L
.47431 .02183 L
.4795 .01945 L
.48418 .01768 L
.48647 .01693 L
.48858 .01637 L
.49079 .01592 L
.49318 .01551 L
.4956 .01517 L
.49688 .01503 L
.49825 .01489 L
.49895 .01483 L
.49961 .01478 L
.5002 .01474 L
.50085 .01472 L
.50146 .01472 L
.5021 .01473 L
.50265 .01474 L
Mistroke
.50326 .01476 L
.50458 .01482 L
.50599 .0149 L
.50854 .0151 L
.51111 .01538 L
.51234 .01554 L
.51349 .01571 L
.51487 .01598 L
.51611 .01627 L
.5189 .01697 L
.52369 .01836 L
.52874 .02012 L
.53858 .02463 L
.54763 .02977 L
.56624 .04283 L
.58605 .05972 L
.60512 .07828 L
.62539 .09952 L
.64491 .1208 L
.66369 .14157 L
.68368 .16355 L
.70292 .18428 L
.72142 .20356 L
.74112 .22319 L
.76008 .24106 L
.78024 .25888 L
.79966 .27482 L
.81833 .28902 L
.83821 .30288 L
.85734 .31506 L
.87768 .32677 L
.89727 .33691 L
.91612 .34568 L
.93617 .354 L
.95548 .36109 L
.97405 .36712 L
.97619 .36777 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{222.938, 137.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003N0000RA000`40O003h00O<Goo00<007ooOol0
ZWoo000aOol00`00Oomoo`2ZOol0035oo`03001oogoo0:Yoo`00<Goo00<007ooOol0>Goo1@00K7oo
000aOol2000fOol40004Ool3001ZOol0035oo`03001oogoo03Aoo`03001oogoo00Uoo`@006Ioo`00
<Goo00<007ooOol0<Woo0P0047oo00<007ooOol0Hgoo000aOol00`00Oomoo`0aOol00`00Oomoo`0A
Ool2001SOol0035oo`03001oogoo031oo`03001oogoo01Aoo`03001oogoo061oo`00<Goo00<007oo
Ool0;goo00<007ooOol05Woo00<007ooOol0Ggoo000aOol2000_Ool00`00Oomoo`0HOol2001OOol0
035oo`03001oogoo02eoo`03001oogoo01]oo`03001oogoo05aoo`00<Goo00<007ooOol0;Goo00<0
07ooOol077oo00<007ooOol0Fgoo000aOol00`00Oomoo`0/Ool00`00Oomoo`0NOol00`00Oomoo`1J
Ool000aoo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`8000Eoo`03001oogoo02]oo`03001o
ogoo021oo`03001oogoo05Uoo`002goo00@007ooOol000=oo`8000=oo`04001oogoo0002Ool01000
Oomoo`000Woo00@007ooOol0009oo`04001oogoo0004Ool00`00Oomoo`0[Ool00`00Oomoo`0QOol0
0`00Oomoo`1HOol000Eoo`D00005Ool007ooOol00008Ool01000Oomoo`000Woo00@007ooOol0009o
o`04001oogoo0002Ool01000Oomoo`0017oo0`00:Woo00<007ooOol08goo00<007ooOol0Egoo000;
Ool01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`000Woo0`001Goo
00<007ooOol0:Woo00<007ooOol097oo00<007ooOol0EWoo000;Ool01000Oomoo`0027oo00@007oo
Ool0009oo`04001oogoo0002Ool01000Oomoo`000Woo00<007ooOol01Goo00<007ooOol0:Goo00<0
07ooOol09Woo00<007ooOol0EGoo000<Ool2000:Ool20004Ool20004Ool20004Ool30004Ool00`00
Oomoo`0YOol00`00Oomoo`0WOol00`00Oomoo`1DOol0035oo`03001oogoo02Qoo`03001oogoo02Uo
o`03001oogoo05=oo`00<Goo0P00:Goo00<007ooOol0:Woo00<007ooOol0DWoo000aOol00`00Oomo
o`0WOol00`00Oomoo`0/Ool00`00Oomoo`1AOol0035oo`03001oogoo02Moo`03001oogoo02aoo`03
001oogoo055oo`00<Goo00<007ooOol09Woo00<007ooOol0;Woo00<007ooOol0D7oo000aOol00`00
Oomoo`0VOol00`00Oomoo`0_Ool00`00Oomoo`1?Ool0035oo`03001oogoo02Eoo`03001oogoo035o
o`03001oogoo04ioo`00<Goo0P009Woo00<007ooOol0<Woo00<007ooOol0CGoo000aOol00`00Oomo
o`0TOol00`00Oomoo`0dOol00`00Oomoo`1<Ool0035oo`03001oogoo02Aoo`03001oogoo03Eoo`03
001oogoo04]oo`00<Goo00<007ooOol08goo00<007ooOol0=goo00<007ooOol0BWoo000aOol00`00
Oomoo`0SOol00`00Oomoo`0hOol00`00Oomoo`19Ool0035oo`03001oogoo029oo`03001oogoo03Yo
o`03001oogoo04Qoo`00<Goo0P008goo00<007ooOol0>goo00<007ooOol0Agoo000aOol00`00Oomo
o`0QOol00`00Oomoo`0mOol00`00Oomoo`16Ool0035oo`03001oogoo025oo`03001oogoo03ioo`03
001oogoo04Eoo`0037oo0P0017oo0P0017oo0P0017oo0P0017oo0P0017oo0`0017oo00<007ooOol0
8Goo00<007ooOol0?goo00<007ooOol0A7oo000;Ool01000Oomoo`000goo0P000goo00@007ooOol0
009oo`04001oogoo0002Ool01000Oomoo`0017oo00<007ooOol00goo00<007ooOol087oo00<007oo
Ool0@7oo00<007ooOol0A7oo0005Ool500001Goo001oogoo000027oo00@007ooOol0009oo`04001o
ogoo0002Ool01000Oomoo`000Woo100017oo0`0087oo00<007ooOol0@Goo00<007ooOol0@goo000;
Ool01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`000Woo00<007oo
00001Goo00<007ooOol087oo00<007ooOol0@Woo00<007ooOol0@Woo000;Ool01000Oomoo`0027oo
00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`000Woo00<007oo00001Goo00<007ooOol0
7goo00<007ooOol0A7oo00<007ooOol0@Goo000<Ool2000:Ool20004Ool20004Ool20004Ool20005
Ool00`00Oomoo`0OOol00`00Oomoo`15Ool00`00Oomoo`10Ool0035oo`03001oogoo01moo`03001o
ogoo04Ioo`03001oogoo03moo`00<Goo00<007ooOol07Woo00<007ooOol0B7oo00<007ooOol0?Woo
000aOol2000OOol00`00Oomoo`19Ool00`00Oomoo`0mOol0035oo`03001oogoo01ioo`03001oogoo
04Yoo`03001oogoo03aoo`00<Goo00<007ooOol07Goo00<007ooOol0C7oo00<007ooOol0>goo000a
Ool00`00Oomoo`0MOol00`00Oomoo`1=Ool00`00Oomoo`0jOol0035oo`03001oogoo01eoo`03001o
ogoo04ioo`03001oogoo03Uoo`00<Goo00<007ooOol077oo00<007ooOol0D7oo0P00>Goo000aOol2
000MOol00`00Oomoo`1BOol00`00Oomoo`0fOol0035oo`03001oogoo01aoo`03001oogoo05=oo`03
001oogoo03Eoo`00<Goo00<007ooOol077oo00<007ooOol0E7oo00<007ooOol0=7oo000aOol00`00
Oomoo`0KOol00`00Oomoo`1FOol00`00Oomoo`0cOol0035oo`03001oogoo01]oo`03001oogoo05Mo
o`03001oogoo039oo`00<Goo0P0077oo00<007ooOol0F7oo00<007ooOol0<Goo000aOol00`00Oomo
o`0JOol00`00Oomoo`1JOol00`00Oomoo`0`Ool000moo`@001ioo`03001oogoo01Yoo`03001oogoo
05]oo`03001oogoo02moo`003Woo00<007ooOol00Woo00<007ooOol06goo00<007ooOol06Woo00<0
07ooOol0G7oo00<007ooOol0;Woo000<Ool20004Ool30003Ool20004Ool20004Ool20003Ool40004
Ool00`00Oomoo`0IOol00`00Oomoo`1NOol00`00Oomoo`0]Ool000]oo`800003Ool007oo009oo`80
0004Ool007oo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`05001oogooOol00006Ool00`00
Oomoo`0IOol00`00Oomoo`1OOol00`00Oomoo`0/Ool000Eoo`D00005Ool007ooOol00006Ool00`00
Ool00002Ool01000Oomoo`000Woo00@007ooOol0009oo`03001oogoo009oo`03001oogoo00=oo`<0
01Uoo`03001oogoo061oo`8002aoo`002Woo0P000Woo00<007ooOol01Goo0P000Woo00@007ooOol0
009oo`04001oogoo0002Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`0HOol00`00Oomo
o`1SOol00`00Oomoo`0YOol000Uoo`03001oo`00009oo`03001oogoo00Eoo`80009oo`04001oogoo
0002Ool01000Oomoo`000Woo00@007ooOol0009oo`03001oogoo009oo`03001oogoo01Qoo`03001o
ogoo06Aoo`03001oogoo02Qoo`002Goo00<007ooOol00P002Goo0`0017oo0P0017oo0P0017oo0P00
1Goo00<007ooOol067oo00<007ooOol0IGoo0P00:7oo0008Ool00`00Oomoo`0<Ool00`00Oomoo`0G
Ool00`00Oomoo`0GOol00`00Oomoo`1XOol2000VOol000Qoo`03001oogoo00aoo`03001oogoo01Mo
o`03001oogoo01Moo`03001oogoo06Yoo`03001oogoo02=oo`001goo00<007ooOol03Woo00<007oo
Ool05Woo0P0067oo00<007ooOol0Jgoo0P008goo0007Ool00`00Oomoo`0>Ool00`00Oomoo`0FOol0
0`00Oomoo`0FOol00`00Oomoo`1^Ool00`00Oomoo`0POol000Ioo`03001oogoo011oo`03001oogoo
01Eoo`03001oogoo01Ioo`03001oogoo06moo`03001oogoo01moo`001Woo00<007ooOol047oo00<0
07ooOol05Goo00<007ooOol05Woo00<007ooOol0L7oo0P007goo0005Ool00`00Oomoo`0AOol00`00
Oomoo`0EOol00`00Oomoo`0EOol00`00Oomoo`1cOol2000MOol001Yoo`03001oogoo01Aoo`03001o
ogoo01Eoo`03001oogoo07Eoo`8001]oo`006Woo00<007ooOol057oo0P005Woo00<007ooOol0Mgoo
0`0067oo000>Ool40003Ool20003Ool3000DOol00`00Oomoo`0BOol01000Oomoo`000Woo0P000goo
0`0047oo0`0017oo0P0037oo0`000goo10000goo0P000goo0`003Goo0`000goo0`003Goo0`0017oo
00<007ooOol00goo0P000goo1@002Woo10000goo0P000goo000?Ool00`00Oomoo`03Ool20003Ool0
1000Oomoo`004goo00<007ooOol04Woo00<007oo00000goo0P001Woo00<007ooOol03Woo00<007oo
Ool00Woo00@007ooOol000aoo`03001oogoo00=oo`03001oogoo00=oo`8000Ioo`03001oogoo00]o
o`03001oogoo00Eoo`03001oogoo00]oo`03001oogoo00=oo`03001oogoo00=oo`8000Ioo`04001o
o`000009Ool00`00Oomoo`02Ool01000Oomoo`000Woo000@Ool00`00Oomoo`08Ool00`00Ool0000C
Ool00`00Oomoo`0COol2000;Ool00`00Oomoo`0>Ool00`00Oomoo`02Ool01000Oomoo`0037oo00<0
07ooOol017oo00<007ooOol02Woo00<007ooOol02goo00<007ooOol01Goo00<007ooOol02goo00<0
07ooOol017oo00<007ooOol02Woo00@007ooOomoo`8000Qoo`05001oogooOol00002Ool10001Ool1
Ool0015oo`03001oogoo00Ioo`<001Aoo`03001oogoo01=oo`8000Qoo`<0015oo`03001oogoo009o
o`04001oogoo000<Ool00`00Oomoo`05Ool00`00Oomoo`06Ool3000>Ool00`00Oomoo`02Ool3000>
Ool00`00Oomoo`04Ool00`00Oomoo`07Ool30006Ool20007Ool01000Oomoo`000Woo0@000Goo0Goo
000>Ool01000Oomoo`0027oo0P005Goo0P005Goo00<007ooOol01Woo00<007ooOol04Goo00<007oo
Ool00Woo00@007ooOol000aoo`03001oogoo009oo`04001oogoo0008Ool00`00Oomoo`0>Ool00`00
Oomoo`02Ool00`00Oomoo`0>Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool20002
Ool01000Oomoo`000Woo00@007ooOol0009oo`003goo0P002Goo10004goo00<007ooOol04Goo1000
27oo10003goo0P001Goo0P0037oo0P001Goo0P002Goo100037oo0P0017oo100037oo0P0017oo1000
27oo10002Goo1@0017oo0P000goo000LOol00`00Oomoo`0BOol00`00Oomoo`0COol00`00Oomoo`29
Ool30008Ool001aoo`03001oogoo019oo`03001oogoo01=oo`03001oogoo08aoo`8000Ioo`0077oo
00<007ooOol04Woo00<007ooOol04Woo00<007ooOol0UGoo000MOol00`00Oomoo`0AOol00`00Oomo
o`0BOol00`00Oomoo`2EOol00=d0005oo`001Goo00<007ooOol00goo00<007ooOol00Woo00<007oo
Ool00goo00<007ooOol00Woo00<007oo00000goo00<007ooOol00Woo00<007ooOol00goo00<007oo
Ool00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00@007ooOomoo`8000Eoo`03
001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo00=oo`03
001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03
001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03
001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo00=oo`03
001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03
001oogoo00Aoo`007Goo00<007ooOol04Goo00<007ooOol04Goo00<007ooOol0UWoo000NOol00`00
Oomoo`0@Ool00`00Oomoo`0AOol00`00Oomoo`2FOol001ioo`03001oogoo011oo`03001oogoo015o
o`03001oogoo09Ioo`007Woo00<007ooOol047oo00<007ooOol04Goo00<007ooOol0UWoo000OOol0
0`00Oomoo`0?Ool2000AOol00`00Oomoo`2GOol001moo`03001oogoo00moo`03001oogoo011oo`03
001oogoo09Moo`007goo00<007ooOol03goo00<007ooOol047oo00<007ooOol0Ugoo000POol00`00
Oomoo`0>Ool00`00Oomoo`0@Ool00`00Oomoo`2GOol0021oo`03001oogoo00ioo`03001oogoo00mo
o`03001oogoo09Qoo`0087oo00<007ooOol03Woo0P0047oo00<007ooOol0V7oo000QOol00`00Oomo
o`0=Ool00`00Oomoo`0?Ool00`00Oomoo`2HOol0025oo`03001oogoo00eoo`03001oogoo00ioo`03
001oogoo09Uoo`008Goo00<007ooOol03Goo00<007ooOol03Woo00<007ooOol0VGoo000ROol00`00
Oomoo`0<Ool00`00Oomoo`0>Ool00`00Oomoo`2IOol0029oo`03001oogoo00aoo`03001oogoo00io
o`03001oogoo09Uoo`008Woo00<007ooOol037oo0P003Woo00<007ooOol0VWoo000SOol00`00Oomo
o`0;Ool00`00Oomoo`0=Ool00`00Oomoo`2JOol002=oo`03001oogoo00]oo`03001oogoo00eoo`03
001oogoo09Yoo`008goo00<007ooOol02goo00<007ooOol03Goo00<007ooOol0VWoo000<Ool20004
Ool20004Ool20004Ool20003Ool30003Ool40004Ool00`00Oomoo`0<Ool00`00Oomoo`2KOol000]o
o`04001oogoo0003Ool20003Ool01000Oomoo`000Woo00@007ooOol0009oo`800003Ool007oo009o
o`03001oogoo00Aoo`03001oogoo00aoo`03001oogoo09]oo`002goo00@007ooOol000Qoo`04001o
ogoo0002Ool01000Oomoo`000Woo0P0000=oo`00Ool00goo00<007ooOol00goo0`0037oo00<007oo
Ool0Vgoo000;Ool01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0002Ool200000goo001o
o`04Ool00`00Oomoo`02Ool00`00Oomoo`0;Ool00`00Oomoo`2LOol000]oo`04001oogoo0008Ool0
1000Oomoo`000Woo00@007ooOol0009oo`800005Ool007ooOol00002Ool00`00Oomoo`02Ool00`00
Oomoo`0;Ool00`00Oomoo`2LOol000aoo`8000Yoo`8000Aoo`8000Aoo`8000Aoo`8000Eoo`03001o
ogoo00]oo`03001oogoo09aoo`009Goo00<007ooOol02Goo00<007ooOol02Woo00<007ooOol0WGoo
000UOol00`00Oomoo`09Ool2000;Ool00`00Oomoo`2MOol002Ioo`03001oogoo00Qoo`03001oogoo
00Yoo`03001oogoo09eoo`009Woo00<007ooOol027oo00<007ooOol02Goo00<007ooOol0WWoo000V
Ool00`00Oomoo`08Ool00`00Oomoo`09Ool00`00Oomoo`2NOol002Ioo`03001oogoo00Qoo`03001o
ogoo00Uoo`03001oogoo09ioo`009goo00<007ooOol01goo00<007ooOol027oo00<007ooOol0Wgoo
000WOol00`00Oomoo`07Ool20009Ool00`00Oomoo`2OOol002Moo`03001oogoo00Moo`03001oogoo
00Moo`03001oogoo0:1oo`00:7oo00<007ooOol01Woo00<007ooOol01goo00<007ooOol0X7oo000X
Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`2QOol002Qoo`03001oogoo00Ioo`03001o
ogoo00Ioo`03001oogoo0:5oo`00:Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol0XGoo
000YOol00`00Oomoo`05Ool20006Ool00`00Oomoo`2ROol002Uoo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo0:9oo`00:Woo00<007ooOol017oo00<007ooOol017oo00<007ooOol0Xgoo000Z
Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`2SOol000aoo`8000Aoo`8000Aoo`8000Ao
o`8000Aoo`8000Aoo`<000Aoo`03001oogoo00=oo`03001oogoo0:Aoo`002goo00@007ooOol000=o
o`8000=oo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol000Aoo`03001oogoo00=oo`03
001oogoo009oo`03001oogoo0:Eoo`002goo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomo
o`000Woo00@007ooOol0009oo`@000Aoo`<0009oo`03001oogoo0:Eoo`002goo00@007ooOol000Qo
o`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oo`00Ool20002Ool01@00
Oomoogoo0000Z7oo000;Ool01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0002Ool01000
Oomoo`000Woo00<007oo00000goo00<007oo00000Woo00<007ooOol0Ygoo000<Ool2000:Ool20004
Ool20004Ool20004Ool20003Ool5002ZOol0035oo`03001oogoo0:Yoo`00<Goo0P00Zgoo000aOol0
0`00Oomoo`2ZOol00001\
\>"],
  ImageRangeCache->{{{0, 221.938}, {136.438, 0}} -> {0.505387, -0.000735902, 
  0.0903698, 8.76863*^-6}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(samples = 
      TableForm[
        Join[{{"\<S\>", \ "\<Exact\>", "\<Implicit FD\>", \ 
              "\<Error\>"}}, \n\t
          Table[\((PaddedForm[N[#1], {5, 5}] &)\)/@\n\t\t{S, \ 
                Valuation[10, 0.05, 0, S, 5, 
                  0.2], \n\t\t\t\t\t\tBlackScholesPut[S, 10, 0.2, 0.05, 0, 
                  5], \n\t\t\t\t\t\tValuation[10, 0.05, 0, S, 5, 0.2] - 
                  BlackScholesPut[S, 10, 0.2, 0.05, 0, 5]}, \t{S, 2, 16, 
              1}]]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"Exact\"\>", "\<\"Implicit FD\"\>", 
            "\<\"Error\"\>"},
          {
            TagBox[
              InterpretationBox["\<\" 2.00000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 5.78830\"\>",
                5.788342827929311,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 5.78860\"\>",
                5.7885809005343702,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00024\"\>",
                -0.00023807260505925854,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.00000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 4.80040\"\>",
                4.8004374258300215,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 4.80050\"\>",
                4.800508718641141,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00007\"\>",
                -7.1292811119505473*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.00000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.86180\"\>",
                3.8618070539618539,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.86150\"\>",
                3.8615280263060248,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.00028\"\>",
                0.0002790276558291005,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.00000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.02130\"\>",
                3.0212922400949052,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.02090\"\>",
                3.0208607031816879,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.00043\"\>",
                0.00043153691321728971,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.00000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 2.31110\"\>",
                2.3111221069989911,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 2.31080\"\>",
                2.310845529420245,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.00028\"\>",
                0.00027657757874610667,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.00000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.73860\"\>",
                1.7386159822265614,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.73870\"\>",
                1.738661710302452,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00005\"\>",
                -4.5728075890538733*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.00000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.29290\"\>",
                1.2928527734966409,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.29320\"\>",
                1.2932190592768538,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00037\"\>",
                -0.00036628578021291602,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.00000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.95419\"\>",
                0.95419075881089754,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.95478\"\>",
                0.95478044388596572,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00059\"\>",
                -0.00058968507506818035,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.00000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.70118\"\>",
                0.70117550054831079,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.70187\"\>",
                0.70186980510265284,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00069\"\>",
                -0.00069430455434205118,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.00000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.51422\"\>",
                0.51422106182800575,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.51492\"\>",
                0.51492057347081155,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00070\"\>",
                -0.00069951164280579459,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.00000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.37702\"\>",
                0.37702194372905939,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.37766\"\>",
                0.37766075385942499,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00064\"\>",
                -0.00063881013036559686,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.00000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.27672\"\>",
                0.27671890726831672,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.27726\"\>",
                0.27726230863712598,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00054\"\>",
                -0.00054340136880925982,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.00000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.20351\"\>",
                0.20350616138802433,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.20394\"\>",
                0.2039428604577227,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00044\"\>",
                -0.0004366990696983708,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.00000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.15007\"\>",
                0.15006557539234966,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.15040\"\>",
                0.15039922861977628,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00033\"\>",
                -0.0003336532274266224,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.00000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.11101\"\>",
                0.1110103170174139,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.11125\"\>",
                0.11125262488724719,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00024\"\>",
                -0.00024230786983328589,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell["\<\
Note that we obtain no improvement in accuracy over the explicit \
scheme. The only advantage of the fully implicit scheme over the explicit \
scheme is the fact that we can, if we wish, increase \[Alpha], that is, the \
time-step for a given price-step, without the system going unstable.\
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["15.6 Crank-Nicholson", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We now increase the time-step by a factor of 20, so that ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 8\)]],
  ". Otherwise all this proceeds as before:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Initialization:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
M=20; nminus = 160; nplus = 160;
dx = 0.025; dtau = 0.005; alpha = dtau/dx^2\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(7.99999999999999911`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(initial = 
      Table[PutExercise[\((k - 1 - nminus)\)\ dx, 0.05, 0, 0.2], \ {k, 
          nminus + nplus + 1}]; \n
    lower = Table[
        f[\(-nminus\)\ dx, \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \n
    upper = Table[
        g[\(+nplus\)\ dx, \ \ \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \nwold\  = \ initial; \nwvold\  = \ wold; \n
    wnew\  = \ wold; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  AspectRatioFixed->True],

Cell[BoxData[
    \(CNCMatrix[alpha_, \ nminus_, \ nplus_]\  := \n
      Sequence[Table[\(-alpha\)/2, \ {nplus + nminus - 2}], \n
        Table[1 + alpha, \ {nplus + nminus - 1}], \n
        Table[\(-alpha\)/2, \ {nplus + nminus - 2}]]; \n\n
    CNDMatrix[alpha_, \ vec_List]\  := \ 
      Module[{temp}, \n
        temp\  = \ \((1\  - \ alpha)\)*
              vec\  + \ \((alpha/2)\)*\((RotateRight[vec]\  + \ 
                  RotateLeft[vec])\); \n
        temp[\([1]\)]\  = \ 
          Simplify[First[temp]\  - \ \nalpha*Last[vec]/2]; \n
        temp[\([\(-1\)]\)]\  = \ 
          Simplify[Last[temp]\  - \ alpha*First[vec]/2]; \ntemp]; \n\n
    CMat\  = \ CNCMatrix[alpha, nminus, nplus]; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Evolution:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
For[m=2, m<=M+1, m++,
(wvold = wold;
wold = wnew;
rhs = CNDMatrix[alpha, Take[wold, {2, -2}]]+
Table[
Which[
k==1, alpha*(lower[[m-1]] + lower[[m]])/2,
k== nplus + nminus-1,  alpha*(upper[[m-1]] + upper[[m]])/2,
True, 0],
{k, 1, nplus + nminus-1}];
temp = CompTridiagSolve[CMat, rhs];
wnew = Join[{lower[[m]]}, temp, {upper[[m]]}])
]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Interpolation", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
interpoldatab = 
Table[{(k - nminus - 1)*dx, wnew[[k]]}, {k, 1, nminus+nplus+1}]; \
\>", 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
ufuncb = Interpolation[interpoldatab, InterpolationOrder -> \
3];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Valuation[strike_, r_, q_, S_, T_, sd_] :=
ValuationMultiplier[strike, r, q, Log[S/strike], (sd^2*T)/2, \
sd]*ufuncb[Log[S/strike]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Error Plot", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      Valuation[10, 0.05, 0, S, 5, 0.2] - 
        BlackScholesPut[S, 10, 0.2, 0.05, 0, 5], {S, 1, 20}, 
      PlotPoints \[Rule] 50, PlotRange \[Rule] All]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0263158 0.0501253 0.443348 253.385 [
[.099 .43085 -9 -9 ]
[.099 .43085 9 0 ]
[.34962 .43085 -9 -9 ]
[.34962 .43085 9 0 ]
[.47494 .43085 -6 -9 ]
[.47494 .43085 6 0 ]
[.60025 .43085 -12 -9 ]
[.60025 .43085 12 0 ]
[.72556 .43085 -6 -9 ]
[.72556 .43085 6 0 ]
[.85088 .43085 -12 -9 ]
[.85088 .43085 12 0 ]
[.97619 .43085 -6 -9 ]
[.97619 .43085 6 0 ]
[.21181 .06327 -42 -4.5 ]
[.21181 .06327 0 4.5 ]
[.21181 .18996 -36 -4.5 ]
[.21181 .18996 0 4.5 ]
[.21181 .31666 -42 -4.5 ]
[.21181 .31666 0 4.5 ]
[.21181 .57004 -36 -4.5 ]
[.21181 .57004 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.099 .44335 m
.099 .4496 L
s
[(2.5)] .099 .43085 0 1 Mshowa
.34962 .44335 m
.34962 .4496 L
s
[(7.5)] .34962 .43085 0 1 Mshowa
.47494 .44335 m
.47494 .4496 L
s
[(10)] .47494 .43085 0 1 Mshowa
.60025 .44335 m
.60025 .4496 L
s
[(12.5)] .60025 .43085 0 1 Mshowa
.72556 .44335 m
.72556 .4496 L
s
[(15)] .72556 .43085 0 1 Mshowa
.85088 .44335 m
.85088 .4496 L
s
[(17.5)] .85088 .43085 0 1 Mshowa
.97619 .44335 m
.97619 .4496 L
s
[(20)] .97619 .43085 0 1 Mshowa
.125 Mabswid
.12406 .44335 m
.12406 .4471 L
s
.14912 .44335 m
.14912 .4471 L
s
.17419 .44335 m
.17419 .4471 L
s
.19925 .44335 m
.19925 .4471 L
s
.24937 .44335 m
.24937 .4471 L
s
.27444 .44335 m
.27444 .4471 L
s
.2995 .44335 m
.2995 .4471 L
s
.32456 .44335 m
.32456 .4471 L
s
.37469 .44335 m
.37469 .4471 L
s
.39975 .44335 m
.39975 .4471 L
s
.42481 .44335 m
.42481 .4471 L
s
.44987 .44335 m
.44987 .4471 L
s
.5 .44335 m
.5 .4471 L
s
.52506 .44335 m
.52506 .4471 L
s
.55013 .44335 m
.55013 .4471 L
s
.57519 .44335 m
.57519 .4471 L
s
.62531 .44335 m
.62531 .4471 L
s
.65038 .44335 m
.65038 .4471 L
s
.67544 .44335 m
.67544 .4471 L
s
.7005 .44335 m
.7005 .4471 L
s
.75063 .44335 m
.75063 .4471 L
s
.77569 .44335 m
.77569 .4471 L
s
.80075 .44335 m
.80075 .4471 L
s
.82581 .44335 m
.82581 .4471 L
s
.87594 .44335 m
.87594 .4471 L
s
.901 .44335 m
.901 .4471 L
s
.92607 .44335 m
.92607 .4471 L
s
.95113 .44335 m
.95113 .4471 L
s
.07393 .44335 m
.07393 .4471 L
s
.04887 .44335 m
.04887 .4471 L
s
.02381 .44335 m
.02381 .4471 L
s
.25 Mabswid
0 .44335 m
1 .44335 L
s
.22431 .06327 m
.23056 .06327 L
s
[(-0.0015)] .21181 .06327 1 0 Mshowa
.22431 .18996 m
.23056 .18996 L
s
[(-0.001)] .21181 .18996 1 0 Mshowa
.22431 .31666 m
.23056 .31666 L
s
[(-0.0005)] .21181 .31666 1 0 Mshowa
.22431 .57004 m
.23056 .57004 L
s
[(0.0005)] .21181 .57004 1 0 Mshowa
.125 Mabswid
.22431 .08861 m
.22806 .08861 L
s
.22431 .11395 m
.22806 .11395 L
s
.22431 .13929 m
.22806 .13929 L
s
.22431 .16463 m
.22806 .16463 L
s
.22431 .2153 m
.22806 .2153 L
s
.22431 .24064 m
.22806 .24064 L
s
.22431 .26598 m
.22806 .26598 L
s
.22431 .29132 m
.22806 .29132 L
s
.22431 .34199 m
.22806 .34199 L
s
.22431 .36733 m
.22806 .36733 L
s
.22431 .39267 m
.22806 .39267 L
s
.22431 .41801 m
.22806 .41801 L
s
.22431 .46869 m
.22806 .46869 L
s
.22431 .49403 m
.22806 .49403 L
s
.22431 .51936 m
.22806 .51936 L
s
.22431 .5447 m
.22806 .5447 L
s
.22431 .03793 m
.22806 .03793 L
s
.22431 .01259 m
.22806 .01259 L
s
.22431 .59538 m
.22806 .59538 L
s
.25 Mabswid
.22431 0 m
.22431 .61803 L
s
.5 Mabswid
.02381 .43524 m
.02609 .43377 L
.02821 .43272 L
.03302 .43096 L
.03771 .4297 L
.04273 .42865 L
.0454 .42821 L
.04829 .4278 L
.04961 .42766 L
.05102 .42752 L
.05222 .42741 L
.0529 .42737 L
.05354 .42734 L
.05415 .42729 L
.05449 .42726 L
.05481 .42726 L
.05539 .42725 L
.05601 .42722 L
.05662 .42719 L
.0572 .4272 L
.05772 .4272 L
.05828 .42718 L
.05889 .42718 L
.05924 .4272 L
.05957 .42721 L
.05991 .42722 L
.06022 .42722 L
.06059 .42722 L
.06093 .42723 L
.06154 .42727 L
.0621 .4273 L
.06272 .42732 L
.06303 .42733 L
.06337 .42737 L
.06455 .42748 L
.06564 .42757 L
.06809 .42787 L
.07251 .42861 L
.07504 .42916 L
.07774 .42985 L
.0826 .4313 L
.10293 .43969 L
.11312 .44437 L
.12251 .44835 L
.13166 .45152 L
.13628 .45277 L
.13888 .45335 L
.14135 .45382 L
.14373 .4542 L
.14633 .45451 L
Mistroke
.14747 .45462 L
.1481 .45468 L
.14869 .45473 L
.14924 .45476 L
.14984 .45479 L
.15088 .45482 L
.15222 .45486 L
.15288 .45488 L
.15347 .45489 L
.1541 .45488 L
.15478 .45485 L
.15621 .4548 L
.15742 .45475 L
.15775 .45474 L
.1581 .45472 L
.15873 .45466 L
.16111 .45441 L
.16351 .45409 L
.16571 .45371 L
.17067 .45261 L
.17503 .45136 L
.17963 .44977 L
.19832 .44086 L
.21821 .42812 L
.23736 .41444 L
.25771 .39989 L
.27732 .387 L
.29618 .37611 L
.31625 .36665 L
.33558 .35951 L
.34529 .3566 L
.35416 .35441 L
.3637 .3526 L
.37394 .35112 L
.37885 .3504 L
.38407 .34951 L
.39344 .34732 L
.39773 .34628 L
.39891 .34605 L
.40014 .34584 L
.4012 .34571 L
.4018 .34565 L
.40236 .34561 L
.40267 .3456 L
.40302 .34558 L
.40364 .34558 L
.40425 .34559 L
.40481 .34562 L
.40542 .34571 L
.40576 .34579 L
Mistroke
.40608 .34588 L
.40743 .34629 L
.40973 .3472 L
.4122 .34845 L
.41345 .34918 L
.41477 .35003 L
.41542 .35047 L
.41602 .35089 L
.41657 .35161 L
.41716 .35241 L
.41978 .35588 L
.42122 .35757 L
.42256 .35892 L
.42374 .35988 L
.42435 .36027 L
.42501 .36061 L
.42538 .36076 L
.42573 .36087 L
.42606 .36094 L
.42641 .36099 L
.42702 .361 L
.42736 .36092 L
.42769 .36072 L
.42887 .35975 L
.42996 .35851 L
.43115 .35675 L
.4324 .35444 L
.43486 .34851 L
.43708 .34147 L
.43768 .3393 L
.43832 .33687 L
.43865 .33555 L
.43901 .33202 L
.43965 .32488 L
.44208 .29846 L
.44432 .27758 L
.44558 .26859 L
.44618 .26521 L
.44673 .26266 L
.44725 .26081 L
.4478 .25947 L
.44841 .25878 L
.44896 .25892 L
.44949 .25978 L
.45005 .26153 L
.45059 .26647 L
.45108 .2821 L
.45554 .43749 L
.458 .51652 L
.45921 .54913 L
Mistroke
.46032 .57407 L
.46095 .58553 L
.46164 .59577 L
.46197 .59976 L
.46233 .60332 L
.46271 .59899 L
.46307 .584 L
.46557 .46236 L
.47045 .19611 L
.4717 .13485 L
.47302 .07689 L
.47366 .05218 L
.47427 .0313 L
.47481 .01472 L
.4754 .02496 L
.47574 .03591 L
.47605 .04679 L
.47677 .07373 L
.47821 .13599 L
.4808 .26487 L
.48324 .39283 L
.48555 .50653 L
.48682 .56171 L
.48717 .57554 L
.48754 .58954 L
.48786 .59083 L
.48821 .58774 L
.48881 .58102 L
.48946 .5719 L
.49064 .55102 L
.49307 .49402 L
.49532 .43038 L
.49801 .35091 L
.49928 .31483 L
.49984 .29971 L
.50045 .28374 L
.50105 .27758 L
.50168 .27601 L
.50202 .2755 L
.50237 .27519 L
.50302 .27521 L
.50333 .27548 L
.50367 .27594 L
.50429 .27726 L
.50489 .27907 L
.50545 .28118 L
.5068 .28785 L
.50803 .29547 L
.51078 .31623 L
.51318 .33617 L
Mistroke
.51376 .34101 L
.51438 .34407 L
.51474 .34516 L
.51507 .34613 L
.5157 .34797 L
.52023 .35853 L
.52274 .36267 L
.52406 .36438 L
.52547 .36586 L
.52669 .36688 L
.52703 .36711 L
.52739 .36734 L
.52773 .36752 L
.52804 .36759 L
.5286 .36768 L
.52921 .36773 L
.52984 .36773 L
.53043 .36768 L
.53105 .36758 L
.53137 .36751 L
.53172 .36743 L
.53291 .36704 L
.53429 .36643 L
.53559 .36573 L
.53814 .36412 L
.54046 .36251 L
.54099 .36215 L
.54154 .36176 L
.54215 .36153 L
.54272 .36138 L
.54486 .36083 L
.54731 .36031 L
.54957 .35993 L
.5518 .35966 L
.55292 .35956 L
.55355 .35951 L
.55415 .35948 L
.5548 .35945 L
.55539 .35943 L
.55602 .35942 L
.55638 .35945 L
.55671 .35949 L
.55909 .35979 L
.56116 .36014 L
.56344 .36059 L
.56804 .36165 L
.58778 .36632 L
.60677 .36976 L
.62697 .3731 L
.64642 .37647 L
Mistroke
.66513 .37981 L
.68504 .38335 L
.70421 .38673 L
.72458 .39025 L
.74421 .39357 L
.76309 .39668 L
.78318 .3999 L
.80253 .40289 L
.82113 .40566 L
.84093 .4085 L
.85999 .41111 L
.88025 .41375 L
.89977 .41616 L
.91855 .41836 L
.93853 .42057 L
.95777 .42257 L
.97619 .42437 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{222.938, 137.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003N0000RA000`40O003h00O<Goo00<007ooOol0
ZWoo000aOol00`00Oomoo`2ZOol0035oo`03001oogoo0:Yoo`00<Goo0P00=Woo00<007ooOol0LWoo
000aOol00`00Oomoo`0eOol00`00Oomoo`1bOol0035oo`03001oogoo03Eoo`03001oogoo079oo`00
<Goo00<007ooOol0=7oo0P00M7oo000aOol00`00Oomoo`0dOol2001dOol0035oo`8003Eoo`8007Ao
o`00<Goo00<007ooOol0=7oo0P00M7oo000aOol00`00Oomoo`0dOol2001dOol0035oo`03001oogoo
03Aoo`8007Aoo`0037oo0P0017oo0P0017oo0P0017oo0P000goo0`000goo0`001Goo00<007ooOol0
=7oo0P00M7oo000;Ool01000Oomoo`000goo0P000goo00@007ooOol0009oo`04001oogoo0003Ool0
0`00Oomoo`05Ool00`00Oomoo`02Ool00`00Oomoo`0dOol2001dOol000Eoo`D00005Ool007ooOol0
0008Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oogoo00Eoo`03001oogoo009oo`<003Ao
o`8007Aoo`002goo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`000goo00<007ooOol0
0Woo0`001Goo00<007ooOol0=7oo0P00M7oo000;Ool01000Oomoo`0027oo00@007ooOol0009oo`04
001oogoo0003Ool00`00Oomoo`02Ool00`00Oomoo`05Ool00`00Oomoo`0dOol2001dOol000aoo`80
00Yoo`8000Aoo`8000=oo`8000Aoo`@000Aoo`03001oogoo03Aoo`8007Aoo`00<Goo00<007ooOol0
=7oo0P00M7oo000aOol00`00Oomoo`0dOol2001dOol0035oo`8003Eoo`8007Aoo`00<Goo00<007oo
Ool0=7oo0P00M7oo000aOol00`00Oomoo`0dOol2001dOol0035oo`03001oogoo03Aoo`8007Aoo`00
<Goo00<007ooOol0=7oo0P00M7oo000aOol2000eOol2001dOol0035oo`03001oogoo03Aoo`8007Ao
o`00<Goo00<007ooOol0=7oo0P00M7oo000aOol00`00Oomoo`0dOol2001dOol0035oo`03001oogoo
03Aoo`8007Aoo`00<Goo00<007ooOol0=7oo0P00M7oo000aOol2000eOol2001dOol0035oo`03001o
ogoo03Aoo`8007Aoo`00<Goo00<007ooOol0=7oo0P00M7oo000aOol00`00Oomoo`0dOol2001dOol0
035oo`03001oogoo03Aoo`8007Aoo`00<Goo0P00=Goo0P00M7oo000aOol00`00Oomoo`0dOol2001d
Ool0035oo`03001oogoo03Aoo`8007Aoo`00<Goo00<007ooOol0=7oo0P00M7oo000BOol20004Ool2
0004Ool20004Ool20003Ool30005Ool00`00Oomoo`0dOol2001dOol0015oo`04001oogoo0003Ool2
0003Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo03Aoo`8007Ao
o`002goo1@0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool01000Oomoo`000goo00<007ooOol0
17oo0`00=7oo0P00M7oo000AOol01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0003Ool0
0`00Oomoo`04Ool00`00Oomoo`0dOol2001dOol0015oo`04001oogoo0008Ool01000Oomoo`000Woo
00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo03Aoo`8007Aoo`004Woo0P002Woo0P0017oo
0P000goo0P001Woo00<007ooOol0=7oo00<007oo0000Lgoo000aOol00`00Oomoo`0dOol00`00Ool0
001cOol0035oo`03001oogoo03Aoo`03001oo`0007=oo`00<Goo0P00=Goo00<007oo0000Lgoo000a
Ool00`00Oomoo`0dOol00`00Ool0001cOol0035oo`03001oogoo03Aoo`03001oo`0007=oo`00<Goo
00<007ooOol0=7oo00<007oo0000Lgoo000aOol00`00Oomoo`0dOol00`00Ool0001cOol0035oo`80
03Eoo`03001oo`0007=oo`00<Goo00<007ooOol0=7oo00<007oo0000Lgoo000aOol00`00Oomoo`0d
Ool00`00Ool0001cOol0035oo`03001oogoo03Aoo`03001oo`0007=oo`00<Goo00<007ooOol0;goo
00<007ooOol00Woo00<007oo0000Lgoo000aOol00`00Oomoo`0^Ool20004Ool00`00Ool0001cOol0
035oo`8002moo`8000=oo`04001oogoo001cOol0035oo`03001oogoo02ioo`8000=oo`04001oogoo
001cOol0035oo`03001oogoo02ioo`8000=oo`04001oogoo0003Ool2001^Ool0035oo`03001oogoo
02ioo`8000=oo`04001oogoo0003Ool2001^Ool0035oo`03001oogoo02ioo`8000=oo`04001oogoo
0003Ool2001^Ool0035oo`8002ioo`03001oo`0000=oo`04001oogoo0003Ool00`00Ool0001]Ool0
035oo`03001oogoo02eoo`03001oo`0000=oo`04001oogoo0003Ool00`00Ool0001]Ool0035oo`03
001oogoo02eoo`03001oo`0000=oo`04001oogoo0003Ool00`00Ool0001]Ool0035oo`03001oogoo
02eoo`03001oo`0000=oo`04001oogoo0003Ool00`00Ool0001]Ool000aoo`8000Aoo`8000Aoo`80
00Aoo`8000Aoo`8000=oo`<000Eoo`03001oogoo02eoo`03001oo`0000=oo`04001oogoo0003Ool0
0`00Ool0001]Ool000]oo`04001oogoo0003Ool20003Ool01000Oomoo`000Woo00@007ooOol0009o
o`04001oogoo0005Ool00`00Oomoo`02Ool00`00Oomoo`0]Ool00`00Ool00003Ool01000Oomoo`00
0goo00<007oo0000KGoo0005Ool500001Goo001oogoo000027oo00@007ooOol0009oo`04001oogoo
0002Ool01000Oomoo`001Goo00<007ooOol00Woo0`00;Goo00<007oo00000goo00@007ooOol000=o
o`03001oo`0006eoo`002goo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@0
07ooOol0009oo`<000Eoo`03001oogoo02eoo`03001oo`0000=oo`04001oogoo0003Ool00`00Ool0
001]Ool000]oo`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0002
Ool00`00Oomoo`05Ool00`00Oomoo`0]Ool00`00Ool00003Ool01000Oomoo`000goo00<007oo0000
KGoo000<Ool2000:Ool20004Ool20004Ool20003Ool40004Ool00`00Oomoo`0]Ool00`00Ool00003
Ool01000Oomoo`000goo00@007ooOol006aoo`00<Goo00<007ooOol0;7oo0P0000=oo`00Ool00Woo
00@007ooOol000=oo`04001oogoo001/Ool0035oo`03001oogoo02aoo`04001oogoo0003Ool01000
Oomoo`000goo00@007ooOol006aoo`00<Goo0P009Goo0P001Woo00@007ooOol000=oo`04001oogoo
0003Ool01000Oomoo`00K7oo000aOol00`00Oomoo`0OOol600000goo00000004Ool01000Oomoo`00
0goo00@007ooOol000=oo`05001oogooOol0001[Ool0035oo`03001oogoo01Uoo`H000Uoo`04001o
ogoo0003Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000=oo`03001oogoo06Uoo`00<Goo00<0
07ooOol05Goo10003goo10000goo00D007ooOomoo`00009oo`05001oogooOol00004Ool00`00Oomo
o`04Ool2001ROol0035oo`03001oogoo01=oo`8001Aoo`8000Eoo`04001oogoo0002Ool01@00Oomo
ogoo000017oo00@007ooOomoo`<0009oo`@005ioo`00<Goo0P004Woo0P007Goo00@007ooOol0009o
o`05001oogooOol00004Ool40009Ool5001IOol0035oo`03001oogoo00ioo`<001moo`04001oogoo
0002Ool01@00Oomoogoo00005Woo2000DGoo000aOol00`00Oomoo`0<Ool2000ROol01000Oomoo`00
0Woo00D007ooOomoo`0001ioo`D004aoo`00<Goo00<007ooOol02Woo0P0097oo00@007ooOol0009o
o`05001oogooOol0000SOol40018Ool0035oo`03001oogoo00Qoo`8002Ioo`04001oogoo0002Ool0
1@00Oomoogoo00009goo2@00?goo000aOol00`00Oomoo`07Ool00`00Oomoo`0VOol01000Oomoo`00
0Woo00D007ooOomoo`00031oo`@003]oo`00<Goo0P001goo00<007ooOol09goo00@007ooOol0009o
o`04001oogoo000eOol4000gOol000ioo`@000=oo`8000=oo`<001Aoo`03001oogoo00Aoo`8000ao
o`03001oogoo00=oo`8000=oo`<0011oo`<000=oo`@000]oo`<000=oo`@000=oo`8000=oo`<000eo
o`<000=oo`<000=oo`T00004Ool000000004Ool00`00Oomoo`03Ool20003Ool3000<Ool40003Ool2
0003Ool000moo`03001oogoo00=oo`8000Ioo`03001oogoo015oo`03001oogoo00=oo`03001oogoo
00aoo`03001oogoo00=oo`8000Ioo`03001oogoo00eoo`<000=oo`04001oogoo000<Ool00`00Oomo
o`03Ool00`00Oomoo`03Ool20006Ool00`00Oomoo`0;Ool00`00Oomoo`05Ool00`00Oomoo`09Ool4
0004Ool00`00Oomoo`03Ool20006Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool01000Oomoo`000Woo
000@Ool00`00Oomoo`0:Ool00`00Oomoo`0AOol00`00Oomoo`02Ool00`00Oomoo`0>Ool00`00Oomo
o`0:Ool00`00Oomoo`0=Ool30003Ool01000Oomoo`0037oo00<007ooOol017oo00<007ooOol02Woo
00<007ooOol02goo00<007ooOol01Goo00<007ooOol02goo00<007oo000020002Goo00<007ooOol0
2goo00D007ooOomoo`00009oo`40005oo`5oo`004Goo00<007ooOol01Woo0`0057oo00D007ooOomo
o`00015oo`03001oogoo00Moo`<0011oo`<000=oo`04001oogoo000<Ool00`00Oomoo`05Ool00`00
Oomoo`06Ool3000>Ool00`00Oomoo`02Ool3000>Ool00`00Oomoo`04Ool01000Oomoogoo2@003goo
00@007ooOol0009oo`40005oo`5oo`003Woo00@007ooOol000Qoo`03001oogoo01Aoo`@001=oo`03
001oogoo00Ioo`03001oogoo011oo`<000=oo`04001oogoo000<Ool00`00Oomoo`02Ool01000Oomo
o`0027oo00<007ooOol03Woo00<007ooOol00Woo00<007ooOol03Woo00<007ooOol01Goo00<007oo
Ool01Woo00<007oo0000200017oo00@007ooOol0009oo`04001oogoo0002Ool000moo`8000Uoo`@0
01=oo`80019oo`@000Qoo`@000moo`<000=oo`@000]oo`8000Eoo`8000Uoo`@000aoo`8000Aoo`@0
00aoo`8000Aoo`@000Qoo`@000Moo`P000=oo`8000=oo`002goo10008Woo00<007ooOol0<7oo00<0
07oo00000goo00@007ooOol006Qoo`8000Ioo`001goo100017oo1@006goo0`00<Woo00<007oo0000
0goo00@007ooOol0071oo`001Goo0P003Goo0P005goo0P000Woo00<007ooOol0<7oo00<007oo0000
0goo00@007ooOol0071oo`005Woo0`004Woo0P0017oo00<007ooOol0<7oo00<007oo00000goo00@0
07ooOol0071oo`00g@000Goo0005Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool0
0`00Oomoo`02Ool20004Ool00`00Oomoo`02Ool20004Ool00`00Oomoo`02Ool00`00Oomoo`03Ool0
0`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool0
0`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool2000017oo001oogoo0P000Woo0P001Goo
00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo
00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo
00<007ooOol00Woo00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo
00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol017oo000MOol:000:Ool0
0`00Oomoo`0`Ool00`00Ool00004Ool00`00Ool0001`Ool0035oo`03001oogoo031oo`03001oo`00
00Aoo`03001oo`00071oo`00<Goo00<007ooOol0<7oo00<007oo000017oo00<007oo0000L7oo000a
Ool00`00Oomoo`0`Ool00`00Ool00004Ool00`00Ool0001`Ool0035oo`80035oo`03001oo`0000Ao
o`03001oo`00071oo`00<Goo00<007ooOol0<7oo00<007oo000017oo00<007oo0000L7oo000aOol0
0`00Oomoo`0aOol20004Ool00`00Ool0001`Ool0035oo`03001oogoo035oo`8000Aoo`03001oo`00
071oo`00<Goo00<007ooOol0<Goo0P0017oo00<007oo0000L7oo000aOol2000bOol20004Ool00`00
Ool0001`Ool0035oo`03001oogoo035oo`8000Aoo`03001oo`00071oo`00<Goo00<007ooOol0<Goo
0P0017oo00<007oo0000L7oo000aOol00`00Oomoo`0aOol20004Ool00`00Ool0001`Ool0035oo`03
001oogoo035oo`8000Aoo`03001oo`00071oo`00<Goo00<007ooOol0<Goo0P0017oo00<007oo0000
L7oo000aOol2000bOol20004Ool00`00Ool0001`Ool0035oo`03001oogoo035oo`8000Aoo`80075o
o`00<Goo00<007ooOol0<Goo0P0017oo0P00LGoo000aOol00`00Oomoo`0aOol20004Ool2001aOol0
035oo`03001oogoo035oo`8000Aoo`80075oo`00<Goo0P00<Woo0P0017oo0P00LGoo000aOol00`00
Oomoo`0aOol20004Ool2001aOol0035oo`03001oogoo035oo`8000Aoo`80075oo`00<Goo00<007oo
Ool0<Goo0P0017oo0P00LGoo000<Ool20004Ool20004Ool20004Ool20004Ool20003Ool30005Ool0
0`00Oomoo`0aOol20004Ool2001aOol000]oo`04001oogoo0003Ool20003Ool01000Oomoo`000Woo
00@007ooOol0009oo`04001oogoo0005Ool00`00Oomoo`02Ool00`00Oomoo`0aOol20004Ool2001a
Ool000]oo`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0005Ool0
0`00Oomoo`02Ool3000aOol20004Ool2001aOol000]oo`04001oogoo0008Ool01000Oomoo`000Woo
00@007ooOol0009oo`04001oogoo0002Ool30005Ool00`00Oomoo`0aOol20004Ool2001aOol000]o
o`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0002Ool00`00Oomo
o`05Ool00`00Oomoo`0aOol20004Ool2001aOol000aoo`8000Yoo`8000Aoo`8000Aoo`8000=oo`@0
00Aoo`03001oogoo035oo`8000Aoo`80075oo`00<Goo00<007ooOol0<Goo0P0017oo00<007ooOol0
L7oo000aOol2000cOol00`00Oomoo`1eOol0035oo`03001oogoo039oo`03001oogoo07Eoo`00<Goo
00<007ooOol0<Woo00<007ooOol0MGoo000aOol00`00Oomoo`2ZOol0035oo`03001oogoo0:Yoo`00
<Goo00<007ooOol0ZWoo0000\
\>"],
  ImageRangeCache->{{{0, 221.938}, {136.438, 0}} -> {0.505387, -0.00174971, 
  0.0903698, 1.78772*^-5}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(samples = 
      TableForm[
        Join[{{"\<S\>", \ "\<Exact\>", "\<Crank-Nich\>", \ "\<Error\>"}}, \n\t
          Table[\((PaddedForm[N[#1], {5, 5}] &)\)/@\n\t\t{S, \ 
                Valuation[10, 0.05, 0, S, 5, 
                  0.2], \n\t\t\t\t\t\tBlackScholesPut[S, 10, 0.2, 0.05, 0, 
                  5], \n\t\t\t\t\t\tValuation[10, 0.05, 0, S, 5, 0.2] - 
                  BlackScholesPut[S, 10, 0.2, 0.05, 0, 5]}, {S, 2, 16, 
              1}]]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"Exact\"\>", "\<\"Crank-Nich\"\>", 
            "\<\"Error\"\>"},
          {
            TagBox[
              InterpretationBox["\<\" 2.00000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 5.78850\"\>",
                5.7885240115866363,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 5.78860\"\>",
                5.7885809005343702,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00006\"\>",
                -5.6888947733924056*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.00000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 4.80050\"\>",
                4.8005307827247785,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 4.80050\"\>",
                4.800508718641141,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.00002\"\>",
                2.2064083637474141*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.00000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.86160\"\>",
                3.8615606878961346,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.86150\"\>",
                3.8615280263060248,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.00003\"\>",
                3.2661590109839267*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.00000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.02080\"\>",
                3.0207837332050143,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.02090\"\>",
                3.0208607031816879,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00008\"\>",
                -7.6969976673613161*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.00000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 2.31060\"\>",
                2.3106300485906175,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 2.31080\"\>",
                2.310845529420245,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00022\"\>",
                -0.00021548082962752346,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.00000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.73830\"\>",
                1.738345966714504,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.73870\"\>",
                1.738661710302452,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00032\"\>",
                -0.00031574358794794755,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.00000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.29290\"\>",
                1.2928546746188088,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.29320\"\>",
                1.2932190592768538,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00036\"\>",
                -0.00036438465804500275,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.00000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.95445\"\>",
                0.95445354876017041,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.95478\"\>",
                0.95478044388596572,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00033\"\>",
                -0.00032689512579531232,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.00000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.70016\"\>",
                0.70016427468984921,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.70187\"\>",
                0.70186980510265284,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00171\"\>",
                -0.0017055304128036353,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.00000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.51461\"\>",
                0.51461322974790802,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.51492\"\>",
                0.51492057347081155,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00031\"\>",
                -0.00030734372290353029,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.00000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.37735\"\>",
                0.37734532467151632,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.37766\"\>",
                0.37766075385942499,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00032\"\>",
                -0.00031542918790866681,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.00000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.27698\"\>",
                0.2769839744309141,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.27726\"\>",
                0.27726230863712598,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00028\"\>",
                -0.0002783342062118721,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.00000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.20370\"\>",
                0.20369945788368626,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.20394\"\>",
                0.2039428604577227,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00024\"\>",
                -0.00024340257403643317,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.00000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.15019\"\>",
                0.15019034588237126,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.15040\"\>",
                0.15039922861977628,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00021\"\>",
                -0.0002088827374050195,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.00000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.11108\"\>",
                0.11107646494757184,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.11125\"\>",
                0.11125262488724719,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.00018\"\>",
                -0.0001761599396753466,
                AutoDelete->True],
              (PaddedForm[ #, {5, 5}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell["\<\
Note that we obtain comparable accuracy to the explicit method with \
1/20 the number of time-steps, but near the strike things are becoming \
\"interesting\". We obtain a sharply oscillatory error in the neighbourhood \
of the strike. Note that the size of the error is quite small, but it is \
steeply sloped. Bearing in mind the warnings given in Chapter 14 this is now \
the time to build the Greeks.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Construction and Verification of Interpolated Valuation and \
Greeks\
\>", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
We begin by writing down a function that does numerical \
differentiation of a list. This function uses a simple central difference \
algorithm for points in the interior of the list. The end points are treated \
using a special difference algorithm. \
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
listd[data_, step_] :=
Module[{dleft,dright,len},
len = Length[data];
dleft = (4*data[[2]]-3*data[[1]]-data[[3]])/(2*step);
dright = (3*data[[len]]-4*data[[len-1]]+data[[len-2]])/(2*step);
Join[{dleft}, Take[RotateLeft[data]-RotateRight[data], {2, -2}]/(2*step), \
{dright}]
]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[TextData[{
  "Next we fix the values of the ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  "-parameters:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(kt\  = \ ktwo[0.05, \ 0, \ 0.2]; \nko\  = \ kone[0.05, \ 0.2]; \)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(deltadata = listd[wnew, dx] - 1\/2\ \((kt - 1)\)\ wnew; \)\)], "Input",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(gammadata = 
      listd[deltadata, dx] - 1\/2\ \((kt + 1)\)\ deltadata; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(thetadata = \(3  wnew - 4  wold + wvold\)\/\(2  dtau\) - \((1\/4\ \
\((kt - 1)\)\^2 + ko)\) wnew; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
points = Table[(k - nminus - 1)*dx, {k, 1, nminus + nplus + 1}]; \
\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(deltainterpoldata = Transpose[{points, deltadata}]; \)\), 
    \(\(gammainterpoldata = Transpose[{points, gammadata}]; \)\), 
    \(\(thetainterpoldata = Transpose[{points, thetadata}]; \)\)}], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[{
    \(dfunc = 
      Interpolation[deltainterpoldata, InterpolationOrder \[Rule] 3]; \n
    gfunc = Interpolation[gammainterpoldata, 
        InterpolationOrder \[Rule] 3]; \), 
    \(\(tfunc = 
      Interpolation[thetainterpoldata, InterpolationOrder \[Rule] 3]; \)\)}], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(CNDelta[strike_, r_, q_, S_, T_, 
        sd_] := \(1\/S\) \((ValuationMultiplier[strike, r, q, 
              Log[S\/strike], \(sd\^2\ T\)\/2, sd]\ dfunc[
              Log[S\/strike]])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(CNGamma[strike_, r_, q_, S_, T_, 
        sd_] := \(1\/S\^2\) \((ValuationMultiplier[strike, r, q, 
              Log[S\/strike], \(sd\^2\ T\)\/2, sd]\ gfunc[
              Log[S\/strike]])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(CNTheta[strike_, r_, q_, S_, T_, 
        sd_] := \(-\(1\/2\)\)\ sd\^2\ ValuationMultiplier[strike, r, q, 
          Log[S\/strike], \(sd\^2\ T\)\/2, sd]\ tfunc[Log[S\/strike]]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Delta Analysis", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
To investigate the errors in Delta we make a comparison with the \
exact solution, which we have already loaded as a function:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?BlackScholesPutDelta\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("BlackScholesPutDelta[price, strike, vol, riskfree, divyield, expiry]  \
returns the delta of a vanilla European Put."\)], "Print"]
}, Open  ]],

Cell["\<\
We make a table of values of the percentage errors in delta, given \
in the last column of the following table:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
deltasamples = TableForm[
Table[Map[PaddedForm[N[#], {5, 4}]&, 
{S, CNDelta[10, 0.05, 0, S, 5, 0.2],
BlackScholesPutDelta[S,10,0.2,0.05,0, 5],
100*(CNDelta[10,0.05,0,S,5,0.2]/BlackScholesPutDelta[S,10,0.2,0.05,0, \
5]-1)}],
{S, 2, 16, 1}]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9979\"\>",
                -0.99789375767850963,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9976\"\>",
                -0.99757014495884477,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0324\"\>",
                0.032440096698982401,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9721\"\>",
                -0.97213118602394577,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9719\"\>",
                -0.97190384659171136,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0234\"\>",
                0.023391144405038311,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.8975\"\>",
                -0.897475007938634,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.8973\"\>",
                -0.89729081174698944,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0205\"\>",
                0.020528037201894556,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.7786\"\>",
                -0.77864974571872902,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.7785\"\>",
                -0.77854854120728401,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0130\"\>",
                0.012999126719592802,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.6404\"\>",
                -0.640430441472018,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.6404\"\>",
                -0.64043322578235118,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0004\"\>",
                -0.00043475419779781532,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.5059\"\>",
                -0.50588004463153458,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.5060\"\>",
                -0.50595424315123527,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0147\"\>",
                -0.014665065211938444,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.3882\"\>",
                -0.38823610067766268,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.3883\"\>",
                -0.38833561265254934,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0256\"\>",
                -0.025625250851180059,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2921\"\>",
                -0.29213049994175488,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2922\"\>",
                -0.2921788833062795,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0166\"\>",
                -0.016559500802082461,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2168\"\>",
                -0.21678717930183716,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2169\"\>",
                -0.2169240328832196,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0631\"\>",
                -0.063088252400378408,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1594\"\>",
                -0.15935901386221152,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1597\"\>",
                -0.15968731335422298,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2056\"\>",
                -0.20558896327802501,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1169\"\>",
                -0.11690375708199056,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1170\"\>",
                -0.11696283853778006,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0505\"\>",
                -0.050513014670383605,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0854\"\>",
                -0.085420957554986104,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0855\"\>",
                -0.085454614659577766,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0394\"\>",
                -0.039385941561775883,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0624\"\>",
                -0.062364897495330494,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0624\"\>",
                -0.062392087714616684,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0436\"\>",
                -0.043579595237397406,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0456\"\>",
                -0.045562433175029983,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0456\"\>",
                -0.045583721879110306,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0467\"\>",
                -0.04670242622307752,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0333\"\>",
                -0.033340616188132353,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0334\"\>",
                -0.033357895170051416,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0518\"\>",
                -0.051798777563694109,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell["\<\
The error in Delta is at most 0.2 % of the exact value, near the \
strike. This is still not too bad, and we can plot the error in Delta.\
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Plot of Computed and Exact Delta", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
If we overlay the computed and exact Delta, the wobbles are \
starting to show:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[{CNDelta[10, \ 0.05, \ 0, \ S, \ 5, \ 0.2], \n
        BlackScholesPutDelta[S, 10, 0.2, 0.05, 0, \ 5]}, \ {S, \ 8, \ 
        12}, \ \[IndentingNewLine]PlotPoints\  -> \ 50, 
      PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.88095 0.238095 0.856826 2.16851 [
[.2619 .19377 -3 -9 ]
[.2619 .19377 3 0 ]
[.5 .19377 -6 -9 ]
[.5 .19377 6 0 ]
[.7381 .19377 -6 -9 ]
[.7381 .19377 6 0 ]
[.97619 .19377 -6 -9 ]
[.97619 .19377 6 0 ]
[.01131 .09785 -30 -4.5 ]
[.01131 .09785 0 4.5 ]
[.01131 .3147 -30 -4.5 ]
[.01131 .3147 0 4.5 ]
[.01131 .42312 -24 -4.5 ]
[.01131 .42312 0 4.5 ]
[.01131 .53155 -30 -4.5 ]
[.01131 .53155 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .20627 m
.2619 .21252 L
s
[(9)] .2619 .19377 0 1 Mshowa
.5 .20627 m
.5 .21252 L
s
[(10)] .5 .19377 0 1 Mshowa
.7381 .20627 m
.7381 .21252 L
s
[(11)] .7381 .19377 0 1 Mshowa
.97619 .20627 m
.97619 .21252 L
s
[(12)] .97619 .19377 0 1 Mshowa
.125 Mabswid
.07143 .20627 m
.07143 .21002 L
s
.11905 .20627 m
.11905 .21002 L
s
.16667 .20627 m
.16667 .21002 L
s
.21429 .20627 m
.21429 .21002 L
s
.30952 .20627 m
.30952 .21002 L
s
.35714 .20627 m
.35714 .21002 L
s
.40476 .20627 m
.40476 .21002 L
s
.45238 .20627 m
.45238 .21002 L
s
.54762 .20627 m
.54762 .21002 L
s
.59524 .20627 m
.59524 .21002 L
s
.64286 .20627 m
.64286 .21002 L
s
.69048 .20627 m
.69048 .21002 L
s
.78571 .20627 m
.78571 .21002 L
s
.83333 .20627 m
.83333 .21002 L
s
.88095 .20627 m
.88095 .21002 L
s
.92857 .20627 m
.92857 .21002 L
s
.25 Mabswid
0 .20627 m
1 .20627 L
s
.02381 .09785 m
.03006 .09785 L
s
[(-0.35)] .01131 .09785 1 0 Mshowa
.02381 .3147 m
.03006 .3147 L
s
[(-0.25)] .01131 .3147 1 0 Mshowa
.02381 .42312 m
.03006 .42312 L
s
[(-0.2)] .01131 .42312 1 0 Mshowa
.02381 .53155 m
.03006 .53155 L
s
[(-0.15)] .01131 .53155 1 0 Mshowa
.125 Mabswid
.02381 .11953 m
.02756 .11953 L
s
.02381 .14122 m
.02756 .14122 L
s
.02381 .1629 m
.02756 .1629 L
s
.02381 .18459 m
.02756 .18459 L
s
.02381 .22796 m
.02756 .22796 L
s
.02381 .24964 m
.02756 .24964 L
s
.02381 .27133 m
.02756 .27133 L
s
.02381 .29301 m
.02756 .29301 L
s
.02381 .33638 m
.02756 .33638 L
s
.02381 .35807 m
.02756 .35807 L
s
.02381 .37975 m
.02756 .37975 L
s
.02381 .40144 m
.02756 .40144 L
s
.02381 .44481 m
.02756 .44481 L
s
.02381 .46649 m
.02756 .46649 L
s
.02381 .48818 m
.02756 .48818 L
s
.02381 .50986 m
.02756 .50986 L
s
.02381 .07616 m
.02756 .07616 L
s
.02381 .05448 m
.02756 .05448 L
s
.02381 .03279 m
.02756 .03279 L
s
.02381 .01111 m
.02756 .01111 L
s
.02381 .55323 m
.02756 .55323 L
s
.02381 .57492 m
.02756 .57492 L
s
.02381 .5966 m
.02756 .5966 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
.5 Mabswid
.02381 .01493 m
.04273 .03323 L
.06337 .05284 L
.08275 .07093 L
.10139 .08804 L
.12123 .10595 L
.14033 .12291 L
.16063 .14061 L
.18019 .15739 L
.199 .17322 L
.21902 .18968 L
.23829 .20506 L
.25683 .21946 L
.27656 .23427 L
.29555 .24775 L
.31574 .26221 L
.33519 .27742 L
.3539 .2935 L
.3738 .30977 L
.39297 .32218 L
.41334 .33203 L
.43296 .34132 L
.45184 .35251 L
.47193 .36656 L
.49127 .38046 L
.50987 .39364 L
.52967 .40701 L
.54872 .41893 L
.56898 .42932 L
.5885 .43713 L
.60727 .44448 L
.62725 .45351 L
.64648 .46471 L
.66497 .47571 L
.68466 .48711 L
.70361 .49618 L
.72376 .50514 L
.74317 .51338 L
.76183 .5213 L
.7817 .52972 L
.80082 .53768 L
.82115 .54598 L
.84073 .55385 L
.85957 .56126 L
.87961 .56897 L
.89891 .57619 L
.91747 .58296 L
.93723 .58999 L
.95624 .59658 L
.97619 .60332 L
Mistroke
Mfstroke
.02381 .01472 m
.04273 .03303 L
.06337 .05265 L
.08275 .07076 L
.10139 .08787 L
.12123 .10577 L
.14033 .12268 L
.16063 .14032 L
.18019 .15699 L
.199 .17273 L
.21902 .18915 L
.23829 .20465 L
.25683 .21927 L
.27656 .23454 L
.29555 .24893 L
.31574 .26392 L
.33519 .27806 L
.3539 .29138 L
.3738 .30526 L
.39297 .31834 L
.41334 .33193 L
.43296 .34474 L
.45184 .35681 L
.47193 .36936 L
.49127 .38117 L
.50987 .39229 L
.52967 .40387 L
.54872 .41476 L
.56898 .42607 L
.5885 .43672 L
.60727 .44673 L
.62725 .45714 L
.64648 .46692 L
.66497 .47612 L
.68466 .4857 L
.70361 .49469 L
.72376 .50403 L
.74317 .51282 L
.76183 .52107 L
.7817 .52964 L
.80082 .5377 L
.82115 .54605 L
.84073 .5539 L
.85957 .56127 L
.87961 .56892 L
.89891 .57611 L
.91747 .58286 L
.93723 .58987 L
.95624 .59645 L
.97619 .60319 L
Mistroke
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{222.938, 137.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003N0000RA000`40O003h00OgWoo003NOol00=io
o`00gWoo003NOol00=ioo`00gWoo003NOol00=ioo`0087oo00<007ooOol0^goo000POol00`00Oomo
o`2kOol0021oo`800;aoo`0087oo0P00_7oo000POol00`00Ool0002kOol0021oo`04001oogoo002j
Ool0021oo`80009oo`03001oogoo0;Moo`0087oo00<007ooOol00Woo00<007ooOol0]Woo000POol0
0`00Oomoo`03Ool00`00Oomoo`2eOol0021oo`03001oogoo00Aoo`03001oogoo0;Aoo`0087oo0P00
1Woo00<007ooOol0/goo000POol00`00Oomoo`06Ool00`00Oomoo`2bOol0021oo`03001oogoo00Mo
o`03001oogoo0;5oo`0087oo00<007ooOol027oo00<007ooOol0/7oo000POol2000:Ool00`00Oomo
o`2_Ool0021oo`03001oogoo00Yoo`03001oogoo0:ioo`0087oo00<007ooOol02goo00<007ooOol0
[Goo0008Ool20004Ool20003Ool30003Ool30004Ool00`00Oomoo`0<Ool2002]Ool000Moo`04001o
ogoo0003Ool20006Ool00`00Oomoo`03Ool01@00Oomoogoo000047oo00<007ooOol0ZWoo00000goo
0000000300001Goo001oogoo00002goo00<007ooOol00goo00@007ooOomoo`80011oo`03001oogoo
0:Uoo`001goo00@007ooOol000Uoo`8000=oo`<000Aoo`03001oogoo011oo`03001oogoo0:Qoo`00
1goo00@007ooOol000]oo`04001oogoo0006Ool00`00Oomoo`0AOol00`00Oomoo`2WOol000Qoo`80
00Uoo`<000=oo`@000=oo`03001oogoo019oo`03001oogoo0:Ioo`0087oo0P0057oo00<007ooOol0
YGoo000POol00`00Oomoo`0DOol00`00Oomoo`2TOol0021oo`03001oogoo01Eoo`03001oogoo0:=o
o`0087oo00<007ooOol05Woo00<007ooOol0XWoo000POol2000HOol2002ROol0021oo`03001oogoo
01Uoo`03001oogoo09moo`0087oo00<007ooOol06Woo00<007ooOol0WWoo000POol00`00Oomoo`0K
Ool2000<Ool3000XOol30004Ool2000UOol30003Ool3000TOol30003Ool40002Ool0021oo`8001io
o`03001oogoo00aoo`03001oogoo02Ioo`03001oogoo009oo`04001oogoo000UOol00`00Oomoo`03
Ool00`00Oomoo`0TOol00`00Oomoo`03Ool00`00Oomoo`02Ool0021oo`03001oogoo01ioo`03001o
ogoo00Uoo`<002Qoo`03001oogoo009oo`04001oogoo000UOol00`00Oomoo`03Ool00`00Oomoo`0T
Ool00`00Oomoo`04Ool00`00Oomoo`01Ool0021oo`03001oogoo01moo`8000Qoo`04001oogoo000X
Ool00`00Oomoo`02Ool01000Oomoo`009Goo00<007ooOol00goo00<007ooOol097oo00<007ooOol0
1Goo0@000Goo0Goo000POol00`00Oomoo`0QOol00`00Oomoo`05Ool01000Oomoo`00:7oo00<007oo
Ool00Woo00@007ooOol002Eoo`03001oogoo00=oo`03001oogoo02Aoo`03001oogoo009oo`04001o
ogoo0002Ool0021oo`8002=oo`03001oogoo00Eoo`8002Qoo`8000Eoo`8002Eoo`8000Aoo`8002Eo
o`8000Eoo`8000=oo`0087oo00<007ooOol08goo00<007ooOol0UGoo000POol00`00Oomoo`0TOol0
0`00Oomoo`2DOol0021oo`03001oogoo02Eoo`03001oogoo09=oo`006goo`P000Goo000POol00`00
Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`03Ool01000
Oomoo`0027oo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01goo00<0
07ooOol01Woo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01goo00<0
07ooOol01Woo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01Woo00<0
07ooOol017oo000POol00`00Oomoo`0YOol00`00Ool0000]Ool00`00Oomoo`0[Ool00`00Oomoo`0Z
Ool00`00Oomoo`04Ool0021oo`03001oogoo02Yoo`8008moo`0087oo00<007ooOol0;7oo00<007oo
Ool0S7oo000POol2000^Ool00`00Oomoo`2;Ool0021oo`03001oogoo02ioo`8008]oo`0087oo00<0
07ooOol0;goo0P00RWoo000POol00`00Oomoo`0`Ool30028Ool0021oo`80039oo`<008Moo`0087oo
00<007ooOol0<goo0P00QWoo000POol00`00Oomoo`0dOol30024Ool0021oo`03001oogoo03Eoo`<0
08=oo`0087oo0P00>7oo0P00PWoo000POol00`00Oomoo`0iOol20020Ool0021oo`03001oogoo03]o
o`03001oogoo07eoo`0087oo00<007ooOol0?7oo00<007ooOol0O7oo000POol2000nOol00`00Oomo
o`1kOol0021oo`03001oogoo03ioo`03001oogoo07Yoo`0027oo0P0017oo0P000goo10000Woo0`00
17oo00<007ooOol0?goo00<007ooOol0NGoo0007Ool01000Oomoo`000goo0P0017oo00<007ooOol0
1Goo00D007ooOomoo`00049oo`8007Uoo`0000=oo`0000000`0000Eoo`00Oomoo`0000Yoo`03001o
ogoo00Aoo`04001oogooOol20013Ool2001gOol000Moo`04001oogoo000;Ool00`00Oomoo`030004
Ool00`00Oomoo`13Ool3001eOol000Moo`04001oogoo0008Ool01000Oomoo`000Woo00<007ooOol0
17oo00<007ooOol0A7oo0`00M7oo0008Ool2000:Ool20003Ool40003Ool00`00Oomoo`16Ool2001c
Ool0021oo`03001oogoo04Qoo`80075oo`0087oo0P00Bgoo0P00Kgoo000POol00`00Oomoo`1<Ool2
001]Ool0021oo`03001oogoo04ioo`03001oogoo06Yoo`0087oo00<007ooOol0Cgoo0P00JWoo000P
Ool2001BOol2001XOol0021oo`03001oogoo059oo`<006Ioo`0087oo00<007ooOol0Dgoo0`00IGoo
000POol00`00Oomoo`1EOol2001TOol0021oo`8005Qoo`80069oo`0087oo00<007ooOol0FGoo0P00
H7oo000POol00`00Oomoo`1JOol2001OOol0021oo`03001oogoo05]oo`8005ioo`0087oo0P00GWoo
00<007ooOol0Fgoo000POol00`00Oomoo`1NOol2001KOol000ioo`8000Aoo`8000=oo`@000=oo`03
001oogoo061oo`8005Uoo`003Goo00@007ooOol000=oo`8000Aoo`03001oogoo00=oo`03001oogoo
069oo`8005Moo`001goo1@0000Eoo`00Oomoo`0000Yoo`03001oogoo009oo`8006Eoo`8005Eoo`00
3Goo00@007ooOol000]oo`05001oogooOol0001XOol2001COol000eoo`04001oogoo0008Ool01000
Oomoo`000goo00<007ooOol0J7oo0P00DGoo000>Ool2000:Ool20004Ool00`00Oomoo`1ZOol3001>
Ool0021oo`8006eoo`<004aoo`0087oo00<007ooOol0KGoo1000BWoo000POol00`00Oomoo`1_Ool4
0018Ool0021oo`03001oogoo075oo`@004Ioo`0087oo00<007ooOol0Lgoo1000A7oo000POol2001f
Ool30013Ool0021oo`03001oogoo07Qoo`80045oo`0087oo00<007ooOol0NWoo0P00?goo000POol0
0`00Oomoo`1kOol3000mOol0021oo`8007eoo`@003]oo`0087oo00<007ooOol0Ogoo0`00>Goo000P
Ool00`00Oomoo`22Ool2000gOol0021oo`03001oogoo08Aoo`8003Eoo`0087oo0P00Qgoo0P00<goo
000POol00`00Oomoo`28Ool3000`Ool000Qoo`8000Aoo`8000=oo`<000=oo`<000Aoo`03001oogoo
08]oo`8002ioo`001goo00@007ooOol000=oo`8000Aoo`03001oogoo00Eoo`05001oogooOol0002?
Ool2000/Ool00003Ool0000000<00005Ool007ooOol00009Ool00`00Oomoo`05Ool01000Oomoogoo
0P00T7oo0`00:Goo0007Ool01000Oomoo`002Goo00<007ooOol00Woo0`0017oo00<007ooOol0TWoo
0`009Woo0007Ool01000Oomoo`002Goo00<007ooOol00Woo00<007ooOol017oo00<007ooOol0UGoo
0P0097oo0008Ool20009Ool20004Ool40003Ool00`00Oomoo`2GOol3000QOol0021oo`8009]oo`80
01moo`0087oo00<007ooOol0W7oo0P007Goo000POol00`00Oomoo`2NOol3000JOol0021oo`03001o
ogoo0:5oo`<001Moo`0087oo00<007ooOol0Y7oo0P005Goo000POol2002WOol3000BOol0021oo`03
001oogoo0:Uoo`<000moo`0087oo00<007ooOol0[7oo0`0037oo000POol00`00Oomoo`2_Ool2000:
Ool0021oo`800;9oo`<000Moo`0087oo00<007ooOol0]7oo00<007ooOol017oo000POol00`00Oomo
o`2kOol0021oo`03001oogoo0;]oo`0087oo00<007ooOol0^goo003NOol00=ioo`00gWoo003NOol0
0=ioo`00gWoo003NOol00=ioo`00\
\>"],
  ImageRangeCache->{{{0, 221.938}, {136.438, 0}} -> {7.29362, -0.415871, 
  0.0217952, 0.00239304}}]
}, Open  ]],

Cell["Plotting the difference makes the problem clear:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      CNDelta[10, \ 0.05, \ 0, \ S, \ 5, \ 0.2] - 
        BlackScholesPutDelta[S, 10, 0.2, 0.05, 0, \ 5], \ {S, \ 8, \ 12}, \ 
      PlotPoints\  -> \ 50, \n\tPlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.88095 0.238095 0.291619 131.036 [
[.2619 .27912 -3 -9 ]
[.2619 .27912 3 0 ]
[.5 .27912 -6 -9 ]
[.5 .27912 6 0 ]
[.7381 .27912 -6 -9 ]
[.7381 .27912 6 0 ]
[.97619 .27912 -6 -9 ]
[.97619 .27912 6 0 ]
[.01131 .02955 -36 -4.5 ]
[.01131 .02955 0 4.5 ]
[.01131 .16058 -36 -4.5 ]
[.01131 .16058 0 4.5 ]
[.01131 .42266 -30 -4.5 ]
[.01131 .42266 0 4.5 ]
[.01131 .55369 -30 -4.5 ]
[.01131 .55369 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .29162 m
.2619 .29787 L
s
[(9)] .2619 .27912 0 1 Mshowa
.5 .29162 m
.5 .29787 L
s
[(10)] .5 .27912 0 1 Mshowa
.7381 .29162 m
.7381 .29787 L
s
[(11)] .7381 .27912 0 1 Mshowa
.97619 .29162 m
.97619 .29787 L
s
[(12)] .97619 .27912 0 1 Mshowa
.125 Mabswid
.07143 .29162 m
.07143 .29537 L
s
.11905 .29162 m
.11905 .29537 L
s
.16667 .29162 m
.16667 .29537 L
s
.21429 .29162 m
.21429 .29537 L
s
.30952 .29162 m
.30952 .29537 L
s
.35714 .29162 m
.35714 .29537 L
s
.40476 .29162 m
.40476 .29537 L
s
.45238 .29162 m
.45238 .29537 L
s
.54762 .29162 m
.54762 .29537 L
s
.59524 .29162 m
.59524 .29537 L
s
.64286 .29162 m
.64286 .29537 L
s
.69048 .29162 m
.69048 .29537 L
s
.78571 .29162 m
.78571 .29537 L
s
.83333 .29162 m
.83333 .29537 L
s
.88095 .29162 m
.88095 .29537 L
s
.92857 .29162 m
.92857 .29537 L
s
.25 Mabswid
0 .29162 m
1 .29162 L
s
.02381 .02955 m
.03006 .02955 L
s
[(-0.002)] .01131 .02955 1 0 Mshowa
.02381 .16058 m
.03006 .16058 L
s
[(-0.001)] .01131 .16058 1 0 Mshowa
.02381 .42266 m
.03006 .42266 L
s
[(0.001)] .01131 .42266 1 0 Mshowa
.02381 .55369 m
.03006 .55369 L
s
[(0.002)] .01131 .55369 1 0 Mshowa
.125 Mabswid
.02381 .05575 m
.02756 .05575 L
s
.02381 .08196 m
.02756 .08196 L
s
.02381 .10817 m
.02756 .10817 L
s
.02381 .13438 m
.02756 .13438 L
s
.02381 .18679 m
.02756 .18679 L
s
.02381 .213 m
.02756 .213 L
s
.02381 .2392 m
.02756 .2392 L
s
.02381 .26541 m
.02756 .26541 L
s
.02381 .31783 m
.02756 .31783 L
s
.02381 .34403 m
.02756 .34403 L
s
.02381 .37024 m
.02756 .37024 L
s
.02381 .39645 m
.02756 .39645 L
s
.02381 .44886 m
.02756 .44886 L
s
.02381 .47507 m
.02756 .47507 L
s
.02381 .50128 m
.02756 .50128 L
s
.02381 .52748 m
.02756 .52748 L
s
.02381 .00334 m
.02756 .00334 L
s
.02381 .5799 m
.02756 .5799 L
s
.02381 .60611 m
.02756 .60611 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
.5 Mabswid
.02381 .30466 m
.04273 .30388 L
.06337 .303 L
.0734 .30255 L
.0783 .30233 L
.08275 .30215 L
.08707 .30201 L
.08935 .30195 L
.09176 .3019 L
.09298 .30188 L
.09433 .30185 L
.09558 .30184 L
.09671 .30183 L
.09731 .30182 L
.09788 .30182 L
.09847 .30182 L
.09881 .30182 L
.09913 .30182 L
.0997 .30182 L
.10022 .30182 L
.10083 .30182 L
.10139 .30182 L
.10196 .30182 L
.10257 .30182 L
.10367 .30183 L
.10498 .30185 L
.10617 .30187 L
.10877 .30193 L
.11123 .302 L
.11556 .30217 L
.11675 .30223 L
.1174 .30227 L
.11801 .30231 L
.11909 .30242 L
.12027 .30255 L
.13922 .30516 L
.14885 .30684 L
.15937 .3089 L
.16423 .30992 L
.16691 .31049 L
.16943 .31123 L
.17433 .31317 L
.17878 .3149 L
.18775 .31815 L
.19285 .31975 L
.19762 .32104 L
.20232 .32207 L
.20473 .3225 L
.20736 .32287 L
.20879 .32303 L
Mistroke
.21012 .32316 L
.21139 .32325 L
.21204 .32329 L
.21274 .32332 L
.21344 .32335 L
.21407 .32336 L
.21468 .32337 L
.21533 .32338 L
.2159 .32338 L
.21644 .32337 L
.21703 .32335 L
.21766 .32333 L
.21826 .3233 L
.21892 .32327 L
.21957 .32322 L
.21991 .3232 L
.22026 .32316 L
.22091 .32301 L
.2215 .32287 L
.22266 .32257 L
.22809 .32084 L
.2328 .3189 L
.23795 .31632 L
.24757 .3102 L
.25784 .30181 L
.26287 .297 L
.26831 .29127 L
.27081 .28846 L
.27218 .28687 L
.27285 .28609 L
.27346 .2853 L
.27402 .28357 L
.27454 .28193 L
.27571 .27831 L
.27819 .2706 L
.2871 .2435 L
.29695 .21706 L
.3016 .20677 L
.30662 .19779 L
.30939 .19393 L
.31193 .19113 L
.31435 .18918 L
.31557 .18848 L
.31627 .18817 L
.31691 .18794 L
.31751 .18777 L
.31815 .18765 L
.31879 .18758 L
.31915 .18757 L
.31949 .18757 L
Mistroke
.32011 .18763 L
.32067 .18772 L
.32098 .18779 L
.32132 .18789 L
.32193 .1881 L
.32308 .18866 L
.3243 .18947 L
.3265 .19153 L
.32715 .19228 L
.3275 .19272 L
.32783 .19315 L
.32819 .19513 L
.32858 .19822 L
.32927 .20372 L
.33188 .22509 L
.33676 .26699 L
.35583 .43658 L
.36554 .51258 L
.37066 .54624 L
.37609 .57561 L
.38092 .59521 L
.38224 .5994 L
.38293 .60139 L
.38331 .60241 L
.38366 .60332 L
.3843 .6006 L
.38464 .59795 L
.38501 .59509 L
.38624 .58509 L
.39562 .49695 L
.41595 .26714 L
.42615 .15334 L
.43063 .10742 L
.43555 .06114 L
.43783 .04145 L
.43909 .03115 L
.44028 .02174 L
.44082 .01763 L
.44139 .01472 L
.44201 .01503 L
.44259 .01542 L
.4437 .01639 L
.44471 .01752 L
.44711 .02121 L
.4497 .02665 L
.4544 .04007 L
.45907 .05753 L
.46419 .08075 L
.47452 .13795 L
.49435 .27027 L
Mistroke
.51344 .39508 L
.52317 .45 L
.53374 .49916 L
.53867 .51742 L
.5439 .53298 L
.54837 .54287 L
.55091 .54701 L
.55329 .54985 L
.55455 .55094 L
.55572 .55169 L
.55639 .55201 L
.557 .55222 L
.55734 .5523 L
.55771 .55237 L
.55805 .55241 L
.55837 .55243 L
.55895 .55241 L
.55949 .55233 L
.56008 .55218 L
.56072 .54913 L
.56139 .54459 L
.56202 .54022 L
.5632 .53183 L
.57227 .46077 L
.59104 .29368 L
.60053 .21031 L
.61101 .12781 L
.61578 .0954 L
.61829 .07998 L
.61965 .07208 L
.62093 .06506 L
.6215 .06199 L
.62205 .05916 L
.62267 .06033 L
.62324 .06164 L
.62578 .06824 L
.63024 .0826 L
.64003 .12456 L
.64907 .17247 L
.66836 .28714 L
.67806 .34283 L
.68229 .36535 L
.68346 .37125 L
.68409 .37444 L
.68469 .3774 L
.68525 .38015 L
.68586 .38136 L
.68641 .38174 L
.68692 .38207 L
.68815 .38277 L
Mistroke
.68929 .3833 L
.69049 .38373 L
.69116 .38392 L
.6918 .38407 L
.69236 .38417 L
.69297 .38425 L
.69353 .3843 L
.69404 .38432 L
.69459 .38432 L
.6952 .38429 L
.69584 .38424 L
.69644 .38416 L
.69704 .38405 L
.69769 .38391 L
.69901 .38352 L
.70023 .38305 L
.70136 .38253 L
.70393 .38105 L
.70667 .37904 L
.71169 .37429 L
.71714 .36781 L
.72701 .35337 L
.73657 .3373 L
.74164 .32841 L
.74707 .31886 L
.74836 .31663 L
.74903 .31545 L
.74975 .31421 L
.75038 .31314 L
.75074 .31287 L
.75107 .31264 L
.75229 .31183 L
.75706 .30878 L
.76639 .30346 L
.78532 .29513 L
.79512 .29202 L
.80011 .29074 L
.80545 .28959 L
.8105 .28869 L
.8151 .28805 L
.81618 .28792 L
.81677 .28785 L
.81732 .28781 L
.81791 .28779 L
.81857 .28777 L
.81916 .28775 L
.81972 .28774 L
.82025 .28773 L
.82082 .28772 L
.82134 .28772 L
Mistroke
.82183 .28771 L
.8224 .28771 L
.82292 .28771 L
.82353 .28771 L
.82409 .28772 L
.82466 .28772 L
.82527 .28773 L
.82637 .28775 L
.82768 .28778 L
.82888 .28782 L
.83147 .28793 L
.83393 .28807 L
.83826 .28838 L
.84298 .28881 L
.85267 .28995 L
.86296 .29143 L
.88145 .29441 L
.89093 .29553 L
.89626 .29597 L
.90115 .29635 L
.92056 .2977 L
.93923 .2987 L
.94883 .29908 L
.9516 .29918 L
.95296 .29922 L
.95421 .29926 L
.9548 .29928 L
.95535 .29929 L
.95592 .2993 L
.95656 .29931 L
.9591 .29933 L
.96129 .29935 L
.96245 .29936 L
.96368 .29936 L
.96486 .29937 L
.96594 .29937 L
.96646 .29937 L
.96702 .29938 L
.96754 .29938 L
.96803 .29938 L
.96851 .29938 L
.96902 .29938 L
.96958 .29938 L
.9701 .29938 L
.97064 .29938 L
.97122 .29938 L
.97177 .29938 L
.97228 .29938 L
.97319 .29937 L
.97418 .29937 L
Mistroke
.97619 .29936 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{222.938, 137.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003N0000RA000`40O003h00OgWoo003NOol00=io
o`00gWoo003NOol00=ioo`00gWoo003NOol00=ioo`00gWoo003NOol002Ioo`800;Ioo`009Woo00<0
07ooOol0]Goo000VOol00`00Oomoo`1;Ool00`00Oomoo`1WOol000Qoo`8000Aoo`8000Aoo`8000Ao
o`8000=oo`@000=oo`03001oogoo04]oo`03001oogoo06Moo`001goo00@007ooOol000=oo`8000=o
o`04001oogoo0002Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol0BWoo00<007oo0000
J7oo00000goo0000000300001Goo001oogoo000027oo00@007ooOol0009oo`04001oogoo0004Ool0
0`00Oomoo`02Ool2001;Ool00`00Ool0001XOol000Moo`04001oogoo0008Ool01000Oomoo`000Woo
00@007ooOol000Eoo`05001oogooOol0001<Ool00`00Ool0001XOol000Moo`04001oogoo0008Ool0
1000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`1:Ool01000Oomoo`00
Igoo0008Ool2000:Ool20004Ool20004Ool20004Ool00`00Oomoo`1:Ool01000Oomoo`00Igoo000V
Ool00`00Oomoo`1:Ool01@00Oomoogoo0000IWoo000VOol2001;Ool01@00Oomoogoo0000IWoo000V
Ool00`00Oomoo`1:Ool01@00Oomoogoo00007Goo0P00Agoo000VOol00`00Oomoo`1:Ool01@00Oomo
ogoo00007Goo0P00Agoo000VOol00`00Oomoo`1:Ool00`00Oomoo`02Ool00`00Oomoo`0JOol20017
Ool002Ioo`03001oogoo04Yoo`03001oogoo009oo`03001oogoo01Yoo`03001oo`0004Ioo`009Woo
0P00Bgoo00<007ooOol00Woo00<007ooOol06Woo00<007oo0000AWoo000VOol00`00Oomoo`19Ool0
0`00Oomoo`03Ool00`00Oomoo`0JOol00`00Ool00016Ool002Ioo`03001oogoo04Uoo`03001oogoo
00=oo`03001oogoo01Uoo`04001oogoo0016Ool002Ioo`03001oogoo04Uoo`03001oogoo00Aoo`03
001oogoo01Qoo`05001oogooOol00015Ool002Ioo`03001oogoo04Uoo`03001oogoo00Aoo`03001o
ogoo01Qoo`05001oogooOol00015Ool002Ioo`8004Yoo`03001oogoo00Aoo`03001oogoo01Qoo`05
001oogooOol00015Ool002Ioo`03001oogoo04Uoo`03001oogoo00Aoo`03001oogoo01Moo`03001o
ogoo009oo`03001oogoo04=oo`009Woo00<007ooOol0BGoo00<007ooOol017oo00<007ooOol05goo
00<007ooOol00goo00<007ooOol0@Woo000VOol00`00Oomoo`19Ool00`00Oomoo`05Ool00`00Oomo
o`0FOol00`00Oomoo`03Ool00`00Oomoo`12Ool002Ioo`03001oogoo04Uoo`03001oogoo00Eoo`03
001oogoo01Ioo`03001oogoo00=oo`03001oogoo049oo`009Woo0P00BGoo00<007ooOol01Woo00<0
07ooOol05Woo00<007ooOol00goo00<007ooOol0@Woo000VOol00`00Oomoo`18Ool00`00Oomoo`06
Ool00`00Oomoo`0FOol00`00Oomoo`03Ool00`00Oomoo`12Ool002Ioo`03001oogoo04Qoo`03001o
ogoo00Ioo`03001oogoo01Ioo`03001oogoo00=oo`03001oogoo049oo`0027oo0P0017oo0P0017oo
0P0017oo0P000goo0`0017oo00<007ooOol0B7oo00<007ooOol01Woo00<007ooOol05Goo00<007oo
Ool01Goo00<007ooOol0@Goo0007Ool01000Oomoo`000goo0P000goo00@007ooOol0009oo`04001o
ogoo0003Ool00`00Oomoo`03Ool00`00Oomoo`18Ool00`00Oomoo`07Ool00`00Oomoo`0DOol00`00
Oomoo`05Ool00`00Oomoo`11Ool00003Ool0000000<00005Ool007ooOol00008Ool01000Oomoo`00
0Woo00@007ooOol000=oo`03001oogoo00=oo`8004Uoo`03001oogoo00Moo`03001oogoo01Aoo`03
001oogoo00Eoo`03001oogoo045oo`001goo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomo
o`000goo00<007ooOol00goo00<007ooOol0B7oo00<007ooOol01goo00<007ooOol057oo00<007oo
Ool01Goo00<007ooOol0@Goo0007Ool01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0003
Ool00`00Oomoo`03Ool00`00Oomoo`18Ool00`00Oomoo`07Ool00`00Oomoo`0DOol00`00Oomoo`05
Ool00`00Oomoo`11Ool000Qoo`8000Yoo`8000Aoo`8000=oo`8000Eoo`03001oogoo04Qoo`03001o
ogoo00Moo`03001oogoo01Aoo`03001oogoo00Eoo`03001oogoo045oo`009Woo0P00B7oo00<007oo
Ool027oo00<007ooOol057oo00<007ooOol01Goo00<007ooOol0@Goo000VOol00`00Oomoo`0cOol3
000AOol00`00Oomoo`09Ool00`00Oomoo`0COol00`00Oomoo`05Ool00`00Oomoo`11Ool002Ioo`03
001oogoo039oo`03001oogoo0080011oo`03001oogoo00Uoo`03001oogoo019oo`03001oogoo00Mo
o`03001oogoo041oo`009Woo00<007ooOol0<Woo00D007ooOomoo`00011oo`03001oogoo00Uoo`03
001oogoo019oo`03001oogoo00Moo`03001oogoo041oo`009Woo00<007ooOol0<Goo00<007ooOol0
0Woo00<007ooOol03Woo00<007ooOol02Goo00<007ooOol04Woo00<007ooOol01goo00<007ooOol0
@7oo000VOol2000bOol00`00Oomoo`02Ool00`00Oomoo`0>Ool00`00Oomoo`09Ool00`00Oomoo`0B
Ool00`00Oomoo`07Ool00`00Oomoo`10Ool002Ioo`03001oogoo031oo`03001oogoo00=oo`03001o
ogoo00ioo`03001oogoo00Uoo`03001oogoo019oo`03001oogoo00Moo`03001oogoo041oo`009Woo
00<007ooOol0<7oo00<007ooOol00goo00<007ooOol03Woo00<007ooOol02Goo00<007ooOol04Woo
00<007ooOol027oo00<007ooOol0?goo000VOol00`00Oomoo`0_Ool00`00Oomoo`04Ool00`00Oomo
o`0>Ool00`00Oomoo`0:Ool00`00Oomoo`0AOol00`00Oomoo`08Ool00`00Oomoo`0oOol002Ioo`03
001oogoo02moo`03001oogoo00Aoo`03001oogoo00ioo`03001oogoo00Yoo`03001oogoo015oo`03
001oogoo00Qoo`03001oogoo03moo`009Woo0P00<7oo00<007ooOol017oo00<007ooOol03Woo00<0
07ooOol02Woo00<007ooOol04Goo00<007ooOol027oo00<007ooOol0?goo000VOol00`00Oomoo`0W
Ool30004Ool00`00Oomoo`05Ool00`00Oomoo`0=Ool00`00Oomoo`08Ool40003Ool2000>Ool00`00
Oomoo`08Ool00`00Oomoo`07Ool30003Ool3000SOol30003Ool40002Ool002Ioo`03001oogoo02Yo
o`05001oogooOol00008Ool00`00Oomoo`0<Ool00`00Oomoo`09Ool00`00Ool00002Ool01000Oomo
o`003Goo00<007ooOol02Goo00<007ooOol01goo00<007ooOol00goo00<007ooOol08goo00<007oo
Ool00goo00<007ooOol00Woo000VOol00`00Oomoo`0XOol30002Ool00`00Oomoo`07Ool00`00Oomo
o`0<Ool00`00Oomoo`09Ool00`00Ool00002Ool01000Oomoo`0037oo00<007ooOol02Woo00<007oo
Ool01goo00<007ooOol00goo00<007ooOol08goo00<007ooOol017oo00<007ooOol00Goo000VOol0
0`00Oomoo`0WOol01000Oomoo`000Woo00<007ooOol01goo00<007ooOol037oo00<007ooOol02Goo
00H007ooOol007oo0002Ool00`00Oomoo`0:Ool00`00Oomoo`0:Ool00`00Oomoo`07Ool00`00Oomo
o`03Ool00`00Oomoo`0SOol00`00Oomoo`05Ool10001Ool1Ool002Ioo`8002Qoo`04001oogoo0002
Ool00`00Oomoo`07Ool00`00Oomoo`0<Ool00`00Oomoo`09Ool01P00Oomoo`00Ool0009oo`03001o
ogoo00Yoo`03001oogoo00Yoo`03001oogoo00Moo`03001oogoo00=oo`03001oogoo02=oo`03001o
ogoo009oo`04001oogoo0002Ool002Ioo`03001oogoo02Qoo`80009oo`03001oogoo00Qoo`03001o
ogoo00aoo`03001oogoo00Qoo`80009oo`03001oogoo008000eoo`03001oogoo00Yoo`03001oogoo
00Ioo`8000Aoo`8002Aoo`8000Eoo`8000=oo`009Woo00<007ooOol0;7oo00<007ooOol027oo00<0
07ooOol037oo00<007ooOol037oo00<007ooOol03goo00<007ooOol02goo00<007ooOol0?Goo000V
Ool00`00Oomoo`0/Ool00`00Oomoo`08Ool00`00Oomoo`0<Ool00`00Oomoo`0<Ool00`00Oomoo`0?
Ool00`00Oomoo`0;Ool00`00Oomoo`0mOol002Ioo`03001oogoo02]oo`03001oogoo00Uoo`03001o
ogoo00aoo`03001oogoo00aoo`03001oogoo00moo`03001oogoo00]oo`03001oogoo01Qoo`H001mo
o`008Goo_0000Goo000VOol00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomo
o`06Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`03Ool01000Oomoo`0027oo0P001goo
00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00Woo
00D007ooOomoo`0000Qoo`03001oogoo00Aoo`<000Qoo`03001oogoo00Eooa<000Ioo`009Woo5@00
5Woo00<007ooOol03Goo00<007ooOol02Woo00<007ooOol03Woo00<007ooOol03Woo00<007ooOol0
2goo00<007ooOol03goo0P00;7oo000VOol00`00Oomoo`0BOol6000@Ool00`00Oomoo`0=Ool00`00
Oomoo`0:Ool00`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0<Ool00`00Oomoo`0=Ool00`00
Oomoo`0/Ool002Ioo`03001oogoo01Qoo`<000]oo`80011oo`03001oogoo00Yoo`03001oogoo00io
o`03001oogoo00ioo`03001oogoo00aoo`03001oogoo00]oo`8002moo`009Woo0P0077oo0P001Woo
0`004Woo00<007ooOol02Woo00<007ooOol03Woo00<007ooOol03Goo00<007ooOol03Goo00<007oo
Ool02goo00<007ooOol0;Woo000VOol00`00Oomoo`0MOol6000EOol00`00Oomoo`0:Ool00`00Oomo
o`0>Ool00`00Oomoo`0=Ool00`00Oomoo`0=Ool00`00Oomoo`0;Ool00`00Oomoo`0^Ool002Ioo`03
001oogoo03Qoo`03001oogoo00Yoo`03001oogoo00ioo`03001oogoo00eoo`03001oogoo00eoo`03
001oogoo00Yoo`03001oogoo02moo`009Woo00<007ooOol0>Goo00<007ooOol02Goo00<007ooOol0
3goo00<007ooOol037oo00<007ooOol03Woo00<007ooOol02Goo00<007ooOol0;goo000VOol00`00
Oomoo`0iOol00`00Oomoo`09Ool00`00Oomoo`0?Ool00`00Oomoo`0<Ool00`00Oomoo`0>Ool00`00
Oomoo`08Ool00`00Oomoo`0`Ool002Ioo`8003Yoo`03001oogoo00Uoo`03001oogoo00moo`03001o
ogoo00aoo`03001oogoo00ioo`03001oogoo00Qoo`03001oogoo031oo`009Woo00<007ooOol0>Goo
00<007ooOol02Goo00<007ooOol03goo00<007ooOol037oo00<007ooOol03Woo00<007ooOol01goo
00<007ooOol0<Goo000VOol00`00Oomoo`0iOol00`00Oomoo`08Ool00`00Oomoo`0@Ool00`00Oomo
o`0<Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`0bOol002Ioo`03001oogoo03Uoo`03
001oogoo00Qoo`03001oogoo011oo`03001oogoo00]oo`03001oogoo011oo`03001oogoo00Aoo`03
001oogoo03=oo`009Woo00<007ooOol0>Goo00<007ooOol027oo00<007ooOol047oo00<007ooOol0
2goo00<007ooOol047oo00<007ooOol00goo00<007ooOol0=7oo000VOol2000jOol00`00Oomoo`08
Ool00`00Oomoo`0@Ool00`00Oomoo`0;Ool00`00Oomoo`0@Ool00`00Oomoo`02Ool00`00Oomoo`0e
Ool002Ioo`03001oogoo03Yoo`03001oogoo00Moo`03001oogoo015oo`03001oogoo00Yoo`03001o
ogoo011oo`05001oogooOol0000hOol002Ioo`03001oogoo03Yoo`03001oogoo00Moo`03001oogoo
015oo`03001oogoo00Yoo`03001oogoo011oo`D003Qoo`009Woo00<007ooOol0>Woo00<007ooOol0
1goo00<007ooOol04Goo00<007ooOol02Woo00<007ooOol0CGoo000VOol00`00Oomoo`0jOol00`00
Oomoo`07Ool00`00Oomoo`0AOol00`00Oomoo`0:Ool00`00Oomoo`1=Ool002Ioo`8003]oo`03001o
ogoo00Moo`03001oogoo015oo`03001oogoo00Yoo`03001oogoo04eoo`009Woo00<007ooOol0>Woo
00<007ooOol01goo00<007ooOol04Goo00<007ooOol02Goo00<007ooOol0CWoo0008Ool20004Ool2
0004Ool20004Ool20003Ool30004Ool00`00Oomoo`0jOol00`00Oomoo`07Ool00`00Oomoo`0AOol0
0`00Oomoo`09Ool00`00Oomoo`1>Ool000Moo`04001oogoo0003Ool20003Ool01000Oomoo`000Woo
00@007ooOol000=oo`03001oogoo00=oo`03001oogoo03Yoo`03001oogoo00Ioo`03001oogoo01=o
o`03001oogoo00Qoo`03001oogoo04ioo`001goo00@007ooOol000Qoo`04001oogoo0002Ool01000
Oomoo`000goo00<007ooOol00goo0P00?7oo00<007ooOol01Goo00<007ooOol04goo00<007ooOol0
27oo00<007ooOol0CWoo0007Ool01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0003Ool0
0`00Oomoo`03Ool00`00Oomoo`0kOol00`00Oomoo`05Ool00`00Oomoo`0COol00`00Oomoo`08Ool0
0`00Oomoo`1>Ool000Moo`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol000=oo`03001o
ogoo00=oo`03001oogoo03]oo`03001oogoo00Eoo`03001oogoo01=oo`03001oogoo00Qoo`03001o
ogoo04ioo`0027oo0P002Woo0P0017oo0P000goo0P001Goo00<007ooOol0>goo00<007ooOol01Goo
00<007ooOol04goo00<007ooOol027oo00<007ooOol0CWoo000VOol00`00Oomoo`0kOol00`00Oomo
o`05Ool00`00Oomoo`0DOol00`00Oomoo`07Ool00`00Oomoo`1>Ool002Ioo`8003aoo`03001oogoo
00Eoo`03001oogoo01Aoo`03001oogoo00Ioo`03001oogoo04moo`009Woo00<007ooOol0>goo00<0
07ooOol01Goo00<007ooOol057oo00<007ooOol01Woo00<007ooOol0Cgoo000VOol00`00Oomoo`0l
Ool00`00Oomoo`04Ool00`00Oomoo`0DOol00`00Oomoo`06Ool00`00Oomoo`1?Ool002Ioo`03001o
ogoo03aoo`03001oogoo00Aoo`03001oogoo01Aoo`03001oogoo00Ioo`03001oogoo04moo`009Woo
00<007ooOol0?7oo00<007ooOol017oo00<007ooOol05Goo00<007ooOol01Goo00<007ooOol0Cgoo
000VOol2000mOol00`00Oomoo`03Ool00`00Oomoo`0FOol00`00Oomoo`05Ool00`00Oomoo`1?Ool0
02Ioo`03001oogoo03aoo`03001oogoo00=oo`03001oogoo01Ioo`03001oogoo00Eoo`03001oogoo
04moo`009Woo00<007ooOol0?7oo00<007ooOol00goo00<007ooOol05Woo00<007ooOol017oo00<0
07ooOol0D7oo000VOol00`00Oomoo`0lOol00`00Oomoo`03Ool00`00Oomoo`0FOol00`00Oomoo`04
Ool00`00Oomoo`1@Ool002Ioo`03001oogoo03eoo`03001oogoo009oo`03001oogoo01Moo`03001o
ogoo00=oo`03001oogoo051oo`009Woo0P00?Woo00<007ooOol00Woo00<007ooOol05goo00<007oo
Ool00goo00<007ooOol0D7oo000VOol00`00Oomoo`0mOol00`00Oomoo`02Ool00`00Oomoo`0GOol0
0`00Oomoo`03Ool00`00Oomoo`1@Ool002Ioo`03001oogoo03eoo`03001oogoo009oo`03001oogoo
01Qoo`03001oogoo009oo`03001oogoo051oo`009Woo00<007ooOol0?Goo00<007ooOol00Woo00<0
07ooOol067oo00<007ooOol00Woo00<007ooOol0D7oo000VOol00`00Oomoo`0mOol01@00Oomoogoo
00006goo00D007ooOomoo`0005=oo`009Woo0P00?Woo00D007ooOomoo`0001aoo`04001oogoo001C
Ool002Ioo`03001oogoo03ioo`04001oogoo000LOol01000Oomoo`00Dgoo000VOol00`00Oomoo`0n
Ool01000Oomoo`0077oo00@007ooOol005=oo`0027oo0P0017oo0P0017oo0P0017oo0P000goo1000
0goo00<007ooOol0?Woo00@007ooOol001eoo`03001oo`0005=oo`001goo00@007ooOol000=oo`80
00=oo`04001oogoo0002Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol0?Woo00@007oo
Ool001eoo`<005=oo`001goo00@007ooOol000Qoo`04001oogoo0002Ool01000Oomoo`0017oo00<0
07ooOol00Woo0P00?goo00@007ooOol001ioo`8005=oo`001goo00@007ooOol000Qoo`04001oogoo
0002Ool01000Oomoo`001Goo00D007ooOomoo`00041oo`04001oogoo001cOol000Moo`04001oogoo
0008Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`0oOol00`00
Ool0001cOol000Qoo`8000Yoo`8000Aoo`8000Aoo`8000Aoo`03001oogoo03moo`8007Aoo`009Woo
00<007ooOol0?goo0P00M7oo000VOol20010Ool2001dOol002Ioo`03001oogoo03moo`8007Aoo`00
9Woo00<007ooOol0?goo0P00M7oo000VOol00`00Oomoo`0oOol2001dOol002Ioo`03001oogoo041o
o`03001oogoo079oo`009Woo0P00]Woo000VOol00`00Oomoo`2eOol002Ioo`03001oogoo0;Eoo`00
gWoo003NOol00=ioo`00gWoo003NOol00=ioo`00gWoo003NOol00=ioo`00gWoo0000\
\>"],
  ImageRangeCache->{{{0, 221.938}, {136.438, 0}} -> {7.13669, -0.00265895, 
  0.0225219, 4.09228*^-5}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gamma Analysis", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Here we tabulate the exact and numerical solution, with the \
percentage error in Gamma in the last column:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
gammasamples = TableForm[Table[Map[PaddedForm[N[#], {5, 4}]&, 
{S, CNGamma[10, 0.05, 0, S, 5, 0.2],
BlackScholesPutGamma[S,10,0.2,0.05,0, 5],
100*(CNGamma[10,0.05,0,S,5,0.2]/BlackScholesPutGamma[S,10,0.2,0.05,0,5]-1)}],{\
S, 2, 16, 1}]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0084\"\>",
                0.0083525052717234698,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0085\"\>",
                0.0084564450905314295,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-1.2291\"\>",
                -1.2291195377634567,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0481\"\>",
                0.048125072767634405,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0480\"\>",
                0.048026336475153919,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2056\"\>",
                0.20558780812183208,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1002\"\>",
                0.10024816786612789,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1000\"\>",
                0.10003582365214997,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2123\"\>",
                0.21226817176644985,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1332\"\>",
                0.1331576053781017,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1329\"\>",
                0.13291672718548236,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1812\"\>",
                0.18122488998935093,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1395\"\>",
                0.13953272614859044,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1394\"\>",
                0.13936745826081531,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1186\"\>",
                0.11858427342905653,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1275\"\>",
                0.12748646023164076,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1274\"\>",
                0.12742324276041067,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0496\"\>",
                0.049612197791071289,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1070\"\>",
                0.10704760284488107,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1071\"\>",
                0.10711068765967688,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0589\"\>",
                -0.058896844165767792,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0830\"\>",
                0.082967596345932246,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0853\"\>",
                0.085343981515403081,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-2.7845\"\>",
                -2.7844789137731363,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0738\"\>",
                0.073824759926070002,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0657\"\>",
                0.065673835817759235,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 12.4110\"\>",
                12.411219790677475,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0490\"\>",
                0.048962209071906626,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0494\"\>",
                0.04939689354947454,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.8800\"\>",
                -0.87998342878089497,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0366\"\>",
                0.036604966520081286,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0366\"\>",
                0.036606047267158145,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0030\"\>",
                -0.0029523730573077778,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0269\"\>",
                0.026853448142883249,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0269\"\>",
                0.026872712419241808,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0717\"\>",
                -0.071687130268116572,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0196\"\>",
                0.019602852527910747,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0196\"\>",
                0.019616441041218111,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0693\"\>",
                -0.069271042993024334,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0143\"\>",
                0.014268747620205776,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0143\"\>",
                0.014277240229150501,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0595\"\>",
                -0.059483547299188366,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0104\"\>",
                0.010375336516466421,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0104\"\>",
                0.010380531698771796,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0500\"\>",
                -0.050047362275185137,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell["\<\
The error in \[CapitalGamma] is 12.4% of the exact value at the \
strike.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell["Gamma Plot", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True],

Cell["\<\
The wobbles in \[CapitalGamma] are now manifest in a plot of the \
computed and exact version - the error is of a similar scale to the value of \
\[CapitalGamma]:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[{CNGamma[10, \ 0.05, \ 0, \ S, \ 5, \ 0.2], 
        BlackScholesPutGamma[S, 10, 0.2, 0.05, 0, \ 5]}, \ {S, \ 8, \ 12}, \ 
      PlotPoints\  -> \ 50, PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.88095 0.238095 -0.290875 8.34831 [
[.2619 .53146 -3 -9 ]
[.2619 .53146 3 0 ]
[.5 .53146 -6 -9 ]
[.5 .53146 6 0 ]
[.7381 .53146 -6 -9 ]
[.7381 .53146 6 0 ]
[.97619 .53146 -6 -9 ]
[.97619 .53146 6 0 ]
[.01131 .04306 -24 -4.5 ]
[.01131 .04306 0 4.5 ]
[.01131 .12654 -24 -4.5 ]
[.01131 .12654 0 4.5 ]
[.01131 .21002 -24 -4.5 ]
[.01131 .21002 0 4.5 ]
[.01131 .29351 -24 -4.5 ]
[.01131 .29351 0 4.5 ]
[.01131 .37699 -24 -4.5 ]
[.01131 .37699 0 4.5 ]
[.01131 .46047 -24 -4.5 ]
[.01131 .46047 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .54396 m
.2619 .55021 L
s
[(9)] .2619 .53146 0 1 Mshowa
.5 .54396 m
.5 .55021 L
s
[(10)] .5 .53146 0 1 Mshowa
.7381 .54396 m
.7381 .55021 L
s
[(11)] .7381 .53146 0 1 Mshowa
.97619 .54396 m
.97619 .55021 L
s
[(12)] .97619 .53146 0 1 Mshowa
.125 Mabswid
.07143 .54396 m
.07143 .54771 L
s
.11905 .54396 m
.11905 .54771 L
s
.16667 .54396 m
.16667 .54771 L
s
.21429 .54396 m
.21429 .54771 L
s
.30952 .54396 m
.30952 .54771 L
s
.35714 .54396 m
.35714 .54771 L
s
.40476 .54396 m
.40476 .54771 L
s
.45238 .54396 m
.45238 .54771 L
s
.54762 .54396 m
.54762 .54771 L
s
.59524 .54396 m
.59524 .54771 L
s
.64286 .54396 m
.64286 .54771 L
s
.69048 .54396 m
.69048 .54771 L
s
.78571 .54396 m
.78571 .54771 L
s
.83333 .54396 m
.83333 .54771 L
s
.88095 .54396 m
.88095 .54771 L
s
.92857 .54396 m
.92857 .54771 L
s
.25 Mabswid
0 .54396 m
1 .54396 L
s
.02381 .04306 m
.03006 .04306 L
s
[(0.04)] .01131 .04306 1 0 Mshowa
.02381 .12654 m
.03006 .12654 L
s
[(0.05)] .01131 .12654 1 0 Mshowa
.02381 .21002 m
.03006 .21002 L
s
[(0.06)] .01131 .21002 1 0 Mshowa
.02381 .29351 m
.03006 .29351 L
s
[(0.07)] .01131 .29351 1 0 Mshowa
.02381 .37699 m
.03006 .37699 L
s
[(0.08)] .01131 .37699 1 0 Mshowa
.02381 .46047 m
.03006 .46047 L
s
[(0.09)] .01131 .46047 1 0 Mshowa
.125 Mabswid
.02381 .05975 m
.02756 .05975 L
s
.02381 .07645 m
.02756 .07645 L
s
.02381 .09315 m
.02756 .09315 L
s
.02381 .10984 m
.02756 .10984 L
s
.02381 .14324 m
.02756 .14324 L
s
.02381 .15993 m
.02756 .15993 L
s
.02381 .17663 m
.02756 .17663 L
s
.02381 .19333 m
.02756 .19333 L
s
.02381 .22672 m
.02756 .22672 L
s
.02381 .24342 m
.02756 .24342 L
s
.02381 .26011 m
.02756 .26011 L
s
.02381 .27681 m
.02756 .27681 L
s
.02381 .3102 m
.02756 .3102 L
s
.02381 .3269 m
.02756 .3269 L
s
.02381 .3436 m
.02756 .3436 L
s
.02381 .36029 m
.02756 .36029 L
s
.02381 .39369 m
.02756 .39369 L
s
.02381 .41038 m
.02756 .41038 L
s
.02381 .42708 m
.02756 .42708 L
s
.02381 .44378 m
.02756 .44378 L
s
.02381 .47717 m
.02756 .47717 L
s
.02381 .49387 m
.02756 .49387 L
s
.02381 .51056 m
.02756 .51056 L
s
.02381 .52726 m
.02756 .52726 L
s
.02381 .02636 m
.02756 .02636 L
s
.02381 .00966 m
.02756 .00966 L
s
.02381 .56065 m
.02756 .56065 L
s
.02381 .57735 m
.02756 .57735 L
s
.02381 .59405 m
.02756 .59405 L
s
.02381 .61074 m
.02756 .61074 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
.5 Mabswid
.02381 .60279 m
.04273 .58813 L
.06337 .57229 L
.08275 .55776 L
.10139 .544 L
.12123 .52956 L
.14033 .51602 L
.16063 .50097 L
.18019 .48503 L
.199 .4684 L
.21902 .44966 L
.23829 .42585 L
.25683 .40609 L
.26633 .39848 L
.26907 .39669 L
.27042 .39589 L
.27167 .39518 L
.2728 .39458 L
.27344 .39427 L
.27401 .39458 L
.27656 .39602 L
.29618 .4096 L
.30518 .41533 L
.31029 .41788 L
.31279 .41888 L
.31507 .41962 L
.31622 .41993 L
.31749 .42022 L
.31858 .42042 L
.31978 .42059 L
.32043 .42066 L
.32105 .42071 L
.32139 .42073 L
.3217 .42074 L
.32204 .42076 L
.3224 .42076 L
.323 .42076 L
.32356 .42075 L
.32418 .42072 L
.32452 .42069 L
.32483 .42067 L
.32547 .4206 L
.32616 .42051 L
.32674 .42041 L
.32738 .42028 L
.32773 .42021 L
.32806 .42 L
.32869 .41901 L
.33014 .41667 L
.33515 .40794 L
Mistroke
.35462 .36761 L
.37336 .32595 L
.37875 .31456 L
.38382 .30438 L
.38837 .29771 L
.39329 .29112 L
.40244 .28051 L
.40764 .27549 L
.41248 .27148 L
.4176 .26796 L
.42226 .26542 L
.42725 .26342 L
.42999 .26263 L
.43133 .26233 L
.43259 .2621 L
.43377 .26193 L
.43484 .26181 L
.43543 .26176 L
.43604 .26172 L
.43666 .26169 L
.43701 .26168 L
.43733 .26167 L
.43765 .26167 L
.43799 .26167 L
.43861 .26168 L
.43921 .2617 L
.43976 .26173 L
.44036 .26177 L
.44071 .2618 L
.44103 .26183 L
.44136 .26202 L
.44172 .26245 L
.44236 .26324 L
.45139 .27521 L
.47089 .30242 L
.4798 .31293 L
.48488 .31779 L
.48965 .32141 L
.49191 .32277 L
.4943 .32393 L
.49555 .32441 L
.49692 .32485 L
.49816 .32515 L
.49876 .32527 L
.49932 .32535 L
.49996 .32543 L
.5003 .32493 L
.50066 .32431 L
.50192 .32209 L
.50429 .3177 L
Mistroke
.50961 .30694 L
.52815 .2627 L
.54789 .21278 L
.55698 .19181 L
.55947 .18648 L
.55981 .18576 L
.56018 .18499 L
.56053 .18454 L
.56086 .18422 L
.56212 .18301 L
.56689 .17889 L
.57193 .17526 L
.57725 .17218 L
.58194 .17004 L
.58709 .16825 L
.58966 .16757 L
.59209 .16703 L
.59431 .16664 L
.59665 .16631 L
.59787 .16618 L
.5992 .16606 L
.60042 .16597 L
.60156 .16591 L
.60222 .16588 L
.6029 .16586 L
.60349 .16584 L
.60413 .16583 L
.6045 .16583 L
.60483 .16583 L
.60521 .16583 L
.60557 .16583 L
.60621 .16584 L
.60657 .16584 L
.6069 .16585 L
.60751 .16587 L
.6081 .16589 L
.60919 .16593 L
.61037 .166 L
.61165 .16609 L
.6161 .16652 L
.61864 .16684 L
.62003 .16704 L
.62135 .16724 L
.62257 .16749 L
.6239 .16785 L
.62621 .1685 L
.63581 .17138 L
.64633 .17445 L
.65117 .17569 L
.65636 .17683 L
Mistroke
.65887 .17729 L
.66125 .17766 L
.66355 .17796 L
.6657 .17818 L
.66688 .17827 L
.66796 .17834 L
.66916 .1784 L
.66982 .17842 L
.67045 .17844 L
.67105 .17845 L
.6717 .17845 L
.6723 .17845 L
.67286 .17844 L
.67319 .17843 L
.6735 .17843 L
.67421 .1784 L
.67483 .17838 L
.67548 .17834 L
.67656 .17826 L
.67771 .17816 L
.67979 .17792 L
.68226 .17752 L
.68342 .1773 L
.68451 .17706 L
.68505 .17694 L
.68563 .17667 L
.68627 .1761 L
.68687 .17557 L
.68937 .17327 L
.69386 .16887 L
.70402 .15795 L
.72342 .13511 L
.74208 .11341 L
.74702 .10808 L
.74945 .10555 L
.75168 .10384 L
.75426 .10243 L
.75706 .10097 L
.76194 .0986 L
.78124 .09105 L
.79979 .08555 L
.81954 .08001 L
.83855 .07254 L
.85876 .06436 L
.87823 .05629 L
.89696 .0483 L
.91688 .03966 L
.93607 .03138 L
.95646 .02274 L
.9761 .01475 L
Mistroke
.97619 .01472 L
Mfstroke
.02381 .60332 m
.04273 .58877 L
.06337 .57286 L
.08275 .5579 L
.10139 .54351 L
.12123 .5282 L
.14033 .5135 L
.16063 .49793 L
.18019 .483 L
.199 .46871 L
.21902 .45359 L
.23829 .43915 L
.25683 .42536 L
.27656 .41081 L
.29555 .39693 L
.31574 .38233 L
.33519 .36841 L
.3539 .35518 L
.3738 .34126 L
.39297 .32803 L
.41334 .31416 L
.43296 .30098 L
.45184 .28848 L
.47193 .27537 L
.49127 .26294 L
.50987 .25117 L
.52967 .23883 L
.54872 .22715 L
.56898 .21495 L
.5885 .20339 L
.60727 .19246 L
.62725 .18104 L
.64648 .17025 L
.66497 .16006 L
.68466 .1494 L
.70361 .13934 L
.72376 .12884 L
.74317 .11894 L
.76183 .10959 L
.7817 .09985 L
.80082 .09065 L
.82115 .08108 L
.84073 .07206 L
.85957 .06356 L
.87961 .05471 L
.89891 .04636 L
.91747 .03851 L
.93723 .03033 L
.95624 .02263 L
.97619 .01472 L
Mistroke
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{209.938, 129.062},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003A0000PA000`40O003h00OdGoo003AOol00=5o
o`00dGoo003AOol00=5oo`00dGoo000JOol00`00Oomoo`2dOol001Yoo`03001oogoo0;Aoo`006Woo
0P00]Goo000JOol00`00Oomoo`2/Ool20006Ool001Yoo`03001oogoo0:Uoo`<000Qoo`006Woo0P00
Z7oo0P002goo0002Ool20004Ool20004Ool20004Ool30003Ool00`00Oomoo`2UOol2000=Ool00005
Ool007ooOol00003Ool20003Ool01000Oomoo`0017oo00<007ooOol00Woo00<007ooOol0XWoo0`00
3goo00001Goo001oogoo000027oo00@007ooOol0009oo`@000=oo`800:5oo`<0015oo`0000Eoo`00
Oomoo`0000Qoo`04001oogoo0002Ool00`00Ool00004Ool00`00Oomoo`2NOol3000COol00005Ool0
07ooOol00008Ool01000Oomoo`000Woo00<007oo000017oo00<007ooOol0Vgoo0`005Woo0002Ool2
000:Ool20004Ool20004Ool2002JOol3000HOol001Yoo`03001oogoo09Moo`<001Yoo`006Woo00<0
07ooOol0UGoo0P007Goo000JOol2002COol3000OOol001Yoo`03001oogoo08moo`@0025oo`006Woo
00<007ooOol0Rgoo1@0097oo000JOol20029Ool300000goo0000000VOol001Yoo`03001oogoo08Io
o`80009oo`8002Qoo`006Woo00<007ooOol0Q7oo0P000Woo0P00:Woo000JOol00`00Oomoo`23Ool0
0`00Ool00002000/Ool001Yoo`8008=oo`<002moo`000Woo0P0017oo0P0017oo0P000goo0`0017oo
00<007ooOol0PGoo0P00<Goo00001Goo001oogoo00000goo0P000goo00@007ooOol000Eoo`05001o
ogooOol00022Ool2000bOol00005Ool007ooOol00008Ool01000Oomoo`001Goo00@007ooOomoo`80
07moo`<003=oo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool30004Ool00`00Oomoo`1lOol2
00000goo001oo`0cOol00005Ool007ooOol00008Ool01000Oomoo`000Woo00<007ooOol017oo00<0
07ooOol0NWoo0P000Woo00<007ooOol0<goo0002Ool2000:Ool20003Ool40003Ool2001iOol20003
Ool00`00Oomoo`0dOol001Yoo`03001oogoo07Ioo`8000Aoo`03001oogoo03Eoo`006Woo00<007oo
Ool0MGoo00<007ooOol00goo00<007ooOol0=Woo000JOol2001dOol20005Ool00`00Oomoo`0gOol0
01Yoo`03001oogoo06Eoo`T000=oo`8000Ioo`03001oogoo03Qoo`006Woo00<007ooOol0Hgoo0P00
2Goo0`001goo00<007ooOol0>Goo000JOol2001SOol00`00Oomoo`09Ool00`00Ool00008000kOol0
01Yoo`03001oogoo061oo`8000Yoo`8004Ioo`006Woo00<007ooOol0H7oo00<007ooOol01goo0P00
B7oo000JOol2001QOol00`00Oomoo`05Ool2001:Ool0009oo`8000Aoo`8000Aoo`8000Aoo`8000Ao
o`03001oogoo061oo`03001oogoo00Aoo`03001oogoo04Yoo`0000Eoo`00Oomoo`0000=oo`8000=o
o`04001oogoo0002Ool01000Oomoo`000goo00<007ooOol0Ggoo00<007ooOol00goo0P00CGoo0000
1Goo001oogoo000027oo00@007ooOol0009oo`04001oogoo0003Ool2001POol01000Oomoogoo0P00
Cgoo00001Goo001oogoo000027oo00@007ooOol0009oo`<000Aoo`03001oogoo05ioo`03001oogoo
0080055oo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool00`00Oomoo`04Ool00`00Oomoo`1N
Ool3001COol0009oo`8000Yoo`8000Aoo`<000=oo`8005ioo`8005Eoo`006Woo00<007ooOol0G7oo
0P00EWoo000JOol00`00Oomoo`1JOol3001GOol001Yoo`8005Yoo`04001oogoo001GOol001Yoo`03
001oogoo05Qoo`05001oogooOol0001GOol001Yoo`03001oogoo05Ioo`8000=oo`03001oogoo05Io
o`006Woo0P00EWoo00<007ooOol00goo00<007ooOol0EWoo000JOol00`00Oomoo`17Ool40008Ool2
0005Ool00`00Oomoo`1GOol001Yoo`03001oogoo04Ioo`03001oogoo009oo`03001oogoo00=oo`80
00Moo`03001oogoo05Moo`006Woo0P00AGoo0P001Woo00<007ooOol00P0027oo00<007ooOol0F7oo
000JOol00`00Oomoo`13Ool00`00Oomoo`06Ool00`00Ool0000:Ool00`00Oomoo`1HOol0009oo`80
00Aoo`8000Aoo`8000Aoo`03001oogoo00=oo`03001oogoo049oo`03001oogoo00Qoo`03001oogoo
00Uoo`03001oogoo05Qoo`0000Eoo`00Oomoo`0000=oo`8000=oo`04001oogoo0003Ool00`00Oomo
o`03Ool00`00Oomoo`11Ool00`00Oomoo`07Ool200000goo001oo`08Ool00`00Oomoo`1IOol00005
Ool007ooOol00008Ool01000Oomoo`0017oo00<007ooOol00Woo0P00@Goo00<007ooOol01goo00<0
07ooOol00Woo00<007ooOol01Woo00<007ooOol0FGoo00001Goo001oogoo000027oo00@007ooOol0
00Aoo`03001oogoo009oo`03001oogoo041oo`03001oogoo00Eoo`8000Ioo`03001oogoo00Eoo`03
001oogoo05Uoo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0005Ool01@00Oomoogoo0000@Woo00<0
07ooOol00goo0P0027oo00<007ooOol017oo00<007ooOol0FWoo0002Ool2000:Ool20003Ool40003
Ool20010Ool00`00Oomoo`02Ool2000;Ool00`00Oomoo`03Ool00`00Oomoo`1JOol001Yoo`03001o
ogoo03moo`05001oogooOol0000>Ool01@00Oomoogoo0000GGoo000JOol00`00Oomoo`0nOol01@00
Oomoogoo000047oo0`00GWoo000JOol2000oOol01000Ool000004goo00<007ooOol0G7oo000JOol0
0`00Oomoo`0nOol2001dOol001Yoo`03001oogoo03eoo`8007Eoo`006Woo0P00?Goo0P00MWoo000J
Ool00`00Oomoo`0kOol2001gOol001Yoo`03001oogoo03Uoo`<007Qoo`006Woo0P00>Goo00@007oo
Ool007Qoo`000Woo0P0017oo0P0017oo0P0017oo0P0017oo00<007ooOol0=Woo0P000Woo00<007oo
Ool0Mgoo00001Goo001oogoo00000goo0P000goo00@007ooOol0009oo`04001oogoo0003Ool00`00
Oomoo`0eOol00`00Oomoo`02Ool00`00Oomoo`1gOol00005Ool007ooOol00008Ool01000Oomoo`00
0Woo00@007ooOol000=oo`8003Eoo`03001oogoo009oo`03001oogoo07Qoo`0000Eoo`00Oomoo`00
00Qoo`04001oogoo0003Ool20004Ool00`00Oomoo`0bOol20005Ool00`00Oomoo`1hOol00005Ool0
07ooOol00008Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oogoo035oo`03001oogoo00Eo
o`03001oogoo07Qoo`000Woo0P002Woo0P0017oo0P0017oo0P00;7oo00<007ooOol00Woo00<007oo
Ool01Goo00<007ooOol0NGoo000JOol00`00Oomoo`0YOol70008Ool00`00Oomoo`1iOol001Yoo`03
001oogoo02Qoo`03001oogoo009oo`8000Uoo`03001oogoo07Uoo`006Woo0P00:Goo00<007ooOol0
0P000Woo0P001Woo00<007ooOol0NWoo000JOol00`00Oomoo`0WOol01000Oomoo`001Woo00<007oo
Ool00Woo00<007ooOol0Ngoo000JOol00`00Oomoo`0VOol01000Oomoo`0027oo1000OWoo000JOol2
000VOol01000Oomoo`00Rgoo000JOol00`00Oomoo`0UOol00`00Ool0002<Ool001Yoo`03001oogoo
02Aoo`03001oo`0008eoo`006Woo0P0097oo0`00SWoo000JOol00`00Oomoo`0ROol2002@Ool0009o
o`8000Aoo`8000Aoo`8000=oo`<000Aoo`03001oogoo021oo`80099oo`0000Eoo`00Oomoo`0000=o
o`8000=oo`04001oogoo0005Ool01@00Oomoogoo00008Goo00<007ooOol0TWoo00001Goo001oogoo
000027oo00@007ooOol000=oo`<000=oo`8001moo`03001oogoo09=oo`0000Eoo`00Oomoo`0000Qo
o`04001oogoo0002Ool01000Oomoo`000goo00<007ooOol07Goo00<007ooOol0U7oo00001Goo001o
ogoo000027oo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`0LOol00`00Oomoo`2EOol0
009oo`8000Yoo`8000Aoo`8000Aoo`8001aoo`03001oogoo09Ioo`006Woo00<007ooOol06Goo0P00
VGoo000JOol00`00Oomoo`0HOol00`00Oomoo`2IOol001Yoo`8001Qoo`03001oogoo00eoo`<002Io
o`<000Aoo`8002=oo`<000=oo`<002=oo`<000=oo`@0009oo`006Woo00<007ooOol05Goo0P004goo
00<007ooOol097oo00<007ooOol00Woo00@007ooOol002=oo`03001oogoo00=oo`03001oogoo02=o
o`03001oogoo00=oo`03001oogoo009oo`006Woo00<007ooOol04goo0P004goo0`009Woo00<007oo
Ool00Woo00@007ooOol002=oo`03001oogoo00=oo`03001oogoo02=oo`03001oogoo00Aoo`03001o
ogoo005oo`006Woo0P004goo0P004goo00@007ooOol002Ioo`03001oogoo009oo`04001oogoo000S
Ool00`00Oomoo`03Ool00`00Oomoo`0SOol00`00Oomoo`05Ool10001Ool1Ool001Yoo`03001oogoo
015oo`8001Aoo`04001oogoo000VOol00`00Oomoo`02Ool01000Oomoo`008goo00<007ooOol00goo
00<007ooOol08goo00<007ooOol00Woo00@007ooOol0009oo`006Woo00<007ooOol047oo00<007oo
Ool05Goo0P009Woo0P001Goo0P008goo0P0017oo0P0097oo0P001Goo0P000goo000JOol2000?Ool2
002TOol001Yoo`03001oogoo00eoo`03001oogoo0:Aoo`006Woo00<007ooOol037oo00<007ooOol0
YGoo000EOonk0001Ool001Yoo`03001oogoo00Ioo`05001oogooOol00003Ool00`00Oomoo`06Ool0
0`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`05Ool0
0`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`06Ool0
0`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`06Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`04Ool001Yoo`03001oogoo00Uoo`03001oogoo
0:Qoo`006Woo0P0027oo0P00Zgoo000JOol00`00Oomoo`05Ool2002]Ool001Yoo`03001oogoo00=o
o`800:moo`006Woo0P000goo00<007ooOol0[goo000JOol01@00Oomoogoo0000/Woo000JOol01000
Oomoo`00/goo000JOol3002dOol001Yoo`800;Eoo`006Woo00<007ooOol0]7oo000JOol2002eOol0
01Yoo`03001oogoo0;Aoo`006Woo00<007ooOol0]7oo003AOol00=5oo`00dGoo003AOol00=5oo`00
dGoo0000\
\>"],
  ImageRangeCache->{{{0, 208.938}, {128.062, 0}} -> {7.40393, 0.0304987, 
  0.0226478, 0.000645918}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Theta Analysis", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
Now we make a table of percentage errors with the Crank-Nicholson \
scheme:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
thetasamples = TableForm[
Table[Map[PaddedForm[N[#], {5, 4}]&, 
{S, CNTheta[10, 0.05, 0, S, 5, 0.2],
BlackScholesPutTheta[S,10,0.2,0.05,0, 5],
100*(CNTheta[10,0.05,0,S,5,0.2]/BlackScholesPutTheta[S,10,0.2,0.05,0,5]-1)}], \
{S, 2, 16, 1}]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakBelow->False,
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3885\"\>",
                0.38851270554641615,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3885\"\>",
                0.38850954391536052,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0008\"\>",
                0.00081378465603965111,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3772\"\>",
                0.37721347538880601,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3772\"\>",
                0.3771662723552861,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0125\"\>",
                0.012515178842775754,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3406\"\>",
                0.34056885865693026,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3405\"\>",
                0.34052310009601122,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0134\"\>",
                0.013437725929943412,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2792\"\>",
                0.27917806412874396,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2792\"\>",
                0.27922180686816422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0157\"\>",
                -0.015665946693377286,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2072\"\>",
                0.20721556335139257,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2073\"\>",
                0.20732767425793053,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0541\"\>",
                -0.054074260437841914,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1390\"\>",
                0.13903380250201147,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1391\"\>",
                0.13914229271285256,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0780\"\>",
                -0.07797069368763454,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0828\"\>",
                0.082807518829334043,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0829\"\>",
                0.082893517820476131,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1037\"\>",
                -0.10374634036927066,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0401\"\>",
                0.040056074018968581,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0410\"\>",
                0.040962269627171116,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-2.2123\"\>",
                -2.212269037947634,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0362\"\>",
                0.036243666764467948,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0122\"\>",
                0.012207835061223929,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 196.8900\"\>",
                196.88856855209056,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0065\"\>",
                -0.0064906233576429674,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0060\"\>",
                -0.005966431371365169,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 8.7857\"\>",
                8.7856870154170217,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0163\"\>",
                -0.016282040558196806,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0164\"\>",
                -0.016364675313776202,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.5050\"\>",
                -0.50495811248899569,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0214\"\>",
                -0.02141592585866452,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0214\"\>",
                -0.021421153016455467,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0244\"\>",
                -0.024401850763733357,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0230\"\>",
                -0.02302796572818885,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0230\"\>",
                -0.023024844458457053,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0136\"\>",
                0.013556094754219039,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0226\"\>",
                -0.022554893068099297,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0225\"\>",
                -0.022539828190855676,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0668\"\>",
                0.066836699534955635,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0209\"\>",
                -0.020920496184261408,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0209\"\>",
                -0.020899374917308117,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1011\"\>",
                0.10106171613677439,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell[TextData[StyleBox[
"Near the strike the error in Theta peaks at 197% of the exact value.",
  FontWeight->"Bold"]], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Theta Plots", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[{CNTheta[10, \ 0.05, \ 0, \ S, \ 5, \ 0.2], \n
        BlackScholesPutTheta[S, 10, 0.2, 0.05, 0, \ 5]}, \ {S, \ 8, \ 12}, \ 
      PlotPoints\  -> \ 50, \n\tPlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.88095 0.238095 0.111758 5.93003 [
[.2619 .09926 -3 -9 ]
[.2619 .09926 3 0 ]
[.5 .09926 -6 -9 ]
[.5 .09926 6 0 ]
[.7381 .09926 -6 -9 ]
[.7381 .09926 6 0 ]
[.97619 .09926 -6 -9 ]
[.97619 .09926 6 0 ]
[.01131 .23036 -24 -4.5 ]
[.01131 .23036 0 4.5 ]
[.01131 .34896 -24 -4.5 ]
[.01131 .34896 0 4.5 ]
[.01131 .46756 -24 -4.5 ]
[.01131 .46756 0 4.5 ]
[.01131 .58616 -24 -4.5 ]
[.01131 .58616 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .11176 m
.2619 .11801 L
s
[(9)] .2619 .09926 0 1 Mshowa
.5 .11176 m
.5 .11801 L
s
[(10)] .5 .09926 0 1 Mshowa
.7381 .11176 m
.7381 .11801 L
s
[(11)] .7381 .09926 0 1 Mshowa
.97619 .11176 m
.97619 .11801 L
s
[(12)] .97619 .09926 0 1 Mshowa
.125 Mabswid
.07143 .11176 m
.07143 .11551 L
s
.11905 .11176 m
.11905 .11551 L
s
.16667 .11176 m
.16667 .11551 L
s
.21429 .11176 m
.21429 .11551 L
s
.30952 .11176 m
.30952 .11551 L
s
.35714 .11176 m
.35714 .11551 L
s
.40476 .11176 m
.40476 .11551 L
s
.45238 .11176 m
.45238 .11551 L
s
.54762 .11176 m
.54762 .11551 L
s
.59524 .11176 m
.59524 .11551 L
s
.64286 .11176 m
.64286 .11551 L
s
.69048 .11176 m
.69048 .11551 L
s
.78571 .11176 m
.78571 .11551 L
s
.83333 .11176 m
.83333 .11551 L
s
.88095 .11176 m
.88095 .11551 L
s
.92857 .11176 m
.92857 .11551 L
s
.25 Mabswid
0 .11176 m
1 .11176 L
s
.02381 .23036 m
.03006 .23036 L
s
[(0.02)] .01131 .23036 1 0 Mshowa
.02381 .34896 m
.03006 .34896 L
s
[(0.04)] .01131 .34896 1 0 Mshowa
.02381 .46756 m
.03006 .46756 L
s
[(0.06)] .01131 .46756 1 0 Mshowa
.02381 .58616 m
.03006 .58616 L
s
[(0.08)] .01131 .58616 1 0 Mshowa
.125 Mabswid
.02381 .14141 m
.02756 .14141 L
s
.02381 .17106 m
.02756 .17106 L
s
.02381 .20071 m
.02756 .20071 L
s
.02381 .26001 m
.02756 .26001 L
s
.02381 .28966 m
.02756 .28966 L
s
.02381 .31931 m
.02756 .31931 L
s
.02381 .37861 m
.02756 .37861 L
s
.02381 .40826 m
.02756 .40826 L
s
.02381 .43791 m
.02756 .43791 L
s
.02381 .49721 m
.02756 .49721 L
s
.02381 .52686 m
.02756 .52686 L
s
.02381 .55651 m
.02756 .55651 L
s
.02381 .08211 m
.02756 .08211 L
s
.02381 .05246 m
.02756 .05246 L
s
.02381 .02281 m
.02756 .02281 L
s
.02381 .61581 m
.02756 .61581 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
.5 Mabswid
.02381 .60281 m
.04273 .58001 L
.06337 .55582 L
.08275 .5338 L
.10139 .5132 L
.12123 .49184 L
.14033 .4718 L
.16063 .45087 L
.18019 .43079 L
.199 .41161 L
.21902 .39153 L
.23829 .37182 L
.25683 .35392 L
.27656 .33695 L
.29555 .3238 L
.3052 .318 L
.31574 .31234 L
.32062 .30995 L
.3233 .3087 L
.32583 .30756 L
.32697 .30706 L
.3276 .30679 L
.32819 .30666 L
.32882 .30675 L
.32951 .30685 L
.33074 .30703 L
.33519 .30769 L
.33764 .30803 L
.34031 .30838 L
.34266 .30863 L
.34395 .30875 L
.34516 .30884 L
.34624 .3089 L
.34679 .30893 L
.3474 .30895 L
.34803 .30897 L
.34837 .30898 L
.34869 .30898 L
.34927 .30899 L
.34988 .30898 L
.35044 .30898 L
.35106 .30896 L
.35163 .30894 L
.35217 .30892 L
.35318 .30885 L
.35425 .30876 L
.35549 .30863 L
.35666 .30846 L
.35887 .30806 L
.36124 .30748 L
Mistroke
.36383 .30665 L
.36849 .30461 L
.3728 .30202 L
.37799 .29789 L
.38054 .29543 L
.38165 .29425 L
.38288 .29289 L
.38341 .29228 L
.38398 .29138 L
.38452 .28954 L
.38501 .28785 L
.38726 .27997 L
.392 .26282 L
.41154 .1903 L
.42048 .16042 L
.42557 .1455 L
.43034 .13329 L
.4354 .12255 L
.43762 .11866 L
.43887 .11669 L
.44004 .11502 L
.44064 .11421 L
.44121 .1135 L
.44152 .1144 L
.44186 .11541 L
.44246 .11721 L
.445 .12532 L
.45034 .14425 L
.4704 .2276 L
.48039 .26837 L
.48973 .30097 L
.49433 .31405 L
.4966 .31959 L
.49869 .32415 L
.4993 .32534 L
.49986 .32642 L
.50045 .32531 L
.50076 .32434 L
.5011 .32327 L
.50372 .31453 L
.50831 .2973 L
.52845 .20254 L
.53849 .1508 L
.54784 .10466 L
.55247 .0835 L
.55685 .06499 L
.55803 .0603 L
.55927 .05548 L
.55992 .05303 L
.56029 .05172 L
Mistroke
.56064 .05174 L
.56124 .05181 L
.56159 .05186 L
.5619 .05192 L
.56302 .05221 L
.56366 .05242 L
.56425 .05265 L
.56649 .05381 L
.56898 .0556 L
.57167 .05807 L
.57655 .06384 L
.58566 .07824 L
.60576 .11894 L
.61059 .12904 L
.61577 .13946 L
.61828 .14427 L
.61951 .14656 L
.62065 .14864 L
.62169 .15048 L
.62221 .15115 L
.62279 .15111 L
.62336 .15105 L
.62398 .15099 L
.62511 .15082 L
.62737 .15037 L
.62984 .14967 L
.63482 .14768 L
.63943 .14521 L
.64371 .14243 L
.66285 .12578 L
.67261 .11565 L
.67775 .11015 L
.68318 .10433 L
.68804 .10026 L
.69338 .09691 L
.70278 .09129 L
.72254 .0806 L
.74157 .07176 L
.7618 .06431 L
.78128 .05869 L
.80002 .05407 L
.81997 .04956 L
.83917 .04497 L
.85957 .04025 L
.87923 .03584 L
.89815 .03158 L
.91827 .02711 L
.93764 .02294 L
.95628 .01909 L
.97611 .01522 L
Mistroke
.97619 .01521 L
Mfstroke
.02381 .60332 m
.04273 .58045 L
.06337 .55612 L
.08275 .53387 L
.10139 .51301 L
.12123 .49138 L
.14033 .47112 L
.16063 .45018 L
.18019 .43058 L
.199 .41225 L
.21902 .39331 L
.23829 .37562 L
.25683 .35911 L
.27656 .34205 L
.29555 .32614 L
.31574 .30976 L
.33519 .29449 L
.3539 .28027 L
.3738 .26564 L
.39297 .25203 L
.41334 .23806 L
.43296 .22508 L
.45184 .21303 L
.47193 .20066 L
.49127 .18919 L
.50987 .17856 L
.52967 .16765 L
.54872 .15755 L
.56898 .14723 L
.5885 .13768 L
.60727 .12885 L
.62725 .11983 L
.64648 .11151 L
.66497 .10383 L
.68466 .09598 L
.70361 .08875 L
.72376 .0814 L
.74317 .07464 L
.76183 .06842 L
.7817 .0621 L
.80082 .0563 L
.82115 .05043 L
.84073 .04506 L
.85957 .04014 L
.87961 .03516 L
.89891 .03062 L
.91747 .02647 L
.93723 .02229 L
.95624 .01849 L
.97619 .01472 L
Mistroke
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{222.938, 137.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003N0000RA000`40O003h00OgWoo003NOol00=io
o`00gWoo003NOol00=ioo`00gWoo000JOol00`00Oomoo`31Ool001Yoo`03001oogoo0<5oo`006Woo
00<007ooOol0`Goo000JOol00`00Oomoo`2eOol60006Ool001Yoo`800;9oo`L000Uoo`006Woo00<0
07ooOol0[Goo100047oo000JOol00`00Oomoo`2YOol4000DOol001Yoo`03001oogoo0:Eoo`@001Qo
o`006Woo00<007ooOol0XGoo100077oo000JOol00`00Oomoo`2MOol4000POol001Yoo`8006Uoo`03
001oogoo02ioo`D002=oo`006Woo00<007ooOol0J7oo0`00:7oo1`009goo000JOol00`00Oomoo`1W
Ool01000Oomoo`009Woo1@00:goo000JOol00`00Oomoo`0ZOol3000ZOol30004Ool20007Ool01@00
Oomoogoo00006Woo0`000goo1P008goo0`000goo10000Woo000JOol00`00Oomoo`0]Ool00`00Oomo
o`0XOol00`00Oomoo`02Ool01000Oomoo`001Woo00<007ooOol00Woo00<007ooOol067oo00@007oo
Oomoo`@002Moo`03001oogoo00=oo`03001oogoo009oo`006Woo00<007ooOol0:goo0`00:Woo00<0
07ooOol00Woo00@007ooOol000Eoo`03001oogoo00Aoo`03001oogoo01Moo`03001oo`0000@002Qo
o`03001oogoo00Aoo`03001oogoo005oo`006Woo0P00:goo00@007ooOol002Yoo`03001oogoo009o
o`04001oogoo0005Ool00`00Oomoo`04Ool00`00Oomoo`0FOol40003Ool00`00Oomoo`0VOol00`00
Oomoo`05Ool10001Ool1Ool001Yoo`03001oogoo02Yoo`04001oogoo000ZOol00`00Oomoo`02Ool0
1000Oomoo`001Goo00<007ooOol01Goo00<007ooOol04Woo1@001Goo00<007ooOol09Woo00<007oo
Ool00Woo00@007ooOol0009oo`006Woo00<007ooOol0:goo0P00:Woo0P001Goo0P001Woo00<007oo
Ool01Goo00<007ooOol047oo100000=oo`00000017oo0P009goo0P001Goo0P000goo000JOol00`00
Oomoo`1UOol00`00Oomoo`07Ool00`00Oomoo`0=Ool200000goo001oo`0mOol001Yoo`03001oogoo
06Eoo`03001oogoo00Moo`03001oogoo00Yoo`<0009oo`03001oogoo03eoo`006Woo00<007ooOol0
IGoo00<007ooOol027oo00<007ooOol01goo0P0017oo00<007ooOol0?Woo000EOoo80001Ool001Yo
o`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Io
o`03001oogoo00Moo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00=oo`800003Ool007oo
00Qoo`03001oogoo00Ioo`03001oogoo00Moo`03001oo`0000Aoo`<000Eoo`05001oogooOol00008
Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`06
Ool00`00Oomoo`04Ool001Yoo`03001oogoo04moo`8001=oo`03001oogoo00Yoo`03001oo`000080
00Moo`03001oogoo045oo`006Woo00<007ooOol0Cgoo00<007oo00004Woo00<007ooOol02Goo0`00
27oo0P00A7oo000JOol00`00Oomoo`1>Ool01000Oomoo`004Woo00<007ooOol01goo0P000Woo00<0
07ooOol01Goo00<007ooOol0A7oo000JOol00`00Oomoo`1>Ool01000Oomoo`004Woo00<007ooOol0
1Goo0P001Goo00<007ooOol00goo00<007ooOol0AGoo000JOol2001>Ool00`00Oomoo`02Ool00`00
Oomoo`0?Ool00`00Oomoo`03Ool20007Ool00`00Oomoo`02Ool00`00Oomoo`16Ool001Yoo`03001o
ogoo04eoo`03001oogoo009oo`03001oogoo00ioo`03001oogoo009oo`8000Uoo`80009oo`03001o
ogoo04Moo`006Woo00<007ooOol0CGoo00<007ooOol00Woo00<007ooOol03Woo00<007ooOol00P00
37oo0`00BWoo000JOol00`00Oomoo`1<Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool3001KOol001Yo
o`03001oogoo04aoo`03001oogoo00Aoo`03001oogoo00eoo`03001oogoo05]oo`006Woo00<007oo
Ool0C7oo00<007ooOol017oo00<007ooOol02goo0P00GWoo000JOol2001<Ool00`00Oomoo`05Ool0
0`00Oomoo`09Ool200000goo001oo`1MOol001Yoo`03001oogoo04]oo`03001oogoo00Eoo`03001o
ogoo00Moo`8000=oo`03001oogoo05aoo`006Woo00<007ooOol0Bgoo00<007ooOol01Woo00<007oo
Ool017oo0P001Goo00<007ooOol0G7oo000JOol00`00Oomoo`1:Ool00`00Oomoo`07Ool00`00Oomo
o`02Ool20007Ool00`00Oomoo`1LOol001Yoo`03001oogoo04Yoo`03001oogoo00Moo`05001oogoo
Ool00008Ool00`00Oomoo`1MOol001Yoo`03001oogoo04Yoo`03001oogoo00Moo`04001oogoo0009
Ool00`00Oomoo`1MOol001Yoo`8004]oo`03001oogoo00Qoo`8000Yoo`03001oogoo05eoo`006Woo
00<007ooOol0BGoo00<007ooOol01goo0`002goo00<007ooOol0GGoo000JOol00`00Oomoo`19Ool0
0`00Oomoo`05Ool20002Ool00`00Oomoo`09Ool00`00Oomoo`1MOol001Yoo`03001oogoo04Uoo`03
001oogoo00Aoo`03001oogoo009oo`03001oogoo00Qoo`03001oogoo05ioo`000Woo0P0017oo0P00
17oo0P000goo10000goo00<007ooOol0B7oo00<007ooOol017oo00<007ooOol017oo00<007ooOol0
1goo00<007ooOol0GWoo00001Goo001oogoo00000goo0P000goo00@007ooOol000=oo`03001oogoo
00=oo`03001oogoo04Qoo`03001oogoo009oo`8000Moo`03001oogoo00Moo`03001oogoo05ioo`00
00Eoo`00Oomoo`0000Qoo`04001oogoo0004Ool00`00Oomoo`02Ool20019Ool00`00Oomoo`020009
Ool00`00Oomoo`07Ool00`00Oomoo`1NOol00005Ool007ooOol00008Ool01000Oomoo`001Goo00D0
07ooOomoo`0004Yoo`<000]oo`03001oogoo00Moo`03001oogoo05ioo`0000Eoo`00Oomoo`0000Qo
o`04001oogoo0002Ool01000Oomoo`000goo00<007ooOol0Agoo0P003Woo00<007ooOol01Goo00<0
07ooOol0Ggoo0002Ool2000:Ool20004Ool20004Ool00`00Oomoo`17Ool00`00Oomoo`0=Ool00`00
Oomoo`05Ool00`00Oomoo`1OOol001Yoo`03001oogoo04Ioo`8000moo`03001oogoo00Eoo`03001o
ogoo05moo`006Woo00<007ooOol0AGoo0P0047oo00<007ooOol01Goo00<007ooOol0Ggoo000JOol2
0015Ool00`00Ool0000AOol00`00Oomoo`04Ool00`00Oomoo`1OOol001Yoo`03001oogoo049oo`80
009oo`03001oogoo00moo`03001oogoo00=oo`03001oogoo061oo`006Woo00<007ooOol0@Goo00<0
07ooOol00Woo00<007ooOol03goo00<007ooOol00goo00<007ooOol0H7oo000JOol00`00Oomoo`10
Ool00`00Oomoo`02Ool00`00Oomoo`0@Ool00`00Oomoo`03Ool00`00Oomoo`1POol001Yoo`03001o
ogoo03ioo`8000Eoo`03001oogoo015oo`03001oogoo009oo`03001oogoo061oo`006Woo0P00?Goo
0P001goo00<007ooOol04Goo00<007ooOol00Woo00<007ooOol0H7oo000JOol00`00Oomoo`0jOol2
0008Ool2000COol01@00Oomoogoo0000Hgoo000JOol00`00Oomoo`0iOol00`00Oomoo`07Ool2000E
Ool01000Oomoo`00Hgoo000JOol00`00Oomoo`0hOol00`00Oomoo`07Ool00`00Oomoo`0EOol01000
Oomoo`00Hgoo000JOol00`00Oomoo`0gOol;000HOol00`00Ool0001TOol001Yoo`03001oogoo03Io
o`8002=oo`8006Aoo`006Woo0P00=Goo0P009Goo0P00I7oo000JOol00`00Oomoo`0bOol3000VOol2
001TOol001Yoo`03001oogoo035oo`<002Qoo`03001oogoo069oo`006Woo00<007ooOol0<7oo0P00
Sgoo0002Ool20004Ool20004Ool20004Ool30003Ool00`00Oomoo`0^Ool3002@Ool00005Ool007oo
Ool00003Ool20003Ool01000Oomoo`0017oo00<007ooOol00Woo00<007ooOol0;Goo0`00TGoo0000
1Goo001oogoo000027oo00@007ooOol0009oo`@000=oo`8002eoo`8009=oo`0000Eoo`00Oomoo`00
00Qoo`04001oogoo0002Ool00`00Ool00004Ool00`00Oomoo`0[Ool2002DOol00005Ool007ooOol0
0008Ool01000Oomoo`000Woo00<007oo000017oo00<007ooOol0:goo00<007ooOol0Tgoo0002Ool2
000:Ool20004Ool20004Ool00`00Oomoo`0ZOol2002EOol001Yoo`03001oogoo02Uoo`8009Ioo`00
6Woo00<007ooOol0:7oo0P00Ugoo000JOol2000XOol2002HOol001Yoo`03001oogoo02Ioo`03001o
ogoo09Qoo`006Woo00<007ooOol09Goo00<007ooOol0VGoo000JOol00`00Oomoo`0TOol00`00Oomo
o`2JOol001Yoo`03001oogoo02=oo`03001oogoo09]oo`006Woo00<007ooOol08Woo00<007ooOol0
W7oo000JOol2000ROol00`00Oomoo`2MOol001Yoo`03001oogoo01moo`800:1oo`006Woo00<007oo
Ool07Woo0P00XGoo000JOol00`00Oomoo`0MOol2002ROol001Yoo`03001oogoo01aoo`800:=oo`00
6Woo00<007ooOol06goo0P00Y7oo000JOol2000KOol00`00Oomoo`2TOol001Yoo`03001oogoo01Uo
o`03001oogoo0:Eoo`006Woo00<007ooOol067oo00<007ooOol0YWoo000JOol00`00Oomoo`0HOol0
0`00Oomoo`2VOol0009oo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`03001oogoo01Moo`03001oogoo
0:Moo`0000Eoo`00Oomoo`0000=oo`8000=oo`04001oogoo0002Ool01000Oomoo`000goo00<007oo
Ool05Woo00<007ooOol0Z7oo00001Goo001oogoo000027oo00@007ooOol0009oo`04001oogoo0003
Ool2000FOol00`00Oomoo`2YOol00005Ool007ooOol00008Ool01000Oomoo`000Woo0`0017oo00<0
07ooOol057oo00<007ooOol0ZWoo00001Goo001oogoo000027oo00@007ooOol0009oo`03001oogoo
00Aoo`03001oogoo01=oo`03001oogoo0:]oo`000Woo0P002Woo0P0017oo0`000goo00<007ooOol0
4Woo00<007ooOol0[7oo000JOol00`00Oomoo`0AOol00`00Oomoo`2]Ool001Yoo`03001oogoo011o
o`03001oogoo0:ioo`006Woo0P0047oo00<007ooOol0[goo000JOol00`00Oomoo`0>Ool00`00Oomo
o`2`Ool001Yoo`03001oogoo00eoo`03001oogoo0;5oo`006Woo00<007ooOol037oo00<007ooOol0
/Woo000JOol00`00Oomoo`0<Ool00`00Oomoo`2bOol001Yoo`03001oogoo00]oo`03001oogoo0;=o
o`006Woo0P002goo00<007ooOol0]7oo000JOol00`00Oomoo`09Ool00`00Oomoo`2eOol001Yoo`03
001oogoo00Qoo`03001oogoo0;Ioo`006Woo00<007ooOol01goo00<007ooOol0]goo000JOol00`00
Oomoo`06Ool00`00Oomoo`2hOol001Yoo`03001oogoo00Eoo`03001oogoo0;Uoo`006Woo0P001Woo
00<007ooOol0^Goo000JOol00`00Oomoo`04Ool00`00Oomoo`2jOol001Yoo`03001oogoo00=oo`03
001oogoo0;]oo`000Woo0P0017oo0P0017oo0P0017oo0P0017oo00<007ooOol00Woo00<007ooOol0
_7oo00001Goo001oogoo00000goo0P000goo00@007ooOol0009oo`04001oogoo0003Ool01@00Oomo
ogoo0000_goo00001Goo001oogoo000027oo00@007ooOol0009oo`04001oogoo0003Ool20002Ool0
0`00Oomoo`2mOol00005Ool007ooOol00008Ool01000Oomoo`000goo0P0017oo00@007ooOol00<1o
o`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool01000Oomoo`000goo00<007oo0000`Goo0002
Ool2000:Ool20004Ool20004Ool20032Ool001Yoo`03001oogoo0<5oo`006Woo00<007ooOol0`Goo
000JOol20032Ool001Yoo`03001oogoo0<5oo`00gWoo003NOol00=ioo`00gWoo003NOol00=ioo`00
\
\>"],
  ImageRangeCache->{{{0, 221.938}, {136.438, 0}} -> {7.44075, -0.0245674, 
  0.0211139, 0.000847737}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Remark on Two-Time-Level Douglas", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This time, there is no benefit in going to the simple Douglas scheme as \
described in Chapter 14. The oscillations in the neighbourhood of the strike \
are just as bad. For example, here is the error in valuation with ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 8\)]],
  ", for comparison. There are corresponding problems in the Greeks."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0263158 0.0501253 0.380262 110.364 [
[.099 .36776 -9 -9 ]
[.099 .36776 9 0 ]
[.34962 .36776 -9 -9 ]
[.34962 .36776 9 0 ]
[.47494 .36776 -6 -9 ]
[.47494 .36776 6 0 ]
[.60025 .36776 -12 -9 ]
[.60025 .36776 12 0 ]
[.72556 .36776 -6 -9 ]
[.72556 .36776 6 0 ]
[.85088 .36776 -12 -9 ]
[.85088 .36776 12 0 ]
[.97619 .36776 -6 -9 ]
[.97619 .36776 6 0 ]
[.21181 .04917 -36 -4.5 ]
[.21181 .04917 0 4.5 ]
[.21181 .15953 -36 -4.5 ]
[.21181 .15953 0 4.5 ]
[.21181 .2699 -36 -4.5 ]
[.21181 .2699 0 4.5 ]
[.21181 .49063 -30 -4.5 ]
[.21181 .49063 0 4.5 ]
[.21181 .60099 -30 -4.5 ]
[.21181 .60099 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.099 .38026 m
.099 .38651 L
s
[(2.5)] .099 .36776 0 1 Mshowa
.34962 .38026 m
.34962 .38651 L
s
[(7.5)] .34962 .36776 0 1 Mshowa
.47494 .38026 m
.47494 .38651 L
s
[(10)] .47494 .36776 0 1 Mshowa
.60025 .38026 m
.60025 .38651 L
s
[(12.5)] .60025 .36776 0 1 Mshowa
.72556 .38026 m
.72556 .38651 L
s
[(15)] .72556 .36776 0 1 Mshowa
.85088 .38026 m
.85088 .38651 L
s
[(17.5)] .85088 .36776 0 1 Mshowa
.97619 .38026 m
.97619 .38651 L
s
[(20)] .97619 .36776 0 1 Mshowa
.125 Mabswid
.12406 .38026 m
.12406 .38401 L
s
.14912 .38026 m
.14912 .38401 L
s
.17419 .38026 m
.17419 .38401 L
s
.19925 .38026 m
.19925 .38401 L
s
.24937 .38026 m
.24937 .38401 L
s
.27444 .38026 m
.27444 .38401 L
s
.2995 .38026 m
.2995 .38401 L
s
.32456 .38026 m
.32456 .38401 L
s
.37469 .38026 m
.37469 .38401 L
s
.39975 .38026 m
.39975 .38401 L
s
.42481 .38026 m
.42481 .38401 L
s
.44987 .38026 m
.44987 .38401 L
s
.5 .38026 m
.5 .38401 L
s
.52506 .38026 m
.52506 .38401 L
s
.55013 .38026 m
.55013 .38401 L
s
.57519 .38026 m
.57519 .38401 L
s
.62531 .38026 m
.62531 .38401 L
s
.65038 .38026 m
.65038 .38401 L
s
.67544 .38026 m
.67544 .38401 L
s
.7005 .38026 m
.7005 .38401 L
s
.75063 .38026 m
.75063 .38401 L
s
.77569 .38026 m
.77569 .38401 L
s
.80075 .38026 m
.80075 .38401 L
s
.82581 .38026 m
.82581 .38401 L
s
.87594 .38026 m
.87594 .38401 L
s
.901 .38026 m
.901 .38401 L
s
.92607 .38026 m
.92607 .38401 L
s
.95113 .38026 m
.95113 .38401 L
s
.07393 .38026 m
.07393 .38401 L
s
.04887 .38026 m
.04887 .38401 L
s
.02381 .38026 m
.02381 .38401 L
s
.25 Mabswid
0 .38026 m
1 .38026 L
s
.22431 .04917 m
.23056 .04917 L
s
[(-0.003)] .21181 .04917 1 0 Mshowa
.22431 .15953 m
.23056 .15953 L
s
[(-0.002)] .21181 .15953 1 0 Mshowa
.22431 .2699 m
.23056 .2699 L
s
[(-0.001)] .21181 .2699 1 0 Mshowa
.22431 .49063 m
.23056 .49063 L
s
[(0.001)] .21181 .49063 1 0 Mshowa
.22431 .60099 m
.23056 .60099 L
s
[(0.002)] .21181 .60099 1 0 Mshowa
.125 Mabswid
.22431 .07124 m
.22806 .07124 L
s
.22431 .09332 m
.22806 .09332 L
s
.22431 .11539 m
.22806 .11539 L
s
.22431 .13746 m
.22806 .13746 L
s
.22431 .18161 m
.22806 .18161 L
s
.22431 .20368 m
.22806 .20368 L
s
.22431 .22575 m
.22806 .22575 L
s
.22431 .24783 m
.22806 .24783 L
s
.22431 .29197 m
.22806 .29197 L
s
.22431 .31404 m
.22806 .31404 L
s
.22431 .33612 m
.22806 .33612 L
s
.22431 .35819 m
.22806 .35819 L
s
.22431 .40234 m
.22806 .40234 L
s
.22431 .42441 m
.22806 .42441 L
s
.22431 .44648 m
.22806 .44648 L
s
.22431 .46855 m
.22806 .46855 L
s
.22431 .5127 m
.22806 .5127 L
s
.22431 .53477 m
.22806 .53477 L
s
.22431 .55684 m
.22806 .55684 L
s
.22431 .57892 m
.22806 .57892 L
s
.22431 .0271 m
.22806 .0271 L
s
.22431 .00502 m
.22806 .00502 L
s
.25 Mabswid
.22431 0 m
.22431 .61803 L
s
.5 Mabswid
.02381 .38066 m
.02491 .38045 L
.02609 .38028 L
.0271 .38016 L
.02821 .38005 L
.0295 .37995 L
.03073 .37988 L
.03131 .37986 L
.03192 .37982 L
.03244 .37981 L
.03302 .37979 L
.03426 .37976 L
.03487 .37975 L
.03544 .37974 L
.03578 .37974 L
.03609 .37973 L
.03643 .37973 L
.03679 .37972 L
.03743 .37972 L
.03804 .37972 L
.03861 .37972 L
.03921 .37972 L
.03973 .37972 L
.04029 .37973 L
.04086 .37973 L
.04146 .37973 L
.04178 .37974 L
.04212 .37975 L
.04273 .37975 L
.04308 .37975 L
.04345 .37976 L
.0438 .37977 L
.04412 .37978 L
.04543 .3798 L
.04605 .37982 L
.04662 .37983 L
.04723 .37985 L
.04788 .37987 L
.05341 .38009 L
.05822 .38036 L
.06337 .38072 L
.0837 .38269 L
.09388 .38369 L
.09836 .38407 L
.10073 .38423 L
.10328 .38439 L
.10575 .38451 L
.10801 .3846 L
.10853 .38461 L
.10909 .38463 L
Mistroke
.10971 .38464 L
.11028 .38465 L
.11082 .38466 L
.1114 .38467 L
.11189 .38468 L
.11243 .38468 L
.11275 .38468 L
.11308 .38468 L
.11369 .38468 L
.11406 .38468 L
.1144 .38468 L
.11471 .38468 L
.11505 .38468 L
.11567 .38467 L
.11599 .38466 L
.11633 .38466 L
.11752 .38463 L
.11866 .3846 L
.1193 .38457 L
.11989 .38455 L
.12212 .38444 L
.12445 .38428 L
.12693 .38407 L
.13141 .38358 L
.13673 .3828 L
.14155 .38191 L
.15097 .37971 L
.16116 .37668 L
.18123 .36932 L
.20055 .3614 L
.21914 .354 L
.23892 .34718 L
.25796 .3421 L
.26835 .34002 L
.2782 .3385 L
.28292 .33789 L
.28794 .33737 L
.29265 .33699 L
.29696 .33669 L
.29919 .33657 L
.30154 .33648 L
.30278 .33642 L
.3041 .33637 L
.30533 .33633 L
.30649 .3363 L
.30756 .33628 L
.30817 .33627 L
.30872 .33627 L
.31076 .33624 L
.31132 .33623 L
Mistroke
.31193 .33622 L
.31251 .33622 L
.31303 .33622 L
.31362 .33621 L
.31418 .33621 L
.31478 .33621 L
.31512 .33622 L
.31544 .33622 L
.31607 .33622 L
.31673 .33623 L
.31731 .33624 L
.31793 .33625 L
.31827 .33626 L
.31859 .33626 L
.31921 .33626 L
.31993 .33627 L
.32062 .33628 L
.32183 .3363 L
.32295 .33632 L
.3255 .3364 L
.33461 .33675 L
.33933 .33699 L
.34376 .33728 L
.35385 .3381 L
.36375 .33921 L
.37281 .34037 L
.3773 .34085 L
.37981 .34106 L
.38209 .3412 L
.38339 .34127 L
.38374 .34128 L
.38412 .34129 L
.38444 .34129 L
.38479 .34129 L
.38543 .34128 L
.38579 .34128 L
.38611 .34127 L
.38734 .34125 L
.38971 .34118 L
.39223 .3411 L
.39354 .34107 L
.39418 .34105 L
.39477 .34102 L
.39531 .34098 L
.39588 .34094 L
.39645 .34091 L
.39707 .34087 L
.39765 .34085 L
.39818 .34084 L
.39871 .34083 L
Mistroke
.39921 .34083 L
.39979 .34083 L
.40033 .34085 L
.40093 .34088 L
.40156 .34093 L
.40218 .34099 L
.40276 .34106 L
.40338 .34115 L
.40405 .34127 L
.40467 .34141 L
.40525 .34156 L
.4058 .34178 L
.40639 .34204 L
.41171 .34507 L
.41417 .34688 L
.41537 .34785 L
.41647 .34904 L
.41763 .35074 L
.41886 .35254 L
.42149 .356 L
.42285 .35745 L
.42356 .3581 L
.42432 .35868 L
.42498 .35909 L
.42534 .35927 L
.42572 .35943 L
.42606 .35954 L
.42638 .35962 L
.427 .35969 L
.42762 .3595 L
.42818 .35914 L
.42947 .35797 L
.43073 .35636 L
.43208 .35412 L
.43461 .34838 L
.43595 .34454 L
.43736 .33985 L
.43796 .33768 L
.4386 .33521 L
.43896 .33202 L
.4393 .32831 L
.43996 .32101 L
.44235 .29533 L
.44464 .27488 L
.44527 .27043 L
.44594 .26639 L
.44657 .26331 L
.44714 .26115 L
.44774 .25966 L
.44837 .25892 L
Mistroke
.44894 .25908 L
.44953 .26013 L
.45007 .26187 L
.45056 .26609 L
.45111 .28356 L
.45169 .30303 L
.45669 .47739 L
.45805 .51979 L
.45951 .55821 L
.46075 .58402 L
.46145 .59529 L
.46179 .59977 L
.46211 .60332 L
.46269 .60205 L
.46332 .57494 L
.46459 .5144 L
.46686 .39316 L
.47197 .12147 L
.47318 .06975 L
.47384 .04498 L
.47445 .02439 L
.47477 .01472 L
.47512 .01524 L
.4755 .02705 L
.47585 .03869 L
.47714 .08794 L
.48201 .32882 L
.48424 .4451 L
.48543 .50229 L
.48668 .55763 L
.48721 .5787 L
.48778 .59339 L
.4884 .58771 L
.48898 .58065 L
.4901 .56308 L
.49112 .54281 L
.49559 .42366 L
.49805 .35049 L
.49925 .31596 L
.49978 .30144 L
.50037 .28605 L
.50101 .27784 L
.50136 .27681 L
.50169 .27607 L
.50205 .2755 L
.50238 .27519 L
.50275 .27508 L
.50312 .27521 L
.50374 .27594 L
.50431 .27715 L
Mistroke
.50494 .27903 L
.50562 .28162 L
.50693 .28824 L
.50818 .29602 L
.5105 .31336 L
.5117 .32311 L
.51298 .33377 L
.5136 .33891 L
.51418 .34267 L
.51471 .34426 L
.51529 .34592 L
.51769 .35213 L
.52034 .35758 L
.52262 .3612 L
.52387 .36278 L
.52504 .36401 L
.52612 .36494 L
.52711 .36561 L
.52764 .36591 L
.5282 .36599 L
.52877 .36603 L
.52939 .36601 L
.53003 .36594 L
.53063 .36582 L
.53128 .36565 L
.53198 .36541 L
.53314 .36491 L
.53442 .36422 L
.539 .361 L
.54031 .35997 L
.54095 .35947 L
.54155 .359 L
.54188 .35882 L
.54224 .35869 L
.54299 .3584 L
.5443 .35793 L
.54926 .35642 L
.55189 .35581 L
.55431 .35538 L
.55492 .35529 L
.55558 .3552 L
.55595 .35516 L
.5563 .35514 L
.55664 .35513 L
.55697 .35513 L
.5573 .35513 L
.55766 .35513 L
.55831 .35514 L
.55867 .35514 L
.559 .35515 L
Mistroke
.55977 .35518 L
.56038 .35521 L
.56104 .35525 L
.56225 .35534 L
.56453 .35555 L
.56965 .35619 L
.58009 .35744 L
.58501 .35795 L
.58955 .35831 L
.59428 .3586 L
.59941 .35882 L
.6087 .35914 L
.62803 .35961 L
.64662 .36016 L
.66641 .36079 L
.68545 .3614 L
.7057 .36202 L
.7252 .36262 L
.74396 .3632 L
.76393 .36384 L
.78315 .36446 L
.80162 .36506 L
.8213 .36571 L
.84024 .36635 L
.86038 .36702 L
.87977 .36767 L
.89843 .36829 L
.91828 .36895 L
.93739 .36957 L
.95771 .37022 L
.97619 .37079 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{222.875, 137.375},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004F0000Za000`40O003h00O?Woo00<007ooOol0
eGoo000nOol2003FOol003ioo`03001oogoo0=Eoo`00?Woo00<007ooOol0eGoo000nOol00`00Oomo
o`12Ool00`00Oomoo`2@Ool003ioo`03001oogoo049oo`03001oogoo091oo`00?Woo00<007ooOol0
@Woo00<007ooOol0T7oo000nOol20013Ool00`00Oomoo`2@Ool003ioo`03001oogoo049oo`03001o
ogoo091oo`00?Woo00<007ooOol0@Woo00<007ooOol0T7oo000nOol00`00Oomoo`12Ool00`00Oomo
o`2@Ool001Ioo`<000Ioo`8000Aoo`<000Eoo`<000Eoo`<000Ioo`03001oogoo049oo`03001oogoo
091oo`005Goo00D007ooOomoo`0000Eoo`8000=oo`05001oogooOol00003Ool01@00Oomoogoo0000
0goo00D007ooOomoo`0000Eoo`03001oogoo049oo`03001oogoo091oo`003Goo1P000Woo00D007oo
Oomoo`0000Yoo`05001oogooOol00003Ool01@00Oomoogoo00001goo00<007ooOol00goo0`00@Woo
00<007ooOol0T7oo000EOol01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05001oogooOol0
0007Ool00`00Oomoo`03Ool00`00Oomoo`12Ool00`00Oomoo`2@Ool001Eoo`05001oogooOol0000:
Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Eoo`8000Ioo`03001oogoo049oo`03001oogoo
091oo`005Goo00D007ooOomoo`0000Yoo`05001oogooOol00003Ool01@00Oomoogoo00001goo00<0
07ooOol00goo00<007ooOol0@Goo0P00TWoo000EOol01@00Oomoogoo00002Woo00D007ooOomoo`00
00=oo`05001oogooOol00003Ool01@00Oomoogoo00001Goo00<007ooOol0@Goo0P00TWoo000FOol3
000<Ool30005Ool30005Ool30006Ool00`00Oomoo`11Ool00`00Ool0002AOol003ioo`80049oo`03
001oo`00095oo`00?Woo00<007ooOol0@Goo00<007oo0000TGoo000nOol00`00Oomoo`11Ool00`00
Ool0002AOol003ioo`03001oogoo045oo`03001oo`00095oo`00?Woo00<007ooOol0@Goo00<007oo
0000TGoo000nOol00`00Oomoo`11Ool00`00Ool0002AOol003ioo`80049oo`03001oo`00095oo`00
?Woo00<007ooOol0@Goo00<007oo0000TGoo000nOol00`00Oomoo`11Ool00`00Ool0002AOol003io
o`03001oogoo045oo`03001oo`00095oo`00?Woo00<007ooOol0@Goo00<007oo0000TGoo000nOol0
0`00Oomoo`11Ool00`00Ool0002AOol003ioo`03001oogoo045oo`03001oo`00095oo`00?Woo0P00
@Woo00<007oo0000TGoo000nOol00`00Oomoo`11Ool00`00Ool0002AOol003ioo`03001oogoo045o
o`03001oo`00095oo`00?Woo00<007ooOol0@Goo00<007oo0000TGoo000nOol00`00Oomoo`11Ool0
0`00Ool0002AOol003ioo`03001oogoo045oo`03001oo`00095oo`00?Woo0P00@Woo00<007oo0000
TGoo000nOol00`00Oomoo`11Ool00`00Ool0002AOol003ioo`03001oogoo045oo`03001oo`00095o
o`005Woo0`001Woo0P0017oo0`001Goo0`0017oo1@001Goo00<007ooOol0@Goo00<007oo0000TGoo
000EOol01@00Oomoogoo00001Goo0P000goo00D007ooOomoo`0000=oo`05001oogooOol00004Ool0
1000Oomoo`001Goo00<007ooOol0@Goo00<007oo0000TGoo000=Ool60002Ool01@00Oomoogoo0000
2Woo00D007ooOomoo`0000=oo`05001oogooOol00005Ool00`00Oomoo`05Ool00`00Oomoo`11Ool0
0`00Ool0002AOol001Eoo`05001oogooOol0000:Ool01@00Oomoogoo00000goo00D007ooOomoo`00
00Ioo`03001oogoo00Aoo`<0045oo`03001oo`00095oo`005Goo00D007ooOomoo`0000Yoo`05001o
ogooOol00003Ool01@00Oomoogoo00001goo00<007ooOol00goo00<007ooOol0@Goo00<007oo0000
TGoo000EOol01@00Oomoogoo00002Woo00D007ooOomoo`0000=oo`05001oogooOol00007Ool00`00
Oomoo`03Ool00`00Oomoo`11Ool00`00Ool0002AOol001Eoo`05001oogooOol0000:Ool01@00Oomo
ogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00005Ool00`00Oomoo`11Ool00`00Ool0
002AOol001Ioo`<000aoo`<000Eoo`<000Eoo`<000Ioo`03001oogoo045oo`03001oo`00095oo`00
?Woo00<007ooOol0@Goo00<007oo0000TGoo000nOol20012Ool00`00Ool0002AOol003ioo`03001o
ogoo045oo`03001oo`00095oo`00?Woo00<007ooOol0@Goo00<007oo0000TGoo000nOol00`00Oomo
o`11Ool00`00Ool0002AOol003ioo`03001oogoo045oo`03001oo`00095oo`00?Woo00<007ooOol0
@Goo00<007oo0000TGoo000nOol20012Ool00`00Ool0002AOol003ioo`03001oogoo045oo`03001o
o`00095oo`00?Woo00<007ooOol0@Goo00@007ooOol0091oo`00?Woo00<007ooOol0@Goo00@007oo
Ool0091oo`00?Woo00<007ooOol0@Goo00@007ooOol0091oo`00?Woo00<007ooOol0@Goo00@007oo
Ool0091oo`00?Woo0P00@Woo00@007ooOol0091oo`00?Woo00<007ooOol0@Goo00@007ooOol0091o
o`00?Woo00<007ooOol0@Goo00@007ooOol0091oo`00?Woo00<007ooOol0@Goo00@007ooOol0091o
o`00?Woo00<007ooOol0@Goo00@007ooOol0091oo`00?Woo00<007ooOol0@Goo00@007ooOol0091o
o`00?Woo0P00@Woo00@007ooOol0091oo`00?Woo00<007ooOol0@Goo00@007ooOol0091oo`00?Woo
00<007ooOol0@Goo00@007ooOol0091oo`005Woo0`001Woo0P0017oo0`001Goo0`001Goo1@0017oo
00<007ooOol0>Woo0P0017oo00D007ooOomoo`00091oo`005Goo00D007ooOomoo`0000Eoo`8000=o
o`05001oogooOol00003Ool01@00Oomoogoo00001Woo00<007ooOol017oo00<007ooOol0>Woo0P00
17oo00D007ooOomoo`00091oo`003Goo1P000Woo00D007ooOomoo`0000Yoo`05001oogooOol00003
Ool01@00Oomoogoo00001Woo00<007ooOol017oo00<007ooOol0>Woo0P0017oo00D007ooOomoo`00
091oo`005Goo00D007ooOomoo`0000Yoo`05001oogooOol00003Ool01@00Oomoogoo00001Woo00<0
07ooOol017oo0`00>Woo0P0017oo00D007ooOomoo`00091oo`005Goo00D007ooOomoo`0000Yoo`05
001oogooOol00003Ool01@00Oomoogoo00001Woo00<007ooOol017oo00<007ooOol0>Woo0P0017oo
00D007ooOomoo`0000Aoo`8008Yoo`005Goo00D007ooOomoo`0000Yoo`05001oogooOol00003Ool0
1@00Oomoogoo00001Woo00<007ooOol017oo00<007ooOol0>Woo0P0017oo00D007ooOomoo`0000Ao
o`8008Yoo`005Goo00D007ooOomoo`0000Yoo`05001oogooOol00003Ool01@00Oomoogoo00001Woo
00<007ooOol017oo00<007ooOol0>Woo0P0017oo00D007ooOomoo`0000Aoo`8008Yoo`005Woo0`00
37oo0`001Goo0`001Goo0`001Woo00<007ooOol0>Woo0P0017oo00D007ooOomoo`0000Aoo`8008Yo
o`00?Woo00<007ooOol0>Goo00<007oo000017oo00D007ooOomoo`0000Aoo`03001oo`0008Uoo`00
?Woo0P00>Woo00<007oo000017oo00D007ooOomoo`0000Aoo`03001oo`0008Uoo`00?Woo00<007oo
Ool0>Goo00@007ooOol000=oo`05001oogooOol00004Ool00`00Ool00029Ool003ioo`03001oogoo
03Uoo`04001oogoo0003Ool01@00Oomoogoo000017oo00<007oo0000RGoo000nOol00`00Oomoo`0i
Ool01000Oomoo`000goo00D007ooOomoo`0000Aoo`03001oo`0008Uoo`00?Woo00<007ooOol0>Goo
00@007ooOol000=oo`05001oogooOol00004Ool01000Oomoo`00R7oo000nOol00`00Oomoo`0hOol0
1@00Oomoogoo00000goo00D007ooOomoo`0000Aoo`04001oogoo0028Ool003ioo`8003Uoo`05001o
ogooOol00003Ool01@00Oomoogoo000017oo00@007ooOol008Qoo`00?Woo00<007ooOol0>7oo00D0
07ooOomoo`0000=oo`05001oogooOol00004Ool01000Oomoo`00R7oo000nOol00`00Oomoo`0hOol0
1@00Oomoogoo00000goo00D007ooOomoo`0000Aoo`04001oogoo0028Ool003ioo`03001oogoo03Qo
o`05001oogooOol00003Ool01@00Oomoogoo000017oo00@007ooOol008Qoo`00?Woo00<007ooOol0
>7oo00D007ooOomoo`0000=oo`05001oogooOol00004Ool01000Oomoo`00R7oo000nOol00`00Oomo
o`0hOol01@00Oomoogoo00000goo00D007ooOomoo`0000Aoo`04001oogoo0028Ool0015oo`D000Eo
o`8000Eoo`@001Qoo`8000mooa<000Eoo`@000ioo`04001oogooOol50003Ool300000goo001oo`02
Ool00`00Oomoo`08Ool50002Ool50005Ool20004Ool4000?Ool50003Ool4000?Ool50004Ool00`00
Oomoo`05Ool20004Ool4000>Ool50004Ool30004Ool0019oo`04001oogoo0005Ool20009Ool00`00
Oomoo`0EOol00`00Oomoo`08Ool60009Ool00`00Oomoo`05Ool:00000goo000000030009Ool02@00
Oomoogoo001oo`00Ool00002Ool20002Ool20004Ool00`00Oomoo`09Ool00`00Oomoo`03Ool01000
Oomoo`001Goo0P0027oo00<007ooOol03Woo00<007ooOol01goo00<007ooOol03Woo00<007ooOol0
17oo00<007ooOol01Goo0P0027oo00<007ooOol037oo00@007ooOol000=oo`05001oogooOol00003
Ool001=oo`03001oogoo011oo`03001oogoo01Eoo`03001oogoo00Aoo`@0011oo`03001oogoo00eo
o`@0009oo`<000Ioo`03001oogoo009oo`05001oo`00Ool00002Ool20002Ool20004Ool00`00Oomo
o`09Ool00`00Oomoo`04Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomoo`07Ool00`00Oomo
o`0>Ool00`00Oomoo`05Ool00`00Oomoo`0>Ool00`00Oomoo`0=Ool00`00Oomoo`03Ool01@00Oomo
ogoo00000goo000DOol00`00Oomoo`0?Ool00`00Oomoo`0EOol00`00Oomoo`04000DOol00`00Oomo
o`0?Ool00`00Oomoo`04Ool20004Ool00`00Oomoo`02Ool01@00Ool007oo00000Woo0P000Woo0P00
17oo00<007ooOol02Goo00<007ooOol01Goo00<007ooOol03Woo00<007ooOol03Woo00<007ooOol0
1goo00<007ooOol03Woo00<007ooOol01Goo00<007ooOol03Woo00<007ooOol03Woo00<007ooOol0
0Woo00D007ooOomoo`0000=oo`005Goo00<007ooOol02Woo100067oo0`0067oo00<007ooOol02goo
100027oo00<007ooOol00Woo00<007ooOol00Woo00D007oo001oo`00009oo`80009oo`8000Aoo`03
001oogoo00Uoo`03001oogoo00Ioo`03001oogoo00Uoo`@0015oo`03001oogoo00=oo`@0015oo`03
001oogoo00Eoo`03001oogoo00Yoo`@0019oo`05001oogooOol00003Ool00`00Oomoo`01Ool001Eo
o`03001oogoo00Yoo`03001oogoo01Moo`<001]oo`03001oogoo00Yoo`03001oogoo00Yoo`04001o
ogoo0005Ool01@00Ool007oo00000Woo0P000Woo0P001Goo00<007ooOol01goo10001Woo00<007oo
Ool02Goo00<007ooOol04Woo00<007ooOol00goo00<007ooOol04Woo00<007ooOol01Woo00<007oo
Ool02Goo00<007ooOol04goo00D007ooOomoo`0000=oo`03001oogoo005oo`004Goo00D007ooOomo
o`0000aoo`03001oogoo01Aoo`<0009oo`8001Ioo`05001oogooOol0000<Ool00`00Oomoo`0:Ool4
0005Ool01@00Ool007oo00000Woo0P000Woo0P001Goo00<007ooOol00goo100000Aoo`00Oomoo`l0
00=oo`03001oogoo019oo`03001oogoo00=oo`03001oogoo019oo`03001oogoo009oo`05001oogoo
Ool0000;Ool00`00Oomoo`0?Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`004Woo0`00
3Goo10004Goo0P001Goo00<007ooOol05Goo1@0037oo10002Woo0P001Woo0`0000=oo`00Ool00Woo
0`0000=oo`00Ool01Goo00@007ooOomoo`80009oo`<000Eoo`<000Uoob8000moo`<000Aoo`D000]o
o`@000moo`<000Eoo`<000Aoo`00=7oo0`001goo00<007ooOol0?7oo00D007ooOomoo`0000=oo`05
001oogooOol00007Ool3000hOolO000[Ool0035oo`<000Yoo`03001oogoo03aoo`05001oogooOol0
0003Ool01@00Oomoogoo0000HGoo8@002Woo000^Ool3000=Ool00`00Oomoo`0lOol01@00Oomoogoo
00000goo00D007ooOomoo`00089oo`8000Qoo`00;7oo0P0047oo00<007ooOol0?7oo00D007ooOomo
o`0000=oo`05001oogooOol0002<Ool00?l001D0009oo`001goo00<007ooOol017oo00<007ooOol0
17oo5@001Woo00<007ooOol017oo00<007ooOol017oo00<007ooOol017oo00<007ooOol017oo00<0
07ooOol017oo00<007ooOol017oo00<007ooOol017oo00<007ooOol00goo00<007ooOol017oo00<0
07ooOol017oo00<007ooOol017oo0P000goo00D007oo001oo`0000=oo`8000Ioo`03001oogoo00Ao
o`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Ao
o`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00=o
o`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Ao
o`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Ioo`0077oo00<007ooOol07goo00<007oo
Ool087oo00<007ooOol06Goo00T007ooOomoo`00Ool007oo00000goo00<007ooOol06Goo00<007oo
Ool087oo00<007ooOol07goo00<007ooOol087oo00<007ooOol01Woo000nOol00`00Oomoo`0lOol0
1@00Oomoogoo00000goo00D007ooOomoo`0008aoo`00?Woo00<007ooOol0?Goo00@007ooOol000=o
o`05001oogooOol0002<Ool003ioo`03001oogoo03eoo`04001oogoo0003Ool01@00Oomoogoo0000
S7oo000nOol2000nOol01000Oomoo`000goo00D007ooOomoo`0008aoo`00?Woo00<007ooOol0?Goo
00@007ooOol000=oo`05001oogooOol0002<Ool003ioo`03001oogoo03eoo`04001oogoo0003Ool0
1@00Oomoogoo0000S7oo000nOol00`00Oomoo`0mOol01000Oomoo`000goo00D007ooOomoo`0008ao
o`00?Woo00<007ooOol0?Goo00@007ooOol000=oo`05001oogooOol0002<Ool003ioo`03001oogoo
03eoo`04001oogoo0003Ool01@00Oomoogoo0000S7oo000nOol2000nOol01000Oomoo`000goo00D0
07ooOomoo`0008aoo`00?Woo00<007ooOol0?Goo00@007ooOol000=oo`05001oogooOol0002<Ool0
03ioo`03001oogoo03eoo`04001oogoo0003Ool01@00Oomoogoo0000S7oo000nOol00`00Oomoo`0m
Ool01000Oomoo`000goo00D007ooOomoo`0008aoo`00?Woo00<007ooOol0?Goo00@007ooOol000=o
o`05001oogooOol0002<Ool003ioo`03001oogoo03eoo`04001oogoo0003Ool01@00Oomoogoo0000
S7oo000nOol2000nOol01000Oomoo`000goo00D007ooOomoo`0008aoo`00?Woo00<007ooOol0?Goo
00@007ooOol000=oo`05001oogooOol0002<Ool003ioo`03001oogoo03eoo`04001oogoo0003Ool0
1@00Oomoogoo0000S7oo000nOol00`00Oomoo`0mOol00`00Ool00004Ool01000Oomoo`00SGoo000n
Ool00`00Oomoo`0mOol00`00Ool00004Ool01000Oomoo`00SGoo000nOol00`00Oomoo`0mOol00`00
Ool00004Ool01000Oomoo`00SGoo000nOol2000nOol00`00Ool00004Ool01000Oomoo`00SGoo000n
Ool00`00Oomoo`0mOol00`00Ool00004Ool01000Oomoo`00SGoo000nOol00`00Oomoo`0mOol00`00
Ool00004Ool01000Oomoo`00SGoo000nOol00`00Oomoo`0mOol00`00Ool00005Ool00`00Ool0002=
Ool001Ioo`<000Ioo`8000Eoo`<000Aoo`<000Eoo`D000Aoo`03001oogoo03eoo`03001oo`0000Eo
o`03001oo`0008eoo`005Goo00D007ooOomoo`0000Eoo`8000Aoo`05001oogooOol00002Ool01@00
Oomoogoo00001Woo00<007ooOol017oo00<007ooOol0?Goo00<007oo00001Goo00<007oo0000SGoo
000EOol01@00Oomoogoo00002goo00D007ooOomoo`00009oo`05001oogooOol00006Ool00`00Oomo
o`04Ool00`00Oomoo`0mOol00`00Ool00005Ool00`00Ool0002=Ool001Eoo`05001oogooOol0000;
Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000Ioo`03001oogoo00Aoo`<003eoo`03001oo`00
00Eoo`03001oo`0008eoo`005Goo00D007ooOomoo`0000]oo`05001oogooOol00002Ool01@00Oomo
ogoo00001Woo00<007ooOol017oo00<007ooOol0?Goo00<007oo00001Goo00<007oo0000SGoo000E
Ool01@00Oomoogoo00002goo00D007ooOomoo`00009oo`05001oogooOol00006Ool00`00Oomoo`04
Ool00`00Oomoo`0mOol00`00Ool00005Ool00`00Ool0002=Ool001Eoo`05001oogooOol0000;Ool0
1@00Oomoogoo00000Woo00D007ooOomoo`0000Ioo`03001oogoo00Aoo`03001oogoo03eoo`03001o
o`0000Eoo`03001oo`0008eoo`005Woo0`003Goo0`0017oo0`001Goo0`001Woo00<007ooOol0?Goo
00<007oo00001Goo00<007oo0000SGoo000nOol00`00Oomoo`0mOol00`00Ool00005Ool00`00Ool0
002=Ool003ioo`8003ioo`03001oo`0000Eoo`03001oo`0008eoo`00?Woo00<007ooOol0?Goo00<0
07oo00001Goo0P00SWoo000nOol00`00Oomoo`0mOol00`00Ool00005Ool2002>Ool003ioo`03001o
ogoo03eoo`03001oo`0000Eoo`8008ioo`00?Woo00<007ooOol0?Goo00<007oo00001Goo0P00SWoo
000nOol00`00Oomoo`0mOol00`00Ool00005Ool2002>Ool003ioo`8003ioo`03001oo`0000Eoo`80
08ioo`00?Woo00<007ooOol0?Goo00<007oo00001Goo0P00SWoo000nOol00`00Oomoo`0nOol20005
Ool2002>Ool003ioo`03001oogoo03ioo`8000Eoo`8008ioo`00?Woo00<007ooOol0?Woo0P001Goo
0P00SWoo000nOol00`00Oomoo`0nOol20005Ool2002>Ool003ioo`8003moo`8000Eoo`8008ioo`00
?Woo00<007ooOol0?Woo0P001Goo0P00SWoo000nOol00`00Oomoo`0nOol20005Ool2002>Ool003io
o`03001oogoo03ioo`8000Eoo`8008ioo`00?Woo00<007ooOol0?Woo0P001Goo0P00SWoo000nOol0
0`00Oomoo`0nOol20005Ool2002>Ool003ioo`8003moo`8000Eoo`8008ioo`00?Woo00<007ooOol0
?Woo0P001Goo0P00SWoo000nOol00`00Oomoo`0nOol20005Ool2002>Ool001Ioo`<000Ioo`8000Eo
o`<000Aoo`<000Aoo`D000Eoo`03001oogoo03ioo`8000Eoo`8008ioo`005Goo00D007ooOomoo`00
00Eoo`8000Aoo`05001oogooOol00002Ool01@00Oomoogoo000017oo00@007ooOol000Eoo`03001o
ogoo03ioo`8000Eoo`03001oogoo08eoo`005Goo00D007ooOomoo`0000]oo`05001oogooOol00002
Ool01@00Oomoogoo00001Goo00<007ooOol01Goo00<007ooOol0?Woo0P00UGoo000EOol01@00Oomo
ogoo00002goo00D007ooOomoo`00009oo`05001oogooOol00006Ool00`00Oomoo`04Ool3000nOol2
002EOol001Eoo`05001oogooOol0000;Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000Moo`03
001oogoo00=oo`03001oogoo03ioo`03001oogoo09Aoo`005Goo00D007ooOomoo`0000]oo`05001o
ogooOol00002Ool01@00Oomoogoo00001goo00<007ooOol00goo00<007ooOol0eGoo000EOol01@00
Oomoogoo00002goo00D007ooOomoo`00009oo`05001oogooOol00003Ool01@00Oomoogoo00001Goo
00<007ooOol0eGoo000FOol3000=Ool30004Ool30005Ool30006Ool00`00Oomoo`3EOol003ioo`03
001oogoo0=Eoo`00\
\>"],
  ImageRangeCache->{{{0, 277.562}, {170.688, 0}} -> {0.454046, -0.00344555, 
  0.090798, 4.12388}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["15.7 Douglas Three-Time-Level Solution", "Section",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
We have to work a little harder to develop a scheme that eliminates \
the oscillation problem. What we do is to implement the three-time-level \
extension of the Douglas scheme discussed in Section 15.2. We shall not go \
into a detailed theoretical discussion of this algorithm - we shall content \
ourselves with an explicit demonstration that it works.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Necessary Functions", "Subsubsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
DougCMatrix[alpha_, nminus_, nplus_] :=
Sequence[Table[1-6*alpha, {nplus+nminus-2}], Table[10+12*alpha, \
{nplus+nminus-1}],
Table[1-6*alpha, {nplus+nminus-2}]]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
DougCCMatrix[alpha_, nminus_, nplus_] :=
Sequence[Table[1/8-alpha, {nplus+nminus-2}], 
Table[5/4+2*alpha, {nplus+nminus-1}],
Table[1/8-alpha, {nplus+nminus-2}]]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
DougDMatrix[alpha_, vec_List] := Module[{temp},
temp = (10 - 12*alpha)*vec + (1+6*alpha)*(RotateRight[vec] + \
RotateLeft[vec]);
temp[[1]] = Simplify[First[temp] - \.0c(1+6*alpha)*Last[vec]];
temp[[-1]] = Simplify[Last[temp] - (1 + 6*alpha)*First[vec]];
temp]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
DougDDMatrix[vec_List] := Module[{temp},
temp = (10*vec + RotateRight[vec] + RotateLeft[vec]);
temp[[1]] = Simplify[First[temp] - Last[vec]];
temp[[-1]] = Simplify[Last[temp] - First[vec]];
temp/6]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[" Douglas Three Time-Level Solution Evolution", "Subsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
The initialization consists of defining the grid parameters, \
setting boundary and initial conditions and defining the various \
matrices.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
dx = 0.025;
dtau = 0.005;
alpha = dtau/dx^2;
M=20;
nminus = 160;
nplus = 160;
alpha\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(7.99999999999999911`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(initial = 
      Table[PutExercise[\((k - 1 - nminus)\)\ dx, 0.05, 0, 0.2], \ {k, 
          nminus + nplus + 1}]; \n
    lower = Table[
        f[\(-nminus\)\ dx, \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \n
    upper = Table[
        g[\(+nplus\)\ dx, \ \ \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \)], "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["w = Table[0, {m, 1, 3},   {k, 1, nminus+nplus+1}];", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["vold = Take[initial, {2, -2}];", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
w[[1,1]] = f[-nminus*dx, dtau/4, 0.05, 0, 0.2];
w[[1, nminus+nplus+1]] = g[nplus*dx, dtau/4, 0.05, 0, 0.2];
w[[2,1]] = f[-nminus*dx, dtau/2, 0.05, 0, 0.2];
w[[2, nminus+nplus+1]] = g[nplus*dx, dtau/2, 0.05, 0, 0.2];
w[[3,1]] = f[-nminus*dx, dtau, 0.05, 0, 0.2];
w[[3, nminus+nplus+1]] = g[nplus*dx, dtau, 0.05, 0, 0.2];\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
CMat = DougCMatrix[alpha,nminus,nplus];
CCMat = DougCCMatrix[alpha,nminus,nplus];

CMatQ = DougCMatrix[alpha/4,nminus,nplus];
CMatH = DougCMatrix[alpha/2,nminus,nplus];

CCMatQ = DougCCMatrix[alpha/4,nminus,nplus];
CCMatH = DougCCMatrix[alpha/2,nminus,nplus];\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Kick-Off Phase", "Subsubsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
This begins with the simple Douglas scheme with two time-levels and \
1/4 the basic time-step.\
\>", "Text",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
rhs = DougDMatrix[alpha/4, vold]+
Table[
Which[
k==1, (6*alpha/4+1)*lower[[1]] + (6*alpha/4-1)*w[[1, 1]],
k== nplus + nminus-1,  (6*alpha/4+1)*upper[[1]] +
(6*alpha/4-1)*w[[1, nplus+nminus+1]],
True, 0],
{k, 1, nplus + nminus-1}];
vnew = CompTridiagSolve[CMatQ, rhs];
w[[1]] = Join[{w[[1,1]]}, vnew, {w[[1,nplus+nminus+1]]}];
vvold = vold;
vold = vnew;\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Now we have two iterations of the three-time-level Douglas system, \
doubling the time-step at each stage:\
\>", "Text",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
rhs = DougDDMatrix[vold] - DougDDMatrix[vvold]/4 + 
Table[
Which[
k==1, (alpha/4-1/8)*w[[2, 1]] + w[[1,1]]/6 - lower[[1]]/24,
k==nplus + nminus-1,  (alpha/4-1/8)*w[[2, nplus+nminus+1]] + 
w[[1,nplus+nminus+1]]/6 - upper[[1]]/24,
True, 0],
{k, 1, nplus + nminus-1}];
vnew = CompTridiagSolve[CCMatQ, rhs];
w[[2]] = Join[{w[[2,1]]}, vnew, {w[[2,nplus+nminus+1]]}];
vold = vnew;\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
rhs = DougDDMatrix[vold] - DougDDMatrix[vvold]/4 + 
Table[
Which[
k==1, (alpha/2-1/8)*w[[3, 1]] + w[[2,1]]/6 - lower[[1]]/24,
k==nplus + nminus-1,  (alpha/2-1/8)*w[[3, nplus+nminus+1]] + 
w[[2,nplus+nminus+1]]/6 - upper[[1]]/24,
True, 0],
{k, 1, nplus + nminus-1}];
vnew = CompTridiagSolve[CCMatH, rhs];
w[[3]] = Join[{w[[3,1]]}, vnew, {w[[3,nplus+nminus+1]]}];
wold = initial;
wvold = initial;
wnew = w[[3]];\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Main Evolution Phase and Interpolation of Solution", "Subsubsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
For[m=3, m<=M+1, m++,
(wvold = wold;
wold = wnew;
rhs = DougDDMatrix[Take[wold, {2, -2}]] -
DougDDMatrix[Take[wvold, {2, -2}]]/4 + 
Table[
Which[
k==1, 
(alpha-1/8)*lower[[m]] + lower[[m-1]]/6 - lower[[m-2]]/24,
k==nplus + nminus-1, 
(alpha-1/8)*upper[[m]] + upper[[m-1]]/6 - upper[[m-2]]/24,
True, 0],
{k, 1, nplus + nminus-1}];
temp = CompTridiagSolve[CCMat, rhs];
wnew = Join[{lower[[m]]}, temp, {upper[[m]]}])
]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(points\  = \ 
      Table[\((k - nminus - 1)\)*dx, \ \ \n{k, \ 1, \ 
          nplus + nminus + 1}]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["finalstep = wnew;", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["prevstep = wold;", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["pprevstep = wvold;", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(interpoldata\  = \ Transpose[{points, finalstep}]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
ufunc = Interpolation[interpoldata, InterpolationOrder -> 3];\
\>", 
  "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Valuation[strike_, r_, q_, S_, T_, sd_] :=
ValuationMultiplier[strike, r, q, Log[S/strike], sd^2*T/2, sd]*
ufunc[Log[S/strike]]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Valuation Analysis", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Plot[Valuation[10, 0.05, 0, S, 5, 0.2]-
BlackScholesPut[S,10,0.2,0.05,0,5], {S, 1, 20}, 
PlotPoints -> 50, PlotRange -> All];\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0263158 0.0501253 0.47023 626.492 [
[.099 .45773 -9 -9 ]
[.099 .45773 9 0 ]
[.34962 .45773 -9 -9 ]
[.34962 .45773 9 0 ]
[.47494 .45773 -6 -9 ]
[.47494 .45773 6 0 ]
[.60025 .45773 -12 -9 ]
[.60025 .45773 12 0 ]
[.72556 .45773 -6 -9 ]
[.72556 .45773 6 0 ]
[.85088 .45773 -12 -9 ]
[.85088 .45773 12 0 ]
[.97619 .45773 -6 -9 ]
[.97619 .45773 6 0 ]
[.21181 .09433 -42 -4.5 ]
[.21181 .09433 0 4.5 ]
[.21181 .21963 -42 -4.5 ]
[.21181 .21963 0 4.5 ]
[.21181 .34493 -42 -4.5 ]
[.21181 .34493 0 4.5 ]
[.21181 .59553 -36 -4.5 ]
[.21181 .59553 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.099 .47023 m
.099 .47648 L
s
[(2.5)] .099 .45773 0 1 Mshowa
.34962 .47023 m
.34962 .47648 L
s
[(7.5)] .34962 .45773 0 1 Mshowa
.47494 .47023 m
.47494 .47648 L
s
[(10)] .47494 .45773 0 1 Mshowa
.60025 .47023 m
.60025 .47648 L
s
[(12.5)] .60025 .45773 0 1 Mshowa
.72556 .47023 m
.72556 .47648 L
s
[(15)] .72556 .45773 0 1 Mshowa
.85088 .47023 m
.85088 .47648 L
s
[(17.5)] .85088 .45773 0 1 Mshowa
.97619 .47023 m
.97619 .47648 L
s
[(20)] .97619 .45773 0 1 Mshowa
.125 Mabswid
.12406 .47023 m
.12406 .47398 L
s
.14912 .47023 m
.14912 .47398 L
s
.17419 .47023 m
.17419 .47398 L
s
.19925 .47023 m
.19925 .47398 L
s
.24937 .47023 m
.24937 .47398 L
s
.27444 .47023 m
.27444 .47398 L
s
.2995 .47023 m
.2995 .47398 L
s
.32456 .47023 m
.32456 .47398 L
s
.37469 .47023 m
.37469 .47398 L
s
.39975 .47023 m
.39975 .47398 L
s
.42481 .47023 m
.42481 .47398 L
s
.44987 .47023 m
.44987 .47398 L
s
.5 .47023 m
.5 .47398 L
s
.52506 .47023 m
.52506 .47398 L
s
.55013 .47023 m
.55013 .47398 L
s
.57519 .47023 m
.57519 .47398 L
s
.62531 .47023 m
.62531 .47398 L
s
.65038 .47023 m
.65038 .47398 L
s
.67544 .47023 m
.67544 .47398 L
s
.7005 .47023 m
.7005 .47398 L
s
.75063 .47023 m
.75063 .47398 L
s
.77569 .47023 m
.77569 .47398 L
s
.80075 .47023 m
.80075 .47398 L
s
.82581 .47023 m
.82581 .47398 L
s
.87594 .47023 m
.87594 .47398 L
s
.901 .47023 m
.901 .47398 L
s
.92607 .47023 m
.92607 .47398 L
s
.95113 .47023 m
.95113 .47398 L
s
.07393 .47023 m
.07393 .47398 L
s
.04887 .47023 m
.04887 .47398 L
s
.02381 .47023 m
.02381 .47398 L
s
.25 Mabswid
0 .47023 m
1 .47023 L
s
.22431 .09433 m
.23056 .09433 L
s
[(-0.0006)] .21181 .09433 1 0 Mshowa
.22431 .21963 m
.23056 .21963 L
s
[(-0.0004)] .21181 .21963 1 0 Mshowa
.22431 .34493 m
.23056 .34493 L
s
[(-0.0002)] .21181 .34493 1 0 Mshowa
.22431 .59553 m
.23056 .59553 L
s
[(0.0002)] .21181 .59553 1 0 Mshowa
.125 Mabswid
.22431 .12566 m
.22806 .12566 L
s
.22431 .15698 m
.22806 .15698 L
s
.22431 .18831 m
.22806 .18831 L
s
.22431 .25096 m
.22806 .25096 L
s
.22431 .28228 m
.22806 .28228 L
s
.22431 .31361 m
.22806 .31361 L
s
.22431 .37626 m
.22806 .37626 L
s
.22431 .40758 m
.22806 .40758 L
s
.22431 .43891 m
.22806 .43891 L
s
.22431 .50155 m
.22806 .50155 L
s
.22431 .53288 m
.22806 .53288 L
s
.22431 .5642 m
.22806 .5642 L
s
.22431 .06301 m
.22806 .06301 L
s
.22431 .03169 m
.22806 .03169 L
s
.22431 .00036 m
.22806 .00036 L
s
.25 Mabswid
.22431 0 m
.22431 .61803 L
s
.5 Mabswid
.02381 .46429 m
.02491 .46279 L
.02609 .4615 L
.02821 .45976 L
.0295 .459 L
.03073 .45842 L
.03183 .45804 L
.03302 .45774 L
.03419 .45753 L
.03451 .45748 L
.03485 .45741 L
.03547 .45737 L
.03617 .45733 L
.03651 .45729 L
.03683 .4573 L
.03742 .45734 L
.03774 .45734 L
.03807 .45733 L
.03868 .4574 L
.03933 .45748 L
.03995 .45753 L
.04052 .45767 L
.04273 .45821 L
.04393 .45862 L
.04521 .45911 L
.04752 .46031 L
.05277 .46412 L
.05562 .46698 L
.05831 .47012 L
.06337 .47742 L
.07312 .49632 L
.08214 .51835 L
.09206 .54506 L
.10138 .56918 L
.11027 .58804 L
.11271 .59217 L
.11528 .59586 L
.11764 .59862 L
.11987 .6007 L
.12104 .60154 L
.12232 .6023 L
.12288 .60257 L
.12347 .6028 L
.12403 .60298 L
.12454 .60311 L
.12513 .60322 L
.12577 .6033 L
.12636 .60332 L
.12692 .6033 L
.12723 .60326 L
Mistroke
.12757 .60321 L
.12817 .60309 L
.12889 .60288 L
.12954 .60263 L
.1307 .60205 L
.13176 .60135 L
.13417 .5992 L
.13629 .59666 L
.13853 .59328 L
.14319 .58408 L
.14817 .57101 L
.15716 .53946 L
.16758 .49177 L
.17744 .43817 L
.19624 .32406 L
.21624 .20532 L
.22618 .1532 L
.2355 .11115 L
.24452 .07699 L
.25401 .04919 L
.25875 .03835 L
.26392 .02903 L
.26862 .02254 L
.27089 .02019 L
.27299 .01827 L
.27417 .0174 L
.27547 .01663 L
.27678 .01604 L
.27816 .01553 L
.27937 .0151 L
.28005 .01493 L
.28067 .01482 L
.28125 .01475 L
.28188 .01472 L
.28248 .01472 L
.28303 .01476 L
.28355 .01482 L
.28411 .01492 L
.28464 .01504 L
.28513 .01517 L
.28623 .01544 L
.28744 .01581 L
.28863 .01631 L
.28973 .01689 L
.29214 .01857 L
.29728 .02328 L
.30215 .02952 L
.31125 .0448 L
.33007 .08913 L
.3501 .14858 L
Mistroke
.36938 .21079 L
.38792 .26993 L
.40766 .32798 L
.42666 .37637 L
.44687 .4183 L
.45696 .43532 L
.46632 .44876 L
.47596 .46026 L
.48504 .46915 L
.4951 .47682 L
.50422 .48183 L
.509 .48382 L
.51406 .48532 L
.51614 .48591 L
.51837 .48643 L
.51966 .48668 L
.52082 .48688 L
.52311 .48717 L
.52423 .48727 L
.52547 .48735 L
.52609 .48738 L
.52676 .4874 L
.52797 .48744 L
.52901 .4875 L
.52959 .48752 L
.53012 .48754 L
.53074 .48755 L
.53133 .48756 L
.53187 .48756 L
.53246 .48755 L
.53277 .48754 L
.53312 .48753 L
.53381 .4875 L
.53445 .48747 L
.53505 .48743 L
.53636 .48732 L
.53777 .48716 L
.54263 .4864 L
.5476 .48548 L
.55041 .48476 L
.55301 .48398 L
.56286 .48057 L
.58206 .47127 L
.60246 .45876 L
.62212 .44534 L
.64104 .43184 L
.66116 .41759 L
.68054 .40452 L
.69917 .39296 L
.719 .38198 L
Mistroke
.7381 .3729 L
.75839 .36492 L
.77794 .35889 L
.79675 .3546 L
.80646 .35293 L
.81676 .35155 L
.82137 .35105 L
.82633 .35062 L
.83097 .35029 L
.83529 .35004 L
.84023 .34983 L
.84158 .34978 L
.84285 .34975 L
.84431 .34972 L
.84563 .34969 L
.84683 .34968 L
.84748 .34967 L
.84809 .34967 L
.84862 .34966 L
.84917 .34966 L
.84979 .34966 L
.85036 .34966 L
.85068 .34966 L
.85101 .34966 L
.85162 .34966 L
.85195 .34966 L
.85227 .34966 L
.85298 .34967 L
.85361 .34967 L
.85418 .34968 L
.85547 .34969 L
.85779 .34973 L
.86027 .34979 L
.86474 .34994 L
.86747 .35007 L
.87005 .3502 L
.87487 .35051 L
.88467 .3513 L
.89371 .35224 L
.913 .35484 L
.93155 .35802 L
.95131 .36199 L
.97032 .36627 L
.97619 .36768 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{222.938, 137.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003N0000RA000`40O003h00O<Goo0P00Zgoo000a
Ool00`00Oomoo`2ZOol0035oo`03001oogoo0:Yoo`00<Goo00<007ooOol02Goo0`00WWoo000aOol0
0`00Oomoo`08Ool200000goo0000002MOol0035oo`03001oogoo00Moo`8000Aoo`03001oogoo09Yo
o`00<Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol0VWoo000aOol20007Ool00`00Oomo
o`05Ool00`00Oomoo`2IOol0035oo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo09Uoo`00
<Goo00<007ooOol01Goo00<007ooOol01goo00<007ooOol0V7oo000aOol00`00Oomoo`05Ool00`00
Oomoo`08Ool00`00Oomoo`2GOol0035oo`03001oogoo00Aoo`03001oogoo00Uoo`03001oogoo09Mo
o`00<Goo00<007ooOol017oo00<007ooOol02Woo00<007ooOol0UWoo000aOol00`00Oomoo`03Ool0
0`00Oomoo`0;Ool00`00Oomoo`2FOol0035oo`8000Aoo`03001oogoo00aoo`03001oogoo09Eoo`00
<Goo00<007ooOol00goo00<007ooOol037oo00<007ooOol0UGoo000aOol00`00Oomoo`02Ool00`00
Oomoo`0>Ool00`00Oomoo`2DOol0035oo`03001oogoo009oo`03001oogoo00ioo`03001oogoo09Ao
o`00<Goo00<007ooOol00Woo00<007ooOol03goo00<007ooOol0Tgoo000<Ool20004Ool20004Ool2
0004Ool20004Ool20004Ool20005Ool00`00Oomoo`02Ool00`00Oomoo`0?Ool00`00Oomoo`2COol0
00]oo`04001oogoo0003Ool20003Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0002
Ool01000Oomoo`0017oo00D007ooOomoo`0001=oo`03001oogoo099oo`001Goo1@0000Eoo`00Oomo
o`0000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0004Ool3
00000goo001oo`0BOol00`00Oomoo`2BOol000]oo`04001oogoo0008Ool01000Oomoo`000Woo00@0
07ooOol0009oo`04001oogoo0002Ool30005Ool01@00Oomoogoo00004goo00<007ooOol0TWoo000;
Ool01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`000Woo00<007oo
Ool01Goo00D007ooOomoo`0001Aoo`03001oogoo095oo`0037oo0P002Woo0P0017oo0P0017oo0P00
17oo0`0017oo00@007ooOol001Eoo`03001oogoo095oo`00<Goo00@007ooOol001Eoo`03001oogoo
095oo`00<Goo00@007ooOol001Ioo`03001oogoo091oo`00<Goo00@007ooOol001Ioo`03001oogoo
091oo`00<Goo0`005goo00<007ooOol0T7oo000aOol00`00Ool0000HOol00`00Oomoo`2?Ool0035o
o`03001oo`0001Qoo`03001oogoo08moo`00<Goo00<007oo000067oo00<007ooOol0Sgoo000aOol0
0`00Ool0000IOol00`00Oomoo`2>Ool0035oo`8001Yoo`03001oogoo08ioo`00<Goo0P006Woo00<0
07ooOol0SWoo000aOol2000JOol00`00Oomoo`2>Ool0035oo`8001]oo`03001oogoo08eoo`00<Goo
00<007ooOol06Woo00<007ooOol0SGoo000aOol00`00Oomoo`0JOol00`00Oomoo`2=Ool0035oo`03
001oogoo01]oo`03001oogoo08aoo`00<7oo0P007Goo00<007ooOol0S7oo000`Ool2000MOol00`00
Oomoo`2<Ool0031oo`<001aoo`03001oogoo08aoo`00<7oo0P007Woo00<007ooOol0Rgoo000_Ool0
0`00Ool0000NOol00`00Oomoo`2;Ool002moo`03001oo`0001ioo`03001oogoo08]oo`00;goo00<0
07oo00007goo00<007ooOol0RWoo000<Ool20004Ool20004Ool20004Ool20004Ool20004Ool30002
Ool00`00Ool0000OOol00`00Oomoo`2:Ool000]oo`04001oogoo0003Ool20003Ool01000Oomoo`00
0Woo00@007ooOol0009oo`04001oogoo0004Ool01`00Oomoogoo001oo`0001moo`03001oogoo08Yo
o`001Goo1@0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool01000Oomoo`000Woo00@007ooOol0
009oo`@00004Ool007ooOol3000MOol00`00Oomoo`2:Ool000]oo`04001oogoo0008Ool01000Oomo
o`000Woo00@007ooOol0009oo`04001oogoo0002Ool00`00Ool00002Ool01000Oomoo`0087oo00<0
07ooOol0RGoo000;Ool01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0002Ool01000Oomo
o`000Woo00<007oo00000Woo00@007ooOol0021oo`03001oogoo08Uoo`0037oo0P002Woo0P0017oo
0P0017oo0P0017oo0P000Woo00@007ooOol0021oo`03001oogoo08Uoo`00;Woo00@007ooOol0025o
o`03001oogoo08Qoo`00;Woo00@007ooOol0025oo`03001oogoo08Qoo`00;Woo00<007ooOol00P00
87oo00<007ooOol0R7oo000]Ool01@00Oomoogoo00008Woo00<007ooOol0Qgoo000]Ool01@00Oomo
ogoo00008Woo00<007ooOol0Qgoo000]Ool01@00Oomoogoo00008Woo00<007ooOol0Qgoo000]Ool0
1@00Oomoogoo00008goo00<007ooOol0QWoo000]Ool01@00Oomoogoo00008goo00<007ooOol0QWoo
000]Ool01@00Oomoogoo00008goo00<007ooOol0QWoo000]Ool01000Oomoogoo0P008goo00<007oo
Ool0QGoo000/Ool00`00Oomoo`02Ool00`00Oomoo`0ROol00`00Oomoo`25Ool002aoo`03001oogoo
009oo`03001oogoo029oo`03001oogoo08Eoo`00;7oo00<007ooOol00Woo00<007ooOol08goo00<0
07ooOol0Q7oo000/Ool00`00Oomoo`02Ool00`00Oomoo`0SOol00`00Oomoo`24Ool002aoo`03001o
ogoo009oo`03001oogoo02Aoo`03001oogoo08=oo`00;7oo00<007ooOol00Woo00<007ooOol097oo
00<007ooOol0Pgoo000/Ool00`00Oomoo`02Ool2000VOol00`00Oomoo`22Ool002]oo`03001oogoo
00=oo`03001oogoo02Eoo`03001oogoo089oo`00:goo00<007ooOol00goo00<007ooOol09Goo00<0
07ooOol0PWoo000[Ool00`00Oomoo`03Ool00`00Oomoo`0VOol00`00Oomoo`21Ool002]oo`03001o
ogoo00=oo`03001oogoo02Ioo`03001oogoo085oo`0037oo0P0017oo0P0017oo0P0017oo0P0017oo
0P000goo100017oo00<007ooOol09goo00<007ooOol0P7oo000;Ool01000Oomoo`000goo0P000goo
00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`000goo0P001Goo00<007ooOol09goo00<0
07ooOol0P7oo0005Ool500001Goo001oogoo000027oo00@007ooOol0009oo`04001oogoo0002Ool0
1000Oomoo`000goo0P001Goo0`009goo00<007ooOol0P7oo000;Ool01000Oomoo`0027oo00@007oo
Ool0009oo`04001oogoo0002Ool01000Oomoo`000goo00<007oo000017oo00<007ooOol0:7oo00<0
07ooOol0Egoo3@006goo000;Ool01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0002Ool0
1000Oomoo`000Woo0P0000=oo`00Ool00goo00<007ooOol0:7oo00<007ooOol0Cgoo20003Goo2@00
4Woo000<Ool2000:Ool20004Ool20004Ool20004Ool20005Ool00`00Oomoo`0XOol00`00Oomoo`1;
Ool4000NOol5000=Ool002Yoo`03001oogoo00Aoo`03001oogoo02Uoo`03001oogoo04Qoo`8002Mo
o`@000Uoo`00:Woo00<007ooOol017oo00<007ooOol0:Goo00<007ooOol0AGoo0`00;Goo0`001Woo
000YOol00`00Oomoo`05Ool00`00Oomoo`0YOol00`00Oomoo`12Ool3000iOol002Uoo`03001oogoo
00Eoo`8002]oo`03001oogoo03ioo`<003aoo`00:Goo00<007ooOol01Goo00<007ooOol0:Woo00<0
07ooOol0?7oo0P00?goo000YOol00`00Oomoo`05Ool00`00Oomoo`0[Ool00`00Oomoo`0jOol00`00
Oomoo`0oOol002Uoo`03001oogoo00Eoo`03001oogoo02]oo`03001oogoo03Uoo`03001oogoo041o
o`00:Goo00<007ooOol01Goo00<007ooOol0;7oo00<007ooOol0=Woo0P00@goo000YOol00`00Oomo
o`05Ool00`00Oomoo`0/Ool00`00Oomoo`0dOol20015Ool002Qoo`03001oogoo00Ioo`03001oogoo
02aoo`03001oogoo039oo`8004Moo`00:7oo00<007ooOol01Woo0P00;Woo00<007ooOol0<7oo00<0
07ooOol0Agoo000XOol00`00Oomoo`06Ool00`00Oomoo`0]Ool00`00Oomoo`0_Ool00`00Oomoo`18
Ool002Qoo`03001oogoo00Ioo`03001oogoo02ioo`03001oogoo02aoo`8004]oo`00:7oo00<007oo
Ool01Woo00<007ooOol0;Woo00<007ooOol0:Woo0P00CGoo000>Ool40003Ool20003Ool3000;Ool0
0`00Oomoo`06Ool00`00Oomoo`0BOol00`00Oomoo`03Ool20003Ool3000?Ool40004Ool2000<Ool3
0003Ool40003Ool20003Ool4000<Ool30003Ool3000=Ool30004Ool00`00Oomoo`03Ool20003Ool3
000<Ool40003Ool20003Ool000moo`03001oogoo00=oo`8000Ioo`03001oogoo00Moo`03001oogoo
00Moo`03001oogoo019oo`03001oogoo00=oo`8000Ioo`03001oogoo00eoo`8000Aoo`04001oogoo
000<Ool00`00Oomoo`03Ool00`00Oomoo`03Ool20004Ool3000=Ool00`00Oomoo`05Ool00`00Oomo
o`0;Ool00`00Oomoo`03Ool00`00Oomoo`03Ool20006Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool0
1000Oomoo`000Woo000@Ool00`00Oomoo`0:Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomo
o`0COol00`00Oomoo`0:Ool00`00Oomoo`0>Ool00`00Oomoo`02Ool01000Oomoo`0037oo00<007oo
Ool017oo00<007ooOol01goo00@007ooOol000eoo`03001oogoo00Eoo`03001oogoo00]oo`03001o
ogoo00Aoo`03001oogoo00Yoo`03001oogoo00]oo`05001oogooOol00002Ool10001Ool1Ool0015o
o`03001oogoo00Ioo`<000Yoo`03001oogoo00Moo`8001Aoo`03001oogoo00Moo`<0015oo`03001o
ogoo009oo`04001oogoo000<Ool00`00Oomoo`05Ool00`00Oomoo`05Ool4000>Ool00`00Oomoo`02
Ool3000>Ool00`00Oomoo`04Ool00`00Oomoo`07Ool3000?Ool01000Oomoo`000Woo0@000Goo0Goo
000>Ool01000Oomoo`0027oo00<007ooOol02Woo00<007ooOol01goo00<007ooOol057oo00<007oo
Ool01Woo00<007ooOol04Goo0P000goo00@007ooOol000aoo`03001oogoo009oo`04001oogoo0005
Ool200000goo001oo`0?Ool00`00Oomoo`02Ool00`00Oomoo`0>Ool00`00Oomoo`05Ool00`00Oomo
o`06Ool00`00Oomoo`0<Ool01000Oomoo`000Woo00@007ooOol0009oo`003goo0P002Goo10002Goo
00<007ooOol01goo00<007ooOol04Goo100027oo10003goo0P0000=oo`00Ool00Woo0P0037oo0P00
1Goo0P001Goo00@007ooOomoo`@000aoo`8000Aoo`@000aoo`8000Aoo`@000Qoo`@000aoo`8000Ao
o`8000=oo`009goo00<007ooOol01goo00<007ooOol0<goo00<007ooOol077oo00<007ooOol0EGoo
0006Ool5000KOol00`00Oomoo`08Ool00`00Oomoo`0dOol00`00Oomoo`0IOol2001HOol000Eoo`80
00=oo`03001oogoo01Uoo`03001oogoo00Qoo`03001oogoo03Eoo`03001oogoo01Ioo`8005Yoo`00
1Goo00<007ooOol00goo0P006Goo00<007ooOol027oo00<007ooOol0=Woo00<007ooOol057oo00<0
07ooOol0FWoo003M0001Ool000Eoo`03001oogoo00=oo`06001oogoo001oo`001Goo00<007ooOol0
0Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0
0Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00goo00@007ooOomo
o`8000Eoo`03001oogoo009oo`04001oogooOol30004Ool00`00Oomoo`03Ool00`00Oomoo`02Ool0
0`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool0
0`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool0
0`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`04Ool000ioo`03001oogoo
01Eoo`03001oogoo00Qoo`03001oogoo03]oo`03001oogoo00Uoo`80065oo`003goo00<007ooOol0
4goo00<007ooOol02Goo00<007ooOol0?7oo0`0017oo1000Hgoo000?Ool00`00Oomoo`0COol00`00
Oomoo`09Ool00`00Oomoo`0nOol5001WOol000moo`03001oogoo01=oo`03001oogoo00Uoo`03001o
ogoo0:Yoo`0047oo00<007ooOol04Woo00<007ooOol02Goo00<007ooOol0ZWoo000@Ool00`00Oomo
o`0BOol00`00Oomoo`09Ool2002[Ool0015oo`03001oogoo011oo`03001oogoo00Yoo`03001oogoo
0:Yoo`004Goo00<007ooOol047oo00<007ooOol02Woo00<007ooOol0ZWoo000BOol00`00Oomoo`0?
Ool00`00Oomoo`0:Ool00`00Oomoo`2ZOol0019oo`03001oogoo00ioo`03001oogoo00]oo`03001o
ogoo0:Yoo`004goo00<007ooOol03Goo00<007ooOol02goo00<007ooOol0ZWoo000COol00`00Oomo
o`0=Ool00`00Oomoo`0;Ool00`00Oomoo`2ZOol001=oo`03001oogoo00aoo`03001oogoo00aoo`80
0:]oo`0057oo00<007ooOol02goo00<007ooOol037oo00<007ooOol0ZWoo000DOol00`00Oomoo`0;
Ool00`00Oomoo`0<Ool00`00Oomoo`2ZOol001Aoo`03001oogoo00]oo`03001oogoo00aoo`03001o
ogoo0:Yoo`005Goo00<007ooOol02Goo00<007ooOol03Goo00<007ooOol0ZWoo000EOol00`00Oomo
o`09Ool00`00Oomoo`0=Ool00`00Oomoo`2ZOol001Eoo`03001oogoo00Uoo`03001oogoo00eoo`03
001oogoo0:Yoo`005Woo00<007ooOol01goo00<007ooOol03Woo0P00Zgoo000FOol00`00Oomoo`07
Ool00`00Oomoo`0>Ool00`00Oomoo`2ZOol001Ioo`03001oogoo00Moo`03001oogoo00ioo`03001o
ogoo0:Yoo`005goo00<007ooOol01Goo00<007ooOol03goo00<007ooOol0ZWoo000GOol00`00Oomo
o`05Ool00`00Oomoo`0?Ool00`00Oomoo`2ZOol000aoo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`80
00=oo`@000Aoo`03001oogoo0:Yoo`002goo00@007ooOol000=oo`8000=oo`04001oo`000002Ool2
00001Goo001oogoo00000Woo00D007ooOomoo`0000Ioo`03001oogoo0:Yoo`002goo00@007ooOol0
00Qoo`04001oo`000002Ool200001Goo001oogoo00000Woo00<007ooOol00Woo00<007ooOol00goo
0`00ZWoo000;Ool01000Oomoo`0027oo00<007ooOol010000Woo00@007ooOol0009oo`03001oogoo
00=oo`03001oogoo009oo`03001oogoo0:Yoo`002goo00@007ooOol000Qoo`04001oogoo0002Ool0
1000Oomoo`000Woo00@007ooOol0009oo`04001oogoo0004Ool00`00Oomoo`2ZOol000aoo`8000Yo
o`8000Aoo`8000Aoo`8000Aoo`8000Eoo`03001oogoo0:Yoo`00<Goo00<007ooOol0ZWoo0000\
\>"],
  ImageRangeCache->{{{0, 221.938}, {136.438, 0}} -> {0.505387, -0.000750581, 
  0.0903698, 7.23044*^-6}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
samples = TableForm[
Table[Map[PaddedForm[N[#], {5, 4}]&, {S, Valuation[10, 0.05, 0, S, 5, 0.2],
BlackScholesPut[S,10,0.2,0.05,0,5],
Valuation[10, 0.05, 0, S, 5, 0.2]-
BlackScholesPut[S,10,0.2,0.05,0,5]}],
{S, 2, 16, 1}]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 5.7886\"\>",
                5.7886254763466622,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 5.7886\"\>",
                5.7885809005343702,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0000\"\>",
                4.4575812292002581*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 4.8007\"\>",
                4.8007206268369833,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 4.8005\"\>",
                4.800508718641141,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0002\"\>",
                0.00021190819584226972,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.8615\"\>",
                3.8615060709126934,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.8615\"\>",
                3.8615280263060248,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0000\"\>",
                -2.1955393331385409*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.0204\"\>",
                3.0203695163297484,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.0209\"\>",
                3.0208607031816879,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0005\"\>",
                -0.00049118685193949574,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 2.3101\"\>",
                2.3101224507555664,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 2.3108\"\>",
                2.310845529420245,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0007\"\>",
                -0.00072307866467857806,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.7380\"\>",
                1.7380303274063418,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.7387\"\>",
                1.738661710302452,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0006\"\>",
                -0.00063138289611019971,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.2928\"\>",
                1.2928323707373692,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.2932\"\>",
                1.2932190592768538,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0004\"\>",
                -0.00038668853948453652,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.9546\"\>",
                0.95462373549019008,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.9548\"\>",
                0.95478044388596572,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0002\"\>",
                -0.00015670839577563989,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.7019\"\>",
                0.70185205285644436,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.7019\"\>",
                0.70186980510265284,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0000\"\>",
                -1.7752246208480038*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.5149\"\>",
                0.51494785969766799,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.5149\"\>",
                0.51492057347081155,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0000\"\>",
                2.7286226856437779*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3777\"\>",
                0.37766825057222125,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3777\"\>",
                0.37766075385942499,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox[\(" 7.4967"\[Times]10\^"-6"\),
                7.4967127962621838*^-06,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2772\"\>",
                0.27721899477546702,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2773\"\>",
                0.27726230863712598,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0000\"\>",
                -4.3313861658955677*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2038\"\>",
                0.20384334554317909,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2039\"\>",
                0.2039428604577227,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0001\"\>",
                -9.9514914543602506*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1503\"\>",
                0.15025313382584179,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1504\"\>",
                0.15039922861977628,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0001\"\>",
                -0.00014609479393448699,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1111\"\>",
                0.11107590846063452,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1113\"\>",
                0.11125262488724719,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0002\"\>",
                -0.00017671642661266918,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Construction and Verification of Interpolated Valuation and \
Greeks\
\>", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
In this section we define some functions to compute Delta, Gamma \
and Theta directly from the finite-difference grid.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
listd[data_, step_] :=
Module[{dleft,dright,len},
len = Length[data];
dleft = (4*data[[2]]-3*data[[1]]-data[[3]])/(2*step);
dright = (3*data[[len]]-4*data[[len-1]]+data[[len-2]])/(2*step);
Join[{dleft}, Take[RotateLeft[data]-RotateRight[data], {2, -2}]/(2*step), \
{dright}]
]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(kt\  = \ ktwo[0.05, \ 0, \ 0.2]; \nko\  = \ kone[0.05, \ 0.2]; \)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(deltadata = 
      listd[finalstep, dx] - 1\/2\ \((kt - 1)\)\ finalstep; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(gammadata = 
      listd[deltadata, dx] - 1\/2\ \((kt + 1)\)\ deltadata; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(thetadata = \(3  finalstep - 4  prevstep + pprevstep\)\/\(2  dtau\) - \
\((1\/4\ \((kt - 1)\)\^2 + ko)\) finalstep; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(deltainterpoldata = Transpose[{points, deltadata}]; \)\), 
    \(\(gammainterpoldata = Transpose[{points, gammadata}]; \)\), 
    \(\(thetainterpoldata = Transpose[{points, thetadata}]; \)\)}], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[{
    \(dfunc = 
      Interpolation[deltainterpoldata, 
        InterpolationOrder \[Rule] 3]; \n\(gfunc = 
      Interpolation[gammainterpoldata, InterpolationOrder \[Rule] 3]; \)\), 
    \(\(tfunc = 
      Interpolation[thetainterpoldata, InterpolationOrder \[Rule] 3]; \)\)}], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(DougDelta[strike_, r_, q_, S_, T_, 
        sd_] := \(1\/S\) \((ValuationMultiplier[strike, r, q, 
              Log[S\/strike], \(sd\^2\ T\)\/2, sd]\ dfunc[
              Log[S\/strike]])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(DougGamma[strike_, r_, q_, S_, T_, 
        sd_] := \(1\/S\^2\) \((ValuationMultiplier[strike, r, q, 
              Log[S\/strike], \(sd\^2\ T\)\/2, sd]\ gfunc[
              Log[S\/strike]])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(DougTheta[strike_, r_, q_, S_, T_, 
        sd_] := \(-\(1\/2\)\)\ sd\^2\ ValuationMultiplier[strike, r, q, 
          Log[S\/strike], \(sd\^2\ T\)\/2, sd]\ tfunc[Log[S\/strike]]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Delta Analysis", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The percentage error in Delta is given in the last column of the \
following table.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
deltasamples = TableForm[
Table[Map[PaddedForm[N[#], {5, 4}]&, 
{S, DougDelta[10, 0.05, 0, S, 5, 0.2],
BlackScholesPutDelta[S,10,0.2,0.05,0,5],
100*(DougDelta[10,0.05,0,S,5,0.2]/BlackScholesPutDelta[S,10,0.2,0.05,0,5]-1)}]\
,
{S, 2, 16, 1}]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9978\"\>",
                -0.99775532180579918,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9976\"\>",
                -0.99757014495884477,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0186\"\>",
                0.018562789583276285,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9722\"\>",
                -0.9721835440667429,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9719\"\>",
                -0.97190384659171136,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0288\"\>",
                0.028778307238153111,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.8979\"\>",
                -0.89785932122015366,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.8973\"\>",
                -0.89729081174698944,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0634\"\>",
                0.063358441401772936,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.7789\"\>",
                -0.77891003083445409,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.7785\"\>",
                -0.77854854120728401,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0464\"\>",
                0.046431225291310163,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.6404\"\>",
                -0.64035692552323276,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.6404\"\>",
                -0.64043322578235118,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0119\"\>",
                -0.011913850819533778,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.5056\"\>",
                -0.50560385061711732,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.5060\"\>",
                -0.50595424315123527,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0693\"\>",
                -0.069253798907908948,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.3879\"\>",
                -0.3879483695693447,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.3883\"\>",
                -0.38833561265254934,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0997\"\>",
                -0.099718663596060075,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2919\"\>",
                -0.29189322883535807,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2922\"\>",
                -0.2921788833062795,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0978\"\>",
                -0.09776698017631702,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2168\"\>",
                -0.21677319370842119,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2169\"\>",
                -0.2169240328832196,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0695\"\>",
                -0.069535483364180362,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1596\"\>",
                -0.15964614783727951,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1597\"\>",
                -0.15968731335422298,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0258\"\>",
                -0.025778827433942109,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1170\"\>",
                -0.11698917319787783,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1170\"\>",
                -0.11696283853778006,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0225\"\>",
                0.022515407822698563,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0855\"\>",
                -0.085510977954020434,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0855\"\>",
                -0.085454614659577766,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0660\"\>",
                0.065956993273208475,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0625\"\>",
                -0.062452931143697592,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0624\"\>",
                -0.062392087714616684,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0975\"\>",
                0.097517860532581047,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0456\"\>",
                -0.04563504299101126,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0456\"\>",
                -0.045583721879110306,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1126\"\>",
                0.11258648874057364,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0334\"\>",
                -0.03339419476906854,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0334\"\>",
                -0.033357895170051416,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1088\"\>",
                0.10881861350087174,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell["\<\
The error in Delta is at most 0.1% of the exact value.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Delta Plots", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[{DougDelta[10, \ 0.05, \ 0, \ S, \ 5, \ 0.2], \n
        BlackScholesPutDelta[S, 10, 0.2, 0.05, 0, 5]}, \ {S, \ 8, \ 12}, \ 
      PlotPoints\  -> \ 50, \n\tPlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.88095 0.238095 0.85701 2.16899 [
[.2619 .19381 -3 -9 ]
[.2619 .19381 3 0 ]
[.5 .19381 -6 -9 ]
[.5 .19381 6 0 ]
[.7381 .19381 -6 -9 ]
[.7381 .19381 6 0 ]
[.97619 .19381 -6 -9 ]
[.97619 .19381 6 0 ]
[.01131 .09786 -30 -4.5 ]
[.01131 .09786 0 4.5 ]
[.01131 .31476 -30 -4.5 ]
[.01131 .31476 0 4.5 ]
[.01131 .42321 -24 -4.5 ]
[.01131 .42321 0 4.5 ]
[.01131 .53166 -30 -4.5 ]
[.01131 .53166 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .20631 m
.2619 .21256 L
s
[(9)] .2619 .19381 0 1 Mshowa
.5 .20631 m
.5 .21256 L
s
[(10)] .5 .19381 0 1 Mshowa
.7381 .20631 m
.7381 .21256 L
s
[(11)] .7381 .19381 0 1 Mshowa
.97619 .20631 m
.97619 .21256 L
s
[(12)] .97619 .19381 0 1 Mshowa
.125 Mabswid
.07143 .20631 m
.07143 .21006 L
s
.11905 .20631 m
.11905 .21006 L
s
.16667 .20631 m
.16667 .21006 L
s
.21429 .20631 m
.21429 .21006 L
s
.30952 .20631 m
.30952 .21006 L
s
.35714 .20631 m
.35714 .21006 L
s
.40476 .20631 m
.40476 .21006 L
s
.45238 .20631 m
.45238 .21006 L
s
.54762 .20631 m
.54762 .21006 L
s
.59524 .20631 m
.59524 .21006 L
s
.64286 .20631 m
.64286 .21006 L
s
.69048 .20631 m
.69048 .21006 L
s
.78571 .20631 m
.78571 .21006 L
s
.83333 .20631 m
.83333 .21006 L
s
.88095 .20631 m
.88095 .21006 L
s
.92857 .20631 m
.92857 .21006 L
s
.25 Mabswid
0 .20631 m
1 .20631 L
s
.02381 .09786 m
.03006 .09786 L
s
[(-0.35)] .01131 .09786 1 0 Mshowa
.02381 .31476 m
.03006 .31476 L
s
[(-0.25)] .01131 .31476 1 0 Mshowa
.02381 .42321 m
.03006 .42321 L
s
[(-0.2)] .01131 .42321 1 0 Mshowa
.02381 .53166 m
.03006 .53166 L
s
[(-0.15)] .01131 .53166 1 0 Mshowa
.125 Mabswid
.02381 .11955 m
.02756 .11955 L
s
.02381 .14124 m
.02756 .14124 L
s
.02381 .16293 m
.02756 .16293 L
s
.02381 .18462 m
.02756 .18462 L
s
.02381 .228 m
.02756 .228 L
s
.02381 .24969 m
.02756 .24969 L
s
.02381 .27138 m
.02756 .27138 L
s
.02381 .29307 m
.02756 .29307 L
s
.02381 .33645 m
.02756 .33645 L
s
.02381 .35814 m
.02756 .35814 L
s
.02381 .37983 m
.02756 .37983 L
s
.02381 .40152 m
.02756 .40152 L
s
.02381 .4449 m
.02756 .4449 L
s
.02381 .46659 m
.02756 .46659 L
s
.02381 .48828 m
.02756 .48828 L
s
.02381 .50997 m
.02756 .50997 L
s
.02381 .07617 m
.02756 .07617 L
s
.02381 .05448 m
.02756 .05448 L
s
.02381 .03279 m
.02756 .03279 L
s
.02381 .01111 m
.02756 .01111 L
s
.02381 .55335 m
.02756 .55335 L
s
.02381 .57504 m
.02756 .57504 L
s
.02381 .59673 m
.02756 .59673 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
.5 Mabswid
.02381 .01556 m
.04273 .03386 L
.06337 .05348 L
.08275 .07157 L
.10139 .08868 L
.12123 .10656 L
.14033 .12346 L
.16063 .14108 L
.18019 .15774 L
.199 .17346 L
.21902 .18986 L
.23829 .20534 L
.25683 .21994 L
.27656 .23519 L
.29555 .24956 L
.31574 .26453 L
.33519 .27864 L
.3539 .29194 L
.3738 .3058 L
.39297 .31886 L
.41334 .33243 L
.43296 .34522 L
.45184 .35727 L
.47193 .36979 L
.49127 .38159 L
.50987 .39269 L
.52967 .40424 L
.54872 .41512 L
.56898 .42641 L
.5885 .43704 L
.60727 .44703 L
.62725 .45742 L
.64648 .46719 L
.66497 .47638 L
.68466 .48593 L
.70361 .49492 L
.72376 .50424 L
.74317 .51301 L
.76183 .52125 L
.7817 .52981 L
.80082 .53785 L
.82115 .54619 L
.84073 .55403 L
.85957 .56139 L
.87961 .56904 L
.89891 .57621 L
.91747 .58295 L
.93723 .58996 L
.95624 .59653 L
.97619 .60326 L
Mistroke
Mfstroke
.02381 .01472 m
.04273 .03303 L
.06337 .05266 L
.08275 .07077 L
.10139 .08789 L
.12123 .10579 L
.14033 .1227 L
.16063 .14035 L
.18019 .15702 L
.199 .17277 L
.21902 .18919 L
.23829 .2047 L
.25683 .21932 L
.27656 .23458 L
.29555 .24898 L
.31574 .26397 L
.33519 .27811 L
.3539 .29144 L
.3738 .30532 L
.39297 .3184 L
.41334 .332 L
.43296 .34482 L
.45184 .35688 L
.47193 .36943 L
.49127 .38125 L
.50987 .39237 L
.52967 .40395 L
.54872 .41484 L
.56898 .42616 L
.5885 .43681 L
.60727 .44682 L
.62725 .45723 L
.64648 .46702 L
.66497 .47622 L
.68466 .4858 L
.70361 .4948 L
.72376 .50414 L
.74317 .51292 L
.76183 .52118 L
.7817 .52975 L
.80082 .53781 L
.82115 .54616 L
.84073 .55402 L
.85957 .56139 L
.87961 .56904 L
.89891 .57623 L
.91747 .58298 L
.93723 .59 L
.95624 .59658 L
.97619 .60332 L
Mistroke
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{222.938, 137.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003N0000RA000`40O003h00OgWoo003NOol00=io
o`00gWoo003NOol00=ioo`00gWoo003NOol00=ioo`0087oo00<007ooOol0^goo000POol00`00Oomo
o`2kOol0021oo`800;aoo`0087oo0P00_7oo000POol00`00Ool0002kOol0021oo`04001oogoo002j
Ool0021oo`80009oo`03001oogoo0;Moo`0087oo00<007ooOol00Woo00<007ooOol0]Woo000POol0
0`00Oomoo`03Ool00`00Oomoo`2eOol0021oo`03001oogoo00Aoo`03001oogoo0;Aoo`0087oo0P00
1Woo00<007ooOol0/goo000POol00`00Oomoo`05Ool2002dOol0021oo`03001oogoo00Ioo`800;=o
o`0087oo00<007ooOol01goo0P00/Woo000POol20009Ool2002aOol0021oo`03001oogoo00Yoo`03
001oogoo0:ioo`0087oo00<007ooOol02goo00<007ooOol0[Goo0008Ool20004Ool20003Ool30003
Ool30004Ool00`00Oomoo`0<Ool2002]Ool000Moo`04001oogoo0003Ool20006Ool00`00Oomoo`03
Ool01@00Oomoogoo000047oo00<007ooOol0ZWoo00000goo0000000300001Goo001oogoo00002goo
00<007ooOol00goo00@007ooOomoo`80011oo`03001oogoo0:Uoo`001goo00@007ooOol000Uoo`80
00=oo`<000Aoo`03001oogoo011oo`03001oogoo0:Qoo`001goo00@007ooOol000]oo`04001oogoo
0006Ool00`00Oomoo`0@Ool2002YOol000Qoo`8000Uoo`<000=oo`@000=oo`03001oogoo015oo`80
0:Qoo`0087oo0P004goo0P00Ygoo000POol00`00Oomoo`0COol2002VOol0021oo`03001oogoo01Eo
o`03001oogoo0:=oo`0087oo00<007ooOol05Woo00<007ooOol0XWoo000POol2000HOol2002ROol0
021oo`03001oogoo01Uoo`03001oogoo09moo`0087oo00<007ooOol06Woo00<007ooOol0WWoo000P
Ool00`00Oomoo`0KOol2000<Ool3000XOol30004Ool2000UOol30003Ool3000TOol30003Ool40002
Ool0021oo`8001ioo`03001oogoo00aoo`03001oogoo02Ioo`03001oogoo009oo`04001oogoo000U
Ool00`00Oomoo`03Ool00`00Oomoo`0TOol00`00Oomoo`03Ool00`00Oomoo`02Ool0021oo`03001o
ogoo01ioo`03001oogoo00Uoo`<002Qoo`03001oogoo009oo`04001oogoo000UOol00`00Oomoo`03
Ool00`00Oomoo`0TOol00`00Oomoo`04Ool00`00Oomoo`01Ool0021oo`03001oogoo01moo`8000Qo
o`04001oogoo000XOol00`00Oomoo`02Ool01000Oomoo`009Goo00<007ooOol00goo00<007ooOol0
97oo00<007ooOol01Goo0@000Goo0Goo000POol00`00Oomoo`0QOol00`00Oomoo`05Ool01000Oomo
o`00:7oo00<007ooOol00Woo00@007ooOol002Eoo`03001oogoo00=oo`03001oogoo02Aoo`03001o
ogoo009oo`04001oogoo0002Ool0021oo`8002=oo`03001oogoo00Eoo`8002Qoo`8000Eoo`8002Eo
o`8000Aoo`8002Eoo`8000Eoo`8000=oo`0087oo00<007ooOol08goo00<007ooOol0UGoo000POol0
0`00Oomoo`0TOol00`00Oomoo`2DOol0021oo`03001oogoo02Eoo`03001oogoo09=oo`006goo`P00
0Goo000POol00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00
Oomoo`03Ool01000Oomoo`0027oo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01Woo00<0
07ooOol01goo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01Woo00<0
07ooOol01goo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01Woo00<0
07ooOol01Woo00<007ooOol017oo000POol00`00Oomoo`0YOol00`00Ool0000]Ool00`00Oomoo`0[
Ool00`00Oomoo`0ZOol00`00Oomoo`04Ool0021oo`03001oogoo02Yoo`8008moo`0087oo00<007oo
Ool0;7oo00<007ooOol0S7oo000POol2000^Ool00`00Oomoo`2;Ool0021oo`03001oogoo02ioo`03
001oogoo08Yoo`0087oo00<007ooOol0;goo00<007ooOol0RGoo000POol00`00Oomoo`0`Ool00`00
Oomoo`28Ool0021oo`80039oo`8008Qoo`0087oo00<007ooOol0<goo00<007ooOol0QGoo000POol0
0`00Oomoo`0dOol00`00Oomoo`24Ool0021oo`03001oogoo03Eoo`8008Aoo`0087oo0P00>7oo0P00
PWoo000POol00`00Oomoo`0iOol20020Ool0021oo`03001oogoo03]oo`03001oogoo07eoo`0087oo
00<007ooOol0?7oo00<007ooOol0O7oo000POol2000nOol00`00Oomoo`1kOol0021oo`03001oogoo
03ioo`03001oogoo07Yoo`0027oo0P0017oo0P000goo10000Woo0`0017oo00<007ooOol0?goo00<0
07ooOol0NGoo0007Ool01000Oomoo`000goo0P0017oo00<007ooOol01Goo00D007ooOomoo`00049o
o`8007Uoo`0000=oo`0000000`0000Eoo`00Oomoo`0000Yoo`03001oogoo00Aoo`04001oogooOol2
0013Ool2001gOol000Moo`04001oogoo000;Ool00`00Oomoo`030004Ool00`00Oomoo`14Ool2001e
Ool000Moo`04001oogoo0008Ool01000Oomoo`000Woo00<007ooOol017oo00<007ooOol0AWoo00<0
07ooOol0LWoo0008Ool2000:Ool20003Ool40003Ool00`00Oomoo`17Ool00`00Oomoo`1aOol0021o
o`03001oogoo04Qoo`80075oo`0087oo0P00Bgoo0P00Kgoo000POol00`00Oomoo`1<Ool00`00Oomo
o`1/Ool0021oo`03001oogoo04eoo`03001oogoo06]oo`0087oo00<007ooOol0CWoo00<007ooOol0
JWoo000POol2001@Ool2001ZOol0021oo`03001oogoo055oo`8006Qoo`0087oo00<007ooOol0Dgoo
0P00IWoo000POol00`00Oomoo`1EOol2001TOol0021oo`8005Qoo`80069oo`0087oo00<007ooOol0
FGoo0P00H7oo000POol00`00Oomoo`1KOol00`00Oomoo`1MOol0021oo`03001oogoo05aoo`03001o
ogoo05aoo`0087oo0P00GWoo00<007ooOol0Fgoo000POol00`00Oomoo`1NOol2001KOol000ioo`80
00Aoo`8000=oo`@000=oo`03001oogoo061oo`8005Uoo`003Goo00@007ooOol000=oo`8000Aoo`03
001oogoo00=oo`03001oogoo069oo`8005Moo`001goo1@0000Eoo`00Oomoo`0000Yoo`03001oogoo
009oo`8006Eoo`8005Eoo`003Goo00@007ooOol000]oo`05001oogooOol0001XOol2001COol000eo
o`04001oogoo0008Ool01000Oomoo`000goo00<007ooOol0J7oo0P00DGoo000>Ool2000:Ool20004
Ool00`00Oomoo`1ZOol2001?Ool0021oo`8006eoo`03001oogoo04aoo`0087oo00<007ooOol0KGoo
0P00C7oo000POol00`00Oomoo`1_Ool2001:Ool0021oo`03001oogoo075oo`8004Qoo`0087oo00<0
07ooOol0Lgoo0P00AWoo000POol2001fOol20014Ool0021oo`03001oogoo07Moo`03001oogoo045o
o`0087oo00<007ooOol0N7oo0`00@7oo000POol00`00Oomoo`1kOol3000mOol0021oo`8007moo`80
03]oo`0087oo00<007ooOol0P7oo0P00>Goo000POol00`00Oomoo`22Ool2000gOol0021oo`03001o
ogoo08Aoo`8003Eoo`0087oo0P00Qgoo0P00<goo000POol00`00Oomoo`28Ool3000`Ool000Qoo`80
00Aoo`8000=oo`<000=oo`<000Aoo`03001oogoo08]oo`8002ioo`001goo00@007ooOol000=oo`80
00Aoo`03001oogoo00Eoo`05001oogooOol0002?Ool2000/Ool00003Ool0000000<00005Ool007oo
Ool00009Ool00`00Oomoo`05Ool01000Oomoogoo0P00T7oo0`00:Goo0007Ool01000Oomoo`002Goo
00<007ooOol00Woo0`0017oo00<007ooOol0TWoo0`009Woo0007Ool01000Oomoo`002Goo00<007oo
Ool00Woo00<007ooOol017oo00<007ooOol0UGoo0P0097oo0008Ool20009Ool20004Ool40003Ool0
0`00Oomoo`2GOol3000QOol0021oo`8009]oo`8001moo`0087oo00<007ooOol0W7oo0P007Goo000P
Ool00`00Oomoo`2NOol3000JOol0021oo`03001oogoo0:5oo`<001Moo`0087oo00<007ooOol0Y7oo
0P005Goo000POol2002WOol3000BOol0021oo`03001oogoo0:Uoo`<000moo`0087oo00<007ooOol0
[7oo0`0037oo000POol00`00Oomoo`2_Ool2000:Ool0021oo`800;9oo`<000Moo`0087oo00<007oo
Ool0]7oo00<007ooOol017oo000POol00`00Oomoo`2kOol0021oo`03001oogoo0;]oo`0087oo00<0
07ooOol0^goo003NOol00=ioo`00gWoo003NOol00=ioo`00gWoo003NOol00=ioo`00\
\>"],
  ImageRangeCache->{{{0, 221.938}, {136.438, 0}} -> {7.29362, -0.415864, 
  0.0217952, 0.00239251}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gamma Analysis", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
The percentage error in Gamma is given by the last column of the \
following table:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
gammasamples = TableForm[Table[Map[PaddedForm[N[#], {5, 4}]&, 
{S, DougGamma[10, 0.05, 0, S, 5, 0.2],
BlackScholesPutGamma[S,10,0.2,0.05,0,5],
100*(DougGamma[10,0.05,0,S,5,0.2]/BlackScholesPutGamma[S,10,0.2,0.05,0,5]-1)}]\
,
{S,2,16,1}]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0084\"\>",
                0.0084286672187828173,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0085\"\>",
                0.0084564450905314295,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.3285\"\>",
                -0.32848166636492149,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0477\"\>",
                0.047708595485093715,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0480\"\>",
                0.048026336475153919,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.6616\"\>",
                -0.6615973929733121,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1001\"\>",
                0.1001290488811498,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1000\"\>",
                0.10003582365214997,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0932\"\>",
                0.093191844277717983,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1335\"\>",
                0.1334633674232093,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1329\"\>",
                0.13291672718548236,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.4113\"\>",
                0.41126519536107686,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1398\"\>",
                0.13983214408317604,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1394\"\>",
                0.13936745826081531,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3334\"\>",
                0.33342491006120856,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1276\"\>",
                0.12758585937497094,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1274\"\>",
                0.12742324276041067,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1276\"\>",
                0.12761927183568833,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1070\"\>",
                0.10704470704656614,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1071\"\>",
                0.10711068765967688,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0616\"\>",
                -0.061600400998618188,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0852\"\>",
                0.0851874049936448,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0853\"\>",
                0.085343981515403081,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1835\"\>",
                -0.18346521802480309,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0655\"\>",
                0.065521013907191228,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0657\"\>",
                0.065673835817759235,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2327\"\>",
                -0.23269831686408837,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0493\"\>",
                0.049286988724486669,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0494\"\>",
                0.04939689354947454,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2225\"\>",
                -0.2224933939981355,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0365\"\>",
                0.036543448693238959,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0366\"\>",
                0.036606047267158145,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1710\"\>",
                -0.1710061003372787,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0268\"\>",
                0.026846901143156875,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0269\"\>",
                0.026872712419241808,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0961\"\>",
                -0.096050133243896774,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0196\"\>",
                0.01961394180350768,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0196\"\>",
                0.019616441041218111,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0127\"\>",
                -0.012740525690557725,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0143\"\>",
                0.014286816880327714,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0143\"\>",
                0.014277240229150501,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0671\"\>",
                0.067076346853500723,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0104\"\>",
                0.010394523615865932,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0104\"\>",
                0.010380531698771796,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1348\"\>",
                0.13478998475378035,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  PageBreakAbove->False,
  FontSize->8]
}, Open  ]],

Cell["\<\
The error is at most 0.7% of the exact value - near the strike it \
is now only 0.2%.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell["Plot of Computed and Exact Gamma", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[{DougGamma[10, \ 0.05, \ 0, \ S, \ 5, \ 0.2], \n
        BlackScholesPutGamma[S, 10, 0.2, 0.05, 0, 5]}, \ {S, \ 8, \ 12}, \ 
      PlotPoints\  -> \ 50, \n\tPlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.88095 0.238095 -0.290095 8.34104 [
[.2619 .53151 -3 -9 ]
[.2619 .53151 3 0 ]
[.5 .53151 -6 -9 ]
[.5 .53151 6 0 ]
[.7381 .53151 -6 -9 ]
[.7381 .53151 6 0 ]
[.97619 .53151 -6 -9 ]
[.97619 .53151 6 0 ]
[.01131 .04355 -24 -4.5 ]
[.01131 .04355 0 4.5 ]
[.01131 .12696 -24 -4.5 ]
[.01131 .12696 0 4.5 ]
[.01131 .21037 -24 -4.5 ]
[.01131 .21037 0 4.5 ]
[.01131 .29378 -24 -4.5 ]
[.01131 .29378 0 4.5 ]
[.01131 .37719 -24 -4.5 ]
[.01131 .37719 0 4.5 ]
[.01131 .4606 -24 -4.5 ]
[.01131 .4606 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .54401 m
.2619 .55026 L
s
[(9)] .2619 .53151 0 1 Mshowa
.5 .54401 m
.5 .55026 L
s
[(10)] .5 .53151 0 1 Mshowa
.7381 .54401 m
.7381 .55026 L
s
[(11)] .7381 .53151 0 1 Mshowa
.97619 .54401 m
.97619 .55026 L
s
[(12)] .97619 .53151 0 1 Mshowa
.125 Mabswid
.07143 .54401 m
.07143 .54776 L
s
.11905 .54401 m
.11905 .54776 L
s
.16667 .54401 m
.16667 .54776 L
s
.21429 .54401 m
.21429 .54776 L
s
.30952 .54401 m
.30952 .54776 L
s
.35714 .54401 m
.35714 .54776 L
s
.40476 .54401 m
.40476 .54776 L
s
.45238 .54401 m
.45238 .54776 L
s
.54762 .54401 m
.54762 .54776 L
s
.59524 .54401 m
.59524 .54776 L
s
.64286 .54401 m
.64286 .54776 L
s
.69048 .54401 m
.69048 .54776 L
s
.78571 .54401 m
.78571 .54776 L
s
.83333 .54401 m
.83333 .54776 L
s
.88095 .54401 m
.88095 .54776 L
s
.92857 .54401 m
.92857 .54776 L
s
.25 Mabswid
0 .54401 m
1 .54401 L
s
.02381 .04355 m
.03006 .04355 L
s
[(0.04)] .01131 .04355 1 0 Mshowa
.02381 .12696 m
.03006 .12696 L
s
[(0.05)] .01131 .12696 1 0 Mshowa
.02381 .21037 m
.03006 .21037 L
s
[(0.06)] .01131 .21037 1 0 Mshowa
.02381 .29378 m
.03006 .29378 L
s
[(0.07)] .01131 .29378 1 0 Mshowa
.02381 .37719 m
.03006 .37719 L
s
[(0.08)] .01131 .37719 1 0 Mshowa
.02381 .4606 m
.03006 .4606 L
s
[(0.09)] .01131 .4606 1 0 Mshowa
.125 Mabswid
.02381 .06023 m
.02756 .06023 L
s
.02381 .07691 m
.02756 .07691 L
s
.02381 .09359 m
.02756 .09359 L
s
.02381 .11027 m
.02756 .11027 L
s
.02381 .14364 m
.02756 .14364 L
s
.02381 .16032 m
.02756 .16032 L
s
.02381 .177 m
.02756 .177 L
s
.02381 .19368 m
.02756 .19368 L
s
.02381 .22705 m
.02756 .22705 L
s
.02381 .24373 m
.02756 .24373 L
s
.02381 .26041 m
.02756 .26041 L
s
.02381 .2771 m
.02756 .2771 L
s
.02381 .31046 m
.02756 .31046 L
s
.02381 .32714 m
.02756 .32714 L
s
.02381 .34382 m
.02756 .34382 L
s
.02381 .36051 m
.02756 .36051 L
s
.02381 .39387 m
.02756 .39387 L
s
.02381 .41055 m
.02756 .41055 L
s
.02381 .42723 m
.02756 .42723 L
s
.02381 .44392 m
.02756 .44392 L
s
.02381 .47728 m
.02756 .47728 L
s
.02381 .49396 m
.02756 .49396 L
s
.02381 .51064 m
.02756 .51064 L
s
.02381 .52733 m
.02756 .52733 L
s
.02381 .02686 m
.02756 .02686 L
s
.02381 .01018 m
.02756 .01018 L
s
.02381 .56069 m
.02756 .56069 L
s
.02381 .57737 m
.02756 .57737 L
s
.02381 .59405 m
.02756 .59405 L
s
.02381 .61074 m
.02756 .61074 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
.5 Mabswid
.02381 .60277 m
.04273 .58814 L
.06337 .57214 L
.08275 .5571 L
.10139 .54265 L
.12123 .52728 L
.14033 .51253 L
.16063 .49692 L
.18019 .48194 L
.199 .46762 L
.21902 .45248 L
.23829 .43802 L
.25683 .42422 L
.27656 .40965 L
.29555 .39577 L
.31574 .38117 L
.33519 .36726 L
.3539 .35404 L
.3738 .34013 L
.39297 .32692 L
.41334 .31307 L
.43296 .29991 L
.45184 .28743 L
.47193 .27435 L
.49127 .26195 L
.50987 .25021 L
.52967 .23791 L
.54872 .22626 L
.56898 .2141 L
.5885 .20258 L
.60727 .19169 L
.62725 .18031 L
.64648 .16956 L
.66497 .1594 L
.68466 .14879 L
.70361 .13877 L
.72376 .12832 L
.74317 .11845 L
.76183 .10915 L
.7817 .09944 L
.80082 .09029 L
.82115 .08077 L
.84073 .07178 L
.85957 .06332 L
.87961 .05451 L
.89891 .04621 L
.91747 .03839 L
.93723 .03025 L
.95624 .02258 L
.97619 .01472 L
Mistroke
Mfstroke
.02381 .60332 m
.04273 .58879 L
.06337 .57289 L
.08275 .55794 L
.10139 .54356 L
.12123 .52827 L
.14033 .51358 L
.16063 .49803 L
.18019 .4831 L
.199 .46882 L
.21902 .45372 L
.23829 .43929 L
.25683 .42552 L
.27656 .41098 L
.29555 .39711 L
.31574 .38252 L
.33519 .36862 L
.3539 .3554 L
.3738 .34149 L
.39297 .32827 L
.41334 .31441 L
.43296 .30124 L
.45184 .28875 L
.47193 .27566 L
.49127 .26324 L
.50987 .25148 L
.52967 .23915 L
.54872 .22748 L
.56898 .21528 L
.5885 .20374 L
.60727 .19282 L
.62725 .18141 L
.64648 .17063 L
.66497 .16044 L
.68466 .14979 L
.70361 .13974 L
.72376 .12926 L
.74317 .11936 L
.76183 .11002 L
.7817 .10028 L
.80082 .0911 L
.82115 .08154 L
.84073 .07252 L
.85957 .06403 L
.87961 .05518 L
.89891 .04685 L
.91747 .039 L
.93723 .03083 L
.95624 .02313 L
.97619 .01524 L
Mistroke
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{222.938, 137.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003N0000RA000`40O003h00OgWoo003NOol00=io
o`00gWoo003NOol00=ioo`00gWoo000JOol00`00Oomoo`31Ool001Yoo`03001oogoo0<5oo`006Woo
0P00`Woo000JOol00`00Oomoo`2hOol30006Ool001Yoo`03001oogoo0;Eoo`<000Uoo`006Woo0P00
]Goo00<007ooOol02Woo0002Ool20004Ool20004Ool20004Ool30003Ool00`00Oomoo`2aOol3000=
Ool00005Ool007ooOol00003Ool20003Ool01000Oomoo`0017oo00<007ooOol00Woo00<007ooOol0
[Woo10003goo00001Goo001oogoo000027oo00@007ooOol0009oo`@000=oo`800:eoo`@0015oo`00
00Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool00`00Ool00004Ool00`00Oomoo`2ZOol3000DOol0
0005Ool007ooOol00008Ool01000Oomoo`000Woo00<007oo000017oo00<007ooOol0Z7oo0P005goo
0002Ool2000:Ool20004Ool20004Ool00`00Oomoo`2UOol3000IOol001Yoo`800:=oo`@001]oo`00
6Woo00<007ooOol0X7oo10007Goo000JOol00`00Oomoo`2NOol3000POol001Yoo`8009ioo`03001o
ogoo025oo`006Woo00<007ooOol0Vgoo0P0097oo000JOol00`00Oomoo`2IOol2000VOol001Yoo`80
09Qoo`8002Qoo`006Woo00<007ooOol0UGoo0P00:Woo000JOol00`00Oomoo`2COol2000/Ool001Yo
o`03001oogoo095oo`8002ioo`006Woo0P00Sgoo0`00<7oo0002Ool20004Ool20004Ool20003Ool3
0004Ool00`00Oomoo`2;Ool4000bOol00005Ool007ooOol00003Ool20003Ool01000Oomoo`001Goo
00D007ooOomoo`0008]oo`@003Aoo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0005Ool01000Oomo
ogoo0P00R7oo1000=Woo00001Goo001oogoo000027oo00@007ooOol0009oo`<000Aoo`03001oogoo
08Eoo`@003Qoo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool00`00Oomoo`04Ool00`00Oomo
o`23Ool4000jOol0009oo`8000Yoo`8000=oo`@000=oo`80089oo`@003aoo`006Woo00<007ooOol0
P7oo0`00?Woo000JOol00`00Oomoo`1oOol20010Ool001Yoo`03001oogoo07ioo`03001oogoo041o
o`006Woo0P00OGoo0P00@goo000JOol00`00Oomoo`1jOol20015Ool001Yoo`03001oogoo07Qoo`80
04Moo`006Woo0P00Mgoo0P00BGoo000JOol00`00Oomoo`1dOol2001;Ool001Yoo`03001oogoo079o
o`8004eoo`006Woo0P00LGoo0P00Cgoo000JOol00`00Oomoo`1^Ool2001AOol0009oo`8000Aoo`80
00Aoo`8000Aoo`8000Aoo`03001oogoo06aoo`8005=oo`0000Eoo`00Oomoo`0000=oo`8000=oo`04
001oogoo0002Ool01000Oomoo`000goo00<007ooOol0JWoo0P00EGoo00001Goo001oogoo000027oo
00@007ooOol0009oo`04001oogoo0003Ool2001YOol3001FOol00005Ool007ooOol00008Ool01000
Oomoo`000Woo0`0017oo00<007ooOol0Igoo0`00Egoo00001Goo001oogoo000027oo00@007ooOol0
009oo`03001oogoo00Aoo`03001oogoo06Ioo`8005Uoo`000Woo0P002Woo0P0017oo0`000goo0P00
IGoo0P00Fgoo000JOol00`00Oomoo`1SOol00`00Oomoo`1KOol001Yoo`03001oogoo065oo`8005io
o`006Woo0P00HGoo00<007ooOol0GWoo000JOol00`00Oomoo`1OOol00`00Oomoo`1OOol001Yoo`03
001oogoo05eoo`80069oo`006Woo00<007ooOol0Fgoo0P00I7oo000JOol2001JOol2001VOol001Yo
o`03001oogoo05Moo`8006Qoo`006Woo00<007ooOol0EGoo0`00JGoo000JOol2001EOol3001ZOol0
009oo`8000Aoo`8000Aoo`8000Aoo`03001oogoo00=oo`03001oogoo05=oo`8006aoo`0000Eoo`00
Oomoo`0000=oo`8000=oo`04001oogoo0003Ool00`00Oomoo`03Ool00`00Oomoo`1AOol2001^Ool0
0005Ool007ooOol00008Ool01000Oomoo`0017oo00<007ooOol00Woo0P00D7oo0P00L7oo00001Goo
001oogoo000027oo00@007ooOol000Aoo`03001oogoo009oo`03001oogoo04eoo`<0075oo`0000Eo
o`00Oomoo`0000Qoo`04001oogoo0005Ool01@00Oomoogoo0000CWoo0`00LWoo0002Ool2000:Ool2
0003Ool40003Ool00`00Oomoo`1;Ool2001dOol001Yoo`8004Yoo`8007Ioo`006Woo00<007ooOol0
B7oo00<007ooOol0MWoo000JOol00`00Oomoo`17Ool00`00Oomoo`1gOol001Yoo`8004Ioo`8007Yo
o`006Woo00<007ooOol0A7oo0P00Ngoo000JOol00`00Oomoo`12Ool3001lOol001Yoo`80049oo`<0
07eoo`006Woo00<007ooOol0@7oo0P00Ogoo000JOol00`00Oomoo`0nOol30020Ool001Yoo`03001o
ogoo03eoo`<0085oo`006Woo0P00?Goo0P00Pgoo0002Ool20004Ool20004Ool20004Ool20004Ool0
0`00Oomoo`0jOol20025Ool00005Ool007ooOol00003Ool20003Ool01000Oomoo`000Woo00@007oo
Ool000=oo`03001oogoo03Uoo`03001oogoo08Eoo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002
Ool01000Oomoo`000goo0P00>Goo00<007ooOol0QWoo00001Goo001oogoo000027oo00@007ooOol0
00=oo`8000Aoo`03001oogoo03Ioo`8008Uoo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool0
1000Oomoo`000goo00<007ooOol0=Goo00<007ooOol0RGoo0002Ool2000:Ool20004Ool20004Ool2
000eOol00`00Oomoo`2:Ool001Yoo`03001oogoo039oo`8008eoo`006Woo00<007ooOol0<7oo0P00
Sgoo000JOol2000_Ool3002@Ool001Yoo`03001oogoo02eoo`<0095oo`006Woo00<007ooOol0;7oo
0P00Tgoo000JOol00`00Oomoo`0[Ool00`00Oomoo`2COol001Yoo`8002]oo`03001oogoo09Aoo`00
6Woo00<007ooOol0:Goo00<007ooOol0UGoo000JOol00`00Oomoo`0WOol2002HOol001Yoo`8002Mo
o`03001oogoo09Qoo`000Woo0P0017oo0P0017oo0P000goo0`0017oo00<007ooOol09Goo00<007oo
Ool0VGoo00001Goo001oogoo00000goo0P000goo00@007ooOol000Eoo`05001oogooOol0000UOol2
002LOol00005Ool007ooOol00008Ool01000Oomoo`000goo0`000goo0P008goo00<007ooOol0W7oo
00001Goo001oogoo000027oo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`0QOol00`00
Oomoo`2MOol00005Ool007ooOol00008Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oogoo
01moo`800:1oo`000Woo0P002Woo0P0017oo0P0017oo00<007ooOol07Woo00<007ooOol0X7oo000J
Ool2000NOol00`00Oomoo`2QOol001Yoo`03001oogoo01]oo`800:Aoo`006Woo00<007ooOol06Woo
00<007ooOol0Y7oo000JOol2000JOol00`00Oomoo`2UOol001Yoo`03001oogoo01Moo`80015oo`<0
02Yoo`<000Aoo`8002Ioo`<000=oo`<002Ioo`<000=oo`@0009oo`006Woo00<007ooOol05Woo00<0
07ooOol057oo00<007ooOol0:7oo00<007ooOol00Woo00@007ooOol002Ioo`03001oogoo00=oo`03
001oogoo02Ioo`03001oogoo00=oo`03001oogoo009oo`006Woo0P005Woo00<007ooOol04goo0`00
:Woo00<007ooOol00Woo00@007ooOol002Ioo`03001oogoo00=oo`03001oogoo02Ioo`03001oogoo
00Aoo`03001oogoo005oo`006Woo00<007ooOol04goo0P005Goo00@007ooOol002Yoo`03001oogoo
009oo`04001oogoo000VOol00`00Oomoo`03Ool00`00Oomoo`0VOol00`00Oomoo`05Ool10001Ool1
Ool001Yoo`03001oogoo019oo`03001oogoo01Eoo`04001oogoo000ZOol00`00Oomoo`02Ool01000
Oomoo`009Woo00<007ooOol00goo00<007ooOol09Woo00<007ooOol00Woo00@007ooOol0009oo`00
6Woo00<007ooOol04Goo00<007ooOol05goo0P00:Woo0P001Goo0P009Woo0P0017oo0P009goo0P00
1Goo0P000goo000JOol2000@Ool2002`Ool001Yoo`03001oogoo00ioo`03001oogoo0;1oo`006Woo
00<007ooOol03Goo00<007ooOol0/Goo000EOoo80001Ool001Yoo`03001oogoo00Ioo`03001oogoo
009oo`03001oogoo009oo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Ioo`03001oogoo
00Moo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo
00Ioo`03001oogoo00Moo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Ioo`03001oogoo
00Moo`03001oogoo00Ioo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Ioo`03001oogoo
00Aoo`006Woo00<007ooOol02Woo00<007ooOol0]7oo000JOol20009Ool2002gOol001Yoo`03001o
ogoo00Moo`03001oogoo0;Moo`006Woo00<007ooOol01Woo00<007ooOol0^7oo000JOol00`00Oomo
o`04Ool2002kOol001Yoo`8000Aoo`03001oogoo0;]oo`006Woo00<007ooOol00Woo00<007ooOol0
_7oo000JOol00`00Oomoo`02002oOol001Yoo`<00<5oo`006Woo0P00`Woo000JOol00`00Oomoo`31
Ool001Yoo`800<9oo`006Woo00<007ooOol0`Goo000JOol00`00Oomoo`31Ool00=ioo`00gWoo003N
Ool00=ioo`00gWoo003NOol00001\
\>"],
  ImageRangeCache->{{{0, 221.938}, {136.438, 0}} -> {7.44075, 0.0307117, 
  0.0211139, 0.000602695}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Theta Analysis", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
The percentage error in Theta is given by the last column of the \
following table:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
thetasamples = TableForm[
Table[Map[PaddedForm[N[#], {5, 4}]&, 
{S, DougTheta[10, 0.05, 0, S, 5, 0.2],
BlackScholesPutTheta[S,10,0.2,0.05,0,5],
100*(DougTheta[10,0.05,0,S,5,0.2]/BlackScholesPutTheta[S,10,0.2,0.05,0,5]-1)}]\
,
{S, 2, 16, 1}]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3885\"\>",
                0.38847661442308001,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3885\"\>",
                0.38850954391536052,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0085\"\>",
                -0.008475851570755033,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3773\"\>",
                0.37726436312006728,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3772\"\>",
                0.3771662723552861,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0260\"\>",
                0.026007300220309837,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3407\"\>",
                0.34069274619979045,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3405\"\>",
                0.34052310009601122,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0498\"\>",
                0.049819264458528245,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2792\"\>",
                0.27915700782462416,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2792\"\>",
                0.27922180686816422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0232\"\>",
                -0.023207013902981632,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2071\"\>",
                0.20705896391613995,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2073\"\>",
                0.20732767425793053,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1296\"\>",
                -0.1296065962985149,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1389\"\>",
                0.13889257121514137,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1391\"\>",
                0.13914229271285256,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1795\"\>",
                -0.17947203028092495,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0828\"\>",
                0.082800990721341852,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0829\"\>",
                0.082893517820476131,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1116\"\>",
                -0.1116216340759757,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0410\"\>",
                0.041026404044477896,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0410\"\>",
                0.040962269627171116,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1566\"\>",
                0.15656949160902656,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0124\"\>",
                0.012355940744260744,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0122\"\>",
                0.012207835061223929,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.2132\"\>",
                1.2132018682595724,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0058\"\>",
                -0.0058115328929221922,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0060\"\>",
                -0.005966431371365169,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-2.5962\"\>",
                -2.5961662642494332,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0163\"\>",
                -0.016252958776504959,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0164\"\>",
                -0.016364675313776202,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.6827\"\>",
                -0.68266882861524047,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0214\"\>",
                -0.021371387978781589,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0214\"\>",
                -0.021421153016455467,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2323\"\>",
                -0.23231726898943483,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0230\"\>",
                -0.023033401185746993,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0230\"\>",
                -0.023024844458457053,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0372\"\>",
                0.037163018865893349,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0226\"\>",
                -0.022591575925454535,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0225\"\>",
                -0.022539828190855676,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2296\"\>",
                0.22958353613296012,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0210\"\>",
                -0.020976110145520574,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0209\"\>",
                -0.020899374917308117,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3672\"\>",
                0.36716518324626524,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell["\<\
Near the strike the error in \[Theta] peaks at 2.6% of the exact \
value. Note that with the two-time-level Crank-Nicholson scheme the error was \
almost 200%.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Theta Plots", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[{DougTheta[10, \ 0.05, \ 0, \ S, \ 5, \ 0.2], \n
        BlackScholesPutTheta[S, 10, 0.2, 0.05, 0, 5]}, \ {S, \ 8, \ 12}, \ 
      PlotPoints\  -> \ 50, \n\tPlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.88095 0.238095 0.111758 5.93003 [
[.2619 .09926 -3 -9 ]
[.2619 .09926 3 0 ]
[.5 .09926 -6 -9 ]
[.5 .09926 6 0 ]
[.7381 .09926 -6 -9 ]
[.7381 .09926 6 0 ]
[.97619 .09926 -6 -9 ]
[.97619 .09926 6 0 ]
[.01131 .23036 -24 -4.5 ]
[.01131 .23036 0 4.5 ]
[.01131 .34896 -24 -4.5 ]
[.01131 .34896 0 4.5 ]
[.01131 .46756 -24 -4.5 ]
[.01131 .46756 0 4.5 ]
[.01131 .58616 -24 -4.5 ]
[.01131 .58616 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .11176 m
.2619 .11801 L
s
[(9)] .2619 .09926 0 1 Mshowa
.5 .11176 m
.5 .11801 L
s
[(10)] .5 .09926 0 1 Mshowa
.7381 .11176 m
.7381 .11801 L
s
[(11)] .7381 .09926 0 1 Mshowa
.97619 .11176 m
.97619 .11801 L
s
[(12)] .97619 .09926 0 1 Mshowa
.125 Mabswid
.07143 .11176 m
.07143 .11551 L
s
.11905 .11176 m
.11905 .11551 L
s
.16667 .11176 m
.16667 .11551 L
s
.21429 .11176 m
.21429 .11551 L
s
.30952 .11176 m
.30952 .11551 L
s
.35714 .11176 m
.35714 .11551 L
s
.40476 .11176 m
.40476 .11551 L
s
.45238 .11176 m
.45238 .11551 L
s
.54762 .11176 m
.54762 .11551 L
s
.59524 .11176 m
.59524 .11551 L
s
.64286 .11176 m
.64286 .11551 L
s
.69048 .11176 m
.69048 .11551 L
s
.78571 .11176 m
.78571 .11551 L
s
.83333 .11176 m
.83333 .11551 L
s
.88095 .11176 m
.88095 .11551 L
s
.92857 .11176 m
.92857 .11551 L
s
.25 Mabswid
0 .11176 m
1 .11176 L
s
.02381 .23036 m
.03006 .23036 L
s
[(0.02)] .01131 .23036 1 0 Mshowa
.02381 .34896 m
.03006 .34896 L
s
[(0.04)] .01131 .34896 1 0 Mshowa
.02381 .46756 m
.03006 .46756 L
s
[(0.06)] .01131 .46756 1 0 Mshowa
.02381 .58616 m
.03006 .58616 L
s
[(0.08)] .01131 .58616 1 0 Mshowa
.125 Mabswid
.02381 .14141 m
.02756 .14141 L
s
.02381 .17106 m
.02756 .17106 L
s
.02381 .20071 m
.02756 .20071 L
s
.02381 .26001 m
.02756 .26001 L
s
.02381 .28966 m
.02756 .28966 L
s
.02381 .31931 m
.02756 .31931 L
s
.02381 .37861 m
.02756 .37861 L
s
.02381 .40826 m
.02756 .40826 L
s
.02381 .43791 m
.02756 .43791 L
s
.02381 .49721 m
.02756 .49721 L
s
.02381 .52686 m
.02756 .52686 L
s
.02381 .55651 m
.02756 .55651 L
s
.02381 .08211 m
.02756 .08211 L
s
.02381 .05246 m
.02756 .05246 L
s
.02381 .02281 m
.02756 .02281 L
s
.02381 .61581 m
.02756 .61581 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
.5 Mabswid
.02381 .60277 m
.04273 .57998 L
.06337 .55575 L
.08275 .53358 L
.10139 .51279 L
.12123 .49125 L
.14033 .47106 L
.16063 .4502 L
.18019 .43067 L
.199 .41242 L
.21902 .39355 L
.23829 .37593 L
.25683 .35947 L
.27656 .34247 L
.29555 .32662 L
.31574 .31029 L
.33519 .29508 L
.3539 .28091 L
.3738 .26632 L
.39297 .25275 L
.41334 .23882 L
.43296 .22587 L
.45184 .21385 L
.47193 .20151 L
.49127 .19006 L
.50987 .17945 L
.52967 .16856 L
.54872 .15847 L
.56898 .14816 L
.5885 .13862 L
.60727 .1298 L
.62725 .12078 L
.64648 .11246 L
.66497 .10477 L
.68466 .09692 L
.70361 .08969 L
.72376 .08233 L
.74317 .07555 L
.76183 .06932 L
.7817 .06299 L
.80082 .05717 L
.82115 .05128 L
.84073 .04589 L
.85957 .04095 L
.87961 .03595 L
.89891 .03139 L
.91747 .02722 L
.93723 .02301 L
.95624 .01918 L
.97619 .01538 L
Mistroke
Mfstroke
.02381 .60332 m
.04273 .58045 L
.06337 .55612 L
.08275 .53387 L
.10139 .51301 L
.12123 .49138 L
.14033 .47112 L
.16063 .45018 L
.18019 .43058 L
.199 .41225 L
.21902 .39331 L
.23829 .37562 L
.25683 .35911 L
.27656 .34205 L
.29555 .32614 L
.31574 .30976 L
.33519 .29449 L
.3539 .28027 L
.3738 .26564 L
.39297 .25203 L
.41334 .23806 L
.43296 .22508 L
.45184 .21303 L
.47193 .20066 L
.49127 .18919 L
.50987 .17856 L
.52967 .16765 L
.54872 .15755 L
.56898 .14723 L
.5885 .13768 L
.60727 .12885 L
.62725 .11983 L
.64648 .11151 L
.66497 .10383 L
.68466 .09598 L
.70361 .08875 L
.72376 .0814 L
.74317 .07464 L
.76183 .06842 L
.7817 .0621 L
.80082 .0563 L
.82115 .05043 L
.84073 .04506 L
.85957 .04014 L
.87961 .03516 L
.89891 .03062 L
.91747 .02647 L
.93723 .02229 L
.95624 .01849 L
.97619 .01472 L
Mistroke
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{222.938, 137.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003N0000RA000`40O003h00OgWoo003NOol00=io
o`00gWoo003NOol00=ioo`00gWoo000JOol00`00Oomoo`31Ool001Yoo`03001oogoo0<5oo`006Woo
00<007ooOol0`Goo000JOol00`00Oomoo`2eOol60006Ool001Yoo`800;9oo`L000Uoo`006Woo00<0
07ooOol0[Goo100047oo000JOol00`00Oomoo`2YOol4000DOol001Yoo`03001oogoo0:Eoo`@001Qo
o`006Woo00<007ooOol0XGoo100077oo000JOol00`00Oomoo`2NOol3000POol001Yoo`8009]oo`@0
02=oo`006Woo00<007ooOol0UWoo10009goo000JOol00`00Oomoo`2BOol4000[Ool001Yoo`03001o
ogoo02Yoo`<002Yoo`<000Aoo`8002Ioo`<000=oo`L0029oo`<000=oo`@0009oo`006Woo00<007oo
Ool0;Goo00<007ooOol0:7oo00<007ooOol00Woo00@007ooOol002Ioo`03001oogoo00=oo`@002Eo
o`03001oogoo00=oo`03001oogoo009oo`006Woo00<007ooOol0:goo0`00:Woo00<007ooOol00Woo
00@007ooOol002Ioo`03001oogoo00@002Qoo`03001oogoo00Aoo`03001oogoo005oo`006Woo0P00
:goo00@007ooOol002Yoo`03001oogoo009oo`04001oogoo000UOol40003Ool00`00Oomoo`0VOol0
0`00Oomoo`05Ool10001Ool1Ool001Yoo`03001oogoo02Yoo`04001oogoo000ZOol00`00Oomoo`02
Ool01000Oomoo`008Woo1@001Goo00<007ooOol09Woo00<007ooOol00Woo00@007ooOol0009oo`00
6Woo00<007ooOol0:goo0P00:Woo0P001Goo0P008Goo100000=oo`00000017oo0P009goo0P001Goo
0P000goo000JOol00`00Oomoo`1oOol3000oOol001Yoo`03001oogoo07aoo`<0049oo`006Woo00<0
07ooOol0NWoo1000@goo000EOoo80001Ool001Yoo`03001oogoo00Ioo`03001oogoo00Moo`03001o
ogoo00Ioo`03001oogoo00Moo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Ioo`03001o
ogoo00Moo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Ioo`03001oogoo00Moo`03001o
ogoo00Aoo`<000Uoo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Ioo`03001oogoo00Io
o`03001oogoo00Moo`03001oogoo00Ioo`03001oogoo00Aoo`006Woo00<007ooOol0Lgoo0`00Bgoo
000JOol00`00Oomoo`1`Ool4001=Ool001Yoo`03001oogoo06ioo`@004moo`006Woo00<007ooOol0
K7oo0`00DWoo000JOol2001[Ool2001EOol001Yoo`03001oogoo06Qoo`8005Moo`006Woo00<007oo
Ool0IWoo0P00FGoo000JOol00`00Oomoo`1TOol2001KOol001Yoo`03001oogoo06=oo`03001oogoo
05]oo`006Woo00<007ooOol0HGoo0P00GWoo000JOol2001POol2001POol001Yoo`03001oogoo05eo
o`<0065oo`006Woo00<007ooOol0Fgoo1000HWoo000JOol00`00Oomoo`1IOol4001TOol001Yoo`03
001oogoo05Qoo`<006Ioo`006Woo00<007ooOol0Egoo0P00J7oo000JOol2001FOol2001ZOol001Yo
o`03001oogoo05=oo`8006aoo`006Woo00<007ooOol0DGoo0P00KWoo000JOol00`00Oomoo`1@Ool0
0`00Oomoo`1^Ool0009oo`8000Aoo`8000Aoo`8000=oo`@000=oo`03001oogoo04moo`03001oogoo
06moo`0000Eoo`00Oomoo`0000=oo`8000=oo`04001oogoo0003Ool00`00Oomoo`03Ool00`00Oomo
o`1=Ool2001bOol00005Ool007ooOol00008Ool01000Oomoo`0017oo00<007ooOol00Woo0P00C7oo
0P00M7oo00001Goo001oogoo000027oo00@007ooOol000Eoo`05001oogooOol0001;Ool2001fOol0
0005Ool007ooOol00008Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oogoo04Qoo`03001o
ogoo07Ioo`000Woo0P002Woo0P0017oo0P0017oo00<007ooOol0Agoo00<007ooOol0Mgoo000JOol0
0`00Oomoo`16Ool00`00Oomoo`1hOol001Yoo`03001oogoo04Eoo`03001oogoo07Uoo`006Woo0P00
AGoo00<007ooOol0NWoo000JOol00`00Oomoo`12Ool2001mOol001Yoo`03001oogoo045oo`03001o
ogoo07eoo`006Woo00<007ooOol0@7oo00<007ooOol0OWoo000JOol00`00Oomoo`0nOol20021Ool0
01Yoo`8003eoo`8008=oo`006Woo00<007ooOol0>Woo0`00Q7oo000JOol00`00Oomoo`0iOol30025
Ool001Yoo`03001oogoo03Qoo`8008Moo`006Woo00<007ooOol0=goo0P00R7oo000JOol00`00Oomo
o`0fOol20029Ool001Yoo`8003Ioo`03001oogoo08Uoo`006Woo00<007ooOol0=7oo00<007ooOol0
RWoo000JOol00`00Oomoo`0bOol2002=Ool001Yoo`03001oogoo035oo`03001oogoo08eoo`000Woo
0P0017oo0P0017oo0P0017oo0`000goo00<007ooOol0<7oo00<007ooOol0SWoo00001Goo001oogoo
00000goo0P000goo00@007ooOol000Aoo`03001oogoo009oo`03001oogoo02ioo`80095oo`0000Eo
o`00Oomoo`0000Qoo`04001oogoo0002Ool40003Ool2000^Ool00`00Oomoo`2AOol00005Ool007oo
Ool00008Ool01000Oomoo`000Woo00<007oo000017oo00<007ooOol0;7oo00<007ooOol0TWoo0000
1Goo001oogoo000027oo00@007ooOol0009oo`03001oo`0000Aoo`03001oogoo02]oo`03001oogoo
09=oo`000Woo0P002Woo0P0017oo0P0017oo00<007ooOol0:goo00<007ooOol0Tgoo000JOol00`00
Oomoo`0ZOol00`00Oomoo`2DOol001Yoo`03001oogoo02Uoo`03001oogoo09Eoo`006Woo0P00:7oo
0P00V7oo000JOol00`00Oomoo`0VOol00`00Oomoo`2HOol001Yoo`03001oogoo02Eoo`03001oogoo
09Uoo`006Woo00<007ooOol097oo00<007ooOol0VWoo000JOol00`00Oomoo`0SOol00`00Oomoo`2K
Ool001Yoo`03001oogoo029oo`03001oogoo09aoo`006Woo0P008Woo00<007ooOol0WGoo000JOol0
0`00Oomoo`0OOol2002POol001Yoo`03001oogoo01ioo`03001oogoo0:1oo`006Woo00<007ooOol0
7Goo00<007ooOol0XGoo000JOol00`00Oomoo`0LOol00`00Oomoo`2ROol001Yoo`03001oogoo01]o
o`03001oogoo0:=oo`006Woo0P006goo00<007ooOol0Y7oo000JOol00`00Oomoo`0IOol00`00Oomo
o`2UOol001Yoo`03001oogoo01Qoo`03001oogoo0:Ioo`006Woo00<007ooOol067oo00<007ooOol0
YWoo0002Ool20004Ool20004Ool20004Ool20004Ool00`00Oomoo`0GOol00`00Oomoo`2WOol00005
Ool007ooOol00003Ool20003Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oogoo01Ioo`03
001oogoo0:Qoo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool01000Oomoo`000goo0P005Woo
00<007ooOol0ZGoo00001Goo001oogoo000027oo00@007ooOol0009oo`<000Aoo`03001oogoo01Ao
o`03001oogoo0:Yoo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool00`00Oomoo`04Ool00`00
Oomoo`0COol00`00Oomoo`2[Ool0009oo`8000Yoo`8000Aoo`<000=oo`03001oogoo019oo`03001o
ogoo0:aoo`006Woo00<007ooOol04Goo00<007ooOol0[Goo000JOol00`00Oomoo`0@Ool00`00Oomo
o`2^Ool001Yoo`80011oo`03001oogoo0:moo`006Woo00<007ooOol03Woo00<007ooOol0/7oo000J
Ool00`00Oomoo`0=Ool00`00Oomoo`2aOol001Yoo`03001oogoo00aoo`03001oogoo0;9oo`006Woo
00<007ooOol037oo00<007ooOol0/Woo000JOol00`00Oomoo`0;Ool00`00Oomoo`2cOol001Yoo`80
00]oo`03001oogoo0;Aoo`006Woo00<007ooOol02Goo00<007ooOol0]Goo000JOol00`00Oomoo`08
Ool00`00Oomoo`2fOol001Yoo`03001oogoo00Moo`03001oogoo0;Moo`006Woo00<007ooOol01Woo
00<007ooOol0^7oo000JOol00`00Oomoo`05Ool00`00Oomoo`2iOol001Yoo`8000Ioo`03001oogoo
0;Uoo`006Woo00<007ooOol017oo00<007ooOol0^Woo000JOol00`00Oomoo`03Ool00`00Oomoo`2k
Ool0009oo`8000Aoo`8000Aoo`8000Aoo`8000Aoo`03001oogoo009oo`03001oogoo0;aoo`0000Eo
o`00Oomoo`0000=oo`8000=oo`04001oogoo0002Ool01000Oomoo`000goo00D007ooOomoo`000;mo
o`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool01000Oomoo`000goo0P000Woo00<007ooOol0
_Goo00001Goo001oogoo000027oo00@007ooOol000=oo`8000Aoo`04001oogoo0030Ool00005Ool0
07ooOol00008Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oo`000<5oo`000Woo0P002Woo
0P0017oo0P0017oo0P00`Woo000JOol00`00Oomoo`31Ool001Yoo`03001oogoo0<5oo`006Woo0P00
`Woo000JOol00`00Oomoo`31Ool00=ioo`00gWoo003NOol00=ioo`00gWoo003NOol00001\
\>"],
  ImageRangeCache->{{{0, 221.938}, {136.438, 0}} -> {7.44075, -0.0245674, 
  0.0211139, 0.000847737}}]
}, Open  ]],

Cell["\<\
As a reminder, here is the corresponding plot for the \
Crank-Nicholson scheme.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.88095 0.238095 0.111758 5.93003 [
[.2619 .09926 -3 -9 ]
[.2619 .09926 3 0 ]
[.5 .09926 -6 -9 ]
[.5 .09926 6 0 ]
[.7381 .09926 -6 -9 ]
[.7381 .09926 6 0 ]
[.97619 .09926 -6 -9 ]
[.97619 .09926 6 0 ]
[.01131 .23036 -24 -4.5 ]
[.01131 .23036 0 4.5 ]
[.01131 .34896 -24 -4.5 ]
[.01131 .34896 0 4.5 ]
[.01131 .46756 -24 -4.5 ]
[.01131 .46756 0 4.5 ]
[.01131 .58616 -24 -4.5 ]
[.01131 .58616 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .11176 m
.2619 .11801 L
s
[(9)] .2619 .09926 0 1 Mshowa
.5 .11176 m
.5 .11801 L
s
[(10)] .5 .09926 0 1 Mshowa
.7381 .11176 m
.7381 .11801 L
s
[(11)] .7381 .09926 0 1 Mshowa
.97619 .11176 m
.97619 .11801 L
s
[(12)] .97619 .09926 0 1 Mshowa
.125 Mabswid
.07143 .11176 m
.07143 .11551 L
s
.11905 .11176 m
.11905 .11551 L
s
.16667 .11176 m
.16667 .11551 L
s
.21429 .11176 m
.21429 .11551 L
s
.30952 .11176 m
.30952 .11551 L
s
.35714 .11176 m
.35714 .11551 L
s
.40476 .11176 m
.40476 .11551 L
s
.45238 .11176 m
.45238 .11551 L
s
.54762 .11176 m
.54762 .11551 L
s
.59524 .11176 m
.59524 .11551 L
s
.64286 .11176 m
.64286 .11551 L
s
.69048 .11176 m
.69048 .11551 L
s
.78571 .11176 m
.78571 .11551 L
s
.83333 .11176 m
.83333 .11551 L
s
.88095 .11176 m
.88095 .11551 L
s
.92857 .11176 m
.92857 .11551 L
s
.25 Mabswid
0 .11176 m
1 .11176 L
s
.02381 .23036 m
.03006 .23036 L
s
[(0.02)] .01131 .23036 1 0 Mshowa
.02381 .34896 m
.03006 .34896 L
s
[(0.04)] .01131 .34896 1 0 Mshowa
.02381 .46756 m
.03006 .46756 L
s
[(0.06)] .01131 .46756 1 0 Mshowa
.02381 .58616 m
.03006 .58616 L
s
[(0.08)] .01131 .58616 1 0 Mshowa
.125 Mabswid
.02381 .14141 m
.02756 .14141 L
s
.02381 .17106 m
.02756 .17106 L
s
.02381 .20071 m
.02756 .20071 L
s
.02381 .26001 m
.02756 .26001 L
s
.02381 .28966 m
.02756 .28966 L
s
.02381 .31931 m
.02756 .31931 L
s
.02381 .37861 m
.02756 .37861 L
s
.02381 .40826 m
.02756 .40826 L
s
.02381 .43791 m
.02756 .43791 L
s
.02381 .49721 m
.02756 .49721 L
s
.02381 .52686 m
.02756 .52686 L
s
.02381 .55651 m
.02756 .55651 L
s
.02381 .08211 m
.02756 .08211 L
s
.02381 .05246 m
.02756 .05246 L
s
.02381 .02281 m
.02756 .02281 L
s
.02381 .61581 m
.02756 .61581 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
.5 Mabswid
.02381 .60281 m
.04273 .58001 L
.06337 .55582 L
.08275 .5338 L
.10139 .5132 L
.12123 .49184 L
.14033 .4718 L
.16063 .45087 L
.18019 .43079 L
.199 .41161 L
.21902 .39153 L
.23829 .37182 L
.25683 .35392 L
.27656 .33695 L
.29555 .3238 L
.3052 .318 L
.31574 .31234 L
.32062 .30995 L
.3233 .3087 L
.32583 .30756 L
.32697 .30706 L
.3276 .30679 L
.32819 .30666 L
.32882 .30675 L
.32951 .30685 L
.33074 .30703 L
.33519 .30769 L
.33764 .30803 L
.34031 .30838 L
.34266 .30863 L
.34395 .30875 L
.34516 .30884 L
.34624 .3089 L
.34679 .30893 L
.3474 .30895 L
.34803 .30897 L
.34837 .30898 L
.34869 .30898 L
.34927 .30899 L
.34988 .30898 L
.35044 .30898 L
.35106 .30896 L
.35163 .30894 L
.35217 .30892 L
.35318 .30885 L
.35425 .30876 L
.35549 .30863 L
.35666 .30846 L
.35887 .30806 L
.36124 .30748 L
Mistroke
.36383 .30665 L
.36849 .30461 L
.3728 .30202 L
.37799 .29789 L
.38054 .29543 L
.38165 .29425 L
.38288 .29289 L
.38341 .29228 L
.38398 .29138 L
.38452 .28954 L
.38501 .28785 L
.38726 .27997 L
.392 .26282 L
.41154 .1903 L
.42048 .16042 L
.42557 .1455 L
.43034 .13329 L
.4354 .12255 L
.43762 .11866 L
.43887 .11669 L
.44004 .11502 L
.44064 .11421 L
.44121 .1135 L
.44152 .1144 L
.44186 .11541 L
.44246 .11721 L
.445 .12532 L
.45034 .14425 L
.4704 .2276 L
.48039 .26837 L
.48973 .30097 L
.49433 .31405 L
.4966 .31959 L
.49869 .32415 L
.4993 .32534 L
.49986 .32642 L
.50045 .32531 L
.50076 .32434 L
.5011 .32327 L
.50372 .31453 L
.50831 .2973 L
.52845 .20254 L
.53849 .1508 L
.54784 .10466 L
.55247 .0835 L
.55685 .06499 L
.55803 .0603 L
.55927 .05548 L
.55992 .05303 L
.56029 .05172 L
Mistroke
.56064 .05174 L
.56124 .05181 L
.56159 .05186 L
.5619 .05192 L
.56302 .05221 L
.56366 .05242 L
.56425 .05265 L
.56649 .05381 L
.56898 .0556 L
.57167 .05807 L
.57655 .06384 L
.58566 .07824 L
.60576 .11894 L
.61059 .12904 L
.61577 .13946 L
.61828 .14427 L
.61951 .14656 L
.62065 .14864 L
.62169 .15048 L
.62221 .15115 L
.62279 .15111 L
.62336 .15105 L
.62398 .15099 L
.62511 .15082 L
.62737 .15037 L
.62984 .14967 L
.63482 .14768 L
.63943 .14521 L
.64371 .14243 L
.66285 .12578 L
.67261 .11565 L
.67775 .11015 L
.68318 .10433 L
.68804 .10026 L
.69338 .09691 L
.70278 .09129 L
.72254 .0806 L
.74157 .07176 L
.7618 .06431 L
.78128 .05869 L
.80002 .05407 L
.81997 .04956 L
.83917 .04497 L
.85957 .04025 L
.87923 .03584 L
.89815 .03158 L
.91827 .02711 L
.93764 .02294 L
.95628 .01909 L
.97611 .01522 L
Mistroke
.97619 .01521 L
Mfstroke
.02381 .60332 m
.04273 .58045 L
.06337 .55612 L
.08275 .53387 L
.10139 .51301 L
.12123 .49138 L
.14033 .47112 L
.16063 .45018 L
.18019 .43058 L
.199 .41225 L
.21902 .39331 L
.23829 .37562 L
.25683 .35911 L
.27656 .34205 L
.29555 .32614 L
.31574 .30976 L
.33519 .29449 L
.3539 .28027 L
.3738 .26564 L
.39297 .25203 L
.41334 .23806 L
.43296 .22508 L
.45184 .21303 L
.47193 .20066 L
.49127 .18919 L
.50987 .17856 L
.52967 .16765 L
.54872 .15755 L
.56898 .14723 L
.5885 .13768 L
.60727 .12885 L
.62725 .11983 L
.64648 .11151 L
.66497 .10383 L
.68466 .09598 L
.70361 .08875 L
.72376 .0814 L
.74317 .07464 L
.76183 .06842 L
.7817 .0621 L
.80082 .0563 L
.82115 .05043 L
.84073 .04506 L
.85957 .04014 L
.87961 .03516 L
.89891 .03062 L
.91747 .02647 L
.93723 .02229 L
.95624 .01849 L
.97619 .01472 L
Mistroke
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{222.938, 137.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004F0000Za000`40O003h00Oogoo5goo003oOolG
Ool00?mooaMoo`00ogoo5goo003oOolGOol00?mooaMoo`00ogoo5goo003oOolGOol0025oo`03001o
ogoo0?9oo`008Goo00<007ooOol0lWoo000QOol00`00Oomoo`3bOol0025oo`03001oogoo0?9oo`00
8Goo00<007ooOol0j7oo0`001goo000QOol00`00Oomoo`3SOol5000:Ool0025oo`800=moo`D000mo
o`008Goo00<007ooOol0fGoo1@0057oo000QOol00`00Oomoo`3DOol5000IOol0025oo`03001oogoo
0<moo`D001ioo`008Goo00<007ooOol0bgoo1@008Woo000QOol00`00Oomoo`37Ool4000WOol0025o
o`03001oogoo0<Aoo`<002]oo`008Goo0P00Pgoo0P00?Goo0`00;Woo000QOol00`00Oomoo`22Ool0
0`00Ool0000gOol5000aOol0025oo`03001oogoo089oo`04001oogoo000aOol6000eOol0025oo`03
001oogoo03Eoo`<003Eoo`D000=oo`<000Uoo`03001oogoo009oo`03001oogoo021oo`D000=oo`T0
02Yoo`D0009oo`D0009oo`008Goo00<007ooOol0>7oo00<007ooOol0=7oo00<007ooOol00Woo00D0
07ooOomoo`0000Qoo`03001oogoo009oo`03001oogoo029oo`03001oogoo009oo`L002moo`03001o
ogoo00=oo`04001oogoo0002Ool0025oo`03001oogoo03Uoo`03001oogoo03=oo`03001oogoo009o
o`05001oogooOol00008Ool00`00Oomoo`03Ool00`00Oomoo`0QOol00`00Oomoo`02000017oo0000
0000<Woo00<007ooOol017oo00<007ooOol00Woo000QOol00`00Oomoo`0fOol4000eOol00`00Oomo
o`02Ool01@00Oomoogoo00001goo00<007ooOol017oo00<007ooOol08Goo1P000Woo00<007ooOol0
<7oo00<007ooOol01Goo00<007ooOol00Goo000QOol00`00Oomoo`0eOol01@00Oomoogoo0000=Goo
00<007ooOol00Woo00D007ooOomoo`0000Moo`03001oogoo00Eoo`03001oogoo01moo`@000Eoo`03
001oogoo031oo`03001oogoo00Ioo`40005oo`5oo`008Goo0P00=Woo00D007ooOomoo`0003Eoo`03
001oogoo009oo`05001oogooOol00007Ool00`00Oomoo`05Ool00`00Oomoo`0LOol300000goo001o
o`06Ool00`00Oomoo`0`Ool00`00Oomoo`06Ool10001Ool1Ool0025oo`03001oogoo03Eoo`05001o
ogooOol0000eOol00`00Oomoo`02Ool01@00Oomoogoo00001goo00<007ooOol01Woo00<007ooOol0
6Goo0`000goo00<007ooOol01Goo00<007ooOol0<7oo00<007ooOol00Woo00D007ooOomoo`00009o
o`008Goo00<007ooOol0=Woo0`00=7oo0`001Goo0`0027oo00<007ooOol01Woo00<007ooOol05Woo
1@000Woo0`001Goo0`00<7oo0`001Goo0`000goo000QOol00`00Oomoo`1oOol00`00Oomoo`08Ool0
0`00Oomoo`0COol200000goo0000001=Ool0025oo`03001oogoo07moo`03001oogoo00Qoo`03001o
ogoo011oo`<0009oo`03001oogoo04eoo`008Goo00<007ooOol0Ogoo00<007ooOol02Goo00<007oo
Ool03Goo0P0017oo00<007ooOol0CWoo000QOol00`00Oomoo`1oOol00`00Oomoo`09Ool00`00Oomo
o`0;Ool20005Ool00`00Oomoo`1?Ool001]oooX0005oo`008Goo00<007ooOol027oo00<007ooOol0
2Goo00<007ooOol02Goo00<007ooOol02Goo00<007ooOol02Goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol027oo00<007ooOol01Woo0P0000=oo`00Ool02Woo00<007ooOol02Goo00<007oo
Ool02Goo0P001goo10001Woo00<007ooOol00Woo00<007ooOol027oo00<007ooOol02Goo00<007oo
Ool02Goo00<007ooOol02Goo00<007ooOol02Goo00<007ooOol02Goo00<007ooOol01Goo000QOol0
0`00Oomoo`0hOol00`00Oomoo`0YOol2000=Ool00`00Oomoo`08Ool00`00Oomoo`0<Ool00`00Oomo
o`02Ool2000:Ool00`00Oomoo`0=Ool00`00Oomoo`0iOol00`00Oomoo`05Ool0025oo`03001oogoo
06=oo`03001oo`0001Qoo`03001oogoo00eoo`@000]oo`03001oogoo059oo`008Goo00<007ooOol0
Hgoo00<007oo000067oo00<007ooOol037oo0`002goo0P00EGoo000QOol00`00Oomoo`1ROol01@00
Oomoogoo00005goo00<007ooOol02Goo0`000Woo00<007ooOol027oo00<007ooOol0EGoo000QOol0
0`00Oomoo`1ROol01@00Oomoogoo00005goo00<007ooOol01goo0P001Woo00<007ooOol01Woo00<0
07ooOol0EWoo000QOol2001ROol00`00Oomoo`02Ool00`00Oomoo`0EOol00`00Oomoo`05Ool20008
Ool00`00Oomoo`05Ool00`00Oomoo`1GOol0025oo`03001oogoo065oo`03001oogoo00=oo`03001o
ogoo01=oo`03001oogoo00Eoo`03001oogoo00Uoo`03001oogoo009oo`8005Yoo`008Goo00<007oo
Ool0HGoo00<007ooOol00goo00<007ooOol04goo00<007ooOol00goo0P0037oo00<007oo00000P00
G7oo000QOol00`00Oomoo`1POol00`00Oomoo`04Ool00`00Oomoo`0COol00`00Oomoo`03000>Ool3
001NOol0025oo`03001oogoo061oo`03001oogoo00Eoo`03001oogoo019oo`<0079oo`008Goo00<0
07ooOol0Ggoo00<007ooOol01Woo00<007ooOol04Woo00<007ooOol0LWoo000QOol00`00Oomoo`1O
Ool00`00Oomoo`06Ool00`00Oomoo`0AOol00`00Oomoo`1cOol0025oo`03001oogoo05moo`03001o
ogoo00Ioo`03001oogoo00moo`<007Eoo`008Goo0P00Ggoo00<007ooOol027oo00<007ooOol037oo
0P000Woo00<007ooOol0Lgoo000QOol00`00Oomoo`1NOol00`00Oomoo`08Ool00`00Oomoo`0;Ool0
0`00Oomoo`02Ool00`00Oomoo`1cOol0025oo`03001oogoo05ioo`03001oogoo00Qoo`03001oogoo
00Uoo`8000Aoo`03001oogoo07Aoo`008Goo00<007ooOol0GWoo00<007ooOol027oo00<007ooOol0
1Woo0`001Woo00<007ooOol0M7oo000QOol00`00Oomoo`1MOol00`00Oomoo`0:Ool00`00Oomoo`03
Ool20009Ool00`00Oomoo`1dOol0025oo`03001oogoo05eoo`03001oogoo00Yoo`04001oogooOol2
000;Ool00`00Oomoo`1dOol0025oo`03001oogoo05eoo`03001oogoo00Yoo`04001oogoo000=Ool0
0`00Oomoo`1dOol0025oo`8005ioo`03001oogoo00Yoo`<000eoo`03001oogoo07Eoo`008Goo00<0
07ooOol0G7oo00<007ooOol02Woo0P003goo00<007ooOol0MGoo000QOol00`00Oomoo`1LOol00`00
Oomoo`09Ool01000Oomoo`003Woo00<007ooOol0MGoo000QOol00`00Oomoo`1LOol00`00Oomoo`07
Ool20003Ool00`00Oomoo`0<Ool00`00Oomoo`1eOol0025oo`03001oogoo05]oo`03001oogoo00Io
o`8000Eoo`03001oogoo00]oo`03001oogoo07Ioo`000`001Woo0P001Goo0`000goo1@001Woo00<0
07ooOol0Fgoo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol02goo00<007ooOol0MWoo0003
Ool00`00Oomoo`03Ool20004Ool01@00Oomoogoo00000goo00@007ooOol000Ioo`03001oogoo05]o
o`03001oogoo00=oo`8000Uoo`03001oogoo00Yoo`03001oogoo07Ioo`000goo00<007ooOol02Goo
00D007ooOomoo`0000Aoo`03001oogoo00Ioo`03001oogoo05]oo`03001oogoo009oo`03001oogoo
00Uoo`03001oogoo00Yoo`03001oogoo07Ioo`000goo00<007ooOol02Goo00D007ooOomoo`0000Eo
o`03001oogoo00Eoo`8005]oo`03001oogoo009oo`03001oogoo00Yoo`03001oogoo00Yoo`03001o
ogoo07Ioo`000goo00<007ooOol02Goo00D007ooOomoo`0000Ioo`03001oogoo00Aoo`03001oogoo
05Yoo`03001oogoo008000eoo`03001oogoo00Uoo`03001oogoo07Moo`000goo00<007ooOol02Goo
00D007ooOomoo`0000Ioo`03001oogoo00Aoo`03001oogoo05Yoo`03001oo`00011oo`03001oogoo
00Qoo`03001oogoo07Moo`000goo00<007ooOol02Goo00D007ooOomoo`00009oo`05001oogooOol0
0006Ool00`00Oomoo`1JOol2000AOol00`00Oomoo`08Ool00`00Oomoo`1gOol000<000eoo`<000Ao
o`<000Moo`03001oogoo05Uoo`03001oogoo015oo`03001oogoo00Qoo`03001oogoo07Moo`008Goo
00<007ooOol0Egoo0`004goo00<007ooOol01goo00<007ooOol0N7oo000QOol00`00Oomoo`1FOol0
1000Oomoo`004goo00<007ooOol01goo00<007ooOol0N7oo000QOol2001FOol01000Oomoo`005Goo
00<007ooOol01Woo00<007ooOol0N7oo000QOol00`00Oomoo`1COol20003Ool00`00Oomoo`0COol0
0`00Oomoo`06Ool00`00Oomoo`1hOol0025oo`03001oogoo059oo`03001oogoo00=oo`03001oogoo
01=oo`03001oogoo00Ioo`03001oogoo07Qoo`008Goo00<007ooOol0DGoo00<007ooOol017oo00<0
07ooOol04goo00<007ooOol01Goo00<007ooOol0NGoo000QOol00`00Oomoo`1@Ool00`00Oomoo`04
Ool00`00Oomoo`0EOol00`00Oomoo`04Ool00`00Oomoo`1iOol0025oo`03001oogoo04ioo`8000Mo
o`03001oogoo01Eoo`03001oogoo00Aoo`03001oogoo07Uoo`008Goo00<007ooOol0CGoo00<007oo
Ool01goo00<007ooOol05Goo00<007ooOol017oo00<007ooOol0NGoo000QOol2001=Ool00`00Oomo
o`07Ool2000HOol00`00Oomoo`03Ool00`00Oomoo`1iOol0025oo`03001oogoo04]oo`03001oogoo
00Qoo`03001oogoo01Moo`03001oogoo009oo`03001oogoo07Yoo`008Goo00<007ooOol0BGoo0P00
2Goo0P006Woo00<007ooOol00Woo00<007ooOol0NWoo000QOol00`00Oomoo`17Ool2000:Ool00`00
Oomoo`0KOol01@00Oomoogoo0000O7oo000QOol00`00Oomoo`16Ool00`00Oomoo`09Ool00`00Oomo
o`0LOol01@00Oomoogoo0000O7oo000QOol00`00Oomoo`14Ool>000POol00`00Ool0001mOol0025o
o`03001oogoo04=oo`<002aoo`03001oo`0007eoo`008Goo00<007ooOol0@Woo0P00;Woo00<007oo
0000OGoo000QOol20012Ool00`00Oomoo`0^Ool2001nOol0025oo`03001oogoo03moo`80039oo`03
001oogoo07aoo`008Goo00<007ooOol0?Woo0P00/Woo000QOol00`00Oomoo`0mOol2002cOol0025o
o`03001oogoo03]oo`<00;Aoo`000`001Woo0P001Goo0`001Goo0`001Woo00<007ooOol0>Woo00<0
07oo0000]Goo0003Ool00`00Oomoo`03Ool20004Ool01@00Oomoogoo00001Goo00<007ooOol01Goo
00<007ooOol0>Goo00<007oo0000]Woo0003Ool00`00Oomoo`09Ool01@00Oomoogoo00000Woo1@00
1Woo0P00>Goo00<007oo0000]goo0003Ool00`00Oomoo`09Ool01@00Oomoogoo00000Woo00@007oo
Ool000Moo`03001oogoo03Moo`03001oo`000;Qoo`000goo00<007ooOol02Goo00D007ooOomoo`00
00=oo`03001oo`0000Moo`03001oogoo03Ioo`03001oo`000;Uoo`000goo00<007ooOol02Goo00D0
07ooOomoo`0000=oo`03001oo`0000Moo`03001oogoo03Eoo`<00;Yoo`000goo00<007ooOol02Goo
00D007ooOomoo`0000Aoo`8000Moo`03001oogoo03Aoo`800;aoo`000`003Goo0`001Woo00<007oo
Ool01Goo00<007ooOol0<goo0P00_Goo000QOol00`00Oomoo`0bOol2002nOol0025oo`80039oo`80
0;moo`008Goo00<007ooOol0<7oo00<007ooOol0_goo000QOol00`00Oomoo`0_Ool00`00Oomoo`30
Ool0025oo`03001oogoo02ioo`03001oogoo0<5oo`008Goo00<007ooOol0;Goo00<007ooOol0`Woo
000QOol00`00Oomoo`0/Ool00`00Oomoo`33Ool0025oo`03001oogoo02]oo`03001oogoo0<Aoo`00
8Goo00<007ooOol0:Woo00<007ooOol0aGoo000QOol2000ZOol00`00Oomoo`36Ool0025oo`03001o
ogoo02Qoo`03001oogoo0<Moo`008Goo00<007ooOol09goo00<007ooOol0b7oo000QOol00`00Oomo
o`0VOol00`00Oomoo`39Ool0025oo`03001oogoo02Eoo`03001oogoo0<Yoo`008Goo00<007ooOol0
97oo00<007ooOol0bgoo000QOol00`00Oomoo`0SOol00`00Oomoo`3<Ool0025oo`8002=oo`03001o
ogoo0<eoo`008Goo00<007ooOol08Goo00<007ooOol0cWoo000QOol00`00Oomoo`0POol00`00Oomo
o`3?Ool0025oo`03001oogoo01moo`03001oogoo0=1oo`008Goo00<007ooOol07Woo0P00dWoo0003
0006Ool20005Ool30004Ool30007Ool00`00Oomoo`0MOol2003COol000=oo`03001oogoo00=oo`80
00Aoo`05001oogooOol00002Ool01@00Oomoogoo00001Woo00<007ooOol077oo0P00e7oo0003Ool0
0`00Oomoo`09Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000Ioo`03001oogoo01]oo`800=Eo
o`000goo00<007ooOol02Goo00D007ooOomoo`00009oo`05001oogooOol00006Ool2000KOol2003F
Ool000=oo`03001oogoo00Uoo`05001oogooOol00002Ool40007Ool00`00Oomoo`0JOol00`00Oomo
o`3EOol000=oo`03001oogoo00Uoo`05001oogooOol00002Ool00`00Oomoo`08Ool00`00Oomoo`0I
Ool00`00Oomoo`3FOol000=oo`03001oogoo00Uoo`05001oogooOol00003Ool00`00Oomoo`07Ool0
0`00Oomoo`0HOol00`00Oomoo`3GOol000<000eoo`<000Eoo`<000Ioo`03001oogoo01Moo`03001o
ogoo0=Qoo`008Goo00<007ooOol05Woo00<007ooOol0fGoo000QOol00`00Oomoo`0EOol00`00Oomo
o`3JOol0025oo`8001Eoo`03001oogoo0=]oo`008Goo00<007ooOol04goo00<007ooOol0g7oo000Q
Ool00`00Oomoo`0BOol00`00Oomoo`3MOol0025oo`03001oogoo015oo`03001oogoo0=ioo`008Goo
00<007ooOol047oo00<007ooOol0ggoo000QOol00`00Oomoo`0?Ool00`00Oomoo`3POol0025oo`03
001oogoo00ioo`03001oogoo0>5oo`008Goo0P003Woo00<007ooOol0hWoo000QOol00`00Oomoo`0<
Ool00`00Oomoo`3SOol0025oo`03001oogoo00]oo`03001oogoo0>Aoo`008Goo00<007ooOol02goo
00<007ooOol0i7oo000QOol00`00Oomoo`0:Ool00`00Oomoo`3UOol0025oo`03001oogoo00Uoo`03
001oogoo0>Ioo`008Goo00<007ooOol027oo00<007ooOol0igoo000QOol00`00Oomoo`07Ool00`00
Oomoo`3XOol0025oo`8000Moo`03001oogoo0>Uoo`008Goo00<007ooOol01Woo00<007ooOol0jGoo
000QOol00`00Oomoo`05Ool00`00Oomoo`3ZOol0025oo`03001oogoo00Aoo`03001oogoo0>]oo`00
8Goo00<007ooOol00goo00<007ooOol0k7oo00030006Ool20005Ool30004Ool30007Ool00`00Oomo
o`02Ool00`00Oomoo`3]Ool000=oo`03001oogoo00=oo`8000Aoo`05001oogooOol00002Ool01@00
Oomoogoo00001Woo00D007ooOomoo`000?1oo`000goo00<007ooOol02Goo00D007ooOomoo`00009o
o`05001oogooOol00006Ool20002Ool00`00Oomoo`3^Ool000=oo`03001oogoo00Uoo`05001oogoo
Ool00002Ool01@00Oomoogoo00001Woo00@007ooOol00?5oo`000goo00<007ooOol02Goo00D007oo
Oomoo`0000=oo`<000Moo`03001oo`000?9oo`000goo00<007ooOol02Goo00D007ooOomoo`00009o
o`05001oogooOol00006Ool2003cOol000=oo`03001oogoo00Uoo`05001oogooOol00002Ool01@00
Oomoogoo00001Woo00<007ooOol0lWoo0003000=Ool30004Ool30007Ool00`00Oomoo`3bOol0025o
o`03001oogoo0?9oo`008Goo0P00lgoo000QOol00`00Oomoo`3bOol00?mooaMoo`00ogoo5goo003o
OolGOol00?mooaMoo`00ogoo5goo003oOolGOol00?mooaMoo`00ogoo5goo0000\
\>"],
  ImageRangeCache->{{{0, 277.625}, {170.75, 0}} -> {7.44118, -0.0245873, 
  0.021096, 0.000847019}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["15.8 Summary", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The introduction of implicit schemes is motivated by the desire to get \
accurate and stable solutions for larger values of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\)]],
  ". When we need to compute the values of both a function and its first and \
second derivatives, popular schemes such as the ordinary two-time-level \
Crank-Nicholson scheme are unsuitable for option-pricing problems, unless \
small to moderate values of \[Alpha] are used. This is because of the fact \
that non-smoothness in initial data (almost always present in option payoffs) \
propagates through the solution causing small oscillatory errors. When \
amplified by the process of differentiation, these may cause substantial \
errors in the Greeks. The errors in the valuation may be deceptively small - \
to quote Wilmott ",
  StyleBox["et al",
    FontSlant->"Italic"],
  " (1993) when discussing a verification example with the Crank-Nicholson \
scheme, they remark that \"Even with ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 10\)]],
  ", the numerical and exact results differ only marginally.\" This is \
absolutely right, at least when we just look at the valuation. When we \
inspect the Greeks a rather more disturbing picture emerges. We have given an \
example where estimates of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Theta]\)]],
  " from a computation with ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 8\)]],
  " are in error by about 200%. \n\nThe oscillation problem and the resulting \
corruption of the Greeks can be cured by the use of a more suitable \
difference scheme. When the initial data were smooth, we saw in Chapter 14 \
that the two-time-level Douglas scheme was sufficient, but in the presence of \
non-smooth data the corresponding three-time-level Douglas scheme cures the \
problems quite dramatically. \n\nThe transition to such a scheme for \
practitioners should not be a great leap. We have already seen that the \
two-time-level Douglas scheme is the natural implicit generalization of the \
trinomial model, when carried out on a grid rather than a tree. The next step \
to a three-time-level scheme allows larger time steps still to be taken \
without a loss of accuracy or corruption of the Greeks. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 15 Bibliography", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "\nRichtmyer, R.D. and Morton, K.W., 1957, ",
  StyleBox["Difference Methods for Initial Value Problems",
    FontSlant->"Italic"],
  ", Krieger reprint, 1994, reprinted from the Wiley 1957/1967 original.\n \n\
Smith, G.D., 1985,  ",
  StyleBox[
  "Numerical Solution of Partial Differential Equations: Finite Difference \
Methods",
    FontSlant->"Italic"],
  ", Oxford University Press.\n\nWilmott, P., Dewynne, J. and Howison, S., \
1993, ",
  StyleBox["Option Pricing - Mathematical Models and Computation",
    FontSlant->"Italic"],
  ", Oxford Financial Press.\n"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{656, 641},
WindowMargins->{{4, Automatic}, {Automatic, 1}},
PrintingCopies->1,
PrintingStartingPageNumber->279,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"15. Finite-Differences and Black-Scholes"}]]}, {Cell[ 
        TextData[ {"15. Finite-Differences and Black-Scholes"}]], Inherited, 
      Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True,
"FirstPageFace"->Right},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
PrivateFontOptions->{"FontType"->"Outline"},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8^on3ohP==PTc0b85N`?N0080004/0B`000006G<AMP01
0000I00000400@4100800BL?00400@RoU?0000000000064/01T1T00000000000
00000000007oo@RoU^3oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 309, 10, 208, "Title"],
Cell[2043, 63, 268, 7, 60, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2336, 74, 113, 2, 49, "Section"],
Cell[2452, 78, 2644, 41, 359, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5133, 124, 173, 3, 48, "Section",
  PageBreakAbove->True],
Cell[5309, 129, 1252, 23, 133, "Text"],
Cell[6564, 154, 502, 8, 56, "NumberedEquation"],
Cell[7069, 164, 295, 7, 29, "Text"],
Cell[7367, 173, 411, 6, 20, "NumberedEquation"],
Cell[7781, 181, 399, 12, 20, "Text"],
Cell[8183, 195, 350, 8, 29, "Text"],

Cell[CellGroupData[{
Cell[8558, 207, 121, 1, 28, "Subsubsection"],
Cell[8682, 210, 957, 16, 188, "Input",
  PageBreakWithin->True],
Cell[9642, 228, 1162, 21, 150, "Input"],
Cell[10807, 251, 263, 7, 57, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11119, 264, 126, 1, 38, "Section"],
Cell[11248, 267, 607, 10, 62, "Text"],

Cell[CellGroupData[{
Cell[11880, 281, 115, 1, 32, "Subsection"],
Cell[11998, 284, 112, 1, 20, "Text"],
Cell[12113, 287, 289, 6, 32, "NumberedEquation"],
Cell[12405, 295, 574, 12, 52, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13016, 312, 155, 3, 32, "Subsection"],
Cell[13174, 317, 287, 9, 20, "Text"],
Cell[13464, 328, 520, 11, 48, "NumberedEquation",
  PageBreakWithin->True,
  InitializationCell->True],
Cell[13987, 341, 208, 6, 20, "Text"],
Cell[14198, 349, 549, 10, 109, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[14784, 364, 154, 3, 32, "Subsection"],
Cell[14941, 369, 163, 4, 20, "Text"],
Cell[15107, 375, 394, 9, 92, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[15538, 389, 163, 3, 32, "Subsection"],
Cell[15704, 394, 131, 1, 20, "Text"],
Cell[15838, 397, 182, 4, 25, "Input",
  InitializationCell->True],
Cell[16023, 403, 177, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[16225, 411, 139, 2, 25, "Input"],
Cell[16367, 415, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16458, 421, 153, 3, 28, "Subsubsection"],
Cell[16614, 426, 256, 5, 29, "Text"],
Cell[16873, 433, 466, 9, 106, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[17400, 449, 152, 3, 38, "Section"],
Cell[17555, 454, 169, 3, 38, "Input"],
Cell[17727, 459, 156, 3, 25, "Input"],

Cell[CellGroupData[{
Cell[17908, 466, 203, 6, 28, "Subsubsection"],
Cell[18114, 474, 466, 11, 62, "Input"],
Cell[18583, 487, 217, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[18825, 496, 169, 4, 25, "Input"],
Cell[18997, 502, 17792, 700, 144, 9828, 594, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[36838, 1208, 160, 3, 32, "Subsection"],
Cell[37001, 1213, 319, 8, 29, "Text"],
Cell[37323, 1223, 158, 3, 25, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37518, 1231, 172, 3, 32, "Subsection"],
Cell[37693, 1236, 234, 5, 37, "Input"],
Cell[37930, 1243, 197, 5, 25, "Input"],
Cell[38130, 1250, 270, 5, 81, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[38437, 1260, 137, 3, 32, "Subsection"],

Cell[CellGroupData[{
Cell[38599, 1267, 314, 7, 50, "Input"],
Cell[38916, 1276, 16039, 541, 144, 5827, 408, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[54992, 1822, 655, 13, 112, "Input"],
Cell[55650, 1837, 11537, 325, 193, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[67248, 2169, 158, 3, 38, "Section"],
Cell[67409, 2174, 155, 4, 20, "Text"],
Cell[67567, 2180, 755, 17, 165, "Input",
  PageBreakAbove->False],
Cell[68325, 2199, 121, 1, 20, "Text"],
Cell[68449, 2202, 385, 16, 156, "Input"],
Cell[68837, 2220, 126, 1, 20, "Text"],
Cell[68966, 2223, 197, 5, 36, "Input"],
Cell[69166, 2230, 173, 5, 25, "Input"],
Cell[69342, 2237, 244, 6, 46, "Input"],

Cell[CellGroupData[{
Cell[69611, 2247, 137, 3, 32, "Subsection"],

Cell[CellGroupData[{
Cell[69773, 2254, 284, 6, 50, "Input"],
Cell[70060, 2262, 17439, 556, 144, 5879, 407, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[87536, 2823, 591, 12, 88, "Input"],
Cell[88130, 2837, 11521, 325, 192, "Output"]
}, Open  ]],
Cell[99666, 3165, 421, 9, 40, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[100136, 3180, 144, 3, 38, "Section"],
Cell[100283, 3185, 251, 6, 20, "Text"],
Cell[100537, 3193, 136, 3, 20, "Text"],

Cell[CellGroupData[{
Cell[100698, 3200, 184, 5, 36, "Input"],
Cell[100885, 3207, 54, 1, 36, "Output"]
}, Open  ]],
Cell[100954, 3211, 561, 13, 100, "Input",
  PageBreakAbove->Automatic],
Cell[101518, 3226, 768, 15, 188, "Input"],
Cell[102289, 3243, 83, 1, 20, "Text"],
Cell[102375, 3246, 441, 16, 156, "Input"],

Cell[CellGroupData[{
Cell[102841, 3266, 140, 3, 32, "Subsection"],
Cell[102984, 3271, 193, 6, 36, "Input"],
Cell[103180, 3279, 173, 5, 25, "Input"],
Cell[103356, 3286, 239, 6, 46, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[103632, 3297, 137, 3, 32, "Subsection"],

Cell[CellGroupData[{
Cell[103794, 3304, 284, 6, 50, "Input"],
Cell[104081, 3312, 17956, 668, 144, 7851, 537, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[122074, 3985, 573, 11, 88, "Input"],
Cell[122650, 3998, 11522, 325, 192, "Output"]
}, Open  ]],
Cell[134187, 4326, 533, 10, 52, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[134757, 4341, 203, 6, 32, "Subsection"],
Cell[134963, 4349, 332, 6, 40, "Text"],
Cell[135298, 4357, 384, 11, 101, "Input"],
Cell[135685, 4370, 185, 6, 20, "Text"],
Cell[135873, 4378, 156, 3, 37, "Input"],
Cell[136032, 4383, 178, 4, 37, "Input"],
Cell[136213, 4389, 193, 4, 37, "Input"],
Cell[136409, 4395, 218, 4, 37, "Input"],
Cell[136630, 4401, 175, 5, 25, "Input"],
Cell[136808, 4408, 308, 5, 50, "Input"],
Cell[137119, 4415, 382, 9, 50, "Input"],
Cell[137504, 4426, 304, 6, 90, "Input"],
Cell[137811, 4434, 307, 6, 90, "Input"],
Cell[138121, 4442, 295, 6, 81, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[138453, 4453, 93, 1, 32, "Subsection"],
Cell[138549, 4456, 207, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[138781, 4464, 115, 2, 25, "Input"],
Cell[138899, 4468, 151, 2, 32, "Print"]
}, Open  ]],
Cell[139065, 4473, 192, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[139282, 4481, 398, 13, 101, "Input"],
Cell[139683, 4496, 11388, 323, 180, "Output"]
}, Open  ]],
Cell[151086, 4822, 221, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[151332, 4831, 114, 1, 28, "Subsubsection"],
Cell[151449, 4834, 160, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[151634, 4842, 333, 6, 50, "Input"],
Cell[151970, 4850, 10618, 398, 144, 4500, 317, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[162603, 5251, 121, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[162749, 5256, 313, 6, 62, "Input"],
Cell[163065, 5264, 19639, 663, 144, 7565, 509, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[182765, 5934, 93, 1, 32, "Subsection"],
Cell[182861, 5937, 188, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[183074, 5945, 395, 11, 80, "Input"],
Cell[183472, 5958, 11365, 323, 180, "Output"]
}, Open  ]],
Cell[194852, 6284, 179, 5, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[195056, 6293, 116, 2, 28, "Subsubsection",
  PageBreakAbove->True],
Cell[195175, 6297, 243, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[195443, 6306, 298, 5, 50, "Input",
  PageBreakAbove->False],
Cell[195744, 6313, 14847, 602, 136, 7376, 504, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[210652, 6922, 118, 2, 32, "Subsection",
  PageBreakAbove->False],
Cell[210773, 6926, 156, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[210954, 6934, 447, 14, 90, "Input",
  PageBreakAbove->False,
  PageBreakBelow->False],
Cell[211404, 6950, 11373, 323, 180, "Output"]
}, Open  ]],
Cell[222792, 7276, 184, 3, 20, "Text"],

Cell[CellGroupData[{
Cell[223001, 7283, 123, 2, 28, "Subsubsection"],

Cell[CellGroupData[{
Cell[223149, 7289, 309, 5, 50, "Input"],
Cell[223461, 7296, 14233, 508, 144, 5873, 399, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[237755, 7811, 111, 1, 32, "Subsection"],
Cell[237869, 7814, 438, 8, 40, "Text"],
Cell[238310, 7824, 23051, 794, 144, 9119, 617, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[261410, 8624, 182, 4, 38, "Section"],
Cell[261595, 8630, 436, 7, 52, "Text"],

Cell[CellGroupData[{
Cell[262056, 8641, 169, 4, 28, "Subsubsection"],
Cell[262228, 8647, 288, 8, 57, "Input"],
Cell[262519, 8657, 288, 8, 57, "Input"],
Cell[262810, 8667, 387, 10, 80, "Input"],
Cell[263200, 8679, 325, 9, 68, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[263562, 8693, 191, 4, 32, "Subsection"],
Cell[263756, 8699, 220, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[264001, 8708, 211, 11, 90, "Input"],
Cell[264215, 8721, 54, 1, 36, "Output"]
}, Open  ]],
Cell[264284, 8725, 486, 12, 62, "Input"],
Cell[264773, 8739, 170, 3, 25, "Input"],
Cell[264946, 8744, 150, 3, 25, "Input"],
Cell[265099, 8749, 447, 10, 80, "Input"],
Cell[265549, 8761, 387, 12, 101, "Input"],

Cell[CellGroupData[{
Cell[265961, 8777, 164, 4, 28, "Subsubsection"],
Cell[266128, 8783, 243, 7, 20, "Text"],
Cell[266374, 8792, 480, 16, 145, "Input"],
Cell[266857, 8810, 255, 7, 20, "Text"],
Cell[267115, 8819, 502, 15, 134, "Input"],
Cell[267620, 8836, 537, 17, 156, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[268194, 8858, 200, 4, 28, "Subsubsection"],
Cell[268397, 8864, 543, 20, 189, "Input"],
Cell[268943, 8886, 195, 4, 37, "Input"],
Cell[269141, 8892, 137, 3, 25, "Input"],
Cell[269281, 8897, 136, 3, 25, "Input"],
Cell[269420, 8902, 138, 3, 25, "Input"],
Cell[269561, 8907, 145, 2, 25, "Input"],
Cell[269709, 8911, 192, 6, 25, "Input"],
Cell[269904, 8919, 255, 7, 46, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[270208, 8932, 97, 1, 32, "Subsection"],

Cell[CellGroupData[{
Cell[270330, 8937, 253, 7, 46, "Input"],
Cell[270586, 8946, 17248, 588, 144, 6584, 450, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[287871, 9539, 350, 11, 101, "Input"],
Cell[288224, 9552, 11387, 323, 182, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[299660, 9881, 203, 6, 32, "Subsection"],
Cell[299866, 9889, 199, 4, 29, "Text"],
Cell[300068, 9895, 384, 11, 101, "Input"],
Cell[300455, 9908, 156, 3, 37, "Input"],
Cell[300614, 9913, 193, 4, 37, "Input"],
Cell[300810, 9919, 193, 4, 37, "Input"],
Cell[301006, 9925, 236, 4, 52, "Input"],
Cell[301245, 9931, 308, 5, 50, "Input"],
Cell[301556, 9938, 388, 9, 50, "Input"],
Cell[301947, 9949, 306, 6, 90, "Input"],
Cell[302256, 9957, 309, 6, 90, "Input"],
Cell[302568, 9965, 297, 6, 81, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[302902, 9976, 93, 1, 32, "Subsection"],
Cell[302998, 9979, 164, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[303187, 9987, 370, 12, 101, "Input"],
Cell[303560, 10001, 11380, 323, 180, "Output"]
}, Open  ]],
Cell[314955, 10327, 135, 3, 20, "Text"],

Cell[CellGroupData[{
Cell[315115, 10334, 93, 1, 28, "Subsubsection"],

Cell[CellGroupData[{
Cell[315233, 10339, 279, 4, 50, "Input"],
Cell[315515, 10345, 10649, 398, 144, 4496, 317, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[326225, 10750, 118, 2, 32, "Subsection",
  PageBreakAbove->False],
Cell[326346, 10754, 164, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[326535, 10762, 391, 12, 90, "Input",
  PageBreakAbove->False],
Cell[326929, 10776, 11382, 324, 180, "Output",
  PageBreakAbove->False]
}, Open  ]],
Cell[338326, 11103, 191, 5, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[338542, 11112, 139, 2, 28, "Subsubsection",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[338706, 11118, 279, 4, 50, "Input"],
Cell[338988, 11124, 11646, 438, 144, 4965, 350, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[350695, 11569, 118, 2, 33, "Subsection",
  PageBreakAbove->False],
Cell[350816, 11573, 164, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[351005, 11581, 395, 13, 77, "Input",
  PageBreakAbove->False],
Cell[351403, 11596, 11369, 323, 164, "Output"]
}, Open  ]],
Cell[362787, 11922, 240, 5, 33, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[363064, 11932, 115, 2, 33, "Subsection",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[363204, 11938, 279, 4, 40, "Input"],
Cell[363486, 11944, 10670, 380, 144, 4268, 296, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]],
Cell[374171, 12327, 185, 5, 20, "Text",
  PageBreakAbove->False],
Cell[374359, 12334, 16963, 541, 144, 5873, 399, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[391371, 12881, 88, 1, 42, "Section"],
Cell[391462, 12884, 2346, 41, 228, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[393845, 12930, 99, 1, 38, "Section"],
Cell[393947, 12933, 656, 16, 117, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

