(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    246497,       7476]*)
(*NotebookOutlinePosition[    248390,       7533]*)
(*  CellTagsIndexPosition[    248346,       7529]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 16.
SOR and PSOR Schemes for the Three-Time-Level Douglas Scheme 
and Application to American Options\
\>", "Title",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["16.1 Introduction", "Section",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The purpose of our finite-difference analysis is to define, \
implement, test and apply an accurate and efficient implicit \
finite-difference scheme for the valuation of derivatives with a long time to \
expiry. We consider, as our next example, American options with a long time \
to expiry. Other related instruments include convertible bonds (CBs).

In the previous two chapters I explained why Crank-Nicholson and other simple \
two-time-level schemes are unsuitable for the accurate modelling of \
option-pricing problems, due to their poor management of non-smooth payoffs, \
and the resulting errors in the Greeks arising from the propagation of errors \
from non-smooth points in the payoff. An example was given to justify the \
replacement of Crank-Nicholson methods with three-time-level Douglas \
finite-difference schemes (3TLD). This is justified for two reasons:

(a) a better truncation error (arising from the use of a Douglas-type \
scheme);
(b) a better decay of oscillations arising from non-smoothness of the payoff \
at the strike.

We have also made the point that schemes of the Douglas type are actually the \
natural implicit generalization of the trinomial model. So we shall use 3TLD \
from here on. As a reminder, we wish to use implicit finite-difference models \
for long-expiry options because, compared to explicit FD or binomial methods, \
they

(a) are just as accurate, but with a larger time-step;
(b) allow more straightforward and accurate computation of some of the \
Greeks.

Here we get this scheme going in a framework designed to allow the valuation \
of American options and CBs and other options with a long time to expiry. We \
test it by re-valuing European options, and we apply it by testing American \
option models with long times to expiry. Exactly the same methods can be used \
to look at simple convertible bonds. These are obtained from the model \
described here by making simple changes to expiry/boundary conditions and to \
payoff constraints. Otherwise the maths of the solution scheme is identical, \
though one needs to combine the management of discrete dividends (Chapter 24) \
with the form of the model suitable for modelling coupons and time-dependent \
interest rates, as discussed in Chapter 3. 

In this chapter we shall implement a solution method for European options \
(SOR, for successive over-relaxation) that is generalizable to American \
options (PSOR, or projected SOR). When this generalization is made some new \
issues arise regarding computation of Greeks in the neighbourhood of the \
moving boundary (this is just the critical stock price that gives the \
boundary between regions where early exercise is desirable and where it is \
not). The use of standard differencing formulae gives rise to problems, \
particularly in \[CapitalGamma], arising from the fact that \"standard\" \
formulae essentially smooth the Greeks over the boundary region. In fact, \
\[CapitalGamma] has a discontinuity at the critical price, and special \
techniques are needed to capture this. We expect this issue to arise whenever \
there is some type of boundary or barrier/trigger within a problem. We give \
some suggestions for fixing the problem and show how they work. Additional \
modifications to PSOR can also be used to treat embedded Puts and Calls in a \
CB model. 

We compare the results of the PSOR-3TLD model with the analytical \
approximation for the Carr-Jarrow-Myneni scheme given in Chapter 11, and \
demonstrate good consistency between our analytical and numerical methods for \
American options. Later in the book we shall also explore the corresponding \
binomial tree results. We emphasize that we regard the 3TLD model as vastly \
superior to binomial.

We wish to point out that the considerations of this chapter are intensively \
numerical. A fast machine is recommended!\
\>", "Text",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.2 Mathematical Reminders", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Transforming to the Diffusion Equation", "Subsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["The Black-Scholes differential equation", "Text",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(\ \[Sigma]\^2\)\/2\ S\^2\ \[PartialD]\^2 V\/\
\[PartialD]S\^2 - 
        r\ V + \((r - 
              q)\)\ S\ \[PartialD]V\/\[PartialD]S + \
\[PartialD]V\/\[PartialD]t = 0\)], "NumberedEquation",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
can be transformed into the diffusion equation by a standard change \
of variables, as given in Chapter 15. When all this is done, one obtains\
\>",
   "Text",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[PartialD]\^2 u\/\[PartialD]x\^2 = \[PartialD]u\/\
\[PartialD]\[Tau]\)], "NumberedEquation",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We now wish to solve this with appropriate modifications for early \
exercise.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["3TLD Reminder", "Subsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The diffusion equation (2) is solved by introducing a discrete grid with \
steps ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\ x\)]],
  " and \[CapitalDelta]\[Tau], and setting"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`u\_n\%m = 
      u(n\[CapitalDelta]\ x, m\[CapitalDelta]\[Tau])\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
A key role is played by the parameter \[Alpha], given by\
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Alpha] = \(\[CapitalDelta]\[Tau]\/\((\[CapitalDelta]\
\ x)\)\^2\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \[CapitalDelta]\ \
log\ S\  = \ \[Sigma] \@\(\( \[CapitalDelta]\ T\)\/\(2  \[Alpha]\)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In the previous chapter, we explored various schemes for solving for ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  ", and decided that the three-time-level Douglas scheme best satisfied the \
goals of accuracy, particularly around the strike, and eliminated problems \
such as oscillations that corrupted the Greeks. As in the previous chapter, \
the scheme we use is 3TLD:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\((1\/8\  - \ \[Alpha])\) \((u\_\(n - 1\)\%\(m + 1\) + 
              u\_\(n + 1\)\%\(m + 1\))\)\  + \ \((5\/4\  + \ 2  \[Alpha])\) 
          u\_n\%\(m + 1\)\[IndentingNewLine] = \(1\/6\) \((u\_\(n - 1\)\%m + 
              u\_\(n + 1\)\%m + 
              10  u\_n\%m)\)\  - \ \(1\/24\) \((u\_\(n - 1\)\%\(m - 1\) + 
              u\_\(n + 1\)\%\(m - 1\) + 10  u\_n\%\(m - 1\))\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This type of process requires a kick-off procedure. We use the \
ordinary Douglas two-time-level scheme to kick off the process. This is given \
by\[AutoOperand]\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\((1\  - \ 6  \[Alpha])\) \((u\_\(n - 1\)\%\(m + 1\) + 
              u\_\(n + 1\)\%\(m + 1\))\)\  + \ \((10\  + \ 12  \[Alpha])\) 
          u\_n\%\(m + 1\) = \((1\  + \ 6  \[Alpha])\) \((u\_\(n - 1\)\%m + 
              u\_\(n + 1\)\%m)\)\  + \((10 - 12  \[Alpha])\) 
          u\_n\%\(m\ \)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and is used once with \[Alpha] replaced by ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]/4\)]],
  ", followed by the three-time-level scheme with ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]/4\)]],
  " and then again with ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]/2\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.3 Detailed Modelling Studies", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Equation Solvers", "Subsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The solution of equations such as (5) and (6) requires a solver. \
Furthermore, when treating American options or CBs, we wish to solve \
differential inequalities related to equation (1), based on an associated \
constraint surface (the payoff). We therefore need a solution mechanism for \
(5) and (6) that allows this. The tridiagonal solver used in the previous \
chapter does not allow for early exercise problems, so we replace it by the \
successive over-relaxation (SOR) method, and for early exercise problems, or \
those with other constraints, the projected form (PSOR). For details of the \
SOR and PSOR methods in general, you should see Wilmott",
  StyleBox[" et al",
    FontSlant->"Italic"],
  " (1993). Although we are using rather novel difference schemes here, the \
solution methods of SOR and PSOR are otherwise identical to those of Wilmott",
  
  StyleBox[" et al",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compiled SOR and PSOR Solvers ", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The matrix equations resulting from (5) and (6) are symmetric and \
also have constant values down the diagonal and off-diagonal lines, so we can \
simplify matters in constructing the SOR/PSOR solvers: \
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(\(CompSORSolve\  = \ 
      Compile[\n\t\t\t{{diag, \ _Real, \ 0}, \ {offdiag, \ _Real, \ 
            0}, \ {r, \ _Real, \ 1}, \ {eps, \ _Real, \ 
            0}, \ {relax, \ _Real, \ 0}, {kickoff, \ _Real, \ 
            1}}, \n\t\t\tModule[{len\  = \ Length[r], \ yu\  = \ r, \ 
            yold\  = \ kickoff, \ ynew\  = \ r, \ 
            error\  = \ 100.0}, \n\t\t\t\t\tWhile[
            error\  > \ 
              eps^2, \n\t\((Do[\((yu[\([i]\)] = 
                    r[\([i]\)]/
                        diag - \((offdiag/
                            diag)\)*\((If[i == 1, \ 0, ynew[\([i - 1]\)]] + 
                            If[i < len, yold[\([i + 1]\)], 
                              0])\); \n\t\tynew[\([i]\)]\  = \ 
                    yold[\([i]\)]\  + \ 
                      relax*\((yu[\([i]\)]\  - \ yold[\([i]\)])\))\), \ {i, \ 
                  1, \ len}]; \n\t
              error\  = \ \((ynew - yold)\) . \((ynew - yold)\); \n\t
              yold\  = \ ynew)\)]; yold]]; \)\t\t\t\t\t\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We make one change to this routine to define the corresponding \
projected form suitable for valuing American options:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(CompPSORSolve\  = \ 
      Compile[\n\t\t\t{{diag, \ _Real, \ 0}, \ {offdiag, \ _Real, \ 
            0}, \ {r, \ _Real, \ 1}, \ {eps, \ _Real, \ 
            0}, \ {relax, \ _Real, \ 0}, {kickoff, \ _Real, \ 
            1}}, \n\t\t\tModule[{len\  = \ Length[r], \ yu\  = \ r, \ 
            yold\  = \ kickoff, \ ynew\  = \ r, \ 
            error\  = \ 100.0}, \n\t\t\t\t\tWhile[
            error\  > \ 
              eps^2, \n\t\((Do[\((yu[\([i]\)] = 
                    r[\([i]\)]/
                        diag - \((offdiag/
                            diag)\)*\((If[i == 1, \ 0, ynew[\([i - 1]\)]] + 
                            If[i < len, yold[\([i + 1]\)], 
                              0])\); \n\t\tynew[\([i]\)]\  = \ 
                    Max[kickoff[\([i]\)], \ 
                      yold[\([i]\)]\  + \ 
                        relax*\((yu[\([i]\)]\  - \ 
                              yold[\([i]\)])\)]\ \ )\), \ {i, \ 1, \ 
                  len}]; \n\t
              error\  = \ \((ynew - yold)\) . \((ynew - yold)\); \n\t
              yold\  = \ ynew)\)]; \tyold]]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Standardization of Variables", "Subsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Here we implement the standard transformations used to obtain the \
diffusion equation:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(\(NonDimExpiry[T_, sd_] := \(sd\^2\ T\)\/2; \)\), 
    \(\(kone[r_, sd_] := \(2  r\)\/sd\^2; \)\), 
    \(\(ktwo[r_, q_, sd_] := \(2\ \((r - q)\)\)\/sd\^2; \)\), 
    \(ValuationMultiplier[strike_, r_, q_, x_, tau_, sd_] := 
      strike\ Exp[\(-\(1\/2\)\)\ \((ktwo[r, q, sd] - 
                  1)\)\ x - \((1\/4\ \((ktwo[r, q, sd] - 1)\)\^2 + 
                  kone[r, sd])\)\ tau]\)}], "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initial (Expiry) Conditions", "Subsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(CallExercise[x_, r_, q_, sd_] := 
      Max[Exp[1\/2\ \((ktwo[r, q, sd] - 1)\)\ x]\ \((Exp[x] - 1)\), 
        0]; \)\), 
    \(\(PutExercise[x_, r_, q_, sd_] := 
      Max[Exp[1\/2\ \((ktwo[r, q, sd] - 1)\)\ x]\ \((1 - Exp[x])\), 
        0]; \)\)}], "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Black-Scholes Model for Verification", "Subsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
We load a Package built with the approximate analytical model \
described in Chapter 11. This model describes the American Put by an accurate \
analytical approximation.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Derivatives`AnalyticAmerican`\>"]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Derivatives`AnalyticAmerican`*\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
AmericanPutCJM      AmericanPutGammaCJM Front
AmericanPutDeltaCJM\
\>", "Print",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?AmericanPutCJM\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("AmericanPutCJM[price,strike,vol,r,t]  returns an APPROXIMATE value for \
a vanilla zero-dividend American Put according to an approximation of the \
Carr-Jarrow-Myneni integral equation."\)], "Print"]
}, Open  ]],

Cell["\<\
Note that this Package automatically loads the ordinary vanilla \
European options package:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?BlackScholesPut\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("BlackScholesPut[price, strike, vol, riskfree, divyield, expiry]  \
returns the Black-Scholes value of a vanilla European Put."\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Boundary Conditions and Payoff Constraints", "Subsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Since the most interesting basic early-exercise problem arises for \
the Put, with zero dividends, we focus on that problem. We define both the \
boundary conditions and the payoff constraint function. Note that these are \
expressed in terms of the variables of the diffusion equation.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Put (q=0) Boundary Conditions", "Subsubsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(g[x_, tau_, r_, q_, sd_] := 0; \)\), 
    \(\(f[x_, tau_, r_, q_, sd_] := 
      Exp[1\/2\ \((ktwo[r, q, sd] - 1)\)\ x + 
            1\/4\ \((\((ktwo[r, q, sd] - 1)\)\^2 + 4\ kone[r, sd])\)\ tau] - 
        Exp[1\/2\ \((ktwo[r, q, sd] + 1)\)\ x + 
            1\/4\ \((\((ktwo[r, q, sd] - 1)\)\^2 + 
                  4\ kone[r, sd])\)\ tau]; \)\)}], "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Payoff Constraint Function", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(PutPayoff[x_, \ tau_, \ r_, \ q_, \ sd_]\  := \ 
      Exp[\((\((ktwo[r, q, sd] - 1)\)^2/4\  + \ kone[r, sd])\)*tau]*\n
        Max[Exp[\((ktwo[r, q, sd] - 1)\)*x/2] \((1 - Exp[x])\), \ 0]; \)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.4 Douglas Three-Time-Level SOR Solution Functions", "Section",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->Automatic,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Necessary Functions", "Subsubsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell["\<\
The matrix used in solving the two-time-level Douglas system is \
characterized by its diagonal and off-diagonal entries:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    \(DougDiag[alpha_]\  := \ 10\  + \ 12  alpha; \n
    DougOffDiag[alpha_]\  := \ 1\  - \ 6  alpha; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
The matrix used in solving the three-time-level Douglas system is \
characterized by its own diagonal and off-diagonal entries:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    \(ThreeDougDiag[alpha_]\  := \ 5/4 + \ 2  alpha; \n
    ThreeDougOffDiag[alpha_]\  := \ 1/8\  - \ alpha; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
The right-hand side of our implicit difference scheme is \
constructed using the same matrices as before:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
DougDMatrix[alpha_, vec_List] := Module[{temp},
temp = (10 - 12*alpha)*vec + (1+6*alpha)*(RotateRight[vec] + \
RotateLeft[vec]);
temp[[1]] = Simplify[First[temp] - \.0c(1+6*alpha)*Last[vec]];
temp[[-1]] = Simplify[Last[temp] - (1 + 6*alpha)*First[vec]];
temp]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell["\<\
DougDDMatrix[vec_List] := Module[{temp},
temp = (10*vec + RotateRight[vec] + RotateLeft[vec]);
temp[[1]] = Simplify[First[temp] - Last[vec]];
temp[[-1]] = Simplify[Last[temp] - First[vec]];
temp/6]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.5 Verification of 3TLD-SOR Algorithm on European Options", "Section",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We quickly check that the replacement of the tridiagonal solver (used in \
the previous chapter) by the SOR solver makes no significant difference \
provided the tolerance and relaxation parameters are suitably set",
  StyleBox[
  ". Readers who wish to take this on trust can skip to Section 16.6 where we \
treat the American option, but should note that here we will show that we can \
use just 20 time-steps to deal accurately with a problem with five years to \
expiry. ",
    FontSlant->"Italic"]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
dx = 0.0125; dtau = 0.005; alpha = dtau/dx^2;
M=20; nminus = 320; nplus = 160; alpha\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(31.9999999999999973`\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "So we have a very large value of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 32\)]],
  ". Next we introduce the additional parameters for the SOR solver. A \
variety of experiments, not reported here, reveal that a good value for the \
relaxation parameter is about 1.1. The choice of tolerance parameter affects \
the accuracy and speed of solution. Here we are being fairly demanding on \
accuracy, and have set a value of ",
  Cell[BoxData[
      \(TraditionalForm\`10\^\(-6\)\)]],
  " at the price of speed. You are encouraged to explore the impact of \
changing this parameter for yourselves."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(relax\  = \ 1.1; \ntolerance\  = \ 0.000001; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(initial = 
      Table[PutExercise[\((k - 1 - nminus)\)\ dx, 0.05, 0, 0.2], \ {k, 
          nminus + nplus + 1}]; \n
    lower = Table[
        f[\(-nminus\)\ dx, \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \n
    upper = Table[
        g[\(+nplus\)\ dx, \ \ \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \)], "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["w = Table[0, {m, 1, 3},   {k, 1, nminus+nplus+1}];", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
vold = Take[initial, {2, -2}];
payoff = vold;\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
w[[1,1]] = f[-nminus*dx, dtau/4, 0.05, 0, 0.2];
w[[1, nminus+nplus+1]] = g[nplus*dx, dtau/4, 0.05, 0, 0.2];
w[[2,1]] = f[-nminus*dx, dtau/2, 0.05, 0, 0.2];
w[[2, nminus+nplus+1]] = g[nplus*dx, dtau/2, 0.05, 0, 0.2];
w[[3,1]] = f[-nminus*dx, dtau, 0.05, 0, 0.2];
w[[3, nminus+nplus+1]] = g[nplus*dx, dtau, 0.05, 0, 0.2];\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["First Kick-Off Component", "Subsubsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Simple Douglas with two time-levels and ",
  Cell[BoxData[
      \(TraditionalForm\`d\ t/4\)]],
  " time-step."
}], "Text",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
rhs = DougDMatrix[alpha/4, vold]+
Table[
Which[
k==1, (6*alpha/4+1)*lower[[1]] + (6*alpha/4-1)*w[[1, 1]],
k== nplus + nminus-1,  (6*alpha/4+1)*upper[[1]] +
(6*alpha/4-1)*w[[1, nplus+nminus+1]],
True, 0],
{k, 1, nplus + nminus-1}];
vnew = CompSORSolve[DougDiag[alpha/4], DougOffDiag[alpha/4], rhs, tolerance, \
relax, payoff];
w[[1]] = Join[{w[[1,1]]}, vnew, {w[[1,nplus+nminus+1]]}];
vvold = vold;
vold = vnew;\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell["\<\
Now we have two iterations of the three-time-level Douglas system, \
doubling the time-step at each stage:\
\>", "Text",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
rhs = DougDDMatrix[vold] - DougDDMatrix[vvold]/4 + 
Table[
Which[
k==1, (alpha/4-1/8)*w[[2, 1]] + w[[1,1]]/6 - lower[[1]]/24,
k==nplus + nminus-1,  (alpha/4-1/8)*w[[2, nplus+nminus+1]] + 
w[[1,nplus+nminus+1]]/6 - upper[[1]]/24,
True, 0],
{k, 1, nplus + nminus-1}];
vnew = CompSORSolve[ThreeDougDiag[alpha/4], ThreeDougOffDiag[alpha/4], rhs, \
tolerance, relax, payoff];
w[[2]] = Join[{w[[2,1]]}, vnew, {w[[2,nplus+nminus+1]]}];
vold = vnew;\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
rhs = DougDDMatrix[vold] - DougDDMatrix[vvold]/4 + 
Table[
Which[
k==1, (alpha/2-1/8)*w[[3, 1]] + w[[2,1]]/6 - lower[[1]]/24,
k==nplus + nminus-1,  (alpha/2-1/8)*w[[3, nplus+nminus+1]] + 
w[[2,nplus+nminus+1]]/6 - upper[[1]]/24,
True, 0],
{k, 1, nplus + nminus-1}];
vnew = CompSORSolve[ThreeDougDiag[alpha/2], ThreeDougOffDiag[alpha/2], rhs, \
tolerance, relax, payoff];
w[[3]] = Join[{w[[3,1]]}, vnew, {w[[3,nplus+nminus+1]]}];
wold = initial;
wvold = initial;
wnew = w[[3]];\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Main Block", "Subsubsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We evolve this using the SOR solver. You are warned that this method takes \
a lot longer than the tridiagonal solution method. It is not recommended as a \
working model for European options. Here we are carrying out a verification \
of the SOR solution method, before proceeding to the PSOR approach for \
American options. (In the following we have suppressed the output of ",
  StyleBox["Print",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " that shows progress.)"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
For[m=3, m<=M+1, m++,
(wvold = wold;
wold = wnew;
Print[m];
rhs = DougDDMatrix[Take[wold, {2, -2}]] -
DougDDMatrix[Take[wvold, {2, -2}]]/4 + 
Table[
Which[
k==1, 
(alpha-1/8)*lower[[m]] + lower[[m-1]]/6 - lower[[m-2]]/24,
k==nplus + nminus-1, 
(alpha-1/8)*upper[[m]] + upper[[m-1]]/6 - upper[[m-2]]/24,
True, 0],
{k, 1, nplus + nminus-1}];
temp = CompSORSolve[ThreeDougDiag[alpha], ThreeDougOffDiag[alpha], rhs, \
tolerance, relax, payoff];
wnew = Join[{lower[[m]]}, temp, {upper[[m]]}])
]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell["\<\
points = Table[(k-nminus-1)*dx,  {k, 1, nplus+nminus+1}];
finalstep = wnew;
prevstep = wold;
pprevstep = wvold;
interpoldata = Transpose[{points,finalstep}];
ufunc = Interpolation[interpoldata, InterpolationOrder -> 3];
Valuation[strike_, r_, q_, S_, T_, sd_] :=
ValuationMultiplier[strike, r, q, Log[S/strike], sd^2*T/2, sd]*
ufunc[Log[S/strike]]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Sample Errors in Valuation", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We now tabulate, for verification purposes, the computed and exact values \
and the absolute error. Note that this is a five-year option and we have only \
used 20 time-steps (",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 32\)]],
  ")."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
samples = TableForm[Join[{{\"S\", \"SOR\", \"Exact\", \"Absolute \
Error\"}},
Table[Map[PaddedForm[N[#], {5, 4}]&, {S, Valuation[10, 0.05, 0, S, 5, 0.2],
BlackScholesPut[S,10,0.2,0.05,0,5],
Valuation[10, 0.05, 0, S, 5, 0.2]-BlackScholesPut[S,10,0.2,0.05,0,5]}],
{S, 2, 16, 1}]]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"SOR\"\>", "\<\"Exact\"\>", 
            "\<\"Absolute Error\"\>"},
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 5.7889\"\>",
                5.7888876343647881,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 5.7886\"\>",
                5.7885809005343702,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0003\"\>",
                0.00030673383041790458,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 4.8012\"\>",
                4.8012405191326986,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 4.8005\"\>",
                4.800508718641141,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0007\"\>",
                0.00073180049155752158,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.8621\"\>",
                3.8621109098649762,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.8615\"\>",
                3.8615280263060248,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0006\"\>",
                0.00058288355895141919,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.0209\"\>",
                3.0209049923119577,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.0209\"\>",
                3.0208607031816879,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0000\"\>",
                4.4289130269881127*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 2.3105\"\>",
                2.3105133808922167,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 2.3108\"\>",
                2.310845529420245,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0003\"\>",
                -0.00033214852802831984,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.7383\"\>",
                1.7382760991448964,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.7387\"\>",
                1.738661710302452,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0004\"\>",
                -0.0003856111575555321,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.2930\"\>",
                1.2929701971891183,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.2932\"\>",
                1.2932190592768538,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0002\"\>",
                -0.00024886208773544638,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.9547\"\>",
                0.95469543714000937,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.9548\"\>",
                0.95478044388596572,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0001\"\>",
                -8.5006745956350827*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.7019\"\>",
                0.70188913566005906,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.7019\"\>",
                0.70186980510265284,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0000\"\>",
                1.9330557406216364*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.5150\"\>",
                0.51496916371732726,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.5149\"\>",
                0.51492057347081155,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0000\"\>",
                4.8590246515711755*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3777\"\>",
                0.37768361567030329,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3777\"\>",
                0.37766075385942499,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0000\"\>",
                2.2861810878294087*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2772\"\>",
                0.27723265491364063,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2773\"\>",
                0.27726230863712598,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0000\"\>",
                -2.9653723485345562*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2039\"\>",
                0.20385670814950765,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2039\"\>",
                0.2039428604577227,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0001\"\>",
                -8.6152308215042828*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1503\"\>",
                0.15026632859799396,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1504\"\>",
                0.15039922861977628,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0001\"\>",
                -0.0001329000217823173,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1111\"\>",
                0.11108856993948228,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1113\"\>",
                0.11125262488724719,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0002\"\>",
                -0.00016405494776490503,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell[TextData[{
  "So even with a large ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\)]],
  " we have obtained excellent results."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Construction and Verification of Interpolated Valuation and \
Greeks\
\>", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "In this section we define some functions to compute Delta, Gamma and Theta \
directly from the finite-difference grid. ",
  StyleBox["These are done with ordinary differencing methods.",
    FontSlant->"Italic"]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
listd[data_, step_] :=
Module[{dleft,dright,len},
len = Length[data];
dleft = (4*data[[2]]-3*data[[1]]-data[[3]])/(2*step);
dright = (3*data[[len]]-4*data[[len-1]]+data[[len-2]])/(2*step);
Join[{dleft}, Take[RotateLeft[data]-RotateRight[data], {2, -2}]/(2*step), \
{dright}]
]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(kt\  = \ ktwo[0.05, \ 0, \ 0.2]; \nko\  = \ kone[0.05, \ 0.2]; \)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(deltadata = 
      listd[finalstep, dx] - 1\/2\ \((kt - 1)\)\ finalstep; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(gammadata = 
      listd[deltadata, dx] - 1\/2\ \((kt + 1)\)\ deltadata; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(thetadata = \(3  finalstep - 4  prevstep + pprevstep\)\/\(2  dtau\) - \
\((1\/4\ \((kt - 1)\)\^2 + ko)\) finalstep; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(deltainterpoldata = Transpose[{points, deltadata}]; \)\), 
    \(\(gammainterpoldata = Transpose[{points, gammadata}]; \)\), 
    \(\(thetainterpoldata = Transpose[{points, thetadata}]; \)\)}], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[{
    \(dfunc = 
      Interpolation[deltainterpoldata, InterpolationOrder \[Rule] 3]; \n
    gfunc = Interpolation[gammainterpoldata, 
        InterpolationOrder \[Rule] 3]; \), 
    \(\(tfunc = 
      Interpolation[thetainterpoldata, InterpolationOrder \[Rule] 3]; \)\)}], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(SORDelta[strike_, r_, q_, S_, T_, 
        sd_] := \(1\/S\) \((ValuationMultiplier[strike, r, q, 
              Log[S\/strike], \(sd\^2\ T\)\/2, sd]\ dfunc[
              Log[S\/strike]])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(SORGamma[strike_, r_, q_, S_, T_, 
        sd_] := \(1\/S\^2\) \((ValuationMultiplier[strike, r, q, 
              Log[S\/strike], \(sd\^2\ T\)\/2, sd]\ gfunc[
              Log[S\/strike]])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(SORTheta[strike_, r_, q_, S_, T_, 
        sd_] := \(-\(1\/2\)\)\ sd\^2\ ValuationMultiplier[strike, r, q, 
          Log[S\/strike], \(sd\^2\ T\)\/2, sd]\ tfunc[Log[S\/strike]]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Table of Percentage Errors in Delta", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
deltasamples = TableForm[Join[{{\"S\", \"SOR Delta\", \"Exact Delta\
\", \"% Error\"}},
Table[Map[PaddedForm[N[#], {5, 4}]&, 
{S, SORDelta[10, 0.05, 0, S, 5, 0.2],
BlackScholesPutDelta[S,10,0.2,0.05,0,5],
100*(SORDelta[10,0.05,0,S,5,0.2]/BlackScholesPutDelta[S,10,0.2,0.05,0,5]-1)}],\

{S, 2, 16, 1}]]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"SOR Delta\"\>", "\<\"Exact Delta\"\>", 
            "\<\"% Error\"\>"},
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9972\"\>",
                -0.99715607968460407,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9976\"\>",
                -0.99757014495884477,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0415\"\>",
                -0.041507384351191501,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9718\"\>",
                -0.97177993217860348,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9719\"\>",
                -0.97190384659171136,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0127\"\>",
                -0.012749657647970203,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.8978\"\>",
                -0.89776941914511543,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.8973\"\>",
                -0.89729081174698944,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0533\"\>",
                0.053339161825816106,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.7791\"\>",
                -0.77906203968257315,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.7785\"\>",
                -0.77854854120728401,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0660\"\>",
                0.06595587148527926,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.6406\"\>",
                -0.64060776815953446,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.6404\"\>",
                -0.64043322578235118,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0273\"\>",
                0.027253797922499601,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.5058\"\>",
                -0.50584443284151548,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.5060\"\>",
                -0.50595424315123527,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0217\"\>",
                -0.021703604862743919,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.3881\"\>",
                -0.38813119866599943,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.3883\"\>",
                -0.38833561265254934,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0526\"\>",
                -0.052638485858580442,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2920\"\>",
                -0.29201338818805128,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2922\"\>",
                -0.2921788833062795,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0566\"\>",
                -0.056641710843541304,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2168\"\>",
                -0.21684321468771314,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.2169\"\>",
                -0.2169240328832196,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0373\"\>",
                -0.03725645076402273,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1597\"\>",
                -0.15968162814398001,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1597\"\>",
                -0.15968731335422298,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0036\"\>",
                -0.0035602140981416674,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1170\"\>",
                -0.1170029176905396,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1170\"\>",
                -0.11696283853778006,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0343\"\>",
                0.034266569844398909,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0855\"\>",
                -0.085512047229909793,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0855\"\>",
                -0.085454614659577766,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0672\"\>",
                0.06720827255592976,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0624\"\>",
                -0.062447222504542467,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0624\"\>",
                -0.062392087714616684,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0884\"\>",
                0.088368240181302404,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0456\"\>",
                -0.045626189184197169,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0456\"\>",
                -0.045583721879110306,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0932\"\>",
                0.09316331211279838,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0334\"\>",
                -0.033384303271296691,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0334\"\>",
                -0.033357895170051416,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0792\"\>",
                0.07916596988704061,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell["The error in Delta peaks at about 0.1%.", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Table of Percentage Errors in Gamma", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The corresponding table of values of Gamma reveals a maximum error \
of about 1.3% of the true value.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
gammasamples = TableForm[Join[{{\"S\", \"SOR Gamma\", \"Exact Gamma\
\", \"% Error\"}},
Table[Map[PaddedForm[N[#], {5, 4}]&, 
{S, SORGamma[10, 0.05, 0, S, 5, 0.2],
BlackScholesPutGamma[S,10,0.2,0.05,0,5],
100*(SORGamma[10,0.05,0,S,5,0.2]/BlackScholesPutGamma[S,10,0.2,0.05,0,5]-1)}],\

{S, 2, 16, 1}]]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"SOR Gamma\"\>", "\<\"Exact Gamma\"\>", 
            "\<\"% Error\"\>"},
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0085\"\>",
                0.0085132302634964817,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0085\"\>",
                0.0084564450905314295,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.6715\"\>",
                0.67150170499696848,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0474\"\>",
                0.047384934891746062,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0480\"\>",
                0.048026336475153919,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-1.3355\"\>",
                -1.3355205299485751,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0997\"\>",
                0.099708166144139534,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1000\"\>",
                0.10003582365214997,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.3275\"\>",
                -0.3275401711588577,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1332\"\>",
                0.13319014977712926,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1329\"\>",
                0.13291672718548236,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2057\"\>",
                0.20570969315647503,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1397\"\>",
                0.13974530238403524,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1394\"\>",
                0.13936745826081531,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2711\"\>",
                0.27111359275335811,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1276\"\>",
                0.12761671344929723,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1274\"\>",
                0.12742324276041067,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1518\"\>",
                0.15183312298081564,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1071\"\>",
                0.10711951973585455,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1071\"\>",
                0.10711068765967688,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0082\"\>",
                0.0082457468723706739,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0853\"\>",
                0.0852634225356209,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0853\"\>",
                0.085343981515403081,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0944\"\>",
                -0.094393275719917291,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0656\"\>",
                0.065581829206112469,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0657\"\>",
                0.065673835817759235,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1401\"\>",
                -0.14009629634255116,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0493\"\>",
                0.049330163240238548,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0494\"\>",
                0.04939689354947454,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1351\"\>",
                -0.1350900925969345,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0366\"\>",
                0.036571732708436173,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0366\"\>",
                0.036606047267158145,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0937\"\>",
                -0.093740136626985215,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0269\"\>",
                0.026864171823395708,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0269\"\>",
                0.026872712419241808,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0318\"\>",
                -0.031781666520502849,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0196\"\>",
                0.019623641938469012,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0196\"\>",
                0.019616441041218111,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0367\"\>",
                0.036708479564517837,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0143\"\>",
                0.014291578986346067,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0143\"\>",
                0.014277240229150501,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1004\"\>",
                0.10043087435265274,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0104\"\>",
                0.010396210718787402,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0104\"\>",
                0.010380531698771796,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1510\"\>",
                0.1510425522563752,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Table of Percentage Errors in Theta", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
thetasamples = TableForm[Join[{{\"S\", \"SOR Theta\", \"Exact Theta\
\", \"% Error\"}},
Table[Map[PaddedForm[N[#], {5, 4}]&, 
{S, SORTheta[10, 0.05, 0, S, 5, 0.2],
BlackScholesPutTheta[S,10,0.2,0.05,0,5],
100*(SORTheta[10,0.05,0,S,5,0.2]/BlackScholesPutTheta[S,10,0.2,0.05,0,5]-1)}],\

{S, 2, 16, 1}]]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"SOR Theta\"\>", "\<\"Exact Theta\"\>", 
            "\<\"% Error\"\>"},
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3884\"\>",
                0.38840374987209797,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3885\"\>",
                0.38850954391536052,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0272\"\>",
                -0.027230745015005553,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3772\"\>",
                0.37716565134430258,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3772\"\>",
                0.3771662723552861,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0002\"\>",
                -0.00016465178067637254,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3406\"\>",
                0.34061586103315084,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.3405\"\>",
                0.34052310009601122,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0272\"\>",
                0.027240717917065282,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2791\"\>",
                0.27911871224621032,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2792\"\>",
                0.27922180686816422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0369\"\>",
                -0.036922124066962603,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2071\"\>",
                0.20705574482933303,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2073\"\>",
                0.20732767425793053,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1312\"\>",
                -0.13115925289317598,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1389\"\>",
                0.13891502857186586,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1391\"\>",
                0.13914229271285256,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1633\"\>",
                -0.16333218071640232,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0828\"\>",
                0.082840243624642679,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0829\"\>",
                0.082893517820476131,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0643\"\>",
                -0.064268228969155405,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0411\"\>",
                0.04107506455293608,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0410\"\>",
                0.040962269627171116,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2754\"\>",
                0.27536297864252379,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0124\"\>",
                0.012408101224790439,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0122\"\>",
                0.012207835061223929,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.6405\"\>",
                1.6404723897574858,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0058\"\>",
                -0.0057604172364359345,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0060\"\>",
                -0.005966431371365169,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-3.4529\"\>",
                -3.452887029220908,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0162\"\>",
                -0.016205841398504544,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0164\"\>",
                -0.016364675313776202,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.9706\"\>",
                -0.97059008031737948,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0213\"\>",
                -0.021329919653937393,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0214\"\>",
                -0.021421153016455467,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.4259\"\>",
                -0.42590313625036069,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0230\"\>",
                -0.022998187840233827,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0230\"\>",
                -0.023024844458457053,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.1158\"\>",
                -0.11577328251368701,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0226\"\>",
                -0.022562528068814706,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0225\"\>",
                -0.022539828190855676,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1007\"\>",
                0.10071007536889276,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0210\"\>",
                -0.020952728517574608,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"-0.0209\"\>",
                -0.020899374917308117,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2553\"\>",
                0.25528801927134914,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell[TextData[{
  "The maximum error in Theta is about 3.5% of the true value. This has been \
obtained with ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 32\)]],
  ", and we remind the reader that a corresponding treatment with a \
Crank-Nicholson scheme with ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 8\)]],
  " led to errors of about 200%. So the use of the SOR solver preserves the \
good characteristics of the 3TLD system. Note that we have used just 20 \
time-steps to deal accurately with a problem with five years to expiry. The \
price of the use of the SOR method is a decrease in speed compared to the \
tridiagonal solver. Note that we are not recommending the use of such a \
method for vanilla European options. Its appearance here is purely to act as \
a test of the method - to demonstrate that we have correctly implemented \
successive over-relaxation as a method for solving the difference problem."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.6 3TLD-PSOR Analysis of American Options", "Section",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Now we proceed to the first \"real\" application of these methods, and \
look at the valuation of American Puts using a PSOR scheme based on ",
  "3TLD",
  ". We shall keep most parameters to be identical with the previous European \
case, but consider expiry times of one, five and ten years, and also we shall \
turn on \"early exercise\" within the solution scheme by turning on the \
projection onto the payoff constraint surface. We shall fix the number of \
time-steps at 20, so in the case of one year to expiry, ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 6.4\)]],
  ". This case has been contrived to reveal an important issue regarding the \
calculation of Greeks near the critical price."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(dx\  = \ 0.0125; \ dtau\  = \ 0.001; \ alpha\  = \ dtau/dx^2; \n
    M = 20; \ nminus\  = \ 320; \ nplus\  = \ 160; \ alpha\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(6.39999999999999946`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(relax\  = \ 1.1; \ntolerance\  = \ 0.000001; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(initial = 
      Table[PutExercise[\((k - 1 - nminus)\)\ dx, 0.05, 0, 0.2], \ {k, 
          nminus + nplus + 1}]; \n
    lower = Table[
        f[\(-nminus\)\ dx, \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \n
    upper = Table[
        g[\(+nplus\)\ dx, \ \ \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \)], "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["w = Table[0, {m, 1, 3},   {k, 1, nminus+nplus+1}];", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["vold = Take[initial, {2, -2}];", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
w[[1,1]] = f[-nminus*dx, dtau/4, 0.05, 0, 0.2];
w[[1, nminus+nplus+1]] = g[nplus*dx, dtau/4, 0.05, 0, 0.2];
w[[2,1]] = f[-nminus*dx, dtau/2, 0.05, 0, 0.2];
w[[2, nminus+nplus+1]] = g[nplus*dx, dtau/2, 0.05, 0, 0.2];
w[[3,1]] = f[-nminus*dx, dtau, 0.05, 0, 0.2];
w[[3, nminus+nplus+1]] = g[nplus*dx, dtau, 0.05, 0, 0.2];\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["First Kick-Off Component", "Subsubsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Simple Douglas with two time levels and ",
  Cell[BoxData[
      \(TraditionalForm\`d\ t/4\)]],
  " time-step. We treat the American case by replacing SOR with PSOR based on \
the payoff function. You should note that there are only two differences \
between this analysis and the previous European case:\n\n(a) the payoff \
constraint function is re-computed for each time step;\n(b) the solver is the \
",
  StyleBox["CompPSORSolve",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function, which projects the solution onto the payoff constraint \
surface."
}], "Text",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(payoff\  = \ 
      Table[PutPayoff[\((k - 1 - nminus)\)*dx, \ dtau/4, \ 0.05, \ 0, \ 
          0.2], \ {k, \ 2, \ nminus + nplus}]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
rhs = DougDMatrix[alpha/4, vold]+
Table[
Which[
k==1, (6*alpha/4+1)*lower[[1]] + (6*alpha/4-1)*w[[1, 1]],
k== nplus + nminus-1,  (6*alpha/4+1)*upper[[1]] +
(6*alpha/4-1)*w[[1, nplus+nminus+1]],
True, 0],
{k, 1, nplus + nminus-1}];
vnew = CompPSORSolve[DougDiag[alpha/4], DougOffDiag[alpha/4], rhs, tolerance, \
relax, payoff];
w[[1]] = Join[{w[[1,1]]}, vnew, {w[[1,nplus+nminus+1]]}];
vvold = vold;
vold = vnew;\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Now we have two iterations of the three-time-level Douglas system, \
doubling the time-step at each stage, and re-computing the payoff.\
\>", 
  "Text",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(payoff\  = \ 
      Table[PutPayoff[\((k - 1 - nminus)\)*dx, \ dtau/2, \ 0.05, \ 0, \ 
          0.2], \ {k, \ 2, \ nminus + nplus}]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
rhs = DougDDMatrix[vold] - DougDDMatrix[vvold]/4 + 
Table[
Which[
k==1, (alpha/4-1/8)*w[[2, 1]] + w[[1,1]]/6 - lower[[1]]/24,
k==nplus + nminus-1,  (alpha/4-1/8)*w[[2, nplus+nminus+1]] + 
w[[1,nplus+nminus+1]]/6 - upper[[1]]/24,
True, 0],
{k, 1, nplus + nminus-1}];
vnew = CompPSORSolve[ThreeDougDiag[alpha/4], ThreeDougOffDiag[alpha/4], rhs, \
tolerance, relax, payoff];
w[[2]] = Join[{w[[2,1]]}, vnew, {w[[2,nplus+nminus+1]]}];
vold = vnew;\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(payoff\  = \ 
      Table[PutPayoff[\((k - 1 - nminus)\)*dx, dtau, \ 0.05, \ 0, \ 
          0.2], \ {k, \ 2, \ nminus + nplus}]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
rhs = DougDDMatrix[vold] - DougDDMatrix[vvold]/4 + 
Table[
Which[
k==1, (alpha/2-1/8)*w[[3, 1]] + w[[2,1]]/6 - lower[[1]]/24,
k==nplus + nminus-1,  (alpha/2-1/8)*w[[3, nplus+nminus+1]] + 
w[[2,nplus+nminus+1]]/6 - upper[[1]]/24,
True, 0],
{k, 1, nplus + nminus-1}];
vnew = CompPSORSolve[ThreeDougDiag[alpha/2], ThreeDougOffDiag[alpha/2], rhs, \
tolerance, relax, payoff];
w[[3]] = Join[{w[[3,1]]}, vnew, {w[[3,nplus+nminus+1]]}];
wold = initial;
wvold = initial;
wnew = w[[3]];\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Main Block", "Subsubsection",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Once the kick-off phase has been completed, we go into the main block. \
Again, the only difference between this case and European is re-computation \
of the payoff constraint and the use of the PSOR solver. The output of the ",
  StyleBox["Print",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " statement is not shown."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
For[m=3, m<=M+1, m++,
(wvold = wold;
wold = wnew;
Print[m];
rhs = DougDDMatrix[Take[wold, {2, -2}]] -
DougDDMatrix[Take[wvold, {2, -2}]]/4 + 
Table[
Which[
k==1, 
(alpha-1/8)*lower[[m]] + lower[[m-1]]/6 - lower[[m-2]]/24,
k==nplus + nminus-1, 
(alpha-1/8)*upper[[m]] + upper[[m-1]]/6 - upper[[m-2]]/24,
True, 0],
{k, 1, nplus + nminus-1}];
payoff = Table[PutPayoff[(k-1-nminus)*dx, (m-1)*dtau, 0.05, 0, 0.2], {k, 2, \
nminus+nplus}];
temp = CompPSORSolve[ThreeDougDiag[alpha], ThreeDougOffDiag[alpha], rhs, \
tolerance, relax, payoff];
wnew = Join[{lower[[m]]}, temp, {upper[[m]]}])
]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(points\  = \ 
      Table[\((k - nminus - 1)\)*dx, \ \ {k, \ 1, \ nplus + nminus + 1}]; \n
    finalstep\  = \ wnew; \nprevstep\  = \ wold; \npprevstep\  = \ wvold; \n
    interpoldata\  = \ Transpose[{points, finalstep}]; \n
    ufunc\  = \ Interpolation[interpoldata, \ InterpolationOrder\  -> \ 3]; \n
    Valuation[strike_, \ r_, \ q_, \ S_, \ T_, \ sd_]\  := \n
      ValuationMultiplier[strike, \ r, \ q, \ Log[S/strike], \ sd^2*T/2, \ 
          sd]*\nufunc[Log[S/strike]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plot Sample Valuation", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Plot[Valuation[10, 0.05, 0, S, 1, 0.2], {S, 2, 20}, 
PlotPoints -> 50, PlotRange -> All];\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0820106 0.0529101 0.0147096 0.0735762 [
[.05026 .00221 -9 -9 ]
[.05026 .00221 9 0 ]
[.31481 .00221 -9 -9 ]
[.31481 .00221 9 0 ]
[.44709 .00221 -6 -9 ]
[.44709 .00221 6 0 ]
[.57937 .00221 -12 -9 ]
[.57937 .00221 12 0 ]
[.71164 .00221 -6 -9 ]
[.71164 .00221 6 0 ]
[.84392 .00221 -12 -9 ]
[.84392 .00221 12 0 ]
[.97619 .00221 -6 -9 ]
[.97619 .00221 6 0 ]
[.17004 .16186 -6 -4.5 ]
[.17004 .16186 0 4.5 ]
[.17004 .30901 -6 -4.5 ]
[.17004 .30901 0 4.5 ]
[.17004 .45617 -6 -4.5 ]
[.17004 .45617 0 4.5 ]
[.17004 .60332 -6 -4.5 ]
[.17004 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.05026 .01471 m
.05026 .02096 L
s
[(2.5)] .05026 .00221 0 1 Mshowa
.31481 .01471 m
.31481 .02096 L
s
[(7.5)] .31481 .00221 0 1 Mshowa
.44709 .01471 m
.44709 .02096 L
s
[(10)] .44709 .00221 0 1 Mshowa
.57937 .01471 m
.57937 .02096 L
s
[(12.5)] .57937 .00221 0 1 Mshowa
.71164 .01471 m
.71164 .02096 L
s
[(15)] .71164 .00221 0 1 Mshowa
.84392 .01471 m
.84392 .02096 L
s
[(17.5)] .84392 .00221 0 1 Mshowa
.97619 .01471 m
.97619 .02096 L
s
[(20)] .97619 .00221 0 1 Mshowa
.125 Mabswid
.07672 .01471 m
.07672 .01846 L
s
.10317 .01471 m
.10317 .01846 L
s
.12963 .01471 m
.12963 .01846 L
s
.15608 .01471 m
.15608 .01846 L
s
.20899 .01471 m
.20899 .01846 L
s
.23545 .01471 m
.23545 .01846 L
s
.2619 .01471 m
.2619 .01846 L
s
.28836 .01471 m
.28836 .01846 L
s
.34127 .01471 m
.34127 .01846 L
s
.36772 .01471 m
.36772 .01846 L
s
.39418 .01471 m
.39418 .01846 L
s
.42063 .01471 m
.42063 .01846 L
s
.47354 .01471 m
.47354 .01846 L
s
.5 .01471 m
.5 .01846 L
s
.52646 .01471 m
.52646 .01846 L
s
.55291 .01471 m
.55291 .01846 L
s
.60582 .01471 m
.60582 .01846 L
s
.63228 .01471 m
.63228 .01846 L
s
.65873 .01471 m
.65873 .01846 L
s
.68519 .01471 m
.68519 .01846 L
s
.7381 .01471 m
.7381 .01846 L
s
.76455 .01471 m
.76455 .01846 L
s
.79101 .01471 m
.79101 .01846 L
s
.81746 .01471 m
.81746 .01846 L
s
.87037 .01471 m
.87037 .01846 L
s
.89683 .01471 m
.89683 .01846 L
s
.92328 .01471 m
.92328 .01846 L
s
.94974 .01471 m
.94974 .01846 L
s
.02381 .01471 m
.02381 .01846 L
s
.25 Mabswid
0 .01471 m
1 .01471 L
s
.18254 .16186 m
.18879 .16186 L
s
[(2)] .17004 .16186 1 0 Mshowa
.18254 .30901 m
.18879 .30901 L
s
[(4)] .17004 .30901 1 0 Mshowa
.18254 .45617 m
.18879 .45617 L
s
[(6)] .17004 .45617 1 0 Mshowa
.18254 .60332 m
.18879 .60332 L
s
[(8)] .17004 .60332 1 0 Mshowa
.125 Mabswid
.18254 .0515 m
.18629 .0515 L
s
.18254 .08829 m
.18629 .08829 L
s
.18254 .12507 m
.18629 .12507 L
s
.18254 .19865 m
.18629 .19865 L
s
.18254 .23544 m
.18629 .23544 L
s
.18254 .27223 m
.18629 .27223 L
s
.18254 .3458 m
.18629 .3458 L
s
.18254 .38259 m
.18629 .38259 L
s
.18254 .41938 m
.18629 .41938 L
s
.18254 .49295 m
.18629 .49295 L
s
.18254 .52974 m
.18629 .52974 L
s
.18254 .56653 m
.18629 .56653 L
s
.25 Mabswid
.18254 0 m
.18254 .61803 L
s
.5 Mabswid
.02381 .60332 m
.04273 .577 L
.06337 .54831 L
.08275 .52135 L
.10139 .49543 L
.12123 .46784 L
.14033 .44128 L
.16063 .41306 L
.18019 .38586 L
.199 .35969 L
.21902 .33186 L
.23829 .30506 L
.25683 .27929 L
.27656 .25185 L
.29555 .22544 L
.31574 .19736 L
.33519 .17032 L
.3539 .14462 L
.3738 .12039 L
.39297 .10044 L
.41334 .08255 L
.43296 .06825 L
.45184 .0569 L
.47193 .0471 L
.49127 .03957 L
.50987 .03381 L
.52967 .02901 L
.54872 .02544 L
.56898 .02254 L
.5885 .02046 L
.60727 .01894 L
.62725 .01775 L
.64648 .01691 L
.66497 .01631 L
.68466 .01585 L
.70361 .01552 L
.72376 .01528 L
.74317 .01511 L
.76183 .015 L
.77224 .01495 L
.7817 .01491 L
.80082 .01485 L
.81101 .01483 L
.82041 .01481 L
.83925 .01478 L
.85855 .01476 L
.8771 .01474 L
.88663 .01474 L
.89686 .01473 L
.90653 .01473 L
Mistroke
.91708 .01473 L
.93655 .01472 L
.94621 .01472 L
.95649 .01472 L
.97619 .01472 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{196.312, 121.125},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003e0000Ua000`40O003h00OmGoo003eOol000Yo
o`D000Eoo`8000Eoo`@002Qoo`03001oogoo00Eoo`8000Eoo`@000eoo`D000=oo`<000]oo`D0009o
o`D000Eoo`8000Aoo`@000Yoo`D000=oo`@000Uoo`D000Aoo`03001oogoo00Eoo`8000Aoo`@000Yo
o`D000Aoo`<000eoo`002goo00@007ooOol000Eoo`8000Uoo`03001oogoo02Eoo`03001oogoo00Eo
o`8000Uoo`03001oogoo00aoo`03001oogoo009oo`05001oogooOol0000<Ool00`00Oomoo`03Ool0
1000Oomoo`001Goo0P0027oo00<007ooOol02Goo00<007ooOol01goo00<007ooOol027oo00<007oo
Ool017oo00<007ooOol01Goo0P0027oo00<007ooOol027oo00@007ooOol000=oo`05001oogooOol0
000<Ool000aoo`03001oogoo011oo`03001oogoo02Ioo`03001oogoo00moo`03001oogoo00aoo`03
001oogoo009oo`05001oogooOol0000<Ool00`00Oomoo`04Ool00`00Oomoo`0?Ool00`00Oomoo`09
Ool00`00Oomoo`07Ool00`00Oomoo`08Ool00`00Oomoo`05Ool00`00Oomoo`0>Ool00`00Oomoo`09
Ool00`00Oomoo`03Ool01@00Oomoogoo000037oo000=Ool00`00Oomoo`0?Ool00`00Oomoo`0VOol0
0`00Oomoo`0?Ool00`00Oomoo`0<Ool00`00Oomoo`02Ool01@00Oomoogoo000037oo00<007ooOol0
1Goo00<007ooOol03Woo00<007ooOol02Goo00<007ooOol01goo00<007ooOol027oo00<007ooOol0
1Goo00<007ooOol03Woo00<007ooOol02Woo00<007ooOol00Woo00D007ooOomoo`0000aoo`003Woo
00<007ooOol02Woo1000:Goo00<007ooOol02goo10003goo00<007ooOol00Woo00D007ooOomoo`00
00aoo`03001oogoo00Ioo`03001oogoo00Uoo`@000aoo`03001oogoo00=oo`@000]oo`03001oogoo
00Eoo`03001oogoo00Yoo`@000ioo`05001oogooOol00003Ool00`00Oomoo`0:Ool000ioo`03001o
ogoo00Yoo`03001oogoo02]oo`03001oogoo00Yoo`03001oogoo011oo`03001oogoo009oo`05001o
ogooOol0000<Ool00`00Oomoo`06Ool00`00Oomoo`09Ool00`00Oomoo`0=Ool00`00Oomoo`03Ool0
0`00Oomoo`0<Ool00`00Oomoo`06Ool00`00Oomoo`09Ool00`00Oomoo`0?Ool01@00Oomoogoo0000
0goo00<007ooOol02Woo000:Ool01@00Oomoogoo000037oo00<007ooOol09goo00D007ooOomoo`00
00aoo`03001oogoo011oo`03001oogoo009oo`05001oogooOol0000<Ool00`00Oomoo`02Ool01@00
Oomoogoo00002goo00<007ooOol03Goo00<007ooOol00goo00<007ooOol037oo00<007ooOol00Woo
00D007ooOomoo`0000]oo`03001oogoo00]oo`05001oogooOol00003Ool01@00Oomoogoo000037oo
000;Ool3000=Ool4000VOol5000<Ool4000=Ool30005Ool3000;Ool30005Ool3000<Ool4000:Ool3
0005Ool40009Ool30004Ool5000;Ool4000;Ool30005Ool3000=Ool00?Eoo`00<Woo00<007ooOol0
`7oo000bOol00`00Oomoo`30Ool0039oo`03001oogoo0<1oo`002Woog`0037oo000?Ool00`00Oomo
o`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomo
o`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomo
o`02Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomo
o`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomo
o`03OolF0002Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool0
0`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool0
0`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`0@Ool001Eoo`03001oogoo01Yoo`03001oogoo
01Yoo`03001oogoo01]oo`03001oogoo01Ioo`D001eoo`03001oogoo01Yoo`03001oogoo01Yoo`03
001oogoo011oo`00<Woo00<007ooOol0B7oo2@00Kgoo000bOol00`00Oomoo`15Ool3001hOol0039o
o`03001oogoo04=oo`8007]oo`00<Woo00<007ooOol0@7oo0`00OGoo000bOol00`00Oomoo`0lOol4
0020Ool0039oo`8003aoo`03001oogoo089oo`00<Woo00<007ooOol0>Woo00<007ooOol0Pgoo000b
Ool00`00Oomoo`0hOol20026Ool0039oo`03001oogoo03Ioo`8008Qoo`00<Woo00<007ooOol0=7oo
0P00RWoo000bOol00`00Oomoo`0bOol2002<Ool0039oo`03001oogoo035oo`03001oogoo08aoo`00
<Woo00<007ooOol0<7oo00<007ooOol0SGoo000bOol2000`Ool00`00Oomoo`2>Ool0039oo`03001o
ogoo02ioo`03001oogoo08moo`00<Woo00<007ooOol0;Goo00<007ooOol0T7oo000bOol00`00Oomo
o`0/Ool00`00Oomoo`2AOol0039oo`03001oogoo02aoo`03001oogoo095oo`00<Woo00<007ooOol0
:goo00<007ooOol0TWoo000bOol00`00Oomoo`0ZOol00`00Oomoo`2COol0039oo`03001oogoo02Uo
o`03001oogoo09Aoo`00<Woo00<007ooOol0:7oo00<007ooOol0UGoo000bOol2000XOol00`00Oomo
o`2FOol0039oo`03001oogoo02Ioo`03001oogoo09Moo`00<Woo00<007ooOol09Goo00<007ooOol0
V7oo000bOol00`00Oomoo`0TOol00`00Oomoo`2IOol0039oo`03001oogoo02=oo`03001oogoo09Yo
o`00:Goo1@0017oo00<007ooOol08goo00<007ooOol0VWoo000ZOol01000Oomoo`0017oo00<007oo
Ool08Woo00<007ooOol0Vgoo000[Ool00`00Oomoo`04Ool00`00Oomoo`0QOol00`00Oomoo`2LOol0
02aoo`03001oogoo00=oo`<0025oo`03001oogoo09aoo`00;Goo00<007ooOol00Woo00<007ooOol0
87oo00<007ooOol0WGoo000]Ool00`00Oomoo`02Ool00`00Oomoo`0OOol00`00Oomoo`2NOol002Uo
o`05001oogooOol00004Ool00`00Oomoo`0OOol00`00Oomoo`2NOol002Yoo`<000Eoo`03001oogoo
01ioo`03001oogoo09moo`00<Woo00<007ooOol07Goo00<007ooOol0X7oo000bOol00`00Oomoo`0M
Ool00`00Oomoo`2POol0039oo`03001oogoo01aoo`03001oogoo0:5oo`00<Woo0P0077oo00<007oo
Ool0XWoo000bOol00`00Oomoo`0KOol00`00Oomoo`2ROol0039oo`03001oogoo01Yoo`03001oogoo
0:=oo`00<Woo00<007ooOol06Goo00<007ooOol0Y7oo000bOol00`00Oomoo`0HOol00`00Oomoo`2U
Ool0039oo`03001oogoo01Moo`03001oogoo0:Ioo`00<Woo00<007ooOol05Woo00<007ooOol0Ygoo
000bOol00`00Oomoo`0FOol00`00Oomoo`2WOol0039oo`8001Ioo`03001oogoo0:Qoo`00<Woo00<0
07ooOol057oo00<007ooOol0ZGoo000bOol00`00Oomoo`0DOol00`00Oomoo`2YOol0039oo`03001o
ogoo01=oo`03001oogoo0:Yoo`00<Woo00<007ooOol04Woo00<007ooOol0Zgoo000bOol00`00Oomo
o`0BOol00`00Oomoo`2[Ool0039oo`03001oogoo015oo`03001oogoo0:aoo`00<Woo00<007ooOol0
47oo00<007ooOol0[Goo000bOol2000AOol00`00Oomoo`2]Ool0039oo`03001oogoo00moo`03001o
ogoo0:ioo`00<Woo00<007ooOol03Woo00<007ooOol0[goo000bOol00`00Oomoo`0>Ool00`00Oomo
o`2_Ool0039oo`03001oogoo00eoo`03001oogoo0;1oo`00<Woo00<007ooOol037oo00<007ooOol0
/Goo000[Ool30004Ool00`00Oomoo`0<Ool00`00Oomoo`2aOol002aoo`03001oogoo00=oo`03001o
ogoo00]oo`03001oogoo0;9oo`00:Goo1@0017oo00<007ooOol02Woo00<007ooOol0/goo000YOol0
1000Oomoo`001Goo0`002Woo00<007ooOol0/goo000ZOol00`00Ool00005Ool00`00Oomoo`09Ool0
0`00Oomoo`2dOol002Yoo`03001oo`0000Eoo`03001oogoo00Qoo`03001oogoo0;Eoo`00:goo0P00
1Goo00<007ooOol01goo00<007ooOol0]Woo000/Ool00`00Oomoo`03Ool00`00Oomoo`06Ool00`00
Oomoo`2gOol0039oo`03001oogoo00Eoo`03001oogoo0;Qoo`00<Woo00<007ooOol01Goo00<007oo
Ool0^7oo000bOol00`00Oomoo`04Ool00`00Oomoo`2iOol0039oo`8000Aoo`03001oogoo0;Yoo`00
<Woo00<007ooOol00goo00<007ooOol0^Woo000bOol00`00Oomoo`02Ool00`00Oomoo`2kOol0039o
o`05001oogooOol0002nOol0039oo`05001oogooOol0002nOol0039oo`04001oogoo002oOol0039o
o`03001oo`000<1oo`00<Woo00<007oo0000`7oo000bOol20031Ool0039oo`03001oogoo0<1oo`00
<Woo00<007ooOol0`7oo000aOol20032Ool0031oo`03001oo`000<9oo`00;goo00@007ooOol00<9o
o`00;Woo00D007ooOomoo`000<9oo`00;Goo00<007ooOol00Woo00<007ooOol0`7oo000]Ool00`00
Oomoo`02Ool20031Ool002aoo`03001oogoo00=oo`03001oogoo0<1oo`00:goo00<007ooOol017oo
00<007ooOol0`7oo000[Ool00`00Oomoo`04Ool00`00Oomoo`30Ool002Yoo`03001oogoo00Eoo`03
001oogoo0<1oo`00:Goo00<007ooOol01Woo00<007ooOol0`7oo000YOol40005Ool00`00Oomoo`30
Ool002Qoo`8000=oo`03001oogoo009oo`03001oogoo0<1oo`009goo00<007oo00000goo00<007oo
Ool00Woo0`00`7oo000WOol00`00Ool00003Ool00`00Oomoo`02Ool00`00Oomoo`30Ool002Ioo`03
001oogoo00@000Eoo`03001oogoo0<1oo`009Goo00D007ooOomoo`0000Qoo`03001oogoo0<1oo`00
9Goo00<007ooOol00Woo00<007ooOol01Goo00<007ooOol0`7oo000TOol00`00Oomoo`04Ool30004
Ool00`00Oomoo`30Ool002=oo`03001oogoo00aoo`03001oogoo0<1oo`008Woo00<007ooOol03Goo
00<007ooOol0`7oo000QOol00`00Oomoo`0>Ool20031Ool0021oo`03001oogoo00moo`03001oogoo
0<1oo`0087oo00<007ooOol03goo00<007ooOol0`7oo000OOol00`00Oomoo`0@Ool00`00Oomoo`30
Ool001ioo`03001oogoo015oo`03001oogoo0<1oo`007Woo00<007ooOol04Goo00<007ooOol0`7oo
000MOol00`00Oomoo`0BOol00`00Oomoo`30Ool001aoo`03001oogoo01=oo`03001oogoo0<1oo`00
77oo00<007ooOol04goo00<007ooOol0`7oo000KOol00`00Oomoo`0DOol20031Ool001Yoo`03001o
ogoo01Eoo`03001oogoo0<1oo`006Woo00<007ooOol05Goo00<007ooOol0`7oo000IOol00`00Oomo
o`0FOol00`00Oomoo`30Ool001Qoo`03001oogoo01Moo`03001oogoo0<1oo`0067oo00<007ooOol0
5goo00<007ooOol0`7oo000GOol00`00Oomoo`0HOol00`00Oomoo`30Ool001Ioo`03001oogoo01Uo
o`03001oogoo0<1oo`005Goo00<007ooOol06Woo0P00`Goo000DOol00`00Oomoo`0KOol00`00Oomo
o`30Ool001=oo`03001oogoo01aoo`03001oogoo0<1oo`004goo00<007ooOol077oo00<007ooOol0
`7oo000BOol00`00Oomoo`0MOol00`00Oomoo`30Ool0015oo`03001oogoo01Ioo`<000Eoo`03001o
ogoo0<1oo`004Goo00<007ooOol05Goo00D007ooOomoo`0000Aoo`03001oogoo0<1oo`0047oo00<0
07ooOol05Woo00D007ooOomoo`0000Aoo`03001oogoo0<1oo`003goo00<007ooOol05goo00D007oo
Oomoo`0000Aoo`<00<1oo`00:Woo0`001Goo00<007ooOol0`7oo000YOol01@00Oomoogoo000017oo
00<007ooOol0`7oo000YOol01@00Oomoogoo000017oo00<007ooOol0`7oo000ZOol30038Ool00?Eo
o`00\
\>"],
  ImageRangeCache->{{{0, 244.375}, {150.375, 0}} -> {0.692008, -0.858409, 
  0.106377, 0.0764979}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Construction and Verification of Interpolated Valuation and Greeks \
- First Simple Method\
\>", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
In this subsection we define some functions to compute Delta, Gamma \
and Theta directly from the finite-difference grid. Note that these are based \
on the same simple differencing and interpolation methods as were used for \
the European case, but we should anticipate that these methods introduce some \
inappropriate smoothing in the neighbourhood of the critical price frontier, \
where \[CapitalGamma] is actually discontinuous.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
listd[data_, step_] :=
Module[{dleft,dright,len},
len = Length[data];
dleft = (4*data[[2]]-3*data[[1]]-data[[3]])/(2*step);
dright = (3*data[[len]]-4*data[[len-1]]+data[[len-2]])/(2*step);
Join[{dleft}, Take[RotateLeft[data]-RotateRight[data], {2, -2}]/(2*step), \
{dright}]
]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(kt\  = \ ktwo[0.05, \ 0, \ 0.2]; \nko\  = \ kone[0.05, \ 0.2]; \)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(deltadata = 
      listd[finalstep, dx] - 1\/2\ \((kt - 1)\)\ finalstep; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(gammadata = 
      listd[deltadata, dx] - 1\/2\ \((kt + 1)\)\ deltadata; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(thetadata = \(3  finalstep - 4  prevstep + pprevstep\)\/\(2  dtau\) - \
\((1\/4\ \((kt - 1)\)\^2 + ko)\) finalstep; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[{
    \(\(deltainterpoldata = Transpose[{points, deltadata}]; \)\), 
    \(\(gammainterpoldata = Transpose[{points, gammadata}]; \)\), 
    \(\(thetainterpoldata = Transpose[{points, thetadata}]; \)\)}], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[{
    \(dfunc = 
      Interpolation[deltainterpoldata, InterpolationOrder \[Rule] 3]; \n
    gfunc = Interpolation[gammainterpoldata, 
        InterpolationOrder \[Rule] 3]; \), 
    \(\(tfunc = 
      Interpolation[thetainterpoldata, InterpolationOrder \[Rule] 3]; \)\)}], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(SORDelta[strike_, r_, q_, S_, T_, 
        sd_] := \(1\/S\) \((ValuationMultiplier[strike, r, q, 
              Log[S\/strike], \(sd\^2\ T\)\/2, sd]\ dfunc[
              Log[S\/strike]])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(SORGamma[strike_, r_, q_, S_, T_, 
        sd_] := \(1\/S\^2\) \((ValuationMultiplier[strike, r, q, 
              Log[S\/strike], \(sd\^2\ T\)\/2, sd]\ gfunc[
              Log[S\/strike]])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(SORTheta[strike_, r_, q_, S_, T_, 
        sd_] := \(-\(1\/2\)\)\ sd\^2\ ValuationMultiplier[strike, r, q, 
          Log[S\/strike], \(sd\^2\ T\)\/2, sd]\ tfunc[Log[S\/strike]]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.7 Summary of Finite-Difference Results - One Year Expiry", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
In the following table we give a table of underlying values, option \
valuations, Deltas, Gammas and Thetas for a range of underlying prices with \
the strike at 10. (1 year to expiry, 20% volatility, 5% interest rate, 20 \
time-steps, 480 price-steps)\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
samples = TableForm[Join[{{\"S\", \"SOR Value\", \"SOR Delta\", \
\"SOR Gamma\", \"SOR Theta\"}},
Table[Map[PaddedForm[N[#], {5, 3}]&, {S, Valuation[10, 0.05, 0, S, 1, 0.2],
SORDelta[10, 0.05, 0, S, 1, 0.2],SORGamma[10, 0.05, 0, S, 1, 0.2],
SORTheta[10, 0.05, 0, S, 1, 0.2]}],
{S, 2, 16, 1}]]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"SOR Value\"\>", "\<\"SOR Delta\"\>", 
            "\<\"SOR Gamma\"\>", "\<\"SOR Theta\"\>"},
          {
            TagBox[
              InterpretationBox["\<\"  2.000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  8.000\"\>",
                8.000000006442864,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0000846346409988,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.000\"\>",
                -9.7260576840482052*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox[\("  1.528"\[Times]10\^"-6"\),
                1.5283744917375685*^-06,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  3.000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  7.000\"\>",
                7.0000000121685861,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0001029445701166,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.000\"\>",
                -5.8736909107429827*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox[\("  1.337"\[Times]10\^"-6"\),
                1.3373277087885782*^-06,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  4.000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  6.000\"\>",
                6.0000000162686291,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0001121000123763,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.000\"\>",
                -4.1763878817950018*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox[\("  1.146"\[Times]10\^"-6"\),
                1.1462808624797512*^-06,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  5.000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  5.000\"\>",
                5.0000000247436596,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0001175923360759,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.000\"\>",
                -3.2312477502273939*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox[\("  9.552"\[Times]10\^"-7"\),
                9.5523406153801201*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  6.000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  4.000\"\>",
                4.0000000133505234,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0001212574117795,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.000\"\>",
                -2.6316717102226714*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox[\("  7.642"\[Times]10\^"-7"\),
                7.6418725615637421*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  7.000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  3.000\"\>",
                3.0000000355957677,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0001238702299087,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.000\"\>",
                -2.218350383319721*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox[\("  5.731"\[Times]10\^"-7"\),
                5.7314044505118465*^-07,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  8.000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  2.000\"\>",
                1.999954099602395,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -0.99958248743375577,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.065\"\>",
                0.065264631241674198,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.000\"\>",
                0.00046804746359208147,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  9.000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  1.149\"\>",
                1.1494711750124658,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.683\"\>",
                -0.682754613126345,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.312\"\>",
                0.31241397958969319,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.141\"\>",
                -0.14131477908204851,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.609\"\>",
                0.60942631400524472,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.411\"\>",
                -0.41104032482420749,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.229\"\>",
                0.2292940206454234,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.223\"\>",
                -0.22283850377004935,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.299\"\>",
                0.29887983579703908,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.224\"\>",
                -0.22390568161474864,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.147\"\>",
                0.14666003093686336,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.217\"\>",
                -0.21706268790801422,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.137\"\>",
                0.1367074685150278,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.111\"\>",
                -0.11129968436969899,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.082\"\>",
                0.082395268993197227,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.164\"\>",
                -0.16375000364752662,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.059\"\>",
                0.05894449939967715,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.051\"\>",
                -0.051171668360039765,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.042\"\>",
                0.041527255439493885,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.104\"\>",
                -0.1040769359799097,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.024\"\>",
                0.024230387309982841,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.022\"\>",
                -0.022073835614025889,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.019\"\>",
                0.019179202715814969,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.058\"\>",
                -0.0583898554526684,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.010\"\>",
                0.0096016375998791079,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.009\"\>",
                -0.0090566168633703769,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.008\"\>",
                0.0082707960425093228,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.030\"\>",
                -0.02982803085934168,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.004\"\>",
                0.0037057687693005459,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.004\"\>",
                -0.0035788159879549153,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.003\"\>",
                0.0033849600716333852,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.014\"\>",
                -0.01419925692317377,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell["\<\
Note the value of Gamma at a stock price of 8 - the option value \
and Delta would suggest exercise has occurred, but \
\[CapitalGamma]\[NotEqual]0.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.8 Review of the Analytical Approximation ", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->Automatic,
  AspectRatioFixed->True],

Cell["\<\
We can apply the CJM approximation scheme to our test problem under \
consideration. For values of the underlying beyond the critical price, it \
returns exact results:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \({AmericanPutCJM[7, \ 10, \ 0.2, \ 1, \ 
        0.05], \[IndentingNewLine]AmericanPutDeltaCJM[7, \ 10, \ 0.2, \ 1, \ 
        0.05], \ \ AmericanPutGammaCJM[7, \ 10, \ 0.2, \ 1, \ 0.05]}\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({3, \(-1\), 0}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "We have obtained the following results by applying this to several values \
in the neighbourhood of the strike. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " may produce some complaints about underflow error or slow convergence of \
integrals when you run this model. This calculation takes some time, and this \
in some ways emphasizes the power of a good finite-difference scheme - once \
the FD solution has been computed on a rectangular grid, it is then very \
little effort to extract the valuation, Delta, Gamma and Theta for various \
values of the underlying. In contrast, in our analytical model, an integral \
has to be computed for each result. This could, in principle, be optimized \
further by precomputing a large number of values of the critical price and \
storing these for use in an approximate integration scheme. Here we shall \
proceed just using the Package functions already defined."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
CJMsamples = TableForm[Join[{{\"S\", \"CJM Value\", \"CJM Delta\", \
\"CJM Gamma\"}},
Table[Map[PaddedForm[N[#], {5, 3}]&, {S, 
AmericanPutCJM[S, 10, 0.2, 0.05, 1], AmericanPutDeltaCJM[S, 10, 0.2, 0.05, \
1],
AmericanPutGammaCJM[S, 10, 0.2, 0.05, 1]}],{S, 2, 16, 1}]]]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"CJM Value\"\>", "\<\"CJM Delta\"\>", 
            "\<\"CJM Gamma\"\>"},
          {
            TagBox[
              InterpretationBox["\<\"  2.000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  8.000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  3.000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  7.000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  4.000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  6.000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  5.000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  5.000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  6.000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  4.000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  7.000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  3.000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  8.000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  2.000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  9.000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  1.149\"\>",
                1.1488658676025576,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.683\"\>",
                -0.68276815770854504,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.312\"\>",
                0.31226180872405418,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.609\"\>",
                0.60893034818223868,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.411\"\>",
                -0.41090389271263533,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.230\"\>",
                0.22969183236201016,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.299\"\>",
                0.29863237278239435,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.224\"\>",
                -0.22357086837723561,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.147\"\>",
                0.14677141980936745,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.137\"\>",
                0.13671048869021063,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.111\"\>",
                -0.11103895034078155,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.082\"\>",
                0.0822494149224966,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.059\"\>",
                0.05905211410595506,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.051\"\>",
                -0.051073836727936875,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.041\"\>",
                0.041372973293551947,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.024\"\>",
                0.024323784117340964,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.022\"\>",
                -0.022072629456133724,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.019\"\>",
                0.019107760640956761,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.010\"\>",
                0.009645263349923645,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.009\"\>",
                -0.0090777896694677011,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.008\"\>",
                0.008256750955290007,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.004\"\>",
                0.0037121771744850667,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.004\"\>",
                -0.0035911293408605383,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.003\"\>",
                0.0033889804880831696,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell[TextData[{
  "This gives very good agreement with the finite-difference model - at this \
level of precision, the level of agreement is almost perfect, giving \
confidence in both the analytical approximation and the FD scheme. We can't \
rule out some form of common-mode failure affecting both approaches, but the \
agreement is reassuring. We have already noted that the FD model has a glitch \
in \[CapitalGamma] at ",
  Cell[BoxData[
      \(TraditionalForm\`S\  = \ 8\)]],
  ". We can ask the analytical model where the front is, approximately:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Front[10, \ 0.05, \ 0.2, \ 1]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \(8.08037554093908738`\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "So \[CapitalGamma] is going to be discontinuous around ",
  Cell[BoxData[
      \(TraditionalForm\`S\  = \ 8.08\)]],
  ", and the FD estimate is smoothed somewhat, as is revealed by the \
following plot of the numerical \[CapitalGamma]."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      SORGamma[10, \ 0.05, \ 0, \ S, \ 1, \ 0.2], \ {S, 5, \ 15}]; \)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.452381 0.0952381 0.0198319 1.58442 [
[.30952 .00733 -3 -9 ]
[.30952 .00733 3 0 ]
[.5 .00733 -6 -9 ]
[.5 .00733 6 0 ]
[.69048 .00733 -6 -9 ]
[.69048 .00733 6 0 ]
[.88095 .00733 -6 -9 ]
[.88095 .00733 6 0 ]
[.10655 .09905 -24 -4.5 ]
[.10655 .09905 0 4.5 ]
[.10655 .17827 -18 -4.5 ]
[.10655 .17827 0 4.5 ]
[.10655 .25749 -24 -4.5 ]
[.10655 .25749 0 4.5 ]
[.10655 .33672 -18 -4.5 ]
[.10655 .33672 0 4.5 ]
[.10655 .41594 -24 -4.5 ]
[.10655 .41594 0 4.5 ]
[.10655 .49516 -18 -4.5 ]
[.10655 .49516 0 4.5 ]
[.10655 .57438 -24 -4.5 ]
[.10655 .57438 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.30952 .01983 m
.30952 .02608 L
s
[(8)] .30952 .00733 0 1 Mshowa
.5 .01983 m
.5 .02608 L
s
[(10)] .5 .00733 0 1 Mshowa
.69048 .01983 m
.69048 .02608 L
s
[(12)] .69048 .00733 0 1 Mshowa
.88095 .01983 m
.88095 .02608 L
s
[(14)] .88095 .00733 0 1 Mshowa
.125 Mabswid
.16667 .01983 m
.16667 .02358 L
s
.21429 .01983 m
.21429 .02358 L
s
.2619 .01983 m
.2619 .02358 L
s
.35714 .01983 m
.35714 .02358 L
s
.40476 .01983 m
.40476 .02358 L
s
.45238 .01983 m
.45238 .02358 L
s
.54762 .01983 m
.54762 .02358 L
s
.59524 .01983 m
.59524 .02358 L
s
.64286 .01983 m
.64286 .02358 L
s
.7381 .01983 m
.7381 .02358 L
s
.78571 .01983 m
.78571 .02358 L
s
.83333 .01983 m
.83333 .02358 L
s
.07143 .01983 m
.07143 .02358 L
s
.02381 .01983 m
.02381 .02358 L
s
.92857 .01983 m
.92857 .02358 L
s
.97619 .01983 m
.97619 .02358 L
s
.25 Mabswid
0 .01983 m
1 .01983 L
s
.11905 .09905 m
.1253 .09905 L
s
[(0.05)] .10655 .09905 1 0 Mshowa
.11905 .17827 m
.1253 .17827 L
s
[(0.1)] .10655 .17827 1 0 Mshowa
.11905 .25749 m
.1253 .25749 L
s
[(0.15)] .10655 .25749 1 0 Mshowa
.11905 .33672 m
.1253 .33672 L
s
[(0.2)] .10655 .33672 1 0 Mshowa
.11905 .41594 m
.1253 .41594 L
s
[(0.25)] .10655 .41594 1 0 Mshowa
.11905 .49516 m
.1253 .49516 L
s
[(0.3)] .10655 .49516 1 0 Mshowa
.11905 .57438 m
.1253 .57438 L
s
[(0.35)] .10655 .57438 1 0 Mshowa
.125 Mabswid
.11905 .03568 m
.1228 .03568 L
s
.11905 .05152 m
.1228 .05152 L
s
.11905 .06736 m
.1228 .06736 L
s
.11905 .08321 m
.1228 .08321 L
s
.11905 .1149 m
.1228 .1149 L
s
.11905 .13074 m
.1228 .13074 L
s
.11905 .14659 m
.1228 .14659 L
s
.11905 .16243 m
.1228 .16243 L
s
.11905 .19412 m
.1228 .19412 L
s
.11905 .20996 m
.1228 .20996 L
s
.11905 .22581 m
.1228 .22581 L
s
.11905 .24165 m
.1228 .24165 L
s
.11905 .27334 m
.1228 .27334 L
s
.11905 .28918 m
.1228 .28918 L
s
.11905 .30503 m
.1228 .30503 L
s
.11905 .32087 m
.1228 .32087 L
s
.11905 .35256 m
.1228 .35256 L
s
.11905 .3684 m
.1228 .3684 L
s
.11905 .38425 m
.1228 .38425 L
s
.11905 .40009 m
.1228 .40009 L
s
.11905 .43178 m
.1228 .43178 L
s
.11905 .44763 m
.1228 .44763 L
s
.11905 .46347 m
.1228 .46347 L
s
.11905 .47931 m
.1228 .47931 L
s
.11905 .511 m
.1228 .511 L
s
.11905 .52685 m
.1228 .52685 L
s
.11905 .54269 m
.1228 .54269 L
s
.11905 .55853 m
.1228 .55853 L
s
.11905 .00399 m
.1228 .00399 L
s
.11905 .59022 m
.1228 .59022 L
s
.11905 .60607 m
.1228 .60607 L
s
.25 Mabswid
.11905 0 m
.11905 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01978 m
.06244 .01979 L
.10458 .01979 L
.14415 .01979 L
.18221 .01979 L
.22272 .0198 L
.26171 .0198 L
.27193 .0198 L
.27731 .0198 L
.28302 .0198 L
.28542 .0198 L
.28674 .0198 L
.288 .0198 L
.28871 .0198 L
.28948 .01959 L
.29018 .01858 L
.29083 .01769 L
.29149 .01686 L
.2922 .01608 L
.29349 .01505 L
.29477 .01472 L
.29594 .01516 L
.29702 .01635 L
.29817 .01853 L
.2994 .02196 L
.30054 .02636 L
.3018 .03276 L
.30316 .04163 L
.30572 .06493 L
.30701 .08026 L
.30844 .10156 L
.31096 .15435 L
.31333 .2101 L
.3191 .35655 L
.32163 .41934 L
.3244 .48213 L
.32668 .52696 L
.32795 .54635 L
.32911 .55934 L
.33044 .57174 L
.33168 .58127 L
.33282 .58826 L
.33408 .59425 L
.33534 .59864 L
.33604 .6004 L
.33668 .60167 L
.33784 .60301 L
.33909 .60332 L
.33978 .60306 L
.34054 .60248 L
Mistroke
.34126 .60167 L
.34192 .60072 L
.34452 .59561 L
.36508 .56572 L
.3837 .54185 L
.42382 .48979 L
.46242 .43679 L
.50348 .37813 L
.54301 .32187 L
.58104 .27038 L
.62152 .22044 L
.66048 .17839 L
.70189 .14073 L
.74179 .11118 L
.78017 .08848 L
.821 .0697 L
.86032 .05599 L
.89812 .04606 L
.93838 .03826 L
.97619 .03294 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{180.312, 111.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003Q0000Ra000`40O003h00OhGoo003QOol00>5o
o`00BWoo0`008Goo1@000goo0`0077oo1@000Woo1@0077oo1@0017oo0`0067oo0019Ool01@00Oomo
ogoo00008Woo00<007ooOol00Woo00D007ooOomoo`0001eoo`03001oogoo00=oo`04001oogoo000N
Ool00`00Oomoo`05Ool00`00Oomoo`0GOol004Uoo`05001oogooOol0000ROol00`00Oomoo`02Ool0
1@00Oomoogoo00007Goo00<007ooOol017oo00<007ooOol07Woo00<007ooOol00Woo1@0067oo0019
Ool01@00Oomoogoo00008Woo00<007ooOol00Woo00D007ooOomoo`0001eoo`03001oogoo00Eoo`03
001oogoo01eoo`03001oogoo009oo`04001oogoo000IOol004Yoo`<002=oo`03001oogoo009oo`05
001oogooOol0000MOol00`00Oomoo`06Ool00`00Oomoo`0LOol00`00Oomoo`03Ool00`00Ool0000I
Ool004Uoo`05001oogooOol0000ROol00`00Oomoo`02Ool01@00Oomoogoo00007Goo00<007ooOol0
1Woo00<007ooOol077oo00<007ooOol00goo00<007oo00006Goo0019Ool01@00Oomoogoo00008Woo
00<007ooOol00Woo00D007ooOomoo`0001eoo`03001oogoo009oo`05001oogooOol0000NOol00`00
Oomoo`04Ool2000IOol002Eoo`03001oogoo029oo`<0025oo`<000Eoo`<001aoo`<000Eoo`<001eo
o`<000Moo`03001oogoo01Moo`009Goo00<007ooOol0^Goo000UOol00`00Oomoo`2iOol002Eoo`03
001oogoo025oo`03001oogoo09Eoo`0037ood0001Goo000AOol00`00Oomoo`07Ool00`00Oomoo`07
Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`05Ool00`00Ool00009
Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`06
Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`07
Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`07Ool002Eoo`03001o
ogoo029oo`03001oo`0002Moo`03001oogoo02Aoo`03001oogoo02Eoo`03001oogoo01]oo`009Goo
00<007ooOol08Woo00<007ooOol0QWoo1@002Goo000UOol00`00Oomoo`0ROol00`00Oomoo`20Ool6
000>Ool002Eoo`03001oogoo02=oo`03001oogoo07Yoo`D001Aoo`009Goo00<007ooOol08goo00<0
07ooOol0MWoo10006Goo000UOol00`00Oomoo`0SOol00`00Oomoo`1bOol4000MOol002Eoo`03001o
ogoo02=oo`03001oogoo06moo`<0025oo`009Goo00<007ooOol08goo00<007ooOol0K7oo0`0097oo
000UOol00`00Oomoo`0SOol00`00Oomoo`1YOol3000WOol002Eoo`03001oogoo02=oo`03001oogoo
06Moo`8002Yoo`009Goo00<007ooOol097oo00<007ooOol0I7oo0P00;7oo000UOol00`00Oomoo`0T
Ool00`00Oomoo`1ROol2000^Ool000Eoo`<000Ioo`8000Eoo`<000Aoo`@000Eoo`03001oogoo02Ao
o`03001oogoo061oo`80031oo`0017oo00D007ooOomoo`0000Eoo`8000Aoo`05001oogooOol00007
Ool00`00Oomoo`02Ool00`00Oomoo`0TOol00`00Oomoo`1NOol2000bOol000Aoo`05001oogooOol0
000;Ool01@00Oomoogoo00001goo00<007ooOol00Woo00<007ooOol097oo00<007ooOol0G7oo0P00
=7oo0004Ool01@00Oomoogoo00002goo00D007ooOomoo`0000Moo`03001oogoo009oo`8002Eoo`03
001oogoo05Yoo`8003Ioo`0017oo00D007ooOomoo`0000]oo`05001oogooOol00003Ool40005Ool0
0`00Oomoo`0TOol00`00Oomoo`1HOol2000hOol000Aoo`05001oogooOol0000;Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00<007ooOol097oo00<007ooOol0EWoo0P00>Woo0004Ool01@00Oomo
ogoo00002goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`03001oogoo02Aoo`03001oogoo05Eo
o`03001oogoo03Yoo`001Goo0`003Goo0`0017oo10001Goo00<007ooOol097oo00<007ooOol0E7oo
00<007ooOol0>goo000UOol00`00Oomoo`0TOol00`00Oomoo`1BOol2000nOol002Eoo`03001oogoo
02Aoo`03001oogoo055oo`03001oogoo03ioo`009Goo00<007ooOol097oo00<007ooOol0D7oo00<0
07ooOol0?goo000UOol00`00Oomoo`0TOol00`00Oomoo`1>Ool20012Ool002Eoo`03001oogoo02Ao
o`03001oogoo04eoo`03001oogoo049oo`009Goo00<007ooOol097oo00<007ooOol0C7oo00<007oo
Ool0@goo000UOol00`00Oomoo`0TOol00`00Oomoo`1;Ool00`00Oomoo`14Ool002Eoo`03001oogoo
02Aoo`03001oogoo04Yoo`03001oogoo04Eoo`009Goo00<007ooOol097oo00<007ooOol0BGoo00<0
07ooOol0AWoo000=Ool30006Ool20005Ool50003Ool00`00Oomoo`0TOol00`00Oomoo`18Ool00`00
Oomoo`17Ool000aoo`05001oogooOol00005Ool20007Ool00`00Oomoo`03Ool00`00Oomoo`0TOol0
0`00Oomoo`17Ool00`00Oomoo`18Ool000aoo`05001oogooOol0000>Ool00`00Oomoo`03Ool2000U
Ool00`00Oomoo`16Ool00`00Oomoo`19Ool000aoo`05001oogooOol0000>Ool00`00Oomoo`03Ool0
0`00Oomoo`0TOol00`00Oomoo`16Ool00`00Oomoo`19Ool000aoo`05001oogooOol0000>Ool00`00
Oomoo`03Ool00`00Oomoo`0UOol00`00Oomoo`14Ool00`00Oomoo`1:Ool000aoo`05001oogooOol0
000>Ool00`00Oomoo`03Ool00`00Oomoo`0UOol00`00Oomoo`13Ool00`00Oomoo`1;Ool000aoo`05
001oogooOol0000>Ool00`00Oomoo`03Ool00`00Oomoo`0UOol00`00Oomoo`12Ool00`00Oomoo`1<
Ool000eoo`<000eoo`<000Eoo`03001oogoo02Eoo`03001oogoo045oo`03001oogoo04eoo`009Goo
00<007ooOol09Goo00<007ooOol0@7oo00<007ooOol0CWoo000UOol00`00Oomoo`0UOol00`00Oomo
o`0oOol00`00Oomoo`1?Ool002Eoo`03001oogoo02Eoo`03001oogoo03ioo`03001oogoo051oo`00
9Goo00<007ooOol09Goo00<007ooOol0?Goo00<007ooOol0DGoo000UOol00`00Oomoo`0UOol00`00
Oomoo`0mOol00`00Oomoo`1AOol002Eoo`03001oogoo02Eoo`03001oogoo03aoo`03001oogoo059o
o`009Goo00<007ooOol09Goo00<007ooOol0>goo00<007ooOol0Dgoo000UOol00`00Oomoo`0UOol0
0`00Oomoo`0jOol00`00Oomoo`1DOol000Eoo`<000Ioo`8000Eoo`D0009oo`@000Eoo`03001oogoo
02Eoo`03001oogoo03Uoo`03001oogoo05Eoo`0017oo00D007ooOomoo`0000Eoo`8000Moo`03001o
ogoo00Ioo`03001oogoo009oo`03001oogoo02Eoo`03001oogoo03Qoo`03001oogoo05Ioo`0017oo
00D007ooOomoo`0000ioo`03001oogoo00Ioo`03001oogoo009oo`03001oogoo02Eoo`03001oogoo
03Moo`03001oogoo05Moo`0017oo00D007ooOomoo`0000ioo`03001oogoo00Ioo`03001oogoo009o
o`8002Ioo`03001oogoo03Ioo`03001oogoo05Qoo`0017oo00D007ooOomoo`0000ioo`03001oogoo
009oo`@000Eoo`03001oogoo02Eoo`03001oogoo03Eoo`03001oogoo05Uoo`0017oo00D007ooOomo
o`0000ioo`03001oogoo009oo`03001oogoo00Ioo`03001oogoo02Eoo`03001oogoo03Aoo`03001o
ogoo05Yoo`0017oo00D007ooOomoo`0000ioo`03001oogoo009oo`03001oogoo00Ioo`03001oogoo
02Eoo`03001oogoo03Aoo`03001oogoo05Yoo`001Goo0`003Goo0`0017oo10001Goo00<007ooOol0
9Goo00<007ooOol0<goo00<007ooOol0Fgoo000UOol00`00Oomoo`0UOol00`00Oomoo`0bOol00`00
Oomoo`1LOol002Eoo`03001oogoo02Ioo`03001oogoo035oo`03001oogoo05aoo`009Goo00<007oo
Ool09Woo00<007ooOol0<7oo00<007ooOol0GGoo000UOol00`00Oomoo`0VOol00`00Oomoo`0_Ool0
0`00Oomoo`1NOol002Eoo`03001oogoo02Ioo`03001oogoo02moo`03001oogoo05ioo`009Goo00<0
07ooOol09Woo00<007ooOol0;Woo00<007ooOol0Ggoo000UOol00`00Oomoo`0VOol00`00Oomoo`0^
Ool00`00Oomoo`1OOol002Eoo`03001oogoo02Ioo`03001oogoo02eoo`03001oogoo061oo`009Goo
00<007ooOol09Woo00<007ooOol0;7oo00<007ooOol0HGoo000=Ool30006Ool20004Ool50004Ool0
0`00Oomoo`0VOol00`00Oomoo`0/Ool00`00Oomoo`1QOol000aoo`05001oogooOol00005Ool20005
Ool01000Oomoo`0017oo00<007ooOol09Woo00<007ooOol0:goo00<007ooOol0HWoo000<Ool01@00
Oomoogoo00003Goo00<007ooOol017oo0P009goo00<007ooOol0:Woo00<007ooOol0Hgoo000<Ool0
1@00Oomoogoo00003Woo00<007ooOol00goo00<007ooOol09Woo00<007ooOol0:Goo00<007ooOol0
I7oo000<Ool01@00Oomoogoo00003goo00<007ooOol00Woo00<007ooOol09Woo00<007ooOol0:Goo
00<007ooOol0I7oo000<Ool01@00Oomoogoo00003goo00<007ooOol00Woo00<007ooOol09Woo00<0
07ooOol0:7oo00<007ooOol0IGoo000<Ool01@00Oomoogoo00002goo00D007ooOomoo`0000Aoo`03
001oogoo02Ioo`03001oogoo02Moo`03001oogoo06Ioo`003Goo0`003Goo0`001Goo00<007ooOol0
9Woo00<007ooOol09Woo00<007ooOol0Igoo000UOol00`00Oomoo`0VOol00`00Oomoo`0VOol00`00
Oomoo`1WOol002Eoo`03001oogoo02Ioo`03001oogoo02Eoo`03001oogoo06Qoo`009Goo00<007oo
Ool09Woo00<007ooOol097oo00<007ooOol0JGoo000UOol00`00Oomoo`0VOol00`00Oomoo`0SOol0
0`00Oomoo`1ZOol002Eoo`03001oogoo02Ioo`03001oogoo02=oo`03001oogoo06Yoo`009Goo00<0
07ooOol09goo00<007ooOol08Goo00<007ooOol0Jgoo000UOol00`00Oomoo`0WOol00`00Oomoo`0P
Ool00`00Oomoo`1/Ool002Eoo`03001oogoo02Moo`03001oogoo021oo`03001oogoo06aoo`001Goo
0`001Woo0P0017oo1@000goo10001Goo00<007ooOol09goo00<007ooOol07goo00<007ooOol0KGoo
0004Ool01@00Oomoogoo00001Goo0P001Goo00@007ooOol000Moo`03001oogoo009oo`03001oogoo
02Moo`03001oogoo01ioo`03001oogoo06ioo`0017oo00D007ooOomoo`0000eoo`03001oogoo00Mo
o`03001oogoo009oo`03001oogoo02Moo`03001oogoo01ioo`03001oogoo06ioo`0017oo00D007oo
Oomoo`0000ioo`03001oogoo00Ioo`03001oogoo009oo`8002Qoo`03001oogoo01eoo`03001oogoo
06moo`0017oo00D007ooOomoo`0000moo`04001oogooOol40005Ool00`00Oomoo`0WOol00`00Oomo
o`0LOol00`00Oomoo`1`Ool000Aoo`05001oogooOol0000?Ool01@00Oomoogoo000027oo00<007oo
Ool09goo00<007ooOol077oo00<007ooOol0L7oo0004Ool01@00Oomoogoo00002goo00D007ooOomo
o`0000=oo`03001oogoo00Ioo`03001oogoo02Moo`03001oogoo01]oo`03001oogoo075oo`001Goo
0`003Goo0`0017oo10001Goo00<007ooOol09goo00<007ooOol06Woo00<007ooOol0LWoo000UOol0
0`00Oomoo`0WOol00`00Oomoo`0JOol00`00Oomoo`1bOol002Eoo`03001oogoo02Moo`03001oogoo
01Uoo`03001oogoo07=oo`009Goo00<007ooOol09goo00<007ooOol067oo00<007ooOol0M7oo000U
Ool00`00Oomoo`0WOol00`00Oomoo`0GOol00`00Oomoo`1eOol002Eoo`03001oogoo02Moo`03001o
ogoo01Moo`03001oogoo07Eoo`009Goo00<007ooOol09goo00<007ooOol05Woo00<007ooOol0MWoo
000UOol00`00Oomoo`0WOol00`00Oomoo`0EOol00`00Oomoo`1gOol002Eoo`03001oogoo02Moo`03
001oogoo01Eoo`03001oogoo07Moo`003Goo0`001Woo0P001Goo0`001Goo00<007ooOol09goo00<0
07ooOol057oo00<007ooOol0N7oo000<Ool01@00Oomoogoo00001Goo0P0017oo00D007ooOomoo`00
00Aoo`03001oogoo02Moo`03001oogoo01=oo`03001oogoo07Uoo`0037oo00D007ooOomoo`0000mo
o`03001oogoo009oo`03001oogoo02Moo`03001oogoo019oo`03001oogoo07Yoo`0037oo00D007oo
Oomoo`0000moo`03001oogoo009oo`8002Qoo`03001oogoo019oo`03001oogoo07Yoo`0037oo00D0
07ooOomoo`0000eoo`8000Eoo`03001oogoo02Moo`03001oogoo015oo`03001oogoo07]oo`0037oo
00D007ooOomoo`0000moo`03001oogoo009oo`03001oogoo02Moo`03001oogoo011oo`03001oogoo
07aoo`0037oo00D007ooOomoo`0000]oo`05001oogooOol00004Ool00`00Oomoo`0XOol00`00Oomo
o`0>Ool00`00Oomoo`1mOol000eoo`<000eoo`<000Eoo`03001oogoo02Qoo`03001oogoo00eoo`03
001oogoo07ioo`009Goo00<007ooOol0:7oo00<007ooOol037oo00<007ooOol0Ogoo000UOol00`00
Oomoo`0XOol00`00Oomoo`0<Ool00`00Oomoo`1oOol002Eoo`03001oogoo02Qoo`03001oogoo00]o
o`03001oogoo081oo`009Goo00<007ooOol0:7oo00<007ooOol02Woo00<007ooOol0PGoo000UOol0
0`00Oomoo`0XOol00`00Oomoo`09Ool00`00Oomoo`22Ool002Eoo`03001oogoo02Qoo`03001oogoo
00Qoo`03001oogoo08=oo`009Goo00<007ooOol0:7oo00<007ooOol027oo00<007ooOol0Pgoo000U
Ool00`00Oomoo`0XOol00`00Oomoo`07Ool00`00Oomoo`24Ool002Eoo`03001oogoo02Qoo`03001o
ogoo00Ioo`03001oogoo08Eoo`001Goo0`001Woo0P001Goo0`0017oo10001Goo00<007ooOol0:7oo
00<007ooOol01Woo00<007ooOol0QGoo0004Ool01@00Oomoogoo00001Goo0P0017oo00D007ooOomo
o`0000Moo`03001oogoo009oo`03001oogoo02Qoo`03001oogoo00Eoo`03001oogoo08Ioo`0017oo
00D007ooOomoo`0000moo`03001oogoo00Eoo`03001oogoo009oo`8002Uoo`03001oogoo00Eoo`03
001oogoo08Ioo`0017oo00D007ooOomoo`0000moo`03001oogoo00Eoo`03001oogoo009oo`03001o
ogoo02Qoo`03001oogoo00Aoo`03001oogoo08Moo`0017oo00D007ooOomoo`0000eoo`8000Aoo`@0
00Eoo`03001oogoo02Uoo`03001oogoo009oo`03001oogoo08Qoo`0017oo00D007ooOomoo`0000mo
o`05001oogooOol00008Ool00`00Oomoo`0YOol01@00Oomoogoo0000Rgoo0004Ool01@00Oomoogoo
00002goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`03001oogoo02Uoo`04001oogoo002<Ool0
00Eoo`<000eoo`<000Aoo`@000Eoo`03001oogoo02Uoo`03001oo`0008eoo`009Goo00<007ooOol0
:Woo0P00SGoo000UOol00`00Oomoo`2iOol002Eoo`03001oogoo0;Uoo`009Goo00<007ooOol0^Goo
0000\
\>"],
  ImageRangeCache->{{{0, 224.375}, {138.062, 0}} -> {4.12801, -0.0420866, 
  0.0632076, 0.00379936}}]
}, Open  ]],

Cell[TextData[{
  "Note that the interpolation scheme has also produced a spurious negative \
value for \[CapitalGamma] near ",
  Cell[BoxData[
      \(TraditionalForm\`S = 8\)]],
  ". We need to make a modification of the means by which \[CapitalGamma] is \
estimated in a neighbourhood of the front. The corresponding plot for \
\[CapitalDelta] is"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      SORDelta[10, \ 0.05, \ 0, \ S, \ 1, \ 0.2], \ {S, 5, \ 15}]; \)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.452381 0.0952381 0.603319 0.58817 [
[.30952 .59082 -3 -9 ]
[.30952 .59082 3 0 ]
[.5 .59082 -6 -9 ]
[.5 .59082 6 0 ]
[.69048 .59082 -6 -9 ]
[.69048 .59082 6 0 ]
[.88095 .59082 -6 -9 ]
[.88095 .59082 6 0 ]
[.10655 .01515 -12 -4.5 ]
[.10655 .01515 0 4.5 ]
[.10655 .13278 -24 -4.5 ]
[.10655 .13278 0 4.5 ]
[.10655 .25042 -24 -4.5 ]
[.10655 .25042 0 4.5 ]
[.10655 .36805 -24 -4.5 ]
[.10655 .36805 0 4.5 ]
[.10655 .48568 -24 -4.5 ]
[.10655 .48568 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.30952 .60332 m
.30952 .60957 L
s
[(8)] .30952 .59082 0 1 Mshowa
.5 .60332 m
.5 .60957 L
s
[(10)] .5 .59082 0 1 Mshowa
.69048 .60332 m
.69048 .60957 L
s
[(12)] .69048 .59082 0 1 Mshowa
.88095 .60332 m
.88095 .60957 L
s
[(14)] .88095 .59082 0 1 Mshowa
.125 Mabswid
.16667 .60332 m
.16667 .60707 L
s
.21429 .60332 m
.21429 .60707 L
s
.2619 .60332 m
.2619 .60707 L
s
.35714 .60332 m
.35714 .60707 L
s
.40476 .60332 m
.40476 .60707 L
s
.45238 .60332 m
.45238 .60707 L
s
.54762 .60332 m
.54762 .60707 L
s
.59524 .60332 m
.59524 .60707 L
s
.64286 .60332 m
.64286 .60707 L
s
.7381 .60332 m
.7381 .60707 L
s
.78571 .60332 m
.78571 .60707 L
s
.83333 .60332 m
.83333 .60707 L
s
.07143 .60332 m
.07143 .60707 L
s
.02381 .60332 m
.02381 .60707 L
s
.92857 .60332 m
.92857 .60707 L
s
.97619 .60332 m
.97619 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.11905 .01515 m
.1253 .01515 L
s
[(-1)] .10655 .01515 1 0 Mshowa
.11905 .13278 m
.1253 .13278 L
s
[(-0.8)] .10655 .13278 1 0 Mshowa
.11905 .25042 m
.1253 .25042 L
s
[(-0.6)] .10655 .25042 1 0 Mshowa
.11905 .36805 m
.1253 .36805 L
s
[(-0.4)] .10655 .36805 1 0 Mshowa
.11905 .48568 m
.1253 .48568 L
s
[(-0.2)] .10655 .48568 1 0 Mshowa
.125 Mabswid
.11905 .04456 m
.1228 .04456 L
s
.11905 .07397 m
.1228 .07397 L
s
.11905 .10337 m
.1228 .10337 L
s
.11905 .16219 m
.1228 .16219 L
s
.11905 .1916 m
.1228 .1916 L
s
.11905 .22101 m
.1228 .22101 L
s
.11905 .27983 m
.1228 .27983 L
s
.11905 .30923 m
.1228 .30923 L
s
.11905 .33864 m
.1228 .33864 L
s
.11905 .39746 m
.1228 .39746 L
s
.11905 .42687 m
.1228 .42687 L
s
.11905 .45628 m
.1228 .45628 L
s
.11905 .51509 m
.1228 .51509 L
s
.11905 .5445 m
.1228 .5445 L
s
.11905 .57391 m
.1228 .57391 L
s
.25 Mabswid
.11905 0 m
.11905 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01508 m
.06244 .01508 L
.10458 .01508 L
.14415 .01508 L
.18221 .01508 L
.22272 .01508 L
.26171 .01508 L
.27193 .01508 L
.28302 .01508 L
.2881 .01508 L
.29349 .01508 L
.2957 .01508 L
.29693 .01508 L
.29809 .01508 L
.29874 .01507 L
.29945 .015 L
.3007 .01488 L
.30198 .01478 L
.30316 .01473 L
.30444 .01472 L
.30509 .01473 L
.3058 .01477 L
.30706 .0149 L
.30823 .0151 L
.30962 .01542 L
.31088 .01583 L
.31234 .01647 L
.31372 .01725 L
.31492 .01807 L
.3162 .01911 L
.31849 .02161 L
.32096 .02528 L
.32368 .03001 L
.34228 .07141 L
.38249 .15752 L
.42118 .23273 L
.46232 .3042 L
.50194 .36429 L
.54005 .41382 L
.58061 .45784 L
.61965 .49232 L
.66115 .5214 L
.70113 .54309 L
.7396 .55908 L
.78052 .57186 L
.81992 .58092 L
.85781 .58731 L
.89815 .59223 L
.93697 .59559 L
.97619 .59799 L
Mistroke
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{180.312, 111.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003Q0000Ra000`40O003h00OhGoo003QOol00>5o
o`0087oo00<007ooOol0_Woo000HOol50003Ool00`00Oomoo`2nOol001Yoo`03001oogoo00=oo`03
001oogoo0;ioo`003goo1P001Goo00<007ooOol00goo00<007ooOol0_Woo000<Oom0002EOol001Yo
o`03001oogoo00=oo`03001oogoo02Qoo`8009Aoo`006Woo00<007ooOol00goo00<007ooOol0:Goo
00<007ooOol0TWoo000JOol00`00Oomoo`03Ool00`00Oomoo`0ZOol00`00Oomoo`2AOol001Qoo`<0
00Eoo`03001oogoo02Yoo`03001oogoo095oo`0087oo00<007ooOol0:goo00<007ooOol0T7oo000P
Ool2000/Ool00`00Oomoo`2@Ool0021oo`03001oogoo02aoo`03001oogoo08moo`0087oo00<007oo
Ool0;7oo00<007ooOol0Sgoo000POol00`00Oomoo`0/Ool00`00Oomoo`2?Ool0021oo`03001oogoo
02eoo`03001oogoo08ioo`0087oo00<007ooOol0;Goo00<007ooOol0SWoo000POol2000_Ool00`00
Oomoo`2=Ool0021oo`03001oogoo02ioo`03001oogoo08eoo`0087oo00<007ooOol0;goo00<007oo
Ool0S7oo000POol00`00Oomoo`0_Ool00`00Oomoo`2<Ool0021oo`03001oogoo031oo`03001oogoo
08]oo`0087oo00<007ooOol0<7oo00<007ooOol0Rgoo000POol00`00Oomoo`0aOol00`00Oomoo`2:
Ool0021oo`80039oo`03001oogoo08Yoo`0087oo00<007ooOol0<Woo00<007ooOol0RGoo000POol0
0`00Oomoo`0bOol00`00Oomoo`29Ool0021oo`03001oogoo03=oo`03001oogoo08Qoo`0027oo0`00
1Woo0P0017oo0`001Woo00<007ooOol0<goo00<007ooOol0R7oo0007Ool01@00Oomoogoo00001Goo
0P000goo00D007ooOomoo`0000Eoo`03001oogoo03=oo`03001oogoo08Qoo`001@000Woo00D007oo
Oomoo`0000Yoo`05001oogooOol00005Ool3000dOol00`00Oomoo`27Ool000Moo`05001oogooOol0
000:Ool01@00Oomoogoo00001Goo00<007ooOol0=7oo00<007ooOol0Qgoo0007Ool01@00Oomoogoo
00002goo0`001Woo00<007ooOol0=Goo00<007ooOol0QWoo0007Ool01@00Oomoogoo00002Woo00D0
07ooOomoo`0000Eoo`03001oogoo03Eoo`03001oogoo08Ioo`001goo00D007ooOomoo`0000Yoo`05
001oogooOol00005Ool00`00Oomoo`0fOol00`00Oomoo`25Ool000Qoo`<000aoo`<000Ioo`03001o
ogoo03Ioo`03001oogoo08Eoo`0087oo00<007ooOol0=goo00<007ooOol0Q7oo000POol2000hOol0
0`00Oomoo`24Ool0021oo`03001oogoo03Qoo`03001oogoo08=oo`0087oo00<007ooOol0>7oo00<0
07ooOol0Pgoo000POol00`00Oomoo`0iOol00`00Oomoo`22Ool0021oo`03001oogoo03Uoo`03001o
ogoo089oo`0087oo00<007ooOol0>Woo00<007ooOol0PGoo000POol2000kOol00`00Oomoo`21Ool0
021oo`03001oogoo03]oo`03001oogoo081oo`0087oo00<007ooOol0>goo00<007ooOol0P7oo000P
Ool00`00Oomoo`0lOol00`00Oomoo`1oOol0021oo`03001oogoo03aoo`03001oogoo07moo`0087oo
00<007ooOol0?Goo00<007ooOol0OWoo000POol2000nOol00`00Oomoo`1nOol0021oo`03001oogoo
03ioo`03001oogoo07eoo`0087oo00<007ooOol0?Woo00<007ooOol0OGoo000POol00`00Oomoo`0o
Ool00`00Oomoo`1lOol000Qoo`<000Ioo`8000Aoo`<000Ioo`03001oogoo03moo`03001oogoo07ao
o`001goo00D007ooOomoo`0000Eoo`8000=oo`05001oogooOol00005Ool00`00Oomoo`10Ool00`00
Oomoo`1kOol000D0009oo`05001oogooOol0000:Ool01@00Oomoogoo00001Goo00<007ooOol0@7oo
00<007ooOol0Ngoo0007Ool01@00Oomoogoo00002Woo00D007ooOomoo`0000Eoo`<0045oo`03001o
ogoo07Yoo`001goo00D007ooOomoo`0000Yoo`@000Ioo`03001oogoo049oo`03001oogoo07Uoo`00
1goo00D007ooOomoo`0000Yoo`03001oogoo00Moo`03001oogoo049oo`03001oogoo07Uoo`001goo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo04=oo`03001oogoo07Qoo`0027oo0`00
3Goo0`001Goo00<007ooOol0@goo00<007ooOol0N7oo000POol00`00Oomoo`14Ool00`00Oomoo`1g
Ool0021oo`8004Eoo`03001oogoo07Moo`0087oo00<007ooOol0AGoo00<007ooOol0MWoo000POol0
0`00Oomoo`15Ool00`00Oomoo`1fOol0021oo`03001oogoo04Ioo`03001oogoo07Eoo`0087oo00<0
07ooOol0Agoo00<007ooOol0M7oo000POol00`00Oomoo`17Ool00`00Oomoo`1dOol0021oo`03001o
ogoo04Qoo`03001oogoo07=oo`0087oo0P00BGoo00<007ooOol0Lgoo000POol00`00Oomoo`19Ool0
0`00Oomoo`1bOol0021oo`03001oogoo04Yoo`03001oogoo075oo`0087oo00<007ooOol0BWoo00<0
07ooOol0LGoo000POol00`00Oomoo`1;Ool00`00Oomoo`1`Ool0021oo`03001oogoo04aoo`03001o
ogoo06moo`0087oo0P00CWoo00<007ooOol0KWoo000POol00`00Oomoo`1=Ool00`00Oomoo`1^Ool0
021oo`03001oogoo04ioo`03001oogoo06eoo`0087oo00<007ooOol0Cgoo00<007ooOol0K7oo0008
Ool30006Ool20005Ool30005Ool00`00Oomoo`1?Ool00`00Oomoo`1/Ool000Moo`05001oogooOol0
0005Ool20006Ool00`00Oomoo`04Ool00`00Oomoo`1@Ool00`00Oomoo`1[Ool000D0009oo`05001o
ogooOol0000:Ool50005Ool3001AOol00`00Oomoo`1ZOol000Moo`05001oogooOol0000:Ool01000
Oomoo`001Woo00<007ooOol0DGoo00<007ooOol0JWoo0007Ool01@00Oomoogoo00002goo00<007oo
00001Woo00<007ooOol0DWoo00<007ooOol0JGoo0007Ool01@00Oomoogoo00002goo00<007oo0000
1Woo00<007ooOol0Dgoo00<007ooOol0J7oo0007Ool01@00Oomoogoo000037oo0P001Woo00<007oo
Ool0E7oo00<007ooOol0Igoo0008Ool3000>Ool00`00Oomoo`04Ool00`00Oomoo`1EOol00`00Oomo
o`1VOol0021oo`03001oogoo05Eoo`03001oogoo06Ioo`0087oo0P00Egoo00<007ooOol0IGoo000P
Ool00`00Oomoo`1GOol00`00Oomoo`1TOol0021oo`03001oogoo05Qoo`03001oogoo06=oo`0087oo
00<007ooOol0FGoo00<007ooOol0HWoo000POol00`00Oomoo`1IOol00`00Oomoo`1ROol0021oo`03
001oogoo05Yoo`03001oogoo065oo`0087oo0P00G7oo00<007ooOol0H7oo000POol00`00Oomoo`1L
Ool00`00Oomoo`1OOol0021oo`03001oogoo05eoo`03001oogoo05ioo`0087oo00<007ooOol0GWoo
00<007ooOol0GGoo000POol00`00Oomoo`1OOol00`00Oomoo`1LOol0021oo`03001oogoo061oo`03
001oogoo05]oo`0087oo00<007ooOol0HGoo00<007ooOol0FWoo000POol2001SOol00`00Oomoo`1I
Ool0021oo`03001oogoo06=oo`03001oogoo05Qoo`0087oo00<007ooOol0I7oo00<007ooOol0Egoo
000POol00`00Oomoo`1UOol00`00Oomoo`1FOol000Qoo`<000Ioo`8000=oo`D000Eoo`03001oogoo
06Ioo`03001oogoo05Eoo`001goo00D007ooOomoo`0000Eoo`8000Aoo`04001oogoo0005Ool00`00
Oomoo`1WOol00`00Oomoo`1DOol000D0009oo`05001oogooOol0000<Ool00`00Oomoo`05Ool3001X
Ool00`00Oomoo`1COol000Moo`05001oogooOol0000=Ool00`00Oomoo`04Ool00`00Oomoo`1YOol0
0`00Oomoo`1BOol000Moo`05001oogooOol0000>Ool00`00Oomoo`03Ool00`00Oomoo`1ZOol2001B
Ool000Moo`05001oogooOol0000>Ool00`00Oomoo`03Ool00`00Oomoo`1/Ool00`00Oomoo`1?Ool0
00Moo`05001oogooOol0000:Ool01@00Oomoogoo00001Goo00<007ooOol0KGoo0P00Cgoo0008Ool3
000<Ool30006Ool00`00Oomoo`1_Ool00`00Oomoo`1<Ool0021oo`80075oo`8004aoo`0087oo00<0
07ooOol0LWoo00<007ooOol0BGoo000POol00`00Oomoo`1cOol20019Ool0021oo`03001oogoo07Eo
o`8004Moo`0087oo00<007ooOol0Mgoo0P00AGoo000POol00`00Oomoo`1iOol00`00Oomoo`12Ool0
021oo`03001oogoo07Yoo`80049oo`0087oo0P00OGoo0P00@7oo000POol00`00Oomoo`0UOol3000R
Ool50003Ool3000OOol50002Ool6000LOol50004Ool3000EOol0021oo`03001oogoo02Aoo`05001o
ogooOol0000SOol00`00Oomoo`02Ool01@00Oomoogoo000087oo00<007ooOol00goo00D007ooOol0
07oo00<001]oo`03001oogoo00Eoo`03001oogoo01Aoo`0087oo00<007ooOol097oo00D007ooOomo
o`0002=oo`03001oogoo009oo`05001oogooOol0000POol00`00Oomoo`04Ool00`00Oomoo`04Ool3
000HOol00`00Oomoo`02Ool5000EOol0021oo`03001oogoo02Aoo`05001oogooOol0000SOol00`00
Oomoo`02Ool01@00Oomoogoo000087oo00<007ooOol01Goo00<007ooOol01Woo0`005Goo00<007oo
Ool00Woo00@007ooOol001Ioo`0087oo00<007ooOol09Goo0`0097oo00<007ooOol00Woo00D007oo
Oomoo`00021oo`03001oogoo00Ioo`03001oogoo00Qoo`<0019oo`03001oogoo00=oo`03001oo`00
01Ioo`0087oo0P009Goo00D007ooOomoo`0002=oo`03001oogoo009oo`05001oogooOol0000POol0
0`00Oomoo`06Ool00`00Oomoo`0;Ool3000?Ool00`00Oomoo`03Ool00`00Ool0000FOol0021oo`03
001oogoo02Aoo`05001oogooOol0000SOol00`00Oomoo`02Ool01@00Oomoogoo000087oo00<007oo
Ool00Woo00D007ooOomoo`00011oo`D000Yoo`03001oogoo00Aoo`8001Ioo`0087oo00<007ooOol0
9Goo0`008Woo0`001Goo0`007goo0`001Goo0`005Woo1P000Woo0`001goo00<007ooOol057oo000P
Ool00`00Oomoo`2KOol9000JOol0021oo`03001oogoo0:Aoo`P0019oo`0087oo00<007ooOol0[7oo
3@001Goo000POol00`00Oomoo`2nOol000Ioom/0000087oo00<007ooOol09goo00<007ooOol09Woo
00<007ooOol09goo00<007ooOol09Woo00<007ooOol067oo000POol00`00Oomoo`2nOol0021oo`03
001oogoo0;ioo`00\
\>"],
  ImageRangeCache->{{{0, 224.375}, {138.062, 0}} -> {4.42081, -1.04896, 
  0.06033, 0.0097688}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Revised Method for Greeks", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
In this sub-section we define some new functions to compute Delta, \
Gamma and Theta directly from the finite-difference grid. Note that these are \
based on the same simple differencing and interpolation methods as were used \
for the European case, but we should expect these to introduce some \
inappropriate smoothing in the neighbourhood of the critical price frontier, \
where \[CapitalGamma] is actually discontinuous.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
listd[data_, step_] := Module[{dleft,dright,len},
len = Length[data]; 
dleft = (4*data[[2]]-3*data[[1]]-data[[3]])/(2*step);
dright = (3*data[[len]]-4*data[[len-1]]+data[[len-2]])/(2*step);
Join[{dleft}, Take[RotateLeft[data]-RotateRight[data], {2, -2}]/(2*step), \
{dright}]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(kt\  = \ ktwo[0.05, \ 0, \ 0.2]; \ ko\  = \ kone[0.05, \ 0.2]; \)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(rawdeltadata = 
      listd[finalstep, dx] - 1\/2\ \((kt - 1)\)\ finalstep; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(rawgammadata = 
      listd[rawdeltadata, dx] - 1\/2\ \((kt + 1)\)\ rawdeltadata; \)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[TextData[{
  "At this point we interrupt the process, and work out the derivative of \
\[CapitalGamma] with respect to ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " - this has a spike at the frontier:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(thirddata\  = \ 
      listd[rawgammadata, \ dx]; \[IndentingNewLine]thirdinterpoldata = 
      Transpose[{points, thirddata}]; \[IndentingNewLine]ListPlot[
      thirdinterpoldata, \ PlotJoined\  -> \ True, \ 
      PlotRange\  -> \ All]; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.65873 0.15873 0.0687647 0.00769441 [
[.02381 .05626 -6 -9 ]
[.02381 .05626 6 0 ]
[.18254 .05626 -6 -9 ]
[.18254 .05626 6 0 ]
[.34127 .05626 -6 -9 ]
[.34127 .05626 6 0 ]
[.5 .05626 -6 -9 ]
[.5 .05626 6 0 ]
[.81746 .05626 -3 -9 ]
[.81746 .05626 3 0 ]
[.97619 .05626 -3 -9 ]
[.97619 .05626 3 0 ]
[.64623 .22265 -12 -4.5 ]
[.64623 .22265 0 4.5 ]
[.64623 .37654 -12 -4.5 ]
[.64623 .37654 0 4.5 ]
[.64623 .53043 -12 -4.5 ]
[.64623 .53043 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .06876 m
.02381 .07501 L
s
[(-4)] .02381 .05626 0 1 Mshowa
.18254 .06876 m
.18254 .07501 L
s
[(-3)] .18254 .05626 0 1 Mshowa
.34127 .06876 m
.34127 .07501 L
s
[(-2)] .34127 .05626 0 1 Mshowa
.5 .06876 m
.5 .07501 L
s
[(-1)] .5 .05626 0 1 Mshowa
.81746 .06876 m
.81746 .07501 L
s
[(1)] .81746 .05626 0 1 Mshowa
.97619 .06876 m
.97619 .07501 L
s
[(2)] .97619 .05626 0 1 Mshowa
.125 Mabswid
.05556 .06876 m
.05556 .07251 L
s
.0873 .06876 m
.0873 .07251 L
s
.11905 .06876 m
.11905 .07251 L
s
.15079 .06876 m
.15079 .07251 L
s
.21429 .06876 m
.21429 .07251 L
s
.24603 .06876 m
.24603 .07251 L
s
.27778 .06876 m
.27778 .07251 L
s
.30952 .06876 m
.30952 .07251 L
s
.37302 .06876 m
.37302 .07251 L
s
.40476 .06876 m
.40476 .07251 L
s
.43651 .06876 m
.43651 .07251 L
s
.46825 .06876 m
.46825 .07251 L
s
.53175 .06876 m
.53175 .07251 L
s
.56349 .06876 m
.56349 .07251 L
s
.59524 .06876 m
.59524 .07251 L
s
.62698 .06876 m
.62698 .07251 L
s
.69048 .06876 m
.69048 .07251 L
s
.72222 .06876 m
.72222 .07251 L
s
.75397 .06876 m
.75397 .07251 L
s
.78571 .06876 m
.78571 .07251 L
s
.84921 .06876 m
.84921 .07251 L
s
.88095 .06876 m
.88095 .07251 L
s
.9127 .06876 m
.9127 .07251 L
s
.94444 .06876 m
.94444 .07251 L
s
.25 Mabswid
0 .06876 m
1 .06876 L
s
.65873 .22265 m
.66498 .22265 L
s
[(20)] .64623 .22265 1 0 Mshowa
.65873 .37654 m
.66498 .37654 L
s
[(40)] .64623 .37654 1 0 Mshowa
.65873 .53043 m
.66498 .53043 L
s
[(60)] .64623 .53043 1 0 Mshowa
.125 Mabswid
.65873 .10724 m
.66248 .10724 L
s
.65873 .14571 m
.66248 .14571 L
s
.65873 .18418 m
.66248 .18418 L
s
.65873 .26112 m
.66248 .26112 L
s
.65873 .2996 m
.66248 .2996 L
s
.65873 .33807 m
.66248 .33807 L
s
.65873 .41501 m
.66248 .41501 L
s
.65873 .45348 m
.66248 .45348 L
s
.65873 .49196 m
.66248 .49196 L
s
.65873 .03029 m
.66248 .03029 L
s
.65873 .5689 m
.66248 .5689 L
s
.65873 .60737 m
.66248 .60737 L
s
.25 Mabswid
.65873 0 m
.65873 .61803 L
s
.5 Mabswid
.02381 .06868 m
.02579 .06871 L
.02778 .06875 L
.02976 .06876 L
.03175 .06876 L
.03373 .06876 L
.03571 .06876 L
.0377 .06876 L
.03968 .06876 L
.04167 .06876 L
.04365 .06876 L
.04563 .06876 L
.04762 .06876 L
.0496 .06876 L
.05159 .06876 L
.05357 .06876 L
.05556 .06876 L
.05754 .06876 L
.05952 .06876 L
.06151 .06876 L
.06349 .06876 L
.06548 .06876 L
.06746 .06876 L
.06944 .06876 L
.07143 .06876 L
.07341 .06876 L
.0754 .06876 L
.07738 .06876 L
.07937 .06876 L
.08135 .06876 L
.08333 .06876 L
.08532 .06876 L
.0873 .06876 L
.08929 .06876 L
.09127 .06876 L
.09325 .06876 L
.09524 .06876 L
.09722 .06876 L
.09921 .06876 L
.10119 .06876 L
.10317 .06876 L
.10516 .06876 L
.10714 .06876 L
.10913 .06876 L
.11111 .06876 L
.1131 .06876 L
.11508 .06876 L
.11706 .06876 L
.11905 .06876 L
.12103 .06876 L
Mistroke
.12302 .06876 L
.125 .06876 L
.12698 .06876 L
.12897 .06876 L
.13095 .06876 L
.13294 .06876 L
.13492 .06876 L
.1369 .06876 L
.13889 .06876 L
.14087 .06876 L
.14286 .06876 L
.14484 .06876 L
.14683 .06876 L
.14881 .06876 L
.15079 .06876 L
.15278 .06876 L
.15476 .06876 L
.15675 .06876 L
.15873 .06876 L
.16071 .06876 L
.1627 .06876 L
.16468 .06876 L
.16667 .06876 L
.16865 .06876 L
.17063 .06876 L
.17262 .06876 L
.1746 .06876 L
.17659 .06876 L
.17857 .06876 L
.18056 .06876 L
.18254 .06876 L
.18452 .06876 L
.18651 .06876 L
.18849 .06876 L
.19048 .06876 L
.19246 .06876 L
.19444 .06876 L
.19643 .06876 L
.19841 .06876 L
.2004 .06876 L
.20238 .06876 L
.20437 .06876 L
.20635 .06876 L
.20833 .06876 L
.21032 .06876 L
.2123 .06876 L
.21429 .06876 L
.21627 .06876 L
.21825 .06876 L
.22024 .06876 L
Mistroke
.22222 .06876 L
.22421 .06876 L
.22619 .06876 L
.22817 .06876 L
.23016 .06876 L
.23214 .06876 L
.23413 .06876 L
.23611 .06876 L
.2381 .06876 L
.24008 .06876 L
.24206 .06876 L
.24405 .06876 L
.24603 .06876 L
.24802 .06876 L
.25 .06876 L
.25198 .06876 L
.25397 .06876 L
.25595 .06876 L
.25794 .06876 L
.25992 .06876 L
.2619 .06876 L
.26389 .06876 L
.26587 .06876 L
.26786 .06876 L
.26984 .06876 L
.27183 .06876 L
.27381 .06876 L
.27579 .06876 L
.27778 .06876 L
.27976 .06876 L
.28175 .06876 L
.28373 .06876 L
.28571 .06876 L
.2877 .06876 L
.28968 .06876 L
.29167 .06876 L
.29365 .06876 L
.29563 .06876 L
.29762 .06876 L
.2996 .06876 L
.30159 .06876 L
.30357 .06876 L
.30556 .06876 L
.30754 .06876 L
.30952 .06876 L
.31151 .06876 L
.31349 .06876 L
.31548 .06876 L
.31746 .06876 L
.31944 .06876 L
Mistroke
.32143 .06876 L
.32341 .06876 L
.3254 .06876 L
.32738 .06876 L
.32937 .06876 L
.33135 .06876 L
.33333 .06876 L
.33532 .06876 L
.3373 .06876 L
.33929 .06876 L
.34127 .06876 L
.34325 .06876 L
.34524 .06876 L
.34722 .06876 L
.34921 .06876 L
.35119 .06876 L
.35317 .06876 L
.35516 .06876 L
.35714 .06876 L
.35913 .06876 L
.36111 .06876 L
.3631 .06876 L
.36508 .06876 L
.36706 .06876 L
.36905 .06876 L
.37103 .06876 L
.37302 .06876 L
.375 .06876 L
.37698 .06876 L
.37897 .06876 L
.38095 .06876 L
.38294 .06876 L
.38492 .06876 L
.3869 .06876 L
.38889 .06876 L
.39087 .06876 L
.39286 .06876 L
.39484 .06876 L
.39683 .06876 L
.39881 .06876 L
.40079 .06876 L
.40278 .06876 L
.40476 .06876 L
.40675 .06876 L
.40873 .06876 L
.41071 .06876 L
.4127 .06876 L
.41468 .06876 L
.41667 .06876 L
.41865 .06876 L
Mistroke
.42063 .06876 L
.42262 .06876 L
.4246 .06876 L
.42659 .06876 L
.42857 .06876 L
.43056 .06876 L
.43254 .06876 L
.43452 .06876 L
.43651 .06876 L
.43849 .06876 L
.44048 .06876 L
.44246 .06876 L
.44444 .06876 L
.44643 .06876 L
.44841 .06876 L
.4504 .06876 L
.45238 .06876 L
.45437 .06876 L
.45635 .06876 L
.45833 .06876 L
.46032 .06876 L
.4623 .06876 L
.46429 .06876 L
.46627 .06876 L
.46825 .06876 L
.47024 .06876 L
.47222 .06876 L
.47421 .06876 L
.47619 .06876 L
.47817 .06876 L
.48016 .06876 L
.48214 .06876 L
.48413 .06876 L
.48611 .06876 L
.4881 .06876 L
.49008 .06876 L
.49206 .06876 L
.49405 .06876 L
.49603 .06876 L
.49802 .06876 L
.5 .06876 L
.50198 .06876 L
.50397 .06876 L
.50595 .06876 L
.50794 .06876 L
.50992 .06876 L
.5119 .06876 L
.51389 .06876 L
.51587 .06876 L
.51786 .06876 L
Mistroke
.51984 .06876 L
.52183 .06876 L
.52381 .06876 L
.52579 .06876 L
.52778 .06876 L
.52976 .06876 L
.53175 .06876 L
.53373 .06876 L
.53571 .06876 L
.5377 .06876 L
.53968 .06876 L
.54167 .06876 L
.54365 .06876 L
.54563 .06876 L
.54762 .06876 L
.5496 .06876 L
.55159 .06876 L
.55357 .06876 L
.55556 .06876 L
.55754 .06876 L
.55952 .06876 L
.56151 .06876 L
.56349 .06876 L
.56548 .06876 L
.56746 .06876 L
.56944 .06876 L
.57143 .06876 L
.57341 .06876 L
.5754 .06876 L
.57738 .06876 L
.57937 .06876 L
.58135 .06876 L
.58333 .06876 L
.58532 .06876 L
.5873 .06876 L
.58929 .06876 L
.59127 .06876 L
.59325 .06876 L
.59524 .06876 L
.59722 .06876 L
.59921 .06876 L
.60119 .06876 L
.60317 .06876 L
.60516 .06876 L
.60714 .06876 L
.60913 .06876 L
.61111 .06876 L
.6131 .06876 L
.61508 .06876 L
.61706 .06876 L
Mistroke
.61905 .06876 L
.62103 .15278 L
.62302 .41431 L
.625 .60332 L
.62698 .44167 L
.62897 .17705 L
.63095 .08507 L
.63294 .08426 L
.63492 .08361 L
.6369 .08271 L
.63889 .08138 L
.64087 .0796 L
.64286 .07737 L
.64484 .07475 L
.64683 .07179 L
.64881 .06853 L
.65079 .06504 L
.65278 .06135 L
.65476 .05753 L
.65675 .05361 L
.65873 .04966 L
.66071 .04571 L
.6627 .04184 L
.66468 .03807 L
.66667 .03446 L
.66865 .03106 L
.67063 .0279 L
.67262 .02502 L
.6746 .02246 L
.67659 .02024 L
.67857 .01838 L
.68056 .01689 L
.68254 .01578 L
.68452 .01506 L
.68651 .01472 L
.68849 .01474 L
.69048 .01512 L
.69246 .01583 L
.69444 .01686 L
.69643 .01816 L
.69841 .01972 L
.7004 .02151 L
.70238 .02348 L
.70437 .0256 L
.70635 .02784 L
.70833 .03018 L
.71032 .03257 L
.7123 .03498 L
.71429 .0374 L
.71627 .03978 L
Mistroke
.71825 .04212 L
.72024 .04439 L
.72222 .04658 L
.72421 .04866 L
.72619 .05064 L
.72817 .0525 L
.73016 .05424 L
.73214 .05585 L
.73413 .05734 L
.73611 .0587 L
.7381 .05994 L
.74008 .06106 L
.74206 .06207 L
.74405 .06297 L
.74603 .06377 L
.74802 .06448 L
.75 .0651 L
.75198 .06565 L
.75397 .06613 L
.75595 .06654 L
.75794 .06689 L
.75992 .0672 L
.7619 .06746 L
.76389 .06768 L
.76587 .06787 L
.76786 .06802 L
.76984 .06816 L
.77183 .06827 L
.77381 .06836 L
.77579 .06843 L
.77778 .0685 L
.77976 .06855 L
.78175 .06859 L
.78373 .06863 L
.78571 .06865 L
.7877 .06868 L
.78968 .06869 L
.79167 .06871 L
.79365 .06872 L
.79563 .06873 L
.79762 .06874 L
.7996 .06874 L
.80159 .06875 L
.80357 .06875 L
.80556 .06875 L
.80754 .06876 L
.80952 .06876 L
.81151 .06876 L
.81349 .06876 L
.81548 .06876 L
Mistroke
.81746 .06876 L
.81944 .06876 L
.82143 .06876 L
.82341 .06876 L
.8254 .06876 L
.82738 .06876 L
.82937 .06876 L
.83135 .06876 L
.83333 .06876 L
.83532 .06876 L
.8373 .06876 L
.83929 .06876 L
.84127 .06876 L
.84325 .06876 L
.84524 .06876 L
.84722 .06876 L
.84921 .06876 L
.85119 .06876 L
.85317 .06876 L
.85516 .06876 L
.85714 .06876 L
.85913 .06876 L
.86111 .06876 L
.8631 .06876 L
.86508 .06876 L
.86706 .06876 L
.86905 .06876 L
.87103 .06876 L
.87302 .06876 L
.875 .06876 L
.87698 .06876 L
.87897 .06876 L
.88095 .06876 L
.88294 .06876 L
.88492 .06876 L
.8869 .06876 L
.88889 .06876 L
.89087 .06876 L
.89286 .06876 L
.89484 .06876 L
.89683 .06876 L
.89881 .06876 L
.90079 .06876 L
.90278 .06876 L
.90476 .06876 L
.90675 .06876 L
.90873 .06876 L
.91071 .06876 L
.9127 .06876 L
.91468 .06876 L
Mistroke
.91667 .06876 L
.91865 .06876 L
.92063 .06876 L
.92262 .06876 L
.9246 .06876 L
.92659 .06876 L
.92857 .06876 L
.93056 .06876 L
.93254 .06876 L
.93452 .06876 L
.93651 .06876 L
.93849 .06876 L
.94048 .06876 L
.94246 .06876 L
.94444 .06876 L
.94643 .06876 L
.94841 .06876 L
.9504 .06876 L
.95238 .06876 L
.95437 .06876 L
.95635 .06876 L
.95833 .06876 L
.96032 .06876 L
.9623 .06876 L
.96429 .06876 L
.96627 .06876 L
.96825 .06876 L
.97024 .06876 L
.97222 .06876 L
.97421 .06876 L
.97619 .06876 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{196.312, 121.125},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003e0000Ua000`40O003h00OXGoo00<007ooOol0
DGoo002QOol00`00Oomoo`1AOol00:5oo`03001oogoo055oo`00XGoo00<007ooOol0DGoo002QOol0
0`00Oomoo`02Ool5001:Ool000Yoo`<002=oo`<0029oo`D002=oo`D001moo`04001oogooOol20003
Ool2000JOol5000POol50005Ool000]oo`03001oogoo025oo`05001oogooOol0000ROol01000Oomo
o`009Goo00<007ooOol07goo00<007ooOol00P001Goo0P006goo00<007ooOol08Goo00@007ooOol0
00Eoo`001P000Woo1@006Woo1P001Woo00<007ooOol05goo1P0017oo00<007ooOol06Woo1P001Goo
00<007ooOol07goo00@007ooOol000Moo`8001Yoo`03001oogoo029oo`03001oogoo00Eoo`0027oo
00@007ooOol002Moo`03001oogoo029oo`03001oogoo02Aoo`03001oogoo01moo`<000Uoo`8001Uo
o`03001oogoo02=oo`03001oogoo00Aoo`002Goo00<007oo00009Goo0P009Woo00<007ooOol08goo
00<007ooOol07goo00<007oo00002Woo0P0067oo00<007ooOol097oo00<007ooOol00goo0009Ool0
0`00Ool0000WOol00`00Oomoo`0SOol00`00Oomoo`0SOol00`00Oomoo`0OOol3000;Ool00`00Oomo
o`0FOol00`00Oomoo`0TOol00`00Oomoo`03Ool000Yoo`8002=oo`05001oogooOol0000QOol01@00
Oomoogoo00009Goo00<007ooOol07goo0P003Goo00<007ooOol05Goo00<007ooOol087oo00D007oo
Oomoo`0000Eoo`002goo00<007ooOol08Woo0`008goo0`0097oo0`008Goo00<007ooOol037oo0P00
57oo0`008goo0`001Woo002QOol00`00Oomoo`0=Ool20012Ool00:1oo`80015oo`03001oogoo03mo
o`00X7oo0P004Woo0P00?goo002OOol00`00Ool0000DOol4000kOol00003Ool000000?8000001goo
00<007ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo
00<007ooOol017oo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol017oo00<007ooOol01Goo
00<007ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo
00<007ooOol017oo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol00Woo00<007oo000017oo
00@007ooOol000Moo`03001oogoo00Aoo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Eo
o`03001oogoo00Aoo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Aoo`03001oogoo00Eo
o`03001oogoo00Aoo`001goo00<007ooOol097oo00<007ooOol08goo00<007ooOol097oo00<007oo
Ool06Goo00<007ooOol00goo00D007ooOomoo`0002Ioo`03001oogoo02=oo`03001oogoo00Aoo`00
Ugoo00<007ooOol00Woo0P000goo00<007ooOol0DGoo002GOol00`00Oomoo`020005Ool00`00Oomo
o`1AOol009Moo`04001oogoo0006Ool00`00Oomoo`1AOol009Moo`04001oogoo0006Ool00`00Oomo
o`1AOol009Moo`04001oogoo0006Ool00`00Oomoo`1AOol009Moo`04001oogoo0006Ool00`00Oomo
o`1AOol009Moo`04001oogoo0006Ool2001BOol009Moo`04001oogoo0006Ool00`00Oomoo`1AOol0
09Qoo`03001oo`0000Ioo`03001oogoo055oo`00V7oo00<007oo00001Woo00<007ooOol0DGoo002H
Ool00`00Ool00006Ool00`00Oomoo`1AOol009Qoo`03001oo`0000Ioo`03001oogoo055oo`00V7oo
00<007oo00001Woo00<007ooOol0DGoo002HOol00`00Ool00006Ool00`00Oomoo`1AOol009Qoo`03
001oo`0000Ioo`03001oogoo055oo`00V7oo00<007oo00001Woo00<007ooOol0DGoo002HOol00`00
Ool00006Ool2001BOol009Qoo`03001oo`0000Ioo`03001oogoo055oo`00V7oo00<007oo00001Woo
00<007ooOol0DGoo002HOol00`00Ool00006Ool00`00Oomoo`1AOol009Qoo`03001oo`0000Ioo`03
001oogoo055oo`00V7oo00<007oo00001Woo00<007ooOol0DGoo002HOol00`00Ool00006Ool00`00
Oomoo`1AOol009Qoo`03001oo`0000Ioo`03001oogoo055oo`00V7oo00<007oo00001Woo00<007oo
Ool0DGoo002HOol00`00Ool00006Ool2001BOol009Qoo`03001oo`0000Ioo`03001oogoo055oo`00
V7oo00<007oo00001Woo00<007ooOol0DGoo002HOol00`00Ool00006Ool00`00Oomoo`1AOol009Qo
o`03001oo`0000Ioo`03001oogoo055oo`00V7oo00<007oo00001Woo00<007ooOol0DGoo002HOol0
0`00Ool00006Ool00`00Oomoo`1AOol008moo`D000Aoo`<000Ioo`03001oogoo055oo`00T7oo00@0
07ooOol000=oo`800003Ool0000000Eoo`03001oogoo055oo`00TGoo00<007ooOol00goo0P0000=o
o`0000001Goo0`00DGoo002BOol00`00Oomoo`02Ool200000goo00000005Ool00`00Oomoo`1AOol0
09=oo`04001oogooOol200000goo00000005Ool00`00Oomoo`1AOol009=oo`04001oogooOol20000
0goo00000005Ool00`00Oomoo`1AOol008moo`05001oogooOol00003Ool200000goo00000005Ool0
0`00Oomoo`1AOol0091oo`<000Eoo`<000Ioo`03001oogoo055oo`00V7oo00<007oo00001Woo00<0
07ooOol0DGoo002HOol00`00Ool00006Ool00`00Oomoo`1AOol009Qoo`03001oo`0000Ioo`03001o
ogoo055oo`00V7oo00<007oo00001Woo00<007ooOol0DGoo002HOol00`00Ool00006Ool2001BOol0
09Qoo`03001oo`0000Ioo`03001oogoo055oo`00V7oo00<007oo00001Woo00<007ooOol0DGoo002H
Ool00`00Ool00006Ool00`00Oomoo`1AOol009Qoo`03001oo`0000Ioo`03001oogoo055oo`00V7oo
00<007oo00001Woo00<007ooOol0DGoo002HOol00`00Ool00006Ool00`00Oomoo`1AOol009Qoo`03
001oo`0000Ioo`03001oogoo055oo`00V7oo00<007oo00001Woo00<007ooOol0DGoo002HOol00`00
Ool00006Ool2001BOol009Qoo`03001oo`0000Ioo`03001oogoo055oo`00V7oo00<007oo00001Woo
00<007ooOol0DGoo002HOol20007Ool00`00Oomoo`1AOol009Qoo`8000Moo`03001oogoo055oo`00
V7oo0P001goo00<007ooOol0DGoo002HOol20007Ool00`00Oomoo`1AOol009Qoo`8000Moo`03001o
ogoo055oo`00V7oo0P001goo00<007ooOol0DGoo002HOol20007Ool2001BOol009Qoo`8000Moo`03
001oogoo055oo`00V7oo0P001goo00<007ooOol0DGoo002HOol20007Ool00`00Oomoo`1AOol009Qo
o`8000Moo`03001oogoo055oo`00V7oo0P001goo00<007ooOol0DGoo002HOol20007Ool00`00Oomo
o`1AOol0095oo`<000Aoo`<000Ioo`03001oogoo055oo`00TWoo00<007ooOol00Woo0`0000=oo`00
Ool017oo00<007ooOol0DGoo002?Ool50003Ool300000goo001oo`04Ool00`00Oomoo`1AOol008mo
o`04001oogoo0004Ool300000goo001oo`04Ool3001AOol0091oo`03001oo`0000Aoo`<00003Ool0
07oo00Aoo`03001oogoo055oo`00T7oo00<007oo000017oo0`0000=oo`00Ool017oo00<007ooOol0
DGoo002AOol20004Ool300000goo001oo`04Ool00`00Oomoo`1AOol0099oo`03001oogoo00=oo`<0
00Ioo`03001oogoo055oo`00V7oo0P001goo00<007ooOol0DGoo002HOol20007Ool00`00Oomoo`1A
Ool009Qoo`8000Moo`03001oogoo055oo`00V7oo0P001goo00<007ooOol0DGoo002HOol20007Ool2
001BOol009Qoo`8000Moo`03001oogoo055oo`00V7oo0P001goo00<007ooOol0DGoo002HOol20007
Ool00`00Oomoo`1AOol009Qoo`8000Moo`03001oogoo055oo`00V7oo0P001goo00<007ooOol0DGoo
002HOol20007Ool00`00Oomoo`1AOol009Qoo`8000Moo`03001oogoo055oo`00V7oo0P001goo00<0
07ooOol0DGoo002HOol20007Ool2001BOol009Qoo`8000Moo`03001oogoo055oo`00V7oo0P001goo
00<007ooOol0DGoo002HOol20007Ool00`00Oomoo`1AOol009Qoo`8000Moo`03001oogoo055oo`00
V7oo0P001goo00<007ooOol0DGoo002HOol20007Ool00`00Oomoo`1AOol009Qoo`8000Moo`03001o
ogoo055oo`00V7oo0P001goo00<007ooOol0DGoo002HOol20007Ool00`00Oomoo`1AOol009Qoo`80
00Moo`80059oo`00V7oo0P001goo00<007ooOol0DGoo002HOol20007Ool00`00Oomoo`1AOol009Qo
o`8000Moo`03001oogoo055oo`00V7oo0P001goo00<007ooOol0DGoo002IOol00`00Oomoo`05Ool0
0`00Oomoo`1AOol009Uoo`03001oogoo00Eoo`03001oogoo055oo`00T7oo0`001Goo0`001Woo00<0
07ooOol0DGoo002?Ool01@00Oomoogoo00000goo00D007oo001oo`0000Eoo`03001oogoo055oo`00
Sgoo00D007ooOomoo`0000=oo`05001oo`00Ool00005Ool3001AOol008moo`05001oogooOol00003
Ool01@00Ool007oo00001Goo00<007ooOol0DGoo002?Ool40004Ool01@00Ool007oo00001Goo00<0
07ooOol0DGoo002?Ool00`00Oomoo`05Ool01@00Ool007oo00001Goo00<007ooOol0DGoo002@Ool0
0`00Oomoo`04Ool01@00Ool007oo00001Goo00<007ooOol0DGoo002AOol30004Ool30006Ool00`00
Oomoo`1AOol009Uoo`03001oogoo00Eoo`03001oogoo055oo`00VGoo00<007ooOol01Goo00<007oo
Ool0DGoo002IOol00`00Oomoo`05Ool00`00Oomoo`1AOol009Uoo`03001oogoo00Eoo`80059oo`00
VGoo00<007ooOol01Goo00<007ooOol0DGoo002IOol00`00Oomoo`05Ool00`00Oomoo`1AOol009Uo
o`03001oogoo00Eoo`03001oogoo055oo`00VGoo00<007ooOol01Goo00<007ooOol0DGoo002IOol0
0`00Oomoo`05Ool00`00Oomoo`1AOol009Uoo`03001oogoo00Eoo`03001oogoo055oo`00VGoo00<0
07ooOol01Goo00<007ooOol0DGoo002IOol00`00Oomoo`05Ool00`00Oomoo`1AOol009Uoo`03001o
ogoo00Eoo`03001oogoo055oo`00XGoo0P00DWoo002QOol00`00Oomoo`1AOol00:5oo`03001oogoo
055oo`00\
\>"],
  ImageRangeCache->{{{0, 244.375}, {150.375, 0}} -> {-4.19476, -9.04675, 
  0.0324542, 0.669505}}]
}, Open  ]],

Cell["We can locate the maximum easily:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(sortthird = Transpose[{Abs[thirddata], \ points}]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Last[Sort[sortthird]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({69.4731020898122153`, \(-0.2125`\)}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(criticalx\  = \ \(-0.2125\); \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "So the maximum is attained at the value of ",
  Cell[BoxData[
      \(TraditionalForm\`x\  = \ \(-0.2125\)\)]],
  ". This is a simple estimate of the location of the critical price. We now \
split the data into two blocks, each side of this region. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(rawdata\  = \ Transpose[{points, \ finalstep}]; 
    finala\  = \ Select[rawdata, \ \((#[\([1]\)] >= criticalx &)\)]; \n
    finalb\  = \ Select[rawdata, \ \((#[\([1]\)] <= criticalx &)\)]; \)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(xa\  = \ \(Transpose[finala]\)[\([1]\)]; \ 
    xb\  = \ \(Transpose[finalb]\)[\([1]\)]; \n
    dataa\  = \ \(Transpose[finala]\)[\([2]\)]; \ 
    datab\  = \ \(Transpose[finalb]\)[\([2]\)]; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(deltadataa = listd[dataa, dx] - 1\/2\ \((kt - 1)\)\ dataa; \n
    deltadatab = listd[datab, dx] - 1\/2\ \((kt - 1)\)\ datab; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(gammadataa = listd[deltadataa, dx] - 1\/2\ \((kt + 1)\) deltadataa; \n
    gammadatab = listd[deltadatab, dx] - 1\/2\ \((kt + 1)\) deltadatab; \)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(deltainterpoldataa = Transpose[{xa, deltadataa}]; \n
    deltainterpoldatab = Transpose[{xb, deltadatab}]; \n
    gammainterpoldataa = Transpose[{xa, gammadataa}]; \n
    gammainterpoldatab = Transpose[{xb, gammadatab}]; \n\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(dfunca = 
      Interpolation[deltainterpoldataa, InterpolationOrder \[Rule] 3]; \n
    dfuncb = Interpolation[deltainterpoldatab, 
        InterpolationOrder \[Rule] 3]; \n
    gfunca = Interpolation[gammainterpoldataa, 
        InterpolationOrder \[Rule] 3]; \n
    gfuncb = Interpolation[gammainterpoldatab, 
        InterpolationOrder \[Rule] 3]; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
We now give piecewise definitions of \[CapitalDelta] and \
\[CapitalGamma]:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
SORDelta[strike_, r_, q_, S_, T_, sd_] := 
  If[Log[S/strike] >= criticalx, 1/S*ValuationMultiplier[strike, r, q, \
Log[S/strike], (sd^2*T)/2, sd]*dfunca[Log[S/strike]],-1]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
SORGamma[strike_, r_, q_, S_, T_, sd_] := 
  1/S^2*ValuationMultiplier[strike, r, q, Log[S/strike], (sd^2*T)/2, \
sd]*If[Log[S/strike] >= criticalx, gfunca[Log[S/strike]], 0]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["The value at 8 is now precisely zero:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(SORGamma[10, \ 0.05, \ 0, \ 8, \ 1, \ 0.2]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell["\<\
The plot now has a sharp discontinuity and no negative \
regions:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      SORGamma[10, \ 0.05, \ 0, \ S, \ 1, \ 0.2], \ {S, 5, \ 15}]; \)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.452381 0.0952381 0.0147151 1.50661 [
[.30952 .00222 -3 -9 ]
[.30952 .00222 3 0 ]
[.5 .00222 -6 -9 ]
[.5 .00222 6 0 ]
[.69048 .00222 -6 -9 ]
[.69048 .00222 6 0 ]
[.88095 .00222 -6 -9 ]
[.88095 .00222 6 0 ]
[.10655 .16538 -18 -4.5 ]
[.10655 .16538 0 4.5 ]
[.10655 .31604 -18 -4.5 ]
[.10655 .31604 0 4.5 ]
[.10655 .4667 -18 -4.5 ]
[.10655 .4667 0 4.5 ]
[.10655 .61736 -18 -4.5 ]
[.10655 .61736 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.30952 .01472 m
.30952 .02097 L
s
[(8)] .30952 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(10)] .5 .00222 0 1 Mshowa
.69048 .01472 m
.69048 .02097 L
s
[(12)] .69048 .00222 0 1 Mshowa
.88095 .01472 m
.88095 .02097 L
s
[(14)] .88095 .00222 0 1 Mshowa
.125 Mabswid
.16667 .01472 m
.16667 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.2619 .01472 m
.2619 .01847 L
s
.35714 .01472 m
.35714 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.45238 .01472 m
.45238 .01847 L
s
.54762 .01472 m
.54762 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.64286 .01472 m
.64286 .01847 L
s
.7381 .01472 m
.7381 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.83333 .01472 m
.83333 .01847 L
s
.07143 .01472 m
.07143 .01847 L
s
.02381 .01472 m
.02381 .01847 L
s
.92857 .01472 m
.92857 .01847 L
s
.97619 .01472 m
.97619 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.11905 .16538 m
.1253 .16538 L
s
[(0.1)] .10655 .16538 1 0 Mshowa
.11905 .31604 m
.1253 .31604 L
s
[(0.2)] .10655 .31604 1 0 Mshowa
.11905 .4667 m
.1253 .4667 L
s
[(0.3)] .10655 .4667 1 0 Mshowa
.11905 .61736 m
.1253 .61736 L
s
[(0.4)] .10655 .61736 1 0 Mshowa
.125 Mabswid
.11905 .04485 m
.1228 .04485 L
s
.11905 .07498 m
.1228 .07498 L
s
.11905 .10511 m
.1228 .10511 L
s
.11905 .13524 m
.1228 .13524 L
s
.11905 .19551 m
.1228 .19551 L
s
.11905 .22564 m
.1228 .22564 L
s
.11905 .25577 m
.1228 .25577 L
s
.11905 .28591 m
.1228 .28591 L
s
.11905 .34617 m
.1228 .34617 L
s
.11905 .3763 m
.1228 .3763 L
s
.11905 .40643 m
.1228 .40643 L
s
.11905 .43657 m
.1228 .43657 L
s
.11905 .49683 m
.1228 .49683 L
s
.11905 .52696 m
.1228 .52696 L
s
.11905 .5571 m
.1228 .5571 L
s
.11905 .58723 m
.1228 .58723 L
s
.25 Mabswid
.11905 0 m
.11905 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .01472 L
.10458 .01472 L
.14415 .01472 L
.18221 .01472 L
.22272 .01472 L
.26171 .01472 L
.28158 .01472 L
.29205 .01472 L
.30316 .01472 L
.30791 .01472 L
.31304 .01472 L
.31454 .01472 L
.31597 .01472 L
.31725 .01472 L
.31798 .60332 L
.31865 .60182 L
.32392 .59086 L
.34309 .56107 L
.38221 .5129 L
.4238 .46162 L
.46386 .40926 L
.50241 .35687 L
.54342 .30139 L
.5829 .25065 L
.62088 .20618 L
.6613 .16471 L
.70021 .13099 L
.74157 .10171 L
.78142 .07938 L
.81975 .06262 L
.86053 .04904 L
.89979 .0393 L
.93754 .03237 L
.97619 .02718 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{196.312, 121.125},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003e0000Ua000`40O003h00OmGoo003eOol004io
o`<002=oo`D000=oo`<001ioo`D0009oo`D001ioo`D000Aoo`<0029oo`00CGoo00D007ooOomoo`00
02Aoo`03001oogoo009oo`05001oogooOol0000OOol00`00Oomoo`03Ool01000Oomoo`0087oo00<0
07ooOol01Goo00<007ooOol08Goo001=Ool01@00Oomoogoo000097oo00<007ooOol00Woo00D007oo
Oomoo`0001moo`03001oogoo00Aoo`03001oogoo021oo`03001oogoo009oo`D0029oo`00CGoo00D0
07ooOomoo`0002Aoo`03001oogoo009oo`05001oogooOol0000OOol00`00Oomoo`05Ool00`00Oomo
o`0OOol00`00Oomoo`02Ool01000Oomoo`008goo001>Ool3000UOol00`00Oomoo`02Ool01@00Oomo
ogoo00007goo00<007ooOol01Woo00<007ooOol07Woo00<007ooOol00goo00<007oo00008goo001=
Ool01@00Oomoogoo000097oo00<007ooOol00Woo00D007ooOomoo`0001moo`03001oogoo00Ioo`03
001oogoo01ioo`03001oogoo00=oo`03001oo`0002=oo`00CGoo00D007ooOomoo`0002Aoo`03001o
ogoo009oo`05001oogooOol0000OOol00`00Oomoo`02Ool01@00Oomoogoo000087oo00<007ooOol0
17oo0P008goo001>Ool3000SOol30005Ool3000NOol30005Ool3000OOol30007Ool00`00Oomoo`0Q
Ool00?Eoo`009goo00<007ooOol0bgoo000WOol00`00Oomoo`3;Ool002Moo`03001oogoo0<]oo`00
3GoofP003Woo000BOol00`00Oomoo`08Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`08
Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Ool00008Ool00`00Oomoo`07Ool00`00Oomoo`07
Ool00`00Oomoo`08Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`08Ool00`00Oomoo`07
Ool00`00Oomoo`07Ool00`00Oomoo`08Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`08
Ool00`00Oomoo`07Ool00`00Oomoo`0AOol002Moo`03001oogoo02Qoo`03001oogoo0:1oo`009goo
00<007ooOol0:7oo00<007ooOol0R7oo1@004goo000WOol00`00Oomoo`0XOol00`00Oomoo`20Ool8
000HOol002Moo`03001oogoo02Qoo`03001oogoo07Yoo`H0021oo`009goo00<007ooOol0:7oo00<0
07ooOol0Mgoo0`009Woo000WOol2000YOol00`00Oomoo`1dOol3000YOol002Moo`03001oogoo02Qo
o`03001oogoo075oo`<002aoo`009goo00<007ooOol0:7oo00<007ooOol0K7oo1@00;goo000WOol0
0`00Oomoo`0XOol00`00Oomoo`1YOol3000dOol002Moo`03001oogoo02Qoo`03001oogoo06Moo`80
03Moo`009goo00<007ooOol0:7oo00<007ooOol0IGoo0P00>Goo000WOol2000YOol00`00Oomoo`1S
Ool2000kOol002Moo`03001oogoo02Qoo`03001oogoo065oo`8003eoo`009goo00<007ooOol0:7oo
00<007ooOol0Ggoo0P00?goo000WOol00`00Oomoo`0XOol00`00Oomoo`1MOol20011Ool002Moo`03
001oogoo02Qoo`03001oogoo05aoo`03001oogoo045oo`009goo00<007ooOol0:7oo00<007ooOol0
FWoo0P00A7oo000WOol00`00Oomoo`0XOol00`00Oomoo`1HOol20016Ool002Moo`8002Uoo`03001o
ogoo05Moo`03001oogoo04Ioo`009goo00<007ooOol0:7oo00<007ooOol0EGoo0P00BGoo000WOol0
0`00Oomoo`0XOol00`00Oomoo`1DOol00`00Oomoo`19Ool002Moo`03001oogoo02Qoo`03001oogoo
059oo`8004aoo`009goo00<007ooOol0:7oo00<007ooOol0DGoo00<007ooOol0C7oo000WOol00`00
Oomoo`0XOol00`00Oomoo`1@Ool00`00Oomoo`1=Ool002Moo`8002Uoo`03001oogoo04moo`03001o
ogoo04ioo`009goo00<007ooOol0:7oo00<007ooOol0CWoo00<007ooOol0Cgoo000WOol00`00Oomo
o`0XOol00`00Oomoo`1=Ool00`00Oomoo`1@Ool002Moo`03001oogoo02Qoo`03001oogoo04aoo`03
001oogoo055oo`003goo0`001Woo0P001Goo1@000goo00<007ooOol0:7oo00<007ooOol0Bgoo00<0
07ooOol0DWoo000>Ool01@00Oomoogoo00001Goo0P001goo00<007ooOol00goo00<007ooOol0:7oo
00<007ooOol0BWoo00<007ooOol0Dgoo000>Ool01@00Oomoogoo00003Woo00<007ooOol00goo00<0
07ooOol0:7oo00<007ooOol0BGoo00<007ooOol0E7oo000>Ool01@00Oomoogoo00003Woo00<007oo
Ool00goo0P00:Goo00<007ooOol0B7oo00<007ooOol0EGoo000>Ool01@00Oomoogoo00003Woo00<0
07ooOol00goo00<007ooOol0:7oo00<007ooOol0Agoo00<007ooOol0EWoo000>Ool01@00Oomoogoo
00003Woo00<007ooOol00goo00<007ooOol0:7oo00<007ooOol0AWoo00<007ooOol0Egoo000>Ool0
1@00Oomoogoo00003Woo00<007ooOol00goo00<007ooOol0:7oo00<007ooOol0AGoo00<007ooOol0
F7oo000?Ool3000=Ool30005Ool00`00Oomoo`0XOol00`00Oomoo`14Ool00`00Oomoo`1IOol002Mo
o`03001oogoo02Qoo`03001oogoo04=oo`03001oogoo05Yoo`009goo0P00:Goo00<007ooOol0@Woo
00<007ooOol0Fgoo000WOol00`00Oomoo`0XOol00`00Oomoo`11Ool00`00Oomoo`1LOol002Moo`03
001oogoo02Qoo`03001oogoo041oo`03001oogoo05eoo`009goo00<007ooOol0:7oo00<007ooOol0
?goo00<007ooOol0GWoo000WOol00`00Oomoo`0XOol00`00Oomoo`0oOol00`00Oomoo`1NOol002Mo
o`03001oogoo02Qoo`03001oogoo03ioo`03001oogoo05moo`009goo00<007ooOol0:7oo00<007oo
Ool0?Goo00<007ooOol0H7oo000WOol2000YOol00`00Oomoo`0lOol00`00Oomoo`1QOol002Moo`03
001oogoo02Qoo`03001oogoo03]oo`03001oogoo069oo`009goo00<007ooOol0:7oo00<007ooOol0
>Woo00<007ooOol0Hgoo000WOol00`00Oomoo`0XOol00`00Oomoo`0iOol00`00Oomoo`1TOol002Mo
o`03001oogoo02Qoo`03001oogoo03Qoo`03001oogoo06Eoo`009goo00<007ooOol0:7oo00<007oo
Ool0=goo00<007ooOol0IWoo000WOol2000YOol00`00Oomoo`0gOol00`00Oomoo`1VOol002Moo`03
001oogoo02Qoo`03001oogoo03Ioo`03001oogoo06Moo`009goo00<007ooOol0:7oo00<007ooOol0
=Goo00<007ooOol0J7oo000WOol00`00Oomoo`0XOol00`00Oomoo`0dOol00`00Oomoo`1YOol002Mo
o`03001oogoo02Qoo`03001oogoo03=oo`03001oogoo06Yoo`009goo00<007ooOol0:7oo00<007oo
Ool0<Woo00<007ooOol0Jgoo000WOol00`00Oomoo`0XOol00`00Oomoo`0bOol00`00Oomoo`1[Ool0
02Moo`8002Uoo`03001oogoo035oo`03001oogoo06aoo`009goo00<007ooOol0:7oo00<007ooOol0
<7oo00<007ooOol0KGoo000WOol00`00Oomoo`0XOol00`00Oomoo`0_Ool00`00Oomoo`1^Ool002Mo
o`03001oogoo02Qoo`03001oogoo02ioo`03001oogoo06moo`003goo0`001Woo0P0017oo1@0017oo
00<007ooOol0:7oo00<007ooOol0;Woo00<007ooOol0Kgoo000>Ool01@00Oomoogoo00001Goo0P00
1Goo00@007ooOol000Aoo`03001oogoo02Qoo`03001oogoo02eoo`03001oogoo071oo`003Woo00D0
07ooOomoo`0000eoo`03001oogoo00Aoo`8002Uoo`03001oogoo02aoo`03001oogoo075oo`003Woo
00D007ooOomoo`0000ioo`03001oogoo00=oo`03001oogoo02Qoo`03001oogoo02]oo`03001oogoo
079oo`003Woo00D007ooOomoo`0000moo`03001oogoo009oo`03001oogoo02Qoo`03001oogoo02]o
o`03001oogoo079oo`003Woo00D007ooOomoo`0000moo`03001oogoo009oo`03001oogoo02Qoo`03
001oogoo02Yoo`03001oogoo07=oo`003Woo00D007ooOomoo`0000]oo`05001oogooOol00004Ool0
0`00Oomoo`0XOol00`00Oomoo`0YOol00`00Oomoo`1dOol000moo`<000eoo`<000Eoo`03001oogoo
02Qoo`03001oogoo02Qoo`03001oogoo07Eoo`009goo00<007ooOol0:7oo00<007ooOol0:7oo00<0
07ooOol0MGoo000WOol2000YOol00`00Oomoo`0WOol00`00Oomoo`1fOol002Moo`03001oogoo02Qo
o`03001oogoo02Ioo`03001oogoo07Moo`009goo00<007ooOol0:7oo00<007ooOol09Goo00<007oo
Ool0N7oo000WOol00`00Oomoo`0XOol00`00Oomoo`0UOol00`00Oomoo`1hOol002Moo`03001oogoo
02Qoo`03001oogoo02Aoo`03001oogoo07Uoo`009goo00<007ooOol0:7oo00<007ooOol08goo00<0
07ooOol0NWoo000WOol00`00Oomoo`0XOol00`00Oomoo`0SOol00`00Oomoo`1jOol002Moo`8002Uo
o`03001oogoo029oo`03001oogoo07]oo`009goo00<007ooOol0:7oo00<007ooOol08Goo00<007oo
Ool0O7oo000WOol00`00Oomoo`0XOol00`00Oomoo`0QOol00`00Oomoo`1lOol002Moo`03001oogoo
02Qoo`03001oogoo021oo`03001oogoo07eoo`009goo00<007ooOol0:7oo00<007ooOol07goo00<0
07ooOol0OWoo000WOol00`00Oomoo`0XOol00`00Oomoo`0OOol00`00Oomoo`1nOol002Moo`8002Uo
o`03001oogoo01ioo`03001oogoo07moo`009goo00<007ooOol0:7oo00<007ooOol07Goo00<007oo
Ool0P7oo000WOol00`00Oomoo`0XOol00`00Oomoo`0MOol00`00Oomoo`20Ool002Moo`03001oogoo
02Qoo`03001oogoo01aoo`03001oogoo085oo`009goo00<007ooOol0:7oo00<007ooOol06goo00<0
07ooOol0PWoo000WOol00`00Oomoo`0XOol00`00Oomoo`0JOol00`00Oomoo`23Ool002Moo`03001o
ogoo02Qoo`03001oogoo01Uoo`03001oogoo08Aoo`009goo0P00:Goo00<007ooOol067oo00<007oo
Ool0QGoo000WOol00`00Oomoo`0XOol00`00Oomoo`0HOol00`00Oomoo`25Ool002Moo`03001oogoo
02Qoo`03001oogoo01Moo`03001oogoo08Ioo`009goo00<007ooOol0:7oo00<007ooOol05Woo00<0
07ooOol0Qgoo000?Ool30006Ool20005Ool30005Ool00`00Oomoo`0XOol00`00Oomoo`0EOol00`00
Oomoo`28Ool000ioo`05001oogooOol00005Ool20004Ool01@00Oomoogoo000017oo00<007ooOol0
:7oo00<007ooOol057oo00<007ooOol0RGoo000>Ool01@00Oomoogoo00003goo00<007ooOol00Woo
0P00:Goo00<007ooOol057oo00<007ooOol0RGoo000>Ool01@00Oomoogoo00003goo00<007ooOol0
0Woo00<007ooOol0:7oo00<007ooOol04goo00<007ooOol0RWoo000>Ool01@00Oomoogoo00003Goo
0P001Goo00<007ooOol0:7oo00<007ooOol04Woo00<007ooOol0Rgoo000>Ool01@00Oomoogoo0000
3goo00<007ooOol00Woo00<007ooOol0:7oo00<007ooOol04Goo00<007ooOol0S7oo000>Ool01@00
Oomoogoo00002goo00D007ooOomoo`0000Aoo`03001oogoo02Qoo`03001oogoo011oo`03001oogoo
08eoo`003goo0`003Goo0`001Goo00<007ooOol0:7oo00<007ooOol03goo00<007ooOol0SWoo000W
Ool00`00Oomoo`0XOol00`00Oomoo`0?Ool00`00Oomoo`2>Ool002Moo`8002Uoo`03001oogoo00io
o`03001oogoo08moo`009goo00<007ooOol0:7oo00<007ooOol03Goo00<007ooOol0T7oo000WOol0
0`00Oomoo`0XOol00`00Oomoo`0<Ool00`00Oomoo`2AOol002Moo`03001oogoo02Qoo`03001oogoo
00]oo`03001oogoo099oo`009goo00<007ooOol0:7oo00<007ooOol02goo00<007ooOol0TWoo000W
Ool00`00Oomoo`0XOol00`00Oomoo`0:Ool00`00Oomoo`2COol002Moo`8002Uoo`03001oogoo00Uo
o`03001oogoo09Aoo`009goo00<007ooOol0:7oo00<007ooOol027oo00<007ooOol0UGoo000WOol0
0`00Oomoo`0XOol00`00Oomoo`08Ool00`00Oomoo`2EOol002Moo`03001oogoo02Qoo`03001oogoo
00Moo`03001oogoo09Ioo`009goo00<007ooOol0:7oo00<007ooOol01Woo00<007ooOol0Ugoo000W
Ool00`00Oomoo`0XOol00`00Oomoo`06Ool00`00Oomoo`2GOol002Moo`03001oogoo02Qoo`03001o
ogoo00Eoo`03001oogoo09Qoo`009goo0P00:Goo00<007ooOol017oo00<007ooOol0VGoo000WOol0
0`00Oomoo`0XOol00`00Oomoo`03Ool00`00Oomoo`2JOol002Moo`03001oogoo02Qoo`03001oogoo
00=oo`03001oogoo09Yoo`009goo00<007ooOol0:7oo00<007ooOol00Woo00<007ooOol0Vgoo000W
Ool00`00Oomoo`0XOol01@00Oomoogoo0000WWoo000WOol00`00Oomoo`0XOol01000Oomoo`00Wgoo
000WOol2000YOol00`00Ool0002POol002Moo`03001oogoo02Qoo`800:5oo`009goo00<007ooOol0
:7oo0P00XGoo000WOol00`00Oomoo`0XOol00`00Oomoo`2POol000moo`<000Ioo`8000Ioo`<000Ao
o`03001oogoo02Qoo`03001oogoo0:1oo`003Woo00D007ooOomoo`0000Eoo`8000Moo`03001oogoo
00=oo`03001oogoo0<]oo`003Woo00D007ooOomoo`0000]oo`D000Aoo`03001oogoo0<]oo`003Woo
00D007ooOomoo`0000]oo`04001oogoo0005Ool2003<Ool000ioo`05001oogooOol0000<Ool00`00
Ool0003COol000ioo`05001oogooOol0000<Ool00`00Ool0003COol000ioo`05001oogooOol0000=
Ool2003COol000moo`<000moo`03001oogoo0=5oo`00mGoo0000\
\>"],
  ImageRangeCache->{{{0, 244.375}, {150.375, 0}} -> {4.08829, -0.0427027, 
  0.0604778, 0.00382302}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.9 American Options - Five-Year Expiry", "Section",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Now we proceed to a second test application of these methods, and \
look at the valuation of American Puts with a longer time to expiry, using a \
PSOR scheme based on 3TLD. We shall keep most parameters to be identical with \
the previous one-year case, but multiply the time-step by five. Since we are \
using implicit methods, we can do so without adjusting any other parameters. \
Previously \[Alpha] was 6.4 - it is now 32 (64 times the size in a binomial \
model with the same price-steps).\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Summary of Finite-Difference Results - Five-Year Expiry", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
In the following table we give a table of valuations, Deltas, \
Gammas and Thetas for a range of underlying prices with the strike at 10. (5 \
years to expiry, 20% volatility, 5% interest rate, 20 time-steps, 480 \
price-steps) Note that this has been re-computed without showing the details.\
\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
samples = TableForm[Table[Map[PaddedForm[N[#], {6, 4}]&, {S, \
Valuation[10, 0.05, 0, S, 5, 0.2],
SORDelta[10, 0.05, 0, S, 5, 0.2],SORGamma[10, 0.05, 0, S, 5, 0.2],
SORTheta[10, 0.05, 0, S, 5, 0.2]}],
{S, 2, 16, 1}]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {
            TagBox[
              InterpretationBox["\<\"  2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  8.0000\"\>",
                8.00000000644286,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.0000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                3.78605069060172*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  7.0000\"\>",
                7.00000001216859,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.0000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                3.31279435747232*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  6.0000\"\>",
                6.00000001626863,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.0000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                2.83953802387402*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  5.0000\"\>",
                5.00000002474366,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.0000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                2.36628169152611*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  4.0000\"\>",
                4.00000001335052,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.0000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                1.89302535004948*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  3.0000\"\>",
                3.00000003559577,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.0000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                1.41976902481957*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  2.0614\"\>",
                2.06142299724587,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.7856\"\>",
                -0.785566979583584,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.3367\"\>",
                0.336697391856977,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0136\"\>",
                -0.0135737691231335,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  1.4196\"\>",
                1.41959810788877,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.5194\"\>",
                -0.519354189985027,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.2083\"\>",
                0.208270871153861,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0326\"\>",
                -0.032586191945055,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.9904\"\>",
                0.990449830660745,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.3512\"\>",
                -0.351216665858549,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.1343\"\>",
                0.134329224918284,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0434\"\>",
                -0.0434127839467488,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.6980\"\>",
                0.697971292700068,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.2413\"\>",
                -0.241268296542135,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0891\"\>",
                0.0890701214384528,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0479\"\>",
                -0.0478583981075459,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.4959\"\>",
                0.495907459953808,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.1677\"\>",
                -0.167672690370586,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0602\"\>",
                0.0601946810245402,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0479\"\>",
                -0.0478866853204789,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.3549\"\>",
                0.354855486984872,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.1176\"\>",
                -0.117598599032565,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0412\"\>",
                0.0412420265001925,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0452\"\>",
                -0.0451579478184116,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.2556\"\>",
                0.255562185538988,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0831\"\>",
                -0.0831147425888535,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0286\"\>",
                0.0285542496897066,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0409\"\>",
                -0.040928757803315,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.1852\"\>",
                0.185158743525997,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0591\"\>",
                -0.0591413753700296,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0199\"\>",
                0.0199382964098404,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0361\"\>",
                -0.0360722437776292,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.1349\"\>",
                0.134915747284646,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0423\"\>",
                -0.0423434471423205,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0140\"\>",
                0.0140234356646754,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0312\"\>",
                -0.0311501145008623,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  CellMargins->{{42, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  GroupPageBreakWithin->True,
  FontSize->8]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
CJMsamples = TableForm[Join[{{\"S\", \"CJM Value\", \"CJM Delta\", \
\"CJM Gamma\"}},Table[Map[PaddedForm[N[#], {5, 3}]&, {S, AmericanPutCJM[S, \
10, 0.2, 0.05, 5],AmericanPutDeltaCJM[S, 10, 0.2, 0.05, \
5],AmericanPutGammaCJM[S, 10, 0.2, 0.05, 5]}], {S, 5, 15, 2}]]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"CJM Value\"\>", "\<\"CJM Delta\"\>", 
            "\<\"CJM Gamma\"\>"},
          {
            TagBox[
              InterpretationBox["\<\"  5.000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  5.000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  7.000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  3.000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  9.000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  1.419\"\>",
                1.4188683134554108,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.520\"\>",
                -0.52005180146324415,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.209\"\>",
                0.20915998934515861,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.697\"\>",
                0.6968836952802262,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.241\"\>",
                -0.24114397452821695,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.089\"\>",
                0.089216497687857943,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.354\"\>",
                0.35414722320731562,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.117\"\>",
                -0.11737621592024028,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.041\"\>",
                0.041231086358354256,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.185\"\>",
                0.18482418361109873,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.059\"\>",
                -0.058981850155616689,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.020\"\>",
                0.01989763059927107,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  CellMargins->{{42, Inherited}, {Inherited, Inherited}},
  FontSize->8]
}, Open  ]],

Cell["\<\
Again we can see good agreement between our analytical model and \
the finite-difference scheme.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["16.10 American Options - Ten-Year Expiry", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Summary of Finite-Difference Results", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In the following table we give a table of valuations, Deltas, Gammas and \
Thetas for a range of underlying prices with the strike at 10. (10 years to \
expiry, 20% volatility, 5% interest rate, 40 time-steps, 320 price-steps, ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 32\)]],
  ")"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
samples = TableForm[Table[Map[PaddedForm[N[#], {6, 4}]&, {S, \
Valuation[10, 0.05, 0, S, 10, 0.2],
SORDelta[10, 0.05, 0, S, 10, 0.2],SORGamma[10, 0.05, 0, S, 10, 0.2],
SORTheta[10, 0.05, 0, S, 10, 0.2]}], {S, 2, 16, 1}]]
\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {
            TagBox[
              InterpretationBox["\<\"  2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  8.0000\"\>",
                8.00000000644286,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.0000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                3.78605068974764*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  7.0000\"\>",
                7.00000001216859,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.0000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                3.31279435714354*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  6.0000\"\>",
                6.00000001626863,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.0000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                2.83953802310319*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  5.0000\"\>",
                5.00000002474366,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.0000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                2.36628169061431*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  4.0000\"\>",
                4.00000001335052,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.0000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                1.89302535033216*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  3.0000\"\>",
                3.00000003559577,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.0000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0000\"\>",
                1.41976902458204*^-05,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  2.1077\"\>",
                2.10770997752037,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.7209\"\>",
                -0.720882344604537,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.3121\"\>",
                0.312101532687065,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0057\"\>",
                -0.00567368783987018,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  1.5187\"\>",
                1.51867718887202,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.4779\"\>",
                -0.477941725336398,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.1869\"\>",
                0.18686958231742,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0116\"\>",
                -0.0116248069658932,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  1.1211\"\>",
                1.12105364935597,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.3288\"\>",
                -0.328759845906071,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.1179\"\>",
                0.117918720596282,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0153\"\>",
                -0.0153122928482333,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.8437\"\>",
                0.843664376021599,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.2327\"\>",
                -0.232742911679444,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0775\"\>",
                0.0775170762710948,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0173\"\>",
                -0.017318736831994,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.6451\"\>",
                0.645056780095162,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.1686\"\>",
                -0.168618730095786,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0526\"\>",
                0.0526463325343865,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0181\"\>",
                -0.018129985427607,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.4998\"\>",
                0.499825417293618,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.1245\"\>",
                -0.12450456213571,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0367\"\>",
                0.0367152509459057,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0181\"\>",
                -0.0181232734676781,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.3918\"\>",
                0.391750101407898,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0934\"\>",
                -0.0934098018066392,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0262\"\>",
                0.0261731244961086,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0176\"\>",
                -0.0175796611096079,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.3101\"\>",
                0.31012428202242,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0710\"\>",
                -0.0710432210630784,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0190\"\>",
                0.0190061172670391,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0167\"\>",
                -0.0167027170464843,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.2477\"\>",
                0.247683376556884,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0547\"\>",
                -0.0546758992821053,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.0140\"\>",
                0.0140216625438139,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.0156\"\>",
                -0.0156365489742552,
                AutoDelete->True],
              (PaddedForm[ #, {6, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  CellMargins->{{42, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakWithin->True,
  GroupPageBreakWithin->True,
  FontSize->8]
}, Open  ]],

Cell["\<\
For comparison, here are the corresponding results from the CJM \
approximation. We still get good agreement, even with 10-year maturity and \
only 40 time-steps in our numerical scheme.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
CJMsamples = TableForm[Join[{{\"S\", \"CJM Value\", \"CJM Delta\", \
\"CJM Gamma\"}},
Table[Map[PaddedForm[N[#], {5, 3}]&, {S, AmericanPutCJM[S, 10, 0.2, 0.05, \
10],AmericanPutDeltaCJM[S, 10, 0.2, 0.05, 10],AmericanPutGammaCJM[S, 10, 0.2, \
0.05, 10]}],
{S, 5, 15, 2}]]]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"CJM Value\"\>", "\<\"CJM Delta\"\>", 
            "\<\"CJM Gamma\"\>"},
          {
            TagBox[
              InterpretationBox["\<\"  5.000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  5.000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  7.000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  3.000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -1.000\"\>",
                -1.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.000\"\>",
                0,
                Editable->False],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\"  9.000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  1.527\"\>",
                1.5267994451994227,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.483\"\>",
                -0.48258672489346205,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.190\"\>",
                0.18981440143322068,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.846\"\>",
                0.84638838251061532,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.234\"\>",
                -0.23415837801887138,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.078\"\>",
                0.078307392759506433,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.501\"\>",
                0.5008083211587171,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.125\"\>",
                -0.1249886894330324,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.037\"\>",
                0.036965093880939548,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.310\"\>",
                0.31048947826757317,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\" -0.071\"\>",
                -0.071218743352615493,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)], 
            TagBox[
              InterpretationBox["\<\"  0.019\"\>",
                0.019093483884424028,
                AutoDelete->True],
              (PaddedForm[ #, {5, 3}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  CellMargins->{{42, Inherited}, {Inherited, Inherited}},
  FontSize->8]
}, Open  ]],

Cell["\<\
We have shown that the 3TLD scheme gives results in agreement with \
the CJM approximation using very few time-steps, and that the agreement \
extends to the basic Greeks. The computation of Rho and Vega for American \
options, so far as this author is aware, requires recomputation with shifted \
values for the interest rate and volatility. A generalization to American \
options of the identities relating Rho to Delta and Vega to Gamma would be of \
considerable benefit! 

The 3TLD scheme was recommended for non-smooth initial data at least forty \
years ago, but has been widely ignored in option-pricing. Given the \
universality of non-smooth initial data in option-pricing problems, it is \
perhaps appropriate to suggest that this should change. Of course, \
practitioners use other devices to get around the problem, such as smoothing \
the initial data, or performing some analytical smoothing for small times to \
expiry. The fully-implicit scheme is also quite good, but requires smaller \
steps for comparable accuracy. No other scheme combines the properties of \
smoothing and high order truncation error that are the characteristics of \
3TLD. Its only real drawback is that the scheme must be managed carefully in \
the neighbourhood of discrete events such as dividends and coupons, or \
time-discontinuous shifts in payoffs - under such situations the scheme \
should either be re-started, or numerical data adjusted with care. \
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 16 Bibliography", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Wilmott, P., Dewynne, J. and Howison, S., 1993, ",
  StyleBox["Option Pricing - Mathematical Models and Computation",
    FontSlant->"Italic"],
  ", Oxford Financial Press.\n"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{673, 680},
WindowMargins->{{7, Automatic}, {Automatic, -4}},
PrintingCopies->1,
PrintingStartingPageNumber->306,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"16. Finite Differences and Americans"}]]}, {Cell[ 
        TextData[ {"16. Finite Differences and Americans"}]], Inherited, 
      Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
PrivateFontOptions->{"FontType"->"Outline"},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8^on3ohP==PTc0b85N`?N0080004/0B`000006G<AMP01
0000I00000400@4100800BL?00400@RoU?0000000000064/01T1T00000000000
00000000007oo@S0`63oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 306, 10, 245, "Title"],

Cell[CellGroupData[{
Cell[2062, 65, 113, 2, 49, "Section"],
Cell[2178, 69, 3968, 68, 659, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6183, 142, 103, 1, 38, "Section"],

Cell[CellGroupData[{
Cell[6311, 147, 185, 4, 32, "Subsection"],
Cell[6499, 153, 132, 2, 20, "Text"],
Cell[6634, 157, 312, 7, 32, "NumberedEquation"],
Cell[6949, 166, 246, 6, 29, "Text"],
Cell[7198, 174, 207, 4, 32, "NumberedEquation"],
Cell[7408, 180, 159, 4, 20, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7604, 189, 112, 2, 32, "Subsection"],
Cell[7719, 193, 280, 7, 20, "Text"],
Cell[8002, 202, 181, 3, 20, "NumberedEquation"],
Cell[8186, 207, 140, 4, 20, "Text"],
Cell[8329, 213, 325, 5, 38, "NumberedEquation"],
Cell[8657, 220, 469, 9, 52, "Text"],
Cell[9129, 231, 502, 8, 56, "NumberedEquation"],
Cell[9634, 241, 242, 5, 29, "Text"],
Cell[9879, 248, 411, 6, 20, "NumberedEquation"],
Cell[10293, 256, 390, 12, 29, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10732, 274, 107, 1, 38, "Section"],

Cell[CellGroupData[{
Cell[10864, 279, 115, 2, 32, "Subsection"],
Cell[10982, 283, 1008, 19, 96, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12027, 307, 109, 1, 32, "Subsection"],
Cell[12139, 310, 284, 5, 29, "Text"],
Cell[12426, 317, 1108, 20, 164, "Input"],
Cell[13537, 339, 199, 4, 29, "Text"],
Cell[13739, 345, 1182, 22, 164, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14958, 372, 175, 4, 32, "Subsection"],
Cell[15136, 378, 168, 4, 20, "Text"],
Cell[15307, 384, 551, 11, 140, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[15895, 400, 174, 4, 32, "Subsection"],
Cell[16072, 406, 414, 10, 92, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[16523, 421, 183, 4, 32, "Subsection"],
Cell[16709, 427, 250, 5, 29, "Text"],
Cell[16962, 434, 132, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[17119, 440, 125, 2, 25, "Input"],
Cell[17247, 444, 147, 4, 29, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17431, 453, 109, 2, 25, "Input"],
Cell[17543, 457, 221, 3, 44, "Print"]
}, Open  ]],
Cell[17779, 463, 172, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[17976, 471, 110, 2, 25, "Input"],
Cell[18089, 475, 160, 2, 32, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[18298, 483, 189, 4, 32, "Subsection"],
Cell[18490, 489, 367, 6, 40, "Text"],

Cell[CellGroupData[{
Cell[18882, 499, 179, 4, 28, "Subsubsection"],
Cell[19064, 505, 486, 10, 106, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19587, 520, 108, 1, 28, "Subsubsection"],
Cell[19698, 523, 287, 5, 50, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[20046, 535, 225, 5, 38, "Section",
  PageBreakAbove->Automatic],

Cell[CellGroupData[{
Cell[20296, 544, 194, 5, 28, "Subsubsection",
  PageBreakAbove->False],
Cell[20493, 551, 227, 5, 29, "Text",
  PageBreakAbove->False],
Cell[20723, 558, 211, 4, 38, "Input",
  PageBreakAbove->False],
Cell[20937, 564, 233, 5, 29, "Text",
  PageBreakAbove->False],
Cell[21173, 571, 218, 4, 38, "Input",
  PageBreakAbove->False],
Cell[21394, 577, 211, 5, 20, "Text",
  PageBreakAbove->False],
Cell[21608, 584, 412, 11, 80, "Input",
  PageBreakAbove->False],
Cell[22023, 597, 350, 10, 68, "Input",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[22422, 613, 203, 4, 38, "Section"],
Cell[22628, 619, 590, 11, 62, "Text"],

Cell[CellGroupData[{
Cell[23243, 634, 262, 8, 36, "Input",
  PageBreakAbove->False,
  PageBreakWithin->True],
Cell[23508, 644, 54, 1, 36, "Output"]
}, Open  ]],
Cell[23577, 648, 697, 14, 62, "Text"],
Cell[24277, 664, 135, 2, 37, "Input"],
Cell[24415, 668, 486, 12, 62, "Input"],
Cell[24904, 682, 170, 3, 25, "Input"],
Cell[25077, 687, 173, 6, 36, "Input"],
Cell[25253, 695, 447, 10, 80, "Input"],

Cell[CellGroupData[{
Cell[25725, 709, 174, 4, 28, "Subsubsection"],
Cell[25902, 715, 267, 9, 20, "Text"],
Cell[26172, 726, 563, 18, 156, "Input",
  PageBreakWithin->True],
Cell[26738, 746, 255, 7, 20, "Text"],
Cell[26996, 755, 569, 16, 145, "Input"],
Cell[27568, 773, 604, 18, 168, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28209, 796, 160, 4, 28, "Subsubsection"],
Cell[28372, 802, 566, 11, 52, "Text"],
Cell[28941, 815, 642, 23, 212, "Input",
  PageBreakWithin->True],
Cell[29586, 840, 475, 13, 112, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[30098, 858, 105, 1, 32, "Subsection"],
Cell[30206, 861, 333, 8, 29, "Text"],

Cell[CellGroupData[{
Cell[30564, 873, 437, 12, 90, "Input"],
Cell[31004, 887, 11466, 325, 192, "Output"]
}, Open  ]],
Cell[42485, 1215, 207, 6, 20, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42729, 1226, 203, 6, 32, "Subsection"],
Cell[42935, 1234, 299, 6, 29, "Text"],
Cell[43237, 1242, 384, 11, 101, "Input"],
Cell[43624, 1255, 156, 3, 37, "Input"],
Cell[43783, 1260, 193, 4, 37, "Input"],
Cell[43979, 1266, 193, 4, 37, "Input"],
Cell[44175, 1272, 236, 4, 52, "Input"],
Cell[44414, 1278, 308, 5, 50, "Input"],
Cell[44725, 1285, 382, 9, 50, "Input"],
Cell[45110, 1296, 305, 6, 90, "Input"],
Cell[45418, 1304, 308, 6, 90, "Input"],
Cell[45729, 1312, 296, 6, 81, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46062, 1323, 114, 1, 32, "Subsection"],

Cell[CellGroupData[{
Cell[46201, 1328, 431, 13, 112, "Input"],
Cell[46635, 1343, 11484, 325, 192, "Output"]
}, Open  ]],
Cell[58134, 1671, 112, 1, 20, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[58283, 1677, 114, 1, 32, "Subsection"],
Cell[58400, 1680, 182, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[58607, 1688, 431, 13, 112, "Input"],
Cell[59041, 1703, 11460, 325, 192, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[70550, 2034, 114, 1, 32, "Subsection"],

Cell[CellGroupData[{
Cell[70689, 2039, 461, 14, 112, "Input"],
Cell[71153, 2055, 11474, 325, 192, "Output"]
}, Open  ]],
Cell[82642, 2383, 1010, 18, 96, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[83701, 2407, 187, 4, 38, "Section"],
Cell[83891, 2413, 798, 14, 73, "Text"],

Cell[CellGroupData[{
Cell[84714, 2431, 213, 3, 37, "Input"],
Cell[84930, 2436, 54, 1, 36, "Output"]
}, Open  ]],
Cell[84999, 2440, 135, 2, 37, "Input"],
Cell[85137, 2444, 486, 12, 62, "Input"],
Cell[85626, 2458, 170, 3, 25, "Input"],
Cell[85799, 2463, 150, 3, 25, "Input"],
Cell[85952, 2468, 447, 10, 80, "Input"],

Cell[CellGroupData[{
Cell[86424, 2482, 174, 4, 28, "Subsubsection"],
Cell[86601, 2488, 724, 18, 84, "Text"],
Cell[87328, 2508, 228, 4, 37, "Input"],
Cell[87559, 2514, 539, 17, 156, "Input"],
Cell[88101, 2533, 287, 8, 29, "Text"],
Cell[88391, 2543, 228, 4, 37, "Input"],
Cell[88622, 2549, 570, 16, 145, "Input"],
Cell[89195, 2567, 224, 4, 37, "Input"],
Cell[89422, 2573, 605, 18, 168, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[90064, 2596, 160, 4, 28, "Subsubsection"],
Cell[90227, 2602, 417, 9, 40, "Text"],
Cell[90647, 2613, 737, 25, 233, "Input",
  PageBreakWithin->True],
Cell[91387, 2640, 569, 9, 125, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[91993, 2654, 100, 1, 32, "Subsection"],

Cell[CellGroupData[{
Cell[92118, 2659, 217, 6, 36, "Input"],
Cell[92338, 2667, 12516, 402, 128, 4185, 294, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[104903, 3075, 225, 6, 44, "Subsection"],
Cell[105131, 3083, 515, 8, 52, "Text"],
Cell[105649, 3093, 384, 11, 101, "Input"],
Cell[106036, 3106, 156, 3, 37, "Input"],
Cell[106195, 3111, 193, 4, 37, "Input"],
Cell[106391, 3117, 193, 4, 37, "Input"],
Cell[106587, 3123, 236, 4, 52, "Input"],
Cell[106826, 3129, 308, 5, 50, "Input"],
Cell[107137, 3136, 382, 9, 50, "Input"],
Cell[107522, 3147, 305, 6, 90, "Input"],
Cell[107830, 3155, 308, 6, 90, "Input"],
Cell[108141, 3163, 296, 6, 81, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[108486, 3175, 135, 1, 38, "Section"],
Cell[108624, 3178, 333, 6, 40, "Text"],

Cell[CellGroupData[{
Cell[108982, 3188, 422, 11, 101, "Input"],
Cell[109407, 3201, 14383, 400, 204, "Output"]
}, Open  ]],
Cell[123805, 3604, 229, 5, 29, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[124071, 3614, 197, 4, 38, "Section",
  PageBreakAbove->Automatic],
Cell[124271, 3620, 249, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[124545, 3629, 279, 5, 50, "Input"],
Cell[124827, 3636, 48, 1, 36, "Output"]
}, Open  ]],
Cell[124890, 3640, 1010, 16, 106, "Text"],

Cell[CellGroupData[{
Cell[125925, 3660, 396, 10, 80, "Input"],
Cell[126324, 3672, 11095, 325, 192, "Output"]
}, Open  ]],
Cell[137434, 4000, 625, 11, 62, "Text"],

Cell[CellGroupData[{
Cell[138084, 4015, 149, 3, 25, "Input"],
Cell[138236, 4020, 54, 1, 36, "Output"]
}, Open  ]],
Cell[138305, 4024, 325, 7, 29, "Text"],

Cell[CellGroupData[{
Cell[138655, 4035, 170, 4, 25, "Input"],
Cell[138828, 4041, 15186, 463, 118, 4586, 327, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[154029, 4507, 419, 9, 40, "Text"],

Cell[CellGroupData[{
Cell[154473, 4520, 170, 4, 25, "Input"],
Cell[154646, 4526, 10887, 343, 118, 3520, 247, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[165570, 4874, 152, 3, 32, "Subsection"],
Cell[165725, 4879, 506, 8, 52, "Text"],
Cell[166234, 4889, 409, 10, 80, "Input",
  PageBreakWithin->True],
Cell[166646, 4901, 156, 3, 25, "Input"],
Cell[166805, 4906, 196, 4, 37, "Input"],
Cell[167004, 4912, 205, 5, 37, "Input"],
Cell[167212, 4919, 284, 7, 29, "Text"],

Cell[CellGroupData[{
Cell[167521, 4930, 330, 6, 50, "Input"],
Cell[167854, 4938, 17879, 796, 128, 10598, 701, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[185748, 5737, 106, 1, 20, "Text"],
Cell[185857, 5740, 145, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[186027, 5746, 111, 2, 25, "Input"],
Cell[186141, 5750, 70, 1, 36, "Output"]
}, Open  ]],
Cell[186226, 5754, 123, 2, 25, "Input"],
Cell[186352, 5758, 338, 7, 29, "Text"],
Cell[186693, 5767, 283, 5, 50, "Input"],
Cell[186979, 5774, 281, 5, 37, "Input"],
Cell[187263, 5781, 241, 4, 64, "Input"],
Cell[187507, 5787, 262, 5, 64, "Input"],
Cell[187772, 5794, 343, 6, 76, "Input"],
Cell[188118, 5802, 467, 10, 62, "Input"],
Cell[188588, 5814, 156, 4, 20, "Text"],
Cell[188747, 5820, 280, 6, 46, "Input"],
Cell[189030, 5828, 282, 6, 46, "Input"],
Cell[189315, 5836, 110, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[189450, 5841, 132, 2, 25, "Input"],
Cell[189585, 5845, 35, 1, 36, "Output"]
}, Open  ]],
Cell[189635, 5849, 146, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[189806, 5857, 200, 5, 25, "Input"],
Cell[190009, 5864, 12727, 351, 128, 3182, 228, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[202797, 6222, 184, 4, 38, "Section"],
Cell[202984, 6228, 577, 9, 62, "Text"],

Cell[CellGroupData[{
Cell[203586, 6241, 134, 1, 32, "Subsection"],
Cell[203723, 6244, 375, 7, 40, "Text"],

Cell[CellGroupData[{
Cell[204123, 6255, 375, 11, 80, "Input"],
Cell[204501, 6268, 14051, 401, 164, "Output",
  PageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[218589, 6674, 396, 9, 48, "Input"],
Cell[218988, 6685, 4723, 137, 80, "Output"]
}, Open  ]],
Cell[223726, 6825, 177, 4, 20, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[223952, 6835, 116, 1, 42, "Section"],

Cell[CellGroupData[{
Cell[224093, 6840, 115, 1, 33, "Subsection"],
Cell[224211, 6843, 383, 8, 33, "Text"],

Cell[CellGroupData[{
Cell[224619, 6855, 379, 10, 48, "Input"],
Cell[225001, 6867, 14073, 402, 164, "Output",
  PageBreakAbove->False,
  PageBreakWithin->True]
}, Open  ]],
Cell[239089, 7272, 267, 5, 33, "Text"],

Cell[CellGroupData[{
Cell[239381, 7281, 399, 10, 48, "Input"],
Cell[239783, 7293, 4723, 137, 80, "Output"]
}, Open  ]],
Cell[244521, 7433, 1534, 24, 164, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[246104, 7463, 99, 1, 42, "Section"],
Cell[246206, 7466, 263, 6, 33, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

