(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     98813,       3271]*)
(*NotebookOutlinePosition[    100670,       3327]*)
(*  CellTagsIndexPosition[    100626,       3323]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 17.
Linear Programming Alternatives to PSOR and Regression\
\>", "Title",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["The Dempster-Hutton LP Method, and Robust Regression via LP", 
  "Subsubtitle",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["17.1 Introduction ", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The PSOR method described in the previous chapters works, but raises \
questions of efficiency, even when employed with a cunning difference scheme. \
The number of time-steps is small, but there is quite a lot of effort per \
time-step to solve the problem. I am not aware of a complete answer to the \
question of what the most efficient solution method is - the standard ",
  StyleBox["explicit",
    FontSlant->"Italic"],
  " methods employing trees will be discussed in the next three chapters -  \
but an interesting alternative for dealing with an implicit approach, with \
considerable potential, is the linear programming (LP) approach introduced \
recently by M. Dempster and J. Hutton (1996) - henceforth abbreviated LPDH. \
The idea is to map the constrained matrix equations into a linear programming \
problem, and then to solve them using standard, or, preferably, carefully \
optimized, LP techniques. \n\nThere are several different ways of solving LP \
problems, and it is possible to consider:\n\n(a) exploitation of the dual \
problem;\n(b) use of interior point methods instead of the simplex method;\n\
(c) exploitation of the special structure of the problem due to the \
tridiagonal matrix equations.\n\nThis is very much research in progress, and \
the purpose of this chapter is really to perform the simplest task of \
implementing one version of the algorithm using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s built-in LP solver, in order to illustrate the basic idea, and, \
hopefully, to stimulate efforts into further algorithm optimization. The \
author would be delighted to hear of much more efficient implementations. \n\n\
Once we get familiar with the LP solver in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", we shall also show quickly how to use it to do robust regression \
analysis. Although not directly related to derivative modelling, fitting \
lines to data is commonplace in finance, and frequently the data are very \
noisy. LP methods allow one to implement one form of \"robust\" regression \
algorithm."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["17.2 Naive Summary of LPDH Approach", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
What we want to do is to obtain an alternative to the PSOR \
routine\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(CompPSORSolve\  = \ 
      Compile[\n\t\t\t{{diag, \ _Real, \ 0}, \ {offdiag, \ _Real, \ 
            0}, \ {r, \ _Real, \ 1}, \ {eps, \ _Real, \ 
            0}, \ {relax, \ _Real, \ 0}, {kickoff, \ _Real, \ 
            1}}, \n\t\t\tModule[{len\  = \ Length[r], \ yu\  = \ r, \ 
            yold\  = \ kickoff, \ ynew\  = \ r, \ 
            error\  = \ 100.0}, \n\t\t\t\t\tWhile[
            error\  > \ 
              eps^2, \n\t\((Do[\((yu[\([i]\)] = 
                    r[\([i]\)]/
                        diag - \((offdiag/
                            diag)\)*\((If[i == 1, \ 0, ynew[\([i - 1]\)]] + 
                            If[i < len, yold[\([i + 1]\)], 
                              0])\); \n\t\tynew[\([i]\)]\  = \ 
                    Max[kickoff[\([i]\)], \ 
                      yold[\([i]\)]\  + \ 
                        relax*\((yu[\([i]\)]\  - \ 
                              yold[\([i]\)])\)]\ \ )\), \ {i, \ 1, \ 
                  len}]; \n\t
              error\  = \ \((ynew - yold)\) . \((ynew - yold)\); \n\t
              yold\  = \ ynew)\)]; \tyold]]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
for solving a constrained tridiagonal matrix problem. The matrix \
problem, that is the discretization of the diffusion equation, can be written \
formally in the form\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`A . y = b\)], "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["where the diffusion equation is satisfied, and the condition", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`y \[GreaterEqual] payoff\)], "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
expresses the early exercise constraint. In this text we have not \
concerned ourselves with the formal manner in which these equations can be \
represented and combined mathematically. If we wish to extend (1) to cover \
both the possibility of early exercise and smooth diffusion evolution, we \
have to write it in the form\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`A . y \[GreaterEqual] b\)], "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "What this inequality means financially is that the return on a \
risk-neutral portfolio is at least that given by a bank deposit, leading to \
the Black-Scholes partial differential ",
  StyleBox["inequality",
    FontSlant->"Italic"],
  ", and formula (3) once discretized. We also have the condition that either \
the payoff inequality, or the (discretized) PDE inequality, must be satisfied \
as equality. Putting it all together, we get the conditions"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(TraditionalForm\`y \[GreaterEqual] payoff\), 
    \(TraditionalForm\`A . y \[GreaterEqual] b\), 
    \(TraditionalForm\`Min[A . y - b, \ y - payoff]\  = \ 0\)}], 
  "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where the minimization applies component by component. This type of \
representation is known as an order complementarity problem, and a detailed \
discussion of the associated mathematics is given by Borwein and Dempster \
(1989). This type of formulation has been popularized by Wilmott ",
  StyleBox["et al",
    FontSlant->"Italic"],
  " (1993) who call it a linear complementarity problem. \n\nUnder certain \
circumstances, which amount to requiring that the log-price discrete step is \
sufficiently small, the class of schemes represented by the \[Theta] method \
are shown by Dempster and Hutton to be equivalent to an LP problem:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(TraditionalForm\`Min[c\^T . y]\), 
    \(TraditionalForm\`y \[GreaterEqual] payoff\), 
    \(TraditionalForm\`A . y \[GreaterEqual] b\)}], "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`c\)]],
  " is a positive vector with the same length as the desired solution. This \
can be standardized further to the problem"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    FormBox[\(Min[c\^T . u]\), TraditionalForm], 
    FormBox[
      RowBox[{
        FormBox[\(u \[GreaterEqual] 0\),
          "TraditionalForm"], " "}], TraditionalForm], 
    FormBox[\(A . u \[GreaterEqual] b - A . payoff\), TraditionalForm]}], 
  "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "It will be evident that this is now an absolutely standard LP problem, \
which can be passed to a generic or suitably optimized solver. Clearly our \
next task is to understand the generic LP solver provided by ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "17.3 Linear Programming in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"]
}], "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "There are three basic functions in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " that deal with the task of finding the minimum or maximum of a linear \
function with a linear constraint set. Here is the first pair (",
  StyleBox["ConstrainedMax",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is obvious from this one):"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["?ConstrainedMin", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \("ConstrainedMin[f, {inequalities}, {x, y, ... }] finds the global \
minimum of f in the domain specified by the inequalities. The variables x, y, \
...  are all assumed to be non-negative."\)], "Print"]
}, Open  ]],

Cell[TextData[{
  "These two are symbolic in their operation, and require the introduction of \
named symbols to operate. The third, just called ",
  StyleBox["LinearProgramming",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", just takes the numerical matrix entries:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?LinearProgramming\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("LinearProgramming[c, m, b] finds the vector x which minimizes the \
quantity c.x subject to the constraints m.x >= b and x >= 0."\)], "Print"]
}, Open  ]],

Cell["They can be used in equivalent ways - for example:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ConstrainedMin[
      x\  + \ 2\ y\  - \ z, \ {x\  - \ y - z > 2\ }, \ {x, \ y, \ z}]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({2, {x \[Rule] 2, y \[Rule] 0, z \[Rule] 0}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(LinearProgramming[{1, 2, \(-1\)}, \ {{1, \(-1\), \(-1\)}}, \ {2}]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({2, 0, 0}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "We shall use ",
  StyleBox["LinearProgramming",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to implement the LPDH method. Later we shall see how to use ",
  StyleBox["ConstrainedMin",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to solve a regression problem."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["17.4 Basic Implementation of LPDH", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True],

Cell[TextData[{
  "We now have all we need to implement a basic LPDH solution of the American \
option problem, and will use the Put as our working example. We shall do so \
on the basis of the two-time-level Douglas algorithm, where the ",
  Cell[BoxData[
      \(TraditionalForm\`A\)]],
  " matrix is defined by the difference equation"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\((1\  - \ 6  \[Alpha])\) \((u\_\(n - 1\)\%\(m + 1\) + 
              u\_\(n + 1\)\%\(m + 1\))\)\  + \ \((10\  + \ 12  \[Alpha])\) 
          u\_n\%\(m + 1\) = \((1\  + \ 6  \[Alpha])\) \((u\_\(n - 1\)\%m + 
              u\_\(n + 1\)\%m)\)\  + \((10 - 12  \[Alpha])\) 
          u\_n\%\(m\ \)\)], "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The right side of this form of the discretized diffusion equation \
is given by relations involving the matrix:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
DougDMatrix[alpha_, vec_List] := Module[{temp},
temp = (10 - 12*alpha)*vec + (1+6*alpha)*(RotateRight[vec] + \
RotateLeft[vec]);
temp[[1]] = Simplify[First[temp] - (1+6*alpha)*Last[vec]];
temp[[-1]] = Simplify[Last[temp] - (1 + 6*alpha)*First[vec]];
temp]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Previously, for the left side of the discrete equation, we just \
wrote down the sequence of tridiagonal entries\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
DougCMatrix[alpha_, nminus_, nplus_] :=
Sequence[Table[1-6*alpha, {nplus+nminus-2}], Table[10+12*alpha, \
{nplus+nminus-1}],
Table[1-6*alpha, {nplus+nminus-2}]]\
\>", "Input",
  CellFrame->False,
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
or even just the elements of the diagonal and off-diagonal entries:\
\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(DougDiag[alpha_]\  := \ 10\  + \ 12  alpha; \n
    DougOffDiag[alpha_]\  := \ 1\  - \ 6  alpha; \)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The first of these, for example, could be fed to the tridiagonal \
solver, to treat European-style options:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(CompTridiagSolve\  = \ \t\t\tCompile[{{a, \ _Real, \ 
            1}, \ {b, _Real, \ 1}, \ \t\t\t{c, \ _Real, \ 
            1}, \ {r, \ _Real, \ 1}}, \n\ \ \ \ Module[{len = Length[r], \ 
            solution = r, \ \taux\  = \ 1/\((b[\([1]\)])\), \ \t\t\taux1 = 
              r, \ a1\  = \ Prepend[a, \ 0.0], \ 
            iter}, \ \tsolution[\([1]\)]\  = \ aux\ *r[\([1]\)]; \ \t\tDo[
            aux1[\([iter]\)]\  = \ 
              c[\([iter - 1]\)]\ aux; \n\t\t\t\t\t\ \ \ \ aux\  = \ 
              1/\((b[\([iter]\)]\  - \ 
                    a1[\([iter]\)]*
                      aux1[\([iter]\)])\); \n\t\t\t\t\t\t\ \ \ \ \
solution[\([iter]\)]\  = \ \((r[\([iter]\)] - 
                    a1[\([iter]\)]\ solution[\([iter - 
                            1]\)])\)\ aux, \n\t\ \ \ \ {iter, \ 2, \ 
              len}]; \ \t\t\t\tDo[
            solution[\([iter]\)]\  -= \ 
              aux1[\([iter + 1]\)]\ solution[\([iter + 
                      1]\)], \n\t\ \ \ \ {iter, \ len - 1, \ 1, \ \(-1\)}]; 
          solution]]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
and the second simpler form could be fed to the PSOR solver given \
earlier, for American-style implementations. In our implementation of the \
LPDH alternative for this case, we need a full matrix to supply to the LP \
solver, at least in its default implementation:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(FullDougCMatrix[alpha_, \ nminus_, \ nplus_]\  := \n
      Table[Which[i == j, 
          10 + 12*alpha, \n\t\t\ti == j - 1 || i == j + 1, \ \t1 - 6*alpha, \ 
          True, \ 0], \ {i, \ 1, \ nplus + nminus - 1}, \n\t\t\ {j, \ 1, \ 
          nplus + nminus - 1}]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["For example:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatrixForm[FullDougCMatrix[1/2, \ 3, \ 3]]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"16", \(-2\), "0", "0", "0"},
            {\(-2\), "16", \(-2\), "0", "0"},
            {"0", \(-2\), "16", \(-2\), "0"},
            {"0", "0", \(-2\), "16", \(-2\)},
            {"0", "0", "0", \(-2\), "16"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell["\<\
It is clear that for realistically sized problems, this particular \
implementation is not suitable, as we waste memory and time creating and \
filling a large array consisting mostly of zeros. However, here we are using \
it for illustration, and will now proceed to solve the Put equation. As a \
reminder, the changes of variables used to obtain the diffusion equation are, \
as before,\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(\(NonDimExpiry[T_, sd_] := \(sd\^2\ T\)\/2; \)\), 
    \(kone[r_, sd_] := \(2  r\)\/sd\^2; \ 
    ktwo[r_, q_, sd_] := \(2\ \((r - q)\)\)\/sd\^2; \n
    ValuationMultiplier[strike_, r_, q_, x_, tau_, sd_] := 
      strike\ Exp[\(-\(1\/2\)\)\ \((ktwo[r, q, sd] - 
                  1)\)\ x - \((1\/4\ \((ktwo[r, q, sd] - 1)\)\^2 + 
                  kone[r, sd])\)\ tau]\)}], "Input",
  CellFrame->False,
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["The boundary conditions for a Put are expressed by", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(\(g[x_, tau_, r_, q_, sd_] := 0; \)\), 
    \(\(f[x_, tau_, r_, q_, sd_] := 
      Exp[1\/2\ \((ktwo[r, q, sd] - 1)\)\ x + 
            1\/4\ \((\((ktwo[r, q, sd] - 1)\)\^2 + 4\ kone[r, sd])\)\ tau] - 
        Exp[1\/2\ \((ktwo[r, q, sd] + 1)\)\ x + 
            1\/4\ \((\((ktwo[r, q, sd] - 1)\)\^2 + 
                  4\ kone[r, sd])\)\ tau]; \)\)}], "Input",
  CellFrame->False,
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["Initial conditions are defined by", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(PutExercise[x_, r_, q_, sd_] := 
      Max[Exp[1\/2\ \((ktwo[r, q, sd] - 1)\)\ x]\ \((1 - Exp[x])\), 
        0]; \)\)], "Input",
  CellFrame->False,
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["\<\
This is a special case of the general payoff constraint function, \
which is given by\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(PutPayoff[x_, \ tau_, \ r_, \ q_, \ sd_]\  := \ 
      Exp[\((\((ktwo[r, q, sd] - 1)\)^2/4\  + \ kone[r, sd])\)*tau]*\n
        Max[Exp[\((ktwo[r, q, sd] - 1)\)*x/2] \((1 - Exp[x])\), \ 0]; \)\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Now let's build a sample solution, with \[Alpha] = 8, first using \
the old PSOR routine in its standard implementation. The initialization is \
essentially the same whether we use PSOR or LPDH, and is set up for a \
one-year-expiry American Put option with zero continuous dividend yield.\
\>",
   "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(M = 20; \ nminus\  = \ 60; \ nplus\  = \ 60; \ dx\  = \ 0.05; \ 
    dtau\  = \ 0.001; \ \nalpha\  = \ dtau/dx^2; \n
    initial = 
      Table[PutExercise[\((k - 1 - nminus)\)\ dx, 0.05, 0, 0.2], \ {k, 
          nminus + nplus + 1}]; \n
    lower = Table[
        f[\(-nminus\)\ dx, \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \n
    upper = Table[
        g[\(+nplus\)\ dx, \ \ \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \nwold\  = \ initial; wvold\  = \ wold; wnew\  = \ wold; \n
    CMat\  = \ DougCMatrix[alpha, nminus, nplus]; \nrelax\  = \ 1.1; \n
    tolerance\  = \ 0.000001; \)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
So first we do the evolution with PSOR, and time it (266 MHz G3 \
results);\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Timing[For[m=2, m<=M+1, m++,
(wvold = wold;
wold = wnew;
rhs = DougDMatrix[alpha, Take[wold, {2, -2}]]+
Table[
Which[
k==1, (6*alpha+1)*lower[[m-1]] + (6*alpha-1)*lower[[m]],
k== nplus + nminus-1, (6*alpha+1)*upper[[m-1]] + (6*alpha-1)*upper[[m]],
True, 0],
{k, 1, nplus + nminus-1}];
payoff = Table[PutPayoff[(k-1-nminus)*dx, (m-1)*dtau, 0.05, 0, 0.2], {k, 2, \
nminus+nplus}];
temp = CompPSORSolve[DougDiag[alpha], DougOffDiag[alpha], rhs, tolerance, \
relax, payoff];
wnew = Join[{lower[[m]]}, temp, {upper[[m]]}])
]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({2.41666666666424134`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell["Next we do an interpolation to supply a continuous function:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
interpoldatab = 
   Table[{(k - nminus - 1)*dx, wnew[[k]]}, {k, 1, nminus + nplus + 1}]; 
ufuncb = Interpolation[interpoldatab, InterpolationOrder -> 3];
Valuation[strike_, r_, q_, S_, T_, sd_] := 
  ValuationMultiplier[strike, r, q, Log[S/strike], (sd^2*T)/2, sd]*
   ufuncb[Log[S/strike]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["Next we do a valuation plot and tabulation of sample values:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[Valuation[10, 0.05, 0, S, 1, 0.2], {S, \ 2, \ 18}, \ 
      PlotPoints \[Rule] 50, PlotRange \[Rule] All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.0595238 0.0146796 0.0735799 [
[.05357 .00218 -9 -9 ]
[.05357 .00218 9 0 ]
[.35119 .00218 -9 -9 ]
[.35119 .00218 9 0 ]
[.5 .00218 -6 -9 ]
[.5 .00218 6 0 ]
[.64881 .00218 -12 -9 ]
[.64881 .00218 12 0 ]
[.79762 .00218 -6 -9 ]
[.79762 .00218 6 0 ]
[.94643 .00218 -12 -9 ]
[.94643 .00218 12 0 ]
[.18988 .16184 -6 -4.5 ]
[.18988 .16184 0 4.5 ]
[.18988 .309 -6 -4.5 ]
[.18988 .309 0 4.5 ]
[.18988 .45616 -6 -4.5 ]
[.18988 .45616 0 4.5 ]
[.18988 .60332 -6 -4.5 ]
[.18988 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.05357 .01468 m
.05357 .02093 L
s
[(2.5)] .05357 .00218 0 1 Mshowa
.35119 .01468 m
.35119 .02093 L
s
[(7.5)] .35119 .00218 0 1 Mshowa
.5 .01468 m
.5 .02093 L
s
[(10)] .5 .00218 0 1 Mshowa
.64881 .01468 m
.64881 .02093 L
s
[(12.5)] .64881 .00218 0 1 Mshowa
.79762 .01468 m
.79762 .02093 L
s
[(15)] .79762 .00218 0 1 Mshowa
.94643 .01468 m
.94643 .02093 L
s
[(17.5)] .94643 .00218 0 1 Mshowa
.125 Mabswid
.08333 .01468 m
.08333 .01843 L
s
.1131 .01468 m
.1131 .01843 L
s
.14286 .01468 m
.14286 .01843 L
s
.17262 .01468 m
.17262 .01843 L
s
.23214 .01468 m
.23214 .01843 L
s
.2619 .01468 m
.2619 .01843 L
s
.29167 .01468 m
.29167 .01843 L
s
.32143 .01468 m
.32143 .01843 L
s
.38095 .01468 m
.38095 .01843 L
s
.41071 .01468 m
.41071 .01843 L
s
.44048 .01468 m
.44048 .01843 L
s
.47024 .01468 m
.47024 .01843 L
s
.52976 .01468 m
.52976 .01843 L
s
.55952 .01468 m
.55952 .01843 L
s
.58929 .01468 m
.58929 .01843 L
s
.61905 .01468 m
.61905 .01843 L
s
.67857 .01468 m
.67857 .01843 L
s
.70833 .01468 m
.70833 .01843 L
s
.7381 .01468 m
.7381 .01843 L
s
.76786 .01468 m
.76786 .01843 L
s
.82738 .01468 m
.82738 .01843 L
s
.85714 .01468 m
.85714 .01843 L
s
.8869 .01468 m
.8869 .01843 L
s
.91667 .01468 m
.91667 .01843 L
s
.02381 .01468 m
.02381 .01843 L
s
.97619 .01468 m
.97619 .01843 L
s
.25 Mabswid
0 .01468 m
1 .01468 L
s
.20238 .16184 m
.20863 .16184 L
s
[(2)] .18988 .16184 1 0 Mshowa
.20238 .309 m
.20863 .309 L
s
[(4)] .18988 .309 1 0 Mshowa
.20238 .45616 m
.20863 .45616 L
s
[(6)] .18988 .45616 1 0 Mshowa
.20238 .60332 m
.20863 .60332 L
s
[(8)] .18988 .60332 1 0 Mshowa
.125 Mabswid
.20238 .05147 m
.20613 .05147 L
s
.20238 .08826 m
.20613 .08826 L
s
.20238 .12505 m
.20613 .12505 L
s
.20238 .19863 m
.20613 .19863 L
s
.20238 .23542 m
.20613 .23542 L
s
.20238 .27221 m
.20613 .27221 L
s
.20238 .34579 m
.20613 .34579 L
s
.20238 .38258 m
.20613 .38258 L
s
.20238 .41937 m
.20613 .41937 L
s
.20238 .49295 m
.20613 .49295 L
s
.20238 .52974 m
.20613 .52974 L
s
.20238 .56653 m
.20613 .56653 L
s
.25 Mabswid
.20238 0 m
.20238 .61803 L
s
.5 Mabswid
.02381 .60332 m
.04273 .57993 L
.06337 .55442 L
.08275 .53046 L
.10139 .50742 L
.12123 .48289 L
.14033 .45928 L
.16063 .43419 L
.18019 .41001 L
.199 .38675 L
.21902 .36201 L
.23829 .33819 L
.25683 .31528 L
.27656 .29089 L
.29555 .26741 L
.31574 .24245 L
.33519 .21841 L
.3539 .19529 L
.3738 .17054 L
.39297 .14704 L
.41334 .12451 L
.43296 .10557 L
.45184 .08981 L
.47193 .07547 L
.49127 .06383 L
.50987 .05442 L
.52967 .04611 L
.54872 .03957 L
.56898 .03394 L
.5885 .02961 L
.60727 .02629 L
.62725 .02351 L
.64648 .02142 L
.66497 .01985 L
.68466 .01856 L
.70361 .01761 L
.72376 .01684 L
.74317 .01628 L
.76183 .01588 L
.7817 .01556 L
.80082 .01533 L
.82115 .01515 L
.84073 .01502 L
.85957 .01493 L
.87961 .01486 L
.89891 .01481 L
.91747 .01477 L
.93723 .01475 L
.95624 .01473 L
.97619 .01472 L
Mistroke
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{170.75, 105.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002Z0000JA000`40O003h00OZWoo002ZOol000Mo
o`@000=oo`8000=oo`<001moo`03001oogoo00=oo`8000=oo`<000]oo`<000Aoo`8000Moo`<000=o
o`@000=oo`8000=oo`<000Qoo`<000=oo`<000Moo`<000Aoo`03001oogoo00=oo`8000=oo`<000Yo
o`0027oo00<007ooOol00goo0P001Woo00<007ooOol077oo00<007ooOol00goo0P001Woo00<007oo
Ool02Goo00<007ooOol00Woo00@007ooOol000Moo`03001oogoo00=oo`03001oogoo00=oo`8000Io
o`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo00=o
o`8000Ioo`03001oogoo00Moo`002Goo00<007ooOol02Woo00<007ooOol07Goo00<007ooOol02Woo
00<007ooOol02Goo00<007ooOol00Woo00@007ooOol000Moo`03001oogoo00Aoo`03001oogoo00Yo
o`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Aoo`03001oogoo00Yo
o`03001oogoo00Moo`002Woo00<007ooOol01Woo0`0087oo00<007ooOol01goo0`0037oo00<007oo
Ool00Woo00@007ooOol000Moo`03001oogoo00Eoo`03001oogoo00Ioo`<000Uoo`03001oogoo009o
o`<000Qoo`03001oogoo00Aoo`03001oogoo00Moo`<000Yoo`001goo00@007ooOol000Qoo`03001o
ogoo025oo`03001oogoo00Ioo`03001oogoo00aoo`03001oogoo009oo`04001oogoo0007Ool00`00
Oomoo`02Ool01000Oomoo`0027oo00<007ooOol02Goo00<007ooOol00Woo00<007ooOol027oo00<0
07ooOol01Goo00<007ooOol01Woo00<007ooOol02Woo0008Ool20009Ool4000MOol40008Ool4000:
Ool20005Ool20007Ool20005Ool20009Ool40007Ool20004Ool40006Ool20004Ool40008Ool40009
Ool00:Yoo`009Woo00<007ooOol0PGoo000VOol00`00Oomoo`21Ool000MooiT000Yoo`002goo00D0
07ooOomoo`0000Aoo`05001oogooOol00004Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Ao
o`05001oogooOol00004Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo`05001oogooOol0
0004Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo``00003Ool007oo00=oo`05001oogoo
Ool00004Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo`05001oogooOol00004Ool00`00
Oomoo`0<Ool002Ioo`03001oogoo03Ioo`H004Eoo`009Woo00<007ooOol0<goo0`00Bgoo000VOol0
0`00Oomoo`0`Ool3001>Ool002Ioo`03001oogoo02eoo`<0055oo`009Woo00<007ooOol0:Woo0`00
E7oo000VOol00`00Oomoo`0XOol2001GOol002Ioo`03001oogoo02Moo`03001oogoo05Moo`009Woo
00<007ooOol09Goo0P00FWoo000VOol00`00Oomoo`0TOol00`00Oomoo`1JOol002Ioo`03001oogoo
02=oo`03001oogoo05]oo`009Woo00<007ooOol08Woo00<007ooOol0G7oo000VOol00`00Oomoo`0Q
Ool00`00Oomoo`1MOol002Ioo`03001oogoo021oo`03001oogoo05ioo`009Woo00<007ooOol07goo
00<007ooOol0Ggoo000VOol00`00Oomoo`0NOol00`00Oomoo`1POol002Ioo`03001oogoo01eoo`03
001oogoo065oo`009Woo00<007ooOol077oo00<007ooOol0HWoo000VOol00`00Oomoo`0KOol00`00
Oomoo`1SOol001moo`@000=oo`03001oogoo01Yoo`03001oogoo06Aoo`0087oo00<007ooOol00goo
00<007ooOol06Woo00<007ooOol0I7oo000QOol00`00Oomoo`02Ool2000JOol00`00Oomoo`1UOol0
029oo`05001oogooOol0000JOol00`00Oomoo`1VOol001moo`04001oogoo0003Ool00`00Oomoo`0G
Ool00`00Oomoo`1WOol0021oo`8000Aoo`03001oogoo01Ioo`03001oogoo06Qoo`009Woo00<007oo
Ool05Goo00<007ooOol0JGoo000VOol00`00Oomoo`0DOol00`00Oomoo`1ZOol002Ioo`03001oogoo
01Aoo`03001oogoo06Yoo`009Woo00<007ooOol04goo00<007ooOol0Jgoo000VOol00`00Oomoo`0B
Ool00`00Oomoo`1/Ool002Ioo`03001oogoo015oo`03001oogoo06eoo`009Woo00<007ooOol047oo
00<007ooOol0KWoo000VOol00`00Oomoo`0?Ool00`00Oomoo`1_Ool002Ioo`03001oogoo00ioo`03
001oogoo071oo`009Woo00<007ooOol03Woo00<007ooOol0L7oo000VOol00`00Oomoo`0=Ool00`00
Oomoo`1aOol002Ioo`03001oogoo00aoo`03001oogoo079oo`009Woo00<007ooOol02goo00<007oo
Ool0Lgoo000VOol00`00Oomoo`0;Ool00`00Oomoo`1cOol002Ioo`03001oogoo00Yoo`03001oogoo
07Aoo`009Woo00<007ooOol02Goo00<007ooOol0MGoo000POol30003Ool00`00Oomoo`08Ool00`00
Oomoo`1fOol0025oo`03001oogoo009oo`03001oogoo00Moo`03001oogoo07Moo`007goo10000goo
0P001goo00<007ooOol0N7oo000OOol00`00Ool00004Ool00`00Oomoo`05Ool00`00Oomoo`1iOol0
01moo`03001oo`0000Aoo`03001oogoo00Eoo`03001oogoo07Uoo`0087oo0P0017oo00<007ooOol0
17oo00<007ooOol0NWoo000VOol00`00Oomoo`03Ool00`00Oomoo`1kOol002Ioo`03001oogoo009o
o`03001oogoo07aoo`009Woo00D007ooOomoo`0007moo`009Woo00@007ooOol0081oo`009Woo00<0
07oo0000PGoo000VOol00`00Ool00021Ool002Ioo`80089oo`009Woo00<007ooOol0PGoo000UOol2
0023Ool002Eoo`8008=oo`0097oo00<007oo0000Pgoo000SOol01000Oomoo`00Pgoo000ROol01@00
Oomoogoo0000Pgoo000ROol01@00Oomoogoo0000Pgoo000QOol00`00Oomoo`02Ool00`00Oomoo`21
Ool0021oo`03001oogoo00=oo`03001oogoo085oo`007goo00<007ooOol017oo00<007ooOol0PGoo
000NOol01000Ool0000017oo00<007ooOol0PGoo000MOol00`00Ool00002Ool01@00Oomoogoo0000
Pgoo000LOol01000Oomoo`000Woo00@007ooOomoo`80089oo`0077oo00<007ooOol00`0017oo00<0
07ooOol0PGoo000KOol01@00Oomoogoo00001Woo00<007ooOol0PGoo000KOol00`00Oomoo`02Ool3
0003Ool00`00Oomoo`21Ool001Yoo`03001oogoo00Uoo`03001oogoo085oo`006Goo00<007ooOol0
2Woo00<007ooOol0PGoo000HOol00`00Oomoo`0;Ool00`00Oomoo`21Ool001Moo`03001oogoo00ao
o`03001oogoo085oo`005goo00<007ooOol037oo00<007ooOol0PGoo000FOol00`00Oomoo`0=Ool0
0`00Oomoo`21Ool001Eoo`03001oogoo00ioo`03001oogoo085oo`0057oo00<007ooOol03goo00<0
07ooOol0PGoo000DOol00`00Oomoo`0?Ool00`00Oomoo`21Ool001=oo`03001oogoo011oo`03001o
ogoo085oo`004Woo00<007ooOol04Goo00<007ooOol0PGoo000AOol00`00Oomoo`0BOol00`00Oomo
o`21Ool0011oo`03001oogoo01=oo`03001oogoo085oo`003goo00<007ooOol057oo00<007ooOol0
PGoo000>Ool00`00Oomoo`0EOol00`00Oomoo`21Ool000ioo`03001oogoo01Eoo`03001oogoo085o
o`003Goo00<007ooOol047oo0P0017oo00<007ooOol0PGoo000<Ool00`00Oomoo`0@Ool01000Oomo
o`000goo00<007ooOol0PGoo000;Ool00`00Oomoo`0AOol01000Oomoo`000goo0P00PWoo000POol2
0004Ool00`00Oomoo`21Ool001moo`04001oogoo0003Ool00`00Oomoo`21Ool0021oo`8008Qoo`00
ZWoo0000\
\>"],
  ImageRangeCache->{{{0, 169.75}, {104.438, 0}} -> {0.740616, -0.978494, 
  0.111057, 0.0898417}}]
}, Open  ]],

Cell["", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(samples = 
      TableForm[
        Join[{{"\<S\>", "\<Douglas PSOR\>"}}, \n\t
          Table[\((PaddedForm[N[#1], {5, 4}] &)\)/@\n\t\t{S, \ 
                Valuation[10, 0.05, 0, S, 1, 0.2]}, \n\t{S, 2, 16, 1}]]]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"Douglas PSOR\"\>"},
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.000001307694907,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0000009561576002,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0000043089918824,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0000029315856489,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0000049902500834,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.000003881424266,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.9968\"\>",
                1.9968153550085213,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.1462\"\>",
                1.1462425439619635,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.6051\"\>",
                0.60508996469467302,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2953\"\>",
                0.29531953518163817,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1344\"\>",
                0.13440120370623268,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0576\"\>",
                0.057632316370901271,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0235\"\>",
                0.023541290083514837,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0093\"\>",
                0.0092565269316829818,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0035\"\>",
                0.0035245926695905709,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell["\<\
These values are approximately consistent with those given in \
Section 16.7 - it should be noted that here, for our experiments, we are \
using rather fewer stock-price-steps.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Evolution with LPDH - Simple Version", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The initialization is almost identical, except that we compute and store \
the matrix, and the ",
  Cell[BoxData[
      \(TraditionalForm\`c\)]],
  "-vector to be used in the objective function - here this is taken to be a \
vector of unit elements. The initialization takes longer as we are setting up \
a big square array for the constraint matrix - this and the definition of the \
default objective function are highlighted by a comment."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(M = 20; \ nminus\  = \ 60; \ nplus\  = \ 60; \ dx\  = \ 0.05; \ 
    dtau\  = \ 0.001; \ \nalpha\  = \ dtau/dx^2; \n
    initial = 
      Table[PutExercise[\((k - 1 - nminus)\)\ dx, 0.05, 0, 0.2], \ {k, 
          nminus + nplus + 1}]; \n
    lower = Table[
        f[\(-nminus\)\ dx, \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \n
    upper = Table[
        g[\(+nplus\)\ dx, \ \ \((m - 1)\)\ dtau, 0.05, 0, 0.2], \ {m, \ 1, \ 
          M + 1}]; \nwold\  = \ initial; \ wvold\  = \ wold; \n
    wnew\  = \ wold; \n (*\ 
      Creation\ of\ the\ constraint\ matrix\ and\ objective\ function\ for\ \
LP\ *) \nAMat\  = FullDougCMatrix[alpha, \ nminus, \ nplus]; \n
    cvec\  = \ Table[1, \ {k, 1, nminus\  + \ nplus - 1}]; \)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Next we do the evolution with a simple implementation of LPDH. Note \
that only two lines have changed - these are highlighted by comments:\
\>", 
  "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Timing[For[m=2, m<=M+1, m++,
(wvold = wold;
wold = wnew;
rhs = DougDMatrix[alpha, Take[wold, {2, -2}]]+
Table[
Which[
k==1, (6*alpha+1)*lower[[m-1]] + (6*alpha-1)*lower[[m]],
k== nplus + nminus-1, (6*alpha+1)*upper[[m-1]] + (6*alpha-1)*upper[[m]],
True, 0],
{k, 1, nplus + nminus-1}];
payoff = Table[PutPayoff[(k-1-nminus)*dx, (m-1)*dtau, 0.05, 0, 0.2], {k, 2, \
nminus+nplus}];
(* The next two lines replace the PSOR with LPDH *)
fullrhs = rhs - AMat.payoff;
temp = payoff + LinearProgramming[cvec, AMat, fullrhs];
wnew = Join[{lower[[m]]}, temp, {upper[[m]]}])
]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({56.28333333333285`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell["\<\
We shall discuss the efficiency presently - first we need to check \
the result. We do an interpolation to supply a continuous function:\
\>", 
  "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
interpoldatab = 
   Table[{(k - nminus - 1)*dx, wnew[[k]]}, {k, 1, nminus + nplus + 1}]; 
ufuncb = Interpolation[interpoldatab, InterpolationOrder -> 3];
Valuation[strike_, r_, q_, S_, T_, sd_] := 
  ValuationMultiplier[strike, r, q, Log[S/strike], (sd^2*T)/2, sd]*
   ufuncb[Log[S/strike]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
As before, we next do a valuation plot and tabulation of sample \
values:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[Valuation[10, 0.05, 0, S, 1, 0.2], {S, \ 2, \ 18}, \ 
      PlotPoints \[Rule] 50, PlotRange \[Rule] All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0952381 0.0595238 0.0146796 0.0735799 [
[.05357 .00218 -9 -9 ]
[.05357 .00218 9 0 ]
[.35119 .00218 -9 -9 ]
[.35119 .00218 9 0 ]
[.5 .00218 -6 -9 ]
[.5 .00218 6 0 ]
[.64881 .00218 -12 -9 ]
[.64881 .00218 12 0 ]
[.79762 .00218 -6 -9 ]
[.79762 .00218 6 0 ]
[.94643 .00218 -12 -9 ]
[.94643 .00218 12 0 ]
[.18988 .16184 -6 -4.5 ]
[.18988 .16184 0 4.5 ]
[.18988 .309 -6 -4.5 ]
[.18988 .309 0 4.5 ]
[.18988 .45616 -6 -4.5 ]
[.18988 .45616 0 4.5 ]
[.18988 .60332 -6 -4.5 ]
[.18988 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.05357 .01468 m
.05357 .02093 L
s
[(2.5)] .05357 .00218 0 1 Mshowa
.35119 .01468 m
.35119 .02093 L
s
[(7.5)] .35119 .00218 0 1 Mshowa
.5 .01468 m
.5 .02093 L
s
[(10)] .5 .00218 0 1 Mshowa
.64881 .01468 m
.64881 .02093 L
s
[(12.5)] .64881 .00218 0 1 Mshowa
.79762 .01468 m
.79762 .02093 L
s
[(15)] .79762 .00218 0 1 Mshowa
.94643 .01468 m
.94643 .02093 L
s
[(17.5)] .94643 .00218 0 1 Mshowa
.125 Mabswid
.08333 .01468 m
.08333 .01843 L
s
.1131 .01468 m
.1131 .01843 L
s
.14286 .01468 m
.14286 .01843 L
s
.17262 .01468 m
.17262 .01843 L
s
.23214 .01468 m
.23214 .01843 L
s
.2619 .01468 m
.2619 .01843 L
s
.29167 .01468 m
.29167 .01843 L
s
.32143 .01468 m
.32143 .01843 L
s
.38095 .01468 m
.38095 .01843 L
s
.41071 .01468 m
.41071 .01843 L
s
.44048 .01468 m
.44048 .01843 L
s
.47024 .01468 m
.47024 .01843 L
s
.52976 .01468 m
.52976 .01843 L
s
.55952 .01468 m
.55952 .01843 L
s
.58929 .01468 m
.58929 .01843 L
s
.61905 .01468 m
.61905 .01843 L
s
.67857 .01468 m
.67857 .01843 L
s
.70833 .01468 m
.70833 .01843 L
s
.7381 .01468 m
.7381 .01843 L
s
.76786 .01468 m
.76786 .01843 L
s
.82738 .01468 m
.82738 .01843 L
s
.85714 .01468 m
.85714 .01843 L
s
.8869 .01468 m
.8869 .01843 L
s
.91667 .01468 m
.91667 .01843 L
s
.02381 .01468 m
.02381 .01843 L
s
.97619 .01468 m
.97619 .01843 L
s
.25 Mabswid
0 .01468 m
1 .01468 L
s
.20238 .16184 m
.20863 .16184 L
s
[(2)] .18988 .16184 1 0 Mshowa
.20238 .309 m
.20863 .309 L
s
[(4)] .18988 .309 1 0 Mshowa
.20238 .45616 m
.20863 .45616 L
s
[(6)] .18988 .45616 1 0 Mshowa
.20238 .60332 m
.20863 .60332 L
s
[(8)] .18988 .60332 1 0 Mshowa
.125 Mabswid
.20238 .05147 m
.20613 .05147 L
s
.20238 .08826 m
.20613 .08826 L
s
.20238 .12505 m
.20613 .12505 L
s
.20238 .19863 m
.20613 .19863 L
s
.20238 .23542 m
.20613 .23542 L
s
.20238 .27221 m
.20613 .27221 L
s
.20238 .34579 m
.20613 .34579 L
s
.20238 .38258 m
.20613 .38258 L
s
.20238 .41937 m
.20613 .41937 L
s
.20238 .49295 m
.20613 .49295 L
s
.20238 .52974 m
.20613 .52974 L
s
.20238 .56653 m
.20613 .56653 L
s
.25 Mabswid
.20238 0 m
.20238 .61803 L
s
.5 Mabswid
.02381 .60332 m
.04273 .57993 L
.06337 .55442 L
.08275 .53046 L
.10139 .50742 L
.12123 .48289 L
.14033 .45928 L
.16063 .43419 L
.18019 .41001 L
.199 .38675 L
.21902 .36201 L
.23829 .33819 L
.25683 .31528 L
.27656 .29089 L
.29555 .26741 L
.31574 .24245 L
.33519 .21841 L
.3539 .19529 L
.3738 .17054 L
.39297 .14704 L
.41334 .12451 L
.43296 .10557 L
.45184 .08981 L
.47193 .07547 L
.49127 .06383 L
.50987 .05442 L
.52967 .04611 L
.54872 .03957 L
.56898 .03394 L
.5885 .02961 L
.60727 .02629 L
.62725 .02351 L
.64648 .02142 L
.66497 .01985 L
.68466 .01856 L
.70361 .01761 L
.72376 .01684 L
.74317 .01628 L
.76183 .01588 L
.7817 .01556 L
.80082 .01533 L
.82115 .01515 L
.84073 .01502 L
.85957 .01493 L
.87961 .01486 L
.89891 .01481 L
.91747 .01477 L
.93723 .01475 L
.95624 .01473 L
.97619 .01472 L
Mistroke
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{170.75, 105.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002Z0000JA000`40O003h00OZWoo002ZOol000Mo
o`@000=oo`8000=oo`<001moo`03001oogoo00=oo`8000=oo`<000]oo`<000Aoo`8000Moo`<000=o
o`@000=oo`8000=oo`<000Qoo`<000=oo`<000Moo`<000Aoo`03001oogoo00=oo`8000=oo`<000Yo
o`0027oo00<007ooOol00goo0P001Woo00<007ooOol077oo00<007ooOol00goo0P001Woo00<007oo
Ool02Goo00<007ooOol00Woo00@007ooOol000Moo`03001oogoo00=oo`03001oogoo00=oo`8000Io
o`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo00=o
o`8000Ioo`03001oogoo00Moo`002Goo00<007ooOol02Woo00<007ooOol07Goo00<007ooOol02Woo
00<007ooOol02Goo00<007ooOol00Woo00@007ooOol000Moo`03001oogoo00Aoo`03001oogoo00Yo
o`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Aoo`03001oogoo00Yo
o`03001oogoo00Moo`002Woo00<007ooOol01Woo0`0087oo00<007ooOol01goo0`0037oo00<007oo
Ool00Woo00@007ooOol000Moo`03001oogoo00Eoo`03001oogoo00Ioo`<000Uoo`03001oogoo009o
o`<000Qoo`03001oogoo00Aoo`03001oogoo00Moo`<000Yoo`001goo00@007ooOol000Qoo`03001o
ogoo025oo`03001oogoo00Ioo`03001oogoo00aoo`03001oogoo009oo`04001oogoo0007Ool00`00
Oomoo`02Ool01000Oomoo`0027oo00<007ooOol02Goo00<007ooOol00Woo00<007ooOol027oo00<0
07ooOol01Goo00<007ooOol01Woo00<007ooOol02Woo0008Ool20009Ool4000MOol40008Ool4000:
Ool20005Ool20007Ool20005Ool20009Ool40007Ool20004Ool40006Ool20004Ool40008Ool40009
Ool00:Yoo`009Woo00<007ooOol0PGoo000VOol00`00Oomoo`21Ool000MooiT000Yoo`002goo00D0
07ooOomoo`0000Aoo`05001oogooOol00004Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Ao
o`05001oogooOol00004Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo`05001oogooOol0
0004Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo``00003Ool007oo00=oo`05001oogoo
Ool00004Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo`05001oogooOol00004Ool00`00
Oomoo`0<Ool002Ioo`03001oogoo03Ioo`H004Eoo`009Woo00<007ooOol0<goo0`00Bgoo000VOol0
0`00Oomoo`0`Ool3001>Ool002Ioo`03001oogoo02eoo`<0055oo`009Woo00<007ooOol0:Woo0`00
E7oo000VOol00`00Oomoo`0XOol2001GOol002Ioo`03001oogoo02Moo`03001oogoo05Moo`009Woo
00<007ooOol09Goo0P00FWoo000VOol00`00Oomoo`0TOol00`00Oomoo`1JOol002Ioo`03001oogoo
02=oo`03001oogoo05]oo`009Woo00<007ooOol08Woo00<007ooOol0G7oo000VOol00`00Oomoo`0Q
Ool00`00Oomoo`1MOol002Ioo`03001oogoo021oo`03001oogoo05ioo`009Woo00<007ooOol07goo
00<007ooOol0Ggoo000VOol00`00Oomoo`0NOol00`00Oomoo`1POol002Ioo`03001oogoo01eoo`03
001oogoo065oo`009Woo00<007ooOol077oo00<007ooOol0HWoo000VOol00`00Oomoo`0KOol00`00
Oomoo`1SOol001moo`@000=oo`03001oogoo01Yoo`03001oogoo06Aoo`0087oo00<007ooOol00goo
00<007ooOol06Woo00<007ooOol0I7oo000QOol00`00Oomoo`02Ool2000JOol00`00Oomoo`1UOol0
029oo`05001oogooOol0000JOol00`00Oomoo`1VOol001moo`04001oogoo0003Ool00`00Oomoo`0G
Ool00`00Oomoo`1WOol0021oo`8000Aoo`03001oogoo01Ioo`03001oogoo06Qoo`009Woo00<007oo
Ool05Goo00<007ooOol0JGoo000VOol00`00Oomoo`0DOol00`00Oomoo`1ZOol002Ioo`03001oogoo
01Aoo`03001oogoo06Yoo`009Woo00<007ooOol04goo00<007ooOol0Jgoo000VOol00`00Oomoo`0B
Ool00`00Oomoo`1/Ool002Ioo`03001oogoo015oo`03001oogoo06eoo`009Woo00<007ooOol047oo
00<007ooOol0KWoo000VOol00`00Oomoo`0?Ool00`00Oomoo`1_Ool002Ioo`03001oogoo00ioo`03
001oogoo071oo`009Woo00<007ooOol03Woo00<007ooOol0L7oo000VOol00`00Oomoo`0=Ool00`00
Oomoo`1aOol002Ioo`03001oogoo00aoo`03001oogoo079oo`009Woo00<007ooOol02goo00<007oo
Ool0Lgoo000VOol00`00Oomoo`0;Ool00`00Oomoo`1cOol002Ioo`03001oogoo00Yoo`03001oogoo
07Aoo`009Woo00<007ooOol02Goo00<007ooOol0MGoo000POol30003Ool00`00Oomoo`08Ool00`00
Oomoo`1fOol0025oo`03001oogoo009oo`03001oogoo00Moo`03001oogoo07Moo`007goo10000goo
0P001goo00<007ooOol0N7oo000OOol00`00Ool00004Ool00`00Oomoo`05Ool00`00Oomoo`1iOol0
01moo`03001oo`0000Aoo`03001oogoo00Eoo`03001oogoo07Uoo`0087oo0P0017oo00<007ooOol0
17oo00<007ooOol0NWoo000VOol00`00Oomoo`03Ool00`00Oomoo`1kOol002Ioo`03001oogoo009o
o`03001oogoo07aoo`009Woo00D007ooOomoo`0007moo`009Woo00@007ooOol0081oo`009Woo00<0
07oo0000PGoo000VOol00`00Ool00021Ool002Ioo`80089oo`009Woo00<007ooOol0PGoo000UOol2
0023Ool002Eoo`8008=oo`0097oo00<007oo0000Pgoo000SOol01000Oomoo`00Pgoo000ROol01@00
Oomoogoo0000Pgoo000ROol01@00Oomoogoo0000Pgoo000QOol00`00Oomoo`02Ool00`00Oomoo`21
Ool0021oo`03001oogoo00=oo`03001oogoo085oo`007goo00<007ooOol017oo00<007ooOol0PGoo
000NOol01000Ool0000017oo00<007ooOol0PGoo000MOol00`00Ool00002Ool01@00Oomoogoo0000
Pgoo000LOol01000Oomoo`000Woo00@007ooOomoo`80089oo`0077oo00<007ooOol00`0017oo00<0
07ooOol0PGoo000KOol01@00Oomoogoo00001Woo00<007ooOol0PGoo000KOol00`00Oomoo`02Ool3
0003Ool00`00Oomoo`21Ool001Yoo`03001oogoo00Uoo`03001oogoo085oo`006Goo00<007ooOol0
2Woo00<007ooOol0PGoo000HOol00`00Oomoo`0;Ool00`00Oomoo`21Ool001Moo`03001oogoo00ao
o`03001oogoo085oo`005goo00<007ooOol037oo00<007ooOol0PGoo000FOol00`00Oomoo`0=Ool0
0`00Oomoo`21Ool001Eoo`03001oogoo00ioo`03001oogoo085oo`0057oo00<007ooOol03goo00<0
07ooOol0PGoo000DOol00`00Oomoo`0?Ool00`00Oomoo`21Ool001=oo`03001oogoo011oo`03001o
ogoo085oo`004Woo00<007ooOol04Goo00<007ooOol0PGoo000AOol00`00Oomoo`0BOol00`00Oomo
o`21Ool0011oo`03001oogoo01=oo`03001oogoo085oo`003goo00<007ooOol057oo00<007ooOol0
PGoo000>Ool00`00Oomoo`0EOol00`00Oomoo`21Ool000ioo`03001oogoo01Eoo`03001oogoo085o
o`003Goo00<007ooOol047oo0P0017oo00<007ooOol0PGoo000<Ool00`00Oomoo`0@Ool01000Oomo
o`000goo00<007ooOol0PGoo000;Ool00`00Oomoo`0AOol01000Oomoo`000goo0P00PWoo000POol2
0004Ool00`00Oomoo`21Ool001moo`04001oogoo0003Ool00`00Oomoo`21Ool0021oo`8008Qoo`00
ZWoo0000\
\>"],
  ImageRangeCache->{{{0, 169.75}, {104.438, 0}} -> {0.740616, -0.978494, 
  0.111057, 0.0898417}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(samples = 
      TableForm[
        Join[{{"\<S\>", "\<Douglas LPDH\>"}}, \n\t
          Table[\((PaddedForm[N[#1], {5, 4}] &)\)/@\n\t\t{S, \ 
                Valuation[10, 0.05, 0, S, 1, 0.2]}, \n\t{S, 2, 16, 1}]]]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"S\"\>", "\<\"Douglas LPDH\"\>"},
          {
            TagBox[
              InterpretationBox["\<\" 2.0000\"\>",
                2.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.000001307694907,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0000009561576002,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0000043089918824,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 5.0000\"\>",
                5.0000029315856489,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 6.0000\"\>",
                6.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 4.0000\"\>",
                4.0000049902500834,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 7.0000\"\>",
                7.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 3.0000\"\>",
                3.000003881424266,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 8.0000\"\>",
                8.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.9968\"\>",
                1.9968153153314108,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 9.0000\"\>",
                9.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 1.1462\"\>",
                1.1462428541842993,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 10.0000\"\>",
                10.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.6051\"\>",
                0.60508994442262287,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 11.0000\"\>",
                11.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.2953\"\>",
                0.29531946843781187,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 12.0000\"\>",
                12.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.1344\"\>",
                0.13440114902175179,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 13.0000\"\>",
                13.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0576\"\>",
                0.057632282394497011,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 14.0000\"\>",
                14.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0235\"\>",
                0.023541271384604507,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 15.0000\"\>",
                15.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0093\"\>",
                0.009256517342998323,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]},
          {
            TagBox[
              InterpretationBox["\<\" 16.0000\"\>",
                16.0,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)], 
            TagBox[
              InterpretationBox["\<\" 0.0035\"\>",
                0.0035245879840721211,
                AutoDelete->True],
              (PaddedForm[ #, {5, 4}]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output",
  FontSize->8]
}, Open  ]],

Cell[TextData[{
  "We note that the results are ",
  StyleBox["identical",
    FontSlant->"Italic"],
  ", to the precision given, with those from the PSOR approach, which \
constitutes a crude numerical verification of the LPDH algorithm."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Towards Efficient Implementations of LPDH", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The example above illustrates the ease with which we can try out a new \
algorithm with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", bearing in mind the rich supply of built-in functions. However, this \
particular implementation of LPDH and the timing comparison with PSOR are \
maximally idiotic and unfair respectively! Our PSOR routine was carefully \
coded and compiled to exploit the constant and tridiagonal nature of the \
constraint matrix, whereas we have built a huge square matrix, mostly of \
zeros, to feed to the standard built-in LP function, which expects a matrix. \
The important point is that the two methods give identical answers, and no \
particular significance should be attached to this particular timing \
comparison.  \n\nIn particular, we have not used the tridiagonal nature of \
the matrix ",
  Cell[BoxData[
      \(TraditionalForm\`A\)]],
  " at all. It was very explicitly exploited in the PSOR solver we have \
described. The efficient implementation of LPDH relies on a form of the \
simplex method tailored to constraint matrices that are tridiagonal. A \
discussion of this is beyond the scope of this text, as is a proper treatment \
of the application of duality and interior point methods, but work in \
progress suggests that a massive improvement in speed can be obtained by a \
carefully optimized LP solver (Dempster ",
  StyleBox["et al",
    FontSlant->"Italic"],
  " 1997)."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["17.5 LP and Robust Regression", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "As we are exploring ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s LP functions, it is a good point at which to introduce the subject of \
robust regression. One form of \"robust\" technique is ",
  Cell[BoxData[
      \(TraditionalForm\`L\^1\)]],
  " regression, and it is easy to use the ",
  StyleBox["ConstrainedMin",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function to implement it symbolically.\n\nConventional regression is \
based on the method of least squares. This is highly prone to the influence \
of outliers. An alternative is to use the method of \"least absolute \
deviation\", where the fit is determined by minimizing the sum of the \
absolute deviations of the fitted and actual values. It is well known how to \
transform this problem into an LP form. Given the data, we work out the \
number (\"",
  Cell[BoxData[
      \(TraditionalForm\`len\)]],
  "\") of observations. Then we isolate the ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "-values and the",
  Cell[BoxData[
      \(TraditionalForm\`\(\ y\)\)]],
  "-values, by transposition if we are given a list of pairs, and invent ",
  Cell[BoxData[
      \(TraditionalForm\`2  len\)]],
  " variables ",
  Cell[BoxData[
      \(TraditionalForm\`n[i]\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`p[i]\)]],
  " to plug into the LP solver, where we minimize the sum of the ",
  Cell[BoxData[
      \(TraditionalForm\`n[i]\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`p[i]\)]],
  " subject to the constraints that the observed ",
  Cell[BoxData[
      \(TraditionalForm\`y[i]\)]],
  " differs from the fitted ",
  Cell[BoxData[
      \(TraditionalForm\`y[i]\)]],
  " by ",
  Cell[BoxData[
      \(TraditionalForm\`n[i] - p[i]\)]],
  ". The slope and intercept are free to take positive or negative values, \
but the ",
  Cell[BoxData[
      \(TraditionalForm\`n[i]\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`p[i]\)]],
  " are non-negative. The LP solver requires that all variables are \
non-negative, so we write the intercept, for example, as ",
  Cell[BoxData[
      \(TraditionalForm\`a\  = \ ap - an\)]],
  ", with ",
  Cell[BoxData[
      \(TraditionalForm\`ap \[GreaterEqual] 0\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`an\  \[LessEqual] \ 0\)]],
  ". The following module takes care of all this, and nicely illustrates the \
symbolic LP solver, ",
  StyleBox["ConstrainedMin",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", in action:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
LADLPFit[data_?MatrixQ, x_] :=
Module[{tr=Transpose[data],len, nvar,n,pvar,p,ap,an,bp,bn, vbls, minfun, \
ineq, lhs}, len = Length[tr[[1]]];
(* Invent all the LP variables *)
nvar = Array[n, len]; pvar = Array[p, len];
vbls = Join[nvar,pvar,{ap,an,bp,bn}];
(* Write objective function as sum of absolute deviations *)
minfun = Apply[Plus, Join[nvar,pvar]];
lhs = (ap-an)*Table[1,{len}]+(bp-bn)*tr[[1]]+nvar-pvar;
ineq = MapThread[#1==#2&,{lhs,tr[[2]]}];
ap-an+(bp-bn)*x /. ConstrainedMin[minfun,ineq,vbls][[2]]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell["\<\
To visualize the answers, we introduce the following function to \
plot the data and the fitted function (of x).\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
plotfit[fn_, data_] := 
Module[{minind, maxind, mindep, maxdep},
minind = Min[Transpose[data][[1]]]; 
maxind = Max[Transpose[data][[1]]];
mindep = Min[Transpose[data][[2]]]; 
maxdep = Max[Transpose[data][[2]]];
Plot[fn, {x, Floor[minind], Ceiling[maxind]}, 
AxesOrigin -> {Floor[minind], 0},
PlotRange -> {Min[0, Floor[mindep]], Ceiling[maxdep] + 1},
Evaluate[Epilog -> {PointSize[0.02], Map[Point, data]}]]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["The Pilot Plant Example", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The book by Rousseeuw and Leroy (1987) gives some interesting \
examples that we can use to check our approach and illustrate the method. \
Here are the basic clean \"Pilot Plant\" data described by Daniel and Wood \
(1971):\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
rdata = {{123, 76}, {109, 70}, {62, 55}, {104, 71}, {57, 55}, {37, \
48}, 
{44, 50}, {100, 66}, {16, 41}, {28, 43}, {138, 82}, {105, 68}, {159, 88}, \
{75, 58}, {88, 64}, {164, 88}, {169, 89}, {167, 88}, {149, 84}, {167, 88}};\
\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["First we do a standard least-squares fit, and look at it.", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["lsqfit = Fit[rdata, {1, x}, x]", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(35.4582724474052568`\[InvisibleSpace]\) + 
      0.321608224673409592`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["plotfit[lsqfit, rdata];", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00563539 0 0.00686704 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.16469 -0.0125 -6 -9 ]
[.16469 -0.0125 6 0 ]
[.30558 -0.0125 -6 -9 ]
[.30558 -0.0125 6 0 ]
[.44646 -0.0125 -6 -9 ]
[.44646 -0.0125 6 0 ]
[.58735 -0.0125 -9 -9 ]
[.58735 -0.0125 9 0 ]
[.72823 -0.0125 -9 -9 ]
[.72823 -0.0125 9 0 ]
[.86912 -0.0125 -9 -9 ]
[.86912 -0.0125 9 0 ]
[.10148 .13734 -12 -4.5 ]
[.10148 .13734 0 4.5 ]
[.10148 .27468 -12 -4.5 ]
[.10148 .27468 0 4.5 ]
[.10148 .41202 -12 -4.5 ]
[.10148 .41202 0 4.5 ]
[.10148 .54936 -12 -4.5 ]
[.10148 .54936 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.16469 0 m
.16469 .00625 L
s
[(25)] .16469 -0.0125 0 1 Mshowa
.30558 0 m
.30558 .00625 L
s
[(50)] .30558 -0.0125 0 1 Mshowa
.44646 0 m
.44646 .00625 L
s
[(75)] .44646 -0.0125 0 1 Mshowa
.58735 0 m
.58735 .00625 L
s
[(100)] .58735 -0.0125 0 1 Mshowa
.72823 0 m
.72823 .00625 L
s
[(125)] .72823 -0.0125 0 1 Mshowa
.86912 0 m
.86912 .00625 L
s
[(150)] .86912 -0.0125 0 1 Mshowa
.125 Mabswid
.05199 0 m
.05199 .00375 L
s
.08016 0 m
.08016 .00375 L
s
.10834 0 m
.10834 .00375 L
s
.13652 0 m
.13652 .00375 L
s
.19287 0 m
.19287 .00375 L
s
.22105 0 m
.22105 .00375 L
s
.24923 0 m
.24923 .00375 L
s
.2774 0 m
.2774 .00375 L
s
.33376 0 m
.33376 .00375 L
s
.36193 0 m
.36193 .00375 L
s
.39011 0 m
.39011 .00375 L
s
.41829 0 m
.41829 .00375 L
s
.47464 0 m
.47464 .00375 L
s
.50282 0 m
.50282 .00375 L
s
.53099 0 m
.53099 .00375 L
s
.55917 0 m
.55917 .00375 L
s
.61553 0 m
.61553 .00375 L
s
.6437 0 m
.6437 .00375 L
s
.67188 0 m
.67188 .00375 L
s
.70006 0 m
.70006 .00375 L
s
.75641 0 m
.75641 .00375 L
s
.78459 0 m
.78459 .00375 L
s
.81276 0 m
.81276 .00375 L
s
.84094 0 m
.84094 .00375 L
s
.8973 0 m
.8973 .00375 L
s
.92547 0 m
.92547 .00375 L
s
.95365 0 m
.95365 .00375 L
s
.98183 0 m
.98183 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.11398 .13734 m
.12023 .13734 L
s
[(20)] .10148 .13734 1 0 Mshowa
.11398 .27468 m
.12023 .27468 L
s
[(40)] .10148 .27468 1 0 Mshowa
.11398 .41202 m
.12023 .41202 L
s
[(60)] .10148 .41202 1 0 Mshowa
.11398 .54936 m
.12023 .54936 L
s
[(80)] .10148 .54936 1 0 Mshowa
.125 Mabswid
.11398 .03434 m
.11773 .03434 L
s
.11398 .06867 m
.11773 .06867 L
s
.11398 .10301 m
.11773 .10301 L
s
.11398 .17168 m
.11773 .17168 L
s
.11398 .20601 m
.11773 .20601 L
s
.11398 .24035 m
.11773 .24035 L
s
.11398 .30902 m
.11773 .30902 L
s
.11398 .34335 m
.11773 .34335 L
s
.11398 .37769 m
.11773 .37769 L
s
.11398 .44636 m
.11773 .44636 L
s
.11398 .48069 m
.11773 .48069 L
s
.11398 .51503 m
.11773 .51503 L
s
.11398 .5837 m
.11773 .5837 L
s
.25 Mabswid
.11398 0 m
.11398 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.11398 .27883 m
.14895 .29254 L
.1871 .30749 L
.22292 .32153 L
.25738 .33503 L
.29405 .3494 L
.32935 .36324 L
.36687 .37794 L
.40302 .39211 L
.4378 .40574 L
.4748 .42024 L
.51042 .4342 L
.54468 .44762 L
.58115 .46191 L
.61625 .47567 L
.65357 .4903 L
.68952 .50439 L
.7241 .51794 L
.7609 .53236 L
.79632 .54624 L
.83397 .56099 L
.87024 .57521 L
.90514 .58889 L
.94226 .60343 L
.97619 .61673 L
s
.02 w
.71696 .5219 Mdot
.63807 .48069 Mdot
.3732 .37769 Mdot
.60989 .48756 Mdot
.34503 .37769 Mdot
.23232 .32962 Mdot
.27177 .34335 Mdot
.58735 .45322 Mdot
.11398 .28155 Mdot
.1816 .29528 Mdot
.80149 .5631 Mdot
.61553 .46696 Mdot
.91984 .6043 Mdot
.44646 .39829 Mdot
.51972 .43949 Mdot
.94801 .6043 Mdot
.97619 .61117 Mdot
.96492 .6043 Mdot
.86348 .57683 Mdot
.96492 .6043 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{176.312, 108.938},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002`0000K1000`40O003h00O/7oo002`Ool000]o
o`80015oo`@0009oo`<000eoo`<000Aoo`8000ioo`03001oogoo009oo`<000Yoo`<000Aoo`8000Ao
o`8000Moo`<000=oo`@0009oo`<000Moo`<000=oo`<000Aoo`8001Qoo`002Woo00@007ooOol0015o
o`03001oogoo00Eoo`03001oogoo00eoo`04001oogoo0002Ool00`00Oomoo`0;Ool00`00Oomoo`05
Ool00`00Oomoo`08Ool00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol000Moo`03001oogoo
00=oo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Eoo`04001oogoo0002Ool00`00Oomo
o`0EOol000Yoo`04001oogoo000BOol00`00Oomoo`04Ool00`00Oomoo`0=Ool01000Oomoo`000Woo
00<007ooOol037oo00<007ooOol017oo00<007ooOol027oo00<007ooOol00Woo00@007ooOol0009o
o`04001oogoo0007Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`05
Ool01000Oomoo`000Woo00<007ooOol05Goo000:Ool01000Oomoo`004goo00<007ooOol00`003Goo
0`000goo00@007ooOol000ioo`04001oogooOol3000;Ool00`00Oomoo`02Ool01000Oomoo`000Woo
00@007ooOol000Moo`03001oogoo00Eoo`03001oogoo00<000Qoo`03001oogoo009oo`<000=oo`04
001oogoo000GOol000Yoo`04001oogoo000@Ool01000Oomoo`000Woo00<007ooOol03Goo00<007oo
Ool00goo00@007ooOol000moo`04001oogoo000=Ool00`00Oomoo`02Ool01000Oomoo`000Woo00@0
07ooOol000Moo`03001oogoo009oo`04001oogoo0002Ool00`00Oomoo`08Ool00`00Oomoo`02Ool0
0`00Oomoo`03Ool01000Oomoo`005goo000;Ool2000BOol20003Ool4000<Ool40003Ool2000=Ool4
0002Ool40009Ool20005Ool20004Ool20007Ool20005Ool20003Ool40006Ool20004Ool40003Ool2
000HOol00;1oo`00/7oo002`Ool000Uooih000Uoo`0037oo00<007ooOol037oo00<007ooOol01Goo
00<007ooOol04goo00<007ooOol04goo00<007ooOol04goo00<007ooOol04goo00<007ooOol04goo
00<007ooOol077oo000KOol00`00Oomoo`2BOol001]oo`03001oogoo099oo`006goo00<007ooOol0
TWoo000KOol00`00Oomoo`2BOol001]oo`03001oogoo099oo`006goo00<007ooOol0TWoo000KOol0
0`00Oomoo`2BOol001]oo`03001oogoo099oo`006goo00<007ooOol0TWoo000KOol00`00Oomoo`2B
Ool001]oo`03001oogoo099oo`006goo00<007ooOol0TWoo000KOol00`00Oomoo`2BOol001]oo`03
001oogoo099oo`006goo00<007ooOol0TWoo000KOol00`00Oomoo`2BOol001]oo`03001oogoo099o
o`003Woo10000goo0P0017oo00<007ooOol0TWoo000?Ool00`00Oomoo`02Ool01000Oomoo`000goo
00<007ooOol0TWoo000@Ool01@00Oomoogoo00000Woo00@007ooOomoo`8009=oo`004Goo00@007oo
Ool0009oo`05001oogooOol0002DOol000ioo`04001oogoo0002Ool01000Oomoo`000goo00<007oo
Ool0TWoo000?Ool20004Ool20004Ool00`00Oomoo`2BOol001]oo`03001oogoo099oo`006goo00<0
07ooOol0TWoo000KOol00`00Oomoo`2BOol001]oo`03001oogoo099oo`006goo00<007ooOol0TWoo
000KOol00`00Oomoo`2BOol001]oo`03001oogoo099oo`006goo00<007ooOol0TWoo000KOol00`00
Oomoo`2BOol001]oo`03001oogoo099oo`006goo00<007ooOol0TWoo000KOol00`00Oomoo`2BOol0
01]oo`03001oogoo099oo`006goo00<007ooOol0TWoo000KOol00`00Oomoo`2BOol001]oo`03001o
ogoo099oo`003goo0`000goo0P0017oo00<007ooOol0TWoo000@Ool01@00Oomoogoo00000Woo00D0
07ooOomoo`0009Aoo`003Woo10000Woo00@007ooOol0009oo`<009=oo`003Woo00<007oo00000goo
00@007ooOol0009oo`H0091oo`003Woo00<007oo00000goo00@007ooOol0009oo`<000=oo`80009o
o`<008Uoo`003goo0P0017oo0P0017oo00<007ooOol017oo1@00RGoo000KOol00`00Oomoo`06Ool3
0029Ool001]oo`03001oogoo00Qoo`<008Moo`006goo00<007ooOol02goo0`00Q7oo000KOol00`00
Oomoo`0>Ool30021Ool001]oo`03001oogoo00ioo`D007moo`006goo00<007ooOol03Woo0`000Woo
1000Ngoo000KOol00`00Oomoo`0DOol5001iOol001]oo`03001oogoo01Aoo`<0009oo`8007Moo`00
6goo00<007ooOol06goo0P00MGoo000KOol00`00Oomoo`0MOol00`00Oomoo`1bOol001]oo`03001o
ogoo01ioo`<0075oo`006goo00<007ooOol087oo1`00Jgoo000KOol00`00Oomoo`0POol3000017oo
00000000Jgoo000KOol00`00Oomoo`0POol300000goo00000004001XOol001]oo`03001oogoo02Yo
o`8000Aoo`<005moo`003goo0P0017oo0P0017oo00<007ooOol0;7oo0`0000Aoo`00000005moo`00
3Woo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`0_Ool4001OOol000ioo`04001oogoo
0002Ool01000Oomoo`000goo0P00<goo0`00GGoo000>Ool30003Ool01000Oomoo`000goo00<007oo
Ool0=Goo0P00Fgoo000>Ool00`00Oomoo`03Ool01000Oomoo`000goo00<007ooOol0=goo0P00FGoo
000?Ool30003Ool20004Ool00`00Oomoo`0iOol2001GOol001]oo`03001oogoo03]oo`<005Aoo`00
6goo00<007ooOol0>goo1@00DWoo000KOol00`00Oomoo`0kOol30002Ool30003Ool30019Ool001]o
o`03001oogoo04=oo`H004Uoo`006goo00<007ooOol0AWoo0`0000Aoo`00000004Eoo`006goo00<0
07ooOol0B7oo1@00AGoo000KOol00`00Oomoo`1:Ool70011Ool001]oo`03001oogoo04Uoo`<0009o
o`<0045oo`006goo00<007ooOol0BGoo0`000Woo1@00?goo000KOol00`00Oomoo`19Ool30007Ool2
000mOol001]oo`03001oogoo05Eoo`8003]oo`006goo00<007ooOol0Egoo0P00>Goo000KOol00`00
Oomoo`1IOol4000eOol001]oo`03001oogoo05Yoo`D003=oo`006goo00<007ooOol0FWoo0`000Woo
0`00<7oo000KOol00`00Oomoo`1ROol3000]Ool000moo`8000Aoo`8000Aoo`03001oogoo06Eoo`<0
02Yoo`003Woo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`1XOol2000XOol000ioo`04
001oogoo0002Ool01000Oomoo`000goo0P00Jgoo0P009Woo000?Ool20003Ool01000Oomoo`000goo
00<007ooOol0Igoo0`000Woo0P0097oo000>Ool01000Oomoo`000Woo00@007ooOol000=oo`03001o
ogoo06Moo`<000Aoo`80029oo`003goo0P0017oo0P0017oo00<007ooOol0Igoo0`001Woo10007Woo
000KOol00`00Oomoo`1aOol5000LOol001]oo`03001oogoo075oo`<0009oo`<001Uoo`006goo00<0
07ooOol0NGoo0`005Woo000KOol00`00Oomoo`1jOol:000>Ool001]oo`03001oogoo07Yoo`<00003
Ool0000000@000ioo`006goo00<007ooOol0NWoo0`0000=oo`0000001P0037oo000KOol00`00Oomo
o`23Ool3000<Ool00001\
\>"],
  ImageRangeCache->{{{0, 175.312}, {107.938, 0}} -> {-14.6307, -10.1734, 
  1.1309, 0.928068}}]
}, Open  ]],

Cell["\<\
In this case the LP solution is given in exact form, and is very \
close to the least squares solution.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["LADLPFit[rdata, x]", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(1329\/37 + \(47\ x\)\/148\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["%//N", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(35.9189189189189184`\[InvisibleSpace]\) + 
      0.317567567567567543`\ x\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Making an Outlier", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["We move one of the points to create an outlier:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["rdata[[6]] = {370, 48};", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["The least squares approach now fails dramatically:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["lsqfit = Fit[rdata, {1, x}, x]", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(58.9388304184686173`\[InvisibleSpace]\) + 
      0.0807115253260766607`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["plotfit[lsqfit, rdata];", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.002574 0 0.00686704 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.15251 -0.0125 -6 -9 ]
[.15251 -0.0125 6 0 ]
[.28121 -0.0125 -9 -9 ]
[.28121 -0.0125 9 0 ]
[.40991 -0.0125 -9 -9 ]
[.40991 -0.0125 9 0 ]
[.53861 -0.0125 -9 -9 ]
[.53861 -0.0125 9 0 ]
[.66731 -0.0125 -9 -9 ]
[.66731 -0.0125 9 0 ]
[.79601 -0.0125 -9 -9 ]
[.79601 -0.0125 9 0 ]
[.92471 -0.0125 -9 -9 ]
[.92471 -0.0125 9 0 ]
[.05249 .13734 -12 -4.5 ]
[.05249 .13734 0 4.5 ]
[.05249 .27468 -12 -4.5 ]
[.05249 .27468 0 4.5 ]
[.05249 .41202 -12 -4.5 ]
[.05249 .41202 0 4.5 ]
[.05249 .54936 -12 -4.5 ]
[.05249 .54936 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.15251 0 m
.15251 .00625 L
s
[(50)] .15251 -0.0125 0 1 Mshowa
.28121 0 m
.28121 .00625 L
s
[(100)] .28121 -0.0125 0 1 Mshowa
.40991 0 m
.40991 .00625 L
s
[(150)] .40991 -0.0125 0 1 Mshowa
.53861 0 m
.53861 .00625 L
s
[(200)] .53861 -0.0125 0 1 Mshowa
.66731 0 m
.66731 .00625 L
s
[(250)] .66731 -0.0125 0 1 Mshowa
.79601 0 m
.79601 .00625 L
s
[(300)] .79601 -0.0125 0 1 Mshowa
.92471 0 m
.92471 .00625 L
s
[(350)] .92471 -0.0125 0 1 Mshowa
.125 Mabswid
.04955 0 m
.04955 .00375 L
s
.07529 0 m
.07529 .00375 L
s
.10103 0 m
.10103 .00375 L
s
.12677 0 m
.12677 .00375 L
s
.17825 0 m
.17825 .00375 L
s
.20399 0 m
.20399 .00375 L
s
.22973 0 m
.22973 .00375 L
s
.25547 0 m
.25547 .00375 L
s
.30695 0 m
.30695 .00375 L
s
.33269 0 m
.33269 .00375 L
s
.35843 0 m
.35843 .00375 L
s
.38417 0 m
.38417 .00375 L
s
.43565 0 m
.43565 .00375 L
s
.46139 0 m
.46139 .00375 L
s
.48713 0 m
.48713 .00375 L
s
.51287 0 m
.51287 .00375 L
s
.56435 0 m
.56435 .00375 L
s
.59009 0 m
.59009 .00375 L
s
.61583 0 m
.61583 .00375 L
s
.64157 0 m
.64157 .00375 L
s
.69305 0 m
.69305 .00375 L
s
.71879 0 m
.71879 .00375 L
s
.74453 0 m
.74453 .00375 L
s
.77027 0 m
.77027 .00375 L
s
.82175 0 m
.82175 .00375 L
s
.84749 0 m
.84749 .00375 L
s
.87323 0 m
.87323 .00375 L
s
.89897 0 m
.89897 .00375 L
s
.95045 0 m
.95045 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.06499 .13734 m
.07124 .13734 L
s
[(20)] .05249 .13734 1 0 Mshowa
.06499 .27468 m
.07124 .27468 L
s
[(40)] .05249 .27468 1 0 Mshowa
.06499 .41202 m
.07124 .41202 L
s
[(60)] .05249 .41202 1 0 Mshowa
.06499 .54936 m
.07124 .54936 L
s
[(80)] .05249 .54936 1 0 Mshowa
.125 Mabswid
.06499 .03434 m
.06874 .03434 L
s
.06499 .06867 m
.06874 .06867 L
s
.06499 .10301 m
.06874 .10301 L
s
.06499 .17168 m
.06874 .17168 L
s
.06499 .20601 m
.06874 .20601 L
s
.06499 .24035 m
.06874 .24035 L
s
.06499 .30902 m
.06874 .30902 L
s
.06499 .34335 m
.06874 .34335 L
s
.06499 .37769 m
.06874 .37769 L
s
.06499 .44636 m
.06874 .44636 L
s
.06499 .48069 m
.06874 .48069 L
s
.06499 .51503 m
.06874 .51503 L
s
.06499 .5837 m
.06874 .5837 L
s
.25 Mabswid
.06499 0 m
.06499 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.06499 .4136 m
.10196 .42156 L
.14227 .43024 L
.18013 .4384 L
.21654 .44624 L
.2553 .45458 L
.29261 .46261 L
.33226 .47115 L
.37046 .47938 L
.40722 .48729 L
.44631 .49571 L
.48396 .50382 L
.52016 .51161 L
.55871 .51991 L
.5958 .5279 L
.63525 .53639 L
.67324 .54457 L
.70978 .55244 L
.74867 .56082 L
.7861 .56888 L
.82589 .57744 L
.86422 .5857 L
.90111 .59364 L
.94034 .60209 L
.97619 .60981 L
s
.02 w
.34041 .5219 Mdot
.30438 .48069 Mdot
.1834 .37769 Mdot
.29151 .48756 Mdot
.17053 .37769 Mdot
.97619 .32962 Mdot
.13707 .34335 Mdot
.28121 .45322 Mdot
.06499 .28155 Mdot
.09588 .29528 Mdot
.37902 .5631 Mdot
.29408 .46696 Mdot
.43308 .6043 Mdot
.21686 .39829 Mdot
.25032 .43949 Mdot
.44595 .6043 Mdot
.45882 .61117 Mdot
.45367 .6043 Mdot
.40734 .57683 Mdot
.45367 .6043 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{176.312, 108.938},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002`0000K1000`40O003h00O/7oo002`Ool000eo
o`8000moo`<000Aoo`8000Qoo`<000Aoo`8000Aoo`8000Eoo`<000=oo`<000Aoo`8000Eoo`@000=o
o`8000Aoo`8000Eoo`@0009oo`<000Aoo`8000Eoo`<000Aoo`8000Aoo`8000Ioo`<000=oo`<000Ao
o`8000eoo`0037oo00@007ooOol0015oo`04001oogoo0002Ool00`00Oomoo`06Ool00`00Oomoo`02
Ool01000Oomoo`000Woo00@007ooOol000Eoo`03001oogoo00Eoo`04001oogoo0002Ool00`00Oomo
o`03Ool00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol000Eoo`03001oogoo00Eoo`04001o
ogoo0002Ool00`00Oomoo`05Ool01000Oomoo`000Woo00@007ooOol0009oo`03001oogoo00Ioo`03
001oogoo00=oo`04001oogoo0002Ool00`00Oomoo`0:Ool000aoo`04001oogoo000AOol01000Oomo
o`000Woo00<007ooOol01Woo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo0005Ool00`00
Oomoo`05Ool01000Oomoo`000Woo00<007ooOol017oo00D007ooOomoo`00009oo`04001oogoo0002
Ool00`00Oomoo`04Ool00`00Oomoo`04Ool01000Oomoo`000Woo00<007ooOol01Goo00@007ooOol0
009oo`04001oogoo0002Ool00`00Oomoo`06Ool00`00Oomoo`03Ool01000Oomoo`000Woo00<007oo
Ool02Woo000<Ool01000Oomoo`003Woo0`000goo00@007ooOol000Qoo`03001oogoo009oo`04001o
ogoo0002Ool01000Oomoo`001Goo00<007ooOol00Woo0`000goo00@007ooOol000Moo`04001oogoo
0002Ool01000Oomoo`000Woo00<007ooOol01Goo00<007ooOol00`000goo00@007ooOol000Eoo`80
00=oo`04001oogoo0002Ool01000Oomoo`001Woo0P000goo0`000goo00@007ooOol000aoo`0037oo
00@007ooOol000ioo`03001oogoo00=oo`04001oogoo0008Ool00`00Oomoo`02Ool01000Oomoo`00
0Woo00@007ooOol000Eoo`03001oogoo009oo`03001oogoo00=oo`04001oogoo0004Ool01000Oomo
o`000Woo00@007ooOol0009oo`04001oogoo0004Ool01000Oomoo`000Woo00<007ooOol00goo00@0
07ooOol000Moo`04001oogoo0002Ool01000Oomoo`000Woo00<007ooOol01Woo00@007ooOol000Eo
o`04001oogoo000<Ool000eoo`8000moo`@000=oo`8000Qoo`8000Eoo`8000Aoo`8000Eoo`8000Ao
o`@000=oo`8000Ioo`8000Aoo`8000Aoo`8000Ioo`8000=oo`@000=oo`8000Eoo`<000Aoo`8000Ao
o`8000Ioo`<000=oo`@000=oo`8000eoo`00/7oo002`Ool00;1oo`002gooW@0027oo000>Ool00`00
Oomoo`04Ool00`00Oomoo`0;Ool00`00Oomoo`0AOol00`00Oomoo`0AOol00`00Oomoo`0AOol00`00
Oomoo`0AOol00`00Oomoo`0AOol00`00Oomoo`0BOol00`00Oomoo`0AOol001Eoo`03001oogoo09Qo
o`005Goo00<007ooOol0V7oo000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<0
07ooOol0V7oo000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo
000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo000EOol00`00
Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo000EOol00`00Oomoo`2HOol0
01Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo0008Ool40003Ool20004Ool00`00Oomoo`2H
Ool000Uoo`03001oogoo009oo`04001oogoo0003Ool00`00Oomoo`2HOol000Yoo`05001oogooOol0
0002Ool01000Oomoogoo0P00VGoo000;Ool01000Oomoo`000Woo00D007ooOomoo`0009Yoo`0027oo
00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`2HOol000Uoo`8000Aoo`8000Aoo`03001o
ogoo09Qoo`005Goo00<007ooOol0V7oo000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`00
5Goo00<007ooOol0V7oo000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007oo
Ool0V7oo000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo000E
Ool00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo000EOol00`00Oomo
o`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo0009Ool30003Ool20004Ool00`00
Oomoo`2HOol000Yoo`05001oogooOol00002Ool01@00Oomoogoo0000VWoo0008Ool40002Ool01000
Oomoo`000Woo0`00VGoo0008Ool00`00Ool00003Ool01000Oomoo`000Woo0`00VGoo0008Ool00`00
Ool00003Ool01000Oomoo`000Woo0`000Woo0`00U7oo0009Ool20004Ool20004Ool01000Oomoogoo
0`00U7oo000EOol01000Oomoogoo0`00U7oo000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qo
o`005Goo00<007ooOol0Rgoo0`002Woo000EOol00`00Oomoo`2;Ool3000:Ool001Eoo`03001oogoo
00Moo`<0085oo`<000Yoo`005Goo00<007ooOol01goo0`00SWoo000EOol00`00Oomoo`07Ool3002>
Ool001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo000EOol00`00Oomoo`2HOol001Eoo`03
001oogoo00aoo`D008Moo`005Goo00<007ooOol037oo1@00Qgoo000EOol00`00Oomoo`0<Ool50027
Ool001Eoo`03001oogoo01Aoo`<0085oo`002Goo0P0017oo0P0017oo00<007ooOol057oo0`00PGoo
0008Ool01000Oomoo`000Woo00@007ooOol000=oo`03001oogoo01Aoo`<0085oo`0027oo00@007oo
Ool0009oo`04001oogoo0003Ool3002HOol000Qoo`<000=oo`04001oogoo0003Ool00`00Oomoo`03
002EOol000Qoo`03001oogoo00=oo`04001oogoo0003Ool00`00Oomoo`03Ool4002AOol000Uoo`<0
00=oo`8000Aoo`03001oogoo00Moo`H008]oo`005Goo00<007ooOol03Goo1@001goo0`00O7oo000E
Ool00`00Oomoo`0BOol30004Ool3001lOol001Eoo`03001oogoo01Eoo`L0009oo`<007Moo`005Goo
00<007ooOol06Goo2000Mgoo000EOol00`00Oomoo`0NOol7001cOol001Eoo`03001oogoo021oo`<0
009oo`D006ioo`005Goo00<007ooOol087oo10001Woo0`00Jgoo000EOol00`00Oomoo`0OOol50009
Ool4001WOol001Eoo`03001oogoo01moo`D000eoo`H0065oo`005Goo00<007ooOol07goo0`005Goo
1@00G7oo000EOol00`00Oomoo`0lOol3001IOol001Eoo`03001oogoo03moo`@005Eoo`005Goo00<0
07ooOol09goo0`006Goo1@00D7oo000EOol00`00Oomoo`0WOol3000NOol6001:Ool001Eoo`03001o
ogoo02Moo`<002Aoo`D004Eoo`005Goo00<007ooOol0Dgoo0`00@Woo0009Ool20004Ool20004Ool0
0`00Oomoo`1FOol4000nOol000Qoo`04001oogoo0002Ool01000Oomoo`000goo00<007ooOol0FWoo
1P00>7oo0008Ool01000Oomoo`000Woo00@007ooOol000=oo`80065oo`D003=oo`002Goo0P000goo
00@007ooOol000=oo`03001oogoo02eoo`<003Eoo`<0031oo`0027oo00@007ooOol0009oo`04001o
ogoo0003Ool00`00Oomoo`0]Ool3000hOol4000/Ool000Uoo`8000Aoo`8000Aoo`03001oogoo02eo
o`<00004Ool00000000hOol6000VOol001Eoo`03001oogoo035oo`<003ioo`H0021oo`005Goo00<0
07ooOol0<Goo0`00A7oo1@006goo000EOol00`00Oomoo`1mOol3000HOol001Eoo`03001oogoo03Eo
o`L004Aoo`@001Aoo`005Goo00<007ooOol0=Goo1`00B7oo1P003Woo000EOol00`00Oomoo`0eOol8
001=Ool3000;Ool001Eoo`03001oogoo03Yoo`<005]oo`00\
\>"],
  ImageRangeCache->{{{0, 175.312}, {107.938, 0}} -> {-36.6911, -10.1734, 
  2.47594, 0.928068}}]
}, Open  ]],

Cell["The LAD approach still tracks the main trend.", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["pilotladfit = LADLPFit[rdata, x]//N", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(36.4065040650406501`\[InvisibleSpace]\) + 
      0.308943089430894346`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["plotfit[pilotladfit, rdata];", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.002574 0 0.00686704 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.15251 -0.0125 -6 -9 ]
[.15251 -0.0125 6 0 ]
[.28121 -0.0125 -9 -9 ]
[.28121 -0.0125 9 0 ]
[.40991 -0.0125 -9 -9 ]
[.40991 -0.0125 9 0 ]
[.53861 -0.0125 -9 -9 ]
[.53861 -0.0125 9 0 ]
[.66731 -0.0125 -9 -9 ]
[.66731 -0.0125 9 0 ]
[.79601 -0.0125 -9 -9 ]
[.79601 -0.0125 9 0 ]
[.92471 -0.0125 -9 -9 ]
[.92471 -0.0125 9 0 ]
[.05249 .13734 -12 -4.5 ]
[.05249 .13734 0 4.5 ]
[.05249 .27468 -12 -4.5 ]
[.05249 .27468 0 4.5 ]
[.05249 .41202 -12 -4.5 ]
[.05249 .41202 0 4.5 ]
[.05249 .54936 -12 -4.5 ]
[.05249 .54936 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(0)] .02381 -0.0125 0 1 Mshowa
.15251 0 m
.15251 .00625 L
s
[(50)] .15251 -0.0125 0 1 Mshowa
.28121 0 m
.28121 .00625 L
s
[(100)] .28121 -0.0125 0 1 Mshowa
.40991 0 m
.40991 .00625 L
s
[(150)] .40991 -0.0125 0 1 Mshowa
.53861 0 m
.53861 .00625 L
s
[(200)] .53861 -0.0125 0 1 Mshowa
.66731 0 m
.66731 .00625 L
s
[(250)] .66731 -0.0125 0 1 Mshowa
.79601 0 m
.79601 .00625 L
s
[(300)] .79601 -0.0125 0 1 Mshowa
.92471 0 m
.92471 .00625 L
s
[(350)] .92471 -0.0125 0 1 Mshowa
.125 Mabswid
.04955 0 m
.04955 .00375 L
s
.07529 0 m
.07529 .00375 L
s
.10103 0 m
.10103 .00375 L
s
.12677 0 m
.12677 .00375 L
s
.17825 0 m
.17825 .00375 L
s
.20399 0 m
.20399 .00375 L
s
.22973 0 m
.22973 .00375 L
s
.25547 0 m
.25547 .00375 L
s
.30695 0 m
.30695 .00375 L
s
.33269 0 m
.33269 .00375 L
s
.35843 0 m
.35843 .00375 L
s
.38417 0 m
.38417 .00375 L
s
.43565 0 m
.43565 .00375 L
s
.46139 0 m
.46139 .00375 L
s
.48713 0 m
.48713 .00375 L
s
.51287 0 m
.51287 .00375 L
s
.56435 0 m
.56435 .00375 L
s
.59009 0 m
.59009 .00375 L
s
.61583 0 m
.61583 .00375 L
s
.64157 0 m
.64157 .00375 L
s
.69305 0 m
.69305 .00375 L
s
.71879 0 m
.71879 .00375 L
s
.74453 0 m
.74453 .00375 L
s
.77027 0 m
.77027 .00375 L
s
.82175 0 m
.82175 .00375 L
s
.84749 0 m
.84749 .00375 L
s
.87323 0 m
.87323 .00375 L
s
.89897 0 m
.89897 .00375 L
s
.95045 0 m
.95045 .00375 L
s
.97619 0 m
.97619 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.06499 .13734 m
.07124 .13734 L
s
[(20)] .05249 .13734 1 0 Mshowa
.06499 .27468 m
.07124 .27468 L
s
[(40)] .05249 .27468 1 0 Mshowa
.06499 .41202 m
.07124 .41202 L
s
[(60)] .05249 .41202 1 0 Mshowa
.06499 .54936 m
.07124 .54936 L
s
[(80)] .05249 .54936 1 0 Mshowa
.125 Mabswid
.06499 .03434 m
.06874 .03434 L
s
.06499 .06867 m
.06874 .06867 L
s
.06499 .10301 m
.06874 .10301 L
s
.06499 .17168 m
.06874 .17168 L
s
.06499 .20601 m
.06874 .20601 L
s
.06499 .24035 m
.06874 .24035 L
s
.06499 .30902 m
.06874 .30902 L
s
.06499 .34335 m
.06874 .34335 L
s
.06499 .37769 m
.06874 .37769 L
s
.06499 .44636 m
.06874 .44636 L
s
.06499 .48069 m
.06874 .48069 L
s
.06499 .51503 m
.06874 .51503 L
s
.06499 .5837 m
.06874 .5837 L
s
.25 Mabswid
.06499 0 m
.06499 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.06499 .28395 m
.10196 .31442 L
.14227 .34764 L
.18013 .37885 L
.21654 .40886 L
.2553 .4408 L
.29261 .47155 L
.33226 .50423 L
.37046 .53572 L
.40722 .56601 L
.44631 .59824 L
s
.44631 .59824 m
.47033 .61803 L
s
.02 w
.34041 .5219 Mdot
.30438 .48069 Mdot
.1834 .37769 Mdot
.29151 .48756 Mdot
.17053 .37769 Mdot
.97619 .32962 Mdot
.13707 .34335 Mdot
.28121 .45322 Mdot
.06499 .28155 Mdot
.09588 .29528 Mdot
.37902 .5631 Mdot
.29408 .46696 Mdot
.43308 .6043 Mdot
.21686 .39829 Mdot
.25032 .43949 Mdot
.44595 .6043 Mdot
.45882 .61117 Mdot
.45367 .6043 Mdot
.40734 .57683 Mdot
.45367 .6043 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{176.312, 108.938},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002`0000K1000`40O003h00O/7oo002`Ool000eo
o`8000moo`<000Aoo`8000Qoo`<000Aoo`8000Aoo`8000Eoo`<000=oo`<000Aoo`8000Eoo`@000=o
o`8000Aoo`8000Eoo`@0009oo`<000Aoo`8000Eoo`<000Aoo`8000Aoo`8000Ioo`<000=oo`<000Ao
o`8000eoo`0037oo00@007ooOol0015oo`04001oogoo0002Ool00`00Oomoo`06Ool00`00Oomoo`02
Ool01000Oomoo`000Woo00@007ooOol000Eoo`03001oogoo00Eoo`04001oogoo0002Ool00`00Oomo
o`03Ool00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol000Eoo`03001oogoo00Eoo`04001o
ogoo0002Ool00`00Oomoo`05Ool01000Oomoo`000Woo00@007ooOol0009oo`03001oogoo00Ioo`03
001oogoo00=oo`04001oogoo0002Ool00`00Oomoo`0:Ool000aoo`04001oogoo000AOol01000Oomo
o`000Woo00<007ooOol01Woo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo0005Ool00`00
Oomoo`05Ool01000Oomoo`000Woo00<007ooOol017oo00D007ooOomoo`00009oo`04001oogoo0002
Ool00`00Oomoo`04Ool00`00Oomoo`04Ool01000Oomoo`000Woo00<007ooOol01Goo00@007ooOol0
009oo`04001oogoo0002Ool00`00Oomoo`06Ool00`00Oomoo`03Ool01000Oomoo`000Woo00<007oo
Ool02Woo000<Ool01000Oomoo`003Woo0`000goo00@007ooOol000Qoo`03001oogoo009oo`04001o
ogoo0002Ool01000Oomoo`001Goo00<007ooOol00Woo0`000goo00@007ooOol000Moo`04001oogoo
0002Ool01000Oomoo`000Woo00<007ooOol01Goo00<007ooOol00`000goo00@007ooOol000Eoo`80
00=oo`04001oogoo0002Ool01000Oomoo`001Woo0P000goo0`000goo00@007ooOol000aoo`0037oo
00@007ooOol000ioo`03001oogoo00=oo`04001oogoo0008Ool00`00Oomoo`02Ool01000Oomoo`00
0Woo00@007ooOol000Eoo`03001oogoo009oo`03001oogoo00=oo`04001oogoo0004Ool01000Oomo
o`000Woo00@007ooOol0009oo`04001oogoo0004Ool01000Oomoo`000Woo00<007ooOol00goo00@0
07ooOol000Moo`04001oogoo0002Ool01000Oomoo`000Woo00<007ooOol01Woo00@007ooOol000Eo
o`04001oogoo000<Ool000eoo`8000moo`@000=oo`8000Qoo`8000Eoo`8000Aoo`8000Eoo`8000Ao
o`@000=oo`8000Ioo`8000Aoo`8000Aoo`8000Ioo`8000=oo`@000=oo`8000Eoo`<000Aoo`8000Ao
o`8000Ioo`<000=oo`@000=oo`8000eoo`00/7oo002`Ool00;1oo`002gooW@0027oo000>Ool00`00
Oomoo`04Ool00`00Oomoo`0;Ool00`00Oomoo`0AOol00`00Oomoo`0AOol00`00Oomoo`0AOol00`00
Oomoo`0AOol00`00Oomoo`0AOol00`00Oomoo`0BOol00`00Oomoo`0AOol001Eoo`03001oogoo09Qo
o`005Goo00<007ooOol0V7oo000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<0
07ooOol0V7oo000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo
000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo000EOol00`00
Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo000EOol00`00Oomoo`2HOol0
01Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo0008Ool40003Ool20004Ool00`00Oomoo`2H
Ool000Uoo`03001oogoo009oo`04001oogoo0003Ool00`00Oomoo`2HOol000Yoo`05001oogooOol0
0002Ool01000Oomoogoo0P00VGoo000;Ool01000Oomoo`000Woo00D007ooOomoo`0009Yoo`0027oo
00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`2HOol000Uoo`8000Aoo`8000Aoo`03001o
ogoo09Qoo`005Goo00<007ooOol0V7oo000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`00
5Goo00<007ooOol0V7oo000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007oo
Ool0V7oo000EOol00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo000E
Ool00`00Oomoo`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo000EOol00`00Oomo
o`2HOol001Eoo`03001oogoo09Qoo`005Goo00<007ooOol0V7oo0009Ool30003Ool20004Ool00`00
Oomoo`2HOol000Yoo`05001oogooOol00002Ool01@00Oomoogoo0000VWoo0008Ool40002Ool01000
Oomoo`000Woo0`00VGoo0008Ool00`00Ool00003Ool01000Oomoo`000Woo0`00VGoo0008Ool00`00
Ool00003Ool01000Oomoo`000Woo100000Aoo`00000009Aoo`002Goo0P0017oo0P0017oo00<007oo
Ool01000U7oo000EOol01000Oomoogoo0`00U7oo000EOol00`00Oomoo`02Ool00`00Oomoo`2COol0
01Eoo`03001oogoo00=oo`8009=oo`005Goo00<007ooOol01Goo00<007ooOol0Pgoo0`002Woo000E
Ool00`00Oomoo`06Ool00`00Oomoo`22Ool3000:Ool001Eoo`03001oogoo00Moo`<0085oo`<000Yo
o`005Goo00<007ooOol01goo0`00SWoo000EOol00`00Oomoo`07Ool4002=Ool001Eoo`03001oogoo
00]oo`03001oogoo08Yoo`005Goo00<007ooOol037oo00<007ooOol0RGoo000EOol00`00Oomoo`0=
Ool00`00Oomoo`28Ool001Eoo`03001oogoo00aoo`D008Moo`005Goo00<007ooOol037oo1@00Qgoo
000EOol00`00Oomoo`0<Ool60026Ool001Eoo`03001oogoo019oo`03001oo`000080085oo`002Goo
0P0017oo0P0017oo00<007ooOol04goo1000PGoo0008Ool01000Oomoo`000Woo00@007ooOol000=o
o`03001oogoo01Aoo`<0085oo`0027oo00@007ooOol0009oo`04001oogoo0003Ool2000FOol20021
Ool000Qoo`<000=oo`04001oogoo0003Ool00`00Oomoo`0GOol00`00Oomoo`1nOol000Qoo`03001o
ogoo00=oo`04001oogoo0003Ool00`00Oomoo`0HOol00`00Oomoo`1mOol000Uoo`<000=oo`8000Ao
o`03001oogoo01Uoo`03001oogoo07aoo`005Goo00<007ooOol06Goo0`00O7oo000EOol00`00Oomo
o`0IOol3001lOol001Eoo`03001oogoo01Uoo`@00004Ool00000001gOol001Eoo`03001oogoo01eo
o`@007Moo`005Goo00<007ooOol07Woo1@00MGoo000EOol00`00Oomoo`0OOol4001eOol001Eoo`03
001oogoo021oo`@007Aoo`005Goo00<007ooOol07goo1@00M7oo000EOol00`00Oomoo`0OOol6001c
Ool001Eoo`03001oogoo01moo`<000=oo`03001oogoo071oo`005Goo00<007ooOol09Woo00<007oo
Ool0Kgoo000EOol00`00Oomoo`0WOol2001_Ool001Eoo`03001oogoo02Moo`<006ioo`005Goo00<0
07ooOol09goo1000KGoo000EOol00`00Oomoo`0WOol300000goo001oo`1[Ool001Eoo`03001oogoo
02aoo`03001oogoo06Uoo`002Goo0P0017oo0P0017oo00<007ooOol0;Goo0P00JGoo0008Ool01000
Oomoo`000Woo00@007ooOol000=oo`03001oogoo02moo`03001oogoo06Ioo`0027oo00@007ooOol0
009oo`04001oogoo0003Ool2000aOol00`00Oomoo`1UOol000Uoo`8000=oo`04001oogoo0003Ool0
0`00Oomoo`0]Ool300000goo001oo`1UOol000Qoo`04001oogoo0002Ool01000Oomoo`000goo00<0
07ooOol0;Goo0`000Woo0P00I7oo0009Ool20004Ool20004Ool00`00Oomoo`0]Ool300000goo0000
0002001SOol001Eoo`03001oogoo035oo`<00003Ool00000065oo`005Goo00<007ooOol0<Goo0`00
0goo00<007ooOol0GWoo000EOol00`00Oomoo`0hOol00`00Oomoo`1MOol001Eoo`03001oogoo03Eo
o`L005aoo`005Goo00<007ooOol0=Goo1`00G7oo000EOol00`00Oomoo`0eOol8001KOol001Eoo`03
001oogoo03Yoo`<005]oo`00\
\>"],
  ImageRangeCache->{{{0, 175.312}, {107.938, 0}} -> {-36.6911, -10.1734, 
  2.47594, 0.928068}}]
}, Open  ]],

Cell["\<\
This is one illustration of the power of robust methods - for \
others see Rousseeuw and Leroy (1987), who present many techniques. The \
approach given above is easily generalized to many variables (Shaw and \
Wilson, 1998).\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 17 Bibliography", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Borwein, J.M. and Dempster, M.A.H., 1989, The linear order complementarity \
problem, ",
  StyleBox["Mathmeatics of OR",
    FontSlant->"Italic"],
  ", 14, p. 534.\n\nDaniel, C. and Wood, F.S., 1971, ",
  StyleBox["Fitting Equations to Data",
    FontSlant->"Italic"],
  ", John Wiley and Sons.\n\nDemspter, M.A.H. and Hutton, J.P., 1996, Pricing \
American options by linear programming, submitted to ",
  StyleBox["Mathematical Finance",
    FontSlant->"Italic"],
  ".\n\n",
  StyleBox["Demspter, M.A.H., Richards, D.G. and Hutton, J.P., 1997, ",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["LP valuations of exotic American options exploiting structure,",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" Judge Institute, Cambridge, working paper.\n",
    FontVariations->{"CompatibilityType"->0}],
  "\nRousseeuw, P.J. and Leroy, A.M., 1987, ",
  StyleBox["Robust Regression and Outlier Detection",
    FontSlant->"Italic"],
  ", John Wiley and Sons.\n\nShaw, W.T. and Wilson, R., 1998, Robust \
regression in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " - submitted to the ",
  StyleBox["Mathematical Journal",
    FontSlant->"Italic"],
  ". \n\nWilmott, P., Dewynne, J. and Howison, S., 1993,",
  StyleBox[" Option Pricing - Mathematical Models and Computation",
    FontSlant->"Italic"],
  ", Oxford Financial Press.\n"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{"RulerBar", "EditBar"},
CellGrouping->Automatic,
WindowSize->{625, 665},
WindowMargins->{{4, Automatic}, {Automatic, 1}},
PrintingCopies->1,
PrintingStartingPageNumber->331,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"17. Linear Programming"}]]}, {Cell[ 
        TextData[ {"17. Linear Programming"}]], Inherited, Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True,
"FirstPageFace"->Right},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8^on3ohP==PTc0b85N`?N0080004/0B`000006G<AMP01
0000I00000400@4100800BL?00400A929;0000000000060X01T1T00000000000
00000000007oo`00003oo`000mP:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 243, 8, 174, "Title"],
Cell[1977, 61, 190, 4, 47, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2192, 69, 94, 1, 50, "Section"],
Cell[2289, 72, 2173, 35, 385, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4499, 112, 159, 3, 49, "Section"],
Cell[4661, 117, 149, 4, 29, "Text"],
Cell[4813, 123, 1182, 22, 160, "Input"],
Cell[5998, 147, 248, 5, 29, "Text"],
Cell[6249, 154, 127, 2, 20, "NumberedEquation"],
Cell[6379, 158, 133, 1, 20, "Text"],
Cell[6515, 161, 142, 2, 20, "NumberedEquation"],
Cell[6660, 165, 407, 7, 40, "Text"],
Cell[7070, 174, 141, 2, 20, "NumberedEquation"],
Cell[7214, 178, 544, 10, 52, "Text"],
Cell[7761, 190, 262, 5, 44, "NumberedEquation"],
Cell[8026, 197, 728, 12, 96, "Text"],
Cell[8757, 211, 236, 4, 45, "NumberedEquation"],
Cell[8996, 217, 266, 7, 29, "Text"],
Cell[9265, 226, 343, 8, 44, "NumberedEquation"],
Cell[9611, 236, 359, 8, 40, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10007, 249, 171, 5, 38, "Section"],
Cell[10181, 256, 480, 13, 40, "Text"],

Cell[CellGroupData[{
Cell[10686, 273, 115, 2, 25, "Input"],
Cell[10804, 277, 222, 3, 44, "Print"]
}, Open  ]],
Cell[11041, 283, 347, 8, 29, "Text"],

Cell[CellGroupData[{
Cell[11413, 295, 112, 2, 25, "Input"],
Cell[11528, 299, 163, 2, 32, "Print"]
}, Open  ]],
Cell[11706, 304, 123, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[11854, 309, 178, 4, 25, "Input"],
Cell[12035, 315, 78, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12150, 321, 158, 3, 25, "Input"],
Cell[12311, 326, 43, 1, 36, "Output"]
}, Open  ]],
Cell[12369, 330, 396, 12, 29, "Text",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[12802, 347, 133, 2, 38, "Section",
  PageBreakAbove->True],
Cell[12938, 351, 407, 8, 40, "Text"],
Cell[13348, 361, 411, 6, 20, "NumberedEquation"],
Cell[13762, 369, 192, 4, 20, "Text"],
Cell[13957, 375, 383, 10, 80, "Input"],
Cell[14343, 387, 193, 4, 20, "Text"],
Cell[14539, 393, 288, 8, 57, "Input"],
Cell[14830, 403, 150, 4, 20, "Text"],
Cell[14983, 409, 186, 3, 38, "Input"],
Cell[15172, 414, 188, 4, 20, "Text"],
Cell[15363, 420, 1136, 20, 137, "Input"],
Cell[16502, 442, 348, 6, 40, "Text"],
Cell[16853, 450, 355, 6, 76, "Input"],
Cell[17211, 458, 85, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[17321, 463, 132, 2, 25, "Input"],
Cell[17456, 467, 337, 9, 88, "Output"]
}, Open  ]],
Cell[17808, 479, 470, 8, 52, "Text"],
Cell[18281, 489, 535, 11, 110, "Input",
  InitializationCell->True],
Cell[18819, 502, 123, 1, 20, "Text"],
Cell[18945, 505, 486, 10, 106, "Input"],
Cell[19434, 517, 106, 1, 20, "Text"],
Cell[19543, 520, 282, 7, 52, "Input",
  InitializationCell->True],
Cell[19828, 529, 166, 4, 20, "Text"],
Cell[19997, 535, 287, 5, 50, "Input"],
Cell[20287, 542, 373, 7, 40, "Text"],
Cell[20663, 551, 749, 15, 137, "Input"],
Cell[21415, 568, 156, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[21596, 576, 628, 19, 200, "Input"],
Cell[22227, 597, 70, 1, 36, "Output"]
}, Open  ]],
Cell[22312, 601, 181, 3, 20, "Text"],
Cell[22496, 606, 398, 9, 90, "Input"],
Cell[22897, 617, 181, 3, 20, "Text"],

Cell[CellGroupData[{
Cell[23103, 624, 232, 4, 37, "Input"],
Cell[23338, 630, 9376, 347, 112, 3878, 274, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[32729, 980, 100, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[32854, 986, 334, 8, 62, "Input"],
Cell[33191, 996, 5795, 174, 192, "Output"]
}, Open  ]],
Cell[39001, 1173, 257, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[39283, 1182, 118, 1, 28, "Subsubsection"],
Cell[39404, 1185, 530, 10, 52, "Text"],
Cell[39937, 1197, 862, 17, 150, "Input"],
Cell[40802, 1216, 223, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[41050, 1225, 673, 20, 212, "Input"],
Cell[41726, 1247, 68, 1, 36, "Output"]
}, Open  ]],
Cell[41809, 1251, 220, 5, 29, "Text"],
Cell[42032, 1258, 398, 9, 90, "Input"],
Cell[42433, 1269, 202, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[42660, 1279, 232, 4, 37, "Input"],
Cell[42895, 1285, 9376, 347, 112, 3878, 274, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[52308, 1637, 334, 8, 62, "Input"],
Cell[52645, 1647, 5794, 174, 192, "Output"]
}, Open  ]],
Cell[58454, 1824, 308, 7, 29, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[58799, 1836, 120, 1, 32, "Subsection"],
Cell[58922, 1839, 1525, 28, 161, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[60496, 1873, 105, 1, 38, "Section"],
Cell[60604, 1876, 2647, 77, 172, "Text"],
Cell[63254, 1955, 644, 15, 134, "Input",
  PageBreakWithin->True],
Cell[63901, 1972, 241, 6, 29, "Text"],
Cell[64145, 1980, 516, 13, 124, "Input"],

Cell[CellGroupData[{
Cell[64686, 1997, 105, 1, 28, "Subsubsection"],
Cell[64794, 2000, 353, 8, 40, "Text"],
Cell[65150, 2010, 336, 8, 68, "Input"],
Cell[65489, 2020, 130, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[65644, 2025, 130, 2, 25, "Input"],
Cell[65777, 2029, 109, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[65923, 2036, 123, 2, 25, "Input"],
Cell[66049, 2040, 9174, 354, 116, 3909, 284, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[75238, 2397, 232, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[75495, 2407, 118, 2, 25, "Input"],
Cell[75616, 2411, 59, 1, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[75712, 2417, 104, 2, 25, "Input"],
Cell[75819, 2421, 109, 2, 36, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[75977, 2429, 147, 3, 28, "Subsubsection"],
Cell[76127, 2434, 120, 1, 20, "Text"],
Cell[76250, 2437, 123, 2, 25, "Input"],
Cell[76376, 2441, 123, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[76524, 2446, 130, 2, 25, "Input"],
Cell[76657, 2450, 110, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[76804, 2457, 123, 2, 25, "Input"],
Cell[76930, 2461, 9702, 370, 116, 4084, 296, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[86647, 2834, 118, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[86790, 2839, 135, 2, 25, "Input"],
Cell[86928, 2843, 109, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[87074, 2850, 128, 2, 25, "Input"],
Cell[87205, 2854, 9655, 361, 116, 3899, 285, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[96875, 3218, 306, 6, 36, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[97230, 3230, 99, 1, 44, "Section"],
Cell[97332, 3233, 1453, 34, 224, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

