(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    546294,      17220]*)
(*NotebookOutlinePosition[    548173,      17277]*)
(*  CellTagsIndexPosition[    548129,      17273]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  "Chapter 19. \nTree Implementation in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\nand Basic Tree Pathology"
}], "Title",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["\<\
Recursive Tree Modelling, Extracting Greeks, and Problems with \
Barriers\
\>", "Subsubtitle",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["19.1 Introduction", "Section",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The discussion in Chapter 18 established the theoretical forms of the CRR, \
JR and supersymmetric tree models that we wish to consider. We have two forms \
of each of CRR and JR, based on exact and approximate matching of the second \
moments. We shall now turn to their implementation in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", and explore some basic problems that arise. We proceed first on the \
basis of a recursive definition of the option values, based directly on the \
results of Chapter 18, using the work of Benninga ",
  StyleBox["et al ",
    FontSlant->"Italic"],
  "(1993) as a basic prototype, but extending their approach to include the \
Greeks. In Chapter 20 we shall see how these models can be optimized using \
the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " compiler, at the expense, perhaps, of a little elegance.\n\nBefore \
reading this chapter you are encouraged to review Section 2.7 of Chapter 2, \
and the sub-subsection on \"Defining a Function Recursively\", which looks at \
a prototype recursive algorithm using the Fibonacci series as a model case. \
It is important that you are aware of the syntax for getting ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to remember previously computed values.\n\nOur plan is to first look at \
the definition of European options, making the usual comparisons with the \
exact solutions to get a feel for the accuracy of the various models. Then we \
shall consider the modification necessary for American-style options and \
explore some valuations. Finally we shall look at the nasty problems which \
arise when trees are naively applied to problems involving Barriers. \n\n\
Throughout this chapter we shall be using recursive structures rather larger \
than is standard - to this end we re-set the following ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " system parameter:"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["$RecursionLimit=5000;", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "19.2 ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Tree Models 1 - CRR Forms"
}], "Section",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We proceed to define the various functions to explore the CRR form of a \
tree, following the general plan of Benninga ",
  StyleBox["et al",
    FontSlant->"Italic"],
  " (1993), but generalizing to allow non-zero continuous dividend yield, and \
also implementing the slightly more accurate form that matches the second \
moment exactly. First we have the universal functions giving the up and down \
steps for a CRR-like binomial tree, and the discounting factor, for the case \
where the second moments are matched exactly."
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
CRRupBin[n_, r_, q_, \[Sigma]_, T_] := Module[{f = Exp[-(r-q)*T/n], \

g = Exp[(r-q+\[Sigma]^2)*T/n]},
1/2*(f+g + Sqrt[(f+g)^2-4])];
CRRdownBin[n_, r_, q_, \[Sigma]_, T_] := Module[{f = Exp[-(r-q)*T/n], 
g = Exp[(r-q+\[Sigma]^2)*T/n]},
1/2*(f+g - Sqrt[(f+g)^2-4])];
Discount[n_, r_, T_] := N[Exp[-r T/n]];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
When the simplified form of the up and down step is used (cf. the \
description of the approximate model by Hull, 1996):\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
CRRupBinApp[n_, r_, q_, \[Sigma]_, T_] := Exp[\[Sigma] Sqrt[T/n]];
CRRdownBinApp[n_, r_, q_, \[Sigma]_, T_] := Exp[-\[Sigma] Sqrt[T/n]];\
\>", 
  "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["We also need the function", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(afunc[n_, \ r_, \ q_, \ T_]\  := \ N[Exp[\((r - q)\)\ T/n]]; \)\)], 
  "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Now in the implementation of the original CRR form, we only need the \
probabilities of up and down, and they are always multiplied by the discount \
factor - these are called ",
  Cell[BoxData[
      \(TraditionalForm\`P\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`Q\)]],
  ":"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Pfunc[up_,down_, a_, disc_] := N[disc*(a-down)/(up-down)];
Qfunc[up_, down_, a_, disc_] := N[disc-Pfunc[up,down,a,disc]];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Now for verification purposes, we do want to evaluate these \
functions rather more quickly than the general recursive tree model permits, \
so we define two functions appropriate only to the European case, where we \
just extract the valuation by just summing over final values:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
FastCRREuropeanOption[S_, n_, \[Sigma]_, T_, r_, q_, \
exercise_Function] :=
Module[{u = CRRupBin[n,r,q,\[Sigma],T], d = CRRdownBin[n,r,q,\[Sigma],T],
a = afunc[n,r,q,T],disc = Discount[n,r,T],P,Q},
P = Pfunc[u,d,a,disc];
Q = Qfunc[u,d,a,disc];
Sum[exercise[S u^j d^(n-j)] Binomial[n,j] P^j Q^(n-j), {j,0,n}]];\
\>", 
  "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
FastCRREuropeanOptionApp[S_, n_, \[Sigma]_, T_, r_, q_, \
exercise_Function] :=
Module[{u = CRRupBinApp[n,r,q,\[Sigma],T], d = \
CRRdownBinApp[n,r,q,\[Sigma],T],
a = afunc[n,r,q,T],disc = Discount[n,r,T],P,Q},
P = Pfunc[u,d,a,disc];
Q = Qfunc[u,d,a,disc];
Sum[exercise[S u^j d^(n-j)] Binomial[n,j] P^j Q^(n-j), {j,0,n}]];\
\>", 
  "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Next we make functions specific to Calls and Puts, for each of the \
two versions:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
FastCRREuropeanPut[S_, K_, n_, r_, q_, \[Sigma]_, T_] \
:=FastCRREuropeanOption[S,n,\[Sigma],T,r,q,Max[K-#,0]&];
FastCRREuropeanCall[S_, K_, n_, r_, q_, \[Sigma]_, T_] \
:=FastCRREuropeanOption[S,n,\[Sigma],T,r,q,Max[#-K,0]&];
FastCRREuropeanPutApp[S_, K_, n_, r_, q_, \[Sigma]_, T_] \
:=FastCRREuropeanOptionApp[S,n,\[Sigma],T,r,q,Max[K-#,0]&];
FastCRREuropeanCallApp[S_, K_, n_, r_, q_, \[Sigma]_, T_] \
:=FastCRREuropeanOptionApp[S,n,\[Sigma],T,r,q,Max[#-K,0]&];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[TextData[{
  "For the American case, the same approach is again used whatever the \
exercise function, and given the availability of information about all nodes \
in the tree, we also define those Greeks that can be computed by one call to \
the tree. Note carefully the definitions of Delta, Gamma and Theta. They are \
in fact estimates of Delta and Theta at time ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\ t\)]],
  ", and of Gamma at time ",
  Cell[BoxData[
      \(TraditionalForm\`2  \[CapitalDelta]\ t\)]],
  ". Vega and Rho can be calculated in two ways. The first is potentially \
both more accurate but numerically precarious - one changes the volatility \
and interest rate and re-computes, using a one-sided or central difference. A \
less accurate but robust method is to use the Vega-Gamma identity and the \
Rho-Delta identity for a robust estimate."
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
CRRAmericanOption[S_, n_, \[Sigma]_, T_, r_, q_, exercise_Function] \
:=
Module[{u = CRRupBin[n,r,q,\[Sigma],T], d = CRRdownBin[n,r,q,\[Sigma],T],
a = afunc[n,r,q,T],
disc = Discount[n,r,T],
P,Q,TreeOp,value,delta,gamma,theta},
P = Pfunc[u,d,a,disc];
Q = Qfunc[u,d,a,disc];
TreeOp[node_,level_] := TreeOp[node,level] = 
If[level==n,exercise[S d^node u^(level-node)],
Max[{P,Q}.{TreeOp[node,level+1],TreeOp[node+1,level+1]},
exercise[S d^node u^(level-node)]]];
value=TreeOp[0,0];
delta = (TreeOp[0,1] - TreeOp[1,1])/((u-d)*S);
gamma = ((TreeOp[0,2] - TreeOp[1,2])/(u^2-1) - 
(TreeOp[1,2]-TreeOp[2,2])/(1-d^2))*2/(S^2 (u^2-d^2));
theta = (TreeOp[1,2] - TreeOp[0,0])/(2*T/n);
Clear[TreeOp];
{value, delta, gamma, theta}];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["This can be done with the approximate weights also:", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
CRRAmericanOptionApp[S_, n_, \[Sigma]_, T_, r_, q_, \
exercise_Function] :=
Module[{u = CRRupBinApp[n,r,q,\[Sigma],T], d = \
CRRdownBinApp[n,r,q,\[Sigma],T],
a = afunc[n,r,q,T],
disc = Discount[n,r,T],
P,Q,TreeOp,value,delta,gamma,theta},
P = Pfunc[u,d,a,disc];
Q = Qfunc[u,d,a,disc];
TreeOp[node_,level_] := TreeOp[node,level] = 
If[level==n,exercise[S d^node u^(level-node)],
Max[{P,Q}.{TreeOp[node,level+1],TreeOp[node+1,level+1]},
exercise[S d^node u^(level-node)]]];
value=TreeOp[0,0];
delta = (TreeOp[0,1] - TreeOp[1,1])/((u-d)*S);
gamma = ((TreeOp[0,2] - TreeOp[1,2])/(u^2-1) - 
(TreeOp[1,2]-TreeOp[2,2])/(1-d^2))*2/(S^2 (u^2-d^2));
theta = (TreeOp[1,2] - TreeOp[0,0])/(2*T/n);
Clear[TreeOp];
{value, delta, gamma, theta}];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell["Next we give functions specific to a Call and a Put:", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
CRRAmericanPut[S_, K_, n_, r_, q_, \[Sigma]_, T_] \
:=CRRAmericanOption[S,n,\[Sigma],T,r,q,Max[K-#,0]&];
CRRAmericanCall[S_, K_, n_, r_, q_, \[Sigma]_, T_] :=CRRAmericanOption[S,n,\
\[Sigma],T,r,q,Max[#-K,0]&];
CRRAmericanPutApp[S_, K_, n_, r_, q_, \[Sigma]_, T_] \
:=CRRAmericanOptionApp[S,n,\[Sigma],T,r,q,Max[K-#,0]&];
CRRAmericanCallApp[S_, K_, n_, r_, q_, \[Sigma]_, T_] \
:=CRRAmericanOptionApp[S,n,\[Sigma],T,r,q,Max[#-K,0]&];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Evaluation and Verification of CRR-Style Models", "Subsection",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["We load the standard European options Package.", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Derivatives`BlackScholes`\>"]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \({BlackScholesCall[9, \ 10, \ 0.2, \ 0.1, \ 0, \ 1], \ 
      BlackScholesCallDelta[9, \ 10, \ 0.2, \ 0.1, \ 0, \ 1], 
      BlackScholesCallGamma[9, \ 10, \ 0.2, \ 0.1, \ 0, \ 1], \n\t
      BlackScholesCallTheta[9, \ 10, \ 0.2, \ 0.1, \ 0, \ 1]}\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.694897939192013147`, 0.529175490968577122`, 
      0.22104165095119006`, \(-0.764855622493445963`\)}\)], "Output"]
}, Open  ]],

Cell["\<\
What we shall do is to compute sample values for a Call and a Put \
in both forms of the CRR model. We shall look at a Call and a Put and will \
sample their values for trees with between 15 and 300 time-steps, and explore \
the convergence. The expiry will be fairly short, at just one year. \
\>", 
  "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(ExactData\  = \ \ Table[{k, 
          BlackScholesCall[9, \ 10, \ 0.2, \ 0.1, \ 0, \ 1]}, \ {k, \ 15, \ 
          300}]; \nExactDataP\  = \ \ Table[{k, 
          BlackScholesPut[9, \ 10, \ 0.2, \ 0.1, \ 0, \ 1]}, \ {k, \ 15, \ 
          300}]; \nexplot\  = \ 
      ListPlot[ExactData, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \n
    explotP\  = \ 
      ListPlot[ExactDataP, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
In the following we work with the \"fast\" versions, so that we can \
compute a large number of different trees.\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(CRRdataC\  = \ 
      Table[{k, 
          FastCRREuropeanCall[9, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 1]}, \ {k, \ 
          15, \ 300}]; \n
    CRRdataP\  = \ 
      Table[{k, \ 
          FastCRREuropeanPut[9, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 1]}, \ {k, \ 
          15, \ 300}]; \n
    CRRdataCApp\  = \ 
      Table[{k, \ 
          FastCRREuropeanCallApp[9, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 
            1]}, \ {k, \ 15, \ 300}]; \n
    CRRdataPApp\  = \ 
      Table[{k, \ 
          FastCRREuropeanPutApp[9, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 
            1]}, \ {k, \ 15, \ 300}]; \)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(CRRPlotC\  = \ 
      ListPlot[CRRdataC, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \n
    CRRPlotP = \ 
      ListPlot[CRRdataP, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \n
    CRRPlotCApp\  = \ 
      ListPlot[CRRdataCApp, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \n
    CRRPlotPApp = \ 
      ListPlot[CRRdataPApp, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We can now view the convergence properties. First the Call with the \
\"exact\" form of the tree rule:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[explot, \ CRRPlotC, \ \ PlotRange\  -> \ {0.69, \ 0.70}, \ 
      DisplayFunction\  -> \ $DisplayFunction]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -42.6443 61.8034 [
[.18254 .60553 -6 -9 ]
[.18254 .60553 6 0 ]
[.34127 .60553 -9 -9 ]
[.34127 .60553 9 0 ]
[.5 .60553 -9 -9 ]
[.5 .60553 9 0 ]
[.65873 .60553 -9 -9 ]
[.65873 .60553 9 0 ]
[.81746 .60553 -9 -9 ]
[.81746 .60553 9 0 ]
[.97619 .60553 -9 -9 ]
[.97619 .60553 9 0 ]
[.01131 0 -24 -4.5 ]
[.01131 0 0 4.5 ]
[.01131 .12361 -30 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -30 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -30 -4.5 ]
[.01131 .49443 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .62428 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .61803 m
.18254 .62428 L
s
[(50)] .18254 .60553 0 1 Mshowa
.34127 .61803 m
.34127 .62428 L
s
[(100)] .34127 .60553 0 1 Mshowa
.5 .61803 m
.5 .62428 L
s
[(150)] .5 .60553 0 1 Mshowa
.65873 .61803 m
.65873 .62428 L
s
[(200)] .65873 .60553 0 1 Mshowa
.81746 .61803 m
.81746 .62428 L
s
[(250)] .81746 .60553 0 1 Mshowa
.97619 .61803 m
.97619 .62428 L
s
[(300)] .97619 .60553 0 1 Mshowa
.125 Mabswid
.05556 .61803 m
.05556 .62178 L
s
.0873 .61803 m
.0873 .62178 L
s
.11905 .61803 m
.11905 .62178 L
s
.15079 .61803 m
.15079 .62178 L
s
.21429 .61803 m
.21429 .62178 L
s
.24603 .61803 m
.24603 .62178 L
s
.27778 .61803 m
.27778 .62178 L
s
.30952 .61803 m
.30952 .62178 L
s
.37302 .61803 m
.37302 .62178 L
s
.40476 .61803 m
.40476 .62178 L
s
.43651 .61803 m
.43651 .62178 L
s
.46825 .61803 m
.46825 .62178 L
s
.53175 .61803 m
.53175 .62178 L
s
.56349 .61803 m
.56349 .62178 L
s
.59524 .61803 m
.59524 .62178 L
s
.62698 .61803 m
.62698 .62178 L
s
.69048 .61803 m
.69048 .62178 L
s
.72222 .61803 m
.72222 .62178 L
s
.75397 .61803 m
.75397 .62178 L
s
.78571 .61803 m
.78571 .62178 L
s
.84921 .61803 m
.84921 .62178 L
s
.88095 .61803 m
.88095 .62178 L
s
.9127 .61803 m
.9127 .62178 L
s
.94444 .61803 m
.94444 .62178 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.02381 0 m
.03006 0 L
s
[(0.69)] .01131 0 1 0 Mshowa
.02381 .12361 m
.03006 .12361 L
s
[(0.692)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.694)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.696)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.698)] .01131 .49443 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .30271 m
.0746 .30271 L
.07778 .30271 L
.08095 .30271 L
.08413 .30271 L
.0873 .30271 L
.09048 .30271 L
.09365 .30271 L
.09683 .30271 L
.1 .30271 L
.10317 .30271 L
.10635 .30271 L
.10952 .30271 L
.1127 .30271 L
.11587 .30271 L
.11905 .30271 L
.12222 .30271 L
.1254 .30271 L
.12857 .30271 L
.13175 .30271 L
.13492 .30271 L
.1381 .30271 L
.14127 .30271 L
.14444 .30271 L
.14762 .30271 L
.15079 .30271 L
.15397 .30271 L
.15714 .30271 L
.16032 .30271 L
.16349 .30271 L
.16667 .30271 L
.16984 .30271 L
.17302 .30271 L
.17619 .30271 L
.17937 .30271 L
.18254 .30271 L
.18571 .30271 L
.18889 .30271 L
.19206 .30271 L
.19524 .30271 L
.19841 .30271 L
.20159 .30271 L
.20476 .30271 L
.20794 .30271 L
.21111 .30271 L
.21429 .30271 L
.21746 .30271 L
.22063 .30271 L
.22381 .30271 L
.22698 .30271 L
Mistroke
.23016 .30271 L
.23333 .30271 L
.23651 .30271 L
.23968 .30271 L
.24286 .30271 L
.24603 .30271 L
.24921 .30271 L
.25238 .30271 L
.25556 .30271 L
.25873 .30271 L
.2619 .30271 L
.26508 .30271 L
.26825 .30271 L
.27143 .30271 L
.2746 .30271 L
.27778 .30271 L
.28095 .30271 L
.28413 .30271 L
.2873 .30271 L
.29048 .30271 L
.29365 .30271 L
.29683 .30271 L
.3 .30271 L
.30317 .30271 L
.30635 .30271 L
.30952 .30271 L
.3127 .30271 L
.31587 .30271 L
.31905 .30271 L
.32222 .30271 L
.3254 .30271 L
.32857 .30271 L
.33175 .30271 L
.33492 .30271 L
.3381 .30271 L
.34127 .30271 L
.34444 .30271 L
.34762 .30271 L
.35079 .30271 L
.35397 .30271 L
.35714 .30271 L
.36032 .30271 L
.36349 .30271 L
.36667 .30271 L
.36984 .30271 L
.37302 .30271 L
.37619 .30271 L
.37937 .30271 L
.38254 .30271 L
.38571 .30271 L
Mistroke
.38889 .30271 L
.39206 .30271 L
.39524 .30271 L
.39841 .30271 L
.40159 .30271 L
.40476 .30271 L
.40794 .30271 L
.41111 .30271 L
.41429 .30271 L
.41746 .30271 L
.42063 .30271 L
.42381 .30271 L
.42698 .30271 L
.43016 .30271 L
.43333 .30271 L
.43651 .30271 L
.43968 .30271 L
.44286 .30271 L
.44603 .30271 L
.44921 .30271 L
.45238 .30271 L
.45556 .30271 L
.45873 .30271 L
.4619 .30271 L
.46508 .30271 L
.46825 .30271 L
.47143 .30271 L
.4746 .30271 L
.47778 .30271 L
.48095 .30271 L
.48413 .30271 L
.4873 .30271 L
.49048 .30271 L
.49365 .30271 L
.49683 .30271 L
.5 .30271 L
.50317 .30271 L
.50635 .30271 L
.50952 .30271 L
.5127 .30271 L
.51587 .30271 L
.51905 .30271 L
.52222 .30271 L
.5254 .30271 L
.52857 .30271 L
.53175 .30271 L
.53492 .30271 L
.5381 .30271 L
.54127 .30271 L
.54444 .30271 L
Mistroke
.54762 .30271 L
.55079 .30271 L
.55397 .30271 L
.55714 .30271 L
.56032 .30271 L
.56349 .30271 L
.56667 .30271 L
.56984 .30271 L
.57302 .30271 L
.57619 .30271 L
.57937 .30271 L
.58254 .30271 L
.58571 .30271 L
.58889 .30271 L
.59206 .30271 L
.59524 .30271 L
.59841 .30271 L
.60159 .30271 L
.60476 .30271 L
.60794 .30271 L
.61111 .30271 L
.61429 .30271 L
.61746 .30271 L
.62063 .30271 L
.62381 .30271 L
.62698 .30271 L
.63016 .30271 L
.63333 .30271 L
.63651 .30271 L
.63968 .30271 L
.64286 .30271 L
.64603 .30271 L
.64921 .30271 L
.65238 .30271 L
.65556 .30271 L
.65873 .30271 L
.6619 .30271 L
.66508 .30271 L
.66825 .30271 L
.67143 .30271 L
.6746 .30271 L
.67778 .30271 L
.68095 .30271 L
.68413 .30271 L
.6873 .30271 L
.69048 .30271 L
.69365 .30271 L
.69683 .30271 L
.7 .30271 L
.70317 .30271 L
Mistroke
.70635 .30271 L
.70952 .30271 L
.7127 .30271 L
.71587 .30271 L
.71905 .30271 L
.72222 .30271 L
.7254 .30271 L
.72857 .30271 L
.73175 .30271 L
.73492 .30271 L
.7381 .30271 L
.74127 .30271 L
.74444 .30271 L
.74762 .30271 L
.75079 .30271 L
.75397 .30271 L
.75714 .30271 L
.76032 .30271 L
.76349 .30271 L
.76667 .30271 L
.76984 .30271 L
.77302 .30271 L
.77619 .30271 L
.77937 .30271 L
.78254 .30271 L
.78571 .30271 L
.78889 .30271 L
.79206 .30271 L
.79524 .30271 L
.79841 .30271 L
.80159 .30271 L
.80476 .30271 L
.80794 .30271 L
.81111 .30271 L
.81429 .30271 L
.81746 .30271 L
.82063 .30271 L
.82381 .30271 L
.82698 .30271 L
.83016 .30271 L
.83333 .30271 L
.83651 .30271 L
.83968 .30271 L
.84286 .30271 L
.84603 .30271 L
.84921 .30271 L
.85238 .30271 L
.85556 .30271 L
.85873 .30271 L
.8619 .30271 L
Mistroke
.86508 .30271 L
.86825 .30271 L
.87143 .30271 L
.8746 .30271 L
.87778 .30271 L
.88095 .30271 L
.88413 .30271 L
.8873 .30271 L
.89048 .30271 L
.89365 .30271 L
.89683 .30271 L
.9 .30271 L
.90317 .30271 L
.90635 .30271 L
.90952 .30271 L
.9127 .30271 L
.91587 .30271 L
.91905 .30271 L
.92222 .30271 L
.9254 .30271 L
.92857 .30271 L
.93175 .30271 L
.93492 .30271 L
.9381 .30271 L
.94127 .30271 L
.94444 .30271 L
.94762 .30271 L
.95079 .30271 L
.95397 .30271 L
.95714 .30271 L
.96032 .30271 L
.96349 .30271 L
.96667 .30271 L
.96984 .30271 L
.97302 .30271 L
.97619 .30271 L
Mfstroke
.07295 .61803 m
.07452 0 L
s
.07469 0 m
.07647 .61803 L
s
.0796 .61803 m
.08095 .28053 L
s
.08095 .28053 m
.0826 .61803 L
s
.08634 .61803 m
.0873 .48207 L
s
.0873 .48207 m
.08873 .61803 L
s
.0935 .61803 m
.09365 .60984 L
s
.09365 .60984 m
.09448 .61803 L
s
.1012 .61803 m
.10317 .50473 L
s
.10317 .50473 m
.10479 .61803 L
s
.10734 .61803 m
.10952 .3746 L
s
.10952 .3746 m
.11162 .61803 L
s
.11351 .61803 m
.11587 .251 L
s
.11587 .251 m
.11825 .61803 L
s
.11969 .61803 m
.12222 .1338 L
s
.12222 .1338 m
.12482 .61803 L
s
.1259 .61803 m
.12857 .04846 L
s
.12857 .04846 m
.13135 .61803 L
s
.13223 .61803 m
.13492 .17192 L
s
.13492 .17192 m
.13779 .61803 L
s
.13847 .61803 m
.14127 .26979 L
s
.14127 .26979 m
.14437 .61803 L
s
.14454 .61803 m
.14762 .34689 L
.15079 .58445 L
.15397 .40699 L
.15714 .53946 L
.16032 .45309 L
.16349 .49262 L
.16667 .4876 L
.16984 .44457 L
.17302 .51247 L
.17619 .39582 L
.17937 .52927 L
.18254 .34672 L
.18571 .53931 L
.18889 .29758 L
.19206 .54366 L
.19524 .24863 L
.19841 .5432 L
.20159 .20003 L
.20476 .53868 L
.20794 .15191 L
.21111 .53072 L
.21429 .20511 L
.21746 .51985 L
.22063 .25171 L
.22381 .50651 L
.22698 .29167 L
.23016 .49107 L
.23333 .32574 L
.23651 .47385 L
.23968 .35459 L
.24286 .45514 L
.24603 .3788 L
.24921 .43516 L
.25238 .39887 L
.25556 .41411 L
.25873 .41526 L
.2619 .39218 L
.26508 .42834 L
.26825 .3695 L
.27143 .43848 L
.2746 .34621 L
.27778 .44596 L
.28095 .32242 L
.28413 .45107 L
.2873 .29823 L
.29048 .45404 L
.29365 .27373 L
.29683 .45508 L
.3 .24898 L
Mistroke
.30317 .4544 L
.30635 .22405 L
.30952 .45216 L
.3127 .21388 L
.31587 .44851 L
.31905 .24086 L
.32222 .4436 L
.3254 .26507 L
.32857 .43754 L
.33175 .28673 L
.33492 .43045 L
.3381 .30604 L
.34127 .42243 L
.34444 .32317 L
.34762 .41357 L
.35079 .33829 L
.35397 .40395 L
.35714 .35156 L
.36032 .39364 L
.36349 .36311 L
.36667 .38272 L
.36984 .37307 L
.37302 .37125 L
.37619 .38155 L
.37937 .35928 L
.38254 .38867 L
.38571 .34686 L
.38889 .39453 L
.39206 .33403 L
.39524 .3992 L
.39841 .32085 L
.40159 .40278 L
.40476 .30734 L
.40794 .40535 L
.41111 .29355 L
.41429 .40697 L
.41746 .27951 L
.42063 .40771 L
.42381 .26523 L
.42698 .40763 L
.43016 .25076 L
.43333 .40679 L
.43651 .23612 L
.43968 .40524 L
.44286 .25092 L
.44603 .40303 L
.44921 .26579 L
.45238 .4002 L
.45556 .27945 L
.45873 .3968 L
Mistroke
.4619 .29198 L
.46508 .39286 L
.46825 .30342 L
.47143 .38843 L
.4746 .31385 L
.47778 .38353 L
.48095 .32332 L
.48413 .37819 L
.4873 .33188 L
.49048 .37246 L
.49365 .33958 L
.49683 .36634 L
.5 .34647 L
.50317 .35988 L
.50635 .3526 L
.50952 .35309 L
.5127 .358 L
.51587 .34599 L
.51905 .3627 L
.52222 .33861 L
.5254 .36676 L
.52857 .33097 L
.53175 .3702 L
.53492 .32308 L
.5381 .37305 L
.54127 .31496 L
.54444 .37535 L
.54762 .30663 L
.55079 .37712 L
.55397 .29811 L
.55714 .3784 L
.56032 .2894 L
.56349 .3792 L
.56667 .28052 L
.56984 .37954 L
.57302 .27148 L
.57619 .37947 L
.57937 .2623 L
.58254 .37899 L
.58571 .25369 L
.58889 .37812 L
.59206 .2638 L
.59524 .37689 L
.59841 .27326 L
.60159 .37531 L
.60476 .28209 L
.60794 .3734 L
.61111 .29033 L
.61429 .37118 L
.61746 .29799 L
Mistroke
.62063 .36867 L
.62381 .30511 L
.62698 .36587 L
.63016 .3117 L
.63333 .3628 L
.63651 .31779 L
.63968 .35948 L
.64286 .3234 L
.64603 .35591 L
.64921 .32854 L
.65238 .35212 L
.65556 .33324 L
.65873 .3481 L
.6619 .33752 L
.66508 .34388 L
.66825 .34139 L
.67143 .33946 L
.6746 .34487 L
.67778 .33486 L
.68095 .34797 L
.68413 .33007 L
.6873 .35071 L
.69048 .32512 L
.69365 .35311 L
.69683 .32001 L
.7 .35518 L
.70317 .31474 L
.70635 .35692 L
.70952 .30933 L
.7127 .35837 L
.71587 .30378 L
.71905 .35951 L
.72222 .29811 L
.7254 .36038 L
.72857 .2923 L
.73175 .36097 L
.73492 .28638 L
.7381 .3613 L
.74127 .28035 L
.74444 .36139 L
.74762 .27422 L
.75079 .36123 L
.75397 .26798 L
.75714 .36084 L
.76032 .2684 L
.76349 .36022 L
.76667 .275 L
.76984 .3594 L
.77302 .28123 L
.77619 .35837 L
Mistroke
.77937 .28712 L
.78254 .35714 L
.78571 .29268 L
.78889 .35572 L
.79206 .2979 L
.79524 .35411 L
.79841 .30282 L
.80159 .35234 L
.80476 .30743 L
.80794 .35039 L
.81111 .31175 L
.81429 .34828 L
.81746 .31578 L
.82063 .34601 L
.82381 .31954 L
.82698 .34359 L
.83016 .32303 L
.83333 .34103 L
.83651 .32626 L
.83968 .33833 L
.84286 .32925 L
.84603 .3355 L
.84921 .332 L
.85238 .33253 L
.85556 .33451 L
.85873 .32945 L
.8619 .33679 L
.86508 .32624 L
.86825 .33886 L
.87143 .32292 L
.8746 .34072 L
.87778 .31949 L
.88095 .34237 L
.88413 .31595 L
.8873 .34382 L
.89048 .31231 L
.89365 .34508 L
.89683 .30857 L
.9 .34615 L
.90317 .30474 L
.90635 .34704 L
.90952 .30082 L
.9127 .34776 L
.91587 .29681 L
.91905 .34831 L
.92222 .29272 L
.9254 .34869 L
.92857 .28855 L
.93175 .34891 L
.93492 .2843 L
Mistroke
.9381 .34898 L
.94127 .27998 L
.94444 .3489 L
.94762 .27558 L
.95079 .34868 L
.95397 .27468 L
.95714 .34831 L
.96032 .27935 L
.96349 .34781 L
.96667 .28381 L
.96984 .34718 L
.97302 .28805 L
.97619 .34642 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00OogooKWoo003oOom^
Ool000Qoo`8000Aoo`8000Aoo`8000=oo`<00?mooe9oo`001goo00@007ooOol000=oo`8000=oo`04
001oogoo0005Ool00`00Oomoo`3oOom?Ool000Moo`04001oogoo0008Ool01000Oomoo`000goo0`00
1Goo0`003Woo00<007ooOol0ogoo>7oo0007Ool01000Oomoo`0027oo0`000goo00@007ooOol000Eo
o`03001oogoo00ioo`03001oogoo0?moocQoo`001goo00@007ooOol000Qoo`03001oogoo00=oo`04
001oogoo0005Ool00`00Oomoo`0>Ool00`00Oomoo`3oOolhOol000Qoo`8000Yoo`<000=oo`8000Io
o`03001oogoo00ioo`03001oogoo0?moocQoo`008Woo00<007ooOol03Woo00<007ooOol0ogoo>7oo
000ROol2000?Ool00`00Oomoo`3oOolhOol0029oo`03001oogoo00ioo`03001oogoo0?moocQoo`00
8Woo00<007ooOol03Woo00<007ooOol0ogoo>7oo000ROol00`00Oomoo`0>Ool00`00Oomoo`0?Ool0
0`00Oomoo`3oOolVOol0029oo`03001oogoo00ioo`03001oogoo00moo`03001oogoo0?moobIoo`00
8Woo0P003goo00<007ooOol03goo00<007ooOol0ogoo9Woo000ROol00`00Oomoo`0>Ool00`00Oomo
o`0?Ool00`00Oomoo`3oOolVOol0029oo`03001oogoo00ioo`03001oogoo00moo`03001oogoo0?mo
obIoo`008Woo00<007ooOol03Woo00<007ooOol03goo00<007ooOol0ogoo9Woo000ROol00`00Oomo
o`0>Ool00`00Oomoo`0?Ool00`00Oomoo`3oOolVOol0029oo`8000moo`03001oogoo00moo`03001o
ogoo0?moobIoo`008Woo00<007ooOol03Woo00<007ooOol03goo00<007ooOol0ogoo9Woo000ROol0
0`00Oomoo`0>Ool00`00Oomoo`0?Ool00`00Oomoo`3oOolVOol0009oo`8000Aoo`8000Aoo`8000=o
o`<000=oo`@000Eoo`03001oogoo00ioo`03001oogoo00moo`03001oogoo0?moobIoo`0000Eoo`00
Oomoo`0000=oo`8000=oo`04001oogoo0005Ool01@00Oomoogoo00001goo00<007ooOol03Woo00<0
07ooOol03goo00<007ooOol0ogoo9Woo00001Goo001oogoo000027oo00@007ooOol000=oo`<000Ao
o`03001oogoo00Aoo`<000ioo`03001oogoo00moo`03001oogoo0?moobIoo`0000Eoo`00Oomoo`00
00Qoo`<000=oo`04001oogoo0005Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`0?Ool0
0`00Oomoo`3oOolVOol00005Ool007ooOol00008Ool00`00Oomoo`03Ool01000Oomoo`000Woo00@0
07ooOol000Eoo`03001oogoo00ioo`03001oogoo00eoo`03001oo`000?moobQoo`000Woo0P002Woo
0`000goo0P0017oo0P001Woo00<007ooOol03Woo00<007ooOol03Goo00<007oo0000ogoo:7oo000R
Ool00`00Oomoo`0>Ool00`00Oomoo`0=Ool00`00Ool0003oOolXOol0029oo`8000moo`03001oogoo
00eoo`03001oo`0001Yoo`03001oogoo0?moo`]oo`008Woo00<007ooOol03Woo00<007ooOol03Goo
00<007oo00006Woo00<007ooOol0ogoo2goo000ROol00`00Oomoo`0>Ool00`00Oomoo`0=Ool00`00
Ool0000JOol00`00Oomoo`3oOol;Ool0029oo`03001oogoo00ioo`03001oogoo00eoo`05001oo`00
Ool0000HOol00`00Oomoo`3oOol;Ool0029oo`03001oogoo00ioo`03001oogoo00eoo`05001oo`00
Ool0000HOol00`00Oomoo`3oOol;Ool0029oo`8000moo`03001oogoo00eoo`05001oo`00Ool0000H
Ool00`00Oomoo`3oOol;Ool0029oo`03001oogoo00ioo`03001oogoo00eoo`05001oo`00Ool0000H
Ool00`00Oomoo`3oOol;Ool0029oo`03001oogoo00ioo`03001oogoo00eoo`05001oo`00Ool0000F
Ool00`00Ool0003oOol=Ool0029oo`03001oogoo00ioo`03001oogoo00eoo`05001oo`00Ool0000F
Ool01@00Ool007oo0000ogoo2goo000ROol00`00Oomoo`0>Ool00`00Oomoo`0=Ool01@00Ool007oo
00005Woo00D007oo001oo`00021oo`03001oogoo0>Moo`008Woo0P003goo00<007ooOol03Goo00D0
07oo001oo`0001Ioo`05001oo`00Ool0000POol00`00Oomoo`3WOol0029oo`03001oogoo00ioo`03
001oogoo00eoo`05001oo`00Ool0000FOol01@00Ool007oo00007Woo00<007oo0000jGoo000ROol0
0`00Oomoo`0>Ool00`00Oomoo`0=Ool01@00Ool007oo00005Woo00D007oo001oo`0001ioo`03001o
o`000>Uoo`000Woo0P0017oo0P0017oo0P000goo0`0017oo0`001Goo00<007ooOol03Woo00<007oo
Ool03Goo00D007oo001oo`0001Ioo`05001oo`00Ool0000NOol00`00Ool0000YOol00`00Oomoo`2m
Ool00005Ool007ooOol00003Ool20003Ool01000Oomoo`001Goo00<007ooOol00Woo00<007ooOol0
17oo00<007ooOol03Woo00<007ooOol03Goo00D007oo001oo`0001Ioo`05001oo`00Ool0000NOol0
1@00Ool007oo00009goo00<007ooOol0_Goo00001Goo001oogoo000027oo00@007ooOol000=oo`<0
009oo`@000Eoo`<000ioo`03001oogoo00]oo`07001oo`00Ool007oo000057oo00T007oo001oo`00
Ool007oo00006Woo00L007oo001oo`00Ool0000UOol01@00Ool007oo0000_Goo00001Goo001oogoo
000027oo0`000goo00@007ooOol0009oo`03001oo`0000Ioo`03001oogoo00ioo`03001oogoo00]o
o`07001oo`00Ool007oo000057oo00T007oo001oo`00Ool007oo00006Woo00L007oo001oo`00Ool0
000UOol01@00Ool007oo0000;goo00<007ooOol0Rgoo00001Goo001oogoo000027oo00<007ooOol0
0goo00@007ooOol0009oo`03001oo`0000Ioo`03001oogoo00ioo`03001oogoo00]oo`07001oo`00
Ool007oo000057oo00T007oo001oo`00Ool007oo00006Woo00T007oo001oo`00Ool007oo00008goo
00D007oo001oo`0002eoo`05001oo`00Ool0002;Ool0009oo`8000Yoo`<000=oo`8000Aoo`8000Io
o`03001oogoo00ioo`03001oogoo00]oo`09001oo`00Ool007oo001oo`00019oo`09001oo`00Ool0
07oo001oo`0001Yoo`09001oo`00Ool007oo001oo`00025oo`09001oo`00Ool007oo001oo`0002]o
o`05001oo`00Ool0000fOol00`00Ool0001BOol0029oo`03001oogoo00ioo`03001oogoo00]oo`09
001oo`00Ool007oo001oo`00019oo`09001oo`00Ool007oo001oo`0001Qoo`0;001oo`00Ool007oo
001oo`00Ool0000QOol02@00Ool007oo001oo`00Ool0000YOol01`00Ool007oo001oo`00009oo`03
001oogoo02moo`07001oo`00Ool007oo0000?7oo00@007ooOol0011oo`008Woo0P003goo00<007oo
00002goo00T007oo001oo`00Ool007oo00004Woo00T007oo001oo`00Ool007oo000067oo00/007oo
001oo`00Ool007oo001oo`0001moo`0=001oo`00Ool007oo001oo`00Ool007oo00009Goo00T007oo
001oo`00Ool007oo00000Woo00<007oo0000;Goo00/007oo001oo`00Ool007oo001oo`0003Qoo`03
001oo`00009oo`05001oo`00Ool0000<Ool0029oo`03001oogoo00ioo`03001oo`0000]oo`09001o
o`00Ool007oo001oo`00019oo`09001oo`00Ool007oo001oo`0001Qoo`0;001oo`00Ool007oo001o
o`00Ool00002Ool00`00Oomoo`0JOol03@00Ool007oo001oo`00Ool007oo001oo`0002=oo`0;001o
o`00Ool007oo001oo`00Ool00002Ool00`00Ool0000[Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`03001oogoo02moo`07001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0000:Ool0
029oo`03001oogoo00ioo`03001oo`0000]oo`09001oo`00Ool007oo001oo`00019oo`0;001oo`00
Ool007oo001oo`00Ool0000FOol02`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol05goo
010007ooOol007oo001oo`00Ool007oo001oo`00Ool0009oo`03001oogoo01ioo`0;001oo`00Ool0
07oo001oo`00Ool00002Ool01@00Ool007oo00009Woo010007ooOol007oo001oo`00Ool007oo001o
o`00Ool0009oo`03001oo`0002eoo`09001oo`00Ool007oo001oo`00009oo`07001oo`00Ool007oo
00002Woo000ROol00`00Oomoo`0=Oooo000c0009Ool0029oo`8000moo`03001oo`0000]oo`09001o
o`00Ool007oo001oo`0000moo`0>001oogoo001oo`00Ool007oo001oo`00Ool001Aoo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00<007oo00005Goo00<007oo00000Woo00d007oo001oo`00
Ool007oo001oo`00Ool00002Ool00`00Ool0000IOol04@00Oomoo`00Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000Qoo`00Ool007oo001oo`007Woo00D007oo001oo`00009oo`0<001oo`00Ool0
07oo001oo`00Ool007oo0P0000Yoo`00Ool007oo001oo`00Ool001ioo`03001oo`00009oo`0>001o
o`00Ool007oo001oo`00Ool007oo001oo`800006Ool007oo001oo`002Woo000ROol00`00Oomoo`0=
Ool200000goo001oo`0:Ool02@00Ool007oo001oo`00Ool0000?Ool03P00Oomoo`00Ool007oo001o
o`00Ool007oo000DOol03000Ool007oo001oo`00Ool007oo001oo`800004Ool007oo000EOol00`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0000EOol00`00
Ool00002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool200002Woo001oo`00Ool007oo001o
o`0067oo00T007oo001oo`00Ool007oo00000Woo00`007oo001oo`00Ool007oo001oo`00Ool20000
37oo001oo`00Ool007oo001oo`00Ool001Yoo`05001oo`00Ool00002Ool03P00Ool007oo001oo`00
Ool007oo001oo`00Ool200001Woo001oo`00Ool000Yoo`008Woo00<007ooOol03Goo0P0000=oo`00
Ool02Woo00T007oo001oo`00Ool007oo00003goo00h007ooOol007oo001oo`00Ool007oo001oo`00
57oo00`007oo001oo`00Ool007oo001oo`00Ool2000017oo001oo`004goo00H007oo001oo`00Ool2
000037oo001oo`00Ool007oo001oo`00Ool0009oo`05001oo`00Ool0000EOol00`00Ool00002Ool0
2P00Ool007oo001oo`00Ool007oo0`0000moo`00001oo`00Ool007oo001oo`00Ool007oo000057oo
00X007oo001oo`00Ool007oo001oo`800005Ool007oo001oo`0200000goo001oo`0500003Woo001o
o`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`0?Ool02P00Ool007oo001oo`00Ool007oo
0P0000Moo`00Ool007oo001oo`0200000goo001oo`0200001Goo001oo`00Ool00`002Goo000ROol0
0`00Oomoo`0=Ool200000goo001oo`0:Ool02@00Ool007oo001oo`00Ool0000?Ool04000Oomoo`00
Ool007oo001oo`00Ool007oo001oo`003goo00l007ooOol007oo001oo`00Ool007oo001oo`00Ool0
0P0000Ioo`00Ool007oo000AOol01P00Ool007oo001oo`800003Ool007oo00800008Ool00000Ool0
07oo001oo`800006Ool007oo001oo`004Goo00H007oo001oo`00Ool200001Goo001oo`00Ool00P00
00Eoo`00Ool007oo00<0000<Ool007oo001oo`00Ool007oo001oo`000Woo00<007ooOol03Goo00`0
07oo001oo`00Ool007oo001oo`00Ool200003Woo000007oo001oo`00Ool007oo001oo`00Ool20000
3Goo000007oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool00009Ool03P00Ool007oo001o
o`00Ool007oo001oo`00Ool2000037oo000007oo001oo`00Ool007oo001oo`800008Ool007oo001o
o`00Ool000Uoo`008Woo00<007ooOol03Goo0P0000=oo`00Ool02Woo00T007oo001oo`00Ool007oo
00003goo010007oo000007oo001oo`00Ool007oo001oo`00Ool000moo`0?001oogoo001oo`00Ool0
07oo001oo`00Ool007oo00800006Ool007oo001oo`003goo00P007oo001oo`00Ool007oo0P0000Qo
o`00001oo`00Ool007oo0P0000=oo`00Ool00P0000Qoo`00Ool007oo001oo`003Goo00P007oo001o
o`00Ool007oo0P0000=oo`00Ool00P0000Moo`00Ool007oo001oo`0200000goo001oo`02000027oo
001oo`00Ool007oo0002Ool01@00Ool007oo000027oo00/007ooOol007oo001oo`00Ool007oo00@0
000@Ool00000Ool007oo001oo`00Ool007oo001oo`00Ool200002Goo001oo`00Ool007oo001oo`02
000037oo000007oo001oo`00Ool007oo001oo`800003Ool007oo00800005Ool007oo001oo`020000
1Goo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02000027oo001oo`00Ool0
07oo0009Ool0029oo`8000ioo`800003Ool007oo00Yoo`05001oo`00001oo`0200000goo001oo`0>
Ool04000Ool00000Ool007oo001oo`00Ool007oo001oo`003goo00T007ooOol007oo001oo`00Ool0
0P0000Aoo`00001oo`800008Ool007oo001oo`00Ool000eoo`08001oo`00Ool007oo001oo`80000:
Ool00000Ool007oo001oo`00Ool200003Goo000007oo001oo`00Ool007oo001oo`0000Uoo`0:001o
o`00Ool007oo001oo`00Ool300002goo001oo`00Ool007oo001oo`00Ool00P0000Moo`00Ool007oo
001oo`0400000goo001oo`0200001Woo001oo`00Ool0009oo`05001oo`00Ool00002Ool400002Goo
001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000Qoo`00Ool007oo001oo`000Woo00<007oo00000Woo00`0
07oo001oo`00Ool007oo001oo`00Ool200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool0
1`00Ool007oo001oo`0000Uoo`008Woo00<007ooOol03Goo0P0000=oo`00Ool02Woo00D007oo0000
07oo00800004Ool007oo000;Ool01000Ool007oo0P0000aoo`00Ool007oo001oo`00Ool007oo000=
Ool00`00Ool00002Ool01000Ool007oo0P0000=oo`00Ool00P0000]oo`00001oo`00Ool007oo001o
o`0000]oo`0:001oo`00Ool007oo001oo`00Ool300002goo001oo`00Ool007oo001oo`00Ool00`00
00aoo`00Ool007oo001oo`00Ool007oo0005Ool02000Ool007oo001oo`00Ool2000047oo000007oo
001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
1Goo00000000Ool0100000Eoo`00000007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`05001oo`00Ool0000=Ool02`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00
Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`0000Uoo`008Woo00<007ooOol03Goo0P00
00=oo`00Ool02Woo00D007oo000007oo00800004Ool007oo000;Ool01000Ool007oo0P0000Aoo`00
001oo`800006Ool007oo001oo`003Goo00<007oo00000Woo00@007oo001oo`800005Ool007oo001o
o`0300002Woo001oo`00Ool007oo001oo`002Goo01P007oo001oo`00Ool007oo001oo`00Ool007oo
001oo`00Ool007oo001oo`00Ool200000goo0000000400001Woo001oo`00Ool0009oo`800005Ool0
07oo001oo`0400000goo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool0
07oo001oo`00Ool007oo001oo`00Ool00P0000Ioo`00Ool007oo0006Ool04000Oomoo`00Ool007oo
001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`0001ioo`0@001oogoo001oo`00Ool007oo001oo`00Ool007oo0002
Ool01@00Ool007oo00002goo0002Ool20004Ool20004Ool20003Ool30004Ool20006Ool00`00Oomo
o`0=Ool200000goo001oo`0:Ool01@00Ool00000Ool00P0000Aoo`00Ool000]oo`04001oo`00Ool2
000017oo000007oo0P0000Ioo`00Ool007oo0002Ool00`00Oomoo`08Ool00`00Ool00002Ool06000
Ool00000Ool007oo001oo`00Ool007oo001oo`00Ool007oo001oo`00Ool000Moo`06001oo`00Ool0
07oo1000011oo`00001oo`00Ool007oo001oo`00Ool007oo001oo`800009Ool007oo001oo`00Ool0
07oo00H0009oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000ioo`00Ool007oo001oo`00Ool0
07oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`0?Ool02@00
Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`07001oo`00
Ool007oo0000AWoo00001Goo001oogoo00000goo0P000goo00@007ooOol000Eoo`04001oogoo0002
Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool200000goo001oo`0:Ool01@00Ool00000Ool00P0000Ao
o`00Ool000]oo`04001oo`00Ool2000017oo000007oo0P0000Ioo`00Ool007oo0002Ool00`00Oomo
o`08Ool01`00Ool00000Ool007oo0080000;Ool007oo001oo`00Ool007oo001oo`03000027oo001o
o`00Ool007oo0005Ool01000Ool007oo0P0000Eoo`00Ool007oo0080000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`001Goo00/007oo001oo`00
Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00/007oo001oo`00
Ool007oo001oo`0001ioo`0>001oogoo001oo`00Ool007oo001oo`00Ool0055oo`0000Eoo`00Oomo
o`0000Qoo`04001oogoo0003Ool30002Ool01000Oomoo`001Goo0`003Goo0P0000=oo`00Ool02Woo
00D007oo000007oo00800004Ool007oo000;Ool01000Ool007oo0P0000Aoo`00001oo`@00004Ool0
07oo0002Ool00`00Oomoo`06Ool01000Ool007oo0P0000ioo`00001oo`00Ool007oo001oo`00Ool0
07oo0P0000=oo`0000000P0000Ioo`00Ool007oo0002Ool200000goo0000000200001goo001oo`00
Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000aoo`00Ool007oo001oo`00
Ool007oo0009Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`07001oo`00Ool007oo0000PGoo00001Goo001oogoo000027oo0`000goo00@007ooOol0
009oo`<000Ioo`03001oogoo00eoo`800003Ool007oo00Qoo`04001oo`00Ool200001goo000007oo
001oo`0000]oo`04001oo`00Ool300001Goo001oo`00Ool00P0000Aoo`00Ool0009oo`03001oogoo
00Ioo`04001oo`00Ool200003Woo000007oo001oo`00Ool007oo001oo`00Ool200001Goo001oo`00
Ool0200000ioo`00001oo`00Ool007oo001oo`00Ool007oo0P0000ioo`00Ool007oo001oo`00Ool0
07oo001oo`000Woo00T007oo001oo`00Ool007oo00003goo00D007oo001oo`00009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00<007oo0000QGoo00001Goo001oogoo000027oo00<007oo
Ool00goo00@007ooOol0009oo`03001oogoo00Ioo`03001oogoo00eoo`800003Ool007oo00Qoo`07
001oo`00001oo`00Ool00P0000Aoo`00Ool000]oo`04001oo`00Ool300001Goo001oo`00Ool01000
00Moo`00Oomoo`00Ool00006Ool01000Ool007oo0`0000eoo`00Ool007oo001oo`00Ool007oo001o
o`02000037oo001oo`00Ool007oo001oo`00Ool000=oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo000077oo00L007oo
001oo`00Ool0002>Ool0009oo`8000Yoo`<000=oo`8000Aoo`<000Eoo`03001oogoo00eoo`800003
Ool007oo00Qoo`07001oo`00001oo`00Ool00P0000Aoo`00Ool000]oo`04001oo`00Ool300001goo
001oo`00Ool007oo00800007Ool007ooOol007oo00001Woo00D007oo000007oo0080000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000Yoo`00Ool007oo001oo`00Ool000Moo`0;001oo`00Ool0
07oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oo`000;=o
o`008Woo00<007ooOol03Goo0P0000=oo`00Ool027oo00L007oo000007oo001oo`02000017oo001o
o`002Goo00H007oo001oo`00Ool300001goo001oo`00Ool007oo00@00005Ool00000Ool00004Ool0
0`00Ool00003000047oo000007oo001oo`00Ool007oo001oo`00Ool007oo0P0000Yoo`00Ool007oo
001oo`00Ool000Uoo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00<007ooOol0/goo000ROol2000>Ool200000goo001oo`08Ool02@00Ool00000Ool0
07oo001oo`0200000goo001oo`08Ool04000Ool007oo001oo`00Ool007oo001oo`00Ool007oo1000
00Moo`00001oo`00Ool00002Ool400000goo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0
009oo`09001oo`00Ool007oo001oo`0000eoo`07001oo`00Ool007oo00000Woo00/007oo001oo`00
Ool007oo001oo`000;Yoo`008Woo00<007ooOol03Goo0P0000=oo`00Ool027oo00T007oo000007oo
001oo`00Ool00P0000Eoo`00001oo`0000Ioo`0B001oo`00Ool007oo001oo`00Ool007oo001oo`00
Ool007oo1@0000Eoo`00Ool007oo00<00005Ool007oo001oo`02000037oo001oo`00Ool007oo001o
o`00Ool0009oo`09001oo`00Ool007oo001oo`00015oo`03001oo`00009oo`07001oo`00Ool007oo
0000_Woo000ROol00`00Oomoo`0=Ool200000goo001oo`08Ool02@00Ool00000Ool007oo001oo`02
00001Goo000007oo00001Woo01@007oo001oo`00Ool007oo001oo`00Ool007oo001oo`00Ool007oo
0P0000=oo`000000100000Moo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0
009oo`07001oo`00Ool007oo0000gGoo000ROol00`00Oomoo`0=Ool200000goo001oo`08Ool02@00
Ool00000Ool007oo001oo`0200001Goo000007oo00001Woo01@007oo001oo`00Ool007oo001oo`00
Ool007oo001oo`00Ool007oo0P0000Aoo`00000000Aoo`06001oo`00Ool007oo0P0000aoo`00Ool0
07oo001oo`00Ool007oo0002Ool01@00Ool007oo0000ggoo000ROol00`00Oomoo`0=Ool200000goo
001oo`08Ool02@00Ool00000Ool007oo001oo`0200001Goo000007oo00001Woo00D007oo000007oo
0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Aoo`00Ool000Aoo`05001oo`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oo`000>5oo`008Woo0P003Woo
0P0000=oo`00Ool027oo00T007oo000007oo001oo`00Ool00P0000Eoo`00001oo`0000Ioo`05001o
o`00001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00800004Ool007oo0006Ool00`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oo`000>5oo`008Woo00<0
07ooOol03Goo0P0000=oo`00Ool027oo0P0000Moo`00Ool007oo001oo`0200001Goo000007oo0000
1Woo1000011oo`00001oo`00Ool007oo001oo`00Ool007oo001oo`800003Ool007oo00Moo`03001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0hGoo000ROol00`00
Oomoo`0=Ool200000goo001oo`08Ool200001goo001oo`00Ool007oo00800005Ool00000Ool00004
Ool01`00Ool00000Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000=oo`00
Ool02Goo010007ooOol007oo001oo`00Ool007oo001oo`00Ool00>Ioo`008Woo00<007ooOol03Goo
0P0000=oo`00Ool027oo0P0000Uoo`00Ool007oo001oo`00Ool00`0000Aoo`00Ool0009oo`07001o
o`00001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00<007ooOol02goo
00/007oo001oo`00Ool007oo001oo`000>Qoo`008Woo00<007ooOol03Goo0P0000=oo`00000027oo
0P0000Uoo`00Ool007oo001oo`00Ool00`0000Aoo`00Ool0009oo`@00003Ool007oo0080000>Ool0
07oo001oo`00Ool007oo001oo`00Ool0009oo`03001oogoo00moo`03001oo`000>aoo`008Woo0P00
3Woo0P0000=oo`00000027oo0P0000Uoo`00Ool007oo001oo`00Ool00`0000Aoo`00Ool0009oo`80
0005Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo003oOol4Ool0029oo`03
001oogoo00eoo`800003Ool0000000Qoo`800009Ool007oo001oo`00Ool007oo00<00005Ool007oo
001oo`0300001Goo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`00ogoo17oo
000ROol00`00Oomoo`0=Ool200000goo00000008Ool200002Goo001oo`00Ool007oo001oo`030000
0goo0000000500001Woo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000ogoo
17oo0002Ool20004Ool20004Ool20003Ool30004Ool20006Ool00`00Oomoo`0=Ool2000017oo0000
00001goo0P0000Uoo`00Ool007oo001oo`00Ool00`0000=oo`0000000`0000Qoo`00Ool007oo001o
o`000Woo00/007oo001oo`00Ool007oo001oo`000?moo`Ioo`0000Eoo`00Oomoo`0000=oo`8000=o
o`04001oogoo0005Ool01000Oomoo`000Woo00<007ooOol00goo00<007ooOol03Goo0P0000Aoo`00
000000Moo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool01@0000Qoo`00Ool007oo001oo`00
0Woo00/007oo001oo`00Ool007oo001oo`000?moo`Ioo`0000Eoo`00Oomoo`0000Qoo`04001oogoo
0003Ool30002Ool01000Oomoo`001Goo0`003Goo0P0000Aoo`00000000Moo`80000=Ool007oo001o
o`00Ool007oo001oo`00Ool010000Woo00L007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00
Ool0003oOol8Ool00005Ool007ooOol00008Ool30003Ool01000Oomoo`000goo0P001Woo00<007oo
Ool03Goo0P0000Aoo`00000000Ioo`<0000;Ool007oo001oo`00Ool007oo001oo`0200000goo001o
o`03Ool01`00Ool007oo001oo`00009oo`09001oo`00Ool007oo001oo`000?moo`Qoo`0000Eoo`00
Oomoo`0000Qoo`03001oogoo00=oo`04001oogoo0002Ool01000Oomoo`001Goo00<007ooOol03Goo
0P0000Aoo`00000000Aoo`03001oo`000080000;Ool007oo001oo`00Ool007oo001oo`0200000goo
001oo`03Ool01`00Ool007oo001oo`00009oo`09001oo`00Ool007oo001oo`000?moo`Qoo`000Woo
0P002Woo0`000goo0P0017oo0P001Woo00<007ooOol03Goo0P0000Aoo`00000000Aoo`03001oo`00
0080000;Ool007oo001oo`00Ool007oo001oo`0200000goo001oo`03Ool01`00Ool007oo001oo`00
009oo`07001oo`00Ool007oo0000ogoo2Woo000ROol00`00Oomoo`0=Ool2000017oo0000000017oo
00<007oo00000P0000]oo`00Ool007oo001oo`00Ool007oo00800003Ool007oo00Eoo`05001oo`00
Ool00002Ool01`00Ool007oo001oo`000?moo`Yoo`008Woo0P003Woo0P0000Aoo`00000000Aoo`03
001oo`000080000:Ool007oo001oo`00Ool007oo0002Ool00`00Ool00006Ool01@00Ool007oo0000
0Woo00D007oo001oo`000?moo`aoo`008Woo00<007ooOol03Goo0P0000Aoo`00000000Aoo`03001o
o`000080000:Ool007oo001oo`00Ool007oo0002Ool00`00Ool00008Ool00`00Ool00002Ool01@00
Ool007oo0000ogoo37oo000ROol00`00Oomoo`0=Ool2000017oo0000000017oo00<007oo00000P00
00Yoo`00Ool007oo001oo`00Ool0009oo`03001oo`0000Qoo`03001oo`00009oo`03001oo`000?mo
o`ioo`008Woo00<007ooOol03Goo0P0000Aoo`00000000Aoo`03001oo`000080000:Ool007oo001o
o`00Ool007oo0002Ool00`00Oomoo`0:Ool01000Oomoo`00ogoo47oo000ROol00`00Oomoo`0=Ool2
000017oo0000000017oo00<007oo00000P0000Yoo`00Ool007oo001oo`00Ool0009oo`03001oogoo
0?mooaioo`008Woo0P003Woo0P0000=oo`0000000P000goo00<007oo00000P0000Yoo`00Ool007oo
001oo`00Ool0009oo`03001oogoo0?mooaioo`008Woo00<007ooOol03Goo0P0000=oo`0000000P00
0goo00<007oo00000P0000Yoo`00Ool007oo001oo`00Ool0009oo`03001oogoo009oo`<000Aoo`80
02Yoo`<000Aoo`8000Aoo`8002Ioo`<000=oo`<000Aoo`8002Moo`@000=oo`8000Aoo`8002Ioo`@0
009oo`<000Aoo`8002Moo`<000Aoo`8000Aoo`8000=oo`008Woo00<007ooOol03Goo0P0000=oo`00
00000P000goo1@0000Yoo`00Ool007oo001oo`00Ool0009oo`03001oogoo00Eoo`04001oogoo0002
Ool00`00Oomoo`0XOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol002Ioo`03001oogoo
00Eoo`04001oogoo0002Ool00`00Oomoo`0UOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007oo
Ool002Ioo`03001oogoo00Eoo`04001oogoo0002Ool00`00Oomoo`0WOol01000Oomoo`000Woo00@0
07ooOol0009oo`40005oo`5oo`008Woo00<007ooOol03Goo0P0000=oo`0000000P000goo1@0000Yo
o`00Ool007oo001oo`00Ool0009oo`03001oogoo00Eoo`04001oogoo0002Ool00`00Oomoo`0XOol0
0`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol002Ioo`03001oogoo00Eoo`04001oogoo0002
Ool00`00Oomoo`0VOol01@00Oomoogoo00000Woo00@007ooOol0009oo`03001oogoo02Eoo`03001o
ogoo00Aoo`04001oogoo0002Ool00`00Oomoo`0WOol01000Oomoo`000Woo00@007ooOol0009oo`40
005oo`5oo`008Woo00<007ooOol03Goo0P0000=oo`0000000P000goo1@0000Yoo`00Ool007oo001o
o`00Ool0009oo`03001oogoo009oo`<000=oo`04001oogoo000ZOol00`00Oomoo`02Ool01000Oomo
o`000Woo00@007ooOol002Ioo`03001oogoo009oo`<000=oo`04001oogoo000YOol01000Oomoo`00
0Woo00@007ooOol0009oo`03001oogoo02Ioo`03001oogoo00<000=oo`04001oogoo000WOol20003
Ool01000Oomoo`000Woo00@007ooOol0009oo`008Woo0P003Woo0P0000=oo`0000000P000goo1@00
00Yoo`00Ool007oo001oo`00Ool000Moo`03001oogoo00=oo`04001oogoo000ZOol00`00Oomoo`02
Ool01000Oomoo`000Woo00@007ooOol002Ioo`03001oogoo009oo`03001oogoo00=oo`04001oogoo
000VOol01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo000UOol01000Oomoo`000Woo00<0
07ooOol00goo00@007ooOol002Uoo`04001oogoo0002Ool01000Oomoo`000Woo0@000Goo0Goo000R
Ool00`00Oomoo`0=Ool200000goo000000020003Ool500002Woo001oo`00Ool007oo001oo`001goo
10000goo0P00:Woo0P001Goo0P0017oo0P009Woo0P0017oo10000goo0P00:7oo0P0017oo0P0017oo
0P009goo0P000goo10000goo0P009goo0`0017oo0P0017oo0P000goo000ROol00`00Oomoo`0=Ool2
00000goo000000020003Ool500002Woo001oo`00Ool007oo001oo`00ogoo8goo000ROol00`00Oomo
o`0=Ool200000goo000000030002Ool500002Woo001oo`00Ool007oo001oo`00ogoo8goo000ROol0
0`00Oomoo`0=Ool200000goo000000030002Ool500002Woo001oo`00Ool007oo001oo`00ogoo8goo
000JOooo001C0001Ool00001\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 0.68954, 
  0.933765, 0.000101968}}]
}, Open  ]],

Cell["Next the Call with the approximate tree rule:", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[explot, \ CRRPlotCApp, \ \ PlotRange\  -> \ {0.69, \ 0.70}, \ 
      DisplayFunction\  -> \ $DisplayFunction]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -42.6443 61.8034 [
[.18254 .60553 -6 -9 ]
[.18254 .60553 6 0 ]
[.34127 .60553 -9 -9 ]
[.34127 .60553 9 0 ]
[.5 .60553 -9 -9 ]
[.5 .60553 9 0 ]
[.65873 .60553 -9 -9 ]
[.65873 .60553 9 0 ]
[.81746 .60553 -9 -9 ]
[.81746 .60553 9 0 ]
[.97619 .60553 -9 -9 ]
[.97619 .60553 9 0 ]
[.01131 0 -24 -4.5 ]
[.01131 0 0 4.5 ]
[.01131 .12361 -30 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -30 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -30 -4.5 ]
[.01131 .49443 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .62428 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .61803 m
.18254 .62428 L
s
[(50)] .18254 .60553 0 1 Mshowa
.34127 .61803 m
.34127 .62428 L
s
[(100)] .34127 .60553 0 1 Mshowa
.5 .61803 m
.5 .62428 L
s
[(150)] .5 .60553 0 1 Mshowa
.65873 .61803 m
.65873 .62428 L
s
[(200)] .65873 .60553 0 1 Mshowa
.81746 .61803 m
.81746 .62428 L
s
[(250)] .81746 .60553 0 1 Mshowa
.97619 .61803 m
.97619 .62428 L
s
[(300)] .97619 .60553 0 1 Mshowa
.125 Mabswid
.05556 .61803 m
.05556 .62178 L
s
.0873 .61803 m
.0873 .62178 L
s
.11905 .61803 m
.11905 .62178 L
s
.15079 .61803 m
.15079 .62178 L
s
.21429 .61803 m
.21429 .62178 L
s
.24603 .61803 m
.24603 .62178 L
s
.27778 .61803 m
.27778 .62178 L
s
.30952 .61803 m
.30952 .62178 L
s
.37302 .61803 m
.37302 .62178 L
s
.40476 .61803 m
.40476 .62178 L
s
.43651 .61803 m
.43651 .62178 L
s
.46825 .61803 m
.46825 .62178 L
s
.53175 .61803 m
.53175 .62178 L
s
.56349 .61803 m
.56349 .62178 L
s
.59524 .61803 m
.59524 .62178 L
s
.62698 .61803 m
.62698 .62178 L
s
.69048 .61803 m
.69048 .62178 L
s
.72222 .61803 m
.72222 .62178 L
s
.75397 .61803 m
.75397 .62178 L
s
.78571 .61803 m
.78571 .62178 L
s
.84921 .61803 m
.84921 .62178 L
s
.88095 .61803 m
.88095 .62178 L
s
.9127 .61803 m
.9127 .62178 L
s
.94444 .61803 m
.94444 .62178 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.02381 0 m
.03006 0 L
s
[(0.69)] .01131 0 1 0 Mshowa
.02381 .12361 m
.03006 .12361 L
s
[(0.692)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.694)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.696)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.698)] .01131 .49443 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .30271 m
.0746 .30271 L
.07778 .30271 L
.08095 .30271 L
.08413 .30271 L
.0873 .30271 L
.09048 .30271 L
.09365 .30271 L
.09683 .30271 L
.1 .30271 L
.10317 .30271 L
.10635 .30271 L
.10952 .30271 L
.1127 .30271 L
.11587 .30271 L
.11905 .30271 L
.12222 .30271 L
.1254 .30271 L
.12857 .30271 L
.13175 .30271 L
.13492 .30271 L
.1381 .30271 L
.14127 .30271 L
.14444 .30271 L
.14762 .30271 L
.15079 .30271 L
.15397 .30271 L
.15714 .30271 L
.16032 .30271 L
.16349 .30271 L
.16667 .30271 L
.16984 .30271 L
.17302 .30271 L
.17619 .30271 L
.17937 .30271 L
.18254 .30271 L
.18571 .30271 L
.18889 .30271 L
.19206 .30271 L
.19524 .30271 L
.19841 .30271 L
.20159 .30271 L
.20476 .30271 L
.20794 .30271 L
.21111 .30271 L
.21429 .30271 L
.21746 .30271 L
.22063 .30271 L
.22381 .30271 L
.22698 .30271 L
Mistroke
.23016 .30271 L
.23333 .30271 L
.23651 .30271 L
.23968 .30271 L
.24286 .30271 L
.24603 .30271 L
.24921 .30271 L
.25238 .30271 L
.25556 .30271 L
.25873 .30271 L
.2619 .30271 L
.26508 .30271 L
.26825 .30271 L
.27143 .30271 L
.2746 .30271 L
.27778 .30271 L
.28095 .30271 L
.28413 .30271 L
.2873 .30271 L
.29048 .30271 L
.29365 .30271 L
.29683 .30271 L
.3 .30271 L
.30317 .30271 L
.30635 .30271 L
.30952 .30271 L
.3127 .30271 L
.31587 .30271 L
.31905 .30271 L
.32222 .30271 L
.3254 .30271 L
.32857 .30271 L
.33175 .30271 L
.33492 .30271 L
.3381 .30271 L
.34127 .30271 L
.34444 .30271 L
.34762 .30271 L
.35079 .30271 L
.35397 .30271 L
.35714 .30271 L
.36032 .30271 L
.36349 .30271 L
.36667 .30271 L
.36984 .30271 L
.37302 .30271 L
.37619 .30271 L
.37937 .30271 L
.38254 .30271 L
.38571 .30271 L
Mistroke
.38889 .30271 L
.39206 .30271 L
.39524 .30271 L
.39841 .30271 L
.40159 .30271 L
.40476 .30271 L
.40794 .30271 L
.41111 .30271 L
.41429 .30271 L
.41746 .30271 L
.42063 .30271 L
.42381 .30271 L
.42698 .30271 L
.43016 .30271 L
.43333 .30271 L
.43651 .30271 L
.43968 .30271 L
.44286 .30271 L
.44603 .30271 L
.44921 .30271 L
.45238 .30271 L
.45556 .30271 L
.45873 .30271 L
.4619 .30271 L
.46508 .30271 L
.46825 .30271 L
.47143 .30271 L
.4746 .30271 L
.47778 .30271 L
.48095 .30271 L
.48413 .30271 L
.4873 .30271 L
.49048 .30271 L
.49365 .30271 L
.49683 .30271 L
.5 .30271 L
.50317 .30271 L
.50635 .30271 L
.50952 .30271 L
.5127 .30271 L
.51587 .30271 L
.51905 .30271 L
.52222 .30271 L
.5254 .30271 L
.52857 .30271 L
.53175 .30271 L
.53492 .30271 L
.5381 .30271 L
.54127 .30271 L
.54444 .30271 L
Mistroke
.54762 .30271 L
.55079 .30271 L
.55397 .30271 L
.55714 .30271 L
.56032 .30271 L
.56349 .30271 L
.56667 .30271 L
.56984 .30271 L
.57302 .30271 L
.57619 .30271 L
.57937 .30271 L
.58254 .30271 L
.58571 .30271 L
.58889 .30271 L
.59206 .30271 L
.59524 .30271 L
.59841 .30271 L
.60159 .30271 L
.60476 .30271 L
.60794 .30271 L
.61111 .30271 L
.61429 .30271 L
.61746 .30271 L
.62063 .30271 L
.62381 .30271 L
.62698 .30271 L
.63016 .30271 L
.63333 .30271 L
.63651 .30271 L
.63968 .30271 L
.64286 .30271 L
.64603 .30271 L
.64921 .30271 L
.65238 .30271 L
.65556 .30271 L
.65873 .30271 L
.6619 .30271 L
.66508 .30271 L
.66825 .30271 L
.67143 .30271 L
.6746 .30271 L
.67778 .30271 L
.68095 .30271 L
.68413 .30271 L
.6873 .30271 L
.69048 .30271 L
.69365 .30271 L
.69683 .30271 L
.7 .30271 L
.70317 .30271 L
Mistroke
.70635 .30271 L
.70952 .30271 L
.7127 .30271 L
.71587 .30271 L
.71905 .30271 L
.72222 .30271 L
.7254 .30271 L
.72857 .30271 L
.73175 .30271 L
.73492 .30271 L
.7381 .30271 L
.74127 .30271 L
.74444 .30271 L
.74762 .30271 L
.75079 .30271 L
.75397 .30271 L
.75714 .30271 L
.76032 .30271 L
.76349 .30271 L
.76667 .30271 L
.76984 .30271 L
.77302 .30271 L
.77619 .30271 L
.77937 .30271 L
.78254 .30271 L
.78571 .30271 L
.78889 .30271 L
.79206 .30271 L
.79524 .30271 L
.79841 .30271 L
.80159 .30271 L
.80476 .30271 L
.80794 .30271 L
.81111 .30271 L
.81429 .30271 L
.81746 .30271 L
.82063 .30271 L
.82381 .30271 L
.82698 .30271 L
.83016 .30271 L
.83333 .30271 L
.83651 .30271 L
.83968 .30271 L
.84286 .30271 L
.84603 .30271 L
.84921 .30271 L
.85238 .30271 L
.85556 .30271 L
.85873 .30271 L
.8619 .30271 L
Mistroke
.86508 .30271 L
.86825 .30271 L
.87143 .30271 L
.8746 .30271 L
.87778 .30271 L
.88095 .30271 L
.88413 .30271 L
.8873 .30271 L
.89048 .30271 L
.89365 .30271 L
.89683 .30271 L
.9 .30271 L
.90317 .30271 L
.90635 .30271 L
.90952 .30271 L
.9127 .30271 L
.91587 .30271 L
.91905 .30271 L
.92222 .30271 L
.9254 .30271 L
.92857 .30271 L
.93175 .30271 L
.93492 .30271 L
.9381 .30271 L
.94127 .30271 L
.94444 .30271 L
.94762 .30271 L
.95079 .30271 L
.95397 .30271 L
.95714 .30271 L
.96032 .30271 L
.96349 .30271 L
.96667 .30271 L
.96984 .30271 L
.97302 .30271 L
.97619 .30271 L
Mfstroke
.07155 .61803 m
.07327 0 L
s
.07603 0 m
.07778 .57988 L
s
.07778 .57988 m
.08038 0 L
s
.08162 0 m
.08413 .48141 L
.0873 .10588 L
.09048 .37481 L
.09365 .26083 L
.09683 .26576 L
.1 .36146 L
.10317 .15742 L
.10635 .42345 L
.10952 .05152 L
.1127 .45749 L
s
.1127 .45749 m
.11555 0 L
s
.11618 0 m
.11905 .47103 L
s
.11905 .47103 m
.12146 0 L
s
.12299 0 m
.1254 .4694 L
s
.1254 .4694 m
.12771 0 L
s
.12945 0 m
.13175 .45647 L
s
.13175 .45647 m
.13466 0 L
s
.13519 0 m
.1381 .43507 L
.14127 .06581 L
.14444 .40733 L
.14762 .15117 L
.15079 .37486 L
.15397 .21893 L
.15714 .33886 L
.16032 .27213 L
.16349 .3003 L
.16667 .31325 L
.16984 .25988 L
.17302 .34428 L
.17619 .21817 L
.17937 .36683 L
.18254 .17561 L
.18571 .38226 L
.18889 .13255 L
.19206 .39165 L
.19524 .08926 L
.19841 .39593 L
.20159 .04595 L
.20476 .39588 L
.20794 .02109 L
.21111 .39212 L
.21429 .07862 L
.21746 .38522 L
.22063 .12846 L
.22381 .37563 L
.22698 .17149 L
.23016 .36374 L
.23333 .2085 L
.23651 .3499 L
.23968 .24016 L
.24286 .33438 L
.24603 .26705 L
.24921 .31744 L
.25238 .28968 L
.25556 .29929 L
.25873 .30851 L
.2619 .28011 L
.26508 .32395 L
.26825 .26006 L
.27143 .33633 L
.2746 .23928 L
.27778 .34596 L
.28095 .21789 L
.28413 .35314 L
.2873 .196 L
.29048 .3581 L
Mistroke
.29365 .17369 L
.29683 .36106 L
.3 .15104 L
.30317 .36221 L
.30635 .12814 L
.30952 .36174 L
.3127 .12968 L
.31587 .3598 L
.31905 .15812 L
.32222 .35652 L
.3254 .18373 L
.32857 .35205 L
.33175 .20675 L
.33492 .34649 L
.3381 .22737 L
.34127 .33994 L
.34444 .24577 L
.34762 .33251 L
.35079 .26213 L
.35397 .32427 L
.35714 .2766 L
.36032 .3153 L
.36349 .28931 L
.36667 .30567 L
.36984 .3004 L
.37302 .29544 L
.37619 .30998 L
.37937 .28468 L
.38254 .31817 L
.38571 .27343 L
.38889 .32506 L
.39206 .26174 L
.39524 .33074 L
.39841 .24966 L
.40159 .33529 L
.40476 .23723 L
.40794 .33881 L
.41111 .22447 L
.41429 .34135 L
.41746 .21143 L
.42063 .34299 L
.42381 .19814 L
.42698 .34379 L
.43016 .18462 L
.43333 .3438 L
.43651 .17531 L
.43968 .34308 L
.44286 .19219 L
.44603 .34168 L
.44921 .20778 L
Mistroke
.45238 .33964 L
.45556 .22213 L
.45873 .33701 L
.4619 .23534 L
.46508 .33383 L
.46825 .24745 L
.47143 .33013 L
.4746 .25852 L
.47778 .32594 L
.48095 .26863 L
.48413 .32131 L
.4873 .27781 L
.49048 .31625 L
.49365 .28612 L
.49683 .3108 L
.5 .29361 L
.50317 .30499 L
.50635 .30032 L
.50952 .29883 L
.5127 .30628 L
.51587 .29236 L
.51905 .31155 L
.52222 .28559 L
.5254 .31615 L
.52857 .27854 L
.53175 .32013 L
.53492 .27123 L
.5381 .3235 L
.54127 .26369 L
.54444 .32632 L
.54762 .25591 L
.55079 .32859 L
.55397 .24793 L
.55714 .33036 L
.56032 .23976 L
.56349 .33164 L
.56667 .2314 L
.56984 .33247 L
.57302 .22288 L
.57619 .33286 L
.57937 .2142 L
.58254 .33283 L
.58571 .2097 L
.58889 .33241 L
.59206 .22021 L
.59524 .33162 L
.59841 .23006 L
.60159 .33047 L
.60476 .23928 L
.60794 .32899 L
Mistroke
.61111 .2479 L
.61429 .32719 L
.61746 .25594 L
.62063 .32508 L
.62381 .26342 L
.62698 .32268 L
.63016 .27038 L
.63333 .32 L
.63651 .27682 L
.63968 .31707 L
.64286 .28278 L
.64603 .31388 L
.64921 .28827 L
.65238 .31046 L
.65556 .29331 L
.65873 .30681 L
.6619 .29792 L
.66508 .30295 L
.66825 .30212 L
.67143 .29889 L
.6746 .30592 L
.67778 .29463 L
.68095 .30934 L
.68413 .29019 L
.6873 .31239 L
.69048 .28558 L
.69365 .3151 L
.69683 .28079 L
.7 .31747 L
.70317 .27585 L
.70635 .31951 L
.70952 .27076 L
.7127 .32125 L
.71587 .26553 L
.71905 .32269 L
.72222 .26016 L
.7254 .32384 L
.72857 .25467 L
.73175 .32471 L
.73492 .24905 L
.7381 .32532 L
.74127 .24331 L
.74444 .32567 L
.74762 .23747 L
.75079 .32578 L
.75397 .23152 L
.75714 .32566 L
.76032 .23464 L
.76349 .32531 L
.76667 .24147 L
Mistroke
.76984 .32474 L
.77302 .24794 L
.77619 .32396 L
.77937 .25406 L
.78254 .32298 L
.78571 .25984 L
.78889 .32181 L
.79206 .26529 L
.79524 .32045 L
.79841 .27043 L
.80159 .31891 L
.80476 .27526 L
.80794 .3172 L
.81111 .27979 L
.81429 .31532 L
.81746 .28404 L
.82063 .31328 L
.82381 .28801 L
.82698 .31109 L
.83016 .29171 L
.83333 .30875 L
.83651 .29515 L
.83968 .30627 L
.84286 .29834 L
.84603 .30366 L
.84921 .30128 L
.85238 .30091 L
.85556 .30399 L
.85873 .29803 L
.8619 .30648 L
.86508 .29503 L
.86825 .30874 L
.87143 .29192 L
.8746 .31078 L
.87778 .28869 L
.88095 .31262 L
.88413 .28536 L
.8873 .31426 L
.89048 .28192 L
.89365 .3157 L
.89683 .27838 L
.9 .31696 L
.90317 .27474 L
.90635 .31803 L
.90952 .27101 L
.9127 .31892 L
.91587 .26719 L
.91905 .31964 L
.92222 .26328 L
.9254 .3202 L
Mistroke
.92857 .2593 L
.93175 .3206 L
.93492 .25523 L
.9381 .32084 L
.94127 .25108 L
.94444 .32092 L
.94762 .24687 L
.95079 .32086 L
.95397 .24784 L
.95714 .32066 L
.96032 .25266 L
.96349 .32032 L
.96667 .25727 L
.96984 .31985 L
.97302 .26166 L
.97619 .31924 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00OogooKWoo003oOom^
Ool000Qoo`8000Aoo`8000Aoo`8000=oo`<00?mooe9oo`001goo00@007ooOol000=oo`8000=oo`04
001oogoo0005Ool00`00Oomoo`3oOom?Ool000Moo`04001oogoo0008Ool01000Oomoo`000goo0`00
1Goo0`003Goo0P0000=oo`00Ool02Woo00L007oo001oo`00Ool0003oOolVOol000Moo`04001oogoo
0008Ool30003Ool01000Oomoo`001Goo00<007ooOol03Goo0P0000=oo`00Ool02Woo00L007oo001o
o`00Ool0003oOolVOol000Moo`04001oogoo0008Ool00`00Oomoo`03Ool01000Oomoo`001Goo00<0
07ooOol03Goo0P0000=oo`00Ool02Woo00L007oo001oo`00Ool0003oOolVOol000Qoo`8000Yoo`<0
00=oo`8000Ioo`03001oogoo00eoo`800003Ool007oo00Yoo`07001oo`00Ool007oo0000ogoo9Woo
000ROol00`00Oomoo`0=Ool200000goo001oo`0:Ool01`00Ool007oo001oo`0001Qoo`03001oogoo
0?moo`]oo`008Woo0P003Woo0P0000=oo`00Ool02Woo00L007oo001oo`00Ool0000HOol00`00Oomo
o`3oOol;Ool0029oo`03001oogoo00eoo`800003Ool007oo00Yoo`07001oo`00Ool007oo000067oo
00<007ooOol0ogoo2goo000ROol00`00Oomoo`0=Ool200000goo001oo`0:Ool01`00Ool007oo001o
o`0001Qoo`03001oogoo0?moo`]oo`008Woo00<007ooOol03Goo0P0000=oo`00Ool02Woo00L007oo
001oo`00Ool0000FOol00`00Ool0003oOol=Ool0029oo`03001oogoo00eoo`800003Ool007oo00Qo
o`09001oo`00Ool007oo001oo`0001Ioo`03001oo`000?moo`eoo`008Woo0P003Woo0P0000=oo`00
Ool027oo00T007oo001oo`00Ool007oo00005Woo00<007oo0000ogoo3Goo000ROol00`00Oomoo`0=
Ool200000goo001oo`08Ool02`00Ool007oo001oo`00Ool007oo000057oo00<007oo0000ogoo3Goo
000ROol00`00Oomoo`0=Ool200000goo001oo`08Ool02`00Ool007oo001oo`00Ool007oo000057oo
00<007oo0000ogoo3Goo000ROol00`00Oomoo`0=Ool200000goo001oo`08Ool02`00Ool007oo001o
o`00Ool007oo000057oo00D007oo001oo`000?moo`]oo`008Woo00<007ooOol03Goo0P0000=oo`00
Ool027oo00/007oo001oo`00Ool007oo001oo`0001Aoo`05001oo`00Ool0003oOol;Ool0029oo`80
00ioo`800003Ool007oo00Qoo`0;001oo`00Ool007oo001oo`00Ool0000BOol01`00Ool007oo001o
o`000?moo`]oo`008Woo00<007ooOol03Goo0P0000=oo`00Ool027oo00/007oo001oo`00Ool007oo
001oo`00019oo`07001oo`00Ool007oo0000ogoo2goo000ROol00`00Oomoo`0=Ool2000017oo001o
o`001goo00/007oo001oo`00Ool007oo001oo`00019oo`07001oo`00Ool007oo0000ogoo2goo0002
Ool20004Ool20004Ool20003Ool30003Ool40005Ool00`00Oomoo`0=Ool2000017oo001oo`001goo
00/007oo001oo`00Ool007oo001oo`00019oo`07001oo`00Ool007oo0000ogoo2goo00001Goo001o
ogoo00000goo0P000goo00@007ooOol000Eoo`05001oogooOol00007Ool00`00Oomoo`0=Ool20000
17oo001oo`001goo00/007oo001oo`00Ool007oo001oo`00019oo`07001oo`00Ool007oo0000ogoo
2goo00001Goo001oogoo000027oo00@007ooOol000=oo`<000Aoo`03001oogoo00Aoo`<000eoo`80
0004Ool007oo0007Ool02`00Ool007oo001oo`00Ool007oo00004Woo00L007oo001oo`00Ool0003o
Ool;Ool00005Ool007ooOol00008Ool30003Ool01000Oomoo`001Goo00<007ooOol00goo00<007oo
Ool03Goo0P0000Aoo`00Ool000Moo`0;001oo`00Ool007oo001oo`00Ool0000?Ool03000Oomoo`00
Ool007oo001oo`00Ool001aoo`03001oo`000>Uoo`0000Eoo`00Oomoo`0000Qoo`03001oogoo00=o
o`04001oogoo0002Ool01000Oomoo`001Goo00<007ooOol03Goo0P0000Aoo`00Ool000Moo`0;001o
o`00Ool007oo001oo`00Ool0000?Ool03000Oomoo`00Ool007oo001oo`00Ool001aoo`03001oo`00
0>Uoo`000Woo0P002Woo0`000goo0P0017oo0P001Woo00<007ooOol03Goo0P0000Aoo`00Ool000Mo
o`0;001oo`00Ool007oo001oo`00Ool0000?Ool03000Oomoo`00Ool007oo001oo`00Ool001aoo`03
001oo`000>Uoo`008Woo00<007ooOol03Goo0P0000Aoo`00Ool000Moo`0=001oo`00Ool007oo001o
o`00Ool007oo00003Goo00`007ooOol007oo001oo`00Ool007oo000JOol01@00Ool007oo0000jGoo
000ROol2000>Ool2000017oo001oo`0017oo010007ooOol007oo001oo`00Ool007oo001oo`00Ool0
00eoo`0<001oogoo001oo`00Ool007oo001oo`006Woo00D007oo001oo`000>Uoo`008Woo00<007oo
Ool03Goo0P0000Aoo`00Ool000Aoo`0@001oogoo001oo`00Ool007oo001oo`00Ool007oo000=Ool0
3000Oomoo`00Ool007oo001oo`00Ool001Yoo`07001oo`00Ool007oo0000igoo000ROol00`00Oomo
o`0=Ool2000017oo001oo`0017oo010007ooOol007oo001oo`00Ool007oo001oo`00Ool000eoo`0<
001oogoo001oo`00Ool007oo001oo`006Woo00L007oo001oo`00Ool0003WOol0029oo`03001oogoo
00eoo`800004Ool007oo0004Ool04000Oomoo`00Ool007oo001oo`00Ool007oo001oo`002goo00@0
07oo001oo`80000:Ool007oo001oo`00Ool007oo000FOol02@00Ool007oo001oo`00Ool0000WOol0
0`00Oomoo`2mOol0029oo`03001oogoo00eoo`800004Ool007oo0004Ool04000Oomoo`00Ool007oo
001oo`00Ool007oo001oo`002goo00@007oo001oo`80000:Ool007oo001oo`00Ool007oo000FOol0
2@00Ool007oo001oo`00Ool0000WOol00`00Oomoo`2mOol0029oo`8000ioo`800004Ool007oo0004
Ool04000Oomoo`00Ool007oo001oo`00Ool007oo001oo`002goo00@007oo001oo`80000:Ool007oo
001oo`00Ool007oo000FOol02`00Ool007oo001oo`00Ool007oo00008goo00<007oo0000_goo000R
Ool00`00Oomoo`0=Ool2000017oo001oo`0017oo010007ooOol007oo001oo`00Ool007oo001oo`00
Ool000]oo`04001oo`00Ool200002Woo001oo`00Ool007oo001oo`005Woo00/007oo001oo`00Ool0
07oo001oo`0002=oo`05001oo`00Ool0002mOol0029oo`03001oogoo00eoo`800004Ool007oo0004
Ool04000Oomoo`00Ool007oo001oo`00Ool007oo001oo`002goo00@007oo001oo`80000:Ool007oo
001oo`00Ool007oo000DOol03@00Ool007oo001oo`00Ool007oo001oo`00025oo`07001oo`00Ool0
07oo0000_Goo000ROol00`00Oomoo`0=Ool2000017oo001oo`0017oo010007ooOol007oo001oo`00
Ool007oo001oo`00Ool000]oo`04001oo`00Ool200002Woo001oo`00Ool007oo001oo`0057oo00d0
07oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`0LOol02@00Ool007oo001oo`00Ool0
002kOol0029oo`03001oogoo00eoo`800004Ool007oo0004Ool03`00Oomoo`00Ool007oo001oo`00
Ool007oo001oo`02000:Ool01000Ool007oo0P0000Aoo`00001oo`800006Ool007oo001oo`004Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`0JOol02`00Ool007oo001oo`00
Ool007oo0000:goo00<007oo0000SGoo000ROol2000>Ool2000017oo001oo`0017oo00/007ooOol0
07oo001oo`00Ool007oo00<00005Ool00000Ool00006Ool01P00Ool007oo001oo`800004Ool00000
Ool200001Woo001oo`00Ool000moo`0@001oogoo001oo`00Ool007oo001oo`00Ool007oo0002Ool0
0`00Oomoo`0JOol02`00Ool007oo001oo`00Ool007oo0000:goo00D007oo001oo`0008]oo`008Woo
00<007ooOol03Goo0P0000Aoo`00Ool000Aoo`0;001oogoo001oo`00Ool007oo001oo`0300001Goo
000007oo00001Woo00H007oo001oo`00Ool300000goo001oo`0200001Woo001oo`00Ool000moo`0@
001oogoo001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`0GOol04000Oomoo`00Ool0
07oo001oo`00Ool007oo001oo`009goo00L007oo001oo`00Ool0002;Ool0029oo`03001oogoo00eo
o`800004Ool007oo0004Ool200001goo00000000Ool007oo00800004Ool00000Ool200000goo001o
o`05Ool01P00Ool007oo001oo`<00003Ool007oo00800006Ool007oo001oo`003goo00l007ooOol0
07oo001oo`00Ool007oo001oo`00Ool00P0000Aoo`00Ool001Moo`0@001oogoo001oo`00Ool007oo
001oo`00Ool007oo000UOol02@00Ool007oo001oo`00Ool00002Ool00`00Oomoo`0aOol00`00Oomo
o`1BOol0009oo`8000Aoo`8000Aoo`8000=oo`<000Aoo`<000Eoo`03001oogoo00eoo`800004Ool0
07oo0004Ool200002Woo000007oo001oo`00Ool007oo0P0000Eoo`00001oo`0000Ioo`06001oo`00
Ool007oo0`0000=oo`00Ool0100000Aoo`00Ool000eoo`03001oo`00009oo`0<001oo`00Ool007oo
001oo`00Ool007oo0P0000Aoo`00Ool001Eoo`03001oo`00009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00<007ooOol07Woo00/007oo001oo`00Ool007oo001oo`00009oo`03001oo`00
02moo`05001oo`00Ool0001BOol00005Ool007ooOol00003Ool20003Ool01000Oomoo`001Goo00<0
07ooOol00Woo00<007ooOol017oo00<007ooOol03Goo0P0000=oo`0000000P000goo1@0000Moo`00
Ool007oo001oo`0500000goo001oo`05Ool01P00Ool007oo001oo`<00005Ool007oo001oo`020000
17oo001oo`000Woo00<007ooOol027oo00<007oo00000Woo00`007oo001oo`00Ool007oo001oo`00
Ool200001Woo001oo`00Ool001=oo`03001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00<007ooOol07Woo00/007oo001oo`00Ool007oo001oo`00009oo`03001oo`0002eoo`09
001oo`00Ool007oo001oo`00051oo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0003Ool30002Ool4
0005Ool3000=Ool200000goo000000020003Ool500001goo001oo`00Ool007oo00D00003Ool007oo
00Eoo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`800004Ool007oo0002Ool00`00Oomoo`08
Ool00`00Ool00002Ool01000Ool007oo0P0000=oo`00Ool00P0000Uoo`00001oo`00Ool007oo0000
4Goo00H007oo001oo`00Ool2000037oo001oo`00Ool007oo001oo`00Ool0009oo`03001oo`0001ao
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00D007oo001oo`0002Uoo`0=001oo`00Ool0
07oo001oo`00Ool007oo0000>7oo00<007oo00000Woo00<007ooOol03Woo00001Goo001oogoo0000
27oo0`000goo00@007ooOol0009oo`03001oo`0000Ioo`03001oogoo00eoo`800003Ool000000080
00=oo`D00009Ool007oo001oo`00Ool007oo00<00003Ool007oo00Eoo`0>001oo`00Ool007oo001o
o`00Ool007oo001oo`@00005Ool007ooOol0000:Ool02@00Ool00000Ool007oo001oo`0200000goo
001oo`0200002Goo000007oo001oo`00Ool0000AOol01P00Ool007oo001oo`80000<Ool007oo001o
o`00Ool007oo001oo`000Woo00D007oo001oo`0001Moo`0@001oogoo001oo`00Ool007oo001oo`00
Ool007oo0002Ool01`00Ool007oo001oo`0002Aoo`0@001oogoo001oo`00Ool007oo001oo`00Ool0
07oo0002Ool00`00Oomoo`0aOol01@00Ool007oo00000Woo00D007oo001oo`0000aoo`0000Eoo`00
Oomoo`0000Qoo`03001oogoo00=oo`04001oogoo0002Ool00`00Ool00006Ool00`00Oomoo`0=Ool2
00000goo0000000300000goo001oo`0200002goo001oo`00Ool007oo001oo`00Ool00`0000=oo`00
Ool00goo00@007oo001oo`<0000;Ool007oo001oo`00Ool007oo001oo`0200001Goo001oogoo0000
27oo00@007oo001oo`800003Ool007oo00800005Ool007oo001oo`05000027oo001oo`00Ool007oo
000=Ool02000Ool007oo001oo`00Ool200000goo001oo`0200001goo000007oo001oo`00009oo`05
001oo`00Ool0000EOol00`00Ool00002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool20000
27oo001oo`00Ool007oo000POol00`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`03001oo`0002eoo`09001oo`00Ool007oo001oo`00009oo`07001oo`00Ool007oo00002Woo
0002Ool2000:Ool30003Ool20004Ool20006Ool00`00Oomoo`0=Ool200000goo0000000500003Woo
000007oo001oo`00Ool007oo001oo`00Ool300000goo001oo`03Ool01000Ool007oo0P0000aoo`00
001oo`00Ool007oo001oo`00Ool200001goo001oogoo001oo`0000Ioo`04001oo`00Ool200000goo
0000000200001goo001oo`00Ool007oo00<00008Ool007oo001oo`00Ool000eoo`08001oo`00Ool0
07oo001oo`800006Ool00000Ool007oo0P0000Eoo`00Ool007oo00800006Ool007oo001oo`004Goo
00D007oo001oo`00009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`80000:Ool007oo001o
o`00Ool007oo000JOol01`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00L007oo001oo`00Ool0000UOol03@00Ool007oo001oo`00Ool007oo001oo`00009oo`07
001oo`00Ool007oo00002Woo000ROol00`00Oomoo`0=Ool200000goo0000000500003Woo000007oo
001oo`00Ool007oo001oo`00Ool3000017oo001oo`000Woo00<007oo00000`0000aoo`00001oo`00
Ool007oo001oo`00Ool400001Goo000007oo00001Woo00@007oo001oo`80000IOol00000Ool007oo
001oo`00Ool007oo001oo`00Ool007oo001oo`00Ool007oo00002Goo00X007oo001oo`00Ool007oo
001oo`<00007Ool007oo001oo`00Ool0100000Uoo`00001oo`00Ool007oo00004Goo00H007oo001o
o`00Ool200001Goo001oo`00Ool00P0000=oo`00Ool00P0000moo`00001oo`00Ool007oo001oo`00
Ool007oo00005Woo00T007oo001oo`00Ool007oo00000Woo00`007oo001oo`00Ool007oo001oo`00
Ool200002Woo001oo`00Ool007oo001oo`0087oo014007ooOol007oo001oo`00Ool007oo001oo`00
Ool007oo00800006Ool007oo001oo`002Woo000ROol2000>Ool200000goo0000000500004Woo0000
07oo001oo`00Ool007oo001oo`00Ool007oo001oo`<0009oo`07001oo`00001oo`00Ool00P0000]o
o`00Ool007oo001oo`00Ool007oo00800005Ool00000Ool00004Ool01P00Ool007oo001oo`<0000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Uoo`00001oo`00Ool007oo00002Goo01H007oo
001oo`00Ool007oo001oo`00Ool007oo001oo`00Ool007oo001oo`80000;Ool00000Ool007oo001o
o`00Ool0000=Ool02000Ool007oo001oo`00Ool200000goo001oo`0200001goo001oo`00Ool007oo
00<0000<Ool007oo001oo`00Ool007oo001oo`000Woo00<007ooOol03goo00X007oo001oo`00Ool0
07oo001oo`800005Ool007oo001oo`0200001Woo000007oo001oo`80000<Ool007oo001oo`00Ool0
07oo001oo`006Woo00D007oo001oo`00009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`80
0006Ool007oo001oo`002Woo000ROol00`00Oomoo`0=Ool200000goo0000000500004Woo000007oo
001oo`00Ool007oo001oo`00Ool007oo001oo`<0009oo`@00003Ool007oo0080000;Ool007oo001o
o`00Ool007oo001oo`05000017oo001oo`000Woo00@007oo001oo`80000@Ool00000Ool007oo001o
o`00Ool007oo001oo`00Ool200000goo0000000200001Woo001oo`00Ool000Eoo`08001oo`00Ool0
07oo001oo`80000@Ool00000Ool007oo001oo`00Ool007oo001oo`00Ool200002goo000007oo001o
o`00Ool007oo00002Goo00X007oo001oo`00Ool007oo001oo`<0000;Ool007oo001oo`00Ool007oo
001oo`0200000goo001oo`02000027oo001oo`00Ool007oo0002Ool00`00Ool0000:Ool03`00Oomo
o`00Ool007oo001oo`00Ool007oo001oo`02000037oo000007oo001oo`00Ool007oo001oo`<0000>
Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`03001oogoo00moo`0:001oo`00Ool007oo001o
o`00Ool200001goo001oo`00Ool007oo00800003Ool007oo00800007Ool00000Ool007oo00002Woo
000ROol00`00Oomoo`0=Ool01000Ool007oo1P00019oo`00001oo`00Ool007oo001oo`00Ool007oo
001oo`00Ool900000goo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00800005Ool0
0000Ool00002Ool01`00Ool00000Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000Eoo`00Ool007oo00800004Ool007oo0002Ool200001Goo001oo`00Ool0100000=oo`00Ool0
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200000goo001oo`0400001Woo001oo`00Ool0
00Eoo`08001oo`00Ool007oo001oo`<0000?Ool007oo001oo`00Ool007oo001oo`00Ool007oo0080
0007Ool007oo001oo`00Ool00P0000Eoo`00Ool007oo00800004Ool007oo0006Ool00`00Ool00002
Ool02P00Ool007oo001oo`00Ool007oo0P00011oo`00001oo`00Ool007oo001oo`00Ool007oo001o
o`80000=Ool00000Ool007oo001oo`00Ool007oo00000Woo00D007oo001oo`0000Uoo`0>001oo`00
Ool007oo001oo`00Ool007oo001oo`80000<Ool00000Ool007oo001oo`00Ool007oo0P0000Qoo`00
Ool007oo001oo`002Goo000ROol00`00Oomoo`0=Oooo000c0009Ool0029oo`8000ioo`06001oo`00
Ool007oo1000011oo`00001oo`00Ool007oo001oo`00Ool007oo001oo`800003Ool0000000800007
Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02000017oo001oo`00
0Woo00P007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02000037oo
001oo`00Ool007oo001oo`00Ool000=oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0003Ool0
3000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`02000027oo001oo`00Ool007oo0002Ool01000Ool0
07oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00
Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Qoo`00Ool007oo001oo`00
0Woo00<007oo00000Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Woo001oo`00Ool007oo
001oo`00Ool007oo0002Ool01`00Ool007oo001oo`0000Uoo`008Woo00<007ooOol03Goo00H007oo
001oo`00Ool4000047oo000007oo001oo`00Ool007oo001oo`00Ool007oo0P0000aoo`00Oomoo`00
Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo00800004Ool007oo0004
Ool01P00Ool007oo001oo`80000<Ool007oo001oo`00Ool007oo001oo`000Woo00/007oo001oo`00
Ool007oo001oo`0000Eoo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool000Moo`0:001oo`00Ool007oo
001oo`00Ool200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`03001oo`0000Qoo`0@001oogoo001oo`00Ool007oo001oo`00Ool007oo
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00<007oo00003goo00/007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool0
07oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool00009Ool0029oo`03001oogoo00eoo`06
001oo`00Ool007oo0P0000=oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
0goo001oo`03Ool01P00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00Aoo`00Ool000Aoo`06001oo`00Ool007oo0P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool0
2@00Ool007oo001oo`00Ool00009Ool02`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0000;Ool02@00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001o
o`00Ool007oo000057oo00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool02`00Ool007oo001oo`00Ool007oo000077oo00D007oo001oo`00009oo`0=001o
o`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool00009Ool0029oo`03001oogoo
00eoo`07001oo`00Ool007oo00000Woo010007oo000007oo001oo`00Ool007oo001oo`00Ool0009o
o`03001oo`0000Aoo`07001oo`00Ool007oo00000Woo00`007oo001oo`00Ool007oo001oo`00Ool2
00000goo001oo`05Ool01@00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool02@00Ool007oo001oo`00Ool0000;Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0000COol01@00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0000NOol00`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool00018Ool0029oo`03001o
ogoo00eoo`07001oo`00Ool007oo00000Woo010007oo000007oo001oo`00Ool007oo001oo`00Ool0
009oo`03001oo`0000Aoo`07001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool00`00Oomoo`06Ool00`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`07001oo`00Ool007oo00003goo00L007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`03001oo`0001Uoo`0@001oogoo001oo`00Ool007oo001oo`00Ool007oo
0002Ool00`00Ool00025Ool0029oo`8000ioo`07001oo`00Ool007oo00000Woo010007oo000007oo
001oo`00Ool007oo001oo`00Ool0009oo`03001oo`0000Aoo`07001oo`00Ool007oo00000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool0000;Ool00`00Ool00002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`05001oo`00Ool0000COol01@00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool0002hOol0029oo`03001oogoo00eoo`07001oo`00Ool007oo00000Woo010007oo
000007oo001oo`00Ool007oo001oo`00Ool0009oo`03001oogoo00Aoo`07001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool0000=Ool04000Oomoo`00Ool007oo001oo`00Ool0
07oo001oo`000Woo00<007oo00006Goo00`007ooOol007oo001oo`00Ool007oo002lOol0029oo`03
001oogoo00eoo`07001oo`00Ool007oo00000Woo010007oo000007oo001oo`00Ool007oo001oo`00
Ool0009oo`03001oogoo00Ioo`05001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00011oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0hGoo0002Ool20004
Ool20004Ool20003Ool30004Ool20006Ool00`00Oomoo`0=Ool01`00Ool007oo001oo`00009oo`0@
001oo`00001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`06Ool01@00Ool007oo0000
0Woo00/007oo001oo`00Ool007oo001oo`0001Aoo`0;001oo`00Ool007oo001oo`00Ool0003VOol0
0005Ool007ooOol00003Ool20003Ool01000Oomoo`001Goo00@007ooOol0009oo`03001oogoo00=o
o`03001oogoo00eoo`07001oo`00Ool007oo00000Woo010007ooOol007oo001oo`00Ool007oo001o
o`00Ool0009oo`03001oogoo00Ioo`05001oo`00Ool00002Ool02`00Ool007oo001oo`00Ool007oo
00005Woo00L007oo001oo`00Ool0003XOol00005Ool007ooOol00008Ool01000Oomoo`000goo0`00
0Woo00@007ooOol000Eoo`<000eoo`07001oo`00Ool007oo00001Goo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool00`00Oomoo`06Ool01@00Ool007oo00000Woo00T007oo001oo`00Ool007oo
0000ogoo27oo00001Goo001oogoo000027oo0`000goo00@007ooOol0009oo`<000Ioo`03001oogoo
00eoo`07001oo`00Ool007oo00001Goo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00
Oomoo`08Ool00`00Ool00002Ool02@00Ool007oo001oo`00Ool0003oOol8Ool00005Ool007ooOol0
0008Ool00`00Oomoo`03Ool01000Oomoo`000Woo00<007ooOol01Woo00<007ooOol03Goo00D007oo
001oo`0000Moo`0=001oo`00Ool007oo001oo`00Ool007oo00003Goo00<007oo00000Woo00L007oo
001oo`00Ool0003oOol:Ool0009oo`8000Yoo`<000=oo`8000Aoo`<000Eoo`03001oogoo00eoo`05
001oo`00Ool00007Ool03@00Ool007oo001oo`00Ool007oo001oo`0000moo`0:001oogoo001oo`00
Ool007oo003oOol:Ool0029oo`03001oogoo00eoo`05001oo`00Ool00007Ool03@00Ool007oo001o
o`00Ool007oo001oo`0000moo`08001oogoo001oo`00Ool00?moo`aoo`008Woo0P003Woo00D007oo
001oo`0000Moo`0=001oo`00Ool007oo001oo`00Ool007oo0000ogoo8goo000ROol00`00Oomoo`0=
Ool01@00Ool007oo00001goo00d007oo001oo`00Ool007oo001oo`00Ool0003oOolSOol0029oo`03
001oogoo00eoo`05001oo`00Ool00007Ool02`00Ool007oo001oo`00Ool007oo0000ogoo9Goo000R
Ool00`00Oomoo`0=Ool01@00Ool007oo00001goo00/007oo001oo`00Ool007oo001oo`000?moobEo
o`008Woo00<007ooOol03Goo00D007oo001oo`0000Moo`0;001oo`00Ool007oo001oo`00Ool0003o
OolUOol0029oo`8000ioo`05001oo`00Ool00009Ool02@00Ool007oo001oo`00Ool0003oOolUOol0
029oo`03001oogoo00eoo`05001oo`00Ool00009Ool01`00Ool007oo001oo`000?moobMoo`008Woo
00<007ooOol03Goo00D007oo001oo`0000Uoo`07001oo`00Ool007oo0000ogoo9goo000ROol00`00
Oomoo`0=Ool01@00Ool007oo00002Goo00L007oo001oo`00Ool0003oOolWOol0029oo`03001oogoo
00eoo`05001oo`00Ool00009Ool01`00Ool007oo001oo`000?moobMoo`008Woo0P003Woo00D007oo
001oo`0000]oo`03001oo`000?moobUoo`008Woo00<007ooOol03Goo00D007oo001oo`0000]oo`03
001oo`000?moobUoo`008Woo00<007ooOol03Goo00D007oo001oo`000?moocMoo`000Woo0P0017oo
0P0017oo0P000goo0`0017oo0P001Woo00<007ooOol03Goo00D007oo001oo`000?moocMoo`0000Eo
o`00Oomoo`0000=oo`8000=oo`04001oogoo0005Ool01000Oomoo`000Woo00<007ooOol00goo00<0
07ooOol03Goo00<007oo0000ogoo>Goo00001Goo001oogoo000027oo00@007ooOol000=oo`<0009o
o`04001oogoo0005Ool3000=Ool00`00Ool0003oOoliOol00005Ool007ooOol00008Ool30003Ool0
1000Oomoo`000goo0P001Woo00<007ooOol03Goo00<007oo0000ogoo>Goo00001Goo001oogoo0000
27oo00<007ooOol00goo00@007ooOol0009oo`04001oogoo0005Ool00`00Oomoo`0=Ool00`00Ool0
003oOoliOol0009oo`8000Yoo`<000=oo`8000Aoo`8000Ioo`03001oogoo00eoo`03001oo`000?mo
ocUoo`008Woo00<007ooOol03Goo00<007oo0000ogoo>Goo000ROol2000>Ool00`00Ool0003oOoli
Ool0029oo`03001oogoo00eoo`03001oo`000?moocUoo`008Woo00<007ooOol03Goo00<007oo0000
ogoo>Goo000ROol00`00Oomoo`0=Ool00`00Ool0003oOoliOol0029oo`03001oogoo00eoo`03001o
o`000?moocUoo`008Woo0P003Woo00<007oo0000ogoo>Goo000ROol00`00Oomoo`0=Ool00`00Ool0
000MOol30004Ool2000ZOol30004Ool20004Ool2000VOol30003Ool30004Ool2000WOol40003Ool2
0004Ool2000VOol40002Ool30004Ool2000WOol30004Ool20004Ool20003Ool0029oo`03001oogoo
00eoo`03001oo`00021oo`04001oogoo0002Ool00`00Oomoo`0XOol00`00Oomoo`02Ool01000Oomo
o`000Woo00@007ooOol002Ioo`03001oogoo00Eoo`04001oogoo0002Ool00`00Oomoo`0UOol00`00
Oomoo`02Ool01000Oomoo`000Woo00@007ooOol002Ioo`03001oogoo00Eoo`04001oogoo0002Ool0
0`00Oomoo`0WOol01000Oomoo`000Woo00@007ooOol0009oo`40005oo`5oo`008Woo00<007ooOol0
3Goo00<007oo000087oo00@007ooOol0009oo`03001oogoo02Qoo`03001oogoo009oo`04001oogoo
0002Ool01000Oomoo`009Woo00<007ooOol01Goo00@007ooOol0009oo`03001oogoo02Ioo`05001o
ogooOol00002Ool01000Oomoo`000Woo00<007ooOol09Goo00<007ooOol017oo00@007ooOol0009o
o`03001oogoo02Moo`04001oogoo0002Ool01000Oomoo`000Woo0@000Goo0Goo000ROol00`00Oomo
o`0=Ool00`00Oomoo`0MOol30003Ool01000Oomoo`00:Woo00<007ooOol00Woo00@007ooOol0009o
o`04001oogoo000VOol00`00Oomoo`02Ool30003Ool01000Oomoo`00:Goo00@007ooOol0009oo`04
001oogoo0002Ool00`00Oomoo`0VOol00`00Oomoo`030003Ool01000Oomoo`009goo0P000goo00@0
07ooOol0009oo`04001oogoo0002Ool0029oo`8000ioo`03001oogoo01eoo`03001oogoo00=oo`04
001oogoo000ZOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol002Ioo`03001oogoo009o
o`03001oogoo00=oo`04001oogoo000VOol01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo
000UOol01000Oomoo`000Woo00<007ooOol00goo00@007ooOol002Uoo`04001oogoo0002Ool01000
Oomoo`000Woo0@000Goo0Goo000ROol00`00Oomoo`0=Ool00`00Oomoo`0MOol40003Ool2000ZOol2
0005Ool20004Ool2000VOol20004Ool40003Ool2000XOol20004Ool20004Ool2000WOol20003Ool4
0003Ool2000WOol30004Ool20004Ool20003Ool0029oo`03001oogoo00eoo`03001oogoo0?moocUo
o`008Woo00<007ooOol03Goo00<007ooOol0ogoo>Goo000ROol00`00Oomoo`0=Ool00`00Oomoo`3o
OoliOol001Yoool005<0005oo`00\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 0.68954, 
  0.933765, 0.000101968}}]
}, Open  ]],

Cell["Next the Put with the \"exact\" form of the tree rule:", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[explotP, \ CRRPlotP, \ \ PlotRange\  -> {0.74, \ 0.75}, \ 
      DisplayFunction\  -> \ $DisplayFunction]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -45.7345 61.8034 [
[.18254 -0.0125 -6 -9 ]
[.18254 -0.0125 6 0 ]
[.34127 -0.0125 -9 -9 ]
[.34127 -0.0125 9 0 ]
[.5 -0.0125 -9 -9 ]
[.5 -0.0125 9 0 ]
[.65873 -0.0125 -9 -9 ]
[.65873 -0.0125 9 0 ]
[.81746 -0.0125 -9 -9 ]
[.81746 -0.0125 9 0 ]
[.97619 -0.0125 -9 -9 ]
[.97619 -0.0125 9 0 ]
[.01131 .12361 -30 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -30 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -30 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -24 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 0 m
.18254 .00625 L
s
[(50)] .18254 -0.0125 0 1 Mshowa
.34127 0 m
.34127 .00625 L
s
[(100)] .34127 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(150)] .5 -0.0125 0 1 Mshowa
.65873 0 m
.65873 .00625 L
s
[(200)] .65873 -0.0125 0 1 Mshowa
.81746 0 m
.81746 .00625 L
s
[(250)] .81746 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(300)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.15079 0 m
.15079 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.27778 0 m
.27778 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.53175 0 m
.53175 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.9127 0 m
.9127 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(0.742)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.744)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.746)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.748)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(0.75)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .20223 m
.0746 .20223 L
.07778 .20223 L
.08095 .20223 L
.08413 .20223 L
.0873 .20223 L
.09048 .20223 L
.09365 .20223 L
.09683 .20223 L
.1 .20223 L
.10317 .20223 L
.10635 .20223 L
.10952 .20223 L
.1127 .20223 L
.11587 .20223 L
.11905 .20223 L
.12222 .20223 L
.1254 .20223 L
.12857 .20223 L
.13175 .20223 L
.13492 .20223 L
.1381 .20223 L
.14127 .20223 L
.14444 .20223 L
.14762 .20223 L
.15079 .20223 L
.15397 .20223 L
.15714 .20223 L
.16032 .20223 L
.16349 .20223 L
.16667 .20223 L
.16984 .20223 L
.17302 .20223 L
.17619 .20223 L
.17937 .20223 L
.18254 .20223 L
.18571 .20223 L
.18889 .20223 L
.19206 .20223 L
.19524 .20223 L
.19841 .20223 L
.20159 .20223 L
.20476 .20223 L
.20794 .20223 L
.21111 .20223 L
.21429 .20223 L
.21746 .20223 L
.22063 .20223 L
.22381 .20223 L
.22698 .20223 L
Mistroke
.23016 .20223 L
.23333 .20223 L
.23651 .20223 L
.23968 .20223 L
.24286 .20223 L
.24603 .20223 L
.24921 .20223 L
.25238 .20223 L
.25556 .20223 L
.25873 .20223 L
.2619 .20223 L
.26508 .20223 L
.26825 .20223 L
.27143 .20223 L
.2746 .20223 L
.27778 .20223 L
.28095 .20223 L
.28413 .20223 L
.2873 .20223 L
.29048 .20223 L
.29365 .20223 L
.29683 .20223 L
.3 .20223 L
.30317 .20223 L
.30635 .20223 L
.30952 .20223 L
.3127 .20223 L
.31587 .20223 L
.31905 .20223 L
.32222 .20223 L
.3254 .20223 L
.32857 .20223 L
.33175 .20223 L
.33492 .20223 L
.3381 .20223 L
.34127 .20223 L
.34444 .20223 L
.34762 .20223 L
.35079 .20223 L
.35397 .20223 L
.35714 .20223 L
.36032 .20223 L
.36349 .20223 L
.36667 .20223 L
.36984 .20223 L
.37302 .20223 L
.37619 .20223 L
.37937 .20223 L
.38254 .20223 L
.38571 .20223 L
Mistroke
.38889 .20223 L
.39206 .20223 L
.39524 .20223 L
.39841 .20223 L
.40159 .20223 L
.40476 .20223 L
.40794 .20223 L
.41111 .20223 L
.41429 .20223 L
.41746 .20223 L
.42063 .20223 L
.42381 .20223 L
.42698 .20223 L
.43016 .20223 L
.43333 .20223 L
.43651 .20223 L
.43968 .20223 L
.44286 .20223 L
.44603 .20223 L
.44921 .20223 L
.45238 .20223 L
.45556 .20223 L
.45873 .20223 L
.4619 .20223 L
.46508 .20223 L
.46825 .20223 L
.47143 .20223 L
.4746 .20223 L
.47778 .20223 L
.48095 .20223 L
.48413 .20223 L
.4873 .20223 L
.49048 .20223 L
.49365 .20223 L
.49683 .20223 L
.5 .20223 L
.50317 .20223 L
.50635 .20223 L
.50952 .20223 L
.5127 .20223 L
.51587 .20223 L
.51905 .20223 L
.52222 .20223 L
.5254 .20223 L
.52857 .20223 L
.53175 .20223 L
.53492 .20223 L
.5381 .20223 L
.54127 .20223 L
.54444 .20223 L
Mistroke
.54762 .20223 L
.55079 .20223 L
.55397 .20223 L
.55714 .20223 L
.56032 .20223 L
.56349 .20223 L
.56667 .20223 L
.56984 .20223 L
.57302 .20223 L
.57619 .20223 L
.57937 .20223 L
.58254 .20223 L
.58571 .20223 L
.58889 .20223 L
.59206 .20223 L
.59524 .20223 L
.59841 .20223 L
.60159 .20223 L
.60476 .20223 L
.60794 .20223 L
.61111 .20223 L
.61429 .20223 L
.61746 .20223 L
.62063 .20223 L
.62381 .20223 L
.62698 .20223 L
.63016 .20223 L
.63333 .20223 L
.63651 .20223 L
.63968 .20223 L
.64286 .20223 L
.64603 .20223 L
.64921 .20223 L
.65238 .20223 L
.65556 .20223 L
.65873 .20223 L
.6619 .20223 L
.66508 .20223 L
.66825 .20223 L
.67143 .20223 L
.6746 .20223 L
.67778 .20223 L
.68095 .20223 L
.68413 .20223 L
.6873 .20223 L
.69048 .20223 L
.69365 .20223 L
.69683 .20223 L
.7 .20223 L
.70317 .20223 L
Mistroke
.70635 .20223 L
.70952 .20223 L
.7127 .20223 L
.71587 .20223 L
.71905 .20223 L
.72222 .20223 L
.7254 .20223 L
.72857 .20223 L
.73175 .20223 L
.73492 .20223 L
.7381 .20223 L
.74127 .20223 L
.74444 .20223 L
.74762 .20223 L
.75079 .20223 L
.75397 .20223 L
.75714 .20223 L
.76032 .20223 L
.76349 .20223 L
.76667 .20223 L
.76984 .20223 L
.77302 .20223 L
.77619 .20223 L
.77937 .20223 L
.78254 .20223 L
.78571 .20223 L
.78889 .20223 L
.79206 .20223 L
.79524 .20223 L
.79841 .20223 L
.80159 .20223 L
.80476 .20223 L
.80794 .20223 L
.81111 .20223 L
.81429 .20223 L
.81746 .20223 L
.82063 .20223 L
.82381 .20223 L
.82698 .20223 L
.83016 .20223 L
.83333 .20223 L
.83651 .20223 L
.83968 .20223 L
.84286 .20223 L
.84603 .20223 L
.84921 .20223 L
.85238 .20223 L
.85556 .20223 L
.85873 .20223 L
.8619 .20223 L
Mistroke
.86508 .20223 L
.86825 .20223 L
.87143 .20223 L
.8746 .20223 L
.87778 .20223 L
.88095 .20223 L
.88413 .20223 L
.8873 .20223 L
.89048 .20223 L
.89365 .20223 L
.89683 .20223 L
.9 .20223 L
.90317 .20223 L
.90635 .20223 L
.90952 .20223 L
.9127 .20223 L
.91587 .20223 L
.91905 .20223 L
.92222 .20223 L
.9254 .20223 L
.92857 .20223 L
.93175 .20223 L
.93492 .20223 L
.9381 .20223 L
.94127 .20223 L
.94444 .20223 L
.94762 .20223 L
.95079 .20223 L
.95397 .20223 L
.95714 .20223 L
.96032 .20223 L
.96349 .20223 L
.96667 .20223 L
.96984 .20223 L
.97302 .20223 L
.97619 .20223 L
Mfstroke
.07269 .61803 m
.07427 0 L
s
.07498 0 m
.07675 .61803 L
s
.0792 .61803 m
.08095 .18005 L
s
.08095 .18005 m
.0831 .61803 L
s
.08562 .61803 m
.0873 .38158 L
s
.0873 .38158 m
.08979 .61803 L
s
.09166 .61803 m
.09365 .50936 L
.09683 .5407 L
.1 .58631 L
.10317 .40424 L
s
.10317 .40424 m
.10621 .61803 L
s
.10643 .61803 m
.10952 .27412 L
s
.10952 .27412 m
.11248 .61803 L
s
.11286 .61803 m
.11587 .15052 L
s
.11587 .15052 m
.1189 .61803 L
s
.11916 .61803 m
.12222 .03331 L
s
.12222 .03331 m
.12536 .61803 L
s
.12543 .61803 m
.12833 0 L
s
.12883 0 m
.13175 .59808 L
.13492 .07144 L
.1381 .56481 L
.14127 .16931 L
.14444 .52628 L
.14762 .24641 L
.15079 .48397 L
.15397 .30651 L
.15714 .43898 L
.16032 .35261 L
.16349 .39214 L
.16667 .38712 L
.16984 .34409 L
.17302 .41199 L
.17619 .29533 L
.17937 .42879 L
.18254 .24624 L
.18571 .43883 L
.18889 .1971 L
.19206 .44318 L
.19524 .14815 L
.19841 .44272 L
.20159 .09955 L
.20476 .4382 L
.20794 .05143 L
.21111 .43024 L
.21429 .10463 L
.21746 .41937 L
.22063 .15123 L
.22381 .40603 L
.22698 .19119 L
.23016 .39059 L
.23333 .22526 L
.23651 .37337 L
.23968 .25411 L
.24286 .35466 L
.24603 .27832 L
.24921 .33468 L
.25238 .29839 L
.25556 .31363 L
.25873 .31478 L
.2619 .29169 L
.26508 .32786 L
.26825 .26902 L
.27143 .338 L
.2746 .24573 L
.27778 .34548 L
.28095 .22194 L
.28413 .35058 L
Mistroke
.2873 .19775 L
.29048 .35355 L
.29365 .17325 L
.29683 .3546 L
.3 .1485 L
.30317 .35392 L
.30635 .12357 L
.30952 .35168 L
.3127 .11339 L
.31587 .34803 L
.31905 .14038 L
.32222 .34312 L
.3254 .16459 L
.32857 .33706 L
.33175 .18625 L
.33492 .32997 L
.3381 .20555 L
.34127 .32195 L
.34444 .22268 L
.34762 .31309 L
.35079 .23781 L
.35397 .30347 L
.35714 .25108 L
.36032 .29316 L
.36349 .26263 L
.36667 .28224 L
.36984 .27259 L
.37302 .27077 L
.37619 .28107 L
.37937 .2588 L
.38254 .28819 L
.38571 .24638 L
.38889 .29405 L
.39206 .23355 L
.39524 .29872 L
.39841 .22037 L
.40159 .3023 L
.40476 .20686 L
.40794 .30487 L
.41111 .19307 L
.41429 .30649 L
.41746 .17903 L
.42063 .30723 L
.42381 .16475 L
.42698 .30715 L
.43016 .15028 L
.43333 .30631 L
.43651 .13564 L
.43968 .30476 L
.44286 .15044 L
Mistroke
.44603 .30255 L
.44921 .16531 L
.45238 .29972 L
.45556 .17897 L
.45873 .29632 L
.4619 .1915 L
.46508 .29238 L
.46825 .20294 L
.47143 .28795 L
.4746 .21337 L
.47778 .28305 L
.48095 .22284 L
.48413 .27771 L
.4873 .2314 L
.49048 .27198 L
.49365 .2391 L
.49683 .26586 L
.5 .24599 L
.50317 .2594 L
.50635 .25212 L
.50952 .25261 L
.5127 .25751 L
.51587 .24551 L
.51905 .26222 L
.52222 .23813 L
.5254 .26628 L
.52857 .23049 L
.53175 .26972 L
.53492 .2226 L
.5381 .27257 L
.54127 .21448 L
.54444 .27487 L
.54762 .20615 L
.55079 .27664 L
.55397 .19763 L
.55714 .27791 L
.56032 .18892 L
.56349 .27871 L
.56667 .18004 L
.56984 .27906 L
.57302 .171 L
.57619 .27899 L
.57937 .16182 L
.58254 .27851 L
.58571 .15321 L
.58889 .27764 L
.59206 .16332 L
.59524 .27641 L
.59841 .17278 L
.60159 .27483 L
Mistroke
.60476 .18161 L
.60794 .27292 L
.61111 .18985 L
.61429 .2707 L
.61746 .19751 L
.62063 .26819 L
.62381 .20463 L
.62698 .26539 L
.63016 .21122 L
.63333 .26232 L
.63651 .21731 L
.63968 .259 L
.64286 .22291 L
.64603 .25543 L
.64921 .22806 L
.65238 .25164 L
.65556 .23276 L
.65873 .24762 L
.6619 .23704 L
.66508 .2434 L
.66825 .24091 L
.67143 .23898 L
.6746 .24439 L
.67778 .23438 L
.68095 .24749 L
.68413 .22959 L
.6873 .25023 L
.69048 .22464 L
.69365 .25263 L
.69683 .21953 L
.7 .2547 L
.70317 .21426 L
.70635 .25644 L
.70952 .20885 L
.7127 .25788 L
.71587 .2033 L
.71905 .25903 L
.72222 .19762 L
.7254 .2599 L
.72857 .19182 L
.73175 .26049 L
.73492 .1859 L
.7381 .26082 L
.74127 .17987 L
.74444 .2609 L
.74762 .17374 L
.75079 .26075 L
.75397 .1675 L
.75714 .26036 L
.76032 .16792 L
Mistroke
.76349 .25974 L
.76667 .17452 L
.76984 .25892 L
.77302 .18075 L
.77619 .25788 L
.77937 .18664 L
.78254 .25665 L
.78571 .19219 L
.78889 .25524 L
.79206 .19742 L
.79524 .25363 L
.79841 .20234 L
.80159 .25185 L
.80476 .20695 L
.80794 .24991 L
.81111 .21126 L
.81429 .2478 L
.81746 .2153 L
.82063 .24553 L
.82381 .21906 L
.82698 .24311 L
.83016 .22255 L
.83333 .24055 L
.83651 .22578 L
.83968 .23785 L
.84286 .22877 L
.84603 .23502 L
.84921 .23151 L
.85238 .23205 L
.85556 .23403 L
.85873 .22896 L
.8619 .23631 L
.86508 .22576 L
.86825 .23838 L
.87143 .22244 L
.8746 .24024 L
.87778 .21901 L
.88095 .24189 L
.88413 .21547 L
.8873 .24334 L
.89048 .21183 L
.89365 .2446 L
.89683 .20809 L
.9 .24567 L
.90317 .20426 L
.90635 .24656 L
.90952 .20034 L
.9127 .24728 L
.91587 .19633 L
.91905 .24783 L
Mistroke
.92222 .19224 L
.9254 .24821 L
.92857 .18807 L
.93175 .24843 L
.93492 .18382 L
.9381 .2485 L
.94127 .1795 L
.94444 .24842 L
.94762 .1751 L
.95079 .2482 L
.95397 .17419 L
.95714 .24783 L
.96032 .17887 L
.96349 .24733 L
.96667 .18333 L
.96984 .2467 L
.97302 .18757 L
.97619 .24594 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00OogooKWoo003oOom^
Ool0059oo`<000Aoo`8002Yoo`<000Aoo`8000Aoo`8002Ioo`<000=oo`<000Aoo`8002Moo`@000=o
o`8000Aoo`8002Ioo`@0009oo`<000Aoo`8002Moo`<000Aoo`8000Aoo`8000=oo`00EGoo00@007oo
Ool0009oo`03001oogoo02Qoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo00<0
07ooOol01Goo00@007ooOol0009oo`03001oogoo02Eoo`03001oogoo009oo`04001oogoo0002Ool0
1000Oomoo`009Woo00<007ooOol01Goo00@007ooOol0009oo`03001oogoo02Moo`04001oogoo0002
Ool01000Oomoo`000Woo0@000Goo0Goo001EOol01000Oomoo`000Woo00<007ooOol0:7oo00<007oo
Ool00Woo00@007ooOol0009oo`04001oogoo000VOol00`00Oomoo`05Ool01000Oomoo`000Woo00<0
07ooOol09Woo00D007ooOomoo`00009oo`04001oogoo0002Ool00`00Oomoo`0UOol00`00Oomoo`04
Ool01000Oomoo`000Woo00<007ooOol09goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1
Ool0059oo`<000=oo`04001oogoo000ZOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol0
02Ioo`03001oogoo009oo`<000=oo`04001oogoo000YOol01000Oomoo`000Woo00@007ooOol0009o
o`03001oogoo02Ioo`03001oogoo00<000=oo`04001oogoo000WOol20003Ool01000Oomoo`000Woo
00@007ooOol0009oo`00DWoo00<007ooOol00goo00@007ooOol002Yoo`03001oogoo009oo`04001o
ogoo0002Ool01000Oomoo`009Woo00<007ooOol00Woo00<007ooOol00goo00@007ooOol002Ioo`04
001oogoo0002Ool01000Oomoo`000Woo00@007ooOol002Eoo`04001oogoo0002Ool00`00Oomoo`03
Ool01000Oomoo`00:Goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1Ool0059oo`@000=o
o`8002Yoo`8000Eoo`8000Aoo`8002Ioo`8000Aoo`@000=oo`8002Qoo`8000Aoo`8000Aoo`8002Mo
o`8000=oo`@000=oo`8002Moo`<000Aoo`8000Aoo`8000=oo`00ogooKWoo003oOom^Ool00?moofio
o`006Wooo`00D`000Goo000ROol00`00Oomoo`0>Ool00`00Oomoo`0?Ool00`00Oomoo`0?Ool00`00
Oomoo`0cOol00`00Oomoo`0bOol00`00Oomoo`0cOol00`00Oomoo`0bOol00`00Oomoo`0cOol00`00
Oomoo`07Ool0029oo`03001oogoo00ioo`03001oogoo00moo`03001oogoo0?moobIoo`008Woo00<0
07ooOol03Woo00<007ooOol03goo00<007ooOol0ogoo9Woo000ROol2000?Ool00`00Oomoo`0=Ool0
0`00Ool0003oOolXOol0029oo`03001oogoo00ioo`03001oogoo00eoo`03001oo`000?moobQoo`00
8Woo00<007ooOol03Woo00<007ooOol03Goo00<007oo0000ogoo:7oo000ROol00`00Oomoo`0>Ool0
0`00Oomoo`0=Ool00`00Ool0000JOol00`00Oomoo`3oOol;Ool0029oo`03001oogoo00ioo`03001o
ogoo00eoo`03001oo`0001Yoo`03001oogoo0?moo`]oo`008Woo0P003goo00<007ooOol03Goo00<0
07oo00006Woo00<007ooOol0ogoo2goo000ROol00`00Oomoo`0>Ool00`00Oomoo`0=Ool01@00Ool0
07oo000067oo00<007ooOol0ogoo2goo000ROol00`00Oomoo`0>Ool00`00Oomoo`0=Ool01@00Ool0
07oo000067oo00<007ooOol0ogoo2goo000ROol00`00Oomoo`0>Ool00`00Oomoo`0=Ool01@00Ool0
07oo000067oo00<007ooOol0ogoo2goo000ROol2000?Ool00`00Oomoo`0=Ool01@00Ool007oo0000
67oo00<007ooOol0ogoo2goo000ROol00`00Oomoo`0>Ool00`00Oomoo`0=Ool01@00Ool007oo0000
5Woo00<007oo0000ogoo3Goo0002Ool20004Ool20004Ool00`00Oomoo`03Ool30002Ool40005Ool0
0`00Oomoo`0>Ool00`00Oomoo`0=Ool01@00Ool007oo00005Woo00D007oo001oo`000?moo`]oo`00
00Eoo`00Oomoo`0000=oo`8000Aoo`03001oogoo00Aoo`03001oogoo009oo`03001oogoo00Eoo`03
001oogoo00ioo`03001oogoo00eoo`05001oo`00Ool0000FOol01@00Ool007oo000087oo00<007oo
Ool0igoo00001Goo001oogoo00002Woo00@007ooOomoo`@000Aoo`03001oogoo00Aoo`<000ioo`03
001oogoo00eoo`05001oo`00Ool0000FOol01@00Ool007oo00007Woo00<007oo0000jGoo00001Goo
001oogoo00002Woo00L007ooOomoo`00Ool00006Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00
Oomoo`0=Ool01@00Ool007oo00005Woo00D007oo001oo`0001ioo`03001oo`000>Uoo`0000Eoo`00
Oomoo`0000]oo`06001oogoo001oo`000goo00@007ooOol000Eoo`03001oogoo00ioo`03001oogoo
00eoo`05001oo`00Ool0000FOol01@00Ool007oo00007Woo00<007oo0000:Goo00<007ooOol0_Goo
0002Ool20009Ool40003Ool20004Ool20006Ool00`00Oomoo`0>Ool00`00Oomoo`0=Ool01@00Ool0
07oo00005Woo00D007oo001oo`0001ioo`05001oo`00Ool0000WOol00`00Oomoo`2mOol0029oo`03
001oogoo00ioo`03001oogoo00]oo`07001oo`00Ool007oo000057oo00T007oo001oo`00Ool007oo
00006Woo00L007oo001oo`00Ool0000UOol01@00Ool007oo0000_Goo000ROol2000?Ool00`00Oomo
o`0;Ool01`00Ool007oo001oo`0001Aoo`09001oo`00Ool007oo001oo`0001Yoo`07001oo`00Ool0
07oo00009Goo00D007oo001oo`0002moo`03001oogoo08]oo`008Woo00<007ooOol03Woo00<007oo
Ool02goo00L007oo001oo`00Ool0000DOol02@00Ool007oo001oo`00Ool0000JOol02@00Ool007oo
001oo`00Ool0000QOol02@00Ool007oo001oo`00Ool0000[Ool01@00Ool007oo0000Rgoo000ROol0
0`00Oomoo`0>Ool00`00Oomoo`0;Ool02@00Ool007oo001oo`00Ool0000BOol02@00Ool007oo001o
o`00Ool0000HOol02`00Ool007oo001oo`00Ool007oo00008Goo00T007oo001oo`00Ool007oo0000
:Goo00L007oo001oo`00Ool00002Ool00`00Oomoo`0aOol00`00Ool0001BOol0029oo`03001oogoo
00ioo`03001oo`0000]oo`09001oo`00Ool007oo001oo`00019oo`09001oo`00Ool007oo001oo`00
01Qoo`0;001oo`00Ool007oo001oo`00Ool0000OOol03@00Ool007oo001oo`00Ool007oo001oo`00
02Eoo`09001oo`00Ool007oo001oo`00009oo`03001oogoo02eoo`09001oo`00Ool007oo001oo`00
03Yoo`03001oo`00009oo`03001oo`0000ioo`008Woo0P003goo00<007oo00002goo00T007oo001o
o`00Ool007oo00004Woo00T007oo001oo`00Ool007oo000067oo00/007oo001oo`00Ool007oo001o
o`00009oo`03001oogoo01Yoo`0=001oo`00Ool007oo001oo`00Ool007oo00009Goo00T007oo001o
o`00Ool007oo00000Woo00<007oo0000:goo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool0
0`00Oomoo`0aOol01@00Ool007oo00000Woo00D007oo001oo`0000aoo`008Woo00<007ooOol03Woo
00<007oo00002goo00T007oo001oo`00Ool007oo00004Woo00/007oo001oo`00Ool007oo001oo`00
01Ioo`0;001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`0GOol04000Oomoo`00Ool007oo
001oo`00Ool007oo001oo`000Woo00<007ooOol07Woo00/007oo001oo`00Ool007oo001oo`00009o
o`05001oo`00Ool0000VOol04000Oomoo`00Ool007oo001oo`00Ool007oo001oo`000Woo00<007oo
0000;Goo00T007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0000:Ool0029oo`03001o
ogoo00ioo`03001oo`0000]oo`09001oo`00Ool007oo001oo`0000moo`0>001oogoo001oo`00Ool0
07oo001oo`00Ool001Aoo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol05goo
010007ooOol007oo001oo`00Ool007oo001oo`00Ool0009oo`03001oogoo01aoo`0=001oo`00Ool0
07oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0000ROol00`00Ool00002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0000WOol03P00Ool007oo001oo`00Ool0
07oo001oo`00Ool200001Woo001oo`00Ool000Yoo`008Woo00<007ooOol03Gooo`00<`002Goo000R
Ool00`00Oomoo`0>Ool00`00Ool0000;Ool02@00Ool007oo001oo`00Ool0000?Ool03P00Oomoo`00
Ool007oo001oo`00Ool007oo000DOol03000Ool007oo001oo`00Ool007oo001oo`800004Ool007oo
000EOol01000Ool007oo0P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool01@00Ool007oo0000
5Goo00<007oo00000Woo00h007oo001oo`00Ool007oo001oo`00Ool007oo0P0000Yoo`00Ool007oo
001oo`00Ool001Qoo`08001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`005Woo00L007oo001oo`00Ool00002Ool0
3P00Ool007oo001oo`00Ool007oo001oo`00Ool200001Woo001oo`00Ool000Yoo`008Woo0P003goo
00<007oo00002goo00T007oo001oo`00Ool007oo00003goo00h007ooOol007oo001oo`00Ool007oo
001oo`004Goo00l007ooOol007oo001oo`00Ool007oo001oo`00Ool00P0000Aoo`00Ool001=oo`06
001oo`00Ool007oo0P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool01@00Ool007oo00005Goo
00<007oo00000Woo00X007oo001oo`00Ool007oo001oo`<0000?Ool00000Ool007oo001oo`00Ool0
07oo001oo`0001Aoo`0:001oo`00Ool007oo001oo`00Ool200001Goo001oo`00Ool00P0000=oo`00
Ool00P00015oo`00001oo`00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oogoo00eoo`0<
001oo`00Ool007oo001oo`00Ool007oo0P0000Eoo`00Ool007oo00800005Ool007oo001oo`020000
1Goo001oo`00Ool00P002Woo000ROol00`00Oomoo`0>Ool00`00Ool0000;Ool02@00Ool007oo001o
o`00Ool0000?Ool04000Ool00000Ool007oo001oo`00Ool007oo001oo`003goo00l007ooOol007oo
001oo`00Ool007oo001oo`00Ool00P0000Ioo`00Ool007oo000AOol01P00Ool007oo001oo`800003
Ool007oo00800008Ool00000Ool007oo001oo`800006Ool007oo001oo`004Goo00H007oo001oo`00
Ool200001Goo001oo`00Ool00P0000Eoo`00Ool007oo0080000=Ool00000Ool007oo001oo`00Ool0
07oo00000Woo00<007oo00003Goo00`007oo001oo`00Ool007oo001oo`00Ool300003Goo001oo`00
Ool007oo001oo`00Ool007oo00800003Ool007oo00800007Ool007oo001oo`00Ool00P0000Qoo`00
Ool007oo001oo`0017oo01l007oo000007oo001oo`00Ool007oo001oo`00Ool007oo001oo`00Ool0
07oo001oo`00Ool007oo001oo`02000027oo001oo`00Ool007oo0009Ool0009oo`8000Aoo`8000Ao
o`03001oogoo00=oo`<000=oo`<000Eoo`03001oogoo00ioo`03001oo`0000]oo`09001oo`00Ool0
07oo001oo`0000moo`0@001oo`00001oo`00Ool007oo001oo`00Ool007oo000?Ool02`00Oomoo`00
Ool007oo001oo`00Ool00`0000Uoo`00001oo`00Ool007oo00003goo00P007oo001oo`00Ool007oo
0P0000Qoo`00001oo`00Ool007oo0P0000=oo`00Ool00P0000Qoo`00Ool007oo001oo`003Goo00P0
07oo001oo`00Ool007oo0P0000aoo`00001oo`00Ool007oo001oo`00Ool200000goo001oo`040000
1Woo001oo`00Ool0009oo`05001oo`00Ool00006Ool00`00Ool00002Ool02000Ool007oo001oo`00
Ool200000goo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000?Ool007oo001o
o`00Ool007oo001oo`00Ool007oo00<00003Ool007oo00D00003Ool007oo0080000;Ool007oo001o
o`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool01`00Ool007oo
001oo`0000Uoo`0000Eoo`00Oomoo`0000=oo`8000Aoo`03001oogoo00Aoo`03001oogoo00=oo`03
001oogoo00Aoo`03001oogoo00ioo`03001oo`0000]oo`09001oo`00Ool007oo001oo`0000moo`0@
001oo`00001oo`00Ool007oo001oo`00Ool007oo000?Ool01`00Oomoo`00Ool007oo00800003Ool0
07oo0080000;Ool00000Ool007oo001oo`00Ool0000=Ool02000Ool007oo001oo`00Ool300002Goo
001oo`00Ool007oo001oo`0500002Woo001oo`00Ool007oo001oo`002Goo01P007oo001oo`00Ool0
07oo001oo`00Ool007oo001oo`00Ool007oo001oo`00Ool200002Goo001oo`00Ool007oo001oo`04
000027oo000007oo001oo`00Ool400000goo001oo`0500002goo001oo`00Ool007oo001oo`00Ool0
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool0
07oo00800006Ool007oo001oo`002Goo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00
Ool007oo001oo`00Ool007oo001oo`00009oo`07001oo`00Ool007oo00002Goo00001Goo001oogoo
00002Woo00@007ooOomoo`@0009oo`@000Eoo`<000ioo`03001oo`0000]oo`0;001oo`00Ool007oo
001oo`00Ool0000;Ool01000Ool007oo0P0000Aoo`00001oo`800006Ool007oo001oo`003Goo00<0
07oo00000Woo00@007oo001oo`800005Ool007oo001oo`03000027oo001oo`00Ool007oo000;Ool0
6000Ool007oo001oo`00Ool007oo001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool00000
Ool007oo001oo`00Ool007oo00001Goo00H007oo001oo`00Ool200000goo001oo`0200003Goo001o
o`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Io
o`00Ool007oo0004Ool00`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool00`00Oomoo`0EOol01@00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool01`00Ool007oo001oo`0000Uoo`0000Eoo`00Oomoo`0000Yoo`07001oogooOol007oo00000goo
00<007oo00001Woo00<007ooOol03Woo00<007oo00002goo00/007oo001oo`00Ool007oo001oo`00
00]oo`04001oo`00Ool2000017oo000007oo0P0000Ioo`00Ool007oo0002Ool00`00Oomoo`08Ool0
0`00Ool00002Ool02`00Ool00000Ool007oo001oo`00Ool00`0000Yoo`00Ool007oo001oo`00Ool0
00Uoo`06001oo`00Ool007oo0P00011oo`00001oo`00Ool007oo001oo`00Ool007oo001oo`800005
Ool007oo001oo`0400001Goo001oo`00Ool00`0000=oo`0000000P0000Moo`00Ool007oo001oo`02
00003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03000Ool007oo001oo`00Ool007oo001o
o`800004Ool007oo000=Ool02`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool00014Ool00005Ool007ooOol0000;Ool0
1P00Oomoo`00Ool000=oo`03001oo`0000Ioo`03001oogoo00ioo`03001oo`0000]oo`0;001oo`00
Ool007oo001oo`00Ool0000;Ool01000Ool007oo0P0000Aoo`00001oo`800006Ool007oo001oo`00
0Woo00<007ooOol027oo00L007oo000007oo001oo`0200002goo001oo`00Ool007oo001oo`00Ool0
0`0000Qoo`00Ool007oo001oo`001Goo00@007oo001oo`@00003Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000goo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo000067oo00D007oo001oo`00009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00<007ooOol0BWoo0002Ool20009Ool40003Ool20004Ool20006Ool0
0`00Oomoo`0>Ool00`00Ool0000;Ool02`00Ool007oo001oo`00Ool007oo00002goo00@007oo001o
o`800004Ool00000Ool4000017oo001oo`000Woo00<007ooOol01Woo00@007oo001oo`80000>Ool0
0000Ool007oo001oo`00Ool007oo001oo`800003Ool0000000800006Ool007oo001oo`000Woo0P00
00=oo`00Ool00P0000Moo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000<Ool007oo001oo`00Ool007oo001oo`002Goo00T007oo001oo`00Ool007oo00000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0001oOol0029oo`80
00moo`03001oo`0000Uoo`0=001oo`00Ool007oo001oo`00Ool007oo00002goo00@007oo001oo`<0
0005Ool007oo001oo`02000017oo001oo`000Woo00<007oo00001Woo00@007oo001oo`80000>Ool0
0000Ool007oo001oo`00Ool007oo001oo`800003Ool007oo00H00005Ool00000001oo`0400002Woo
001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool0
07oo001oo`00Ool0000?Ool01@00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool00`00Ool00025Ool0029oo`03001oogoo00ioo`03001oo`0000Uoo`0=001oo`00Ool007oo
001oo`00Ool007oo00002goo00@007oo001oo`<00005Ool007oo001oo`0400001goo001oogoo001o
o`0000Ioo`04001oo`00Ool300003Goo001oo`00Ool007oo001oo`00Ool007oo0080000<Ool007oo
001oo`00Ool007oo001oo`000goo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0000LOol01`00Ool007oo001oo`0008io
o`008Woo00<007ooOol03Woo00<007oo00002Goo00d007oo001oo`00Ool007oo001oo`00Ool0000;
Ool01000Ool007oo0`0000Moo`00Ool007oo001oo`0200000goo001oo`0200000goo001oo`03Ool0
1000Ool007oo0P00011oo`00001oo`00Ool007oo001oo`00Ool007oo001oo`80000:Ool007oo001o
o`00Ool007oo0007Ool02`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool00`00Ool0002cOol0029oo`03001oogoo00ioo`03001oo`0000Uoo`0=001o
o`00Ool007oo001oo`00Ool007oo00002Goo00H007oo001oo`00Ool300001goo001oo`00Ool007oo
00@00007Ool00000Ool007oo00000Woo00<007oo00000`00011oo`00001oo`00Ool007oo001oo`00
Ool007oo001oo`80000:Ool007oo001oo`00Ool007oo0009Ool02@00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`000;Qoo`008Woo00<007ooOol03Woo00<007oo0000
2Goo00H007oo001oo`00Ool3000017oo001oo`002Goo018007oo001oo`00Ool007oo001oo`00Ool0
07oo001oo`00Ool5000017oo001oo`000Woo100000=oo`00Ool00P0000aoo`00Ool007oo001oo`00
Ool007oo0002Ool02@00Ool007oo001oo`00Ool0000?Ool01@00Ool007oo00000Woo00/007oo001o
o`00Ool007oo001oo`000;Yoo`008Woo0P003goo00<007oo00002Goo00H007oo001oo`00Ool30000
0goo001oo`0200000goo001oo`05Ool04P00Ool007oo001oo`00Ool007oo001oo`00Ool007oo001o
o`@00003Ool0000000H00005Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009o
o`07001oo`00Ool007oo000067oo00<007oo0000`Woo000ROol00`00Oomoo`0=Ool01000Ool00000
2Goo00@007oo001oo`800006Ool00000Ool007oo0P0000=oo`00Ool01Goo01@007oo001oo`00Ool0
07oo001oo`00Ool007oo001oo`00Ool007oo0P0000=oo`00Ool0100000Moo`00Ool007oo001oo`02
000037oo001oo`00Ool007oo001oo`00Ool0009oo`05001oo`00Ool0003OOol0029oo`03001oogoo
00eoo`04001oo`000009Ool01000Ool007oo0P0000=oo`0000000P0000Eoo`00001oo`0000Ioo`05
001oo`00001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00800004Ool007oo0004Ool0
1@00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000
ggoo000ROol00`00Oomoo`0=Ool01000Ool000002Goo00L007oo000007oo001oo`0400001Goo0000
07oo00001Woo00D007oo000007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Ao
o`00Ool000Aoo`05001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03
001oo`000>5oo`008Woo0P003Woo00@007oo000000Uoo`09001oo`00001oo`00Ool007oo00800005
Ool00000Ool00006Ool01@00Ool00000Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
000017oo001oo`001Woo00<007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool0
0`00Oomoo`3QOol0029oo`03001oogoo00eoo`04001oo`000009Ool02@00Ool00000Ool007oo001o
o`0200001Goo000007oo000017oo00L007oo000007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo00800003Ool007oo00Uoo`0@001oogoo001oo`00Ool007oo001oo`00Ool007oo003V
Ool0029oo`03001oogoo00eoo`04001oo`000009Ool02@00Ool00000Ool007oo001oo`05000017oo
001oo`000Woo00L007oo000007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002
Ool00`00Oomoo`0;Ool02@00Ool007oo001oo`00Ool0003ZOol0009oo`8000Aoo`8000Aoo`03001o
ogoo00=oo`<000=oo`8000Ioo`03001oogoo00eoo`04001oo`000009Ool02@00Ool00000Ool007oo
001oo`05000017oo001oo`000Woo100000=oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo
001oo`00ogoo17oo00001Goo001oogoo00000goo0P0017oo00<007ooOol017oo00D007ooOomoo`00
009oo`03001oogoo00=oo`03001oogoo00eoo`04001oo`000009Ool02@00Ool00000Ool007oo001o
o`05000017oo001oo`000Woo0P0000Eoo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001o
o`00Ool00?moo`Aoo`0000Eoo`00Oomoo`0000Yoo`04001oogooOol40002Ool01000Oomoo`001Goo
0`003Goo00@007oo000000Uoo`0;001oo`00001oo`00Ool007oo001oo`0300001Goo001oo`00Ool0
0`0000Ioo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?moo`Aoo`0000Eo
o`00Oomoo`0000Yoo`07001oogooOol007oo00000goo0`001Woo00<007ooOol03Goo00H007oo0000
07oo0007Ool02`00Ool00000Ool007oo001oo`00Ool00`0000=oo`0000001@0000Ioo`00Ool007oo
0002Ool02`00Ool007oo001oo`00Ool007oo0000ogoo1Woo00001Goo001oogoo00002goo00H007oo
Ool007oo0003Ool00`00Oomoo`06Ool00`00Oomoo`0=Ool01P00Ool00000Ool000Moo`800009Ool0
07oo001oo`00Ool007oo00<00003Ool0000000<00008Ool007oo001oo`00Ool0009oo`0;001oo`00
Ool007oo001oo`00Ool0003oOol6Ool0009oo`8000Uoo`@000=oo`8000Aoo`<000Eoo`03001oogoo
00eoo`06001oo`00001oo`001goo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`040002Ool0
1`00Ool007oo001oo`00009oo`0;001oo`00Ool007oo001oo`00Ool0003oOol6Ool0029oo`8000io
o`04001oo`00Ool20004Ool00`00Oomoo`0200002goo001oo`00Ool007oo001oo`00Ool00P0000=o
o`00Ool00goo00L007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0003oOol8Ool0029o
o`03001oogoo00eoo`04001oo`00Ool20004Ool00`00Oomoo`0200002goo001oo`00Ool007oo001o
o`00Ool00P0000=oo`00Ool00goo00L007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0
003oOol8Ool0029oo`03001oogoo00eoo`04001oo`00Ool20004Ool00`00Oomoo`0200002goo001o
o`00Ool007oo001oo`00Ool00P0000=oo`00Ool00goo00L007oo001oo`00Ool00002Ool01`00Ool0
07oo001oo`000?moo`Yoo`008Woo00<007ooOol03Goo00@007oo001oo`8000Aoo`03001oogoo0080
000;Ool007oo001oo`00Ool007oo001oo`0200000goo001oo`05Ool01@00Ool007oo00000Woo00L0
07oo001oo`00Ool0003oOol:Ool0029oo`03001oogoo00eoo`04001oo`00Ool20004Ool00`00Oomo
o`0200002Woo001oo`00Ool007oo001oo`000Woo00<007oo00001Woo00D007oo001oo`00009oo`05
001oo`00Ool0003oOol<Ool0029oo`8000ioo`04001oo`00Ool20004Ool00`00Oomoo`0200002Woo
001oo`00Ool007oo001oo`000Woo00<007oo000027oo00<007oo00000Woo00<007oo0000ogoo3Woo
000ROol00`00Oomoo`0=Ool01000Ool007oo0P0017oo00<007ooOol00P0000Yoo`00Ool007oo001o
o`00Ool0009oo`03001oogoo00Yoo`04001oogoo003oOol@Ool0029oo`03001oogoo00eoo`04001o
o`00Ool20004Ool03`00Ool007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`3oOolN
Ool0029oo`03001oogoo00eoo`04001oo`00Ool20004Ool03`00Ool007oo001oo`00Ool007oo001o
o`00Ool00002Ool00`00Oomoo`3oOolNOol0029oo`8000ioo`04001oo`00Ool20004Ool3000037oo
001oo`00Ool007oo001oo`00Ool0009oo`03001oogoo0?mooaioo`008Woo00<007ooOol03Goo00@0
07oo001oo`8000Aoo`<0000<Ool007oo001oo`00Ool007oo001oo`000Woo00<007ooOol0ogoo7Woo
0002Ool20004Ool20004Ool00`00Oomoo`03Ool30003Ool20006Ool00`00Oomoo`0=Ool01000Ool0
07oo0P0017oo0`0000aoo`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`3oOolNOol00005
Ool007ooOol00003Ool20004Ool00`00Oomoo`04Ool01@00Oomoogoo00000Woo00<007ooOol00goo
00<007ooOol03Goo00@007oo001oo`8000Aoo`<0000<Ool007oo001oo`00Ool007oo001oo`000Woo
00<007ooOol0ogoo7Woo00001Goo001oogoo00002Woo00@007ooOomoo`@0009oo`04001oogoo0005
Ool3000=Ool01000Ool007oo0P0017oo0`0000aoo`00Ool007oo001oo`00Ool007oo003oOolSOol0
0005Ool007ooOol0000:Ool01`00Oomoogoo001oo`0000Aoo`8000Ioo`03001oogoo00eoo`04001o
o`00Ool20003Ool04000Ool00000Ool007oo001oo`00Ool007oo001oo`00ogoo8goo00001Goo001o
ogoo00002goo00H007ooOol007oo0003Ool01000Oomoo`001Goo00<007ooOol03Goo00@007oo001o
o`@00003Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`00ogoo8goo0002Ool20009Ool4
0003Ool20004Ool20006Ool00`00Oomoo`0=Ool01000Ool007oo100000=oo`00Ool00P0000aoo`00
Ool007oo001oo`00Ool007oo003oOolSOol0029oo`03001oogoo00eoo`04001oo`00Ool400000goo
001oo`02000037oo001oo`00Ool007oo001oo`00Ool00?moob=oo`008Woo0P003Woo00@007oo001o
o`H0000?Ool00000Ool007oo001oo`00Ool007oo001oo`000?moob=oo`008Woo00<007ooOol03Goo
00@007oo001oo`H0000=Ool00000Ool007oo001oo`00Ool007oo0000ogoo9Goo000ROol00`00Oomo
o`0=Ool01000Ool007oo1P0000eoo`00001oo`00Ool007oo001oo`00Ool0003oOolUOol0029oo`03
001oogoo00eoo`04001oo`00Ool600003Goo000007oo001oo`00Ool007oo001oo`000?moobEoo`00
8Woo0P003Woo00@007oo001oo`H0000=Ool00000Ool007oo001oo`00Ool007oo0000ogoo9Goo000R
Ool00`00Oomoo`0=Ool01000Ool007oo1P0000eoo`00001oo`00Ool007oo001oo`00Ool0003oOolU
Ool0029oo`03001oogoo00eoo`04001oo`00Ool400003Goo001oogoo001oo`00Ool007oo001oo`00
0?moobMoo`008Woo00<007ooOol03Goo00@007oo001oo`@0000=Ool007ooOol007oo001oo`00Ool0
07oo0000ogoo9goo000ROol00`00Oomoo`0=Ool01000Ool007oo100000eoo`00Oomoo`00Ool007oo
001oo`00Ool0003oOolWOol0029oo`8000ioo`04001oo`00Ool40004Ool02@00Ool007oo001oo`00
Ool0003oOolWOol0029oo`03001oogoo00eoo`04001oo`00Ool40004Ool02@00Ool007oo001oo`00
Ool0003oOolWOol000Qoo`8000Aoo`8000Aoo`03001oogoo009oo`<000Ioo`03001oogoo00eoo`04
001oo`00Ool40004Ool01`00Ool007oo001oo`000?moobUoo`001goo00@007ooOol000=oo`8000Ao
o`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo00eoo`04001oo`00Ool40004Ool01`00Ool0
07oo001oo`000?moobUoo`001goo00@007ooOol000Yoo`03001oogoo00Aoo`03001oogoo00=oo`<0
00eoo`04001oo`00Ool40004Ool01`00Ool007oo001oo`000?moobUoo`001goo00@007ooOol000Yo
o`04001oogooOol3003oOomBOol000Moo`04001oogoo000;Ool01000Oomoo`00ogooE7oo0008Ool2
0009Ool40002Ool4003oOomAOol00?moofioo`00\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 0.738778, 
  0.933765, 0.00011328}}]
}, Open  ]],

Cell["Finally the Put with the approximate rule:", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[explotP, \ CRRPlotPApp, \ \ PlotRange\  -> {0.74, \ 0.75}, \ 
      DisplayFunction\  -> \ $DisplayFunction]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -45.7345 61.8034 [
[.18254 -0.0125 -6 -9 ]
[.18254 -0.0125 6 0 ]
[.34127 -0.0125 -9 -9 ]
[.34127 -0.0125 9 0 ]
[.5 -0.0125 -9 -9 ]
[.5 -0.0125 9 0 ]
[.65873 -0.0125 -9 -9 ]
[.65873 -0.0125 9 0 ]
[.81746 -0.0125 -9 -9 ]
[.81746 -0.0125 9 0 ]
[.97619 -0.0125 -9 -9 ]
[.97619 -0.0125 9 0 ]
[.01131 .12361 -30 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -30 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -30 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -24 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 0 m
.18254 .00625 L
s
[(50)] .18254 -0.0125 0 1 Mshowa
.34127 0 m
.34127 .00625 L
s
[(100)] .34127 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(150)] .5 -0.0125 0 1 Mshowa
.65873 0 m
.65873 .00625 L
s
[(200)] .65873 -0.0125 0 1 Mshowa
.81746 0 m
.81746 .00625 L
s
[(250)] .81746 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(300)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.15079 0 m
.15079 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.27778 0 m
.27778 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.53175 0 m
.53175 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.9127 0 m
.9127 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(0.742)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.744)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.746)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.748)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(0.75)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .20223 m
.0746 .20223 L
.07778 .20223 L
.08095 .20223 L
.08413 .20223 L
.0873 .20223 L
.09048 .20223 L
.09365 .20223 L
.09683 .20223 L
.1 .20223 L
.10317 .20223 L
.10635 .20223 L
.10952 .20223 L
.1127 .20223 L
.11587 .20223 L
.11905 .20223 L
.12222 .20223 L
.1254 .20223 L
.12857 .20223 L
.13175 .20223 L
.13492 .20223 L
.1381 .20223 L
.14127 .20223 L
.14444 .20223 L
.14762 .20223 L
.15079 .20223 L
.15397 .20223 L
.15714 .20223 L
.16032 .20223 L
.16349 .20223 L
.16667 .20223 L
.16984 .20223 L
.17302 .20223 L
.17619 .20223 L
.17937 .20223 L
.18254 .20223 L
.18571 .20223 L
.18889 .20223 L
.19206 .20223 L
.19524 .20223 L
.19841 .20223 L
.20159 .20223 L
.20476 .20223 L
.20794 .20223 L
.21111 .20223 L
.21429 .20223 L
.21746 .20223 L
.22063 .20223 L
.22381 .20223 L
.22698 .20223 L
Mistroke
.23016 .20223 L
.23333 .20223 L
.23651 .20223 L
.23968 .20223 L
.24286 .20223 L
.24603 .20223 L
.24921 .20223 L
.25238 .20223 L
.25556 .20223 L
.25873 .20223 L
.2619 .20223 L
.26508 .20223 L
.26825 .20223 L
.27143 .20223 L
.2746 .20223 L
.27778 .20223 L
.28095 .20223 L
.28413 .20223 L
.2873 .20223 L
.29048 .20223 L
.29365 .20223 L
.29683 .20223 L
.3 .20223 L
.30317 .20223 L
.30635 .20223 L
.30952 .20223 L
.3127 .20223 L
.31587 .20223 L
.31905 .20223 L
.32222 .20223 L
.3254 .20223 L
.32857 .20223 L
.33175 .20223 L
.33492 .20223 L
.3381 .20223 L
.34127 .20223 L
.34444 .20223 L
.34762 .20223 L
.35079 .20223 L
.35397 .20223 L
.35714 .20223 L
.36032 .20223 L
.36349 .20223 L
.36667 .20223 L
.36984 .20223 L
.37302 .20223 L
.37619 .20223 L
.37937 .20223 L
.38254 .20223 L
.38571 .20223 L
Mistroke
.38889 .20223 L
.39206 .20223 L
.39524 .20223 L
.39841 .20223 L
.40159 .20223 L
.40476 .20223 L
.40794 .20223 L
.41111 .20223 L
.41429 .20223 L
.41746 .20223 L
.42063 .20223 L
.42381 .20223 L
.42698 .20223 L
.43016 .20223 L
.43333 .20223 L
.43651 .20223 L
.43968 .20223 L
.44286 .20223 L
.44603 .20223 L
.44921 .20223 L
.45238 .20223 L
.45556 .20223 L
.45873 .20223 L
.4619 .20223 L
.46508 .20223 L
.46825 .20223 L
.47143 .20223 L
.4746 .20223 L
.47778 .20223 L
.48095 .20223 L
.48413 .20223 L
.4873 .20223 L
.49048 .20223 L
.49365 .20223 L
.49683 .20223 L
.5 .20223 L
.50317 .20223 L
.50635 .20223 L
.50952 .20223 L
.5127 .20223 L
.51587 .20223 L
.51905 .20223 L
.52222 .20223 L
.5254 .20223 L
.52857 .20223 L
.53175 .20223 L
.53492 .20223 L
.5381 .20223 L
.54127 .20223 L
.54444 .20223 L
Mistroke
.54762 .20223 L
.55079 .20223 L
.55397 .20223 L
.55714 .20223 L
.56032 .20223 L
.56349 .20223 L
.56667 .20223 L
.56984 .20223 L
.57302 .20223 L
.57619 .20223 L
.57937 .20223 L
.58254 .20223 L
.58571 .20223 L
.58889 .20223 L
.59206 .20223 L
.59524 .20223 L
.59841 .20223 L
.60159 .20223 L
.60476 .20223 L
.60794 .20223 L
.61111 .20223 L
.61429 .20223 L
.61746 .20223 L
.62063 .20223 L
.62381 .20223 L
.62698 .20223 L
.63016 .20223 L
.63333 .20223 L
.63651 .20223 L
.63968 .20223 L
.64286 .20223 L
.64603 .20223 L
.64921 .20223 L
.65238 .20223 L
.65556 .20223 L
.65873 .20223 L
.6619 .20223 L
.66508 .20223 L
.66825 .20223 L
.67143 .20223 L
.6746 .20223 L
.67778 .20223 L
.68095 .20223 L
.68413 .20223 L
.6873 .20223 L
.69048 .20223 L
.69365 .20223 L
.69683 .20223 L
.7 .20223 L
.70317 .20223 L
Mistroke
.70635 .20223 L
.70952 .20223 L
.7127 .20223 L
.71587 .20223 L
.71905 .20223 L
.72222 .20223 L
.7254 .20223 L
.72857 .20223 L
.73175 .20223 L
.73492 .20223 L
.7381 .20223 L
.74127 .20223 L
.74444 .20223 L
.74762 .20223 L
.75079 .20223 L
.75397 .20223 L
.75714 .20223 L
.76032 .20223 L
.76349 .20223 L
.76667 .20223 L
.76984 .20223 L
.77302 .20223 L
.77619 .20223 L
.77937 .20223 L
.78254 .20223 L
.78571 .20223 L
.78889 .20223 L
.79206 .20223 L
.79524 .20223 L
.79841 .20223 L
.80159 .20223 L
.80476 .20223 L
.80794 .20223 L
.81111 .20223 L
.81429 .20223 L
.81746 .20223 L
.82063 .20223 L
.82381 .20223 L
.82698 .20223 L
.83016 .20223 L
.83333 .20223 L
.83651 .20223 L
.83968 .20223 L
.84286 .20223 L
.84603 .20223 L
.84921 .20223 L
.85238 .20223 L
.85556 .20223 L
.85873 .20223 L
.8619 .20223 L
Mistroke
.86508 .20223 L
.86825 .20223 L
.87143 .20223 L
.8746 .20223 L
.87778 .20223 L
.88095 .20223 L
.88413 .20223 L
.8873 .20223 L
.89048 .20223 L
.89365 .20223 L
.89683 .20223 L
.9 .20223 L
.90317 .20223 L
.90635 .20223 L
.90952 .20223 L
.9127 .20223 L
.91587 .20223 L
.91905 .20223 L
.92222 .20223 L
.9254 .20223 L
.92857 .20223 L
.93175 .20223 L
.93492 .20223 L
.9381 .20223 L
.94127 .20223 L
.94444 .20223 L
.94762 .20223 L
.95079 .20223 L
.95397 .20223 L
.95714 .20223 L
.96032 .20223 L
.96349 .20223 L
.96667 .20223 L
.96984 .20223 L
.97302 .20223 L
.97619 .20223 L
Mfstroke
.07143 .55933 m
0 0 rlineto s
.07143 .55933 m
.07299 0 L
s
.07634 0 m
.07778 .4794 L
s
.07778 .4794 m
.07993 0 L
s
.08214 0 m
.08413 .38093 L
.0873 .0054 L
.09048 .27433 L
.09365 .16035 L
.09683 .16528 L
.1 .26098 L
.10317 .05693 L
.10635 .32297 L
s
.10635 .32297 m
.10911 0 L
s
.10991 0 m
.1127 .357 L
s
.1127 .357 m
.11493 0 L
s
.11679 0 m
.11905 .37055 L
s
.11905 .37055 m
.12094 0 L
s
.12351 0 m
.1254 .36892 L
s
.1254 .36892 m
.12722 0 L
s
.12995 0 m
.13175 .35598 L
s
.13175 .35598 m
.13402 0 L
s
.13586 0 m
.1381 .33459 L
s
.1381 .33459 m
.14097 0 L
s
.14159 0 m
.14444 .30685 L
.14762 .05069 L
.15079 .27437 L
.15397 .11845 L
.15714 .23838 L
.16032 .17165 L
.16349 .19982 L
.16667 .21277 L
.16984 .15939 L
.17302 .2438 L
.17619 .11769 L
.17937 .26635 L
.18254 .07513 L
.18571 .28177 L
.18889 .03207 L
.19206 .29117 L
s
.19206 .29117 m
.19512 0 L
s
.19535 0 m
.19841 .29545 L
s
.19841 .29545 m
.20109 0 L
s
.20208 0 m
.20476 .2954 L
s
.20476 .2954 m
.20726 0 L
s
.20862 0 m
.21111 .29164 L
s
.21111 .29164 m
.21406 0 L
s
.21451 0 m
.21746 .28474 L
.22063 .02798 L
.22381 .27515 L
.22698 .07101 L
.23016 .26326 L
.23333 .10802 L
.23651 .24942 L
.23968 .13968 L
.24286 .2339 L
.24603 .16656 L
.24921 .21696 L
.25238 .1892 L
.25556 .19881 L
.25873 .20803 L
.2619 .17963 L
.26508 .22346 L
.26825 .15958 L
.27143 .23584 L
.2746 .1388 L
.27778 .24548 L
.28095 .11741 L
.28413 .25266 L
.2873 .09551 L
.29048 .25762 L
.29365 .07321 L
.29683 .26057 L
.3 .05056 L
.30317 .26173 L
.30635 .02765 L
.30952 .26126 L
.3127 .0292 L
.31587 .25932 L
.31905 .05764 L
.32222 .25604 L
.3254 .08325 L
.32857 .25157 L
.33175 .10627 L
.33492 .24601 L
.3381 .12689 L
.34127 .23946 L
.34444 .14529 L
.34762 .23203 L
.35079 .16165 L
.35397 .22379 L
.35714 .17612 L
.36032 .21482 L
.36349 .18883 L
.36667 .20519 L
.36984 .19992 L
Mistroke
.37302 .19496 L
.37619 .2095 L
.37937 .1842 L
.38254 .21769 L
.38571 .17295 L
.38889 .22457 L
.39206 .16126 L
.39524 .23025 L
.39841 .14918 L
.40159 .23481 L
.40476 .13675 L
.40794 .23833 L
.41111 .12399 L
.41429 .24087 L
.41746 .11095 L
.42063 .24251 L
.42381 .09766 L
.42698 .24331 L
.43016 .08414 L
.43333 .24332 L
.43651 .07483 L
.43968 .2426 L
.44286 .09171 L
.44603 .2412 L
.44921 .10729 L
.45238 .23916 L
.45556 .12165 L
.45873 .23653 L
.4619 .13485 L
.46508 .23335 L
.46825 .14696 L
.47143 .22964 L
.4746 .15804 L
.47778 .22546 L
.48095 .16815 L
.48413 .22083 L
.4873 .17733 L
.49048 .21577 L
.49365 .18564 L
.49683 .21032 L
.5 .19313 L
.50317 .20451 L
.50635 .19983 L
.50952 .19835 L
.5127 .2058 L
.51587 .19188 L
.51905 .21107 L
.52222 .18511 L
.5254 .21567 L
.52857 .17806 L
Mistroke
.53175 .21965 L
.53492 .17075 L
.5381 .22302 L
.54127 .1632 L
.54444 .22584 L
.54762 .15543 L
.55079 .22811 L
.55397 .14745 L
.55714 .22988 L
.56032 .13927 L
.56349 .23116 L
.56667 .13092 L
.56984 .23199 L
.57302 .12239 L
.57619 .23237 L
.57937 .11371 L
.58254 .23235 L
.58571 .10922 L
.58889 .23193 L
.59206 .11973 L
.59524 .23114 L
.59841 .12958 L
.60159 .22999 L
.60476 .1388 L
.60794 .22851 L
.61111 .14742 L
.61429 .2267 L
.61746 .15546 L
.62063 .2246 L
.62381 .16294 L
.62698 .2222 L
.63016 .16989 L
.63333 .21952 L
.63651 .17634 L
.63968 .21659 L
.64286 .1823 L
.64603 .2134 L
.64921 .18779 L
.65238 .20998 L
.65556 .19283 L
.65873 .20633 L
.6619 .19744 L
.66508 .20247 L
.66825 .20164 L
.67143 .19841 L
.6746 .20544 L
.67778 .19415 L
.68095 .20886 L
.68413 .18971 L
.6873 .21191 L
Mistroke
.69048 .18509 L
.69365 .21462 L
.69683 .18031 L
.7 .21699 L
.70317 .17537 L
.70635 .21903 L
.70952 .17028 L
.7127 .22077 L
.71587 .16505 L
.71905 .22221 L
.72222 .15968 L
.7254 .22336 L
.72857 .15419 L
.73175 .22423 L
.73492 .14857 L
.7381 .22484 L
.74127 .14283 L
.74444 .22519 L
.74762 .13699 L
.75079 .2253 L
.75397 .13104 L
.75714 .22518 L
.76032 .13415 L
.76349 .22483 L
.76667 .14099 L
.76984 .22426 L
.77302 .14746 L
.77619 .22348 L
.77937 .15358 L
.78254 .2225 L
.78571 .15936 L
.78889 .22133 L
.79206 .16481 L
.79524 .21997 L
.79841 .16995 L
.80159 .21843 L
.80476 .17478 L
.80794 .21671 L
.81111 .17931 L
.81429 .21484 L
.81746 .18356 L
.82063 .2128 L
.82381 .18753 L
.82698 .21061 L
.83016 .19123 L
.83333 .20827 L
.83651 .19467 L
.83968 .20579 L
.84286 .19786 L
.84603 .20317 L
Mistroke
.84921 .2008 L
.85238 .20043 L
.85556 .20351 L
.85873 .19755 L
.8619 .20599 L
.86508 .19455 L
.86825 .20825 L
.87143 .19144 L
.8746 .2103 L
.87778 .18821 L
.88095 .21214 L
.88413 .18488 L
.8873 .21378 L
.89048 .18144 L
.89365 .21522 L
.89683 .17789 L
.9 .21647 L
.90317 .17426 L
.90635 .21755 L
.90952 .17053 L
.9127 .21844 L
.91587 .16671 L
.91905 .21916 L
.92222 .1628 L
.9254 .21972 L
.92857 .15881 L
.93175 .22012 L
.93492 .15475 L
.9381 .22035 L
.94127 .1506 L
.94444 .22044 L
.94762 .14639 L
.95079 .22038 L
.95397 .14736 L
.95714 .22018 L
.96032 .15218 L
.96349 .21984 L
.96667 .15679 L
.96984 .21937 L
.97302 .16118 L
.97619 .21876 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00OogooKWoo003oOom^
Ool0059oo`<000Aoo`8002Yoo`<000Aoo`8000Aoo`8002Ioo`<000=oo`<000Aoo`8002Moo`@000=o
o`8000Aoo`8002Ioo`@0009oo`<000Aoo`8002Moo`<000Aoo`8000Aoo`8000=oo`00EGoo00@007oo
Ool0009oo`03001oogoo02Qoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo00<0
07ooOol01Goo00@007ooOol0009oo`03001oogoo02Eoo`03001oogoo009oo`04001oogoo0002Ool0
1000Oomoo`009Woo00<007ooOol01Goo00@007ooOol0009oo`03001oogoo02Moo`04001oogoo0002
Ool01000Oomoo`000Woo0@000Goo0Goo001EOol01000Oomoo`000Woo00<007ooOol0:7oo00<007oo
Ool00Woo00@007ooOol0009oo`04001oogoo000VOol00`00Oomoo`05Ool01000Oomoo`000Woo00<0
07ooOol09Woo00D007ooOomoo`00009oo`04001oogoo0002Ool00`00Oomoo`0UOol00`00Oomoo`04
Ool01000Oomoo`000Woo00<007ooOol09goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1
Ool0059oo`<000=oo`04001oogoo000ZOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol0
02Ioo`03001oogoo009oo`<000=oo`04001oogoo000YOol01000Oomoo`000Woo00@007ooOol0009o
o`03001oogoo02Ioo`03001oogoo00<000=oo`04001oogoo000WOol20003Ool01000Oomoo`000Woo
00@007ooOol0009oo`00DWoo00<007ooOol00goo00@007ooOol002Yoo`03001oogoo009oo`04001o
ogoo0002Ool01000Oomoo`009Woo00<007ooOol00Woo00<007ooOol00goo00@007ooOol002Ioo`04
001oogoo0002Ool01000Oomoo`000Woo00@007ooOol002Eoo`04001oogoo0002Ool00`00Oomoo`03
Ool01000Oomoo`00:Goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1Ool0059oo`@000=o
o`8002Yoo`8000Eoo`8000Aoo`8002Ioo`8000Aoo`@000=oo`8002Qoo`8000Aoo`8000Aoo`8002Mo
o`8000=oo`@000=oo`8002Moo`<000Aoo`8000Aoo`8000=oo`00ogooKWoo003oOom^Ool00?moofio
o`006Wooo`00D`000Goo000ROol00`00Oomoo`0=Ool2000017oo001oo`001Woo1P0000=oo`000000
0`003Goo00<007ooOol00Woo00L007oo001oo`00Ool0000ZOol00`00Oomoo`0bOol00`00Oomoo`0c
Ool00`00Oomoo`0bOol00`00Oomoo`0cOol00`00Oomoo`07Ool0029oo`03001oogoo00eoo`800004
Ool007oo0006Ool600000goo00000003000BOol01`00Ool007oo001oo`000?moo`]oo`008Woo00<0
07ooOol03Goo0P0000Aoo`00Ool000Ioo`H00003Ool0000000<0019oo`07001oo`00Ool007oo0000
ogoo2goo000ROol2000>Ool2000017oo001oo`001Woo1P0000=oo`0000000`003goo00`007ooOol0
07oo001oo`00Ool007oo000LOol00`00Ool0003YOol0029oo`03001oogoo00eoo`800004Ool007oo
0006Ool600000goo00000003000?Ool03000Oomoo`00Ool007oo001oo`00Ool001aoo`03001oo`00
0>Uoo`008Woo00<007ooOol03Goo0P0000Aoo`00Ool000Ioo`H00003Ool0000000<000moo`0<001o
ogoo001oo`00Ool007oo001oo`0077oo00<007oo0000jGoo000ROol00`00Oomoo`0=Ool2000017oo
001oo`001Woo1P0000=oo`0000000`0000=oo`00Ool037oo00`007ooOol007oo001oo`00Ool007oo
000JOol01@00Ool007oo0000jGoo000ROol00`00Oomoo`0=Ool2000017oo001oo`0017oo00<007oo
00001@0000=oo`0000000`0000=oo`00Ool037oo00`007ooOol007oo001oo`00Ool007oo000JOol0
1`00Ool007oo001oo`000>Moo`008Woo0P003Woo0P0000Aoo`00Ool000Aoo`03001oo`0000D00003
Ool0000000<00003Ool007oo00aoo`0<001oogoo001oo`00Ool007oo001oo`006Woo00L007oo001o
o`00Ool0003WOol0029oo`03001oogoo00eoo`800004Ool007oo0004Ool00`00Ool0000500000goo
0000000300000goo001oo`0:Ool00`00Ool00002Ool02`00Ool007oo001oo`00Ool007oo00005Woo
00T007oo001oo`00Ool007oo00009goo00<007ooOol0_Goo000ROol00`00Oomoo`0=Ool2000017oo
001oo`0017oo00<007oo00001@0000=oo`0000000`0000=oo`00Ool02Woo00@007oo001oo`80000:
Ool007oo001oo`00Ool007oo000FOol02@00Ool007oo001oo`00Ool0000WOol00`00Oomoo`2mOol0
029oo`03001oogoo00eoo`800004Ool007oo0004Ool00`00Ool0000500000goo0000000300000goo
001oo`0:Ool01000Ool007oo0P0000Yoo`00Ool007oo001oo`00Ool001Ioo`0;001oo`00Ool007oo
001oo`00Ool0000SOol00`00Ool0002oOol0029oo`8000ioo`800004Ool007oo0004Ool00`00Ool0
000500000goo0000000300000goo001oo`0:Ool01000Ool007oo0P0000Yoo`00Ool007oo001oo`00
Ool001Aoo`0=001oo`00Ool007oo001oo`00Ool007oo00008goo00D007oo001oo`000;eoo`008Woo
00<007ooOol03Goo0P0000Aoo`00Ool000Aoo`03001oo`0000D00003Ool0000000<00003Ool007oo
00Yoo`04001oo`00Ool200002Woo001oo`00Ool007oo001oo`0057oo00d007oo001oo`00Ool007oo
001oo`00Ool0000QOol01`00Ool007oo001oo`000;eoo`000Woo0P0017oo0P0017oo00<007ooOol0
0goo0`000Woo10001Goo00<007ooOol03Goo0P0000Aoo`00Ool000Aoo`03001oo`0000D00003Ool0
000000<00003Ool007oo00Yoo`04001oo`00Ool2000037oo001oo`00Ool007oo001oo`00Ool0019o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol077oo00T007oo001oo`00Ool0
07oo0000;Goo00<007ooOol0Rgoo00001Goo001oogoo00000goo0P0017oo00<007ooOol017oo00<0
07ooOol00Woo00<007ooOol01Goo00<007ooOol03Goo0P0000Aoo`00Ool000Aoo`03001oo`0000D0
0003Ool0000000<00003Ool0000000Yoo`04001oo`00Ool2000037oo001oo`00Ool007oo001oo`00
Ool000moo`0@001oogoo001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`0JOol02`00
Ool007oo001oo`00Ool007oo0000:goo00<007oo0000SGoo00001Goo001oogoo00002Woo00@007oo
Oomoo`@000Aoo`03001oogoo00Aoo`<000eoo`800004Ool007oo0004Ool00`00Ool0000500000goo
0000000300001Goo000007oo00001Woo00H007oo001oo`00Ool2000037oo001oo`00Ool007oo001o
o`00Ool000moo`0@001oogoo001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`0GOol0
4000Oomoo`00Ool007oo001oo`00Ool007oo001oo`009goo00L007oo001oo`00Ool0002;Ool00005
Ool007ooOol0000:Ool01`00Oomoogoo001oo`0000Ioo`03001oogoo00=oo`03001oogoo00eoo`80
0004Ool007oo0004Ool800000goo0000000300001Goo000007oo00001Woo00H007oo001oo`00Ool2
000037oo001oo`00Ool007oo001oo`00Ool000moo`0@001oogoo001oo`00Ool007oo001oo`00Ool0
07oo0002Ool00`00Ool0000GOol04000Oomoo`00Ool007oo001oo`00Ool007oo001oo`009Goo00T0
07oo001oo`00Ool007oo00000Woo00<007ooOol0<Goo00<007ooOol0DWoo00001Goo001oogoo0000
2goo00H007ooOol007oo0003Ool01000Oomoo`001Goo00<007ooOol03Goo0P0000Aoo`00Ool000Ao
o`P00003Ool0000000<00005Ool00000Ool00006Ool01P00Ool007oo001oo`80000<Ool007oo001o
o`00Ool007oo001oo`003goo00l007ooOol007oo001oo`00Ool007oo001oo`00Ool00P0000Aoo`00
Ool001Eoo`03001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0
87oo00T007oo001oo`00Ool007oo00000Woo00<007ooOol0;goo00D007oo001oo`00059oo`000Woo
0P002Goo10000goo0P0017oo0P001Woo00<007ooOol03Goo0P0000=oo`00Ool00P000goo200000=o
o`0000000`0000Eoo`00001oo`0000Ioo`06001oo`00Ool007oo0P0000aoo`00Ool007oo001oo`00
Ool007oo0002Ool00`00Oomoo`08Ool00`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001o
o`800006Ool007oo001oo`004goo00<007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool00`00Oomoo`0NOol02`00Ool007oo001oo`00Ool007oo00000Woo00<007oo0000;Goo00T0
07oo001oo`00Ool007oo0000D7oo000ROol00`00Oomoo`0=Ool200000goo001oo`020003Ool80000
0goo0000000300001Goo000007oo00001Woo00H007oo001oo`00Ool200000goo001oo`0200001goo
000007oo001oo`00009oo`03001oogoo00Qoo`03001oo`00009oo`04001oo`00Ool200000goo001o
o`0200002Goo000007oo001oo`00Ool0000AOol01P00Ool007oo001oo`80000<Ool007oo001oo`00
Ool007oo001oo`000Woo00<007oo000077oo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool0
1@00Ool007oo0000:Goo00d007oo001oo`00Ool007oo001oo`00Ool0000hOol00`00Ool00002Ool0
0`00Ool0000>Ool0029oo`8000ioo`800003Ool007oo008000=oo`P00003Ool0000000<00005Ool0
0000Ool00004Ool04000Ool007oo001oo`00Ool007oo001oo`00Ool007oo100000Eoo`00Oomoo`00
00Yoo`09001oo`00001oo`00Ool007oo00800003Ool007oo00800009Ool00000Ool007oo001oo`00
015oo`06001oo`00Ool007oo0P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool01@00Ool007oo
00005goo010007ooOol007oo001oo`00Ool007oo001oo`00Ool0009oo`07001oo`00Ool007oo0000
97oo010007ooOol007oo001oo`00Ool007oo001oo`00Ool0009oo`03001oo`0002moo`07001oo`00
Ool007oo00000Woo00D007oo001oo`0000aoo`008Woo00<007ooOol03Goo0P0000=oo`00Ool01@00
00=oo`0000001@0000=oo`0000001P0000=oo`00Ool00goo00@007oo001oo`80000<Ool00000Ool0
07oo001oo`00Ool007oo0P0000Eoo`00Oomoo`0000Qoo`04001oo`00Ool200000goo001oo`020000
1goo001oo`00Ool007oo00<00008Ool007oo001oo`00Ool000eoo`08001oo`00Ool007oo001oo`80
0003Ool007oo00800007Ool00000Ool007oo00000Woo00D007oo001oo`0001Eoo`03001oo`00009o
o`0>001oo`00Ool007oo001oo`00Ool007oo001oo`800008Ool007oo001oo`00Ool001ioo`05001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0000[Ool0
2@00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`0000Yoo`008Woo00<007ooOol03Goo
0P0000=oo`00Ool01@0000=oo`0000001@0000=oo`0000001P0000Aoo`00Ool0009oo`04001oo`00
Ool2000037oo000007oo001oo`00Ool007oo001oo`@0009oo`03001oo`0000Ioo`04001oo`00Ool2
000037oo000007oo001oo`00Ool007oo001oo`<00008Ool007oo001oo`00Ool000]oo`0:001oo`00
Ool007oo001oo`00Ool2000027oo000007oo001oo`00Ool200000goo001oo`0200001Woo001oo`00
Ool0015oo`05001oo`00Ool00002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool200002Woo
001oo`00Ool007oo001oo`006Woo00L007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`07001oo`00Ool007oo00009Goo00h007oo001oo`00Ool007oo001oo`00Ool0
07oo0P0000Ioo`00Ool007oo000:Ool0029oo`03001oogoo00eoo`800003Ool007oo00D00003Ool0
000000D00003Ool0000000800008Ool00000001oo`00Ool0009oo`03001oo`0000<0000>Ool00000
Ool007oo001oo`00Ool007oo001oo`800005Ool00000Ool00004Ool01P00Ool007oo001oo`D0000F
Ool007oo001oo`00Ool007oo001oo`00Ool007oo001oo`00Ool007oo0009Ool02P00Ool007oo001o
o`00Ool007oo0`0000Uoo`00Ool007oo001oo`00Ool00P0000]oo`00001oo`00Ool007oo001oo`00
00eoo`08001oo`00Ool007oo001oo`800005Ool007oo001oo`0200000goo001oo`0200003goo0000
07oo001oo`00Ool007oo001oo`00Ool0000DOol02P00Ool007oo001oo`00Ool007oo0P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool001aoo`03001o
o`00009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`800006Ool007oo001oo`002Woo000R
Ool2000>Ool200000goo001oo`05000017oo000007oo100000Yoo`00001oo`00Ool007oo001oo`<0
009oo`07001oo`00001oo`00Ool00P0000]oo`00Ool007oo001oo`00Ool007oo00D00003Ool007oo
00=oo`04001oo`00Ool2000047oo000007oo001oo`00Ool007oo001oo`00Ool007oo0P0000Uoo`00
001oo`00Ool007oo00001goo00P007oo001oo`00Ool007oo0P00011oo`00001oo`00Ool007oo001o
o`00Ool007oo001oo`<0000:Ool007oo001oo`00Ool007oo0009Ool02P00Ool007oo001oo`00Ool0
07oo0P0000aoo`00001oo`00Ool007oo001oo`00Ool200003Goo000007oo001oo`00Ool007oo001o
o`00009oo`03001oogoo00eoo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000=oo`00Ool00P00
00Eoo`00Ool007oo0080000AOol00000Ool007oo001oo`00Ool007oo001oo`00Ool0000FOol01`00
Ool007oo001oo`00009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`800006Ool007oo001o
o`002Woo000ROol00`00Oomoo`0=Ool200000goo001oo`0500004Woo000007oo001oo`00Ool007oo
001oo`00Ool007oo001oo`T00003Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000Eoo`00001oo`00009oo`03001oo`0000<0000@Ool00000Ool007oo001oo`00Ool007oo001o
o`00Ool200000goo001oo`04000017oo001oo`001Goo00H007oo001oo`00Ool200000goo001oo`02
00003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool00000Ool007oo001oo`00Ool007oo
00001Goo00`007oo001oo`00Ool007oo001oo`00Ool300002goo001oo`00Ool007oo001oo`00Ool0
0P0000Eoo`00Ool007oo00800006Ool007oo001oo`000Woo00D007oo001oo`0000Ioo`03001oo`00
009oo`0<001oo`00Ool007oo001oo`00Ool007oo0`00015oo`00Ool007oo001oo`00Ool007oo001o
o`00Ool007oo00<00008Ool007oo001oo`00Ool0009oo`03001oo`0000eoo`0<001oo`00Ool007oo
001oo`00Ool007oo0P0000=oo`00Ool00P0000Moo`00Ool007oo001oo`0200001Goo001oo`00Ool0
0P002Woo000ROol00`00Oomoo`0=Ool2000017oo000007oo1000019oo`00001oo`00Ool007oo001o
o`00Ool007oo001oo`00Ool700001Goo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200000goo0000000800000goo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
00800005Ool007oo001oo`0600001Goo00000000Ool01P0000Eoo`00Ool007oo0080000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000Eoo`00Ool007oo00@00005Ool007oo001oo`0300001Goo
001oo`00Ool0100000=oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200002goo
001oo`00Ool007oo001oo`00Ool00P0000Uoo`00001oo`00Ool007oo00000Woo00D007oo001oo`00
009oo`<0000:Ool00000Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo
00800007Ool007oo001oo`00Ool00P0000=oo`0000000P0000Uoo`00Ool007oo001oo`00Ool00P00
00=oo`00Ool00P0000Moo`00Ool007oo001oo`0300004Goo001oo`00Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000Qoo`00Ool007oo001oo`002Goo000ROol00`00Oomoo`0=Oooo000c0009Ool0
029oo`03001oogoo00eoo`800004Ool00000Ool4000047oo000007oo001oo`00Ool007oo001oo`00
Ool007oo0P0000aoo`00Oomoo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00
Ool007oo00800004Ool007oo0004Ool01P00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000Yoo`00Ool007oo001oo`00Ool000Eoo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00`007oo001oo`00Ool007oo001oo`00Ool2000037oo001oo`00Ool007oo001oo`00
Ool000Moo`0;001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000Aoo`00Ool000Qoo`0@001oogoo001o
o`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007oo00003goo00/007oo001oo`00Ool007oo
001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool00009
Ool0029oo`8000ioo`800004Ool00000Ool200000goo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo00800003Ool007oo00=oo`06001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00
Ool007oo001oo`02000017oo001oo`0017oo00H007oo001oo`00Ool2000037oo001oo`00Ool007oo
001oo`00Ool0009oo`09001oo`00Ool007oo001oo`0000Uoo`0;001oo`00Ool007oo001oo`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`09001oo`00Ool007oo001oo`0000]o
o`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool0000DOol02@00Ool007oo001oo`00Ool00002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`0;001oo`00Ool007oo001oo`00Ool0000NOol00`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`07001oo`00Ool007oo00002Goo000R
Ool00`00Oomoo`0=Ool200001Goo000007oo00000Woo010007oo000007oo001oo`00Ool007oo001o
o`00Ool0009oo`03001oo`0000Aoo`07001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool00`00Oomoo`04Ool01@00Ool007oo00000Woo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0000;Ool02@00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0000COol01@00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`00029oo`0@
001oogoo001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00Ool0001:Ool0009oo`8000Aoo`80
00Aoo`03001oogoo00=oo`<000=oo`<000Eoo`03001oogoo00eoo`800005Ool00000Ool00002Ool0
4000Ool00000Ool007oo001oo`00Ool007oo001oo`000Woo00<007oo000017oo00L007oo001oo`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oogoo00Ioo`03001oo`00
009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0000?Ool01`00
Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007oo000077oo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`25Ool00005Ool007ooOol00003
Ool20004Ool00`00Oomoo`04Ool00`00Oomoo`03Ool00`00Oomoo`04Ool00`00Oomoo`0=Ool20000
1Goo000007oo00000Woo010007oo000007oo001oo`00Ool007oo001oo`00Ool0009oo`03001oo`00
00Aoo`07001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0000=Ool04000
Oomoo`00Ool007oo001oo`00Ool007oo001oo`000Woo00D007oo001oo`0001Eoo`03001oo`00009o
o`0=001oo`00Ool007oo001oo`00Ool007oo0000^7oo00001Goo001oogoo00002Woo00@007ooOomo
o`@0009oo`@000Eoo`<000eoo`07001oo`00Ool007oo00000Woo010007oo000007oo001oo`00Ool0
07oo001oo`00Ool0009oo`03001oogoo00Aoo`07001oo`00Ool007oo00000Woo00d007oo001oo`00
Ool007oo001oo`00Ool0000=Ool04000Oomoo`00Ool007oo001oo`00Ool007oo001oo`000Woo00<0
07oo0000hGoo00001Goo001oogoo00002Woo00L007ooOomoo`00Ool00003Ool00`00Ool00006Ool0
0`00Oomoo`0=Ool01`00Ool007oo001oo`00009oo`0@001oo`00001oo`00Ool007oo001oo`00Ool0
07oo0002Ool00`00Oomoo`06Ool01@00Ool007oo00000Woo00/007oo001oo`00Ool007oo001oo`00
019oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0hGoo00001Goo001oogoo
00002goo00H007ooOol007oo0003Ool00`00Ool00006Ool00`00Oomoo`0=Ool01`00Ool007oo001o
o`00009oo`0@001oogoo001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`06Ool01@00
Ool007oo00000Woo00/007oo001oo`00Ool007oo001oo`0001Aoo`09001oo`00Ool007oo001oo`00
0>Qoo`000Woo0P002Goo10000goo0P0017oo0P001Woo00<007ooOol03Goo00L007oo001oo`00Ool0
0005Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oogoo00Ioo`05001oo`00Ool0
0002Ool02@00Ool007oo001oo`00Ool0003oOol8Ool0029oo`8000ioo`07001oo`00Ool007oo0000
1Goo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`08Ool00`00Ool00002Ool0
2@00Ool007oo001oo`00Ool0003oOol8Ool0029oo`03001oogoo00eoo`05001oo`00Ool00007Ool0
3@00Ool007oo001oo`00Ool007oo001oo`0000eoo`03001oo`00009oo`07001oo`00Ool007oo0000
ogoo2Woo000ROol00`00Oomoo`0=Ool01@00Ool007oo00001goo00d007oo001oo`00Ool007oo001o
o`00Ool0000?Ool02000Oomoo`00Ool007oo003oOol<Ool0029oo`03001oogoo00eoo`05001oo`00
Ool00007Ool03@00Ool007oo001oo`00Ool007oo001oo`00019oo`03001oo`000?moo`ioo`008Woo
00<007ooOol03Goo00D007oo001oo`0000Moo`0=001oo`00Ool007oo001oo`00Ool007oo0000ogoo
8goo000ROol2000>Ool01@00Ool007oo00001goo00d007oo001oo`00Ool007oo001oo`00Ool0003o
OolSOol0029oo`03001oogoo00eoo`05001oo`00Ool00007Ool02`00Ool007oo001oo`00Ool007oo
0000ogoo9Goo000ROol00`00Oomoo`0=Ool01@00Ool007oo00001goo00/007oo001oo`00Ool007oo
001oo`000?moobEoo`008Woo00<007ooOol03Goo00D007oo001oo`0000Uoo`09001oo`00Ool007oo
001oo`000?moobEoo`008Woo0P003Woo00D007oo001oo`0000Uoo`09001oo`00Ool007oo001oo`00
0?moobEoo`008Woo00<007ooOol03Goo00D007oo001oo`0000Uoo`07001oo`00Ool007oo0000ogoo
9goo000ROol00`00Oomoo`0=Ool01@00Ool007oo00002Goo00L007oo001oo`00Ool0003oOolWOol0
009oo`8000Aoo`8000Aoo`03001oogoo00=oo`<000=oo`8000Ioo`03001oogoo00eoo`05001oo`00
Ool00009Ool01`00Ool007oo001oo`000?moobMoo`0000Eoo`00Oomoo`0000=oo`8000Aoo`03001o
ogoo00Aoo`05001oogooOol00002Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool01@00Ool007oo0000
2goo00<007oo0000ogoo:Goo00001Goo001oogoo00002Woo00@007ooOomoo`@0009oo`04001oogoo
0005Ool3000=Ool01@00Ool007oo00002goo00<007ooOol0ogoo:Goo00001Goo001oogoo00002Woo
00L007ooOomoo`00Ool00003Ool30006Ool00`00Oomoo`0=Ool01@00Ool007oo0000ogoo=goo0000
1Goo001oogoo00002goo00H007ooOol007oo0003Ool00`00Oomoo`06Ool00`00Oomoo`0=Ool00`00
Ool0003oOoliOol0009oo`8000Uoo`@000=oo`8000Aoo`<000Eoo`03001oogoo00eoo`03001oo`00
0?moocUoo`008Woo0P003Woo00<007oo0000ogoo>Goo000ROol00`00Oomoo`0=Ool00`00Ool0003o
OoliOol0029oo`03001oogoo00eoo`03001oo`000?moocUoo`008Woo00<007ooOol03Goo00<007oo
0000ogoo>Goo000ROol00`00Oomoo`0=Ool00`00Ool0003oOoliOol0029oo`8000ioo`03001oo`00
0?moocUoo`008Woo00<007ooOol03Goo00<007oo0000ogoo>Goo000ROol00`00Oomoo`0=Ool00`00
Ool0003oOoliOol0029oo`03001oogoo00eoo`03001oo`000?moocUoo`008Woo0P003Woo00<007oo
0000ogoo>Goo000ROol00`00Oomoo`0=Ool00`00Ool0003oOoliOol0009oo`8000Aoo`8000Aoo`03
001oogoo00=oo`<000=oo`8000Ioo`03001oogoo00eoo`03001oo`000?moocUoo`0000Eoo`00Oomo
o`0000=oo`8000Aoo`03001oogoo00Aoo`05001oogooOol00002Ool00`00Oomoo`03Ool00`00Oomo
o`0=Ool00`00Oomoo`3oOoliOol00005Ool007ooOol0000:Ool01000Oomoogoo10000Woo00@007oo
Ool000Eoo`<000eoo`03001oogoo0?moocUoo`0000Eoo`00Oomoo`0000Yoo`07001oogooOol007oo
000017oo0P001Woo00<007ooOol03Goo00<007ooOol0ogoo>Goo00001Goo001oogoo00002goo00H0
07ooOol007oo0003Ool01000Oomoo`001Goo00<007ooOol03Goo00<007ooOol0ogoo>Goo0002Ool2
0009Ool40003Ool20004Ool20006Ool00`00Oomoo`0=Ool00`00Oomoo`3oOoliOol0029oo`03001o
ogoo00eoo`03001oogoo0?moocUoo`008Woo0P003Woo00<007ooOol0ogoo>Goo000ROol00`00Oomo
o`0=Ool00`00Oomoo`3oOoliOol0029oo`03001oogoo00eoo`03001oogoo0?moocUoo`008Woo00<0
07ooOol03Goo00<007ooOol0ogoo>Goo000ROol2000>Ool00`00Oomoo`3oOoliOol0029oo`03001o
ogoo00eoo`03001oogoo0?moocUoo`008Woo00<007ooOol0ogooBGoo000ROol00`00Oomoo`3oOom9
Ool0029oo`03001oogoo0?moodUoo`008Woo0P00ogooBWoo000ROol00`00Oomoo`3oOom9Ool000Qo
o`8000Aoo`8000Aoo`03001oogoo009oo`<000Ioo`03001oogoo0?moodUoo`001goo00@007ooOol0
00=oo`8000Aoo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo0?moodUoo`001goo00@007oo
Ool000Yoo`03001oogoo00Aoo`03001oogoo00=oo`<00?moodUoo`001goo00@007ooOol000Yoo`04
001oogooOol3003oOomBOol000Moo`04001oogoo000;Ool01000Oomoo`00ogooE7oo0008Ool20009
Ool40002Ool4003oOomAOol00?moofioo`00\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 0.738778, 
  0.933765, 0.00011328}}]
}, Open  ]],

Cell["\<\
These results dramatically illustrate the fact that the \"exact\" \
form of the model is not necessarily better than the approximate form. The \
convergence properties of both are rather odd. This interesting state of \
affairs is somewhat perplexing, but gives weight to the idea that \
verification is a good idea!

However, the convergence of both the exact and the approximate scheme is \
problematic, being highly oscillatory. Even the envelope of the fast \
oscillations does not decrease monotonically. The envelope is also \
asymmetric, so that the common practice of averaging the odd-even \
calculations is not particularly reliable either. The average is shown, \
roughly, by the interference pattern in the plots. In the exact and \
approximate cases it bounces off the exact solution from different \
sides.\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Magic Tree Sizes 1 - Approximate CRR Models", "Subsection",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "There are clearly values of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " where the approximate CRR model is particularly good. We can locate these \
values by hypothesizing that they may have something to do with the location \
of tree nodes at expiry in relation to the strike. Two possibilities stand \
out (and the author has to admit to being slightly surprised by the outcome) \
- one where a node sits right on the strike, and the other where the strike \
is between two nodes. A quick calculation shows that the strike sits on a \
node when"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`n = \(m\^2\ \[Sigma]\^2\ T\)\/\(\(log\^2\)(K\/S)\)\)], 
  "NumberedEquation",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["and the strike is placed half-way between two nodes when", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`n = \(\((m + 1\/2)\)\^2\ \[Sigma]\^2\ \
T\)\/\(\(log\^2\)(K\/S)\)\)], "NumberedEquation",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " functions for these cases are"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(sitCRR[m_, \ \[Sigma]_, \ T_, \ S_, \ K_]\  := \ 
      m^2\ \[Sigma]^2\ T/Log[K/S]^2; \n
    straddleCRR[m_, \ \[Sigma]_, \ T_, \ S_, \ 
        K_]\  := \ \((m + 1/2)\)^2\ \[Sigma]^2\ T/Log[K/S]^2\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Let's evaluate these values (in real form - you would take the \
nearest integer in practice) for the parameters used above:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[sitCRR[m, 0.2, 1, 9, 10], \ {m, \ 1, \ 10}]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({3.60333148400830793`, 14.4133259360332317`, 32.4299833560747785`, 
      57.6533037441329287`, 90.0832871002076984`, 129.719933424299105`, 
      176.563242716407087`, 230.613214976531732`, 291.869850204672953`, 
      360.333148400830793`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[straddleCRR[m, 0.2, 1, 9, 10], \ {m, \ 1, \ 10}]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({8.10749583901869463`, 22.5208217750519246`, 44.140810679101774`, 
      72.9674625511682428`, 109.000777391251313`, 152.240755199351029`, 
      202.687395975467321`, 260.340699719600277`, 325.200666431749851`, 
      397.26729611191601`}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Inspection of the plots makes the point - the latter gives the locations \
where the model is best. This illustrates the point that the time-step of the \
tree should be adapted to the option. It is also a first example of what we \
call a magic tree size. These magic tree sizes are not universal - sometimes \
the size has to be adapted to meet other considerations. For an approximate \
CRR tree they are given by setting ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " to be the integer closest to"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(\((m + 1\/2)\)\^2\ \[Sigma]\^2\ \
T\)\/\(\(log\^2\)(K\/S)\)\)], "NumberedEquation",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "for some integer ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  ". Let's look at the convergence on the straddling cases:"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(CRRdataCMagic\  = \ 
      Table[{k, \ 
          FastCRREuropeanCallApp[9, \ 10, \ 
            Floor[straddleCRR[k, 0.2, 1, 9, 10]], \ 0.1, \ 0, \ 0.2, \ 
            1]}, \ {k, 3, \ 20}]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(CRRdataCMagic\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{3, 0.694858899388745498`}, {4, 0.69468712600119975`}, {5, 
        0.694860609958781427`}, {6, 0.694859206757777059`}, {7, 
        0.694901902695912099`}, {8, 0.694874875385752766`}, {9, 
        0.694882464708190017`}, {10, 0.694891712243472348`}, {11, 
        0.694894958298143539`}, {12, 0.694891427802385308`}, {13, 
        0.69489615830209459`}, {14, 0.694895677523875221`}, {15, 
        0.694890790471641928`}, {16, 0.694894120256560654`}, {17, 
        0.694893045991256474`}, {18, 0.694895400284030451`}, {19, 
        0.694895491713523405`}, {20, 0.6948948802867692`}}\)], "Output"]
}, Open  ]],

Cell["Here is a reminder of the exact value:", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesCall[9, \ 10, \ 0.2, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.694897939192013147`\)], "Output"]
}, Open  ]],

Cell["\<\
This is clearly much more satisfactory. However, it is evident from \
the magic tree size formula that things go wrong with this notion when the \
option is at the money:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(CRRdataATM\  = \ 
      Table[{k, 
          FastCRREuropeanCall[10, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 
            1]}, \ {k, \ 15, \ 300}]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[CRRdataATM, \ PlotJoined\  -> \ True]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -65.0967 49.2705 [
[.18254 .4205 -6 -9 ]
[.18254 .4205 6 0 ]
[.34127 .4205 -9 -9 ]
[.34127 .4205 9 0 ]
[.5 .4205 -9 -9 ]
[.5 .4205 9 0 ]
[.65873 .4205 -9 -9 ]
[.65873 .4205 9 0 ]
[.81746 .4205 -9 -9 ]
[.81746 .4205 9 0 ]
[.97619 .4205 -9 -9 ]
[.97619 .4205 9 0 ]
[.01131 .03884 -30 -4.5 ]
[.01131 .03884 0 4.5 ]
[.01131 .13738 -30 -4.5 ]
[.01131 .13738 0 4.5 ]
[.01131 .23592 -30 -4.5 ]
[.01131 .23592 0 4.5 ]
[.01131 .33446 -30 -4.5 ]
[.01131 .33446 0 4.5 ]
[.01131 .53154 -30 -4.5 ]
[.01131 .53154 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .433 m
.18254 .43925 L
s
[(50)] .18254 .4205 0 1 Mshowa
.34127 .433 m
.34127 .43925 L
s
[(100)] .34127 .4205 0 1 Mshowa
.5 .433 m
.5 .43925 L
s
[(150)] .5 .4205 0 1 Mshowa
.65873 .433 m
.65873 .43925 L
s
[(200)] .65873 .4205 0 1 Mshowa
.81746 .433 m
.81746 .43925 L
s
[(250)] .81746 .4205 0 1 Mshowa
.97619 .433 m
.97619 .43925 L
s
[(300)] .97619 .4205 0 1 Mshowa
.125 Mabswid
.05556 .433 m
.05556 .43675 L
s
.0873 .433 m
.0873 .43675 L
s
.11905 .433 m
.11905 .43675 L
s
.15079 .433 m
.15079 .43675 L
s
.21429 .433 m
.21429 .43675 L
s
.24603 .433 m
.24603 .43675 L
s
.27778 .433 m
.27778 .43675 L
s
.30952 .433 m
.30952 .43675 L
s
.37302 .433 m
.37302 .43675 L
s
.40476 .433 m
.40476 .43675 L
s
.43651 .433 m
.43651 .43675 L
s
.46825 .433 m
.46825 .43675 L
s
.53175 .433 m
.53175 .43675 L
s
.56349 .433 m
.56349 .43675 L
s
.59524 .433 m
.59524 .43675 L
s
.62698 .433 m
.62698 .43675 L
s
.69048 .433 m
.69048 .43675 L
s
.72222 .433 m
.72222 .43675 L
s
.75397 .433 m
.75397 .43675 L
s
.78571 .433 m
.78571 .43675 L
s
.84921 .433 m
.84921 .43675 L
s
.88095 .433 m
.88095 .43675 L
s
.9127 .433 m
.9127 .43675 L
s
.94444 .433 m
.94444 .43675 L
s
.25 Mabswid
0 .433 m
1 .433 L
s
.02381 .03884 m
.03006 .03884 L
s
[(1.322)] .01131 .03884 1 0 Mshowa
.02381 .13738 m
.03006 .13738 L
s
[(1.324)] .01131 .13738 1 0 Mshowa
.02381 .23592 m
.03006 .23592 L
s
[(1.326)] .01131 .23592 1 0 Mshowa
.02381 .33446 m
.03006 .33446 L
s
[(1.328)] .01131 .33446 1 0 Mshowa
.02381 .53154 m
.03006 .53154 L
s
[(1.332)] .01131 .53154 1 0 Mshowa
.125 Mabswid
.02381 .06347 m
.02756 .06347 L
s
.02381 .08811 m
.02756 .08811 L
s
.02381 .11274 m
.02756 .11274 L
s
.02381 .16201 m
.02756 .16201 L
s
.02381 .18665 m
.02756 .18665 L
s
.02381 .21128 m
.02756 .21128 L
s
.02381 .26055 m
.02756 .26055 L
s
.02381 .28519 m
.02756 .28519 L
s
.02381 .30982 m
.02756 .30982 L
s
.02381 .3591 m
.02756 .3591 L
s
.02381 .38373 m
.02756 .38373 L
s
.02381 .40837 m
.02756 .40837 L
s
.02381 .45764 m
.02756 .45764 L
s
.02381 .48227 m
.02756 .48227 L
s
.02381 .50691 m
.02756 .50691 L
s
.02381 .0142 m
.02756 .0142 L
s
.02381 .55618 m
.02756 .55618 L
s
.02381 .58081 m
.02756 .58081 L
s
.02381 .60545 m
.02756 .60545 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07284 .61803 m
.0746 .01472 L
s
.0746 .01472 m
.07654 .61803 L
s
.07906 .61803 m
.08095 .04475 L
s
.08095 .04475 m
.08301 .61803 L
s
.08527 .61803 m
.0873 .06876 L
s
.0873 .06876 m
.08949 .61803 L
s
.09149 .61803 m
.09365 .08839 L
s
.09365 .08839 m
.09596 .61803 L
s
.09771 .61803 m
.1 .10473 L
s
.1 .10473 m
.10244 .61803 L
s
.10392 .61803 m
.10635 .11855 L
s
.10635 .11855 m
.10892 .61803 L
s
.11014 .61803 m
.1127 .13039 L
s
.1127 .13039 m
.11539 .61803 L
s
.11636 .61803 m
.11905 .14065 L
s
.11905 .14065 m
.12187 .61803 L
s
.12258 .61803 m
.1254 .14962 L
s
.1254 .14962 m
.12835 .61803 L
s
.1288 .61803 m
.13175 .15753 L
s
.13175 .15753 m
.13483 .61803 L
s
.13502 .61803 m
.1381 .16456 L
.14127 .61323 L
.14444 .17084 L
.14762 .59627 L
.15079 .1765 L
.15397 .58096 L
.15714 .18161 L
.16032 .56709 L
.16349 .18626 L
.16667 .55445 L
.16984 .19051 L
.17302 .54289 L
.17619 .1944 L
.17937 .53227 L
.18254 .19797 L
.18571 .52249 L
.18889 .20128 L
.19206 .51346 L
.19524 .20433 L
.19841 .50507 L
.20159 .20717 L
.20476 .49728 L
.20794 .20981 L
.21111 .49002 L
.21429 .21228 L
.21746 .48324 L
.22063 .21458 L
.22381 .47689 L
.22698 .21674 L
.23016 .47093 L
.23333 .21877 L
.23651 .46532 L
.23968 .22068 L
.24286 .46004 L
.24603 .22249 L
.24921 .45506 L
.25238 .22419 L
.25556 .45036 L
.25873 .2258 L
.2619 .4459 L
.26508 .22732 L
.26825 .44168 L
.27143 .22876 L
.2746 .43767 L
.27778 .23014 L
.28095 .43386 L
.28413 .23144 L
.2873 .43023 L
.29048 .23269 L
Mistroke
.29365 .42677 L
.29683 .23387 L
.3 .42348 L
.30317 .23501 L
.30635 .42033 L
.30952 .23609 L
.3127 .41732 L
.31587 .23712 L
.31905 .41444 L
.32222 .23811 L
.3254 .41168 L
.32857 .23906 L
.33175 .40904 L
.33492 .23997 L
.3381 .4065 L
.34127 .24085 L
.34444 .40406 L
.34762 .24168 L
.35079 .40172 L
.35397 .24249 L
.35714 .39947 L
.36032 .24327 L
.36349 .3973 L
.36667 .24402 L
.36984 .39521 L
.37302 .24474 L
.37619 .39319 L
.37937 .24543 L
.38254 .39125 L
.38571 .2461 L
.38889 .38938 L
.39206 .24675 L
.39524 .38757 L
.39841 .24737 L
.40159 .38582 L
.40476 .24798 L
.40794 .38413 L
.41111 .24856 L
.41429 .38249 L
.41746 .24913 L
.42063 .3809 L
.42381 .24968 L
.42698 .37937 L
.43016 .25021 L
.43333 .37788 L
.43651 .25072 L
.43968 .37644 L
.44286 .25122 L
.44603 .37505 L
.44921 .2517 L
Mistroke
.45238 .37369 L
.45556 .25217 L
.45873 .37237 L
.4619 .25263 L
.46508 .37109 L
.46825 .25307 L
.47143 .36985 L
.4746 .2535 L
.47778 .36864 L
.48095 .25392 L
.48413 .36747 L
.4873 .25433 L
.49048 .36633 L
.49365 .25472 L
.49683 .36522 L
.5 .25511 L
.50317 .36413 L
.50635 .25548 L
.50952 .36308 L
.5127 .25585 L
.51587 .36205 L
.51905 .25621 L
.52222 .36105 L
.5254 .25655 L
.52857 .36008 L
.53175 .25689 L
.53492 .35913 L
.5381 .25722 L
.54127 .3582 L
.54444 .25754 L
.54762 .35729 L
.55079 .25786 L
.55397 .35641 L
.55714 .25816 L
.56032 .35555 L
.56349 .25846 L
.56667 .35471 L
.56984 .25876 L
.57302 .35388 L
.57619 .25904 L
.57937 .35308 L
.58254 .25932 L
.58571 .35229 L
.58889 .25959 L
.59206 .35153 L
.59524 .25986 L
.59841 .35077 L
.60159 .26012 L
.60476 .35004 L
.60794 .26038 L
Mistroke
.61111 .34932 L
.61429 .26063 L
.61746 .34862 L
.62063 .26087 L
.62381 .34793 L
.62698 .26111 L
.63016 .34725 L
.63333 .26135 L
.63651 .34659 L
.63968 .26158 L
.64286 .34595 L
.64603 .2618 L
.64921 .34531 L
.65238 .26202 L
.65556 .34469 L
.65873 .26224 L
.6619 .34408 L
.66508 .26245 L
.66825 .34349 L
.67143 .26266 L
.6746 .3429 L
.67778 .26286 L
.68095 .34233 L
.68413 .26306 L
.6873 .34177 L
.69048 .26325 L
.69365 .34122 L
.69683 .26345 L
.7 .34067 L
.70317 .26363 L
.70635 .34014 L
.70952 .26382 L
.7127 .33962 L
.71587 .264 L
.71905 .33911 L
.72222 .26418 L
.7254 .33861 L
.72857 .26435 L
.73175 .33811 L
.73492 .26453 L
.7381 .33763 L
.74127 .2647 L
.74444 .33715 L
.74762 .26486 L
.75079 .33668 L
.75397 .26502 L
.75714 .33622 L
.76032 .26518 L
.76349 .33577 L
.76667 .26534 L
Mistroke
.76984 .33533 L
.77302 .2655 L
.77619 .33489 L
.77937 .26565 L
.78254 .33446 L
.78571 .2658 L
.78889 .33404 L
.79206 .26595 L
.79524 .33362 L
.79841 .26609 L
.80159 .33322 L
.80476 .26623 L
.80794 .33281 L
.81111 .26637 L
.81429 .33242 L
.81746 .26651 L
.82063 .33203 L
.82381 .26665 L
.82698 .33165 L
.83016 .26678 L
.83333 .33127 L
.83651 .26691 L
.83968 .3309 L
.84286 .26704 L
.84603 .33053 L
.84921 .26717 L
.85238 .33017 L
.85556 .26729 L
.85873 .32982 L
.8619 .26742 L
.86508 .32947 L
.86825 .26754 L
.87143 .32912 L
.8746 .26766 L
.87778 .32879 L
.88095 .26778 L
.88413 .32845 L
.8873 .26789 L
.89048 .32812 L
.89365 .26801 L
.89683 .3278 L
.9 .26812 L
.90317 .32748 L
.90635 .26823 L
.90952 .32717 L
.9127 .26834 L
.91587 .32685 L
.91905 .26845 L
.92222 .32655 L
.9254 .26856 L
Mistroke
.92857 .32625 L
.93175 .26866 L
.93492 .32595 L
.9381 .26877 L
.94127 .32566 L
.94444 .26887 L
.94762 .32537 L
.95079 .26897 L
.95397 .32508 L
.95714 .26907 L
.96032 .3248 L
.96349 .26917 L
.96667 .32452 L
.96984 .26926 L
.97302 .32425 L
.97619 .26936 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00O8Woo00<007ooOol0
ogooBGoo000ROol00`00Oomoo`3oOom9Ool0029oo`8000moo`03001oogoo0?moocQoo`008Woo00<0
07ooOol03Woo00<007ooOol0ogoo>7oo000017oo0000000017oo0P000goo0`000goo10000Woo1000
1Goo00<007ooOol03Woo00<007ooOol0ogoo>7oo0002Ool00`00Oomoo`03Ool20006Ool01@00Oomo
ogoo00001Goo00<007ooOol01Goo00<007ooOol03Woo00<007ooOol0ogoo>7oo0002Ool00`00Oomo
o`0;Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`04Ool3000>Ool00`00Oomoo`3oOolh
Ool0009oo`03001oogoo00Uoo`8000Ioo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00io
o`03001oo`000?moocQoo`000Woo00<007ooOol02goo00@007ooOol0009oo`04001oogoo0002Ool0
0`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Ool0003oOolhOol00003Ool0000000Yoo`<000Aoo`80
00Aoo`8000Ioo`03001oogoo00ioo`03001oo`000?moocQoo`008Woo00<007ooOol03Woo00<007oo
0000ogoo>7oo000ROol2000?Ool01@00Ool007oo0000ogoo=Woo000ROol00`00Oomoo`0>Ool01@00
Ool007oo0000ogoo=Woo000ROol00`00Oomoo`0>Ool01@00Ool007oo0000ogoo=Woo000ROol00`00
Oomoo`0>Ool01@00Ool007oo0000ogoo=Woo000ROol2000?Ool01`00Ool007oo001oo`000?moocAo
o`008Woo00<007ooOol03Woo00L007oo001oo`00Ool0003oOoldOol0029oo`03001oogoo00ioo`09
001oo`00Ool007oo001oo`000?mooc9oo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo
0000ogoo<Woo000ROol2000?Ool02@00Ool007oo001oo`00Ool0003oOolbOol0029oo`03001oogoo
00ioo`09001oo`00Ool007oo001oo`00009oo`03001oogoo0?moobeoo`0000Aoo`00000000Aoo`80
00=oo`<000=oo`@000=oo`<000Eoo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`03
001oogoo0?moobeoo`000Woo00<007ooOol00goo0P001Woo00D007ooOomoo`0000Ioo`03001oogoo
00Aoo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`03001oo`000?moobeoo`000Woo
00<007ooOol02goo00<007ooOol00Woo00@007ooOomoo`@000Eoo`<000ioo`09001oo`00Ool007oo
001oo`00009oo`03001oo`000?moobeoo`000Woo00<007ooOol02Goo0P001Woo00H007ooOol007oo
0006Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000ogoo:goo
0002Ool00`00Oomoo`0;Ool01000Oomoo`000Woo00H007ooOol007oo0006Ool00`00Oomoo`0>Ool0
2@00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`000?moobUoo`0000=oo`0000002Woo
0`0017oo0P0017oo0P001Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00T007oo
001oo`00Ool007oo0000ogoo9goo000ROol2000?Ool02@00Ool007oo001oo`00Ool00002Ool02@00
Ool007oo001oo`00Ool0003oOolWOol0029oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00
009oo`0;001oo`00Ool007oo001oo`00Ool0003oOolUOol0029oo`03001oogoo00ioo`09001oo`00
Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000ogoo8goo000ROol00`00
Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`03001oo`000?mooaioo`008Woo0P003goo00T007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000ogoo77oo000ROol00`00Oomoo`0>
Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`07
001oo`00Ool007oo0000ogoo6Woo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0;001oo`00Ool007oo001oo`00Ool0003o
OolFOol0029oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`3o
Ool?Ool0029oo`8000moo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000ogoo
3Goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00/007oo001oo`00
Ool007oo001oo`000?moo`Moo`0000Aoo`00000000Aoo`8000=oo`<000=oo`@000=oo`8000Ioo`03
001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`03001oo`000?moo`000Woo00<007ooOol00goo0P001Woo00D007ooOomoo`0000Ao
o`04001oogoo0005Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?Eoo`00
0Woo00<007ooOol02goo00<007ooOol00Woo00D007ooOomoo`00009oo`03001oogoo00=oo`<000io
o`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00T007oo001oo`00Ool007oo0000jWoo0002
Ool00`00Oomoo`09Ool20006Ool00`00Oomoo`030006Ool00`00Oomoo`0>Ool02@00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001o
o`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001o
o`00Ool0003MOol0009oo`03001oogoo00]oo`04001oogoo0002Ool01000Oomoo`0027oo00<007oo
Ool03Woo00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00/007oo001oo`00Ool007oo
001oo`000<Yoo`0000=oo`0000002Woo0`0017oo0P0017oo0`001Goo00<007ooOol03Woo00T007oo
001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`07001oo`00Ool007oo0000/7oo000ROol0
0`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001o
o`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool03`00Ool007oo001oo`00Ool007oo001oo`00Ool00002Ool0
3@00Ool007oo001oo`00Ool007oo001oo`0008Yoo`008Woo0P003goo00T007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00l007oo
001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`03001oo`0004Uoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001o
o`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00l007oo001o
o`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool0
3@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001o
o`00Ool007oo00000Woo00L007oo001oo`00Ool00009Ool0029oo`03001oogoo00ioo`09001oo`00
Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00
Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00
Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`0?001oo`00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00
Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00
Ool007oo001oo`00Ool007oo001oo`00009oo`07001oo`00Ool007oo00002Goo000ROol00`00Oomo
o`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool03`00Ool007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`0000Uo
o`008Woo0P003goo00T007oo001oo`00Ool007oo00000Woo00h007oo001oo`00Ool007oo001oo`00
Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001o
o`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eo
o`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080
000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001o
o`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001o
o`00Ool0009oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00800008Ool007oo001oo`00Ool0
00Uoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00h007oo001oo`00Ool0
07oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
00003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00
Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00
Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00
Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Qoo`00
Ool007oo001oo`002Goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03P00
Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo0080
000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001o
o`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001o
o`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eo
o`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080
000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000]oo`00Ool007oo001oo`00Ool007oo0080
0009Ool00000Ool007oo001oo`0000Yoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo
00000Woo00h007oo001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool0
07oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool0
07oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
3Goo001oo`00Ool007oo001oo`00Ool007oo00800007Ool007oo001oo`00Ool00P0000Eoo`00Ool0
07oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool0
07oo001oo`00Ool007oo00800006Ool007oo001oo`002Woo000ROol2000?Ool02@00Ool007oo001o
o`00Ool00002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo
001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00=oo`00Ool00P0000Uoo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo
001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000Ioo`00Ool007oo000:Ool0029oo`03001oogoo00eoo`0:
001oo`00001oo`00Ool007oo0002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Uoo`00Ool007oo001oo`00Ool00`00
00Ioo`00001oo`00Ool200001goo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000moo`00Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001o
o`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ao
o`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0000:Ool00004Ool00000
0004Ool20003Ool30003Ool40003Ool20006Ool00`00Oomoo`0=Ool02P00Ool00000Ool007oo001o
o`000Woo00h007oo001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200001Goo001oo`00Ool0
0`0000=oo`00Ool00`0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000moo`00Ool0
07oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
3Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0
009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00L007oo001oo`00Ool0000:Ool0009oo`03001oogoo00=oo`8000Ioo`05001o
ogooOol00004Ool01000Oomoo`001Goo00<007ooOol03Goo00X007oo000007oo001oo`00Ool0009o
o`0>001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00
Ool007oo00800005Ool007oo001oo`0200001Goo00000000Ool00P0000eoo`00Ool007oo001oo`00
Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00
Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003goo001oo`00
Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000<
Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool0
3@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00/007oo001oo`00Ool007oo001o
o`0001Eoo`000Woo00<007ooOol02goo00<007ooOol00Woo00D007ooOomoo`00009oo`03001oogoo
00=oo`<000eoo`04001oo`00Ool2000017oo001oo`000Woo00h007oo001oo`00Ool007oo001oo`00
Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo00800009Ool007oo001oo`00Ool007oo0080000@Ool00000Ool007oo001oo`00Ool0
07oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00
Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000?Ool007oo001oo`00Ool007oo001oo`00Ool007oo0080000<
Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool0
3@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`03001oo`0003]oo`000Woo00<007ooOol02Goo0P001Woo00@007ooOomoo`8000Ioo`03
001oogoo00eoo`04001oo`00Ool2000017oo001oo`000Woo00h007oo001oo`00Ool007oo001oo`00
Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo00800003Ool0000000800007Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001o
o`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00<007oo0000FGoo0002Ool00`00Oomoo`0;Ool01000Oomoo`000Woo00@007ooOol0009o
o`03001oogoo00=oo`03001oogoo00eoo`06001oo`00Ool007oo0P00019oo`00Oomoo`00Ool007oo
001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo00800009Ool0
07oo001oo`00Ool007oo00D0000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool0
07oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool0
07oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool01`00Ool007oo001oo`0007=oo`0000=oo`0000002Woo0`0017oo0P0017oo0P001Woo
00<007ooOol03Goo00H007oo001oo`00Ool200004Woo001oogoo001oo`00Ool007oo001oo`00Ool0
07oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000=oo`00Ool0100000Eoo`00
Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00
Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00
Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001o
o`00Ool007oo001oo`00Ool007oo00000Woo00<007oo0000QWoo000ROol2000>Ool01P00Ool007oo
001oo`@0009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`80000?Ool007oo001oo`00Ool0
07oo001oo`00Ool007oo00<00009Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00
Ool007oo001oo`00Ool007oo001oo`00009oo`03001oo`0009Eoo`008Woo00<007ooOol03Goo00H0
07oo001oo`00Ool500003goo001oo`00Ool007oo001oo`00Ool007oo001oo`0200002Goo001oo`00
Ool007oo001oo`02000047oo000007oo001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool0
07oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool0
07oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`03001oo`000:Aoo`008Woo00<007ooOol03Goo00H007oo001oo`00Ool700003Goo001oo`00
Ool007oo001oo`00Ool007oo00800005Ool007oo001oo`0400000goo001oo`0200003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool0
3@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00L007oo001oo`00Ool0002_Ool0029oo`03001oogoo00eoo`06001oo`00Ool007oo200000ao
o`00001oo`00Ool007oo001oo`00Ool200000goo0000000400000goo001oo`0400003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000]oo`00
Ool007oo001oo`00Ool007oo00<0009oo`03001oo`0000800003Ool007oo00<0000AOol007ooOol0
07oo001oo`00Ool007oo001oo`00Ool00002Ool02`00Ool007oo001oo`00Ool007oo00001goo0`00
0goo0`0017oo0P009goo10000goo0P0017oo0P009Woo10000Woo0`0017oo0P009goo0`0017oo0P00
17oo0P000goo000ROol2000>Ool01P00Ool007oo001oo`P00003Ool007oo0080000?Ool007oo001o
o`00Ool007oo001oo`00Ool007oo00<00003Ool007oo0080000=Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200002goo001oo`00Ool007oo001o
o`00Ool00`000Woo0`0000=oo`00Ool00`0000Aoo`00Ool0009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00<007oo000047oo00<007ooOol01Goo00@007ooOol0009oo`03001oogoo02Eo
o`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo00<007ooOol01Goo00@007ooOol0
009oo`03001oogoo02Moo`04001oogoo0002Ool01000Oomoo`000Woo0@000Goo0Goo000ROol00`00
Oomoo`0=Ool01P00Ool007oo001oo`P00007Ool007oo001oo`00Ool00P0000]oo`00000007oo001o
o`00Ool007oo00<00003Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000io
o`00Ool007oo001oo`00Ool007oo001oo`000Woo00X007oo001oo`00Ool007oo001oo`<0009oo`<0
0003Ool007oo00<00004Ool007oo0002Ool02@00Ool007oo001oo`00Ool0000IOol00`00Oomoo`05
Ool01000Oomoo`000Woo00<007ooOol09Woo00D007ooOomoo`00009oo`04001oogoo0002Ool00`00
Oomoo`0UOol00`00Oomoo`04Ool01000Oomoo`000Woo00<007ooOol09goo00@007ooOol0009oo`04
001oogoo0002Ool10001Ool1Ool0029oo`03001oogoo00eoo`06001oo`00Ool007oo200000Uoo`00
Ool007oo001oo`00Ool0100000Eoo`00Ool007oo00<00005Ool007oo001oo`0200003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0:001o
o`00Ool007oo001oo`00Ool30002Ool300000goo001oo`03000017oo001oo`000Woo00<007ooOol0
7goo00<007ooOol00Woo0`000goo00@007ooOol002Uoo`04001oogoo0002Ool01000Oomoo`000Woo
00<007ooOol09Woo00<007ooOol00`000goo00@007ooOol002Moo`8000=oo`04001oogoo0002Ool0
1000Oomoo`000Woo000ROol00`00Oomoo`0=Ool01P00Ool007oo001oo`P0000;Ool007oo001oo`00
Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00
Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00X007oo001o
o`00Ool007oo001oo`<0009oo`<00003Ool007oo00<00003Ool007oo02Eoo`03001oogoo009oo`03
001oogoo00=oo`04001oogoo000VOol01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo000U
Ool01000Oomoo`000Woo00<007ooOol00goo00@007ooOol002Uoo`04001oogoo0002Ool01000Oomo
o`000Woo0@000Goo0Goo000ROol2000>Ool01P00Ool007oo001oo`P0000;Ool007oo001oo`00Ool0
07oo001oo`0200001Goo001oo`00Ool01@0000=oo`0000000P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool02P00Ool007oo001oo`00
Ool007oo0`000Woo00<007oo00000P000goo0P009Woo0P0017oo10000goo0P00:7oo0P0017oo0P00
17oo0P009goo0P000goo10000goo0P009goo0`0017oo0P0017oo0P000goo000ROol00`00Oomoo`0=
Ool01P00Ool007oo001oo`P0000;Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0003UOol0029o
o`03001oogoo00eoo`06001oo`00Ool007oo200000]oo`00Ool007oo001oo`00Ool007oo0080000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`00
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0003YOol0
029oo`03001oogoo00eoo`06001oo`00Ool007oo200000]oo`00Ool007oo001oo`00Ool007oo0080
000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001o
o`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000kGoo000JOooo
001C0001Ool0029oo`03001oogoo00Moo`03001oogoo00=oo`04001oo`00Ool:00002goo00000000
Ool007oo001oo`00Ool0100000Moo`00Ool007oo001oo`03000027oo000007oo001oo`00Ool30000
1Woo001oo`00Ool0009oo`<00007Ool007oo001oo`00Ool00`002Goo00<007ooOol027oo00<007oo
Ool027oo00<007ooOol01goo00<007ooOol027oo00<007ooOol027oo00<007ooOol01goo00<007oo
Ool027oo00<007ooOol027oo00<007ooOol01goo00<007ooOol027oo00<007ooOol027oo00<007oo
Ool027oo00<007ooOol01goo00<007ooOol027oo00<007ooOol027oo00<007ooOol01goo00<007oo
Ool027oo00<007ooOol027oo00<007ooOol027oo00<007ooOol01goo00<007ooOol027oo00<007oo
Ool01goo000ROol00`00Oomoo`0=Ool01P00Ool007oo001oo`P0000;Ool007oo001oo`00Ool007oo
001oo`0200002Goo001oo`00Ool007oo001oo`0300004Goo001oogoo001oo`00Ool007oo001oo`00
Ool007oo00000Woo00T007oo001oo`00Ool007oo000067oo00<007ooOol0<Woo00<007ooOol0<goo
00<007ooOol0<Woo00<007ooOol0<goo00<007ooOol01goo000ROol00`00Oomoo`0=Ool01P00Ool0
07oo001oo`P0000;Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00
Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`07001oo`00Ool007oo0000
nWoo000ROol00`00Oomoo`0=Ool01P00Ool007oo001oo`P0000;Ool007oo001oo`00Ool007oo001o
o`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`03001oo`000?ioo`008Woo0P003Woo00H007oo001oo`00Ool800002goo001oo`00
Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`3nOol0029oo`03001oogoo00eoo`06001oo`00
Ool007oo200000]oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001o
o`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000ogoo17oo000ROol00`00Oomoo`0=
Ool01P00Ool007oo001oo`P0000;Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool0
07oo001oo`00Ool007oo0002Ool02`00Ool007oo001oo`00Ool007oo0000ogoo1Woo000ROol00`00
Oomoo`0=Ool01P00Ool007oo001oo`P0000;Ool007oo001oo`00Ool007oo001oo`0200003Woo001o
o`00Ool007oo001oo`00Ool007oo0002Ool02@00Ool007oo001oo`00Ool0003oOol8Ool0029oo`80
00ioo`06001oo`00Ool007oo200000]oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001o
o`00Ool007oo001oo`00Ool0009oo`07001oo`00Ool007oo0000ogoo2Woo000ROol00`00Oomoo`0=
Ool01P00Ool007oo001oo`P0000;Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool0
07oo001oo`00Ool007oo0002Ool01@00Ool007oo0000ogoo37oo000ROol00`00Oomoo`0=Ool01P00
Ool007oo001oo`P0000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool00`00Ool0003oOol>Ool0029oo`03001oogoo00eoo`06001oo`00Ool007oo
200000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`03001oogoo0?moo`ioo`008Woo0P003Woo00H007oo001oo`00Ool8000037oo001oo`00Ool0
07oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0ogoo
3Woo000ROol00`00Oomoo`0=Ool01P00Ool007oo001oo`P0000<Ool007oo001oo`00Ool007oo001o
o`000Woo00d007oo001oo`00Ool007oo001oo`00Ool0003oOolCOol00004Ool000000004Ool20003
Ool30003Ool30003Ool40005Ool00`00Oomoo`0=Ool01P00Ool007oo001oo`P0000<Ool007oo001o
o`00Ool007oo001oo`000Woo00/007oo001oo`00Ool007oo001oo`000?mooaEoo`000Woo00<007oo
Ool00goo0P001Woo00<007ooOol00goo00D007ooOomoo`0000Moo`03001oogoo00eoo`06001oo`00
Ool007oo200000aoo`00Ool007oo001oo`00Ool007oo0002Ool02@00Ool007oo001oo`00Ool0003o
OolGOol0009oo`03001oogoo00]oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00Aoo`<0
00eoo`06001oo`00Ool007oo200000aoo`00Ool007oo001oo`00Ool007oo0002Ool02@00Ool007oo
001oo`00Ool0003oOolGOol0009oo`03001oogoo00Uoo`8000Aoo`8000Ioo`03001oogoo00=oo`03
001oogoo00eoo`06001oo`00Ool007oo200000aoo`00Ool007oo001oo`00Ool007oo0002Ool01`00
Ool007oo001oo`000?mooaUoo`000Woo00<007ooOol02goo00<007ooOol00goo00@007ooOol0009o
o`03001oogoo00=oo`03001oogoo00eoo`06001oo`00Ool007oo200000aoo`00Ool007oo001oo`00
Ool007oo0002Ool01`00Ool007oo001oo`000?mooaUoo`0000=oo`0000002Woo0`000goo0`0017oo
0P001Woo00<007ooOol03Goo00H007oo001oo`00Ool8000037oo001oo`00Ool007oo001oo`00Ool0
009oo`05001oo`00Ool0003oOolKOol0029oo`8000ioo`06001oo`00Ool007oo200000aoo`00Ool0
07oo001oo`00Ool007oo0002Ool01@00Ool007oo0000ogoo6goo000ROol00`00Oomoo`0=Ool01P00
Ool007oo001oo`P0000<Ool007oo001oo`00Ool007oo001oo`000Woo00<007oo0000ogoo7Goo000R
Ool00`00Oomoo`0=Ool01P00Ool007oo001oo`P0000<Ool007oo001oo`00Ool007oo001oo`000Woo
00<007oo0000ogoo7Goo000ROol00`00Oomoo`0=Ool01P00Ool007oo001oo`P0000<Ool007oo001o
o`00Ool007oo001oo`000Woo00<007ooOol0ogoo7Goo000ROol2000>Ool01P00Ool007oo001oo`P0
000<Ool007oo001oo`00Ool007oo001oo`000Woo00<007ooOol0ogoo7Goo000ROol00`00Oomoo`0=
Ool01P00Ool007oo001oo`P0000<Ool007oo001oo`00Ool007oo001oo`00ogoo8Woo000ROol00`00
Oomoo`0=Ool01P00Ool007oo001oo`P0000<Ool007oo001oo`00Ool007oo001oo`00ogoo8Woo000R
Ool00`00Oomoo`0=Ool01P00Ool007oo001oo`P0000<Ool007oo001oo`00Ool007oo001oo`00ogoo
8Woo000ROol2000>Ool01P00Ool007oo001oo`P0000:Ool007oo001oo`00Ool007oo003oOolTOol0
029oo`03001oogoo00eoo`06001oo`00Ool007oo200000Yoo`00Ool007oo001oo`00Ool00?moobAo
o`008Woo00<007ooOol03Goo00H007oo001oo`00Ool800002Woo001oo`00Ool007oo001oo`00ogoo
97oo0000\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 1.32121, 
  0.933765, 0.000121122}}]
}, Open  ]],

Cell["\<\
As now expected, the magic tree sizes no longer exist - we have an \
oscillatory convergence that takes a very long time to attain the exact \
value.\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesCall[10, \ 10, \ 0.2, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \(1.32696765846608943`\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "19.3 ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Tree Models 2 - JR Forms"
}], "Section",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
We proceed to define the various functions to explore the JR form \
of a trees. The up and down weights are always given by\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
JRUpWeight[disc_] := N[disc/2];
JRDownWeight[disc_] :=N[disc/2];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
In the exact form of the model, where all terms are matched \
exactly, the up and down scale factors are given by\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
JRupBin[n_, r_, q_, \[Sigma]_, T_] := Exp[(r-q)*T/n]*(1 + Sqrt[Exp[\
\[Sigma]^2*T/n]-1]);
JRdownBin[n_, r_, q_, \[Sigma]_, T_] := Exp[(r-q)*T/n]*(1 - \
Sqrt[Exp[\[Sigma]^2*T/n]-1]);\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
In the approximate form of the model, the scale factors are given \
instead by the functions\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
JRupBinApp[n_, r_, q_, \[Sigma]_, T_] := Exp[(r-q-\[Sigma]^2/2)*T/n \
+ \[Sigma] Sqrt[T/n]];
JRdownBinApp[n_, r_, q_, \[Sigma]_, T_] := Exp[(r-q-\[Sigma]^2/2)*T/n - \
\[Sigma] Sqrt[T/n]];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
The same approach is used whatever the exercise function, first for \
results of a European-style option. The approximate form just uses the \
approximate up and down states (recall that this does have the advantage that \
the latter does not vanish):\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
FastJREuropeanOption[S_, n_, \[Sigma]_, T_, r_, q_, \
exercise_Function] :=
Module[{u = JRupBin[n,r,q,\[Sigma],T], d = JRdownBin[n,r,q,\[Sigma],T],disc = \
Discount[n,r,T]},
(disc/2)^n Sum[exercise[S u^j d^(n-j)] Binomial[n,j], {j,0,n}]];\
\>", "Input",\

  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
FastJREuropeanOptionApp[S_, n_, \[Sigma]_, T_, r_, q_, \
exercise_Function] :=
Module[{u = JRupBinApp[n,r,q,\[Sigma],T], d = \
JRdownBinApp[n,r,q,\[Sigma],T],disc = Discount[n,r,T],P,Q},
(disc/2)^n Sum[exercise[S u^j d^(n-j)] Binomial[n,j], {j,0,n}]];\
\>", "Input",\

  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["Next we give functions specific to a Call and a Put:", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
FastJREuropeanPut[S_, K_, n_, r_, q_, \[Sigma]_, T_] \
:=FastJREuropeanOption[S,n,\[Sigma],T,r,q,Max[K-#,0]&];
FastJREuropeanCall[S_, K_, n_, r_, q_, \[Sigma]_, T_] := \
FastJREuropeanOption[S, n, \[Sigma], T, r, q, Max[# - K, 0]&];
FastJREuropeanPutApp[S_, K_, n_, r_, q_, \[Sigma]_, T_] \
:=FastJREuropeanOptionApp[S,n,\[Sigma],T,r,q,Max[K-#,0]&];
FastJREuropeanCallApp[S_, K_, n_, r_, q_, \[Sigma]_, T_] := \
FastJREuropeanOptionApp[S, n, \[Sigma], T, r, q, Max[# - K, 0]&];
\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[TextData[{
  "Next we give the American versions. These follow the same pattern as for \
CRR, but we have to make some careful adjustments to the Greeks. Delta can be \
done the same way, and Gamma needs a little more care with the differencing. \
Both are evaluated at not quite the right time and not exactly the right \
stock price. Theta has to be evaluated by first correcting the [1,2] node for \
Delta effects. There is a common misconception that it is somehow more \
difficult to extract the basic Greeks from one call to a tree in the \
Jarrow-Rudd model. This is nonsense. One just has to remember that ",
  Cell[BoxData[
      \(TraditionalForm\`u\ d\)]],
  " is not unity!"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
JRAmericanOption[S_, n_, \[Sigma]_, T_, r_, q_, exercise_Function] \
:=
Module[{u,d,disc = Discount[n,r,T],P,Q,TreeOp,value,delta,gamma,theta},
u = JRupBin[n,r,q,\[Sigma],T]; 
d = JRdownBin[n,r,q,\[Sigma],T];
P = JRUpWeight[disc];
Q = P;
TreeOp[node_,level_] := TreeOp[node,level] = 
If[level==n,exercise[S d^node u^(level-node)],
Max[{P,Q}.{TreeOp[node,level+1],TreeOp[node+1,level+1]},
exercise[S d^node u^(level-node)]]];
value=TreeOp[0,0];
delta = (TreeOp[0,1] - TreeOp[1,1])/((u-d)*S);
gamma = ((TreeOp[0,2] -TreeOp[1,2])/(u^2-u*d) -
(TreeOp[1,2]-TreeOp[2,2])/(u*d-d^2))*2/(S^2(u^2-d^2));
theta = (TreeOp[1,2]-(u*d-1)*S*delta - TreeOp[0,0])/(2*T/n);
Clear[TreeOp];
{value, delta, gamma, theta}];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
JRAmericanOptionApp[S_, n_, \[Sigma]_, T_, r_, q_, \
exercise_Function] :=
Module[{u,d,disc = Discount[n,r,T],P,Q,TreeOp,value,delta,gamma,theta},
u = JRupBinApp[n,r,q,\[Sigma],T]; 
d = JRdownBinApp[n,r,q,\[Sigma],T];
P = JRUpWeight[disc];
Q = P;
TreeOp[node_,level_] := TreeOp[node,level] = 
If[level==n,exercise[S d^node u^(level-node)],
Max[{P,Q}.{TreeOp[node,level+1],TreeOp[node+1,level+1]},
exercise[S d^node u^(level-node)]]];
value=TreeOp[0,0];
delta = (TreeOp[0,1] - TreeOp[1,1])/((u-d)*S);
gamma = ((TreeOp[0,2] -TreeOp[1,2])/(u^2-u*d) -
(TreeOp[1,2]-TreeOp[2,2])/(u*d-d^2))*2/(S^2(u^2-d^2));
theta = (TreeOp[1,2]-(u*d-1)*S*delta - TreeOp[0,0])/(2*T/n);
Clear[TreeOp];
{value, delta, gamma, theta}];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["Finally functions specific to an American Call and a Put:", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
JRAmericanPut[S_, K_, n_, r_, q_, \[Sigma]_, T_] \
:=JRAmericanOption[S,n,\[Sigma],T,r,q,Max[K-#,0]&];
JRAmericanCall[S_, K_, n_, r_, q_, \[Sigma]_, T_] :=JRAmericanOption[S,n,\
\[Sigma],T,r,q,Max[#-K,0]&];
JRAmericanPutApp[S_, K_, n_, r_, q_, \[Sigma]_, T_] \
:=JRAmericanOptionApp[S,n,\[Sigma],T,r,q,Max[K-#,0]&];
JRAmericanCallApp[S_, K_, n_, r_, q_, \[Sigma]_, T_] \
:=JRAmericanOptionApp[S,n,\[Sigma],T,r,q,Max[#-K,0]&];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Evaluation and Verification of JR-Style Models", "Subsection",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
What we shall do is to compute sample values for a Call and a Put \
in both forms of the JR model:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(JRdataC\  = \ 
      Table[{k, 
          FastJREuropeanCall[9, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 1]}, \ {k, \ 
          15, \ 300}]; \n
    JRdataP\  = \ 
      Table[{k, 
          FastJREuropeanPut[9, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 1]}, \ {k, \ 
          15, \ 300}]; \n
    JRdataCApp\  = \ 
      Table[{k, \ 
          FastJREuropeanCallApp[9, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 
            1]}, \ {k, \ 15, \ 300}]; \n
    JRdataPApp\  = \ 
      Table[{k, \ 
          FastJREuropeanPutApp[9, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 
            1]}, \ {k, \ 15, \ 300}]; \)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(JRPlotC\  = \ 
      ListPlot[JRdataC, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \n
    JRPlotP = \ 
      ListPlot[JRdataP, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \n
    JRPlotCApp\  = \ 
      ListPlot[JRdataCApp, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \n
    JRPlotPApp = \ 
      ListPlot[JRdataPApp, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[explot, JRPlotC, \ \ PlotRange\  -> \ {0.69, \ 0.70}, \ 
      DisplayFunction\  -> \ $DisplayFunction]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -42.6443 61.8034 [
[.18254 .60553 -6 -9 ]
[.18254 .60553 6 0 ]
[.34127 .60553 -9 -9 ]
[.34127 .60553 9 0 ]
[.5 .60553 -9 -9 ]
[.5 .60553 9 0 ]
[.65873 .60553 -9 -9 ]
[.65873 .60553 9 0 ]
[.81746 .60553 -9 -9 ]
[.81746 .60553 9 0 ]
[.97619 .60553 -9 -9 ]
[.97619 .60553 9 0 ]
[.01131 0 -24 -4.5 ]
[.01131 0 0 4.5 ]
[.01131 .12361 -30 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -30 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -30 -4.5 ]
[.01131 .49443 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .62428 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .61803 m
.18254 .62428 L
s
[(50)] .18254 .60553 0 1 Mshowa
.34127 .61803 m
.34127 .62428 L
s
[(100)] .34127 .60553 0 1 Mshowa
.5 .61803 m
.5 .62428 L
s
[(150)] .5 .60553 0 1 Mshowa
.65873 .61803 m
.65873 .62428 L
s
[(200)] .65873 .60553 0 1 Mshowa
.81746 .61803 m
.81746 .62428 L
s
[(250)] .81746 .60553 0 1 Mshowa
.97619 .61803 m
.97619 .62428 L
s
[(300)] .97619 .60553 0 1 Mshowa
.125 Mabswid
.05556 .61803 m
.05556 .62178 L
s
.0873 .61803 m
.0873 .62178 L
s
.11905 .61803 m
.11905 .62178 L
s
.15079 .61803 m
.15079 .62178 L
s
.21429 .61803 m
.21429 .62178 L
s
.24603 .61803 m
.24603 .62178 L
s
.27778 .61803 m
.27778 .62178 L
s
.30952 .61803 m
.30952 .62178 L
s
.37302 .61803 m
.37302 .62178 L
s
.40476 .61803 m
.40476 .62178 L
s
.43651 .61803 m
.43651 .62178 L
s
.46825 .61803 m
.46825 .62178 L
s
.53175 .61803 m
.53175 .62178 L
s
.56349 .61803 m
.56349 .62178 L
s
.59524 .61803 m
.59524 .62178 L
s
.62698 .61803 m
.62698 .62178 L
s
.69048 .61803 m
.69048 .62178 L
s
.72222 .61803 m
.72222 .62178 L
s
.75397 .61803 m
.75397 .62178 L
s
.78571 .61803 m
.78571 .62178 L
s
.84921 .61803 m
.84921 .62178 L
s
.88095 .61803 m
.88095 .62178 L
s
.9127 .61803 m
.9127 .62178 L
s
.94444 .61803 m
.94444 .62178 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.02381 0 m
.03006 0 L
s
[(0.69)] .01131 0 1 0 Mshowa
.02381 .12361 m
.03006 .12361 L
s
[(0.692)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.694)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.696)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.698)] .01131 .49443 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .30271 m
.0746 .30271 L
.07778 .30271 L
.08095 .30271 L
.08413 .30271 L
.0873 .30271 L
.09048 .30271 L
.09365 .30271 L
.09683 .30271 L
.1 .30271 L
.10317 .30271 L
.10635 .30271 L
.10952 .30271 L
.1127 .30271 L
.11587 .30271 L
.11905 .30271 L
.12222 .30271 L
.1254 .30271 L
.12857 .30271 L
.13175 .30271 L
.13492 .30271 L
.1381 .30271 L
.14127 .30271 L
.14444 .30271 L
.14762 .30271 L
.15079 .30271 L
.15397 .30271 L
.15714 .30271 L
.16032 .30271 L
.16349 .30271 L
.16667 .30271 L
.16984 .30271 L
.17302 .30271 L
.17619 .30271 L
.17937 .30271 L
.18254 .30271 L
.18571 .30271 L
.18889 .30271 L
.19206 .30271 L
.19524 .30271 L
.19841 .30271 L
.20159 .30271 L
.20476 .30271 L
.20794 .30271 L
.21111 .30271 L
.21429 .30271 L
.21746 .30271 L
.22063 .30271 L
.22381 .30271 L
.22698 .30271 L
Mistroke
.23016 .30271 L
.23333 .30271 L
.23651 .30271 L
.23968 .30271 L
.24286 .30271 L
.24603 .30271 L
.24921 .30271 L
.25238 .30271 L
.25556 .30271 L
.25873 .30271 L
.2619 .30271 L
.26508 .30271 L
.26825 .30271 L
.27143 .30271 L
.2746 .30271 L
.27778 .30271 L
.28095 .30271 L
.28413 .30271 L
.2873 .30271 L
.29048 .30271 L
.29365 .30271 L
.29683 .30271 L
.3 .30271 L
.30317 .30271 L
.30635 .30271 L
.30952 .30271 L
.3127 .30271 L
.31587 .30271 L
.31905 .30271 L
.32222 .30271 L
.3254 .30271 L
.32857 .30271 L
.33175 .30271 L
.33492 .30271 L
.3381 .30271 L
.34127 .30271 L
.34444 .30271 L
.34762 .30271 L
.35079 .30271 L
.35397 .30271 L
.35714 .30271 L
.36032 .30271 L
.36349 .30271 L
.36667 .30271 L
.36984 .30271 L
.37302 .30271 L
.37619 .30271 L
.37937 .30271 L
.38254 .30271 L
.38571 .30271 L
Mistroke
.38889 .30271 L
.39206 .30271 L
.39524 .30271 L
.39841 .30271 L
.40159 .30271 L
.40476 .30271 L
.40794 .30271 L
.41111 .30271 L
.41429 .30271 L
.41746 .30271 L
.42063 .30271 L
.42381 .30271 L
.42698 .30271 L
.43016 .30271 L
.43333 .30271 L
.43651 .30271 L
.43968 .30271 L
.44286 .30271 L
.44603 .30271 L
.44921 .30271 L
.45238 .30271 L
.45556 .30271 L
.45873 .30271 L
.4619 .30271 L
.46508 .30271 L
.46825 .30271 L
.47143 .30271 L
.4746 .30271 L
.47778 .30271 L
.48095 .30271 L
.48413 .30271 L
.4873 .30271 L
.49048 .30271 L
.49365 .30271 L
.49683 .30271 L
.5 .30271 L
.50317 .30271 L
.50635 .30271 L
.50952 .30271 L
.5127 .30271 L
.51587 .30271 L
.51905 .30271 L
.52222 .30271 L
.5254 .30271 L
.52857 .30271 L
.53175 .30271 L
.53492 .30271 L
.5381 .30271 L
.54127 .30271 L
.54444 .30271 L
Mistroke
.54762 .30271 L
.55079 .30271 L
.55397 .30271 L
.55714 .30271 L
.56032 .30271 L
.56349 .30271 L
.56667 .30271 L
.56984 .30271 L
.57302 .30271 L
.57619 .30271 L
.57937 .30271 L
.58254 .30271 L
.58571 .30271 L
.58889 .30271 L
.59206 .30271 L
.59524 .30271 L
.59841 .30271 L
.60159 .30271 L
.60476 .30271 L
.60794 .30271 L
.61111 .30271 L
.61429 .30271 L
.61746 .30271 L
.62063 .30271 L
.62381 .30271 L
.62698 .30271 L
.63016 .30271 L
.63333 .30271 L
.63651 .30271 L
.63968 .30271 L
.64286 .30271 L
.64603 .30271 L
.64921 .30271 L
.65238 .30271 L
.65556 .30271 L
.65873 .30271 L
.6619 .30271 L
.66508 .30271 L
.66825 .30271 L
.67143 .30271 L
.6746 .30271 L
.67778 .30271 L
.68095 .30271 L
.68413 .30271 L
.6873 .30271 L
.69048 .30271 L
.69365 .30271 L
.69683 .30271 L
.7 .30271 L
.70317 .30271 L
Mistroke
.70635 .30271 L
.70952 .30271 L
.7127 .30271 L
.71587 .30271 L
.71905 .30271 L
.72222 .30271 L
.7254 .30271 L
.72857 .30271 L
.73175 .30271 L
.73492 .30271 L
.7381 .30271 L
.74127 .30271 L
.74444 .30271 L
.74762 .30271 L
.75079 .30271 L
.75397 .30271 L
.75714 .30271 L
.76032 .30271 L
.76349 .30271 L
.76667 .30271 L
.76984 .30271 L
.77302 .30271 L
.77619 .30271 L
.77937 .30271 L
.78254 .30271 L
.78571 .30271 L
.78889 .30271 L
.79206 .30271 L
.79524 .30271 L
.79841 .30271 L
.80159 .30271 L
.80476 .30271 L
.80794 .30271 L
.81111 .30271 L
.81429 .30271 L
.81746 .30271 L
.82063 .30271 L
.82381 .30271 L
.82698 .30271 L
.83016 .30271 L
.83333 .30271 L
.83651 .30271 L
.83968 .30271 L
.84286 .30271 L
.84603 .30271 L
.84921 .30271 L
.85238 .30271 L
.85556 .30271 L
.85873 .30271 L
.8619 .30271 L
Mistroke
.86508 .30271 L
.86825 .30271 L
.87143 .30271 L
.8746 .30271 L
.87778 .30271 L
.88095 .30271 L
.88413 .30271 L
.8873 .30271 L
.89048 .30271 L
.89365 .30271 L
.89683 .30271 L
.9 .30271 L
.90317 .30271 L
.90635 .30271 L
.90952 .30271 L
.9127 .30271 L
.91587 .30271 L
.91905 .30271 L
.92222 .30271 L
.9254 .30271 L
.92857 .30271 L
.93175 .30271 L
.93492 .30271 L
.9381 .30271 L
.94127 .30271 L
.94444 .30271 L
.94762 .30271 L
.95079 .30271 L
.95397 .30271 L
.95714 .30271 L
.96032 .30271 L
.96349 .30271 L
.96667 .30271 L
.96984 .30271 L
.97302 .30271 L
.97619 .30271 L
Mfstroke
.08292 .61803 m
.08413 .56979 L
s
.08413 .56979 m
.08545 .61803 L
s
.08853 .61803 m
.09048 .51019 L
s
.09048 .51019 m
.09256 .61803 L
s
.09449 .61803 m
.09683 .461 L
s
.09683 .461 m
.0993 .61803 L
s
.10058 .61803 m
.10317 .41972 L
s
.10317 .41972 m
.1059 .61803 L
s
.10674 .61803 m
.10952 .38458 L
s
.10952 .38458 m
.11243 .61803 L
s
.11294 .61803 m
.11587 .35431 L
s
.11587 .35431 m
.11893 .61803 L
s
.11915 .61803 m
.12222 .32796 L
.1254 .61687 L
.12857 .30481 L
.13175 .60622 L
.13492 .28432 L
.1381 .59596 L
.14127 .26605 L
.14444 .5861 L
.14762 .24966 L
.15079 .57662 L
.15397 .23487 L
.15714 .56751 L
.16032 .22147 L
.16349 .55876 L
.16667 .20926 L
.16984 .55035 L
.17302 .19809 L
.17619 .54226 L
.17937 .18783 L
.18254 .53448 L
.18571 .17838 L
.18889 .527 L
.19206 .16965 L
.19524 .51979 L
.19841 .16155 L
.20159 .51284 L
.20476 .15402 L
.20794 .50614 L
.21111 .14701 L
.21429 .49967 L
.21746 .14045 L
.22063 .49343 L
.22381 .14355 L
.22698 .48739 L
.23016 .15902 L
.23333 .48156 L
.23651 .17325 L
.23968 .47591 L
.24286 .18637 L
.24603 .47045 L
.24921 .19848 L
.25238 .46515 L
.25556 .20968 L
.25873 .46002 L
.2619 .22004 L
.26508 .45504 L
.26825 .22964 L
.27143 .4502 L
.2746 .23855 L
Mistroke
.27778 .44551 L
.28095 .24682 L
.28413 .44095 L
.2873 .25451 L
.29048 .43651 L
.29365 .26167 L
.29683 .4322 L
.3 .26834 L
.30317 .428 L
.30635 .27455 L
.30952 .42392 L
.3127 .28034 L
.31587 .41994 L
.31905 .28575 L
.32222 .41606 L
.3254 .2908 L
.32857 .41228 L
.33175 .29551 L
.33492 .40859 L
.3381 .29992 L
.34127 .405 L
.34444 .30404 L
.34762 .40149 L
.35079 .3079 L
.35397 .39806 L
.35714 .3115 L
.36032 .39471 L
.36349 .31487 L
.36667 .39144 L
.36984 .31803 L
.37302 .38824 L
.37619 .32099 L
.37937 .38511 L
.38254 .32375 L
.38571 .38205 L
.38889 .32634 L
.39206 .37905 L
.39524 .32876 L
.39841 .37612 L
.40159 .33103 L
.40476 .37325 L
.40794 .33315 L
.41111 .37044 L
.41429 .33513 L
.41746 .36769 L
.42063 .33699 L
.42381 .36499 L
.42698 .33872 L
.43016 .36235 L
.43333 .34034 L
Mistroke
.43651 .35975 L
.43968 .34185 L
.44286 .35721 L
.44603 .34326 L
.44921 .35471 L
.45238 .34457 L
.45556 .35226 L
.45873 .34579 L
.4619 .34986 L
.46508 .34693 L
.46825 .3475 L
.47143 .34799 L
.4746 .34519 L
.47778 .34897 L
.48095 .34291 L
.48413 .34987 L
.4873 .34068 L
.49048 .35071 L
.49365 .33848 L
.49683 .35149 L
.5 .33632 L
.50317 .3522 L
.50635 .3342 L
.50952 .35285 L
.5127 .33211 L
.51587 .35345 L
.51905 .33006 L
.52222 .354 L
.5254 .32805 L
.52857 .3545 L
.53175 .32606 L
.53492 .35495 L
.5381 .32411 L
.54127 .35535 L
.54444 .32219 L
.54762 .35572 L
.55079 .3203 L
.55397 .35604 L
.55714 .31843 L
.56032 .35632 L
.56349 .3166 L
.56667 .35657 L
.56984 .31479 L
.57302 .35678 L
.57619 .31302 L
.57937 .35696 L
.58254 .31126 L
.58571 .35711 L
.58889 .30954 L
.59206 .35723 L
Mistroke
.59524 .30784 L
.59841 .35732 L
.60159 .30616 L
.60476 .35738 L
.60794 .30451 L
.61111 .35742 L
.61429 .30288 L
.61746 .35743 L
.62063 .30128 L
.62381 .35742 L
.62698 .29969 L
.63016 .35738 L
.63333 .29813 L
.63651 .35733 L
.63968 .29659 L
.64286 .35725 L
.64603 .29507 L
.64921 .35716 L
.65238 .29357 L
.65556 .35704 L
.65873 .29209 L
.6619 .35691 L
.66508 .29064 L
.66825 .35677 L
.67143 .2892 L
.6746 .3566 L
.67778 .28777 L
.68095 .35642 L
.68413 .28637 L
.6873 .35623 L
.69048 .28498 L
.69365 .35602 L
.69683 .28362 L
.7 .3558 L
.70317 .28227 L
.70635 .35556 L
.70952 .28093 L
.7127 .35532 L
.71587 .27961 L
.71905 .35506 L
.72222 .27831 L
.7254 .35479 L
.72857 .27702 L
.73175 .35451 L
.73492 .27575 L
.7381 .35422 L
.74127 .2745 L
.74444 .35392 L
.74762 .27326 L
.75079 .35362 L
Mistroke
.75397 .27203 L
.75714 .3533 L
.76032 .27082 L
.76349 .35297 L
.76667 .26962 L
.76984 .35264 L
.77302 .26844 L
.77619 .3523 L
.77937 .26727 L
.78254 .35195 L
.78571 .26611 L
.78889 .3516 L
.79206 .26496 L
.79524 .35124 L
.79841 .26383 L
.80159 .35087 L
.80476 .26271 L
.80794 .3505 L
.81111 .2616 L
.81429 .35012 L
.81746 .26227 L
.82063 .34973 L
.82381 .26398 L
.82698 .34934 L
.83016 .26565 L
.83333 .34895 L
.83651 .26728 L
.83968 .34855 L
.84286 .26886 L
.84603 .34815 L
.84921 .27041 L
.85238 .34774 L
.85556 .27191 L
.85873 .34733 L
.8619 .27338 L
.86508 .34691 L
.86825 .27481 L
.87143 .3465 L
.8746 .27621 L
.87778 .34607 L
.88095 .27757 L
.88413 .34565 L
.8873 .2789 L
.89048 .34522 L
.89365 .28019 L
.89683 .34479 L
.9 .28145 L
.90317 .34436 L
.90635 .28268 L
.90952 .34392 L
Mistroke
.9127 .28388 L
.91587 .34348 L
.91905 .28505 L
.92222 .34304 L
.9254 .2862 L
.92857 .3426 L
.93175 .28731 L
.93492 .34215 L
.9381 .28839 L
.94127 .34171 L
.94444 .28945 L
.94762 .34126 L
.95079 .29049 L
.95397 .34081 L
.95714 .29149 L
.96032 .34036 L
.96349 .29248 L
.96667 .3399 L
.96984 .29343 L
.97302 .33945 L
.97619 .29437 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00OogooKWoo003oOom^
Ool000Qoo`8000Aoo`8000Aoo`8000=oo`<00?mooe9oo`001goo00@007ooOol000=oo`8000=oo`04
001oogoo0005Ool00`00Oomoo`3oOom?Ool000Moo`04001oogoo0008Ool01000Oomoo`000goo0`00
1Goo0`00ogooBGoo0007Ool01000Oomoo`0027oo0`000goo00@007ooOol000Eoo`03001oogoo0?mo
odUoo`001goo00@007ooOol000Qoo`03001oogoo00=oo`04001oogoo0005Ool00`00Oomoo`3oOom9
Ool000Qoo`8000Yoo`<000=oo`8000Ioo`03001oogoo0?moodUoo`008Woo00<007ooOol0ogooBGoo
000ROol2003oOom:Ool0029oo`03001oogoo0?moodUoo`008Woo00<007ooOol0ogooBGoo000ROol0
0`00Oomoo`3oOom9Ool0029oo`03001oogoo0?moodUoo`008Woo0P00ogooBWoo000ROol00`00Oomo
o`3oOom9Ool0029oo`03001oogoo0?moodUoo`008Woo00<007ooOol0ogooBGoo000ROol00`00Oomo
o`3oOom9Ool0029oo`800?moodYoo`008Woo00<007ooOol0ogooBGoo000ROol00`00Oomoo`3oOom9
Ool0009oo`8000Aoo`8000Aoo`8000=oo`<000=oo`@000Eoo`03001oogoo0?moodUoo`0000Eoo`00
Oomoo`0000=oo`8000=oo`04001oogoo0005Ool01@00Oomoogoo00001goo00<007ooOol0ogooBGoo
00001Goo001oogoo000027oo00@007ooOol000=oo`<000Aoo`03001oogoo00Aoo`<00?moodUoo`00
00Eoo`00Oomoo`0000Qoo`<000=oo`04001oogoo0005Ool00`00Oomoo`03Ool00`00Oomoo`3oOom9
Ool00005Ool007ooOol00008Ool00`00Oomoo`03Ool01000Oomoo`000Woo00@007ooOol000Eoo`03
001oogoo0?moodUoo`000Woo0P002Woo0`000goo0P0017oo0P001Woo00<007ooOol0?Woo00<007oo
0000ogoo27oo000ROol00`00Oomoo`0lOol01@00Ool007oo0000ogoo27oo000ROol2000kOol01`00
Ool007oo001oo`000?moo`Qoo`008Woo00<007ooOol0>7oo00/007oo001oo`00Ool007oo001oo`00
0?moo`Ioo`008Woo00<007ooOol0=Goo00h007ooOol007oo001oo`00Ool007oo001oo`00ogoo1Woo
000ROol00`00Oomoo`0eOol04000Oomoo`00Ool007oo001oo`00Ool007oo001oo`00ogoo17oo000R
Ool00`00Oomoo`0cOol00`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?moo`Ao
o`008Woo0P00<Woo00D007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00<007ooOol0oWoo000ROol00`00Oomoo`0aOol01@00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool00`00Oomoo`3nOol0029oo`03001oogoo02moo`07001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Ool0003nOol0029oo`03001o
ogoo02moo`07001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool0
0`00Ool0003nOol0029oo`03001oogoo02eoo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00D007oo001oo`000?aoo`008Woo0P00;Woo00T007oo001o
o`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001o
o`000?Yoo`008Woo00<007ooOol0:goo00/007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0003jOol0029oo`03001oogoo02]o
o`0;001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009o
o`09001oo`00Ool007oo001oo`000?Qoo`000Woo0P0017oo0P0017oo0P000goo0`0017oo0`001Goo
00<007ooOol0:Goo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`0;001oo`00Ool007oo001oo`00Ool0003fOol00005Ool007ooOol00003
Ool20003Ool01000Oomoo`001Goo00<007ooOol00Woo00<007ooOol017oo00<007ooOol0:Goo00d0
07oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009o
o`0;001oo`00Ool007oo001oo`00Ool0003fOol00005Ool007ooOol00008Ool01000Oomoo`000goo
0`000Woo10001Goo0`009Woo010007ooOol007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001o
o`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0003dOol0
0005Ool007ooOol00008Ool30003Ool01000Oomoo`000Woo00<007oo00001Woo00<007ooOol09Woo
010007ooOol007oo001oo`00Ool007oo001oo`00Ool0009oo`0<001oo`00Ool007oo001oo`00Ool0
07oo0P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00<007ooOol0kgoo00001Goo001o
ogoo000027oo00<007ooOol00goo00@007ooOol0009oo`03001oo`0000Ioo`03001oogoo02Ioo`0?
001oogoo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00<007oo0000Ygoo00T007oo001o
o`00Ool007oo00000Woo00<007ooOol0>Woo0002Ool2000:Ool30003Ool20004Ool20006Ool00`00
Oomoo`0TOol00`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00D007oo
001oo`0009Yoo`05001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`09
001oo`00Ool007oo001oo`0003Aoo`008Woo00<007ooOol097oo00<007oo00000Woo00`007oo001o
o`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo
001oo`00Ool007oo001oo`00Ool0009oo`07001oo`00Ool007oo0000SGoo010007ooOol007oo001o
o`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000:Goo000ROol2000UOol00`00Ool0
0002Ool03000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00T007oo001oo`00Ool007oo0000
PGoo00/007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol077oo000ROol00`00
Oomoo`0ROol01@00Ool007oo00000Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Goo001o
o`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0;
001oo`00Ool007oo001oo`00Ool0001fOol01@00Ool007oo00000Woo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0<001oo`00Ool007oo
001oo`00Ool007oo0P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00
Ool007oo001oo`00Ool00002Ool02`00Ool007oo001oo`00Ool007oo000057oo000ROol00`00Oomo
o`0ROol01@00Ool007oo00000Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001o
o`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0I7oo010007ooOol007oo001oo`00Ool0
07oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00`007oo001oo`00
Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001o
o`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`0000Uoo`008Woo00<007ooOol0
3Gooo`00<`002Goo000ROol2000QOol01`00Ool007oo001oo`00009oo`0<001oo`00Ool007oo001o
o`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`07001oo`00Ool0
07oo0000DGoo010007ooOol007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool0
07oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool0
07oo001oo`00Ool007oo001oo`00Ool0009oo`07001oo`00Ool007oo00002Goo000ROol00`00Oomo
o`0POol01`00Ool007oo001oo`00009oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>
Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0;001oo`00Ool007oo001oo`00Ool00015Ool0
2@00Ool007oo001oo`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200001goo
001oo`00Ool007oo00800005Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
00800009Ool007oo001oo`00Ool007oo00800009Ool00000001oo`00Ool007oo00800005Ool007oo
001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00800008Ool007oo001oo`00Ool000Uo
o`008Woo00<007ooOol087oo00L007oo001oo`00Ool00002Ool03000Ool007oo001oo`00Ool007oo
001oo`800009Ool007oo001oo`00Ool007oo00<0000@Ool00000Ool007oo001oo`00Ool007oo001o
o`00Ool200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`03001oogoo03Moo`0@001oogoo001oo`00Ool007oo001oo`00Ool007oo0002
Ool03000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000]oo`00Ool007oo001oo`00Ool007oo0080000@Ool00000Ool007oo001oo`00Ool007oo001o
o`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200002goo001oo`00Ool007oo
001oo`00Ool00`0000Moo`00001oo`00Ool0000:Ool0029oo`03001oogoo01ioo`09001oo`00Ool0
07oo001oo`00009oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000Eoo`00Ool007oo00800003
Ool007oo00D0000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00
Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000
;Goo00T007oo001oo`00Ool007oo00000Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Goo
001oo`00Ool007oo001oo`00Ool007oo00800007Ool007oo001oo`00Ool00P0000Eoo`00Ool007oo
0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo
001oo`00Ool007oo00800006Ool007oo001oo`002Woo000ROol00`00Oomoo`0NOol02@00Ool007oo
001oo`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`800003Ool007oo0080000;Ool0
07oo001oo`00Ool007oo001oo`0300002Goo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0;001oo`00
Ool007oo001oo`00Ool0000QOol03P00Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001o
o`00Ool007oo001oo`00Ool007oo00800003Ool007oo00800009Ool007oo001oo`00Ool007oo0080
000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001o
o`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001o
o`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`800006Ool007oo001oo`002Woo
000ROol2000OOol02000Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000>Ool00000Ool007oo001oo`00Ool007oo001oo`800003Ool007oo00@00005Ool007oo001o
o`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001o
o`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00D007oo001oo`00015oo`05
001oo`00Ool00002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool200000goo001oo`020000
2Goo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool0
07oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
37oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0000:Ool0029o
o`03001oogoo01ioo`08001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00
Ool00`0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200001goo001oo`00Ool007oo00L0000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
00003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03000Ool007oo001oo`00Ool007oo001o
o`80000@Ool007ooOol007oo001oo`00Ool007oo001oo`00Ool2000037oo000007oo001oo`00Ool0
07oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000aoo`00Ool007oo001oo`00
Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001o
o`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001o
ogoo00ioo`008Woo00<007ooOol07Woo00P007oo001oo`00Ool007oo0P0000]oo`00Ool007oo001o
o`00Ool007oo0080000@Ool00000Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00
Ool007oo001oo`00Ool007oo00800003Ool0000000@00005Ool007oo001oo`0200003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000]oo`00
Ool007oo001oo`00Ool007oo0080009oo`0>001oo`00001oo`00Ool007oo001oo`00Ool0009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo
001oo`00Ool0000aOol0009oo`8000Aoo`8000Aoo`8000=oo`<000Aoo`8000Ioo`03001oogoo01ao
o`0:001oo`00Ool007oo001oo`00Ool200002goo001oo`00Ool007oo001oo`00Ool00P00011oo`00
001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200000goo001oo`0300001Goo00000000Ool0
100000Eoo`00000007oo00<00005Ool007oo001oo`0200002Woo001oo`00Ool007oo001oo`005Woo
00/007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0001EOol00005
Ool007ooOol00003Ool20003Ool01000Oomoo`001Goo00@007ooOol0009oo`03001oogoo00=oo`03
001oogoo01aoo`0:001oo`00Ool007oo001oo`00Ool200002Goo001oo`00Ool007oo001oo`020000
0goo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Ioo`00Ool0
07oo002oOol00005Ool007ooOol00008Ool01000Oomoo`000goo0`000Woo00@007ooOol000Eoo`<0
01aoo`0:001oo`00Ool007oo001oo`00Ool200001goo001oo`00Ool007oo00800005Ool007oo001o
o`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00<007ooOol0`goo00001Goo
001oogoo000027oo0`000goo00@007ooOol0009oo`<000Ioo`03001oogoo01aoo`0:001oo`00Ool0
07oo001oo`00Ool200001goo001oo`00Ool007oo00800005Ool007oo001oo`0200003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0;001o
o`00Ool007oo001oo`00Ool0003:Ool00005Ool007ooOol00008Ool00`00Oomoo`03Ool01000Oomo
o`000Woo00<007ooOol01Woo00<007ooOol06Woo00`007oo001oo`00Ool007oo001oo`00Ool20000
1Goo001oo`00Ool00P0000Moo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool0
07oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`07001oo`00Ool007oo0000cWoo
0002Ool2000:Ool30003Ool20004Ool30005Ool00`00Oomoo`0JOol03000Ool007oo001oo`00Ool0
07oo001oo`800003Ool007oo00800009Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00
Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02000037oo001oo`00
Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007oo0000
dWoo000ROol00`00Oomoo`0JOol03000Ool007oo001oo`00Ool007oo001oo`800003Ool007oo0080
0009Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ao
o`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo0000egoo000ROol2000KOol03000Ool007oo001oo`00Ool0
07oo001oo`80000>Ool00000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`09001oo`00Ool007oo001oo`000=]oo`008Woo00<007ooOol06Woo00`0
07oo001oo`00Ool007oo001oo`00Ool200003Woo000007oo001oo`00Ool007oo001oo`00Ool20000
37oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000ggoo000ROol00`00Oomo
o`0JOol03000Ool007oo001oo`00Ool007oo001oo`<0000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0hGoo000ROol00`00
Oomoo`0GOol03@00Oomoo`00Ool007oo001oo`00Ool007oo0080000@Ool00000Ool007oo001oo`00
Ool007oo001oo`00Ool2000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00/007oo001oo`00Ool007oo001oo`000>Qoo`008Woo00<007ooOol0
5goo00d007ooOol007oo001oo`00Ool007oo001oo`02000047oo000007oo001oo`00Ool007oo001o
o`00Ool007oo0P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`09001oo`00Ool007oo001oo`000>Yoo`008Woo0P0067oo00d007ooOol007oo
001oo`00Ool007oo001oo`02000047oo000007oo001oo`00Ool007oo001oo`00Ool007oo0P0000ao
o`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05
001oo`00Ool0003^Ool0029oo`03001oogoo01Moo`0;001oogoo001oo`00Ool007oo001oo`020000
0goo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Ool0003`Ool0029o
o`03001oogoo01Moo`0;001oogoo001oo`00Ool007oo001oo`0200000goo001oo`02000037oo001o
o`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool0003eOol0029oo`03001oogoo01Moo`09001oogoo001oo`00Ool0
07oo00800005Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00/007oo001oo`00Ool007oo001oo`000?Moo`008Woo00<0
07ooOol05goo00T007ooOol007oo001oo`00Ool00P0000Eoo`00Ool007oo0080000<Ool007oo001o
o`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo
001oo`00Ool0003iOol0029oo`8001Ioo`03001oo`00009oo`04001oo`00Ool200001goo001oo`00
Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool01@00Ool007oo0000oGoo000ROol00`00Oomoo`0EOol00`00Ool00002Ool01000
Ool007oo0P0000Moo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007oo0000ogoo000ROol00`00Oomoo`0EOol0
0`00Ool00002Ool03@00Ool00000Ool007oo001oo`00Ool007oo0080000<Ool007oo001oo`00Ool0
07oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`3oOol0009o
o`8000Aoo`8000Aoo`8000=oo`<000Aoo`8000Ioo`03001oogoo01Eoo`03001oo`00009oo`0=001o
o`00001oo`00Ool007oo001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00
Ool007oo001oo`00Ool007oo001oo`000?moo`Eoo`0000Eoo`00Oomoo`0000=oo`8000=oo`04001o
ogoo0005Ool01000Oomoo`000Woo00<007ooOol00goo00<007ooOol05Goo00<007oo00000Woo00d0
07oo000007oo001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0;
001oo`00Ool007oo001oo`00Ool0003oOol7Ool00005Ool007ooOol00008Ool01000Oomoo`000goo
0`000Woo00@007ooOol000Eoo`<001Eoo`03001oo`00009oo`80000;Ool007oo001oo`00Ool007oo
001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`09001oo`00Ool007oo001oo`000?mo
o`Uoo`0000Eoo`00Oomoo`0000Qoo`<000=oo`04001oogoo0003Ool20006Ool00`00Oomoo`0EOol0
0`00Ool00002Ool200002Woo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool01`00Ool007oo001oo`000?moo`]oo`0000Eoo`00Oomoo`0000Qoo`03001oogoo
00=oo`04001oogoo0002Ool01000Oomoo`001Goo00<007ooOol04goo01<007oo001oo`00Ool007oo
001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05
001oo`00Ool0003oOol=Ool0009oo`8000Yoo`<000=oo`8000Aoo`8000Ioo`03001oogoo01=oo`0C
001oo`00Ool007oo001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool00`00Ool0003oOol?Ool0029oo`03001oogoo01=oo`0C001oo`00Ool007oo
001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool00`00Oomoo`3oOol?Ool0029oo`8001Aoo`04001oo`00Ool3000037oo001oo`00Ool007oo001o
o`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000ogoo57oo000ROol00`00Oomoo`0C
Ool01000Ool007oo0`0000aoo`00Ool007oo001oo`00Ool007oo0002Ool02`00Ool007oo001oo`00
Ool007oo0000ogoo5Woo000ROol00`00Oomoo`0COol01000Ool007oo0`0000aoo`00Ool007oo001o
o`00Ool007oo0002Ool02@00Ool007oo001oo`00Ool0003oOolHOol0029oo`03001oogoo01=oo`03
001oo`0000@0000<Ool007oo001oo`00Ool007oo001oo`000Woo00L007oo001oo`00Ool0003oOolJ
Ool0029oo`03001oogoo01=oo`03001oo`0000@0000<Ool007oo001oo`00Ool007oo001oo`000Woo
00L007oo001oo`00Ool0003oOolJOol0029oo`8001Aoo`03001oo`0000@0000<Ool007oo001oo`00
Ool007oo001oo`000Woo00D007oo001oo`000?mooaaoo`008Woo00<007ooOol04goo00<007oo0000
100000aoo`00Ool007oo001oo`00Ool007oo0002Ool00`00Ool00002Ool30004Ool2000ZOol30004
Ool20004Ool2000VOol30003Ool30004Ool2000WOol40003Ool20004Ool2000VOol40002Ool30004
Ool2000WOol30004Ool20004Ool20003Ool0029oo`03001oogoo015oo`03001oo`0000H0000<Ool0
07oo001oo`00Ool007oo001oo`000Woo00<007ooOol01Goo00@007ooOol0009oo`03001oogoo02Qo
o`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo00<007ooOol01Goo00@007ooOol0
009oo`03001oogoo02Eoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo00<007oo
Ool01Goo00@007ooOol0009oo`03001oogoo02Moo`04001oogoo0002Ool01000Oomoo`000Woo0@00
0Goo0Goo000ROol00`00Oomoo`0AOol00`00Ool00006000037oo001oo`00Ool007oo001oo`00Ool0
009oo`03001oogoo00Eoo`04001oogoo0002Ool00`00Oomoo`0XOol00`00Oomoo`02Ool01000Oomo
o`000Woo00@007ooOol002Ioo`03001oogoo00Eoo`04001oogoo0002Ool00`00Oomoo`0VOol01@00
Oomoogoo00000Woo00@007ooOol0009oo`03001oogoo02Eoo`03001oogoo00Aoo`04001oogoo0002
Ool00`00Oomoo`0WOol01000Oomoo`000Woo00@007ooOol0009oo`40005oo`5oo`008Woo00<007oo
Ool04Goo00<007oo00001P0000aoo`00Ool007oo001oo`00Ool007oo0007Ool30003Ool01000Oomo
o`00:Woo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo000VOol00`00Oomoo`02Ool30003
Ool01000Oomoo`00:Goo00@007ooOol0009oo`04001oogoo0002Ool00`00Oomoo`0VOol00`00Oomo
o`030003Ool01000Oomoo`009goo0P000goo00@007ooOol0009oo`04001oogoo0002Ool0029oo`80
019oo`03001oo`0000H0000:Ool007oo001oo`00Ool007oo0009Ool00`00Oomoo`03Ool01000Oomo
o`00:Woo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo000VOol00`00Oomoo`02Ool00`00
Oomoo`03Ool01000Oomoo`009Woo00@007ooOol0009oo`04001oogoo0002Ool01000Oomoo`009Goo
00@007ooOol0009oo`03001oogoo00=oo`04001oogoo000YOol01000Oomoo`000Woo00@007ooOol0
009oo`40005oo`5oo`008Woo00<007ooOol04Goo00<007oo00001P0000Yoo`00Ool007oo001oo`00
Ool000Uoo`@000=oo`8002Yoo`8000Eoo`8000Aoo`8002Ioo`8000Aoo`@000=oo`8002Qoo`8000Ao
o`8000Aoo`8002Moo`8000=oo`@000=oo`8002Moo`<000Aoo`8000Aoo`8000=oo`008Woo00<007oo
Ool04Goo2@0000Qoo`00Ool007oo001oo`00ogoo9goo000ROol00`00Oomoo`0AOol9000027oo001o
o`00Ool007oo003oOolWOol0029oo`03001oogoo015oo`T00006Ool007oo001oo`00ogoo:Goo000J
Oooo001C0001Ool00001\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 0.68954, 
  0.933765, 0.000101968}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[explot, \ JRPlotCApp, \ \ PlotRange\  -> \ {0.69, \ 0.70}, \ 
      DisplayFunction\  -> \ $DisplayFunction]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -42.6443 61.8034 [
[.18254 .60553 -6 -9 ]
[.18254 .60553 6 0 ]
[.34127 .60553 -9 -9 ]
[.34127 .60553 9 0 ]
[.5 .60553 -9 -9 ]
[.5 .60553 9 0 ]
[.65873 .60553 -9 -9 ]
[.65873 .60553 9 0 ]
[.81746 .60553 -9 -9 ]
[.81746 .60553 9 0 ]
[.97619 .60553 -9 -9 ]
[.97619 .60553 9 0 ]
[.01131 0 -24 -4.5 ]
[.01131 0 0 4.5 ]
[.01131 .12361 -30 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -30 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -30 -4.5 ]
[.01131 .49443 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .62428 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .61803 m
.18254 .62428 L
s
[(50)] .18254 .60553 0 1 Mshowa
.34127 .61803 m
.34127 .62428 L
s
[(100)] .34127 .60553 0 1 Mshowa
.5 .61803 m
.5 .62428 L
s
[(150)] .5 .60553 0 1 Mshowa
.65873 .61803 m
.65873 .62428 L
s
[(200)] .65873 .60553 0 1 Mshowa
.81746 .61803 m
.81746 .62428 L
s
[(250)] .81746 .60553 0 1 Mshowa
.97619 .61803 m
.97619 .62428 L
s
[(300)] .97619 .60553 0 1 Mshowa
.125 Mabswid
.05556 .61803 m
.05556 .62178 L
s
.0873 .61803 m
.0873 .62178 L
s
.11905 .61803 m
.11905 .62178 L
s
.15079 .61803 m
.15079 .62178 L
s
.21429 .61803 m
.21429 .62178 L
s
.24603 .61803 m
.24603 .62178 L
s
.27778 .61803 m
.27778 .62178 L
s
.30952 .61803 m
.30952 .62178 L
s
.37302 .61803 m
.37302 .62178 L
s
.40476 .61803 m
.40476 .62178 L
s
.43651 .61803 m
.43651 .62178 L
s
.46825 .61803 m
.46825 .62178 L
s
.53175 .61803 m
.53175 .62178 L
s
.56349 .61803 m
.56349 .62178 L
s
.59524 .61803 m
.59524 .62178 L
s
.62698 .61803 m
.62698 .62178 L
s
.69048 .61803 m
.69048 .62178 L
s
.72222 .61803 m
.72222 .62178 L
s
.75397 .61803 m
.75397 .62178 L
s
.78571 .61803 m
.78571 .62178 L
s
.84921 .61803 m
.84921 .62178 L
s
.88095 .61803 m
.88095 .62178 L
s
.9127 .61803 m
.9127 .62178 L
s
.94444 .61803 m
.94444 .62178 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.02381 0 m
.03006 0 L
s
[(0.69)] .01131 0 1 0 Mshowa
.02381 .12361 m
.03006 .12361 L
s
[(0.692)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.694)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.696)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.698)] .01131 .49443 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .30271 m
.0746 .30271 L
.07778 .30271 L
.08095 .30271 L
.08413 .30271 L
.0873 .30271 L
.09048 .30271 L
.09365 .30271 L
.09683 .30271 L
.1 .30271 L
.10317 .30271 L
.10635 .30271 L
.10952 .30271 L
.1127 .30271 L
.11587 .30271 L
.11905 .30271 L
.12222 .30271 L
.1254 .30271 L
.12857 .30271 L
.13175 .30271 L
.13492 .30271 L
.1381 .30271 L
.14127 .30271 L
.14444 .30271 L
.14762 .30271 L
.15079 .30271 L
.15397 .30271 L
.15714 .30271 L
.16032 .30271 L
.16349 .30271 L
.16667 .30271 L
.16984 .30271 L
.17302 .30271 L
.17619 .30271 L
.17937 .30271 L
.18254 .30271 L
.18571 .30271 L
.18889 .30271 L
.19206 .30271 L
.19524 .30271 L
.19841 .30271 L
.20159 .30271 L
.20476 .30271 L
.20794 .30271 L
.21111 .30271 L
.21429 .30271 L
.21746 .30271 L
.22063 .30271 L
.22381 .30271 L
.22698 .30271 L
Mistroke
.23016 .30271 L
.23333 .30271 L
.23651 .30271 L
.23968 .30271 L
.24286 .30271 L
.24603 .30271 L
.24921 .30271 L
.25238 .30271 L
.25556 .30271 L
.25873 .30271 L
.2619 .30271 L
.26508 .30271 L
.26825 .30271 L
.27143 .30271 L
.2746 .30271 L
.27778 .30271 L
.28095 .30271 L
.28413 .30271 L
.2873 .30271 L
.29048 .30271 L
.29365 .30271 L
.29683 .30271 L
.3 .30271 L
.30317 .30271 L
.30635 .30271 L
.30952 .30271 L
.3127 .30271 L
.31587 .30271 L
.31905 .30271 L
.32222 .30271 L
.3254 .30271 L
.32857 .30271 L
.33175 .30271 L
.33492 .30271 L
.3381 .30271 L
.34127 .30271 L
.34444 .30271 L
.34762 .30271 L
.35079 .30271 L
.35397 .30271 L
.35714 .30271 L
.36032 .30271 L
.36349 .30271 L
.36667 .30271 L
.36984 .30271 L
.37302 .30271 L
.37619 .30271 L
.37937 .30271 L
.38254 .30271 L
.38571 .30271 L
Mistroke
.38889 .30271 L
.39206 .30271 L
.39524 .30271 L
.39841 .30271 L
.40159 .30271 L
.40476 .30271 L
.40794 .30271 L
.41111 .30271 L
.41429 .30271 L
.41746 .30271 L
.42063 .30271 L
.42381 .30271 L
.42698 .30271 L
.43016 .30271 L
.43333 .30271 L
.43651 .30271 L
.43968 .30271 L
.44286 .30271 L
.44603 .30271 L
.44921 .30271 L
.45238 .30271 L
.45556 .30271 L
.45873 .30271 L
.4619 .30271 L
.46508 .30271 L
.46825 .30271 L
.47143 .30271 L
.4746 .30271 L
.47778 .30271 L
.48095 .30271 L
.48413 .30271 L
.4873 .30271 L
.49048 .30271 L
.49365 .30271 L
.49683 .30271 L
.5 .30271 L
.50317 .30271 L
.50635 .30271 L
.50952 .30271 L
.5127 .30271 L
.51587 .30271 L
.51905 .30271 L
.52222 .30271 L
.5254 .30271 L
.52857 .30271 L
.53175 .30271 L
.53492 .30271 L
.5381 .30271 L
.54127 .30271 L
.54444 .30271 L
Mistroke
.54762 .30271 L
.55079 .30271 L
.55397 .30271 L
.55714 .30271 L
.56032 .30271 L
.56349 .30271 L
.56667 .30271 L
.56984 .30271 L
.57302 .30271 L
.57619 .30271 L
.57937 .30271 L
.58254 .30271 L
.58571 .30271 L
.58889 .30271 L
.59206 .30271 L
.59524 .30271 L
.59841 .30271 L
.60159 .30271 L
.60476 .30271 L
.60794 .30271 L
.61111 .30271 L
.61429 .30271 L
.61746 .30271 L
.62063 .30271 L
.62381 .30271 L
.62698 .30271 L
.63016 .30271 L
.63333 .30271 L
.63651 .30271 L
.63968 .30271 L
.64286 .30271 L
.64603 .30271 L
.64921 .30271 L
.65238 .30271 L
.65556 .30271 L
.65873 .30271 L
.6619 .30271 L
.66508 .30271 L
.66825 .30271 L
.67143 .30271 L
.6746 .30271 L
.67778 .30271 L
.68095 .30271 L
.68413 .30271 L
.6873 .30271 L
.69048 .30271 L
.69365 .30271 L
.69683 .30271 L
.7 .30271 L
.70317 .30271 L
Mistroke
.70635 .30271 L
.70952 .30271 L
.7127 .30271 L
.71587 .30271 L
.71905 .30271 L
.72222 .30271 L
.7254 .30271 L
.72857 .30271 L
.73175 .30271 L
.73492 .30271 L
.7381 .30271 L
.74127 .30271 L
.74444 .30271 L
.74762 .30271 L
.75079 .30271 L
.75397 .30271 L
.75714 .30271 L
.76032 .30271 L
.76349 .30271 L
.76667 .30271 L
.76984 .30271 L
.77302 .30271 L
.77619 .30271 L
.77937 .30271 L
.78254 .30271 L
.78571 .30271 L
.78889 .30271 L
.79206 .30271 L
.79524 .30271 L
.79841 .30271 L
.80159 .30271 L
.80476 .30271 L
.80794 .30271 L
.81111 .30271 L
.81429 .30271 L
.81746 .30271 L
.82063 .30271 L
.82381 .30271 L
.82698 .30271 L
.83016 .30271 L
.83333 .30271 L
.83651 .30271 L
.83968 .30271 L
.84286 .30271 L
.84603 .30271 L
.84921 .30271 L
.85238 .30271 L
.85556 .30271 L
.85873 .30271 L
.8619 .30271 L
Mistroke
.86508 .30271 L
.86825 .30271 L
.87143 .30271 L
.8746 .30271 L
.87778 .30271 L
.88095 .30271 L
.88413 .30271 L
.8873 .30271 L
.89048 .30271 L
.89365 .30271 L
.89683 .30271 L
.9 .30271 L
.90317 .30271 L
.90635 .30271 L
.90952 .30271 L
.9127 .30271 L
.91587 .30271 L
.91905 .30271 L
.92222 .30271 L
.9254 .30271 L
.92857 .30271 L
.93175 .30271 L
.93492 .30271 L
.9381 .30271 L
.94127 .30271 L
.94444 .30271 L
.94762 .30271 L
.95079 .30271 L
.95397 .30271 L
.95714 .30271 L
.96032 .30271 L
.96349 .30271 L
.96667 .30271 L
.96984 .30271 L
.97302 .30271 L
.97619 .30271 L
Mfstroke
.07566 .61803 m
.07778 .58033 L
s
.07778 .58033 m
.07993 .61803 L
s
.08142 .61803 m
.08413 .51335 L
s
.08413 .51335 m
.08694 .61803 L
s
.08755 .61803 m
.09048 .45919 L
s
.09048 .45919 m
.09351 .61803 L
s
.09376 .61803 m
.09683 .41449 L
.1 .61759 L
.10317 .37697 L
.10635 .60927 L
.10952 .34502 L
.1127 .6006 L
.11587 .3175 L
.11905 .59182 L
.12222 .29355 L
.1254 .58307 L
.12857 .2725 L
.13175 .57442 L
.13492 .25387 L
.1381 .56594 L
.14127 .23726 L
.14444 .55767 L
.14762 .22235 L
.15079 .54962 L
.15397 .20891 L
.15714 .54181 L
.16032 .19672 L
.16349 .53423 L
.16667 .18561 L
.16984 .5269 L
.17302 .17545 L
.17619 .51979 L
.17937 .16613 L
.18254 .51292 L
.18571 .15753 L
.18889 .50627 L
.19206 .14959 L
.19524 .49983 L
.19841 .14223 L
.20159 .4936 L
.20476 .13538 L
.20794 .48757 L
.21111 .129 L
.21429 .48173 L
.21746 .12303 L
.22063 .47607 L
.22381 .1265 L
.22698 .47058 L
.23016 .14249 L
.23333 .46526 L
.23651 .15722 L
.23968 .46009 L
.24286 .1708 L
.24603 .45508 L
.24921 .18335 L
Mistroke
.25238 .45021 L
.25556 .19495 L
.25873 .44549 L
.2619 .20571 L
.26508 .44089 L
.26825 .21568 L
.27143 .43642 L
.2746 .22494 L
.27778 .43207 L
.28095 .23355 L
.28413 .42784 L
.2873 .24156 L
.29048 .42372 L
.29365 .24902 L
.29683 .41971 L
.3 .25598 L
.30317 .4158 L
.30635 .26247 L
.30952 .41199 L
.3127 .26853 L
.31587 .40827 L
.31905 .27419 L
.32222 .40464 L
.3254 .27948 L
.32857 .4011 L
.33175 .28443 L
.33492 .39764 L
.3381 .28906 L
.34127 .39427 L
.34444 .29339 L
.34762 .39097 L
.35079 .29745 L
.35397 .38774 L
.35714 .30126 L
.36032 .38459 L
.36349 .30482 L
.36667 .3815 L
.36984 .30817 L
.37302 .37849 L
.37619 .3113 L
.37937 .37553 L
.38254 .31424 L
.38571 .37264 L
.38889 .31699 L
.39206 .36981 L
.39524 .31957 L
.39841 .36704 L
.40159 .32199 L
.40476 .36432 L
.40794 .32426 L
Mistroke
.41111 .36166 L
.41429 .32639 L
.41746 .35904 L
.42063 .32839 L
.42381 .35648 L
.42698 .33025 L
.43016 .35397 L
.43333 .332 L
.43651 .35151 L
.43968 .33364 L
.44286 .34909 L
.44603 .33517 L
.44921 .34672 L
.45238 .33661 L
.45556 .34439 L
.45873 .33795 L
.4619 .3421 L
.46508 .3392 L
.46825 .33985 L
.47143 .34036 L
.4746 .33764 L
.47778 .34145 L
.48095 .33547 L
.48413 .34246 L
.4873 .33334 L
.49048 .3434 L
.49365 .33124 L
.49683 .34427 L
.5 .32918 L
.50317 .34508 L
.50635 .32716 L
.50952 .34583 L
.5127 .32516 L
.51587 .34652 L
.51905 .3232 L
.52222 .34715 L
.5254 .32127 L
.52857 .34774 L
.53175 .31937 L
.53492 .34827 L
.5381 .3175 L
.54127 .34876 L
.54444 .31566 L
.54762 .3492 L
.55079 .31385 L
.55397 .3496 L
.55714 .31206 L
.56032 .34996 L
.56349 .31031 L
.56667 .35029 L
Mistroke
.56984 .30857 L
.57302 .35057 L
.57619 .30687 L
.57937 .35082 L
.58254 .30519 L
.58571 .35104 L
.58889 .30353 L
.59206 .35123 L
.59524 .30189 L
.59841 .35138 L
.60159 .30028 L
.60476 .35151 L
.60794 .2987 L
.61111 .35161 L
.61429 .29713 L
.61746 .35169 L
.62063 .29559 L
.62381 .35173 L
.62698 .29406 L
.63016 .35176 L
.63333 .29256 L
.63651 .35176 L
.63968 .29108 L
.64286 .35175 L
.64603 .28962 L
.64921 .35171 L
.65238 .28817 L
.65556 .35165 L
.65873 .28675 L
.6619 .35157 L
.66508 .28534 L
.66825 .35148 L
.67143 .28395 L
.6746 .35136 L
.67778 .28258 L
.68095 .35123 L
.68413 .28123 L
.6873 .35109 L
.69048 .27989 L
.69365 .35093 L
.69683 .27857 L
.7 .35076 L
.70317 .27727 L
.70635 .35057 L
.70952 .27598 L
.7127 .35037 L
.71587 .27471 L
.71905 .35016 L
.72222 .27345 L
.7254 .34993 L
Mistroke
.72857 .27221 L
.73175 .3497 L
.73492 .27098 L
.7381 .34945 L
.74127 .26977 L
.74444 .34919 L
.74762 .26857 L
.75079 .34893 L
.75397 .26739 L
.75714 .34865 L
.76032 .26621 L
.76349 .34837 L
.76667 .26506 L
.76984 .34807 L
.77302 .26391 L
.77619 .34777 L
.77937 .26278 L
.78254 .34746 L
.78571 .26166 L
.78889 .34714 L
.79206 .26055 L
.79524 .34682 L
.79841 .25945 L
.80159 .34649 L
.80476 .25837 L
.80794 .34615 L
.81111 .2573 L
.81429 .34581 L
.81746 .25798 L
.82063 .34546 L
.82381 .25972 L
.82698 .3451 L
.83016 .26142 L
.83333 .34474 L
.83651 .26308 L
.83968 .34438 L
.84286 .2647 L
.84603 .34401 L
.84921 .26628 L
.85238 .34363 L
.85556 .26781 L
.85873 .34325 L
.8619 .26931 L
.86508 .34287 L
.86825 .27078 L
.87143 .34248 L
.8746 .2722 L
.87778 .34209 L
.88095 .27359 L
.88413 .34169 L
Mistroke
.8873 .27495 L
.89048 .34129 L
.89365 .27627 L
.89683 .34089 L
.9 .27756 L
.90317 .34048 L
.90635 .27882 L
.90952 .34008 L
.9127 .28005 L
.91587 .33966 L
.91905 .28124 L
.92222 .33925 L
.9254 .28241 L
.92857 .33883 L
.93175 .28355 L
.93492 .33842 L
.9381 .28466 L
.94127 .338 L
.94444 .28575 L
.94762 .33757 L
.95079 .28681 L
.95397 .33715 L
.95714 .28784 L
.96032 .33672 L
.96349 .28885 L
.96667 .33629 L
.96984 .28983 L
.97302 .33586 L
.97619 .29079 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00OogooKWoo003oOom^
Ool000Qoo`8000Aoo`8000Aoo`8000=oo`<00?mooe9oo`001goo00@007ooOol000=oo`8000=oo`04
001oogoo0005Ool00`00Oomoo`3oOom?Ool000Moo`04001oogoo0008Ool01000Oomoo`000goo0`00
1Goo0`00ogooBGoo0007Ool01000Oomoo`0027oo0`000goo00@007ooOol000Eoo`03001oogoo0?mo
odUoo`001goo00@007ooOol000Qoo`03001oogoo00=oo`04001oogoo0005Ool00`00Oomoo`3oOom9
Ool000Qoo`8000Yoo`<000=oo`8000Ioo`03001oogoo0?moodUoo`008Woo00<007ooOol0ogooBGoo
000ROol2003oOom:Ool0029oo`03001oogoo0?moodUoo`008Woo00<007ooOol0ogooBGoo000ROol0
0`00Oomoo`3oOom9Ool0029oo`03001oogoo0?moodUoo`008Woo0P00ogooBWoo000ROol00`00Oomo
o`3oOom9Ool0029oo`03001oogoo0?moodUoo`008Woo00<007ooOol0ogooBGoo000ROol00`00Oomo
o`3oOom9Ool0029oo`800?moodYoo`008Woo00<007ooOol0ogooBGoo000ROol00`00Oomoo`3oOom9
Ool0009oo`8000Aoo`8000Aoo`8000=oo`<000=oo`@000Eoo`03001oogoo0?moodUoo`0000Eoo`00
Oomoo`0000=oo`8000=oo`04001oogoo0005Ool01@00Oomoogoo00001goo00<007ooOol0ogooBGoo
00001Goo001oogoo000027oo00@007ooOol000=oo`<000Aoo`03001oogoo00Aoo`<003ioo`03001o
ogoo0?moo`Qoo`0000Eoo`00Oomoo`0000Qoo`<000=oo`04001oogoo0005Ool00`00Oomoo`03Ool0
0`00Oomoo`0lOol01@00Ool007oo0000ogoo27oo00001Goo001oogoo000027oo00<007ooOol00goo
00@007ooOol0009oo`04001oogoo0005Ool00`00Oomoo`0jOol01`00Ool007oo001oo`000?moo`Qo
o`000Woo0P002Woo0`000goo0P0017oo0P001Woo00<007ooOol0>7oo00/007oo001oo`00Ool007oo
001oo`000?moo`Ioo`008Woo00<007ooOol0=Goo00h007ooOol007oo001oo`00Ool007oo001oo`00
ogoo1Woo000ROol2000dOol00`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?mo
o`Aoo`008Woo00<007ooOol0<goo00<007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
003oOol4Ool0029oo`03001oogoo035oo`05001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`000?moo`Aoo`008Woo00<007ooOol0;goo00L007oo001oo`00Ool00002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`03001oogoo0?ioo`008Woo00<007ooOol0;goo00L007oo
001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oogoo0?ioo`00
8Woo0P00;Woo00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool00`00Ool0003nOol0029oo`03001oogoo02eoo`09001oo`00Ool007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00D007oo001oo`000?aoo`008Woo00<007ooOol0
:goo00/007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00D007oo001oo`000?aoo`008Woo00<007ooOol0:goo00/007oo001oo`00Ool007oo001oo`00
009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0003jOol0029o
o`03001oogoo02Uoo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool01`00Ool007oo001oo`000?Yoo`008Woo0P00:Woo00d007oo001oo`00
Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`09001oo`00
Ool007oo001oo`000?Qoo`008Woo00<007ooOol09Woo010007ooOol007oo001oo`00Ool007oo001o
o`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00/007oo001oo`00Ool007oo
001oo`000?Ioo`008Woo00<007ooOol09Woo010007ooOol007oo001oo`00Ool007oo001oo`00Ool0
009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00
Ool0003dOol0009oo`8000Aoo`8000Aoo`8000=oo`<000Aoo`<000Eoo`03001oogoo02Aoo`03001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00`007oo001oo`00Ool007oo001o
o`00Ool200003Woo001oo`00Ool007oo001oo`00Ool007oo003dOol00005Ool007ooOol00003Ool2
0003Ool01000Oomoo`001Goo00<007ooOol00Woo00<007ooOol017oo00<007ooOol097oo00<007oo
00000Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool0
07oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`03001oogoo0>moo`0000Eoo`00
Oomoo`0000Qoo`04001oogoo0003Ool30002Ool40005Ool3000TOol00`00Ool00002Ool03000Ool0
07oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00
Ool007oo001oo`00Ool007oo001oo`000Woo00<007oo0000kgoo00001Goo001oogoo000027oo0`00
0goo00@007ooOol0009oo`03001oo`0000Ioo`03001oogoo029oo`05001oo`00Ool00002Ool03000
Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000io
o`00Ool007oo001oo`00Ool007oo001oo`000Woo00D007oo001oo`000:Uoo`05001oo`00Ool0000o
Ool00005Ool007ooOol00008Ool00`00Oomoo`03Ool01000Oomoo`000Woo00<007oo00001Woo00<0
07ooOol08Woo00D007oo001oo`00009oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool0
1`00Ool007oo001oo`0009Yoo`03001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00L007oo001oo`00Ool0000fOol0009oo`8000Yoo`<000=oo`8000Aoo`8000Ioo`03001oogoo
029oo`05001oo`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00T007oo
001oo`00Ool007oo0000SWoo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007oo
Ool0:goo000ROol00`00Oomoo`0POol01`00Ool007oo001oo`00009oo`0<001oo`00Ool007oo001o
o`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo
001oo`00Ool007oo0002Ool02`00Ool007oo001oo`00Ool007oo0000PGoo00T007oo001oo`00Ool0
07oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00/007oo001oo`00Ool0
07oo001oo`0002=oo`008Woo0P008Goo00L007oo001oo`00Ool00002Ool03000Ool007oo001oo`00
Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001o
o`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool0001dOol01@00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000ioo`00Ool007oo001oo`00Ool0
07oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`00
01Qoo`008Woo00<007ooOol087oo00L007oo001oo`00Ool00002Ool03000Ool007oo001oo`00Ool0
07oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00
Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00Ool0001WOol0
3@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo00002goo000R
Ool00`00Oomoo`0NOol02@00Ool007oo001oo`00Ool00002Ool03000Ool007oo001oo`00Ool007oo
001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool01@00Ool007oo0000G7oo
00L007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Goo001o
o`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000io
o`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool01`00Ool007oo001oo`0000Uoo`008Woo00<007ooOol03Gooo`00<`002Goo000ROol2000OOol0
2@00Ool007oo001oo`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`800005Ool007oo
001oo`0200000goo001oo`0500003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo
001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000@goo00T0
07oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03000Ool0
07oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200001Goo001oo`00Ool00P0000Moo`00Ool007oo001oo`02
00001goo001oo`00Ool007oo00800003Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02000027oo001oo`00Ool007oo0009Ool0
029oo`03001oogoo01ioo`08001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000=oo`00Ool00P0001Moo`00Ool007oo001oo`00Ool007oo001oo`00Ool007oo001o
o`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00<007oo0000=goo010007ooOol007oo001oo`00Ool007oo001oo`00
Ool0009oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00<0000=Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Moo`00Ool007oo
001oo`0200000goo001oo`02000027oo001oo`00Ool007oo0009Ool0029oo`03001oogoo01aoo`0:
001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool0
0000Ool007oo001oo`00Ool007oo001oo`80000<Ool00000Ool007oo001oo`00Ool007oo0P0000eo
o`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002
Ool01`00Ool007oo001oo`0002]oo`09001oo`00Ool007oo001oo`00009oo`0<001oo`00Ool007oo
001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200001goo001oo`00Ool0
07oo00800005Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool0
07oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
3Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200001Woo001oo`00Ool000Yoo`008Woo00<0
07ooOol077oo00X007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00
Ool00`0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200001Goo001oo`00Ool0100000=oo`00
Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00
Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`0001moo`0>001oo`00Ool007oo001o
o`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Eoo`00Ool007oo
00800007Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00aoo`00Ool007oo001oo`00Ool007oo0002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool2
00003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000Ioo`00Ool007oo000:Ool0029oo`03001oogoo01aoo`0:001oo`00Ool007oo001oo`00
Ool200002goo001oo`00Ool007oo001oo`00Ool00P00011oo`00001oo`00Ool007oo001oo`00Ool0
07oo001oo`800009Ool007oo001oo`00Ool007oo00D0000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00
Ool007oo0002Ool01`00Ool007oo001oo`0000moo`05001oo`00Ool00002Ool03P00Ool007oo001o
o`00Ool007oo001oo`00Ool200000goo001oo`0200002Goo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00L007oo001oo`00Ool0000:Ool0029oo`8001eoo`0:001oo`00Ool007oo001oo`00
Ool200002Goo001oo`00Ool007oo001oo`0200000goo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000=oo`000000100000Eo
o`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00800003Ool000000080
000=Ool007oo001oo`00Ool007oo001oo`00Ool00`0000Uoo`00Ool007oo001oo`00Ool00P0000eo
o`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080
000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0000:
Ool0029oo`03001oogoo01Yoo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000Uoo`00Ool007oo
001oo`00Ool00P0000=oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000;Ool007oo001oo`00Ool007oo001oo`0300000goo
001oo`0500001Goo00000000Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200001Goo
001oo`00Ool00P0000Ioo`00000007oo0002Ool00`00Oomoo`02Ool02@00Ool007oo001oo`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool0
07oo001oo`00Ool007oo00000Woo00/007oo001oo`00Ool007oo001oo`0002Aoo`008Woo00<007oo
Ool06Woo00`007oo001oo`00Ool007oo001oo`00Ool200001goo001oo`00Ool007oo00800005Ool0
07oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Qoo`00Ool0
07oo001oo`006Woo00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`05001oo`00Ool00018Ool0009oo`8000Aoo`8000Aoo`8000=oo`<000Ao
o`8000Ioo`03001oogoo01Yoo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000Eoo`00Ool007oo
00800007Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009o
o`05001oo`00Ool00031Ool00005Ool007ooOol00003Ool20003Ool01000Oomoo`001Goo00@007oo
Ool0009oo`03001oogoo00=oo`03001oogoo01Yoo`0<001oo`00Ool007oo001oo`00Ool007oo0P00
00=oo`00Ool00P0000Uoo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`000<Qoo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0003Ool30002Ool01000Oomoo`00
1Goo0`006Woo00`007oo001oo`00Ool007oo001oo`00Ool200000goo001oo`0200002Goo001oo`00
Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00
Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02000037oo001oo`00
Ool007oo001oo`00Ool0009oo`09001oo`00Ool007oo001oo`000<aoo`0000Eoo`00Oomoo`0000Qo
o`<000=oo`04001oogoo0002Ool30006Ool00`00Oomoo`0GOol03`00Oomoo`00Ool007oo001oo`00
Ool007oo001oo`0200003Woo000007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000aoo`00Ool0
07oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00
Ool0003@Ool00005Ool007ooOol00008Ool00`00Oomoo`03Ool01000Oomoo`000Woo00<007ooOol0
1Woo00<007ooOol05goo00l007ooOol007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00001o
o`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02000037oo
001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool0003GOol0009oo`8000Yoo`<000=oo`8000Aoo`<000Eoo`03
001oogoo01Moo`0?001oogoo001oo`00Ool007oo001oo`00Ool007oo00<0000=Ool007oo001oo`00
Ool007oo001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00T007oo001o
o`00Ool007oo0000fgoo000ROol00`00Oomoo`0GOol03@00Oomoo`00Ool007oo001oo`00Ool007oo
0080000@Ool00000Ool007oo001oo`00Ool007oo001oo`00Ool2000037oo001oo`00Ool007oo001o
o`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool01@00Ool007oo0000ggoo000ROol2000HOol03@00Oomoo`00Ool007oo001o
o`00Ool007oo0080000@Ool00000Ool007oo001oo`00Ool007oo001oo`00Ool2000037oo001oo`00
Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool00`00Ool0003QOol0029oo`03001oogoo01Moo`0;001oogoo
001oo`00Ool007oo001oo`0200000goo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`000>Ioo`008Woo00<007ooOol05Goo00<007oo
00000Woo00P007oo001oo`00Ool007oo0P0000=oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00T007oo001oo`00Ool007oo0000jWoo000ROol00`00Oomoo`0EOol00`00Ool0
0002Ool01P00Ool007oo001oo`800005Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00
Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool01@00Ool007oo0000kWoo000ROol00`00Oomoo`0EOol00`00Ool00002Ool01P00
Ool007oo001oo`800005Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool00`00Ool0003`Ool0029oo`8001Ioo`03001oo`00009oo`04001oo`00Ool200001goo001oo`00
Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?Eoo`008Woo00<007ooOol05Goo
00T007oo000007oo001oo`00Ool00P0000Moo`00Ool007oo001oo`02000037oo001oo`00Ool007oo
001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00/007oo001oo`00Ool0
07oo001oo`000?Moo`008Woo00<007ooOol05Goo00L007oo000007oo001oo`0200002Goo001oo`00
Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001o
o`00Ool007oo00000Woo00L007oo001oo`00Ool0003kOol0029oo`03001oogoo01Eoo`07001oo`00
001oo`00Ool00P0000Uoo`00Ool007oo001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0003mOol0029oo`03
001oogoo01=oo`04001oo`00Ool200000goo001oo`0200002Goo001oo`00Ool007oo001oo`020000
37oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00<007oo0000ogoo000ROol2000DOol01000Ool007oo0P0000ioo`00001oo`00Ool007oo001oo`00
Ool007oo0P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo
001oo`000?moo`Eoo`008Woo00<007ooOol04goo00@007oo001oo`80000>Ool00000Ool007oo001o
o`00Ool007oo001oo`80000<Ool007oo001oo`00Ool007oo001oo`000Woo00/007oo001oo`00Ool0
07oo001oo`000?moo`Moo`008Woo00<007ooOol04goo00@007oo001oo`80000=Ool00000Ool007oo
001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo
001oo`00Ool0003oOol9Ool0009oo`8000Aoo`8000Aoo`8000=oo`<000Aoo`8000Ioo`03001oogoo
01=oo`04001oo`00Ool3000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0003oOol;Ool00005Ool007ooOol00003Ool2
0003Ool01000Oomoo`001Goo00@007ooOol0009oo`03001oogoo00=oo`03001oogoo01=oo`04001o
o`00Ool3000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00D007oo001oo`000?moo`eoo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0003Ool3
0002Ool01000Oomoo`001Goo0`004goo00@007oo001oo`<0000<Ool007oo001oo`00Ool007oo001o
o`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Ool0003oOol?Ool00005Ool0
07ooOol00008Ool30003Ool01000Oomoo`000goo0P001Woo00<007ooOol04goo00D007oo000007oo
0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool00`00Oomoo`3oOol?Ool00005Ool007ooOol00008Ool00`00Oomoo`03Ool01000Oomoo`00
0Woo00@007ooOol000Eoo`03001oogoo01=oo`05001oo`00001oo`02000037oo001oo`00Ool007oo
001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000ogoo57oo0002Ool2000:Ool3
0003Ool20004Ool20006Ool00`00Oomoo`0AOol01000Ool007oo0P0000moo`00001oo`00Ool007oo
001oo`00Ool007oo00000Woo00/007oo001oo`00Ool007oo001oo`000?mooaIoo`008Woo00<007oo
Ool04Goo00L007oo000007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`09001o
o`00Ool007oo001oo`000?mooaQoo`008Woo0P004Woo00L007oo000007oo001oo`02000037oo001o
o`00Ool007oo001oo`00Ool0009oo`07001oo`00Ool007oo0000ogoo6Woo000ROol00`00Oomoo`0A
Ool01`00Ool00000Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00D007oo001o
o`000?mooaaoo`008Woo00<007ooOol04Goo00L007oo000007oo001oo`02000037oo001oo`00Ool0
07oo001oo`00Ool0009oo`03001oo`000?mooaioo`008Woo00<007ooOol04Goo0P0000Eoo`00Ool0
07oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00<007ooOol0ogoo7Woo000ROol00`00
Oomoo`0AOol200001Goo001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool00`00
Oomoo`3oOolNOol0029oo`80019oo`800004Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo
001oo`000?moob=oo`008Woo00<007ooOol04Goo0P0000Aoo`00Ool0009oo`0;001oo`00Ool007oo
001oo`00Ool00009Ool30004Ool2000ZOol30004Ool20004Ool2000VOol30003Ool30004Ool2000W
Ool40003Ool20004Ool2000VOol40002Ool30004Ool2000WOol30004Ool20004Ool20003Ool0029o
o`03001oogoo011oo`07001oo`00Ool007oo00000Woo00T007oo001oo`00Ool007oo00003Woo00@0
07ooOol0009oo`03001oogoo02Qoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo
00<007ooOol01Goo00@007ooOol0009oo`03001oogoo02Eoo`03001oogoo009oo`04001oogoo0002
Ool01000Oomoo`009Woo00<007ooOol01Goo00@007ooOol0009oo`03001oogoo02Moo`04001oogoo
0002Ool01000Oomoo`000Woo0@000Goo0Goo000ROol00`00Oomoo`0@Ool01`00Ool007oo001oo`00
009oo`09001oo`00Ool007oo001oo`0000ioo`04001oogoo0002Ool00`00Oomoo`0XOol00`00Oomo
o`02Ool01000Oomoo`000Woo00@007ooOol002Ioo`03001oogoo00Eoo`04001oogoo0002Ool00`00
Oomoo`0VOol01@00Oomoogoo00000Woo00@007ooOol0009oo`03001oogoo02Eoo`03001oogoo00Ao
o`04001oogoo0002Ool00`00Oomoo`0WOol01000Oomoo`000Woo00@007ooOol0009oo`40005oo`5o
o`008Woo00<007ooOol03goo0P0000Ioo`00Ool007oo0002Ool01`00Ool007oo001oo`0000eoo`<0
00=oo`04001oogoo000ZOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol002Ioo`03001o
ogoo009oo`<000=oo`04001oogoo000YOol01000Oomoo`000Woo00@007ooOol0009oo`03001oogoo
02Ioo`03001oogoo00<000=oo`04001oogoo000WOol20003Ool01000Oomoo`000Woo00@007ooOol0
009oo`008Woo0P0047oo0P0000Ioo`00Ool007oo0002Ool01@00Ool007oo00003goo00<007ooOol0
0goo00@007ooOol002Yoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo00<007oo
Ool00Woo00<007ooOol00goo00@007ooOol002Ioo`04001oogoo0002Ool01000Oomoo`000Woo00@0
07ooOol002Eoo`04001oogoo0002Ool00`00Oomoo`03Ool01000Oomoo`00:Goo00@007ooOol0009o
o`04001oogoo0002Ool10001Ool1Ool0029oo`03001oogoo00moo`800006Ool007oo001oo`000Woo
00<007oo00004Goo10000goo0P00:Woo0P001Goo0P0017oo0P009Woo0P0017oo10000goo0P00:7oo
0P0017oo0P0017oo0P009goo0P000goo10000goo0P009goo0`0017oo0P0017oo0P000goo000ROol0
0`00Oomoo`0?Ool200001Woo001oo`00Ool0009oo`03001oo`000?moobeoo`008Woo00<007ooOol0
3Woo00T007oo001oo`00Ool007oo00000Woo00<007ooOol0ogoo;Goo000ROol00`00Oomoo`0>Ool0
2@00Ool007oo001oo`00Ool0003oOolbOol001Yoool005<0005oo`00\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 0.68954, 
  0.933765, 0.000101968}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[explotP, \ JRPlotP, \ \ PlotRange\  -> {0.74, \ 0.75}, \ 
      DisplayFunction\  -> \ $DisplayFunction]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -45.7345 61.8034 [
[.18254 -0.0125 -6 -9 ]
[.18254 -0.0125 6 0 ]
[.34127 -0.0125 -9 -9 ]
[.34127 -0.0125 9 0 ]
[.5 -0.0125 -9 -9 ]
[.5 -0.0125 9 0 ]
[.65873 -0.0125 -9 -9 ]
[.65873 -0.0125 9 0 ]
[.81746 -0.0125 -9 -9 ]
[.81746 -0.0125 9 0 ]
[.97619 -0.0125 -9 -9 ]
[.97619 -0.0125 9 0 ]
[.01131 .12361 -30 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -30 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -30 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -24 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 0 m
.18254 .00625 L
s
[(50)] .18254 -0.0125 0 1 Mshowa
.34127 0 m
.34127 .00625 L
s
[(100)] .34127 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(150)] .5 -0.0125 0 1 Mshowa
.65873 0 m
.65873 .00625 L
s
[(200)] .65873 -0.0125 0 1 Mshowa
.81746 0 m
.81746 .00625 L
s
[(250)] .81746 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(300)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.15079 0 m
.15079 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.27778 0 m
.27778 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.53175 0 m
.53175 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.9127 0 m
.9127 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(0.742)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.744)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.746)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.748)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(0.75)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .20223 m
.0746 .20223 L
.07778 .20223 L
.08095 .20223 L
.08413 .20223 L
.0873 .20223 L
.09048 .20223 L
.09365 .20223 L
.09683 .20223 L
.1 .20223 L
.10317 .20223 L
.10635 .20223 L
.10952 .20223 L
.1127 .20223 L
.11587 .20223 L
.11905 .20223 L
.12222 .20223 L
.1254 .20223 L
.12857 .20223 L
.13175 .20223 L
.13492 .20223 L
.1381 .20223 L
.14127 .20223 L
.14444 .20223 L
.14762 .20223 L
.15079 .20223 L
.15397 .20223 L
.15714 .20223 L
.16032 .20223 L
.16349 .20223 L
.16667 .20223 L
.16984 .20223 L
.17302 .20223 L
.17619 .20223 L
.17937 .20223 L
.18254 .20223 L
.18571 .20223 L
.18889 .20223 L
.19206 .20223 L
.19524 .20223 L
.19841 .20223 L
.20159 .20223 L
.20476 .20223 L
.20794 .20223 L
.21111 .20223 L
.21429 .20223 L
.21746 .20223 L
.22063 .20223 L
.22381 .20223 L
.22698 .20223 L
Mistroke
.23016 .20223 L
.23333 .20223 L
.23651 .20223 L
.23968 .20223 L
.24286 .20223 L
.24603 .20223 L
.24921 .20223 L
.25238 .20223 L
.25556 .20223 L
.25873 .20223 L
.2619 .20223 L
.26508 .20223 L
.26825 .20223 L
.27143 .20223 L
.2746 .20223 L
.27778 .20223 L
.28095 .20223 L
.28413 .20223 L
.2873 .20223 L
.29048 .20223 L
.29365 .20223 L
.29683 .20223 L
.3 .20223 L
.30317 .20223 L
.30635 .20223 L
.30952 .20223 L
.3127 .20223 L
.31587 .20223 L
.31905 .20223 L
.32222 .20223 L
.3254 .20223 L
.32857 .20223 L
.33175 .20223 L
.33492 .20223 L
.3381 .20223 L
.34127 .20223 L
.34444 .20223 L
.34762 .20223 L
.35079 .20223 L
.35397 .20223 L
.35714 .20223 L
.36032 .20223 L
.36349 .20223 L
.36667 .20223 L
.36984 .20223 L
.37302 .20223 L
.37619 .20223 L
.37937 .20223 L
.38254 .20223 L
.38571 .20223 L
Mistroke
.38889 .20223 L
.39206 .20223 L
.39524 .20223 L
.39841 .20223 L
.40159 .20223 L
.40476 .20223 L
.40794 .20223 L
.41111 .20223 L
.41429 .20223 L
.41746 .20223 L
.42063 .20223 L
.42381 .20223 L
.42698 .20223 L
.43016 .20223 L
.43333 .20223 L
.43651 .20223 L
.43968 .20223 L
.44286 .20223 L
.44603 .20223 L
.44921 .20223 L
.45238 .20223 L
.45556 .20223 L
.45873 .20223 L
.4619 .20223 L
.46508 .20223 L
.46825 .20223 L
.47143 .20223 L
.4746 .20223 L
.47778 .20223 L
.48095 .20223 L
.48413 .20223 L
.4873 .20223 L
.49048 .20223 L
.49365 .20223 L
.49683 .20223 L
.5 .20223 L
.50317 .20223 L
.50635 .20223 L
.50952 .20223 L
.5127 .20223 L
.51587 .20223 L
.51905 .20223 L
.52222 .20223 L
.5254 .20223 L
.52857 .20223 L
.53175 .20223 L
.53492 .20223 L
.5381 .20223 L
.54127 .20223 L
.54444 .20223 L
Mistroke
.54762 .20223 L
.55079 .20223 L
.55397 .20223 L
.55714 .20223 L
.56032 .20223 L
.56349 .20223 L
.56667 .20223 L
.56984 .20223 L
.57302 .20223 L
.57619 .20223 L
.57937 .20223 L
.58254 .20223 L
.58571 .20223 L
.58889 .20223 L
.59206 .20223 L
.59524 .20223 L
.59841 .20223 L
.60159 .20223 L
.60476 .20223 L
.60794 .20223 L
.61111 .20223 L
.61429 .20223 L
.61746 .20223 L
.62063 .20223 L
.62381 .20223 L
.62698 .20223 L
.63016 .20223 L
.63333 .20223 L
.63651 .20223 L
.63968 .20223 L
.64286 .20223 L
.64603 .20223 L
.64921 .20223 L
.65238 .20223 L
.65556 .20223 L
.65873 .20223 L
.6619 .20223 L
.66508 .20223 L
.66825 .20223 L
.67143 .20223 L
.6746 .20223 L
.67778 .20223 L
.68095 .20223 L
.68413 .20223 L
.6873 .20223 L
.69048 .20223 L
.69365 .20223 L
.69683 .20223 L
.7 .20223 L
.70317 .20223 L
Mistroke
.70635 .20223 L
.70952 .20223 L
.7127 .20223 L
.71587 .20223 L
.71905 .20223 L
.72222 .20223 L
.7254 .20223 L
.72857 .20223 L
.73175 .20223 L
.73492 .20223 L
.7381 .20223 L
.74127 .20223 L
.74444 .20223 L
.74762 .20223 L
.75079 .20223 L
.75397 .20223 L
.75714 .20223 L
.76032 .20223 L
.76349 .20223 L
.76667 .20223 L
.76984 .20223 L
.77302 .20223 L
.77619 .20223 L
.77937 .20223 L
.78254 .20223 L
.78571 .20223 L
.78889 .20223 L
.79206 .20223 L
.79524 .20223 L
.79841 .20223 L
.80159 .20223 L
.80476 .20223 L
.80794 .20223 L
.81111 .20223 L
.81429 .20223 L
.81746 .20223 L
.82063 .20223 L
.82381 .20223 L
.82698 .20223 L
.83016 .20223 L
.83333 .20223 L
.83651 .20223 L
.83968 .20223 L
.84286 .20223 L
.84603 .20223 L
.84921 .20223 L
.85238 .20223 L
.85556 .20223 L
.85873 .20223 L
.8619 .20223 L
Mistroke
.86508 .20223 L
.86825 .20223 L
.87143 .20223 L
.8746 .20223 L
.87778 .20223 L
.88095 .20223 L
.88413 .20223 L
.8873 .20223 L
.89048 .20223 L
.89365 .20223 L
.89683 .20223 L
.9 .20223 L
.90317 .20223 L
.90635 .20223 L
.90952 .20223 L
.9127 .20223 L
.91587 .20223 L
.91905 .20223 L
.92222 .20223 L
.9254 .20223 L
.92857 .20223 L
.93175 .20223 L
.93492 .20223 L
.9381 .20223 L
.94127 .20223 L
.94444 .20223 L
.94762 .20223 L
.95079 .20223 L
.95397 .20223 L
.95714 .20223 L
.96032 .20223 L
.96349 .20223 L
.96667 .20223 L
.96984 .20223 L
.97302 .20223 L
.97619 .20223 L
Mfstroke
.07325 .61803 m
.0746 .60433 L
.07778 .54302 L
.08095 .59591 L
.08413 .46931 L
.0873 .58548 L
.09048 .4097 L
.09365 .57407 L
.09683 .36052 L
.1 .56228 L
.10317 .31924 L
.10635 .55045 L
.10952 .2841 L
.1127 .53879 L
.11587 .25383 L
.11905 .52742 L
.12222 .22747 L
.1254 .51639 L
.12857 .20433 L
.13175 .50574 L
.13492 .18384 L
.1381 .49548 L
.14127 .16557 L
.14444 .48562 L
.14762 .14918 L
.15079 .47614 L
.15397 .13439 L
.15714 .46703 L
.16032 .12099 L
.16349 .45828 L
.16667 .10878 L
.16984 .44987 L
.17302 .09761 L
.17619 .44178 L
.17937 .08735 L
.18254 .434 L
.18571 .0779 L
.18889 .42652 L
.19206 .06917 L
.19524 .41931 L
.19841 .06107 L
.20159 .41236 L
.20476 .05354 L
.20794 .40566 L
.21111 .04653 L
.21429 .39919 L
.21746 .03997 L
.22063 .39295 L
.22381 .04307 L
.22698 .38691 L
Mistroke
.23016 .05854 L
.23333 .38108 L
.23651 .07277 L
.23968 .37543 L
.24286 .08589 L
.24603 .36997 L
.24921 .098 L
.25238 .36467 L
.25556 .1092 L
.25873 .35953 L
.2619 .11956 L
.26508 .35455 L
.26825 .12916 L
.27143 .34972 L
.2746 .13807 L
.27778 .34503 L
.28095 .14634 L
.28413 .34047 L
.2873 .15403 L
.29048 .33603 L
.29365 .16119 L
.29683 .33172 L
.3 .16786 L
.30317 .32752 L
.30635 .17407 L
.30952 .32344 L
.3127 .17986 L
.31587 .31946 L
.31905 .18527 L
.32222 .31558 L
.3254 .19032 L
.32857 .3118 L
.33175 .19503 L
.33492 .30811 L
.3381 .19944 L
.34127 .30452 L
.34444 .20356 L
.34762 .30101 L
.35079 .20741 L
.35397 .29758 L
.35714 .21102 L
.36032 .29423 L
.36349 .21439 L
.36667 .29095 L
.36984 .21755 L
.37302 .28776 L
.37619 .22051 L
.37937 .28463 L
.38254 .22327 L
.38571 .28157 L
Mistroke
.38889 .22586 L
.39206 .27857 L
.39524 .22828 L
.39841 .27564 L
.40159 .23055 L
.40476 .27277 L
.40794 .23267 L
.41111 .26996 L
.41429 .23465 L
.41746 .26721 L
.42063 .23651 L
.42381 .26451 L
.42698 .23824 L
.43016 .26186 L
.43333 .23986 L
.43651 .25927 L
.43968 .24137 L
.44286 .25673 L
.44603 .24278 L
.44921 .25423 L
.45238 .24409 L
.45556 .25178 L
.45873 .24531 L
.4619 .24938 L
.46508 .24645 L
.46825 .24702 L
.47143 .2475 L
.4746 .24471 L
.47778 .24848 L
.48095 .24243 L
.48413 .24939 L
.4873 .24019 L
.49048 .25023 L
.49365 .238 L
.49683 .251 L
.5 .23584 L
.50317 .25172 L
.50635 .23372 L
.50952 .25237 L
.5127 .23163 L
.51587 .25297 L
.51905 .22958 L
.52222 .25352 L
.5254 .22756 L
.52857 .25402 L
.53175 .22558 L
.53492 .25447 L
.5381 .22363 L
.54127 .25487 L
.54444 .22171 L
Mistroke
.54762 .25523 L
.55079 .21981 L
.55397 .25556 L
.55714 .21795 L
.56032 .25584 L
.56349 .21612 L
.56667 .25609 L
.56984 .21431 L
.57302 .2563 L
.57619 .21253 L
.57937 .25648 L
.58254 .21078 L
.58571 .25663 L
.58889 .20906 L
.59206 .25675 L
.59524 .20736 L
.59841 .25684 L
.60159 .20568 L
.60476 .2569 L
.60794 .20403 L
.61111 .25694 L
.61429 .2024 L
.61746 .25695 L
.62063 .20079 L
.62381 .25694 L
.62698 .19921 L
.63016 .2569 L
.63333 .19765 L
.63651 .25685 L
.63968 .19611 L
.64286 .25677 L
.64603 .19459 L
.64921 .25668 L
.65238 .19309 L
.65556 .25656 L
.65873 .19161 L
.6619 .25643 L
.66508 .19015 L
.66825 .25628 L
.67143 .18871 L
.6746 .25612 L
.67778 .18729 L
.68095 .25594 L
.68413 .18589 L
.6873 .25575 L
.69048 .1845 L
.69365 .25554 L
.69683 .18314 L
.7 .25532 L
.70317 .18178 L
Mistroke
.70635 .25508 L
.70952 .18045 L
.7127 .25484 L
.71587 .17913 L
.71905 .25458 L
.72222 .17783 L
.7254 .25431 L
.72857 .17654 L
.73175 .25403 L
.73492 .17527 L
.7381 .25374 L
.74127 .17402 L
.74444 .25344 L
.74762 .17278 L
.75079 .25313 L
.75397 .17155 L
.75714 .25282 L
.76032 .17034 L
.76349 .25249 L
.76667 .16914 L
.76984 .25216 L
.77302 .16796 L
.77619 .25182 L
.77937 .16678 L
.78254 .25147 L
.78571 .16563 L
.78889 .25112 L
.79206 .16448 L
.79524 .25076 L
.79841 .16335 L
.80159 .25039 L
.80476 .16223 L
.80794 .25002 L
.81111 .16112 L
.81429 .24964 L
.81746 .16179 L
.82063 .24925 L
.82381 .1635 L
.82698 .24886 L
.83016 .16517 L
.83333 .24847 L
.83651 .1668 L
.83968 .24807 L
.84286 .16838 L
.84603 .24767 L
.84921 .16993 L
.85238 .24726 L
.85556 .17143 L
.85873 .24685 L
.8619 .1729 L
Mistroke
.86508 .24643 L
.86825 .17433 L
.87143 .24601 L
.8746 .17573 L
.87778 .24559 L
.88095 .17709 L
.88413 .24517 L
.8873 .17842 L
.89048 .24474 L
.89365 .17971 L
.89683 .24431 L
.9 .18097 L
.90317 .24387 L
.90635 .1822 L
.90952 .24344 L
.9127 .1834 L
.91587 .243 L
.91905 .18457 L
.92222 .24256 L
.9254 .18571 L
.92857 .24212 L
.93175 .18683 L
.93492 .24167 L
.9381 .18791 L
.94127 .24123 L
.94444 .18897 L
.94762 .24078 L
.95079 .19 L
.95397 .24033 L
.95714 .19101 L
.96032 .23988 L
.96349 .19199 L
.96667 .23942 L
.96984 .19295 L
.97302 .23897 L
.97619 .19389 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00OogooKWoo003oOom^
Ool0059oo`<000Aoo`8002Yoo`<000Aoo`8000Aoo`8002Ioo`<000=oo`<000Aoo`8002Moo`@000=o
o`8000Aoo`8002Ioo`@0009oo`<000Aoo`8002Moo`<000Aoo`8000Aoo`8000=oo`00EGoo00@007oo
Ool0009oo`03001oogoo02Qoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo00<0
07ooOol01Goo00@007ooOol0009oo`03001oogoo02Eoo`03001oogoo009oo`04001oogoo0002Ool0
1000Oomoo`009Woo00<007ooOol01Goo00@007ooOol0009oo`03001oogoo02Moo`04001oogoo0002
Ool01000Oomoo`000Woo0@000Goo0Goo001EOol01000Oomoo`000Woo00<007ooOol0:7oo00<007oo
Ool00Woo00@007ooOol0009oo`04001oogoo000VOol00`00Oomoo`05Ool01000Oomoo`000Woo00<0
07ooOol09Woo00D007ooOomoo`00009oo`04001oogoo0002Ool00`00Oomoo`0UOol00`00Oomoo`04
Ool01000Oomoo`000Woo00<007ooOol09goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1
Ool0059oo`<000=oo`04001oogoo000ZOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol0
02Ioo`03001oogoo009oo`<000=oo`04001oogoo000YOol01000Oomoo`000Woo00@007ooOol0009o
o`03001oogoo02Ioo`03001oogoo00<000=oo`04001oogoo000WOol20003Ool01000Oomoo`000Woo
00@007ooOol0009oo`00DWoo00<007ooOol00goo00@007ooOol002Yoo`03001oogoo009oo`04001o
ogoo0002Ool01000Oomoo`009Woo00<007ooOol00Woo00<007ooOol00goo00@007ooOol002Ioo`04
001oogoo0002Ool01000Oomoo`000Woo00@007ooOol002Eoo`04001oogoo0002Ool00`00Oomoo`03
Ool01000Oomoo`00:Goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1Ool0059oo`@000=o
o`8002Yoo`8000Eoo`8000Aoo`8002Ioo`8000Aoo`@000=oo`8002Qoo`8000Aoo`8000Aoo`8002Mo
o`8000=oo`@000=oo`8002Moo`<000Aoo`8000Aoo`8000=oo`00ogooKWoo003oOom^Ool00?moofio
o`006Wooo`00D`000Goo000ROol00`00Oomoo`0bOol00`00Oomoo`0cOol00`00Oomoo`0bOol00`00
Oomoo`0cOol00`00Oomoo`0bOol00`00Oomoo`0cOol00`00Oomoo`07Ool0029oo`03001oogoo0?mo
odUoo`008Woo00<007ooOol0ogooBGoo000ROol2003oOom:Ool0029oo`03001oogoo03ioo`03001o
ogoo0?moo`Qoo`008Woo00<007ooOol0?7oo00D007oo001oo`000?moo`Qoo`008Woo00<007ooOol0
>Woo00L007oo001oo`00Ool0003oOol8Ool0029oo`03001oogoo03Qoo`0;001oo`00Ool007oo001o
o`00Ool0003oOol6Ool0029oo`8003Uoo`0;001oo`00Ool007oo001oo`00Ool0003oOol6Ool0029o
o`03001oogoo03Eoo`0@001oogoo001oo`00Ool007oo001oo`00Ool007oo003oOol4Ool0029oo`03
001oogoo03=oo`03001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000ogoo17oo000R
Ool00`00Oomoo`0aOol01@00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool00`00Oomoo`3nOol0029oo`80039oo`05001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`03001oogoo0?ioo`008Woo00<007ooOol0;goo00L007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oo`000?ioo`000Woo0P0017oo0P00
17oo00<007ooOol00goo0`000Woo10001Goo00<007ooOol0;Goo00T007oo001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000o7oo00001Goo001o
ogoo00000goo0P0017oo00<007ooOol017oo00<007ooOol00Woo00<007ooOol01Goo00<007ooOol0
;Goo00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool0
1@00Ool007oo0000o7oo00001Goo001oogoo00002Woo00@007ooOomoo`@000Aoo`03001oogoo00Ao
o`<002]oo`0;001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`07001oo`00Ool007oo0000nWoo00001Goo001oogoo00002Woo00L007ooOomoo`00Ool0
0006Ool00`00Oomoo`03Ool00`00Oomoo`0[Ool02`00Ool007oo001oo`00Ool007oo00000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0003hOol00005Ool0
07ooOol0000;Ool01P00Oomoo`00Ool000=oo`04001oogoo0005Ool00`00Oomoo`0YOol03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00/0
07oo001oo`00Ool007oo001oo`000?Ioo`000Woo0P002Goo10000goo0P0017oo0P001Woo00<007oo
Ool0:Goo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`0;001oo`00Ool007oo001oo`00Ool0003fOol0029oo`03001oogoo02Ioo`0@001o
ogoo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000m7oo000ROol2000WOol04000Oomoo`00Ool0
07oo001oo`00Ool007oo001oo`000Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Woo001o
o`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`3_Ool0029oo`03001oogoo02Aoo`03001o
o`00009oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00Ool0002UOol02`00Ool0
07oo001oo`00Ool007oo00000Woo00<007oo0000>Woo000ROol00`00Oomoo`0TOol00`00Ool00002
Ool03000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00D007oo001oo`0009Ioo`09001oo`00
Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00
Ool007oo001oo`00Ool0000`Ool0029oo`03001oogoo02Aoo`03001oo`00009oo`0<001oo`00Ool0
07oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00
Ool007oo001oo`00Ool007oo0002Ool02@00Ool007oo001oo`00Ool00025Ool01`00Ool007oo001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`09001oo`00Ool007oo001o
o`0002Eoo`008Woo0P008goo00D007oo001oo`00009oo`0<001oo`00Ool007oo001oo`00Ool007oo
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool0
07oo0002Ool02`00Ool007oo001oo`00Ool007oo0000N7oo00<007oo00000Woo00d007oo001oo`00
Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00
Ool007oo001oo`00Ool007oo001oo`00009oo`07001oo`00Ool007oo000067oo000ROol00`00Oomo
o`0ROol01@00Ool007oo00000Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001o
o`00Ool007oo001oo`00Ool007oo0000KWoo00/007oo001oo`00Ool007oo001oo`00009oo`0=001o
o`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool0
3000Ool007oo001oo`00Ool007oo001oo`80000>Ool007oo001oo`00Ool007oo001oo`00Ool0009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool01`00Ool007oo001oo`0000Uoo`008Woo00<007ooOol08Woo00D007oo001oo`00009oo`0<
001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
3Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`03001oo`0005ioo`07001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0<001oo`00Ool007oo001oo`00
Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00L007oo001oo`00Ool00009Ool0029oo`03001oogoo00eoool003<0
00Uoo`008Woo00<007ooOol087oo00L007oo001oo`00Ool00002Ool03000Ool007oo001oo`00Ool0
07oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00
Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool02`00Ool007oo001o
o`00Ool007oo0000AGoo00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo000007oo001oo`00
Ool007oo001oo`00Ool200006goo001oo`00Ool007oo001oo`00Ool007oo001oo`00Ool007oo001o
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00800008Ool007oo001o
o`00Ool000Uoo`008Woo0P008Goo00L007oo001oo`00Ool00002Ool03000Ool007oo001oo`00Ool0
07oo001oo`800007Ool007oo001oo`00Ool01@00011oo`00001oo`00Ool007oo001oo`00Ool007oo
001oo`80000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00<007ooOol0=Goo00<007oo00000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000Eoo`00Ool007oo00800007Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001o
o`00Ool007oo001oo`00Ool007oo00800006Ool007oo001oo`002Woo000ROol00`00Oomoo`0NOol0
2@00Ool007oo001oo`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`800005Ool007oo
001oo`0200001Goo001oo`00Ool00`0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009o
o`07001oo`00Ool007oo0000:goo00T007oo001oo`00Ool007oo00000Woo00`007oo001oo`00Ool0
07oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool00000Ool007oo
001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool0
07oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool0
07oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
3Goo001oo`00Ool007oo001oo`00Ool007oo00800006Ool007oo001oo`002Woo0002Ool20004Ool2
0004Ool00`00Oomoo`03Ool30003Ool30005Ool00`00Oomoo`0NOol02@00Ool007oo001oo`00Ool0
0002Ool03000Ool007oo001oo`00Ool007oo001oo`800003Ool007oo00800009Ool007oo001oo`00
Ool007oo0080000<Ool00000Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00
Ool007oo001oo`0001aoo`0A001oogoo001oo`00Ool007oo001oo`00Ool007oo001oo`0200002goo
001oo`00Ool007oo001oo`00Ool00P00011oo`00001oo`00Ool007oo001oo`00Ool007oo001oo`80
000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001o
o`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001o
o`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0000:
Ool00005Ool007ooOol00003Ool20004Ool00`00Oomoo`04Ool00`00Oomoo`03Ool00`00Oomoo`04
Ool00`00Oomoo`0NOol02000Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool0
07oo0080000>Ool00000Ool007oo001oo`00Ool007oo001oo`800005Ool007oo001oo`0400000goo
001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00T007oo001oo`00Ool0
07oo00002goo00L007oo001oo`00Ool00002Ool02P00Ool007oo001oo`00Ool007oo0P0000=oo`00
Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00
Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001o
o`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001o
o`00Ool0000:Ool00005Ool007ooOol0000:Ool01000Oomoogoo10000Woo10001Goo0`007Woo00P0
07oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0300003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000;Ool007oo001oo`00Ool007oo001oo`0300003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00P00003
Ool007oo00<0000BOol00000Ool007oo001oo`00Ool007oo001oo`00Ool007oo0P0000aoo`00Ool0
07oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00
Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
029oo`0000Eoo`00Oomoo`0000Yoo`07001oogooOol007oo00000goo00<007oo00001Woo00<007oo
Ool077oo00X007oo001oo`00Ool007oo001oo`80000;Ool007oo001oo`00Ool007oo001oo`020000
47oo000007oo001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200001Goo001oo`00Ool00`0000=oo`00Ool00`0000=oo`00Ool00`0000Eoo`00000007oo
00@00007Ool00000001oo`00Ool00`0000Qoo`00001oo`00Ool007oo0P0000Qoo`00Ool007oo001o
o`003Goo00<007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool00016Ool00005Ool007oo
Ool0000;Ool01P00Oomoo`00Ool000=oo`03001oo`0000Ioo`03001oogoo01aoo`0:001oo`00Ool0
07oo001oo`00Ool200002Goo001oo`00Ool007oo001oo`0200000goo001oo`0200003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Ioo`00Ool007oo002oOol0009oo`8000Uoo`@0
00=oo`8000Aoo`8000Ioo`03001oogoo01aoo`0:001oo`00Ool007oo001oo`00Ool200001goo001o
o`00Ool007oo00800005Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080
000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001o
o`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001o
o`000Woo00<007oo0000`goo000ROol2000MOol02P00Ool007oo001oo`00Ool007oo0P0000Moo`00
Ool007oo001oo`0200001Goo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
00003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool02`00Ool007oo001oo`00Ool007oo0000
bWoo000ROol00`00Oomoo`0JOol03000Ool007oo001oo`00Ool007oo001oo`800005Ool007oo001o
o`0200001goo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eo
o`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080
000<Ool007oo001oo`00Ool007oo001oo`000Woo00L007oo001oo`00Ool0003>Ool0029oo`03001o
ogoo01Yoo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000=oo`00Ool00P0000Uoo`00Ool007oo
001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool00`00Oomoo`3BOol0029oo`03001oogoo01Yoo`0<001oo`00Ool007oo
001oo`00Ool007oo0P0000ioo`00001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo
001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool0
07oo001oo`00Ool007oo00000Woo00/007oo001oo`00Ool007oo001oo`000=Uoo`008Woo00<007oo
Ool06Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Woo000007oo001oo`00Ool007oo001o
o`00Ool2000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`000=eo
o`008Woo0P006goo00`007oo001oo`00Ool007oo001oo`00Ool300003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oo`000>5oo`00
8Woo00<007ooOol05goo00l007ooOol007oo001oo`00Ool007oo001oo`00Ool00`0000eoo`00Ool0
07oo001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0003VOol0029o
o`03001oogoo01Moo`0=001oogoo001oo`00Ool007oo001oo`00Ool00P00011oo`00001oo`00Ool0
07oo001oo`00Ool007oo001oo`80000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0003ZOol0029oo`03001oogoo
01Moo`0;001oogoo001oo`00Ool007oo001oo`0200000goo001oo`0200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool01`00Ool007oo001oo`000>aoo`008Woo0P0067oo00/007ooOol007oo
001oo`00Ool007oo00800003Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009o
o`03001oo`000?1oo`008Woo00<007ooOol05goo00/007ooOol007oo001oo`00Ool007oo00800003
Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?Eoo`008Woo00<007ooOol05goo
00T007ooOol007oo001oo`00Ool00P0000Eoo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo
001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool02`00Ool007oo001oo`00Ool0
07oo0000mgoo0002Ool20004Ool20004Ool00`00Oomoo`03Ool30003Ool20006Ool00`00Oomoo`0E
Ool00`00Ool00002Ool01P00Ool007oo001oo`800005Ool007oo001oo`02000037oo001oo`00Ool0
07oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00
Ool0003kOol00005Ool007ooOol00003Ool20004Ool00`00Oomoo`04Ool01@00Oomoogoo00000Woo
00<007ooOol00goo00<007ooOol05Goo00<007oo00000Woo00@007oo001oo`800007Ool007oo001o
o`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`05001oo`00Ool0003mOol00005Ool007ooOol0000:Ool01000Oomoogoo10000Woo
00@007ooOol000Eoo`<001Eoo`03001oo`00009oo`04001oo`00Ool200001goo001oo`00Ool007oo
0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool00`00Ool0003oOol00005Ool007ooOol0000:Ool01`00Oomoogoo001oo`0000=oo`<000Io
o`03001oogoo01Eoo`07001oo`00001oo`00Ool00P0000Uoo`00Ool007oo001oo`00Ool00P0000ao
o`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?moo`Eo
o`0000Eoo`00Oomoo`0000]oo`06001oogoo001oo`000goo00<007ooOol01Woo00<007ooOol05Goo
00L007oo000007oo001oo`0200002Goo001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo
001oo`00Ool0009oo`0;001oo`00Ool007oo001oo`00Ool0003oOol7Ool0009oo`8000Uoo`@000=o
o`8000Aoo`<000Eoo`03001oogoo01Eoo`07001oo`00001oo`00Ool00P0000Uoo`00Ool007oo001o
o`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool02@00Ool007oo001oo`00Ool0003o
Ool9Ool0029oo`8001Ioo`05001oo`00001oo`0200002Woo001oo`00Ool007oo001oo`000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`000?moo`]oo`008Woo00<0
07ooOol04goo00@007oo001oo`80000=Ool00000Ool007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000ogoo3Goo000ROol00`00Oomoo`0C
Ool01000Ool007oo0P0000eoo`00001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`03001oo`000?moo`moo`008Woo00<007ooOol04goo00@007oo001o
o`<0000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool00`00Oomoo`3oOol?Ool0029oo`03001oogoo01=oo`04001oo`00Ool3000037oo001oo`00
Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000ogoo57oo000ROol2
000DOol01000Ool007oo0`0000aoo`00Ool007oo001oo`00Ool007oo0002Ool02`00Ool007oo001o
o`00Ool007oo0000ogoo5Woo000ROol00`00Oomoo`0COol01@00Ool00000Ool00P0000aoo`00Ool0
07oo001oo`00Ool007oo0002Ool02@00Ool007oo001oo`00Ool0003oOolHOol0029oo`03001oogoo
01=oo`05001oo`00001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`07001oo`00Ool0
07oo0000ogoo6Woo000ROol00`00Oomoo`0COol01@00Ool00000Ool00P0000aoo`00Ool007oo001o
o`00Ool007oo0002Ool01@00Ool007oo0000ogoo77oo000ROol2000DOol01@00Ool00000Ool00P00
00aoo`00Ool007oo001oo`00Ool007oo0002Ool00`00Ool0003oOolNOol0029oo`03001oogoo015o
o`07001oo`00001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`3o
OolNOol0009oo`8000Aoo`8000Aoo`03001oogoo00=oo`<000=oo`8000Ioo`03001oogoo015oo`07
001oo`00001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`3oOolN
Ool00005Ool007ooOol00003Ool20004Ool00`00Oomoo`04Ool01@00Oomoogoo00000Woo00<007oo
Ool00goo00<007ooOol04Goo00L007oo000007oo001oo`02000037oo001oo`00Ool007oo001oo`00
Ool00?moob=oo`0000Eoo`00Oomoo`0000Yoo`04001oogooOol40002Ool01000Oomoo`001Goo0`00
4Goo00L007oo000007oo001oo`0200002Woo001oo`00Ool007oo001oo`00ogoo9Goo00001Goo001o
ogoo00002Woo00L007ooOomoo`00Ool00004Ool20006Ool00`00Oomoo`0AOol2000017oo001oo`00
0Woo00T007oo001oo`00Ool007oo0000ogoo9goo00001Goo001oogoo00002goo00H007ooOol007oo
0003Ool01000Oomoo`001Goo00<007ooOol04Goo0P0000Aoo`00Ool0009oo`09001oo`00Ool007oo
001oo`000?moobMoo`000Woo0P002Goo10000goo0P0017oo0P001Woo00<007ooOol04Goo0P0000Ao
o`00Ool0009oo`07001oo`00Ool007oo0000ogoo:Goo000ROol00`00Oomoo`0AOol2000017oo001o
o`000Woo00D007oo001oo`000?moob]oo`008Woo0P004Woo0P0000Aoo`00Ool0009oo`05001oo`00
Ool0003oOol[Ool0029oo`03001oogoo015oo`800004Ool007oo0002Ool00`00Ool0003oOol]Ool0
029oo`03001oogoo00moo`800006Ool007oo001oo`000Woo00<007oo0000ogoo;Goo000ROol00`00
Oomoo`0?Ool200001Woo001oo`00Ool0009oo`03001oogoo0?moobeoo`008Woo0P0047oo0P0000Io
o`00Ool007oo003oOolbOol0029oo`03001oogoo00moo`800006Ool007oo001oo`00ogoo<Woo000R
Ool00`00Oomoo`0?Ool2000017oo001oo`00ogoo=7oo000ROol00`00Oomoo`0>Ool01`00Ool007oo
001oo`000?moocAoo`008Woo00<007ooOol03Woo00D007oo001oo`000?moocIoo`008Woo0P003goo
00<007oo0000ogoo>7oo000ROol00`00Oomoo`0>Ool00`00Ool0003oOolhOol000Qoo`8000Aoo`80
00Aoo`03001oogoo009oo`<000Ioo`03001oogoo00ioo`03001oogoo0?moocQoo`001goo00@007oo
Ool000=oo`8000Aoo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo00ioo`03001oogoo0?mo
ocQoo`001goo00@007ooOol000Yoo`03001oogoo00Aoo`03001oogoo00=oo`<000eoo`03001oogoo
0?moocUoo`001goo00@007ooOol000Yoo`04001oogooOol3003oOomBOol000Moo`04001oogoo000;
Ool01000Oomoo`00ogooE7oo0008Ool20009Ool40002Ool4003oOomAOol00?moofioo`00\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 0.738778, 
  0.933765, 0.00011328}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[explotP, JRPlotPApp, \ \ PlotRange\  -> {0.74, \ 0.75}, \ 
      DisplayFunction\  -> \ $DisplayFunction]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -45.7345 61.8034 [
[.18254 -0.0125 -6 -9 ]
[.18254 -0.0125 6 0 ]
[.34127 -0.0125 -9 -9 ]
[.34127 -0.0125 9 0 ]
[.5 -0.0125 -9 -9 ]
[.5 -0.0125 9 0 ]
[.65873 -0.0125 -9 -9 ]
[.65873 -0.0125 9 0 ]
[.81746 -0.0125 -9 -9 ]
[.81746 -0.0125 9 0 ]
[.97619 -0.0125 -9 -9 ]
[.97619 -0.0125 9 0 ]
[.01131 .12361 -30 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -30 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -30 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -24 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 0 m
.18254 .00625 L
s
[(50)] .18254 -0.0125 0 1 Mshowa
.34127 0 m
.34127 .00625 L
s
[(100)] .34127 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(150)] .5 -0.0125 0 1 Mshowa
.65873 0 m
.65873 .00625 L
s
[(200)] .65873 -0.0125 0 1 Mshowa
.81746 0 m
.81746 .00625 L
s
[(250)] .81746 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(300)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.15079 0 m
.15079 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.27778 0 m
.27778 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.53175 0 m
.53175 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.9127 0 m
.9127 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(0.742)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.744)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.746)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.748)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(0.75)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .20223 m
.0746 .20223 L
.07778 .20223 L
.08095 .20223 L
.08413 .20223 L
.0873 .20223 L
.09048 .20223 L
.09365 .20223 L
.09683 .20223 L
.1 .20223 L
.10317 .20223 L
.10635 .20223 L
.10952 .20223 L
.1127 .20223 L
.11587 .20223 L
.11905 .20223 L
.12222 .20223 L
.1254 .20223 L
.12857 .20223 L
.13175 .20223 L
.13492 .20223 L
.1381 .20223 L
.14127 .20223 L
.14444 .20223 L
.14762 .20223 L
.15079 .20223 L
.15397 .20223 L
.15714 .20223 L
.16032 .20223 L
.16349 .20223 L
.16667 .20223 L
.16984 .20223 L
.17302 .20223 L
.17619 .20223 L
.17937 .20223 L
.18254 .20223 L
.18571 .20223 L
.18889 .20223 L
.19206 .20223 L
.19524 .20223 L
.19841 .20223 L
.20159 .20223 L
.20476 .20223 L
.20794 .20223 L
.21111 .20223 L
.21429 .20223 L
.21746 .20223 L
.22063 .20223 L
.22381 .20223 L
.22698 .20223 L
Mistroke
.23016 .20223 L
.23333 .20223 L
.23651 .20223 L
.23968 .20223 L
.24286 .20223 L
.24603 .20223 L
.24921 .20223 L
.25238 .20223 L
.25556 .20223 L
.25873 .20223 L
.2619 .20223 L
.26508 .20223 L
.26825 .20223 L
.27143 .20223 L
.2746 .20223 L
.27778 .20223 L
.28095 .20223 L
.28413 .20223 L
.2873 .20223 L
.29048 .20223 L
.29365 .20223 L
.29683 .20223 L
.3 .20223 L
.30317 .20223 L
.30635 .20223 L
.30952 .20223 L
.3127 .20223 L
.31587 .20223 L
.31905 .20223 L
.32222 .20223 L
.3254 .20223 L
.32857 .20223 L
.33175 .20223 L
.33492 .20223 L
.3381 .20223 L
.34127 .20223 L
.34444 .20223 L
.34762 .20223 L
.35079 .20223 L
.35397 .20223 L
.35714 .20223 L
.36032 .20223 L
.36349 .20223 L
.36667 .20223 L
.36984 .20223 L
.37302 .20223 L
.37619 .20223 L
.37937 .20223 L
.38254 .20223 L
.38571 .20223 L
Mistroke
.38889 .20223 L
.39206 .20223 L
.39524 .20223 L
.39841 .20223 L
.40159 .20223 L
.40476 .20223 L
.40794 .20223 L
.41111 .20223 L
.41429 .20223 L
.41746 .20223 L
.42063 .20223 L
.42381 .20223 L
.42698 .20223 L
.43016 .20223 L
.43333 .20223 L
.43651 .20223 L
.43968 .20223 L
.44286 .20223 L
.44603 .20223 L
.44921 .20223 L
.45238 .20223 L
.45556 .20223 L
.45873 .20223 L
.4619 .20223 L
.46508 .20223 L
.46825 .20223 L
.47143 .20223 L
.4746 .20223 L
.47778 .20223 L
.48095 .20223 L
.48413 .20223 L
.4873 .20223 L
.49048 .20223 L
.49365 .20223 L
.49683 .20223 L
.5 .20223 L
.50317 .20223 L
.50635 .20223 L
.50952 .20223 L
.5127 .20223 L
.51587 .20223 L
.51905 .20223 L
.52222 .20223 L
.5254 .20223 L
.52857 .20223 L
.53175 .20223 L
.53492 .20223 L
.5381 .20223 L
.54127 .20223 L
.54444 .20223 L
Mistroke
.54762 .20223 L
.55079 .20223 L
.55397 .20223 L
.55714 .20223 L
.56032 .20223 L
.56349 .20223 L
.56667 .20223 L
.56984 .20223 L
.57302 .20223 L
.57619 .20223 L
.57937 .20223 L
.58254 .20223 L
.58571 .20223 L
.58889 .20223 L
.59206 .20223 L
.59524 .20223 L
.59841 .20223 L
.60159 .20223 L
.60476 .20223 L
.60794 .20223 L
.61111 .20223 L
.61429 .20223 L
.61746 .20223 L
.62063 .20223 L
.62381 .20223 L
.62698 .20223 L
.63016 .20223 L
.63333 .20223 L
.63651 .20223 L
.63968 .20223 L
.64286 .20223 L
.64603 .20223 L
.64921 .20223 L
.65238 .20223 L
.65556 .20223 L
.65873 .20223 L
.6619 .20223 L
.66508 .20223 L
.66825 .20223 L
.67143 .20223 L
.6746 .20223 L
.67778 .20223 L
.68095 .20223 L
.68413 .20223 L
.6873 .20223 L
.69048 .20223 L
.69365 .20223 L
.69683 .20223 L
.7 .20223 L
.70317 .20223 L
Mistroke
.70635 .20223 L
.70952 .20223 L
.7127 .20223 L
.71587 .20223 L
.71905 .20223 L
.72222 .20223 L
.7254 .20223 L
.72857 .20223 L
.73175 .20223 L
.73492 .20223 L
.7381 .20223 L
.74127 .20223 L
.74444 .20223 L
.74762 .20223 L
.75079 .20223 L
.75397 .20223 L
.75714 .20223 L
.76032 .20223 L
.76349 .20223 L
.76667 .20223 L
.76984 .20223 L
.77302 .20223 L
.77619 .20223 L
.77937 .20223 L
.78254 .20223 L
.78571 .20223 L
.78889 .20223 L
.79206 .20223 L
.79524 .20223 L
.79841 .20223 L
.80159 .20223 L
.80476 .20223 L
.80794 .20223 L
.81111 .20223 L
.81429 .20223 L
.81746 .20223 L
.82063 .20223 L
.82381 .20223 L
.82698 .20223 L
.83016 .20223 L
.83333 .20223 L
.83651 .20223 L
.83968 .20223 L
.84286 .20223 L
.84603 .20223 L
.84921 .20223 L
.85238 .20223 L
.85556 .20223 L
.85873 .20223 L
.8619 .20223 L
Mistroke
.86508 .20223 L
.86825 .20223 L
.87143 .20223 L
.8746 .20223 L
.87778 .20223 L
.88095 .20223 L
.88413 .20223 L
.8873 .20223 L
.89048 .20223 L
.89365 .20223 L
.89683 .20223 L
.9 .20223 L
.90317 .20223 L
.90635 .20223 L
.90952 .20223 L
.9127 .20223 L
.91587 .20223 L
.91905 .20223 L
.92222 .20223 L
.9254 .20223 L
.92857 .20223 L
.93175 .20223 L
.93492 .20223 L
.9381 .20223 L
.94127 .20223 L
.94444 .20223 L
.94762 .20223 L
.95079 .20223 L
.95397 .20223 L
.95714 .20223 L
.96032 .20223 L
.96349 .20223 L
.96667 .20223 L
.96984 .20223 L
.97302 .20223 L
.97619 .20223 L
Mfstroke
.07143 .56974 m
.0746 .54101 L
.07778 .48421 L
.08095 .53968 L
.08413 .41677 L
.0873 .53491 L
.09048 .36224 L
.09365 .52813 L
.09683 .31723 L
.1 .5202 L
.10317 .27945 L
.10635 .51164 L
.10952 .24729 L
.1127 .50277 L
.11587 .21958 L
.11905 .49381 L
.12222 .19546 L
.1254 .4849 L
.12857 .17427 L
.13175 .47612 L
.13492 .15551 L
.1381 .46752 L
.14127 .13878 L
.14444 .45914 L
.14762 .12377 L
.15079 .45099 L
.15397 .11024 L
.15714 .44309 L
.16032 .09796 L
.16349 .43544 L
.16667 .08678 L
.16984 .42803 L
.17302 .07655 L
.17619 .42086 L
.17937 .06716 L
.18254 .41392 L
.18571 .05851 L
.18889 .40721 L
.19206 .05051 L
.19524 .40073 L
.19841 .04309 L
.20159 .39445 L
.20476 .0362 L
.20794 .38837 L
.21111 .02977 L
.21429 .38248 L
.21746 .02377 L
.22063 .37678 L
.22381 .0272 L
.22698 .37126 L
Mistroke
.23016 .04315 L
.23333 .3659 L
.23651 .05784 L
.23968 .3607 L
.24286 .07139 L
.24603 .35566 L
.24921 .08391 L
.25238 .35076 L
.25556 .09549 L
.25873 .34601 L
.2619 .10621 L
.26508 .34139 L
.26825 .11616 L
.27143 .33689 L
.2746 .1254 L
.27778 .33252 L
.28095 .13398 L
.28413 .32827 L
.2873 .14197 L
.29048 .32413 L
.29365 .14941 L
.29683 .32009 L
.3 .15635 L
.30317 .31616 L
.30635 .16282 L
.30952 .31233 L
.3127 .16886 L
.31587 .30859 L
.31905 .1745 L
.32222 .30495 L
.3254 .17978 L
.32857 .30139 L
.33175 .18471 L
.33492 .29792 L
.3381 .18933 L
.34127 .29453 L
.34444 .19365 L
.34762 .29121 L
.35079 .19769 L
.35397 .28797 L
.35714 .20148 L
.36032 .28481 L
.36349 .20504 L
.36667 .28171 L
.36984 .20836 L
.37302 .27868 L
.37619 .21149 L
.37937 .27571 L
.38254 .21441 L
.38571 .27281 L
Mistroke
.38889 .21715 L
.39206 .26997 L
.39524 .21973 L
.39841 .26718 L
.40159 .22214 L
.40476 .26446 L
.40794 .2244 L
.41111 .26178 L
.41429 .22651 L
.41746 .25916 L
.42063 .2285 L
.42381 .25659 L
.42698 .23036 L
.43016 .25407 L
.43333 .2321 L
.43651 .2516 L
.43968 .23373 L
.44286 .24917 L
.44603 .23525 L
.44921 .24679 L
.45238 .23667 L
.45556 .24445 L
.45873 .23801 L
.4619 .24216 L
.46508 .23925 L
.46825 .2399 L
.47143 .24041 L
.4746 .23768 L
.47778 .24149 L
.48095 .23551 L
.48413 .24249 L
.4873 .23337 L
.49048 .24342 L
.49365 .23126 L
.49683 .24429 L
.5 .2292 L
.50317 .24509 L
.50635 .22716 L
.50952 .24583 L
.5127 .22516 L
.51587 .24652 L
.51905 .2232 L
.52222 .24715 L
.5254 .22126 L
.52857 .24772 L
.53175 .21935 L
.53492 .24825 L
.5381 .21748 L
.54127 .24873 L
.54444 .21563 L
Mistroke
.54762 .24917 L
.55079 .21381 L
.55397 .24957 L
.55714 .21202 L
.56032 .24992 L
.56349 .21026 L
.56667 .25024 L
.56984 .20852 L
.57302 .25052 L
.57619 .20681 L
.57937 .25077 L
.58254 .20513 L
.58571 .25098 L
.58889 .20346 L
.59206 .25116 L
.59524 .20183 L
.59841 .25131 L
.60159 .20021 L
.60476 .25143 L
.60794 .19862 L
.61111 .25153 L
.61429 .19705 L
.61746 .2516 L
.62063 .1955 L
.62381 .25165 L
.62698 .19397 L
.63016 .25167 L
.63333 .19247 L
.63651 .25167 L
.63968 .19098 L
.64286 .25164 L
.64603 .18951 L
.64921 .2516 L
.65238 .18807 L
.65556 .25154 L
.65873 .18664 L
.6619 .25146 L
.66508 .18523 L
.66825 .25136 L
.67143 .18384 L
.6746 .25124 L
.67778 .18246 L
.68095 .25111 L
.68413 .18111 L
.6873 .25096 L
.69048 .17977 L
.69365 .2508 L
.69683 .17844 L
.7 .25062 L
.70317 .17714 L
Mistroke
.70635 .25043 L
.70952 .17584 L
.7127 .25023 L
.71587 .17457 L
.71905 .25002 L
.72222 .17331 L
.7254 .24979 L
.72857 .17206 L
.73175 .24955 L
.73492 .17083 L
.7381 .2493 L
.74127 .16962 L
.74444 .24904 L
.74762 .16842 L
.75079 .24877 L
.75397 .16723 L
.75714 .24849 L
.76032 .16605 L
.76349 .2482 L
.76667 .16489 L
.76984 .24791 L
.77302 .16374 L
.77619 .2476 L
.77937 .16261 L
.78254 .24729 L
.78571 .16148 L
.78889 .24697 L
.79206 .16037 L
.79524 .24664 L
.79841 .15928 L
.80159 .24631 L
.80476 .15819 L
.80794 .24597 L
.81111 .15711 L
.81429 .24562 L
.81746 .15779 L
.82063 .24527 L
.82381 .15954 L
.82698 .24491 L
.83016 .16123 L
.83333 .24455 L
.83651 .16289 L
.83968 .24418 L
.84286 .16451 L
.84603 .24381 L
.84921 .16608 L
.85238 .24343 L
.85556 .16762 L
.85873 .24305 L
.8619 .16911 L
Mistroke
.86508 .24266 L
.86825 .17057 L
.87143 .24227 L
.8746 .172 L
.87778 .24188 L
.88095 .17338 L
.88413 .24148 L
.8873 .17474 L
.89048 .24108 L
.89365 .17606 L
.89683 .24068 L
.9 .17735 L
.90317 .24027 L
.90635 .17861 L
.90952 .23986 L
.9127 .17983 L
.91587 .23945 L
.91905 .18103 L
.92222 .23903 L
.9254 .18219 L
.92857 .23861 L
.93175 .18333 L
.93492 .23819 L
.9381 .18444 L
.94127 .23777 L
.94444 .18552 L
.94762 .23735 L
.95079 .18658 L
.95397 .23692 L
.95714 .18761 L
.96032 .23649 L
.96349 .18862 L
.96667 .23606 L
.96984 .1896 L
.97302 .23563 L
.97619 .19056 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00OogooKWoo003oOom^
Ool0059oo`<000Aoo`8002Yoo`<000Aoo`8000Aoo`8002Ioo`<000=oo`<000Aoo`8002Moo`@000=o
o`8000Aoo`8002Ioo`@0009oo`<000Aoo`8002Moo`<000Aoo`8000Aoo`8000=oo`00EGoo00@007oo
Ool0009oo`03001oogoo02Qoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo00<0
07ooOol01Goo00@007ooOol0009oo`03001oogoo02Eoo`03001oogoo009oo`04001oogoo0002Ool0
1000Oomoo`009Woo00<007ooOol01Goo00@007ooOol0009oo`03001oogoo02Moo`04001oogoo0002
Ool01000Oomoo`000Woo0@000Goo0Goo001EOol01000Oomoo`000Woo00<007ooOol0:7oo00<007oo
Ool00Woo00@007ooOol0009oo`04001oogoo000VOol00`00Oomoo`05Ool01000Oomoo`000Woo00<0
07ooOol09Woo00D007ooOomoo`00009oo`04001oogoo0002Ool00`00Oomoo`0UOol00`00Oomoo`04
Ool01000Oomoo`000Woo00<007ooOol09goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1
Ool0059oo`<000=oo`04001oogoo000ZOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol0
02Ioo`03001oogoo009oo`<000=oo`04001oogoo000YOol01000Oomoo`000Woo00@007ooOol0009o
o`03001oogoo02Ioo`03001oogoo00<000=oo`04001oogoo000WOol20003Ool01000Oomoo`000Woo
00@007ooOol0009oo`00DWoo00<007ooOol00goo00@007ooOol002Yoo`03001oogoo009oo`04001o
ogoo0002Ool01000Oomoo`009Woo00<007ooOol00Woo00<007ooOol00goo00@007ooOol002Ioo`04
001oogoo0002Ool01000Oomoo`000Woo00@007ooOol002Eoo`04001oogoo0002Ool00`00Oomoo`03
Ool01000Oomoo`00:Goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1Ool0059oo`@000=o
o`8002Yoo`8000Eoo`8000Aoo`8002Ioo`8000Aoo`@000=oo`8002Qoo`8000Aoo`8000Aoo`8002Mo
o`8000=oo`@000=oo`8002Moo`<000Aoo`8000Aoo`8000=oo`00ogooKWoo003oOom^Ool00?moofio
o`006Wooo`00D`000Goo000ROol00`00Oomoo`0bOol00`00Oomoo`0cOol00`00Oomoo`0bOol00`00
Oomoo`0cOol00`00Oomoo`0bOol00`00Oomoo`0cOol00`00Oomoo`07Ool0029oo`03001oogoo0?mo
odUoo`008Woo00<007ooOol0?Woo00<007ooOol0ogoo27oo000ROol2000mOol01@00Ool007oo0000
ogoo27oo000ROol00`00Oomoo`0jOol01`00Ool007oo001oo`000?moo`Qoo`008Woo00<007ooOol0
>7oo00/007oo001oo`00Ool007oo001oo`000?moo`Ioo`008Woo00<007ooOol0=Goo00h007ooOol0
07oo001oo`00Ool007oo001oo`00ogoo1Woo000ROol00`00Oomoo`0cOol00`00Ool00002Ool03@00
Ool007oo001oo`00Ool007oo001oo`000?moo`Aoo`008Woo0P00<Woo00D007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo0000ogoo17oo000ROol00`00Oomoo`0aOol01@00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`3nOol0029oo`03001o
ogoo02moo`07001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool0
0`00Oomoo`3nOol0029oo`03001oogoo02eoo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00<007oo0000oWoo000ROol2000^Ool02@00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0003lOol0
029oo`03001oogoo02]oo`0;001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`05001oo`00Ool0003lOol0009oo`8000Aoo`8000Aoo`03001oogoo00=o
o`<0009oo`@000Eoo`03001oogoo02Uoo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`000?Yoo`0000Eoo`00Oomo
o`0000=oo`8000Aoo`03001oogoo00Aoo`03001oogoo009oo`03001oogoo00Eoo`03001oogoo02Uo
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool02@00Ool007oo001oo`00Ool0003hOol00005Ool007ooOol0000:Ool01000Oomoogoo1000
17oo00<007ooOol017oo0`009Woo010007ooOol007oo001oo`00Ool007oo001oo`00Ool0009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00T007oo001oo`00Ool007oo0000n7oo00001Goo
001oogoo00002Woo00L007ooOomoo`00Ool00006Ool00`00Oomoo`03Ool00`00Oomoo`0VOol04000
Oomoo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool02`00Ool007oo001oo`00Ool007oo0000mWoo00001Goo001oogoo00002goo00H007ooOol0
07oo0003Ool01000Oomoo`001Goo00<007ooOol09Woo010007ooOol007oo001oo`00Ool007oo001o
o`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo
001oo`00Ool0003dOol0009oo`8000Uoo`@000=oo`8000Aoo`8000Ioo`03001oogoo02Aoo`03001o
o`00009oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`3_Ool0029oo`03
001oogoo02Aoo`03001oo`00009oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool0
07oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00
Ool0003_Ool0029oo`8002=oo`05001oo`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001o
o`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo
001oo`000Woo00D007oo001oo`000:Moo`07001oo`00Ool007oo0000?goo000ROol00`00Oomoo`0R
Ool01@00Ool007oo00000Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`07001oo`00
Ool007oo0000V7oo00D007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00T007oo001oo`00Ool007oo0000=7oo000ROol00`00Oomoo`0ROol01@00Ool007oo00000Woo00`0
07oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>
Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`09001oo`00Ool007oo001oo`0008Uoo`03001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0000YOol0
029oo`03001oogoo021oo`07001oo`00Ool007oo00000Woo00`007oo001oo`00Ool007oo001oo`00
Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001o
o`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000Ngoo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`03001oogoo01aoo`008Woo0P008Goo00L007oo001oo`00Ool0
0002Ool03000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool00`00Oomoo`1[Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00`007oo001o
o`00Ool007oo001oo`00Ool200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00004Woo000R
Ool00`00Oomoo`0POol01`00Ool007oo001oo`00009oo`0<001oo`00Ool007oo001oo`00Ool007oo
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool0
07oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`05001oo`00Ool0001NOol01@00
Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001o
o`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo
001oo`00Ool00009Ool0029oo`03001oogoo01ioo`09001oo`00Ool007oo001oo`00009oo`0<001o
o`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009o
o`09001oo`00Ool007oo001oo`00059oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`80000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
00003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00L007oo001oo`00Ool00009Ool0
029oo`03001oogoo00eoool003<000Uoo`008Woo00<007ooOol07Woo00P007oo001oo`00Ool007oo
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200000goo001oo`0200001goo001oo`00Ool0
07oo00<0000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00Ool0000gOol04000
Oomoo`00Ool007oo001oo`00Ool007oo001oo`000Woo00`007oo001oo`00Ool007oo001oo`00Ool2
00003Goo001oo`00Ool007oo001oo`00Ool007oo00800009Ool007oo001oo`00Ool007oo00800003
Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00
Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00
Ool007oo001oo`00Ool00P0000aoo`00001oo`00Ool007oo001oo`00Ool2000027oo001oo`00Ool0
07oo0009Ool0029oo`8001eoo`0:001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000>Ool00000Ool007oo001oo`00Ool007oo001oo`80000<Ool00000Ool0
07oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00
Ool007oo001oo`00Ool007oo0002Ool02@00Ool007oo001oo`00Ool0000YOol02@00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0<001oo`00Ool007oo001o
o`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0300003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo
001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200001Woo
001oo`00Ool000Yoo`008Woo00<007ooOol077oo00X007oo001oo`00Ool007oo001oo`80000=Ool0
07oo001oo`00Ool007oo001oo`00Ool00`0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
1Goo001oo`00Ool0100000=oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
3Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`03001oogoo01Moo`0A001oogoo001oo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo00800007Ool007oo001oo`00Ool00P0000Eoo`00Ool007oo
0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03P00Ool007oo001oo`00Ool0
07oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo00800006Ool007oo001oo`00
2Woo0002Ool20004Ool20004Ool00`00Oomoo`03Ool30003Ool30005Ool00`00Oomoo`0LOol02P00
Ool007oo001oo`00Ool007oo0P0000]oo`00Ool007oo001oo`00Ool007oo0080000@Ool00000Ool0
07oo001oo`00Ool007oo001oo`00Ool200002Goo001oo`00Ool007oo001oo`0500003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00
Ool007oo001oo`00Ool007oo001oo`000Woo00T007oo001oo`00Ool007oo00002goo00L007oo001o
o`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`800006Ool00000Ool007oo0`0000Mo
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0000:Ool00005Ool007oo
Ool00003Ool20004Ool00`00Oomoo`04Ool00`00Oomoo`03Ool00`00Oomoo`04Ool00`00Oomoo`0L
Ool02P00Ool007oo001oo`00Ool007oo0P0000Uoo`00Ool007oo001oo`00Ool00P0000=oo`00Ool0
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool0
07oo00800005Ool007oo001oo`0400000goo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool0
07oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0800000goo001oo`0300004Woo000007oo001oo`00Ool007oo001oo`00Ool007oo001o
o`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oo`0000io
o`0000Eoo`00Oomoo`0000Yoo`04001oogooOol40002Ool40005Ool3000JOol03000Ool007oo001o
o`00Ool007oo001oo`800007Ool007oo001oo`00Ool00P0000Eoo`00Ool007oo0080000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000?Ool007oo001oo`00Ool007oo001oo`00Ool007oo
0080000;Ool00000001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool000eo
o`03001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool0
07oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00T0
07oo001oo`00Ool007oo0000=Goo00001Goo001oogoo00002Woo00L007ooOomoo`00Ool00003Ool0
0`00Ool00006Ool00`00Oomoo`0JOol03000Ool007oo001oo`00Ool007oo001oo`800007Ool007oo
001oo`00Ool00P0000Eoo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200001Woo001oo`00Ool002eoo`07001oo`00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0
001VOol00005Ool007ooOol0000;Ool01P00Oomoo`00Ool000=oo`03001oo`0000Ioo`03001oogoo
01Yoo`0<001oo`00Ool007oo001oo`00Ool007oo0P0000Eoo`00Ool007oo00800007Ool007oo001o
o`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`03001oogoo0<=oo`00
0Woo0P002Goo10000goo0P0017oo0P001Woo00<007ooOol06Woo00`007oo001oo`00Ool007oo001o
o`00Ool200000goo001oo`0200002Goo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo
001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`09001oo`00Ool0
07oo001oo`000<aoo`008Woo0P006goo00`007oo001oo`00Ool007oo001oo`00Ool200003Woo0000
07oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool0
3@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0003@Ool0029oo`03001oogoo
01Moo`0?001oogoo001oo`00Ool007oo001oo`00Ool007oo00<0000=Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool00`00Oomoo`3BOol0029oo`03001oogoo01Moo`0?001oogoo001oo`00
Ool007oo001oo`00Ool007oo00<0000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000aoo`00
Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001o
o`00Ool007oo001oo`00Ool007oo00000Woo00T007oo001oo`00Ool007oo0000fgoo000ROol00`00
Oomoo`0GOol03@00Oomoo`00Ool007oo001oo`00Ool007oo0080000@Ool00000Ool007oo001oo`00
Ool007oo001oo`00Ool2000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo
0000ggoo000ROol00`00Oomoo`0GOol03@00Oomoo`00Ool007oo001oo`00Ool007oo0080000@Ool0
0000Ool007oo001oo`00Ool007oo001oo`00Ool2000037oo001oo`00Ool007oo001oo`00Ool0009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool00`00Oomoo`3QOol0029oo`8001Qoo`0;001oogoo001oo`00Ool007oo001oo`0200000goo
001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo
001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool02`00Ool007oo001oo`00Ool0
07oo0000j7oo000ROol00`00Oomoo`0EOol00`00Ool00002Ool02000Ool007oo001oo`00Ool20000
0goo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`00
0>aoo`008Woo00<007ooOol05Goo00<007oo00000Woo00H007oo001oo`00Ool200001Goo001oo`00
Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007oo0000l7oo000ROol00`00
Oomoo`0EOol00`00Ool00002Ool01P00Ool007oo001oo`800005Ool007oo001oo`02000037oo001o
o`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`3`Ool0029oo`8001Ioo`03001oo`00009o
o`04001oo`00Ool200001goo001oo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`00
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool02`00Ool007oo001oo`00Ool007oo0000
mgoo000ROol00`00Oomoo`0EOol02@00Ool00000Ool007oo001oo`0200001goo001oo`00Ool007oo
0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool01`00Ool007oo001oo`000?]oo`008Woo00<007ooOol05Goo00L007oo000007oo001oo`02
00002Goo001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001o
o`00Ool007oo001oo`00Ool007oo00000Woo00D007oo001oo`000?eoo`000Woo0P0017oo0P0017oo
00<007ooOol00goo0`000goo0P001Woo00<007ooOol04goo00@007oo001oo`800003Ool007oo0080
0009Ool007oo001oo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool00`00Ool0003oOol00005Ool007ooOol00003Ool20004
Ool00`00Oomoo`04Ool01@00Oomoogoo00000Woo00<007ooOol00goo00<007ooOol04goo00@007oo
001oo`80000>Ool00000Ool007oo001oo`00Ool007oo001oo`80000<Ool007oo001oo`00Ool007oo
001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool0003oOol5Ool00005Ool007ooOol0000:
Ool01000Oomoogoo10000Woo00@007ooOol000Eoo`<001=oo`04001oo`00Ool200003Goo000007oo
001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00/007oo
001oo`00Ool007oo001oo`000?moo`Moo`0000Eoo`00Oomoo`0000Yoo`07001oogooOol007oo0000
0goo0`001Woo00<007ooOol04goo00@007oo001oo`80000=Ool00000Ool007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0003o
Ool9Ool00005Ool007ooOol0000;Ool01P00Oomoo`00Ool000=oo`03001oogoo00Ioo`03001oogoo
01=oo`04001oo`00Ool3000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00D007oo001oo`000?moo`eoo`000Woo0P002Goo10000goo0P0017oo
0`001Goo00<007ooOol04goo00@007oo001oo`<0000<Ool007oo001oo`00Ool007oo001oo`000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Ool0003oOol?Ool0029oo`8001Aoo`04
001oo`00Ool3000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00<007ooOol0ogoo3goo000ROol00`00Oomoo`0COol01@00Ool00000Ool00P00
00aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?mo
oaAoo`008Woo00<007ooOol04Goo00@007oo001oo`80000?Ool00000Ool007oo001oo`00Ool007oo
001oo`00009oo`0;001oo`00Ool007oo001oo`00Ool0003oOolFOol0029oo`03001oogoo015oo`04
001oo`00Ool200003goo000007oo001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo001o
o`00Ool0003oOolHOol0029oo`03001oogoo015oo`07001oo`00001oo`00Ool00P0000aoo`00Ool0
07oo001oo`00Ool007oo0002Ool01`00Ool007oo001oo`000?mooaYoo`008Woo0P004Woo00L007oo
000007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`05001oo`00Ool0003oOolL
Ool0029oo`03001oogoo015oo`07001oo`00001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool0
07oo0002Ool00`00Ool0003oOolNOol0029oo`03001oogoo015oo`800005Ool007oo001oo`020000
37oo001oo`00Ool007oo001oo`00Ool0009oo`03001oogoo0?mooaioo`008Woo00<007ooOol04Goo
0P0000Eoo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`00ogoo8goo000ROol2000B
Ool2000017oo001oo`000Woo00/007oo001oo`00Ool007oo001oo`000?moobEoo`008Woo00<007oo
Ool04Goo0P0000Aoo`00Ool0009oo`09001oo`00Ool007oo001oo`000?moobMoo`000Woo0P0017oo
0P0017oo00<007ooOol00goo0`000goo0P001Woo00<007ooOol04Goo0P0000Aoo`00Ool0009oo`09
001oo`00Ool007oo001oo`000?moobMoo`0000Eoo`00Oomoo`0000=oo`8000Aoo`03001oogoo00Ao
o`05001oogooOol00002Ool00`00Oomoo`03Ool00`00Oomoo`0?Ool200001Woo001oo`00Ool0009o
o`07001oo`00Ool007oo0000ogoo:Goo00001Goo001oogoo00002Woo00@007ooOomoo`@0009oo`04
001oogoo0005Ool3000?Ool200001Woo001oo`00Ool0009oo`05001oo`00Ool0003oOol[Ool00005
Ool007ooOol0000:Ool01`00Oomoogoo001oo`0000Aoo`8000Ioo`03001oogoo00moo`800006Ool0
07oo001oo`000Woo00<007oo0000ogoo;Goo00001Goo001oogoo00002goo00H007ooOol007oo0003
Ool01000Oomoo`001Goo00<007ooOol03goo0P0000Ioo`00Ool007oo0002Ool00`00Ool0003oOol]
Ool0009oo`8000Uoo`@000=oo`8000Aoo`8000Ioo`03001oogoo00moo`800006Ool007oo001oo`00
0Woo00<007ooOol0ogoo;Goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool0003oOolb
Ool0029oo`8000moo`07001oo`00Ool007oo0000ogoo=7oo000ROol00`00Oomoo`0>Ool01@00Ool0
07oo0000ogoo=Woo000ROol00`00Oomoo`0>Ool00`00Ool0003oOolhOol0029oo`03001oogoo00io
o`03001oogoo0?moocQoo`008Woo0P003goo00<007ooOol0ogoo>7oo000ROol00`00Oomoo`0=Ool0
0`00Oomoo`3oOoliOol0029oo`03001oogoo00eoo`03001oogoo0?moocUoo`008Woo00<007ooOol0
ogooBGoo000ROol00`00Oomoo`3oOom9Ool0029oo`800?moodYoo`008Woo00<007ooOol0ogooBGoo
0008Ool20004Ool20004Ool00`00Oomoo`02Ool30006Ool00`00Oomoo`3oOom9Ool000Moo`04001o
ogoo0003Ool20004Ool00`00Oomoo`05Ool00`00Oomoo`03Ool00`00Oomoo`3oOom9Ool000Moo`04
001oogoo000:Ool00`00Oomoo`04Ool00`00Oomoo`03Ool3003oOom9Ool000Moo`04001oogoo000:
Ool01000Oomoogoo0`00ogooDWoo0007Ool01000Oomoo`002goo00@007ooOol00?mooeAoo`0027oo
0P002Goo10000Woo1000ogooDGoo003oOom^Ool00001\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 0.738778, 
  0.933765, 0.00011328}}]
}, Open  ]],

Cell["\<\


However, the convergence of both the exact and the approximate scheme remains \
problematic, being highly oscillatory, as before.\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Magic Tree Sizes 2 - Approximate JR Models", "Subsection",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
A simple formula for the strike-straddling can be given in the \
approximate form of this model. This is\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`n = \(\((m + 1\/2)\)\^2\ \[Sigma]\^2\ \
T\)\/\((log(K\/S) - \((r - q - \[Sigma]\^2\/2)\))\)\^2\)], "NumberedEquation",\

  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "A ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " function for this case is"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(straddleJR[m_, \ \[Sigma]_, \ T_, \ S_, \ K_, \ r_, \ 
        q_]\  := \ \((\((m\  + \ 1/2)\)^2*\[Sigma]^2*
            T)\)/\((Log[K/S]\  - \ \((r - q - \[Sigma]^2/2)\))\)^2\)], "Input",\

  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Let's evaluate these values (in real form - you would take the \
nearest integer in practice) for the parameters used above:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[straddleJR[m, 0.2, 1, 9, 10, \ 0.1, \ 0], \ {m, \ 1, \ 10}]\)], 
  "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({139.934999228364631`, 388.70833118990177`, 761.868329132207389`, 
      1259.41499305528137`, 1881.34832295912417`, 2627.66831884373575`, 
      3498.3749807091149`, 4493.46830855526313`, 5612.94830238218089`, 
      6856.8149621898673`}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "This formula explains the locations of the points in the JR plots compared \
to those in the CRR forms - only the first is visible in this particular \
example. The denominator, for this particular case, is smaller, resulting in \
the oscillation envelope having a longer frequency as a function of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ". Note that the magic tree formula can go wrong when the expected value of \
the stock price at expiry coincides with the strike. The Jarrow-Rudd tree \
then always has a node right on the strike. "
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "19.4 ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Tree Models 3 - Supersymmetric Forms"
}], "Section",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
We proceed to define the various functions to explore the \
supersymmetric form of a tree. We need the following functions also used in \
the CRR forms:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(afunc[n_, \ r_, \ q_, \ T_]\  := \ N[Exp[\((r - q)\)\ T/n]]; \n
    Pfunc[up_, down_, \ a_, \ disc_]\  := \ 
      N[disc*\((a - down)\)/\((up - down)\)]; \n
    Qfunc[up_, \ down_, \ a_, \ disc_]\  := \ 
      N[disc - Pfunc[up, down, a, disc]]; \)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The up and down multipliers, in the exact and approximate forms of \
the model, are\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
SSupBin[n_, r_, q_, \[Sigma]_, T_] := Module[{f = (1 + \
Exp[\[Sigma]^2*T/n])},
Exp[(r-q)*T/n]*(f + Sqrt[f^2-4])/2];
SSdownBin[n_, r_, q_, \[Sigma]_, T_] := Module[{f = (1 + \
Exp[\[Sigma]^2*T/n])},
Exp[(r-q)*T/n]*2/(f + Sqrt[f^2-4])];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
SSupBinApp[n_, r_, q_, \[Sigma]_, T_] :=Exp[(r-q)*T/n+\[Sigma] \
Sqrt[T/n]];
SSdownBinApp[n_, r_, q_, \[Sigma]_, T_] := Exp[(r-q)*T/n-\[Sigma] Sqrt[T/n]];\
\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["European options are dealt with by", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
FastSSEuropeanOption[S_, n_, \[Sigma]_, T_, r_, q_, \
exercise_Function] :=
Module[{u = SSupBin[n,r,q,\[Sigma],T], d = SSdownBin[n,r,q,\[Sigma],T],
a = afunc[n,r,q,T],disc = Discount[n,r,T],P,Q},
P = Pfunc[u,d,a,disc];
Q = Qfunc[u,d,a,disc];
Sum[exercise[S u^j d^(n-j)] Binomial[n,j] P^j Q^(n-j), {j,0,n}]];\
\>", 
  "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
FastSSEuropeanOptionApp[S_, n_, \[Sigma]_, T_, r_, q_, \
exercise_Function] :=
Module[{u = SSupBinApp[n,r,q,\[Sigma],T], d = SSdownBinApp[n,r,q,\[Sigma],T],
a = afunc[n,r,q,T],disc = Discount[n,r,T],P,Q},
P = Pfunc[u,d,a,disc];
Q = Qfunc[u,d,a,disc];
Sum[exercise[S u^j d^(n-j)] Binomial[n,j] P^j Q^(n-j), {j,0,n}]];\
\>", 
  "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
FastSSEuropeanPut[S_, K_, n_, r_, q_, \[Sigma]_, T_] :=
FastSSEuropeanOption[S,n,\[Sigma],T,r,q,Max[K-#,0]&];
FastSSEuropeanCall[S_, K_, n_, r_, q_, \[Sigma]_, T_] := 
FastSSEuropeanOption[S, n, \[Sigma], T, r, q, Max[# - K, 0]&];
FastSSEuropeanPutApp[S_, K_, n_, r_, q_, \[Sigma]_, T_] :=
FastSSEuropeanOptionApp[S,n,\[Sigma],T,r,q,Max[K-#,0]&];
FastSSEuropeanCallApp[S_, K_, n_, r_, q_, \[Sigma]_, T_] := 
FastSSEuropeanOptionApp[S, n, \[Sigma], T, r, q, Max[# - K, 0]&];\
\>", 
  "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
American options, as before, are given by the following. The same \
estimates of the Greeks are used as for the JR case.\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
SSAmericanOption[S_, n_, \[Sigma]_, T_, r_, q_, exercise_Function] \
:=
Module[{u = SSupBin[n,r,q,\[Sigma],T], d = SSdownBin[n,r,q,\[Sigma],T],
a = afunc[n,r,q,T],
disc = Discount[n,r,T],
P,Q,TreeOp,value,delta,gamma,theta},
P = Pfunc[u,d,a,disc];
Q = Qfunc[u,d,a,disc];
TreeOp[node_,level_] := TreeOp[node,level] = 
If[level==n,exercise[S d^node u^(level-node)],
Max[{P,Q}.{TreeOp[node,level+1],TreeOp[node+1,level+1]},
exercise[S d^node u^(level-node)]]];
value=TreeOp[0,0];
delta = (TreeOp[0,1] - TreeOp[1,1])/((u-d)*S);
gamma = ((TreeOp[0,2] - TreeOp[1,2])/(u^2-u*d) - 
(TreeOp[1,2]-TreeOp[2,2])/(u*d-d^2))*2/(S^2 (u^2-d^2));
theta = (TreeOp[1,2]-(u*d-1)*S*delta - TreeOp[0,0])/(2*T/n);
Clear[TreeOp];
{value, delta, gamma, theta}];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
SSAmericanOptionApp[S_, n_, \[Sigma]_, T_, r_, q_, \
exercise_Function] :=
Module[{u = SSupBinApp[n,r,q,\[Sigma],T], d = SSdownBinApp[n,r,q,\[Sigma],T],
a = afunc[n,r,q,T],
disc = Discount[n,r,T],
P,Q,TreeOp,value,delta,gamma,theta},
P = Pfunc[u,d,a,disc];
Q = Qfunc[u,d,a,disc];
TreeOp[node_,level_] := TreeOp[node,level] = 
If[level==n,exercise[S d^node u^(level-node)],
Max[{P,Q}.{TreeOp[node,level+1],TreeOp[node+1,level+1]},
exercise[S d^node u^(level-node)]]];
value=TreeOp[0,0];
delta = (TreeOp[0,1] - TreeOp[1,1])/((u-d)*S);
gamma = ((TreeOp[0,2] - TreeOp[1,2])/(u^2-u*d) - 
(TreeOp[1,2]-TreeOp[2,2])/(u*d-d^2))*2/(S^2 (u^2-d^2));
theta = (TreeOp[1,2]-(u*d-1)*S*delta - TreeOp[0,0])/(2*T/n);
Clear[TreeOp];
{value, delta, gamma, theta}];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell["\<\
SSAmericanPut[S_, K_, n_, r_, q_, \[Sigma]_, T_] :=
SSAmericanOption[S,n,\[Sigma],T,r,q,Max[K-#,0]&];
SSAmericanCall[S_, K_, n_, r_, q_, \[Sigma]_, T_] :=
SSAmericanOption[S,n,\[Sigma],T,r,q,Max[#-K,0]&];
SSAmericanPutApp[S_, K_, n_, r_, q_, \[Sigma]_, T_] :=
SSAmericanOptionApp[S,n,\[Sigma],T,r,q,Max[K-#,0]&];
SSAmericanCallApp[S_, K_, n_, r_, q_, \[Sigma]_, T_] :=
SSAmericanOptionApp[S,n,\[Sigma],T,r,q,Max[#-K,0]&];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Evaluation and Verification of Supersymmetric Models", "Subsection",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
What we shall do is to compute sample values for a Call and a Put \
in the supersymmetric model:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(SSdataC\  = \ 
      Table[{k, 
          FastSSEuropeanCall[9, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 1]}, \ {k, \ 
          15, \ 300}]; \n
    SSdataP\  = \ 
      Table[{k, 
          FastSSEuropeanPut[9, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 1]}, \ {k, \ 
          15, \ 300}]; \)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(SSdataCApp\  = \ 
      Table[{k, 
          FastSSSEuropeanCallApp[9, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 
            1]}, \ {k, \ 15, \ 300}]; \n
    SSdataPApp\  = \ 
      Table[{k, 
          FastSSEuropeanPutApp[9, \ 10, \ k, \ 0.1, \ 0, \ 0.2, \ 
            1]}, \ {k, \ 15, \ 300}]; \)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(SSPlotC\  = \ 
      ListPlot[SSdataC, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \n
    SSPlotP = \ 
      ListPlot[SSdataP, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(SSPlotCApp\  = \ 
      ListPlot[SSdataCApp, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \n
    SSPlotPApp = \ 
      ListPlot[SSdataPApp, \ PlotJoined\  -> \ True, \ 
        DisplayFunction\  -> \ Identity]; \)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[explot, OTSPlotC, \ \ PlotRange\  -> \ {0.69, \ 0.70}, \ 
      DisplayFunction\  -> \ $DisplayFunction]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -42.6443 61.8034 [
[.18254 .60553 -6 -9 ]
[.18254 .60553 6 0 ]
[.34127 .60553 -9 -9 ]
[.34127 .60553 9 0 ]
[.5 .60553 -9 -9 ]
[.5 .60553 9 0 ]
[.65873 .60553 -9 -9 ]
[.65873 .60553 9 0 ]
[.81746 .60553 -9 -9 ]
[.81746 .60553 9 0 ]
[.97619 .60553 -9 -9 ]
[.97619 .60553 9 0 ]
[.01131 0 -24 -4.5 ]
[.01131 0 0 4.5 ]
[.01131 .12361 -30 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -30 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -30 -4.5 ]
[.01131 .49443 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .62428 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .61803 m
.18254 .62428 L
s
[(50)] .18254 .60553 0 1 Mshowa
.34127 .61803 m
.34127 .62428 L
s
[(100)] .34127 .60553 0 1 Mshowa
.5 .61803 m
.5 .62428 L
s
[(150)] .5 .60553 0 1 Mshowa
.65873 .61803 m
.65873 .62428 L
s
[(200)] .65873 .60553 0 1 Mshowa
.81746 .61803 m
.81746 .62428 L
s
[(250)] .81746 .60553 0 1 Mshowa
.97619 .61803 m
.97619 .62428 L
s
[(300)] .97619 .60553 0 1 Mshowa
.125 Mabswid
.05556 .61803 m
.05556 .62178 L
s
.0873 .61803 m
.0873 .62178 L
s
.11905 .61803 m
.11905 .62178 L
s
.15079 .61803 m
.15079 .62178 L
s
.21429 .61803 m
.21429 .62178 L
s
.24603 .61803 m
.24603 .62178 L
s
.27778 .61803 m
.27778 .62178 L
s
.30952 .61803 m
.30952 .62178 L
s
.37302 .61803 m
.37302 .62178 L
s
.40476 .61803 m
.40476 .62178 L
s
.43651 .61803 m
.43651 .62178 L
s
.46825 .61803 m
.46825 .62178 L
s
.53175 .61803 m
.53175 .62178 L
s
.56349 .61803 m
.56349 .62178 L
s
.59524 .61803 m
.59524 .62178 L
s
.62698 .61803 m
.62698 .62178 L
s
.69048 .61803 m
.69048 .62178 L
s
.72222 .61803 m
.72222 .62178 L
s
.75397 .61803 m
.75397 .62178 L
s
.78571 .61803 m
.78571 .62178 L
s
.84921 .61803 m
.84921 .62178 L
s
.88095 .61803 m
.88095 .62178 L
s
.9127 .61803 m
.9127 .62178 L
s
.94444 .61803 m
.94444 .62178 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.02381 0 m
.03006 0 L
s
[(0.69)] .01131 0 1 0 Mshowa
.02381 .12361 m
.03006 .12361 L
s
[(0.692)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.694)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.696)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.698)] .01131 .49443 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .30271 m
.0746 .30271 L
.07778 .30271 L
.08095 .30271 L
.08413 .30271 L
.0873 .30271 L
.09048 .30271 L
.09365 .30271 L
.09683 .30271 L
.1 .30271 L
.10317 .30271 L
.10635 .30271 L
.10952 .30271 L
.1127 .30271 L
.11587 .30271 L
.11905 .30271 L
.12222 .30271 L
.1254 .30271 L
.12857 .30271 L
.13175 .30271 L
.13492 .30271 L
.1381 .30271 L
.14127 .30271 L
.14444 .30271 L
.14762 .30271 L
.15079 .30271 L
.15397 .30271 L
.15714 .30271 L
.16032 .30271 L
.16349 .30271 L
.16667 .30271 L
.16984 .30271 L
.17302 .30271 L
.17619 .30271 L
.17937 .30271 L
.18254 .30271 L
.18571 .30271 L
.18889 .30271 L
.19206 .30271 L
.19524 .30271 L
.19841 .30271 L
.20159 .30271 L
.20476 .30271 L
.20794 .30271 L
.21111 .30271 L
.21429 .30271 L
.21746 .30271 L
.22063 .30271 L
.22381 .30271 L
.22698 .30271 L
Mistroke
.23016 .30271 L
.23333 .30271 L
.23651 .30271 L
.23968 .30271 L
.24286 .30271 L
.24603 .30271 L
.24921 .30271 L
.25238 .30271 L
.25556 .30271 L
.25873 .30271 L
.2619 .30271 L
.26508 .30271 L
.26825 .30271 L
.27143 .30271 L
.2746 .30271 L
.27778 .30271 L
.28095 .30271 L
.28413 .30271 L
.2873 .30271 L
.29048 .30271 L
.29365 .30271 L
.29683 .30271 L
.3 .30271 L
.30317 .30271 L
.30635 .30271 L
.30952 .30271 L
.3127 .30271 L
.31587 .30271 L
.31905 .30271 L
.32222 .30271 L
.3254 .30271 L
.32857 .30271 L
.33175 .30271 L
.33492 .30271 L
.3381 .30271 L
.34127 .30271 L
.34444 .30271 L
.34762 .30271 L
.35079 .30271 L
.35397 .30271 L
.35714 .30271 L
.36032 .30271 L
.36349 .30271 L
.36667 .30271 L
.36984 .30271 L
.37302 .30271 L
.37619 .30271 L
.37937 .30271 L
.38254 .30271 L
.38571 .30271 L
Mistroke
.38889 .30271 L
.39206 .30271 L
.39524 .30271 L
.39841 .30271 L
.40159 .30271 L
.40476 .30271 L
.40794 .30271 L
.41111 .30271 L
.41429 .30271 L
.41746 .30271 L
.42063 .30271 L
.42381 .30271 L
.42698 .30271 L
.43016 .30271 L
.43333 .30271 L
.43651 .30271 L
.43968 .30271 L
.44286 .30271 L
.44603 .30271 L
.44921 .30271 L
.45238 .30271 L
.45556 .30271 L
.45873 .30271 L
.4619 .30271 L
.46508 .30271 L
.46825 .30271 L
.47143 .30271 L
.4746 .30271 L
.47778 .30271 L
.48095 .30271 L
.48413 .30271 L
.4873 .30271 L
.49048 .30271 L
.49365 .30271 L
.49683 .30271 L
.5 .30271 L
.50317 .30271 L
.50635 .30271 L
.50952 .30271 L
.5127 .30271 L
.51587 .30271 L
.51905 .30271 L
.52222 .30271 L
.5254 .30271 L
.52857 .30271 L
.53175 .30271 L
.53492 .30271 L
.5381 .30271 L
.54127 .30271 L
.54444 .30271 L
Mistroke
.54762 .30271 L
.55079 .30271 L
.55397 .30271 L
.55714 .30271 L
.56032 .30271 L
.56349 .30271 L
.56667 .30271 L
.56984 .30271 L
.57302 .30271 L
.57619 .30271 L
.57937 .30271 L
.58254 .30271 L
.58571 .30271 L
.58889 .30271 L
.59206 .30271 L
.59524 .30271 L
.59841 .30271 L
.60159 .30271 L
.60476 .30271 L
.60794 .30271 L
.61111 .30271 L
.61429 .30271 L
.61746 .30271 L
.62063 .30271 L
.62381 .30271 L
.62698 .30271 L
.63016 .30271 L
.63333 .30271 L
.63651 .30271 L
.63968 .30271 L
.64286 .30271 L
.64603 .30271 L
.64921 .30271 L
.65238 .30271 L
.65556 .30271 L
.65873 .30271 L
.6619 .30271 L
.66508 .30271 L
.66825 .30271 L
.67143 .30271 L
.6746 .30271 L
.67778 .30271 L
.68095 .30271 L
.68413 .30271 L
.6873 .30271 L
.69048 .30271 L
.69365 .30271 L
.69683 .30271 L
.7 .30271 L
.70317 .30271 L
Mistroke
.70635 .30271 L
.70952 .30271 L
.7127 .30271 L
.71587 .30271 L
.71905 .30271 L
.72222 .30271 L
.7254 .30271 L
.72857 .30271 L
.73175 .30271 L
.73492 .30271 L
.7381 .30271 L
.74127 .30271 L
.74444 .30271 L
.74762 .30271 L
.75079 .30271 L
.75397 .30271 L
.75714 .30271 L
.76032 .30271 L
.76349 .30271 L
.76667 .30271 L
.76984 .30271 L
.77302 .30271 L
.77619 .30271 L
.77937 .30271 L
.78254 .30271 L
.78571 .30271 L
.78889 .30271 L
.79206 .30271 L
.79524 .30271 L
.79841 .30271 L
.80159 .30271 L
.80476 .30271 L
.80794 .30271 L
.81111 .30271 L
.81429 .30271 L
.81746 .30271 L
.82063 .30271 L
.82381 .30271 L
.82698 .30271 L
.83016 .30271 L
.83333 .30271 L
.83651 .30271 L
.83968 .30271 L
.84286 .30271 L
.84603 .30271 L
.84921 .30271 L
.85238 .30271 L
.85556 .30271 L
.85873 .30271 L
.8619 .30271 L
Mistroke
.86508 .30271 L
.86825 .30271 L
.87143 .30271 L
.8746 .30271 L
.87778 .30271 L
.88095 .30271 L
.88413 .30271 L
.8873 .30271 L
.89048 .30271 L
.89365 .30271 L
.89683 .30271 L
.9 .30271 L
.90317 .30271 L
.90635 .30271 L
.90952 .30271 L
.9127 .30271 L
.91587 .30271 L
.91905 .30271 L
.92222 .30271 L
.9254 .30271 L
.92857 .30271 L
.93175 .30271 L
.93492 .30271 L
.9381 .30271 L
.94127 .30271 L
.94444 .30271 L
.94762 .30271 L
.95079 .30271 L
.95397 .30271 L
.95714 .30271 L
.96032 .30271 L
.96349 .30271 L
.96667 .30271 L
.96984 .30271 L
.97302 .30271 L
.97619 .30271 L
Mfstroke
.07264 .61803 m
.07436 0 L
s
.07487 0 m
.07674 .61803 L
s
.07888 .61803 m
.08086 0 L
s
.08105 0 m
.08319 .61803 L
s
.08512 .61803 m
.0873 .01547 L
s
.0873 .01547 m
.08964 .61803 L
s
.09135 .61803 m
.09365 .05174 L
s
.09365 .05174 m
.0961 .61803 L
s
.09758 .61803 m
.1 .08153 L
s
.1 .08153 m
.10257 .61803 L
s
.10381 .61803 m
.10635 .10636 L
s
.10635 .10636 m
.10904 .61803 L
s
.11003 .61803 m
.1127 .12734 L
s
.1127 .12734 m
.11551 .61803 L
s
.11625 .61803 m
.11905 .14527 L
s
.11905 .14527 m
.12199 .61803 L
s
.12247 .61803 m
.1254 .16074 L
s
.1254 .16074 m
.12847 .61803 L
s
.12868 .61803 m
.13175 .1742 L
.13492 .61372 L
.1381 .18601 L
.14127 .59599 L
.14444 .19643 L
.14762 .58009 L
.15079 .20568 L
.15397 .56574 L
.15714 .21395 L
.16032 .55273 L
.16349 .22136 L
.16667 .54088 L
.16984 .22804 L
.17302 .53004 L
.17619 .23409 L
.17937 .52009 L
.18254 .23959 L
.18571 .51092 L
.18889 .24459 L
.19206 .50244 L
.19524 .24917 L
.19841 .49458 L
.20159 .25337 L
.20476 .48728 L
.20794 .25723 L
.21111 .48047 L
.21429 .26079 L
.21746 .4741 L
.22063 .26408 L
.22381 .46815 L
.22698 .26712 L
.23016 .46256 L
.23333 .26995 L
.23651 .4573 L
.23968 .27258 L
.24286 .45235 L
.24603 .27502 L
.24921 .44768 L
.25238 .27731 L
.25556 .44326 L
.25873 .27944 L
.2619 .43908 L
.26508 .28144 L
.26825 .43512 L
.27143 .28331 L
.2746 .43136 L
.27778 .28506 L
.28095 .42779 L
.28413 .28672 L
Mistroke
.2873 .42438 L
.29048 .28827 L
.29365 .42114 L
.29683 .28973 L
.3 .41805 L
.30317 .29111 L
.30635 .4151 L
.30952 .29241 L
.3127 .41227 L
.31587 .29364 L
.31905 .40957 L
.32222 .29481 L
.3254 .40698 L
.32857 .29591 L
.33175 .4045 L
.33492 .29695 L
.3381 .40212 L
.34127 .29794 L
.34444 .39983 L
.34762 .29888 L
.35079 .39764 L
.35397 .29978 L
.35714 .39552 L
.36032 .30062 L
.36349 .39349 L
.36667 .30143 L
.36984 .39153 L
.37302 .3022 L
.37619 .38964 L
.37937 .30293 L
.38254 .38782 L
.38571 .30363 L
.38889 .38606 L
.39206 .30429 L
.39524 .38436 L
.39841 .30492 L
.40159 .38272 L
.40476 .30553 L
.40794 .38113 L
.41111 .30611 L
.41429 .3796 L
.41746 .30666 L
.42063 .37811 L
.42381 .30719 L
.42698 .37667 L
.43016 .30769 L
.43333 .37528 L
.43651 .30817 L
.43968 .37393 L
.44286 .30864 L
Mistroke
.44603 .37261 L
.44921 .30908 L
.45238 .37134 L
.45556 .3095 L
.45873 .37011 L
.4619 .30991 L
.46508 .36891 L
.46825 .3103 L
.47143 .36774 L
.4746 .31067 L
.47778 .36661 L
.48095 .31103 L
.48413 .36551 L
.4873 .31137 L
.49048 .36444 L
.49365 .3117 L
.49683 .36339 L
.5 .31202 L
.50317 .36238 L
.50635 .31232 L
.50952 .36139 L
.5127 .31261 L
.51587 .36043 L
.51905 .3129 L
.52222 .35949 L
.5254 .31316 L
.52857 .35857 L
.53175 .31342 L
.53492 .35768 L
.5381 .31367 L
.54127 .35681 L
.54444 .31391 L
.54762 .35596 L
.55079 .31414 L
.55397 .35513 L
.55714 .31436 L
.56032 .35432 L
.56349 .31458 L
.56667 .35353 L
.56984 .31478 L
.57302 .35276 L
.57619 .31498 L
.57937 .35201 L
.58254 .31517 L
.58571 .35127 L
.58889 .31535 L
.59206 .35055 L
.59524 .31553 L
.59841 .34984 L
.60159 .3157 L
Mistroke
.60476 .34915 L
.60794 .31586 L
.61111 .34848 L
.61429 .31602 L
.61746 .34782 L
.62063 .31617 L
.62381 .34717 L
.62698 .31631 L
.63016 .34654 L
.63333 .31645 L
.63651 .34592 L
.63968 .31659 L
.64286 .34531 L
.64603 .31672 L
.64921 .34472 L
.65238 .31684 L
.65556 .34414 L
.65873 .31696 L
.6619 .34357 L
.66508 .31708 L
.66825 .34301 L
.67143 .31719 L
.6746 .34246 L
.67778 .3173 L
.68095 .34192 L
.68413 .3174 L
.6873 .34139 L
.69048 .3175 L
.69365 .34087 L
.69683 .3176 L
.7 .34037 L
.70317 .31769 L
.70635 .33987 L
.70952 .31778 L
.7127 .33938 L
.71587 .31786 L
.71905 .3389 L
.72222 .31795 L
.7254 .33843 L
.72857 .31803 L
.73175 .33796 L
.73492 .3181 L
.7381 .33751 L
.74127 .31818 L
.74444 .33706 L
.74762 .31825 L
.75079 .33662 L
.75397 .31831 L
.75714 .33619 L
.76032 .31838 L
Mistroke
.76349 .33577 L
.76667 .31844 L
.76984 .33535 L
.77302 .3185 L
.77619 .33494 L
.77937 .31856 L
.78254 .33454 L
.78571 .31861 L
.78889 .33414 L
.79206 .31867 L
.79524 .33375 L
.79841 .31872 L
.80159 .33337 L
.80476 .31877 L
.80794 .33299 L
.81111 .31881 L
.81429 .33262 L
.81746 .31886 L
.82063 .33225 L
.82381 .3189 L
.82698 .33189 L
.83016 .31894 L
.83333 .33154 L
.83651 .31898 L
.83968 .33119 L
.84286 .31902 L
.84603 .33085 L
.84921 .31906 L
.85238 .33051 L
.85556 .31909 L
.85873 .33018 L
.8619 .31912 L
.86508 .32985 L
.86825 .31916 L
.87143 .32953 L
.8746 .31918 L
.87778 .32921 L
.88095 .31921 L
.88413 .3289 L
.8873 .31924 L
.89048 .32859 L
.89365 .31927 L
.89683 .32829 L
.9 .31929 L
.90317 .32799 L
.90635 .31931 L
.90952 .32769 L
.9127 .31933 L
.91587 .3274 L
.91905 .31935 L
Mistroke
.92222 .32711 L
.9254 .31937 L
.92857 .32683 L
.93175 .31939 L
.93492 .32655 L
.9381 .31941 L
.94127 .32628 L
.94444 .31943 L
.94762 .326 L
.95079 .31944 L
.95397 .32574 L
.95714 .31945 L
.96032 .32547 L
.96349 .31947 L
.96667 .32521 L
.96984 .31948 L
.97302 .32495 L
.97619 .31949 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00OogooKWoo003oOom^
Ool000Qoo`8000Aoo`8000Aoo`8000=oo`<00?mooe9oo`001goo00@007ooOol000=oo`8000=oo`04
001oogoo0005Ool00`00Oomoo`3oOom?Ool000Moo`04001oogoo0008Ool01000Oomoo`000goo0`00
1Goo0`003Woo00<007oo0000ogoo>7oo0007Ool01000Oomoo`0027oo0`000goo00@007ooOol000Eo
o`03001oogoo00ioo`03001oo`000?moocQoo`001goo00@007ooOol000Qoo`03001oogoo00=oo`04
001oogoo0005Ool00`00Oomoo`0>Ool00`00Ool0003oOolhOol000Qoo`8000Yoo`<000=oo`8000Io
o`03001oogoo00ioo`05001oo`00Ool0003oOolfOol0029oo`03001oogoo00ioo`05001oo`00Ool0
003oOolfOol0029oo`8000moo`05001oo`00Ool0003oOolfOol0029oo`03001oogoo00ioo`05001o
o`00Ool0003oOolfOol0029oo`03001oogoo00ioo`05001oo`00Ool0003oOolfOol0029oo`03001o
ogoo00ioo`05001oo`00Ool0003oOolfOol0029oo`03001oogoo00ioo`07001oo`00Ool007oo0000
ogoo=7oo000ROol2000?Ool01`00Ool007oo001oo`000?moocAoo`008Woo00<007ooOol03Woo00L0
07oo001oo`00Ool0003oOoldOol0029oo`03001oogoo00ioo`07001oo`00Ool007oo0000ogoo=7oo
000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool0003oOolbOol0029oo`03001oogoo00io
o`09001oo`00Ool007oo001oo`000?mooc9oo`008Woo0P003goo00T007oo001oo`00Ool007oo0000
ogoo<Woo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool0003oOolbOol0029oo`03001o
ogoo00ioo`09001oo`00Ool007oo001oo`00009oo`03001oogoo0?moobeoo`000Woo0P0017oo0P00
17oo0P000goo0`000goo10001Goo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00<0
07ooOol0ogoo;Goo00001Goo001oogoo00000goo0P000goo00@007ooOol000Eoo`05001oogooOol0
0007Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool00`00Oomoo`3oOol]Ool00005
Ool007ooOol00008Ool01000Oomoo`000goo0`0017oo00<007ooOol017oo0`003Woo00T007oo001o
o`00Ool007oo00000Woo00<007ooOol0ogoo;Goo00001Goo001oogoo000027oo0`000goo00@007oo
Ool000Eoo`03001oogoo00=oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`03001o
o`000?moobeoo`0000Eoo`00Oomoo`0000Qoo`03001oogoo00=oo`04001oogoo0002Ool01000Oomo
o`001Goo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00<007oo0000ogoo;Goo0002
Ool2000:Ool30003Ool20004Ool20006Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002
Ool01@00Ool007oo0000ogoo:goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002
Ool01@00Ool007oo0000ogoo:goo000ROol2000?Ool02@00Ool007oo001oo`00Ool00002Ool01@00
Ool007oo0000ogoo:goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool01`00
Ool007oo001oo`000?moobUoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo
00L007oo001oo`00Ool0003oOolYOol0029oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00
009oo`09001oo`00Ool007oo001oo`000?moobMoo`008Woo00<007ooOol03Woo00T007oo001oo`00
Ool007oo00000Woo00T007oo001oo`00Ool007oo0000ogoo9goo000ROol2000?Ool02@00Ool007oo
001oo`00Ool00002Ool02`00Ool007oo001oo`00Ool007oo0000ogoo9Goo000ROol00`00Oomoo`0>
Ool02@00Ool007oo001oo`00Ool00002Ool02`00Ool007oo001oo`00Ool007oo0000ogoo9Goo000R
Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`000?moob=oo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`3oOolNOol0029oo`03001oogoo00ioo`09
001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007oo
0000ogoo7Woo000ROol2000?Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`03001oo`000?mooaioo`008Woo00<007ooOol03Woo00T007oo001oo`00
Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000ogoo
77oo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`07001oo`00Ool007oo0000ogoo6Woo0002Ool20004Ool20004Ool20003
Ool30004Ool30005Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`0;001oo`00Ool007oo001oo`00Ool0003oOolFOol00005Ool0
07ooOol00003Ool20003Ool01000Oomoo`001Goo00<007ooOol00Woo00<007ooOol017oo00<007oo
Ool03Woo00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`000?mooaAoo`0000Eoo`00Oomoo`0000Qoo`04001o
ogoo0003Ool30002Ool40005Ool3000>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007oo
Ool0ogoo3goo00001Goo001oogoo000027oo0`000goo00@007ooOol0009oo`03001oo`0000Ioo`03
001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000ogoo3Goo0000
1Goo001oogoo000027oo00<007ooOol00goo00@007ooOol0009oo`03001oo`0000Ioo`03001oogoo
00ioo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0003oOol9Ool0
009oo`8000Yoo`<000=oo`8000Aoo`8000Ioo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00
009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?moo`Eoo`008Woo00<007ooOol03Woo
00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00
Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00D007oo001oo`000?eoo`008Woo0P003goo00T007oo001oo`00Ool007oo00000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001o
o`00Ool007oo001oo`00Ool007oo00000Woo00/007oo001oo`00Ool007oo001oo`000?Moo`008Woo
00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000kWoo
000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool0
07oo001oo`00Ool007oo00000Woo00<007ooOol0hGoo000ROol00`00Oomoo`0=Oooo000c0009Ool0
029oo`8000moo`09001oo`00Ool007oo001oo`00009oo`0>001oo`00Ool007oo001oo`00Ool007oo
001oo`80000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`0;001oo`00Ool007oo001oo`00Ool0002kOol0029oo`03001oogoo00eo
o`04001oo`00Ool2000017oo001oo`000Woo00h007oo001oo`00Ool007oo001oo`00Ool007oo0P00
00aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`0?001oo`00Ool007oo001oo`00Ool007oo001oo`00009oo`0;001oo`00Ool0
07oo001oo`00Ool0002<Ool0029oo`03001oogoo00eoo`04001oo`00Ool2000017oo001oo`000Woo
00h007oo001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001o
o`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0?001oo`00Ool007oo
001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool0
07oo00800008Ool007oo001oo`00Ool000Uoo`008Woo00<007ooOol03Goo00H007oo001oo`00Ool2
00004Woo001oogoo001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000aoo`00Ool0
07oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03`00
Ool007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00`007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000;Ool007oo001oo`00Ool007oo001oo`020002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`07001oo`00Ool007oo00002Woo000ROol00`00Oomoo`0=Ool01P00Ool0
07oo001oo`80000BOol007ooOol007oo001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool0
07oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool0
07oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
3Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool0
07oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000=oo`00Ool00P0000Uoo`00Ool007oo001oo`00Ool00P0000eoo`00Ool0
07oo001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00/007oo001oo`00Ool007oo001oo`0002Aoo`008Woo0P00
3Woo00H007oo001oo`00Ool200004Woo001oogoo001oo`00Ool007oo001oo`00Ool007oo001oo`80
000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001o
o`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200000goo001oo`0200002Goo001oo`00Ool007oo001oo`0200003Goo001o
o`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eo
o`00Ool007oo001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007oo0000BWoo000ROol00`00Oomoo`0=Ool0
1P00Ool007oo001oo`@0009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`0200006goo001oo`00Ool007oo001oo`00Ool007oo
001oo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000?Ool007oo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`03001oo`0006Qoo`008Woo00<007ooOol03Goo00H007oo001oo`00Ool40002
Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool0
07oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00800007Ool007oo001oo`00Ool0
0P0000=oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool0
07oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool02@00
Ool007oo001oo`00Ool00020Ool0009oo`8000Aoo`8000Aoo`8000=oo`<000Aoo`8000Ioo`03001o
ogoo00eoo`06001oo`00Ool007oo1@0000moo`00Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Aoo`00001oo`<00005Ool007oo001o
o`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001o
o`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0002C
Ool00005Ool007ooOol00003Ool20003Ool01000Oomoo`001Goo00@007ooOol0009oo`03001oogoo
00=oo`03001oogoo00eoo`06001oo`00Ool007oo1@0000moo`00Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo00800004Ool00000Ool300001Goo001oo`00Ool00P0000eoo`00Ool007oo001oo`00
Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00
Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool00`00Ool0002TOol00005Ool007ooOol00008Ool01000Oomo
o`000goo0`000Woo00@007ooOol000Eoo`<000eoo`06001oo`00Ool007oo1`0000eoo`00Ool007oo
001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00800007Ool007oo
001oo`00Ool01000011oo`00001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001o
o`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05
001oo`00Ool0002aOol00005Ool007ooOol00008Ool30003Ool01000Oomoo`000Woo0`001Woo00<0
07ooOol03Goo00H007oo001oo`00Ool700003Goo001oo`00Ool007oo001oo`00Ool007oo0080000;
Ool007oo001oo`00Ool007oo001oo`0300003Goo00000000Ool007oo001oo`00Ool007oo0080000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
00003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`09001oo`00Ool007oo001oo`000;aoo`0000Eoo`00Oomoo`0000Qoo`03001oogoo00=o
o`04001oogoo0002Ool00`00Oomoo`06Ool00`00Oomoo`0=Ool01P00Ool007oo001oo`P0000<Ool0
0000Ool007oo001oo`00Ool007oo0P0000Moo`00Ool007oo001oo`0200000goo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`000<Moo`000Woo0P002Woo0`000goo0P0017oo
0`001Goo00<007ooOol03Goo00H007oo001oo`00Ool800000goo001oo`0200002Goo001oo`00Ool0
07oo001oo`0500001goo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool0
07oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0003?Ool0029o
o`03001oogoo00eoo`06001oo`00Ool007oo200000Eoo`00Ool007oo00800003Ool0000000<0000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
00003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000eWoo000ROol2000>Ool01P00Ool007oo
001oo`P00007Ool007oo001oo`00Ool00`00011oo`00001oo`00Ool007oo001oo`00Ool007oo001o
o`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo
001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`000=ao
o`008Woo00<007ooOol03Goo00H007oo001oo`00Ool800002goo001oo`00Ool007oo001oo`00Ool0
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool0
07oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00<007ooOol0h7oo000ROol00`00Oomoo`0=Ool01P00Ool007oo001oo`P0000;
Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`00
0Woo00T007oo001oo`00Ool007oo0000jGoo000ROol00`00Oomoo`0=Ool01P00Ool007oo001oo`P0
000;Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080
000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00D007oo001oo`000>eoo`008Woo00<007ooOol03Goo00H007oo001oo`00Ool800002goo
001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo
001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009o
o`03001oogoo0>moo`008Woo0P003Woo00H007oo001oo`00Ool800002goo001oo`00Ool007oo001o
o`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001o
o`00Ool007oo0002Ool02`00Ool007oo001oo`00Ool007oo0000mWoo000ROol00`00Oomoo`0=Ool0
1P00Ool007oo001oo`P0000;Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`07001oo`00Ool0
07oo0000nWoo000ROol00`00Oomoo`0=Ool01P00Ool007oo001oo`P0000;Ool007oo001oo`00Ool0
07oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`05001oo`00Ool0003lOol0029oo`03001oogoo00eoo`06001oo`00Ool0
07oo200000]oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00
Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0oWoo000ROol00`00
Oomoo`0=Ool01P00Ool007oo001oo`P0000;Ool007oo001oo`00Ool007oo001oo`0200003Woo001o
o`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?moo`Ao
o`008Woo0P003Woo00H007oo001oo`00Ool800002goo001oo`00Ool007oo001oo`00Ool00P0000io
o`00Ool007oo001oo`00Ool007oo001oo`000Woo00/007oo001oo`00Ool007oo001oo`000?moo`Io
o`008Woo00<007ooOol03Goo00H007oo001oo`00Ool800002goo001oo`00Ool007oo001oo`00Ool0
0P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00T007oo001oo`00Ool007oo0000ogoo
27oo000ROol00`00Oomoo`0=Ool01P00Ool007oo001oo`P0000;Ool007oo001oo`00Ool007oo001o
o`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool01`00Ool007oo001oo`000?moo`Yo
o`000Woo0P0017oo0P0017oo0P000goo0`0017oo0P001Woo00<007ooOol03Goo00H007oo001oo`00
Ool800002goo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001o
o`000Woo00D007oo001oo`000?moo`aoo`0000Eoo`00Oomoo`0000=oo`8000=oo`04001oogoo0005
Ool01000Oomoo`000Woo00<007ooOol00goo00<007ooOol03Goo00H007oo001oo`00Ool8000037oo
001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<0
07oo0000ogoo3Woo00001Goo001oogoo000027oo00@007ooOol000=oo`<0009oo`04001oogoo0005
Ool3000=Ool01P00Ool007oo001oo`P0000<Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`3oOol>Ool00005Ool007ooOol00008Ool3
0003Ool01000Oomoo`000goo0P001Woo00<007ooOol03Goo00H007oo001oo`00Ool8000037oo001o
o`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000ogoo4goo0000
1Goo001oogoo000027oo00<007ooOol00goo00@007ooOol0009oo`04001oogoo0005Ool00`00Oomo
o`0=Ool01P00Ool007oo001oo`P0000<Ool007oo001oo`00Ool007oo001oo`000Woo00/007oo001o
o`00Ool007oo001oo`000?mooaEoo`000Woo0P002Woo0`000goo0P0017oo0P001Woo00<007ooOol0
3Goo00H007oo001oo`00Ool8000037oo001oo`00Ool007oo001oo`00Ool0009oo`0;001oo`00Ool0
07oo001oo`00Ool0003oOolEOol0029oo`03001oogoo00eoo`06001oo`00Ool007oo200000aoo`00
Ool007oo001oo`00Ool007oo0002Ool02@00Ool007oo001oo`00Ool0003oOolGOol0029oo`8000io
o`06001oo`00Ool007oo200000aoo`00Ool007oo001oo`00Ool007oo0002Ool01`00Ool007oo001o
o`000?mooaUoo`008Woo00<007ooOol03Goo00H007oo001oo`00Ool8000037oo001oo`00Ool007oo
001oo`00Ool0009oo`07001oo`00Ool007oo0000ogoo6Goo000ROol00`00Oomoo`0=Ool01P00Ool0
07oo001oo`P0000<Ool007oo001oo`00Ool007oo001oo`000Woo00D007oo001oo`000?mooa]oo`00
8Woo00<007ooOol03Goo00H007oo001oo`00Ool8000037oo001oo`00Ool007oo001oo`00Ool0009o
o`03001oo`000?mooaeoo`008Woo00<007ooOol03Goo00H007oo001oo`00Ool8000037oo001oo`00
Ool007oo001oo`00Ool0009oo`03001oo`000?mooaeoo`008Woo0P003Woo00H007oo001oo`00Ool8
000037oo001oo`00Ool007oo001oo`00Ool0009oo`03001oogoo0?mooaeoo`008Woo00<007ooOol0
3Goo00H007oo001oo`00Ool8000037oo001oo`00Ool007oo001oo`00Ool0009oo`04001oogooOol3
0004Ool2000ZOol30004Ool20004Ool2000VOol30003Ool30004Ool2000WOol40003Ool20004Ool2
000VOol40002Ool30004Ool2000WOol30004Ool20004Ool20003Ool0029oo`03001oogoo00eoo`06
001oo`00Ool007oo200000aoo`00Ool007oo001oo`00Ool007oo0009Ool01000Oomoo`000Woo00<0
07ooOol0:7oo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo000VOol00`00Oomoo`05Ool0
1000Oomoo`000Woo00<007ooOol09Goo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo000V
Ool00`00Oomoo`05Ool01000Oomoo`000Woo00<007ooOol09goo00@007ooOol0009oo`04001oogoo
0002Ool10001Ool1Ool0029oo`03001oogoo00eoo`06001oo`00Ool007oo200000aoo`00Ool007oo
001oo`00Ool007oo0009Ool01000Oomoo`000Woo00<007ooOol0:7oo00<007ooOol00Woo00@007oo
Ool0009oo`04001oogoo000VOol00`00Oomoo`05Ool01000Oomoo`000Woo00<007ooOol09Woo00D0
07ooOomoo`00009oo`04001oogoo0002Ool00`00Oomoo`0UOol00`00Oomoo`04Ool01000Oomoo`00
0Woo00<007ooOol09goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1Ool0029oo`03001o
ogoo00eoo`06001oo`00Ool007oo200000Yoo`00Ool007oo001oo`00Ool000Qoo`<000=oo`04001o
ogoo000ZOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol002Ioo`03001oogoo009oo`<0
00=oo`04001oogoo000YOol01000Oomoo`000Woo00@007ooOol0009oo`03001oogoo02Ioo`03001o
ogoo00<000=oo`04001oogoo000WOol20003Ool01000Oomoo`000Woo00@007ooOol0009oo`008Woo
0P003Woo00H007oo001oo`00Ool800002Woo001oo`00Ool007oo001oo`0027oo00<007ooOol00goo
00@007ooOol002Yoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo00<007ooOol0
0Woo00<007ooOol00goo00@007ooOol002Ioo`04001oogoo0002Ool01000Oomoo`000Woo00@007oo
Ool002Eoo`04001oogoo0002Ool00`00Oomoo`03Ool01000Oomoo`00:Goo00@007ooOol0009oo`04
001oogoo0002Ool10001Ool1Ool0029oo`03001oogoo00eoo`06001oo`00Ool007oo200000Yoo`00
Ool007oo001oo`00Ool000Qoo`@000=oo`8002Yoo`8000Eoo`8000Aoo`8002Ioo`8000Aoo`@000=o
o`8002Qoo`8000Aoo`8000Aoo`8002Moo`8000=oo`@000=oo`8002Moo`<000Aoo`8000Aoo`8000=o
o`008Woo00<007ooOol03Goo00H007oo001oo`00Ool8000027oo001oo`00Ool007oo003oOolVOol0
029oo`03001oogoo00eoo`06001oo`00Ool007oo200000Qoo`00Ool007oo001oo`00ogoo9Woo000R
Ool00`00Oomoo`0=Ool01P00Ool007oo001oo`P00008Ool007oo001oo`00Ool00?moobIoo`006Woo
o`00D`000Goo0000\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 0.68954, 
  0.933765, 0.000101968}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[explot, OTSPlotCApp, \ \ PlotRange\  -> \ {0.69, \ 0.70}, \ 
      DisplayFunction\  -> \ $DisplayFunction]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -42.6443 61.8034 [
[.18254 .60553 -6 -9 ]
[.18254 .60553 6 0 ]
[.34127 .60553 -9 -9 ]
[.34127 .60553 9 0 ]
[.5 .60553 -9 -9 ]
[.5 .60553 9 0 ]
[.65873 .60553 -9 -9 ]
[.65873 .60553 9 0 ]
[.81746 .60553 -9 -9 ]
[.81746 .60553 9 0 ]
[.97619 .60553 -9 -9 ]
[.97619 .60553 9 0 ]
[.01131 0 -24 -4.5 ]
[.01131 0 0 4.5 ]
[.01131 .12361 -30 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -30 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -30 -4.5 ]
[.01131 .49443 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .62428 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .61803 m
.18254 .62428 L
s
[(50)] .18254 .60553 0 1 Mshowa
.34127 .61803 m
.34127 .62428 L
s
[(100)] .34127 .60553 0 1 Mshowa
.5 .61803 m
.5 .62428 L
s
[(150)] .5 .60553 0 1 Mshowa
.65873 .61803 m
.65873 .62428 L
s
[(200)] .65873 .60553 0 1 Mshowa
.81746 .61803 m
.81746 .62428 L
s
[(250)] .81746 .60553 0 1 Mshowa
.97619 .61803 m
.97619 .62428 L
s
[(300)] .97619 .60553 0 1 Mshowa
.125 Mabswid
.05556 .61803 m
.05556 .62178 L
s
.0873 .61803 m
.0873 .62178 L
s
.11905 .61803 m
.11905 .62178 L
s
.15079 .61803 m
.15079 .62178 L
s
.21429 .61803 m
.21429 .62178 L
s
.24603 .61803 m
.24603 .62178 L
s
.27778 .61803 m
.27778 .62178 L
s
.30952 .61803 m
.30952 .62178 L
s
.37302 .61803 m
.37302 .62178 L
s
.40476 .61803 m
.40476 .62178 L
s
.43651 .61803 m
.43651 .62178 L
s
.46825 .61803 m
.46825 .62178 L
s
.53175 .61803 m
.53175 .62178 L
s
.56349 .61803 m
.56349 .62178 L
s
.59524 .61803 m
.59524 .62178 L
s
.62698 .61803 m
.62698 .62178 L
s
.69048 .61803 m
.69048 .62178 L
s
.72222 .61803 m
.72222 .62178 L
s
.75397 .61803 m
.75397 .62178 L
s
.78571 .61803 m
.78571 .62178 L
s
.84921 .61803 m
.84921 .62178 L
s
.88095 .61803 m
.88095 .62178 L
s
.9127 .61803 m
.9127 .62178 L
s
.94444 .61803 m
.94444 .62178 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.02381 0 m
.03006 0 L
s
[(0.69)] .01131 0 1 0 Mshowa
.02381 .12361 m
.03006 .12361 L
s
[(0.692)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.694)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.696)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.698)] .01131 .49443 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .30271 m
.0746 .30271 L
.07778 .30271 L
.08095 .30271 L
.08413 .30271 L
.0873 .30271 L
.09048 .30271 L
.09365 .30271 L
.09683 .30271 L
.1 .30271 L
.10317 .30271 L
.10635 .30271 L
.10952 .30271 L
.1127 .30271 L
.11587 .30271 L
.11905 .30271 L
.12222 .30271 L
.1254 .30271 L
.12857 .30271 L
.13175 .30271 L
.13492 .30271 L
.1381 .30271 L
.14127 .30271 L
.14444 .30271 L
.14762 .30271 L
.15079 .30271 L
.15397 .30271 L
.15714 .30271 L
.16032 .30271 L
.16349 .30271 L
.16667 .30271 L
.16984 .30271 L
.17302 .30271 L
.17619 .30271 L
.17937 .30271 L
.18254 .30271 L
.18571 .30271 L
.18889 .30271 L
.19206 .30271 L
.19524 .30271 L
.19841 .30271 L
.20159 .30271 L
.20476 .30271 L
.20794 .30271 L
.21111 .30271 L
.21429 .30271 L
.21746 .30271 L
.22063 .30271 L
.22381 .30271 L
.22698 .30271 L
Mistroke
.23016 .30271 L
.23333 .30271 L
.23651 .30271 L
.23968 .30271 L
.24286 .30271 L
.24603 .30271 L
.24921 .30271 L
.25238 .30271 L
.25556 .30271 L
.25873 .30271 L
.2619 .30271 L
.26508 .30271 L
.26825 .30271 L
.27143 .30271 L
.2746 .30271 L
.27778 .30271 L
.28095 .30271 L
.28413 .30271 L
.2873 .30271 L
.29048 .30271 L
.29365 .30271 L
.29683 .30271 L
.3 .30271 L
.30317 .30271 L
.30635 .30271 L
.30952 .30271 L
.3127 .30271 L
.31587 .30271 L
.31905 .30271 L
.32222 .30271 L
.3254 .30271 L
.32857 .30271 L
.33175 .30271 L
.33492 .30271 L
.3381 .30271 L
.34127 .30271 L
.34444 .30271 L
.34762 .30271 L
.35079 .30271 L
.35397 .30271 L
.35714 .30271 L
.36032 .30271 L
.36349 .30271 L
.36667 .30271 L
.36984 .30271 L
.37302 .30271 L
.37619 .30271 L
.37937 .30271 L
.38254 .30271 L
.38571 .30271 L
Mistroke
.38889 .30271 L
.39206 .30271 L
.39524 .30271 L
.39841 .30271 L
.40159 .30271 L
.40476 .30271 L
.40794 .30271 L
.41111 .30271 L
.41429 .30271 L
.41746 .30271 L
.42063 .30271 L
.42381 .30271 L
.42698 .30271 L
.43016 .30271 L
.43333 .30271 L
.43651 .30271 L
.43968 .30271 L
.44286 .30271 L
.44603 .30271 L
.44921 .30271 L
.45238 .30271 L
.45556 .30271 L
.45873 .30271 L
.4619 .30271 L
.46508 .30271 L
.46825 .30271 L
.47143 .30271 L
.4746 .30271 L
.47778 .30271 L
.48095 .30271 L
.48413 .30271 L
.4873 .30271 L
.49048 .30271 L
.49365 .30271 L
.49683 .30271 L
.5 .30271 L
.50317 .30271 L
.50635 .30271 L
.50952 .30271 L
.5127 .30271 L
.51587 .30271 L
.51905 .30271 L
.52222 .30271 L
.5254 .30271 L
.52857 .30271 L
.53175 .30271 L
.53492 .30271 L
.5381 .30271 L
.54127 .30271 L
.54444 .30271 L
Mistroke
.54762 .30271 L
.55079 .30271 L
.55397 .30271 L
.55714 .30271 L
.56032 .30271 L
.56349 .30271 L
.56667 .30271 L
.56984 .30271 L
.57302 .30271 L
.57619 .30271 L
.57937 .30271 L
.58254 .30271 L
.58571 .30271 L
.58889 .30271 L
.59206 .30271 L
.59524 .30271 L
.59841 .30271 L
.60159 .30271 L
.60476 .30271 L
.60794 .30271 L
.61111 .30271 L
.61429 .30271 L
.61746 .30271 L
.62063 .30271 L
.62381 .30271 L
.62698 .30271 L
.63016 .30271 L
.63333 .30271 L
.63651 .30271 L
.63968 .30271 L
.64286 .30271 L
.64603 .30271 L
.64921 .30271 L
.65238 .30271 L
.65556 .30271 L
.65873 .30271 L
.6619 .30271 L
.66508 .30271 L
.66825 .30271 L
.67143 .30271 L
.6746 .30271 L
.67778 .30271 L
.68095 .30271 L
.68413 .30271 L
.6873 .30271 L
.69048 .30271 L
.69365 .30271 L
.69683 .30271 L
.7 .30271 L
.70317 .30271 L
Mistroke
.70635 .30271 L
.70952 .30271 L
.7127 .30271 L
.71587 .30271 L
.71905 .30271 L
.72222 .30271 L
.7254 .30271 L
.72857 .30271 L
.73175 .30271 L
.73492 .30271 L
.7381 .30271 L
.74127 .30271 L
.74444 .30271 L
.74762 .30271 L
.75079 .30271 L
.75397 .30271 L
.75714 .30271 L
.76032 .30271 L
.76349 .30271 L
.76667 .30271 L
.76984 .30271 L
.77302 .30271 L
.77619 .30271 L
.77937 .30271 L
.78254 .30271 L
.78571 .30271 L
.78889 .30271 L
.79206 .30271 L
.79524 .30271 L
.79841 .30271 L
.80159 .30271 L
.80476 .30271 L
.80794 .30271 L
.81111 .30271 L
.81429 .30271 L
.81746 .30271 L
.82063 .30271 L
.82381 .30271 L
.82698 .30271 L
.83016 .30271 L
.83333 .30271 L
.83651 .30271 L
.83968 .30271 L
.84286 .30271 L
.84603 .30271 L
.84921 .30271 L
.85238 .30271 L
.85556 .30271 L
.85873 .30271 L
.8619 .30271 L
Mistroke
.86508 .30271 L
.86825 .30271 L
.87143 .30271 L
.8746 .30271 L
.87778 .30271 L
.88095 .30271 L
.88413 .30271 L
.8873 .30271 L
.89048 .30271 L
.89365 .30271 L
.89683 .30271 L
.9 .30271 L
.90317 .30271 L
.90635 .30271 L
.90952 .30271 L
.9127 .30271 L
.91587 .30271 L
.91905 .30271 L
.92222 .30271 L
.9254 .30271 L
.92857 .30271 L
.93175 .30271 L
.93492 .30271 L
.9381 .30271 L
.94127 .30271 L
.94444 .30271 L
.94762 .30271 L
.95079 .30271 L
.95397 .30271 L
.95714 .30271 L
.96032 .30271 L
.96349 .30271 L
.96667 .30271 L
.96984 .30271 L
.97302 .30271 L
.97619 .30271 L
Mfstroke
.07257 .61803 m
.0743 0 L
s
.07493 0 m
.0768 .61803 L
s
.07881 .61803 m
.08079 0 L
s
.08112 0 m
.08325 .61803 L
s
.08505 .61803 m
.08729 0 L
s
.08731 0 m
.08971 .61803 L
s
.09128 .61803 m
.09365 .03516 L
s
.09365 .03516 m
.09617 .61803 L
s
.09751 .61803 m
.1 .06631 L
s
.1 .06631 m
.10264 .61803 L
s
.10373 .61803 m
.10635 .0923 L
s
.10635 .0923 m
.10911 .61803 L
s
.10996 .61803 m
.1127 .11428 L
s
.1127 .11428 m
.11558 .61803 L
s
.11617 .61803 m
.11905 .13307 L
s
.11905 .13307 m
.12206 .61803 L
s
.12239 .61803 m
.1254 .1493 L
s
.1254 .1493 m
.12854 .61803 L
s
.1286 .61803 m
.13175 .16343 L
.13492 .6031 L
.1381 .17583 L
.14127 .58595 L
.14444 .18678 L
.14762 .57057 L
.15079 .19652 L
.15397 .55668 L
.15714 .20521 L
.16032 .5441 L
.16349 .21302 L
.16667 .53263 L
.16984 .22007 L
.17302 .52215 L
.17619 .22645 L
.17937 .51252 L
.18254 .23224 L
.18571 .50365 L
.18889 .23753 L
.19206 .49544 L
.19524 .24237 L
.19841 .48784 L
.20159 .24681 L
.20476 .48077 L
.20794 .2509 L
.21111 .47418 L
.21429 .25467 L
.21746 .46803 L
.22063 .25815 L
.22381 .46227 L
.22698 .26138 L
.23016 .45686 L
.23333 .26438 L
.23651 .45177 L
.23968 .26717 L
.24286 .44698 L
.24603 .26977 L
.24921 .44246 L
.25238 .2722 L
.25556 .43819 L
.25873 .27447 L
.2619 .43415 L
.26508 .2766 L
.26825 .43031 L
.27143 .27859 L
.2746 .42668 L
.27778 .28047 L
.28095 .42322 L
.28413 .28223 L
Mistroke
.2873 .41993 L
.29048 .28389 L
.29365 .41679 L
.29683 .28545 L
.3 .4138 L
.30317 .28693 L
.30635 .41094 L
.30952 .28832 L
.3127 .40821 L
.31587 .28964 L
.31905 .40559 L
.32222 .29089 L
.3254 .40309 L
.32857 .29208 L
.33175 .40069 L
.33492 .2932 L
.3381 .39838 L
.34127 .29426 L
.34444 .39617 L
.34762 .29527 L
.35079 .39405 L
.35397 .29624 L
.35714 .392 L
.36032 .29715 L
.36349 .39003 L
.36667 .29802 L
.36984 .38814 L
.37302 .29885 L
.37619 .38631 L
.37937 .29964 L
.38254 .38455 L
.38571 .3004 L
.38889 .38284 L
.39206 .30112 L
.39524 .3812 L
.39841 .3018 L
.40159 .37961 L
.40476 .30246 L
.40794 .37808 L
.41111 .30309 L
.41429 .37659 L
.41746 .30369 L
.42063 .37515 L
.42381 .30426 L
.42698 .37376 L
.43016 .30481 L
.43333 .37241 L
.43651 .30534 L
.43968 .3711 L
.44286 .30585 L
Mistroke
.44603 .36983 L
.44921 .30633 L
.45238 .3686 L
.45556 .30679 L
.45873 .36741 L
.4619 .30724 L
.46508 .36625 L
.46825 .30767 L
.47143 .36512 L
.4746 .30808 L
.47778 .36402 L
.48095 .30847 L
.48413 .36296 L
.4873 .30885 L
.49048 .36192 L
.49365 .30921 L
.49683 .36091 L
.5 .30956 L
.50317 .35993 L
.50635 .3099 L
.50952 .35897 L
.5127 .31022 L
.51587 .35804 L
.51905 .31053 L
.52222 .35713 L
.5254 .31083 L
.52857 .35625 L
.53175 .31112 L
.53492 .35538 L
.5381 .3114 L
.54127 .35454 L
.54444 .31167 L
.54762 .35372 L
.55079 .31192 L
.55397 .35292 L
.55714 .31217 L
.56032 .35214 L
.56349 .31241 L
.56667 .35137 L
.56984 .31264 L
.57302 .35062 L
.57619 .31286 L
.57937 .34989 L
.58254 .31308 L
.58571 .34918 L
.58889 .31328 L
.59206 .34848 L
.59524 .31348 L
.59841 .3478 L
.60159 .31367 L
Mistroke
.60476 .34713 L
.60794 .31386 L
.61111 .34648 L
.61429 .31404 L
.61746 .34584 L
.62063 .31421 L
.62381 .34522 L
.62698 .31437 L
.63016 .34461 L
.63333 .31453 L
.63651 .34401 L
.63968 .31469 L
.64286 .34342 L
.64603 .31484 L
.64921 .34284 L
.65238 .31498 L
.65556 .34228 L
.65873 .31512 L
.6619 .34173 L
.66508 .31526 L
.66825 .34119 L
.67143 .31538 L
.6746 .34066 L
.67778 .31551 L
.68095 .34014 L
.68413 .31563 L
.6873 .33962 L
.69048 .31575 L
.69365 .33912 L
.69683 .31586 L
.7 .33863 L
.70317 .31597 L
.70635 .33815 L
.70952 .31607 L
.7127 .33768 L
.71587 .31617 L
.71905 .33721 L
.72222 .31627 L
.7254 .33676 L
.72857 .31637 L
.73175 .33631 L
.73492 .31646 L
.7381 .33587 L
.74127 .31655 L
.74444 .33543 L
.74762 .31663 L
.75079 .33501 L
.75397 .31671 L
.75714 .33459 L
.76032 .31679 L
Mistroke
.76349 .33418 L
.76667 .31687 L
.76984 .33378 L
.77302 .31694 L
.77619 .33338 L
.77937 .31701 L
.78254 .33299 L
.78571 .31708 L
.78889 .33261 L
.79206 .31714 L
.79524 .33223 L
.79841 .31721 L
.80159 .33186 L
.80476 .31727 L
.80794 .33149 L
.81111 .31733 L
.81429 .33114 L
.81746 .31739 L
.82063 .33078 L
.82381 .31744 L
.82698 .33043 L
.83016 .31749 L
.83333 .33009 L
.83651 .31754 L
.83968 .32976 L
.84286 .31759 L
.84603 .32942 L
.84921 .31764 L
.85238 .3291 L
.85556 .31768 L
.85873 .32877 L
.8619 .31773 L
.86508 .32846 L
.86825 .31777 L
.87143 .32815 L
.8746 .31781 L
.87778 .32784 L
.88095 .31785 L
.88413 .32754 L
.8873 .31788 L
.89048 .32724 L
.89365 .31792 L
.89683 .32694 L
.9 .31795 L
.90317 .32665 L
.90635 .31799 L
.90952 .32637 L
.9127 .31802 L
.91587 .32609 L
.91905 .31805 L
Mistroke
.92222 .32581 L
.9254 .31808 L
.92857 .32553 L
.93175 .3181 L
.93492 .32526 L
.9381 .31813 L
.94127 .325 L
.94444 .31815 L
.94762 .32473 L
.95079 .31818 L
.95397 .32448 L
.95714 .3182 L
.96032 .32422 L
.96349 .31822 L
.96667 .32397 L
.96984 .31824 L
.97302 .32372 L
.97619 .31826 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00OogooKWoo003oOom^
Ool000Qoo`8000Aoo`8000Aoo`8000=oo`<00?mooe9oo`001goo00@007ooOol000=oo`8000=oo`04
001oogoo0005Ool00`00Oomoo`3oOom?Ool000Moo`04001oogoo0008Ool01000Oomoo`000goo0`00
1Goo0`003Woo00D007oo001oo`000?moocIoo`001goo00@007ooOol000Qoo`<000=oo`04001oogoo
0005Ool00`00Oomoo`0>Ool01@00Ool007oo0000ogoo=Woo0007Ool01000Oomoo`0027oo00<007oo
Ool00goo00@007ooOol000Eoo`03001oogoo00ioo`05001oo`00Ool0003oOolfOol000Qoo`8000Yo
o`<000=oo`8000Ioo`03001oogoo00ioo`05001oo`00Ool0003oOolfOol0029oo`03001oogoo00io
o`05001oo`00Ool0003oOolfOol0029oo`8000moo`05001oo`00Ool0003oOolfOol0029oo`03001o
ogoo00ioo`07001oo`00Ool007oo0000ogoo=7oo000ROol00`00Oomoo`0>Ool01`00Ool007oo001o
o`000?moocAoo`008Woo00<007ooOol03Woo00L007oo001oo`00Ool0003oOoldOol0029oo`03001o
ogoo00ioo`07001oo`00Ool007oo0000ogoo=7oo000ROol2000?Ool01`00Ool007oo001oo`000?mo
ocAoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo0000ogoo<Woo000ROol00`00Oomo
o`0>Ool02@00Ool007oo001oo`00Ool0003oOolbOol0029oo`03001oogoo00ioo`09001oo`00Ool0
07oo001oo`000?mooc9oo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo0000ogoo<Woo
000ROol2000?Ool02@00Ool007oo001oo`00Ool00002Ool00`00Oomoo`3oOol]Ool0029oo`03001o
ogoo00ioo`09001oo`00Ool007oo001oo`00009oo`03001oogoo0?moobeoo`008Woo00<007ooOol0
3Woo00T007oo001oo`00Ool007oo00000Woo00<007ooOol0ogoo;Goo0002Ool20004Ool20004Ool2
0003Ool30003Ool40005Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool00`00Oomo
o`3oOol]Ool00005Ool007ooOol00003Ool20003Ool01000Oomoo`001Goo00D007ooOomoo`0000Mo
o`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`03001oo`000?moobeoo`0000Eoo`00
Oomoo`0000Qoo`04001oogoo0003Ool30004Ool00`00Oomoo`04Ool3000>Ool02@00Ool007oo001o
o`00Ool00002Ool00`00Ool0003oOol]Ool00005Ool007ooOol00008Ool30003Ool01000Oomoo`00
1Goo00<007ooOol00goo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00<007oo0000
ogoo;Goo00001Goo001oogoo000027oo00<007ooOol00goo00@007ooOol0009oo`04001oogoo0005
Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000ogoo:goo0002
Ool2000:Ool30003Ool20004Ool20006Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002
Ool01@00Ool007oo0000ogoo:goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002
Ool01`00Ool007oo001oo`000?moobUoo`008Woo0P003goo00T007oo001oo`00Ool007oo00000Woo
00L007oo001oo`00Ool0003oOolYOol0029oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00
009oo`09001oo`00Ool007oo001oo`000?moobMoo`008Woo00<007ooOol03Woo00T007oo001oo`00
Ool007oo00000Woo00T007oo001oo`00Ool007oo0000ogoo9goo000ROol00`00Oomoo`0>Ool02@00
Ool007oo001oo`00Ool00002Ool02`00Ool007oo001oo`00Ool007oo0000ogoo9Goo000ROol00`00
Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool02`00Ool007oo001oo`00Ool007oo0000ogoo
9Goo000ROol2000?Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`000?moob=oo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool0003oOolSOol0029oo`03001oogoo00ioo`09001oo`00Ool007oo
001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0ogoo7Woo000R
Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`03001oo`000?mooaioo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000ogoo77oo000R
Ool2000?Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`07001oo`00Ool007oo0000ogoo6Woo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`09001oo`00Ool007oo001oo`00
0?mooaQoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00
Ool007oo001oo`00Ool00002Ool02`00Ool007oo001oo`00Ool007oo0000ogoo5Woo0002Ool20004
Ool20004Ool20003Ool30004Ool30005Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
0000ogoo57oo00001Goo001oogoo00000goo0P000goo00@007ooOol000Eoo`03001oogoo009oo`03
001oogoo00Aoo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`3o
Ool?Ool00005Ool007ooOol00008Ool01000Oomoo`000goo0`000Woo10001Goo0`003Woo00T007oo
001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`05001oo`00Ool0003oOol=Ool00005Ool007ooOol00008Ool3
0003Ool01000Oomoo`000Woo00<007oo00001Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`09001oo`00Ool007oo001oo`000?moo`Uoo`0000Eoo`00Oomoo`0000Qoo`03001o
ogoo00=oo`04001oogoo0002Ool00`00Ool00006Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00
Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0003oOol5Ool0009oo`8000Yoo`<0
00=oo`8000Aoo`8000Ioo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00
Ool007oo001oo`00Ool007oo001oo`00009oo`03001oo`000?moo`008Woo00<007ooOol03Woo00T0
07oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00T0
07oo001oo`00Ool007oo0000nGoo000ROol2000?Ool02@00Ool007oo001oo`00Ool00002Ool03@00
Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`03001oo`000?1oo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool02`00Ool007oo001oo`00Ool007oo0000j7oo000ROol00`00Oomoo`0>Ool02@00Ool007oo
001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00T007oo
001oo`00Ool007oo0000fgoo000ROol00`00Oomoo`0=Oooo000c0009Ool0029oo`8000moo`09001o
o`00Ool007oo001oo`00009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`80000<Ool007oo
001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d0
07oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00T007oo001oo`00Ool007oo0000[Woo000R
Ool00`00Oomoo`0=Ool01P00Ool007oo001oo`80000BOol007ooOol007oo001oo`00Ool007oo001o
o`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo
001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool0
07oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`0?001oo`00Ool007oo001oo`00Ool007oo001oo`00009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool02@00Ool007oo001oo`00Ool0001`Ool0029oo`03001oogoo00eoo`06001oo`00Ool007oo
0P00019oo`00Oomoo`00Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000aoo`00Ool007oo
001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool0
07oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00l0
07oo001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00
Ool007oo001oo`00Ool00P0000Qoo`00Ool007oo001oo`002Goo000ROol00`00Oomoo`0=Ool01P00
Ool007oo001oo`80000BOol007ooOol007oo001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
00003Goo001oo`00Ool007oo001oo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001oo`00
0Woo00h007oo001oo`00Ool007oo001oo`00Ool007oo0P0000aoo`00Ool007oo001oo`00Ool007oo
0002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00
Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00
Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03000Ool007oo001o
o`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo
001oo`00Ool007oo001oo`0200000goo001oo`02000027oo001oo`00Ool007oo0002Ool03@00Ool0
07oo001oo`00Ool007oo001oo`00009oo`07001oo`00Ool007oo00002Woo000ROol00`00Oomoo`0=
Ool02000Ool007oo001oo`00Ool20002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool20000
3Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool0
07oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool0
07oo001oo`00Ool007oo00800003Ool007oo00800009Ool007oo001oo`00Ool007oo0080000=Ool0
07oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
37oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Ool0000/Ool0029oo`8000ioo`08001o
o`00Ool007oo001oo`80009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Qoo`00001oo`00Ool007oo0P0001=o
o`00Ool007oo001oo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000aoo`00Ool007oo001o
o`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`09001oo`00Ool007oo
001oo`0005=oo`008Woo00<007ooOol03Goo00P007oo001oo`00Ool007oo0P000Woo00h007oo001o
o`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000<Ool00000Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001o
o`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001o
o`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0001aOol0029oo`03001oogoo00eoo`08001o
o`00Ool007oo001oo`<0000?Ool007oo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo00800007Ool007oo001oo`00Ool00P0000=oo`0000000P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`03001oo`0008Ioo`000Woo0P0017oo0P0017oo0P000goo0`0017oo0P001Woo
00<007ooOol03Goo00P007oo001oo`00Ool007oo0`0000moo`00Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo00800005Ool007oo001oo`0200001Goo00000000Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`0009Yoo`00
00Eoo`00Oomoo`0000=oo`8000=oo`04001oogoo0005Ool01000Oomoo`000Woo00<007ooOol00goo
00<007ooOol03Goo00P007oo001oo`00Ool007oo1@0000eoo`00Ool007oo001oo`00Ool007oo001o
o`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00800007Ool007oo001oo`00Ool01000011o
o`00001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00
Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001o
o`00Ool007oo0000ZGoo00001Goo001oogoo000027oo00@007ooOol000=oo`<0009oo`04001oogoo
0005Ool3000=Ool02000Ool007oo001oo`00Ool500003Goo001oo`00Ool007oo001oo`00Ool007oo
00800009Ool007oo001oo`00Ool007oo00D0000=Ool00000001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`2cOol00005Ool0
07ooOol00008Ool30003Ool01000Oomoo`000Woo0`001Woo00<007ooOol03Goo00P007oo001oo`00
Ool007oo1P0000aoo`00001oo`00Ool007oo001oo`00Ool200000goo001oo`0400001Goo001oo`00
Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00
Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00
Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000
`7oo00001Goo001oogoo000027oo00<007ooOol00goo00@007ooOol0009oo`03001oogoo00Ioo`03
001oogoo00eoo`08001oo`00Ool007oo001oo`H00003Ool007oo00800003Ool007oo00D0000=Ool0
0000001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
3Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0
009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00T007oo001oo`00Ool007oo0000bgoo
0002Ool2000:Ool30003Ool20004Ool30005Ool00`00Oomoo`0=Ool02000Ool007oo001oo`00Ool6
00001Goo001oo`00Ool01@00011oo`00001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool0
07oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
3Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`03001oogoo0=5oo`008Woo00<007ooOol03Goo00P007oo001oo`00Ool007oo1P0000]oo`00
Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool0
3@00Ool007oo001oo`00Ool007oo001oo`00009oo`09001oo`00Ool007oo001oo`000=Yoo`008Woo
0P003Woo00P007oo001oo`00Ool007oo1P0000]oo`00Ool007oo001oo`00Ool007oo0080000=Ool0
07oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
3Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`03001oo`000>1oo`008Woo00<007ooOol03Goo00P007oo001oo`00Ool007oo1P0000]oo`00
Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool0
2`00Ool007oo001oo`00Ool007oo0000igoo000ROol00`00Oomoo`0=Ool02000Ool007oo001oo`00
Ool600002goo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001o
o`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`07001oo`00Ool007oo0000jgoo000ROol00`00Oomoo`0=Ool02000Ool007oo001o
o`00Ool600002goo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`03001oo`000>moo`008Woo00<007ooOol03Goo00P007oo001oo`00Ool007oo
1P0000]oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00
Ool0003dOol0029oo`8000ioo`08001oo`00Ool007oo001oo`H0000;Ool007oo001oo`00Ool007oo
001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo
001oo`00Ool0009oo`09001oo`00Ool007oo001oo`000?Qoo`008Woo00<007ooOol03Goo00P007oo
001oo`00Ool007oo1P0000]oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool0
07oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00D007oo001oo`00
0?aoo`008Woo00<007ooOol03Goo00P007oo001oo`00Ool007oo1P0000]oo`00Ool007oo001oo`00
Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001o
o`00Ool007oo00000Woo00<007oo0000oWoo000ROol00`00Oomoo`0=Ool02000Ool007oo001oo`00
Ool600002goo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001o
o`000Woo00d007oo001oo`00Ool007oo001oo`00Ool0003oOol4Ool0029oo`03001oogoo00eoo`08
001oo`00Ool007oo001oo`H0000;Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool0
07oo001oo`00Ool007oo0002Ool02`00Ool007oo001oo`00Ool007oo0000ogoo1Woo000ROol2000>
Ool02000Ool007oo001oo`00Ool600002goo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool0
07oo001oo`00Ool007oo001oo`000Woo00T007oo001oo`00Ool007oo0000ogoo27oo000ROol00`00
Oomoo`0=Ool02000Ool007oo001oo`00Ool600002goo001oo`00Ool007oo001oo`00Ool00P0000io
o`00Ool007oo001oo`00Ool007oo001oo`000Woo00L007oo001oo`00Ool0003oOol:Ool0029oo`03
001oogoo00eoo`08001oo`00Ool007oo001oo`H0000;Ool007oo001oo`00Ool007oo001oo`020000
3Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool01@00Ool007oo0000ogoo37oo0002Ool20004
Ool20004Ool20003Ool30004Ool20006Ool00`00Oomoo`0=Ool02000Ool007oo001oo`00Ool60000
37oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00<007oo0000ogoo3Woo00001Goo001oogoo00000goo0P000goo00@007ooOol000Eoo`04001oogoo
0002Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool02000Ool007oo001oo`00Ool6000037oo001oo`00
Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0
ogoo3Woo00001Goo001oogoo000027oo00@007ooOol000=oo`<0009oo`04001oogoo0005Ool3000=
Ool02000Ool007oo001oo`00Ool6000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00
Ool007oo001oo`00Ool007oo0000ogoo4goo00001Goo001oogoo000027oo0`000goo00@007ooOol0
00=oo`8000Ioo`03001oogoo00eoo`08001oo`00Ool007oo001oo`H0000<Ool007oo001oo`00Ool0
07oo001oo`000Woo00/007oo001oo`00Ool007oo001oo`000?mooaEoo`0000Eoo`00Oomoo`0000Qo
o`03001oogoo00=oo`04001oogoo0002Ool01000Oomoo`001Goo00<007ooOol03Goo00P007oo001o
o`00Ool007oo1P0000aoo`00Ool007oo001oo`00Ool007oo0002Ool02@00Ool007oo001oo`00Ool0
003oOolGOol0009oo`8000Yoo`<000=oo`8000Aoo`8000Ioo`03001oogoo00eoo`08001oo`00Ool0
07oo001oo`H0000<Ool007oo001oo`00Ool007oo001oo`000Woo00T007oo001oo`00Ool007oo0000
ogoo5goo000ROol00`00Oomoo`0=Ool02000Ool007oo001oo`00Ool6000037oo001oo`00Ool007oo
001oo`00Ool0009oo`07001oo`00Ool007oo0000ogoo6Goo000ROol2000>Ool02000Ool007oo001o
o`00Ool6000037oo001oo`00Ool007oo001oo`00Ool0009oo`05001oo`00Ool0003oOolKOol0029o
o`03001oogoo00eoo`08001oo`00Ool007oo001oo`H0000<Ool007oo001oo`00Ool007oo001oo`00
0Woo00D007oo001oo`000?mooa]oo`008Woo00<007ooOol03Goo00P007oo001oo`00Ool007oo1P00
00aoo`00Ool007oo001oo`00Ool007oo0002Ool00`00Ool0003oOolMOol0029oo`03001oogoo00eo
o`08001oo`00Ool007oo001oo`H0000<Ool007oo001oo`00Ool007oo001oo`000Woo00<007oo0000
ogoo7Goo000ROol00`00Oomoo`0=Ool02000Ool007oo001oo`00Ool6000037oo001oo`00Ool007oo
001oo`00Ool0009oo`03001oogoo0?mooaeoo`008Woo0P003Woo00P007oo001oo`00Ool007oo1P00
00aoo`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`3oOolMOol0029oo`03001oogoo00eo
o`08001oo`00Ool007oo001oo`H0000<Ool007oo001oo`00Ool007oo001oo`001Woo0`0017oo0P00
:Woo0`0017oo0P0017oo0P009Woo0`000goo0`0017oo0P009goo10000goo0P0017oo0P009Woo1000
0Woo0`0017oo0P009goo0`0017oo0P0017oo0P000goo000ROol00`00Oomoo`0=Ool02000Ool007oo
001oo`00Ool6000037oo001oo`00Ool007oo001oo`00Ool000Uoo`04001oogoo0002Ool00`00Oomo
o`0XOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol002Ioo`03001oogoo00Eoo`04001o
ogoo0002Ool00`00Oomoo`0UOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol002Ioo`03
001oogoo00Eoo`04001oogoo0002Ool00`00Oomoo`0WOol01000Oomoo`000Woo00@007ooOol0009o
o`40005oo`5oo`008Woo00<007ooOol03Goo00P007oo001oo`00Ool007oo1P0000Yoo`00Ool007oo
001oo`00Ool000]oo`04001oogoo0002Ool00`00Oomoo`0XOol00`00Oomoo`02Ool01000Oomoo`00
0Woo00@007ooOol002Ioo`03001oogoo00Eoo`04001oogoo0002Ool00`00Oomoo`0VOol01@00Oomo
ogoo00000Woo00@007ooOol0009oo`03001oogoo02Eoo`03001oogoo00Aoo`04001oogoo0002Ool0
0`00Oomoo`0WOol01000Oomoo`000Woo00@007ooOol0009oo`40005oo`5oo`008Woo00<007ooOol0
3Goo00P007oo001oo`00Ool007oo1P0000Yoo`00Ool007oo001oo`00Ool000Qoo`<000=oo`04001o
ogoo000ZOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol002Ioo`03001oogoo009oo`<0
00=oo`04001oogoo000YOol01000Oomoo`000Woo00@007ooOol0009oo`03001oogoo02Ioo`03001o
ogoo00<000=oo`04001oogoo000WOol20003Ool01000Oomoo`000Woo00@007ooOol0009oo`008Woo
0P003Woo00P007oo001oo`00Ool007oo1P0000Qoo`00Ool007oo001oo`002Woo00<007ooOol00goo
00@007ooOol002Yoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo00<007ooOol0
0Woo00<007ooOol00goo00@007ooOol002Ioo`04001oogoo0002Ool01000Oomoo`000Woo00@007oo
Ool002Eoo`04001oogoo0002Ool00`00Oomoo`03Ool01000Oomoo`00:Goo00@007ooOol0009oo`04
001oogoo0002Ool10001Ool1Ool0029oo`03001oogoo00eoo`08001oo`00Ool007oo001oo`H00008
Ool007oo001oo`00Ool000Yoo`@000=oo`8002Yoo`8000Eoo`8000Aoo`8002Ioo`8000Aoo`@000=o
o`8002Qoo`8000Aoo`8000Aoo`8002Moo`8000=oo`@000=oo`8002Moo`<000Aoo`8000Aoo`8000=o
o`008Woo00<007ooOol03Goo00P007oo001oo`00Ool007oo1P0000Qoo`00Ool007oo001oo`00ogoo
9Woo000ROol00`00Oomoo`0=Ool02000Ool007oo001oo`00Ool600001Woo001oo`00Ool00?moobQo
o`008Woo00<007ooOol03Goo00P007oo001oo`00Ool007oo1P0000Ioo`00Ool007oo003oOolXOol0
01Yoool005<0005oo`00\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 0.68954, 
  0.933765, 0.000101968}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[explotP, OTSPlotP, \ \ PlotRange\  -> {0.74, \ 0.75}, \ 
      DisplayFunction\  -> \ $DisplayFunction]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -45.7345 61.8034 [
[.18254 -0.0125 -6 -9 ]
[.18254 -0.0125 6 0 ]
[.34127 -0.0125 -9 -9 ]
[.34127 -0.0125 9 0 ]
[.5 -0.0125 -9 -9 ]
[.5 -0.0125 9 0 ]
[.65873 -0.0125 -9 -9 ]
[.65873 -0.0125 9 0 ]
[.81746 -0.0125 -9 -9 ]
[.81746 -0.0125 9 0 ]
[.97619 -0.0125 -9 -9 ]
[.97619 -0.0125 9 0 ]
[.01131 .12361 -30 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -30 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -30 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -24 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 0 m
.18254 .00625 L
s
[(50)] .18254 -0.0125 0 1 Mshowa
.34127 0 m
.34127 .00625 L
s
[(100)] .34127 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(150)] .5 -0.0125 0 1 Mshowa
.65873 0 m
.65873 .00625 L
s
[(200)] .65873 -0.0125 0 1 Mshowa
.81746 0 m
.81746 .00625 L
s
[(250)] .81746 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(300)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.15079 0 m
.15079 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.27778 0 m
.27778 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.53175 0 m
.53175 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.9127 0 m
.9127 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(0.742)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.744)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.746)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.748)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(0.75)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .20223 m
.0746 .20223 L
.07778 .20223 L
.08095 .20223 L
.08413 .20223 L
.0873 .20223 L
.09048 .20223 L
.09365 .20223 L
.09683 .20223 L
.1 .20223 L
.10317 .20223 L
.10635 .20223 L
.10952 .20223 L
.1127 .20223 L
.11587 .20223 L
.11905 .20223 L
.12222 .20223 L
.1254 .20223 L
.12857 .20223 L
.13175 .20223 L
.13492 .20223 L
.1381 .20223 L
.14127 .20223 L
.14444 .20223 L
.14762 .20223 L
.15079 .20223 L
.15397 .20223 L
.15714 .20223 L
.16032 .20223 L
.16349 .20223 L
.16667 .20223 L
.16984 .20223 L
.17302 .20223 L
.17619 .20223 L
.17937 .20223 L
.18254 .20223 L
.18571 .20223 L
.18889 .20223 L
.19206 .20223 L
.19524 .20223 L
.19841 .20223 L
.20159 .20223 L
.20476 .20223 L
.20794 .20223 L
.21111 .20223 L
.21429 .20223 L
.21746 .20223 L
.22063 .20223 L
.22381 .20223 L
.22698 .20223 L
Mistroke
.23016 .20223 L
.23333 .20223 L
.23651 .20223 L
.23968 .20223 L
.24286 .20223 L
.24603 .20223 L
.24921 .20223 L
.25238 .20223 L
.25556 .20223 L
.25873 .20223 L
.2619 .20223 L
.26508 .20223 L
.26825 .20223 L
.27143 .20223 L
.2746 .20223 L
.27778 .20223 L
.28095 .20223 L
.28413 .20223 L
.2873 .20223 L
.29048 .20223 L
.29365 .20223 L
.29683 .20223 L
.3 .20223 L
.30317 .20223 L
.30635 .20223 L
.30952 .20223 L
.3127 .20223 L
.31587 .20223 L
.31905 .20223 L
.32222 .20223 L
.3254 .20223 L
.32857 .20223 L
.33175 .20223 L
.33492 .20223 L
.3381 .20223 L
.34127 .20223 L
.34444 .20223 L
.34762 .20223 L
.35079 .20223 L
.35397 .20223 L
.35714 .20223 L
.36032 .20223 L
.36349 .20223 L
.36667 .20223 L
.36984 .20223 L
.37302 .20223 L
.37619 .20223 L
.37937 .20223 L
.38254 .20223 L
.38571 .20223 L
Mistroke
.38889 .20223 L
.39206 .20223 L
.39524 .20223 L
.39841 .20223 L
.40159 .20223 L
.40476 .20223 L
.40794 .20223 L
.41111 .20223 L
.41429 .20223 L
.41746 .20223 L
.42063 .20223 L
.42381 .20223 L
.42698 .20223 L
.43016 .20223 L
.43333 .20223 L
.43651 .20223 L
.43968 .20223 L
.44286 .20223 L
.44603 .20223 L
.44921 .20223 L
.45238 .20223 L
.45556 .20223 L
.45873 .20223 L
.4619 .20223 L
.46508 .20223 L
.46825 .20223 L
.47143 .20223 L
.4746 .20223 L
.47778 .20223 L
.48095 .20223 L
.48413 .20223 L
.4873 .20223 L
.49048 .20223 L
.49365 .20223 L
.49683 .20223 L
.5 .20223 L
.50317 .20223 L
.50635 .20223 L
.50952 .20223 L
.5127 .20223 L
.51587 .20223 L
.51905 .20223 L
.52222 .20223 L
.5254 .20223 L
.52857 .20223 L
.53175 .20223 L
.53492 .20223 L
.5381 .20223 L
.54127 .20223 L
.54444 .20223 L
Mistroke
.54762 .20223 L
.55079 .20223 L
.55397 .20223 L
.55714 .20223 L
.56032 .20223 L
.56349 .20223 L
.56667 .20223 L
.56984 .20223 L
.57302 .20223 L
.57619 .20223 L
.57937 .20223 L
.58254 .20223 L
.58571 .20223 L
.58889 .20223 L
.59206 .20223 L
.59524 .20223 L
.59841 .20223 L
.60159 .20223 L
.60476 .20223 L
.60794 .20223 L
.61111 .20223 L
.61429 .20223 L
.61746 .20223 L
.62063 .20223 L
.62381 .20223 L
.62698 .20223 L
.63016 .20223 L
.63333 .20223 L
.63651 .20223 L
.63968 .20223 L
.64286 .20223 L
.64603 .20223 L
.64921 .20223 L
.65238 .20223 L
.65556 .20223 L
.65873 .20223 L
.6619 .20223 L
.66508 .20223 L
.66825 .20223 L
.67143 .20223 L
.6746 .20223 L
.67778 .20223 L
.68095 .20223 L
.68413 .20223 L
.6873 .20223 L
.69048 .20223 L
.69365 .20223 L
.69683 .20223 L
.7 .20223 L
.70317 .20223 L
Mistroke
.70635 .20223 L
.70952 .20223 L
.7127 .20223 L
.71587 .20223 L
.71905 .20223 L
.72222 .20223 L
.7254 .20223 L
.72857 .20223 L
.73175 .20223 L
.73492 .20223 L
.7381 .20223 L
.74127 .20223 L
.74444 .20223 L
.74762 .20223 L
.75079 .20223 L
.75397 .20223 L
.75714 .20223 L
.76032 .20223 L
.76349 .20223 L
.76667 .20223 L
.76984 .20223 L
.77302 .20223 L
.77619 .20223 L
.77937 .20223 L
.78254 .20223 L
.78571 .20223 L
.78889 .20223 L
.79206 .20223 L
.79524 .20223 L
.79841 .20223 L
.80159 .20223 L
.80476 .20223 L
.80794 .20223 L
.81111 .20223 L
.81429 .20223 L
.81746 .20223 L
.82063 .20223 L
.82381 .20223 L
.82698 .20223 L
.83016 .20223 L
.83333 .20223 L
.83651 .20223 L
.83968 .20223 L
.84286 .20223 L
.84603 .20223 L
.84921 .20223 L
.85238 .20223 L
.85556 .20223 L
.85873 .20223 L
.8619 .20223 L
Mistroke
.86508 .20223 L
.86825 .20223 L
.87143 .20223 L
.8746 .20223 L
.87778 .20223 L
.88095 .20223 L
.88413 .20223 L
.8873 .20223 L
.89048 .20223 L
.89365 .20223 L
.89683 .20223 L
.9 .20223 L
.90317 .20223 L
.90635 .20223 L
.90952 .20223 L
.9127 .20223 L
.91587 .20223 L
.91905 .20223 L
.92222 .20223 L
.9254 .20223 L
.92857 .20223 L
.93175 .20223 L
.93492 .20223 L
.9381 .20223 L
.94127 .20223 L
.94444 .20223 L
.94762 .20223 L
.95079 .20223 L
.95397 .20223 L
.95714 .20223 L
.96032 .20223 L
.96349 .20223 L
.96667 .20223 L
.96984 .20223 L
.97302 .20223 L
.97619 .20223 L
Mfstroke
.07236 .61803 m
.07408 0 L
s
.07517 0 m
.07704 .61803 L
s
.07856 .61803 m
.08054 0 L
s
.0814 0 m
.08353 .61803 L
s
.08475 .61803 m
.08699 0 L
s
.08763 0 m
.09003 .61803 L
s
.09094 .61803 m
.09345 0 L
s
.09386 0 m
.09654 .61803 L
s
.09713 .61803 m
.09991 0 L
s
.10009 0 m
.10305 .61803 L
s
.10331 .61803 m
.10635 .00588 L
.10952 .61052 L
.1127 .02686 L
.11587 .58115 L
.11905 .04479 L
.12222 .55558 L
.1254 .06026 L
.12857 .53312 L
.13175 .07372 L
.13492 .51324 L
.1381 .08553 L
.14127 .49551 L
.14444 .09595 L
.14762 .47961 L
.15079 .1052 L
.15397 .46526 L
.15714 .11347 L
.16032 .45225 L
.16349 .12088 L
.16667 .4404 L
.16984 .12756 L
.17302 .42956 L
.17619 .13361 L
.17937 .41961 L
.18254 .13911 L
.18571 .41044 L
.18889 .14411 L
.19206 .40196 L
.19524 .14869 L
.19841 .3941 L
.20159 .15289 L
.20476 .38679 L
.20794 .15675 L
.21111 .37998 L
.21429 .16031 L
.21746 .37362 L
.22063 .1636 L
.22381 .36766 L
.22698 .16664 L
.23016 .36207 L
.23333 .16947 L
.23651 .35682 L
.23968 .1721 L
.24286 .35187 L
.24603 .17454 L
.24921 .3472 L
.25238 .17683 L
.25556 .34278 L
.25873 .17896 L
Mistroke
.2619 .3386 L
.26508 .18096 L
.26825 .33464 L
.27143 .18283 L
.2746 .33088 L
.27778 .18458 L
.28095 .32731 L
.28413 .18623 L
.2873 .3239 L
.29048 .18779 L
.29365 .32066 L
.29683 .18925 L
.3 .31757 L
.30317 .19063 L
.30635 .31462 L
.30952 .19193 L
.3127 .31179 L
.31587 .19316 L
.31905 .30909 L
.32222 .19433 L
.3254 .3065 L
.32857 .19543 L
.33175 .30402 L
.33492 .19647 L
.3381 .30164 L
.34127 .19746 L
.34444 .29935 L
.34762 .1984 L
.35079 .29716 L
.35397 .19929 L
.35714 .29504 L
.36032 .20014 L
.36349 .29301 L
.36667 .20095 L
.36984 .29105 L
.37302 .20172 L
.37619 .28916 L
.37937 .20245 L
.38254 .28734 L
.38571 .20314 L
.38889 .28558 L
.39206 .20381 L
.39524 .28388 L
.39841 .20444 L
.40159 .28224 L
.40476 .20505 L
.40794 .28065 L
.41111 .20563 L
.41429 .27912 L
.41746 .20618 L
Mistroke
.42063 .27763 L
.42381 .20671 L
.42698 .27619 L
.43016 .20721 L
.43333 .2748 L
.43651 .20769 L
.43968 .27344 L
.44286 .20815 L
.44603 .27213 L
.44921 .2086 L
.45238 .27086 L
.45556 .20902 L
.45873 .26963 L
.4619 .20943 L
.46508 .26843 L
.46825 .20982 L
.47143 .26726 L
.4746 .21019 L
.47778 .26613 L
.48095 .21055 L
.48413 .26503 L
.4873 .21089 L
.49048 .26395 L
.49365 .21122 L
.49683 .26291 L
.5 .21154 L
.50317 .2619 L
.50635 .21184 L
.50952 .26091 L
.5127 .21213 L
.51587 .25994 L
.51905 .21241 L
.52222 .25901 L
.5254 .21268 L
.52857 .25809 L
.53175 .21294 L
.53492 .2572 L
.5381 .21319 L
.54127 .25633 L
.54444 .21343 L
.54762 .25548 L
.55079 .21366 L
.55397 .25465 L
.55714 .21388 L
.56032 .25384 L
.56349 .2141 L
.56667 .25305 L
.56984 .2143 L
.57302 .25228 L
.57619 .2145 L
Mistroke
.57937 .25152 L
.58254 .21469 L
.58571 .25079 L
.58889 .21487 L
.59206 .25007 L
.59524 .21505 L
.59841 .24936 L
.60159 .21522 L
.60476 .24867 L
.60794 .21538 L
.61111 .248 L
.61429 .21554 L
.61746 .24734 L
.62063 .21569 L
.62381 .24669 L
.62698 .21583 L
.63016 .24606 L
.63333 .21597 L
.63651 .24544 L
.63968 .21611 L
.64286 .24483 L
.64603 .21624 L
.64921 .24424 L
.65238 .21636 L
.65556 .24366 L
.65873 .21648 L
.6619 .24309 L
.66508 .2166 L
.66825 .24253 L
.67143 .21671 L
.6746 .24198 L
.67778 .21682 L
.68095 .24144 L
.68413 .21692 L
.6873 .24091 L
.69048 .21702 L
.69365 .24039 L
.69683 .21712 L
.7 .23989 L
.70317 .21721 L
.70635 .23939 L
.70952 .2173 L
.7127 .2389 L
.71587 .21738 L
.71905 .23842 L
.72222 .21747 L
.7254 .23795 L
.72857 .21755 L
.73175 .23748 L
.73492 .21762 L
Mistroke
.7381 .23703 L
.74127 .21769 L
.74444 .23658 L
.74762 .21777 L
.75079 .23614 L
.75397 .21783 L
.75714 .23571 L
.76032 .2179 L
.76349 .23529 L
.76667 .21796 L
.76984 .23487 L
.77302 .21802 L
.77619 .23446 L
.77937 .21808 L
.78254 .23406 L
.78571 .21813 L
.78889 .23366 L
.79206 .21819 L
.79524 .23327 L
.79841 .21824 L
.80159 .23289 L
.80476 .21829 L
.80794 .23251 L
.81111 .21833 L
.81429 .23214 L
.81746 .21838 L
.82063 .23177 L
.82381 .21842 L
.82698 .23141 L
.83016 .21846 L
.83333 .23106 L
.83651 .2185 L
.83968 .23071 L
.84286 .21854 L
.84603 .23037 L
.84921 .21858 L
.85238 .23003 L
.85556 .21861 L
.85873 .2297 L
.8619 .21864 L
.86508 .22937 L
.86825 .21867 L
.87143 .22905 L
.8746 .2187 L
.87778 .22873 L
.88095 .21873 L
.88413 .22842 L
.8873 .21876 L
.89048 .22811 L
.89365 .21878 L
Mistroke
.89683 .2278 L
.9 .21881 L
.90317 .2275 L
.90635 .21883 L
.90952 .22721 L
.9127 .21885 L
.91587 .22692 L
.91905 .21887 L
.92222 .22663 L
.9254 .21889 L
.92857 .22635 L
.93175 .21891 L
.93492 .22607 L
.9381 .21893 L
.94127 .22579 L
.94444 .21894 L
.94762 .22552 L
.95079 .21896 L
.95397 .22526 L
.95714 .21897 L
.96032 .22499 L
.96349 .21899 L
.96667 .22473 L
.96984 .219 L
.97302 .22447 L
.97619 .21901 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00OogooKWoo003oOom^
Ool0059oo`<000Aoo`8002Yoo`<000Aoo`8000Aoo`8002Ioo`<000=oo`<000Aoo`8002Moo`@000=o
o`8000Aoo`8002Ioo`@0009oo`<000Aoo`8002Moo`<000Aoo`8000Aoo`8000=oo`00EGoo00@007oo
Ool0009oo`03001oogoo02Qoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo00<0
07ooOol01Goo00@007ooOol0009oo`03001oogoo02Eoo`03001oogoo009oo`04001oogoo0002Ool0
1000Oomoo`009Woo00<007ooOol01Goo00@007ooOol0009oo`03001oogoo02Moo`04001oogoo0002
Ool01000Oomoo`000Woo0@000Goo0Goo001EOol01000Oomoo`000Woo00<007ooOol0:7oo00<007oo
Ool00Woo00@007ooOol0009oo`04001oogoo000VOol00`00Oomoo`05Ool01000Oomoo`000Woo00<0
07ooOol09Woo00D007ooOomoo`00009oo`04001oogoo0002Ool00`00Oomoo`0UOol00`00Oomoo`04
Ool01000Oomoo`000Woo00<007ooOol09goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1
Ool0059oo`<000=oo`04001oogoo000ZOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol0
02Ioo`03001oogoo009oo`<000=oo`04001oogoo000YOol01000Oomoo`000Woo00@007ooOol0009o
o`03001oogoo02Ioo`03001oogoo00<000=oo`04001oogoo000WOol20003Ool01000Oomoo`000Woo
00@007ooOol0009oo`00DWoo00<007ooOol00goo00@007ooOol002Yoo`03001oogoo009oo`04001o
ogoo0002Ool01000Oomoo`009Woo00<007ooOol00Woo00<007ooOol00goo00@007ooOol002Ioo`04
001oogoo0002Ool01000Oomoo`000Woo00@007ooOol002Eoo`04001oogoo0002Ool00`00Oomoo`03
Ool01000Oomoo`00:Goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1Ool0059oo`@000=o
o`8002Yoo`8000Eoo`8000Aoo`8002Ioo`8000Aoo`@000=oo`8002Qoo`8000Aoo`8000Aoo`8002Mo
o`8000=oo`@000=oo`8002Moo`<000Aoo`8000Aoo`8000=oo`00ogooKWoo003oOom^Ool00?moofio
o`006Wooo`00D`000Goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool00`00
Oomoo`0FOol00`00Oomoo`0cOol00`00Oomoo`0bOol00`00Oomoo`0cOol00`00Oomoo`0bOol00`00
Oomoo`0cOol00`00Oomoo`07Ool0029oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009o
o`03001oogoo0?moobeoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00<0
07ooOol0ogoo;Goo000ROol2000?Ool02@00Ool007oo001oo`00Ool00002Ool00`00Ool0003oOol]
Ool0029oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`03001oo`000?moobeoo`00
8Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00D007oo001oo`000?moob]oo`00
8Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00D007oo001oo`000?moob]oo`00
8Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0003oOolY
Ool0029oo`8000moo`09001oo`00Ool007oo001oo`00009oo`07001oo`00Ool007oo0000ogoo:Goo
000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0
003oOolWOol0029oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`09001oo`00Ool0
07oo001oo`000?moobMoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00/0
07oo001oo`00Ool007oo001oo`000?moobEoo`008Woo0P003goo00T007oo001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool0003oOolSOol0029oo`03001oogoo00ioo`09001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000ogoo8goo0002Ool2
0004Ool20004Ool00`00Oomoo`03Ool30002Ool40005Ool00`00Oomoo`0>Ool02@00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oogoo0?mooaioo`00
00Eoo`00Oomoo`0000=oo`8000Aoo`03001oogoo00Aoo`03001oogoo009oo`03001oogoo00Eoo`03
001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00<007oo0000ogoo7Woo00001Goo001oogoo00002Woo00@007ooOomoo`@000Aoo`03001o
ogoo00Aoo`<000ioo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00D007oo001oo`000?mooaaoo`0000Eoo`00Oomoo`0000Yoo`07001oogooOol007oo
00001Woo00<007ooOol00goo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`000?mooaYoo`0000Eoo`00Oomo
o`0000]oo`06001oogoo001oo`000goo00@007ooOol000Eoo`03001oogoo00ioo`09001oo`00Ool0
07oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00T007oo001oo`00Ool0
07oo0000ogoo67oo0002Ool20009Ool40003Ool20004Ool20006Ool00`00Oomoo`0>Ool02@00Ool0
07oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool0
07oo001oo`00Ool007oo0000ogoo57oo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00<007ooOol0ogoo3goo000ROol2000?Ool02@00Ool007oo001oo`00Ool00002Ool0
3@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00D007oo001oo`000?moo`eoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`09001oo`00Ool007oo001oo`000?moo`Uoo`008Woo00<007ooOol03Woo00T007oo001o
o`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0
ogoo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00
Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00
Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`000?]oo`008Woo0P003goo00T007oo001o
o`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool00`00Oomoo`3`Ool0029oo`03001oogoo00ioo`09001oo`00
Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00
Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0003VOol0029o
o`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool0
07oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`000=Moo`008Woo00<007oo
Ool03Gooo`00<`002Goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03P00
Ool007oo001oo`00Ool007oo001oo`00Ool2000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00l007oo001oo`00Ool007oo
001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Ool00025
Ool0029oo`8000moo`09001oo`00Ool007oo001oo`00009oo`0>001oo`00Ool007oo001oo`00Ool0
07oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000aoo`00Ool007oo001oo`00
Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001o
o`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool03`00Ool007oo001oo`00Ool007oo001oo`00Ool00002Ool0
3@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo
001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02000027oo001oo`00Ool007oo
0009Ool0029oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`0>001oo`00Ool007oo
001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool0
07oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000<Ool0
07oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00
Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03P00Ool007oo001oo`00Ool007oo001oo`00
Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001o
o`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo
001oo`00Ool00002Ool03000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0001aoo`00Ool007oo001oo`00Ool007oo001oo`00Ool007oo001oo`00Ool0
07oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00
Ool0000:Ool0009oo`8000Aoo`8000Aoo`03001oogoo00=oo`<000=oo`<000Eoo`03001oogoo00io
o`09001oo`00Ool007oo001oo`00009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`80000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
00003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00
Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00
Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000;Ool007oo001oo`00
Ool007oo001oo`0200001Woo000007oo001oo`800007Ool007oo001oo`00Ool00P0000Eoo`00Ool0
07oo00800007Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
37oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo
00d007oo001oo`00Ool007oo001oo`00Ool0000aOol00005Ool007ooOol00003Ool20004Ool00`00
Oomoo`04Ool00`00Oomoo`03Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00
Ool00002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001o
o`00Ool007oo001oo`00Ool007oo00800007Ool007oo001oo`00Ool00P0000=oo`00Ool00P0000mo
o`00Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001o
o`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000<Ool007oo001oo`00Ool007oo001o
o`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000Egoo00001Goo
001oogoo00002Woo00@007ooOomoo`@0009oo`@000Eoo`<000ioo`09001oo`00Ool007oo001oo`00
009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000]oo`00Ool007oo001o
o`00Ool007oo00@00005Ool007oo001oo`0200001Goo001oo`00Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001o
o`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000MGoo00001Goo001oogoo00002Woo
00L007ooOomoo`00Ool00003Ool00`00Ool00006Ool00`00Oomoo`0>Ool02@00Ool007oo001oo`00
Ool00002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001o
o`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200001goo001oo`00Ool007oo00800003Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001o
o`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00/007oo001oo`00Ool007oo001oo`0008eoo`00
00Eoo`00Oomoo`0000]oo`06001oogoo001oo`000goo00<007oo00001Woo00<007ooOol03Woo00T0
07oo001oo`00Ool007oo00000Woo00h007oo001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00800005
Ool007oo001oo`03000017oo000007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`020000
3Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool01`00Ool007oo001oo`000:1oo`000Woo0P002Goo10000goo0P0017oo0P001Woo00<0
07ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00h007oo001oo`00Ool007oo001oo`00Ool0
07oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200001goo001oo`00Ool007oo00@0000@
Ool00000Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool0
07oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool0
07oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`00
0:moo`008Woo0P003goo00T007oo001oo`00Ool007oo00000Woo00h007oo001oo`00Ool007oo001o
o`00Ool007oo0P0000]oo`00Ool007oo001oo`00Ool007oo00<0000=Ool00000001oo`00Ool007oo
001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool0
07oo001oo`00Ool007oo00000Woo00T007oo001oo`00Ool007oo0000_7oo000ROol00`00Oomoo`0>
Ool02@00Ool007oo001oo`00Ool00002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool20000
1Goo001oo`00Ool01P00011oo`00001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo
001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`000<Mo
o`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00`007oo001oo`00Ool007oo
001oo`00Ool300003Goo00000000Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool0
07oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05001oo`00
Ool0003?Ool0029oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`08001oo`00Ool0
07oo001oo`@0000@Ool00000Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00
Ool007oo001oo`00Ool007oo00000Woo00/007oo001oo`00Ool007oo001oo`000=Qoo`008Woo00<0
07ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00@007oo001oo`H00003Ool007oo0080000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
00003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`03001oo`000>1oo`008Woo0P003goo00T007oo001oo`00Ool007oo00000Woo00<007oo
00000`0000Moo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`00
0Woo00d007oo001oo`00Ool007oo001oo`00Ool0003UOol0029oo`03001oogoo00eoo`0:001oo`00
Ool007oo001oo`00Ool200003Goo00000000Ool007oo001oo`00Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`000>]oo`008Woo00<007ooOol0
3Goo00X007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oo`000>moo`008Woo00<007oo
Ool03Goo00X007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool0
07oo0002Ool02`00Ool007oo001oo`00Ool007oo0000mWoo000ROol2000>Ool02P00Ool007oo001o
o`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`09001oo`00Ool0
07oo001oo`000?Qoo`008Woo00<007ooOol03Goo00X007oo001oo`00Ool007oo001oo`80000=Ool0
07oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo
00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000o7oo000ROol00`00Oomo
o`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
00003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`03001oogoo0?ioo`000Woo0P0017oo0P0017oo00<007ooOol00goo0`000goo0P001Woo
00<007ooOol03Goo00X007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo
001oo`00Ool0003oOol4Ool00005Ool007ooOol00003Ool20004Ool00`00Oomoo`04Ool01@00Oomo
ogoo00000Woo00<007ooOol00goo00<007ooOol03Goo00X007oo001oo`00Ool007oo001oo`80000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`00
0Woo00T007oo001oo`00Ool007oo0000ogoo27oo00001Goo001oogoo00002Woo00@007ooOomoo`@0
009oo`04001oogoo0005Ool3000=Ool02P00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo
001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool01`00Ool0
07oo001oo`000?moo`Yoo`0000Eoo`00Oomoo`0000Yoo`07001oogooOol007oo00000goo0`001Woo
00<007ooOol03Goo00X007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00D007oo001oo`000?moo`ao
o`0000Eoo`00Oomoo`0000]oo`06001oogoo001oo`000goo00<007ooOol01Woo00<007ooOol03Goo
00X007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000io
o`00Ool007oo001oo`00Ool007oo001oo`000Woo00<007oo0000ogoo3Woo0002Ool20009Ool40003
Ool20004Ool30005Ool00`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000ioo`00Ool0
07oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00
Oomoo`3oOol>Ool0029oo`8000ioo`0:001oo`00Ool007oo001oo`00Ool200003Woo001oo`00Ool0
07oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?mooa=oo`008Woo
00<007ooOol03Goo00X007oo001oo`00Ool007oo001oo`80000>Ool007oo001oo`00Ool007oo001o
o`00Ool0009oo`0;001oo`00Ool007oo001oo`00Ool0003oOolEOol0029oo`03001oogoo00eoo`0:
001oo`00Ool007oo001oo`00Ool200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool02@00
Ool007oo001oo`00Ool0003oOolGOol0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00
Ool200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool02@00Ool007oo001oo`00Ool0003o
OolGOol0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool200003Woo001oo`00Ool0
07oo001oo`00Ool007oo0002Ool01`00Ool007oo001oo`000?mooaUoo`008Woo0P003Woo00X007oo
001oo`00Ool007oo001oo`80000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`05001oo`00
Ool0003oOolKOol0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool200003Woo001o
o`00Ool007oo001oo`00Ool007oo0002Ool01@00Ool007oo0000ogoo6goo000ROol00`00Oomoo`0=
Ool02P00Ool007oo001oo`00Ool007oo0P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo
00<007oo0000ogoo7Goo000ROol00`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000io
o`00Ool007oo001oo`00Ool007oo001oo`000Woo00<007ooOol0ogoo7Goo000ROol2000>Ool02P00
Ool007oo001oo`00Ool007oo0P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00<007oo
Ool0ogoo7Goo000ROol00`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000ioo`00Ool0
07oo001oo`00Ool007oo001oo`00ogoo8Woo0002Ool20004Ool20004Ool00`00Oomoo`03Ool30003
Ool20006Ool00`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000ioo`00Ool007oo001o
o`00Ool007oo001oo`00ogoo8Woo00001Goo001oogoo00000goo0P0017oo00<007ooOol017oo00D0
07ooOomoo`00009oo`03001oogoo00=oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool2
000037oo001oo`00Ool007oo001oo`00Ool00?moobAoo`0000Eoo`00Oomoo`0000Yoo`04001oogoo
Ool40002Ool01000Oomoo`001Goo0`003Goo00X007oo001oo`00Ool007oo001oo`80000<Ool007oo
001oo`00Ool007oo001oo`00ogoo97oo00001Goo001oogoo00002Woo00L007ooOomoo`00Ool00004
Ool20006Ool00`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000Yoo`00Ool007oo001o
o`00Ool00?moobIoo`0000Eoo`00Oomoo`0000]oo`06001oogoo001oo`000goo00@007ooOol000Eo
o`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool200002Woo001oo`00Ool007oo001oo`00
ogoo9Woo0002Ool20009Ool40003Ool20004Ool20006Ool00`00Oomoo`0=Ool02P00Ool007oo001o
o`00Ool007oo0P0000Yoo`00Ool007oo001oo`00Ool00?moobIoo`008Woo00<007ooOol03Goo00X0
07oo001oo`00Ool007oo001oo`800008Ool007oo001oo`00Ool00?moobQoo`008Woo0P003Woo00X0
07oo001oo`00Ool007oo001oo`800008Ool007oo001oo`00Ool00?moobQoo`008Woo00<007ooOol0
3Goo00X007oo001oo`00Ool007oo001oo`800008Ool007oo001oo`00Ool00?moobQoo`008Woo00<0
07ooOol03Goo00X007oo001oo`00Ool007oo001oo`800006Ool007oo001oo`00ogoo:Woo000ROol0
0`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000Ioo`00Ool007oo003oOolZOol0029o
o`8000ioo`0:001oo`00Ool007oo001oo`00Ool200001Woo001oo`00Ool00?moobYoo`008Woo00<0
07ooOol03Goo00X007oo001oo`00Ool007oo001oo`800004Ool007oo003oOol/Ool0029oo`03001o
ogoo00eoo`0:001oo`00Ool007oo001oo`00Ool2000017oo001oo`00ogoo;7oo000ROol00`00Oomo
o`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000Aoo`00Ool00?moobaoo`008Woo00<007ooOol0
3Goo00X007oo001oo`00Ool007oo001oo`800004Ool007oo003oOol/Ool0029oo`8000ioo`0:001o
o`00Ool007oo001oo`00Ool200000goo001oo`3oOol]Ool0029oo`03001oogoo00eoo`0:001oo`00
Ool007oo001oo`00Ool200000goo001oo`3oOol]Ool000Qoo`8000Aoo`8000Aoo`03001oogoo009o
o`<000Ioo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool200000goo001oo`3oOol]Ool0
00Moo`04001oogoo0003Ool20004Ool00`00Oomoo`05Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool0
2P00Ool007oo001oo`00Ool007oo0P0000=oo`00Ool0ogoo;Goo0007Ool01000Oomoo`002Woo00<0
07ooOol017oo00<007ooOol00goo0`003Goo00X007oo001oo`00Ool007oo001oo`800?mooc1oo`00
1goo00@007ooOol000Yoo`04001oogooOol3003oOomBOol000Moo`04001oogoo000;Ool01000Oomo
o`00ogooE7oo0008Ool20009Ool40002Ool4003oOomAOol00?moofioo`00\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 0.738778, 
  0.933765, 0.00011328}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[explotP, OTSPlotPApp, \ \ PlotRange\  -> {0.74, \ 0.75}, \ 
      DisplayFunction\  -> \ $DisplayFunction]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0031746 -45.7345 61.8034 [
[.18254 -0.0125 -6 -9 ]
[.18254 -0.0125 6 0 ]
[.34127 -0.0125 -9 -9 ]
[.34127 -0.0125 9 0 ]
[.5 -0.0125 -9 -9 ]
[.5 -0.0125 9 0 ]
[.65873 -0.0125 -9 -9 ]
[.65873 -0.0125 9 0 ]
[.81746 -0.0125 -9 -9 ]
[.81746 -0.0125 9 0 ]
[.97619 -0.0125 -9 -9 ]
[.97619 -0.0125 9 0 ]
[.01131 .12361 -30 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -30 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -30 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -24 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 0 m
.18254 .00625 L
s
[(50)] .18254 -0.0125 0 1 Mshowa
.34127 0 m
.34127 .00625 L
s
[(100)] .34127 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(150)] .5 -0.0125 0 1 Mshowa
.65873 0 m
.65873 .00625 L
s
[(200)] .65873 -0.0125 0 1 Mshowa
.81746 0 m
.81746 .00625 L
s
[(250)] .81746 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(300)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.15079 0 m
.15079 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.27778 0 m
.27778 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.53175 0 m
.53175 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.9127 0 m
.9127 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(0.742)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.744)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.746)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.748)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(0.75)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.07143 .20223 m
.0746 .20223 L
.07778 .20223 L
.08095 .20223 L
.08413 .20223 L
.0873 .20223 L
.09048 .20223 L
.09365 .20223 L
.09683 .20223 L
.1 .20223 L
.10317 .20223 L
.10635 .20223 L
.10952 .20223 L
.1127 .20223 L
.11587 .20223 L
.11905 .20223 L
.12222 .20223 L
.1254 .20223 L
.12857 .20223 L
.13175 .20223 L
.13492 .20223 L
.1381 .20223 L
.14127 .20223 L
.14444 .20223 L
.14762 .20223 L
.15079 .20223 L
.15397 .20223 L
.15714 .20223 L
.16032 .20223 L
.16349 .20223 L
.16667 .20223 L
.16984 .20223 L
.17302 .20223 L
.17619 .20223 L
.17937 .20223 L
.18254 .20223 L
.18571 .20223 L
.18889 .20223 L
.19206 .20223 L
.19524 .20223 L
.19841 .20223 L
.20159 .20223 L
.20476 .20223 L
.20794 .20223 L
.21111 .20223 L
.21429 .20223 L
.21746 .20223 L
.22063 .20223 L
.22381 .20223 L
.22698 .20223 L
Mistroke
.23016 .20223 L
.23333 .20223 L
.23651 .20223 L
.23968 .20223 L
.24286 .20223 L
.24603 .20223 L
.24921 .20223 L
.25238 .20223 L
.25556 .20223 L
.25873 .20223 L
.2619 .20223 L
.26508 .20223 L
.26825 .20223 L
.27143 .20223 L
.2746 .20223 L
.27778 .20223 L
.28095 .20223 L
.28413 .20223 L
.2873 .20223 L
.29048 .20223 L
.29365 .20223 L
.29683 .20223 L
.3 .20223 L
.30317 .20223 L
.30635 .20223 L
.30952 .20223 L
.3127 .20223 L
.31587 .20223 L
.31905 .20223 L
.32222 .20223 L
.3254 .20223 L
.32857 .20223 L
.33175 .20223 L
.33492 .20223 L
.3381 .20223 L
.34127 .20223 L
.34444 .20223 L
.34762 .20223 L
.35079 .20223 L
.35397 .20223 L
.35714 .20223 L
.36032 .20223 L
.36349 .20223 L
.36667 .20223 L
.36984 .20223 L
.37302 .20223 L
.37619 .20223 L
.37937 .20223 L
.38254 .20223 L
.38571 .20223 L
Mistroke
.38889 .20223 L
.39206 .20223 L
.39524 .20223 L
.39841 .20223 L
.40159 .20223 L
.40476 .20223 L
.40794 .20223 L
.41111 .20223 L
.41429 .20223 L
.41746 .20223 L
.42063 .20223 L
.42381 .20223 L
.42698 .20223 L
.43016 .20223 L
.43333 .20223 L
.43651 .20223 L
.43968 .20223 L
.44286 .20223 L
.44603 .20223 L
.44921 .20223 L
.45238 .20223 L
.45556 .20223 L
.45873 .20223 L
.4619 .20223 L
.46508 .20223 L
.46825 .20223 L
.47143 .20223 L
.4746 .20223 L
.47778 .20223 L
.48095 .20223 L
.48413 .20223 L
.4873 .20223 L
.49048 .20223 L
.49365 .20223 L
.49683 .20223 L
.5 .20223 L
.50317 .20223 L
.50635 .20223 L
.50952 .20223 L
.5127 .20223 L
.51587 .20223 L
.51905 .20223 L
.52222 .20223 L
.5254 .20223 L
.52857 .20223 L
.53175 .20223 L
.53492 .20223 L
.5381 .20223 L
.54127 .20223 L
.54444 .20223 L
Mistroke
.54762 .20223 L
.55079 .20223 L
.55397 .20223 L
.55714 .20223 L
.56032 .20223 L
.56349 .20223 L
.56667 .20223 L
.56984 .20223 L
.57302 .20223 L
.57619 .20223 L
.57937 .20223 L
.58254 .20223 L
.58571 .20223 L
.58889 .20223 L
.59206 .20223 L
.59524 .20223 L
.59841 .20223 L
.60159 .20223 L
.60476 .20223 L
.60794 .20223 L
.61111 .20223 L
.61429 .20223 L
.61746 .20223 L
.62063 .20223 L
.62381 .20223 L
.62698 .20223 L
.63016 .20223 L
.63333 .20223 L
.63651 .20223 L
.63968 .20223 L
.64286 .20223 L
.64603 .20223 L
.64921 .20223 L
.65238 .20223 L
.65556 .20223 L
.65873 .20223 L
.6619 .20223 L
.66508 .20223 L
.66825 .20223 L
.67143 .20223 L
.6746 .20223 L
.67778 .20223 L
.68095 .20223 L
.68413 .20223 L
.6873 .20223 L
.69048 .20223 L
.69365 .20223 L
.69683 .20223 L
.7 .20223 L
.70317 .20223 L
Mistroke
.70635 .20223 L
.70952 .20223 L
.7127 .20223 L
.71587 .20223 L
.71905 .20223 L
.72222 .20223 L
.7254 .20223 L
.72857 .20223 L
.73175 .20223 L
.73492 .20223 L
.7381 .20223 L
.74127 .20223 L
.74444 .20223 L
.74762 .20223 L
.75079 .20223 L
.75397 .20223 L
.75714 .20223 L
.76032 .20223 L
.76349 .20223 L
.76667 .20223 L
.76984 .20223 L
.77302 .20223 L
.77619 .20223 L
.77937 .20223 L
.78254 .20223 L
.78571 .20223 L
.78889 .20223 L
.79206 .20223 L
.79524 .20223 L
.79841 .20223 L
.80159 .20223 L
.80476 .20223 L
.80794 .20223 L
.81111 .20223 L
.81429 .20223 L
.81746 .20223 L
.82063 .20223 L
.82381 .20223 L
.82698 .20223 L
.83016 .20223 L
.83333 .20223 L
.83651 .20223 L
.83968 .20223 L
.84286 .20223 L
.84603 .20223 L
.84921 .20223 L
.85238 .20223 L
.85556 .20223 L
.85873 .20223 L
.8619 .20223 L
Mistroke
.86508 .20223 L
.86825 .20223 L
.87143 .20223 L
.8746 .20223 L
.87778 .20223 L
.88095 .20223 L
.88413 .20223 L
.8873 .20223 L
.89048 .20223 L
.89365 .20223 L
.89683 .20223 L
.9 .20223 L
.90317 .20223 L
.90635 .20223 L
.90952 .20223 L
.9127 .20223 L
.91587 .20223 L
.91905 .20223 L
.92222 .20223 L
.9254 .20223 L
.92857 .20223 L
.93175 .20223 L
.93492 .20223 L
.9381 .20223 L
.94127 .20223 L
.94444 .20223 L
.94762 .20223 L
.95079 .20223 L
.95397 .20223 L
.95714 .20223 L
.96032 .20223 L
.96349 .20223 L
.96667 .20223 L
.96984 .20223 L
.97302 .20223 L
.97619 .20223 L
Mfstroke
.07229 .61803 m
.07402 0 L
s
.07524 0 m
.07711 .61803 L
s
.07849 .61803 m
.08047 0 L
s
.08147 0 m
.0836 .61803 L
s
.08468 .61803 m
.08693 0 L
s
.0877 0 m
.0901 .61803 L
s
.09087 .61803 m
.09338 0 L
s
.09393 0 m
.09661 .61803 L
s
.09705 .61803 m
.09985 0 L
s
.10016 0 m
.10312 .61803 L
s
.10323 .61803 m
.10631 0 L
s
.10639 0 m
.10952 .59673 L
.1127 .0138 L
.11587 .56832 L
.11905 .03259 L
.12222 .54358 L
.1254 .04882 L
.12857 .52186 L
.13175 .06295 L
.13492 .50262 L
.1381 .07535 L
.14127 .48547 L
.14444 .0863 L
.14762 .47008 L
.15079 .09603 L
.15397 .4562 L
.15714 .10473 L
.16032 .44362 L
.16349 .11254 L
.16667 .43215 L
.16984 .11959 L
.17302 .42167 L
.17619 .12596 L
.17937 .41204 L
.18254 .13176 L
.18571 .40316 L
.18889 .13705 L
.19206 .39496 L
.19524 .14189 L
.19841 .38736 L
.20159 .14633 L
.20476 .38029 L
.20794 .15042 L
.21111 .3737 L
.21429 .15419 L
.21746 .36755 L
.22063 .15767 L
.22381 .36178 L
.22698 .1609 L
.23016 .35638 L
.23333 .1639 L
.23651 .35129 L
.23968 .16669 L
.24286 .3465 L
.24603 .16929 L
.24921 .34198 L
.25238 .17172 L
.25556 .33771 L
.25873 .17399 L
.2619 .33367 L
Mistroke
.26508 .17612 L
.26825 .32983 L
.27143 .17811 L
.2746 .32619 L
.27778 .17999 L
.28095 .32274 L
.28413 .18175 L
.2873 .31944 L
.29048 .18341 L
.29365 .31631 L
.29683 .18497 L
.3 .31332 L
.30317 .18645 L
.30635 .31046 L
.30952 .18784 L
.3127 .30773 L
.31587 .18916 L
.31905 .30511 L
.32222 .19041 L
.3254 .30261 L
.32857 .19159 L
.33175 .30021 L
.33492 .19272 L
.3381 .2979 L
.34127 .19378 L
.34444 .29569 L
.34762 .19479 L
.35079 .29357 L
.35397 .19575 L
.35714 .29152 L
.36032 .19667 L
.36349 .28955 L
.36667 .19754 L
.36984 .28766 L
.37302 .19837 L
.37619 .28583 L
.37937 .19916 L
.38254 .28406 L
.38571 .19992 L
.38889 .28236 L
.39206 .20064 L
.39524 .28072 L
.39841 .20132 L
.40159 .27913 L
.40476 .20198 L
.40794 .2776 L
.41111 .20261 L
.41429 .27611 L
.41746 .20321 L
.42063 .27467 L
Mistroke
.42381 .20378 L
.42698 .27328 L
.43016 .20433 L
.43333 .27193 L
.43651 .20486 L
.43968 .27062 L
.44286 .20536 L
.44603 .26935 L
.44921 .20585 L
.45238 .26812 L
.45556 .20631 L
.45873 .26693 L
.4619 .20676 L
.46508 .26577 L
.46825 .20719 L
.47143 .26464 L
.4746 .2076 L
.47778 .26354 L
.48095 .20799 L
.48413 .26248 L
.4873 .20837 L
.49048 .26144 L
.49365 .20873 L
.49683 .26043 L
.5 .20908 L
.50317 .25945 L
.50635 .20942 L
.50952 .25849 L
.5127 .20974 L
.51587 .25756 L
.51905 .21005 L
.52222 .25665 L
.5254 .21035 L
.52857 .25577 L
.53175 .21064 L
.53492 .2549 L
.5381 .21092 L
.54127 .25406 L
.54444 .21118 L
.54762 .25324 L
.55079 .21144 L
.55397 .25244 L
.55714 .21169 L
.56032 .25165 L
.56349 .21193 L
.56667 .25089 L
.56984 .21216 L
.57302 .25014 L
.57619 .21238 L
.57937 .24941 L
Mistroke
.58254 .21259 L
.58571 .2487 L
.58889 .2128 L
.59206 .248 L
.59524 .213 L
.59841 .24732 L
.60159 .21319 L
.60476 .24665 L
.60794 .21338 L
.61111 .246 L
.61429 .21355 L
.61746 .24536 L
.62063 .21373 L
.62381 .24474 L
.62698 .21389 L
.63016 .24412 L
.63333 .21405 L
.63651 .24353 L
.63968 .21421 L
.64286 .24294 L
.64603 .21436 L
.64921 .24236 L
.65238 .2145 L
.65556 .2418 L
.65873 .21464 L
.6619 .24125 L
.66508 .21477 L
.66825 .24071 L
.67143 .2149 L
.6746 .24017 L
.67778 .21503 L
.68095 .23965 L
.68413 .21515 L
.6873 .23914 L
.69048 .21527 L
.69365 .23864 L
.69683 .21538 L
.7 .23815 L
.70317 .21549 L
.70635 .23767 L
.70952 .21559 L
.7127 .2372 L
.71587 .21569 L
.71905 .23673 L
.72222 .21579 L
.7254 .23627 L
.72857 .21589 L
.73175 .23583 L
.73492 .21598 L
.7381 .23539 L
Mistroke
.74127 .21606 L
.74444 .23495 L
.74762 .21615 L
.75079 .23453 L
.75397 .21623 L
.75714 .23411 L
.76032 .21631 L
.76349 .2337 L
.76667 .21639 L
.76984 .2333 L
.77302 .21646 L
.77619 .2329 L
.77937 .21653 L
.78254 .23251 L
.78571 .2166 L
.78889 .23213 L
.79206 .21666 L
.79524 .23175 L
.79841 .21673 L
.80159 .23138 L
.80476 .21679 L
.80794 .23101 L
.81111 .21685 L
.81429 .23065 L
.81746 .2169 L
.82063 .2303 L
.82381 .21696 L
.82698 .22995 L
.83016 .21701 L
.83333 .22961 L
.83651 .21706 L
.83968 .22927 L
.84286 .21711 L
.84603 .22894 L
.84921 .21716 L
.85238 .22862 L
.85556 .2172 L
.85873 .22829 L
.8619 .21725 L
.86508 .22798 L
.86825 .21729 L
.87143 .22766 L
.8746 .21733 L
.87778 .22736 L
.88095 .21737 L
.88413 .22705 L
.8873 .2174 L
.89048 .22676 L
.89365 .21744 L
.89683 .22646 L
Mistroke
.9 .21747 L
.90317 .22617 L
.90635 .21751 L
.90952 .22589 L
.9127 .21754 L
.91587 .2256 L
.91905 .21757 L
.92222 .22533 L
.9254 .21759 L
.92857 .22505 L
.93175 .21762 L
.93492 .22478 L
.9381 .21765 L
.94127 .22452 L
.94444 .21767 L
.94762 .22425 L
.95079 .2177 L
.95397 .22399 L
.95714 .21772 L
.96032 .22374 L
.96349 .21774 L
.96667 .22349 L
.96984 .21776 L
.97302 .22324 L
.97619 .21778 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->False,
  ImageSize->{365.5, 104.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005]0000J1000`40O003h00OogooKWoo003oOom^
Ool0059oo`<000Aoo`8002Yoo`<000Aoo`8000Aoo`8002Ioo`<000=oo`<000Aoo`8002Moo`@000=o
o`8000Aoo`8002Ioo`@0009oo`<000Aoo`8002Moo`<000Aoo`8000Aoo`8000=oo`00EGoo00@007oo
Ool0009oo`03001oogoo02Qoo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`009Woo00<0
07ooOol01Goo00@007ooOol0009oo`03001oogoo02Eoo`03001oogoo009oo`04001oogoo0002Ool0
1000Oomoo`009Woo00<007ooOol01Goo00@007ooOol0009oo`03001oogoo02Moo`04001oogoo0002
Ool01000Oomoo`000Woo0@000Goo0Goo001EOol01000Oomoo`000Woo00<007ooOol0:7oo00<007oo
Ool00Woo00@007ooOol0009oo`04001oogoo000VOol00`00Oomoo`05Ool01000Oomoo`000Woo00<0
07ooOol09Woo00D007ooOomoo`00009oo`04001oogoo0002Ool00`00Oomoo`0UOol00`00Oomoo`04
Ool01000Oomoo`000Woo00<007ooOol09goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1
Ool0059oo`<000=oo`04001oogoo000ZOol00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol0
02Ioo`03001oogoo009oo`<000=oo`04001oogoo000YOol01000Oomoo`000Woo00@007ooOol0009o
o`03001oogoo02Ioo`03001oogoo00<000=oo`04001oogoo000WOol20003Ool01000Oomoo`000Woo
00@007ooOol0009oo`00DWoo00<007ooOol00goo00@007ooOol002Yoo`03001oogoo009oo`04001o
ogoo0002Ool01000Oomoo`009Woo00<007ooOol00Woo00<007ooOol00goo00@007ooOol002Ioo`04
001oogoo0002Ool01000Oomoo`000Woo00@007ooOol002Eoo`04001oogoo0002Ool00`00Oomoo`03
Ool01000Oomoo`00:Goo00@007ooOol0009oo`04001oogoo0002Ool10001Ool1Ool0059oo`@000=o
o`8002Yoo`8000Eoo`8000Aoo`8002Ioo`8000Aoo`@000=oo`8002Qoo`8000Aoo`8000Aoo`8002Mo
o`8000=oo`@000=oo`8002Moo`<000Aoo`8000Aoo`8000=oo`00ogooKWoo003oOom^Ool00?moofio
o`006Wooo`00D`000Goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool00`00
Oomoo`0FOol00`00Oomoo`0cOol00`00Oomoo`0bOol00`00Oomoo`0cOol00`00Oomoo`0bOol00`00
Oomoo`0cOol00`00Oomoo`07Ool0029oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009o
o`03001oo`000?moobeoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00<0
07oo0000ogoo;Goo000ROol2000?Ool02@00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000
ogoo:goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000
ogoo:goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000
ogoo:goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001o
o`000?moobUoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00L007oo001o
o`00Ool0003oOolYOol0029oo`8000moo`09001oo`00Ool007oo001oo`00009oo`09001oo`00Ool0
07oo001oo`000?moobMoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00/0
07oo001oo`00Ool007oo001oo`000?moobEoo`008Woo00<007ooOol03Woo00T007oo001oo`00Ool0
07oo00000Woo00/007oo001oo`00Ool007oo001oo`000?moobEoo`008Woo00<007ooOol03Woo00T0
07oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0003oOolSOol0029o
o`8000moo`09001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00<007ooOol0ogoo7Woo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool0
3@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001oogoo0?mooaioo`000Woo0P0017oo0P00
17oo00<007ooOol00goo0`000Woo10001Goo00<007ooOol03Woo00T007oo001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Ool0003oOolNOol00005Ool007oo
Ool00003Ool20004Ool00`00Oomoo`04Ool00`00Oomoo`02Ool00`00Oomoo`05Ool00`00Oomoo`0>
Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`05
001oo`00Ool0003oOolLOol00005Ool007ooOol0000:Ool01000Oomoogoo100017oo00<007ooOol0
17oo0`003Woo00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool01`00Ool007oo001oo`000?mooaYoo`0000Eoo`00Oomoo`0000Yoo`07001oogooOol007oo
00001Woo00<007ooOol00goo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo001oo`00Ool0003oOolHOol00005Ool0
07ooOol0000;Ool01P00Oomoo`00Ool000=oo`04001oogoo0005Ool00`00Oomoo`0>Ool02@00Ool0
07oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool0
07oo001oo`00Ool007oo0000ogoo57oo0002Ool20009Ool40003Ool20004Ool20006Ool00`00Oomo
o`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0ogoo3goo000ROol00`00Oomo
o`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00D007oo001oo`000?moo`eoo`008Woo0P00
3goo00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool0
3@00Ool007oo001oo`00Ool007oo001oo`00009oo`09001oo`00Ool007oo001oo`000?moo`Uoo`00
8Woo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001o
o`00Ool007oo0000ogoo1Goo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool0
3@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000oGoo000ROol00`00
Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00
009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?Eoo`008Woo0P003goo00T007oo001o
o`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool01`00Ool007oo001oo`000>aoo`008Woo00<007ooOol03Woo
00T007oo001oo`00Ool007oo00000Woo00h007oo001oo`00Ool007oo001oo`00Ool007oo0P0000ao
o`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002
Ool01@00Ool007oo0000ggoo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool0
3P00Ool007oo001oo`00Ool007oo001oo`00Ool2000037oo001oo`00Ool007oo001oo`00Ool0009o
o`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool0
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool0
07oo00000Woo00T007oo001oo`00Ool007oo0000c7oo000ROol00`00Oomoo`0=Oooo000c0009Ool0
029oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`0>001oo`00Ool007oo001oo`00
Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000aoo`00Ool007oo001o
o`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo
001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo
001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool03`00Ool007oo001oo`00Ool007oo001oo`00Ool00002
Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo
00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool00`00Ool0001WOol0029oo`8000mo
o`09001oo`00Ool007oo001oo`00009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`80000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000
0Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool03`00Ool007oo001oo`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001o
o`00Ool007oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0=001o
o`00Ool007oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool0
3000Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`02000027oo001oo`00Ool007oo0009Ool0029oo`03
001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`0>001oo`00Ool007oo001oo`00Ool007oo
001oo`80000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool0
07oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool0
07oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool0
07oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000aoo`00Ool0
07oo001oo`00Ool007oo0002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001o
o`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eo
o`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080
000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001o
o`0200001Goo001oo`00Ool00P0000Ioo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo
001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0000:
Ool0009oo`8000Aoo`8000Aoo`03001oogoo00=oo`<000=oo`<000Eoo`03001oogoo00ioo`09001o
o`00Ool007oo001oo`00009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo
001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo
001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P00
00eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo00800005Ool007oo001oo`0200001goo
001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo
001oo`00Ool007oo001oo`02000037oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool0
07oo001oo`00Ool007oo00000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool0
07oo0000>Goo00001Goo001oogoo00000goo0P0017oo00<007ooOol017oo00<007ooOol00goo00<0
07ooOol017oo00<007ooOol03Woo00T007oo001oo`00Ool007oo00000Woo00h007oo001oo`00Ool0
07oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00
Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Eoo`00Ool007oo00800005Ool007oo001oo`02
00003Goo001oo`00Ool007oo001oo`00Ool007oo0080000?Ool007oo001oo`00Ool007oo001oo`00
Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000aoo`00Ool007oo001oo`00
Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`0;001oo`00Ool007oo001o
o`00Ool0001POol00005Ool007ooOol0000:Ool01000Oomoogoo10000Woo10001Goo0`003Woo00T0
07oo001oo`00Ool007oo00000Woo00h007oo001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00
Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=
Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
000017oo000007oo0P0000Ioo`00001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo
0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`00009oo`0=001oo`00Ool007oo001oo`00Ool007oo0000O7oo00001Goo001oogoo0000
2Woo00L007ooOomoo`00Ool00003Ool00`00Ool00006Ool00`00Oomoo`0>Ool02@00Ool007oo001o
o`00Ool00002Ool03P00Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000Uoo`00Ool007oo
001oo`00Ool00P00011oo`00001oo`00Ool007oo001oo`00Ool007oo001oo`80000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001o
o`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=
001oo`00Ool007oo001oo`00Ool007oo00000Woo00D007oo001oo`0009=oo`0000Eoo`00Oomoo`00
00]oo`06001oogoo001oo`000goo00<007oo00001Woo00<007ooOol03Woo00T007oo001oo`00Ool0
07oo00000Woo00h007oo001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00
Ool007oo001oo`0200001goo001oo`00Ool007oo00<0000AOol00000001oo`00Ool007oo001oo`00
Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00
Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001o
o`00Ool007oo001oo`00Ool00002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03001o
o`000:Aoo`000Woo0P002Goo10000goo0P0017oo0P001Woo00<007ooOol03Woo00T007oo001oo`00
Ool007oo00000Woo00h007oo001oo`00Ool007oo001oo`00Ool007oo0P0000]oo`00Ool007oo001o
o`00Ool007oo00<0000=Ool00000001oo`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001o
o`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo
001oo`00Ool007oo001oo`00Ool00002Ool00`00Ool0002cOol0029oo`8000moo`09001oo`00Ool0
07oo001oo`00009oo`0>001oo`00Ool007oo001oo`00Ool007oo001oo`800003Ool007oo00P0000@
Ool00000Ool007oo001oo`00Ool007oo001oo`00Ool200003Goo001oo`00Ool007oo001oo`00Ool0
07oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00Ool0
07oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool01@00Ool007oo0000`7oo
000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool02P00Ool007oo001oo`00Ool0
07oo1@0000eoo`00000007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00
Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo001oo`00
Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool007oo001o
o`00Ool0003;Ool0029oo`03001oogoo00ioo`09001oo`00Ool007oo001oo`00009oo`04001oo`00
Ool8000047oo000007oo001oo`00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00
Ool007oo001oo`0200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00
Ool007oo001oo`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007ooOol0
dGoo000ROol00`00Oomoo`0>Ool02@00Ool007oo001oo`00Ool00002Ool00`00Ool0000500001Goo
001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Goo001oo`00Ool007oo
001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001oo`00Ool0
07oo001oo`00Ool007oo00000Woo00L007oo001oo`00Ool0003LOol0029oo`03001oogoo00ioo`09
001oo`00Ool007oo001oo`00009oo`@00009Ool007oo001oo`00Ool007oo0080000=Ool007oo001o
o`00Ool007oo001oo`00Ool00P0000eoo`00Ool007oo001oo`00Ool007oo001oo`0200003Woo001o
o`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001oo`00009oo`03
001oogoo0>1oo`008Woo0P003goo00T007oo001oo`00Ool007oo00000Woo0P0000]oo`00Ool007oo
001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000ioo`00Ool007oo
001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00Ool00002Ool02@00Ool0
07oo001oo`00Ool0003YOol0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool20000
3Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool0
0P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001oo`00
Ool00002Ool00`00Ool0003_Ool0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool2
00003Goo001oo`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001o
o`00Ool0003dOol0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool200003Goo001o
o`00Ool007oo001oo`00Ool007oo0080000=Ool007oo001oo`00Ool007oo001oo`00Ool00P0000io
o`00Ool007oo001oo`00Ool007oo001oo`000Woo00T007oo001oo`00Ool007oo0000n7oo000ROol2
000>Ool02P00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo001oo`02
00003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool007oo001o
o`00009oo`05001oo`00Ool0003lOol0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00
Ool200003Goo001oo`00Ool007oo001oo`00Ool007oo0080000>Ool007oo001oo`00Ool007oo001o
o`00Ool0009oo`0=001oo`00Ool007oo001oo`00Ool007oo00000Woo00<007oo0000oWoo000ROol0
0`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool007oo001oo`00Ool007oo
001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool03@00Ool007oo001oo`00Ool0
07oo001oo`000?moo`Aoo`000Woo0P0017oo0P0017oo00<007ooOol00goo0`000goo0P001Woo00<0
07ooOol03Goo00X007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001oo`00
Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00/007oo001oo`00Ool007oo001o
o`000?moo`Ioo`0000Eoo`00Oomoo`0000=oo`8000Aoo`03001oogoo00Aoo`05001oogooOol00002
Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000eoo`00Ool0
07oo001oo`00Ool007oo001oo`0200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool01`00
Ool007oo001oo`000?moo`Yoo`0000Eoo`00Oomoo`0000Yoo`04001oogooOol40002Ool01000Oomo
o`001Goo0`003Goo00X007oo001oo`00Ool007oo001oo`80000=Ool007oo001oo`00Ool007oo001o
o`00Ool00P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00D007oo001oo`000?moo`ao
o`0000Eoo`00Oomoo`0000Yoo`07001oogooOol007oo00000goo0`001Woo00<007ooOol03Goo00X0
07oo001oo`00Ool007oo001oo`80000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0=001o
o`00Ool007oo001oo`00Ool007oo00000Woo00<007oo0000ogoo3Woo00001Goo001oogoo00002goo
00H007ooOol007oo0003Ool00`00Oomoo`06Ool00`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool0
07oo0P0000ioo`00Ool007oo001oo`00Ool007oo001oo`000Woo00d007oo001oo`00Ool007oo001o
o`00Ool00002Ool00`00Oomoo`3oOol>Ool0009oo`8000Uoo`@000=oo`8000Aoo`<000Eoo`03001o
ogoo00eoo`0:001oo`00Ool007oo001oo`00Ool200003Woo001oo`00Ool007oo001oo`00Ool007oo
0002Ool03@00Ool007oo001oo`00Ool007oo001oo`000?mooa=oo`008Woo0P003Woo00X007oo001o
o`00Ool007oo001oo`80000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`0;001oo`00Ool0
07oo001oo`00Ool0003oOolEOol0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool2
00003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool02@00Ool007oo001oo`00Ool0003oOolG
Ool0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool200003Woo001oo`00Ool007oo
001oo`00Ool007oo0002Ool02@00Ool007oo001oo`00Ool0003oOolGOol0029oo`03001oogoo00eo
o`0:001oo`00Ool007oo001oo`00Ool200003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool0
1`00Ool007oo001oo`000?mooaUoo`008Woo00<007ooOol03Goo00X007oo001oo`00Ool007oo001o
o`80000>Ool007oo001oo`00Ool007oo001oo`00Ool0009oo`05001oo`00Ool0003oOolKOol0029o
o`8000ioo`0:001oo`00Ool007oo001oo`00Ool200003Woo001oo`00Ool007oo001oo`00Ool007oo
0002Ool00`00Ool0003oOolMOol0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool2
00003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00Ool0003oOolMOol0029oo`03001o
ogoo00eoo`0:001oo`00Ool007oo001oo`00Ool200003Woo001oo`00Ool007oo001oo`00Ool007oo
0002Ool00`00Oomoo`3oOolMOol0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool2
00003Woo001oo`00Ool007oo001oo`00Ool007oo0002Ool00`00Oomoo`3oOolMOol0029oo`8000io
o`0:001oo`00Ool007oo001oo`00Ool200003Woo001oo`00Ool007oo001oo`00Ool007oo003oOolR
Ool0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool200003Woo001oo`00Ool007oo
001oo`00Ool007oo003oOolROol0009oo`8000Aoo`8000Aoo`03001oogoo00=oo`<000=oo`8000Io
o`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool2000037oo001oo`00Ool007oo001oo`00
Ool00?moobAoo`0000Eoo`00Oomoo`0000=oo`8000Aoo`03001oogoo00Aoo`05001oogooOol00002
Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000aoo`00Ool0
07oo001oo`00Ool007oo003oOolTOol00005Ool007ooOol0000:Ool01000Oomoogoo10000Woo00@0
07ooOol000Eoo`<000eoo`0:001oo`00Ool007oo001oo`00Ool200002Woo001oo`00Ool007oo001o
o`00ogoo9Woo00001Goo001oogoo00002Woo00L007ooOomoo`00Ool00004Ool20006Ool00`00Oomo
o`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000Yoo`00Ool007oo001oo`00Ool00?moobIoo`00
00Eoo`00Oomoo`0000]oo`06001oogoo001oo`000goo00@007ooOol000Eoo`03001oogoo00eoo`0:
001oo`00Ool007oo001oo`00Ool200002Woo001oo`00Ool007oo001oo`00ogoo9Woo0002Ool20009
Ool40003Ool20004Ool20006Ool00`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000Qo
o`00Ool007oo001oo`00ogoo:7oo000ROol00`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool007oo
0P0000Qoo`00Ool007oo001oo`00ogoo:7oo000ROol2000>Ool02P00Ool007oo001oo`00Ool007oo
0P0000Ioo`00Ool007oo003oOolZOol0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00
Ool200001Woo001oo`00Ool00?moobYoo`008Woo00<007ooOol03Goo00X007oo001oo`00Ool007oo
001oo`800006Ool007oo001oo`00ogoo:Woo000ROol00`00Oomoo`0=Ool02P00Ool007oo001oo`00
Ool007oo0P0000Aoo`00Ool00?moobaoo`008Woo0P003Woo00X007oo001oo`00Ool007oo001oo`80
0004Ool007oo003oOol/Ool0029oo`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool20000
17oo001oo`00ogoo;7oo000ROol00`00Oomoo`0=Ool02P00Ool007oo001oo`00Ool007oo0P0000Ao
o`00Ool00?moobaoo`008Woo00<007ooOol03Goo00X007oo001oo`00Ool007oo001oo`800003Ool0
07oo0?moobeoo`008Woo00<007ooOol03Goo00X007oo001oo`00Ool007oo001oo`800003Ool007oo
0?moobeoo`008Woo0P003Woo00X007oo001oo`00Ool007oo001oo`800003Ool007oo0?moobeoo`00
8Woo00<007ooOol03Goo00X007oo001oo`00Ool007oo001oo`800003Ool007oo0?moobeoo`0027oo
0P0017oo0P0017oo00<007ooOol00Woo0`001Woo00<007ooOol03Goo00X007oo001oo`00Ool007oo
001oo`800?mooc1oo`001goo00@007ooOol000=oo`8000Aoo`03001oogoo00Eoo`03001oogoo00=o
o`03001oogoo00eoo`0:001oo`00Ool007oo001oo`00Ool2003oOol`Ool000Moo`04001oogoo000:
Ool00`00Oomoo`04Ool00`00Oomoo`03Ool3000=Ool02P00Ool007oo001oo`00Ool007oo0P00ogoo
<7oo0007Ool01000Oomoo`002Woo00@007ooOomoo`<00?mooe9oo`001goo00@007ooOol000]oo`04
001oogoo003oOomDOol000Qoo`8000Uoo`@0009oo`@00?mooe5oo`00ogooKWoo0000\
\>"],
  ImageRangeCache->{{{0, 364.5}, {103.562, 0}} -> {-31.9518, 0.738778, 
  0.933765, 0.00011328}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Magic Tree Sizes 3 - Approximate Supersymmetric Models", "Subsection",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
A simple formula for the strike-straddling can be given in the \
approximate form of this model. It is\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`n = \(\((m + 1\/2)\)\^2\ \[Sigma]\^2\ \
T\)\/\((log(K\/S) - \((r - q)\))\)\^2\)], "NumberedEquation",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "A ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " function for this case is:"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(straddleSS[m_, \ \[Sigma]_, \ T_, \ S_, \ K_, \ r_, \ 
        q_]\  := \ \((\((m\  + \ 1/2)\)^2*\[Sigma]^2*
            T)\)/\((Log[K/S]\  - \ \((r - q)\))\)^2\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Let's evaluate these values (in real form - you would take the \
nearest integer in practice) for the parameters used above:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[straddleSS[m, 0.2, 1, 9, 10, \ 0.1, \ 0], \ {m, \ 1, \ 10}]\)], 
  "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({3132.05494094334735`, 8700.15261373151993`, 17052.2991229137765`, 
      28188.494468490127`, 42108.7386504605554`, 58813.0316688250776`, 
      78301.3735235836794`, 100573.76421473636`, 125630.203742283153`, 
      153470.692106224015`}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The denominator here is very small as the forward price is very close to \
the strike. \n\nIt should be appreciated that the character of the \
oscillations for these three (approximate) models depends very much on the \
closeness of:\n\n(a) (CRR) The price to the strike: the denominator is ",
  Cell[BoxData[
      \(TraditionalForm\`\((log(K\/S))\)\^2\)]],
  ".\n\n(b) (JR) The expected price at maturity to the strike: the \
denominator is ",
  Cell[BoxData[
      \(TraditionalForm\`\((log(K\/S) - \((r - q - \
\[Sigma]\^2\/2)\))\)\^2\)]],
  ".\n\n(c) (SS) The forward price to the strike: the denominator is ",
  Cell[BoxData[
      \(TraditionalForm\`\((log(K\/S) - \((r - q)\))\)\^2\)]],
  ".\n\nThe large-scale oscillation frequency is determined by the inverse \
squares of these quantities. It happens that in the particular numerical \
example given, the SS frequency is larger than the JR frequency, which is in \
turn larger than the CRR frequency. In other cases this situation can be \
permuted around. We can see that there is no one model that can be relied on \
in all situations. This is a fundamental problem with binomial trees. Both \
the supersymmetric model and the approximate JR models avoid the problem of \
negative weights or abscissas, but depending on the problem parameters, may \
or may not offer improved convergence characteristics. ",
  StyleBox[
  "One can of course pick the model with the largest denominator in the magic \
tree size formula, and work along the magic tree sizes that result until \
convergence is obtained - this is not a bad practical rule",
    FontSlant->"Italic"],
  ". The averaging of odd and even ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " results - a technique in common use - smooths out the high-frequency \
oscillations, but cannot in general be relied on to give more accurate \
results."
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakWithin->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["19.5 Brief Investigation of the American Variants", "Section",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True],

Cell[TextData[{
  "To investigate the recursive formulae for American options, we need a good \
benchmark value. In Sections 16.7 and 16.8 we gave some data for American Put \
options with one year to expiry where we had close agreement between the \
integral equation approximation and the implicit finite-difference method. We \
shall use these as a benchmark. The strike was 10 units, the volatility 20%, \
the risk-free rate 5%, with zero yield and one year to expiry. The results \
from either method are summarized, for three values (in, at, out, the money):\
\n\n",
  Cell[BoxData[
      \(TraditionalForm\`S = 9\)]],
  ", value ",
  Cell[BoxData[
      \(TraditionalForm\`1.149\)]],
  ", Delta ",
  Cell[BoxData[
      \(TraditionalForm\`\(-0.683\)\)]],
  ", Gamma ",
  Cell[BoxData[
      \(TraditionalForm\`0.312\)]],
  ", Theta ",
  Cell[BoxData[
      \(TraditionalForm\`\(-0.141\)\)]],
  ";\n",
  Cell[BoxData[
      \(TraditionalForm\`S = 10\)]],
  ", value ",
  Cell[BoxData[
      \(TraditionalForm\`0.609\)]],
  ", Delta ",
  Cell[BoxData[
      \(TraditionalForm\`\(-0.411\)\)]],
  ", Gamma ",
  Cell[BoxData[
      \(TraditionalForm\`0.230\)]],
  ", Theta",
  Cell[BoxData[
      \(TraditionalForm\`\(\ \(-0.223\)\)\)]],
  ";\n",
  Cell[BoxData[
      \(TraditionalForm\`S = 11\)]],
  ", value ",
  Cell[BoxData[
      \(TraditionalForm\`0.299\)]],
  ", Delta, ",
  Cell[BoxData[
      \(TraditionalForm\`\(-0.224\)\)]],
  ", Gamma ",
  Cell[BoxData[
      \(TraditionalForm\`0.147\)]],
  ", Theta ",
  Cell[BoxData[
      \(TraditionalForm\`\(-0.217\)\)]],
  ".\n\nIn the following we output the results for value, Delta, Gamma, \
Theta, from each of our six binomial models."
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "CRR-exact results, ",
  Cell[BoxData[
      \(TraditionalForm\`n = 100\)]],
  ":"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatrixForm[
      Table[CRRAmericanPut[S, 10, 100, \ 0.05, \ 0, \ 0.20, \ 1]\ , \ {S, \ 
          9, \ 11, 1}]]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1.15024047692952824`", \(-0.683383824961148178`\), 
              "0.313254259048144145`", \(-0.142711634478498439`\)},
            {"0.608718954693363567`", \(-0.411635345088275439`\), 
              "0.231216626773835676`", \(-0.226477960827076163`\)},
            {"0.300102456994193556`", \(-0.223995251584633869`\), 
              "0.147282141887667545`", \(-0.218472162531929692`\)}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output",
  CellMargins->{{45, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[TextData[{
  "CRR-approximate results, ",
  Cell[BoxData[
      \(TraditionalForm\`n = 100\)]],
  ":"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatrixForm[
      Table[CRRAmericanPutApp[S, 10, 100, \ 0.05, \ 0, \ 0.20, \ 
          1]\ , \ {S, \ 9, \ 11, 1}]]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1.14986134030643705`", \(-0.683635902070812129`\), 
              "0.313591307170827571`", \(-0.142507038992223389`\)},
            {"0.608235440914251324`", \(-0.411635612591410371`\), 
              "0.231394543552620479`", \(-0.226260044053921971`\)},
            {"0.299688439226735159`", \(-0.223883242807114202`\), 
              "0.14733721548032892`", \(-0.218227082041605457`\)}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output",
  CellMargins->{{45, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[TextData[{
  "JR-exact results, ",
  Cell[BoxData[
      \(TraditionalForm\`n = 100\)]],
  ":"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatrixForm[
      Table[JRAmericanPut[S, 10, \ 100, \ 0.05, \ 0, \ 0.20, \ 1]\ , \ {S, \ 
          9, \ 11, 1}]]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1.14921899074750033`", \(-0.68344931805398259`\), 
              "0.313033759925326915`", \(-0.142832531528114259`\)},
            {"0.61017688551814464`", \(-0.410758719684808237`\), 
              "0.229927768098265472`", \(-0.224677763362352012`\)},
            {"0.298426418609314225`", \(-0.223286344346625398`\), 
              "0.14718211167925677`", \(-0.219024660479058574`\)}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output",
  CellMargins->{{45, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[TextData[{
  "JR-approximate results, ",
  Cell[BoxData[
      \(TraditionalForm\`n = 100\)]],
  ":"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatrixForm[
      Table[JRAmericanPutApp[S, 10, \ 100, \ 0.05, \ 0, \ 0.20, \ 
          1]\ , \ {S, \ 9, \ 11, 1}]]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1.14908711399662388`", \(-0.683539560274240898`\), 
              "0.313150654156319552`", \(-0.142759381286039488`\)},
            {"0.610003493268827234`", \(-0.410759718788354799`\), 
              "0.229991509902299728`", \(-0.224604205931594469`\)},
            {"0.298283618231177669`", \(-0.223247462618328373`\), 
              "0.147200963008027629`", \(-0.218935733189432024`\)}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output",
  CellMargins->{{45, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False]
}, Open  ]],

Cell[TextData[{
  "SS-exact results, ",
  Cell[BoxData[
      \(TraditionalForm\`n = 100\)]],
  ":"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatrixForm[
      Table[SSAmericanPut[S, 10, \ 100, \ 0.05, \ 0, \ 0.20, \ 1]\ , \ {S, \ 
          9, \ 11, 1}]]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1.15006399344295884`", \(-0.681894264987348996`\), 
              "0.311989092982432136`", \(-0.14209895705014075`\)},
            {"0.609478230382533858`", \(-0.410378611455997166`\), 
              "0.229748402551063479`", \(-0.22481984598466953`\)},
            {"0.299629277965335827`", \(-0.223230344588837992`\), 
              "0.146608623762492351`", \(-0.217818349033244151`\)}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output",
  CellMargins->{{45, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False]
}, Open  ]],

Cell[TextData[{
  "SS-approximate results, ",
  Cell[BoxData[
      \(TraditionalForm\`n = 100\)]],
  ":"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatrixForm[
      Table[SSAmericanPutApp[S, 10, 100, \ 0.05, \ 0, \ 0.20, \ 1]\ , \ {S, \ 
          9, \ 11, 1}]]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1.15001651053271913`", \(-0.681927049797337847`\), 
              "0.31203136930685309`", \(-0.142070759860113415`\)},
            {"0.609416249554250644`", \(-0.410378446882039238`\), 
              "0.229770664137129587`", \(-0.224790872724806023`\)},
            {"0.299574690586433156`", \(-0.223215626787911292`\), 
              "0.14661585642043093`", \(-0.217787437214880785`\)}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output",
  CellMargins->{{45, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False]
}, Open  ]],

Cell["\<\
These results can all be improved by taking more time-steps, \
bearing in mind that all the comments that related to the European case also \
apply here. Note also that the results for Delta, Gamma and Theta are all \
reasonable - indeed the results from the supersymmetric tree are excellent. \
This confirms our earlier assertion that it is easy to get the Delta, Gamma, \
Theta Greeks from any form of the tree model - there is nothing special about \
the CRR form, in contradiction to a common misconception. Any of these trees \
can also be started at earlier time-steps, to refine the estimates of the \
basic Greeks.\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["19.6 More Difficult Tree Pathology", "Section",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
We have so far investigated the numerical issues raised by the use \
of tree models in \"vanilla\" situations. A wonderful article by Boyle and \
Lau (1994) exposed some of the problems that may arise when exotics are \
considered. Similar issues were considered by Kat and Verdonk (1995). This \
section is meant to be a brief warning as to what can go wrong. Difficulties \
can arise when there are barriers, or caps to the option value. In such \
cases, the oscillations in the valuation, as a function of the number of \
nodes, can become massive compared to those we have already observed for the \
vanilla case.

We shall discuss the Barrier issue, as it raises the most severe numerical \
difficulties, and we shall follow Boyle and Lau in investigating a type of \
down-and-out Call. We already have functions to deal with the continuously \
sampled case, introduced in Chapter 8. Kat and Verdonk discuss the issues \
which also arise in a more realistic context with discrete barrier sampling. \
Go and load the functions if you wish to check the following:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?DownAndOutCall\)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("Global`DownAndOutCall"\)], "Print",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    InterpretationBox[
      StyleBox[\(DownAndOutCall[reb_, \ p_, \ k_, \ h_, \ sd_, \ r_, \ q_, \ 
            t_]\  := \ 
          If[k\  >= \ h, \ 
            intone[p, \ k, \ sd, \ r, \ q, \ t, \ 1]\  - \ 
              intthree[p, \ k, \ h, \ sd, \ r, \ q, \ t, \ 1, \ 1]\  + \ 
              intsix[reb, \ p, \ h, \ sd, \ r, \ q, \ t, \ 1], \ 
            inttwo[p, \ k, \ h, \ sd, \ r, \ q, \ t, \ 1]\  - \ 
              intfour[p, \ k, \ h, \ sd, \ r, \ q, \ t, \ 1, \ 1]\  + \ 
              intsix[reb, \ p, \ h, \ sd, \ r, \ q, \ t, \ 1]]\),
        ShowStringCharacters->True,
        NumberMarks->True],
      InputForm[ 
        Definition[ DownAndOutCall]],
      Editable->True,
      AutoDelete->True]], "Print",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell["Boyle's example is the case", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(DownAndOutCall[0, 95, 100, \ 90, \ 0.25, 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(5.99684186816958586`\)], "Output",
  CellMargins->{{45, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell["\<\
Another reason for discussing this case is that it illustrates just \
how easy it is to adapt a tree to cover new payoffs or boundary conditions. \
One might say that this is a case of \"modify in haste, repent \
indefinitely\".\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
CRRDownAndOutOptionApp[S_, n_, \[Sigma]_, T_, r_, q_, H_, \
exercise_Function] :=
Module[{u = CRRupBinApp[n,r,q,\[Sigma],T], d = \
CRRdownBinApp[n,r,q,\[Sigma],T],
a = afunc[n,r,q,T],
disc = Discount[n,r,T],
P,Q,TreeOp,value,delta,gamma,theta},
P = Pfunc[u,d,a,disc];
Q = Qfunc[u,d,a,disc];
TreeOp[node_,level_] := TreeOp[node,level] = 
If[level==n,If[S d^node u^(level-node)> H, exercise[S d^node u^(level-node)], \
0],
If[S d^node u^(level-node)> H,
Max[{P,Q}.{TreeOp[node,level+1],TreeOp[node+1,level+1]},
exercise[S d^node u^(level-node)]],0]];
value=TreeOp[0,0];
delta = (TreeOp[0,1] - TreeOp[1,1])/((u-d)*S);
gamma = ((TreeOp[0,2] - TreeOp[1,2])/(u^2-1) - 
(TreeOp[1,2]-TreeOp[2,2])/(1-d^2))*2/(S^2 (u^2-d^2));
theta = (TreeOp[1,2] - TreeOp[0,0])/(2*T/n);
Clear[TreeOp];
{value, delta, gamma, theta}];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(CRRDownAndOutCallApp[S_, \ K_, H_, \ n_, \ r_, \ q_, \ \[Sigma]_, \ 
        T_]\  := 
      CRRDownAndOutOptionApp[S, n, \[Sigma], T, r, q, H, \ 
        Max[# - K, 0] &]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Let's take a look at this function for tree sizes in multiples of \
25 up to 200 - we just look at the value:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{k, \ \(CRRDownAndOutCallApp[95, \ 100, \ 90, \ 25*k, \ 0.1, \ 
            0, \ 0.25, \ 1]\)[\([1]\)]}, \ {k, \ 1, \ 8}]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{1, 8.84862983654701729`}, {2, 7.24052906454806333`}, {3, 
        6.30006139568719447`}, {4, 7.50453562184177869`}, {5, 
        6.98518889617253435`}, {6, 6.56115081574273162`}, {7, 
        6.20959126863292176`}, {8, 7.23071033946884256`}}\)], "Output",
  CellMargins->{{45, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[TextData[{
  "These agree with the values given by Boyle and Lau, and illustrate the \
abominable convergence. Note that if we take ",
  Cell[BoxData[
      \(TraditionalForm\`n = 21\)]],
  ", we get"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\ \(CRRDownAndOutCallApp[95, \ 100, \ 90, \ 21, \ 0.1, \ 0, \ 0.25, \ 
        1]\)[\([1]\)]\)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(6.18582066575724631`\)], "Output",
  CellMargins->{{45, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[TextData[{
  "which is not bad, and neither is the value for ",
  Cell[BoxData[
      \(TraditionalForm\`n = 85\)]],
  ":"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\ \(CRRDownAndOutCallApp[95, \ 100, \ 90, \ 85, \ 0.1, \ 0, \ 0.25, \ 
        1]\)[\([1]\)]\)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(6.01963446472505125`\)], "Output",
  CellMargins->{{45, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell["\<\
But look what happens if we increase the number of time intervals \
by one:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\ \(CRRDownAndOutCallApp[95, \ 100, \ 90, \ 22, \ 0.1, \ 0, \ 0.25, \ 
        1]\)[\([1]\)]\)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(9.21180643306044011`\)], "Output",
  CellMargins->{{45, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\ \(CRRDownAndOutCallApp[95, \ 100, \ 90, \ 86, \ 0.1, \ 0, \ 0.25, \ 
        1]\)[\([1]\)]\)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(7.86280298404806821`\)], "Output",
  CellMargins->{{45, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell["\<\
The following takes a while to evaluate, but gives the first part \
of the general pattern:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(sawtooth\  = \ \[IndentingNewLine]Table[{k, \ \(CRRDownAndOutCallApp[
              95, \ 100, \ 90, \ k, \ 0.1, \ 0, \ 0.25, \ 
              1]\)[\([1]\)]}, \ {k, \ 2, 100}]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell["\<\
We plot it, bearing in mind that the exact solution is almost \
exactly 6, which is given by the lower bound of the plot:\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[sawtooth, \ PlotJoined\  -> \ True]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00952381 -0.631464 0.107345 [
[.21429 .00011 -6 -9 ]
[.21429 .00011 6 0 ]
[.40476 .00011 -6 -9 ]
[.40476 .00011 6 0 ]
[.59524 .00011 -6 -9 ]
[.59524 .00011 6 0 ]
[.78571 .00011 -6 -9 ]
[.78571 .00011 6 0 ]
[.97619 .00011 -9 -9 ]
[.97619 .00011 9 0 ]
[.01131 .11995 -6 -4.5 ]
[.01131 .11995 0 4.5 ]
[.01131 .2273 -6 -4.5 ]
[.01131 .2273 0 4.5 ]
[.01131 .33464 -6 -4.5 ]
[.01131 .33464 0 4.5 ]
[.01131 .44199 -12 -4.5 ]
[.01131 .44199 0 4.5 ]
[.01131 .54933 -12 -4.5 ]
[.01131 .54933 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .01261 m
.21429 .01886 L
s
[(20)] .21429 .00011 0 1 Mshowa
.40476 .01261 m
.40476 .01886 L
s
[(40)] .40476 .00011 0 1 Mshowa
.59524 .01261 m
.59524 .01886 L
s
[(60)] .59524 .00011 0 1 Mshowa
.78571 .01261 m
.78571 .01886 L
s
[(80)] .78571 .00011 0 1 Mshowa
.97619 .01261 m
.97619 .01886 L
s
[(100)] .97619 .00011 0 1 Mshowa
.125 Mabswid
.07143 .01261 m
.07143 .01636 L
s
.11905 .01261 m
.11905 .01636 L
s
.16667 .01261 m
.16667 .01636 L
s
.2619 .01261 m
.2619 .01636 L
s
.30952 .01261 m
.30952 .01636 L
s
.35714 .01261 m
.35714 .01636 L
s
.45238 .01261 m
.45238 .01636 L
s
.5 .01261 m
.5 .01636 L
s
.54762 .01261 m
.54762 .01636 L
s
.64286 .01261 m
.64286 .01636 L
s
.69048 .01261 m
.69048 .01636 L
s
.7381 .01261 m
.7381 .01636 L
s
.83333 .01261 m
.83333 .01636 L
s
.88095 .01261 m
.88095 .01636 L
s
.92857 .01261 m
.92857 .01636 L
s
.25 Mabswid
0 .01261 m
1 .01261 L
s
.02381 .11995 m
.03006 .11995 L
s
[(7)] .01131 .11995 1 0 Mshowa
.02381 .2273 m
.03006 .2273 L
s
[(8)] .01131 .2273 1 0 Mshowa
.02381 .33464 m
.03006 .33464 L
s
[(9)] .01131 .33464 1 0 Mshowa
.02381 .44199 m
.03006 .44199 L
s
[(10)] .01131 .44199 1 0 Mshowa
.02381 .54933 m
.03006 .54933 L
s
[(11)] .01131 .54933 1 0 Mshowa
.125 Mabswid
.02381 .03408 m
.02756 .03408 L
s
.02381 .05555 m
.02756 .05555 L
s
.02381 .07701 m
.02756 .07701 L
s
.02381 .09848 m
.02756 .09848 L
s
.02381 .14142 m
.02756 .14142 L
s
.02381 .16289 m
.02756 .16289 L
s
.02381 .18436 m
.02756 .18436 L
s
.02381 .20583 m
.02756 .20583 L
s
.02381 .24877 m
.02756 .24877 L
s
.02381 .27024 m
.02756 .27024 L
s
.02381 .2917 m
.02756 .2917 L
s
.02381 .31317 m
.02756 .31317 L
s
.02381 .35611 m
.02756 .35611 L
s
.02381 .37758 m
.02756 .37758 L
s
.02381 .39905 m
.02756 .39905 L
s
.02381 .42052 m
.02756 .42052 L
s
.02381 .46346 m
.02756 .46346 L
s
.02381 .48493 m
.02756 .48493 L
s
.02381 .5064 m
.02756 .5064 L
s
.02381 .52786 m
.02756 .52786 L
s
.02381 .5708 m
.02756 .5708 L
s
.02381 .59227 m
.02756 .59227 L
s
.02381 .61374 m
.02756 .61374 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.04286 .60332 m
.05238 .52181 L
.0619 .45805 L
.07143 .39112 L
.08095 .35236 L
.09048 .30134 L
.1 .27407 L
.10952 .23396 L
.11905 .21322 L
.12857 .18067 L
.1381 .1641 L
.14762 .137 L
.15714 .1233 L
.16667 .10027 L
.17619 .08867 L
.18571 .06877 L
.19524 .05876 L
.20476 .04132 L
.21429 .03255 L
.22381 .0171 L
.23333 .35738 L
.24286 .33812 L
.25238 .33553 L
.2619 .31839 L
.27143 .31517 L
.28095 .30018 L
.29048 .29615 L
.3 .28292 L
.30952 .27833 L
.31905 .26655 L
.32857 .26159 L
.3381 .25103 L
.34762 .24583 L
.35714 .2363 L
.36667 .23094 L
.37619 .2223 L
.38571 .21686 L
.39524 .20898 L
.40476 .20351 L
.41429 .19629 L
.42381 .19083 L
.43333 .18419 L
.44286 .17876 L
.45238 .17263 L
.4619 .16726 L
.47143 .16158 L
.48095 .15627 L
.49048 .15099 L
.5 .14577 L
.50952 .14085 L
Mistroke
.51905 .13572 L
.52857 .13111 L
.5381 .12608 L
.54762 .12176 L
.55714 .11682 L
.56667 .11277 L
.57619 .10793 L
.58571 .10411 L
.59524 .09937 L
.60476 .09577 L
.61429 .09113 L
.62381 .08773 L
.63333 .08319 L
.64286 .07996 L
.65238 .07552 L
.6619 .07246 L
.67143 .06812 L
.68095 .06521 L
.69048 .06096 L
.7 .05819 L
.70952 .05404 L
.71905 .0514 L
.72857 .04734 L
.7381 .04482 L
.74762 .04084 L
.75714 .03844 L
.76667 .03455 L
.77619 .03224 L
.78571 .02844 L
.79524 .02623 L
.80476 .02251 L
.81429 .02039 L
.82381 .01675 L
.83333 .01472 L
.84286 .21257 L
.85238 .21096 L
.8619 .20666 L
.87143 .20512 L
.88095 .20089 L
.89048 .19942 L
.9 .19524 L
.90952 .19383 L
.91905 .18972 L
.92857 .18837 L
.9381 .18437 L
.94762 .18303 L
.95714 .17919 L
.96667 .17779 L
.97619 .17411 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  ImageSize->{194.562, 120.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000320000N1000`40O003h00O`Woo0032Ool002]o
o`@000=oo`8001Yoo`<000=oo`8001Yoo`8000Aoo`8001Yoo`8000Aoo`8001Ioo`<000Aoo`8000Ao
o`8000=oo`00;7oo00<007ooOol00Woo00@007ooOol001Yoo`05001oogooOol00002Ool00`00Oomo
o`0FOol01000Oomoo`000Woo00@007ooOol001Qoo`04001oogoo0002Ool01000Oomoo`005Woo00<0
07ooOol00Woo00@007ooOol0009oo`04001oogoo0002Ool002eoo`05001oogooOol00002Ool00`00
Oomoo`0FOol40002Ool01000Oomoo`0067oo00@007ooOol0009oo`04001oogoo000HOol01000Oomo
o`000Woo00@007ooOol001Ioo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`000Woo000^
Ool01000Oomoo`000Woo00<007ooOol05Woo00<007oo00000goo00@007ooOol001Qoo`<000=oo`04
001oogoo000IOol20003Ool01000Oomoo`005Woo00<007ooOol00Woo00@007ooOol0009oo`04001o
ogoo0002Ool002]oo`04001oogoo0002Ool01000Oomoo`0067oo00<007oo00000goo00@007ooOol0
01Qoo`03001oogoo00=oo`04001oogoo000HOol01000Oomoo`000Woo00@007ooOol001Ioo`03001o
ogoo009oo`04001oogoo0002Ool01000Oomoo`000Woo000/Ool20004Ool2000JOol20004Ool2000J
Ool30003Ool2000JOol20004Ool2000FOol20005Ool20004Ool20003Ool00<9oo`003Woo00<007oo
Ool0/Goo000>Ool00`00Oomoo`2aOol000ioo`03001oogoo0;5oo`002Woo/`001Goo000>Ool00`00
Oomoo`0OOol00`00Ool0000OOol00`00Oomoo`0OOol00`00Oomoo`0OOol00`00Oomoo`030002Ool0
0`00Oomoo`0GOol00`00Oomoo`07Ool000ioo`03001oogoo021oo`80069oo`@000Eoo`03001oogoo
025oo`003Woo0P0087oo00<007oo0000Ggoo0`002Goo00<007ooOol08Goo000>Ool00`00Oomoo`0N
Ool01000Oomoo`00GGoo0P0037oo00<007ooOol08Goo000>Ool00`00Oomoo`0MOol01@00Oomoogoo
0000FWoo0`003Woo00<007ooOol08Goo000>Ool00`00Oomoo`0MOol01@00Oomoogoo0000EWoo1000
4Goo00<007ooOol08Goo000>Ool2000MOol00`00Oomoo`02Ool00`00Oomoo`1COol00`00Oomoo`0C
Ool00`00Oomoo`0QOol000ioo`03001oogoo01aoo`03001oogoo009oo`03001oogoo04moo`@001Io
o`03001oogoo025oo`003Woo00<007ooOol06goo00<007ooOol00goo00<007ooOol0C7oo0`006Woo
00<007ooOol08Goo000>Ool00`00Oomoo`0JOol00`00Oomoo`04Ool00`00Oomoo`1:Ool2000NOol0
0`00Oomoo`0POol000ioo`8001]oo`03001oogoo00Aoo`03001oogoo04Qoo`80021oo`03001oogoo
021oo`003Woo00<007ooOol06Goo00<007ooOol01Goo00<007ooOol0AGoo0`008Woo00<007ooOol0
87oo000>Ool00`00Oomoo`0HOol00`00Oomoo`06Ool00`00Oomoo`13Ool2000UOol00`00Oomoo`0P
Ool000ioo`03001oogoo01Moo`03001oogoo00Moo`03001oogoo041oo`<002Moo`03001oogoo021o
o`003Woo0P005goo00<007ooOol027oo00<007ooOol0?Woo0P00:Woo00<007ooOol087oo000>Ool0
0`00Oomoo`0FOol00`00Oomoo`08Ool00`00Oomoo`0lOol2000/Ool00`00Oomoo`0POol000Qoo`03
001oogoo00=oo`03001oogoo01Ioo`03001oogoo00Qoo`03001oogoo03Uoo`<002ioo`03001oogoo
021oo`0027oo00<007ooOol00goo00<007ooOol05Goo00<007ooOol02Goo00<007ooOol0=goo0P00
<Goo00<007ooOol087oo0009Ool00`00Oomoo`02Ool2000FOol00`00Oomoo`09Ool00`00Oomoo`0f
Ool00`00Oomoo`0aOol00`00Oomoo`0POol000Uoo`03001oogoo009oo`03001oogoo01Eoo`03001o
ogoo00Uoo`03001oogoo03Aoo`8003Aoo`03001oogoo021oo`002Woo00D007ooOomoo`0001Ioo`03
001oogoo00Yoo`03001oogoo035oo`<003Ioo`03001oogoo021oo`001goo10000goo00<007ooOol0
4goo00<007ooOol02goo00<007ooOol0;goo0P00>Goo00<007ooOol087oo000>Ool2000DOol00`00
Oomoo`0;Ool00`00Oomoo`0]Ool2000kOol00`00Oomoo`0POol000ioo`03001oogoo019oo`03001o
ogoo00aoo`03001oogoo02aoo`03001oogoo03]oo`03001oogoo021oo`003Woo00<007ooOol04Woo
00<007ooOol037oo00<007ooOol0:Woo0P00?Woo00<007ooOol087oo000>Ool2000COol00`00Oomo
o`0<Ool00`00Oomoo`0XOol20010Ool00`00Oomoo`0POol000ioo`03001oogoo015oo`03001oogoo
00eoo`03001oogoo02Ioo`8004=oo`03001oogoo01moo`003Woo00<007ooOol04Goo00<007ooOol0
3Goo00<007ooOol09Goo00<007ooOol0@goo00<007ooOol057oo0P002Goo000>Ool00`00Oomoo`0@
Ool00`00Oomoo`0>Ool00`00Oomoo`0SOol20016Ool00`00Oomoo`0AOol3000;Ool000ioo`80015o
o`03001oogoo00ioo`03001oogoo025oo`8004Qoo`03001oogoo00eoo`@000ioo`003Woo00<007oo
Ool047oo00<007ooOol03goo00<007ooOol07goo00<007ooOol0B7oo00<007ooOol02Woo0`004Woo
000>Ool00`00Oomoo`0?Ool00`00Oomoo`0@Ool00`00Oomoo`0NOol00`00Oomoo`19Ool00`00Oomo
o`07Ool3000EOol000ioo`03001oogoo00moo`03001oogoo011oo`03001oogoo01eoo`03001oogoo
04Yoo`03001oogoo00=oo`@001Qoo`003Woo0P0047oo00<007ooOol047oo00<007ooOol06goo0P00
CGoo00<007ooOol00`0077oo000>Ool00`00Oomoo`0>Ool00`00Oomoo`0AOol00`00Oomoo`0JOol0
0`00Oomoo`1=Ool3000OOol000Qoo`8000Aoo`03001oogoo00ioo`03001oogoo015oo`03001oogoo
01Qoo`80079oo`001goo00@007ooOol000=oo`03001oogoo00eoo`03001oogoo019oo`03001oogoo
01Ioo`8007Aoo`001goo00@007ooOol000=oo`8000ioo`03001oogoo019oo`03001oogoo01Aoo`80
07Ioo`0027oo0P0017oo00<007ooOol037oo00<007ooOol04goo00<007ooOol04goo00<007ooOol0
MWoo0007Ool01000Oomoo`000goo00<007ooOol037oo00<007ooOol04goo00<007ooOol04goo00<0
07ooOol0MWoo0008Ool20004Ool00`00Oomoo`0<Ool00`00Oomoo`0COol00`00Oomoo`0AOol2001i
Ool000ioo`8000eoo`03001oogoo01=oo`03001oogoo011oo`03001oogoo07Uoo`003Woo00<007oo
Ool02goo00<007ooOol057oo00<007ooOol03goo00<007ooOol0NWoo000>Ool00`00Oomoo`0;Ool0
0`00Oomoo`0DOol00`00Oomoo`0>Ool00`00Oomoo`1kOol000ioo`8000aoo`03001oogoo01Aoo`03
001oogoo00eoo`03001oogoo07aoo`003Woo00<007ooOol02goo00<007ooOol057oo00<007ooOol0
37oo00<007ooOol0OGoo000>Ool00`00Oomoo`0;Ool00`00Oomoo`0DOol00`00Oomoo`0:Ool20020
Ool000ioo`03001oogoo00Yoo`03001oogoo01Eoo`03001oogoo00Uoo`03001oogoo081oo`003Woo
0P002goo00<007ooOol05Goo00<007ooOol02Goo00<007ooOol0P7oo000>Ool00`00Oomoo`0:Ool0
0`00Oomoo`0EOol00`00Oomoo`07Ool20023Ool000ioo`03001oogoo00Uoo`03001oogoo01Ioo`03
001oogoo00Ioo`03001oogoo08=oo`003Woo00<007ooOol02Goo00<007ooOol05Woo00<007ooOol0
1Goo00<007ooOol0Q7oo000>Ool2000:Ool00`00Oomoo`0FOol00`00Oomoo`03Ool20027Ool000io
o`03001oogoo00Qoo`03001oogoo01Moo`03001oogoo009oo`03001oogoo08Moo`001goo0`0017oo
00<007ooOol027oo00<007ooOol05goo00<007ooOol00Woo00<007ooOol0Qgoo000:Ool01@00Oomo
ogoo00002Woo00<007ooOol05goo00D007ooOomoo`0008Yoo`0027oo0`000goo0P002Goo00<007oo
Ool05goo00<007ooOol00P00RWoo0007Ool01000Oomoo`000goo00<007ooOol027oo00<007ooOol0
5goo00<007oo0000S7oo0007Ool01000Oomoo`000goo00<007ooOol01goo00<007ooOol067oo00<0
07oo0000S7oo0008Ool20004Ool00`00Oomoo`07Ool00`00Oomoo`0HOol2002=Ool000ioo`8000Qo
o`03001oogoo01Qoo`03001oogoo08aoo`003Woo00<007ooOol01goo00<007ooOol0Ygoo000>Ool0
0`00Oomoo`06Ool00`00Oomoo`2XOol000ioo`03001oogoo00Ioo`03001oogoo0:Qoo`003Woo0P00
1goo00<007ooOol0Z7oo000>Ool00`00Oomoo`05Ool00`00Oomoo`2YOol000ioo`03001oogoo00Eo
o`03001oogoo0:Uoo`003Woo0P001Woo00<007ooOol0ZGoo000>Ool00`00Oomoo`05Ool00`00Oomo
o`2YOol000ioo`03001oogoo00Eoo`03001oogoo0:Uoo`003Woo00<007ooOol01Goo00<007ooOol0
ZGoo000>Ool20006Ool00`00Oomoo`2YOol000ioo`03001oogoo00Eoo`03001oogoo0:Uoo`0000Ao
o`00000000Aoo`8000Aoo`03001oogoo00Aoo`03001oogoo0:Yoo`000Woo00<007ooOol00Woo00@0
07ooOol000=oo`03001oogoo00Aoo`03001oogoo0:Yoo`000Woo00<007ooOol00Woo00@007ooOol0
00=oo`8000Eoo`03001oogoo0:Yoo`000Woo00<007ooOol00Woo00@007ooOol000=oo`03001oogoo
00Aoo`03001oogoo0:Yoo`000Woo00<007ooOol00Woo00@007ooOol000=oo`03001oogoo00Aoo`03
001oogoo0:Yoo`0000=oo`0000001Goo0P0017oo00<007ooOol017oo00<007ooOol0ZWoo000>Ool2
0005Ool00`00Oomoo`2ZOol000ioo`03001oogoo00Aoo`03001oogoo0:Yoo`003Woo00<007ooOol0
17oo00<007ooOol0ZWoo000>Ool00`00Oomoo`03Ool00`00Oomoo`2[Ool000ioo`8000Aoo`03001o
ogoo0:]oo`003Woo00<007ooOol00goo00<007ooOol0Zgoo000>Ool00`00Oomoo`03Ool00`00Oomo
o`2[Ool000ioo`03001oogoo00=oo`03001oogoo0:]oo`003Woo0P000goo00<007ooOol0[7oo000>
Ool00`00Oomoo`02Ool00`00Oomoo`2/Ool000ioo`03001oogoo009oo`03001oogoo0:aoo`003Woo
0P000goo00<007ooOol0[7oo000>Ool00`00Oomoo`02Ool00`00Oomoo`2/Ool00004Ool000000003
Ool30004Ool00`00Oomoo`02Ool00`00Oomoo`2/Ool0009oo`03001oogoo00=oo`03001oogoo00=o
o`03001oogoo009oo`03001oogoo0:aoo`000Woo00<007ooOol00goo00<007ooOol00goo0P000Woo
00<007ooOol0[Goo0002Ool00`00Oomoo`03Ool00`00Oomoo`03Ool01@00Oomoogoo0000[goo0002
Ool00`00Oomoo`03Ool00`00Oomoo`03Ool01@00Oomoogoo0000[goo00000goo00000004Ool20005
Ool01@00Oomoogoo0000[goo000>Ool20002Ool00`00Oomoo`2]Ool000ioo`05001oogooOol0002_
Ool000ioo`05001oogooOol0002_Ool000ioo`04001oogoo002`Ool000ioo`800003Ool007oo0:mo
o`003Woo00@007ooOol00;1oo`003Woo00@007ooOol00;1oo`003Woo00<007ooOol0/Goo000>Ool2
002bOol00001\
\>"],
  ImageRangeCache->{{{0, 193.562}, {119.188, 0}} -> {-8.52768, 5.41338, 
  0.588835, 0.0522423}}]
}, Open  ]],

Cell[TextData[{
  "This reveals the massive nature of the error, and the particularly nasty \
jumps from 21 to 22 and from 85 to 86. This, like the rather smaller errors \
we encountered in the vanilla case, can be dealt with by careful control of \
the time-step - another case of magic tree sizes - and the calculation is a \
lot like the one we have already done. Boyle and Lau argue that to arrange \
things so that the barrier is close to, but just above, a layer of horizontal \
nodes in the tree, ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " must be the largest integer that is smaller than"
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(m\^2\ \[Sigma]\^2\ T\)\/\(\(log\^2\)(S\/H)\)\)], 
  "NumberedEquation",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This formula is obtained, as before, by counting the number of CRR \
price-steps needed to get to the barrier. We can work it out for the case \
under consideration. \
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(BoyleLau[m_, \ \[Sigma]_, \ T_, \ S_, \ H_]\  := \ 
      Floor[T*\((m*\[Sigma]/Log[S/H])\)^2]\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[BoyleLau[m, \ 0.25, \ 1, \ 95, \ 90], \ {m, \ 1, \ 5}]\)], "Input",\

  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({21, 85, 192, 342, 534}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "One point not considered by Boyle and Lau is the fact that the other types \
of tree cannot be remedied by a cunning choice of ",
  Cell[BoxData[
      \(TraditionalForm\`\(n . \)\)]],
  " Let's take a very quick look at the JR model - the same problem arises \
with the supersymmetric form."
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
JRDownAndOutOptionApp[S_, n_, \[Sigma]_, T_, r_, q_, H_, \
exercise_Function] :=
Module[{u,d,disc = Discount[n,r,T],P,Q,TreeOp,value,delta,gamma,theta},
u = JRupBinApp[n,r,q,\[Sigma],T]; 
d = JRdownBinApp[n,r,q,\[Sigma],T];
P = JRUpWeight[disc];
Q = P;
TreeOp[node_,level_] := TreeOp[node,level] = 
If[level==n,If[S d^node u^(level-node)> H, exercise[S d^node u^(level-node)], \
0],
If[S d^node u^(level-node)> H,
Max[{P,Q}.{TreeOp[node,level+1],TreeOp[node+1,level+1]},
exercise[S d^node u^(level-node)]],0]];
value=TreeOp[0,0];
delta = (TreeOp[0,1] - TreeOp[1,1])/((u-d)*S);
gamma = ((TreeOp[0,2] -TreeOp[1,2])/(u^2-u*d) -
(TreeOp[1,2]-TreeOp[2,2])/(u*d-d^2))*2/(S^2(u^2-d^2));
theta = (TreeOp[1,2]-(u*d-1)*S*delta - TreeOp[0,0])/(2*T/n);
Clear[TreeOp];
{value, delta, gamma, theta}];\
\>", "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(JRDownAndOutCallApp[S_, \ K_, H_, \ n_, \ r_, \ q_, \ \[Sigma]_, \ 
        T_]\  := 
      JRDownAndOutOptionApp[S, n, \[Sigma], T, r, q, H, \ 
        Max[# - K, 0] &]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(jrtooth\  = \ 
      Table[{k, \ \(JRDownAndOutCallApp[95, \ 100, \ 90, \ k, \ 0.1, \ 0, \ 
              0.25, \ 1]\)[\([1]\)]}, \ {k, \ 2, 100}]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We plot this also - the behaviour of this plot is a reflection of \
the fact that with drift built into the tree, it is just not possible to \
align the constant barrier with the tree nodes - the result does not get \
anywhere near 6.\
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[jrtooth, \ PlotJoined\  -> \ True]; \)\)], "Input",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00952381 -1.37164 0.209012 [
[.21429 .07894 -6 -9 ]
[.21429 .07894 6 0 ]
[.40476 .07894 -6 -9 ]
[.40476 .07894 6 0 ]
[.59524 .07894 -6 -9 ]
[.59524 .07894 6 0 ]
[.78571 .07894 -6 -9 ]
[.78571 .07894 6 0 ]
[.97619 .07894 -9 -9 ]
[.97619 .07894 9 0 ]
[.01131 .19595 -18 -4.5 ]
[.01131 .19595 0 4.5 ]
[.01131 .30046 -6 -4.5 ]
[.01131 .30046 0 4.5 ]
[.01131 .40496 -18 -4.5 ]
[.01131 .40496 0 4.5 ]
[.01131 .50947 -6 -4.5 ]
[.01131 .50947 0 4.5 ]
[.01131 .61397 -18 -4.5 ]
[.01131 .61397 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .09144 m
.21429 .09769 L
s
[(20)] .21429 .07894 0 1 Mshowa
.40476 .09144 m
.40476 .09769 L
s
[(40)] .40476 .07894 0 1 Mshowa
.59524 .09144 m
.59524 .09769 L
s
[(60)] .59524 .07894 0 1 Mshowa
.78571 .09144 m
.78571 .09769 L
s
[(80)] .78571 .07894 0 1 Mshowa
.97619 .09144 m
.97619 .09769 L
s
[(100)] .97619 .07894 0 1 Mshowa
.125 Mabswid
.07143 .09144 m
.07143 .09519 L
s
.11905 .09144 m
.11905 .09519 L
s
.16667 .09144 m
.16667 .09519 L
s
.2619 .09144 m
.2619 .09519 L
s
.30952 .09144 m
.30952 .09519 L
s
.35714 .09144 m
.35714 .09519 L
s
.45238 .09144 m
.45238 .09519 L
s
.5 .09144 m
.5 .09519 L
s
.54762 .09144 m
.54762 .09519 L
s
.64286 .09144 m
.64286 .09519 L
s
.69048 .09144 m
.69048 .09519 L
s
.7381 .09144 m
.7381 .09519 L
s
.83333 .09144 m
.83333 .09519 L
s
.88095 .09144 m
.88095 .09519 L
s
.92857 .09144 m
.92857 .09519 L
s
.25 Mabswid
0 .09144 m
1 .09144 L
s
.02381 .19595 m
.03006 .19595 L
s
[(7.5)] .01131 .19595 1 0 Mshowa
.02381 .30046 m
.03006 .30046 L
s
[(8)] .01131 .30046 1 0 Mshowa
.02381 .40496 m
.03006 .40496 L
s
[(8.5)] .01131 .40496 1 0 Mshowa
.02381 .50947 m
.03006 .50947 L
s
[(9)] .01131 .50947 1 0 Mshowa
.02381 .61397 m
.03006 .61397 L
s
[(9.5)] .01131 .61397 1 0 Mshowa
.125 Mabswid
.02381 .11234 m
.02756 .11234 L
s
.02381 .13325 m
.02756 .13325 L
s
.02381 .15415 m
.02756 .15415 L
s
.02381 .17505 m
.02756 .17505 L
s
.02381 .21685 m
.02756 .21685 L
s
.02381 .23775 m
.02756 .23775 L
s
.02381 .25865 m
.02756 .25865 L
s
.02381 .27955 m
.02756 .27955 L
s
.02381 .32136 m
.02756 .32136 L
s
.02381 .34226 m
.02756 .34226 L
s
.02381 .36316 m
.02756 .36316 L
s
.02381 .38406 m
.02756 .38406 L
s
.02381 .42586 m
.02756 .42586 L
s
.02381 .44676 m
.02756 .44676 L
s
.02381 .46766 m
.02756 .46766 L
s
.02381 .48857 m
.02756 .48857 L
s
.02381 .53037 m
.02756 .53037 L
s
.02381 .55127 m
.02756 .55127 L
s
.02381 .57217 m
.02756 .57217 L
s
.02381 .59307 m
.02756 .59307 L
s
.02381 .07054 m
.02756 .07054 L
s
.02381 .04964 m
.02756 .04964 L
s
.02381 .02874 m
.02756 .02874 L
s
.02381 .00784 m
.02756 .00784 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.05959 .61803 m
.0619 .55845 L
.07143 .49727 L
.08095 .36779 L
.09048 .33867 L
.1 .2529 L
.10952 .23044 L
.11905 .16723 L
.12857 .14724 L
.1381 .09771 L
.14762 .19303 L
.15714 .15044 L
.16667 .13411 L
.17619 .09819 L
.18571 .08235 L
.19524 .05144 L
.20476 .48651 L
.21429 .45321 L
.22381 .43746 L
.23333 .40863 L
.24286 .39432 L
.25238 .36889 L
.2619 .35559 L
.27143 .33285 L
.28095 .32032 L
.29048 .29977 L
.3 .28786 L
.30952 .26914 L
.31905 .25776 L
.32857 .25125 L
.3381 .24053 L
.34762 .22472 L
.35714 .21437 L
.36667 .19969 L
.37619 .18969 L
.38571 .176 L
.39524 .16633 L
.40476 .15352 L
.41429 .14416 L
.42381 .13239 L
.43333 .13746 L
.44286 .12655 L
.45238 .1181 L
.4619 .10794 L
.47143 .09985 L
.48095 .09034 L
.49048 .08257 L
.5 .07364 L
.50952 .0835 L
.51905 .07503 L
Mistroke
.52857 .06777 L
.5381 .05975 L
.54762 .05272 L
.55714 .04511 L
.56667 .03829 L
.57619 .05303 L
.58571 .04635 L
.59524 .03939 L
.60476 .03288 L
.61429 .02623 L
.62381 .01989 L
.63333 .04513 L
.64286 .03889 L
.65238 .03271 L
.6619 .02661 L
.67143 .02068 L
.68095 .01472 L
.69048 .05016 L
.7 .04419 L
.70952 .03863 L
.71905 .03282 L
.72857 .02753 L
.7381 .02186 L
.74762 .08175 L
.75714 .076 L
.76667 .07099 L
.77619 .06875 L
.78571 .06396 L
.79524 .1872 L
.80476 .18223 L
.81429 .17643 L
.82381 .17166 L
.83333 .16598 L
.84286 .16138 L
.85238 .15582 L
.8619 .15138 L
.87143 .15011 L
.88095 .14582 L
.89048 .14048 L
.9 .13633 L
.90952 .13108 L
.91905 .12707 L
.92857 .12192 L
.9381 .11803 L
.94762 .11297 L
.95714 .1138 L
.96667 .10882 L
.97619 .10516 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{231.562, 143.062},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004Q0000/Q000`40O003h00Oogoo8Woo003oOolR
Ool00?moob9oo`00ogoo8Woo003oOolROol001Uoo`03001oogoo0?moo`Ioo`006Goo00<007ooOol0
ogoo1Woo000IOol2003oOol7Ool001Uoo`03001oogoo0?moo`Ioo`006Goo00<007ooOol0ZWoo0P00
FGoo000IOol00`00Oomoo`2KOol2000;Ool200000goo001oo`1HOol001Uoo`03001oogoo09Yoo`03
001oo`0000Yoo`05001oogooOol0000=Ool2001:Ool001Uoo`8009Uoo`80009oo`03001oogoo00Io
o`8000Eoo`03001oogoo00Moo`<00003Ool007oo04Uoo`006Goo00<007ooOol0UWoo0P001Goo00<0
07ooOol00goo0P001goo00<007ooOol01Goo0P0017oo00<007ooOol0B7oo000IOol00`00Oomoo`2E
Ool00`00Oomoo`05Ool00`00Oomoo`02Ool00`00Oomoo`07Ool00`00Oomoo`04Ool00`00Oomoo`04
Ool00`00Oomoo`18Ool001Uoo`03001oogoo08aoo`8000Ioo`03001oogoo00Ioo`03001oogoo0080
00Yoo`03001oogoo009oo`8000Moo`03001oogoo04Qoo`006Goo00<007ooOol0Rgoo00<007oo0000
1Goo00<007ooOol027oo0P0037oo00<007ooOol00P002Woo00<007ooOol0Agoo000IOol00`00Oomo
o`0YOol50004Ool3000XOol30004Ool3000SOol01000Oomoogoo1@000goo0`001Goo00<007ooOol0
37oo0P0037oo00<007ooOol00goo0`001Goo0`008goo1@000goo0`001Goo0`000goo000IOol2000[
Ool200000goo001oo`02Ool01@00Oomoogoo0000:7oo00<007ooOol00Woo00D007ooOomoo`00025o
o`04001oogooOol200000goo00000003Ool01@00Oomoogoo00004goo00<007ooOol02goo00<007oo
Ool00Woo00D007ooOomoo`0000=oo`05001oogooOol0000TOol00`00Oomoo`02Ool01@00Oomoogoo
00000goo00D007ooOomoo`00009oo`006Goo00<007ooOol0:goo00<007ooOol00goo00D007ooOomo
o`0002Eoo`D000=oo`05001oogooOol0000OOol20004Ool01@00Ool007oo00000goo00D007ooOomo
o`00025oo`03001oogoo009oo`05001oogooOol00003Ool01@00Oomoogoo000097oo00<007ooOol0
0Woo00D007ooOomoo`0000=oo`05001oogooOol00002Ool001Uoo`03001oogoo02Yoo`<000Aoo`05
001oogooOol0000UOol01000Oomoo`0017oo00D007ooOomoo`0001ioo`03001oogoo00Aoo`05001o
ogooOol00003Ool01@00Oomoogoo00008Goo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001o
ogooOol0000TOol00`00Oomoo`02Ool01@00Oomoogoo00000goo00D007ooOomoo`00009oo`006Goo
00<007ooOol0:Woo0P0000=oo`00Ool00Woo00D007ooOomoo`0002Ioo`03001oo`0000Aoo`05001o
ogooOol0000MOol00`00Oomoo`05Ool40004Ool01@00Oomoogoo00008Goo00<007ooOol00goo0`00
17oo00D007ooOomoo`0002Aoo`03001oogoo009oo`05001oogooOol00003Ool01@00Oomoogoo0000
0Woo000IOol00`00Oomoo`0ZOol200000goo001oo`02Ool01@00Oomoogoo00009Woo00<007oo0000
17oo00D007ooOomoo`0001aoo`03001oogoo00Ioo`03001oogoo00Eoo`05001oogooOol0000QOol0
0`00Oomoo`02Ool01000Oomoogoo0`0000=oo`00Ool00Woo00<007ooOol08Woo00<007ooOol00Woo
00D007ooOomoo`0000=oo`05001oogooOol00002Ool001Uoo`8002Yoo`05001oo`00Ool00003Ool0
1@00Oomoogoo00009goo0P0017oo00D007ooOomoo`0001Yoo`8000Yoo`03001oogoo00Aoo`05001o
ogooOol0000ROol01000Oomoogoo1@0000Aoo`00Ool000=oo`03001oogoo029oo`03001oogoo009o
o`05001oogooOol00003Ool01@00Oomoogoo00000Woo000IOol00`00Oomoo`0YOol40005Ool3000Y
Ool00`00Oomoo`03Ool3000DOol00`00Oomoo`03Ool00`00Oomoo`0;Ool30004Ool3000SOol01000
Oomoogoo10000Woo00<007ooOol00`008goo0`001Goo0`001Goo0`000goo000IOol00`00Oomoo`0Y
Ool00`00Ool0001>Ool00`00Ool00003Ool00`00Oomoo`0iOol01000Ool000001Woo00<007ooOol0
?7oo000IOol00`00Oomoo`0XOol01000Oomoo`00CGoo00L007ooOomoo`00Ool0000lOol20008Ool0
0`00Oomoo`0lOol001Uoo`03001oogoo02Qoo`04001oogoo001<Ool00`00Oomoo`03Ool00`00Oomo
o`15Ool00`00Oomoo`0lOol001Uoo`03001oogoo02Moo`05001oogooOol0001;Ool00`00Oomoo`1=
Ool00`00Oomoo`0kOol001=oool000X000Eoo`006Goo00<007ooOol02Woo00<007ooOol02Goo00<0
07ooOol02Woo00D007ooOomoo`0000=oo`03001oogoo009oo`03001oogoo00Uoo`03001oogoo00Yo
o`03001oogoo00Uoo`03001oogoo00Yoo`03001oogoo00Uoo`03001oogoo00=oo`03001oogoo00Ao
o`03001oogoo00Yoo`03001oogoo00Uoo`03001oogoo00Yoo`03001oogoo00Uoo`03001oogoo00Yo
o`03001oogoo00Uoo`8000]oo`03001oogoo00Yoo`03001oogoo00Uoo`03001oogoo00Yoo`03001o
ogoo00Uoo`006Goo00<007ooOol06goo00<007ooOol027oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol0;goo00<007ooOol03Woo00<007ooOol07Woo00<007ooOol0;goo0P00<Goo00<007oo
Ool02Goo000IOol00`00Oomoo`0KOol00`00Oomoo`08Ool00`00Oomoo`02Ool00`00Oomoo`14Ool0
0`00Oomoo`1BOol00`00Oomoo`0kOol001Uoo`03001oogoo01]oo`03001oogoo00Moo`03001oogoo
00=oo`03001oogoo04=oo`03001oogoo05=oo`03001oogoo02aoo`@000]oo`006Goo0P0077oo00<0
07ooOol01goo00<007ooOol00goo00<007ooOol0@Woo00<007ooOol0E7oo00<007ooOol0:goo00<0
07ooOol03Goo000IOol00`00Oomoo`0JOol20009Ool00`00Oomoo`03Ool00`00Oomoo`11Ool00`00
Oomoo`1EOol00`00Oomoo`0VOol5000@Ool001Uoo`03001oogoo01Yoo`03001oo`0000Moo`03001o
ogoo00Aoo`03001oogoo041oo`03001oogoo05Ioo`03001oogoo02Aoo`8001Eoo`006Goo00<007oo
Ool06Woo00<007oo00001goo00<007ooOol017oo00<007ooOol0?goo00<007ooOol0Egoo00<007oo
Ool08Goo0`005goo000IOol00`00Oomoo`0JOol00`00Ool00007Ool00`00Oomoo`04Ool00`00Oomo
o`0nOol00`00Oomoo`1HOol00`00Oomoo`0OOol2000JOol001Uoo`03001oogoo01Yoo`03001oo`00
00Ioo`03001oogoo00Eoo`03001oogoo03eoo`03001oogoo05Yoo`03001oogoo01aoo`8001aoo`00
6Goo0P006goo00<007oo00001Woo00<007ooOol01Goo00<007ooOol0>Goo0P0000=oo`00Ool0G7oo
00<007ooOol06goo00<007ooOol077oo000IOol00`00Oomoo`0JOol00`00Ool00006Ool00`00Oomo
o`05Ool00`00Oomoo`0hOol01000Oomoo`00GWoo00<007ooOol06Goo0P007goo000IOol00`00Oomo
o`0IOol01000Oomoo`001Goo00<007ooOol01Woo00<007ooOol0=goo00<007ooOol0H7oo00<007oo
Ool05Woo0`008Goo000IOol00`00Oomoo`0IOol01000Oomoo`0017oo00<007ooOol01goo00<007oo
Ool0=Woo00<007ooOol0HGoo00<007ooOol057oo0P0097oo000IOol00`00Oomoo`0IOol01@00Oomo
ogoo00000Woo00<007ooOol027oo00<007ooOol0=Goo00<007ooOol0HWoo00<007ooOol04Goo0`00
9Woo000IOol2000JOol01@00Oomoogoo00000Woo00<007ooOol027oo00<007ooOol0=7oo00<007oo
Ool0Hgoo00<007ooOol03goo0P00:Goo000IOol00`00Oomoo`0HOol00`00Oomoo`02Ool01000Oomo
o`002Woo00<007ooOol0=7oo00<007ooOol0Hgoo00<007ooOol037oo0`00:goo000IOol00`00Oomo
o`0GOol00`00Oomoo`03Ool01000Oomoo`002Woo00<007ooOol0<goo00<007ooOol0I7oo00<007oo
Ool02Woo0P00;Woo000IOol00`00Oomoo`0GOol00`00Oomoo`03Ool00`00Ool0000<Ool00`00Oomo
o`0aOol00`00Oomoo`1UOol00`00Oomoo`09Ool00`00Oomoo`0^Ool001Uoo`03001oogoo01Ioo`03
001oogoo00Aoo`03001oo`0000aoo`03001oogoo031oo`03001oogoo06Ioo`03001oogoo00Moo`80
035oo`006Goo00<007ooOol05Woo00<007ooOol017oo00<007oo000037oo00<007ooOol0;goo00<0
07ooOol0J7oo00<007ooOol00goo0`00<goo000IOol2000GOol00`00Oomoo`04Ool00`00Ool0000<
Ool00`00Oomoo`0^Ool00`00Oomoo`1YOol01000Oomoogoo0P00=Woo000IOol00`00Oomoo`0FOol0
0`00Oomoo`04Ool00`00Ool0000<Ool00`00Oomoo`0^Ool00`00Oomoo`1YOol01000Oomoo`00>7oo
000IOol00`00Oomoo`0FOol00`00Oomoo`05Ool2000<Ool00`00Oomoo`0]Ool00`00Oomoo`1ZOol3
000iOol0009oo`03001oogoo00Eoo`8000Eoo`@000Aoo`03001oogoo01Eoo`03001oogoo00Ioo`03
001oogoo00]oo`03001oogoo02aoo`03001oogoo06]oo`03001oogoo03Uoo`000Woo00<007ooOol0
1Goo0P002Goo00D007ooOomoo`0001Moo`03001oogoo00Ioo`03001oogoo00]oo`03001oogoo02]o
o`03001oogoo0:Qoo`000goo00<007ooOol03goo00@007ooOomoo`<001Eoo`03001oogoo00Ioo`03
001oogoo00]oo`03001oogoo02Yoo`03001oogoo0:Uoo`000goo00<007ooOol03goo00D007ooOomo
o`0001Moo`03001oogoo01Aoo`03001oogoo02Uoo`03001oogoo0:Yoo`000goo00<007ooOol02goo
100017oo00<007ooOol05Goo00<007ooOol057oo00<007ooOol0:Goo00<007ooOol0ZWoo0004Ool0
0`00Oomoo`0:Ool00`00Oomoo`05Ool00`00Oomoo`0EOol00`00Oomoo`0DOol00`00Oomoo`0XOol0
0`00Oomoo`2[Ool00005001oogooOol0000<Ool00`00Oomoo`05Ool00`00Oomoo`0EOol00`00Oomo
o`0DOol00`00Oomoo`0WOol00`00Oomoo`2/Ool000D000aoo`@000Aoo`03001oogoo01Eoo`03001o
ogoo01Aoo`03001oogoo02Ioo`03001oogoo0:eoo`006Goo0P005Woo00<007ooOol057oo00<007oo
Ool09Woo00<007ooOol0[Goo000IOol00`00Oomoo`0DOol00`00Oomoo`0EOol00`00Oomoo`0UOol0
0`00Oomoo`2^Ool001Uoo`03001oogoo01Aoo`03001oogoo01Eoo`03001oogoo02Aoo`03001oogoo
0:moo`006Goo00<007ooOol057oo00<007ooOol05Goo00<007ooOol097oo00<007ooOol0[goo000I
Ool00`00Oomoo`0DOol00`00Oomoo`0EOol00`00Oomoo`0SOol00`00Oomoo`2`Ool001Uoo`03001o
ogoo01Aoo`03001oogoo01Eoo`03001oogoo029oo`03001oogoo0;5oo`006Goo0P0057oo00<007oo
Ool05Woo00<007ooOol08Goo00<007ooOol0/Woo000IOol00`00Oomoo`0COol00`00Oomoo`0FOol0
0`00Oomoo`0QOol00`00Oomoo`2bOol001Uoo`03001oogoo019oo`03001oogoo01Moo`03001oogoo
021oo`03001oogoo0;=oo`006Goo00<007ooOol04Woo00<007ooOol05goo00<007ooOol07Woo0P00
]Woo000IOol00`00Oomoo`0AOol00`00Oomoo`0HOol00`00Oomoo`0MOol00`00Oomoo`2fOol001Uo
o`80019oo`03001oogoo01Qoo`03001oogoo01aoo`03001oogoo0;Moo`006Goo00<007ooOol04Goo
00<007ooOol067oo00<007ooOol077oo00<007ooOol0]goo000IOol00`00Oomoo`0AOol00`00Oomo
o`0HOol00`00Oomoo`0KOol00`00Oomoo`2hOol001Uoo`03001oogoo015oo`03001oogoo01Qoo`03
001oogoo01Yoo`03001oogoo0;Uoo`006Goo00<007ooOol04Goo00<007ooOol067oo00<007ooOol0
6Woo00<007ooOol0^Goo000IOol00`00Oomoo`0@Ool00`00Oomoo`0IOol00`00Oomoo`0IOol00`00
Oomoo`2jOol001Uoo`80015oo`03001oogoo01Uoo`03001oogoo01Qoo`03001oogoo0;]oo`006Goo
00<007ooOol047oo00<007ooOol06Goo00<007ooOol067oo00<007ooOol0^goo000IOol00`00Oomo
o`0@Ool00`00Oomoo`0IOol00`00Oomoo`0GOol00`00Oomoo`2lOol0011oo`<000Ioo`03001oogoo
011oo`03001oogoo01Uoo`03001oogoo01Moo`03001oogoo0;aoo`003goo00D007ooOomoo`0000Eo
o`03001oogoo011oo`03001oogoo01Uoo`03001oogoo01Ioo`03001oogoo0;eoo`003goo00D007oo
Oomoo`0000Eoo`<0011oo`03001oogoo01Uoo`03001oogoo01Eoo`03001oogoo0;ioo`003goo00D0
07ooOomoo`0000Eoo`03001oogoo011oo`03001oogoo01Uoo`03001oogoo01Eoo`03001oogoo0;io
o`0047oo0`001Woo00<007ooOol047oo00<007ooOol06Goo00<007ooOol057oo00<007ooOol0_goo
000?Ool01@00Oomoogoo00001Goo00<007ooOol047oo00<007ooOol06Goo00<007ooOol04goo00<0
07ooOol0`7oo000?Ool01@00Oomoogoo00001Goo00<007ooOol047oo00<007ooOol06Goo00<007oo
Ool04goo00<007ooOol0`7oo000@Ool30006Ool00`00Oomoo`0?Ool00`00Oomoo`0JOol00`00Oomo
o`0BOol00`00Oomoo`31Ool001Uoo`80011oo`03001oogoo01Yoo`03001oogoo019oo`03001oogoo
0<5oo`006Goo00<007ooOol03goo00<007ooOol06Woo00<007ooOol04Goo00<007ooOol0`Woo000I
Ool00`00Oomoo`0?Ool00`00Oomoo`0JOol00`00Oomoo`0AOol00`00Oomoo`32Ool001Uoo`03001o
ogoo00moo`03001oogoo01Yoo`03001oogoo011oo`03001oogoo0<=oo`006Goo00<007ooOol03goo
00<007ooOol06Woo00<007ooOol047oo00<007ooOol0`goo000IOol2000@Ool00`00Oomoo`0JOol0
0`00Oomoo`0?Ool00`00Oomoo`34Ool001Uoo`03001oogoo00ioo`03001oogoo01]oo`03001oogoo
00moo`03001oogoo0<Aoo`006Goo00<007ooOol03Woo00<007ooOol06goo00<007ooOol03Woo00<0
07ooOol0aGoo000IOol00`00Oomoo`0>Ool00`00Oomoo`0KOol00`00Oomoo`0>Ool00`00Oomoo`35
Ool001Uoo`03001oogoo00eoo`03001oogoo01aoo`03001oogoo00eoo`03001oogoo0<Ioo`006Goo
00<007ooOol03Goo00<007ooOol077oo00<007ooOol03Goo00<007ooOol0aWoo000IOol2000>Ool0
0`00Oomoo`0LOol00`00Oomoo`0<Ool00`00Oomoo`37Ool001Uoo`03001oogoo00aoo`03001oogoo
01eoo`03001oogoo00]oo`03001oogoo0<Qoo`006Goo00<007ooOol037oo00<007ooOol07Goo00<0
07ooOol02goo00<007ooOol0b7oo000IOol00`00Oomoo`0<Ool00`00Oomoo`0MOol00`00Oomoo`0:
Ool00`00Oomoo`39Ool001Uoo`03001oogoo00aoo`03001oogoo01ioo`03001oogoo00Uoo`03001o
ogoo0<Uoo`006Goo0P003Goo00<007ooOol07Woo00<007ooOol027oo00<007ooOol0bWoo000IOol0
0`00Oomoo`0<Ool00`00Oomoo`0NOol00`00Oomoo`08Ool00`00Oomoo`3:Ool001Uoo`03001oogoo
00aoo`03001oogoo01ioo`03001oogoo00Qoo`03001oogoo0<Yoo`0000Aoo`00000000Ioo`8000Eo
o`@000Aoo`03001oogoo00aoo`03001oogoo01ioo`03001oogoo00Moo`03001oogoo0<]oo`0000D0
07ooOomoo`0000Eoo`8000Uoo`05001oogooOol0000>Ool00`00Oomoo`0NOol00`00Oomoo`07Ool0
0`00Oomoo`3;Ool00005001oogooOol0000@Ool01@00Oomoogoo00003Goo00<007ooOol07goo00<0
07ooOol01Woo00<007ooOol0c7oo00001@00Oomoogoo000047oo00@007ooOomoo`<000]oo`03001o
ogoo01moo`03001oogoo00Ioo`03001oogoo0<aoo`0000Aoo`00000000eoo`@000Aoo`03001oogoo
00]oo`03001oogoo01moo`03001oogoo00Eoo`03001oogoo0<eoo`0000D007ooOomoo`0000aoo`03
001oogoo00Eoo`03001oogoo00]oo`03001oogoo01moo`03001oogoo00Eoo`03001oogoo0<eoo`00
00D007ooOomoo`0000aoo`03001oogoo00Eoo`03001oogoo00]oo`03001oogoo01moo`03001oogoo
00Aoo`03001oogoo0<ioo`0000Aoo`00000000eoo`@000Aoo`03001oogoo00]oo`03001oogoo01mo
o`03001oogoo00Aoo`03001oogoo0<ioo`006Goo0P0037oo00<007ooOol07goo00<007ooOol00goo
00<007ooOol0cgoo000IOol00`00Oomoo`0;Ool00`00Oomoo`0OOol00`00Oomoo`03Ool00`00Oomo
o`3?Ool001Uoo`03001oogoo00]oo`03001oogoo01moo`03001oogoo00=oo`03001oogoo0<moo`00
6Goo00<007ooOol02goo00<007ooOol07goo00<007ooOol00Woo00<007ooOol0d7oo000IOol00`00
Oomoo`0;Ool00`00Oomoo`0OOol00`00Oomoo`02Ool00`00Oomoo`3@Ool001Uoo`03001oogoo00]o
o`03001oogoo01moo`05001oogooOol0003COol001Uoo`8000aoo`03001oogoo01moo`05001oogoo
Ool0003COol001Uoo`03001oogoo00]oo`03001oogoo01moo`04001oogoo003DOol001Uoo`03001o
ogoo00]oo`03001oogoo01moo`04001oogoo003DOol001Uoo`03001oogoo00]oo`03001oogoo01mo
o`03001oo`000=Eoo`006Goo00<007ooOol02goo00<007ooOol07goo00<007oo0000eGoo000IOol2
000;Ool00`00Oomoo`0POol00`00Ool0003EOol001Uoo`03001oogoo00Yoo`03001oogoo021oo`80
0=Ioo`006Goo00<007ooOol02Woo00<007ooOol087oo0P00eWoo000IOol00`00Oomoo`0:Ool00`00
Oomoo`0POol2003FOol001Uoo`03001oogoo00Yoo`03001oogoo021oo`03001oogoo0=Eoo`006Goo
00<007ooOol02Woo00<007ooOol087oo00<007ooOol0eGoo000IOol2000;Ool00`00Oomoo`3hOol0
01Uoo`03001oogoo00Yoo`03001oogoo0?Qoo`006Goo00<007ooOol02Woo00<007ooOol0n7oo000?
Ool30007Ool00`00Oomoo`0:Ool00`00Oomoo`3hOol0019oo`03001oogoo00Aoo`03001oogoo00Yo
o`03001oogoo0?Qoo`004goo00<007ooOol00goo0`002Woo00<007ooOol0n7oo000@Ool40005Ool0
0`00Oomoo`09Ool00`00Oomoo`3iOol000moo`05001oogooOol00005Ool00`00Oomoo`09Ool00`00
Oomoo`3iOol000moo`05001oogooOol00005Ool00`00Oomoo`09Ool00`00Oomoo`3iOol000moo`05
001oogooOol00005Ool00`00Oomoo`09Ool00`00Oomoo`3iOol0011oo`<000Ioo`03001oogoo00Uo
o`03001oogoo0?Uoo`006Goo0P002Goo00<007ooOol0nWoo000IOol00`00Oomoo`08Ool00`00Oomo
o`3jOol001Uoo`03001oogoo00Qoo`03001oogoo0?Yoo`006Goo00<007ooOol027oo00<007ooOol0
nWoo000IOol00`00Oomoo`08Ool00`00Oomoo`3jOol001Uoo`8000Qoo`03001oogoo0?]oo`006Goo
00<007ooOol01goo00<007ooOol0ngoo000IOol00`00Oomoo`07Ool00`00Oomoo`3kOol001Uoo`03
001oogoo00Moo`03001oogoo0?]oo`006Goo00<007ooOol01goo00<007ooOol0ngoo000IOol00`00
Oomoo`07Ool00`00Oomoo`3kOol001Uoo`8000Qoo`03001oogoo0?]oo`006Goo00<007ooOol01goo
00<007ooOol0ngoo000IOol00`00Oomoo`07Ool00`00Oomoo`3kOol001Uoo`03001oogoo00Moo`03
001oogoo0?]oo`006Goo00<007ooOol01goo00<007ooOol0ngoo000IOol20008Ool00`00Oomoo`3k
Ool001Uoo`03001oogoo00Moo`03001oogoo0?]oo`006Goo00<007ooOol01goo00<007ooOol0ngoo
00030007Ool20005Ool40004Ool00`00Oomoo`07Ool00`00Oomoo`3kOol000=oo`03001oogoo00Ao
o`8000Uoo`05001oogooOol00009Ool00`00Oomoo`3kOol000Aoo`03001oogoo00ioo`05001oogoo
Ool00009Ool00`00Oomoo`3kOol00003Ool000000080011oo`04001oogooOol30007Ool00`00Oomo
o`3kOol00005001oogooOol0000<Ool40004Ool00`00Oomoo`07Ool00`00Oomoo`3kOol00005001o
ogooOol0000<Ool00`00Oomoo`3oOol>Ool00005001oogooOol0000<Ool00`00Oomoo`3oOol>Ool0
0004Ool00000000=Ool4003oOol=Ool00?moob9oo`00ogoo8Woo003oOolROol00?moob9oo`00ogoo
8Woo003oOolROol00001\
\>"],
  ImageRangeCache->{{{0, 288.438}, {177.812, 0}} -> {-10.2633, 6.47071, 
  0.497244, 0.0226573}}]
}, Open  ]],

Cell["\<\
This illustrates one important advantage of the CRR model - the \
number of time slices in the tree can be tuned to accommodate precisely one \
constant Barrier. With more complex geometries there is a good case for \
abandoning simple binomial trees and moving to trinomial systems with a \
general price-step, or, better still, to finite-difference models with grids \
adapted to the boundaries. The reader may wish to adapt the finite-difference \
models described in Chapters 15 and 16 to see how well these models behave \
when the boundary of the grid is set to the location of barriers. There are \
of course additional complications in the presence of non-flat interest-rate \
term structures, but that is beyond the scope of this book. \
\>", "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  PageBreakWithin->True],

Cell[TextData[StyleBox[
"The main point is to appreciate that with \"standard\" trees, there may be a \
severe mismatch between where the nodes go and any special locations induced \
by the option contract. If this is ignored, the values are not trustworthy. \
In simple vanilla cases, the best course of action is to pick a scheme where \
the nodes can be arranged to straddle the strike, using the methods we have \
described. With Barriers, the Boyle-Lau method can be used in some cases.",
  FontVariations->{"CompatibilityType"->0}]], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  PageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 19 Bibliography", "Section",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Benninga, S., Steinmetz, R., and Stroughair, J., 1993, Implementing \
numerical option pricing models, ",
  StyleBox["Mathematica Journal",
    FontSlant->"Italic"],
  ", 3 issue 4, p. 66, 1993.\n\nBoyle, P.P. and Lau, S.H., 1994, Bumping up \
against the barrier with the binomial method, ",
  StyleBox["Journal of Derivatives",
    FontSlant->"Italic"],
  ", Summer.\n\nCox, J., Ross, S. and Rubenstein, M., 1979, Option pricing: a \
simplified approach. ",
  StyleBox["Journal of Financial Economics,",
    FontSlant->"Italic"],
  " 7, p. 229. \n\nHull, J., 1996, ",
  StyleBox["Options, Futures and Other Derivatives",
    FontSlant->"Italic"],
  ", 3rd edition, Prentice-Hall.\n\nKat, H. and Verdonk, L., 1995, Tree \
surgery, ",
  StyleBox["RISK Magazine",
    FontSlant->"Italic"],
  ", February.\n\nJarrow, R.A. and Rudd, A., 1983, ",
  StyleBox["Option Pricing, ",
    FontSlant->"Italic"],
  " Irwin."
}], "Text",
  CellMargins->{{6.875, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{713, 698},
WindowMargins->{{4, Automatic}, {Automatic, 2}},
PrintingCopies->1,
PrintingStartingPageNumber->363,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"19. Trees - Implementation and Pathology"}]]}, {Cell[ 
        TextData[ {"19. Trees - Implementation and Pathology"}]], Inherited, 
      Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True,
"FirstPageFace"->Right},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8`?n3ohP==PTh0fL5N`?P0080004/0B`000006G<AP001
0000I00000400@4100800BL?00400@00000000000000064X01T1T000000@0000
00000000007oo`00003oo`000mP:o`00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 310, 10, 174, "Title"],
Cell[2044, 63, 165, 4, 47, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2234, 71, 97, 1, 50, "Section"],
Cell[2334, 74, 2001, 36, 292, "Text"],
Cell[4338, 112, 150, 3, 26, "Input",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[4525, 120, 233, 8, 50, "Section"],
Cell[4761, 130, 668, 13, 79, "Text"],
Cell[5432, 145, 417, 11, 90, "Input"],
Cell[5852, 158, 205, 4, 29, "Text"],
Cell[6060, 164, 251, 6, 36, "Input"],
Cell[6314, 172, 102, 1, 20, "Text"],
Cell[6419, 175, 162, 3, 25, "Input"],
Cell[6584, 180, 383, 11, 29, "Text"],
Cell[6970, 193, 233, 5, 36, "Input"],
Cell[7206, 200, 364, 6, 40, "Text"],
Cell[7573, 208, 425, 11, 80, "Input"],
Cell[8001, 221, 436, 12, 80, "Input"],
Cell[8440, 235, 167, 4, 20, "Text"],
Cell[8610, 241, 577, 11, 101, "Input"],
Cell[9190, 254, 1008, 19, 84, "Text"],
Cell[10201, 275, 831, 22, 212, "Input"],
Cell[11035, 299, 128, 1, 20, "Text"],
Cell[11166, 302, 867, 24, 212, "Input",
  PageBreakWithin->True],
Cell[12036, 328, 129, 1, 20, "Text"],
Cell[12168, 331, 545, 11, 101, "Input"],

Cell[CellGroupData[{
Cell[12738, 346, 130, 1, 32, "Subsection"],
Cell[12871, 349, 123, 1, 20, "Text"],
Cell[12997, 352, 132, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[13154, 358, 340, 5, 62, "Input"],
Cell[13497, 365, 137, 2, 36, "Output"]
}, Open  ]],
Cell[13649, 370, 381, 7, 40, "Text"],
Cell[14033, 379, 575, 11, 112, "Input"],
Cell[14611, 392, 197, 4, 29, "Text"],
Cell[14811, 398, 675, 17, 112, "Input"],
Cell[15489, 417, 571, 13, 112, "Input"],
Cell[16063, 432, 187, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[16275, 440, 211, 3, 37, "Input"],
Cell[16489, 445, 35094, 1140, 112, 12703, 858, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[51598, 1588, 122, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[51745, 1593, 244, 4, 37, "Input"],
Cell[51992, 1599, 34747, 1118, 112, 12484, 837, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[86754, 2720, 131, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[86910, 2725, 210, 3, 37, "Input"],
Cell[87123, 2730, 33503, 1110, 112, 12473, 845, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[120641, 3843, 119, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[120785, 3848, 213, 3, 37, "Input"],
Cell[121001, 3853, 31863, 1106, 112, 12538, 862, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[152879, 4962, 905, 16, 106, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[153821, 4983, 126, 1, 32, "Subsection"],
Cell[153950, 4986, 650, 12, 62, "Text"],
Cell[154603, 5000, 175, 3, 36, "NumberedEquation"],
Cell[154781, 5005, 133, 1, 20, "Text"],
Cell[154917, 5008, 187, 3, 40, "NumberedEquation"],
Cell[155107, 5013, 175, 5, 20, "Text"],
Cell[155285, 5020, 292, 5, 38, "Input"],
Cell[155580, 5027, 209, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[155814, 5035, 143, 2, 25, "Input"],
Cell[155960, 5039, 275, 4, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[156272, 5048, 148, 2, 25, "Input"],
Cell[156423, 5052, 273, 4, 48, "Output"]
}, Open  ]],
Cell[156711, 5059, 601, 11, 62, "Text"],
Cell[157315, 5072, 183, 3, 40, "NumberedEquation"],
Cell[157501, 5077, 220, 6, 20, "Text"],
Cell[157724, 5085, 289, 6, 37, "Input"],

Cell[CellGroupData[{
Cell[158038, 5095, 107, 2, 25, "Input"],
Cell[158148, 5099, 617, 9, 85, "Output"]
}, Open  ]],
Cell[158780, 5111, 115, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[158920, 5116, 143, 2, 25, "Input"],
Cell[159066, 5120, 55, 1, 36, "Output"]
}, Open  ]],
Cell[159136, 5124, 255, 5, 29, "Text"],
Cell[159394, 5131, 238, 5, 37, "Input"],

Cell[CellGroupData[{
Cell[159657, 5140, 146, 2, 25, "Input"],
Cell[159806, 5144, 37950, 948, 112, 8204, 575, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[197771, 6095, 234, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[198030, 6104, 174, 3, 25, "Input"],
Cell[198207, 6109, 54, 1, 36, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[198322, 6117, 232, 8, 38, "Section"],
Cell[198557, 6127, 256, 6, 29, "Text"],
Cell[198816, 6135, 176, 5, 36, "Input"],
Cell[198995, 6142, 198, 4, 29, "Text"],
Cell[199196, 6148, 293, 7, 57, "Input"],
Cell[199492, 6157, 177, 4, 20, "Text"],
Cell[199672, 6163, 299, 7, 36, "Input"],
Cell[199974, 6172, 384, 8, 40, "Text"],
Cell[200361, 6182, 352, 9, 57, "Input"],
Cell[200716, 6193, 365, 9, 57, "Input"],
Cell[201084, 6204, 129, 1, 20, "Text"],
Cell[201216, 6207, 590, 12, 112, "Input"],
Cell[201809, 6221, 764, 13, 73, "Text"],
Cell[202576, 6236, 812, 21, 200, "Input"],
Cell[203391, 6259, 821, 21, 200, "Input"],
Cell[204215, 6282, 134, 1, 20, "Text"],
Cell[204352, 6285, 537, 11, 101, "Input"],

Cell[CellGroupData[{
Cell[204914, 6300, 129, 1, 32, "Subsection"],
Cell[205046, 6303, 183, 4, 20, "Text"],
Cell[205232, 6309, 665, 17, 112, "Input"],
Cell[205900, 6328, 563, 13, 112, "Input"],

Cell[CellGroupData[{
Cell[206488, 6345, 208, 3, 37, "Input"],
Cell[206699, 6350, 34330, 1111, 112, 12510, 836, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[241066, 7466, 243, 4, 37, "Input"],
Cell[241312, 7472, 34649, 1106, 112, 12439, 826, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[275998, 8583, 209, 3, 37, "Input"],
Cell[276210, 8588, 32805, 1074, 112, 12229, 815, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[309052, 9667, 210, 3, 37, "Input"],
Cell[309265, 9672, 32862, 1075, 112, 12233, 815, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[342142, 10750, 216, 6, 52, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[342395, 10761, 125, 1, 32, "Subsection"],
Cell[342523, 10764, 189, 4, 20, "Text"],
Cell[342715, 10770, 218, 4, 44, "NumberedEquation"],
Cell[342936, 10776, 179, 6, 20, "Text"],
Cell[343118, 10784, 271, 5, 37, "Input"],
Cell[343392, 10791, 209, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[343626, 10799, 162, 3, 25, "Input"],
Cell[343791, 10804, 272, 4, 48, "Output"]
}, Open  ]],
Cell[344078, 10811, 643, 11, 62, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[344770, 10828, 244, 8, 38, "Section"],
Cell[345017, 10838, 285, 7, 29, "Text"],
Cell[345305, 10847, 341, 6, 52, "Input"],
Cell[345649, 10855, 168, 4, 20, "Text"],
Cell[345820, 10861, 347, 9, 57, "Input"],
Cell[346170, 10872, 268, 7, 36, "Input"],
Cell[346441, 10881, 159, 3, 20, "Text"],
Cell[346603, 10886, 422, 11, 80, "Input"],
Cell[347028, 10899, 431, 11, 80, "Input"],
Cell[347462, 10912, 588, 12, 101, "Input"],
Cell[348053, 10926, 205, 4, 29, "Text"],
Cell[348261, 10932, 848, 22, 212, "Input"],
Cell[349112, 10956, 882, 23, 212, "Input",
  PageBreakWithin->True],
Cell[349997, 10981, 533, 11, 101, "Input"],

Cell[CellGroupData[{
Cell[350555, 10996, 135, 1, 32, "Subsection"],
Cell[350693, 10999, 181, 4, 20, "Text"],
Cell[350877, 11005, 366, 9, 62, "Input"],
Cell[351246, 11016, 383, 9, 76, "Input"],
Cell[351632, 11027, 319, 7, 62, "Input"],
Cell[351954, 11036, 331, 7, 62, "Input"],

Cell[CellGroupData[{
Cell[352310, 11047, 239, 4, 37, "Input"],
Cell[352552, 11053, 34008, 1121, 112, 12648, 851, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[386597, 12179, 212, 3, 37, "Input"],
Cell[386812, 12184, 34213, 1123, 112, 12636, 851, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[421062, 13312, 208, 3, 37, "Input"],
Cell[421273, 13317, 32927, 1094, 112, 12363, 835, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[454237, 14416, 211, 3, 37, "Input"],
Cell[454451, 14421, 32857, 1095, 112, 12366, 837, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[487357, 15522, 137, 1, 32, "Subsection"],
Cell[487497, 15525, 187, 4, 20, "Text"],
Cell[487687, 15531, 199, 3, 44, "NumberedEquation"],
Cell[487889, 15536, 180, 6, 20, "Text"],
Cell[488072, 15544, 254, 4, 37, "Input"],
Cell[488329, 15550, 209, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[488563, 15558, 162, 3, 25, "Input"],
Cell[488728, 15563, 273, 4, 48, "Output"]
}, Open  ]],
Cell[489016, 15570, 2011, 38, 252, "Text",
  PageBreakAbove->False,
  PageBreakWithin->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[491076, 15614, 153, 2, 38, "Section",
  PageBreakAbove->True],
Cell[491232, 15618, 1769, 56, 150, "Text"],
Cell[493004, 15676, 173, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[493202, 15686, 206, 4, 37, "Input"],
Cell[493411, 15692, 575, 11, 61, "Output"]
}, Open  ]],
Cell[494001, 15706, 179, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[494205, 15716, 209, 4, 37, "Input"],
Cell[494417, 15722, 574, 11, 61, "Output"]
}, Open  ]],
Cell[495006, 15736, 172, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[495203, 15746, 207, 4, 37, "Input"],
Cell[495413, 15752, 572, 11, 61, "Output"]
}, Open  ]],
Cell[496000, 15766, 178, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[496203, 15776, 210, 4, 37, "Input"],
Cell[496416, 15782, 600, 12, 61, "Output",
  PageBreakAbove->False]
}, Open  ]],
Cell[497031, 15797, 197, 7, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[497253, 15808, 232, 5, 37, "Input",
  PageBreakAbove->False],
Cell[497488, 15815, 598, 12, 61, "Output",
  PageBreakAbove->False]
}, Open  ]],
Cell[498101, 15830, 203, 7, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[498329, 15841, 233, 5, 37, "Input",
  PageBreakAbove->False],
Cell[498565, 15848, 598, 12, 61, "Output",
  PageBreakAbove->False]
}, Open  ]],
Cell[499178, 15863, 733, 12, 73, "Text",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[499948, 15880, 139, 2, 38, "Section",
  PageBreakAbove->False],
Cell[500090, 15884, 1224, 21, 140, "Text",
  PageBreakWithin->True],

Cell[CellGroupData[{
Cell[501339, 15909, 113, 2, 25, "Input"],
Cell[501455, 15913, 117, 2, 20, "Print"],
Cell[501575, 15917, 808, 17, 69, "Print"]
}, Open  ]],
Cell[502398, 15937, 104, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[502527, 15942, 149, 2, 25, "Input"],
Cell[502679, 15946, 112, 2, 36, "Output"]
}, Open  ]],
Cell[502806, 15951, 313, 6, 29, "Text"],
Cell[503122, 15959, 919, 25, 244, "Input"],
Cell[504044, 15986, 271, 5, 37, "Input"],
Cell[504318, 15993, 194, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[504537, 16001, 221, 3, 50, "Input"],
Cell[504761, 16006, 335, 5, 48, "Output"]
}, Open  ]],
Cell[505111, 16014, 278, 7, 29, "Text"],

Cell[CellGroupData[{
Cell[505414, 16025, 190, 3, 25, "Input"],
Cell[505607, 16030, 112, 2, 36, "Output"]
}, Open  ]],
Cell[505734, 16035, 200, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[505959, 16045, 190, 3, 25, "Input"],
Cell[506152, 16050, 112, 2, 36, "Output"]
}, Open  ]],
Cell[506279, 16055, 160, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[506464, 16063, 190, 3, 25, "Input"],
Cell[506657, 16068, 112, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[506806, 16075, 190, 3, 25, "Input"],
Cell[506999, 16080, 112, 2, 36, "Output"]
}, Open  ]],
Cell[507126, 16085, 176, 4, 20, "Text"],
Cell[507305, 16091, 305, 5, 62, "Input"],
Cell[507613, 16098, 206, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[507844, 16106, 144, 2, 25, "Input"],
Cell[507991, 16110, 12659, 427, 128, 4642, 323, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[520665, 16540, 681, 12, 62, "Text"],
Cell[521349, 16554, 171, 3, 36, "NumberedEquation"],
Cell[521523, 16559, 251, 5, 29, "Text"],
Cell[521777, 16566, 188, 3, 25, "Input"],

Cell[CellGroupData[{
Cell[521990, 16573, 156, 3, 25, "Input"],
Cell[522149, 16578, 57, 1, 36, "Output"]
}, Open  ]],
Cell[522221, 16582, 384, 8, 40, "Text"],
Cell[522608, 16592, 923, 24, 224, "Input",
  PageBreakWithin->True],
Cell[523534, 16618, 269, 5, 32, "Input"],
Cell[523806, 16625, 246, 4, 20, "Input"],
Cell[524055, 16631, 344, 7, 36, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[524424, 16642, 168, 3, 20, "Input",
  PageBreakAbove->False],
Cell[524595, 16647, 18961, 507, 152, 4674, 325, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]],
Cell[543571, 17157, 884, 14, 92, "Text",
  PageBreakAbove->Automatic,
  PageBreakWithin->True],
Cell[544458, 17173, 662, 10, 64, "Text",
  PageBreakAbove->Automatic,
  PageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[545157, 17188, 103, 1, 44, "Section"],
Cell[545263, 17191, 1003, 25, 210, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

