(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    339739,       9709]*)
(*NotebookOutlinePosition[    341560,       9764]*)
(*  CellTagsIndexPosition[    341516,       9760]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 21. 
Monte Carlo and Wozniakowski 
Sampling\
\>", "Title",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["Random and Pseudo-random Simulation of Paths and Events", "Subsubtitle",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["21.1 Introduction", "Section",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "\nThis chapter presents a sketch of the foundations of Monte Carlo \
sampling within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". We take it as a given that the values of options are given by the \
discounted (present) value of the expectation of the payoff under the \
risk-neutral distribution, as discussed in Chapter 3, and in detail by Baxter \
and Rennie (1996).  One of our goals here is to explore the computations of \
such expectations. \n\nMonte Carlo methods appear to be most useful for the \
computation of option values when the following conditions are met:\n\n(a) \
there is no analytical characterization;\n(b) the option is European in \
style, in the sense that early exercise is prohibited;\n(c) the option \
involves path-dependent features, and/or many assets.\n\nPath-dependent \
options may also be valued by the PDE approach, by suitably augmenting the \
variables to include a counter for the path-dependent features (see e.g. \
Wilmott ",
  StyleBox["et al",
    FontSlant->"Italic"],
  " 1993), but the additional complexity that this involves appears, to this \
author at least, not to be worthwhile unless there are American features, and \
is certainly computationally infeasible when there are several assets \
involved. Recent work also aims to extend the Monte Carlo approach to \
American-style options (see e.g. Broadie and Glasserman, 1996) but again at \
the price of a considerable increase in complexity. The view taken here is \
that Monte Carlo (abbreviated here on as MC) is technique of choice when \
conditions (a), (b), (c) above apply.\n\nWe shall consider the computation of \
hedge parameters by simulation, and we shall also give a brief introduction \
to the use of ",
  StyleBox["non-random",
    FontSlant->"Italic"],
  " samples for performing probabilistic integrals in option pricing."
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["21.2 Sampling the Normal Distribution", "Section",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "At the heart of the simulation method is the simulation of random numbers \
that are normally distributed. Computer programming systems usually come with \
a built-in random number generator that does some distribution or other, and \
we are required to use this to get the required normally distributed samples. \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is no exception, but it turns out that the most obvious default \
approaches are not very useful. To make the point, we shall play with random \
numbers for a little while, and shall begin by looking at the ",
  StyleBox["Random",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function."
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Random\)\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("Random[ ] gives a uniformly distributed pseudorandom Real in the range \
0 to 1. Random[type, range] gives a pseudorandom number of the specified \
type, lying in the specified range. Possible types are: Integer, Real and \
Complex. The default range is 0 to 1. You can give the range {min, max} \
explicitly; a range specification of max is equivalent to {0, max}."\)], 
  "Print"]
}, Open  ]],

Cell[TextData[{
  "Great! So we can get the uniform distribution really easily. How do we \
turn this into samples from the normal distribution? This is where matters \
become rather more interesting. A ",
  StyleBox["little",
    FontSlant->"Italic"],
  " knowledge can be rather counter-productive here. The elementary theory of \
probability tells us that if a (general) distribution has density function ",
  Cell[BoxData[
      \(TraditionalForm\`f(x)\)]],
  " and distribution function ",
  Cell[BoxData[
      \(TraditionalForm\`F(x)\)]],
  ", with ",
  Cell[BoxData[
      \(TraditionalForm\`f(x)\  = \ F' \((x)\)\)]],
  ", then the variable"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`y = F(x)\)], "NumberedEquation",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "is uniformly distributed on ",
  Cell[BoxData[
      \(TraditionalForm\`\([0, 1]\)\)]],
  ". So given a uniform distribution, to generate samples from such a \
distribution, we can simply invert this relationship, and set, for ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " uniformly distributed on ",
  Cell[BoxData[
      \(TraditionalForm\`\([0, 1]\)\)]],
  ", "
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`z = \(F\^\(-1\)\)(x)\)], "NumberedEquation",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
For the normal distribution, the cumulative distribution is related \
to the error function, and the required inverse mapping is\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Convert[x_]\  := \ N[Sqrt[2]*InverseErf[2*x - 1]]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \({Convert[0.025], \ Convert[0.975]}\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\(-1.9599639845400528`\), 1.9599639845400528`}\)], "Output"]
}, Open  ]],

Cell["\<\
In order to analyse this and other distributions, it is useful to \
have the statistics and graphics Packages to hand, so we load them, and make \
some samples:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Statistics`Master`\>"]; \ 
    Needs["\<Graphics`Graphics`\>"]; \)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(ndataone\  = \ 
      Map[\((Convert[#] &)\), \ Table[Random[], \ {1000}]]; \)\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Our point will be that this takes too long, as you will have \
worried if you execute this in a Notebook. For the record, on a 266 MHz (G3) \
Power Mac:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Map[\((Convert[#] &)\), \ Table[Random[], \ {1000}]]; ]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({3.18333333333333001`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell["Let's take a look at the distribution as a histogram:", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\($DefaultFont\  = \ {"\<Courier\>", \ 7}; \)\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(BarChart[BinCounts[ndataone, \ {\(-3.5\), \ 3.5, \ 0.5}], \ 
      PlotRange\  -> \ All, \n
      BarLabels\  -> 
        Map[\ ToString, \ 
          Table[\(-3.25\)\  + \ 0.5*k, \ {k, \ 0, \ 13}]]]\ ; \)\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 7  scalefont  setfont
% Scaling calculations
-0.0175983 0.0690131 0.0147151 0.00297275 [
[.05141 .00222 -12.4688 -9 ]
[.05141 .00222 12.4688 0 ]
[.12043 .00222 -12.4688 -9 ]
[.12043 .00222 12.4688 0 ]
[.18944 .00222 -12.4688 -9 ]
[.18944 .00222 12.4688 0 ]
[.25845 .00222 -12.4688 -9 ]
[.25845 .00222 12.4688 0 ]
[.32747 .00222 -12.4688 -9 ]
[.32747 .00222 12.4688 0 ]
[.39648 .00222 -12.4688 -9 ]
[.39648 .00222 12.4688 0 ]
[.46549 .00222 -12.4688 -9 ]
[.46549 .00222 12.4688 0 ]
[.53451 .00222 -10.375 -9 ]
[.53451 .00222 10.375 0 ]
[.60352 .00222 -10.375 -9 ]
[.60352 .00222 10.375 0 ]
[.67253 .00222 -10.375 -9 ]
[.67253 .00222 10.375 0 ]
[.74155 .00222 -10.375 -9 ]
[.74155 .00222 10.375 0 ]
[.81056 .00222 -10.375 -9 ]
[.81056 .00222 10.375 0 ]
[.87957 .00222 -10.375 -9 ]
[.87957 .00222 10.375 0 ]
[.94859 .00222 -10.375 -9 ]
[.94859 .00222 10.375 0 ]
[-0.00249 .16335 -8.4 -3.15 ]
[-0.00249 .16335 0 3.15 ]
[-0.00249 .31199 -12.6 -3.15 ]
[-0.00249 .31199 0 3.15 ]
[-0.00249 .46063 -12.6 -3.15 ]
[-0.00249 .46063 0 3.15 ]
[-0.00249 .60926 -12.6 -3.15 ]
[-0.00249 .60926 0 3.15 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.05141 .01472 m
.05141 .02097 L
s
gsave
.05141 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.25) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.12043 .01472 m
.12043 .02097 L
s
gsave
.12043 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.75) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.18944 .01472 m
.18944 .02097 L
s
gsave
.18944 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.25) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.25845 .01472 m
.25845 .02097 L
s
gsave
.25845 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.75) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.32747 .01472 m
.32747 .02097 L
s
gsave
.32747 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.25) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.39648 .01472 m
.39648 .02097 L
s
gsave
.39648 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.75) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.46549 .01472 m
.46549 .02097 L
s
gsave
.46549 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.25) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.53451 .01472 m
.53451 .02097 L
s
gsave
.53451 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.25) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.60352 .01472 m
.60352 .02097 L
s
gsave
.60352 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.75) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.67253 .01472 m
.67253 .02097 L
s
gsave
.67253 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.25) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.74155 .01472 m
.74155 .02097 L
s
gsave
.74155 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.75) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.81056 .01472 m
.81056 .02097 L
s
gsave
.81056 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.25) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.87957 .01472 m
.87957 .02097 L
s
gsave
.87957 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.75) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.94859 .01472 m
.94859 .02097 L
s
gsave
.94859 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.25) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
0 .01472 m
1 .01472 L
s
.01001 .16335 m
.01626 .16335 L
s
[(50)] -0.00249 .16335 1 0 Mshowa
.01001 .31199 m
.01626 .31199 L
s
[(100)] -0.00249 .31199 1 0 Mshowa
.01001 .46063 m
.01626 .46063 L
s
[(150)] -0.00249 .46063 1 0 Mshowa
.01001 .60926 m
.01626 .60926 L
s
[(200)] -0.00249 .60926 1 0 Mshowa
.125 Mabswid
.01001 .04444 m
.01376 .04444 L
s
.01001 .07417 m
.01376 .07417 L
s
.01001 .1039 m
.01376 .1039 L
s
.01001 .13362 m
.01376 .13362 L
s
.01001 .19308 m
.01376 .19308 L
s
.01001 .22281 m
.01376 .22281 L
s
.01001 .25253 m
.01376 .25253 L
s
.01001 .28226 m
.01376 .28226 L
s
.01001 .34172 m
.01376 .34172 L
s
.01001 .37144 m
.01376 .37144 L
s
.01001 .40117 m
.01376 .40117 L
s
.01001 .4309 m
.01376 .4309 L
s
.01001 .49035 m
.01376 .49035 L
s
.01001 .52008 m
.01376 .52008 L
s
.01001 .54981 m
.01376 .54981 L
s
.01001 .57954 m
.01376 .57954 L
s
.25 Mabswid
.01001 0 m
.01001 .61803 L
s
1 0 0 r
.02381 .01472 m
.02381 .01472 L
.07902 .01472 L
.07902 .01472 L
F
0 g
.5 Mabswid
.02381 .01472 m
.02381 .01472 L
.07902 .01472 L
.07902 .01472 L
.02381 .01472 L
s
1 0 0 r
.09282 .01472 m
.09282 .02661 L
.14803 .02661 L
.14803 .01472 L
F
0 g
.09282 .01472 m
.09282 .02661 L
.14803 .02661 L
.14803 .01472 L
.09282 .01472 L
s
1 0 0 r
.16184 .01472 m
.16184 .05336 L
.21705 .05336 L
.21705 .01472 L
F
0 g
.16184 .01472 m
.16184 .05336 L
.21705 .05336 L
.21705 .01472 L
.16184 .01472 L
s
1 0 0 r
.23085 .01472 m
.23085 .12471 L
.28606 .12471 L
.28606 .01472 L
F
0 g
.23085 .01472 m
.23085 .12471 L
.28606 .12471 L
.28606 .01472 L
.23085 .01472 L
s
1 0 0 r
.29986 .01472 m
.29986 .25253 L
.35507 .25253 L
.35507 .01472 L
F
0 g
.29986 .01472 m
.29986 .25253 L
.35507 .25253 L
.35507 .01472 L
.29986 .01472 L
s
1 0 0 r
.36888 .01472 m
.36888 .53495 L
.42409 .53495 L
.42409 .01472 L
F
0 g
.36888 .01472 m
.36888 .53495 L
.42409 .53495 L
.42409 .01472 L
.36888 .01472 L
s
1 0 0 r
.43789 .01472 m
.43789 .58846 L
.4931 .58846 L
.4931 .01472 L
F
0 g
.43789 .01472 m
.43789 .58846 L
.4931 .58846 L
.4931 .01472 L
.43789 .01472 L
s
1 0 0 r
.5069 .01472 m
.5069 .60332 L
.56211 .60332 L
.56211 .01472 L
F
0 g
.5069 .01472 m
.5069 .60332 L
.56211 .60332 L
.56211 .01472 L
.5069 .01472 L
s
1 0 0 r
.57591 .01472 m
.57591 .42793 L
.63112 .42793 L
.63112 .01472 L
F
0 g
.57591 .01472 m
.57591 .42793 L
.63112 .42793 L
.63112 .01472 L
.57591 .01472 L
s
1 0 0 r
.64493 .01472 m
.64493 .28226 L
.70014 .28226 L
.70014 .01472 L
F
0 g
.64493 .01472 m
.64493 .28226 L
.70014 .28226 L
.70014 .01472 L
.64493 .01472 L
s
1 0 0 r
.71394 .01472 m
.71394 .14552 L
.76915 .14552 L
.76915 .01472 L
F
0 g
.71394 .01472 m
.71394 .14552 L
.76915 .14552 L
.76915 .01472 L
.71394 .01472 L
s
1 0 0 r
.78295 .01472 m
.78295 .06822 L
.83816 .06822 L
.83816 .01472 L
F
0 g
.78295 .01472 m
.78295 .06822 L
.83816 .06822 L
.83816 .01472 L
.78295 .01472 L
s
1 0 0 r
.85197 .01472 m
.85197 .03255 L
.90718 .03255 L
.90718 .01472 L
F
0 g
.85197 .01472 m
.85197 .03255 L
.90718 .03255 L
.90718 .01472 L
.85197 .01472 L
s
1 0 0 r
.92098 .01472 m
.92098 .02066 L
.97619 .02066 L
.97619 .01472 L
F
0 g
.92098 .01472 m
.92098 .02066 L
.97619 .02066 L
.97619 .01472 L
.92098 .01472 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  ImageSize->{323.625, 199.625},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006D0000nA000`40O003h00OogooUGoo003oOonE
Ool00?mooiEoo`00ogooUGoo003oOonEOol00?mooiEoo`00ogooUGoo003oOonEOol001ioo`<000=o
o`8000=oo`@00004Ool000000008Ool40002Ool20004Ool01000Oomoogoo0`001goo10000Woo0P00
0goo100000Aoo`00000000Uoo`<0009oo`8000Aoo`04001oogooOol30008Ool30002Ool20003Ool4
000017oo000000002Goo0P000goo0P0017oo00@007ooOomoo`<000Qoo`8000=oo`8000=oo`@00004
Ool000000006Ool20003Ool20003Ool4000017oo000000002Goo0P000goo0P0017oo00@007ooOomo
o`<000Qoo`<0009oo`8000=oo`@00004Ool000000009Ool30002Ool20004Ool01000Oomoogoo0`00
1goo10000Woo0P000goo100000Aoo`00000000Qoo`@0009oo`8000Aoo`04001oogooOol30008Ool3
0003Ool20003Ool4000017oo0000000037oo000IOol40004Ool00`00Oomoo`020004Ool00`00Oomo
o`04Ool00`00Oomoo`040002Ool00`00Oomoo`02Ool20004Ool00`00Oomoo`04Ool00`00Ool00003
0002Ool00`00Oomoo`02Ool20004Ool00`00Oomoo`04Ool00`00Oomoo`040003Ool01000Oomoogoo
0P0017oo00<007ooOol017oo00<007oo00000`000goo00@007ooOomoo`8000Aoo`03001oogoo00Ao
o`03001oogoo00@00005Ool007ooOol00002Ool20004Ool00`00Oomoo`04Ool00`00Ool000030000
1Goo001oogoo00000Woo0P0017oo00<007ooOol017oo00<007ooOol00Woo00@007ooOol0009oo`80
00Aoo`03001oogoo00Aoo`03001oogoo00Eoo`04001oogoo0002Ool20004Ool00`00Oomoo`04Ool0
0`00Oomoo`06Ool01000Oomoogoo0P0017oo00<007ooOol017oo00<007ooOol01goo00@007ooOomo
o`8000Aoo`03001oogoo00Aoo`03001oogoo00Eoo`03001oogoo009oo`8000Aoo`03001oogoo00Ao
o`03001oogoo00Ioo`03001oogoo009oo`8000Aoo`03001oogoo00Aoo`03001oogoo00Qoo`03001o
ogoo008000Aoo`03001oogoo00Aoo`03001oogoo00Uoo`005Goo00<007ooOol02Goo00<007ooOol0
1goo00<007ooOol00goo00<007ooOol01goo00<007ooOol027oo00<007ooOol00goo00<007ooOol0
1Woo00<007ooOol027oo00<007ooOol00goo00<007ooOol01goo00<007ooOol027oo00<007ooOol0
0goo00<007ooOol01Woo00<007ooOol027oo00<007ooOol00goo00<007ooOol01Goo00@007ooOol0
00Uoo`03001oogoo00=oo`03001oogoo00Aoo`04001oogoo0009Ool00`00Oomoo`03Ool00`00Oomo
o`02Ool01000Oomoo`002Goo00<007ooOol00goo00<007ooOol01Goo00@007ooOol000Uoo`03001o
ogoo00=oo`03001oogoo00Ioo`03001oogoo00Qoo`03001oogoo00=oo`03001oogoo00Moo`03001o
ogoo00Qoo`03001oogoo00=oo`03001oogoo00Ioo`03001oogoo00Qoo`03001oogoo00=oo`03001o
ogoo00Moo`03001oogoo00Qoo`03001oogoo00=oo`03001oogoo00Qoo`03001oogoo00Moo`03001o
ogoo00=oo`03001oogoo00Uoo`005Goo00<007ooOol01goo0P002goo00<007oo00000P002goo00<0
07ooOol01goo00<007ooOol00`002Woo00<007ooOol027oo00<007oo00000P002Woo00<007ooOol0
27oo00<007ooOol00`002Goo00<007ooOol02Goo00<007oo00000P0027oo00@007ooOol000Uoo`03
001oogoo00<000Moo`04001oogoo000:Ool00`00Ool000020005Ool01000Oomoo`002Woo00<007oo
00000P0027oo00@007ooOol000Uoo`03001oogoo00<000Uoo`03001oogoo00Uoo`03001oo`000080
00Yoo`03001oogoo00Qoo`03001oogoo00<000Yoo`03001oogoo00Qoo`03001oo`00008000]oo`03
001oogoo00Moo`03001oogoo00<000Uoo`8000]oo`03001oo`00008000aoo`005Goo00<007ooOol0
2Goo00<007ooOol027oo00<007oo00003Goo00<007ooOol027oo00<007oo000037oo00<007ooOol0
27oo00<007oo000037oo00<007ooOol02Goo00<007oo00002goo00<007ooOol02Goo00<007oo0000
2Woo00@007ooOol000Yoo`03001oo`0000Uoo`04001oogoo000:Ool00`00Ool00007Ool01000Oomo
o`002Woo00<007oo00002Woo00@007ooOol000Yoo`03001oo`0000]oo`03001oogoo00Uoo`03001o
o`0000aoo`03001oogoo00Uoo`03001oo`0000aoo`03001oogoo00Qoo`03001oo`0000eoo`03001o
ogoo00Qoo`03001oo`0000eoo`03001oogoo00Qoo`03001oo`0000ioo`005Goo00<007ooOol01Woo
0`0027oo0`000Woo10001goo0`0027oo100000=oo`0000000P001Woo0`0027oo0`000Woo100027oo
0P0027oo100000=oo`0000000P001goo0P0027oo0`000Woo100027oo0P0027oo100000=oo`000000
0P001goo0P0027oo0`000Woo10001Goo0P0027oo0`000Woo100027oo0P0027oo100000=oo`000000
0P001goo0P0027oo0`000Woo100027oo0P0027oo100000=oo`0000000P001Woo0`0027oo0`000Woo
10001goo0`0027oo100000=oo`0000000P001goo0`0027oo0`000Woo10002goo000EOol00`00Oomo
o`3oOommOol001Eoo`03001oogoo0?moogeoo`004Gooo`00P`000Goo000EOol00`00Oomoo`0=Ool0
0`00Oomoo`0=Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool0
0`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO000
0`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool0
0`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool00`00O01l000BO000
0`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`02Ool0
0`00O01l000CO0000`00Oomoo`08Ool001Eoo`03001oogoo00eoo`03001oogoo00eoo`03001l07`0
019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo
00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0
019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo
009oo`03001l07`001=l0003001oogoo009oo`03001l07`0019l0003001oogoo009oo`03001l07`0
01=l0003001oogoo009oo`03001l07`0019l0003001oogoo009ooaL000Yoo`005Goo00<007ooOol0
7Goo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O000
4W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0
0goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O000
4W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0
0Woo00<007`0O0004g`000<007ooOol00Woo00<007`0O0004W`000<007ooOol08goo000EOol00`00
Oomoo`0MOol00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00
O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00
Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00
O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool00`00O01l000BO0000`00
Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`0SOol001Eo
o`03001oogoo01eooaH000Eoo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001o
ogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l
07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001o
ogoo009oo`03001l07`001=l0003001oogoo009oo`03001l07`0019l0003001oogoo009oo`03001l
07`001=l0003001oogoo009oo`03001l07`0019l0003001oogoo02=oo`005Goo00<007ooOol0>7oo
00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`0
00<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo
00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`0
00<007ooOol00Woo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol00Woo
00<007`0O0004W`000<007ooOol08goo000EOol00`00Oomoo`0hOol00`00O01l000BO0000`00Oomo
o`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l
000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomo
o`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool00`00O01l
000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02OolF000UOol001Eoo`03001oogoo
03Qoo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0
019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo
009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0
01=l0003001oogoo009oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo
03eoo`005Goo00<007ooOol0>7oo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<0
07ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<0
07`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<0
07ooOol00Woo00<007`0O0004g`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00Woo00<0
07`0O0004g`000<007ooOol0?Goo000EOol00`00Oomoo`0hOol00`00O01l000BO0000`00Oomoo`02
Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000B
O0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03
Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool00`00O01l000B
O0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`0mOol001Eoo`<003Qoo`03001l07`0019l
0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009o
o`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l
0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo009o
o`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo03eoo`005Goo00<007oo
Ool0>7oo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0
O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007oo
Ool00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0
O0004g`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007oo
Ool0?Goo000EOol00`00Oomoo`0hOol00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO000
0`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool0
0`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO000
0`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`02Ool0
0`00O01l000CO0000`00Oomoo`0mOol001Eoo`03001oogoo03Qoo`03001l07`0019l0003001oogoo
009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0
019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo
00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo009oo`03001l07`0
019l0003001oogoo009oo`03001l07`001=l0003001oogoo03eoo`005Goo00<007ooOol0>7oo5P00
17oo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O000
4W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0
0goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol00Woo00<007`0O000
4W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0?Goo000EOol00`00Oomoo`1BOol00`00
O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00
Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00
O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool00`00O01l000BO0000`00
Oomoo`02Ool00`00O01l000CO0000`00Oomoo`0mOol001Eoo`03001oogoo059oo`03001l07`0019l
0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=o
o`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l
0003001oogoo009oo`03001l07`001=l0003001oogoo009oo`03001l07`0019l0003001oogoo009o
o`03001l07`001=l0003001oogoo03eoo`005Goo00<007ooOol0DWoo00<007`0O0004W`000<007oo
Ool00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0
O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007oo
Ool00Woo00<007`0O0004g`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00Woo00<007`0
O0004g`000<007ooOol0?Goo000EOol00`00Oomoo`1BOol00`00O01l000BO0000`00Oomoo`03Ool0
0`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO000
0`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool0
0`00O01l000CO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO000
0`00Oomoo`0mOol001Eoo`03001oogoo059oo`03001l07`0019l0003001oogoo00=oo`03001l07`0
019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo
009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0
01=l0003001oogoo009oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo
03eoo`005Goo00<007ooOol0DWoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<0
07ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<0
07`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<0
07ooOol00Woo00<007`0O0004W`000<007ooOol00Woo5`00?goo000EOol00`00Oomoo`1BOol00`00
O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00
Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00
O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool00`00O01l000BO0000`00
Oomoo`1HOol001Eoo`<0059oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001o
ogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l
07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001o
ogoo009oo`03001l07`0019l0003001oogoo05Qoo`005Goo00<007ooOol0DWoo00<007`0O0004W`0
00<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo
00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`0
00<007ooOol00Woo00<007`0O0004g`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0F7oo
000EOol00`00Oomoo`1BOol00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomo
o`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l
000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomo
o`02Ool00`00O01l000BO0000`00Oomoo`1HOol001Eoo`03001oogoo059oo`03001l07`0019l0003
001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03
001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003
001oogoo009oo`03001l07`001=l0003001oogoo009oo`03001l07`0019l0003001oogoo05Qoo`00
5Goo00<007ooOol0DWoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol0
0Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O000
4W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0
0Woo00<007`0O0004W`000<007ooOol0F7oo000EOol00`00Oomoo`1BOol00`00O01l000BO0000`00
Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00
O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00
Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`1HOol001Eo
o`03001oogoo059oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009o
o`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l
0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo009o
o`03001l07`0019l0003001oogoo05Qoo`005Goo00<007ooOol0DWoo00<007`0O0004W`000<007oo
Ool00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0
O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007oo
Ool00Woo00<007`0O0004g`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0F7oo000EOol0
0`00Oomoo`1BOol00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool0
0`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO000
0`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool0
0`00O01l000BO0000`00Oomoo`1HOol001Eoo`03001oogoo059oo`03001l07`0019l0003001oogoo
00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0
019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo
009oo`03001l07`001=l0003001oogoo009oo`03001l07`0019l0003001oogoo05Qoo`005Goo00<0
07ooOol0DWoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<0
07`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<0
07ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol00Woo00<0
07`0O0004W`000<007ooOol0F7oo000EOol3001BOol00`00O01l000BO0000`00Oomoo`03Ool00`00
O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00
Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00
O01l000CO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`1HOol001Eoo`03001oogoo059o
o`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l
0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=o
o`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo009oo`03001l07`0019l
0003001oogoo05Qoo`005Goo00<007ooOol0DWoo00<007`0O0004W`000<007ooOol00goo00<007`0
O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007oo
Ool00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0
O0004g`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0F7oo000EOol00`00Oomoo`1BOol0
0`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO000
0`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool0
0`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool00`00O01l000BO000
0`00Oomoo`1HOol001Eoo`03001oogoo059oo`03001l07`0019l0003001oogoo00=oo`03001l07`0
019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo
009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0
01=l0003001oogoo009oo`03001l07`0019l0003001oogoo05Qoo`005Goo00<007ooOol0DWoo00<0
07`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<0
07ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<0
07`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol00Woo00<007`0O0004W`000<0
07ooOol0F7oo000EOol00`00Oomoo`1BOol00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000B
O0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02
Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000C
O0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`1HOol001Eoo`03001oogoo059oo`03001l
07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001o
ogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l
07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo009oo`03001l07`0019l0003001o
ogoo05Qoo`005Goo00<007ooOol0DWoo5P001Goo00<007`0O0004W`000<007ooOol00Woo00<007`0
O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007oo
Ool00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol00Woo00<007`0
O0004W`000<007ooOol0F7oo000EOol00`00Oomoo`1]Ool00`00O01l000BO0000`00Oomoo`02Ool0
0`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO000
0`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`02Ool0
0`00O01l000BO0000`00Oomoo`1HOol001Eoo`03001oogoo06eoo`03001l07`0019l0003001oogoo
009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0
019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo
009oo`03001l07`0019l0003001oogoo05Qoo`005Goo00<007ooOol0KGoo00<007`0O0004W`000<0
07ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<0
07`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<0
07ooOol00Woo00<007`0O0004W`000<007ooOol0F7oo000EOol3001]Ool00`00O01l000BO0000`00
Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00
O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00
Oomoo`02Ool00`00O01l000BO0000`00Oomoo`1HOol001Eoo`03001oogoo06eoo`03001l07`0019l
0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009o
o`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l
0003001oogoo009oo`03001l07`0019l0003001oogoo05Qoo`005Goo00<007ooOol0KGoo00<007`0
O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007oo
Ool00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0
O0004g`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0F7oo000EOol00`00Oomoo`1]Ool0
0`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO000
0`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool0
0`00O01l000CO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`1HOol001Eoo`03001oogoo
06eoo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0
019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo
009oo`03001l07`001=l0003001oogoo009ooaH005Yoo`005Goo00<007ooOol0KGoo00<007`0O000
4W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol0
0Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O000
4g`000<007ooOol0LWoo000EOol00`00Oomoo`1]Ool00`00O01l000BO0000`00Oomoo`02Ool00`00
O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00
Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`1bOol001Eo
o`03001oogoo06eoo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=o
o`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l
0003001oogoo009oo`03001l07`001=l0003001oogoo079oo`005Goo00<007ooOol0KGoo00<007`0
O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007oo
Ool00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0
O0004g`000<007ooOol0LWoo000EOol00`00Oomoo`1]Ool00`00O01l000BO0000`00Oomoo`02Ool0
0`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO000
0`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`1bOol0
00Moo`<000=oo`8000Ioo`03001oogoo06eoo`03001l07`0019l0003001oogoo009oo`03001l07`0
019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo
00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo079oo`002Woo00<0
07oo00000Woo00<007ooOol00goo1000K7oo00<007`0O0004W`000<007ooOol00Woo00<007`0O000
4W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0
0goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000:Ool00`00
Ool00002Ool00`00Oomoo`03Ool00`00Oomoo`1]Ool00`00O01l000BO0000`00Oomoo`02Ool00`00
O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00
Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`1bOol000Mo
o`<0009oo`04001oogoo0005Ool00`00Oomoo`1]Ool00`00O01l000BO0000`00Oomoo`02Ool00`00
O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00
Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`1bOol000Mo
o`03001oogoo009oo`04001oogoo0005Ool00`00Oomoo`1]Ool00`00O01l000BO0000`00Oomoo`02
Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000B
O0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`1b
Ool000Moo`@0009oo`8000Ioo`03001oogoo06eoo`03001l07`0019l0003001oogoo009oo`03001l
07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001o
ogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo079oo`005Goo
00<007ooOol0KGoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo
00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`0
00<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000EOol00`00Oomoo`1]Ool00`00O01l
000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomo
o`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l
000CO0000`00Oomoo`1bOol001Eoo`03001oogoo06eoo`03001l07`0019l0003001oogoo009oo`03
001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003
001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo079oo`00
5Goo00<007ooOol0KGoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0
0goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O000
4W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000EOol00`00Oomoo`1]Ool00`00
O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00
Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00
O01l000CO0000`00Oomoo`1bOol001Eoo`03001oogoo06eoo`03001l07`0019l0003001oogoo009o
o`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l
0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo079o
o`005Goo00<007ooOol0KGoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007oo
Ool00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0
O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000EOol3001]Ool00`00O01l
000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomo
o`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l
000CO0000`00Oomoo`1bOol001Eoo`03001oogoo06eoo`03001l07`0019l0003001oogoo009oo`03
001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003
001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo079oo`00
5Goo00<007ooOol0KGoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0
0goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O000
4W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000EOol00`00Oomoo`1]Ool00`00
O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00
Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00
O01l000CO0000`00Oomoo`1bOol001Eoo`03001oogoo06eoo`03001l07`0019l0003001oogoo009o
o`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l
0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo079o
o`005Goo00<007ooOol0KGoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007oo
Ool00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0
O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000EOol00`00Oomoo`1]Ool0
0`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO000
0`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool0
0`00O01l000CO0000`00Oomoo`1bOol001Eoo`03001oogoo06eoo`03001l07`0019l0003001oogoo
009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0
019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo
079oo`005Goo00<007ooOol0KGoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<0
07ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<0
07`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000EOol00`00Oomoo`1]
Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000B
O0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02
Ool00`00O01l000CO0000`00Oomoo`1bOol001Eoo`03001oogoo06eoo`03001l07`0019l0003001o
ogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l
07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001o
ogoo079oo`005Goo0`00KGoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007oo
Ool00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0
O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000EOol00`00Oomoo`1]Ool0
0`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO000
0`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool0
0`00O01l000CO0000`00Oomoo`1bOol001Eoo`03001oogoo06eoo`03001l07`0019l0003001oogoo
009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0
019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo
079oo`005Goo00<007ooOol0KGoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<0
07ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<0
07`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000EOol00`00Oomoo`1]
Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000B
O0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02
Ool00`00O01l000CO0000`00Oomoo`1bOol001Eoo`03001oogoo06eoo`03001l07`0019l0003001o
ogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l
07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001o
ogoo079oo`005Goo00<007ooOol0KGoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`0
00<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo
00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000EOol00`00Oomo
o`1]Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l
000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomo
o`02Ool00`00O01l000CO0000`00Oomoo`1bOol001Eoo`03001oogoo06eoo`03001l07`0019l0003
001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03
001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003
001oogoo079oo`005Goo00<007ooOol0KGoo00<007`0O0004W`000<007ooOol00Woo00<007`0O000
4W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0
0goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000EOol00`00
Oomoo`1]Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00
O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00
Oomoo`02Ool00`00O01l000CO0000`00Oomoo`1bOol001Eoo`<006eooaH000Aoo`03001l07`0019l
0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=o
o`03001l07`0019l0003001oogoo009oo`03001l07`001=l0003001oogoo079oo`005Goo00<007oo
Ool0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0
O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007oo
Ool0LWoo000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO000
0`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool0
0`00O01l000CO0000`00Oomoo`1bOol001Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo
00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0
019l0003001oogoo009oo`03001l07`001=l0003001oogoo079oo`005Goo00<007ooOol0Qgoo00<0
07`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<0
07ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000E
Ool00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02
Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000C
O0000`00Oomoo`1bOol001Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l
07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001o
ogoo009oo`03001l07`001=l0003001oogoo079oo`005Goo00<007ooOol0Qgoo00<007`0O0004W`0
00<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo
00<007`0O0004W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000EOol00`00Oomo
o`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l
000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomo
o`1bOol001Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003
001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03
001l07`001=l0003001oogoo079oo`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol0
0goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O000
4W`000<007ooOol00Woo00<007`0O0004g`000<007ooOol0LWoo000EOol00`00Oomoo`27Ool00`00
O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00
Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000CO0000`00Oomoo`1bOol001Eo
o`<008Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l
07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009ooaL007Aoo`005Goo00<007oo
Ool0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0
O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol0SGoo000EOol00`00Oomoo`27Ool0
0`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO000
0`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`2=Ool001Eoo`03001oogoo08Moo`03001l07`0
019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo
00=oo`03001l07`0019l0003001oogoo08eoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<0
07ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<0
07`0O0004W`000<007ooOol0SGoo000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03
Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000B
O0000`00Oomoo`2=Ool001Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l
07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001o
ogoo08eoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`0
00<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol0SGoo
000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomo
o`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`2=Ool001Eoo`03
001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03
001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo08eoo`000Woo0`000Woo0P00
17oo0P001Woo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<0
07ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol0SGoo0003
Ool01000Oomoo`000Woo00@007ooOol0009oo`03001oogoo00=oo`@008Ioo`03001l07`0019l0003
001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03
001l07`0019l0003001oogoo08eoo`000goo00@007ooOol0009oo`04001oogoo0002Ool00`00Oomo
o`03Ool00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomo
o`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`2=Ool000=oo`04
001oogoo0002Ool01000Oomoo`000Woo00<007ooOol00goo00<007ooOol0Qgoo00<007`0O0004W`0
00<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo
00<007`0O0004W`000<007ooOol0SGoo0003Ool01000Oomoo`000Woo00@007ooOol0009oo`03001o
ogoo00=oo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001o
ogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo08eoo`000Woo
0P000goo0P0017oo0P001Woo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0
O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007oo
Ool0SGoo000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO000
0`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`2=Ool0
01Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo
009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo08eoo`005Goo00<0
07ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<0
07`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol0SGoo000EOol00`00Oomoo`27
Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000B
O0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`2=Ool001Eoo`03001oogoo08Moo`03001l
07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001o
ogoo00=oo`03001l07`0019l0003001oogoo08eoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`0
00<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo
00<007`0O0004W`000<007ooOol0SGoo000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomo
o`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l
000BO0000`00Oomoo`2=Ool001Eoo`<008Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0
019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo
08eoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<0
07ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol0SGoo000E
Ool00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02
Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`2=Ool001Eoo`03001o
ogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l
07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo08eoo`005Goo00<007ooOol0Qgoo
00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`0
00<007ooOol00goo00<007`0O0004W`000<007ooOol0SGoo000EOol00`00Oomoo`27Ool00`00O01l
000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomo
o`03Ool00`00O01l000BO0000`00Oomoo`2=Ool001Eoo`03001oogoo08Moo`03001l07`0019l0003
001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03
001l07`0019l0003001oogoo08eoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol0
0goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O000
4W`000<007ooOol0SGoo000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00
O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00
Oomoo`2=Ool001Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l
0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo08eo
o`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007oo
Ool00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol0SGoo000EOol3
0027Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l
000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`2=Ool001Eoo`03001oogoo08Moo`03
001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003
001oogoo00=oo`03001l07`0019l0003001oogoo08eoo`005Goo00<007ooOol0Qgoo00<007`0O000
4W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0
0goo00<007`0O0004W`000<007ooOol0SGoo000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00
Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00
O01l000BO0000`00Oomoo`2=Ool001Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=o
o`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l
0003001oogoo08eoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0
O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007oo
Ool0SGoo000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO000
0`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`2=Ool0
01Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo
009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo08eoo`005Goo00<0
07ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<0
07`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol0SGoo000EOol00`00Oomoo`27
Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000B
O0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`2=Ool001Eoo`03001oogoo08Moo`03001l
07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001o
ogoo00=oo`03001l07`0019l0003001oogoo08eoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`0
00<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo
00<007`0O0004W`000<007ooOol0SGoo000EOol30027Ool00`00O01l000BO0000`00Oomoo`03Ool0
0`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO000
0`00Oomoo`2=Ool001Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0
019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo
08eoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<0
07ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol0SGoo000E
Ool00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02
Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`2=Ool001Eoo`03001o
ogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l
07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo08eoo`005Goo00<007ooOol0Qgoo
00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`0
00<007ooOol00goo00<007`0O0004W`000<007ooOol0SGoo000EOol00`00Oomoo`27Ool00`00O01l
000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomo
o`03Ool00`00O01l000BO0000`00Oomoo`2=Ool001Eoo`03001oogoo08Moo`03001l07`0019l0003
001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo00=oo`03
001l07`0019l0003001oogoo08eoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol0
0goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol00goo00<007`0O000
4W`000<007ooOol0SGoo000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00
O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00
Oomoo`2=Ool001Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l
0003001oogoo009oo`03001l07`0019l0003001oogoo00=ooaH008moo`005Goo0`00Qgoo00<007`0
O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007oo
Ool0Z7oo000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO000
0`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2XOol001Eoo`03001oogoo08Moo`03001l07`0
019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo
0:Qoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<0
07ooOol00Woo00<007`0O0004W`000<007ooOol0Z7oo000EOol00`00Oomoo`27Ool00`00O01l000B
O0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2X
Ool001Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001o
ogoo009oo`03001l07`0019l0003001oogoo0:Qoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`0
00<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0Z7oo
000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomo
o`02Ool00`00O01l000BO0000`00Oomoo`2XOol001Eoo`03001oogoo08Moo`03001l07`0019l0003
001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo0:Qoo`00
5Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol0
0Woo00<007`0O0004W`000<007ooOol0Z7oo0002Ool3000017oo0000000017oo0P001Woo00<007oo
Ool0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0
O0004W`000<007ooOol0Z7oo0003Ool00`00Oomoo`03Ool01000Oomoo`000Woo00<007ooOol00goo
1000QWoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0
O0004W`000<007ooOol0Z7oo0003Ool00`00Oomoo`03Ool01000Oomoo`000Woo00<007ooOol00goo
00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo
00<007`0O0004W`000<007ooOol0Z7oo0003Ool00`00Oomoo`030003Ool01000Oomoo`001Goo00<0
07ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<0
07`0O0004W`000<007ooOol0Z7oo0003Ool01000Oomoo`001Goo00@007ooOol000Eoo`03001oogoo
08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0
019l0003001oogoo0:Qoo`000Woo0P000Woo10000goo0P001Woo00<007ooOol0Qgoo00<007`0O000
4W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0
Z7oo000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00
Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2XOol001Eoo`03001oogoo08Moo`03001l07`0019l
0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo0:Qo
o`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007oo
Ool00Woo00<007`0O0004W`000<007ooOol0Z7oo000EOol00`00Oomoo`27Ool00`00O01l000BO000
0`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2XOol0
01Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo
009oo`03001l07`0019l0003001oogoo0:Qoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<0
07ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0Z7oo000E
Ool00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02
Ool00`00O01l000BO0000`00Oomoo`2XOol001Eoo`<008Moo`03001l07`0019l0003001oogoo00=o
o`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo0:Qoo`005Goo00<007oo
Ool0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0
O0004W`000<007ooOol0Z7oo000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool0
0`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2XOol001Eoo`03001oogoo
08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0
019l0003001oogoo0:Qoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<0
07`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0Z7oo000EOol00`00Oomoo`27
Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000B
O0000`00Oomoo`2XOol001Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l
07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo0:Qoo`005Goo00<007ooOol0Qgoo
00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`0
00<007ooOol0Z7oo000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l
000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2XOol001Eoo`03001oogoo08Moo`03
001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003
001oogoo0:Qoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O000
4W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0Z7oo000EOol30027Ool00`00O01l000B
O0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2X
Ool001Eoo`03001oogoo08Moo`03001l07`0019l0003001oogoo00=oo`03001l07`0019l0003001o
ogoo009oo`03001l07`0019l0003001oogoo0:Qoo`005Goo00<007ooOol0Qgoo00<007`0O0004W`0
00<007ooOol00goo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0Z7oo
000EOol00`00Oomoo`27Ool00`00O01l000BO0000`00Oomoo`03Ool00`00O01l000BO0000`00Oomo
o`02Ool00`00O01l000BO0000`00Oomoo`2XOol001Eoo`03001oogoo08Moo`03001l07`0019l0003
001oogoo00=oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo0:Qoo`00
5Goo00<007ooOol0Qgoo00<007`0O0004W`000<007ooOol00goo00<007`0O0004W`000<007ooOol0
0Woo00<007`0O0004W`000<007ooOol0Z7oo000EOol00`00Oomoo`27OolF0005Ool00`00O01l000B
O0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2XOol001Eoo`03001oogoo0:9oo`03001l
07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo0:Qoo`005Goo00<007ooOol0XWoo
00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0Z7oo000EOol00`00Oomo
o`2ROol00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2XOol001Eoo`03
001oogoo0:9oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo0:Qoo`00
5Goo00<007ooOol0XWoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0
Z7oo000EOol3002ROol00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2X
Ool001Eoo`03001oogoo0:9oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001o
ogoo0:Qoo`005Goo00<007ooOol0XWoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`0
00<007ooOol0Z7oo000EOol00`00Oomoo`2ROol00`00O01l000BO0000`00Oomoo`02Ool00`00O01l
000BO0000`00Oomoo`2XOol001Eoo`03001oogoo0:9oo`03001l07`0019l0003001oogoo009oo`03
001l07`0019l0003001oogoo0:Qoo`005Goo00<007ooOol0XWoo00<007`0O0004W`000<007ooOol0
0Woo00<007`0O0004W`000<007ooOol0Z7oo000EOol00`00Oomoo`2ROol00`00O01l000BO0000`00
Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2XOol001Eoo`03001oogoo0:9oo`03001l07`0019l
0003001oogoo009oo`03001l07`0019l0003001oogoo0:Qoo`005Goo00<007ooOol0XWoo00<007`0
O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0Z7oo000EOol00`00Oomoo`2ROol0
0`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2XOol001Eoo`03001oogoo
0:9oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo0:Qoo`005Goo0`00
XWoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007ooOol0Z7oo000EOol00`00
Oomoo`2ROol00`00O01l000BO0000`00Oomoo`02Ool00`00O01l000BO0000`00Oomoo`2XOol001Eo
o`03001oogoo0:9oo`03001l07`0019l0003001oogoo009oo`03001l07`0019l0003001oogoo0:Qo
o`005Goo00<007ooOol0XWoo00<007`0O0004W`000<007ooOol00Woo00<007`0O0004W`000<007oo
Ool0Z7oo000EOol00`00Oomoo`2ROolF0004Ool00`00O01l000BO0000`00Oomoo`2XOol001Eoo`03
001oogoo0;aoo`03001l07`0019l0003001oogoo0:Qoo`005Goo00<007ooOol0_7oo00<007`0O000
4W`000<007ooOol0Z7oo000EOol00`00Oomoo`2lOol00`00O01l000BO0000`00Oomoo`2XOol001Eo
o`03001oogoo0;aoo`03001l07`0019l0003001oogoo0:Qoo`005Goo00<007ooOol0_7oo5P00ZWoo
00000goo000000020002Ool20004Ool20006Ool00`00Oomoo`3oOommOol0009oo`05001oogooOol0
0002Ool01000Oomoo`000Woo00<007ooOol00goo00<007ooOol0ogooOGoo0003Ool01000Oomoo`00
0Woo00@007ooOol0009oo`03001oogoo00=oo`@00?moogaoo`0017oo00<007oo00000Woo00@007oo
Ool0009oo`03001oogoo00=oo`03001oogoo0?moogeoo`0017oo00<007oo00000Woo00@007ooOol0
009oo`03001oogoo00=oo`03001oogoo0?moogeoo`0000Aoo`00000000=oo`8000Aoo`8000Ioo`03
001oogoo0?moogeoo`00\
\>"],
  ImageRangeCache->{{{0, 403.5}, {248.5, 0}} -> {-0.421098, -14.0506, 
  0.0471262, 1.09405}}]
}, Open  ]],

Cell["\<\
So far so good. We can use the statistics functions to output other \
information:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Mean[ndataone]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.0221018739931188212`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Variance[ndataone]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.961321928694554195`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(StandardDeviation[ndataone]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.980470258954627382`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(StandardErrorOfSampleMean[ndataone]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.0310051919635172446`\)], "Output"]
}, Open  ]],

Cell["\<\
But the time taken to generate this sample is worrying. However, \
when we loaded the statistics Packages we secretly loaded another more \
interesting set of functions that relate specifically to the normal \
distribution:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Statistics`NormalDistribution`*\)\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
ChiSquareDistribution NormalDistribution    PercentagePoint       \
StudentTDistribution
FRatioDistribution\
\>", "Print",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[TextData[{
  "This Package comes with a random number generator tailored to the normal \
distribution. This function has been improved in the transition from ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 2.2 to 3.0. In versions prior to 3.0 it took the form (do not load \
this!)"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(NormalDistribution /: \n\t
      Random[NormalDistribution[mu_:  0, \ sigma_:  1]]\  := \ 
      N[mu + sigma\ Sqrt[\(-2\)\ Log[Random[]]]\ Cos[2\ Pi\ Random[]]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This is one expression of the Box-Muller method for generating \
Gaussian deviates. In version 3.0 the Package contains a more efficient \
compiled form, with additional functionality (do not load these either - the \
Package has already done it!):\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(normal\  = \ 
      Compile[{{mu, \ _Real}, \ {sigma, \ _Real}, \ {q1, \ _Real}, \ {q2, \ \
_Real}}, \n\t\tmu\  + \ sigma\ Sqrt[\(-2\)\ Log[q1]]\ Cos[2  Pi\ q2]\t]\n\), 
    \(NormalDistribution /: \ 
      Random[NormalDistribution[mu_:  0, \ sigma_:  1]]\  := \n\t
      normal[mu, \ sigma, \ Random[], \ Random[]]\n\), 
    \(normalpair\  = \ 
      Compile[{{mu, \ _Real}, \ {sigma, \ _Real}, \ {q1, \ _Real}, \ {q2, \ \
_Real}}, \n\t\tmu\  + \ 
          sigma\ Sqrt[\(-2\)\ Log[q1]]\ {Cos[2  Pi\ q2], \ 
              Sin[2  Pi\ q2]}\ \ \ ]\n\)}], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The last of these expresses the fact that the conversion from random \
sampling on a square to the normal distribution actually generates a pair of \
normal random numbers. Let's look at the efficiency of using the standard \
Package function (note that we are now using ",
  StyleBox["ten",
    FontSlant->"Italic"],
  " thousand calls):"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(nora[mu_, \ sigma_]\  := \ \ Random[
        NormalDistribution[mu, \ sigma]]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Table[nora[0, 1], \ {10000}]; ]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1.05000000000001136`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell["\<\
In fact, re-naming it to save us typing later slowed us down \
slightly:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Table[Random[NormalDistribution[0, \ 1]], \ {10000}]; ]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.833333333333314385`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The most significant observation, at least with current versions (3.0) of \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", is that matters can still be improved significantly, by direct \
compilation of the Package algorithm as a global function:"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(norb = \ 
      Compile[{mu, \ sigma}, \ 
        mu\  + \ sigma*
            Sqrt[\(-2\)\ Log[Random[]]]\ Cos[2\ Pi\ Random[]]]; \)\)], "Input",\

  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Table[norb[0, 1], \ {10000}]; ]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.25`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Now Press ",
  StyleBox["et al",
    FontSlant->"Italic"],
  " (1992) have argued that this type of approach is still not optimal, on \
the basis that it should be even more efficient to avoid the trigonometric \
computations altogether with the following routine, based on sampling with a \
square on ",
  Cell[BoxData[
      \(TraditionalForm\`{{\(-1\), \ 1}, \ {\(-1\), \ 1}}\)]],
  ", and using only those points within the unit circle:"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(norm = \ 
      Compile[{mu, \ 
          sigma}, \ \n\t\tModule[{va, \ vb, \ rad\  = \ 2.0, \ 
            den}, \n\t\t\tWhile[
            rad\  >= 
              1.00, \n\t\t\t\t\t\((va\  = \ 
                2.0*Random[]\  - \ 1.0; \n\t\t\t\t\tvb\  = \ 
                2.0*Random[]\  - 1.0; \n\t\t\t\t\trad\  = \ 
                va*va + vb*vb)\)]; \n\t\t\tden\  = \ 
            Sqrt[\(-2.0\)*Log[rad]/rad]; \n\t\t\tmu + sigma*va*den]]; \)\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The benefits of this, in this particular form, ",
  StyleBox["almost",
    FontSlant->"Italic"],
  " outweigh the penalty of discarding the points outside the unit circle:"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Table[norm[0, 1], \ {10000}]; ]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.333333333333314385`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell["\<\
When this routine is used in its most efficient guise, outputting \
pairs, it finally wins the speed war:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(normpair = \ 
      Compile[{mu, \ sigma}, \ 
        Module[{va, \ vb, \ rad\  = \ 2.0, \ den}, \n\t\t\tWhile[
            rad\  >= 1.00, \n\t\t\t\t\t\((va\  = \ 2.0*Random[]\  - \ 1.0; \ 
              vb\  = \ 2.0*Random[]\  - 1.0; \n\t\t\t\t\trad\  = \ 
                va*va + vb*vb)\)]; \n\t\t\tden\  = \ 
            Sqrt[\(-2.0\)*Log[rad]/rad]; \n\t\t\t{mu + sigma*va*den, \ 
            mu + sigma*vb*den}]]; \)\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(ndatatwo\  = \ Flatten[Table[normpair[0, 1], \ {5000}]]; \)\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[Flatten[Table[normpair[0, 1], \ {5000}]]; ]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.233333333333348491`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell["\<\
If we make a similar algorithm out of the Package function, the two \
trigonometric calls appear to slow things down:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(normalpair\  = \ 
      Compile[{{mu, \ _Real}, \ {sigma, \ _Real}, \ {q1, \ _Real}, \ {q2, \ \
_Real}}, \n\t\tmu\  + \ 
          sigma\ Sqrt[\(-2\)\ Log[q1]]\ {Cos[2  Pi\ q2], \ 
              Sin[2  Pi\ q2]}\ \ \ ]; \)\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      Flatten[Table[normalpair[0, 1, Random[], \ Random[]], \ {5000}]]; ]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.28333333333335986`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "There are other tricks one can try, based on interpolations or rational \
function approximations to the inverse error function, but my own \
investigations suggest that these do not offer any speed advantages, and you \
pay a price in terms of approximation - as we shall see in the next chapter, \
the accuracy of simulation methods is a somewhat delicate matter, and we do \
not wish matters to be clouded further by approximations at the basic \
sampling level. In what follows we shall standardize on the algorithm \
recommended by Press ",
  StyleBox["et al",
    FontSlant->"Italic"],
  " (1992).\n\nWe can analyse this new data set ",
  StyleBox["ndatatwo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " in the same way as before:"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(BarChart[BinCounts[ndatatwo, \ {\(-3.5\), \ 3.5, \ 0.5}], \ 
      PlotRange\  -> \ All, \n
      BarLabels\  -> 
        Map[\ ToString, \ 
          Table[\(-3.25\)\  + \ 0.5*k, \ {k, \ 0, \ 13}]]]\ ; \)\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 7  scalefont  setfont
% Scaling calculations
-0.0175983 0.0690131 0.0147151 0.000304345 [
[.05141 .00222 -12.4688 -9 ]
[.05141 .00222 12.4688 0 ]
[.12043 .00222 -12.4688 -9 ]
[.12043 .00222 12.4688 0 ]
[.18944 .00222 -12.4688 -9 ]
[.18944 .00222 12.4688 0 ]
[.25845 .00222 -12.4688 -9 ]
[.25845 .00222 12.4688 0 ]
[.32747 .00222 -12.4688 -9 ]
[.32747 .00222 12.4688 0 ]
[.39648 .00222 -12.4688 -9 ]
[.39648 .00222 12.4688 0 ]
[.46549 .00222 -12.4688 -9 ]
[.46549 .00222 12.4688 0 ]
[.53451 .00222 -10.375 -9 ]
[.53451 .00222 10.375 0 ]
[.60352 .00222 -10.375 -9 ]
[.60352 .00222 10.375 0 ]
[.67253 .00222 -10.375 -9 ]
[.67253 .00222 10.375 0 ]
[.74155 .00222 -10.375 -9 ]
[.74155 .00222 10.375 0 ]
[.81056 .00222 -10.375 -9 ]
[.81056 .00222 10.375 0 ]
[.87957 .00222 -10.375 -9 ]
[.87957 .00222 10.375 0 ]
[.94859 .00222 -10.375 -9 ]
[.94859 .00222 10.375 0 ]
[-0.00249 .16689 -12.6 -3.15 ]
[-0.00249 .16689 0 3.15 ]
[-0.00249 .31906 -16.8 -3.15 ]
[-0.00249 .31906 0 3.15 ]
[-0.00249 .47123 -16.8 -3.15 ]
[-0.00249 .47123 0 3.15 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.05141 .01472 m
.05141 .02097 L
s
gsave
.05141 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.25) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.12043 .01472 m
.12043 .02097 L
s
gsave
.12043 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.75) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.18944 .01472 m
.18944 .02097 L
s
gsave
.18944 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.25) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.25845 .01472 m
.25845 .02097 L
s
gsave
.25845 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.75) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.32747 .01472 m
.32747 .02097 L
s
gsave
.32747 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.25) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.39648 .01472 m
.39648 .02097 L
s
gsave
.39648 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.75) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.46549 .01472 m
.46549 .02097 L
s
gsave
.46549 .00222 -73.4688 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
67.188 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.25) show
83.938 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.53451 .01472 m
.53451 .02097 L
s
gsave
.53451 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.25) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.60352 .01472 m
.60352 .02097 L
s
gsave
.60352 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.75) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.67253 .01472 m
.67253 .02097 L
s
gsave
.67253 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.25) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.74155 .01472 m
.74155 .02097 L
s
gsave
.74155 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1.75) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.81056 .01472 m
.81056 .02097 L
s
gsave
.81056 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.25) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.87957 .01472 m
.87957 .02097 L
s
gsave
.87957 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2.75) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.94859 .01472 m
.94859 .02097 L
s
gsave
.94859 .00222 -71.375 -13 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 13.000 moveto
 541.000 13.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 13.000 lineto
closepath
clip newpath
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.25) show
79.750 10.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
0 .01472 m
1 .01472 L
s
.01001 .16689 m
.01626 .16689 L
s
[(500)] -0.00249 .16689 1 0 Mshowa
.01001 .31906 m
.01626 .31906 L
s
[(1000)] -0.00249 .31906 1 0 Mshowa
.01001 .47123 m
.01626 .47123 L
s
[(1500)] -0.00249 .47123 1 0 Mshowa
.125 Mabswid
.01001 .04515 m
.01376 .04515 L
s
.01001 .07558 m
.01376 .07558 L
s
.01001 .10602 m
.01376 .10602 L
s
.01001 .13645 m
.01376 .13645 L
s
.01001 .19732 m
.01376 .19732 L
s
.01001 .22776 m
.01376 .22776 L
s
.01001 .25819 m
.01376 .25819 L
s
.01001 .28863 m
.01376 .28863 L
s
.01001 .34949 m
.01376 .34949 L
s
.01001 .37993 m
.01376 .37993 L
s
.01001 .41036 m
.01376 .41036 L
s
.01001 .4408 m
.01376 .4408 L
s
.01001 .50167 m
.01376 .50167 L
s
.01001 .5321 m
.01376 .5321 L
s
.01001 .56254 m
.01376 .56254 L
s
.01001 .59297 m
.01376 .59297 L
s
.25 Mabswid
.01001 0 m
.01001 .61803 L
s
1 0 0 r
.02381 .01472 m
.02381 .01715 L
.07902 .01715 L
.07902 .01472 L
F
0 g
.5 Mabswid
.02381 .01472 m
.02381 .01715 L
.07902 .01715 L
.07902 .01472 L
.02381 .01472 L
s
1 0 0 r
.09282 .01472 m
.09282 .02628 L
.14803 .02628 L
.14803 .01472 L
F
0 g
.09282 .01472 m
.09282 .02628 L
.14803 .02628 L
.14803 .01472 L
.09282 .01472 L
s
1 0 0 r
.16184 .01472 m
.16184 .06767 L
.21705 .06767 L
.21705 .01472 L
F
0 g
.16184 .01472 m
.16184 .06767 L
.21705 .06767 L
.21705 .01472 L
.16184 .01472 L
s
1 0 0 r
.23085 .01472 m
.23085 .14832 L
.28606 .14832 L
.28606 .01472 L
F
0 g
.23085 .01472 m
.23085 .14832 L
.28606 .14832 L
.28606 .01472 L
.23085 .01472 L
s
1 0 0 r
.29986 .01472 m
.29986 .30445 L
.35507 .30445 L
.35507 .01472 L
F
0 g
.29986 .01472 m
.29986 .30445 L
.35507 .30445 L
.35507 .01472 L
.29986 .01472 L
s
1 0 0 r
.36888 .01472 m
.36888 .47093 L
.42409 .47093 L
.42409 .01472 L
F
0 g
.36888 .01472 m
.36888 .47093 L
.42409 .47093 L
.42409 .01472 L
.36888 .01472 L
s
1 0 0 r
.43789 .01472 m
.43789 .59723 L
.4931 .59723 L
.4931 .01472 L
F
0 g
.43789 .01472 m
.43789 .59723 L
.4931 .59723 L
.4931 .01472 L
.43789 .01472 L
s
1 0 0 r
.5069 .01472 m
.5069 .60332 L
.56211 .60332 L
.56211 .01472 L
F
0 g
.5069 .01472 m
.5069 .60332 L
.56211 .60332 L
.56211 .01472 L
.5069 .01472 L
s
1 0 0 r
.57591 .01472 m
.57591 .45632 L
.63112 .45632 L
.63112 .01472 L
F
0 g
.57591 .01472 m
.57591 .45632 L
.63112 .45632 L
.63112 .01472 L
.57591 .01472 L
s
1 0 0 r
.64493 .01472 m
.64493 .29076 L
.70014 .29076 L
.70014 .01472 L
F
0 g
.64493 .01472 m
.64493 .29076 L
.70014 .29076 L
.70014 .01472 L
.64493 .01472 L
s
1 0 0 r
.71394 .01472 m
.71394 .15563 L
.76915 .15563 L
.76915 .01472 L
F
0 g
.71394 .01472 m
.71394 .15563 L
.76915 .15563 L
.76915 .01472 L
.71394 .01472 L
s
1 0 0 r
.78295 .01472 m
.78295 .06402 L
.83816 .06402 L
.83816 .01472 L
F
0 g
.78295 .01472 m
.78295 .06402 L
.83816 .06402 L
.83816 .01472 L
.78295 .01472 L
s
1 0 0 r
.85197 .01472 m
.85197 .02871 L
.90718 .02871 L
.90718 .01472 L
F
0 g
.85197 .01472 m
.85197 .02871 L
.90718 .02871 L
.90718 .01472 L
.85197 .01472 L
s
1 0 0 r
.92098 .01472 m
.92098 .01715 L
.97619 .01715 L
.97619 .01472 L
F
0 g
.92098 .01472 m
.92098 .01715 L
.97619 .01715 L
.97619 .01472 L
.92098 .01472 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  PageBreakBelow->Automatic,
  GroupPageBreakWithin->True,
  ImageSize->{320, 197.125},
  ImageMargins->{{33, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000500000aA000`40O003h00Oogoo@Goo003oOom1
Ool00?mood5oo`00ogoo@Goo003oOom1Ool00?mood5oo`00ogoo@Goo003oOom1Ool001]oo`<00003
Ool007oo009oo`<0009oo`<000Eoo`<00003Ool007oo00=oo`04001oogooOol30005Ool300000goo
001oo`02Ool30002Ool30005Ool300000goo001oo`03Ool01000Oomoogoo0`0017oo0`0000=oo`00
Ool00Woo0`000Woo0`001Goo0`0000=oo`00Ool00goo00@007ooOomoo`<000Eoo`<00003Ool007oo
009oo`<0009oo`<000=oo`<00003Ool007oo009oo`<0009oo`<000Eoo`<00003Ool007oo00=oo`04
001oogooOol30004Ool300000goo001oo`02Ool30002Ool30005Ool300000goo001oo`03Ool01000
Oomoogoo0`001Goo0`0000=oo`00Ool00Woo0`000Woo0`001Goo0`0000=oo`00Ool00goo00@007oo
Oomoo`<000Eoo`<00003Ool007oo009oo`<0009oo`<000Qoo`0057oo00<007ooOol00`000goo00<0
07ooOol017oo00<007ooOol00goo00<007oo00000P000Woo00<007ooOol01Goo00<007ooOol00goo
00<007oo00000P000Woo00<007ooOol01Goo00<007ooOol00goo10000goo00<007ooOol01Goo00<0
07ooOol00goo10000Woo00<007ooOol01Goo00<007ooOol00goo00<007oo00000P0000Aoo`00Ool0
00Ioo`03001oogoo00=oo`03001oo`0000800004Ool007oo0006Ool00`00Oomoo`03Ool01`00Oomo
ogoo001oo`0000Ioo`03001oogoo00=oo`03001oogoo00=oo`03001oo`0000Ioo`03001oogoo00=o
o`03001oogoo00=oo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Eo
o`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo00Ao
o`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo00Eoo`03001oogoo00Aoo`03001oogoo00=o
o`03001oogoo00Ioo`0057oo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00`001goo00<0
07ooOol01Goo00<007ooOol00`001goo00<007ooOol01Goo00<007ooOol00`001Woo00<007ooOol0
1Woo00<007ooOol00`001Goo00<007ooOol01Woo00<007ooOol00`001Goo00<007oo00001goo00<0
07ooOol00`001Goo00<007oo00001goo00<007ooOol00`000goo00<007oo00001goo00<007ooOol0
0`001Goo00<007oo00001goo00<007ooOol00`001Goo00<007ooOol01Woo00<007ooOol00`001Woo
00<007ooOol01Woo00<007ooOol00`001goo00<007ooOol01Goo00<007ooOol00`001goo00<007oo
Ool01Goo00<007ooOol00`001Woo00<007ooOol01Woo00<007ooOol00`0027oo000DOol00`00Oomo
o`04Ool30005Ool30002Ool30005Ool30005Ool30002Ool30005Ool30005Ool30002Ool30005Ool2
0006Ool30002Ool30004Ool20006Ool30002Ool30005Ool30005Ool30002Ool30005Ool30005Ool3
0002Ool30003Ool30005Ool30002Ool30005Ool30005Ool30002Ool30004Ool20006Ool30002Ool3
0005Ool20006Ool30002Ool30005Ool30005Ool30002Ool30005Ool30005Ool30002Ool30005Ool3
0005Ool30002Ool30008Ool001Aoo`03001oogoo0?moobYoo`0057oo00<007ooOol0ogoo:Woo000A
Oooo000`000001Aoo`04001oogooOolB0003Ool00`00O01l000>O0001@00Oomoogoo00003g`000<0
07ooOol00Woo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomo
ogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`000D0
07ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000@007ooOomooa4000Qo
o`0057oo00<007ooOol02Woo00<007ooOol02Goo00<007`0O0003W`000D007ooOomoo`0000ml0003
001oogoo009oo`03001l07`000el0005001oogooOol0000@O0001@00Oomoogoo000047`000D007oo
Oomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo00003g`000D007ooOomoo`00011l0005
001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo00Uoo`03
001oogoo00ioo`0057oo00<007ooOol05Woo4P000goo00<007`0O0003G`000<007ooOol00Woo00<0
07`0O0003G`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D0
07ooOomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`000D007ooOomoo`00011l
0005001oogooOol0000@O0001@00Oomoogoo000047`000<007ooOol06Woo000DOol00`00Oomoo`0[
Ool00`00O01l000=O0000`00Oomoo`02Ool00`00O01l000=O0001@00Oomoogoo000047`000D007oo
Oomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml0005
001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0004001oogooOolB000L
Ool001Aoo`03001oogoo02]oo`03001l07`000el0003001oogoo009oo`03001l07`000el0005001o
ogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O000
1@00Oomoogoo00003g`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo0000
47`000<007ooOol0;goo000DOol00`00Oomoo`0[Ool00`00O01l000=O0000`00Oomoo`02Ool00`00
O01l000=O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00
Oomoogoo000047`000D007ooOomoo`0000ml0005001oogooOol0000@O0001@00Oomoogoo000047`0
00D007ooOomoo`00011l0003001oogoo02moo`0057oo00<007ooOol0:goo00<007`0O0003G`000<0
07ooOol00Woo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomo
ogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`000D0
07ooOomoo`00011l0005001oogooOol0000@O0000`00Oomoo`0_Ool001Aoo`03001oogoo02]oo`03
001l07`000el0003001oogoo009oo`03001l07`000el0005001oogooOol0000@O0001@00Oomoogoo
000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo00003g`000D007oo
Oomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000<007ooOol0;goo000DOol2
000/Ool00`00O01l000=O0000`00Oomoo`02Ool00`00O01l000=O0001@00Oomoogoo000047`000D0
07ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml
0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo02mo
o`0057oo00<007ooOol0:goo00<007`0O0003G`000<007ooOol00Woo00<007`0O0003G`000D007oo
Oomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005
001oogooOol0000?O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@
O0000`00Oomoo`0_Ool001Aoo`03001oogoo02]oo`03001l07`000el0003001oogoo009oo`03001l
07`000el0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001o
ogooOol0000@O0001@00Oomoogoo00003g`000D007ooOomoo`00011l0005001oogooOol0000@O000
1@00Oomoogoo000047`000<007ooOol0;goo000DOol00`00Oomoo`0[Ool00`00O01l000=O0000`00
Oomoo`02Ool00`00O01l000=O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogoo
Ool0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml0005001oogooOol0000@O0001@00
Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo02moo`0057oo00<007ooOol0:goo00<0
07`0O0003G`000<007ooOol00Woo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0
000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomo
ogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0000`00Oomoo`0_Ool001Aoo`03
001oogoo02]oo`03001l07`000el0003001oogoo009oo`03001l07`000el0005001oogooOol0000@
O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo
00003g`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000<007oo
Ool0;goo000DOol00`00Oomoo`0[Ool00`00O01l000=O0000`00Oomoo`02Ool00`00O01l000=O000
1@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo0000
47`000D007ooOomoo`0000ml0005001oogooOol0000@O0001@00Oomoogoo000047`000@007ooOomo
oa80035oo`0057oo00<007ooOol0:goo4@0017oo00<007`0O0003G`000D007ooOomoo`00011l0005
001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?
O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo04Aoo`0057oo00<007ooOol0
@7oo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo0000
47`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`000D007ooOomo
o`00011l0003001oogoo04Aoo`0057oo0P00@Goo00<007`0O0003G`000D007ooOomoo`00011l0005
001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?
O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo04Aoo`0057oo00<007ooOol0
@7oo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo0000
47`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`000D007ooOomo
o`00011l0003001oogoo04Aoo`0057oo00<007ooOol0@7oo00<007`0O0003G`000D007ooOomoo`00
011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogoo
Ool0000?O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo04Aoo`0057oo00<0
07ooOol0@7oo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomo
ogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`000D0
07ooOomoo`00011l0003001oogoo04Aoo`0057oo00<007ooOol0@7oo00<007`0O0003G`000D007oo
Oomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005
001oogooOol0000?O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo04Aoo`00
57oo00<007ooOol0@7oo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0000@O000
1@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo0000
47`000D007ooOomoo`00011l0003001oogoo04Aoo`0057oo00<007ooOol0@7oo00<007`0O0003G`0
00D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00
011l0005001oogooOol0000?O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo
04Aoo`0057oo00<007ooOol0@7oo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0
000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomo
ogoo000047`000D007ooOomoo`00011l0003001oogoo04Aoo`0057oo00<007ooOol0@7oo00<007`0
O0003G`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007oo
Oomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003
001oogoo04Aoo`0057oo0P00@Goo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0
000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomo
ogoo000047`000D007ooOomoo`00011l0003001oogoo04Aoo`0057oo00<007ooOol0@7oo00<007`0
O0003G`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007oo
Oomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003
001oogoo04Aoo`0057oo00<007ooOol0@7oo00<007`0O0003G`000D007ooOomoo`00011l0005001o
ogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O000
1@00Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo04Aoo`0057oo00<007ooOol0@7oo
00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`0
00D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`000D007ooOomoo`00
011l0003001oogoo04Aoo`0057oo00<007ooOol0@7oo00<007`0O0003G`000D007ooOomoo`00011l
0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0
000?O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo04Aoo`0057oo00<007oo
Ool0@7oo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo
000047`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`000D007oo
Oomoo`00011l0003001oogoo04Aoo`0057oo00<007ooOol0@7oo00<007`0O0003G`000D007ooOomo
o`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001o
ogooOol0000?O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo04Aoo`0057oo
00<007ooOol0@7oo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00
Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`0
00D007ooOomoo`00011l0003001oogoo04Aoo`0057oo00<007ooOol0@7oo00<007`0O0003G`000D0
07ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l
0005001oogooOol0000?O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo04Ao
o`0057oo00<007ooOol0@7oo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0000@
O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo
000047`000D007ooOomoo`00011l0003001oogoo04Aoo`0057oo0P00@Goo00<007`0O0003G`000D0
07ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l
0005001oogooOol0000?O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003001oogoo04Ao
o`0057oo00<007ooOol0@7oo00<007`0O0003G`000D007ooOomoo`00011l0005001oogooOol0000@
O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo
000047`000D007ooOomoo`00011l0003001oogoo04Aoo`0057oo00<007ooOol0@7oo00<007`0O000
3G`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomo
o`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`000D007ooOomoo`00011l0003001o
ogoo04Aoo`0057oo00<007ooOol0@7oo4@000goo00<007`0O0003W`000D007ooOomoo`00011l0005
001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml0005001oogooOol0000@
O0001@00Oomoogoo000047`000<007ooOol0A7oo000DOol00`00Oomoo`1DOol00`00O01l000>O000
1@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo0000
3g`000D007ooOomoo`00011l0005001oogooOol0000@O0000`00Oomoo`14Ool001Aoo`03001oogoo
05Aoo`03001l07`000il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00
011l0005001oogooOol0000?O0001@00Oomoogoo000047`000@007ooOomooa8004Ioo`0057oo00<0
07ooOol0E7oo00<007`0O0003W`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomo
ogoo000047`000D007ooOomoo`0000ml0005001oogooOol0000@O0000`00Oomoo`1IOol001Aoo`03
001oogoo05Aoo`03001l07`000il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007oo
Oomoo`00011l0005001oogooOol0000?O0001@00Oomoogoo000047`000<007ooOol0FGoo0005Ool3
00001Goo00000000Ool00`0017oo00<007ooOol0E7oo00<007`0O0003W`000D007ooOomoo`00011l
0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml0005001oogooOol0
000@O0000`00Oomoo`1IOol000Moo`09001oo`00Ool007oo001oo`0000Aoo`<005Aoo`03001l07`0
00il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogoo
Ool0000?O0001@00Oomoogoo000047`000<007ooOol0FGoo0005Ool3000027oo001oo`00Ool007oo
0004Ool00`00Oomoo`1DOol00`00O01l000>O0001@00Oomoogoo000047`000D007ooOomoo`00011l
0005001oogooOol0000@O0001@00Oomoogoo00003g`000D007ooOomoo`00011l0003001oogoo05Uo
o`001Goo0`0000Eoo`00000007oo00<000Aoo`03001oogoo05Aoo`03001l07`000il0005001oogoo
Ool0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0001@00
Oomoogoo000047`000<007ooOol0FGoo000DOol00`00Oomoo`1DOol00`00O01l000>O0001@00Oomo
ogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo00003g`000D0
07ooOomoo`00011l0003001oogoo05Uoo`0057oo00<007ooOol0E7oo00<007`0O0003W`000D007oo
Oomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml0005
001oogooOol0000@O0000`00Oomoo`1IOol001Aoo`03001oogoo05Aoo`03001l07`000il0005001o
ogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O000
1@00Oomoogoo000047`000<007ooOol0FGoo000DOol00`00Oomoo`1DOol00`00O01l000>O0001@00
Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo00003g`0
00D007ooOomoo`00011l0003001oogoo05Uoo`0057oo00<007ooOol0E7oo00<007`0O0003W`000D0
07ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml
0005001oogooOol0000@O0000`00Oomoo`1IOol001Aoo`03001oogoo05Aoo`03001l07`000il0005
001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?
O0001@00Oomoogoo000047`000<007ooOol0FGoo000DOol2001EOol00`00O01l000>O0001@00Oomo
ogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo00003g`000D0
07ooOomoo`00011l0003001oogoo05Uoo`0057oo00<007ooOol0E7oo00<007`0O0003W`000D007oo
Oomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml0005
001oogooOol0000@O0000`00Oomoo`1IOol001Aoo`03001oogoo05Aoo`03001l07`000il0005001o
ogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O000
1@00Oomoogoo000047`000<007ooOol0FGoo000DOol00`00Oomoo`1DOol00`00O01l000>O0001@00
Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo00003g`0
00D007ooOomoo`00011l0003001oogoo05Uoo`0057oo00<007ooOol0E7oo00<007`0O0003W`000D0
07ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml
0005001oogooOol0000@O0000`00Oomoo`1IOol001Aoo`03001oogoo05Aoo`03001l07`000il0005
001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?
O0001@00Oomoogoo000047`000<007ooOol0FGoo000DOol00`00Oomoo`1DOol00`00O01l000>O000
1@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo0000
3g`000D007ooOomoo`00011l0003001oogoo05Uoo`0057oo00<007ooOol0E7oo00<007`0O0003W`0
00D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00
00ml0005001oogooOol0000@O0000`00Oomoo`1IOol001Aoo`03001oogoo05Aoo`03001l07`000il
0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0
000?O0001@00Oomoogoo000047`000<007ooOol0FGoo000DOol2001EOol00`00O01l000>O0001@00
Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo00003g`0
00D007ooOomoo`00011l0003001oogoo05Uoo`0057oo00<007ooOol0E7oo00<007`0O0003W`000D0
07ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml
0005001oogooOol0000@O0000`00Oomoo`1IOol001Aoo`03001oogoo05Aoo`03001l07`000il0005
001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?
O0001@00Oomoogoo000047`000<007ooOol0FGoo000DOol00`00Oomoo`1DOol00`00O01l000>O000
1@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo0000
3g`000D007ooOomoo`00011l0003001oogoo05Uoo`0057oo00<007ooOol0E7oo00<007`0O0003W`0
00D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00
00ml0005001oogooOol0000@O0000`00Oomoo`1IOol001Aoo`03001oogoo05Aoo`03001l07`000il
0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0
000?O0001@00Oomoogoo000047`000<007ooOol0FGoo000DOol00`00Oomoo`1DOol00`00O01l000>
O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo
00003g`000D007ooOomoo`00011l0003001oogoo05Uoo`0057oo00<007ooOol0E7oo00<007`0O000
3W`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomo
o`0000ml0005001oogooOol0000@O0000`00Oomoo`1IOol001Aoo`03001oogoo05Aoo`03001l07`0
00il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogoo
Ool0000?O0001@00Oomoogoo000047`000<007ooOol0FGoo000DOol2001EOol00`00O01l000>O000
1@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo0000
3g`000D007ooOomoo`00011l0003001oogoo05Uoo`0057oo00<007ooOol0E7oo00<007`0O0003W`0
00D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00
00ml0005001oogooOol0000@O0000`00Oomoo`1IOol001Aoo`03001oogoo05Aoo`03001l07`000il
0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0
000?O0001@00Oomoogoo000047`000<007ooOol0FGoo000DOol00`00Oomoo`1DOol00`00O01l000>
O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo
00003g`000D007ooOomoo`00011l0003001oogoo05Uoo`0057oo00<007ooOol0E7oo00<007`0O000
3W`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomo
o`0000ml0005001oogooOol0000@O0000`00Oomoo`1IOol001Aoo`03001oogoo05Aoo`03001l07`0
00il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogoo
Ool0000?O0001@00Oomoogoo000047`000<007ooOol0FGoo000DOol00`00Oomoo`1DOol00`00O01l
000>O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomo
ogoo00003g`000D007ooOomoo`00011l0003001oogoo05Uoo`0057oo00<007ooOol0E7oo00<007`0
O0003W`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007oo
Oomoo`0000ml0005001oogooOol0000@O0000`00Oomoo`1IOol001Aoo`03001oogoo05Aoo`03001l
07`000il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001o
ogooOol0000?O0001@00Oomoogoo000047`000<007ooOol0FGoo000DOol2001EOol00`00O01l000>
O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo
00003g`000D007ooOomoo`00011l0003001oogoo05Uoo`0057oo00<007ooOol0E7oo00<007`0O000
3W`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomo
o`0000ml0004001oogooOolB001KOol001Aoo`03001oogoo05Aoo`03001l07`000il0005001oogoo
Ool0000@O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0000`00
Oomoo`1^Ool001Aoo`03001oogoo05Aoo`03001l07`000il0005001oogooOol0000@O0001@00Oomo
ogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0000`00Oomoo`1^Ool001Aoo`03
001oogoo05Aoo`03001l07`000il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007oo
Oomoo`00011l0005001oogooOol0000?O0000`00Oomoo`1^Ool001Aoo`03001oogoo05Aooa8000=o
o`03001l07`000il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml
0003001oogoo06ioo`0057oo00<007ooOol0JGoo00<007`0O0003W`000D007ooOomoo`00011l0005
001oogooOol0000@O0001@00Oomoogoo00003g`000<007ooOol0KWoo000DOol00`00Oomoo`1YOol0
0`00O01l000>O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O000
0`00Oomoo`1^Ool001Aoo`03001oogoo06Uoo`03001l07`000il0005001oogooOol0000@O0001@00
Oomoogoo000047`000D007ooOomoo`0000ml0003001oogoo06ioo`0000Eoo`00000007oo00<00004
Ool000000002Ool30003Ool00`00Oomoo`1YOol00`00O01l000>O0001@00Oomoogoo000047`000D0
07ooOomoo`00011l0005001oogooOol0000?O0000`00Oomoo`1^Ool0009oo`0:001oogoo001oo`00
Ool007oo0002Ool00`00Ool00003Ool3001YOol00`00O01l000>O0001@00Oomoogoo000047`000D0
07ooOomoo`00011l0005001oogooOol0000?O0000`00Oomoo`1^Ool0009oo`0:001oogoo001oo`00
Ool007oo0002Ool00`00Ool00003Ool00`00Oomoo`1YOol00`00O01l000>O0001@00Oomoogoo0000
47`000D007ooOomoo`00011l0005001oogooOol0000?O0000`00Oomoo`1^Ool00003Ool00000009o
o`<00004Ool000000002Ool30003Ool00`00Oomoo`1YOol00`00O01l000>O0001@00Oomoogoo0000
47`000D007ooOomoo`00011l0005001oogooOol0000?O0000`00Oomoo`1^Ool001Aoo`03001oogoo
06Uoo`03001l07`000il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00
00ml0003001oogoo06ioo`0057oo00<007ooOol0JGoo00<007`0O0003W`000D007ooOomoo`00011l
0005001oogooOol0000@O0001@00Oomoogoo00003g`000<007ooOol0KWoo000DOol00`00Oomoo`1Y
Ool00`00O01l000>O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?
O0000`00Oomoo`1^Ool001Aoo`03001oogoo06Uoo`03001l07`000il0005001oogooOol0000@O000
1@00Oomoogoo000047`000D007ooOomoo`0000ml0003001oogoo06ioo`0057oo00<007ooOol0JGoo
00<007`0O0003W`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo00003g`0
00<007ooOol0KWoo000DOol00`00Oomoo`1YOol00`00O01l000>O0001@00Oomoogoo000047`000D0
07ooOomoo`00011l0005001oogooOol0000?O0000`00Oomoo`1^Ool001Aoo`8006Yoo`03001l07`0
00il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml0003001oogoo
06ioo`0057oo00<007ooOol0JGoo00<007`0O0003W`000D007ooOomoo`00011l0005001oogooOol0
000@O0001@00Oomoogoo00003g`000<007ooOol0KWoo000DOol00`00Oomoo`1YOol00`00O01l000>
O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0000`00Oomoo`1^
Ool001Aoo`03001oogoo06Uoo`03001l07`000il0005001oogooOol0000@O0001@00Oomoogoo0000
47`000D007ooOomoo`0000ml0003001oogoo06ioo`0057oo00<007ooOol0JGoo00<007`0O0003W`0
00D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo00003g`000<007ooOol0KWoo
000DOol00`00Oomoo`1YOol00`00O01l000>O0001@00Oomoogoo000047`000D007ooOomoo`00011l
0005001oogooOol0000?O0000`00Oomoo`1^Ool001Aoo`03001oogoo06Uoo`03001l07`000il0005
001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml0003001oogoo06ioo`00
57oo00<007ooOol0JGoo00<007`0O0003W`000D007ooOomoo`00011l0005001oogooOol0000@O000
1@00Oomoogoo00003g`000<007ooOol0KWoo000DOol00`00Oomoo`1YOol00`00O01l000>O0001@00
Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0000`00Oomoo`1^Ool001Ao
o`8006Yoo`03001l07`000il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomo
o`0000ml0003001oogoo06ioo`0057oo00<007ooOol0JGoo00<007`0O0003W`000D007ooOomoo`00
011l0005001oogooOol0000@O0001@00Oomoogoo00003g`000<007ooOol0KWoo000DOol00`00Oomo
o`1YOol00`00O01l000>O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0
000?O0000`00Oomoo`1^Ool001Aoo`03001oogoo06Uoo`03001l07`000il0005001oogooOol0000@
O0001@00Oomoogoo000047`000D007ooOomoo`0000ml0003001oogoo06ioo`0057oo00<007ooOol0
JGoo00<007`0O0003W`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo0000
3g`000<007ooOol0KWoo000DOol00`00Oomoo`1YOol00`00O01l000>O0001@00Oomoogoo000047`0
00D007ooOomoo`00011l0005001oogooOol0000?O0000`00Oomoo`1^Ool001Aoo`03001oogoo06Uo
o`03001l07`000il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml
0003001oogoo06ioo`0057oo00<007ooOol0JGoo00<007`0O0003W`000D007ooOomoo`00011l0005
001oogooOol0000@O0001@00Oomoogoo00003g`000<007ooOol0KWoo000DOol00`00Oomoo`1YOol0
0`00O01l000>O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O000
0`00Oomoo`1^Ool001Aoo`8006Yoo`03001l07`000il0005001oogooOol0000@O0001@00Oomoogoo
000047`000D007ooOomoo`0000ml0003001oogoo06ioo`0057oo00<007ooOol0JGoo00<007`0O000
3W`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo00003g`000<007ooOol0
KWoo000DOol00`00Oomoo`1YOol00`00O01l000>O0001@00Oomoogoo000047`000D007ooOomoo`00
011l0005001oogooOol0000?O0000`00Oomoo`1^Ool001Aoo`03001oogoo06Uoo`03001l07`000il
0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml0003001oogoo06io
o`0057oo00<007ooOol0JGoo00<007`0O0003W`000D007ooOomoo`00011l0005001oogooOol0000@
O0001@00Oomoogoo00003g`000<007ooOol0KWoo000DOol00`00Oomoo`1YOol00`00O01l000>O000
1@00Oomoogoo000047`000D007ooOomoo`00011l0005001oogooOol0000?O0000`00Oomoo`1^Ool0
01Aoo`03001oogoo06Uoo`03001l07`000il0005001oogooOol0000@O0001@00Oomoogoo000047`0
00D007ooOomoo`0000ml0003001oogoo06ioo`0057oo00<007ooOol0JGoo00<007`0O0003W`000D0
07ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo00003g`000<007ooOol0KWoo000D
Ool00`00Oomoo`1YOol00`00O01l000>O0001@00Oomoogoo000047`000D007ooOomoo`00011l0005
001oogooOol0000?O0000`00Oomoo`1^Ool001Aoo`8006Yoo`03001l07`000il0005001oogooOol0
000@O0001@00Oomoogoo000047`000D007ooOomoo`0000ml0003001oogoo06ioo`0057oo00<007oo
Ool0JGoo00<007`0O0003W`000D007ooOomoo`00011l0005001oogooOol0000@O0001@00Oomoogoo
00003g`000<007ooOol0KWoo000DOol00`00Oomoo`1YOol00`00O01l000>O0001@00Oomoogoo0000
47`000D007ooOomoo`00011l0005001oogooOol0000?O0000`00Oomoo`1^Ool001Aoo`03001oogoo
06Uoo`03001l07`000il0005001oogooOol0000@O0001@00Oomoogoo000047`000D007ooOomoo`00
00ml0003001oogoo06ioo`0057oo00<007ooOol0JGoo00<007`0O0003W`000D007ooOomoo`00011l
0005001oogooOol0000@O0001@00Oomoogoo00003g`000<007ooOol0KWoo000DOol00`00Oomoo`1Y
Ool00`00O01l000>O0001@00Oomoogoo000047`000D007ooOomoo`00011l0004001oogooOolA001`
Ool001Aoo`03001oogoo06Uoo`03001l07`000il0005001oogooOol0000@O0001@00Oomoogoo0000
47`000<007ooOol0PWoo000DOol00`00Oomoo`1YOol00`00O01l000>O0001@00Oomoogoo000047`0
00D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0JGoo00<007`0O0003W`000D0
07ooOomoo`00011l0005001oogooOol0000@O0000`00Oomoo`22Ool00005Ool00000001oo`030000
17oo000000000Woo0`000goo00<007ooOol0JGoo4P000goo00<007`0O0003W`000D007ooOomoo`00
011l0003001oogoo089oo`000Woo00<007ooOol00Woo00D007oo001oo`00009oo`03001oo`0000=o
o`<007ioo`03001l07`000il0005001oogooOol0000@O0000`00Oomoo`22Ool0009oo`03001oogoo
00<00004Ool007oo0002Ool00`00Ool00003Ool00`00Oomoo`1nOol00`00O01l000>O0001@00Oomo
ogoo000047`000<007ooOol0PWoo00000goo00000002Ool3000017oo000000000Woo0`000goo00<0
07ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007oo
Ool0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0
OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo
00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo00<0
07`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0
O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0O000
3W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo0P00Ogoo00<007`0O0003W`000D007oo
Oomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomo
o`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00
011l0003001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l
0003001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003
001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001o
ogoo089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo
089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089o
o`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`00
57oo0P00Ogoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007oo
Ool0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0
OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo
00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo00<0
07`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0
O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0O000
3W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`0
00D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D0
07ooOomoo`00011l0003001oogoo089oo`0057oo0P00Ogoo00<007`0O0003W`000D007ooOomoo`00
011l0003001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l
0003001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003
001oogoo089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001o
ogoo089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo
089oo`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089o
o`0057oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`00
57oo00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo
00<007ooOol0OWoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo0P00
Ogoo00<007`0O0003W`000D007ooOomoo`00011l0003001oogoo089oo`0057oo00<007ooOol0OWoo
4P000goo00<007`0O0003W`000<007ooOol0PWoo000DOol00`00Oomoo`2COol00`00O01l000>O000
0`00Oomoo`22Ool001Aoo`03001oogoo09=ooa8008Aoo`0057oo00<007ooOol0ogoo:Woo000DOol0
0`00Oomoo`3oOolZOol001Aoo`03001oogoo0?moobYoo`0057oo00<007ooOol0ogoo:Woo000DOol0
0`00Oomoo`3oOolZOol00001\
\>"],
  ImageRangeCache->{{{0, 319}, {196.125, 0}} -> {-0.5887, -147.283, 
  0.0480683, 10.8999}}]
}, Open  ]],

Cell["\<\
As before, we can use the statistics functions to output other \
information:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Mean[ndatatwo]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-0.00393838340918172935`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(StandardDeviation[ndatatwo]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.99631040245447231`\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The functions ",
  StyleBox["norm",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["normpair",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", and some obvious antithetic extensions, will form the basis of our \
sampling routines henceforth. The author would be delighted to hear of still \
more efficient schemes."
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Antithetic Variations", "Subsubsection",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Here we generate pairs of samples, using the standardized normal \
variate of opposite sign:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(normat = \ 
      Compile[{mu, \ 
          sigma}, \ \n\t\tModule[{va, \ vb, \ rad\  = \ 2.0, \ 
            den}, \n\t\t\tWhile[
            rad\  >= 
              1.00, \n\t\t\t\t\t\((va\  = \ 
                2.0*Random[]\  - \ 1.0; \n\t\t\t\t\tvb\  = \ 
                2.0*Random[]\  - 1.0; \n\t\t\t\t\trad\  = \ 
                va*va + vb*vb)\)]; \n\t\t\tden\  = \ 
            Sqrt[\(-2.0\)*Log[rad]/rad]; \n\t\t\t{mu + sigma*va*den, \ 
            mu - sigma*va*den}]]; \)\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(normpairat = \ 
      Compile[{mu, \ 
          sigma}, \ \n\t\tModule[{va, \ vb, \ rad\  = \ 2.0, \ 
            den}, \n\t\t\tWhile[
            rad\  >= 
              1.00, \n\t\t\t\t\t\((va\  = \ 
                2.0*Random[]\  - \ 1.0; \n\t\t\t\t\tvb\  = \ 
                2.0*Random[]\  - 1.0; \n\t\t\t\t\trad\  = \ 
                va*va + vb*vb)\)]; \n\t\t\tden\  = \ 
            Sqrt[\(-2.0\)*Log[rad]/rad]; \n\t\t\t{mu + sigma*va*den, \ 
            mu - sigma*va*den, \ mu + sigma*vb*den, \ 
            mu - sigma*vb*den}]]; \)\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \({normat[0, 1], \ normat[1, 1]}\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{0.130282627729943123`, \(-0.130282627729943123`\)}, \
{\(-0.20639051602161329`\), 2.20639051602161329`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(normpairat[0, 1]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\(-0.536588237603055961`\), 
      0.536588237603055961`, \(-1.80106218815144778`\), 
      1.80106218815144778`}\)], "Output"]
}, Open  ]],

Cell["\<\
The output of four random numbers from one call to the routine is \
the most efficient method the author is aware of.\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["21.3 Simulation of Paths in Fine Detail", "Section",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
There are various types of simulation that we need to consider when \
dealing with a single underlying asset. In one case we wish to simulate the \
entire stock-price path in as much detail as possible, or just at a high \
frequency, as efficiently as possible. In the second case we wish to simulate \
asset values at a few particular points in time, and these may be regularly \
spaced or irregularly spaced. So we have three basic cases:

(a) fine detail, clockwork sampling;
(b) coarse but clockwork sampling;
(c) irregular sampling at any level of detail.

These three cases are best treated slightly differently, in order to optimize \
the efficiency of the process. We shall refer to the first case as \"fine \
path simulation\", the second as \"coarse path simulation\", and the third as \
\"event simulation\". The second method can be applied to the first case, and \
the third method can be applied to the first two.\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Fine Path Simulation", "Subsection",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In the following the parameters are:\n\n",
  Cell[BoxData[
      \(TraditionalForm\`s\)]],
  " - starting asset price;\n",
  Cell[BoxData[
      \(TraditionalForm\`r\)]],
  " - risk free rate;\n",
  Cell[BoxData[
      \(TraditionalForm\`q\)]],
  " - dividend yield;\n\[Sigma] - volatility;\n",
  Cell[BoxData[
      \(TraditionalForm\`t\)]],
  " - time to maturity;\n",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " - number of sample points on the path;\nnavg - number of paths to be \
sampled.\n\nWe build the path by using the fundamental random walk \
characterization"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(d\ S\)\/S = \((r - q)\)\ \[CapitalDelta]\ T + 
        Z\ \[Sigma]\ \@\(\[CapitalDelta]\ T\)\)], "NumberedEquation",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The simplest such path generator uses the simple ",
  StyleBox["norm",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function:"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
path[s_, r_, q_, \[Sigma]_, t_, n_, navg_] :=
Module[
{mpath = 1 + (r-q)*t/(navg - 1), spath = \[Sigma] Sqrt[t/(navg - 1)]},
Table[
NestList[# norm[mpath, spath]&, s, navg - 1], {i, n}] ];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[TextData[{
  "We can instead use the independent pairs generated by ",
  StyleBox["normpair",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and reduce the number of paths tabulated separately by a factor of two. \
The function ",
  StyleBox["pathpair",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " generates ",
  Cell[BoxData[
      \(TraditionalForm\`2  n\)]],
  " paths:"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
pathpair[s_, r_, q_, \[Sigma]_, t_, n_, navg_] :=
Module[
{mpath = N[1 + (r-q)*t/(navg - 1)], 
spath = N[\[Sigma] Sqrt[t/(navg - 1)]]},
Flatten[
 Table[
  Transpose[
   NestList[# normpair[mpath, spath]&, {s, s}, navg - 1]], 
  {i, n}],1]];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(path[100, \ 0.1, \ 0, \ 0.2, \ 1, \ 4, \ 4]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{100, 114.775685044624564`, 128.523850688744879`, 
        133.19805692689377`}, {100, 77.4592785070039191`, 
        88.5970204925500581`, 86.304754569774893`}, {100, 
        115.592524080623748`, 98.2660200953703899`, 
        102.493052827761754`}, {100, 112.345663400993611`, 
        106.665573405364044`, 129.607510566901961`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pathpair[100, \ 0.1, \ 0, \ 0.2, \ 1, \ 2, \ 4]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{100, 75.4607564371254291`, 82.326351031197813`, 
        87.4882060157604968`}, {100, 129.311571083124189`, 
        120.267826770420471`, 135.048221143123758`}, {100, 
        107.241574901164815`, 96.6208423720670239`, 
        101.291230026471223`}, {100, 111.789908507086988`, 
        95.7270620098438307`, 90.2203396543818847`}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Similarly, we can generate ",
  Cell[BoxData[
      FormBox[
        RowBox[{"4", 
          FormBox["n",
            "TraditionalForm"]}], TraditionalForm]]],
  " paths using two sets of antithetic pairs:"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
pathpairat[s_, r_, q_, \[Sigma]_, t_, n_, navg_] :=
Module[
{mpath = N[1 + (r-q)*t/(navg - 1)], 
spath = N[\[Sigma] Sqrt[t/(navg - 1)]]},
Flatten[
 Table[
  Transpose[
   NestList[# normpairat[mpath, spath]&, {s, s, s, s}, navg - 1]], 
  {i, n}],1]];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(pathpairat[100, \ 0.1, \ 0, \ 0.2, \ 1, \ 1, \ 4]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{100, 116.67548780305208`, 121.282704389820183`, 
        110.840547264979449`}, {100, 89.9911788636145892`, 
        92.437069309350015`, 106.558156021190564`}, {100, 
        95.7165520812582037`, 113.446758387437673`, 
        132.907111431371727`}, {100, 110.950114585408488`, 
        97.7947696371751717`, 87.5389681256218032`}}\)], "Output"]
}, Open  ]],

Cell["\<\
Let's take a look at the relative timings, to obtain 100 paths of \
100 sample points each:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[u\  = \ path[100, \ 0.1, \ 0, \ 0.2, \ 1, \ 100, \ 100]; ]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.516666666666651508`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      v\  = \ pathpair[100, \ 0.1, \ 0, \ 0.2, \ 1, \ 50, \ 100]; ]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.416666666666628771`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      w\  = \ pathpairat[100, \ 0.1, \ 0, \ 0.2, \ 1, \ 25, \ 100]; ]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.266666666666651508`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Length, {u, \ v, \ w}]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({100, 100, 100}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["What Do Paths Look like?", "Subsection",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["Let's take a look at 20 paths of 40 steps each:", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
example = path[100, 0.1, 0, 0.2, 1, 20, 40];
plots = Table[ListPlot[example[[i]], PlotJoined -> True,
DisplayFunction -> Identity], {i, 1, Length[example]}];
Show[plots, DisplayFunction -> $DisplayFunction];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 7  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 -0.370232 0.00531536 [
[.2619 .0425 -4.2 -6.3 ]
[.2619 .0425 4.2 0 ]
[.5 .0425 -4.2 -6.3 ]
[.5 .0425 4.2 0 ]
[.7381 .0425 -4.2 -6.3 ]
[.7381 .0425 4.2 0 ]
[.97619 .0425 -4.2 -6.3 ]
[.97619 .0425 4.2 0 ]
[.01131 .1613 -12.6 -3.15 ]
[.01131 .1613 0 3.15 ]
[.01131 .26761 -12.6 -3.15 ]
[.01131 .26761 0 3.15 ]
[.01131 .37392 -12.6 -3.15 ]
[.01131 .37392 0 3.15 ]
[.01131 .48023 -12.6 -3.15 ]
[.01131 .48023 0 3.15 ]
[.01131 .58653 -12.6 -3.15 ]
[.01131 .58653 0 3.15 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .055 m
.2619 .06125 L
s
[(10)] .2619 .0425 0 1 Mshowa
.5 .055 m
.5 .06125 L
s
[(20)] .5 .0425 0 1 Mshowa
.7381 .055 m
.7381 .06125 L
s
[(30)] .7381 .0425 0 1 Mshowa
.97619 .055 m
.97619 .06125 L
s
[(40)] .97619 .0425 0 1 Mshowa
.125 Mabswid
.07143 .055 m
.07143 .05875 L
s
.11905 .055 m
.11905 .05875 L
s
.16667 .055 m
.16667 .05875 L
s
.21429 .055 m
.21429 .05875 L
s
.30952 .055 m
.30952 .05875 L
s
.35714 .055 m
.35714 .05875 L
s
.40476 .055 m
.40476 .05875 L
s
.45238 .055 m
.45238 .05875 L
s
.54762 .055 m
.54762 .05875 L
s
.59524 .055 m
.59524 .05875 L
s
.64286 .055 m
.64286 .05875 L
s
.69048 .055 m
.69048 .05875 L
s
.78571 .055 m
.78571 .05875 L
s
.83333 .055 m
.83333 .05875 L
s
.88095 .055 m
.88095 .05875 L
s
.92857 .055 m
.92857 .05875 L
s
.25 Mabswid
0 .055 m
1 .055 L
s
.02381 .1613 m
.03006 .1613 L
s
[(100)] .01131 .1613 1 0 Mshowa
.02381 .26761 m
.03006 .26761 L
s
[(120)] .01131 .26761 1 0 Mshowa
.02381 .37392 m
.03006 .37392 L
s
[(140)] .01131 .37392 1 0 Mshowa
.02381 .48023 m
.03006 .48023 L
s
[(160)] .01131 .48023 1 0 Mshowa
.02381 .58653 m
.03006 .58653 L
s
[(180)] .01131 .58653 1 0 Mshowa
.125 Mabswid
.02381 .08157 m
.02756 .08157 L
s
.02381 .10815 m
.02756 .10815 L
s
.02381 .13473 m
.02756 .13473 L
s
.02381 .18788 m
.02756 .18788 L
s
.02381 .21446 m
.02756 .21446 L
s
.02381 .24103 m
.02756 .24103 L
s
.02381 .29419 m
.02756 .29419 L
s
.02381 .32077 m
.02756 .32077 L
s
.02381 .34734 m
.02756 .34734 L
s
.02381 .4005 m
.02756 .4005 L
s
.02381 .42707 m
.02756 .42707 L
s
.02381 .45365 m
.02756 .45365 L
s
.02381 .5068 m
.02756 .5068 L
s
.02381 .53338 m
.02756 .53338 L
s
.02381 .55996 m
.02756 .55996 L
s
.02381 .02842 m
.02756 .02842 L
s
.02381 .00184 m
.02756 .00184 L
s
.02381 .61311 m
.02756 .61311 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.04762 .1613 m
.07143 .16059 L
.09524 .1923 L
.11905 .17231 L
.14286 .1616 L
.16667 .18436 L
.19048 .22478 L
.21429 .23903 L
.2381 .24904 L
.2619 .26757 L
.28571 .25667 L
.30952 .26017 L
.33333 .26807 L
.35714 .29174 L
.38095 .28742 L
.40476 .29421 L
.42857 .32015 L
.45238 .32602 L
.47619 .34089 L
.5 .33389 L
.52381 .33521 L
.54762 .3248 L
.57143 .36519 L
.59524 .38275 L
.61905 .40664 L
.64286 .40255 L
.66667 .40359 L
.69048 .3868 L
.71429 .38952 L
.7381 .40914 L
.7619 .42236 L
.78571 .39316 L
.80952 .41258 L
.83333 .43726 L
.85714 .448 L
.88095 .47079 L
.90476 .48189 L
.92857 .42769 L
.95238 .47354 L
.97619 .48534 L
s
.04762 .1613 m
.07143 .16078 L
.09524 .16869 L
.11905 .16139 L
.14286 .18053 L
.16667 .19351 L
.19048 .21224 L
.21429 .21641 L
.2381 .20947 L
.2619 .25003 L
.28571 .28323 L
.30952 .28143 L
.33333 .26739 L
.35714 .28475 L
.38095 .29917 L
.40476 .27519 L
.42857 .28558 L
.45238 .27888 L
.47619 .27775 L
.5 .24202 L
.52381 .27267 L
.54762 .27926 L
.57143 .30867 L
.59524 .27205 L
.61905 .28348 L
.64286 .29511 L
.66667 .31769 L
.69048 .29978 L
.71429 .29318 L
.7381 .31736 L
.7619 .33061 L
.78571 .33319 L
.80952 .33264 L
.83333 .33087 L
.85714 .32997 L
.88095 .34348 L
.90476 .3736 L
.92857 .38344 L
.95238 .3817 L
.97619 .36114 L
s
.04762 .1613 m
.07143 .19141 L
.09524 .17163 L
.11905 .16553 L
.14286 .1805 L
.16667 .22262 L
.19048 .21606 L
.21429 .19391 L
.2381 .19744 L
.2619 .19424 L
.28571 .18518 L
.30952 .18802 L
.33333 .17639 L
.35714 .18138 L
.38095 .1943 L
.40476 .21071 L
.42857 .22829 L
.45238 .23743 L
.47619 .23264 L
.5 .25616 L
.52381 .28326 L
.54762 .27222 L
.57143 .26964 L
.59524 .24697 L
.61905 .25569 L
.64286 .2821 L
.66667 .28657 L
.69048 .28339 L
.71429 .26361 L
.7381 .31279 L
.7619 .33892 L
.78571 .3073 L
.80952 .29338 L
.83333 .30391 L
.85714 .32985 L
.88095 .34556 L
.90476 .33923 L
.92857 .36578 L
.95238 .37919 L
.97619 .35347 L
s
.04762 .1613 m
.07143 .16303 L
.09524 .16089 L
.11905 .17412 L
.14286 .1814 L
.16667 .17589 L
.19048 .16914 L
.21429 .16547 L
.2381 .16386 L
.2619 .15492 L
.28571 .17597 L
.30952 .16525 L
.33333 .14767 L
.35714 .15345 L
.38095 .15012 L
.40476 .1533 L
.42857 .14518 L
.45238 .13303 L
.47619 .12672 L
.5 .11694 L
.52381 .1026 L
.54762 .08764 L
.57143 .09443 L
.59524 .08708 L
.61905 .08939 L
.64286 .0809 L
.66667 .10294 L
.69048 .09276 L
.71429 .101 L
.7381 .09194 L
.7619 .10587 L
.78571 .11209 L
.80952 .10969 L
.83333 .08682 L
.85714 .09582 L
.88095 .07981 L
.90476 .08414 L
.92857 .09834 L
.95238 .08445 L
.97619 .07225 L
s
.04762 .1613 m
.07143 .1736 L
.09524 .17702 L
.11905 .1503 L
.14286 .14635 L
.16667 .14534 L
.19048 .13846 L
.21429 .14123 L
.2381 .13061 L
.2619 .14202 L
.28571 .13457 L
.30952 .14294 L
.33333 .1441 L
.35714 .13129 L
.38095 .13125 L
.40476 .16113 L
.42857 .1508 L
.45238 .14265 L
.47619 .12736 L
.5 .12692 L
.52381 .0892 L
.54762 .05912 L
.57143 .06851 L
.59524 .08364 L
.61905 .11065 L
.64286 .1359 L
.66667 .13767 L
.69048 .163 L
.71429 .16702 L
.7381 .16485 L
.7619 .15811 L
.78571 .16919 L
.80952 .18412 L
.83333 .17493 L
.85714 .17502 L
.88095 .17978 L
.90476 .19066 L
.92857 .18531 L
.95238 .18584 L
.97619 .19024 L
s
.04762 .1613 m
.07143 .15848 L
.09524 .15751 L
.11905 .12456 L
.14286 .11711 L
.16667 .13582 L
.19048 .1462 L
.21429 .15025 L
.2381 .16053 L
.2619 .15582 L
.28571 .15022 L
.30952 .16873 L
.33333 .19145 L
.35714 .18177 L
.38095 .17519 L
.40476 .17781 L
.42857 .18879 L
.45238 .20477 L
.47619 .23783 L
.5 .23702 L
.52381 .21429 L
.54762 .21476 L
.57143 .21452 L
.59524 .23743 L
.61905 .2383 L
.64286 .24453 L
.66667 .21868 L
.69048 .19788 L
.71429 .20003 L
.7381 .2161 L
.7619 .19483 L
.78571 .18452 L
.80952 .20043 L
.83333 .23772 L
.85714 .25551 L
.88095 .25385 L
.90476 .28476 L
.92857 .28817 L
.95238 .29817 L
.97619 .27223 L
s
.04762 .1613 m
.07143 .16934 L
.09524 .1852 L
.11905 .17014 L
.14286 .19034 L
.16667 .20247 L
.19048 .17442 L
.21429 .17453 L
.2381 .16019 L
.2619 .15725 L
.28571 .12073 L
.30952 .1065 L
.33333 .08553 L
.35714 .09712 L
.38095 .12037 L
.40476 .12962 L
.42857 .14297 L
.45238 .1611 L
.47619 .15078 L
.5 .12738 L
.52381 .12757 L
.54762 .11373 L
.57143 .15181 L
.59524 .14634 L
.61905 .16104 L
.64286 .205 L
.66667 .1981 L
.69048 .19804 L
.71429 .18673 L
.7381 .16742 L
.7619 .19076 L
.78571 .20443 L
.80952 .20335 L
.83333 .22158 L
.85714 .23244 L
.88095 .20519 L
.90476 .20662 L
.92857 .20067 L
.95238 .21145 L
.97619 .23188 L
s
.04762 .1613 m
.07143 .14892 L
.09524 .13092 L
.11905 .13291 L
.14286 .10998 L
.16667 .09282 L
.19048 .1131 L
.21429 .10801 L
.2381 .10483 L
.2619 .08834 L
.28571 .1136 L
.30952 .09856 L
.33333 .0951 L
.35714 .0817 L
.38095 .07216 L
.40476 .07885 L
.42857 .04739 L
.45238 .04219 L
.47619 .03066 L
.5 .04969 L
.52381 .03932 L
.54762 .04498 L
.57143 .0433 L
.59524 .0554 L
.61905 .05483 L
.64286 .04689 L
.66667 .04266 L
.69048 .02769 L
.71429 .05754 L
.7381 .02926 L
.7619 .03783 L
.78571 .04182 L
.80952 .04619 L
.83333 .07744 L
.85714 .09541 L
.88095 .08024 L
.90476 .0799 L
.92857 .1006 L
.95238 .08187 L
.97619 .10255 L
s
.04762 .1613 m
.07143 .1675 L
.09524 .16897 L
.11905 .20659 L
.14286 .191 L
.16667 .18747 L
.19048 .20479 L
.21429 .23105 L
.2381 .25072 L
.2619 .24839 L
.28571 .27268 L
.30952 .2795 L
.33333 .23837 L
.35714 .24293 L
.38095 .25917 L
.40476 .25616 L
.42857 .2734 L
.45238 .27784 L
.47619 .28987 L
.5 .32665 L
.52381 .36168 L
.54762 .36952 L
.57143 .34571 L
.59524 .35747 L
.61905 .40591 L
.64286 .4241 L
.66667 .39044 L
.69048 .3781 L
.71429 .36182 L
.7381 .37433 L
.7619 .36429 L
.78571 .36223 L
.80952 .41167 L
.83333 .42809 L
.85714 .44917 L
.88095 .46918 L
.90476 .45385 L
.92857 .46507 L
.95238 .47517 L
.97619 .47327 L
s
.04762 .1613 m
.07143 .14226 L
.09524 .14685 L
.11905 .13291 L
.14286 .12986 L
.16667 .13714 L
.19048 .13744 L
.21429 .15836 L
.2381 .15066 L
.2619 .17498 L
.28571 .1962 L
.30952 .15576 L
.33333 .15575 L
.35714 .18894 L
.38095 .18092 L
.40476 .15101 L
.42857 .15329 L
.45238 .17539 L
.47619 .18053 L
.5 .17048 L
.52381 .14694 L
.54762 .13762 L
.57143 .13592 L
.59524 .12398 L
.61905 .13513 L
.64286 .13215 L
.66667 .10369 L
.69048 .06884 L
.71429 .1008 L
.7381 .09008 L
.7619 .08603 L
.78571 .10662 L
.80952 .11679 L
.83333 .1508 L
.85714 .17219 L
.88095 .16602 L
.90476 .17029 L
.92857 .17256 L
.95238 .18063 L
.97619 .1945 L
s
.04762 .1613 m
.07143 .16193 L
.09524 .17202 L
.11905 .18291 L
.14286 .20145 L
.16667 .18992 L
.19048 .19461 L
.21429 .19125 L
.2381 .18886 L
.2619 .19209 L
.28571 .20197 L
.30952 .20086 L
.33333 .18921 L
.35714 .20056 L
.38095 .2246 L
.40476 .2395 L
.42857 .25962 L
.45238 .22949 L
.47619 .23316 L
.5 .24857 L
.52381 .26091 L
.54762 .28667 L
.57143 .30001 L
.59524 .26829 L
.61905 .30657 L
.64286 .31253 L
.66667 .3031 L
.69048 .32842 L
.71429 .32566 L
.7381 .33187 L
.7619 .29549 L
.78571 .2984 L
.80952 .28816 L
.83333 .27749 L
.85714 .28996 L
.88095 .27721 L
.90476 .3019 L
.92857 .32526 L
.95238 .35932 L
.97619 .33902 L
s
.04762 .1613 m
.07143 .16956 L
.09524 .16982 L
.11905 .15105 L
.14286 .15851 L
.16667 .16526 L
.19048 .18015 L
.21429 .19723 L
.2381 .22207 L
.2619 .20653 L
.28571 .20066 L
.30952 .21771 L
.33333 .20586 L
.35714 .23073 L
.38095 .23331 L
.40476 .22742 L
.42857 .25636 L
.45238 .22638 L
.47619 .23577 L
.5 .2471 L
.52381 .2395 L
.54762 .25711 L
.57143 .23969 L
.59524 .23048 L
.61905 .24023 L
.64286 .2558 L
.66667 .2656 L
.69048 .27265 L
.71429 .26916 L
.7381 .31058 L
.7619 .31212 L
.78571 .32793 L
.80952 .31024 L
.83333 .3121 L
.85714 .32274 L
.88095 .3069 L
.90476 .28307 L
.92857 .27344 L
.95238 .2716 L
.97619 .29672 L
s
.04762 .1613 m
.07143 .17613 L
.09524 .15598 L
.11905 .16607 L
.14286 .15501 L
.16667 .12712 L
.19048 .13372 L
.21429 .14785 L
.2381 .14259 L
.2619 .15581 L
.28571 .1727 L
.30952 .20326 L
.33333 .19102 L
.35714 .19365 L
.38095 .20302 L
.40476 .20947 L
.42857 .19675 L
.45238 .2251 L
.47619 .23844 L
.5 .25848 L
.52381 .27512 L
.54762 .30114 L
.57143 .34042 L
.59524 .34847 L
.61905 .34991 L
.64286 .36343 L
.66667 .36831 L
.69048 .39475 L
.71429 .38353 L
.7381 .37139 L
.7619 .33629 L
.78571 .33262 L
.80952 .33874 L
.83333 .30093 L
.85714 .32743 L
.88095 .32204 L
.90476 .32714 L
.92857 .36358 L
.95238 .35877 L
.97619 .34294 L
s
.04762 .1613 m
.07143 .18086 L
.09524 .18772 L
.11905 .15744 L
.14286 .1618 L
.16667 .13099 L
.19048 .15792 L
.21429 .14448 L
.2381 .15524 L
.2619 .17293 L
.28571 .18737 L
.30952 .21016 L
.33333 .24431 L
.35714 .24259 L
.38095 .22784 L
.40476 .22709 L
.42857 .21366 L
.45238 .21098 L
.47619 .2076 L
.5 .19565 L
.52381 .18091 L
.54762 .15967 L
.57143 .1519 L
.59524 .15652 L
.61905 .16126 L
.64286 .1836 L
.66667 .1706 L
.69048 .21447 L
.71429 .19649 L
.7381 .18283 L
.7619 .19426 L
.78571 .20609 L
.80952 .22072 L
.83333 .19656 L
.85714 .17656 L
.88095 .18517 L
.90476 .19218 L
.92857 .16915 L
.95238 .15816 L
.97619 .13569 L
s
.04762 .1613 m
.07143 .16297 L
.09524 .14653 L
.11905 .13346 L
.14286 .14132 L
.16667 .12349 L
.19048 .09362 L
.21429 .08903 L
.2381 .07027 L
.2619 .05518 L
.28571 .07371 L
.30952 .08982 L
.33333 .09116 L
.35714 .06449 L
.38095 .0774 L
.40476 .06901 L
.42857 .06361 L
.45238 .0442 L
.47619 .03855 L
.5 .0403 L
.52381 .07075 L
.54762 .0484 L
.57143 .03891 L
.59524 .05174 L
.61905 .06377 L
.64286 .07506 L
.66667 .05922 L
.69048 .07108 L
.71429 .07447 L
.7381 .09189 L
.7619 .08398 L
.78571 .10286 L
.80952 .08376 L
.83333 .07813 L
.85714 .0828 L
.88095 .08975 L
.90476 .10533 L
.92857 .12983 L
.95238 .13301 L
.97619 .15521 L
s
.04762 .1613 m
.07143 .15781 L
.09524 .15734 L
.11905 .13603 L
.14286 .14248 L
.16667 .11206 L
.19048 .12139 L
.21429 .11801 L
.2381 .13319 L
.2619 .14753 L
.28571 .1545 L
.30952 .12544 L
.33333 .14147 L
.35714 .13146 L
.38095 .13463 L
.40476 .12056 L
.42857 .12019 L
.45238 .11693 L
.47619 .09656 L
.5 .07305 L
.52381 .08407 L
.54762 .10059 L
.57143 .14023 L
.59524 .1632 L
.61905 .16419 L
.64286 .17543 L
.66667 .1649 L
.69048 .15196 L
.71429 .13883 L
.7381 .13971 L
.7619 .1414 L
.78571 .17354 L
.80952 .18146 L
.83333 .20115 L
.85714 .19321 L
.88095 .1891 L
.90476 .18292 L
.92857 .19823 L
.95238 .20279 L
.97619 .17515 L
s
.04762 .1613 m
.07143 .17486 L
.09524 .17266 L
.11905 .16839 L
.14286 .16609 L
.16667 .18761 L
.19048 .20987 L
.21429 .18872 L
.2381 .18232 L
.2619 .1784 L
.28571 .17643 L
.30952 .20061 L
.33333 .18741 L
.35714 .17189 L
.38095 .14536 L
.40476 .16778 L
.42857 .17129 L
.45238 .17114 L
.47619 .1571 L
.5 .149 L
.52381 .1354 L
.54762 .15084 L
.57143 .13192 L
.59524 .13592 L
.61905 .15015 L
.64286 .14344 L
.66667 .08974 L
.69048 .07512 L
.71429 .05128 L
.7381 .03026 L
.7619 .02377 L
.78571 .01472 L
.80952 .03329 L
.83333 .0362 L
.85714 .05057 L
.88095 .07323 L
.90476 .07404 L
.92857 .05637 L
.95238 .05238 L
.97619 .05983 L
s
.04762 .1613 m
.07143 .16365 L
.09524 .20155 L
.11905 .19611 L
.14286 .20572 L
.16667 .18627 L
.19048 .22494 L
.21429 .24301 L
.2381 .2754 L
.2619 .28557 L
.28571 .27821 L
.30952 .28796 L
.33333 .27354 L
.35714 .26064 L
.38095 .26243 L
.40476 .25177 L
.42857 .26635 L
.45238 .23845 L
.47619 .23233 L
.5 .21263 L
.52381 .22522 L
.54762 .21083 L
.57143 .21732 L
.59524 .2363 L
.61905 .23509 L
.64286 .21373 L
.66667 .20655 L
.69048 .20269 L
.71429 .21058 L
.7381 .20739 L
.7619 .20433 L
.78571 .18315 L
.80952 .17027 L
.83333 .17507 L
.85714 .14148 L
.88095 .14865 L
.90476 .1682 L
.92857 .17425 L
.95238 .14876 L
.97619 .14545 L
s
.04762 .1613 m
.07143 .17081 L
.09524 .18466 L
.11905 .18754 L
.14286 .17821 L
.16667 .17697 L
.19048 .19581 L
.21429 .20582 L
.2381 .1815 L
.2619 .17856 L
.28571 .15255 L
.30952 .15498 L
.33333 .13995 L
.35714 .12972 L
.38095 .11684 L
.40476 .10985 L
.42857 .11422 L
.45238 .11375 L
.47619 .11917 L
.5 .11168 L
.52381 .11197 L
.54762 .10223 L
.57143 .09212 L
.59524 .09615 L
.61905 .08926 L
.64286 .09676 L
.66667 .10358 L
.69048 .09657 L
.71429 .10684 L
.7381 .10436 L
.7619 .14603 L
.78571 .14801 L
.80952 .18029 L
.83333 .19323 L
.85714 .17301 L
.88095 .17114 L
.90476 .14504 L
.92857 .14094 L
.95238 .1244 L
.97619 .12767 L
s
.04762 .1613 m
.07143 .15578 L
.09524 .17273 L
.11905 .19527 L
.14286 .17567 L
.16667 .1905 L
.19048 .19234 L
.21429 .22966 L
.2381 .22953 L
.2619 .22401 L
.28571 .23407 L
.30952 .25315 L
.33333 .23157 L
.35714 .19997 L
.38095 .17723 L
.40476 .20474 L
.42857 .21841 L
.45238 .24754 L
.47619 .243 L
.5 .26715 L
.52381 .2937 L
.54762 .32519 L
.57143 .3445 L
.59524 .35162 L
.61905 .36119 L
.64286 .38587 L
.66667 .40268 L
.69048 .42506 L
.71429 .46298 L
.7381 .53025 L
.7619 .53413 L
.78571 .55977 L
.80952 .60332 L
.83333 .59012 L
.85714 .57761 L
.88095 .54336 L
.90476 .52633 L
.92857 .52649 L
.95238 .55274 L
.97619 .56169 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{223.188, 137.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  RenderingOptions->{"RenderThickness"->False},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004F0000Za000`40O003h00Oogoo5goo003oOolG
Ool00?mooaMoo`004goo00<007ooOol0ogoo0Goo000COol2003oOol2Ool001=oo`03001oogoo0?mo
o`5oo`004goo00<007ooOol0ogoo0Goo000COol00`00Oomoo`36Ool3000gOol001=oo`03001oogoo
0<=oo`<000=oo`03001oogoo03Aoo`004goo00<007ooOol0>7oo0`000Woo0P00=goo10000Woo0P00
>7oo0`000goo0P000goo0P001goo00<007ooOol0:Woo00@007ooOomoo`8000=oo`004goo00<007oo
Ool0>Goo00@007ooOol0009oo`03001oogoo03Eoo`05001oogooOol00002Ool00`00Oomoo`0hOol0
1000Ool007oo10002Woo00<007ooOol09goo100000Eoo`00Oomoo`00009oo`004goo0P00>Woo00@0
07ooOol0009oo`03001oogoo039oo`<00005Ool007ooOol00002Ool00`00Oomoo`0[Ool2000;Ool4
00000goo001oo`0<Ool00`00Oomoo`0VOol00`00Ool00002Ool01000Oomoo`000Woo000COol00`00
Oomoo`0iOol01000Oomoo`000Woo00<007ooOol0;goo0`000goo00D007oo001oo`00009oo`03001o
ogoo02Yoo`03001oo`0000Uoo`D00004Ool00000000=Ool4000TOol00`00Ool00002Ool01000Oomo
o`000Woo000COol00`00Oomoo`0iOol01000Oomoo`000Woo00<007ooOol0;Goo0P000Woo1`0000Eo
o`00Oomoo`0002Yoo`8000=oo`03001oogoo00Moo`800006Ool007ooOol007oo1@0037oo10008Goo
0P000Woo00@007ooOol0009oo`004goo00<007ooOol0>7oo0P000goo0P00;Woo0P0000Aoo`000000
00=oo`P0009oo`D000Moo`@001Qoo`03001oogoo00Aoo`03001oogoo00Aoo`<000=oo`8000Moo`D0
00]oo`80021oo`04001oogooOol20003Ool001=oo`03001oogoo06Yoo`H000Yoo`800004Ool00000
0005Ool=000BOol40008Ool00`00Oomoo`03Ool2000BOol3000:Ool00`00Oomoo`0VOol001=oo`03
001oogoo06Qoo`80009oo`03001oogoo00aoo`8000Yoo`8000Moo`@000]oo`D000eoo`07001oogoo
Ool007oo00005Goo00<007ooOol027oo0P009Woo000COol00`00Oomoo`1XOol01000Oomoo`004Goo
00<007ooOol01Woo00<007ooOol02Goo10001Woo0`004goo0`0000=oo`00Ool05Goo00<007ooOol0
2Woo00<007ooOol04goo1P002Woo000<Oooo000;000001=oo`03001oogoo00Uoo`03001oogoo00Yo
o`03001oogoo00Uoo`03001oogoo00Yoo`03001oogoo00Moo`800004Ool007oo0009Ool00`00Oomo
o`0:Ool00`00Oomoo`0:Ool01@00Oomoogoo00000Woo00<007ooOol00Woo00<007ooOol02Woo00@0
07ooOol000Eoo`03001oogoo00<000Yoo`04001oogooOol20007Ool00`00Oomoo`02Ool30004Ool0
1@00Oomoogoo00000Woo00<007ooOol00goo00<007ooOol02Woo00<007ooOol01Goo00D007ooOomo
o`0000Qoo`05001oogooOol00009Ool00`00Ool0000<Ool00`00Oomoo`04Ool001=oo`03001oogoo
03Qoo`05001oogooOol00002Ool00`00Oomoo`0COol2000>Ool01000Ool000004Woo00D007ooOomo
o`0000=oo`05001oogooOol00002Ool3000=Ool20009Ool01000Oomoogoo0P001Goo00<007ooOol0
1goo00<007ooOol04goo00<007ooOol02Woo00<007ooOol02Goo00<007ooOol03Goo00<007ooOol0
17oo000COol00`00Oomoo`0fOol20008Ool00`00Oomoo`0BOol00`00Ool0000<Ool3000IOol00`00
Ool00003Ool00`00Oomoo`04Ool3000<Ool30004Ool20006Ool20002Ool00`00Oomoo`0NOol00`00
Oomoo`0;Ool00`00Oomoo`07Ool00`00Oomoo`0EOol001=oo`03001oogoo03Aoo`8000]oo`03001o
ogoo011oo`04001oogooOol60002Ool5000LOol01@00Oomoogoo00002Woo00<007ooOol037oo0P00
00=oo`00Ool02Goo1@007goo00<007ooOol02goo00<007ooOol01Goo00<007ooOol03Woo0P001Woo
000COol00`00Oomoo`0cOol00`00Oomoo`0<Ool2000?Ool00`00Oomoo`02Ool30002Ool40002Ool0
0`00Oomoo`0EOol20007Ool00`00Oomoo`0:Ool2000>Ool00`00Oomoo`08Ool20004Ool4000LOol0
0`00Oomoo`0;Ool7000@Ool00`00Oomoo`06Ool001=oo`03001oogoo039oo`03001oogoo00moo`03
001oogoo00]oo`04001oogooOol20003Ool20004Ool2000GOol00`00Oomoo`020005Ool00`00Oomo
o`0<Ool00`00Oomoo`0EOol200000goo001oo`06Ool00`00Oomoo`0GOol80007Ool8000>Ool20009
Ool001=oo`80039oo`03001oogoo015oo`8000Yoo`03001oogoo008002=oo`03001oogoo00=oo`80
009oo`03001oogoo00ioo`03001oogoo00Qoo`<000Moo`<000=oo`03001oogoo00Eoo`03001oogoo
01=oo`<000Eoo`03001oogoo00D00003Ool0000000H00003Ool007oo00Uoo`<000]oo`004goo00<0
07ooOol0<7oo00<007ooOol057oo00<007ooOol017oo0`0000=oo`00000097oo00<007ooOol01Woo
0P0047oo1@0017oo0`000goo00<007ooOol00goo00@007ooOol000Aoo`03001oogoo00Eoo`03001o
ogoo009oo`D000Yoo`03001oogoo00Aoo`800003Ool0000000Eoo`<000Ioo`<000Qoo`<00003Ool0
07oo00Uoo`004goo00<007ooOol0;7oo10002goo0P002goo2`0097oo00<007ooOol01goo00<007oo
00000Woo1P0017oo0`000Woo00<007ooOol01@001Woo00D007ooOomoo`00009oo`03001oogoo00Ao
o`03001oogoo00Eoo`H000=oo`03001oogoo00Ioo`03001oogoo00Aoo`03001oogoo00@000=oo`80
009oo`8000Ioo`8000Eoo`<000Aoo`03001oogoo00Moo`004goo00<007ooOol08Goo0`001Goo0`00
3Goo0P000Woo00<007ooOol037oo10000goo0P008Goo00<007ooOol027oo00<007ooOol00P001Woo
1P0017oo0`0017oo0`0017oo00H007ooOol007oo0002Ool40002Ool00`00Oomoo`02Ool50007Ool2
0005Ool00`00Oomoo`04Ool00`00Oomoo`04Ool200000goo001oo`05Ool00`00Oomoo`05Ool20000
0goo00000008Ool00`00Oomoo`06Ool001=oo`03001oogoo021oo`05001oogooOol00004Ool00`00
Oomoo`0<Ool00`00Oomoo`03Ool00`00Oomoo`06Ool50009Ool00`00Oomoo`0LOol2000:Ool01000
Ool000000Woo00<007ooOol00`001Woo100000=oo`00Ool01goo100000=oo`0000000Woo1P0000=o
o`0000000P0000=oo`0000000P000goo0P001Woo0P000goo00<007ooOol017oo00<007ooOol01goo
00<007ooOol01Woo0P001Woo0`002Woo00<007ooOol01Goo000COol00`00Oomoo`0NOol20005Ool2
00000goo001oo`0<Ool20007Ool00`00Oomoo`04Ool01@00Oomoogoo00002goo00<007ooOol06Woo
00<007ooOol02Goo0`0017oo0`003Woo00<007ooOol02Woo0`0000=oo`0000000P000goo0`0000Ao
o`00000000Qoo`8000Eoo`800003Ool007oo00Eoo`03001oogoo01=oo`03001oogoo00Aoo`03001o
ogoo00Yoo`03001oogoo00Aoo`004goo00<007ooOol07Goo00<007ooOol01goo00<007ooOol027oo
10002Woo00@007ooOomoo`8000=oo`03001oogoo00]oo`03001oogoo01Moo`8000aoo`03001oogoo
009oo`800003Ool007oo00moo`03001oogoo00Yoo`04001oo`000009Ool20003Ool40005Ool30003
Ool20006Ool00`00Oomoo`0EOol00`00Oomoo`0GOol001=oo`8001eoo`03001oogoo00Moo`03001o
o`0000=oo`H000moo`03001oo`0000=oo`8000moo`03001oogoo00Eoo`D000]oo`03001oogoo00Yo
o`8000=oo`8000Aoo`03001oogoo00ioo`03001oogoo00Uoo`03001oo`0000aoo`<000=oo`03001o
ogoo00Ioo`<00003Ool0000000D001Qoo`03001oogoo01Moo`004goo00<007ooOol077oo00<007oo
Ool00goo10000goo0`005Woo00<007ooOol00P004Woo00@007ooOomoo`<000Eoo`<000=oo`H000Uo
o`P000Aoo`800003Ool007oo00moo`8000Uoo`8001Aoo`03001oogoo00Qoo`L001Yoo`03001oogoo
01Ioo`004goo00<007ooOol06goo0`0017oo00<007oo00000`006goo0P005Goo0`002goo1`000Woo
0`000goo1@0000=oo`00Ool01Woo00D007oo001oo`00019oo`03001oogoo00Eoo`8001Eoo`03001o
ogoo00moo`03001oogoo01Qoo`03001oogoo01Eoo`004goo00<007ooOol067oo0`000goo00H007oo
Ool007oo0004Ool9000AOol00`00Oomoo`0DOol01000Ool0000017oo2P002Goo0`0000=oo`000000
0Woo00<007ooOol017oo0P000goo00<007oo00004Woo00<007ooOol017oo00<007ooOol05Goo00<0
07ooOol03Woo00<007ooOol06Goo00<007ooOol057oo000COol00`00Oomoo`0FOol200000goo001o
o`04Ool4000>Ool00`00Oomoo`0>Ool00`00Oomoo`0BOol20004Ool4000EOol20004Ool00`00Oomo
o`03Ool00`00Oomoo`04Ool00`00Ool00007Ool30008Ool00`00Oomoo`02Ool00`00Oomoo`0FOol0
0`00Oomoo`0?Ool00`00Oomoo`0IOol00`00Oomoo`05Ool50009Ool001=oo`03001oogoo01Ioo`03
001oo`0000Ioo`800004Ool00000000=Ool2000=Ool00`00Oomoo`04Ool2000<Ool00`00Oomoo`05
Ool01000Ool000004Goo3`001goo00@007ooOol000Aoo`8000=oo`8000Moo`03001oogoo008001Uo
o`03001oogoo00moo`03001oogoo01Uoo`03001oogoo00=oo`03001oogoo00=oo`<000Ioo`004goo
00<007ooOol03goo1@0000Eoo`00Ool007oo00L00003Ool00000009oo`<000aoo`<000Uoo`03001o
ogoo00Eoo`04001oo`000008Ool:0004Ool2000<Ool40006Ool00`00Oomoo`0=Ool00`00Ool00005
0006Ool20004Ool5000KOol00`00Oomoo`0>Ool00`00Oomoo`0JOol5000?Ool001=oo`8000moo`03
001oogoo00=oo`D000Eoo`H000Aoo`<000Moo`800003Ool0000000<000Ioo`D0009oo`03001oogoo
009oo`03001oogoo009oo`<000Eoo`<000Qoo`8000Moo`<00003Ool007oo00Ioo`03001oogoo00Ao
o`<000Qoo`@000=oo`@000Aoo`@000=oo`@001Uoo`03001oogoo00moo`03001oogoo01]oo`03001o
ogoo00@000Aoo`03001oogoo00Aoo`004goo00<007ooOol037oo0P001Goo0`000goo0`0000Aoo`00
Oomoo`d000=oo`8000Eoo`D0009oo`<000Aoo`<000Eoo`D000Yoo`03001oogoo00Moo`8000=oo`80
009oo`8000Moo`03001oogoo00Aoo`05001oogooOol00002Ool60002Ool00`00Oomoo`05Ool00`00
Oomoo`09Ool00`00Oomoo`03000;Ool;00000goo001oo`0@Ool00`00Oomoo`0IOol00`00Oomoo`05
Ool01000Oomoo`001goo000COol00`00Oomoo`08Ool60003Ool40007Ool00`00Oomoo`020003Ool5
0002Ool00`00Ool000020002Ool50003Ool400000goo001oo`04Ool00`00Oomoo`08000=Ool00`00
Oomoo`08Ool30002Ool20008Ool00`00Oomoo`03Ool20004Ool30003Ool01P00Oomoo`00Ool000Qo
o`8000Ioo`<000Ioo`03001oogoo00Ioo`8000]oo`<0011oo`03001oogoo00Ioo`<000aoo`@000Uo
o`8000Qoo`004goo00<007ooOol01goo00@007ooOol000=oo`<00003Ool0000000Eoo`/0009oo`03
001oogoo00@00003Ool00000008000Eoo`8000=oo`<000Eoo`03001oogoo00Aoo`03001oogoo00@0
00Uoo`800003Ool007oo00Moo`800004Ool000000009Ool00`00Oomoo`03Ool00`00Oomoo`02Ool2
0003Ool00`00Ool00003Ool01000Oomoo`000goo0`000goo00<007ooOol00`002Woo00<007ooOol0
17oo00<007ooOol03Goo1@003Goo00<007ooOol01Goo00<007ooOol00`001Woo0`003Goo10001Woo
000COol00`00Oomoo`06Ool01000Ool000000goo0P000goo00<007ooOol0100017oo00<007oo0000
1goo00<007ooOol00Woo1@0000=oo`0000000P000goo00H007ooOomoo`00Ool50005Ool20002Ool0
0`00Oomoo`02Ool30003Ool80002Ool800000goo001oo`07Ool20004Ool20004Ool00`00Oomoo`04
Ool00`00Oomoo`03Ool00`00Oomoo`030003Ool01000Oomoogoo0P003Woo00@007ooOomoo`80015o
o`04001oogooOol3000;Ool20004Ool00`00Oomoo`04Ool20004Ool00`00Oomoo`09Ool40002Ool0
0`00Oomoo`05Ool001=oo`03001oogoo00Eoo`<000Aoo`03001oogoo009oo`06001oogoo001oo`00
0Woo100000=oo`00Ool027oo00@007oo0000009oo`03001oogoo00<00003Ool0000000=oo`L00003
Ool0000000D0009oo`8000Qoo`<0009oo`06001oogooOol007oo100000=oo`0000000P000goo00<0
07ooOol00Woo0`000goo0`001Goo00<007ooOol02Woo1P001Goo0P004Woo0`0057oo00<007ooOol0
0Woo00<007ooOol02goo00@007ooOol000Uoo`04001oogoo000;Ool01000Oomoo`0017oo00<007oo
Ool017oo000017oo000000000Woo0P0017oo0P001Goo00<007ooOol017oo400000=oo`0000001000
00Aoo`00000000Yoo`03001oogoo00=oo`80009oo`80009oo`L000Aoo`04001oogoo0002Ool80009
Ool00`00Oomoo`03Ool01000Ool007oo0P0017oo00H007ooOomoo`00Ool20003Ool30007Ool00`00
Oomoo`08Ool30006Ool600000goo001oo`0?Ool3000BOol300000goo001oo`03Ool00`00Oomoo`0;
Ool2000;Ool2000;Ool01000Ool0000037oo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`03
001oogoo009oo`8000AooaH00004Ool000000002Ool3000BOol60004Ool01000Oomoo`0017oo00@0
07oo000000Eoo`03001oogoo00Ioo`03001oogoo00Eoo`03001oo`0000Moo`05001oogooOol00003
Ool20009Ool00`00Oomoo`06Ool3000:Ool00`00Oomoo`02Ool4000<Ool20003Ool3000<Ool30003
Ool30004Ool00`00Oomoo`0:Ool2000;Ool2000:Ool3000>Ool0009oo`04001oogoo0002Ool01000
Oomoo`000Woo00<007ooOol00Woo00<007ooOol00goo00<007oo00002@000Woo3@0017oo0P0027oo
2P000goo0P0017oo0P001Woo0P001goo00<007ooOol017oo00<007ooOol01Goo00<007oo00002Goo
00<007ooOol00goo00<007ooOol027oo00<007ooOol01Woo00<007ooOol037oo2@002Goo0P0027oo
0`0017oo1@002Goo0P000goo00<007ooOol02Goo00<007oo00001Woo1@0000=oo`00Ool01Woo0P00
4Goo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`03001oogoo009oo`03001oogoo00Aoo`03
001oo`0001<000=oo`8000Eoo`800003Ool0000000<000Eoo`8000Moo`8000=oo`<000=oo`800003
Ool0000000Moo`03001oogoo009oo`03001oogoo00Eoo`04001oogooOol40006Ool00`00Oomoo`02
0009Ool20008Ool00`00Oomoo`0DOol40005Ool2000=Ool40002Ool2000;Ool300000goo001oo`04
0005Ool01@00Oomoogoo00000Woo0`000Woo00<007ooOol01P000Woo0P004Woo0002Ool01000Oomo
o`000Woo00@007ooOol0009oo`03001oogoo009oo`03001oogoo00Eoo`03001oo`0001<000=oo`80
00Aoo`<000Uoo`03001oogoo00Qoo`@0009oo`@00003Ool007oo009oo`03001oogoo00Eoo`04001o
o`000007Ool00`00Oomoo`06Ool90009Ool20009Ool00`00Oomoo`0EOol00`00Oomoo`0300000goo
00000002Ool2000@Ool01000Oomoo`002goo0`0000=oo`00Ool00goo0`0000=oo`00Ool00goo2000
2Goo1@004Goo00000goo00000003Ool20004Ool20005Ool00`00Oomoo`06Ool00`00Ool0000<0003
Ool600000goo000000030003Ool9000<Ool70002Ool00`00Oomoo`03Ool00`00Oomoo`060004Ool<
000:Ool50003Ool2000:Ool00`00Oomoo`0GOol01P00Oomoo`00Ool0009oo`800003Ool007oo00io
o`03001oogoo009oo`03001oogoo00Yoo`@000Eoo``000aoo`03001oogoo009oo`<000Moo`03001o
ogoo00Aoo`004goo00<007ooOol01Woo00<007ooOol01`0000=oo`0000000`000goo1P0000=oo`00
00000Woo00@007ooOomoo`<000Yoo``0009oo`03001oo`0000Eoo`<000=oo`L0009oo`8000Qoo`80
00eoo`<000Yoo`8001]oo`05001oogoo001oo`020004Ool00`00Oomoo`0;Ool00`00Oomoo`04Ool0
0`00Oomoo`08Ool01000Oomoogoo1P001Goo0P0017oo1@001goo00<007ooOol01Woo0`0017oo00<0
07ooOol017oo000COol00`00Oomoo`07Ool01000Oomoogoo100000=oo`0000000P000Woo1@0000=o
o`00Ool01@000Woo0`000goo00<007ooOol017oo1`001Woo00<007oo000010000goo0P000Woo00D0
07ooOol007oo00<0009oo`<0009oo`03001oogoo00Moo`8001Moo`03001oogoo01]oo`05001oogoo
Ool00006Ool00`00Oomoo`0:Ool01000Oomoogoo0`000Woo00<007ooOol01Goo0`0017oo0`001Woo
0P0027oo2@000Woo1P0017oo00@007ooOol000Moo`004goo0P002Goo00<007oo00000Woo00<007oo
00000P0000=oo`00000010000Woo00@007oo001oo`D0009oo`8000Aoo`03001oogoo00/000Eoo`80
0004Ool00000Ool40005Ool40003Ool200000goo00000003Ool00`00Oomoo`08Ool2000COol2000O
Ool00`00Oomoo`08Ool00`00Oomoo`07Ool20002Ool20003Ool200000goo001oo`02Ool30003Ool0
0`00Oomoo`03Ool30003Ool00`00Oomoo`07Ool700000goo000000020006Ool70008Ool001=oo`03
001oogoo00Uoo`03001oogoo009oo`03001oo`00009oo`08001oo`00Ool007oo001oo`@00003Ool0
000000P0009oo`H00003Ool007oo00Eoo`D0009oo`04001oo`00Ool20004Ool80004Ool01000Oomo
o`000goo00<007ooOol02Goo00<007ooOol03goo00<007ooOol07goo00<007ooOol027oo00<007oo
Ool017oo0`000Woo0P001goo1`001Goo0P0017oo00D007oo001oo`0000Moo`<000Moo`800003Ool0
000000]oo`@000Ioo`004goo00<007ooOol03Woo00<007ooOol00goo10000Woo00@007ooOomoo`<0
0003Ool0000000P00003Ool0000000@000Ioo`@00003Ool00000009oo`04001oo`00Ool200000goo
000000030005Ool50003Ool20002Ool00`00Oomoo`05Ool20002Ool00`00Oomoo`0<Ool2000SOol0
0`00Oomoo`08Ool01000Oomoogoo0P000goo0P002goo1@001goo00<007ooOol00Woo00<007oo0000
17oo10003goo00<007ooOol01goo00D007ooOomoo`0000Ioo`004goo00<007ooOol03goo00<007oo
Ool010000Woo0P0000=oo`00Ool00P0000Aoo`00001oo`800003Ool00000009oo`<00006Ool00000
Ool007oo1P001goo100000=oo`00Ool010002Woo00<007ooOol00`000Woo0P0000=oo`00Ool017oo
00<007oo00000goo00<007ooOol02Goo0P009Woo00<007ooOol00goo40002Goo00<007oo00001@00
1Woo00<007ooOol00Woo0P0000Aoo`00000001=oo`H000=oo`03001oogoo00Uoo`004goo00<007oo
Ool047oo0P000goo00<007oo00000Woo100000Ioo`00Oomoo`00Ool50002Ool01P00Ool00000Ool0
00eoo`<00003Ool0000000D000aoo`03001oogoo00=oo`<00004Ool007oo0002Ool20003Ool01@00
Oomoogoo000027oo0P00:7oo00<007ooOol00`0017oo200017oo00<007ooOol017oo100017oo2P00
0goo0`004goo0`000goo1P0037oo000COol00`00Oomoo`0FOol00`00Oomoo`03Ool01@00Oomoogoo
000017oo20000Woo00<007oo00002Goo0`001goo00<007oo00001Woo0P0017oo00<007oo00001goo
1P001Woo00D007ooOomoo`0000=oo`@002]oo`8000=oo`D00004Ool007ooOol40004Ool70009Ool2
0005Ool200000goo001oo`0=Ool80009Ool2000=Ool001=oo`03001oogoo021oo`03001oogoo00=o
o`05001oo`00001oo`020005Ool01000Ool000000goo0P002goo00<007oo00000Woo0`000Woo00@0
07ooOol000=oo`03001oogoo00Moo`D000Eoo`L001Aoo`@001Uoo`<000=oo`06001oogoo001oo`00
17oo100000=oo`00Ool00Woo00<007ooOol02Woo00<007ooOol00goo0P003goo00<007ooOol047oo
00<007ooOol02Woo000COol2000ROol00`00Oomoo`03Ool00`00Ool000030004Ool300001Goo001o
ogoo00003Woo10001Woo00<007oo000017oo00<007ooOol02Goo00<007oo00001@000goo00<007oo
Ool027oo0`0017oo3P004Goo0`001Goo00<007ooOol00Woo00<007ooOol02Goo00<007oo00003Woo
0P000goo00<007oo00003Goo00<007ooOol04Woo00<007ooOol02Goo000COol00`00Oomoo`0QOol0
0`00Oomoo`02Ool400000goo001oo`040003Ool01000Ool0000047oo0P0027oo00<007ooOol017oo
00<007ooOol02Goo0P0000=oo`00Ool00Woo00D007ooOomoo`0000Qoo`04001oogooOol2000017oo
001oogoo0P0027oo0`003goo00<007ooOol01Goo00<007ooOol047oo00<007ooOol03goo00L007oo
001oo`00Ool0000;Ool00`00Oomoo`0DOol00`00Oomoo`08Ool001=oo`03001oogoo029oo`@0009o
o`07001oogooOol007oo00001Woo00<007oo00004Woo00<007ooOol017oo00<007oo00001Woo00<0
07ooOol01Woo0P0000Aoo`00Ool000Aoo`05001oogooOol00006Ool00`00Oomoo`04Ool200000goo
001oo`0<Ool00`00Oomoo`0;Ool00`00Oomoo`06Ool00`00Oomoo`0SOol01000Oomoo`000Woo0P00
27oo00<007ooOol05Woo00<007ooOol01goo000COol00`00Oomoo`0YOol20002Ool20008Ool00`00
Ool00005000<Ool00`00Oomoo`03Ool01000Oomoo`001Woo0P000Woo100017oo00<007oo000017oo
0P000Woo00<007ooOol00Woo00<007ooOol017oo00@007ooOol000moo`03001oogoo00Uoo`03001o
ogoo00Ioo`03001oogoo02Qoo`04001oogooOol30004Ool00`00Oomoo`0HOol00`00Oomoo`06Ool0
01=oo`03001oogoo02]oo`03001oogoo00/000Ioo`<000Yoo`05001oogooOol00004Ool400000goo
000000070005Ool30003Ool60002Ool00`00Oomoo`04Ool00`00Oomoo`0BOol40006Ool00`00Oomo
o`06Ool00`00Oomoo`0ZOol00`00Oomoo`03Ool200000goo001oo`0KOol00`00Oomoo`05Ool001=o
o`03001oogoo02aoo`800003Ool0000000Qoo`03001oogoo00Ioo`8000Qoo`04001oo`000009Ool7
0002Ool00`00Oomoo`06Ool40003Ool50005Ool00`00Oomoo`0BOol30002Ool20003Ool00`00Oomo
o`06Ool00`00Oomoo`0[Ool00`00Oomoo`05Ool00`00Oomoo`0LOol00`00Oomoo`04Ool001=oo`03
001oogoo02eoo`80009oo`03001oogoo00Eoo`03001oogoo00Qoo`03001oogoo00Ioo`03001oo`00
008000Eoo`8000Moo`800003Ool007oo00Iooa4001=oo`8000=oo`T000Ioo`03001oogoo02eoo`80
02]oo`004goo0P00;goo1@001goo00<007ooOol027oo00<007ooOol00goo0P0000=oo`0000000Woo
0`0000=oo`00Ool02Woo0`001Woo0P0000=oo`0000001Goo1@0000=oo`00Ool00goo0`002Woo0P00
2goo0P0000Aoo`000000009oo`03001oogoo031oo`03001oogoo02Qoo`004goo00<007ooOol0;goo
00<007ooOol00`001Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`T000eoo`8000Eoo`04001o
o`000003Ool400000goo00000003000017oo000000000goo00<007ooOol01Goo0P003goo0P000Woo
0P00=7oo0P00:7oo000COol00`00Oomoo`0_Ool00`00Oomoo`02Ool30002Ool4000:Ool00`00Ool0
0003Ool00`00Oomoo`07Ool00`00Oomoo`0;Ool20003Ool200000goo000000030004Ool00`00Ool0
00040007Ool20004Ool00`00Oomoo`07Ool30007Ool00`00Oomoo`0gOol00`00Oomoo`0UOol001=o
o`03001oogoo031oo`03001oogoo009oo`@000=oo`03001oogoo00Uoo`03001oogoo009oo`03001o
ogoo00Qoo`8000Moo`<0009oo`800005Ool00000Ool0000:Ool01000Ool000000Woo0P001goo00@0
07oo000000Uoo`04001oogooOol30005Ool2000hOol00`00Oomoo`0TOol001=oo`03001oogoo035o
o`03001oogoo00Aoo`80009oo`8000=oo`D000Ioo`03001oogoo00]oo`04001oogooOol70002Ool3
00000goo001oo`0;Ool400000goo001oo`020006Ool00`00Oomoo`08Ool00`00Oomoo`05Ool00`00
Oomoo`04Ool2000gOol7000OOol001=oo`03001oogoo039oo`03001oogoo00Eoo`03001oogoo00@0
00Eoo`H000Qoo`D0009oo`@000Eoo`<00004Ool007oo000=Ool200001Goo000007oo00000Woo00<0
07ooOol03Goo00<007ooOol01goo00<007ooOol01Goo0P00>goo00<007ooOol07Goo00001Goo0000
0000Ool010000goo0P001Goo00<007ooOol0<Woo00<007ooOol01Woo00<007oo00000P002goo0`00
1Goo00<007ooOol00goo0`002Goo00D007oo001oo`0000ioo`04001oo`00Ool2000017oo001oo`00
3Woo00<007ooOol02Goo00<007ooOol01Woo0P0037oo0P00;7oo00<007ooOol077oo0002Ool01@00
Oomoogoo000017oo00@007ooOol000Aoo`8003Aoo`03001oogoo00Ioo`03001oogoo008000Uoo`04
001oogooOol5000COol00`00Ool0000>Ool01P00Oomoogoo001oo`<0009oo`03001oogoo00Yoo`03
001oogoo009oo`03001oogoo00Ioo`03001oogoo00Moo`<000Qoo`03001oo`0002eoo`03001oogoo
01]oo`000Woo00<007ooOol00Woo00D007ooOomoo`00009oo`03001oogoo009oo`03001oogoo03Ao
o`03001oogoo00Qoo`03001oo`0000Qoo`05001oo`00001oo`03000EOol00`00Oomoo`0<Ool00`00
Oomoo`02Ool01P00Oomoo`00Ool0009oo`03001oogoo00T000Eoo`8000Qoo`03001oogoo00Uoo`<0
00=oo`H002eoo`03001oogoo00Yoo`@000Eoo`03001oogoo00Aoo`000Woo00<007ooOol00goo00@0
07ooOol0009oo`03001oogoo009oo`03001oogoo03Eoo`8000Uoo`03001oo`0000<000=oo`03001o
o`00009oo`<00003Ool007oo00ioo`8000Aoo`@000Yoo`03001oogoo00=oo`05001oogoo001oo`05
000=Ool50007Ool00`00Oomoo`0;Ool40003Ool2000]Ool00`00Oomoo`05Ool50003Ool00`00Oomo
o`03Ool00`00Oomoo`04Ool0009oo`03001oogoo00=oo`04001oogoo0002Ool00`00Oomoo`02Ool0
0`00Oomoo`0gOol30007Ool30002Ool500000goo00000003Ool00`00Ool0000=Ool00`00Oomoo`02
0006Ool:0007Ool00`00Oomoo`04000017oo000000002Woo0P0000Aoo`00Oomoo`8000Ioo`03001o
ogoo00]oo`03001oogoo00Aoo`8001]oo`<000Yoo`8000=oo`04001oogooOol30009Ool01@00Oomo
ogoo00001goo00000goo00000002Ool30004Ool20005Ool00`00Oomoo`0jOol20002Ool300000goo
000000050002Ool00`00Oomoo`04Ool00`00Ool0000;Ool00`00Oomoo`03Ool20002Ool30003Ool3
000;Ool00`00Oomoo`020003Ool2000:Ool01@00Oomoogoo00000goo0P001Goo10001Goo10001goo
0P006Goo0P000goo0`001Goo0P000Woo00@007ooOomoo`<000eoo`03001oo`0000Qoo`004goo00<0
07ooOol0?7oo0P0027oo0P0000=oo`00000027oo00@007oo000000=oo`@00003Ool007oo00Moo`80
00Uoo`8000Yoo`05001oogooOol00003Ool20008Ool20003Ool00`00Oomoo`03Ool20007Ool5000<
Ool2000FOol20008Ool200000goo00000005Ool01000Oomoogoo100037oo00<007ooOol01goo000C
Ool00`00Oomoo`18Ool00`00Oomoo`09Ool00`00Oomoo`030004Ool3000DOol00`00Oomoo`08Ool0
1@00Oomoogoo000017oo0P001Goo0P001Goo00<007ooOol017oo00<007ooOol05Goo0P0057oo0P00
37oo00<007ooOol01Woo00<007oo000017oo1@001Woo00<007oo000027oo000COol2001FOol20000
0goo00000003Ool01000Oomoogoo0P004Woo00<007ooOol02Goo00D007ooOomoo`0000Aoo`8000Ao
o`8000Ioo`03001oogoo00Aoo`80015oo`<0009oo`03001oogoo00moo`H001Eoo`03001oogoo00Qo
o`<0009oo`05001oogooOol00007Ool001=oo`03001oogoo05Yoo`03001oo`0000Eoo`03001oogoo
015oo`03001oogoo00Qoo`03001oogoo009oo`03001oogoo009oo`<00003Ool0000000Moo`03001o
ogoo00Ioo`03001oogoo00]oo`<000=oo`04001oogoo0007Ool90002Ool01000Oomoogoo0P004Woo
00<007oo000037oo0P001Goo00<007ooOol017oo000COol00`00Oomoo`1KOol00`00Oomoo`05Ool0
0`00Oomoo`0AOol00`00Oomoo`08Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01000Ool000002Goo
00<007ooOol01Woo00<007ooOol01goo0`001goo00<007oo00001goo00<007ooOol01Woo0P001Woo
0`003Woo00D007ooOomoo`0001Uoo`004goo00<007ooOol0I7oo00<007ooOol047oo00<007ooOol0
2Goo00D007ooOomoo`0000Eoo`03001oo`0000Uoo`03001oogoo00Moo`03001oogoo008000=oo`03
001oogoo00Uoo`03001oo`0000Eoo`03001oogoo00Ioo`03001oogoo00Qoo`8000aoo`03001oogoo
009oo`03001oogoo01Moo`004goo00<007ooOol0IGoo00<007ooOol047oo00<007ooOol027oo00<0
07ooOol00Woo00<007ooOol00goo00<007ooOol02Goo1@0017oo0`000Woo00<007oo00003Goo0P00
17oo00<007ooOol01Woo00<007ooOol027oo00@007oo000000Uoo`8000Ioo`03001oogoo01Ioo`00
4goo00<007ooOol0IWoo00<007ooOol03goo00<007ooOol02Goo00<007ooOol00Woo00<007ooOol0
4goo10000Woo00D007ooOomoo`0000moo`T000Eoo`03001oogoo009oo`/000Moo`03001oogoo00Mo
o`03001oogoo01Eoo`004goo00<007ooOol0Igoo00<007ooOol03goo00<007ooOol02Goo00D007oo
Oomoo`0001aoo`05001oo`00Ool0000?Ool200000goo001oo`03Ool20002Ool00`00Oomoo`02Ool0
0`00Oomoo`03Ool01000Oomoogoo0`0017oo0P002goo00<007ooOol057oo000COol2001YOol3000?
Ool00`00Oomoo`08Ool00`00Oomoo`02Ool00`00Oomoo`0JOol01@00Oomoogoo00003goo0P001goo
0P000goo0P001Goo00<007ooOol017oo0`0000=oo`00Ool03Goo00<007ooOol04goo000COol00`00
Oomoo`1[Ool3000<Ool00`00Oomoo`09Ool20003Ool00`00Oomoo`0NOol00`00Oomoo`0=Ool30005
Ool300000goo001oo`06Ool00`00Oomoo`05Ool30002Ool70007Ool00`00Oomoo`0BOol001=oo`03
001oogoo06ioo`8000]oo`03001oogoo00Ioo`800003Ool00000009oo`03001oogoo01moo`/000=o
o`05001oo`00Ool00003Ool01@00Oomoogoo00001goo00<007ooOol01goo10001goo0`001Goo00<0
07ooOol04Goo000COol00`00Oomoo`1`Ool20009Ool00`00Oomoo`04Ool20004Ool20002Ool00`00
Oomoo`0YOol30003Ool00`00Ool0000K000:Ool00`00Oomoo`03Ool00`00Oomoo`0AOol001=oo`03
001oogoo079oo`03001oogoo00Eoo`P000Moo`@0039oo`D00003Ool0000000H00003Ool007oo00ao
o`8000Uoo`03001oogoo00=oo`03001oogoo011oo`004goo00<007ooOol0Lgoo0P000Woo0`000goo
00<007ooOol02Goo00<007ooOol00P00<Woo00<007ooOol02Woo00<007ooOol03Goo0`001Goo0P00
00=oo`00Ool017oo00<007ooOol027oo00<007ooOol017oo000COol00`00Oomoo`1eOol20006Ool0
0`00Oomoo`0:Ool00`00Oomoo`04000^Ool00`00Oomoo`0MOol2000027oo00000000Ool007oo0006
Ool00`00Oomoo`06Ool20006Ool001=oo`8007moo`03001oogoo00Yoo`03001oo`0000H002Yoo`03
001oogoo01moo`8000Eoo`03001oo`0000Ioo`03001oogoo00Aoo`03001oogoo00Ioo`004goo00<0
07ooOol0Ogoo00<007ooOol02Goo0P000goo30008Woo00<007ooOol087oo00<007ooOol017oo0P00
1Woo00<007ooOol00Woo0P002Goo000COol00`00Oomoo`1oOol00`00Oomoo`09Ool20005Ool50005
Ool2000OOol00`00Oomoo`0ROol00`00Oomoo`04Ool20006Ool00`00Oomoo`020003Ool00`00Oomo
o`04Ool001=oo`03001oogoo081oo`03001oogoo00Moo`03001oo`0000Moo`03001oogoo00<000Ao
o`03001oogoo01aoo`03001oogoo02=oo`03001oogoo00Aoo`8000Eoo`@000Aoo`03001oogoo00Ao
o`004goo00<007ooOol0PGoo0`001Goo00D007ooOomoo`0000Ioo`03001oogoo00=oo`03001oogoo
009oo`80019oo`<000Ioo`03001oogoo00Eoo`@001]oo`03001oogoo00Eoo`03001oogoo00<0009o
o`03001oogoo009oo`8000Ioo`004goo00<007ooOol0Q7oo0`0000=oo`00Ool017oo0P001Goo00<0
07ooOol00goo00<007ooOol00goo10003Goo00D007ooOomoo`0000Aoo`04001oogooOol50003Ool0
0`00Oomoo`0JOol00`00Oomoo`05Ool20009Ool00`00Oomoo`06Ool00004Ool000000003Ool00`00
Oomoo`02Ool20005Ool00`00Oomoo`27Ool00`00Oomoo`06Ool00`00Oomoo`02Ool00`00Oomoo`04
Ool00`00Oomoo`06Ool30008Ool20005Ool20002Ool01000Ool000002Goo00<007ooOol06Woo00<0
07ooOol01Woo0P001Woo00<007ooOol01goo0002Ool00`00Oomoo`040002Ool01000Oomoo`0017oo
0P00TWoo00<007ooOol00Woo00<007ooOol017oo00<007ooOol01goo00<007ooOol01Goo00<007oo
Ool01goo10002goo00<007ooOol06goo0P0027oo00<007ooOol00Woo00<007ooOol027oo0002Ool0
1P00Oomoo`00Ool000=oo`04001oogoo0004Ool00`00Oomoo`2BOol01@00Oomoogoo00001goo00<0
07ooOol01goo00<007ooOol00goo00<007ooOol01Woo0P0000=oo`00Ool03Goo00<007ooOol077oo
0P001goo0P0000=oo`00Ool02Woo0002Ool01P00Oomoo`00Ool000=oo`04001oogoo0004Ool00`00
Oomoo`2COol01@00Oomoogoo00001goo00<007ooOol01goo00<007ooOol00P001goo0P004Woo00<0
07ooOol07Woo0P0017oo100037oo0002Ool01000Oomoogoo0P000goo00@007ooOol000Aoo`03001o
ogoo09Aoo`04001oogoo0008Ool00`00Oomoo`07Ool20007Ool2000EOol00`00Oomoo`0OOol4000@
Ool00003Ool0000000Aoo`03001oogoo009oo`8000Eoo`03001oogoo09Eoo`03001oo`0000Uoo`80
00Eoo`800003Ool0000000H001Moo`03001oogoo03=oo`004goo00<007ooOol0UWoo00<007oo0000
2Woo00D007ooOomoo`0000=oo`800003Ool00000009oo`@001Aoo`03001oogoo03=oo`004goo00<0
07ooOol0Ugoo0P002goo0P0000=oo`00Ool00P000Woo00<007ooOol01Woo00<007ooOol03Woo0P00
0Woo00<007ooOol0<Woo000COol2002IOol2000<Ool00`00Ool0000>Ool00`00Oomoo`0<Ool01P00
Oomoo`00Ool003Aoo`004goo00<007ooOol0VGoo00<007ooOol00Woo2P0047oo00<007ooOol02Woo
00<007ooOol00Woo00<007oo0000<goo000COol00`00Oomoo`2JOol40007Ool01000Oomoo`0047oo
00<007ooOol027oo00<007ooOol017oo0P00<goo000COol00`00Oomoo`2LOol00`00Oomoo`05Ool0
0`00Oomoo`02Ool00`00Oomoo`0>Ool20007Ool00`00Oomoo`06Ool3000aOol001=oo`03001oogoo
09eoo`8000Aoo`03001oogoo00Aoo`03001oogoo00moo`8000Aoo`03001oogoo00Qoo`03001oo`00
031oo`004goo00<007ooOol0Wgoo00D007ooOomoo`0000Moo`03001oogoo011oo`800003Ool007oo
00]oo`03001oo`0002moo`004goo00<007ooOol0X7oo00<007oo00002Goo00<007ooOol04Goo00<0
07ooOol037oo00<007oo0000;Woo000COol00`00Oomoo`2QOol00`00Oomoo`09Ool00`00Oomoo`0P
Ool00`00Ool0000]Ool001=oo`800:ioo`03001oogoo025oo`04001oo`00000GOol00`00Oomoo`0A
Ool001=oo`03001oogoo0:ioo`03001oogoo025oo`04001oogoo000FOol00`00Oomoo`0AOol001=o
o`03001oogoo0:moo`03001oogoo025oo`800003Ool0000001=oo`03001oo`00019oo`004goo00<0
07ooOol0[goo00<007ooOol08goo10004Woo00<007oo00004Woo000COol00`00Oomoo`2`Ool00`00
Oomoo`0UOol2000@Ool01@00Oomoogoo00004Goo000COol00`00Oomoo`2aOol00`00Oomoo`0VOol0
0`00Oomoo`0=Ool00`00Oomoo`02Ool00`00Oomoo`0>Ool001=oo`03001oogoo0;9oo`03001oogoo
02Ioo`03001oogoo00aoo`03001oogoo009oo`03001oogoo00ioo`004goo0P00/goo00<007ooOol0
9goo00<007ooOol01Woo0`0000=oo`00Ool01Goo00<007ooOol03Goo000COol00`00Oomoo`2cOol0
0`00Oomoo`0WOol00`00Oomoo`04Ool01000Oomoogoo0P001Woo00<007ooOol03Goo000COol00`00
Oomoo`2dOol00`00Oomoo`0WOol01000Oomoogoo0P0017oo00@007oo000000Eoo`03001oogoo00ao
o`004goo00<007ooOol0]7oo00<007ooOol0:7oo00<007oo00001Woo00@007ooOomoo`<000=oo`03
001oogoo00]oo`004goo00<007ooOol0]Goo00<007ooOol0:7oo0P001Goo00<007ooOol01Goo1000
3Goo000017oo000000000Woo0P0017oo0P001Goo00<007ooOol0]Goo00<007ooOol0:Woo0P000goo
00<007ooOol027oo20001Woo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`03001oogoo009o
o`03001oogoo0;Eoo`03001oogoo02aoo`800003Ool007oo00aoo`8000Uoo`000Woo00@007ooOol0
009oo`04001oogoo0002Ool00`00Oomoo`02Ool2002gOol00`00Oomoo`0]Ool00`00Oomoo`0>Ool2
0007Ool0009oo`03001oogoo00<000=oo`04001oogoo0004Ool00`00Oomoo`2fOol00`00Oomoo`10
Ool00`00Oomoo`04Ool0009oo`04001oogoo0005Ool01000Oomoo`0017oo00<007ooOol0]Woo00<0
07ooOol0Agoo00000goo00000003Ool30003Ool20005Ool00`00Oomoo`2gOol00`00Oomoo`16Ool0
01=oo`03001oogoo0;Moo`03001oogoo04Ioo`004goo00<007ooOol0]goo00<007ooOol0AWoo000C
Ool00`00Oomoo`2hOol00`00Oomoo`15Ool001=oo`800;Uoo`03001oogoo04Eoo`004goo00<007oo
Ool0^7oo00<007ooOol0AGoo000COol00`00Oomoo`2iOol00`00Oomoo`14Ool001=oo`03001oogoo
0;Uoo`03001oogoo04Aoo`004goo00<007ooOol0^Goo00<007ooOol0A7oo000COol00`00Oomoo`2j
Ool00`00Oomoo`0XOol8000COol001=oo`03001oogoo0;Yoo`@002Ioo`03001oogoo00Ioo`03001o
ogoo011oo`004goo0P00_goo0`008Goo0P002Woo00<007ooOol03goo000COol00`00Oomoo`31Ool0
0`00Oomoo`0MOol00`00Oomoo`0;Ool00`00Oomoo`0>Ool001=oo`03001oogoo0<9oo`03001oogoo
01]oo`03001oogoo00eoo`03001oogoo00eoo`004goo00<007ooOol0`goo00<007ooOol06Woo00<0
07ooOol03Woo00<007ooOol037oo000COol00`00Oomoo`34Ool00`00Oomoo`0HOol00`00Oomoo`0@
Ool00`00Oomoo`0;Ool001=oo`03001oogoo0<Eoo`03001oogoo01Ioo`03001oogoo019oo`<000Yo
o`004goo00<007ooOol0aWoo00<007ooOol05Goo00<007ooOol05Goo0`001goo000COol20038Ool0
0`00Oomoo`0COol00`00Oomoo`0IOol00`00Oomoo`04Ool001=oo`03001oogoo0<Moo`03001oogoo
019oo`03001oogoo025oo`004goo00<007ooOol0b7oo00<007ooOol04Goo00<007ooOol08Goo000C
Ool00`00Oomoo`38Ool00`00Oomoo`0@Ool00`00Oomoo`0ROol001=oo`03001oogoo0<Uoo`03001o
ogoo00eoo`8002Eoo`0000Aoo`000000009oo`8000Aoo`8000Eoo`03001oogoo0<Uoo`03001oogoo
00]oo`8002Moo`000Woo00@007ooOol0009oo`04001oogoo0002Ool00`00Oomoo`02Ool00`00Oomo
o`3:Ool00`00Oomoo`09Ool00`00Oomoo`0WOol0009oo`04001oogoo0002Ool01000Oomoo`000Woo
00<007ooOol00Woo0P00c7oo00<007ooOol01Woo0P00:Woo0002Ool01000Oomoogoo0P000goo00@0
07ooOol000Aoo`03001oogoo0<]oo`03001oogoo00Aoo`8002aoo`000Woo00@007ooOol0009oo`04
001oogoo0002Ool00`00Oomoo`02Ool00`00Oomoo`3<Ool01000Oomoogoo0P00;Woo00000goo0000
0003Ool20004Ool20005Ool00`00Oomoo`3<Ool01000Ool00000<7oo000COol00`00Oomoo`3=Ool0
0`00Oomoo`0`Ool001=oo`03001oogoo0?moo`5oo`004goo00<007ooOol0ogoo0Goo000COol2003o
Ool2Ool001=oo`03001oogoo0?moo`5oo`00ogoo5goo003oOolGOol00?mooaMoo`00\
\>"],
  ImageRangeCache->{{{0, 277.938}, {170.562, 0}} -> {-3.01968, 67.2869, 
  0.197976, 0.886808}}]
}, Open  ]],

Cell["\<\
In this and the two subsequent investigations, note that any \
graphics you generate yourself will look different - these paths are \
random!\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Looking at Antithetic Paths", "Subsubsection",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
If we take a look at two antithetic pairs, we can see the symmetry \
that is generated:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["example = pathpairat[100, 0.1, 0, 0.2, 1, 1, 40];", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
plots = Table[ListPlot[example[[i]], PlotJoined -> True,
DisplayFunction -> Identity], {i, 1, Length[example]}];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Show[plots, DisplayFunction -> $DisplayFunction];", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 7  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 -0.865673 0.0106397 [
[.2619 .0794 -4.2 -6.3 ]
[.2619 .0794 4.2 0 ]
[.5 .0794 -4.2 -6.3 ]
[.5 .0794 4.2 0 ]
[.7381 .0794 -4.2 -6.3 ]
[.7381 .0794 4.2 0 ]
[.97619 .0794 -4.2 -6.3 ]
[.97619 .0794 4.2 0 ]
[.01131 .19829 -12.6 -3.15 ]
[.01131 .19829 0 3.15 ]
[.01131 .30469 -12.6 -3.15 ]
[.01131 .30469 0 3.15 ]
[.01131 .41109 -12.6 -3.15 ]
[.01131 .41109 0 3.15 ]
[.01131 .51748 -12.6 -3.15 ]
[.01131 .51748 0 3.15 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .0919 m
.2619 .09815 L
s
[(10)] .2619 .0794 0 1 Mshowa
.5 .0919 m
.5 .09815 L
s
[(20)] .5 .0794 0 1 Mshowa
.7381 .0919 m
.7381 .09815 L
s
[(30)] .7381 .0794 0 1 Mshowa
.97619 .0919 m
.97619 .09815 L
s
[(40)] .97619 .0794 0 1 Mshowa
.125 Mabswid
.07143 .0919 m
.07143 .09565 L
s
.11905 .0919 m
.11905 .09565 L
s
.16667 .0919 m
.16667 .09565 L
s
.21429 .0919 m
.21429 .09565 L
s
.30952 .0919 m
.30952 .09565 L
s
.35714 .0919 m
.35714 .09565 L
s
.40476 .0919 m
.40476 .09565 L
s
.45238 .0919 m
.45238 .09565 L
s
.54762 .0919 m
.54762 .09565 L
s
.59524 .0919 m
.59524 .09565 L
s
.64286 .0919 m
.64286 .09565 L
s
.69048 .0919 m
.69048 .09565 L
s
.78571 .0919 m
.78571 .09565 L
s
.83333 .0919 m
.83333 .09565 L
s
.88095 .0919 m
.88095 .09565 L
s
.92857 .0919 m
.92857 .09565 L
s
.25 Mabswid
0 .0919 m
1 .0919 L
s
.02381 .19829 m
.03006 .19829 L
s
[(100)] .01131 .19829 1 0 Mshowa
.02381 .30469 m
.03006 .30469 L
s
[(110)] .01131 .30469 1 0 Mshowa
.02381 .41109 m
.03006 .41109 L
s
[(120)] .01131 .41109 1 0 Mshowa
.02381 .51748 m
.03006 .51748 L
s
[(130)] .01131 .51748 1 0 Mshowa
.125 Mabswid
.02381 .11318 m
.02756 .11318 L
s
.02381 .13446 m
.02756 .13446 L
s
.02381 .15573 m
.02756 .15573 L
s
.02381 .17701 m
.02756 .17701 L
s
.02381 .21957 m
.02756 .21957 L
s
.02381 .24085 m
.02756 .24085 L
s
.02381 .26213 m
.02756 .26213 L
s
.02381 .28341 m
.02756 .28341 L
s
.02381 .32597 m
.02756 .32597 L
s
.02381 .34725 m
.02756 .34725 L
s
.02381 .36853 m
.02756 .36853 L
s
.02381 .38981 m
.02756 .38981 L
s
.02381 .43237 m
.02756 .43237 L
s
.02381 .45364 m
.02756 .45364 L
s
.02381 .47492 m
.02756 .47492 L
s
.02381 .4962 m
.02756 .4962 L
s
.02381 .07062 m
.02756 .07062 L
s
.02381 .04934 m
.02756 .04934 L
s
.02381 .02806 m
.02756 .02806 L
s
.02381 .00678 m
.02756 .00678 L
s
.02381 .53876 m
.02756 .53876 L
s
.02381 .56004 m
.02756 .56004 L
s
.02381 .58132 m
.02756 .58132 L
s
.02381 .6026 m
.02756 .6026 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.04762 .19829 m
.07143 .17621 L
.09524 .14631 L
.11905 .09815 L
.14286 .06414 L
.16667 .05279 L
.19048 .03394 L
.21429 .03725 L
.2381 .02139 L
.2619 .03826 L
.28571 .08218 L
.30952 .08734 L
.33333 .15232 L
.35714 .20895 L
.38095 .18042 L
.40476 .14012 L
.42857 .1221 L
.45238 .08463 L
.47619 .05113 L
.5 .04494 L
.52381 .07362 L
.54762 .06267 L
.57143 .03631 L
.59524 .04497 L
.61905 .08025 L
.64286 .12485 L
.66667 .15639 L
.69048 .13263 L
.71429 .08596 L
.7381 .06612 L
.7619 .07535 L
.78571 .08561 L
.80952 .0967 L
.83333 .08877 L
.85714 .10247 L
.88095 .14246 L
.90476 .12637 L
.92857 .1488 L
.95238 .18147 L
.97619 .10586 L
s
.04762 .19829 m
.07143 .22583 L
.09524 .26275 L
.11905 .32224 L
.14286 .37025 L
.16667 .39167 L
.19048 .42393 L
.21429 .4258 L
.2381 .45511 L
.2619 .43677 L
.28571 .38015 L
.30952 .37977 L
.33333 .30123 L
.35714 .2423 L
.38095 .2774 L
.40476 .3273 L
.42857 .35479 L
.45238 .40734 L
.47619 .45875 L
.5 .47449 L
.52381 .43914 L
.54762 .46106 L
.57143 .50552 L
.59524 .4994 L
.61905 .45351 L
.64286 .39807 L
.66667 .36432 L
.69048 .39922 L
.71429 .46484 L
.7381 .4994 L
.7619 .49288 L
.78571 .48503 L
.80952 .47621 L
.83333 .49415 L
.85714 .4816 L
.88095 .43286 L
.90476 .46024 L
.92857 .43707 L
.95238 .40179 L
.97619 .49981 L
s
.04762 .19829 m
.07143 .19175 L
.09524 .21725 L
.11905 .22128 L
.14286 .23014 L
.16667 .17792 L
.19048 .18017 L
.21429 .20204 L
.2381 .18104 L
.2619 .16481 L
.28571 .21217 L
.30952 .19891 L
.33333 .22947 L
.35714 .32028 L
.38095 .33175 L
.40476 .29609 L
.42857 .31759 L
.45238 .31896 L
.47619 .29336 L
.5 .32543 L
.52381 .29099 L
.54762 .30214 L
.57143 .30203 L
.59524 .29503 L
.61905 .27778 L
.64286 .32947 L
.66667 .28409 L
.69048 .22768 L
.71429 .19893 L
.7381 .1829 L
.7619 .15273 L
.78571 .10279 L
.80952 .02402 L
.83333 .06756 L
.85714 .01472 L
.88095 .05874 L
.90476 .06779 L
.92857 .11905 L
.95238 .09782 L
.97619 .07009 L
s
.04762 .19829 m
.07143 .21029 L
.09524 .18987 L
.11905 .19135 L
.14286 .18815 L
.16667 .24378 L
.19048 .24708 L
.21429 .22951 L
.2381 .25667 L
.2619 .27983 L
.28571 .23306 L
.30952 .25221 L
.33333 .22585 L
.35714 .14093 L
.38095 .13636 L
.40476 .17135 L
.42857 .15747 L
.45238 .16153 L
.47619 .189 L
.5 .16522 L
.52381 .20032 L
.54762 .1955 L
.57143 .20105 L
.59524 .21291 L
.61905 .23447 L
.64286 .19039 L
.66667 .2359 L
.69048 .2956 L
.71429 .33208 L
.7381 .35626 L
.7619 .39769 L
.78571 .46611 L
.80952 .58127 L
.83333 .51788 L
.85714 .60332 L
.88095 .5374 L
.90476 .53086 L
.92857 .46133 L
.95238 .49673 L
.97619 .54294 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{223.188, 137.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004F0000Za000`40O003h00Oogoo5goo003oOolG
Ool00?mooaMoo`004goo00<007ooOol0ogoo0Goo000COol00`00Oomoo`3oOol1Ool001=oo`800?mo
o`9oo`004goo00<007ooOol0ogoo0Goo000COol00`00Oomoo`3JOol00`00Oomoo`0SOol001=oo`03
001oogoo0=Yoo`03001oogoo02=oo`004goo00<007ooOol0=7oo0`00XWoo00<007oo000097oo000C
Ool00`00Oomoo`0cOol01000Oomoogoo0P00U7oo00<007ooOol02Goo00<007oo000097oo000COol2
000bOol20006Ool00`00Oomoo`2AOol00`00Oomoo`08Ool01@00Oomoogoo00008goo000COol00`00
Oomoo`0XOol50003Ool00`00Oomoo`07Ool2002AOol20009Ool01@00Oomoogoo00008goo000COol0
0`00Oomoo`0WOol00`00Oomoo`03Ool3000<Ool00`00Oomoo`1?Ool3000kOol00`00Ool00008Ool0
0`00Oomoo`03Ool00`00Oomoo`0POol001=oo`03001oogoo02Eoo`8001Eoo`03001oogoo04ioo`04
001oogooOol3000hOol01000Oomoo`001goo00<007ooOol00goo00<007ooOol087oo000COol00`00
Oomoo`0TOol00`00Oomoo`0FOol00`00Oomoo`0jOol2000@Ool00`00Oomoo`05Ool00`00Oomoo`0e
Ool01000Oomoo`001Woo00<007ooOol01Goo00<007ooOol07goo000COol2000TOol00`00Oomoo`0G
Ool00`00Oomoo`0gOol300000goo001oo`0>Ool00`00Oomoo`06Ool00`00Oomoo`0dOol00`00Oomo
o`02Ool00`00Oomoo`03Ool00`00Oomoo`05Ool00`00Oomoo`0OOol001=oo`03001oogoo025oo`80
01]oo`03001oogoo03Aoo`8000Eoo`03001oogoo00]oo`03001oogoo00Qoo`03001oogoo03=oo`03
001oogoo00=oo`05001oogooOol00009Ool00`00Oomoo`0NOol001=oo`03001oogoo01moo`8001Mo
o`<0009oo`8003Ioo`D0009oo`8000]oo`03001oogoo00Yoo`03001oogoo01eoo`<000=oo`8000eo
o`03001oogoo00=oo`05001oogooOol00009Ool00`00Oomoo`0EOol01000Oomoogoo0P000goo000C
Ool00`00Oomoo`0MOol2000JOol01000Oomoo`000Woo00<007ooOol0<Woo00@007ooOol000=oo`04
001oogoo0009Ool00`00Oomoo`0;Ool00`00Oomoo`0POol00`00Ool00002Ool00`00Oomoo`0:Ool0
0`00Oomoo`04Ool00`00Ool0000;Ool3000BOol400001Goo001oogoo00000Woo000COol00`00Oomo
o`0LOol00`00Oomoo`0JOol01000Oomoo`000Woo00<007ooOol0<Goo00<007ooOol00Woo00@007oo
Ool0009oo`03001oogoo00Eoo`8000moo`03001oogoo01moo`03001oo`00009oo`03001oogoo00Uo
o`03001oogoo00Eoo`03001oo`0000ioo`<000moo`03001oo`00009oo`04001oogoo0002Ool001=o
o`03001oogoo01aoo`03001oogoo01Yoo`04001oogoo0002Ool2000aOol00`00Oomoo`04Ool00`00
Ool00002Ool20004Ool2000AOol00`00Oomoo`0MOol600000goo001oo`0:Ool00`00Oomoo`06Ool0
0`00Oomoo`0@Ool00`00Oomoo`0<Ool00`00Ool00002Ool01000Oomoo`000Woo000COol2000LOol0
0`00Oomoo`0KOol01000Oomoo`000Woo0P00<7oo00<007ooOol01Goo00<007oo00000Woo00@007oo
001oo`8001Aoo`03001oogoo01eoo`800003Ool007oo00<000Yoo`03001oogoo01Uoo`03001oogoo
00eoo`800003Ool00000009oo`40005oo`5oo`004goo00<007ooOol06Woo00<007ooOol06goo0P00
0goo0P000Woo00<007ooOol0;Goo00<007ooOol00Woo0`000goo0P000goo00<007ooOol05Goo00<0
07ooOol06Woo0`000goo0P000Woo0P001goo00<007ooOol06goo00<007ooOol03Goo00@007oo001o
o`8000=oo`004goo00<007ooOol06Woo00<007ooOol097oo00<007ooOol0;7oo00<007ooOol09Woo
00<007ooOol06Woo00<007ooOol02Goo0`0017oo00<007ooOol06goo00<007ooOol03Woo00<007oo
Ool01Goo000COol00`00Oomoo`0IOol00`00Oomoo`0VOol4000YOol00`00Oomoo`0XOol00`00Oomo
o`0HOol00`00Oomoo`0=Ool20002Ool00`00Oomoo`0LOol00`00Oomoo`0<Ool00`00Oomoo`06Ool0
01=oo`03001oogoo01Qoo`03001oogoo02]oo`<002Eoo`03001oogoo02Uoo`03001oogoo01Moo`03
001oogoo011oo`<001ioo`03001oogoo00]oo`03001oogoo00Moo`004goo00<007ooOol067oo00<0
07ooOol0;Goo00<007ooOol08goo00<007ooOol0:Woo00<007ooOol05Woo00<007ooOol04Goo0`00
1goo0`004goo00<007ooOol02Woo00<007ooOol027oo000<Oooo000;000001=oo`03001oogoo00Uo
o`03001oogoo00Yoo`03001oogoo00Uoo`03001oogoo00Yoo`03001oogoo00Yoo`03001oogoo00Uo
o`8000]oo`03001oogoo00Yoo`03001oogoo00Moo`03001oo`0000aoo`03001oogoo00Uoo`03001o
ogoo00Yoo`03001oogoo00Eoo`03001oogoo009oo`03001oogoo00Uoo`03001oogoo00=oo`03001o
ogoo00Aoo`03001oogoo00Yoo`8000Aoo`8000Aoo`03001oogoo009oo`8000Ioo`03001oogoo00Io
o`04001oogoo0005Ool20005Ool00`00Oomoo`04Ool001=oo`03001oogoo01Ioo`03001oogoo031o
o`03001oogoo021oo`03001oogoo02ioo`03001oogoo019oo`03001oogoo019oo`03001oogoo011o
o`03001oogoo00eoo`03001oogoo00Aoo`03001oogoo00aoo`004goo00<007ooOol05Goo00<007oo
Ool0<Woo00<007ooOol07Woo00<007ooOol0<7oo00<007ooOol047oo00<007ooOol04goo00<007oo
Ool047oo00<007ooOol03Goo00<007ooOol00goo00<007ooOol01Woo00<007ooOol017oo000COol0
0`00Oomoo`0EOol00`00Oomoo`0bOol00`00Oomoo`0NOol00`00Oomoo`0`Ool00`00Oomoo`0@Ool0
0`00Oomoo`0COol00`00Oomoo`0AOol00`00Oomoo`0=Ool01@00Oomoogoo00002Goo00<007ooOol0
17oo000COol2000EOol00`00Oomoo`0dOol00`00Oomoo`0LOol00`00Oomoo`0bOol00`00Oomoo`0>
Ool00`00Oomoo`0COol00`00Oomoo`0BOol00`00Oomoo`0=Ool01000Oomoo`002Woo00<007ooOol0
17oo000COol00`00Oomoo`0DOol00`00Oomoo`0dOol00`00Oomoo`0KOol00`00Oomoo`0cOol00`00
Oomoo`0>Ool00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0<Ool00`00Ool0000:Ool00`00
Oomoo`05Ool001=oo`03001oogoo01=oo`03001oogoo03Eoo`03001oogoo01]oo`03001oogoo03Ao
o`03001oogoo00aoo`03001oogoo01=oo`03001oogoo01Aoo`03001oogoo00eoo`03001oogoo00Uo
o`03001oogoo00Eoo`004goo00<007ooOol04goo00<007ooOol0=Woo00<007ooOol067oo0P00>7oo
00<007ooOol02Woo00<007ooOol057oo00<007ooOol05Goo00<007ooOol067oo00<007ooOol01Goo
000COol00`00Oomoo`0COol00`00Oomoo`0fOol00`00Oomoo`0GOol00`00Oomoo`0hOol00`00Oomo
o`0:Ool00`00Oomoo`0COol00`00Oomoo`0GOol00`00Oomoo`04Ool2000@Ool00`00Oomoo`06Ool0
01=oo`03001oogoo019oo`03001oogoo03Moo`03001oogoo01Eoo`8003aoo`03001oogoo00Qoo`03
001oogoo01=oo`03001oogoo01Qoo`03001oogoo00=oo`04001oogoo000?Ool00`00Oomoo`06Ool0
01=oo`8001=oo`03001oogoo03Qoo`03001oogoo00Yoo`8000Moo`03001oogoo03eoo`03001oogoo
00Moo`03001oogoo01=oo`03001oogoo01Uoo`03001oogoo008000Aoo`03001oogoo00aoo`03001o
ogoo00Ioo`004goo00<007ooOol04Goo00<007ooOol0>Goo00<007ooOol01goo0`0000=oo`00Ool0
1Goo00<007ooOol0?Woo00<007ooOol01Woo00<007ooOol04goo00<007ooOol06Woo00<007oo0000
1goo00<007ooOol02Woo00<007ooOol01goo000COol00`00Oomoo`0AOol00`00Oomoo`0jOol00`00
Oomoo`04Ool20005Ool00`00Oomoo`03Ool00`00Oomoo`0oOol00`00Oomoo`04Ool00`00Oomoo`0D
Ool00`00Oomoo`0KOol00`00Oomoo`07Ool00`00Oomoo`09Ool00`00Oomoo`07Ool001=oo`03001o
ogoo011oo`03001oogoo03]oo`03001oogoo00Aoo`03001oogoo00Eoo`05001oogooOol00013Ool0
0`00Oomoo`02Ool00`00Oomoo`0DOol00`00Oomoo`0WOol00`00Oomoo`08Ool00`00Oomoo`07Ool0
01=oo`03001oogoo011oo`03001oogoo03]oo`03001oogoo00Aoo`03001oogoo00Eoo`04001oogoo
0014Ool01@00Oomoogoo00005goo00<007ooOol0:7oo00<007ooOol01goo00<007ooOol01goo000C
Ool00`00Oomoo`0?Ool00`00Oomoo`0mOol00`00Oomoo`02Ool00`00Oomoo`07Ool00`00Ool00015
Ool00`00Ool0000GOol00`00Oomoo`0YOol00`00Oomoo`06Ool00`00Oomoo`08Ool001=oo`8000mo
o`03001oogoo03ioo`03001oogoo009oo`03001oogoo00Qoo`03001oogoo00Eoo`D003]oo`03001o
ogoo01Ioo`03001oogoo02Yoo`03001oogoo00Eoo`03001oogoo00Qoo`004goo00<007ooOol03Goo
00<007ooOol0@7oo00D007ooOomoo`0000Uoo`03001oo`0000Eoo`03001oogoo00=oo`<0051oo`03
001oogoo02aoo`03001oogoo00Aoo`03001oogoo00Qoo`004goo00<007ooOol037oo00<007ooOol0
;7oo0P004goo00@007ooOol000Yoo`03001oo`0000=oo`8000Uoo`03001oogoo00Uoo`03001oogoo
041oo`03001oogoo02ioo`03001oogoo009oo`03001oogoo00Uoo`004goo00<007ooOol02goo00<0
07ooOol0:goo0P0000=oo`00Ool04Woo00@007ooOol000Uoo`07001oogooOol007oo000037oo00<0
07ooOol01goo0P00@Goo00<007ooOol0<7oo00D007ooOomoo`0000]oo`004goo00<007ooOol02Woo
00<007ooOol0:goo00<007ooOol00Woo00<007ooOol04Goo00<007oo00002Goo00<007ooOol00Woo
00<007ooOol037oo00<007ooOol01Goo00@007ooOol0041oo`03001oogoo035oo`04001oogoo000;
Ool001=oo`8000Yoo`03001oogoo01Ioo`D000moo`8000Eoo`03001oogoo015oo`03001oo`0000Qo
o`03001oogoo01=oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo03aoo`03001oogoo03=o
o`8000aoo`004goo00<007ooOol027oo00<007ooOol05goo00<007ooOol00Woo0`002goo00<007oo
Ool01Woo00<007ooOol04Goo00<007ooOol01Woo00<007ooOol05Goo00D007ooOomoo`0000Eoo`03
001oogoo03]oo`03001oogoo03Eoo`03001oogoo00Yoo`004goo00<007ooOol01goo00<007ooOol0
5goo00<007ooOol01Woo00<007ooOol01goo00<007ooOol01goo00<007ooOol04Goo00<007ooOol0
1Woo00<007ooOol05Woo00<007oo00001goo00<007ooOol0>7oo0P00AGoo000COol00`00Oomoo`06
Ool00`00Oomoo`0@Ool40004Ool00`00Oomoo`07Ool00`00Oomoo`05Ool00`00Oomoo`09Ool00`00
Oomoo`0@Ool00`00Oomoo`05Ool00`00Oomoo`0HOol00`00Oomoo`07Ool00`00Oomoo`0fOol00`00
Oomoo`15Ool00004Ool000000002Ool20004Ool20005Ool00`00Oomoo`05Ool00`00Oomoo`020005
Ool:0003Ool01@00Oomoogoo00002goo00<007ooOol00goo00<007ooOol02Woo00<007ooOol047oo
0P001Goo00<007ooOol08goo00<007ooOol07goo00<007ooOol04Woo0P00B7oo0002Ool01000Oomo
o`000Woo00@007ooOol0009oo`03001oogoo009oo`03001oogoo00Aoo`@0009oo`05001oogooOol0
000>Ool01000Oomoo`0037oo00D007ooOomoo`0000eoo`03001oogoo00moo`03001oo`0000Aoo`03
001oogoo02Eoo`04001oogooOol6000GOol00`00Oomoo`0AOol00`00Oomoo`18Ool0009oo`04001o
ogoo0002Ool01000Oomoo`000Woo00<007ooOol00Woo0P0017oo0P001Woo0`003goo00@007ooOol0
00eoo`03001oo`0000moo`03001oogoo00Aoo`8000Qoo`04001oogoo0003Ool00`00Oomoo`0VOol3
0006Ool3000COol00`00Ool0000AOol00`00Oomoo`19Ool0009oo`04001oogoo0002Ool01000Oomo
o`000Woo00<007ooOol00Woo00<007ooOol00goo00@007oo000000Aoo`80011oo`03001oo`0000mo
o`03001oogoo00ioo`03001oogoo00=oo`03001oo`0000Qoo`04001oogoo0002Ool00`00Oomoo`0c
Ool3000?Ool01000Oomoo`004Goo00<007ooOol0BGoo0002Ool01000Oomoo`000Woo00@007ooOol0
009oo`03001oogoo009oo`03001oogoo00Aoo`03001oogoo00800005Ool007ooOol0000@Ool2000Q
Ool00`00Oomoo`020003Ool00`00Oomoo`04Ool01`00Oomoogoo001oo`0003Uoo`8000eoo`05001o
ogooOol0000?Ool00`00Oomoo`1:Ool00003Ool0000000=oo`8000Aoo`8000Eoo`03001oogoo00Eo
o`05001oogooOol00004Ool00`00Oomoo`0=Ool00`00Oomoo`0POol00`00Ool00006Ool00`00Oomo
o`03Ool00`00Oomoo`02Ool00`00Oomoo`0jOol2000:Ool00`00Oomoo`02Ool00`00Oomoo`0<Ool0
0`00Oomoo`1;Ool001=oo`03001oogoo00Ioo`03001oogoo00Ioo`03001oogoo00aoo`80029oo`03
001oogoo00Ioo`03001oogoo009oo`03001oogoo045oo`8000Moo`03001oogoo00Aoo`03001oogoo
00Yoo`03001oogoo04aoo`004goo0P0027oo00<007ooOol01Woo10002Goo00<007oo0000;7oo00D0
07ooOomoo`0004Eoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Uoo`03001oogoo04eo
o`004goo00<007ooOol027oo00<007ooOol02Goo1@0017oo00<007oo0000;Goo00<007oo0000Agoo
00<007ooOol00Woo00<007ooOol01Woo00<007ooOol01goo00<007ooOol0CWoo000COol00`00Oomo
o`09Ool00`00Oomoo`0=Ool300001Goo001oogoo0000;Goo0P00B7oo00D007ooOomoo`0000Qoo`03
001oogoo00Ioo`03001oogoo04moo`004goo00<007ooOol02Goo00<007ooOol047oo00D007ooOomo
o`0000eoo`8001ioo`8004Uoo`03001oo`0000Yoo`03001oogoo00Aoo`03001oogoo051oo`004goo
00<007ooOol02Woo00<007ooOol057oo00<007ooOol02Goo00<007oo00004Woo0P002Goo00<007oo
0000BWoo00<007ooOol02Goo00<007ooOol017oo00<007ooOol0D7oo000COol00`00Oomoo`0;Ool0
0`00Oomoo`0COol00`00Oomoo`08Ool01@00Oomoogoo00004Goo00<007oo00001goo00@007ooOol0
05Moo`03001oogoo009oo`03001oogoo055oo`004goo0P0037oo00<007ooOol04goo00<007ooOol0
1goo00<007ooOol00goo00<007ooOol03Goo00D007ooOomoo`0000Eoo`03001oogoo009oo`03001o
ogoo05Aoo`03001oogoo009oo`03001oogoo055oo`004goo00<007ooOol037oo00<007ooOol04goo
1@000goo00<007ooOol01Goo00<007ooOol037oo00<007ooOol00Woo00D007ooOomoo`0000Eoo`03
001oogoo009oo`03001oogoo051oo`05001oogooOol0001COol001=oo`03001oogoo00eoo`03001o
ogoo01Moo`<000Uoo`03001oogoo00Yoo`03001oogoo00Aoo`03001oo`0000Ioo`03001oogoo009o
o`03001oogoo051oo`04001oogoo001DOol001=oo`03001oogoo00ioo`03001oogoo02=oo`03001o
ogoo00Uoo`03001oogoo00Eoo`03001oogoo00Eoo`07001oogooOol007oo0000DGoo00@007ooOol0
05Aoo`004goo00<007ooOol03Woo00<007ooOol097oo0P0027oo00<007ooOol03goo00@007ooOol0
009oo`03001oogoo04moo`8005Eoo`004goo00<007ooOol03goo00<007ooOol09Goo00<007ooOol0
1Goo00<007ooOol03goo00@007ooOol0009oo`03001oogoo04moo`8005Eoo`004goo0P004Goo00<0
07ooOol09Goo00<007ooOol00goo00<007ooOol047oo00<007oo000017oo00<007ooOol0CWoo0P00
EGoo000COol00`00Oomoo`0@Ool00`00Oomoo`0VOol00`00Oomoo`02Ool00`00Oomoo`0@Ool00`00
Ool00005Ool00`00Oomoo`1=Ool2001EOol001=oo`03001oogoo015oo`03001oogoo02Ioo`04001o
ogoo000DOol00`00Oomoo`04Ool00`00Oomoo`1=Ool2001EOol001=oo`03001oogoo015oo`03001o
ogoo02Moo`03001oo`0001Aoo`03001oogoo00Eoo`03001oogoo04]oo`04001oogoo001DOol001=o
o`03001oogoo015oo`03001oogoo02Qoo`03001oogoo01=oo`03001oogoo00Ioo`03001oogoo03]o
o`8000eoo`04001oogoo001DOol001=oo`8001=oo`03001oogoo03aoo`8000Qoo`03001oogoo03Yo
o`03001oo`0000eoo`04001oogoo001DOol001=oo`03001oogoo019oo`03001oogoo03aoo`03001o
o`0000Qoo`03001oogoo03Qoo`05001oogooOol0000;Ool00`00Oomoo`02Ool00`00Oomoo`1AOol0
01=oo`03001oogoo019oo`03001oogoo03]oo`04001oogoo0008Ool00`00Oomoo`0ROol2000COol0
0`00Oomoo`02Ool00`00Oomoo`09Ool00`00Oomoo`02Ool00`00Oomoo`1AOol001=oo`03001oogoo
01=oo`03001oogoo03Yoo`04001oogoo0009Ool00`00Oomoo`0EOol00`00Oomoo`09Ool01000Ool0
000047oo00<007ooOol017oo00<007ooOol01goo00<007ooOol00goo00<007ooOol0DGoo000COol0
0`00Oomoo`0COol00`00Oomoo`0jOol01000Oomoo`002Goo00<007ooOol00Woo0P0047oo0P002Woo
00<007ooOol00Woo0P002Woo10001goo00<007ooOol01goo00<007ooOol017oo00<007ooOol0D7oo
000017oo000000000Woo0`000goo0P001Goo00<007ooOol057oo00<007ooOol0>7oo00<007ooOol0
0Woo00<007ooOol01goo00L007ooOomoo`00Ool0000>Ool01000Oomoo`0027oo00<007ooOol01Goo
2P0037oo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol0D7oo0002Ool00`00Oomoo`02Ool0
1@00Oomoogoo00000Woo00<007ooOol00Woo0P005Goo00<007ooOol0>7oo00<007ooOol00Woo00<0
07ooOol027oo00<007oo00000goo00<007ooOol02Woo00<007ooOol00Woo00<007ooOol01Goo00<0
07ooOol06goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol0Cgoo0002Ool00`00Oomoo`02
Ool01@00Oomoogoo00000Woo00<007ooOol00Woo00<007ooOol057oo00<007ooOol0=goo00<007oo
Ool00goo00<007ooOol027oo0P001Goo00<007ooOol027oo00<007ooOol00goo00<007ooOol017oo
00<007ooOol07Goo00<007ooOol00goo00<007ooOol027oo00<007ooOol0CWoo0002Ool00`00Oomo
o`02Ool01@00Oomoogoo00000Woo00<007ooOol00Woo00<007ooOol05Goo00<007ooOol0=Woo00<0
07ooOol00goo00<007ooOol027oo0P001Woo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
0goo00<007ooOol07Goo00<007ooOol00goo00<007ooOol02Goo00<007ooOol0CGoo0002Ool00`00
Oomoo`02Ool01@00Oomoogoo00000Woo00<007ooOol00Woo00<007ooOol05Goo00<007ooOol0=Woo
00<007ooOol017oo00<007ooOol01Woo00<007oo00001goo10000goo00<007ooOol01goo00D007oo
Oomoo`00025oo`05001oogooOol0000=Ool00`00Oomoo`1<Ool00003Ool0000000=oo`8000Aoo`80
00Eoo`03001oogoo01Eoo`03001oogoo03Eoo`03001oogoo00Eoo`8000Moo`04001oogoo000:Ool3
000:Ool01000Oomoo`008Woo00D007ooOomoo`0000eoo`03001oogoo04aoo`004goo00<007ooOol0
5Woo00<007ooOol0=7oo00<007ooOol01goo0P0017oo00D007ooOomoo`0001Qoo`03001oo`0002=o
o`03001oo`0000moo`03001oogoo04]oo`004goo0P005goo00<007ooOol0=7oo00<007ooOol02Goo
0P0000=oo`00Ool017oo00<007ooOol05Woo00<007ooOol08Woo00<007oo000047oo00<007ooOol0
BWoo000COol00`00Oomoo`0GOol00`00Oomoo`0bOol00`00Oomoo`0<Ool00`00Oomoo`04Ool00`00
Oomoo`0lOol00`00Oomoo`0@Ool00`00Oomoo`19Ool001=oo`03001oogoo01Moo`03001oogoo039o
o`03001oogoo01Aoo`03001oogoo04ioo`03001oogoo04Uoo`004goo00<007ooOol067oo00<007oo
Ool0<Goo00<007ooOol05Goo00<007ooOol0CWoo00<007ooOol0B7oo000COol00`00Oomoo`0HOol0
0`00Oomoo`0`Ool00`00Oomoo`0GOol00`00Oomoo`1>Ool00`00Oomoo`17Ool001=oo`8001Yoo`03
001oogoo02moo`03001oogoo01Qoo`03001oogoo04ioo`03001oogoo04Ioo`004goo00<007ooOol0
6Goo00<007ooOol0;goo00<007ooOol06Goo00<007ooOol0CWoo00<007ooOol0AGoo000COol00`00
Oomoo`0JOol00`00Oomoo`0]Ool00`00Oomoo`0KOol00`00Oomoo`1>Ool00`00Oomoo`14Ool001=o
o`03001oogoo01Yoo`03001oogoo02eoo`03001oogoo01]oo`03001oogoo04moo`03001oogoo04=o
o`004goo00<007ooOol06goo00<007ooOol0;7oo00<007ooOol077oo00<007ooOol0CWoo00<007oo
Ool0@goo000COol00`00Oomoo`0KOol00`00Oomoo`0[Ool00`00Oomoo`0MOol00`00Oomoo`0kOol0
0`00Oomoo`0AOol00`00Oomoo`12Ool001=oo`8001eoo`03001oogoo02Yoo`03001oogoo01ioo`03
001oogoo03Yoo`03001oogoo015oo`03001oogoo049oo`004goo00<007ooOol07Goo00<007ooOol0
:Goo00<007ooOol07Woo00<007ooOol0>Goo00<007oo00004goo00<007ooOol0@Goo000COol00`00
Oomoo`0NOol00`00Oomoo`0WOol00`00Oomoo`0OOol00`00Oomoo`0hOol01@00Oomoogoo00004Woo
00<007ooOol0@Goo000COol00`00Oomoo`0OOol00`00Oomoo`0POol7000ROol00`00Oomoo`0gOol0
1@00Oomoogoo00004goo00<007ooOol0@7oo000COol00`00Oomoo`0POol00`00Oomoo`0OOol00`00
Oomoo`0VOol00`00Oomoo`0fOol00`00Oomoo`03Ool00`00Oomoo`0AOol00`00Oomoo`0oOol001=o
o`03001oogoo025oo`03001oogoo01eoo`03001oogoo02Qoo`03001oogoo03Aoo`03001oogoo00Eo
o`03001oogoo011oo`03001oogoo03moo`004goo0P008goo00<007ooOol077oo00<007ooOol0:7oo
00<007ooOol0=7oo00<007ooOol01Goo00<007ooOol04Goo00<007ooOol0?Woo000COol00`00Oomo
o`0ROol00`00Oomoo`0LOol00`00Oomoo`0YOol00`00Oomoo`0bOol00`00Oomoo`07Ool00`00Oomo
o`0@Ool00`00Oomoo`0nOol001=oo`03001oogoo02=oo`03001oogoo01Yoo`03001oogoo02Yoo`03
001oogoo035oo`03001oogoo00Uoo`03001oogoo011oo`03001oogoo03eoo`004goo00<007ooOol0
97oo00<007ooOol06Goo00<007ooOol0:Woo00<007ooOol0<Goo00<007ooOol02Goo00<007ooOol0
47oo00<007ooOol0<7oo00<007ooOol02Woo00001Goo00000000Ool010000goo0P001Goo00<007oo
Ool09Goo00<007ooOol05goo00<007ooOol0;7oo00<007ooOol0;goo00<007ooOol02goo00<007oo
Ool047oo00<007ooOol0;goo00<007ooOol02Woo0002Ool01@00Oomoogoo000017oo00@007ooOol0
00Aoo`8002Moo`03001oogoo01Ioo`03001oogoo02aoo`03001oogoo02moo`03001oogoo00]oo`03
001oogoo011oo`03001oogoo02ioo`8000aoo`000Woo00<007ooOol00Woo00D007ooOomoo`00009o
o`03001oogoo009oo`03001oogoo02Ioo`03001oogoo01Ioo`03001oogoo02eoo`03001oogoo02eo
o`03001oogoo00aoo`03001oogoo011oo`03001oogoo02eoo`04001oogoo000;Ool0009oo`03001o
ogoo00=oo`04001oogoo0002Ool00`00Oomoo`02Ool00`00Oomoo`0WOol00`00Oomoo`0DOol00`00
Oomoo`0^Ool00`00Oomoo`0]Ool00`00Oomoo`0=Ool00`00Oomoo`0@Ool00`00Oomoo`0[Ool01@00
Oomoogoo00002goo0002Ool00`00Oomoo`03Ool01000Oomoo`000Woo00<007ooOol00Woo00<007oo
Ool0:7oo00<007ooOol04goo00<007ooOol0;goo00<007ooOol0:goo00<007ooOol03Woo00<007oo
Ool047oo00<007ooOol0:Woo00<007ooOol00Woo00<007ooOol02Goo00000goo00000002Ool30004
Ool20005Ool00`00Oomoo`0YOol7000=Ool00`00Oomoo`0`Ool00`00Oomoo`0[Ool00`00Oomoo`0?
Ool00`00Oomoo`0@Ool00`00Oomoo`0YOol00`00Oomoo`02Ool00`00Oomoo`09Ool001=oo`03001o
ogoo02moo`03001oogoo00]oo`03001oogoo035oo`03001oogoo02Uoo`03001oogoo011oo`03001o
ogoo011oo`03001oogoo02Qoo`03001oogoo00Aoo`03001oogoo00Qoo`004goo0P00<Goo00<007oo
Ool02Woo00<007ooOol0<Goo00<007ooOol0:Goo00<007ooOol047oo00<007ooOol047oo00<007oo
Ool06Woo00<007ooOol02Woo00<007ooOol01Goo00<007ooOol027oo000COol00`00Oomoo`0aOol0
0`00Oomoo`07Ool2000eOol00`00Oomoo`0XOol00`00Oomoo`0AOol00`00Oomoo`0@Ool00`00Oomo
o`0IOol00`00Oomoo`09Ool00`00Oomoo`06Ool00`00Oomoo`08Ool001=oo`03001oogoo039oo`03
001oogoo00Eoo`03001oogoo03Eoo`03001oogoo00aoo`03001oogoo01Qoo`03001oogoo019oo`03
001oogoo011oo`03001oogoo01Qoo`03001oo`0000Uoo`03001oogoo00Moo`03001oogoo00Qoo`00
4goo00<007ooOol0<Woo00<007ooOol00goo0P00>Goo00<007ooOol02goo0P006Goo00<007ooOol0
4Woo00<007ooOol047oo00<007ooOol067oo00@007ooOol000Moo`03001oogoo00Qoo`03001oogoo
00Qoo`004goo00<007ooOol0<goo00D007ooOomoo`0003]oo`03001oogoo00Yoo`04001oogoo000G
Ool00`00Oomoo`0DOol00`00Oomoo`0@Ool00`00Oomoo`0FOol00`00Oomoo`02Ool00`00Oomoo`03
Ool00`00Oomoo`0:Ool00`00Oomoo`07Ool001=oo`03001oogoo03Aoo`03001oo`0003eoo`03001o
ogoo00Qoo`03001oogoo009oo`03001oogoo01Aoo`03001oogoo01Aoo`03001oogoo011oo`03001o
ogoo01Ioo`03001oogoo00=oo`05001oogooOol0000=Ool00`00Oomoo`07Ool001=oo`8003Ioo`03
001oogoo03aoo`03001oogoo00Qoo`03001oogoo00=oo`03001oogoo019oo`03001oogoo01Ioo`03
001oogoo011oo`03001oogoo01Aoo`03001oogoo00Eoo`03001oo`0000ioo`03001oogoo00Moo`00
4goo00<007ooOol0MGoo0P001goo00<007ooOol01Goo00<007ooOol04Goo00<007ooOol05Woo00<0
07ooOol047oo00<007ooOol057oo00<007ooOol01Woo00<007ooOol03Goo00<007ooOol01goo000C
Ool00`00Oomoo`1gOol00`00Oomoo`03Ool00`00Oomoo`07Ool00`00Oomoo`0?Ool00`00Oomoo`0G
Ool00`00Oomoo`0@Ool00`00Oomoo`0DOol00`00Oomoo`0<Ool00`00Oomoo`08Ool00`00Oomoo`06
Ool001=oo`03001oogoo07Qoo`8000=oo`03001oogoo00Moo`03001oogoo00moo`03001oogoo01Qo
o`03001oogoo011oo`03001oogoo019oo`03001oogoo00eoo`03001oogoo00Qoo`03001oogoo00Io
o`004goo00<007ooOol0NWoo00<007oo00002goo00<007ooOol03Goo00<007ooOol06Goo00<007oo
Ool047oo00<007ooOol04Woo00<007ooOol037oo00<007oo00002Goo00<007ooOol01Woo000COol2
001lOol00`00Oomoo`0;Ool00`00Oomoo`0<Ool00`00Oomoo`0JOol00`00Oomoo`0?Ool00`00Oomo
o`0AOol00`00Oomoo`0=Ool01000Oomoo`0027oo00<007ooOol01Woo000COol00`00Oomoo`29Ool0
0`00Oomoo`0;Ool00`00Oomoo`0LOol00`00Oomoo`0?Ool01000Oomoogoo0`0037oo00<007ooOol0
3Goo00D007ooOomoo`0000Qoo`03001oogoo00Eoo`004goo00<007ooOol0RWoo00<007ooOol02Woo
00<007ooOol077oo00<007ooOol03goo10000goo00<007ooOol01goo0P003goo00<007ooOol00goo
00<007ooOol01Goo00<007ooOol01Goo000COol00`00Oomoo`2;Ool00`00Oomoo`08Ool00`00Oomo
o`0NOol00`00Oomoo`0;Ool40007Ool20004Ool3000AOol00`00Oomoo`03Ool00`00Oomoo`05Ool0
0`00Oomoo`05Ool001=oo`03001oogoo08]oo`03001oogoo00Qoo`03001oogoo01moo`03001oogoo
00Moo`<000=oo`03001oogoo00Moo`04001oo`00000DOol00`00Oomoo`04Ool00`00Oomoo`04Ool0
0`00Oomoo`05Ool001=oo`03001oogoo08aoo`03001oogoo00Ioo`03001oogoo021oo`03001oogoo
00Aoo`<000Ioo`03001oogoo00Qoo`03001oogoo01=oo`03001oogoo00Ioo`03001oogoo00=oo`03
001oogoo00Eoo`004goo0P00SGoo00<007ooOol01Woo00<007ooOol08Goo00<007ooOol00`002Woo
00<007ooOol07Goo00<007ooOol01goo00<007ooOol00goo00<007ooOol017oo000COol00`00Oomo
o`2=Ool01000Oomoogoo10009Goo0P003Goo00<007ooOol07Goo00<007ooOol01goo00<007ooOol0
0goo00<007ooOol017oo000COol00`00Oomoo`2>Ool3000hOol00`00Oomoo`0LOol00`00Oomoo`09
Ool00`00Oomoo`09Ool001=oo`03001oogoo0<Uoo`03001oogoo01aoo`03001oogoo00Uoo`03001o
ogoo00Uoo`0000Eoo`00000007oo00<000Aoo`8000Eoo`03001oogoo0<Uoo`03001oogoo01aoo`03
001oogoo00Yoo`03001oogoo00Qoo`000Woo00<007ooOol00goo00@007ooOol0009oo`03001oogoo
009oo`800<]oo`03001oogoo01Yoo`03001oogoo00]oo`03001oogoo00Qoo`000Woo00<007ooOol0
0goo00@007ooOol0009oo`03001oogoo009oo`03001oogoo0<Yoo`03001oogoo00Ioo`03001oogoo
015oo`03001oogoo00aoo`03001oogoo00Moo`000Woo00@007ooOomoo`8000=oo`04001oogoo0004
Ool00`00Oomoo`3:Ool00`00Oomoo`06Ool00`00Oomoo`0AOol00`00Oomoo`0<Ool00`00Oomoo`07
Ool0009oo`03001oogoo00=oo`04001oogoo0002Ool00`00Oomoo`02Ool00`00Oomoo`3:Ool00`00
Oomoo`05Ool2000BOol00`00Oomoo`0>Ool00`00Oomoo`06Ool00003Ool00000009oo`<000Aoo`80
00Eoo`03001oogoo0<Yoo`03001oogoo00Eoo`03001oo`00011oo`80011oo`03001oogoo00Ioo`00
4goo00<007ooOol0bgoo00<007ooOol017oo00<007oo00003Goo0`004goo00<007ooOol01Goo000C
Ool2003<Ool00`00Oomoo`03Ool01000Oomoo`002goo0P005Woo00<007ooOol01Goo000COol00`00
Oomoo`3;Ool00`00Oomoo`03Ool01@00Oomoogoo00002Woo00<007ooOol05Woo00<007ooOol017oo
000COol00`00Oomoo`3;Ool00`00Oomoo`03Ool01@00Oomoogoo00002Goo00<007ooOol07Woo000C
Ool00`00Oomoo`3;Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`07Ool00`00Oomoo`0N
Ool001=oo`03001oogoo0<aoo`05001oogooOol00005Ool00`00Oomoo`06Ool00`00Oomoo`0NOol0
01=oo`03001oogoo0<aoo`04001oogoo0006Ool00`00Oomoo`05Ool00`00Oomoo`0OOol001=oo`80
0<eoo`04001oogoo0006Ool00`00Oomoo`05Ool00`00Oomoo`0OOol001=oo`03001oogoo0<aoo`04
001oogoo0007Ool00`00Oomoo`04Ool00`00Oomoo`0OOol001=oo`03001oogoo0<aoo`03001oo`00
00Qoo`03001oogoo00=oo`03001oogoo021oo`004goo00<007ooOol0cGoo0P0027oo00<007ooOol0
0goo00<007ooOol087oo000COol00`00Oomoo`3=Ool20009Ool00`00Oomoo`02Ool00`00Oomoo`0P
Ool001=oo`800<ioo`03001oogoo00Qoo`05001oogooOol0000SOol001=oo`03001oogoo0=Qoo`05
001oogooOol0000SOol001=oo`03001oogoo0=Uoo`04001oogoo000SOol001=oo`03001oogoo0=Uo
o`03001oo`0002Aoo`004goo00<007ooOol0fGoo00<007oo000097oo000COol00`00Oomoo`3JOol2
000TOol001=oo`800=]oo`03001oogoo02=oo`004goo00<007ooOol0ogoo0Goo000COol00`00Oomo
o`3oOol1Ool001=oo`03001oogoo0?moo`5oo`004goo00<007ooOol0ogoo0Goo003oOolGOol00?mo
oaMoo`00ogoo5goo0000\
\>"],
  ImageRangeCache->{{{0, 277.938}, {170.562, 0}} -> {-3.01968, 80.1804, 
  0.197976, 0.44303}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
example = pathpairat[100, 0.1, 0, 0.2, 1, 5, 40];
plots = Table[ListPlot[example[[i]], PlotJoined -> True,
DisplayFunction -> Identity], {i, 1, Length[example]}];
Show[plots, DisplayFunction -> $DisplayFunction];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 7  scalefont  setfont
% Scaling calculations
0.0238095 0.0238095 -0.574205 0.00801523 [
[.2619 .05451 -4.2 -6.3 ]
[.2619 .05451 4.2 0 ]
[.5 .05451 -4.2 -6.3 ]
[.5 .05451 4.2 0 ]
[.7381 .05451 -4.2 -6.3 ]
[.7381 .05451 4.2 0 ]
[.97619 .05451 -4.2 -6.3 ]
[.97619 .05451 4.2 0 ]
[.01131 .14717 -8.4 -3.15 ]
[.01131 .14717 0 3.15 ]
[.01131 .22732 -12.6 -3.15 ]
[.01131 .22732 0 3.15 ]
[.01131 .30747 -12.6 -3.15 ]
[.01131 .30747 0 3.15 ]
[.01131 .38762 -12.6 -3.15 ]
[.01131 .38762 0 3.15 ]
[.01131 .46777 -12.6 -3.15 ]
[.01131 .46777 0 3.15 ]
[.01131 .54793 -12.6 -3.15 ]
[.01131 .54793 0 3.15 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .06701 m
.2619 .07326 L
s
[(10)] .2619 .05451 0 1 Mshowa
.5 .06701 m
.5 .07326 L
s
[(20)] .5 .05451 0 1 Mshowa
.7381 .06701 m
.7381 .07326 L
s
[(30)] .7381 .05451 0 1 Mshowa
.97619 .06701 m
.97619 .07326 L
s
[(40)] .97619 .05451 0 1 Mshowa
.125 Mabswid
.07143 .06701 m
.07143 .07076 L
s
.11905 .06701 m
.11905 .07076 L
s
.16667 .06701 m
.16667 .07076 L
s
.21429 .06701 m
.21429 .07076 L
s
.30952 .06701 m
.30952 .07076 L
s
.35714 .06701 m
.35714 .07076 L
s
.40476 .06701 m
.40476 .07076 L
s
.45238 .06701 m
.45238 .07076 L
s
.54762 .06701 m
.54762 .07076 L
s
.59524 .06701 m
.59524 .07076 L
s
.64286 .06701 m
.64286 .07076 L
s
.69048 .06701 m
.69048 .07076 L
s
.78571 .06701 m
.78571 .07076 L
s
.83333 .06701 m
.83333 .07076 L
s
.88095 .06701 m
.88095 .07076 L
s
.92857 .06701 m
.92857 .07076 L
s
.25 Mabswid
0 .06701 m
1 .06701 L
s
.02381 .14717 m
.03006 .14717 L
s
[(90)] .01131 .14717 1 0 Mshowa
.02381 .22732 m
.03006 .22732 L
s
[(100)] .01131 .22732 1 0 Mshowa
.02381 .30747 m
.03006 .30747 L
s
[(110)] .01131 .30747 1 0 Mshowa
.02381 .38762 m
.03006 .38762 L
s
[(120)] .01131 .38762 1 0 Mshowa
.02381 .46777 m
.03006 .46777 L
s
[(130)] .01131 .46777 1 0 Mshowa
.02381 .54793 m
.03006 .54793 L
s
[(140)] .01131 .54793 1 0 Mshowa
.125 Mabswid
.02381 .08304 m
.02756 .08304 L
s
.02381 .09907 m
.02756 .09907 L
s
.02381 .1151 m
.02756 .1151 L
s
.02381 .13114 m
.02756 .13114 L
s
.02381 .1632 m
.02756 .1632 L
s
.02381 .17923 m
.02756 .17923 L
s
.02381 .19526 m
.02756 .19526 L
s
.02381 .21129 m
.02756 .21129 L
s
.02381 .24335 m
.02756 .24335 L
s
.02381 .25938 m
.02756 .25938 L
s
.02381 .27541 m
.02756 .27541 L
s
.02381 .29144 m
.02756 .29144 L
s
.02381 .3235 m
.02756 .3235 L
s
.02381 .33953 m
.02756 .33953 L
s
.02381 .35556 m
.02756 .35556 L
s
.02381 .37159 m
.02756 .37159 L
s
.02381 .40365 m
.02756 .40365 L
s
.02381 .41968 m
.02756 .41968 L
s
.02381 .43571 m
.02756 .43571 L
s
.02381 .45174 m
.02756 .45174 L
s
.02381 .48381 m
.02756 .48381 L
s
.02381 .49984 m
.02756 .49984 L
s
.02381 .51587 m
.02756 .51587 L
s
.02381 .5319 m
.02756 .5319 L
s
.02381 .05098 m
.02756 .05098 L
s
.02381 .03495 m
.02756 .03495 L
s
.02381 .01892 m
.02756 .01892 L
s
.02381 .00289 m
.02756 .00289 L
s
.02381 .56396 m
.02756 .56396 L
s
.02381 .57999 m
.02756 .57999 L
s
.02381 .59602 m
.02756 .59602 L
s
.02381 .61205 m
.02756 .61205 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.04762 .22732 m
.07143 .17402 L
.09524 .17245 L
.11905 .15066 L
.14286 .13529 L
.16667 .12763 L
.19048 .11159 L
.21429 .13531 L
.2381 .13919 L
.2619 .16138 L
.28571 .16762 L
.30952 .14158 L
.33333 .09594 L
.35714 .11446 L
.38095 .12558 L
.40476 .13452 L
.42857 .11542 L
.45238 .15123 L
.47619 .15927 L
.5 .1344 L
.52381 .13733 L
.54762 .12384 L
.57143 .10503 L
.59524 .11264 L
.61905 .08847 L
.64286 .08441 L
.66667 .0971 L
.69048 .11935 L
.71429 .09534 L
.7381 .10199 L
.7619 .13574 L
.78571 .13871 L
.80952 .15029 L
.83333 .13575 L
.85714 .13104 L
.88095 .12444 L
.90476 .13112 L
.92857 .11585 L
.95238 .11816 L
.97619 .15566 L
s
.04762 .22732 m
.07143 .28472 L
.09524 .29094 L
.11905 .32062 L
.14286 .34418 L
.16667 .3588 L
.19048 .38491 L
.21429 .35666 L
.2381 .35635 L
.2619 .33217 L
.28571 .32913 L
.30952 .36547 L
.33333 .43021 L
.35714 .4076 L
.38095 .39679 L
.40476 .38935 L
.42857 .42027 L
.45238 .37372 L
.47619 .36808 L
.5 .40486 L
.52381 .40583 L
.54762 .42944 L
.57143 .46163 L
.59524 .45533 L
.61905 .49684 L
.64286 .5089 L
.66667 .49358 L
.69048 .46367 L
.71429 .50492 L
.7381 .49973 L
.7619 .45165 L
.78571 .45261 L
.80952 .4412 L
.83333 .46679 L
.85714 .47903 L
.88095 .49429 L
.90476 .48955 L
.92857 .51804 L
.95238 .51999 L
.97619 .46633 L
s
.04762 .22732 m
.07143 .21811 L
.09524 .21461 L
.11905 .20169 L
.14286 .17311 L
.16667 .18049 L
.19048 .1577 L
.21429 .18651 L
.2381 .17721 L
.2619 .16288 L
.28571 .17169 L
.30952 .17498 L
.33333 .14584 L
.35714 .1445 L
.38095 .21106 L
.40476 .19933 L
.42857 .18914 L
.45238 .16012 L
.47619 .16434 L
.5 .15751 L
.52381 .13323 L
.54762 .14265 L
.57143 .14181 L
.59524 .12104 L
.61905 .08997 L
.64286 .12013 L
.66667 .15844 L
.69048 .14256 L
.71429 .16829 L
.7381 .19113 L
.7619 .19919 L
.78571 .19378 L
.80952 .21763 L
.83333 .28813 L
.85714 .29446 L
.88095 .2895 L
.90476 .31108 L
.92857 .27244 L
.95238 .27751 L
.97619 .28116 L
s
.04762 .22732 m
.07143 .24064 L
.09524 .24841 L
.11905 .26611 L
.14286 .30137 L
.16667 .29722 L
.19048 .328 L
.21429 .2971 L
.2381 .31223 L
.2619 .33368 L
.28571 .32749 L
.30952 .32813 L
.33333 .36786 L
.35714 .37445 L
.38095 .29145 L
.40476 .30882 L
.42857 .32498 L
.45238 .36378 L
.47619 .3632 L
.5 .37667 L
.52381 .41309 L
.54762 .40501 L
.57143 .41118 L
.59524 .44482 L
.61905 .49559 L
.64286 .4525 L
.66667 .40112 L
.69048 .42726 L
.71429 .39643 L
.7381 .37156 L
.7619 .36644 L
.78571 .37786 L
.80952 .35317 L
.83333 .27536 L
.85714 .27347 L
.88095 .28266 L
.90476 .26565 L
.92857 .30661 L
.95238 .30585 L
.97619 .30659 L
s
.04762 .22732 m
.07143 .22597 L
.09524 .21972 L
.11905 .23941 L
.14286 .20567 L
.16667 .22712 L
.19048 .28836 L
.21429 .29021 L
.2381 .30377 L
.2619 .32607 L
.28571 .34347 L
.30952 .33757 L
.33333 .34072 L
.35714 .32998 L
.38095 .29992 L
.40476 .30747 L
.42857 .32142 L
.45238 .29881 L
.47619 .29745 L
.5 .28261 L
.52381 .28499 L
.54762 .28717 L
.57143 .27711 L
.59524 .28469 L
.61905 .30302 L
.64286 .31433 L
.66667 .31127 L
.69048 .33517 L
.71429 .34693 L
.7381 .3286 L
.7619 .35569 L
.78571 .31563 L
.80952 .27314 L
.83333 .22507 L
.85714 .18644 L
.88095 .16492 L
.90476 .16085 L
.92857 .15209 L
.95238 .15169 L
.97619 .16034 L
s
.04762 .22732 m
.07143 .23277 L
.09524 .24321 L
.11905 .22714 L
.14286 .26447 L
.16667 .24572 L
.19048 .18726 L
.21429 .18952 L
.2381 .18147 L
.2619 .16614 L
.28571 .15563 L
.30952 .16407 L
.33333 .16531 L
.35714 .17778 L
.38095 .20664 L
.40476 .2039 L
.42857 .19557 L
.45238 .21895 L
.47619 .22426 L
.5 .24195 L
.52381 .24386 L
.54762 .24598 L
.57143 .25977 L
.59524 .25662 L
.61905 .24315 L
.64286 .2368 L
.66667 .24376 L
.69048 .22587 L
.71429 .21963 L
.7381 .2395 L
.7619 .21925 L
.78571 .2575 L
.80952 .30149 L
.83333 .35565 L
.85714 .40535 L
.88095 .4381 L
.90476 .44887 L
.92857 .4663 L
.95238 .47221 L
.97619 .46511 L
s
.04762 .22732 m
.07143 .21004 L
.09524 .19525 L
.11905 .19744 L
.14286 .18403 L
.16667 .1733 L
.19048 .15847 L
.21429 .15718 L
.2381 .11899 L
.2619 .1352 L
.28571 .14863 L
.30952 .17595 L
.33333 .22418 L
.35714 .17255 L
.38095 .15371 L
.40476 .14133 L
.42857 .16539 L
.45238 .17098 L
.47619 .16427 L
.5 .18851 L
.52381 .1648 L
.54762 .16582 L
.57143 .16176 L
.59524 .17948 L
.61905 .19468 L
.64286 .19776 L
.66667 .24109 L
.69048 .27214 L
.71429 .27662 L
.7381 .29499 L
.7619 .28804 L
.78571 .28476 L
.80952 .27475 L
.83333 .20463 L
.85714 .21908 L
.88095 .1785 L
.90476 .21569 L
.92857 .25524 L
.95238 .25692 L
.97619 .21746 L
s
.04762 .22732 m
.07143 .24871 L
.09524 .26844 L
.11905 .27037 L
.14286 .28938 L
.16667 .30603 L
.19048 .328 L
.21429 .33422 L
.2381 .38631 L
.2619 .36877 L
.28571 .35576 L
.30952 .32538 L
.33333 .27216 L
.35714 .33123 L
.38095 .35871 L
.40476 .37938 L
.42857 .35219 L
.45238 .34995 L
.47619 .363 L
.5 .33705 L
.52381 .37005 L
.54762 .37358 L
.57143 .38364 L
.59524 .36549 L
.61905 .35137 L
.64286 .3524 L
.66667 .30515 L
.69048 .27617 L
.71429 .27602 L
.7381 .26203 L
.7619 .27301 L
.78571 .28057 L
.80952 .29491 L
.83333 .37116 L
.85714 .35847 L
.88095 .41096 L
.90476 .36733 L
.92857 .32502 L
.95238 .32781 L
.97619 .37526 L
s
.04762 .22732 m
.07143 .21269 L
.09524 .23727 L
.11905 .2597 L
.14286 .26577 L
.16667 .2671 L
.19048 .24888 L
.21429 .2334 L
.2381 .21742 L
.2619 .20797 L
.28571 .20017 L
.30952 .20068 L
.33333 .20548 L
.35714 .20983 L
.38095 .20009 L
.40476 .19931 L
.42857 .16732 L
.45238 .20977 L
.47619 .22072 L
.5 .23894 L
.52381 .24796 L
.54762 .23936 L
.57143 .2245 L
.59524 .24364 L
.61905 .29494 L
.64286 .27005 L
.66667 .30126 L
.69048 .29255 L
.71429 .32506 L
.7381 .39303 L
.7619 .40528 L
.78571 .41477 L
.80952 .41946 L
.83333 .46704 L
.85714 .48826 L
.88095 .46667 L
.90476 .56633 L
.92857 .56444 L
.95238 .52318 L
.97619 .55102 L
s
.04762 .22732 m
.07143 .24605 L
.09524 .22464 L
.11905 .20665 L
.14286 .20498 L
.16667 .20774 L
.19048 .22868 L
.21429 .2479 L
.2381 .26838 L
.2619 .28277 L
.28571 .2957 L
.30952 .29959 L
.33333 .29866 L
.35714 .29826 L
.38095 .31357 L
.40476 .31903 L
.42857 .36054 L
.45238 .31183 L
.47619 .304 L
.5 .28838 L
.52381 .28323 L
.54762 .2966 L
.57143 .31696 L
.59524 .30019 L
.61905 .24982 L
.64286 .27764 L
.66667 .25052 L
.69048 .26295 L
.71429 .23584 L
.7381 .17877 L
.7619 .1731 L
.78571 .16969 L
.80952 .16998 L
.83333 .13816 L
.85714 .12729 L
.88095 .14515 L
.90476 .07996 L
.92857 .0844 L
.95238 .11164 L
.97619 .09776 L
s
.04762 .22732 m
.07143 .25928 L
.09524 .26967 L
.11905 .26078 L
.14286 .24786 L
.16667 .26026 L
.19048 .24862 L
.21429 .23571 L
.2381 .21188 L
.2619 .22513 L
.28571 .1938 L
.30952 .19411 L
.33333 .22023 L
.35714 .2031 L
.38095 .23637 L
.40476 .23966 L
.42857 .24976 L
.45238 .2471 L
.47619 .24914 L
.5 .2059 L
.52381 .21587 L
.54762 .21589 L
.57143 .23774 L
.59524 .23978 L
.61905 .22114 L
.64286 .21061 L
.66667 .24802 L
.69048 .26109 L
.71429 .29724 L
.7381 .32172 L
.7619 .33221 L
.78571 .30461 L
.80952 .29782 L
.83333 .33902 L
.85714 .36469 L
.88095 .37187 L
.90476 .36103 L
.92857 .38839 L
.95238 .4363 L
.97619 .41571 L
s
.04762 .22732 m
.07143 .19946 L
.09524 .19379 L
.11905 .20582 L
.14286 .22189 L
.16667 .21396 L
.19048 .229 L
.21429 .24572 L
.2381 .27404 L
.2619 .2641 L
.28571 .30126 L
.30952 .30539 L
.33333 .28 L
.35714 .3028 L
.38095 .26976 L
.40476 .27066 L
.42857 .26451 L
.45238 .27152 L
.47619 .27376 L
.5 .32264 L
.52381 .31578 L
.54762 .32032 L
.57143 .30017 L
.59524 .30245 L
.61905 .32702 L
.64286 .34358 L
.66667 .30454 L
.69048 .29507 L
.71429 .26192 L
.7381 .24271 L
.7619 .23734 L
.78571 .26621 L
.80952 .27701 L
.83333 .24116 L
.85714 .22242 L
.88095 .22042 L
.90476 .23359 L
.92857 .2141 L
.95238 .17892 L
.97619 .19812 L
s
.04762 .22732 m
.07143 .23728 L
.09524 .25909 L
.11905 .22051 L
.14286 .24074 L
.16667 .26983 L
.19048 .30721 L
.21429 .31112 L
.2381 .33747 L
.2619 .29624 L
.28571 .28075 L
.30952 .26826 L
.33333 .26589 L
.35714 .20339 L
.38095 .23865 L
.40476 .20576 L
.42857 .20564 L
.45238 .22211 L
.47619 .19655 L
.5 .15496 L
.52381 .14677 L
.54762 .16641 L
.57143 .19058 L
.59524 .18595 L
.61905 .19392 L
.64286 .21027 L
.66667 .20515 L
.69048 .20119 L
.71429 .22932 L
.7381 .23645 L
.7619 .23724 L
.78571 .27165 L
.80952 .29097 L
.83333 .27471 L
.85714 .27574 L
.88095 .32818 L
.90476 .32988 L
.92857 .35034 L
.95238 .41416 L
.97619 .4602 L
s
.04762 .22732 m
.07143 .22146 L
.09524 .20416 L
.11905 .24419 L
.14286 .22755 L
.16667 .20305 L
.19048 .17261 L
.21429 .17313 L
.2381 .15472 L
.2619 .19142 L
.28571 .20897 L
.30952 .22443 L
.33333 .23077 L
.35714 .29479 L
.38095 .25983 L
.40476 .29786 L
.42857 .30247 L
.45238 .28845 L
.47619 .32056 L
.5 .37344 L
.52381 .38893 L
.54762 .36764 L
.57143 .34173 L
.59524 .35197 L
.61905 .34701 L
.64286 .33213 L
.66667 .34269 L
.69048 .35205 L
.71429 .3232 L
.7381 .31984 L
.7619 .32355 L
.78571 .29009 L
.80952 .27478 L
.83333 .29509 L
.85714 .29849 L
.88095 .24912 L
.90476 .25179 L
.92857 .23734 L
.95238 .18548 L
.97619 .15398 L
s
.04762 .22732 m
.07143 .21783 L
.09524 .21547 L
.11905 .19217 L
.14286 .14352 L
.16667 .1035 L
.19048 .10809 L
.21429 .14513 L
.2381 .14132 L
.2619 .16407 L
.28571 .1565 L
.30952 .12812 L
.33333 .14605 L
.35714 .0911 L
.38095 .0527 L
.40476 .03328 L
.42857 .04379 L
.45238 .01598 L
.47619 .03592 L
.5 .0202 L
.52381 .01472 L
.54762 .0159 L
.57143 .04059 L
.59524 .02752 L
.61905 .023 L
.64286 .04846 L
.66667 .05704 L
.69048 .0453 L
.71429 .0551 L
.7381 .04958 L
.7619 .0681 L
.78571 .04709 L
.80952 .06946 L
.83333 .07274 L
.85714 .09099 L
.88095 .08663 L
.90476 .06923 L
.92857 .09144 L
.95238 .08137 L
.97619 .11784 L
s
.04762 .22732 m
.07143 .24091 L
.09524 .24752 L
.11905 .27599 L
.14286 .33431 L
.16667 .38963 L
.19048 .38805 L
.21429 .34074 L
.2381 .35028 L
.2619 .32562 L
.28571 .33947 L
.30952 .37964 L
.33333 .36018 L
.35714 .43625 L
.38095 .49976 L
.40476 .53854 L
.42857 .525 L
.45238 .5801 L
.47619 .54702 L
.5 .58166 L
.52381 .59825 L
.54762 .6019 L
.57143 .55872 L
.59524 .58863 L
.61905 .60332 L
.64286 .55915 L
.66667 .54934 L
.69048 .57601 L
.71429 .56371 L
.7381 .57953 L
.7619 .5512 L
.78571 .59378 L
.80952 .55771 L
.83333 .55775 L
.85714 .53163 L
.88095 .54454 L
.90476 .57973 L
.92857 .54581 L
.95238 .56851 L
.97619 .5108 L
s
.04762 .22732 m
.07143 .20035 L
.09524 .22949 L
.11905 .21735 L
.14286 .20694 L
.16667 .18797 L
.19048 .18169 L
.21429 .17172 L
.2381 .20873 L
.2619 .24491 L
.28571 .23419 L
.30952 .22154 L
.33333 .20472 L
.35714 .19319 L
.38095 .21446 L
.40476 .22664 L
.42857 .21457 L
.45238 .22099 L
.47619 .261 L
.5 .27839 L
.52381 .32508 L
.54762 .31777 L
.57143 .32656 L
.59524 .31036 L
.61905 .31863 L
.64286 .26864 L
.66667 .33826 L
.69048 .35864 L
.71429 .34802 L
.7381 .36886 L
.7619 .3168 L
.78571 .3114 L
.80952 .33456 L
.83333 .34867 L
.85714 .35883 L
.88095 .32287 L
.90476 .26068 L
.92857 .24956 L
.95238 .26002 L
.97619 .23842 L
s
.04762 .22732 m
.07143 .2584 L
.09524 .23134 L
.11905 .24764 L
.14286 .26266 L
.16667 .28728 L
.19048 .29879 L
.21429 .31478 L
.2381 .27524 L
.2619 .24034 L
.28571 .25517 L
.30952 .27241 L
.33333 .29465 L
.35714 .31196 L
.38095 .29194 L
.40476 .28301 L
.42857 .30033 L
.45238 .29769 L
.47619 .2583 L
.5 .24523 L
.52381 .20456 L
.54762 .21489 L
.57143 .21115 L
.59524 .22931 L
.61905 .22591 L
.64286 .27482 L
.66667 .20904 L
.69048 .19556 L
.71429 .20827 L
.7381 .1946 L
.7619 .24099 L
.78571 .25011 L
.80952 .23278 L
.83333 .22439 L
.85714 .21969 L
.88095 .25436 L
.90476 .31604 L
.92857 .33248 L
.95238 .3256 L
.97619 .35352 L
s
.04762 .22732 m
.07143 .30217 L
.09524 .30779 L
.11905 .29579 L
.14286 .32453 L
.16667 .33668 L
.19048 .30203 L
.21429 .33542 L
.2381 .31649 L
.2619 .29193 L
.28571 .30384 L
.30952 .32424 L
.33333 .30206 L
.35714 .28718 L
.38095 .28107 L
.40476 .28002 L
.42857 .28029 L
.45238 .24669 L
.47619 .27587 L
.5 .29251 L
.52381 .28681 L
.54762 .32589 L
.57143 .27768 L
.59524 .31133 L
.61905 .33198 L
.64286 .29675 L
.66667 .28594 L
.69048 .27508 L
.71429 .2759 L
.7381 .28148 L
.7619 .25836 L
.78571 .25185 L
.80952 .25943 L
.83333 .2851 L
.85714 .32045 L
.88095 .31283 L
.90476 .27422 L
.92857 .24769 L
.95238 .31638 L
.97619 .35182 L
s
.04762 .22732 m
.07143 .15657 L
.09524 .15564 L
.11905 .16931 L
.14286 .14857 L
.16667 .1425 L
.19048 .17343 L
.21429 .14878 L
.2381 .16754 L
.2619 .19179 L
.28571 .18519 L
.30952 .17143 L
.33333 .19367 L
.35714 .21065 L
.38095 .22024 L
.40476 .22528 L
.42857 .22913 L
.45238 .26484 L
.47619 .23932 L
.5 .22756 L
.52381 .23695 L
.54762 .20429 L
.57143 .24998 L
.59524 .22165 L
.61905 .20718 L
.64286 .24156 L
.66667 .25587 L
.69048 .2706 L
.71429 .27412 L
.7381 .2729 L
.7619 .30014 L
.78571 .31146 L
.80952 .30787 L
.83333 .28523 L
.85714 .25429 L
.88095 .26559 L
.90476 .30645 L
.92857 .33851 L
.95238 .26691 L
.97619 .23775 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{223.188, 137.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004F0000Za000`40O003h00Oogoo5goo003oOolG
Ool00?mooaMoo`004goo00<007ooOol0ogoo0Goo000COol2003oOol2Ool001=oo`03001oogoo0?mo
o`5oo`004goo00<007ooOol0ogoo0Goo000COol00`00Oomoo`20Ool8001hOol001=oo`8006moo`80
00eoo`<000Qoo`03001oogoo07Eoo`004goo00<007ooOol0KWoo00<007oo00002Goo0`0037oo00<0
07ooOol03Goo0P00IGoo000COol00`00Oomoo`1]Ool01000Oomoogoo0P001Woo00<007ooOol03Woo
00<007ooOol02Goo0`000Woo00<007ooOol0HWoo000COol00`00Oomoo`1/Ool00`00Oomoo`04Ool0
1000Oomoogoo0P004Woo00<007ooOol01Goo0`001Woo00<007ooOol0HGoo000COol00`00Oomoo`0h
Ool30002Ool2000ROol30007Ool00`00Oomoo`06Ool00`00Ool000030002Ool2000>Ool00`00Oomo
o`02Ool2000:Ool00`00Oomoo`0FOol30003Ool2000iOol01000Oomoogoo0P000goo000COol2000j
Ool01000Oomoo`000Woo00<007ooOol07Woo00@007ooOomoo`8000Aoo`03001oogoo00Qoo`03001o
o`0000=oo`04001oogoo000>Ool01000Ool000003Goo00<007ooOol067oo00<007oo00000Woo00<0
07ooOol0=7oo100000Eoo`00Oomoo`00009oo`004goo00<007ooOol0>Goo00@007ooOol0009oo`03
001oogoo01aoo`8000Ioo`800003Ool007oo00eoo`04001oogoo0002Ool00`00Oomoo`0=Ool00`00
Oomoo`0>Ool00`00Oomoo`0GOol00`00Ool00002Ool00`00Oomoo`0dOol00`00Ool00002Ool01000
Oomoo`000Woo000COol00`00Oomoo`0iOol01000Oomoo`000Woo00<007ooOol06goo00<007ooOol0
27oo00<007ooOol03Woo00<007oo00000Woo00<007ooOol07goo00<007ooOol02Goo0`0027oo0P00
0Woo00@007ooOol003Ioo`03001oo`00009oo`04001oogoo0002Ool001=oo`03001oogoo03Uoo`04
001oogoo0002Ool00`00Oomoo`0JOol00`00Oomoo`0JOol00`00Ool00002Ool00`00Oomoo`0POol3
0006Ool20003Ool30007Ool30002Ool00`00Oomoo`05Ool2000^Ool20002Ool01000Oomoo`000Woo
000COol2000iOol20003Ool2000LOol00`00Oomoo`0HOol30003Ool2000VOol300000goo00000008
Ool20002Ool30003Ool20007Ool01000Oomoo`00;Woo00@007ooOomoo`8000=oo`004goo00<007oo
Ool0Fgoo00<007ooOol0BGoo00<007ooOol02Woo0P001goo00<007ooOol00goo0P0017oo00<007oo
Ool0=7oo000COol00`00Oomoo`1JOol00`00Oomoo`1QOol01@00Oomoogoo00001goo00<007ooOol0
<goo000COol00`00Oomoo`1IOol00`00Oomoo`1SOol00`00Ool00009Ool00`00Oomoo`0bOol000ao
ool000/000004goo00<007ooOol02Goo00<007ooOol02Woo00<007ooOol02Goo00<007ooOol02Woo
00<007ooOol02Woo00<007ooOol02Goo00<007ooOol02Woo00@007ooOol000Uoo`03001oogoo00Uo
o`03001oogoo00Yoo`03001oogoo00Uoo`03001oogoo00Yoo`03001oogoo00Yoo`03001oogoo00Uo
o`03001oogoo00Yoo`03001oogoo00Yoo`03001oogoo00=oo`@0009oo`03001oogoo00Yoo`03001o
ogoo009oo`8000Eoo`03001oogoo00Yoo`03001oogoo00Aoo`004goo00<007ooOol0?7oo00<007oo
Ool06Goo00<007ooOol087oo00<007ooOol0?7oo00<007ooOol057oo10003goo00@007ooOol0015o
o`03001oogoo00Aoo`004goo00<007ooOol0Egoo00<007ooOol0Ngoo0P002goo0P000Woo0`005goo
000COol2001GOol00`00Oomoo`1nOol00`00Oomoo`07Ool00`00Oomoo`02Ool00`00Oomoo`030006
Ool2000<Ool001=oo`03001oogoo05Ioo`03001oogoo04Eoo`D003Eoo`8000=oo`@000Aoo`03001o
ogoo009oo`03001oo`00009oo`<00003Ool007oo00]oo`004goo00<007ooOol0EGoo00<007ooOol0
@Woo10001Goo0P00=Goo0`0027oo00<007ooOol00goo00@007oo000000Eoo`03001oogoo00Uoo`00
4goo00<007ooOol0EGoo00<007ooOol0@Goo0P002Woo0P00?Woo00<007ooOol017oo00<007oo0000
1Woo00<007ooOol027oo000COol00`00Oomoo`1?Ool20003Ool00`00Oomoo`11Ool200000goo001o
o`0:Ool00`00Oomoo`09Ool5000/Ool00`00Oomoo`08Ool00`00Oomoo`03Ool00`00Oomoo`020006
Ool001=oo`80051oo`03001oo`00009oo`03001oogoo041oo`8000=oo`03001oogoo00Uoo`03001o
ogoo00Moo`03001oogoo00=oo`<002Uoo`03001oogoo00Uoo`03001oogoo00=oo`8000Qoo`004goo
00<007ooOol08Woo1@009goo00L007ooOomoo`00Ool0000hOol40005Ool20005Ool00`00Oomoo`09
Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`0WOol00`00Oomoo`0:Ool01000Oomoogoo
0P002Goo000COol00`00Oomoo`0ROol00`00Oomoo`02Ool3000TOol00`00Oomoo`02Ool00`00Oomo
o`0fOol00`00Oomoo`02Ool3000017oo001oo`001Woo00<007ooOol02Goo00<007ooOol00goo00<0
07ooOol027oo00<007ooOol09Goo00<007ooOol037oo00<007oo00000Woo00<007ooOol01Woo000C
Ool00`00Oomoo`0QOol00`00Oomoo`04Ool2000SOol00`00Oomoo`03Ool2000eOol20008Ool00`00
Ool00008Ool00`00Oomoo`09Ool01@00Oomoogoo000037oo00<007ooOol097oo00<007ooOol03Goo
00<007ooOol00Woo00<007ooOol01Goo000COol2000QOol00`00Oomoo`03Ool20002Ool00`00Oomo
o`0OOol00`00Oomoo`03Ool00`00Oomoo`02000@Ool2000POol00`00Oomoo`09Ool00`00Oomoo`08
Ool00`00Oomoo`09Ool00`00Ool0000=Ool00`00Oomoo`0TOol00`00Oomoo`05Ool60007Ool00`00
Oomoo`05Ool001=oo`03001oogoo021oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo00Mo
o`8001Eoo`03001oogoo00=oo`03001oogoo009oo`8000eoo`03001oo`0001moo`03001oogoo00Uo
o`03001oogoo00Yoo`03001oogoo00Uoo`03001oogoo00eoo`03001oogoo029oo`03001oogoo00Eo
o`03001oogoo00Aoo`<000Eoo`03001oogoo00Aoo`004goo00<007ooOol07goo00@007ooOomoo`80
00Ioo`8000Qoo`04001oo`00000BOol00`00Oomoo`04Ool00`00Oomoo`04Ool2000:Ool01@00Oomo
ogoo000077oo0P002goo00<007ooOol02goo00<007ooOol06Woo00<007ooOol07Woo10001Goo0P00
2Goo00<007ooOol02Woo000COol00`00Oomoo`0OOol01000Ool000002Goo0P001Woo00<007ooOol0
0Woo00<007ooOol037oo0P0000=oo`00Ool017oo00<007ooOol01goo0`001Woo00<007ooOol00goo
00<007ooOol067oo00<007ooOol02Woo00<007ooOol03Goo00<007ooOol06Goo00<007ooOol067oo
1P000goo100000=oo`00Ool02goo00<007ooOol02Goo000COol2000OOol3000;Ool00`00Ool00004
Ool00`00Oomoo`04Ool2000<Ool00`00Ool00006Ool00`00Oomoo`0:Ool30002Ool00`00Oomoo`04
Ool00`00Oomoo`0FOol2000<Ool00`00Oomoo`0?Ool00`00Oomoo`0IOol00`00Oomoo`0DOol70004
Ool00`00Oomoo`02Ool00`00Oomoo`0<Ool00`00Oomoo`08Ool001=oo`03001oogoo01]oo`<000mo
o`03001oo`0000@000Qoo`8000Uoo`04001oo`00Ool20003Ool00`00Oomoo`0>Ool20008Ool00`00
Oomoo`0=Ool40002Ool3000<Ool00`00Oomoo`0@Ool00`00Oomoo`0JOol5000=Ool50006Ool01@00
Oomoogoo00004goo00<007ooOol027oo000COol00`00Oomoo`0JOol01000Oomoo`0047oo00@007oo
Oomoo`D000Moo`03001oogoo00Eoo`04001oogoo0003Ool01000Oomoo`006goo00<007ooOol037oo
00@007ooOomoo`<0009oo`<000Qoo`03001oogoo019oo`03001oogoo01ioo`@000Qoo`03001oo`00
00Yoo`04001oogoo000DOol00`00Oomoo`07Ool000Ioo`<000=oo`8000Eoo`03001oogoo01Qoo`80
009oo`03001oogoo009oo`8000]oo`04001oogooOol60007Ool20004Ool01000Oomoo`001Goo00<0
07oo000047oo0P002Goo00<007ooOol02goo00<007ooOol00Woo00<007ooOol01Goo20005Goo00<0
07ooOol01Goo0P006goo0P0017oo0P000Woo00<007ooOol02Goo0P005Goo00<007ooOol01goo0009
Ool00`00Ool00002Ool00`00Oomoo`02Ool2000HOol00`00Oomoo`02Ool00`00Ool0000200000goo
001oo`0;Ool30005Ool20008Ool01@00Oomoogoo00000Woo00<007ooOol017oo1`0027oo0`000Woo
00<007ooOol01goo00<007ooOol02Goo00<007ooOol00Woo00<007oo00000P008Woo00<007ooOol0
0goo00@007ooOol001aoo`800003Ool007oo009oo`03001oogoo00]oo`03001oogoo01Aoo`03001o
ogoo00Ioo`001goo0`0000Eoo`00Oomoo`0000Aoo`03001oogoo01Ioo`03001oogoo009oo`<000Eo
o`03001oogoo00Uoo`8000Moo`8000Qoo`03001oo`00009oo`03001oogoo00Eoo`03001oogoo00=o
o`03001oogoo00Aoo`8000Ioo`03001oogoo00Moo`8000Qoo`03001oogoo009oo`<000=oo`03001o
ogoo021oo`03001oogoo008000Aoo`03001oogoo01]oo`05001oogooOol0000NOol50003Ool00`00
Oomoo`05Ool000Ioo`06001oogoo001oo`000Woo00<007ooOol00Woo00<007ooOol05Goo00<007oo
Ool00Woo0P0027oo00<007ooOol01goo00@007oo0000009oo`8000=oo`8000Moo`80009oo`03001o
ogoo00Eoo`03001oogoo00Eoo`04001oogooOol20009Ool00`00Oomoo`08Ool30004Ool01000Oomo
ogoo0P001goo00<007ooOol07goo00<007oo00001goo00<007ooOol07Goo00<007ooOol067oo1@00
1Goo0P0000=oo`0000001Woo0006Ool01P00Oomoo`00Ool0009oo`03001oogoo009oo`03001oogoo
00Uoo`T0009oo`03001oogoo009oo`8000Yoo`03001oogoo00P0009oo`800003Ool007oo00=oo`80
00Eoo`D000Moo`03001oogoo00Ioo`04001oogoo000<Ool00`00Oomoo`0:Ool200000goo001oo`02
Ool20009Ool00`00Oomoo`0OOol00`00Oomoo`07Ool00`00Oomoo`0LOol00`00Oomoo`0EOol3000<
Ool30006Ool000Moo`8000=oo`8000Eoo`03001oogoo00Uoo`03001oogoo00Ioo`8000Aoo`03001o
o`0000]oo`800005Ool007ooOol00005Ool20002Ool00`00Oomoo`02Ool60003Ool40004Ool00`00
Oomoo`07Ool00`00Ool0000>Ool00`00Oomoo`04Ool50002Ool00`00Ool00003000;Ool00`00Oomo
o`02Ool5000ROol00`00Oomoo`0JOol00`00Oomoo`0AOol5000@Ool20006Ool001=oo`8000Yoo`03
001oogoo00Ioo`08001oo`00001oo`00Ool000]oo`<0009oo`8000Eoo`04001oogoo0002Ool00`00
Ool00008000017oo001oo`000Woo2@001Woo00<007ooOol03Woo0`000goo00<007ooOol00Woo1000
0goo00<007ooOol017oo2P001Goo00<007ooOol087oo00<007ooOol067oo00<007ooOol03goo0`00
57oo00<007ooOol01Woo000COol00`00Oomoo`08Ool00`00Oomoo`06Ool00`00Oomoo`02Ool01000
Oomoo`002Goo0`000Woo00@007oo000000Aoo`03001oogoo009oo`H000Eoo`<000=oo`05001oogoo
Ool00006Ool20003Ool2000@Ool00`00Oomoo`030002Ool30003Ool01000Oomoo`001Woo00<007oo
Ool01Goo00<007ooOol01Woo00<007ooOol087oo00<007ooOol03Goo2P004Goo00<007ooOol057oo
00<007ooOol01goo000COol00`00Oomoo`08Ool90008Ool00`00Oomoo`030002Ool200000goo001o
o`03Ool70006Ool30009Ool4000017oo000000002Goo0`0000=oo`00Ool03goo00H007ooOomoo`00
Ool20007Ool00`00Ool00005Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`0POol00`00
Oomoo`07Ool5000JOol00`00Oomoo`0EOol00`00Oomoo`07Ool001=oo`03001oogoo00Qoo`8000io
o`05001oogooOol00002Ool300000goo001oo`04Ool00`00Ool0000200000goo001oo`03Ool6000;
Ool400000goo001oo`09Ool01000Oomoo`003Woo00D007oo001oo`0000aoo`03001oogoo00=oo`03
001oogoo00Uoo`03001oogoo00Ioo`03001oogoo021oo`03001oogoo00=oo`<001ioo`03001oogoo
01Eoo`03001oogoo00Qoo`004goo0P0027oo0P003goo00D007ooOol007oo008000=oo`03001oogoo
009oo`@0009oo`05001oogoo001oo`040003Ool20008Ool20003Ool01000Oomoo`0027oo0P000Woo
00<007ooOol02goo00<007ooOol00P0037oo00<007oo00000goo00<007ooOol02goo00<007ooOol0
1Woo0P008Goo00<007ooOol00P0087oo00D007ooOomoo`00019oo`8000]oo`004goo00<007ooOol0
1goo0P003Woo00<007ooOol00`001goo10001Woo00<007oo00000`001Woo00<007ooOol017oo0P00
1Woo00@007ooOol000Moo`05001oo`00Ool0000<Ool01000Oomoogoo0P0037oo00H007ooOol007oo
000?Ool00`00Oomoo`07Ool00`00Oomoo`0OOol00`00Ool0000POol00`00Oomoo`02Ool00`00Oomo
o`0@Ool00`00Ool0000:Ool001=oo`03001oogoo00Moo`8000ioo`04001oo`000006Ool30002Ool0
0`00Oomoo`040002Ool30009Ool00`00Oomoo`030008Ool01@00Oomoogoo00001Goo00L007ooOomo
o`00Ool0000;Ool01P00Ool00000Ool000]oo`03001oogoo009oo`03001oogoo00moo`04001oogoo
Ool8000POol00`00Oomoo`0NOol00`00Oomoo`02Ool00`00Ool0000@Ool20002Ool00`00Oomoo`07
Ool001=oo`03001oogoo00Ioo`8000ioo`04001oo`000006Ool00`00Oomoo`02Ool01@00Oomoogoo
00000goo0`000Woo00<007ooOol01goo0P0037oo00D007ooOomoo`0000Aoo`03001oogoo009oo`80
00Yoo`<000Aoo`03001oogoo00Qoo`03001oogoo01Eoo`<000Ioo`<001moo`@001eoo`05001oogoo
Ool00003Ool00`00Oomoo`0<Ool20004Ool00`00Oomoo`06Ool001=oo`8000Moo`8000Eoo`L00003
Ool007oo008000Ioo`03001oogoo009oo`05001oogooOol00009Ool00`00Oomoo`09Ool01000Oomo
ogoo1`0000=oo`00Ool00goo0P000Woo100000=oo`0000002Goo0P000Woo0P000goo00<007ooOol0
1Woo00<007ooOol08Woo1@003Goo0`002Woo0P000Woo0`0017oo10004Goo00<007ooOol00Woo00D0
07ooOomoo`0000eoo`03001oo`0000Eoo`03001oogoo00Eoo`004goo00<007ooOol01Woo00<007oo
Ool01P000goo1P001Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol027oo00<007ooOol0
2Goo00<007oo00002P001Goo0`000goo00<007oo00002@0000Aoo`000000009oo`04001oogoo0007
Ool00`00Oomoo`0UOol2000017oo000000002Goo00@007ooOomoo`8000Moo`03001oo`0000Eoo`@0
00Aoo`03001oogoo00eoo`03001oogoo009oo`03001oogoo00=oo`03001oogoo00Uoo`03001oo`00
00Moo`03001oogoo00Aoo`004goo00<007ooOol01Goo0P000Woo00@007ooOol0009oo`80009oo`@0
00Ioo`03001oogoo009oo`03001oogoo009oo`03001oogoo00Yoo`03001oogoo00Moo`@000Qoo`@0
009oo`H0009oo`<000=oo`H000Ioo`04001oogoo0005Ool00`00Oomoo`0AOol00`00Oomoo`0DOol0
1000Oomoo`001goo1@000goo00<007ooOol00Woo0`000Woo00<007ooOol02goo00<007ooOol037oo
00<007ooOol00Woo00<007ooOol017oo00<007ooOol027oo00<007oo00003Woo000COol00`00Oomo
o`05Ool200000goo001oo`020002Ool01000Ool007oo4P000goo00<007ooOol02Woo00<007ooOol0
17oo0`000Woo0P002Goo0P0000=oo`000000200000=oo`00000010000goo200000Aoo`00Ool000Ao
o`03001oogoo00Ioo`8000Aoo`<000=oo`03001oogoo00moo`8000Aoo`800003Ool007oo00H000Ao
o`04001oogooOol2000017oo001oo`000goo00<007ooOol037oo00<007ooOol01Woo0P000Woo00<0
07ooOol00Woo00<007ooOol01Goo00<007ooOol01goo00<007oo00003goo000COol20006Ool00`00
Ool00002000017oo000000000goo0P0000=oo`00Ool00Woo0`0000Aoo`00Ool0009oo`06001oogoo
001oo`003Woo0P0000Aoo`00000000Aoo`03001oogoo008000Qoo`8000=oo`H000=oo`8000Ioo`03
001oogoo00Eoo`04001oo`00Ool200000goo001oo`08Ool01000Ool000000Woo00<007ooOol00`00
00Aoo`00Oomoo`D000Qoo`03001oo`0000Eoo`@000=oo`03001oogoo00Eoo`03001oogoo009oo`04
001oogoo0004Ool00`00Oomoo`0<Ool00`00Oomoo`05Ool00`00Ool000020003Ool00`00Oomoo`07
Ool00`00Oomoo`05Ool01000Oomoo`003goo000COol00`00Oomoo`04Ool>00000goo001oo`02Ool5
0003Ool500000goo0000000>Ool40006Ool00`00Oomoo`02Ool00`00Oomoo`04Ool00`00Ool00002
Ool20002Ool00`00Oomoo`020002Ool20004Ool00`00Oomoo`03Ool70002Ool30007Ool00`00Oomo
o`02Ool20004Ool00`00Ool000030005Ool00`00Oomoo`03Ool20003Ool00`00Ool00002000017oo
001oo`000goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol00Woo00<007ooOol03Goo00<0
07ooOol00goo00@007ooOol0009oo`800003Ool007oo00Uoo`03001oogoo009oo`80009oo`03001o
ogoo00ioo`004goo00<007ooOol017oo00<007oo00001`0017oo00<007ooOol00P000goo0P000Woo
0`000goo0`003Goo0`000Woo00<007ooOol00Woo00<007ooOol017oo0P000Woo0P000goo00<007oo
00000goo00D007ooOol007oo0080009oo`@000Aoo`8000Aoo`H0009oo`8000Eoo`03001oogoo009o
o`03001oo`0000P000Moo`05001oogooOol00005Ool20005Ool20002Ool00`00Oomoo`08Ool00`00
Oomoo`02Ool00`00Oomoo`04Ool01000Oomoo`0037oo00<007ooOol00Woo00@007ooOol000Aoo`03
001oogoo00Uoo`05001oogooOol00004Ool00`00Oomoo`07Ool00`00Oomoo`04Ool00004Ool00000
0002Ool20004Ool20005Ool00`00Oomoo`04Ool60002Ool900000goo00000002Ool00`00Ool00002
0002Ool01000Oomoogoo0`002goo00D007ooOomoo`00009oo`800003Ool007oo00Qoo`8000Eoo`<0
009oo`03001oogoo009oo`03001oogoo00L00003Ool0000000Uoo`03001oogoo00<00003Ool00000
009oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo009oo`03001oogoo00Ioo`03001oo`00
00Aoo`80009oo`03001oogoo00=oo`800003Ool007oo00Uoo`H000Ioo`04001oo`00000<Ool00`00
Oomoo`02Ool00`00Ool00002Ool;0005Ool00`00Ool00004Ool00`00Oomoo`08Ool00`00Oomoo`04
Ool0009oo`04001oogoo0002Ool01000Oomoo`000Woo00<007ooOol00Woo00<007ooOol00goo2`00
00=oo`0000000`000Woo00@007oo000000=oo`03001oogoo009oo`05001oogoo001oo`020008Ool2
0005Ool01@00Oomoogoo000027oo0P0000=oo`0000000`000Woo00D007ooOomoo`0000Aoo`8000Ao
o`03001oo`0000D000Qoo`03001oogoo00=oo`@00003Ool007oo00=oo`03001oogoo009oo`03001o
ogoo00Aoo`800004Ool000000003Ool20002Ool40003Ool00`00Oomoo`02Ool01000Ool000001Goo
1@000Woo00@007oo000000Moo`04001oogoo000;Ool01000Oomoogoo1P0000Aoo`00Ool000Moo`80
00=oo`8000Eoo`03001oogoo00Moo`03001oogoo00Eoo`000Woo00@007ooOol0009oo`04001oogoo
0002Ool00`00Oomoo`02Ool20004Ool40007Ool00`00Ool00002Ool4000017oo000007oo0P001Goo
00<007oo000017oo0P001Goo0P001goo00<007ooOol01goo0P0027oo0`000goo00<007ooOol00goo
00<007ooOol00Woo00<007ooOol01Goo0`001Woo00<007ooOol01Goo0P0000=oo`0000000`0017oo
00<007ooOol01Goo0P000goo00<007ooOol01@000Woo00D007ooOomoo`0000Eoo`8000Eoo`03001o
ogoo00Ioo`@000Eoo`8000=oo`03001oogoo00Uoo`@00004Ool007ooOol20003Ool00`00Oomoo`06
Ool200000goo00000004Ool00`00Oomoo`07Ool00`00Oomoo`06Ool0009oo`04001oogoo0002Ool0
1000Oomoo`000Woo00<007ooOol00Woo00<007ooOol00goo00<007oo00001@000Woo00<007oo0000
0P000Woo00L007oo001oo`00Ool00008Ool20006Ool00`00Oomoo`030008Ool00`00Oomoo`05Ool2
000<Ool01000Oomoo`001goo00@007ooOol000Yoo`03001oogoo00Eoo`03001oogoo00Aoo`<000=o
o`@00003Ool007oo00Ioo`800003Ool007oo009oo`8000=oo`03001oogoo009oo`05001oogooOol0
0004Ool30003Ool00`00Oomoo`08Ool00`00Ool000020002Ool01000Oomoo`000Woo00<007ooOol0
1goo0`000Woo0P000Woo00<007ooOol017oo00<007ooOol01goo00<007oo000017oo00<007ooOol0
1goo00<007ooOol01Woo0002Ool01000Oomoo`000Woo00@007ooOol0009oo`03001oogoo009oo`03
001oogoo00=oo`800003Ool0000000H00004Ool00000Ool400001Woo00000000Ool000Moo`03001o
ogoo00Ioo`@000Yoo`03001oogoo009oo`8000moo`03001oo`0000Moo`H000Uoo`03001oogoo00Ao
o`03001oogoo00=oo`80009oo`03001oo`0000Aoo`03001oogoo00Aoo`80009oo`03001oo`0000@0
009oo`03001oogoo00=oo`04001oogooOol500000goo00000002Ool00`00Oomoo`08Ool00`00Oomo
o`02Ool80002Ool00`00Oomoo`05Ool02P00Oomoo`00Ool007oo001oo`001goo00<007ooOol02Woo
00@007oo000000Uoo`04001oogoo0006Ool00003Ool0000000=oo`8000Aoo`8000Eoo`03001oogoo
00Aoo`800003Ool007oo00<0009oo`800003Ool007oo00H000=oo`03001oogoo00Aoo`03001oogoo
00Eoo`@000aoo`@0015oo`03001oo`0000Qoo`03001oogoo009oo`@000Ioo`03001oogoo00Aoo`04
001oogooOol20004Ool40007Ool20004Ool00`00Oomoo`02Ool40007Ool30002Ool200000goo0000
0003000:Ool00`00Oomoo`04Ool00`00Ool0000200000goo00000002Ool00`00Oomoo`03Ool00`00
Oomoo`02Ool300000goo001oo`08Ool00`00Oomoo`0:Ool00`00Oomoo`09Ool01000Ool000001Woo
000COol20005Ool02000Ool007oo001oo`00Ool300000goo001oo`0200001Goo000007oo00000goo
00<007ooOol00Woo0P0000=oo`00Ool017oo0P000Woo0P0037oo00<007oo00004Goo0P004Woo0P00
17oo00<007ooOol01Goo00D007oo001oo`0000=oo`80009oo`P000Moo`04001oogoo0003Ool00`00
Oomoo`02Ool40006Ool200000goo0000000:Ool00`00Oomoo`04Ool30005Ool40004Ool00`00Oomo
o`03Ool300000goo001oo`09Ool00`00Oomoo`07Ool3000:Ool01000Ool000001goo000COol00`00
Oomoo`04Ool30002Ool2000017oo000007oo1@000Woo0P0000=oo`00Ool00`000Woo00D007ooOomo
o`0000=oo`8000Eoo`8000Yoo`04001oogooOol2000@Ool00`00Oomoo`0BOol20003Ool;0003Ool0
0`00Oomoo`05Ool20002Ool40004Ool00`00Ool00004Ool00`00Oomoo`020004Ool00`00Oomoo`04
Ool200000goo00000008Ool00`00Oomoo`04Ool00`00Oomoo`07Ool00`00Oomoo`0200000goo001o
o`05Ool200000goo001oo`0:Ool00`00Ool000030003Ool01000Oomoogoo0`001Woo00@007oo0000
00Qoo`004goo00<007ooOol01Goo00<007oo00000goo00D007ooOol007oo00<00003Ool007oo009o
o`<000=oo`8000Eoo`<000Qoo`8000Qoo`03001oogoo00Aoo`03001oogoo00eoo`03001oogoo01Ao
o`@0009oo`80009oo`8000=oo`80011oo`<0009oo`03001oogoo00Aoo`8000=oo`05001oogooOol0
0004Ool00`00Oomoo`040006Ool00`00Oomoo`04Ool00`00Oomoo`09Ool00`00Ool000040005Ool3
000=Ool20003Ool30002Ool4000017oo000000000goo0`002Goo000COol00`00Oomoo`05Ool20000
1Goo001oo`00Ool00P0000Uoo`00Ool007oo001oo`00Ool010000goo100000=oo`0000000P002Woo
0P001Woo00<007ooOol01Woo0P0037oo0P006Woo0P000Woo0P000Woo00<007oo00005Goo0P000goo
1@0017oo0P000Woo00<007ooOol00Woo00@007ooOomoo`D000=oo`03001oogoo00Aoo`03001oogoo
00Uoo`<00003Ool007oo009oo`<0009oo`<000Qoo`8000=oo`8000Ioo`80009oo`X00003Ool007oo
00Qoo`004goo0P001Woo0P000Woo0P000goo00H007oo001oo`00Ool50002Ool01@00Ool007oo0000
0Woo00D007oo001oo`0000]oo`8000Aoo`03001oogoo00Uoo`03001oogoo00Uoo`03001oo`0000Qo
o`03001oogoo00ioo`80009oo`8000=oo`8001Moo`<000Eoo`04001oogoo0002Ool01000Oomoo`00
0goo00@007ooOol0009oo`@00003Ool007oo00Eoo`03001oogoo00Qoo`80009oo`800003Ool007oo
00Aoo`@000Uoo`05001oo`00001oo`020005Ool01@00Oomoogoo00000Woo00<007ooOol00goo00<0
07oo00002Woo000COol00`00Oomoo`06Ool00`00Oomoo`02Ool300000goo001oo`02Ool30003Ool4
0003Ool01000Oomoo`000Woo00<007ooOol02Goo00<007oo00000goo00<007ooOol00P0027oo00<0
07ooOol027oo00<007oo000027oo00<007ooOol02Goo100000Eoo`00001oo`00009oo`06001oogoo
001oo`007Woo00@007ooOol000=oo`03001oo`00009oo`04001oogoo0004Ool200000goo001oo`05
Ool00`00Oomoo`02Ool30003Ool00`00Oomoo`03Ool30005Ool50007Ool00`00Oomoo`02Ool20000
0goo001oo`03Ool01000Oomoo`000goo00<007ooOol017oo00<007ooOol02Goo000COol00`00Oomo
o`06Ool20006Ool700000goo001oo`04Ool70003Ool00`00Oomoo`0:Ool00`00Ool0000500000goo
001oo`08Ool:0002Ool00`00Oomoo`05Ool00`00Ool00007Ool30004Ool40004Ool20003Ool00`00
Oomoo`0LOol20005Ool01`00Ool007oo001oo`0000Moo`8000Ioo`03001oogoo008000=oo`<000Mo
o`<000Eoo`800003Ool007oo00Eoo`03001oogoo00=oo`04001oo`000003Ool200000goo001oo`04
Ool00`00Oomoo`02Ool00`00Oomoo`0:Ool001=oo`03001oogoo00Ioo`8000Qoo`04001oogooOol4
0006Ool01@00Oomoogoo000017oo00<007ooOol02Woo00<007oo00000P0017oo00<007ooOol01Woo
0P001goo00D007ooOomoo`0000Eoo`03001oo`0000P000Ioo`04001oogooOol70004Ool00`00Oomo
o`0KOol20006Ool50009Ool500000goo001oo`020006Ool30007Ool40002Ool300000goo001oo`04
Ool00`00Oomoo`03Ool00`00Oomoo`020002Ool40006Ool00`00Oomoo`02Ool00`00Oomoo`0:Ool0
01=oo`8000Qoo`8000eoo`<000Ioo`04001oogoo0004Ool00`00Oomoo`0;Ool01@00Oomoogoo0000
17oo00<007ooOol00goo0P000Woo0P001Goo00D007ooOomoo`0000Eoo`05001oo`00Ool0000<Ool0
0`00Oomoo`06Ool40003Ool00`00Oomoo`0JOol00`00Oomoo`05Ool40009Ool?00000goo001oo`02
Ool30006Ool00`00Ool0000300000goo001oo`0<0003Ool3000017oo0000000017oo1@0000=oo`00
Ool037oo000COol00`00Oomoo`07Ool2000=Ool01000Ool000001Woo00@007ooOol000Aoo`03001o
ogoo00Yoo`03001oogoo009oo`80009oo`03001oogoo009oo`03001oogoo00Aoo`07001oogooOol0
07oo00000Woo00<007ooOol00Woo00<007oo00000goo00<007ooOol027oo00<007ooOol01Woo00H0
07ooOol007oo0003Ool00`00Oomoo`0>Ool50006Ool20005Ool00`00Ool000020007Ool200000goo
0000000300001Goo00000000Ool01@001goo0`000goo0`0000=oo`0000000Woo0`0000=oo`00Ool0
0goo00<007ooOol00`0000Ioo`00001oo`00Ool20004Ool01@00Oomoo`00Ool01P002Goo000COol0
0`00Oomoo`07Ool2000>Ool200000goo001oo`05Ool01000Oomoo`000goo00<007ooOol02Goo00<0
07ooOol01Goo0P0000=oo`00Ool00P0027oo0P0000Aoo`00000000=oo`03001oogoo018000Qoo`03
001oogoo009oo`04001oo`00Ool5000017oo000000001Woo0`000Woo00<007ooOol00`000goo0P00
17oo00D007ooOol007oo008000Aoo`8000=oo`03001oogoo008000Aoo`04001oogooOol2000:Ool7
0002Ool2000017oo000000001Woo0P000goo00@007oo001oo`80009oo`04001oogoo0003Ool01000
Oomoo`0017oo0`001Woo000COol00`00Oomoo`08Ool60009Ool01@00Ool007oo00001Woo0`000goo
00<007ooOol02Goo00<007ooOol01goo0`0037oo00@007ooOol000=oo`@00003Ool007oo00=oo`<0
00=oo`03001oogoo00Uoo`03001oogoo00=oo`04001oogoo0002Ool=0004Ool00`00Ool00004Ool3
0002Ool01000Oomoo`000Woo00L007ooOol007oo001oo`020004Ool01000Oomoogoo0P001Goo0P00
0goo00<007ooOol01Goo2@0000Eoo`00Ool007oo008000Moo`8000Eoo`<000=oo`03001oo`0000Eo
o`8000ioo`004goo00<007ooOol027oo00<007ooOol00goo0`001Woo00D007ooOol007oo008000Io
o`8000=oo`P0009oo`03001oogoo00Qoo`<000]oo`<0009oo`08001oo`00001oo`00Ool000=oo`80
009oo`03001oogoo009oo`03001oogoo00Eoo`800003Ool007oo00Eoo`800003Ool0000000@00004
Ool007ooOol20007Ool01000Oomoo`001Goo00H007ooOol007oo0003Ool01000Oomoogoo0`001Goo
00D007ooOomoo`00009oo`03001oogoo00Aoo`04001oogooOol70003Ool01000Oomoogoo0`000goo
1`000goo1P000Woo0P0017oo0P001Goo0P003Woo000COol20009Ool00`00Oomoo`05Ool00`00Oomo
o`05Ool00`00Ool00003Ool00`00Oomoo`05Ool00`00Oomoo`08Ool00`00Ool00007Ool30003Ool0
0`00Oomoo`07Ool20002Ool40002Ool00`00Ool00002Ool30004Ool00`00Oomoo`02Ool00`00Oomo
o`02Ool200000goo00000006Ool200000goo001oo`030004Ool01@00Oomoogoo00001Goo00D007oo
Oomoo`0000Aoo`03001oo`00009oo`05001oogooOol00003Ool200000goo001oo`03Ool00`00Ool0
00020004Ool00`00Oomoo`04Ool300000goo001oo`07Ool01000Oomoogoo0`000goo0P0000=oo`00
Ool00Woo1@0017oo00@007oo000000Aoo`8000Eoo`8000ioo`004goo00<007ooOol02Goo00<007oo
Ool01Goo00@007ooOomoo`800005Ool007ooOol00003Ool40004Ool20006Ool200000goo00000006
Ool01000Ool007oo0P0000=oo`0000000`0017oo00<007oo00002`0000Eoo`00Ool007oo008000=o
o`@00003Ool007oo00L0009oo`@00003Ool0000000Aoo`03001oogoo009oo`03001oogoo009oo`80
00=oo`03001oogoo009oo`05001oogooOol00002Ool00`00Oomoo`02Ool01`00Oomoogoo001oo`00
009oo`03001oogoo00D000Ioo`03001oogoo00Qoo`03001oogoo00Aoo`06001oogooOol007oo0P00
0Woo0P0000=oo`0000001P0000=oo`00Ool017oo0`0017oo0P001Goo0P003Woo000017oo00000000
0Woo0`000goo0P001Goo00<007ooOol02Goo0`001Woo0`0000Aoo`00Ool000=oo`<0009oo`8000Ao
o`04001oo`000004Ool00`00Ool00002Ool00`00Oomoo`02Ool01000Oomoo`000goo30000Woo0P00
0Woo00@007oo001oo`<0009oo`<00003Ool007oo00Eoo`D000=oo`@000Aoo`03001oo`0000Eoo`03
001oogoo009oo`03001oogoo00=oo`04001oo`00Ool80004Ool01@00Oomoogoo00000goo00@007oo
Ool0009oo`D0009oo`03001oogoo00=oo`03001oogoo00Uoo`8000=oo`04001oogooOol300001Woo
001oo`00Ool000Aoo`03001oogoo00Aoo`03001oogoo009oo`@0009oo`04001oogoo0004Ool2000>
Ool0009oo`03001oogoo009oo`05001oogooOol00002Ool00`00Oomoo`02Ool00`00Oomoo`0<Ool3
0002Ool30003Ool00`00Ool00008Ool20004Ool200001Goo001oogoo00000Woo0`0000Eoo`00001o
o`00009oo`03001oogoo00Aoo`80009oo`<0009oo`03001oo`0000Aoo`800003Ool007oo00D0009o
o`@000]oo`06001oogooOol007oo0`000Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol0
17oo0`001Goo00<007oo00001Goo0`000goo00D007ooOomoo`00009oo`8000Ioo`03001oogoo00=o
o`03001oogoo00Yoo`<0009oo`<000=oo`80009oo`03001oogoo00=oo`03001oogoo00=oo`04001o
ogooOol200001Goo000007oo00000goo00@007ooOomoo`P000Uoo`000Woo00<007ooOol00Woo00D0
07ooOomoo`00009oo`03001oogoo009oo`80011oo`80009oo`03001oogoo009oo`8000Yoo`04001o
ogooOol60002Ool01@00Oomoogoo00000Woo00@007ooOol000Uoo`03001oogoo00Aoo`8000Ioo`80
0003Ool007oo009oo`03001oogoo00Aoo`<000Qoo`04001oogooOol30006Ool00`00Oomoo`03Ool0
0`00Oomoo`02Ool00`00Oomoo`03Ool30003Ool2000017oo001oo`001Woo100017oo00@007ooOol0
00Qoo`03001oogoo00=oo`03001oogoo00Yoo`07001oo`00001oo`00Ool01@000Woo00<007ooOol0
0goo00<007ooOol017oo00<007ooOol00P000Woo0`0017oo10000Woo00<007ooOol00Woo0`001Woo
0002Ool00`00Oomoo`02Ool01@00Oomoogoo00000Woo00<007ooOol00Woo00<007ooOol057oo00<0
07ooOol00Woo0P002Woo00<007oo00000goo1P0017oo0P0000Aoo`00Ool000Yoo`03001oogoo009o
o`8000Moo`03001oo`0000=oo`D000Ioo`03001oogoo00Aoo`03001oogoo008000=oo`03001oogoo
00=oo`03001oogoo00=oo`03001oogoo00=oo`05001oogooOol00003Ool00`00Ool00003Ool40004
Ool00`00Oomoo`030003Ool40009Ool00`00Oomoo`03Ool00`00Oomoo`09Ool70004Ool30007Ool0
0`00Oomoo`03Ool40003Ool20008Ool01000Oomoo`003Goo0002Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000Woo00<007ooOol00Woo00<007ooOol05Goo00<007ooOol00Woo00<007ooOol02Goo0P00
0goo0P000Woo00<007ooOol00Woo0P0000Aoo`00Ool000aoo`05001oogooOol00008Ool00`00Oomo
o`02Ool20004Ool30004Ool20003Ool01@00Oomoogoo00001Goo00<007ooOol00goo00<007ooOol0
0goo1P000Woo0P0017oo00D007ooOomoo`0000=oo`@00004Ool007oo0003Ool300001Goo000007oo
00002Goo00<007ooOol00goo00<007ooOol01Goo1@000Woo0P001Goo0`0027oo00<007ooOol01000
00=oo`00Ool00goo0P001Woo00<007ooOol00Woo00<007ooOol02Woo00000goo00000003Ool20004
Ool20005Ool00`00Oomoo`0FOol01000Oomoogoo0P002Woo0P000goo0P000goo00@007ooOol000Ao
o`03001oo`0000aoo`03001oo`0000Yoo`04001oogooOol20007Ool20004Ool00`00Ool00003Ool0
0`00Oomoo`05Ool00`00Oomoo`02Ool00`00Oomoo`040003Ool60006Ool20005Ool200001Goo001o
ogoo00001goo00D007ooOomoo`0000Uoo`04001oogooOol70002Ool200000goo001oo`02Ool00`00
Ool00005Ool20009Ool20003Ool00`00Ool00006Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00
Oomoo`0:Ool001=oo`8001Moo`03001oogoo009oo`<000Moo`06001oogoo001oo`000Woo00@007oo
Oomoo`8000Eoo`800003Ool007oo00]oo`8000Yoo`06001oogoo001oo`0027oo00<007ooOol00goo
00<007ooOol00Woo00<007ooOol01Goo00<007ooOol00goo0P0017oo0`000Woo00@007oo001oo`<0
009oo`03001oogoo00Ioo`80009oo`03001oogoo00Eoo`<000=oo`03001oogoo00Moo`<000=oo`<0
0006Ool00000001oo`000goo00D007ooOomoo`0000Aoo`03001oo`0000eoo`8000Qoo`800003Ool0
000000<000=oo`03001oogoo00Uoo`004goo00<007ooOol05goo00D007ooOomoo`00009oo`03001o
ogoo00=oo`03001oogoo009oo`<00003Ool007oo009oo`8000Aoo`03001oogoo00@000=oo`P000]o
o`03001oo`00009oo`03001oogoo00Moo`03001oogoo009oo`05001oogooOol00009Ool00`00Oomo
o`0>Ool00`00Oomoo`03Ool2000:Ool30008Ool00`00Ool00004Ool00`00Oomoo`06Ool00`00Oomo
o`02Ool20002Ool300000goo001oo`02Ool00`00Oomoo`03Ool01000Oomoogoo0`003Woo10001Goo
00<007oo00000P000Woo100000=oo`00Ool02Goo000COol00`00Oomoo`0HOol01000Oomoo`000goo
0P000goo00<007ooOol01Goo0`0000Eoo`00Oomoo`0000=oo`03001oogoo00T000Eoo`03001oogoo
00Uoo`80009oo`03001oogoo00Qoo`03001oogoo00=oo`04001oogoo0009Ool00`00Oomoo`0QOol0
1000Ool000000Woo1@0000=oo`00Ool017oo00<007ooOol017oo00<007oo00000Woo00<007oo0000
0goo00<007oo000017oo00<007ooOol017oo00<007ooOol00P0000=oo`00Ool02goo00<007ooOol0
0Woo10000Woo1P000goo00<007oo00002Woo000COol00`00Oomoo`0IOol01000Oomoo`0017oo00<0
07oo00002goo00<007ooOol00Woo00<007oo00000Woo00<007oo00000P0000=oo`0000000P0017oo
00<007oo000027oo0P0000Eoo`00Oomoo`0000]oo`03001oogoo009oo`03001oo`0000]oo`03001o
ogoo02Aoo`80009oo`<0009oo`03001oogoo00=oo`8000=oo`06001oogoo001oo`000goo00@007oo
Ool000Eoo`03001oogoo00Ioo`@00003Ool007oo00Yoo`03001oogoo00Moo`06001oogoo001oo`00
1Woo00<007oo00002Goo000COol2000KOol00`00Ool00005Ool00`00Oomoo`0:Ool00`00Oomoo`03
Ool01P00Oomoo`00Ool000Ioo`<00003Ool0000000H000=oo`8000Aoo`03001oo`0000]oo`03001o
ogoo00=oo`8000]oo`05001oogooOol0000QOol01@00Ool00000Ool00P0000=oo`0000000P001Woo
0`000goo0P001Goo00<007oo000017oo00<007ooOol027oo100000=oo`00Ool02Goo00<007ooOol0
27oo0P0000=oo`00Ool01Woo0`002Goo000COol00`00Oomoo`0KOol00`00Ool0000AOol30005Ool0
0`00Ool00008Ool40004Ool00`00Oomoo`030007Ool2000<Ool00`00Oomoo`02Ool2000;Ool01@00
Oomoogoo00004Woo0P002goo0P0017oo00<007oo00000goo0`001Woo00D007oo000007oo008000Io
o`03001oogoo00=oo`03001oogoo00Qoo`04001oo`000002Ool00`00Oomoo`06Ool00`00Oomoo`0:
Ool00`00Oomoo`08Ool30008Ool001=oo`03001oogoo01aoo`80015oo`800008Ool00000001oo`00
Ool000Yoo`<000Eoo`03001oogoo00Uoo`8000eoo`04001oogooOol2000<Ool01@00Ool007oo0000
47oo00<007ooOol00P001goo0P001goo00<007ooOol017oo0P0017oo00<007ooOol00Woo00<007oo
00001Woo00<007ooOol00Woo00<007ooOol02Goo00<007ooOol00P000Woo00<007ooOol017oo00<0
07ooOol02Woo00<007oo00002Goo00@007oo000000Moo`004goo00<007ooOol07Goo0`003Woo00@0
07ooOol000=oo`03001oo`0000]oo`<000Ioo`03001oogoo00Qoo`03001oo`0000aoo`04001oogoo
Ool60008Ool20003Ool00`00Oomoo`0<Ool00`00Oomoo`03Ool20002Ool90003Ool00`00Oomoo`06
Ool200000goo001oo`02Ool50005Ool00`00Ool00003Ool00`00Oomoo`08Ool01000Oomoogoo1P00
1Goo00<007ooOol02Woo00@007ooOol000Uoo`03001oo`0000Moo`004goo00<007ooOol07Woo00@0
07oo000000aoo`04001oogoo0004Ool00`00Oomoo`0:Ool200000goo001oo`04Ool00`00Oomoo`08
Ool01000Oomoo`0037oo100000=oo`00Ool00Woo0P001Woo0P0017oo00<007ooOol02Woo00<007oo
Ool01Woo0P000Woo00<007ooOol017oo0`002Woo0P0000Aoo`00000000Aoo`8000Aoo`06001oogoo
001oo`002goo00@007ooOomoo`8000=oo`<0009oo`03001oogoo00Yoo`05001oogooOol00009Ool0
1000Oomoo`001Woo000COol2000OOol01@00Oomoogoo00002Woo00<007oo00001P002goo0P0000Ao
o`00Ool000Ioo`03001oogoo00Ioo`03001oogoo009oo`8000Yoo`<000=oo`03001oogoo009oo`03
001oogoo009oo`03001oo`0000Aoo`03001oogoo00Uoo`03001oogoo00Uoo`8001Eoo`8000Qoo`05
001oogooOol00004Ool00`00Oomoo`09Ool00`00Oomoo`03Ool20004Ool3000=Ool00`00Oomoo`02
Ool00`00Oomoo`06Ool00`00Oomoo`07Ool001=oo`03001oogoo01moo`05001oogooOol00009Ool0
0`00Ool00002Ool00`00Oomoo`0:Ool20004Ool00`00Ool00005Ool30006Ool00`00Oomoo`04Ool0
0`00Oomoo`07Ool20004Ool00`00Oomoo`03Ool200000goo001oo`02Ool00`00Oomoo`02Ool00`00
Oomoo`07Ool00`00Oomoo`09Ool00`00Oomoo`0OOol20002Ool00`00Oomoo`0=Ool00`00Oomoo`04
Ool20003Ool4000:Ool30005Ool00`00Oomoo`07Ool00`00Oomoo`06Ool001=oo`03001oogoo01mo
o`03001oogoo009oo`03001oogoo00Eoo`03001oo`0000=oo`03001oogoo00Qoo`8000Moo`03001o
o`0000Aoo`<000Ioo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Eoo`T000=oo`03001o
ogoo00=oo`03001oogoo00Eoo`03001oogoo00Qoo`8002=oo`800003Ool007oo009oo`<000Qoo`03
001oogoo00Eoo`05001oo`00Ool00003Ool30006Ool200000goo00000005Ool00`00Oomoo`07Ool0
0`00Oomoo`06Ool001=oo`03001oogoo01moo`03001oogoo00=oo`03001oogoo00Aoo`8000Eoo`03
001oogoo00Eoo`8000Uoo`03001oo`00009oo`80009oo`<000Aoo`03001oogoo00Ioo`03001oogoo
00=oo`03001oogoo00Yoo`@00005Ool00000Ool00006Ool400000goo0000000:Ool00`00Oomoo`0S
Ool01000Ool007oo0`000goo0`0017oo00<007ooOol01goo0`001Goo00<007oo00000P0000=oo`00
00000goo0P001Woo00<007ooOol01goo00<007ooOol01Goo000COol2000QOol00`00Oomoo`03Ool0
0`00Oomoo`02Ool20006Ool00`00Oomoo`04Ool00`00Oomoo`0:Ool01000Ool000001Goo00<007oo
00000`002Woo00D007ooOomoo`0000Uoo`@000=oo`04001oogooOol40008Ool40008Ool00`00Oomo
o`0TOol01000Ool000002Goo0P0000=oo`00Ool02Goo00<007oo00001Woo00D007ooOomoo`0000Eo
o`03001oo`0000Eoo`03001oogoo00Moo`03001oogoo00Eoo`004goo00<007ooOol087oo00<007oo
Ool017oo00D007ooOomoo`0000Qoo`04001oogooOol2000=Ool00`00Ool00006Ool00`00Oomoo`02
Ool00`00Oomoo`09Ool00`00Ool0000:Ool00`00Oomoo`05Ool00`00Oomoo`02Ool00`00Ool00006
Ool00`00Oomoo`02Ool30004Ool00`00Oomoo`0VOol00`00Oomoo`0;Ool00`00Oomoo`0;Ool00`00
Oomoo`04Ool00`00Oomoo`06Ool01@00Oomoogoo00001Goo00<007ooOol01goo00<007ooOol017oo
000COol00`00Oomoo`0QOol00`00Oomoo`04Ool00`00Ool00009Ool01000Oomoo`0047oo0P001Woo
00<007ooOol03goo00<007ooOol027oo00<007ooOol01goo00D007ooOomoo`00009oo`8000=oo`03
001oogoo00Ioo`800003Ool007oo02Moo`8001aoo`03001oogoo00=oo`03001oogoo00Ioo`03001o
ogoo009oo`03001oogoo009oo`03001oogoo00ioo`0000Eoo`00000007oo00@000=oo`8000Eoo`03
001oogoo025oo`03001oogoo00Eoo`8000Uoo`03001oo`0001=oo`03001oogoo00Aoo`03001oogoo
01Uoo`03001oogoo00Moo`03001oogoo009oo`07001oogooOol007oo00002goo00<007ooOol09Woo
00<007oo00007Goo00<007ooOol00goo00<007ooOol017oo00<007ooOol017oo00<007ooOol00Woo
00<007ooOol03Goo0002Ool01@00Oomoogoo000017oo00@007ooOol000Aoo`80029oo`03001oogoo
009oo`@000]oo`03001oogoo019oo`03001oogoo00Aoo`03001oogoo01Qoo`03001oogoo00Uoo`03
001oogoo009oo`05001oogooOol0000dOol01@00Oomoogoo000077oo00<007ooOol00goo00<007oo
Ool017oo00<007ooOol01Goo00D007ooOomoo`0000moo`000Woo00<007ooOol00Woo00D007ooOomo
o`00009oo`03001oogoo009oo`03001oogoo029oo`@002Eoo`03001oogoo00=oo`03001oogoo00]o
o`@000Uoo`03001oogoo00Uoo`03001oogoo009oo`03001oogoo03Eoo`03001oogoo009oo`8001ao
o`03001oogoo00=oo`03001oogoo009oo`03001oogoo00Ioo`05001oogooOol0000?Ool0009oo`03
001oogoo00=oo`04001oogoo0002Ool00`00Oomoo`02Ool00`00Oomoo`1;Ool00`00Oomoo`04Ool0
0`00Oomoo`05Ool50003Ool00`00Oomoo`06Ool00`00Oomoo`0;Ool00`00Oomoo`02Ool00`00Oomo
o`0cOol00`00Oomoo`05Ool00`00Oomoo`0IOol00`00Oomoo`03Ool01@00Oomoogoo00002Woo00D0
07ooOomoo`0000ioo`000Woo00<007ooOol00goo00@007ooOol0009oo`03001oogoo009oo`03001o
ogoo04aoo`03001oogoo00=oo`03001oogoo00=oo`8000Uoo`03001oogoo00Eoo`03001oogoo00ao
o`05001oogooOol0000eOol00`00Oomoo`06Ool2000JOol00`00Oomoo`03Ool01000Oomoo`002Woo
00D007ooOomoo`0000ioo`0000=oo`0000000Woo0`0017oo0P001Goo0P00CGoo00<007ooOol00goo
00@007ooOomoo`8000aoo`03001oogoo00Aoo`03001oogoo00aoo`03001oogoo009oo`03001oogoo
02Eoo`03001oogoo00Uoo`03001oogoo00Uoo`03001oogoo01Moo`03001oogoo00=oo`03001oo`00
00aoo`05001oogooOol0000=Ool001=oo`03001oogoo04aoo`03001oogoo00=oo`04001oogoo000?
Ool00`00Oomoo`02Ool00`00Oomoo`0>Ool70004Ool6000KOol00`00Oomoo`08Ool00`00Oomoo`0;
Ool3000GOol00`00Oomoo`02Ool00`00Ool0000<Ool01@00Oomoogoo00003Goo000COol00`00Oomo
o`1=Ool00`00Oomoo`03Ool2000@Ool00`00Oomoo`02Ool00`00Oomoo`0COol00`00Ool000030006
Ool3000GOol00`00Ool00008Ool00`00Oomoo`0?Ool3000DOol00`00Oomoo`03Ool00`00Oomoo`0<
Ool01000Oomoo`003Goo000COol00`00Oomoo`1=Ool00`00Oomoo`03Ool00`00Oomoo`0@Ool01000
Oomoo`005goo0`002Woo00<007ooOol05Goo00@007ooOol000Ioo`03001oogoo01=oo`@0015oo`03
001oogoo019oo`04001oogoo000<Ool001=oo`03001oogoo04ioo`04001oogooOol2000COol00`00
Ool0000JOol00`00Oomoo`08Ool00`00Oomoo`0COol00`00Oomoo`02Ool00`00Oomoo`03Ool00`00
Oomoo`0GOol3000?Ool00`00Oomoo`0AOol01000Oomoo`0017oo0P001Woo000COol2001?Ool01000
Oomoo`000Woo00<007ooOol04Goo00<007ooOol06Woo00<007ooOol027oo00<007ooOol04Woo00<0
07ooOol00goo00D007ooOomoo`0001eoo`03001oogoo00eoo`03001oogoo015oo`04001oogoo0002
Ool00`00Oomoo`06Ool001=oo`03001oogoo04ioo`03001oo`0000=oo`03001oogoo02moo`03001o
ogoo00Moo`03001oogoo015oo`03001oogoo00Eoo`03001oo`0001ioo`03001oogoo00eoo`03001o
ogoo015oo`06001oogoo001oo`002Goo000COol00`00Oomoo`1?Ool00`00Oomoo`02Ool00`00Oomo
o`0`Ool00`00Oomoo`07Ool00`00Oomoo`0@Ool00`00Oomoo`06Ool00`00Oomoo`0NOol00`00Oomo
o`0=Ool00`00Oomoo`0AOol01000Oomoo`002Woo000COol00`00Oomoo`1EOol00`00Oomoo`0_Ool0
0`00Oomoo`08Ool00`00Oomoo`0?Ool00`00Oomoo`0WOol00`00Oomoo`0>Ool00`00Oomoo`0@Ool0
1000Ool000002Woo000COol2001FOol00`00Oomoo`0`Ool00`00Oomoo`07Ool00`00Oomoo`0>Ool0
0`00Oomoo`0YOol00`00Oomoo`0>Ool2000AOol01000Oomoo`002Goo000COol00`00Oomoo`1EOol0
0`00Oomoo`0aOol00`00Oomoo`07Ool00`00Oomoo`0=Ool00`00Oomoo`0VOol200000goo001oo`0A
Ool2000BOol00`00Oomoo`07Ool001=oo`03001oogoo05Ioo`03001oogoo035oo`03001oogoo00Mo
o`03001oogoo00]oo`03001oogoo02Eoo`800005Ool007ooOol0000COol2000AOol00`00Oomoo`06
Ool001=oo`03001oogoo05Ioo`03001oogoo039oo`03001oogoo00Ioo`03001oogoo00]oo`03001o
ogoo02=oo`8000Aoo`03001oo`0001Eoo`8000moo`03001oogoo00Ioo`004goo0P00Egoo00<007oo
Ool0<Woo00<007ooOol01goo00<007ooOol02Goo00<007ooOol077oo20001goo00<007oo00005Woo
00<007ooOol03Goo00<007ooOol01Goo000COol00`00Oomoo`1GOol00`00Oomoo`0bOol00`00Oomo
o`04Ool3000;Ool00`00Oomoo`0LOol00`00Oomoo`0=Ool2000GOol00`00Oomoo`0<Ool00`00Oomo
o`05Ool001=oo`03001oogoo05Moo`03001oogoo03=oo`03001oogoo00<00004Ool007oo0009Ool0
0`00Oomoo`0LOol00`00Oomoo`0?Ool2000GOol00`00Oomoo`0<Ool00`00Oomoo`04Ool00005Ool0
0000001oo`030004Ool20005Ool00`00Oomoo`1GOol00`00Oomoo`0dOol20005Ool20009Ool00`00
Oomoo`0:Ool00`00Oomoo`0?Ool00`00Oomoo`0@Ool00`00Oomoo`0FOol00`00Oomoo`0BOol0009o
o`03001oogoo00=oo`04001oogoo0002Ool00`00Oomoo`02Ool2001IOol00`00Oomoo`0jOol20008
Ool00`00Oomoo`0;Ool00`00Oomoo`0>Ool00`00Oomoo`0BOol2000;Ool00`00Oomoo`09Ool50005
Ool40006Ool0009oo`03001oogoo00=oo`04001oogoo0002Ool00`00Oomoo`02Ool00`00Oomoo`1H
Ool00`00Oomoo`0kOol20006Ool00`00Oomoo`0;Ool00`00Ool0000?Ool00`00Oomoo`0DOol30007
Ool2000@Ool50003Ool00`00Oomoo`04Ool0009oo`04001oogooOol20003Ool01000Oomoo`0017oo
00<007ooOol0F7oo00<007ooOol0>goo0P001Woo00<007ooOol02Woo00D007ooOomoo`0000eoo`03
001oogoo01Ioo`04001oo`000004Ool00`00Ool0000GOol00`00Oomoo`05Ool0009oo`03001oogoo
00=oo`04001oogoo0002Ool00`00Oomoo`02Ool00`00Oomoo`1IOol00`00Oomoo`0kOol20004Ool0
0`00Oomoo`0:Ool00`00Oomoo`02Ool00`00Oomoo`0;Ool00`00Oomoo`0GOol00`00Oomoo`020000
0goo001oo`02Ool00`00Oomoo`0DOol00`00Oomoo`05Ool00003Ool00000009oo`<000Aoo`8000Eo
o`03001oogoo05Uoo`03001oogoo03aoo`03001oogoo009oo`03001oogoo00Yoo`03001oogoo00=o
o`03001oogoo00Yoo`03001oogoo01Qoo`05001oogooOol00004Ool00`00Oomoo`0COol00`00Oomo
o`06Ool001=oo`8005Yoo`03001oogoo03aoo`80009oo`03001oogoo00Yoo`03001oogoo00Eoo`03
001oogoo00Qoo`03001oogoo01Yoo`04001oo`00Ool20002Ool00`00Oomoo`0COol00`00Oomoo`06
Ool001=oo`03001oogoo05Yoo`03001oogoo03aoo`03001oo`0000aoo`03001oogoo00Ioo`03001o
ogoo00Qoo`03001oogoo01]oo`03001oogoo009oo`04001oo`00Ool4000@Ool00`00Oomoo`06Ool0
01=oo`03001oogoo05Yoo`03001oogoo03aoo`03001oo`0000Yoo`8000Yoo`03001oogoo00Ioo`03
001oogoo029oo`<000=oo`03001oogoo00eoo`03001oogoo00Moo`004goo00<007ooOol0FWoo00<0
07ooOol0?Goo0P002Goo00<007ooOol02Woo00<007ooOol01Woo00<007ooOol097oo00<007ooOol0
0Woo00<007ooOol037oo00<007ooOol01goo000COol2001LOol00`00Oomoo`0nOol30004Ool2000>
Ool01000Oomoogoo10009goo00<007ooOol00goo00<007ooOol02Woo00<007ooOol027oo000COol0
0`00Oomoo`1KOol00`00Oomoo`11Ool200000goo001oo`0@Ool3000[Ool00`00Oomoo`04Ool00`00
Oomoo`09Ool00`00Oomoo`08Ool001=oo`03001oogoo05aoo`03001oogoo049oo`03001oogoo03io
o`03001oogoo00Aoo`03001oogoo00Qoo`03001oogoo00Uoo`004goo00<007ooOol0GGoo00<007oo
Ool0Pgoo00<007ooOol017oo00<007ooOol01goo00<007ooOol00Woo00<007ooOol017oo000COol2
001NOol00`00Oomoo`23Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`02Ool00`00Oomo
o`04Ool001=oo`03001oogoo05ioo`03001oogoo089oo`03001oogoo00Ioo`P000Aoo`03001oogoo
00Eoo`004goo00<007ooOol0Ggoo00<007ooOol01Goo0P00NWoo00<007ooOol03Goo00<007ooOol0
0Woo00<007ooOol01Goo000COol00`00Oomoo`1POol00`00Oomoo`03Ool00`00Ool0001kOol00`00
Oomoo`0<Ool00`00Oomoo`02Ool00`00Oomoo`05Ool001=oo`80065oo`04001oogooOol20003Ool0
0`00Oomoo`1^Ool20008Ool00`00Oomoo`0;Ool00`00Ool00002Ool00`00Oomoo`06Ool001=oo`03
001oogoo065oo`03001oo`0000Eoo`03001oogoo06eoo`04001oogoo0007Ool00`00Oomoo`0:Ool0
1`00Oomoogoo001oo`0000Qoo`004goo00<007ooOol0HWoo00<007ooOol017oo00<007ooOol0K7oo
00<007ooOol00Woo0P001Goo00<007ooOol02Woo00<007ooOol00Woo00<007ooOol01goo000017oo
000000000goo00<007ooOol00Woo0P001Goo00<007ooOol0JWoo00<007ooOol0JWoo00<007ooOol0
1Goo00<007ooOol00goo00<007ooOol027oo00<007ooOol00goo0P0027oo0002Ool00`00Oomoo`04
0002Ool01000Oomoo`0017oo00<007ooOol0JWoo00<007ooOol027oo00<007ooOol0GWoo00<007oo
Ool01goo00<007ooOol00Woo00<007ooOol017oo0P0000=oo`00Ool01Goo00<007oo00001goo0002
Ool01P00Oomoo`00Ool000=oo`04001oogoo0004Ool2001/Ool00`00Oomoo`07Ool00`00Oomoo`0^
Ool2000HOol00`00Oomoo`0BOol00`00Oomoo`08Ool00`00Oomoo`02Ool00`00Oomoo`04Ool01000
Ool000001Goo00D007ooOomoo`0000Ioo`000Woo00H007ooOol007oo0003Ool01000Oomoo`0017oo
00<007ooOol0Jgoo00<007ooOol01Woo00<007oo0000;Goo0P0000=oo`00Ool05goo00<007ooOol0
4Goo00<007ooOol02Woo00D007ooOomoo`0000Eoo`03001oogoo008000Eoo`03001oogoo00Qoo`00
0Woo00@007ooOomoo`8000=oo`04001oogoo0004Ool00`00Oomoo`1[Ool00`00Oomoo`05Ool01@00
Oomoogoo00005Woo00<007ooOol04Goo0P0017oo00<007ooOol057oo00<007oo00002goo1`003Woo
00@007ooOol000Aoo`07001oogooOol007oo00000goo00<007ooOol02Goo00000goo00000004Ool0
0`00Oomoo`02Ool20005Ool00`00Oomoo`1/Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomo
o`0DOol00`00Oomoo`0@Ool00`00Oomoo`05Ool00`00Oomoo`0BOol01000Oomoo`002goo00<007oo
Ool04Woo00D007ooOomoo`0000=oo`04001oogoo0003Ool01000Oomoo`002goo000COol2001]Ool0
0`00Oomoo`02Ool00`00Oomoo`04Ool00`00Oomoo`0BOol00`00Ool0000AOol00`00Oomoo`06Ool0
0`00Oomoo`06Ool30007Ool00`00Oomoo`02Ool00`00Oomoo`07Ool00`00Oomoo`0DOol00`00Oomo
o`070005Ool00`00Ool0000;Ool001=oo`03001oogoo06eoo`05001oogooOol00006Ool00`00Oomo
o`0BOol01000Oomoo`003goo00<007ooOol027oo00<007ooOol00Woo0`000goo00<007ooOol017oo
00<007ooOol00goo00<007ooOol01Goo00<007ooOol05Woo00D007ooOomoo`0000Yoo`03001oogoo
00Yoo`004goo00<007ooOol0KGoo00@007ooOol000Qoo`03001oogoo011oo`03001oogoo009oo`03
001oogoo00]oo`03001oogoo00Yoo`04001oo`000007Ool20003Ool00`00Oomoo`04Ool00`00Oomo
o`05Ool00`00Oomoo`0FOol01@00Oomoogoo00005goo000COol00`00Oomoo`1]Ool00`00Ool0000:
Ool00`00Oomoo`0>Ool00`00Oomoo`03Ool00`00Oomoo`0;Ool00`00Oomoo`0;Ool00`00Oomoo`09
Ool00`00Ool00008Ool00`00Oomoo`03Ool00`00Oomoo`0HOol00`00Ool0000HOol001=oo`8006mo
o`03001oogoo00Uoo`03001oogoo00ioo`03001oogoo00Aoo`03001oogoo00Uoo`03001oogoo01Uo
o`03001oogoo00Moo`03001oogoo00=oo`03001oogoo01Uoo`03001oogoo01Moo`004goo00<007oo
Ool0Ngoo0P003Goo00<007ooOol01Woo00<007ooOol01goo00<007ooOol09Goo00D007ooOomoo`00
03Ioo`004goo00<007ooOol0OGoo00<007ooOol02Woo00<007ooOol01goo0P001goo00<007ooOol0
9Woo00<007oo0000=goo000COol00`00Oomoo`1nOol20009Ool00`00Oomoo`0:Ool00`00Oomoo`03
Ool00`00Oomoo`0WOol00`00Ool0000gOol001=oo`80085oo`03001oogoo00Eoo`03001oogoo00ao
o`8000=oo`03001oogoo02Qoo`03001oogoo03Ioo`004goo00<007ooOol0PGoo10000goo00<007oo
Ool03Woo00<007oo0000I7oo000COol00`00Oomoo`25Ool3000BOol00`00Oomoo`1SOol001=oo`03
001oogoo0?moo`5oo`004goo00<007ooOol0ogoo0Goo000COol2003oOol2Ool001=oo`03001oogoo
0?moo`5oo`00ogoo5goo003oOolGOol00?mooaMoo`00\
\>"],
  ImageRangeCache->{{{0, 277.938}, {170.562, 0}} -> {-3.01968, 70.07, 
  0.197976, 0.588094}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["21.4 Coarse Paths and Event Simulation", "Section",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True],

Cell[TextData[{
  "Sometimes we shall wish to focus instead on a few regularly or irregularly \
spaced discrete events. In such circumstances it is more accurate to use the \
integrated form of the log-normal distribution to jump straight to each such \
event. We build three functions to generate the stock price at such discrete \
events, which parallel those we have used for the full path. When the times \
between events can vary, this is a good opportunity to use ",
  StyleBox["FoldList",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", the generalization of ",
  StyleBox["NestList",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " that allows a separate parameter to be fed in at each stage. The stock \
price changes in scale between events by a multiplicative factor based on the \
log-normal distribution. The mean parameter is given by"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Mu] = \((\(-\(\[Sigma]\^2\/2\)\) - q + 
            r)\)\ \[CapitalDelta]\ T\)], "NumberedEquation",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The standard deviation is given by ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma] \@\( \[CapitalDelta]\ T\)\)]],
  ". It is left as an exercise for the reader to show that the resulting path \
has the same ",
  StyleBox["mean",
    FontSlant->"Italic"],
  " drift as the one we have considered previously. Let's look first at the \
case of regular clockwork sampling. It is a simple matter to adjust the ",
  StyleBox["path",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function to use the integrated form of the log-normal distribution:"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
coarsepath[s_, r_, q_, \[Sigma]_, t_, n_, navg_] :=
Module[
{mpath = N[(r-q-\[Sigma]^2/2)*t/(navg - 1)], spath = N[\[Sigma] Sqrt[t/(navg \
- 1)]]},
Table[
NestList[# Exp[norm[mpath, spath]]&, s, navg - 1], {i, n}] ];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[TextData[{
  "Similarly, without bothering to write down the intermediate cases, we can \
generate ",
  Cell[BoxData[
      \(TraditionalForm\`4  n\)]],
  " paths using two sets of antithetic pairs:"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
coarsepathpairat[s_, r_, q_, \[Sigma]_, t_, n_, navg_] :=
Module[
{mpath = N[(r-q-\[Sigma]^2/2)*t/(navg - 1)], 
spath = N[\[Sigma] Sqrt[t/(navg - 1)]]},
Flatten[
 Table[
  Transpose[
   NestList[# Exp[normpairat[mpath, spath]]&, {s, s, s, s}, navg - 1]], 
  {i, n}],
 1]
];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["\<\
Note that this is slower (0.27 seconds for the non-integrated form \
on the same computer):\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      w\  = \ coarsepathpairat[100, \ 0.1, \ 0, \ 0.2, \ 1, \ 25, \ 
          100]; ]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.466666666666583296`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell["\<\
When the events are irregularly spaced, first of all we need the \
time-intervals between events. So, for example, suppose we have a list of \
times:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(times\  = \ {ta, \ tb, \ tc}; \)\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We need the time intervals starting at zero - these are given \
by\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Prepend[Drop[RotateLeft[times] - times, \ \(-1\)], \ First[times]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({ta, \(-ta\) + tb, \(-tb\) + tc}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The following function uses ",
  StyleBox["FoldList",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to incorporate the various time intervals into the sampling:"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
eventseq[s_, r_, q_, \[Sigma]_, EventTimes_List, n_] :=
Module[{intervals = Prepend[Drop[RotateLeft[EventTimes]-EventTimes, -1], 
First[EventTimes]],muscale = (r - q - \[Sigma]^2/2)},
Table[
FoldList[(#1*Exp[norm[muscale*#2, \[Sigma]*Sqrt[#2]]])&, s, intervals], {i, \
n}] ];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(eventseq[100, \ 0.1, \ 0, \ 0.2, \ {0.5, \ 1}, \ 4]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{100, 107.184955293423511`, 94.803030566750106`}, {100, 
        121.379650140122352`, 112.390914194852453`}, {100, 
        111.381787814489818`, 134.198183818513205`}, {100, 
        82.8331992828985796`, 59.8647607951378013`}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Similarly, we can generate ",
  Cell[BoxData[
      \(TraditionalForm\`4  n\)]],
  " sets of events using our random pairs with sets of antithetic pairs:"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
eventseqpairat[s_, r_, q_, \[Sigma]_, EventTimes_List, n_] :=
Module[
{intervals = Prepend[Drop[RotateLeft[EventTimes]-EventTimes, -1], \
First[EventTimes]],
muscale = (r - q - \[Sigma]^2/2)},
Flatten[Table[Transpose[FoldList[(#1*Exp[normpairat[muscale*#2, \
\[Sigma]*Sqrt[#2]]])&, {s,s,s,s}, intervals]], {i, n}],1]];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(eventseqpairat[100, \ 0.1, \ 0, \ 0.2, \ {0.5, \ 1}, \ 1]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{100, 116.225993161968466`, 158.816450998120579`}, {100, 
        93.2052321691351437`, 73.8910146660875266`}, {100, 
        84.4241296759301462`, 91.8793772978541767`}, {100, 
        128.314863515117805`, 127.722989152127963`}}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["21.5 Vanilla Options Re-visited", "Section",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
To check that all is working, and to gain some insight, we now look \
again at the pricing of simple vanilla Calls and Puts within the Monte Carlo \
framework. We make some comparisons with the standard analytical models, and \
begin by loading the required Package.\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Derivatives`BlackScholes`\>"]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(MonteCarloEuroCallA[S_, \ K_, \ \[Sigma]_, \ r_, \ q_, \ t_, \ 
        n_]\  := \n
      Map[\((Max[0, \ Last[#] - K] &)\), 
          eventseq[S, \ r, \ q, \ \[Sigma], \ {t}, \ n]] // \n\t
        Exp[\(-r\)*t]*{Mean[#], \ StandardErrorOfSampleMean[#]} &\)], "Input",\

  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(MonteCarloEuroCallB[S_, \ K_, \ \[Sigma]_, \ r_, \ q_, \ t_, \ 
        n_]\  := \n
      Map[\((Max[0, \ Last[#] - K] &)\), 
          eventseqpairat[S, \ r, \ q, \ \[Sigma], \ {t}, \ 
            Ceiling[n/4]]] // \n
        Exp[\(-r\)*t]*{Mean[#], \ StandardErrorOfSampleMean[#]} &\)], "Input",\

  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesCall[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(21.2487714385643755`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      MonteCarloEuroCallA[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1\ , 10000]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({2.03333333333330301`\ Second, {21.5808671851902555`, 
        0.200969849123350385`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      MonteCarloEuroCallB[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1\ , 10000]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1.46666666666669698`\ Second, {21.2068357075144664`, 
        0.19890612929826851`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      MonteCarloEuroCallA[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1\ , 40000]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({8.18333333333339396`\ Second, {21.316178310949696`, 
        0.100076187123207294`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      MonteCarloEuroCallB[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1\ , 40000]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({6.04999999999995452`\ Second, {21.3880873476497912`, 
        0.101128175335723891`}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(MonteCarloEuroPutA[S_, \ K_, \ \[Sigma]_, \ r_, \ q_, \ t_, \ 
        n_]\  := \n
      Map[\((Max[0, \ K - Last[#]] &)\), 
          eventseq[S, \ r, \ q, \ \[Sigma], \ {t}, \ n]] // \n
        Exp[\(-r\)*t]*{Mean[#], \ StandardErrorOfSampleMean[#]} &\)], "Input",\

  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(MonteCarloEuroPutB[S_, \ K_, \ \[Sigma]_, \ r_, \ q_, \ t_, \ 
        n_]\  := \n
      Map[\((Max[0, \ K - \ Last[#]] &)\), 
          eventseqpairat[S, \ r, \ q, \ \[Sigma], \ {t}, \ 
            Ceiling[n/4]]] // \n
        Exp[\(-r\)*t]*{Mean[#], \ StandardErrorOfSampleMean[#]} &\)], "Input",\

  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesPut[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1.7325132421603282`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      MonteCarloEuroPutA[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1\ , 10000]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({2.08333333333337122`\ Second, {1.65490356904975737`, 
        0.0457261479998356179`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      MonteCarloEuroPutB[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1\ , 10000]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1.5166666666666515`\ Second, {1.73589311537192214`, 
        0.047725091994376827`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      MonteCarloEuroPutA[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1\ , 40000]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({8.2666666666666515`\ Second, {1.72039934466544242`, 
        0.0236014454013064023`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      MonteCarloEuroPutB[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1\ , 40000]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({6.16666666666662877`\ Second, {1.72123250539607592`, 
        0.0235463821851643162`}}\)], "Output"]
}, Open  ]],

Cell["\<\
Note that these results suggest that the benefit of antithetic pair \
use is limited to the reduction in variance of the answer obtained from the \
use of twice the number of sample points, but we get a modest speed gain from \
the fact that we do not need to make so many calls to the random number \
generator.\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Hedge Parameters", "Subsection",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The key to getting good Monte Carlo estimates of hedge parameters is to \
understand that you have to isolate the effect of small changes in parameters \
from sample noise. The Golden Rule is: store the paths/event sequences, and \
use the same ones to work with the adjusted parameters. This is most easily \
illustrated using \[Delta] and \[CapitalGamma], but the same principles apply \
for \[Rho], \[CapitalLambda]. To keep the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " description simple, we work with the simplest simulation defined by ",
  StyleBox["eventseq",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", but with just one time, and extend this function to treat \[Delta] and \
\[CapitalGamma]."
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
hedgeeventseq[s_, r_, q_, \[Sigma]_, t_, n_] :=
Module[{muscale = (r - q - \[Sigma]^2/2)},
Table[Exp[norm[muscale*t, \[Sigma]*Sqrt[t]]]*s*{1,1.001,0.999}, {i, n}] \
];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["hedgeeventseq[100,0.1,0,0.2,1,5]", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \({{109.733730148369001`, 109.843463878517377`, 
        109.623996418220626`}, {106.465136683057393`, 106.571601819740457`, 
        106.35867154637435`}, {101.759625706642653`, 101.861385332349274`, 
        101.65786608093601`}, {88.3342650225356251`, 88.422599287558139`, 
        88.2459307575130758`}, {114.319830032645297`, 114.434149862677925`, 
        114.20551020261267`}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(MonteCarloHedgeEuroCall[S_, \ K_, \ \[Sigma]_, \ r_, \ q_, \ t_, \ 
        n_]\  := \n
      Map[Exp[\(-r\)*t]*{Mean[#], \ 
              StandardErrorOfSampleMean[#]} &, \n\t\tTranspose[
          Map[\(({#[\([1]\)], \ \n\t\t\t\t\((#[\([2]\)] - \
#[\([3]\)])\)/\((0.002*S)\), \n\t\t\t\t\((#[\([2]\)] + #[\([3]\)] - 
                        2*#[\([1]\)])\)/\((0.001*
                          S)\)^2} &)\), \n\t\tMap[\((Max[0, \ # - K] &)\), 
              hedgeeventseq[S, \ r, \ q, \ \[Sigma], \ t, \ n], {2}]]]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Let's apply this function to our previous example. It outputs the \
value, \[Delta] and \[CapitalGamma] together with their standard errors:\
\>",
   "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TableForm[
      MonteCarloHedgeEuroCall[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1\ , 
        40000]]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[GridBox[{
          {"21.1580193980229713`", "0.0993958662335060516`"},
          {"0.856831350132799762`", "0.00222584003574292799`"},
          {"0.0121205342863124454`", "0.00125486248266801747`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell["Here are the analytical results for comparison:", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TableForm[{BlackScholesCall[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1], \n\t
        BlackScholesCallDelta[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1], \n\t
        BlackScholesCallGamma[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1]}]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {"21.2487714385643755`"},
          {"0.859159524933854967`"},
          {"0.0101583230391658485`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      TableForm[ {21.248771438564376, 0.85915952493385506, 
        0.010158323039165848}]]], "Output"]
}, Open  ]],

Cell["\<\
It should be noted that the computation of \[CapitalGamma] by this \
method is particularly awkward, and this is reflected in the large standard \
error compared to the value. Most of the time the payoff is a linear function \
of the stock price at expiry, and most of the sampled values of \
\[CapitalGamma] are zero! Only when the differencing is applied to an expiry \
value in the neighbourhood of the strike do we get a contribution, which is \
then very large. This is a critical and persistent difficulty.

Obviously the Puts work the same way:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(MonteCarloHedgeEuroPut[S_, \ K_, \ \[Sigma]_, \ r_, \ q_, \ t_, \ 
        n_]\  := \n
      Map[Exp[\(-r\)*t]*{Mean[#], \ 
              StandardErrorOfSampleMean[#]} &, \n\t\tTranspose[
          Map[\(({#[\([1]\)], \ \((#[\([2]\)] - #[\([3]\)])\)/\((0.002*
                        S)\), \n\t\t\t\t\((#[\([2]\)] + #[\([3]\)] - 
                        2*#[\([1]\)])\)/\((0.001*
                          S)\)^2} &)\), \n\t\tMap[\((Max[0, \ K - #] &)\), 
              hedgeeventseq[S, \ r, \ q, \ \[Sigma], \ t, \ n], {2}]]]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TableForm[
      MonteCarloHedgeEuroPut[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1\ , 
        40000]]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[GridBox[{
          {"1.78356380410276482`", "0.0241045573646320576`"},
          {\(-0.143179898795384978`\), "0.00146465767118332701`"},
          {"0.0104106937822940182`", "0.00113101147994262318`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({BlackScholesPut[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1], \n\t
      BlackScholesPutDelta[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1], \n\t
      BlackScholesPutGamma[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1]}\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1.7325132421603282`, \(-0.140840475066144943`\), 
      0.0101583230391658418`}\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["21.6 Wozniakowski Sampling", "Section",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
One of our methods for generating Gaussian deviates used pairs of \
random numbers within the unit square. We can reorganize our algorithm into \
two stages:

(a) first make points randomly distributed in the unit square;
(b) make samples of a normal distribution.

The first of these can be accomplished by a routine such as\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["randompoints[n_] := Table[{Random[], Random[]}, {k,1,n}];", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The second part can be dealt with by taking the output of this and \
applying the function\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
hammernorm[mu_, sigma_, p_List] := mu+sigma*Sqrt[-2 Log[p[[1]]]] \
Cos[2 Pi p[[2]]]\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
One difficulty is that the points generated do not nicely cover the \
unit square - let's look at 400 of them:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Show[Graphics[{PointSize[0.015], Map[ Point[#]&, 
randompoints[400]]}], AspectRatio -> 1];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 7  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.0238095 0.953392 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.015 w
.86615 .78048 Mdot
.94183 .85428 Mdot
.02479 .07877 Mdot
.78704 .16921 Mdot
.92331 .21056 Mdot
.701 .753 Mdot
.66087 .38 Mdot
.30295 .39676 Mdot
.82785 .80262 Mdot
.04534 .22146 Mdot
.24126 .6566 Mdot
.11694 .52131 Mdot
.35131 .85332 Mdot
.15129 .64423 Mdot
.35033 .79836 Mdot
.34044 .49883 Mdot
.4032 .61161 Mdot
.61563 .72303 Mdot
.71852 .25542 Mdot
.33649 .35008 Mdot
.86686 .43 Mdot
.31496 .15243 Mdot
.64941 .7506 Mdot
.22183 .60832 Mdot
.32191 .87448 Mdot
.09435 .94129 Mdot
.94777 .09992 Mdot
.7301 .46628 Mdot
.56838 .46552 Mdot
.13827 .72045 Mdot
.82605 .23391 Mdot
.77797 .39418 Mdot
.93538 .78112 Mdot
.48682 .26556 Mdot
.30978 .05433 Mdot
.28879 .63445 Mdot
.96406 .15706 Mdot
.21825 .67036 Mdot
.0401 .08094 Mdot
.46434 .22789 Mdot
.44791 .59263 Mdot
.34988 .48464 Mdot
.59805 .38253 Mdot
.5481 .11427 Mdot
.63887 .57861 Mdot
.08509 .82591 Mdot
.3529 .5481 Mdot
.77249 .21528 Mdot
.36503 .41485 Mdot
.57804 .52212 Mdot
.34874 .35772 Mdot
.13751 .31804 Mdot
.87702 .74229 Mdot
.76382 .8106 Mdot
.30278 .38357 Mdot
.23954 .72013 Mdot
.6401 .78216 Mdot
.17826 .87142 Mdot
.31101 .25787 Mdot
.38196 .67996 Mdot
.92217 .82023 Mdot
.78011 .18164 Mdot
.59724 .48632 Mdot
.66641 .8408 Mdot
.69641 .72123 Mdot
.87877 .05401 Mdot
.41744 .36146 Mdot
.66305 .31108 Mdot
.75354 .55651 Mdot
.5086 .41686 Mdot
.46634 .32244 Mdot
.15045 .7141 Mdot
.52036 .47942 Mdot
.34653 .55627 Mdot
.89931 .9703 Mdot
.65632 .69267 Mdot
.22671 .27288 Mdot
.75373 .66247 Mdot
.78545 .88862 Mdot
.1145 .37519 Mdot
.05573 .35592 Mdot
.58209 .93554 Mdot
.56558 .05729 Mdot
.45545 .24524 Mdot
.06904 .55508 Mdot
.13272 .66617 Mdot
.14592 .56198 Mdot
.4526 .95071 Mdot
.8954 .31291 Mdot
.67506 .31205 Mdot
.13376 .40149 Mdot
.58437 .91406 Mdot
.10184 .06938 Mdot
.02609 .95572 Mdot
.51245 .0359 Mdot
.54684 .73429 Mdot
.46722 .45803 Mdot
.43792 .09193 Mdot
.34511 .87325 Mdot
.96152 .11842 Mdot
.4259 .58415 Mdot
.31027 .78357 Mdot
.31595 .20647 Mdot
.70209 .84672 Mdot
.23792 .1609 Mdot
.69981 .8682 Mdot
.70166 .14881 Mdot
.17678 .15772 Mdot
.25825 .66798 Mdot
.71505 .0896 Mdot
.88933 .77193 Mdot
.72972 .94838 Mdot
.48723 .21159 Mdot
.44326 .18861 Mdot
.19509 .02892 Mdot
.71736 .3191 Mdot
.93336 .84523 Mdot
.04137 .4281 Mdot
.25551 .72023 Mdot
.84078 .2942 Mdot
.97346 .07606 Mdot
.14955 .22841 Mdot
.10794 .28133 Mdot
.39602 .25723 Mdot
.5969 .09355 Mdot
.92895 .09243 Mdot
.42561 .08844 Mdot
.23539 .75053 Mdot
.46844 .22041 Mdot
.21783 .34624 Mdot
.23674 .47739 Mdot
.35324 .07585 Mdot
.23947 .42514 Mdot
.22751 .82464 Mdot
.15534 .16762 Mdot
.80768 .59122 Mdot
.53463 .09788 Mdot
.85493 .5226 Mdot
.13283 .03325 Mdot
.64335 .74927 Mdot
.64058 .79004 Mdot
.44932 .42684 Mdot
.42765 .33646 Mdot
.11989 .3748 Mdot
.21199 .88853 Mdot
.86857 .52736 Mdot
.08046 .74471 Mdot
.0847 .91334 Mdot
.52202 .67065 Mdot
.20596 .41456 Mdot
.413 .6612 Mdot
.5388 .64249 Mdot
.74861 .84836 Mdot
.11328 .23947 Mdot
.34476 .53571 Mdot
.96959 .84187 Mdot
.15658 .62439 Mdot
.12483 .33832 Mdot
.09993 .85687 Mdot
.06394 .40218 Mdot
.5541 .21004 Mdot
.83417 .96483 Mdot
.16491 .52604 Mdot
.31919 .34614 Mdot
.3925 .65487 Mdot
.22971 .13049 Mdot
.07155 .14297 Mdot
.23631 .26582 Mdot
.89116 .49579 Mdot
.1353 .9047 Mdot
.81504 .61612 Mdot
.09517 .52633 Mdot
.28474 .42989 Mdot
.23718 .5387 Mdot
.14364 .88106 Mdot
.89419 .21637 Mdot
.72733 .24999 Mdot
.68829 .10969 Mdot
.6796 .13083 Mdot
.47578 .82107 Mdot
.76463 .61224 Mdot
.36429 .89357 Mdot
.92579 .97332 Mdot
.29294 .39106 Mdot
.66485 .56724 Mdot
.07956 .82956 Mdot
.54502 .66339 Mdot
.16156 .637 Mdot
.79388 .43721 Mdot
.44946 .55112 Mdot
.1381 .33019 Mdot
.94987 .70725 Mdot
.34966 .69515 Mdot
.60939 .79088 Mdot
.40006 .69903 Mdot
.34026 .42363 Mdot
.7114 .15559 Mdot
.28451 .57128 Mdot
.19019 .46941 Mdot
.14676 .91148 Mdot
.3725 .05601 Mdot
.67349 .38417 Mdot
.25821 .70302 Mdot
.6998 .65412 Mdot
.88474 .03168 Mdot
.11423 .84044 Mdot
.50849 .30985 Mdot
.75016 .44062 Mdot
.77327 .17807 Mdot
.48946 .84654 Mdot
.60689 .68587 Mdot
.36651 .91226 Mdot
.25821 .65367 Mdot
.66921 .5519 Mdot
.97619 .92785 Mdot
.9456 .87499 Mdot
.11526 .91998 Mdot
.85518 .05835 Mdot
.58296 .63393 Mdot
.12883 .59494 Mdot
.78588 .47967 Mdot
.61557 .7256 Mdot
.2028 .77101 Mdot
.27287 .79053 Mdot
.92078 .14115 Mdot
.57985 .26244 Mdot
.92078 .19051 Mdot
.61044 .36466 Mdot
.82933 .24773 Mdot
.73145 .33011 Mdot
.27018 .591 Mdot
.62643 .71238 Mdot
.46049 .13514 Mdot
.03467 .96398 Mdot
.2815 .34133 Mdot
.73799 .19726 Mdot
.33691 .22399 Mdot
.18195 .91202 Mdot
.39232 .05729 Mdot
.5477 .57117 Mdot
.53918 .78677 Mdot
.79243 .26487 Mdot
.29281 .21957 Mdot
.18981 .5297 Mdot
.80852 .10825 Mdot
.17895 .54292 Mdot
.55083 .74412 Mdot
.41716 .36947 Mdot
.23773 .54394 Mdot
.25902 .43465 Mdot
.8216 .51045 Mdot
.68751 .84068 Mdot
.30623 .70089 Mdot
.87127 .59962 Mdot
.03723 .50513 Mdot
.70527 .09373 Mdot
.2049 .42069 Mdot
.55012 .52802 Mdot
.63026 .65377 Mdot
.15678 .18236 Mdot
.41634 .13364 Mdot
.87395 .72491 Mdot
.57093 .60039 Mdot
.21025 .86143 Mdot
.2885 .8767 Mdot
.31517 .28563 Mdot
.27508 .39538 Mdot
.58609 .2157 Mdot
.09399 .9519 Mdot
.05978 .66489 Mdot
.43992 .32194 Mdot
.8792 .50634 Mdot
.04739 .2121 Mdot
.02906 .75863 Mdot
.45265 .58891 Mdot
.795 .8744 Mdot
.18796 .68941 Mdot
.50364 .61259 Mdot
.88906 .31784 Mdot
.89374 .42069 Mdot
.81888 .34314 Mdot
.85777 .733 Mdot
.40277 .04502 Mdot
.95476 .25047 Mdot
.37919 .81012 Mdot
.94952 .46904 Mdot
.90273 .24501 Mdot
.17832 .57184 Mdot
.73858 .5328 Mdot
.65087 .93645 Mdot
.82571 .23877 Mdot
.73332 .53957 Mdot
.03064 .87283 Mdot
.85175 .78377 Mdot
.60406 .85163 Mdot
.87318 .55711 Mdot
.24868 .06532 Mdot
.89985 .11188 Mdot
.32214 .79751 Mdot
.74534 .51724 Mdot
.55975 .28852 Mdot
.11827 .55799 Mdot
.71024 .07355 Mdot
.36114 .04223 Mdot
.70341 .17792 Mdot
.48558 .23567 Mdot
.12316 .3035 Mdot
.5886 .65576 Mdot
.85067 .26198 Mdot
.66494 .56769 Mdot
.55234 .44168 Mdot
.89579 .07426 Mdot
.96878 .17697 Mdot
.80133 .49346 Mdot
.28235 .12723 Mdot
.46399 .47504 Mdot
.55513 .92651 Mdot
.9546 .26318 Mdot
.45578 .64682 Mdot
.38981 .58462 Mdot
.5813 .40865 Mdot
.70107 .04074 Mdot
.05278 .94417 Mdot
.78147 .94368 Mdot
.06019 .79101 Mdot
.95633 .47403 Mdot
.75403 .6876 Mdot
.51615 .97619 Mdot
.22271 .73829 Mdot
.53774 .73682 Mdot
.74311 .11528 Mdot
.17173 .17601 Mdot
.18562 .68384 Mdot
.44686 .15908 Mdot
.15665 .71687 Mdot
.64158 .1926 Mdot
.12027 .90306 Mdot
.66144 .69577 Mdot
.34243 .23927 Mdot
.1691 .69679 Mdot
.14353 .47818 Mdot
.60755 .93717 Mdot
.37661 .38671 Mdot
.45963 .78497 Mdot
.2148 .68007 Mdot
.03658 .6497 Mdot
.08196 .9404 Mdot
.37119 .48091 Mdot
.93788 .06116 Mdot
.68595 .76233 Mdot
.61926 .79909 Mdot
.54066 .08936 Mdot
.49953 .34472 Mdot
.9093 .12939 Mdot
.14673 .93521 Mdot
.47348 .32162 Mdot
.90812 .27895 Mdot
.4607 .64913 Mdot
.84997 .31575 Mdot
.11332 .19203 Mdot
.88828 .2784 Mdot
.40356 .4069 Mdot
.29283 .45652 Mdot
.83909 .34135 Mdot
.76949 .13561 Mdot
.90599 .23577 Mdot
.64657 .1776 Mdot
.45632 .89134 Mdot
.71464 .87585 Mdot
.9718 .26603 Mdot
.84086 .58391 Mdot
.88229 .09781 Mdot
.92877 .32931 Mdot
.50254 .66811 Mdot
.65975 .85 Mdot
.63964 .35057 Mdot
.86645 .7382 Mdot
.70984 .13862 Mdot
.24369 .58441 Mdot
.27734 .22447 Mdot
.50524 .68577 Mdot
.28173 .93565 Mdot
.64057 .12567 Mdot
.37563 .86165 Mdot
.68799 .77356 Mdot
.84928 .21735 Mdot
.05206 .90077 Mdot
.23345 .84398 Mdot
.1618 .18638 Mdot
.49979 .72917 Mdot
.8943 .57917 Mdot
.24627 .52851 Mdot
.41287 .8706 Mdot
.94073 .57006 Mdot
.74849 .76874 Mdot
.58891 .68561 Mdot
.08431 .97238 Mdot
.71583 .49206 Mdot
.05606 .09542 Mdot
.50619 .62528 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  ImageSize->{111.75, 111.75},
  ImageMargins->{{33.125, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40001_0000Ka000`40O003h00OKgoo001_Ool006mo
o`005Goo00<007ooOol0Egoo000>Ool00`00Oomoo`0WOol00`00Oomoo`0VOol00`00Oomoo`0;Ool0
02Moo`03001oogoo009oo`03001oogoo01ioo`03001oogoo01moo`008Goo00<007ooOol017oo00@0
07ooOol0019oo`03001oogoo021oo`03001oogoo00]oo`006goo00<007ooOol0@7oo00<007ooOol0
1Woo00<007ooOol01Goo000:Ool00`00Oomoo`0IOol00`00Oomoo`0UOol00`00Oomoo`0AOol00`00
Oomoo`06Ool00`00Oomoo`01Ool0009oo`03001oo`0006Yoo`00;Woo00<007oo00002Woo00<007oo
Ool00goo00<007ooOol02Goo0P005goo00<007ooOol01Woo0005Ool00`00Oomoo`0bOol00`00Oomo
o`0TOol00`00Oomoo`04Ool00`00Oomoo`04Ool004]oo`03001oogoo00]oo`03001oogoo00Moo`03
001oogoo00Uoo`00?7oo00<007ooOol04Woo00<007ooOol05Woo00<007ooOol00Woo000NOol00`00
Oomoo`0UOol00`00Oomoo`0KOol00`00Oomoo`08Ool001Uoo`03001oogoo015oo`03001oogoo009o
o`03001oogoo01Eoo`05001oogooOol00005Ool00`00Oomoo`0HOol000Moo`03001oogoo05]oo`03
001oogoo00Moo`008Woo00<007ooOol0:7oo0P006goo00<007ooOol00Woo000COol00`00Oomoo`04
Ool00`00Oomoo`0DOol00`00Oomoo`0kOol0011oo`03001oogoo04=oo`03001oogoo01Ioo`004Goo
0P00=7oo00<007ooOol00goo00<007ooOol01Goo0P004goo00<007ooOol00Woo000<Ool00`00Oomo
o`02Ool00`00Oomoo`0LOol00`00Oomoo`0COol00`00Oomoo`0LOol00`00Oomoo`07Ool0055oo`03
001oogoo01]oo`008Woo00<007ooOol047oo00<007ooOol01Goo00<007ooOol09Goo00<007ooOol0
1goo0004Ool00`00Oomoo`0IOol00`00Oomoo`0@Ool00`00Oomoo`0:Ool00`00Oomoo`0BOol00`00
Oomoo`05Ool00`00Oomoo`02Ool00`00Oomoo`0:Ool000Aoo`03001oogoo00Uoo`03001oogoo00]o
o`03001oogoo00Aoo`03001oogoo00]oo`03001oogoo03Uoo`00=Goo00<007ooOol08goo00<007oo
Ool01Woo00<007ooOol027oo000<Ool00`00Oomoo`0FOol00`00Oomoo`0:Ool00`00Oomoo`0VOol0
0`00Oomoo`05Ool00`00Oomoo`09Ool004moo`8001Qoo`03001oogoo00=oo`008Woo00<007ooOol0
1Woo00<007ooOol01goo00<007ooOol01goo00<007ooOol05Goo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol00goo000HOol2001AOol00`00Oomoo`01Ool000]oo`03001oogoo05Aoo`03001o
o`0000Yoo`008Woo00<007ooOol067oo00<007ooOol0;goo001LOol00`00Oomoo`0@Ool000eoo`03
001oogoo02Qoo`03001oogoo03Aoo`003Woo00<007ooOol0>7oo0P0017oo00<007ooOol02goo00<0
07ooOol00Woo00<007ooOol02Woo000`Ool00`00Oomoo`02000jOol000ioo`03001oogoo01ioo`03
001oogoo01ioo`03001oogoo01=oo`03001oogoo00Ioo`003Goo00<007ooOol01goo00<007ooOol0
17oo00<007ooOol00Woo00<007ooOol04Goo00<007ooOol087oo00<007oo00004Woo0005Ool00`00
Oomoo`0LOol00`00Oomoo`0OOol00`00Oomoo`0VOol002Ioo`03001oogoo00Aoo`03001oogoo01=o
o`03001oogoo02Uoo`00;Goo00<007ooOol0?goo000<Ool00`00Oomoo`0bOol00`00Oomoo`04Ool0
0`00Oomoo`0TOol0021oo`8000Moo`03001oogoo01ioo`03001oogoo029oo`007Woo00@007ooOol0
03=oo`03001oogoo01Moo`001Woo00<007ooOol01Goo00<007ooOol06goo00<007ooOol04Goo00<0
07ooOol0;7oo000FOol00`00Oomoo`0?Ool00`00Oomoo`0=Ool00`00Oomoo`0dOol000Aoo`03001o
ogoo00moo`05001oogooOol0000:Ool00`00Oomoo`09Ool00`00Oomoo`0^Ool00`00Oomoo`0:Ool0
01aoo`04001oogoo000gOol00`00Oomoo`05Ool00`00Oomoo`0=Ool003aoo`03001oogoo01=oo`03
001oogoo01Yoo`00Kgoo000POol00`00Oomoo`0@Ool00`00Oomoo`0iOol001Aoo`03001oogoo02Mo
o`03001oogoo00moo`03001oogoo01Eoo`03001oogoo00Aoo`003goo00<007ooOol01goo00<007oo
Ool037oo00<007ooOol027oo00<007ooOol00goo00<007ooOol06Woo00<007ooOol047oo00<007oo
Ool00goo000VOol00`00Oomoo`0HOol00`00Oomoo`0[Ool002Eoo`03001oogoo02Ioo`03001oogoo
00Moo`03001oogoo00Moo`03001oogoo00Yoo`000goo00<007ooOol0Fgoo00<007ooOol02goo001B
Ool00`00Oomoo`05Ool00`00Oomoo`0BOol000Yoo`03001oo`0000Aoo`03001oogoo02]oo`03001o
ogoo01aoo`8000moo`0057oo00<007ooOol00goo00<007ooOol027oo00<007ooOol057oo00<007oo
Ool04Woo00<007ooOol06goo000COol00`00Oomoo`03Ool2000eOol00`00Oomoo`0LOol002Ioo`03
001oogoo00Qoo`03001oogoo01Eoo`03001oogoo02=oo`001goo00<007ooOol00Woo00@007ooOol0
01Ioo`03001oogoo02Yoo`03001oogoo00Yoo`03001oogoo00aoo`004goo00<007ooOol02Goo00<0
07ooOol06Woo00<007ooOol02Woo00<007ooOol06goo00<007ooOol01Goo0016Ool00`00Oomoo`0I
Ool00`00Oomoo`0:Ool001Yoo`03001oogoo00eoo`05001oogooOol00002Ool00`00Oomoo`0XOol0
0`00Oomoo`0@Ool000eoo`03001oogoo00eoo`03001oogoo015oo`03001oogoo02Eoo`03001oogoo
00Aoo`03001oogoo00aoo`0067oo00<007ooOol08goo00<007ooOol0;Woo000/Ool00`00Oomoo`08
Ool00`00Oomoo`0JOol00`00Oomoo`02Ool00`00Oomoo`0COol0015oo`03001oogoo02=oo`03001o
ogoo03Eoo`007goo00<007ooOol07Woo00<007ooOol0;7oo000@Ool2000YOol00`00Oomoo`0aOol0
00=oo`03001oogoo01Ioo`03001oogoo01=oo`03001oogoo011oo`03001oogoo02Moo`006Woo00<0
07ooOol03Woo00<007oo00004Woo00<007ooOol02Woo00<007ooOol00goo00<007ooOol06Goo0006
Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`02Ool00`00Oomoo`0HOol00`00Ool00002
Ool00`00Oomoo`0`Ool001Moo`03001oogoo00moo`03001oogoo04=oo`0057oo00<007ooOol087oo
00<007ooOol01Woo00<007oo000047oo00<007ooOol06Goo000BOol00`00Oomoo`0AOol00`00Oomo
o`02Ool00`00Oomoo`0JOol00`00Oomoo`0TOol001aoo`03001oogoo009oo`03001oogoo04Aoo`03
001oogoo00Aoo`0047oo0P00<goo00<007ooOol09goo000?Ool00`00Oomoo`08Ool2000WOol00`00
Oomoo`06Ool00`00Oomoo`0AOol00`00Oomoo`0<Ool003Moo`03001oogoo009oo`03001oogoo01mo
o`03001oogoo00ioo`0067oo00<007ooOol087oo0P008Woo0P003Woo0008Ool00`00Oomoo`0>Ool0
0`00Oomoo`0ZOol20005Ool00`00Oomoo`0OOol0009oo`03001oogoo04Ioo`03001oogoo025oo`00
5Woo00<007ooOol0>Goo00<007ooOol06Woo001;Ool00`00Oomoo`0AOol01000Oomoo`0037oo000R
Ool00`00Oomoo`0=Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`0EOol00`00Oomoo`06
Ool00`00Oomoo`05Ool000Ioo`03001oogoo01Aoo`03001oogoo00=oo`04001oogoo000LOol01000
Oomoo`00:7oo0014Ool00`00Oomoo`0DOol00`00Oomoo`0AOol002Uoo`03001oogoo02Qoo`03001o
ogoo01Qoo`0067oo00<007ooOol06Goo00<007ooOol0;Woo00<007ooOol01goo0008Ool2001UOol0
00aoo`03001oogoo03Yoo`03001oo`0001moo`03001oogoo005oo`006Goo00<007ooOol06Goo00<0
07ooOol02Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol04Woo00<0
07ooOol01Goo000:Ool00`00Oomoo`09Ool00`00Oomoo`0=Ool01000Oomoo`00?Goo00<007ooOol0
1Goo000]Ool00`00Oomoo`0MOol00`00Oomoo`0OOol000=oo`03001oogoo00eoo`03001oogoo00Uo
o`07001oogooOol007oo0000:7oo00<007ooOol01Woo00<007ooOol03Woo00<007ooOol017oo000?
Ool00`00Oomoo`05Ool00`00Oomoo`0lOol00`00Oomoo`0FOol002Qoo`03001oogoo00Moo`03001o
ogoo03Yoo`001Goo00<007ooOol01Goo0P00H7oo0009Ool00`00Oomoo`03Ool01@00Oomoogoo0000
57oo00<007ooOol05Goo00<007ooOol0;7oo000<Ool00`00Oomoo`1POol003eoo`03001oogoo02]o
o`03001oogoo005oo`0027oo00<007oo000017oo00<007ooOol037oo00<007ooOol07goo00<007oo
000017oo00<007ooOol09Goo0005Ool00`00Oomoo`02Ool00`00Oomoo`0TOol00`00Oomoo`0LOol0
0`00Oomoo`03Ool00`00Oomoo`0FOol0009oo`03001oogoo06Yoo`000goo00<007ooOol0EWoo00<0
07ooOol047oo0009Ool00`00Oomoo`0/Ool00`00Oomoo`0XOol01000Oomoo`0027oo001_Ool006mo
o`00\
\>"],
  ImageRangeCache->{{{0, 110.75}, {110.75, 0}} -> {-0.0250052, -0.0249787, 
  0.00948091, 0.00947085}}]
}, Open  ]],

Cell[TextData[{
  "There are both clumps and gaps! So it is of some interest to remedy this. \
Of course one can use a grid-like system to make the points more evenly \
distributed, but this requires a very large number of points in high \
dimensions. Wozniakowski (1991) has given an algorithm for numerical \
integration on a hypercube based on making a distribution of points that is \
as even as possible. The locations of the points he uses have been known for \
some time, but Wozniakowski was the first to demonstrate the power \"on \
average\" of the use of such points in integration. Although this \
illustration is only based on a square, a particularly important feature of \
his approach is that it remains viable in integration problems with very high \
dimension. \n\nCentral to the construction is the idea of representing an \
integer in the base of the ",
  Cell[BoxData[
      \(TraditionalForm\`j\)]],
  "th prime number. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " contains all the ingredients we need. The function ",
  StyleBox["IntegerDigits",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " computes the digits of a number in a given base, while ",
  StyleBox["Prime[j]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " returns the ",
  Cell[BoxData[
      \(TraditionalForm\`j\)]],
  "th prime. In two dimensions we only need the first prime, 2. Let's take a \
look at the ",
  StyleBox["IntegerDigits",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function for this case:"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True],

Cell["a[k_Integer] := IntegerDigits[k,2]", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["a[38]", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1, 0, 0, 1, 1, 0}\)], "Output"]
}, Open  ]],

Cell["This corresponds to the binary expansion:", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["1*32 + 0*16 + 0*8 + 1*4 + 1*2 + 0*1", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(38\)], "Output"]
}, Open  ]],

Cell["\<\
The function we need is the \"radical inverse function\", where we \
take the expansion and replace positive powers by negative powers.\
\>", 
  "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
phi[k_] := Module[{b=a[k], len},
len = Length[b];
Sum[b[[len-i]]/2^(i+1), {i,0,len-1}]]\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["phi[38]", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(25\/64\)], "Output"]
}, Open  ]],

Cell["This arises as", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["1/2^6 + 0/2^5 + 0/2^4 + 1/2^3 + 1/2^2 + 0/2", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(25\/64\)], "Output"]
}, Open  ]],

Cell["\<\
Using this function, we can build our nicely spread-out points \
thus:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["X[n_] := Table[{1 - k/(n+1), phi[k]}, {k, 1, n}]", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["Let's take a look.", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["S = X[400];", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Show[Graphics[{PointSize[0.015], Map[ Point[#]&, 
S]}], AspectRatio -> 1];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 7  scalefont  setfont
% Scaling calculations
0.0238095 0.954762 0.0238095 0.956116 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 g
.015 w
.97619 .50187 Mdot
.97381 .26284 Mdot
.97143 .7409 Mdot
.96905 .14332 Mdot
.96667 .62138 Mdot
.96429 .38235 Mdot
.9619 .86041 Mdot
.95952 .08357 Mdot
.95714 .56162 Mdot
.95476 .3226 Mdot
.95238 .80065 Mdot
.95 .20308 Mdot
.94762 .68114 Mdot
.94524 .44211 Mdot
.94286 .92017 Mdot
.94048 .05369 Mdot
.9381 .53175 Mdot
.93571 .29272 Mdot
.93333 .77077 Mdot
.93095 .1732 Mdot
.92857 .65126 Mdot
.92619 .41223 Mdot
.92381 .89029 Mdot
.92143 .11345 Mdot
.91905 .5915 Mdot
.91667 .35247 Mdot
.91429 .83053 Mdot
.9119 .23296 Mdot
.90952 .71102 Mdot
.90714 .47199 Mdot
.90476 .95005 Mdot
.90238 .03875 Mdot
.9 .51681 Mdot
.89762 .27778 Mdot
.89524 .75584 Mdot
.89286 .15826 Mdot
.89048 .63632 Mdot
.8881 .39729 Mdot
.88571 .87535 Mdot
.88333 .09851 Mdot
.88095 .57656 Mdot
.87857 .33754 Mdot
.87619 .81559 Mdot
.87381 .21802 Mdot
.87143 .69608 Mdot
.86905 .45705 Mdot
.86667 .93511 Mdot
.86429 .06863 Mdot
.8619 .54669 Mdot
.85952 .30766 Mdot
.85714 .78571 Mdot
.85476 .18814 Mdot
.85238 .6662 Mdot
.85 .42717 Mdot
.84762 .90523 Mdot
.84524 .12838 Mdot
.84286 .60644 Mdot
.84048 .36741 Mdot
.8381 .84547 Mdot
.83571 .2479 Mdot
.83333 .72596 Mdot
.83095 .48693 Mdot
.82857 .96499 Mdot
.82619 .03128 Mdot
.82381 .50934 Mdot
.82143 .27031 Mdot
.81905 .74837 Mdot
.81667 .15079 Mdot
.81429 .62885 Mdot
.8119 .38982 Mdot
.80952 .86788 Mdot
.80714 .09104 Mdot
.80476 .56909 Mdot
.80238 .33007 Mdot
.8 .80812 Mdot
.79762 .21055 Mdot
.79524 .68861 Mdot
.79286 .44958 Mdot
.79048 .92764 Mdot
.7881 .06116 Mdot
.78571 .53922 Mdot
.78333 .30019 Mdot
.78095 .77824 Mdot
.77857 .18067 Mdot
.77619 .65873 Mdot
.77381 .4197 Mdot
.77143 .89776 Mdot
.76905 .12092 Mdot
.76667 .59897 Mdot
.76429 .35994 Mdot
.7619 .838 Mdot
.75952 .24043 Mdot
.75714 .71849 Mdot
.75476 .47946 Mdot
.75238 .95752 Mdot
.75 .04622 Mdot
.74762 .52428 Mdot
.74524 .28525 Mdot
.74286 .76331 Mdot
.74048 .16573 Mdot
.7381 .64379 Mdot
.73571 .40476 Mdot
.73333 .88282 Mdot
.73095 .10598 Mdot
.72857 .58403 Mdot
.72619 .345 Mdot
.72381 .82306 Mdot
.72143 .22549 Mdot
.71905 .70355 Mdot
.71667 .46452 Mdot
.71429 .94258 Mdot
.7119 .0761 Mdot
.70952 .55415 Mdot
.70714 .31513 Mdot
.70476 .79318 Mdot
.70238 .19561 Mdot
.7 .67367 Mdot
.69762 .43464 Mdot
.69524 .9127 Mdot
.69286 .13585 Mdot
.69048 .61391 Mdot
.6881 .37488 Mdot
.68571 .85294 Mdot
.68333 .25537 Mdot
.68095 .73343 Mdot
.67857 .4944 Mdot
.67619 .97246 Mdot
.67381 .02754 Mdot
.67143 .5056 Mdot
.66905 .26657 Mdot
.66667 .74463 Mdot
.66429 .14706 Mdot
.6619 .62512 Mdot
.65952 .38609 Mdot
.65714 .86415 Mdot
.65476 .0873 Mdot
.65238 .56536 Mdot
.65 .32633 Mdot
.64762 .80439 Mdot
.64524 .20682 Mdot
.64286 .68487 Mdot
.64048 .44585 Mdot
.6381 .9239 Mdot
.63571 .05742 Mdot
.63333 .53548 Mdot
.63095 .29645 Mdot
.62857 .77451 Mdot
.62619 .17694 Mdot
.62381 .655 Mdot
.62143 .41597 Mdot
.61905 .89402 Mdot
.61667 .11718 Mdot
.61429 .59524 Mdot
.6119 .35621 Mdot
.60952 .83427 Mdot
.60714 .23669 Mdot
.60476 .71475 Mdot
.60238 .47572 Mdot
.6 .95378 Mdot
.59762 .04248 Mdot
.59524 .52054 Mdot
.59286 .28151 Mdot
.59048 .75957 Mdot
.5881 .162 Mdot
.58571 .64006 Mdot
.58333 .40103 Mdot
.58095 .87908 Mdot
.57857 .10224 Mdot
.57619 .5803 Mdot
.57381 .34127 Mdot
.57143 .81933 Mdot
.56905 .22176 Mdot
.56667 .69981 Mdot
.56429 .46078 Mdot
.5619 .93884 Mdot
.55952 .07236 Mdot
.55714 .55042 Mdot
.55476 .31139 Mdot
.55238 .78945 Mdot
.55 .19188 Mdot
.54762 .66993 Mdot
.54524 .43091 Mdot
.54286 .90896 Mdot
.54048 .13212 Mdot
.5381 .61018 Mdot
.53571 .37115 Mdot
.53333 .84921 Mdot
.53095 .25163 Mdot
.52857 .72969 Mdot
.52619 .49066 Mdot
.52381 .96872 Mdot
.52143 .03501 Mdot
.51905 .51307 Mdot
.51667 .27404 Mdot
.51429 .7521 Mdot
.5119 .15453 Mdot
.50952 .63259 Mdot
.50714 .39356 Mdot
.50476 .87162 Mdot
.50238 .09477 Mdot
.5 .57283 Mdot
.49762 .3338 Mdot
.49524 .81186 Mdot
.49286 .21429 Mdot
.49048 .69234 Mdot
.4881 .45331 Mdot
.48571 .93137 Mdot
.48333 .06489 Mdot
.48095 .54295 Mdot
.47857 .30392 Mdot
.47619 .78198 Mdot
.47381 .18441 Mdot
.47143 .66246 Mdot
.46905 .42344 Mdot
.46667 .90149 Mdot
.46429 .12465 Mdot
.4619 .60271 Mdot
.45952 .36368 Mdot
.45714 .84174 Mdot
.45476 .24416 Mdot
.45238 .72222 Mdot
.45 .48319 Mdot
.44762 .96125 Mdot
.44524 .04995 Mdot
.44286 .52801 Mdot
.44048 .28898 Mdot
.4381 .76704 Mdot
.43571 .16947 Mdot
.43333 .64753 Mdot
.43095 .4085 Mdot
.42857 .88655 Mdot
.42619 .10971 Mdot
.42381 .58777 Mdot
.42143 .34874 Mdot
.41905 .8268 Mdot
.41667 .22923 Mdot
.41429 .70728 Mdot
.4119 .46825 Mdot
.40952 .94631 Mdot
.40714 .07983 Mdot
.40476 .55789 Mdot
.40238 .31886 Mdot
.4 .79692 Mdot
.39762 .19935 Mdot
.39524 .6774 Mdot
.39286 .43838 Mdot
.39048 .91643 Mdot
.3881 .13959 Mdot
.38571 .61765 Mdot
.38333 .37862 Mdot
.38095 .85668 Mdot
.37857 .2591 Mdot
.37619 .73716 Mdot
.37381 .49813 Mdot
.37143 .97619 Mdot
.36905 .02568 Mdot
.36667 .50373 Mdot
.36429 .26471 Mdot
.3619 .74276 Mdot
.35952 .14519 Mdot
.35714 .62325 Mdot
.35476 .38422 Mdot
.35238 .86228 Mdot
.35 .08543 Mdot
.34762 .56349 Mdot
.34524 .32446 Mdot
.34286 .80252 Mdot
.34048 .20495 Mdot
.3381 .68301 Mdot
.33571 .44398 Mdot
.33333 .92204 Mdot
.33095 .05556 Mdot
.32857 .53361 Mdot
.32619 .29458 Mdot
.32381 .77264 Mdot
.32143 .17507 Mdot
.31905 .65313 Mdot
.31667 .4141 Mdot
.31429 .89216 Mdot
.3119 .11531 Mdot
.30952 .59337 Mdot
.30714 .35434 Mdot
.30476 .8324 Mdot
.30238 .23483 Mdot
.3 .71289 Mdot
.29762 .47386 Mdot
.29524 .95191 Mdot
.29286 .04062 Mdot
.29048 .51867 Mdot
.2881 .27965 Mdot
.28571 .7577 Mdot
.28333 .16013 Mdot
.28095 .63819 Mdot
.27857 .39916 Mdot
.27619 .87722 Mdot
.27381 .10037 Mdot
.27143 .57843 Mdot
.26905 .3394 Mdot
.26667 .81746 Mdot
.26429 .21989 Mdot
.2619 .69795 Mdot
.25952 .45892 Mdot
.25714 .93697 Mdot
.25476 .07049 Mdot
.25238 .54855 Mdot
.25 .30952 Mdot
.24762 .78758 Mdot
.24524 .19001 Mdot
.24286 .66807 Mdot
.24048 .42904 Mdot
.2381 .9071 Mdot
.23571 .13025 Mdot
.23333 .60831 Mdot
.23095 .36928 Mdot
.22857 .84734 Mdot
.22619 .24977 Mdot
.22381 .72782 Mdot
.22143 .4888 Mdot
.21905 .96685 Mdot
.21667 .03315 Mdot
.21429 .5112 Mdot
.2119 .27218 Mdot
.20952 .75023 Mdot
.20714 .15266 Mdot
.20476 .63072 Mdot
.20238 .39169 Mdot
.2 .86975 Mdot
.19762 .0929 Mdot
.19524 .57096 Mdot
.19286 .33193 Mdot
.19048 .80999 Mdot
.1881 .21242 Mdot
.18571 .69048 Mdot
.18333 .45145 Mdot
.18095 .92951 Mdot
.17857 .06303 Mdot
.17619 .54108 Mdot
.17381 .30205 Mdot
.17143 .78011 Mdot
.16905 .18254 Mdot
.16667 .6606 Mdot
.16429 .42157 Mdot
.1619 .89963 Mdot
.15952 .12278 Mdot
.15714 .60084 Mdot
.15476 .36181 Mdot
.15238 .83987 Mdot
.15 .2423 Mdot
.14762 .72035 Mdot
.14524 .48133 Mdot
.14286 .95938 Mdot
.14048 .04809 Mdot
.1381 .52614 Mdot
.13571 .28711 Mdot
.13333 .76517 Mdot
.13095 .1676 Mdot
.12857 .64566 Mdot
.12619 .40663 Mdot
.12381 .88469 Mdot
.12143 .10784 Mdot
.11905 .5859 Mdot
.11667 .34687 Mdot
.11429 .82493 Mdot
.1119 .22736 Mdot
.10952 .70542 Mdot
.10714 .46639 Mdot
.10476 .94444 Mdot
.10238 .07796 Mdot
.1 .55602 Mdot
.09762 .31699 Mdot
.09524 .79505 Mdot
.09286 .19748 Mdot
.09048 .67554 Mdot
.0881 .43651 Mdot
.08571 .91457 Mdot
.08333 .13772 Mdot
.08095 .61578 Mdot
.07857 .37675 Mdot
.07619 .85481 Mdot
.07381 .25724 Mdot
.07143 .73529 Mdot
.06905 .49627 Mdot
.06667 .97432 Mdot
.06429 .02941 Mdot
.0619 .50747 Mdot
.05952 .26844 Mdot
.05714 .7465 Mdot
.05476 .14893 Mdot
.05238 .62698 Mdot
.05 .38796 Mdot
.04762 .86601 Mdot
.04524 .08917 Mdot
.04286 .56723 Mdot
.04048 .3282 Mdot
.0381 .80626 Mdot
.03571 .20868 Mdot
.03333 .68674 Mdot
.03095 .44771 Mdot
.02857 .92577 Mdot
.02619 .05929 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  ImageSize->{111.75, 111.75},
  ImageMargins->{{33.125, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40001_0000Ka000`40O003h00OKgoo001_Ool006mo
o`001Woo00<007ooOol07goo00<007ooOol07goo00<007ooOol08Woo000GOol00`00Oomoo`0OOol0
0`00Oomoo`0OOol00`00Oomoo`05Ool00`00Oomoo`09Ool000moo`03001oogoo00ioo`03001oogoo
01ioo`03001oogoo00ioo`03001oogoo01Yoo`0097oo00<007ooOol02Goo00<007ooOol04goo00<0
07ooOol07Woo00<007ooOol01Goo0002Ool00`00Oomoo`0>Ool00`00Oomoo`0OOol00`00Oomoo`0N
Ool00`00Oomoo`0FOol001]oo`03001oogoo01moo`03001oogoo00ioo`03001oogoo00ioo`03001o
ogoo00eoo`002goo00<007ooOol07Woo00<007ooOol0>Woo00<007ooOol00goo0004Ool00`00Oomo
o`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0OOol00`00Oomoo`0>Ool00`00Oomoo`0COol001ioo`03
001oogoo01Ioo`03001oogoo00Eoo`03001oogoo01moo`03001oogoo00]oo`003Goo00<007ooOol0
7Woo00<007ooOol07goo00<007ooOol077oo000ROol00`00Oomoo`0NOol00`00Oomoo`0>Ool00`00
Oomoo`0>Ool00`00Oomoo`07Ool0015oo`03001oogoo00Eoo`03001oogoo01Moo`03001oogoo02Mo
o`03001oogoo00moo`0027oo00<007ooOol0<7oo00<007ooOol03Woo00<007ooOol087oo000WOol0
1000Oomoo`007Woo00<007ooOol07goo00<007ooOol00Goo0005Ool00`00Oomoo`0>Ool00`00Oomo
o`0OOol00`00Oomoo`0OOol00`00Oomoo`0BOol001moo`03001oogoo01ioo`03001oogoo00ioo`03
001oogoo00ioo`03001oogoo00Yoo`003Woo00<007ooOol04Woo00<007ooOol02Goo00<007ooOol0
=7oo00<007ooOol01Woo000BOol00`00Oomoo`0OOol00`00Oomoo`0>Ool00`00Oomoo`0=Ool00`00
Oomoo`0GOol001Yoo`03001oogoo01moo`03001oogoo01moo`03001oogoo00ioo`002Goo00<007oo
Ool07goo00<007ooOol07goo00<007ooOol07goo0003Ool00`00Oomoo`0OOol00`00Oomoo`0OOol0
0`00Oomoo`0>Ool00`00Oomoo`0=Ool00`00Oomoo`04Ool001Aoo`03001oogoo00Eoo`03001oogoo
01Moo`03001oogoo02Moo`03001oogoo00aoo`0037oo00<007ooOol07Woo00<007ooOol03Woo00<0
07ooOol03Woo00<007ooOol07Goo000QOol00`00Oomoo`0NOol00`00Oomoo`0OOol00`00Oomoo`08
Ool0011oo`03001oogoo01moo`03001oogoo01ioo`03001oogoo01Uoo`0067oo00<007ooOol07goo
00<007ooOol03Woo00<007ooOol03Woo00<007ooOol047oo0007Ool00`00Oomoo`0NOol20011Ool0
0`00Oomoo`01Ool000Ioo`03001oogoo00ioo`03001oogoo01ioo`03001oogoo00ioo`03001oogoo
00ioo`03001oogoo019oo`007goo00<007ooOol07goo00<007ooOol07goo00<007ooOol02Goo000>
Ool00`00Oomoo`0OOol00`00Oomoo`0OOol00`00Oomoo`0BOol00`00Oomoo`05Ool002=oo`03001o
ogoo01moo`03001oogoo00ioo`03001oogoo01Ioo`004Woo00<007ooOol01Woo00<007ooOol05Woo
00<007ooOol09goo00<007ooOol03Woo000:Ool00`00Oomoo`0OOol00`00Oomoo`0>Ool00`00Oomo
o`0>Ool00`00Oomoo`0NOol000Aoo`03001oogoo01ioo`03001oogoo01moo`03001oogoo01moo`03
001oogoo00=oo`005Goo00<007ooOol07Woo00<007ooOol07goo00<007ooOol01Goo00<007ooOol0
37oo000<Ool00`00Oomoo`0>Ool00`00Oomoo`0OOol00`00Oomoo`0>Ool00`00Oomoo`0LOol0025o
o`03001oogoo00Yoo`03001oogoo03Aoo`03001oogoo00Moo`0047oo00<007ooOol07goo00<007oo
Ool03Woo00<007ooOol03Woo00<007ooOol067oo000IOol00`00Oomoo`0OOol00`00Oomoo`0NOol0
0`00Oomoo`0@Ool000Qoo`03001oogoo01moo`03001oogoo01ioo`03001oogoo01aoo`03001oogoo
009oo`001Goo00<007ooOol03Woo00<007ooOol03Goo00<007ooOol07goo00<007ooOol03Woo00<0
07ooOol04goo000NOol00`00Oomoo`0FOol00`00Oomoo`06Ool00`00Oomoo`0OOol00`00Oomoo`0:
Ool000eoo`03001oogoo01moo`03001oogoo01moo`03001oogoo01]oo`008Woo00<007ooOol07goo
00<007ooOol07goo00<007ooOol01Woo000AOol00`00Oomoo`0OOol00`00Oomoo`0OOol00`00Oomo
o`05Ool00`00Oomoo`0?Ool000Uoo`03001oogoo00ioo`03001oogoo01moo`03001oogoo00eoo`03
001oogoo021oo`0097oo00<007ooOol017oo00<007ooOol067oo00<007ooOol07goo00<007ooOol0
17oo0003Ool00`00Oomoo`0=Ool00`00Oomoo`0OOol00`00Oomoo`0OOol00`00Oomoo`0EOol001ao
o`03001oogoo01moo`03001oogoo01ioo`03001oogoo00eoo`002goo00<007ooOol07goo00<007oo
Ool07goo00<007ooOol04Woo00<007ooOol027oo000?Ool00`00Oomoo`0>Ool00`00Oomoo`0OOol0
0`00Oomoo`0>Ool00`00Oomoo`0IOol001Qoo`03001oogoo01Ioo`03001oogoo00Eoo`03001oogoo
01moo`03001oogoo015oo`001goo00<007ooOol07goo00<007ooOol07Woo00<007ooOol08Woo0006
Ool00`00Oomoo`0OOol00`00Oomoo`0OOol00`00Oomoo`0=Ool00`00Oomoo`0>Ool00`00Oomoo`01
Ool001Moo`03001oogoo01moo`03001oogoo02Moo`03001oogoo00Uoo`003Woo00<007ooOol03Woo
00<007ooOol07goo00<007ooOol03Woo00<007ooOol06Woo000TOol00`00Oomoo`09Ool00`00Oomo
o`0BOol00`00Oomoo`0OOol00`00Oomoo`05Ool001=oo`03001oogoo01ioo`03001oogoo01moo`03
001oogoo01Ioo`006goo00<007ooOol07goo00<007ooOol03Woo00<007ooOol03Woo00<007ooOol0
3Goo000:Ool00`00Oomoo`0IOol00`00Oomoo`03Ool00`00Oomoo`0jOol00`00Oomoo`03Ool000Ao
o`03001oogoo00ioo`03001oogoo02moo`03001oogoo00ioo`03001oogoo01Aoo`007Goo00<007oo
Ool05goo00<007ooOol01Goo00<007ooOol07goo00<007ooOol02goo000<Ool00`00Oomoo`0OOol0
0`00Oomoo`0OOol00`00Oomoo`0LOol0025oo`03001oogoo01moo`03001oogoo00ioo`03001oogoo
00ioo`03001oogoo00Moo`004Goo00<007ooOol01Goo00<007ooOol05Woo00<007ooOol0:7oo00<0
07ooOol03goo0008Ool00`00Oomoo`0OOol00`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0P
Ool000Eoo`03001oogoo01moo`03001oogoo01ioo`03001oogoo01moo`03001oogoo009oo`005Woo
00<007ooOol07goo00<007ooOol07Woo00<007ooOol04goo000NOol00`00Oomoo`0OOol00`00Oomo
o`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0:Ool000eoo`03001oogoo01=oo`03001oogoo00Uoo`03
001oogoo03Aoo`03001oogoo00Ioo`004Woo00<007ooOol07Woo00<007ooOol03Woo00<007ooOol0
3Woo00<007ooOol05goo000JOol00`00Oomoo`0OOol00`00Oomoo`0OOol00`00Oomoo`0>Ool000Uo
o`03001oogoo01moo`03001oogoo01moo`03001oogoo01Qoo`03001oogoo00Aoo`000goo00<007oo
Ool07goo00<007ooOol07Woo00<007ooOol03Woo00<007ooOol05Goo000DOol00`00Oomoo`05Ool0
0`00Oomoo`0GOol00`00Oomoo`0WOol00`00Oomoo`0<Ool000]oo`03001oogoo01moo`03001oogoo
00ioo`03001oogoo00ioo`03001oogoo01eoo`0087oo00<007ooOol07goo00<007ooOol07goo00<0
07ooOol027oo000@Ool00`00Oomoo`0NOol00`00Oomoo`0OOol00`00Oomoo`05Ool00`00Oomoo`0A
Ool000Moo`03001oogoo00ioo`03001oogoo01moo`03001oogoo00ioo`03001oogoo025oo`009goo
00<007oo0000@Goo00<007ooOol00Goo0006Ool00`00Oomoo`0=Ool00`00Oomoo`0OOol00`00Oomo
o`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0BOol001moo`03001oogoo01moo`03001oogoo01ioo`03
001oogoo00Yoo`003Woo00<007ooOol07goo00<007ooOol07Woo00<007ooOol04goo00<007ooOol0
1Goo000BOol00`00Oomoo`0>Ool00`00Oomoo`0OOol00`00Oomoo`0>Ool00`00Oomoo`0FOol001]o
o`03001oogoo01Ioo`03001oogoo00Eoo`03001oogoo01moo`03001oogoo00ioo`002Woo00<007oo
Ool07Woo00<007ooOol07goo00<007ooOol07goo0003Ool00`00Oomoo`0OOol00`00Oomoo`0OOol0
0`00Oomoo`0OOol00`00Oomoo`03Ool001Aoo`03001oogoo01moo`03001oogoo01moo`03001oogoo
00Eoo`03001oogoo00aoo`0037oo00<007ooOol03Woo00<007ooOol07Woo00<007ooOol03Woo00<0
07ooOol07Goo000QOol00`00Oomoo`0:Ool00`00Oomoo`0BOol00`00Oomoo`0NOol00`00Oomoo`08
Ool0011oo`03001oogoo01moo`03001oogoo01moo`03001oogoo01Qoo`006Goo00<007ooOol07Woo
00<007ooOol07goo00<007ooOol047oo0008Ool00`00Oomoo`0NOol00`00Oomoo`0OOol00`00Oomo
o`0LOol00`00Oomoo`02Ool000Aoo`03001oogoo00ioo`03001oogoo00ioo`03001oogoo01moo`03
001oogoo00ioo`03001oogoo01=oo`007Woo00<007ooOol05Woo00<007ooOol01Woo00<007ooOol0
7Woo00<007ooOol02goo000=Ool00`00Oomoo`0OOol00`00Oomoo`0NOol00`00Oomoo`0LOol0029o
o`03001oogoo01moo`03001oogoo00ioo`03001oogoo00ioo`03001oogoo00Ioo`004Goo00<007oo
Ool07goo00<007ooOol09goo00<007ooOol03goo0009Ool00`00Oomoo`0>Ool00`00Oomoo`0NOol0
0`00Oomoo`0>Ool00`00Oomoo`0POol002Aoo`03001oogoo00=oo`03001oogoo01Uoo`03001oogoo
01moo`03001oogoo00Aoo`000Woo00<007ooOol03Woo00<007ooOol07goo00<007ooOol07goo00<0
07ooOol05Goo000LOol00`00Oomoo`0NOol00`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0=
Ool000]oo`03001oogoo019oo`03001oogoo00Yoo`03001oogoo03=oo`03001oogoo00Uoo`003goo
00<007ooOol0<7oo00<007ooOol03Woo00<007ooOol06Goo000GOol00`00Oomoo`0GOol00`00Oomo
o`05Ool00`00Oomoo`0OOol00`00Oomoo`0AOol000Moo`03001oogoo01ioo`03001oogoo01moo`03
001oogoo029oo`00Kgoo001_Ool00001\
\>"],
  ImageRangeCache->{{{0, 110.75}, {110.75, 0}} -> {-0.0249429, -0.0249075, 
  0.00945726, 0.00944387}}]
}, Open  ]],

Cell["\<\
We can easily use such distributions of points to define an option \
valuation technique for European options. First we define the distribution of \
final stock values:\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell["\<\
WozSample[s_, r_, q_, \[Sigma]_, t_, n_] :=
Module[{data = N[X[n]], mean = (r - q - \[Sigma]^2/2)*t, sig = \
\[Sigma]*Sqrt[t]},
Map[(s*Exp[hammernorm[mean,sig,#]]&),data]];\
\>", "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["Now we define the corresponding option values:", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(WozEuroCall[S_, \ K_, \ \[Sigma]_, \ r_, \ q_, \ t_, \ n_]\  := \n
      Map[\((Max[0, \ # - K] &)\), 
          WozSample[S, \ r, \ q, \ \[Sigma], \ t, \ n]] // \n\t
        Exp[\(-r\)*t]*{Mean[#], \ StandardErrorOfSampleMean[#]} &\)], "Input",\

  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(WozEuroPut[S_, \ K_, \ \[Sigma]_, \ r_, \ q_, \ t_, \ n_]\  := \n
      Map[\((Max[0, \ K - #] &)\), 
          WozSample[S, \ r, \ q, \ \[Sigma], \ t, \ n]] // \n\t
        Exp[\(-r\)*t]*{Mean[#], \ StandardErrorOfSampleMean[#]} &\)], "Input",\

  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesCall[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(21.2487714385643755`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[WozEuroCall[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1\ , 5000]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({10.3166666666666051`\ Second, {21.2637150496229487`, 
        0.282562130850848269`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesPut[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1.7325132421603282`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[WozEuroPut[110, \ 100, \ 0.2, \ 0.1, \ 0, \ 1\ , 5000]]\)], 
  "Input",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({10.2833333333333021`\ Second, {1.72912821354726382`, 
        0.0667960905422739248`}}\)], "Output"]
}, Open  ]],

Cell["\<\
Superficially, the method is slower than Monte Carlo, but we note \
that the resulting means are close to the true value, given the number of \
sample points, and it is also noteworthy that having decided on a sample \
size, the points only need to be computed once. In principle we could just \
store say 100,000 precomputed sample points on the square, and re-use them \
repeatedly. 

This method can be used for a variety of applications, including Basket and \
path-dependent options, where the power of such a method when sampling in \
high dimensions is particularly pertinent. However, the details of this type \
of application are beyond the scope of this book.\
\>", "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 21 Bibliography", "Section",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Baxter, M. and Rennie, A., 1996, ",
  StyleBox["Financial Calculus - an Introduction to Derivative Pricing, ",
    FontSlant->"Italic"],
  "Cambridge University Press.\n\nBroadie, M. and Glasserman, P., 1996, \
Pricing American-style securities by simulation, submitted to ",
  StyleBox["Journal of",
    FontSlant->"Italic"],
  " ",
  StyleBox["Economic",
    FontSlant->"Italic"],
  " ",
  StyleBox["Dynamics",
    FontSlant->"Italic"],
  " ",
  StyleBox["and",
    FontSlant->"Italic"],
  " ",
  StyleBox["Control",
    FontSlant->"Italic"],
  ".\n\nPress, W.H., Teukolsky, S.A., Vetterling, W.T. and Flannery, B.P., \
1992, ",
  StyleBox["Numerical Recipes in C - the Art of Scientific Computing",
    FontSlant->"Italic"],
  ", 2nd edition, Cambridge University Press.\n\nWilmott, P., Dewynne, J. and \
Howison, S., 1993, ",
  StyleBox["Option Pricing - Mathematical Models and Computation, ",
    FontSlant->"Italic"],
  "Oxford Financial Press.\n\nWozniakowski, H., 1991, Average case complexity \
of multivariate integration, ",
  StyleBox["Bulletin of the American Mathematical Society, ",
    FontSlant->"Italic"],
  "24, p. 185.\n"
}], "Text",
  CellMargins->{{6.9375, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{544, 608},
WindowMargins->{{39, Automatic}, {Automatic, 2}},
PrintingCopies->1,
PrintingStartingPageNumber->400,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"21. Monte Carlo Simulation"}]]}, {Cell[ 
        TextData[ {"21. Monte Carlo Simulation"}]], Inherited, Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8`?n3ohP==PTh0b85N`?P0080004/0B`000006G<AP001
0000I00000400@4100800BL?00400@ilGH0000000000064/01T1T00000040000
00000000007oo`ilDM3oo`000mP:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 233, 9, 212, "Title"],
Cell[1967, 62, 140, 1, 47, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2132, 67, 146, 3, 50, "Section"],
Cell[2281, 72, 1998, 35, 401, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4316, 112, 118, 1, 49, "Section"],
Cell[4437, 115, 764, 16, 96, "Text"],

Cell[CellGroupData[{
Cell[5226, 135, 106, 2, 25, "Input"],
Cell[5335, 139, 403, 6, 81, "Print"]
}, Open  ]],
Cell[5753, 148, 724, 18, 52, "Text"],
Cell[6480, 168, 131, 2, 20, "NumberedEquation"],
Cell[6614, 172, 466, 13, 29, "Text"],
Cell[7083, 187, 143, 2, 20, "NumberedEquation"],
Cell[7229, 191, 214, 4, 29, "Text"],
Cell[7446, 197, 144, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[7615, 203, 129, 2, 25, "Input"],
Cell[7747, 207, 81, 1, 36, "Output"]
}, Open  ]],
Cell[7843, 211, 246, 5, 29, "Text"],
Cell[8092, 218, 168, 3, 25, "Input"],
Cell[8263, 223, 175, 3, 25, "Input"],
Cell[8441, 228, 238, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[8704, 237, 160, 3, 25, "Input"],
Cell[8867, 242, 70, 1, 36, "Output"]
}, Open  ]],
Cell[8952, 246, 131, 1, 20, "Text"],
Cell[9086, 249, 140, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[9251, 255, 302, 6, 37, "Input"],
Cell[9556, 263, 68708, 1681, 206, 21886, 1098, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[78279, 1947, 168, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[78472, 1955, 109, 2, 25, "Input"],
Cell[78584, 1959, 56, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[78677, 1965, 113, 2, 25, "Input"],
Cell[78793, 1969, 55, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[78885, 1975, 122, 2, 25, "Input"],
Cell[79010, 1979, 55, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[79102, 1985, 130, 2, 25, "Input"],
Cell[79235, 1989, 56, 1, 36, "Output"]
}, Open  ]],
Cell[79306, 1993, 309, 6, 40, "Text"],

Cell[CellGroupData[{
Cell[79640, 2003, 131, 2, 25, "Input"],
Cell[79774, 2007, 194, 5, 40, "Print"]
}, Open  ]],
Cell[79983, 2015, 372, 8, 40, "Text"],
Cell[80358, 2025, 259, 5, 50, "Input"],
Cell[80620, 2032, 334, 6, 40, "Text"],
Cell[80957, 2040, 645, 12, 152, "Input"],
Cell[81605, 2054, 432, 9, 40, "Text"],
Cell[82040, 2065, 172, 3, 25, "Input"],

Cell[CellGroupData[{
Cell[82237, 2072, 133, 2, 25, "Input"],
Cell[82373, 2076, 70, 1, 36, "Output"]
}, Open  ]],
Cell[82458, 2080, 158, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[82641, 2088, 160, 3, 25, "Input"],
Cell[82804, 2093, 71, 1, 36, "Output"]
}, Open  ]],
Cell[82890, 2097, 353, 8, 29, "Text"],
Cell[83246, 2107, 231, 6, 37, "Input"],

Cell[CellGroupData[{
Cell[83502, 2117, 133, 2, 25, "Input"],
Cell[83638, 2121, 55, 1, 36, "Output"]
}, Open  ]],
Cell[83708, 2125, 534, 12, 52, "Text"],
Cell[84245, 2139, 545, 12, 112, "Input"],
Cell[84793, 2153, 265, 6, 29, "Text"],

Cell[CellGroupData[{
Cell[85083, 2163, 133, 2, 25, "Input"],
Cell[85219, 2167, 71, 1, 36, "Output"]
}, Open  ]],
Cell[85305, 2171, 191, 4, 20, "Text"],
Cell[85499, 2177, 517, 9, 88, "Input"],
Cell[86019, 2188, 159, 3, 25, "Input"],

Cell[CellGroupData[{
Cell[86203, 2195, 145, 2, 25, "Input"],
Cell[86351, 2199, 71, 1, 36, "Output"]
}, Open  ]],
Cell[86437, 2203, 203, 4, 29, "Text"],
Cell[86643, 2209, 318, 6, 50, "Input"],

Cell[CellGroupData[{
Cell[86986, 2219, 179, 4, 25, "Input"],
Cell[87168, 2225, 70, 1, 36, "Output"]
}, Open  ]],
Cell[87253, 2229, 842, 17, 96, "Text"],

Cell[CellGroupData[{
Cell[88120, 2250, 361, 8, 37, "Input",
  PageBreakBelow->Automatic],
Cell[88484, 2260, 50662, 1455, 204, 21765, 1092, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakBelow->Automatic]
}, Open  ]],
Cell[139161, 3718, 163, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[139349, 3726, 109, 2, 25, "Input"],
Cell[139461, 3730, 62, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[139560, 3736, 122, 2, 25, "Input"],
Cell[139685, 3740, 54, 1, 36, "Output"]
}, Open  ]],
Cell[139754, 3744, 444, 13, 29, "Text"],

Cell[CellGroupData[{
Cell[140223, 3761, 108, 1, 28, "Subsubsection"],
Cell[140334, 3764, 178, 4, 20, "Text"],
Cell[140515, 3770, 580, 12, 112, "Input"],
Cell[141098, 3784, 639, 13, 125, "Input"],

Cell[CellGroupData[{
Cell[141762, 3801, 125, 2, 25, "Input"],
Cell[141890, 3805, 140, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[142067, 3812, 111, 2, 25, "Input"],
Cell[142181, 3816, 148, 3, 36, "Output"]
}, Open  ]],
Cell[142344, 3822, 203, 4, 29, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[142596, 3832, 120, 1, 38, "Section"],
Cell[142719, 3835, 1013, 18, 161, "Text"],

Cell[CellGroupData[{
Cell[143757, 3857, 104, 1, 32, "Subsection"],
Cell[143864, 3860, 673, 20, 128, "Text"],
Cell[144540, 3882, 217, 3, 30, "NumberedEquation"],
Cell[144760, 3887, 230, 7, 20, "Text"],
Cell[144993, 3896, 329, 9, 68, "Input",
  InitializationCell->True],
Cell[145325, 3907, 521, 17, 29, "Text"],
Cell[145849, 3926, 381, 13, 112, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[146255, 3943, 138, 2, 25, "Input"],
Cell[146396, 3947, 369, 6, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[146802, 3958, 142, 2, 25, "Input"],
Cell[146947, 3962, 370, 6, 48, "Output"]
}, Open  ]],
Cell[147332, 3971, 299, 9, 20, "Text"],
Cell[147634, 3982, 391, 13, 112, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[148050, 3999, 144, 2, 25, "Input"],
Cell[148197, 4003, 369, 6, 48, "Output"]
}, Open  ]],
Cell[148581, 4012, 177, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[148783, 4020, 163, 3, 25, "Input"],
Cell[148949, 4025, 71, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[149057, 4031, 173, 4, 25, "Input"],
Cell[149233, 4037, 71, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[149341, 4043, 175, 4, 25, "Input"],
Cell[149519, 4049, 71, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[149627, 4055, 121, 2, 25, "Input"],
Cell[149751, 4059, 49, 1, 36, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[149849, 4066, 108, 1, 32, "Subsection"],
Cell[149960, 4069, 125, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[150110, 4074, 320, 7, 57, "Input"],
Cell[150433, 4083, 43955, 1383, 144, 15375, 1024, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[194403, 5469, 227, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[194655, 5478, 162, 3, 28, "Subsubsection"],
Cell[194820, 5483, 221, 6, 20, "Text"],
Cell[195044, 5491, 154, 2, 25, "Input"],
Cell[195201, 5495, 225, 5, 36, "Input"],

Cell[CellGroupData[{
Cell[195451, 5504, 154, 2, 25, "Input"],
Cell[195608, 5508, 28094, 666, 144, 5443, 380, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[223739, 6179, 325, 7, 57, "Input"],
Cell[224067, 6188, 56634, 1579, 144, 16059, 1072, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[280774, 7775, 143, 2, 38, "Section",
  PageBreakAbove->True],
Cell[280920, 7779, 940, 18, 84, "Text"],
Cell[281863, 7799, 201, 3, 32, "NumberedEquation"],
Cell[282067, 7804, 649, 15, 52, "Text"],
Cell[282719, 7821, 357, 10, 68, "Input",
  InitializationCell->True],
Cell[283079, 7833, 278, 7, 29, "Text"],
Cell[283360, 7842, 414, 15, 145, "Input",
  InitializationCell->True],
Cell[283777, 7859, 177, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[283979, 7867, 189, 4, 25, "Input"],
Cell[284171, 7873, 71, 1, 36, "Output"]
}, Open  ]],
Cell[284257, 7877, 235, 5, 29, "Text"],
Cell[284495, 7884, 129, 2, 25, "Input"],
Cell[284627, 7888, 152, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[284804, 7896, 164, 3, 25, "Input"],
Cell[284971, 7901, 66, 1, 36, "Output"]
}, Open  ]],
Cell[285052, 7905, 264, 7, 20, "Text"],
Cell[285319, 7914, 416, 10, 90, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[285760, 7928, 146, 2, 25, "Input"],
Cell[285909, 7932, 264, 4, 48, "Output"]
}, Open  ]],
Cell[286188, 7939, 247, 6, 20, "Text"],
Cell[286438, 7947, 459, 11, 90, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[286922, 7962, 152, 2, 25, "Input"],
Cell[287077, 7966, 265, 4, 48, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[287391, 7976, 112, 1, 38, "Section"],
Cell[287506, 7979, 352, 6, 40, "Text"],
Cell[287861, 7987, 133, 2, 25, "Input"],
Cell[287997, 7991, 353, 7, 50, "Input"],
Cell[288353, 8000, 381, 8, 62, "Input"],

Cell[CellGroupData[{
Cell[288759, 8012, 147, 2, 25, "Input"],
Cell[288909, 8016, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[289000, 8022, 177, 4, 25, "Input"],
Cell[289180, 8028, 120, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[289337, 8035, 177, 4, 25, "Input"],
Cell[289517, 8041, 119, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[289673, 8048, 177, 4, 25, "Input"],
Cell[289853, 8054, 119, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[290009, 8061, 177, 4, 25, "Input"],
Cell[290189, 8067, 120, 2, 36, "Output"]
}, Open  ]],
Cell[290324, 8072, 350, 7, 50, "Input"],
Cell[290677, 8081, 382, 8, 62, "Input"],

Cell[CellGroupData[{
Cell[291084, 8093, 146, 2, 25, "Input"],
Cell[291233, 8097, 53, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[291323, 8103, 176, 4, 25, "Input"],
Cell[291502, 8109, 121, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[291660, 8116, 176, 4, 25, "Input"],
Cell[291839, 8122, 119, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[291995, 8129, 176, 4, 25, "Input"],
Cell[292174, 8135, 120, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[292331, 8142, 176, 4, 25, "Input"],
Cell[292510, 8148, 121, 2, 36, "Output"]
}, Open  ]],
Cell[292646, 8153, 398, 7, 40, "Text"],

Cell[CellGroupData[{
Cell[293069, 8164, 100, 1, 32, "Subsection"],
Cell[293172, 8167, 825, 16, 73, "Text"],
Cell[294000, 8185, 308, 8, 46, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[294333, 8197, 141, 2, 25, "Input"],
Cell[294477, 8201, 415, 6, 60, "Output"]
}, Open  ]],
Cell[294907, 8210, 613, 11, 88, "Input"],
Cell[295523, 8223, 229, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[295777, 8232, 190, 4, 25, "Input"],
Cell[295970, 8238, 392, 10, 61, "Output"]
}, Open  ]],
Cell[296377, 8251, 125, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[296527, 8256, 307, 5, 50, "Input"],
Cell[296837, 8263, 391, 11, 61, "Output"]
}, Open  ]],
Cell[297243, 8277, 637, 11, 84, "Text"],
Cell[297883, 8290, 625, 11, 76, "Input"],

Cell[CellGroupData[{
Cell[298533, 8305, 189, 4, 25, "Input"],
Cell[298725, 8311, 395, 10, 61, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[299157, 8326, 286, 4, 50, "Input"],
Cell[299446, 8332, 114, 2, 36, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[299621, 8341, 107, 1, 44, "Section"],
Cell[299731, 8344, 411, 10, 105, "Text"],
Cell[300145, 8356, 136, 1, 20, "Input"],
Cell[300284, 8359, 176, 4, 21, "Text"],
Cell[300463, 8365, 170, 4, 20, "Input"],
Cell[300636, 8371, 196, 4, 21, "Text"],

Cell[CellGroupData[{
Cell[300857, 8379, 202, 5, 32, "Input",
  PageBreakAbove->False],
Cell[301062, 8386, 14673, 523, 120, 8032, 436, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[315750, 8912, 1639, 37, 180, "Text",
  PageBreakAbove->True],
Cell[317392, 8951, 113, 1, 20, "Input"],

Cell[CellGroupData[{
Cell[317530, 8956, 84, 1, 20, "Input"],
Cell[317617, 8959, 52, 1, 20, "Output"]
}, Open  ]],
Cell[317684, 8963, 119, 1, 21, "Text"],

Cell[CellGroupData[{
Cell[317828, 8968, 114, 1, 20, "Input"],
Cell[317945, 8971, 36, 1, 20, "Output"]
}, Open  ]],
Cell[317996, 8975, 224, 5, 36, "Text"],
Cell[318223, 8982, 174, 5, 44, "Input"],

Cell[CellGroupData[{
Cell[318422, 8991, 86, 1, 25, "Input"],
Cell[318511, 8994, 40, 1, 48, "Output"]
}, Open  ]],
Cell[318566, 8998, 92, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[318683, 9003, 122, 1, 25, "Input"],
Cell[318808, 9006, 40, 1, 48, "Output"]
}, Open  ]],
Cell[318863, 9010, 156, 4, 20, "Text"],
Cell[319022, 9016, 127, 1, 25, "Input"],
Cell[319152, 9019, 96, 1, 20, "Text"],
Cell[319251, 9022, 90, 1, 25, "Input"],

Cell[CellGroupData[{
Cell[319366, 9027, 161, 4, 36, "Input"],
Cell[319530, 9033, 15547, 534, 118, 7987, 436, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[335092, 9570, 254, 5, 29, "Text"],
Cell[335349, 9577, 313, 8, 46, "Input",
  InitializationCell->True],
Cell[335665, 9587, 124, 1, 20, "Text"],
Cell[335792, 9590, 329, 6, 50, "Input"],
Cell[336124, 9598, 328, 6, 50, "Input"],

Cell[CellGroupData[{
Cell[336477, 9608, 147, 2, 25, "Input"],
Cell[336627, 9612, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[336718, 9618, 161, 3, 25, "Input"],
Cell[336882, 9623, 120, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[337039, 9630, 146, 2, 25, "Input"],
Cell[337188, 9634, 53, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[337278, 9640, 160, 3, 25, "Input"],
Cell[337441, 9645, 121, 2, 36, "Output"]
}, Open  ]],
Cell[337577, 9650, 755, 13, 96, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[338369, 9668, 104, 1, 38, "Section"],
Cell[338476, 9671, 1235, 34, 172, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

