(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    206444,       5309]*)
(*NotebookOutlinePosition[    208342,       5367]*)
(*  CellTagsIndexPosition[    208298,       5363]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 28. 
Modelling Volatility by Elasticity\
\>", "Title",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["The CEV Family Explained and Explored", "Subsubtitle",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["28.1 Introduction", "Section",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Attempts to improve the treatment of volatility have been developed by \
many authors. This can be done in several contexts, and our focus here will \
be on improving the modelling of the volatility of equity options. Different \
methods can be used to treat volatility of FX and interest-rate options. \
Within the equity area, possible approaches include, but are not necessarily \
limited to:\n\n(a) volatility as a deterministic function of the (stochastic) \
stock price (\"level-dependent\" models);\n(b) stochastic volatility as a \
separate factor from the stock price;\n(c) GARCH approaches.\n\nThis chapter \
addresses a subset of models of type (a), known as the \"constant elasticity \
of variance\" (CEV) models, which capture in a simple way the notion that the \
volatility increases as the asset price decreases. There are good reasons for \
considering this case:\n\n(1) the model nicely captures the leverage effect \
associated with asset prices;\n(2) there is research suggesting that the \
pricing of warrants in particular is improved by the use of this model;\n(3) \
it is analytically tractable;\n(4) it predicts skews and, with some \
additional adjustments for spreads, smiles;\n(5) one member of the family is \
equivalent to a non-normal and non-log-normal interest rate model (CIR), \
allowing us to explore aspects of the interest-rate mathematics \
simultaneously with the equity mathematics.\n\nThis last point explains in \
part why this last chapter is placed directly after the discussion of \
elementary interest-rate models - the CEV theory applied to equity \
derivatives is closely related to the CIR model. Furthermore, placing the \
discussion here allows us to end this book at a point closely related to \
where we began, with a discussion of some of the effects that can be seen in \
implied volatility, in an application where it does make sense!\n\nThere are \
quite a few papers now published on elasticity models, some of them with \
minor glitches in them - part of the purpose of this chapter is to straighten \
out some of the technology and to give a comprehensive presentation. Also, \
some of the formulae employed for Call options or warrants turn out to be \
rather complicated. The general CEV Call option formula involves an infinite \
series of Gamma-functions, which is somewhat awkward to evaluate, even in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", let alone in C/C++ directly. There is an approximation for the cases of \
the SRCEV model (where the volatility varies as the inverse square root of \
the asset price - it is this model that is equivalent to the CIR \
interest-rate model without the mean-reversion) known as the Cox-Beckers \
approximation. Unfortunately there are at least three different and \
inconsistent representations of this formula in the literature.\n\nOne of the \
purposes of this chapter is to derive new simplified forms of the exact \
pricing formula. In the particular case of the SRCEV model, we give a simple \
formula involving an integration from 0 to ",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\)]],
  " of the normal distribution, which is readily implementable in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " or C/C++. While it may be possible to derive similar formulae for other \
elasticities, I have not done so, but the values of options can always be \
done directly in terms of integrals of Bessel functions.  This can be coded \
up in C or C++ using standard routines, for example those given by Press",
  StyleBox[" et al ",
    FontSlant->"Italic"],
  "(1992). Other methods for evaluation are given by Schroder (1989), which \
an interesting paper that among other things gives the clearest exposition, \
of the approximation of the non-central chi-square by normal distributions, \
that I have seen.\n\nThe running example considered in detail is that of a \
Call option - this is of particular interest in the development of improved \
warrant-pricing models.\n\nThe analysis presented suggests that at least part \
of the volatility structure observed in market prices may be captured by a \
simple level-dependent model, and one interesting issue relates to how to \
bolt on further stochasticity or GARCH behaviour to this type of approach. \
This approach, although simplistic, nicely captures the \"leverage effect\", \
so it should be considered as at least a part of the overall picture. The \
role of two-factor or GARCH models should perhaps be more properly defined as \
explaining those effects not captured by the type of model presented here."
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["28.2 CIR Interest Rates and Asset Elasticity Models", "Section",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The type of stochastic process under consideration is probably more \
familiar in the CIR interest-rate model, as discussed in the previous \
chapter. The CIR world is governed by an interest-rate diffusion of the \
form\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(d\ r = \(a(
            b - r)\)\ d\ t\  + \[Sigma]\ \@r\ d\ z\ \)\)], "NumberedEquation",\

  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This describes a mean-reverting process. It has nice analytical \
properties, such as the closed-form solution for bond options we have briefly \
outlined in the previous chapter. On the other hand, if we set ",
  Cell[BoxData[
      \(TraditionalForm\`b\  = \ 0\)]],
  ", and ",
  Cell[BoxData[
      \(TraditionalForm\`r \[RightArrow] \ S, \ 
      a\  \[RightArrow] \ \((q - r)\)\)]],
  ", we get a process of the form"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(d\ S = \((r - q)\) 
          S\ d\ t\  + \[Sigma]\ \@S\ d\ z\ \)\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["We can write (2) in the form", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(d\ S\)\/S = \((r - q)\) d\ \ t + \(\[Sigma]\/\@S\) 
          d\ z\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
which is an equity-like process with a volatility varying as the \
inverse square root of the stock price. This is commonly known as an SRCEV \
model - \"Square root constant elasticity of variance\". There are more \
general CEV models, where the process takes the form\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(d\ S\)\/S = \((r - q)\) 
          d\ \ t + \(\[Sigma]\/S\^\(1 - \[Alpha]\)\) d\ z\)], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\)]],
  " is between 0 (normal) and 1 (log-normal). So we see that a special form \
of the CIR interest-rate model is equivalent to a special case of stochastic \
volatility model for the equity world. The only difference is in the details \
and instruments valued - options on bonds (payoffs exponential functions of ",
  
  Cell[BoxData[
      \(TraditionalForm\`r\)]],
  ") cf. options on ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  " (payoffs typically piecewise linear). \n\nA brief review of the asset \
versions of these models was done recently by their principal inventor, J. \
Cox, (1996) in an article entitled: \"The constant elasticity of variance \
option pricing model\". It is well worth a detailed read, but note that the \
formula for the probability distribution is missing a term. Cox goes on to \
develop the infinite series of Gamma-functions as the price of a Call - it is \
this that we wish to try to simplify to make it more straightforward as a \
model. We shall also look at the associated Black-Scholes PDE in some detail, \
and explore its solution for a case slightly more general than that \
considered by Cox, where we have a time-dependent interest-rate and yield \
structure. Our approach will be based on the partial differential equation \
method, and parallels that given for the ordinary Black-Scholes equation, and \
it is here that the detailed development begins. Readers may wish to explore \
the development of an equivalent martingale formulation, and of the \
corresponding interest-rate analogues with mean reversion."
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["28.3 Standardized Form of the CEV Black-Scholes Equation", "Section",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The Black-Scholes equation with time-dependent interest rates and dividend \
yield, and volatility ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma](S), \)\)]],
  " is"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]V\), \(\[PartialD]t\),
            MultilineFunction->None], "-", \(\(r \((t)\)\)\ V\), "+", 
          RowBox[{\(1\/2\), " ", \(\(\[Sigma]\^2\) \((S)\)\), \(S\^2\), " ", 
            FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]S\^2\),
              MultilineFunction->None]}], "+", " ", 
          RowBox[{\((r \((t)\) - q \((t)\))\), "S", " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]S\),
              MultilineFunction->None]}]}], "=", "0"}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "First we make the standard time-reversing transformation ",
  Cell[BoxData[
      \(TraditionalForm\`\(t\^*\) = \ T - t\)]],
  ", with the option expiring at time ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  ". This gives"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{\(\(\ \(\[Sigma]\^2\) \((S)\)\)\/2\), \(S\^2\), " ", 
            FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]S\^2\),
              MultilineFunction->None]}], "+", " ", 
          
          RowBox[{\((r \((T - \(t\^*\))\) - q \((T - \(t\^*\))\))\), "S", 
            " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]S\),
              MultilineFunction->None]}]}], "=", 
        RowBox[{
          FractionBox[\(\[PartialD]V\), \(\[PartialD]\(t\^*\)\),
            MultilineFunction->None], 
          "+", \(\(r \((T - \(t\^*\))\)\)\ V\)}]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Next we make the standard discounting transformation (Section 4.6) ",
  Cell[BoxData[
      \(TraditionalForm\`V = \[ExponentialE]\^\(-\(B(\(t\^*\))\)\)\ V\_1\)]],
  "with ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " arranged so that"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]B\), \(\[PartialD]\(t\^*\)\),
          MultilineFunction->None], "=", \(r \((T - \(t\^*\))\)\)}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["which reduces the equation to", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{\(\(\ \(\[Sigma]\^2\) \((S)\)\)\/2\), \(S\^2\), " ", 
            FractionBox[\(\[PartialD]\^2 V\_1\), \(\[PartialD]S\^2\),
              MultilineFunction->None]}], "+", " ", 
          
          RowBox[{\((r \((T - \(t\^*\))\) - q \((T - \(t\^*\))\))\), "S", 
            " ", 
            FractionBox[\(\[PartialD]V\_1\), \(\[PartialD]S\),
              MultilineFunction->None]}]}], "=", 
        FractionBox[\(\[PartialD]V\_1\), \(\[PartialD]\(t\^*\)\),
          MultilineFunction->None]}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Note that the appropriate solution to the differential equation for ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " is indeed the discount factor:"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[ExponentialE]\^\(-\(B(\(t\^*\))\)\) = \
\[ExponentialE]\^\(-\(\[Integral]\_t\%T\( r(t\_1)\) \
\[DifferentialD]t\_1\)\)\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The next task is to eliminate the drift term, and this again makes \
use of an approach similar to that discussed in Chapter 4 for CBs. We \
introduce new variables\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    FormBox[
      RowBox[{"S", "=", 
        RowBox[{
          SuperscriptBox["S", "\[Prime]",
            MultilineFunction->None], " ", 
          SuperscriptBox["\[ExponentialE]", 
            RowBox[{"F", "(", 
              SuperscriptBox["t", "\[Prime]",
                MultilineFunction->None], ")"}]]}]}], TraditionalForm], 
    FormBox[
      RowBox[{\(t\^*\), "=", 
        SuperscriptBox["t", "\[Prime]",
          MultilineFunction->None]}], TraditionalForm]}], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
and change variables in the PDE (just use the chain rule carefully \
if you want to check this), obtaining\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{\(\(\ \(\[Sigma]\^2\) \((S)\)\)\/2\), \(S'\^2\), " ", 
            FractionBox[\(\[PartialD]\^2 V\_1\), \(\[PartialD]S'\^2\),
              MultilineFunction->None]}], "+", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                FractionBox[\(\[PartialD]F\), 
                  RowBox[{"\[PartialD]", 
                    SuperscriptBox["t", "\[Prime]",
                      MultilineFunction->None]}],
                  MultilineFunction->None], "+", \(r \((T - t')\)\), 
                "-", \(q \((T - t')\)\)}], ")"}], \(S'\), " ", 
            FractionBox[\(\[PartialD]V\_1\), \(\[PartialD]S'\),
              MultilineFunction->None]}]}], "=", 
        FractionBox[\(\[PartialD]V\_1\), \(\[PartialD]t'\),
          MultilineFunction->None]}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Note that the volatility remains expressed in terms of the original \
underlying. So we can eliminate the drift term by choosing ",
  Cell[BoxData[
      \(TraditionalForm\`F\)]],
  " to satisfy the ordinary differential equation"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]F\), 
            RowBox[{"\[PartialD]", 
              SuperscriptBox["t", "\[Prime]",
                MultilineFunction->None]}],
            MultilineFunction->None], "+", \(r \((T - t')\)\), 
          "-", \(q \((T - t')\)\)}], "=", "0"}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We can write the solution formally in terms of an integral of the spot \
rate and yield. We shall write the answer in terms of a function ",
  Cell[BoxData[
      \(TraditionalForm\`Q\)]],
  " that is the reciprocal of ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{"F", "(", 
            SuperscriptBox["t", "\[Prime]",
              MultilineFunction->None], ")"}]], TraditionalForm]]],
  ", i.e., "
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"Q", "(", 
          SuperscriptBox["t", "\[Prime]",
            MultilineFunction->None], ")"}], "=", 
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{"+", 
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", 
                SuperscriptBox["t", "\[Prime]",
                  
                  MultilineFunction->None]], \(\((r \((T - s)\) - 
                    q \((T - s)\))\) \[DifferentialD]s\)}]}]]}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["Then", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{"S", "=", 
        FractionBox[
          SuperscriptBox["S", "\[Prime]",
            MultilineFunction->None], 
          RowBox[{"Q", "(", 
            SuperscriptBox["t", "\[Prime]",
              MultilineFunction->None], ")"}]]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["The PDE is now", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{\(\(\ \(\[Sigma]\^2\) \((S'/Q)\)\)\/2\), \(S'\^2\), " ", 
          FractionBox[\(\[PartialD]\^2 V\_1\), \(\[PartialD]S'\^2\),
            MultilineFunction->None]}], "=", 
        FractionBox[\(\[PartialD]V\_1\), \(\[PartialD]t'\),
          MultilineFunction->None]}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Note that at maturity ",
  Cell[BoxData[
      \(TraditionalForm\`S\  = \ S'\)]],
  ", whereas at initiation"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["S", "\[Prime]",
          MultilineFunction->None], 
        "=", \(S\ \[ExponentialE]\^\(+\(\[Integral]\_0\%T\((r \((s)\) - q \
\((s)\))\) \[DifferentialD]s\)\)\)}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
It is difficult to proceed further without an assumption about the \
form of the volatility. It is here that we make the elasticity \
assumption:\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Sigma](S) = \[Sigma]\_0\ S\^\(\[Alpha] - 1\)\)], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This allows us to collect the independent variables on both sides \
of the equation, obtaining\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox[
            RowBox[{"Q", "(", 
              SuperscriptBox["t", "\[Prime]",
                MultilineFunction->None], ")"}], \(2\ \((\[Alpha] - 1)\)\)], 
          " ", 
          FractionBox[\(\[PartialD]V\_1\), 
            RowBox[{"\[PartialD]", 
              SuperscriptBox["t", "\[Prime]",
                MultilineFunction->None]}],
            MultilineFunction->None]}], "=", 
        RowBox[{\(1\/2\), " ", \(\[Sigma]\_0\%2\), " ", 
          SuperscriptBox[
            SuperscriptBox["S", "\[Prime]",
              MultilineFunction->None], \(2\ \[Alpha]\)], " ", 
          FractionBox[\(\[PartialD]\^2 V\_1\), 
            RowBox[{"\[PartialD]", 
              SuperscriptBox[
                SuperscriptBox["S", "\[Prime]",
                  MultilineFunction->None], "2"]}],
            MultilineFunction->None]}]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We re-parametrize the elasticity variable by writing ",
  Cell[BoxData[
      \(TraditionalForm\`2  \[Alpha] = 2 - 1/\[Nu]\)]],
  ", so that the PDE is now"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox["1", 
            SuperscriptBox[
              RowBox[{"Q", "(", 
                SuperscriptBox["t", "\[Prime]",
                  MultilineFunction->None], ")"}], \(1/\[Nu]\)]], " ", 
          FractionBox[\(\[PartialD]V\_1\), 
            RowBox[{"\[PartialD]", 
              SuperscriptBox["t", "\[Prime]",
                MultilineFunction->None]}],
            MultilineFunction->None]}], "=", 
        RowBox[{\(1\/2\), " ", \(\[Sigma]\_0\%2\), " ", 
          SuperscriptBox[
            SuperscriptBox["S", "\[Prime]",
              MultilineFunction->None], \(2\  - 1/\[Nu]\)], " ", 
          FractionBox[\(\[PartialD]\^2 V\_1\), 
            RowBox[{"\[PartialD]", 
              SuperscriptBox[
                SuperscriptBox["S", "\[Prime]",
                  MultilineFunction->None], "2"]}],
            MultilineFunction->None]}]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We are almost at a recognizable PDE! It is just a matter of making \
one further transformation on each of the time and price variables. To \
simplify the right side, we introduce\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{"X", "=", 
        SuperscriptBox[
          SuperscriptBox["S", "\[Prime]",
            MultilineFunction->None], \(1/\[Nu]\)]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["This gets us to", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(2\ \[Nu]\^2\ \), 
            RowBox[{\(\[Sigma]\_0\%2\), " ", 
              SuperscriptBox["Q", 
                RowBox[{\(1\/\[Nu]\), "(", 
                  SuperscriptBox["t", "\[Prime]",
                    MultilineFunction->None], ")"}]]}]], 
          FractionBox[\(\[PartialD]V\_1\), 
            RowBox[{"\[PartialD]", 
              SuperscriptBox["t", "\[Prime]",
                MultilineFunction->None]}],
            MultilineFunction->None]}], "=", 
        RowBox[{
          RowBox[{"X", " ", 
            FractionBox[\(\[PartialD]\^2 V\_1\), \(\[PartialD]X\^2\),
              MultilineFunction->None]}], "+", 
          RowBox[{\((1 - \[Nu])\), " ", 
            FractionBox[\(\[PartialD]V\_1\), \(\[PartialD]X\),
              MultilineFunction->None]}]}]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The left side is now simplified by picking a new time coordinate ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\)]],
  " with the property that"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{\(d\ \[Tau]\), "=", 
        RowBox[{\(1\/\(2\ \[Nu]\^2\)\), \(\[Sigma]\_0\%2\), 
          " ", \(\(Q\^\(1\/\[Nu]\)\)(t')\), " ", "d", " ", 
          SuperscriptBox["t", "\[Prime]",
            MultilineFunction->None]}]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["This differential equation can be solved formally as", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{"\[Tau]", "=", 
        RowBox[{\(\(\[Sigma]\_0\%2\ \)\/\(2\ \[Nu]\^2\)\), " ", 
          RowBox[{
            SubsuperscriptBox["\[Integral]", "0", 
              SuperscriptBox["t", "\[Prime]",
                MultilineFunction->None]], 
            RowBox[{
              SuperscriptBox["\[ExponentialE]", 
                RowBox[{\(1\/\[Nu]\), 
                  RowBox[{\(\[Integral]\_0\%s\), 
                    RowBox[{
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{"r", "(", 
                            RowBox[{"T", "-", 
                              SuperscriptBox["s", "\[Prime]",
                                MultilineFunction->None]}], ")"}], "-", 
                          RowBox[{"q", "(", 
                            RowBox[{"T", "-", 
                              SuperscriptBox["s", "\[Prime]",
                                MultilineFunction->None]}], ")"}]}], ")"}], 
                      RowBox[{"\[DifferentialD]", 
                        SuperscriptBox["s", "\[Prime]",
                          
                          MultilineFunction->
                            None]}]}]}]}]], \(\[DifferentialD]s\)}]}]}]}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Note that this new time is zero at expiry. If ",
  Cell[BoxData[
      \(TraditionalForm\`\(r\ \)\)]],
  "and ",
  Cell[BoxData[
      \(TraditionalForm\`q\)]],
  " are constant, this simplifies to"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Tau] = \(\[Sigma]\_0\%2\ \)\/\(2\ \[Nu]\ \((r - q)\)\
\)\ \((\[ExponentialE]\^\(\(\((r - q)\)\ \((T - t)\)\)\/\[Nu]\) - 1)\)\)], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "which is the reciprocal of the parameter called ",
  Cell[BoxData[
      \(TraditionalForm\`c\)]],
  " or ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " by Cox and other authors. Whether or not the ",
  Cell[BoxData[
      \(TraditionalForm\`r, q\)]],
  " parameters are constant, we arrive at the standardized CEV equation in \
the form"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]V\_1\), \(\[PartialD]\[Tau]\),
          MultilineFunction->None], "=", 
        RowBox[{
          RowBox[{"X", " ", 
            FractionBox[\(\[PartialD]\^2 V\_1\), \(\[PartialD]X\^2\),
              MultilineFunction->None]}], "+", 
          RowBox[{\((1 - \[Nu])\), " ", 
            FractionBox[\(\[PartialD]V\_1\), \(\[PartialD]X\),
              MultilineFunction->None]}]}]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The elasticity parameter ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\)]],
  " appears now in the first order term - this vanishes in this view of the \
model when ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\  = \ 1\)]],
  ", which is the special square root CEV model. This PDE is akin to that for \
purely radial diffusion in ",
  Cell[BoxData[
      \(TraditionalForm\`\(-\[Nu]\)\)]],
  " dimensions! Students of probability may recognize it as that \
corresponding to a Bessel-squared process. We note that a futher \
transformation can reduce this to the ordinary diffusion equation with a \
non-zero drift - a fact that is of considerable importance, and that also \
applies to all of the interest-rate models, when source terms are also \
admitted.\n\nIn principle we could proceed to analyse this equation in much \
the same way as the ordinary one-dimensional diffusion equation. As the \
details of this are hard - rather beyond the scope of this text - we shall \
just quote the Green's function analogous to that given in Chapter 4. The \
Green's function corresponding to a delta-function source located at ",
  Cell[BoxData[
      \(TraditionalForm\`Y\)]],
  " at time ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\  = \ 0\)]],
  " is:"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`G(X, 
        Y, \[Tau]) = \(1\/\[Tau]\) \[ExponentialE]\^\(-\(\((X + \
Y)\)\/\[Tau]\)\)\ \((X\/Y)\)\^\(\[Nu]/2\)\ \(\(I\_\[Nu]\)(\(2\ \@\(X\ Y\)\)\/\
\[Tau])\)\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "If you wish to satisfy yourself that this is correct, then use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to check that it solves the differential equation, and use the fact that \
for large ",
  Cell[BoxData[
      \(TraditionalForm\`z\)]],
  ", "
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(\(I\_\[Nu]\)(
        z)\)~\(\[ExponentialE]\^z\/\@\(2\ \[Pi]\ z\)\)\)], "NumberedEquation",\

  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "to verify the behaviour for small \[Tau]. Normalization issues are \
discussed presently. Taking ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  " as the Green's function allows us to write down the option valuation and \
interpret it in terms of risk-neutral valuation. Let the option payoff be ",
  Cell[BoxData[
      \(TraditionalForm\`P(S\_T)\)]],
  ". Because ",
  Cell[BoxData[
      \(TraditionalForm\`S'\  = \ S\)]],
  " at maturity, the value of ",
  Cell[BoxData[
      \(TraditionalForm\`X\)]],
  " at maturity, denoted by ",
  Cell[BoxData[
      \(TraditionalForm\`X\_T\)]],
  ",  corresponding to the stock price ",
  Cell[BoxData[
      \(TraditionalForm\`S\_T\)]],
  "is just"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`S\_T = X\_T\%\[Nu]\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The non-discounted value at an earlier time ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\)]],
  " is then"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["V", "\[Prime]",
            MultilineFunction->None], "(", \(X, \[Tau]\), ")"}], 
        "=", \(\[Integral]\(\(P \((X\_T\%\[Nu])\)\)\ \(G \((X, 
                  X\_T, \[Tau])\)\)\) \[DifferentialD]X\_T\)}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Making a change of variable to ",
  Cell[BoxData[
      \(TraditionalForm\`S\_T\)]],
  ", we are led, finally, to"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          SuperscriptBox["V", "\[Prime]",
            MultilineFunction->None], "(", \(X, \[Tau]\), ")"}], 
        "=", \(\[Integral]\(\(P \((S\_T)\)\)\ \(f \((S\_T)\)\)\) \
\[DifferentialD]S\_T\)}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where, if we set ",
  Cell[BoxData[
      \(TraditionalForm\`X\  = \ x\ \[Tau], \ \ X\_T = \ \(x\_T\) \[Tau]\)]],
  ", "
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`f(
        S\_T) = \(1\/\[Nu]\) \[Tau]\^\(-\[Nu]\)\ \((x\ x\_T\%\(2\/\[Nu] - \
3\))\)\^\(\[Nu]/2\)\ \[ExponentialE]\^\(\(-x\) - x\_T\)\ \(\(I\_\[Nu]\)(
          2\ \@\(x\ x\_T\))\)\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The variable ",
  Cell[BoxData[
      \(TraditionalForm\`X\)]],
  " is then related to the price at initiation by"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`X = 
      S\^\(1/\[Nu]\)\ \[ExponentialE]\^\(\(\[Integral]\_0\%T\((r(s) - q(s))\) \
\[DifferentialD]s\)\/\[Nu]\)\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The one catch in dealing with such models is that the expiry distribution \
does not integrate to unity. The density function for ",
  Cell[BoxData[
      \(TraditionalForm\`X\_T\)]],
  " is "
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`U(
        X\_T) = \(1\/\[Tau]\) \[ExponentialE]\^\(-\(\((X + \
X\_T)\)\/\[Tau]\)\)\ \((X\/X\_T)\)\^\(\[Nu]/2\)\ \(\(I\_\[Nu]\)(\(2\ \@\(X\ X\
\_T\)\)\/\[Tau])\)\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["This function has the property that", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Integral]\_0\%\[Infinity]\( U(
            X\_T)\) \[DifferentialD]X\_T = 
      1 - \(\[CapitalGamma](\[Nu], \
X\/\[Tau])\)\/\(\[CapitalGamma](\[Nu])\)\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "So the probability that ",
  Cell[BoxData[
      \(TraditionalForm\`\(X\_T = \ 0\ \)\)]],
  " is just"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(\[CapitalGamma](\[Nu], X\/\[Tau])\)\/\(\
\[CapitalGamma](\[Nu])\)\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This has to be taken into account when valuing options whose payoff \
is non-zero at the origin - a Put being the most obvious case. This \
distribution is exceptional in having both a discrete portion and a \
continuous portion. There are many other interesting probabilistic aspects to \
CEV models, which we shall not consider here. A final reminder - the special \
cases are:

(a) \[Nu] = 1/2 - the absolute Gaussian model;
(b) \[Nu] = 1 - the SRCEV model;
(c) \[Nu] = \[Infinity] - the log-normal model.\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["28.4 CEV Call Option Formulae", "Section",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "By risk-neutral valuation, the value of a European Call with strike ",
  Cell[BoxData[
      \(TraditionalForm\`K\)]],
  " is just"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[ExponentialE]\^\(\(-r\)\ t\)\ \(\[Integral]\_K\%\
\[Infinity]\((S\_T - K)\)\ \(f(S\_T)\) \[DifferentialD]S\_T\)\)], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "There are no known \"nice\" formulae for this. Cox (1996) gives the result \
as an infinite series of Gamma-functions. Cox's model is the standard exact \
solution representation, and can be coded up in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", but requires careful treatment with respect to both evaluation of the \
Gamma-functions and also summation of the infinite series. A more \
down-to-earth representation is the direct integration approach involving the \
Bessel functions (use the routines in Numerical Recipes if working in C) and \
the evaluation of (36). The integral can be simplified a little in terms of a \
standard integral. If we define:"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->Automatic],

Cell[BoxData[
    \(TraditionalForm\`CEVfunc(x, a, \[Nu], 
        n) = \[Integral]\_a\%\[Infinity]\( \[ExponentialE]\^\(-\(z\^2\/\(4\ x\
\)\)\)\ z\^n\ \(\(I\_\[Nu]\)(z)\)\) \[DifferentialD]z\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "then the value of a Call, with strike ",
  Cell[BoxData[
      \(TraditionalForm\`K\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`c\  = \ \[Tau]\^\(-1\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`x\  = \ X/\[Tau]\)]],
  ", is given by"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[ExponentialE]\^\(\(-r\)\ t - x\)\ \((\(CEVfunc(x, 2\ \
\@\(c\ x\ K\^\(1/\[Nu]\)\), \[Nu], \[Nu] + 1)\)\/\(x\ c\^\[Nu]\ 2\^\(\[Nu] + \
1\)\) - K\ \((2\ x)\)\^\(\[Nu] - 1\)\ \(CEVfunc(x, 
              2\ \@\(c\ x\ K\^\(1/\[Nu]\)\), \[Nu], 1 - \[Nu])\))\)\)], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "It is left to the reader to explore an interesting set of special cases \
when the object CEVfunc can be evaluated in closed-form. This occurs when ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\)]],
  " is half an odd integer (cf Schroder, 1989), and provides a useful basis \
for approximating the answer by interpolating bewteen the values obtained for \
half-odd-integer ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\)]],
  ". Our interest will focus on the case of general ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\)]],
  ", and the particular case ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu] = 1\)]],
  "."
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Implementation"
}], "Subsection",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
CEVfunc[x_, a_, \[Nu]_, n_, upper_] := 
  NIntegrate[Exp[-(z^2/(4*x))]*z^n*BesselI[\[Nu], z], {z, a, upper}]\
\>", 
  "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Call Formula", "Subsubsection",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "What we do is to directly integrate to a multiple of the strike, set by \
the variable \"scale\". Note that the ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  Cell[BoxData[
      \(TraditionalForm\`\(\  \)\)]],
  " parameter is set by assuming the volatility is that for the current spot \
price. "
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(CEVCall[S_, \ K_, \ r_, \ q_, \ vol_, \ t_, \ \[Nu]_, \ scale_] := \ \n
      Module[{\[Sigma]\  = \ vol*S^\((1/\((2  \[Nu])\))\), \ x, \ c, \ 
          a}, \n\tc\  = \ \((2*\[Nu]*\((r\  - \ 
                    q)\))\)/\((\[Sigma]^2*\((Exp[\((\((r\  - \ q)\)*
                            t)\)/\[Nu]]\  - \ 1)\))\); \n\t
        x\  = \ c*S^\((1/\[Nu])\)*Exp[\((\((r\  - \ q)\)*t)\)/\[Nu]]; \n\t
        a\  = \ 2\ Sqrt[c\ x\ K^\((1/\[Nu])\)]; \n\t\tExp[\(-r\)*t\  - \ 
              x]*\((CEVfunc[x, a, \ \[Nu], \ \[Nu]\  + \ 1, \ 
                  scale*a]/\((x*
                    c^\[Nu]*2^\((\[Nu]\  + \ 
                          1)\))\)\  - \ \n\ \ \ \ K*\((\((2*
                        x)\)^\((\[Nu]\  - \ 1)\))\)*
                CEVfunc[x, \ a, \ \[Nu], \ 1\  - \ \[Nu], \ scale*a])\)]\)], 
  "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Here it is evaluated for a high value of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\  = \ 10\)]],
  ", which should be close to the Black-Scholes value:"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(CEVCall[100, \ 100, \ 0.1, \ 0, \ 0.2, \ 1, \ 10, \ 10]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(13.269711018867260888374`10.7595\)], "Output"]
}, Open  ]],

Cell["Is it close?", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Derivatives`BlackScholes`\>"]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesCall[100, 100, 0.2, \ 0.1, \ 0, 1]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(13.2696765846608921`\)], "Output"]
}, Open  ]],

Cell["\<\
The answer is yes. As we lower the parameter the value increases \
for these at-the-money examples:\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(CEVCall[100, \ 100, \ 0.1, \ 0, \ 0.2, \ 1, \ 5, \ 10]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(13.269814333577793517597`11.355\)], "Output"]
}, Open  ]],

Cell["Here is the SRCEV case:", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(CEVCall[100, \ 100, \ 0.1, \ 0, \ 0.2, \ 1, \ 1, \ 10]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(13.2731300341921354`\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["28.5 A Simplified Exact Model for SRCEV", "Section",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "There are special cases where the integral can be simplified \
significantly. In addition to the case of \[Nu] equal to half an odd integer, \
where closed-form answers exist, I believe a useful simplification can be \
obtained for integer values also, and the following explores the simplest \
case of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu] = 1\)]],
  ", corresponding to the SRCEV model. The evaluation can be simplified \
dramatically by expressing the Bessel functions using an integral identity, \
and then doing the integration above. This still leaves an integral. However, \
the integrand now involves much simpler functions, and the remaining integral \
is typically over a simple finite interval such as ",
  Cell[BoxData[
      \(TraditionalForm\`\([0, \ \[Pi]]\)\)]],
  ". The integration of periodic functions over such intervals is \
particularly simple, due to the special nature of the numerical error formula \
for periodic integrals. It can be evaluated to any desired degree of \
precision, but in practice only a few samples are needed.  For example, in \
the case ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\  = \ 1\)]],
  ", we can write"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(I\_1\)(
        z)\  = \ \(1\/\[Pi]\) \(\[Integral]\_0\%\[Pi]\(\( cos(
                x)\)\ \[ExponentialE]\^\(z\ \(cos(x)\)\)\) \
\[DifferentialD]x\)\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Integral identities similar to this do exist for other values of \
\[Nu]. The difficult integral from the strike to infinity can be done in \
closed form in terms of the standard cumulative Normal distribution. The \
result is as follows - let\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Alpha] = 2\ c\ K; \[Beta] = 2\ x; \)], 
  "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`f(\[Alpha], \[Beta], \[Theta]) = \((\[Beta]\ \
\(\(cos\^2\)(\[Theta])\) - \[Alpha] + 1)\)\ \((1 - 
              N(\@\[Alpha] - \(cos(\[Theta])\)\ \@\[Beta]))\)\ \
\@\(\[Beta]\/\(2\ \[Pi]\)\) + \(\@\[Beta]\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\((\@\[Alpha] - \(cos(\[Theta])\)\ \@\[Beta])\)\^2\)\ \((\@\[Alpha] + \(cos(\
\[Theta])\)\ \@\[Beta])\)\)\/\(2\ \[Pi]\)\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["Then the value of a call option is just", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(1\/c\) \[ExponentialE]\^\(\(-r\)\ t - \[Beta]\/2\)\ \
\(\[Integral]\_0\%\[Pi]\(\( 
              cos(\[Theta])\)\ \[ExponentialE]\^\(1\/2\ \[Beta]\ \
\(\(cos\^2\)(\[Theta])\)\)\ \(f(\[Alpha], \[Beta], \[Theta])\)\) \
\[DifferentialD]\[Theta]\)\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Let's first build a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " implementation:"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(interfunc[\[Alpha]_, \ \[Beta]_, \ \[Theta]_]\  := \ 
      Sqrt[\[Beta]/\((2\ Pi)\)]*\((1 - \[Alpha] + \[Beta]\ Cos[\[Theta]]^2)\) \
\((1\  - \ Norm[
                Sqrt[\[Alpha]]\  - \ 
                  Cos[\[Theta]]*Sqrt[\[Beta]]])\)\  + \ \((Sqrt[\[Beta]]/\((2*
                  Pi)\))\)*\((Sqrt[\[Alpha]]\  + \ 
              Cos[\[Theta]]*Sqrt[\[Beta]])\)*
          Exp[\(-1\)/
                2 \((Sqrt[\[Alpha]]\  - \ 
                    Cos[\[Theta]]*Sqrt[\[Beta]])\)^2]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(SRCEVCall[S_, \ K_, \ r_, \ q_, \ \[Sigma]_, \ t_] := \n\t
      Module[{sig = \ \[Sigma]*Sqrt[S], \ \[Alpha], \ \[Beta], \ c, \ 
          int}, \n\t\tc\  = \ 
          2*\((r - 
                  q)\)/\((sig^2*\((Exp[\((r - q)\)*t] - 
                      1)\))\); \n\t\t\[Alpha]\  = \ 
          2*c*K; \n\t\t\[Beta]\  = \ 
          2*c*S*Exp[\((r - q)\)*t]; \n\t\tint\  = \ 
          NIntegrate[
            Cos[\[Theta]]\ Exp[\[Beta]\ Cos[\[Theta]]^2/
                    2]\ interfunc[\[Alpha], \ \[Beta], \ \[Theta]], \ {\
\[Theta], \ 0, \ Pi}]; \n\t\tExp[\(-r\)*t - \[Beta]/2]*int/c]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(SRCEVCall[100, \ 100, \ 0.1, \ 0, \ 0.2, \ 1]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(13.2731300259127338`\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "This evaluates rather more quickly, and rather than involving Bessel \
functions and an infinite integration, involves only the normal distribution \
and a finite integration. Because of the nature of periodic functions, the \
integration can be done very efficiently using the trapezoidal rule and \
relatively  few points. We halve the integration done from ",
  Cell[BoxData[
      \(TraditionalForm\`\([0, \ 2  \[Pi]]\)\)]],
  " and proceeding by just summing the values of the integrand sampled at ",
  Cell[BoxData[
      \(TraditionalForm\`\((0, \ 2  \[Pi]/n, \ 
        4\ \[Pi]/n, \( ...  k*2\ \[Pi]/n\), \( ...  
          2  \[Pi](1 - 1/n)\))\)\)]],
  ":"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(NumericalSRCEVCall[S_, \ K_, \ r_, \ q_, \ \[Sigma]_, \ t_, \ 
        samples_] := \n\t
      Module[{sig = \ \[Sigma]*Sqrt[S], \ \[Alpha], \ \[Beta], \ c, \ 
          sum}, \n\t\tc\  = \ 
          2*\((r - 
                  q)\)/\((sig^2*\((Exp[\((r - q)\)*t] - 
                      1)\))\); \n\t\t\[Alpha]\  = \ 
          2*c*K; \n\t\t\[Beta]\  = \ 
          2*c*S*Exp[\((r - q)\)*t]; \n\t\tsum\  = \ \((2\ Pi)\)/samples\ Sum[
              Cos[\[Theta]]\ Exp[\[Beta]\ Cos[\[Theta]]^2/
                      2]\ interfunc[\[Alpha], \ \[Beta], \ \[Theta]], \ {\
\[Theta], \ 0, \ 2*Pi \((1\  - \ 1/samples)\), \ 
                2\ Pi/samples}]; \n\t\tExp[\(-r\)*t - \[Beta]/2]*
          sum/\((2*c)\)]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
60 terms are enough to get agreement to six decimal places, and it \
evaluates very quickly. \
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(NumericalSRCEVCall[100, \ 100, \ 0.1, \ 0, \ 0.2, \ 1, \ 60]\)], "Input",\

  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(13.2731395111651062`\)], "Output"]
}, Open  ]],

Cell["\<\
This is therefore a trivial addition to any derivatives modelling \
system where the cumulative normal distribution is already coded up, and is \
exact within the framework of the model - any desired degree of precision may \
be obtained by taking more terms in the summation.\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "28.6 The Cox-Beckers Approximation to ",
  Cell[BoxData[
      \(TraditionalForm\`SRCEV\)]],
  " "
}], "Section",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This just involves a nest of substitutions and what looks like a normal \
approximation to the non-central chi-square distribution whose pdf we have \
given. It is just a bunch of formulae, but implementation is complicated by \
the fact that the literature has, what appears to this author at least, three \
different and inconsistent representations of this formula. If you wish to \
make your own evaluation of these comments, four papers all containing a \
version of this approximation are Beckers (1980), Jarrow and Rudd (1983), \
Lauterbach and Schultz (1990), Hauser and Lauterbach (1996). In the \
following, ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " is essentially ",
  Cell[BoxData[
      \(TraditionalForm\`\(c\ \)\)]],
  "times the underlying. The implementation in here is based on Jarrow and \
Rudd's (1983) book, which in the humble opinion of this author is the correct \
version, but this is based on the subsequent comparison with the exact \
solution. I cannot claim to have gone into the approximation of the required \
distributions by normal distributions in any detail, but the following \
appears also to be consistent with the remarks by Schroder (1989) who also \
gives more general results that apply for non-zero yield. In any case it is \
probably more efficient to use the exact new model given above, and the \
following is largely included for completeness. Here are the formulae:"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(y[S_, \ r_, \ \[Sigma]_, \ t_]\  := \ 
      4*r*S/\((\[Sigma]^2\ \((1\  - \ Exp[\(-r\)*t])\))\); \n
    z[K_, \ r_, \ \[Sigma]_, \ t_]\  := \ \ 4*r*
        K/\((\[Sigma]^2\ \((Exp[r*t] - 1)\))\); \n
    p[v_, \ y_]\  := \ \((v + 2\ y)\)/\((v + y)\)^2; \n
    h[v_, \ y_]\  := \ 
      1\  - \ 2*\((v + y)\)*\(\((v + 3  y)\)/3\)/\((v + 2\ y)\)^2; \n
    q[v_, \ h_, \ p_, \ z_, \ 
        y_]\  := \ \((1\  + \ h*\((h - 1)\)*p\ \  - \ 
            1/2\ h \((h - 1)\) \((2 - h)\) \((1 - 3  h)\) 
              p^2\  - \ \((z/\((v + y)\))\)^h)\)/
        Sqrt[2*h^2\ p\ \((1\  - \ \((1 - h)\) \((1 - 3\ h)\) p)\)]; \)], 
  "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["The Call formula is then", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`S\ \(N(q(4, h(4, Y), p(4, Y), Z, Y))\) - 
      X\ \[ExponentialE]\^\(\(-r\)\ t\)\ \(N(
          q(0, h(0, Y), p(0, Y), Z, Y))\)\)], "NumberedEquation",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " representation of this is"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(CBAppSRCEVCall[S_, \ X_, \ r_, \ \[Sigma]_, \ t_]\  := \ 
      Module[{sigone, \ Y, \ Z, \ Pzero, \ Pfour, \ Hzero, \ Hfour, \ 
          Qzero, \ Qfour}, \n\t\tsigone\  = \ \[Sigma]*Sqrt[S]; \ 
        Y\  = \ y[S, \ r, \ sigone, \ t]; \tZ\  = \ z[X, \ r, \ sigone, t]; \ 
        Pzero\  = \ p[0, Y]; \t
        Pfour\  = \ p[4, Y]; \n\t\tHzero\  = \ h[0, \ Y]; \ 
        Hfour\  = \ h[4, \ Y]; \[IndentingNewLine]Qzero\  = \ 
          q[0, \ Hzero, \ Pzero, \ Z, \ Y]; \n\t\tQfour\  = \ 
          q[4, \ Hfour, \ Pfour, \ Z, \ Y]; \n\t\tS*Norm[Qfour]\  - \ 
          X\ Exp[\(-r\)*t]*Norm[Qzero]]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Comparisons with Exact Solution", "Subsection",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Let's look at the approximate solution vs the exact solution for \
ATM, ITM, OTM cases:\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \({CBAppSRCEVCall[100, \ 100, \ 0.1, \ 0.2, \ 1], 
      SRCEVCall[100, \ 100, \ 0.1, \ 0, \ 0.2, \ 1]}\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({13.2762542871378319`, 13.2731300259127338`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({CBAppSRCEVCall[110, \ 100, \ 0.1, \ 0.2, \ 1], 
      SRCEVCall[110, \ 100, \ 0.1, \ 0, \ 0.2, \ 1]}\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({21.3726823451241765`, 21.369915300858624`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({CBAppSRCEVCall[90, \ 100, \ 0.1, \ 0.2, \ 1], 
      SRCEVCall[90, \ 100, \ 0.1, \ 0, \ 0.2, \ 1]}\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({6.76843226214641902`, 6.76697222916751961`}\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["28.7 Cox's Gamma-Function Series", "Section",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
For completeness the exact solution given by Cox is presented. We \
can then compare this with the direct integration of the discounted \
risk-neutral expiry distribution. We need first to define some \
Gamma-functions. \
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["?Gamma", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \("Gamma[z] is the Euler gamma function. Gamma[a, z] is the incomplete \
gamma function. Gamma[a, z0, z1] is the generalized incomplete gamma function \
Gamma[a, z0] - Gamma[a, z1]."\)], "Print"]
}, Open  ]],

Cell["g[n_, z_] := Exp[-z] z^(n-1)/Gamma[n] ", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["G1[n_, w_] = Integrate[g[n,w], w]", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(-\(Gamma[n, w]\/Gamma[n]\)\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "This ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  " is the one defined by Cox - it is the complementary cumulative \
distribution function."
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["G[n_, w_] := Gamma[n, w]/Gamma[n]", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["Next we need a series as given by Cox:", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
CoxSeries[a_, b_, x_, y_, m_] := Sum[g[n+a,x]*G[n+b,y], {n, 1, \
m}]\
\>", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
PartialCoxSeries[a_, b_, x_, y_, kk_, m_] := Sum[g[n+a,x]*G[n+b,y], \
{n, kk, m}]\
\>", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["Here is what the series looks like for a few terms:", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["CoxSeries[a,b,x,y,2]", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(E\^\(-x\)\ x\^a\ Gamma[1 + b, y]\)\/\(Gamma[1 + a]\ Gamma[1 + b]\) + \
\(E\^\(-x\)\ x\^\(1 + a\)\ Gamma[2 + b, y]\)\/\(Gamma[2 + a]\ Gamma[2 + \
b]\)\)], "Output"]
}, Open  ]],

Cell["\<\
In the following formula the elasticity parameter corresponds to \
\[Alpha], and the yield is zero.\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
SeriesCEVOption[p_, k_, vol_, r_, t_, elas_, trunc_] :=
Module[{lambda = 0.5/(1-elas), sig = vol*p^(1-elas), x, y, rtl},
rtl = Exp[r(t/lambda)];
x = 2 lambda r p^(1/lambda) rtl/((rtl - 1)*sig^2);
y = 2 lambda r k^(1/lambda)/((rtl - 1)*sig^2);
p*CoxSeries[0,lambda, x, y, trunc] - k*Exp[-r*t]*CoxSeries[lambda,0,x,y, \
trunc]]\
\>", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(SeriesCEVOption[S, K, \[Sigma], r, t, e, n]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{
      RowBox[{\(-E\^\(\(-0.06`\)\ t\)\), " ", "K", " ", 
        RowBox[{\(\[Sum]\+\(n = 1\)\%n\), 
          RowBox[{
            RowBox[{"g", "[", 
              RowBox[{
                RowBox[{"n", "+", 
                  FractionBox[
                    StyleBox["0.5`",
                      StyleBoxAutoDelete->True,
                      PrintPrecision->1], \(1 - e\)]}], 
                ",", \(\(0.06`\ E\^\(0.119999999999999996`\ \((1 - e)\)\ t\)\ \
S\^\(\(-2\) + 2.`\ \((1 - e)\) + 2\ e\)\)\/\(\((1 - e)\)\ \((\(-1\) + 
                          E\^\(0.119999999999999996`\ \((1 - e)\)\ t\))\)\ \
\[Sigma]\^2\)\)}], "]"}], 
            " ", \(G[
              n + 0, \(0.06`\ K\^\(2.`\ \((1 - e)\)\)\ S\^\(\(-2\) + 2\ e\)\)\
\/\(\((1 - e)\)\ \((\(-1\) + E\^\(0.119999999999999996`\ \((1 - e)\)\ t\))\)\ \
\[Sigma]\^2\)]\)}]}]}], "+", 
      RowBox[{"S", " ", 
        RowBox[{\(\[Sum]\+\(n = 1\)\%n\), 
          
          RowBox[{\(g[
              n + 0, \(0.06`\ E\^\(0.119999999999999996`\ \((1 - e)\)\ t\)\ S\
\^\(\(-2\) + 2.`\ \((1 - e)\) + 2\ e\)\)\/\(\((1 - e)\)\ \((\(-1\) + \
E\^\(0.119999999999999996`\ \((1 - e)\)\ t\))\)\ \[Sigma]\^2\)]\), " ", 
            RowBox[{"G", "[", 
              RowBox[{
                RowBox[{"n", "+", 
                  FractionBox[
                    StyleBox["0.5`",
                      StyleBoxAutoDelete->True,
                      PrintPrecision->1], \(1 - e\)]}], 
                ",", \(\(0.06`\ K\^\(2.`\ \((1 - e)\)\)\ S\^\(\(-2\) + 2\ e\)\
\)\/\(\((1 - e)\)\ \((\(-1\) + 
                          E\^\(0.119999999999999996`\ \((1 - e)\)\ t\))\)\ \
\[Sigma]\^2\)\)}], "]"}]}]}]}]}]], "Output"]
}, Open  ]],

Cell["\<\
With enough terms in the series, we can compute a stable result. \
Here we give the SRCEV values.\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["SeriesCEVOption[100, 100, 0.20, 0.1, 1, 0.5, 140]", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(13.2731300247191508`\)], "Output"]
}, Open  ]],

Cell["Just to check that we have enough terms:", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["SeriesCEVOption[100, 100, 0.20, 0.1, 1, 0.5, 150]", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(13.2731300247191508`\)], "Output"]
}, Open  ]],

Cell["\<\
Now we can look closer to Black-Scholes, by increasing the \
elasticity to be closer to unity. This requires very many more terms. The \
formula is singular when the elasticity is unity, so we set it to 0.95, to \
compare with the Bessel function form. Counting in thousands, 6000 terms are \
both necessary and sufficient for 6 SF: \
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["SeriesCEVOption[100, 100, 0.20, 0.1, 1, 0.95, 5000]", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(1.69941862058648496`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["SeriesCEVOption[100, 100, 0.20, 0.1, 1, 0.95, 6000]", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(13.2697110188004141`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["SeriesCEVOption[100, 100, 0.20, 0.1, 1, 0.95, 7000]", "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(13.2697110188004141`\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["28.8 Skews and Smiles by \"Real-World\" Option Pricing", "Section",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
The idea in this section is to explore what happens when the price \
computed by an allegedly more realistic model (here a CEV model) is fed \
backwards through Black-Scholes, to work out the implied volatility. The \
point to be made is that there is a simple mechanism for the emergence of a \
skew, based on an elasticity approach. Note that this is not due to the \
volatility going up as the price at initiation goes down, for we are fixing \
the volatility as a fixed constant at time zero. The skew arises as a result \
of subsequent evolution of the volatility as the asset price changes. \
\>", 
  "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Derivatives`BlackScholes`\>"]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?BlackScholesCallImpVol\)\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("BlackScholesCallImpVol[price, strike, riskfree, divyield, expiry, \
optionprice]  returns the implied volatility of a vanilla European Call."\)], 
  "Print"]
}, Open  ]],

Cell["\<\
So we fix the volatility, price with the SRCEV model, then infer \
the implied volatility from the Black-Scholes model. As before, the \[Sigma] \
parameter is set simply according to the scale of variation of the \
underlying.\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(ForwardBackward[S_, \ K_, \ r_, \ q_, \ \[Sigma]_, \ t_]\  := \n\t
      BlackScholesCallImpVol[S, \ K, \ r, q, \ t, \ 
        SRCEVCall[S, \ K, \ r, q, \ \[Sigma], \ t]]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \({ForwardBackward[100, \ 90, \ 0.1, \ 0, \ 0.2, \ 
        1], \[IndentingNewLine]ForwardBackward[100, \ 100, \ 0.1, \ 0, \ 
        0.2, \ 1], \[IndentingNewLine]ForwardBackward[100, \ 110, \ 0.1, \ 
        0, \ 0.2, \ 1]}\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.205380094351148478`, 0.200103637043059601`, 
      0.195408596763655317`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Skew or Half a Smile", "Subsection",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This model does capture a volatility skew typical of major indices \
under certain circumstances - the volatility at time zero is always 20%, but \
the volatility implied from the BS model varies from nearly 22% with a strike \
at 70, to under 19% with a strike at 130. Note that the out-of-the-money end \
is at the right of this plot.\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      ForwardBackward[100, \ K, \ 0.05, \ 0, \ 0.2, \ 1], \ {K, \ 70, \ 
        130}]; \)\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.0873 0.015873 -3.52965 18.9242 [
[.18254 .24269 -6 -9 ]
[.18254 .24269 6 0 ]
[.34127 .24269 -6 -9 ]
[.34127 .24269 6 0 ]
[.5 .24269 -9 -9 ]
[.5 .24269 9 0 ]
[.65873 .24269 -9 -9 ]
[.65873 .24269 9 0 ]
[.81746 .24269 -9 -9 ]
[.81746 .24269 9 0 ]
[.97619 .24269 -9 -9 ]
[.97619 .24269 9 0 ]
[.01131 .06595 -24 -4.5 ]
[.01131 .06595 0 4.5 ]
[.01131 .16057 -30 -4.5 ]
[.01131 .16057 0 4.5 ]
[.01131 .34982 -30 -4.5 ]
[.01131 .34982 0 4.5 ]
[.01131 .44444 -24 -4.5 ]
[.01131 .44444 0 4.5 ]
[.01131 .53906 -30 -4.5 ]
[.01131 .53906 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .25519 m
.18254 .26144 L
s
[(80)] .18254 .24269 0 1 Mshowa
.34127 .25519 m
.34127 .26144 L
s
[(90)] .34127 .24269 0 1 Mshowa
.5 .25519 m
.5 .26144 L
s
[(100)] .5 .24269 0 1 Mshowa
.65873 .25519 m
.65873 .26144 L
s
[(110)] .65873 .24269 0 1 Mshowa
.81746 .25519 m
.81746 .26144 L
s
[(120)] .81746 .24269 0 1 Mshowa
.97619 .25519 m
.97619 .26144 L
s
[(130)] .97619 .24269 0 1 Mshowa
.125 Mabswid
.05556 .25519 m
.05556 .25894 L
s
.0873 .25519 m
.0873 .25894 L
s
.11905 .25519 m
.11905 .25894 L
s
.15079 .25519 m
.15079 .25894 L
s
.21429 .25519 m
.21429 .25894 L
s
.24603 .25519 m
.24603 .25894 L
s
.27778 .25519 m
.27778 .25894 L
s
.30952 .25519 m
.30952 .25894 L
s
.37302 .25519 m
.37302 .25894 L
s
.40476 .25519 m
.40476 .25894 L
s
.43651 .25519 m
.43651 .25894 L
s
.46825 .25519 m
.46825 .25894 L
s
.53175 .25519 m
.53175 .25894 L
s
.56349 .25519 m
.56349 .25894 L
s
.59524 .25519 m
.59524 .25894 L
s
.62698 .25519 m
.62698 .25894 L
s
.69048 .25519 m
.69048 .25894 L
s
.72222 .25519 m
.72222 .25894 L
s
.75397 .25519 m
.75397 .25894 L
s
.78571 .25519 m
.78571 .25894 L
s
.84921 .25519 m
.84921 .25894 L
s
.88095 .25519 m
.88095 .25894 L
s
.9127 .25519 m
.9127 .25894 L
s
.94444 .25519 m
.94444 .25894 L
s
.25 Mabswid
0 .25519 m
1 .25519 L
s
.02381 .06595 m
.03006 .06595 L
s
[(0.19)] .01131 .06595 1 0 Mshowa
.02381 .16057 m
.03006 .16057 L
s
[(0.195)] .01131 .16057 1 0 Mshowa
.02381 .34982 m
.03006 .34982 L
s
[(0.205)] .01131 .34982 1 0 Mshowa
.02381 .44444 m
.03006 .44444 L
s
[(0.21)] .01131 .44444 1 0 Mshowa
.02381 .53906 m
.03006 .53906 L
s
[(0.215)] .01131 .53906 1 0 Mshowa
.125 Mabswid
.02381 .08488 m
.02756 .08488 L
s
.02381 .1038 m
.02756 .1038 L
s
.02381 .12273 m
.02756 .12273 L
s
.02381 .14165 m
.02756 .14165 L
s
.02381 .1795 m
.02756 .1795 L
s
.02381 .19842 m
.02756 .19842 L
s
.02381 .21735 m
.02756 .21735 L
s
.02381 .23627 m
.02756 .23627 L
s
.02381 .27412 m
.02756 .27412 L
s
.02381 .29304 m
.02756 .29304 L
s
.02381 .31197 m
.02756 .31197 L
s
.02381 .33089 m
.02756 .33089 L
s
.02381 .36874 m
.02756 .36874 L
s
.02381 .38766 m
.02756 .38766 L
s
.02381 .40659 m
.02756 .40659 L
s
.02381 .42551 m
.02756 .42551 L
s
.02381 .46336 m
.02756 .46336 L
s
.02381 .48229 m
.02756 .48229 L
s
.02381 .50121 m
.02756 .50121 L
s
.02381 .52013 m
.02756 .52013 L
s
.02381 .04703 m
.02756 .04703 L
s
.02381 .0281 m
.02756 .0281 L
s
.02381 .00918 m
.02756 .00918 L
s
.02381 .55798 m
.02756 .55798 L
s
.02381 .57691 m
.02756 .57691 L
s
.02381 .59583 m
.02756 .59583 L
s
.02381 .61476 m
.02756 .61476 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .56925 L
.10458 .53358 L
.14415 .50139 L
.18221 .47154 L
.22272 .44088 L
.26171 .4124 L
.30316 .38315 L
.34309 .35591 L
.3815 .33053 L
.42237 .30437 L
.46172 .27994 L
.49955 .25713 L
.53984 .23352 L
.57861 .21144 L
.61984 .18861 L
.65954 .16722 L
.69774 .14717 L
.73838 .12638 L
.77751 .10688 L
.81909 .08666 L
.85916 .06767 L
.89771 .04982 L
.93871 .03128 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{202, 124.625},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  Background->GrayLevel[1],
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003:0000O1000`40O003h00ObWoo003:Ool00<Yo
o`00bWoo003:Ool00<Yoo`00bWoo003:Ool00<Yoo`00bWoo000POol00`00Oomoo`2WOol0021oo`03
001oogoo0:Moo`0087oo00<007ooOol0W7oo0P002Goo000POol00`00Oomoo`2IOol3000;Ool0021o
o`03001oogoo09Moo`8000ioo`0087oo00<007ooOol0UGoo0P0047oo000POol00`00Oomoo`2BOol3
000BOol0021oo`03001oogoo091oo`8001Eoo`0087oo00<007ooOol0SWoo0P005goo0008Ool20004
Ool20003Ool30003Ool30004Ool00`00Oomoo`2<Ool2000IOol000Moo`04001oogoo0003Ool20004
Ool00`00Oomoo`05Ool01@00Oomoogoo0000S7oo0P006goo0007Ool01000Oomoo`002Goo00<007oo
Ool00goo0`000goo0P00RGoo0P007Goo0007Ool01000Oomoo`002Goo00<007ooOol00Woo00@007oo
Ool000=oo`03001oogoo08Eoo`<001moo`001goo00@007ooOol000Uoo`03001oogoo009oo`04001o
ogoo0003Ool00`00Oomoo`23Ool2000ROol000Qoo`8000Uoo`8000Eoo`8000Aoo`03001oogoo085o
o`8002Aoo`0087oo00<007ooOol0Ogoo0P009Woo000POol00`00Oomoo`1nOol00`00Oomoo`0VOol0
021oo`03001oogoo07aoo`8002Uoo`0087oo00<007ooOol0NWoo0P00:goo000POol00`00Oomoo`1h
Ool2000]Ool0021oo`03001oogoo07Ioo`8002moo`0087oo00<007ooOol0M7oo0P00<Goo000POol0
0`00Oomoo`1bOol2000cOol0021oo`03001oogoo075oo`03001oogoo03=oo`0087oo00<007ooOol0
Kgoo0P00=Woo0002Ool20004Ool20003Ool30003Ool30003Ool30004Ool00`00Oomoo`1]Ool2000h
Ool00005Ool007ooOol00003Ool20004Ool00`00Oomoo`05Ool00`00Oomoo`03Ool01@00Oomoogoo
0000K7oo0`00>Woo00001Goo001oogoo00002Goo00<007ooOol00goo0`001Goo00@007ooOomoo`80
06Uoo`8003eoo`0000Eoo`00Oomoo`0000Uoo`03001oogoo009oo`04001oogoo0002Ool30004Ool0
0`00Oomoo`1VOol2000oOol00005Ool007ooOol00009Ool00`00Oomoo`02Ool01000Oomoo`000Woo
00<007ooOol017oo00<007ooOol0I7oo0P00@Goo0002Ool20009Ool20005Ool20003Ool40003Ool0
0`00Oomoo`1ROol20013Ool0021oo`03001oogoo061oo`8004Eoo`0087oo00<007ooOol0GWoo0P00
Agoo000POol00`00Oomoo`1MOol00`00Oomoo`17Ool0021oo`03001oogoo01=oo`8000Aoo`80019o
o`<000Aoo`8000moo`<000Aoo`8000Aoo`8000Moo`80009oo`<000=oo`<000Aoo`8000aoo`<000=o
o`@000=oo`8000aoo`<000=oo`<000Aoo`8000=oo`0087oo00<007ooOol04Woo00@007ooOol0009o
o`04001oogoo000DOol01000Oomoo`000Woo00<007ooOol03Goo00<007ooOol00Woo00@007ooOol0
009oo`04001oogoo0004Ool20005Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`0037oo
00<007ooOol00goo00<007ooOol00Woo00@007ooOol000aoo`03001oogoo00Eoo`04001oogoo0002
Ool10001Ool1Ool0021oo`03001oogoo019oo`04001oogoo0002Ool01000Oomoo`004Woo0`000Woo
00@007ooOol000moo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`000Woo0P001goo00<0
07ooOol00goo00<007ooOol00Woo00@007ooOol000aoo`03001oogoo00Aoo`05001oogooOol00002
Ool00`00Oomoo`0:Ool00`00Oomoo`05Ool01000Oomoo`000Woo0@000Goo0Goo000POol00`00Oomo
o`0COol20003Ool01000Oomoo`004Goo00@007ooOol0009oo`04001oogoo000?Ool00`00Oomoo`02
Ool01000Oomoo`000Woo00H007ooOol007oo0009Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01000
Oomoo`0037oo00<007ooOol01Goo00@007ooOol0009oo`03001oogoo00Yoo`03001oogoo00=oo`80
00=oo`04001oogoo0002Ool0021oo`03001oogoo019oo`04001oogoo0002Ool01000Oomoo`004Goo
00@007ooOol0009oo`04001oogoo000?Ool00`00Oomoo`02Ool01000Oomoo`000Woo00<007ooOol0
0P002Woo00<007ooOol00goo00<007ooOol00Woo00@007ooOol000aoo`03001oogoo009oo`04001o
ogoo0002Ool01000Oomoo`0037oo00<007ooOol01Goo00@007ooOol0009oo`40005oo`5oo`0087oo
00<007ooOol04goo0P0017oo0P004goo0P0017oo0P003goo0P001Goo0P0017oo0P002goo0P0017oo
0P001Goo0P0037oo0P001Goo0P0017oo0P0037oo0P0017oo0`0017oo0P000goo000POol00`00Oomo
o`1@Ool2001EOol0021oo`03001oogoo04moo`03001oogoo05Eoo`0087oo00<007ooOol0CGoo0P00
F7oo000LOonX0006Ool0021oo`03001oogoo01Moo`03001oogoo01Qoo`03001oogoo01Eoo`04001o
ogoo000IOol00`00Oomoo`0HOol00`00Oomoo`0HOol00`00Oomoo`07Ool0021oo`03001oogoo04Qo
o`8005eoo`0087oo00<007ooOol0Agoo00<007ooOol0GGoo000POol00`00Oomoo`15Ool2001POol0
021oo`03001oogoo04Aoo`03001oogoo061oo`0087oo00<007ooOol0@Woo0P00Hgoo000POol00`00
Oomoo`11Ool00`00Oomoo`1SOol0021oo`03001oogoo03moo`8006Ioo`0087oo00<007ooOol0?Goo
0P00J7oo000POol00`00Oomoo`0lOol00`00Oomoo`1XOol0021oo`03001oogoo03Yoo`8006]oo`00
87oo00<007ooOol0>7oo0P00KGoo000POol00`00Oomoo`0gOol00`00Oomoo`1]Ool0009oo`8000Ao
o`8000=oo`@000=oo`8000=oo`<000Aoo`03001oogoo03Eoo`80071oo`0000Eoo`00Oomoo`0000=o
o`8000Aoo`03001oogoo009oo`04001oogoo0005Ool01@00Oomoogoo0000=Woo00<007ooOol0L7oo
00001Goo001oogoo00002Woo00D007ooOomoo`00009oo`03001oogoo00=oo`04001oogooOol2000d
Ool00`00Oomoo`1aOol00005Ool007ooOol0000;Ool01000Oomoo`000Woo00<007ooOol00`0017oo
00<007ooOol0<Goo0P00M7oo00001Goo001oogoo000027oo00@007ooOol0009oo`04001oogoo0002
Ool00`00Oomoo`04Ool00`00Oomoo`0`Ool00`00Oomoo`1dOol0009oo`8000Yoo`8000Aoo`8000=o
o`@000=oo`03001oogoo02ioo`8007Moo`0087oo00<007ooOol0;Goo00<007ooOol0Mgoo000POol0
0`00Oomoo`0[Ool2001jOol0021oo`03001oogoo02Yoo`03001oogoo07Yoo`0087oo00<007ooOol0
:7oo0P00OGoo000POol00`00Oomoo`0WOol00`00Oomoo`1mOol0021oo`03001oogoo02Ioo`03001o
ogoo07ioo`0087oo00<007ooOol097oo0P00PGoo000POol00`00Oomoo`0SOol00`00Oomoo`21Ool0
021oo`03001oogoo025oo`8008Aoo`0027oo0P0017oo0P000goo10000Woo0`0017oo00<007ooOol0
87oo00<007ooOol0Q7oo0007Ool01000Oomoo`000goo0P0017oo00<007ooOol00goo00<007ooOol0
0goo00<007ooOol07goo00<007ooOol0QGoo0007Ool01000Oomoo`002Woo00<007ooOol00Woo00<0
07ooOol00goo0P007Woo0P00R7oo0007Ool01000Oomoo`002goo00D007ooOomoo`0000Eoo`03001o
ogoo01aoo`03001oogoo08Qoo`001goo00@007ooOol000Qoo`04001oogoo0003Ool00`00Oomoo`03
Ool00`00Oomoo`0JOol2002;Ool000Qoo`8000Yoo`8000=oo`8000Eoo`03001oogoo01Uoo`03001o
ogoo08]oo`0087oo00<007ooOol067oo00<007ooOol0S7oo000POol00`00Oomoo`0FOol2002?Ool0
021oo`03001oogoo01Eoo`03001oogoo08moo`0087oo00<007ooOol057oo00<007ooOol0T7oo000P
Ool00`00Oomoo`0COol00`00Oomoo`2AOol0021oo`03001oogoo019oo`03001oogoo099oo`0087oo
00<007ooOol047oo0P00UGoo000POol00`00Oomoo`0?Ool00`00Oomoo`2EOol0021oo`03001oogoo
00eoo`8009Qoo`0087oo00<007ooOol037oo00<007ooOol0V7oo0002Ool20004Ool20003Ool40002
Ool30003Ool30004Ool00`00Oomoo`0;Ool00`00Oomoo`2IOol00005Ool007ooOol00003Ool20004
Ool00`00Oomoo`03Ool00`00Oomoo`05Ool01@00Oomoogoo00002goo0P00W7oo00001Goo001oogoo
00002Woo00<007ooOol00Woo00<007ooOol01Goo00@007ooOomoo`8000Uoo`03001oogoo09aoo`00
00Eoo`00Oomoo`0000]oo`05001oogooOol00004Ool30004Ool00`00Oomoo`07Ool00`00Oomoo`2M
Ool00005Ool007ooOol00008Ool01000Oomoo`000goo00<007ooOol00Woo00<007ooOol017oo00<0
07ooOol01Woo00<007ooOol0WWoo0002Ool2000:Ool20003Ool20004Ool40003Ool00`00Oomoo`05
Ool00`00Oomoo`2OOol0021oo`03001oogoo00Aoo`03001oogoo0:1oo`0087oo00<007ooOol00goo
00<007ooOol0XGoo000POol00`00Oomoo`02Ool00`00Oomoo`2ROol0021oo`05001oogooOol0002U
Ool0021oo`04001oogoo002VOol0021oo`03001oo`000:Moo`0087oo0P00Z7oo000POol00`00Oomo
o`2WOol0021oo`03001oogoo0:Moo`0087oo00<007ooOol0Ygoo000POol00`00Oomoo`2WOol00<Yo
o`00bWoo003:Ool00<Yoo`00bWoo003:Ool00<Yoo`00bWoo003:Ool00001\
\>"],
  ImageRangeCache->{{{0, 201}, {123.625, 0}} -> {57.9551, 0.18339, 0.375235, 
  0.000314735}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["28.9 The Effect of Spread", "Section",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell["\<\
Real-world trading can also involve a bid-offer spread. Suppose we \
add 2.5% to the elasticity result and again compute the implied volatility:\
\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(ForwardBackward[S_, \ K_, \ r_, \ q_, \ \[Sigma]_, \ t_, \ 
        spread_]\  := \n\t
      BlackScholesCallImpVol[S, \ K, \ r, q, \ 
        t, \ \((1 + spread/100)\)*
          SRCEVCall[S, \ K, \ r, q, \ \[Sigma], \ t]]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      ForwardBackward[100, \ K, \ 0.05, \ 0, \ 0.2, \ 1, \ 2.5], \ {K, \ 
        70, \ 130}, \n\t
      PlotLabel\  -> \ 
        "\<Skew for Elasticity\n+ Spread Model of Call\n\>"]; \)\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.0873 0.015873 -0.934135 5.02783 [
[.18254 .05893 -6 -9 ]
[.18254 .05893 6 0 ]
[.34127 .05893 -6 -9 ]
[.34127 .05893 6 0 ]
[.5 .05893 -9 -9 ]
[.5 .05893 9 0 ]
[.65873 .05893 -9 -9 ]
[.65873 .05893 9 0 ]
[.81746 .05893 -9 -9 ]
[.81746 .05893 9 0 ]
[.97619 .05893 -9 -9 ]
[.97619 .05893 9 0 ]
[.01131 .17199 -24 -4.5 ]
[.01131 .17199 0 4.5 ]
[.01131 .27254 -24 -4.5 ]
[.01131 .27254 0 4.5 ]
[.01131 .3731 -24 -4.5 ]
[.01131 .3731 0 4.5 ]
[.01131 .47366 -24 -4.5 ]
[.01131 .47366 0 4.5 ]
[.01131 .57421 -18 -4.5 ]
[.01131 .57421 0 4.5 ]
[.5 .63053 -68 0 ]
[.5 .63053 68 37 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .07143 m
.18254 .07768 L
s
[(80)] .18254 .05893 0 1 Mshowa
.34127 .07143 m
.34127 .07768 L
s
[(90)] .34127 .05893 0 1 Mshowa
.5 .07143 m
.5 .07768 L
s
[(100)] .5 .05893 0 1 Mshowa
.65873 .07143 m
.65873 .07768 L
s
[(110)] .65873 .05893 0 1 Mshowa
.81746 .07143 m
.81746 .07768 L
s
[(120)] .81746 .05893 0 1 Mshowa
.97619 .07143 m
.97619 .07768 L
s
[(130)] .97619 .05893 0 1 Mshowa
.125 Mabswid
.05556 .07143 m
.05556 .07518 L
s
.0873 .07143 m
.0873 .07518 L
s
.11905 .07143 m
.11905 .07518 L
s
.15079 .07143 m
.15079 .07518 L
s
.21429 .07143 m
.21429 .07518 L
s
.24603 .07143 m
.24603 .07518 L
s
.27778 .07143 m
.27778 .07518 L
s
.30952 .07143 m
.30952 .07518 L
s
.37302 .07143 m
.37302 .07518 L
s
.40476 .07143 m
.40476 .07518 L
s
.43651 .07143 m
.43651 .07518 L
s
.46825 .07143 m
.46825 .07518 L
s
.53175 .07143 m
.53175 .07518 L
s
.56349 .07143 m
.56349 .07518 L
s
.59524 .07143 m
.59524 .07518 L
s
.62698 .07143 m
.62698 .07518 L
s
.69048 .07143 m
.69048 .07518 L
s
.72222 .07143 m
.72222 .07518 L
s
.75397 .07143 m
.75397 .07518 L
s
.78571 .07143 m
.78571 .07518 L
s
.84921 .07143 m
.84921 .07518 L
s
.88095 .07143 m
.88095 .07518 L
s
.9127 .07143 m
.9127 .07518 L
s
.94444 .07143 m
.94444 .07518 L
s
.25 Mabswid
0 .07143 m
1 .07143 L
s
.02381 .17199 m
.03006 .17199 L
s
[(0.22)] .01131 .17199 1 0 Mshowa
.02381 .27254 m
.03006 .27254 L
s
[(0.24)] .01131 .27254 1 0 Mshowa
.02381 .3731 m
.03006 .3731 L
s
[(0.26)] .01131 .3731 1 0 Mshowa
.02381 .47366 m
.03006 .47366 L
s
[(0.28)] .01131 .47366 1 0 Mshowa
.02381 .57421 m
.03006 .57421 L
s
[(0.3)] .01131 .57421 1 0 Mshowa
.125 Mabswid
.02381 .09657 m
.02756 .09657 L
s
.02381 .12171 m
.02756 .12171 L
s
.02381 .14685 m
.02756 .14685 L
s
.02381 .19713 m
.02756 .19713 L
s
.02381 .22227 m
.02756 .22227 L
s
.02381 .24741 m
.02756 .24741 L
s
.02381 .29768 m
.02756 .29768 L
s
.02381 .32282 m
.02756 .32282 L
s
.02381 .34796 m
.02756 .34796 L
s
.02381 .39824 m
.02756 .39824 L
s
.02381 .42338 m
.02756 .42338 L
s
.02381 .44852 m
.02756 .44852 L
s
.02381 .4988 m
.02756 .4988 L
s
.02381 .52394 m
.02756 .52394 L
s
.02381 .54907 m
.02756 .54907 L
s
.02381 .04629 m
.02756 .04629 L
s
.02381 .02115 m
.02756 .02115 L
s
.02381 .59935 m
.02756 .59935 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
gsave
.5 .63053 -129 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 45 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 41.000 moveto
 541.000 41.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 41.000 lineto
closepath
clip newpath
63.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Skew) show
93.000 13.000 moveto
(for) show
117.000 13.000 moveto
(Elasticity) show
63.000 25.500 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(+) show
75.000 25.500 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Spread) show
117.000 25.500 moveto
(Model) show
153.000 25.500 moveto
(of) show
171.000 25.500 moveto
(Call) show
63.000 38.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .51432 L
.10458 .43091 L
.14415 .36509 L
.18221 .31229 L
.22272 .26598 L
.26171 .22953 L
.30316 .19785 L
.34309 .17279 L
.3815 .15264 L
.42237 .1345 L
.46172 .11955 L
.49955 .107 L
.53984 .09518 L
.57861 .085 L
.61984 .07522 L
.65954 .06662 L
.69774 .05896 L
.73838 .05138 L
.77751 .04453 L
.81909 .03767 L
.85916 .03141 L
.89771 .02568 L
.93871 .01984 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  ImageSize->{225, 138.688},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003Q0000RQ000`40O003h00O;7oo00<007ooOol0
/Woo000/Ool00`00Oomoo`2bOol002aoo`03001oogoo019oo`8000Aoo`80015oo`<000Aoo`8000eo
o`<000Aoo`8000Aoo`8000Yoo`<000=oo`<000Aoo`8000]oo`<000=oo`@000=oo`8000Yoo`<0009o
o`@000Aoo`8001Ioo`00;7oo0P004Woo00@007ooOol0009oo`04001oogoo000COol01000Oomoo`00
0Woo00<007ooOol02goo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo000:Ool00`00Oomo
o`03Ool00`00Oomoo`02Ool01000Oomoo`002goo00<007ooOol00goo00<007ooOol00Woo00@007oo
Ool000Qoo`H000Aoo`04001oogoo0002Ool00`00Oomoo`0COol002aoo`03001oogoo015oo`04001o
ogoo0002Ool01000Oomoo`004Goo0`000Woo00@007ooOol000eoo`03001oogoo009oo`04001oogoo
0002Ool01000Oomoo`002Woo00<007ooOol00goo00<007ooOol00Woo00@007ooOol000]oo`03001o
ogoo00Aoo`05001oogooOol00002Ool00`00Oomoo`060002Ool00`00Oomoo`05Ool01000Oomoo`00
0Woo00<007ooOol04goo000/Ool00`00Oomoo`0BOol20003Ool01000Oomoo`0047oo00@007ooOol0
009oo`04001oogoo000=Ool00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol000Yoo`03001o
ogoo00=oo`03001oogoo009oo`04001oogoo000;Ool00`00Oomoo`05Ool00`00Ool000060008Ool0
0`00Oomoo`03Ool20003Ool01000Oomoo`005Goo000/Ool00`00Oomoo`0AOol01000Oomoo`000Woo
00@007ooOol0011oo`04001oogoo0002Ool01000Oomoo`003Goo00<007ooOol00Woo00@007ooOol0
009oo`04001oogoo000:Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01000Oomoo`002goo00@007oo
Oomoo`H00005Ool007ooOol0000:Ool00`00Oomoo`05Ool01000Oomoo`000Woo00<007ooOol04goo
000/Ool2000COol20004Ool2000BOol20004Ool2000=Ool20005Ool20004Ool2000:Ool20004Ool2
0005Ool20009Ool70002Ool20004Ool2000:Ool20004Ool30004Ool2000FOol002aoo`03001oogoo
06aoo`H0041oo`00;7oo00<007ooOol0IWoo1P00AWoo000/Ool00`00Oomoo`1QOol5001<Ool002Qo
oj4001Qoo`00;7oo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo1P0017oo00<007oo
Ool00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007oo
Ool00Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007oo
Ool06Woo000/Ool00`00Oomoo`1DOol5001IOol002aoo`03001oogoo055oo`<005ioo`00;7oo0P00
Cgoo0`00HGoo000/Ool00`00Oomoo`1;Ool3001TOol002aoo`03001oogoo04Qoo`<006Moo`00;7oo
00<007ooOol0AGoo0`00JWoo000/Ool20013Ool3001]Ool002aoo`03001oogoo03moo`<0071oo`00
;7oo00<007ooOol0?7oo0`00Lgoo000/Ool00`00Oomoo`0iOol3001fOol002aoo`8003Qoo`8007Uo
o`00;7oo00<007ooOol0=Goo0P00Ngoo000DOol20004Ool20003Ool40002Ool40003Ool00`00Oomo
o`0cOol2001mOol001=oo`04001oogoo0003Ool20004Ool00`00Oomoo`03Ool00`00Oomoo`03Ool0
0`00Oomoo`0aOol2001oOol001=oo`04001oogoo000:Ool00`00Oomoo`03Ool00`00Oomoo`02Ool2
000`Ool20021Ool001=oo`04001oogoo000;Ool00`00Oomoo`03Ool01@00Oomoogoo0000<7oo00<0
07ooOol0PGoo000COol01000Oomoo`0027oo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomo
o`0/Ool20024Ool001Aoo`8000Yoo`8000Aoo`8000Aoo`03001oogoo02]oo`03001oogoo08Aoo`00
;7oo0P00:Woo0P00Qgoo000/Ool00`00Oomoo`0XOol00`00Oomoo`27Ool002aoo`03001oogoo02Io
o`8008Yoo`00;7oo00<007ooOol09Goo00<007ooOol0RWoo000/Ool2000UOol00`00Oomoo`2;Ool0
02aoo`03001oogoo02=oo`03001oogoo08aoo`00;7oo00<007ooOol08Woo00<007ooOol0SGoo000/
Ool00`00Oomoo`0QOol00`00Oomoo`2>Ool002aoo`80025oo`03001oogoo08moo`00;7oo00<007oo
Ool07goo00<007ooOol0T7oo000DOol20004Ool20003Ool40003Ool30003Ool00`00Oomoo`0NOol0
0`00Oomoo`2AOol001=oo`04001oogoo0003Ool20004Ool00`00Oomoo`04Ool00`00Oomoo`02Ool0
0`00Oomoo`0MOol00`00Oomoo`2BOol001=oo`04001oogoo000:Ool01000Oomoogoo10000goo0P00
7Woo00<007ooOol0TWoo000COol01000Oomoo`002goo00H007ooOol007oo0004Ool00`00Oomoo`0L
Ool00`00Oomoo`2COol001=oo`04001oogoo0008Ool01000Oomoo`000Woo00<007oo000017oo00<0
07ooOol06goo00<007ooOol0U7oo000DOol2000:Ool20004Ool20004Ool00`00Oomoo`0JOol00`00
Oomoo`2EOol002aoo`8001Yoo`03001oogoo09Ioo`00;7oo00<007ooOol067oo00<007ooOol0Ugoo
000/Ool00`00Oomoo`0GOol00`00Oomoo`2HOol002aoo`03001oogoo01Ioo`03001oogoo09Uoo`00
;7oo0P005goo00<007ooOol0VGoo000/Ool00`00Oomoo`0EOol00`00Oomoo`2JOol002aoo`03001o
ogoo01Aoo`03001oogoo09]oo`00;7oo00<007ooOol04goo00<007ooOol0W7oo000/Ool2000DOol0
0`00Oomoo`2LOol002aoo`03001oogoo019oo`03001oogoo09eoo`0057oo0P0017oo0P000goo1000
0goo0P0017oo00<007ooOol04Goo00<007ooOol0WWoo000COol01000Oomoo`000goo0P0017oo00<0
07ooOol00Woo00@007ooOol000=oo`03001oogoo011oo`03001oogoo09moo`004goo00@007ooOol0
00Yoo`05001oogooOol00002Ool01000Oomoogoo0P004Goo00<007ooOol0Wgoo000COol01000Oomo
o`002goo00<007ooOol00`0017oo00<007ooOol03goo00<007ooOol0X7oo000COol01000Oomoo`00
27oo00@007ooOol0009oo`03001oogoo00Aoo`03001oogoo00moo`03001oogoo0:1oo`0057oo0P00
2Woo0P0017oo0`000goo00<007ooOol03Woo00<007ooOol0XGoo000/Ool2000?Ool00`00Oomoo`2Q
Ool002aoo`03001oogoo00eoo`03001oogoo0:9oo`00;7oo00<007ooOol037oo00<007ooOol0Xgoo
000/Ool00`00Oomoo`0<Ool00`00Oomoo`2SOol002aoo`8000aoo`03001oogoo0:Aoo`00;7oo00<0
07ooOol02goo00<007ooOol0Y7oo000/Ool00`00Oomoo`0:Ool00`00Oomoo`2UOol002aoo`03001o
ogoo00Yoo`03001oogoo0:Eoo`00;7oo0P002Woo00<007ooOol0YWoo000/Ool00`00Oomoo`09Ool0
0`00Oomoo`2VOol001Aoo`8000Aoo`8000=oo`@000=oo`8000Aoo`03001oogoo00Qoo`03001oogoo
0:Moo`004goo00@007ooOol000=oo`8000Aoo`03001oogoo009oo`04001oogoo0003Ool00`00Oomo
o`08Ool00`00Oomoo`2WOol001=oo`04001oogoo000:Ool01@00Oomoogoo00000Woo00@007ooOomo
o`8000Qoo`03001oogoo0:Qoo`004goo00@007ooOol000]oo`04001oogooOol20004Ool00`00Oomo
o`06Ool00`00Oomoo`2YOol001=oo`04001oogoo0008Ool01000Oomoo`000Woo00@007ooOol000=o
o`03001oogoo00Ioo`03001oogoo0:Uoo`0057oo0P002Woo0P0017oo0P0017oo00<007ooOol01Goo
00<007ooOol0ZWoo000/Ool20006Ool00`00Oomoo`2ZOol002aoo`03001oogoo00Aoo`03001oogoo
0:]oo`00;7oo00<007ooOol017oo00<007ooOol0Zgoo000/Ool00`00Oomoo`03Ool00`00Oomoo`2/
Ool002aoo`8000Aoo`03001oogoo0:aoo`00;7oo00<007ooOol00Woo00<007ooOol0[Goo000/Ool0
0`00Oomoo`02Ool00`00Oomoo`2]Ool002aoo`05001oogooOol0002`Ool002aoo`80009oo`03001o
ogoo0:ioo`00;7oo00D007ooOomoo`000;1oo`00;7oo00@007ooOol00;5oo`006Woo0P0017oo0P00
0goo0`0017oo00@007ooOol00;5oo`006Goo00@007ooOol000=oo`8000Ioo`07001oogooOol007oo
0000/Woo000IOol01000Oomoo`002goo00@007ooOomoo`<00;9oo`006Goo00@007ooOol000Uoo`80
00Aoo`800;=oo`006Goo00@007ooOol000]oo`04001oogooOol2002cOol001Yoo`8000Uoo`<000Ao
o`800;=oo`00;7oo0P00/goo000/Ool00`00Oomoo`2bOol002aoo`03001oogoo0;9oo`00;7oo00<0
07ooOol0/Woo003QOol00>5oo`00hGoo003QOol00>5oo`00hGoo003QOol00>5oo`00hGoo003QOol0
0>5oo`00hGoo003QOol00>5oo`00hGoo0018Ool4002EOol003Uoo`03001oogoo00eoo`03001oogoo
09Eoo`00>Goo00<007ooOol01goo0`000goo10000Woo10000goo100000=oo`0000000`000Woo1@00
1Woo00D007ooOomoo`00009oo`<000=oo`D00003Ool0000000800003Ool0000000<000Qoo`<0009o
o`@000Qoo`@0009oo`D00003Ool0000000<00003Ool0000000<002Moo`00=goo1@001goo00@007oo
Ool0009oo`05001oogooOol00002Ool00`00Oomoo`02Ool00`00Oomoo`03Ool01000Oomoo`000Woo
00D007ooOomoo`0000Moo`07001oogooOol007oo00000goo00<007oo00000goo00<007oo00001goo
00<007ooOol01goo00D007ooOomoo`00009oo`03001oogoo00Moo`03001oogoo00Aoo`04001oogoo
0004Ool00`00Oomoo`03Ool00`00Oomoo`0WOol003Uoo`03001oogoo00Uoo`80009oo`05001oogoo
Ool00002Ool00`00Oomoo`02Ool500000goo000000020002Ool01@00Oomoogoo00001goo00L007oo
001oo`00Ool00003Ool00`00Ool00003Ool00`00Ool000040003Ool00`00Oomoo`07Ool01@00Oomo
ogoo00000Woo00<007ooOol01goo00<007ooOol017oo100017oo00<007ooOol00goo00<007ooOol0
9goo000iOol00`00Oomoo`07Ool20004Ool01@00Oomoogoo00000Woo0P000goo00D007ooOomoo`00
00Aoo`04001oogoo0003Ool00`00Oomoo`05Ool01`00Ool007oo001oo`0000=oo`03001oo`0000=o
o`03001oo`0000=oo`05001oogooOol00009Ool01@00Oomoogoo00000Woo00<007ooOol01goo00<0
07ooOol01goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol09goo0013Ool01@00Oomoo`00
Ool01@000Woo0P0000=oo`0000000Woo0`000goo0`000goo10001goo0P0000=oo`0000000Woo0`00
0goo10000Woo0`0017oo00<007ooOol027oo0`000Woo10001goo00D007ooOomoo`0000=oo`<000Ao
o`03001oogoo00=oo`03001oogoo02Moo`00A7oo0`007Woo00<007ooOol017oo0P000goo0P002Woo
00<007ooOol01goo00<007ooOol03Woo00<007ooOol027oo10002Woo00<007ooOol00goo00<007oo
Ool09goo001TOol2000FOol20007Ool3000@Ool3000DOol30003Ool3000YOol00>5oo`00hGoo003Q
Ool00:9oo`<003aoo`00Xgoo00<007ooOol0>goo000gOol30003Ool01000Ool000000goo100000Eo
o`00Oomoo`0000Qoo`@000=oo`<0009oo`@000Moo`D0009oo`D00003Ool0000000<00003Ool00000
008000=oo`<0009oo`D0009oo`<0009oo`D0009oo`<000=oo`03001oogoo03Yoo`00=goo00@007oo
Ool0009oo`03001oo`0000=oo`03001oogoo00=oo`@000Uoo`03001oogoo009oo`05001oogooOol0
0002Ool00`00Oomoo`08Ool00`00Oomoo`05Ool01@00Oomoogoo00000Woo00<007ooOol00goo00D0
07ooOomoo`00009oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@0
07ooOol0009oo`03001oogoo03Yoo`00>Goo0P000Woo0`000goo1@0000=oo`0000000P002Goo00<0
07ooOol00Woo00D007ooOomoo`00009oo`03001oogoo00Qoo`03001oogoo00Eoo`04001oogooOol4
0002Ool40003Ool00`00Oomoo`04Ool01@00Oomoogoo00001goo00<007ooOol00Woo00<007ooOol0
0Woo00<007oo0000>goo000gOol20004Ool00`00Ool00003Ool01`00Oomoogoo001oo`00009oo`03
001oogoo00Moo`03001oogoo009oo`05001oogooOol00002Ool20009Ool30005Ool00`00Oomoo`04
Ool01000Oomoo`001Woo00<007ooOol017oo00D007ooOomoo`0000=oo`05001oogooOol00004Ool0
0`00Oomoo`02Ool00`00Ool0000kOol003Moo`04001oogoo0002Ool01000Ool000000goo0`0000=o
o`0000000Woo0P001goo10000goo0`000Woo0P0000=oo`0000001goo00<007ooOol01Goo00<007oo
Ool00Woo0`000Woo10000Woo10000goo0P0017oo10000Woo0P000goo100000Aoo`00001oo`8003Yo
o`00>7oo0`000Woo00<007ooOol05Woo00<007ooOol04goo1@0017oo00<007ooOol03Woo00<007oo
Ool05Goo00<007ooOol0@7oo000lOol2000HOol3000JOol3000@Ool00`00Oomoo`04Ool00`00Oomo
o`09Ool00`00Oomoo`02Ool00`00Oomoo`10Ool00>5oo`00hGoo0000\
\>"],
  ImageRangeCache->{{{0, 224}, {137.688, 0}} -> {52.4604, 0.185792, 0.394526, 
  0.00124553}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["28.10 Puts and Smiles", "Section",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
The discussion of Puts is complicated by the fact that there is a \
non-zero probability of the stock price reaching zero and staying there, for \
finite values of \[Nu]. For example, in our test problem, it is very small:\
\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(ZeroProb[S_, \ r_, \ q_, \ vol_, \ t_, \ \[Nu]_] := \ \n
      Module[{\[Sigma]\  = \ vol*S^\((1/\((2  \[Nu])\))\), \ x, \ c}, \n\t
        c\  = \ \((2*\[Nu]*\((r\  - \ 
                    q)\))\)/\((\[Sigma]^2*\((Exp[\((\((r\  - \ q)\)*
                            t)\)/\[Nu]]\  - \ 1)\))\); \n\t
        x\  = \ c*S^\((1/\[Nu])\)*Exp[\((\((r\  - \ q)\)*t)\)/\[Nu]]; \n
        Gamma[\[Nu], \ x]/Gamma[\[Nu]]]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ZeroProb[100, \ 0.05, \ 0, \ 0.2, 1, \ 1]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(5.46869987954394964`*^-23\)], "Output",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell["\<\
As \[Nu] increases (i.e. we move closer to standard log-normal) it \
becomes smaller:\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ZeroProb[100, \ 0.05, \ 0, \ 0.2, 1, \ 10]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(6.8533110772537020593998`12.2601*^-2150\)], "Output"]
}, Open  ]],

Cell["\<\
It can be significant in the Gaussian case, for longer times and \
smaller drifts:\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ZeroProb[100, \ 0.02, \ 0, \ 0.2, 5, \ 0.5]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.018836236970581206`\)], "Output"]
}, Open  ]],

Cell["\<\
What we shall do is model a Put which includes the contribution for \
the final asset price being zero, so that it only remains to consider the \
continuous part of the distribution, which proceeds as before. (You should \
check that this ensures Put-Call parity applies.) The model then becomes the \
pair of functions\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
CEVPutfunc[x_, a_, \[Nu]_, n_] := 
  NIntegrate[Exp[-(z^2/(4*x))]*z^n*BesselI[\[Nu], z], {z, 0, a}]\
\>", "Input",\

  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(CEVPut[S_, \ K_, \ r_, \ q_, \ vol_, \ t_, \ \[Nu]_] := \ \n
      Module[{\[Sigma]\  = \ vol*S^\((1/\((2  \[Nu])\))\), \ x, \ c, \ 
          a}, \n\tc\  = \ \((2*\[Nu]*\((r\  - \ 
                    q)\))\)/\((\[Sigma]^2*\((Exp[\((\((r\  - \ q)\)*
                            t)\)/\[Nu]]\  - \ 1)\))\); \n\t
        x\  = \ c*S^\((1/\[Nu])\)*Exp[\((\((r\  - \ q)\)*t)\)/\[Nu]]; \n\t
        a\  = \ 2\ Sqrt[
              c\ x\ K^\((1/\[Nu])\)]; \n\t\t\ Exp[\(-r\)*t\  - \ 
                x] \((K*\((\((2*x)\)^\((\[Nu]\  - \ 1)\))\)*
                  CEVPutfunc[x, \ a, \ \[Nu], \ 1\  - \ \[Nu]] - 
                CEVPutfunc[x, 
                    a, \ \[Nu], \ \[Nu]\  + \ 1]/\((x*
                      c^\[Nu]*2^\((\[Nu]\  + \ 1)\))\)\ )\)\  + \ 
          K*Exp[\(-r\)*t]*ZeroProb[S, \ r, \ q, \ vol, \ t, \ \[Nu]]]\)], 
  "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(CEVPut[100, \ 100, \ 0.05, \ 0, \ 0.2, 1, \ 1]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(5.57682777878175972`\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "As before, as ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\)]],
  " gets large we approach the BS values:"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesPut[100, \ 100, \ 0.2, \ 0.05, \ 0, \ 1]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(5.57352602225697069`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CEVPut[100, \ 100, \ 0.05, \ 0, \ 0.2, \ 1, \ 100]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(5.5735263514006285047`8.6202\)], "Output"]
}, Open  ]],

Cell["\<\
We can do corresponding implied volatility computations, limiting \
attention, as before, to SRCEV:\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(PutForwardBackward[S_, \ K_, \ r_, \ q_, \ \[Sigma]_, \ t_, \ 
        spread_]\  := \n\t
      BlackScholesPutImpVol[S, \ K, \ r, q, \ 
        t, \ \((1 + spread/100)\)*
          CEVPut[S, \ K, \ r, q, \ \[Sigma], \ t, \ 1/2]]\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell["\<\
With no spread, the skew obtained is as follows - note that the \
out-of-the-money case is at the left of this plot:\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      PutForwardBackward[100, \ K, \ 0.05, \ 0, \ 0.2, 1, \ 0], \ {K, \ 70, \ 
        130}]; \)\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.0873 0.015873 -1.63206 9.39174 [
[.18254 .23379 -6 -9 ]
[.18254 .23379 6 0 ]
[.34127 .23379 -6 -9 ]
[.34127 .23379 6 0 ]
[.5 .23379 -9 -9 ]
[.5 .23379 9 0 ]
[.65873 .23379 -9 -9 ]
[.65873 .23379 9 0 ]
[.81746 .23379 -9 -9 ]
[.81746 .23379 9 0 ]
[.97619 .23379 -9 -9 ]
[.97619 .23379 9 0 ]
[.01131 .05845 -24 -4.5 ]
[.01131 .05845 0 4.5 ]
[.01131 .15237 -24 -4.5 ]
[.01131 .15237 0 4.5 ]
[.01131 .3402 -24 -4.5 ]
[.01131 .3402 0 4.5 ]
[.01131 .43412 -24 -4.5 ]
[.01131 .43412 0 4.5 ]
[.01131 .52804 -24 -4.5 ]
[.01131 .52804 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .24629 m
.18254 .25254 L
s
[(80)] .18254 .23379 0 1 Mshowa
.34127 .24629 m
.34127 .25254 L
s
[(90)] .34127 .23379 0 1 Mshowa
.5 .24629 m
.5 .25254 L
s
[(100)] .5 .23379 0 1 Mshowa
.65873 .24629 m
.65873 .25254 L
s
[(110)] .65873 .23379 0 1 Mshowa
.81746 .24629 m
.81746 .25254 L
s
[(120)] .81746 .23379 0 1 Mshowa
.97619 .24629 m
.97619 .25254 L
s
[(130)] .97619 .23379 0 1 Mshowa
.125 Mabswid
.05556 .24629 m
.05556 .25004 L
s
.0873 .24629 m
.0873 .25004 L
s
.11905 .24629 m
.11905 .25004 L
s
.15079 .24629 m
.15079 .25004 L
s
.21429 .24629 m
.21429 .25004 L
s
.24603 .24629 m
.24603 .25004 L
s
.27778 .24629 m
.27778 .25004 L
s
.30952 .24629 m
.30952 .25004 L
s
.37302 .24629 m
.37302 .25004 L
s
.40476 .24629 m
.40476 .25004 L
s
.43651 .24629 m
.43651 .25004 L
s
.46825 .24629 m
.46825 .25004 L
s
.53175 .24629 m
.53175 .25004 L
s
.56349 .24629 m
.56349 .25004 L
s
.59524 .24629 m
.59524 .25004 L
s
.62698 .24629 m
.62698 .25004 L
s
.69048 .24629 m
.69048 .25004 L
s
.72222 .24629 m
.72222 .25004 L
s
.75397 .24629 m
.75397 .25004 L
s
.78571 .24629 m
.78571 .25004 L
s
.84921 .24629 m
.84921 .25004 L
s
.88095 .24629 m
.88095 .25004 L
s
.9127 .24629 m
.9127 .25004 L
s
.94444 .24629 m
.94444 .25004 L
s
.25 Mabswid
0 .24629 m
1 .24629 L
s
.02381 .05845 m
.03006 .05845 L
s
[(0.18)] .01131 .05845 1 0 Mshowa
.02381 .15237 m
.03006 .15237 L
s
[(0.19)] .01131 .15237 1 0 Mshowa
.02381 .3402 m
.03006 .3402 L
s
[(0.21)] .01131 .3402 1 0 Mshowa
.02381 .43412 m
.03006 .43412 L
s
[(0.22)] .01131 .43412 1 0 Mshowa
.02381 .52804 m
.03006 .52804 L
s
[(0.23)] .01131 .52804 1 0 Mshowa
.125 Mabswid
.02381 .07723 m
.02756 .07723 L
s
.02381 .09602 m
.02756 .09602 L
s
.02381 .1148 m
.02756 .1148 L
s
.02381 .13358 m
.02756 .13358 L
s
.02381 .17115 m
.02756 .17115 L
s
.02381 .18994 m
.02756 .18994 L
s
.02381 .20872 m
.02756 .20872 L
s
.02381 .2275 m
.02756 .2275 L
s
.02381 .26507 m
.02756 .26507 L
s
.02381 .28385 m
.02756 .28385 L
s
.02381 .30264 m
.02756 .30264 L
s
.02381 .32142 m
.02756 .32142 L
s
.02381 .35899 m
.02756 .35899 L
s
.02381 .37777 m
.02756 .37777 L
s
.02381 .39655 m
.02756 .39655 L
s
.02381 .41534 m
.02756 .41534 L
s
.02381 .4529 m
.02756 .4529 L
s
.02381 .47169 m
.02756 .47169 L
s
.02381 .49047 m
.02756 .49047 L
s
.02381 .50925 m
.02756 .50925 L
s
.02381 .54682 m
.02756 .54682 L
s
.02381 .5656 m
.02756 .5656 L
s
.02381 .58439 m
.02756 .58439 L
s
.02381 .60317 m
.02756 .60317 L
s
.02381 .03967 m
.02756 .03967 L
s
.02381 .02088 m
.02756 .02088 L
s
.02381 .0021 m
.02756 .0021 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .56771 L
.10458 .53061 L
.14415 .49729 L
.18221 .46653 L
.22272 .4351 L
.26171 .40602 L
.30316 .3763 L
.34309 .34875 L
.3815 .32319 L
.42237 .29696 L
.46172 .27257 L
.49955 .24989 L
.53984 .22651 L
.57861 .20474 L
.61984 .18231 L
.65954 .16139 L
.69774 .14186 L
.73838 .12169 L
.77751 .10283 L
.81909 .08336 L
.85916 .06514 L
.89771 .04809 L
.93871 .03043 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{214, 132.062},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004;0000YA000`40O003h00Oogoo37oo003oOol<
Ool00?moo`aoo`00ogoo37oo003oOol<Ool00?moo`aoo`00ogoo37oo003oOol<Ool00?moo`aoo`00
ogoo37oo000POol00`00Oomoo`3XOol0021oo`800>Uoo`0087oo00<007ooOol0j7oo000POol00`00
Oomoo`3XOol0021oo`03001oogoo0=]oo`8000]oo`0087oo0P00fGoo0`003Goo000POol00`00Oomo
o`3EOol3000@Ool0021oo`03001oogoo0=9oo`<001=oo`0087oo00<007ooOol0d7oo0P005Woo000P
Ool00`00Oomoo`3>Ool2000HOol0021oo`800<eoo`8001Yoo`000`001Woo0P001Goo1@000Woo0`00
1Woo00<007ooOol0bWoo0P0077oo0003Ool00`00Oomoo`03Ool20007Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol0b7oo0P007Woo0003Ool00`00Oomoo`0<Ool01@00Oomoogoo0000
0goo00<007ooOol00goo00<007ooOol0aWoo0P0087oo0003Ool00`00Oomoo`0<Ool01@00Oomoogoo
00000goo00<007ooOol00goo0`00`goo0`008Woo0003Ool00`00Oomoo`0<Ool00`00Oomoo`02Ool3
0006Ool00`00Oomoo`31Ool2000UOol000=oo`03001oogoo00aoo`05001oogooOol00003Ool00`00
Oomoo`03Ool00`00Oomoo`2oOol2000WOol000=oo`03001oogoo00aoo`05001oogooOol00003Ool0
0`00Oomoo`03Ool00`00Oomoo`2mOol2000YOol000<000eoo`<000Aoo`<000Ioo`800;]oo`<002]o
o`0087oo00<007ooOol0^7oo0P00;Woo000POol00`00Oomoo`2fOol2000`Ool0021oo`03001oogoo
0;=oo`<0039oo`0087oo00<007ooOol0/Goo0P00=Goo000POol2002_Ool3000gOol0021oo`03001o
ogoo0:aoo`8003Yoo`0087oo00<007ooOol0ZWoo0P00?7oo000POol00`00Oomoo`2XOol2000nOol0
021oo`800:Qoo`03001oogoo03ioo`0087oo00<007ooOol0YGoo0P00@Goo000POol00`00Oomoo`2S
Ool20013Ool0021oo`03001oogoo0:5oo`8004Eoo`0087oo00<007ooOol0WWoo0`00Agoo000POol2
002MOol2001:Ool000<000Ioo`8000Eoo`D00004Ool000000007Ool00`00Oomoo`2JOol2001<Ool0
00=oo`03001oogoo00=oo`8000Moo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo09Qoo`80
04ioo`000goo00<007ooOol037oo00<007ooOol01Goo00<007ooOol00goo00<007ooOol0UWoo0P00
D7oo0003Ool00`00Oomoo`0<Ool00`00Oomoo`02Ool40005Ool3002EOol00`00Oomoo`1@Ool000=o
o`03001oogoo00aoo`05001oogooOol00003Ool00`00Oomoo`03Ool00`00Oomoo`2COol2001COol0
00=oo`03001oogoo00aoo`05001oogooOol00003Ool00`00Oomoo`03Ool00`00Oomoo`2AOol2001E
Ool000=oo`03001oogoo00aoo`05001oogooOol00003Ool00`00Oomoo`03Ool00`00Oomoo`2?Ool2
001GOol000<000eoo`<000Aoo`<000Ioo`8008ioo`8005Uoo`0087oo00<007ooOol0S7oo00<007oo
Ool0FGoo000POol00`00Oomoo`2:Ool2001LOol0021oo`03001oogoo08Qoo`8005ioo`0087oo00<0
07ooOol0QWoo0P00H7oo000POol20025Ool2001ROol0021oo`03001oogoo01eoo`<000Eoo`<001Uo
o`<000Ioo`<001Ioo`D000=oo`<000Eoo`<000aoo`8000Aoo`D000=oo`D000=oo`<0019oo`D0009o
o`D000Aoo`<0019oo`D000=oo`<000Eoo`<000=oo`0087oo00<007ooOol077oo00D007ooOomoo`00
00=oo`05001oogooOol0000KOol00`00Oomoo`02Ool01@00Oomoogoo00005goo00<007ooOol00Woo
00D007ooOomoo`0000=oo`05001oogooOol00009Ool20008Ool00`00Oomoo`05Ool00`00Oomoo`02
Ool01@00Oomoogoo00004goo00<007ooOol00goo00@007ooOol000=oo`05001oogooOol0000COol0
0`00Oomoo`02Ool01@00Oomoogoo00000goo00D007ooOomoo`00009oo`0087oo00<007ooOol077oo
00D007ooOomoo`0000=oo`05001oogooOol0000LOol01@00Oomoogoo00000goo00<007ooOol05Goo
00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol00007Ool2000:Ool00`00Oomoo`05
Ool00`00Oomoo`02Ool01@00Oomoogoo00004goo00<007ooOol017oo00<007ooOol00goo00D007oo
Oomoo`0001=oo`03001oogoo00Ioo`05001oogooOol00003Ool10001Ool1Ool0021oo`8001eoo`05
001oogooOol00003Ool01@00Oomoogoo00006Goo10000goo00D007ooOomoo`0001Moo`03001oogoo
009oo`05001oogooOol00003Ool01@00Oomoogoo00001Goo0P0037oo00<007ooOol01Goo00<007oo
Ool00Woo00D007ooOomoo`0001=oo`03001oogoo00Eoo`03001oogoo009oo`05001oogooOol0000C
Ool00`00Oomoo`06Ool01@00Oomoogoo00000goo0@000Goo0Goo000POol00`00Oomoo`0MOol30004
Ool01@00Oomoogoo000067oo00D007ooOomoo`0000=oo`05001oogooOol0000GOol00`00Oomoo`02
Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`8000ioo`03001oogoo00Eoo`03001oogoo
009oo`05001oogooOol0000COol00`00Oomoo`06Ool01@00Oomoogoo00000goo00<007ooOol04Goo
00<007ooOol017oo0P0017oo00D007ooOomoo`00009oo`0087oo00<007ooOol077oo00D007ooOomo
o`0000=oo`05001oogooOol0000HOol01@00Oomoogoo00000goo00D007ooOomoo`0001Moo`03001o
ogoo009oo`05001oogooOol00003Ool01@00Oomoogoo00000Woo00<007ooOol03Woo00<007ooOol0
1Goo00<007ooOol00Woo00D007ooOomoo`0001=oo`03001oogoo00Ioo`05001oogooOol00003Ool0
0`00Oomoo`0AOol00`00Oomoo`06Ool01@00Oomoogoo00000goo0@000Goo0Goo000POol00`00Oomo
o`0LOol01@00Oomoogoo00000goo00D007ooOomoo`0001Qoo`05001oogooOol00003Ool01@00Oomo
ogoo00005goo00<007ooOol00Woo00D007ooOomoo`0000=oo`04001oogooOol3000AOol00`00Oomo
o`05Ool00`00Oomoo`02Ool01@00Oomoogoo00004goo00<007ooOol00Woo00D007ooOomoo`0000=o
o`05001oogooOol0000COol00`00Oomoo`02Ool01@00Oomoogoo00000goo00D007ooOomoo`00009o
o`0087oo0P007Woo0`001Goo0`006Woo0`001Goo0`005Woo0`001Goo0`001Goo10004Goo0`001Goo
0`001Goo0`004Woo0`001Goo0`001Goo0`004Woo0`001Goo0`001Goo0`000goo000POol00`00Oomo
o`1cOol2001cOol0021oo`03001oogoo079oo`03001oogoo07=oo`0087oo00<007ooOol0L7oo0P00
MWoo000POol00`00Oomoo`1^Ool2001hOol001]oonX000Ioo`0087oo00<007ooOol01Goo00<007oo
Ool017oo00<007ooOol01Goo00<007ooOol017oo00<007ooOol017oo00<007ooOol01Goo00<007oo
Ool017oo00<007ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<007ooOol017oo00<007oo
Ool017oo00<007ooOol01Goo00<007ooOol017oo00<007ooOol00goo0`001Woo00<007ooOol017oo
00<007ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<007ooOol017oo00<007ooOol017oo
00<007ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<007ooOol017oo00<007ooOol01Goo
00<007ooOol017oo00<007ooOol017oo00<007ooOol01Goo00<007ooOol02Goo000POol00`00Oomo
o`1ZOol00`00Oomoo`1kOol0021oo`03001oogoo06Qoo`8007ioo`0087oo0P00Igoo0P00P7oo000P
Ool00`00Oomoo`1UOol00`00Oomoo`20Ool0021oo`03001oogoo06=oo`8008=oo`0087oo00<007oo
Ool0HGoo0P00QGoo000POol00`00Oomoo`1POol00`00Oomoo`25Ool0021oo`8005moo`8008Qoo`00
87oo00<007ooOol0G7oo0P00RWoo000POol00`00Oomoo`1KOol00`00Oomoo`2:Ool0021oo`03001o
ogoo05Uoo`8008eoo`0087oo0P00FGoo00<007ooOol0SGoo000POol00`00Oomoo`1FOol2002@Ool0
021oo`03001oogoo05Eoo`03001oogoo091oo`0087oo00<007ooOol0Dgoo0P00Tgoo000POol2001C
Ool00`00Oomoo`2COol0021oo`03001oogoo051oo`8009Ioo`000`001Woo0P0017oo1@000goo1@00
17oo00<007ooOol0Cgoo00<007ooOol0UWoo0003Ool00`00Oomoo`03Ool20005Ool01000Oomoo`00
1Goo00<007ooOol017oo00<007ooOol0CGoo0P00VGoo0003Ool00`00Oomoo`0;Ool00`00Oomoo`05
Ool00`00Oomoo`04Ool00`00Oomoo`1<Ool00`00Oomoo`2IOol000=oo`03001oogoo00aoo`03001o
ogoo00Aoo`03001oogoo00Aoo`<004Yoo`8009aoo`000goo00<007ooOol03Goo00<007ooOol00goo
00<007ooOol017oo00<007ooOol0BGoo00<007ooOol0W7oo0003Ool00`00Oomoo`0=Ool00`00Oomo
o`03Ool00`00Oomoo`04Ool00`00Oomoo`17Ool2002OOol000=oo`03001oogoo00Uoo`05001oogoo
Ool00005Ool00`00Oomoo`04Ool00`00Oomoo`16Ool00`00Oomoo`2OOol000<000eoo`<000Aoo`<0
00Ioo`8004Eoo`800:9oo`0087oo00<007ooOol0@goo00<007ooOol0XWoo000POol00`00Oomoo`11
Ool2002UOol0021oo`03001oogoo041oo`03001oogoo0:Eoo`0087oo00<007ooOol0?Woo0P00Z7oo
000POol2000nOol00`00Oomoo`2XOol0021oo`03001oogoo03]oo`800:]oo`0087oo00<007ooOol0
>Woo00<007ooOol0Zgoo000POol00`00Oomoo`0iOol00`00Oomoo`2/Ool0021oo`8003Qoo`800:mo
o`0087oo00<007ooOol0=Woo00<007ooOol0[goo000POol00`00Oomoo`0dOol2002bOol0021oo`03
001oogoo03=oo`03001oogoo0;9oo`0087oo0P00<goo00<007ooOol0/goo000POol00`00Oomoo`0`
Ool2002fOol000<000Ioo`8000Aoo`D0009oo`D000Eoo`03001oogoo02moo`03001oogoo0;Ioo`00
0goo00<007ooOol00goo0P001Goo00@007ooOol000=oo`04001oogoo0005Ool00`00Oomoo`0^Ool0
0`00Oomoo`2gOol000=oo`03001oogoo00]oo`03001oogoo00Aoo`03001oogoo00Eoo`03001oogoo
02eoo`03001oogoo0;Qoo`000goo00<007ooOol037oo00<007ooOol017oo00<007ooOol017oo0`00
:goo0P00^goo0003Ool00`00Oomoo`0=Ool00`00Oomoo`04Ool00`00Oomoo`03Ool00`00Oomoo`0Z
Ool00`00Oomoo`2kOol000=oo`03001oogoo00eoo`03001oogoo00Aoo`03001oogoo00=oo`03001o
ogoo02Qoo`800;ioo`000goo00<007ooOol02Goo00D007ooOomoo`00009oo`05001oogooOol00005
Ool00`00Oomoo`0WOol00`00Oomoo`2nOol000<000eoo`<000Aoo`<000Ioo`8002Moo`03001oogoo
0;moo`0087oo00<007ooOol097oo0P00`Woo000POol00`00Oomoo`0SOol00`00Oomoo`32Ool0021o
o`03001oogoo025oo`800<Eoo`0087oo0P008Goo00<007ooOol0aGoo000POol00`00Oomoo`0OOol0
0`00Oomoo`36Ool0021oo`03001oogoo01ioo`03001oogoo0<Moo`0087oo00<007ooOol07Goo00<0
07ooOol0b7oo000POol00`00Oomoo`0LOol00`00Oomoo`39Ool0021oo`8001aoo`03001oogoo0<Yo
o`0087oo00<007ooOol06Goo0P00cGoo000POol00`00Oomoo`0HOol00`00Oomoo`3=Ool0021oo`03
001oogoo01Moo`03001oogoo0<ioo`0087oo0P005goo00<007ooOol0cgoo000POol00`00Oomoo`0D
Ool2003BOol000<000Ioo`8000Aoo`D000=oo`<000Ioo`03001oogoo01=oo`03001oogoo0=9oo`00
0goo00<007ooOol00goo0P001Goo00@007ooOol0009oo`05001oogooOol00005Ool00`00Oomoo`0B
Ool00`00Oomoo`3COol000=oo`03001oogoo00]oo`03001oogoo00Ioo`03001oogoo00=oo`03001o
ogoo015oo`03001oogoo0=Aoo`000goo00<007ooOol037oo00<007ooOol01Goo00<007ooOol00goo
0`003goo0P00egoo0003Ool00`00Oomoo`0=Ool00`00Oomoo`02Ool20006Ool00`00Oomoo`0>Ool0
0`00Oomoo`3GOol000=oo`03001oogoo00eoo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo
00eoo`03001oogoo0=Qoo`000goo00<007ooOol02Goo00D007ooOomoo`00009oo`05001oogooOol0
0005Ool00`00Oomoo`0<Ool00`00Oomoo`3IOol000<000eoo`<000Aoo`<000Ioo`8000aoo`03001o
ogoo0=Yoo`0087oo00<007ooOol02Woo00<007ooOol0fgoo000POol00`00Oomoo`09Ool00`00Oomo
o`3LOol0021oo`03001oogoo00Qoo`03001oogoo0=eoo`0087oo0P0027oo00<007ooOol0gWoo000P
Ool00`00Oomoo`05Ool2003QOol0021oo`03001oogoo00Aoo`03001oogoo0>5oo`0087oo00<007oo
Ool00goo00<007ooOol0hWoo000POol00`00Oomoo`02Ool00`00Oomoo`3SOol0021oo`80009oo`03
001oogoo0>Aoo`0087oo00@007ooOol00>Moo`0087oo00<007oo0000j7oo000POol2003YOol0021o
o`800>Uoo`0087oo00<007ooOol0j7oo000POol00`00Oomoo`3XOol0021oo`03001oogoo0>Qoo`00
87oo00<007ooOol0j7oo003oOol<Ool00?moo`aoo`00ogoo37oo003oOol<Ool00?moo`aoo`00ogoo
37oo003oOol<Ool00?moo`aoo`00ogoo37oo0000\
\>"],
  ImageRangeCache->{{{0, 266.5}, {164.062, 0}} -> {61.1161, 0.169264, 
  0.337337, 0.000570134}}]
}, Open  ]],

Cell["\<\
This time folding in the spread effect turns the skew into the \
beginning of a smile.\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      PutForwardBackward[100, \ K, \ 0.05, \ 0, \ 0.2, 1, \ 2.5], \ {K, \ 
        70, \ 130}, \ 
      PlotLabel\  -> \ 
        "\<Smile for Elasticity + Spread Model of Put\n\>"]; \)\)], "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.0873 0.015873 -2.60687 13.4228 [
[.18254 .06519 -6 -9 ]
[.18254 .06519 6 0 ]
[.34127 .06519 -6 -9 ]
[.34127 .06519 6 0 ]
[.5 .06519 -9 -9 ]
[.5 .06519 9 0 ]
[.65873 .06519 -9 -9 ]
[.65873 .06519 9 0 ]
[.81746 .06519 -9 -9 ]
[.81746 .06519 9 0 ]
[.97619 .06519 -9 -9 ]
[.97619 .06519 9 0 ]
[.01131 .21192 -24 -4.5 ]
[.01131 .21192 0 4.5 ]
[.01131 .34615 -24 -4.5 ]
[.01131 .34615 0 4.5 ]
[.01131 .48037 -24 -4.5 ]
[.01131 .48037 0 4.5 ]
[.01131 .6146 -24 -4.5 ]
[.01131 .6146 0 4.5 ]
[.5 .63053 -128 0 ]
[.5 .63053 128 24.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .07769 m
.18254 .08394 L
s
[(80)] .18254 .06519 0 1 Mshowa
.34127 .07769 m
.34127 .08394 L
s
[(90)] .34127 .06519 0 1 Mshowa
.5 .07769 m
.5 .08394 L
s
[(100)] .5 .06519 0 1 Mshowa
.65873 .07769 m
.65873 .08394 L
s
[(110)] .65873 .06519 0 1 Mshowa
.81746 .07769 m
.81746 .08394 L
s
[(120)] .81746 .06519 0 1 Mshowa
.97619 .07769 m
.97619 .08394 L
s
[(130)] .97619 .06519 0 1 Mshowa
.125 Mabswid
.05556 .07769 m
.05556 .08144 L
s
.0873 .07769 m
.0873 .08144 L
s
.11905 .07769 m
.11905 .08144 L
s
.15079 .07769 m
.15079 .08144 L
s
.21429 .07769 m
.21429 .08144 L
s
.24603 .07769 m
.24603 .08144 L
s
.27778 .07769 m
.27778 .08144 L
s
.30952 .07769 m
.30952 .08144 L
s
.37302 .07769 m
.37302 .08144 L
s
.40476 .07769 m
.40476 .08144 L
s
.43651 .07769 m
.43651 .08144 L
s
.46825 .07769 m
.46825 .08144 L
s
.53175 .07769 m
.53175 .08144 L
s
.56349 .07769 m
.56349 .08144 L
s
.59524 .07769 m
.59524 .08144 L
s
.62698 .07769 m
.62698 .08144 L
s
.69048 .07769 m
.69048 .08144 L
s
.72222 .07769 m
.72222 .08144 L
s
.75397 .07769 m
.75397 .08144 L
s
.78571 .07769 m
.78571 .08144 L
s
.84921 .07769 m
.84921 .08144 L
s
.88095 .07769 m
.88095 .08144 L
s
.9127 .07769 m
.9127 .08144 L
s
.94444 .07769 m
.94444 .08144 L
s
.25 Mabswid
0 .07769 m
1 .07769 L
s
.02381 .21192 m
.03006 .21192 L
s
[(0.21)] .01131 .21192 1 0 Mshowa
.02381 .34615 m
.03006 .34615 L
s
[(0.22)] .01131 .34615 1 0 Mshowa
.02381 .48037 m
.03006 .48037 L
s
[(0.23)] .01131 .48037 1 0 Mshowa
.02381 .6146 m
.03006 .6146 L
s
[(0.24)] .01131 .6146 1 0 Mshowa
.125 Mabswid
.02381 .10454 m
.02756 .10454 L
s
.02381 .13138 m
.02756 .13138 L
s
.02381 .15823 m
.02756 .15823 L
s
.02381 .18507 m
.02756 .18507 L
s
.02381 .23876 m
.02756 .23876 L
s
.02381 .26561 m
.02756 .26561 L
s
.02381 .29245 m
.02756 .29245 L
s
.02381 .3193 m
.02756 .3193 L
s
.02381 .37299 m
.02756 .37299 L
s
.02381 .39984 m
.02756 .39984 L
s
.02381 .42668 m
.02756 .42668 L
s
.02381 .45353 m
.02756 .45353 L
s
.02381 .50722 m
.02756 .50722 L
s
.02381 .53406 m
.02756 .53406 L
s
.02381 .56091 m
.02756 .56091 L
s
.02381 .58776 m
.02756 .58776 L
s
.02381 .05084 m
.02756 .05084 L
s
.02381 .024 m
.02756 .024 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
gsave
.5 .63053 -189 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 32.5 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 28.500 moveto
 541.000 28.500 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 28.500 lineto
closepath
clip newpath
63.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Smile) show
99.000 13.000 moveto
(for) show
123.000 13.000 moveto
(Elasticity) show
189.000 13.000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(+) show
201.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Spread) show
243.000 13.000 moveto
(Model) show
279.000 13.000 moveto
(of) show
297.000 13.000 moveto
(Put) show
63.000 25.500 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .55359 L
.10458 .50199 L
.14415 .45591 L
.18221 .41364 L
.22272 .37076 L
.26171 .33147 L
.30316 .29178 L
.34309 .25554 L
.3815 .22252 L
.42237 .18939 L
.46172 .1595 L
.49955 .1327 L
.53984 .1064 L
.57861 .08346 L
.61984 .06189 L
.65954 .04425 L
.69774 .03061 L
.71905 .02462 L
.73838 .02031 L
.74806 .01857 L
.75853 .01704 L
.76334 .01646 L
.76842 .01594 L
.77309 .01553 L
.77751 .01522 L
.77989 .01509 L
.78247 .01496 L
.78462 .01488 L
.78699 .0148 L
.78831 .01477 L
.78955 .01475 L
.79086 .01473 L
.79159 .01472 L
.79227 .01472 L
.7934 .01472 L
.79465 .01472 L
.79594 .01472 L
.79715 .01474 L
.7983 .01476 L
.79954 .01478 L
.80179 .01485 L
.80399 .01493 L
.80601 .01502 L
.81061 .01529 L
.81548 .01568 L
.82492 .01674 L
.83506 .01832 L
.84463 .02026 L
.85337 .02243 L
Mistroke
.87239 .02853 L
.89313 .03749 L
.91284 .04842 L
.93383 .06285 L
.97546 .10081 L
.97619 .10159 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{261, 160.938},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000560000bA000`40O003h00O8goo00<007ooOol0
ogoo8Goo000SOol00`00Oomoo`3oOolQOol002=oo`03001oogoo0?moob5oo`008goo00<007ooOol0
ogoo8Goo000SOol00`00Oomoo`37Ool;001>Ool002=oo`03001oogoo0;ioo`T000]oo`L004Moo`00
8goo00<007ooOol0^goo0`006goo1@00@Woo000SOol2002hOol4000SOol3000oOol002=oo`03001o
ogoo0;Aoo`<002Yoo`8003eoo`008goo00<007ooOol08Woo0`001Goo0`007Woo0`001Woo0`0077oo
1@000goo0`001Goo0`0067oo1@000goo1@000goo1P0057oo1@000Woo1@0017oo0`0027oo0`003Goo
1@000goo0`001Goo0`006Woo000SOol00`00Oomoo`0QOol01@00Oomoogoo00000goo00D007ooOomo
o`00021oo`03001oogoo009oo`05001oogooOol0000MOol00`00Oomoo`02Ool01@00Oomoogoo0000
0goo00D007ooOomoo`0001Uoo`03001oogoo00Eoo`03001oogoo009oo`<00003Ool007oo01Moo`03
001oogoo00=oo`04001oogoo0003Ool01@00Oomoogoo00002Woo0P003Goo00<007ooOol00Woo00D0
07ooOomoo`0000=oo`05001oogooOol0000IOol002=oo`03001oogoo025oo`05001oogooOol00003
Ool01@00Oomoogoo00008Goo00D007ooOomoo`0000=oo`03001oogoo01]oo`03001oogoo009oo`05
001oogooOol00003Ool01@00Oomoogoo00006Goo00<007ooOol01Goo00<007oo00000`000goo00<0
07ooOol05Woo00<007ooOol017oo00<007ooOol00goo00D007ooOomoo`0000aoo`8000]oo`03001o
ogoo00Ioo`05001oogooOol00003Ool00`00Oomoo`0GOol002=oo`03001oogoo025oo`05001oogoo
Ool00003Ool01@00Oomoogoo00007Woo10000goo00D007ooOomoo`0001eoo`03001oogoo009oo`05
001oogooOol00003Ool01@00Oomoogoo00006Goo00<007ooOol017oo0`000goo00D007ooOomoo`00
01Qoo`03001oogoo00Eoo`03001oogoo009oo`05001oogooOol0000>Ool20009Ool00`00Oomoo`06
Ool01@00Oomoogoo00000goo00<007ooOol05goo000SOol00`00Oomoo`0ROol30004Ool01@00Oomo
ogoo00007Goo00D007ooOomoo`0000=oo`05001oogooOol0000MOol00`00Oomoo`02Ool01@00Oomo
ogoo00000goo00D007ooOomoo`0001Uoo`03001oogoo009oo`800003Ool007oo00=oo`05001oogoo
Ool0000HOol00`00Oomoo`06Ool01@00Oomoogoo00000goo00<007ooOol03Woo0P001goo00<007oo
Ool017oo0P0017oo00D007ooOomoo`0001Uoo`008goo0P008Woo00D007ooOomoo`0000=oo`05001o
ogooOol0000MOol01@00Oomoogoo00000goo00D007ooOomoo`0001eoo`03001oogoo009oo`05001o
ogooOol00003Ool01@00Oomoogoo00006Goo00<007oo00000P000goo00<007ooOol00Woo00D007oo
Oomoo`0001Qoo`03001oogoo00Ioo`05001oogooOol00003Ool00`00Oomoo`0@Ool00`00Oomoo`04
Ool00`00Oomoo`06Ool01@00Oomoogoo00000goo00<007ooOol05goo000SOol00`00Oomoo`0QOol0
1@00Oomoogoo00000goo00D007ooOomoo`0001eoo`05001oogooOol00003Ool01@00Oomoogoo0000
7Goo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol0000IOol20006Ool00`00Oomo
o`02Ool01@00Oomoogoo000067oo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol0
000COol00`00Oomoo`03Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo00D007ooOomoo`0001Uo
o`008goo00<007ooOol08Woo0`001Goo0`007goo0`001Goo0`0077oo0`001Goo0`001Goo0`0067oo
0`001Goo0`001Goo0`005goo0`001Goo0`001Goo0`005Goo00<007ooOol00`001Goo0`001Goo0`00
6Woo000SOol00`00Oomoo`2LOol2001COol2000]Ool002=oo`03001oogoo09Yoo`8005Moo`03001o
ogoo02Yoo`008goo00<007ooOol0V7oo0P00FWoo00<007ooOol0:Goo000SOol00`00Oomoo`2GOol0
0`00Oomoo`1KOol00`00Oomoo`0XOol001aoool000l001aoo`008goo00<007ooOol01Goo00<007oo
Ool01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007oo
Ool01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007oo
Ool01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007oo
Ool01Woo00<007ooOol00Woo0P0000=oo`00Ool01goo00<007ooOol01Goo00<007ooOol01Woo00<0
07ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<0
07ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol00P0017oo00<007ooOol087oo
000SOol00`00Oomoo`0WOol00`00Oomoo`0XOol00`00Oomoo`0XOol00`00Oomoo`0AOol2000DOol0
0`00Oomoo`0XOol00`00Oomoo`0TOol01@00Oomoogoo00008Woo000SOol00`00Oomoo`2?Ool2001Y
Ool00`00Oomoo`0SOol002=oo`03001oogoo08ioo`03001oogoo06Yoo`03001oogoo029oo`008goo
00<007ooOol0S7oo0P00KWoo00<007ooOol08Goo000SOol00`00Oomoo`2:Ool2001aOol00`00Oomo
o`0POol002=oo`8008Yoo`03001oogoo075oo`03001oogoo021oo`008goo00<007ooOol0Qgoo0P00
Ugoo000SOol00`00Oomoo`26Ool00`00Oomoo`2GOol002=oo`03001oogoo08Aoo`8009Yoo`008goo
00<007ooOol0PWoo0P00W7oo000SOol00`00Oomoo`21Ool00`00Oomoo`2LOol002=oo`03001oogoo
07moo`8009moo`008goo00<007ooOol0OWoo00<007ooOol0Wgoo000SOol2001mOol2002ROol002=o
o`03001oogoo07]oo`03001oogoo0:9oo`008goo00<007ooOol0NGoo0P00YGoo000SOol00`00Oomo
o`1hOol00`00Oomoo`2UOol002=oo`03001oogoo07Moo`03001oogoo0:Ioo`008goo00<007ooOol0
MGoo0P00ZGoo000SOol00`00Oomoo`1dOol00`00Oomoo`2YOol002=oo`8007=oo`800:aoo`008goo
00<007ooOol0LGoo00<007ooOol0[7oo000SOol00`00Oomoo`1`Ool00`00Oomoo`2]Ool002=oo`03
001oogoo06ioo`800;1oo`008goo00<007ooOol0KGoo00<007ooOol0/7oo000SOol00`00Oomoo`1[
Ool2002cOol002=oo`03001oogoo06Yoo`03001oogoo0;=oo`008goo0P00JWoo00<007ooOol0]7oo
000SOol00`00Oomoo`1WOol2002gOol002=oo`03001oogoo06Ioo`03001oogoo0;Moo`008goo00<0
07ooOol0IGoo00<007ooOol0^7oo000SOol00`00Oomoo`1TOol00`00Oomoo`2iOol0009oo`<000Io
o`8000Aoo`D000=oo`D000Eoo`03001oogoo069oo`800;aoo`0000=oo`00Ool00Woo00<007ooOol0
0goo0P001Goo00@007ooOol000Eoo`03001oogoo00Eoo`03001oogoo065oo`03001oogoo0;aoo`00
00=oo`00Ool00Woo00<007ooOol02goo00<007ooOol01Goo00<007ooOol01Goo0P00HGoo00<007oo
Ool0_Goo00000goo001oo`02Ool00`00Oomoo`0<Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00
Oomoo`1OOol00`00Oomoo`2nOol00003Ool007oo009oo`03001oogoo00eoo`03001oogoo00=oo`03
001oogoo00Eoo`03001oogoo05ioo`03001oogoo0;moo`0000=oo`00Ool00Woo00<007ooOol03Goo
00<007ooOol00goo00<007ooOol01Goo00<007ooOol0G7oo0P00`Woo00000goo001oo`02Ool00`00
Oomoo`09Ool01@00Oomoogoo00001Goo00<007ooOol01Goo00<007ooOol0Fgoo00<007ooOol0`Woo
0002Ool3000=Ool30004Ool30007Ool00`00Oomoo`1JOol00`00Oomoo`33Ool002=oo`03001oogoo
05Uoo`03001oogoo0<Aoo`008goo00<007ooOol0F7oo00<007ooOol0aGoo000SOol2001HOol00`00
Oomoo`36Ool002=oo`03001oogoo05Ioo`03001oogoo0<Moo`008goo00<007ooOol0EGoo00<007oo
Ool0b7oo000SOol00`00Oomoo`1DOol00`00Oomoo`39Ool002=oo`03001oogoo059oo`800<aoo`00
8goo00<007ooOol0DGoo00<007ooOol0c7oo000SOol00`00Oomoo`1@Ool00`00Oomoo`3=Ool002=o
o`80051oo`03001oogoo0<ioo`008goo00<007ooOol0CWoo00<007ooOol0cgoo000SOol00`00Oomo
o`1=Ool00`00Oomoo`3@Ool002=oo`03001oogoo04aoo`03001oogoo0=5oo`008goo00<007ooOol0
Bgoo00<007ooOol0dWoo000SOol00`00Oomoo`1:Ool00`00Oomoo`3COol002=oo`03001oogoo04Uo
o`03001oogoo0=Aoo`008goo0P00B7oo0P00egoo000SOol00`00Oomoo`16Ool00`00Oomoo`3GOol0
02=oo`03001oogoo04Eoo`03001oogoo0=Qoo`008goo00<007ooOol0A7oo00<007ooOol0fGoo000S
Ool00`00Oomoo`13Ool00`00Oomoo`3JOol002=oo`03001oogoo049oo`03001oogoo0=]oo`008goo
00<007ooOol0@Goo00<007ooOol0g7oo000SOol20011Ool00`00Oomoo`3MOol002=oo`03001oogoo
03moo`03001oogoo0=ioo`008goo00<007ooOol0?Woo00<007ooOol0ggoo000SOol00`00Oomoo`0m
Ool00`00Oomoo`3POol002=oo`03001oogoo03aoo`03001oogoo0>5oo`000Woo0`001Woo0P0017oo
1@000Woo1@001Woo00<007ooOol0>goo00<007ooOol0hWoo00000goo001oo`02Ool00`00Oomoo`03
Ool20005Ool01000Oomoo`000goo00@007ooOol000Ioo`03001oogoo03Yoo`03001oogoo0>=oo`00
00=oo`00Ool00Woo00<007ooOol02goo00<007ooOol017oo00<007ooOol01Woo0P00>Woo00<007oo
Ool0i7oo00000goo001oo`02Ool00`00Oomoo`0<Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00
Oomoo`0hOol00`00Oomoo`3UOol00003Ool007oo009oo`03001oogoo00eoo`03001oogoo00Aoo`03
001oogoo00Aoo`03001oogoo03Moo`03001oogoo0>Ioo`0000=oo`00Ool00Woo00<007ooOol03Goo
00<007ooOol017oo00<007ooOol017oo00<007ooOol0=Woo00<007ooOol0igoo00000goo001oo`02
Ool00`00Oomoo`09Ool01@00Oomoogoo00000Woo00D007ooOomoo`0000Ioo`03001oogoo03Eoo`03
001oogoo0>Qoo`000Woo0`003Goo0`0017oo0`001goo00<007ooOol0=7oo00<007ooOol0jGoo000S
Ool00`00Oomoo`0cOol00`00Oomoo`3ZOol002=oo`03001oogoo039oo`03001oogoo0>]oo`008goo
0P00<goo00<007ooOol0jgoo000SOol00`00Oomoo`0aOol00`00Oomoo`3/Ool002=oo`03001oogoo
031oo`03001oogoo0>eoo`008goo00<007ooOol0;goo00<007ooOol0kWoo000SOol00`00Oomoo`0^
Ool00`00Oomoo`3_Ool002=oo`03001oogoo02eoo`03001oogoo0?1oo`008goo00<007ooOol0;7oo
00<007ooOol0lGoo000SOol2000/Ool00`00Oomoo`3bOol002=oo`03001oogoo02Yoo`03001oogoo
0?=oo`008goo00<007ooOol0:Goo00<007ooOol0m7oo000SOol00`00Oomoo`0XOol00`00Oomoo`3e
Ool002=oo`03001oogoo02Moo`03001oogoo0?Ioo`008goo00<007ooOol09goo00<007ooOol0mWoo
000SOol00`00Oomoo`0VOol00`00Oomoo`3gOol002=oo`8002Ioo`03001oogoo0?Qoo`008goo00<0
07ooOol097oo00<007ooOol0nGoo000SOol00`00Oomoo`0SOol00`00Oomoo`3jOol002=oo`03001o
ogoo029oo`03001oogoo0?]oo`008goo00<007ooOol08Goo00<007ooOol0o7oo000SOol00`00Oomo
o`0POol00`00Oomoo`3mOol002=oo`03001oogoo01moo`03001oogoo0?ioo`008goo0P007goo00<0
07ooOol0ogoo000SOol00`00Oomoo`0MOol00`00Oomoo`3oOol1Ool002=oo`03001oogoo01eoo`03
001oogoo0?moo`5oo`008goo00<007ooOol077oo00<007ooOol0ogoo0Woo000SOol00`00Oomoo`0K
Ool00`00Oomoo`3oOol3Ool0009oo`<000Ioo`8000Aoo`D000=oo`<000Moo`03001oogoo01Yoo`03
001oogoo0?moo`Aoo`0000=oo`00Ool00Woo00<007ooOol00goo0P001Goo00@007ooOol0009oo`05
001oogooOol00006Ool00`00Oomoo`0IOol00`00Oomoo`3oOol5Ool00003Ool007oo009oo`03001o
ogoo00]oo`03001oogoo00Ioo`03001oogoo00Aoo`03001oogoo01Qoo`03001oogoo0?moo`Ioo`00
00=oo`00Ool00Woo00<007ooOol037oo00<007ooOol01Goo00<007ooOol017oo0P006Goo00<007oo
Ool0ogoo1Woo00000goo001oo`02Ool00`00Oomoo`0=Ool00`00Oomoo`02Ool20007Ool00`00Oomo
o`0GOol00`00Oomoo`3oOol7Ool00003Ool007oo009oo`03001oogoo00eoo`03001oogoo00Aoo`03
001oogoo00Aoo`03001oogoo01Ioo`03001oogoo0?moo`Qoo`0000=oo`00Ool00Woo00<007ooOol0
2Goo00D007ooOomoo`00009oo`05001oogooOol00006Ool00`00Oomoo`0EOol00`00Oomoo`3oOol9
Ool0009oo`<000eoo`<000Aoo`<000Moo`03001oogoo01Aoo`03001oogoo0?moo`Yoo`008goo00<0
07ooOol04goo00<007ooOol0ogoo2goo000SOol00`00Oomoo`0COol00`00Oomoo`3oOol;Ool002=o
o`8001=oo`03001oogoo0?moo`aoo`008goo00<007ooOol04Goo00<007ooOol0ogoo3Goo000SOol0
0`00Oomoo`0@Ool00`00Oomoo`3oOol>Ool002=oo`03001oogoo00moo`03001oogoo0?moo`moo`00
8goo00<007ooOol03Woo00<007ooOol0ogoo47oo000SOol00`00Oomoo`0=Ool00`00Oomoo`3oOolA
Ool002=oo`03001oogoo00eoo`03001oogoo0?mooa5oo`008goo0P003Goo00<007ooOol0ogoo4Woo
000SOol00`00Oomoo`0;Ool00`00Oomoo`3oOolCOol002=oo`03001oogoo00Yoo`03001oogoo0?mo
oaAoo`008goo00<007ooOol02Goo00<007ooOol0ogoo5Goo000SOol00`00Oomoo`08Ool00`00Oomo
o`3oOolFOol002=oo`03001oogoo00Moo`03001oogoo0?mooaMoo`008goo00<007ooOol01goo00<0
07ooOol0ogoo5goo000SOol20007Ool00`00Oomoo`3oOolHOol002=oo`03001oogoo00Eoo`03001o
ogoo0?mooaUoo`008goo00<007ooOol017oo00<007ooOol0ogoo6Woo000SOol00`00Oomoo`04Ool0
0`00Oomoo`3oOolJOol002=oo`03001oogoo00=oo`03001oogoo0?mooa]oo`008goo00<007ooOol0
0Woo00<007ooOol0ogoo77oo000SOol00`00Oomoo`02Ool00`00Oomoo`3oOolLOol002=oo`80009o
o`03001oogoo0?mooaeoo`008goo00@007ooOol00?moob1oo`008goo00<007oo0000ogoo8Goo000S
Ool00`00Ool0003oOolQOol002=oo`800?moob9oo`000Woo0`001Woo0P0017oo1@0017oo0`001Woo
00<007ooOol0ogoo8Goo00000goo001oo`02Ool00`00Oomoo`03Ool20005Ool01000Oomoo`001Goo
00<007ooOol01Goo00<007ooOol0ogoo8Goo00000goo001oo`02Ool00`00Oomoo`0;Ool00`00Oomo
o`02Ool50006Ool00`00Oomoo`3oOolQOol00003Ool007oo009oo`03001oogoo00aoo`05001oogoo
Ool00002Ool00`00Oomoo`05Ool2003oOolROol00003Ool007oo009oo`03001oogoo00eoo`07001o
ogooOol007oo00001goo00<007ooOol0ogoo8Goo00000goo001oo`02Ool00`00Oomoo`0=Ool01`00
Oomoogoo001oo`000?moob]oo`0000=oo`00Ool00Woo00<007ooOol02Goo00D007ooOomoo`0000Ao
o`800?moob]oo`000Woo0`003Goo0`001Woo00<007ooOol0ogoo:Goo003oOom7Ool00?moodMoo`00
ogooAgoo003oOom7Ool00?moodMoo`00ogooAgoo003oOom7Ool00?moodMoo`00ogooAgoo003oOom7
Ool00?moodMoo`00ogooAgoo003oOom7Ool00?moodMoo`00ogooAgoo003oOom7Ool009Ioo`@0025o
o`@008Moo`00V7oo00<007ooOol08Goo00<007ooOol0Qgoo002IOol00`00Oomoo`0POol00`00Oomo
o`27Ool000Ioo`D0009oo`80009oo`03001oogoo00800003Ool0000000<00003Ool0000000<000Ao
o`@000Uoo`H000=oo`@0009oo`H000Qoo`H000=oo`D000Aoo`D00003Ool0000000<000Eoo`<000Ao
o`D000=oo`@000Aoo`D000=oo`<000Eoo`03001oogoo00]oo`03001oogoo00Uoo`D000Aoo`03001o
o`0000800003Ool0000000@000=oo`@000Aoo`D000=oo`<00003Ool0000000Eoo`<0009oo`<0009o
o`@000Aoo`<00004Ool00000Ool40003Ool5000:Ool40003Ool60009Ool40006Ool50003Ool30006
Ool000Ioo`03001oogoo009oo`04001oogoo0002Ool01000Oomoo`0017oo00<007ooOol00goo00<0
07ooOol00goo00<007ooOol00Woo00<007ooOol027oo00<007ooOol00goo00<007ooOol00Woo00D0
07ooOomoo`0000aoo`05001oogooOol00005Ool00`00Oomoo`03Ool01@00Oomoogoo00000Woo00<0
07ooOol00Woo00D007ooOomoo`0000=oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo009o
o`03001oogoo00=oo`03001oogoo009oo`05001oogooOol00003Ool00`00Ool0000<Ool00`00Oomo
o`09Ool00`00Oomoo`02Ool01000Oomoogoo0P000goo00@007ooOol000Eoo`03001oogoo009oo`04
001oogoo0003Ool01@00Oomoogoo00000goo0P001goo00<007ooOol00Woo00@007ooOol000Aoo`04
001oogoo0003Ool200000goo001oo`03Ool00`00Oomoo`02Ool00`00Oomoo`09Ool00`00Oomoo`02
Ool00`00Oomoo`02Ool00`00Oomoo`0;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000Eoo`001Woo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo0004Ool00`00Oomo
o`03Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01@00Oomo
ogoo000037oo00D007ooOomoo`0000Eoo`03001oogoo00=oo`05001oogooOol00005Ool30003Ool0
0`00Oomoo`07Ool00`00Oomoo`02Ool00`00Oomoo`08Ool00`00Oomoo`02Ool00`00Oomoo`05Ool0
0`00Ool0000:Ool50009Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00@007ooOol000Eoo`03
001oogoo00Eoo`05001oogooOol00003Ool00`00Oomoo`02Ool00`00Oomoo`05Ool00`00Oomoo`02
Ool01000Oomoo`0017oo00@007ooOol000Aoo`03001oo`0000Uoo`03001oogoo00Uoo`03001oogoo
009oo`03001oogoo009oo`03001oogoo00]oo`03001oogoo00Eoo`05001oogooOol00003Ool00`00
Oomoo`07Ool000Uoo`<0009oo`04001oogoo0002Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00
Oomoo`03Ool6000:Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01@00Oomoogoo000037oo00<007oo
00001goo00<007ooOol017oo10000Woo0`001Woo00<007ooOol01goo00<007ooOol00Woo00<007oo
Ool027oo00<007ooOol00Woo00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00aoo`<0
00=oo`03001oogoo009oo`04001oogoo0005Ool60003Ool40003Ool00`00Oomoo`02Ool00`00Oomo
o`05Ool01P00Ool00000Ool0009oo`03001oogoo009oo`04001oogoo0004Ool00`00Ool000050004
Ool00`00Oomoo`09Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`0;Ool40004Ool01@00
Oomoogoo00000goo00<007ooOol01goo0006Ool30005Ool01000Oomoo`000Woo00<007ooOol00Woo
00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol027oo00<007ooOol00goo
00<007ooOol00Woo00@007ooOomoo`8000]oo`<000Moo`03001oogoo00Moo`04001oogoo0004Ool0
1@00Oomoogoo00002Goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0
0Woo00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Uoo`<000Ioo`8000=oo`03001o
ogoo008000Aoo`03001oogoo009oo`03001oogoo00Aoo`05001oogooOol00003Ool20007Ool01P00
Ool00000Ool0009oo`03001oogoo009oo`04001oogoo0003Ool200000goo001oo`03Ool00`00Oomo
o`02Ool00`00Oomoo`09Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`0;Ool01@00Oomo
ogoo00000goo00D007ooOomoo`0000=oo`03001oogoo00Moo`001Woo00<007ooOol00Woo00<007oo
00000`0000=oo`0000000goo0`001Goo00<007ooOol017oo10002Goo1P000goo10000goo0P0000Ao
o`00000000Qoo`05001oo`00Ool00005Ool00`00Oomoo`04Ool30004Ool500000goo000000040004
Ool30005Ool50003Ool30002Ool60002Ool3000017oo000000005Woo00<007ooOol00Woo00<007oo
Ool00P0000Aoo`000000009oo`800004Ool000000002Ool40004Ool30005Ool300000goo001oo`06
Ool20002Ool20003Ool40004Ool3000017oo001oogoo10001Goo00<007ooOol02Woo10000goo1P00
2Woo00D007ooOomoo`00009oo`80009oo`800003Ool0000000@000Ioo`001Woo00<007ooOol00Woo
00<007ooOol04Goo00<007ooOol04goo00<007ooOol06Goo00D007ooOomoo`0000Eoo`03001oogoo
01=oo`03001oogoo01aoo`03001oogoo021oo`03001oogoo009oo`03001oogoo02Aoo`03001oogoo
00Eoo`80009oo`8000moo`03001oogoo00Uoo`03001oogoo01=oo`03001oogoo00]oo`05001oogoo
Ool0000;Ool00`00Oomoo`07Ool000Moo`D000eoo`03001oogoo00=oo`03001oogoo01=oo`03001o
ogoo01Qoo`H000Eoo`03001oogoo01=oo`03001oogoo00Moo`03001oogoo00eoo`03001oogoo009o
o`03001oogoo025oo`D002Ioo`03001oogoo00Aoo`8000Aoo`8000ioo`03001oogoo00Uoo`03001o
ogoo01=oo`03001oogoo00Yoo`D000aoo`03001oogoo00Moo`006Goo00@007ooOomoo`<001Ioo`<0
021oo`<001moo`03001oogoo00eoo`03001oogoo051oo`8001]oo`8000Uoo`<001Ioo`<002Aoo`00
ogooAgoo003oOom7Ool00001\
\>"],
  ImageRangeCache->{{{0, 325.25}, {200.125, 0}} -> {61.727, 0.194212, 
  0.292949, 0.000346423}}]
}, Open  ]],

Cell["\<\
These are qualitatively similar to what one sees in equity markets \
(note that we regard the FX case as totally different - it is not a candidate \
for this type of elasticity treatment). We see that the effect of the \
elasticity in combination with the spread is to dramatically raise the OTM \
implied volatilities, and to flatten or curve the ITM region. It is clear \
that the SRCEV model does capture part of the skew effect seen in equity \
markets. The reader should note the discussion by R.J. Brenner, \"Volatility \
is not constant\", for further comments on CEV and related level-dependent \
volatility models in the book by Nelken (1990). The time-dependence of the \
elasticity and spread-adjusted implied volatility is that the smile becomes \
more of a skew, and the overall level drifts down - here is the implied \
volatility surface for 0.5 to 5 years:\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot3D[
      PutForwardBackward[100, \ K, \ 0.05, \ 0, \ 0.2, t, \ 2.5], \ {K, \ 
        70, \ 130}, {t, \ 0.5, \ 5}, \ \ PlotLabel\  -> \ 
        "\<Vol Surface for Elasticity\n+ Spread Model of Put\n\>"]; \)\)], 
  "Input",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
5.55112e-17 1.04977 -0.0679587 1.04977 [
[.5 .82364 -80 0 ]
[.5 .82364 80 37 ]
[.12586 .2075 -12 -8.99448 ]
[.12586 .2075 0 .00552 ]
[.3345 .12542 -16.7601 -9 ]
[.3345 .12542 1.2399 0 ]
[.56094 .0364 -15.5091 -9 ]
[.56094 .0364 2.49085 0 ]
[.73867 .05295 0 -6.20496 ]
[.73867 .05295 6 2.79504 ]
[.80849 .15988 0 -6.10118 ]
[.80849 .15988 6 2.89882 ]
[.87175 .25674 0 -6.0093 ]
[.87175 .25674 6 2.9907 ]
[.92931 .34489 0 -5.9274 ]
[.92931 .34489 6 3.0726 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.0152 .321 -18 -2.78958 ]
[.0152 .321 0 6.21042 ]
[.00481 .38612 -24 -2.84486 ]
[.00481 .38612 0 6.15514 ]
[-0.00603 .45401 -24 -2.90273 ]
[-0.00603 .45401 0 6.09727 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
gsave
.5 .82364 -141 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 45 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 41.000 moveto
 541.000 41.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 41.000 lineto
closepath
clip newpath
63.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Vol) show
87.000 13.000 moveto
(Surface) show
135.000 13.000 moveto
(for) show
159.000 13.000 moveto
(Elasticity) show
63.000 25.500 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(+) show
75.000 25.500 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Spread) show
117.000 25.500 moveto
(Model) show
153.000 25.500 moveto
(of) show
171.000 25.500 moveto
(Put) show
63.000 38.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.13514 .21677 m
.13978 .22141 L
s
[(80)] .12586 .2075 1 .99877 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(100)] .3345 .12542 .86223 1 Mshowa
.56863 .04703 m
.57247 .05235 L
s
[(120)] .56094 .0364 .72324 1 Mshowa
.125 Mabswid
.18558 .19702 m
.18832 .19985 L
s
.23703 .17687 m
.23971 .17975 L
s
.28951 .15632 m
.29214 .15925 L
s
.39772 .11395 m
.40023 .11699 L
s
.4535 .09211 m
.45595 .0952 L
s
.51046 .06981 m
.51284 .07294 L
s
.08568 .23614 m
.08851 .23887 L
s
.62804 .02377 m
.63027 .02701 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.72639 .0576 m
.72026 .05993 L
s
[(1)] .73867 .05295 -1 .37888 Mshowa
.79613 .16428 m
.78995 .16648 L
s
[(2)] .80849 .15988 -1 .35582 Mshowa
.8593 .26091 m
.85308 .263 L
s
[(3)] .87175 .25674 -1 .3354 Mshowa
.9168 .34886 m
.91054 .35084 L
s
[(4)] .92931 .34489 -1 .3172 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(5)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.74092 .07982 m
.73723 .08119 L
s
.75514 .10158 m
.75145 .10294 L
s
.76908 .1229 m
.76538 .12425 L
s
.78274 .14379 m
.77904 .14513 L
s
.80926 .18436 m
.80555 .18566 L
s
.82213 .20405 m
.81842 .20534 L
s
.83476 .22337 m
.83104 .22465 L
s
.84715 .24232 m
.84342 .24358 L
s
.87123 .27915 m
.8675 .28039 L
s
.88294 .29706 m
.8792 .29829 L
s
.89443 .31464 m
.89069 .31586 L
s
.90572 .33191 m
.90197 .33311 L
s
.92768 .36551 m
.92393 .36668 L
s
.93838 .38186 m
.93462 .38303 L
s
.94888 .39793 m
.94512 .39909 L
s
.9592 .41372 m
.95544 .41486 L
s
.71157 .03492 m
.70789 .03634 L
s
.69643 .01177 m
.69276 .01319 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.02746 .31634 m
.0336 .314 L
s
[(0.2)] .0152 .321 1 -0.38009 Mshowa
.01712 .38159 m
.02328 .37933 L
s
[(0.22)] .00481 .38612 1 -0.36781 Mshowa
.00634 .44962 m
.01252 .44743 L
s
[(0.24)] -0.00603 .45401 1 -0.35495 Mshowa
.125 Mabswid
.03495 .26911 m
.03861 .26768 L
s
.03248 .28469 m
.03615 .28328 L
s
.02998 .30043 m
.03366 .29903 L
s
.02492 .3324 m
.0286 .33101 L
s
.02234 .34863 m
.02603 .34725 L
s
.01975 .36503 m
.02344 .36366 L
s
.01447 .39833 m
.01817 .39698 L
s
.01179 .41525 m
.01549 .41391 L
s
.00908 .43234 m
.01278 .43102 L
s
.00357 .46708 m
.00729 .46578 L
s
.00078 .48473 m
.00449 .48344 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.787 .773 .84 r
.37578 .75969 .39898 .77938 .43809 .74908 .41537 .72894 Metetra
.781 .773 .846 r
.41537 .72894 .43809 .74908 .4773 .72015 .45505 .69959 Metetra
.774 .773 .851 r
.45505 .69959 .4773 .72015 .51666 .69241 .49488 .67144 Metetra
.769 .772 .856 r
.49488 .67144 .51666 .69241 .55623 .6657 .53492 .64433 Metetra
.763 .771 .86 r
.53492 .64433 .55623 .6657 .59603 .63989 .57518 .61812 Metetra
.758 .771 .863 r
.57518 .61812 .59603 .63989 .6361 .61486 .61572 .59271 Metetra
.753 .77 .867 r
.61572 .59271 .6361 .61486 .67648 .59053 .65657 .56799 Metetra
.748 .769 .87 r
.65657 .56799 .67648 .59053 .71719 .5668 .69776 .5439 Metetra
.744 .769 .873 r
.69776 .5439 .71719 .5668 .75827 .54362 .73932 .52036 Metetra
.74 .768 .876 r
.73932 .52036 .75827 .54362 .79973 .52093 .78128 .49732 Metetra
.736 .767 .878 r
.78128 .49732 .79973 .52093 .84161 .49868 .82366 .47473 Metetra
.731 .767 .881 r
.82366 .47473 .84161 .49868 .88394 .47683 .86649 .45256 Metetra
.728 .766 .883 r
.86649 .45256 .88394 .47683 .92674 .45536 .90981 .43078 Metetra
.724 .766 .886 r
.90981 .43078 .92674 .45536 .97003 .43424 .95365 .40938 Metetra
.787 .773 .84 r
.35196 .73947 .37578 .75969 .41537 .72894 .39204 .70827 Metetra
.781 .773 .846 r
.39204 .70827 .41537 .72894 .45505 .69959 .43221 .67848 Metetra
.774 .773 .851 r
.43221 .67848 .45505 .69959 .49488 .67144 .47252 .64991 Metetra
.769 .772 .856 r
.47252 .64991 .49488 .67144 .53492 .64433 .51303 .62239 Metetra
.763 .772 .86 r
.51303 .62239 .53492 .64433 .57518 .61812 .55377 .59578 Metetra
.758 .771 .864 r
.55377 .59578 .57518 .61812 .61572 .59271 .5948 .56997 Metetra
.753 .77 .867 r
.5948 .56997 .61572 .59271 .65657 .56799 .63613 .54487 Metetra
.748 .769 .87 r
.63613 .54487 .65657 .56799 .69776 .5439 .6778 .5204 Metetra
.744 .769 .873 r
.6778 .5204 .69776 .5439 .73932 .52036 .71985 .49649 Metetra
.739 .768 .876 r
.71985 .49649 .73932 .52036 .78128 .49732 .76231 .4731 Metetra
.735 .767 .879 r
.76231 .4731 .78128 .49732 .82366 .47473 .8052 .45017 Metetra
.731 .767 .882 r
.8052 .45017 .82366 .47473 .86649 .45256 .84857 .42769 Metetra
.727 .767 .884 r
.84857 .42769 .86649 .45256 .90981 .43078 .89242 .40561 Metetra
.723 .766 .887 r
.89242 .40561 .90981 .43078 .95365 .40938 .93681 .38393 Metetra
.787 .773 .84 r
.32749 .7187 .35196 .73947 .39204 .70827 .36807 .68704 Metetra
.781 .773 .846 r
.36807 .68704 .39204 .70827 .43221 .67848 .40874 .65681 Metetra
.774 .773 .851 r
.40874 .65681 .43221 .67848 .47252 .64991 .44955 .6278 Metetra
.769 .772 .856 r
.44955 .6278 .47252 .64991 .51303 .62239 .49054 .59986 Metetra
.763 .772 .86 r
.49054 .59986 .51303 .62239 .55377 .59578 .53178 .57283 Metetra
.758 .771 .864 r
.53178 .57283 .55377 .59578 .5948 .56997 .57329 .54662 Metetra
.753 .77 .867 r
.57329 .54662 .5948 .56997 .63613 .54487 .61511 .52113 Metetra
.748 .77 .871 r
.61511 .52113 .63613 .54487 .6778 .5204 .65729 .49628 Metetra
.743 .769 .874 r
.65729 .49628 .6778 .5204 .71985 .49649 .69984 .472 Metetra
.739 .768 .877 r
.69984 .472 .71985 .49649 .76231 .4731 .74281 .44826 Metetra
.734 .768 .88 r
.74281 .44826 .76231 .4731 .8052 .45017 .78623 .425 Metetra
.73 .767 .883 r
.78623 .425 .8052 .45017 .84857 .42769 .83013 .4022 Metetra
.726 .767 .886 r
.83013 .4022 .84857 .42769 .89242 .40561 .87455 .37984 Metetra
.721 .767 .889 r
.87455 .37984 .89242 .40561 .93681 .38393 .91951 .35792 Metetra
.787 .773 .84 r
.30235 .69736 .32749 .7187 .36807 .68704 .34345 .66523 Metetra
.781 .773 .846 r
.34345 .66523 .36807 .68704 .40874 .65681 .38462 .63454 Metetra
.774 .773 .851 r
.38462 .63454 .40874 .65681 .44955 .6278 .42593 .60509 Metetra
.768 .772 .856 r
.42593 .60509 .44955 .6278 .49054 .59986 .46743 .57671 Metetra
.763 .772 .86 r
.46743 .57671 .49054 .59986 .53178 .57283 .50917 .54926 Metetra
.758 .771 .864 r
.50917 .54926 .53178 .57283 .57329 .54662 .55118 .52264 Metetra
.752 .77 .868 r
.55118 .52264 .57329 .54662 .61511 .52113 .59351 .49675 Metetra
.748 .77 .871 r
.59351 .49675 .61511 .52113 .65729 .49628 .63619 .47151 Metetra
.743 .769 .874 r
.63619 .47151 .65729 .49628 .69984 .472 .67927 .44687 Metetra
.738 .769 .877 r
.67927 .44687 .69984 .472 .74281 .44826 .72276 .42277 Metetra
.734 .768 .88 r
.72276 .42277 .74281 .44826 .78623 .425 .76672 .39919 Metetra
.729 .768 .884 r
.76672 .39919 .78623 .425 .83013 .4022 .81117 .37609 Metetra
.725 .768 .887 r
.81117 .37609 .83013 .4022 .87455 .37984 .85616 .35348 Metetra
.72 .768 .89 r
.85616 .35348 .87455 .37984 .91951 .35792 .90172 .33134 Metetra
.787 .773 .84 r
.27651 .67543 .30235 .69736 .34345 .66523 .31814 .64282 Metetra
.781 .773 .846 r
.31814 .64282 .34345 .66523 .38462 .63454 .35983 .61166 Metetra
.774 .773 .851 r
.35983 .61166 .38462 .63454 .42593 .60509 .40166 .58175 Metetra
.768 .772 .856 r
.40166 .58175 .42593 .60509 .46743 .57671 .44367 .55292 Metetra
.763 .772 .86 r
.44367 .55292 .46743 .57671 .50917 .54926 .48592 .52504 Metetra
.757 .771 .864 r
.48592 .52504 .50917 .54926 .55118 .52264 .52844 .498 Metetra
.752 .77 .868 r
.52844 .498 .55118 .52264 .59351 .49675 .57129 .4717 Metetra
.747 .77 .871 r
.57129 .4717 .59351 .49675 .63619 .47151 .61449 .44607 Metetra
.742 .769 .875 r
.61449 .44607 .63619 .47151 .67927 .44687 .6581 .42106 Metetra
.738 .769 .878 r
.6581 .42106 .67927 .44687 .72276 .42277 .70214 .39662 Metetra
.733 .768 .881 r
.70214 .39662 .72276 .42277 .76672 .39919 .74665 .37273 Metetra
.728 .768 .885 r
.74665 .37273 .76672 .39919 .81117 .37609 .79167 .34936 Metetra
.723 .768 .889 r
.79167 .34936 .81117 .37609 .85616 .35348 .83724 .32653 Metetra
.718 .769 .893 r
.83724 .32653 .85616 .35348 .90172 .33134 .88342 .30425 Metetra
.787 .773 .84 r
.24993 .65287 .27651 .67543 .31814 .64282 .29211 .61976 Metetra
.781 .773 .846 r
.29211 .61976 .31814 .64282 .35983 .61166 .33434 .58812 Metetra
.774 .773 .851 r
.33434 .58812 .35983 .61166 .40166 .58175 .37669 .55774 Metetra
.768 .772 .856 r
.37669 .55774 .40166 .58175 .44367 .55292 .41923 .52846 Metetra
.763 .772 .86 r
.41923 .52846 .44367 .55292 .48592 .52504 .462 .50014 Metetra
.757 .771 .865 r
.462 .50014 .48592 .52504 .52844 .498 .50505 .47266 Metetra
.752 .771 .868 r
.50505 .47266 .52844 .498 .57129 .4717 .54842 .44596 Metetra
.747 .77 .872 r
.54842 .44596 .57129 .4717 .61449 .44607 .59216 .41994 Metetra
.742 .77 .876 r
.59216 .41994 .61449 .44607 .6581 .42106 .63631 .39457 Metetra
.737 .769 .879 r
.63631 .39457 .6581 .42106 .70214 .39662 .6809 .3698 Metetra
.732 .769 .883 r
.6809 .3698 .70214 .39662 .74665 .37273 .72598 .34562 Metetra
.726 .769 .887 r
.72598 .34562 .74665 .37273 .79167 .34936 .77159 .32203 Metetra
.721 .769 .891 r
.77159 .32203 .79167 .34936 .83724 .32653 .81778 .29904 Metetra
.715 .77 .896 r
.81778 .29904 .83724 .32653 .88342 .30425 .8646 .27669 Metetra
.787 .773 .84 r
.2226 .62966 .24993 .65287 .29211 .61976 .26533 .59604 Metetra
.78 .773 .846 r
.26533 .59604 .29211 .61976 .33434 .58812 .30811 .5639 Metetra
.774 .773 .851 r
.30811 .5639 .33434 .58812 .37669 .55774 .351 .53304 Metetra
.768 .772 .856 r
.351 .53304 .37669 .55774 .41923 .52846 .39408 .50329 Metetra
.762 .772 .861 r
.39408 .50329 .41923 .52846 .462 .50014 .43738 .47451 Metetra
.757 .771 .865 r
.43738 .47451 .462 .50014 .50505 .47266 .48097 .44661 Metetra
.751 .771 .869 r
.48097 .44661 .50505 .47266 .54842 .44596 .52489 .41949 Metetra
.746 .77 .873 r
.52489 .41949 .54842 .44596 .59216 .41994 .56917 .39309 Metetra
.741 .77 .876 r
.56917 .39309 .59216 .41994 .63631 .39457 .61387 .36737 Metetra
.735 .769 .88 r
.61387 .36737 .63631 .39457 .6809 .3698 .65903 .34231 Metetra
.73 .769 .884 r
.65903 .34231 .6809 .3698 .72598 .34562 .7047 .31788 Metetra
.724 .77 .889 r
.7047 .31788 .72598 .34562 .77159 .32203 .75092 .29413 Metetra
.718 .77 .894 r
.75092 .29413 .77159 .32203 .81778 .29904 .79774 .27108 Metetra
.711 .771 .9 r
.79774 .27108 .81778 .29904 .8646 .27669 .84524 .2488 Metetra
.787 .773 .84 r
.19447 .60576 .2226 .62966 .26533 .59604 .23777 .57161 Metetra
.78 .773 .846 r
.23777 .57161 .26533 .59604 .30811 .5639 .28111 .53896 Metetra
.774 .773 .851 r
.28111 .53896 .30811 .5639 .351 .53304 .32456 .5076 Metetra
.768 .772 .856 r
.32456 .5076 .351 .53304 .39408 .50329 .36819 .47737 Metetra
.762 .772 .861 r
.36819 .47737 .39408 .50329 .43738 .47451 .41204 .44814 Metetra
.756 .771 .865 r
.41204 .44814 .43738 .47451 .48097 .44661 .45618 .4198 Metetra
.751 .771 .869 r
.45618 .4198 .48097 .44661 .52489 .41949 .50065 .39227 Metetra
.745 .77 .873 r
.50065 .39227 .52489 .41949 .56917 .39309 .54549 .36551 Metetra
.74 .77 .877 r
.54549 .36551 .56917 .39309 .61387 .36737 .59076 .33946 Metetra
.734 .77 .882 r
.59076 .33946 .61387 .36737 .65903 .34231 .6365 .31414 Metetra
.728 .77 .886 r
.6365 .31414 .65903 .34231 .7047 .31788 .68277 .28954 Metetra
.721 .771 .892 r
.68277 .28954 .7047 .31788 .75092 .29413 .72962 .26573 Metetra
.714 .772 .898 r
.72962 .26573 .75092 .29413 .79774 .27108 .77711 .24278 Metetra
.705 .774 .905 r
.77711 .24278 .79774 .27108 .84524 .2488 .82533 .22081 Metetra
.787 .773 .84 r
.16551 .58113 .19447 .60576 .23777 .57161 .2094 .54644 Metetra
.78 .773 .846 r
.2094 .54644 .23777 .57161 .28111 .53896 .25332 .51326 Metetra
.774 .772 .851 r
.25332 .51326 .28111 .53896 .32456 .5076 .29734 .48139 Metetra
.768 .772 .856 r
.29734 .48139 .32456 .5076 .36819 .47737 .34152 .45067 Metetra
.762 .772 .861 r
.34152 .45067 .36819 .47737 .41204 .44814 .38594 .42097 Metetra
.756 .771 .865 r
.38594 .42097 .41204 .44814 .45618 .4198 .43064 .3922 Metetra
.75 .771 .87 r
.43064 .3922 .45618 .4198 .50065 .39227 .47567 .36427 Metetra
.744 .77 .874 r
.47567 .36427 .50065 .39227 .54549 .36551 .52108 .33716 Metetra
.738 .77 .879 r
.52108 .33716 .54549 .36551 .59076 .33946 .56693 .31084 Metetra
.732 .77 .884 r
.56693 .31084 .59076 .33946 .6365 .31414 .61327 .28533 Metetra
.725 .771 .889 r
.61327 .28533 .6365 .31414 .68277 .28954 .66016 .26068 Metetra
.717 .772 .896 r
.66016 .26068 .68277 .28954 .72962 .26573 .70767 .23699 Metetra
.708 .774 .904 r
.70767 .23699 .72962 .26573 .77711 .24278 .75587 .2144 Metetra
.696 .777 .914 r
.75587 .2144 .77711 .24278 .82533 .22081 .80487 .19312 Metetra
.787 .773 .84 r
.13569 .55573 .16551 .58113 .2094 .54644 .18018 .52047 Metetra
.78 .772 .846 r
.18018 .52047 .2094 .54644 .25332 .51326 .22469 .48675 Metetra
.774 .772 .851 r
.22469 .48675 .25332 .51326 .29734 .48139 .26929 .45436 Metetra
.767 .772 .856 r
.26929 .45436 .29734 .48139 .34152 .45067 .31405 .42314 Metetra
.761 .771 .861 r
.31405 .42314 .34152 .45067 .38594 .42097 .35904 .39297 Metetra
.755 .771 .866 r
.35904 .39297 .38594 .42097 .43064 .3922 .40431 .36376 Metetra
.749 .771 .87 r
.40431 .36376 .43064 .3922 .47567 .36427 .44992 .33546 Metetra
.743 .77 .875 r
.44992 .33546 .47567 .36427 .52108 .33716 .49591 .30804 Metetra
.736 .771 .881 r
.49591 .30804 .52108 .33716 .56693 .31084 .54236 .28152 Metetra
.729 .771 .886 r
.54236 .28152 .56693 .31084 .61327 .28533 .58931 .25596 Metetra
.721 .772 .894 r
.58931 .25596 .61327 .28533 .66016 .26068 .63685 .23146 Metetra
.711 .775 .902 r
.63685 .23146 .66016 .26068 .70767 .23699 .68504 .20822 Metetra
.698 .778 .913 r
.68504 .20822 .70767 .23699 .75587 .2144 .73401 .18648 Metetra
.68 .783 .928 r
.73401 .18648 .75587 .2144 .80487 .19312 .78388 .16661 Metetra
.787 .772 .84 r
.10497 .5295 .13569 .55573 .18018 .52047 .15008 .49366 Metetra
.78 .772 .846 r
.15008 .49366 .18018 .52047 .22469 .48675 .1952 .45937 Metetra
.773 .772 .851 r
.1952 .45937 .22469 .48675 .26929 .45436 .24039 .42644 Metetra
.767 .771 .856 r
.24039 .42644 .26929 .45436 .31405 .42314 .28574 .39471 Metetra
.761 .771 .861 r
.28574 .39471 .31405 .42314 .35904 .39297 .33131 .36407 Metetra
.754 .771 .866 r
.33131 .36407 .35904 .39297 .40431 .36376 .37716 .33445 Metetra
.748 .77 .871 r
.37716 .33445 .40431 .36376 .44992 .33546 .42336 .30581 Metetra
.741 .77 .877 r
.42336 .30581 .44992 .33546 .49591 .30804 .46995 .27817 Metetra
.733 .771 .883 r
.46995 .27817 .49591 .30804 .54236 .28152 .517 .25159 Metetra
.724 .772 .891 r
.517 .25159 .54236 .28152 .58931 .25596 .56458 .22621 Metetra
.713 .775 .9 r
.56458 .22621 .58931 .25596 .63685 .23146 .61278 .20226 Metetra
.699 .778 .913 r
.61278 .20226 .63685 .23146 .68504 .20822 .66171 .18008 Metetra
.678 .784 .93 r
.66171 .18008 .68504 .20822 .73401 .18648 .71153 .16016 Metetra
.646 .792 .952 r
.71153 .16016 .73401 .18648 .78388 .16661 .76243 .14318 Metetra
.787 .772 .839 r
.07331 .50238 .10497 .5295 .15008 .49366 .11906 .46592 Metetra
.78 .771 .845 r
.11906 .46592 .15008 .49366 .1952 .45937 .1648 .43104 Metetra
.773 .771 .851 r
.1648 .43104 .1952 .45937 .24039 .42644 .2106 .39755 Metetra
.766 .771 .856 r
.2106 .39755 .24039 .42644 .28574 .39471 .25655 .3653 Metetra
.76 .77 .861 r
.25655 .3653 .28574 .39471 .33131 .36407 .30272 .3342 Metetra
.753 .77 .867 r
.30272 .3342 .33131 .36407 .37716 .33445 .34916 .30419 Metetra
.746 .77 .872 r
.34916 .30419 .37716 .33445 .42336 .30581 .39594 .27529 Metetra
.738 .77 .879 r
.39594 .27529 .42336 .30581 .46995 .27817 .44313 .24759 Metetra
.728 .772 .887 r
.44313 .24759 .46995 .27817 .517 .25159 .49079 .22124 Metetra
.716 .774 .898 r
.49079 .22124 .517 .25159 .56458 .22621 .53902 .19654 Metetra
.7 .778 .912 r
.53902 .19654 .56458 .22621 .61278 .20226 .58792 .17398 Metetra
.674 .785 .933 r
.58792 .17398 .61278 .20226 .66171 .18008 .63767 .15426 Metetra
.63 .794 .961 r
.63767 .15426 .66171 .18008 .71153 .16016 .6885 .1384 Metetra
.547 .799 .991 r
.6885 .1384 .71153 .16016 .76243 .14318 .74076 .12773 Metetra
.786 .771 .839 r
.04069 .47427 .07331 .50238 .11906 .46592 .0871 .43716 Metetra
.779 .77 .845 r
.0871 .43716 .11906 .46592 .1648 .43104 .13347 .40165 Metetra
.772 .77 .851 r
.13347 .40165 .1648 .43104 .2106 .39755 .1799 .36757 Metetra
.766 .769 .856 r
.1799 .36757 .2106 .39755 .25655 .3653 .22646 .33478 Metetra
.759 .769 .861 r
.22646 .33478 .25655 .3653 .30272 .3342 .27323 .30323 Metetra
.751 .769 .867 r
.27323 .30323 .30272 .3342 .34916 .30419 .32026 .2729 Metetra
.743 .769 .874 r
.32026 .2729 .34916 .30419 .39594 .27529 .36763 .2439 Metetra
.733 .77 .883 r
.36763 .2439 .39594 .27529 .44313 .24759 .41541 .21647 Metetra
.719 .772 .895 r
.41541 .21647 .44313 .24759 .49079 .22124 .46368 .191 Metetra
.699 .777 .912 r
.46368 .191 .49079 .22124 .53902 .19654 .51255 .16823 Metetra
.664 .785 .939 r
.51255 .16823 .53902 .19654 .58792 .17398 .56222 .1493 Metetra
.594 .793 .976 r
.56222 .1493 .58792 .17398 .63767 .15426 .61295 .13598 Metetra
.425 .774 .996 r
.61295 .13598 .63767 .15426 .6885 .1384 .66521 .13068 Metetra
.03 .585 .821 r
.66521 .13068 .6885 .1384 .74076 .12773 .71968 .13616 Metetra
.786 .769 .838 r
.00707 .44501 .04069 .47427 .0871 .43716 .05416 .40719 Metetra
.779 .769 .844 r
.05416 .40719 .0871 .43716 .13347 .40165 .10119 .371 Metetra
.772 .768 .849 r
.10119 .371 .13347 .40165 .1799 .36757 .14826 .33627 Metetra
.765 .767 .855 r
.14826 .33627 .1799 .36757 .22646 .33478 .19544 .3029 Metetra
.757 .766 .86 r
.19544 .3029 .22646 .33478 .27323 .30323 .24281 .27089 Metetra
.748 .766 .867 r
.24281 .27089 .27323 .30323 .32026 .2729 .29044 .24038 Metetra
.737 .766 .876 r
.29044 .24038 .32026 .2729 .36763 .2439 .33837 .21167 Metetra
.721 .768 .89 r
.33837 .21167 .36763 .2439 .41541 .21647 .3867 .18544 Metetra
.694 .773 .913 r
.3867 .18544 .41541 .21647 .46368 .191 .43551 .16299 Metetra
.635 .781 .952 r
.43551 .16299 .46368 .191 .51255 .16823 .485 .14666 Metetra
.466 .768 .996 r
.485 .14666 .51255 .16823 .56222 .1493 .53553 .14047 Metetra
0 .505 .774 r
.53553 .14047 .56222 .1493 .61295 .13598 .5878 .1503 Metetra
.566 .012 0 r
.5878 .1503 .61295 .13598 .66521 .13068 .64299 .18224 Metetra
.663 .215 .177 r
.64299 .18224 .66521 .13068 .71968 .13616 .70262 .2391 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.13514 .21677 m
.13978 .22141 L
s
[(80)] .12586 .2075 1 .99877 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(100)] .3345 .12542 .86223 1 Mshowa
.56863 .04703 m
.57247 .05235 L
s
[(120)] .56094 .0364 .72324 1 Mshowa
.125 Mabswid
.18558 .19702 m
.18832 .19985 L
s
.23703 .17687 m
.23971 .17975 L
s
.28951 .15632 m
.29214 .15925 L
s
.39772 .11395 m
.40023 .11699 L
s
.4535 .09211 m
.45595 .0952 L
s
.51046 .06981 m
.51284 .07294 L
s
.08568 .23614 m
.08851 .23887 L
s
.62804 .02377 m
.63027 .02701 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{255.875, 207.5},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004o00010a000`40O003h00Oogoo@7oo003oOom0
Ool00:]oo`D0009oo`D000Aoo`<001ioo`80065oo`00[Goo00<007ooOol00goo00@007ooOol000=o
o`05001oogooOol0000JOol300000goo001oo`1POol00:eoo`03001oogoo00Aoo`03001oogoo00=o
o`05001oogooOol0000HOol20004Ool2001POol00:eoo`03001oogoo00Eoo`03001oogoo009oo`05
001oogooOol0000EOol30006Ool3001OOol00:eoo`03001oogoo00Ioo`05001oogooOol00003Ool0
0`00Oomoo`0AOol20009Ool00`00Ool0001OOol00:eoo`03001oogoo00Ioo`05001oogooOol00003
Ool00`00Oomoo`0>Ool3000;Ool01000Oomoo`00GWoo002]Ool00`00Oomoo`02Ool01@00Oomoogoo
00000goo00D007ooOomoo`0000eoo`<000ioo`05001oogooOol0001MOol00:]oo`<000Eoo`<000Eo
o`<000aoo`80009oo`03001oogoo00aoo`05001oogooOol0001MOol00<Moo`<001=oo`03001oogoo
009oo`03001oogoo05Yoo`00aGoo0P005goo00@007ooOomoo`8000Qoo`D004ioo`00`Woo0`006Goo
00<007ooOol00Woo00<007ooOol027oo00<007ooOol0CWoo0030Ool2000LOol00`00Oomoo`03Ool0
0`00Oomoo`07Ool00`00Oomoo`1>Ool00;eoo`<001ioo`03001oogoo00Aoo`03001oogoo00Ioo`03
001oogoo04ioo`00^goo0P000goo00<007ooOol06goo00<007ooOol017oo00<007ooOol01Woo00<0
07ooOol0CWoo002hOol3000SOol00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomoo`1>Ool00;Eo
o`<002Ioo`03001oogoo00Aoo`<000Ioo`03001oogoo04ioo`00/goo0P00:Goo00<007ooOol01Woo
00<007ooOol00Woo0`00D7oo002`Ool3000[Ool00`00Oomoo`07Ool00`00Oomoo`1DOol00:ioo`<0
02eoo`03001oogoo00Moo`03001oogoo05Aoo`00Zgoo0`000goo00<007ooOol0:Woo00<007ooOol0
27oo00<007ooOol0Dgoo002YOol2000cOol00`00Oomoo`09Ool2001COol00:Ioo`<003Eoo`03001o
ogoo00Uoo`03001oogoo059oo`00Y7oo0P00>7oo00<007ooOol02Woo00<007ooOol0DGoo001_Ool5
0003Ool30005Ool3000OOol3000jOol00`00Oomoo`0;Ool00`00Oomoo`1@Ool0075oo`03001oogoo
009oo`05001oogooOol00003Ool01@00Oomoogoo00006goo1@00>goo00<007ooOol02goo00<007oo
Ool0D7oo001aOol00`00Oomoo`02Ool01@00Oomoogoo00000goo00D007ooOomoo`0001Uoo`80045o
o`03001oogoo00]oo`03001oogoo04moo`00LGoo00<007ooOol00Woo00D007ooOomoo`0000=oo`05
001oogooOol0000FOol30013Ool00`00Oomoo`0;Ool2001@Ool0075oo`03001oogoo009oo`05001o
ogooOol00003Ool01@00Oomoogoo000057oo0P00AWoo00<007ooOol037oo00<007ooOol0CWoo001a
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo00D007ooOomoo`00015oo`<004Qoo`03001oogoo
00eoo`03001oogoo04eoo`00LGoo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol0
000?Ool2001;Ool00`00Oomoo`0>Ool00`00Oomoo`1<Ool006moo`<000Eoo`<000Eoo`<000eoo`<0
009oo`03001oogoo04Qoo`03001oogoo00ioo`03001oogoo04aoo`00SGoo0P00D7oo00<007ooOol0
3goo0P00C7oo002:Ool3001BOol00`00Oomoo`0?Ool2001<Ool008Moo`<004Yoo`P000=oo`03001o
ogoo00Ioo`D000Eoo`03001oogoo04Yoo`00QGoo0P00AWoo1`000dP000<005C5E<D00UC53P000dLo
00<007ooOol00goo00<007ooOol0BGoo0022Ool40012Ool50009B0000`00E<EDa@06E<D01000E<ED
aEC510001dLo00<007ooOol00goo00<007ooOol0B7oo0020Ool20004Ool00`00Oomoo`0ZOol80008
Ool6000=B0000`00E<EDa@06E<D00`00E<EDa@03E<D00`00Acm7?`07Acl00`00Oomoo`02Ool00`00
Oomoo`06Ool5000mOol007eoo`<002aoo`L000Hk7`X00082608000m80003001DaEC500IDa@03001D
aEC500=Da@030017?dLo00I7?`<000=oo`8000Qoo`04001oogoo000mOol007]oo`8002]oo`@0010k
7`<000=??`@000m80003001DaEC500MDa@03001DaEC500=Da@030017?dLo00=7?`<000=C?P03001o
ogoo009oo`03001oogoo00Ioo`03001oogoo03eoo`00N7oo0`00:Woo0`000e<N00<003/O>al02c/O
0`000eLn00<004loCcl014lo00<004P0B00034P000<005C5E<D01eC500<005C5E<D00UC500@004Lo
Acm7?`<000MC?P05001oogooOol00009Ool00`00Oomoo`0lOol007Ioo`<002Uoo`<000MC7P8000Pk
7`<000MG?P03001??dlo00A??`8000a80003001DaEC500QDa@04001DaEC5E<D4000;Dch01@00Oomo
ogoo00002Goo00<007ooOol0>goo001cOol30002Ool00`00Oomoo`0TOol3000<Dah00`00>alk7`03
>al2000;Ech00`00Ccm??`05Ccl00`00B0180008B0000`00E<EDa@09E<D01@00E<EDaEC500003e<n
00D007ooOomoo`0000Qoo`03001oogoo03]oo`00L7oo0`00:Goo0`0045<N00<003/O00000P003ULn
00<004loCcl00dlo100024P000<005C5E<D02EC500D005C5E<EDa@00011C?P03001oogoo008000=o
o`05001oogooOol0000mOol006ioo`8002Uoo`<0009K7@03001C7U<N00iC7P80019G?P03001??`00
008000AG?@8000I80003001DaEC500UDa@05001DaEC5E<D0000@Dch300000goo001oo`03Ool3000n
Ool0041oo`<000Eoo`<0021oo`<002Qoo`<000IK7@03001C7U<N00]C7P80009K7@03001G?ULn00mG
?P8000UG?@03001804P000980003001DaEC500YDa@04001DaEC5E<D2000<Dch300001ELm001oogoo
0000AGoo000oOol01@00Oomoogoo00000goo00D007ooOomoo`0001eoo`8002Uoo`8000YK7@03001C
7U<N00QC7P8000EK7@03001G?ULn00aG?P800003Fad005Lm00UG?@8000980003001DaEC500YDa@07
001DaEC5E<D005Lm000025<n0`001ELm00@007ooOol004Aoo`00?goo00D007ooOomoo`0000=oo`05
001oogooOol0000JOol30002Ool00`00Oomoo`0SOol3000=Fad00`00DaiC7P05Dah20008Fad00`00
EciG?P09Ech20004Fad00`00EceG?@09Ecd00`00B000000<E<D01000E<EDa@000eLm00<005<nDch0
0e<n0P002ELm00@007ooOol004=oo`00?goo00D007ooOomoo`0000=oo`05001oogooOol0000HOol2
000YOol2000AFad00`00DaiC7P02Dah2000;Fad00`00EciG?P05Ech30007Fad00`00EceG?@09Ecd2
000<E<D01000E<EDa@0015Lm00<005<n00000P0035Lm00@007oo0000049oo`00@7oo0`0017oo00D0
07ooOomoo`0001Eoo`<002Uoo`80009O7003001K7E/M00mK7@04001C7P00000>Fad00`00EciG?P02
Ech2000;Fad00`00EceG?@07Ecd200000e/M001Da@0:E<D01000E<EDa@0015Lm0P0045Lm00<007oo
0000@Woo000oOol01@00Oomoogoo00000goo00D007ooOomoo`0001=oo`8002]oo`03001O75lL00=O
7003001K7E/M00iK7@80015K7@04001G?P00000=Fad00`00EceG?@05Ecd20004Fad00`00E<EDa@08
E<D01@00E<EDa@00Ecd00`0000=K7@00Ecd03eLm0P0000=oo`00Ool0@7oo000oOol01@00Oomoogoo
00000goo00D007ooOomoo`00011oo`<002]oo`8000MO7003001K7E/M00]K7@800003Ga`005/M011K
7@80011K7@03001G?ELm009G?@8000MK7@03001DaEC500MDa@03001DaEC5008000EK7@03001G?ELm
00]G?@800004Fad007oo0011Ool0041oo`<000Eoo`<000ioo`<0009oo`03001oogoo02Moo`8000YO
7003001K7E/M00QK7@8000AO7003001K7E/M00aK7@800003Fa`005/M011K7@04001G?@00000:Fad0
0`00E<EDa@06E<D01000E<EDa@001e/M00<005LmEcd025Lm0P0015/M00<007oo0000@7oo001GOol2
000^Ool00`00GaaO700;Ga`00`00FaeK7@05Fad20007Ga`00`00FaeK7@0:Fad00`00FaaK7002Fa`0
0`00FaeK7@0>Fad2000=Fad00`00E<EDa@05E<D00`00E<D00009Fad00`00EceG?@05Ecd20007Fad3
000oOol005Aoo`<002ioo`8000mO7003001K7E/M009K7@8000YO7003001K7E/M00MK7@8000IK7003
001K7E/M00]K7@800003Fa`005/M00eK7@03001DaEC500ADa@03001Da@0000YK7@03001G?ELm009G
?@8000YK7@8003moo`00DWoo0P00;goo0P004UlL00@005/M000000eO7003001K7E/M00AK7@8000UK
7003001K7E/M00QK7@8000AK7003001K7E/M00]K7@8000EDa@03001Da@0000]K7@04001G?@00000<
Fad00`00Ool0000nOol004moo`<002moo`800003Ga`005lL015O7080011O7003001K7E/M009K7@03
001K75/L00YK7003001K7E/M00EK7@8000MK7003001K7E/M00QK7@80009K7003001DaEC5009Da@03
001Da@0000]K7@8000mK7@03001oo`0003eoo`00CGoo0P0000=oo`00Ool0;Woo00<005lLGa`00UlL
00<005lLGa`03ElL0P0000=O7000Ga`045lL00@005/M000000iK7003001K7E/M009K7@8000YK7003
001K7E/M00EK7@8000EK7003001DaEC5009Da@8000UK7@800003Fa`005/M00iK7@<003eoo`00BWoo
0`0017oo00<007ooOol0:Woo0P001UlL00<005lLGa`02UlL0P0015lL00<005lLGa`03UlL0P004E/L
00@005/M000000eK7003001K7E/M009K7@8000QK7004001DaEC5E<D20007Fad20004Fa`00`00FaeK
7@0:Fad200000e/L001oo`02000kOol004Qoo`80039oo`8000UO7003001O75lL00MO708000MO7003
001O75lL00]O7080009O7003001K75/L00iK7080011K7004001K7@00000;Fa`00`00E<EDa@020004
Fad30007Fa`00`00FaeK7@07Fad20004Fa`00`00Ool0000kOol004Eoo`<003=oo`03001O75lL00YO
7003001O75lL00EO7003001O75lL00QO7003001O75lL00QO708000EO7003001K75/L00]K70800003
Ga`005/L00mK708000iK7003001Da@0000=K7@8000]K7003001K7E/M00AK7@8000MK708003]oo`00
@Woo0`00=7oo0P003UlL00<005lLGa`00UlL0P0035lL00<005lLGa`01ElL0P0025lL00<005/LFa`0
25/L0P0015lL00<005/LFa`02e/L0P0000=O7000Fa`03U/L0P0000=K7@0000003E/L00<005/MFad0
0U/M0P002E/L00<007oo0000>Woo0010Ool20002Ool00`00Oomoo`0`Ool2000AGa`01000Ga`00000
3elL00<005lLGa`00UlL0P002elL00<005/LFa`01E/L0P001elL00<005/LFa`025/L0P0015lL00<0
05/LFa`03E/L0P0045/L00@005/M000000aK7003001oo`0000Qoo`<002ioo`00?Goo0`00=Woo00<0
05lK00004ElL0P004UlL00@005lL000000iO7003001K75/L009K708000YO7003001K75/L00EK7080
00MO7003001K75/L00YK70@0011K708000iK7004001oo`000006Ool01@00Oomoogoo0000;Goo000k
Ool2000gOol20003Ga/00`00GaaO700=Ga`01000Ga]O6`004ElL0P004ElL00@005/L000000eO7003
001K75/L009K708000YO7003001K75/L00MK70800005Fa`00000Fa`0000=Fa`200000e/L001K700>
Fa`00`00Ool0000:Ool00`00Oomoo`0[Ool003Qoo`<003Qoo`03001O6elK00AO6`03001O75lL00YO
708000AO6`03001O75lL00aO70800003Ga`005lL011O7080011O7004001K7000000<Ga`00`00FaaK
7005Fa`500001E/L001K75/L00002U/L0P0015/L00<005/LFa`035/L0P0000=oo`00Ool027oo00<0
07ooOol0:goo000gOol2000hOol20008Ga/00`00GaaO7008Ga`00`00Ga]O6`05Ga/00`00GaaO700:
Ga`00`00GaaO7002Ga`00`00GaaO700<Ga`200000elL001O700?Ga`2000?Ga`00`00FaaK7002Fa`4
0005Fa`01000FaaK70001e/L0P001e/L00<005/LFa`025/L0`0000AK7000Ool000Moo`8002ioo`00
=goo00<007oo0000=Woo00<005lKGa/02ElK00<005lLGa`01ElL0P002ElK00<005lLGa`01elL0P00
1UlL00<005lLGa`02UlL00<005lLGa`00UlL00<005lLGa`02elL0P0000=O7000Ga`03elL00<005/L
00000P002E/L00@005/LFa`000AK708000YK7003001K75/L00EK708000EK7003001oo`0000Qoo`03
001oogoo02]oo`00=goo00@007ooOol003=oo`8000aO6`03001O75lL00AO7003001O6elK00YO6`03
001O75lL00AO708000UO7003001O75lL00MO708000IO7003001O75lL00UO7003001O75lL009O7003
001O75lL00aO70<000eK7005001K75/L001K7002000<Fa`00`00FaaK7003Fa`20008Fa`30003Ool0
1@00Oomoogoo0000;Goo000gOol20002Ool00`00Oomoo`0_Ool00`00Ga]O6`0=Ga/01000GaaO75lL
0P003UlK00<005lLGa`00UlL00<005lLGa`02UlL00<005lLGa`015lL0P002ElL00<005lLGa`01UlL
0P001UlL00<005lLGa`025lL100000=O7000Fa`035/L00<005/LFa`00P003e/L00<005/L00000P00
2e/L0P0017oo0`00;Woo000fOol20004Ool00`00Oomoo`0/Ool2000AGa/00`00Ga`0000AGa/01000
Ga`000003UlL00<005lLGa`00UlL00<005lLGa`02ElL00<005lLGa`015lL0P002ElL00<005lLGa`0
15lL1@0015lL00<005/LFa`02e/L0P0000=O7000Fa`03U/L0P003U/L00<007oo0000=7oo000fOol0
0`00Oomoo`04Ool00`00Oomoo`0ZOol00`00Ha/0000AGa/2000BGa/2000AGa`01000Ga`000003ElL
00<005lLGa`00UlL00<005lLGa`02ElL00@005lLGaaO70<00003Ga`0000000MO7003001K75/L00QK
70800005Ga`005lLGa`0000<Fa`200000e/L001K700>Fa`2000dOol003Ioo`03001oogoo00Eoo`03
001oogoo02Moo`8000=S6`03001O6elK00aO6`80009S6`03001O6elK00iO6`03001O6`00015O7080
011O7004001O7000000=Ga`30002Ga`2000:Ga`00`00FaaK7005Fa`20003Ga`01@00GaaO75lL0000
2E/L0P0015/L00<005/LFa`035/L1000<Woo000fOol20007Ool00`00Oomoo`0UOol00`00Ha]S6`04
Ha/00`00Ga]O6`0:Ga/00`00Ha]S6`03Ha/00`00Ga]O6`0;Ga/20003Ga/00`00GaaO700<Ga`20000
0elL001O700?Ga`2000=Ga`200000elL001O7002000=Ga`00`00FaaK7002Fa`20006Ga`01@00GaaO
75lL00001U/L0P001e/L00<005/LFa`025/L0`0000AK70000000039oo`00=Woo00<007ooOol01goo
00<007ooOol08Woo0P0026<K00<005lKGa/01elK0P001f<K00<005lKGa/02ElK00<005lKGa/015lK
00<005lLGa`02UlL00<005lLGa`00UlL00<005lLGa`02elL0P0000=O7000Ga`02ElL0`000elL0P00
45lL00@005/L000000UO7004001O75lL0004Fa`2000:Fa`00`00FaaK7005Fa`20005Fa`2000bOol0
03Ioo`03001oogoo00Qoo`03001oogoo021oo`03001S6f<K00US6`03001O6elK00AO6`8000YS6`03
001O6elK00IO6`8000QO6`03001O75lL00MO708000IO7003001O75lL00UO7003001O75lL009O7003
001O75lL00AO70<000AO70800003Ga`005lL00mO708000aO7005001O75lL001K7002000<Fa`00`00
FaaK7003Fa`20008Fa`2000aOol003Ioo`03001oogoo00Uoo`03001oogoo01eoo`8000eS6`03001O
6elK009O6`03001S6f<K00]S6`03001O6elK00AO6`03001O6elK00UO6`03001O75lL00AO708000UO
7003001O75lL00IO708000IO7003001O75lL00<000EO708000AO7003001O75lL00]O70800003Ga`0
05lL00]O7003001O75lL008000mK7003001K7000008000]K7080031oo`00=Goo0P0037oo00<007oo
Ool06goo00<006<KHa/03V<K00@005lK000000mS6`04001O6elKGa/2000=Ga/00`00GaaO7002Ga`0
0`00GaaO7009Ga`00`00GaaO7004Ga`20008Ga`30006Ga`20007Ga`00`00GaaO7008Ga`20004Ga`0
0`00GaaO700:Ga`200000elL001K700>Fa`2000>Fa`3000_Ool003Eoo`03001oogoo00aoo`03001o
ogoo01Qoo`80015S6`80019S6`03001O6`00011O6`04001O7000000=Ga`00`00GaaO7002Ga`00`00
GaaO7005Ga`30002Ga`00`00GaaO7002Ga`2000:Ga`00`00GaaO7005Ga`20007Ga`00`00GaaO7007
Ga`200001ElL001O75lL000035/L0P0000=O7000Fa`03U/L0P00;goo000eOol00`00Oomoo`0=Ool0
0`00Oomoo`0FOol00`00Ha/0000@Ha/01000Ha]S6`004F<K0P004ElK0P0045lL00@005lL000000EO
70<000IO7004001O7000000=Ga`00`00GaaO7002Ga`2000:Ga`00`00GaaO7004Ga`20004Ga`01000
GaaO70002E/L0P0015lL00<005/LFa`035/L0`00;Woo000eOol00`00Oomoo`0>Ool00`00Oomoo`0C
Ool20003Ha/00`00Ha]S6`0<Ha/00`00Ha]S6`02Ha/00`00Ha]S6`0<Ha/20002Ha/00`00Ga]O6`0<
Ga/200000elK001O700?Ga`20004Ga`30009Ga`2000@Ga`01000Ga`0000035lL00<005lLGa`00UlL
0P001UlL00D005lLGaaO700000MK7003001O75lL00EO7003001K75/L00QK70<00003Fa`0000002io
o`00=Goo0P0047oo0P004Woo00<006<KHa/016<K00<006<KHa/02F<K0P001V<K00<006<KHa/02V<K
00<006<KHa/00f<K00<005lKGa/02UlK00<005lKGa/00UlK00<005lLGa`02elL0P000UlK10002elL
00<005lL000045lL0P003elL00@005lL000000UO7005001O75lLGa`00004Fa`20009Ga`00`00FaaK
7005Fa`20005Fa`3000/Ool003Eoo`03001oogoo015oo`03001oogoo00eoo`8000QS6`03001S6f<K
00MS6`03001S6f<K00MS6`03001S6f<K00MS6`8000MS6`03001O6elK00MO6`8000IO6`03001O75lL
00UO7004001O6elKGa/3000;Ga`20003Ga`00`00GaaO700;Ga`200000elL001O700>Ga`2000<Ga`0
1P00GaaO75lL001K708000]O7003001K75/L00=K708000QK708002aoo`0077oo0`001Woo0P0017oo
1@001Goo00<007ooOol04Woo00<007ooOol02goo00<006<KHa/02F<K00<006<KHa/01F<K00<006<K
Ha/02F<K00<006<KHa/016<K0P002V<K00<005lKGa/015lK0P002ElK00<005lLGa`01UlL0P0000AO
6`00000000=O6`03001O75lL00IO708000IO7003001O75lL00UO7003001O75lL009O7003001O75lL
00YO70800003Ga`005lL00aO7003001O75lL008000iO7003001K7000008000]K7003001oogoo02Yo
o`006goo00D007ooOomoo`0000Eoo`8000Eoo`04001oogoo0004Ool00`00Oomoo`0DOol00`00Oomo
o`09Ool00`00Ha]S6`0:Ha/00`00Ha]S6`03Ha/2000=Ha/00`00Ha]S6`02Ha/00`00Ha]S6`0;Ha/0
0`00Ga]O6`02Ga/00`00Ga]O6`09Ga/00`00GaaO7004Ga`50007Ga/00`00GaaO7004Ga`00`00GaaO
7007Ga`00`00GaaO7006Ga`20006Ga`00`00GaaO7007Ga`20004Ga`00`00GaaO700:Ga`300000elL
001O700=Ga`2000>Fa`2000[Ool001]oo`05001oogooOol0000=Ool00`00Oomoo`04Ool00`00Oomo
o`0EOol00`00Oomoo`06Ool2000>Ha/01@00Ha]S6f<K000046<K00@006<K000000mS6`04001O6`00
000=Ga/00`00GaaO7002Ga`3000;Ga/01000GaaO75lL0P002UlL00<005lLGa`015lL0P002ElL00<0
05lLGa`01ElL00<005lLGa`01ElL00<005lLGa`025lL0`000elL00<005lLGa`02ElL0P0000=O7000
Fa`03U/L0P002Goo0`007Woo000KOol01@00Oomoogoo00003Woo00<007ooOol00goo0P005goo00<0
07ooOol017oo00<006<KHa/03f<K00<006<K00004F<K0P004F<K0P0045lK10003elK00<005lL0000
3ElL00<005lLGa`00UlL00<005lLGa`02UlL00<005lLGa`00UlL0P002ElL00<005lLGa`01ElL0P00
0elL00D005lLGaaO700000QO708000AO7003001K75/L00aK70<000Uoo`03001oogoo01eoo`006goo
00D007ooOomoo`0000moo`03001oogoo009oo`03001oogoo01Moo`04001oogooOol3000AHa/3000@
Ha/00`00Ha/0000@Ha/00`00Ha/0000>Ga/4000AGa/2000?Ga`01000Ga`000003UlL00@005lL0000
00aO7003001O75lL009O708000EO7003001O75lL009O7003001O75lL00AO7003001O75lL00EO7003
001K75/L00QK70<00003Ga`0000000Ioo`D001ioo`006goo00D007ooOomoo`0000moo`03001oogoo
009oo`03001oogoo01Qoo`03001oo`0000=S6`03001S6f<K00eS6`05001S6f<KHa/0000=Ha/20003
Ha/00`00Ha]S6`0;Ha/20003Ha/00`00Ga]O6`08Ga/300001ElK001O6elK00003UlK0P0000=O6`00
Ga`03UlL0P0045lL0P003elL00@005lL000000QO7005001O75lLGa`00004Ga`20009Ga`00`00FaaK
7005Fa`20005Ga`20005Ool01000Oomoo`007goo000KOol01@00Oomoogoo00002goo00D007ooOomo
o`0000Aoo`03001oogoo01Uoo`03001S6f<K00=S6`03001S6f<K00]S6`03001S6f<K00=S6`03001S
6f<K00US6`03001S6f<K00AS6`03001S6f<K00US6`03001S6f<K00AS6`03001O6elK00AO6`<0009O
6`8000AO6`03001O6elK00YO6`03001O6elK009O6`03001O75lL00YO70800003Ga`005lL00eO7080
0003Ga`005lL00iO708000]O7006001O75lLGa`005lL0P002elL00<005/LFa`00e/L0P0025lL00<0
07ooOol017oo00<007oo00007goo000LOol3000=Ool30004Ool2000IOol20007Ha/00`00Ha]S6`08
Ha/20007Ha/00`00Ha]S6`06Ha/20008Ha/00`00Ha]S6`06Ha/20008Ha/00`00Ga]O6`030003Ga/2
0007Ga/00`00Ga]O6`07Ga/20006Ga/00`00GaaO7007Ga`20004Ga`00`00GaaO700:Ga`00`00GaaO
7002Ga`00`00GaaO700:Ga`200000elL001O700;Ga`00`00GaaO7002000>Ga`00`00Fa`00002000:
Ga`30004Ool00`00Ool0000OOol003=oo`03001oogoo01Moo`03001S6f<K00QS6`03001S6f<K00IS
6`03001S6f<K00QS6`03001S6f<K00AS6`03001S6f<K00US6`03001S6f<K00AS6`03001S6f<K00US
6`8000EO6`03001O6elK00QO6`03001O6elK00AO6`8000UO6`03001O75lL00EO7003001O75lL00EO
7003001O75lL00MO708000IO7003001O75lL00MO708000AO7003001O75lL00UO70<00003Ga`005lL
00eO708000iO708000Eoo`8001moo`00<goo00<007ooOol05Woo00<006<KHa/02V<K00<006<KHa/0
0f<K0P0036<K00@006<KHa]S6`8000eS6`04001S6f<KHa/20009Ha/300000f<K001O6`02Ga/2000<
Ga/00`00Ga]O6`02Ga/00`00Ga]O6`0:Ga/00`00GaaO7002Ga`20009Ga`00`00GaaO7004Ga`20009
Ga`00`00GaaO7004Ga`20007Ga`00`00GaaO7007Ga`30003Ga`00`00GaaO7009Ga`200000elL001O
700=Ga`20006Ool00`00Oomoo`0MOol003=oo`03001oogoo01Aoo`8000iS6`05001S6f<KHa/0000?
Ha/00`00Ha/0000@Ha/00`00Ha/00008Ha/30005Ha/00`00Ga/0000?Ga/01000Ga/000003UlK00@0
05lL000000aO7003001O75lL009O7003001O75lL00UO7003001O75lL009O708000YO7003001O75lL
00AO708000=O7005001O75lLGa`00008Ga`20004Ga`00`00GaaO7009Ga`200000elL0000000UOol0
03=oo`8001Aoo`03001S6f<K00mS6`03001S6`00011S6`80015S6`8000IS6`<000QS6`80011O6`80
011O6`8000mO7004001O7000000=Ga`01000Ga`0000035lL00<005lLGa`00UlL0P001UlL00D005lL
GaaO700000IO7003001O75lL00EO7003001O75lL00IO708000AO708002Aoo`00<goo00<007ooOol0
4Woo00<006<KHa/046<K0P0046<K00@006<KHa/000mS6`04001S6f<K0002Ha/30009Ha/20002Ha/0
0`00Ga]O6`0<Ga/00`00Ga/0000>Ga/200000elK001O700>Ga`2000?Ga`2000?Ga`01000Ga`00000
25lL00D005lLGaaO700000AO708000UO7003001O75lL00AO7003001O75lL00EO7003001oogoo029o
o`00<goo00<007ooOol047oo0`004F<K00<006<K00003f<K00<006<KHa/00V<K00<006<KHa/02f<K
00@006<KHa]S6`<000]S6`03001S6f<K00=S6`03001O6elK00UO6`8000=O6`03001O6elK00YO6`03
001O6elK009O6`03001O75lL00YO70800003Ga`005lL00eO7003001O700000mO708000]O7006001O
75lLGa`005lL0P002elL00<005lLGa`00UlL0P002ElL00<007ooOol08Goo000bOol00`00Oomoo`0@
Ool01@00Ha]S6f<K00003f<K00D006<KHa]S6`0000aS6`8000IS6`03001S6f<K00QS6`800004Ha/0
00000002Ha/00`00Ha]S6`07Ha/20007Ha/00`00Ga]O6`06Ga/20006Ga/00`00Ga]O6`07Ga/20006
Ga/00`00GaaO7007Ga`20004Ga`00`00GaaO7009Ga`20003Ga`00`00GaaO700:Ga`200000elL001O
700;Ga`00`00GaaO7002000>Ga`01000Ga`000002elL0P008Woo000bOol3000?Ool00`00Ha]S6`03
Ha/00`00Ha]S6`0:Ha/20005Ha/00`00Ha]S6`08Ha/00`00Ha]S6`07Ha/00`00Ha]S6`06Ha/40006
Ha/00`00Ha]S6`04Ha/2000:Ha/00`00Ga]O6`04Ga/00`00Ga]O6`07Ga/00`00Ga]O6`04Ga/20009
Ga/00`00GaaO7005Ga`00`00GaaO7005Ga`00`00GaaO7006Ga`20006Ga`00`00GaaO7007Ga`20004
Ga`00`00GaaO7009Ga`300000elL001O700=Ga`2000>Ga`00`00Oomoo`0POol0039oo`03001oogoo
00eoo`8000MS6`03001S6f<K00QS6`03001S6f<K00IS6`03001S6f<K00IS6`03001S6f<K00US6`03
001S6f<K00=S6`<000YS6`03001S6f<K009S6`03001S6f<K00]S6`04001O6elKGa/2000;Ga/00`00
Ga]O6`02Ga/00`00Ga]O6`0:Ga/00`00GaaO7002Ga`20009Ga`00`00GaaO7004Ga`00`00GaaO7007
Ga`00`00GaaO7005Ga`00`00GaaO7005Ga`00`00GaaO7007Ga`30003Ga`00`00GaaO7009Ga`20000
0elL001O700=Ga`2000QOol0019oo`<000Ioo`8000Aoo`D000=oo`D000Aoo`03001oogoo00aoo`03
001S6f<K00QS6`03001S6f<K00IS6`03001S6f<K00QS6`03001S6f<K00=S6`8000eS6`03001S6f<K
00<000eS6`04001S6`00000?Ha/00`00Ga/0000>Ga/01000Ga/000003UlK00@005lL000000aO7004
001O75lLGa`2000;Ga`00`00GaaO7002Ga`20009Ga`00`00GaaO7004Ga`20003Ga`01000GaaO7000
2ElL0P0015lL00<005lLGa`02ElL0P0000=O7000Ool087oo000AOol01@00Oomoogoo00001Goo0P00
1Goo00@007ooOol000Aoo`04001oogoo0004Ool00`00Oomoo`0;Ool00`00Ha]S6`0:Ha/00`00Ha]S
6`03Ha/2000<Ha/01@00Ha]S6f<K00003f<K10003f<K0P004F<K0P003elK0P0045lK0P003elL00<0
05lL00003UlL00@005lL000000]O7003001O75lL009O708000IO7004001O75lL0006Ga`20006Ga`0
0`00GaaO7007Ga`20004Ga`00`00Oomoo`0NOol0015oo`05001oogooOol0000=Ool00`00Oomoo`05
Ool00`00Oomoo`04Ool00`00Oomoo`09Ool2000>Ha/01@00Ha]S6f<K00003f<K00<006<K00003F<K
0`0000=S6`0000003f<K00<006<K000046<K00<006<K00003UlK00@006<KHa/000iO6`03001O6`00
00mO708000mO708000iO7004001O70000008Ga`01@00GaaO75lL00000elL0P002ElL00<005lLGa`0
15lL0P001UlL0P007goo000AOol01@00Oomoogoo00003Woo00<007ooOol01Goo00<007ooOol00Woo
0P002Woo00<006<KHa/03f<K00<006<K000046<K0`002V<K0`000f<K00@006<KHa/000eS6`05001S
6f<KHa/0000=Ha/20003Ha/00`00Ga]O6`09Ga/20004Ha/00`00Ga]O6`09Ga/20003Ga/00`00GaaO
700:Ga`200000elL001O700=Ga`00`00Ga`0000>Ga`2000;Ga`01@00GaaO7000Ga`00P0035lL00@0
05lLGaaO708000UO708001ioo`004Goo00D007ooOomoo`0000moo`03001oogoo00Eoo`05001oogoo
Ool0000:Ool00`00Ha]S6`0@Ha/3000?Ha/01@00Ha]S6f<K00001V<K0`0016<K0P0016<K00<006<K
Ha/026<K0P001F<K00<006<KHa/02F<K00<006<KHa/016<K00<005lKGa/01UlK0P001f<K00<005lK
Ga/01UlK0P001UlK00<005lLGa`01elL0P0015lL00<005lLGa`02ElL0P000elL00<005lLGa`02ElL
0P0000=O7000Ga`02elL00@005lL000000iO7004001O7000000<Ga`00`00Oomoo`0LOol0015oo`05
001oogooOol0000?Ool00`00Oomoo`05Ool01@00Oomoogoo000027oo0`004F<K00D006<KHa]S6`00
00eS6`03001S6f<K00=S6`03001S6f<K00<000IS6`03001S6f<K00ES6`03001S6f<K00IS6`03001S
6f<K00IS6`03001S6f<K00IS6`8000QS6`03001O6elK00AO6`03001S6f<K00QS6`03001O6elK00AO
6`03001O6elK00MO6`03001O75lL00EO7003001O75lL00EO7003001O75lL00IO708000IO7003001O
75lL00IO708000AO7003001O75lL00UO70800003Ga`005lL00eO708000iO708000Qoo`@0015oo`00
4Goo00D007ooOomoo`0000]oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00<007ooOol0
1Goo00D006LJIaYW6P0000mS6`03001S6f<K00=S6`03001S6f<K00QS6`8000IS6`<000MS6`8000US
6`03001S6f<K00=S6`8000YS6`03001S6f<K00AS6`03001S6f<K00US6`04001O6elKGa/2000<Ha/0
1000Ga]O6elK0P002elK00<005lLGa`00UlL0P002ElL00<005lLGa`015lL00<005lLGa`01elL00<0
05lLGa`015lL00<005lLGa`01ElL00<005lLGa`01UlL0P0000EO7000GaaO700000]O70800003Ga`0
05lL00eO708000aoo`03001oogoo00ioo`004Woo0`003Goo0`001Goo0`0017oo0P001Goo00<006LJ
IaX00fLJ00<006<KHa/02V<K0P001f<K00<006<KHa/01V<K00<006<KHa/00f<K0`000V<K00<006<K
Ha/016<K00<006<KHa/02V<K00D006<KHa]S6`0000eS6`04001S6f<KHa/2000=Ha/00`00Ga/0000?
Ha/00`00Ga/0000>Ga/01000Ga`0000035lL00@005lLGaaO708000YO7003001O75lL009O708000UO
7003001O75lL00AO7003001O75lL009O7004001O75lL0007Ga`30004Ga`00`00GaaO7009Ga`20000
0elL0000000;Ool00`00Oomoo`0>Ool0035oo`03001oogoo00=oo`03001W6VLJ00EW6P03001S6f<K
00QS6`03001S6f<K00QS6`03001S6f<K00=S6`8000AS6`8000IS6`04001S6f<KHa/2000>Ha/00`00
Ha/0000?Ha/00`00Ha/0000?Ha/2000@Ha/2000?Ga/2000?Ga`00`00Ga`0000=Ga`01000Ga`00000
2elL00<005lLGa`00UlL0P001UlL00@005lLGa`000AO708000QO7003001O75lL00EO70<000AO7080
00Yoo`03001oogoo00ioo`00<Goo00<007ooOol00Woo00<006LJIaX01fLJ00<006<KHa/01V<K00<0
06<KHa/02V<K00D006<KHa]S6`0000=S6`<000US6`03001S6`00011S6`80011S6`<000iS6`03001S
6`0000iS6`800003Ha/005lK00eO6`03001O6`0000mO708000iO708000iO7004001O70000009Ga`0
1@00GaaO7000Ga`00P002elL00<005lLGa`00UlL0P0025lL00<007ooOol017oo10004Goo000`Ool0
1000Oomoogoo0P002fLJ00<006<KHa/00f<K0P003V<K00@006<K001S6`<000aS6`80011S6`04001S
6f<K000>Ha/01@00Ha]S6f<K00002f<K0P000f<K00<006<KHa/02V<K00<006<KHa/00V<K00<005lK
Ga/02ElK0P000elK00<005lLGa`02UlL0P0000=O6`00Ga`02elL0P0000=O7000Ga`03ElL0P002elL
00<005lLGa`00P003UlL00@005lL000000YO708000Eoo`03001oogoo019oo`00<7oo0P0000=oo`00
IaX03FLJ00D006<KHa]S6`00011S6`@000iS6`04001S6f<K000=Ha/20004Ha/00`00Ha]S6`09Ha/2
0005Ha/00`00Ha]S6`06Ha/20006Ha/00`00Ha]S6`07Ha/20006Ha/00`00Ga]O6`06Ga/20006Ga/0
0`00GaaO7007Ga`20004Ga/00`00GaaO7007Ga`20004Ga`00`00GaaO7009Ga`200000elL001O700;
Ga`200000elL001O700=Ga`2000=Ga`20004Ool00`00Oomoo`0BOol0031oo`03001oo`00011W6P03
001S6`0000mS6`<00003Ha/006<K00aS6`8000AS6`03001S6f<K00US6`03001S6f<K00ES6`03001S
6f<K00IS6`8000QS6`03001S6f<K00AS6`03001S6f<K00MS6`03001S6f<K00AS6`8000US6`03001O
6elK00AO6`03001O6elK00MO6`03001O75lL00AO708000MO6`03001O75lL00EO7003001O75lL00EO
7003001O75lL00EO70<000AO7003001O75lL00MO70800005Ga`005lLGa`0000;Ga`200000elL001O
700<Ga`20004Ool4000AOol0031oo`80015W6P<000aS6`@000AS6`03001S6f<K00US6`03001S6f<K
00ES6`03001S6f<K00IS6`8000US6`03001S6f<K00AS6`03001S6f<K00US6`04001S6f<KHa/2000;
Ha/00`00Ha]S6`02Ha/00`00Ha]S6`0:Ha/01000Ga]O6elK0P002UlK00<005lLGa`00UlL0P002UlK
00<005lLGa`00UlL0P002ElL00<005lLGa`00UlL0P0025lL00<005lLGa`01ElL00D005lLGaaO7000
00=O7003001O75lL00IO708000AO7003001O75lL00QO70800003Ool007oo01Qoo`00<7oo00<006LJ
IaX03fLJ00D006LJIaYW6P0000QS6`<00003Ha/0000000IS6`03001S6f<K00IS6`8000US6`03001S
6f<K00AS6`03001S6f<K00YS6`04001S6f<KHa/2000=Ha/00`00Ha/0000>Ha/01000Ha/000003V<K
00<005lK00003ElK00@005lL000000eO6`04001O7000000<Ga`01000Ga`000002elL00<005lLGa`0
0UlL0P001ElL00D005lLGaaO700000IO7003001O75lL00EO7003001O75lL00AO70<000=oo`03001o
ogoo01Moo`00;goo0P0046LJ00<006LJIaX00fLJ00<006<KHa/00V<K0`000f<K00<006<KHa/01f<K
00<006<KHa/016<K00<006<KHa/02V<K00@006<KHa]S6`8000iS6`03001S6`0000mS6`8000mS6`80
011S6`8000iO6`8000mO6`8000iO708000iO7004001O70000008Ga`01000GaaO700015lL0P0025lL
00<005lLGa`00UlL0P001Woo00<007ooOol05goo000_Ool3000=IaX20007IaX40005Ha/00`00Ha]S
6`09Ha/01000Ha]S6f<K0P003V<K00<006<K000046<K0P003f<K00<006<K00003V<K00@006<KHa/0
00iS6`03001S6`0000aO6`80009O6`03001O6elK00YO6`800003Ga/005lL00aO7003001O700000iO
708000]O7005001O75lL001O7002000;Ga`01000Ga`000002Goo00<007ooOol05Woo000?Ool30006
Ool20004Ool50005Ool30004Ool01000Oomoo`002fLJ00<006LJIaX01VLJ0`001F<K0P003F<K00<0
06<K000046<K0`003f<K00@006<KHa/000eS6`05001S6f<KHa/0000;Ha/20004Ha/00`00Ha]S6`09
Ha/20003Ha/00`00Ga]O6`07Ga/20005Ga/00`00Ga]O6`07Ga/20004Ga/00`00GaaO7008Ga`20003
Ga`00`00GaaO7009Ga`200000elL001O700:Ga`00`00GaaO7002000=Ga`2000;Ool00`00Oomoo`0F
Ool000ioo`05001oogooOol00005Ool20005Ool01000Oomoo`001Woo00<007ooOol00goo00@007oo
Oomoo`8000QW6P03001W6VLJ00AW6P<000=W6P05001S6f<KHa/0000?Ha/3000?Ha/01@00Ha]S6f<K
00003F<K00<006<KHa/00V<K00<006<KHa/026<K0P001F<K00<006<KHa/01V<K0P001f<K00<006<K
Ha/01V<K0P001V<K00<005lKGa/01ElK00<005lKGa/01UlK00<005lKGa/01ElK00<005lKGa/01ElK
00<005lLGa`01ElL0P001UlL00<005lLGa`01UlL0P0015lL00<005lLGa`02ElL0P0000=O7000Ga`0
2UlL0P003Goo00<007ooOol05Woo000>Ool01@00Oomoogoo00003Goo00<007ooOol00goo1@0017oo
00<007ooOol00goo00<006LJIaX00fLJ0P0016LJ0`001fLJ00<006<K00003f<K00D006<KHa]S6`00
00eS6`03001S6f<K00=S6`03001S6f<K00QS6`8000IS6`03001S6f<K00IS6`03001S6f<K00IS6`03
001S6f<K00AS6`03001S6f<K00QS6`03001S6f<K00AS6`03001S6f<K00MS6`03001O6elK009O6`80
00YO6`03001O6elK009O6`8000UO6`03001O75lL00=O7003001O75lL00MO7003001O75lL00AO7003
001O75lL00EO7003001O75lL00EO70<00005Ga`005lLGa`00008Ga`2000?Ool00`00Oomoo`0FOol0
00ioo`05001oogooOol0000>Ool00`00Oomoo`02Ool01000Oomoo`001Goo0P001Goo00D006LJIaYW
6P0000=W6P<000YW6P<000iS6`03001S6f<K00=S6`03001S6f<K00QS6`8000MS6`03001S6f<K00IS
6`03001S6f<K00MS6`03001S6f<K00=S6`8000YS6`04001S6f<KHa/2000<Ha/01000Ha]S6f<K0P00
2f<K00@005lK000000eO6`04001O6`00000;Ga/01000GaaO75lL0P002UlL00<005lLGa`00UlL0P00
2ElL00<005lLGa`00UlL0P001ElL00@005lLGa`000IO7003001oogoo00moo`03001oogoo01Ioo`00
3Woo00D007ooOomoo`0000moo`03001oogoo009oo`03001oo`0000Eoo`03001oogoo00Eoo`04001W
6P00IaX3000<IaX01@00IaYW6VLJ000036<K00<006<KHa/01F<K00<006<KHa/01V<K00<006<KHa/0
26<K00<006<KHa/00f<K0P002f<K00D006<KHa]S6`0000eS6`03001S6`0000mS6`03001S6`0000eS
6`8000mO6`8000iO6`03001O700000eO7004001O7000000<Ga`01000Ga`000001elL00@005lLGa`0
00AO7080019oo`03001oogoo01Ioo`003Woo00D007ooOomoo`0000moo`03001oogoo009oo`03001o
o`0000Eoo`03001oogoo00Ioo`<000iW6P03001W6VLJ00=W6P03001S6f<K00MS6`8000US6`03001S
6f<K00=S6`8000aS6`05001S6f<KHa/0000>Ha/00`00Ha/0000>Ha/2000@Ha/2000=Ha/00`00Ha/0
000>Ga/00`00Ga/0000>Ga/2000>Ga`2000>Ga`2000:Ga`01@00GaaO7000Ga`00P0057oo00<007oo
Ool05Woo000>Ool01@00Oomoogoo00002goo00D007ooOomoo`0000Eoo`8000Eoo`03001oogoo00=o
o`<0009oo`03001W6VLJ00YW6P8000MW6P03001S6f<K00ES6`03001S6f<K00YS6`05001S6f<KHa/0
000?Ha/00`00Ha/0000?Ha/2000>Ha/01000Ha]S6`003V<K00<006<K00002f<K0P000f<K00<005lK
Ga/02ElK0P000elK00<005lKGa/02ElK0P0000=O6`00Ga`02elL0P0000=O7000Ga`02elL0P0000=O
7000Ga`02UlL00@005lL000001Moo`03001oogoo01Eoo`003goo0`003Goo0`001goo00<007ooOol0
0Woo0P000Woo0`001Woo00<006LJIaX026LJ00<006LJIaX026LJ00<006<KHa/00f<K00<006<KHa/0
36<K00<006<K000046<K0P003f<K00<006<K00003F<K00<006<KHa/00V<K00<006<KHa/02F<K0P00
0f<K00<006<KHa/01V<K0P001V<K00<005lKGa/01UlK0P001UlK00<005lKGa/01UlK0P0015lK00<0
05lLGa`01elL0P0015lL00<005lLGa`01elL0P0015lL00<005lLGa`025lL0P006Goo00<007ooOol0
5Goo000^Ool4000:Ool00`00IaYW6P06IaX00`00IaYW6P0:IaX01@00Ha]S6f<K00003f<K0P0046<K
00<006<K00003F<K0P000f<K00<006<KHa/026<K0P001V<K00<006<KHa/01V<K0P001V<K00<006<K
Ha/016<K00<006<KHa/01f<K00<005lKGa/015lK00<005lKGa/01elK00<005lKGa/015lK00<005lK
Ga/01ElK00<005lLGa`01ElL00<005lLGa`01ElL00<005lLGa`01ElL00<005lLGa`01ElL00<005lL
Ga`015lL0`0000=oo`00Ool067oo00<007ooOol05Goo000^Ool2000=Ool00`00IaYW6P03IaX2000>
IaX00`00Ha/0000?Ha/01000Ha]S6`003F<K0P000f<K00<006<KHa/02F<K00<006<KHa/016<K00<0
06<KHa/01V<K00<006<KHa/01f<K00<006<KHa/016<K00<006<KHa/01f<K00@006<KHa]S6`8000]S
6`04001O6elKGa/2000:Ga/00`00Ga]O6`02Ga/20009Ga/00`00GaaO7002Ga`20009Ga`00`00GaaO
7002Ga`20008Ga`00`00GaaO7002Ga`20005Ool00`00Oomoo`0FOol00`00Oomoo`0EOol0031oo`03
001oogoo00]oo`05001W6VLJIaX0000@IaX3000>Ha/00`00Ha]S6`02Ha/00`00Ha]S6`09Ha/00`00
Ha]S6`04Ha/00`00Ha]S6`06Ha/20008Ha/00`00Ha]S6`03Ha/2000;Ha/01000Ha]S6f<K0P002f<K
00<006<K00003V<K00<005lK00003ElK00@005lK000000aO6`04001O7000000<Ga`01000Ga`00000
2elL00@005lL000000Qoo`03001oogoo01Eoo`03001oogoo01Eoo`00<Goo00<007ooOol02goo00<0
06LJ000046LJ00D006LJIaYW6P0000aS6`03001S6f<K00AS6`03001S6f<K00IS6`8000QS6`03001S
6f<K00AS6`03001S6f<K00US6`05001S6f<KHa/0000>Ha/00`00Ha/0000=Ha/2000?Ha/2000>Ga/2
000>Ga/2000>Ga`2000=Ga`2000:Ool00`00Oomoo`0EOol00`00Oomoo`0EOol0039oo`03001oogoo
00]oo`8000mW6P03001W6VLJ00=W6P03001S6f<K00MS6`8000QS6`03001S6f<K00AS6`03001S6f<K
00US6`04001S6f<KHa/2000=Ha/00`00Ha/0000?Ha/2000=Ha/01000Ha]S6`0036<K0P000V<K00<0
05lKGa/02ElK0P000UlK00<005lKGa/02ElK0P0000=O6`00Ga`02elL0P0000=O7000Ga`02UlL0P00
3Goo00<007ooOol057oo00<007ooOol05Goo000cOol2000=Ool00`00IaYW6P0:IaX20007IaX00`00
Ha]S6`05Ha/00`00Ha]S6`09Ha/01000Ha]S6f<K0P003F<K00<006<K00003f<K0`003V<K00@006<K
Ha/000YS6`8000AS6`03001S6f<K00MS6`8000ES6`03001O6elK00MO6`03001O6elK00=O6`03001O
6elK00MO6`03001O6elK009O6`03001O75lL00MO708000AO7003001O75lL00MO7003001oogoo00io
o`03001oogoo01Aoo`03001oogoo01Aoo`00=Goo00<007ooOol02goo00<006LJIaX026LJ00<006LJ
IaX026LJ00<006<KHa/00f<K00<006<KHa/02f<K00<006<K00003f<K0`003V<K00D006<KHa]S6`00
00]S6`8000AS6`03001S6f<K00IS6`03001S6f<K00ES6`03001S6f<K00ES6`03001S6f<K00IS6`03
001O6elK00AO6`8000MO6`03001O6elK00AO6`8000IO6`03001O75lL00EO7003001O75lL00EO7003
001O75lL00AO7080019oo`03001oogoo01=oo`03001oogoo01Aoo`00=Woo00<007ooOol02goo00<0
06LJIaX01VLJ00<006LJIaX02VLJ00D006<KHa]S6`0000iS6`8000mS6`05001S6f<KHa/0000;Ha/2
0005Ha/00`00Ha]S6`06Ha/20007Ha/00`00Ha]S6`04Ha/00`00Ha]S6`07Ha/00`00Ha]S6`02Ha/2
000:Ha/01000Ga]O6elK0P002UlK00@005lKGa]O6`8000UO6`03001O75lL009O708000UO7004001O
75lLGa`2000DOol00`00Oomoo`0COol00`00Oomoo`0DOol003Moo`03001oogoo00]oo`03001W6VLJ
00=W6P8000iW6P03001S6`0000iS6`04001S6f<K000=Ha/00`00Ha]S6`03Ha/00`00Ha]S6`06Ha/2
0008Ha/00`00Ha]S6`04Ha/00`00Ha]S6`08Ha/01000Ha]S6f<K0P002f<K00@006<K000000eS6`03
001O6`0000eO6`03001O6`0000aO6`04001O7000000<Ga`00`00Ga`0000GOol00`00Oomoo`0BOol0
0`00Oomoo`0DOol003Qoo`03001oogoo00]oo`05001W6VLJIaX0000@IaX2000>Ha/00`00Ha]S6`02
Ha/00`00Ha]S6`08Ha/20007Ha/00`00Ha]S6`04Ha/00`00Ha]S6`09Ha/01000Ha]S6f<K0P0036<K
00<006<K00003F<K0P003f<K0P003UlK0P003ElK0P003UlL0P006Goo00<007ooOol04Goo00<007oo
Ool057oo000iOol2000<Ool00`00IaX0000@IaX01000IaYW6P0036<K00<006<KHa/016<K00<006<K
Ha/01V<K00<006<KHa/026<K00@006<KHa]S6`8000eS6`03001S6`0000iS6`8000eS6`03001S6`00
00iS6`03001S6`0000aO6`800003Ga/005lK00YO6`800003Ga/005lL00]O708001aoo`03001oogoo
011oo`03001oogoo01Aoo`00>goo00<007ooOol02Woo0P003fLJ00<006LJIaX00VLJ00<006<KHa/0
1f<K0P0026<K00<006<KHa/00f<K0P0036<K00<006<K00003f<K0P003V<K00@006<KHa/000YS6`80
00=S6`03001S6f<K00US6`8000=S6`03001O6elK00MO6`8000AO6`03001O6elK00MO6`03001O6elK
009O6`03001O75lL00MO708001ioo`03001oogoo011oo`03001oogoo01Aoo`00?7oo00<007ooOol0
2goo00<006LJIaX02VLJ0P001VLJ00<006<KHa/01F<K00<006<KHa/02F<K00D006<KHa]S6`0000iS
6`<000iS6`04001S6f<K000;Ha/20004Ha/00`00Ha]S6`06Ha/00`00Ha]S6`04Ha/00`00Ha]S6`06
Ha/20006Ha/00`00Ga]O6`05Ga/00`00Ga]O6`05Ga/00`00Ga]O6`04Ga/20006Ga/00`00GaaO7005
Ga`00`00Oomoo`0OOol00`00Oomoo`0@Ool00`00Oomoo`0COol003eoo`03001oogoo00]oo`03001W
6VLJ00QW6P03001W6VLJ00MW6P03001S6f<K00=S6`03001S6f<K00]S6`03001S6`0000iS6`05001S
6f<KHa/0000;Ha/20004Ha/00`00Ha]S6`06Ha/20007Ha/00`00Ha]S6`04Ha/00`00Ha]S6`06Ha/0
0`00Ha]S6`04Ha/00`00Ha]S6`07Ha/00`00Ga]O6`02Ga/20009Ga/01000Ga]O6elK0P002ElK00<0
05lLGa`00UlL0P008goo00<007ooOol03goo00<007ooOol04goo000nOol2000<Ool00`00IaYW6P06
IaX00`00IaYW6P09IaX01@00Ha]S6f<K00003V<K0`003F<K00<006<KHa/00f<K00<006<KHa/01V<K
0P001f<K00<006<KHa/016<K00<006<KHa/026<K00@006<KHa]S6`8000YS6`04001S6f<KHa/2000;
Ha/01000Ga/0000035lK00<005lK000035lK00@005lL000002Eoo`03001oogoo00moo`03001oogoo
01=oo`00@7oo00<007ooOol02Woo00<006LJIaX00fLJ0P003FLJ00<006<K00003V<K00D006<KHa]S
6`0000YS6`8000MS6`03001S6f<K00AS6`03001S6f<K00QS6`04001S6f<KHa/2000<Ha/00`00Ha/0
000=Ha/00`00Ha/0000=Ha/2000>Ga/2000=Ga/2000XOol00`00Oomoo`0>Ool00`00Oomoo`0COol0
045oo`03001oogoo00Yoo`05001W6VLJIaX0000?IaX3000=Ha/00`00Ha]S6`03Ha/00`00Ha]S6`06
Ha/00`00Ha]S6`08Ha/01000Ha]S6f<K0P0036<K00<006<K00003V<K0P003V<K0P003F<K00<006<K
000035lK0P0000=O6`00Ga/02UlK0P00:goo00<007ooOol03Goo00<007ooOol04goo0012Ool00`00
Oomoo`0:Ool00`00IaX0000?IaX01@00IaYW6VLJ00002V<K0P001f<K00<006<KHa/00f<K0P0036<K
00<006<K00003V<K0P003V<K00<006<K000036<K0P0000=S6`00Ha/02V<K0P000f<K00<005lKGa/0
1elK0P0015lK00<005lKGa/01elK00<007ooOol0;7oo00<007ooOol037oo00<007ooOol04goo0013
Ool2000;Ool2000>IaX00`00IaYW6P03IaX00`00Ha]S6`06Ha/00`00Ha]S6`08Ha/01@00Ha]S6f<K
00003V<K0P003V<K00@006<KHa/000]S6`8000=S6`03001S6f<K00MS6`8000AS6`03001S6f<K00IS
6`8000IS6`03001O6elK00EO6`03001O6elK00EO6`03001O6elK00AO6`8002moo`03001oogoo00ao
o`03001oogoo01=oo`00AGoo00<007ooOol02Woo00<006LJIaX02FLJ0P001fLJ00<006<KHa/00f<K
0P0036<K00<006<K00003V<K00<006<K00003F<K00<006<KHa/00V<K00<006<KHa/01V<K0P001V<K
00<006<KHa/01F<K00<006<KHa/01F<K00<006<KHa/016<K00<006<KHa/01f<K00<005lKGa/00UlK
0P002ElK00@005lKGa]O6`80039oo`03001oogoo00aoo`03001oogoo019oo`00AWoo00<007ooOol0
2Woo00<006LJIaX01fLJ00<006LJIaX026LJ00D006<KHa]S6`0000iS6`<000eS6`05001S6f<KHa/0
000:Ha/20006Ha/00`00Ha]S6`04Ha/00`00Ha]S6`07Ha/00`00Ha]S6`02Ha/20009Ha/01000Ha]S
6f<K0P002f<K00@005lK000000aO6`03001O6`0003Eoo`03001oogoo00]oo`03001oogoo019oo`00
Agoo00<007ooOol02Woo0P001FLJ0P0036LJ00<006<K00003V<K00D006<KHa]S6`0000YS6`8000ES
6`03001S6f<K00IS6`03001S6f<K00MS6`04001S6f<KHa/2000;Ha/01000Ha/0000036<K00<006<K
00003F<K0P003UlK0P00=goo00<007ooOol02Woo00<007ooOol04Woo0018Ool00`00Oomoo`0;Ool0
1@00IaYW6VLJ00003VLJ0`003F<K00<006<KHa/00f<K00<006<KHa/01V<K00<006<KHa/01V<K00<0
06<KHa/00f<K0P002f<K00<006<K00003F<K0P003V<K0`002f<K0P0000=S6`00Ga/02elK0P00>Goo
00<007ooOol02Woo00<007ooOol04Woo0019Ool2000<Ool00`00IaX0000>IaX01@00IaYW6VLJ0000
2V<K0P001f<K00<006<KHa/00f<K0P002V<K00D006<KHa]S6`0000eS6`8000eS6`04001S6f<K000;
Ha/20003Ha/00`00Ha]S6`07Ha/00`00Ha]S6`02Ha/00`00Ga]O6`07Ga/2000lOol00`00Oomoo`09
Ool00`00Oomoo`0BOol004]oo`03001oogoo00Yoo`03001W6VLJ00aW6P03001W6VLJ00=W6P03001S
6f<K00IS6`03001S6f<K00QS6`05001S6f<KHa/0000=Ha/00`00Ha/0000=Ha/01000Ha]S6`002V<K
0P0016<K00<006<KHa/01f<K00<006<KHa/016<K00<006<KHa/016<K0P001V<K00<005lKGa/01ElK
00<007ooOol0?Goo00<007ooOol027oo00<007ooOol04Woo001<Ool00`00Oomoo`0:Ool00`00IaYW
6P0:IaX00`00IaYW6P05IaX00`00Ha]S6`03Ha/2000<Ha/00`00Ha/0000>Ha/3000;Ha/20004Ha/0
0`00Ha]S6`06Ha/00`00Ha]S6`05Ha/00`00Ha]S6`04Ha/20008Ha/01000Ha]S6f<K0P002F<K00<0
05lKGa/00UlK0P00@7oo00<007ooOol027oo00<007ooOol04Woo001=Ool00`00Oomoo`0:Ool00`00
IaYW6P07IaX20009IaX01@00Ha]S6f<K00003V<K0P003V<K00D006<KHa]S6`0000QS6`8000MS6`03
001S6f<K00AS6`03001S6f<K00MS6`04001S6f<KHa/2000;Ha/00`00Ha/0000<Ha/01000Ga/00000
@goo00<007ooOol027oo00<007ooOol04Goo001>Ool2000;Ool00`00IaYW6P05IaX00`00IaYW6P0:
IaX00`00Ha/0000>Ha/01000Ha]S6`002f<K0P001F<K00<006<KHa/016<K00<006<KHa/026<K00@0
06<KHa]S6`8000]S6`03001S6`0000eS6`8000eS6`8004Ioo`03001oogoo00Moo`03001oogoo015o
o`00D7oo00<007ooOol02Goo00<006LJIaX00fLJ00<006LJIaX02fLJ0`003F<K00<006<KHa/00V<K
00<006<KHa/01V<K0P0026<K00@006<KHa]S6`8000aS6`03001S6`0000eS6`8000eS6`03001S6`00
00]S6`8004Uoo`03001oogoo00Ioo`03001oogoo015oo`00DGoo00<007ooOol02Goo00D006LJIaYW
6P0000eW6P05001W6VLJIaX0000:Ha/20006Ha/00`00Ha]S6`04Ha/00`00Ha]S6`09Ha/00`00Ha/0
000>Ha/2000=Ha/00`00Ha/0000;Ha/20003Ha/00`00Ha]S6`07Ha/00`00Oomoo`19Ool00`00Oomo
o`06Ool00`00Oomoo`0AOol0059oo`03001oogoo00Uoo`03001W6P0000eW6P03001W6VLJ00=W6P03
001S6f<K00IS6`03001S6f<K00MS6`04001S6f<KHa/2000<Ha/2000>Ha/01000Ha]S6`002V<K0P00
0f<K00<006<KHa/01V<K0P001V<K00<006<KHa/016<K0P00CGoo00<007ooOol01Goo00<007ooOol0
4Goo001COol2000:Ool2000<IaX00`00IaYW6P05IaX00`00Ha]S6`03Ha/2000;Ha/00`00Ha/0000=
Ha/01000Ha]S6`002f<K0P0016<K00<006<KHa/01V<K00<006<KHa/016<K00<006<KHa/016<K00<0
06<KHa/01f<K00@006<KHa]S6`80051oo`03001oogoo00Aoo`03001oogoo015oo`00EGoo00<007oo
Ool02Goo00<006LJIaX01fLJ0P002FLJ00D006<KHa]S6`0000eS6`<000aS6`03001S6f<K009S6`03
001S6f<K00IS6`8000MS6`03001S6f<K00AS6`03001S6f<K00IS6`04001S6f<KHa/2000;Ha/00`00
Ha/0001COol00`00Oomoo`03Ool00`00Oomoo`0AOol005Ioo`03001oogoo00Uoo`03001W6VLJ00EW
6P03001W6VLJ00YW6P03001S6`0000eS6`05001S6f<KHa/0000:Ha/00`00Ha]S6`04Ha/00`00Ha]S
6`04Ha/00`00Ha]S6`08Ha/01000Ha]S6f<K0P002V<K00<006<K00003F<K0P00E7oo00<007ooOol0
17oo00<007ooOol047oo001GOol00`00Oomoo`09Ool00`00IaYW6P03IaX00`00IaYW6P0;IaX3000<
Ha/00`00Ha]S6`03Ha/00`00Ha]S6`05Ha/20008Ha/01000Ha]S6f<K0P0036<K00<006<K000036<K
0`002f<K0P00Egoo00<007ooOol00goo00<007ooOol047oo001HOol00`00Oomoo`09Ool01@00IaYW
6VLJ00003FLJ00D006LJIaYW6P0000YS6`03001S6f<K00ES6`03001S6f<K00=S6`03001S6f<K00US
6`03001S6`0000iS6`8000]S6`8000=S6`03001S6f<K00MS6`03001oogoo05Qoo`03001oogoo009o
o`03001oogoo011oo`00FGoo0P002Woo00<006LJ00003FLJ00<006LJIaX00fLJ00<006<KHa/01F<K
0P002F<K00D006<KHa]S6`0000aS6`8000iS6`04001S6f<K0009Ha/00`00Ha]S6`04Ha/00`00Ha]S
6`04Ha/2001LOol01@00Oomoogoo00004Woo001KOol00`00Oomoo`08Ool2000<IaX00`00IaYW6P05
IaX00`00Ha]S6`03Ha/00`00Ha]S6`0:Ha/00`00Ha/0000<Ha/01000Ha]S6`002f<K0P0016<K00<0
06<KHa/016<K0P0026<K00@006<KHa]S6`8005ioo`05001oogooOol0000BOol005aoo`03001oogoo
00Uoo`03001W6VLJ00MW6P8000UW6P05001S6f<KHa/0000=Ha/2000<Ha/00`00Ha]S6`02Ha/00`00
Ha]S6`06Ha/20007Ha/01000Ha]S6f<K0P002f<K00<006<K0000HGoo00@007ooOol0019oo`00GGoo
00<007ooOol02Goo00<006LJIaX01FLJ00<006LJIaX02VLJ00<006<K00003F<K00@006<KHa/000YS
6`03001S6f<K00AS6`03001S6f<K00AS6`03001S6f<K00QS6`03001S6`0000eS6`8006=oo`03001o
o`00019oo`00GWoo0P002Woo00<006LJIaX00fLJ00<006LJIaX02fLJ0P003F<K00<006<KHa/00V<K
00<006<KHa/01F<K0P0026<K00@006<KHa]S6`8000]S6`8000eS6`03001oogoo06=oo`04001oogoo
000AOol0061oo`03001oogoo00Qoo`05001W6VLJIaX0000=IaX01000IaYW6P002V<K0P001V<K00<0
06<KHa/00f<K00<006<KHa/02F<K00<006<K000036<K00@006<KHa/000YS6`8006Moo`03001oo`00
015oo`00HGoo00<007ooOol027oo00<006LJ00003FLJ00<006LJIaX00VLJ00<006<KHa/01V<K00<0
06<KHa/01f<K00D006<KHa]S6`0000aS6`<000YS6`8000AS6`03001S6f<K00IS6`03001oogoo06Qo
o`80015oo`00HWoo00<007ooOol027oo0P002fLJ0P001VLJ00<006<KHa/00f<K0P002f<K00<006<K
000036<K00D006<KHa]S6`0000QS6`03001S6f<K00ES6`03001S6f<K00AS6`03001oogoo06Qoo`<0
015oo`00Hgoo0P002Woo00<006LJIaX01fLJ00<006LJIaX01fLJ00D006<KHa]S6`0000eS6`<000YS
6`8000ES6`03001S6f<K00AS6`03001S6f<K00MS6`04001S6f<KHa/2001WOol4000DOol006Eoo`03
001oogoo00Qoo`8000EW6P8000]W6P03001S6`0000eS6`05001S6f<KHa/00008Ha/00`00Ha]S6`06
Ha/01000Ha]S6f<K0P002f<K00<006<K0000IGoo100067oo001VOol00`00Oomoo`09Ool01@00IaYW
6VLJ00003FLJ0`002f<K0P001F<K00<006<KHa/016<K00<006<KHa/026<K00<006<K00003F<K0P00
Hgoo0`0077oo001WOol00`00Oomoo`09Ool00`00IaX0000=IaX01@00IaYW6VLJ00002F<K00<006<K
Ha/01V<K00@006<KHa]S6`8000]S6`8000eS6`03001oogoo05moo`@001moo`00J7oo0P002Woo0P00
36LJ00<006LJIaX00fLJ0P001V<K00<006<KHa/026<K00<006<K000036<K00<006<K00002f<K0P00
GWoo10008goo001ZOol00`00Oomoo`09Ool00`00IaYW6P08IaX00`00IaYW6P06IaX01000Ha]S6f<K
0P002f<K0P0036<K00D006<KHa]S6`0000QS6`8005aoo`@002Moo`00Jgoo00<007ooOol02Goo00<0
06LJIaX01FLJ0P002VLJ00<006<K000036<K00@006<KHa/000YS6`03001S6f<K00=S6`03001S6f<K
00AS6`03001oogoo05Qoo`@002]oo`00K7oo00<007ooOol02Goo00<006LJIaX00fLJ00<006LJIaX0
2VLJ0P0036<K00<006<KHa/00V<K00<006<KHa/01F<K0P001f<K00@006<KHa]S6`8005Moo`@002mo
o`00KGoo00<007ooOol02Goo00D006LJIaYW6P0000aW6P04001W6VLJ000:Ha/00`00Ha]S6`04Ha/0
0`00Ha]S6`03Ha/00`00Ha]S6`08Ha/00`00Ha/0001EOol4000cOol006ioo`8000Yoo`03001W6P00
00aW6P03001W6VLJ009W6P03001S6f<K00ES6`8000QS6`05001S6f<KHa/0000;Ha/2001COol3000g
Ool0071oo`03001oogoo00Qoo`8000YW6P8000IW6P03001S6f<K00=S6`03001S6f<K00US6`03001S
6`0000]S6`03001oogoo04moo`@003Yoo`00LGoo00<007ooOol02Goo00<006LJIaX01VLJ00<006LJ
IaX01fLJ00D006<KHa]S6`0000aS6`8000YS6`8004ioo`@003ioo`00LWoo00<007ooOol02Goo00<0
06LJIaX00fLJ0P002fLJ00<006<K000036<K00@006<KHa/000QS6`03001oogoo04Yoo`@0049oo`00
Lgoo0P002Woo00D006LJIaYW6P0000eW6P<000YS6`8000AS6`03001S6f<K00AS6`03001oogoo04Mo
o`@004Ioo`00MGoo00<007ooOol027oo00<006LJ00003FLJ00D006LJIaYW6P0000QS6`03001S6f<K
00ES6`04001S6f<KHa/20016Ool4001:Ool007Ioo`03001oogoo00Qoo`03001W6VLJ00]W6P03001W
6VLJ00=W6P03001S6f<K00AS6`03001S6f<K00MS6`03001S6`0004Aoo`@004ioo`00Mgoo00<007oo
Ool027oo00<006LJIaX026LJ0P001fLJ00@006<KHa]S6`8000YS6`80049oo`<0059oo`00N7oo0P00
2Goo00<006LJIaX01VLJ00<006LJIaX026LJ00<006<K00002f<K00<007ooOol0?Woo1000EGoo001j
Ool00`00Oomoo`07Ool00`00IaYW6P03IaX2000;IaX2000:Ha/2000mOol4001IOol007]oo`03001o
ogoo00Moo`05001W6VLJIaX0000<IaX01000IaYW6P0026<K00<007ooOol0>Goo1000GGoo001lOol0
0`00Oomoo`07Ool00`00IaX0000<IaX00`00IaYW6P02IaX00`00Ha]S6`04Ha/00`00Oomoo`0fOol4
001QOol007eoo`03001oogoo00Moo`8000]W6P03001W6VLJ00AW6P04001S6f<KHa/2000eOol4001U
Ool007ioo`8000Uoo`03001W6VLJ00IW6P8000QW6P03001S6`0003Aoo`<006Uoo`00P7oo00<007oo
Ool01goo0P001FLJ00<006LJIaX026LJ0P00<Goo1000K7oo0021Ool00`00Oomoo`08Ool01@00IaYW
6VLJ00002VLJ00<007ooOol0;Goo1000L7oo0022Ool00`00Oomoo`08Ool00`00IaX0000:IaX00`00
Oomoo`0ZOol4001dOol008=oo`8000Uoo`03001W6VLJ00QW6P03001oogoo02Moo`@007Qoo`00QGoo
00<007ooOol01goo00<006LJIaX01FLJ0P009Woo1000O7oo0026Ool00`00Oomoo`07Ool00`00IaYW
6P03IaX00`00Oomoo`0ROol40020Ool008Moo`03001oogoo00Moo`05001W6VLJIaX0000ROol30024
Ool008Qoo`8000Qoo`03001W6P0001moo`@008Moo`00RWoo00<007ooOol01Woo00<007ooOol06Woo
1000Rgoo002;Ool00`00Oomoo`05Ool00`00Oomoo`0FOol4002?Ool008aoo`03001oogoo00Aoo`03
001oogoo019oo`@009=oo`00SGoo00<007ooOol00goo00<007ooOol03Woo1000Ugoo002>Ool20003
Ool00`00Oomoo`0:Ool4002KOol0091oo`04001oogoo0009Ool3002OOol0095oo`03001oo`0000Eo
o`@00:9oo`00TWoo0P0000=oo`0000000P00YWoo002COol2002ZOol00?mood1oo`00ogoo@7oo003o
Oom0Ool00?mood1oo`00ogoo@7oo003oOom0Ool00?mood1oo`00ogoo@7oo003oOom0Ool00?mood1o
o`00ogoo@7oo003oOom0Ool00?mood1oo`00ogoo@7oo003oOom0Ool00?mood1oo`00ogoo@7oo003o
Oom0Ool00?mood1oo`00ogoo@7oo001TOol4003GOol006Eoo`03001oogoo0=Moo`00IGoo00<007oo
Ool0egoo001@Ool00`00Oomoo`09Ool50004Ool00`00Ool0000200000goo000000040004Ool40004
Ool50003Ool300000goo00000004Ool30002Ool30002Ool40004Ool300000goo00000002Ool40003
Ool50009Ool40003Ool60009Ool40006Ool50003Ool3001FOol0051oo`03001oogoo00Uoo`03001o
ogoo009oo`04001oogooOol20003Ool01000Oomoo`001Woo00<007ooOol00Woo00@007ooOol000=o
o`05001oogooOol00003Ool20006Ool00`00Oomoo`02Ool01000Oomoo`0017oo00@007ooOol000=o
o`80009oo`03001oogoo009oo`03001oogoo009oo`03001oogoo00Qoo`03001oogoo009oo`03001o
ogoo009oo`03001oogoo00]oo`03001oogoo00Eoo`05001oogooOol00003Ool01@00Oomoogoo0000
EGoo001>Ool50009Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00@007ooOol000Ioo`03001o
ogoo00Eoo`05001oogooOol00003Ool00`00Oomoo`02Ool00`00Oomoo`04Ool00`00Oomoo`02Ool0
1000Oomoo`0017oo00@007ooOol000Aoo`04001oogoo0009Ool00`00Oomoo`08Ool00`00Oomoo`02
Ool00`00Oomoo`02Ool00`00Oomoo`0;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007oo
Ool0Egoo001@Ool00`00Oomoo`0<Ool30003Ool00`00Oomoo`02Ool01000Oomoo`001Woo1P000goo
10000goo00<007ooOol00Woo00<007ooOol017oo00H007oo000007oo0002Ool00`00Oomoo`02Ool0
1000Oomoo`0017oo00<007ooOol01P0017oo00<007ooOol027oo00<007ooOol00Woo00<007ooOol0
0Woo00<007ooOol02goo100017oo00D007ooOomoo`0000=oo`03001oogoo05Moo`00D7oo00<007oo
Ool02Goo0`001Woo0P000goo00<007ooOol00P001Goo00<007ooOol00Woo00<007ooOol017oo00D0
07ooOomoo`0000=oo`8000Ioo`06001oo`00001oo`000Woo00<007ooOol00Woo00@007ooOol000=o
o`80009oo`03001oogoo009oo`03001oogoo009oo`03001oogoo00Qoo`03001oogoo009oo`03001o
ogoo009oo`03001oogoo00]oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00<007ooOol0
Egoo001LOol00`00Oomoo`02Ool00`00Oomoo`02000017oo000000000Woo0P0000Aoo`00000000=o
o`@000Aoo`<000Eoo`<00003Ool007oo00Eoo`80009oo`8000=oo`@000Aoo`<00003Ool007oo009o
o`@000Eoo`03001oogoo00Uoo`@000=oo`H000Yoo`05001oogooOol00002Ool20002Ool200000goo
00000004001FOol005aoo`03001oogoo009oo`03001oogoo02Eoo`03001oogoo00Aoo`80009oo`80
00moo`03001oogoo00Yoo`03001oogoo019oo`03001oogoo00]oo`05001oogooOol0000;Ool00`00
Oomoo`1GOol005eoo`D002Moo`03001oogoo00=oo`8000Aoo`8000ioo`03001oogoo00Yoo`03001o
ogoo019oo`03001oogoo00Yoo`D000aoo`03001oogoo05Moo`00R7oo0P006Woo0P002Woo0`005Goo
0`00M7oo003oOom0Ool00?mood1oo`00ogoo@7oo003oOol=Ool4000_Ool00?moo`moo`03001oogoo
02ioo`00ogoo47oo00<007ooOol0;Goo001?Ool20005Ool40003Ool50009Ool50005Ool;0003Ool6
0003Ool50003Ool40004Ool40007Ool60003Ool40002Ool60008Ool60003Ool50004Ool50002Ool5
0005Ool30004Ool50003Ool40004Ool50004Ool30005Ool00`00Oomoo`0]Ool004moo`8000Aoo`03
001oogoo009oo`03001oogoo009oo`03001oogoo00Uoo`03001oogoo009oo`05001oogooOol00003
Ool01@00Oomoogoo000027oo00<007ooOol00goo00D007ooOomoo`0000=oo`03001oogoo009oo`04
001oogoo0004Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01@00Oomoogoo0000
37oo00D007ooOomoo`0000Eoo`03001oogoo00=oo`05001oogooOol00003Ool00`00Oomoo`02Ool0
1@00Oomoogoo00000goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00goo00D007ooOomoo`0000=oo`03001oo`0002ioo`00CWoo10000goo00<007oo
Ool00Woo00<007ooOol00Woo00<007ooOol02Goo00<007ooOol00Woo00D007ooOomoo`0000=oo`05
001oogooOol00008Ool00`00Oomoo`03Ool01@00Oomoogoo00000goo00<007ooOol01Goo00<007oo
Ool02goo00<007ooOol00goo00<007ooOol00Woo00D007ooOomoo`0000aoo`05001oogooOol00005
Ool00`00Oomoo`03Ool01@00Oomoogoo00001Woo0`000goo00<007ooOol01goo00<007ooOol00Woo
00<007ooOol027oo00<007ooOol00goo00<007ooOol01Goo00<007oo0000;Woo001>Ool01000Oomo
o`000goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol037oo0`000goo00D007ooOomoo`00
00=oo`03001oogoo00Ioo`03001oogoo00Aoo`@000=oo`03001oogoo00Eoo`H000Qoo`03001oogoo
00=oo`03001oogoo009oo`05001oogooOol0000<Ool00`00Ool00007Ool00`00Oomoo`04Ool40003
Ool30006Ool00`00Oomoo`07Ool00`00Oomoo`02Ool00`00Oomoo`08Ool00`00Oomoo`03Ool00`00
Oomoo`04Ool01@00Oomoogoo0000;Goo001>Ool01000Oomoo`000goo00<007ooOol00Woo00<007oo
Ool00Woo00<007ooOol02Goo0`001Woo00D007ooOomoo`0000=oo`8000Moo`03001oogoo00Moo`05
001oogooOol00004Ool01000Oomoo`0017oo00<007ooOol01Woo00<007ooOol00goo00<007ooOol0
0Woo00@007ooOomoo`8000]oo`<000Moo`03001oogoo00Moo`05001oogooOol00004Ool01@00Oomo
ogoo00002Goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00goo00<0
07ooOol017oo00D007ooOomoo`0002eoo`00CGoo00<007ooOol00Woo00@007ooOomoo`@000Eoo`03
001oogoo00Uoo`03001oogoo009oo`03001oogoo0080009oo`80009oo`800004Ool000000002Ool6
0003Ool30005Ool50003Ool40007Ool60003Ool40003Ool2000017oo0000000027oo00D007oo001o
o`0000Eoo`03001oogoo00Aoo`<000Eoo`D00003Ool0000000@000Aoo`<000Eoo`D000=oo`<000=o
o`H0009oo`<00004Ool00000000/Ool004eoo`03001oogoo009oo`03001oogoo00Yoo`03001oogoo
00Uoo`03001oogoo009oo`03001oogoo019oo`03001oogoo025oo`03001oogoo01Uoo`05001oogoo
Ool00005Ool00`00Oomoo`0DOol00`00Oomoo`0MOol00`00Oomoo`0fOol004aoo`<0009oo`<000]o
o`03001oogoo00Yoo`D001Aoo`03001oogoo025oo`03001oogoo01Qoo`H000Eoo`03001oogoo01Ao
o`03001oogoo00Moo`03001oogoo00eoo`03001oogoo00=oo`03001oogoo03Ioo`00GGoo0`009Woo
0`008Goo0`0087oo0`0087oo00<007ooOol03Goo00<007ooOol0?7oo003oOom0Ool00?mood1oo`00
\
\>"],
  ImageRangeCache->{{{0, 318.812}, {258.375, 0}} -> {-0.172302, 0.0574078, 
  0.00466661, 0.00466661}}]
}, Open  ]],

Cell["\<\
If, in addition, one adds in the effect of a time-dependent yield \
curve, but uses a constant interest rate in assessing BS implied volatility, \
one can generate a fictitious term structure for the implied volatility. The \
lesson that should be learned is that one should subtract out all the known \
time-dependent and asset-level-dependent effects before attempting to build \
two-factor stochastic or GARCH models of volatility.\
\>", "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["28.11 Closing Remarks", "Section",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "A book has to end somewhere, and this is where matters are brought to a \
close. There is much more that could be said about CEV and its cousins in \
interest-rate modelling. An important point is that the sequence of \
transformations made to analyse the PDE can also be applied, with minor \
modifications, to the family of interest-rate models discussed in Chapters \
25-27. All such single-factor models, and the CEV family, can be reduced to \
the standard diffusion equation with advection and source terms, allowing \
most of the numerical machinery developed in Chapters 13-20 to be brought to \
bear, in a single unified framework. Another point not discussed here is how \
to characterize the probability distributions that arise when ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\)]],
  " is not unity and there is also mean reversion. It is left as an exercise \
for the reader to follow through the sequence of steps of Section 28.3 with \
the mean reversion included. The resulting PDE becomes nice (in a sense that \
becomes obvious when you try it) only if ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu] = 1/2, \ 1\)]],
  " or the mean-reversion is excluded. What the enveloping distribution is \
when mean-reversion is present and \[Nu] is general is not known to this \
author. "
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 28 Bibliography", "Section",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[TextData[{
  "Beckers, S., 1980, The constant elasticity of variance model and its \
implications for option pricing ",
  StyleBox["Journal of Finance",
    FontSlant->"Italic"],
  ", 35, June, p. 661.\n\nCox, J., 1996, The constant elasticity of variance \
option pricing model, ",
  StyleBox["Journal of Portfolio Management",
    FontSlant->"Italic"],
  ", special issue.\n\nHauser, S., and Lauterbach, B., 1996, Tests of warrant \
pricing models: the trading profits perspective, ",
  StyleBox["Journal of Derivatives",
    FontSlant->"Italic"],
  ", Winter, p. 71.\n\nJarrow, R.A. and Rudd, A., 1983, ",
  StyleBox["Option Pricing",
    FontSlant->"Italic"],
  " (Section 11.6), Irwin.\n\nLauterbach, B. and Schultz, P., 1990, Pricing \
warrants: an empirical study of the Black-Scholes model and its alternatives, \
",
  StyleBox["Journal of Finance, ",
    FontSlant->"Italic"],
  "45  (Sept.), p. 1181;\n\nNelken, I. (editor), 1990, ",
  StyleBox["The Handbook of Exotic Options",
    FontSlant->"Italic"],
  ", Irwin.\n\nPress, W.H., Teukolsky, S.A., Vetterling, W.T. and Flannery, \
B.P., 1992, ",
  StyleBox["Numerical Recipes in C - the Art of Scientific Computing",
    FontSlant->"Italic"],
  ", 2nd edition. Cambridge University Press.\n\nSchroder, M., 1989, \
Computing the constant elasticity of variance option pricing formula, ",
  StyleBox["Journal of Finance,",
    FontSlant->"Italic"],
  " 44, March, p. 221.\n"
}], "Text",
  CellMargins->{{6.8125, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
ScreenStyleEnvironment->"Printout",
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{579, 617},
WindowMargins->{{26, Automatic}, {Automatic, 3}},
PrintingCopies->1,
PrintingStartingPageNumber->515,
PrintingPageRange->{531, 533},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"28. CEV Models"}]]}, {Cell[ 
        TextData[ {"28. CEV Models"}]], Inherited, Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True,
"FirstPageFace"->Right},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
PrivateFontOptions->{"FontType"->"Outline"},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;H8`Omooh`=<`Tj0fL5N`?P0080004/0B`000000]P2:001
0000I00000400`<30?l00BL?00400@S<]<000000000006P801T1T00000000000
00000000007oo@00003oo@001HL00000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 229, 8, 95, "Title"],
Cell[1963, 61, 122, 1, 33, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2110, 66, 98, 1, 44, "Section"],
Cell[2211, 69, 4700, 68, 610, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6948, 142, 132, 1, 38, "Section"],
Cell[7083, 145, 306, 6, 40, "Text"],
Cell[7392, 153, 193, 4, 20, "NumberedEquation"],
Cell[7588, 159, 515, 12, 40, "Text"],
Cell[8106, 173, 190, 3, 21, "NumberedEquation"],
Cell[8299, 178, 106, 1, 20, "Text"],
Cell[8408, 181, 191, 3, 33, "NumberedEquation"],
Cell[8602, 186, 356, 6, 40, "Text"],
Cell[8961, 194, 210, 4, 30, "NumberedEquation"],
Cell[9174, 200, 1725, 30, 172, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10936, 235, 137, 1, 38, "Section"],
Cell[11076, 238, 266, 7, 20, "Text"],
Cell[11345, 247, 663, 13, 32, "NumberedEquation"],
Cell[12011, 262, 323, 9, 29, "Text"],
Cell[12337, 273, 741, 17, 32, "NumberedEquation"],
Cell[13081, 292, 337, 9, 20, "Text"],
Cell[13421, 303, 281, 6, 30, "NumberedEquation"],
Cell[13705, 311, 107, 1, 20, "Text"],
Cell[13815, 314, 675, 14, 32, "NumberedEquation"],
Cell[14493, 330, 247, 6, 20, "Text"],
Cell[14743, 338, 243, 4, 24, "NumberedEquation"],
Cell[14989, 344, 250, 5, 29, "Text"],
Cell[15242, 351, 573, 14, 32, "NumberedEquation"],
Cell[15818, 367, 192, 4, 20, "Text"],
Cell[16013, 373, 961, 20, 33, "NumberedEquation"],
Cell[16977, 395, 323, 7, 29, "Text"],
Cell[17303, 404, 442, 11, 30, "NumberedEquation"],
Cell[17748, 417, 542, 14, 29, "Text"],
Cell[18293, 433, 609, 15, 24, "NumberedEquation"],
Cell[18905, 450, 82, 1, 20, "Text"],
Cell[18990, 453, 380, 10, 32, "NumberedEquation"],
Cell[19373, 465, 92, 1, 20, "Text"],
Cell[19468, 468, 429, 8, 33, "NumberedEquation"],
Cell[19900, 478, 202, 6, 20, "Text"],
Cell[20105, 486, 333, 7, 24, "NumberedEquation"],
Cell[20441, 495, 231, 5, 29, "Text"],
Cell[20675, 502, 172, 3, 20, "NumberedEquation"],
Cell[20850, 507, 180, 4, 20, "Text"],
Cell[21033, 513, 1022, 25, 33, "NumberedEquation"],
Cell[22058, 540, 249, 6, 20, "Text"],
Cell[22310, 548, 1036, 25, 36, "NumberedEquation"],
Cell[23349, 575, 265, 5, 29, "Text"],
Cell[23617, 582, 275, 7, 20, "NumberedEquation"],
Cell[23895, 591, 93, 1, 20, "Text"],
Cell[23991, 594, 967, 23, 36, "NumberedEquation"],
Cell[24961, 619, 241, 6, 20, "Text"],
Cell[25205, 627, 370, 8, 29, "NumberedEquation"],
Cell[25578, 637, 130, 1, 20, "Text"],
Cell[25711, 640, 1359, 29, 34, "NumberedEquation"],
Cell[27073, 671, 291, 9, 20, "Text"],
Cell[27367, 682, 252, 4, 33, "NumberedEquation"],
Cell[27622, 688, 439, 13, 29, "Text"],
Cell[28064, 703, 565, 13, 32, "NumberedEquation"],
Cell[28632, 718, 1359, 29, 128, "Text"],
Cell[29994, 749, 283, 5, 37, "NumberedEquation"],
Cell[30280, 756, 357, 10, 29, "Text"],
Cell[30640, 768, 195, 4, 33, "NumberedEquation"],
Cell[30838, 774, 791, 23, 52, "Text"],
Cell[31632, 799, 141, 2, 20, "NumberedEquation"],
Cell[31776, 803, 205, 6, 20, "Text"],
Cell[31984, 811, 402, 9, 32, "NumberedEquation"],
Cell[32389, 822, 207, 6, 20, "Text"],
Cell[32599, 830, 360, 8, 32, "NumberedEquation"],
Cell[32962, 840, 213, 6, 20, "Text"],
Cell[33178, 848, 304, 5, 32, "NumberedEquation"],
Cell[33485, 855, 207, 6, 20, "Text"],
Cell[33695, 863, 236, 4, 25, "NumberedEquation"],
Cell[33934, 869, 285, 7, 29, "Text"],
Cell[34222, 878, 284, 5, 37, "NumberedEquation"],
Cell[34509, 885, 113, 1, 20, "Text"],
Cell[34625, 888, 277, 5, 36, "NumberedEquation"],
Cell[34905, 895, 195, 6, 20, "Text"],
Cell[35103, 903, 190, 3, 34, "NumberedEquation"],
Cell[35296, 908, 594, 12, 96, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[35927, 925, 110, 1, 38, "Section"],
Cell[36040, 928, 224, 6, 20, "Text"],
Cell[36267, 936, 239, 4, 33, "NumberedEquation"],
Cell[36509, 942, 798, 14, 73, "Text",
  PageBreakWithin->Automatic],
Cell[37310, 958, 277, 4, 33, "NumberedEquation"],
Cell[37590, 964, 347, 12, 20, "Text"],
Cell[37940, 978, 381, 6, 40, "NumberedEquation"],
Cell[38324, 986, 726, 18, 52, "Text"],

Cell[CellGroupData[{
Cell[39075, 1008, 167, 5, 32, "Subsection"],
Cell[39245, 1015, 198, 5, 36, "Input"],

Cell[CellGroupData[{
Cell[39468, 1024, 99, 1, 28, "Subsubsection"],
Cell[39570, 1027, 404, 10, 29, "Text"],
Cell[39977, 1039, 897, 15, 112, "Input"],
Cell[40877, 1056, 253, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[41155, 1066, 150, 2, 25, "Input"],
Cell[41308, 1070, 66, 1, 36, "Output"]
}, Open  ]],
Cell[41389, 1074, 90, 1, 20, "Text"],
Cell[41482, 1077, 133, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[41640, 1083, 141, 2, 25, "Input"],
Cell[41784, 1087, 54, 1, 36, "Output"]
}, Open  ]],
Cell[41853, 1091, 185, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[42063, 1099, 149, 2, 25, "Input"],
Cell[42215, 1103, 65, 1, 36, "Output"]
}, Open  ]],
Cell[42295, 1107, 101, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[42421, 1112, 149, 2, 25, "Input"],
Cell[42573, 1116, 54, 1, 36, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[42700, 1125, 120, 1, 38, "Section"],
Cell[42823, 1128, 1270, 24, 117, "Text"],
Cell[44096, 1154, 276, 5, 32, "NumberedEquation"],
Cell[44375, 1161, 329, 6, 40, "Text"],
Cell[44707, 1169, 162, 3, 20, "NumberedEquation"],
Cell[44872, 1174, 485, 7, 73, "NumberedEquation"],
Cell[45360, 1183, 117, 1, 20, "Text"],
Cell[45480, 1186, 368, 6, 32, "NumberedEquation"],
Cell[45851, 1194, 188, 6, 20, "Text"],
Cell[46042, 1202, 579, 11, 50, "Input"],
Cell[46624, 1215, 690, 13, 112, "Input"],

Cell[CellGroupData[{
Cell[47339, 1232, 140, 2, 25, "Input"],
Cell[47482, 1236, 54, 1, 36, "Output"]
}, Open  ]],
Cell[47551, 1240, 759, 15, 62, "Text"],
Cell[48313, 1257, 806, 15, 125, "Input"],
Cell[49122, 1274, 179, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[49326, 1282, 157, 3, 25, "Input"],
Cell[49486, 1287, 54, 1, 36, "Output"]
}, Open  ]],
Cell[49555, 1291, 362, 6, 40, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[49954, 1302, 194, 6, 38, "Section"],
Cell[50151, 1310, 1520, 25, 150, "Text"],
Cell[51674, 1337, 712, 14, 102, "Input"],
Cell[52389, 1353, 102, 1, 20, "Text"],
Cell[52494, 1356, 252, 4, 20, "NumberedEquation"],
Cell[52749, 1362, 182, 6, 20, "Text"],
Cell[52934, 1370, 699, 11, 112, "Input"],

Cell[CellGroupData[{
Cell[53658, 1385, 115, 1, 32, "Subsection"],
Cell[53776, 1388, 173, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[53974, 1396, 196, 3, 37, "Input"],
Cell[54173, 1401, 78, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[54288, 1407, 196, 3, 37, "Input"],
Cell[54487, 1412, 77, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[54601, 1418, 194, 3, 37, "Input"],
Cell[54798, 1423, 78, 1, 36, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[54937, 1431, 113, 1, 38, "Section"],
Cell[55053, 1434, 306, 6, 40, "Text"],

Cell[CellGroupData[{
Cell[55384, 1444, 111, 2, 25, "Input"],
Cell[55498, 1448, 213, 3, 44, "Print"]
}, Open  ]],
Cell[55726, 1454, 143, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[55894, 1460, 138, 2, 25, "Input"],
Cell[56035, 1464, 64, 1, 48, "Output"]
}, Open  ]],
Cell[56114, 1468, 289, 9, 20, "Text"],
Cell[56406, 1479, 138, 2, 25, "Input"],
Cell[56547, 1483, 164, 3, 20, "Text"],
Cell[56714, 1488, 181, 5, 25, "Input"],
Cell[56898, 1495, 194, 5, 36, "Input"],
Cell[57095, 1502, 129, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[57249, 1507, 125, 2, 25, "Input"],
Cell[57377, 1511, 185, 3, 50, "Output"]
}, Open  ]],
Cell[57577, 1517, 185, 4, 20, "Text"],
Cell[57765, 1523, 438, 10, 90, "Input"],

Cell[CellGroupData[{
Cell[58228, 1537, 138, 2, 25, "Input"],
Cell[58369, 1541, 1693, 37, 172, "Output"]
}, Open  ]],
Cell[60077, 1581, 231, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[60333, 1591, 154, 2, 25, "Input"],
Cell[60490, 1595, 54, 1, 36, "Output"]
}, Open  ]],
Cell[60559, 1599, 166, 3, 20, "Text"],

Cell[CellGroupData[{
Cell[60750, 1606, 154, 2, 25, "Input"],
Cell[60907, 1610, 54, 1, 36, "Output"]
}, Open  ]],
Cell[60976, 1614, 467, 9, 52, "Text"],

Cell[CellGroupData[{
Cell[61468, 1627, 156, 2, 25, "Input"],
Cell[61627, 1631, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[61718, 1637, 156, 2, 25, "Input"],
Cell[61877, 1641, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[61968, 1647, 156, 2, 25, "Input"],
Cell[62127, 1651, 54, 1, 36, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[62230, 1658, 160, 2, 38, "Section",
  PageBreakAbove->False],
Cell[62393, 1662, 686, 11, 73, "Text"],
Cell[63082, 1675, 133, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[63240, 1681, 122, 2, 25, "Input"],
Cell[63365, 1685, 178, 3, 44, "Print"]
}, Open  ]],
Cell[63558, 1691, 312, 6, 40, "Text"],
Cell[63873, 1699, 266, 4, 37, "Input"],

Cell[CellGroupData[{
Cell[64164, 1707, 318, 5, 50, "Input"],
Cell[64485, 1714, 110, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[64632, 1721, 106, 1, 32, "Subsection"],
Cell[64741, 1724, 422, 7, 52, "Text"],

Cell[CellGroupData[{
Cell[65188, 1735, 223, 5, 25, "Input"],
Cell[65414, 1742, 10859, 383, 132, 4039, 293, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[76334, 2132, 135, 2, 38, "Section",
  PageBreakAbove->Automatic],
Cell[76472, 2136, 232, 5, 29, "Text"],
Cell[76707, 2143, 318, 6, 50, "Input"],

Cell[CellGroupData[{
Cell[77050, 2153, 320, 7, 37, "Input"],
Cell[77373, 2162, 15023, 465, 145, 5430, 342, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[92445, 2633, 127, 2, 38, "Section",
  PageBreakAbove->False],
Cell[92575, 2637, 310, 6, 29, "Text"],
Cell[92888, 2645, 507, 8, 76, "Input"],

Cell[CellGroupData[{
Cell[93420, 2657, 136, 2, 25, "Input"],
Cell[93559, 2661, 121, 2, 36, "Output"]
}, Open  ]],
Cell[93695, 2666, 171, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[93891, 2674, 137, 2, 25, "Input"],
Cell[94031, 2678, 73, 1, 36, "Output"]
}, Open  ]],
Cell[94119, 2682, 168, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[94312, 2690, 138, 2, 25, "Input"],
Cell[94453, 2694, 55, 1, 36, "Output"]
}, Open  ]],
Cell[94523, 2698, 405, 7, 40, "Text"],
Cell[94931, 2707, 188, 5, 36, "Input"],
Cell[95122, 2714, 924, 16, 112, "Input"],

Cell[CellGroupData[{
Cell[96071, 2734, 141, 2, 25, "Input"],
Cell[96215, 2738, 54, 1, 36, "Output"]
}, Open  ]],
Cell[96284, 2742, 204, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[96513, 2752, 147, 2, 25, "Input"],
Cell[96663, 2756, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[96754, 2762, 145, 2, 25, "Input"],
Cell[96902, 2766, 62, 1, 36, "Output"]
}, Open  ]],
Cell[96979, 2770, 185, 4, 20, "Text"],
Cell[97167, 2776, 324, 6, 50, "Input"],
Cell[97494, 2784, 202, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[97721, 2792, 199, 4, 25, "Input"],
Cell[97923, 2798, 13550, 416, 140, 4026, 293, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]],
Cell[111488, 3217, 197, 5, 20, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[111710, 3226, 343, 8, 37, "Input",
  PageBreakAbove->False],
Cell[112056, 3236, 21411, 566, 168, 5828, 368, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]],
Cell[133482, 3805, 983, 15, 96, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[134490, 3824, 337, 7, 50, "Input",
  PageBreakAbove->False],
Cell[134830, 3833, 67805, 1386, 214, 21944, 814, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]],
Cell[202650, 5222, 545, 9, 52, "Text",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[203232, 5236, 102, 1, 38, "Section"],
Cell[203337, 5239, 1397, 23, 140, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[204771, 5267, 129, 2, 38, "Section",
  PageBreakAbove->False],
Cell[204903, 5271, 1513, 34, 249, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

